/* GXmlDomCollections.c generated by valac 0.34.6, the Vala compiler
 * generated from GXmlDomCollections.vala, do not modify */

/* -*- Mode: vala; indent-tabs-mode: nil; c-basic-offset: 2; tab-width: 2 -*- */
/* GXmlDomCollections.vala
 *
 * Copyright (C) 2016  Daniel Espinosa <esodan@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Daniel Espinosa <esodan@gmail.com>
 */

#include <glib.h>
#include <glib-object.h>
#include <gee.h>
#include <stdlib.h>
#include <string.h>
#include <gio/gio.h>
#include <glib/gi18n-lib.h>


#define GXML_TYPE_DOM_TOKEN_LIST (gxml_dom_token_list_get_type ())
#define GXML_DOM_TOKEN_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_TOKEN_LIST, GXmlDomTokenList))
#define GXML_IS_DOM_TOKEN_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_TOKEN_LIST))
#define GXML_DOM_TOKEN_LIST_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_TOKEN_LIST, GXmlDomTokenListIface))

typedef struct _GXmlDomTokenList GXmlDomTokenList;
typedef struct _GXmlDomTokenListIface GXmlDomTokenListIface;

#define GXML_TYPE_GDOM_TOKEN_LIST (gxml_gdom_token_list_get_type ())
#define GXML_GDOM_TOKEN_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GDOM_TOKEN_LIST, GXmlGDomTokenList))
#define GXML_GDOM_TOKEN_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GDOM_TOKEN_LIST, GXmlGDomTokenListClass))
#define GXML_IS_GDOM_TOKEN_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GDOM_TOKEN_LIST))
#define GXML_IS_GDOM_TOKEN_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GDOM_TOKEN_LIST))
#define GXML_GDOM_TOKEN_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GDOM_TOKEN_LIST, GXmlGDomTokenListClass))

typedef struct _GXmlGDomTokenList GXmlGDomTokenList;
typedef struct _GXmlGDomTokenListClass GXmlGDomTokenListClass;
typedef struct _GXmlGDomTokenListPrivate GXmlGDomTokenListPrivate;

#define GXML_TYPE_DOM_EVENT_TARGET (gxml_dom_event_target_get_type ())
#define GXML_DOM_EVENT_TARGET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_EVENT_TARGET, GXmlDomEventTarget))
#define GXML_IS_DOM_EVENT_TARGET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_EVENT_TARGET))
#define GXML_DOM_EVENT_TARGET_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_EVENT_TARGET, GXmlDomEventTargetIface))

typedef struct _GXmlDomEventTarget GXmlDomEventTarget;
typedef struct _GXmlDomEventTargetIface GXmlDomEventTargetIface;

#define GXML_TYPE_DOM_EVENT_LISTENER (gxml_dom_event_listener_get_type ())
#define GXML_DOM_EVENT_LISTENER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_EVENT_LISTENER, GXmlDomEventListener))
#define GXML_IS_DOM_EVENT_LISTENER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_EVENT_LISTENER))
#define GXML_DOM_EVENT_LISTENER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_EVENT_LISTENER, GXmlDomEventListenerIface))

typedef struct _GXmlDomEventListener GXmlDomEventListener;
typedef struct _GXmlDomEventListenerIface GXmlDomEventListenerIface;

#define GXML_TYPE_DOM_EVENT (gxml_dom_event_get_type ())
#define GXML_DOM_EVENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_EVENT, GXmlDomEvent))
#define GXML_IS_DOM_EVENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_EVENT))
#define GXML_DOM_EVENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_EVENT, GXmlDomEventIface))

typedef struct _GXmlDomEvent GXmlDomEvent;
typedef struct _GXmlDomEventIface GXmlDomEventIface;

#define GXML_TYPE_DOM_TIME_STAMP (gxml_dom_time_stamp_get_type ())
#define GXML_DOM_TIME_STAMP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_TIME_STAMP, GXmlDomTimeStamp))
#define GXML_DOM_TIME_STAMP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_DOM_TIME_STAMP, GXmlDomTimeStampClass))
#define GXML_IS_DOM_TIME_STAMP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_TIME_STAMP))
#define GXML_IS_DOM_TIME_STAMP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_DOM_TIME_STAMP))
#define GXML_DOM_TIME_STAMP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_DOM_TIME_STAMP, GXmlDomTimeStampClass))

typedef struct _GXmlDomTimeStamp GXmlDomTimeStamp;
typedef struct _GXmlDomTimeStampClass GXmlDomTimeStampClass;

#define GXML_DOM_EVENT_TYPE_PHASE (gxml_dom_event_phase_get_type ())

#define GXML_TYPE_DOM_NODE (gxml_dom_node_get_type ())
#define GXML_DOM_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_NODE, GXmlDomNode))
#define GXML_IS_DOM_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_NODE))
#define GXML_DOM_NODE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_NODE, GXmlDomNodeIface))

typedef struct _GXmlDomNode GXmlDomNode;
typedef struct _GXmlDomNodeIface GXmlDomNodeIface;

#define GXML_DOM_NODE_TYPE_DOCUMENT_POSITION (gxml_dom_node_document_position_get_type ())

#define GXML_DOM_NODE_TYPE_NODE_TYPE (gxml_dom_node_node_type_get_type ())

#define GXML_TYPE_DOM_PARENT_NODE (gxml_dom_parent_node_get_type ())
#define GXML_DOM_PARENT_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_PARENT_NODE, GXmlDomParentNode))
#define GXML_IS_DOM_PARENT_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_PARENT_NODE))
#define GXML_DOM_PARENT_NODE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_PARENT_NODE, GXmlDomParentNodeIface))

typedef struct _GXmlDomParentNode GXmlDomParentNode;
typedef struct _GXmlDomParentNodeIface GXmlDomParentNodeIface;

#define GXML_TYPE_DOM_NODE_LIST (gxml_dom_node_list_get_type ())
#define GXML_DOM_NODE_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_NODE_LIST, GXmlDomNodeList))
#define GXML_IS_DOM_NODE_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_NODE_LIST))
#define GXML_DOM_NODE_LIST_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_NODE_LIST, GXmlDomNodeListIface))

typedef struct _GXmlDomNodeList GXmlDomNodeList;
typedef struct _GXmlDomNodeListIface GXmlDomNodeListIface;

#define GXML_TYPE_DOM_ELEMENT_LIST (gxml_dom_element_list_get_type ())
#define GXML_DOM_ELEMENT_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_ELEMENT_LIST, GXmlDomElementList))
#define GXML_DOM_ELEMENT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_DOM_ELEMENT_LIST, GXmlDomElementListClass))
#define GXML_IS_DOM_ELEMENT_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_ELEMENT_LIST))
#define GXML_IS_DOM_ELEMENT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_DOM_ELEMENT_LIST))
#define GXML_DOM_ELEMENT_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_DOM_ELEMENT_LIST, GXmlDomElementListClass))

typedef struct _GXmlDomElementList GXmlDomElementList;
typedef struct _GXmlDomElementListClass GXmlDomElementListClass;

#define GXML_TYPE_DOM_HTML_COLLECTION (gxml_dom_html_collection_get_type ())
#define GXML_DOM_HTML_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_HTML_COLLECTION, GXmlDomHTMLCollection))
#define GXML_IS_DOM_HTML_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_HTML_COLLECTION))
#define GXML_DOM_HTML_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_HTML_COLLECTION, GXmlDomHTMLCollectionIface))

typedef struct _GXmlDomHTMLCollection GXmlDomHTMLCollection;
typedef struct _GXmlDomHTMLCollectionIface GXmlDomHTMLCollectionIface;

#define GXML_TYPE_DOM_NON_ELEMENT_PARENT_NODE (gxml_dom_non_element_parent_node_get_type ())
#define GXML_DOM_NON_ELEMENT_PARENT_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_NON_ELEMENT_PARENT_NODE, GXmlDomNonElementParentNode))
#define GXML_IS_DOM_NON_ELEMENT_PARENT_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_NON_ELEMENT_PARENT_NODE))
#define GXML_DOM_NON_ELEMENT_PARENT_NODE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_NON_ELEMENT_PARENT_NODE, GXmlDomNonElementParentNodeIface))

typedef struct _GXmlDomNonElementParentNode GXmlDomNonElementParentNode;
typedef struct _GXmlDomNonElementParentNodeIface GXmlDomNonElementParentNodeIface;

#define GXML_TYPE_DOM_DOCUMENT (gxml_dom_document_get_type ())
#define GXML_DOM_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_DOCUMENT, GXmlDomDocument))
#define GXML_IS_DOM_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_DOCUMENT))
#define GXML_DOM_DOCUMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_DOCUMENT, GXmlDomDocumentIface))

typedef struct _GXmlDomDocument GXmlDomDocument;
typedef struct _GXmlDomDocumentIface GXmlDomDocumentIface;

#define GXML_TYPE_DOM_DOCUMENT_FRAGMENT (gxml_dom_document_fragment_get_type ())
#define GXML_DOM_DOCUMENT_FRAGMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_DOCUMENT_FRAGMENT, GXmlDomDocumentFragment))
#define GXML_IS_DOM_DOCUMENT_FRAGMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_DOCUMENT_FRAGMENT))
#define GXML_DOM_DOCUMENT_FRAGMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_DOCUMENT_FRAGMENT, GXmlDomDocumentFragmentIface))

typedef struct _GXmlDomDocumentFragment GXmlDomDocumentFragment;
typedef struct _GXmlDomDocumentFragmentIface GXmlDomDocumentFragmentIface;

#define GXML_TYPE_DOM_NON_DOCUMENT_TYPE_CHILD_NODE (gxml_dom_non_document_type_child_node_get_type ())
#define GXML_DOM_NON_DOCUMENT_TYPE_CHILD_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_NON_DOCUMENT_TYPE_CHILD_NODE, GXmlDomNonDocumentTypeChildNode))
#define GXML_IS_DOM_NON_DOCUMENT_TYPE_CHILD_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_NON_DOCUMENT_TYPE_CHILD_NODE))
#define GXML_DOM_NON_DOCUMENT_TYPE_CHILD_NODE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_NON_DOCUMENT_TYPE_CHILD_NODE, GXmlDomNonDocumentTypeChildNodeIface))

typedef struct _GXmlDomNonDocumentTypeChildNode GXmlDomNonDocumentTypeChildNode;
typedef struct _GXmlDomNonDocumentTypeChildNodeIface GXmlDomNonDocumentTypeChildNodeIface;

#define GXML_TYPE_DOM_CHILD_NODE (gxml_dom_child_node_get_type ())
#define GXML_DOM_CHILD_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_CHILD_NODE, GXmlDomChildNode))
#define GXML_IS_DOM_CHILD_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_CHILD_NODE))
#define GXML_DOM_CHILD_NODE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_CHILD_NODE, GXmlDomChildNodeIface))

typedef struct _GXmlDomChildNode GXmlDomChildNode;
typedef struct _GXmlDomChildNodeIface GXmlDomChildNodeIface;

#define GXML_TYPE_DOM_CHARACTER_DATA (gxml_dom_character_data_get_type ())
#define GXML_DOM_CHARACTER_DATA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_CHARACTER_DATA, GXmlDomCharacterData))
#define GXML_IS_DOM_CHARACTER_DATA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_CHARACTER_DATA))
#define GXML_DOM_CHARACTER_DATA_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_CHARACTER_DATA, GXmlDomCharacterDataIface))

typedef struct _GXmlDomCharacterData GXmlDomCharacterData;
typedef struct _GXmlDomCharacterDataIface GXmlDomCharacterDataIface;

#define GXML_TYPE_DOM_TEXT (gxml_dom_text_get_type ())
#define GXML_DOM_TEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_TEXT, GXmlDomText))
#define GXML_IS_DOM_TEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_TEXT))
#define GXML_DOM_TEXT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_TEXT, GXmlDomTextIface))

typedef struct _GXmlDomText GXmlDomText;
typedef struct _GXmlDomTextIface GXmlDomTextIface;

#define GXML_TYPE_DOM_COMMENT (gxml_dom_comment_get_type ())
#define GXML_DOM_COMMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_COMMENT, GXmlDomComment))
#define GXML_IS_DOM_COMMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_COMMENT))
#define GXML_DOM_COMMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_COMMENT, GXmlDomCommentIface))

typedef struct _GXmlDomComment GXmlDomComment;
typedef struct _GXmlDomCommentIface GXmlDomCommentIface;

#define GXML_TYPE_DOM_PROCESSING_INSTRUCTION (gxml_dom_processing_instruction_get_type ())
#define GXML_DOM_PROCESSING_INSTRUCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_PROCESSING_INSTRUCTION, GXmlDomProcessingInstruction))
#define GXML_IS_DOM_PROCESSING_INSTRUCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_PROCESSING_INSTRUCTION))
#define GXML_DOM_PROCESSING_INSTRUCTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_PROCESSING_INSTRUCTION, GXmlDomProcessingInstructionIface))

typedef struct _GXmlDomProcessingInstruction GXmlDomProcessingInstruction;
typedef struct _GXmlDomProcessingInstructionIface GXmlDomProcessingInstructionIface;

#define GXML_TYPE_DOM_RANGE (gxml_dom_range_get_type ())
#define GXML_DOM_RANGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_RANGE, GXmlDomRange))
#define GXML_IS_DOM_RANGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_RANGE))
#define GXML_DOM_RANGE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_RANGE, GXmlDomRangeIface))

typedef struct _GXmlDomRange GXmlDomRange;
typedef struct _GXmlDomRangeIface GXmlDomRangeIface;

#define GXML_DOM_RANGE_TYPE_BOUNDARY_POINTS (gxml_dom_range_boundary_points_get_type ())

#define GXML_TYPE_DOM_NODE_FILTER (gxml_dom_node_filter_get_type ())
#define GXML_DOM_NODE_FILTER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_NODE_FILTER, GXmlDomNodeFilter))
#define GXML_DOM_NODE_FILTER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_DOM_NODE_FILTER, GXmlDomNodeFilterClass))
#define GXML_IS_DOM_NODE_FILTER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_NODE_FILTER))
#define GXML_IS_DOM_NODE_FILTER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_DOM_NODE_FILTER))
#define GXML_DOM_NODE_FILTER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_DOM_NODE_FILTER, GXmlDomNodeFilterClass))

typedef struct _GXmlDomNodeFilter GXmlDomNodeFilter;
typedef struct _GXmlDomNodeFilterClass GXmlDomNodeFilterClass;

#define GXML_TYPE_DOM_NODE_ITERATOR (gxml_dom_node_iterator_get_type ())
#define GXML_DOM_NODE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_NODE_ITERATOR, GXmlDomNodeIterator))
#define GXML_IS_DOM_NODE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_NODE_ITERATOR))
#define GXML_DOM_NODE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_NODE_ITERATOR, GXmlDomNodeIteratorIface))

typedef struct _GXmlDomNodeIterator GXmlDomNodeIterator;
typedef struct _GXmlDomNodeIteratorIface GXmlDomNodeIteratorIface;

#define GXML_TYPE_DOM_TREE_WALKER (gxml_dom_tree_walker_get_type ())
#define GXML_DOM_TREE_WALKER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_TREE_WALKER, GXmlDomTreeWalker))
#define GXML_IS_DOM_TREE_WALKER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_TREE_WALKER))
#define GXML_DOM_TREE_WALKER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_TREE_WALKER, GXmlDomTreeWalkerIface))

typedef struct _GXmlDomTreeWalker GXmlDomTreeWalker;
typedef struct _GXmlDomTreeWalkerIface GXmlDomTreeWalkerIface;

#define GXML_TYPE_DOM_IMPLEMENTATION (gxml_dom_implementation_get_type ())
#define GXML_DOM_IMPLEMENTATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_IMPLEMENTATION, GXmlDomImplementation))
#define GXML_IS_DOM_IMPLEMENTATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_IMPLEMENTATION))
#define GXML_DOM_IMPLEMENTATION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_IMPLEMENTATION, GXmlDomImplementationIface))

typedef struct _GXmlDomImplementation GXmlDomImplementation;
typedef struct _GXmlDomImplementationIface GXmlDomImplementationIface;

#define GXML_TYPE_DOM_DOCUMENT_TYPE (gxml_dom_document_type_get_type ())
#define GXML_DOM_DOCUMENT_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_DOCUMENT_TYPE, GXmlDomDocumentType))
#define GXML_IS_DOM_DOCUMENT_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_DOCUMENT_TYPE))
#define GXML_DOM_DOCUMENT_TYPE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_DOCUMENT_TYPE, GXmlDomDocumentTypeIface))

typedef struct _GXmlDomDocumentType GXmlDomDocumentType;
typedef struct _GXmlDomDocumentTypeIface GXmlDomDocumentTypeIface;

#define GXML_TYPE_DOM_XML_DOCUMENT (gxml_dom_xml_document_get_type ())
#define GXML_DOM_XML_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_XML_DOCUMENT, GXmlDomXMLDocument))
#define GXML_IS_DOM_XML_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_XML_DOCUMENT))
#define GXML_DOM_XML_DOCUMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_XML_DOCUMENT, GXmlDomXMLDocumentIface))

typedef struct _GXmlDomXMLDocument GXmlDomXMLDocument;
typedef struct _GXmlDomXMLDocumentIface GXmlDomXMLDocumentIface;

#define GXML_TYPE_NODE (gxml_node_get_type ())
#define GXML_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_NODE, GXmlNode))
#define GXML_IS_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_NODE))
#define GXML_NODE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_NODE, GXmlNodeIface))

typedef struct _GXmlNode GXmlNode;
typedef struct _GXmlNodeIface GXmlNodeIface;

#define GXML_TYPE_ELEMENT_LIST (gxml_element_list_get_type ())
#define GXML_ELEMENT_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_ELEMENT_LIST, GXmlElementList))
#define GXML_ELEMENT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_ELEMENT_LIST, GXmlElementListClass))
#define GXML_IS_ELEMENT_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_ELEMENT_LIST))
#define GXML_IS_ELEMENT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_ELEMENT_LIST))
#define GXML_ELEMENT_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_ELEMENT_LIST, GXmlElementListClass))

typedef struct _GXmlElementList GXmlElementList;
typedef struct _GXmlElementListClass GXmlElementListClass;

#define GXML_TYPE_NAMESPACE (gxml_namespace_get_type ())
#define GXML_NAMESPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_NAMESPACE, GXmlNamespace))
#define GXML_IS_NAMESPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_NAMESPACE))
#define GXML_NAMESPACE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_NAMESPACE, GXmlNamespaceIface))

typedef struct _GXmlNamespace GXmlNamespace;
typedef struct _GXmlNamespaceIface GXmlNamespaceIface;

#define GXML_TYPE_NODE_TYPE (gxml_node_type_get_type ())

#define GXML_TYPE_DOCUMENT (gxml_document_get_type ())
#define GXML_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOCUMENT, GXmlDocument))
#define GXML_IS_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOCUMENT))
#define GXML_DOCUMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOCUMENT, GXmlDocumentIface))

typedef struct _GXmlDocument GXmlDocument;
typedef struct _GXmlDocumentIface GXmlDocumentIface;

#define GXML_TYPE_DOM_ELEMENT (gxml_dom_element_get_type ())
#define GXML_DOM_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_ELEMENT, GXmlDomElement))
#define GXML_IS_DOM_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_ELEMENT))
#define GXML_DOM_ELEMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_ELEMENT, GXmlDomElementIface))

typedef struct _GXmlDomElement GXmlDomElement;
typedef struct _GXmlDomElementIface GXmlDomElementIface;

#define GXML_TYPE_DOM_NAMED_NODE_MAP (gxml_dom_named_node_map_get_type ())
#define GXML_DOM_NAMED_NODE_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_NAMED_NODE_MAP, GXmlDomNamedNodeMap))
#define GXML_IS_DOM_NAMED_NODE_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_NAMED_NODE_MAP))
#define GXML_DOM_NAMED_NODE_MAP_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_NAMED_NODE_MAP, GXmlDomNamedNodeMapIface))

typedef struct _GXmlDomNamedNodeMap GXmlDomNamedNodeMap;
typedef struct _GXmlDomNamedNodeMapIface GXmlDomNamedNodeMapIface;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))

#define GXML_TYPE_DOM_SETTABLE_TOKEN_LIST (gxml_dom_settable_token_list_get_type ())
#define GXML_DOM_SETTABLE_TOKEN_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_SETTABLE_TOKEN_LIST, GXmlDomSettableTokenList))
#define GXML_IS_DOM_SETTABLE_TOKEN_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_SETTABLE_TOKEN_LIST))
#define GXML_DOM_SETTABLE_TOKEN_LIST_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_SETTABLE_TOKEN_LIST, GXmlDomSettableTokenListIface))

typedef struct _GXmlDomSettableTokenList GXmlDomSettableTokenList;
typedef struct _GXmlDomSettableTokenListIface GXmlDomSettableTokenListIface;

#define GXML_TYPE_GDOM_SETTABLE_TOKEN_LIST (gxml_gdom_settable_token_list_get_type ())
#define GXML_GDOM_SETTABLE_TOKEN_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GDOM_SETTABLE_TOKEN_LIST, GXmlGDomSettableTokenList))
#define GXML_GDOM_SETTABLE_TOKEN_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GDOM_SETTABLE_TOKEN_LIST, GXmlGDomSettableTokenListClass))
#define GXML_IS_GDOM_SETTABLE_TOKEN_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GDOM_SETTABLE_TOKEN_LIST))
#define GXML_IS_GDOM_SETTABLE_TOKEN_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GDOM_SETTABLE_TOKEN_LIST))
#define GXML_GDOM_SETTABLE_TOKEN_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GDOM_SETTABLE_TOKEN_LIST, GXmlGDomSettableTokenListClass))

typedef struct _GXmlGDomSettableTokenList GXmlGDomSettableTokenList;
typedef struct _GXmlGDomSettableTokenListClass GXmlGDomSettableTokenListClass;
typedef struct _GXmlGDomSettableTokenListPrivate GXmlGDomSettableTokenListPrivate;

#define GXML_TYPE_GDOM_HTML_COLLECTION (gxml_gdom_html_collection_get_type ())
#define GXML_GDOM_HTML_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GDOM_HTML_COLLECTION, GXmlGDomHTMLCollection))
#define GXML_GDOM_HTML_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GDOM_HTML_COLLECTION, GXmlGDomHTMLCollectionClass))
#define GXML_IS_GDOM_HTML_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GDOM_HTML_COLLECTION))
#define GXML_IS_GDOM_HTML_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GDOM_HTML_COLLECTION))
#define GXML_GDOM_HTML_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GDOM_HTML_COLLECTION, GXmlGDomHTMLCollectionClass))

typedef struct _GXmlGDomHTMLCollection GXmlGDomHTMLCollection;
typedef struct _GXmlGDomHTMLCollectionClass GXmlGDomHTMLCollectionClass;
typedef struct _GXmlGDomHTMLCollectionPrivate GXmlGDomHTMLCollectionPrivate;

struct _GXmlDomTokenListIface {
	GTypeInterface parent_iface;
	gchar* (*item) (GXmlDomTokenList* self, gint index);
	gboolean (*contains) (GXmlDomTokenList* self, const gchar* token, GError** error);
	void (*add) (GXmlDomTokenList* self, gchar** tokens, int tokens_length1, GError** error);
	void (*remove) (GXmlDomTokenList* self, gchar** tokens, int tokens_length1);
	gboolean (*toggle) (GXmlDomTokenList* self, const gchar* token, gboolean force, gboolean _auto_, GError** error);
	gchar* (*to_string) (GXmlDomTokenList* self);
	gint (*get_length) (GXmlDomTokenList* self);
};

typedef enum  {
	GXML_DOM_EVENT_PHASE_NONE = 0,
	GXML_DOM_EVENT_PHASE_CAPTURING_PHASE,
	GXML_DOM_EVENT_PHASE_AT_TARGET,
	GXML_DOM_EVENT_PHASE_BUBBLING_PHASE
} GXmlDomEventPhase;

struct _GXmlDomEventIface {
	GTypeInterface parent_iface;
	void (*stop_propagation) (GXmlDomEvent* self);
	void (*stop_immediate_propagation) (GXmlDomEvent* self);
	void (*prevent_default) (GXmlDomEvent* self);
	void (*init_event) (GXmlDomEvent* self, const gchar* type, gboolean bubbles, gboolean cancelable);
	const gchar* (*get_etype) (GXmlDomEvent* self);
	GXmlDomEventTarget* (*get_event_target) (GXmlDomEvent* self);
	GXmlDomEventTarget* (*get_current_target) (GXmlDomEvent* self);
	gboolean (*get_bubbles) (GXmlDomEvent* self);
	gboolean (*get_cancelable) (GXmlDomEvent* self);
	gboolean (*get_is_trusted) (GXmlDomEvent* self);
	GXmlDomTimeStamp* (*get_time_stamp) (GXmlDomEvent* self);
	gboolean (*get_default_prevented) (GXmlDomEvent* self);
	GXmlDomEventPhase (*get_event_phase) (GXmlDomEvent* self);
};

struct _GXmlDomEventListenerIface {
	GTypeInterface parent_iface;
	void (*handle_event) (GXmlDomEventListener* self, GXmlDomEvent* event);
};

struct _GXmlDomEventTargetIface {
	GTypeInterface parent_iface;
	void (*add_event_listener) (GXmlDomEventTarget* self, const gchar* type, GXmlDomEventListener* callback, gboolean capture);
	void (*remove_event_listener) (GXmlDomEventTarget* self, const gchar* type, GXmlDomEventListener* callback, gboolean capture);
	gboolean (*dispatch_event) (GXmlDomEventTarget* self, GXmlDomEvent* event, GError** error);
};

typedef enum  {
	GXML_DOM_NODE_DOCUMENT_POSITION_NONE = 1 << 0,
	GXML_DOM_NODE_DOCUMENT_POSITION_DISCONNECTED = 1 << 1,
	GXML_DOM_NODE_DOCUMENT_POSITION_PRECEDING = 1 << 2,
	GXML_DOM_NODE_DOCUMENT_POSITION_FOLLOWING = 1 << 3,
	GXML_DOM_NODE_DOCUMENT_POSITION_CONTAINS = 1 << 4,
	GXML_DOM_NODE_DOCUMENT_POSITION_CONTAINED_BY = 1 << 5,
	GXML_DOM_NODE_DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC = 1 << 6
} GXmlDomNodeDocumentPosition;

typedef enum  {
	GXML_DOM_NODE_NODE_TYPE_INVALID = 0,
	GXML_DOM_NODE_NODE_TYPE_ELEMENT_NODE = 1,
	GXML_DOM_NODE_NODE_TYPE_ATTRIBUTE_NODE,
	GXML_DOM_NODE_NODE_TYPE_TEXT_NODE,
	GXML_DOM_NODE_NODE_TYPE_CDATA_SECTION_NODE,
	GXML_DOM_NODE_NODE_TYPE_ENTITY_REFERENCE_NODE,
	GXML_DOM_NODE_NODE_TYPE_ENTITY_NODE,
	GXML_DOM_NODE_NODE_TYPE_PROCESSING_INSTRUCTION_NODE,
	GXML_DOM_NODE_NODE_TYPE_COMMENT_NODE,
	GXML_DOM_NODE_NODE_TYPE_DOCUMENT_NODE,
	GXML_DOM_NODE_NODE_TYPE_DOCUMENT_TYPE_NODE,
	GXML_DOM_NODE_NODE_TYPE_DOCUMENT_FRAGMENT_NODE,
	GXML_DOM_NODE_NODE_TYPE_NOTATION_NODE
} GXmlDomNodeNodeType;

struct _GXmlDomNodeListIface {
	GTypeInterface parent_iface;
	GXmlDomNode* (*item) (GXmlDomNodeList* self, gint index);
	gint (*get_length) (GXmlDomNodeList* self);
};

struct _GXmlDomHTMLCollectionIface {
	GTypeInterface parent_iface;
	GXmlDomElement* (*get_element) (GXmlDomHTMLCollection* self, gint index);
	GXmlDomElement** (*to_array) (GXmlDomHTMLCollection* self, int* result_length1);
	GXmlDomElement* (*item) (GXmlDomHTMLCollection* self, gint index);
	GXmlDomElement* (*named_item) (GXmlDomHTMLCollection* self, const gchar* name);
	gint (*get_length) (GXmlDomHTMLCollection* self);
};

struct _GXmlDomParentNodeIface {
	GTypeInterface parent_iface;
	GXmlDomElement* (*query_selector) (GXmlDomParentNode* self, const gchar* selectors, GError** error);
	GXmlDomNodeList* (*query_selector_all) (GXmlDomParentNode* self, const gchar* selectors, GError** error);
	GXmlDomElementList* (*get_elements_by_property_value) (GXmlDomParentNode* self, const gchar* property, const gchar* value);
	GXmlDomHTMLCollection* (*get_children) (GXmlDomParentNode* self);
	GXmlDomElement* (*get_first_element_child) (GXmlDomParentNode* self);
	GXmlDomElement* (*get_last_element_child) (GXmlDomParentNode* self);
	gint (*get_child_element_count) (GXmlDomParentNode* self);
};

struct _GXmlDomNonElementParentNodeIface {
	GTypeInterface parent_iface;
	GXmlDomElement* (*get_element_by_id) (GXmlDomNonElementParentNode* self, const gchar* element_id, GError** error);
};

struct _GXmlDomDocumentFragmentIface {
	GTypeInterface parent_iface;
};

struct _GXmlDomNonDocumentTypeChildNodeIface {
	GTypeInterface parent_iface;
	GXmlDomElement* (*get_previous_element_sibling) (GXmlDomNonDocumentTypeChildNode* self);
	GXmlDomElement* (*get_next_element_sibling) (GXmlDomNonDocumentTypeChildNode* self);
};

struct _GXmlDomChildNodeIface {
	GTypeInterface parent_iface;
	void (*remove) (GXmlDomChildNode* self);
};

struct _GXmlDomCharacterDataIface {
	GTypeInterface parent_iface;
	gchar* (*substring_data) (GXmlDomCharacterData* self, gint offset, gint count, GError** error);
	void (*append_data) (GXmlDomCharacterData* self, const gchar* data);
	void (*insert_data) (GXmlDomCharacterData* self, gint offset, const gchar* data, GError** error);
	void (*delete_data) (GXmlDomCharacterData* self, gint offset, gint count, GError** error);
	void (*replace_data) (GXmlDomCharacterData* self, gint offset, gint count, const gchar* data, GError** error);
	gchar* (*get_data) (GXmlDomCharacterData* self);
	void (*set_data) (GXmlDomCharacterData* self, const gchar* value);
	gint (*get_length) (GXmlDomCharacterData* self);
};

struct _GXmlDomTextIface {
	GTypeInterface parent_iface;
	GXmlDomText* (*split_text) (GXmlDomText* self, gint offset, GError** error);
	gchar* (*get_whole_text) (GXmlDomText* self);
};

struct _GXmlDomCommentIface {
	GTypeInterface parent_iface;
};

struct _GXmlDomProcessingInstructionIface {
	GTypeInterface parent_iface;
	gchar* (*get_target) (GXmlDomProcessingInstruction* self);
};

typedef enum  {
	GXML_DOM_RANGE_BOUNDARY_POINTS_START_TO_START = 0,
	GXML_DOM_RANGE_BOUNDARY_POINTS_START_TO_END,
	GXML_DOM_RANGE_BOUNDARY_POINTS_END_TO_END,
	GXML_DOM_RANGE_BOUNDARY_POINTS_END_TO_START
} GXmlDomRangeBoundaryPoints;

struct _GXmlDomRangeIface {
	GTypeInterface parent_iface;
	void (*set_start) (GXmlDomRange* self, GXmlDomNode* node, gint offset, GError** error);
	void (*set_end) (GXmlDomRange* self, GXmlDomNode* node, gint offset, GError** error);
	void (*set_start_before) (GXmlDomRange* self, GXmlDomNode* node, GError** error);
	void (*set_start_after) (GXmlDomRange* self, GXmlDomNode* node, GError** error);
	void (*set_end_before) (GXmlDomRange* self, GXmlDomNode* node, GError** error);
	void (*set_end_after) (GXmlDomRange* self, GXmlDomNode* node, GError** error);
	void (*collapse) (GXmlDomRange* self, gboolean to_start, GError** error);
	void (*select_node) (GXmlDomRange* self, GXmlDomNode* node, GError** error);
	void (*select_node_contents) (GXmlDomRange* self, GXmlDomNode* node, GError** error);
	gint (*compare_boundary_points) (GXmlDomRange* self, GXmlDomRangeBoundaryPoints how, GXmlDomRange* sourceRange, GError** error);
	void (*delete_contents) (GXmlDomRange* self, GError** error);
	GXmlDomDocumentFragment* (*extract_contents) (GXmlDomRange* self, GError** error);
	GXmlDomDocumentFragment* (*clone_contents) (GXmlDomRange* self, GError** error);
	void (*insert_node) (GXmlDomRange* self, GXmlDomNode* node);
	void (*surround_contents) (GXmlDomRange* self, GXmlDomNode* newParent);
	GXmlDomRange* (*clone_range) (GXmlDomRange* self);
	void (*detach) (GXmlDomRange* self);
	gboolean (*is_point_in_range) (GXmlDomRange* self, GXmlDomNode* node, gint offset);
	gshort (*compare_point) (GXmlDomRange* self, GXmlDomNode* node, gint offset);
	gboolean (*intersects_node) (GXmlDomRange* self, GXmlDomNode* node);
	gchar* (*to_string) (GXmlDomRange* self);
	GXmlDomNode* (*get_start_container) (GXmlDomRange* self);
	gint (*get_start_offset) (GXmlDomRange* self);
	GXmlDomNode* (*get_end_container) (GXmlDomRange* self);
	gint (*get_end_offset) (GXmlDomRange* self);
	gboolean (*get_collapsed) (GXmlDomRange* self);
	GXmlDomNode* (*get_common_ancestor_container) (GXmlDomRange* self);
};

struct _GXmlDomNodeIteratorIface {
	GTypeInterface parent_iface;
	GXmlDomNode* (*next_node) (GXmlDomNodeIterator* self);
	GXmlDomNode* (*previous_node) (GXmlDomNodeIterator* self);
	void (*detach) (GXmlDomNodeIterator* self);
	GXmlDomNode* (*get_root) (GXmlDomNodeIterator* self);
	GXmlDomNode* (*get_reference_node) (GXmlDomNodeIterator* self);
	gboolean (*get_pointer_before_reference_node) (GXmlDomNodeIterator* self);
	gint (*get_what_to_show) (GXmlDomNodeIterator* self);
	GXmlDomNodeFilter* (*get_filter) (GXmlDomNodeIterator* self);
};

struct _GXmlDomTreeWalkerIface {
	GTypeInterface parent_iface;
	GXmlDomNode* (*parentNode) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*firstChild) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*lastChild) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*previousSibling) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*nextSibling) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*previousNode) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*nextNode) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*get_root) (GXmlDomTreeWalker* self);
	gint (*get_what_to_show) (GXmlDomTreeWalker* self);
	GXmlDomNodeFilter* (*get_filter) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*get_current_node) (GXmlDomTreeWalker* self);
};

struct _GXmlDomDocumentTypeIface {
	GTypeInterface parent_iface;
	const gchar* (*get_name) (GXmlDomDocumentType* self);
	const gchar* (*get_public_id) (GXmlDomDocumentType* self);
	const gchar* (*get_system_id) (GXmlDomDocumentType* self);
};

struct _GXmlDomXMLDocumentIface {
	GTypeInterface parent_iface;
};

struct _GXmlNamespaceIface {
	GTypeInterface parent_iface;
	gchar* (*get_uri) (GXmlNamespace* self);
	gchar* (*get_prefix) (GXmlNamespace* self);
};

typedef enum  {
	GXML_NODE_TYPE_INVALID = 0,
	GXML_NODE_TYPE_ELEMENT = 1,
	GXML_NODE_TYPE_ATTRIBUTE,
	GXML_NODE_TYPE_TEXT,
	GXML_NODE_TYPE_CDATA_SECTION,
	GXML_NODE_TYPE_ENTITY_REFERENCE,
	GXML_NODE_TYPE_ENTITY,
	GXML_NODE_TYPE_PROCESSING_INSTRUCTION,
	GXML_NODE_TYPE_COMMENT,
	GXML_NODE_TYPE_DOCUMENT,
	GXML_NODE_TYPE_DOCUMENT_TYPE,
	GXML_NODE_TYPE_DOCUMENT_FRAGMENT,
	GXML_NODE_TYPE_NOTATION
} GXmlNodeType;

struct _GXmlNodeIface {
	GTypeInterface parent_iface;
	GXmlNode* (*get) (GXmlNode* self, const gchar* key);
	GXmlElementList* (*get_elements_by_property_value) (GXmlNode* self, const gchar* property, const gchar* value);
	GXmlElementList* (*get_elements_by_name) (GXmlNode* self, const gchar* name);
	GXmlElementList* (*get_elements_by_name_ns) (GXmlNode* self, const gchar* name, const gchar* ns);
	gchar* (*to_string) (GXmlNode* self);
	gboolean (*set_namespace) (GXmlNode* self, const gchar* uri, const gchar* prefix);
	gchar* (*ns_prefix) (GXmlNode* self);
	gchar* (*ns_uri) (GXmlNode* self);
	GeeList* (*get_namespaces) (GXmlNode* self);
	GeeBidirList* (*get_children_nodes) (GXmlNode* self);
	GeeMap* (*get_attrs) (GXmlNode* self);
	gchar* (*get_name) (GXmlNode* self);
	gchar* (*get_value) (GXmlNode* self);
	void (*set_value) (GXmlNode* self, const gchar* value);
	GXmlNodeType (*get_type_node) (GXmlNode* self);
	GXmlDocument* (*get_document) (GXmlNode* self);
	GXmlNode* (*get_parent) (GXmlNode* self);
};

struct _GXmlDocumentIface {
	GTypeInterface parent_iface;
	GXmlNode* (*create_element) (GXmlDocument* self, const gchar* name, GError** error);
	GXmlNode* (*create_text) (GXmlDocument* self, const gchar* text);
	GXmlNode* (*create_comment) (GXmlDocument* self, const gchar* text);
	GXmlNode* (*create_cdata) (GXmlDocument* self, const gchar* text);
	GXmlNode* (*create_pi) (GXmlDocument* self, const gchar* target, const gchar* data);
	gboolean (*save) (GXmlDocument* self, GCancellable* cancellable, GError** error);
	gboolean (*save_as) (GXmlDocument* self, GFile* f, GCancellable* cancellable, GError** error);
	gboolean (*get_indent) (GXmlDocument* self);
	void (*set_indent) (GXmlDocument* self, gboolean value);
	gboolean (*get_ns_top) (GXmlDocument* self);
	void (*set_ns_top) (GXmlDocument* self, gboolean value);
	gboolean (*get_prefix_default_ns) (GXmlDocument* self);
	void (*set_prefix_default_ns) (GXmlDocument* self, gboolean value);
	gboolean (*get_backup) (GXmlDocument* self);
	void (*set_backup) (GXmlDocument* self, gboolean value);
	GXmlNode* (*get_root) (GXmlDocument* self);
	GFile* (*get_file) (GXmlDocument* self);
	void (*set_file) (GXmlDocument* self, GFile* value);
};

struct _GXmlDomImplementationIface {
	GTypeInterface parent_iface;
	GXmlDomDocumentType* (*create_document_type) (GXmlDomImplementation* self, const gchar* qualified_name, const gchar* public_id, const gchar* system_id, GError** error);
	GXmlDomXMLDocument* (*create_document) (GXmlDomImplementation* self, const gchar* nspace, const gchar* qualified_name, GXmlDomDocumentType* doctype, GError** error);
	GXmlDocument* (*create_html_document) (GXmlDomImplementation* self, const gchar* title);
	gboolean (*has_feature) (GXmlDomImplementation* self);
};

struct _GXmlDomDocumentIface {
	GTypeInterface parent_iface;
	GXmlDomHTMLCollection* (*get_elements_by_tag_name) (GXmlDomDocument* self, const gchar* local_name);
	GXmlDomHTMLCollection* (*get_elements_by_tag_name_ns) (GXmlDomDocument* self, const gchar* namespace, const gchar* local_name);
	GXmlDomHTMLCollection* (*get_elements_by_class_name) (GXmlDomDocument* self, const gchar* classNames);
	GXmlDomElement* (*create_element) (GXmlDomDocument* self, const gchar* local_name, GError** error);
	GXmlDomElement* (*create_element_ns) (GXmlDomDocument* self, const gchar* namespace, const gchar* qualified_name, GError** error);
	GXmlDomDocumentFragment* (*create_document_fragment) (GXmlDomDocument* self);
	GXmlDomText* (*create_text_node) (GXmlDomDocument* self, const gchar* data, GError** error);
	GXmlDomComment* (*create_comment) (GXmlDomDocument* self, const gchar* data, GError** error);
	GXmlDomProcessingInstruction* (*create_processing_instruction) (GXmlDomDocument* self, const gchar* target, const gchar* data, GError** error);
	GXmlDomNode* (*import_node) (GXmlDomDocument* self, GXmlDomNode* node, gboolean deep, GError** error);
	GXmlDomNode* (*adopt_node) (GXmlDomDocument* self, GXmlDomNode* node, GError** error);
	GXmlDomEvent* (*create_event) (GXmlDomDocument* self, const gchar* interface, GError** error);
	GXmlDomRange* (*create_range) (GXmlDomDocument* self);
	GXmlDomNodeIterator* (*create_node_iterator) (GXmlDomDocument* self, GXmlDomNode* root, gint whatToShow, GXmlDomNodeFilter* filter);
	GXmlDomTreeWalker* (*create_tree_walker) (GXmlDomDocument* self, GXmlDomNode* root, gint what_to_show, GXmlDomNodeFilter* filter);
	GXmlDomImplementation* (*get_implementation) (GXmlDomDocument* self);
	const gchar* (*get_url) (GXmlDomDocument* self);
	const gchar* (*get_document_uri) (GXmlDomDocument* self);
	const gchar* (*get_origin) (GXmlDomDocument* self);
	const gchar* (*get_compat_mode) (GXmlDomDocument* self);
	const gchar* (*get_character_set) (GXmlDomDocument* self);
	const gchar* (*get_content_type) (GXmlDomDocument* self);
	GXmlDomDocumentType* (*get_doctype) (GXmlDomDocument* self);
	GXmlDomElement* (*get_document_element) (GXmlDomDocument* self);
};

struct _GXmlDomNodeIface {
	GTypeInterface parent_iface;
	gboolean (*has_child_nodes) (GXmlDomNode* self);
	void (*normalize) (GXmlDomNode* self);
	gboolean (*is_equal_node) (GXmlDomNode* self, GXmlDomNode* node);
	GXmlDomNodeDocumentPosition (*compare_document_position) (GXmlDomNode* self, GXmlDomNode* other);
	gboolean (*contains) (GXmlDomNode* self, GXmlDomNode* other);
	gchar* (*lookup_prefix) (GXmlDomNode* self, const gchar* nspace);
	gchar* (*lookup_namespace_uri) (GXmlDomNode* self, const gchar* prefix);
	gboolean (*is_default_namespace) (GXmlDomNode* self, const gchar* nspace);
	GXmlDomNode* (*insert_before) (GXmlDomNode* self, GXmlDomNode* node, GXmlDomNode* child, GError** error);
	GXmlDomNode* (*append_child) (GXmlDomNode* self, GXmlDomNode* node, GError** error);
	GXmlDomNode* (*replace_child) (GXmlDomNode* self, GXmlDomNode* node, GXmlDomNode* child, GError** error);
	GXmlDomNode* (*remove_child) (GXmlDomNode* self, GXmlDomNode* child, GError** error);
	GXmlDomNode* (*clone_node) (GXmlDomNode* self, gboolean deep, GError** error);
	GXmlDomNodeNodeType (*get_node_type) (GXmlDomNode* self);
	gchar* (*get_node_name) (GXmlDomNode* self);
	const gchar* (*get_base_uri) (GXmlDomNode* self);
	GXmlDomDocument* (*get_owner_document) (GXmlDomNode* self);
	void (*set_owner_document) (GXmlDomNode* self, GXmlDomDocument* value);
	GXmlDomNode* (*get_parent_node) (GXmlDomNode* self);
	GXmlDomElement* (*get_parent_element) (GXmlDomNode* self);
	GXmlDomNodeList* (*get_child_nodes) (GXmlDomNode* self);
	GXmlDomNode* (*get_first_child) (GXmlDomNode* self);
	GXmlDomNode* (*get_last_child) (GXmlDomNode* self);
	GXmlDomNode* (*get_previous_sibling) (GXmlDomNode* self);
	GXmlDomNode* (*get_next_sibling) (GXmlDomNode* self);
	gchar* (*get_node_value) (GXmlDomNode* self);
	void (*set_node_value) (GXmlDomNode* self, const gchar* value);
	gchar* (*get_text_content) (GXmlDomNode* self);
	void (*set_text_content) (GXmlDomNode* self, const gchar* value);
};

struct _GXmlDomNamedNodeMapIface {
	GTypeInterface parent_iface;
	GXmlDomNode* (*item) (GXmlDomNamedNodeMap* self, gint index);
	GXmlDomNode* (*get_named_item) (GXmlDomNamedNodeMap* self, const gchar* name);
	GXmlDomNode* (*set_named_item) (GXmlDomNamedNodeMap* self, GXmlDomNode* node, GError** error);
	GXmlDomNode* (*remove_named_item) (GXmlDomNamedNodeMap* self, const gchar* name, GError** error);
	GXmlDomNode* (*remove_named_item_ns) (GXmlDomNamedNodeMap* self, const gchar* namespace_uri, const gchar* localName, GError** error);
	GXmlDomNode* (*get_named_item_ns) (GXmlDomNamedNodeMap* self, const gchar* namespace_uri, const gchar* local_name, GError** error);
	GXmlDomNode* (*set_named_item_ns) (GXmlDomNamedNodeMap* self, GXmlDomNode* node, GError** error);
	gint (*get_length) (GXmlDomNamedNodeMap* self);
};

struct _GXmlDomElementIface {
	GTypeInterface parent_iface;
	gchar* (*get_attribute) (GXmlDomElement* self, const gchar* name);
	gchar* (*get_attribute_ns) (GXmlDomElement* self, const gchar* namespace, const gchar* local_name);
	void (*set_attribute) (GXmlDomElement* self, const gchar* name, const gchar* value, GError** error);
	void (*set_attribute_ns) (GXmlDomElement* self, const gchar* namespace, const gchar* name, const gchar* value, GError** error);
	void (*remove_attribute) (GXmlDomElement* self, const gchar* name);
	void (*remove_attribute_ns) (GXmlDomElement* self, const gchar* namespace, const gchar* local_name);
	gboolean (*has_attribute) (GXmlDomElement* self, const gchar* name);
	gboolean (*has_attribute_ns) (GXmlDomElement* self, const gchar* namespace, const gchar* local_name);
	GXmlDomHTMLCollection* (*get_elements_by_tag_name) (GXmlDomElement* self, const gchar* local_name);
	GXmlDomHTMLCollection* (*get_elements_by_tag_name_ns) (GXmlDomElement* self, const gchar* namespace, const gchar* local_name);
	GXmlDomHTMLCollection* (*get_elements_by_class_name) (GXmlDomElement* self, const gchar* class_names);
	gchar* (*get_namespace_uri) (GXmlDomElement* self);
	gchar* (*get_prefix) (GXmlDomElement* self);
	gchar* (*get_local_name) (GXmlDomElement* self);
	gchar* (*get_tag_name) (GXmlDomElement* self);
	gchar* (*get_id) (GXmlDomElement* self);
	void (*set_id) (GXmlDomElement* self, const gchar* value);
	gchar* (*get_class_name) (GXmlDomElement* self);
	void (*set_class_name) (GXmlDomElement* self, const gchar* value);
	GXmlDomTokenList* (*get_class_list) (GXmlDomElement* self);
	GXmlDomNamedNodeMap* (*get_attributes) (GXmlDomElement* self);
};

struct _GXmlGDomTokenList {
	GeeArrayList parent_instance;
	GXmlGDomTokenListPrivate * priv;
	GXmlDomElement* _element;
	gchar* _attr;
};

struct _GXmlGDomTokenListClass {
	GeeArrayListClass parent_class;
};

typedef enum  {
	GXML_DOM_ERROR_INDEX_SIZE_ERROR = 1,
	GXML_DOM_ERROR_DOMSTRING_SIZE_ERROR,
	GXML_DOM_ERROR_HIERARCHY_REQUEST_ERROR,
	GXML_DOM_ERROR_WRONG_DOCUMENT_ERROR,
	GXML_DOM_ERROR_INVALID_CHARACTER_ERROR,
	GXML_DOM_ERROR_NO_DATA_ALLOWED_ERROR,
	GXML_DOM_ERROR_NO_MODIFICATION_ALLOWED_ERROR,
	GXML_DOM_ERROR_NOT_FOUND_ERROR,
	GXML_DOM_ERROR_NOT_SUPPORTED_ERROR,
	GXML_DOM_ERROR_INUSE_ATTRIBUTE_ERROR,
	GXML_DOM_ERROR_INVALID_STATE_ERROR,
	GXML_DOM_ERROR_SYNTAX_ERROR,
	GXML_DOM_ERROR_INVALID_MODIFICATION_ERROR,
	GXML_DOM_ERROR_NAMESPACE_ERROR,
	GXML_DOM_ERROR_INVALID_ACCESS_ERROR,
	GXML_DOM_ERROR_VALIDATION_ERROR,
	GXML_DOM_ERROR_TYPE_MISMATCH_ERROR,
	GXML_DOM_ERROR_SECURITY_ERROR,
	GXML_DOM_ERROR_NETWORK_ERROR,
	GXML_DOM_ERROR_ABORT_ERROR,
	GXML_DOM_ERROR_URL_MISMATCH_ERROR,
	GXML_DOM_ERROR_QUOTA_EXCEEDED_ERROR,
	GXML_DOM_ERROR_TIME_OUT_ERROR,
	GXML_DOM_ERROR_INVALID_NODE_TYPE_ERROR,
	GXML_DOM_ERROR_DATA_CLONE_ERROR
} GXmlDomError;
#define GXML_DOM_ERROR gxml_dom_error_quark ()
struct _GXmlDomSettableTokenListIface {
	GTypeInterface parent_iface;
	gchar* (*get_value) (GXmlDomSettableTokenList* self);
	void (*set_value) (GXmlDomSettableTokenList* self, const gchar* value);
};

struct _GXmlGDomSettableTokenList {
	GXmlGDomTokenList parent_instance;
	GXmlGDomSettableTokenListPrivate * priv;
};

struct _GXmlGDomSettableTokenListClass {
	GXmlGDomTokenListClass parent_class;
};

struct _GXmlGDomHTMLCollection {
	GeeArrayList parent_instance;
	GXmlGDomHTMLCollectionPrivate * priv;
};

struct _GXmlGDomHTMLCollectionClass {
	GeeArrayListClass parent_class;
};


static gpointer gxml_gdom_token_list_parent_class = NULL;
static GXmlDomTokenListIface* gxml_gdom_token_list_gxml_dom_token_list_parent_iface = NULL;
static gpointer gxml_gdom_settable_token_list_parent_class = NULL;
static GXmlDomSettableTokenListIface* gxml_gdom_settable_token_list_gxml_dom_settable_token_list_parent_iface = NULL;
static gpointer gxml_gdom_html_collection_parent_class = NULL;
static GXmlDomHTMLCollectionIface* gxml_gdom_html_collection_gxml_dom_html_collection_parent_iface = NULL;

GType gxml_dom_token_list_get_type (void) G_GNUC_CONST;
GType gxml_gdom_token_list_get_type (void) G_GNUC_CONST;
GType gxml_dom_time_stamp_get_type (void) G_GNUC_CONST;
GType gxml_dom_event_phase_get_type (void) G_GNUC_CONST;
GType gxml_dom_event_get_type (void) G_GNUC_CONST;
GType gxml_dom_event_listener_get_type (void) G_GNUC_CONST;
GType gxml_dom_event_target_get_type (void) G_GNUC_CONST;
GType gxml_dom_node_document_position_get_type (void) G_GNUC_CONST;
GType gxml_dom_node_node_type_get_type (void) G_GNUC_CONST;
GType gxml_dom_node_list_get_type (void) G_GNUC_CONST;
GType gxml_dom_element_list_get_type (void) G_GNUC_CONST;
GType gxml_dom_html_collection_get_type (void) G_GNUC_CONST;
GType gxml_dom_parent_node_get_type (void) G_GNUC_CONST;
GType gxml_dom_non_element_parent_node_get_type (void) G_GNUC_CONST;
GType gxml_dom_document_fragment_get_type (void) G_GNUC_CONST;
GType gxml_dom_non_document_type_child_node_get_type (void) G_GNUC_CONST;
GType gxml_dom_child_node_get_type (void) G_GNUC_CONST;
GType gxml_dom_character_data_get_type (void) G_GNUC_CONST;
GType gxml_dom_text_get_type (void) G_GNUC_CONST;
GType gxml_dom_comment_get_type (void) G_GNUC_CONST;
GType gxml_dom_processing_instruction_get_type (void) G_GNUC_CONST;
GType gxml_dom_range_boundary_points_get_type (void) G_GNUC_CONST;
GType gxml_dom_range_get_type (void) G_GNUC_CONST;
GType gxml_dom_node_filter_get_type (void) G_GNUC_CONST;
GType gxml_dom_node_iterator_get_type (void) G_GNUC_CONST;
GType gxml_dom_tree_walker_get_type (void) G_GNUC_CONST;
GType gxml_dom_document_type_get_type (void) G_GNUC_CONST;
GType gxml_dom_xml_document_get_type (void) G_GNUC_CONST;
GType gxml_element_list_get_type (void) G_GNUC_CONST;
GType gxml_namespace_get_type (void) G_GNUC_CONST;
GType gxml_node_type_get_type (void) G_GNUC_CONST;
GType gxml_node_get_type (void) G_GNUC_CONST;
GType gxml_document_get_type (void) G_GNUC_CONST;
GType gxml_dom_implementation_get_type (void) G_GNUC_CONST;
GType gxml_dom_document_get_type (void) G_GNUC_CONST;
GType gxml_dom_node_get_type (void) G_GNUC_CONST;
GType gxml_dom_named_node_map_get_type (void) G_GNUC_CONST;
GType gxml_dom_element_get_type (void) G_GNUC_CONST;
enum  {
	GXML_GDOM_TOKEN_LIST_DUMMY_PROPERTY,
	GXML_GDOM_TOKEN_LIST_LENGTH
};
static gchar* gxml_gdom_token_list_real_item (GXmlDomTokenList* base, gint index);
GXmlGDomTokenList* gxml_gdom_token_list_new (GXmlDomElement* e, const gchar* attr);
GXmlGDomTokenList* gxml_gdom_token_list_construct (GType object_type, GXmlDomElement* e, const gchar* attr);
gchar* gxml_dom_element_get_attribute (GXmlDomElement* self, const gchar* name);
static gboolean gxml_gdom_token_list_real_contains (GXmlDomTokenList* base, const gchar* token, GError** error);
GQuark gxml_dom_error_quark (void);
static void gxml_gdom_token_list_real_add (GXmlDomTokenList* base, gchar** tokens, int tokens_length1, GError** error);
void gxml_gdom_token_list_update (GXmlGDomTokenList* self);
static void gxml_gdom_token_list_real_remove (GXmlDomTokenList* base, gchar** tokens, int tokens_length1);
static gboolean gxml_gdom_token_list_real_toggle (GXmlDomTokenList* base, const gchar* token, gboolean force, gboolean _auto_, GError** error);
gboolean gxml_dom_token_list_contains (GXmlDomTokenList* self, const gchar* token, GError** error);
void gxml_dom_element_set_attribute (GXmlDomElement* self, const gchar* name, const gchar* value, GError** error);
gchar* gxml_dom_token_list_to_string (GXmlDomTokenList* self);
static gchar* gxml_gdom_token_list_real_to_string (GXmlDomTokenList* base);
static void gxml_gdom_token_list_finalize (GObject* obj);
gint gxml_dom_token_list_get_length (GXmlDomTokenList* self);
static void _vala_gxml_gdom_token_list_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
GType gxml_dom_settable_token_list_get_type (void) G_GNUC_CONST;
GType gxml_gdom_settable_token_list_get_type (void) G_GNUC_CONST;
enum  {
	GXML_GDOM_SETTABLE_TOKEN_LIST_DUMMY_PROPERTY,
	GXML_GDOM_SETTABLE_TOKEN_LIST_VALUE
};
GXmlGDomSettableTokenList* gxml_gdom_settable_token_list_new (GXmlDomElement* e, const gchar* attr);
GXmlGDomSettableTokenList* gxml_gdom_settable_token_list_construct (GType object_type, GXmlDomElement* e, const gchar* attr);
gchar* gxml_dom_settable_token_list_get_value (GXmlDomSettableTokenList* self);
static void _vala_gxml_gdom_settable_token_list_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
void gxml_dom_settable_token_list_set_value (GXmlDomSettableTokenList* self, const gchar* value);
static void _vala_gxml_gdom_settable_token_list_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType gxml_gdom_html_collection_get_type (void) G_GNUC_CONST;
enum  {
	GXML_GDOM_HTML_COLLECTION_DUMMY_PROPERTY
};
static GXmlDomElement* gxml_gdom_html_collection_real_item (GXmlDomHTMLCollection* base, gint index);
static GXmlDomElement* gxml_gdom_html_collection_real_named_item (GXmlDomHTMLCollection* base, const gchar* name);
gchar* gxml_dom_node_get_node_name (GXmlDomNode* self);
static GXmlDomElement* gxml_gdom_html_collection_real_get_element (GXmlDomHTMLCollection* base, gint index);
GXmlGDomHTMLCollection* gxml_gdom_html_collection_new (void);
GXmlGDomHTMLCollection* gxml_gdom_html_collection_construct (GType object_type);
static void _vala_gxml_gdom_html_collection_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);


static gchar* gxml_gdom_token_list_real_item (GXmlDomTokenList* base, gint index) {
	GXmlGDomTokenList * self;
	gchar* result = NULL;
	gint _tmp0_ = 0;
	gpointer _tmp1_ = NULL;
	self = (GXmlGDomTokenList*) base;
	_tmp0_ = index;
	_tmp1_ = GEE_ABSTRACT_LIST_CLASS (gxml_gdom_token_list_parent_class)->get ((GeeAbstractList*) G_TYPE_CHECK_INSTANCE_CAST (self, GEE_TYPE_ARRAY_LIST, GeeArrayList), _tmp0_);
	result = (gchar*) _tmp1_;
	return result;
}


static gpointer _g_object_ref0 (gpointer self) {
	return self ? g_object_ref (self) : NULL;
}


static gboolean string_contains (const gchar* self, const gchar* needle) {
	gboolean result = FALSE;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (needle != NULL, FALSE);
	_tmp0_ = needle;
	_tmp1_ = strstr ((gchar*) self, (gchar*) _tmp0_);
	result = _tmp1_ != NULL;
	return result;
}


GXmlGDomTokenList* gxml_gdom_token_list_construct (GType object_type, GXmlDomElement* e, const gchar* attr) {
	GXmlGDomTokenList * self = NULL;
	GXmlDomElement* _tmp0_ = NULL;
	GXmlDomElement* _tmp1_ = NULL;
	const gchar* _tmp2_ = NULL;
	gchar* _tmp3_ = NULL;
	const gchar* _tmp4_ = NULL;
	g_return_val_if_fail (e != NULL, NULL);
	self = (GXmlGDomTokenList*) gee_array_list_construct (object_type, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, NULL, NULL, NULL);
	_tmp0_ = e;
	_tmp1_ = _g_object_ref0 (_tmp0_);
	_g_object_unref0 (self->_element);
	self->_element = _tmp1_;
	_tmp2_ = attr;
	_tmp3_ = g_strdup (_tmp2_);
	_g_free0 (self->_attr);
	self->_attr = _tmp3_;
	_tmp4_ = self->_attr;
	if (_tmp4_ != NULL) {
		gchar* av = NULL;
		GXmlDomElement* _tmp5_ = NULL;
		const gchar* _tmp6_ = NULL;
		gchar* _tmp7_ = NULL;
		const gchar* _tmp8_ = NULL;
		const gchar* _tmp9_ = NULL;
		gboolean _tmp10_ = FALSE;
		_tmp5_ = self->_element;
		_tmp6_ = self->_attr;
		_tmp7_ = gxml_dom_element_get_attribute (_tmp5_, _tmp6_);
		av = _tmp7_;
		_tmp8_ = av;
		if (g_strcmp0 (_tmp8_, "") == 0) {
			_g_free0 (av);
			return self;
		}
		_tmp9_ = av;
		_tmp10_ = string_contains (_tmp9_, " ");
		if (_tmp10_) {
			gchar** s = NULL;
			const gchar* _tmp11_ = NULL;
			gchar** _tmp12_ = NULL;
			gchar** _tmp13_ = NULL;
			gint s_length1 = 0;
			gint _s_size_ = 0;
			_tmp11_ = av;
			_tmp13_ = _tmp12_ = g_strsplit (_tmp11_, " ", 0);
			s = _tmp13_;
			s_length1 = _vala_array_length (_tmp12_);
			_s_size_ = s_length1;
			{
				gint i = 0;
				i = 0;
				{
					gboolean _tmp14_ = FALSE;
					_tmp14_ = TRUE;
					while (TRUE) {
						gint _tmp16_ = 0;
						gchar** _tmp17_ = NULL;
						gint _tmp17__length1 = 0;
						gchar** _tmp18_ = NULL;
						gint _tmp18__length1 = 0;
						gint _tmp19_ = 0;
						const gchar* _tmp20_ = NULL;
						if (!_tmp14_) {
							gint _tmp15_ = 0;
							_tmp15_ = i;
							i = _tmp15_ + 1;
						}
						_tmp14_ = FALSE;
						_tmp16_ = i;
						_tmp17_ = s;
						_tmp17__length1 = s_length1;
						if (!(_tmp16_ < _tmp17__length1)) {
							break;
						}
						_tmp18_ = s;
						_tmp18__length1 = s_length1;
						_tmp19_ = i;
						_tmp20_ = _tmp18_[_tmp19_];
						gee_abstract_collection_add ((GeeAbstractCollection*) (G_TYPE_CHECK_INSTANCE_TYPE (self, GEE_TYPE_ARRAY_LIST) ? ((GeeArrayList*) self) : NULL), _tmp20_);
					}
				}
			}
			s = (_vala_array_free (s, s_length1, (GDestroyNotify) g_free), NULL);
		}
		_g_free0 (av);
	}
	return self;
}


GXmlGDomTokenList* gxml_gdom_token_list_new (GXmlDomElement* e, const gchar* attr) {
	return gxml_gdom_token_list_construct (GXML_TYPE_GDOM_TOKEN_LIST, e, attr);
}


static gboolean gxml_gdom_token_list_real_contains (GXmlDomTokenList* base, const gchar* token, GError** error) {
	GXmlGDomTokenList * self;
	gboolean result = FALSE;
	const gchar* _tmp0_ = NULL;
	const gchar* _tmp4_ = NULL;
	gboolean _tmp5_ = FALSE;
	const gchar* _tmp9_ = NULL;
	gboolean _tmp10_ = FALSE;
	GError * _inner_error_ = NULL;
	self = (GXmlGDomTokenList*) base;
	g_return_val_if_fail (token != NULL, FALSE);
	_tmp0_ = token;
	if (g_strcmp0 (_tmp0_, "") == 0) {
		const gchar* _tmp1_ = NULL;
		GError* _tmp2_ = NULL;
		gboolean _tmp3_ = FALSE;
		_tmp1_ = _ ("DOM: Invalid token. No empty string could be used as token to check if" \
" it is contained in token list");
		_tmp2_ = g_error_new_literal (GXML_DOM_ERROR, GXML_DOM_ERROR_SYNTAX_ERROR, _tmp1_);
		_inner_error_ = _tmp2_;
		g_propagate_error (error, _inner_error_);
		return _tmp3_;
	}
	_tmp4_ = token;
	_tmp5_ = string_contains (_tmp4_, " ");
	if (_tmp5_) {
		const gchar* _tmp6_ = NULL;
		GError* _tmp7_ = NULL;
		gboolean _tmp8_ = FALSE;
		_tmp6_ = _ ("DOM: Invalid token. No white spaces could be included as token to chec" \
"k if it is contained in token list");
		_tmp7_ = g_error_new_literal (GXML_DOM_ERROR, GXML_DOM_ERROR_INVALID_CHARACTER_ERROR, _tmp6_);
		_inner_error_ = _tmp7_;
		g_propagate_error (error, _inner_error_);
		return _tmp8_;
	}
	_tmp9_ = token;
	_tmp10_ = GEE_ABSTRACT_COLLECTION_CLASS (gxml_gdom_token_list_parent_class)->contains ((GeeAbstractCollection*) G_TYPE_CHECK_INSTANCE_CAST (self, GEE_TYPE_ARRAY_LIST, GeeArrayList), _tmp9_);
	result = _tmp10_;
	return result;
}


static void gxml_gdom_token_list_real_add (GXmlDomTokenList* base, gchar** tokens, int tokens_length1, GError** error) {
	GXmlGDomTokenList * self;
	gchar** _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
	GError * _inner_error_ = NULL;
	self = (GXmlGDomTokenList*) base;
	_tmp0_ = tokens;
	_tmp0__length1 = tokens_length1;
	{
		gchar** s_collection = NULL;
		gint s_collection_length1 = 0;
		gint _s_collection_size_ = 0;
		gint s_it = 0;
		s_collection = _tmp0_;
		s_collection_length1 = _tmp0__length1;
		for (s_it = 0; s_it < _tmp0__length1; s_it = s_it + 1) {
			gchar* _tmp1_ = NULL;
			gchar* s = NULL;
			_tmp1_ = g_strdup (s_collection[s_it]);
			s = _tmp1_;
			{
				const gchar* _tmp2_ = NULL;
				const gchar* _tmp5_ = NULL;
				gboolean _tmp6_ = FALSE;
				const gchar* _tmp9_ = NULL;
				_tmp2_ = s;
				if (g_strcmp0 (_tmp2_, "") == 0) {
					const gchar* _tmp3_ = NULL;
					GError* _tmp4_ = NULL;
					_tmp3_ = _ ("DOM: Invalid token. Empty string can't be used as token");
					_tmp4_ = g_error_new_literal (GXML_DOM_ERROR, GXML_DOM_ERROR_SYNTAX_ERROR, _tmp3_);
					_inner_error_ = _tmp4_;
					g_propagate_error (error, _inner_error_);
					_g_free0 (s);
					return;
				}
				_tmp5_ = s;
				_tmp6_ = string_contains (_tmp5_, " ");
				if (_tmp6_) {
					const gchar* _tmp7_ = NULL;
					GError* _tmp8_ = NULL;
					_tmp7_ = _ ("DOM: Invalid token. White spaces can't be used as token");
					_tmp8_ = g_error_new_literal (GXML_DOM_ERROR, GXML_DOM_ERROR_INVALID_CHARACTER_ERROR, _tmp7_);
					_inner_error_ = _tmp8_;
					g_propagate_error (error, _inner_error_);
					_g_free0 (s);
					return;
				}
				_tmp9_ = s;
				GEE_ABSTRACT_COLLECTION_CLASS (gxml_gdom_token_list_parent_class)->add ((GeeAbstractCollection*) G_TYPE_CHECK_INSTANCE_CAST (self, GEE_TYPE_ARRAY_LIST, GeeArrayList), _tmp9_);
				_g_free0 (s);
			}
		}
	}
	gxml_gdom_token_list_update (self);
}


static void gxml_gdom_token_list_real_remove (GXmlDomTokenList* base, gchar** tokens, int tokens_length1) {
	GXmlGDomTokenList * self;
	self = (GXmlGDomTokenList*) base;
	{
		gint i = 0;
		i = 0;
		{
			gboolean _tmp0_ = FALSE;
			_tmp0_ = TRUE;
			while (TRUE) {
				gint _tmp2_ = 0;
				gint _tmp3_ = 0;
				gint _tmp4_ = 0;
				gchar* s = NULL;
				gint _tmp5_ = 0;
				gpointer _tmp6_ = NULL;
				gchar** _tmp7_ = NULL;
				gint _tmp7__length1 = 0;
				if (!_tmp0_) {
					gint _tmp1_ = 0;
					_tmp1_ = i;
					i = _tmp1_ + 1;
				}
				_tmp0_ = FALSE;
				_tmp2_ = i;
				_tmp3_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) self);
				_tmp4_ = _tmp3_;
				if (!(_tmp2_ < _tmp4_)) {
					break;
				}
				_tmp5_ = i;
				_tmp6_ = gee_abstract_list_get ((GeeAbstractList*) self, _tmp5_);
				s = (gchar*) _tmp6_;
				_tmp7_ = tokens;
				_tmp7__length1 = tokens_length1;
				{
					gchar** ts_collection = NULL;
					gint ts_collection_length1 = 0;
					gint _ts_collection_size_ = 0;
					gint ts_it = 0;
					ts_collection = _tmp7_;
					ts_collection_length1 = _tmp7__length1;
					for (ts_it = 0; ts_it < _tmp7__length1; ts_it = ts_it + 1) {
						gchar* _tmp8_ = NULL;
						gchar* ts = NULL;
						_tmp8_ = g_strdup (ts_collection[ts_it]);
						ts = _tmp8_;
						{
							const gchar* _tmp9_ = NULL;
							const gchar* _tmp10_ = NULL;
							_tmp9_ = s;
							_tmp10_ = ts;
							if (g_strcmp0 (_tmp9_, _tmp10_) == 0) {
								gint _tmp11_ = 0;
								gpointer _tmp12_ = NULL;
								gchar* _tmp13_ = NULL;
								_tmp11_ = i;
								_tmp12_ = GEE_ABSTRACT_LIST_CLASS (gxml_gdom_token_list_parent_class)->remove_at ((GeeAbstractList*) G_TYPE_CHECK_INSTANCE_CAST (self, GEE_TYPE_ARRAY_LIST, GeeArrayList), _tmp11_);
								_tmp13_ = (gchar*) _tmp12_;
								_g_free0 (_tmp13_);
							}
							_g_free0 (ts);
						}
					}
				}
				_g_free0 (s);
			}
		}
	}
	gxml_gdom_token_list_update (self);
}


/**
   * A convenient method to remove or add tokens. If force is true, this method
   * will add it to the list of tokens, same as {@link add}. If force is false
   * will remove a token from the list {@link remove}
   */
static gboolean gxml_gdom_token_list_real_toggle (GXmlDomTokenList* base, const gchar* token, gboolean force, gboolean _auto_, GError** error) {
	GXmlGDomTokenList * self;
	gboolean result = FALSE;
	const gchar* _tmp0_ = NULL;
	const gchar* _tmp4_ = NULL;
	gboolean _tmp5_ = FALSE;
	gboolean _tmp9_ = FALSE;
	gboolean _tmp10_ = FALSE;
	const gchar* _tmp11_ = NULL;
	gboolean _tmp12_ = FALSE;
	GError * _inner_error_ = NULL;
	self = (GXmlGDomTokenList*) base;
	g_return_val_if_fail (token != NULL, FALSE);
	_tmp0_ = token;
	if (g_strcmp0 (_tmp0_, "") == 0) {
		const gchar* _tmp1_ = NULL;
		GError* _tmp2_ = NULL;
		gboolean _tmp3_ = FALSE;
		_tmp1_ = _ ("DOM: Invalid token. Empty string can't be used as token");
		_tmp2_ = g_error_new_literal (GXML_DOM_ERROR, GXML_DOM_ERROR_SYNTAX_ERROR, _tmp1_);
		_inner_error_ = _tmp2_;
		g_propagate_error (error, _inner_error_);
		return _tmp3_;
	}
	_tmp4_ = token;
	_tmp5_ = string_contains (_tmp4_, " ");
	if (_tmp5_) {
		const gchar* _tmp6_ = NULL;
		GError* _tmp7_ = NULL;
		gboolean _tmp8_ = FALSE;
		_tmp6_ = _ ("DOM: Invalid token. White spaces can't be used as token");
		_tmp7_ = g_error_new_literal (GXML_DOM_ERROR, GXML_DOM_ERROR_INVALID_CHARACTER_ERROR, _tmp6_);
		_inner_error_ = _tmp7_;
		g_propagate_error (error, _inner_error_);
		return _tmp8_;
	}
	_tmp11_ = token;
	_tmp12_ = gxml_dom_token_list_contains ((GXmlDomTokenList*) self, _tmp11_, &_inner_error_);
	_tmp10_ = _tmp12_;
	if (G_UNLIKELY (_inner_error_ != NULL)) {
		gboolean _tmp13_ = FALSE;
		g_propagate_error (error, _inner_error_);
		return _tmp13_;
	}
	if (_tmp10_) {
		gboolean _tmp14_ = FALSE;
		_tmp14_ = _auto_;
		_tmp9_ = _tmp14_;
	} else {
		_tmp9_ = FALSE;
	}
	if (_tmp9_) {
		const gchar* _tmp15_ = NULL;
		gint _tmp16_ = 0;
		gpointer _tmp17_ = NULL;
		gchar* _tmp18_ = NULL;
		_tmp15_ = token;
		_tmp16_ = gee_abstract_list_index_of ((GeeAbstractList*) self, _tmp15_);
		_tmp17_ = gee_abstract_list_remove_at ((GeeAbstractList*) self, _tmp16_);
		_tmp18_ = (gchar*) _tmp17_;
		_g_free0 (_tmp18_);
		result = FALSE;
		return result;
	} else {
		gboolean _tmp19_ = FALSE;
		_tmp19_ = force;
		if (!_tmp19_) {
			result = FALSE;
			return result;
		}
	}
	gxml_gdom_token_list_update (self);
	result = TRUE;
	return result;
}


void gxml_gdom_token_list_update (GXmlGDomTokenList* self) {
	GXmlDomElement* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
	GError * _inner_error_ = NULL;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->_element;
	if (_tmp0_ == NULL) {
		return;
	}
	_tmp1_ = self->_attr;
	if (_tmp1_ == NULL) {
		return;
	}
	{
		GXmlDomElement* _tmp2_ = NULL;
		const gchar* _tmp3_ = NULL;
		gchar* _tmp4_ = NULL;
		gchar* _tmp5_ = NULL;
		_tmp2_ = self->_element;
		_tmp3_ = self->_attr;
		_tmp4_ = gxml_dom_token_list_to_string ((GXmlDomTokenList*) self);
		_tmp5_ = _tmp4_;
		gxml_dom_element_set_attribute (_tmp2_, _tmp3_, _tmp5_, &_inner_error_);
		_g_free0 (_tmp5_);
		if (G_UNLIKELY (_inner_error_ != NULL)) {
			goto __catch11_g_error;
		}
	}
	goto __finally11;
	__catch11_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp6_ = NULL;
		GError* _tmp7_ = NULL;
		const gchar* _tmp8_ = NULL;
		gchar* _tmp9_ = NULL;
		gchar* _tmp10_ = NULL;
		e = _inner_error_;
		_inner_error_ = NULL;
		_tmp6_ = _ ("Update Error: ");
		_tmp7_ = e;
		_tmp8_ = _tmp7_->message;
		_tmp9_ = g_strconcat (_tmp6_, _tmp8_, NULL);
		_tmp10_ = _tmp9_;
		g_warning ("GXmlDomCollections.vala:100: %s", _tmp10_);
		_g_free0 (_tmp10_);
		_g_error_free0 (e);
	}
	__finally11:
	if (G_UNLIKELY (_inner_error_ != NULL)) {
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
		g_clear_error (&_inner_error_);
		return;
	}
}


static gchar* gxml_gdom_token_list_real_to_string (GXmlDomTokenList* base) {
	GXmlGDomTokenList * self;
	gchar* result = NULL;
	gchar* s = NULL;
	gchar* _tmp0_ = NULL;
	self = (GXmlGDomTokenList*) base;
	_tmp0_ = g_strdup ("");
	s = _tmp0_;
	{
		gint i = 0;
		i = 0;
		{
			gboolean _tmp1_ = FALSE;
			_tmp1_ = TRUE;
			while (TRUE) {
				gint _tmp3_ = 0;
				gint _tmp4_ = 0;
				gint _tmp5_ = 0;
				const gchar* _tmp6_ = NULL;
				gint _tmp7_ = 0;
				gpointer _tmp8_ = NULL;
				gchar* _tmp9_ = NULL;
				gchar* _tmp10_ = NULL;
				gint _tmp11_ = 0;
				gint _tmp12_ = 0;
				gint _tmp13_ = 0;
				if (!_tmp1_) {
					gint _tmp2_ = 0;
					_tmp2_ = i;
					i = _tmp2_ + 1;
				}
				_tmp1_ = FALSE;
				_tmp3_ = i;
				_tmp4_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) self);
				_tmp5_ = _tmp4_;
				if (!(_tmp3_ < _tmp5_)) {
					break;
				}
				_tmp6_ = s;
				_tmp7_ = i;
				_tmp8_ = gee_abstract_list_get ((GeeAbstractList*) self, _tmp7_);
				_tmp9_ = (gchar*) _tmp8_;
				_tmp10_ = g_strconcat (_tmp6_, _tmp9_, NULL);
				_g_free0 (s);
				s = _tmp10_;
				_g_free0 (_tmp9_);
				_tmp11_ = i;
				_tmp12_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) self);
				_tmp13_ = _tmp12_;
				if ((_tmp11_ + 1) < _tmp13_) {
					const gchar* _tmp14_ = NULL;
					gchar* _tmp15_ = NULL;
					_tmp14_ = s;
					_tmp15_ = g_strconcat (_tmp14_, " ", NULL);
					_g_free0 (s);
					s = _tmp15_;
				}
			}
		}
	}
	result = s;
	return result;
}


static gint gxml_gdom_token_list_real_get_length (GXmlDomTokenList* base) {
	gint result;
	GXmlGDomTokenList* self;
	gint _tmp0_ = 0;
	gint _tmp1_ = 0;
	self = (GXmlGDomTokenList*) base;
	_tmp0_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) self);
	_tmp1_ = _tmp0_;
	result = _tmp1_;
	return result;
}


static void gxml_gdom_token_list_class_init (GXmlGDomTokenListClass * klass) {
	gxml_gdom_token_list_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->get_property = _vala_gxml_gdom_token_list_get_property;
	G_OBJECT_CLASS (klass)->finalize = gxml_gdom_token_list_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GXML_GDOM_TOKEN_LIST_LENGTH, g_param_spec_int ("length", "length", "length", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void gxml_gdom_token_list_gxml_dom_token_list_interface_init (GXmlDomTokenListIface * iface) {
	gxml_gdom_token_list_gxml_dom_token_list_parent_iface = g_type_interface_peek_parent (iface);
	iface->item = (gchar* (*)(GXmlDomTokenList*, gint)) gxml_gdom_token_list_real_item;
	iface->contains = (gboolean (*)(GXmlDomTokenList*, const gchar*, GError**)) gxml_gdom_token_list_real_contains;
	iface->add = (void (*)(GXmlDomTokenList*, gchar**, int, GError**)) gxml_gdom_token_list_real_add;
	iface->remove = (void (*)(GXmlDomTokenList*, gchar**, int)) gxml_gdom_token_list_real_remove;
	iface->toggle = (gboolean (*)(GXmlDomTokenList*, const gchar*, gboolean, gboolean, GError**)) gxml_gdom_token_list_real_toggle;
	iface->to_string = (gchar* (*)(GXmlDomTokenList*)) gxml_gdom_token_list_real_to_string;
	iface->get_length = gxml_gdom_token_list_real_get_length;
}


static void gxml_gdom_token_list_instance_init (GXmlGDomTokenList * self) {
	self->_attr = NULL;
}


static void gxml_gdom_token_list_finalize (GObject* obj) {
	GXmlGDomTokenList * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GXML_TYPE_GDOM_TOKEN_LIST, GXmlGDomTokenList);
	_g_object_unref0 (self->_element);
	_g_free0 (self->_attr);
	G_OBJECT_CLASS (gxml_gdom_token_list_parent_class)->finalize (obj);
}


/**
 * List of string tokens.
 */
GType gxml_gdom_token_list_get_type (void) {
	static volatile gsize gxml_gdom_token_list_type_id__volatile = 0;
	if (g_once_init_enter (&gxml_gdom_token_list_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GXmlGDomTokenListClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gxml_gdom_token_list_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GXmlGDomTokenList), 0, (GInstanceInitFunc) gxml_gdom_token_list_instance_init, NULL };
		static const GInterfaceInfo gxml_dom_token_list_info = { (GInterfaceInitFunc) gxml_gdom_token_list_gxml_dom_token_list_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType gxml_gdom_token_list_type_id;
		gxml_gdom_token_list_type_id = g_type_register_static (GEE_TYPE_ARRAY_LIST, "GXmlGDomTokenList", &g_define_type_info, 0);
		g_type_add_interface_static (gxml_gdom_token_list_type_id, GXML_TYPE_DOM_TOKEN_LIST, &gxml_dom_token_list_info);
		g_once_init_leave (&gxml_gdom_token_list_type_id__volatile, gxml_gdom_token_list_type_id);
	}
	return gxml_gdom_token_list_type_id__volatile;
}


static void _vala_gxml_gdom_token_list_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GXmlGDomTokenList * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GXML_TYPE_GDOM_TOKEN_LIST, GXmlGDomTokenList);
	switch (property_id) {
		case GXML_GDOM_TOKEN_LIST_LENGTH:
		g_value_set_int (value, gxml_dom_token_list_get_length ((GXmlDomTokenList*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


GXmlGDomSettableTokenList* gxml_gdom_settable_token_list_construct (GType object_type, GXmlDomElement* e, const gchar* attr) {
	GXmlGDomSettableTokenList * self = NULL;
	GXmlDomElement* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
	g_return_val_if_fail (e != NULL, NULL);
	_tmp0_ = e;
	_tmp1_ = attr;
	self = (GXmlGDomSettableTokenList*) gxml_gdom_token_list_construct (object_type, _tmp0_, _tmp1_);
	return self;
}


GXmlGDomSettableTokenList* gxml_gdom_settable_token_list_new (GXmlDomElement* e, const gchar* attr) {
	return gxml_gdom_settable_token_list_construct (GXML_TYPE_GDOM_SETTABLE_TOKEN_LIST, e, attr);
}


static gchar* gxml_gdom_settable_token_list_real_get_value (GXmlDomSettableTokenList* base) {
	gchar* result;
	GXmlGDomSettableTokenList* self;
	gchar* _tmp0_ = NULL;
	self = (GXmlGDomSettableTokenList*) base;
	_tmp0_ = gxml_dom_token_list_to_string ((GXmlDomTokenList*) self);
	result = _tmp0_;
	return result;
}


static void gxml_gdom_settable_token_list_real_set_value (GXmlDomSettableTokenList* base, const gchar* value) {
	GXmlGDomSettableTokenList* self;
	gchar** s = NULL;
	const gchar* _tmp0_ = NULL;
	gchar** _tmp1_ = NULL;
	gchar** _tmp2_ = NULL;
	gint s_length1 = 0;
	gint _s_size_ = 0;
	self = (GXmlGDomSettableTokenList*) base;
	_tmp0_ = value;
	_tmp2_ = _tmp1_ = g_strsplit (_tmp0_, " ", 0);
	s = _tmp2_;
	s_length1 = _vala_array_length (_tmp1_);
	_s_size_ = s_length1;
	{
		gint i = 0;
		i = 0;
		{
			gboolean _tmp3_ = FALSE;
			_tmp3_ = TRUE;
			while (TRUE) {
				gint _tmp5_ = 0;
				gchar** _tmp6_ = NULL;
				gint _tmp6__length1 = 0;
				gchar** _tmp7_ = NULL;
				gint _tmp7__length1 = 0;
				gint _tmp8_ = 0;
				const gchar* _tmp9_ = NULL;
				if (!_tmp3_) {
					gint _tmp4_ = 0;
					_tmp4_ = i;
					i = _tmp4_ + 1;
				}
				_tmp3_ = FALSE;
				_tmp5_ = i;
				_tmp6_ = s;
				_tmp6__length1 = s_length1;
				if (!(_tmp5_ < _tmp6__length1)) {
					break;
				}
				_tmp7_ = s;
				_tmp7__length1 = s_length1;
				_tmp8_ = i;
				_tmp9_ = _tmp7_[_tmp8_];
				gee_abstract_collection_add ((GeeAbstractCollection*) (G_TYPE_CHECK_INSTANCE_TYPE (self, GEE_TYPE_ARRAY_LIST) ? ((GeeArrayList*) self) : NULL), _tmp9_);
			}
		}
	}
	s = (_vala_array_free (s, s_length1, (GDestroyNotify) g_free), NULL);
	g_object_notify ((GObject *) self, "value");
}


static void gxml_gdom_settable_token_list_class_init (GXmlGDomSettableTokenListClass * klass) {
	gxml_gdom_settable_token_list_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->get_property = _vala_gxml_gdom_settable_token_list_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gxml_gdom_settable_token_list_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GXML_GDOM_SETTABLE_TOKEN_LIST_VALUE, g_param_spec_string ("value", "value", "value", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void gxml_gdom_settable_token_list_gxml_dom_settable_token_list_interface_init (GXmlDomSettableTokenListIface * iface) {
	gxml_gdom_settable_token_list_gxml_dom_settable_token_list_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_value = gxml_gdom_settable_token_list_real_get_value;
	iface->set_value = gxml_gdom_settable_token_list_real_set_value;
}


static void gxml_gdom_settable_token_list_instance_init (GXmlGDomSettableTokenList * self) {
}


GType gxml_gdom_settable_token_list_get_type (void) {
	static volatile gsize gxml_gdom_settable_token_list_type_id__volatile = 0;
	if (g_once_init_enter (&gxml_gdom_settable_token_list_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GXmlGDomSettableTokenListClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gxml_gdom_settable_token_list_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GXmlGDomSettableTokenList), 0, (GInstanceInitFunc) gxml_gdom_settable_token_list_instance_init, NULL };
		static const GInterfaceInfo gxml_dom_settable_token_list_info = { (GInterfaceInitFunc) gxml_gdom_settable_token_list_gxml_dom_settable_token_list_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType gxml_gdom_settable_token_list_type_id;
		gxml_gdom_settable_token_list_type_id = g_type_register_static (GXML_TYPE_GDOM_TOKEN_LIST, "GXmlGDomSettableTokenList", &g_define_type_info, 0);
		g_type_add_interface_static (gxml_gdom_settable_token_list_type_id, GXML_TYPE_DOM_SETTABLE_TOKEN_LIST, &gxml_dom_settable_token_list_info);
		g_once_init_leave (&gxml_gdom_settable_token_list_type_id__volatile, gxml_gdom_settable_token_list_type_id);
	}
	return gxml_gdom_settable_token_list_type_id__volatile;
}


static void _vala_gxml_gdom_settable_token_list_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GXmlGDomSettableTokenList * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GXML_TYPE_GDOM_SETTABLE_TOKEN_LIST, GXmlGDomSettableTokenList);
	switch (property_id) {
		case GXML_GDOM_SETTABLE_TOKEN_LIST_VALUE:
		g_value_take_string (value, gxml_dom_settable_token_list_get_value ((GXmlDomSettableTokenList*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_gxml_gdom_settable_token_list_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GXmlGDomSettableTokenList * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GXML_TYPE_GDOM_SETTABLE_TOKEN_LIST, GXmlGDomSettableTokenList);
	switch (property_id) {
		case GXML_GDOM_SETTABLE_TOKEN_LIST_VALUE:
		gxml_dom_settable_token_list_set_value ((GXmlDomSettableTokenList*) self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static GXmlDomElement* gxml_gdom_html_collection_real_item (GXmlDomHTMLCollection* base, gint index) {
	GXmlGDomHTMLCollection * self;
	GXmlDomElement* result = NULL;
	gint _tmp0_ = 0;
	gpointer _tmp1_ = NULL;
	self = (GXmlGDomHTMLCollection*) base;
	_tmp0_ = index;
	_tmp1_ = GEE_ABSTRACT_LIST_CLASS (gxml_gdom_html_collection_parent_class)->get ((GeeAbstractList*) G_TYPE_CHECK_INSTANCE_CAST (self, GEE_TYPE_ARRAY_LIST, GeeArrayList), _tmp0_);
	result = (GXmlDomElement*) _tmp1_;
	return result;
}


static GXmlDomElement* gxml_gdom_html_collection_real_named_item (GXmlDomHTMLCollection* base, const gchar* name) {
	GXmlGDomHTMLCollection * self;
	GXmlDomElement* result = NULL;
	self = (GXmlGDomHTMLCollection*) base;
	g_return_val_if_fail (name != NULL, NULL);
	{
		GXmlGDomHTMLCollection* _e_list = NULL;
		GXmlGDomHTMLCollection* _tmp0_ = NULL;
		gint _e_size = 0;
		GXmlGDomHTMLCollection* _tmp1_ = NULL;
		gint _tmp2_ = 0;
		gint _tmp3_ = 0;
		gint _e_index = 0;
		_tmp0_ = _g_object_ref0 (self);
		_e_list = _tmp0_;
		_tmp1_ = _e_list;
		_tmp2_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp1_);
		_tmp3_ = _tmp2_;
		_e_size = _tmp3_;
		_e_index = -1;
		while (TRUE) {
			gint _tmp4_ = 0;
			gint _tmp5_ = 0;
			gint _tmp6_ = 0;
			GXmlDomElement* e = NULL;
			GXmlGDomHTMLCollection* _tmp7_ = NULL;
			gint _tmp8_ = 0;
			gpointer _tmp9_ = NULL;
			GXmlDomElement* _tmp10_ = NULL;
			gchar* _tmp11_ = NULL;
			gchar* _tmp12_ = NULL;
			gchar* _tmp13_ = NULL;
			const gchar* _tmp14_ = NULL;
			gboolean _tmp15_ = FALSE;
			_tmp4_ = _e_index;
			_e_index = _tmp4_ + 1;
			_tmp5_ = _e_index;
			_tmp6_ = _e_size;
			if (!(_tmp5_ < _tmp6_)) {
				break;
			}
			_tmp7_ = _e_list;
			_tmp8_ = _e_index;
			_tmp9_ = gee_abstract_list_get ((GeeAbstractList*) _tmp7_, _tmp8_);
			e = (GXmlDomElement*) _tmp9_;
			_tmp10_ = e;
			_tmp11_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp10_);
			_tmp12_ = _tmp11_;
			_tmp13_ = _tmp12_;
			_tmp14_ = name;
			_tmp15_ = g_strcmp0 (_tmp13_, _tmp14_) == 0;
			_g_free0 (_tmp13_);
			if (_tmp15_) {
				result = e;
				_g_object_unref0 (_e_list);
				return result;
			}
			_g_object_unref0 (e);
		}
		_g_object_unref0 (_e_list);
	}
	result = NULL;
	return result;
}


static GXmlDomElement* gxml_gdom_html_collection_real_get_element (GXmlDomHTMLCollection* base, gint index) {
	GXmlGDomHTMLCollection * self;
	GXmlDomElement* result = NULL;
	gint _tmp0_ = 0;
	gpointer _tmp1_ = NULL;
	self = (GXmlGDomHTMLCollection*) base;
	_tmp0_ = index;
	_tmp1_ = gee_abstract_list_get ((GeeAbstractList*) self, _tmp0_);
	result = G_TYPE_CHECK_INSTANCE_CAST ((GXmlDomElement*) _tmp1_, GXML_TYPE_DOM_ELEMENT, GXmlDomElement);
	return result;
}


GXmlGDomHTMLCollection* gxml_gdom_html_collection_construct (GType object_type) {
	GXmlGDomHTMLCollection * self = NULL;
	self = (GXmlGDomHTMLCollection*) gee_array_list_construct (object_type, GXML_TYPE_DOM_ELEMENT, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, NULL, NULL, NULL);
	return self;
}


GXmlGDomHTMLCollection* gxml_gdom_html_collection_new (void) {
	return gxml_gdom_html_collection_construct (GXML_TYPE_GDOM_HTML_COLLECTION);
}


static gint gxml_gdom_html_collection_real_get_length (GXmlDomHTMLCollection* base) {
	gint result;
	GXmlGDomHTMLCollection* self;
	gint _tmp0_ = 0;
	gint _tmp1_ = 0;
	self = (GXmlGDomHTMLCollection*) base;
	_tmp0_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) self);
	_tmp1_ = _tmp0_;
	result = _tmp1_;
	return result;
}


static void gxml_gdom_html_collection_class_init (GXmlGDomHTMLCollectionClass * klass) {
	gxml_gdom_html_collection_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->get_property = _vala_gxml_gdom_html_collection_get_property;
}


static void gxml_gdom_html_collection_gxml_dom_html_collection_interface_init (GXmlDomHTMLCollectionIface * iface) {
	gxml_gdom_html_collection_gxml_dom_html_collection_parent_iface = g_type_interface_peek_parent (iface);
	iface->item = (GXmlDomElement* (*)(GXmlDomHTMLCollection*, gint)) gxml_gdom_html_collection_real_item;
	iface->named_item = (GXmlDomElement* (*)(GXmlDomHTMLCollection*, const gchar*)) gxml_gdom_html_collection_real_named_item;
	iface->get_element = (GXmlDomElement* (*)(GXmlDomHTMLCollection*, gint)) gxml_gdom_html_collection_real_get_element;
	iface->get_length = gxml_gdom_html_collection_real_get_length;
}


static void gxml_gdom_html_collection_instance_init (GXmlGDomHTMLCollection * self) {
}


GType gxml_gdom_html_collection_get_type (void) {
	static volatile gsize gxml_gdom_html_collection_type_id__volatile = 0;
	if (g_once_init_enter (&gxml_gdom_html_collection_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GXmlGDomHTMLCollectionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gxml_gdom_html_collection_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GXmlGDomHTMLCollection), 0, (GInstanceInitFunc) gxml_gdom_html_collection_instance_init, NULL };
		static const GInterfaceInfo gxml_dom_html_collection_info = { (GInterfaceInitFunc) gxml_gdom_html_collection_gxml_dom_html_collection_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType gxml_gdom_html_collection_type_id;
		gxml_gdom_html_collection_type_id = g_type_register_static (GEE_TYPE_ARRAY_LIST, "GXmlGDomHTMLCollection", &g_define_type_info, 0);
		g_type_add_interface_static (gxml_gdom_html_collection_type_id, GXML_TYPE_DOM_HTML_COLLECTION, &gxml_dom_html_collection_info);
		g_once_init_leave (&gxml_gdom_html_collection_type_id__volatile, gxml_gdom_html_collection_type_id);
	}
	return gxml_gdom_html_collection_type_id__volatile;
}


static void _vala_gxml_gdom_html_collection_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GXmlGDomHTMLCollection * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GXML_TYPE_GDOM_HTML_COLLECTION, GXmlGDomHTMLCollection);
	switch (property_id) {
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}



