/* HtmlDocumentTest.c generated by valac 0.34.6, the Vala compiler
 * generated from HtmlDocumentTest.vala, do not modify */

/* -*- Mode: vala; indent-tabs-mode: t; c-basic-offset: 2; tab-width: 2 -*- */
/* Notation.vala
 *
 * Copyright (C) 2011-2013  Richard Schwarting <aquarichy@gmail.com>
 * Copyright (C) 2011-2015  Daniel Espinosa <esodan@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Richard Schwarting <aquarichy@gmail.com>
 *      Daniel Espinosa <esodan@gmail.com>
 */

#include <glib.h>
#include <glib-object.h>
#include <gxml/gxml.h>
#include <stdlib.h>
#include <string.h>
#include <gee.h>


#define TYPE_GXML_TEST (gxml_test_get_type ())
#define GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GXML_TEST, GXmlTest))
#define GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GXML_TEST, GXmlTestClass))
#define IS_GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GXML_TEST))
#define IS_GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GXML_TEST))
#define GXML_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GXML_TEST, GXmlTestClass))

typedef struct _GXmlTest GXmlTest;
typedef struct _GXmlTestClass GXmlTestClass;
typedef struct _GXmlTestPrivate GXmlTestPrivate;

#define TYPE_HTML_DOCUMENT_TEST (html_document_test_get_type ())
#define HTML_DOCUMENT_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_HTML_DOCUMENT_TEST, HtmlDocumentTest))
#define HTML_DOCUMENT_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_HTML_DOCUMENT_TEST, HtmlDocumentTestClass))
#define IS_HTML_DOCUMENT_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_HTML_DOCUMENT_TEST))
#define IS_HTML_DOCUMENT_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_HTML_DOCUMENT_TEST))
#define HTML_DOCUMENT_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_HTML_DOCUMENT_TEST, HtmlDocumentTestClass))

typedef struct _HtmlDocumentTest HtmlDocumentTest;
typedef struct _HtmlDocumentTestClass HtmlDocumentTestClass;
typedef struct _HtmlDocumentTestPrivate HtmlDocumentTestPrivate;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _GXmlTest {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GXmlTestPrivate * priv;
};

struct _GXmlTestClass {
	GTypeClass parent_class;
	void (*finalize) (GXmlTest *self);
};

struct _HtmlDocumentTest {
	GXmlTest parent_instance;
	HtmlDocumentTestPrivate * priv;
};

struct _HtmlDocumentTestClass {
	GXmlTestClass parent_class;
};


static gpointer html_document_test_parent_class = NULL;

gpointer gxml_test_ref (gpointer instance);
void gxml_test_unref (gpointer instance);
GParamSpec* param_spec_gxml_test (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_gxml_test (GValue* value, gpointer v_object);
void value_take_gxml_test (GValue* value, gpointer v_object);
gpointer value_get_gxml_test (const GValue* value);
GType gxml_test_get_type (void) G_GNUC_CONST;
GType html_document_test_get_type (void) G_GNUC_CONST;
enum  {
	HTML_DOCUMENT_TEST_DUMMY_PROPERTY
};
void html_document_test_add_tests (void);
static void __lambda191_ (void);
static void ___lambda191__gtest_func (void);
static void __lambda192_ (void);
static void ___lambda192__gtest_func (void);
HtmlDocumentTest* html_document_test_new (void);
HtmlDocumentTest* html_document_test_construct (GType object_type);
GXmlTest* gxml_test_new (void);
GXmlTest* gxml_test_construct (GType object_type);


static void __lambda191_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlHtmlDocument* doc = NULL;
		GXmlHtmlDocument* _tmp0_ = NULL;
		GXmlHtmlDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlNode* _tmp3_ = NULL;
		GXmlNode* _tmp4_ = NULL;
		GXmlHtmlDocument* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlNode* _tmp8_ = NULL;
		gchar* _tmp9_ = NULL;
		gchar* _tmp10_ = NULL;
		gchar* _tmp11_ = NULL;
		gchar* _tmp12_ = NULL;
		gchar* _tmp13_ = NULL;
		gchar* _tmp14_ = NULL;
		gchar* _tmp15_ = NULL;
		GXmlDomElement* n = NULL;
		GXmlHtmlDocument* _tmp16_ = NULL;
		GXmlDomElement* _tmp17_ = NULL;
		GXmlDomElement* _tmp18_ = NULL;
		GXmlDomElement* _tmp19_ = NULL;
		gchar* _tmp20_ = NULL;
		gchar* _tmp21_ = NULL;
		gchar* _tmp22_ = NULL;
		GXmlDomElement* _tmp23_ = NULL;
		GXmlDomElement* _tmp24_ = NULL;
		gchar* _tmp25_ = NULL;
		gchar* _tmp26_ = NULL;
		gchar* _tmp27_ = NULL;
#line 31 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp0_ = gxml_html_document_new_from_path (TEST_DIR "/index.html", 0, &_inner_error_);
#line 31 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		doc = _tmp0_;
#line 31 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 150 "HtmlDocumentTest.c"
			goto __catch177_g_error;
		}
#line 32 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		g_test_message ("Checking root element...");
#line 33 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp1_ = doc;
#line 33 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp2_ = gxml_document_get_root ((GXmlDocument*) _tmp1_);
#line 33 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp3_ = _tmp2_;
#line 33 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp4_ = _tmp3_;
#line 33 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_vala_assert (_tmp4_ != NULL, "doc.root != null");
#line 33 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_object_unref0 (_tmp4_);
#line 34 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp5_ = doc;
#line 34 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp6_ = gxml_document_get_root ((GXmlDocument*) _tmp5_);
#line 34 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp7_ = _tmp6_;
#line 34 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp8_ = _tmp7_;
#line 34 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp9_ = gxml_node_get_name (_tmp8_);
#line 34 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp10_ = _tmp9_;
#line 34 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp11_ = _tmp10_;
#line 34 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp12_ = g_utf8_strdown (_tmp11_, (gssize) -1);
#line 34 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp13_ = _tmp12_;
#line 34 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp14_ = g_utf8_strdown ("html", (gssize) -1);
#line 34 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp15_ = _tmp14_;
#line 34 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp13_, _tmp15_) == 0, "doc.root.name.down () == \"html\".down ()");
#line 34 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_free0 (_tmp15_);
#line 34 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_free0 (_tmp13_);
#line 34 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_free0 (_tmp11_);
#line 34 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_object_unref0 (_tmp8_);
#line 35 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		g_test_message ("Searching for elemento with id 'user'...");
#line 36 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp16_ = doc;
#line 36 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp17_ = gxml_dom_non_element_parent_node_get_element_by_id ((GXmlDomNonElementParentNode*) _tmp16_, "user", &_inner_error_);
#line 36 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		n = _tmp17_;
#line 36 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 36 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
			_g_object_unref0 (doc);
#line 211 "HtmlDocumentTest.c"
			goto __catch177_g_error;
		}
#line 37 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp18_ = n;
#line 37 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_vala_assert (_tmp18_ != NULL, "n != null");
#line 38 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp19_ = n;
#line 38 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp20_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp19_);
#line 38 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp21_ = _tmp20_;
#line 38 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp22_ = _tmp21_;
#line 38 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp22_, "p") == 0, "n.node_name == \"p\"");
#line 38 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_free0 (_tmp22_);
#line 39 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp23_ = n;
#line 39 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp23_, GXML_TYPE_ELEMENT), "n is GXml.Element");
#line 40 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp24_ = n;
#line 40 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp25_ = gxml_element_get_content (G_TYPE_CHECK_INSTANCE_CAST (_tmp24_, GXML_TYPE_ELEMENT, GXmlElement));
#line 40 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp26_ = _tmp25_;
#line 40 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp27_ = _tmp26_;
#line 40 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp27_, "") == 0, "((GXml.Element) n).content == \"\"");
#line 40 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_free0 (_tmp27_);
#line 30 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_object_unref0 (n);
#line 30 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_object_unref0 (doc);
#line 250 "HtmlDocumentTest.c"
	}
	goto __finally177;
	__catch177_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp28_ = NULL;
		gchar* _tmp29_ = NULL;
		gchar* _tmp30_ = NULL;
#line 30 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		e = _inner_error_;
#line 30 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_inner_error_ = NULL;
#line 42 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp28_ = e->message;
#line 42 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp29_ = g_strconcat ("ERROR: ", _tmp28_, NULL);
#line 42 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp30_ = _tmp29_;
#line 42 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		g_test_message ("%s", _tmp30_);
#line 42 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_free0 (_tmp30_);
#line 43 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		g_assert_not_reached ();
#line 30 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_error_free0 (e);
#line 277 "HtmlDocumentTest.c"
	}
	__finally177:
#line 30 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 30 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 30 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 30 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		return;
#line 288 "HtmlDocumentTest.c"
	}
}


static void ___lambda191__gtest_func (void) {
#line 29 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
	__lambda191_ ();
#line 296 "HtmlDocumentTest.c"
}


static gpointer _g_object_ref0 (gpointer self) {
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
	return self ? g_object_ref (self) : NULL;
#line 303 "HtmlDocumentTest.c"
}


static void __lambda192_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlHtmlDocument* doc = NULL;
		GXmlHtmlDocument* _tmp0_ = NULL;
		GXmlNode* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlNode* _tmp3_ = NULL;
		GXmlNode* _tmp4_ = NULL;
		GXmlNode* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		gchar* _tmp7_ = NULL;
		gchar* _tmp8_ = NULL;
		gchar* _tmp9_ = NULL;
		gchar* _tmp10_ = NULL;
		gchar* _tmp11_ = NULL;
		gchar* _tmp12_ = NULL;
		gchar* _tmp13_ = NULL;
		GXmlElementList* np = NULL;
		GXmlNode* _tmp14_ = NULL;
		GXmlNode* _tmp15_ = NULL;
		GXmlNode* _tmp16_ = NULL;
		GXmlElementList* _tmp17_ = NULL;
		GXmlElementList* _tmp18_ = NULL;
		GXmlElementList* _tmp19_ = NULL;
		GXmlElementList* _tmp20_ = NULL;
		gint _tmp21_ = 0;
		gint _tmp22_ = 0;
		GXmlDomHTMLCollection* l = NULL;
		GXmlDomHTMLCollection* _tmp23_ = NULL;
		GXmlDomHTMLCollection* _tmp24_ = NULL;
		GXmlDomHTMLCollection* _tmp25_ = NULL;
		gint _tmp26_ = 0;
		gint _tmp27_ = 0;
		gboolean fdiv = FALSE;
		gboolean fp = FALSE;
		gboolean _tmp28_ = FALSE;
		gboolean _tmp50_ = FALSE;
		gboolean _tmp51_ = FALSE;
#line 48 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp0_ = gxml_html_document_new_from_path (TEST_DIR "/index.html", 0, &_inner_error_);
#line 48 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		doc = _tmp0_;
#line 48 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 352 "HtmlDocumentTest.c"
			goto __catch178_g_error;
		}
#line 49 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		g_test_message ("Checking root element...");
#line 50 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp1_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 50 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp2_ = _tmp1_;
#line 50 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp3_ = _tmp2_;
#line 50 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_vala_assert (_tmp3_ != NULL, "doc.root != null");
#line 50 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_object_unref0 (_tmp3_);
#line 51 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp4_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 51 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp5_ = _tmp4_;
#line 51 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp6_ = _tmp5_;
#line 51 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp7_ = gxml_node_get_name (_tmp6_);
#line 51 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp8_ = _tmp7_;
#line 51 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp9_ = _tmp8_;
#line 51 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp10_ = g_utf8_strdown (_tmp9_, (gssize) -1);
#line 51 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp11_ = _tmp10_;
#line 51 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp12_ = g_utf8_strdown ("html", (gssize) -1);
#line 51 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp13_ = _tmp12_;
#line 51 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp11_, _tmp13_) == 0, "doc.root.name.down () == \"html\".down ()");
#line 51 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_free0 (_tmp13_);
#line 51 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_free0 (_tmp11_);
#line 51 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_free0 (_tmp9_);
#line 51 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_object_unref0 (_tmp6_);
#line 52 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		g_test_message ("Searching for element with property class and value app...");
#line 53 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp14_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 53 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp15_ = _tmp14_;
#line 53 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp16_ = _tmp15_;
#line 53 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp17_ = gxml_node_get_elements_by_property_value (_tmp16_, "class", "app");
#line 53 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp18_ = _tmp17_;
#line 53 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_object_unref0 (_tmp16_);
#line 53 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		np = _tmp18_;
#line 54 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp19_ = np;
#line 54 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_vala_assert (_tmp19_ != NULL, "np != null");
#line 55 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp20_ = np;
#line 55 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp21_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp20_);
#line 55 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp22_ = _tmp21_;
#line 55 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_vala_assert (_tmp22_ == 2, "np.size == 2");
#line 56 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		g_test_message ("Searching for elemento with class 'app'...");
#line 57 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp23_ = gxml_dom_document_get_elements_by_class_name ((GXmlDomDocument*) doc, "app");
#line 57 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		l = _tmp23_;
#line 58 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp24_ = l;
#line 58 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_vala_assert (_tmp24_ != NULL, "l != null");
#line 59 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp25_ = l;
#line 59 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp26_ = gee_collection_get_size ((GeeCollection*) _tmp25_);
#line 59 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp27_ = _tmp26_;
#line 59 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_vala_assert (_tmp27_ == 2, "l.size == 2");
#line 61 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		fp = FALSE;
#line 61 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp28_ = fp;
#line 61 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		fdiv = _tmp28_;
#line 449 "HtmlDocumentTest.c"
		{
			GXmlDomHTMLCollection* _e_list = NULL;
			GXmlDomHTMLCollection* _tmp29_ = NULL;
			GXmlDomHTMLCollection* _tmp30_ = NULL;
			gint _e_size = 0;
			GXmlDomHTMLCollection* _tmp31_ = NULL;
			gint _tmp32_ = 0;
			gint _tmp33_ = 0;
			gint _e_index = 0;
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
			_tmp29_ = l;
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
			_tmp30_ = _g_object_ref0 (_tmp29_);
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
			_e_list = _tmp30_;
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
			_tmp31_ = _e_list;
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
			_tmp32_ = gee_collection_get_size ((GeeCollection*) _tmp31_);
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
			_tmp33_ = _tmp32_;
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
			_e_size = _tmp33_;
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
			_e_index = -1;
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
			while (TRUE) {
#line 477 "HtmlDocumentTest.c"
				gint _tmp34_ = 0;
				gint _tmp35_ = 0;
				gint _tmp36_ = 0;
				GXmlDomElement* e = NULL;
				GXmlDomHTMLCollection* _tmp37_ = NULL;
				gint _tmp38_ = 0;
				gpointer _tmp39_ = NULL;
				GXmlDomElement* _tmp40_ = NULL;
				gchar* _tmp41_ = NULL;
				gchar* _tmp42_ = NULL;
				gchar* _tmp43_ = NULL;
				gboolean _tmp44_ = FALSE;
				GXmlDomElement* _tmp45_ = NULL;
				gchar* _tmp46_ = NULL;
				gchar* _tmp47_ = NULL;
				gchar* _tmp48_ = NULL;
				gboolean _tmp49_ = FALSE;
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				_tmp34_ = _e_index;
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				_e_index = _tmp34_ + 1;
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				_tmp35_ = _e_index;
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				_tmp36_ = _e_size;
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				if (!(_tmp35_ < _tmp36_)) {
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
					break;
#line 507 "HtmlDocumentTest.c"
				}
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				_tmp37_ = _e_list;
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				_tmp38_ = _e_index;
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				_tmp39_ = gee_list_get ((GeeList*) _tmp37_, _tmp38_);
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				e = (GXmlDomElement*) _tmp39_;
#line 63 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				_tmp40_ = e;
#line 63 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				_tmp41_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp40_);
#line 63 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				_tmp42_ = _tmp41_;
#line 63 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				_tmp43_ = _tmp42_;
#line 63 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				_tmp44_ = g_strcmp0 (_tmp43_, "div") == 0;
#line 63 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				_g_free0 (_tmp43_);
#line 63 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				if (_tmp44_) {
#line 63 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
					fdiv = TRUE;
#line 533 "HtmlDocumentTest.c"
				}
#line 64 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				_tmp45_ = e;
#line 64 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				_tmp46_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp45_);
#line 64 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				_tmp47_ = _tmp46_;
#line 64 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				_tmp48_ = _tmp47_;
#line 64 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				_tmp49_ = g_strcmp0 (_tmp48_, "p") == 0;
#line 64 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				_g_free0 (_tmp48_);
#line 64 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				if (_tmp49_) {
#line 64 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
					fp = TRUE;
#line 551 "HtmlDocumentTest.c"
				}
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
				_g_object_unref0 (e);
#line 555 "HtmlDocumentTest.c"
			}
#line 62 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
			_g_object_unref0 (_e_list);
#line 559 "HtmlDocumentTest.c"
		}
#line 66 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp50_ = fdiv;
#line 66 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_vala_assert (_tmp50_, "fdiv");
#line 67 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp51_ = fp;
#line 67 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_vala_assert (_tmp51_, "fp");
#line 47 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_object_unref0 (l);
#line 47 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_object_unref0 (np);
#line 47 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_object_unref0 (doc);
#line 575 "HtmlDocumentTest.c"
	}
	goto __finally178;
	__catch178_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp52_ = NULL;
		gchar* _tmp53_ = NULL;
		gchar* _tmp54_ = NULL;
#line 47 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		e = _inner_error_;
#line 47 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_inner_error_ = NULL;
#line 69 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp52_ = e->message;
#line 69 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp53_ = g_strconcat ("ERROR: ", _tmp52_, NULL);
#line 69 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_tmp54_ = _tmp53_;
#line 69 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		g_test_message ("%s", _tmp54_);
#line 69 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_free0 (_tmp54_);
#line 70 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		g_assert_not_reached ();
#line 47 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		_g_error_free0 (e);
#line 602 "HtmlDocumentTest.c"
	}
	__finally178:
#line 47 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 47 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 47 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 47 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
		return;
#line 613 "HtmlDocumentTest.c"
	}
}


static void ___lambda192__gtest_func (void) {
#line 46 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
	__lambda192_ ();
#line 621 "HtmlDocumentTest.c"
}


void html_document_test_add_tests (void) {
#line 29 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
	g_test_add_func ("/gxml/htmldocument/api/element_id", ___lambda191__gtest_func);
#line 46 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
	g_test_add_func ("/gxml/htmldocument/api/element_class", ___lambda192__gtest_func);
#line 630 "HtmlDocumentTest.c"
}


HtmlDocumentTest* html_document_test_construct (GType object_type) {
	HtmlDocumentTest* self = NULL;
#line 27 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
	self = (HtmlDocumentTest*) gxml_test_construct (object_type);
#line 27 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
	return self;
#line 640 "HtmlDocumentTest.c"
}


HtmlDocumentTest* html_document_test_new (void) {
#line 27 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
	return html_document_test_construct (TYPE_HTML_DOCUMENT_TEST);
#line 647 "HtmlDocumentTest.c"
}


static void html_document_test_class_init (HtmlDocumentTestClass * klass) {
#line 27 "/home/despinosa/Proyectos/gxml/test/HtmlDocumentTest.vala"
	html_document_test_parent_class = g_type_class_peek_parent (klass);
#line 654 "HtmlDocumentTest.c"
}


static void html_document_test_instance_init (HtmlDocumentTest * self) {
}


GType html_document_test_get_type (void) {
	static volatile gsize html_document_test_type_id__volatile = 0;
	if (g_once_init_enter (&html_document_test_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (HtmlDocumentTestClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) html_document_test_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (HtmlDocumentTest), 0, (GInstanceInitFunc) html_document_test_instance_init, NULL };
		GType html_document_test_type_id;
		html_document_test_type_id = g_type_register_static (TYPE_GXML_TEST, "HtmlDocumentTest", &g_define_type_info, 0);
		g_once_init_leave (&html_document_test_type_id__volatile, html_document_test_type_id);
	}
	return html_document_test_type_id__volatile;
}



