#!/usr/bin/gjs

const GXml = imports.gi.GXml;

/* CDATASections are a type of Text node, which are CharacterData
   nodes.  To see more of what you can do with a CDATASection, view
   those examples */

