/* DomAttr.c generated by valac 0.34.6, the Vala compiler
 * generated from DomAttr.vala, do not modify */

/* -*- Mode: vala; indent-tabs-mode: t; c-basic-offset: 2; tab-width: 2 -*- */
/*
 *
 * Copyright (C) 2016  Daniel Espinosa <esodan@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Daniel Espinosa <esodan@gmail.com>
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>


#define GXML_TYPE_DOM_ATTR (gxml_dom_attr_get_type ())
#define GXML_DOM_ATTR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_ATTR, GXmlDomAttr))
#define GXML_IS_DOM_ATTR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_ATTR))
#define GXML_DOM_ATTR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_ATTR, GXmlDomAttrIface))

typedef struct _GXmlDomAttr GXmlDomAttr;
typedef struct _GXmlDomAttrIface GXmlDomAttrIface;

struct _GXmlDomAttrIface {
	GTypeInterface parent_iface;
	gchar* (*get_namespace_uri) (GXmlDomAttr* self);
	gchar* (*get_prefix) (GXmlDomAttr* self);
	gchar* (*get_local_name) (GXmlDomAttr* self);
	gchar* (*get_name) (GXmlDomAttr* self);
	gchar* (*get_value) (GXmlDomAttr* self);
	void (*set_value) (GXmlDomAttr* self, const gchar* value);
	gboolean (*get_specified) (GXmlDomAttr* self);
};



GType gxml_dom_attr_get_type (void) G_GNUC_CONST;
gchar* gxml_dom_attr_get_namespace_uri (GXmlDomAttr* self);
gchar* gxml_dom_attr_get_prefix (GXmlDomAttr* self);
gchar* gxml_dom_attr_get_local_name (GXmlDomAttr* self);
gchar* gxml_dom_attr_get_name (GXmlDomAttr* self);
gchar* gxml_dom_attr_get_value (GXmlDomAttr* self);
void gxml_dom_attr_set_value (GXmlDomAttr* self, const gchar* value);
gboolean gxml_dom_attr_get_specified (GXmlDomAttr* self);


gchar* gxml_dom_attr_get_namespace_uri (GXmlDomAttr* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return GXML_DOM_ATTR_GET_INTERFACE (self)->get_namespace_uri (self);
}


gchar* gxml_dom_attr_get_prefix (GXmlDomAttr* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return GXML_DOM_ATTR_GET_INTERFACE (self)->get_prefix (self);
}


gchar* gxml_dom_attr_get_local_name (GXmlDomAttr* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return GXML_DOM_ATTR_GET_INTERFACE (self)->get_local_name (self);
}


gchar* gxml_dom_attr_get_name (GXmlDomAttr* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return GXML_DOM_ATTR_GET_INTERFACE (self)->get_name (self);
}


gchar* gxml_dom_attr_get_value (GXmlDomAttr* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return GXML_DOM_ATTR_GET_INTERFACE (self)->get_value (self);
}


void gxml_dom_attr_set_value (GXmlDomAttr* self, const gchar* value) {
	g_return_if_fail (self != NULL);
	GXML_DOM_ATTR_GET_INTERFACE (self)->set_value (self, value);
}


gboolean gxml_dom_attr_get_specified (GXmlDomAttr* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return GXML_DOM_ATTR_GET_INTERFACE (self)->get_specified (self);
}


static gboolean gxml_dom_attr_real_get_specified (GXmlDomAttr* base) {
	gboolean result;
	GXmlDomAttr* self;
	self = base;
	result = TRUE;
	return result;
}


static void gxml_dom_attr_base_init (GXmlDomAttrIface * iface) {
	static gboolean initialized = FALSE;
	if (!initialized) {
		initialized = TRUE;
		g_object_interface_install_property (iface, g_param_spec_string ("namespace-uri", "namespace-uri", "namespace-uri", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
		g_object_interface_install_property (iface, g_param_spec_string ("prefix", "prefix", "prefix", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
		g_object_interface_install_property (iface, g_param_spec_string ("local-name", "local-name", "local-name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
		g_object_interface_install_property (iface, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
		g_object_interface_install_property (iface, g_param_spec_string ("value", "value", "value", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
		iface->get_specified = gxml_dom_attr_real_get_specified;
	}
}


GType gxml_dom_attr_get_type (void) {
	static volatile gsize gxml_dom_attr_type_id__volatile = 0;
	if (g_once_init_enter (&gxml_dom_attr_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GXmlDomAttrIface), (GBaseInitFunc) gxml_dom_attr_base_init, (GBaseFinalizeFunc) NULL, (GClassInitFunc) NULL, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
		GType gxml_dom_attr_type_id;
		gxml_dom_attr_type_id = g_type_register_static (G_TYPE_INTERFACE, "GXmlDomAttr", &g_define_type_info, 0);
		g_type_interface_add_prerequisite (gxml_dom_attr_type_id, G_TYPE_OBJECT);
		g_once_init_leave (&gxml_dom_attr_type_id__volatile, gxml_dom_attr_type_id);
	}
	return gxml_dom_attr_type_id__volatile;
}



