/* SerializableMapDualKey.c generated by valac 0.34.6, the Vala compiler
 * generated from SerializableMapDualKey.vala, do not modify */

/* -*- Mode: vala; indent-tabs-mode: nil; c-basic-offset: 2; tab-width: 2 -*- */
/* SerializableGeeTreeModel.vala
 *
 * Copyright (C) 2013  Daniel Espinosa <esodan@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Daniel Espinosa <esodan@gmail.com>
 */

#include <glib.h>
#include <glib-object.h>


#define GXML_TYPE_SERIALIZABLE_MAP_DUAL_KEY (gxml_serializable_map_dual_key_get_type ())
#define GXML_SERIALIZABLE_MAP_DUAL_KEY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_SERIALIZABLE_MAP_DUAL_KEY, GXmlSerializableMapDualKey))
#define GXML_IS_SERIALIZABLE_MAP_DUAL_KEY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_SERIALIZABLE_MAP_DUAL_KEY))
#define GXML_SERIALIZABLE_MAP_DUAL_KEY_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_SERIALIZABLE_MAP_DUAL_KEY, GXmlSerializableMapDualKeyIface))

typedef struct _GXmlSerializableMapDualKey GXmlSerializableMapDualKey;
typedef struct _GXmlSerializableMapDualKeyIface GXmlSerializableMapDualKeyIface;

struct _GXmlSerializableMapDualKeyIface {
	GTypeInterface parent_iface;
	gpointer (*get_map_primary_key) (GXmlSerializableMapDualKey* self);
	gpointer (*get_map_secondary_key) (GXmlSerializableMapDualKey* self);
};



GType gxml_serializable_map_dual_key_get_type (void) G_GNUC_CONST;
gpointer gxml_serializable_map_dual_key_get_map_primary_key (GXmlSerializableMapDualKey* self);
gpointer gxml_serializable_map_dual_key_get_map_secondary_key (GXmlSerializableMapDualKey* self);


/**
   * Implement this function to return the value to be used as primary key on
   * {@link SerializableDualKeyMap} containers.
   */
gpointer gxml_serializable_map_dual_key_get_map_primary_key (GXmlSerializableMapDualKey* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return GXML_SERIALIZABLE_MAP_DUAL_KEY_GET_INTERFACE (self)->get_map_primary_key (self);
}


/**
   * Implement this function to return the value to be used as secondary key on
   * {@link SerializableDualKeyMap} containers.
   */
gpointer gxml_serializable_map_dual_key_get_map_secondary_key (GXmlSerializableMapDualKey* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return GXML_SERIALIZABLE_MAP_DUAL_KEY_GET_INTERFACE (self)->get_map_secondary_key (self);
}


static void gxml_serializable_map_dual_key_base_init (GXmlSerializableMapDualKeyIface * iface) {
	static gboolean initialized = FALSE;
	if (!initialized) {
		initialized = TRUE;
	}
}


/**
 * Serializable Framework. Interface to get two keys to be used to store {@link Serializable} objects.
 * 
 * This interface must be implemented by classes derived from {@link GXml.SerializableDualKeyMap}.
 */
GType gxml_serializable_map_dual_key_get_type (void) {
	static volatile gsize gxml_serializable_map_dual_key_type_id__volatile = 0;
	if (g_once_init_enter (&gxml_serializable_map_dual_key_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GXmlSerializableMapDualKeyIface), (GBaseInitFunc) gxml_serializable_map_dual_key_base_init, (GBaseFinalizeFunc) NULL, (GClassInitFunc) NULL, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
		GType gxml_serializable_map_dual_key_type_id;
		gxml_serializable_map_dual_key_type_id = g_type_register_static (G_TYPE_INTERFACE, "GXmlSerializableMapDualKey", &g_define_type_info, 0);
		g_type_interface_add_prerequisite (gxml_serializable_map_dual_key_type_id, G_TYPE_OBJECT);
		g_once_init_leave (&gxml_serializable_map_dual_key_type_id__volatile, gxml_serializable_map_dual_key_type_id);
	}
	return gxml_serializable_map_dual_key_type_id__volatile;
}



