/* XPathTest.c generated by valac 0.34.6, the Vala compiler
 * generated from XPathTest.vala, do not modify */

/* -*- Mode: vala; indent-tabs-mode: nil; c-basic-offset: 2; tab-width: 2 -*- */
/* Notation.vala
 *
 * Copyright (C) 2016  Daniel Espinosa <esodan@gmail.com>
 * Copyright (C) 2016  Yannick Inizan <inizan.yannick@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Daniel Espinosa <esodan@gmail.com>
 *      Yannick Inizan <inizan.yannick@gmail.com>
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gxml/gxml.h>
#include <gio/gio.h>


#define TYPE_GXML_TEST (gxml_test_get_type ())
#define GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GXML_TEST, GXmlTest))
#define GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GXML_TEST, GXmlTestClass))
#define IS_GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GXML_TEST))
#define IS_GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GXML_TEST))
#define GXML_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GXML_TEST, GXmlTestClass))

typedef struct _GXmlTest GXmlTest;
typedef struct _GXmlTestClass GXmlTestClass;
typedef struct _GXmlTestPrivate GXmlTestPrivate;

#define TYPE_XPATH_TEST (xpath_test_get_type ())
#define XPATH_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_XPATH_TEST, XPathTest))
#define XPATH_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_XPATH_TEST, XPathTestClass))
#define IS_XPATH_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_XPATH_TEST))
#define IS_XPATH_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_XPATH_TEST))
#define XPATH_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_XPATH_TEST, XPathTestClass))

typedef struct _XPathTest XPathTest;
typedef struct _XPathTestClass XPathTestClass;
typedef struct _XPathTestPrivate XPathTestPrivate;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _GXmlTest {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GXmlTestPrivate * priv;
};

struct _GXmlTestClass {
	GTypeClass parent_class;
	void (*finalize) (GXmlTest *self);
};

struct _XPathTest {
	GXmlTest parent_instance;
	XPathTestPrivate * priv;
};

struct _XPathTestClass {
	GXmlTestClass parent_class;
};


static gpointer xpath_test_parent_class = NULL;

gpointer gxml_test_ref (gpointer instance);
void gxml_test_unref (gpointer instance);
GParamSpec* param_spec_gxml_test (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_gxml_test (GValue* value, gpointer v_object);
void value_take_gxml_test (GValue* value, gpointer v_object);
gpointer value_get_gxml_test (const GValue* value);
GType gxml_test_get_type (void) G_GNUC_CONST;
GType xpath_test_get_type (void) G_GNUC_CONST;
enum  {
	XPATH_TEST_DUMMY_PROPERTY
};
#define XPATH_TEST_BOOKS "<bookstore><book category=\"COOKING\"><title lang=\"en\">Everyday Ital" \
"ian</title><author>Giada De Laurentiis</author><year>2005</year><price" \
">30.00</price></book><book category=\"CHILDREN\"><title lang=\"en\">Ha" \
"rry Potter</title><author>J K. Rowling</author><year>2005</year><price" \
">29.99</price></book><book category=\"WEB\"><title lang=\"en\">XQuery " \
"Kick Start</title><author>James McGovern</author><author>Per Bothner</" \
"author><author>Kurt Cagle</author><author>James Linn</author><author>V" \
"aidyanathan Nagarajan</author><year>2003</year><price>49.99</price></b" \
"ook><book category=\"WEB\"><title lang=\"en\">Learning XML</title><aut" \
"hor>Erik T. Ray</author><year>2003</year><price>39.95</price></book></" \
"bookstore>"
void xpath_test_add_tests (void);
static void __lambda207_ (void);
static void ___lambda207__gtest_func (void);
XPathTest* xpath_test_new (void);
XPathTest* xpath_test_construct (GType object_type);
GXmlTest* gxml_test_new (void);
GXmlTest* gxml_test_construct (GType object_type);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);


static void __lambda207_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlDocument* document = NULL;
		GFile* rf = NULL;
		GFile* _tmp0_ = NULL;
		GFile* _tmp1_ = NULL;
		gboolean _tmp2_ = FALSE;
		GXmlDocument* _tmp9_ = NULL;
		GXmlXPathObject* object = NULL;
		GXmlDocument* _tmp10_ = NULL;
		GXmlXPathObject* _tmp11_ = NULL;
		GXmlXPathObject* _tmp12_ = NULL;
		GXmlXPathObjectType _tmp13_ = 0;
		GXmlXPathObjectType _tmp14_ = 0;
		GXmlDomElement** array = NULL;
		GXmlXPathObject* _tmp15_ = NULL;
		GXmlDomHTMLCollection* _tmp16_ = NULL;
		GXmlDomHTMLCollection* _tmp17_ = NULL;
		gint _tmp18_ = 0;
		GXmlDomElement** _tmp19_ = NULL;
		gint array_length1 = 0;
		gint _array_size_ = 0;
		GXmlDomElement** _tmp20_ = NULL;
		gint _tmp20__length1 = 0;
		GXmlDomElement** _tmp21_ = NULL;
		gint _tmp21__length1 = 0;
		GXmlDomElement* _tmp22_ = NULL;
		gchar* _tmp23_ = NULL;
		gchar* _tmp24_ = NULL;
		gchar* _tmp25_ = NULL;
		GXmlDomElement** _tmp26_ = NULL;
		gint _tmp26__length1 = 0;
		GXmlDomElement* _tmp27_ = NULL;
		gchar* _tmp28_ = NULL;
		gchar* _tmp29_ = NULL;
		gchar* _tmp30_ = NULL;
		GXmlDomElement** _tmp31_ = NULL;
		gint _tmp31__length1 = 0;
		GXmlDomElement* _tmp32_ = NULL;
		gchar* _tmp33_ = NULL;
		gchar* _tmp34_ = NULL;
		gchar* _tmp35_ = NULL;
		GXmlDomElement** _tmp36_ = NULL;
		gint _tmp36__length1 = 0;
		GXmlDomElement* _tmp37_ = NULL;
		gchar* _tmp38_ = NULL;
		gchar* _tmp39_ = NULL;
		gchar* _tmp40_ = NULL;
		GXmlXPathObject* _tmp41_ = NULL;
		GXmlDocument* _tmp42_ = NULL;
		GXmlXPathObject* _tmp43_ = NULL;
		GXmlXPathObject* _tmp44_ = NULL;
		GXmlXPathObject* _tmp45_ = NULL;
		GXmlXPathObjectType _tmp46_ = 0;
		GXmlXPathObjectType _tmp47_ = 0;
		GXmlXPathObject* _tmp48_ = NULL;
		GXmlDomHTMLCollection* _tmp49_ = NULL;
		GXmlDomHTMLCollection* _tmp50_ = NULL;
		gint _tmp51_ = 0;
		GXmlDomElement** _tmp52_ = NULL;
		GXmlDomElement** _tmp53_ = NULL;
		gint _tmp53__length1 = 0;
		GXmlDomElement** _tmp54_ = NULL;
		gint _tmp54__length1 = 0;
		GXmlDomElement* _tmp55_ = NULL;
		gchar* _tmp56_ = NULL;
		gchar* _tmp57_ = NULL;
		gchar* _tmp58_ = NULL;
		GXmlXPathObject* _tmp59_ = NULL;
		GXmlDocument* _tmp60_ = NULL;
		GXmlXPathObject* _tmp61_ = NULL;
		GXmlXPathObject* _tmp62_ = NULL;
		GXmlXPathObject* _tmp63_ = NULL;
		GXmlXPathObjectType _tmp64_ = 0;
		GXmlXPathObjectType _tmp65_ = 0;
		GXmlXPathObject* _tmp66_ = NULL;
		GXmlDomHTMLCollection* _tmp67_ = NULL;
		GXmlDomHTMLCollection* _tmp68_ = NULL;
		gint _tmp69_ = 0;
		GXmlDomElement** _tmp70_ = NULL;
		GXmlDomElement** _tmp71_ = NULL;
		gint _tmp71__length1 = 0;
		GXmlDomElement** _tmp72_ = NULL;
		gint _tmp72__length1 = 0;
		GXmlDomElement* _tmp73_ = NULL;
		gchar* _tmp74_ = NULL;
		gchar* _tmp75_ = NULL;
		gchar* _tmp76_ = NULL;
		GXmlDomElement** _tmp77_ = NULL;
		gint _tmp77__length1 = 0;
		GXmlDomElement* _tmp78_ = NULL;
		gchar* _tmp79_ = NULL;
		gchar* _tmp80_ = NULL;
		gchar* _tmp81_ = NULL;
		GXmlDomElement** _tmp82_ = NULL;
		gint _tmp82__length1 = 0;
		GXmlDomElement* _tmp83_ = NULL;
		gchar* _tmp84_ = NULL;
		gchar* _tmp85_ = NULL;
		gchar* _tmp86_ = NULL;
		GXmlDomElement** _tmp87_ = NULL;
		gint _tmp87__length1 = 0;
		GXmlDomElement* _tmp88_ = NULL;
		gchar* _tmp89_ = NULL;
		gchar* _tmp90_ = NULL;
		gchar* _tmp91_ = NULL;
		GXmlXPathObject* _tmp92_ = NULL;
		GXmlDocument* _tmp93_ = NULL;
		GXmlXPathObject* _tmp94_ = NULL;
		GXmlXPathObject* _tmp95_ = NULL;
		GXmlXPathObject* _tmp96_ = NULL;
		GXmlXPathObjectType _tmp97_ = 0;
		GXmlXPathObjectType _tmp98_ = 0;
		GXmlXPathObject* _tmp99_ = NULL;
		GXmlDomHTMLCollection* _tmp100_ = NULL;
		GXmlDomHTMLCollection* _tmp101_ = NULL;
		gint _tmp102_ = 0;
		GXmlDomElement** _tmp103_ = NULL;
		GXmlDomElement** _tmp104_ = NULL;
		gint _tmp104__length1 = 0;
		GXmlDomElement** _tmp105_ = NULL;
		gint _tmp105__length1 = 0;
		GXmlDomElement* _tmp106_ = NULL;
		gchar* _tmp107_ = NULL;
		gchar* _tmp108_ = NULL;
		gchar* _tmp109_ = NULL;
		GXmlDomElement** _tmp110_ = NULL;
		gint _tmp110__length1 = 0;
		GXmlDomElement* _tmp111_ = NULL;
		gchar* _tmp112_ = NULL;
		gchar* _tmp113_ = NULL;
		gchar* _tmp114_ = NULL;
#line 33 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		document = NULL;
#line 34 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp0_ = g_file_new_for_uri ("http://www.w3schools.com/xsl/books.xml");
#line 34 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		rf = _tmp0_;
#line 35 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp1_ = rf;
#line 35 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp2_ = g_file_query_exists (_tmp1_, NULL);
#line 35 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		if (_tmp2_) {
#line 257 "XPathTest.c"
			GXmlGDocument* _tmp3_ = NULL;
			GXmlGDocument* _tmp4_ = NULL;
			GXmlGDocument* _tmp5_ = NULL;
#line 36 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_tmp4_ = gxml_gdocument_new_from_uri ("http://www.w3schools.com/xsl/books.xml", 0, &_inner_error_);
#line 36 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_tmp3_ = _tmp4_;
#line 36 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 36 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
				_g_object_unref0 (rf);
#line 36 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
				_g_object_unref0 (document);
#line 271 "XPathTest.c"
				goto __catch190_g_error;
			}
#line 36 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_tmp5_ = _tmp3_;
#line 36 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_tmp3_ = NULL;
#line 36 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_g_object_unref0 (document);
#line 36 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			document = (GXmlDocument*) _tmp5_;
#line 35 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_g_object_unref0 (_tmp3_);
#line 284 "XPathTest.c"
		} else {
			GXmlGDocument* _tmp6_ = NULL;
			GXmlGDocument* _tmp7_ = NULL;
			GXmlGDocument* _tmp8_ = NULL;
#line 38 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_tmp7_ = gxml_gdocument_new_from_string (XPATH_TEST_BOOKS, 0, &_inner_error_);
#line 38 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_tmp6_ = _tmp7_;
#line 38 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 38 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
				_g_object_unref0 (rf);
#line 38 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
				_g_object_unref0 (document);
#line 299 "XPathTest.c"
				goto __catch190_g_error;
			}
#line 38 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_tmp8_ = _tmp6_;
#line 38 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_tmp6_ = NULL;
#line 38 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_g_object_unref0 (document);
#line 38 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			document = (GXmlDocument*) _tmp8_;
#line 35 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_g_object_unref0 (_tmp6_);
#line 312 "XPathTest.c"
		}
#line 39 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp9_ = document;
#line 39 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_vala_assert (_tmp9_ != NULL, "document != null");
#line 40 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp10_ = document;
#line 40 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp11_ = gxml_xpath_context_evaluate (G_TYPE_CHECK_INSTANCE_TYPE (_tmp10_, GXML_TYPE_XPATH_CONTEXT) ? ((GXmlXPathContext*) _tmp10_) : NULL, "/bookstore/book/title", NULL, &_inner_error_);
#line 40 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		object = _tmp11_;
#line 40 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 40 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_g_object_unref0 (rf);
#line 40 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_g_object_unref0 (document);
#line 330 "XPathTest.c"
			goto __catch190_g_error;
		}
#line 41 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp12_ = object;
#line 41 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp13_ = gxml_xpath_object_get_object_type (_tmp12_);
#line 41 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp14_ = _tmp13_;
#line 41 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_vala_assert (_tmp14_ == GXML_XPATH_OBJECT_TYPE_NODESET, "object.object_type == XPathObjectType.NODESET");
#line 42 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp15_ = object;
#line 42 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp16_ = gxml_xpath_object_get_nodeset (_tmp15_);
#line 42 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp17_ = _tmp16_;
#line 42 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp19_ = gxml_dom_html_collection_to_array (_tmp17_, &_tmp18_);
#line 42 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		array = _tmp19_;
#line 42 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		array_length1 = _tmp18_;
#line 42 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_array_size_ = array_length1;
#line 43 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp20_ = array;
#line 43 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp20__length1 = array_length1;
#line 43 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_vala_assert (_tmp20__length1 == 4, "array.length == 4");
#line 44 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp21_ = array;
#line 44 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp21__length1 = array_length1;
#line 44 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp22_ = _tmp21_[0];
#line 44 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp23_ = gxml_dom_node_get_node_value ((GXmlDomNode*) _tmp22_);
#line 44 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp24_ = _tmp23_;
#line 44 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp25_ = _tmp24_;
#line 44 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_vala_assert (g_strcmp0 (_tmp25_, "Everyday Italian") == 0, "array[0].node_value == \"Everyday Italian\"");
#line 44 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_free0 (_tmp25_);
#line 45 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp26_ = array;
#line 45 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp26__length1 = array_length1;
#line 45 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp27_ = _tmp26_[1];
#line 45 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp28_ = gxml_dom_node_get_node_value ((GXmlDomNode*) _tmp27_);
#line 45 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp29_ = _tmp28_;
#line 45 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp30_ = _tmp29_;
#line 45 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_vala_assert (g_strcmp0 (_tmp30_, "Harry Potter") == 0, "array[1].node_value == \"Harry Potter\"");
#line 45 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_free0 (_tmp30_);
#line 46 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp31_ = array;
#line 46 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp31__length1 = array_length1;
#line 46 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp32_ = _tmp31_[2];
#line 46 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp33_ = gxml_dom_node_get_node_value ((GXmlDomNode*) _tmp32_);
#line 46 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp34_ = _tmp33_;
#line 46 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp35_ = _tmp34_;
#line 46 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_vala_assert (g_strcmp0 (_tmp35_, "XQuery Kick Start") == 0, "array[2].node_value == \"XQuery Kick Start\"");
#line 46 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_free0 (_tmp35_);
#line 47 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp36_ = array;
#line 47 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp36__length1 = array_length1;
#line 47 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp37_ = _tmp36_[3];
#line 47 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp38_ = gxml_dom_node_get_node_value ((GXmlDomNode*) _tmp37_);
#line 47 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp39_ = _tmp38_;
#line 47 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp40_ = _tmp39_;
#line 47 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_vala_assert (g_strcmp0 (_tmp40_, "Learning XML") == 0, "array[3].node_value == \"Learning XML\"");
#line 47 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_free0 (_tmp40_);
#line 48 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp42_ = document;
#line 48 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp43_ = gxml_xpath_context_evaluate (G_TYPE_CHECK_INSTANCE_TYPE (_tmp42_, GXML_TYPE_XPATH_CONTEXT) ? ((GXmlXPathContext*) _tmp42_) : NULL, "/bookstore/book[1]/title", NULL, &_inner_error_);
#line 48 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp41_ = _tmp43_;
#line 48 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 48 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			array = (_vala_array_free (array, array_length1, (GDestroyNotify) g_object_unref), NULL);
#line 48 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_g_object_unref0 (object);
#line 48 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_g_object_unref0 (rf);
#line 48 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_g_object_unref0 (document);
#line 441 "XPathTest.c"
			goto __catch190_g_error;
		}
#line 48 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp44_ = _tmp41_;
#line 48 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp41_ = NULL;
#line 48 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_object_unref0 (object);
#line 48 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		object = _tmp44_;
#line 49 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp45_ = object;
#line 49 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp46_ = gxml_xpath_object_get_object_type (_tmp45_);
#line 49 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp47_ = _tmp46_;
#line 49 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_vala_assert (_tmp47_ == GXML_XPATH_OBJECT_TYPE_NODESET, "object.object_type == XPathObjectType.NODESET");
#line 50 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp48_ = object;
#line 50 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp49_ = gxml_xpath_object_get_nodeset (_tmp48_);
#line 50 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp50_ = _tmp49_;
#line 50 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp52_ = gxml_dom_html_collection_to_array (_tmp50_, &_tmp51_);
#line 50 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		array = (_vala_array_free (array, array_length1, (GDestroyNotify) g_object_unref), NULL);
#line 50 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		array = _tmp52_;
#line 50 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		array_length1 = _tmp51_;
#line 50 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_array_size_ = array_length1;
#line 51 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp53_ = array;
#line 51 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp53__length1 = array_length1;
#line 51 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_vala_assert (_tmp53__length1 == 1, "array.length == 1");
#line 52 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp54_ = array;
#line 52 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp54__length1 = array_length1;
#line 52 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp55_ = _tmp54_[0];
#line 52 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp56_ = gxml_dom_node_get_node_value ((GXmlDomNode*) _tmp55_);
#line 52 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp57_ = _tmp56_;
#line 52 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp58_ = _tmp57_;
#line 52 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_vala_assert (g_strcmp0 (_tmp58_, "Everyday Italian") == 0, "array[0].node_value == \"Everyday Italian\"");
#line 52 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_free0 (_tmp58_);
#line 53 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp60_ = document;
#line 53 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp61_ = gxml_xpath_context_evaluate (G_TYPE_CHECK_INSTANCE_TYPE (_tmp60_, GXML_TYPE_XPATH_CONTEXT) ? ((GXmlXPathContext*) _tmp60_) : NULL, "/bookstore/book/price[text()]", NULL, &_inner_error_);
#line 53 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp59_ = _tmp61_;
#line 53 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 53 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_g_object_unref0 (_tmp41_);
#line 53 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			array = (_vala_array_free (array, array_length1, (GDestroyNotify) g_object_unref), NULL);
#line 53 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_g_object_unref0 (object);
#line 53 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_g_object_unref0 (rf);
#line 53 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_g_object_unref0 (document);
#line 516 "XPathTest.c"
			goto __catch190_g_error;
		}
#line 53 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp62_ = _tmp59_;
#line 53 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp59_ = NULL;
#line 53 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_object_unref0 (object);
#line 53 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		object = _tmp62_;
#line 54 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp63_ = object;
#line 54 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp64_ = gxml_xpath_object_get_object_type (_tmp63_);
#line 54 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp65_ = _tmp64_;
#line 54 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_vala_assert (_tmp65_ == GXML_XPATH_OBJECT_TYPE_NODESET, "object.object_type == XPathObjectType.NODESET");
#line 55 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp66_ = object;
#line 55 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp67_ = gxml_xpath_object_get_nodeset (_tmp66_);
#line 55 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp68_ = _tmp67_;
#line 55 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp70_ = gxml_dom_html_collection_to_array (_tmp68_, &_tmp69_);
#line 55 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		array = (_vala_array_free (array, array_length1, (GDestroyNotify) g_object_unref), NULL);
#line 55 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		array = _tmp70_;
#line 55 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		array_length1 = _tmp69_;
#line 55 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_array_size_ = array_length1;
#line 56 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp71_ = array;
#line 56 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp71__length1 = array_length1;
#line 56 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_vala_assert (_tmp71__length1 == 4, "array.length == 4");
#line 57 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp72_ = array;
#line 57 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp72__length1 = array_length1;
#line 57 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp73_ = _tmp72_[0];
#line 57 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp74_ = gxml_dom_node_get_node_value ((GXmlDomNode*) _tmp73_);
#line 57 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp75_ = _tmp74_;
#line 57 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp76_ = _tmp75_;
#line 57 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_vala_assert (g_strcmp0 (_tmp76_, "30.00") == 0, "array[0].node_value == \"30.00\"");
#line 57 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_free0 (_tmp76_);
#line 58 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp77_ = array;
#line 58 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp77__length1 = array_length1;
#line 58 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp78_ = _tmp77_[1];
#line 58 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp79_ = gxml_dom_node_get_node_value ((GXmlDomNode*) _tmp78_);
#line 58 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp80_ = _tmp79_;
#line 58 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp81_ = _tmp80_;
#line 58 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_vala_assert (g_strcmp0 (_tmp81_, "29.99") == 0, "array[1].node_value == \"29.99\"");
#line 58 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_free0 (_tmp81_);
#line 59 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp82_ = array;
#line 59 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp82__length1 = array_length1;
#line 59 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp83_ = _tmp82_[2];
#line 59 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp84_ = gxml_dom_node_get_node_value ((GXmlDomNode*) _tmp83_);
#line 59 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp85_ = _tmp84_;
#line 59 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp86_ = _tmp85_;
#line 59 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_vala_assert (g_strcmp0 (_tmp86_, "49.99") == 0, "array[2].node_value == \"49.99\"");
#line 59 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_free0 (_tmp86_);
#line 60 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp87_ = array;
#line 60 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp87__length1 = array_length1;
#line 60 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp88_ = _tmp87_[3];
#line 60 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp89_ = gxml_dom_node_get_node_value ((GXmlDomNode*) _tmp88_);
#line 60 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp90_ = _tmp89_;
#line 60 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp91_ = _tmp90_;
#line 60 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_vala_assert (g_strcmp0 (_tmp91_, "39.95") == 0, "array[3].node_value == \"39.95\"");
#line 60 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_free0 (_tmp91_);
#line 61 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp93_ = document;
#line 61 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp94_ = gxml_xpath_context_evaluate (G_TYPE_CHECK_INSTANCE_TYPE (_tmp93_, GXML_TYPE_XPATH_CONTEXT) ? ((GXmlXPathContext*) _tmp93_) : NULL, "/bookstore/book[price>35]/price", NULL, &_inner_error_);
#line 61 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp92_ = _tmp94_;
#line 61 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 61 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_g_object_unref0 (_tmp59_);
#line 61 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_g_object_unref0 (_tmp41_);
#line 61 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			array = (_vala_array_free (array, array_length1, (GDestroyNotify) g_object_unref), NULL);
#line 61 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_g_object_unref0 (object);
#line 61 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_g_object_unref0 (rf);
#line 61 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
			_g_object_unref0 (document);
#line 641 "XPathTest.c"
			goto __catch190_g_error;
		}
#line 61 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp95_ = _tmp92_;
#line 61 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp92_ = NULL;
#line 61 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_object_unref0 (object);
#line 61 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		object = _tmp95_;
#line 62 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp96_ = object;
#line 62 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp97_ = gxml_xpath_object_get_object_type (_tmp96_);
#line 62 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp98_ = _tmp97_;
#line 62 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_vala_assert (_tmp98_ == GXML_XPATH_OBJECT_TYPE_NODESET, "object.object_type == XPathObjectType.NODESET");
#line 63 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp99_ = object;
#line 63 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp100_ = gxml_xpath_object_get_nodeset (_tmp99_);
#line 63 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp101_ = _tmp100_;
#line 63 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp103_ = gxml_dom_html_collection_to_array (_tmp101_, &_tmp102_);
#line 63 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		array = (_vala_array_free (array, array_length1, (GDestroyNotify) g_object_unref), NULL);
#line 63 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		array = _tmp103_;
#line 63 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		array_length1 = _tmp102_;
#line 63 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_array_size_ = array_length1;
#line 64 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp104_ = array;
#line 64 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp104__length1 = array_length1;
#line 64 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_vala_assert (_tmp104__length1 == 2, "array.length == 2");
#line 65 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp105_ = array;
#line 65 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp105__length1 = array_length1;
#line 65 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp106_ = _tmp105_[0];
#line 65 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp107_ = gxml_dom_node_get_node_value ((GXmlDomNode*) _tmp106_);
#line 65 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp108_ = _tmp107_;
#line 65 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp109_ = _tmp108_;
#line 65 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_vala_assert (g_strcmp0 (_tmp109_, "49.99") == 0, "array[0].node_value == \"49.99\"");
#line 65 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_free0 (_tmp109_);
#line 66 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp110_ = array;
#line 66 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp110__length1 = array_length1;
#line 66 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp111_ = _tmp110_[1];
#line 66 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp112_ = gxml_dom_node_get_node_value ((GXmlDomNode*) _tmp111_);
#line 66 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp113_ = _tmp112_;
#line 66 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp114_ = _tmp113_;
#line 66 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_vala_assert (g_strcmp0 (_tmp114_, "39.95") == 0, "array[1].node_value == \"39.95\"");
#line 66 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_free0 (_tmp114_);
#line 32 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_object_unref0 (_tmp92_);
#line 32 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_object_unref0 (_tmp59_);
#line 32 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_object_unref0 (_tmp41_);
#line 32 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		array = (_vala_array_free (array, array_length1, (GDestroyNotify) g_object_unref), NULL);
#line 32 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_object_unref0 (object);
#line 32 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_object_unref0 (rf);
#line 32 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_object_unref0 (document);
#line 728 "XPathTest.c"
	}
	goto __finally190;
	__catch190_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp115_ = NULL;
		gchar* _tmp116_ = NULL;
		gchar* _tmp117_ = NULL;
#line 32 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		e = _inner_error_;
#line 32 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_inner_error_ = NULL;
#line 68 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp115_ = e->message;
#line 68 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp116_ = g_strconcat ("ERROR: ", _tmp115_, NULL);
#line 68 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_tmp117_ = _tmp116_;
#line 68 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		g_message ("XPathTest.vala:68: %s", _tmp117_);
#line 68 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_free0 (_tmp117_);
#line 69 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		g_assert_not_reached ();
#line 32 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		_g_error_free0 (e);
#line 755 "XPathTest.c"
	}
	__finally190:
#line 32 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 32 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 32 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		g_clear_error (&_inner_error_);
#line 32 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
		return;
#line 766 "XPathTest.c"
	}
}


static void ___lambda207__gtest_func (void) {
#line 31 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
	__lambda207_ ();
#line 774 "XPathTest.c"
}


void xpath_test_add_tests (void) {
#line 31 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
	g_test_add_func ("/gxml/gelement/xpath", ___lambda207__gtest_func);
#line 781 "XPathTest.c"
}


XPathTest* xpath_test_construct (GType object_type) {
	XPathTest* self = NULL;
#line 27 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
	self = (XPathTest*) gxml_test_construct (object_type);
#line 27 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
	return self;
#line 791 "XPathTest.c"
}


XPathTest* xpath_test_new (void) {
#line 27 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
	return xpath_test_construct (TYPE_XPATH_TEST);
#line 798 "XPathTest.c"
}


static void xpath_test_class_init (XPathTestClass * klass) {
#line 27 "/home/despinosa/Proyectos/gxml/test/XPathTest.vala"
	xpath_test_parent_class = g_type_class_peek_parent (klass);
#line 805 "XPathTest.c"
}


static void xpath_test_instance_init (XPathTest * self) {
}


GType xpath_test_get_type (void) {
	static volatile gsize xpath_test_type_id__volatile = 0;
	if (g_once_init_enter (&xpath_test_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (XPathTestClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) xpath_test_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (XPathTest), 0, (GInstanceInitFunc) xpath_test_instance_init, NULL };
		GType xpath_test_type_id;
		xpath_test_type_id = g_type_register_static (TYPE_GXML_TEST, "XPathTest", &g_define_type_info, 0);
		g_once_init_leave (&xpath_test_type_id__volatile, xpath_test_type_id);
	}
	return xpath_test_type_id__volatile;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}



