/* -*- Mode: vala; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
using GXml;

class NodeListTest : GXmlTest {
	public static void add_tests () {
		Test.add_func ("/gxml/nodelist/length", () => {
			});
		// TODO: add tests
	}
}
