/* TCDATA.c generated by valac 0.34.6, the Vala compiler
 * generated from TCDATA.vala, do not modify */

/* TCDATA.vala
 *
 * Copyright (C) 2015  Daniel Espinosa <esodan@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Daniel Espinosa <esodan@gmail.com>
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gee.h>
#include <gio/gio.h>
#include <libxml/xmlwriter.h>


#define GXML_TYPE_NODE (gxml_node_get_type ())
#define GXML_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_NODE, GXmlNode))
#define GXML_IS_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_NODE))
#define GXML_NODE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_NODE, GXmlNodeIface))

typedef struct _GXmlNode GXmlNode;
typedef struct _GXmlNodeIface GXmlNodeIface;

#define GXML_TYPE_ELEMENT_LIST (gxml_element_list_get_type ())
#define GXML_ELEMENT_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_ELEMENT_LIST, GXmlElementList))
#define GXML_ELEMENT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_ELEMENT_LIST, GXmlElementListClass))
#define GXML_IS_ELEMENT_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_ELEMENT_LIST))
#define GXML_IS_ELEMENT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_ELEMENT_LIST))
#define GXML_ELEMENT_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_ELEMENT_LIST, GXmlElementListClass))

typedef struct _GXmlElementList GXmlElementList;
typedef struct _GXmlElementListClass GXmlElementListClass;

#define GXML_TYPE_NAMESPACE (gxml_namespace_get_type ())
#define GXML_NAMESPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_NAMESPACE, GXmlNamespace))
#define GXML_IS_NAMESPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_NAMESPACE))
#define GXML_NAMESPACE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_NAMESPACE, GXmlNamespaceIface))

typedef struct _GXmlNamespace GXmlNamespace;
typedef struct _GXmlNamespaceIface GXmlNamespaceIface;

#define GXML_TYPE_NODE_TYPE (gxml_node_type_get_type ())

#define GXML_TYPE_DOCUMENT (gxml_document_get_type ())
#define GXML_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOCUMENT, GXmlDocument))
#define GXML_IS_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOCUMENT))
#define GXML_DOCUMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOCUMENT, GXmlDocumentIface))

typedef struct _GXmlDocument GXmlDocument;
typedef struct _GXmlDocumentIface GXmlDocumentIface;

#define GXML_TYPE_TNODE (gxml_tnode_get_type ())
#define GXML_TNODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_TNODE, GXmlTNode))
#define GXML_TNODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_TNODE, GXmlTNodeClass))
#define GXML_IS_TNODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_TNODE))
#define GXML_IS_TNODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_TNODE))
#define GXML_TNODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_TNODE, GXmlTNodeClass))

typedef struct _GXmlTNode GXmlTNode;
typedef struct _GXmlTNodeClass GXmlTNodeClass;
typedef struct _GXmlTNodePrivate GXmlTNodePrivate;

#define GXML_TYPE_CDATA (gxml_cdata_get_type ())
#define GXML_CDATA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_CDATA, GXmlCDATA))
#define GXML_IS_CDATA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_CDATA))
#define GXML_CDATA_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_CDATA, GXmlCDATAIface))

typedef struct _GXmlCDATA GXmlCDATA;
typedef struct _GXmlCDATAIface GXmlCDATAIface;

#define GXML_TYPE_TCDATA (gxml_tcdata_get_type ())
#define GXML_TCDATA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_TCDATA, GXmlTCDATA))
#define GXML_TCDATA_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_TCDATA, GXmlTCDATAClass))
#define GXML_IS_TCDATA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_TCDATA))
#define GXML_IS_TCDATA_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_TCDATA))
#define GXML_TCDATA_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_TCDATA, GXmlTCDATAClass))

typedef struct _GXmlTCDATA GXmlTCDATA;
typedef struct _GXmlTCDATAClass GXmlTCDATAClass;
typedef struct _GXmlTCDATAPrivate GXmlTCDATAPrivate;
#define _g_free0(var) (var = (g_free (var), NULL))

#define GXML_TYPE_TDOCUMENT (gxml_tdocument_get_type ())
#define GXML_TDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_TDOCUMENT, GXmlTDocument))
#define GXML_TDOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_TDOCUMENT, GXmlTDocumentClass))
#define GXML_IS_TDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_TDOCUMENT))
#define GXML_IS_TDOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_TDOCUMENT))
#define GXML_TDOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_TDOCUMENT, GXmlTDocumentClass))

typedef struct _GXmlTDocument GXmlTDocument;
typedef struct _GXmlTDocumentClass GXmlTDocumentClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _GXmlNamespaceIface {
	GTypeInterface parent_iface;
	gchar* (*get_uri) (GXmlNamespace* self);
	gchar* (*get_prefix) (GXmlNamespace* self);
};

typedef enum  {
	GXML_NODE_TYPE_INVALID = 0,
	GXML_NODE_TYPE_ELEMENT = 1,
	GXML_NODE_TYPE_ATTRIBUTE,
	GXML_NODE_TYPE_TEXT,
	GXML_NODE_TYPE_CDATA_SECTION,
	GXML_NODE_TYPE_ENTITY_REFERENCE,
	GXML_NODE_TYPE_ENTITY,
	GXML_NODE_TYPE_PROCESSING_INSTRUCTION,
	GXML_NODE_TYPE_COMMENT,
	GXML_NODE_TYPE_DOCUMENT,
	GXML_NODE_TYPE_DOCUMENT_TYPE,
	GXML_NODE_TYPE_DOCUMENT_FRAGMENT,
	GXML_NODE_TYPE_NOTATION
} GXmlNodeType;

struct _GXmlDocumentIface {
	GTypeInterface parent_iface;
	GXmlNode* (*create_element) (GXmlDocument* self, const gchar* name, GError** error);
	GXmlNode* (*create_text) (GXmlDocument* self, const gchar* text);
	GXmlNode* (*create_comment) (GXmlDocument* self, const gchar* text);
	GXmlNode* (*create_cdata) (GXmlDocument* self, const gchar* text);
	GXmlNode* (*create_pi) (GXmlDocument* self, const gchar* target, const gchar* data);
	gboolean (*save) (GXmlDocument* self, GCancellable* cancellable, GError** error);
	gboolean (*save_as) (GXmlDocument* self, GFile* f, GCancellable* cancellable, GError** error);
	gboolean (*get_indent) (GXmlDocument* self);
	void (*set_indent) (GXmlDocument* self, gboolean value);
	gboolean (*get_ns_top) (GXmlDocument* self);
	void (*set_ns_top) (GXmlDocument* self, gboolean value);
	gboolean (*get_prefix_default_ns) (GXmlDocument* self);
	void (*set_prefix_default_ns) (GXmlDocument* self, gboolean value);
	gboolean (*get_backup) (GXmlDocument* self);
	void (*set_backup) (GXmlDocument* self, gboolean value);
	GXmlNode* (*get_root) (GXmlDocument* self);
	GFile* (*get_file) (GXmlDocument* self);
	void (*set_file) (GXmlDocument* self, GFile* value);
};

struct _GXmlNodeIface {
	GTypeInterface parent_iface;
	GXmlNode* (*get) (GXmlNode* self, const gchar* key);
	GXmlElementList* (*get_elements_by_property_value) (GXmlNode* self, const gchar* property, const gchar* value);
	GXmlElementList* (*get_elements_by_name) (GXmlNode* self, const gchar* name);
	GXmlElementList* (*get_elements_by_name_ns) (GXmlNode* self, const gchar* name, const gchar* ns);
	gchar* (*to_string) (GXmlNode* self);
	gboolean (*set_namespace) (GXmlNode* self, const gchar* uri, const gchar* prefix);
	gchar* (*ns_prefix) (GXmlNode* self);
	gchar* (*ns_uri) (GXmlNode* self);
	GeeList* (*get_namespaces) (GXmlNode* self);
	GeeBidirList* (*get_children_nodes) (GXmlNode* self);
	GeeMap* (*get_attrs) (GXmlNode* self);
	gchar* (*get_name) (GXmlNode* self);
	gchar* (*get_value) (GXmlNode* self);
	void (*set_value) (GXmlNode* self, const gchar* value);
	GXmlNodeType (*get_type_node) (GXmlNode* self);
	GXmlDocument* (*get_document) (GXmlNode* self);
	GXmlNode* (*get_parent) (GXmlNode* self);
};

struct _GXmlTNode {
	GObject parent_instance;
	GXmlTNodePrivate * priv;
	gchar* _name;
	gchar* _value;
	GXmlDocument* _doc;
	GXmlNode* _parent;
	GXmlNodeType _node_type;
	xmlTextWriter* tw;
};

struct _GXmlTNodeClass {
	GObjectClass parent_class;
	gboolean (*set_namespace) (GXmlTNode* self, const gchar* uri, const gchar* prefix);
	gchar* (*to_string) (GXmlTNode* self);
	void (*set_parent) (GXmlTNode* self, GXmlNode* node);
	GeeMap* (*get_attrs) (GXmlTNode* self);
	GeeBidirList* (*get_children_nodes) (GXmlTNode* self);
	GXmlDocument* (*get_document) (GXmlTNode* self);
	gchar* (*get_name) (GXmlTNode* self);
	GeeList* (*get_namespaces) (GXmlTNode* self);
	GXmlNodeType (*get_type_node) (GXmlTNode* self);
	gchar* (*get_value) (GXmlTNode* self);
	void (*set_value) (GXmlTNode* self, const gchar* value);
	GXmlNode* (*get_parent) (GXmlTNode* self);
};

struct _GXmlCDATAIface {
	GTypeInterface parent_iface;
	gchar* (*get_str) (GXmlCDATA* self);
};

struct _GXmlTCDATA {
	GXmlTNode parent_instance;
	GXmlTCDATAPrivate * priv;
};

struct _GXmlTCDATAClass {
	GXmlTNodeClass parent_class;
};

struct _GXmlTCDATAPrivate {
	gchar* _str;
};


static gpointer gxml_tcdata_parent_class = NULL;
static GXmlCDATAIface* gxml_tcdata_gxml_cdata_parent_iface = NULL;

GType gxml_element_list_get_type (void) G_GNUC_CONST;
GType gxml_namespace_get_type (void) G_GNUC_CONST;
GType gxml_node_type_get_type (void) G_GNUC_CONST;
GType gxml_document_get_type (void) G_GNUC_CONST;
GType gxml_node_get_type (void) G_GNUC_CONST;
GType gxml_tnode_get_type (void) G_GNUC_CONST;
GType gxml_cdata_get_type (void) G_GNUC_CONST;
GType gxml_tcdata_get_type (void) G_GNUC_CONST;
#define GXML_TCDATA_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GXML_TYPE_TCDATA, GXmlTCDATAPrivate))
enum  {
	GXML_TCDATA_DUMMY_PROPERTY,
	GXML_TCDATA_VALUE,
	GXML_TCDATA_STR
};
GXmlTCDATA* gxml_tcdata_new (GXmlDocument* d, const gchar* text);
GXmlTCDATA* gxml_tcdata_construct (GType object_type, GXmlDocument* d, const gchar* text);
GType gxml_tdocument_get_type (void) G_GNUC_CONST;
GXmlTNode* gxml_tnode_construct (GType object_type);
static GObject * gxml_tcdata_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void gxml_tcdata_finalize (GObject* obj);
gchar* gxml_tnode_get_value (GXmlTNode* self);
gchar* gxml_cdata_get_str (GXmlCDATA* self);
static void _vala_gxml_tcdata_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
void gxml_tnode_set_value (GXmlTNode* self, const gchar* value);
static void _vala_gxml_tcdata_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);


static gpointer _g_object_ref0 (gpointer self) {
	return self ? g_object_ref (self) : NULL;
}


GXmlTCDATA* gxml_tcdata_construct (GType object_type, GXmlDocument* d, const gchar* text) {
	GXmlTCDATA * self = NULL;
	GXmlDocument* _tmp0_ = NULL;
	GXmlDocument* _tmp1_ = NULL;
	GXmlDocument* _tmp2_ = NULL;
	const gchar* _tmp3_ = NULL;
	gchar* _tmp4_ = NULL;
	g_return_val_if_fail (d != NULL, NULL);
	g_return_val_if_fail (text != NULL, NULL);
	_tmp0_ = d;
	_vala_return_val_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (_tmp0_, GXML_TYPE_TDOCUMENT), "d is GXml.TDocument", NULL);
	self = (GXmlTCDATA*) gxml_tnode_construct (object_type);
	_tmp1_ = d;
	_tmp2_ = _g_object_ref0 (_tmp1_);
	_g_object_unref0 (((GXmlTNode*) self)->_doc);
	((GXmlTNode*) self)->_doc = _tmp2_;
	_tmp3_ = text;
	_tmp4_ = g_strdup (_tmp3_);
	_g_free0 (self->priv->_str);
	self->priv->_str = _tmp4_;
	return self;
}


GXmlTCDATA* gxml_tcdata_new (GXmlDocument* d, const gchar* text) {
	return gxml_tcdata_construct (GXML_TYPE_TCDATA, d, text);
}


static gchar* gxml_tcdata_real_get_value (GXmlTNode* base) {
	gchar* result;
	GXmlTCDATA* self;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	self = (GXmlTCDATA*) base;
	_tmp0_ = self->priv->_str;
	_tmp1_ = g_strdup (_tmp0_);
	result = _tmp1_;
	return result;
}


static void gxml_tcdata_real_set_value (GXmlTNode* base, const gchar* value) {
	GXmlTCDATA* self;
	self = (GXmlTCDATA*) base;
	g_object_notify ((GObject *) self, "value");
}


static gchar* gxml_tcdata_real_get_str (GXmlCDATA* base) {
	gchar* result;
	GXmlTCDATA* self;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	self = (GXmlTCDATA*) base;
	_tmp0_ = self->priv->_str;
	_tmp1_ = g_strdup (_tmp0_);
	result = _tmp1_;
	return result;
}


static GObject * gxml_tcdata_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GXmlTCDATA * self;
	gchar* _tmp0_ = NULL;
	parent_class = G_OBJECT_CLASS (gxml_tcdata_parent_class);
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GXML_TYPE_TCDATA, GXmlTCDATA);
	_tmp0_ = g_strdup ("#cdata");
	_g_free0 (((GXmlTNode*) self)->_name);
	((GXmlTNode*) self)->_name = _tmp0_;
	((GXmlTNode*) self)->_node_type = GXML_NODE_TYPE_CDATA_SECTION;
	return obj;
}


static void gxml_tcdata_class_init (GXmlTCDATAClass * klass) {
	gxml_tcdata_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GXmlTCDATAPrivate));
	GXML_TNODE_CLASS (klass)->get_value = gxml_tcdata_real_get_value;
	GXML_TNODE_CLASS (klass)->set_value = gxml_tcdata_real_set_value;
	G_OBJECT_CLASS (klass)->get_property = _vala_gxml_tcdata_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gxml_tcdata_set_property;
	G_OBJECT_CLASS (klass)->constructor = gxml_tcdata_constructor;
	G_OBJECT_CLASS (klass)->finalize = gxml_tcdata_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GXML_TCDATA_VALUE, g_param_spec_string ("value", "value", "value", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GXML_TCDATA_STR, g_param_spec_string ("str", "str", "str", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void gxml_tcdata_gxml_cdata_interface_init (GXmlCDATAIface * iface) {
	gxml_tcdata_gxml_cdata_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_str = gxml_tcdata_real_get_str;
}


static void gxml_tcdata_instance_init (GXmlTCDATA * self) {
	self->priv = GXML_TCDATA_GET_PRIVATE (self);
	self->priv->_str = NULL;
}


static void gxml_tcdata_finalize (GObject* obj) {
	GXmlTCDATA * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GXML_TYPE_TCDATA, GXmlTCDATA);
	_g_free0 (self->priv->_str);
	G_OBJECT_CLASS (gxml_tcdata_parent_class)->finalize (obj);
}


/**
 * Class implemeting {@link GXml.CDATA} interface, not tied to libxml-2.0 library.
 */
GType gxml_tcdata_get_type (void) {
	static volatile gsize gxml_tcdata_type_id__volatile = 0;
	if (g_once_init_enter (&gxml_tcdata_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GXmlTCDATAClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gxml_tcdata_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GXmlTCDATA), 0, (GInstanceInitFunc) gxml_tcdata_instance_init, NULL };
		static const GInterfaceInfo gxml_cdata_info = { (GInterfaceInitFunc) gxml_tcdata_gxml_cdata_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType gxml_tcdata_type_id;
		gxml_tcdata_type_id = g_type_register_static (GXML_TYPE_TNODE, "GXmlTCDATA", &g_define_type_info, 0);
		g_type_add_interface_static (gxml_tcdata_type_id, GXML_TYPE_CDATA, &gxml_cdata_info);
		g_once_init_leave (&gxml_tcdata_type_id__volatile, gxml_tcdata_type_id);
	}
	return gxml_tcdata_type_id__volatile;
}


static void _vala_gxml_tcdata_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GXmlTCDATA * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GXML_TYPE_TCDATA, GXmlTCDATA);
	switch (property_id) {
		case GXML_TCDATA_VALUE:
		g_value_take_string (value, gxml_tnode_get_value ((GXmlTNode*) self));
		break;
		case GXML_TCDATA_STR:
		g_value_take_string (value, gxml_cdata_get_str ((GXmlCDATA*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_gxml_tcdata_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GXmlTCDATA * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GXML_TYPE_TCDATA, GXmlTCDATA);
	switch (property_id) {
		case GXML_TCDATA_VALUE:
		gxml_tnode_set_value ((GXmlTNode*) self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}



