/* GomSchemaTest.c generated by valac 0.34.6, the Vala compiler
 * generated from GomSchemaTest.vala, do not modify */

/* -*- Mode: vala; indent-tabs-mode: t; c-basic-offset: 2; tab-width: 2 -*- */
/* GomSchema.vala
 *
 * Copyright (C) 2017  Daniel Espinosa <esodan@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Daniel Espinosa <esodan@gmail.com>
 */

#include <glib.h>
#include <glib-object.h>
#include <gio/gio.h>
#include <gxml/gxml.h>


#define TYPE_GXML_TEST (gxml_test_get_type ())
#define GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GXML_TEST, GXmlTest))
#define GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GXML_TEST, GXmlTestClass))
#define IS_GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GXML_TEST))
#define IS_GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GXML_TEST))
#define GXML_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GXML_TEST, GXmlTestClass))

typedef struct _GXmlTest GXmlTest;
typedef struct _GXmlTestClass GXmlTestClass;
typedef struct _GXmlTestPrivate GXmlTestPrivate;

#define TYPE_GOM_SCHEMA_TEST (gom_schema_test_get_type ())
#define GOM_SCHEMA_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GOM_SCHEMA_TEST, GomSchemaTest))
#define GOM_SCHEMA_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GOM_SCHEMA_TEST, GomSchemaTestClass))
#define IS_GOM_SCHEMA_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GOM_SCHEMA_TEST))
#define IS_GOM_SCHEMA_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GOM_SCHEMA_TEST))
#define GOM_SCHEMA_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GOM_SCHEMA_TEST, GomSchemaTestClass))

typedef struct _GomSchemaTest GomSchemaTest;
typedef struct _GomSchemaTestClass GomSchemaTestClass;
typedef struct _GomSchemaTestPrivate GomSchemaTestPrivate;
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _GXmlTest {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GXmlTestPrivate * priv;
};

struct _GXmlTestClass {
	GTypeClass parent_class;
	void (*finalize) (GXmlTest *self);
};

struct _GomSchemaTest {
	GXmlTest parent_instance;
	GomSchemaTestPrivate * priv;
};

struct _GomSchemaTestClass {
	GXmlTestClass parent_class;
};


static gpointer gom_schema_test_parent_class = NULL;

gpointer gxml_test_ref (gpointer instance);
void gxml_test_unref (gpointer instance);
GParamSpec* param_spec_gxml_test (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_gxml_test (GValue* value, gpointer v_object);
void value_take_gxml_test (GValue* value, gpointer v_object);
gpointer value_get_gxml_test (const GValue* value);
GType gxml_test_get_type (void) G_GNUC_CONST;
GType gom_schema_test_get_type (void) G_GNUC_CONST;
enum  {
	GOM_SCHEMA_TEST_DUMMY_PROPERTY
};
void gom_schema_test_add_tests (void);
static void __lambda259_ (void);
static void ___lambda259__gtest_func (void);
static void __lambda260_ (void);
static void ___lambda260__gtest_func (void);
GomSchemaTest* gom_schema_test_new (void);
GomSchemaTest* gom_schema_test_construct (GType object_type);
GXmlTest* gxml_test_new (void);
GXmlTest* gxml_test_construct (GType object_type);


static void __lambda259_ (void) {
	GError * _inner_error_ = NULL;
	{
		GFile* f = NULL;
		GFile* _tmp0_ = NULL;
		GFile* _tmp1_ = NULL;
		gboolean _tmp2_ = FALSE;
		GXmlGomXsdSchema* sch = NULL;
		GXmlGomXsdSchema* _tmp3_ = NULL;
		GXmlGomXsdSchema* _tmp4_ = NULL;
		gchar* _tmp5_ = NULL;
		gchar* _tmp6_ = NULL;
		gchar* _tmp7_ = NULL;
		GXmlGomXsdSchema* _tmp8_ = NULL;
		GFile* _tmp9_ = NULL;
		GXmlGomXsdSchema* _tmp10_ = NULL;
		GXmlGomXsdListSimpleTypes* _tmp11_ = NULL;
		GXmlGomXsdListSimpleTypes* _tmp12_ = NULL;
		GXmlDomElement* _tmp13_ = NULL;
		GXmlGomXsdSchema* _tmp14_ = NULL;
		GXmlGomXsdListSimpleTypes* _tmp15_ = NULL;
		GXmlGomXsdListSimpleTypes* _tmp16_ = NULL;
		GXmlDomElement* _tmp17_ = NULL;
		GXmlGomXsdSimpleType* st = NULL;
		GXmlDomElement* _tmp18_ = NULL;
		GXmlGomXsdSimpleType* _tmp19_ = NULL;
		GXmlGomXsdSimpleType* _tmp20_ = NULL;
		GXmlGomXsdSimpleType* _tmp21_ = NULL;
		GXmlGomXsdTypeRestriction* _tmp22_ = NULL;
		GXmlGomXsdTypeRestriction* _tmp23_ = NULL;
		GXmlGomXsdSimpleType* _tmp24_ = NULL;
		GXmlGomXsdTypeRestriction* _tmp25_ = NULL;
		GXmlGomXsdTypeRestriction* _tmp26_ = NULL;
		const gchar* _tmp27_ = NULL;
		const gchar* _tmp28_ = NULL;
		GXmlGomXsdSimpleType* _tmp29_ = NULL;
		GXmlGomXsdTypeRestriction* _tmp30_ = NULL;
		GXmlGomXsdTypeRestriction* _tmp31_ = NULL;
		const gchar* _tmp32_ = NULL;
		const gchar* _tmp33_ = NULL;
		GXmlGomXsdSimpleType* _tmp34_ = NULL;
		GXmlGomXsdTypeRestriction* _tmp35_ = NULL;
		GXmlGomXsdTypeRestriction* _tmp36_ = NULL;
		GXmlGomXsdListTypeRestrictionEnumerations* _tmp37_ = NULL;
		GXmlGomXsdListTypeRestrictionEnumerations* _tmp38_ = NULL;
		GXmlGomXsdSimpleType* _tmp39_ = NULL;
		GXmlGomXsdTypeRestriction* _tmp40_ = NULL;
		GXmlGomXsdTypeRestriction* _tmp41_ = NULL;
		GXmlGomXsdListTypeRestrictionEnumerations* _tmp42_ = NULL;
		GXmlGomXsdListTypeRestrictionEnumerations* _tmp43_ = NULL;
		gint _tmp44_ = 0;
		gint _tmp45_ = 0;
		GXmlDomElement* _tmp46_ = NULL;
		GXmlGomXsdSimpleType* _tmp47_ = NULL;
		GXmlGomXsdTypeRestriction* _tmp48_ = NULL;
		GXmlGomXsdTypeRestriction* _tmp49_ = NULL;
		GXmlGomXsdListTypeRestrictionEnumerations* _tmp50_ = NULL;
		GXmlGomXsdListTypeRestrictionEnumerations* _tmp51_ = NULL;
		GXmlDomElement* _tmp52_ = NULL;
		GXmlGomXsdTypeRestrictionEnumeration* enumt1 = NULL;
		GXmlDomElement* _tmp53_ = NULL;
		GXmlGomXsdTypeRestrictionEnumeration* _tmp54_ = NULL;
		GXmlGomXsdTypeRestrictionEnumeration* _tmp55_ = NULL;
		GXmlGomXsdTypeRestrictionEnumeration* _tmp56_ = NULL;
		const gchar* _tmp57_ = NULL;
		const gchar* _tmp58_ = NULL;
		GXmlGomXsdTypeRestrictionEnumeration* _tmp59_ = NULL;
		const gchar* _tmp60_ = NULL;
		const gchar* _tmp61_ = NULL;
		GXmlDomElement* _tmp62_ = NULL;
		GXmlGomXsdSimpleType* _tmp63_ = NULL;
		GXmlGomXsdTypeRestriction* _tmp64_ = NULL;
		GXmlGomXsdTypeRestriction* _tmp65_ = NULL;
		GXmlGomXsdListTypeRestrictionEnumerations* _tmp66_ = NULL;
		GXmlGomXsdListTypeRestrictionEnumerations* _tmp67_ = NULL;
		GXmlDomElement* _tmp68_ = NULL;
		GXmlGomXsdTypeRestrictionEnumeration* enumt20 = NULL;
		GXmlDomElement* _tmp69_ = NULL;
		GXmlGomXsdTypeRestrictionEnumeration* _tmp70_ = NULL;
		const gchar* _tmp71_ = NULL;
		const gchar* _tmp72_ = NULL;
		const gchar* _tmp73_ = NULL;
		const gchar* _tmp74_ = NULL;
#line 29 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp0_ = g_file_new_for_path (TEST_DIR "/schema-test.xsd");
#line 29 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		f = _tmp0_;
#line 30 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp1_ = f;
#line 30 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp2_ = g_file_query_exists (_tmp1_, NULL);
#line 30 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (_tmp2_, "f.query_exists ()");
#line 31 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp3_ = gxml_gom_xsd_schema_new ();
#line 31 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		sch = _tmp3_;
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp4_ = sch;
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp5_ = gxml_dom_element_get_local_name ((GXmlDomElement*) _tmp4_);
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp6_ = _tmp5_;
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp7_ = _tmp6_;
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (g_strcmp0 (_tmp7_, "schema") == 0, "sch.local_name == \"schema\"");
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_g_free0 (_tmp7_);
#line 37 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp8_ = sch;
#line 37 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp9_ = f;
#line 37 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		gxml_gom_element_read_from_file ((GXmlGomElement*) _tmp8_, _tmp9_, NULL, &_inner_error_);
#line 37 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 37 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
			_g_object_unref0 (sch);
#line 37 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
			_g_object_unref0 (f);
#line 228 "GomSchemaTest.c"
			goto __catch283_g_error;
		}
#line 41 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp10_ = sch;
#line 41 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp11_ = gxml_gom_xsd_schema_get_simple_type_definitions (_tmp10_);
#line 41 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp12_ = _tmp11_;
#line 41 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (_tmp12_ != NULL, "sch.simple_type_definitions != null");
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp14_ = sch;
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp15_ = gxml_gom_xsd_schema_get_simple_type_definitions (_tmp14_);
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp16_ = _tmp15_;
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp17_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp16_, 0, &_inner_error_);
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp13_ = _tmp17_;
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
			_g_object_unref0 (sch);
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
			_g_object_unref0 (f);
#line 255 "GomSchemaTest.c"
			goto __catch283_g_error;
		}
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp18_ = _tmp13_;
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp13_ = NULL;
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp19_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp18_, GXML_TYPE_GOM_XSD_SIMPLE_TYPE) ? ((GXmlGomXsdSimpleType*) _tmp18_) : NULL;
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		if (_tmp19_ == NULL) {
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
			_g_object_unref0 (_tmp18_);
#line 268 "GomSchemaTest.c"
		}
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		st = _tmp19_;
#line 43 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp20_ = st;
#line 43 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (_tmp20_ != NULL, "st != null");
#line 44 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp21_ = st;
#line 44 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp22_ = gxml_gom_xsd_simple_type_get_restriction (_tmp21_);
#line 44 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp23_ = _tmp22_;
#line 44 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (_tmp23_ != NULL, "st.restriction != null");
#line 45 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp24_ = st;
#line 45 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp25_ = gxml_gom_xsd_simple_type_get_restriction (_tmp24_);
#line 45 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp26_ = _tmp25_;
#line 45 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp27_ = gxml_gom_xsd_type_restriction_get_base (_tmp26_);
#line 45 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp28_ = _tmp27_;
#line 45 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (_tmp28_ != NULL, "st.restriction.base != null");
#line 46 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp29_ = st;
#line 46 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp30_ = gxml_gom_xsd_simple_type_get_restriction (_tmp29_);
#line 46 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp31_ = _tmp30_;
#line 46 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp32_ = gxml_gom_xsd_type_restriction_get_base (_tmp31_);
#line 46 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp33_ = _tmp32_;
#line 46 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (g_strcmp0 (_tmp33_, "xs:string") == 0, "st.restriction.base == \"xs:string\"");
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp34_ = st;
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp35_ = gxml_gom_xsd_simple_type_get_restriction (_tmp34_);
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp36_ = _tmp35_;
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp37_ = gxml_gom_xsd_type_restriction_get_enumerations (_tmp36_);
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp38_ = _tmp37_;
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (_tmp38_ != NULL, "st.restriction.enumerations != null");
#line 48 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp39_ = st;
#line 48 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp40_ = gxml_gom_xsd_simple_type_get_restriction (_tmp39_);
#line 48 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp41_ = _tmp40_;
#line 48 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp42_ = gxml_gom_xsd_type_restriction_get_enumerations (_tmp41_);
#line 48 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp43_ = _tmp42_;
#line 48 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp44_ = gxml_gom_xsd_list_get_length ((GXmlGomXsdList*) _tmp43_);
#line 48 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp45_ = _tmp44_;
#line 48 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (_tmp45_ == 20, "st.restriction.enumerations.length == 20");
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp47_ = st;
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp48_ = gxml_gom_xsd_simple_type_get_restriction (_tmp47_);
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp49_ = _tmp48_;
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp50_ = gxml_gom_xsd_type_restriction_get_enumerations (_tmp49_);
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp51_ = _tmp50_;
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp52_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp51_, 0, &_inner_error_);
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp46_ = _tmp52_;
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
			_g_object_unref0 (st);
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
			_g_object_unref0 (_tmp13_);
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
			_g_object_unref0 (sch);
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
			_g_object_unref0 (f);
#line 360 "GomSchemaTest.c"
			goto __catch283_g_error;
		}
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp53_ = _tmp46_;
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp46_ = NULL;
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp54_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp53_, GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_ENUMERATION) ? ((GXmlGomXsdTypeRestrictionEnumeration*) _tmp53_) : NULL;
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		if (_tmp54_ == NULL) {
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
			_g_object_unref0 (_tmp53_);
#line 373 "GomSchemaTest.c"
		}
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		enumt1 = _tmp54_;
#line 50 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp55_ = enumt1;
#line 50 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (_tmp55_ != NULL, "enumt1 != null");
#line 51 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp56_ = enumt1;
#line 51 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp57_ = gxml_gom_xsd_type_restriction_enumeration_get_value (_tmp56_);
#line 51 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp58_ = _tmp57_;
#line 51 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (_tmp58_ != NULL, "enumt1.value != null");
#line 52 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp59_ = enumt1;
#line 52 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp60_ = gxml_gom_xsd_type_restriction_enumeration_get_value (_tmp59_);
#line 52 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp61_ = _tmp60_;
#line 52 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (g_strcmp0 (_tmp61_, "01") == 0, "enumt1.value == \"01\"");
#line 53 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp63_ = st;
#line 53 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp64_ = gxml_gom_xsd_simple_type_get_restriction (_tmp63_);
#line 53 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp65_ = _tmp64_;
#line 53 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp66_ = gxml_gom_xsd_type_restriction_get_enumerations (_tmp65_);
#line 53 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp67_ = _tmp66_;
#line 53 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp68_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp67_, 19, &_inner_error_);
#line 53 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp62_ = _tmp68_;
#line 53 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 53 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
			_g_object_unref0 (enumt1);
#line 53 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
			_g_object_unref0 (_tmp46_);
#line 53 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
			_g_object_unref0 (st);
#line 53 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
			_g_object_unref0 (_tmp13_);
#line 53 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
			_g_object_unref0 (sch);
#line 53 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
			_g_object_unref0 (f);
#line 425 "GomSchemaTest.c"
			goto __catch283_g_error;
		}
#line 53 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp69_ = _tmp62_;
#line 53 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp62_ = NULL;
#line 53 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp70_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp69_, GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_ENUMERATION) ? ((GXmlGomXsdTypeRestrictionEnumeration*) _tmp69_) : NULL;
#line 53 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		if (_tmp70_ == NULL) {
#line 53 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
			_g_object_unref0 (_tmp69_);
#line 438 "GomSchemaTest.c"
		}
#line 53 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		enumt20 = _tmp70_;
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (enumt20 != NULL, "enumt20 != null");
#line 55 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp71_ = gxml_gom_xsd_type_restriction_enumeration_get_value (enumt20);
#line 55 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp72_ = _tmp71_;
#line 55 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (_tmp72_ != NULL, "enumt20.value != null");
#line 56 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp73_ = gxml_gom_xsd_type_restriction_enumeration_get_value (enumt20);
#line 56 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp74_ = _tmp73_;
#line 56 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (g_strcmp0 (_tmp74_, "99") == 0, "enumt20.value == \"99\"");
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_g_object_unref0 (enumt20);
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_g_object_unref0 (_tmp62_);
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_g_object_unref0 (enumt1);
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_g_object_unref0 (_tmp46_);
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_g_object_unref0 (st);
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_g_object_unref0 (_tmp13_);
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_g_object_unref0 (sch);
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_g_object_unref0 (f);
#line 472 "GomSchemaTest.c"
	}
	goto __finally283;
	__catch283_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp75_ = NULL;
		gchar* _tmp76_ = NULL;
		gchar* _tmp77_ = NULL;
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		e = _inner_error_;
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_inner_error_ = NULL;
#line 58 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp75_ = e->message;
#line 58 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp76_ = g_strconcat ("Error: ", _tmp75_, NULL);
#line 58 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp77_ = _tmp76_;
#line 58 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		g_message ("GomSchemaTest.vala:58: %s", _tmp77_);
#line 58 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_g_free0 (_tmp77_);
#line 59 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		g_assert_not_reached ();
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_g_error_free0 (e);
#line 499 "GomSchemaTest.c"
	}
	__finally283:
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		g_clear_error (&_inner_error_);
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		return;
#line 510 "GomSchemaTest.c"
	}
}


static void ___lambda259__gtest_func (void) {
#line 27 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
	__lambda259_ ();
#line 518 "GomSchemaTest.c"
}


static void __lambda260_ (void) {
	GError * _inner_error_ = NULL;
	{
		GFile* f = NULL;
		GFile* _tmp0_ = NULL;
		gboolean _tmp1_ = FALSE;
		GXmlGomXsdArrayString* ars = NULL;
		GXmlGomXsdArrayString* _tmp2_ = NULL;
		gboolean _tmp3_ = FALSE;
		gboolean _tmp4_ = FALSE;
		gboolean _tmp5_ = FALSE;
		gboolean _tmp6_ = FALSE;
		gboolean _tmp7_ = FALSE;
		gboolean _tmp8_ = FALSE;
		gboolean _tmp9_ = FALSE;
		gboolean _tmp10_ = FALSE;
#line 64 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp0_ = g_file_new_for_path (TEST_DIR "/schema-test.xsd");
#line 64 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		f = _tmp0_;
#line 65 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp1_ = g_file_query_exists (f, NULL);
#line 65 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (_tmp1_, "f.query_exists ()");
#line 66 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp2_ = gxml_gom_xsd_array_string_new ();
#line 66 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		ars = _tmp2_;
#line 67 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		gxml_gom_xsd_array_string_set_simple_type (ars, "MethodCode");
#line 68 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		gxml_gom_xsd_array_string_set_source (ars, f);
#line 69 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		gxml_gom_xsd_array_string_load (ars, &_inner_error_);
#line 69 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 69 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
			_g_object_unref0 (ars);
#line 69 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
			_g_object_unref0 (f);
#line 562 "GomSchemaTest.c"
			goto __catch284_g_error;
		}
#line 70 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp3_ = gxml_gom_array_string_search ((GXmlGomArrayString*) ars, "01");
#line 70 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (_tmp3_, "ars.search (\"01\")");
#line 71 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp4_ = gxml_gom_array_string_search ((GXmlGomArrayString*) ars, "02");
#line 71 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (_tmp4_, "ars.search (\"02\")");
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp5_ = gxml_gom_array_string_search ((GXmlGomArrayString*) ars, "03");
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (_tmp5_, "ars.search (\"03\")");
#line 73 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp6_ = gxml_gom_array_string_search ((GXmlGomArrayString*) ars, "04");
#line 73 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (_tmp6_, "ars.search (\"04\")");
#line 74 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp7_ = gxml_gom_array_string_search ((GXmlGomArrayString*) ars, "05");
#line 74 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (_tmp7_, "ars.search (\"05\")");
#line 75 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp8_ = gxml_gom_array_string_search ((GXmlGomArrayString*) ars, "06");
#line 75 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (_tmp8_, "ars.search (\"06\")");
#line 76 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp9_ = gxml_gom_array_string_search ((GXmlGomArrayString*) ars, "07");
#line 76 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (!_tmp9_, "!ars.search (\"07\")");
#line 77 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp10_ = gxml_gom_array_string_search ((GXmlGomArrayString*) ars, "99");
#line 77 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_vala_assert (_tmp10_, "ars.search (\"99\")");
#line 63 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_g_object_unref0 (ars);
#line 63 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_g_object_unref0 (f);
#line 601 "GomSchemaTest.c"
	}
	goto __finally284;
	__catch284_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp11_ = NULL;
		gchar* _tmp12_ = NULL;
		gchar* _tmp13_ = NULL;
#line 63 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		e = _inner_error_;
#line 63 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_inner_error_ = NULL;
#line 79 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp11_ = e->message;
#line 79 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp12_ = g_strconcat ("Error: ", _tmp11_, NULL);
#line 79 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_tmp13_ = _tmp12_;
#line 79 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		g_message ("GomSchemaTest.vala:79: %s", _tmp13_);
#line 79 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_g_free0 (_tmp13_);
#line 80 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		g_assert_not_reached ();
#line 63 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		_g_error_free0 (e);
#line 628 "GomSchemaTest.c"
	}
	__finally284:
#line 63 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 63 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 63 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		g_clear_error (&_inner_error_);
#line 63 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
		return;
#line 639 "GomSchemaTest.c"
	}
}


static void ___lambda260__gtest_func (void) {
#line 62 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
	__lambda260_ ();
#line 647 "GomSchemaTest.c"
}


void gom_schema_test_add_tests (void) {
#line 27 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
	g_test_add_func ("/gxml/gom-schema/read", ___lambda259__gtest_func);
#line 62 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
	g_test_add_func ("/gxml/xsd-array-string/attribute-enumeration", ___lambda260__gtest_func);
#line 656 "GomSchemaTest.c"
}


GomSchemaTest* gom_schema_test_construct (GType object_type) {
	GomSchemaTest* self = NULL;
#line 25 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
	self = (GomSchemaTest*) gxml_test_construct (object_type);
#line 25 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
	return self;
#line 666 "GomSchemaTest.c"
}


GomSchemaTest* gom_schema_test_new (void) {
#line 25 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
	return gom_schema_test_construct (TYPE_GOM_SCHEMA_TEST);
#line 673 "GomSchemaTest.c"
}


static void gom_schema_test_class_init (GomSchemaTestClass * klass) {
#line 25 "/home/despinosa/Proyectos/gxml/test/GomSchemaTest.vala"
	gom_schema_test_parent_class = g_type_class_peek_parent (klass);
#line 680 "GomSchemaTest.c"
}


static void gom_schema_test_instance_init (GomSchemaTest * self) {
}


GType gom_schema_test_get_type (void) {
	static volatile gsize gom_schema_test_type_id__volatile = 0;
	if (g_once_init_enter (&gom_schema_test_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSchemaTestClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_schema_test_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSchemaTest), 0, (GInstanceInitFunc) gom_schema_test_instance_init, NULL };
		GType gom_schema_test_type_id;
		gom_schema_test_type_id = g_type_register_static (TYPE_GXML_TEST, "GomSchemaTest", &g_define_type_info, 0);
		g_once_init_leave (&gom_schema_test_type_id__volatile, gom_schema_test_type_id);
	}
	return gom_schema_test_type_id__volatile;
}



