/* SerializableGeeDualKeyMapTest.c generated by valac 0.34.6, the Vala compiler
 * generated from SerializableGeeDualKeyMapTest.vala, do not modify */

/* -*- Mode: vala; indent-tabs-mode: nil; c-basic-offset: 2; tab-width: 2 -*- */

#include <glib.h>
#include <glib-object.h>
#include <gxml/gxml.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <gee.h>


#define TYPE_SPACES (spaces_get_type ())
#define SPACES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SPACES, Spaces))
#define SPACES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SPACES, SpacesClass))
#define IS_SPACES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SPACES))
#define IS_SPACES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SPACES))
#define SPACES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SPACES, SpacesClass))

typedef struct _Spaces Spaces;
typedef struct _SpacesClass SpacesClass;
typedef struct _SpacesPrivate SpacesPrivate;
#define _g_free0(var) (var = (g_free (var), NULL))

#define TYPE_GXML_TEST (gxml_test_get_type ())
#define GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GXML_TEST, GXmlTest))
#define GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GXML_TEST, GXmlTestClass))
#define IS_GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GXML_TEST))
#define IS_GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GXML_TEST))
#define GXML_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GXML_TEST, GXmlTestClass))

typedef struct _GXmlTest GXmlTest;
typedef struct _GXmlTestClass GXmlTestClass;
typedef struct _GXmlTestPrivate GXmlTestPrivate;

#define TYPE_SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST (serializable_gee_dual_key_map_test_get_type ())
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST, SerializableGeeDualKeyMapTest))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST, SerializableGeeDualKeyMapTestClass))
#define IS_SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST))
#define IS_SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST, SerializableGeeDualKeyMapTestClass))

typedef struct _SerializableGeeDualKeyMapTest SerializableGeeDualKeyMapTest;
typedef struct _SerializableGeeDualKeyMapTestClass SerializableGeeDualKeyMapTestClass;
typedef struct _SerializableGeeDualKeyMapTestPrivate SerializableGeeDualKeyMapTestPrivate;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))

#define TYPE_BOOK_STORE (book_store_get_type ())
#define BOOK_STORE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_BOOK_STORE, BookStore))
#define BOOK_STORE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_BOOK_STORE, BookStoreClass))
#define IS_BOOK_STORE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_BOOK_STORE))
#define IS_BOOK_STORE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_BOOK_STORE))
#define BOOK_STORE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_BOOK_STORE, BookStoreClass))

typedef struct _BookStore BookStore;
typedef struct _BookStoreClass BookStoreClass;

#define BOOK_TYPE_ARRAY (book_array_get_type ())
#define BOOK_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOOK_TYPE_ARRAY, BookArray))
#define BOOK_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOOK_TYPE_ARRAY, BookArrayClass))
#define BOOK_IS_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOOK_TYPE_ARRAY))
#define BOOK_IS_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOOK_TYPE_ARRAY))
#define BOOK_ARRAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOOK_TYPE_ARRAY, BookArrayClass))

typedef struct _BookArray BookArray;
typedef struct _BookArrayClass BookArrayClass;

#define TYPE_BOOK (book_get_type ())
#define BOOK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_BOOK, Book))
#define BOOK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_BOOK, BookClass))
#define IS_BOOK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_BOOK))
#define IS_BOOK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_BOOK))
#define BOOK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_BOOK, BookClass))

typedef struct _Book Book;
typedef struct _BookClass BookClass;

#define TYPE_NAME (name_get_type ())
#define NAME(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_NAME, Name))
#define NAME_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_NAME, NameClass))
#define IS_NAME(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_NAME))
#define IS_NAME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_NAME))
#define NAME_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_NAME, NameClass))

typedef struct _Name Name;
typedef struct _NameClass NameClass;

#define TYPE_AUTHORS (authors_get_type ())
#define AUTHORS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_AUTHORS, Authors))
#define AUTHORS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_AUTHORS, AuthorsClass))
#define IS_AUTHORS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_AUTHORS))
#define IS_AUTHORS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_AUTHORS))
#define AUTHORS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_AUTHORS, AuthorsClass))

typedef struct _Authors Authors;
typedef struct _AuthorsClass AuthorsClass;

#define AUTHOR_TYPE_ARRAY (author_array_get_type ())
#define AUTHOR_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), AUTHOR_TYPE_ARRAY, AuthorArray))
#define AUTHOR_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), AUTHOR_TYPE_ARRAY, AuthorArrayClass))
#define AUTHOR_IS_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), AUTHOR_TYPE_ARRAY))
#define AUTHOR_IS_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), AUTHOR_TYPE_ARRAY))
#define AUTHOR_ARRAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), AUTHOR_TYPE_ARRAY, AuthorArrayClass))

typedef struct _AuthorArray AuthorArray;
typedef struct _AuthorArrayClass AuthorArrayClass;

#define INVENTORY_TYPE_DUAL_KEY_MAP (inventory_dual_key_map_get_type ())
#define INVENTORY_DUAL_KEY_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), INVENTORY_TYPE_DUAL_KEY_MAP, InventoryDualKeyMap))
#define INVENTORY_DUAL_KEY_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), INVENTORY_TYPE_DUAL_KEY_MAP, InventoryDualKeyMapClass))
#define INVENTORY_IS_DUAL_KEY_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), INVENTORY_TYPE_DUAL_KEY_MAP))
#define INVENTORY_IS_DUAL_KEY_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), INVENTORY_TYPE_DUAL_KEY_MAP))
#define INVENTORY_DUAL_KEY_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), INVENTORY_TYPE_DUAL_KEY_MAP, InventoryDualKeyMapClass))

typedef struct _InventoryDualKeyMap InventoryDualKeyMap;
typedef struct _InventoryDualKeyMapClass InventoryDualKeyMapClass;

#define TYPE_INVENTORY (inventory_get_type ())
#define INVENTORY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_INVENTORY, Inventory))
#define INVENTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_INVENTORY, InventoryClass))
#define IS_INVENTORY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_INVENTORY))
#define IS_INVENTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_INVENTORY))
#define INVENTORY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_INVENTORY, InventoryClass))

typedef struct _Inventory Inventory;
typedef struct _InventoryClass InventoryClass;

#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BIG_BAG (serializable_gee_dual_key_map_test_big_bag_get_type ())
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BIG_BAG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BIG_BAG, SerializableGeeDualKeyMapTestBigBag))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BIG_BAG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BIG_BAG, SerializableGeeDualKeyMapTestBigBagClass))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_IS_BIG_BAG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BIG_BAG))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_IS_BIG_BAG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BIG_BAG))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BIG_BAG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BIG_BAG, SerializableGeeDualKeyMapTestBigBagClass))

typedef struct _SerializableGeeDualKeyMapTestBigBag SerializableGeeDualKeyMapTestBigBag;
typedef struct _SerializableGeeDualKeyMapTestBigBagClass SerializableGeeDualKeyMapTestBigBagClass;

#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_TYPE_DUAL_MAP (serializable_gee_dual_key_map_test_small_bag_dual_map_get_type ())
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_DUAL_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_TYPE_DUAL_MAP, SerializableGeeDualKeyMapTestSmallBagDualMap))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_DUAL_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_TYPE_DUAL_MAP, SerializableGeeDualKeyMapTestSmallBagDualMapClass))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_IS_DUAL_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_TYPE_DUAL_MAP))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_IS_DUAL_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_TYPE_DUAL_MAP))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_DUAL_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_TYPE_DUAL_MAP, SerializableGeeDualKeyMapTestSmallBagDualMapClass))

typedef struct _SerializableGeeDualKeyMapTestSmallBagDualMap SerializableGeeDualKeyMapTestSmallBagDualMap;
typedef struct _SerializableGeeDualKeyMapTestSmallBagDualMapClass SerializableGeeDualKeyMapTestSmallBagDualMapClass;

#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_SMALL_BAG (serializable_gee_dual_key_map_test_small_bag_get_type ())
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_SMALL_BAG, SerializableGeeDualKeyMapTestSmallBag))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_SMALL_BAG, SerializableGeeDualKeyMapTestSmallBagClass))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_IS_SMALL_BAG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_SMALL_BAG))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_IS_SMALL_BAG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_SMALL_BAG))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_SMALL_BAG, SerializableGeeDualKeyMapTestSmallBagClass))

typedef struct _SerializableGeeDualKeyMapTestSmallBag SerializableGeeDualKeyMapTestSmallBag;
typedef struct _SerializableGeeDualKeyMapTestSmallBagClass SerializableGeeDualKeyMapTestSmallBagClass;

#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_TYPE_DUAL_MAP (serializable_gee_dual_key_map_test_ball_dual_map_get_type ())
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_DUAL_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_TYPE_DUAL_MAP, SerializableGeeDualKeyMapTestBallDualMap))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_DUAL_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_TYPE_DUAL_MAP, SerializableGeeDualKeyMapTestBallDualMapClass))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_IS_DUAL_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_TYPE_DUAL_MAP))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_IS_DUAL_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_TYPE_DUAL_MAP))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_DUAL_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_TYPE_DUAL_MAP, SerializableGeeDualKeyMapTestBallDualMapClass))

typedef struct _SerializableGeeDualKeyMapTestBallDualMap SerializableGeeDualKeyMapTestBallDualMap;
typedef struct _SerializableGeeDualKeyMapTestBallDualMapClass SerializableGeeDualKeyMapTestBallDualMapClass;

#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL (serializable_gee_dual_key_map_test_ball_get_type ())
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL, SerializableGeeDualKeyMapTestBall))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL, SerializableGeeDualKeyMapTestBallClass))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_IS_BALL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_IS_BALL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL, SerializableGeeDualKeyMapTestBallClass))

typedef struct _SerializableGeeDualKeyMapTestBall SerializableGeeDualKeyMapTestBall;
typedef struct _SerializableGeeDualKeyMapTestBallClass SerializableGeeDualKeyMapTestBallClass;

#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL_FILL (serializable_gee_dual_key_map_test_ball_fill_get_type ())
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_FILL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL_FILL, SerializableGeeDualKeyMapTestBallFill))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_FILL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL_FILL, SerializableGeeDualKeyMapTestBallFillClass))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_IS_BALL_FILL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL_FILL))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_IS_BALL_FILL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL_FILL))
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_FILL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL_FILL, SerializableGeeDualKeyMapTestBallFillClass))

typedef struct _SerializableGeeDualKeyMapTestBallFill SerializableGeeDualKeyMapTestBallFill;
typedef struct _SerializableGeeDualKeyMapTestBallFillClass SerializableGeeDualKeyMapTestBallFillClass;
typedef struct _SerializableGeeDualKeyMapTestBallFillPrivate SerializableGeeDualKeyMapTestBallFillPrivate;
typedef struct _SerializableGeeDualKeyMapTestBallPrivate SerializableGeeDualKeyMapTestBallPrivate;
typedef struct _SerializableGeeDualKeyMapTestBallDualMapPrivate SerializableGeeDualKeyMapTestBallDualMapPrivate;
typedef struct _SerializableGeeDualKeyMapTestSmallBagPrivate SerializableGeeDualKeyMapTestSmallBagPrivate;
typedef struct _SerializableGeeDualKeyMapTestSmallBagDualMapPrivate SerializableGeeDualKeyMapTestSmallBagDualMapPrivate;
typedef struct _SerializableGeeDualKeyMapTestBigBagPrivate SerializableGeeDualKeyMapTestBigBagPrivate;
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _Spaces {
	GXmlSerializableObjectModel parent_instance;
	SpacesPrivate * priv;
};

struct _SpacesClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SpacesPrivate {
	gchar* _name;
	gchar* _owner;
};

struct _GXmlTest {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GXmlTestPrivate * priv;
};

struct _GXmlTestClass {
	GTypeClass parent_class;
	void (*finalize) (GXmlTest *self);
};

struct _SerializableGeeDualKeyMapTest {
	GXmlTest parent_instance;
	SerializableGeeDualKeyMapTestPrivate * priv;
};

struct _SerializableGeeDualKeyMapTestClass {
	GXmlTestClass parent_class;
};

struct _SerializableGeeDualKeyMapTestBallFill {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeDualKeyMapTestBallFillPrivate * priv;
};

struct _SerializableGeeDualKeyMapTestBallFillClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeDualKeyMapTestBallFillPrivate {
	gchar* _name;
};

struct _SerializableGeeDualKeyMapTestBall {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeDualKeyMapTestBallPrivate * priv;
};

struct _SerializableGeeDualKeyMapTestBallClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeDualKeyMapTestBallPrivate {
	gchar* _name;
	gchar* _size;
	SerializableGeeDualKeyMapTestBallFill* _ballfill;
};

struct _SerializableGeeDualKeyMapTestBallDualMap {
	GXmlSerializableDualKeyMap parent_instance;
	SerializableGeeDualKeyMapTestBallDualMapPrivate * priv;
};

struct _SerializableGeeDualKeyMapTestBallDualMapClass {
	GXmlSerializableDualKeyMapClass parent_class;
};

struct _SerializableGeeDualKeyMapTestSmallBag {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeDualKeyMapTestSmallBagPrivate * priv;
};

struct _SerializableGeeDualKeyMapTestSmallBagClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeDualKeyMapTestSmallBagPrivate {
	gchar* _name;
	gchar* _cash;
	SerializableGeeDualKeyMapTestBallDualMap* _balls;
};

struct _SerializableGeeDualKeyMapTestSmallBagDualMap {
	GXmlSerializableDualKeyMap parent_instance;
	SerializableGeeDualKeyMapTestSmallBagDualMapPrivate * priv;
};

struct _SerializableGeeDualKeyMapTestSmallBagDualMapClass {
	GXmlSerializableDualKeyMapClass parent_class;
};

struct _SerializableGeeDualKeyMapTestBigBag {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeDualKeyMapTestBigBagPrivate * priv;
};

struct _SerializableGeeDualKeyMapTestBigBagClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeDualKeyMapTestBigBagPrivate {
	gchar* _name;
	SerializableGeeDualKeyMapTestSmallBagDualMap* _bags;
};


static gpointer spaces_parent_class = NULL;
static GXmlSerializableMapDualKeyIface* spaces_gxml_serializable_map_dual_key_parent_iface = NULL;
static gpointer serializable_gee_dual_key_map_test_parent_class = NULL;
static gpointer serializable_gee_dual_key_map_test_ball_fill_parent_class = NULL;
static gpointer serializable_gee_dual_key_map_test_ball_parent_class = NULL;
static gpointer serializable_gee_dual_key_map_test_ball_dual_map_parent_class = NULL;
static GXmlSerializableMapDualKeyIface* serializable_gee_dual_key_map_test_ball_gxml_serializable_map_dual_key_parent_iface = NULL;
static gpointer serializable_gee_dual_key_map_test_small_bag_parent_class = NULL;
static gpointer serializable_gee_dual_key_map_test_small_bag_dual_map_parent_class = NULL;
static GXmlSerializableMapDualKeyIface* serializable_gee_dual_key_map_test_small_bag_gxml_serializable_map_dual_key_parent_iface = NULL;
static gpointer serializable_gee_dual_key_map_test_big_bag_parent_class = NULL;

GType spaces_get_type (void) G_GNUC_CONST;
#define SPACES_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_SPACES, SpacesPrivate))
enum  {
	SPACES_DUMMY_PROPERTY,
	SPACES_NAME,
	SPACES_OWNER
};
static gchar* spaces_real_get_map_primary_key (GXmlSerializableMapDualKey* base);
const gchar* spaces_get_owner (Spaces* self);
static gchar* spaces_real_get_map_secondary_key (GXmlSerializableMapDualKey* base);
const gchar* spaces_get_name (Spaces* self);
Spaces* spaces_new_full (const gchar* owner, const gchar* name);
Spaces* spaces_construct_full (GType object_type, const gchar* owner, const gchar* name);
void spaces_set_name (Spaces* self, const gchar* value);
void spaces_set_owner (Spaces* self, const gchar* value);
static gchar* spaces_real_to_string (GXmlSerializableObjectModel* base);
Spaces* spaces_new (void);
Spaces* spaces_construct (GType object_type);
static void spaces_finalize (GObject* obj);
static void _vala_spaces_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_spaces_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
gpointer gxml_test_ref (gpointer instance);
void gxml_test_unref (gpointer instance);
GParamSpec* param_spec_gxml_test (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_gxml_test (GValue* value, gpointer v_object);
void value_take_gxml_test (GValue* value, gpointer v_object);
gpointer value_get_gxml_test (const GValue* value);
GType gxml_test_get_type (void) G_GNUC_CONST;
GType serializable_gee_dual_key_map_test_get_type (void) G_GNUC_CONST;
enum  {
	SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_DUMMY_PROPERTY
};
void serializable_gee_dual_key_map_test_add_tests (void);
static void __lambda54_ (void);
static void ___lambda54__gtest_func (void);
static void __lambda55_ (void);
static void ___lambda55__gtest_func (void);
static void __lambda56_ (void);
static void ___lambda56__gtest_func (void);
static void __lambda57_ (void);
static void ___lambda57__gtest_func (void);
static void __lambda58_ (void);
GType book_store_get_type (void) G_GNUC_CONST;
BookStore* book_store_new (void);
BookStore* book_store_construct (GType object_type);
const gchar* book_store_get_name (BookStore* self);
GType book_array_get_type (void) G_GNUC_CONST;
BookArray* book_store_get_books (BookStore* self);
GType book_get_type (void) G_GNUC_CONST;
GType name_get_type (void) G_GNUC_CONST;
Name* book_get_name (Book* self);
gchar* name_get_name (Name* self);
const gchar* book_get_year (Book* self);
GType authors_get_type (void) G_GNUC_CONST;
Authors* book_get_authors (Book* self);
GType author_array_get_type (void) G_GNUC_CONST;
AuthorArray* authors_get_array (Authors* self);
GType inventory_dual_key_map_get_type (void) G_GNUC_CONST;
InventoryDualKeyMap* book_get_inventory_registers (Book* self);
GType inventory_get_type (void) G_GNUC_CONST;
gint inventory_get_number (Inventory* self);
const gchar* inventory_get_inventory (Inventory* self);
gint inventory_get_row (Inventory* self);
static void ___lambda58__gtest_func (void);
static void __lambda59_ (void);
static GType serializable_gee_dual_key_map_test_big_bag_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeDualKeyMapTestBigBag* serializable_gee_dual_key_map_test_big_bag_new (void);
static SerializableGeeDualKeyMapTestBigBag* serializable_gee_dual_key_map_test_big_bag_construct (GType object_type);
GType serializable_gee_dual_key_map_test_small_bag_dual_map_get_type (void) G_GNUC_CONST;
static SerializableGeeDualKeyMapTestSmallBagDualMap* serializable_gee_dual_key_map_test_big_bag_get_bags (SerializableGeeDualKeyMapTestBigBag* self);
static GType serializable_gee_dual_key_map_test_small_bag_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeDualKeyMapTestSmallBag* serializable_gee_dual_key_map_test_small_bag_new (void);
static SerializableGeeDualKeyMapTestSmallBag* serializable_gee_dual_key_map_test_small_bag_construct (GType object_type);
static void serializable_gee_dual_key_map_test_small_bag_set_name (SerializableGeeDualKeyMapTestSmallBag* self, const gchar* value);
GType serializable_gee_dual_key_map_test_ball_dual_map_get_type (void) G_GNUC_CONST;
static SerializableGeeDualKeyMapTestBallDualMap* serializable_gee_dual_key_map_test_small_bag_get_balls (SerializableGeeDualKeyMapTestSmallBag* self);
static GType serializable_gee_dual_key_map_test_ball_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeDualKeyMapTestBall* serializable_gee_dual_key_map_test_ball_new (void);
static SerializableGeeDualKeyMapTestBall* serializable_gee_dual_key_map_test_ball_construct (GType object_type);
static void serializable_gee_dual_key_map_test_ball_set_name (SerializableGeeDualKeyMapTestBall* self, const gchar* value);
static SerializableGeeDualKeyMapTestBallFill* serializable_gee_dual_key_map_test_ball_fill_new (void);
static SerializableGeeDualKeyMapTestBallFill* serializable_gee_dual_key_map_test_ball_fill_construct (GType object_type);
static GType serializable_gee_dual_key_map_test_ball_fill_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static void serializable_gee_dual_key_map_test_ball_set_ballfill (SerializableGeeDualKeyMapTestBall* self, SerializableGeeDualKeyMapTestBallFill* value);
static SerializableGeeDualKeyMapTestBallFill* serializable_gee_dual_key_map_test_ball_get_ballfill (SerializableGeeDualKeyMapTestBall* self);
static void serializable_gee_dual_key_map_test_ball_fill_set_text (SerializableGeeDualKeyMapTestBallFill* self, const gchar* txt);
static const gchar* serializable_gee_dual_key_map_test_ball_get_size (SerializableGeeDualKeyMapTestBall* self);
static const gchar* serializable_gee_dual_key_map_test_ball_get_name (SerializableGeeDualKeyMapTestBall* self);
static const gchar* serializable_gee_dual_key_map_test_small_bag_get_cash (SerializableGeeDualKeyMapTestSmallBag* self);
static const gchar* serializable_gee_dual_key_map_test_small_bag_get_name (SerializableGeeDualKeyMapTestSmallBag* self);
static void ___lambda59__gtest_func (void);
SerializableGeeDualKeyMapTest* serializable_gee_dual_key_map_test_new (void);
SerializableGeeDualKeyMapTest* serializable_gee_dual_key_map_test_construct (GType object_type);
GXmlTest* gxml_test_new (void);
GXmlTest* gxml_test_construct (GType object_type);
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_FILL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL_FILL, SerializableGeeDualKeyMapTestBallFillPrivate))
enum  {
	SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_FILL_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_FILL_NAME
};
static gchar* serializable_gee_dual_key_map_test_ball_fill_real_to_string (GXmlSerializableObjectModel* base);
static const gchar* serializable_gee_dual_key_map_test_ball_fill_get_name (SerializableGeeDualKeyMapTestBallFill* self);
static gchar* serializable_gee_dual_key_map_test_ball_fill_real_node_name (GXmlSerializableObjectModel* base);
static gboolean serializable_gee_dual_key_map_test_ball_fill_real_serialize_use_xml_node_value (GXmlSerializableObjectModel* base);
static void serializable_gee_dual_key_map_test_ball_fill_set_name (SerializableGeeDualKeyMapTestBallFill* self, const gchar* value);
static void serializable_gee_dual_key_map_test_ball_fill_finalize (GObject* obj);
static void _vala_serializable_gee_dual_key_map_test_ball_fill_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_dual_key_map_test_ball_fill_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL, SerializableGeeDualKeyMapTestBallPrivate))
enum  {
	SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_NAME,
	SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_SIZE,
	SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_BALLFILL
};
static gchar* serializable_gee_dual_key_map_test_ball_real_to_string (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_dual_key_map_test_ball_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_dual_key_map_test_ball_real_get_map_primary_key (GXmlSerializableMapDualKey* base);
static gchar* serializable_gee_dual_key_map_test_ball_real_get_map_secondary_key (GXmlSerializableMapDualKey* base);
static void serializable_gee_dual_key_map_test_ball_set_size (SerializableGeeDualKeyMapTestBall* self, const gchar* value);
enum  {
	SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_DUAL_MAP_DUMMY_PROPERTY
};
static gboolean serializable_gee_dual_key_map_test_ball_dual_map_real_deserialize_proceed (GXmlSerializableDualKeyMap* base);
static SerializableGeeDualKeyMapTestBallDualMap* serializable_gee_dual_key_map_test_ball_dual_map_new (void);
static SerializableGeeDualKeyMapTestBallDualMap* serializable_gee_dual_key_map_test_ball_dual_map_construct (GType object_type);
static void serializable_gee_dual_key_map_test_ball_finalize (GObject* obj);
static void _vala_serializable_gee_dual_key_map_test_ball_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_dual_key_map_test_ball_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_SMALL_BAG, SerializableGeeDualKeyMapTestSmallBagPrivate))
enum  {
	SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_NAME,
	SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_CASH,
	SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_BALLS
};
static gchar* serializable_gee_dual_key_map_test_small_bag_real_to_string (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_dual_key_map_test_small_bag_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_dual_key_map_test_small_bag_real_get_map_primary_key (GXmlSerializableMapDualKey* base);
static gchar* serializable_gee_dual_key_map_test_small_bag_real_get_map_secondary_key (GXmlSerializableMapDualKey* base);
static void serializable_gee_dual_key_map_test_small_bag_set_cash (SerializableGeeDualKeyMapTestSmallBag* self, const gchar* value);
static void serializable_gee_dual_key_map_test_small_bag_set_balls (SerializableGeeDualKeyMapTestSmallBag* self, SerializableGeeDualKeyMapTestBallDualMap* value);
enum  {
	SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_DUAL_MAP_DUMMY_PROPERTY
};
static gboolean serializable_gee_dual_key_map_test_small_bag_dual_map_real_deserialize_proceed (GXmlSerializableDualKeyMap* base);
static SerializableGeeDualKeyMapTestSmallBagDualMap* serializable_gee_dual_key_map_test_small_bag_dual_map_new (void);
static SerializableGeeDualKeyMapTestSmallBagDualMap* serializable_gee_dual_key_map_test_small_bag_dual_map_construct (GType object_type);
static void serializable_gee_dual_key_map_test_small_bag_finalize (GObject* obj);
static void _vala_serializable_gee_dual_key_map_test_small_bag_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_dual_key_map_test_small_bag_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BIG_BAG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BIG_BAG, SerializableGeeDualKeyMapTestBigBagPrivate))
enum  {
	SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BIG_BAG_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BIG_BAG_NAME,
	SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BIG_BAG_BAGS
};
static gchar* serializable_gee_dual_key_map_test_big_bag_real_to_string (GXmlSerializableObjectModel* base);
static const gchar* serializable_gee_dual_key_map_test_big_bag_get_name (SerializableGeeDualKeyMapTestBigBag* self);
static gchar* serializable_gee_dual_key_map_test_big_bag_real_node_name (GXmlSerializableObjectModel* base);
static void serializable_gee_dual_key_map_test_big_bag_set_name (SerializableGeeDualKeyMapTestBigBag* self, const gchar* value);
static void serializable_gee_dual_key_map_test_big_bag_set_bags (SerializableGeeDualKeyMapTestBigBag* self, SerializableGeeDualKeyMapTestSmallBagDualMap* value);
static void serializable_gee_dual_key_map_test_big_bag_finalize (GObject* obj);
static void _vala_serializable_gee_dual_key_map_test_big_bag_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_dual_key_map_test_big_bag_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);


static gchar* spaces_real_get_map_primary_key (GXmlSerializableMapDualKey* base) {
	Spaces * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (Spaces*) base;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_owner;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp1_;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 506 "SerializableGeeDualKeyMapTest.c"
}


static gchar* spaces_real_get_map_secondary_key (GXmlSerializableMapDualKey* base) {
	Spaces * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (Spaces*) base;
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp1_;
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 525 "SerializableGeeDualKeyMapTest.c"
}


Spaces* spaces_construct_full (GType object_type, const gchar* owner, const gchar* name) {
	Spaces * self = NULL;
	const gchar* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_val_if_fail (owner != NULL, NULL);
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (Spaces*) gxml_serializable_object_model_construct (object_type);
#line 37 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = name;
#line 37 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	spaces_set_name (self, _tmp0_);
#line 38 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = owner;
#line 38 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	spaces_set_owner (self, _tmp1_);
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return self;
#line 549 "SerializableGeeDualKeyMapTest.c"
}


Spaces* spaces_new_full (const gchar* owner, const gchar* name) {
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return spaces_construct_full (TYPE_SPACES, owner, name);
#line 556 "SerializableGeeDualKeyMapTest.c"
}


static gchar* spaces_real_to_string (GXmlSerializableObjectModel* base) {
	Spaces * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (Spaces*) base;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp1_;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 575 "SerializableGeeDualKeyMapTest.c"
}


Spaces* spaces_construct (GType object_type) {
	Spaces * self = NULL;
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (Spaces*) gxml_serializable_object_model_construct (object_type);
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return self;
#line 585 "SerializableGeeDualKeyMapTest.c"
}


Spaces* spaces_new (void) {
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return spaces_construct (TYPE_SPACES);
#line 592 "SerializableGeeDualKeyMapTest.c"
}


const gchar* spaces_get_name (Spaces* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp0_;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 607 "SerializableGeeDualKeyMapTest.c"
}


void spaces_set_name (Spaces* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_if_fail (self != NULL);
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = value;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_free0 (self->priv->_name);
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv->_name = _tmp1_;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 626 "SerializableGeeDualKeyMapTest.c"
}


const gchar* spaces_get_owner (Spaces* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_owner;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp0_;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 641 "SerializableGeeDualKeyMapTest.c"
}


void spaces_set_owner (Spaces* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_if_fail (self != NULL);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = value;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_free0 (self->priv->_owner);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv->_owner = _tmp1_;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_notify ((GObject *) self, "owner");
#line 660 "SerializableGeeDualKeyMapTest.c"
}


static void spaces_class_init (SpacesClass * klass) {
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	spaces_parent_class = g_type_class_peek_parent (klass);
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_type_class_add_private (klass, sizeof (SpacesPrivate));
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) spaces_real_to_string;
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_spaces_get_property;
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_spaces_set_property;
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	G_OBJECT_CLASS (klass)->finalize = spaces_finalize;
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SPACES_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SPACES_OWNER, g_param_spec_string ("owner", "owner", "owner", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 681 "SerializableGeeDualKeyMapTest.c"
}


static void spaces_gxml_serializable_map_dual_key_interface_init (GXmlSerializableMapDualKeyIface * iface) {
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	spaces_gxml_serializable_map_dual_key_parent_iface = g_type_interface_peek_parent (iface);
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	iface->get_map_primary_key = (gpointer (*)(GXmlSerializableMapDualKey*)) spaces_real_get_map_primary_key;
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	iface->get_map_secondary_key = (gpointer (*)(GXmlSerializableMapDualKey*)) spaces_real_get_map_secondary_key;
#line 692 "SerializableGeeDualKeyMapTest.c"
}


static void spaces_instance_init (Spaces * self) {
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv = SPACES_GET_PRIVATE (self);
#line 699 "SerializableGeeDualKeyMapTest.c"
}


static void spaces_finalize (GObject* obj) {
	Spaces * self;
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_SPACES, Spaces);
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_free0 (self->priv->_name);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_free0 (self->priv->_owner);
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	G_OBJECT_CLASS (spaces_parent_class)->finalize (obj);
#line 713 "SerializableGeeDualKeyMapTest.c"
}


/**
 *
 *  GXml.Serializable.BasicTypeTest
 *
 *  Authors:
 *
 *       Daniel Espinosa <esodan@gmail.com>
 *
 *
 *  Copyright (c) 2013-2015 Daniel Espinosa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
GType spaces_get_type (void) {
	static volatile gsize spaces_type_id__volatile = 0;
	if (g_once_init_enter (&spaces_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SpacesClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) spaces_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Spaces), 0, (GInstanceInitFunc) spaces_instance_init, NULL };
		static const GInterfaceInfo gxml_serializable_map_dual_key_info = { (GInterfaceInitFunc) spaces_gxml_serializable_map_dual_key_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType spaces_type_id;
		spaces_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "Spaces", &g_define_type_info, 0);
		g_type_add_interface_static (spaces_type_id, GXML_TYPE_SERIALIZABLE_MAP_DUAL_KEY, &gxml_serializable_map_dual_key_info);
		g_once_init_leave (&spaces_type_id__volatile, spaces_type_id);
	}
	return spaces_type_id__volatile;
}


static void _vala_spaces_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	Spaces * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_SPACES, Spaces);
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	switch (property_id) {
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SPACES_NAME:
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_value_set_string (value, spaces_get_name (self));
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SPACES_OWNER:
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_value_set_string (value, spaces_get_owner (self));
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 772 "SerializableGeeDualKeyMapTest.c"
		default:
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 778 "SerializableGeeDualKeyMapTest.c"
	}
}


static void _vala_spaces_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	Spaces * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_SPACES, Spaces);
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	switch (property_id) {
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SPACES_NAME:
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		spaces_set_name (self, g_value_get_string (value));
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SPACES_OWNER:
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		spaces_set_owner (self, g_value_get_string (value));
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 800 "SerializableGeeDualKeyMapTest.c"
		default:
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 806 "SerializableGeeDualKeyMapTest.c"
	}
}


static void __lambda54_ (void) {
	GXmlSerializableDualKeyMap* c = NULL;
	GXmlSerializableDualKeyMap* _tmp0_ = NULL;
	Spaces* o1 = NULL;
	Spaces* _tmp1_ = NULL;
	Spaces* o2 = NULL;
	Spaces* _tmp2_ = NULL;
	Spaces* o3 = NULL;
	Spaces* _tmp3_ = NULL;
	Spaces* o4 = NULL;
	Spaces* _tmp4_ = NULL;
	const gchar* _tmp5_ = NULL;
	const gchar* _tmp6_ = NULL;
	const gchar* _tmp7_ = NULL;
	const gchar* _tmp8_ = NULL;
	const gchar* _tmp9_ = NULL;
	const gchar* _tmp10_ = NULL;
	const gchar* _tmp11_ = NULL;
	const gchar* _tmp12_ = NULL;
	const gchar* _tmp13_ = NULL;
	const gchar* _tmp14_ = NULL;
	const gchar* _tmp15_ = NULL;
	const gchar* _tmp16_ = NULL;
	const gchar* _tmp17_ = NULL;
	const gchar* _tmp18_ = NULL;
	const gchar* _tmp19_ = NULL;
	const gchar* _tmp20_ = NULL;
	gint _tmp21_ = 0;
	gint _tmp22_ = 0;
	gboolean found1 = FALSE;
	gboolean found2 = FALSE;
	gboolean found3 = FALSE;
	gboolean found4 = FALSE;
	gboolean _tmp66_ = FALSE;
	gboolean _tmp68_ = FALSE;
	gboolean _tmp70_ = FALSE;
	gboolean _tmp72_ = FALSE;
	gboolean _tmp74_ = FALSE;
	gboolean _tmp97_ = FALSE;
	gboolean _tmp99_ = FALSE;
	gboolean _tmp101_ = FALSE;
	gboolean _tmp126_ = FALSE;
	gboolean _tmp128_ = FALSE;
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = gxml_serializable_dual_key_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, TYPE_SPACES, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	c = _tmp0_;
#line 96 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = spaces_new_full ("Floor", "Big");
#line 96 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	o1 = _tmp1_;
#line 97 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp2_ = spaces_new_full ("Wall", "Small");
#line 97 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	o2 = _tmp2_;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp3_ = spaces_new_full ("Floor", "Bigger");
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	o3 = _tmp3_;
#line 99 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp4_ = spaces_new_full ("Wall", "Smallest");
#line 99 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	o4 = _tmp4_;
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp5_ = spaces_get_owner (o1);
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp6_ = _tmp5_;
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp7_ = spaces_get_name (o1);
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp8_ = _tmp7_;
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	gxml_serializable_dual_key_map_set (c, _tmp6_, _tmp8_, o1);
#line 101 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp9_ = spaces_get_owner (o2);
#line 101 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp10_ = _tmp9_;
#line 101 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp11_ = spaces_get_name (o2);
#line 101 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp12_ = _tmp11_;
#line 101 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	gxml_serializable_dual_key_map_set (c, _tmp10_, _tmp12_, o2);
#line 102 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp13_ = spaces_get_owner (o3);
#line 102 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp14_ = _tmp13_;
#line 102 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp15_ = spaces_get_name (o3);
#line 102 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp16_ = _tmp15_;
#line 102 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	gxml_serializable_dual_key_map_set (c, _tmp14_, _tmp16_, o3);
#line 103 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp17_ = spaces_get_owner (o4);
#line 103 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp18_ = _tmp17_;
#line 103 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp19_ = spaces_get_name (o4);
#line 103 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp20_ = _tmp19_;
#line 103 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	gxml_serializable_dual_key_map_set (c, _tmp18_, _tmp20_, o4);
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp21_ = gxml_serializable_dual_key_map_get_size (c);
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp22_ = _tmp21_;
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	if (_tmp22_ != 4) {
#line 920 "SerializableGeeDualKeyMapTest.c"
		FILE* _tmp23_ = NULL;
		gint _tmp24_ = 0;
		gint _tmp25_ = 0;
		gchar* _tmp26_ = NULL;
		gchar* _tmp27_ = NULL;
		gchar* _tmp28_ = NULL;
		gchar* _tmp29_ = NULL;
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp23_ = stdout;
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp24_ = gxml_serializable_dual_key_map_get_size (c);
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp25_ = _tmp24_;
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp26_ = g_strdup_printf ("%i", _tmp25_);
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp27_ = _tmp26_;
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp28_ = g_strconcat ("SIZE: ", _tmp27_, "\n", NULL);
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp29_ = _tmp28_;
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		fprintf (_tmp23_, "%s", _tmp29_);
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_free0 (_tmp29_);
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_free0 (_tmp27_);
#line 106 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_assert_not_reached ();
#line 950 "SerializableGeeDualKeyMapTest.c"
	}
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	found1 = FALSE;
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	found2 = FALSE;
#line 110 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	found3 = FALSE;
#line 111 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	found4 = FALSE;
#line 960 "SerializableGeeDualKeyMapTest.c"
	{
		GeeIterator* _s_it = NULL;
		GeeCollection* _tmp30_ = NULL;
		GeeCollection* _tmp31_ = NULL;
		GeeIterator* _tmp32_ = NULL;
		GeeIterator* _tmp33_ = NULL;
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp30_ = gxml_serializable_dual_key_map_values (c);
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp31_ = _tmp30_;
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp32_ = gee_iterable_iterator ((GeeIterable*) _tmp31_);
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp33_ = _tmp32_;
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp31_);
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_s_it = _tmp33_;
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		while (TRUE) {
#line 981 "SerializableGeeDualKeyMapTest.c"
			GeeIterator* _tmp34_ = NULL;
			gboolean _tmp35_ = FALSE;
			Spaces* s = NULL;
			GeeIterator* _tmp36_ = NULL;
			gpointer _tmp37_ = NULL;
			gboolean _tmp38_ = FALSE;
			Spaces* _tmp39_ = NULL;
			const gchar* _tmp40_ = NULL;
			const gchar* _tmp41_ = NULL;
			gboolean _tmp45_ = FALSE;
			Spaces* _tmp46_ = NULL;
			const gchar* _tmp47_ = NULL;
			const gchar* _tmp48_ = NULL;
			gboolean _tmp52_ = FALSE;
			Spaces* _tmp53_ = NULL;
			const gchar* _tmp54_ = NULL;
			const gchar* _tmp55_ = NULL;
			gboolean _tmp59_ = FALSE;
			Spaces* _tmp60_ = NULL;
			const gchar* _tmp61_ = NULL;
			const gchar* _tmp62_ = NULL;
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp34_ = _s_it;
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp35_ = gee_iterator_next (_tmp34_);
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			if (!_tmp35_) {
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				break;
#line 1011 "SerializableGeeDualKeyMapTest.c"
			}
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp36_ = _s_it;
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp37_ = gee_iterator_get (_tmp36_);
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			s = (Spaces*) _tmp37_;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp39_ = s;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp40_ = spaces_get_owner (_tmp39_);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp41_ = _tmp40_;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			if (g_strcmp0 (_tmp41_, "Floor") == 0) {
#line 1027 "SerializableGeeDualKeyMapTest.c"
				Spaces* _tmp42_ = NULL;
				const gchar* _tmp43_ = NULL;
				const gchar* _tmp44_ = NULL;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp42_ = s;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp43_ = spaces_get_name (_tmp42_);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp44_ = _tmp43_;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp38_ = g_strcmp0 (_tmp44_, "Big") == 0;
#line 1039 "SerializableGeeDualKeyMapTest.c"
			} else {
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp38_ = FALSE;
#line 1043 "SerializableGeeDualKeyMapTest.c"
			}
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			if (_tmp38_) {
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				found1 = TRUE;
#line 1049 "SerializableGeeDualKeyMapTest.c"
			}
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp46_ = s;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp47_ = spaces_get_owner (_tmp46_);
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp48_ = _tmp47_;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			if (g_strcmp0 (_tmp48_, "Wall") == 0) {
#line 1059 "SerializableGeeDualKeyMapTest.c"
				Spaces* _tmp49_ = NULL;
				const gchar* _tmp50_ = NULL;
				const gchar* _tmp51_ = NULL;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp49_ = s;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp50_ = spaces_get_name (_tmp49_);
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp51_ = _tmp50_;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp45_ = g_strcmp0 (_tmp51_, "Small") == 0;
#line 1071 "SerializableGeeDualKeyMapTest.c"
			} else {
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp45_ = FALSE;
#line 1075 "SerializableGeeDualKeyMapTest.c"
			}
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			if (_tmp45_) {
#line 116 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				found2 = TRUE;
#line 1081 "SerializableGeeDualKeyMapTest.c"
			}
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp53_ = s;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp54_ = spaces_get_owner (_tmp53_);
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp55_ = _tmp54_;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			if (g_strcmp0 (_tmp55_, "Floor") == 0) {
#line 1091 "SerializableGeeDualKeyMapTest.c"
				Spaces* _tmp56_ = NULL;
				const gchar* _tmp57_ = NULL;
				const gchar* _tmp58_ = NULL;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp56_ = s;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp57_ = spaces_get_name (_tmp56_);
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp58_ = _tmp57_;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp52_ = g_strcmp0 (_tmp58_, "Bigger") == 0;
#line 1103 "SerializableGeeDualKeyMapTest.c"
			} else {
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp52_ = FALSE;
#line 1107 "SerializableGeeDualKeyMapTest.c"
			}
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			if (_tmp52_) {
#line 118 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				found3 = TRUE;
#line 1113 "SerializableGeeDualKeyMapTest.c"
			}
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp60_ = s;
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp61_ = spaces_get_owner (_tmp60_);
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp62_ = _tmp61_;
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			if (g_strcmp0 (_tmp62_, "Wall") == 0) {
#line 1123 "SerializableGeeDualKeyMapTest.c"
				Spaces* _tmp63_ = NULL;
				const gchar* _tmp64_ = NULL;
				const gchar* _tmp65_ = NULL;
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp63_ = s;
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp64_ = spaces_get_name (_tmp63_);
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp65_ = _tmp64_;
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp59_ = g_strcmp0 (_tmp65_, "Smallest") == 0;
#line 1135 "SerializableGeeDualKeyMapTest.c"
			} else {
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp59_ = FALSE;
#line 1139 "SerializableGeeDualKeyMapTest.c"
			}
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			if (_tmp59_) {
#line 120 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				found4 = TRUE;
#line 1145 "SerializableGeeDualKeyMapTest.c"
			}
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (s);
#line 1149 "SerializableGeeDualKeyMapTest.c"
		}
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_s_it);
#line 1153 "SerializableGeeDualKeyMapTest.c"
	}
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp66_ = found1;
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	if (!_tmp66_) {
#line 1159 "SerializableGeeDualKeyMapTest.c"
		FILE* _tmp67_ = NULL;
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp67_ = stdout;
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		fprintf (_tmp67_, "%s", "Not found 'Floor' & 'Big':\n");
#line 124 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_assert_not_reached ();
#line 1167 "SerializableGeeDualKeyMapTest.c"
	}
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp68_ = found2;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	if (!_tmp68_) {
#line 1173 "SerializableGeeDualKeyMapTest.c"
		FILE* _tmp69_ = NULL;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp69_ = stdout;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		fprintf (_tmp69_, "%s", "Not found 'Wall' & 'Small':\n");
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_assert_not_reached ();
#line 1181 "SerializableGeeDualKeyMapTest.c"
	}
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp70_ = found3;
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	if (!_tmp70_) {
#line 1187 "SerializableGeeDualKeyMapTest.c"
		FILE* _tmp71_ = NULL;
#line 131 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp71_ = stdout;
#line 131 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		fprintf (_tmp71_, "%s", "Not found 'Floor' & 'Bigger':\n");
#line 132 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_assert_not_reached ();
#line 1195 "SerializableGeeDualKeyMapTest.c"
	}
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp72_ = found4;
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	if (!_tmp72_) {
#line 1201 "SerializableGeeDualKeyMapTest.c"
		FILE* _tmp73_ = NULL;
#line 135 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp73_ = stdout;
#line 135 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		fprintf (_tmp73_, "%s", "Not found 'Wall' & 'Smallest':\n");
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_assert_not_reached ();
#line 1209 "SerializableGeeDualKeyMapTest.c"
	}
#line 138 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	found3 = FALSE;
#line 138 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp74_ = found3;
#line 138 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	found1 = _tmp74_;
#line 1217 "SerializableGeeDualKeyMapTest.c"
	{
		GeeIterator* _s_it = NULL;
		GeeCollection* _tmp75_ = NULL;
		GeeCollection* _tmp76_ = NULL;
		GeeIterator* _tmp77_ = NULL;
		GeeIterator* _tmp78_ = NULL;
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp75_ = gxml_serializable_dual_key_map_values_for_key (c, "Floor");
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp76_ = _tmp75_;
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp77_ = gee_iterable_iterator ((GeeIterable*) _tmp76_);
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp78_ = _tmp77_;
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp76_);
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_s_it = _tmp78_;
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		while (TRUE) {
#line 1238 "SerializableGeeDualKeyMapTest.c"
			GeeIterator* _tmp79_ = NULL;
			gboolean _tmp80_ = FALSE;
			Spaces* s = NULL;
			GeeIterator* _tmp81_ = NULL;
			gpointer _tmp82_ = NULL;
			gboolean _tmp83_ = FALSE;
			Spaces* _tmp84_ = NULL;
			const gchar* _tmp85_ = NULL;
			const gchar* _tmp86_ = NULL;
			gboolean _tmp90_ = FALSE;
			Spaces* _tmp91_ = NULL;
			const gchar* _tmp92_ = NULL;
			const gchar* _tmp93_ = NULL;
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp79_ = _s_it;
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp80_ = gee_iterator_next (_tmp79_);
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			if (!_tmp80_) {
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				break;
#line 1260 "SerializableGeeDualKeyMapTest.c"
			}
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp81_ = _s_it;
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp82_ = gee_iterator_get (_tmp81_);
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			s = (Spaces*) _tmp82_;
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp84_ = s;
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp85_ = spaces_get_owner (_tmp84_);
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp86_ = _tmp85_;
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			if (g_strcmp0 (_tmp86_, "Floor") == 0) {
#line 1276 "SerializableGeeDualKeyMapTest.c"
				Spaces* _tmp87_ = NULL;
				const gchar* _tmp88_ = NULL;
				const gchar* _tmp89_ = NULL;
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp87_ = s;
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp88_ = spaces_get_name (_tmp87_);
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp89_ = _tmp88_;
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp83_ = g_strcmp0 (_tmp89_, "Big") == 0;
#line 1288 "SerializableGeeDualKeyMapTest.c"
			} else {
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp83_ = FALSE;
#line 1292 "SerializableGeeDualKeyMapTest.c"
			}
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			if (_tmp83_) {
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				found1 = TRUE;
#line 1298 "SerializableGeeDualKeyMapTest.c"
			}
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp91_ = s;
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp92_ = spaces_get_owner (_tmp91_);
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp93_ = _tmp92_;
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			if (g_strcmp0 (_tmp93_, "Floor") == 0) {
#line 1308 "SerializableGeeDualKeyMapTest.c"
				Spaces* _tmp94_ = NULL;
				const gchar* _tmp95_ = NULL;
				const gchar* _tmp96_ = NULL;
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp94_ = s;
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp95_ = spaces_get_name (_tmp94_);
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp96_ = _tmp95_;
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp90_ = g_strcmp0 (_tmp96_, "Bigger") == 0;
#line 1320 "SerializableGeeDualKeyMapTest.c"
			} else {
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp90_ = FALSE;
#line 1324 "SerializableGeeDualKeyMapTest.c"
			}
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			if (_tmp90_) {
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				found3 = TRUE;
#line 1330 "SerializableGeeDualKeyMapTest.c"
			}
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (s);
#line 1334 "SerializableGeeDualKeyMapTest.c"
		}
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_s_it);
#line 1338 "SerializableGeeDualKeyMapTest.c"
	}
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp97_ = found1;
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	if (!_tmp97_) {
#line 1344 "SerializableGeeDualKeyMapTest.c"
		FILE* _tmp98_ = NULL;
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp98_ = stdout;
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		fprintf (_tmp98_, "%s", "Not found 'Floor' & 'Big':\n");
#line 147 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_assert_not_reached ();
#line 1352 "SerializableGeeDualKeyMapTest.c"
	}
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp99_ = found3;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	if (!_tmp99_) {
#line 1358 "SerializableGeeDualKeyMapTest.c"
		FILE* _tmp100_ = NULL;
#line 150 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp100_ = stdout;
#line 150 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		fprintf (_tmp100_, "%s", "Not found 'Floor' & 'Bigger':\n");
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_assert_not_reached ();
#line 1366 "SerializableGeeDualKeyMapTest.c"
	}
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	found4 = TRUE;
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp101_ = found4;
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	found2 = _tmp101_;
#line 1374 "SerializableGeeDualKeyMapTest.c"
	{
		GeeIterator* _k2_it = NULL;
		GeeCollection* _tmp102_ = NULL;
		GeeCollection* _tmp103_ = NULL;
		GeeIterator* _tmp104_ = NULL;
		GeeIterator* _tmp105_ = NULL;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp102_ = gxml_serializable_dual_key_map_secondary_keys (c, "Wall");
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp103_ = _tmp102_;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp104_ = gee_iterable_iterator ((GeeIterable*) _tmp103_);
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp105_ = _tmp104_;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp103_);
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_k2_it = _tmp105_;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		while (TRUE) {
#line 1395 "SerializableGeeDualKeyMapTest.c"
			GeeIterator* _tmp106_ = NULL;
			gboolean _tmp107_ = FALSE;
			gchar* k2 = NULL;
			GeeIterator* _tmp108_ = NULL;
			gpointer _tmp109_ = NULL;
			Spaces* s = NULL;
			const gchar* _tmp110_ = NULL;
			gpointer _tmp111_ = NULL;
			gboolean _tmp112_ = FALSE;
			Spaces* _tmp113_ = NULL;
			const gchar* _tmp114_ = NULL;
			const gchar* _tmp115_ = NULL;
			gboolean _tmp119_ = FALSE;
			Spaces* _tmp120_ = NULL;
			const gchar* _tmp121_ = NULL;
			const gchar* _tmp122_ = NULL;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp106_ = _k2_it;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp107_ = gee_iterator_next (_tmp106_);
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			if (!_tmp107_) {
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				break;
#line 1420 "SerializableGeeDualKeyMapTest.c"
			}
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp108_ = _k2_it;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp109_ = gee_iterator_get (_tmp108_);
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			k2 = (gchar*) _tmp109_;
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp110_ = k2;
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp111_ = gxml_serializable_dual_key_map_get (c, "Wall", _tmp110_);
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			s = (Spaces*) _tmp111_;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp113_ = s;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp114_ = spaces_get_owner (_tmp113_);
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp115_ = _tmp114_;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			if (g_strcmp0 (_tmp115_, "Wall") == 0) {
#line 1442 "SerializableGeeDualKeyMapTest.c"
				Spaces* _tmp116_ = NULL;
				const gchar* _tmp117_ = NULL;
				const gchar* _tmp118_ = NULL;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp116_ = s;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp117_ = spaces_get_name (_tmp116_);
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp118_ = _tmp117_;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp112_ = g_strcmp0 (_tmp118_, "Small") == 0;
#line 1454 "SerializableGeeDualKeyMapTest.c"
			} else {
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp112_ = FALSE;
#line 1458 "SerializableGeeDualKeyMapTest.c"
			}
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			if (_tmp112_) {
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				found2 = TRUE;
#line 1464 "SerializableGeeDualKeyMapTest.c"
			}
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp120_ = s;
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp121_ = spaces_get_owner (_tmp120_);
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp122_ = _tmp121_;
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			if (g_strcmp0 (_tmp122_, "Wall") == 0) {
#line 1474 "SerializableGeeDualKeyMapTest.c"
				Spaces* _tmp123_ = NULL;
				const gchar* _tmp124_ = NULL;
				const gchar* _tmp125_ = NULL;
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp123_ = s;
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp124_ = spaces_get_name (_tmp123_);
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp125_ = _tmp124_;
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp119_ = g_strcmp0 (_tmp125_, "Smallest") == 0;
#line 1486 "SerializableGeeDualKeyMapTest.c"
			} else {
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp119_ = FALSE;
#line 1490 "SerializableGeeDualKeyMapTest.c"
			}
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			if (_tmp119_) {
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				found4 = TRUE;
#line 1496 "SerializableGeeDualKeyMapTest.c"
			}
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (s);
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (k2);
#line 1502 "SerializableGeeDualKeyMapTest.c"
		}
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_k2_it);
#line 1506 "SerializableGeeDualKeyMapTest.c"
	}
#line 161 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp126_ = found2;
#line 161 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	if (!_tmp126_) {
#line 1512 "SerializableGeeDualKeyMapTest.c"
		FILE* _tmp127_ = NULL;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp127_ = stdout;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		fprintf (_tmp127_, "%s", "Not found 'Wall' & 'Small':\n");
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_assert_not_reached ();
#line 1520 "SerializableGeeDualKeyMapTest.c"
	}
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp128_ = found4;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	if (!_tmp128_) {
#line 1526 "SerializableGeeDualKeyMapTest.c"
		FILE* _tmp129_ = NULL;
#line 166 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp129_ = stdout;
#line 166 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		fprintf (_tmp129_, "%s", "Not found 'Wall' & 'Smallest':\n");
#line 167 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_assert_not_reached ();
#line 1534 "SerializableGeeDualKeyMapTest.c"
	}
#line 94 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_object_unref0 (o4);
#line 94 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_object_unref0 (o3);
#line 94 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_object_unref0 (o2);
#line 94 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_object_unref0 (o1);
#line 94 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_object_unref0 (c);
#line 1546 "SerializableGeeDualKeyMapTest.c"
}


static void ___lambda54__gtest_func (void) {
#line 93 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	__lambda54_ ();
#line 1553 "SerializableGeeDualKeyMapTest.c"
}


static const gchar* string_to_string (const gchar* self) {
	const gchar* result = NULL;
#line 1412 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1413 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	result = self;
#line 1413 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	return result;
#line 1565 "SerializableGeeDualKeyMapTest.c"
}


static void __lambda55_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlSerializableDualKeyMap* c = NULL;
		GXmlSerializableDualKeyMap* _tmp0_ = NULL;
		Spaces* o1 = NULL;
		Spaces* _tmp1_ = NULL;
		Spaces* o2 = NULL;
		Spaces* _tmp2_ = NULL;
		Spaces* o3 = NULL;
		Spaces* _tmp3_ = NULL;
		Spaces* o4 = NULL;
		Spaces* _tmp4_ = NULL;
		GXmlSerializableDualKeyMap* _tmp5_ = NULL;
		Spaces* _tmp6_ = NULL;
		const gchar* _tmp7_ = NULL;
		const gchar* _tmp8_ = NULL;
		Spaces* _tmp9_ = NULL;
		const gchar* _tmp10_ = NULL;
		const gchar* _tmp11_ = NULL;
		Spaces* _tmp12_ = NULL;
		GXmlSerializableDualKeyMap* _tmp13_ = NULL;
		Spaces* _tmp14_ = NULL;
		const gchar* _tmp15_ = NULL;
		const gchar* _tmp16_ = NULL;
		Spaces* _tmp17_ = NULL;
		const gchar* _tmp18_ = NULL;
		const gchar* _tmp19_ = NULL;
		Spaces* _tmp20_ = NULL;
		GXmlSerializableDualKeyMap* _tmp21_ = NULL;
		Spaces* _tmp22_ = NULL;
		const gchar* _tmp23_ = NULL;
		const gchar* _tmp24_ = NULL;
		Spaces* _tmp25_ = NULL;
		const gchar* _tmp26_ = NULL;
		const gchar* _tmp27_ = NULL;
		Spaces* _tmp28_ = NULL;
		GXmlSerializableDualKeyMap* _tmp29_ = NULL;
		Spaces* _tmp30_ = NULL;
		const gchar* _tmp31_ = NULL;
		const gchar* _tmp32_ = NULL;
		Spaces* _tmp33_ = NULL;
		const gchar* _tmp34_ = NULL;
		const gchar* _tmp35_ = NULL;
		Spaces* _tmp36_ = NULL;
		GXmlTDocument* doc = NULL;
		GXmlTDocument* _tmp37_ = NULL;
		GXmlNode* root = NULL;
		GXmlTDocument* _tmp38_ = NULL;
		GXmlNode* _tmp39_ = NULL;
		GXmlTDocument* _tmp40_ = NULL;
		GeeBidirList* _tmp41_ = NULL;
		GeeBidirList* _tmp42_ = NULL;
		GeeBidirList* _tmp43_ = NULL;
		GXmlNode* _tmp44_ = NULL;
		GXmlSerializableDualKeyMap* _tmp45_ = NULL;
		GXmlNode* _tmp46_ = NULL;
		GXmlNode* _tmp47_ = NULL;
		GXmlNode* _tmp48_ = NULL;
		GXmlNode* _tmp49_ = NULL;
		GeeBidirList* _tmp50_ = NULL;
		GeeBidirList* _tmp51_ = NULL;
		GeeBidirList* _tmp52_ = NULL;
		gint _tmp53_ = 0;
		gint _tmp54_ = 0;
		gboolean found1 = FALSE;
		gboolean found2 = FALSE;
		gboolean found3 = FALSE;
		gboolean found4 = FALSE;
		gint nodes = 0;
		gint i = 0;
		gint _tmp130_ = 0;
		gint _tmp140_ = 0;
		gboolean _tmp150_ = FALSE;
		gboolean _tmp157_ = FALSE;
		gboolean _tmp164_ = FALSE;
		gboolean _tmp171_ = FALSE;
#line 173 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp0_ = gxml_serializable_dual_key_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, TYPE_SPACES, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 173 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		c = _tmp0_;
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp1_ = spaces_new_full ("Floor", "Big");
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		o1 = _tmp1_;
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp2_ = spaces_new_full ("Wall", "Small");
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		o2 = _tmp2_;
#line 176 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp3_ = spaces_new_full ("Floor", "Bigger");
#line 176 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		o3 = _tmp3_;
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp4_ = spaces_new_full ("Wall", "Smallest");
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		o4 = _tmp4_;
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp5_ = c;
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp6_ = o1;
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp7_ = spaces_get_owner (_tmp6_);
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp8_ = _tmp7_;
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp9_ = o1;
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp10_ = spaces_get_name (_tmp9_);
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp11_ = _tmp10_;
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp12_ = o1;
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		gxml_serializable_dual_key_map_set (_tmp5_, _tmp8_, _tmp11_, _tmp12_);
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp13_ = c;
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp14_ = o2;
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp15_ = spaces_get_owner (_tmp14_);
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp16_ = _tmp15_;
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp17_ = o2;
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp18_ = spaces_get_name (_tmp17_);
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp19_ = _tmp18_;
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp20_ = o2;
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		gxml_serializable_dual_key_map_set (_tmp13_, _tmp16_, _tmp19_, _tmp20_);
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp21_ = c;
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp22_ = o3;
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp23_ = spaces_get_owner (_tmp22_);
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp24_ = _tmp23_;
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp25_ = o3;
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp26_ = spaces_get_name (_tmp25_);
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp27_ = _tmp26_;
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp28_ = o3;
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		gxml_serializable_dual_key_map_set (_tmp21_, _tmp24_, _tmp27_, _tmp28_);
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp29_ = c;
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp30_ = o4;
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp31_ = spaces_get_owner (_tmp30_);
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp32_ = _tmp31_;
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp33_ = o4;
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp34_ = spaces_get_name (_tmp33_);
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp35_ = _tmp34_;
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp36_ = o4;
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		gxml_serializable_dual_key_map_set (_tmp29_, _tmp32_, _tmp35_, _tmp36_);
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp37_ = gxml_tdocument_new ();
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		doc = _tmp37_;
#line 183 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp38_ = doc;
#line 183 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp39_ = gxml_document_create_element ((GXmlDocument*) _tmp38_, "root", &_inner_error_);
#line 183 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		root = _tmp39_;
#line 183 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 183 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (doc);
#line 183 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (o4);
#line 183 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (o3);
#line 183 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (o2);
#line 183 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (o1);
#line 183 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (c);
#line 1762 "SerializableGeeDualKeyMapTest.c"
			goto __catch77_g_error;
		}
#line 184 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp40_ = doc;
#line 184 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp41_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp40_);
#line 184 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp42_ = _tmp41_;
#line 184 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp43_ = _tmp42_;
#line 184 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp44_ = root;
#line 184 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		gee_collection_add ((GeeCollection*) _tmp43_, _tmp44_);
#line 184 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp43_);
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp45_ = c;
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp46_ = root;
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp47_ = gxml_serializable_dual_key_map_serialize (_tmp45_, _tmp46_, &_inner_error_);
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp48_ = _tmp47_;
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp48_);
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (root);
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (doc);
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (o4);
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (o3);
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (o2);
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (o1);
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (c);
#line 1805 "SerializableGeeDualKeyMapTest.c"
			goto __catch77_g_error;
		}
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp49_ = root;
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp50_ = gxml_node_get_children_nodes (_tmp49_);
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp51_ = _tmp50_;
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp52_ = _tmp51_;
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp53_ = gee_collection_get_size ((GeeCollection*) _tmp52_);
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp54_ = _tmp53_;
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp54_ == 4, "root.children_nodes.size == 4");
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp52_);
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		found1 = FALSE;
#line 188 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		found2 = FALSE;
#line 189 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		found3 = FALSE;
#line 190 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		found4 = FALSE;
#line 191 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		nodes = 0;
#line 192 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		i = 0;
#line 1836 "SerializableGeeDualKeyMapTest.c"
		{
			GeeBidirList* _n_list = NULL;
			GXmlNode* _tmp55_ = NULL;
			GeeBidirList* _tmp56_ = NULL;
			GeeBidirList* _tmp57_ = NULL;
			gint _n_size = 0;
			GeeBidirList* _tmp58_ = NULL;
			gint _tmp59_ = 0;
			gint _tmp60_ = 0;
			gint _n_index = 0;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp55_ = root;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp56_ = gxml_node_get_children_nodes (_tmp55_);
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp57_ = _tmp56_;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_n_list = _tmp57_;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp58_ = _n_list;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp59_ = gee_collection_get_size ((GeeCollection*) _tmp58_);
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp60_ = _tmp59_;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_n_size = _tmp60_;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_n_index = -1;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			while (TRUE) {
#line 1867 "SerializableGeeDualKeyMapTest.c"
				gint _tmp61_ = 0;
				gint _tmp62_ = 0;
				gint _tmp63_ = 0;
				GXmlNode* n = NULL;
				GeeBidirList* _tmp64_ = NULL;
				gint _tmp65_ = 0;
				gpointer _tmp66_ = NULL;
				gint _tmp67_ = 0;
				gboolean _tmp68_ = FALSE;
				GXmlNode* _tmp69_ = NULL;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp61_ = _n_index;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_n_index = _tmp61_ + 1;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp62_ = _n_index;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp63_ = _n_size;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				if (!(_tmp62_ < _tmp63_)) {
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					break;
#line 1890 "SerializableGeeDualKeyMapTest.c"
				}
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp64_ = _n_list;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp65_ = _n_index;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp66_ = gee_list_get ((GeeList*) _tmp64_, _tmp65_);
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				n = (GXmlNode*) _tmp66_;
#line 194 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp67_ = nodes;
#line 194 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				nodes = _tmp67_ + 1;
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp69_ = n;
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp69_, GXML_TYPE_ELEMENT)) {
#line 1908 "SerializableGeeDualKeyMapTest.c"
					GXmlNode* _tmp70_ = NULL;
					gchar* _tmp71_ = NULL;
					gchar* _tmp72_ = NULL;
					gchar* _tmp73_ = NULL;
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp70_ = n;
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp71_ = gxml_node_get_name (_tmp70_);
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp72_ = _tmp71_;
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp73_ = _tmp72_;
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp68_ = g_strcmp0 (_tmp73_, "spaces") == 0;
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_g_free0 (_tmp73_);
#line 1925 "SerializableGeeDualKeyMapTest.c"
				} else {
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp68_ = FALSE;
#line 1929 "SerializableGeeDualKeyMapTest.c"
				}
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				if (_tmp68_) {
#line 1933 "SerializableGeeDualKeyMapTest.c"
					gint _tmp74_ = 0;
					GXmlNode* name = NULL;
					GXmlNode* _tmp75_ = NULL;
					GeeMap* _tmp76_ = NULL;
					GeeMap* _tmp77_ = NULL;
					GeeMap* _tmp78_ = NULL;
					gpointer _tmp79_ = NULL;
					GXmlNode* _tmp80_ = NULL;
					GXmlNode* owner = NULL;
					GXmlNode* _tmp81_ = NULL;
					GeeMap* _tmp82_ = NULL;
					GeeMap* _tmp83_ = NULL;
					GeeMap* _tmp84_ = NULL;
					gpointer _tmp85_ = NULL;
					GXmlNode* _tmp86_ = NULL;
					gboolean _tmp87_ = FALSE;
					GXmlNode* _tmp88_ = NULL;
#line 196 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp74_ = i;
#line 196 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					i = _tmp74_ + 1;
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp75_ = n;
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp76_ = gxml_node_get_attrs (_tmp75_);
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp77_ = _tmp76_;
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp78_ = _tmp77_;
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp79_ = gee_map_get (_tmp78_, "name");
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp80_ = (GXmlNode*) _tmp79_;
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_g_object_unref0 (_tmp78_);
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					name = _tmp80_;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp81_ = n;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp82_ = gxml_node_get_attrs (_tmp81_);
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp83_ = _tmp82_;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp84_ = _tmp83_;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp85_ = gee_map_get (_tmp84_, "owner");
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp86_ = (GXmlNode*) _tmp85_;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_g_object_unref0 (_tmp84_);
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					owner = _tmp86_;
#line 199 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp88_ = name;
#line 199 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					if (_tmp88_ != NULL) {
#line 1991 "SerializableGeeDualKeyMapTest.c"
						GXmlNode* _tmp89_ = NULL;
#line 199 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp89_ = owner;
#line 199 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp87_ = _tmp89_ != NULL;
#line 1997 "SerializableGeeDualKeyMapTest.c"
					} else {
#line 199 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp87_ = FALSE;
#line 2001 "SerializableGeeDualKeyMapTest.c"
					}
#line 199 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					if (_tmp87_) {
#line 2005 "SerializableGeeDualKeyMapTest.c"
						gboolean _tmp90_ = FALSE;
						GXmlNode* _tmp91_ = NULL;
						gchar* _tmp92_ = NULL;
						gchar* _tmp93_ = NULL;
						gchar* _tmp94_ = NULL;
						gboolean _tmp95_ = FALSE;
						gboolean _tmp100_ = FALSE;
						GXmlNode* _tmp101_ = NULL;
						gchar* _tmp102_ = NULL;
						gchar* _tmp103_ = NULL;
						gchar* _tmp104_ = NULL;
						gboolean _tmp105_ = FALSE;
						gboolean _tmp110_ = FALSE;
						GXmlNode* _tmp111_ = NULL;
						gchar* _tmp112_ = NULL;
						gchar* _tmp113_ = NULL;
						gchar* _tmp114_ = NULL;
						gboolean _tmp115_ = FALSE;
						gboolean _tmp120_ = FALSE;
						GXmlNode* _tmp121_ = NULL;
						gchar* _tmp122_ = NULL;
						gchar* _tmp123_ = NULL;
						gchar* _tmp124_ = NULL;
						gboolean _tmp125_ = FALSE;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp91_ = name;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp92_ = gxml_node_get_value (_tmp91_);
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp93_ = _tmp92_;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp94_ = _tmp93_;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp95_ = g_strcmp0 (_tmp94_, "Big") == 0;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_g_free0 (_tmp94_);
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						if (_tmp95_) {
#line 2044 "SerializableGeeDualKeyMapTest.c"
							GXmlNode* _tmp96_ = NULL;
							gchar* _tmp97_ = NULL;
							gchar* _tmp98_ = NULL;
							gchar* _tmp99_ = NULL;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp96_ = owner;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp97_ = gxml_node_get_value (_tmp96_);
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp98_ = _tmp97_;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp99_ = _tmp98_;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp90_ = g_strcmp0 (_tmp99_, "Floor") == 0;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_g_free0 (_tmp99_);
#line 2061 "SerializableGeeDualKeyMapTest.c"
						} else {
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp90_ = FALSE;
#line 2065 "SerializableGeeDualKeyMapTest.c"
						}
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						if (_tmp90_) {
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							found1 = TRUE;
#line 2071 "SerializableGeeDualKeyMapTest.c"
						}
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp101_ = name;
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp102_ = gxml_node_get_value (_tmp101_);
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp103_ = _tmp102_;
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp104_ = _tmp103_;
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp105_ = g_strcmp0 (_tmp104_, "Small") == 0;
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_g_free0 (_tmp104_);
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						if (_tmp105_) {
#line 2087 "SerializableGeeDualKeyMapTest.c"
							GXmlNode* _tmp106_ = NULL;
							gchar* _tmp107_ = NULL;
							gchar* _tmp108_ = NULL;
							gchar* _tmp109_ = NULL;
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp106_ = owner;
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp107_ = gxml_node_get_value (_tmp106_);
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp108_ = _tmp107_;
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp109_ = _tmp108_;
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp100_ = g_strcmp0 (_tmp109_, "Wall") == 0;
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_g_free0 (_tmp109_);
#line 2104 "SerializableGeeDualKeyMapTest.c"
						} else {
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp100_ = FALSE;
#line 2108 "SerializableGeeDualKeyMapTest.c"
						}
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						if (_tmp100_) {
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							found2 = TRUE;
#line 2114 "SerializableGeeDualKeyMapTest.c"
						}
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp111_ = name;
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp112_ = gxml_node_get_value (_tmp111_);
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp113_ = _tmp112_;
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp114_ = _tmp113_;
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp115_ = g_strcmp0 (_tmp114_, "Bigger") == 0;
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_g_free0 (_tmp114_);
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						if (_tmp115_) {
#line 2130 "SerializableGeeDualKeyMapTest.c"
							GXmlNode* _tmp116_ = NULL;
							gchar* _tmp117_ = NULL;
							gchar* _tmp118_ = NULL;
							gchar* _tmp119_ = NULL;
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp116_ = owner;
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp117_ = gxml_node_get_value (_tmp116_);
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp118_ = _tmp117_;
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp119_ = _tmp118_;
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp110_ = g_strcmp0 (_tmp119_, "Floor") == 0;
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_g_free0 (_tmp119_);
#line 2147 "SerializableGeeDualKeyMapTest.c"
						} else {
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp110_ = FALSE;
#line 2151 "SerializableGeeDualKeyMapTest.c"
						}
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						if (_tmp110_) {
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							found3 = TRUE;
#line 2157 "SerializableGeeDualKeyMapTest.c"
						}
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp121_ = name;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp122_ = gxml_node_get_value (_tmp121_);
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp123_ = _tmp122_;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp124_ = _tmp123_;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp125_ = g_strcmp0 (_tmp124_, "Smallest") == 0;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_g_free0 (_tmp124_);
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						if (_tmp125_) {
#line 2173 "SerializableGeeDualKeyMapTest.c"
							GXmlNode* _tmp126_ = NULL;
							gchar* _tmp127_ = NULL;
							gchar* _tmp128_ = NULL;
							gchar* _tmp129_ = NULL;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp126_ = owner;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp127_ = gxml_node_get_value (_tmp126_);
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp128_ = _tmp127_;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp129_ = _tmp128_;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp120_ = g_strcmp0 (_tmp129_, "Wall") == 0;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_g_free0 (_tmp129_);
#line 2190 "SerializableGeeDualKeyMapTest.c"
						} else {
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp120_ = FALSE;
#line 2194 "SerializableGeeDualKeyMapTest.c"
						}
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						if (_tmp120_) {
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							found4 = TRUE;
#line 2200 "SerializableGeeDualKeyMapTest.c"
						}
					}
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_g_object_unref0 (owner);
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_g_object_unref0 (name);
#line 2207 "SerializableGeeDualKeyMapTest.c"
				}
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_g_object_unref0 (n);
#line 2211 "SerializableGeeDualKeyMapTest.c"
			}
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (_n_list);
#line 2215 "SerializableGeeDualKeyMapTest.c"
		}
#line 207 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp130_ = i;
#line 207 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (_tmp130_ != 4) {
#line 2221 "SerializableGeeDualKeyMapTest.c"
			FILE* _tmp131_ = NULL;
			gint _tmp132_ = 0;
			gchar* _tmp133_ = NULL;
			gchar* _tmp134_ = NULL;
			GXmlTDocument* _tmp135_ = NULL;
			gchar* _tmp136_ = NULL;
			gchar* _tmp137_ = NULL;
			gchar* _tmp138_ = NULL;
			gchar* _tmp139_ = NULL;
#line 208 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp131_ = stdout;
#line 208 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp132_ = nodes;
#line 208 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp133_ = g_strdup_printf ("%i", _tmp132_);
#line 208 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp134_ = _tmp133_;
#line 208 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp135_ = doc;
#line 208 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp136_ = gxml_tnode_to_string ((GXmlTNode*) _tmp135_);
#line 208 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp137_ = _tmp136_;
#line 208 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp138_ = g_strconcat ("ERROR: Incorrect number of space nodes. Expected 4, got: ", _tmp134_, "\n", _tmp137_, NULL);
#line 208 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp139_ = _tmp138_;
#line 208 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			fprintf (_tmp131_, "%s", _tmp139_);
#line 208 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp139_);
#line 208 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp137_);
#line 208 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp134_);
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			g_assert_not_reached ();
#line 2259 "SerializableGeeDualKeyMapTest.c"
		}
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp140_ = nodes;
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (_tmp140_ != 4) {
#line 2265 "SerializableGeeDualKeyMapTest.c"
			FILE* _tmp141_ = NULL;
			gint _tmp142_ = 0;
			gchar* _tmp143_ = NULL;
			gchar* _tmp144_ = NULL;
			GXmlTDocument* _tmp145_ = NULL;
			gchar* _tmp146_ = NULL;
			gchar* _tmp147_ = NULL;
			gchar* _tmp148_ = NULL;
			gchar* _tmp149_ = NULL;
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp141_ = stdout;
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp142_ = nodes;
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp143_ = g_strdup_printf ("%i", _tmp142_);
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp144_ = _tmp143_;
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp145_ = doc;
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp146_ = gxml_tnode_to_string ((GXmlTNode*) _tmp145_);
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp147_ = _tmp146_;
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp148_ = g_strconcat ("ERROR: Incorrect number of nodes. Expected 5, got: ", _tmp144_, "\n", _tmp147_, NULL);
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp149_ = _tmp148_;
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			fprintf (_tmp141_, "%s", _tmp149_);
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp149_);
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp147_);
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp144_);
#line 214 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			g_assert_not_reached ();
#line 2303 "SerializableGeeDualKeyMapTest.c"
		}
#line 216 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp150_ = found1;
#line 216 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (!_tmp150_) {
#line 2309 "SerializableGeeDualKeyMapTest.c"
			FILE* _tmp151_ = NULL;
			GXmlTDocument* _tmp152_ = NULL;
			gchar* _tmp153_ = NULL;
			gchar* _tmp154_ = NULL;
			gchar* _tmp155_ = NULL;
			gchar* _tmp156_ = NULL;
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp151_ = stdout;
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp152_ = doc;
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp153_ = gxml_tnode_to_string ((GXmlTNode*) _tmp152_);
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp154_ = _tmp153_;
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp155_ = g_strconcat ("ERROR: 'Big' / 'Floor' not found\n", _tmp154_, NULL);
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp156_ = _tmp155_;
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			fprintf (_tmp151_, "%s", _tmp156_);
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp156_);
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp154_);
#line 218 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			g_assert_not_reached ();
#line 2336 "SerializableGeeDualKeyMapTest.c"
		}
#line 220 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp157_ = found2;
#line 220 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (!_tmp157_) {
#line 2342 "SerializableGeeDualKeyMapTest.c"
			FILE* _tmp158_ = NULL;
			GXmlTDocument* _tmp159_ = NULL;
			gchar* _tmp160_ = NULL;
			gchar* _tmp161_ = NULL;
			gchar* _tmp162_ = NULL;
			gchar* _tmp163_ = NULL;
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp158_ = stdout;
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp159_ = doc;
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp160_ = gxml_tnode_to_string ((GXmlTNode*) _tmp159_);
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp161_ = _tmp160_;
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp162_ = g_strconcat ("ERROR: 'Small' / 'Wall' not found\n", _tmp161_, NULL);
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp163_ = _tmp162_;
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			fprintf (_tmp158_, "%s", _tmp163_);
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp163_);
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp161_);
#line 222 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			g_assert_not_reached ();
#line 2369 "SerializableGeeDualKeyMapTest.c"
		}
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp164_ = found3;
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (!_tmp164_) {
#line 2375 "SerializableGeeDualKeyMapTest.c"
			FILE* _tmp165_ = NULL;
			GXmlTDocument* _tmp166_ = NULL;
			gchar* _tmp167_ = NULL;
			gchar* _tmp168_ = NULL;
			gchar* _tmp169_ = NULL;
			gchar* _tmp170_ = NULL;
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp165_ = stdout;
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp166_ = doc;
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp167_ = gxml_tnode_to_string ((GXmlTNode*) _tmp166_);
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp168_ = _tmp167_;
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp169_ = g_strconcat ("ERROR: 'Bigger' / 'Floor' not found\n", _tmp168_, NULL);
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp170_ = _tmp169_;
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			fprintf (_tmp165_, "%s", _tmp170_);
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp170_);
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp168_);
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			g_assert_not_reached ();
#line 2402 "SerializableGeeDualKeyMapTest.c"
		}
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp171_ = found4;
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (!_tmp171_) {
#line 2408 "SerializableGeeDualKeyMapTest.c"
			FILE* _tmp172_ = NULL;
			GXmlTDocument* _tmp173_ = NULL;
			gchar* _tmp174_ = NULL;
			gchar* _tmp175_ = NULL;
			gchar* _tmp176_ = NULL;
			gchar* _tmp177_ = NULL;
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp172_ = stdout;
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp173_ = doc;
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp174_ = gxml_tnode_to_string ((GXmlTNode*) _tmp173_);
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp175_ = _tmp174_;
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp176_ = g_strconcat ("ERROR: 'Smallest' / 'Wall' not found\n", _tmp175_, NULL);
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp177_ = _tmp176_;
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			fprintf (_tmp172_, "%s", _tmp177_);
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp177_);
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp175_);
#line 230 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			g_assert_not_reached ();
#line 2435 "SerializableGeeDualKeyMapTest.c"
		}
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (root);
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (doc);
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (o4);
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (o3);
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (o2);
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (o1);
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (c);
#line 2451 "SerializableGeeDualKeyMapTest.c"
	}
	goto __finally77;
	__catch77_g_error:
	{
		GError* e = NULL;
		FILE* _tmp178_ = NULL;
		const gchar* _tmp179_ = NULL;
		const gchar* _tmp180_ = NULL;
		gchar* _tmp181_ = NULL;
		gchar* _tmp182_ = NULL;
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		e = _inner_error_;
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_inner_error_ = NULL;
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp178_ = stdout;
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp179_ = e->message;
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp180_ = string_to_string (_tmp179_);
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp181_ = g_strconcat ("ERROR: ", _tmp180_, NULL);
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp182_ = _tmp181_;
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		fprintf (_tmp178_, "%s", _tmp182_);
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_free0 (_tmp182_);
#line 235 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_assert_not_reached ();
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_error_free0 (e);
#line 2484 "SerializableGeeDualKeyMapTest.c"
	}
	__finally77:
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_clear_error (&_inner_error_);
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		return;
#line 2495 "SerializableGeeDualKeyMapTest.c"
	}
}


static void ___lambda55__gtest_func (void) {
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	__lambda55_ ();
#line 2503 "SerializableGeeDualKeyMapTest.c"
}


static void __lambda56_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		GXmlSerializableDualKeyMap* c = NULL;
		GXmlSerializableDualKeyMap* _tmp1_ = NULL;
		GXmlSerializableDualKeyMap* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		GXmlNode* _tmp4_ = NULL;
		GXmlNode* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		GXmlSerializableDualKeyMap* _tmp7_ = NULL;
		gint _tmp8_ = 0;
		gint _tmp9_ = 0;
		gboolean found1 = FALSE;
		gboolean found2 = FALSE;
		gboolean found3 = FALSE;
		gboolean found4 = FALSE;
		gboolean _tmp55_ = FALSE;
		gboolean _tmp62_ = FALSE;
		gboolean _tmp69_ = FALSE;
		gboolean _tmp76_ = FALSE;
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp0_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n" \
"<root><spaces name=\"Small\" owner=\"Wall\"/><spaces name=\"Smallest\"" \
" owner=\"Wall\"/><spaces name=\"Big\" owner=\"Floor\"/><spaces name=\"" \
"Bigger\" owner=\"Floor\"/><spark /></root>", 0, &_inner_error_);
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		doc = _tmp0_;
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 2536 "SerializableGeeDualKeyMapTest.c"
			goto __catch78_g_error;
		}
#line 243 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp1_ = gxml_serializable_dual_key_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, TYPE_SPACES, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 243 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		c = _tmp1_;
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp2_ = c;
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp3_ = doc;
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp4_ = gxml_document_get_root ((GXmlDocument*) _tmp3_);
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp5_ = _tmp4_;
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp6_ = _tmp5_;
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		gxml_serializable_dual_key_map_deserialize (_tmp2_, _tmp6_, &_inner_error_);
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp6_);
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (c);
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (doc);
#line 2563 "SerializableGeeDualKeyMapTest.c"
			goto __catch78_g_error;
		}
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp7_ = c;
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp8_ = gxml_serializable_dual_key_map_get_size (_tmp7_);
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp9_ = _tmp8_;
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (_tmp9_ != 4) {
#line 2574 "SerializableGeeDualKeyMapTest.c"
			FILE* _tmp10_ = NULL;
			GXmlSerializableDualKeyMap* _tmp11_ = NULL;
			gint _tmp12_ = 0;
			gint _tmp13_ = 0;
			gchar* _tmp14_ = NULL;
			gchar* _tmp15_ = NULL;
			gchar* _tmp16_ = NULL;
			gchar* _tmp17_ = NULL;
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp10_ = stdout;
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp11_ = c;
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp12_ = gxml_serializable_dual_key_map_get_size (_tmp11_);
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp13_ = _tmp12_;
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp14_ = g_strdup_printf ("%i", _tmp13_);
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp15_ = _tmp14_;
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp16_ = g_strconcat ("ERROR: incorrect size must be 4 got: ", _tmp15_, "\n", NULL);
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp17_ = _tmp16_;
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			fprintf (_tmp10_, "%s", _tmp17_);
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp17_);
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp15_);
#line 247 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			g_assert_not_reached ();
#line 2607 "SerializableGeeDualKeyMapTest.c"
		}
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		found1 = FALSE;
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		found2 = FALSE;
#line 251 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		found3 = FALSE;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		found4 = FALSE;
#line 2617 "SerializableGeeDualKeyMapTest.c"
		{
			GeeIterator* _s_it = NULL;
			GXmlSerializableDualKeyMap* _tmp18_ = NULL;
			GeeCollection* _tmp19_ = NULL;
			GeeCollection* _tmp20_ = NULL;
			GeeIterator* _tmp21_ = NULL;
			GeeIterator* _tmp22_ = NULL;
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp18_ = c;
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp19_ = gxml_serializable_dual_key_map_values (_tmp18_);
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp20_ = _tmp19_;
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp21_ = gee_iterable_iterator ((GeeIterable*) _tmp20_);
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp22_ = _tmp21_;
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (_tmp20_);
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_s_it = _tmp22_;
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			while (TRUE) {
#line 2641 "SerializableGeeDualKeyMapTest.c"
				GeeIterator* _tmp23_ = NULL;
				gboolean _tmp24_ = FALSE;
				Spaces* s = NULL;
				GeeIterator* _tmp25_ = NULL;
				gpointer _tmp26_ = NULL;
				gboolean _tmp27_ = FALSE;
				Spaces* _tmp28_ = NULL;
				const gchar* _tmp29_ = NULL;
				const gchar* _tmp30_ = NULL;
				gboolean _tmp34_ = FALSE;
				Spaces* _tmp35_ = NULL;
				const gchar* _tmp36_ = NULL;
				const gchar* _tmp37_ = NULL;
				gboolean _tmp41_ = FALSE;
				Spaces* _tmp42_ = NULL;
				const gchar* _tmp43_ = NULL;
				const gchar* _tmp44_ = NULL;
				gboolean _tmp48_ = FALSE;
				Spaces* _tmp49_ = NULL;
				const gchar* _tmp50_ = NULL;
				const gchar* _tmp51_ = NULL;
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp23_ = _s_it;
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp24_ = gee_iterator_next (_tmp23_);
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				if (!_tmp24_) {
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					break;
#line 2671 "SerializableGeeDualKeyMapTest.c"
				}
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp25_ = _s_it;
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp26_ = gee_iterator_get (_tmp25_);
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				s = (Spaces*) _tmp26_;
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp28_ = s;
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp29_ = spaces_get_owner (_tmp28_);
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp30_ = _tmp29_;
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				if (g_strcmp0 (_tmp30_, "Floor") == 0) {
#line 2687 "SerializableGeeDualKeyMapTest.c"
					Spaces* _tmp31_ = NULL;
					const gchar* _tmp32_ = NULL;
					const gchar* _tmp33_ = NULL;
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp31_ = s;
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp32_ = spaces_get_name (_tmp31_);
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp33_ = _tmp32_;
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp27_ = g_strcmp0 (_tmp33_, "Big") == 0;
#line 2699 "SerializableGeeDualKeyMapTest.c"
				} else {
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp27_ = FALSE;
#line 2703 "SerializableGeeDualKeyMapTest.c"
				}
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				if (_tmp27_) {
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					found1 = TRUE;
#line 2709 "SerializableGeeDualKeyMapTest.c"
				}
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp35_ = s;
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp36_ = spaces_get_owner (_tmp35_);
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp37_ = _tmp36_;
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				if (g_strcmp0 (_tmp37_, "Wall") == 0) {
#line 2719 "SerializableGeeDualKeyMapTest.c"
					Spaces* _tmp38_ = NULL;
					const gchar* _tmp39_ = NULL;
					const gchar* _tmp40_ = NULL;
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp38_ = s;
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp39_ = spaces_get_name (_tmp38_);
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp40_ = _tmp39_;
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp34_ = g_strcmp0 (_tmp40_, "Small") == 0;
#line 2731 "SerializableGeeDualKeyMapTest.c"
				} else {
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp34_ = FALSE;
#line 2735 "SerializableGeeDualKeyMapTest.c"
				}
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				if (_tmp34_) {
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					found2 = TRUE;
#line 2741 "SerializableGeeDualKeyMapTest.c"
				}
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp42_ = s;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp43_ = spaces_get_owner (_tmp42_);
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp44_ = _tmp43_;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				if (g_strcmp0 (_tmp44_, "Floor") == 0) {
#line 2751 "SerializableGeeDualKeyMapTest.c"
					Spaces* _tmp45_ = NULL;
					const gchar* _tmp46_ = NULL;
					const gchar* _tmp47_ = NULL;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp45_ = s;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp46_ = spaces_get_name (_tmp45_);
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp47_ = _tmp46_;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp41_ = g_strcmp0 (_tmp47_, "Bigger") == 0;
#line 2763 "SerializableGeeDualKeyMapTest.c"
				} else {
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp41_ = FALSE;
#line 2767 "SerializableGeeDualKeyMapTest.c"
				}
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				if (_tmp41_) {
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					found3 = TRUE;
#line 2773 "SerializableGeeDualKeyMapTest.c"
				}
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp49_ = s;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp50_ = spaces_get_owner (_tmp49_);
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp51_ = _tmp50_;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				if (g_strcmp0 (_tmp51_, "Wall") == 0) {
#line 2783 "SerializableGeeDualKeyMapTest.c"
					Spaces* _tmp52_ = NULL;
					const gchar* _tmp53_ = NULL;
					const gchar* _tmp54_ = NULL;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp52_ = s;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp53_ = spaces_get_name (_tmp52_);
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp54_ = _tmp53_;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp48_ = g_strcmp0 (_tmp54_, "Smallest") == 0;
#line 2795 "SerializableGeeDualKeyMapTest.c"
				} else {
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp48_ = FALSE;
#line 2799 "SerializableGeeDualKeyMapTest.c"
				}
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				if (_tmp48_) {
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					found4 = TRUE;
#line 2805 "SerializableGeeDualKeyMapTest.c"
				}
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_g_object_unref0 (s);
#line 2809 "SerializableGeeDualKeyMapTest.c"
			}
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (_s_it);
#line 2813 "SerializableGeeDualKeyMapTest.c"
		}
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp55_ = found1;
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (!_tmp55_) {
#line 2819 "SerializableGeeDualKeyMapTest.c"
			FILE* _tmp56_ = NULL;
			GXmlGDocument* _tmp57_ = NULL;
			gchar* _tmp58_ = NULL;
			gchar* _tmp59_ = NULL;
			gchar* _tmp60_ = NULL;
			gchar* _tmp61_ = NULL;
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp56_ = stdout;
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp57_ = doc;
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp58_ = gxml_gnode_to_string ((GXmlGNode*) _tmp57_);
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp59_ = _tmp58_;
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp60_ = g_strconcat ("ERROR: 'Big' / 'Floor' not found\n", _tmp59_, NULL);
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp61_ = _tmp60_;
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			fprintf (_tmp56_, "%s", _tmp61_);
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp61_);
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp59_);
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			g_assert_not_reached ();
#line 2846 "SerializableGeeDualKeyMapTest.c"
		}
#line 263 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp62_ = found2;
#line 263 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (!_tmp62_) {
#line 2852 "SerializableGeeDualKeyMapTest.c"
			FILE* _tmp63_ = NULL;
			GXmlGDocument* _tmp64_ = NULL;
			gchar* _tmp65_ = NULL;
			gchar* _tmp66_ = NULL;
			gchar* _tmp67_ = NULL;
			gchar* _tmp68_ = NULL;
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp63_ = stdout;
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp64_ = doc;
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp65_ = gxml_gnode_to_string ((GXmlGNode*) _tmp64_);
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp66_ = _tmp65_;
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp67_ = g_strconcat ("ERROR: 'Small' / 'Wall' not found\n", _tmp66_, NULL);
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp68_ = _tmp67_;
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			fprintf (_tmp63_, "%s", _tmp68_);
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp68_);
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp66_);
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			g_assert_not_reached ();
#line 2879 "SerializableGeeDualKeyMapTest.c"
		}
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp69_ = found3;
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (!_tmp69_) {
#line 2885 "SerializableGeeDualKeyMapTest.c"
			FILE* _tmp70_ = NULL;
			GXmlGDocument* _tmp71_ = NULL;
			gchar* _tmp72_ = NULL;
			gchar* _tmp73_ = NULL;
			gchar* _tmp74_ = NULL;
			gchar* _tmp75_ = NULL;
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp70_ = stdout;
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp71_ = doc;
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp72_ = gxml_gnode_to_string ((GXmlGNode*) _tmp71_);
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp73_ = _tmp72_;
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp74_ = g_strconcat ("ERROR: 'Bigger' / 'Floor' not found\n", _tmp73_, NULL);
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp75_ = _tmp74_;
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			fprintf (_tmp70_, "%s", _tmp75_);
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp75_);
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp73_);
#line 269 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			g_assert_not_reached ();
#line 2912 "SerializableGeeDualKeyMapTest.c"
		}
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp76_ = found4;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (!_tmp76_) {
#line 2918 "SerializableGeeDualKeyMapTest.c"
			FILE* _tmp77_ = NULL;
			GXmlGDocument* _tmp78_ = NULL;
			gchar* _tmp79_ = NULL;
			gchar* _tmp80_ = NULL;
			gchar* _tmp81_ = NULL;
			gchar* _tmp82_ = NULL;
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp77_ = stdout;
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp78_ = doc;
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp79_ = gxml_gnode_to_string ((GXmlGNode*) _tmp78_);
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp80_ = _tmp79_;
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp81_ = g_strconcat ("ERROR: 'Smallest' / 'Wall' not found\n", _tmp80_, NULL);
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp82_ = _tmp81_;
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			fprintf (_tmp77_, "%s", _tmp82_);
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp82_);
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp80_);
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			g_assert_not_reached ();
#line 2945 "SerializableGeeDualKeyMapTest.c"
		}
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (c);
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (doc);
#line 2951 "SerializableGeeDualKeyMapTest.c"
	}
	goto __finally78;
	__catch78_g_error:
	{
		GError* e = NULL;
		FILE* _tmp83_ = NULL;
		const gchar* _tmp84_ = NULL;
		const gchar* _tmp85_ = NULL;
		gchar* _tmp86_ = NULL;
		gchar* _tmp87_ = NULL;
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		e = _inner_error_;
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_inner_error_ = NULL;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp83_ = stdout;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp84_ = e->message;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp85_ = string_to_string (_tmp84_);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp86_ = g_strconcat ("ERROR: ", _tmp85_, NULL);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp87_ = _tmp86_;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		fprintf (_tmp83_, "%s", _tmp87_);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_free0 (_tmp87_);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_assert_not_reached ();
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_error_free0 (e);
#line 2984 "SerializableGeeDualKeyMapTest.c"
	}
	__finally78:
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_clear_error (&_inner_error_);
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		return;
#line 2995 "SerializableGeeDualKeyMapTest.c"
	}
}


static void ___lambda56__gtest_func (void) {
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	__lambda56_ ();
#line 3003 "SerializableGeeDualKeyMapTest.c"
}


static void __lambda57_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* idoc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		GXmlSerializableDualKeyMap* ic = NULL;
		GXmlSerializableDualKeyMap* _tmp1_ = NULL;
		GXmlSerializableDualKeyMap* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		GXmlNode* _tmp4_ = NULL;
		GXmlNode* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		GXmlSerializableDualKeyMap* _tmp7_ = NULL;
		gint _tmp8_ = 0;
		gint _tmp9_ = 0;
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp21_ = NULL;
		GXmlSerializableDualKeyMap* _tmp22_ = NULL;
		GXmlGDocument* _tmp23_ = NULL;
		GXmlNode* _tmp24_ = NULL;
		GXmlNode* _tmp25_ = NULL;
		GXmlNode* _tmp26_ = NULL;
		GXmlNode* _tmp27_ = NULL;
		GXmlNode* _tmp28_ = NULL;
		GXmlSerializableDualKeyMap* c = NULL;
		GXmlSerializableDualKeyMap* _tmp29_ = NULL;
		GXmlSerializableDualKeyMap* _tmp30_ = NULL;
		GXmlGDocument* _tmp31_ = NULL;
		GXmlNode* _tmp32_ = NULL;
		GXmlNode* _tmp33_ = NULL;
		GXmlNode* _tmp34_ = NULL;
		GXmlSerializableDualKeyMap* _tmp35_ = NULL;
		gint _tmp36_ = 0;
		gint _tmp37_ = 0;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp0_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n" \
"<root><spaces name=\"Small\" owner=\"Wall\"/><spaces name=\"Smallest\"" \
" owner=\"Wall\"/><spaces name=\"Big\" owner=\"Floor\"/><spaces name=\"" \
"Bigger\" owner=\"Floor\"/><spark /></root>", 0, &_inner_error_);
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		idoc = _tmp0_;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 3047 "SerializableGeeDualKeyMapTest.c"
			goto __catch79_g_error;
		}
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp1_ = gxml_serializable_dual_key_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, TYPE_SPACES, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		ic = _tmp1_;
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp2_ = ic;
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp3_ = idoc;
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp4_ = gxml_document_get_root ((GXmlDocument*) _tmp3_);
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp5_ = _tmp4_;
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp6_ = _tmp5_;
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		gxml_serializable_dual_key_map_deserialize (_tmp2_, _tmp6_, &_inner_error_);
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp6_);
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (ic);
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (idoc);
#line 3074 "SerializableGeeDualKeyMapTest.c"
			goto __catch79_g_error;
		}
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp7_ = ic;
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp8_ = gxml_serializable_dual_key_map_get_size (_tmp7_);
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp9_ = _tmp8_;
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (_tmp9_ != 4) {
#line 3085 "SerializableGeeDualKeyMapTest.c"
			FILE* _tmp10_ = NULL;
			GXmlSerializableDualKeyMap* _tmp11_ = NULL;
			gint _tmp12_ = 0;
			gint _tmp13_ = 0;
			gchar* _tmp14_ = NULL;
			gchar* _tmp15_ = NULL;
			GXmlGDocument* _tmp16_ = NULL;
			gchar* _tmp17_ = NULL;
			gchar* _tmp18_ = NULL;
			gchar* _tmp19_ = NULL;
			gchar* _tmp20_ = NULL;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp10_ = stdout;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp11_ = ic;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp12_ = gxml_serializable_dual_key_map_get_size (_tmp11_);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp13_ = _tmp12_;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp14_ = g_strdup_printf ("%i", _tmp13_);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp15_ = _tmp14_;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp16_ = idoc;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp17_ = gxml_gnode_to_string ((GXmlGNode*) _tmp16_);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp18_ = _tmp17_;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp19_ = g_strconcat ("ERROR: Incorrect size (1st deserialize). Expected 4, got: ", _tmp15_, "\n", _tmp18_, "\n", NULL);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp20_ = _tmp19_;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			fprintf (_tmp10_, "%s", _tmp20_);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp20_);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp18_);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp15_);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			g_assert_not_reached ();
#line 3129 "SerializableGeeDualKeyMapTest.c"
		}
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp21_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n<root />", 0, &_inner_error_);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		doc = _tmp21_;
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (ic);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (idoc);
#line 3141 "SerializableGeeDualKeyMapTest.c"
			goto __catch79_g_error;
		}
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp22_ = ic;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp23_ = doc;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp24_ = gxml_document_get_root ((GXmlDocument*) _tmp23_);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp25_ = _tmp24_;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp26_ = _tmp25_;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp27_ = gxml_serializable_dual_key_map_serialize (_tmp22_, _tmp26_, &_inner_error_);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp28_ = _tmp27_;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp28_);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp26_);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (doc);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (ic);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (idoc);
#line 3170 "SerializableGeeDualKeyMapTest.c"
			goto __catch79_g_error;
		}
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp29_ = gxml_serializable_dual_key_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, TYPE_SPACES, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		c = _tmp29_;
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp30_ = c;
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp31_ = doc;
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp32_ = gxml_document_get_root ((GXmlDocument*) _tmp31_);
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp33_ = _tmp32_;
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp34_ = _tmp33_;
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		gxml_serializable_dual_key_map_deserialize (_tmp30_, _tmp34_, &_inner_error_);
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp34_);
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (c);
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (doc);
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (ic);
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (idoc);
#line 3201 "SerializableGeeDualKeyMapTest.c"
			goto __catch79_g_error;
		}
#line 297 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp35_ = c;
#line 297 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp36_ = gxml_serializable_dual_key_map_get_size (_tmp35_);
#line 297 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp37_ = _tmp36_;
#line 297 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (_tmp37_ != 4) {
#line 3212 "SerializableGeeDualKeyMapTest.c"
			FILE* _tmp38_ = NULL;
			GXmlSerializableDualKeyMap* _tmp39_ = NULL;
			gint _tmp40_ = 0;
			gint _tmp41_ = 0;
			gchar* _tmp42_ = NULL;
			gchar* _tmp43_ = NULL;
			GXmlGDocument* _tmp44_ = NULL;
			gchar* _tmp45_ = NULL;
			gchar* _tmp46_ = NULL;
			gchar* _tmp47_ = NULL;
			gchar* _tmp48_ = NULL;
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp38_ = stdout;
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp39_ = c;
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp40_ = gxml_serializable_dual_key_map_get_size (_tmp39_);
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp41_ = _tmp40_;
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp42_ = g_strdup_printf ("%i", _tmp41_);
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp43_ = _tmp42_;
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp44_ = doc;
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp45_ = gxml_gnode_to_string ((GXmlGNode*) _tmp44_);
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp46_ = _tmp45_;
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp47_ = g_strconcat ("ERROR: Incorrect size. Expected 4, got: ", _tmp43_, "\n", _tmp46_, "\n", NULL);
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_tmp48_ = _tmp47_;
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			fprintf (_tmp38_, "%s", _tmp48_);
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp48_);
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp46_);
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_free0 (_tmp43_);
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			g_assert_not_reached ();
#line 3256 "SerializableGeeDualKeyMapTest.c"
		}
#line 283 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (c);
#line 283 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (doc);
#line 283 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (ic);
#line 283 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (idoc);
#line 3266 "SerializableGeeDualKeyMapTest.c"
	}
	goto __finally79;
	__catch79_g_error:
	{
		GError* e = NULL;
		FILE* _tmp49_ = NULL;
		const gchar* _tmp50_ = NULL;
		const gchar* _tmp51_ = NULL;
		gchar* _tmp52_ = NULL;
		gchar* _tmp53_ = NULL;
#line 283 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		e = _inner_error_;
#line 283 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_inner_error_ = NULL;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp49_ = stdout;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp50_ = e->message;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp51_ = string_to_string (_tmp50_);
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp52_ = g_strconcat ("ERROR: ", _tmp51_, NULL);
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp53_ = _tmp52_;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		fprintf (_tmp49_, "%s", _tmp53_);
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_free0 (_tmp53_);
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_assert_not_reached ();
#line 283 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_error_free0 (e);
#line 3299 "SerializableGeeDualKeyMapTest.c"
	}
	__finally79:
#line 283 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 283 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 283 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_clear_error (&_inner_error_);
#line 283 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		return;
#line 3310 "SerializableGeeDualKeyMapTest.c"
	}
}


static void ___lambda57__gtest_func (void) {
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	__lambda57_ ();
#line 3318 "SerializableGeeDualKeyMapTest.c"
}


static void __lambda58_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* d = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		BookStore* bs = NULL;
		BookStore* _tmp1_ = NULL;
		BookStore* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		BookStore* _tmp4_ = NULL;
		const gchar* _tmp5_ = NULL;
		const gchar* _tmp6_ = NULL;
		BookStore* _tmp7_ = NULL;
		BookArray* _tmp8_ = NULL;
		BookArray* _tmp9_ = NULL;
		gint _tmp10_ = 0;
		gint _tmp11_ = 0;
		Book* b = NULL;
		BookStore* _tmp12_ = NULL;
		BookArray* _tmp13_ = NULL;
		BookArray* _tmp14_ = NULL;
		gpointer _tmp15_ = NULL;
		Name* _tmp16_ = NULL;
		Name* _tmp17_ = NULL;
		Name* _tmp18_ = NULL;
		Name* _tmp19_ = NULL;
		gchar* _tmp20_ = NULL;
		gchar* _tmp21_ = NULL;
		const gchar* _tmp22_ = NULL;
		const gchar* _tmp23_ = NULL;
		Authors* _tmp24_ = NULL;
		Authors* _tmp25_ = NULL;
		Authors* _tmp26_ = NULL;
		Authors* _tmp27_ = NULL;
		AuthorArray* _tmp28_ = NULL;
		AuthorArray* _tmp29_ = NULL;
		Authors* _tmp30_ = NULL;
		Authors* _tmp31_ = NULL;
		AuthorArray* _tmp32_ = NULL;
		AuthorArray* _tmp33_ = NULL;
		gint _tmp34_ = 0;
		gint _tmp35_ = 0;
		InventoryDualKeyMap* _tmp36_ = NULL;
		InventoryDualKeyMap* _tmp37_ = NULL;
		InventoryDualKeyMap* _tmp38_ = NULL;
		InventoryDualKeyMap* _tmp39_ = NULL;
		gint _tmp40_ = 0;
		gint _tmp41_ = 0;
		Inventory* ir = NULL;
		InventoryDualKeyMap* _tmp42_ = NULL;
		InventoryDualKeyMap* _tmp43_ = NULL;
		gpointer _tmp44_ = NULL;
		gint _tmp45_ = 0;
		gint _tmp46_ = 0;
		const gchar* _tmp47_ = NULL;
		const gchar* _tmp48_ = NULL;
		gint _tmp49_ = 0;
		gint _tmp50_ = 0;
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp0_ = gxml_gdocument_new_from_path (TEST_DIR "/test-collection.xml", 0, &_inner_error_);
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		d = _tmp0_;
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 3386 "SerializableGeeDualKeyMapTest.c"
			goto __catch80_g_error;
		}
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp1_ = book_store_new ();
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		bs = _tmp1_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp2_ = bs;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp3_ = d;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp2_, (GXmlNode*) _tmp3_, &_inner_error_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bs);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (d);
#line 3405 "SerializableGeeDualKeyMapTest.c"
			goto __catch80_g_error;
		}
#line 313 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp4_ = bs;
#line 313 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp5_ = book_store_get_name (_tmp4_);
#line 313 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp6_ = _tmp5_;
#line 313 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp6_, "The Great Book") == 0, "bs.name == \"The Great Book\"");
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp7_ = bs;
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp8_ = book_store_get_books (_tmp7_);
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp9_ = _tmp8_;
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp10_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp9_);
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp11_ = _tmp10_;
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp11_ == 3, "bs.books.size == 3");
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp12_ = bs;
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp13_ = book_store_get_books (_tmp12_);
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp14_ = _tmp13_;
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp15_ = gee_list_first ((GeeList*) _tmp14_);
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		b = (Book*) _tmp15_;
#line 316 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (b != NULL, "b != null");
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp16_ = book_get_name (b);
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp17_ = _tmp16_;
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp17_ != NULL, "b.name != null");
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp18_ = book_get_name (b);
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp19_ = _tmp18_;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp20_ = name_get_name (_tmp19_);
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp21_ = _tmp20_;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp21_, "Book1") == 0, "b.name.get_name () == \"Book1\"");
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_free0 (_tmp21_);
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp22_ = book_get_year (b);
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp23_ = _tmp22_;
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp23_, "2015") == 0, "b.year == \"2015\"");
#line 320 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp24_ = book_get_authors (b);
#line 320 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp25_ = _tmp24_;
#line 320 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp25_ != NULL, "b.authors != null");
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp26_ = book_get_authors (b);
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp27_ = _tmp26_;
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp28_ = authors_get_array (_tmp27_);
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp29_ = _tmp28_;
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp29_ != NULL, "b.authors.array != null");
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp30_ = book_get_authors (b);
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp31_ = _tmp30_;
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp32_ = authors_get_array (_tmp31_);
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp33_ = _tmp32_;
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp34_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp33_);
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp35_ = _tmp34_;
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp35_ == 2, "b.authors.array.size == 2");
#line 323 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp36_ = book_get_inventory_registers (b);
#line 323 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp37_ = _tmp36_;
#line 323 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp37_ != NULL, "b.inventory_registers != null");
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp38_ = book_get_inventory_registers (b);
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp39_ = _tmp38_;
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp40_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp39_);
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp41_ = _tmp40_;
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp41_ == 4, "b.inventory_registers.size == 4");
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp42_ = book_get_inventory_registers (b);
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp43_ = _tmp42_;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp44_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp43_, (gpointer) ((gintptr) 1), "K001");
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		ir = (Inventory*) _tmp44_;
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (ir != NULL, "ir != null");
#line 327 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp45_ = inventory_get_number (ir);
#line 327 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp46_ = _tmp45_;
#line 327 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp46_ == 1, "ir.number == 1");
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp47_ = inventory_get_inventory (ir);
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp48_ = _tmp47_;
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp48_, "K001") == 0, "ir.inventory == \"K001\"");
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp49_ = inventory_get_row (ir);
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp50_ = _tmp49_;
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp50_ == 5, "ir.row == 5");
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (ir);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (b);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (bs);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (d);
#line 3546 "SerializableGeeDualKeyMapTest.c"
	}
	goto __finally80;
	__catch80_g_error:
	{
		GError* e = NULL;
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		e = _inner_error_;
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_inner_error_ = NULL;
#line 334 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_assert_not_reached ();
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_error_free0 (e);
#line 3560 "SerializableGeeDualKeyMapTest.c"
	}
	__finally80:
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_clear_error (&_inner_error_);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		return;
#line 3571 "SerializableGeeDualKeyMapTest.c"
	}
}


static void ___lambda58__gtest_func (void) {
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	__lambda58_ ();
#line 3579 "SerializableGeeDualKeyMapTest.c"
}


static void __lambda59_ (void) {
	GError * _inner_error_ = NULL;
	{
		SerializableGeeDualKeyMapTestBigBag* bag = NULL;
		SerializableGeeDualKeyMapTestBigBag* _tmp0_ = NULL;
		SerializableGeeDualKeyMapTestBigBag* _tmp1_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp2_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp3_ = NULL;
		SerializableGeeDualKeyMapTestBigBag* _tmp58_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp59_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp60_ = NULL;
		gint _tmp61_ = 0;
		gint _tmp62_ = 0;
		GXmlGDocument* d = NULL;
		GXmlGDocument* _tmp63_ = NULL;
		SerializableGeeDualKeyMapTestBigBag* _tmp64_ = NULL;
		GXmlGDocument* _tmp65_ = NULL;
		GXmlNode* _tmp66_ = NULL;
		GXmlNode* _tmp67_ = NULL;
		GXmlGDocument* _tmp68_ = NULL;
		GXmlNode* _tmp69_ = NULL;
		GXmlNode* _tmp70_ = NULL;
		GXmlNode* _tmp71_ = NULL;
		GXmlGDocument* _tmp72_ = NULL;
		GXmlNode* _tmp73_ = NULL;
		GXmlNode* _tmp74_ = NULL;
		GXmlNode* _tmp75_ = NULL;
		gchar* _tmp76_ = NULL;
		gchar* _tmp77_ = NULL;
		gchar* _tmp78_ = NULL;
		GXmlGDocument* _tmp79_ = NULL;
		GXmlNode* _tmp80_ = NULL;
		GXmlNode* _tmp81_ = NULL;
		GXmlNode* _tmp82_ = NULL;
		GeeBidirList* _tmp83_ = NULL;
		GeeBidirList* _tmp84_ = NULL;
		GeeBidirList* _tmp85_ = NULL;
		gint _tmp86_ = 0;
		gint _tmp87_ = 0;
		GXmlGDocument* _tmp88_ = NULL;
		GXmlNode* _tmp89_ = NULL;
		GXmlNode* _tmp90_ = NULL;
		GXmlNode* _tmp91_ = NULL;
		GeeBidirList* _tmp92_ = NULL;
		GeeBidirList* _tmp93_ = NULL;
		GeeBidirList* _tmp94_ = NULL;
		gpointer _tmp95_ = NULL;
		GXmlNode* _tmp96_ = NULL;
		gchar* _tmp97_ = NULL;
		gchar* _tmp98_ = NULL;
		gchar* _tmp99_ = NULL;
		GXmlGDocument* _tmp100_ = NULL;
		GXmlNode* _tmp101_ = NULL;
		GXmlNode* _tmp102_ = NULL;
		GXmlNode* _tmp103_ = NULL;
		GeeBidirList* _tmp104_ = NULL;
		GeeBidirList* _tmp105_ = NULL;
		GeeBidirList* _tmp106_ = NULL;
		gpointer _tmp107_ = NULL;
		GXmlNode* _tmp108_ = NULL;
		GeeBidirList* _tmp109_ = NULL;
		GeeBidirList* _tmp110_ = NULL;
		GeeBidirList* _tmp111_ = NULL;
		gint _tmp112_ = 0;
		gint _tmp113_ = 0;
		GXmlGDocument* _tmp114_ = NULL;
		GXmlNode* _tmp115_ = NULL;
		GXmlNode* _tmp116_ = NULL;
		GXmlNode* _tmp117_ = NULL;
		GeeBidirList* _tmp118_ = NULL;
		GeeBidirList* _tmp119_ = NULL;
		GeeBidirList* _tmp120_ = NULL;
		gpointer _tmp121_ = NULL;
		GXmlNode* _tmp122_ = NULL;
		GeeBidirList* _tmp123_ = NULL;
		GeeBidirList* _tmp124_ = NULL;
		GeeBidirList* _tmp125_ = NULL;
		gpointer _tmp126_ = NULL;
		GXmlNode* _tmp127_ = NULL;
		gchar* _tmp128_ = NULL;
		gchar* _tmp129_ = NULL;
		gchar* _tmp130_ = NULL;
		GXmlGDocument* _tmp131_ = NULL;
		GXmlNode* _tmp132_ = NULL;
		GXmlNode* _tmp133_ = NULL;
		GXmlNode* _tmp134_ = NULL;
		GeeBidirList* _tmp135_ = NULL;
		GeeBidirList* _tmp136_ = NULL;
		GeeBidirList* _tmp137_ = NULL;
		gpointer _tmp138_ = NULL;
		GXmlNode* _tmp139_ = NULL;
		GeeBidirList* _tmp140_ = NULL;
		GeeBidirList* _tmp141_ = NULL;
		GeeBidirList* _tmp142_ = NULL;
		gpointer _tmp143_ = NULL;
		GXmlNode* _tmp144_ = NULL;
		GeeBidirList* _tmp145_ = NULL;
		GeeBidirList* _tmp146_ = NULL;
		GeeBidirList* _tmp147_ = NULL;
		gint _tmp148_ = 0;
		gint _tmp149_ = 0;
		GXmlGDocument* _tmp150_ = NULL;
		GXmlNode* _tmp151_ = NULL;
		GXmlNode* _tmp152_ = NULL;
		GXmlNode* _tmp153_ = NULL;
		GeeBidirList* _tmp154_ = NULL;
		GeeBidirList* _tmp155_ = NULL;
		GeeBidirList* _tmp156_ = NULL;
		gpointer _tmp157_ = NULL;
		GXmlNode* _tmp158_ = NULL;
		GeeBidirList* _tmp159_ = NULL;
		GeeBidirList* _tmp160_ = NULL;
		GeeBidirList* _tmp161_ = NULL;
		gpointer _tmp162_ = NULL;
		GXmlNode* _tmp163_ = NULL;
		GeeBidirList* _tmp164_ = NULL;
		GeeBidirList* _tmp165_ = NULL;
		GeeBidirList* _tmp166_ = NULL;
		gpointer _tmp167_ = NULL;
		GXmlNode* _tmp168_ = NULL;
		gchar* _tmp169_ = NULL;
		gchar* _tmp170_ = NULL;
		gchar* _tmp171_ = NULL;
		GXmlGDocument* _tmp172_ = NULL;
		GXmlNode* _tmp173_ = NULL;
		GXmlNode* _tmp174_ = NULL;
		GXmlNode* _tmp175_ = NULL;
		GeeBidirList* _tmp176_ = NULL;
		GeeBidirList* _tmp177_ = NULL;
		GeeBidirList* _tmp178_ = NULL;
		gpointer _tmp179_ = NULL;
		GXmlNode* _tmp180_ = NULL;
		GeeBidirList* _tmp181_ = NULL;
		GeeBidirList* _tmp182_ = NULL;
		GeeBidirList* _tmp183_ = NULL;
		gpointer _tmp184_ = NULL;
		GXmlNode* _tmp185_ = NULL;
		GeeBidirList* _tmp186_ = NULL;
		GeeBidirList* _tmp187_ = NULL;
		GeeBidirList* _tmp188_ = NULL;
		gpointer _tmp189_ = NULL;
		GXmlNode* _tmp190_ = NULL;
		GeeBidirList* _tmp191_ = NULL;
		GeeBidirList* _tmp192_ = NULL;
		GeeBidirList* _tmp193_ = NULL;
		gint _tmp194_ = 0;
		gint _tmp195_ = 0;
		GXmlGDocument* _tmp196_ = NULL;
		GXmlNode* _tmp197_ = NULL;
		GXmlNode* _tmp198_ = NULL;
		GXmlNode* _tmp199_ = NULL;
		GeeBidirList* _tmp200_ = NULL;
		GeeBidirList* _tmp201_ = NULL;
		GeeBidirList* _tmp202_ = NULL;
		gpointer _tmp203_ = NULL;
		GXmlNode* _tmp204_ = NULL;
		GeeBidirList* _tmp205_ = NULL;
		GeeBidirList* _tmp206_ = NULL;
		GeeBidirList* _tmp207_ = NULL;
		gpointer _tmp208_ = NULL;
		GXmlNode* _tmp209_ = NULL;
		GeeBidirList* _tmp210_ = NULL;
		GeeBidirList* _tmp211_ = NULL;
		GeeBidirList* _tmp212_ = NULL;
		gpointer _tmp213_ = NULL;
		GXmlNode* _tmp214_ = NULL;
		GeeBidirList* _tmp215_ = NULL;
		GeeBidirList* _tmp216_ = NULL;
		GeeBidirList* _tmp217_ = NULL;
		gpointer _tmp218_ = NULL;
		GXmlNode* _tmp219_ = NULL;
		GXmlGDocument* _tmp220_ = NULL;
		GXmlNode* _tmp221_ = NULL;
		GXmlNode* _tmp222_ = NULL;
		GXmlNode* _tmp223_ = NULL;
		GeeBidirList* _tmp224_ = NULL;
		GeeBidirList* _tmp225_ = NULL;
		GeeBidirList* _tmp226_ = NULL;
		gpointer _tmp227_ = NULL;
		GXmlNode* _tmp228_ = NULL;
		GeeBidirList* _tmp229_ = NULL;
		GeeBidirList* _tmp230_ = NULL;
		GeeBidirList* _tmp231_ = NULL;
		gpointer _tmp232_ = NULL;
		GXmlNode* _tmp233_ = NULL;
		GeeBidirList* _tmp234_ = NULL;
		GeeBidirList* _tmp235_ = NULL;
		GeeBidirList* _tmp236_ = NULL;
		gpointer _tmp237_ = NULL;
		GXmlNode* _tmp238_ = NULL;
		GeeBidirList* _tmp239_ = NULL;
		GeeBidirList* _tmp240_ = NULL;
		GeeBidirList* _tmp241_ = NULL;
		gpointer _tmp242_ = NULL;
		GXmlNode* _tmp243_ = NULL;
		gchar* _tmp244_ = NULL;
		gchar* _tmp245_ = NULL;
		gchar* _tmp246_ = NULL;
		SerializableGeeDualKeyMapTestBigBag* bagt = NULL;
		SerializableGeeDualKeyMapTestBigBag* _tmp247_ = NULL;
		SerializableGeeDualKeyMapTestBigBag* _tmp248_ = NULL;
		GXmlGDocument* _tmp249_ = NULL;
		SerializableGeeDualKeyMapTestBigBag* _tmp250_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp251_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp252_ = NULL;
		gint _tmp253_ = 0;
		gint _tmp254_ = 0;
		gboolean _tmp255_ = FALSE;
		SerializableGeeDualKeyMapTestBigBag* _tmp256_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp257_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp258_ = NULL;
		gboolean _tmp259_ = FALSE;
		SerializableGeeDualKeyMapTestBigBag* _tmp260_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp261_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp262_ = NULL;
		gint _tmp263_ = 0;
		gint _tmp264_ = 0;
		gboolean _tmp265_ = FALSE;
		SerializableGeeDualKeyMapTestBigBag* _tmp266_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp267_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp268_ = NULL;
		gpointer _tmp269_ = NULL;
		SerializableGeeDualKeyMapTestSmallBag* _tmp270_ = NULL;
		SerializableGeeDualKeyMapTestBallDualMap* _tmp271_ = NULL;
		SerializableGeeDualKeyMapTestBallDualMap* _tmp272_ = NULL;
		gboolean _tmp273_ = FALSE;
		gboolean _tmp274_ = FALSE;
		SerializableGeeDualKeyMapTestBigBag* _tmp275_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp276_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp277_ = NULL;
		gpointer _tmp278_ = NULL;
		SerializableGeeDualKeyMapTestSmallBag* _tmp279_ = NULL;
		SerializableGeeDualKeyMapTestBallDualMap* _tmp280_ = NULL;
		SerializableGeeDualKeyMapTestBallDualMap* _tmp281_ = NULL;
		gint _tmp282_ = 0;
		gint _tmp283_ = 0;
		SerializableGeeDualKeyMapTestBigBag* _tmp284_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp285_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp286_ = NULL;
		gpointer _tmp287_ = NULL;
		SerializableGeeDualKeyMapTestSmallBag* _tmp288_ = NULL;
		SerializableGeeDualKeyMapTestBallDualMap* _tmp289_ = NULL;
		SerializableGeeDualKeyMapTestBallDualMap* _tmp290_ = NULL;
		gpointer _tmp291_ = NULL;
		SerializableGeeDualKeyMapTestBall* _tmp292_ = NULL;
		const gchar* _tmp293_ = NULL;
		const gchar* _tmp294_ = NULL;
		SerializableGeeDualKeyMapTestBigBag* _tmp295_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp296_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp297_ = NULL;
		gpointer _tmp298_ = NULL;
		SerializableGeeDualKeyMapTestSmallBag* _tmp299_ = NULL;
		SerializableGeeDualKeyMapTestBallDualMap* _tmp300_ = NULL;
		SerializableGeeDualKeyMapTestBallDualMap* _tmp301_ = NULL;
		gpointer _tmp302_ = NULL;
		SerializableGeeDualKeyMapTestBall* _tmp303_ = NULL;
		SerializableGeeDualKeyMapTestBallFill* _tmp304_ = NULL;
		SerializableGeeDualKeyMapTestBallFill* _tmp305_ = NULL;
		SerializableGeeDualKeyMapTestBigBag* _tmp306_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp307_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp308_ = NULL;
		gpointer _tmp309_ = NULL;
		SerializableGeeDualKeyMapTestSmallBag* _tmp310_ = NULL;
		SerializableGeeDualKeyMapTestBallDualMap* _tmp311_ = NULL;
		SerializableGeeDualKeyMapTestBallDualMap* _tmp312_ = NULL;
		gpointer _tmp313_ = NULL;
		SerializableGeeDualKeyMapTestBall* _tmp314_ = NULL;
		SerializableGeeDualKeyMapTestBallFill* _tmp315_ = NULL;
		SerializableGeeDualKeyMapTestBallFill* _tmp316_ = NULL;
		gchar* _tmp317_ = NULL;
		gchar* _tmp318_ = NULL;
		gchar* _tmp319_ = NULL;
		SerializableGeeDualKeyMapTestBigBag* _tmp320_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp321_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp322_ = NULL;
		gpointer _tmp323_ = NULL;
		SerializableGeeDualKeyMapTestSmallBag* _tmp324_ = NULL;
		SerializableGeeDualKeyMapTestBallDualMap* _tmp325_ = NULL;
		SerializableGeeDualKeyMapTestBallDualMap* _tmp326_ = NULL;
		gpointer _tmp327_ = NULL;
		SerializableGeeDualKeyMapTestBall* _tmp328_ = NULL;
		SerializableGeeDualKeyMapTestBallFill* _tmp329_ = NULL;
		SerializableGeeDualKeyMapTestBallFill* _tmp330_ = NULL;
		gchar* _tmp331_ = NULL;
		gchar* _tmp332_ = NULL;
		gchar* _tmp333_ = NULL;
		SerializableGeeDualKeyMapTestBigBag* bag3 = NULL;
		SerializableGeeDualKeyMapTestBigBag* _tmp334_ = NULL;
		SerializableGeeDualKeyMapTestBigBag* _tmp335_ = NULL;
		GXmlGDocument* _tmp336_ = NULL;
		SerializableGeeDualKeyMapTestBigBag* _tmp337_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp338_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp339_ = NULL;
		gint _tmp340_ = 0;
		gint _tmp341_ = 0;
		gboolean _tmp342_ = FALSE;
		SerializableGeeDualKeyMapTestBigBag* _tmp343_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp344_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp345_ = NULL;
		gboolean _tmp346_ = FALSE;
		SerializableGeeDualKeyMapTestBigBag* _tmp347_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp348_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp349_ = NULL;
		gint _tmp350_ = 0;
		gint _tmp351_ = 0;
		SerializableGeeDualKeyMapTestBigBag* _tmp352_ = NULL;
		GXmlGDocument* _tmp353_ = NULL;
		SerializableGeeDualKeyMapTestBigBag* _tmp354_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp355_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp356_ = NULL;
		gboolean _tmp357_ = FALSE;
		SerializableGeeDualKeyMapTestBigBag* bag2 = NULL;
		SerializableGeeDualKeyMapTestBigBag* _tmp358_ = NULL;
		SerializableGeeDualKeyMapTestBigBag* _tmp359_ = NULL;
		GXmlGDocument* _tmp360_ = NULL;
		SerializableGeeDualKeyMapTestBigBag* _tmp361_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp362_ = NULL;
		SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp363_ = NULL;
		gboolean _tmp364_ = FALSE;
		GXmlGDocument* d2 = NULL;
		GXmlGDocument* _tmp365_ = NULL;
		SerializableGeeDualKeyMapTestBigBag* _tmp366_ = NULL;
		GXmlGDocument* _tmp367_ = NULL;
		GXmlNode* _tmp368_ = NULL;
		GXmlNode* _tmp369_ = NULL;
		GXmlGDocument* _tmp370_ = NULL;
		GXmlNode* _tmp371_ = NULL;
		GXmlNode* _tmp372_ = NULL;
		GXmlNode* _tmp373_ = NULL;
		GXmlGDocument* _tmp374_ = NULL;
		GXmlNode* _tmp375_ = NULL;
		GXmlNode* _tmp376_ = NULL;
		GXmlNode* _tmp377_ = NULL;
		gchar* _tmp378_ = NULL;
		gchar* _tmp379_ = NULL;
		gchar* _tmp380_ = NULL;
		GXmlGDocument* _tmp381_ = NULL;
		GXmlNode* _tmp382_ = NULL;
		GXmlNode* _tmp383_ = NULL;
		GXmlNode* _tmp384_ = NULL;
		GeeBidirList* _tmp385_ = NULL;
		GeeBidirList* _tmp386_ = NULL;
		GeeBidirList* _tmp387_ = NULL;
		gint _tmp388_ = 0;
		gint _tmp389_ = 0;
		GXmlGDocument* _tmp390_ = NULL;
		GXmlNode* _tmp391_ = NULL;
		GXmlNode* _tmp392_ = NULL;
		GXmlNode* _tmp393_ = NULL;
		GeeBidirList* _tmp394_ = NULL;
		GeeBidirList* _tmp395_ = NULL;
		GeeBidirList* _tmp396_ = NULL;
		gpointer _tmp397_ = NULL;
		GXmlNode* _tmp398_ = NULL;
		gchar* _tmp399_ = NULL;
		gchar* _tmp400_ = NULL;
		gchar* _tmp401_ = NULL;
		GXmlGDocument* _tmp402_ = NULL;
		GXmlNode* _tmp403_ = NULL;
		GXmlNode* _tmp404_ = NULL;
		GXmlNode* _tmp405_ = NULL;
		GeeBidirList* _tmp406_ = NULL;
		GeeBidirList* _tmp407_ = NULL;
		GeeBidirList* _tmp408_ = NULL;
		gpointer _tmp409_ = NULL;
		GXmlNode* _tmp410_ = NULL;
		GeeBidirList* _tmp411_ = NULL;
		GeeBidirList* _tmp412_ = NULL;
		GeeBidirList* _tmp413_ = NULL;
		gint _tmp414_ = 0;
		gint _tmp415_ = 0;
		GXmlGDocument* _tmp416_ = NULL;
		GXmlNode* _tmp417_ = NULL;
		GXmlNode* _tmp418_ = NULL;
		GXmlNode* _tmp419_ = NULL;
		GeeBidirList* _tmp420_ = NULL;
		GeeBidirList* _tmp421_ = NULL;
		GeeBidirList* _tmp422_ = NULL;
		gpointer _tmp423_ = NULL;
		GXmlNode* _tmp424_ = NULL;
		GeeBidirList* _tmp425_ = NULL;
		GeeBidirList* _tmp426_ = NULL;
		GeeBidirList* _tmp427_ = NULL;
		gpointer _tmp428_ = NULL;
		GXmlNode* _tmp429_ = NULL;
		gchar* _tmp430_ = NULL;
		gchar* _tmp431_ = NULL;
		gchar* _tmp432_ = NULL;
		GXmlGDocument* _tmp433_ = NULL;
		GXmlNode* _tmp434_ = NULL;
		GXmlNode* _tmp435_ = NULL;
		GXmlNode* _tmp436_ = NULL;
		GeeBidirList* _tmp437_ = NULL;
		GeeBidirList* _tmp438_ = NULL;
		GeeBidirList* _tmp439_ = NULL;
		gpointer _tmp440_ = NULL;
		GXmlNode* _tmp441_ = NULL;
		GeeBidirList* _tmp442_ = NULL;
		GeeBidirList* _tmp443_ = NULL;
		GeeBidirList* _tmp444_ = NULL;
		gpointer _tmp445_ = NULL;
		GXmlNode* _tmp446_ = NULL;
		GeeBidirList* _tmp447_ = NULL;
		GeeBidirList* _tmp448_ = NULL;
		GeeBidirList* _tmp449_ = NULL;
		gint _tmp450_ = 0;
		gint _tmp451_ = 0;
		GXmlGDocument* _tmp452_ = NULL;
		GXmlNode* _tmp453_ = NULL;
		GXmlNode* _tmp454_ = NULL;
		GXmlNode* _tmp455_ = NULL;
		GeeBidirList* _tmp456_ = NULL;
		GeeBidirList* _tmp457_ = NULL;
		GeeBidirList* _tmp458_ = NULL;
		gpointer _tmp459_ = NULL;
		GXmlNode* _tmp460_ = NULL;
		GeeBidirList* _tmp461_ = NULL;
		GeeBidirList* _tmp462_ = NULL;
		GeeBidirList* _tmp463_ = NULL;
		gpointer _tmp464_ = NULL;
		GXmlNode* _tmp465_ = NULL;
		GeeBidirList* _tmp466_ = NULL;
		GeeBidirList* _tmp467_ = NULL;
		GeeBidirList* _tmp468_ = NULL;
		gpointer _tmp469_ = NULL;
		GXmlNode* _tmp470_ = NULL;
		gchar* _tmp471_ = NULL;
		gchar* _tmp472_ = NULL;
		gchar* _tmp473_ = NULL;
		GXmlGDocument* _tmp474_ = NULL;
		GXmlNode* _tmp475_ = NULL;
		GXmlNode* _tmp476_ = NULL;
		GXmlNode* _tmp477_ = NULL;
		GeeBidirList* _tmp478_ = NULL;
		GeeBidirList* _tmp479_ = NULL;
		GeeBidirList* _tmp480_ = NULL;
		gpointer _tmp481_ = NULL;
		GXmlNode* _tmp482_ = NULL;
		GeeBidirList* _tmp483_ = NULL;
		GeeBidirList* _tmp484_ = NULL;
		GeeBidirList* _tmp485_ = NULL;
		gpointer _tmp486_ = NULL;
		GXmlNode* _tmp487_ = NULL;
		GeeBidirList* _tmp488_ = NULL;
		GeeBidirList* _tmp489_ = NULL;
		GeeBidirList* _tmp490_ = NULL;
		gpointer _tmp491_ = NULL;
		GXmlNode* _tmp492_ = NULL;
		GeeBidirList* _tmp493_ = NULL;
		GeeBidirList* _tmp494_ = NULL;
		GeeBidirList* _tmp495_ = NULL;
		gint _tmp496_ = 0;
		gint _tmp497_ = 0;
		GXmlGDocument* _tmp498_ = NULL;
		GXmlNode* _tmp499_ = NULL;
		GXmlNode* _tmp500_ = NULL;
		GXmlNode* _tmp501_ = NULL;
		GeeBidirList* _tmp502_ = NULL;
		GeeBidirList* _tmp503_ = NULL;
		GeeBidirList* _tmp504_ = NULL;
		gpointer _tmp505_ = NULL;
		GXmlNode* _tmp506_ = NULL;
		GeeBidirList* _tmp507_ = NULL;
		GeeBidirList* _tmp508_ = NULL;
		GeeBidirList* _tmp509_ = NULL;
		gpointer _tmp510_ = NULL;
		GXmlNode* _tmp511_ = NULL;
		GeeBidirList* _tmp512_ = NULL;
		GeeBidirList* _tmp513_ = NULL;
		GeeBidirList* _tmp514_ = NULL;
		gpointer _tmp515_ = NULL;
		GXmlNode* _tmp516_ = NULL;
		GeeBidirList* _tmp517_ = NULL;
		GeeBidirList* _tmp518_ = NULL;
		GeeBidirList* _tmp519_ = NULL;
		gpointer _tmp520_ = NULL;
		GXmlNode* _tmp521_ = NULL;
		GXmlGDocument* _tmp522_ = NULL;
		GXmlNode* _tmp523_ = NULL;
		GXmlNode* _tmp524_ = NULL;
		GXmlNode* _tmp525_ = NULL;
		GeeBidirList* _tmp526_ = NULL;
		GeeBidirList* _tmp527_ = NULL;
		GeeBidirList* _tmp528_ = NULL;
		gpointer _tmp529_ = NULL;
		GXmlNode* _tmp530_ = NULL;
		GeeBidirList* _tmp531_ = NULL;
		GeeBidirList* _tmp532_ = NULL;
		GeeBidirList* _tmp533_ = NULL;
		gpointer _tmp534_ = NULL;
		GXmlNode* _tmp535_ = NULL;
		GeeBidirList* _tmp536_ = NULL;
		GeeBidirList* _tmp537_ = NULL;
		GeeBidirList* _tmp538_ = NULL;
		gpointer _tmp539_ = NULL;
		GXmlNode* _tmp540_ = NULL;
		GeeBidirList* _tmp541_ = NULL;
		GeeBidirList* _tmp542_ = NULL;
		GeeBidirList* _tmp543_ = NULL;
		gpointer _tmp544_ = NULL;
		GXmlNode* _tmp545_ = NULL;
		gchar* _tmp546_ = NULL;
		gchar* _tmp547_ = NULL;
		gchar* _tmp548_ = NULL;
#line 341 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp0_ = serializable_gee_dual_key_map_test_big_bag_new ();
#line 341 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		bag = _tmp0_;
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp1_ = bag;
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp2_ = serializable_gee_dual_key_map_test_big_bag_get_bags (_tmp1_);
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp3_ = _tmp2_;
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp3_ != NULL, "bag.bags != null");
#line 4099 "SerializableGeeDualKeyMapTest.c"
		{
			gint i = 0;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			i = 0;
#line 4104 "SerializableGeeDualKeyMapTest.c"
			{
				gboolean _tmp4_ = FALSE;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				_tmp4_ = TRUE;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
				while (TRUE) {
#line 4111 "SerializableGeeDualKeyMapTest.c"
					gint _tmp6_ = 0;
					SerializableGeeDualKeyMapTestSmallBag* sbag = NULL;
					SerializableGeeDualKeyMapTestSmallBag* _tmp7_ = NULL;
					SerializableGeeDualKeyMapTestSmallBag* _tmp8_ = NULL;
					gint _tmp9_ = 0;
					gchar* _tmp10_ = NULL;
					gchar* _tmp11_ = NULL;
					gchar* _tmp12_ = NULL;
					gchar* _tmp13_ = NULL;
					SerializableGeeDualKeyMapTestSmallBag* _tmp14_ = NULL;
					SerializableGeeDualKeyMapTestBallDualMap* _tmp15_ = NULL;
					SerializableGeeDualKeyMapTestBallDualMap* _tmp16_ = NULL;
					SerializableGeeDualKeyMapTestSmallBag* _tmp43_ = NULL;
					SerializableGeeDualKeyMapTestBallDualMap* _tmp44_ = NULL;
					SerializableGeeDualKeyMapTestBallDualMap* _tmp45_ = NULL;
					gint _tmp46_ = 0;
					gint _tmp47_ = 0;
					SerializableGeeDualKeyMapTestBigBag* _tmp48_ = NULL;
					SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp49_ = NULL;
					SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp50_ = NULL;
					SerializableGeeDualKeyMapTestSmallBag* _tmp51_ = NULL;
					const gchar* _tmp52_ = NULL;
					const gchar* _tmp53_ = NULL;
					SerializableGeeDualKeyMapTestSmallBag* _tmp54_ = NULL;
					const gchar* _tmp55_ = NULL;
					const gchar* _tmp56_ = NULL;
					SerializableGeeDualKeyMapTestSmallBag* _tmp57_ = NULL;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					if (!_tmp4_) {
#line 4141 "SerializableGeeDualKeyMapTest.c"
						gint _tmp5_ = 0;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						_tmp5_ = i;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						i = _tmp5_ + 1;
#line 4147 "SerializableGeeDualKeyMapTest.c"
					}
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp4_ = FALSE;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp6_ = i;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					if (!(_tmp6_ < 2)) {
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						break;
#line 4157 "SerializableGeeDualKeyMapTest.c"
					}
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp7_ = serializable_gee_dual_key_map_test_small_bag_new ();
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					sbag = _tmp7_;
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp8_ = sbag;
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp9_ = i;
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp10_ = g_strdup_printf ("%i", _tmp9_);
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp11_ = _tmp10_;
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp12_ = g_strconcat ("SmallBag", _tmp11_, NULL);
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp13_ = _tmp12_;
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					serializable_gee_dual_key_map_test_small_bag_set_name (_tmp8_, _tmp13_);
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_g_free0 (_tmp13_);
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_g_free0 (_tmp11_);
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp14_ = sbag;
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp15_ = serializable_gee_dual_key_map_test_small_bag_get_balls (_tmp14_);
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp16_ = _tmp15_;
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_vala_assert (_tmp16_ != NULL, "sbag.balls != null");
#line 4189 "SerializableGeeDualKeyMapTest.c"
					{
						gint j = 0;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
						j = 0;
#line 4194 "SerializableGeeDualKeyMapTest.c"
						{
							gboolean _tmp17_ = FALSE;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							_tmp17_ = TRUE;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
							while (TRUE) {
#line 4201 "SerializableGeeDualKeyMapTest.c"
								gint _tmp19_ = 0;
								SerializableGeeDualKeyMapTestBall* b = NULL;
								SerializableGeeDualKeyMapTestBall* _tmp20_ = NULL;
								SerializableGeeDualKeyMapTestBall* _tmp21_ = NULL;
								gint _tmp22_ = 0;
								gchar* _tmp23_ = NULL;
								gchar* _tmp24_ = NULL;
								gchar* _tmp25_ = NULL;
								gchar* _tmp26_ = NULL;
								SerializableGeeDualKeyMapTestBall* _tmp27_ = NULL;
								SerializableGeeDualKeyMapTestBallFill* _tmp28_ = NULL;
								SerializableGeeDualKeyMapTestBallFill* _tmp29_ = NULL;
								SerializableGeeDualKeyMapTestBall* _tmp30_ = NULL;
								SerializableGeeDualKeyMapTestBallFill* _tmp31_ = NULL;
								SerializableGeeDualKeyMapTestBallFill* _tmp32_ = NULL;
								SerializableGeeDualKeyMapTestSmallBag* _tmp33_ = NULL;
								SerializableGeeDualKeyMapTestBallDualMap* _tmp34_ = NULL;
								SerializableGeeDualKeyMapTestBallDualMap* _tmp35_ = NULL;
								SerializableGeeDualKeyMapTestBall* _tmp36_ = NULL;
								const gchar* _tmp37_ = NULL;
								const gchar* _tmp38_ = NULL;
								SerializableGeeDualKeyMapTestBall* _tmp39_ = NULL;
								const gchar* _tmp40_ = NULL;
								const gchar* _tmp41_ = NULL;
								SerializableGeeDualKeyMapTestBall* _tmp42_ = NULL;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								if (!_tmp17_) {
#line 4229 "SerializableGeeDualKeyMapTest.c"
									gint _tmp18_ = 0;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
									_tmp18_ = j;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
									j = _tmp18_ + 1;
#line 4235 "SerializableGeeDualKeyMapTest.c"
								}
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp17_ = FALSE;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp19_ = j;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								if (!(_tmp19_ < 2)) {
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
									break;
#line 4245 "SerializableGeeDualKeyMapTest.c"
								}
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp20_ = serializable_gee_dual_key_map_test_ball_new ();
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								b = _tmp20_;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp21_ = b;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp22_ = j;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp23_ = g_strdup_printf ("%i", _tmp22_);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp24_ = _tmp23_;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp25_ = g_strconcat ("Ball", _tmp24_, NULL);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp26_ = _tmp25_;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								serializable_gee_dual_key_map_test_ball_set_name (_tmp21_, _tmp26_);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_g_free0 (_tmp26_);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_g_free0 (_tmp24_);
#line 350 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp27_ = b;
#line 350 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp28_ = serializable_gee_dual_key_map_test_ball_fill_new ();
#line 350 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp29_ = _tmp28_;
#line 350 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								serializable_gee_dual_key_map_test_ball_set_ballfill (_tmp27_, _tmp29_);
#line 350 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_g_object_unref0 (_tmp29_);
#line 351 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp30_ = b;
#line 351 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp31_ = serializable_gee_dual_key_map_test_ball_get_ballfill (_tmp30_);
#line 351 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp32_ = _tmp31_;
#line 351 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								serializable_gee_dual_key_map_test_ball_fill_set_text (_tmp32_, "golden dust");
#line 352 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp33_ = sbag;
#line 352 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp34_ = serializable_gee_dual_key_map_test_small_bag_get_balls (_tmp33_);
#line 352 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp35_ = _tmp34_;
#line 352 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp36_ = b;
#line 352 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp37_ = serializable_gee_dual_key_map_test_ball_get_size (_tmp36_);
#line 352 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp38_ = _tmp37_;
#line 352 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp39_ = b;
#line 352 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp40_ = serializable_gee_dual_key_map_test_ball_get_name (_tmp39_);
#line 352 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp41_ = _tmp40_;
#line 352 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_tmp42_ = b;
#line 352 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								gxml_serializable_dual_key_map_set ((GXmlSerializableDualKeyMap*) _tmp35_, _tmp38_, _tmp41_, _tmp42_);
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
								_g_object_unref0 (b);
#line 4311 "SerializableGeeDualKeyMapTest.c"
							}
						}
					}
#line 354 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp43_ = sbag;
#line 354 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp44_ = serializable_gee_dual_key_map_test_small_bag_get_balls (_tmp43_);
#line 354 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp45_ = _tmp44_;
#line 354 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp46_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp45_);
#line 354 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp47_ = _tmp46_;
#line 354 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_vala_assert (_tmp47_ == 2, "sbag.balls.size == 2");
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp48_ = bag;
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp49_ = serializable_gee_dual_key_map_test_big_bag_get_bags (_tmp48_);
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp50_ = _tmp49_;
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp51_ = sbag;
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp52_ = serializable_gee_dual_key_map_test_small_bag_get_cash (_tmp51_);
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp53_ = _tmp52_;
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp54_ = sbag;
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp55_ = serializable_gee_dual_key_map_test_small_bag_get_name (_tmp54_);
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp56_ = _tmp55_;
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_tmp57_ = sbag;
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					gxml_serializable_dual_key_map_set ((GXmlSerializableDualKeyMap*) _tmp50_, _tmp53_, _tmp56_, _tmp57_);
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
					_g_object_unref0 (sbag);
#line 4351 "SerializableGeeDualKeyMapTest.c"
				}
			}
		}
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp58_ = bag;
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp59_ = serializable_gee_dual_key_map_test_big_bag_get_bags (_tmp58_);
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp60_ = _tmp59_;
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp61_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp60_);
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp62_ = _tmp61_;
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp62_ == 2, "bag.bags.size == 2");
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp63_ = gxml_gdocument_new ();
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		d = _tmp63_;
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp64_ = bag;
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp65_ = d;
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp66_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) _tmp64_, (GXmlNode*) _tmp65_, &_inner_error_);
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp67_ = _tmp66_;
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp67_);
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (d);
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bag);
#line 4387 "SerializableGeeDualKeyMapTest.c"
			goto __catch81_g_error;
		}
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp68_ = d;
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp69_ = gxml_document_get_root ((GXmlDocument*) _tmp68_);
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp70_ = _tmp69_;
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp71_ = _tmp70_;
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp71_ != NULL, "d.root != null");
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp71_);
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp72_ = d;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp73_ = gxml_document_get_root ((GXmlDocument*) _tmp72_);
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp74_ = _tmp73_;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp75_ = _tmp74_;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp76_ = gxml_node_get_name (_tmp75_);
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp77_ = _tmp76_;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp78_ = _tmp77_;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp78_, "BigBag") == 0, "d.root.name == \"BigBag\"");
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_free0 (_tmp78_);
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp75_);
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp79_ = d;
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp80_ = gxml_document_get_root ((GXmlDocument*) _tmp79_);
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp81_ = _tmp80_;
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp82_ = _tmp81_;
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp83_ = gxml_node_get_children_nodes (_tmp82_);
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp84_ = _tmp83_;
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp85_ = _tmp84_;
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp86_ = gee_collection_get_size ((GeeCollection*) _tmp85_);
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp87_ = _tmp86_;
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp87_ == 2, "d.root.children_nodes.size == 2");
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp85_);
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp82_);
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp88_ = d;
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp89_ = gxml_document_get_root ((GXmlDocument*) _tmp88_);
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp90_ = _tmp89_;
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp91_ = _tmp90_;
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp92_ = gxml_node_get_children_nodes (_tmp91_);
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp93_ = _tmp92_;
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp94_ = _tmp93_;
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp95_ = gee_list_get ((GeeList*) _tmp94_, 0);
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp96_ = (GXmlNode*) _tmp95_;
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp97_ = gxml_node_get_name (_tmp96_);
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp98_ = _tmp97_;
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp99_ = _tmp98_;
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp99_, "SmallBag") == 0, "d.root.children_nodes[0].name == \"SmallBag\"");
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_free0 (_tmp99_);
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp96_);
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp94_);
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp91_);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp100_ = d;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp101_ = gxml_document_get_root ((GXmlDocument*) _tmp100_);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp102_ = _tmp101_;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp103_ = _tmp102_;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp104_ = gxml_node_get_children_nodes (_tmp103_);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp105_ = _tmp104_;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp106_ = _tmp105_;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp107_ = gee_list_get ((GeeList*) _tmp106_, 0);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp108_ = (GXmlNode*) _tmp107_;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp109_ = gxml_node_get_children_nodes (_tmp108_);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp110_ = _tmp109_;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp111_ = _tmp110_;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp112_ = gee_collection_get_size ((GeeCollection*) _tmp111_);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp113_ = _tmp112_;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp113_ == 2, "d.root.children_nodes[0].children_nodes.size == 2");
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp111_);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp108_);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp106_);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp103_);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp114_ = d;
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp115_ = gxml_document_get_root ((GXmlDocument*) _tmp114_);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp116_ = _tmp115_;
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp117_ = _tmp116_;
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp118_ = gxml_node_get_children_nodes (_tmp117_);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp119_ = _tmp118_;
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp120_ = _tmp119_;
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp121_ = gee_list_get ((GeeList*) _tmp120_, 0);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp122_ = (GXmlNode*) _tmp121_;
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp123_ = gxml_node_get_children_nodes (_tmp122_);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp124_ = _tmp123_;
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp125_ = _tmp124_;
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp126_ = gee_list_get ((GeeList*) _tmp125_, 0);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp127_ = (GXmlNode*) _tmp126_;
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp128_ = gxml_node_get_name (_tmp127_);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp129_ = _tmp128_;
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp130_ = _tmp129_;
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp130_, "Ball") == 0, "d.root.children_nodes[0].children_nodes[0].name == \"Ball\"");
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_free0 (_tmp130_);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp127_);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp125_);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp122_);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp120_);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp117_);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp131_ = d;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp132_ = gxml_document_get_root ((GXmlDocument*) _tmp131_);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp133_ = _tmp132_;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp134_ = _tmp133_;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp135_ = gxml_node_get_children_nodes (_tmp134_);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp136_ = _tmp135_;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp137_ = _tmp136_;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp138_ = gee_list_get ((GeeList*) _tmp137_, 0);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp139_ = (GXmlNode*) _tmp138_;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp140_ = gxml_node_get_children_nodes (_tmp139_);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp141_ = _tmp140_;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp142_ = _tmp141_;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp143_ = gee_list_get ((GeeList*) _tmp142_, 0);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp144_ = (GXmlNode*) _tmp143_;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp145_ = gxml_node_get_children_nodes (_tmp144_);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp146_ = _tmp145_;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp147_ = _tmp146_;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp148_ = gee_collection_get_size ((GeeCollection*) _tmp147_);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp149_ = _tmp148_;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp149_ == 1, "d.root.children_nodes[0].children_nodes[0].children_nodes.size == 1");
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp147_);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp144_);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp142_);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp139_);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp137_);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp134_);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp150_ = d;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp151_ = gxml_document_get_root ((GXmlDocument*) _tmp150_);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp152_ = _tmp151_;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp153_ = _tmp152_;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp154_ = gxml_node_get_children_nodes (_tmp153_);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp155_ = _tmp154_;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp156_ = _tmp155_;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp157_ = gee_list_get ((GeeList*) _tmp156_, 0);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp158_ = (GXmlNode*) _tmp157_;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp159_ = gxml_node_get_children_nodes (_tmp158_);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp160_ = _tmp159_;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp161_ = _tmp160_;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp162_ = gee_list_get ((GeeList*) _tmp161_, 0);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp163_ = (GXmlNode*) _tmp162_;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp164_ = gxml_node_get_children_nodes (_tmp163_);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp165_ = _tmp164_;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp166_ = _tmp165_;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp167_ = gee_list_get ((GeeList*) _tmp166_, 0);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp168_ = (GXmlNode*) _tmp167_;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp169_ = gxml_node_get_name (_tmp168_);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp170_ = _tmp169_;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp171_ = _tmp170_;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp171_, "BallFill") == 0, "d.root.children_nodes[0].children_nodes[0].children_nodes[0].name == \"BallFill\"");
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_free0 (_tmp171_);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp168_);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp166_);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp163_);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp161_);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp158_);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp156_);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp153_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp172_ = d;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp173_ = gxml_document_get_root ((GXmlDocument*) _tmp172_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp174_ = _tmp173_;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp175_ = _tmp174_;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp176_ = gxml_node_get_children_nodes (_tmp175_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp177_ = _tmp176_;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp178_ = _tmp177_;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp179_ = gee_list_get ((GeeList*) _tmp178_, 0);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp180_ = (GXmlNode*) _tmp179_;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp181_ = gxml_node_get_children_nodes (_tmp180_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp182_ = _tmp181_;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp183_ = _tmp182_;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp184_ = gee_list_get ((GeeList*) _tmp183_, 0);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp185_ = (GXmlNode*) _tmp184_;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp186_ = gxml_node_get_children_nodes (_tmp185_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp187_ = _tmp186_;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp188_ = _tmp187_;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp189_ = gee_list_get ((GeeList*) _tmp188_, 0);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp190_ = (GXmlNode*) _tmp189_;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp191_ = gxml_node_get_children_nodes (_tmp190_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp192_ = _tmp191_;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp193_ = _tmp192_;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp194_ = gee_collection_get_size ((GeeCollection*) _tmp193_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp195_ = _tmp194_;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp195_ == 1, "d.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes.size == 1");
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp193_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp190_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp188_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp185_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp183_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp180_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp178_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp175_);
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp196_ = d;
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp197_ = gxml_document_get_root ((GXmlDocument*) _tmp196_);
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp198_ = _tmp197_;
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp199_ = _tmp198_;
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp200_ = gxml_node_get_children_nodes (_tmp199_);
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp201_ = _tmp200_;
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp202_ = _tmp201_;
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp203_ = gee_list_get ((GeeList*) _tmp202_, 0);
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp204_ = (GXmlNode*) _tmp203_;
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp205_ = gxml_node_get_children_nodes (_tmp204_);
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp206_ = _tmp205_;
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp207_ = _tmp206_;
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp208_ = gee_list_get ((GeeList*) _tmp207_, 0);
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp209_ = (GXmlNode*) _tmp208_;
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp210_ = gxml_node_get_children_nodes (_tmp209_);
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp211_ = _tmp210_;
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp212_ = _tmp211_;
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp213_ = gee_list_get ((GeeList*) _tmp212_, 0);
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp214_ = (GXmlNode*) _tmp213_;
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp215_ = gxml_node_get_children_nodes (_tmp214_);
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp216_ = _tmp215_;
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp217_ = _tmp216_;
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp218_ = gee_list_get ((GeeList*) _tmp217_, 0);
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp219_ = (GXmlNode*) _tmp218_;
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp219_, GXML_TYPE_TEXT), "d.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes[0] is Text");
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp219_);
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp217_);
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp214_);
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp212_);
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp209_);
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp207_);
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp204_);
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp202_);
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp199_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp220_ = d;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp221_ = gxml_document_get_root ((GXmlDocument*) _tmp220_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp222_ = _tmp221_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp223_ = _tmp222_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp224_ = gxml_node_get_children_nodes (_tmp223_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp225_ = _tmp224_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp226_ = _tmp225_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp227_ = gee_list_get ((GeeList*) _tmp226_, 0);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp228_ = (GXmlNode*) _tmp227_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp229_ = gxml_node_get_children_nodes (_tmp228_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp230_ = _tmp229_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp231_ = _tmp230_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp232_ = gee_list_get ((GeeList*) _tmp231_, 0);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp233_ = (GXmlNode*) _tmp232_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp234_ = gxml_node_get_children_nodes (_tmp233_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp235_ = _tmp234_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp236_ = _tmp235_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp237_ = gee_list_get ((GeeList*) _tmp236_, 0);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp238_ = (GXmlNode*) _tmp237_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp239_ = gxml_node_get_children_nodes (_tmp238_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp240_ = _tmp239_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp241_ = _tmp240_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp242_ = gee_list_get ((GeeList*) _tmp241_, 0);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp243_ = (GXmlNode*) _tmp242_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp244_ = gxml_node_get_value (_tmp243_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp245_ = _tmp244_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp246_ = _tmp245_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp246_, "golden dust") == 0, "d.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes[0].value == \"golden dust\"");
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_free0 (_tmp246_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp243_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp241_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp238_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp236_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp233_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp231_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp228_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp226_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp223_);
#line 374 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp247_ = serializable_gee_dual_key_map_test_big_bag_new ();
#line 374 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		bagt = _tmp247_;
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp248_ = bagt;
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp249_ = d;
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp248_, (GXmlNode*) _tmp249_, &_inner_error_);
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bagt);
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (d);
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bag);
#line 4908 "SerializableGeeDualKeyMapTest.c"
			goto __catch81_g_error;
		}
#line 376 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp250_ = bagt;
#line 376 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp251_ = serializable_gee_dual_key_map_test_big_bag_get_bags (_tmp250_);
#line 376 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp252_ = _tmp251_;
#line 376 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp253_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp252_);
#line 376 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp254_ = _tmp253_;
#line 376 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp254_ == 0, "bagt.bags.size == 0");
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp256_ = bagt;
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp257_ = serializable_gee_dual_key_map_test_big_bag_get_bags (_tmp256_);
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp258_ = _tmp257_;
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp259_ = gxml_serializable_dual_key_map_deserialize_children ((GXmlSerializableDualKeyMap*) _tmp258_, &_inner_error_);
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp255_ = _tmp259_;
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bagt);
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (d);
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bag);
#line 4941 "SerializableGeeDualKeyMapTest.c"
			goto __catch81_g_error;
		}
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp255_, "bagt.bags.deserialize_children ()");
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp260_ = bagt;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp261_ = serializable_gee_dual_key_map_test_big_bag_get_bags (_tmp260_);
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp262_ = _tmp261_;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp263_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp262_);
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp264_ = _tmp263_;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp264_ == 2, "bagt.bags.size == 2");
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp266_ = bagt;
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp267_ = serializable_gee_dual_key_map_test_big_bag_get_bags (_tmp266_);
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp268_ = _tmp267_;
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp269_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp268_, "money", "SmallBag1");
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp270_ = (SerializableGeeDualKeyMapTestSmallBag*) _tmp269_;
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp271_ = serializable_gee_dual_key_map_test_small_bag_get_balls (_tmp270_);
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp272_ = _tmp271_;
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp273_ = gxml_serializable_dual_key_map_deserialize_children ((GXmlSerializableDualKeyMap*) _tmp272_, &_inner_error_);
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp274_ = _tmp273_;
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp270_);
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp265_ = _tmp274_;
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bagt);
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (d);
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bag);
#line 4988 "SerializableGeeDualKeyMapTest.c"
			goto __catch81_g_error;
		}
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp265_, "bagt.bags.get(\"money\",\"SmallBag1\").balls.deserialize_children ()");
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp275_ = bagt;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp276_ = serializable_gee_dual_key_map_test_big_bag_get_bags (_tmp275_);
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp277_ = _tmp276_;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp278_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp277_, "money", "SmallBag1");
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp279_ = (SerializableGeeDualKeyMapTestSmallBag*) _tmp278_;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp280_ = serializable_gee_dual_key_map_test_small_bag_get_balls (_tmp279_);
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp281_ = _tmp280_;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp282_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp281_);
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp283_ = _tmp282_;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp283_ == 2, "bagt.bags.get(\"money\",\"SmallBag1\").balls.size == 2");
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp279_);
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp284_ = bagt;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp285_ = serializable_gee_dual_key_map_test_big_bag_get_bags (_tmp284_);
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp286_ = _tmp285_;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp287_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp286_, "money", "SmallBag1");
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp288_ = (SerializableGeeDualKeyMapTestSmallBag*) _tmp287_;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp289_ = serializable_gee_dual_key_map_test_small_bag_get_balls (_tmp288_);
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp290_ = _tmp289_;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp291_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp290_, "medium", "Ball1");
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp292_ = (SerializableGeeDualKeyMapTestBall*) _tmp291_;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp293_ = serializable_gee_dual_key_map_test_ball_get_name (_tmp292_);
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp294_ = _tmp293_;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp294_, "Ball1") == 0, "bagt.bags.get(\"money\",\"SmallBag1\").balls.get(\"medium\",\"Ball1\").name == \"Ball1\"");
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp292_);
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp288_);
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp295_ = bagt;
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp296_ = serializable_gee_dual_key_map_test_big_bag_get_bags (_tmp295_);
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp297_ = _tmp296_;
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp298_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp297_, "money", "SmallBag1");
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp299_ = (SerializableGeeDualKeyMapTestSmallBag*) _tmp298_;
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp300_ = serializable_gee_dual_key_map_test_small_bag_get_balls (_tmp299_);
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp301_ = _tmp300_;
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp302_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp301_, "medium", "Ball1");
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp303_ = (SerializableGeeDualKeyMapTestBall*) _tmp302_;
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp304_ = serializable_gee_dual_key_map_test_ball_get_ballfill (_tmp303_);
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp305_ = _tmp304_;
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp305_ != NULL, "bagt.bags.get(\"money\",\"SmallBag1\").balls.get(\"medium\",\"Ball1\").ballfill !=null");
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp303_);
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp299_);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp306_ = bagt;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp307_ = serializable_gee_dual_key_map_test_big_bag_get_bags (_tmp306_);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp308_ = _tmp307_;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp309_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp308_, "money", "SmallBag1");
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp310_ = (SerializableGeeDualKeyMapTestSmallBag*) _tmp309_;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp311_ = serializable_gee_dual_key_map_test_small_bag_get_balls (_tmp310_);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp312_ = _tmp311_;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp313_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp312_, "medium", "Ball1");
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp314_ = (SerializableGeeDualKeyMapTestBall*) _tmp313_;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp315_ = serializable_gee_dual_key_map_test_ball_get_ballfill (_tmp314_);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp316_ = _tmp315_;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp317_ = gxml_serializable_get_serialized_xml_node_value ((GXmlSerializable*) _tmp316_);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp318_ = _tmp317_;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp319_ = _tmp318_;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp319_ != NULL, "bagt.bags.get(\"money\",\"SmallBag1\").balls.get(\"medium\",\"Ball1\").ballfill.serialized_xml_node_value !=null");
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_free0 (_tmp319_);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp314_);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp310_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp320_ = bagt;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp321_ = serializable_gee_dual_key_map_test_big_bag_get_bags (_tmp320_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp322_ = _tmp321_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp323_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp322_, "money", "SmallBag1");
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp324_ = (SerializableGeeDualKeyMapTestSmallBag*) _tmp323_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp325_ = serializable_gee_dual_key_map_test_small_bag_get_balls (_tmp324_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp326_ = _tmp325_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp327_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp326_, "medium", "Ball1");
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp328_ = (SerializableGeeDualKeyMapTestBall*) _tmp327_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp329_ = serializable_gee_dual_key_map_test_ball_get_ballfill (_tmp328_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp330_ = _tmp329_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp331_ = gxml_serializable_get_serialized_xml_node_value ((GXmlSerializable*) _tmp330_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp332_ = _tmp331_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp333_ = _tmp332_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp333_, "golden dust") == 0, "bagt.bags.get(\"money\",\"SmallBag1\").balls.get(\"medium\",\"Ball1\").ballfill.serialized_xml_node_value ==\"golden dust\"");
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_free0 (_tmp333_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp328_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp324_);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp334_ = serializable_gee_dual_key_map_test_big_bag_new ();
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		bag3 = _tmp334_;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp335_ = bag3;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp336_ = d;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp335_, (GXmlNode*) _tmp336_, &_inner_error_);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bag3);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bagt);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (d);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bag);
#line 5163 "SerializableGeeDualKeyMapTest.c"
			goto __catch81_g_error;
		}
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp337_ = bag3;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp338_ = serializable_gee_dual_key_map_test_big_bag_get_bags (_tmp337_);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp339_ = _tmp338_;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp340_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp339_);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp341_ = _tmp340_;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp341_ == 0, "bag3.bags.size == 0");
#line 388 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp343_ = bag3;
#line 388 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp344_ = serializable_gee_dual_key_map_test_big_bag_get_bags (_tmp343_);
#line 388 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp345_ = _tmp344_;
#line 388 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp346_ = gxml_serializable_dual_key_map_deserialize_children ((GXmlSerializableDualKeyMap*) _tmp345_, &_inner_error_);
#line 388 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp342_ = _tmp346_;
#line 388 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 388 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bag3);
#line 388 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bagt);
#line 388 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (d);
#line 388 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bag);
#line 5198 "SerializableGeeDualKeyMapTest.c"
			goto __catch81_g_error;
		}
#line 388 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp342_, "bag3.bags.deserialize_children ()");
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp347_ = bag3;
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp348_ = serializable_gee_dual_key_map_test_big_bag_get_bags (_tmp347_);
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp349_ = _tmp348_;
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp350_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp349_);
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp351_ = _tmp350_;
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp351_ == 2, "bag3.bags.size == 2");
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp352_ = bag3;
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp353_ = d;
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp352_, (GXmlNode*) _tmp353_, &_inner_error_);
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bag3);
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bagt);
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (d);
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bag);
#line 5231 "SerializableGeeDualKeyMapTest.c"
			goto __catch81_g_error;
		}
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp354_ = bag3;
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp355_ = serializable_gee_dual_key_map_test_big_bag_get_bags (_tmp354_);
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp356_ = _tmp355_;
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp357_ = gxml_serializable_dual_key_map_deserialized ((GXmlSerializableDualKeyMap*) _tmp356_);
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (!_tmp357_, "!bag3.bags.deserialized ()");
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp358_ = serializable_gee_dual_key_map_test_big_bag_new ();
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		bag2 = _tmp358_;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp359_ = bag2;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp360_ = d;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp359_, (GXmlNode*) _tmp360_, &_inner_error_);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bag2);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bag3);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bagt);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (d);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bag);
#line 5266 "SerializableGeeDualKeyMapTest.c"
			goto __catch81_g_error;
		}
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp361_ = bag2;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp362_ = serializable_gee_dual_key_map_test_big_bag_get_bags (_tmp361_);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp363_ = _tmp362_;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp364_ = gxml_serializable_dual_key_map_deserialized ((GXmlSerializableDualKeyMap*) _tmp363_);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (!_tmp364_, "!bag2.bags.deserialized ()");
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp365_ = gxml_gdocument_new ();
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		d2 = _tmp365_;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp366_ = bag2;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp367_ = d2;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp368_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) _tmp366_, (GXmlNode*) _tmp367_, &_inner_error_);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp369_ = _tmp368_;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp369_);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (d2);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bag2);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bag3);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bagt);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (d);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
			_g_object_unref0 (bag);
#line 5307 "SerializableGeeDualKeyMapTest.c"
			goto __catch81_g_error;
		}
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp370_ = d2;
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp371_ = gxml_document_get_root ((GXmlDocument*) _tmp370_);
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp372_ = _tmp371_;
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp373_ = _tmp372_;
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp373_ != NULL, "d2.root != null");
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp373_);
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp374_ = d2;
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp375_ = gxml_document_get_root ((GXmlDocument*) _tmp374_);
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp376_ = _tmp375_;
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp377_ = _tmp376_;
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp378_ = gxml_node_get_name (_tmp377_);
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp379_ = _tmp378_;
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp380_ = _tmp379_;
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp380_, "BigBag") == 0, "d2.root.name == \"BigBag\"");
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_free0 (_tmp380_);
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp377_);
#line 401 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp381_ = d2;
#line 401 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp382_ = gxml_document_get_root ((GXmlDocument*) _tmp381_);
#line 401 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp383_ = _tmp382_;
#line 401 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp384_ = _tmp383_;
#line 401 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp385_ = gxml_node_get_children_nodes (_tmp384_);
#line 401 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp386_ = _tmp385_;
#line 401 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp387_ = _tmp386_;
#line 401 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp388_ = gee_collection_get_size ((GeeCollection*) _tmp387_);
#line 401 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp389_ = _tmp388_;
#line 401 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp389_ == 2, "d2.root.children_nodes.size == 2");
#line 401 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp387_);
#line 401 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp384_);
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp390_ = d2;
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp391_ = gxml_document_get_root ((GXmlDocument*) _tmp390_);
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp392_ = _tmp391_;
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp393_ = _tmp392_;
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp394_ = gxml_node_get_children_nodes (_tmp393_);
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp395_ = _tmp394_;
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp396_ = _tmp395_;
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp397_ = gee_list_get ((GeeList*) _tmp396_, 0);
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp398_ = (GXmlNode*) _tmp397_;
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp399_ = gxml_node_get_name (_tmp398_);
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp400_ = _tmp399_;
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp401_ = _tmp400_;
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp401_, "SmallBag") == 0, "d2.root.children_nodes[0].name == \"SmallBag\"");
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_free0 (_tmp401_);
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp398_);
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp396_);
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp393_);
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp402_ = d2;
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp403_ = gxml_document_get_root ((GXmlDocument*) _tmp402_);
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp404_ = _tmp403_;
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp405_ = _tmp404_;
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp406_ = gxml_node_get_children_nodes (_tmp405_);
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp407_ = _tmp406_;
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp408_ = _tmp407_;
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp409_ = gee_list_get ((GeeList*) _tmp408_, 0);
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp410_ = (GXmlNode*) _tmp409_;
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp411_ = gxml_node_get_children_nodes (_tmp410_);
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp412_ = _tmp411_;
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp413_ = _tmp412_;
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp414_ = gee_collection_get_size ((GeeCollection*) _tmp413_);
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp415_ = _tmp414_;
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp415_ == 2, "d2.root.children_nodes[0].children_nodes.size == 2");
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp413_);
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp410_);
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp408_);
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp405_);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp416_ = d2;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp417_ = gxml_document_get_root ((GXmlDocument*) _tmp416_);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp418_ = _tmp417_;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp419_ = _tmp418_;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp420_ = gxml_node_get_children_nodes (_tmp419_);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp421_ = _tmp420_;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp422_ = _tmp421_;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp423_ = gee_list_get ((GeeList*) _tmp422_, 0);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp424_ = (GXmlNode*) _tmp423_;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp425_ = gxml_node_get_children_nodes (_tmp424_);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp426_ = _tmp425_;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp427_ = _tmp426_;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp428_ = gee_list_get ((GeeList*) _tmp427_, 0);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp429_ = (GXmlNode*) _tmp428_;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp430_ = gxml_node_get_name (_tmp429_);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp431_ = _tmp430_;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp432_ = _tmp431_;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp432_, "Ball") == 0, "d2.root.children_nodes[0].children_nodes[0].name == \"Ball\"");
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_free0 (_tmp432_);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp429_);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp427_);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp424_);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp422_);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp419_);
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp433_ = d2;
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp434_ = gxml_document_get_root ((GXmlDocument*) _tmp433_);
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp435_ = _tmp434_;
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp436_ = _tmp435_;
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp437_ = gxml_node_get_children_nodes (_tmp436_);
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp438_ = _tmp437_;
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp439_ = _tmp438_;
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp440_ = gee_list_get ((GeeList*) _tmp439_, 0);
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp441_ = (GXmlNode*) _tmp440_;
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp442_ = gxml_node_get_children_nodes (_tmp441_);
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp443_ = _tmp442_;
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp444_ = _tmp443_;
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp445_ = gee_list_get ((GeeList*) _tmp444_, 0);
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp446_ = (GXmlNode*) _tmp445_;
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp447_ = gxml_node_get_children_nodes (_tmp446_);
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp448_ = _tmp447_;
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp449_ = _tmp448_;
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp450_ = gee_collection_get_size ((GeeCollection*) _tmp449_);
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp451_ = _tmp450_;
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp451_ == 1, "d2.root.children_nodes[0].children_nodes[0].children_nodes.size == 1");
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp449_);
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp446_);
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp444_);
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp441_);
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp439_);
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp436_);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp452_ = d2;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp453_ = gxml_document_get_root ((GXmlDocument*) _tmp452_);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp454_ = _tmp453_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp455_ = _tmp454_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp456_ = gxml_node_get_children_nodes (_tmp455_);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp457_ = _tmp456_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp458_ = _tmp457_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp459_ = gee_list_get ((GeeList*) _tmp458_, 0);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp460_ = (GXmlNode*) _tmp459_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp461_ = gxml_node_get_children_nodes (_tmp460_);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp462_ = _tmp461_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp463_ = _tmp462_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp464_ = gee_list_get ((GeeList*) _tmp463_, 0);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp465_ = (GXmlNode*) _tmp464_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp466_ = gxml_node_get_children_nodes (_tmp465_);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp467_ = _tmp466_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp468_ = _tmp467_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp469_ = gee_list_get ((GeeList*) _tmp468_, 0);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp470_ = (GXmlNode*) _tmp469_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp471_ = gxml_node_get_name (_tmp470_);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp472_ = _tmp471_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp473_ = _tmp472_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp473_, "BallFill") == 0, "d2.root.children_nodes[0].children_nodes[0].children_nodes[0].name == \"BallFill\"");
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_free0 (_tmp473_);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp470_);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp468_);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp465_);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp463_);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp460_);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp458_);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp455_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp474_ = d2;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp475_ = gxml_document_get_root ((GXmlDocument*) _tmp474_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp476_ = _tmp475_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp477_ = _tmp476_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp478_ = gxml_node_get_children_nodes (_tmp477_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp479_ = _tmp478_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp480_ = _tmp479_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp481_ = gee_list_get ((GeeList*) _tmp480_, 0);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp482_ = (GXmlNode*) _tmp481_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp483_ = gxml_node_get_children_nodes (_tmp482_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp484_ = _tmp483_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp485_ = _tmp484_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp486_ = gee_list_get ((GeeList*) _tmp485_, 0);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp487_ = (GXmlNode*) _tmp486_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp488_ = gxml_node_get_children_nodes (_tmp487_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp489_ = _tmp488_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp490_ = _tmp489_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp491_ = gee_list_get ((GeeList*) _tmp490_, 0);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp492_ = (GXmlNode*) _tmp491_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp493_ = gxml_node_get_children_nodes (_tmp492_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp494_ = _tmp493_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp495_ = _tmp494_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp496_ = gee_collection_get_size ((GeeCollection*) _tmp495_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp497_ = _tmp496_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (_tmp497_ == 1, "d2.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes.size == 1");
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp495_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp492_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp490_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp487_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp485_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp482_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp480_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp477_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp498_ = d2;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp499_ = gxml_document_get_root ((GXmlDocument*) _tmp498_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp500_ = _tmp499_;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp501_ = _tmp500_;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp502_ = gxml_node_get_children_nodes (_tmp501_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp503_ = _tmp502_;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp504_ = _tmp503_;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp505_ = gee_list_get ((GeeList*) _tmp504_, 0);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp506_ = (GXmlNode*) _tmp505_;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp507_ = gxml_node_get_children_nodes (_tmp506_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp508_ = _tmp507_;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp509_ = _tmp508_;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp510_ = gee_list_get ((GeeList*) _tmp509_, 0);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp511_ = (GXmlNode*) _tmp510_;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp512_ = gxml_node_get_children_nodes (_tmp511_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp513_ = _tmp512_;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp514_ = _tmp513_;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp515_ = gee_list_get ((GeeList*) _tmp514_, 0);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp516_ = (GXmlNode*) _tmp515_;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp517_ = gxml_node_get_children_nodes (_tmp516_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp518_ = _tmp517_;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp519_ = _tmp518_;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp520_ = gee_list_get ((GeeList*) _tmp519_, 0);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp521_ = (GXmlNode*) _tmp520_;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp521_, GXML_TYPE_TEXT), "d2.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes[0] is GXml.Text");
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp521_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp519_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp516_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp514_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp511_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp509_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp506_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp504_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp501_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp522_ = d2;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp523_ = gxml_document_get_root ((GXmlDocument*) _tmp522_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp524_ = _tmp523_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp525_ = _tmp524_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp526_ = gxml_node_get_children_nodes (_tmp525_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp527_ = _tmp526_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp528_ = _tmp527_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp529_ = gee_list_get ((GeeList*) _tmp528_, 0);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp530_ = (GXmlNode*) _tmp529_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp531_ = gxml_node_get_children_nodes (_tmp530_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp532_ = _tmp531_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp533_ = _tmp532_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp534_ = gee_list_get ((GeeList*) _tmp533_, 0);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp535_ = (GXmlNode*) _tmp534_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp536_ = gxml_node_get_children_nodes (_tmp535_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp537_ = _tmp536_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp538_ = _tmp537_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp539_ = gee_list_get ((GeeList*) _tmp538_, 0);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp540_ = (GXmlNode*) _tmp539_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp541_ = gxml_node_get_children_nodes (_tmp540_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp542_ = _tmp541_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp543_ = _tmp542_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp544_ = gee_list_get ((GeeList*) _tmp543_, 0);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp545_ = (GXmlNode*) _tmp544_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp546_ = gxml_node_get_value (_tmp545_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp547_ = _tmp546_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp548_ = _tmp547_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp548_, "golden dust") == 0, "d2.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes[0].value == \"golden dust\"");
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_free0 (_tmp548_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp545_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp543_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp540_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp538_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp535_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp533_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp530_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp528_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (_tmp525_);
#line 339 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (d2);
#line 339 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (bag2);
#line 339 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (bag3);
#line 339 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (bagt);
#line 339 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (d);
#line 339 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_object_unref0 (bag);
#line 5822 "SerializableGeeDualKeyMapTest.c"
	}
	goto __finally81;
	__catch81_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp549_ = NULL;
		gchar* _tmp550_ = NULL;
		gchar* _tmp551_ = NULL;
#line 339 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		e = _inner_error_;
#line 339 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_inner_error_ = NULL;
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp549_ = e->message;
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp550_ = g_strconcat ("ERROR: ", _tmp549_, NULL);
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_tmp551_ = _tmp550_;
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_message ("SerializableGeeDualKeyMapTest.vala:411: %s", _tmp551_);
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_free0 (_tmp551_);
#line 412 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_assert_not_reached ();
#line 339 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		_g_error_free0 (e);
#line 5849 "SerializableGeeDualKeyMapTest.c"
	}
	__finally81:
#line 339 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 339 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 339 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_clear_error (&_inner_error_);
#line 339 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		return;
#line 5860 "SerializableGeeDualKeyMapTest.c"
	}
}


static void ___lambda59__gtest_func (void) {
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	__lambda59_ ();
#line 5868 "SerializableGeeDualKeyMapTest.c"
}


void serializable_gee_dual_key_map_test_add_tests (void) {
#line 93 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_test_add_func ("/gxml/serializable/dualkeymap/api", ___lambda54__gtest_func);
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_test_add_func ("/gxml/serializable/dualkeymap/serialize", ___lambda55__gtest_func);
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_test_add_func ("/gxml/serializable/dualkeymap/deserialize", ___lambda56__gtest_func);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_test_add_func ("/gxml/serializable/dualkeymap/de-se-deserialize", ___lambda57__gtest_func);
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_test_add_func ("/gxml/serializable/dualkeymap/deserialize-node-names", ___lambda58__gtest_func);
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_test_add_func ("/gxml/serializable/dualkeymap/post-deserialization/serialize/contents", ___lambda59__gtest_func);
#line 5885 "SerializableGeeDualKeyMapTest.c"
}


SerializableGeeDualKeyMapTest* serializable_gee_dual_key_map_test_construct (GType object_type) {
	SerializableGeeDualKeyMapTest* self = NULL;
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTest*) gxml_test_construct (object_type);
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return self;
#line 5895 "SerializableGeeDualKeyMapTest.c"
}


SerializableGeeDualKeyMapTest* serializable_gee_dual_key_map_test_new (void) {
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return serializable_gee_dual_key_map_test_construct (TYPE_SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST);
#line 5902 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_ball_fill_set_text (SerializableGeeDualKeyMapTestBallFill* self, const gchar* txt) {
	const gchar* _tmp0_ = NULL;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_if_fail (self != NULL);
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_if_fail (txt != NULL);
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = txt;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	gxml_serializable_set_serialized_xml_node_value ((GXmlSerializable*) self, _tmp0_);
#line 5916 "SerializableGeeDualKeyMapTest.c"
}


static gchar* serializable_gee_dual_key_map_test_ball_fill_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeDualKeyMapTestBallFill * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTestBallFill*) base;
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp1_;
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 5935 "SerializableGeeDualKeyMapTest.c"
}


static gchar* serializable_gee_dual_key_map_test_ball_fill_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeDualKeyMapTestBallFill * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTestBallFill*) base;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = g_strdup ("BallFill");
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp0_;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 5951 "SerializableGeeDualKeyMapTest.c"
}


static gboolean serializable_gee_dual_key_map_test_ball_fill_real_serialize_use_xml_node_value (GXmlSerializableObjectModel* base) {
	SerializableGeeDualKeyMapTestBallFill * self;
	gboolean result = FALSE;
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTestBallFill*) base;
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = TRUE;
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 5964 "SerializableGeeDualKeyMapTest.c"
}


static SerializableGeeDualKeyMapTestBallFill* serializable_gee_dual_key_map_test_ball_fill_construct (GType object_type) {
	SerializableGeeDualKeyMapTestBallFill * self = NULL;
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTestBallFill*) gxml_serializable_object_model_construct (object_type);
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return self;
#line 5974 "SerializableGeeDualKeyMapTest.c"
}


static SerializableGeeDualKeyMapTestBallFill* serializable_gee_dual_key_map_test_ball_fill_new (void) {
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return serializable_gee_dual_key_map_test_ball_fill_construct (SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL_FILL);
#line 5981 "SerializableGeeDualKeyMapTest.c"
}


static const gchar* serializable_gee_dual_key_map_test_ball_fill_get_name (SerializableGeeDualKeyMapTestBallFill* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp0_;
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 5996 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_ball_fill_set_name (SerializableGeeDualKeyMapTestBallFill* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_if_fail (self != NULL);
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = value;
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_free0 (self->priv->_name);
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv->_name = _tmp1_;
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 6015 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_ball_fill_class_init (SerializableGeeDualKeyMapTestBallFillClass * klass) {
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	serializable_gee_dual_key_map_test_ball_fill_parent_class = g_type_class_peek_parent (klass);
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeDualKeyMapTestBallFillPrivate));
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_dual_key_map_test_ball_fill_real_to_string;
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_dual_key_map_test_ball_fill_real_node_name;
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->serialize_use_xml_node_value = (gboolean (*)(GXmlSerializableObjectModel*)) serializable_gee_dual_key_map_test_ball_fill_real_serialize_use_xml_node_value;
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_dual_key_map_test_ball_fill_get_property;
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_dual_key_map_test_ball_fill_set_property;
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_dual_key_map_test_ball_fill_finalize;
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_FILL_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 6038 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_ball_fill_instance_init (SerializableGeeDualKeyMapTestBallFill * self) {
	gchar* _tmp0_ = NULL;
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv = SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_FILL_GET_PRIVATE (self);
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = g_strdup ("Fill");
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv->_name = _tmp0_;
#line 6050 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_ball_fill_finalize (GObject* obj) {
	SerializableGeeDualKeyMapTestBallFill * self;
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL_FILL, SerializableGeeDualKeyMapTestBallFill);
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_free0 (self->priv->_name);
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	G_OBJECT_CLASS (serializable_gee_dual_key_map_test_ball_fill_parent_class)->finalize (obj);
#line 6062 "SerializableGeeDualKeyMapTest.c"
}


static GType serializable_gee_dual_key_map_test_ball_fill_get_type (void) {
	static volatile gsize serializable_gee_dual_key_map_test_ball_fill_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_dual_key_map_test_ball_fill_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeDualKeyMapTestBallFillClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_dual_key_map_test_ball_fill_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeDualKeyMapTestBallFill), 0, (GInstanceInitFunc) serializable_gee_dual_key_map_test_ball_fill_instance_init, NULL };
		GType serializable_gee_dual_key_map_test_ball_fill_type_id;
		serializable_gee_dual_key_map_test_ball_fill_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeDualKeyMapTestBallFill", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_dual_key_map_test_ball_fill_type_id__volatile, serializable_gee_dual_key_map_test_ball_fill_type_id);
	}
	return serializable_gee_dual_key_map_test_ball_fill_type_id__volatile;
}


static void _vala_serializable_gee_dual_key_map_test_ball_fill_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeDualKeyMapTestBallFill * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL_FILL, SerializableGeeDualKeyMapTestBallFill);
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	switch (property_id) {
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_FILL_NAME:
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_value_set_string (value, serializable_gee_dual_key_map_test_ball_fill_get_name (self));
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 6089 "SerializableGeeDualKeyMapTest.c"
		default:
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 6095 "SerializableGeeDualKeyMapTest.c"
	}
}


static void _vala_serializable_gee_dual_key_map_test_ball_fill_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeDualKeyMapTestBallFill * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL_FILL, SerializableGeeDualKeyMapTestBallFill);
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	switch (property_id) {
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_FILL_NAME:
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		serializable_gee_dual_key_map_test_ball_fill_set_name (self, g_value_get_string (value));
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 6111 "SerializableGeeDualKeyMapTest.c"
		default:
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 6117 "SerializableGeeDualKeyMapTest.c"
	}
}


static gchar* serializable_gee_dual_key_map_test_ball_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeDualKeyMapTestBall * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTestBall*) base;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp1_;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 6137 "SerializableGeeDualKeyMapTest.c"
}


static gchar* serializable_gee_dual_key_map_test_ball_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeDualKeyMapTestBall * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 61 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTestBall*) base;
#line 61 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = g_strdup ("Ball");
#line 61 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp0_;
#line 61 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 6153 "SerializableGeeDualKeyMapTest.c"
}


static gchar* serializable_gee_dual_key_map_test_ball_real_get_map_primary_key (GXmlSerializableMapDualKey* base) {
	SerializableGeeDualKeyMapTestBall * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTestBall*) base;
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_size;
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp1_;
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 6172 "SerializableGeeDualKeyMapTest.c"
}


static gchar* serializable_gee_dual_key_map_test_ball_real_get_map_secondary_key (GXmlSerializableMapDualKey* base) {
	SerializableGeeDualKeyMapTestBall * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 63 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTestBall*) base;
#line 63 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 63 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 63 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp1_;
#line 63 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 6191 "SerializableGeeDualKeyMapTest.c"
}


static SerializableGeeDualKeyMapTestBall* serializable_gee_dual_key_map_test_ball_construct (GType object_type) {
	SerializableGeeDualKeyMapTestBall * self = NULL;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTestBall*) gxml_serializable_object_model_construct (object_type);
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return self;
#line 6201 "SerializableGeeDualKeyMapTest.c"
}


static SerializableGeeDualKeyMapTestBall* serializable_gee_dual_key_map_test_ball_new (void) {
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return serializable_gee_dual_key_map_test_ball_construct (SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL);
#line 6208 "SerializableGeeDualKeyMapTest.c"
}


static const gchar* serializable_gee_dual_key_map_test_ball_get_name (SerializableGeeDualKeyMapTestBall* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp0_;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 6223 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_ball_set_name (SerializableGeeDualKeyMapTestBall* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_if_fail (self != NULL);
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = value;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_free0 (self->priv->_name);
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv->_name = _tmp1_;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 6242 "SerializableGeeDualKeyMapTest.c"
}


static const gchar* serializable_gee_dual_key_map_test_ball_get_size (SerializableGeeDualKeyMapTestBall* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_size;
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp0_;
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 6257 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_ball_set_size (SerializableGeeDualKeyMapTestBall* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_if_fail (self != NULL);
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = value;
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_free0 (self->priv->_size);
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv->_size = _tmp1_;
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_notify ((GObject *) self, "size");
#line 6276 "SerializableGeeDualKeyMapTest.c"
}


static SerializableGeeDualKeyMapTestBallFill* serializable_gee_dual_key_map_test_ball_get_ballfill (SerializableGeeDualKeyMapTestBall* self) {
	SerializableGeeDualKeyMapTestBallFill* result;
	SerializableGeeDualKeyMapTestBallFill* _tmp0_ = NULL;
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_ballfill;
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp0_;
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 6291 "SerializableGeeDualKeyMapTest.c"
}


static gpointer _g_object_ref0 (gpointer self) {
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return self ? g_object_ref (self) : NULL;
#line 6298 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_ball_set_ballfill (SerializableGeeDualKeyMapTestBall* self, SerializableGeeDualKeyMapTestBallFill* value) {
	SerializableGeeDualKeyMapTestBallFill* _tmp0_ = NULL;
	SerializableGeeDualKeyMapTestBallFill* _tmp1_ = NULL;
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_if_fail (self != NULL);
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = value;
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_object_unref0 (self->priv->_ballfill);
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv->_ballfill = _tmp1_;
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_notify ((GObject *) self, "ballfill");
#line 6317 "SerializableGeeDualKeyMapTest.c"
}


static gboolean serializable_gee_dual_key_map_test_ball_dual_map_real_deserialize_proceed (GXmlSerializableDualKeyMap* base) {
	SerializableGeeDualKeyMapTestBallDualMap * self;
	gboolean result = FALSE;
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTestBallDualMap*) base;
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = FALSE;
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 6330 "SerializableGeeDualKeyMapTest.c"
}


static SerializableGeeDualKeyMapTestBallDualMap* serializable_gee_dual_key_map_test_ball_dual_map_construct (GType object_type) {
	SerializableGeeDualKeyMapTestBallDualMap * self = NULL;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTestBallDualMap*) gxml_serializable_dual_key_map_construct (object_type, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return self;
#line 6340 "SerializableGeeDualKeyMapTest.c"
}


static SerializableGeeDualKeyMapTestBallDualMap* serializable_gee_dual_key_map_test_ball_dual_map_new (void) {
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return serializable_gee_dual_key_map_test_ball_dual_map_construct (SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_TYPE_DUAL_MAP);
#line 6347 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_ball_dual_map_class_init (SerializableGeeDualKeyMapTestBallDualMapClass * klass) {
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	serializable_gee_dual_key_map_test_ball_dual_map_parent_class = g_type_class_peek_parent (klass);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	((GXmlSerializableDualKeyMapClass *) klass)->deserialize_proceed = (gboolean (*)(GXmlSerializableDualKeyMap*)) serializable_gee_dual_key_map_test_ball_dual_map_real_deserialize_proceed;
#line 6356 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_ball_dual_map_instance_init (SerializableGeeDualKeyMapTestBallDualMap * self) {
}


GType serializable_gee_dual_key_map_test_ball_dual_map_get_type (void) {
	static volatile gsize serializable_gee_dual_key_map_test_ball_dual_map_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_dual_key_map_test_ball_dual_map_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeDualKeyMapTestBallDualMapClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_dual_key_map_test_ball_dual_map_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeDualKeyMapTestBallDualMap), 0, (GInstanceInitFunc) serializable_gee_dual_key_map_test_ball_dual_map_instance_init, NULL };
		GType serializable_gee_dual_key_map_test_ball_dual_map_type_id;
		serializable_gee_dual_key_map_test_ball_dual_map_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_DUAL_KEY_MAP, "SerializableGeeDualKeyMapTestBallDualMap", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_dual_key_map_test_ball_dual_map_type_id__volatile, serializable_gee_dual_key_map_test_ball_dual_map_type_id);
	}
	return serializable_gee_dual_key_map_test_ball_dual_map_type_id__volatile;
}


static void serializable_gee_dual_key_map_test_ball_class_init (SerializableGeeDualKeyMapTestBallClass * klass) {
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	serializable_gee_dual_key_map_test_ball_parent_class = g_type_class_peek_parent (klass);
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeDualKeyMapTestBallPrivate));
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_dual_key_map_test_ball_real_to_string;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_dual_key_map_test_ball_real_node_name;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_dual_key_map_test_ball_get_property;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_dual_key_map_test_ball_set_property;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_dual_key_map_test_ball_finalize;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_SIZE, g_param_spec_string ("size", "size", "size", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_BALLFILL, g_param_spec_object ("ballfill", "ballfill", "ballfill", SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL_FILL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 6397 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_ball_gxml_serializable_map_dual_key_interface_init (GXmlSerializableMapDualKeyIface * iface) {
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	serializable_gee_dual_key_map_test_ball_gxml_serializable_map_dual_key_parent_iface = g_type_interface_peek_parent (iface);
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	iface->get_map_primary_key = (gpointer (*)(GXmlSerializableMapDualKey*)) serializable_gee_dual_key_map_test_ball_real_get_map_primary_key;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	iface->get_map_secondary_key = (gpointer (*)(GXmlSerializableMapDualKey*)) serializable_gee_dual_key_map_test_ball_real_get_map_secondary_key;
#line 6408 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_ball_instance_init (SerializableGeeDualKeyMapTestBall * self) {
	gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv = SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_GET_PRIVATE (self);
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = g_strdup ("Ball");
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv->_name = _tmp0_;
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = g_strdup ("medium");
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv->_size = _tmp1_;
#line 6425 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_ball_finalize (GObject* obj) {
	SerializableGeeDualKeyMapTestBall * self;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL, SerializableGeeDualKeyMapTestBall);
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_free0 (self->priv->_name);
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_free0 (self->priv->_size);
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_object_unref0 (self->priv->_ballfill);
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	G_OBJECT_CLASS (serializable_gee_dual_key_map_test_ball_parent_class)->finalize (obj);
#line 6441 "SerializableGeeDualKeyMapTest.c"
}


static GType serializable_gee_dual_key_map_test_ball_get_type (void) {
	static volatile gsize serializable_gee_dual_key_map_test_ball_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_dual_key_map_test_ball_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeDualKeyMapTestBallClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_dual_key_map_test_ball_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeDualKeyMapTestBall), 0, (GInstanceInitFunc) serializable_gee_dual_key_map_test_ball_instance_init, NULL };
		static const GInterfaceInfo gxml_serializable_map_dual_key_info = { (GInterfaceInitFunc) serializable_gee_dual_key_map_test_ball_gxml_serializable_map_dual_key_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType serializable_gee_dual_key_map_test_ball_type_id;
		serializable_gee_dual_key_map_test_ball_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeDualKeyMapTestBall", &g_define_type_info, 0);
		g_type_add_interface_static (serializable_gee_dual_key_map_test_ball_type_id, GXML_TYPE_SERIALIZABLE_MAP_DUAL_KEY, &gxml_serializable_map_dual_key_info);
		g_once_init_leave (&serializable_gee_dual_key_map_test_ball_type_id__volatile, serializable_gee_dual_key_map_test_ball_type_id);
	}
	return serializable_gee_dual_key_map_test_ball_type_id__volatile;
}


static void _vala_serializable_gee_dual_key_map_test_ball_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeDualKeyMapTestBall * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL, SerializableGeeDualKeyMapTestBall);
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	switch (property_id) {
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_NAME:
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_value_set_string (value, serializable_gee_dual_key_map_test_ball_get_name (self));
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_SIZE:
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_value_set_string (value, serializable_gee_dual_key_map_test_ball_get_size (self));
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_BALLFILL:
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_value_set_object (value, serializable_gee_dual_key_map_test_ball_get_ballfill (self));
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 6482 "SerializableGeeDualKeyMapTest.c"
		default:
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 6488 "SerializableGeeDualKeyMapTest.c"
	}
}


static void _vala_serializable_gee_dual_key_map_test_ball_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeDualKeyMapTestBall * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BALL, SerializableGeeDualKeyMapTestBall);
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	switch (property_id) {
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_NAME:
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		serializable_gee_dual_key_map_test_ball_set_name (self, g_value_get_string (value));
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_SIZE:
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		serializable_gee_dual_key_map_test_ball_set_size (self, g_value_get_string (value));
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_BALLFILL:
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		serializable_gee_dual_key_map_test_ball_set_ballfill (self, g_value_get_object (value));
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 6516 "SerializableGeeDualKeyMapTest.c"
		default:
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 6522 "SerializableGeeDualKeyMapTest.c"
	}
}


static gchar* serializable_gee_dual_key_map_test_small_bag_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeDualKeyMapTestSmallBag * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTestSmallBag*) base;
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp1_;
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 6542 "SerializableGeeDualKeyMapTest.c"
}


static gchar* serializable_gee_dual_key_map_test_small_bag_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeDualKeyMapTestSmallBag * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTestSmallBag*) base;
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = g_strdup ("SmallBag");
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp0_;
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 6558 "SerializableGeeDualKeyMapTest.c"
}


static gchar* serializable_gee_dual_key_map_test_small_bag_real_get_map_primary_key (GXmlSerializableMapDualKey* base) {
	SerializableGeeDualKeyMapTestSmallBag * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTestSmallBag*) base;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_cash;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp1_;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 6577 "SerializableGeeDualKeyMapTest.c"
}


static gchar* serializable_gee_dual_key_map_test_small_bag_real_get_map_secondary_key (GXmlSerializableMapDualKey* base) {
	SerializableGeeDualKeyMapTestSmallBag * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTestSmallBag*) base;
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp1_;
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 6596 "SerializableGeeDualKeyMapTest.c"
}


static SerializableGeeDualKeyMapTestSmallBag* serializable_gee_dual_key_map_test_small_bag_construct (GType object_type) {
	SerializableGeeDualKeyMapTestSmallBag * self = NULL;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTestSmallBag*) gxml_serializable_object_model_construct (object_type);
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return self;
#line 6606 "SerializableGeeDualKeyMapTest.c"
}


static SerializableGeeDualKeyMapTestSmallBag* serializable_gee_dual_key_map_test_small_bag_new (void) {
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return serializable_gee_dual_key_map_test_small_bag_construct (SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_SMALL_BAG);
#line 6613 "SerializableGeeDualKeyMapTest.c"
}


static const gchar* serializable_gee_dual_key_map_test_small_bag_get_name (SerializableGeeDualKeyMapTestSmallBag* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp0_;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 6628 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_small_bag_set_name (SerializableGeeDualKeyMapTestSmallBag* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_if_fail (self != NULL);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = value;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_free0 (self->priv->_name);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv->_name = _tmp1_;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 6647 "SerializableGeeDualKeyMapTest.c"
}


static const gchar* serializable_gee_dual_key_map_test_small_bag_get_cash (SerializableGeeDualKeyMapTestSmallBag* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_cash;
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp0_;
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 6662 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_small_bag_set_cash (SerializableGeeDualKeyMapTestSmallBag* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_if_fail (self != NULL);
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = value;
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_free0 (self->priv->_cash);
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv->_cash = _tmp1_;
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_notify ((GObject *) self, "cash");
#line 6681 "SerializableGeeDualKeyMapTest.c"
}


static SerializableGeeDualKeyMapTestBallDualMap* serializable_gee_dual_key_map_test_small_bag_get_balls (SerializableGeeDualKeyMapTestSmallBag* self) {
	SerializableGeeDualKeyMapTestBallDualMap* result;
	SerializableGeeDualKeyMapTestBallDualMap* _tmp0_ = NULL;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_balls;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp0_;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 6696 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_small_bag_set_balls (SerializableGeeDualKeyMapTestSmallBag* self, SerializableGeeDualKeyMapTestBallDualMap* value) {
	SerializableGeeDualKeyMapTestBallDualMap* _tmp0_ = NULL;
	SerializableGeeDualKeyMapTestBallDualMap* _tmp1_ = NULL;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_if_fail (self != NULL);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = value;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_object_unref0 (self->priv->_balls);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv->_balls = _tmp1_;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_notify ((GObject *) self, "balls");
#line 6715 "SerializableGeeDualKeyMapTest.c"
}


static gboolean serializable_gee_dual_key_map_test_small_bag_dual_map_real_deserialize_proceed (GXmlSerializableDualKeyMap* base) {
	SerializableGeeDualKeyMapTestSmallBagDualMap * self;
	gboolean result = FALSE;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTestSmallBagDualMap*) base;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = FALSE;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 6728 "SerializableGeeDualKeyMapTest.c"
}


static SerializableGeeDualKeyMapTestSmallBagDualMap* serializable_gee_dual_key_map_test_small_bag_dual_map_construct (GType object_type) {
	SerializableGeeDualKeyMapTestSmallBagDualMap * self = NULL;
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTestSmallBagDualMap*) gxml_serializable_dual_key_map_construct (object_type, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_SMALL_BAG, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return self;
#line 6738 "SerializableGeeDualKeyMapTest.c"
}


static SerializableGeeDualKeyMapTestSmallBagDualMap* serializable_gee_dual_key_map_test_small_bag_dual_map_new (void) {
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return serializable_gee_dual_key_map_test_small_bag_dual_map_construct (SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_TYPE_DUAL_MAP);
#line 6745 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_small_bag_dual_map_class_init (SerializableGeeDualKeyMapTestSmallBagDualMapClass * klass) {
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	serializable_gee_dual_key_map_test_small_bag_dual_map_parent_class = g_type_class_peek_parent (klass);
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	((GXmlSerializableDualKeyMapClass *) klass)->deserialize_proceed = (gboolean (*)(GXmlSerializableDualKeyMap*)) serializable_gee_dual_key_map_test_small_bag_dual_map_real_deserialize_proceed;
#line 6754 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_small_bag_dual_map_instance_init (SerializableGeeDualKeyMapTestSmallBagDualMap * self) {
}


GType serializable_gee_dual_key_map_test_small_bag_dual_map_get_type (void) {
	static volatile gsize serializable_gee_dual_key_map_test_small_bag_dual_map_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_dual_key_map_test_small_bag_dual_map_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeDualKeyMapTestSmallBagDualMapClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_dual_key_map_test_small_bag_dual_map_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeDualKeyMapTestSmallBagDualMap), 0, (GInstanceInitFunc) serializable_gee_dual_key_map_test_small_bag_dual_map_instance_init, NULL };
		GType serializable_gee_dual_key_map_test_small_bag_dual_map_type_id;
		serializable_gee_dual_key_map_test_small_bag_dual_map_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_DUAL_KEY_MAP, "SerializableGeeDualKeyMapTestSmallBagDualMap", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_dual_key_map_test_small_bag_dual_map_type_id__volatile, serializable_gee_dual_key_map_test_small_bag_dual_map_type_id);
	}
	return serializable_gee_dual_key_map_test_small_bag_dual_map_type_id__volatile;
}


static void serializable_gee_dual_key_map_test_small_bag_class_init (SerializableGeeDualKeyMapTestSmallBagClass * klass) {
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	serializable_gee_dual_key_map_test_small_bag_parent_class = g_type_class_peek_parent (klass);
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeDualKeyMapTestSmallBagPrivate));
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_dual_key_map_test_small_bag_real_to_string;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_dual_key_map_test_small_bag_real_node_name;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_dual_key_map_test_small_bag_get_property;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_dual_key_map_test_small_bag_set_property;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_dual_key_map_test_small_bag_finalize;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_CASH, g_param_spec_string ("cash", "cash", "cash", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_BALLS, g_param_spec_object ("balls", "balls", "balls", SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BALL_TYPE_DUAL_MAP, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 6795 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_small_bag_gxml_serializable_map_dual_key_interface_init (GXmlSerializableMapDualKeyIface * iface) {
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	serializable_gee_dual_key_map_test_small_bag_gxml_serializable_map_dual_key_parent_iface = g_type_interface_peek_parent (iface);
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	iface->get_map_primary_key = (gpointer (*)(GXmlSerializableMapDualKey*)) serializable_gee_dual_key_map_test_small_bag_real_get_map_primary_key;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	iface->get_map_secondary_key = (gpointer (*)(GXmlSerializableMapDualKey*)) serializable_gee_dual_key_map_test_small_bag_real_get_map_secondary_key;
#line 6806 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_small_bag_instance_init (SerializableGeeDualKeyMapTestSmallBag * self) {
	gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	SerializableGeeDualKeyMapTestBallDualMap* _tmp2_ = NULL;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv = SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_GET_PRIVATE (self);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = g_strdup ("SmallBag");
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv->_name = _tmp0_;
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = g_strdup ("money");
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv->_cash = _tmp1_;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp2_ = serializable_gee_dual_key_map_test_ball_dual_map_new ();
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv->_balls = _tmp2_;
#line 6828 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_small_bag_finalize (GObject* obj) {
	SerializableGeeDualKeyMapTestSmallBag * self;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_SMALL_BAG, SerializableGeeDualKeyMapTestSmallBag);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_free0 (self->priv->_name);
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_free0 (self->priv->_cash);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_object_unref0 (self->priv->_balls);
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	G_OBJECT_CLASS (serializable_gee_dual_key_map_test_small_bag_parent_class)->finalize (obj);
#line 6844 "SerializableGeeDualKeyMapTest.c"
}


static GType serializable_gee_dual_key_map_test_small_bag_get_type (void) {
	static volatile gsize serializable_gee_dual_key_map_test_small_bag_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_dual_key_map_test_small_bag_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeDualKeyMapTestSmallBagClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_dual_key_map_test_small_bag_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeDualKeyMapTestSmallBag), 0, (GInstanceInitFunc) serializable_gee_dual_key_map_test_small_bag_instance_init, NULL };
		static const GInterfaceInfo gxml_serializable_map_dual_key_info = { (GInterfaceInitFunc) serializable_gee_dual_key_map_test_small_bag_gxml_serializable_map_dual_key_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType serializable_gee_dual_key_map_test_small_bag_type_id;
		serializable_gee_dual_key_map_test_small_bag_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeDualKeyMapTestSmallBag", &g_define_type_info, 0);
		g_type_add_interface_static (serializable_gee_dual_key_map_test_small_bag_type_id, GXML_TYPE_SERIALIZABLE_MAP_DUAL_KEY, &gxml_serializable_map_dual_key_info);
		g_once_init_leave (&serializable_gee_dual_key_map_test_small_bag_type_id__volatile, serializable_gee_dual_key_map_test_small_bag_type_id);
	}
	return serializable_gee_dual_key_map_test_small_bag_type_id__volatile;
}


static void _vala_serializable_gee_dual_key_map_test_small_bag_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeDualKeyMapTestSmallBag * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_SMALL_BAG, SerializableGeeDualKeyMapTestSmallBag);
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	switch (property_id) {
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_NAME:
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_value_set_string (value, serializable_gee_dual_key_map_test_small_bag_get_name (self));
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_CASH:
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_value_set_string (value, serializable_gee_dual_key_map_test_small_bag_get_cash (self));
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_BALLS:
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_value_set_object (value, serializable_gee_dual_key_map_test_small_bag_get_balls (self));
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 6885 "SerializableGeeDualKeyMapTest.c"
		default:
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 6891 "SerializableGeeDualKeyMapTest.c"
	}
}


static void _vala_serializable_gee_dual_key_map_test_small_bag_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeDualKeyMapTestSmallBag * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_SMALL_BAG, SerializableGeeDualKeyMapTestSmallBag);
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	switch (property_id) {
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_NAME:
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		serializable_gee_dual_key_map_test_small_bag_set_name (self, g_value_get_string (value));
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_CASH:
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		serializable_gee_dual_key_map_test_small_bag_set_cash (self, g_value_get_string (value));
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_BALLS:
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		serializable_gee_dual_key_map_test_small_bag_set_balls (self, g_value_get_object (value));
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 6919 "SerializableGeeDualKeyMapTest.c"
		default:
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 6925 "SerializableGeeDualKeyMapTest.c"
	}
}


static gchar* serializable_gee_dual_key_map_test_big_bag_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeDualKeyMapTestBigBag * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 87 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTestBigBag*) base;
#line 87 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 87 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 87 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp1_;
#line 87 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 6945 "SerializableGeeDualKeyMapTest.c"
}


static gchar* serializable_gee_dual_key_map_test_big_bag_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeDualKeyMapTestBigBag * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTestBigBag*) base;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = g_strdup ("BigBag");
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp0_;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 6961 "SerializableGeeDualKeyMapTest.c"
}


static SerializableGeeDualKeyMapTestBigBag* serializable_gee_dual_key_map_test_big_bag_construct (GType object_type) {
	SerializableGeeDualKeyMapTestBigBag * self = NULL;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = (SerializableGeeDualKeyMapTestBigBag*) gxml_serializable_object_model_construct (object_type);
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return self;
#line 6971 "SerializableGeeDualKeyMapTest.c"
}


static SerializableGeeDualKeyMapTestBigBag* serializable_gee_dual_key_map_test_big_bag_new (void) {
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return serializable_gee_dual_key_map_test_big_bag_construct (SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BIG_BAG);
#line 6978 "SerializableGeeDualKeyMapTest.c"
}


static const gchar* serializable_gee_dual_key_map_test_big_bag_get_name (SerializableGeeDualKeyMapTestBigBag* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp0_;
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 6993 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_big_bag_set_name (SerializableGeeDualKeyMapTestBigBag* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_if_fail (self != NULL);
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = value;
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_free0 (self->priv->_name);
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv->_name = _tmp1_;
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 7012 "SerializableGeeDualKeyMapTest.c"
}


static SerializableGeeDualKeyMapTestSmallBagDualMap* serializable_gee_dual_key_map_test_big_bag_get_bags (SerializableGeeDualKeyMapTestBigBag* self) {
	SerializableGeeDualKeyMapTestSmallBagDualMap* result;
	SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp0_ = NULL;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = self->priv->_bags;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	result = _tmp0_;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	return result;
#line 7027 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_big_bag_set_bags (SerializableGeeDualKeyMapTestBigBag* self, SerializableGeeDualKeyMapTestSmallBagDualMap* value) {
	SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp0_ = NULL;
	SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp1_ = NULL;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_return_if_fail (self != NULL);
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = value;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_object_unref0 (self->priv->_bags);
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv->_bags = _tmp1_;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_notify ((GObject *) self, "bags");
#line 7046 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_big_bag_class_init (SerializableGeeDualKeyMapTestBigBagClass * klass) {
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	serializable_gee_dual_key_map_test_big_bag_parent_class = g_type_class_peek_parent (klass);
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeDualKeyMapTestBigBagPrivate));
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_dual_key_map_test_big_bag_real_to_string;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_dual_key_map_test_big_bag_real_node_name;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_dual_key_map_test_big_bag_get_property;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_dual_key_map_test_big_bag_set_property;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_dual_key_map_test_big_bag_finalize;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BIG_BAG_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BIG_BAG_BAGS, g_param_spec_object ("bags", "bags", "bags", SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_SMALL_BAG_TYPE_DUAL_MAP, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 7069 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_big_bag_instance_init (SerializableGeeDualKeyMapTestBigBag * self) {
	gchar* _tmp0_ = NULL;
	SerializableGeeDualKeyMapTestSmallBagDualMap* _tmp1_ = NULL;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv = SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BIG_BAG_GET_PRIVATE (self);
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp0_ = g_strdup ("ball");
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv->_name = _tmp0_;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_tmp1_ = serializable_gee_dual_key_map_test_small_bag_dual_map_new ();
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self->priv->_bags = _tmp1_;
#line 7086 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_big_bag_finalize (GObject* obj) {
	SerializableGeeDualKeyMapTestBigBag * self;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BIG_BAG, SerializableGeeDualKeyMapTestBigBag);
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_free0 (self->priv->_name);
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	_g_object_unref0 (self->priv->_bags);
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	G_OBJECT_CLASS (serializable_gee_dual_key_map_test_big_bag_parent_class)->finalize (obj);
#line 7100 "SerializableGeeDualKeyMapTest.c"
}


static GType serializable_gee_dual_key_map_test_big_bag_get_type (void) {
	static volatile gsize serializable_gee_dual_key_map_test_big_bag_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_dual_key_map_test_big_bag_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeDualKeyMapTestBigBagClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_dual_key_map_test_big_bag_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeDualKeyMapTestBigBag), 0, (GInstanceInitFunc) serializable_gee_dual_key_map_test_big_bag_instance_init, NULL };
		GType serializable_gee_dual_key_map_test_big_bag_type_id;
		serializable_gee_dual_key_map_test_big_bag_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeDualKeyMapTestBigBag", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_dual_key_map_test_big_bag_type_id__volatile, serializable_gee_dual_key_map_test_big_bag_type_id);
	}
	return serializable_gee_dual_key_map_test_big_bag_type_id__volatile;
}


static void _vala_serializable_gee_dual_key_map_test_big_bag_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeDualKeyMapTestBigBag * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BIG_BAG, SerializableGeeDualKeyMapTestBigBag);
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	switch (property_id) {
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BIG_BAG_NAME:
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_value_set_string (value, serializable_gee_dual_key_map_test_big_bag_get_name (self));
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BIG_BAG_BAGS:
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		g_value_set_object (value, serializable_gee_dual_key_map_test_big_bag_get_bags (self));
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 7133 "SerializableGeeDualKeyMapTest.c"
		default:
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 7139 "SerializableGeeDualKeyMapTest.c"
	}
}


static void _vala_serializable_gee_dual_key_map_test_big_bag_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeDualKeyMapTestBigBag * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_TYPE_BIG_BAG, SerializableGeeDualKeyMapTestBigBag);
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	switch (property_id) {
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BIG_BAG_NAME:
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		serializable_gee_dual_key_map_test_big_bag_set_name (self, g_value_get_string (value));
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		case SERIALIZABLE_GEE_DUAL_KEY_MAP_TEST_BIG_BAG_BAGS:
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		serializable_gee_dual_key_map_test_big_bag_set_bags (self, g_value_get_object (value));
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 7161 "SerializableGeeDualKeyMapTest.c"
		default:
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
		break;
#line 7167 "SerializableGeeDualKeyMapTest.c"
	}
}


static void serializable_gee_dual_key_map_test_class_init (SerializableGeeDualKeyMapTestClass * klass) {
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeDualKeyMapTest.vala"
	serializable_gee_dual_key_map_test_parent_class = g_type_class_peek_parent (klass);
#line 7175 "SerializableGeeDualKeyMapTest.c"
}


static void serializable_gee_dual_key_map_test_instance_init (SerializableGeeDualKeyMapTest * self) {
}


GType serializable_gee_dual_key_map_test_get_type (void) {
	static volatile gsize serializable_gee_dual_key_map_test_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_dual_key_map_test_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeDualKeyMapTestClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_dual_key_map_test_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeDualKeyMapTest), 0, (GInstanceInitFunc) serializable_gee_dual_key_map_test_instance_init, NULL };
		GType serializable_gee_dual_key_map_test_type_id;
		serializable_gee_dual_key_map_test_type_id = g_type_register_static (TYPE_GXML_TEST, "SerializableGeeDualKeyMapTest", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_dual_key_map_test_type_id__volatile, serializable_gee_dual_key_map_test_type_id);
	}
	return serializable_gee_dual_key_map_test_type_id__volatile;
}



