/* SerializableObjectModelTest.c generated by valac 0.34.6, the Vala compiler
 * generated from SerializableObjectModelTest.vala, do not modify */

/* -*- Mode: vala; indent-tabs-mode: nil; c-basic-offset: 2; tab-width: 2 -*- */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gxml/gxml.h>
#include <float.h>
#include <math.h>
#include <gee.h>
#include <stdio.h>


#define TYPE_OBJECT_MODEL (object_model_get_type ())
#define OBJECT_MODEL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_OBJECT_MODEL, ObjectModel))
#define OBJECT_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_OBJECT_MODEL, ObjectModelClass))
#define IS_OBJECT_MODEL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_OBJECT_MODEL))
#define IS_OBJECT_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_OBJECT_MODEL))
#define OBJECT_MODEL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_OBJECT_MODEL, ObjectModelClass))

typedef struct _ObjectModel ObjectModel;
typedef struct _ObjectModelClass ObjectModelClass;
typedef struct _ObjectModelPrivate ObjectModelPrivate;
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_param_spec_unref0(var) ((var == NULL) ? NULL : (var = (g_param_spec_unref (var), NULL)))

#define TYPE_COMPUTER (computer_get_type ())
#define COMPUTER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COMPUTER, Computer))
#define COMPUTER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COMPUTER, ComputerClass))
#define IS_COMPUTER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COMPUTER))
#define IS_COMPUTER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COMPUTER))
#define COMPUTER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COMPUTER, ComputerClass))

typedef struct _Computer Computer;
typedef struct _ComputerClass ComputerClass;
typedef struct _ComputerPrivate ComputerPrivate;

#define TYPE_MANUAL (manual_get_type ())
#define MANUAL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MANUAL, Manual))
#define MANUAL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MANUAL, ManualClass))
#define IS_MANUAL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MANUAL))
#define IS_MANUAL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MANUAL))
#define MANUAL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MANUAL, ManualClass))

typedef struct _Manual Manual;
typedef struct _ManualClass ManualClass;
typedef struct _ManualPrivate ManualPrivate;

#define TYPE_PACKAGE (package_get_type ())
#define PACKAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PACKAGE, Package))
#define PACKAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PACKAGE, PackageClass))
#define IS_PACKAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PACKAGE))
#define IS_PACKAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PACKAGE))
#define PACKAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PACKAGE, PackageClass))

typedef struct _Package Package;
typedef struct _PackageClass PackageClass;
typedef struct _PackagePrivate PackagePrivate;
#define _g_array_unref0(var) ((var == NULL) ? NULL : (var = (g_array_unref (var), NULL)))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))

#define TYPE_MONITOR (monitor_get_type ())
#define MONITOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MONITOR, Monitor))
#define MONITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MONITOR, MonitorClass))
#define IS_MONITOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MONITOR))
#define IS_MONITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MONITOR))
#define MONITOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MONITOR, MonitorClass))

typedef struct _Monitor Monitor;
typedef struct _MonitorClass MonitorClass;
typedef struct _MonitorPrivate MonitorPrivate;

#define TYPE_CPU (cpu_get_type ())
#define CPU(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CPU, Cpu))
#define CPU_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CPU, CpuClass))
#define IS_CPU(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CPU))
#define IS_CPU_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CPU))
#define CPU_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CPU, CpuClass))

typedef struct _Cpu Cpu;
typedef struct _CpuClass CpuClass;
typedef struct _CpuPrivate CpuPrivate;

#define TYPE_NODE_NAME (node_name_get_type ())
#define NODE_NAME(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_NODE_NAME, NodeName))
#define NODE_NAME_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_NODE_NAME, NodeNameClass))
#define IS_NODE_NAME(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_NODE_NAME))
#define IS_NODE_NAME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_NODE_NAME))
#define NODE_NAME_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_NODE_NAME, NodeNameClass))

typedef struct _NodeName NodeName;
typedef struct _NodeNameClass NodeNameClass;
typedef struct _NodeNamePrivate NodeNamePrivate;

#define TYPE_CONFIGURATION (configuration_get_type ())
#define CONFIGURATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CONFIGURATION, Configuration))
#define CONFIGURATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CONFIGURATION, ConfigurationClass))
#define IS_CONFIGURATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CONFIGURATION))
#define IS_CONFIGURATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CONFIGURATION))
#define CONFIGURATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CONFIGURATION, ConfigurationClass))

typedef struct _Configuration Configuration;
typedef struct _ConfigurationClass ConfigurationClass;
typedef struct _ConfigurationPrivate ConfigurationPrivate;
#define _g_hash_table_unref0(var) ((var == NULL) ? NULL : (var = (g_hash_table_unref (var), NULL)))

#define TYPE_FAKE_SERIALIZABLE (fake_serializable_get_type ())
#define FAKE_SERIALIZABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_FAKE_SERIALIZABLE, FakeSerializable))
#define FAKE_SERIALIZABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_FAKE_SERIALIZABLE, FakeSerializableClass))
#define IS_FAKE_SERIALIZABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_FAKE_SERIALIZABLE))
#define IS_FAKE_SERIALIZABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_FAKE_SERIALIZABLE))
#define FAKE_SERIALIZABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_FAKE_SERIALIZABLE, FakeSerializableClass))

typedef struct _FakeSerializable FakeSerializable;
typedef struct _FakeSerializableClass FakeSerializableClass;
typedef struct _FakeSerializablePrivate FakeSerializablePrivate;

#define TYPE_UNKNOWN_ATTRIBUTE (unknown_attribute_get_type ())
#define UNKNOWN_ATTRIBUTE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_UNKNOWN_ATTRIBUTE, UnknownAttribute))
#define UNKNOWN_ATTRIBUTE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_UNKNOWN_ATTRIBUTE, UnknownAttributeClass))
#define IS_UNKNOWN_ATTRIBUTE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_UNKNOWN_ATTRIBUTE))
#define IS_UNKNOWN_ATTRIBUTE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_UNKNOWN_ATTRIBUTE))
#define UNKNOWN_ATTRIBUTE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_UNKNOWN_ATTRIBUTE, UnknownAttributeClass))

typedef struct _UnknownAttribute UnknownAttribute;
typedef struct _UnknownAttributeClass UnknownAttributeClass;
typedef struct _UnknownAttributePrivate UnknownAttributePrivate;

#define TYPE_NAME_SPACE (name_space_get_type ())
#define NAME_SPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_NAME_SPACE, NameSpace))
#define NAME_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_NAME_SPACE, NameSpaceClass))
#define IS_NAME_SPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_NAME_SPACE))
#define IS_NAME_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_NAME_SPACE))
#define NAME_SPACE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_NAME_SPACE, NameSpaceClass))

typedef struct _NameSpace NameSpace;
typedef struct _NameSpaceClass NameSpaceClass;
typedef struct _NameSpacePrivate NameSpacePrivate;

#define TYPE_GXML_TEST (gxml_test_get_type ())
#define GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GXML_TEST, GXmlTest))
#define GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GXML_TEST, GXmlTestClass))
#define IS_GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GXML_TEST))
#define IS_GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GXML_TEST))
#define GXML_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GXML_TEST, GXmlTestClass))

typedef struct _GXmlTest GXmlTest;
typedef struct _GXmlTestClass GXmlTestClass;
typedef struct _GXmlTestPrivate GXmlTestPrivate;

#define TYPE_SERIALIZABLE_OBJECT_MODEL_TEST (serializable_object_model_test_get_type ())
#define SERIALIZABLE_OBJECT_MODEL_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SERIALIZABLE_OBJECT_MODEL_TEST, SerializableObjectModelTest))
#define SERIALIZABLE_OBJECT_MODEL_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SERIALIZABLE_OBJECT_MODEL_TEST, SerializableObjectModelTestClass))
#define IS_SERIALIZABLE_OBJECT_MODEL_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SERIALIZABLE_OBJECT_MODEL_TEST))
#define IS_SERIALIZABLE_OBJECT_MODEL_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SERIALIZABLE_OBJECT_MODEL_TEST))
#define SERIALIZABLE_OBJECT_MODEL_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SERIALIZABLE_OBJECT_MODEL_TEST, SerializableObjectModelTestClass))

typedef struct _SerializableObjectModelTest SerializableObjectModelTest;
typedef struct _SerializableObjectModelTestClass SerializableObjectModelTestClass;
typedef struct _SerializableObjectModelTestPrivate SerializableObjectModelTestPrivate;
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _ObjectModel {
	GXmlSerializableObjectModel parent_instance;
	ObjectModelPrivate * priv;
};

struct _ObjectModelClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _Computer {
	ObjectModel parent_instance;
	ComputerPrivate * priv;
};

struct _ComputerClass {
	ObjectModelClass parent_class;
};

struct _ComputerPrivate {
	gchar* _manufacturer;
	gchar* _model;
	gint _cores;
	gfloat _ghz;
	guint _quantity;
};

struct _Manual {
	ObjectModel parent_instance;
	ManualPrivate * priv;
};

struct _ManualClass {
	ObjectModelClass parent_class;
};

struct _ManualPrivate {
	gchar* _document;
	gint _pages;
};

struct _Package {
	ObjectModel parent_instance;
	PackagePrivate * priv;
};

struct _PackageClass {
	ObjectModelClass parent_class;
};

struct _PackagePrivate {
	GArray* _tags;
	Computer* _computer;
	Manual* _manual;
	gchar* _source;
	gchar* _destiny;
};

struct _Monitor {
	ObjectModel parent_instance;
	MonitorPrivate * priv;
};

struct _MonitorClass {
	ObjectModelClass parent_class;
};

struct _MonitorPrivate {
	gchar* _resolution;
	gint _ac_power;
	gint _dc_power;
};

struct _Cpu {
	ObjectModel parent_instance;
	CpuPrivate * priv;
};

struct _CpuClass {
	ObjectModelClass parent_class;
};

struct _CpuPrivate {
	gfloat _ghz;
	GeeArrayList* _piles;
};

struct _NodeName {
	ObjectModel parent_instance;
	NodeNamePrivate * priv;
};

struct _NodeNameClass {
	ObjectModelClass parent_class;
};

struct _NodeNamePrivate {
	gboolean _invalid;
};

struct _Configuration {
	ObjectModel parent_instance;
	ConfigurationPrivate * priv;
};

struct _ConfigurationClass {
	ObjectModelClass parent_class;
};

struct _ConfigurationPrivate {
	gboolean _invalid;
	gchar* _device;
};

struct _FakeSerializable {
	ObjectModel parent_instance;
	FakeSerializablePrivate * priv;
};

struct _FakeSerializableClass {
	ObjectModelClass parent_class;
};

struct _FakeSerializablePrivate {
	gchar* _none;
};

struct _UnknownAttribute {
	ObjectModel parent_instance;
	UnknownAttributePrivate * priv;
};

struct _UnknownAttributeClass {
	ObjectModelClass parent_class;
};

struct _UnknownAttributePrivate {
	gchar* _name;
	GeeArrayList* _array;
	FakeSerializable* _fake;
};

struct _NameSpace {
	GXmlSerializableObjectModel parent_instance;
	NameSpacePrivate * priv;
};

struct _NameSpaceClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _GXmlTest {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GXmlTestPrivate * priv;
};

struct _GXmlTestClass {
	GTypeClass parent_class;
	void (*finalize) (GXmlTest *self);
};

struct _SerializableObjectModelTest {
	GXmlTest parent_instance;
	SerializableObjectModelTestPrivate * priv;
};

struct _SerializableObjectModelTestClass {
	GXmlTestClass parent_class;
};


static gpointer object_model_parent_class = NULL;
static gpointer computer_parent_class = NULL;
static gpointer manual_parent_class = NULL;
static gpointer package_parent_class = NULL;
static gpointer monitor_parent_class = NULL;
static gpointer cpu_parent_class = NULL;
static gpointer node_name_parent_class = NULL;
static gpointer configuration_parent_class = NULL;
static gpointer fake_serializable_parent_class = NULL;
static gpointer unknown_attribute_parent_class = NULL;
static gpointer name_space_parent_class = NULL;
static gpointer serializable_object_model_test_parent_class = NULL;

#define XML_COMPUTER_FILE "<?xml version=\"1.0\"?>\n" \
"<computer manufacturer=\"ThecnicalGroup\" model=\"YH576G\" cores=\"4\"" \
" ghz=\"2.8\"/>"
#define SERIALIZED_XML_COMPUTER_FILE "<?xml version=\"1.0\"?><computer manufacturer=\"MexicanLaptop, Inc.\" " \
"model=\"LQ59678\" cores=\"8\" ghz=\"3.5\"/>"
#define XML_PACKAGE_FILE "<?xml version=\"1.0\"?>\n" \
"<PACKAGE source=\"Mexico/Central\" destiny=\"Japan\">\n" \
"<manual document=\"Specification\" pages=\"3\">This is an Specificatio" \
"n file</manual>\n" \
"<Computer manufacturer=\"LanCorop\" model=\"Lap39120\" cores=\"16\" gh" \
"z=\"3.5\"/>\n" \
"<tag>Printer</tag><tag>Partner</tag><tag>Support</tag>\n" \
"</PACKAGE>"
#define XML_PACKAGE_UNKNOWN_NODES_FILE "<?xml version=\"1.0\"?>\n" \
"<PACKAGE source=\"Mexico/North\" destiny=\"Brazil\" Hope=\"2/4.04\">\n" \
"<manual document=\"Sales Card\" pages=\"1\">Selling Card Specification" \
"</manual>\n" \
"<Computer manufacturer=\"BoxingLAN\" model=\"J24-EX9\" cores=\"32\" gh" \
"z=\"1.8\"/>\n" \
"<Box size=\"1\" volume=\"33.15\" units=\"cm3\" />\n" \
"</PACKAGE>"
#define XML_CPU_FILE "<?xml version=\"1.0\"?>\n<cpu ghz=\"3.85\" piles=\"1,2,3\"/>"
GType object_model_get_type (void) G_GNUC_CONST;
enum  {
	OBJECT_MODEL_DUMMY_PROPERTY
};
static gchar* object_model_real_to_string (GXmlSerializableObjectModel* base);
ObjectModel* object_model_new (void);
ObjectModel* object_model_construct (GType object_type);
GType computer_get_type (void) G_GNUC_CONST;
#define COMPUTER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_COMPUTER, ComputerPrivate))
enum  {
	COMPUTER_DUMMY_PROPERTY,
	COMPUTER_MANUFACTURER,
	COMPUTER_MODEL,
	COMPUTER_CORES,
	COMPUTER_GHZ,
	COMPUTER_QUANTITY
};
Computer* computer_new (void);
Computer* computer_construct (GType object_type);
void computer_set_manufacturer (Computer* self, const gchar* value);
void computer_set_model (Computer* self, const gchar* value);
void computer_set_cores (Computer* self, gint value);
void computer_set_ghz (Computer* self, gfloat value);
const gchar* computer_get_manufacturer (Computer* self);
const gchar* computer_get_model (Computer* self);
gint computer_get_cores (Computer* self);
gfloat computer_get_ghz (Computer* self);
guint computer_get_quantity (Computer* self);
void computer_set_quantity (Computer* self, guint value);
static void computer_finalize (GObject* obj);
static void _vala_computer_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_computer_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType manual_get_type (void) G_GNUC_CONST;
#define MANUAL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_MANUAL, ManualPrivate))
enum  {
	MANUAL_DUMMY_PROPERTY,
	MANUAL_DOCUMENT,
	MANUAL_PAGES
};
gchar* manual_get_contents (Manual* self);
void manual_set_contents (Manual* self, const gchar* val);
Manual* manual_new (void);
Manual* manual_construct (GType object_type);
void manual_set_document (Manual* self, const gchar* value);
void manual_set_pages (Manual* self, gint value);
static gchar* manual_real_to_string (GXmlSerializableObjectModel* base);
static gboolean manual_real_serialize_use_xml_node_value (GXmlSerializableObjectModel* base);
const gchar* manual_get_document (Manual* self);
gint manual_get_pages (Manual* self);
static void manual_finalize (GObject* obj);
static void _vala_manual_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_manual_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType package_get_type (void) G_GNUC_CONST;
#define PACKAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_PACKAGE, PackagePrivate))
enum  {
	PACKAGE_DUMMY_PROPERTY,
	PACKAGE_COMPUTER,
	PACKAGE_MANUAL,
	PACKAGE_SOURCE,
	PACKAGE_DESTINY,
	PACKAGE_TAGS
};
static gboolean package_real_property_use_nick (GXmlSerializableObjectModel* base);
static gboolean package_real_get_enable_unknown_serializable_property (GXmlSerializableObjectModel* base);
Package* package_new (void);
Package* package_construct (GType object_type);
void package_set_computer (Package* self, Computer* value);
void package_set_manual (Package* self, Manual* value);
void package_set_source (Package* self, const gchar* value);
void package_set_destiny (Package* self, const gchar* value);
static void __lambda14_ (Package* self, GXmlNode* element, GParamSpec* prop, GXmlNode** node);
GArray* package_get_tags (Package* self);
static void ___lambda14__gxml_serializable_serialize_unknown_property (GXmlSerializable* _sender, GXmlNode* element, GParamSpec* prop, GXmlNode** node, gpointer self);
static void __lambda15_ (Package* self, GXmlNode* element, GParamSpec* prop);
static void ___lambda15__gxml_serializable_deserialize_unknown_property (GXmlSerializable* _sender, GXmlNode* node, GParamSpec* prop, gpointer self);
gchar* package_unknown_to_string (Package* self);
Computer* package_get_computer (Package* self);
Manual* package_get_manual (Package* self);
const gchar* package_get_source (Package* self);
const gchar* package_get_destiny (Package* self);
static void package_finalize (GObject* obj);
static void _vala_package_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_package_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType monitor_get_type (void) G_GNUC_CONST;
#define MONITOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_MONITOR, MonitorPrivate))
enum  {
	MONITOR_DUMMY_PROPERTY,
	MONITOR_RESOLUTION,
	MONITOR_AC_POWER,
	MONITOR_DC_POWER
};
static gboolean monitor_real_property_use_nick (GXmlSerializableObjectModel* base);
Monitor* monitor_new (void);
Monitor* monitor_construct (GType object_type);
const gchar* monitor_get_resolution (Monitor* self);
void monitor_set_resolution (Monitor* self, const gchar* value);
gint monitor_get_ac_power (Monitor* self);
void monitor_set_ac_power (Monitor* self, gint value);
gint monitor_get_dc_power (Monitor* self);
void monitor_set_dc_power (Monitor* self, gint value);
static void monitor_finalize (GObject* obj);
static void _vala_monitor_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_monitor_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType cpu_get_type (void) G_GNUC_CONST;
#define CPU_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_CPU, CpuPrivate))
enum  {
	CPU_DUMMY_PROPERTY,
	CPU_GHZ,
	CPU_PILES
};
Cpu* cpu_new (void);
Cpu* cpu_construct (GType object_type);
void cpu_set_piles (Cpu* self, GeeArrayList* value);
gchar* cpu_piles_to_string (Cpu* self);
GeeArrayList* cpu_get_piles (Cpu* self);
gfloat cpu_get_ghz (Cpu* self);
void cpu_set_ghz (Cpu* self, gfloat value);
static void cpu_finalize (GObject* obj);
static void _vala_cpu_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_cpu_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType node_name_get_type (void) G_GNUC_CONST;
#define NODE_NAME_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_NODE_NAME, NodeNamePrivate))
enum  {
	NODE_NAME_DUMMY_PROPERTY,
	NODE_NAME_INVALID
};
static gchar* node_name_real_node_name (GXmlSerializableObjectModel* base);
NodeName* node_name_new (void);
NodeName* node_name_construct (GType object_type);
gboolean node_name_get_invalid (NodeName* self);
void node_name_set_invalid (NodeName* self, gboolean value);
static void node_name_finalize (GObject* obj);
static void _vala_node_name_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_node_name_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType configuration_get_type (void) G_GNUC_CONST;
#define CONFIGURATION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_CONFIGURATION, ConfigurationPrivate))
enum  {
	CONFIGURATION_DUMMY_PROPERTY,
	CONFIGURATION_INVALID,
	CONFIGURATION_DEVICE
};
static gchar* configuration_real_node_name (GXmlSerializableObjectModel* base);
static gboolean configuration_real_property_use_nick (GXmlSerializableObjectModel* base);
static GParamSpec** configuration_real_list_serializable_properties (GXmlSerializableObjectModel* base, int* result_length1);
static void _g_free0_ (gpointer var);
static void _g_param_spec_unref0_ (gpointer var);
static void _vala_array_add1 (GParamSpec*** array, int* length, int* size, GParamSpec* value);
static GXmlNode* configuration_real_serialize (GXmlSerializableObjectModel* base, GXmlNode* node, GError** error);
static gboolean configuration_real_deserialize (GXmlSerializableObjectModel* base, GXmlNode* node, GError** error);
void configuration_set_invalid (Configuration* self, gboolean value);
Configuration* configuration_new (void);
Configuration* configuration_construct (GType object_type);
gboolean configuration_get_invalid (Configuration* self);
const gchar* configuration_get_device (Configuration* self);
void configuration_set_device (Configuration* self, const gchar* value);
static void configuration_finalize (GObject* obj);
static void _vala_configuration_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_configuration_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType fake_serializable_get_type (void) G_GNUC_CONST;
#define FAKE_SERIALIZABLE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_FAKE_SERIALIZABLE, FakeSerializablePrivate))
enum  {
	FAKE_SERIALIZABLE_DUMMY_PROPERTY,
	FAKE_SERIALIZABLE_NONE
};
FakeSerializable* fake_serializable_new (void);
FakeSerializable* fake_serializable_construct (GType object_type);
const gchar* fake_serializable_get_none (FakeSerializable* self);
void fake_serializable_set_none (FakeSerializable* self, const gchar* value);
static void fake_serializable_finalize (GObject* obj);
static void _vala_fake_serializable_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_fake_serializable_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType unknown_attribute_get_type (void) G_GNUC_CONST;
#define UNKNOWN_ATTRIBUTE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_UNKNOWN_ATTRIBUTE, UnknownAttributePrivate))
enum  {
	UNKNOWN_ATTRIBUTE_DUMMY_PROPERTY,
	UNKNOWN_ATTRIBUTE_NAME,
	UNKNOWN_ATTRIBUTE_ARRAY,
	UNKNOWN_ATTRIBUTE_FAKE
};
static gboolean unknown_attribute_real_get_enable_unknown_serializable_property (GXmlSerializableObjectModel* base);
UnknownAttribute* unknown_attribute_new (void);
UnknownAttribute* unknown_attribute_construct (GType object_type);
const gchar* unknown_attribute_get_name (UnknownAttribute* self);
void unknown_attribute_set_name (UnknownAttribute* self, const gchar* value);
GeeArrayList* unknown_attribute_get_array (UnknownAttribute* self);
void unknown_attribute_set_array (UnknownAttribute* self, GeeArrayList* value);
FakeSerializable* unknown_attribute_get_fake (UnknownAttribute* self);
void unknown_attribute_set_fake (UnknownAttribute* self, FakeSerializable* value);
static void unknown_attribute_finalize (GObject* obj);
static void _vala_unknown_attribute_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_unknown_attribute_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType name_space_get_type (void) G_GNUC_CONST;
enum  {
	NAME_SPACE_DUMMY_PROPERTY
};
static gboolean name_space_real_set_default_namespace (GXmlSerializableObjectModel* base, GXmlNode* node);
static gchar* name_space_real_to_string (GXmlSerializableObjectModel* base);
NameSpace* name_space_new (void);
NameSpace* name_space_construct (GType object_type);
gpointer gxml_test_ref (gpointer instance);
void gxml_test_unref (gpointer instance);
GParamSpec* param_spec_gxml_test (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_gxml_test (GValue* value, gpointer v_object);
void value_take_gxml_test (GValue* value, gpointer v_object);
gpointer value_get_gxml_test (const GValue* value);
GType gxml_test_get_type (void) G_GNUC_CONST;
GType serializable_object_model_test_get_type (void) G_GNUC_CONST;
enum  {
	SERIALIZABLE_OBJECT_MODEL_TEST_DUMMY_PROPERTY
};
void serializable_object_model_test_add_tests (void);
static void __lambda9_ (void);
static void ___lambda9__gtest_func (void);
static void __lambda10_ (void);
static void ___lambda10__gtest_func (void);
static void __lambda11_ (void);
static void ___lambda11__gtest_func (void);
static void __lambda12_ (void);
static void serializable_object_model_test_serialize_manual_check (GXmlGElement* element, Manual* manual);
static void ___lambda12__gtest_func (void);
static void __lambda13_ (void);
static void ___lambda13__gtest_func (void);
static void __lambda16_ (void);
static void ___lambda16__gtest_func (void);
static void __lambda17_ (void);
static void ___lambda17__gtest_func (void);
static void __lambda18_ (void);
static void ___lambda18__gtest_func (void);
static void __lambda19_ (void);
static void ___lambda19__gtest_func (void);
static void __lambda20_ (void);
static void ___lambda20__gtest_func (void);
static void __lambda21_ (void);
static void ___lambda21__gtest_func (void);
static void __lambda22_ (void);
static void ___lambda22__gtest_func (void);
static void __lambda23_ (void);
static void ___lambda23__gtest_func (void);
static void __lambda24_ (void);
static void ___lambda24__gtest_func (void);
static void __lambda25_ (void);
static void ___lambda25__gtest_func (void);
static void __lambda26_ (void);
static void ___lambda26__gtest_func (void);
static void __lambda27_ (void);
static void ___lambda27__gtest_func (void);
static void __lambda28_ (void);
static void ___lambda28__gtest_func (void);
static void __lambda29_ (void);
static void ___lambda29__gtest_func (void);
SerializableObjectModelTest* serializable_object_model_test_new (void);
SerializableObjectModelTest* serializable_object_model_test_construct (GType object_type);
GXmlTest* gxml_test_new (void);
GXmlTest* gxml_test_construct (GType object_type);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);


static gpointer _g_param_spec_ref0 (gpointer self) {
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return self ? g_param_spec_ref (self) : NULL;
#line 621 "SerializableObjectModelTest.c"
}


static const gchar* string_to_string (const gchar* self) {
	const gchar* result = NULL;
#line 1412 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1413 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	result = self;
#line 1413 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	return result;
#line 633 "SerializableObjectModelTest.c"
}


static gchar* object_model_real_to_string (GXmlSerializableObjectModel* base) {
	ObjectModel * self;
	gchar* result = NULL;
	GParamSpec** lp = NULL;
	gint _tmp0_ = 0;
	GParamSpec** _tmp1_ = NULL;
	gint lp_length1 = 0;
	gint _lp_size_ = 0;
	gchar* ret = NULL;
	GType _tmp2_ = 0UL;
	const gchar* _tmp3_ = NULL;
	gchar* _tmp4_ = NULL;
	GParamSpec** _tmp5_ = NULL;
	gint _tmp5__length1 = 0;
	const gchar* _tmp29_ = NULL;
	gchar* _tmp30_ = NULL;
	GError * _inner_error_ = NULL;
#line 56 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (ObjectModel*) base;
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = gxml_serializable_object_model_list_serializable_properties ((GXmlSerializableObjectModel*) self, &_tmp0_);
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	lp = _tmp1_;
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	lp_length1 = _tmp0_;
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_lp_size_ = lp_length1;
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp2_ = G_TYPE_FROM_INSTANCE ((GObject*) self);
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp3_ = g_type_name (_tmp2_);
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp4_ = g_strconcat (_tmp3_, "{Properties:\n", NULL);
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	ret = _tmp4_;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp5_ = lp;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp5__length1 = lp_length1;
#line 676 "SerializableObjectModelTest.c"
	{
		GParamSpec** p_collection = NULL;
		gint p_collection_length1 = 0;
		gint _p_collection_size_ = 0;
		gint p_it = 0;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		p_collection = _tmp5_;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		p_collection_length1 = _tmp5__length1;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		for (p_it = 0; p_it < _tmp5__length1; p_it = p_it + 1) {
#line 688 "SerializableObjectModelTest.c"
			GParamSpec* _tmp6_ = NULL;
			GParamSpec* p = NULL;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp6_ = _g_param_spec_ref0 (p_collection[p_it]);
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			p = _tmp6_;
#line 695 "SerializableObjectModelTest.c"
			{
				GValue v = {0};
				GParamSpec* _tmp7_ = NULL;
				GType _tmp8_ = 0UL;
				GValue _tmp9_ = {0};
				GParamSpec* _tmp10_ = NULL;
				const gchar* _tmp11_ = NULL;
				gchar* t = NULL;
				const gchar* _tmp17_ = NULL;
				GParamSpec* _tmp18_ = NULL;
				const gchar* _tmp19_ = NULL;
				const gchar* _tmp20_ = NULL;
				gchar* _tmp21_ = NULL;
				gchar* _tmp22_ = NULL;
				const gchar* _tmp23_ = NULL;
				gchar* _tmp24_ = NULL;
				gchar* _tmp25_ = NULL;
				gchar* _tmp26_ = NULL;
				gchar* _tmp27_ = NULL;
				gchar* _tmp28_ = NULL;
#line 61 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp7_ = p;
#line 61 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp8_ = _tmp7_->value_type;
#line 61 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				g_value_init (&_tmp9_, _tmp8_);
#line 61 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				v = _tmp9_;
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp10_ = p;
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp11_ = _tmp10_->name;
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				g_object_get_property ((GObject*) self, _tmp11_, &v);
#line 730 "SerializableObjectModelTest.c"
				{
					gchar* _tmp12_ = NULL;
					GValue _tmp13_ = {0};
					gchar* _tmp14_ = NULL;
					gchar* _tmp15_ = NULL;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp13_ = v;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp14_ = gxml_serializable_gvalue_to_string (&_tmp13_, &_inner_error_);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp12_ = _tmp14_;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 744 "SerializableObjectModelTest.c"
						goto __catch41_g_error;
					}
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp15_ = _tmp12_;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp12_ = NULL;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_free0 (t);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					t = _tmp15_;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_free0 (_tmp12_);
#line 757 "SerializableObjectModelTest.c"
				}
				goto __finally41;
				__catch41_g_error:
				{
					gchar* _tmp16_ = NULL;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					g_clear_error (&_inner_error_);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_inner_error_ = NULL;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp16_ = g_strdup ("[CANT_TRANSFORM]");
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_free0 (t);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					t = _tmp16_;
#line 773 "SerializableObjectModelTest.c"
				}
				__finally41:
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_free0 (t);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					G_IS_VALUE (&v) ? (g_value_unset (&v), NULL) : NULL;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_param_spec_unref0 (p);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_free0 (ret);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					lp = (_vala_array_free (lp, lp_length1, (GDestroyNotify) g_param_spec_unref), NULL);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					g_clear_error (&_inner_error_);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					return NULL;
#line 794 "SerializableObjectModelTest.c"
				}
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp17_ = ret;
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp18_ = p;
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp19_ = _tmp18_->name;
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp20_ = string_to_string (_tmp19_);
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp21_ = g_strconcat ("[", _tmp20_, "]{", NULL);
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp22_ = _tmp21_;
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp23_ = t;
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp24_ = g_strconcat (_tmp22_, _tmp23_, NULL);
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp25_ = _tmp24_;
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp26_ = g_strconcat (_tmp25_, "}\n", NULL);
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp27_ = _tmp26_;
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp28_ = g_strconcat (_tmp17_, _tmp27_, NULL);
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_g_free0 (ret);
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				ret = _tmp28_;
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_g_free0 (_tmp27_);
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_g_free0 (_tmp25_);
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_g_free0 (_tmp22_);
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_g_free0 (t);
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				G_IS_VALUE (&v) ? (g_value_unset (&v), NULL) : NULL;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_g_param_spec_unref0 (p);
#line 836 "SerializableObjectModelTest.c"
			}
		}
	}
#line 67 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp29_ = ret;
#line 67 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp30_ = g_strconcat (_tmp29_, "}", NULL);
#line 67 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp30_;
#line 67 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (ret);
#line 67 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	lp = (_vala_array_free (lp, lp_length1, (GDestroyNotify) g_param_spec_unref), NULL);
#line 67 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 852 "SerializableObjectModelTest.c"
}


ObjectModel* object_model_construct (GType object_type) {
	ObjectModel * self = NULL;
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (ObjectModel*) gxml_serializable_object_model_construct (object_type);
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return self;
#line 862 "SerializableObjectModelTest.c"
}


ObjectModel* object_model_new (void) {
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return object_model_construct (TYPE_OBJECT_MODEL);
#line 869 "SerializableObjectModelTest.c"
}


static void object_model_class_init (ObjectModelClass * klass) {
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	object_model_parent_class = g_type_class_peek_parent (klass);
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) object_model_real_to_string;
#line 878 "SerializableObjectModelTest.c"
}


static void object_model_instance_init (ObjectModel * self) {
}


GType object_model_get_type (void) {
	static volatile gsize object_model_type_id__volatile = 0;
	if (g_once_init_enter (&object_model_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (ObjectModelClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) object_model_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ObjectModel), 0, (GInstanceInitFunc) object_model_instance_init, NULL };
		GType object_model_type_id;
		object_model_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "ObjectModel", &g_define_type_info, 0);
		g_once_init_leave (&object_model_type_id__volatile, object_model_type_id);
	}
	return object_model_type_id__volatile;
}


Computer* computer_construct (GType object_type) {
	Computer * self = NULL;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (Computer*) object_model_construct (object_type);
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	computer_set_manufacturer (self, "MexicanLaptop, Inc.");
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	computer_set_model (self, "LQ59678");
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	computer_set_cores (self, 8);
#line 84 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	computer_set_ghz (self, (gfloat) 3.5);
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return self;
#line 912 "SerializableObjectModelTest.c"
}


Computer* computer_new (void) {
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return computer_construct (TYPE_COMPUTER);
#line 919 "SerializableObjectModelTest.c"
}


const gchar* computer_get_manufacturer (Computer* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_manufacturer;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 934 "SerializableObjectModelTest.c"
}


void computer_set_manufacturer (Computer* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (self->priv->_manufacturer);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_manufacturer = _tmp1_;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "manufacturer");
#line 953 "SerializableObjectModelTest.c"
}


const gchar* computer_get_model (Computer* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_model;
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 968 "SerializableObjectModelTest.c"
}


void computer_set_model (Computer* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (self->priv->_model);
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_model = _tmp1_;
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "model");
#line 987 "SerializableObjectModelTest.c"
}


gint computer_get_cores (Computer* self) {
	gint result;
	gint _tmp0_ = 0;
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_cores;
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 1002 "SerializableObjectModelTest.c"
}


void computer_set_cores (Computer* self, gint value) {
	gint _tmp0_ = 0;
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_cores = _tmp0_;
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "cores");
#line 1016 "SerializableObjectModelTest.c"
}


gfloat computer_get_ghz (Computer* self) {
	gfloat result;
	gfloat _tmp0_ = 0.0F;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, 0.0F);
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_ghz;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 1031 "SerializableObjectModelTest.c"
}


void computer_set_ghz (Computer* self, gfloat value) {
	gfloat _tmp0_ = 0.0F;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_ghz = _tmp0_;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "ghz");
#line 1045 "SerializableObjectModelTest.c"
}


guint computer_get_quantity (Computer* self) {
	guint result;
	guint _tmp0_ = 0U;
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, 0U);
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_quantity;
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 1060 "SerializableObjectModelTest.c"
}


void computer_set_quantity (Computer* self, guint value) {
	guint _tmp0_ = 0U;
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_quantity = _tmp0_;
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "quantity");
#line 1074 "SerializableObjectModelTest.c"
}


static void computer_class_init (ComputerClass * klass) {
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	computer_parent_class = g_type_class_peek_parent (klass);
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_type_class_add_private (klass, sizeof (ComputerPrivate));
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_computer_get_property;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_computer_set_property;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->finalize = computer_finalize;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), COMPUTER_MANUFACTURER, g_param_spec_string ("manufacturer", "Manufacturer", "manufacturer", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), COMPUTER_MODEL, g_param_spec_string ("model", "model", "model", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), COMPUTER_CORES, g_param_spec_int ("cores", "cores", "cores", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), COMPUTER_GHZ, g_param_spec_float ("ghz", "ghz", "ghz", -G_MAXFLOAT, G_MAXFLOAT, 0.0F, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), COMPUTER_QUANTITY, g_param_spec_uint ("quantity", "quantity", "quantity", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 1099 "SerializableObjectModelTest.c"
}


static void computer_instance_init (Computer * self) {
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv = COMPUTER_GET_PRIVATE (self);
#line 1106 "SerializableObjectModelTest.c"
}


static void computer_finalize (GObject* obj) {
	Computer * self;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_COMPUTER, Computer);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (self->priv->_manufacturer);
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (self->priv->_model);
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (computer_parent_class)->finalize (obj);
#line 1120 "SerializableObjectModelTest.c"
}


GType computer_get_type (void) {
	static volatile gsize computer_type_id__volatile = 0;
	if (g_once_init_enter (&computer_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (ComputerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) computer_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Computer), 0, (GInstanceInitFunc) computer_instance_init, NULL };
		GType computer_type_id;
		computer_type_id = g_type_register_static (TYPE_OBJECT_MODEL, "Computer", &g_define_type_info, 0);
		g_once_init_leave (&computer_type_id__volatile, computer_type_id);
	}
	return computer_type_id__volatile;
}


static void _vala_computer_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	Computer * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_COMPUTER, Computer);
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	switch (property_id) {
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case COMPUTER_MANUFACTURER:
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_string (value, computer_get_manufacturer (self));
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case COMPUTER_MODEL:
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_string (value, computer_get_model (self));
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case COMPUTER_CORES:
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_int (value, computer_get_cores (self));
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case COMPUTER_GHZ:
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_float (value, computer_get_ghz (self));
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case COMPUTER_QUANTITY:
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_uint (value, computer_get_quantity (self));
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 1171 "SerializableObjectModelTest.c"
		default:
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 1177 "SerializableObjectModelTest.c"
	}
}


static void _vala_computer_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	Computer * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_COMPUTER, Computer);
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	switch (property_id) {
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case COMPUTER_MANUFACTURER:
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		computer_set_manufacturer (self, g_value_get_string (value));
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case COMPUTER_MODEL:
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		computer_set_model (self, g_value_get_string (value));
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case COMPUTER_CORES:
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		computer_set_cores (self, g_value_get_int (value));
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case COMPUTER_GHZ:
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		computer_set_ghz (self, g_value_get_float (value));
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case COMPUTER_QUANTITY:
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		computer_set_quantity (self, g_value_get_uint (value));
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 1217 "SerializableObjectModelTest.c"
		default:
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 1223 "SerializableObjectModelTest.c"
	}
}


gchar* manual_get_contents (Manual* self) {
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = gxml_serializable_get_serialized_xml_node_value ((GXmlSerializable*) self);
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = _tmp0_;
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp1_;
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 1242 "SerializableObjectModelTest.c"
}


void manual_set_contents (Manual* self, const gchar* val) {
	const gchar* _tmp0_ = NULL;
#line 93 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 93 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (val != NULL);
#line 93 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = val;
#line 93 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	gxml_serializable_set_serialized_xml_node_value ((GXmlSerializable*) self, _tmp0_);
#line 1256 "SerializableObjectModelTest.c"
}


Manual* manual_construct (GType object_type) {
	Manual * self = NULL;
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (Manual*) object_model_construct (object_type);
#line 97 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	manual_set_document (self, "MANUAL DOCUMENTATION");
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	manual_set_pages (self, 3);
#line 99 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	manual_set_contents (self, "TEXT INTO THE MANUAL DOCUMENT");
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return self;
#line 1272 "SerializableObjectModelTest.c"
}


Manual* manual_new (void) {
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return manual_construct (TYPE_MANUAL);
#line 1279 "SerializableObjectModelTest.c"
}


static gchar* manual_real_to_string (GXmlSerializableObjectModel* base) {
	Manual * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	gchar* _tmp2_ = NULL;
	gchar* _tmp3_ = NULL;
	const gchar* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	gchar* _tmp6_ = NULL;
	gchar* _tmp7_ = NULL;
	gchar* _tmp8_ = NULL;
#line 102 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (Manual*) base;
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = GXML_SERIALIZABLE_OBJECT_MODEL_CLASS (manual_parent_class)->to_string ((GXmlSerializableObjectModel*) G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_OBJECT_MODEL, ObjectModel));
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = _tmp0_;
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp2_ = manual_get_contents (self);
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp3_ = _tmp2_;
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp4_ = string_to_string (_tmp3_);
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp5_ = g_strconcat ("CONTENTS: { ", _tmp4_, "}", NULL);
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp6_ = _tmp5_;
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp7_ = g_strconcat (_tmp1_, _tmp6_, NULL);
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp8_ = _tmp7_;
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (_tmp6_);
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (_tmp3_);
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (_tmp1_);
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp8_;
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 1325 "SerializableObjectModelTest.c"
}


static gboolean manual_real_serialize_use_xml_node_value (GXmlSerializableObjectModel* base) {
	Manual * self;
	gboolean result = FALSE;
#line 106 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (Manual*) base;
#line 106 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = TRUE;
#line 106 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 1338 "SerializableObjectModelTest.c"
}


const gchar* manual_get_document (Manual* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_document;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 1353 "SerializableObjectModelTest.c"
}


void manual_set_document (Manual* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (self->priv->_document);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_document = _tmp1_;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "document");
#line 1372 "SerializableObjectModelTest.c"
}


gint manual_get_pages (Manual* self) {
	gint result;
	gint _tmp0_ = 0;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_pages;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 1387 "SerializableObjectModelTest.c"
}


void manual_set_pages (Manual* self, gint value) {
	gint _tmp0_ = 0;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_pages = _tmp0_;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "pages");
#line 1401 "SerializableObjectModelTest.c"
}


static void manual_class_init (ManualClass * klass) {
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	manual_parent_class = g_type_class_peek_parent (klass);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_type_class_add_private (klass, sizeof (ManualPrivate));
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) manual_real_to_string;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->serialize_use_xml_node_value = (gboolean (*)(GXmlSerializableObjectModel*)) manual_real_serialize_use_xml_node_value;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_manual_get_property;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_manual_set_property;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->finalize = manual_finalize;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MANUAL_DOCUMENT, g_param_spec_string ("document", "document", "document", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MANUAL_PAGES, g_param_spec_int ("pages", "pages", "pages", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 1424 "SerializableObjectModelTest.c"
}


static void manual_instance_init (Manual * self) {
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv = MANUAL_GET_PRIVATE (self);
#line 1431 "SerializableObjectModelTest.c"
}


static void manual_finalize (GObject* obj) {
	Manual * self;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_MANUAL, Manual);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (self->priv->_document);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (manual_parent_class)->finalize (obj);
#line 1443 "SerializableObjectModelTest.c"
}


GType manual_get_type (void) {
	static volatile gsize manual_type_id__volatile = 0;
	if (g_once_init_enter (&manual_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (ManualClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) manual_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Manual), 0, (GInstanceInitFunc) manual_instance_init, NULL };
		GType manual_type_id;
		manual_type_id = g_type_register_static (TYPE_OBJECT_MODEL, "Manual", &g_define_type_info, 0);
		g_once_init_leave (&manual_type_id__volatile, manual_type_id);
	}
	return manual_type_id__volatile;
}


static void _vala_manual_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	Manual * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_MANUAL, Manual);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	switch (property_id) {
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case MANUAL_DOCUMENT:
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_string (value, manual_get_document (self));
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case MANUAL_PAGES:
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_int (value, manual_get_pages (self));
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 1476 "SerializableObjectModelTest.c"
		default:
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 1482 "SerializableObjectModelTest.c"
	}
}


static void _vala_manual_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	Manual * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_MANUAL, Manual);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	switch (property_id) {
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case MANUAL_DOCUMENT:
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		manual_set_document (self, g_value_get_string (value));
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case MANUAL_PAGES:
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		manual_set_pages (self, g_value_get_int (value));
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 1504 "SerializableObjectModelTest.c"
		default:
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 1510 "SerializableObjectModelTest.c"
	}
}


static gboolean package_real_property_use_nick (GXmlSerializableObjectModel* base) {
	Package * self;
	gboolean result = FALSE;
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (Package*) base;
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = TRUE;
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 1524 "SerializableObjectModelTest.c"
}


static gboolean package_real_get_enable_unknown_serializable_property (GXmlSerializableObjectModel* base) {
	Package * self;
	gboolean result = FALSE;
#line 120 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (Package*) base;
#line 120 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = TRUE;
#line 120 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 1537 "SerializableObjectModelTest.c"
}


static void __lambda14_ (Package* self, GXmlNode* element, GParamSpec* prop, GXmlNode** node) {
	GXmlNode* _vala_node = NULL;
	GParamSpec* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
	GError * _inner_error_ = NULL;
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (element != NULL);
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (prop != NULL);
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = prop;
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = _tmp0_->name;
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (g_strcmp0 (_tmp1_, "tags") == 0) {
#line 1556 "SerializableObjectModelTest.c"
		{
			gint i = 0;
#line 132 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			i = 0;
#line 1561 "SerializableObjectModelTest.c"
			{
				gboolean _tmp2_ = FALSE;
#line 132 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp2_ = TRUE;
#line 132 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				while (TRUE) {
#line 1568 "SerializableObjectModelTest.c"
					gint _tmp4_ = 0;
					GArray* _tmp5_ = NULL;
					GArray* _tmp6_ = NULL;
					guint _tmp7_ = 0U;
					gchar* str = NULL;
					GArray* _tmp8_ = NULL;
					GArray* _tmp9_ = NULL;
					gint _tmp10_ = 0;
					const gchar* _tmp11_ = NULL;
					gchar* _tmp12_ = NULL;
					GXmlNode* _tmp22_ = NULL;
					const gchar* _tmp23_ = NULL;
					GXmlNode* _tmp24_ = NULL;
					GeeBidirList* _tmp25_ = NULL;
					GeeBidirList* _tmp26_ = NULL;
					GeeBidirList* _tmp27_ = NULL;
					GXmlNode* _tmp28_ = NULL;
#line 132 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					if (!_tmp2_) {
#line 1588 "SerializableObjectModelTest.c"
						gint _tmp3_ = 0;
#line 132 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp3_ = i;
#line 132 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						i = _tmp3_ + 1;
#line 1594 "SerializableObjectModelTest.c"
					}
#line 132 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp2_ = FALSE;
#line 132 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp4_ = i;
#line 132 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp5_ = package_get_tags (self);
#line 132 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp6_ = _tmp5_;
#line 132 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp7_ = _tmp6_->len;
#line 132 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					if (!(((guint) _tmp4_) < _tmp7_)) {
#line 132 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						break;
#line 1610 "SerializableObjectModelTest.c"
					}
#line 133 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp8_ = package_get_tags (self);
#line 133 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp9_ = _tmp8_;
#line 133 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp10_ = i;
#line 133 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp11_ = g_array_index (_tmp9_, gchar*, (guint) _tmp10_);
#line 133 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp12_ = g_strdup (_tmp11_);
#line 133 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					str = _tmp12_;
#line 1624 "SerializableObjectModelTest.c"
					{
						GXmlNode* _tmp13_ = NULL;
						GXmlNode* _tmp14_ = NULL;
						GXmlDocument* _tmp15_ = NULL;
						GXmlDocument* _tmp16_ = NULL;
						GXmlNode* _tmp17_ = NULL;
						GXmlNode* _tmp18_ = NULL;
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp14_ = element;
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp15_ = gxml_node_get_document (_tmp14_);
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp16_ = _tmp15_;
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp17_ = gxml_document_create_element (_tmp16_, "tag", &_inner_error_);
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp13_ = _tmp17_;
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1644 "SerializableObjectModelTest.c"
							goto __catch42_g_error;
						}
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp18_ = _tmp13_;
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp13_ = NULL;
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_g_object_unref0 (_vala_node);
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_vala_node = (GXmlNode*) G_TYPE_CHECK_INSTANCE_CAST (_tmp18_, GXML_TYPE_ELEMENT, GXmlElement);
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_g_object_unref0 (_tmp13_);
#line 1657 "SerializableObjectModelTest.c"
					}
					goto __finally42;
					__catch42_g_error:
					{
						GError* e = NULL;
						const gchar* _tmp19_ = NULL;
						gchar* _tmp20_ = NULL;
						gchar* _tmp21_ = NULL;
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						e = _inner_error_;
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_inner_error_ = NULL;
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp19_ = e->message;
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp20_ = g_strconcat ("Error: ", _tmp19_, NULL);
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp21_ = _tmp20_;
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						g_message ("SerializableObjectModelTest.vala:136: %s", _tmp21_);
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_g_free0 (_tmp21_);
#line 137 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						g_assert_not_reached ();
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_g_error_free0 (e);
#line 1684 "SerializableObjectModelTest.c"
					}
					__finally42:
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_g_free0 (str);
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						g_clear_error (&_inner_error_);
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						return;
#line 1697 "SerializableObjectModelTest.c"
					}
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp22_ = _vala_node;
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp23_ = str;
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					gxml_element_set_content (G_TYPE_CHECK_INSTANCE_CAST (_tmp22_, GXML_TYPE_ELEMENT, GXmlElement), _tmp23_);
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp24_ = element;
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp25_ = gxml_node_get_children_nodes (_tmp24_);
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp26_ = _tmp25_;
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp27_ = _tmp26_;
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp28_ = _vala_node;
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					gee_collection_add ((GeeCollection*) _tmp27_, _tmp28_);
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_object_unref0 (_tmp27_);
#line 132 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_free0 (str);
#line 1721 "SerializableObjectModelTest.c"
				}
			}
		}
	}
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (node) {
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		*node = _vala_node;
#line 1730 "SerializableObjectModelTest.c"
	} else {
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_vala_node);
#line 1734 "SerializableObjectModelTest.c"
	}
}


static void ___lambda14__gxml_serializable_serialize_unknown_property (GXmlSerializable* _sender, GXmlNode* element, GParamSpec* prop, GXmlNode** node, gpointer self) {
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	__lambda14_ ((Package*) self, element, prop, node);
#line 1742 "SerializableObjectModelTest.c"
}


static void __lambda15_ (Package* self, GXmlNode* element, GParamSpec* prop) {
	GXmlNode* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	gchar* _tmp2_ = NULL;
	gchar* _tmp3_ = NULL;
	gboolean _tmp4_ = FALSE;
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (element != NULL);
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (prop != NULL);
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = element;
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = gxml_node_get_name (_tmp0_);
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp2_ = _tmp1_;
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp3_ = _tmp2_;
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp4_ = g_strcmp0 (_tmp3_, "tag") == 0;
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (_tmp3_);
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (_tmp4_) {
#line 1770 "SerializableObjectModelTest.c"
		GArray* _tmp5_ = NULL;
		GArray* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		gchar* _tmp8_ = NULL;
		gchar* _tmp9_ = NULL;
#line 147 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = package_get_tags (self);
#line 147 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = _tmp5_;
#line 147 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp7_ = element;
#line 147 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp8_ = gxml_element_get_content ((GXmlElement*) G_TYPE_CHECK_INSTANCE_CAST (_tmp7_, GXML_TYPE_GELEMENT, GXmlGElement));
#line 147 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp9_ = _tmp8_;
#line 147 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_array_append_val (_tmp6_, _tmp9_);
#line 1788 "SerializableObjectModelTest.c"
	}
}


static void ___lambda15__gxml_serializable_deserialize_unknown_property (GXmlSerializable* _sender, GXmlNode* node, GParamSpec* prop, gpointer self) {
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	__lambda15_ ((Package*) self, node, prop);
#line 1796 "SerializableObjectModelTest.c"
}


Package* package_construct (GType object_type) {
	Package * self = NULL;
	Computer* _tmp0_ = NULL;
	Computer* _tmp1_ = NULL;
	Manual* _tmp2_ = NULL;
	Manual* _tmp3_ = NULL;
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (Package*) object_model_construct (object_type);
#line 124 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = computer_new ();
#line 124 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = _tmp0_;
#line 124 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	package_set_computer (self, _tmp1_);
#line 124 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (_tmp1_);
#line 125 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp2_ = manual_new ();
#line 125 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp3_ = _tmp2_;
#line 125 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	package_set_manual (self, _tmp3_);
#line 125 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (_tmp3_);
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	package_set_source (self, "Mexico");
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	package_set_destiny (self, "World");
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (self, GXML_TYPE_SERIALIZABLE, GXmlSerializable), "serialize-unknown-property", (GCallback) ___lambda14__gxml_serializable_serialize_unknown_property, self, 0);
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_signal_connect_object (G_TYPE_CHECK_INSTANCE_CAST (self, GXML_TYPE_SERIALIZABLE, GXmlSerializable), "deserialize-unknown-property", (GCallback) ___lambda15__gxml_serializable_deserialize_unknown_property, self, 0);
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return self;
#line 1834 "SerializableObjectModelTest.c"
}


Package* package_new (void) {
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return package_construct (TYPE_PACKAGE);
#line 1841 "SerializableObjectModelTest.c"
}


gchar* package_unknown_to_string (Package* self) {
	gchar* result = NULL;
	gchar* t = NULL;
	gchar* _tmp0_ = NULL;
	const gchar* _tmp18_ = NULL;
	const gchar* _tmp19_ = NULL;
	gchar* _tmp20_ = NULL;
#line 152 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = g_strdup ("");
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	t = _tmp0_;
#line 1858 "SerializableObjectModelTest.c"
	{
		GeeIterator* _node_it = NULL;
		GeeMap* _tmp1_ = NULL;
		GeeMap* _tmp2_ = NULL;
		GeeMap* _tmp3_ = NULL;
		GeeCollection* _tmp4_ = NULL;
		GeeCollection* _tmp5_ = NULL;
		GeeCollection* _tmp6_ = NULL;
		GeeIterator* _tmp7_ = NULL;
		GeeIterator* _tmp8_ = NULL;
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp1_ = gxml_serializable_get_unknown_serializable_properties ((GXmlSerializable*) self);
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp2_ = _tmp1_;
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp3_ = _tmp2_;
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp4_ = gee_map_get_values (_tmp3_);
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = _tmp4_;
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = _tmp5_;
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp7_ = gee_iterable_iterator ((GeeIterable*) _tmp6_);
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp8_ = _tmp7_;
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp6_);
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp3_);
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_node_it = _tmp8_;
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		while (TRUE) {
#line 1893 "SerializableObjectModelTest.c"
			GeeIterator* _tmp9_ = NULL;
			gboolean _tmp10_ = FALSE;
			GXmlAttribute* node = NULL;
			GeeIterator* _tmp11_ = NULL;
			gpointer _tmp12_ = NULL;
			const gchar* _tmp13_ = NULL;
			GXmlAttribute* _tmp14_ = NULL;
			gchar* _tmp15_ = NULL;
			gchar* _tmp16_ = NULL;
			gchar* _tmp17_ = NULL;
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp9_ = _node_it;
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp10_ = gee_iterator_next (_tmp9_);
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			if (!_tmp10_) {
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				break;
#line 1912 "SerializableObjectModelTest.c"
			}
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp11_ = _node_it;
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp12_ = gee_iterator_get (_tmp11_);
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			node = (GXmlAttribute*) _tmp12_;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp13_ = t;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp14_ = node;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp15_ = gxml_node_to_string ((GXmlNode*) _tmp14_);
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp16_ = _tmp15_;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp17_ = g_strconcat (_tmp13_, _tmp16_, NULL);
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (t);
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			t = _tmp17_;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp16_);
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (node);
#line 1938 "SerializableObjectModelTest.c"
		}
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_node_it);
#line 1942 "SerializableObjectModelTest.c"
	}
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp18_ = t;
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp19_ = string_to_string (_tmp18_);
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp20_ = g_strconcat ("Unknown Properties: {", _tmp19_, "}", NULL);
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp20_;
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (t);
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 1956 "SerializableObjectModelTest.c"
}


Computer* package_get_computer (Package* self) {
	Computer* result;
	Computer* _tmp0_ = NULL;
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_computer;
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 1971 "SerializableObjectModelTest.c"
}


static gpointer _g_object_ref0 (gpointer self) {
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return self ? g_object_ref (self) : NULL;
#line 1978 "SerializableObjectModelTest.c"
}


void package_set_computer (Package* self, Computer* value) {
	Computer* _tmp0_ = NULL;
	Computer* _tmp1_ = NULL;
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (self->priv->_computer);
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_computer = _tmp1_;
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "computer");
#line 1997 "SerializableObjectModelTest.c"
}


Manual* package_get_manual (Package* self) {
	Manual* result;
	Manual* _tmp0_ = NULL;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_manual;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 2012 "SerializableObjectModelTest.c"
}


void package_set_manual (Package* self, Manual* value) {
	Manual* _tmp0_ = NULL;
	Manual* _tmp1_ = NULL;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (self->priv->_manual);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_manual = _tmp1_;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "manual");
#line 2031 "SerializableObjectModelTest.c"
}


const gchar* package_get_source (Package* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_source;
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 2046 "SerializableObjectModelTest.c"
}


void package_set_source (Package* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (self->priv->_source);
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_source = _tmp1_;
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "source");
#line 2065 "SerializableObjectModelTest.c"
}


const gchar* package_get_destiny (Package* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_destiny;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 2080 "SerializableObjectModelTest.c"
}


void package_set_destiny (Package* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (self->priv->_destiny);
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_destiny = _tmp1_;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "destiny");
#line 2099 "SerializableObjectModelTest.c"
}


GArray* package_get_tags (Package* self) {
	GArray* result;
	GArray* _tmp0_ = NULL;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_tags;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 2114 "SerializableObjectModelTest.c"
}


static void package_class_init (PackageClass * klass) {
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	package_parent_class = g_type_class_peek_parent (klass);
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_type_class_add_private (klass, sizeof (PackagePrivate));
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->property_use_nick = (gboolean (*)(GXmlSerializableObjectModel*)) package_real_property_use_nick;
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->get_enable_unknown_serializable_property = (gboolean (*)(GXmlSerializableObjectModel*)) package_real_get_enable_unknown_serializable_property;
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_package_get_property;
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_package_set_property;
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->finalize = package_finalize;
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), PACKAGE_COMPUTER, g_param_spec_object ("computer", "computer", "computer", TYPE_COMPUTER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), PACKAGE_MANUAL, g_param_spec_object ("manual", "manual", "manual", TYPE_MANUAL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), PACKAGE_SOURCE, g_param_spec_string ("source", "source", "source", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), PACKAGE_DESTINY, g_param_spec_string ("destiny", "destiny", "destiny", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), PACKAGE_TAGS, g_param_spec_boxed ("tags", "tag", "tags in package", G_TYPE_ARRAY, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
#line 2143 "SerializableObjectModelTest.c"
}


static void package_instance_init (Package * self) {
	GArray* _tmp0_ = NULL;
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv = PACKAGE_GET_PRIVATE (self);
#line 111 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = g_array_new (TRUE, TRUE, sizeof (gchar*));
#line 111 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_tags = _tmp0_;
#line 2155 "SerializableObjectModelTest.c"
}


static void package_finalize (GObject* obj) {
	Package * self;
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_PACKAGE, Package);
#line 111 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_array_unref0 (self->priv->_tags);
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (self->priv->_computer);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (self->priv->_manual);
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (self->priv->_source);
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (self->priv->_destiny);
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (package_parent_class)->finalize (obj);
#line 2175 "SerializableObjectModelTest.c"
}


GType package_get_type (void) {
	static volatile gsize package_type_id__volatile = 0;
	if (g_once_init_enter (&package_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (PackageClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) package_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Package), 0, (GInstanceInitFunc) package_instance_init, NULL };
		GType package_type_id;
		package_type_id = g_type_register_static (TYPE_OBJECT_MODEL, "Package", &g_define_type_info, 0);
		g_once_init_leave (&package_type_id__volatile, package_type_id);
	}
	return package_type_id__volatile;
}


static void _vala_package_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	Package * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_PACKAGE, Package);
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	switch (property_id) {
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case PACKAGE_COMPUTER:
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_object (value, package_get_computer (self));
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case PACKAGE_MANUAL:
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_object (value, package_get_manual (self));
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case PACKAGE_SOURCE:
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_string (value, package_get_source (self));
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case PACKAGE_DESTINY:
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_string (value, package_get_destiny (self));
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case PACKAGE_TAGS:
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_boxed (value, package_get_tags (self));
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 2226 "SerializableObjectModelTest.c"
		default:
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 2232 "SerializableObjectModelTest.c"
	}
}


static void _vala_package_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	Package * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_PACKAGE, Package);
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	switch (property_id) {
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case PACKAGE_COMPUTER:
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		package_set_computer (self, g_value_get_object (value));
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case PACKAGE_MANUAL:
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		package_set_manual (self, g_value_get_object (value));
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case PACKAGE_SOURCE:
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		package_set_source (self, g_value_get_string (value));
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case PACKAGE_DESTINY:
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		package_set_destiny (self, g_value_get_string (value));
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 2266 "SerializableObjectModelTest.c"
		default:
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 2272 "SerializableObjectModelTest.c"
	}
}


static gboolean monitor_real_property_use_nick (GXmlSerializableObjectModel* base) {
	Monitor * self;
	gboolean result = FALSE;
#line 171 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (Monitor*) base;
#line 171 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = TRUE;
#line 171 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 2286 "SerializableObjectModelTest.c"
}


Monitor* monitor_construct (GType object_type) {
	Monitor * self = NULL;
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (Monitor*) object_model_construct (object_type);
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return self;
#line 2296 "SerializableObjectModelTest.c"
}


Monitor* monitor_new (void) {
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return monitor_construct (TYPE_MONITOR);
#line 2303 "SerializableObjectModelTest.c"
}


const gchar* monitor_get_resolution (Monitor* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_resolution;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 2318 "SerializableObjectModelTest.c"
}


void monitor_set_resolution (Monitor* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (self->priv->_resolution);
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_resolution = _tmp1_;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "resolution");
#line 2337 "SerializableObjectModelTest.c"
}


gint monitor_get_ac_power (Monitor* self) {
	gint result;
	gint _tmp0_ = 0;
#line 167 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 167 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_ac_power;
#line 167 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 167 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 2352 "SerializableObjectModelTest.c"
}


void monitor_set_ac_power (Monitor* self, gint value) {
	gint _tmp0_ = 0;
#line 167 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 167 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 167 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_ac_power = _tmp0_;
#line 167 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "ac-power");
#line 2366 "SerializableObjectModelTest.c"
}


gint monitor_get_dc_power (Monitor* self) {
	gint result;
	gint _tmp0_ = 0;
#line 169 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 169 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_dc_power;
#line 169 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 169 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 2381 "SerializableObjectModelTest.c"
}


void monitor_set_dc_power (Monitor* self, gint value) {
	gint _tmp0_ = 0;
#line 169 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 169 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 169 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_dc_power = _tmp0_;
#line 169 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "dc-power");
#line 2395 "SerializableObjectModelTest.c"
}


static void monitor_class_init (MonitorClass * klass) {
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	monitor_parent_class = g_type_class_peek_parent (klass);
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_type_class_add_private (klass, sizeof (MonitorPrivate));
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->property_use_nick = (gboolean (*)(GXmlSerializableObjectModel*)) monitor_real_property_use_nick;
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_monitor_get_property;
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_monitor_set_property;
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->finalize = monitor_finalize;
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MONITOR_RESOLUTION, g_param_spec_string ("resolution", "resolution", "resolution", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MONITOR_AC_POWER, g_param_spec_int ("ac-power", "AcPower", "ac-power", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MONITOR_DC_POWER, g_param_spec_int ("dc-power", "DcPower", "dc-power", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 2418 "SerializableObjectModelTest.c"
}


static void monitor_instance_init (Monitor * self) {
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv = MONITOR_GET_PRIVATE (self);
#line 2425 "SerializableObjectModelTest.c"
}


static void monitor_finalize (GObject* obj) {
	Monitor * self;
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_MONITOR, Monitor);
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (self->priv->_resolution);
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (monitor_parent_class)->finalize (obj);
#line 2437 "SerializableObjectModelTest.c"
}


GType monitor_get_type (void) {
	static volatile gsize monitor_type_id__volatile = 0;
	if (g_once_init_enter (&monitor_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (MonitorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) monitor_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Monitor), 0, (GInstanceInitFunc) monitor_instance_init, NULL };
		GType monitor_type_id;
		monitor_type_id = g_type_register_static (TYPE_OBJECT_MODEL, "Monitor", &g_define_type_info, 0);
		g_once_init_leave (&monitor_type_id__volatile, monitor_type_id);
	}
	return monitor_type_id__volatile;
}


static void _vala_monitor_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	Monitor * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_MONITOR, Monitor);
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	switch (property_id) {
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case MONITOR_RESOLUTION:
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_string (value, monitor_get_resolution (self));
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case MONITOR_AC_POWER:
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_int (value, monitor_get_ac_power (self));
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case MONITOR_DC_POWER:
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_int (value, monitor_get_dc_power (self));
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 2476 "SerializableObjectModelTest.c"
		default:
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 2482 "SerializableObjectModelTest.c"
	}
}


static void _vala_monitor_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	Monitor * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_MONITOR, Monitor);
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	switch (property_id) {
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case MONITOR_RESOLUTION:
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		monitor_set_resolution (self, g_value_get_string (value));
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case MONITOR_AC_POWER:
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		monitor_set_ac_power (self, g_value_get_int (value));
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case MONITOR_DC_POWER:
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		monitor_set_dc_power (self, g_value_get_int (value));
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 2510 "SerializableObjectModelTest.c"
		default:
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 2516 "SerializableObjectModelTest.c"
	}
}


Cpu* cpu_construct (GType object_type) {
	Cpu * self = NULL;
	GeeArrayList* _tmp0_ = NULL;
	GeeArrayList* _tmp1_ = NULL;
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (Cpu*) object_model_construct (object_type);
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = gee_array_list_new (G_TYPE_INT, NULL, NULL, NULL, NULL, NULL);
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = _tmp0_;
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	cpu_set_piles (self, _tmp1_);
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (_tmp1_);
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return self;
#line 2537 "SerializableObjectModelTest.c"
}


Cpu* cpu_new (void) {
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return cpu_construct (TYPE_CPU);
#line 2544 "SerializableObjectModelTest.c"
}


gchar* cpu_piles_to_string (Cpu* self) {
	gchar* result = NULL;
	gchar* str = NULL;
	gchar* _tmp0_ = NULL;
	gint i = 0;
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 214 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = g_strdup ("");
#line 214 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	str = _tmp0_;
#line 215 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	i = 0;
#line 216 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	while (TRUE) {
#line 2563 "SerializableObjectModelTest.c"
		gint _tmp1_ = 0;
		GeeArrayList* _tmp2_ = NULL;
		gint _tmp3_ = 0;
		gint _tmp4_ = 0;
		const gchar* _tmp5_ = NULL;
		GeeArrayList* _tmp6_ = NULL;
		gint _tmp7_ = 0;
		gpointer _tmp8_ = NULL;
		gchar* _tmp9_ = NULL;
		gchar* _tmp10_ = NULL;
		gchar* _tmp11_ = NULL;
		gint _tmp12_ = 0;
		GeeArrayList* _tmp13_ = NULL;
		gint _tmp14_ = 0;
		gint _tmp15_ = 0;
		gint _tmp18_ = 0;
#line 216 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp1_ = i;
#line 216 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp2_ = self->priv->_piles;
#line 216 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp3_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp2_);
#line 216 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp4_ = _tmp3_;
#line 216 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (!(_tmp1_ < _tmp4_)) {
#line 216 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			break;
#line 2592 "SerializableObjectModelTest.c"
		}
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = str;
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = self->priv->_piles;
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp7_ = i;
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp8_ = gee_abstract_list_get ((GeeAbstractList*) _tmp6_, _tmp7_);
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp9_ = g_strdup_printf ("%i", (gint) ((gintptr) _tmp8_));
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp10_ = _tmp9_;
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp11_ = g_strconcat (_tmp5_, _tmp10_, NULL);
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (str);
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		str = _tmp11_;
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp10_);
#line 218 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp12_ = i;
#line 218 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp13_ = self->priv->_piles;
#line 218 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp14_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp13_);
#line 218 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp15_ = _tmp14_;
#line 218 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if ((_tmp12_ + 1) < _tmp15_) {
#line 2624 "SerializableObjectModelTest.c"
			const gchar* _tmp16_ = NULL;
			gchar* _tmp17_ = NULL;
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp16_ = str;
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp17_ = g_strconcat (_tmp16_, ",", NULL);
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (str);
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			str = _tmp17_;
#line 2635 "SerializableObjectModelTest.c"
		}
#line 220 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp18_ = i;
#line 220 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		i = _tmp18_ + 1;
#line 2641 "SerializableObjectModelTest.c"
	}
#line 222 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = str;
#line 222 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 2647 "SerializableObjectModelTest.c"
}


gfloat cpu_get_ghz (Cpu* self) {
	gfloat result;
	gfloat _tmp0_ = 0.0F;
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, 0.0F);
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_ghz;
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 2662 "SerializableObjectModelTest.c"
}


void cpu_set_ghz (Cpu* self, gfloat value) {
	gfloat _tmp0_ = 0.0F;
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_ghz = _tmp0_;
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "ghz");
#line 2676 "SerializableObjectModelTest.c"
}


GeeArrayList* cpu_get_piles (Cpu* self) {
	GeeArrayList* result;
	GeeArrayList* _tmp0_ = NULL;
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_piles;
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 2691 "SerializableObjectModelTest.c"
}


void cpu_set_piles (Cpu* self, GeeArrayList* value) {
	GeeArrayList* _tmp0_ = NULL;
	GeeArrayList* _tmp1_ = NULL;
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (self->priv->_piles);
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_piles = _tmp1_;
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "piles");
#line 2710 "SerializableObjectModelTest.c"
}


static void cpu_class_init (CpuClass * klass) {
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	cpu_parent_class = g_type_class_peek_parent (klass);
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_type_class_add_private (klass, sizeof (CpuPrivate));
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_cpu_get_property;
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_cpu_set_property;
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->finalize = cpu_finalize;
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CPU_GHZ, g_param_spec_float ("ghz", "ghz", "ghz", -G_MAXFLOAT, G_MAXFLOAT, (gfloat) 0.0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CPU_PILES, g_param_spec_object ("piles", "piles", "piles", GEE_TYPE_ARRAY_LIST, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 2729 "SerializableObjectModelTest.c"
}


static void cpu_instance_init (Cpu * self) {
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv = CPU_GET_PRIVATE (self);
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_ghz = (gfloat) 0.0;
#line 2738 "SerializableObjectModelTest.c"
}


static void cpu_finalize (GObject* obj) {
	Cpu * self;
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_CPU, Cpu);
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (self->priv->_piles);
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (cpu_parent_class)->finalize (obj);
#line 2750 "SerializableObjectModelTest.c"
}


GType cpu_get_type (void) {
	static volatile gsize cpu_type_id__volatile = 0;
	if (g_once_init_enter (&cpu_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (CpuClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) cpu_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Cpu), 0, (GInstanceInitFunc) cpu_instance_init, NULL };
		GType cpu_type_id;
		cpu_type_id = g_type_register_static (TYPE_OBJECT_MODEL, "Cpu", &g_define_type_info, 0);
		g_once_init_leave (&cpu_type_id__volatile, cpu_type_id);
	}
	return cpu_type_id__volatile;
}


static void _vala_cpu_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	Cpu * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_CPU, Cpu);
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	switch (property_id) {
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case CPU_GHZ:
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_float (value, cpu_get_ghz (self));
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case CPU_PILES:
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_object (value, cpu_get_piles (self));
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 2783 "SerializableObjectModelTest.c"
		default:
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 2789 "SerializableObjectModelTest.c"
	}
}


static void _vala_cpu_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	Cpu * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_CPU, Cpu);
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	switch (property_id) {
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case CPU_GHZ:
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		cpu_set_ghz (self, g_value_get_float (value));
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case CPU_PILES:
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		cpu_set_piles (self, g_value_get_object (value));
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 2811 "SerializableObjectModelTest.c"
		default:
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 2817 "SerializableObjectModelTest.c"
	}
}


static gchar* node_name_real_node_name (GXmlSerializableObjectModel* base) {
	NodeName * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (NodeName*) base;
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = g_strdup ("NodeName");
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 2834 "SerializableObjectModelTest.c"
}


NodeName* node_name_construct (GType object_type) {
	NodeName * self = NULL;
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (NodeName*) object_model_construct (object_type);
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return self;
#line 2844 "SerializableObjectModelTest.c"
}


NodeName* node_name_new (void) {
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return node_name_construct (TYPE_NODE_NAME);
#line 2851 "SerializableObjectModelTest.c"
}


gboolean node_name_get_invalid (NodeName* self) {
	gboolean result;
	gboolean _tmp0_ = FALSE;
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_invalid;
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 2866 "SerializableObjectModelTest.c"
}


void node_name_set_invalid (NodeName* self, gboolean value) {
	gboolean _tmp0_ = FALSE;
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_invalid = _tmp0_;
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "invalid");
#line 2880 "SerializableObjectModelTest.c"
}


static void node_name_class_init (NodeNameClass * klass) {
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	node_name_parent_class = g_type_class_peek_parent (klass);
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_type_class_add_private (klass, sizeof (NodeNamePrivate));
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) node_name_real_node_name;
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_node_name_get_property;
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_node_name_set_property;
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->finalize = node_name_finalize;
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), NODE_NAME_INVALID, g_param_spec_boolean ("invalid", "invalid", "invalid", TRUE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 2899 "SerializableObjectModelTest.c"
}


static void node_name_instance_init (NodeName * self) {
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv = NODE_NAME_GET_PRIVATE (self);
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_invalid = TRUE;
#line 2908 "SerializableObjectModelTest.c"
}


static void node_name_finalize (GObject* obj) {
	NodeName * self;
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_NODE_NAME, NodeName);
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (node_name_parent_class)->finalize (obj);
#line 2918 "SerializableObjectModelTest.c"
}


GType node_name_get_type (void) {
	static volatile gsize node_name_type_id__volatile = 0;
	if (g_once_init_enter (&node_name_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (NodeNameClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) node_name_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (NodeName), 0, (GInstanceInitFunc) node_name_instance_init, NULL };
		GType node_name_type_id;
		node_name_type_id = g_type_register_static (TYPE_OBJECT_MODEL, "NodeName", &g_define_type_info, 0);
		g_once_init_leave (&node_name_type_id__volatile, node_name_type_id);
	}
	return node_name_type_id__volatile;
}


static void _vala_node_name_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	NodeName * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_NODE_NAME, NodeName);
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	switch (property_id) {
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case NODE_NAME_INVALID:
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_boolean (value, node_name_get_invalid (self));
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 2945 "SerializableObjectModelTest.c"
		default:
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 2951 "SerializableObjectModelTest.c"
	}
}


static void _vala_node_name_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	NodeName * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_NODE_NAME, NodeName);
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	switch (property_id) {
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case NODE_NAME_INVALID:
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		node_name_set_invalid (self, g_value_get_boolean (value));
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 2967 "SerializableObjectModelTest.c"
		default:
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 2973 "SerializableObjectModelTest.c"
	}
}


static gchar* configuration_real_node_name (GXmlSerializableObjectModel* base) {
	Configuration * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 236 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (Configuration*) base;
#line 236 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = g_strdup ("Configuration");
#line 236 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 236 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 2990 "SerializableObjectModelTest.c"
}


static gboolean configuration_real_property_use_nick (GXmlSerializableObjectModel* base) {
	Configuration * self;
	gboolean result = FALSE;
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (Configuration*) base;
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = TRUE;
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 3003 "SerializableObjectModelTest.c"
}


static void _g_free0_ (gpointer var) {
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	var = (g_free (var), NULL);
#line 3010 "SerializableObjectModelTest.c"
}


static void _g_param_spec_unref0_ (gpointer var) {
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	(var == NULL) ? NULL : (var = (g_param_spec_unref (var), NULL));
#line 3017 "SerializableObjectModelTest.c"
}


static void _vala_array_add1 (GParamSpec*** array, int* length, int* size, GParamSpec* value) {
#line 247 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if ((*length) == (*size)) {
#line 247 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 247 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		*array = g_renew (GParamSpec*, *array, (*size) + 1);
#line 3028 "SerializableObjectModelTest.c"
	}
#line 247 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	(*array)[(*length)++] = value;
#line 247 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	(*array)[*length] = NULL;
#line 3034 "SerializableObjectModelTest.c"
}


static GParamSpec** configuration_real_list_serializable_properties (GXmlSerializableObjectModel* base, int* result_length1) {
	Configuration * self;
	GParamSpec** result = NULL;
	GParamSpec** props = NULL;
	GParamSpec** _tmp0_ = NULL;
	gint props_length1 = 0;
	gint _props_size_ = 0;
	GHashTable* l = NULL;
	GHashFunc _tmp1_ = NULL;
	GEqualFunc _tmp2_ = NULL;
	GHashTable* _tmp3_ = NULL;
	GHashTable* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	GObjectClass* _tmp6_ = NULL;
	GParamSpec* _tmp7_ = NULL;
	GParamSpec* _tmp8_ = NULL;
	gint _tmp9_ = 0;
	GParamSpec** _tmp10_ = NULL;
	GParamSpec** _tmp19_ = NULL;
	gint _tmp19__length1 = 0;
#line 239 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (Configuration*) base;
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = g_new0 (GParamSpec*, 0 + 1);
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	props = _tmp0_;
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	props_length1 = 0;
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_props_size_ = props_length1;
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = g_str_hash;
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp2_ = g_str_equal;
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp3_ = g_hash_table_new_full (_tmp1_, _tmp2_, _g_free0_, _g_param_spec_unref0_);
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	l = _tmp3_;
#line 243 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp4_ = l;
#line 243 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp5_ = g_strdup ("invalid");
#line 243 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp6_ = G_OBJECT_GET_CLASS ((GObject*) self);
#line 243 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp7_ = g_object_class_find_property (_tmp6_, "invalid");
#line 243 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp8_ = _g_param_spec_ref0 (_tmp7_);
#line 243 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_hash_table_insert (_tmp4_, _tmp5_, _tmp8_);
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp10_ = gxml_serializable_default_list_serializable_properties ((GXmlSerializable*) self, &_tmp9_);
#line 3090 "SerializableObjectModelTest.c"
	{
		GParamSpec** spec_collection = NULL;
		gint spec_collection_length1 = 0;
		gint _spec_collection_size_ = 0;
		gint spec_it = 0;
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		spec_collection = _tmp10_;
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		spec_collection_length1 = _tmp9_;
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		for (spec_it = 0; spec_it < _tmp9_; spec_it = spec_it + 1) {
#line 3102 "SerializableObjectModelTest.c"
			GParamSpec* _tmp11_ = NULL;
			GParamSpec* spec = NULL;
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp11_ = _g_param_spec_ref0 (spec_collection[spec_it]);
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			spec = _tmp11_;
#line 3109 "SerializableObjectModelTest.c"
			{
				GHashTable* _tmp12_ = NULL;
				GParamSpec* _tmp13_ = NULL;
				const gchar* _tmp14_ = NULL;
				gboolean _tmp15_ = FALSE;
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp12_ = l;
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp13_ = spec;
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp14_ = _tmp13_->name;
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp15_ = g_hash_table_contains (_tmp12_, _tmp14_);
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				if (!_tmp15_) {
#line 3125 "SerializableObjectModelTest.c"
					GParamSpec** _tmp16_ = NULL;
					gint _tmp16__length1 = 0;
					GParamSpec* _tmp17_ = NULL;
					GParamSpec* _tmp18_ = NULL;
#line 247 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp16_ = props;
#line 247 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp16__length1 = props_length1;
#line 247 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp17_ = spec;
#line 247 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp18_ = _g_param_spec_ref0 (_tmp17_);
#line 247 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_vala_array_add1 (&props, &props_length1, &_props_size_, _tmp18_);
#line 3140 "SerializableObjectModelTest.c"
				}
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_g_param_spec_unref0 (spec);
#line 3144 "SerializableObjectModelTest.c"
			}
		}
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		spec_collection = (_vala_array_free (spec_collection, spec_collection_length1, (GDestroyNotify) g_param_spec_unref), NULL);
#line 3149 "SerializableObjectModelTest.c"
	}
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp19_ = props;
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp19__length1 = props_length1;
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (result_length1) {
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		*result_length1 = _tmp19__length1;
#line 3159 "SerializableObjectModelTest.c"
	}
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp19_;
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_hash_table_unref0 (l);
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 3167 "SerializableObjectModelTest.c"
}


static GXmlNode* configuration_real_serialize (GXmlSerializableObjectModel* base, GXmlNode* node, GError** error) {
	Configuration * self;
	GXmlNode* result = NULL;
	GXmlNode* n = NULL;
	GXmlNode* _tmp0_ = NULL;
	GXmlNode* _tmp1_ = NULL;
	GXmlNode* _tmp2_ = NULL;
	GError * _inner_error_ = NULL;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (Configuration*) base;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (node != NULL, NULL);
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = node;
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = gxml_serializable_object_model_default_serialize ((GXmlSerializableObjectModel*) self, _tmp0_, &_inner_error_);
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	n = _tmp1_;
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_propagate_error (error, _inner_error_);
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		return NULL;
#line 3195 "SerializableObjectModelTest.c"
	}
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	gxml_node_set_namespace (n, "http://www.gnome.org/gxml/0.4", "om");
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp2_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (n, GXML_TYPE_NODE, GXmlNode));
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp2_;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (n);
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 3207 "SerializableObjectModelTest.c"
}


static gboolean configuration_real_deserialize (GXmlSerializableObjectModel* base, GXmlNode* node, GError** error) {
	Configuration * self;
	gboolean result = FALSE;
	GXmlNode* n = NULL;
	GXmlNode* _tmp0_ = NULL;
	gboolean _tmp28_ = FALSE;
	GXmlNode* _tmp29_ = NULL;
	gboolean _tmp30_ = FALSE;
	GError * _inner_error_ = NULL;
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (Configuration*) base;
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (node != NULL, FALSE);
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = node;
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp0_, GXML_TYPE_DOCUMENT)) {
#line 3228 "SerializableObjectModelTest.c"
		GXmlNode* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlNode* _tmp3_ = NULL;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp1_ = node;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp2_ = gxml_document_get_root (G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, GXML_TYPE_DOCUMENT, GXmlDocument));
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp3_ = _tmp2_;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (n);
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		n = G_TYPE_CHECK_INSTANCE_CAST (_tmp3_, GXML_TYPE_NODE, GXmlNode);
#line 3242 "SerializableObjectModelTest.c"
	} else {
		GXmlNode* _tmp4_ = NULL;
		GXmlNode* _tmp5_ = NULL;
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp4_ = node;
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = _g_object_ref0 (_tmp4_);
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (n);
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		n = _tmp5_;
#line 3254 "SerializableObjectModelTest.c"
	}
	{
		GeeList* _ns_list = NULL;
		GXmlNode* _tmp6_ = NULL;
		GeeList* _tmp7_ = NULL;
		GeeList* _tmp8_ = NULL;
		gint _ns_size = 0;
		GeeList* _tmp9_ = NULL;
		gint _tmp10_ = 0;
		gint _tmp11_ = 0;
		gint _ns_index = 0;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = n;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp7_ = gxml_node_get_namespaces (_tmp6_);
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp8_ = _tmp7_;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_ns_list = _tmp8_;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp9_ = _ns_list;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp10_ = gee_collection_get_size ((GeeCollection*) _tmp9_);
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp11_ = _tmp10_;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_ns_size = _tmp11_;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_ns_index = -1;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		while (TRUE) {
#line 3286 "SerializableObjectModelTest.c"
			gint _tmp12_ = 0;
			gint _tmp13_ = 0;
			gint _tmp14_ = 0;
			GXmlNamespace* ns = NULL;
			GeeList* _tmp15_ = NULL;
			gint _tmp16_ = 0;
			gpointer _tmp17_ = NULL;
			gboolean _tmp18_ = FALSE;
			GXmlNamespace* _tmp19_ = NULL;
			gchar* _tmp20_ = NULL;
			gchar* _tmp21_ = NULL;
			gchar* _tmp22_ = NULL;
			gboolean _tmp23_ = FALSE;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp12_ = _ns_index;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_ns_index = _tmp12_ + 1;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp13_ = _ns_index;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp14_ = _ns_size;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			if (!(_tmp13_ < _tmp14_)) {
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				break;
#line 3312 "SerializableObjectModelTest.c"
			}
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp15_ = _ns_list;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp16_ = _ns_index;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp17_ = gee_list_get (_tmp15_, _tmp16_);
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			ns = (GXmlNamespace*) _tmp17_;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp19_ = ns;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp20_ = gxml_namespace_get_prefix (_tmp19_);
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp21_ = _tmp20_;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp22_ = _tmp21_;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp23_ = g_strcmp0 (_tmp22_, "om") == 0;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp22_);
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			if (_tmp23_) {
#line 3336 "SerializableObjectModelTest.c"
				GXmlNamespace* _tmp24_ = NULL;
				gchar* _tmp25_ = NULL;
				gchar* _tmp26_ = NULL;
				gchar* _tmp27_ = NULL;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp24_ = ns;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp25_ = gxml_namespace_get_uri (_tmp24_);
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp26_ = _tmp25_;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp27_ = _tmp26_;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp18_ = g_strcmp0 (_tmp27_, "http://www.gnome.org/gxml/0.4") == 0;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_g_free0 (_tmp27_);
#line 3353 "SerializableObjectModelTest.c"
			} else {
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp18_ = FALSE;
#line 3357 "SerializableObjectModelTest.c"
			}
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			if (_tmp18_) {
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				configuration_set_invalid (self, FALSE);
#line 3363 "SerializableObjectModelTest.c"
			}
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (ns);
#line 3367 "SerializableObjectModelTest.c"
		}
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_ns_list);
#line 3371 "SerializableObjectModelTest.c"
	}
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp29_ = node;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp30_ = gxml_serializable_object_model_default_deserialize ((GXmlSerializableObjectModel*) self, _tmp29_, &_inner_error_);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp28_ = _tmp30_;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 3381 "SerializableObjectModelTest.c"
		gboolean _tmp31_ = FALSE;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_propagate_error (error, _inner_error_);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (n);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		return _tmp31_;
#line 3389 "SerializableObjectModelTest.c"
	}
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp28_;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (n);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 3397 "SerializableObjectModelTest.c"
}


Configuration* configuration_construct (GType object_type) {
	Configuration * self = NULL;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (Configuration*) object_model_construct (object_type);
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return self;
#line 3407 "SerializableObjectModelTest.c"
}


Configuration* configuration_new (void) {
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return configuration_construct (TYPE_CONFIGURATION);
#line 3414 "SerializableObjectModelTest.c"
}


gboolean configuration_get_invalid (Configuration* self) {
	gboolean result;
	gboolean _tmp0_ = FALSE;
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_invalid;
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 3429 "SerializableObjectModelTest.c"
}


void configuration_set_invalid (Configuration* self, gboolean value) {
	gboolean _tmp0_ = FALSE;
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_invalid = _tmp0_;
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "invalid");
#line 3443 "SerializableObjectModelTest.c"
}


const gchar* configuration_get_device (Configuration* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 235 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 235 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_device;
#line 235 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 235 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 3458 "SerializableObjectModelTest.c"
}


void configuration_set_device (Configuration* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 235 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 235 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 235 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 235 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (self->priv->_device);
#line 235 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_device = _tmp1_;
#line 235 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "device");
#line 3477 "SerializableObjectModelTest.c"
}


static void configuration_class_init (ConfigurationClass * klass) {
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	configuration_parent_class = g_type_class_peek_parent (klass);
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_type_class_add_private (klass, sizeof (ConfigurationPrivate));
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) configuration_real_node_name;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->property_use_nick = (gboolean (*)(GXmlSerializableObjectModel*)) configuration_real_property_use_nick;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->list_serializable_properties = (GParamSpec** (*)(GXmlSerializableObjectModel*, int*)) configuration_real_list_serializable_properties;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->serialize = (GXmlNode* (*)(GXmlSerializableObjectModel*, GXmlNode*, GError**)) configuration_real_serialize;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->deserialize = (gboolean (*)(GXmlSerializableObjectModel*, GXmlNode*, GError**)) configuration_real_deserialize;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_configuration_get_property;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_configuration_set_property;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->finalize = configuration_finalize;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CONFIGURATION_INVALID, g_param_spec_boolean ("invalid", "invalid", "invalid", TRUE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CONFIGURATION_DEVICE, g_param_spec_string ("device", "device", "device", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 3506 "SerializableObjectModelTest.c"
}


static void configuration_instance_init (Configuration * self) {
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv = CONFIGURATION_GET_PRIVATE (self);
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_invalid = TRUE;
#line 3515 "SerializableObjectModelTest.c"
}


static void configuration_finalize (GObject* obj) {
	Configuration * self;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_CONFIGURATION, Configuration);
#line 235 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (self->priv->_device);
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (configuration_parent_class)->finalize (obj);
#line 3527 "SerializableObjectModelTest.c"
}


GType configuration_get_type (void) {
	static volatile gsize configuration_type_id__volatile = 0;
	if (g_once_init_enter (&configuration_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (ConfigurationClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) configuration_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Configuration), 0, (GInstanceInitFunc) configuration_instance_init, NULL };
		GType configuration_type_id;
		configuration_type_id = g_type_register_static (TYPE_OBJECT_MODEL, "Configuration", &g_define_type_info, 0);
		g_once_init_leave (&configuration_type_id__volatile, configuration_type_id);
	}
	return configuration_type_id__volatile;
}


static void _vala_configuration_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	Configuration * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_CONFIGURATION, Configuration);
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	switch (property_id) {
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case CONFIGURATION_INVALID:
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_boolean (value, configuration_get_invalid (self));
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case CONFIGURATION_DEVICE:
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_string (value, configuration_get_device (self));
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 3560 "SerializableObjectModelTest.c"
		default:
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 3566 "SerializableObjectModelTest.c"
	}
}


static void _vala_configuration_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	Configuration * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_CONFIGURATION, Configuration);
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	switch (property_id) {
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case CONFIGURATION_INVALID:
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		configuration_set_invalid (self, g_value_get_boolean (value));
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case CONFIGURATION_DEVICE:
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		configuration_set_device (self, g_value_get_string (value));
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 3588 "SerializableObjectModelTest.c"
		default:
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 3594 "SerializableObjectModelTest.c"
	}
}


FakeSerializable* fake_serializable_construct (GType object_type) {
	FakeSerializable * self = NULL;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (FakeSerializable*) object_model_construct (object_type);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return self;
#line 3605 "SerializableObjectModelTest.c"
}


FakeSerializable* fake_serializable_new (void) {
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return fake_serializable_construct (TYPE_FAKE_SERIALIZABLE);
#line 3612 "SerializableObjectModelTest.c"
}


const gchar* fake_serializable_get_none (FakeSerializable* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_none;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 3627 "SerializableObjectModelTest.c"
}


void fake_serializable_set_none (FakeSerializable* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (self->priv->_none);
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_none = _tmp1_;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "none");
#line 3646 "SerializableObjectModelTest.c"
}


static void fake_serializable_class_init (FakeSerializableClass * klass) {
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	fake_serializable_parent_class = g_type_class_peek_parent (klass);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_type_class_add_private (klass, sizeof (FakeSerializablePrivate));
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_fake_serializable_get_property;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_fake_serializable_set_property;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->finalize = fake_serializable_finalize;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), FAKE_SERIALIZABLE_NONE, g_param_spec_string ("none", "none", "none", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 3663 "SerializableObjectModelTest.c"
}


static void fake_serializable_instance_init (FakeSerializable * self) {
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv = FAKE_SERIALIZABLE_GET_PRIVATE (self);
#line 3670 "SerializableObjectModelTest.c"
}


static void fake_serializable_finalize (GObject* obj) {
	FakeSerializable * self;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_FAKE_SERIALIZABLE, FakeSerializable);
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (self->priv->_none);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (fake_serializable_parent_class)->finalize (obj);
#line 3682 "SerializableObjectModelTest.c"
}


GType fake_serializable_get_type (void) {
	static volatile gsize fake_serializable_type_id__volatile = 0;
	if (g_once_init_enter (&fake_serializable_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (FakeSerializableClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) fake_serializable_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (FakeSerializable), 0, (GInstanceInitFunc) fake_serializable_instance_init, NULL };
		GType fake_serializable_type_id;
		fake_serializable_type_id = g_type_register_static (TYPE_OBJECT_MODEL, "FakeSerializable", &g_define_type_info, 0);
		g_once_init_leave (&fake_serializable_type_id__volatile, fake_serializable_type_id);
	}
	return fake_serializable_type_id__volatile;
}


static void _vala_fake_serializable_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	FakeSerializable * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_FAKE_SERIALIZABLE, FakeSerializable);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	switch (property_id) {
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case FAKE_SERIALIZABLE_NONE:
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_string (value, fake_serializable_get_none (self));
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 3709 "SerializableObjectModelTest.c"
		default:
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 3715 "SerializableObjectModelTest.c"
	}
}


static void _vala_fake_serializable_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	FakeSerializable * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_FAKE_SERIALIZABLE, FakeSerializable);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	switch (property_id) {
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case FAKE_SERIALIZABLE_NONE:
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		fake_serializable_set_none (self, g_value_get_string (value));
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 3731 "SerializableObjectModelTest.c"
		default:
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 3737 "SerializableObjectModelTest.c"
	}
}


static gboolean unknown_attribute_real_get_enable_unknown_serializable_property (GXmlSerializableObjectModel* base) {
	UnknownAttribute * self;
	gboolean result = FALSE;
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (UnknownAttribute*) base;
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = TRUE;
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 3751 "SerializableObjectModelTest.c"
}


UnknownAttribute* unknown_attribute_construct (GType object_type) {
	UnknownAttribute * self = NULL;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (UnknownAttribute*) object_model_construct (object_type);
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return self;
#line 3761 "SerializableObjectModelTest.c"
}


UnknownAttribute* unknown_attribute_new (void) {
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return unknown_attribute_construct (TYPE_UNKNOWN_ATTRIBUTE);
#line 3768 "SerializableObjectModelTest.c"
}


const gchar* unknown_attribute_get_name (UnknownAttribute* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_name;
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 3783 "SerializableObjectModelTest.c"
}


void unknown_attribute_set_name (UnknownAttribute* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (self->priv->_name);
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_name = _tmp1_;
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 3802 "SerializableObjectModelTest.c"
}


GeeArrayList* unknown_attribute_get_array (UnknownAttribute* self) {
	GeeArrayList* result;
	GeeArrayList* _tmp0_ = NULL;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_array;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 3817 "SerializableObjectModelTest.c"
}


void unknown_attribute_set_array (UnknownAttribute* self, GeeArrayList* value) {
	GeeArrayList* _tmp0_ = NULL;
	GeeArrayList* _tmp1_ = NULL;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (self->priv->_array);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_array = _tmp1_;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "array");
#line 3836 "SerializableObjectModelTest.c"
}


FakeSerializable* unknown_attribute_get_fake (UnknownAttribute* self) {
	FakeSerializable* result;
	FakeSerializable* _tmp0_ = NULL;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = self->priv->_fake;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 3851 "SerializableObjectModelTest.c"
}


void unknown_attribute_set_fake (UnknownAttribute* self, FakeSerializable* value) {
	FakeSerializable* _tmp0_ = NULL;
	FakeSerializable* _tmp1_ = NULL;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (self != NULL);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = value;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (self->priv->_fake);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv->_fake = _tmp1_;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_notify ((GObject *) self, "fake");
#line 3870 "SerializableObjectModelTest.c"
}


static void unknown_attribute_class_init (UnknownAttributeClass * klass) {
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	unknown_attribute_parent_class = g_type_class_peek_parent (klass);
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_type_class_add_private (klass, sizeof (UnknownAttributePrivate));
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->get_enable_unknown_serializable_property = (gboolean (*)(GXmlSerializableObjectModel*)) unknown_attribute_real_get_enable_unknown_serializable_property;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_unknown_attribute_get_property;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_unknown_attribute_set_property;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (klass)->finalize = unknown_attribute_finalize;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), UNKNOWN_ATTRIBUTE_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), UNKNOWN_ATTRIBUTE_ARRAY, g_param_spec_object ("array", "array", "array", GEE_TYPE_ARRAY_LIST, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), UNKNOWN_ATTRIBUTE_FAKE, g_param_spec_object ("fake", "fake", "fake", TYPE_FAKE_SERIALIZABLE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 3893 "SerializableObjectModelTest.c"
}


static void unknown_attribute_instance_init (UnknownAttribute * self) {
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self->priv = UNKNOWN_ATTRIBUTE_GET_PRIVATE (self);
#line 3900 "SerializableObjectModelTest.c"
}


static void unknown_attribute_finalize (GObject* obj) {
	UnknownAttribute * self;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_UNKNOWN_ATTRIBUTE, UnknownAttribute);
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (self->priv->_name);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (self->priv->_array);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (self->priv->_fake);
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	G_OBJECT_CLASS (unknown_attribute_parent_class)->finalize (obj);
#line 3916 "SerializableObjectModelTest.c"
}


GType unknown_attribute_get_type (void) {
	static volatile gsize unknown_attribute_type_id__volatile = 0;
	if (g_once_init_enter (&unknown_attribute_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (UnknownAttributeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) unknown_attribute_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (UnknownAttribute), 0, (GInstanceInitFunc) unknown_attribute_instance_init, NULL };
		GType unknown_attribute_type_id;
		unknown_attribute_type_id = g_type_register_static (TYPE_OBJECT_MODEL, "UnknownAttribute", &g_define_type_info, 0);
		g_once_init_leave (&unknown_attribute_type_id__volatile, unknown_attribute_type_id);
	}
	return unknown_attribute_type_id__volatile;
}


static void _vala_unknown_attribute_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	UnknownAttribute * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_UNKNOWN_ATTRIBUTE, UnknownAttribute);
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	switch (property_id) {
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case UNKNOWN_ATTRIBUTE_NAME:
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_string (value, unknown_attribute_get_name (self));
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case UNKNOWN_ATTRIBUTE_ARRAY:
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_object (value, unknown_attribute_get_array (self));
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case UNKNOWN_ATTRIBUTE_FAKE:
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_value_set_object (value, unknown_attribute_get_fake (self));
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 3955 "SerializableObjectModelTest.c"
		default:
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 3961 "SerializableObjectModelTest.c"
	}
}


static void _vala_unknown_attribute_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	UnknownAttribute * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_UNKNOWN_ATTRIBUTE, UnknownAttribute);
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	switch (property_id) {
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case UNKNOWN_ATTRIBUTE_NAME:
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		unknown_attribute_set_name (self, g_value_get_string (value));
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case UNKNOWN_ATTRIBUTE_ARRAY:
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		unknown_attribute_set_array (self, g_value_get_object (value));
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		case UNKNOWN_ATTRIBUTE_FAKE:
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		unknown_attribute_set_fake (self, g_value_get_object (value));
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 3989 "SerializableObjectModelTest.c"
		default:
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		break;
#line 3995 "SerializableObjectModelTest.c"
	}
}


static gboolean name_space_real_set_default_namespace (GXmlSerializableObjectModel* base, GXmlNode* node) {
	NameSpace * self;
	gboolean result = FALSE;
	GXmlNode* _tmp0_ = NULL;
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (NameSpace*) base;
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_val_if_fail (node != NULL, FALSE);
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_test_message ("Setting default namespace");
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = node;
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	gxml_node_set_namespace (_tmp0_, "http://www.gnome.org/GXml", "gxml");
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = TRUE;
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 4018 "SerializableObjectModelTest.c"
}


static gchar* name_space_real_to_string (GXmlSerializableObjectModel* base) {
	NameSpace * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (NameSpace*) base;
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = g_strdup ("");
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	result = _tmp0_;
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return result;
#line 4034 "SerializableObjectModelTest.c"
}


NameSpace* name_space_construct (GType object_type) {
	NameSpace * self = NULL;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (NameSpace*) gxml_serializable_object_model_construct (object_type);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return self;
#line 4044 "SerializableObjectModelTest.c"
}


NameSpace* name_space_new (void) {
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return name_space_construct (TYPE_NAME_SPACE);
#line 4051 "SerializableObjectModelTest.c"
}


static void name_space_class_init (NameSpaceClass * klass) {
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	name_space_parent_class = g_type_class_peek_parent (klass);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->set_default_namespace = (gboolean (*)(GXmlSerializableObjectModel*, GXmlNode*)) name_space_real_set_default_namespace;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) name_space_real_to_string;
#line 4062 "SerializableObjectModelTest.c"
}


static void name_space_instance_init (NameSpace * self) {
}


GType name_space_get_type (void) {
	static volatile gsize name_space_type_id__volatile = 0;
	if (g_once_init_enter (&name_space_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (NameSpaceClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) name_space_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (NameSpace), 0, (GInstanceInitFunc) name_space_instance_init, NULL };
		GType name_space_type_id;
		name_space_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "NameSpace", &g_define_type_info, 0);
		g_once_init_leave (&name_space_type_id__volatile, name_space_type_id);
	}
	return name_space_type_id__volatile;
}


static gdouble double_parse (const gchar* str) {
	gdouble result = 0.0;
	const gchar* _tmp0_ = NULL;
	gdouble _tmp1_ = 0.0;
#line 869 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	g_return_val_if_fail (str != NULL, 0.0);
#line 870 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	_tmp0_ = str;
#line 870 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	_tmp1_ = g_ascii_strtod (_tmp0_, NULL);
#line 870 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	result = _tmp1_;
#line 870 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	return result;
#line 4096 "SerializableObjectModelTest.c"
}


static void __lambda9_ (void) {
	GError * _inner_error_ = NULL;
	{
		Computer* computer = NULL;
		Computer* _tmp0_ = NULL;
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlNode* _tmp3_ = NULL;
		GXmlNode* _tmp4_ = NULL;
		GXmlNode* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		gchar* _tmp7_ = NULL;
		gchar* _tmp8_ = NULL;
		gchar* _tmp9_ = NULL;
		gchar* _tmp10_ = NULL;
		gchar* _tmp11_ = NULL;
		gboolean _tmp12_ = FALSE;
		GXmlNode* m = NULL;
		GXmlNode* _tmp14_ = NULL;
		GXmlNode* _tmp15_ = NULL;
		GXmlElement* _tmp16_ = NULL;
		GXmlElement* _tmp17_ = NULL;
		GXmlNode* _tmp18_ = NULL;
		GXmlNode* _tmp19_ = NULL;
		gchar* _tmp20_ = NULL;
		gchar* _tmp21_ = NULL;
		gchar* _tmp22_ = NULL;
		gboolean _tmp23_ = FALSE;
		GXmlNode* mod = NULL;
		GXmlNode* _tmp25_ = NULL;
		GXmlNode* _tmp26_ = NULL;
		GXmlElement* _tmp27_ = NULL;
		GXmlElement* _tmp28_ = NULL;
		GXmlNode* _tmp29_ = NULL;
		GXmlNode* _tmp30_ = NULL;
		gchar* _tmp31_ = NULL;
		gchar* _tmp32_ = NULL;
		gchar* _tmp33_ = NULL;
		gboolean _tmp34_ = FALSE;
		GXmlNode* c = NULL;
		GXmlNode* _tmp36_ = NULL;
		GXmlNode* _tmp37_ = NULL;
		GXmlElement* _tmp38_ = NULL;
		GXmlElement* _tmp39_ = NULL;
		GXmlNode* _tmp40_ = NULL;
		GXmlNode* _tmp41_ = NULL;
		gchar* _tmp42_ = NULL;
		gchar* _tmp43_ = NULL;
		gchar* _tmp44_ = NULL;
		gboolean _tmp45_ = FALSE;
		GXmlNode* g = NULL;
		GXmlNode* _tmp47_ = NULL;
		GXmlNode* _tmp48_ = NULL;
		GXmlElement* _tmp49_ = NULL;
		GXmlElement* _tmp50_ = NULL;
		GXmlNode* _tmp51_ = NULL;
		GXmlNode* _tmp52_ = NULL;
		gchar* _tmp53_ = NULL;
		gchar* _tmp54_ = NULL;
		gchar* _tmp55_ = NULL;
		gdouble _tmp56_ = 0.0;
		gboolean _tmp57_ = FALSE;
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp0_ = computer_new ();
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		computer = _tmp0_;
#line 316 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp1_ = gxml_gdocument_new ();
#line 316 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		doc = _tmp1_;
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp2_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) computer, (GXmlNode*) doc, &_inner_error_);
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp3_ = _tmp2_;
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp3_);
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (doc);
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (computer);
#line 4183 "SerializableObjectModelTest.c"
			goto __catch43_g_error;
		}
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp4_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = _tmp4_;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = _tmp5_;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp7_ = gxml_node_get_name (_tmp6_);
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp8_ = _tmp7_;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp9_ = _tmp8_;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp10_ = g_utf8_strdown (_tmp9_, (gssize) -1);
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp11_ = _tmp10_;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp12_ = g_strcmp0 (_tmp11_, "computer") != 0;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp11_);
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp9_);
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp6_);
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp12_) {
#line 4212 "SerializableObjectModelTest.c"
			FILE* _tmp13_ = NULL;
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp13_ = stdout;
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp13_, "ERROR XML_COMPUTER: computer\n");
#line 320 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 4220 "SerializableObjectModelTest.c"
		}
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp14_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp15_ = _tmp14_;
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp16_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp15_, GXML_TYPE_ELEMENT) ? ((GXmlElement*) _tmp15_) : NULL;
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp16_ == NULL) {
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (_tmp15_);
#line 4232 "SerializableObjectModelTest.c"
		}
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp17_ = _tmp16_;
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp18_ = gxml_element_get_attr (_tmp17_, "manufacturer");
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp19_ = _tmp18_;
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp17_);
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		m = _tmp19_;
#line 323 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (m == NULL) {
#line 323 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 4248 "SerializableObjectModelTest.c"
		}
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp20_ = gxml_node_get_value (m);
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp21_ = _tmp20_;
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp22_ = _tmp21_;
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp23_ = g_strcmp0 (_tmp22_, "MexicanLaptop, Inc.") != 0;
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp22_);
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp23_) {
#line 4262 "SerializableObjectModelTest.c"
			FILE* _tmp24_ = NULL;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp24_ = stdout;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp24_, "ERROR XML_COMPUTER: manufacturer\n");
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 4270 "SerializableObjectModelTest.c"
		}
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp25_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp26_ = _tmp25_;
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp27_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp26_, GXML_TYPE_ELEMENT) ? ((GXmlElement*) _tmp26_) : NULL;
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp27_ == NULL) {
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (_tmp26_);
#line 4282 "SerializableObjectModelTest.c"
		}
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp28_ = _tmp27_;
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp29_ = gxml_element_get_attr (_tmp28_, "model");
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp30_ = _tmp29_;
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp28_);
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		mod = _tmp30_;
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (mod == NULL) {
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 4298 "SerializableObjectModelTest.c"
		}
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp31_ = gxml_node_get_value (mod);
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp32_ = _tmp31_;
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp33_ = _tmp32_;
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp34_ = g_strcmp0 (_tmp33_, "LQ59678") != 0;
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp33_);
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp34_) {
#line 4312 "SerializableObjectModelTest.c"
			FILE* _tmp35_ = NULL;
#line 331 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp35_ = stdout;
#line 331 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp35_, "ERROR XML_COMPUTER: model\n");
#line 332 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 4320 "SerializableObjectModelTest.c"
		}
#line 334 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp36_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 334 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp37_ = _tmp36_;
#line 334 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp38_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp37_, GXML_TYPE_ELEMENT) ? ((GXmlElement*) _tmp37_) : NULL;
#line 334 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp38_ == NULL) {
#line 334 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (_tmp37_);
#line 4332 "SerializableObjectModelTest.c"
		}
#line 334 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp39_ = _tmp38_;
#line 334 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp40_ = gxml_element_get_attr (_tmp39_, "cores");
#line 334 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp41_ = _tmp40_;
#line 334 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp39_);
#line 334 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		c = _tmp41_;
#line 335 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (c == NULL) {
#line 335 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 4348 "SerializableObjectModelTest.c"
		}
#line 336 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp42_ = gxml_node_get_value (c);
#line 336 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp43_ = _tmp42_;
#line 336 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp44_ = _tmp43_;
#line 336 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp45_ = g_strcmp0 (_tmp44_, "8") != 0;
#line 336 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp44_);
#line 336 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp45_) {
#line 4362 "SerializableObjectModelTest.c"
			FILE* _tmp46_ = NULL;
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp46_ = stdout;
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp46_, "ERROR XML_COMPUTER: cores val\n");
#line 338 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 4370 "SerializableObjectModelTest.c"
		}
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp47_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp48_ = _tmp47_;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp49_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp48_, GXML_TYPE_ELEMENT) ? ((GXmlElement*) _tmp48_) : NULL;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp49_ == NULL) {
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (_tmp48_);
#line 4382 "SerializableObjectModelTest.c"
		}
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp50_ = _tmp49_;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp51_ = gxml_element_get_attr (_tmp50_, "ghz");
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp52_ = _tmp51_;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp50_);
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g = _tmp52_;
#line 341 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (g == NULL) {
#line 341 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 4398 "SerializableObjectModelTest.c"
		}
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp53_ = gxml_node_get_value (g);
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp54_ = _tmp53_;
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp55_ = _tmp54_;
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp56_ = double_parse (_tmp55_);
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp57_ = _tmp56_ != ((gdouble) 3.5);
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp55_);
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp57_) {
#line 4414 "SerializableObjectModelTest.c"
			FILE* _tmp58_ = NULL;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp58_ = stdout;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp58_, "ERROR XML_COMPUTER: ghz val\n");
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 4422 "SerializableObjectModelTest.c"
		}
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (g);
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (c);
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (mod);
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (m);
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (doc);
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (computer);
#line 4436 "SerializableObjectModelTest.c"
	}
	goto __finally43;
	__catch43_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp59_ = NULL;
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		e = _inner_error_;
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_inner_error_ = NULL;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp59_ = e->message;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_message ("SerializableObjectModelTest.vala:349: %s", _tmp59_);
#line 350 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_assert_not_reached ();
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_error_free0 (e);
#line 4455 "SerializableObjectModelTest.c"
	}
	__finally43:
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_clear_error (&_inner_error_);
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		return;
#line 4466 "SerializableObjectModelTest.c"
	}
}


static void ___lambda9__gtest_func (void) {
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	__lambda9_ ();
#line 4474 "SerializableObjectModelTest.c"
}


static void __lambda10_ (void) {
	Computer* computer = NULL;
	Computer* _tmp0_ = NULL;
	GError * _inner_error_ = NULL;
#line 356 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = computer_new ();
#line 356 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	computer = _tmp0_;
#line 4486 "SerializableObjectModelTest.c"
	{
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp1_ = NULL;
		Computer* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		Computer* _tmp4_ = NULL;
		const gchar* _tmp5_ = NULL;
		const gchar* _tmp6_ = NULL;
		Computer* _tmp14_ = NULL;
		const gchar* _tmp15_ = NULL;
		const gchar* _tmp16_ = NULL;
		Computer* _tmp24_ = NULL;
		gint _tmp25_ = 0;
		gint _tmp26_ = 0;
		Computer* _tmp35_ = NULL;
		gfloat _tmp36_ = 0.0F;
		gfloat _tmp37_ = 0.0F;
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp1_ = gxml_gdocument_new_from_string (XML_COMPUTER_FILE, 0, &_inner_error_);
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		doc = _tmp1_;
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 4510 "SerializableObjectModelTest.c"
			goto __catch44_g_error;
		}
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp2_ = computer;
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp3_ = doc;
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp2_, (GXmlNode*) _tmp3_, &_inner_error_);
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (doc);
#line 4523 "SerializableObjectModelTest.c"
			goto __catch44_g_error;
		}
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp4_ = computer;
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = computer_get_manufacturer (_tmp4_);
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = _tmp5_;
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (g_strcmp0 (_tmp6_, "ThecnicalGroup") != 0) {
#line 4534 "SerializableObjectModelTest.c"
			FILE* _tmp7_ = NULL;
			Computer* _tmp8_ = NULL;
			const gchar* _tmp9_ = NULL;
			const gchar* _tmp10_ = NULL;
			const gchar* _tmp11_ = NULL;
			gchar* _tmp12_ = NULL;
			gchar* _tmp13_ = NULL;
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp7_ = stdout;
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp8_ = computer;
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp9_ = computer_get_manufacturer (_tmp8_);
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp10_ = _tmp9_;
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp11_ = string_to_string (_tmp10_);
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp12_ = g_strconcat ("ERROR XML_COMPUTER: manufacturer val: ", _tmp11_, "\n", NULL);
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp13_ = _tmp12_;
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp7_, "%s", _tmp13_);
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp13_);
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 4562 "SerializableObjectModelTest.c"
		}
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp14_ = computer;
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp15_ = computer_get_model (_tmp14_);
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp16_ = _tmp15_;
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (g_strcmp0 (_tmp16_, "YH576G") != 0) {
#line 4572 "SerializableObjectModelTest.c"
			FILE* _tmp17_ = NULL;
			Computer* _tmp18_ = NULL;
			const gchar* _tmp19_ = NULL;
			const gchar* _tmp20_ = NULL;
			const gchar* _tmp21_ = NULL;
			gchar* _tmp22_ = NULL;
			gchar* _tmp23_ = NULL;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp17_ = stdout;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp18_ = computer;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp19_ = computer_get_model (_tmp18_);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp20_ = _tmp19_;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp21_ = string_to_string (_tmp20_);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp22_ = g_strconcat ("ERROR XML_COMPUTER: model val: ", _tmp21_, "\n", NULL);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp23_ = _tmp22_;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp17_, "%s", _tmp23_);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp23_);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 4600 "SerializableObjectModelTest.c"
		}
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp24_ = computer;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp25_ = computer_get_cores (_tmp24_);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp26_ = _tmp25_;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp26_ != 4) {
#line 4610 "SerializableObjectModelTest.c"
			FILE* _tmp27_ = NULL;
			Computer* _tmp28_ = NULL;
			gint _tmp29_ = 0;
			gint _tmp30_ = 0;
			gchar* _tmp31_ = NULL;
			gchar* _tmp32_ = NULL;
			gchar* _tmp33_ = NULL;
			gchar* _tmp34_ = NULL;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp27_ = stdout;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp28_ = computer;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp29_ = computer_get_cores (_tmp28_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp30_ = _tmp29_;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp31_ = g_strdup_printf ("%i", _tmp30_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp32_ = _tmp31_;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp33_ = g_strconcat ("ERROR XML_COMPUTER: cores val: ", _tmp32_, "\n", NULL);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp34_ = _tmp33_;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp27_, "%s", _tmp34_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp34_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp32_);
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 4643 "SerializableObjectModelTest.c"
		}
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp35_ = computer;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp36_ = computer_get_ghz (_tmp35_);
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp37_ = _tmp36_;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp37_ != ((gfloat) 2.8)) {
#line 4653 "SerializableObjectModelTest.c"
			FILE* _tmp38_ = NULL;
			Computer* _tmp39_ = NULL;
			gfloat _tmp40_ = 0.0F;
			gfloat _tmp41_ = 0.0F;
			gchar* _tmp42_ = NULL;
			gchar* _tmp43_ = NULL;
			gchar* _tmp44_ = NULL;
			gchar* _tmp45_ = NULL;
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp38_ = stdout;
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp39_ = computer;
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp40_ = computer_get_ghz (_tmp39_);
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp41_ = _tmp40_;
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp42_ = g_strdup_printf ("%g", _tmp41_);
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp43_ = _tmp42_;
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp44_ = g_strconcat ("ERROR XML_COMPUTER: ghz val: ", _tmp43_, "\n", NULL);
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp45_ = _tmp44_;
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp38_, "%s", _tmp45_);
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp45_);
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp43_);
#line 374 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 4686 "SerializableObjectModelTest.c"
		}
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (doc);
#line 4690 "SerializableObjectModelTest.c"
	}
	goto __finally44;
	__catch44_g_error:
	{
		GError* e = NULL;
		Computer* _tmp46_ = NULL;
		gchar* _tmp47_ = NULL;
		gchar* _tmp48_ = NULL;
		const gchar* _tmp49_ = NULL;
		const gchar* _tmp50_ = NULL;
		const gchar* _tmp51_ = NULL;
		gchar* _tmp52_ = NULL;
		gchar* _tmp53_ = NULL;
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		e = _inner_error_;
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_inner_error_ = NULL;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp46_ = computer;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp47_ = gxml_serializable_object_model_to_string ((GXmlSerializableObjectModel*) _tmp46_);
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp48_ = _tmp47_;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp49_ = string_to_string (_tmp48_);
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp50_ = e->message;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp51_ = string_to_string (_tmp50_);
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp52_ = g_strconcat ("GHz : ", _tmp49_, " ERROR: ", _tmp51_, NULL);
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp53_ = _tmp52_;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_message ("SerializableObjectModelTest.vala:378: %s", _tmp53_);
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp53_);
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp48_);
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_assert_not_reached ();
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_error_free0 (e);
#line 4734 "SerializableObjectModelTest.c"
	}
	__finally44:
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (computer);
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_clear_error (&_inner_error_);
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		return;
#line 4747 "SerializableObjectModelTest.c"
	}
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (computer);
#line 4751 "SerializableObjectModelTest.c"
}


static void ___lambda10__gtest_func (void) {
#line 354 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	__lambda10_ ();
#line 4758 "SerializableObjectModelTest.c"
}


static void __lambda11_ (void) {
	Manual* manual = NULL;
	Manual* _tmp0_ = NULL;
	GError * _inner_error_ = NULL;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = manual_new ();
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	manual = _tmp0_;
#line 4770 "SerializableObjectModelTest.c"
	{
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp1_ = NULL;
		Manual* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		Manual* _tmp4_ = NULL;
		const gchar* _tmp5_ = NULL;
		const gchar* _tmp6_ = NULL;
		Manual* _tmp14_ = NULL;
		gint _tmp15_ = 0;
		gint _tmp16_ = 0;
		Manual* _tmp25_ = NULL;
		gchar* _tmp26_ = NULL;
		gchar* _tmp27_ = NULL;
		gboolean _tmp28_ = FALSE;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp1_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n" \
"                       <manual document=\"Specification\" pages=\"3\">" \
"This is an Specification file</manual>", 0, &_inner_error_);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		doc = _tmp1_;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 4792 "SerializableObjectModelTest.c"
			goto __catch45_g_error;
		}
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp2_ = manual;
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp3_ = doc;
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp2_, (GXmlNode*) _tmp3_, &_inner_error_);
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (doc);
#line 4805 "SerializableObjectModelTest.c"
			goto __catch45_g_error;
		}
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp4_ = manual;
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = manual_get_document (_tmp4_);
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = _tmp5_;
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (g_strcmp0 (_tmp6_, "Specification") != 0) {
#line 4816 "SerializableObjectModelTest.c"
			FILE* _tmp7_ = NULL;
			Manual* _tmp8_ = NULL;
			const gchar* _tmp9_ = NULL;
			const gchar* _tmp10_ = NULL;
			const gchar* _tmp11_ = NULL;
			gchar* _tmp12_ = NULL;
			gchar* _tmp13_ = NULL;
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp7_ = stdout;
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp8_ = manual;
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp9_ = manual_get_document (_tmp8_);
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp10_ = _tmp9_;
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp11_ = string_to_string (_tmp10_);
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp12_ = g_strconcat ("ERROR MANUAL:  Bad document value. Expected 'Specification', got: ", _tmp11_, "\n", NULL);
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp13_ = _tmp12_;
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp7_, "%s", _tmp13_);
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp13_);
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 4844 "SerializableObjectModelTest.c"
		}
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp14_ = manual;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp15_ = manual_get_pages (_tmp14_);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp16_ = _tmp15_;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp16_ != 3) {
#line 4854 "SerializableObjectModelTest.c"
			FILE* _tmp17_ = NULL;
			Manual* _tmp18_ = NULL;
			gint _tmp19_ = 0;
			gint _tmp20_ = 0;
			gchar* _tmp21_ = NULL;
			gchar* _tmp22_ = NULL;
			gchar* _tmp23_ = NULL;
			gchar* _tmp24_ = NULL;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp17_ = stdout;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp18_ = manual;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp19_ = manual_get_pages (_tmp18_);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp20_ = _tmp19_;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp21_ = g_strdup_printf ("%i", _tmp20_);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp22_ = _tmp21_;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp23_ = g_strconcat ("ERROR MANUAL:  Bad pages value. Expected '3', got: ", _tmp22_, "\n", NULL);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp24_ = _tmp23_;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp17_, "%s", _tmp24_);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp24_);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp22_);
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 4887 "SerializableObjectModelTest.c"
		}
#line 398 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp25_ = manual;
#line 398 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp26_ = manual_get_contents (_tmp25_);
#line 398 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp27_ = _tmp26_;
#line 398 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp28_ = g_strcmp0 (_tmp27_, "This is an Specification file") != 0;
#line 398 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp27_);
#line 398 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp28_) {
#line 4901 "SerializableObjectModelTest.c"
			FILE* _tmp29_ = NULL;
			Manual* _tmp30_ = NULL;
			gchar* _tmp31_ = NULL;
			gchar* _tmp32_ = NULL;
			const gchar* _tmp33_ = NULL;
			gchar* _tmp34_ = NULL;
			gchar* _tmp35_ = NULL;
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp29_ = stdout;
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp30_ = manual;
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp31_ = manual_get_contents (_tmp30_);
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp32_ = _tmp31_;
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp33_ = string_to_string (_tmp32_);
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp34_ = g_strconcat ("ERROR MANUAL:  Bad GElement content value. Expected 'This is an Specif" \
"ication file', got: ", _tmp33_, "\n", NULL);
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp35_ = _tmp34_;
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp29_, "%s", _tmp35_);
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp35_);
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp32_);
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 4931 "SerializableObjectModelTest.c"
		}
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (doc);
#line 4935 "SerializableObjectModelTest.c"
	}
	goto __finally45;
	__catch45_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp36_ = NULL;
		const gchar* _tmp37_ = NULL;
		gchar* _tmp38_ = NULL;
		gchar* _tmp39_ = NULL;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		e = _inner_error_;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_inner_error_ = NULL;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp36_ = e->message;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp37_ = string_to_string (_tmp36_);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp38_ = g_strconcat ("Error: ", _tmp37_, NULL);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp39_ = _tmp38_;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_message ("SerializableObjectModelTest.vala:404: %s", _tmp39_);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp39_);
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_assert_not_reached ();
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_error_free0 (e);
#line 4965 "SerializableObjectModelTest.c"
	}
	__finally45:
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (manual);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_clear_error (&_inner_error_);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		return;
#line 4978 "SerializableObjectModelTest.c"
	}
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (manual);
#line 4982 "SerializableObjectModelTest.c"
}


static void ___lambda11__gtest_func (void) {
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	__lambda11_ ();
#line 4989 "SerializableObjectModelTest.c"
}


static void __lambda12_ (void) {
	GXmlGDocument* doc = NULL;
	GXmlGDocument* _tmp0_ = NULL;
	Manual* manual = NULL;
	Manual* _tmp1_ = NULL;
	GError * _inner_error_ = NULL;
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = gxml_gdocument_new ();
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	doc = _tmp0_;
#line 412 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = manual_new ();
#line 412 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	manual = _tmp1_;
#line 5007 "SerializableObjectModelTest.c"
	{
		GXmlNode* _tmp2_ = NULL;
		GXmlNode* _tmp3_ = NULL;
		GXmlNode* _tmp4_ = NULL;
		GXmlNode* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlNode* _tmp8_ = NULL;
		GXmlNode* _tmp9_ = NULL;
		gchar* _tmp10_ = NULL;
		gchar* _tmp11_ = NULL;
		gchar* _tmp12_ = NULL;
		GXmlGElement* element = NULL;
		GXmlNode* _tmp13_ = NULL;
		GXmlNode* _tmp14_ = NULL;
		GXmlGElement* _tmp15_ = NULL;
#line 414 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp2_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) manual, (GXmlNode*) doc, &_inner_error_);
#line 414 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp3_ = _tmp2_;
#line 414 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp3_);
#line 414 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 5032 "SerializableObjectModelTest.c"
			goto __catch46_g_error;
		}
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp4_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = _tmp4_;
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = _tmp5_;
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (_tmp6_ != NULL, "doc.root != null");
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp6_);
#line 416 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp7_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 416 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp8_ = _tmp7_;
#line 416 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp9_ = _tmp8_;
#line 416 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp10_ = gxml_node_get_name (_tmp9_);
#line 416 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp11_ = _tmp10_;
#line 416 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp12_ = _tmp11_;
#line 416 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (g_strcmp0 (_tmp12_, "manual") == 0, "doc.root.name == \"manual\"");
#line 416 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp12_);
#line 416 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp9_);
#line 417 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp13_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 417 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp14_ = _tmp13_;
#line 417 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp15_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp14_, GXML_TYPE_GELEMENT) ? ((GXmlGElement*) _tmp14_) : NULL;
#line 417 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp15_ == NULL) {
#line 417 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (_tmp14_);
#line 5073 "SerializableObjectModelTest.c"
		}
#line 417 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		element = _tmp15_;
#line 418 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		serializable_object_model_test_serialize_manual_check (element, manual);
#line 413 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (element);
#line 5081 "SerializableObjectModelTest.c"
	}
	goto __finally46;
	__catch46_g_error:
	{
		GError* e = NULL;
		FILE* _tmp16_ = NULL;
		const gchar* _tmp17_ = NULL;
		const gchar* _tmp18_ = NULL;
#line 413 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		e = _inner_error_;
#line 413 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_inner_error_ = NULL;
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp16_ = stdout;
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp17_ = e->message;
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp18_ = string_to_string (_tmp17_);
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		fprintf (_tmp16_, "%s", _tmp18_);
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_assert_not_reached ();
#line 413 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_error_free0 (e);
#line 5106 "SerializableObjectModelTest.c"
	}
	__finally46:
#line 413 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 413 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (manual);
#line 413 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (doc);
#line 413 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 413 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_clear_error (&_inner_error_);
#line 413 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		return;
#line 5121 "SerializableObjectModelTest.c"
	}
#line 410 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (manual);
#line 410 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (doc);
#line 5127 "SerializableObjectModelTest.c"
}


static void ___lambda12__gtest_func (void) {
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	__lambda12_ ();
#line 5134 "SerializableObjectModelTest.c"
}


static void __lambda13_ (void) {
	Package* package = NULL;
	Package* _tmp0_ = NULL;
	GError * _inner_error_ = NULL;
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = package_new ();
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	package = _tmp0_;
#line 5146 "SerializableObjectModelTest.c"
	{
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp1_ = NULL;
		Package* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		Package* _tmp4_ = NULL;
		const gchar* _tmp5_ = NULL;
		const gchar* _tmp6_ = NULL;
		Package* _tmp14_ = NULL;
		const gchar* _tmp15_ = NULL;
		const gchar* _tmp16_ = NULL;
		Package* _tmp24_ = NULL;
		Manual* _tmp25_ = NULL;
		Manual* _tmp26_ = NULL;
		const gchar* _tmp27_ = NULL;
		const gchar* _tmp28_ = NULL;
		Package* _tmp38_ = NULL;
		Manual* _tmp39_ = NULL;
		Manual* _tmp40_ = NULL;
		gint _tmp41_ = 0;
		gint _tmp42_ = 0;
		Package* _tmp53_ = NULL;
		Manual* _tmp54_ = NULL;
		Manual* _tmp55_ = NULL;
		gchar* _tmp56_ = NULL;
		gchar* _tmp57_ = NULL;
		gboolean _tmp58_ = FALSE;
		Package* _tmp68_ = NULL;
		Computer* _tmp69_ = NULL;
		Computer* _tmp70_ = NULL;
		const gchar* _tmp71_ = NULL;
		const gchar* _tmp72_ = NULL;
		Package* _tmp82_ = NULL;
		Computer* _tmp83_ = NULL;
		Computer* _tmp84_ = NULL;
		const gchar* _tmp85_ = NULL;
		const gchar* _tmp86_ = NULL;
		Package* _tmp96_ = NULL;
		Computer* _tmp97_ = NULL;
		Computer* _tmp98_ = NULL;
		gint _tmp99_ = 0;
		gint _tmp100_ = 0;
		Package* _tmp111_ = NULL;
		Computer* _tmp112_ = NULL;
		Computer* _tmp113_ = NULL;
		gfloat _tmp114_ = 0.0F;
		gfloat _tmp115_ = 0.0F;
#line 429 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp1_ = gxml_gdocument_new_from_string (XML_PACKAGE_FILE, 0, &_inner_error_);
#line 429 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		doc = _tmp1_;
#line 429 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 5200 "SerializableObjectModelTest.c"
			goto __catch47_g_error;
		}
#line 430 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp2_ = package;
#line 430 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp3_ = doc;
#line 430 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp2_, (GXmlNode*) _tmp3_, &_inner_error_);
#line 430 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 430 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (doc);
#line 5213 "SerializableObjectModelTest.c"
			goto __catch47_g_error;
		}
#line 431 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp4_ = package;
#line 431 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = package_get_source (_tmp4_);
#line 431 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = _tmp5_;
#line 431 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (g_strcmp0 (_tmp6_, "Mexico/Central") != 0) {
#line 5224 "SerializableObjectModelTest.c"
			FILE* _tmp7_ = NULL;
			Package* _tmp8_ = NULL;
			const gchar* _tmp9_ = NULL;
			const gchar* _tmp10_ = NULL;
			const gchar* _tmp11_ = NULL;
			gchar* _tmp12_ = NULL;
			gchar* _tmp13_ = NULL;
#line 432 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp7_ = stdout;
#line 432 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp8_ = package;
#line 432 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp9_ = package_get_source (_tmp8_);
#line 432 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp10_ = _tmp9_;
#line 432 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp11_ = string_to_string (_tmp10_);
#line 432 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp12_ = g_strconcat ("ERROR PACKAGE: source: ", _tmp11_, "\n", NULL);
#line 432 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp13_ = _tmp12_;
#line 432 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp7_, "%s", _tmp13_);
#line 432 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp13_);
#line 433 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 5252 "SerializableObjectModelTest.c"
		}
#line 435 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp14_ = package;
#line 435 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp15_ = package_get_destiny (_tmp14_);
#line 435 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp16_ = _tmp15_;
#line 435 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (g_strcmp0 (_tmp16_, "Japan") != 0) {
#line 5262 "SerializableObjectModelTest.c"
			FILE* _tmp17_ = NULL;
			Package* _tmp18_ = NULL;
			const gchar* _tmp19_ = NULL;
			const gchar* _tmp20_ = NULL;
			const gchar* _tmp21_ = NULL;
			gchar* _tmp22_ = NULL;
			gchar* _tmp23_ = NULL;
#line 436 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp17_ = stdout;
#line 436 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp18_ = package;
#line 436 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp19_ = package_get_destiny (_tmp18_);
#line 436 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp20_ = _tmp19_;
#line 436 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp21_ = string_to_string (_tmp20_);
#line 436 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp22_ = g_strconcat ("ERROR PACKAGE: destiny: ", _tmp21_, "\n", NULL);
#line 436 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp23_ = _tmp22_;
#line 436 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp17_, "%s", _tmp23_);
#line 436 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp23_);
#line 437 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 5290 "SerializableObjectModelTest.c"
		}
#line 443 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp24_ = package;
#line 443 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp25_ = package_get_manual (_tmp24_);
#line 443 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp26_ = _tmp25_;
#line 443 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp27_ = manual_get_document (_tmp26_);
#line 443 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp28_ = _tmp27_;
#line 443 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (g_strcmp0 (_tmp28_, "Specification") != 0) {
#line 5304 "SerializableObjectModelTest.c"
			FILE* _tmp29_ = NULL;
			Package* _tmp30_ = NULL;
			Manual* _tmp31_ = NULL;
			Manual* _tmp32_ = NULL;
			const gchar* _tmp33_ = NULL;
			const gchar* _tmp34_ = NULL;
			const gchar* _tmp35_ = NULL;
			gchar* _tmp36_ = NULL;
			gchar* _tmp37_ = NULL;
#line 444 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp29_ = stdout;
#line 444 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp30_ = package;
#line 444 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp31_ = package_get_manual (_tmp30_);
#line 444 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp32_ = _tmp31_;
#line 444 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp33_ = manual_get_document (_tmp32_);
#line 444 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp34_ = _tmp33_;
#line 444 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp35_ = string_to_string (_tmp34_);
#line 444 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp36_ = g_strconcat ("ERROR PACKAGE: manual document: ", _tmp35_, "\n", NULL);
#line 444 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp37_ = _tmp36_;
#line 444 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp29_, "%s", _tmp37_);
#line 444 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp37_);
#line 445 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 5338 "SerializableObjectModelTest.c"
		}
#line 447 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp38_ = package;
#line 447 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp39_ = package_get_manual (_tmp38_);
#line 447 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp40_ = _tmp39_;
#line 447 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp41_ = manual_get_pages (_tmp40_);
#line 447 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp42_ = _tmp41_;
#line 447 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp42_ != 3) {
#line 5352 "SerializableObjectModelTest.c"
			FILE* _tmp43_ = NULL;
			Package* _tmp44_ = NULL;
			Manual* _tmp45_ = NULL;
			Manual* _tmp46_ = NULL;
			gint _tmp47_ = 0;
			gint _tmp48_ = 0;
			gchar* _tmp49_ = NULL;
			gchar* _tmp50_ = NULL;
			gchar* _tmp51_ = NULL;
			gchar* _tmp52_ = NULL;
#line 448 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp43_ = stdout;
#line 448 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp44_ = package;
#line 448 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp45_ = package_get_manual (_tmp44_);
#line 448 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp46_ = _tmp45_;
#line 448 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp47_ = manual_get_pages (_tmp46_);
#line 448 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp48_ = _tmp47_;
#line 448 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp49_ = g_strdup_printf ("%i", _tmp48_);
#line 448 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp50_ = _tmp49_;
#line 448 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp51_ = g_strconcat ("ERROR PACKAGE: manual pages: ", _tmp50_, "\n", NULL);
#line 448 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp52_ = _tmp51_;
#line 448 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp43_, "%s", _tmp52_);
#line 448 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp52_);
#line 448 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp50_);
#line 449 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 5391 "SerializableObjectModelTest.c"
		}
#line 451 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp53_ = package;
#line 451 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp54_ = package_get_manual (_tmp53_);
#line 451 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp55_ = _tmp54_;
#line 451 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp56_ = manual_get_contents (_tmp55_);
#line 451 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp57_ = _tmp56_;
#line 451 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp58_ = g_strcmp0 (_tmp57_, "This is an Specification file") != 0;
#line 451 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp57_);
#line 451 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp58_) {
#line 5409 "SerializableObjectModelTest.c"
			FILE* _tmp59_ = NULL;
			Package* _tmp60_ = NULL;
			Manual* _tmp61_ = NULL;
			Manual* _tmp62_ = NULL;
			gchar* _tmp63_ = NULL;
			gchar* _tmp64_ = NULL;
			const gchar* _tmp65_ = NULL;
			gchar* _tmp66_ = NULL;
			gchar* _tmp67_ = NULL;
#line 452 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp59_ = stdout;
#line 452 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp60_ = package;
#line 452 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp61_ = package_get_manual (_tmp60_);
#line 452 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp62_ = _tmp61_;
#line 452 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp63_ = manual_get_contents (_tmp62_);
#line 452 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp64_ = _tmp63_;
#line 452 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp65_ = string_to_string (_tmp64_);
#line 452 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp66_ = g_strconcat ("ERROR PACKAGE: manual value: ", _tmp65_, "\n", NULL);
#line 452 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp67_ = _tmp66_;
#line 452 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp59_, "%s", _tmp67_);
#line 452 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp67_);
#line 452 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp64_);
#line 453 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 5445 "SerializableObjectModelTest.c"
		}
#line 455 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp68_ = package;
#line 455 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp69_ = package_get_computer (_tmp68_);
#line 455 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp70_ = _tmp69_;
#line 455 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp71_ = computer_get_manufacturer (_tmp70_);
#line 455 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp72_ = _tmp71_;
#line 455 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (g_strcmp0 (_tmp72_, "LanCorop") != 0) {
#line 5459 "SerializableObjectModelTest.c"
			FILE* _tmp73_ = NULL;
			Package* _tmp74_ = NULL;
			Computer* _tmp75_ = NULL;
			Computer* _tmp76_ = NULL;
			const gchar* _tmp77_ = NULL;
			const gchar* _tmp78_ = NULL;
			const gchar* _tmp79_ = NULL;
			gchar* _tmp80_ = NULL;
			gchar* _tmp81_ = NULL;
#line 456 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp73_ = stdout;
#line 456 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp74_ = package;
#line 456 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp75_ = package_get_computer (_tmp74_);
#line 456 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp76_ = _tmp75_;
#line 456 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp77_ = computer_get_manufacturer (_tmp76_);
#line 456 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp78_ = _tmp77_;
#line 456 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp79_ = string_to_string (_tmp78_);
#line 456 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp80_ = g_strconcat ("ERROR PACKAGE: computer manufacturer: ", _tmp79_, "\n", NULL);
#line 456 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp81_ = _tmp80_;
#line 456 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp73_, "%s", _tmp81_);
#line 456 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp81_);
#line 457 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 5493 "SerializableObjectModelTest.c"
		}
#line 459 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp82_ = package;
#line 459 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp83_ = package_get_computer (_tmp82_);
#line 459 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp84_ = _tmp83_;
#line 459 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp85_ = computer_get_model (_tmp84_);
#line 459 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp86_ = _tmp85_;
#line 459 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (g_strcmp0 (_tmp86_, "Lap39120") != 0) {
#line 5507 "SerializableObjectModelTest.c"
			FILE* _tmp87_ = NULL;
			Package* _tmp88_ = NULL;
			Computer* _tmp89_ = NULL;
			Computer* _tmp90_ = NULL;
			const gchar* _tmp91_ = NULL;
			const gchar* _tmp92_ = NULL;
			const gchar* _tmp93_ = NULL;
			gchar* _tmp94_ = NULL;
			gchar* _tmp95_ = NULL;
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp87_ = stdout;
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp88_ = package;
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp89_ = package_get_computer (_tmp88_);
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp90_ = _tmp89_;
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp91_ = computer_get_model (_tmp90_);
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp92_ = _tmp91_;
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp93_ = string_to_string (_tmp92_);
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp94_ = g_strconcat ("ERROR PACKAGE: computer model: ", _tmp93_, "\n", NULL);
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp95_ = _tmp94_;
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp87_, "%s", _tmp95_);
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp95_);
#line 461 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 5541 "SerializableObjectModelTest.c"
		}
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp96_ = package;
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp97_ = package_get_computer (_tmp96_);
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp98_ = _tmp97_;
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp99_ = computer_get_cores (_tmp98_);
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp100_ = _tmp99_;
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp100_ != 16) {
#line 5555 "SerializableObjectModelTest.c"
			FILE* _tmp101_ = NULL;
			Package* _tmp102_ = NULL;
			Computer* _tmp103_ = NULL;
			Computer* _tmp104_ = NULL;
			gint _tmp105_ = 0;
			gint _tmp106_ = 0;
			gchar* _tmp107_ = NULL;
			gchar* _tmp108_ = NULL;
			gchar* _tmp109_ = NULL;
			gchar* _tmp110_ = NULL;
#line 464 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp101_ = stdout;
#line 464 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp102_ = package;
#line 464 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp103_ = package_get_computer (_tmp102_);
#line 464 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp104_ = _tmp103_;
#line 464 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp105_ = computer_get_cores (_tmp104_);
#line 464 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp106_ = _tmp105_;
#line 464 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp107_ = g_strdup_printf ("%i", _tmp106_);
#line 464 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp108_ = _tmp107_;
#line 464 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp109_ = g_strconcat ("ERROR PACKAGE: computer cores: ", _tmp108_, "\n", NULL);
#line 464 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp110_ = _tmp109_;
#line 464 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp101_, "%s", _tmp110_);
#line 464 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp110_);
#line 464 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp108_);
#line 465 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 5594 "SerializableObjectModelTest.c"
		}
#line 467 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp111_ = package;
#line 467 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp112_ = package_get_computer (_tmp111_);
#line 467 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp113_ = _tmp112_;
#line 467 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp114_ = computer_get_ghz (_tmp113_);
#line 467 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp115_ = _tmp114_;
#line 467 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp115_ != ((gfloat) 3.5)) {
#line 5608 "SerializableObjectModelTest.c"
			FILE* _tmp116_ = NULL;
			Package* _tmp117_ = NULL;
			Computer* _tmp118_ = NULL;
			Computer* _tmp119_ = NULL;
			gfloat _tmp120_ = 0.0F;
			gfloat _tmp121_ = 0.0F;
			gchar* _tmp122_ = NULL;
			gchar* _tmp123_ = NULL;
			gchar* _tmp124_ = NULL;
			gchar* _tmp125_ = NULL;
#line 468 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp116_ = stdout;
#line 468 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp117_ = package;
#line 468 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp118_ = package_get_computer (_tmp117_);
#line 468 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp119_ = _tmp118_;
#line 468 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp120_ = computer_get_ghz (_tmp119_);
#line 468 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp121_ = _tmp120_;
#line 468 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp122_ = g_strdup_printf ("%g", _tmp121_);
#line 468 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp123_ = _tmp122_;
#line 468 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp124_ = g_strconcat ("ERROR PACKAGE: computer ghz ", _tmp123_, "\n", NULL);
#line 468 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp125_ = _tmp124_;
#line 468 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp116_, "%s", _tmp125_);
#line 468 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp125_);
#line 468 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp123_);
#line 469 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 5647 "SerializableObjectModelTest.c"
		}
#line 428 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (doc);
#line 5651 "SerializableObjectModelTest.c"
	}
	goto __finally47;
	__catch47_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp126_ = NULL;
		const gchar* _tmp127_ = NULL;
		gchar* _tmp128_ = NULL;
		gchar* _tmp129_ = NULL;
#line 428 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		e = _inner_error_;
#line 428 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_inner_error_ = NULL;
#line 473 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp126_ = e->message;
#line 473 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp127_ = string_to_string (_tmp126_);
#line 473 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp128_ = g_strconcat ("Error: ", _tmp127_, NULL);
#line 473 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp129_ = _tmp128_;
#line 473 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_message ("SerializableObjectModelTest.vala:473: %s", _tmp129_);
#line 473 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp129_);
#line 474 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_assert_not_reached ();
#line 428 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_error_free0 (e);
#line 5681 "SerializableObjectModelTest.c"
	}
	__finally47:
#line 428 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 428 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (package);
#line 428 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 428 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_clear_error (&_inner_error_);
#line 428 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		return;
#line 5694 "SerializableObjectModelTest.c"
	}
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (package);
#line 5698 "SerializableObjectModelTest.c"
}


static void ___lambda13__gtest_func (void) {
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	__lambda13_ ();
#line 5705 "SerializableObjectModelTest.c"
}


static void __lambda16_ (void) {
	GXmlGDocument* doc = NULL;
	GXmlGDocument* _tmp0_ = NULL;
	Package* package = NULL;
	Package* _tmp1_ = NULL;
	GError * _inner_error_ = NULL;
#line 480 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = gxml_gdocument_new ();
#line 480 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	doc = _tmp0_;
#line 481 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = package_new ();
#line 481 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	package = _tmp1_;
#line 5723 "SerializableObjectModelTest.c"
	{
		GXmlNode* _tmp2_ = NULL;
		GXmlNode* _tmp3_ = NULL;
		GXmlNode* _tmp4_ = NULL;
		GXmlNode* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		gchar* _tmp7_ = NULL;
		gchar* _tmp8_ = NULL;
		gchar* _tmp9_ = NULL;
		gboolean _tmp10_ = FALSE;
		GXmlElement* element = NULL;
		GXmlNode* _tmp21_ = NULL;
		GXmlNode* _tmp22_ = NULL;
		GXmlElement* _tmp23_ = NULL;
		GXmlNode* source = NULL;
		GXmlNode* _tmp24_ = NULL;
		gchar* _tmp25_ = NULL;
		gchar* _tmp26_ = NULL;
		gchar* _tmp27_ = NULL;
		gboolean _tmp28_ = FALSE;
		GXmlNode* destiny = NULL;
		GXmlNode* _tmp36_ = NULL;
		gchar* _tmp37_ = NULL;
		gchar* _tmp38_ = NULL;
		gchar* _tmp39_ = NULL;
		gboolean _tmp40_ = FALSE;
#line 483 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp2_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) package, (GXmlNode*) doc, &_inner_error_);
#line 483 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp3_ = _tmp2_;
#line 483 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp3_);
#line 483 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 5758 "SerializableObjectModelTest.c"
			goto __catch48_g_error;
		}
#line 484 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp4_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 484 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = _tmp4_;
#line 484 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = _tmp5_;
#line 484 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp7_ = gxml_node_get_name (_tmp6_);
#line 484 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp8_ = _tmp7_;
#line 484 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp9_ = _tmp8_;
#line 484 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp10_ = g_strcmp0 (_tmp9_, "package") != 0;
#line 484 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp9_);
#line 484 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp6_);
#line 484 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp10_) {
#line 5781 "SerializableObjectModelTest.c"
			FILE* _tmp11_ = NULL;
			GXmlNode* _tmp12_ = NULL;
			GXmlNode* _tmp13_ = NULL;
			GXmlNode* _tmp14_ = NULL;
			gchar* _tmp15_ = NULL;
			gchar* _tmp16_ = NULL;
			gchar* _tmp17_ = NULL;
			const gchar* _tmp18_ = NULL;
			gchar* _tmp19_ = NULL;
			gchar* _tmp20_ = NULL;
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp11_ = stdout;
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp12_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp13_ = _tmp12_;
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp14_ = _tmp13_;
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp15_ = gxml_node_get_name (_tmp14_);
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp16_ = _tmp15_;
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp17_ = _tmp16_;
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp18_ = string_to_string (_tmp17_);
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp19_ = g_strconcat ("ERROR MANUAL:  GElement: ", _tmp18_, "\n", NULL);
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp20_ = _tmp19_;
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp11_, "%s", _tmp20_);
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp20_);
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp17_);
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (_tmp14_);
#line 486 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 5822 "SerializableObjectModelTest.c"
		}
#line 488 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp21_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 488 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp22_ = _tmp21_;
#line 488 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp23_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp22_, GXML_TYPE_ELEMENT) ? ((GXmlElement*) _tmp22_) : NULL;
#line 488 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp23_ == NULL) {
#line 488 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (_tmp22_);
#line 5834 "SerializableObjectModelTest.c"
		}
#line 488 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		element = _tmp23_;
#line 489 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp24_ = gxml_element_get_attr (element, "source");
#line 489 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		source = _tmp24_;
#line 490 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (source == NULL) {
#line 490 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 5846 "SerializableObjectModelTest.c"
		}
#line 491 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp25_ = gxml_node_get_value (source);
#line 491 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp26_ = _tmp25_;
#line 491 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp27_ = _tmp26_;
#line 491 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp28_ = g_strcmp0 (_tmp27_, "Mexico") != 0;
#line 491 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp27_);
#line 491 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp28_) {
#line 5860 "SerializableObjectModelTest.c"
			FILE* _tmp29_ = NULL;
			gchar* _tmp30_ = NULL;
			gchar* _tmp31_ = NULL;
			gchar* _tmp32_ = NULL;
			const gchar* _tmp33_ = NULL;
			gchar* _tmp34_ = NULL;
			gchar* _tmp35_ = NULL;
#line 492 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp29_ = stdout;
#line 492 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp30_ = gxml_node_get_value (source);
#line 492 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp31_ = _tmp30_;
#line 492 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp32_ = _tmp31_;
#line 492 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp33_ = string_to_string (_tmp32_);
#line 492 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp34_ = g_strconcat ("ERROR PACKAGE: source: ", _tmp33_, "\n", NULL);
#line 492 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp35_ = _tmp34_;
#line 492 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp29_, "%s", _tmp35_);
#line 492 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp35_);
#line 492 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp32_);
#line 493 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 5890 "SerializableObjectModelTest.c"
		}
#line 495 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp36_ = gxml_element_get_attr (element, "destiny");
#line 495 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		destiny = _tmp36_;
#line 496 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (destiny == NULL) {
#line 496 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 5900 "SerializableObjectModelTest.c"
		}
#line 497 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp37_ = gxml_node_get_value (destiny);
#line 497 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp38_ = _tmp37_;
#line 497 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp39_ = _tmp38_;
#line 497 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp40_ = g_strcmp0 (_tmp39_, "World") != 0;
#line 497 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp39_);
#line 497 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp40_) {
#line 5914 "SerializableObjectModelTest.c"
			FILE* _tmp41_ = NULL;
			gchar* _tmp42_ = NULL;
			gchar* _tmp43_ = NULL;
			gchar* _tmp44_ = NULL;
			const gchar* _tmp45_ = NULL;
			gchar* _tmp46_ = NULL;
			gchar* _tmp47_ = NULL;
#line 498 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp41_ = stdout;
#line 498 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp42_ = gxml_node_get_value (destiny);
#line 498 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp43_ = _tmp42_;
#line 498 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp44_ = _tmp43_;
#line 498 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp45_ = string_to_string (_tmp44_);
#line 498 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp46_ = g_strconcat ("ERROR PACKAGE: source: ", _tmp45_, "\n", NULL);
#line 498 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp47_ = _tmp46_;
#line 498 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp41_, "%s", _tmp47_);
#line 498 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp47_);
#line 498 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp44_);
#line 499 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 5944 "SerializableObjectModelTest.c"
		}
#line 482 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (destiny);
#line 482 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (source);
#line 482 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (element);
#line 5952 "SerializableObjectModelTest.c"
	}
	goto __finally48;
	__catch48_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp48_ = NULL;
		const gchar* _tmp49_ = NULL;
		gchar* _tmp50_ = NULL;
		gchar* _tmp51_ = NULL;
#line 482 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		e = _inner_error_;
#line 482 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_inner_error_ = NULL;
#line 503 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp48_ = e->message;
#line 503 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp49_ = string_to_string (_tmp48_);
#line 503 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp50_ = g_strconcat ("Error: ", _tmp49_, NULL);
#line 503 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp51_ = _tmp50_;
#line 503 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_message ("SerializableObjectModelTest.vala:503: %s", _tmp51_);
#line 503 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp51_);
#line 504 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_assert_not_reached ();
#line 482 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_error_free0 (e);
#line 5982 "SerializableObjectModelTest.c"
	}
	__finally48:
#line 482 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 482 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (package);
#line 482 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (doc);
#line 482 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 482 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_clear_error (&_inner_error_);
#line 482 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		return;
#line 5997 "SerializableObjectModelTest.c"
	}
#line 479 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (package);
#line 479 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (doc);
#line 6003 "SerializableObjectModelTest.c"
}


static void ___lambda16__gtest_func (void) {
#line 478 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	__lambda16_ ();
#line 6010 "SerializableObjectModelTest.c"
}


static void __lambda17_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		Package* package = NULL;
		Package* _tmp1_ = NULL;
		Package* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		Package* _tmp4_ = NULL;
		GArray* _tmp5_ = NULL;
		GArray* _tmp6_ = NULL;
		guint _tmp7_ = 0U;
		Package* _tmp17_ = NULL;
		GArray* _tmp18_ = NULL;
		GArray* _tmp19_ = NULL;
		const gchar* _tmp20_ = NULL;
		Package* _tmp29_ = NULL;
		GArray* _tmp30_ = NULL;
		GArray* _tmp31_ = NULL;
		const gchar* _tmp32_ = NULL;
		Package* _tmp41_ = NULL;
		GArray* _tmp42_ = NULL;
		GArray* _tmp43_ = NULL;
		const gchar* _tmp44_ = NULL;
#line 511 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp0_ = gxml_gdocument_new_from_string (XML_PACKAGE_FILE, 0, &_inner_error_);
#line 511 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		doc = _tmp0_;
#line 511 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 6045 "SerializableObjectModelTest.c"
			goto __catch49_g_error;
		}
#line 512 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp1_ = package_new ();
#line 512 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		package = _tmp1_;
#line 513 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp2_ = package;
#line 513 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp3_ = doc;
#line 513 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp2_, (GXmlNode*) _tmp3_, &_inner_error_);
#line 513 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 513 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (package);
#line 513 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (doc);
#line 6064 "SerializableObjectModelTest.c"
			goto __catch49_g_error;
		}
#line 514 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp4_ = package;
#line 514 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = package_get_tags (_tmp4_);
#line 514 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = _tmp5_;
#line 514 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp7_ = _tmp6_->len;
#line 514 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp7_ != ((guint) 3)) {
#line 6077 "SerializableObjectModelTest.c"
			FILE* _tmp8_ = NULL;
			Package* _tmp9_ = NULL;
			GArray* _tmp10_ = NULL;
			GArray* _tmp11_ = NULL;
			guint _tmp12_ = 0U;
			gchar* _tmp13_ = NULL;
			gchar* _tmp14_ = NULL;
			gchar* _tmp15_ = NULL;
			gchar* _tmp16_ = NULL;
#line 515 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp8_ = stdout;
#line 515 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp9_ = package;
#line 515 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp10_ = package_get_tags (_tmp9_);
#line 515 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp11_ = _tmp10_;
#line 515 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp12_ = _tmp11_->len;
#line 515 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp13_ = g_strdup_printf ("%u", _tmp12_);
#line 515 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp14_ = _tmp13_;
#line 515 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp15_ = g_strconcat ("ERROR PACKAGE: tags length: ", _tmp14_, NULL);
#line 515 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp16_ = _tmp15_;
#line 515 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp8_, "%s", _tmp16_);
#line 515 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp16_);
#line 515 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp14_);
#line 516 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 6113 "SerializableObjectModelTest.c"
		}
#line 518 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp17_ = package;
#line 518 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp18_ = package_get_tags (_tmp17_);
#line 518 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp19_ = _tmp18_;
#line 518 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp20_ = g_array_index (_tmp19_, gchar*, (guint) 0);
#line 518 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (g_strcmp0 (_tmp20_, "Printer") != 0) {
#line 6125 "SerializableObjectModelTest.c"
			FILE* _tmp21_ = NULL;
			Package* _tmp22_ = NULL;
			GArray* _tmp23_ = NULL;
			GArray* _tmp24_ = NULL;
			const gchar* _tmp25_ = NULL;
			const gchar* _tmp26_ = NULL;
			gchar* _tmp27_ = NULL;
			gchar* _tmp28_ = NULL;
#line 519 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp21_ = stdout;
#line 519 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp22_ = package;
#line 519 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp23_ = package_get_tags (_tmp22_);
#line 519 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp24_ = _tmp23_;
#line 519 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp25_ = g_array_index (_tmp24_, gchar*, (guint) 0);
#line 519 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp26_ = string_to_string (_tmp25_);
#line 519 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp27_ = g_strconcat ("ERROR PACKAGE: tags index 0: ", _tmp26_, NULL);
#line 519 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp28_ = _tmp27_;
#line 519 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp21_, "%s", _tmp28_);
#line 519 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp28_);
#line 520 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 6156 "SerializableObjectModelTest.c"
		}
#line 522 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp29_ = package;
#line 522 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp30_ = package_get_tags (_tmp29_);
#line 522 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp31_ = _tmp30_;
#line 522 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp32_ = g_array_index (_tmp31_, gchar*, (guint) 1);
#line 522 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (g_strcmp0 (_tmp32_, "Partner") != 0) {
#line 6168 "SerializableObjectModelTest.c"
			FILE* _tmp33_ = NULL;
			Package* _tmp34_ = NULL;
			GArray* _tmp35_ = NULL;
			GArray* _tmp36_ = NULL;
			const gchar* _tmp37_ = NULL;
			const gchar* _tmp38_ = NULL;
			gchar* _tmp39_ = NULL;
			gchar* _tmp40_ = NULL;
#line 523 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp33_ = stdout;
#line 523 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp34_ = package;
#line 523 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp35_ = package_get_tags (_tmp34_);
#line 523 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp36_ = _tmp35_;
#line 523 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp37_ = g_array_index (_tmp36_, gchar*, (guint) 1);
#line 523 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp38_ = string_to_string (_tmp37_);
#line 523 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp39_ = g_strconcat ("ERROR PACKAGE: tags index 1: ", _tmp38_, NULL);
#line 523 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp40_ = _tmp39_;
#line 523 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp33_, "%s", _tmp40_);
#line 523 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp40_);
#line 524 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 6199 "SerializableObjectModelTest.c"
		}
#line 526 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp41_ = package;
#line 526 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp42_ = package_get_tags (_tmp41_);
#line 526 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp43_ = _tmp42_;
#line 526 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp44_ = g_array_index (_tmp43_, gchar*, (guint) 2);
#line 526 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (g_strcmp0 (_tmp44_, "Support") != 0) {
#line 6211 "SerializableObjectModelTest.c"
			FILE* _tmp45_ = NULL;
			Package* _tmp46_ = NULL;
			GArray* _tmp47_ = NULL;
			GArray* _tmp48_ = NULL;
			const gchar* _tmp49_ = NULL;
			const gchar* _tmp50_ = NULL;
			gchar* _tmp51_ = NULL;
			gchar* _tmp52_ = NULL;
#line 527 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp45_ = stdout;
#line 527 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp46_ = package;
#line 527 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp47_ = package_get_tags (_tmp46_);
#line 527 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp48_ = _tmp47_;
#line 527 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp49_ = g_array_index (_tmp48_, gchar*, (guint) 2);
#line 527 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp50_ = string_to_string (_tmp49_);
#line 527 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp51_ = g_strconcat ("ERROR PACKAGE: tags index 0: ", _tmp50_, NULL);
#line 527 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp52_ = _tmp51_;
#line 527 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp45_, "%s", _tmp52_);
#line 527 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp52_);
#line 528 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 6242 "SerializableObjectModelTest.c"
		}
#line 510 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (package);
#line 510 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (doc);
#line 6248 "SerializableObjectModelTest.c"
	}
	goto __finally49;
	__catch49_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp53_ = NULL;
		const gchar* _tmp54_ = NULL;
		gchar* _tmp55_ = NULL;
		gchar* _tmp56_ = NULL;
#line 510 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		e = _inner_error_;
#line 510 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_inner_error_ = NULL;
#line 532 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp53_ = e->message;
#line 532 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp54_ = string_to_string (_tmp53_);
#line 532 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp55_ = g_strconcat ("Error: ", _tmp54_, NULL);
#line 532 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp56_ = _tmp55_;
#line 532 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_message ("SerializableObjectModelTest.vala:532: %s", _tmp56_);
#line 532 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp56_);
#line 533 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_assert_not_reached ();
#line 510 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_error_free0 (e);
#line 6278 "SerializableObjectModelTest.c"
	}
	__finally49:
#line 510 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 510 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 510 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_clear_error (&_inner_error_);
#line 510 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		return;
#line 6289 "SerializableObjectModelTest.c"
	}
}


static void ___lambda17__gtest_func (void) {
#line 508 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	__lambda17_ ();
#line 6297 "SerializableObjectModelTest.c"
}


static void __lambda18_ (void) {
	GXmlGDocument* doc = NULL;
	GXmlGDocument* _tmp0_ = NULL;
	Package* package = NULL;
	Package* _tmp1_ = NULL;
	GArray* _tmp2_ = NULL;
	GArray* _tmp3_ = NULL;
	gchar* _tmp4_ = NULL;
	GArray* _tmp5_ = NULL;
	GArray* _tmp6_ = NULL;
	gchar* _tmp7_ = NULL;
	GArray* _tmp8_ = NULL;
	GArray* _tmp9_ = NULL;
	gchar* _tmp10_ = NULL;
	GError * _inner_error_ = NULL;
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = gxml_gdocument_new ();
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	doc = _tmp0_;
#line 539 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = package_new ();
#line 539 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	package = _tmp1_;
#line 540 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp2_ = package_get_tags (package);
#line 540 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp3_ = _tmp2_;
#line 540 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp4_ = g_strdup ("Computer");
#line 540 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_array_append_val (_tmp3_, _tmp4_);
#line 541 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp5_ = package_get_tags (package);
#line 541 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp6_ = _tmp5_;
#line 541 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp7_ = g_strdup ("Customer");
#line 541 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_array_append_val (_tmp6_, _tmp7_);
#line 542 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp8_ = package_get_tags (package);
#line 542 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp9_ = _tmp8_;
#line 542 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp10_ = g_strdup ("Sale");
#line 542 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_array_append_val (_tmp9_, _tmp10_);
#line 6348 "SerializableObjectModelTest.c"
	{
		GXmlNode* _tmp11_ = NULL;
		GXmlNode* _tmp12_ = NULL;
		GXmlNode* _tmp13_ = NULL;
		GXmlNode* _tmp14_ = NULL;
		GXmlNode* _tmp15_ = NULL;
		gchar* _tmp16_ = NULL;
		gchar* _tmp17_ = NULL;
		gchar* _tmp18_ = NULL;
		gboolean _tmp19_ = FALSE;
		GXmlElement* element = NULL;
		GXmlNode* _tmp20_ = NULL;
		GXmlNode* _tmp21_ = NULL;
		GXmlElement* _tmp22_ = NULL;
		gboolean com = FALSE;
		gboolean cus = FALSE;
		gboolean sal = FALSE;
		gboolean _tmp55_ = FALSE;
		gboolean _tmp57_ = FALSE;
		gboolean _tmp59_ = FALSE;
#line 544 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp11_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) package, (GXmlNode*) doc, &_inner_error_);
#line 544 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp12_ = _tmp11_;
#line 544 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp12_);
#line 544 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 6377 "SerializableObjectModelTest.c"
			goto __catch50_g_error;
		}
#line 546 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp13_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 546 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp14_ = _tmp13_;
#line 546 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp15_ = _tmp14_;
#line 546 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp16_ = gxml_node_get_name (_tmp15_);
#line 546 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp17_ = _tmp16_;
#line 546 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp18_ = _tmp17_;
#line 546 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp19_ = g_strcmp0 (_tmp18_, "package") != 0;
#line 546 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp18_);
#line 546 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp15_);
#line 546 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp19_) {
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 6402 "SerializableObjectModelTest.c"
		}
#line 548 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp20_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 548 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp21_ = _tmp20_;
#line 548 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp22_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp21_, GXML_TYPE_ELEMENT) ? ((GXmlElement*) _tmp21_) : NULL;
#line 548 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp22_ == NULL) {
#line 548 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (_tmp21_);
#line 6414 "SerializableObjectModelTest.c"
		}
#line 548 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		element = _tmp22_;
#line 549 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		com = FALSE;
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		cus = FALSE;
#line 551 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		sal = FALSE;
#line 6424 "SerializableObjectModelTest.c"
		{
			GeeBidirList* _n_list = NULL;
			GXmlElement* _tmp23_ = NULL;
			GeeBidirList* _tmp24_ = NULL;
			GeeBidirList* _tmp25_ = NULL;
			gint _n_size = 0;
			GeeBidirList* _tmp26_ = NULL;
			gint _tmp27_ = 0;
			gint _tmp28_ = 0;
			gint _n_index = 0;
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp23_ = element;
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp24_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp23_);
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp25_ = _tmp24_;
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_n_list = _tmp25_;
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp26_ = _n_list;
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp27_ = gee_collection_get_size ((GeeCollection*) _tmp26_);
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp28_ = _tmp27_;
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_n_size = _tmp28_;
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_n_index = -1;
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			while (TRUE) {
#line 6455 "SerializableObjectModelTest.c"
				gint _tmp29_ = 0;
				gint _tmp30_ = 0;
				gint _tmp31_ = 0;
				GXmlNode* n = NULL;
				GeeBidirList* _tmp32_ = NULL;
				gint _tmp33_ = 0;
				gpointer _tmp34_ = NULL;
				GXmlNode* _tmp35_ = NULL;
				gchar* _tmp36_ = NULL;
				gchar* _tmp37_ = NULL;
				gchar* _tmp38_ = NULL;
				gboolean _tmp39_ = FALSE;
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp29_ = _n_index;
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_n_index = _tmp29_ + 1;
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp30_ = _n_index;
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp31_ = _n_size;
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				if (!(_tmp30_ < _tmp31_)) {
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					break;
#line 6480 "SerializableObjectModelTest.c"
				}
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp32_ = _n_list;
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp33_ = _n_index;
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp34_ = gee_list_get ((GeeList*) _tmp32_, _tmp33_);
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				n = (GXmlNode*) _tmp34_;
#line 554 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp35_ = n;
#line 554 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp36_ = gxml_node_get_name (_tmp35_);
#line 554 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp37_ = _tmp36_;
#line 554 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp38_ = _tmp37_;
#line 554 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp39_ = g_strcmp0 (_tmp38_, "tag") == 0;
#line 554 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_g_free0 (_tmp38_);
#line 554 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				if (_tmp39_) {
#line 6504 "SerializableObjectModelTest.c"
					GXmlNode* _tmp40_ = NULL;
					gchar* _tmp41_ = NULL;
					gchar* _tmp42_ = NULL;
					gchar* _tmp43_ = NULL;
					gboolean _tmp44_ = FALSE;
					GXmlNode* _tmp45_ = NULL;
					gchar* _tmp46_ = NULL;
					gchar* _tmp47_ = NULL;
					gchar* _tmp48_ = NULL;
					gboolean _tmp49_ = FALSE;
					GXmlNode* _tmp50_ = NULL;
					gchar* _tmp51_ = NULL;
					gchar* _tmp52_ = NULL;
					gchar* _tmp53_ = NULL;
					gboolean _tmp54_ = FALSE;
#line 556 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp40_ = n;
#line 556 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp41_ = gxml_element_get_content ((GXmlElement*) G_TYPE_CHECK_INSTANCE_CAST (_tmp40_, GXML_TYPE_GELEMENT, GXmlGElement));
#line 556 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp42_ = _tmp41_;
#line 556 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp43_ = _tmp42_;
#line 556 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp44_ = g_strcmp0 (_tmp43_, "Computer") == 0;
#line 556 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_free0 (_tmp43_);
#line 556 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					if (_tmp44_) {
#line 557 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						com = TRUE;
#line 6536 "SerializableObjectModelTest.c"
					}
#line 558 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp45_ = n;
#line 558 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp46_ = gxml_element_get_content ((GXmlElement*) G_TYPE_CHECK_INSTANCE_CAST (_tmp45_, GXML_TYPE_GELEMENT, GXmlGElement));
#line 558 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp47_ = _tmp46_;
#line 558 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp48_ = _tmp47_;
#line 558 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp49_ = g_strcmp0 (_tmp48_, "Customer") == 0;
#line 558 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_free0 (_tmp48_);
#line 558 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					if (_tmp49_) {
#line 559 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						cus = TRUE;
#line 6554 "SerializableObjectModelTest.c"
					}
#line 560 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp50_ = n;
#line 560 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp51_ = gxml_element_get_content ((GXmlElement*) G_TYPE_CHECK_INSTANCE_CAST (_tmp50_, GXML_TYPE_GELEMENT, GXmlGElement));
#line 560 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp52_ = _tmp51_;
#line 560 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp53_ = _tmp52_;
#line 560 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp54_ = g_strcmp0 (_tmp53_, "Sale") == 0;
#line 560 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_free0 (_tmp53_);
#line 560 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					if (_tmp54_) {
#line 561 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						sal = TRUE;
#line 6572 "SerializableObjectModelTest.c"
					}
				}
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_g_object_unref0 (n);
#line 6577 "SerializableObjectModelTest.c"
			}
#line 552 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (_n_list);
#line 6581 "SerializableObjectModelTest.c"
		}
#line 564 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp55_ = com;
#line 564 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (!_tmp55_) {
#line 6587 "SerializableObjectModelTest.c"
			FILE* _tmp56_ = NULL;
#line 565 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp56_ = stdout;
#line 565 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp56_, "%s", "ERROR PACKAGE tag Computer not found!");
#line 566 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 6595 "SerializableObjectModelTest.c"
		}
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp57_ = cus;
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (!_tmp57_) {
#line 6601 "SerializableObjectModelTest.c"
			FILE* _tmp58_ = NULL;
#line 569 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp58_ = stdout;
#line 569 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp58_, "%s", "ERROR PACKAGE tag Customer not found!");
#line 570 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 6609 "SerializableObjectModelTest.c"
		}
#line 572 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp59_ = sal;
#line 572 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (!_tmp59_) {
#line 6615 "SerializableObjectModelTest.c"
			FILE* _tmp60_ = NULL;
#line 573 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp60_ = stdout;
#line 573 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp60_, "%s", "ERROR PACKAGE tag Sale not found!");
#line 574 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 6623 "SerializableObjectModelTest.c"
		}
#line 543 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (element);
#line 6627 "SerializableObjectModelTest.c"
	}
	goto __finally50;
	__catch50_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp61_ = NULL;
		const gchar* _tmp62_ = NULL;
		gchar* _tmp63_ = NULL;
		gchar* _tmp64_ = NULL;
#line 543 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		e = _inner_error_;
#line 543 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_inner_error_ = NULL;
#line 578 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp61_ = e->message;
#line 578 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp62_ = string_to_string (_tmp61_);
#line 578 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp63_ = g_strconcat ("Error: ", _tmp62_, NULL);
#line 578 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp64_ = _tmp63_;
#line 578 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_message ("SerializableObjectModelTest.vala:578: %s", _tmp64_);
#line 578 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp64_);
#line 579 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_assert_not_reached ();
#line 543 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_error_free0 (e);
#line 6657 "SerializableObjectModelTest.c"
	}
	__finally50:
#line 543 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 543 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (package);
#line 543 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (doc);
#line 543 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 543 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_clear_error (&_inner_error_);
#line 543 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		return;
#line 6672 "SerializableObjectModelTest.c"
	}
#line 537 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (package);
#line 537 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (doc);
#line 6678 "SerializableObjectModelTest.c"
}


static void ___lambda18__gtest_func (void) {
#line 536 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	__lambda18_ ();
#line 6685 "SerializableObjectModelTest.c"
}


static void __lambda19_ (void) {
	GXmlGDocument* doc = NULL;
	GXmlGDocument* _tmp0_ = NULL;
	Monitor* monitor = NULL;
	Monitor* _tmp1_ = NULL;
	GError * _inner_error_ = NULL;
#line 585 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = gxml_gdocument_new ();
#line 585 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	doc = _tmp0_;
#line 586 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = monitor_new ();
#line 586 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	monitor = _tmp1_;
#line 6703 "SerializableObjectModelTest.c"
	{
		GXmlNode* _tmp2_ = NULL;
		GXmlNode* _tmp3_ = NULL;
		GXmlNode* _tmp4_ = NULL;
		GXmlNode* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		gboolean _tmp7_ = FALSE;
		GXmlElement* element = NULL;
		GXmlNode* _tmp9_ = NULL;
		GXmlNode* _tmp10_ = NULL;
		GXmlElement* _tmp11_ = NULL;
		gchar* _tmp12_ = NULL;
		gchar* _tmp13_ = NULL;
		gchar* _tmp14_ = NULL;
		gboolean _tmp15_ = FALSE;
		GXmlNode* ac = NULL;
		GXmlNode* _tmp23_ = NULL;
		gchar* _tmp25_ = NULL;
		gchar* _tmp26_ = NULL;
		gchar* _tmp27_ = NULL;
		gboolean _tmp28_ = FALSE;
		GXmlNode* dc = NULL;
		GXmlNode* _tmp36_ = NULL;
		gchar* _tmp38_ = NULL;
		gchar* _tmp39_ = NULL;
		gchar* _tmp40_ = NULL;
		gboolean _tmp41_ = FALSE;
		GXmlNode* r = NULL;
		GXmlNode* _tmp49_ = NULL;
		gchar* _tmp51_ = NULL;
		gchar* _tmp52_ = NULL;
		gchar* _tmp53_ = NULL;
		gboolean _tmp54_ = FALSE;
#line 588 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		monitor_set_resolution (monitor, "1204x720");
#line 589 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		monitor_set_ac_power (monitor, 120);
#line 590 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		monitor_set_dc_power (monitor, 125);
#line 591 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp2_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) monitor, (GXmlNode*) doc, &_inner_error_);
#line 591 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp3_ = _tmp2_;
#line 591 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp3_);
#line 591 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 6751 "SerializableObjectModelTest.c"
			goto __catch51_g_error;
		}
#line 593 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp4_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 593 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = _tmp4_;
#line 593 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = _tmp5_;
#line 593 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp7_ = _tmp6_ == NULL;
#line 593 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp6_);
#line 593 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp7_) {
#line 6766 "SerializableObjectModelTest.c"
			FILE* _tmp8_ = NULL;
#line 594 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp8_ = stdout;
#line 594 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp8_, "ERROR MONITOR: No root GElement");
#line 595 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 6774 "SerializableObjectModelTest.c"
		}
#line 597 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp9_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 597 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp10_ = _tmp9_;
#line 597 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp11_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp10_, GXML_TYPE_ELEMENT) ? ((GXmlElement*) _tmp10_) : NULL;
#line 597 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp11_ == NULL) {
#line 597 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (_tmp10_);
#line 6786 "SerializableObjectModelTest.c"
		}
#line 597 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		element = _tmp11_;
#line 598 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp12_ = gxml_node_get_name ((GXmlNode*) element);
#line 598 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp13_ = _tmp12_;
#line 598 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp14_ = _tmp13_;
#line 598 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp15_ = g_strcmp0 (_tmp14_, "monitor") != 0;
#line 598 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp14_);
#line 598 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp15_) {
#line 6802 "SerializableObjectModelTest.c"
			FILE* _tmp16_ = NULL;
			gchar* _tmp17_ = NULL;
			gchar* _tmp18_ = NULL;
			gchar* _tmp19_ = NULL;
			const gchar* _tmp20_ = NULL;
			gchar* _tmp21_ = NULL;
			gchar* _tmp22_ = NULL;
#line 599 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp16_ = stdout;
#line 599 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp17_ = gxml_node_get_name ((GXmlNode*) element);
#line 599 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp18_ = _tmp17_;
#line 599 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp19_ = _tmp18_;
#line 599 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp20_ = string_to_string (_tmp19_);
#line 599 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp21_ = g_strconcat ("ERROR MONITOR: root GElement ", _tmp20_, NULL);
#line 599 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp22_ = _tmp21_;
#line 599 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp16_, "%s", _tmp22_);
#line 599 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp22_);
#line 599 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp19_);
#line 600 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 6832 "SerializableObjectModelTest.c"
		}
#line 602 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp23_ = gxml_element_get_attr (element, "AcPower");
#line 602 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		ac = _tmp23_;
#line 603 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (ac == NULL) {
#line 6840 "SerializableObjectModelTest.c"
			FILE* _tmp24_ = NULL;
#line 604 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp24_ = stdout;
#line 604 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp24_, "%s", "ERROR MONITOR: attribute AcPower not found");
#line 605 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 6848 "SerializableObjectModelTest.c"
		}
#line 607 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp25_ = gxml_node_get_value (ac);
#line 607 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp26_ = _tmp25_;
#line 607 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp27_ = _tmp26_;
#line 607 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp28_ = g_strcmp0 (_tmp27_, "120") != 0;
#line 607 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp27_);
#line 607 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp28_) {
#line 6862 "SerializableObjectModelTest.c"
			FILE* _tmp29_ = NULL;
			gchar* _tmp30_ = NULL;
			gchar* _tmp31_ = NULL;
			gchar* _tmp32_ = NULL;
			const gchar* _tmp33_ = NULL;
			gchar* _tmp34_ = NULL;
			gchar* _tmp35_ = NULL;
#line 608 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp29_ = stdout;
#line 608 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp30_ = gxml_node_get_value (ac);
#line 608 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp31_ = _tmp30_;
#line 608 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp32_ = _tmp31_;
#line 608 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp33_ = string_to_string (_tmp32_);
#line 608 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp34_ = g_strconcat ("ERROR MONITOR: AcPower value ", _tmp33_, NULL);
#line 608 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp35_ = _tmp34_;
#line 608 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp29_, "%s", _tmp35_);
#line 608 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp35_);
#line 608 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp32_);
#line 609 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 6892 "SerializableObjectModelTest.c"
		}
#line 611 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp36_ = gxml_element_get_attr (element, "DcPower");
#line 611 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		dc = _tmp36_;
#line 612 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (dc == NULL) {
#line 6900 "SerializableObjectModelTest.c"
			FILE* _tmp37_ = NULL;
#line 613 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp37_ = stdout;
#line 613 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp37_, "%s", "ERROR MONITOR: attribute DcPower not found");
#line 614 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 6908 "SerializableObjectModelTest.c"
		}
#line 616 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp38_ = gxml_node_get_value (dc);
#line 616 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp39_ = _tmp38_;
#line 616 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp40_ = _tmp39_;
#line 616 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp41_ = g_strcmp0 (_tmp40_, "125") != 0;
#line 616 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp40_);
#line 616 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp41_) {
#line 6922 "SerializableObjectModelTest.c"
			FILE* _tmp42_ = NULL;
			gchar* _tmp43_ = NULL;
			gchar* _tmp44_ = NULL;
			gchar* _tmp45_ = NULL;
			const gchar* _tmp46_ = NULL;
			gchar* _tmp47_ = NULL;
			gchar* _tmp48_ = NULL;
#line 617 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp42_ = stdout;
#line 617 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp43_ = gxml_node_get_value (dc);
#line 617 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp44_ = _tmp43_;
#line 617 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp45_ = _tmp44_;
#line 617 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp46_ = string_to_string (_tmp45_);
#line 617 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp47_ = g_strconcat ("ERROR MONITOR: AcPower value ", _tmp46_, NULL);
#line 617 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp48_ = _tmp47_;
#line 617 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp42_, "%s", _tmp48_);
#line 617 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp48_);
#line 617 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp45_);
#line 618 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 6952 "SerializableObjectModelTest.c"
		}
#line 620 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp49_ = gxml_element_get_attr (element, "resolution");
#line 620 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		r = _tmp49_;
#line 621 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (r == NULL) {
#line 6960 "SerializableObjectModelTest.c"
			FILE* _tmp50_ = NULL;
#line 622 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp50_ = stdout;
#line 622 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp50_, "%s", "ERROR MONITOR: attribute resolution not found");
#line 623 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 6968 "SerializableObjectModelTest.c"
		}
#line 625 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp51_ = gxml_node_get_value (r);
#line 625 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp52_ = _tmp51_;
#line 625 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp53_ = _tmp52_;
#line 625 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp54_ = g_strcmp0 (_tmp53_, "1204x720") != 0;
#line 625 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp53_);
#line 625 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp54_) {
#line 6982 "SerializableObjectModelTest.c"
			FILE* _tmp55_ = NULL;
			gchar* _tmp56_ = NULL;
			gchar* _tmp57_ = NULL;
			gchar* _tmp58_ = NULL;
			const gchar* _tmp59_ = NULL;
			gchar* _tmp60_ = NULL;
			gchar* _tmp61_ = NULL;
#line 626 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp55_ = stdout;
#line 626 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp56_ = gxml_node_get_value (r);
#line 626 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp57_ = _tmp56_;
#line 626 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp58_ = _tmp57_;
#line 626 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp59_ = string_to_string (_tmp58_);
#line 626 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp60_ = g_strconcat ("ERROR MONITOR: resolution value ", _tmp59_, NULL);
#line 626 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp61_ = _tmp60_;
#line 626 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp55_, "%s", _tmp61_);
#line 626 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp61_);
#line 626 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp58_);
#line 627 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 7012 "SerializableObjectModelTest.c"
		}
#line 587 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (r);
#line 587 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (dc);
#line 587 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (ac);
#line 587 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (element);
#line 7022 "SerializableObjectModelTest.c"
	}
	goto __finally51;
	__catch51_g_error:
	{
		GError* e = NULL;
		FILE* _tmp62_ = NULL;
		const gchar* _tmp63_ = NULL;
		const gchar* _tmp64_ = NULL;
		gchar* _tmp65_ = NULL;
		gchar* _tmp66_ = NULL;
#line 587 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		e = _inner_error_;
#line 587 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_inner_error_ = NULL;
#line 631 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp62_ = stdout;
#line 631 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp63_ = e->message;
#line 631 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp64_ = string_to_string (_tmp63_);
#line 631 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp65_ = g_strconcat ("Error: ", _tmp64_, NULL);
#line 631 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp66_ = _tmp65_;
#line 631 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		fprintf (_tmp62_, "%s", _tmp66_);
#line 631 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp66_);
#line 632 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_assert_not_reached ();
#line 587 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_error_free0 (e);
#line 7055 "SerializableObjectModelTest.c"
	}
	__finally51:
#line 587 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 587 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (monitor);
#line 587 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (doc);
#line 587 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 587 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_clear_error (&_inner_error_);
#line 587 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		return;
#line 7070 "SerializableObjectModelTest.c"
	}
#line 584 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (monitor);
#line 584 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (doc);
#line 7076 "SerializableObjectModelTest.c"
}


static void ___lambda19__gtest_func (void) {
#line 583 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	__lambda19_ ();
#line 7083 "SerializableObjectModelTest.c"
}


static void __lambda20_ (void) {
	GXmlGDocument* doc = NULL;
	GXmlGDocument* _tmp0_ = NULL;
	Configuration* configuration = NULL;
	Configuration* _tmp1_ = NULL;
	GError * _inner_error_ = NULL;
#line 714 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = gxml_gdocument_new ();
#line 714 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	doc = _tmp0_;
#line 715 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = configuration_new ();
#line 715 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	configuration = _tmp1_;
#line 716 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	configuration_set_device (configuration, "Controller");
#line 7103 "SerializableObjectModelTest.c"
	{
		GXmlNode* _tmp2_ = NULL;
		GXmlNode* _tmp3_ = NULL;
		GXmlNode* _tmp4_ = NULL;
		GXmlNode* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		gboolean _tmp7_ = FALSE;
		GXmlElement* element = NULL;
		GXmlNode* _tmp8_ = NULL;
		GXmlNode* _tmp9_ = NULL;
		gchar* _tmp10_ = NULL;
		gchar* _tmp11_ = NULL;
		gchar* _tmp12_ = NULL;
		gboolean _tmp13_ = FALSE;
		gboolean found = FALSE;
		gboolean _tmp35_ = FALSE;
#line 718 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp2_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) configuration, (GXmlNode*) doc, &_inner_error_);
#line 718 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp3_ = _tmp2_;
#line 718 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp3_);
#line 718 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 7128 "SerializableObjectModelTest.c"
			goto __catch52_g_error;
		}
#line 720 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp4_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 720 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = _tmp4_;
#line 720 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = _tmp5_;
#line 720 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp7_ = _tmp6_ == NULL;
#line 720 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp6_);
#line 720 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp7_) {
#line 724 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 7145 "SerializableObjectModelTest.c"
		}
#line 726 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp8_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 726 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp9_ = _tmp8_;
#line 726 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		element = G_TYPE_CHECK_INSTANCE_CAST (_tmp9_, GXML_TYPE_ELEMENT, GXmlElement);
#line 727 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp10_ = gxml_node_get_name ((GXmlNode*) element);
#line 727 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp11_ = _tmp10_;
#line 727 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp12_ = _tmp11_;
#line 727 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp13_ = g_strcmp0 (_tmp12_, "Configuration") != 0;
#line 727 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp12_);
#line 727 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp13_) {
#line 731 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 7167 "SerializableObjectModelTest.c"
		}
#line 733 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		found = FALSE;
#line 7171 "SerializableObjectModelTest.c"
		{
			GeeList* _n_list = NULL;
			GeeList* _tmp14_ = NULL;
			GeeList* _tmp15_ = NULL;
			gint _n_size = 0;
			GeeList* _tmp16_ = NULL;
			gint _tmp17_ = 0;
			gint _tmp18_ = 0;
			gint _n_index = 0;
#line 734 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp14_ = gxml_node_get_namespaces ((GXmlNode*) element);
#line 734 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp15_ = _tmp14_;
#line 734 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_n_list = _tmp15_;
#line 734 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp16_ = _n_list;
#line 734 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp17_ = gee_collection_get_size ((GeeCollection*) _tmp16_);
#line 734 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp18_ = _tmp17_;
#line 734 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_n_size = _tmp18_;
#line 734 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_n_index = -1;
#line 734 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			while (TRUE) {
#line 7199 "SerializableObjectModelTest.c"
				gint _tmp19_ = 0;
				gint _tmp20_ = 0;
				gint _tmp21_ = 0;
				GXmlNamespace* n = NULL;
				GeeList* _tmp22_ = NULL;
				gint _tmp23_ = 0;
				gpointer _tmp24_ = NULL;
				gboolean _tmp25_ = FALSE;
				GXmlNamespace* _tmp26_ = NULL;
				gchar* _tmp27_ = NULL;
				gchar* _tmp28_ = NULL;
				gchar* _tmp29_ = NULL;
				gboolean _tmp30_ = FALSE;
#line 734 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp19_ = _n_index;
#line 734 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_n_index = _tmp19_ + 1;
#line 734 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp20_ = _n_index;
#line 734 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp21_ = _n_size;
#line 734 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				if (!(_tmp20_ < _tmp21_)) {
#line 734 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					break;
#line 7225 "SerializableObjectModelTest.c"
				}
#line 734 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp22_ = _n_list;
#line 734 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp23_ = _n_index;
#line 734 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp24_ = gee_list_get (_tmp22_, _tmp23_);
#line 734 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				n = (GXmlNamespace*) _tmp24_;
#line 736 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp26_ = n;
#line 736 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp27_ = gxml_namespace_get_prefix (_tmp26_);
#line 736 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp28_ = _tmp27_;
#line 736 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp29_ = _tmp28_;
#line 736 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp30_ = g_strcmp0 (_tmp29_, "om") == 0;
#line 736 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_g_free0 (_tmp29_);
#line 736 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				if (_tmp30_) {
#line 7249 "SerializableObjectModelTest.c"
					GXmlNamespace* _tmp31_ = NULL;
					gchar* _tmp32_ = NULL;
					gchar* _tmp33_ = NULL;
					gchar* _tmp34_ = NULL;
#line 736 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp31_ = n;
#line 736 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp32_ = gxml_namespace_get_uri (_tmp31_);
#line 736 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp33_ = _tmp32_;
#line 736 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp34_ = _tmp33_;
#line 736 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp25_ = g_strcmp0 (_tmp34_, "http://www.gnome.org/gxml/0.4") == 0;
#line 736 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_free0 (_tmp34_);
#line 7266 "SerializableObjectModelTest.c"
				} else {
#line 736 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp25_ = FALSE;
#line 7270 "SerializableObjectModelTest.c"
				}
#line 736 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				if (_tmp25_) {
#line 737 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					found = TRUE;
#line 7276 "SerializableObjectModelTest.c"
				}
#line 734 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_g_object_unref0 (n);
#line 7280 "SerializableObjectModelTest.c"
			}
#line 734 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (_n_list);
#line 7284 "SerializableObjectModelTest.c"
		}
#line 739 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp35_ = found;
#line 739 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (!_tmp35_) {
#line 746 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 7292 "SerializableObjectModelTest.c"
		}
#line 717 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (element);
#line 7296 "SerializableObjectModelTest.c"
	}
	goto __finally52;
	__catch52_g_error:
	{
		GError* e = NULL;
		FILE* _tmp36_ = NULL;
		const gchar* _tmp37_ = NULL;
		const gchar* _tmp38_ = NULL;
		gchar* _tmp39_ = NULL;
		gchar* _tmp40_ = NULL;
#line 717 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		e = _inner_error_;
#line 717 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_inner_error_ = NULL;
#line 750 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp36_ = stdout;
#line 750 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp37_ = e->message;
#line 750 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp38_ = string_to_string (_tmp37_);
#line 750 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp39_ = g_strconcat ("Error: ", _tmp38_, NULL);
#line 750 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp40_ = _tmp39_;
#line 750 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		fprintf (_tmp36_, "%s", _tmp40_);
#line 750 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp40_);
#line 751 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_assert_not_reached ();
#line 717 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_error_free0 (e);
#line 7329 "SerializableObjectModelTest.c"
	}
	__finally52:
#line 717 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 717 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (configuration);
#line 717 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (doc);
#line 717 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 717 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_clear_error (&_inner_error_);
#line 717 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		return;
#line 7344 "SerializableObjectModelTest.c"
	}
#line 713 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (configuration);
#line 713 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (doc);
#line 7350 "SerializableObjectModelTest.c"
}


static void ___lambda20__gtest_func (void) {
#line 712 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	__lambda20_ ();
#line 7357 "SerializableObjectModelTest.c"
}


static void __lambda21_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		Configuration* configuration = NULL;
		Configuration* _tmp1_ = NULL;
		Configuration* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		Configuration* _tmp4_ = NULL;
		gboolean _tmp5_ = FALSE;
		gboolean _tmp6_ = FALSE;
#line 757 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp0_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n" \
"       <Configuration xmlns:om=\"http://www.gnome.org/gxml/0.4\" devic" \
"e=\"Sampler\"/>", 0, &_inner_error_);
#line 757 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		doc = _tmp0_;
#line 757 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 7379 "SerializableObjectModelTest.c"
			goto __catch53_g_error;
		}
#line 759 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp1_ = configuration_new ();
#line 759 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		configuration = _tmp1_;
#line 763 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp2_ = configuration;
#line 763 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp3_ = doc;
#line 763 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp2_, (GXmlNode*) _tmp3_, &_inner_error_);
#line 763 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 763 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (configuration);
#line 763 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (doc);
#line 7398 "SerializableObjectModelTest.c"
			goto __catch53_g_error;
		}
#line 767 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp4_ = configuration;
#line 767 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = configuration_get_invalid (_tmp4_);
#line 767 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = _tmp5_;
#line 767 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp6_ == TRUE) {
#line 774 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 7411 "SerializableObjectModelTest.c"
		}
#line 756 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (configuration);
#line 756 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (doc);
#line 7417 "SerializableObjectModelTest.c"
	}
	goto __finally53;
	__catch53_g_error:
	{
		GError* e = NULL;
		FILE* _tmp7_ = NULL;
		const gchar* _tmp8_ = NULL;
		const gchar* _tmp9_ = NULL;
		gchar* _tmp10_ = NULL;
		gchar* _tmp11_ = NULL;
#line 756 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		e = _inner_error_;
#line 756 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_inner_error_ = NULL;
#line 778 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp7_ = stdout;
#line 778 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp8_ = e->message;
#line 778 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp9_ = string_to_string (_tmp8_);
#line 778 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp10_ = g_strconcat ("Error: ", _tmp9_, NULL);
#line 778 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp11_ = _tmp10_;
#line 778 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		fprintf (_tmp7_, "%s", _tmp11_);
#line 778 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp11_);
#line 779 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_assert_not_reached ();
#line 756 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_error_free0 (e);
#line 7450 "SerializableObjectModelTest.c"
	}
	__finally53:
#line 756 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 756 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 756 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_clear_error (&_inner_error_);
#line 756 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		return;
#line 7461 "SerializableObjectModelTest.c"
	}
}


static void ___lambda21__gtest_func (void) {
#line 754 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	__lambda21_ ();
#line 7469 "SerializableObjectModelTest.c"
}


static void __lambda22_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		NodeName* nodename = NULL;
		NodeName* _tmp1_ = NULL;
		NodeName* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
#line 785 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp0_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?><NodeName />", 0, &_inner_error_);
#line 785 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		doc = _tmp0_;
#line 785 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 7488 "SerializableObjectModelTest.c"
			goto __catch54_g_error;
		}
#line 786 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp1_ = node_name_new ();
#line 786 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		nodename = _tmp1_;
#line 787 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp2_ = nodename;
#line 787 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp3_ = doc;
#line 787 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp2_, (GXmlNode*) _tmp3_, &_inner_error_);
#line 787 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 787 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (nodename);
#line 787 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (doc);
#line 7507 "SerializableObjectModelTest.c"
			goto __catch54_g_error;
		}
#line 784 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (nodename);
#line 784 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (doc);
#line 7514 "SerializableObjectModelTest.c"
	}
	goto __finally54;
	__catch54_g_error:
	{
		GError* e = NULL;
		FILE* _tmp4_ = NULL;
		const gchar* _tmp5_ = NULL;
		const gchar* _tmp6_ = NULL;
		gchar* _tmp7_ = NULL;
		gchar* _tmp8_ = NULL;
#line 784 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		e = _inner_error_;
#line 784 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_inner_error_ = NULL;
#line 790 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp4_ = stdout;
#line 790 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = e->message;
#line 790 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = string_to_string (_tmp5_);
#line 790 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp7_ = g_strconcat ("Error: ", _tmp6_, NULL);
#line 790 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp8_ = _tmp7_;
#line 790 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		fprintf (_tmp4_, "%s", _tmp8_);
#line 790 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp8_);
#line 791 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_assert_not_reached ();
#line 784 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_error_free0 (e);
#line 7547 "SerializableObjectModelTest.c"
	}
	__finally54:
#line 784 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 784 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 784 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_clear_error (&_inner_error_);
#line 784 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		return;
#line 7558 "SerializableObjectModelTest.c"
	}
}


static void ___lambda22__gtest_func (void) {
#line 782 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	__lambda22_ ();
#line 7566 "SerializableObjectModelTest.c"
}


static gchar* bool_to_string (gboolean self) {
	gchar* result = NULL;
#line 37 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	if (self) {
#line 7574 "SerializableObjectModelTest.c"
		gchar* _tmp0_ = NULL;
#line 38 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
		_tmp0_ = g_strdup ("true");
#line 38 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
		result = _tmp0_;
#line 38 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
		return result;
#line 7582 "SerializableObjectModelTest.c"
	} else {
		gchar* _tmp1_ = NULL;
#line 40 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
		_tmp1_ = g_strdup ("false");
#line 40 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
		result = _tmp1_;
#line 40 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
		return result;
#line 7591 "SerializableObjectModelTest.c"
	}
}


static void __lambda23_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		UnknownAttribute* unknown_property = NULL;
		UnknownAttribute* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlNode* _tmp3_ = NULL;
		GXmlNode* name = NULL;
		GXmlNode* _tmp4_ = NULL;
		GXmlNode* _tmp5_ = NULL;
		GXmlGElement* _tmp6_ = NULL;
		GXmlGElement* _tmp7_ = NULL;
		GXmlNode* _tmp8_ = NULL;
		GXmlNode* _tmp9_ = NULL;
		GXmlNode* array = NULL;
		GXmlNode* _tmp17_ = NULL;
		GXmlNode* _tmp18_ = NULL;
		GXmlElement* _tmp19_ = NULL;
		GXmlElement* _tmp20_ = NULL;
		GXmlNode* _tmp21_ = NULL;
		GXmlNode* _tmp22_ = NULL;
		GXmlNode* _tmp30_ = NULL;
		GXmlNode* _tmp31_ = NULL;
		GXmlNode* _tmp32_ = NULL;
		GeeBidirList* _tmp33_ = NULL;
		GeeBidirList* _tmp34_ = NULL;
		GeeBidirList* _tmp35_ = NULL;
		gint _tmp36_ = 0;
		gint _tmp37_ = 0;
		gboolean _tmp38_ = FALSE;
#line 797 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp0_ = gxml_gdocument_new ();
#line 797 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		doc = _tmp0_;
#line 798 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp1_ = unknown_attribute_new ();
#line 798 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		unknown_property = _tmp1_;
#line 799 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp2_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) unknown_property, (GXmlNode*) doc, &_inner_error_);
#line 799 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp3_ = _tmp2_;
#line 799 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp3_);
#line 799 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 799 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (unknown_property);
#line 799 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (doc);
#line 7648 "SerializableObjectModelTest.c"
			goto __catch55_g_error;
		}
#line 801 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp4_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 801 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = _tmp4_;
#line 801 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp5_, GXML_TYPE_GELEMENT) ? ((GXmlGElement*) _tmp5_) : NULL;
#line 801 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp6_ == NULL) {
#line 801 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (_tmp5_);
#line 7661 "SerializableObjectModelTest.c"
		}
#line 801 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp7_ = _tmp6_;
#line 801 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp8_ = gxml_element_get_attr ((GXmlElement*) _tmp7_, "name");
#line 801 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp9_ = _tmp8_;
#line 801 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp7_);
#line 801 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		name = _tmp9_;
#line 802 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (name != NULL) {
#line 7675 "SerializableObjectModelTest.c"
			FILE* _tmp10_ = NULL;
			gchar* _tmp11_ = NULL;
			gchar* _tmp12_ = NULL;
			gchar* _tmp13_ = NULL;
			const gchar* _tmp14_ = NULL;
			gchar* _tmp15_ = NULL;
			gchar* _tmp16_ = NULL;
#line 803 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp10_ = stdout;
#line 803 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp11_ = gxml_node_get_name (name);
#line 803 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp12_ = _tmp11_;
#line 803 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp13_ = _tmp12_;
#line 803 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp14_ = string_to_string (_tmp13_);
#line 803 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp15_ = g_strconcat ("ERROR: NULL ATTRIBUTE SERIALIZATION: name found ", _tmp14_, NULL);
#line 803 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp16_ = _tmp15_;
#line 803 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp10_, "%s", _tmp16_);
#line 803 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp16_);
#line 803 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp13_);
#line 804 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 7705 "SerializableObjectModelTest.c"
		}
#line 806 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp17_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 806 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp18_ = _tmp17_;
#line 806 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp19_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp18_, GXML_TYPE_ELEMENT) ? ((GXmlElement*) _tmp18_) : NULL;
#line 806 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp19_ == NULL) {
#line 806 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (_tmp18_);
#line 7717 "SerializableObjectModelTest.c"
		}
#line 806 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp20_ = _tmp19_;
#line 806 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp21_ = gxml_element_get_attr (_tmp20_, "array");
#line 806 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp22_ = _tmp21_;
#line 806 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp20_);
#line 806 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		array = _tmp22_;
#line 807 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (array != NULL) {
#line 7731 "SerializableObjectModelTest.c"
			FILE* _tmp23_ = NULL;
			gchar* _tmp24_ = NULL;
			gchar* _tmp25_ = NULL;
			gchar* _tmp26_ = NULL;
			const gchar* _tmp27_ = NULL;
			gchar* _tmp28_ = NULL;
			gchar* _tmp29_ = NULL;
#line 808 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp23_ = stdout;
#line 808 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp24_ = gxml_node_get_name (array);
#line 808 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp25_ = _tmp24_;
#line 808 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp26_ = _tmp25_;
#line 808 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp27_ = string_to_string (_tmp26_);
#line 808 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp28_ = g_strconcat ("ERROR: NULL ATTRIBUTE SERIALIZATION: array found ", _tmp27_, NULL);
#line 808 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp29_ = _tmp28_;
#line 808 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp23_, "%s", _tmp29_);
#line 808 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp29_);
#line 808 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp26_);
#line 809 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 7761 "SerializableObjectModelTest.c"
		}
#line 811 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp30_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 811 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp31_ = _tmp30_;
#line 811 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp32_ = _tmp31_;
#line 811 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp33_ = gxml_node_get_children_nodes (_tmp32_);
#line 811 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp34_ = _tmp33_;
#line 811 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp35_ = _tmp34_;
#line 811 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp36_ = gee_collection_get_size ((GeeCollection*) _tmp35_);
#line 811 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp37_ = _tmp36_;
#line 811 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp38_ = _tmp37_ > 0;
#line 811 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp35_);
#line 811 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp32_);
#line 811 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp38_) {
#line 7787 "SerializableObjectModelTest.c"
			FILE* _tmp39_ = NULL;
			GXmlNode* _tmp40_ = NULL;
			GXmlNode* _tmp41_ = NULL;
			GXmlNode* _tmp42_ = NULL;
			GeeBidirList* _tmp43_ = NULL;
			GeeBidirList* _tmp44_ = NULL;
			GeeBidirList* _tmp45_ = NULL;
			gint _tmp46_ = 0;
			gint _tmp47_ = 0;
			gchar* _tmp48_ = NULL;
			gchar* _tmp49_ = NULL;
			gchar* _tmp50_ = NULL;
			gchar* _tmp51_ = NULL;
#line 812 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp39_ = stdout;
#line 812 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp40_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 812 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp41_ = _tmp40_;
#line 812 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp42_ = _tmp41_;
#line 812 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp43_ = gxml_node_get_children_nodes (_tmp42_);
#line 812 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp44_ = _tmp43_;
#line 812 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp45_ = _tmp44_;
#line 812 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp46_ = gee_collection_get_size ((GeeCollection*) _tmp45_);
#line 812 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp47_ = _tmp46_;
#line 812 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp48_ = bool_to_string (_tmp47_ > 0);
#line 812 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp49_ = _tmp48_;
#line 812 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp50_ = g_strconcat ("ERROR: NULL ATTRIBUTE SERIALIZATION: Nodes found ", _tmp49_, NULL);
#line 812 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp51_ = _tmp50_;
#line 812 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp39_, "%s", _tmp51_);
#line 812 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp51_);
#line 812 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp49_);
#line 812 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (_tmp45_);
#line 812 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (_tmp42_);
#line 813 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 7839 "SerializableObjectModelTest.c"
		}
#line 796 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (array);
#line 796 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (name);
#line 796 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (unknown_property);
#line 796 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (doc);
#line 7849 "SerializableObjectModelTest.c"
	}
	goto __finally55;
	__catch55_g_error:
	{
		GError* e = NULL;
		FILE* _tmp52_ = NULL;
		const gchar* _tmp53_ = NULL;
		const gchar* _tmp54_ = NULL;
		gchar* _tmp55_ = NULL;
		gchar* _tmp56_ = NULL;
#line 796 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		e = _inner_error_;
#line 796 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_inner_error_ = NULL;
#line 817 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp52_ = stdout;
#line 817 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp53_ = e->message;
#line 817 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp54_ = string_to_string (_tmp53_);
#line 817 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp55_ = g_strconcat ("Error: ", _tmp54_, NULL);
#line 817 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp56_ = _tmp55_;
#line 817 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		fprintf (_tmp52_, "%s", _tmp56_);
#line 817 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp56_);
#line 818 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_assert_not_reached ();
#line 796 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_error_free0 (e);
#line 7882 "SerializableObjectModelTest.c"
	}
	__finally55:
#line 796 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 796 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 796 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_clear_error (&_inner_error_);
#line 796 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		return;
#line 7893 "SerializableObjectModelTest.c"
	}
}


static void ___lambda23__gtest_func (void) {
#line 794 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	__lambda23_ ();
#line 7901 "SerializableObjectModelTest.c"
}


static void __lambda24_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		GXmlGDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlNode* _tmp3_ = NULL;
		GXmlNode* _tmp4_ = NULL;
		GXmlGDocument* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlNode* _tmp8_ = NULL;
		gchar* _tmp9_ = NULL;
		gchar* _tmp10_ = NULL;
		gchar* _tmp11_ = NULL;
		GXmlGDocument* _tmp12_ = NULL;
		GXmlNode* _tmp13_ = NULL;
		GXmlNode* _tmp14_ = NULL;
		GXmlNode* _tmp15_ = NULL;
		GeeBidirList* _tmp16_ = NULL;
		GeeBidirList* _tmp17_ = NULL;
		GeeBidirList* _tmp18_ = NULL;
		gint _tmp19_ = 0;
		gint _tmp20_ = 0;
		UnknownAttribute* unknown_property = NULL;
		UnknownAttribute* _tmp21_ = NULL;
		UnknownAttribute* _tmp22_ = NULL;
		GXmlGDocument* _tmp23_ = NULL;
		GeeMap* ukp = NULL;
		UnknownAttribute* _tmp24_ = NULL;
		GeeMap* _tmp25_ = NULL;
		GeeMap* _tmp26_ = NULL;
		GeeMap* _tmp27_ = NULL;
		gint _tmp28_ = 0;
		gint _tmp29_ = 0;
		GXmlAttribute* ignore = NULL;
		GeeMap* _tmp30_ = NULL;
		gpointer _tmp31_ = NULL;
		GXmlAttribute* _tmp32_ = NULL;
		GXmlAttribute* ignore2 = NULL;
		GeeMap* _tmp33_ = NULL;
		gpointer _tmp34_ = NULL;
		GXmlAttribute* _tmp35_ = NULL;
		UnknownAttribute* _tmp36_ = NULL;
		GeeCollection* _tmp37_ = NULL;
		GeeCollection* _tmp38_ = NULL;
		GeeCollection* _tmp39_ = NULL;
		gint _tmp40_ = 0;
		gint _tmp41_ = 0;
		gboolean foundn = FALSE;
		gboolean foundt = FALSE;
		gboolean _tmp77_ = FALSE;
		gboolean _tmp78_ = FALSE;
#line 824 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp0_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n" \
"      <UnknownAttribute ignore=\"true\" ignore2=\"test\"><UnknownNode " \
"toignore = \"true\" />TEXT</UnknownAttribute>", 0, &_inner_error_);
#line 824 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		doc = _tmp0_;
#line 824 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 7965 "SerializableObjectModelTest.c"
			goto __catch56_g_error;
		}
#line 826 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp1_ = doc;
#line 826 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp2_ = gxml_document_get_root ((GXmlDocument*) _tmp1_);
#line 826 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp3_ = _tmp2_;
#line 826 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp4_ = _tmp3_;
#line 826 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (_tmp4_ != NULL, "doc.root != null");
#line 826 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp4_);
#line 827 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = doc;
#line 827 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = gxml_document_get_root ((GXmlDocument*) _tmp5_);
#line 827 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp7_ = _tmp6_;
#line 827 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp8_ = _tmp7_;
#line 827 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp9_ = gxml_node_get_name (_tmp8_);
#line 827 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp10_ = _tmp9_;
#line 827 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp11_ = _tmp10_;
#line 827 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (g_strcmp0 (_tmp11_, "UnknownAttribute") == 0, "doc.root.name == \"UnknownAttribute\"");
#line 827 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp11_);
#line 827 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp8_);
#line 835 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp12_ = doc;
#line 835 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp13_ = gxml_document_get_root ((GXmlDocument*) _tmp12_);
#line 835 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp14_ = _tmp13_;
#line 835 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp15_ = _tmp14_;
#line 835 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp16_ = gxml_node_get_children_nodes (_tmp15_);
#line 835 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp17_ = _tmp16_;
#line 835 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp18_ = _tmp17_;
#line 835 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp19_ = gee_collection_get_size ((GeeCollection*) _tmp18_);
#line 835 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp20_ = _tmp19_;
#line 835 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (_tmp20_ == 2, "doc.root.children_nodes.size == 2");
#line 835 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp18_);
#line 835 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp15_);
#line 836 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp21_ = unknown_attribute_new ();
#line 836 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		unknown_property = _tmp21_;
#line 837 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp22_ = unknown_property;
#line 837 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp23_ = doc;
#line 837 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp22_, (GXmlNode*) _tmp23_, &_inner_error_);
#line 837 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 837 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (unknown_property);
#line 837 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (doc);
#line 8040 "SerializableObjectModelTest.c"
			goto __catch56_g_error;
		}
#line 848 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp24_ = unknown_property;
#line 848 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp25_ = gxml_serializable_get_unknown_serializable_properties ((GXmlSerializable*) _tmp24_);
#line 848 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp26_ = _tmp25_;
#line 848 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		ukp = _tmp26_;
#line 849 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp27_ = ukp;
#line 849 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp28_ = gee_map_get_size (_tmp27_);
#line 849 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp29_ = _tmp28_;
#line 849 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (_tmp29_ == 2, "ukp.size == 2");
#line 850 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp30_ = ukp;
#line 850 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp31_ = gee_map_get (_tmp30_, "ignore");
#line 850 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		ignore = (GXmlAttribute*) _tmp31_;
#line 851 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp32_ = ignore;
#line 851 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (_tmp32_ != NULL, "ignore != null");
#line 852 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp33_ = ukp;
#line 852 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp34_ = gee_map_get (_tmp33_, "ignore2");
#line 852 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		ignore2 = (GXmlAttribute*) _tmp34_;
#line 853 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp35_ = ignore2;
#line 853 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (_tmp35_ != NULL, "ignore2 != null");
#line 857 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp36_ = unknown_property;
#line 857 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp37_ = gxml_serializable_get_unknown_serializable_nodes ((GXmlSerializable*) _tmp36_);
#line 857 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp38_ = _tmp37_;
#line 857 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp39_ = _tmp38_;
#line 857 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp40_ = gee_collection_get_size (_tmp39_);
#line 857 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp41_ = _tmp40_;
#line 857 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (_tmp41_ == 2, "unknown_property.unknown_serializable_nodes.size == 2");
#line 857 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp39_);
#line 858 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		foundn = FALSE;
#line 859 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		foundt = FALSE;
#line 8099 "SerializableObjectModelTest.c"
		{
			GeeIterator* _n_it = NULL;
			UnknownAttribute* _tmp42_ = NULL;
			GeeCollection* _tmp43_ = NULL;
			GeeCollection* _tmp44_ = NULL;
			GeeCollection* _tmp45_ = NULL;
			GeeIterator* _tmp46_ = NULL;
			GeeIterator* _tmp47_ = NULL;
#line 860 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp42_ = unknown_property;
#line 860 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp43_ = gxml_serializable_get_unknown_serializable_nodes ((GXmlSerializable*) _tmp42_);
#line 860 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp44_ = _tmp43_;
#line 860 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp45_ = _tmp44_;
#line 860 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp46_ = gee_iterable_iterator ((GeeIterable*) _tmp45_);
#line 860 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp47_ = _tmp46_;
#line 860 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (_tmp45_);
#line 860 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_n_it = _tmp47_;
#line 860 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			while (TRUE) {
#line 8126 "SerializableObjectModelTest.c"
				GeeIterator* _tmp48_ = NULL;
				gboolean _tmp49_ = FALSE;
				GXmlNode* n = NULL;
				GeeIterator* _tmp50_ = NULL;
				gpointer _tmp51_ = NULL;
				GXmlNode* _tmp52_ = NULL;
				gchar* _tmp53_ = NULL;
				gchar* _tmp54_ = NULL;
				gchar* _tmp55_ = NULL;
				gboolean _tmp56_ = FALSE;
				GXmlNode* _tmp72_ = NULL;
#line 860 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp48_ = _n_it;
#line 860 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp49_ = gee_iterator_next (_tmp48_);
#line 860 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				if (!_tmp49_) {
#line 860 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					break;
#line 8146 "SerializableObjectModelTest.c"
				}
#line 860 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp50_ = _n_it;
#line 860 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp51_ = gee_iterator_get (_tmp50_);
#line 860 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				n = (GXmlNode*) _tmp51_;
#line 861 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp52_ = n;
#line 861 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp53_ = gxml_node_get_name (_tmp52_);
#line 861 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp54_ = _tmp53_;
#line 861 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp55_ = _tmp54_;
#line 861 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp56_ = g_strcmp0 (_tmp55_, "UnknownNode") == 0;
#line 861 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_g_free0 (_tmp55_);
#line 861 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				if (_tmp56_) {
#line 8168 "SerializableObjectModelTest.c"
					GXmlNode* _tmp57_ = NULL;
					GeeMap* _tmp58_ = NULL;
					GeeMap* _tmp59_ = NULL;
					GeeMap* _tmp60_ = NULL;
					gpointer _tmp61_ = NULL;
					GXmlNode* _tmp62_ = NULL;
					GXmlNode* _tmp63_ = NULL;
					GeeMap* _tmp64_ = NULL;
					GeeMap* _tmp65_ = NULL;
					GeeMap* _tmp66_ = NULL;
					gpointer _tmp67_ = NULL;
					GXmlNode* _tmp68_ = NULL;
					gchar* _tmp69_ = NULL;
					gchar* _tmp70_ = NULL;
					gchar* _tmp71_ = NULL;
#line 862 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					foundn = TRUE;
#line 863 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp57_ = n;
#line 863 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp58_ = gxml_node_get_attrs (_tmp57_);
#line 863 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp59_ = _tmp58_;
#line 863 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp60_ = _tmp59_;
#line 863 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp61_ = gee_map_get (_tmp60_, "toignore");
#line 863 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp62_ = (GXmlNode*) _tmp61_;
#line 863 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_vala_assert (_tmp62_ != NULL, "n.attrs.get (\"toignore\") != null");
#line 863 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_object_unref0 (_tmp62_);
#line 863 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_object_unref0 (_tmp60_);
#line 864 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp63_ = n;
#line 864 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp64_ = gxml_node_get_attrs (_tmp63_);
#line 864 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp65_ = _tmp64_;
#line 864 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp66_ = _tmp65_;
#line 864 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp67_ = gee_map_get (_tmp66_, "toignore");
#line 864 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp68_ = (GXmlNode*) _tmp67_;
#line 864 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp69_ = gxml_node_get_value (_tmp68_);
#line 864 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp70_ = _tmp69_;
#line 864 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp71_ = _tmp70_;
#line 864 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_vala_assert (g_strcmp0 (_tmp71_, "true") == 0, "n.attrs.get (\"toignore\").value == \"true\"");
#line 864 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_free0 (_tmp71_);
#line 864 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_object_unref0 (_tmp68_);
#line 864 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_object_unref0 (_tmp66_);
#line 8230 "SerializableObjectModelTest.c"
				}
#line 866 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp72_ = n;
#line 866 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp72_, GXML_TYPE_TEXT)) {
#line 8236 "SerializableObjectModelTest.c"
					GXmlNode* _tmp73_ = NULL;
					gchar* _tmp74_ = NULL;
					gchar* _tmp75_ = NULL;
					gchar* _tmp76_ = NULL;
#line 867 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					foundt = TRUE;
#line 868 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp73_ = n;
#line 868 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp74_ = gxml_node_get_value (_tmp73_);
#line 868 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp75_ = _tmp74_;
#line 868 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp76_ = _tmp75_;
#line 868 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_vala_assert (g_strcmp0 (_tmp76_, "TEXT") == 0, "n.value == \"TEXT\"");
#line 868 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_free0 (_tmp76_);
#line 8255 "SerializableObjectModelTest.c"
				}
#line 860 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_g_object_unref0 (n);
#line 8259 "SerializableObjectModelTest.c"
			}
#line 860 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (_n_it);
#line 8263 "SerializableObjectModelTest.c"
		}
#line 871 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp77_ = foundn;
#line 871 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (_tmp77_, "foundn");
#line 872 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp78_ = foundt;
#line 872 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (_tmp78_, "foundt");
#line 823 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (ignore2);
#line 823 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (ignore);
#line 823 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (ukp);
#line 823 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (unknown_property);
#line 823 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (doc);
#line 8283 "SerializableObjectModelTest.c"
	}
	goto __finally56;
	__catch56_g_error:
	{
		GError* e = NULL;
		FILE* _tmp79_ = NULL;
		const gchar* _tmp80_ = NULL;
		const gchar* _tmp81_ = NULL;
		gchar* _tmp82_ = NULL;
		gchar* _tmp83_ = NULL;
#line 823 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		e = _inner_error_;
#line 823 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_inner_error_ = NULL;
#line 875 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp79_ = stdout;
#line 875 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp80_ = e->message;
#line 875 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp81_ = string_to_string (_tmp80_);
#line 875 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp82_ = g_strconcat ("Error: ", _tmp81_, NULL);
#line 875 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp83_ = _tmp82_;
#line 875 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		fprintf (_tmp79_, "%s", _tmp83_);
#line 875 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp83_);
#line 876 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_assert_not_reached ();
#line 823 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_error_free0 (e);
#line 8316 "SerializableObjectModelTest.c"
	}
	__finally56:
#line 823 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 823 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 823 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_clear_error (&_inner_error_);
#line 823 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		return;
#line 8327 "SerializableObjectModelTest.c"
	}
}


static void ___lambda24__gtest_func (void) {
#line 821 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	__lambda24_ ();
#line 8335 "SerializableObjectModelTest.c"
}


static void __lambda25_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		GXmlGDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlNode* _tmp3_ = NULL;
		GXmlNode* _tmp4_ = NULL;
		gchar* _tmp5_ = NULL;
		gchar* _tmp6_ = NULL;
		gchar* _tmp7_ = NULL;
		GXmlGDocument* _tmp8_ = NULL;
		GXmlNode* _tmp9_ = NULL;
		GXmlNode* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GeeBidirList* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		gint _tmp15_ = 0;
		gint _tmp16_ = 0;
		GXmlGDocument* _tmp17_ = NULL;
		GXmlNode* _tmp18_ = NULL;
		GXmlNode* _tmp19_ = NULL;
		GXmlNode* _tmp20_ = NULL;
		GeeBidirList* _tmp21_ = NULL;
		GeeBidirList* _tmp22_ = NULL;
		GeeBidirList* _tmp23_ = NULL;
		gpointer _tmp24_ = NULL;
		GXmlNode* _tmp25_ = NULL;
		gchar* _tmp26_ = NULL;
		gchar* _tmp27_ = NULL;
		gchar* _tmp28_ = NULL;
		GXmlGDocument* _tmp29_ = NULL;
		GXmlNode* _tmp30_ = NULL;
		GXmlNode* _tmp31_ = NULL;
		GXmlNode* _tmp32_ = NULL;
		GeeBidirList* _tmp33_ = NULL;
		GeeBidirList* _tmp34_ = NULL;
		GeeBidirList* _tmp35_ = NULL;
		gpointer _tmp36_ = NULL;
		GXmlNode* _tmp37_ = NULL;
		gchar* _tmp38_ = NULL;
		gchar* _tmp39_ = NULL;
		gchar* _tmp40_ = NULL;
		GXmlGDocument* _tmp41_ = NULL;
		GXmlNode* _tmp42_ = NULL;
		GXmlNode* _tmp43_ = NULL;
		GXmlNode* _tmp44_ = NULL;
		GeeBidirList* _tmp45_ = NULL;
		GeeBidirList* _tmp46_ = NULL;
		GeeBidirList* _tmp47_ = NULL;
		gpointer _tmp48_ = NULL;
		GXmlNode* _tmp49_ = NULL;
		GeeBidirList* _tmp50_ = NULL;
		GeeBidirList* _tmp51_ = NULL;
		GeeBidirList* _tmp52_ = NULL;
		gint _tmp53_ = 0;
		gint _tmp54_ = 0;
		GXmlGDocument* _tmp55_ = NULL;
		GXmlNode* _tmp56_ = NULL;
		GXmlNode* _tmp57_ = NULL;
		GXmlNode* _tmp58_ = NULL;
		GeeBidirList* _tmp59_ = NULL;
		GeeBidirList* _tmp60_ = NULL;
		GeeBidirList* _tmp61_ = NULL;
		gpointer _tmp62_ = NULL;
		GXmlNode* _tmp63_ = NULL;
		GeeBidirList* _tmp64_ = NULL;
		GeeBidirList* _tmp65_ = NULL;
		GeeBidirList* _tmp66_ = NULL;
		gpointer _tmp67_ = NULL;
		GXmlNode* _tmp68_ = NULL;
		gchar* _tmp69_ = NULL;
		gchar* _tmp70_ = NULL;
		gchar* _tmp71_ = NULL;
		GXmlGDocument* _tmp72_ = NULL;
		GXmlNode* _tmp73_ = NULL;
		GXmlNode* _tmp74_ = NULL;
		GXmlNode* _tmp75_ = NULL;
		GeeBidirList* _tmp76_ = NULL;
		GeeBidirList* _tmp77_ = NULL;
		GeeBidirList* _tmp78_ = NULL;
		gpointer _tmp79_ = NULL;
		GXmlNode* _tmp80_ = NULL;
		GeeBidirList* _tmp81_ = NULL;
		GeeBidirList* _tmp82_ = NULL;
		GeeBidirList* _tmp83_ = NULL;
		gpointer _tmp84_ = NULL;
		GXmlNode* _tmp85_ = NULL;
		GeeBidirList* _tmp86_ = NULL;
		GeeBidirList* _tmp87_ = NULL;
		GeeBidirList* _tmp88_ = NULL;
		gint _tmp89_ = 0;
		gint _tmp90_ = 0;
		GXmlGDocument* _tmp91_ = NULL;
		GXmlNode* _tmp92_ = NULL;
		GXmlNode* _tmp93_ = NULL;
		GXmlNode* _tmp94_ = NULL;
		GeeBidirList* _tmp95_ = NULL;
		GeeBidirList* _tmp96_ = NULL;
		GeeBidirList* _tmp97_ = NULL;
		gpointer _tmp98_ = NULL;
		GXmlNode* _tmp99_ = NULL;
		GeeBidirList* _tmp100_ = NULL;
		GeeBidirList* _tmp101_ = NULL;
		GeeBidirList* _tmp102_ = NULL;
		gpointer _tmp103_ = NULL;
		GXmlNode* _tmp104_ = NULL;
		GeeBidirList* _tmp105_ = NULL;
		GeeBidirList* _tmp106_ = NULL;
		GeeBidirList* _tmp107_ = NULL;
		gpointer _tmp108_ = NULL;
		GXmlNode* _tmp109_ = NULL;
		gchar* _tmp110_ = NULL;
		gchar* _tmp111_ = NULL;
		gchar* _tmp112_ = NULL;
		GXmlGDocument* _tmp113_ = NULL;
		GXmlNode* _tmp114_ = NULL;
		GXmlNode* _tmp115_ = NULL;
		GXmlNode* _tmp116_ = NULL;
		GeeBidirList* _tmp117_ = NULL;
		GeeBidirList* _tmp118_ = NULL;
		GeeBidirList* _tmp119_ = NULL;
		gpointer _tmp120_ = NULL;
		GXmlNode* _tmp121_ = NULL;
		GeeBidirList* _tmp122_ = NULL;
		GeeBidirList* _tmp123_ = NULL;
		GeeBidirList* _tmp124_ = NULL;
		gpointer _tmp125_ = NULL;
		GXmlNode* _tmp126_ = NULL;
		GeeBidirList* _tmp127_ = NULL;
		GeeBidirList* _tmp128_ = NULL;
		GeeBidirList* _tmp129_ = NULL;
		gpointer _tmp130_ = NULL;
		GXmlNode* _tmp131_ = NULL;
		GeeBidirList* _tmp132_ = NULL;
		GeeBidirList* _tmp133_ = NULL;
		GeeBidirList* _tmp134_ = NULL;
		gint _tmp135_ = 0;
		gint _tmp136_ = 0;
		GXmlGDocument* _tmp137_ = NULL;
		GXmlNode* _tmp138_ = NULL;
		GXmlNode* _tmp139_ = NULL;
		GXmlNode* _tmp140_ = NULL;
		GeeBidirList* _tmp141_ = NULL;
		GeeBidirList* _tmp142_ = NULL;
		GeeBidirList* _tmp143_ = NULL;
		gpointer _tmp144_ = NULL;
		GXmlNode* _tmp145_ = NULL;
		GeeBidirList* _tmp146_ = NULL;
		GeeBidirList* _tmp147_ = NULL;
		GeeBidirList* _tmp148_ = NULL;
		gpointer _tmp149_ = NULL;
		GXmlNode* _tmp150_ = NULL;
		GeeBidirList* _tmp151_ = NULL;
		GeeBidirList* _tmp152_ = NULL;
		GeeBidirList* _tmp153_ = NULL;
		gpointer _tmp154_ = NULL;
		GXmlNode* _tmp155_ = NULL;
		GeeBidirList* _tmp156_ = NULL;
		GeeBidirList* _tmp157_ = NULL;
		GeeBidirList* _tmp158_ = NULL;
		gpointer _tmp159_ = NULL;
		GXmlNode* _tmp160_ = NULL;
		GXmlGDocument* _tmp161_ = NULL;
		GXmlNode* _tmp162_ = NULL;
		GXmlNode* _tmp163_ = NULL;
		GXmlNode* _tmp164_ = NULL;
		GeeBidirList* _tmp165_ = NULL;
		GeeBidirList* _tmp166_ = NULL;
		GeeBidirList* _tmp167_ = NULL;
		gpointer _tmp168_ = NULL;
		GXmlNode* _tmp169_ = NULL;
		GeeBidirList* _tmp170_ = NULL;
		GeeBidirList* _tmp171_ = NULL;
		GeeBidirList* _tmp172_ = NULL;
		gpointer _tmp173_ = NULL;
		GXmlNode* _tmp174_ = NULL;
		GeeBidirList* _tmp175_ = NULL;
		GeeBidirList* _tmp176_ = NULL;
		GeeBidirList* _tmp177_ = NULL;
		gpointer _tmp178_ = NULL;
		GXmlNode* _tmp179_ = NULL;
		GeeBidirList* _tmp180_ = NULL;
		GeeBidirList* _tmp181_ = NULL;
		GeeBidirList* _tmp182_ = NULL;
		gpointer _tmp183_ = NULL;
		GXmlNode* _tmp184_ = NULL;
		gchar* _tmp185_ = NULL;
		gchar* _tmp186_ = NULL;
		gchar* _tmp187_ = NULL;
		UnknownAttribute* unknown_property = NULL;
		UnknownAttribute* _tmp188_ = NULL;
		UnknownAttribute* _tmp189_ = NULL;
		GXmlGDocument* _tmp190_ = NULL;
		GXmlGDocument* doc2 = NULL;
		GXmlGDocument* _tmp191_ = NULL;
		UnknownAttribute* _tmp192_ = NULL;
		GXmlGDocument* _tmp193_ = NULL;
		GXmlNode* _tmp194_ = NULL;
		GXmlNode* _tmp195_ = NULL;
		GXmlGDocument* _tmp196_ = NULL;
		GXmlNode* _tmp197_ = NULL;
		GXmlNode* _tmp198_ = NULL;
		GXmlNode* _tmp199_ = NULL;
		GXmlGDocument* _tmp200_ = NULL;
		GXmlNode* _tmp201_ = NULL;
		GXmlNode* _tmp202_ = NULL;
		GXmlNode* _tmp203_ = NULL;
		gchar* _tmp204_ = NULL;
		gchar* _tmp205_ = NULL;
		gchar* _tmp206_ = NULL;
		GXmlGDocument* _tmp207_ = NULL;
		GXmlNode* _tmp208_ = NULL;
		GXmlNode* _tmp209_ = NULL;
		GXmlNode* _tmp210_ = NULL;
		GeeBidirList* _tmp211_ = NULL;
		GeeBidirList* _tmp212_ = NULL;
		GeeBidirList* _tmp213_ = NULL;
		gint _tmp214_ = 0;
		gint _tmp215_ = 0;
		GXmlGDocument* _tmp216_ = NULL;
		GXmlNode* _tmp217_ = NULL;
		GXmlNode* _tmp218_ = NULL;
		GXmlNode* _tmp219_ = NULL;
		GeeBidirList* _tmp220_ = NULL;
		GeeBidirList* _tmp221_ = NULL;
		GeeBidirList* _tmp222_ = NULL;
		gpointer _tmp223_ = NULL;
		GXmlNode* _tmp224_ = NULL;
		gchar* _tmp225_ = NULL;
		gchar* _tmp226_ = NULL;
		gchar* _tmp227_ = NULL;
		GXmlGDocument* _tmp228_ = NULL;
		GXmlNode* _tmp229_ = NULL;
		GXmlNode* _tmp230_ = NULL;
		GXmlNode* _tmp231_ = NULL;
		GeeBidirList* _tmp232_ = NULL;
		GeeBidirList* _tmp233_ = NULL;
		GeeBidirList* _tmp234_ = NULL;
		gpointer _tmp235_ = NULL;
		GXmlNode* _tmp236_ = NULL;
		GeeBidirList* _tmp237_ = NULL;
		GeeBidirList* _tmp238_ = NULL;
		GeeBidirList* _tmp239_ = NULL;
		gint _tmp240_ = 0;
		gint _tmp241_ = 0;
		GXmlGDocument* _tmp242_ = NULL;
		GXmlNode* _tmp243_ = NULL;
		GXmlNode* _tmp244_ = NULL;
		GXmlNode* _tmp245_ = NULL;
		GeeBidirList* _tmp246_ = NULL;
		GeeBidirList* _tmp247_ = NULL;
		GeeBidirList* _tmp248_ = NULL;
		gpointer _tmp249_ = NULL;
		GXmlNode* _tmp250_ = NULL;
		gchar* _tmp251_ = NULL;
		gchar* _tmp252_ = NULL;
		gchar* _tmp253_ = NULL;
		GXmlGDocument* _tmp254_ = NULL;
		GXmlNode* _tmp255_ = NULL;
		GXmlNode* _tmp256_ = NULL;
		GXmlNode* _tmp257_ = NULL;
		GeeBidirList* _tmp258_ = NULL;
		GeeBidirList* _tmp259_ = NULL;
		GeeBidirList* _tmp260_ = NULL;
		gpointer _tmp261_ = NULL;
		GXmlNode* _tmp262_ = NULL;
		GeeBidirList* _tmp263_ = NULL;
		GeeBidirList* _tmp264_ = NULL;
		GeeBidirList* _tmp265_ = NULL;
		gpointer _tmp266_ = NULL;
		GXmlNode* _tmp267_ = NULL;
		gchar* _tmp268_ = NULL;
		gchar* _tmp269_ = NULL;
		gchar* _tmp270_ = NULL;
		GXmlGDocument* _tmp271_ = NULL;
		GXmlNode* _tmp272_ = NULL;
		GXmlNode* _tmp273_ = NULL;
		GXmlNode* _tmp274_ = NULL;
		GeeBidirList* _tmp275_ = NULL;
		GeeBidirList* _tmp276_ = NULL;
		GeeBidirList* _tmp277_ = NULL;
		gpointer _tmp278_ = NULL;
		GXmlNode* _tmp279_ = NULL;
		GeeBidirList* _tmp280_ = NULL;
		GeeBidirList* _tmp281_ = NULL;
		GeeBidirList* _tmp282_ = NULL;
		gpointer _tmp283_ = NULL;
		GXmlNode* _tmp284_ = NULL;
		GeeBidirList* _tmp285_ = NULL;
		GeeBidirList* _tmp286_ = NULL;
		GeeBidirList* _tmp287_ = NULL;
		gpointer _tmp288_ = NULL;
		GXmlNode* _tmp289_ = NULL;
		gchar* _tmp290_ = NULL;
		gchar* _tmp291_ = NULL;
		gchar* _tmp292_ = NULL;
		GXmlGDocument* _tmp293_ = NULL;
		GXmlNode* _tmp294_ = NULL;
		GXmlNode* _tmp295_ = NULL;
		GXmlNode* _tmp296_ = NULL;
		GeeBidirList* _tmp297_ = NULL;
		GeeBidirList* _tmp298_ = NULL;
		GeeBidirList* _tmp299_ = NULL;
		gpointer _tmp300_ = NULL;
		GXmlNode* _tmp301_ = NULL;
		GeeBidirList* _tmp302_ = NULL;
		GeeBidirList* _tmp303_ = NULL;
		GeeBidirList* _tmp304_ = NULL;
		gpointer _tmp305_ = NULL;
		GXmlNode* _tmp306_ = NULL;
		GeeBidirList* _tmp307_ = NULL;
		GeeBidirList* _tmp308_ = NULL;
		GeeBidirList* _tmp309_ = NULL;
		gint _tmp310_ = 0;
		gint _tmp311_ = 0;
		GXmlGDocument* _tmp312_ = NULL;
		GXmlNode* _tmp313_ = NULL;
		GXmlNode* _tmp314_ = NULL;
		GXmlNode* _tmp315_ = NULL;
		GeeBidirList* _tmp316_ = NULL;
		GeeBidirList* _tmp317_ = NULL;
		GeeBidirList* _tmp318_ = NULL;
		gpointer _tmp319_ = NULL;
		GXmlNode* _tmp320_ = NULL;
		GeeBidirList* _tmp321_ = NULL;
		GeeBidirList* _tmp322_ = NULL;
		GeeBidirList* _tmp323_ = NULL;
		gpointer _tmp324_ = NULL;
		GXmlNode* _tmp325_ = NULL;
		GeeBidirList* _tmp326_ = NULL;
		GeeBidirList* _tmp327_ = NULL;
		GeeBidirList* _tmp328_ = NULL;
		gpointer _tmp329_ = NULL;
		GXmlNode* _tmp330_ = NULL;
		GeeBidirList* _tmp331_ = NULL;
		GeeBidirList* _tmp332_ = NULL;
		GeeBidirList* _tmp333_ = NULL;
		gint _tmp334_ = 0;
		gint _tmp335_ = 0;
		GXmlGDocument* _tmp336_ = NULL;
		GXmlNode* _tmp337_ = NULL;
		GXmlNode* _tmp338_ = NULL;
		GXmlNode* _tmp339_ = NULL;
		GeeBidirList* _tmp340_ = NULL;
		GeeBidirList* _tmp341_ = NULL;
		GeeBidirList* _tmp342_ = NULL;
		gpointer _tmp343_ = NULL;
		GXmlNode* _tmp344_ = NULL;
		GeeBidirList* _tmp345_ = NULL;
		GeeBidirList* _tmp346_ = NULL;
		GeeBidirList* _tmp347_ = NULL;
		gpointer _tmp348_ = NULL;
		GXmlNode* _tmp349_ = NULL;
		GeeBidirList* _tmp350_ = NULL;
		GeeBidirList* _tmp351_ = NULL;
		GeeBidirList* _tmp352_ = NULL;
		gpointer _tmp353_ = NULL;
		GXmlNode* _tmp354_ = NULL;
		GeeBidirList* _tmp355_ = NULL;
		GeeBidirList* _tmp356_ = NULL;
		GeeBidirList* _tmp357_ = NULL;
		gpointer _tmp358_ = NULL;
		GXmlNode* _tmp359_ = NULL;
		GXmlGDocument* _tmp360_ = NULL;
		GXmlNode* _tmp361_ = NULL;
		GXmlNode* _tmp362_ = NULL;
		GXmlNode* _tmp363_ = NULL;
		GeeBidirList* _tmp364_ = NULL;
		GeeBidirList* _tmp365_ = NULL;
		GeeBidirList* _tmp366_ = NULL;
		gpointer _tmp367_ = NULL;
		GXmlNode* _tmp368_ = NULL;
		GeeBidirList* _tmp369_ = NULL;
		GeeBidirList* _tmp370_ = NULL;
		GeeBidirList* _tmp371_ = NULL;
		gpointer _tmp372_ = NULL;
		GXmlNode* _tmp373_ = NULL;
		GeeBidirList* _tmp374_ = NULL;
		GeeBidirList* _tmp375_ = NULL;
		GeeBidirList* _tmp376_ = NULL;
		gpointer _tmp377_ = NULL;
		GXmlNode* _tmp378_ = NULL;
		GeeBidirList* _tmp379_ = NULL;
		GeeBidirList* _tmp380_ = NULL;
		GeeBidirList* _tmp381_ = NULL;
		gpointer _tmp382_ = NULL;
		GXmlNode* _tmp383_ = NULL;
		gchar* _tmp384_ = NULL;
		gchar* _tmp385_ = NULL;
		gchar* _tmp386_ = NULL;
#line 882 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp0_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n" \
"       <UnknownAttribute ignore=\"true\" ignore2=\"test\">\n" \
"        <UnknownNode direction = \"fordward\">\n" \
"         <UnknownChild t = \"test\">\n" \
"          <UnknownChildTwo t = \"test\">SECOND FAKE TEXT</UnknownChild" \
"Two>\n" \
"         </UnknownChild>\n" \
"       </UnknownNode>FAKE TEXT</UnknownAttribute>", 0, &_inner_error_);
#line 882 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		doc = _tmp0_;
#line 882 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 8738 "SerializableObjectModelTest.c"
			goto __catch57_g_error;
		}
#line 889 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp1_ = doc;
#line 889 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp2_ = gxml_document_get_root ((GXmlDocument*) _tmp1_);
#line 889 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp3_ = _tmp2_;
#line 889 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp4_ = _tmp3_;
#line 889 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = gxml_node_get_name (_tmp4_);
#line 889 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = _tmp5_;
#line 889 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp7_ = _tmp6_;
#line 889 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (g_strcmp0 (_tmp7_, "UnknownAttribute") == 0, "doc.root.name == \"UnknownAttribute\"");
#line 889 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp7_);
#line 889 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp4_);
#line 890 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp8_ = doc;
#line 890 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp9_ = gxml_document_get_root ((GXmlDocument*) _tmp8_);
#line 890 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp10_ = _tmp9_;
#line 890 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp11_ = _tmp10_;
#line 890 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp12_ = gxml_node_get_children_nodes (_tmp11_);
#line 890 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp13_ = _tmp12_;
#line 890 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp14_ = _tmp13_;
#line 890 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp15_ = gee_collection_get_size ((GeeCollection*) _tmp14_);
#line 890 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp16_ = _tmp15_;
#line 890 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (_tmp16_ == 3, "doc.root.children_nodes.size == 3");
#line 890 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp14_);
#line 890 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp11_);
#line 891 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp17_ = doc;
#line 891 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp18_ = gxml_document_get_root ((GXmlDocument*) _tmp17_);
#line 891 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp19_ = _tmp18_;
#line 891 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp20_ = _tmp19_;
#line 891 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp21_ = gxml_node_get_children_nodes (_tmp20_);
#line 891 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp22_ = _tmp21_;
#line 891 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp23_ = _tmp22_;
#line 891 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp24_ = gee_list_get ((GeeList*) _tmp23_, 1);
#line 891 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp25_ = (GXmlNode*) _tmp24_;
#line 891 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp26_ = gxml_node_get_name (_tmp25_);
#line 891 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp27_ = _tmp26_;
#line 891 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp28_ = _tmp27_;
#line 891 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (g_strcmp0 (_tmp28_, "UnknownNode") == 0, "doc.root.children_nodes[1].name == \"UnknownNode\"");
#line 891 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp28_);
#line 891 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp25_);
#line 891 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp23_);
#line 891 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp20_);
#line 892 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp29_ = doc;
#line 892 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp30_ = gxml_document_get_root ((GXmlDocument*) _tmp29_);
#line 892 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp31_ = _tmp30_;
#line 892 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp32_ = _tmp31_;
#line 892 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp33_ = gxml_node_get_children_nodes (_tmp32_);
#line 892 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp34_ = _tmp33_;
#line 892 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp35_ = _tmp34_;
#line 892 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp36_ = gee_list_get ((GeeList*) _tmp35_, 2);
#line 892 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp37_ = (GXmlNode*) _tmp36_;
#line 892 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp38_ = gxml_node_get_value (_tmp37_);
#line 892 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp39_ = _tmp38_;
#line 892 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp40_ = _tmp39_;
#line 892 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (g_strcmp0 (_tmp40_, "FAKE TEXT") == 0, "doc.root.children_nodes[2].value == \"FAKE TEXT\"");
#line 892 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp40_);
#line 892 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp37_);
#line 892 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp35_);
#line 892 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp32_);
#line 893 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp41_ = doc;
#line 893 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp42_ = gxml_document_get_root ((GXmlDocument*) _tmp41_);
#line 893 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp43_ = _tmp42_;
#line 893 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp44_ = _tmp43_;
#line 893 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp45_ = gxml_node_get_children_nodes (_tmp44_);
#line 893 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp46_ = _tmp45_;
#line 893 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp47_ = _tmp46_;
#line 893 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp48_ = gee_list_get ((GeeList*) _tmp47_, 1);
#line 893 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp49_ = (GXmlNode*) _tmp48_;
#line 893 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp50_ = gxml_node_get_children_nodes (_tmp49_);
#line 893 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp51_ = _tmp50_;
#line 893 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp52_ = _tmp51_;
#line 893 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp53_ = gee_collection_get_size ((GeeCollection*) _tmp52_);
#line 893 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp54_ = _tmp53_;
#line 893 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (_tmp54_ == 3, "doc.root.children_nodes[1].children_nodes.size == 3");
#line 893 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp52_);
#line 893 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp49_);
#line 893 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp47_);
#line 893 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp44_);
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp55_ = doc;
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp56_ = gxml_document_get_root ((GXmlDocument*) _tmp55_);
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp57_ = _tmp56_;
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp58_ = _tmp57_;
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp59_ = gxml_node_get_children_nodes (_tmp58_);
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp60_ = _tmp59_;
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp61_ = _tmp60_;
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp62_ = gee_list_get ((GeeList*) _tmp61_, 1);
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp63_ = (GXmlNode*) _tmp62_;
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp64_ = gxml_node_get_children_nodes (_tmp63_);
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp65_ = _tmp64_;
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp66_ = _tmp65_;
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp67_ = gee_list_get ((GeeList*) _tmp66_, 1);
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp68_ = (GXmlNode*) _tmp67_;
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp69_ = gxml_node_get_name (_tmp68_);
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp70_ = _tmp69_;
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp71_ = _tmp70_;
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (g_strcmp0 (_tmp71_, "UnknownChild") == 0, "doc.root.children_nodes[1].children_nodes[1].name == \"UnknownChild\"");
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp71_);
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp68_);
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp66_);
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp63_);
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp61_);
#line 894 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp58_);
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp72_ = doc;
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp73_ = gxml_document_get_root ((GXmlDocument*) _tmp72_);
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp74_ = _tmp73_;
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp75_ = _tmp74_;
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp76_ = gxml_node_get_children_nodes (_tmp75_);
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp77_ = _tmp76_;
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp78_ = _tmp77_;
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp79_ = gee_list_get ((GeeList*) _tmp78_, 1);
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp80_ = (GXmlNode*) _tmp79_;
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp81_ = gxml_node_get_children_nodes (_tmp80_);
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp82_ = _tmp81_;
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp83_ = _tmp82_;
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp84_ = gee_list_get ((GeeList*) _tmp83_, 1);
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp85_ = (GXmlNode*) _tmp84_;
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp86_ = gxml_node_get_children_nodes (_tmp85_);
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp87_ = _tmp86_;
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp88_ = _tmp87_;
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp89_ = gee_collection_get_size ((GeeCollection*) _tmp88_);
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp90_ = _tmp89_;
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (_tmp90_ == 3, "doc.root.children_nodes[1].children_nodes[1].children_nodes.size == 3");
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp88_);
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp85_);
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp83_);
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp80_);
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp78_);
#line 895 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp75_);
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp91_ = doc;
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp92_ = gxml_document_get_root ((GXmlDocument*) _tmp91_);
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp93_ = _tmp92_;
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp94_ = _tmp93_;
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp95_ = gxml_node_get_children_nodes (_tmp94_);
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp96_ = _tmp95_;
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp97_ = _tmp96_;
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp98_ = gee_list_get ((GeeList*) _tmp97_, 1);
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp99_ = (GXmlNode*) _tmp98_;
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp100_ = gxml_node_get_children_nodes (_tmp99_);
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp101_ = _tmp100_;
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp102_ = _tmp101_;
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp103_ = gee_list_get ((GeeList*) _tmp102_, 1);
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp104_ = (GXmlNode*) _tmp103_;
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp105_ = gxml_node_get_children_nodes (_tmp104_);
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp106_ = _tmp105_;
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp107_ = _tmp106_;
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp108_ = gee_list_get ((GeeList*) _tmp107_, 1);
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp109_ = (GXmlNode*) _tmp108_;
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp110_ = gxml_node_get_name (_tmp109_);
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp111_ = _tmp110_;
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp112_ = _tmp111_;
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (g_strcmp0 (_tmp112_, "UnknownChildTwo") == 0, "doc.root.children_nodes[1].children_nodes[1].children_nodes[1].name == \"UnknownChildTwo\"");
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp112_);
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp109_);
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp107_);
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp104_);
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp102_);
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp99_);
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp97_);
#line 896 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp94_);
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp113_ = doc;
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp114_ = gxml_document_get_root ((GXmlDocument*) _tmp113_);
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp115_ = _tmp114_;
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp116_ = _tmp115_;
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp117_ = gxml_node_get_children_nodes (_tmp116_);
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp118_ = _tmp117_;
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp119_ = _tmp118_;
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp120_ = gee_list_get ((GeeList*) _tmp119_, 1);
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp121_ = (GXmlNode*) _tmp120_;
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp122_ = gxml_node_get_children_nodes (_tmp121_);
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp123_ = _tmp122_;
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp124_ = _tmp123_;
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp125_ = gee_list_get ((GeeList*) _tmp124_, 1);
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp126_ = (GXmlNode*) _tmp125_;
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp127_ = gxml_node_get_children_nodes (_tmp126_);
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp128_ = _tmp127_;
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp129_ = _tmp128_;
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp130_ = gee_list_get ((GeeList*) _tmp129_, 1);
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp131_ = (GXmlNode*) _tmp130_;
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp132_ = gxml_node_get_children_nodes (_tmp131_);
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp133_ = _tmp132_;
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp134_ = _tmp133_;
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp135_ = gee_collection_get_size ((GeeCollection*) _tmp134_);
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp136_ = _tmp135_;
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (_tmp136_ == 1, "doc.root.children_nodes[1].children_nodes[1].children_nodes[1].children_nodes.size == 1");
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp134_);
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp131_);
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp129_);
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp126_);
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp124_);
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp121_);
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp119_);
#line 897 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp116_);
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp137_ = doc;
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp138_ = gxml_document_get_root ((GXmlDocument*) _tmp137_);
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp139_ = _tmp138_;
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp140_ = _tmp139_;
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp141_ = gxml_node_get_children_nodes (_tmp140_);
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp142_ = _tmp141_;
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp143_ = _tmp142_;
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp144_ = gee_list_get ((GeeList*) _tmp143_, 1);
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp145_ = (GXmlNode*) _tmp144_;
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp146_ = gxml_node_get_children_nodes (_tmp145_);
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp147_ = _tmp146_;
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp148_ = _tmp147_;
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp149_ = gee_list_get ((GeeList*) _tmp148_, 1);
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp150_ = (GXmlNode*) _tmp149_;
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp151_ = gxml_node_get_children_nodes (_tmp150_);
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp152_ = _tmp151_;
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp153_ = _tmp152_;
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp154_ = gee_list_get ((GeeList*) _tmp153_, 1);
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp155_ = (GXmlNode*) _tmp154_;
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp156_ = gxml_node_get_children_nodes (_tmp155_);
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp157_ = _tmp156_;
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp158_ = _tmp157_;
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp159_ = gee_list_get ((GeeList*) _tmp158_, 0);
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp160_ = (GXmlNode*) _tmp159_;
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp160_, GXML_TYPE_TEXT), "doc.root.children_nodes[1].children_nodes[1].children_nodes[1].children_nodes[0] is GXml.Text");
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp160_);
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp158_);
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp155_);
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp153_);
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp150_);
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp148_);
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp145_);
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp143_);
#line 898 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp140_);
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp161_ = doc;
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp162_ = gxml_document_get_root ((GXmlDocument*) _tmp161_);
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp163_ = _tmp162_;
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp164_ = _tmp163_;
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp165_ = gxml_node_get_children_nodes (_tmp164_);
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp166_ = _tmp165_;
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp167_ = _tmp166_;
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp168_ = gee_list_get ((GeeList*) _tmp167_, 1);
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp169_ = (GXmlNode*) _tmp168_;
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp170_ = gxml_node_get_children_nodes (_tmp169_);
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp171_ = _tmp170_;
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp172_ = _tmp171_;
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp173_ = gee_list_get ((GeeList*) _tmp172_, 1);
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp174_ = (GXmlNode*) _tmp173_;
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp175_ = gxml_node_get_children_nodes (_tmp174_);
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp176_ = _tmp175_;
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp177_ = _tmp176_;
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp178_ = gee_list_get ((GeeList*) _tmp177_, 1);
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp179_ = (GXmlNode*) _tmp178_;
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp180_ = gxml_node_get_children_nodes (_tmp179_);
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp181_ = _tmp180_;
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp182_ = _tmp181_;
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp183_ = gee_list_get ((GeeList*) _tmp182_, 0);
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp184_ = (GXmlNode*) _tmp183_;
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp185_ = gxml_node_get_value (_tmp184_);
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp186_ = _tmp185_;
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp187_ = _tmp186_;
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (g_strcmp0 (_tmp187_, "SECOND FAKE TEXT") == 0, "doc.root.children_nodes[1].children_nodes[1].children_nodes[1].children_nodes[0].value == \"SECOND FAKE TEXT\"");
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp187_);
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp184_);
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp182_);
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp179_);
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp177_);
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp174_);
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp172_);
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp169_);
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp167_);
#line 899 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp164_);
#line 900 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp188_ = unknown_attribute_new ();
#line 900 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		unknown_property = _tmp188_;
#line 901 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp189_ = unknown_property;
#line 901 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp190_ = doc;
#line 901 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp189_, (GXmlNode*) _tmp190_, &_inner_error_);
#line 901 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 901 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (unknown_property);
#line 901 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (doc);
#line 9279 "SerializableObjectModelTest.c"
			goto __catch57_g_error;
		}
#line 902 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp191_ = gxml_gdocument_new ();
#line 902 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		doc2 = _tmp191_;
#line 903 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp192_ = unknown_property;
#line 903 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp193_ = doc2;
#line 903 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp194_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) _tmp192_, (GXmlNode*) _tmp193_, &_inner_error_);
#line 903 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp195_ = _tmp194_;
#line 903 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp195_);
#line 903 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 903 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (doc2);
#line 903 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (unknown_property);
#line 903 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (doc);
#line 9304 "SerializableObjectModelTest.c"
			goto __catch57_g_error;
		}
#line 913 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp196_ = doc2;
#line 913 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp197_ = gxml_document_get_root ((GXmlDocument*) _tmp196_);
#line 913 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp198_ = _tmp197_;
#line 913 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp199_ = _tmp198_;
#line 913 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (_tmp199_ != NULL, "doc2.root != null");
#line 913 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp199_);
#line 914 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp200_ = doc;
#line 914 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp201_ = gxml_document_get_root ((GXmlDocument*) _tmp200_);
#line 914 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp202_ = _tmp201_;
#line 914 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp203_ = _tmp202_;
#line 914 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp204_ = gxml_node_get_name (_tmp203_);
#line 914 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp205_ = _tmp204_;
#line 914 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp206_ = _tmp205_;
#line 914 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (g_strcmp0 (_tmp206_, "UnknownAttribute") == 0, "doc.root.name == \"UnknownAttribute\"");
#line 914 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp206_);
#line 914 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp203_);
#line 915 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp207_ = doc2;
#line 915 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp208_ = gxml_document_get_root ((GXmlDocument*) _tmp207_);
#line 915 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp209_ = _tmp208_;
#line 915 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp210_ = _tmp209_;
#line 915 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp211_ = gxml_node_get_children_nodes (_tmp210_);
#line 915 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp212_ = _tmp211_;
#line 915 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp213_ = _tmp212_;
#line 915 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp214_ = gee_collection_get_size ((GeeCollection*) _tmp213_);
#line 915 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp215_ = _tmp214_;
#line 915 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (_tmp215_ == 3, "doc2.root.children_nodes.size == 3");
#line 915 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp213_);
#line 915 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp210_);
#line 916 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp216_ = doc2;
#line 916 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp217_ = gxml_document_get_root ((GXmlDocument*) _tmp216_);
#line 916 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp218_ = _tmp217_;
#line 916 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp219_ = _tmp218_;
#line 916 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp220_ = gxml_node_get_children_nodes (_tmp219_);
#line 916 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp221_ = _tmp220_;
#line 916 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp222_ = _tmp221_;
#line 916 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp223_ = gee_list_get ((GeeList*) _tmp222_, 1);
#line 916 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp224_ = (GXmlNode*) _tmp223_;
#line 916 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp225_ = gxml_node_get_name (_tmp224_);
#line 916 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp226_ = _tmp225_;
#line 916 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp227_ = _tmp226_;
#line 916 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (g_strcmp0 (_tmp227_, "UnknownNode") == 0, "doc2.root.children_nodes[1].name == \"UnknownNode\"");
#line 916 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp227_);
#line 916 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp224_);
#line 916 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp222_);
#line 916 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp219_);
#line 917 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp228_ = doc2;
#line 917 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp229_ = gxml_document_get_root ((GXmlDocument*) _tmp228_);
#line 917 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp230_ = _tmp229_;
#line 917 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp231_ = _tmp230_;
#line 917 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp232_ = gxml_node_get_children_nodes (_tmp231_);
#line 917 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp233_ = _tmp232_;
#line 917 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp234_ = _tmp233_;
#line 917 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp235_ = gee_list_get ((GeeList*) _tmp234_, 1);
#line 917 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp236_ = (GXmlNode*) _tmp235_;
#line 917 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp237_ = gxml_node_get_children_nodes (_tmp236_);
#line 917 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp238_ = _tmp237_;
#line 917 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp239_ = _tmp238_;
#line 917 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp240_ = gee_collection_get_size ((GeeCollection*) _tmp239_);
#line 917 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp241_ = _tmp240_;
#line 917 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (_tmp241_ == 3, "doc2.root.children_nodes[1].children_nodes.size == 3");
#line 917 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp239_);
#line 917 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp236_);
#line 917 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp234_);
#line 917 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp231_);
#line 918 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp242_ = doc2;
#line 918 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp243_ = gxml_document_get_root ((GXmlDocument*) _tmp242_);
#line 918 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp244_ = _tmp243_;
#line 918 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp245_ = _tmp244_;
#line 918 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp246_ = gxml_node_get_children_nodes (_tmp245_);
#line 918 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp247_ = _tmp246_;
#line 918 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp248_ = _tmp247_;
#line 918 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp249_ = gee_list_get ((GeeList*) _tmp248_, 2);
#line 918 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp250_ = (GXmlNode*) _tmp249_;
#line 918 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp251_ = gxml_node_get_value (_tmp250_);
#line 918 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp252_ = _tmp251_;
#line 918 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp253_ = _tmp252_;
#line 918 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (g_strcmp0 (_tmp253_, "FAKE TEXT") == 0, "doc2.root.children_nodes[2].value == \"FAKE TEXT\"");
#line 918 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp253_);
#line 918 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp250_);
#line 918 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp248_);
#line 918 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp245_);
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp254_ = doc2;
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp255_ = gxml_document_get_root ((GXmlDocument*) _tmp254_);
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp256_ = _tmp255_;
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp257_ = _tmp256_;
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp258_ = gxml_node_get_children_nodes (_tmp257_);
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp259_ = _tmp258_;
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp260_ = _tmp259_;
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp261_ = gee_list_get ((GeeList*) _tmp260_, 1);
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp262_ = (GXmlNode*) _tmp261_;
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp263_ = gxml_node_get_children_nodes (_tmp262_);
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp264_ = _tmp263_;
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp265_ = _tmp264_;
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp266_ = gee_list_get ((GeeList*) _tmp265_, 1);
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp267_ = (GXmlNode*) _tmp266_;
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp268_ = gxml_node_get_name (_tmp267_);
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp269_ = _tmp268_;
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp270_ = _tmp269_;
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (g_strcmp0 (_tmp270_, "UnknownChild") == 0, "doc2.root.children_nodes[1].children_nodes[1].name == \"UnknownChild\"");
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp270_);
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp267_);
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp265_);
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp262_);
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp260_);
#line 919 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp257_);
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp271_ = doc2;
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp272_ = gxml_document_get_root ((GXmlDocument*) _tmp271_);
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp273_ = _tmp272_;
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp274_ = _tmp273_;
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp275_ = gxml_node_get_children_nodes (_tmp274_);
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp276_ = _tmp275_;
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp277_ = _tmp276_;
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp278_ = gee_list_get ((GeeList*) _tmp277_, 1);
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp279_ = (GXmlNode*) _tmp278_;
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp280_ = gxml_node_get_children_nodes (_tmp279_);
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp281_ = _tmp280_;
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp282_ = _tmp281_;
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp283_ = gee_list_get ((GeeList*) _tmp282_, 1);
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp284_ = (GXmlNode*) _tmp283_;
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp285_ = gxml_node_get_children_nodes (_tmp284_);
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp286_ = _tmp285_;
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp287_ = _tmp286_;
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp288_ = gee_list_get ((GeeList*) _tmp287_, 1);
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp289_ = (GXmlNode*) _tmp288_;
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp290_ = gxml_node_get_name (_tmp289_);
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp291_ = _tmp290_;
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp292_ = _tmp291_;
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (g_strcmp0 (_tmp292_, "UnknownChildTwo") == 0, "doc2.root.children_nodes[1].children_nodes[1].children_nodes[1].name == \"UnknownChildTwo\"");
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp292_);
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp289_);
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp287_);
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp284_);
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp282_);
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp279_);
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp277_);
#line 920 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp274_);
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp293_ = doc2;
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp294_ = gxml_document_get_root ((GXmlDocument*) _tmp293_);
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp295_ = _tmp294_;
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp296_ = _tmp295_;
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp297_ = gxml_node_get_children_nodes (_tmp296_);
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp298_ = _tmp297_;
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp299_ = _tmp298_;
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp300_ = gee_list_get ((GeeList*) _tmp299_, 1);
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp301_ = (GXmlNode*) _tmp300_;
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp302_ = gxml_node_get_children_nodes (_tmp301_);
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp303_ = _tmp302_;
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp304_ = _tmp303_;
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp305_ = gee_list_get ((GeeList*) _tmp304_, 1);
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp306_ = (GXmlNode*) _tmp305_;
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp307_ = gxml_node_get_children_nodes (_tmp306_);
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp308_ = _tmp307_;
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp309_ = _tmp308_;
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp310_ = gee_collection_get_size ((GeeCollection*) _tmp309_);
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp311_ = _tmp310_;
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (_tmp311_ == 3, "doc2.root.children_nodes[1].children_nodes[1].children_nodes.size == 3");
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp309_);
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp306_);
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp304_);
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp301_);
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp299_);
#line 921 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp296_);
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp312_ = doc2;
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp313_ = gxml_document_get_root ((GXmlDocument*) _tmp312_);
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp314_ = _tmp313_;
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp315_ = _tmp314_;
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp316_ = gxml_node_get_children_nodes (_tmp315_);
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp317_ = _tmp316_;
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp318_ = _tmp317_;
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp319_ = gee_list_get ((GeeList*) _tmp318_, 1);
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp320_ = (GXmlNode*) _tmp319_;
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp321_ = gxml_node_get_children_nodes (_tmp320_);
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp322_ = _tmp321_;
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp323_ = _tmp322_;
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp324_ = gee_list_get ((GeeList*) _tmp323_, 1);
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp325_ = (GXmlNode*) _tmp324_;
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp326_ = gxml_node_get_children_nodes (_tmp325_);
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp327_ = _tmp326_;
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp328_ = _tmp327_;
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp329_ = gee_list_get ((GeeList*) _tmp328_, 1);
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp330_ = (GXmlNode*) _tmp329_;
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp331_ = gxml_node_get_children_nodes (_tmp330_);
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp332_ = _tmp331_;
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp333_ = _tmp332_;
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp334_ = gee_collection_get_size ((GeeCollection*) _tmp333_);
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp335_ = _tmp334_;
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (_tmp335_ == 1, "doc2.root.children_nodes[1].children_nodes[1].children_nodes[1].children_nodes.size == 1");
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp333_);
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp330_);
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp328_);
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp325_);
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp323_);
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp320_);
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp318_);
#line 922 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp315_);
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp336_ = doc2;
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp337_ = gxml_document_get_root ((GXmlDocument*) _tmp336_);
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp338_ = _tmp337_;
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp339_ = _tmp338_;
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp340_ = gxml_node_get_children_nodes (_tmp339_);
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp341_ = _tmp340_;
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp342_ = _tmp341_;
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp343_ = gee_list_get ((GeeList*) _tmp342_, 1);
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp344_ = (GXmlNode*) _tmp343_;
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp345_ = gxml_node_get_children_nodes (_tmp344_);
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp346_ = _tmp345_;
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp347_ = _tmp346_;
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp348_ = gee_list_get ((GeeList*) _tmp347_, 1);
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp349_ = (GXmlNode*) _tmp348_;
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp350_ = gxml_node_get_children_nodes (_tmp349_);
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp351_ = _tmp350_;
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp352_ = _tmp351_;
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp353_ = gee_list_get ((GeeList*) _tmp352_, 1);
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp354_ = (GXmlNode*) _tmp353_;
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp355_ = gxml_node_get_children_nodes (_tmp354_);
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp356_ = _tmp355_;
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp357_ = _tmp356_;
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp358_ = gee_list_get ((GeeList*) _tmp357_, 0);
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp359_ = (GXmlNode*) _tmp358_;
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp359_, GXML_TYPE_TEXT), "doc2.root.children_nodes[1].children_nodes[1].children_nodes[1].children_nodes[0] is GXml.Text");
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp359_);
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp357_);
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp354_);
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp352_);
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp349_);
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp347_);
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp344_);
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp342_);
#line 923 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp339_);
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp360_ = doc2;
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp361_ = gxml_document_get_root ((GXmlDocument*) _tmp360_);
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp362_ = _tmp361_;
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp363_ = _tmp362_;
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp364_ = gxml_node_get_children_nodes (_tmp363_);
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp365_ = _tmp364_;
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp366_ = _tmp365_;
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp367_ = gee_list_get ((GeeList*) _tmp366_, 1);
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp368_ = (GXmlNode*) _tmp367_;
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp369_ = gxml_node_get_children_nodes (_tmp368_);
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp370_ = _tmp369_;
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp371_ = _tmp370_;
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp372_ = gee_list_get ((GeeList*) _tmp371_, 1);
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp373_ = (GXmlNode*) _tmp372_;
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp374_ = gxml_node_get_children_nodes (_tmp373_);
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp375_ = _tmp374_;
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp376_ = _tmp375_;
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp377_ = gee_list_get ((GeeList*) _tmp376_, 1);
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp378_ = (GXmlNode*) _tmp377_;
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp379_ = gxml_node_get_children_nodes (_tmp378_);
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp380_ = _tmp379_;
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp381_ = _tmp380_;
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp382_ = gee_list_get ((GeeList*) _tmp381_, 0);
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp383_ = (GXmlNode*) _tmp382_;
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp384_ = gxml_node_get_value (_tmp383_);
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp385_ = _tmp384_;
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp386_ = _tmp385_;
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (g_strcmp0 (_tmp386_, "SECOND FAKE TEXT") == 0, "doc2.root.children_nodes[1].children_nodes[1].children_nodes[1].children_nodes[0].value == \"SECOND FAKE TEXT\"");
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp386_);
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp383_);
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp381_);
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp378_);
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp376_);
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp373_);
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp371_);
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp368_);
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp366_);
#line 924 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp363_);
#line 881 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (doc2);
#line 881 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (unknown_property);
#line 881 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (doc);
#line 9847 "SerializableObjectModelTest.c"
	}
	goto __finally57;
	__catch57_g_error:
	{
		GError* e = NULL;
		FILE* _tmp387_ = NULL;
		const gchar* _tmp388_ = NULL;
		const gchar* _tmp389_ = NULL;
		gchar* _tmp390_ = NULL;
		gchar* _tmp391_ = NULL;
#line 881 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		e = _inner_error_;
#line 881 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_inner_error_ = NULL;
#line 927 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp387_ = stdout;
#line 927 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp388_ = e->message;
#line 927 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp389_ = string_to_string (_tmp388_);
#line 927 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp390_ = g_strconcat ("Error: ", _tmp389_, NULL);
#line 927 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp391_ = _tmp390_;
#line 927 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		fprintf (_tmp387_, "%s", _tmp391_);
#line 927 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp391_);
#line 928 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_assert_not_reached ();
#line 881 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_error_free0 (e);
#line 9880 "SerializableObjectModelTest.c"
	}
	__finally57:
#line 881 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 881 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 881 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_clear_error (&_inner_error_);
#line 881 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		return;
#line 9891 "SerializableObjectModelTest.c"
	}
}


static void ___lambda25__gtest_func (void) {
#line 879 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	__lambda25_ ();
#line 9899 "SerializableObjectModelTest.c"
}


static void __lambda26_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		UnknownAttribute* unknown_property = NULL;
		UnknownAttribute* _tmp1_ = NULL;
		UnknownAttribute* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		GXmlGDocument* ndoc = NULL;
		GXmlGDocument* _tmp4_ = NULL;
		UnknownAttribute* _tmp5_ = NULL;
		GXmlGDocument* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlNode* _tmp8_ = NULL;
		GXmlGDocument* _tmp9_ = NULL;
		GXmlNode* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GXmlNode* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GeeBidirList* _tmp15_ = NULL;
		gint _tmp16_ = 0;
		gint _tmp17_ = 0;
		gboolean _tmp18_ = FALSE;
#line 934 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp0_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n" \
"<UnknownAttribute ignore=\"true\" ignore2=\"test\"><UnknownNode toigno" \
"re = \"true\" />TEXT</UnknownAttribute>", 0, &_inner_error_);
#line 934 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		doc = _tmp0_;
#line 934 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 9934 "SerializableObjectModelTest.c"
			goto __catch58_g_error;
		}
#line 936 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp1_ = unknown_attribute_new ();
#line 936 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		unknown_property = _tmp1_;
#line 937 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp2_ = unknown_property;
#line 937 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp3_ = doc;
#line 937 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp2_, (GXmlNode*) _tmp3_, &_inner_error_);
#line 937 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 937 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (unknown_property);
#line 937 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (doc);
#line 9953 "SerializableObjectModelTest.c"
			goto __catch58_g_error;
		}
#line 938 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp4_ = gxml_gdocument_new ();
#line 938 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		ndoc = _tmp4_;
#line 939 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = unknown_property;
#line 939 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = ndoc;
#line 939 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp7_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) _tmp5_, (GXmlNode*) _tmp6_, &_inner_error_);
#line 939 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp8_ = _tmp7_;
#line 939 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp8_);
#line 939 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 939 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (ndoc);
#line 939 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (unknown_property);
#line 939 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (doc);
#line 9978 "SerializableObjectModelTest.c"
			goto __catch58_g_error;
		}
#line 940 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp9_ = ndoc;
#line 940 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp10_ = gxml_document_get_root ((GXmlDocument*) _tmp9_);
#line 940 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp11_ = _tmp10_;
#line 940 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp12_ = _tmp11_;
#line 940 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp13_ = gxml_node_get_children_nodes (_tmp12_);
#line 940 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp14_ = _tmp13_;
#line 940 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp15_ = _tmp14_;
#line 940 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp16_ = gee_collection_get_size ((GeeCollection*) _tmp15_);
#line 940 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp17_ = _tmp16_;
#line 940 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp18_ = _tmp17_ != 2;
#line 940 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp15_);
#line 940 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp12_);
#line 940 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (_tmp18_) {
#line 10007 "SerializableObjectModelTest.c"
			FILE* _tmp19_ = NULL;
			GXmlGDocument* _tmp20_ = NULL;
			GXmlNode* _tmp21_ = NULL;
			GXmlNode* _tmp22_ = NULL;
			GXmlNode* _tmp23_ = NULL;
			GeeBidirList* _tmp24_ = NULL;
			GeeBidirList* _tmp25_ = NULL;
			GeeBidirList* _tmp26_ = NULL;
			gint _tmp27_ = 0;
			gint _tmp28_ = 0;
			gchar* _tmp29_ = NULL;
			gchar* _tmp30_ = NULL;
			gchar* _tmp31_ = NULL;
			gchar* _tmp32_ = NULL;
			FILE* _tmp68_ = NULL;
			GXmlGDocument* _tmp69_ = NULL;
			gchar* _tmp70_ = NULL;
			gchar* _tmp71_ = NULL;
			gchar* _tmp72_ = NULL;
			gchar* _tmp73_ = NULL;
#line 941 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp19_ = stdout;
#line 941 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp20_ = doc;
#line 941 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp21_ = gxml_document_get_root ((GXmlDocument*) _tmp20_);
#line 941 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp22_ = _tmp21_;
#line 941 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp23_ = _tmp22_;
#line 941 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp24_ = gxml_node_get_children_nodes (_tmp23_);
#line 941 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp25_ = _tmp24_;
#line 941 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp26_ = _tmp25_;
#line 941 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp27_ = gee_collection_get_size ((GeeCollection*) _tmp26_);
#line 941 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp28_ = _tmp27_;
#line 941 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp29_ = g_strdup_printf ("%i", _tmp28_);
#line 941 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp30_ = _tmp29_;
#line 941 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp31_ = g_strconcat ("ERROR: Root incorrect child node number: found '", _tmp30_, "\n", NULL);
#line 941 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp32_ = _tmp31_;
#line 941 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp19_, "%s", _tmp32_);
#line 941 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp32_);
#line 941 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp30_);
#line 941 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (_tmp26_);
#line 941 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (_tmp23_);
#line 10066 "SerializableObjectModelTest.c"
			{
				GeeBidirList* _rn_list = NULL;
				GXmlGDocument* _tmp33_ = NULL;
				GXmlNode* _tmp34_ = NULL;
				GXmlNode* _tmp35_ = NULL;
				GXmlNode* _tmp36_ = NULL;
				GeeBidirList* _tmp37_ = NULL;
				GeeBidirList* _tmp38_ = NULL;
				GeeBidirList* _tmp39_ = NULL;
				gint _rn_size = 0;
				GeeBidirList* _tmp40_ = NULL;
				gint _tmp41_ = 0;
				gint _tmp42_ = 0;
				gint _rn_index = 0;
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp33_ = ndoc;
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp34_ = gxml_document_get_root ((GXmlDocument*) _tmp33_);
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp35_ = _tmp34_;
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp36_ = _tmp35_;
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp37_ = gxml_node_get_children_nodes (_tmp36_);
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp38_ = _tmp37_;
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp39_ = _tmp38_;
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_g_object_unref0 (_tmp36_);
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_rn_list = _tmp39_;
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp40_ = _rn_list;
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp41_ = gee_collection_get_size ((GeeCollection*) _tmp40_);
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp42_ = _tmp41_;
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_rn_size = _tmp42_;
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_rn_index = -1;
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				while (TRUE) {
#line 10111 "SerializableObjectModelTest.c"
					gint _tmp43_ = 0;
					gint _tmp44_ = 0;
					gint _tmp45_ = 0;
					GXmlNode* rn = NULL;
					GeeBidirList* _tmp46_ = NULL;
					gint _tmp47_ = 0;
					gpointer _tmp48_ = NULL;
					gchar* nv = NULL;
					gchar* _tmp49_ = NULL;
					GXmlNode* _tmp50_ = NULL;
					gchar* _tmp51_ = NULL;
					gchar* _tmp52_ = NULL;
					gchar* _tmp53_ = NULL;
					gboolean _tmp54_ = FALSE;
					FILE* _tmp58_ = NULL;
					GXmlNode* _tmp59_ = NULL;
					gchar* _tmp60_ = NULL;
					gchar* _tmp61_ = NULL;
					gchar* _tmp62_ = NULL;
					const gchar* _tmp63_ = NULL;
					const gchar* _tmp64_ = NULL;
					const gchar* _tmp65_ = NULL;
					gchar* _tmp66_ = NULL;
					gchar* _tmp67_ = NULL;
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp43_ = _rn_index;
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_rn_index = _tmp43_ + 1;
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp44_ = _rn_index;
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp45_ = _rn_size;
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					if (!(_tmp44_ < _tmp45_)) {
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						break;
#line 10148 "SerializableObjectModelTest.c"
					}
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp46_ = _rn_list;
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp47_ = _rn_index;
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp48_ = gee_list_get ((GeeList*) _tmp46_, _tmp47_);
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					rn = (GXmlNode*) _tmp48_;
#line 943 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp49_ = g_strdup ("__NULL__");
#line 943 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					nv = _tmp49_;
#line 944 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp50_ = rn;
#line 944 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp51_ = gxml_node_get_value (_tmp50_);
#line 944 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp52_ = _tmp51_;
#line 944 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp53_ = _tmp52_;
#line 944 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp54_ = _tmp53_ != NULL;
#line 944 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_free0 (_tmp53_);
#line 944 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					if (_tmp54_) {
#line 10176 "SerializableObjectModelTest.c"
						GXmlNode* _tmp55_ = NULL;
						gchar* _tmp56_ = NULL;
						gchar* _tmp57_ = NULL;
#line 945 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp55_ = rn;
#line 945 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp56_ = gxml_node_get_value (_tmp55_);
#line 945 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp57_ = _tmp56_;
#line 945 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_g_free0 (nv);
#line 945 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						nv = _tmp57_;
#line 10190 "SerializableObjectModelTest.c"
					}
#line 946 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp58_ = stdout;
#line 946 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp59_ = rn;
#line 946 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp60_ = gxml_node_get_name (_tmp59_);
#line 946 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp61_ = _tmp60_;
#line 946 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp62_ = _tmp61_;
#line 946 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp63_ = string_to_string (_tmp62_);
#line 946 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp64_ = nv;
#line 946 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp65_ = string_to_string (_tmp64_);
#line 946 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp66_ = g_strconcat ("Node: ", _tmp63_, " / Value: '", _tmp65_, "'\n", NULL);
#line 946 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp67_ = _tmp66_;
#line 946 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					fprintf (_tmp58_, "%s", _tmp67_);
#line 946 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_free0 (_tmp67_);
#line 946 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_free0 (_tmp62_);
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_free0 (nv);
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_object_unref0 (rn);
#line 10222 "SerializableObjectModelTest.c"
				}
#line 942 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_g_object_unref0 (_rn_list);
#line 10226 "SerializableObjectModelTest.c"
			}
#line 948 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp68_ = stdout;
#line 948 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp69_ = ndoc;
#line 948 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp70_ = gxml_gnode_to_string ((GXmlGNode*) _tmp69_);
#line 948 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp71_ = _tmp70_;
#line 948 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp72_ = g_strconcat (_tmp71_, "\n", NULL);
#line 948 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp73_ = _tmp72_;
#line 948 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			fprintf (_tmp68_, "%s", _tmp73_);
#line 948 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp73_);
#line 948 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_free0 (_tmp71_);
#line 949 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			g_assert_not_reached ();
#line 10248 "SerializableObjectModelTest.c"
		}
		{
			GeeBidirList* _n_list = NULL;
			GXmlGDocument* _tmp74_ = NULL;
			GXmlNode* _tmp75_ = NULL;
			GXmlNode* _tmp76_ = NULL;
			GXmlNode* _tmp77_ = NULL;
			GeeBidirList* _tmp78_ = NULL;
			GeeBidirList* _tmp79_ = NULL;
			GeeBidirList* _tmp80_ = NULL;
			gint _n_size = 0;
			GeeBidirList* _tmp81_ = NULL;
			gint _tmp82_ = 0;
			gint _tmp83_ = 0;
			gint _n_index = 0;
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp74_ = ndoc;
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp75_ = gxml_document_get_root ((GXmlDocument*) _tmp74_);
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp76_ = _tmp75_;
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp77_ = _tmp76_;
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp78_ = gxml_node_get_children_nodes (_tmp77_);
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp79_ = _tmp78_;
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp80_ = _tmp79_;
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (_tmp77_);
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_n_list = _tmp80_;
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp81_ = _n_list;
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp82_ = gee_collection_get_size ((GeeCollection*) _tmp81_);
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp83_ = _tmp82_;
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_n_size = _tmp83_;
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_n_index = -1;
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			while (TRUE) {
#line 10294 "SerializableObjectModelTest.c"
				gint _tmp84_ = 0;
				gint _tmp85_ = 0;
				gint _tmp86_ = 0;
				GXmlNode* n = NULL;
				GeeBidirList* _tmp87_ = NULL;
				gint _tmp88_ = 0;
				gpointer _tmp89_ = NULL;
				GXmlNode* _tmp90_ = NULL;
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp84_ = _n_index;
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_n_index = _tmp84_ + 1;
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp85_ = _n_index;
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp86_ = _n_size;
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				if (!(_tmp85_ < _tmp86_)) {
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					break;
#line 10315 "SerializableObjectModelTest.c"
				}
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp87_ = _n_list;
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp88_ = _n_index;
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp89_ = gee_list_get ((GeeList*) _tmp87_, _tmp88_);
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				n = (GXmlNode*) _tmp89_;
#line 952 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp90_ = n;
#line 952 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp90_, GXML_TYPE_TEXT)) {
#line 10329 "SerializableObjectModelTest.c"
					GXmlNode* _tmp91_ = NULL;
					gchar* _tmp92_ = NULL;
					gchar* _tmp93_ = NULL;
					gchar* _tmp94_ = NULL;
					gboolean _tmp95_ = FALSE;
#line 953 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp91_ = n;
#line 953 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp92_ = gxml_node_get_value (_tmp91_);
#line 953 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp93_ = _tmp92_;
#line 953 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp94_ = _tmp93_;
#line 953 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_tmp95_ = g_strcmp0 (_tmp94_, "TEXT") != 0;
#line 953 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					_g_free0 (_tmp94_);
#line 953 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					if (_tmp95_) {
#line 10349 "SerializableObjectModelTest.c"
						FILE* _tmp96_ = NULL;
						GXmlNode* _tmp97_ = NULL;
						gchar* _tmp98_ = NULL;
						gchar* _tmp99_ = NULL;
						gchar* _tmp100_ = NULL;
						const gchar* _tmp101_ = NULL;
						gchar* _tmp102_ = NULL;
						gchar* _tmp103_ = NULL;
#line 954 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp96_ = stdout;
#line 954 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp97_ = n;
#line 954 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp98_ = gxml_node_get_value (_tmp97_);
#line 954 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp99_ = _tmp98_;
#line 954 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp100_ = _tmp99_;
#line 954 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp101_ = string_to_string (_tmp100_);
#line 954 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp102_ = g_strconcat ("ERROR: Unknown Text GElement not set: found '", _tmp101_, "\n", NULL);
#line 954 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_tmp103_ = _tmp102_;
#line 954 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						fprintf (_tmp96_, "%s", _tmp103_);
#line 954 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_g_free0 (_tmp103_);
#line 954 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						_g_free0 (_tmp100_);
#line 955 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
						g_assert_not_reached ();
#line 10382 "SerializableObjectModelTest.c"
					}
				}
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_g_object_unref0 (n);
#line 10387 "SerializableObjectModelTest.c"
			}
#line 951 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (_n_list);
#line 10391 "SerializableObjectModelTest.c"
		}
#line 933 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (ndoc);
#line 933 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (unknown_property);
#line 933 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (doc);
#line 10399 "SerializableObjectModelTest.c"
	}
	goto __finally58;
	__catch58_g_error:
	{
		GError* e = NULL;
		FILE* _tmp104_ = NULL;
		const gchar* _tmp105_ = NULL;
		const gchar* _tmp106_ = NULL;
		gchar* _tmp107_ = NULL;
		gchar* _tmp108_ = NULL;
#line 933 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		e = _inner_error_;
#line 933 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_inner_error_ = NULL;
#line 960 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp104_ = stdout;
#line 960 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp105_ = e->message;
#line 960 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp106_ = string_to_string (_tmp105_);
#line 960 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp107_ = g_strconcat ("ERROR: ", _tmp106_, "\n", NULL);
#line 960 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp108_ = _tmp107_;
#line 960 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		fprintf (_tmp104_, "%s", _tmp108_);
#line 960 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp108_);
#line 961 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_assert_not_reached ();
#line 933 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_error_free0 (e);
#line 10432 "SerializableObjectModelTest.c"
	}
	__finally58:
#line 933 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 933 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 933 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_clear_error (&_inner_error_);
#line 933 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		return;
#line 10443 "SerializableObjectModelTest.c"
	}
}


static void ___lambda26__gtest_func (void) {
#line 931 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	__lambda26_ ();
#line 10451 "SerializableObjectModelTest.c"
}


static void __lambda27_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		GXmlGDocument* _tmp1_ = NULL;
		gchar* _tmp2_ = NULL;
		gchar* _tmp3_ = NULL;
		gchar* _tmp4_ = NULL;
		gchar* _tmp5_ = NULL;
		Package* pkg = NULL;
		Package* _tmp6_ = NULL;
		Package* _tmp7_ = NULL;
		GXmlGDocument* _tmp8_ = NULL;
#line 967 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp0_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n" \
"                            <PACKAGE source=\"Mexico/North\" destiny=\"" \
"Brazil\" Hope=\"2/4.04\">\n" \
"                            <manual document=\"Sales Card\" pages=\"1\"" \
">Selling Card Specification</manual>\n" \
"                            <Computer manufacturer=\"BoxingLAN\" model" \
"=\"J24-EX9\" cores=\"32.5\" ghz=\"1.8\" quantity=\"0.2\" />\n" \
"                            <Box size=\"1\" volume=\"33.15\" units=\"c" \
"m3\" />\n" \
"                            </PACKAGE>", 0, &_inner_error_);
#line 967 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		doc = _tmp0_;
#line 967 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 10475 "SerializableObjectModelTest.c"
			goto __catch59_g_error;
		}
#line 974 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp1_ = doc;
#line 974 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp2_ = gxml_gnode_to_string ((GXmlGNode*) _tmp1_);
#line 974 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp3_ = _tmp2_;
#line 974 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp4_ = g_strconcat ("XML:\n", _tmp3_, NULL);
#line 974 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = _tmp4_;
#line 974 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_test_message ("%s", _tmp5_);
#line 974 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp5_);
#line 974 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp3_);
#line 975 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = package_new ();
#line 975 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		pkg = _tmp6_;
#line 976 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp7_ = pkg;
#line 976 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp8_ = doc;
#line 976 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp7_, (GXmlNode*) _tmp8_, &_inner_error_);
#line 976 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 976 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (pkg);
#line 976 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (doc);
#line 10510 "SerializableObjectModelTest.c"
			goto __catch59_g_error;
		}
#line 966 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (pkg);
#line 966 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (doc);
#line 10517 "SerializableObjectModelTest.c"
	}
	goto __finally59;
	__catch59_g_error:
	{
		GError* e = NULL;
#line 966 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		e = _inner_error_;
#line 966 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_inner_error_ = NULL;
#line 979 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_test_message ("Error thrown for invalid string to guint");
#line 980 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_assert_not_reached ();
#line 966 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_error_free0 (e);
#line 10533 "SerializableObjectModelTest.c"
	}
	__finally59:
#line 966 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 966 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 966 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_clear_error (&_inner_error_);
#line 966 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		return;
#line 10544 "SerializableObjectModelTest.c"
	}
}


static void ___lambda27__gtest_func (void) {
#line 964 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	__lambda27_ ();
#line 10552 "SerializableObjectModelTest.c"
}


static void __lambda28_ (void) {
	GError * _inner_error_ = NULL;
	{
		NameSpace* ns = NULL;
		NameSpace* _tmp0_ = NULL;
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlNode* _tmp3_ = NULL;
		gchar* _tmp4_ = NULL;
		gchar* _tmp5_ = NULL;
		gchar* _tmp6_ = NULL;
		gchar* _tmp7_ = NULL;
		GXmlNode* _tmp8_ = NULL;
		GXmlNode* _tmp9_ = NULL;
		GXmlNode* _tmp10_ = NULL;
		gchar* _tmp11_ = NULL;
		gchar* _tmp12_ = NULL;
#line 986 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp0_ = name_space_new ();
#line 986 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		ns = _tmp0_;
#line 987 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp1_ = gxml_gdocument_new ();
#line 987 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		doc = _tmp1_;
#line 988 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp2_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) ns, (GXmlNode*) doc, &_inner_error_);
#line 988 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp3_ = _tmp2_;
#line 988 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp3_);
#line 988 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 988 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (doc);
#line 988 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (ns);
#line 10594 "SerializableObjectModelTest.c"
			goto __catch60_g_error;
		}
#line 989 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp4_ = gxml_gnode_to_string ((GXmlGNode*) doc);
#line 989 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = _tmp4_;
#line 989 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = g_strconcat ("DOC ROOT: ", _tmp5_, NULL);
#line 989 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp7_ = _tmp6_;
#line 989 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_test_message ("%s", _tmp7_);
#line 989 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp7_);
#line 989 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp5_);
#line 990 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp8_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 990 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp9_ = _tmp8_;
#line 990 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp10_ = _tmp9_;
#line 990 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp11_ = gxml_node_to_string (_tmp10_);
#line 990 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp12_ = _tmp11_;
#line 990 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (g_strcmp0 (_tmp12_, "<namespace xmlns:gxml=\"http://www.gnome.org/GXml\"/>") == 0, "doc.root.to_string () == \"<namespace xmlns:gxml=\\\"http://www.gnome.org/GXml\\\"/>\"");
#line 990 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp12_);
#line 990 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp10_);
#line 985 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (doc);
#line 985 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (ns);
#line 10631 "SerializableObjectModelTest.c"
	}
	goto __finally60;
	__catch60_g_error:
	{
		GError* e = NULL;
#line 985 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		e = _inner_error_;
#line 985 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_inner_error_ = NULL;
#line 995 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_assert_not_reached ();
#line 985 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_error_free0 (e);
#line 10645 "SerializableObjectModelTest.c"
	}
	__finally60:
#line 985 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 985 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 985 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_clear_error (&_inner_error_);
#line 985 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		return;
#line 10656 "SerializableObjectModelTest.c"
	}
}


static void ___lambda28__gtest_func (void) {
#line 984 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	__lambda28_ ();
#line 10664 "SerializableObjectModelTest.c"
}


static void __lambda29_ (void) {
	GError * _inner_error_ = NULL;
	{
		Package* p = NULL;
		Package* _tmp0_ = NULL;
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp1_ = NULL;
		Package* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		Package* _tmp4_ = NULL;
		GeeMap* _tmp5_ = NULL;
		GeeMap* _tmp6_ = NULL;
		GeeMap* _tmp7_ = NULL;
		GXmlAttribute* ukattr = NULL;
		Package* _tmp8_ = NULL;
		GeeMap* _tmp9_ = NULL;
		GeeMap* _tmp10_ = NULL;
		GeeMap* _tmp11_ = NULL;
		gpointer _tmp12_ = NULL;
		GXmlAttribute* _tmp13_ = NULL;
		gchar* _tmp14_ = NULL;
		gchar* _tmp15_ = NULL;
		gchar* _tmp16_ = NULL;
		gchar* _tmp17_ = NULL;
		gchar* _tmp18_ = NULL;
		gchar* _tmp19_ = NULL;
#line 1000 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp0_ = package_new ();
#line 1000 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		p = _tmp0_;
#line 1001 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp1_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n" \
"<PACKAGE source=\"Mexico/North\" destiny=\"Brazil\" Unknown=\"2/4.04\"" \
">\n" \
"<manual document=\"Sales Card\" pages=\"1\">Selling Card Specification" \
"</manual>\n" \
"<Computer manufacturer=\"BoxingLAN\" model=\"J24-EX9\" cores=\"32\" gh" \
"z=\"1.8\"/>\n" \
"<Box size=\"1\" volume=\"33.15\" units=\"cm3\" />\n" \
"UNKNOWN CONTENT\n" \
"</PACKAGE>", 0, &_inner_error_);
#line 1001 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		doc = _tmp1_;
#line 1001 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1001 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (p);
#line 10706 "SerializableObjectModelTest.c"
			goto __catch61_g_error;
		}
#line 1008 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp2_ = p;
#line 1008 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp3_ = doc;
#line 1008 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp2_, (GXmlNode*) _tmp3_, &_inner_error_);
#line 1008 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1008 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (doc);
#line 1008 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (p);
#line 10721 "SerializableObjectModelTest.c"
			goto __catch61_g_error;
		}
#line 1009 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp4_ = p;
#line 1009 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp5_ = gxml_serializable_get_unknown_serializable_properties ((GXmlSerializable*) _tmp4_);
#line 1009 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp6_ = _tmp5_;
#line 1009 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp7_ = _tmp6_;
#line 1009 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (_tmp7_ != NULL, "p.unknown_serializable_properties != null");
#line 1009 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp7_);
#line 1010 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp8_ = p;
#line 1010 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp9_ = gxml_serializable_get_unknown_serializable_properties ((GXmlSerializable*) _tmp8_);
#line 1010 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp10_ = _tmp9_;
#line 1010 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp11_ = _tmp10_;
#line 1010 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp12_ = gee_map_get (_tmp11_, "Unknown");
#line 1010 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp13_ = (GXmlAttribute*) _tmp12_;
#line 1010 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_tmp11_);
#line 1010 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		ukattr = _tmp13_;
#line 1011 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (ukattr != NULL, "ukattr != null");
#line 1012 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp14_ = gxml_node_get_name ((GXmlNode*) ukattr);
#line 1012 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp15_ = _tmp14_;
#line 1012 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp16_ = _tmp15_;
#line 1012 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (g_strcmp0 (_tmp16_, "Unknown") == 0, "ukattr.name == \"Unknown\"");
#line 1012 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp16_);
#line 1013 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp17_ = gxml_node_get_value ((GXmlNode*) ukattr);
#line 1013 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp18_ = _tmp17_;
#line 1013 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp19_ = _tmp18_;
#line 1013 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_vala_assert (g_strcmp0 (_tmp19_, "2/4.04") == 0, "ukattr.value == \"2/4.04\"");
#line 1013 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_free0 (_tmp19_);
#line 999 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (ukattr);
#line 999 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (doc);
#line 999 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (p);
#line 10780 "SerializableObjectModelTest.c"
	}
	goto __finally61;
	__catch61_g_error:
	{
		GError* e = NULL;
#line 999 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		e = _inner_error_;
#line 999 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_inner_error_ = NULL;
#line 1018 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_assert_not_reached ();
#line 999 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_error_free0 (e);
#line 10794 "SerializableObjectModelTest.c"
	}
	__finally61:
#line 999 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 999 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 999 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		g_clear_error (&_inner_error_);
#line 999 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		return;
#line 10805 "SerializableObjectModelTest.c"
	}
}


static void ___lambda29__gtest_func (void) {
#line 998 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	__lambda29_ ();
#line 10813 "SerializableObjectModelTest.c"
}


void serializable_object_model_test_add_tests (void) {
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_test_add_func ("/gxml/serializable/object_model/serialize/simple_object", ___lambda9__gtest_func);
#line 354 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_test_add_func ("/gxml/serializable/object_model/deserialize_simple_object", ___lambda10__gtest_func);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_test_add_func ("/gxml/serializable/object_model/deserialize_object_contents", ___lambda11__gtest_func);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_test_add_func ("/gxml/serializable/object_model/serialize_object_contents", ___lambda12__gtest_func);
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_test_add_func ("/gxml/serializable/object_model/deserialize_serializable_properties", ___lambda13__gtest_func);
#line 478 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_test_add_func ("/gxml/serializable/object_model/serialize_serializable_properties", ___lambda16__gtest_func);
#line 508 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_test_add_func ("/gxml/serializable/object_model/deserialize_array_property", ___lambda17__gtest_func);
#line 536 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_test_add_func ("/gxml/serializable/object_model/serialize_array_property", ___lambda18__gtest_func);
#line 583 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_test_add_func ("/gxml/serializable/object_model/serialize_property_nick", ___lambda19__gtest_func);
#line 712 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_test_add_func ("/gxml/serializable/object_model/override_serialize", ___lambda20__gtest_func);
#line 754 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_test_add_func ("/gxml/serializable/object_model/override_deserialize", ___lambda21__gtest_func);
#line 782 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_test_add_func ("/gxml/serializable/object_model/custome_node_name", ___lambda22__gtest_func);
#line 794 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_test_add_func ("/gxml/serializable/object_model/no_serialize_null_property", ___lambda23__gtest_func);
#line 821 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_test_add_func ("/gxml/serializable/object_model/unknown_property", ___lambda24__gtest_func);
#line 879 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_test_add_func ("/gxml/serializable/object_model/deserialize_unknown_property", ___lambda25__gtest_func);
#line 931 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_test_add_func ("/gxml/serializable/object_model/serialize_unknown_property", ___lambda26__gtest_func);
#line 964 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_test_add_func ("/gxml/serializable/object_model/deserialize_incorrect_uint", ___lambda27__gtest_func);
#line 984 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_test_add_func ("/gxml/serializable/object_model/set_namespace", ___lambda28__gtest_func);
#line 998 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_test_add_func ("/gxml/serializable/object_model/find-unknown_property", ___lambda29__gtest_func);
#line 10856 "SerializableObjectModelTest.c"
}


static void serializable_object_model_test_serialize_manual_check (GXmlGElement* element, Manual* manual) {
	GXmlNode* document = NULL;
	GXmlGElement* _tmp0_ = NULL;
	GXmlNode* _tmp1_ = NULL;
	GXmlNode* _tmp2_ = NULL;
	GXmlNode* _tmp3_ = NULL;
	gchar* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	gchar* _tmp6_ = NULL;
	Manual* _tmp7_ = NULL;
	const gchar* _tmp8_ = NULL;
	const gchar* _tmp9_ = NULL;
	GXmlNode* pages = NULL;
	GXmlGElement* _tmp10_ = NULL;
	GXmlNode* _tmp11_ = NULL;
	GXmlNode* _tmp12_ = NULL;
	GXmlNode* _tmp13_ = NULL;
	gchar* _tmp14_ = NULL;
	gchar* _tmp15_ = NULL;
	gchar* _tmp16_ = NULL;
	gint _tmp17_ = 0;
	Manual* _tmp18_ = NULL;
	gint _tmp19_ = 0;
	gint _tmp20_ = 0;
	gboolean found = FALSE;
	gboolean _tmp42_ = FALSE;
#line 1022 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (element != NULL);
#line 1022 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	g_return_if_fail (manual != NULL);
#line 1023 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp0_ = element;
#line 1023 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp1_ = gxml_element_get_attr ((GXmlElement*) _tmp0_, "document");
#line 1023 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	document = _tmp1_;
#line 1024 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp2_ = document;
#line 1024 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_vala_assert (_tmp2_ != NULL, "document != null");
#line 1025 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp3_ = document;
#line 1025 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp4_ = gxml_node_get_value (_tmp3_);
#line 1025 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp5_ = _tmp4_;
#line 1025 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp6_ = _tmp5_;
#line 1025 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp7_ = manual;
#line 1025 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp8_ = manual_get_document (_tmp7_);
#line 1025 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp9_ = _tmp8_;
#line 1025 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_vala_assert (g_strcmp0 (_tmp6_, _tmp9_) == 0, "document.value == manual.document");
#line 1025 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (_tmp6_);
#line 1026 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp10_ = element;
#line 1026 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp11_ = gxml_element_get_attr ((GXmlElement*) _tmp10_, "pages");
#line 1026 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	pages = _tmp11_;
#line 1027 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp12_ = pages;
#line 1027 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_vala_assert (_tmp12_ != NULL, "pages != null");
#line 1028 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp13_ = pages;
#line 1028 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp14_ = gxml_node_get_value (_tmp13_);
#line 1028 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp15_ = _tmp14_;
#line 1028 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp16_ = _tmp15_;
#line 1028 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp17_ = atoi (_tmp16_);
#line 1028 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp18_ = manual;
#line 1028 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp19_ = manual_get_pages (_tmp18_);
#line 1028 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp20_ = _tmp19_;
#line 1028 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_vala_assert (_tmp17_ == _tmp20_, "int.parse (pages.value) == manual.pages");
#line 1028 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_free0 (_tmp16_);
#line 1029 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	found = FALSE;
#line 10950 "SerializableObjectModelTest.c"
	{
		GeeBidirList* _n_list = NULL;
		GXmlGElement* _tmp21_ = NULL;
		GeeBidirList* _tmp22_ = NULL;
		GeeBidirList* _tmp23_ = NULL;
		gint _n_size = 0;
		GeeBidirList* _tmp24_ = NULL;
		gint _tmp25_ = 0;
		gint _tmp26_ = 0;
		gint _n_index = 0;
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp21_ = element;
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp22_ = gxml_gnode_get_children_nodes ((GXmlGNode*) _tmp21_);
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp23_ = _tmp22_;
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_n_list = _tmp23_;
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp24_ = _n_list;
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp25_ = gee_collection_get_size ((GeeCollection*) _tmp24_);
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_tmp26_ = _tmp25_;
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_n_size = _tmp26_;
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_n_index = -1;
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		while (TRUE) {
#line 10981 "SerializableObjectModelTest.c"
			gint _tmp27_ = 0;
			gint _tmp28_ = 0;
			gint _tmp29_ = 0;
			GXmlNode* n = NULL;
			GeeBidirList* _tmp30_ = NULL;
			gint _tmp31_ = 0;
			gpointer _tmp32_ = NULL;
			GXmlNode* _tmp33_ = NULL;
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp27_ = _n_index;
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_n_index = _tmp27_ + 1;
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp28_ = _n_index;
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp29_ = _n_size;
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			if (!(_tmp28_ < _tmp29_)) {
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				break;
#line 11002 "SerializableObjectModelTest.c"
			}
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp30_ = _n_list;
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp31_ = _n_index;
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp32_ = gee_list_get ((GeeList*) _tmp30_, _tmp31_);
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			n = (GXmlNode*) _tmp32_;
#line 1031 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_tmp33_ = n;
#line 1031 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp33_, GXML_TYPE_TEXT)) {
#line 11016 "SerializableObjectModelTest.c"
				GXmlNode* _tmp34_ = NULL;
				gchar* _tmp35_ = NULL;
				gchar* _tmp36_ = NULL;
				gchar* _tmp37_ = NULL;
				Manual* _tmp38_ = NULL;
				gchar* _tmp39_ = NULL;
				gchar* _tmp40_ = NULL;
				gboolean _tmp41_ = FALSE;
#line 1032 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp34_ = n;
#line 1032 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp35_ = gxml_node_get_value (_tmp34_);
#line 1032 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp36_ = _tmp35_;
#line 1032 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp37_ = _tmp36_;
#line 1032 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp38_ = manual;
#line 1032 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp39_ = manual_get_contents (_tmp38_);
#line 1032 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp40_ = _tmp39_;
#line 1032 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_tmp41_ = g_strcmp0 (_tmp37_, _tmp40_) == 0;
#line 1032 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_g_free0 (_tmp40_);
#line 1032 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				_g_free0 (_tmp37_);
#line 1032 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
				if (_tmp41_) {
#line 1032 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
					found = TRUE;
#line 11049 "SerializableObjectModelTest.c"
				}
			}
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
			_g_object_unref0 (n);
#line 11054 "SerializableObjectModelTest.c"
		}
#line 1030 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
		_g_object_unref0 (_n_list);
#line 11058 "SerializableObjectModelTest.c"
	}
#line 1034 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_tmp42_ = found;
#line 1034 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_vala_assert (_tmp42_, "found");
#line 1022 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (pages);
#line 1022 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	_g_object_unref0 (document);
#line 11068 "SerializableObjectModelTest.c"
}


SerializableObjectModelTest* serializable_object_model_test_construct (GType object_type) {
	SerializableObjectModelTest* self = NULL;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	self = (SerializableObjectModelTest*) gxml_test_construct (object_type);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return self;
#line 11078 "SerializableObjectModelTest.c"
}


SerializableObjectModelTest* serializable_object_model_test_new (void) {
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	return serializable_object_model_test_construct (TYPE_SERIALIZABLE_OBJECT_MODEL_TEST);
#line 11085 "SerializableObjectModelTest.c"
}


static void serializable_object_model_test_class_init (SerializableObjectModelTestClass * klass) {
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableObjectModelTest.vala"
	serializable_object_model_test_parent_class = g_type_class_peek_parent (klass);
#line 11092 "SerializableObjectModelTest.c"
}


static void serializable_object_model_test_instance_init (SerializableObjectModelTest * self) {
}


GType serializable_object_model_test_get_type (void) {
	static volatile gsize serializable_object_model_test_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_object_model_test_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableObjectModelTestClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_object_model_test_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableObjectModelTest), 0, (GInstanceInitFunc) serializable_object_model_test_instance_init, NULL };
		GType serializable_object_model_test_type_id;
		serializable_object_model_test_type_id = g_type_register_static (TYPE_GXML_TEST, "SerializableObjectModelTest", &g_define_type_info, 0);
		g_once_init_leave (&serializable_object_model_test_type_id__volatile, serializable_object_model_test_type_id);
	}
	return serializable_object_model_test_type_id__volatile;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}



