/* SerializableGeeTreeMapTest.c generated by valac 0.34.6, the Vala compiler
 * generated from SerializableGeeTreeMapTest.vala, do not modify */

/* -*- Mode: vala; indent-tabs-mode: nil; c-basic-offset: 2; tab-width: 2 -*- */

#include <glib.h>
#include <glib-object.h>
#include <gxml/gxml.h>
#include <stdlib.h>
#include <string.h>
#include <gee.h>
#include <stdio.h>


#define TYPE_GXML_TEST (gxml_test_get_type ())
#define GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GXML_TEST, GXmlTest))
#define GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GXML_TEST, GXmlTestClass))
#define IS_GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GXML_TEST))
#define IS_GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GXML_TEST))
#define GXML_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GXML_TEST, GXmlTestClass))

typedef struct _GXmlTest GXmlTest;
typedef struct _GXmlTestClass GXmlTestClass;
typedef struct _GXmlTestPrivate GXmlTestPrivate;

#define TYPE_SERIALIZABLE_GEE_TREE_MAP_TEST (serializable_gee_tree_map_test_get_type ())
#define SERIALIZABLE_GEE_TREE_MAP_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SERIALIZABLE_GEE_TREE_MAP_TEST, SerializableGeeTreeMapTest))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SERIALIZABLE_GEE_TREE_MAP_TEST, SerializableGeeTreeMapTestClass))
#define IS_SERIALIZABLE_GEE_TREE_MAP_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SERIALIZABLE_GEE_TREE_MAP_TEST))
#define IS_SERIALIZABLE_GEE_TREE_MAP_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SERIALIZABLE_GEE_TREE_MAP_TEST))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SERIALIZABLE_GEE_TREE_MAP_TEST, SerializableGeeTreeMapTestClass))

typedef struct _SerializableGeeTreeMapTest SerializableGeeTreeMapTest;
typedef struct _SerializableGeeTreeMapTestClass SerializableGeeTreeMapTestClass;
typedef struct _SerializableGeeTreeMapTestPrivate SerializableGeeTreeMapTestPrivate;

#define SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE (serializable_gee_tree_map_test_space_get_type ())
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE, SerializableGeeTreeMapTestSpace))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE, SerializableGeeTreeMapTestSpaceClass))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_IS_SPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_IS_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE, SerializableGeeTreeMapTestSpaceClass))

typedef struct _SerializableGeeTreeMapTestSpace SerializableGeeTreeMapTestSpace;
typedef struct _SerializableGeeTreeMapTestSpaceClass SerializableGeeTreeMapTestSpaceClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))

#define SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE_CONTAINER (serializable_gee_tree_map_test_space_container_get_type ())
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE_CONTAINER, SerializableGeeTreeMapTestSpaceContainer))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE_CONTAINER, SerializableGeeTreeMapTestSpaceContainerClass))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_IS_SPACE_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE_CONTAINER))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_IS_SPACE_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE_CONTAINER))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_CONTAINER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE_CONTAINER, SerializableGeeTreeMapTestSpaceContainerClass))

typedef struct _SerializableGeeTreeMapTestSpaceContainer SerializableGeeTreeMapTestSpaceContainer;
typedef struct _SerializableGeeTreeMapTestSpaceContainerClass SerializableGeeTreeMapTestSpaceContainerClass;

#define SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_TYPE_COLLECTION (serializable_gee_tree_map_test_space_collection_get_type ())
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_TYPE_COLLECTION, SerializableGeeTreeMapTestSpaceCollection))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_TYPE_COLLECTION, SerializableGeeTreeMapTestSpaceCollectionClass))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_TYPE_COLLECTION))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_IS_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_TYPE_COLLECTION))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_TYPE_COLLECTION, SerializableGeeTreeMapTestSpaceCollectionClass))

typedef struct _SerializableGeeTreeMapTestSpaceCollection SerializableGeeTreeMapTestSpaceCollection;
typedef struct _SerializableGeeTreeMapTestSpaceCollectionClass SerializableGeeTreeMapTestSpaceCollectionClass;

#define TYPE_BOOK_STORE (book_store_get_type ())
#define BOOK_STORE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_BOOK_STORE, BookStore))
#define BOOK_STORE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_BOOK_STORE, BookStoreClass))
#define IS_BOOK_STORE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_BOOK_STORE))
#define IS_BOOK_STORE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_BOOK_STORE))
#define BOOK_STORE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_BOOK_STORE, BookStoreClass))

typedef struct _BookStore BookStore;
typedef struct _BookStoreClass BookStoreClass;

#define BOOK_TYPE_ARRAY (book_array_get_type ())
#define BOOK_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOOK_TYPE_ARRAY, BookArray))
#define BOOK_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOOK_TYPE_ARRAY, BookArrayClass))
#define BOOK_IS_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOOK_TYPE_ARRAY))
#define BOOK_IS_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOOK_TYPE_ARRAY))
#define BOOK_ARRAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOOK_TYPE_ARRAY, BookArrayClass))

typedef struct _BookArray BookArray;
typedef struct _BookArrayClass BookArrayClass;

#define TYPE_BOOK (book_get_type ())
#define BOOK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_BOOK, Book))
#define BOOK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_BOOK, BookClass))
#define IS_BOOK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_BOOK))
#define IS_BOOK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_BOOK))
#define BOOK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_BOOK, BookClass))

typedef struct _Book Book;
typedef struct _BookClass BookClass;

#define TYPE_NAME (name_get_type ())
#define NAME(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_NAME, Name))
#define NAME_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_NAME, NameClass))
#define IS_NAME(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_NAME))
#define IS_NAME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_NAME))
#define NAME_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_NAME, NameClass))

typedef struct _Name Name;
typedef struct _NameClass NameClass;

#define TYPE_AUTHORS (authors_get_type ())
#define AUTHORS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_AUTHORS, Authors))
#define AUTHORS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_AUTHORS, AuthorsClass))
#define IS_AUTHORS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_AUTHORS))
#define IS_AUTHORS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_AUTHORS))
#define AUTHORS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_AUTHORS, AuthorsClass))

typedef struct _Authors Authors;
typedef struct _AuthorsClass AuthorsClass;

#define AUTHOR_TYPE_ARRAY (author_array_get_type ())
#define AUTHOR_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), AUTHOR_TYPE_ARRAY, AuthorArray))
#define AUTHOR_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), AUTHOR_TYPE_ARRAY, AuthorArrayClass))
#define AUTHOR_IS_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), AUTHOR_TYPE_ARRAY))
#define AUTHOR_IS_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), AUTHOR_TYPE_ARRAY))
#define AUTHOR_ARRAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), AUTHOR_TYPE_ARRAY, AuthorArrayClass))

typedef struct _AuthorArray AuthorArray;
typedef struct _AuthorArrayClass AuthorArrayClass;

#define RESUME_TYPE_MAP (resume_map_get_type ())
#define RESUME_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RESUME_TYPE_MAP, ResumeMap))
#define RESUME_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), RESUME_TYPE_MAP, ResumeMapClass))
#define RESUME_IS_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RESUME_TYPE_MAP))
#define RESUME_IS_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RESUME_TYPE_MAP))
#define RESUME_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), RESUME_TYPE_MAP, ResumeMapClass))

typedef struct _ResumeMap ResumeMap;
typedef struct _ResumeMapClass ResumeMapClass;

#define TYPE_RESUME (resume_get_type ())
#define RESUME(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_RESUME, Resume))
#define RESUME_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_RESUME, ResumeClass))
#define IS_RESUME(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_RESUME))
#define IS_RESUME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_RESUME))
#define RESUME_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_RESUME, ResumeClass))

typedef struct _Resume Resume;
typedef struct _ResumeClass ResumeClass;

#define SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BIG_BAG (serializable_gee_tree_map_test_big_bag_get_type ())
#define SERIALIZABLE_GEE_TREE_MAP_TEST_BIG_BAG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BIG_BAG, SerializableGeeTreeMapTestBigBag))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_BIG_BAG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BIG_BAG, SerializableGeeTreeMapTestBigBagClass))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_IS_BIG_BAG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BIG_BAG))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_IS_BIG_BAG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BIG_BAG))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_BIG_BAG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BIG_BAG, SerializableGeeTreeMapTestBigBagClass))

typedef struct _SerializableGeeTreeMapTestBigBag SerializableGeeTreeMapTestBigBag;
typedef struct _SerializableGeeTreeMapTestBigBagClass SerializableGeeTreeMapTestBigBagClass;

#define SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_TYPE_TREE_MAP (serializable_gee_tree_map_test_small_bag_tree_map_get_type ())
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_TREE_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_TYPE_TREE_MAP, SerializableGeeTreeMapTestSmallBagTreeMap))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_TREE_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_TYPE_TREE_MAP, SerializableGeeTreeMapTestSmallBagTreeMapClass))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_IS_TREE_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_TYPE_TREE_MAP))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_IS_TREE_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_TYPE_TREE_MAP))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_TREE_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_TYPE_TREE_MAP, SerializableGeeTreeMapTestSmallBagTreeMapClass))

typedef struct _SerializableGeeTreeMapTestSmallBagTreeMap SerializableGeeTreeMapTestSmallBagTreeMap;
typedef struct _SerializableGeeTreeMapTestSmallBagTreeMapClass SerializableGeeTreeMapTestSmallBagTreeMapClass;

#define SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SMALL_BAG (serializable_gee_tree_map_test_small_bag_get_type ())
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SMALL_BAG, SerializableGeeTreeMapTestSmallBag))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SMALL_BAG, SerializableGeeTreeMapTestSmallBagClass))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_IS_SMALL_BAG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SMALL_BAG))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_IS_SMALL_BAG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SMALL_BAG))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SMALL_BAG, SerializableGeeTreeMapTestSmallBagClass))

typedef struct _SerializableGeeTreeMapTestSmallBag SerializableGeeTreeMapTestSmallBag;
typedef struct _SerializableGeeTreeMapTestSmallBagClass SerializableGeeTreeMapTestSmallBagClass;

#define SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_TYPE_TREE_MAP (serializable_gee_tree_map_test_ball_tree_map_get_type ())
#define SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_TREE_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_TYPE_TREE_MAP, SerializableGeeTreeMapTestBallTreeMap))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_TREE_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_TYPE_TREE_MAP, SerializableGeeTreeMapTestBallTreeMapClass))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_IS_TREE_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_TYPE_TREE_MAP))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_IS_TREE_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_TYPE_TREE_MAP))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_TREE_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_TYPE_TREE_MAP, SerializableGeeTreeMapTestBallTreeMapClass))

typedef struct _SerializableGeeTreeMapTestBallTreeMap SerializableGeeTreeMapTestBallTreeMap;
typedef struct _SerializableGeeTreeMapTestBallTreeMapClass SerializableGeeTreeMapTestBallTreeMapClass;

#define SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL (serializable_gee_tree_map_test_ball_get_type ())
#define SERIALIZABLE_GEE_TREE_MAP_TEST_BALL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL, SerializableGeeTreeMapTestBall))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL, SerializableGeeTreeMapTestBallClass))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_IS_BALL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_IS_BALL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL, SerializableGeeTreeMapTestBallClass))

typedef struct _SerializableGeeTreeMapTestBall SerializableGeeTreeMapTestBall;
typedef struct _SerializableGeeTreeMapTestBallClass SerializableGeeTreeMapTestBallClass;

#define SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL_FILL (serializable_gee_tree_map_test_ball_fill_get_type ())
#define SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_FILL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL_FILL, SerializableGeeTreeMapTestBallFill))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_FILL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL_FILL, SerializableGeeTreeMapTestBallFillClass))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_IS_BALL_FILL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL_FILL))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_IS_BALL_FILL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL_FILL))
#define SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_FILL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL_FILL, SerializableGeeTreeMapTestBallFillClass))

typedef struct _SerializableGeeTreeMapTestBallFill SerializableGeeTreeMapTestBallFill;
typedef struct _SerializableGeeTreeMapTestBallFillClass SerializableGeeTreeMapTestBallFillClass;
typedef struct _SerializableGeeTreeMapTestSpacePrivate SerializableGeeTreeMapTestSpacePrivate;
typedef struct _SerializableGeeTreeMapTestSpaceCollectionPrivate SerializableGeeTreeMapTestSpaceCollectionPrivate;
typedef struct _SerializableGeeTreeMapTestSpaceContainerPrivate SerializableGeeTreeMapTestSpaceContainerPrivate;
typedef struct _SerializableGeeTreeMapTestBallFillPrivate SerializableGeeTreeMapTestBallFillPrivate;
typedef struct _SerializableGeeTreeMapTestBallPrivate SerializableGeeTreeMapTestBallPrivate;
typedef struct _SerializableGeeTreeMapTestBallTreeMapPrivate SerializableGeeTreeMapTestBallTreeMapPrivate;
typedef struct _SerializableGeeTreeMapTestSmallBagPrivate SerializableGeeTreeMapTestSmallBagPrivate;
typedef struct _SerializableGeeTreeMapTestSmallBagTreeMapPrivate SerializableGeeTreeMapTestSmallBagTreeMapPrivate;
typedef struct _SerializableGeeTreeMapTestBigBagPrivate SerializableGeeTreeMapTestBigBagPrivate;
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _GXmlTest {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GXmlTestPrivate * priv;
};

struct _GXmlTestClass {
	GTypeClass parent_class;
	void (*finalize) (GXmlTest *self);
};

struct _SerializableGeeTreeMapTest {
	GXmlTest parent_instance;
	SerializableGeeTreeMapTestPrivate * priv;
};

struct _SerializableGeeTreeMapTestClass {
	GXmlTestClass parent_class;
};

struct _SerializableGeeTreeMapTestSpace {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeTreeMapTestSpacePrivate * priv;
};

struct _SerializableGeeTreeMapTestSpaceClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeTreeMapTestSpacePrivate {
	gchar* _name;
};

struct _SerializableGeeTreeMapTestSpaceCollection {
	GXmlSerializableTreeMap parent_instance;
	SerializableGeeTreeMapTestSpaceCollectionPrivate * priv;
};

struct _SerializableGeeTreeMapTestSpaceCollectionClass {
	GXmlSerializableTreeMapClass parent_class;
};

struct _SerializableGeeTreeMapTestSpaceContainer {
	GXmlSerializableContainer parent_instance;
	SerializableGeeTreeMapTestSpaceContainerPrivate * priv;
};

struct _SerializableGeeTreeMapTestSpaceContainerClass {
	GXmlSerializableContainerClass parent_class;
};

struct _SerializableGeeTreeMapTestSpaceContainerPrivate {
	gchar* _owner;
	SerializableGeeTreeMapTestSpaceCollection* _storage;
};

struct _SerializableGeeTreeMapTestBallFill {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeTreeMapTestBallFillPrivate * priv;
};

struct _SerializableGeeTreeMapTestBallFillClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeTreeMapTestBallFillPrivate {
	gchar* _name;
};

struct _SerializableGeeTreeMapTestBall {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeTreeMapTestBallPrivate * priv;
};

struct _SerializableGeeTreeMapTestBallClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeTreeMapTestBallPrivate {
	gchar* _name;
	SerializableGeeTreeMapTestBallFill* _ballfill;
};

struct _SerializableGeeTreeMapTestBallTreeMap {
	GXmlSerializableTreeMap parent_instance;
	SerializableGeeTreeMapTestBallTreeMapPrivate * priv;
};

struct _SerializableGeeTreeMapTestBallTreeMapClass {
	GXmlSerializableTreeMapClass parent_class;
};

struct _SerializableGeeTreeMapTestSmallBag {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeTreeMapTestSmallBagPrivate * priv;
};

struct _SerializableGeeTreeMapTestSmallBagClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeTreeMapTestSmallBagPrivate {
	gchar* _name;
	SerializableGeeTreeMapTestBallTreeMap* _balls;
};

struct _SerializableGeeTreeMapTestSmallBagTreeMap {
	GXmlSerializableTreeMap parent_instance;
	SerializableGeeTreeMapTestSmallBagTreeMapPrivate * priv;
};

struct _SerializableGeeTreeMapTestSmallBagTreeMapClass {
	GXmlSerializableTreeMapClass parent_class;
};

struct _SerializableGeeTreeMapTestBigBag {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeTreeMapTestBigBagPrivate * priv;
};

struct _SerializableGeeTreeMapTestBigBagClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeTreeMapTestBigBagPrivate {
	gchar* _name;
	SerializableGeeTreeMapTestSmallBagTreeMap* _bags;
};


static gpointer serializable_gee_tree_map_test_parent_class = NULL;
static gpointer serializable_gee_tree_map_test_space_parent_class = NULL;
static gpointer serializable_gee_tree_map_test_space_collection_parent_class = NULL;
static GXmlSerializableMapKeyIface* serializable_gee_tree_map_test_space_gxml_serializable_map_key_parent_iface = NULL;
static gpointer serializable_gee_tree_map_test_space_container_parent_class = NULL;
static gpointer serializable_gee_tree_map_test_ball_fill_parent_class = NULL;
static gpointer serializable_gee_tree_map_test_ball_parent_class = NULL;
static gpointer serializable_gee_tree_map_test_ball_tree_map_parent_class = NULL;
static GXmlSerializableMapKeyIface* serializable_gee_tree_map_test_ball_gxml_serializable_map_key_parent_iface = NULL;
static gpointer serializable_gee_tree_map_test_small_bag_parent_class = NULL;
static gpointer serializable_gee_tree_map_test_small_bag_tree_map_parent_class = NULL;
static GXmlSerializableMapKeyIface* serializable_gee_tree_map_test_small_bag_gxml_serializable_map_key_parent_iface = NULL;
static gpointer serializable_gee_tree_map_test_big_bag_parent_class = NULL;

gpointer gxml_test_ref (gpointer instance);
void gxml_test_unref (gpointer instance);
GParamSpec* param_spec_gxml_test (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_gxml_test (GValue* value, gpointer v_object);
void value_take_gxml_test (GValue* value, gpointer v_object);
gpointer value_get_gxml_test (const GValue* value);
GType gxml_test_get_type (void) G_GNUC_CONST;
GType serializable_gee_tree_map_test_get_type (void) G_GNUC_CONST;
enum  {
	SERIALIZABLE_GEE_TREE_MAP_TEST_DUMMY_PROPERTY
};
void serializable_gee_tree_map_test_add_tests (void);
static void __lambda39_ (void);
static GType serializable_gee_tree_map_test_space_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeTreeMapTestSpace* serializable_gee_tree_map_test_space_new_named (const gchar* name);
static SerializableGeeTreeMapTestSpace* serializable_gee_tree_map_test_space_construct_named (GType object_type, const gchar* name);
static const gchar* serializable_gee_tree_map_test_space_get_name (SerializableGeeTreeMapTestSpace* self);
static void ___lambda39__gtest_func (void);
static void __lambda40_ (void);
static void ___lambda40__gtest_func (void);
static void __lambda41_ (void);
static void ___lambda41__gtest_func (void);
static void __lambda42_ (void);
static GType serializable_gee_tree_map_test_space_container_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeTreeMapTestSpaceContainer* serializable_gee_tree_map_test_space_container_new (void);
static SerializableGeeTreeMapTestSpaceContainer* serializable_gee_tree_map_test_space_container_construct (GType object_type);
static const gchar* serializable_gee_tree_map_test_space_container_get_owner (SerializableGeeTreeMapTestSpaceContainer* self);
GType serializable_gee_tree_map_test_space_collection_get_type (void) G_GNUC_CONST;
static SerializableGeeTreeMapTestSpaceCollection* serializable_gee_tree_map_test_space_container_get_storage (SerializableGeeTreeMapTestSpaceContainer* self);
static void ___lambda42__gtest_func (void);
static void __lambda43_ (void);
static SerializableGeeTreeMapTestSpaceCollection* serializable_gee_tree_map_test_space_collection_new (void);
static SerializableGeeTreeMapTestSpaceCollection* serializable_gee_tree_map_test_space_collection_construct (GType object_type);
static void serializable_gee_tree_map_test_space_container_set_storage (SerializableGeeTreeMapTestSpaceContainer* self, SerializableGeeTreeMapTestSpaceCollection* value);
static void ___lambda43__gtest_func (void);
static void __lambda44_ (void);
GType book_store_get_type (void) G_GNUC_CONST;
BookStore* book_store_new (void);
BookStore* book_store_construct (GType object_type);
const gchar* book_store_get_name (BookStore* self);
GType book_array_get_type (void) G_GNUC_CONST;
BookArray* book_store_get_books (BookStore* self);
GType book_get_type (void) G_GNUC_CONST;
GType name_get_type (void) G_GNUC_CONST;
Name* book_get_name (Book* self);
gchar* name_get_name (Name* self);
const gchar* book_get_year (Book* self);
GType authors_get_type (void) G_GNUC_CONST;
Authors* book_get_authors (Book* self);
GType author_array_get_type (void) G_GNUC_CONST;
AuthorArray* authors_get_array (Authors* self);
GType resume_map_get_type (void) G_GNUC_CONST;
ResumeMap* book_get_resumes (Book* self);
GType resume_get_type (void) G_GNUC_CONST;
const gchar* resume_get_chapter (Resume* self);
const gchar* resume_get_text (Resume* self);
static void ___lambda44__gtest_func (void);
static void __lambda45_ (void);
static GType serializable_gee_tree_map_test_big_bag_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeTreeMapTestBigBag* serializable_gee_tree_map_test_big_bag_new (void);
static SerializableGeeTreeMapTestBigBag* serializable_gee_tree_map_test_big_bag_construct (GType object_type);
GType serializable_gee_tree_map_test_small_bag_tree_map_get_type (void) G_GNUC_CONST;
static SerializableGeeTreeMapTestSmallBagTreeMap* serializable_gee_tree_map_test_big_bag_get_bags (SerializableGeeTreeMapTestBigBag* self);
static GType serializable_gee_tree_map_test_small_bag_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeTreeMapTestSmallBag* serializable_gee_tree_map_test_small_bag_new (void);
static SerializableGeeTreeMapTestSmallBag* serializable_gee_tree_map_test_small_bag_construct (GType object_type);
static void serializable_gee_tree_map_test_small_bag_set_name (SerializableGeeTreeMapTestSmallBag* self, const gchar* value);
GType serializable_gee_tree_map_test_ball_tree_map_get_type (void) G_GNUC_CONST;
static SerializableGeeTreeMapTestBallTreeMap* serializable_gee_tree_map_test_small_bag_get_balls (SerializableGeeTreeMapTestSmallBag* self);
static GType serializable_gee_tree_map_test_ball_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeTreeMapTestBall* serializable_gee_tree_map_test_ball_new (void);
static SerializableGeeTreeMapTestBall* serializable_gee_tree_map_test_ball_construct (GType object_type);
static void serializable_gee_tree_map_test_ball_set_name (SerializableGeeTreeMapTestBall* self, const gchar* value);
static SerializableGeeTreeMapTestBallFill* serializable_gee_tree_map_test_ball_fill_new (void);
static SerializableGeeTreeMapTestBallFill* serializable_gee_tree_map_test_ball_fill_construct (GType object_type);
static GType serializable_gee_tree_map_test_ball_fill_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static void serializable_gee_tree_map_test_ball_set_ballfill (SerializableGeeTreeMapTestBall* self, SerializableGeeTreeMapTestBallFill* value);
static SerializableGeeTreeMapTestBallFill* serializable_gee_tree_map_test_ball_get_ballfill (SerializableGeeTreeMapTestBall* self);
static void serializable_gee_tree_map_test_ball_fill_set_text (SerializableGeeTreeMapTestBallFill* self, const gchar* txt);
static const gchar* serializable_gee_tree_map_test_ball_get_name (SerializableGeeTreeMapTestBall* self);
static const gchar* serializable_gee_tree_map_test_small_bag_get_name (SerializableGeeTreeMapTestSmallBag* self);
static void ___lambda45__gtest_func (void);
SerializableGeeTreeMapTest* serializable_gee_tree_map_test_new (void);
SerializableGeeTreeMapTest* serializable_gee_tree_map_test_construct (GType object_type);
GXmlTest* gxml_test_new (void);
GXmlTest* gxml_test_construct (GType object_type);
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE, SerializableGeeTreeMapTestSpacePrivate))
enum  {
	SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_NAME
};
static gchar* serializable_gee_tree_map_test_space_real_get_map_key (GXmlSerializableMapKey* base);
static void serializable_gee_tree_map_test_space_set_name (SerializableGeeTreeMapTestSpace* self, const gchar* value);
static gchar* serializable_gee_tree_map_test_space_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_tree_map_test_space_real_to_string (GXmlSerializableObjectModel* base);
static SerializableGeeTreeMapTestSpace* serializable_gee_tree_map_test_space_new (void);
static SerializableGeeTreeMapTestSpace* serializable_gee_tree_map_test_space_construct (GType object_type);
enum  {
	SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_COLLECTION_DUMMY_PROPERTY
};
static void serializable_gee_tree_map_test_space_finalize (GObject* obj);
static void _vala_serializable_gee_tree_map_test_space_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_tree_map_test_space_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_CONTAINER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE_CONTAINER, SerializableGeeTreeMapTestSpaceContainerPrivate))
enum  {
	SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_CONTAINER_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_CONTAINER_OWNER,
	SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_CONTAINER_STORAGE
};
static gchar* serializable_gee_tree_map_test_space_container_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_tree_map_test_space_container_real_to_string (GXmlSerializableObjectModel* base);
static void serializable_gee_tree_map_test_space_container_real_init_containers (GXmlSerializableContainer* base);
static void serializable_gee_tree_map_test_space_container_set_owner (SerializableGeeTreeMapTestSpaceContainer* self, const gchar* value);
static void serializable_gee_tree_map_test_space_container_finalize (GObject* obj);
static void _vala_serializable_gee_tree_map_test_space_container_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_tree_map_test_space_container_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_FILL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL_FILL, SerializableGeeTreeMapTestBallFillPrivate))
enum  {
	SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_FILL_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_FILL_NAME
};
static gchar* serializable_gee_tree_map_test_ball_fill_real_to_string (GXmlSerializableObjectModel* base);
static const gchar* serializable_gee_tree_map_test_ball_fill_get_name (SerializableGeeTreeMapTestBallFill* self);
static gchar* serializable_gee_tree_map_test_ball_fill_real_node_name (GXmlSerializableObjectModel* base);
static gboolean serializable_gee_tree_map_test_ball_fill_real_serialize_use_xml_node_value (GXmlSerializableObjectModel* base);
static void serializable_gee_tree_map_test_ball_fill_set_name (SerializableGeeTreeMapTestBallFill* self, const gchar* value);
static void serializable_gee_tree_map_test_ball_fill_finalize (GObject* obj);
static void _vala_serializable_gee_tree_map_test_ball_fill_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_tree_map_test_ball_fill_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL, SerializableGeeTreeMapTestBallPrivate))
enum  {
	SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_NAME,
	SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_BALLFILL
};
static gchar* serializable_gee_tree_map_test_ball_real_to_string (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_tree_map_test_ball_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_tree_map_test_ball_real_get_map_key (GXmlSerializableMapKey* base);
enum  {
	SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_TREE_MAP_DUMMY_PROPERTY
};
static gboolean serializable_gee_tree_map_test_ball_tree_map_real_deserialize_proceed (GXmlSerializableTreeMap* base);
static SerializableGeeTreeMapTestBallTreeMap* serializable_gee_tree_map_test_ball_tree_map_new (void);
static SerializableGeeTreeMapTestBallTreeMap* serializable_gee_tree_map_test_ball_tree_map_construct (GType object_type);
static void serializable_gee_tree_map_test_ball_finalize (GObject* obj);
static void _vala_serializable_gee_tree_map_test_ball_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_tree_map_test_ball_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SMALL_BAG, SerializableGeeTreeMapTestSmallBagPrivate))
enum  {
	SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_NAME,
	SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_BALLS
};
static gchar* serializable_gee_tree_map_test_small_bag_real_to_string (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_tree_map_test_small_bag_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_tree_map_test_small_bag_real_get_map_key (GXmlSerializableMapKey* base);
static void serializable_gee_tree_map_test_small_bag_set_balls (SerializableGeeTreeMapTestSmallBag* self, SerializableGeeTreeMapTestBallTreeMap* value);
enum  {
	SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_TREE_MAP_DUMMY_PROPERTY
};
static gboolean serializable_gee_tree_map_test_small_bag_tree_map_real_deserialize_proceed (GXmlSerializableTreeMap* base);
static SerializableGeeTreeMapTestSmallBagTreeMap* serializable_gee_tree_map_test_small_bag_tree_map_new (void);
static SerializableGeeTreeMapTestSmallBagTreeMap* serializable_gee_tree_map_test_small_bag_tree_map_construct (GType object_type);
static void serializable_gee_tree_map_test_small_bag_finalize (GObject* obj);
static void _vala_serializable_gee_tree_map_test_small_bag_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_tree_map_test_small_bag_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_TREE_MAP_TEST_BIG_BAG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BIG_BAG, SerializableGeeTreeMapTestBigBagPrivate))
enum  {
	SERIALIZABLE_GEE_TREE_MAP_TEST_BIG_BAG_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_TREE_MAP_TEST_BIG_BAG_NAME,
	SERIALIZABLE_GEE_TREE_MAP_TEST_BIG_BAG_BAGS
};
static gchar* serializable_gee_tree_map_test_big_bag_real_to_string (GXmlSerializableObjectModel* base);
static const gchar* serializable_gee_tree_map_test_big_bag_get_name (SerializableGeeTreeMapTestBigBag* self);
static gchar* serializable_gee_tree_map_test_big_bag_real_node_name (GXmlSerializableObjectModel* base);
static void serializable_gee_tree_map_test_big_bag_set_name (SerializableGeeTreeMapTestBigBag* self, const gchar* value);
static void serializable_gee_tree_map_test_big_bag_set_bags (SerializableGeeTreeMapTestBigBag* self, SerializableGeeTreeMapTestSmallBagTreeMap* value);
static void serializable_gee_tree_map_test_big_bag_finalize (GObject* obj);
static void _vala_serializable_gee_tree_map_test_big_bag_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_tree_map_test_big_bag_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);


static void __lambda39_ (void) {
	GXmlSerializableTreeMap* c = NULL;
	GXmlSerializableTreeMap* _tmp0_ = NULL;
	SerializableGeeTreeMapTestSpace* o1 = NULL;
	SerializableGeeTreeMapTestSpace* _tmp1_ = NULL;
	SerializableGeeTreeMapTestSpace* o2 = NULL;
	SerializableGeeTreeMapTestSpace* _tmp2_ = NULL;
	GXmlSerializableTreeMap* _tmp3_ = NULL;
	SerializableGeeTreeMapTestSpace* _tmp4_ = NULL;
	const gchar* _tmp5_ = NULL;
	const gchar* _tmp6_ = NULL;
	SerializableGeeTreeMapTestSpace* _tmp7_ = NULL;
	GXmlSerializableTreeMap* _tmp8_ = NULL;
	SerializableGeeTreeMapTestSpace* _tmp9_ = NULL;
	const gchar* _tmp10_ = NULL;
	const gchar* _tmp11_ = NULL;
	SerializableGeeTreeMapTestSpace* _tmp12_ = NULL;
	gboolean found1 = FALSE;
	gboolean found2 = FALSE;
	gboolean _tmp29_ = FALSE;
	gboolean _tmp31_ = FALSE;
	gboolean _tmp33_ = FALSE;
	gboolean _tmp58_ = FALSE;
	gboolean _tmp60_ = FALSE;
#line 101 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = gxml_serializable_tree_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 101 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	c = _tmp0_;
#line 102 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = serializable_gee_tree_map_test_space_new_named ("Big");
#line 102 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	o1 = _tmp1_;
#line 103 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp2_ = serializable_gee_tree_map_test_space_new_named ("Small");
#line 103 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	o2 = _tmp2_;
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp3_ = c;
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp4_ = o1;
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp5_ = serializable_gee_tree_map_test_space_get_name (_tmp4_);
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp6_ = _tmp5_;
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp7_ = o1;
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	gee_abstract_map_set ((GeeAbstractMap*) _tmp3_, _tmp6_, _tmp7_);
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp8_ = c;
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp9_ = o2;
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp10_ = serializable_gee_tree_map_test_space_get_name (_tmp9_);
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp11_ = _tmp10_;
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp12_ = o2;
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	gee_abstract_map_set ((GeeAbstractMap*) _tmp8_, _tmp11_, _tmp12_);
#line 106 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	found1 = FALSE;
#line 107 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	found2 = FALSE;
#line 614 "SerializableGeeTreeMapTest.c"
	{
		GeeIterator* _o_it = NULL;
		GXmlSerializableTreeMap* _tmp13_ = NULL;
		GeeCollection* _tmp14_ = NULL;
		GeeCollection* _tmp15_ = NULL;
		GeeCollection* _tmp16_ = NULL;
		GeeIterator* _tmp17_ = NULL;
		GeeIterator* _tmp18_ = NULL;
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp13_ = c;
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp14_ = gee_abstract_map_get_values ((GeeAbstractMap*) _tmp13_);
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp15_ = _tmp14_;
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp16_ = _tmp15_;
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp17_ = gee_iterable_iterator ((GeeIterable*) _tmp16_);
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp18_ = _tmp17_;
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp16_);
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_o_it = _tmp18_;
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		while (TRUE) {
#line 641 "SerializableGeeTreeMapTest.c"
			GeeIterator* _tmp19_ = NULL;
			gboolean _tmp20_ = FALSE;
			SerializableGeeTreeMapTestSpace* o = NULL;
			GeeIterator* _tmp21_ = NULL;
			gpointer _tmp22_ = NULL;
			SerializableGeeTreeMapTestSpace* _tmp23_ = NULL;
			const gchar* _tmp24_ = NULL;
			const gchar* _tmp25_ = NULL;
			SerializableGeeTreeMapTestSpace* _tmp26_ = NULL;
			const gchar* _tmp27_ = NULL;
			const gchar* _tmp28_ = NULL;
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp19_ = _o_it;
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp20_ = gee_iterator_next (_tmp19_);
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			if (!_tmp20_) {
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				break;
#line 661 "SerializableGeeTreeMapTest.c"
			}
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp21_ = _o_it;
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp22_ = gee_iterator_get (_tmp21_);
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			o = (SerializableGeeTreeMapTestSpace*) _tmp22_;
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp23_ = o;
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp24_ = serializable_gee_tree_map_test_space_get_name (_tmp23_);
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp25_ = _tmp24_;
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			if (g_strcmp0 (_tmp25_, "Big") == 0) {
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				found1 = TRUE;
#line 679 "SerializableGeeTreeMapTest.c"
			}
#line 110 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp26_ = o;
#line 110 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp27_ = serializable_gee_tree_map_test_space_get_name (_tmp26_);
#line 110 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp28_ = _tmp27_;
#line 110 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			if (g_strcmp0 (_tmp28_, "Small") == 0) {
#line 110 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				found2 = TRUE;
#line 691 "SerializableGeeTreeMapTest.c"
			}
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (o);
#line 695 "SerializableGeeTreeMapTest.c"
		}
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_o_it);
#line 699 "SerializableGeeTreeMapTest.c"
	}
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp29_ = found1;
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	if (!_tmp29_) {
#line 705 "SerializableGeeTreeMapTest.c"
		FILE* _tmp30_ = NULL;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp30_ = stdout;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		fprintf (_tmp30_, "%s", "Big is not found\n");
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_assert_not_reached ();
#line 713 "SerializableGeeTreeMapTest.c"
	}
#line 116 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp31_ = found2;
#line 116 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	if (!_tmp31_) {
#line 719 "SerializableGeeTreeMapTest.c"
		FILE* _tmp32_ = NULL;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp32_ = stdout;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		fprintf (_tmp32_, "%s", "Small is not found\n");
#line 118 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_assert_not_reached ();
#line 727 "SerializableGeeTreeMapTest.c"
	}
#line 120 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	found2 = FALSE;
#line 120 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp33_ = found2;
#line 120 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	found1 = _tmp33_;
#line 735 "SerializableGeeTreeMapTest.c"
	{
		GeeIterator* _k_it = NULL;
		GXmlSerializableTreeMap* _tmp34_ = NULL;
		GeeSet* _tmp35_ = NULL;
		GeeSet* _tmp36_ = NULL;
		GeeSet* _tmp37_ = NULL;
		GeeIterator* _tmp38_ = NULL;
		GeeIterator* _tmp39_ = NULL;
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp34_ = c;
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp35_ = gee_abstract_map_get_keys ((GeeAbstractMap*) _tmp34_);
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp36_ = _tmp35_;
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp37_ = _tmp36_;
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp38_ = gee_iterable_iterator ((GeeIterable*) _tmp37_);
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp39_ = _tmp38_;
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp37_);
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_k_it = _tmp39_;
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		while (TRUE) {
#line 762 "SerializableGeeTreeMapTest.c"
			GeeIterator* _tmp40_ = NULL;
			gboolean _tmp41_ = FALSE;
			gchar* k = NULL;
			GeeIterator* _tmp42_ = NULL;
			gpointer _tmp43_ = NULL;
			GXmlSerializableTreeMap* _tmp44_ = NULL;
			const gchar* _tmp45_ = NULL;
			gpointer _tmp46_ = NULL;
			SerializableGeeTreeMapTestSpace* _tmp47_ = NULL;
			const gchar* _tmp48_ = NULL;
			const gchar* _tmp49_ = NULL;
			gboolean _tmp50_ = FALSE;
			GXmlSerializableTreeMap* _tmp51_ = NULL;
			const gchar* _tmp52_ = NULL;
			gpointer _tmp53_ = NULL;
			SerializableGeeTreeMapTestSpace* _tmp54_ = NULL;
			const gchar* _tmp55_ = NULL;
			const gchar* _tmp56_ = NULL;
			gboolean _tmp57_ = FALSE;
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp40_ = _k_it;
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp41_ = gee_iterator_next (_tmp40_);
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			if (!_tmp41_) {
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				break;
#line 790 "SerializableGeeTreeMapTest.c"
			}
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp42_ = _k_it;
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp43_ = gee_iterator_get (_tmp42_);
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			k = (gchar*) _tmp43_;
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp44_ = c;
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp45_ = k;
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp46_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp44_, _tmp45_);
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp47_ = (SerializableGeeTreeMapTestSpace*) _tmp46_;
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp48_ = serializable_gee_tree_map_test_space_get_name (_tmp47_);
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp49_ = _tmp48_;
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp50_ = g_strcmp0 (_tmp49_, "Big") == 0;
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (_tmp47_);
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			if (_tmp50_) {
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				found1 = TRUE;
#line 818 "SerializableGeeTreeMapTest.c"
			}
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp51_ = c;
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp52_ = k;
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp53_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp51_, _tmp52_);
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp54_ = (SerializableGeeTreeMapTestSpace*) _tmp53_;
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp55_ = serializable_gee_tree_map_test_space_get_name (_tmp54_);
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp56_ = _tmp55_;
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp57_ = g_strcmp0 (_tmp56_, "Small") == 0;
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (_tmp54_);
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			if (_tmp57_) {
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				found2 = TRUE;
#line 840 "SerializableGeeTreeMapTest.c"
			}
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (k);
#line 844 "SerializableGeeTreeMapTest.c"
		}
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_k_it);
#line 848 "SerializableGeeTreeMapTest.c"
	}
#line 125 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp58_ = found1;
#line 125 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	if (!_tmp58_) {
#line 854 "SerializableGeeTreeMapTest.c"
		FILE* _tmp59_ = NULL;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp59_ = stdout;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		fprintf (_tmp59_, "%s", "Big key value is not found\n");
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_assert_not_reached ();
#line 862 "SerializableGeeTreeMapTest.c"
	}
#line 129 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp60_ = found2;
#line 129 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	if (!_tmp60_) {
#line 868 "SerializableGeeTreeMapTest.c"
		FILE* _tmp61_ = NULL;
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp61_ = stdout;
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		fprintf (_tmp61_, "%s", "Small key value is not found\n");
#line 131 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_assert_not_reached ();
#line 876 "SerializableGeeTreeMapTest.c"
	}
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_object_unref0 (o2);
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_object_unref0 (o1);
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_object_unref0 (c);
#line 884 "SerializableGeeTreeMapTest.c"
}


static void ___lambda39__gtest_func (void) {
#line 99 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	__lambda39_ ();
#line 891 "SerializableGeeTreeMapTest.c"
}


static const gchar* string_to_string (const gchar* self) {
	const gchar* result = NULL;
#line 1412 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1413 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	result = self;
#line 1413 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	return result;
#line 903 "SerializableGeeTreeMapTest.c"
}


static void __lambda40_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlSerializableTreeMap* c = NULL;
		GXmlSerializableTreeMap* _tmp0_ = NULL;
		SerializableGeeTreeMapTestSpace* o1 = NULL;
		SerializableGeeTreeMapTestSpace* _tmp1_ = NULL;
		SerializableGeeTreeMapTestSpace* o2 = NULL;
		SerializableGeeTreeMapTestSpace* _tmp2_ = NULL;
		GXmlSerializableTreeMap* _tmp3_ = NULL;
		SerializableGeeTreeMapTestSpace* _tmp4_ = NULL;
		const gchar* _tmp5_ = NULL;
		const gchar* _tmp6_ = NULL;
		SerializableGeeTreeMapTestSpace* _tmp7_ = NULL;
		GXmlSerializableTreeMap* _tmp8_ = NULL;
		SerializableGeeTreeMapTestSpace* _tmp9_ = NULL;
		const gchar* _tmp10_ = NULL;
		const gchar* _tmp11_ = NULL;
		SerializableGeeTreeMapTestSpace* _tmp12_ = NULL;
		GXmlTDocument* doc = NULL;
		GXmlTDocument* _tmp13_ = NULL;
		GXmlNode* root = NULL;
		GXmlTDocument* _tmp14_ = NULL;
		GXmlNode* _tmp15_ = NULL;
		GXmlTDocument* _tmp16_ = NULL;
		GeeBidirList* _tmp17_ = NULL;
		GeeBidirList* _tmp18_ = NULL;
		GeeBidirList* _tmp19_ = NULL;
		GXmlNode* _tmp20_ = NULL;
		GXmlSerializableTreeMap* _tmp21_ = NULL;
		GXmlNode* _tmp22_ = NULL;
		GXmlNode* _tmp23_ = NULL;
		GXmlNode* _tmp24_ = NULL;
		GXmlNode* _tmp25_ = NULL;
		GeeBidirList* _tmp26_ = NULL;
		GeeBidirList* _tmp27_ = NULL;
		GeeBidirList* _tmp28_ = NULL;
		gint _tmp29_ = 0;
		gint _tmp30_ = 0;
		gboolean found1 = FALSE;
		gboolean found2 = FALSE;
		gboolean _tmp66_ = FALSE;
		gboolean _tmp73_ = FALSE;
#line 137 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp0_ = gxml_serializable_tree_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 137 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		c = _tmp0_;
#line 138 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp1_ = serializable_gee_tree_map_test_space_new_named ("Big");
#line 138 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		o1 = _tmp1_;
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp2_ = serializable_gee_tree_map_test_space_new_named ("Small");
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		o2 = _tmp2_;
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp3_ = c;
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp4_ = o1;
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp5_ = serializable_gee_tree_map_test_space_get_name (_tmp4_);
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp6_ = _tmp5_;
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp7_ = o1;
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		gee_abstract_map_set ((GeeAbstractMap*) _tmp3_, _tmp6_, _tmp7_);
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp8_ = c;
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp9_ = o2;
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp10_ = serializable_gee_tree_map_test_space_get_name (_tmp9_);
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp11_ = _tmp10_;
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp12_ = o2;
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		gee_abstract_map_set ((GeeAbstractMap*) _tmp8_, _tmp11_, _tmp12_);
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp13_ = gxml_tdocument_new ();
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		doc = _tmp13_;
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp14_ = doc;
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp15_ = gxml_document_create_element ((GXmlDocument*) _tmp14_, "root", &_inner_error_);
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		root = _tmp15_;
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (doc);
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (o2);
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (o1);
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (c);
#line 1006 "SerializableGeeTreeMapTest.c"
			goto __catch71_g_error;
		}
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp16_ = doc;
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp17_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp16_);
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp18_ = _tmp17_;
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp19_ = _tmp18_;
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp20_ = root;
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		gee_collection_add ((GeeCollection*) _tmp19_, _tmp20_);
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp19_);
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp21_ = c;
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp22_ = root;
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp23_ = gxml_serializable_tree_map_serialize (_tmp21_, _tmp22_, &_inner_error_);
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp24_ = _tmp23_;
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp24_);
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (root);
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (doc);
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (o2);
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (o1);
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (c);
#line 1045 "SerializableGeeTreeMapTest.c"
			goto __catch71_g_error;
		}
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp25_ = root;
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp26_ = gxml_node_get_children_nodes (_tmp25_);
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp27_ = _tmp26_;
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp28_ = _tmp27_;
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp29_ = gee_collection_get_size ((GeeCollection*) _tmp28_);
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp30_ = _tmp29_;
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp30_ == 2, "root.children_nodes.size == 2");
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp28_);
#line 147 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		found1 = FALSE;
#line 148 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		found2 = FALSE;
#line 1068 "SerializableGeeTreeMapTest.c"
		{
			GeeBidirList* _n_list = NULL;
			GXmlNode* _tmp31_ = NULL;
			GeeBidirList* _tmp32_ = NULL;
			GeeBidirList* _tmp33_ = NULL;
			gint _n_size = 0;
			GeeBidirList* _tmp34_ = NULL;
			gint _tmp35_ = 0;
			gint _tmp36_ = 0;
			gint _n_index = 0;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp31_ = root;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp32_ = gxml_node_get_children_nodes (_tmp31_);
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp33_ = _tmp32_;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_n_list = _tmp33_;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp34_ = _n_list;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp35_ = gee_collection_get_size ((GeeCollection*) _tmp34_);
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp36_ = _tmp35_;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_n_size = _tmp36_;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_n_index = -1;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			while (TRUE) {
#line 1099 "SerializableGeeTreeMapTest.c"
				gint _tmp37_ = 0;
				gint _tmp38_ = 0;
				gint _tmp39_ = 0;
				GXmlNode* n = NULL;
				GeeBidirList* _tmp40_ = NULL;
				gint _tmp41_ = 0;
				gpointer _tmp42_ = NULL;
				gboolean _tmp43_ = FALSE;
				GXmlNode* _tmp44_ = NULL;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp37_ = _n_index;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_n_index = _tmp37_ + 1;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp38_ = _n_index;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp39_ = _n_size;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				if (!(_tmp38_ < _tmp39_)) {
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					break;
#line 1121 "SerializableGeeTreeMapTest.c"
				}
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp40_ = _n_list;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp41_ = _n_index;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp42_ = gee_list_get ((GeeList*) _tmp40_, _tmp41_);
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				n = (GXmlNode*) _tmp42_;
#line 150 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp44_ = n;
#line 150 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp44_, GXML_TYPE_ELEMENT)) {
#line 1135 "SerializableGeeTreeMapTest.c"
					GXmlNode* _tmp45_ = NULL;
					gchar* _tmp46_ = NULL;
					gchar* _tmp47_ = NULL;
					gchar* _tmp48_ = NULL;
#line 150 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp45_ = n;
#line 150 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp46_ = gxml_node_get_name (_tmp45_);
#line 150 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp47_ = _tmp46_;
#line 150 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp48_ = _tmp47_;
#line 150 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp43_ = g_strcmp0 (_tmp48_, "space") == 0;
#line 150 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_g_free0 (_tmp48_);
#line 1152 "SerializableGeeTreeMapTest.c"
				} else {
#line 150 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp43_ = FALSE;
#line 1156 "SerializableGeeTreeMapTest.c"
				}
#line 150 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				if (_tmp43_) {
#line 1160 "SerializableGeeTreeMapTest.c"
					GXmlNode* name = NULL;
					GXmlNode* _tmp49_ = NULL;
					GeeMap* _tmp50_ = NULL;
					GeeMap* _tmp51_ = NULL;
					GeeMap* _tmp52_ = NULL;
					gpointer _tmp53_ = NULL;
					GXmlNode* _tmp54_ = NULL;
					GXmlNode* _tmp55_ = NULL;
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp49_ = n;
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp50_ = gxml_node_get_attrs (_tmp49_);
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp51_ = _tmp50_;
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp52_ = _tmp51_;
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp53_ = gee_map_get (_tmp52_, "name");
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp54_ = (GXmlNode*) _tmp53_;
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_g_object_unref0 (_tmp52_);
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					name = _tmp54_;
#line 152 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp55_ = name;
#line 152 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					if (_tmp55_ != NULL) {
#line 1189 "SerializableGeeTreeMapTest.c"
						GXmlNode* _tmp56_ = NULL;
						gchar* _tmp57_ = NULL;
						gchar* _tmp58_ = NULL;
						gchar* _tmp59_ = NULL;
						gboolean _tmp60_ = FALSE;
						GXmlNode* _tmp61_ = NULL;
						gchar* _tmp62_ = NULL;
						gchar* _tmp63_ = NULL;
						gchar* _tmp64_ = NULL;
						gboolean _tmp65_ = FALSE;
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_tmp56_ = name;
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_tmp57_ = gxml_node_get_value (_tmp56_);
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_tmp58_ = _tmp57_;
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_tmp59_ = _tmp58_;
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_tmp60_ = g_strcmp0 (_tmp59_, "Big") == 0;
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_g_free0 (_tmp59_);
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						if (_tmp60_) {
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
							found1 = TRUE;
#line 1216 "SerializableGeeTreeMapTest.c"
						}
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_tmp61_ = name;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_tmp62_ = gxml_node_get_value (_tmp61_);
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_tmp63_ = _tmp62_;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_tmp64_ = _tmp63_;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_tmp65_ = g_strcmp0 (_tmp64_, "Small") == 0;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_g_free0 (_tmp64_);
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						if (_tmp65_) {
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
							found2 = TRUE;
#line 1234 "SerializableGeeTreeMapTest.c"
						}
					}
#line 150 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_g_object_unref0 (name);
#line 1239 "SerializableGeeTreeMapTest.c"
				}
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_g_object_unref0 (n);
#line 1243 "SerializableGeeTreeMapTest.c"
			}
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (_n_list);
#line 1247 "SerializableGeeTreeMapTest.c"
		}
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp66_ = found1;
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (!_tmp66_) {
#line 1253 "SerializableGeeTreeMapTest.c"
			FILE* _tmp67_ = NULL;
			GXmlTDocument* _tmp68_ = NULL;
			gchar* _tmp69_ = NULL;
			gchar* _tmp70_ = NULL;
			gchar* _tmp71_ = NULL;
			gchar* _tmp72_ = NULL;
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp67_ = stdout;
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp68_ = doc;
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp69_ = gxml_tnode_to_string ((GXmlTNode*) _tmp68_);
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp70_ = _tmp69_;
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp71_ = g_strconcat ("ERROR: Big space node is not found\n", _tmp70_, "\n", NULL);
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp72_ = _tmp71_;
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			fprintf (_tmp67_, "%s", _tmp72_);
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp72_);
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp70_);
#line 160 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			g_assert_not_reached ();
#line 1280 "SerializableGeeTreeMapTest.c"
		}
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp73_ = found2;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (!_tmp73_) {
#line 1286 "SerializableGeeTreeMapTest.c"
			FILE* _tmp74_ = NULL;
			GXmlTDocument* _tmp75_ = NULL;
			gchar* _tmp76_ = NULL;
			gchar* _tmp77_ = NULL;
			gchar* _tmp78_ = NULL;
			gchar* _tmp79_ = NULL;
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp74_ = stdout;
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp75_ = doc;
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp76_ = gxml_tnode_to_string ((GXmlTNode*) _tmp75_);
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp77_ = _tmp76_;
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp78_ = g_strconcat ("ERROR: Small space node is not found\n", _tmp77_, "\n", NULL);
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp79_ = _tmp78_;
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			fprintf (_tmp74_, "%s", _tmp79_);
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp79_);
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp77_);
#line 164 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			g_assert_not_reached ();
#line 1313 "SerializableGeeTreeMapTest.c"
		}
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (root);
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (doc);
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (o2);
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (o1);
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (c);
#line 1325 "SerializableGeeTreeMapTest.c"
	}
	goto __finally71;
	__catch71_g_error:
	{
		GError* e = NULL;
		FILE* _tmp80_ = NULL;
		const gchar* _tmp81_ = NULL;
		const gchar* _tmp82_ = NULL;
		gchar* _tmp83_ = NULL;
		gchar* _tmp84_ = NULL;
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		e = _inner_error_;
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_inner_error_ = NULL;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp80_ = stdout;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp81_ = e->message;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp82_ = string_to_string (_tmp81_);
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp83_ = g_strconcat ("ERROR: ", _tmp82_, NULL);
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp84_ = _tmp83_;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		fprintf (_tmp80_, "%s", _tmp84_);
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_free0 (_tmp84_);
#line 169 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_assert_not_reached ();
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_error_free0 (e);
#line 1358 "SerializableGeeTreeMapTest.c"
	}
	__finally71:
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_clear_error (&_inner_error_);
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		return;
#line 1369 "SerializableGeeTreeMapTest.c"
	}
}


static void ___lambda40__gtest_func (void) {
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	__lambda40_ ();
#line 1377 "SerializableGeeTreeMapTest.c"
}


static void __lambda41_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		GXmlSerializableTreeMap* c = NULL;
		GXmlSerializableTreeMap* _tmp1_ = NULL;
		GXmlSerializableTreeMap* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		GXmlNode* _tmp4_ = NULL;
		GXmlNode* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		GXmlSerializableTreeMap* _tmp7_ = NULL;
		gint _tmp8_ = 0;
		gint _tmp9_ = 0;
		gboolean found1 = FALSE;
		gboolean found2 = FALSE;
		gboolean _tmp42_ = FALSE;
		gboolean _tmp44_ = FALSE;
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp0_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n" \
"  <root><space name=\"Big\"/><space name=\"Small\"/></root>", 0, &_inner_error_);
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		doc = _tmp0_;
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1406 "SerializableGeeTreeMapTest.c"
			goto __catch72_g_error;
		}
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp1_ = gxml_serializable_tree_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		c = _tmp1_;
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp2_ = c;
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp3_ = doc;
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp4_ = gxml_document_get_root ((GXmlDocument*) _tmp3_);
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp5_ = _tmp4_;
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp6_ = _tmp5_;
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		gxml_serializable_tree_map_deserialize (_tmp2_, _tmp6_, &_inner_error_);
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp6_);
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (c);
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (doc);
#line 1433 "SerializableGeeTreeMapTest.c"
			goto __catch72_g_error;
		}
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp7_ = c;
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp8_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp7_);
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp9_ = _tmp8_;
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (_tmp9_ != 2) {
#line 1444 "SerializableGeeTreeMapTest.c"
			FILE* _tmp10_ = NULL;
			GXmlSerializableTreeMap* _tmp11_ = NULL;
			gint _tmp12_ = 0;
			gint _tmp13_ = 0;
			gchar* _tmp14_ = NULL;
			gchar* _tmp15_ = NULL;
			gchar* _tmp16_ = NULL;
			gchar* _tmp17_ = NULL;
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp10_ = stdout;
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp11_ = c;
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp12_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp11_);
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp13_ = _tmp12_;
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp14_ = g_strdup_printf ("%i", _tmp13_);
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp15_ = _tmp14_;
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp16_ = g_strconcat ("ERROR: incorrect size must be 2 got: ", _tmp15_, "\n", NULL);
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp17_ = _tmp16_;
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			fprintf (_tmp10_, "%s", _tmp17_);
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp17_);
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp15_);
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			g_assert_not_reached ();
#line 1477 "SerializableGeeTreeMapTest.c"
		}
#line 183 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		found1 = FALSE;
#line 184 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		found2 = FALSE;
#line 1483 "SerializableGeeTreeMapTest.c"
		{
			GeeIterator* _k_it = NULL;
			GXmlSerializableTreeMap* _tmp18_ = NULL;
			GeeSet* _tmp19_ = NULL;
			GeeSet* _tmp20_ = NULL;
			GeeSet* _tmp21_ = NULL;
			GeeIterator* _tmp22_ = NULL;
			GeeIterator* _tmp23_ = NULL;
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp18_ = c;
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp19_ = gee_abstract_map_get_keys ((GeeAbstractMap*) _tmp18_);
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp20_ = _tmp19_;
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp21_ = _tmp20_;
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp22_ = gee_iterable_iterator ((GeeIterable*) _tmp21_);
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp23_ = _tmp22_;
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (_tmp21_);
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_k_it = _tmp23_;
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			while (TRUE) {
#line 1510 "SerializableGeeTreeMapTest.c"
				GeeIterator* _tmp24_ = NULL;
				gboolean _tmp25_ = FALSE;
				gchar* k = NULL;
				GeeIterator* _tmp26_ = NULL;
				gpointer _tmp27_ = NULL;
				GXmlSerializableTreeMap* _tmp28_ = NULL;
				const gchar* _tmp29_ = NULL;
				gpointer _tmp30_ = NULL;
				SerializableGeeTreeMapTestSpace* _tmp31_ = NULL;
				const gchar* _tmp32_ = NULL;
				const gchar* _tmp33_ = NULL;
				gboolean _tmp34_ = FALSE;
				GXmlSerializableTreeMap* _tmp35_ = NULL;
				const gchar* _tmp36_ = NULL;
				gpointer _tmp37_ = NULL;
				SerializableGeeTreeMapTestSpace* _tmp38_ = NULL;
				const gchar* _tmp39_ = NULL;
				const gchar* _tmp40_ = NULL;
				gboolean _tmp41_ = FALSE;
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp24_ = _k_it;
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp25_ = gee_iterator_next (_tmp24_);
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				if (!_tmp25_) {
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					break;
#line 1538 "SerializableGeeTreeMapTest.c"
				}
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp26_ = _k_it;
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp27_ = gee_iterator_get (_tmp26_);
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				k = (gchar*) _tmp27_;
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp28_ = c;
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp29_ = k;
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp30_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp28_, _tmp29_);
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp31_ = (SerializableGeeTreeMapTestSpace*) _tmp30_;
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp32_ = serializable_gee_tree_map_test_space_get_name (_tmp31_);
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp33_ = _tmp32_;
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp34_ = g_strcmp0 (_tmp33_, "Big") == 0;
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_g_object_unref0 (_tmp31_);
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				if (_tmp34_) {
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					found1 = TRUE;
#line 1566 "SerializableGeeTreeMapTest.c"
				}
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp35_ = c;
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp36_ = k;
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp37_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp35_, _tmp36_);
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp38_ = (SerializableGeeTreeMapTestSpace*) _tmp37_;
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp39_ = serializable_gee_tree_map_test_space_get_name (_tmp38_);
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp40_ = _tmp39_;
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp41_ = g_strcmp0 (_tmp40_, "Small") == 0;
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_g_object_unref0 (_tmp38_);
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				if (_tmp41_) {
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					found2 = TRUE;
#line 1588 "SerializableGeeTreeMapTest.c"
				}
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_g_free0 (k);
#line 1592 "SerializableGeeTreeMapTest.c"
			}
#line 185 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (_k_it);
#line 1596 "SerializableGeeTreeMapTest.c"
		}
#line 189 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp42_ = found1;
#line 189 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (!_tmp42_) {
#line 1602 "SerializableGeeTreeMapTest.c"
			FILE* _tmp43_ = NULL;
#line 190 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp43_ = stdout;
#line 190 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			fprintf (_tmp43_, "%s", "ERROR: Big key value is not found\n");
#line 191 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			g_assert_not_reached ();
#line 1610 "SerializableGeeTreeMapTest.c"
		}
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp44_ = found2;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (!_tmp44_) {
#line 1616 "SerializableGeeTreeMapTest.c"
			FILE* _tmp45_ = NULL;
#line 194 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp45_ = stdout;
#line 194 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			fprintf (_tmp45_, "%s", "ERROR: Small key value is not found\n");
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			g_assert_not_reached ();
#line 1624 "SerializableGeeTreeMapTest.c"
		}
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (c);
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (doc);
#line 1630 "SerializableGeeTreeMapTest.c"
	}
	goto __finally72;
	__catch72_g_error:
	{
		GError* e = NULL;
		FILE* _tmp46_ = NULL;
		const gchar* _tmp47_ = NULL;
		const gchar* _tmp48_ = NULL;
		gchar* _tmp49_ = NULL;
		gchar* _tmp50_ = NULL;
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		e = _inner_error_;
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_inner_error_ = NULL;
#line 199 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp46_ = stdout;
#line 199 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp47_ = e->message;
#line 199 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp48_ = string_to_string (_tmp47_);
#line 199 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp49_ = g_strconcat ("ERROR: ", _tmp48_, NULL);
#line 199 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp50_ = _tmp49_;
#line 199 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		fprintf (_tmp46_, "%s", _tmp50_);
#line 199 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_free0 (_tmp50_);
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_assert_not_reached ();
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_error_free0 (e);
#line 1663 "SerializableGeeTreeMapTest.c"
	}
	__finally72:
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_clear_error (&_inner_error_);
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		return;
#line 1674 "SerializableGeeTreeMapTest.c"
	}
}


static void ___lambda41__gtest_func (void) {
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	__lambda41_ ();
#line 1682 "SerializableGeeTreeMapTest.c"
}


static void __lambda42_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		SerializableGeeTreeMapTestSpaceContainer* c = NULL;
		SerializableGeeTreeMapTestSpaceContainer* _tmp1_ = NULL;
		SerializableGeeTreeMapTestSpaceContainer* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		SerializableGeeTreeMapTestSpaceContainer* _tmp4_ = NULL;
		const gchar* _tmp5_ = NULL;
		const gchar* _tmp6_ = NULL;
		SerializableGeeTreeMapTestSpaceContainer* _tmp17_ = NULL;
		SerializableGeeTreeMapTestSpaceCollection* _tmp18_ = NULL;
		SerializableGeeTreeMapTestSpaceCollection* _tmp19_ = NULL;
		SerializableGeeTreeMapTestSpaceContainer* _tmp26_ = NULL;
		SerializableGeeTreeMapTestSpaceCollection* _tmp27_ = NULL;
		SerializableGeeTreeMapTestSpaceCollection* _tmp28_ = NULL;
		gint _tmp29_ = 0;
		gint _tmp30_ = 0;
		gboolean found1 = FALSE;
		gboolean found2 = FALSE;
		gboolean _tmp74_ = FALSE;
		gboolean _tmp76_ = FALSE;
#line 206 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp0_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n" \
"  <spacecontainer owner=\"Earth\"><space name=\"Big\"/><space name=\"S" \
"mall\"/></spacecontainer>", 0, &_inner_error_);
#line 206 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		doc = _tmp0_;
#line 206 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1716 "SerializableGeeTreeMapTest.c"
			goto __catch73_g_error;
		}
#line 208 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp1_ = serializable_gee_tree_map_test_space_container_new ();
#line 208 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		c = _tmp1_;
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp2_ = c;
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp3_ = doc;
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp2_, (GXmlNode*) _tmp3_, &_inner_error_);
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (c);
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (doc);
#line 1735 "SerializableGeeTreeMapTest.c"
			goto __catch73_g_error;
		}
#line 210 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp4_ = c;
#line 210 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp5_ = serializable_gee_tree_map_test_space_container_get_owner (_tmp4_);
#line 210 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp6_ = _tmp5_;
#line 210 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (g_strcmp0 (_tmp6_, "Earth") != 0) {
#line 1746 "SerializableGeeTreeMapTest.c"
			FILE* _tmp7_ = NULL;
			SerializableGeeTreeMapTestSpaceContainer* _tmp8_ = NULL;
			const gchar* _tmp9_ = NULL;
			const gchar* _tmp10_ = NULL;
			const gchar* _tmp11_ = NULL;
			GXmlGDocument* _tmp12_ = NULL;
			gchar* _tmp13_ = NULL;
			gchar* _tmp14_ = NULL;
			gchar* _tmp15_ = NULL;
			gchar* _tmp16_ = NULL;
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp7_ = stdout;
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp8_ = c;
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp9_ = serializable_gee_tree_map_test_space_container_get_owner (_tmp8_);
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp10_ = _tmp9_;
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp11_ = string_to_string (_tmp10_);
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp12_ = doc;
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp13_ = gxml_gnode_to_string ((GXmlGNode*) _tmp12_);
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp14_ = _tmp13_;
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp15_ = g_strconcat ("ERROR: owner must be 'Earth' got: ", _tmp11_, "\n", _tmp14_, "\n", NULL);
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp16_ = _tmp15_;
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			fprintf (_tmp7_, "%s", _tmp16_);
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp16_);
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp14_);
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			g_assert_not_reached ();
#line 1785 "SerializableGeeTreeMapTest.c"
		}
#line 214 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp17_ = c;
#line 214 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp18_ = serializable_gee_tree_map_test_space_container_get_storage (_tmp17_);
#line 214 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp19_ = _tmp18_;
#line 214 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (_tmp19_ == NULL) {
#line 1795 "SerializableGeeTreeMapTest.c"
			FILE* _tmp20_ = NULL;
			GXmlGDocument* _tmp21_ = NULL;
			gchar* _tmp22_ = NULL;
			gchar* _tmp23_ = NULL;
			gchar* _tmp24_ = NULL;
			gchar* _tmp25_ = NULL;
#line 215 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp20_ = stdout;
#line 215 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp21_ = doc;
#line 215 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp22_ = gxml_gnode_to_string ((GXmlGNode*) _tmp21_);
#line 215 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp23_ = _tmp22_;
#line 215 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp24_ = g_strconcat ("ERROR: storage doesn't exist\n", _tmp23_, "\n", NULL);
#line 215 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp25_ = _tmp24_;
#line 215 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			fprintf (_tmp20_, "%s", _tmp25_);
#line 215 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp25_);
#line 215 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp23_);
#line 216 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			g_assert_not_reached ();
#line 1822 "SerializableGeeTreeMapTest.c"
		}
#line 218 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp26_ = c;
#line 218 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp27_ = serializable_gee_tree_map_test_space_container_get_storage (_tmp26_);
#line 218 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp28_ = _tmp27_;
#line 218 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp29_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp28_);
#line 218 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp30_ = _tmp29_;
#line 218 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (_tmp30_ != 2) {
#line 1836 "SerializableGeeTreeMapTest.c"
			FILE* _tmp31_ = NULL;
			SerializableGeeTreeMapTestSpaceContainer* _tmp32_ = NULL;
			SerializableGeeTreeMapTestSpaceCollection* _tmp33_ = NULL;
			SerializableGeeTreeMapTestSpaceCollection* _tmp34_ = NULL;
			gint _tmp35_ = 0;
			gint _tmp36_ = 0;
			gchar* _tmp37_ = NULL;
			gchar* _tmp38_ = NULL;
			GXmlGDocument* _tmp39_ = NULL;
			gchar* _tmp40_ = NULL;
			gchar* _tmp41_ = NULL;
			gchar* _tmp42_ = NULL;
			gchar* _tmp43_ = NULL;
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp31_ = stdout;
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp32_ = c;
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp33_ = serializable_gee_tree_map_test_space_container_get_storage (_tmp32_);
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp34_ = _tmp33_;
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp35_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp34_);
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp36_ = _tmp35_;
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp37_ = g_strdup_printf ("%i", _tmp36_);
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp38_ = _tmp37_;
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp39_ = doc;
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp40_ = gxml_gnode_to_string ((GXmlGNode*) _tmp39_);
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp41_ = _tmp40_;
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp42_ = g_strconcat ("ERROR: Size must be 2 got: ", _tmp38_, "\n", _tmp41_, "\n", NULL);
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp43_ = _tmp42_;
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			fprintf (_tmp31_, "%s", _tmp43_);
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp43_);
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp41_);
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp38_);
#line 220 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			g_assert_not_reached ();
#line 1886 "SerializableGeeTreeMapTest.c"
		}
#line 222 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		found1 = FALSE;
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		found2 = FALSE;
#line 1892 "SerializableGeeTreeMapTest.c"
		{
			GeeIterator* _k_it = NULL;
			SerializableGeeTreeMapTestSpaceContainer* _tmp44_ = NULL;
			SerializableGeeTreeMapTestSpaceCollection* _tmp45_ = NULL;
			SerializableGeeTreeMapTestSpaceCollection* _tmp46_ = NULL;
			GeeSet* _tmp47_ = NULL;
			GeeSet* _tmp48_ = NULL;
			GeeSet* _tmp49_ = NULL;
			GeeIterator* _tmp50_ = NULL;
			GeeIterator* _tmp51_ = NULL;
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp44_ = c;
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp45_ = serializable_gee_tree_map_test_space_container_get_storage (_tmp44_);
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp46_ = _tmp45_;
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp47_ = gee_abstract_map_get_keys ((GeeAbstractMap*) _tmp46_);
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp48_ = _tmp47_;
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp49_ = _tmp48_;
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp50_ = gee_iterable_iterator ((GeeIterable*) _tmp49_);
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp51_ = _tmp50_;
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (_tmp49_);
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_k_it = _tmp51_;
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			while (TRUE) {
#line 1925 "SerializableGeeTreeMapTest.c"
				GeeIterator* _tmp52_ = NULL;
				gboolean _tmp53_ = FALSE;
				gchar* k = NULL;
				GeeIterator* _tmp54_ = NULL;
				gpointer _tmp55_ = NULL;
				SerializableGeeTreeMapTestSpaceContainer* _tmp56_ = NULL;
				SerializableGeeTreeMapTestSpaceCollection* _tmp57_ = NULL;
				SerializableGeeTreeMapTestSpaceCollection* _tmp58_ = NULL;
				const gchar* _tmp59_ = NULL;
				gpointer _tmp60_ = NULL;
				SerializableGeeTreeMapTestSpace* _tmp61_ = NULL;
				const gchar* _tmp62_ = NULL;
				const gchar* _tmp63_ = NULL;
				gboolean _tmp64_ = FALSE;
				SerializableGeeTreeMapTestSpaceContainer* _tmp65_ = NULL;
				SerializableGeeTreeMapTestSpaceCollection* _tmp66_ = NULL;
				SerializableGeeTreeMapTestSpaceCollection* _tmp67_ = NULL;
				const gchar* _tmp68_ = NULL;
				gpointer _tmp69_ = NULL;
				SerializableGeeTreeMapTestSpace* _tmp70_ = NULL;
				const gchar* _tmp71_ = NULL;
				const gchar* _tmp72_ = NULL;
				gboolean _tmp73_ = FALSE;
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp52_ = _k_it;
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp53_ = gee_iterator_next (_tmp52_);
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				if (!_tmp53_) {
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					break;
#line 1957 "SerializableGeeTreeMapTest.c"
				}
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp54_ = _k_it;
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp55_ = gee_iterator_get (_tmp54_);
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				k = (gchar*) _tmp55_;
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp56_ = c;
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp57_ = serializable_gee_tree_map_test_space_container_get_storage (_tmp56_);
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp58_ = _tmp57_;
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp59_ = k;
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp60_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp58_, _tmp59_);
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp61_ = (SerializableGeeTreeMapTestSpace*) _tmp60_;
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp62_ = serializable_gee_tree_map_test_space_get_name (_tmp61_);
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp63_ = _tmp62_;
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp64_ = g_strcmp0 (_tmp63_, "Big") == 0;
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_g_object_unref0 (_tmp61_);
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				if (_tmp64_) {
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					found1 = TRUE;
#line 1989 "SerializableGeeTreeMapTest.c"
				}
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp65_ = c;
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp66_ = serializable_gee_tree_map_test_space_container_get_storage (_tmp65_);
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp67_ = _tmp66_;
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp68_ = k;
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp69_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp67_, _tmp68_);
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp70_ = (SerializableGeeTreeMapTestSpace*) _tmp69_;
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp71_ = serializable_gee_tree_map_test_space_get_name (_tmp70_);
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp72_ = _tmp71_;
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp73_ = g_strcmp0 (_tmp72_, "Small") == 0;
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_g_object_unref0 (_tmp70_);
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				if (_tmp73_) {
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					found2 = TRUE;
#line 2015 "SerializableGeeTreeMapTest.c"
				}
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_g_free0 (k);
#line 2019 "SerializableGeeTreeMapTest.c"
			}
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (_k_it);
#line 2023 "SerializableGeeTreeMapTest.c"
		}
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp74_ = found1;
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (!_tmp74_) {
#line 2029 "SerializableGeeTreeMapTest.c"
			FILE* _tmp75_ = NULL;
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp75_ = stdout;
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			fprintf (_tmp75_, "%s", "ERROR: Big key value is not found\n");
#line 230 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			g_assert_not_reached ();
#line 2037 "SerializableGeeTreeMapTest.c"
		}
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp76_ = found2;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (!_tmp76_) {
#line 2043 "SerializableGeeTreeMapTest.c"
			FILE* _tmp77_ = NULL;
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp77_ = stdout;
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			fprintf (_tmp77_, "%s", "ERROR: Small key value is not found\n");
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			g_assert_not_reached ();
#line 2051 "SerializableGeeTreeMapTest.c"
		}
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (c);
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (doc);
#line 2057 "SerializableGeeTreeMapTest.c"
	}
	goto __finally73;
	__catch73_g_error:
	{
		GError* e = NULL;
		FILE* _tmp78_ = NULL;
		const gchar* _tmp79_ = NULL;
		const gchar* _tmp80_ = NULL;
		gchar* _tmp81_ = NULL;
		gchar* _tmp82_ = NULL;
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		e = _inner_error_;
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_inner_error_ = NULL;
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp78_ = stdout;
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp79_ = e->message;
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp80_ = string_to_string (_tmp79_);
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp81_ = g_strconcat ("ERROR: ", _tmp80_, NULL);
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp82_ = _tmp81_;
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		fprintf (_tmp78_, "%s", _tmp82_);
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_free0 (_tmp82_);
#line 239 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_assert_not_reached ();
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_error_free0 (e);
#line 2090 "SerializableGeeTreeMapTest.c"
	}
	__finally73:
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_clear_error (&_inner_error_);
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		return;
#line 2101 "SerializableGeeTreeMapTest.c"
	}
}


static void ___lambda42__gtest_func (void) {
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	__lambda42_ ();
#line 2109 "SerializableGeeTreeMapTest.c"
}


static void __lambda43_ (void) {
	GError * _inner_error_ = NULL;
	{
		SerializableGeeTreeMapTestSpaceContainer* c = NULL;
		SerializableGeeTreeMapTestSpaceContainer* _tmp0_ = NULL;
		SerializableGeeTreeMapTestSpace* o1 = NULL;
		SerializableGeeTreeMapTestSpace* _tmp1_ = NULL;
		SerializableGeeTreeMapTestSpace* o2 = NULL;
		SerializableGeeTreeMapTestSpace* _tmp2_ = NULL;
		SerializableGeeTreeMapTestSpaceCollection* _tmp3_ = NULL;
		SerializableGeeTreeMapTestSpaceCollection* _tmp4_ = NULL;
		SerializableGeeTreeMapTestSpaceCollection* _tmp5_ = NULL;
		SerializableGeeTreeMapTestSpaceCollection* _tmp6_ = NULL;
		const gchar* _tmp7_ = NULL;
		const gchar* _tmp8_ = NULL;
		SerializableGeeTreeMapTestSpaceCollection* _tmp9_ = NULL;
		SerializableGeeTreeMapTestSpaceCollection* _tmp10_ = NULL;
		const gchar* _tmp11_ = NULL;
		const gchar* _tmp12_ = NULL;
		GXmlTDocument* doc = NULL;
		GXmlTDocument* _tmp13_ = NULL;
		GXmlNode* _tmp14_ = NULL;
		GXmlNode* _tmp15_ = NULL;
		GXmlNode* _tmp16_ = NULL;
		GXmlNode* _tmp17_ = NULL;
		GXmlNode* _tmp18_ = NULL;
		gboolean _tmp19_ = FALSE;
		GXmlNode* _tmp25_ = NULL;
		GXmlNode* _tmp26_ = NULL;
		GXmlNode* _tmp27_ = NULL;
		gchar* _tmp28_ = NULL;
		gchar* _tmp29_ = NULL;
		gchar* _tmp30_ = NULL;
		gboolean _tmp31_ = FALSE;
		GXmlNode* root = NULL;
		GXmlNode* _tmp44_ = NULL;
		GXmlNode* _tmp45_ = NULL;
		GXmlNode* _tmp46_ = NULL;
		GeeBidirList* _tmp47_ = NULL;
		GeeBidirList* _tmp48_ = NULL;
		GeeBidirList* _tmp49_ = NULL;
		gint _tmp50_ = 0;
		gint _tmp51_ = 0;
		gboolean found1 = FALSE;
		gboolean found2 = FALSE;
		gboolean _tmp87_ = FALSE;
		gboolean _tmp93_ = FALSE;
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp0_ = serializable_gee_tree_map_test_space_container_new ();
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		c = _tmp0_;
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp1_ = serializable_gee_tree_map_test_space_new_named ("Big");
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		o1 = _tmp1_;
#line 247 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp2_ = serializable_gee_tree_map_test_space_new_named ("Small");
#line 247 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		o2 = _tmp2_;
#line 248 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp3_ = serializable_gee_tree_map_test_space_collection_new ();
#line 248 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp4_ = _tmp3_;
#line 248 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		serializable_gee_tree_map_test_space_container_set_storage (c, _tmp4_);
#line 248 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp4_);
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp5_ = serializable_gee_tree_map_test_space_container_get_storage (c);
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp6_ = _tmp5_;
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp7_ = serializable_gee_tree_map_test_space_get_name (o1);
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp8_ = _tmp7_;
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		gee_abstract_map_set ((GeeAbstractMap*) _tmp6_, _tmp8_, o1);
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp9_ = serializable_gee_tree_map_test_space_container_get_storage (c);
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp10_ = _tmp9_;
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp11_ = serializable_gee_tree_map_test_space_get_name (o2);
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp12_ = _tmp11_;
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		gee_abstract_map_set ((GeeAbstractMap*) _tmp10_, _tmp12_, o2);
#line 251 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp13_ = gxml_tdocument_new ();
#line 251 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		doc = _tmp13_;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp14_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) c, (GXmlNode*) doc, &_inner_error_);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp15_ = _tmp14_;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp15_);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (doc);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (o2);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (o1);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (c);
#line 2220 "SerializableGeeTreeMapTest.c"
			goto __catch74_g_error;
		}
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp16_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp17_ = _tmp16_;
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp18_ = _tmp17_;
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp19_ = _tmp18_ == NULL;
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp18_);
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (_tmp19_) {
#line 2235 "SerializableGeeTreeMapTest.c"
			FILE* _tmp20_ = NULL;
			gchar* _tmp21_ = NULL;
			gchar* _tmp22_ = NULL;
			gchar* _tmp23_ = NULL;
			gchar* _tmp24_ = NULL;
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp20_ = stdout;
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp21_ = gxml_tnode_to_string ((GXmlTNode*) doc);
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp22_ = _tmp21_;
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp23_ = g_strconcat ("ERROR: doc have no root node\n", _tmp22_, "\n", NULL);
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp24_ = _tmp23_;
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			fprintf (_tmp20_, "%s", _tmp24_);
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp24_);
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp22_);
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			g_assert_not_reached ();
#line 2259 "SerializableGeeTreeMapTest.c"
		}
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp25_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp26_ = _tmp25_;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp27_ = _tmp26_;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp28_ = gxml_node_get_name (_tmp27_);
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp29_ = _tmp28_;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp30_ = _tmp29_;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp31_ = g_strcmp0 (_tmp30_, "spacecontainer") != 0;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_free0 (_tmp30_);
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp27_);
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (_tmp31_) {
#line 2281 "SerializableGeeTreeMapTest.c"
			FILE* _tmp32_ = NULL;
			GXmlNode* _tmp33_ = NULL;
			GXmlNode* _tmp34_ = NULL;
			GXmlNode* _tmp35_ = NULL;
			gchar* _tmp36_ = NULL;
			gchar* _tmp37_ = NULL;
			gchar* _tmp38_ = NULL;
			const gchar* _tmp39_ = NULL;
			gchar* _tmp40_ = NULL;
			gchar* _tmp41_ = NULL;
			gchar* _tmp42_ = NULL;
			gchar* _tmp43_ = NULL;
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp32_ = stdout;
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp33_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp34_ = _tmp33_;
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp35_ = _tmp34_;
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp36_ = gxml_node_get_name (_tmp35_);
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp37_ = _tmp36_;
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp38_ = _tmp37_;
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp39_ = string_to_string (_tmp38_);
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp40_ = gxml_tnode_to_string ((GXmlTNode*) doc);
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp41_ = _tmp40_;
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp42_ = g_strconcat ("ERROR: bad doc root node's name: ", _tmp39_, "\n", _tmp41_, "\n", NULL);
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp43_ = _tmp42_;
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			fprintf (_tmp32_, "%s", _tmp43_);
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp43_);
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp41_);
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp38_);
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (_tmp35_);
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			g_assert_not_reached ();
#line 2330 "SerializableGeeTreeMapTest.c"
		}
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp44_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp45_ = _tmp44_;
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		root = _tmp45_;
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp46_ = root;
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp47_ = gxml_node_get_children_nodes (_tmp46_);
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp48_ = _tmp47_;
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp49_ = _tmp48_;
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp50_ = gee_collection_get_size ((GeeCollection*) _tmp49_);
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp51_ = _tmp50_;
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp51_ == 2, "root.children_nodes.size == 2");
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp49_);
#line 263 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		found1 = FALSE;
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		found2 = FALSE;
#line 2358 "SerializableGeeTreeMapTest.c"
		{
			GeeBidirList* _n_list = NULL;
			GXmlNode* _tmp52_ = NULL;
			GeeBidirList* _tmp53_ = NULL;
			GeeBidirList* _tmp54_ = NULL;
			gint _n_size = 0;
			GeeBidirList* _tmp55_ = NULL;
			gint _tmp56_ = 0;
			gint _tmp57_ = 0;
			gint _n_index = 0;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp52_ = root;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp53_ = gxml_node_get_children_nodes (_tmp52_);
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp54_ = _tmp53_;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_n_list = _tmp54_;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp55_ = _n_list;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp56_ = gee_collection_get_size ((GeeCollection*) _tmp55_);
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp57_ = _tmp56_;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_n_size = _tmp57_;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_n_index = -1;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			while (TRUE) {
#line 2389 "SerializableGeeTreeMapTest.c"
				gint _tmp58_ = 0;
				gint _tmp59_ = 0;
				gint _tmp60_ = 0;
				GXmlNode* n = NULL;
				GeeBidirList* _tmp61_ = NULL;
				gint _tmp62_ = 0;
				gpointer _tmp63_ = NULL;
				gboolean _tmp64_ = FALSE;
				GXmlNode* _tmp65_ = NULL;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp58_ = _n_index;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_n_index = _tmp58_ + 1;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp59_ = _n_index;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp60_ = _n_size;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				if (!(_tmp59_ < _tmp60_)) {
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					break;
#line 2411 "SerializableGeeTreeMapTest.c"
				}
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp61_ = _n_list;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp62_ = _n_index;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp63_ = gee_list_get ((GeeList*) _tmp61_, _tmp62_);
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				n = (GXmlNode*) _tmp63_;
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp65_ = n;
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp65_, GXML_TYPE_ELEMENT)) {
#line 2425 "SerializableGeeTreeMapTest.c"
					GXmlNode* _tmp66_ = NULL;
					gchar* _tmp67_ = NULL;
					gchar* _tmp68_ = NULL;
					gchar* _tmp69_ = NULL;
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp66_ = n;
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp67_ = gxml_node_get_name (_tmp66_);
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp68_ = _tmp67_;
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp69_ = _tmp68_;
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp64_ = g_strcmp0 (_tmp69_, "space") == 0;
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_g_free0 (_tmp69_);
#line 2442 "SerializableGeeTreeMapTest.c"
				} else {
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp64_ = FALSE;
#line 2446 "SerializableGeeTreeMapTest.c"
				}
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				if (_tmp64_) {
#line 2450 "SerializableGeeTreeMapTest.c"
					GXmlNode* name = NULL;
					GXmlNode* _tmp70_ = NULL;
					GeeMap* _tmp71_ = NULL;
					GeeMap* _tmp72_ = NULL;
					GeeMap* _tmp73_ = NULL;
					gpointer _tmp74_ = NULL;
					GXmlNode* _tmp75_ = NULL;
					GXmlNode* _tmp76_ = NULL;
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp70_ = n;
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp71_ = gxml_node_get_attrs (_tmp70_);
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp72_ = _tmp71_;
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp73_ = _tmp72_;
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp74_ = gee_map_get (_tmp73_, "name");
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp75_ = (GXmlNode*) _tmp74_;
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_g_object_unref0 (_tmp73_);
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					name = _tmp75_;
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp76_ = name;
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					if (_tmp76_ != NULL) {
#line 2479 "SerializableGeeTreeMapTest.c"
						GXmlNode* _tmp77_ = NULL;
						gchar* _tmp78_ = NULL;
						gchar* _tmp79_ = NULL;
						gchar* _tmp80_ = NULL;
						gboolean _tmp81_ = FALSE;
						GXmlNode* _tmp82_ = NULL;
						gchar* _tmp83_ = NULL;
						gchar* _tmp84_ = NULL;
						gchar* _tmp85_ = NULL;
						gboolean _tmp86_ = FALSE;
#line 269 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_tmp77_ = name;
#line 269 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_tmp78_ = gxml_node_get_value (_tmp77_);
#line 269 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_tmp79_ = _tmp78_;
#line 269 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_tmp80_ = _tmp79_;
#line 269 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_tmp81_ = g_strcmp0 (_tmp80_, "Big") == 0;
#line 269 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_g_free0 (_tmp80_);
#line 269 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						if (_tmp81_) {
#line 269 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
							found1 = TRUE;
#line 2506 "SerializableGeeTreeMapTest.c"
						}
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_tmp82_ = name;
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_tmp83_ = gxml_node_get_value (_tmp82_);
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_tmp84_ = _tmp83_;
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_tmp85_ = _tmp84_;
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_tmp86_ = g_strcmp0 (_tmp85_, "Small") == 0;
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_g_free0 (_tmp85_);
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						if (_tmp86_) {
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
							found2 = TRUE;
#line 2524 "SerializableGeeTreeMapTest.c"
						}
					}
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_g_object_unref0 (name);
#line 2529 "SerializableGeeTreeMapTest.c"
				}
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_g_object_unref0 (n);
#line 2533 "SerializableGeeTreeMapTest.c"
			}
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (_n_list);
#line 2537 "SerializableGeeTreeMapTest.c"
		}
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp87_ = found1;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (!_tmp87_) {
#line 2543 "SerializableGeeTreeMapTest.c"
			FILE* _tmp88_ = NULL;
			gchar* _tmp89_ = NULL;
			gchar* _tmp90_ = NULL;
			gchar* _tmp91_ = NULL;
			gchar* _tmp92_ = NULL;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp88_ = stdout;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp89_ = gxml_tnode_to_string ((GXmlTNode*) doc);
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp90_ = _tmp89_;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp91_ = g_strconcat ("ERROR: Big space node is not found\n", _tmp90_, "\n", NULL);
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp92_ = _tmp91_;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			fprintf (_tmp88_, "%s", _tmp92_);
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp92_);
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp90_);
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			g_assert_not_reached ();
#line 2567 "SerializableGeeTreeMapTest.c"
		}
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp93_ = found2;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (!_tmp93_) {
#line 2573 "SerializableGeeTreeMapTest.c"
			FILE* _tmp94_ = NULL;
			gchar* _tmp95_ = NULL;
			gchar* _tmp96_ = NULL;
			gchar* _tmp97_ = NULL;
			gchar* _tmp98_ = NULL;
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp94_ = stdout;
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp95_ = gxml_tnode_to_string ((GXmlTNode*) doc);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp96_ = _tmp95_;
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp97_ = g_strconcat ("ERROR: Small space node is not found\n", _tmp96_, "\n", NULL);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_tmp98_ = _tmp97_;
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			fprintf (_tmp94_, "%s", _tmp98_);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp98_);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_free0 (_tmp96_);
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			g_assert_not_reached ();
#line 2597 "SerializableGeeTreeMapTest.c"
		}
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (root);
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (doc);
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (o2);
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (o1);
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (c);
#line 2609 "SerializableGeeTreeMapTest.c"
	}
	goto __finally74;
	__catch74_g_error:
	{
		GError* e = NULL;
		FILE* _tmp99_ = NULL;
		const gchar* _tmp100_ = NULL;
		const gchar* _tmp101_ = NULL;
		gchar* _tmp102_ = NULL;
		gchar* _tmp103_ = NULL;
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		e = _inner_error_;
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_inner_error_ = NULL;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp99_ = stdout;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp100_ = e->message;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp101_ = string_to_string (_tmp100_);
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp102_ = g_strconcat ("ERROR: ", _tmp101_, NULL);
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp103_ = _tmp102_;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		fprintf (_tmp99_, "%s", _tmp103_);
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_free0 (_tmp103_);
#line 285 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_assert_not_reached ();
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_error_free0 (e);
#line 2642 "SerializableGeeTreeMapTest.c"
	}
	__finally74:
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_clear_error (&_inner_error_);
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		return;
#line 2653 "SerializableGeeTreeMapTest.c"
	}
}


static void ___lambda43__gtest_func (void) {
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	__lambda43_ ();
#line 2661 "SerializableGeeTreeMapTest.c"
}


static void __lambda44_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* d = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		BookStore* bs = NULL;
		BookStore* _tmp1_ = NULL;
		BookStore* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		BookStore* _tmp4_ = NULL;
		const gchar* _tmp5_ = NULL;
		const gchar* _tmp6_ = NULL;
		BookStore* _tmp7_ = NULL;
		BookArray* _tmp8_ = NULL;
		BookArray* _tmp9_ = NULL;
		gint _tmp10_ = 0;
		gint _tmp11_ = 0;
		Book* b = NULL;
		BookStore* _tmp12_ = NULL;
		BookArray* _tmp13_ = NULL;
		BookArray* _tmp14_ = NULL;
		gpointer _tmp15_ = NULL;
		Name* _tmp16_ = NULL;
		Name* _tmp17_ = NULL;
		Name* _tmp18_ = NULL;
		Name* _tmp19_ = NULL;
		gchar* _tmp20_ = NULL;
		gchar* _tmp21_ = NULL;
		const gchar* _tmp22_ = NULL;
		const gchar* _tmp23_ = NULL;
		Authors* _tmp24_ = NULL;
		Authors* _tmp25_ = NULL;
		Authors* _tmp26_ = NULL;
		Authors* _tmp27_ = NULL;
		AuthorArray* _tmp28_ = NULL;
		AuthorArray* _tmp29_ = NULL;
		Authors* _tmp30_ = NULL;
		Authors* _tmp31_ = NULL;
		AuthorArray* _tmp32_ = NULL;
		AuthorArray* _tmp33_ = NULL;
		gint _tmp34_ = 0;
		gint _tmp35_ = 0;
		ResumeMap* _tmp36_ = NULL;
		ResumeMap* _tmp37_ = NULL;
		ResumeMap* _tmp38_ = NULL;
		ResumeMap* _tmp39_ = NULL;
		gint _tmp40_ = 0;
		gint _tmp41_ = 0;
		Resume* r = NULL;
		ResumeMap* _tmp42_ = NULL;
		ResumeMap* _tmp43_ = NULL;
		gpointer _tmp44_ = NULL;
		const gchar* _tmp45_ = NULL;
		const gchar* _tmp46_ = NULL;
		const gchar* _tmp47_ = NULL;
		const gchar* _tmp48_ = NULL;
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp0_ = gxml_gdocument_new_from_path (TEST_DIR "/test-collection.xml", 0, &_inner_error_);
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		d = _tmp0_;
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 2727 "SerializableGeeTreeMapTest.c"
			goto __catch75_g_error;
		}
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp1_ = book_store_new ();
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		bs = _tmp1_;
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp2_ = bs;
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp3_ = d;
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp2_, (GXmlNode*) _tmp3_, &_inner_error_);
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bs);
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (d);
#line 2746 "SerializableGeeTreeMapTest.c"
			goto __catch75_g_error;
		}
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp4_ = bs;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp5_ = book_store_get_name (_tmp4_);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp6_ = _tmp5_;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp6_, "The Great Book") == 0, "bs.name == \"The Great Book\"");
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp7_ = bs;
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp8_ = book_store_get_books (_tmp7_);
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp9_ = _tmp8_;
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp10_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp9_);
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp11_ = _tmp10_;
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp11_ == 3, "bs.books.size == 3");
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp12_ = bs;
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp13_ = book_store_get_books (_tmp12_);
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp14_ = _tmp13_;
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp15_ = gee_list_first ((GeeList*) _tmp14_);
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		b = (Book*) _tmp15_;
#line 297 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (b != NULL, "b != null");
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp16_ = book_get_name (b);
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp17_ = _tmp16_;
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp17_ != NULL, "b.name != null");
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp18_ = book_get_name (b);
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp19_ = _tmp18_;
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp20_ = name_get_name (_tmp19_);
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp21_ = _tmp20_;
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp21_, "Book1") == 0, "b.name.get_name () == \"Book1\"");
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_free0 (_tmp21_);
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp22_ = book_get_year (b);
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp23_ = _tmp22_;
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp23_, "2015") == 0, "b.year == \"2015\"");
#line 301 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp24_ = book_get_authors (b);
#line 301 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp25_ = _tmp24_;
#line 301 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp25_ != NULL, "b.authors != null");
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp26_ = book_get_authors (b);
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp27_ = _tmp26_;
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp28_ = authors_get_array (_tmp27_);
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp29_ = _tmp28_;
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp29_ != NULL, "b.authors.array != null");
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp30_ = book_get_authors (b);
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp31_ = _tmp30_;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp32_ = authors_get_array (_tmp31_);
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp33_ = _tmp32_;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp34_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp33_);
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp35_ = _tmp34_;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp35_ == 2, "b.authors.array.size == 2");
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp36_ = book_get_resumes (b);
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp37_ = _tmp36_;
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp37_ != NULL, "b.resumes != null");
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp38_ = book_get_resumes (b);
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp39_ = _tmp38_;
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp40_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp39_);
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp41_ = _tmp40_;
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp41_ == 2, "b.resumes.size == 2");
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp42_ = book_get_resumes (b);
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp43_ = _tmp42_;
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp44_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp43_, "II");
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		r = (Resume*) _tmp44_;
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (r != NULL, "r != null");
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp45_ = resume_get_chapter (r);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp46_ = _tmp45_;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp46_, "II") == 0, "r.chapter == \"II\"");
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp47_ = resume_get_text (r);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp48_ = _tmp47_;
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp48_, "Nothing to say") == 0, "r.text == \"Nothing to say\"");
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (r);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (b);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (bs);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (d);
#line 2881 "SerializableGeeTreeMapTest.c"
	}
	goto __finally75;
	__catch75_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp49_ = NULL;
		gchar* _tmp50_ = NULL;
		gchar* _tmp51_ = NULL;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		e = _inner_error_;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_inner_error_ = NULL;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp49_ = e->message;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp50_ = g_strconcat ("ERROR: ", _tmp49_, NULL);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp51_ = _tmp50_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_message ("SerializableGeeTreeMapTest.vala:311: %s", _tmp51_);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_free0 (_tmp51_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_assert_not_reached ();
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_error_free0 (e);
#line 2908 "SerializableGeeTreeMapTest.c"
	}
	__finally75:
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_clear_error (&_inner_error_);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		return;
#line 2919 "SerializableGeeTreeMapTest.c"
	}
}


static void ___lambda44__gtest_func (void) {
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	__lambda44_ ();
#line 2927 "SerializableGeeTreeMapTest.c"
}


static void __lambda45_ (void) {
	GError * _inner_error_ = NULL;
	{
		SerializableGeeTreeMapTestBigBag* bag = NULL;
		SerializableGeeTreeMapTestBigBag* _tmp0_ = NULL;
		SerializableGeeTreeMapTestBigBag* _tmp1_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp2_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp3_ = NULL;
		SerializableGeeTreeMapTestBigBag* _tmp52_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp53_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp54_ = NULL;
		gint _tmp55_ = 0;
		gint _tmp56_ = 0;
		GXmlGDocument* d = NULL;
		GXmlGDocument* _tmp57_ = NULL;
		SerializableGeeTreeMapTestBigBag* _tmp58_ = NULL;
		GXmlGDocument* _tmp59_ = NULL;
		GXmlNode* _tmp60_ = NULL;
		GXmlNode* _tmp61_ = NULL;
		GXmlGDocument* _tmp62_ = NULL;
		GXmlNode* _tmp63_ = NULL;
		GXmlNode* _tmp64_ = NULL;
		GXmlNode* _tmp65_ = NULL;
		GXmlGDocument* _tmp66_ = NULL;
		GXmlNode* _tmp67_ = NULL;
		GXmlNode* _tmp68_ = NULL;
		GXmlNode* _tmp69_ = NULL;
		gchar* _tmp70_ = NULL;
		gchar* _tmp71_ = NULL;
		gchar* _tmp72_ = NULL;
		GXmlGDocument* _tmp73_ = NULL;
		GXmlNode* _tmp74_ = NULL;
		GXmlNode* _tmp75_ = NULL;
		GXmlNode* _tmp76_ = NULL;
		GeeBidirList* _tmp77_ = NULL;
		GeeBidirList* _tmp78_ = NULL;
		GeeBidirList* _tmp79_ = NULL;
		gint _tmp80_ = 0;
		gint _tmp81_ = 0;
		GXmlGDocument* _tmp82_ = NULL;
		GXmlNode* _tmp83_ = NULL;
		GXmlNode* _tmp84_ = NULL;
		GXmlNode* _tmp85_ = NULL;
		GeeBidirList* _tmp86_ = NULL;
		GeeBidirList* _tmp87_ = NULL;
		GeeBidirList* _tmp88_ = NULL;
		gpointer _tmp89_ = NULL;
		GXmlNode* _tmp90_ = NULL;
		gchar* _tmp91_ = NULL;
		gchar* _tmp92_ = NULL;
		gchar* _tmp93_ = NULL;
		GXmlGDocument* _tmp94_ = NULL;
		GXmlNode* _tmp95_ = NULL;
		GXmlNode* _tmp96_ = NULL;
		GXmlNode* _tmp97_ = NULL;
		GeeBidirList* _tmp98_ = NULL;
		GeeBidirList* _tmp99_ = NULL;
		GeeBidirList* _tmp100_ = NULL;
		gpointer _tmp101_ = NULL;
		GXmlNode* _tmp102_ = NULL;
		GeeBidirList* _tmp103_ = NULL;
		GeeBidirList* _tmp104_ = NULL;
		GeeBidirList* _tmp105_ = NULL;
		gint _tmp106_ = 0;
		gint _tmp107_ = 0;
		GXmlGDocument* _tmp108_ = NULL;
		GXmlNode* _tmp109_ = NULL;
		GXmlNode* _tmp110_ = NULL;
		GXmlNode* _tmp111_ = NULL;
		GeeBidirList* _tmp112_ = NULL;
		GeeBidirList* _tmp113_ = NULL;
		GeeBidirList* _tmp114_ = NULL;
		gpointer _tmp115_ = NULL;
		GXmlNode* _tmp116_ = NULL;
		GeeBidirList* _tmp117_ = NULL;
		GeeBidirList* _tmp118_ = NULL;
		GeeBidirList* _tmp119_ = NULL;
		gpointer _tmp120_ = NULL;
		GXmlNode* _tmp121_ = NULL;
		gchar* _tmp122_ = NULL;
		gchar* _tmp123_ = NULL;
		gchar* _tmp124_ = NULL;
		GXmlGDocument* _tmp125_ = NULL;
		GXmlNode* _tmp126_ = NULL;
		GXmlNode* _tmp127_ = NULL;
		GXmlNode* _tmp128_ = NULL;
		GeeBidirList* _tmp129_ = NULL;
		GeeBidirList* _tmp130_ = NULL;
		GeeBidirList* _tmp131_ = NULL;
		gpointer _tmp132_ = NULL;
		GXmlNode* _tmp133_ = NULL;
		GeeBidirList* _tmp134_ = NULL;
		GeeBidirList* _tmp135_ = NULL;
		GeeBidirList* _tmp136_ = NULL;
		gpointer _tmp137_ = NULL;
		GXmlNode* _tmp138_ = NULL;
		GeeBidirList* _tmp139_ = NULL;
		GeeBidirList* _tmp140_ = NULL;
		GeeBidirList* _tmp141_ = NULL;
		gint _tmp142_ = 0;
		gint _tmp143_ = 0;
		GXmlGDocument* _tmp144_ = NULL;
		GXmlNode* _tmp145_ = NULL;
		GXmlNode* _tmp146_ = NULL;
		GXmlNode* _tmp147_ = NULL;
		GeeBidirList* _tmp148_ = NULL;
		GeeBidirList* _tmp149_ = NULL;
		GeeBidirList* _tmp150_ = NULL;
		gpointer _tmp151_ = NULL;
		GXmlNode* _tmp152_ = NULL;
		GeeBidirList* _tmp153_ = NULL;
		GeeBidirList* _tmp154_ = NULL;
		GeeBidirList* _tmp155_ = NULL;
		gpointer _tmp156_ = NULL;
		GXmlNode* _tmp157_ = NULL;
		GeeBidirList* _tmp158_ = NULL;
		GeeBidirList* _tmp159_ = NULL;
		GeeBidirList* _tmp160_ = NULL;
		gpointer _tmp161_ = NULL;
		GXmlNode* _tmp162_ = NULL;
		gchar* _tmp163_ = NULL;
		gchar* _tmp164_ = NULL;
		gchar* _tmp165_ = NULL;
		GXmlGDocument* _tmp166_ = NULL;
		GXmlNode* _tmp167_ = NULL;
		GXmlNode* _tmp168_ = NULL;
		GXmlNode* _tmp169_ = NULL;
		GeeBidirList* _tmp170_ = NULL;
		GeeBidirList* _tmp171_ = NULL;
		GeeBidirList* _tmp172_ = NULL;
		gpointer _tmp173_ = NULL;
		GXmlNode* _tmp174_ = NULL;
		GeeBidirList* _tmp175_ = NULL;
		GeeBidirList* _tmp176_ = NULL;
		GeeBidirList* _tmp177_ = NULL;
		gpointer _tmp178_ = NULL;
		GXmlNode* _tmp179_ = NULL;
		GeeBidirList* _tmp180_ = NULL;
		GeeBidirList* _tmp181_ = NULL;
		GeeBidirList* _tmp182_ = NULL;
		gpointer _tmp183_ = NULL;
		GXmlNode* _tmp184_ = NULL;
		GeeBidirList* _tmp185_ = NULL;
		GeeBidirList* _tmp186_ = NULL;
		GeeBidirList* _tmp187_ = NULL;
		gint _tmp188_ = 0;
		gint _tmp189_ = 0;
		GXmlGDocument* _tmp190_ = NULL;
		GXmlNode* _tmp191_ = NULL;
		GXmlNode* _tmp192_ = NULL;
		GXmlNode* _tmp193_ = NULL;
		GeeBidirList* _tmp194_ = NULL;
		GeeBidirList* _tmp195_ = NULL;
		GeeBidirList* _tmp196_ = NULL;
		gpointer _tmp197_ = NULL;
		GXmlNode* _tmp198_ = NULL;
		GeeBidirList* _tmp199_ = NULL;
		GeeBidirList* _tmp200_ = NULL;
		GeeBidirList* _tmp201_ = NULL;
		gpointer _tmp202_ = NULL;
		GXmlNode* _tmp203_ = NULL;
		GeeBidirList* _tmp204_ = NULL;
		GeeBidirList* _tmp205_ = NULL;
		GeeBidirList* _tmp206_ = NULL;
		gpointer _tmp207_ = NULL;
		GXmlNode* _tmp208_ = NULL;
		GeeBidirList* _tmp209_ = NULL;
		GeeBidirList* _tmp210_ = NULL;
		GeeBidirList* _tmp211_ = NULL;
		gpointer _tmp212_ = NULL;
		GXmlNode* _tmp213_ = NULL;
		GXmlGDocument* _tmp214_ = NULL;
		GXmlNode* _tmp215_ = NULL;
		GXmlNode* _tmp216_ = NULL;
		GXmlNode* _tmp217_ = NULL;
		GeeBidirList* _tmp218_ = NULL;
		GeeBidirList* _tmp219_ = NULL;
		GeeBidirList* _tmp220_ = NULL;
		gpointer _tmp221_ = NULL;
		GXmlNode* _tmp222_ = NULL;
		GeeBidirList* _tmp223_ = NULL;
		GeeBidirList* _tmp224_ = NULL;
		GeeBidirList* _tmp225_ = NULL;
		gpointer _tmp226_ = NULL;
		GXmlNode* _tmp227_ = NULL;
		GeeBidirList* _tmp228_ = NULL;
		GeeBidirList* _tmp229_ = NULL;
		GeeBidirList* _tmp230_ = NULL;
		gpointer _tmp231_ = NULL;
		GXmlNode* _tmp232_ = NULL;
		GeeBidirList* _tmp233_ = NULL;
		GeeBidirList* _tmp234_ = NULL;
		GeeBidirList* _tmp235_ = NULL;
		gpointer _tmp236_ = NULL;
		GXmlNode* _tmp237_ = NULL;
		gchar* _tmp238_ = NULL;
		gchar* _tmp239_ = NULL;
		gchar* _tmp240_ = NULL;
		SerializableGeeTreeMapTestBigBag* bagt = NULL;
		SerializableGeeTreeMapTestBigBag* _tmp241_ = NULL;
		SerializableGeeTreeMapTestBigBag* _tmp242_ = NULL;
		GXmlGDocument* _tmp243_ = NULL;
		SerializableGeeTreeMapTestBigBag* _tmp244_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp245_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp246_ = NULL;
		gint _tmp247_ = 0;
		gint _tmp248_ = 0;
		gboolean _tmp249_ = FALSE;
		SerializableGeeTreeMapTestBigBag* _tmp250_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp251_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp252_ = NULL;
		gboolean _tmp253_ = FALSE;
		SerializableGeeTreeMapTestBigBag* _tmp254_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp255_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp256_ = NULL;
		gint _tmp257_ = 0;
		gint _tmp258_ = 0;
		gboolean _tmp259_ = FALSE;
		SerializableGeeTreeMapTestBigBag* _tmp260_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp261_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp262_ = NULL;
		gpointer _tmp263_ = NULL;
		SerializableGeeTreeMapTestSmallBag* _tmp264_ = NULL;
		SerializableGeeTreeMapTestBallTreeMap* _tmp265_ = NULL;
		SerializableGeeTreeMapTestBallTreeMap* _tmp266_ = NULL;
		gboolean _tmp267_ = FALSE;
		gboolean _tmp268_ = FALSE;
		SerializableGeeTreeMapTestBigBag* _tmp269_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp270_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp271_ = NULL;
		gpointer _tmp272_ = NULL;
		SerializableGeeTreeMapTestSmallBag* _tmp273_ = NULL;
		SerializableGeeTreeMapTestBallTreeMap* _tmp274_ = NULL;
		SerializableGeeTreeMapTestBallTreeMap* _tmp275_ = NULL;
		gint _tmp276_ = 0;
		gint _tmp277_ = 0;
		SerializableGeeTreeMapTestBigBag* _tmp278_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp279_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp280_ = NULL;
		gpointer _tmp281_ = NULL;
		SerializableGeeTreeMapTestSmallBag* _tmp282_ = NULL;
		SerializableGeeTreeMapTestBallTreeMap* _tmp283_ = NULL;
		SerializableGeeTreeMapTestBallTreeMap* _tmp284_ = NULL;
		gpointer _tmp285_ = NULL;
		SerializableGeeTreeMapTestBall* _tmp286_ = NULL;
		const gchar* _tmp287_ = NULL;
		const gchar* _tmp288_ = NULL;
		SerializableGeeTreeMapTestBigBag* _tmp289_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp290_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp291_ = NULL;
		gpointer _tmp292_ = NULL;
		SerializableGeeTreeMapTestSmallBag* _tmp293_ = NULL;
		SerializableGeeTreeMapTestBallTreeMap* _tmp294_ = NULL;
		SerializableGeeTreeMapTestBallTreeMap* _tmp295_ = NULL;
		gpointer _tmp296_ = NULL;
		SerializableGeeTreeMapTestBall* _tmp297_ = NULL;
		SerializableGeeTreeMapTestBallFill* _tmp298_ = NULL;
		SerializableGeeTreeMapTestBallFill* _tmp299_ = NULL;
		SerializableGeeTreeMapTestBigBag* _tmp300_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp301_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp302_ = NULL;
		gpointer _tmp303_ = NULL;
		SerializableGeeTreeMapTestSmallBag* _tmp304_ = NULL;
		SerializableGeeTreeMapTestBallTreeMap* _tmp305_ = NULL;
		SerializableGeeTreeMapTestBallTreeMap* _tmp306_ = NULL;
		gpointer _tmp307_ = NULL;
		SerializableGeeTreeMapTestBall* _tmp308_ = NULL;
		SerializableGeeTreeMapTestBallFill* _tmp309_ = NULL;
		SerializableGeeTreeMapTestBallFill* _tmp310_ = NULL;
		gchar* _tmp311_ = NULL;
		gchar* _tmp312_ = NULL;
		gchar* _tmp313_ = NULL;
		SerializableGeeTreeMapTestBigBag* _tmp314_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp315_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp316_ = NULL;
		gpointer _tmp317_ = NULL;
		SerializableGeeTreeMapTestSmallBag* _tmp318_ = NULL;
		SerializableGeeTreeMapTestBallTreeMap* _tmp319_ = NULL;
		SerializableGeeTreeMapTestBallTreeMap* _tmp320_ = NULL;
		gpointer _tmp321_ = NULL;
		SerializableGeeTreeMapTestBall* _tmp322_ = NULL;
		SerializableGeeTreeMapTestBallFill* _tmp323_ = NULL;
		SerializableGeeTreeMapTestBallFill* _tmp324_ = NULL;
		gchar* _tmp325_ = NULL;
		gchar* _tmp326_ = NULL;
		gchar* _tmp327_ = NULL;
		SerializableGeeTreeMapTestBigBag* bag3 = NULL;
		SerializableGeeTreeMapTestBigBag* _tmp328_ = NULL;
		SerializableGeeTreeMapTestBigBag* _tmp329_ = NULL;
		GXmlGDocument* _tmp330_ = NULL;
		SerializableGeeTreeMapTestBigBag* _tmp331_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp332_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp333_ = NULL;
		gint _tmp334_ = 0;
		gint _tmp335_ = 0;
		gboolean _tmp336_ = FALSE;
		SerializableGeeTreeMapTestBigBag* _tmp337_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp338_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp339_ = NULL;
		gboolean _tmp340_ = FALSE;
		SerializableGeeTreeMapTestBigBag* _tmp341_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp342_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp343_ = NULL;
		gint _tmp344_ = 0;
		gint _tmp345_ = 0;
		SerializableGeeTreeMapTestBigBag* _tmp346_ = NULL;
		GXmlGDocument* _tmp347_ = NULL;
		SerializableGeeTreeMapTestBigBag* _tmp348_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp349_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp350_ = NULL;
		gboolean _tmp351_ = FALSE;
		SerializableGeeTreeMapTestBigBag* bag2 = NULL;
		SerializableGeeTreeMapTestBigBag* _tmp352_ = NULL;
		SerializableGeeTreeMapTestBigBag* _tmp353_ = NULL;
		GXmlGDocument* _tmp354_ = NULL;
		SerializableGeeTreeMapTestBigBag* _tmp355_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp356_ = NULL;
		SerializableGeeTreeMapTestSmallBagTreeMap* _tmp357_ = NULL;
		gboolean _tmp358_ = FALSE;
		GXmlGDocument* d2 = NULL;
		GXmlGDocument* _tmp359_ = NULL;
		SerializableGeeTreeMapTestBigBag* _tmp360_ = NULL;
		GXmlGDocument* _tmp361_ = NULL;
		GXmlNode* _tmp362_ = NULL;
		GXmlNode* _tmp363_ = NULL;
		GXmlGDocument* _tmp364_ = NULL;
		GXmlNode* _tmp365_ = NULL;
		GXmlNode* _tmp366_ = NULL;
		GXmlNode* _tmp367_ = NULL;
		GXmlGDocument* _tmp368_ = NULL;
		GXmlNode* _tmp369_ = NULL;
		GXmlNode* _tmp370_ = NULL;
		GXmlNode* _tmp371_ = NULL;
		gchar* _tmp372_ = NULL;
		gchar* _tmp373_ = NULL;
		gchar* _tmp374_ = NULL;
		GXmlGDocument* _tmp375_ = NULL;
		GXmlNode* _tmp376_ = NULL;
		GXmlNode* _tmp377_ = NULL;
		GXmlNode* _tmp378_ = NULL;
		GeeBidirList* _tmp379_ = NULL;
		GeeBidirList* _tmp380_ = NULL;
		GeeBidirList* _tmp381_ = NULL;
		gint _tmp382_ = 0;
		gint _tmp383_ = 0;
		GXmlGDocument* _tmp384_ = NULL;
		GXmlNode* _tmp385_ = NULL;
		GXmlNode* _tmp386_ = NULL;
		GXmlNode* _tmp387_ = NULL;
		GeeBidirList* _tmp388_ = NULL;
		GeeBidirList* _tmp389_ = NULL;
		GeeBidirList* _tmp390_ = NULL;
		gpointer _tmp391_ = NULL;
		GXmlNode* _tmp392_ = NULL;
		gchar* _tmp393_ = NULL;
		gchar* _tmp394_ = NULL;
		gchar* _tmp395_ = NULL;
		GXmlGDocument* _tmp396_ = NULL;
		GXmlNode* _tmp397_ = NULL;
		GXmlNode* _tmp398_ = NULL;
		GXmlNode* _tmp399_ = NULL;
		GeeBidirList* _tmp400_ = NULL;
		GeeBidirList* _tmp401_ = NULL;
		GeeBidirList* _tmp402_ = NULL;
		gpointer _tmp403_ = NULL;
		GXmlNode* _tmp404_ = NULL;
		GeeBidirList* _tmp405_ = NULL;
		GeeBidirList* _tmp406_ = NULL;
		GeeBidirList* _tmp407_ = NULL;
		gint _tmp408_ = 0;
		gint _tmp409_ = 0;
		GXmlGDocument* _tmp410_ = NULL;
		GXmlNode* _tmp411_ = NULL;
		GXmlNode* _tmp412_ = NULL;
		GXmlNode* _tmp413_ = NULL;
		GeeBidirList* _tmp414_ = NULL;
		GeeBidirList* _tmp415_ = NULL;
		GeeBidirList* _tmp416_ = NULL;
		gpointer _tmp417_ = NULL;
		GXmlNode* _tmp418_ = NULL;
		GeeBidirList* _tmp419_ = NULL;
		GeeBidirList* _tmp420_ = NULL;
		GeeBidirList* _tmp421_ = NULL;
		gpointer _tmp422_ = NULL;
		GXmlNode* _tmp423_ = NULL;
		gchar* _tmp424_ = NULL;
		gchar* _tmp425_ = NULL;
		gchar* _tmp426_ = NULL;
		GXmlGDocument* _tmp427_ = NULL;
		GXmlNode* _tmp428_ = NULL;
		GXmlNode* _tmp429_ = NULL;
		GXmlNode* _tmp430_ = NULL;
		GeeBidirList* _tmp431_ = NULL;
		GeeBidirList* _tmp432_ = NULL;
		GeeBidirList* _tmp433_ = NULL;
		gpointer _tmp434_ = NULL;
		GXmlNode* _tmp435_ = NULL;
		GeeBidirList* _tmp436_ = NULL;
		GeeBidirList* _tmp437_ = NULL;
		GeeBidirList* _tmp438_ = NULL;
		gpointer _tmp439_ = NULL;
		GXmlNode* _tmp440_ = NULL;
		GeeBidirList* _tmp441_ = NULL;
		GeeBidirList* _tmp442_ = NULL;
		GeeBidirList* _tmp443_ = NULL;
		gint _tmp444_ = 0;
		gint _tmp445_ = 0;
		GXmlGDocument* _tmp446_ = NULL;
		GXmlNode* _tmp447_ = NULL;
		GXmlNode* _tmp448_ = NULL;
		GXmlNode* _tmp449_ = NULL;
		GeeBidirList* _tmp450_ = NULL;
		GeeBidirList* _tmp451_ = NULL;
		GeeBidirList* _tmp452_ = NULL;
		gpointer _tmp453_ = NULL;
		GXmlNode* _tmp454_ = NULL;
		GeeBidirList* _tmp455_ = NULL;
		GeeBidirList* _tmp456_ = NULL;
		GeeBidirList* _tmp457_ = NULL;
		gpointer _tmp458_ = NULL;
		GXmlNode* _tmp459_ = NULL;
		GeeBidirList* _tmp460_ = NULL;
		GeeBidirList* _tmp461_ = NULL;
		GeeBidirList* _tmp462_ = NULL;
		gpointer _tmp463_ = NULL;
		GXmlNode* _tmp464_ = NULL;
		gchar* _tmp465_ = NULL;
		gchar* _tmp466_ = NULL;
		gchar* _tmp467_ = NULL;
		GXmlGDocument* _tmp468_ = NULL;
		GXmlNode* _tmp469_ = NULL;
		GXmlNode* _tmp470_ = NULL;
		GXmlNode* _tmp471_ = NULL;
		GeeBidirList* _tmp472_ = NULL;
		GeeBidirList* _tmp473_ = NULL;
		GeeBidirList* _tmp474_ = NULL;
		gpointer _tmp475_ = NULL;
		GXmlNode* _tmp476_ = NULL;
		GeeBidirList* _tmp477_ = NULL;
		GeeBidirList* _tmp478_ = NULL;
		GeeBidirList* _tmp479_ = NULL;
		gpointer _tmp480_ = NULL;
		GXmlNode* _tmp481_ = NULL;
		GeeBidirList* _tmp482_ = NULL;
		GeeBidirList* _tmp483_ = NULL;
		GeeBidirList* _tmp484_ = NULL;
		gpointer _tmp485_ = NULL;
		GXmlNode* _tmp486_ = NULL;
		GeeBidirList* _tmp487_ = NULL;
		GeeBidirList* _tmp488_ = NULL;
		GeeBidirList* _tmp489_ = NULL;
		gint _tmp490_ = 0;
		gint _tmp491_ = 0;
		GXmlGDocument* _tmp492_ = NULL;
		GXmlNode* _tmp493_ = NULL;
		GXmlNode* _tmp494_ = NULL;
		GXmlNode* _tmp495_ = NULL;
		GeeBidirList* _tmp496_ = NULL;
		GeeBidirList* _tmp497_ = NULL;
		GeeBidirList* _tmp498_ = NULL;
		gpointer _tmp499_ = NULL;
		GXmlNode* _tmp500_ = NULL;
		GeeBidirList* _tmp501_ = NULL;
		GeeBidirList* _tmp502_ = NULL;
		GeeBidirList* _tmp503_ = NULL;
		gpointer _tmp504_ = NULL;
		GXmlNode* _tmp505_ = NULL;
		GeeBidirList* _tmp506_ = NULL;
		GeeBidirList* _tmp507_ = NULL;
		GeeBidirList* _tmp508_ = NULL;
		gpointer _tmp509_ = NULL;
		GXmlNode* _tmp510_ = NULL;
		GeeBidirList* _tmp511_ = NULL;
		GeeBidirList* _tmp512_ = NULL;
		GeeBidirList* _tmp513_ = NULL;
		gpointer _tmp514_ = NULL;
		GXmlNode* _tmp515_ = NULL;
		GXmlGDocument* _tmp516_ = NULL;
		GXmlNode* _tmp517_ = NULL;
		GXmlNode* _tmp518_ = NULL;
		GXmlNode* _tmp519_ = NULL;
		GeeBidirList* _tmp520_ = NULL;
		GeeBidirList* _tmp521_ = NULL;
		GeeBidirList* _tmp522_ = NULL;
		gpointer _tmp523_ = NULL;
		GXmlNode* _tmp524_ = NULL;
		GeeBidirList* _tmp525_ = NULL;
		GeeBidirList* _tmp526_ = NULL;
		GeeBidirList* _tmp527_ = NULL;
		gpointer _tmp528_ = NULL;
		GXmlNode* _tmp529_ = NULL;
		GeeBidirList* _tmp530_ = NULL;
		GeeBidirList* _tmp531_ = NULL;
		GeeBidirList* _tmp532_ = NULL;
		gpointer _tmp533_ = NULL;
		GXmlNode* _tmp534_ = NULL;
		GeeBidirList* _tmp535_ = NULL;
		GeeBidirList* _tmp536_ = NULL;
		GeeBidirList* _tmp537_ = NULL;
		gpointer _tmp538_ = NULL;
		GXmlNode* _tmp539_ = NULL;
		gchar* _tmp540_ = NULL;
		gchar* _tmp541_ = NULL;
		gchar* _tmp542_ = NULL;
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp0_ = serializable_gee_tree_map_test_big_bag_new ();
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		bag = _tmp0_;
#line 320 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp1_ = bag;
#line 320 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp2_ = serializable_gee_tree_map_test_big_bag_get_bags (_tmp1_);
#line 320 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp3_ = _tmp2_;
#line 320 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp3_ != NULL, "bag.bags != null");
#line 3447 "SerializableGeeTreeMapTest.c"
		{
			gint i = 0;
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			i = 0;
#line 3452 "SerializableGeeTreeMapTest.c"
			{
				gboolean _tmp4_ = FALSE;
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				_tmp4_ = TRUE;
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
				while (TRUE) {
#line 3459 "SerializableGeeTreeMapTest.c"
					gint _tmp6_ = 0;
					SerializableGeeTreeMapTestSmallBag* sbag = NULL;
					SerializableGeeTreeMapTestSmallBag* _tmp7_ = NULL;
					SerializableGeeTreeMapTestSmallBag* _tmp8_ = NULL;
					gint _tmp9_ = 0;
					gchar* _tmp10_ = NULL;
					gchar* _tmp11_ = NULL;
					gchar* _tmp12_ = NULL;
					gchar* _tmp13_ = NULL;
					SerializableGeeTreeMapTestSmallBag* _tmp14_ = NULL;
					SerializableGeeTreeMapTestBallTreeMap* _tmp15_ = NULL;
					SerializableGeeTreeMapTestBallTreeMap* _tmp16_ = NULL;
					SerializableGeeTreeMapTestSmallBag* _tmp40_ = NULL;
					SerializableGeeTreeMapTestBallTreeMap* _tmp41_ = NULL;
					SerializableGeeTreeMapTestBallTreeMap* _tmp42_ = NULL;
					gint _tmp43_ = 0;
					gint _tmp44_ = 0;
					SerializableGeeTreeMapTestBigBag* _tmp45_ = NULL;
					SerializableGeeTreeMapTestSmallBagTreeMap* _tmp46_ = NULL;
					SerializableGeeTreeMapTestSmallBagTreeMap* _tmp47_ = NULL;
					SerializableGeeTreeMapTestSmallBag* _tmp48_ = NULL;
					const gchar* _tmp49_ = NULL;
					const gchar* _tmp50_ = NULL;
					SerializableGeeTreeMapTestSmallBag* _tmp51_ = NULL;
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					if (!_tmp4_) {
#line 3486 "SerializableGeeTreeMapTest.c"
						gint _tmp5_ = 0;
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						_tmp5_ = i;
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						i = _tmp5_ + 1;
#line 3492 "SerializableGeeTreeMapTest.c"
					}
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp4_ = FALSE;
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp6_ = i;
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					if (!(_tmp6_ < 2)) {
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						break;
#line 3502 "SerializableGeeTreeMapTest.c"
					}
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp7_ = serializable_gee_tree_map_test_small_bag_new ();
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					sbag = _tmp7_;
#line 323 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp8_ = sbag;
#line 323 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp9_ = i;
#line 323 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp10_ = g_strdup_printf ("%i", _tmp9_);
#line 323 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp11_ = _tmp10_;
#line 323 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp12_ = g_strconcat ("SmallBag", _tmp11_, NULL);
#line 323 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp13_ = _tmp12_;
#line 323 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					serializable_gee_tree_map_test_small_bag_set_name (_tmp8_, _tmp13_);
#line 323 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_g_free0 (_tmp13_);
#line 323 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_g_free0 (_tmp11_);
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp14_ = sbag;
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp15_ = serializable_gee_tree_map_test_small_bag_get_balls (_tmp14_);
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp16_ = _tmp15_;
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_vala_assert (_tmp16_ != NULL, "sbag.balls != null");
#line 3534 "SerializableGeeTreeMapTest.c"
					{
						gint j = 0;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
						j = 0;
#line 3539 "SerializableGeeTreeMapTest.c"
						{
							gboolean _tmp17_ = FALSE;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
							_tmp17_ = TRUE;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
							while (TRUE) {
#line 3546 "SerializableGeeTreeMapTest.c"
								gint _tmp19_ = 0;
								SerializableGeeTreeMapTestBall* b = NULL;
								SerializableGeeTreeMapTestBall* _tmp20_ = NULL;
								SerializableGeeTreeMapTestBall* _tmp21_ = NULL;
								gint _tmp22_ = 0;
								gchar* _tmp23_ = NULL;
								gchar* _tmp24_ = NULL;
								gchar* _tmp25_ = NULL;
								gchar* _tmp26_ = NULL;
								SerializableGeeTreeMapTestBall* _tmp27_ = NULL;
								SerializableGeeTreeMapTestBallFill* _tmp28_ = NULL;
								SerializableGeeTreeMapTestBallFill* _tmp29_ = NULL;
								SerializableGeeTreeMapTestBall* _tmp30_ = NULL;
								SerializableGeeTreeMapTestBallFill* _tmp31_ = NULL;
								SerializableGeeTreeMapTestBallFill* _tmp32_ = NULL;
								SerializableGeeTreeMapTestSmallBag* _tmp33_ = NULL;
								SerializableGeeTreeMapTestBallTreeMap* _tmp34_ = NULL;
								SerializableGeeTreeMapTestBallTreeMap* _tmp35_ = NULL;
								SerializableGeeTreeMapTestBall* _tmp36_ = NULL;
								const gchar* _tmp37_ = NULL;
								const gchar* _tmp38_ = NULL;
								SerializableGeeTreeMapTestBall* _tmp39_ = NULL;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								if (!_tmp17_) {
#line 3571 "SerializableGeeTreeMapTest.c"
									gint _tmp18_ = 0;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
									_tmp18_ = j;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
									j = _tmp18_ + 1;
#line 3577 "SerializableGeeTreeMapTest.c"
								}
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp17_ = FALSE;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp19_ = j;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								if (!(_tmp19_ < 2)) {
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
									break;
#line 3587 "SerializableGeeTreeMapTest.c"
								}
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp20_ = serializable_gee_tree_map_test_ball_new ();
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								b = _tmp20_;
#line 327 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp21_ = b;
#line 327 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp22_ = j;
#line 327 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp23_ = g_strdup_printf ("%i", _tmp22_);
#line 327 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp24_ = _tmp23_;
#line 327 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp25_ = g_strconcat ("Ball", _tmp24_, NULL);
#line 327 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp26_ = _tmp25_;
#line 327 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								serializable_gee_tree_map_test_ball_set_name (_tmp21_, _tmp26_);
#line 327 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_g_free0 (_tmp26_);
#line 327 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_g_free0 (_tmp24_);
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp27_ = b;
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp28_ = serializable_gee_tree_map_test_ball_fill_new ();
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp29_ = _tmp28_;
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								serializable_gee_tree_map_test_ball_set_ballfill (_tmp27_, _tmp29_);
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_g_object_unref0 (_tmp29_);
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp30_ = b;
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp31_ = serializable_gee_tree_map_test_ball_get_ballfill (_tmp30_);
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp32_ = _tmp31_;
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								serializable_gee_tree_map_test_ball_fill_set_text (_tmp32_, "golden dust");
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp33_ = sbag;
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp34_ = serializable_gee_tree_map_test_small_bag_get_balls (_tmp33_);
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp35_ = _tmp34_;
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp36_ = b;
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp37_ = serializable_gee_tree_map_test_ball_get_name (_tmp36_);
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp38_ = _tmp37_;
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_tmp39_ = b;
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								gee_abstract_map_set ((GeeAbstractMap*) _tmp35_, _tmp38_, _tmp39_);
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
								_g_object_unref0 (b);
#line 3647 "SerializableGeeTreeMapTest.c"
							}
						}
					}
#line 332 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp40_ = sbag;
#line 332 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp41_ = serializable_gee_tree_map_test_small_bag_get_balls (_tmp40_);
#line 332 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp42_ = _tmp41_;
#line 332 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp43_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp42_);
#line 332 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp44_ = _tmp43_;
#line 332 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_vala_assert (_tmp44_ == 2, "sbag.balls.size == 2");
#line 333 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp45_ = bag;
#line 333 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp46_ = serializable_gee_tree_map_test_big_bag_get_bags (_tmp45_);
#line 333 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp47_ = _tmp46_;
#line 333 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp48_ = sbag;
#line 333 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp49_ = serializable_gee_tree_map_test_small_bag_get_name (_tmp48_);
#line 333 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp50_ = _tmp49_;
#line 333 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_tmp51_ = sbag;
#line 333 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					gee_abstract_map_set ((GeeAbstractMap*) _tmp47_, _tmp50_, _tmp51_);
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
					_g_object_unref0 (sbag);
#line 3681 "SerializableGeeTreeMapTest.c"
				}
			}
		}
#line 335 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp52_ = bag;
#line 335 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp53_ = serializable_gee_tree_map_test_big_bag_get_bags (_tmp52_);
#line 335 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp54_ = _tmp53_;
#line 335 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp55_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp54_);
#line 335 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp56_ = _tmp55_;
#line 335 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp56_ == 2, "bag.bags.size == 2");
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp57_ = gxml_gdocument_new ();
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		d = _tmp57_;
#line 338 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp58_ = bag;
#line 338 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp59_ = d;
#line 338 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp60_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) _tmp58_, (GXmlNode*) _tmp59_, &_inner_error_);
#line 338 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp61_ = _tmp60_;
#line 338 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp61_);
#line 338 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 338 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (d);
#line 338 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bag);
#line 3717 "SerializableGeeTreeMapTest.c"
			goto __catch76_g_error;
		}
#line 339 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp62_ = d;
#line 339 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp63_ = gxml_document_get_root ((GXmlDocument*) _tmp62_);
#line 339 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp64_ = _tmp63_;
#line 339 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp65_ = _tmp64_;
#line 339 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp65_ != NULL, "d.root != null");
#line 339 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp65_);
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp66_ = d;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp67_ = gxml_document_get_root ((GXmlDocument*) _tmp66_);
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp68_ = _tmp67_;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp69_ = _tmp68_;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp70_ = gxml_node_get_name (_tmp69_);
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp71_ = _tmp70_;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp72_ = _tmp71_;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp72_, "BigBag") == 0, "d.root.name == \"BigBag\"");
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_free0 (_tmp72_);
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp69_);
#line 341 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp73_ = d;
#line 341 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp74_ = gxml_document_get_root ((GXmlDocument*) _tmp73_);
#line 341 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp75_ = _tmp74_;
#line 341 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp76_ = _tmp75_;
#line 341 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp77_ = gxml_node_get_children_nodes (_tmp76_);
#line 341 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp78_ = _tmp77_;
#line 341 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp79_ = _tmp78_;
#line 341 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp80_ = gee_collection_get_size ((GeeCollection*) _tmp79_);
#line 341 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp81_ = _tmp80_;
#line 341 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp81_ == 2, "d.root.children_nodes.size == 2");
#line 341 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp79_);
#line 341 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp76_);
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp82_ = d;
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp83_ = gxml_document_get_root ((GXmlDocument*) _tmp82_);
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp84_ = _tmp83_;
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp85_ = _tmp84_;
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp86_ = gxml_node_get_children_nodes (_tmp85_);
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp87_ = _tmp86_;
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp88_ = _tmp87_;
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp89_ = gee_list_get ((GeeList*) _tmp88_, 0);
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp90_ = (GXmlNode*) _tmp89_;
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp91_ = gxml_node_get_name (_tmp90_);
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp92_ = _tmp91_;
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp93_ = _tmp92_;
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp93_, "SmallBag") == 0, "d.root.children_nodes[0].name == \"SmallBag\"");
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_free0 (_tmp93_);
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp90_);
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp88_);
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp85_);
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp94_ = d;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp95_ = gxml_document_get_root ((GXmlDocument*) _tmp94_);
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp96_ = _tmp95_;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp97_ = _tmp96_;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp98_ = gxml_node_get_children_nodes (_tmp97_);
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp99_ = _tmp98_;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp100_ = _tmp99_;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp101_ = gee_list_get ((GeeList*) _tmp100_, 0);
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp102_ = (GXmlNode*) _tmp101_;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp103_ = gxml_node_get_children_nodes (_tmp102_);
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp104_ = _tmp103_;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp105_ = _tmp104_;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp106_ = gee_collection_get_size ((GeeCollection*) _tmp105_);
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp107_ = _tmp106_;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp107_ == 2, "d.root.children_nodes[0].children_nodes.size == 2");
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp105_);
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp102_);
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp100_);
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp97_);
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp108_ = d;
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp109_ = gxml_document_get_root ((GXmlDocument*) _tmp108_);
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp110_ = _tmp109_;
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp111_ = _tmp110_;
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp112_ = gxml_node_get_children_nodes (_tmp111_);
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp113_ = _tmp112_;
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp114_ = _tmp113_;
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp115_ = gee_list_get ((GeeList*) _tmp114_, 0);
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp116_ = (GXmlNode*) _tmp115_;
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp117_ = gxml_node_get_children_nodes (_tmp116_);
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp118_ = _tmp117_;
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp119_ = _tmp118_;
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp120_ = gee_list_get ((GeeList*) _tmp119_, 0);
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp121_ = (GXmlNode*) _tmp120_;
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp122_ = gxml_node_get_name (_tmp121_);
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp123_ = _tmp122_;
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp124_ = _tmp123_;
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp124_, "Ball") == 0, "d.root.children_nodes[0].children_nodes[0].name == \"Ball\"");
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_free0 (_tmp124_);
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp121_);
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp119_);
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp116_);
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp114_);
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp111_);
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp125_ = d;
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp126_ = gxml_document_get_root ((GXmlDocument*) _tmp125_);
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp127_ = _tmp126_;
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp128_ = _tmp127_;
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp129_ = gxml_node_get_children_nodes (_tmp128_);
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp130_ = _tmp129_;
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp131_ = _tmp130_;
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp132_ = gee_list_get ((GeeList*) _tmp131_, 0);
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp133_ = (GXmlNode*) _tmp132_;
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp134_ = gxml_node_get_children_nodes (_tmp133_);
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp135_ = _tmp134_;
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp136_ = _tmp135_;
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp137_ = gee_list_get ((GeeList*) _tmp136_, 0);
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp138_ = (GXmlNode*) _tmp137_;
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp139_ = gxml_node_get_children_nodes (_tmp138_);
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp140_ = _tmp139_;
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp141_ = _tmp140_;
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp142_ = gee_collection_get_size ((GeeCollection*) _tmp141_);
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp143_ = _tmp142_;
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp143_ == 1, "d.root.children_nodes[0].children_nodes[0].children_nodes.size == 1");
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp141_);
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp138_);
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp136_);
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp133_);
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp131_);
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp128_);
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp144_ = d;
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp145_ = gxml_document_get_root ((GXmlDocument*) _tmp144_);
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp146_ = _tmp145_;
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp147_ = _tmp146_;
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp148_ = gxml_node_get_children_nodes (_tmp147_);
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp149_ = _tmp148_;
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp150_ = _tmp149_;
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp151_ = gee_list_get ((GeeList*) _tmp150_, 0);
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp152_ = (GXmlNode*) _tmp151_;
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp153_ = gxml_node_get_children_nodes (_tmp152_);
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp154_ = _tmp153_;
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp155_ = _tmp154_;
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp156_ = gee_list_get ((GeeList*) _tmp155_, 0);
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp157_ = (GXmlNode*) _tmp156_;
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp158_ = gxml_node_get_children_nodes (_tmp157_);
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp159_ = _tmp158_;
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp160_ = _tmp159_;
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp161_ = gee_list_get ((GeeList*) _tmp160_, 0);
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp162_ = (GXmlNode*) _tmp161_;
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp163_ = gxml_node_get_name (_tmp162_);
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp164_ = _tmp163_;
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp165_ = _tmp164_;
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp165_, "BallFill") == 0, "d.root.children_nodes[0].children_nodes[0].children_nodes[0].name == \"BallFill\"");
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_free0 (_tmp165_);
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp162_);
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp160_);
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp157_);
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp155_);
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp152_);
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp150_);
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp147_);
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp166_ = d;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp167_ = gxml_document_get_root ((GXmlDocument*) _tmp166_);
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp168_ = _tmp167_;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp169_ = _tmp168_;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp170_ = gxml_node_get_children_nodes (_tmp169_);
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp171_ = _tmp170_;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp172_ = _tmp171_;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp173_ = gee_list_get ((GeeList*) _tmp172_, 0);
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp174_ = (GXmlNode*) _tmp173_;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp175_ = gxml_node_get_children_nodes (_tmp174_);
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp176_ = _tmp175_;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp177_ = _tmp176_;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp178_ = gee_list_get ((GeeList*) _tmp177_, 0);
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp179_ = (GXmlNode*) _tmp178_;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp180_ = gxml_node_get_children_nodes (_tmp179_);
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp181_ = _tmp180_;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp182_ = _tmp181_;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp183_ = gee_list_get ((GeeList*) _tmp182_, 0);
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp184_ = (GXmlNode*) _tmp183_;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp185_ = gxml_node_get_children_nodes (_tmp184_);
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp186_ = _tmp185_;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp187_ = _tmp186_;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp188_ = gee_collection_get_size ((GeeCollection*) _tmp187_);
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp189_ = _tmp188_;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp189_ == 1, "d.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes.size == 1");
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp187_);
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp184_);
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp182_);
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp179_);
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp177_);
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp174_);
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp172_);
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp169_);
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp190_ = d;
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp191_ = gxml_document_get_root ((GXmlDocument*) _tmp190_);
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp192_ = _tmp191_;
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp193_ = _tmp192_;
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp194_ = gxml_node_get_children_nodes (_tmp193_);
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp195_ = _tmp194_;
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp196_ = _tmp195_;
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp197_ = gee_list_get ((GeeList*) _tmp196_, 0);
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp198_ = (GXmlNode*) _tmp197_;
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp199_ = gxml_node_get_children_nodes (_tmp198_);
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp200_ = _tmp199_;
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp201_ = _tmp200_;
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp202_ = gee_list_get ((GeeList*) _tmp201_, 0);
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp203_ = (GXmlNode*) _tmp202_;
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp204_ = gxml_node_get_children_nodes (_tmp203_);
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp205_ = _tmp204_;
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp206_ = _tmp205_;
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp207_ = gee_list_get ((GeeList*) _tmp206_, 0);
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp208_ = (GXmlNode*) _tmp207_;
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp209_ = gxml_node_get_children_nodes (_tmp208_);
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp210_ = _tmp209_;
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp211_ = _tmp210_;
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp212_ = gee_list_get ((GeeList*) _tmp211_, 0);
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp213_ = (GXmlNode*) _tmp212_;
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp213_, GXML_TYPE_TEXT), "d.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes[0] is Text");
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp213_);
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp211_);
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp208_);
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp206_);
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp203_);
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp201_);
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp198_);
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp196_);
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp193_);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp214_ = d;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp215_ = gxml_document_get_root ((GXmlDocument*) _tmp214_);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp216_ = _tmp215_;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp217_ = _tmp216_;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp218_ = gxml_node_get_children_nodes (_tmp217_);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp219_ = _tmp218_;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp220_ = _tmp219_;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp221_ = gee_list_get ((GeeList*) _tmp220_, 0);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp222_ = (GXmlNode*) _tmp221_;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp223_ = gxml_node_get_children_nodes (_tmp222_);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp224_ = _tmp223_;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp225_ = _tmp224_;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp226_ = gee_list_get ((GeeList*) _tmp225_, 0);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp227_ = (GXmlNode*) _tmp226_;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp228_ = gxml_node_get_children_nodes (_tmp227_);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp229_ = _tmp228_;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp230_ = _tmp229_;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp231_ = gee_list_get ((GeeList*) _tmp230_, 0);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp232_ = (GXmlNode*) _tmp231_;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp233_ = gxml_node_get_children_nodes (_tmp232_);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp234_ = _tmp233_;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp235_ = _tmp234_;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp236_ = gee_list_get ((GeeList*) _tmp235_, 0);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp237_ = (GXmlNode*) _tmp236_;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp238_ = gxml_node_get_value (_tmp237_);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp239_ = _tmp238_;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp240_ = _tmp239_;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp240_, "golden dust") == 0, "d.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes[0].value == \"golden dust\"");
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_free0 (_tmp240_);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp237_);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp235_);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp232_);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp230_);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp227_);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp225_);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp222_);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp220_);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp217_);
#line 352 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp241_ = serializable_gee_tree_map_test_big_bag_new ();
#line 352 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		bagt = _tmp241_;
#line 353 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp242_ = bagt;
#line 353 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp243_ = d;
#line 353 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp242_, (GXmlNode*) _tmp243_, &_inner_error_);
#line 353 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 353 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bagt);
#line 353 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (d);
#line 353 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bag);
#line 4238 "SerializableGeeTreeMapTest.c"
			goto __catch76_g_error;
		}
#line 354 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp244_ = bagt;
#line 354 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp245_ = serializable_gee_tree_map_test_big_bag_get_bags (_tmp244_);
#line 354 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp246_ = _tmp245_;
#line 354 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp247_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp246_);
#line 354 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp248_ = _tmp247_;
#line 354 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp248_ == 0, "bagt.bags.size == 0");
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp250_ = bagt;
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp251_ = serializable_gee_tree_map_test_big_bag_get_bags (_tmp250_);
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp252_ = _tmp251_;
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp253_ = gxml_serializable_tree_map_deserialize_children ((GXmlSerializableTreeMap*) _tmp252_, &_inner_error_);
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp249_ = _tmp253_;
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bagt);
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (d);
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bag);
#line 4271 "SerializableGeeTreeMapTest.c"
			goto __catch76_g_error;
		}
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp249_, "bagt.bags.deserialize_children ()");
#line 356 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp254_ = bagt;
#line 356 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp255_ = serializable_gee_tree_map_test_big_bag_get_bags (_tmp254_);
#line 356 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp256_ = _tmp255_;
#line 356 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp257_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp256_);
#line 356 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp258_ = _tmp257_;
#line 356 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp258_ == 2, "bagt.bags.size == 2");
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp260_ = bagt;
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp261_ = serializable_gee_tree_map_test_big_bag_get_bags (_tmp260_);
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp262_ = _tmp261_;
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp263_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp262_, "SmallBag1");
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp264_ = (SerializableGeeTreeMapTestSmallBag*) _tmp263_;
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp265_ = serializable_gee_tree_map_test_small_bag_get_balls (_tmp264_);
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp266_ = _tmp265_;
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp267_ = gxml_serializable_tree_map_deserialize_children ((GXmlSerializableTreeMap*) _tmp266_, &_inner_error_);
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp268_ = _tmp267_;
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp264_);
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp259_ = _tmp268_;
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bagt);
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (d);
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bag);
#line 4318 "SerializableGeeTreeMapTest.c"
			goto __catch76_g_error;
		}
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp259_, "bagt.bags.get(\"SmallBag1\").balls.deserialize_children ()");
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp269_ = bagt;
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp270_ = serializable_gee_tree_map_test_big_bag_get_bags (_tmp269_);
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp271_ = _tmp270_;
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp272_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp271_, "SmallBag1");
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp273_ = (SerializableGeeTreeMapTestSmallBag*) _tmp272_;
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp274_ = serializable_gee_tree_map_test_small_bag_get_balls (_tmp273_);
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp275_ = _tmp274_;
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp276_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp275_);
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp277_ = _tmp276_;
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp277_ == 2, "bagt.bags.get(\"SmallBag1\").balls.size == 2");
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp273_);
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp278_ = bagt;
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp279_ = serializable_gee_tree_map_test_big_bag_get_bags (_tmp278_);
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp280_ = _tmp279_;
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp281_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp280_, "SmallBag1");
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp282_ = (SerializableGeeTreeMapTestSmallBag*) _tmp281_;
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp283_ = serializable_gee_tree_map_test_small_bag_get_balls (_tmp282_);
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp284_ = _tmp283_;
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp285_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp284_, "Ball1");
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp286_ = (SerializableGeeTreeMapTestBall*) _tmp285_;
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp287_ = serializable_gee_tree_map_test_ball_get_name (_tmp286_);
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp288_ = _tmp287_;
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp288_, "Ball1") == 0, "bagt.bags.get(\"SmallBag1\").balls.get(\"Ball1\").name == \"Ball1\"");
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp286_);
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp282_);
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp289_ = bagt;
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp290_ = serializable_gee_tree_map_test_big_bag_get_bags (_tmp289_);
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp291_ = _tmp290_;
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp292_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp291_, "SmallBag1");
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp293_ = (SerializableGeeTreeMapTestSmallBag*) _tmp292_;
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp294_ = serializable_gee_tree_map_test_small_bag_get_balls (_tmp293_);
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp295_ = _tmp294_;
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp296_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp295_, "Ball1");
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp297_ = (SerializableGeeTreeMapTestBall*) _tmp296_;
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp298_ = serializable_gee_tree_map_test_ball_get_ballfill (_tmp297_);
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp299_ = _tmp298_;
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp299_ != NULL, "bagt.bags.get(\"SmallBag1\").balls.get(\"Ball1\").ballfill !=null");
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp297_);
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp293_);
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp300_ = bagt;
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp301_ = serializable_gee_tree_map_test_big_bag_get_bags (_tmp300_);
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp302_ = _tmp301_;
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp303_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp302_, "SmallBag1");
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp304_ = (SerializableGeeTreeMapTestSmallBag*) _tmp303_;
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp305_ = serializable_gee_tree_map_test_small_bag_get_balls (_tmp304_);
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp306_ = _tmp305_;
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp307_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp306_, "Ball1");
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp308_ = (SerializableGeeTreeMapTestBall*) _tmp307_;
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp309_ = serializable_gee_tree_map_test_ball_get_ballfill (_tmp308_);
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp310_ = _tmp309_;
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp311_ = gxml_serializable_get_serialized_xml_node_value ((GXmlSerializable*) _tmp310_);
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp312_ = _tmp311_;
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp313_ = _tmp312_;
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp313_ != NULL, "bagt.bags.get(\"SmallBag1\").balls.get(\"Ball1\").ballfill.serialized_xml_node_value !=null");
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_free0 (_tmp313_);
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp308_);
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp304_);
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp314_ = bagt;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp315_ = serializable_gee_tree_map_test_big_bag_get_bags (_tmp314_);
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp316_ = _tmp315_;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp317_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp316_, "SmallBag1");
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp318_ = (SerializableGeeTreeMapTestSmallBag*) _tmp317_;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp319_ = serializable_gee_tree_map_test_small_bag_get_balls (_tmp318_);
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp320_ = _tmp319_;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp321_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp320_, "Ball1");
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp322_ = (SerializableGeeTreeMapTestBall*) _tmp321_;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp323_ = serializable_gee_tree_map_test_ball_get_ballfill (_tmp322_);
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp324_ = _tmp323_;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp325_ = gxml_serializable_get_serialized_xml_node_value ((GXmlSerializable*) _tmp324_);
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp326_ = _tmp325_;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp327_ = _tmp326_;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp327_, "golden dust") == 0, "bagt.bags.get(\"SmallBag1\").balls.get(\"Ball1\").ballfill.serialized_xml_node_value ==\"golden dust\"");
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_free0 (_tmp327_);
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp322_);
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp318_);
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp328_ = serializable_gee_tree_map_test_big_bag_new ();
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		bag3 = _tmp328_;
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp329_ = bag3;
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp330_ = d;
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp329_, (GXmlNode*) _tmp330_, &_inner_error_);
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bag3);
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bagt);
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (d);
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bag);
#line 4493 "SerializableGeeTreeMapTest.c"
			goto __catch76_g_error;
		}
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp331_ = bag3;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp332_ = serializable_gee_tree_map_test_big_bag_get_bags (_tmp331_);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp333_ = _tmp332_;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp334_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp333_);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp335_ = _tmp334_;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp335_ == 0, "bag3.bags.size == 0");
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp337_ = bag3;
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp338_ = serializable_gee_tree_map_test_big_bag_get_bags (_tmp337_);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp339_ = _tmp338_;
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp340_ = gxml_serializable_tree_map_deserialize_children ((GXmlSerializableTreeMap*) _tmp339_, &_inner_error_);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp336_ = _tmp340_;
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bag3);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bagt);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (d);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bag);
#line 4528 "SerializableGeeTreeMapTest.c"
			goto __catch76_g_error;
		}
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp336_, "bag3.bags.deserialize_children ()");
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp341_ = bag3;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp342_ = serializable_gee_tree_map_test_big_bag_get_bags (_tmp341_);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp343_ = _tmp342_;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp344_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp343_);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp345_ = _tmp344_;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp345_ == 2, "bag3.bags.size == 2");
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp346_ = bag3;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp347_ = d;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp346_, (GXmlNode*) _tmp347_, &_inner_error_);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bag3);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bagt);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (d);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bag);
#line 4561 "SerializableGeeTreeMapTest.c"
			goto __catch76_g_error;
		}
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp348_ = bag3;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp349_ = serializable_gee_tree_map_test_big_bag_get_bags (_tmp348_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp350_ = _tmp349_;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp351_ = gxml_serializable_tree_map_deserialized ((GXmlSerializableTreeMap*) _tmp350_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (!_tmp351_, "!bag3.bags.deserialized ()");
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp352_ = serializable_gee_tree_map_test_big_bag_new ();
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		bag2 = _tmp352_;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp353_ = bag2;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp354_ = d;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp353_, (GXmlNode*) _tmp354_, &_inner_error_);
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bag2);
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bag3);
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bagt);
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (d);
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bag);
#line 4596 "SerializableGeeTreeMapTest.c"
			goto __catch76_g_error;
		}
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp355_ = bag2;
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp356_ = serializable_gee_tree_map_test_big_bag_get_bags (_tmp355_);
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp357_ = _tmp356_;
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp358_ = gxml_serializable_tree_map_deserialized ((GXmlSerializableTreeMap*) _tmp357_);
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (!_tmp358_, "!bag2.bags.deserialized ()");
#line 374 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp359_ = gxml_gdocument_new ();
#line 374 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		d2 = _tmp359_;
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp360_ = bag2;
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp361_ = d2;
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp362_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) _tmp360_, (GXmlNode*) _tmp361_, &_inner_error_);
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp363_ = _tmp362_;
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp363_);
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (d2);
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bag2);
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bag3);
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bagt);
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (d);
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
			_g_object_unref0 (bag);
#line 4637 "SerializableGeeTreeMapTest.c"
			goto __catch76_g_error;
		}
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp364_ = d2;
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp365_ = gxml_document_get_root ((GXmlDocument*) _tmp364_);
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp366_ = _tmp365_;
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp367_ = _tmp366_;
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp367_ != NULL, "d2.root != null");
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp367_);
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp368_ = d2;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp369_ = gxml_document_get_root ((GXmlDocument*) _tmp368_);
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp370_ = _tmp369_;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp371_ = _tmp370_;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp372_ = gxml_node_get_name (_tmp371_);
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp373_ = _tmp372_;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp374_ = _tmp373_;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp374_, "BigBag") == 0, "d2.root.name == \"BigBag\"");
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_free0 (_tmp374_);
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp371_);
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp375_ = d2;
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp376_ = gxml_document_get_root ((GXmlDocument*) _tmp375_);
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp377_ = _tmp376_;
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp378_ = _tmp377_;
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp379_ = gxml_node_get_children_nodes (_tmp378_);
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp380_ = _tmp379_;
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp381_ = _tmp380_;
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp382_ = gee_collection_get_size ((GeeCollection*) _tmp381_);
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp383_ = _tmp382_;
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp383_ == 2, "d2.root.children_nodes.size == 2");
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp381_);
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp378_);
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp384_ = d2;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp385_ = gxml_document_get_root ((GXmlDocument*) _tmp384_);
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp386_ = _tmp385_;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp387_ = _tmp386_;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp388_ = gxml_node_get_children_nodes (_tmp387_);
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp389_ = _tmp388_;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp390_ = _tmp389_;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp391_ = gee_list_get ((GeeList*) _tmp390_, 0);
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp392_ = (GXmlNode*) _tmp391_;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp393_ = gxml_node_get_name (_tmp392_);
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp394_ = _tmp393_;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp395_ = _tmp394_;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp395_, "SmallBag") == 0, "d2.root.children_nodes[0].name == \"SmallBag\"");
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_free0 (_tmp395_);
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp392_);
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp390_);
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp387_);
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp396_ = d2;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp397_ = gxml_document_get_root ((GXmlDocument*) _tmp396_);
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp398_ = _tmp397_;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp399_ = _tmp398_;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp400_ = gxml_node_get_children_nodes (_tmp399_);
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp401_ = _tmp400_;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp402_ = _tmp401_;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp403_ = gee_list_get ((GeeList*) _tmp402_, 0);
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp404_ = (GXmlNode*) _tmp403_;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp405_ = gxml_node_get_children_nodes (_tmp404_);
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp406_ = _tmp405_;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp407_ = _tmp406_;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp408_ = gee_collection_get_size ((GeeCollection*) _tmp407_);
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp409_ = _tmp408_;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp409_ == 2, "d2.root.children_nodes[0].children_nodes.size == 2");
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp407_);
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp404_);
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp402_);
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp399_);
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp410_ = d2;
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp411_ = gxml_document_get_root ((GXmlDocument*) _tmp410_);
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp412_ = _tmp411_;
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp413_ = _tmp412_;
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp414_ = gxml_node_get_children_nodes (_tmp413_);
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp415_ = _tmp414_;
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp416_ = _tmp415_;
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp417_ = gee_list_get ((GeeList*) _tmp416_, 0);
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp418_ = (GXmlNode*) _tmp417_;
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp419_ = gxml_node_get_children_nodes (_tmp418_);
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp420_ = _tmp419_;
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp421_ = _tmp420_;
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp422_ = gee_list_get ((GeeList*) _tmp421_, 0);
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp423_ = (GXmlNode*) _tmp422_;
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp424_ = gxml_node_get_name (_tmp423_);
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp425_ = _tmp424_;
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp426_ = _tmp425_;
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp426_, "Ball") == 0, "d2.root.children_nodes[0].children_nodes[0].name == \"Ball\"");
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_free0 (_tmp426_);
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp423_);
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp421_);
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp418_);
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp416_);
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp413_);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp427_ = d2;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp428_ = gxml_document_get_root ((GXmlDocument*) _tmp427_);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp429_ = _tmp428_;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp430_ = _tmp429_;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp431_ = gxml_node_get_children_nodes (_tmp430_);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp432_ = _tmp431_;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp433_ = _tmp432_;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp434_ = gee_list_get ((GeeList*) _tmp433_, 0);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp435_ = (GXmlNode*) _tmp434_;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp436_ = gxml_node_get_children_nodes (_tmp435_);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp437_ = _tmp436_;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp438_ = _tmp437_;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp439_ = gee_list_get ((GeeList*) _tmp438_, 0);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp440_ = (GXmlNode*) _tmp439_;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp441_ = gxml_node_get_children_nodes (_tmp440_);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp442_ = _tmp441_;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp443_ = _tmp442_;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp444_ = gee_collection_get_size ((GeeCollection*) _tmp443_);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp445_ = _tmp444_;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp445_ == 1, "d2.root.children_nodes[0].children_nodes[0].children_nodes.size == 1");
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp443_);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp440_);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp438_);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp435_);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp433_);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp430_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp446_ = d2;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp447_ = gxml_document_get_root ((GXmlDocument*) _tmp446_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp448_ = _tmp447_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp449_ = _tmp448_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp450_ = gxml_node_get_children_nodes (_tmp449_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp451_ = _tmp450_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp452_ = _tmp451_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp453_ = gee_list_get ((GeeList*) _tmp452_, 0);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp454_ = (GXmlNode*) _tmp453_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp455_ = gxml_node_get_children_nodes (_tmp454_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp456_ = _tmp455_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp457_ = _tmp456_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp458_ = gee_list_get ((GeeList*) _tmp457_, 0);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp459_ = (GXmlNode*) _tmp458_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp460_ = gxml_node_get_children_nodes (_tmp459_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp461_ = _tmp460_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp462_ = _tmp461_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp463_ = gee_list_get ((GeeList*) _tmp462_, 0);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp464_ = (GXmlNode*) _tmp463_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp465_ = gxml_node_get_name (_tmp464_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp466_ = _tmp465_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp467_ = _tmp466_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp467_, "BallFill") == 0, "d2.root.children_nodes[0].children_nodes[0].children_nodes[0].name == \"BallFill\"");
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_free0 (_tmp467_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp464_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp462_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp459_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp457_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp454_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp452_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp449_);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp468_ = d2;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp469_ = gxml_document_get_root ((GXmlDocument*) _tmp468_);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp470_ = _tmp469_;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp471_ = _tmp470_;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp472_ = gxml_node_get_children_nodes (_tmp471_);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp473_ = _tmp472_;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp474_ = _tmp473_;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp475_ = gee_list_get ((GeeList*) _tmp474_, 0);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp476_ = (GXmlNode*) _tmp475_;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp477_ = gxml_node_get_children_nodes (_tmp476_);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp478_ = _tmp477_;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp479_ = _tmp478_;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp480_ = gee_list_get ((GeeList*) _tmp479_, 0);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp481_ = (GXmlNode*) _tmp480_;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp482_ = gxml_node_get_children_nodes (_tmp481_);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp483_ = _tmp482_;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp484_ = _tmp483_;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp485_ = gee_list_get ((GeeList*) _tmp484_, 0);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp486_ = (GXmlNode*) _tmp485_;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp487_ = gxml_node_get_children_nodes (_tmp486_);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp488_ = _tmp487_;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp489_ = _tmp488_;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp490_ = gee_collection_get_size ((GeeCollection*) _tmp489_);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp491_ = _tmp490_;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (_tmp491_ == 1, "d2.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes.size == 1");
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp489_);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp486_);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp484_);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp481_);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp479_);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp476_);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp474_);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp471_);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp492_ = d2;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp493_ = gxml_document_get_root ((GXmlDocument*) _tmp492_);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp494_ = _tmp493_;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp495_ = _tmp494_;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp496_ = gxml_node_get_children_nodes (_tmp495_);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp497_ = _tmp496_;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp498_ = _tmp497_;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp499_ = gee_list_get ((GeeList*) _tmp498_, 0);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp500_ = (GXmlNode*) _tmp499_;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp501_ = gxml_node_get_children_nodes (_tmp500_);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp502_ = _tmp501_;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp503_ = _tmp502_;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp504_ = gee_list_get ((GeeList*) _tmp503_, 0);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp505_ = (GXmlNode*) _tmp504_;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp506_ = gxml_node_get_children_nodes (_tmp505_);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp507_ = _tmp506_;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp508_ = _tmp507_;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp509_ = gee_list_get ((GeeList*) _tmp508_, 0);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp510_ = (GXmlNode*) _tmp509_;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp511_ = gxml_node_get_children_nodes (_tmp510_);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp512_ = _tmp511_;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp513_ = _tmp512_;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp514_ = gee_list_get ((GeeList*) _tmp513_, 0);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp515_ = (GXmlNode*) _tmp514_;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp515_, GXML_TYPE_TEXT), "d2.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes[0] is GXml.Text");
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp515_);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp513_);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp510_);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp508_);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp505_);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp503_);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp500_);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp498_);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp495_);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp516_ = d2;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp517_ = gxml_document_get_root ((GXmlDocument*) _tmp516_);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp518_ = _tmp517_;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp519_ = _tmp518_;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp520_ = gxml_node_get_children_nodes (_tmp519_);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp521_ = _tmp520_;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp522_ = _tmp521_;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp523_ = gee_list_get ((GeeList*) _tmp522_, 0);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp524_ = (GXmlNode*) _tmp523_;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp525_ = gxml_node_get_children_nodes (_tmp524_);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp526_ = _tmp525_;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp527_ = _tmp526_;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp528_ = gee_list_get ((GeeList*) _tmp527_, 0);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp529_ = (GXmlNode*) _tmp528_;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp530_ = gxml_node_get_children_nodes (_tmp529_);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp531_ = _tmp530_;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp532_ = _tmp531_;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp533_ = gee_list_get ((GeeList*) _tmp532_, 0);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp534_ = (GXmlNode*) _tmp533_;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp535_ = gxml_node_get_children_nodes (_tmp534_);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp536_ = _tmp535_;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp537_ = _tmp536_;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp538_ = gee_list_get ((GeeList*) _tmp537_, 0);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp539_ = (GXmlNode*) _tmp538_;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp540_ = gxml_node_get_value (_tmp539_);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp541_ = _tmp540_;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp542_ = _tmp541_;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp542_, "golden dust") == 0, "d2.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes[0].value == \"golden dust\"");
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_free0 (_tmp542_);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp539_);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp537_);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp534_);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp532_);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp529_);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp527_);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp524_);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp522_);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp519_);
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (d2);
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (bag2);
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (bag3);
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (bagt);
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (d);
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (bag);
#line 5152 "SerializableGeeTreeMapTest.c"
	}
	goto __finally76;
	__catch76_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp543_ = NULL;
		gchar* _tmp544_ = NULL;
		gchar* _tmp545_ = NULL;
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		e = _inner_error_;
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_inner_error_ = NULL;
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp543_ = e->message;
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp544_ = g_strconcat ("ERROR: ", _tmp543_, NULL);
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp545_ = _tmp544_;
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_message ("SerializableGeeTreeMapTest.vala:389: %s", _tmp545_);
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_free0 (_tmp545_);
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_assert_not_reached ();
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_error_free0 (e);
#line 5179 "SerializableGeeTreeMapTest.c"
	}
	__finally76:
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_clear_error (&_inner_error_);
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		return;
#line 5190 "SerializableGeeTreeMapTest.c"
	}
}


static void ___lambda45__gtest_func (void) {
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	__lambda45_ ();
#line 5198 "SerializableGeeTreeMapTest.c"
}


void serializable_gee_tree_map_test_add_tests (void) {
#line 99 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_test_add_func ("/gxml/serializable/treemap/api", ___lambda39__gtest_func);
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_test_add_func ("/gxml/serializable/treemap/serialize", ___lambda40__gtest_func);
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_test_add_func ("/gxml/serializable/treemap/deserialize", ___lambda41__gtest_func);
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_test_add_func ("/gxml/serializable/treemap/container_class/deserialize", ___lambda42__gtest_func);
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_test_add_func ("/gxml/serializable/treemap/containder_class/serialize", ___lambda43__gtest_func);
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_test_add_func ("/gxml/serializable/treemap/deserialize-node-names", ___lambda44__gtest_func);
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_test_add_func ("/gxml/serializable/treemap/post-deserialization/serialize/contents", ___lambda45__gtest_func);
#line 5217 "SerializableGeeTreeMapTest.c"
}


SerializableGeeTreeMapTest* serializable_gee_tree_map_test_construct (GType object_type) {
	SerializableGeeTreeMapTest* self = NULL;
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTest*) gxml_test_construct (object_type);
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return self;
#line 5227 "SerializableGeeTreeMapTest.c"
}


SerializableGeeTreeMapTest* serializable_gee_tree_map_test_new (void) {
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return serializable_gee_tree_map_test_construct (TYPE_SERIALIZABLE_GEE_TREE_MAP_TEST);
#line 5234 "SerializableGeeTreeMapTest.c"
}


static gchar* serializable_gee_tree_map_test_space_real_get_map_key (GXmlSerializableMapKey* base) {
	SerializableGeeTreeMapTestSpace * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestSpace*) base;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp1_;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 5253 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestSpace* serializable_gee_tree_map_test_space_construct_named (GType object_type, const gchar* name) {
	SerializableGeeTreeMapTestSpace * self = NULL;
	const gchar* _tmp0_ = NULL;
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestSpace*) gxml_serializable_object_model_construct (object_type);
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = name;
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	serializable_gee_tree_map_test_space_set_name (self, _tmp0_);
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return self;
#line 5270 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestSpace* serializable_gee_tree_map_test_space_new_named (const gchar* name) {
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return serializable_gee_tree_map_test_space_construct_named (SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE, name);
#line 5277 "SerializableGeeTreeMapTest.c"
}


static gchar* serializable_gee_tree_map_test_space_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeTreeMapTestSpace * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 36 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestSpace*) base;
#line 36 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = g_strdup ("space");
#line 36 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp0_;
#line 36 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 5293 "SerializableGeeTreeMapTest.c"
}


static gchar* serializable_gee_tree_map_test_space_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeTreeMapTestSpace * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 37 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestSpace*) base;
#line 37 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 37 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 37 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp1_;
#line 37 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 5312 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestSpace* serializable_gee_tree_map_test_space_construct (GType object_type) {
	SerializableGeeTreeMapTestSpace * self = NULL;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestSpace*) gxml_serializable_object_model_construct (object_type);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return self;
#line 5322 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestSpace* serializable_gee_tree_map_test_space_new (void) {
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return serializable_gee_tree_map_test_space_construct (SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE);
#line 5329 "SerializableGeeTreeMapTest.c"
}


static const gchar* serializable_gee_tree_map_test_space_get_name (SerializableGeeTreeMapTestSpace* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp0_;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 5344 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_space_set_name (SerializableGeeTreeMapTestSpace* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_if_fail (self != NULL);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = value;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_free0 (self->priv->_name);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv->_name = _tmp1_;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 5363 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestSpaceCollection* serializable_gee_tree_map_test_space_collection_construct (GType object_type) {
	SerializableGeeTreeMapTestSpaceCollection * self = NULL;
#line 38 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestSpaceCollection*) gxml_serializable_tree_map_construct (object_type, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 38 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return self;
#line 5373 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestSpaceCollection* serializable_gee_tree_map_test_space_collection_new (void) {
#line 38 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return serializable_gee_tree_map_test_space_collection_construct (SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_TYPE_COLLECTION);
#line 5380 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_space_collection_class_init (SerializableGeeTreeMapTestSpaceCollectionClass * klass) {
#line 38 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	serializable_gee_tree_map_test_space_collection_parent_class = g_type_class_peek_parent (klass);
#line 5387 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_space_collection_instance_init (SerializableGeeTreeMapTestSpaceCollection * self) {
}


GType serializable_gee_tree_map_test_space_collection_get_type (void) {
	static volatile gsize serializable_gee_tree_map_test_space_collection_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_tree_map_test_space_collection_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeTreeMapTestSpaceCollectionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_tree_map_test_space_collection_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeTreeMapTestSpaceCollection), 0, (GInstanceInitFunc) serializable_gee_tree_map_test_space_collection_instance_init, NULL };
		GType serializable_gee_tree_map_test_space_collection_type_id;
		serializable_gee_tree_map_test_space_collection_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_TREE_MAP, "SerializableGeeTreeMapTestSpaceCollection", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_tree_map_test_space_collection_type_id__volatile, serializable_gee_tree_map_test_space_collection_type_id);
	}
	return serializable_gee_tree_map_test_space_collection_type_id__volatile;
}


static void serializable_gee_tree_map_test_space_class_init (SerializableGeeTreeMapTestSpaceClass * klass) {
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	serializable_gee_tree_map_test_space_parent_class = g_type_class_peek_parent (klass);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeTreeMapTestSpacePrivate));
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_tree_map_test_space_real_node_name;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_tree_map_test_space_real_to_string;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_tree_map_test_space_get_property;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_tree_map_test_space_set_property;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_tree_map_test_space_finalize;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 5424 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_space_gxml_serializable_map_key_interface_init (GXmlSerializableMapKeyIface * iface) {
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	serializable_gee_tree_map_test_space_gxml_serializable_map_key_parent_iface = g_type_interface_peek_parent (iface);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	iface->get_map_key = (gpointer (*)(GXmlSerializableMapKey*)) serializable_gee_tree_map_test_space_real_get_map_key;
#line 5433 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_space_instance_init (SerializableGeeTreeMapTestSpace * self) {
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv = SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_GET_PRIVATE (self);
#line 5440 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_space_finalize (GObject* obj) {
	SerializableGeeTreeMapTestSpace * self;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE, SerializableGeeTreeMapTestSpace);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_free0 (self->priv->_name);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (serializable_gee_tree_map_test_space_parent_class)->finalize (obj);
#line 5452 "SerializableGeeTreeMapTest.c"
}


static GType serializable_gee_tree_map_test_space_get_type (void) {
	static volatile gsize serializable_gee_tree_map_test_space_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_tree_map_test_space_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeTreeMapTestSpaceClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_tree_map_test_space_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeTreeMapTestSpace), 0, (GInstanceInitFunc) serializable_gee_tree_map_test_space_instance_init, NULL };
		static const GInterfaceInfo gxml_serializable_map_key_info = { (GInterfaceInitFunc) serializable_gee_tree_map_test_space_gxml_serializable_map_key_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType serializable_gee_tree_map_test_space_type_id;
		serializable_gee_tree_map_test_space_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeTreeMapTestSpace", &g_define_type_info, 0);
		g_type_add_interface_static (serializable_gee_tree_map_test_space_type_id, GXML_TYPE_SERIALIZABLE_MAP_KEY, &gxml_serializable_map_key_info);
		g_once_init_leave (&serializable_gee_tree_map_test_space_type_id__volatile, serializable_gee_tree_map_test_space_type_id);
	}
	return serializable_gee_tree_map_test_space_type_id__volatile;
}


static void _vala_serializable_gee_tree_map_test_space_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeTreeMapTestSpace * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE, SerializableGeeTreeMapTestSpace);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	switch (property_id) {
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		case SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_NAME:
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_value_set_string (value, serializable_gee_tree_map_test_space_get_name (self));
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 5481 "SerializableGeeTreeMapTest.c"
		default:
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 5487 "SerializableGeeTreeMapTest.c"
	}
}


static void _vala_serializable_gee_tree_map_test_space_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeTreeMapTestSpace * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE, SerializableGeeTreeMapTestSpace);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	switch (property_id) {
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		case SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_NAME:
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		serializable_gee_tree_map_test_space_set_name (self, g_value_get_string (value));
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 5503 "SerializableGeeTreeMapTest.c"
		default:
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 5509 "SerializableGeeTreeMapTest.c"
	}
}


static gchar* serializable_gee_tree_map_test_space_container_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeTreeMapTestSpaceContainer * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 45 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestSpaceContainer*) base;
#line 45 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = g_strdup ("spacecontainer");
#line 45 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp0_;
#line 45 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 5526 "SerializableGeeTreeMapTest.c"
}


static gchar* serializable_gee_tree_map_test_space_container_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeTreeMapTestSpaceContainer * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestSpaceContainer*) base;
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = self->priv->_owner;
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp1_;
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 5545 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_space_container_real_init_containers (GXmlSerializableContainer* base) {
	SerializableGeeTreeMapTestSpaceContainer * self;
	SerializableGeeTreeMapTestSpaceCollection* _tmp0_ = NULL;
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestSpaceContainer*) base;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = self->priv->_storage;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	if (_tmp0_ == NULL) {
#line 5558 "SerializableGeeTreeMapTest.c"
		SerializableGeeTreeMapTestSpaceCollection* _tmp1_ = NULL;
		SerializableGeeTreeMapTestSpaceCollection* _tmp2_ = NULL;
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp1_ = serializable_gee_tree_map_test_space_collection_new ();
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_tmp2_ = _tmp1_;
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		serializable_gee_tree_map_test_space_container_set_storage (self, _tmp2_);
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		_g_object_unref0 (_tmp2_);
#line 5569 "SerializableGeeTreeMapTest.c"
	}
}


static SerializableGeeTreeMapTestSpaceContainer* serializable_gee_tree_map_test_space_container_construct (GType object_type) {
	SerializableGeeTreeMapTestSpaceContainer * self = NULL;
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestSpaceContainer*) gxml_serializable_container_construct (object_type);
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return self;
#line 5580 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestSpaceContainer* serializable_gee_tree_map_test_space_container_new (void) {
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return serializable_gee_tree_map_test_space_container_construct (SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE_CONTAINER);
#line 5587 "SerializableGeeTreeMapTest.c"
}


static const gchar* serializable_gee_tree_map_test_space_container_get_owner (SerializableGeeTreeMapTestSpaceContainer* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = self->priv->_owner;
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp0_;
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 5602 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_space_container_set_owner (SerializableGeeTreeMapTestSpaceContainer* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_if_fail (self != NULL);
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = value;
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_free0 (self->priv->_owner);
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv->_owner = _tmp1_;
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_object_notify ((GObject *) self, "owner");
#line 5621 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestSpaceCollection* serializable_gee_tree_map_test_space_container_get_storage (SerializableGeeTreeMapTestSpaceContainer* self) {
	SerializableGeeTreeMapTestSpaceCollection* result;
	SerializableGeeTreeMapTestSpaceCollection* _tmp0_ = NULL;
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = self->priv->_storage;
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp0_;
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 5636 "SerializableGeeTreeMapTest.c"
}


static gpointer _g_object_ref0 (gpointer self) {
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return self ? g_object_ref (self) : NULL;
#line 5643 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_space_container_set_storage (SerializableGeeTreeMapTestSpaceContainer* self, SerializableGeeTreeMapTestSpaceCollection* value) {
	SerializableGeeTreeMapTestSpaceCollection* _tmp0_ = NULL;
	SerializableGeeTreeMapTestSpaceCollection* _tmp1_ = NULL;
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_if_fail (self != NULL);
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = value;
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_object_unref0 (self->priv->_storage);
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv->_storage = _tmp1_;
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_object_notify ((GObject *) self, "storage");
#line 5662 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_space_container_class_init (SerializableGeeTreeMapTestSpaceContainerClass * klass) {
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	serializable_gee_tree_map_test_space_container_parent_class = g_type_class_peek_parent (klass);
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeTreeMapTestSpaceContainerPrivate));
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_tree_map_test_space_container_real_node_name;
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_tree_map_test_space_container_real_to_string;
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	((GXmlSerializableContainerClass *) klass)->init_containers = (void (*)(GXmlSerializableContainer*)) serializable_gee_tree_map_test_space_container_real_init_containers;
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_tree_map_test_space_container_get_property;
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_tree_map_test_space_container_set_property;
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_tree_map_test_space_container_finalize;
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_CONTAINER_OWNER, g_param_spec_string ("owner", "owner", "owner", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_CONTAINER_STORAGE, g_param_spec_object ("storage", "storage", "storage", SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_TYPE_COLLECTION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 5687 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_space_container_instance_init (SerializableGeeTreeMapTestSpaceContainer * self) {
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv = SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_CONTAINER_GET_PRIVATE (self);
#line 5694 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_space_container_finalize (GObject* obj) {
	SerializableGeeTreeMapTestSpaceContainer * self;
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE_CONTAINER, SerializableGeeTreeMapTestSpaceContainer);
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_free0 (self->priv->_owner);
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_object_unref0 (self->priv->_storage);
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (serializable_gee_tree_map_test_space_container_parent_class)->finalize (obj);
#line 5708 "SerializableGeeTreeMapTest.c"
}


static GType serializable_gee_tree_map_test_space_container_get_type (void) {
	static volatile gsize serializable_gee_tree_map_test_space_container_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_tree_map_test_space_container_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeTreeMapTestSpaceContainerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_tree_map_test_space_container_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeTreeMapTestSpaceContainer), 0, (GInstanceInitFunc) serializable_gee_tree_map_test_space_container_instance_init, NULL };
		GType serializable_gee_tree_map_test_space_container_type_id;
		serializable_gee_tree_map_test_space_container_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_CONTAINER, "SerializableGeeTreeMapTestSpaceContainer", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_tree_map_test_space_container_type_id__volatile, serializable_gee_tree_map_test_space_container_type_id);
	}
	return serializable_gee_tree_map_test_space_container_type_id__volatile;
}


static void _vala_serializable_gee_tree_map_test_space_container_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeTreeMapTestSpaceContainer * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE_CONTAINER, SerializableGeeTreeMapTestSpaceContainer);
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	switch (property_id) {
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		case SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_CONTAINER_OWNER:
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_value_set_string (value, serializable_gee_tree_map_test_space_container_get_owner (self));
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		case SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_CONTAINER_STORAGE:
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_value_set_object (value, serializable_gee_tree_map_test_space_container_get_storage (self));
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 5741 "SerializableGeeTreeMapTest.c"
		default:
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 5747 "SerializableGeeTreeMapTest.c"
	}
}


static void _vala_serializable_gee_tree_map_test_space_container_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeTreeMapTestSpaceContainer * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SPACE_CONTAINER, SerializableGeeTreeMapTestSpaceContainer);
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	switch (property_id) {
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		case SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_CONTAINER_OWNER:
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		serializable_gee_tree_map_test_space_container_set_owner (self, g_value_get_string (value));
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		case SERIALIZABLE_GEE_TREE_MAP_TEST_SPACE_CONTAINER_STORAGE:
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		serializable_gee_tree_map_test_space_container_set_storage (self, g_value_get_object (value));
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 5769 "SerializableGeeTreeMapTest.c"
		default:
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 5775 "SerializableGeeTreeMapTest.c"
	}
}


static void serializable_gee_tree_map_test_ball_fill_set_text (SerializableGeeTreeMapTestBallFill* self, const gchar* txt) {
	const gchar* _tmp0_ = NULL;
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_if_fail (self != NULL);
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_if_fail (txt != NULL);
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = txt;
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	gxml_serializable_set_serialized_xml_node_value ((GXmlSerializable*) self, _tmp0_);
#line 5790 "SerializableGeeTreeMapTest.c"
}


static gchar* serializable_gee_tree_map_test_ball_fill_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeTreeMapTestBallFill * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestBallFill*) base;
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp1_;
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 5809 "SerializableGeeTreeMapTest.c"
}


static gchar* serializable_gee_tree_map_test_ball_fill_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeTreeMapTestBallFill * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestBallFill*) base;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = g_strdup ("BallFill");
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp0_;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 5825 "SerializableGeeTreeMapTest.c"
}


static gboolean serializable_gee_tree_map_test_ball_fill_real_serialize_use_xml_node_value (GXmlSerializableObjectModel* base) {
	SerializableGeeTreeMapTestBallFill * self;
	gboolean result = FALSE;
#line 61 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestBallFill*) base;
#line 61 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = TRUE;
#line 61 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 5838 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestBallFill* serializable_gee_tree_map_test_ball_fill_construct (GType object_type) {
	SerializableGeeTreeMapTestBallFill * self = NULL;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestBallFill*) gxml_serializable_object_model_construct (object_type);
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return self;
#line 5848 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestBallFill* serializable_gee_tree_map_test_ball_fill_new (void) {
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return serializable_gee_tree_map_test_ball_fill_construct (SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL_FILL);
#line 5855 "SerializableGeeTreeMapTest.c"
}


static const gchar* serializable_gee_tree_map_test_ball_fill_get_name (SerializableGeeTreeMapTestBallFill* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp0_;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 5870 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_ball_fill_set_name (SerializableGeeTreeMapTestBallFill* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_if_fail (self != NULL);
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = value;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_free0 (self->priv->_name);
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv->_name = _tmp1_;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 5889 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_ball_fill_class_init (SerializableGeeTreeMapTestBallFillClass * klass) {
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	serializable_gee_tree_map_test_ball_fill_parent_class = g_type_class_peek_parent (klass);
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeTreeMapTestBallFillPrivate));
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_tree_map_test_ball_fill_real_to_string;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_tree_map_test_ball_fill_real_node_name;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->serialize_use_xml_node_value = (gboolean (*)(GXmlSerializableObjectModel*)) serializable_gee_tree_map_test_ball_fill_real_serialize_use_xml_node_value;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_tree_map_test_ball_fill_get_property;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_tree_map_test_ball_fill_set_property;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_tree_map_test_ball_fill_finalize;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_FILL_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 5912 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_ball_fill_instance_init (SerializableGeeTreeMapTestBallFill * self) {
	gchar* _tmp0_ = NULL;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv = SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_FILL_GET_PRIVATE (self);
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = g_strdup ("Fill");
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv->_name = _tmp0_;
#line 5924 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_ball_fill_finalize (GObject* obj) {
	SerializableGeeTreeMapTestBallFill * self;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL_FILL, SerializableGeeTreeMapTestBallFill);
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_free0 (self->priv->_name);
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (serializable_gee_tree_map_test_ball_fill_parent_class)->finalize (obj);
#line 5936 "SerializableGeeTreeMapTest.c"
}


static GType serializable_gee_tree_map_test_ball_fill_get_type (void) {
	static volatile gsize serializable_gee_tree_map_test_ball_fill_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_tree_map_test_ball_fill_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeTreeMapTestBallFillClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_tree_map_test_ball_fill_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeTreeMapTestBallFill), 0, (GInstanceInitFunc) serializable_gee_tree_map_test_ball_fill_instance_init, NULL };
		GType serializable_gee_tree_map_test_ball_fill_type_id;
		serializable_gee_tree_map_test_ball_fill_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeTreeMapTestBallFill", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_tree_map_test_ball_fill_type_id__volatile, serializable_gee_tree_map_test_ball_fill_type_id);
	}
	return serializable_gee_tree_map_test_ball_fill_type_id__volatile;
}


static void _vala_serializable_gee_tree_map_test_ball_fill_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeTreeMapTestBallFill * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL_FILL, SerializableGeeTreeMapTestBallFill);
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	switch (property_id) {
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		case SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_FILL_NAME:
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_value_set_string (value, serializable_gee_tree_map_test_ball_fill_get_name (self));
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 5963 "SerializableGeeTreeMapTest.c"
		default:
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 5969 "SerializableGeeTreeMapTest.c"
	}
}


static void _vala_serializable_gee_tree_map_test_ball_fill_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeTreeMapTestBallFill * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL_FILL, SerializableGeeTreeMapTestBallFill);
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	switch (property_id) {
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		case SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_FILL_NAME:
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		serializable_gee_tree_map_test_ball_fill_set_name (self, g_value_get_string (value));
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 5985 "SerializableGeeTreeMapTest.c"
		default:
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 5991 "SerializableGeeTreeMapTest.c"
	}
}


static gchar* serializable_gee_tree_map_test_ball_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeTreeMapTestBall * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestBall*) base;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp1_;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 6011 "SerializableGeeTreeMapTest.c"
}


static gchar* serializable_gee_tree_map_test_ball_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeTreeMapTestBall * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestBall*) base;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = g_strdup ("Ball");
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp0_;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 6027 "SerializableGeeTreeMapTest.c"
}


static gchar* serializable_gee_tree_map_test_ball_real_get_map_key (GXmlSerializableMapKey* base) {
	SerializableGeeTreeMapTestBall * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestBall*) base;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp1_;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 6046 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestBall* serializable_gee_tree_map_test_ball_construct (GType object_type) {
	SerializableGeeTreeMapTestBall * self = NULL;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestBall*) gxml_serializable_object_model_construct (object_type);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return self;
#line 6056 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestBall* serializable_gee_tree_map_test_ball_new (void) {
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return serializable_gee_tree_map_test_ball_construct (SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL);
#line 6063 "SerializableGeeTreeMapTest.c"
}


static const gchar* serializable_gee_tree_map_test_ball_get_name (SerializableGeeTreeMapTestBall* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp0_;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 6078 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_ball_set_name (SerializableGeeTreeMapTestBall* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_if_fail (self != NULL);
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = value;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_free0 (self->priv->_name);
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv->_name = _tmp1_;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 6097 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestBallFill* serializable_gee_tree_map_test_ball_get_ballfill (SerializableGeeTreeMapTestBall* self) {
	SerializableGeeTreeMapTestBallFill* result;
	SerializableGeeTreeMapTestBallFill* _tmp0_ = NULL;
#line 67 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 67 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = self->priv->_ballfill;
#line 67 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp0_;
#line 67 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 6112 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_ball_set_ballfill (SerializableGeeTreeMapTestBall* self, SerializableGeeTreeMapTestBallFill* value) {
	SerializableGeeTreeMapTestBallFill* _tmp0_ = NULL;
	SerializableGeeTreeMapTestBallFill* _tmp1_ = NULL;
#line 67 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_if_fail (self != NULL);
#line 67 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = value;
#line 67 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 67 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_object_unref0 (self->priv->_ballfill);
#line 67 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv->_ballfill = _tmp1_;
#line 67 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_object_notify ((GObject *) self, "ballfill");
#line 6131 "SerializableGeeTreeMapTest.c"
}


static gboolean serializable_gee_tree_map_test_ball_tree_map_real_deserialize_proceed (GXmlSerializableTreeMap* base) {
	SerializableGeeTreeMapTestBallTreeMap * self;
	gboolean result = FALSE;
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestBallTreeMap*) base;
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = FALSE;
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 6144 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestBallTreeMap* serializable_gee_tree_map_test_ball_tree_map_construct (GType object_type) {
	SerializableGeeTreeMapTestBallTreeMap * self = NULL;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestBallTreeMap*) gxml_serializable_tree_map_construct (object_type, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return self;
#line 6154 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestBallTreeMap* serializable_gee_tree_map_test_ball_tree_map_new (void) {
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return serializable_gee_tree_map_test_ball_tree_map_construct (SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_TYPE_TREE_MAP);
#line 6161 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_ball_tree_map_class_init (SerializableGeeTreeMapTestBallTreeMapClass * klass) {
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	serializable_gee_tree_map_test_ball_tree_map_parent_class = g_type_class_peek_parent (klass);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	((GXmlSerializableTreeMapClass *) klass)->deserialize_proceed = (gboolean (*)(GXmlSerializableTreeMap*)) serializable_gee_tree_map_test_ball_tree_map_real_deserialize_proceed;
#line 6170 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_ball_tree_map_instance_init (SerializableGeeTreeMapTestBallTreeMap * self) {
}


GType serializable_gee_tree_map_test_ball_tree_map_get_type (void) {
	static volatile gsize serializable_gee_tree_map_test_ball_tree_map_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_tree_map_test_ball_tree_map_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeTreeMapTestBallTreeMapClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_tree_map_test_ball_tree_map_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeTreeMapTestBallTreeMap), 0, (GInstanceInitFunc) serializable_gee_tree_map_test_ball_tree_map_instance_init, NULL };
		GType serializable_gee_tree_map_test_ball_tree_map_type_id;
		serializable_gee_tree_map_test_ball_tree_map_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_TREE_MAP, "SerializableGeeTreeMapTestBallTreeMap", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_tree_map_test_ball_tree_map_type_id__volatile, serializable_gee_tree_map_test_ball_tree_map_type_id);
	}
	return serializable_gee_tree_map_test_ball_tree_map_type_id__volatile;
}


static void serializable_gee_tree_map_test_ball_class_init (SerializableGeeTreeMapTestBallClass * klass) {
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	serializable_gee_tree_map_test_ball_parent_class = g_type_class_peek_parent (klass);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeTreeMapTestBallPrivate));
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_tree_map_test_ball_real_to_string;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_tree_map_test_ball_real_node_name;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_tree_map_test_ball_get_property;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_tree_map_test_ball_set_property;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_tree_map_test_ball_finalize;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_BALLFILL, g_param_spec_object ("ballfill", "ballfill", "ballfill", SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL_FILL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 6209 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_ball_gxml_serializable_map_key_interface_init (GXmlSerializableMapKeyIface * iface) {
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	serializable_gee_tree_map_test_ball_gxml_serializable_map_key_parent_iface = g_type_interface_peek_parent (iface);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	iface->get_map_key = (gpointer (*)(GXmlSerializableMapKey*)) serializable_gee_tree_map_test_ball_real_get_map_key;
#line 6218 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_ball_instance_init (SerializableGeeTreeMapTestBall * self) {
	gchar* _tmp0_ = NULL;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv = SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_GET_PRIVATE (self);
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = g_strdup ("Ball");
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv->_name = _tmp0_;
#line 6230 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_ball_finalize (GObject* obj) {
	SerializableGeeTreeMapTestBall * self;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL, SerializableGeeTreeMapTestBall);
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_free0 (self->priv->_name);
#line 67 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_object_unref0 (self->priv->_ballfill);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (serializable_gee_tree_map_test_ball_parent_class)->finalize (obj);
#line 6244 "SerializableGeeTreeMapTest.c"
}


static GType serializable_gee_tree_map_test_ball_get_type (void) {
	static volatile gsize serializable_gee_tree_map_test_ball_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_tree_map_test_ball_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeTreeMapTestBallClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_tree_map_test_ball_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeTreeMapTestBall), 0, (GInstanceInitFunc) serializable_gee_tree_map_test_ball_instance_init, NULL };
		static const GInterfaceInfo gxml_serializable_map_key_info = { (GInterfaceInitFunc) serializable_gee_tree_map_test_ball_gxml_serializable_map_key_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType serializable_gee_tree_map_test_ball_type_id;
		serializable_gee_tree_map_test_ball_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeTreeMapTestBall", &g_define_type_info, 0);
		g_type_add_interface_static (serializable_gee_tree_map_test_ball_type_id, GXML_TYPE_SERIALIZABLE_MAP_KEY, &gxml_serializable_map_key_info);
		g_once_init_leave (&serializable_gee_tree_map_test_ball_type_id__volatile, serializable_gee_tree_map_test_ball_type_id);
	}
	return serializable_gee_tree_map_test_ball_type_id__volatile;
}


static void _vala_serializable_gee_tree_map_test_ball_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeTreeMapTestBall * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL, SerializableGeeTreeMapTestBall);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	switch (property_id) {
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		case SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_NAME:
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_value_set_string (value, serializable_gee_tree_map_test_ball_get_name (self));
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		case SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_BALLFILL:
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_value_set_object (value, serializable_gee_tree_map_test_ball_get_ballfill (self));
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 6279 "SerializableGeeTreeMapTest.c"
		default:
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 6285 "SerializableGeeTreeMapTest.c"
	}
}


static void _vala_serializable_gee_tree_map_test_ball_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeTreeMapTestBall * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BALL, SerializableGeeTreeMapTestBall);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	switch (property_id) {
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		case SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_NAME:
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		serializable_gee_tree_map_test_ball_set_name (self, g_value_get_string (value));
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		case SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_BALLFILL:
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		serializable_gee_tree_map_test_ball_set_ballfill (self, g_value_get_object (value));
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 6307 "SerializableGeeTreeMapTest.c"
		default:
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 6313 "SerializableGeeTreeMapTest.c"
	}
}


static gchar* serializable_gee_tree_map_test_small_bag_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeTreeMapTestSmallBag * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 80 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestSmallBag*) base;
#line 80 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 80 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 80 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp1_;
#line 80 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 6333 "SerializableGeeTreeMapTest.c"
}


static gchar* serializable_gee_tree_map_test_small_bag_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeTreeMapTestSmallBag * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestSmallBag*) base;
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = g_strdup ("SmallBag");
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp0_;
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 6349 "SerializableGeeTreeMapTest.c"
}


static gchar* serializable_gee_tree_map_test_small_bag_real_get_map_key (GXmlSerializableMapKey* base) {
	SerializableGeeTreeMapTestSmallBag * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestSmallBag*) base;
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp1_;
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 6368 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestSmallBag* serializable_gee_tree_map_test_small_bag_construct (GType object_type) {
	SerializableGeeTreeMapTestSmallBag * self = NULL;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestSmallBag*) gxml_serializable_object_model_construct (object_type);
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return self;
#line 6378 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestSmallBag* serializable_gee_tree_map_test_small_bag_new (void) {
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return serializable_gee_tree_map_test_small_bag_construct (SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SMALL_BAG);
#line 6385 "SerializableGeeTreeMapTest.c"
}


static const gchar* serializable_gee_tree_map_test_small_bag_get_name (SerializableGeeTreeMapTestSmallBag* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp0_;
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 6400 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_small_bag_set_name (SerializableGeeTreeMapTestSmallBag* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_if_fail (self != NULL);
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = value;
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_free0 (self->priv->_name);
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv->_name = _tmp1_;
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 6419 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestBallTreeMap* serializable_gee_tree_map_test_small_bag_get_balls (SerializableGeeTreeMapTestSmallBag* self) {
	SerializableGeeTreeMapTestBallTreeMap* result;
	SerializableGeeTreeMapTestBallTreeMap* _tmp0_ = NULL;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = self->priv->_balls;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp0_;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 6434 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_small_bag_set_balls (SerializableGeeTreeMapTestSmallBag* self, SerializableGeeTreeMapTestBallTreeMap* value) {
	SerializableGeeTreeMapTestBallTreeMap* _tmp0_ = NULL;
	SerializableGeeTreeMapTestBallTreeMap* _tmp1_ = NULL;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_if_fail (self != NULL);
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = value;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_object_unref0 (self->priv->_balls);
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv->_balls = _tmp1_;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_object_notify ((GObject *) self, "balls");
#line 6453 "SerializableGeeTreeMapTest.c"
}


static gboolean serializable_gee_tree_map_test_small_bag_tree_map_real_deserialize_proceed (GXmlSerializableTreeMap* base) {
	SerializableGeeTreeMapTestSmallBagTreeMap * self;
	gboolean result = FALSE;
#line 84 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestSmallBagTreeMap*) base;
#line 84 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = FALSE;
#line 84 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 6466 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestSmallBagTreeMap* serializable_gee_tree_map_test_small_bag_tree_map_construct (GType object_type) {
	SerializableGeeTreeMapTestSmallBagTreeMap * self = NULL;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestSmallBagTreeMap*) gxml_serializable_tree_map_construct (object_type, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SMALL_BAG, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return self;
#line 6476 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestSmallBagTreeMap* serializable_gee_tree_map_test_small_bag_tree_map_new (void) {
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return serializable_gee_tree_map_test_small_bag_tree_map_construct (SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_TYPE_TREE_MAP);
#line 6483 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_small_bag_tree_map_class_init (SerializableGeeTreeMapTestSmallBagTreeMapClass * klass) {
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	serializable_gee_tree_map_test_small_bag_tree_map_parent_class = g_type_class_peek_parent (klass);
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	((GXmlSerializableTreeMapClass *) klass)->deserialize_proceed = (gboolean (*)(GXmlSerializableTreeMap*)) serializable_gee_tree_map_test_small_bag_tree_map_real_deserialize_proceed;
#line 6492 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_small_bag_tree_map_instance_init (SerializableGeeTreeMapTestSmallBagTreeMap * self) {
}


GType serializable_gee_tree_map_test_small_bag_tree_map_get_type (void) {
	static volatile gsize serializable_gee_tree_map_test_small_bag_tree_map_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_tree_map_test_small_bag_tree_map_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeTreeMapTestSmallBagTreeMapClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_tree_map_test_small_bag_tree_map_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeTreeMapTestSmallBagTreeMap), 0, (GInstanceInitFunc) serializable_gee_tree_map_test_small_bag_tree_map_instance_init, NULL };
		GType serializable_gee_tree_map_test_small_bag_tree_map_type_id;
		serializable_gee_tree_map_test_small_bag_tree_map_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_TREE_MAP, "SerializableGeeTreeMapTestSmallBagTreeMap", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_tree_map_test_small_bag_tree_map_type_id__volatile, serializable_gee_tree_map_test_small_bag_tree_map_type_id);
	}
	return serializable_gee_tree_map_test_small_bag_tree_map_type_id__volatile;
}


static void serializable_gee_tree_map_test_small_bag_class_init (SerializableGeeTreeMapTestSmallBagClass * klass) {
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	serializable_gee_tree_map_test_small_bag_parent_class = g_type_class_peek_parent (klass);
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeTreeMapTestSmallBagPrivate));
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_tree_map_test_small_bag_real_to_string;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_tree_map_test_small_bag_real_node_name;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_tree_map_test_small_bag_get_property;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_tree_map_test_small_bag_set_property;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_tree_map_test_small_bag_finalize;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_BALLS, g_param_spec_object ("balls", "balls", "balls", SERIALIZABLE_GEE_TREE_MAP_TEST_BALL_TYPE_TREE_MAP, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 6531 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_small_bag_gxml_serializable_map_key_interface_init (GXmlSerializableMapKeyIface * iface) {
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	serializable_gee_tree_map_test_small_bag_gxml_serializable_map_key_parent_iface = g_type_interface_peek_parent (iface);
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	iface->get_map_key = (gpointer (*)(GXmlSerializableMapKey*)) serializable_gee_tree_map_test_small_bag_real_get_map_key;
#line 6540 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_small_bag_instance_init (SerializableGeeTreeMapTestSmallBag * self) {
	gchar* _tmp0_ = NULL;
	SerializableGeeTreeMapTestBallTreeMap* _tmp1_ = NULL;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv = SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_GET_PRIVATE (self);
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = g_strdup ("SmallBag");
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv->_name = _tmp0_;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = serializable_gee_tree_map_test_ball_tree_map_new ();
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv->_balls = _tmp1_;
#line 6557 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_small_bag_finalize (GObject* obj) {
	SerializableGeeTreeMapTestSmallBag * self;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SMALL_BAG, SerializableGeeTreeMapTestSmallBag);
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_free0 (self->priv->_name);
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_object_unref0 (self->priv->_balls);
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (serializable_gee_tree_map_test_small_bag_parent_class)->finalize (obj);
#line 6571 "SerializableGeeTreeMapTest.c"
}


static GType serializable_gee_tree_map_test_small_bag_get_type (void) {
	static volatile gsize serializable_gee_tree_map_test_small_bag_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_tree_map_test_small_bag_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeTreeMapTestSmallBagClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_tree_map_test_small_bag_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeTreeMapTestSmallBag), 0, (GInstanceInitFunc) serializable_gee_tree_map_test_small_bag_instance_init, NULL };
		static const GInterfaceInfo gxml_serializable_map_key_info = { (GInterfaceInitFunc) serializable_gee_tree_map_test_small_bag_gxml_serializable_map_key_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType serializable_gee_tree_map_test_small_bag_type_id;
		serializable_gee_tree_map_test_small_bag_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeTreeMapTestSmallBag", &g_define_type_info, 0);
		g_type_add_interface_static (serializable_gee_tree_map_test_small_bag_type_id, GXML_TYPE_SERIALIZABLE_MAP_KEY, &gxml_serializable_map_key_info);
		g_once_init_leave (&serializable_gee_tree_map_test_small_bag_type_id__volatile, serializable_gee_tree_map_test_small_bag_type_id);
	}
	return serializable_gee_tree_map_test_small_bag_type_id__volatile;
}


static void _vala_serializable_gee_tree_map_test_small_bag_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeTreeMapTestSmallBag * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SMALL_BAG, SerializableGeeTreeMapTestSmallBag);
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	switch (property_id) {
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		case SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_NAME:
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_value_set_string (value, serializable_gee_tree_map_test_small_bag_get_name (self));
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		case SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_BALLS:
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_value_set_object (value, serializable_gee_tree_map_test_small_bag_get_balls (self));
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 6606 "SerializableGeeTreeMapTest.c"
		default:
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 6612 "SerializableGeeTreeMapTest.c"
	}
}


static void _vala_serializable_gee_tree_map_test_small_bag_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeTreeMapTestSmallBag * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_SMALL_BAG, SerializableGeeTreeMapTestSmallBag);
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	switch (property_id) {
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		case SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_NAME:
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		serializable_gee_tree_map_test_small_bag_set_name (self, g_value_get_string (value));
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		case SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_BALLS:
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		serializable_gee_tree_map_test_small_bag_set_balls (self, g_value_get_object (value));
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 6634 "SerializableGeeTreeMapTest.c"
		default:
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 6640 "SerializableGeeTreeMapTest.c"
	}
}


static gchar* serializable_gee_tree_map_test_big_bag_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeTreeMapTestBigBag * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestBigBag*) base;
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp1_;
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 6660 "SerializableGeeTreeMapTest.c"
}


static gchar* serializable_gee_tree_map_test_big_bag_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeTreeMapTestBigBag * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 93 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestBigBag*) base;
#line 93 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = g_strdup ("BigBag");
#line 93 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp0_;
#line 93 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 6676 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestBigBag* serializable_gee_tree_map_test_big_bag_construct (GType object_type) {
	SerializableGeeTreeMapTestBigBag * self = NULL;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = (SerializableGeeTreeMapTestBigBag*) gxml_serializable_object_model_construct (object_type);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return self;
#line 6686 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestBigBag* serializable_gee_tree_map_test_big_bag_new (void) {
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return serializable_gee_tree_map_test_big_bag_construct (SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BIG_BAG);
#line 6693 "SerializableGeeTreeMapTest.c"
}


static const gchar* serializable_gee_tree_map_test_big_bag_get_name (SerializableGeeTreeMapTestBigBag* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp0_;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 6708 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_big_bag_set_name (SerializableGeeTreeMapTestBigBag* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_if_fail (self != NULL);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = value;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_free0 (self->priv->_name);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv->_name = _tmp1_;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 6727 "SerializableGeeTreeMapTest.c"
}


static SerializableGeeTreeMapTestSmallBagTreeMap* serializable_gee_tree_map_test_big_bag_get_bags (SerializableGeeTreeMapTestBigBag* self) {
	SerializableGeeTreeMapTestSmallBagTreeMap* result;
	SerializableGeeTreeMapTestSmallBagTreeMap* _tmp0_ = NULL;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = self->priv->_bags;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	result = _tmp0_;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	return result;
#line 6742 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_big_bag_set_bags (SerializableGeeTreeMapTestBigBag* self, SerializableGeeTreeMapTestSmallBagTreeMap* value) {
	SerializableGeeTreeMapTestSmallBagTreeMap* _tmp0_ = NULL;
	SerializableGeeTreeMapTestSmallBagTreeMap* _tmp1_ = NULL;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_return_if_fail (self != NULL);
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = value;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_object_unref0 (self->priv->_bags);
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv->_bags = _tmp1_;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_object_notify ((GObject *) self, "bags");
#line 6761 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_big_bag_class_init (SerializableGeeTreeMapTestBigBagClass * klass) {
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	serializable_gee_tree_map_test_big_bag_parent_class = g_type_class_peek_parent (klass);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeTreeMapTestBigBagPrivate));
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_tree_map_test_big_bag_real_to_string;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_tree_map_test_big_bag_real_node_name;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_tree_map_test_big_bag_get_property;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_tree_map_test_big_bag_set_property;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_tree_map_test_big_bag_finalize;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_TREE_MAP_TEST_BIG_BAG_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_TREE_MAP_TEST_BIG_BAG_BAGS, g_param_spec_object ("bags", "bags", "bags", SERIALIZABLE_GEE_TREE_MAP_TEST_SMALL_BAG_TYPE_TREE_MAP, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 6784 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_big_bag_instance_init (SerializableGeeTreeMapTestBigBag * self) {
	gchar* _tmp0_ = NULL;
	SerializableGeeTreeMapTestSmallBagTreeMap* _tmp1_ = NULL;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv = SERIALIZABLE_GEE_TREE_MAP_TEST_BIG_BAG_GET_PRIVATE (self);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp0_ = g_strdup ("ball");
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv->_name = _tmp0_;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_tmp1_ = serializable_gee_tree_map_test_small_bag_tree_map_new ();
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self->priv->_bags = _tmp1_;
#line 6801 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_big_bag_finalize (GObject* obj) {
	SerializableGeeTreeMapTestBigBag * self;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BIG_BAG, SerializableGeeTreeMapTestBigBag);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_free0 (self->priv->_name);
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	_g_object_unref0 (self->priv->_bags);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	G_OBJECT_CLASS (serializable_gee_tree_map_test_big_bag_parent_class)->finalize (obj);
#line 6815 "SerializableGeeTreeMapTest.c"
}


static GType serializable_gee_tree_map_test_big_bag_get_type (void) {
	static volatile gsize serializable_gee_tree_map_test_big_bag_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_tree_map_test_big_bag_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeTreeMapTestBigBagClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_tree_map_test_big_bag_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeTreeMapTestBigBag), 0, (GInstanceInitFunc) serializable_gee_tree_map_test_big_bag_instance_init, NULL };
		GType serializable_gee_tree_map_test_big_bag_type_id;
		serializable_gee_tree_map_test_big_bag_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeTreeMapTestBigBag", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_tree_map_test_big_bag_type_id__volatile, serializable_gee_tree_map_test_big_bag_type_id);
	}
	return serializable_gee_tree_map_test_big_bag_type_id__volatile;
}


static void _vala_serializable_gee_tree_map_test_big_bag_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeTreeMapTestBigBag * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BIG_BAG, SerializableGeeTreeMapTestBigBag);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	switch (property_id) {
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		case SERIALIZABLE_GEE_TREE_MAP_TEST_BIG_BAG_NAME:
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_value_set_string (value, serializable_gee_tree_map_test_big_bag_get_name (self));
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		case SERIALIZABLE_GEE_TREE_MAP_TEST_BIG_BAG_BAGS:
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		g_value_set_object (value, serializable_gee_tree_map_test_big_bag_get_bags (self));
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 6848 "SerializableGeeTreeMapTest.c"
		default:
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 6854 "SerializableGeeTreeMapTest.c"
	}
}


static void _vala_serializable_gee_tree_map_test_big_bag_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeTreeMapTestBigBag * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_TREE_MAP_TEST_TYPE_BIG_BAG, SerializableGeeTreeMapTestBigBag);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	switch (property_id) {
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		case SERIALIZABLE_GEE_TREE_MAP_TEST_BIG_BAG_NAME:
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		serializable_gee_tree_map_test_big_bag_set_name (self, g_value_get_string (value));
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		case SERIALIZABLE_GEE_TREE_MAP_TEST_BIG_BAG_BAGS:
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		serializable_gee_tree_map_test_big_bag_set_bags (self, g_value_get_object (value));
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 6876 "SerializableGeeTreeMapTest.c"
		default:
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
		break;
#line 6882 "SerializableGeeTreeMapTest.c"
	}
}


static void serializable_gee_tree_map_test_class_init (SerializableGeeTreeMapTestClass * klass) {
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeTreeMapTest.vala"
	serializable_gee_tree_map_test_parent_class = g_type_class_peek_parent (klass);
#line 6890 "SerializableGeeTreeMapTest.c"
}


static void serializable_gee_tree_map_test_instance_init (SerializableGeeTreeMapTest * self) {
}


/**
 *
 *  GXml.Serializable.BasicTypeTest
 *
 *  Authors:
 *
 *       Daniel Espinosa <esodan@gmail.com>
 *
 *
 *  Copyright (c) 2013-2015 Daniel Espinosa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
GType serializable_gee_tree_map_test_get_type (void) {
	static volatile gsize serializable_gee_tree_map_test_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_tree_map_test_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeTreeMapTestClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_tree_map_test_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeTreeMapTest), 0, (GInstanceInitFunc) serializable_gee_tree_map_test_instance_init, NULL };
		GType serializable_gee_tree_map_test_type_id;
		serializable_gee_tree_map_test_type_id = g_type_register_static (TYPE_GXML_TEST, "SerializableGeeTreeMapTest", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_tree_map_test_type_id__volatile, serializable_gee_tree_map_test_type_id);
	}
	return serializable_gee_tree_map_test_type_id__volatile;
}



