/* DomGDocumentTest.c generated by valac 0.34.6, the Vala compiler
 * generated from DomGDocumentTest.vala, do not modify */

/* -*- Mode: vala; indent-tabs-mode: t; c-basic-offset: 2; tab-width: 2 -*- */
/* DomGDocumentTest.vala
 *
 * Copyright (C) 2016  Daniel Espinosa <esodan@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Daniel Espinosa <esodan@gmail.com>
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gxml/gxml.h>
#include <gee.h>


#define TYPE_GXML_TEST (gxml_test_get_type ())
#define GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GXML_TEST, GXmlTest))
#define GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GXML_TEST, GXmlTestClass))
#define IS_GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GXML_TEST))
#define IS_GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GXML_TEST))
#define GXML_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GXML_TEST, GXmlTestClass))

typedef struct _GXmlTest GXmlTest;
typedef struct _GXmlTestClass GXmlTestClass;
typedef struct _GXmlTestPrivate GXmlTestPrivate;

#define TYPE_DOM_GDOCUMENT_TEST (dom_gdocument_test_get_type ())
#define DOM_GDOCUMENT_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DOM_GDOCUMENT_TEST, DomGDocumentTest))
#define DOM_GDOCUMENT_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DOM_GDOCUMENT_TEST, DomGDocumentTestClass))
#define IS_DOM_GDOCUMENT_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DOM_GDOCUMENT_TEST))
#define IS_DOM_GDOCUMENT_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DOM_GDOCUMENT_TEST))
#define DOM_GDOCUMENT_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DOM_GDOCUMENT_TEST, DomGDocumentTestClass))

typedef struct _DomGDocumentTest DomGDocumentTest;
typedef struct _DomGDocumentTestClass DomGDocumentTestClass;
typedef struct _DomGDocumentTestPrivate DomGDocumentTestPrivate;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _GXmlTest {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GXmlTestPrivate * priv;
};

struct _GXmlTestClass {
	GTypeClass parent_class;
	void (*finalize) (GXmlTest *self);
};

struct _DomGDocumentTest {
	GXmlTest parent_instance;
	DomGDocumentTestPrivate * priv;
};

struct _DomGDocumentTestClass {
	GXmlTestClass parent_class;
};


static gpointer dom_gdocument_test_parent_class = NULL;

gpointer gxml_test_ref (gpointer instance);
void gxml_test_unref (gpointer instance);
GParamSpec* param_spec_gxml_test (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_gxml_test (GValue* value, gpointer v_object);
void value_take_gxml_test (GValue* value, gpointer v_object);
gpointer value_get_gxml_test (const GValue* value);
GType gxml_test_get_type (void) G_GNUC_CONST;
GType dom_gdocument_test_get_type (void) G_GNUC_CONST;
enum  {
	DOM_GDOCUMENT_TEST_DUMMY_PROPERTY
};
#define DOM_GDOCUMENT_TEST_STRDOC "<?xml version=\"1.0\"?>\n" \
"<!-- Comment -->\n" \
"<Sentences>\n" \
"  <Sentence lang=\"en\">I like the colour blue.</Sentence>\n" \
"  <Sentence lang=\"de\">Ich liebe die T&#xFC;r.</Sentence>\n" \
"  <Authors>\n" \
"    <Author>\n" \
"      <Name>Fred</Name>\n" \
"      <Email>fweasley@hogwarts.co.uk</Email>\n" \
"    </Author>\n" \
"    <Author>\n" \
"      <Name>George</Name>\n" \
"      <Email>gweasley@hogwarts.co.uk</Email>\n" \
"    </Author>\n" \
"  </Authors>\n" \
"</Sentences>\n"
#define DOM_GDOCUMENT_TEST_HTMLDOC "\n" \
"<html>\n" \
"<body>\n" \
"<p class=\"black\">Text content</p>\n" \
"<p id=\"p01\">p01 p id</p>\n" \
"<p class=\"black block\">Two classes</p>\n" \
"<p class=\"   time request hole   \">Three classes</p>\n" \
"</body>\n" \
"</html>\n"
#define DOM_GDOCUMENT_TEST_XMLDOC "<?xml version=\"1.0\"?>\n" \
"<root>\n" \
"<project xmlns:gxml=\"http://live.gnome.org/GXml\">\n" \
"<code class=\"parent\"/>\n" \
"<code class=\"node parent\"/>\n" \
"<page class=\"node\"/>\n" \
"<page class=\"parent node hole\"/>\n" \
"</project>\n" \
"<Author name=\"You\" />\n" \
"</root>\n"
void dom_gdocument_test_add_tests (void);
static void __lambda193_ (void);
static void ___lambda193__gtest_func (void);
static void __lambda194_ (void);
static void ___lambda194__gtest_func (void);
static void __lambda195_ (void);
static void ___lambda195__gtest_func (void);
static void __lambda196_ (void);
static void ___lambda196__gtest_func (void);
static void __lambda197_ (void);
static void ___lambda197__gtest_func (void);
static void __lambda198_ (void);
static void ___lambda198__gtest_func (void);
static void __lambda199_ (void);
static void ___lambda199__gtest_func (void);
static void __lambda200_ (void);
static void ___lambda200__gtest_func (void);
static void __lambda201_ (void);
static void ___lambda201__gtest_func (void);
static void __lambda202_ (void);
static void ___lambda202__gtest_func (void);
static void __lambda203_ (void);
static void ___lambda203__gtest_func (void);
static void __lambda204_ (void);
static void ___lambda204__gtest_func (void);
static void __lambda205_ (void);
static void ___lambda205__gtest_func (void);
static void __lambda206_ (void);
static void ___lambda206__gtest_func (void);
DomGDocumentTest* dom_gdocument_test_new (void);
DomGDocumentTest* dom_gdocument_test_construct (GType object_type);
GXmlTest* gxml_test_new (void);
GXmlTest* gxml_test_construct (GType object_type);


static void __lambda193_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* _tmp0_ = NULL;
		GXmlGDocument* _tmp1_ = NULL;
		GXmlDomDocument* doc = NULL;
		GXmlGDocument* _tmp2_ = NULL;
		GXmlDomDocument* _tmp3_ = NULL;
		GXmlDomElement* root = NULL;
		GXmlDomElement* _tmp4_ = NULL;
		GXmlDomElement* _tmp5_ = NULL;
		gchar* _tmp6_ = NULL;
		gchar* _tmp7_ = NULL;
		gchar* _tmp8_ = NULL;
		gboolean _tmp9_ = FALSE;
		GXmlDomHTMLCollection* _tmp10_ = NULL;
		GXmlDomHTMLCollection* _tmp11_ = NULL;
		GXmlDomHTMLCollection* _tmp12_ = NULL;
		gint _tmp13_ = 0;
		gint _tmp14_ = 0;
		GXmlDomHTMLCollection* _tmp15_ = NULL;
		GXmlDomHTMLCollection* _tmp16_ = NULL;
		GXmlDomHTMLCollection* _tmp17_ = NULL;
		gpointer _tmp18_ = NULL;
		GXmlDomElement* _tmp19_ = NULL;
		gchar* _tmp20_ = NULL;
		gchar* _tmp21_ = NULL;
		gchar* _tmp22_ = NULL;
		GXmlDomHTMLCollection* _tmp23_ = NULL;
		GXmlDomHTMLCollection* _tmp24_ = NULL;
		GXmlDomHTMLCollection* _tmp25_ = NULL;
		gpointer _tmp26_ = NULL;
		GXmlDomElement* _tmp27_ = NULL;
		gchar* _tmp28_ = NULL;
		gchar* _tmp29_ = NULL;
		GXmlDomHTMLCollection* _tmp30_ = NULL;
		GXmlDomHTMLCollection* _tmp31_ = NULL;
		GXmlDomHTMLCollection* _tmp32_ = NULL;
		gpointer _tmp33_ = NULL;
		GXmlDomElement* _tmp34_ = NULL;
		gchar* _tmp35_ = NULL;
		gchar* _tmp36_ = NULL;
		gchar* _tmp37_ = NULL;
#line 74 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp1_ = gxml_gdocument_new_from_string (DOM_GDOCUMENT_TEST_STRDOC, 0, &_inner_error_);
#line 74 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp0_ = _tmp1_;
#line 74 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 185 "DomGDocumentTest.c"
			goto __catch179_g_error;
		}
#line 74 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp2_ = _tmp0_;
#line 74 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp0_ = NULL;
#line 74 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp3_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp2_, GXML_TYPE_DOM_DOCUMENT) ? ((GXmlDomDocument*) _tmp2_) : NULL;
#line 74 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp3_ == NULL) {
#line 74 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 198 "DomGDocumentTest.c"
		}
#line 74 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		doc = _tmp3_;
#line 75 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp4_ = gxml_dom_document_get_document_element (doc);
#line 75 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp5_ = _tmp4_;
#line 75 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		root = _tmp5_;
#line 76 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (root != NULL, "root != null");
#line 77 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (root, GXML_TYPE_DOM_ELEMENT), "root is DomElement");
#line 78 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp6_ = gxml_dom_node_get_node_name ((GXmlDomNode*) root);
#line 78 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp7_ = _tmp6_;
#line 78 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp8_ = _tmp7_;
#line 78 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp8_, "Sentences") == 0, "root.node_name == \"Sentences\"");
#line 78 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp8_);
#line 79 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp9_ = gxml_dom_node_has_child_nodes ((GXmlDomNode*) root);
#line 79 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp9_, "root.has_child_nodes ()");
#line 80 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp10_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) root);
#line 80 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp11_ = _tmp10_;
#line 80 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp12_ = _tmp11_;
#line 80 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp13_ = gee_collection_get_size ((GeeCollection*) _tmp12_);
#line 80 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp14_ = _tmp13_;
#line 80 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp14_ == 3, "root.children.size == 3");
#line 80 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp12_);
#line 81 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp15_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) root);
#line 81 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp16_ = _tmp15_;
#line 81 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp17_ = _tmp16_;
#line 81 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp18_ = gee_list_get ((GeeList*) _tmp17_, 0);
#line 81 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp19_ = (GXmlDomElement*) _tmp18_;
#line 81 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp20_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp19_);
#line 81 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp21_ = _tmp20_;
#line 81 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp22_ = _tmp21_;
#line 81 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp22_, "Sentence") == 0, "root.children[0].node_name == \"Sentence\"");
#line 81 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp22_);
#line 81 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp19_);
#line 81 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp17_);
#line 82 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp23_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) root);
#line 82 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp24_ = _tmp23_;
#line 82 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp25_ = _tmp24_;
#line 82 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp26_ = gee_list_get ((GeeList*) _tmp25_, 0);
#line 82 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp27_ = (GXmlDomElement*) _tmp26_;
#line 82 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp28_ = gxml_dom_element_get_attribute (_tmp27_, "lang");
#line 82 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp29_ = _tmp28_;
#line 82 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp29_, "en") == 0, "root.children[0].get_attribute (\"lang\") == \"en\"");
#line 82 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp29_);
#line 82 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp27_);
#line 82 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp25_);
#line 83 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp30_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) root);
#line 83 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp31_ = _tmp30_;
#line 83 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp32_ = _tmp31_;
#line 83 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp33_ = gee_list_get ((GeeList*) _tmp32_, 0);
#line 83 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp34_ = (GXmlDomElement*) _tmp33_;
#line 83 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp35_ = gxml_dom_node_get_node_value ((GXmlDomNode*) _tmp34_);
#line 83 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp36_ = _tmp35_;
#line 83 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp37_ = _tmp36_;
#line 83 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp37_, "I like the colour blue.") == 0, "root.children[0].node_value == \"I like the colour blue.\"");
#line 83 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp37_);
#line 83 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp34_);
#line 83 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp32_);
#line 70 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (root);
#line 70 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (doc);
#line 70 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp0_);
#line 316 "DomGDocumentTest.c"
	}
	goto __finally179;
	__catch179_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp38_ = NULL;
		gchar* _tmp39_ = NULL;
		gchar* _tmp40_ = NULL;
#line 70 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		e = _inner_error_;
#line 70 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_inner_error_ = NULL;
#line 85 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp38_ = e->message;
#line 85 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp39_ = g_strconcat ("Error: ", _tmp38_, NULL);
#line 85 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp40_ = _tmp39_;
#line 85 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_message ("DomGDocumentTest.vala:85: %s", _tmp40_);
#line 85 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp40_);
#line 86 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_assert_not_reached ();
#line 70 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_error_free0 (e);
#line 343 "DomGDocumentTest.c"
	}
	__finally179:
#line 70 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 70 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 70 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 70 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		return;
#line 354 "DomGDocumentTest.c"
	}
}


static void ___lambda193__gtest_func (void) {
#line 69 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	__lambda193_ ();
#line 362 "DomGDocumentTest.c"
}


static void __lambda194_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* _tmp0_ = NULL;
		GXmlGDocument* _tmp1_ = NULL;
		GXmlDomDocument* doc = NULL;
		GXmlGDocument* _tmp2_ = NULL;
		GXmlDomDocument* _tmp3_ = NULL;
		GXmlDomNodeList* _tmp4_ = NULL;
		GXmlDomNodeList* _tmp5_ = NULL;
		GXmlDomNodeList* _tmp6_ = NULL;
		GXmlDomNodeList* _tmp7_ = NULL;
		GXmlDomNodeList* _tmp8_ = NULL;
		GXmlDomNodeList* _tmp9_ = NULL;
		gint _tmp10_ = 0;
		gint _tmp11_ = 0;
		GXmlDomNodeList* _tmp12_ = NULL;
		GXmlDomNodeList* _tmp13_ = NULL;
		GXmlDomNodeList* _tmp14_ = NULL;
		gpointer _tmp15_ = NULL;
		GXmlDomNode* _tmp16_ = NULL;
		GXmlDomNodeList* _tmp17_ = NULL;
		GXmlDomNodeList* _tmp18_ = NULL;
		GXmlDomNodeList* _tmp19_ = NULL;
		gpointer _tmp20_ = NULL;
		GXmlDomNode* _tmp21_ = NULL;
		GXmlDomNodeList* _tmp22_ = NULL;
		GXmlDomNodeList* _tmp23_ = NULL;
		GXmlDomNodeList* _tmp24_ = NULL;
		gpointer _tmp25_ = NULL;
		GXmlDomNode* _tmp26_ = NULL;
		gchar* _tmp27_ = NULL;
		gchar* _tmp28_ = NULL;
		gchar* _tmp29_ = NULL;
		GXmlDomElement* _tmp30_ = NULL;
		GXmlDomElement* _tmp31_ = NULL;
		GXmlDomElement* _tmp32_ = NULL;
		GXmlDomElement* _tmp33_ = NULL;
		GXmlDomElement* _tmp34_ = NULL;
		GXmlDomElement* _tmp35_ = NULL;
		GXmlDomElement* auths = NULL;
		GXmlDomElement* _tmp36_ = NULL;
		GXmlDomElement* _tmp37_ = NULL;
		GXmlDomElement* _tmp38_ = NULL;
		GXmlDomHTMLCollection* _tmp39_ = NULL;
		GXmlDomHTMLCollection* _tmp40_ = NULL;
		GXmlDomHTMLCollection* _tmp41_ = NULL;
		gpointer _tmp42_ = NULL;
		GXmlDomElement* _tmp43_ = NULL;
		gchar* _tmp44_ = NULL;
		gchar* _tmp45_ = NULL;
		gchar* _tmp46_ = NULL;
		GXmlDomNodeList* _tmp47_ = NULL;
		GXmlDomNodeList* _tmp48_ = NULL;
		GXmlDomNodeList* _tmp49_ = NULL;
		gint _tmp50_ = 0;
		gint _tmp51_ = 0;
		GXmlDomNodeList* _tmp52_ = NULL;
		GXmlDomNodeList* _tmp53_ = NULL;
		GXmlDomNodeList* _tmp54_ = NULL;
		gpointer _tmp55_ = NULL;
		GXmlDomNode* _tmp56_ = NULL;
#line 94 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp1_ = gxml_gdocument_new_from_string (DOM_GDOCUMENT_TEST_STRDOC, 0, &_inner_error_);
#line 94 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp0_ = _tmp1_;
#line 94 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 434 "DomGDocumentTest.c"
			goto __catch180_g_error;
		}
#line 94 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp2_ = _tmp0_;
#line 94 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp0_ = NULL;
#line 94 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp3_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp2_, GXML_TYPE_DOM_DOCUMENT) ? ((GXmlDomDocument*) _tmp2_) : NULL;
#line 94 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp3_ == NULL) {
#line 94 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 447 "DomGDocumentTest.c"
		}
#line 94 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		doc = _tmp3_;
#line 95 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (doc, GXML_TYPE_DOM_DOCUMENT), "doc is DomDocument");
#line 96 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp4_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) doc);
#line 96 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp5_ = _tmp4_;
#line 96 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp6_ = _tmp5_;
#line 96 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp6_ != NULL, "doc.child_nodes != null");
#line 96 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp6_);
#line 97 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp7_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) doc);
#line 97 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp8_ = _tmp7_;
#line 97 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp9_ = _tmp8_;
#line 97 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp10_ = gee_collection_get_size ((GeeCollection*) _tmp9_);
#line 97 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp11_ = _tmp10_;
#line 97 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp11_ == 2, "doc.child_nodes.size == 2");
#line 97 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp9_);
#line 98 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp12_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) doc);
#line 98 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp13_ = _tmp12_;
#line 98 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp14_ = _tmp13_;
#line 98 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp15_ = gee_list_get ((GeeList*) _tmp14_, 0);
#line 98 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp16_ = (GXmlDomNode*) _tmp15_;
#line 98 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp16_, GXML_TYPE_DOM_COMMENT), "doc.child_nodes[0] is DomComment");
#line 98 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp16_);
#line 98 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp14_);
#line 99 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp17_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) doc);
#line 99 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp18_ = _tmp17_;
#line 99 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp19_ = _tmp18_;
#line 99 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp20_ = gee_list_get ((GeeList*) _tmp19_, 1);
#line 99 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp21_ = (GXmlDomNode*) _tmp20_;
#line 99 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp21_, GXML_TYPE_DOM_ELEMENT), "doc.child_nodes[1] is DomElement");
#line 99 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp21_);
#line 99 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp19_);
#line 100 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp22_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) doc);
#line 100 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp23_ = _tmp22_;
#line 100 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp24_ = _tmp23_;
#line 100 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp25_ = gee_list_get ((GeeList*) _tmp24_, 1);
#line 100 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp26_ = (GXmlDomNode*) _tmp25_;
#line 100 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp27_ = gxml_dom_node_get_node_name (_tmp26_);
#line 100 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp28_ = _tmp27_;
#line 100 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp29_ = _tmp28_;
#line 100 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp29_, "Sentences") == 0, "doc.child_nodes[1].node_name == \"Sentences\"");
#line 100 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp29_);
#line 100 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp26_);
#line 100 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp24_);
#line 101 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp30_ = gxml_dom_document_get_document_element (doc);
#line 101 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp31_ = _tmp30_;
#line 101 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp32_ = _tmp31_;
#line 101 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp32_ != NULL, "doc.document_element != null");
#line 101 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp32_);
#line 102 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp33_ = gxml_dom_document_get_document_element (doc);
#line 102 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp34_ = _tmp33_;
#line 102 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp35_ = _tmp34_;
#line 102 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp35_, GXML_TYPE_DOM_ELEMENT), "doc.document_element is DomElement");
#line 102 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp35_);
#line 103 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp36_ = gxml_dom_document_get_document_element (doc);
#line 103 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp37_ = _tmp36_;
#line 103 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp38_ = _tmp37_;
#line 103 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp39_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp38_);
#line 103 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp40_ = _tmp39_;
#line 103 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp41_ = _tmp40_;
#line 103 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp42_ = gee_list_get ((GeeList*) _tmp41_, 2);
#line 103 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp43_ = (GXmlDomElement*) _tmp42_;
#line 103 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp41_);
#line 103 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp38_);
#line 103 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		auths = _tmp43_;
#line 104 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp44_ = gxml_dom_node_get_node_name ((GXmlDomNode*) auths);
#line 104 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp45_ = _tmp44_;
#line 104 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp46_ = _tmp45_;
#line 104 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp46_, "Authors") == 0, "auths.node_name == \"Authors\"");
#line 104 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp46_);
#line 105 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp47_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) auths);
#line 105 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp48_ = _tmp47_;
#line 105 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp49_ = _tmp48_;
#line 105 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp50_ = gee_collection_get_size ((GeeCollection*) _tmp49_);
#line 105 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp51_ = _tmp50_;
#line 105 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp51_ == 5, "auths.child_nodes.size == 5");
#line 105 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp49_);
#line 106 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp52_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) auths);
#line 106 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp53_ = _tmp52_;
#line 106 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp54_ = _tmp53_;
#line 106 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp55_ = gee_list_get ((GeeList*) _tmp54_, 4);
#line 106 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp56_ = (GXmlDomNode*) _tmp55_;
#line 106 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp56_, GXML_TYPE_DOM_TEXT), "auths.child_nodes[4] is DomText");
#line 106 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp56_);
#line 106 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp54_);
#line 90 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (auths);
#line 90 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (doc);
#line 90 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp0_);
#line 621 "DomGDocumentTest.c"
	}
	goto __finally180;
	__catch180_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp57_ = NULL;
		gchar* _tmp58_ = NULL;
		gchar* _tmp59_ = NULL;
#line 90 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		e = _inner_error_;
#line 90 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_inner_error_ = NULL;
#line 108 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp57_ = e->message;
#line 108 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp58_ = g_strconcat ("Error: ", _tmp57_, NULL);
#line 108 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp59_ = _tmp58_;
#line 108 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_message ("DomGDocumentTest.vala:108: %s", _tmp59_);
#line 108 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp59_);
#line 109 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_assert_not_reached ();
#line 90 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_error_free0 (e);
#line 648 "DomGDocumentTest.c"
	}
	__finally180:
#line 90 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 90 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 90 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 90 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		return;
#line 659 "DomGDocumentTest.c"
	}
}


static void ___lambda194__gtest_func (void) {
#line 89 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	__lambda194_ ();
#line 667 "DomGDocumentTest.c"
}


static void __lambda195_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* _tmp0_ = NULL;
		GXmlGDocument* _tmp1_ = NULL;
		GXmlDomDocument* doc = NULL;
		GXmlGDocument* _tmp2_ = NULL;
		GXmlDomDocument* _tmp3_ = NULL;
		GXmlDomDocument* _tmp4_ = NULL;
		GXmlDomHTMLCollection* le = NULL;
		GXmlDomDocument* _tmp5_ = NULL;
		GXmlDomHTMLCollection* _tmp6_ = NULL;
		GXmlDomHTMLCollection* _tmp7_ = NULL;
		gint _tmp8_ = 0;
		gint _tmp9_ = 0;
		GXmlDomHTMLCollection* _tmp10_ = NULL;
		gpointer _tmp11_ = NULL;
		GXmlDomElement* _tmp12_ = NULL;
		gchar* _tmp13_ = NULL;
		gchar* _tmp14_ = NULL;
		GXmlDomHTMLCollection* _tmp15_ = NULL;
		gpointer _tmp16_ = NULL;
		GXmlDomElement* _tmp17_ = NULL;
		gchar* _tmp18_ = NULL;
		gchar* _tmp19_ = NULL;
		GXmlDomHTMLCollection* lc = NULL;
		GXmlDomDocument* _tmp20_ = NULL;
		GXmlDomHTMLCollection* _tmp21_ = NULL;
		GXmlDomHTMLCollection* _tmp22_ = NULL;
		gint _tmp23_ = 0;
		gint _tmp24_ = 0;
		GXmlDomHTMLCollection* _tmp25_ = NULL;
		gpointer _tmp26_ = NULL;
		GXmlDomElement* _tmp27_ = NULL;
		gchar* _tmp28_ = NULL;
		gchar* _tmp29_ = NULL;
		gchar* _tmp30_ = NULL;
		GXmlDomHTMLCollection* _tmp31_ = NULL;
		gpointer _tmp32_ = NULL;
		GXmlDomElement* _tmp33_ = NULL;
		gchar* _tmp34_ = NULL;
		gchar* _tmp35_ = NULL;
		GXmlDomElement* nid = NULL;
		GXmlDomDocument* _tmp36_ = NULL;
		GXmlDomElement* _tmp37_ = NULL;
		GXmlDomElement* _tmp38_ = NULL;
		GXmlDomElement* _tmp39_ = NULL;
		gchar* _tmp40_ = NULL;
		gchar* _tmp41_ = NULL;
		gchar* _tmp42_ = NULL;
		GXmlDomElement* _tmp43_ = NULL;
		gchar* _tmp44_ = NULL;
		gchar* _tmp45_ = NULL;
#line 117 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp1_ = gxml_gdocument_new_from_string (DOM_GDOCUMENT_TEST_HTMLDOC, 0, &_inner_error_);
#line 117 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp0_ = _tmp1_;
#line 117 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 730 "DomGDocumentTest.c"
			goto __catch181_g_error;
		}
#line 117 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp2_ = _tmp0_;
#line 117 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp0_ = NULL;
#line 117 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp3_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp2_, GXML_TYPE_DOM_DOCUMENT) ? ((GXmlDomDocument*) _tmp2_) : NULL;
#line 117 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp3_ == NULL) {
#line 117 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 743 "DomGDocumentTest.c"
		}
#line 117 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		doc = _tmp3_;
#line 118 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp4_ = doc;
#line 118 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp4_, GXML_TYPE_DOM_DOCUMENT), "doc is DomDocument");
#line 119 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp5_ = doc;
#line 119 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp6_ = gxml_dom_document_get_elements_by_tag_name (_tmp5_, "p");
#line 119 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		le = _tmp6_;
#line 120 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp7_ = le;
#line 120 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp8_ = gee_collection_get_size ((GeeCollection*) _tmp7_);
#line 120 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp9_ = _tmp8_;
#line 120 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp9_ == 4, "le.size == 4");
#line 121 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp10_ = le;
#line 121 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp11_ = gee_list_get ((GeeList*) _tmp10_, 0);
#line 121 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp12_ = (GXmlDomElement*) _tmp11_;
#line 121 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp13_ = gxml_dom_element_get_attribute (_tmp12_, "class");
#line 121 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp14_ = _tmp13_;
#line 121 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp14_, "black") == 0, "le[0].get_attribute (\"class\") == \"black\"");
#line 121 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp14_);
#line 121 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp12_);
#line 122 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp15_ = le;
#line 122 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp16_ = gee_list_get ((GeeList*) _tmp15_, 1);
#line 122 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp17_ = (GXmlDomElement*) _tmp16_;
#line 122 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp18_ = gxml_dom_element_get_attribute (_tmp17_, "id");
#line 122 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp19_ = _tmp18_;
#line 122 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp19_, "p01") == 0, "le[1].get_attribute (\"id\") == \"p01\"");
#line 122 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp19_);
#line 122 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp17_);
#line 123 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp20_ = doc;
#line 123 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp21_ = gxml_dom_document_get_elements_by_class_name (_tmp20_, "black");
#line 123 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		lc = _tmp21_;
#line 127 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp22_ = lc;
#line 127 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp23_ = gee_collection_get_size ((GeeCollection*) _tmp22_);
#line 127 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp24_ = _tmp23_;
#line 127 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp24_ == 2, "lc.size == 2");
#line 128 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp25_ = lc;
#line 128 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp26_ = gee_list_get ((GeeList*) _tmp25_, 0);
#line 128 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp27_ = (GXmlDomElement*) _tmp26_;
#line 128 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp28_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp27_);
#line 128 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp29_ = _tmp28_;
#line 128 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp30_ = _tmp29_;
#line 128 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp30_, "p") == 0, "lc[0].node_name == \"p\"");
#line 128 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp30_);
#line 128 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp27_);
#line 129 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp31_ = lc;
#line 129 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp32_ = gee_list_get ((GeeList*) _tmp31_, 0);
#line 129 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp33_ = (GXmlDomElement*) _tmp32_;
#line 129 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp34_ = gxml_dom_element_get_attribute (_tmp33_, "class");
#line 129 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp35_ = _tmp34_;
#line 129 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp35_, "black block") == 0, "lc[0].get_attribute (\"class\") == \"black block\"");
#line 129 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp35_);
#line 129 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp33_);
#line 130 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp36_ = doc;
#line 130 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp37_ = gxml_dom_non_element_parent_node_get_element_by_id ((GXmlDomNonElementParentNode*) _tmp36_, "p01", &_inner_error_);
#line 130 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		nid = _tmp37_;
#line 130 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 130 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lc);
#line 130 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (le);
#line 130 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 130 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 861 "DomGDocumentTest.c"
			goto __catch181_g_error;
		}
#line 131 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp38_ = nid;
#line 131 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp38_ != NULL, "nid != null");
#line 132 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp39_ = nid;
#line 132 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp40_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp39_);
#line 132 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp41_ = _tmp40_;
#line 132 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp42_ = _tmp41_;
#line 132 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp42_, "p") == 0, "nid.node_name == \"p\"");
#line 132 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp42_);
#line 133 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp43_ = nid;
#line 133 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp44_ = gxml_dom_element_get_attribute (_tmp43_, "id");
#line 133 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp45_ = _tmp44_;
#line 133 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp45_, "p01") == 0, "nid.get_attribute (\"id\") == \"p01\"");
#line 133 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp45_);
#line 113 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (nid);
#line 113 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (lc);
#line 113 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (le);
#line 113 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (doc);
#line 113 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp0_);
#line 900 "DomGDocumentTest.c"
	}
	goto __finally181;
	__catch181_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp46_ = NULL;
		gchar* _tmp47_ = NULL;
		gchar* _tmp48_ = NULL;
#line 113 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		e = _inner_error_;
#line 113 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_inner_error_ = NULL;
#line 135 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp46_ = e->message;
#line 135 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp47_ = g_strconcat ("Error: ", _tmp46_, NULL);
#line 135 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp48_ = _tmp47_;
#line 135 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_message ("DomGDocumentTest.vala:135: %s", _tmp48_);
#line 135 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp48_);
#line 136 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_assert_not_reached ();
#line 113 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_error_free0 (e);
#line 927 "DomGDocumentTest.c"
	}
	__finally181:
#line 113 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 113 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 113 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 113 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		return;
#line 938 "DomGDocumentTest.c"
	}
}


static void ___lambda195__gtest_func (void) {
#line 112 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	__lambda195_ ();
#line 946 "DomGDocumentTest.c"
}


static gboolean string_contains (const gchar* self, const gchar* needle) {
	gboolean result = FALSE;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 1376 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1376 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	g_return_val_if_fail (needle != NULL, FALSE);
#line 1377 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	_tmp0_ = needle;
#line 1377 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	_tmp1_ = strstr ((gchar*) self, (gchar*) _tmp0_);
#line 1377 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	result = _tmp1_ != NULL;
#line 1377 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	return result;
#line 966 "DomGDocumentTest.c"
}


static void __lambda196_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* _tmp0_ = NULL;
		GXmlGDocument* _tmp1_ = NULL;
		GXmlDomDocument* doc = NULL;
		GXmlGDocument* _tmp2_ = NULL;
		GXmlDomDocument* _tmp3_ = NULL;
		GXmlDomElement* _tmp4_ = NULL;
		GXmlDomElement* _tmp5_ = NULL;
		GXmlDomElement* _tmp6_ = NULL;
		GXmlDomHTMLCollection* _tmp7_ = NULL;
		GXmlDomHTMLCollection* _tmp8_ = NULL;
		GXmlDomHTMLCollection* _tmp9_ = NULL;
		gint _tmp10_ = 0;
		gint _tmp11_ = 0;
		GXmlDomHTMLCollection* le = NULL;
		GXmlDomElement* _tmp12_ = NULL;
		GXmlDomElement* _tmp13_ = NULL;
		GXmlDomElement* _tmp14_ = NULL;
		GXmlDomHTMLCollection* _tmp15_ = NULL;
		GXmlDomHTMLCollection* _tmp16_ = NULL;
		gint _tmp17_ = 0;
		gint _tmp18_ = 0;
		gpointer _tmp19_ = NULL;
		GXmlDomElement* _tmp20_ = NULL;
		gchar* _tmp21_ = NULL;
		gchar* _tmp22_ = NULL;
		gpointer _tmp23_ = NULL;
		GXmlDomElement* _tmp24_ = NULL;
		gchar* _tmp25_ = NULL;
		gchar* _tmp26_ = NULL;
		GXmlDomHTMLCollection* lc = NULL;
		GXmlDomElement* _tmp27_ = NULL;
		GXmlDomElement* _tmp28_ = NULL;
		GXmlDomElement* _tmp29_ = NULL;
		GXmlDomHTMLCollection* _tmp30_ = NULL;
		GXmlDomHTMLCollection* _tmp31_ = NULL;
		gint _tmp32_ = 0;
		gint _tmp33_ = 0;
		gpointer _tmp34_ = NULL;
		GXmlDomElement* _tmp35_ = NULL;
		gchar* _tmp36_ = NULL;
		gchar* _tmp37_ = NULL;
		gchar* _tmp38_ = NULL;
		gpointer _tmp39_ = NULL;
		GXmlDomElement* _tmp40_ = NULL;
		gchar* _tmp41_ = NULL;
		gchar* _tmp42_ = NULL;
		gboolean _tmp43_ = FALSE;
		GXmlDomHTMLCollection* lc2 = NULL;
		GXmlDomElement* _tmp44_ = NULL;
		GXmlDomElement* _tmp45_ = NULL;
		GXmlDomElement* _tmp46_ = NULL;
		GXmlDomHTMLCollection* _tmp47_ = NULL;
		GXmlDomHTMLCollection* _tmp48_ = NULL;
		gint _tmp49_ = 0;
		gint _tmp50_ = 0;
		GXmlDomHTMLCollection* lc3 = NULL;
		GXmlDomElement* _tmp51_ = NULL;
		GXmlDomElement* _tmp52_ = NULL;
		GXmlDomElement* _tmp53_ = NULL;
		GXmlDomHTMLCollection* _tmp54_ = NULL;
		GXmlDomHTMLCollection* _tmp55_ = NULL;
		gint _tmp56_ = 0;
		gint _tmp57_ = 0;
		GXmlDomHTMLCollection* lc4 = NULL;
		GXmlDomElement* _tmp58_ = NULL;
		GXmlDomElement* _tmp59_ = NULL;
		GXmlDomElement* _tmp60_ = NULL;
		GXmlDomHTMLCollection* _tmp61_ = NULL;
		GXmlDomHTMLCollection* _tmp62_ = NULL;
		gint _tmp63_ = 0;
		gint _tmp64_ = 0;
#line 141 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_message ("DomGDocumentTest.vala:141: %s", "Doc: " DOM_GDOCUMENT_TEST_HTMLDOC);
#line 142 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp1_ = gxml_gdocument_new_from_string (DOM_GDOCUMENT_TEST_HTMLDOC, 0, &_inner_error_);
#line 142 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp0_ = _tmp1_;
#line 142 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1052 "DomGDocumentTest.c"
			goto __catch182_g_error;
		}
#line 142 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp2_ = _tmp0_;
#line 142 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp0_ = NULL;
#line 142 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp3_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp2_, GXML_TYPE_DOM_DOCUMENT) ? ((GXmlDomDocument*) _tmp2_) : NULL;
#line 142 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp3_ == NULL) {
#line 142 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 1065 "DomGDocumentTest.c"
		}
#line 142 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		doc = _tmp3_;
#line 143 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (doc, GXML_TYPE_DOM_DOCUMENT), "doc is DomDocument");
#line 144 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp4_ = gxml_dom_document_get_document_element (doc);
#line 144 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp5_ = _tmp4_;
#line 144 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp6_ = _tmp5_;
#line 144 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp7_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp6_);
#line 144 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp8_ = _tmp7_;
#line 144 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp9_ = _tmp8_;
#line 144 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp10_ = gee_collection_get_size ((GeeCollection*) _tmp9_);
#line 144 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp11_ = _tmp10_;
#line 144 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp11_ == 1, "doc.document_element.children.size == 1");
#line 144 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp9_);
#line 144 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp6_);
#line 145 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp12_ = gxml_dom_document_get_document_element (doc);
#line 145 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp13_ = _tmp12_;
#line 145 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp14_ = _tmp13_;
#line 145 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp15_ = gxml_dom_element_get_elements_by_tag_name (_tmp14_, "p");
#line 145 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp16_ = _tmp15_;
#line 145 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp14_);
#line 145 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		le = _tmp16_;
#line 146 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp17_ = gee_collection_get_size ((GeeCollection*) le);
#line 146 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp18_ = _tmp17_;
#line 146 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp18_ == 4, "le.size == 4");
#line 147 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp19_ = gee_list_get ((GeeList*) le, 0);
#line 147 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp20_ = (GXmlDomElement*) _tmp19_;
#line 147 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp21_ = gxml_dom_element_get_attribute (_tmp20_, "class");
#line 147 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp22_ = _tmp21_;
#line 147 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp22_, "black") == 0, "le[0].get_attribute (\"class\") == \"black\"");
#line 147 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp22_);
#line 147 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp20_);
#line 148 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp23_ = gee_list_get ((GeeList*) le, 1);
#line 148 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp24_ = (GXmlDomElement*) _tmp23_;
#line 148 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp25_ = gxml_dom_element_get_attribute (_tmp24_, "id");
#line 148 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp26_ = _tmp25_;
#line 148 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp26_, "p01") == 0, "le[1].get_attribute (\"id\") == \"p01\"");
#line 148 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp26_);
#line 148 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp24_);
#line 149 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp27_ = gxml_dom_document_get_document_element (doc);
#line 149 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp28_ = _tmp27_;
#line 149 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp29_ = _tmp28_;
#line 149 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp30_ = gxml_dom_element_get_elements_by_class_name (_tmp29_, "black");
#line 149 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp31_ = _tmp30_;
#line 149 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp29_);
#line 149 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		lc = _tmp31_;
#line 153 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp32_ = gee_collection_get_size ((GeeCollection*) lc);
#line 153 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp33_ = _tmp32_;
#line 153 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp33_ == 2, "lc.size == 2");
#line 154 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp34_ = gee_list_get ((GeeList*) lc, 0);
#line 154 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp35_ = (GXmlDomElement*) _tmp34_;
#line 154 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp36_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp35_);
#line 154 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp37_ = _tmp36_;
#line 154 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp38_ = _tmp37_;
#line 154 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp38_, "p") == 0, "lc[0].node_name == \"p\"");
#line 154 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp38_);
#line 154 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp35_);
#line 155 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp39_ = gee_list_get ((GeeList*) lc, 0);
#line 155 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp40_ = (GXmlDomElement*) _tmp39_;
#line 155 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp41_ = gxml_dom_element_get_attribute (_tmp40_, "class");
#line 155 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp42_ = _tmp41_;
#line 155 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp43_ = string_contains (_tmp42_, "black");
#line 155 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp43_, "\"black\" in lc[0].get_attribute (\"class\")");
#line 155 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp42_);
#line 155 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp40_);
#line 156 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp44_ = gxml_dom_document_get_document_element (doc);
#line 156 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp45_ = _tmp44_;
#line 156 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp46_ = _tmp45_;
#line 156 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp47_ = gxml_dom_element_get_elements_by_class_name (_tmp46_, "time");
#line 156 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp48_ = _tmp47_;
#line 156 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp46_);
#line 156 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		lc2 = _tmp48_;
#line 157 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp49_ = gee_collection_get_size ((GeeCollection*) lc2);
#line 157 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp50_ = _tmp49_;
#line 157 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp50_ == 1, "lc2.size == 1");
#line 158 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp51_ = gxml_dom_document_get_document_element (doc);
#line 158 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp52_ = _tmp51_;
#line 158 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp53_ = _tmp52_;
#line 158 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp54_ = gxml_dom_element_get_elements_by_class_name (_tmp53_, "time request");
#line 158 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp55_ = _tmp54_;
#line 158 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp53_);
#line 158 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		lc3 = _tmp55_;
#line 159 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp56_ = gee_collection_get_size ((GeeCollection*) lc3);
#line 159 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp57_ = _tmp56_;
#line 159 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp57_ == 1, "lc3.size == 1");
#line 160 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp58_ = gxml_dom_document_get_document_element (doc);
#line 160 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp59_ = _tmp58_;
#line 160 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp60_ = _tmp59_;
#line 160 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp61_ = gxml_dom_element_get_elements_by_class_name (_tmp60_, "time request hole");
#line 160 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp62_ = _tmp61_;
#line 160 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp60_);
#line 160 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		lc4 = _tmp62_;
#line 161 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp63_ = gee_collection_get_size ((GeeCollection*) lc4);
#line 161 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp64_ = _tmp63_;
#line 161 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp64_ == 1, "lc4.size == 1");
#line 140 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (lc4);
#line 140 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (lc3);
#line 140 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (lc2);
#line 140 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (lc);
#line 140 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (le);
#line 140 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (doc);
#line 140 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp0_);
#line 1267 "DomGDocumentTest.c"
	}
	goto __finally182;
	__catch182_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp65_ = NULL;
		gchar* _tmp66_ = NULL;
		gchar* _tmp67_ = NULL;
#line 140 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		e = _inner_error_;
#line 140 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_inner_error_ = NULL;
#line 163 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp65_ = e->message;
#line 163 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp66_ = g_strconcat ("Error: ", _tmp65_, NULL);
#line 163 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp67_ = _tmp66_;
#line 163 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_message ("DomGDocumentTest.vala:163: %s", _tmp67_);
#line 163 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp67_);
#line 164 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_assert_not_reached ();
#line 140 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_error_free0 (e);
#line 1294 "DomGDocumentTest.c"
	}
	__finally182:
#line 140 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 140 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 140 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 140 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		return;
#line 1305 "DomGDocumentTest.c"
	}
}


static void ___lambda196__gtest_func (void) {
#line 139 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	__lambda196_ ();
#line 1313 "DomGDocumentTest.c"
}


static void __lambda197_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* _tmp0_ = NULL;
		GXmlGDocument* _tmp1_ = NULL;
		GXmlDomDocument* doc = NULL;
		GXmlGDocument* _tmp2_ = NULL;
		GXmlDomDocument* _tmp3_ = NULL;
		GXmlDomDocument* _tmp4_ = NULL;
		GXmlDomDocument* _tmp5_ = NULL;
		GXmlDomElement* _tmp6_ = NULL;
		GXmlDomElement* _tmp7_ = NULL;
		GXmlDomElement* _tmp8_ = NULL;
		GXmlDomHTMLCollection* _tmp9_ = NULL;
		GXmlDomHTMLCollection* _tmp10_ = NULL;
		GXmlDomHTMLCollection* _tmp11_ = NULL;
		gint _tmp12_ = 0;
		gint _tmp13_ = 0;
		GXmlDomDocument* _tmp14_ = NULL;
		GXmlDomElement* _tmp15_ = NULL;
		GXmlDomElement* _tmp16_ = NULL;
		GXmlDomElement* _tmp17_ = NULL;
		GXmlDomHTMLCollection* _tmp18_ = NULL;
		GXmlDomHTMLCollection* _tmp19_ = NULL;
		GXmlDomHTMLCollection* _tmp20_ = NULL;
		gpointer _tmp21_ = NULL;
		GXmlDomElement* _tmp22_ = NULL;
		GXmlDomDocument* _tmp23_ = NULL;
		GXmlDomElement* _tmp24_ = NULL;
		GXmlDomElement* _tmp25_ = NULL;
		GXmlDomElement* _tmp26_ = NULL;
		GXmlDomHTMLCollection* _tmp27_ = NULL;
		GXmlDomHTMLCollection* _tmp28_ = NULL;
		GXmlDomHTMLCollection* _tmp29_ = NULL;
		gpointer _tmp30_ = NULL;
		GXmlDomElement* _tmp31_ = NULL;
		GXmlDomHTMLCollection* _tmp32_ = NULL;
		GXmlDomHTMLCollection* _tmp33_ = NULL;
		GXmlDomHTMLCollection* _tmp34_ = NULL;
		gpointer _tmp35_ = NULL;
		GXmlDomElement* _tmp36_ = NULL;
		GXmlDomElement* e = NULL;
		GXmlDomDocument* _tmp37_ = NULL;
		GXmlDomElement* _tmp38_ = NULL;
		GXmlDomElement* _tmp39_ = NULL;
		GXmlDomElement* _tmp40_ = NULL;
		GXmlDomHTMLCollection* _tmp41_ = NULL;
		GXmlDomHTMLCollection* _tmp42_ = NULL;
		GXmlDomHTMLCollection* _tmp43_ = NULL;
		gpointer _tmp44_ = NULL;
		GXmlDomElement* _tmp45_ = NULL;
		GXmlDomHTMLCollection* _tmp46_ = NULL;
		GXmlDomHTMLCollection* _tmp47_ = NULL;
		GXmlDomHTMLCollection* _tmp48_ = NULL;
		gpointer _tmp49_ = NULL;
		GXmlDomElement* _tmp50_ = NULL;
		GXmlDomElement* _tmp51_ = NULL;
		GXmlDomDocument* _tmp52_ = NULL;
		GXmlDomDocument* _tmp53_ = NULL;
		GXmlDomDocument* _tmp54_ = NULL;
		GXmlDomElement* _tmp55_ = NULL;
		GXmlDomNode* _tmp56_ = NULL;
		GXmlDomNode* _tmp57_ = NULL;
		GXmlDomNode* _tmp58_ = NULL;
		GXmlDomElement* _tmp59_ = NULL;
		GXmlDomNode* _tmp60_ = NULL;
		GXmlDomNode* _tmp61_ = NULL;
		GXmlDomNode* _tmp62_ = NULL;
		gchar* _tmp63_ = NULL;
		gchar* _tmp64_ = NULL;
		gchar* _tmp65_ = NULL;
		GXmlDomElement* _tmp66_ = NULL;
		GXmlDomElement* _tmp67_ = NULL;
		GXmlDomElement* _tmp68_ = NULL;
		GXmlDomElement* _tmp69_ = NULL;
		GXmlDomElement* _tmp70_ = NULL;
		GXmlDomElement* _tmp71_ = NULL;
		GXmlDomElement* _tmp72_ = NULL;
		GXmlDomElement* _tmp73_ = NULL;
		gchar* _tmp74_ = NULL;
		gchar* _tmp75_ = NULL;
		gchar* _tmp76_ = NULL;
		GXmlDomElement* _tmp77_ = NULL;
		GXmlDomNodeList* _tmp78_ = NULL;
		GXmlDomNodeList* _tmp79_ = NULL;
		GXmlDomNodeList* _tmp80_ = NULL;
		GXmlDomElement* _tmp81_ = NULL;
		GXmlDomNodeList* _tmp82_ = NULL;
		GXmlDomNodeList* _tmp83_ = NULL;
		GXmlDomNodeList* _tmp84_ = NULL;
		gint _tmp85_ = 0;
		gint _tmp86_ = 0;
		GXmlDomElement* _tmp87_ = NULL;
		GXmlDomNodeList* _tmp88_ = NULL;
		GXmlDomNodeList* _tmp89_ = NULL;
		GXmlDomNodeList* _tmp90_ = NULL;
		GXmlDomNode* _tmp91_ = NULL;
		GXmlDomNode* _tmp92_ = NULL;
		GXmlDomElement* _tmp93_ = NULL;
		GXmlDomNodeList* _tmp94_ = NULL;
		GXmlDomNodeList* _tmp95_ = NULL;
		GXmlDomNodeList* _tmp96_ = NULL;
		GXmlDomNode* _tmp97_ = NULL;
		GXmlDomNode* _tmp98_ = NULL;
		GXmlDomElement* _tmp99_ = NULL;
		GXmlDomNode* _tmp100_ = NULL;
		GXmlDomNode* _tmp101_ = NULL;
		GXmlDomNode* _tmp102_ = NULL;
		GXmlDomElement* _tmp103_ = NULL;
		GXmlDomNode* _tmp104_ = NULL;
		GXmlDomNode* _tmp105_ = NULL;
		GXmlDomNode* _tmp106_ = NULL;
		GXmlDomElement* _tmp107_ = NULL;
		GXmlDomNode* _tmp108_ = NULL;
		GXmlDomNode* _tmp109_ = NULL;
		GXmlDomNode* _tmp110_ = NULL;
		GXmlDomElement* _tmp111_ = NULL;
		GXmlDomNode* _tmp112_ = NULL;
		GXmlDomNode* _tmp113_ = NULL;
		GXmlDomNode* _tmp114_ = NULL;
		GXmlDomNode* t = NULL;
		GXmlDomElement* _tmp115_ = NULL;
		GXmlDomNodeList* _tmp116_ = NULL;
		GXmlDomNodeList* _tmp117_ = NULL;
		GXmlDomNodeList* _tmp118_ = NULL;
		GXmlDomNode* _tmp119_ = NULL;
		GXmlDomNode* _tmp120_ = NULL;
		GXmlDomNode* _tmp121_ = NULL;
		GXmlDomNode* _tmp122_ = NULL;
		GXmlDomNode* _tmp123_ = NULL;
		GXmlDomNode* _tmp124_ = NULL;
		GXmlDomNode* _tmp125_ = NULL;
		GXmlDomNode* _tmp126_ = NULL;
		GXmlDomNode* _tmp127_ = NULL;
		GXmlDomNode* _tmp128_ = NULL;
		GXmlDomNode* _tmp129_ = NULL;
		gchar* _tmp130_ = NULL;
		gchar* _tmp131_ = NULL;
		gchar* _tmp132_ = NULL;
		GXmlDomNode* _tmp133_ = NULL;
		gchar* _tmp134_ = NULL;
		gchar* _tmp135_ = NULL;
		gchar* _tmp136_ = NULL;
		GXmlDomElement* _tmp137_ = NULL;
		GXmlDomNode* _tmp138_ = NULL;
		GXmlDomNode* _tmp139_ = NULL;
		GXmlDomNode* _tmp140_ = NULL;
		gchar* _tmp141_ = NULL;
		gchar* _tmp142_ = NULL;
		gchar* _tmp143_ = NULL;
		GXmlDomElement* _tmp144_ = NULL;
		GXmlDomNode* _tmp145_ = NULL;
		GXmlDomNode* _tmp146_ = NULL;
		GXmlDomNode* _tmp147_ = NULL;
		gboolean _tmp148_ = FALSE;
		GXmlDomElement* _tmp149_ = NULL;
		GXmlDomNode* _tmp150_ = NULL;
		GXmlDomNode* _tmp151_ = NULL;
		GXmlDomNode* _tmp152_ = NULL;
		GXmlDomElement* _tmp153_ = NULL;
		GXmlDomNode* _tmp154_ = NULL;
		GXmlDomNode* _tmp155_ = NULL;
		GXmlDomNode* _tmp156_ = NULL;
		gchar* _tmp157_ = NULL;
		gchar* _tmp158_ = NULL;
		gchar* _tmp159_ = NULL;
		GXmlDomNode* _tmp160_ = NULL;
		GXmlDomElement* _tmp161_ = NULL;
		GXmlDomNode* _tmp162_ = NULL;
		GXmlDomElement* cn = NULL;
		GXmlDomNode* _tmp163_ = NULL;
		GXmlDomElement* _tmp164_ = NULL;
		GXmlDomElement* _tmp165_ = NULL;
		gchar* _tmp166_ = NULL;
		gchar* _tmp167_ = NULL;
		gchar* _tmp168_ = NULL;
		GXmlDomElement* _tmp169_ = NULL;
		gchar* _tmp170_ = NULL;
		gchar* _tmp171_ = NULL;
		GXmlDomElement* _tmp172_ = NULL;
		GXmlDomNodeList* _tmp173_ = NULL;
		GXmlDomNodeList* _tmp174_ = NULL;
		GXmlDomNodeList* _tmp175_ = NULL;
		GXmlDomElement* _tmp176_ = NULL;
		GXmlDomNodeList* _tmp177_ = NULL;
		GXmlDomNodeList* _tmp178_ = NULL;
		GXmlDomNodeList* _tmp179_ = NULL;
		gint _tmp180_ = 0;
		gint _tmp181_ = 0;
		GXmlDomNode* _tmp182_ = NULL;
		GXmlDomElement* _tmp183_ = NULL;
		GXmlDomNode* _tmp184_ = NULL;
		GXmlDomElement* cn2 = NULL;
		GXmlDomNode* _tmp185_ = NULL;
		GXmlDomElement* _tmp186_ = NULL;
		GXmlDomElement* _tmp187_ = NULL;
		gchar* _tmp188_ = NULL;
		gchar* _tmp189_ = NULL;
		gchar* _tmp190_ = NULL;
		GXmlDomElement* _tmp191_ = NULL;
		gchar* _tmp192_ = NULL;
		gchar* _tmp193_ = NULL;
		GXmlDomElement* _tmp194_ = NULL;
		GXmlDomNodeList* _tmp195_ = NULL;
		GXmlDomNodeList* _tmp196_ = NULL;
		GXmlDomNodeList* _tmp197_ = NULL;
		GXmlDomElement* _tmp198_ = NULL;
		GXmlDomNodeList* _tmp199_ = NULL;
		GXmlDomNodeList* _tmp200_ = NULL;
		GXmlDomNodeList* _tmp201_ = NULL;
		gint _tmp202_ = 0;
		gint _tmp203_ = 0;
		GXmlDomElement* _tmp204_ = NULL;
		GXmlDomNodeList* _tmp205_ = NULL;
		GXmlDomNodeList* _tmp206_ = NULL;
		GXmlDomNodeList* _tmp207_ = NULL;
		gpointer _tmp208_ = NULL;
		GXmlDomNode* _tmp209_ = NULL;
		GXmlDomElement* _tmp210_ = NULL;
		gchar* _tmp211_ = NULL;
		gchar* _tmp212_ = NULL;
		gchar* _tmp213_ = NULL;
		GXmlDomElement* _tmp214_ = NULL;
		GXmlDomElement* _tmp215_ = NULL;
		gboolean _tmp216_ = FALSE;
		GXmlDomElement* _tmp217_ = NULL;
		GXmlDomElement* _tmp218_ = NULL;
		gboolean _tmp219_ = FALSE;
		GXmlDomElement* e0 = NULL;
		GXmlDomDocument* _tmp220_ = NULL;
		GXmlDomElement* _tmp221_ = NULL;
		GXmlDomElement* _tmp222_ = NULL;
		GXmlDomElement* _tmp223_ = NULL;
		GXmlDomHTMLCollection* _tmp224_ = NULL;
		GXmlDomHTMLCollection* _tmp225_ = NULL;
		GXmlDomHTMLCollection* _tmp226_ = NULL;
		gpointer _tmp227_ = NULL;
		GXmlDomElement* _tmp228_ = NULL;
		GXmlDomHTMLCollection* _tmp229_ = NULL;
		GXmlDomHTMLCollection* _tmp230_ = NULL;
		GXmlDomHTMLCollection* _tmp231_ = NULL;
		gpointer _tmp232_ = NULL;
		GXmlDomElement* _tmp233_ = NULL;
		GXmlDomElement* _tmp234_ = NULL;
		gchar* _tmp235_ = NULL;
		gchar* _tmp236_ = NULL;
		gchar* _tmp237_ = NULL;
		GXmlDomDocument* _tmp238_ = NULL;
		GXmlDomElement* _tmp239_ = NULL;
		GXmlDomElement* _tmp240_ = NULL;
		GXmlDomElement* _tmp241_ = NULL;
		GXmlDomHTMLCollection* _tmp242_ = NULL;
		GXmlDomHTMLCollection* _tmp243_ = NULL;
		GXmlDomHTMLCollection* _tmp244_ = NULL;
		gpointer _tmp245_ = NULL;
		GXmlDomElement* _tmp246_ = NULL;
		GXmlDomNodeList* _tmp247_ = NULL;
		GXmlDomNodeList* _tmp248_ = NULL;
		GXmlDomNodeList* _tmp249_ = NULL;
		GXmlDomElement* _tmp250_ = NULL;
		gint _tmp251_ = 0;
		GXmlDomElement* e1 = NULL;
		GXmlDomDocument* _tmp252_ = NULL;
		GXmlDomElement* _tmp253_ = NULL;
		GXmlDomElement* _tmp254_ = NULL;
		GXmlDomElement* _tmp255_ = NULL;
		GXmlDomHTMLCollection* _tmp256_ = NULL;
		GXmlDomHTMLCollection* _tmp257_ = NULL;
		GXmlDomHTMLCollection* _tmp258_ = NULL;
		gpointer _tmp259_ = NULL;
		GXmlDomElement* _tmp260_ = NULL;
		GXmlDomHTMLCollection* _tmp261_ = NULL;
		GXmlDomHTMLCollection* _tmp262_ = NULL;
		GXmlDomHTMLCollection* _tmp263_ = NULL;
		gpointer _tmp264_ = NULL;
		GXmlDomElement* _tmp265_ = NULL;
		GXmlDomElement* _tmp266_ = NULL;
		gchar* _tmp267_ = NULL;
		gchar* _tmp268_ = NULL;
		gchar* _tmp269_ = NULL;
		GXmlDomDocument* _tmp270_ = NULL;
		GXmlDomElement* _tmp271_ = NULL;
		GXmlDomElement* _tmp272_ = NULL;
		GXmlDomElement* _tmp273_ = NULL;
		GXmlDomHTMLCollection* _tmp274_ = NULL;
		GXmlDomHTMLCollection* _tmp275_ = NULL;
		GXmlDomHTMLCollection* _tmp276_ = NULL;
		gpointer _tmp277_ = NULL;
		GXmlDomElement* _tmp278_ = NULL;
		GXmlDomNodeList* _tmp279_ = NULL;
		GXmlDomNodeList* _tmp280_ = NULL;
		GXmlDomNodeList* _tmp281_ = NULL;
		GXmlDomElement* _tmp282_ = NULL;
		gint _tmp283_ = 0;
		GXmlDomElement* e2 = NULL;
		GXmlDomDocument* _tmp284_ = NULL;
		GXmlDomElement* _tmp285_ = NULL;
		GXmlDomElement* _tmp286_ = NULL;
		GXmlDomElement* _tmp287_ = NULL;
		GXmlDomHTMLCollection* _tmp288_ = NULL;
		GXmlDomHTMLCollection* _tmp289_ = NULL;
		GXmlDomHTMLCollection* _tmp290_ = NULL;
		gpointer _tmp291_ = NULL;
		GXmlDomElement* _tmp292_ = NULL;
		GXmlDomHTMLCollection* _tmp293_ = NULL;
		GXmlDomHTMLCollection* _tmp294_ = NULL;
		GXmlDomHTMLCollection* _tmp295_ = NULL;
		gpointer _tmp296_ = NULL;
		GXmlDomElement* _tmp297_ = NULL;
		GXmlDomElement* _tmp298_ = NULL;
		gchar* _tmp299_ = NULL;
		gchar* _tmp300_ = NULL;
		gchar* _tmp301_ = NULL;
		GXmlDomDocument* _tmp302_ = NULL;
		GXmlDomElement* _tmp303_ = NULL;
		GXmlDomElement* _tmp304_ = NULL;
		GXmlDomElement* _tmp305_ = NULL;
		GXmlDomHTMLCollection* _tmp306_ = NULL;
		GXmlDomHTMLCollection* _tmp307_ = NULL;
		GXmlDomHTMLCollection* _tmp308_ = NULL;
		gpointer _tmp309_ = NULL;
		GXmlDomElement* _tmp310_ = NULL;
		GXmlDomNodeList* _tmp311_ = NULL;
		GXmlDomNodeList* _tmp312_ = NULL;
		GXmlDomNodeList* _tmp313_ = NULL;
		GXmlDomElement* _tmp314_ = NULL;
		gint _tmp315_ = 0;
		GXmlDomElement* _tmp316_ = NULL;
		GXmlDomNode* _tmp317_ = NULL;
		GXmlDomNode* _tmp318_ = NULL;
		GXmlDomNode* _tmp319_ = NULL;
		GXmlDomElement* _tmp320_ = NULL;
		gboolean _tmp321_ = FALSE;
		GXmlDomElement* _tmp322_ = NULL;
		GXmlDomNode* _tmp323_ = NULL;
		GXmlDomNode* _tmp324_ = NULL;
		GXmlDomNode* _tmp325_ = NULL;
		GXmlDomElement* _tmp326_ = NULL;
		gboolean _tmp327_ = FALSE;
		GXmlDomElement* _tmp328_ = NULL;
		GXmlDomNode* _tmp329_ = NULL;
		GXmlDomNode* _tmp330_ = NULL;
		GXmlDomNode* _tmp331_ = NULL;
		GXmlDomElement* _tmp332_ = NULL;
		gboolean _tmp333_ = FALSE;
		gboolean _tmp334_ = FALSE;
		GXmlDomElement* _tmp335_ = NULL;
		GXmlDomNode* _tmp336_ = NULL;
		GXmlDomNode* _tmp337_ = NULL;
		GXmlDomNode* _tmp338_ = NULL;
		GXmlDomElement* _tmp339_ = NULL;
		gboolean _tmp340_ = FALSE;
		gboolean _tmp341_ = FALSE;
		GXmlDomElement* _tmp348_ = NULL;
		GXmlDomElement* _tmp349_ = NULL;
		GXmlDomNodeDocumentPosition _tmp350_ = 0;
		gboolean _tmp351_ = FALSE;
		GXmlDomElement* _tmp352_ = NULL;
		GXmlDomNode* _tmp353_ = NULL;
		GXmlDomNode* _tmp354_ = NULL;
		GXmlDomNode* _tmp355_ = NULL;
		GXmlDomElement* _tmp356_ = NULL;
		gboolean _tmp357_ = FALSE;
		gboolean _tmp358_ = FALSE;
		GXmlDomElement* _tmp365_ = NULL;
		GXmlDomElement* _tmp366_ = NULL;
		GXmlDomNodeDocumentPosition _tmp367_ = 0;
		GXmlDomElement* _tmp368_ = NULL;
		GXmlDomNode* _tmp369_ = NULL;
		GXmlDomNode* _tmp370_ = NULL;
		GXmlDomNode* _tmp371_ = NULL;
		GXmlDomElement* _tmp372_ = NULL;
		gboolean _tmp373_ = FALSE;
		GXmlDomElement* _tmp374_ = NULL;
		GXmlDomElement* _tmp375_ = NULL;
		GXmlDomNode* _tmp376_ = NULL;
		GXmlDomNode* _tmp377_ = NULL;
		GXmlDomNode* _tmp378_ = NULL;
		GXmlDomNodeDocumentPosition _tmp379_ = 0;
		GXmlDomElement* _tmp380_ = NULL;
		GXmlDomElement* _tmp381_ = NULL;
		GXmlDomNodeList* _tmp382_ = NULL;
		GXmlDomNodeList* _tmp383_ = NULL;
		GXmlDomNodeList* _tmp384_ = NULL;
		gpointer _tmp385_ = NULL;
		GXmlDomNode* _tmp386_ = NULL;
		gboolean _tmp387_ = FALSE;
		GXmlDomElement* _tmp388_ = NULL;
		GXmlDomElement* _tmp389_ = NULL;
		GXmlDomNodeList* _tmp390_ = NULL;
		GXmlDomNodeList* _tmp391_ = NULL;
		GXmlDomNodeList* _tmp392_ = NULL;
		gpointer _tmp393_ = NULL;
		GXmlDomNode* _tmp394_ = NULL;
		GXmlDomNodeDocumentPosition _tmp395_ = 0;
		GXmlDomElement* b = NULL;
		GXmlDomDocument* _tmp396_ = NULL;
		GXmlDomElement* _tmp397_ = NULL;
		GXmlDomElement* _tmp398_ = NULL;
		GXmlDomElement* _tmp399_ = NULL;
		GXmlDomHTMLCollection* _tmp400_ = NULL;
		GXmlDomHTMLCollection* _tmp401_ = NULL;
		GXmlDomHTMLCollection* _tmp402_ = NULL;
		gpointer _tmp403_ = NULL;
		GXmlDomElement* _tmp404_ = NULL;
		GXmlDomElement* _tmp405_ = NULL;
		gchar* _tmp406_ = NULL;
		gchar* _tmp407_ = NULL;
		gchar* _tmp408_ = NULL;
		GXmlDomElement* ng = NULL;
		GXmlDomDocument* _tmp409_ = NULL;
		GXmlDomElement* _tmp410_ = NULL;
		GXmlDomElement* _tmp411_ = NULL;
		GXmlDomNodeList* _tmp412_ = NULL;
		GXmlDomNodeList* _tmp413_ = NULL;
		GXmlDomNodeList* _tmp414_ = NULL;
		GXmlDomElement* _tmp415_ = NULL;
		GXmlDomElement* _tmp416_ = NULL;
		gchar* _tmp417_ = NULL;
		gchar* _tmp418_ = NULL;
		GXmlDomElement* _tmp419_ = NULL;
		gchar* _tmp420_ = NULL;
		gchar* _tmp421_ = NULL;
		GXmlDomElement* _tmp422_ = NULL;
		gboolean _tmp423_ = FALSE;
		GXmlDomElement* ng2 = NULL;
		GXmlDomDocument* _tmp424_ = NULL;
		GXmlDomElement* _tmp425_ = NULL;
		GXmlDomElement* _tmp426_ = NULL;
		GXmlDomNodeList* _tmp427_ = NULL;
		GXmlDomNodeList* _tmp428_ = NULL;
		GXmlDomNodeList* _tmp429_ = NULL;
		GXmlDomElement* _tmp430_ = NULL;
		GXmlDomElement* _tmp431_ = NULL;
		gchar* _tmp432_ = NULL;
		gchar* _tmp433_ = NULL;
		GXmlDomElement* _tmp434_ = NULL;
		gchar* _tmp435_ = NULL;
		gchar* _tmp436_ = NULL;
		GXmlDomElement* _tmp437_ = NULL;
		gboolean _tmp438_ = FALSE;
		GXmlDomElement* _tmp439_ = NULL;
		GXmlDomDocument* _tmp440_ = NULL;
		GXmlDomElement* _tmp441_ = NULL;
		GXmlDomElement* pn = NULL;
		GXmlDomElement* _tmp442_ = NULL;
		GXmlDomElement* _tmp443_ = NULL;
		GXmlDomElement* _tmp444_ = NULL;
		GXmlDomElement* p = NULL;
		GXmlDomDocument* _tmp445_ = NULL;
		GXmlDomElement* _tmp446_ = NULL;
		GXmlDomElement* _tmp447_ = NULL;
		GXmlDomElement* _tmp448_ = NULL;
		GXmlDomHTMLCollection* _tmp449_ = NULL;
		GXmlDomHTMLCollection* _tmp450_ = NULL;
		GXmlDomHTMLCollection* _tmp451_ = NULL;
		gpointer _tmp452_ = NULL;
		GXmlDomElement* _tmp453_ = NULL;
		GXmlDomElement* _tmp454_ = NULL;
		gchar* _tmp455_ = NULL;
		gchar* _tmp456_ = NULL;
		gchar* _tmp457_ = NULL;
		GXmlDomElement* cp0 = NULL;
		GXmlDomElement* _tmp458_ = NULL;
		GXmlDomHTMLCollection* _tmp459_ = NULL;
		GXmlDomHTMLCollection* _tmp460_ = NULL;
		GXmlDomHTMLCollection* _tmp461_ = NULL;
		gpointer _tmp462_ = NULL;
		GXmlDomElement* _tmp463_ = NULL;
		GXmlDomElement* _tmp464_ = NULL;
		gchar* _tmp465_ = NULL;
		gchar* _tmp466_ = NULL;
		gchar* _tmp467_ = NULL;
		GXmlDomElement* _tmp468_ = NULL;
		gchar* _tmp469_ = NULL;
		gchar* _tmp470_ = NULL;
		GXmlDomElement* _tmp471_ = NULL;
		GXmlDomElement* _tmp472_ = NULL;
		gboolean _tmp473_ = FALSE;
		GXmlDomElement* _tmp474_ = NULL;
		GXmlDomNode* _tmp475_ = NULL;
		GXmlDomNode* _tmp476_ = NULL;
		GXmlDomNode* _tmp477_ = NULL;
		GXmlDomElement* _tmp478_ = NULL;
		gboolean _tmp479_ = FALSE;
		GXmlDomElement* _tmp480_ = NULL;
		GXmlDomElement* _tmp481_ = NULL;
		GXmlDomElement* _tmp482_ = NULL;
		GXmlDomNode* _tmp483_ = NULL;
		GXmlDomNode* _tmp484_ = NULL;
		GXmlDomElement* ppn = NULL;
		GXmlDomElement* _tmp485_ = NULL;
		GXmlDomHTMLCollection* _tmp486_ = NULL;
		GXmlDomHTMLCollection* _tmp487_ = NULL;
		GXmlDomHTMLCollection* _tmp488_ = NULL;
		gpointer _tmp489_ = NULL;
		GXmlDomElement* _tmp490_ = NULL;
		GXmlDomElement* _tmp491_ = NULL;
		gchar* _tmp492_ = NULL;
		gchar* _tmp493_ = NULL;
		gchar* _tmp494_ = NULL;
		GXmlDomElement* _tmp495_ = NULL;
		gboolean _tmp496_ = FALSE;
		GXmlDomElement* _tmp497_ = NULL;
		gchar* _tmp498_ = NULL;
		gchar* _tmp499_ = NULL;
		GXmlDomElement* _tmp500_ = NULL;
		GXmlDomDocument* _tmp501_ = NULL;
		GXmlDomElement* _tmp502_ = NULL;
		GXmlDomElement* pn2 = NULL;
		GXmlDomElement* _tmp503_ = NULL;
		GXmlDomElement* _tmp504_ = NULL;
		GXmlDomElement* _tmp505_ = NULL;
		GXmlDomElement* _tmp506_ = NULL;
		GXmlDomElement* _tmp507_ = NULL;
		GXmlDomNode* _tmp508_ = NULL;
		GXmlDomNode* _tmp509_ = NULL;
		GXmlDomElement* _tmp510_ = NULL;
		GXmlDomHTMLCollection* _tmp511_ = NULL;
		GXmlDomHTMLCollection* _tmp512_ = NULL;
		GXmlDomHTMLCollection* _tmp513_ = NULL;
		gint _tmp514_ = 0;
		gint _tmp515_ = 0;
		GXmlDomElement* _tmp516_ = NULL;
		GXmlDomHTMLCollection* _tmp517_ = NULL;
		GXmlDomHTMLCollection* _tmp518_ = NULL;
		GXmlDomHTMLCollection* _tmp519_ = NULL;
		gpointer _tmp520_ = NULL;
		GXmlDomElement* _tmp521_ = NULL;
		GXmlDomElement* _tmp522_ = NULL;
		GXmlDomHTMLCollection* _tmp523_ = NULL;
		GXmlDomHTMLCollection* _tmp524_ = NULL;
		GXmlDomHTMLCollection* _tmp525_ = NULL;
		gpointer _tmp526_ = NULL;
		GXmlDomElement* _tmp527_ = NULL;
		gchar* _tmp528_ = NULL;
		gchar* _tmp529_ = NULL;
		gchar* _tmp530_ = NULL;
		GXmlDomElement* _tmp531_ = NULL;
		GXmlDomHTMLCollection* _tmp532_ = NULL;
		GXmlDomHTMLCollection* _tmp533_ = NULL;
		GXmlDomHTMLCollection* _tmp534_ = NULL;
		gpointer _tmp535_ = NULL;
		GXmlDomElement* _tmp536_ = NULL;
		gchar* _tmp537_ = NULL;
		gchar* _tmp538_ = NULL;
		GXmlDomElement* _tmp539_ = NULL;
		GXmlDomDocument* _tmp540_ = NULL;
		GXmlDomElement* _tmp541_ = NULL;
		GXmlDomElement* pn3 = NULL;
		GXmlDomElement* _tmp542_ = NULL;
		GXmlDomElement* _tmp543_ = NULL;
		GXmlDomElement* _tmp544_ = NULL;
		GXmlDomElement* _tmp545_ = NULL;
		GXmlDomElement* _tmp546_ = NULL;
		GXmlDomElement* _tmp547_ = NULL;
		GXmlDomElement* _tmp548_ = NULL;
		GXmlDomNode* _tmp549_ = NULL;
		GXmlDomNode* _tmp550_ = NULL;
		GXmlDomElement* _tmp551_ = NULL;
		GXmlDomHTMLCollection* _tmp552_ = NULL;
		GXmlDomHTMLCollection* _tmp553_ = NULL;
		GXmlDomHTMLCollection* _tmp554_ = NULL;
		gint _tmp555_ = 0;
		gint _tmp556_ = 0;
		GXmlDomElement* _tmp557_ = NULL;
		GXmlDomHTMLCollection* _tmp558_ = NULL;
		GXmlDomHTMLCollection* _tmp559_ = NULL;
		GXmlDomHTMLCollection* _tmp560_ = NULL;
		gpointer _tmp561_ = NULL;
		GXmlDomElement* _tmp562_ = NULL;
		GXmlDomElement* _tmp563_ = NULL;
		GXmlDomHTMLCollection* _tmp564_ = NULL;
		GXmlDomHTMLCollection* _tmp565_ = NULL;
		GXmlDomHTMLCollection* _tmp566_ = NULL;
		gpointer _tmp567_ = NULL;
		GXmlDomElement* _tmp568_ = NULL;
		gchar* _tmp569_ = NULL;
		gchar* _tmp570_ = NULL;
		gchar* _tmp571_ = NULL;
		GXmlDomElement* _tmp572_ = NULL;
		GXmlDomHTMLCollection* _tmp573_ = NULL;
		GXmlDomHTMLCollection* _tmp574_ = NULL;
		GXmlDomHTMLCollection* _tmp575_ = NULL;
		gpointer _tmp576_ = NULL;
		GXmlDomElement* _tmp577_ = NULL;
		gchar* _tmp578_ = NULL;
		gchar* _tmp579_ = NULL;
		GXmlDomElement* _tmp580_ = NULL;
		GXmlDomHTMLCollection* _tmp581_ = NULL;
		GXmlDomHTMLCollection* _tmp582_ = NULL;
		GXmlDomHTMLCollection* _tmp583_ = NULL;
		gpointer _tmp584_ = NULL;
		GXmlDomElement* _tmp585_ = NULL;
		gchar* _tmp586_ = NULL;
		gchar* _tmp587_ = NULL;
		GXmlDomElement* _tmp588_ = NULL;
		GXmlDomDocument* _tmp589_ = NULL;
		GXmlDomElement* _tmp590_ = NULL;
		GXmlDomElement* pn4 = NULL;
		GXmlDomElement* _tmp591_ = NULL;
		GXmlDomElement* _tmp592_ = NULL;
		GXmlDomElement* _tmp593_ = NULL;
		GXmlDomElement* _tmp594_ = NULL;
		GXmlDomElement* _tmp595_ = NULL;
		GXmlDomElement* _tmp596_ = NULL;
		GXmlDomElement* _tmp597_ = NULL;
		GXmlDomNodeList* _tmp598_ = NULL;
		GXmlDomNodeList* _tmp599_ = NULL;
		GXmlDomNodeList* _tmp600_ = NULL;
		gpointer _tmp601_ = NULL;
		GXmlDomNode* _tmp602_ = NULL;
		GXmlDomNode* _tmp603_ = NULL;
		GXmlDomNode* _tmp604_ = NULL;
		GXmlDomElement* _tmp605_ = NULL;
		GXmlDomHTMLCollection* _tmp606_ = NULL;
		GXmlDomHTMLCollection* _tmp607_ = NULL;
		GXmlDomHTMLCollection* _tmp608_ = NULL;
		gint _tmp609_ = 0;
		gint _tmp610_ = 0;
		GXmlDomElement* _tmp611_ = NULL;
		GXmlDomHTMLCollection* _tmp612_ = NULL;
		GXmlDomHTMLCollection* _tmp613_ = NULL;
		GXmlDomHTMLCollection* _tmp614_ = NULL;
		gpointer _tmp615_ = NULL;
		GXmlDomElement* _tmp616_ = NULL;
		GXmlDomElement* _tmp617_ = NULL;
		GXmlDomHTMLCollection* _tmp618_ = NULL;
		GXmlDomHTMLCollection* _tmp619_ = NULL;
		GXmlDomHTMLCollection* _tmp620_ = NULL;
		gpointer _tmp621_ = NULL;
		GXmlDomElement* _tmp622_ = NULL;
		gchar* _tmp623_ = NULL;
		gchar* _tmp624_ = NULL;
		gchar* _tmp625_ = NULL;
		GXmlDomElement* _tmp626_ = NULL;
		GXmlDomHTMLCollection* _tmp627_ = NULL;
		GXmlDomHTMLCollection* _tmp628_ = NULL;
		GXmlDomHTMLCollection* _tmp629_ = NULL;
		gpointer _tmp630_ = NULL;
		GXmlDomElement* _tmp631_ = NULL;
		gchar* _tmp632_ = NULL;
		gchar* _tmp633_ = NULL;
		GXmlDomElement* _tmp634_ = NULL;
		GXmlDomHTMLCollection* _tmp635_ = NULL;
		GXmlDomHTMLCollection* _tmp636_ = NULL;
		GXmlDomHTMLCollection* _tmp637_ = NULL;
		gpointer _tmp638_ = NULL;
		GXmlDomElement* _tmp639_ = NULL;
		gchar* _tmp640_ = NULL;
		gchar* _tmp641_ = NULL;
		GXmlDomElement* _tmp642_ = NULL;
		GXmlDomElement* _tmp643_ = NULL;
		GXmlDomHTMLCollection* _tmp644_ = NULL;
		GXmlDomHTMLCollection* _tmp645_ = NULL;
		GXmlDomHTMLCollection* _tmp646_ = NULL;
		gpointer _tmp647_ = NULL;
		GXmlDomElement* _tmp648_ = NULL;
		GXmlDomNode* _tmp649_ = NULL;
		GXmlDomNode* _tmp650_ = NULL;
		GXmlDomElement* _tmp651_ = NULL;
		GXmlDomHTMLCollection* _tmp652_ = NULL;
		GXmlDomHTMLCollection* _tmp653_ = NULL;
		GXmlDomHTMLCollection* _tmp654_ = NULL;
		gint _tmp655_ = 0;
		gint _tmp656_ = 0;
		GXmlDomElement* _tmp657_ = NULL;
		GXmlDomHTMLCollection* _tmp658_ = NULL;
		GXmlDomHTMLCollection* _tmp659_ = NULL;
		GXmlDomHTMLCollection* _tmp660_ = NULL;
		gpointer _tmp661_ = NULL;
		GXmlDomElement* _tmp662_ = NULL;
		GXmlDomElement* _tmp663_ = NULL;
		GXmlDomHTMLCollection* _tmp664_ = NULL;
		GXmlDomHTMLCollection* _tmp665_ = NULL;
		GXmlDomHTMLCollection* _tmp666_ = NULL;
		gpointer _tmp667_ = NULL;
		GXmlDomElement* _tmp668_ = NULL;
		gchar* _tmp669_ = NULL;
		gchar* _tmp670_ = NULL;
		gchar* _tmp671_ = NULL;
		GXmlDomElement* _tmp672_ = NULL;
		gboolean _tmp673_ = FALSE;
		GXmlDomElement* _tmp674_ = NULL;
		GXmlDomHTMLCollection* _tmp675_ = NULL;
		GXmlDomHTMLCollection* _tmp676_ = NULL;
		GXmlDomHTMLCollection* _tmp677_ = NULL;
		gpointer _tmp678_ = NULL;
		GXmlDomElement* _tmp679_ = NULL;
		gchar* _tmp680_ = NULL;
		gchar* _tmp681_ = NULL;
		GXmlDomElement* _tmp682_ = NULL;
		GXmlDomElement* _tmp683_ = NULL;
		gchar* _tmp684_ = NULL;
		gchar* _tmp685_ = NULL;
		gchar* _tmp686_ = NULL;
		GXmlDomElement* _tmp687_ = NULL;
		gchar* _tmp688_ = NULL;
		gchar* _tmp689_ = NULL;
		GXmlDomElement* _tmp690_ = NULL;
		gchar* _tmp691_ = NULL;
		gchar* _tmp692_ = NULL;
		GXmlDomElement* _tmp693_ = NULL;
		gchar* _tmp694_ = NULL;
		gchar* _tmp695_ = NULL;
		gchar* _tmp696_ = NULL;
		GXmlDomHTMLCollection* l = NULL;
		GXmlDomDocument* _tmp697_ = NULL;
		GXmlDomElement* _tmp698_ = NULL;
		GXmlDomElement* _tmp699_ = NULL;
		GXmlDomElement* _tmp700_ = NULL;
		GXmlDomHTMLCollection* _tmp701_ = NULL;
		GXmlDomHTMLCollection* _tmp702_ = NULL;
		gint _tmp703_ = 0;
		gint _tmp704_ = 0;
		gpointer _tmp705_ = NULL;
		GXmlDomElement* _tmp706_ = NULL;
		gpointer _tmp707_ = NULL;
		GXmlDomElement* _tmp708_ = NULL;
		gpointer _tmp709_ = NULL;
		GXmlDomElement* _tmp710_ = NULL;
		gpointer _tmp711_ = NULL;
		GXmlDomElement* _tmp712_ = NULL;
		gchar* _tmp713_ = NULL;
		gchar* _tmp714_ = NULL;
		gchar* _tmp715_ = NULL;
		gpointer _tmp716_ = NULL;
		GXmlDomElement* _tmp717_ = NULL;
		gchar* _tmp718_ = NULL;
		gchar* _tmp719_ = NULL;
		gchar* _tmp720_ = NULL;
		gpointer _tmp721_ = NULL;
		GXmlDomElement* _tmp722_ = NULL;
		gchar* _tmp723_ = NULL;
		gchar* _tmp724_ = NULL;
		gchar* _tmp725_ = NULL;
		GXmlDomDocument* _tmp726_ = NULL;
		GXmlDomElement* _tmp727_ = NULL;
		GXmlDomElement* _tmp728_ = NULL;
		GXmlDomElement* _tmp729_ = NULL;
		GXmlDomHTMLCollection* _tmp730_ = NULL;
		GXmlDomHTMLCollection* _tmp731_ = NULL;
		GXmlDomHTMLCollection* _tmp732_ = NULL;
		gint _tmp733_ = 0;
		gint _tmp734_ = 0;
		GXmlDomHTMLCollection* lnst = NULL;
		GXmlDomDocument* _tmp735_ = NULL;
		GXmlDomElement* _tmp736_ = NULL;
		GXmlDomElement* _tmp737_ = NULL;
		GXmlDomElement* _tmp738_ = NULL;
		GXmlDomHTMLCollection* _tmp739_ = NULL;
		GXmlDomHTMLCollection* _tmp740_ = NULL;
		gint _tmp741_ = 0;
		gint _tmp742_ = 0;
		GXmlDomElement* nnst = NULL;
		GXmlDomElement* _tmp743_ = NULL;
		gchar* _tmp744_ = NULL;
		gchar* _tmp745_ = NULL;
		gchar* _tmp746_ = NULL;
		GXmlDomHTMLCollection* lns = NULL;
		GXmlDomDocument* _tmp747_ = NULL;
		GXmlDomElement* _tmp748_ = NULL;
		GXmlDomElement* _tmp749_ = NULL;
		GXmlDomElement* _tmp750_ = NULL;
		GXmlDomHTMLCollection* _tmp751_ = NULL;
		GXmlDomHTMLCollection* _tmp752_ = NULL;
		gint _tmp753_ = 0;
		gint _tmp754_ = 0;
		GXmlDomElement* _tmp755_ = NULL;
		GXmlDomElement* _tmp756_ = NULL;
		GXmlDomElement* _tmp757_ = NULL;
		GXmlDomElement* _tmp758_ = NULL;
		gchar* _tmp759_ = NULL;
		gchar* _tmp760_ = NULL;
		gchar* _tmp761_ = NULL;
		GXmlDomHTMLCollection* lcl = NULL;
		GXmlDomDocument* _tmp762_ = NULL;
		GXmlDomElement* _tmp763_ = NULL;
		GXmlDomElement* _tmp764_ = NULL;
		GXmlDomElement* _tmp765_ = NULL;
		GXmlDomHTMLCollection* _tmp766_ = NULL;
		GXmlDomHTMLCollection* _tmp767_ = NULL;
		gint _tmp768_ = 0;
		gint _tmp769_ = 0;
		GXmlDomElement* _tmp770_ = NULL;
		GXmlDomElement* _tmp771_ = NULL;
		GXmlDomElement* _tmp772_ = NULL;
		GXmlDomElement* _tmp773_ = NULL;
		GXmlDomElement* _tmp774_ = NULL;
		GXmlDomElement* _tmp775_ = NULL;
		gchar* _tmp776_ = NULL;
		gchar* _tmp777_ = NULL;
		gchar* _tmp778_ = NULL;
		GXmlDomElement* _tmp779_ = NULL;
		GXmlDomElement* _tmp780_ = NULL;
		gchar* _tmp781_ = NULL;
		gchar* _tmp782_ = NULL;
		gchar* _tmp783_ = NULL;
#line 172 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp1_ = gxml_gdocument_new_from_string (DOM_GDOCUMENT_TEST_HTMLDOC, 0, &_inner_error_);
#line 172 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp0_ = _tmp1_;
#line 172 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 2121 "DomGDocumentTest.c"
			goto __catch183_g_error;
		}
#line 172 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp2_ = _tmp0_;
#line 172 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp0_ = NULL;
#line 172 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp3_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp2_, GXML_TYPE_DOM_DOCUMENT) ? ((GXmlDomDocument*) _tmp2_) : NULL;
#line 172 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp3_ == NULL) {
#line 172 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 2134 "DomGDocumentTest.c"
		}
#line 172 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		doc = _tmp3_;
#line 173 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp4_ = doc;
#line 173 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp4_, GXML_TYPE_DOM_DOCUMENT), "doc is DomDocument");
#line 174 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp5_ = doc;
#line 174 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp6_ = gxml_dom_document_get_document_element (_tmp5_);
#line 174 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp7_ = _tmp6_;
#line 174 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp8_ = _tmp7_;
#line 174 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp9_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp8_);
#line 174 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp10_ = _tmp9_;
#line 174 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp11_ = _tmp10_;
#line 174 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp12_ = gee_collection_get_size ((GeeCollection*) _tmp11_);
#line 174 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp13_ = _tmp12_;
#line 174 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp13_ == 1, "doc.document_element.children.size == 1");
#line 174 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp11_);
#line 174 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp8_);
#line 175 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp14_ = doc;
#line 175 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp15_ = gxml_dom_document_get_document_element (_tmp14_);
#line 175 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp16_ = _tmp15_;
#line 175 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp17_ = _tmp16_;
#line 175 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp18_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp17_);
#line 175 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp19_ = _tmp18_;
#line 175 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp20_ = _tmp19_;
#line 175 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp21_ = gee_list_get ((GeeList*) _tmp20_, 0);
#line 175 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp22_ = (GXmlDomElement*) _tmp21_;
#line 175 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp22_ != NULL, "doc.document_element.children[0] != null");
#line 175 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp22_);
#line 175 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp20_);
#line 175 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp17_);
#line 176 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp23_ = doc;
#line 176 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp24_ = gxml_dom_document_get_document_element (_tmp23_);
#line 176 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp25_ = _tmp24_;
#line 176 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp26_ = _tmp25_;
#line 176 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp27_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp26_);
#line 176 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp28_ = _tmp27_;
#line 176 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp29_ = _tmp28_;
#line 176 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp30_ = gee_list_get ((GeeList*) _tmp29_, 0);
#line 176 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp31_ = (GXmlDomElement*) _tmp30_;
#line 176 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp32_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp31_);
#line 176 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp33_ = _tmp32_;
#line 176 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp34_ = _tmp33_;
#line 176 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp35_ = gee_list_get ((GeeList*) _tmp34_, 1);
#line 176 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp36_ = (GXmlDomElement*) _tmp35_;
#line 176 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp36_ != NULL, "doc.document_element.children[0].children[1] != null");
#line 176 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp36_);
#line 176 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp34_);
#line 176 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp31_);
#line 176 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp29_);
#line 176 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp26_);
#line 177 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp37_ = doc;
#line 177 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp38_ = gxml_dom_document_get_document_element (_tmp37_);
#line 177 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp39_ = _tmp38_;
#line 177 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp40_ = _tmp39_;
#line 177 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp41_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp40_);
#line 177 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp42_ = _tmp41_;
#line 177 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp43_ = _tmp42_;
#line 177 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp44_ = gee_list_get ((GeeList*) _tmp43_, 0);
#line 177 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp45_ = (GXmlDomElement*) _tmp44_;
#line 177 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp46_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp45_);
#line 177 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp47_ = _tmp46_;
#line 177 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp48_ = _tmp47_;
#line 177 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp49_ = gee_list_get ((GeeList*) _tmp48_, 1);
#line 177 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp50_ = (GXmlDomElement*) _tmp49_;
#line 177 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp48_);
#line 177 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp45_);
#line 177 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp43_);
#line 177 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp40_);
#line 177 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		e = _tmp50_;
#line 178 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp51_ = e;
#line 178 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp52_ = gxml_dom_node_get_owner_document ((GXmlDomNode*) _tmp51_);
#line 178 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp53_ = _tmp52_;
#line 178 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp54_ = doc;
#line 178 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp53_ == G_TYPE_CHECK_INSTANCE_CAST (_tmp54_, GXML_TYPE_DOM_DOCUMENT, GXmlDomDocument), "e.owner_document == (DomDocument) doc");
#line 179 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp55_ = e;
#line 179 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp56_ = gxml_dom_node_get_parent_node ((GXmlDomNode*) _tmp55_);
#line 179 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp57_ = _tmp56_;
#line 179 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp58_ = _tmp57_;
#line 179 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp58_ != NULL, "e.parent_node != null");
#line 179 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp58_);
#line 180 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp59_ = e;
#line 180 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp60_ = gxml_dom_node_get_parent_node ((GXmlDomNode*) _tmp59_);
#line 180 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp61_ = _tmp60_;
#line 180 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp62_ = _tmp61_;
#line 180 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp63_ = gxml_dom_node_get_node_name (_tmp62_);
#line 180 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp64_ = _tmp63_;
#line 180 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp65_ = _tmp64_;
#line 180 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp65_, "body") == 0, "e.parent_node.node_name == \"body\"");
#line 180 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp65_);
#line 180 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp62_);
#line 181 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp66_ = e;
#line 181 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp67_ = gxml_dom_node_get_parent_element ((GXmlDomNode*) _tmp66_);
#line 181 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp68_ = _tmp67_;
#line 181 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp69_ = _tmp68_;
#line 181 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp69_ != NULL, "e.parent_element != null");
#line 181 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp69_);
#line 182 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp70_ = e;
#line 182 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp71_ = gxml_dom_node_get_parent_element ((GXmlDomNode*) _tmp70_);
#line 182 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp72_ = _tmp71_;
#line 182 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp73_ = _tmp72_;
#line 182 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp74_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp73_);
#line 182 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp75_ = _tmp74_;
#line 182 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp76_ = _tmp75_;
#line 182 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp76_, "body") == 0, "e.parent_element.node_name == \"body\"");
#line 182 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp76_);
#line 182 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp73_);
#line 183 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp77_ = e;
#line 183 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp78_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp77_);
#line 183 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp79_ = _tmp78_;
#line 183 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp80_ = _tmp79_;
#line 183 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp80_ != NULL, "e.child_nodes != null");
#line 183 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp80_);
#line 184 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp81_ = e;
#line 184 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp82_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp81_);
#line 184 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp83_ = _tmp82_;
#line 184 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp84_ = _tmp83_;
#line 184 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp85_ = gee_collection_get_size ((GeeCollection*) _tmp84_);
#line 184 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp86_ = _tmp85_;
#line 184 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp86_ == 1, "e.child_nodes.size == 1");
#line 184 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp84_);
#line 185 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp87_ = e;
#line 185 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp88_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp87_);
#line 185 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp89_ = _tmp88_;
#line 185 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp90_ = _tmp89_;
#line 185 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp91_ = gxml_dom_node_list_item (_tmp90_, 0);
#line 185 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp92_ = _tmp91_;
#line 185 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp92_ != NULL, "e.child_nodes.item (0) != null");
#line 185 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp92_);
#line 185 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp90_);
#line 186 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp93_ = e;
#line 186 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp94_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp93_);
#line 186 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp95_ = _tmp94_;
#line 186 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp96_ = _tmp95_;
#line 186 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp97_ = gxml_dom_node_list_item (_tmp96_, 0);
#line 186 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp98_ = _tmp97_;
#line 186 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp98_, GXML_TYPE_DOM_TEXT), "e.child_nodes.item (0) is DomText");
#line 186 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp98_);
#line 186 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp96_);
#line 187 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp99_ = e;
#line 187 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp100_ = gxml_dom_node_get_previous_sibling ((GXmlDomNode*) _tmp99_);
#line 187 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp101_ = _tmp100_;
#line 187 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp102_ = _tmp101_;
#line 187 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp102_ != NULL, "e.previous_sibling != null");
#line 187 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp102_);
#line 188 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp103_ = e;
#line 188 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp104_ = gxml_dom_node_get_previous_sibling ((GXmlDomNode*) _tmp103_);
#line 188 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp105_ = _tmp104_;
#line 188 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp106_ = _tmp105_;
#line 188 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp106_, GXML_TYPE_DOM_TEXT), "e.previous_sibling is DomText");
#line 188 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp106_);
#line 189 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp107_ = e;
#line 189 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp108_ = gxml_dom_node_get_next_sibling ((GXmlDomNode*) _tmp107_);
#line 189 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp109_ = _tmp108_;
#line 189 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp110_ = _tmp109_;
#line 189 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp110_ != NULL, "e.next_sibling != null");
#line 189 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp110_);
#line 190 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp111_ = e;
#line 190 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp112_ = gxml_dom_node_get_next_sibling ((GXmlDomNode*) _tmp111_);
#line 190 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp113_ = _tmp112_;
#line 190 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp114_ = _tmp113_;
#line 190 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp114_, GXML_TYPE_DOM_TEXT), "e.next_sibling is DomText");
#line 190 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp114_);
#line 191 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp115_ = e;
#line 191 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp116_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp115_);
#line 191 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp117_ = _tmp116_;
#line 191 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp118_ = _tmp117_;
#line 191 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp119_ = gxml_dom_node_list_item (_tmp118_, 0);
#line 191 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp120_ = _tmp119_;
#line 191 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp118_);
#line 191 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		t = _tmp120_;
#line 192 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp121_ = t;
#line 192 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp122_ = gxml_dom_node_get_previous_sibling (_tmp121_);
#line 192 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp123_ = _tmp122_;
#line 192 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp124_ = _tmp123_;
#line 192 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp124_ == NULL, "t.previous_sibling == null");
#line 192 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp124_);
#line 193 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp125_ = t;
#line 193 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp126_ = gxml_dom_node_get_next_sibling (_tmp125_);
#line 193 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp127_ = _tmp126_;
#line 193 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp128_ = _tmp127_;
#line 193 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp128_ == NULL, "t.next_sibling == null");
#line 193 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp128_);
#line 194 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp129_ = t;
#line 194 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp130_ = gxml_dom_node_get_text_content (_tmp129_);
#line 194 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp131_ = _tmp130_;
#line 194 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp132_ = _tmp131_;
#line 194 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp132_ != NULL, "t.text_content != null");
#line 194 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp132_);
#line 195 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp133_ = t;
#line 195 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp134_ = gxml_dom_node_get_text_content (_tmp133_);
#line 195 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp135_ = _tmp134_;
#line 195 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp136_ = _tmp135_;
#line 195 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp136_, "p01 p id") == 0, "t.text_content == \"p01 p id\"");
#line 195 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp136_);
#line 196 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp137_ = e;
#line 196 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp138_ = gxml_dom_node_get_parent_node ((GXmlDomNode*) _tmp137_);
#line 196 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp139_ = _tmp138_;
#line 196 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp140_ = _tmp139_;
#line 196 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp141_ = gxml_dom_node_get_text_content (_tmp140_);
#line 196 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp142_ = _tmp141_;
#line 196 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp143_ = _tmp142_;
#line 196 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp143_, "\n\n\n\n\n") == 0, "e.parent_node.text_content == \"\\n\\n\\n\\n\\n\"");
#line 196 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp143_);
#line 196 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp140_);
#line 197 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp144_ = e;
#line 197 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp145_ = gxml_dom_node_get_parent_node ((GXmlDomNode*) _tmp144_);
#line 197 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp146_ = _tmp145_;
#line 197 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp147_ = _tmp146_;
#line 197 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp148_ = gxml_dom_node_has_child_nodes (_tmp147_);
#line 197 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp148_, "e.parent_node.has_child_nodes ()");
#line 197 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp147_);
#line 198 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp149_ = e;
#line 198 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp150_ = gxml_dom_node_get_parent_node ((GXmlDomNode*) _tmp149_);
#line 198 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp151_ = _tmp150_;
#line 198 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp152_ = _tmp151_;
#line 198 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		gxml_dom_node_normalize (_tmp152_);
#line 198 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp152_);
#line 199 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp153_ = e;
#line 199 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp154_ = gxml_dom_node_get_parent_node ((GXmlDomNode*) _tmp153_);
#line 199 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp155_ = _tmp154_;
#line 199 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp156_ = _tmp155_;
#line 199 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp157_ = gxml_dom_node_get_text_content (_tmp156_);
#line 199 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp158_ = _tmp157_;
#line 199 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp159_ = _tmp158_;
#line 199 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp159_ == NULL, "e.parent_node.text_content == null");
#line 199 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp159_);
#line 199 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp156_);
#line 200 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp161_ = e;
#line 200 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp162_ = gxml_dom_node_clone_node ((GXmlDomNode*) _tmp161_, FALSE, &_inner_error_);
#line 200 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp160_ = _tmp162_;
#line 200 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 200 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 200 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e);
#line 200 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 200 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 2602 "DomGDocumentTest.c"
			goto __catch183_g_error;
		}
#line 200 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp163_ = _tmp160_;
#line 200 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp160_ = NULL;
#line 200 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp164_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp163_, GXML_TYPE_DOM_ELEMENT) ? ((GXmlDomElement*) _tmp163_) : NULL;
#line 200 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp164_ == NULL) {
#line 200 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp163_);
#line 2615 "DomGDocumentTest.c"
		}
#line 200 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		cn = _tmp164_;
#line 201 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp165_ = cn;
#line 201 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp166_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp165_);
#line 201 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp167_ = _tmp166_;
#line 201 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp168_ = _tmp167_;
#line 201 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp168_, "p") == 0, "cn.node_name == \"p\"");
#line 201 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp168_);
#line 202 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp169_ = cn;
#line 202 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp170_ = gxml_dom_element_get_attribute (_tmp169_, "id");
#line 202 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp171_ = _tmp170_;
#line 202 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp171_, "p01") == 0, "cn.get_attribute (\"id\") == \"p01\"");
#line 202 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp171_);
#line 203 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp172_ = cn;
#line 203 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp173_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp172_);
#line 203 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp174_ = _tmp173_;
#line 203 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp175_ = _tmp174_;
#line 203 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp175_ != NULL, "cn.child_nodes != null");
#line 203 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp175_);
#line 204 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp176_ = cn;
#line 204 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp177_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp176_);
#line 204 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp178_ = _tmp177_;
#line 204 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp179_ = _tmp178_;
#line 204 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp180_ = gee_collection_get_size ((GeeCollection*) _tmp179_);
#line 204 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp181_ = _tmp180_;
#line 204 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp181_ == 0, "cn.child_nodes.size == 0");
#line 204 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp179_);
#line 205 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp183_ = e;
#line 205 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp184_ = gxml_dom_node_clone_node ((GXmlDomNode*) _tmp183_, TRUE, &_inner_error_);
#line 205 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp182_ = _tmp184_;
#line 205 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 205 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn);
#line 205 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp160_);
#line 205 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 205 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e);
#line 205 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 205 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 2689 "DomGDocumentTest.c"
			goto __catch183_g_error;
		}
#line 205 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp185_ = _tmp182_;
#line 205 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp182_ = NULL;
#line 205 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp186_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp185_, GXML_TYPE_DOM_ELEMENT) ? ((GXmlDomElement*) _tmp185_) : NULL;
#line 205 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp186_ == NULL) {
#line 205 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp185_);
#line 2702 "DomGDocumentTest.c"
		}
#line 205 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		cn2 = _tmp186_;
#line 206 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp187_ = cn2;
#line 206 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp188_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp187_);
#line 206 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp189_ = _tmp188_;
#line 206 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp190_ = _tmp189_;
#line 206 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp190_, "p") == 0, "cn2.node_name == \"p\"");
#line 206 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp190_);
#line 207 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp191_ = cn2;
#line 207 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp192_ = gxml_dom_element_get_attribute (_tmp191_, "id");
#line 207 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp193_ = _tmp192_;
#line 207 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp193_, "p01") == 0, "cn2.get_attribute (\"id\") == \"p01\"");
#line 207 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp193_);
#line 208 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp194_ = cn2;
#line 208 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp195_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp194_);
#line 208 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp196_ = _tmp195_;
#line 208 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp197_ = _tmp196_;
#line 208 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp197_ != NULL, "cn2.child_nodes != null");
#line 208 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp197_);
#line 209 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp198_ = cn2;
#line 209 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp199_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp198_);
#line 209 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp200_ = _tmp199_;
#line 209 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp201_ = _tmp200_;
#line 209 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp202_ = gee_collection_get_size ((GeeCollection*) _tmp201_);
#line 209 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp203_ = _tmp202_;
#line 209 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp203_ == 1, "cn2.child_nodes.size == 1");
#line 209 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp201_);
#line 210 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp204_ = cn2;
#line 210 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp205_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp204_);
#line 210 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp206_ = _tmp205_;
#line 210 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp207_ = _tmp206_;
#line 210 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp208_ = gee_list_get ((GeeList*) _tmp207_, 0);
#line 210 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp209_ = (GXmlDomNode*) _tmp208_;
#line 210 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp209_, GXML_TYPE_DOM_TEXT), "cn2.child_nodes[0] is DomText");
#line 210 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp209_);
#line 210 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp207_);
#line 211 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp210_ = cn2;
#line 211 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp211_ = gxml_dom_node_get_text_content ((GXmlDomNode*) _tmp210_);
#line 211 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp212_ = _tmp211_;
#line 211 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp213_ = _tmp212_;
#line 211 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp213_, "p01 p id") == 0, "cn2.text_content == \"p01 p id\"");
#line 211 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp213_);
#line 212 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp214_ = e;
#line 212 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp215_ = cn;
#line 212 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp216_ = gxml_dom_node_is_equal_node ((GXmlDomNode*) _tmp214_, (GXmlDomNode*) _tmp215_);
#line 212 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (!_tmp216_, "!e.is_equal_node (cn)");
#line 213 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp217_ = e;
#line 213 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp218_ = cn2;
#line 213 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp219_ = gxml_dom_node_is_equal_node ((GXmlDomNode*) _tmp217_, (GXmlDomNode*) _tmp218_);
#line 213 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp219_, "e.is_equal_node (cn2)");
#line 214 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp220_ = doc;
#line 214 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp221_ = gxml_dom_document_get_document_element (_tmp220_);
#line 214 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp222_ = _tmp221_;
#line 214 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp223_ = _tmp222_;
#line 214 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp224_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp223_);
#line 214 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp225_ = _tmp224_;
#line 214 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp226_ = _tmp225_;
#line 214 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp227_ = gee_list_get ((GeeList*) _tmp226_, 0);
#line 214 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp228_ = (GXmlDomElement*) _tmp227_;
#line 214 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp229_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp228_);
#line 214 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp230_ = _tmp229_;
#line 214 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp231_ = _tmp230_;
#line 214 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp232_ = gee_list_get ((GeeList*) _tmp231_, 0);
#line 214 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp233_ = (GXmlDomElement*) _tmp232_;
#line 214 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp231_);
#line 214 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp228_);
#line 214 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp226_);
#line 214 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp223_);
#line 214 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		e0 = _tmp233_;
#line 215 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp234_ = e0;
#line 215 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp235_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp234_);
#line 215 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp236_ = _tmp235_;
#line 215 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp237_ = _tmp236_;
#line 215 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp237_, "p") == 0, "e0.node_name == \"p\"");
#line 215 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp237_);
#line 216 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp238_ = doc;
#line 216 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp239_ = gxml_dom_document_get_document_element (_tmp238_);
#line 216 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp240_ = _tmp239_;
#line 216 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp241_ = _tmp240_;
#line 216 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp242_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp241_);
#line 216 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp243_ = _tmp242_;
#line 216 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp244_ = _tmp243_;
#line 216 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp245_ = gee_list_get ((GeeList*) _tmp244_, 0);
#line 216 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp246_ = (GXmlDomElement*) _tmp245_;
#line 216 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp247_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp246_);
#line 216 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp248_ = _tmp247_;
#line 216 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp249_ = _tmp248_;
#line 216 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp250_ = e0;
#line 216 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp251_ = gee_list_index_of ((GeeList*) _tmp249_, (GXmlDomNode*) _tmp250_);
#line 216 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp251_ == 0, "doc.document_element.children[0].child_nodes.index_of (e0) == 0");
#line 216 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp249_);
#line 216 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp246_);
#line 216 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp244_);
#line 216 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp241_);
#line 217 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp252_ = doc;
#line 217 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp253_ = gxml_dom_document_get_document_element (_tmp252_);
#line 217 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp254_ = _tmp253_;
#line 217 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp255_ = _tmp254_;
#line 217 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp256_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp255_);
#line 217 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp257_ = _tmp256_;
#line 217 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp258_ = _tmp257_;
#line 217 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp259_ = gee_list_get ((GeeList*) _tmp258_, 0);
#line 217 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp260_ = (GXmlDomElement*) _tmp259_;
#line 217 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp261_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp260_);
#line 217 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp262_ = _tmp261_;
#line 217 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp263_ = _tmp262_;
#line 217 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp264_ = gee_list_get ((GeeList*) _tmp263_, 1);
#line 217 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp265_ = (GXmlDomElement*) _tmp264_;
#line 217 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp263_);
#line 217 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp260_);
#line 217 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp258_);
#line 217 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp255_);
#line 217 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		e1 = _tmp265_;
#line 218 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp266_ = e1;
#line 218 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp267_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp266_);
#line 218 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp268_ = _tmp267_;
#line 218 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp269_ = _tmp268_;
#line 218 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp269_, "p") == 0, "e1.node_name == \"p\"");
#line 218 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp269_);
#line 219 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp270_ = doc;
#line 219 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp271_ = gxml_dom_document_get_document_element (_tmp270_);
#line 219 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp272_ = _tmp271_;
#line 219 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp273_ = _tmp272_;
#line 219 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp274_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp273_);
#line 219 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp275_ = _tmp274_;
#line 219 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp276_ = _tmp275_;
#line 219 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp277_ = gee_list_get ((GeeList*) _tmp276_, 0);
#line 219 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp278_ = (GXmlDomElement*) _tmp277_;
#line 219 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp279_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp278_);
#line 219 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp280_ = _tmp279_;
#line 219 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp281_ = _tmp280_;
#line 219 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp282_ = e1;
#line 219 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp283_ = gee_list_index_of ((GeeList*) _tmp281_, (GXmlDomNode*) _tmp282_);
#line 219 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp283_ == 1, "doc.document_element.children[0].child_nodes.index_of (e1) == 1");
#line 219 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp281_);
#line 219 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp278_);
#line 219 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp276_);
#line 219 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp273_);
#line 220 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp284_ = doc;
#line 220 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp285_ = gxml_dom_document_get_document_element (_tmp284_);
#line 220 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp286_ = _tmp285_;
#line 220 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp287_ = _tmp286_;
#line 220 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp288_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp287_);
#line 220 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp289_ = _tmp288_;
#line 220 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp290_ = _tmp289_;
#line 220 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp291_ = gee_list_get ((GeeList*) _tmp290_, 0);
#line 220 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp292_ = (GXmlDomElement*) _tmp291_;
#line 220 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp293_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp292_);
#line 220 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp294_ = _tmp293_;
#line 220 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp295_ = _tmp294_;
#line 220 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp296_ = gee_list_get ((GeeList*) _tmp295_, 2);
#line 220 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp297_ = (GXmlDomElement*) _tmp296_;
#line 220 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp295_);
#line 220 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp292_);
#line 220 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp290_);
#line 220 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp287_);
#line 220 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		e2 = _tmp297_;
#line 221 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp298_ = e2;
#line 221 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp299_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp298_);
#line 221 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp300_ = _tmp299_;
#line 221 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp301_ = _tmp300_;
#line 221 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp301_, "p") == 0, "e2.node_name == \"p\"");
#line 221 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp301_);
#line 222 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp302_ = doc;
#line 222 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp303_ = gxml_dom_document_get_document_element (_tmp302_);
#line 222 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp304_ = _tmp303_;
#line 222 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp305_ = _tmp304_;
#line 222 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp306_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp305_);
#line 222 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp307_ = _tmp306_;
#line 222 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp308_ = _tmp307_;
#line 222 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp309_ = gee_list_get ((GeeList*) _tmp308_, 0);
#line 222 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp310_ = (GXmlDomElement*) _tmp309_;
#line 222 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp311_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp310_);
#line 222 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp312_ = _tmp311_;
#line 222 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp313_ = _tmp312_;
#line 222 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp314_ = e2;
#line 222 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp315_ = gee_list_index_of ((GeeList*) _tmp313_, (GXmlDomNode*) _tmp314_);
#line 222 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp315_ == 2, "doc.document_element.children[0].child_nodes.index_of (e2) == 2");
#line 222 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp313_);
#line 222 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp310_);
#line 222 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp308_);
#line 222 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp305_);
#line 223 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp316_ = e;
#line 223 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp317_ = gxml_dom_node_get_parent_node ((GXmlDomNode*) _tmp316_);
#line 223 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp318_ = _tmp317_;
#line 223 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp319_ = _tmp318_;
#line 223 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp320_ = e0;
#line 223 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp321_ = gxml_dom_node_contains (_tmp319_, (GXmlDomNode*) _tmp320_);
#line 223 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp321_, "e.parent_node.contains (e0)");
#line 223 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp319_);
#line 224 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp322_ = e;
#line 224 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp323_ = gxml_dom_node_get_parent_node ((GXmlDomNode*) _tmp322_);
#line 224 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp324_ = _tmp323_;
#line 224 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp325_ = _tmp324_;
#line 224 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp326_ = e1;
#line 224 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp327_ = gxml_dom_node_contains (_tmp325_, (GXmlDomNode*) _tmp326_);
#line 224 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp327_, "e.parent_node.contains (e1)");
#line 224 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp325_);
#line 225 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp328_ = e;
#line 225 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp329_ = gxml_dom_node_get_parent_node ((GXmlDomNode*) _tmp328_);
#line 225 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp330_ = _tmp329_;
#line 225 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp331_ = _tmp330_;
#line 225 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp332_ = e2;
#line 225 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp333_ = gxml_dom_node_contains (_tmp331_, (GXmlDomNode*) _tmp332_);
#line 225 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp333_, "e.parent_node.contains (e2)");
#line 225 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp331_);
#line 226 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp335_ = e1;
#line 226 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp336_ = gxml_dom_node_get_parent_node ((GXmlDomNode*) _tmp335_);
#line 226 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp337_ = _tmp336_;
#line 226 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp338_ = _tmp337_;
#line 226 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp339_ = e0;
#line 226 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp340_ = gxml_dom_node_contains (_tmp338_, (GXmlDomNode*) _tmp339_);
#line 226 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp341_ = _tmp340_;
#line 226 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp338_);
#line 226 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp341_) {
#line 3132 "DomGDocumentTest.c"
			GXmlDomElement* _tmp342_ = NULL;
			GXmlDomNode* _tmp343_ = NULL;
			GXmlDomNode* _tmp344_ = NULL;
			GXmlDomNode* _tmp345_ = NULL;
			GXmlDomElement* _tmp346_ = NULL;
			gboolean _tmp347_ = FALSE;
#line 226 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_tmp342_ = e0;
#line 226 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_tmp343_ = gxml_dom_node_get_parent_node ((GXmlDomNode*) _tmp342_);
#line 226 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_tmp344_ = _tmp343_;
#line 226 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_tmp345_ = _tmp344_;
#line 226 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_tmp346_ = e1;
#line 226 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_tmp347_ = gxml_dom_node_contains (_tmp345_, (GXmlDomNode*) _tmp346_);
#line 226 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_tmp334_ = _tmp347_;
#line 226 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp345_);
#line 3155 "DomGDocumentTest.c"
		} else {
#line 226 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_tmp334_ = FALSE;
#line 3159 "DomGDocumentTest.c"
		}
#line 226 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp334_, "e1.parent_node.contains (e0) && e0.parent_node.contains (e1)");
#line 227 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp348_ = e1;
#line 227 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp349_ = e0;
#line 227 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp350_ = gxml_dom_node_compare_document_position ((GXmlDomNode*) _tmp348_, (GXmlDomNode*) _tmp349_);
#line 227 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp350_ == GXML_DOM_NODE_DOCUMENT_POSITION_PRECEDING, "e1.compare_document_position (e0) == DomNode.DocumentPosition.PRECEDING");
#line 228 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp352_ = e1;
#line 228 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp353_ = gxml_dom_node_get_parent_node ((GXmlDomNode*) _tmp352_);
#line 228 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp354_ = _tmp353_;
#line 228 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp355_ = _tmp354_;
#line 228 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp356_ = e2;
#line 228 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp357_ = gxml_dom_node_contains (_tmp355_, (GXmlDomNode*) _tmp356_);
#line 228 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp358_ = _tmp357_;
#line 228 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp355_);
#line 228 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp358_) {
#line 3189 "DomGDocumentTest.c"
			GXmlDomElement* _tmp359_ = NULL;
			GXmlDomNode* _tmp360_ = NULL;
			GXmlDomNode* _tmp361_ = NULL;
			GXmlDomNode* _tmp362_ = NULL;
			GXmlDomElement* _tmp363_ = NULL;
			gboolean _tmp364_ = FALSE;
#line 228 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_tmp359_ = e2;
#line 228 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_tmp360_ = gxml_dom_node_get_parent_node ((GXmlDomNode*) _tmp359_);
#line 228 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_tmp361_ = _tmp360_;
#line 228 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_tmp362_ = _tmp361_;
#line 228 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_tmp363_ = e1;
#line 228 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_tmp364_ = gxml_dom_node_contains (_tmp362_, (GXmlDomNode*) _tmp363_);
#line 228 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_tmp351_ = _tmp364_;
#line 228 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp362_);
#line 3212 "DomGDocumentTest.c"
		} else {
#line 228 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_tmp351_ = FALSE;
#line 3216 "DomGDocumentTest.c"
		}
#line 228 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp351_, "e1.parent_node.contains (e2) && e2.parent_node.contains (e1)");
#line 229 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp365_ = e1;
#line 229 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp366_ = e2;
#line 229 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp367_ = gxml_dom_node_compare_document_position ((GXmlDomNode*) _tmp365_, (GXmlDomNode*) _tmp366_);
#line 229 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp367_ == GXML_DOM_NODE_DOCUMENT_POSITION_FOLLOWING, "e1.compare_document_position (e2) == DomNode.DocumentPosition.FOLLOWING");
#line 230 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp368_ = e1;
#line 230 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp369_ = gxml_dom_node_get_parent_node ((GXmlDomNode*) _tmp368_);
#line 230 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp370_ = _tmp369_;
#line 230 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp371_ = _tmp370_;
#line 230 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp372_ = e1;
#line 230 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp373_ = gxml_dom_node_contains (_tmp371_, (GXmlDomNode*) _tmp372_);
#line 230 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp373_, "e1.parent_node.contains (e1)");
#line 230 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp371_);
#line 231 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp374_ = e1;
#line 231 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp375_ = e1;
#line 231 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp376_ = gxml_dom_node_get_parent_node ((GXmlDomNode*) _tmp375_);
#line 231 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp377_ = _tmp376_;
#line 231 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp378_ = _tmp377_;
#line 231 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp379_ = gxml_dom_node_compare_document_position ((GXmlDomNode*) _tmp374_, _tmp378_);
#line 231 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp379_ == (GXML_DOM_NODE_DOCUMENT_POSITION_CONTAINS & GXML_DOM_NODE_DOCUMENT_POSITION_FOLLOWING), "e1.compare_document_position (e1.parent_node) == (DomNode.DocumentPosition.CONTAINS & DomNode.DocumentPosition.FOLLOWING)");
#line 231 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp378_);
#line 232 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp380_ = e1;
#line 232 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp381_ = e1;
#line 232 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp382_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp381_);
#line 232 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp383_ = _tmp382_;
#line 232 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp384_ = _tmp383_;
#line 232 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp385_ = gee_list_get ((GeeList*) _tmp384_, 0);
#line 232 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp386_ = (GXmlDomNode*) _tmp385_;
#line 232 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp387_ = gxml_dom_node_contains ((GXmlDomNode*) _tmp380_, _tmp386_);
#line 232 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp387_, "e1.contains (e1.child_nodes[0])");
#line 232 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp386_);
#line 232 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp384_);
#line 233 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp388_ = e1;
#line 233 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp389_ = e1;
#line 233 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp390_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp389_);
#line 233 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp391_ = _tmp390_;
#line 233 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp392_ = _tmp391_;
#line 233 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp393_ = gee_list_get ((GeeList*) _tmp392_, 0);
#line 233 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp394_ = (GXmlDomNode*) _tmp393_;
#line 233 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp395_ = gxml_dom_node_compare_document_position ((GXmlDomNode*) _tmp388_, _tmp394_);
#line 233 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp395_ == (GXML_DOM_NODE_DOCUMENT_POSITION_CONTAINED_BY & GXML_DOM_NODE_DOCUMENT_POSITION_PRECEDING), "e1.compare_document_position (e1.child_nodes[0]) == (DomNode.DocumentPosition.CONTAINED_BY & DomNode.DocumentPosition.PRECEDING)");
#line 233 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp394_);
#line 233 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp392_);
#line 234 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp396_ = doc;
#line 234 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp397_ = gxml_dom_document_get_document_element (_tmp396_);
#line 234 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp398_ = _tmp397_;
#line 234 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp399_ = _tmp398_;
#line 234 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp400_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp399_);
#line 234 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp401_ = _tmp400_;
#line 234 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp402_ = _tmp401_;
#line 234 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp403_ = gee_list_get ((GeeList*) _tmp402_, 0);
#line 234 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp404_ = (GXmlDomElement*) _tmp403_;
#line 234 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp402_);
#line 234 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp399_);
#line 234 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		b = _tmp404_;
#line 235 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp405_ = b;
#line 235 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp406_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp405_);
#line 235 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp407_ = _tmp406_;
#line 235 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp408_ = _tmp407_;
#line 235 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp408_, "body") == 0, "b.node_name == \"body\"");
#line 235 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp408_);
#line 236 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp409_ = doc;
#line 236 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp410_ = gxml_dom_document_create_element_ns (_tmp409_, "http://git.gnome.org/browse/gxml", "gxml:MyNode", &_inner_error_);
#line 236 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		ng = _tmp410_;
#line 236 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 236 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (b);
#line 236 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e2);
#line 236 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e1);
#line 236 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e0);
#line 236 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn2);
#line 236 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp182_);
#line 236 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn);
#line 236 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp160_);
#line 236 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 236 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e);
#line 236 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 236 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 3372 "DomGDocumentTest.c"
			goto __catch183_g_error;
		}
#line 237 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp411_ = b;
#line 237 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp412_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp411_);
#line 237 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp413_ = _tmp412_;
#line 237 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp414_ = _tmp413_;
#line 237 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp415_ = ng;
#line 237 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		gee_collection_add ((GeeCollection*) _tmp414_, (GXmlDomNode*) _tmp415_);
#line 237 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp414_);
#line 238 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp416_ = ng;
#line 238 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp417_ = gxml_dom_node_lookup_prefix ((GXmlDomNode*) _tmp416_, "http://git.gnome.org/browse/gxml");
#line 238 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp418_ = _tmp417_;
#line 238 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp418_, "gxml") == 0, "ng.lookup_prefix (\"http://git.gnome.org/browse/gxml\") == \"gxml\"");
#line 238 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp418_);
#line 239 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp419_ = ng;
#line 239 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp420_ = gxml_dom_node_lookup_namespace_uri ((GXmlDomNode*) _tmp419_, "gxml");
#line 239 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp421_ = _tmp420_;
#line 239 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp421_, "http://git.gnome.org/browse/gxml") == 0, "ng.lookup_namespace_uri (\"gxml\") == \"http://git.gnome.org/browse/gxml\"");
#line 239 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp421_);
#line 240 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp422_ = ng;
#line 240 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp423_ = gxml_dom_node_is_default_namespace ((GXmlDomNode*) _tmp422_, "gxml:http://git.gnome.org/browse/gxml");
#line 240 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (!_tmp423_, "!ng.is_default_namespace (\"gxml:http://git.gnome.org/browse/gxml\")");
#line 241 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp424_ = doc;
#line 241 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp425_ = gxml_dom_document_create_element_ns (_tmp424_, "http://live.gnome.org/GXml", "OtherNode", &_inner_error_);
#line 241 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		ng2 = _tmp425_;
#line 241 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 241 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng);
#line 241 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (b);
#line 241 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e2);
#line 241 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e1);
#line 241 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e0);
#line 241 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn2);
#line 241 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp182_);
#line 241 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn);
#line 241 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp160_);
#line 241 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 241 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e);
#line 241 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 241 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 3449 "DomGDocumentTest.c"
			goto __catch183_g_error;
		}
#line 242 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp426_ = b;
#line 242 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp427_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp426_);
#line 242 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp428_ = _tmp427_;
#line 242 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp429_ = _tmp428_;
#line 242 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp430_ = ng2;
#line 242 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		gee_collection_add ((GeeCollection*) _tmp429_, (GXmlDomNode*) _tmp430_);
#line 242 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp429_);
#line 243 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp431_ = ng2;
#line 243 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp432_ = gxml_dom_node_lookup_prefix ((GXmlDomNode*) _tmp431_, "http://live.gnome.org/GXml");
#line 243 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp433_ = _tmp432_;
#line 243 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp433_ == NULL, "ng2.lookup_prefix (\"http://live.gnome.org/GXml\") == null");
#line 243 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp433_);
#line 244 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp434_ = ng2;
#line 244 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp435_ = gxml_dom_node_lookup_namespace_uri ((GXmlDomNode*) _tmp434_, NULL);
#line 244 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp436_ = _tmp435_;
#line 244 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp436_, "http://live.gnome.org/GXml") == 0, "ng2.lookup_namespace_uri (null) == \"http://live.gnome.org/GXml\"");
#line 244 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp436_);
#line 245 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp437_ = ng2;
#line 245 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp438_ = gxml_dom_node_is_default_namespace ((GXmlDomNode*) _tmp437_, "http://live.gnome.org/GXml");
#line 245 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp438_, "ng2.is_default_namespace (\"http://live.gnome.org/GXml\")");
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp440_ = doc;
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp441_ = gxml_dom_document_create_element (_tmp440_, "p", &_inner_error_);
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp439_ = _tmp441_;
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng2);
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng);
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (b);
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e2);
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e1);
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e0);
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn2);
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp182_);
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn);
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp160_);
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e);
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 3528 "DomGDocumentTest.c"
			goto __catch183_g_error;
		}
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp442_ = _tmp439_;
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp439_ = NULL;
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp443_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp442_, GXML_TYPE_DOM_ELEMENT) ? ((GXmlDomElement*) _tmp442_) : NULL;
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp443_ == NULL) {
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp442_);
#line 3541 "DomGDocumentTest.c"
		}
#line 246 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		pn = _tmp443_;
#line 247 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp444_ = pn;
#line 247 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		gxml_dom_element_set_attribute (_tmp444_, "id", "insertedp01", &_inner_error_);
#line 247 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 247 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn);
#line 247 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp439_);
#line 247 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng2);
#line 247 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng);
#line 247 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (b);
#line 247 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e2);
#line 247 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e1);
#line 247 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e0);
#line 247 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn2);
#line 247 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp182_);
#line 247 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn);
#line 247 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp160_);
#line 247 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 247 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e);
#line 247 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 247 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 3583 "DomGDocumentTest.c"
			goto __catch183_g_error;
		}
#line 248 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp445_ = doc;
#line 248 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp446_ = gxml_dom_document_get_document_element (_tmp445_);
#line 248 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp447_ = _tmp446_;
#line 248 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp448_ = _tmp447_;
#line 248 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp449_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp448_);
#line 248 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp450_ = _tmp449_;
#line 248 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp451_ = _tmp450_;
#line 248 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp452_ = gee_list_get ((GeeList*) _tmp451_, 0);
#line 248 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp453_ = (GXmlDomElement*) _tmp452_;
#line 248 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp451_);
#line 248 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp448_);
#line 248 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		p = _tmp453_;
#line 249 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp454_ = p;
#line 249 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp455_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp454_);
#line 249 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp456_ = _tmp455_;
#line 249 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp457_ = _tmp456_;
#line 249 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp457_, "body") == 0, "p.node_name == \"body\"");
#line 249 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp457_);
#line 250 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp458_ = p;
#line 250 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp459_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp458_);
#line 250 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp460_ = _tmp459_;
#line 250 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp461_ = _tmp460_;
#line 250 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp462_ = gee_list_get ((GeeList*) _tmp461_, 0);
#line 250 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp463_ = (GXmlDomElement*) _tmp462_;
#line 250 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp461_);
#line 250 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		cp0 = _tmp463_;
#line 251 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp464_ = cp0;
#line 251 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp465_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp464_);
#line 251 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp466_ = _tmp465_;
#line 251 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp467_ = _tmp466_;
#line 251 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp467_, "p") == 0, "cp0.node_name == \"p\"");
#line 251 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp467_);
#line 252 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp468_ = cp0;
#line 252 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp469_ = gxml_dom_element_get_attribute (_tmp468_, "class");
#line 252 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp470_ = _tmp469_;
#line 252 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp470_, "black") == 0, "cp0.get_attribute (\"class\") == \"black\"");
#line 252 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp470_);
#line 253 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp471_ = p;
#line 253 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp472_ = cp0;
#line 253 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp473_ = gxml_dom_node_contains ((GXmlDomNode*) _tmp471_, (GXmlDomNode*) _tmp472_);
#line 253 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp473_, "p.contains (cp0)");
#line 254 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp474_ = cp0;
#line 254 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp475_ = gxml_dom_node_get_parent_node ((GXmlDomNode*) _tmp474_);
#line 254 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp476_ = _tmp475_;
#line 254 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp477_ = _tmp476_;
#line 254 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp478_ = cp0;
#line 254 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp479_ = gxml_dom_node_contains (_tmp477_, (GXmlDomNode*) _tmp478_);
#line 254 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp479_, "cp0.parent_node.contains (cp0)");
#line 254 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp477_);
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp480_ = p;
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp481_ = pn;
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp482_ = cp0;
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp483_ = gxml_dom_node_insert_before ((GXmlDomNode*) _tmp480_, (GXmlDomNode*) _tmp481_, (GXmlDomNode*) _tmp482_, &_inner_error_);
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp484_ = _tmp483_;
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp484_);
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cp0);
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (p);
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn);
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp439_);
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng2);
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng);
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (b);
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e2);
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e1);
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e0);
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn2);
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp182_);
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn);
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp160_);
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e);
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 255 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 3734 "DomGDocumentTest.c"
			goto __catch183_g_error;
		}
#line 256 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp485_ = p;
#line 256 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp486_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp485_);
#line 256 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp487_ = _tmp486_;
#line 256 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp488_ = _tmp487_;
#line 256 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp489_ = gee_list_get ((GeeList*) _tmp488_, 0);
#line 256 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp490_ = (GXmlDomElement*) _tmp489_;
#line 256 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp488_);
#line 256 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		ppn = _tmp490_;
#line 257 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp491_ = ppn;
#line 257 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp492_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp491_);
#line 257 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp493_ = _tmp492_;
#line 257 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp494_ = _tmp493_;
#line 257 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp494_, "p") == 0, "ppn.node_name == \"p\"");
#line 257 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp494_);
#line 258 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp495_ = ppn;
#line 258 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp496_ = gxml_dom_element_has_attribute (_tmp495_, "id");
#line 258 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp496_, "ppn.has_attribute (\"id\")");
#line 259 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp497_ = ppn;
#line 259 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp498_ = gxml_dom_element_get_attribute (_tmp497_, "id");
#line 259 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp499_ = _tmp498_;
#line 259 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp499_, "insertedp01") == 0, "ppn.get_attribute (\"id\") == \"insertedp01\"");
#line 259 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp499_);
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp501_ = doc;
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp502_ = gxml_dom_document_create_element (_tmp501_, "p", &_inner_error_);
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp500_ = _tmp502_;
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ppn);
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cp0);
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (p);
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn);
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp439_);
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng2);
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng);
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (b);
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e2);
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e1);
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e0);
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn2);
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp182_);
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn);
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp160_);
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e);
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 3827 "DomGDocumentTest.c"
			goto __catch183_g_error;
		}
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp503_ = _tmp500_;
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp500_ = NULL;
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp504_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp503_, GXML_TYPE_DOM_ELEMENT) ? ((GXmlDomElement*) _tmp503_) : NULL;
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp504_ == NULL) {
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp503_);
#line 3840 "DomGDocumentTest.c"
		}
#line 260 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		pn2 = _tmp504_;
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp505_ = pn2;
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		gxml_dom_element_set_attribute (_tmp505_, "id", "newp", &_inner_error_);
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn2);
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp500_);
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ppn);
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cp0);
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (p);
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn);
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp439_);
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng2);
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng);
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (b);
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e2);
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e1);
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e0);
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn2);
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp182_);
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn);
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp160_);
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e);
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 261 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 3892 "DomGDocumentTest.c"
			goto __catch183_g_error;
		}
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp506_ = p;
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp507_ = pn2;
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp508_ = gxml_dom_node_append_child ((GXmlDomNode*) _tmp506_, (GXmlDomNode*) _tmp507_, &_inner_error_);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp509_ = _tmp508_;
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp509_);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn2);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp500_);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ppn);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cp0);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (p);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp439_);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng2);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (b);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e2);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e1);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e0);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn2);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp182_);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp160_);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 262 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 3949 "DomGDocumentTest.c"
			goto __catch183_g_error;
		}
#line 263 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp510_ = p;
#line 263 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp511_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp510_);
#line 263 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp512_ = _tmp511_;
#line 263 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp513_ = _tmp512_;
#line 263 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp514_ = gxml_dom_html_collection_get_length (_tmp513_);
#line 263 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp515_ = _tmp514_;
#line 263 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp515_ == 8, "p.children.length == 8");
#line 263 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp513_);
#line 264 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp516_ = p;
#line 264 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp517_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp516_);
#line 264 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp518_ = _tmp517_;
#line 264 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp519_ = _tmp518_;
#line 264 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp520_ = gee_list_get ((GeeList*) _tmp519_, 7);
#line 264 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp521_ = (GXmlDomElement*) _tmp520_;
#line 264 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp521_, GXML_TYPE_DOM_ELEMENT), "p.children[7] is DomElement");
#line 264 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp521_);
#line 264 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp519_);
#line 265 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp522_ = p;
#line 265 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp523_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp522_);
#line 265 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp524_ = _tmp523_;
#line 265 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp525_ = _tmp524_;
#line 265 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp526_ = gee_list_get ((GeeList*) _tmp525_, 7);
#line 265 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp527_ = (GXmlDomElement*) _tmp526_;
#line 265 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp528_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp527_);
#line 265 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp529_ = _tmp528_;
#line 265 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp530_ = _tmp529_;
#line 265 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp530_, "p") == 0, "p.children[7].node_name == \"p\"");
#line 265 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp530_);
#line 265 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp527_);
#line 265 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp525_);
#line 266 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp531_ = p;
#line 266 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp532_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp531_);
#line 266 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp533_ = _tmp532_;
#line 266 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp534_ = _tmp533_;
#line 266 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp535_ = gee_list_get ((GeeList*) _tmp534_, 7);
#line 266 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp536_ = (GXmlDomElement*) _tmp535_;
#line 266 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp537_ = gxml_dom_element_get_attribute (_tmp536_, "id");
#line 266 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp538_ = _tmp537_;
#line 266 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp538_, "newp") == 0, "p.children[7].get_attribute (\"id\") == \"newp\"");
#line 266 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp538_);
#line 266 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp536_);
#line 266 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp534_);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp540_ = doc;
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp541_ = gxml_dom_document_create_element (_tmp540_, "p", &_inner_error_);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp539_ = _tmp541_;
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn2);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp500_);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ppn);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cp0);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (p);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp439_);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng2);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (b);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e2);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e1);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e0);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn2);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp182_);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp160_);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 4086 "DomGDocumentTest.c"
			goto __catch183_g_error;
		}
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp542_ = _tmp539_;
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp539_ = NULL;
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp543_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp542_, GXML_TYPE_DOM_ELEMENT) ? ((GXmlDomElement*) _tmp542_) : NULL;
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp543_ == NULL) {
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp542_);
#line 4099 "DomGDocumentTest.c"
		}
#line 267 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		pn3 = _tmp543_;
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp544_ = pn3;
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		gxml_dom_element_set_attribute (_tmp544_, "id", "newp1", &_inner_error_);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn3);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp539_);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn2);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp500_);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ppn);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cp0);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (p);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp439_);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng2);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (b);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e2);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e1);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e0);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn2);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp182_);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp160_);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 268 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 4155 "DomGDocumentTest.c"
			goto __catch183_g_error;
		}
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp545_ = pn3;
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		gxml_dom_element_set_attribute (_tmp545_, "class", "black", &_inner_error_);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn3);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp539_);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn2);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp500_);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ppn);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cp0);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (p);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp439_);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng2);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (b);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e2);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e1);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e0);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn2);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp182_);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp160_);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 269 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 4210 "DomGDocumentTest.c"
			goto __catch183_g_error;
		}
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp546_ = p;
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp547_ = pn3;
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp548_ = pn2;
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp549_ = gxml_dom_node_replace_child ((GXmlDomNode*) _tmp546_, (GXmlDomNode*) _tmp547_, (GXmlDomNode*) _tmp548_, &_inner_error_);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp550_ = _tmp549_;
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp550_);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn3);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp539_);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn2);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp500_);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ppn);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cp0);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (p);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp439_);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng2);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (b);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e2);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e1);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e0);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn2);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp182_);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp160_);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 270 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 4273 "DomGDocumentTest.c"
			goto __catch183_g_error;
		}
#line 271 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp551_ = p;
#line 271 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp552_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp551_);
#line 271 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp553_ = _tmp552_;
#line 271 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp554_ = _tmp553_;
#line 271 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp555_ = gxml_dom_html_collection_get_length (_tmp554_);
#line 271 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp556_ = _tmp555_;
#line 271 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp556_ == 8, "p.children.length == 8");
#line 271 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp554_);
#line 272 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp557_ = p;
#line 272 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp558_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp557_);
#line 272 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp559_ = _tmp558_;
#line 272 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp560_ = _tmp559_;
#line 272 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp561_ = gee_list_get ((GeeList*) _tmp560_, 7);
#line 272 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp562_ = (GXmlDomElement*) _tmp561_;
#line 272 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp562_, GXML_TYPE_DOM_ELEMENT), "p.children[7] is DomElement");
#line 272 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp562_);
#line 272 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp560_);
#line 273 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp563_ = p;
#line 273 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp564_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp563_);
#line 273 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp565_ = _tmp564_;
#line 273 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp566_ = _tmp565_;
#line 273 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp567_ = gee_list_get ((GeeList*) _tmp566_, 7);
#line 273 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp568_ = (GXmlDomElement*) _tmp567_;
#line 273 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp569_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp568_);
#line 273 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp570_ = _tmp569_;
#line 273 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp571_ = _tmp570_;
#line 273 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp571_, "p") == 0, "p.children[7].node_name == \"p\"");
#line 273 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp571_);
#line 273 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp568_);
#line 273 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp566_);
#line 274 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp572_ = p;
#line 274 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp573_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp572_);
#line 274 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp574_ = _tmp573_;
#line 274 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp575_ = _tmp574_;
#line 274 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp576_ = gee_list_get ((GeeList*) _tmp575_, 7);
#line 274 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp577_ = (GXmlDomElement*) _tmp576_;
#line 274 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp578_ = gxml_dom_element_get_attribute (_tmp577_, "id");
#line 274 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp579_ = _tmp578_;
#line 274 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp579_, "newp1") == 0, "p.children[7].get_attribute (\"id\") == \"newp1\"");
#line 274 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp579_);
#line 274 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp577_);
#line 274 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp575_);
#line 275 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp580_ = p;
#line 275 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp581_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp580_);
#line 275 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp582_ = _tmp581_;
#line 275 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp583_ = _tmp582_;
#line 275 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp584_ = gee_list_get ((GeeList*) _tmp583_, 7);
#line 275 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp585_ = (GXmlDomElement*) _tmp584_;
#line 275 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp586_ = gxml_dom_element_get_attribute (_tmp585_, "class");
#line 275 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp587_ = _tmp586_;
#line 275 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp587_, "black") == 0, "p.children[7].get_attribute (\"class\") == \"black\"");
#line 275 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp587_);
#line 275 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp585_);
#line 275 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp583_);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp589_ = doc;
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp590_ = gxml_dom_document_create_element (_tmp589_, "p", &_inner_error_);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp588_ = _tmp590_;
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn3);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp539_);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn2);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp500_);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ppn);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cp0);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (p);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp439_);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng2);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (b);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e2);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e1);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e0);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn2);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp182_);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp160_);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 4438 "DomGDocumentTest.c"
			goto __catch183_g_error;
		}
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp591_ = _tmp588_;
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp588_ = NULL;
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp592_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp591_, GXML_TYPE_DOM_ELEMENT) ? ((GXmlDomElement*) _tmp591_) : NULL;
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp592_ == NULL) {
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp591_);
#line 4451 "DomGDocumentTest.c"
		}
#line 276 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		pn4 = _tmp592_;
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp593_ = pn4;
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		gxml_dom_element_set_attribute (_tmp593_, "id", "newp2", &_inner_error_);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn4);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp588_);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn3);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp539_);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn2);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp500_);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ppn);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cp0);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (p);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp439_);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng2);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (b);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e2);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e1);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e0);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn2);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp182_);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp160_);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 277 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 4511 "DomGDocumentTest.c"
			goto __catch183_g_error;
		}
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp594_ = pn4;
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		gxml_dom_element_set_attribute (_tmp594_, "class", "black", &_inner_error_);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn4);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp588_);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn3);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp539_);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn2);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp500_);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ppn);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cp0);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (p);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp439_);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng2);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (b);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e2);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e1);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e0);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn2);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp182_);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp160_);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 278 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 4570 "DomGDocumentTest.c"
			goto __catch183_g_error;
		}
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp595_ = p;
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp596_ = pn4;
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp597_ = p;
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp598_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp597_);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp599_ = _tmp598_;
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp600_ = _tmp599_;
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp601_ = gee_list_get ((GeeList*) _tmp600_, 0);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp602_ = (GXmlDomNode*) _tmp601_;
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp603_ = gxml_dom_node_replace_child ((GXmlDomNode*) _tmp595_, (GXmlDomNode*) _tmp596_, _tmp602_, &_inner_error_);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp604_ = _tmp603_;
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp604_);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp602_);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp600_);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn4);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp588_);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn3);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp539_);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn2);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp500_);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ppn);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cp0);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (p);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp439_);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng2);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (b);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e2);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e1);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e0);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn2);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp182_);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp160_);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 279 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 4651 "DomGDocumentTest.c"
			goto __catch183_g_error;
		}
#line 280 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp605_ = p;
#line 280 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp606_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp605_);
#line 280 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp607_ = _tmp606_;
#line 280 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp608_ = _tmp607_;
#line 280 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp609_ = gxml_dom_html_collection_get_length (_tmp608_);
#line 280 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp610_ = _tmp609_;
#line 280 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp610_ == 8, "p.children.length == 8");
#line 280 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp608_);
#line 281 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp611_ = p;
#line 281 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp612_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp611_);
#line 281 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp613_ = _tmp612_;
#line 281 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp614_ = _tmp613_;
#line 281 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp615_ = gee_list_get ((GeeList*) _tmp614_, 0);
#line 281 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp616_ = (GXmlDomElement*) _tmp615_;
#line 281 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp616_, GXML_TYPE_DOM_ELEMENT), "p.children[0] is DomElement");
#line 281 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp616_);
#line 281 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp614_);
#line 282 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp617_ = p;
#line 282 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp618_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp617_);
#line 282 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp619_ = _tmp618_;
#line 282 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp620_ = _tmp619_;
#line 282 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp621_ = gee_list_get ((GeeList*) _tmp620_, 0);
#line 282 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp622_ = (GXmlDomElement*) _tmp621_;
#line 282 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp623_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp622_);
#line 282 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp624_ = _tmp623_;
#line 282 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp625_ = _tmp624_;
#line 282 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp625_, "p") == 0, "p.children[0].node_name == \"p\"");
#line 282 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp625_);
#line 282 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp622_);
#line 282 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp620_);
#line 283 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp626_ = p;
#line 283 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp627_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp626_);
#line 283 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp628_ = _tmp627_;
#line 283 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp629_ = _tmp628_;
#line 283 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp630_ = gee_list_get ((GeeList*) _tmp629_, 0);
#line 283 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp631_ = (GXmlDomElement*) _tmp630_;
#line 283 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp632_ = gxml_dom_element_get_attribute (_tmp631_, "id");
#line 283 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp633_ = _tmp632_;
#line 283 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp633_, "newp2") == 0, "p.children[0].get_attribute (\"id\") == \"newp2\"");
#line 283 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp633_);
#line 283 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp631_);
#line 283 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp629_);
#line 284 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp634_ = p;
#line 284 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp635_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp634_);
#line 284 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp636_ = _tmp635_;
#line 284 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp637_ = _tmp636_;
#line 284 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp638_ = gee_list_get ((GeeList*) _tmp637_, 0);
#line 284 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp639_ = (GXmlDomElement*) _tmp638_;
#line 284 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp640_ = gxml_dom_element_get_attribute (_tmp639_, "class");
#line 284 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp641_ = _tmp640_;
#line 284 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp641_, "black") == 0, "p.children[0].get_attribute (\"class\") == \"black\"");
#line 284 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp641_);
#line 284 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp639_);
#line 284 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp637_);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp642_ = p;
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp643_ = p;
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp644_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp643_);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp645_ = _tmp644_;
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp646_ = _tmp645_;
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp647_ = gee_list_get ((GeeList*) _tmp646_, 0);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp648_ = (GXmlDomElement*) _tmp647_;
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp649_ = gxml_dom_node_remove_child ((GXmlDomNode*) _tmp642_, (GXmlDomNode*) _tmp648_, &_inner_error_);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp650_ = _tmp649_;
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp650_);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp648_);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp646_);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn4);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp588_);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn3);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp539_);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn2);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp500_);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ppn);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cp0);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (p);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pn);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp439_);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng2);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ng);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (b);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e2);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e1);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e0);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn2);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp182_);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (cn);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp160_);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (e);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 285 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 4838 "DomGDocumentTest.c"
			goto __catch183_g_error;
		}
#line 286 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp651_ = p;
#line 286 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp652_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp651_);
#line 286 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp653_ = _tmp652_;
#line 286 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp654_ = _tmp653_;
#line 286 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp655_ = gxml_dom_html_collection_get_length (_tmp654_);
#line 286 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp656_ = _tmp655_;
#line 286 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp656_ == 7, "p.children.length == 7");
#line 286 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp654_);
#line 287 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp657_ = p;
#line 287 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp658_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp657_);
#line 287 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp659_ = _tmp658_;
#line 287 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp660_ = _tmp659_;
#line 287 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp661_ = gee_list_get ((GeeList*) _tmp660_, 0);
#line 287 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp662_ = (GXmlDomElement*) _tmp661_;
#line 287 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp662_, GXML_TYPE_DOM_ELEMENT), "p.children[0] is DomElement");
#line 287 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp662_);
#line 287 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp660_);
#line 288 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp663_ = p;
#line 288 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp664_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp663_);
#line 288 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp665_ = _tmp664_;
#line 288 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp666_ = _tmp665_;
#line 288 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp667_ = gee_list_get ((GeeList*) _tmp666_, 0);
#line 288 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp668_ = (GXmlDomElement*) _tmp667_;
#line 288 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp669_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp668_);
#line 288 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp670_ = _tmp669_;
#line 288 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp671_ = _tmp670_;
#line 288 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp671_, "p") == 0, "p.children[0].node_name == \"p\"");
#line 288 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp671_);
#line 288 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp668_);
#line 288 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp666_);
#line 289 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp672_ = p;
#line 289 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp673_ = gxml_dom_element_has_attribute (_tmp672_, "id");
#line 289 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (!_tmp673_, "!p.has_attribute (\"id\")");
#line 290 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp674_ = p;
#line 290 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp675_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp674_);
#line 290 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp676_ = _tmp675_;
#line 290 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp677_ = _tmp676_;
#line 290 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp678_ = gee_list_get ((GeeList*) _tmp677_, 0);
#line 290 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp679_ = (GXmlDomElement*) _tmp678_;
#line 290 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp680_ = gxml_dom_element_get_attribute (_tmp679_, "class");
#line 290 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp681_ = _tmp680_;
#line 290 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp681_, "black") == 0, "p.children[0].get_attribute (\"class\") == \"black\"");
#line 290 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp681_);
#line 290 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp679_);
#line 290 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp677_);
#line 292 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp682_ = ng2;
#line 292 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp682_, GXML_TYPE_DOM_ELEMENT), "ng2 is DomElement");
#line 293 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp683_ = ng2;
#line 293 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp684_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp683_);
#line 293 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp685_ = _tmp684_;
#line 293 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp686_ = _tmp685_;
#line 293 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp686_, "OtherNode") == 0, "ng2.node_name == \"OtherNode\"");
#line 293 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp686_);
#line 294 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp687_ = ng2;
#line 294 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp688_ = gxml_dom_node_lookup_namespace_uri ((GXmlDomNode*) _tmp687_, NULL);
#line 294 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp689_ = _tmp688_;
#line 294 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp689_, "http://live.gnome.org/GXml") == 0, "ng2.lookup_namespace_uri (null) == \"http://live.gnome.org/GXml\"");
#line 294 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp689_);
#line 295 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp690_ = ng2;
#line 295 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp691_ = gxml_dom_node_lookup_prefix ((GXmlDomNode*) _tmp690_, "http://live.gnome.org/GXml");
#line 295 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp692_ = _tmp691_;
#line 295 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp692_ == NULL, "ng2.lookup_prefix (\"http://live.gnome.org/GXml\") == null");
#line 295 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp692_);
#line 296 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp693_ = ng2;
#line 296 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp694_ = gxml_dom_element_get_tag_name (_tmp693_);
#line 296 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp695_ = _tmp694_;
#line 296 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp696_ = _tmp695_;
#line 296 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp696_, "OtherNode") == 0, "ng2.tag_name == \"OtherNode\"");
#line 296 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp696_);
#line 301 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp697_ = doc;
#line 301 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp698_ = gxml_dom_document_get_document_element (_tmp697_);
#line 301 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp699_ = _tmp698_;
#line 301 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp700_ = _tmp699_;
#line 301 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp701_ = gxml_dom_element_get_elements_by_tag_name (_tmp700_, "p");
#line 301 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp702_ = _tmp701_;
#line 301 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp700_);
#line 301 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		l = _tmp702_;
#line 302 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (l != NULL, "l != null");
#line 303 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (l, GXML_TYPE_DOM_HTML_COLLECTION), "l is DomHTMLCollection");
#line 304 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp703_ = gxml_dom_html_collection_get_length (l);
#line 304 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp704_ = _tmp703_;
#line 304 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp704_ == 5, "l.length == 5");
#line 305 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp705_ = gee_list_get ((GeeList*) l, 0);
#line 305 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp706_ = (GXmlDomElement*) _tmp705_;
#line 305 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp706_, GXML_TYPE_DOM_ELEMENT), "l[0] is DomElement");
#line 305 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp706_);
#line 306 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp707_ = gee_list_get ((GeeList*) l, 1);
#line 306 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp708_ = (GXmlDomElement*) _tmp707_;
#line 306 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp708_, GXML_TYPE_DOM_ELEMENT), "l[1] is DomElement");
#line 306 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp708_);
#line 307 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp709_ = gee_list_get ((GeeList*) l, 1);
#line 307 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp710_ = (GXmlDomElement*) _tmp709_;
#line 307 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp710_, GXML_TYPE_DOM_ELEMENT), "l[1] is DomElement");
#line 307 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp710_);
#line 308 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp711_ = gee_list_get ((GeeList*) l, 0);
#line 308 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp712_ = (GXmlDomElement*) _tmp711_;
#line 308 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp713_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp712_);
#line 308 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp714_ = _tmp713_;
#line 308 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp715_ = _tmp714_;
#line 308 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp715_, "p") == 0, "l[0].node_name == \"p\"");
#line 308 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp715_);
#line 308 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp712_);
#line 309 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp716_ = gee_list_get ((GeeList*) l, 1);
#line 309 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp717_ = (GXmlDomElement*) _tmp716_;
#line 309 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp718_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp717_);
#line 309 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp719_ = _tmp718_;
#line 309 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp720_ = _tmp719_;
#line 309 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp720_, "p") == 0, "l[1].node_name == \"p\"");
#line 309 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp720_);
#line 309 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp717_);
#line 310 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp721_ = gee_list_get ((GeeList*) l, 2);
#line 310 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp722_ = (GXmlDomElement*) _tmp721_;
#line 310 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp723_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp722_);
#line 310 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp724_ = _tmp723_;
#line 310 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp725_ = _tmp724_;
#line 310 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp725_, "p") == 0, "l[2].node_name == \"p\"");
#line 310 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp725_);
#line 310 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp722_);
#line 311 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp726_ = doc;
#line 311 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp727_ = gxml_dom_document_get_document_element (_tmp726_);
#line 311 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp728_ = _tmp727_;
#line 311 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp729_ = _tmp728_;
#line 311 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp730_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp729_);
#line 311 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp731_ = _tmp730_;
#line 311 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp732_ = _tmp731_;
#line 311 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp733_ = gxml_dom_html_collection_get_length (_tmp732_);
#line 311 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp734_ = _tmp733_;
#line 311 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp734_ == 1, "doc.document_element.children.length == 1");
#line 311 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp732_);
#line 311 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp729_);
#line 312 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp735_ = doc;
#line 312 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp736_ = gxml_dom_document_get_document_element (_tmp735_);
#line 312 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp737_ = _tmp736_;
#line 312 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp738_ = _tmp737_;
#line 312 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp739_ = gxml_dom_element_get_elements_by_tag_name (_tmp738_, "OtherNode");
#line 312 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp740_ = _tmp739_;
#line 312 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp738_);
#line 312 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		lnst = _tmp740_;
#line 313 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp741_ = gxml_dom_html_collection_get_length (lnst);
#line 313 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp742_ = _tmp741_;
#line 313 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp742_ == 1, "lnst.length == 1");
#line 314 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp743_ = gxml_dom_html_collection_item (lnst, 0);
#line 314 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		nnst = _tmp743_;
#line 315 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp744_ = gxml_dom_element_get_namespace_uri (nnst);
#line 315 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp745_ = _tmp744_;
#line 315 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp746_ = _tmp745_;
#line 315 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp746_, "http://live.gnome.org/GXml") == 0, "nnst.namespace_uri == \"http://live.gnome.org/GXml\"");
#line 315 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp746_);
#line 316 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp747_ = doc;
#line 316 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp748_ = gxml_dom_document_get_document_element (_tmp747_);
#line 316 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp749_ = _tmp748_;
#line 316 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp750_ = _tmp749_;
#line 316 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp751_ = gxml_dom_element_get_elements_by_tag_name_ns (_tmp750_, "http://live.gnome.org/GXml", "OtherNode");
#line 316 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp752_ = _tmp751_;
#line 316 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp750_);
#line 316 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		lns = _tmp752_;
#line 317 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (lns != NULL, "lns != null");
#line 318 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (lns, GXML_TYPE_DOM_HTML_COLLECTION), "lns is DomHTMLCollection");
#line 322 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp753_ = gxml_dom_html_collection_get_length (lns);
#line 322 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp754_ = _tmp753_;
#line 322 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp754_ == 1, "lns.length == 1");
#line 323 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp755_ = gxml_dom_html_collection_item (lns, 0);
#line 323 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp756_ = _tmp755_;
#line 323 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp756_, GXML_TYPE_DOM_ELEMENT), "lns.item (0) is DomElement");
#line 323 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp756_);
#line 324 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp757_ = gxml_dom_html_collection_item (lns, 0);
#line 324 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp758_ = _tmp757_;
#line 324 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp759_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp758_);
#line 324 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp760_ = _tmp759_;
#line 324 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp761_ = _tmp760_;
#line 324 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp761_, "OtherNode") == 0, "lns.item (0).node_name == \"OtherNode\"");
#line 324 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp761_);
#line 324 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp758_);
#line 325 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp762_ = doc;
#line 325 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp763_ = gxml_dom_document_get_document_element (_tmp762_);
#line 325 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp764_ = _tmp763_;
#line 325 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp765_ = _tmp764_;
#line 325 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp766_ = gxml_dom_element_get_elements_by_class_name (_tmp765_, "black");
#line 325 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp767_ = _tmp766_;
#line 325 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp765_);
#line 325 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		lcl = _tmp767_;
#line 326 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (lcl != NULL, "lcl != null");
#line 327 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (lcl, GXML_TYPE_DOM_HTML_COLLECTION), "lcl is DomHTMLCollection");
#line 328 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp768_ = gxml_dom_html_collection_get_length (lcl);
#line 328 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp769_ = _tmp768_;
#line 328 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp769_ == 3, "lcl.length == 3");
#line 329 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp770_ = gxml_dom_html_collection_item (lcl, 0);
#line 329 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp771_ = _tmp770_;
#line 329 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp771_, GXML_TYPE_DOM_ELEMENT), "lcl.item (0) is DomElement");
#line 329 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp771_);
#line 330 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp772_ = gxml_dom_html_collection_item (lcl, 1);
#line 330 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp773_ = _tmp772_;
#line 330 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp773_, GXML_TYPE_DOM_ELEMENT), "lcl.item (1) is DomElement");
#line 330 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp773_);
#line 331 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp774_ = gxml_dom_html_collection_item (lcl, 0);
#line 331 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp775_ = _tmp774_;
#line 331 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp776_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp775_);
#line 331 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp777_ = _tmp776_;
#line 331 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp778_ = _tmp777_;
#line 331 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp778_, "p") == 0, "lcl.item (0).node_name == \"p\"");
#line 331 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp778_);
#line 331 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp775_);
#line 332 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp779_ = gxml_dom_html_collection_item (lcl, 1);
#line 332 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp780_ = _tmp779_;
#line 332 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp781_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp780_);
#line 332 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp782_ = _tmp781_;
#line 332 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp783_ = _tmp782_;
#line 332 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp783_, "p") == 0, "lcl.item (1).node_name == \"p\"");
#line 332 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp783_);
#line 332 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp780_);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (lcl);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (lns);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (nnst);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (lnst);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (l);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (pn4);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp588_);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (pn3);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp539_);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (pn2);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp500_);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (ppn);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (cp0);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (p);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (pn);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp439_);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (ng2);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (ng);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (b);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (e2);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (e1);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (e0);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (cn2);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp182_);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (cn);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp160_);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (t);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (e);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (doc);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp0_);
#line 5321 "DomGDocumentTest.c"
	}
	goto __finally183;
	__catch183_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp784_ = NULL;
		gchar* _tmp785_ = NULL;
		gchar* _tmp786_ = NULL;
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		e = _inner_error_;
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_inner_error_ = NULL;
#line 334 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp784_ = e->message;
#line 334 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp785_ = g_strconcat ("Error: ", _tmp784_, NULL);
#line 334 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp786_ = _tmp785_;
#line 334 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_message ("DomGDocumentTest.vala:334: %s", _tmp786_);
#line 334 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp786_);
#line 335 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_assert_not_reached ();
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_error_free0 (e);
#line 5348 "DomGDocumentTest.c"
	}
	__finally183:
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 168 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		return;
#line 5359 "DomGDocumentTest.c"
	}
}


static void ___lambda197__gtest_func (void) {
#line 167 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	__lambda197_ ();
#line 5367 "DomGDocumentTest.c"
}


static void __lambda198_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* _tmp0_ = NULL;
		GXmlGDocument* _tmp1_ = NULL;
		GXmlDomDocument* doc = NULL;
		GXmlGDocument* _tmp2_ = NULL;
		GXmlDomDocument* _tmp3_ = NULL;
		GXmlDomDocument* _tmp4_ = NULL;
		GXmlDomDocument* _tmp5_ = NULL;
		GXmlDomElement* _tmp6_ = NULL;
		GXmlDomElement* _tmp7_ = NULL;
		GXmlDomElement* _tmp8_ = NULL;
		GXmlDomHTMLCollection* _tmp9_ = NULL;
		GXmlDomHTMLCollection* _tmp10_ = NULL;
		GXmlDomHTMLCollection* _tmp11_ = NULL;
		gint _tmp12_ = 0;
		gint _tmp13_ = 0;
		GXmlDomElement* n1 = NULL;
		GXmlDomDocument* _tmp14_ = NULL;
		GXmlDomElement* _tmp15_ = NULL;
		GXmlDomDocument* _tmp16_ = NULL;
		GXmlDomElement* _tmp17_ = NULL;
		GXmlDomElement* _tmp18_ = NULL;
		GXmlDomElement* _tmp19_ = NULL;
		GXmlDomElement* _tmp20_ = NULL;
		GXmlDomNode* _tmp21_ = NULL;
		GXmlDomNode* _tmp22_ = NULL;
		GXmlDomElement* n = NULL;
		GXmlDomDocument* _tmp23_ = NULL;
		GXmlDomElement* _tmp24_ = NULL;
		GXmlDomElement* _tmp25_ = NULL;
		GXmlDomElement* _tmp26_ = NULL;
		GXmlDomHTMLCollection* _tmp27_ = NULL;
		GXmlDomHTMLCollection* _tmp28_ = NULL;
		GXmlDomHTMLCollection* _tmp29_ = NULL;
		gpointer _tmp30_ = NULL;
		GXmlDomElement* _tmp31_ = NULL;
		GXmlDomElement* _tmp32_ = NULL;
		GXmlDomElement* _tmp33_ = NULL;
		GXmlDomElement* _tmp34_ = NULL;
		gchar* _tmp35_ = NULL;
		gchar* _tmp36_ = NULL;
		gchar* _tmp37_ = NULL;
		GXmlDomElement* _tmp38_ = NULL;
		GXmlDomElement* _tmp39_ = NULL;
		GXmlDomElement* _tmp40_ = NULL;
		GeeList* _tmp41_ = NULL;
		GeeList* _tmp42_ = NULL;
		GeeList* _tmp43_ = NULL;
		gint _tmp44_ = 0;
		gint _tmp45_ = 0;
		GXmlDomElement* _tmp46_ = NULL;
		GeeList* _tmp47_ = NULL;
		GeeList* _tmp48_ = NULL;
		GeeList* _tmp49_ = NULL;
		gpointer _tmp50_ = NULL;
		GXmlNamespace* _tmp51_ = NULL;
		gchar* _tmp52_ = NULL;
		gchar* _tmp53_ = NULL;
		gchar* _tmp54_ = NULL;
		GXmlDomElement* _tmp55_ = NULL;
		GeeList* _tmp56_ = NULL;
		GeeList* _tmp57_ = NULL;
		GeeList* _tmp58_ = NULL;
		gpointer _tmp59_ = NULL;
		GXmlNamespace* _tmp60_ = NULL;
		gchar* _tmp61_ = NULL;
		gchar* _tmp62_ = NULL;
		gchar* _tmp63_ = NULL;
		GXmlDomElement* _tmp64_ = NULL;
		gchar* _tmp65_ = NULL;
		gchar* _tmp66_ = NULL;
		gchar* _tmp67_ = NULL;
		GXmlDomElement* _tmp68_ = NULL;
		gchar* _tmp69_ = NULL;
		gchar* _tmp70_ = NULL;
		gchar* _tmp71_ = NULL;
		GXmlDomElement* _tmp72_ = NULL;
		gchar* _tmp73_ = NULL;
		gchar* _tmp74_ = NULL;
		gchar* _tmp75_ = NULL;
		GXmlDomElement* _tmp76_ = NULL;
		gchar* _tmp77_ = NULL;
		gchar* _tmp78_ = NULL;
		gchar* _tmp79_ = NULL;
		GXmlDomElement* _tmp80_ = NULL;
		gchar* _tmp81_ = NULL;
		gchar* _tmp82_ = NULL;
		gchar* _tmp83_ = NULL;
		GXmlDomElement* _tmp84_ = NULL;
		GXmlDomTokenList* _tmp85_ = NULL;
		GXmlDomTokenList* _tmp86_ = NULL;
		GXmlDomTokenList* _tmp87_ = NULL;
		GXmlDomElement* _tmp88_ = NULL;
		GXmlDomTokenList* _tmp89_ = NULL;
		GXmlDomTokenList* _tmp90_ = NULL;
		GXmlDomTokenList* _tmp91_ = NULL;
		gint _tmp92_ = 0;
		gint _tmp93_ = 0;
		GXmlDomElement* _tmp94_ = NULL;
		GXmlDomTokenList* _tmp95_ = NULL;
		GXmlDomTokenList* _tmp96_ = NULL;
		GXmlDomTokenList* _tmp97_ = NULL;
		gchar* _tmp98_ = NULL;
		gchar* _tmp99_ = NULL;
		GXmlDomElement* _tmp100_ = NULL;
		GXmlDomTokenList* _tmp101_ = NULL;
		GXmlDomTokenList* _tmp102_ = NULL;
		GXmlDomTokenList* _tmp103_ = NULL;
		gchar* _tmp104_ = NULL;
		gchar* _tmp105_ = NULL;
		GXmlDomElement* _tmp106_ = NULL;
		GXmlDomNamedNodeMap* _tmp107_ = NULL;
		GXmlDomNamedNodeMap* _tmp108_ = NULL;
		GXmlDomNamedNodeMap* _tmp109_ = NULL;
		GXmlDomElement* _tmp110_ = NULL;
		GXmlDomNamedNodeMap* _tmp111_ = NULL;
		GXmlDomNamedNodeMap* _tmp112_ = NULL;
		GXmlDomNamedNodeMap* _tmp113_ = NULL;
		gint _tmp114_ = 0;
		gint _tmp115_ = 0;
		GXmlDomElement* _tmp116_ = NULL;
		GXmlDomNamedNodeMap* _tmp117_ = NULL;
		GXmlDomNamedNodeMap* _tmp118_ = NULL;
		GXmlDomNamedNodeMap* _tmp119_ = NULL;
		GXmlDomNode* _tmp120_ = NULL;
		GXmlDomNode* _tmp121_ = NULL;
		GXmlDomElement* _tmp122_ = NULL;
		GXmlDomNamedNodeMap* _tmp123_ = NULL;
		GXmlDomNamedNodeMap* _tmp124_ = NULL;
		GXmlDomNamedNodeMap* _tmp125_ = NULL;
		GXmlDomNode* _tmp126_ = NULL;
		GXmlDomNode* _tmp127_ = NULL;
		gchar* _tmp128_ = NULL;
		gchar* _tmp129_ = NULL;
		gchar* _tmp130_ = NULL;
		GXmlDomElement* _tmp131_ = NULL;
		GXmlDomNamedNodeMap* _tmp132_ = NULL;
		GXmlDomNamedNodeMap* _tmp133_ = NULL;
		GXmlDomNamedNodeMap* _tmp134_ = NULL;
		GXmlDomNode* _tmp135_ = NULL;
		GXmlDomNode* _tmp136_ = NULL;
		gchar* _tmp137_ = NULL;
		gchar* _tmp138_ = NULL;
		gchar* _tmp139_ = NULL;
		GXmlDomElement* _tmp140_ = NULL;
		gchar* _tmp141_ = NULL;
		gchar* _tmp142_ = NULL;
		gchar* _tmp143_ = NULL;
		GXmlDomElement* _tmp144_ = NULL;
		gchar* _tmp145_ = NULL;
		gchar* _tmp146_ = NULL;
		GXmlDomElement* _tmp147_ = NULL;
		gchar* _tmp148_ = NULL;
		gchar* _tmp149_ = NULL;
		GXmlDomElement* _tmp150_ = NULL;
		gchar* _tmp151_ = NULL;
		gchar* _tmp152_ = NULL;
		gchar* _tmp153_ = NULL;
		GXmlDomElement* _tmp154_ = NULL;
		GXmlDomElement* _tmp155_ = NULL;
		gchar* _tmp156_ = NULL;
		gchar* _tmp157_ = NULL;
		GXmlDomElement* _tmp158_ = NULL;
		gboolean _tmp159_ = FALSE;
		GXmlDomElement* n2 = NULL;
		GXmlDomDocument* _tmp160_ = NULL;
		GXmlDomElement* _tmp161_ = NULL;
		GXmlDomDocument* _tmp162_ = NULL;
		GXmlDomElement* _tmp163_ = NULL;
		GXmlDomElement* _tmp164_ = NULL;
		GXmlDomElement* _tmp165_ = NULL;
		GXmlDomElement* _tmp166_ = NULL;
		GXmlDomNode* _tmp167_ = NULL;
		GXmlDomNode* _tmp168_ = NULL;
		GXmlDomDocument* _tmp169_ = NULL;
		GXmlDomElement* _tmp170_ = NULL;
		GXmlDomElement* _tmp171_ = NULL;
		GXmlDomElement* _tmp172_ = NULL;
		GXmlDomHTMLCollection* _tmp173_ = NULL;
		GXmlDomHTMLCollection* _tmp174_ = NULL;
		GXmlDomHTMLCollection* _tmp175_ = NULL;
		gint _tmp176_ = 0;
		gint _tmp177_ = 0;
		GXmlDomElement* _tmp178_ = NULL;
		GXmlDomNamedNodeMap* _tmp179_ = NULL;
		GXmlDomNamedNodeMap* _tmp180_ = NULL;
		GXmlDomNamedNodeMap* _tmp181_ = NULL;
		gint _tmp182_ = 0;
		gint _tmp183_ = 0;
		GXmlDomElement* _tmp184_ = NULL;
		GXmlDomElement* _tmp185_ = NULL;
		GXmlDomNamedNodeMap* _tmp186_ = NULL;
		GXmlDomNamedNodeMap* _tmp187_ = NULL;
		GXmlDomNamedNodeMap* _tmp188_ = NULL;
		gint _tmp189_ = 0;
		gint _tmp190_ = 0;
		GXmlDomElement* _tmp191_ = NULL;
		gboolean _tmp192_ = FALSE;
		GXmlDomElement* _tmp193_ = NULL;
		gboolean _tmp194_ = FALSE;
		GXmlDomElement* _tmp195_ = NULL;
		gchar* _tmp196_ = NULL;
		gchar* _tmp197_ = NULL;
		GXmlDomElement* _tmp198_ = NULL;
		gchar* _tmp199_ = NULL;
		gchar* _tmp200_ = NULL;
#line 343 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp1_ = gxml_gdocument_new_from_string (DOM_GDOCUMENT_TEST_HTMLDOC, 0, &_inner_error_);
#line 343 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp0_ = _tmp1_;
#line 343 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 5585 "DomGDocumentTest.c"
			goto __catch184_g_error;
		}
#line 343 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp2_ = _tmp0_;
#line 343 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp0_ = NULL;
#line 343 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp3_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp2_, GXML_TYPE_DOM_DOCUMENT) ? ((GXmlDomDocument*) _tmp2_) : NULL;
#line 343 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp3_ == NULL) {
#line 343 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 5598 "DomGDocumentTest.c"
		}
#line 343 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		doc = _tmp3_;
#line 344 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp4_ = doc;
#line 344 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp4_, GXML_TYPE_DOM_DOCUMENT), "doc is DomDocument");
#line 345 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp5_ = doc;
#line 345 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp6_ = gxml_dom_document_get_document_element (_tmp5_);
#line 345 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp7_ = _tmp6_;
#line 345 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp8_ = _tmp7_;
#line 345 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp9_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp8_);
#line 345 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp10_ = _tmp9_;
#line 345 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp11_ = _tmp10_;
#line 345 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp12_ = gee_collection_get_size ((GeeCollection*) _tmp11_);
#line 345 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp13_ = _tmp12_;
#line 345 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp13_ == 1, "doc.document_element.children.size == 1");
#line 345 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp11_);
#line 345 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp8_);
#line 346 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp14_ = doc;
#line 346 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp15_ = gxml_dom_document_create_element_ns (_tmp14_, "http://live.gnome.org/GXml", "gxml:code", &_inner_error_);
#line 346 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		n1 = _tmp15_;
#line 346 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 346 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 346 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 5642 "DomGDocumentTest.c"
			goto __catch184_g_error;
		}
#line 347 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp16_ = doc;
#line 347 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp17_ = gxml_dom_document_get_document_element (_tmp16_);
#line 347 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp18_ = _tmp17_;
#line 347 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp19_ = _tmp18_;
#line 347 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp20_ = n1;
#line 347 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp21_ = gxml_dom_node_append_child ((GXmlDomNode*) _tmp19_, (GXmlDomNode*) _tmp20_, &_inner_error_);
#line 347 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp22_ = _tmp21_;
#line 347 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp22_);
#line 347 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp19_);
#line 347 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 347 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n1);
#line 347 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 347 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 5671 "DomGDocumentTest.c"
			goto __catch184_g_error;
		}
#line 348 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp23_ = doc;
#line 348 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp24_ = gxml_dom_document_get_document_element (_tmp23_);
#line 348 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp25_ = _tmp24_;
#line 348 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp26_ = _tmp25_;
#line 348 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp27_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp26_);
#line 348 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp28_ = _tmp27_;
#line 348 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp29_ = _tmp28_;
#line 348 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp30_ = gee_list_get ((GeeList*) _tmp29_, 1);
#line 348 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp31_ = (GXmlDomElement*) _tmp30_;
#line 348 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp32_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp31_, GXML_TYPE_DOM_ELEMENT) ? ((GXmlDomElement*) _tmp31_) : NULL;
#line 348 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp32_ == NULL) {
#line 348 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp31_);
#line 5698 "DomGDocumentTest.c"
		}
#line 348 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp33_ = _tmp32_;
#line 348 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp29_);
#line 348 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp26_);
#line 348 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		n = _tmp33_;
#line 349 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp34_ = n;
#line 349 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp35_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp34_);
#line 349 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp36_ = _tmp35_;
#line 349 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp37_ = _tmp36_;
#line 349 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp37_, "code") == 0, "n.node_name == \"code\"");
#line 349 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp37_);
#line 350 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp38_ = n;
#line 350 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		gxml_dom_element_set_attribute (_tmp38_, "id", "0y1", &_inner_error_);
#line 350 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 350 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 350 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n1);
#line 350 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 350 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 5734 "DomGDocumentTest.c"
			goto __catch184_g_error;
		}
#line 351 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp39_ = n;
#line 351 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		gxml_dom_element_set_attribute (_tmp39_, "class", "login black", &_inner_error_);
#line 351 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 351 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 351 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n1);
#line 351 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 351 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 5751 "DomGDocumentTest.c"
			goto __catch184_g_error;
		}
#line 352 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp40_ = n;
#line 352 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp41_ = gxml_node_get_namespaces (G_TYPE_CHECK_INSTANCE_TYPE (_tmp40_, GXML_TYPE_NODE) ? ((GXmlNode*) _tmp40_) : NULL);
#line 352 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp42_ = _tmp41_;
#line 352 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp43_ = _tmp42_;
#line 352 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp44_ = gee_collection_get_size ((GeeCollection*) _tmp43_);
#line 352 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp45_ = _tmp44_;
#line 352 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp45_ == 1, "(n as GXml.Node).namespaces.size == 1");
#line 352 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp43_);
#line 353 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp46_ = n;
#line 353 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp47_ = gxml_node_get_namespaces (G_TYPE_CHECK_INSTANCE_TYPE (_tmp46_, GXML_TYPE_NODE) ? ((GXmlNode*) _tmp46_) : NULL);
#line 353 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp48_ = _tmp47_;
#line 353 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp49_ = _tmp48_;
#line 353 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp50_ = gee_list_get (_tmp49_, 0);
#line 353 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp51_ = (GXmlNamespace*) _tmp50_;
#line 353 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp52_ = gxml_namespace_get_uri (_tmp51_);
#line 353 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp53_ = _tmp52_;
#line 353 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp54_ = _tmp53_;
#line 353 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp54_, "http://live.gnome.org/GXml") == 0, "(n as GXml.Node).namespaces[0].uri == \"http://live.gnome.org/GXml\"");
#line 353 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp54_);
#line 353 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp51_);
#line 353 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp49_);
#line 354 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp55_ = n;
#line 354 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp56_ = gxml_node_get_namespaces (G_TYPE_CHECK_INSTANCE_TYPE (_tmp55_, GXML_TYPE_NODE) ? ((GXmlNode*) _tmp55_) : NULL);
#line 354 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp57_ = _tmp56_;
#line 354 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp58_ = _tmp57_;
#line 354 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp59_ = gee_list_get (_tmp58_, 0);
#line 354 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp60_ = (GXmlNamespace*) _tmp59_;
#line 354 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp61_ = gxml_namespace_get_prefix (_tmp60_);
#line 354 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp62_ = _tmp61_;
#line 354 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp63_ = _tmp62_;
#line 354 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp63_, "gxml") == 0, "(n as GXml.Node).namespaces[0].prefix == \"gxml\"");
#line 354 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp63_);
#line 354 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp60_);
#line 354 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp58_);
#line 355 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp64_ = n;
#line 355 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp65_ = gxml_dom_element_get_namespace_uri (_tmp64_);
#line 355 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp66_ = _tmp65_;
#line 355 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp67_ = _tmp66_;
#line 355 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp67_, "http://live.gnome.org/GXml") == 0, "n.namespace_uri == \"http://live.gnome.org/GXml\"");
#line 355 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp67_);
#line 356 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp68_ = n;
#line 356 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp69_ = gxml_dom_element_get_prefix (_tmp68_);
#line 356 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp70_ = _tmp69_;
#line 356 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp71_ = _tmp70_;
#line 356 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp71_, "gxml") == 0, "n.prefix == \"gxml\"");
#line 356 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp71_);
#line 357 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp72_ = n;
#line 357 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp73_ = gxml_dom_element_get_local_name (_tmp72_);
#line 357 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp74_ = _tmp73_;
#line 357 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp75_ = _tmp74_;
#line 357 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp75_, "code") == 0, "n.local_name == \"code\"");
#line 357 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp75_);
#line 358 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp76_ = n;
#line 358 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp77_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp76_);
#line 358 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp78_ = _tmp77_;
#line 358 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp79_ = _tmp78_;
#line 358 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp79_, "code") == 0, "n.node_name == \"code\"");
#line 358 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp79_);
#line 359 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp80_ = n;
#line 359 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp81_ = gxml_dom_element_get_id (_tmp80_);
#line 359 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp82_ = _tmp81_;
#line 359 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp83_ = _tmp82_;
#line 359 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp83_, "0y1") == 0, "n.id == \"0y1\"");
#line 359 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp83_);
#line 360 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp84_ = n;
#line 360 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp85_ = gxml_dom_element_get_class_list (_tmp84_);
#line 360 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp86_ = _tmp85_;
#line 360 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp87_ = _tmp86_;
#line 360 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp87_ != NULL, "n.class_list != null");
#line 360 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp87_);
#line 361 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp88_ = n;
#line 361 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp89_ = gxml_dom_element_get_class_list (_tmp88_);
#line 361 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp90_ = _tmp89_;
#line 361 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp91_ = _tmp90_;
#line 361 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp92_ = gxml_dom_token_list_get_length (_tmp91_);
#line 361 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp93_ = _tmp92_;
#line 361 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp93_ == 2, "n.class_list.length == 2");
#line 361 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp91_);
#line 362 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp94_ = n;
#line 362 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp95_ = gxml_dom_element_get_class_list (_tmp94_);
#line 362 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp96_ = _tmp95_;
#line 362 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp97_ = _tmp96_;
#line 362 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp98_ = gxml_dom_token_list_item (_tmp97_, 0);
#line 362 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp99_ = _tmp98_;
#line 362 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp99_, "login") == 0, "n.class_list.item (0) == \"login\"");
#line 362 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp99_);
#line 362 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp97_);
#line 363 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp100_ = n;
#line 363 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp101_ = gxml_dom_element_get_class_list (_tmp100_);
#line 363 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp102_ = _tmp101_;
#line 363 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp103_ = _tmp102_;
#line 363 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp104_ = gxml_dom_token_list_item (_tmp103_, 1);
#line 363 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp105_ = _tmp104_;
#line 363 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp105_, "black") == 0, "n.class_list.item (1) == \"black\"");
#line 363 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp105_);
#line 363 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp103_);
#line 364 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp106_ = n;
#line 364 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp107_ = gxml_dom_element_get_attributes (_tmp106_);
#line 364 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp108_ = _tmp107_;
#line 364 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp109_ = _tmp108_;
#line 364 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp109_ != NULL, "n.attributes != null");
#line 364 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp109_);
#line 365 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp110_ = n;
#line 365 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp111_ = gxml_dom_element_get_attributes (_tmp110_);
#line 365 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp112_ = _tmp111_;
#line 365 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp113_ = _tmp112_;
#line 365 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp114_ = gxml_dom_named_node_map_get_length (_tmp113_);
#line 365 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp115_ = _tmp114_;
#line 365 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp115_ == 2, "n.attributes.length == 2");
#line 365 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp113_);
#line 366 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp116_ = n;
#line 366 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp117_ = gxml_dom_element_get_attributes (_tmp116_);
#line 366 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp118_ = _tmp117_;
#line 366 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp119_ = _tmp118_;
#line 366 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp120_ = gxml_dom_named_node_map_get_named_item (_tmp119_, "id");
#line 366 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp121_ = _tmp120_;
#line 366 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp121_, GXML_TYPE_DOM_NODE), "n.attributes.get_named_item (\"id\") is DomNode");
#line 366 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp121_);
#line 366 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp119_);
#line 367 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp122_ = n;
#line 367 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp123_ = gxml_dom_element_get_attributes (_tmp122_);
#line 367 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp124_ = _tmp123_;
#line 367 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp125_ = _tmp124_;
#line 367 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp126_ = gxml_dom_named_node_map_get_named_item (_tmp125_, "id");
#line 367 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp127_ = _tmp126_;
#line 367 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp128_ = gxml_dom_node_get_node_name (_tmp127_);
#line 367 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp129_ = _tmp128_;
#line 367 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp130_ = _tmp129_;
#line 367 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp130_, "id") == 0, "n.attributes.get_named_item (\"id\").node_name == \"id\"");
#line 367 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp130_);
#line 367 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp127_);
#line 367 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp125_);
#line 368 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp131_ = n;
#line 368 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp132_ = gxml_dom_element_get_attributes (_tmp131_);
#line 368 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp133_ = _tmp132_;
#line 368 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp134_ = _tmp133_;
#line 368 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp135_ = gxml_dom_named_node_map_get_named_item (_tmp134_, "id");
#line 368 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp136_ = _tmp135_;
#line 368 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp137_ = gxml_dom_node_get_node_value (_tmp136_);
#line 368 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp138_ = _tmp137_;
#line 368 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp139_ = _tmp138_;
#line 368 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp139_, "0y1") == 0, "n.attributes.get_named_item (\"id\").node_value == \"0y1\"");
#line 368 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp139_);
#line 368 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp136_);
#line 368 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp134_);
#line 369 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp140_ = n;
#line 369 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp141_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp140_);
#line 369 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp142_ = _tmp141_;
#line 369 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp143_ = _tmp142_;
#line 369 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp143_, "code") == 0, "n.node_name == \"code\"");
#line 369 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp143_);
#line 370 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp144_ = n;
#line 370 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp145_ = gxml_dom_node_lookup_namespace_uri (G_TYPE_CHECK_INSTANCE_TYPE (_tmp144_, GXML_TYPE_DOM_NODE) ? ((GXmlDomNode*) _tmp144_) : NULL, "gxml");
#line 370 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp146_ = _tmp145_;
#line 370 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp146_, "http://live.gnome.org/GXml") == 0, "(n as DomNode).lookup_namespace_uri (\"gxml\") == \"http://live.gnome.org/GXml\"");
#line 370 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp146_);
#line 371 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp147_ = n;
#line 371 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp148_ = gxml_dom_node_lookup_prefix (G_TYPE_CHECK_INSTANCE_TYPE (_tmp147_, GXML_TYPE_DOM_NODE) ? ((GXmlDomNode*) _tmp147_) : NULL, "http://live.gnome.org/GXml");
#line 371 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp149_ = _tmp148_;
#line 371 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp149_, "gxml") == 0, "(n as DomNode).lookup_prefix (\"http://live.gnome.org/GXml\") == \"gxml\"");
#line 371 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp149_);
#line 372 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp150_ = n;
#line 372 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp151_ = gxml_dom_element_get_tag_name (_tmp150_);
#line 372 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp152_ = _tmp151_;
#line 372 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp153_ = _tmp152_;
#line 372 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp153_, "gxml:code") == 0, "n.tag_name == \"gxml:code\"");
#line 372 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp153_);
#line 373 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp154_ = n;
#line 373 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		gxml_dom_element_remove_attribute (_tmp154_, "id");
#line 374 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp155_ = n;
#line 374 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp156_ = gxml_dom_element_get_attribute (_tmp155_, "id");
#line 374 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp157_ = _tmp156_;
#line 374 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp157_ == NULL, "n.get_attribute (\"id\") == null");
#line 374 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp157_);
#line 375 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp158_ = n;
#line 375 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp159_ = gxml_dom_element_has_attribute (_tmp158_, "id");
#line 375 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (!_tmp159_, "!n.has_attribute (\"id\")");
#line 376 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp160_ = doc;
#line 376 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp161_ = gxml_dom_document_create_element (_tmp160_, "p", &_inner_error_);
#line 376 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		n2 = _tmp161_;
#line 376 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 376 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 376 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n1);
#line 376 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 376 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 6124 "DomGDocumentTest.c"
			goto __catch184_g_error;
		}
#line 377 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp162_ = doc;
#line 377 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp163_ = gxml_dom_document_get_document_element (_tmp162_);
#line 377 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp164_ = _tmp163_;
#line 377 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp165_ = _tmp164_;
#line 377 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp166_ = n2;
#line 377 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp167_ = gxml_dom_node_append_child ((GXmlDomNode*) _tmp165_, (GXmlDomNode*) _tmp166_, &_inner_error_);
#line 377 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp168_ = _tmp167_;
#line 377 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp168_);
#line 377 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp165_);
#line 377 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 377 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n2);
#line 377 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 377 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n1);
#line 377 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 377 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 6157 "DomGDocumentTest.c"
			goto __catch184_g_error;
		}
#line 378 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp169_ = doc;
#line 378 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp170_ = gxml_dom_document_get_document_element (_tmp169_);
#line 378 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp171_ = _tmp170_;
#line 378 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp172_ = _tmp171_;
#line 378 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp173_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp172_);
#line 378 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp174_ = _tmp173_;
#line 378 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp175_ = _tmp174_;
#line 378 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp176_ = gxml_dom_html_collection_get_length (_tmp175_);
#line 378 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp177_ = _tmp176_;
#line 378 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp177_ == 3, "doc.document_element.children.length == 3");
#line 378 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp175_);
#line 378 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp172_);
#line 382 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp178_ = n2;
#line 382 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp179_ = gxml_dom_element_get_attributes (_tmp178_);
#line 382 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp180_ = _tmp179_;
#line 382 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp181_ = _tmp180_;
#line 382 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp182_ = gxml_dom_named_node_map_get_length (_tmp181_);
#line 382 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp183_ = _tmp182_;
#line 382 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp183_ == 0, "n2.attributes.length == 0");
#line 382 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp181_);
#line 386 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp184_ = n2;
#line 386 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		gxml_dom_element_set_attribute_ns (_tmp184_, "http://devel.org/", "dev:nice", "good", &_inner_error_);
#line 386 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 386 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n2);
#line 386 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 386 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n1);
#line 386 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 386 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 6216 "DomGDocumentTest.c"
			goto __catch184_g_error;
		}
#line 387 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp185_ = n2;
#line 387 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp186_ = gxml_dom_element_get_attributes (_tmp185_);
#line 387 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp187_ = _tmp186_;
#line 387 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp188_ = _tmp187_;
#line 387 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp189_ = gxml_dom_named_node_map_get_length (_tmp188_);
#line 387 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp190_ = _tmp189_;
#line 387 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp190_ == 1, "n2.attributes.length == 1");
#line 387 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp188_);
#line 388 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp191_ = n2;
#line 388 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp192_ = gxml_dom_element_has_attribute_ns (_tmp191_, "http://devel.org/", "nice");
#line 388 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp192_, "n2.has_attribute_ns (\"http://devel.org/\",\"nice\")");
#line 389 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp193_ = n2;
#line 389 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp194_ = gxml_dom_element_has_attribute_ns (_tmp193_, "http://devel.org/", "dev:nice");
#line 389 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (!_tmp194_, "!n2.has_attribute_ns (\"http://devel.org/\",\"dev:nice\")");
#line 393 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp195_ = n2;
#line 393 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp196_ = gxml_dom_element_get_attribute_ns (_tmp195_, "http://devel.org/", "nice");
#line 393 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp197_ = _tmp196_;
#line 393 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp197_, "good") == 0, "n2.get_attribute_ns (\"http://devel.org/\",\"nice\") == \"good\"");
#line 393 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp197_);
#line 394 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp198_ = n2;
#line 394 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp199_ = gxml_dom_element_get_attribute_ns (_tmp198_, "http://devel.org/", "dev:nice");
#line 394 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp200_ = _tmp199_;
#line 394 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp200_ == NULL, "n2.get_attribute_ns (\"http://devel.org/\",\"dev:nice\") == null");
#line 394 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp200_);
#line 339 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (n2);
#line 339 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (n);
#line 339 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (n1);
#line 339 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (doc);
#line 339 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp0_);
#line 6277 "DomGDocumentTest.c"
	}
	goto __finally184;
	__catch184_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp201_ = NULL;
		gchar* _tmp202_ = NULL;
		gchar* _tmp203_ = NULL;
#line 339 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		e = _inner_error_;
#line 339 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_inner_error_ = NULL;
#line 396 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp201_ = e->message;
#line 396 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp202_ = g_strconcat ("Error: ", _tmp201_, NULL);
#line 396 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp203_ = _tmp202_;
#line 396 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_message ("DomGDocumentTest.vala:396: %s", _tmp203_);
#line 396 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp203_);
#line 397 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_assert_not_reached ();
#line 339 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_error_free0 (e);
#line 6304 "DomGDocumentTest.c"
	}
	__finally184:
#line 339 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 339 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 339 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 339 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		return;
#line 6315 "DomGDocumentTest.c"
	}
}


static void ___lambda198__gtest_func (void) {
#line 338 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	__lambda198_ ();
#line 6323 "DomGDocumentTest.c"
}


static void __lambda199_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* _tmp0_ = NULL;
		GXmlGDocument* _tmp1_ = NULL;
		GXmlDomDocument* doc = NULL;
		GXmlGDocument* _tmp2_ = NULL;
		GXmlDomDocument* _tmp3_ = NULL;
		GXmlDomDocument* _tmp4_ = NULL;
		const gchar* _tmp5_ = NULL;
		const gchar* _tmp6_ = NULL;
		GXmlDomDocument* _tmp7_ = NULL;
		const gchar* _tmp8_ = NULL;
		const gchar* _tmp9_ = NULL;
		GXmlDomDocument* _tmp10_ = NULL;
		const gchar* _tmp11_ = NULL;
		const gchar* _tmp12_ = NULL;
		GXmlDomDocument* _tmp13_ = NULL;
		const gchar* _tmp14_ = NULL;
		const gchar* _tmp15_ = NULL;
		GXmlDomDocument* _tmp16_ = NULL;
		const gchar* _tmp17_ = NULL;
		const gchar* _tmp18_ = NULL;
		GXmlDomDocument* _tmp19_ = NULL;
		const gchar* _tmp20_ = NULL;
		const gchar* _tmp21_ = NULL;
		GXmlDomDocument* _tmp22_ = NULL;
		GXmlDomDocumentType* _tmp23_ = NULL;
		GXmlDomDocumentType* _tmp24_ = NULL;
		GXmlDomDocumentType* _tmp25_ = NULL;
		GXmlDomDocument* _tmp26_ = NULL;
		GXmlDomElement* _tmp27_ = NULL;
		GXmlDomElement* _tmp28_ = NULL;
		GXmlDomElement* _tmp29_ = NULL;
		GXmlDomDocument* _tmp30_ = NULL;
		GXmlDomElement* _tmp31_ = NULL;
		GXmlDomElement* _tmp32_ = NULL;
		GXmlDomElement* _tmp33_ = NULL;
		GXmlDomDocument* _tmp34_ = NULL;
		GXmlDomElement* _tmp35_ = NULL;
		GXmlDomElement* _tmp36_ = NULL;
		GXmlDomElement* _tmp37_ = NULL;
		gchar* _tmp38_ = NULL;
		gchar* _tmp39_ = NULL;
		gchar* _tmp40_ = NULL;
		GXmlDomHTMLCollection* le = NULL;
		GXmlDomDocument* _tmp41_ = NULL;
		GXmlDomHTMLCollection* _tmp42_ = NULL;
		GXmlDomHTMLCollection* _tmp43_ = NULL;
		gint _tmp44_ = 0;
		gint _tmp45_ = 0;
		GXmlDomHTMLCollection* _tmp46_ = NULL;
		GXmlDomElement* _tmp47_ = NULL;
		GXmlDomElement* _tmp48_ = NULL;
		GXmlDomHTMLCollection* _tmp49_ = NULL;
		GXmlDomElement* _tmp50_ = NULL;
		GXmlDomElement* _tmp51_ = NULL;
		gchar* _tmp52_ = NULL;
		gchar* _tmp53_ = NULL;
		gchar* _tmp54_ = NULL;
		GXmlDomElement* n = NULL;
		GXmlDomDocument* _tmp55_ = NULL;
		GXmlDomElement* _tmp56_ = NULL;
		GXmlDomNode* _tmp57_ = NULL;
		GXmlDomDocument* _tmp58_ = NULL;
		GXmlDomElement* _tmp59_ = NULL;
		GXmlDomElement* _tmp60_ = NULL;
		GXmlDomElement* _tmp61_ = NULL;
		GXmlDomElement* _tmp62_ = NULL;
		GXmlDomNode* _tmp63_ = NULL;
		GXmlDomNode* _tmp64_ = NULL;
		GXmlDomElement* n2 = NULL;
		GXmlDomNode* _tmp65_ = NULL;
		GXmlDomElement* _tmp66_ = NULL;
		GXmlDomElement* _tmp67_ = NULL;
		GXmlDomHTMLCollection* lens = NULL;
		GXmlDomDocument* _tmp68_ = NULL;
		GXmlDomHTMLCollection* _tmp69_ = NULL;
		GXmlDomHTMLCollection* _tmp70_ = NULL;
		gint _tmp71_ = 0;
		gint _tmp72_ = 0;
		GXmlDomHTMLCollection* _tmp73_ = NULL;
		GXmlDomElement* _tmp74_ = NULL;
		GXmlDomElement* _tmp75_ = NULL;
		GXmlDomHTMLCollection* _tmp76_ = NULL;
		GXmlDomElement* _tmp77_ = NULL;
		GXmlDomElement* _tmp78_ = NULL;
		gchar* _tmp79_ = NULL;
		gchar* _tmp80_ = NULL;
		gchar* _tmp81_ = NULL;
		GXmlDomHTMLCollection* lec = NULL;
		GXmlDomDocument* _tmp82_ = NULL;
		GXmlDomHTMLCollection* _tmp83_ = NULL;
		GXmlDomHTMLCollection* _tmp84_ = NULL;
		gint _tmp85_ = 0;
		gint _tmp86_ = 0;
		GXmlDomHTMLCollection* _tmp87_ = NULL;
		GXmlDomElement* _tmp88_ = NULL;
		GXmlDomElement* _tmp89_ = NULL;
		GXmlDomHTMLCollection* _tmp90_ = NULL;
		GXmlDomElement* _tmp91_ = NULL;
		GXmlDomElement* _tmp92_ = NULL;
		gchar* _tmp93_ = NULL;
		gchar* _tmp94_ = NULL;
		gchar* _tmp95_ = NULL;
		GXmlDomHTMLCollection* _tmp96_ = NULL;
		GXmlDomElement* _tmp97_ = NULL;
		GXmlDomElement* _tmp98_ = NULL;
		GXmlDomHTMLCollection* _tmp99_ = NULL;
		GXmlDomElement* _tmp100_ = NULL;
		GXmlDomElement* _tmp101_ = NULL;
		gchar* _tmp102_ = NULL;
		gchar* _tmp103_ = NULL;
		gchar* _tmp104_ = NULL;
		GXmlDomElement* _tmp105_ = NULL;
		GXmlDomHTMLCollection* lec2 = NULL;
		GXmlDomDocument* _tmp106_ = NULL;
		GXmlDomHTMLCollection* _tmp107_ = NULL;
		GXmlDomHTMLCollection* _tmp108_ = NULL;
		gint _tmp109_ = 0;
		gint _tmp110_ = 0;
		GXmlDomHTMLCollection* _tmp111_ = NULL;
		GXmlDomElement* _tmp112_ = NULL;
		GXmlDomElement* _tmp113_ = NULL;
		GXmlDomHTMLCollection* lec3 = NULL;
		GXmlDomDocument* _tmp114_ = NULL;
		GXmlDomHTMLCollection* _tmp115_ = NULL;
		GXmlDomHTMLCollection* _tmp116_ = NULL;
		gint _tmp117_ = 0;
		gint _tmp118_ = 0;
		GXmlDomHTMLCollection* lec4 = NULL;
		GXmlDomDocument* _tmp119_ = NULL;
		GXmlDomHTMLCollection* _tmp120_ = NULL;
		GXmlDomHTMLCollection* _tmp121_ = NULL;
		gint _tmp122_ = 0;
		gint _tmp123_ = 0;
		GXmlDomHTMLCollection* lec5 = NULL;
		GXmlDomDocument* _tmp124_ = NULL;
		GXmlDomHTMLCollection* _tmp125_ = NULL;
		GXmlDomHTMLCollection* _tmp126_ = NULL;
		gint _tmp127_ = 0;
		gint _tmp128_ = 0;
		GXmlDomHTMLCollection* _tmp129_ = NULL;
		GXmlDomElement* _tmp130_ = NULL;
		GXmlDomElement* _tmp131_ = NULL;
		GXmlDomHTMLCollection* _tmp132_ = NULL;
		GXmlDomElement* _tmp133_ = NULL;
		GXmlDomElement* _tmp134_ = NULL;
		gchar* _tmp135_ = NULL;
		gchar* _tmp136_ = NULL;
		gchar* _tmp137_ = NULL;
		GXmlDomHTMLCollection* _tmp138_ = NULL;
		GXmlDomElement* _tmp139_ = NULL;
		GXmlDomElement* _tmp140_ = NULL;
		GXmlDomHTMLCollection* _tmp141_ = NULL;
		GXmlDomElement* _tmp142_ = NULL;
		GXmlDomElement* _tmp143_ = NULL;
		gchar* _tmp144_ = NULL;
		gchar* _tmp145_ = NULL;
		gchar* _tmp146_ = NULL;
		GXmlDomHTMLCollection* _tmp147_ = NULL;
		GXmlDomElement* _tmp148_ = NULL;
		GXmlDomElement* _tmp149_ = NULL;
		GXmlDomHTMLCollection* _tmp150_ = NULL;
		GXmlDomElement* _tmp151_ = NULL;
		GXmlDomElement* _tmp152_ = NULL;
		gchar* _tmp153_ = NULL;
		gchar* _tmp154_ = NULL;
		gchar* _tmp155_ = NULL;
		GXmlDomText* t = NULL;
		GXmlDomDocument* _tmp156_ = NULL;
		GXmlDomText* _tmp157_ = NULL;
		GXmlDomElement* _tmp158_ = NULL;
		GXmlDomText* _tmp159_ = NULL;
		GXmlDomNode* _tmp160_ = NULL;
		GXmlDomNode* _tmp161_ = NULL;
		GXmlDomElement* _tmp162_ = NULL;
		GXmlDomNodeList* _tmp163_ = NULL;
		GXmlDomNodeList* _tmp164_ = NULL;
		GXmlDomNodeList* _tmp165_ = NULL;
		gpointer _tmp166_ = NULL;
		GXmlDomNode* _tmp167_ = NULL;
		GXmlDomElement* _tmp168_ = NULL;
		GXmlDomNodeList* _tmp169_ = NULL;
		GXmlDomNodeList* _tmp170_ = NULL;
		GXmlDomNodeList* _tmp171_ = NULL;
		gpointer _tmp172_ = NULL;
		GXmlDomNode* _tmp173_ = NULL;
		gchar* _tmp174_ = NULL;
		gchar* _tmp175_ = NULL;
		gchar* _tmp176_ = NULL;
		GXmlDomComment* comment = NULL;
		GXmlDomDocument* _tmp177_ = NULL;
		GXmlDomComment* _tmp178_ = NULL;
		GXmlDomDocument* _tmp179_ = NULL;
		GXmlDomElement* _tmp180_ = NULL;
		GXmlDomElement* _tmp181_ = NULL;
		GXmlDomElement* _tmp182_ = NULL;
		GXmlDomComment* _tmp183_ = NULL;
		GXmlDomNode* _tmp184_ = NULL;
		GXmlDomNode* _tmp185_ = NULL;
		GXmlDomDocument* _tmp186_ = NULL;
		GXmlDomElement* _tmp187_ = NULL;
		GXmlDomElement* _tmp188_ = NULL;
		GXmlDomElement* _tmp189_ = NULL;
		GXmlDomNode* _tmp190_ = NULL;
		GXmlDomNode* _tmp191_ = NULL;
		GXmlDomNode* _tmp192_ = NULL;
		GXmlDomDocument* _tmp193_ = NULL;
		GXmlDomElement* _tmp194_ = NULL;
		GXmlDomElement* _tmp195_ = NULL;
		GXmlDomElement* _tmp196_ = NULL;
		GXmlDomNode* _tmp197_ = NULL;
		GXmlDomNode* _tmp198_ = NULL;
		GXmlDomNode* _tmp199_ = NULL;
		gchar* _tmp200_ = NULL;
		gchar* _tmp201_ = NULL;
		gchar* _tmp202_ = NULL;
		GXmlDomProcessingInstruction* pi = NULL;
		GXmlDomDocument* _tmp203_ = NULL;
		GXmlDomProcessingInstruction* _tmp204_ = NULL;
		GXmlDomDocument* _tmp205_ = NULL;
		GXmlDomElement* _tmp206_ = NULL;
		GXmlDomElement* _tmp207_ = NULL;
		GXmlDomElement* _tmp208_ = NULL;
		GXmlDomProcessingInstruction* _tmp209_ = NULL;
		GXmlDomNode* _tmp210_ = NULL;
		GXmlDomNode* _tmp211_ = NULL;
		GXmlDomDocument* _tmp212_ = NULL;
		GXmlDomElement* _tmp213_ = NULL;
		GXmlDomElement* _tmp214_ = NULL;
		GXmlDomElement* _tmp215_ = NULL;
		GXmlDomNode* _tmp216_ = NULL;
		GXmlDomNode* _tmp217_ = NULL;
		GXmlDomNode* _tmp218_ = NULL;
#line 405 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp1_ = gxml_gdocument_new_from_string (DOM_GDOCUMENT_TEST_XMLDOC, 0, &_inner_error_);
#line 405 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp0_ = _tmp1_;
#line 405 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 6568 "DomGDocumentTest.c"
			goto __catch185_g_error;
		}
#line 405 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp2_ = _tmp0_;
#line 405 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp0_ = NULL;
#line 405 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp3_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp2_, GXML_TYPE_DOM_DOCUMENT) ? ((GXmlDomDocument*) _tmp2_) : NULL;
#line 405 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp3_ == NULL) {
#line 405 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 6581 "DomGDocumentTest.c"
		}
#line 405 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		doc = _tmp3_;
#line 406 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp4_ = doc;
#line 406 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp5_ = gxml_dom_document_get_url (_tmp4_);
#line 406 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp6_ = _tmp5_;
#line 406 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp6_, "about:blank") == 0, "doc.url == \"about:blank\"");
#line 407 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp7_ = doc;
#line 407 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp8_ = gxml_dom_document_get_document_uri (_tmp7_);
#line 407 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp9_ = _tmp8_;
#line 407 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp9_, "about:blank") == 0, "doc.document_uri == \"about:blank\"");
#line 408 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp10_ = doc;
#line 408 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp11_ = gxml_dom_document_get_origin (_tmp10_);
#line 408 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp12_ = _tmp11_;
#line 408 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp12_, "") == 0, "doc.origin == \"\"");
#line 409 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp13_ = doc;
#line 409 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp14_ = gxml_dom_document_get_compat_mode (_tmp13_);
#line 409 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp15_ = _tmp14_;
#line 409 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp15_, "") == 0, "doc.compat_mode == \"\"");
#line 410 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp16_ = doc;
#line 410 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp17_ = gxml_dom_document_get_character_set (_tmp16_);
#line 410 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp18_ = _tmp17_;
#line 410 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp18_, "utf-8") == 0, "doc.character_set == \"utf-8\"");
#line 411 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp19_ = doc;
#line 411 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp20_ = gxml_dom_document_get_content_type (_tmp19_);
#line 411 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp21_ = _tmp20_;
#line 411 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp21_, "application/xml") == 0, "doc.content_type == \"application/xml\"");
#line 412 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp22_ = doc;
#line 412 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp23_ = gxml_dom_document_get_doctype (_tmp22_);
#line 412 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp24_ = _tmp23_;
#line 412 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp25_ = _tmp24_;
#line 412 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp25_ == NULL, "doc.doctype == null");
#line 412 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp25_);
#line 413 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp26_ = doc;
#line 413 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp27_ = gxml_dom_document_get_document_element (_tmp26_);
#line 413 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp28_ = _tmp27_;
#line 413 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp29_ = _tmp28_;
#line 413 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp29_ != NULL, "doc.document_element != null");
#line 413 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp29_);
#line 414 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp30_ = doc;
#line 414 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp31_ = gxml_dom_document_get_document_element (_tmp30_);
#line 414 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp32_ = _tmp31_;
#line 414 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp33_ = _tmp32_;
#line 414 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp33_, GXML_TYPE_DOM_ELEMENT), "doc.document_element is DomElement");
#line 414 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp33_);
#line 415 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp34_ = doc;
#line 415 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp35_ = gxml_dom_document_get_document_element (_tmp34_);
#line 415 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp36_ = _tmp35_;
#line 415 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp37_ = _tmp36_;
#line 415 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp38_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp37_);
#line 415 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp39_ = _tmp38_;
#line 415 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp40_ = _tmp39_;
#line 415 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp40_, "root") == 0, "doc.document_element.node_name == \"root\"");
#line 415 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp40_);
#line 415 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp37_);
#line 416 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp41_ = doc;
#line 416 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp42_ = gxml_dom_document_get_elements_by_tag_name (_tmp41_, "code");
#line 416 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		le = _tmp42_;
#line 417 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp43_ = le;
#line 417 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp44_ = gxml_dom_html_collection_get_length (_tmp43_);
#line 417 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp45_ = _tmp44_;
#line 417 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp45_ == 2, "le.length == 2");
#line 418 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp46_ = le;
#line 418 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp47_ = gxml_dom_html_collection_item (_tmp46_, 0);
#line 418 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp48_ = _tmp47_;
#line 418 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp48_, GXML_TYPE_DOM_ELEMENT), "le.item (0) is DomElement");
#line 418 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp48_);
#line 419 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp49_ = le;
#line 419 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp50_ = gxml_dom_html_collection_item (_tmp49_, 0);
#line 419 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp51_ = _tmp50_;
#line 419 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp52_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp51_);
#line 419 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp53_ = _tmp52_;
#line 419 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp54_ = _tmp53_;
#line 419 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp54_, "code") == 0, "le.item (0).node_name == \"code\"");
#line 419 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp54_);
#line 419 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp51_);
#line 420 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp55_ = doc;
#line 420 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp56_ = gxml_dom_document_create_element_ns (_tmp55_, "http://git.gnome.org/browse/gxml", "git:MyNode", &_inner_error_);
#line 420 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		n = _tmp56_;
#line 420 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 420 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (le);
#line 420 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 420 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 6745 "DomGDocumentTest.c"
			goto __catch185_g_error;
		}
#line 421 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp58_ = doc;
#line 421 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp59_ = gxml_dom_document_get_document_element (_tmp58_);
#line 421 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp60_ = _tmp59_;
#line 421 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp61_ = _tmp60_;
#line 421 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp62_ = n;
#line 421 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp63_ = gxml_dom_node_append_child ((GXmlDomNode*) _tmp61_, (GXmlDomNode*) _tmp62_, &_inner_error_);
#line 421 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp64_ = _tmp63_;
#line 421 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp61_);
#line 421 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp57_ = _tmp64_;
#line 421 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 421 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 421 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (le);
#line 421 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 421 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 6776 "DomGDocumentTest.c"
			goto __catch185_g_error;
		}
#line 421 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp65_ = _tmp57_;
#line 421 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp57_ = NULL;
#line 421 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp66_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp65_, GXML_TYPE_DOM_ELEMENT) ? ((GXmlDomElement*) _tmp65_) : NULL;
#line 421 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp66_ == NULL) {
#line 421 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp65_);
#line 6789 "DomGDocumentTest.c"
		}
#line 421 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		n2 = _tmp66_;
#line 422 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp67_ = n2;
#line 422 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		gxml_dom_element_set_attribute (_tmp67_, "class", "node", &_inner_error_);
#line 422 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 422 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n2);
#line 422 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp57_);
#line 422 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 422 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (le);
#line 422 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 422 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 6811 "DomGDocumentTest.c"
			goto __catch185_g_error;
		}
#line 423 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp68_ = doc;
#line 423 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp69_ = gxml_dom_document_get_elements_by_tag_name_ns (_tmp68_, "http://git.gnome.org/browse/gxml", "MyNode");
#line 423 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		lens = _tmp69_;
#line 424 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp70_ = lens;
#line 424 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp71_ = gxml_dom_html_collection_get_length (_tmp70_);
#line 424 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp72_ = _tmp71_;
#line 424 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp72_ == 1, "lens.length == 1");
#line 425 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp73_ = lens;
#line 425 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp74_ = gxml_dom_html_collection_item (_tmp73_, 0);
#line 425 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp75_ = _tmp74_;
#line 425 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp75_, GXML_TYPE_DOM_ELEMENT), "lens.item (0) is DomElement");
#line 425 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp75_);
#line 426 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp76_ = lens;
#line 426 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp77_ = gxml_dom_html_collection_item (_tmp76_, 0);
#line 426 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp78_ = _tmp77_;
#line 426 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp79_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp78_);
#line 426 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp80_ = _tmp79_;
#line 426 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp81_ = _tmp80_;
#line 426 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp81_, "MyNode") == 0, "lens.item (0).node_name == \"MyNode\"");
#line 426 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp81_);
#line 426 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp78_);
#line 430 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp82_ = doc;
#line 430 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp83_ = gxml_dom_document_get_elements_by_class_name (_tmp82_, "node");
#line 430 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		lec = _tmp83_;
#line 431 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp84_ = lec;
#line 431 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp85_ = gxml_dom_html_collection_get_length (_tmp84_);
#line 431 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp86_ = _tmp85_;
#line 431 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp86_ == 4, "lec.length == 4");
#line 432 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp87_ = lec;
#line 432 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp88_ = gxml_dom_html_collection_item (_tmp87_, 0);
#line 432 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp89_ = _tmp88_;
#line 432 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp89_, GXML_TYPE_DOM_ELEMENT), "lec.item (0) is DomElement");
#line 432 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp89_);
#line 433 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp90_ = lec;
#line 433 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp91_ = gxml_dom_html_collection_item (_tmp90_, 0);
#line 433 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp92_ = _tmp91_;
#line 433 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp93_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp92_);
#line 433 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp94_ = _tmp93_;
#line 433 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp95_ = _tmp94_;
#line 433 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp95_, "MyNode") == 0, "lec.item (0).node_name == \"MyNode\"");
#line 433 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp95_);
#line 433 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp92_);
#line 434 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp96_ = lec;
#line 434 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp97_ = gxml_dom_html_collection_item (_tmp96_, 1);
#line 434 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp98_ = _tmp97_;
#line 434 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp98_, GXML_TYPE_DOM_ELEMENT), "lec.item (1) is DomElement");
#line 434 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp98_);
#line 435 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp99_ = lec;
#line 435 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp100_ = gxml_dom_html_collection_item (_tmp99_, 1);
#line 435 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp101_ = _tmp100_;
#line 435 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp102_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp101_);
#line 435 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp103_ = _tmp102_;
#line 435 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp104_ = _tmp103_;
#line 435 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp104_, "page") == 0, "lec.item (1).node_name == \"page\"");
#line 435 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp104_);
#line 435 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp101_);
#line 436 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp105_ = n;
#line 436 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		gxml_dom_element_set_attribute (_tmp105_, "class", "node parent", &_inner_error_);
#line 436 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 436 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec);
#line 436 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lens);
#line 436 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n2);
#line 436 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp57_);
#line 436 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 436 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (le);
#line 436 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 436 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 6948 "DomGDocumentTest.c"
			goto __catch185_g_error;
		}
#line 437 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp106_ = doc;
#line 437 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp107_ = gxml_dom_document_get_elements_by_class_name (_tmp106_, "parent");
#line 437 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		lec2 = _tmp107_;
#line 438 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp108_ = lec2;
#line 438 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp109_ = gxml_dom_html_collection_get_length (_tmp108_);
#line 438 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp110_ = _tmp109_;
#line 438 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp110_ == 4, "lec2.length == 4");
#line 439 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp111_ = lec2;
#line 439 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp112_ = gxml_dom_html_collection_item (_tmp111_, 0);
#line 439 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp113_ = _tmp112_;
#line 439 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp113_, GXML_TYPE_DOM_ELEMENT), "lec2.item (0) is DomElement");
#line 439 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp113_);
#line 440 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp114_ = doc;
#line 440 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp115_ = gxml_dom_document_get_elements_by_class_name (_tmp114_, "parent code");
#line 440 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		lec3 = _tmp115_;
#line 441 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp116_ = lec3;
#line 441 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp117_ = gxml_dom_html_collection_get_length (_tmp116_);
#line 441 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp118_ = _tmp117_;
#line 441 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp118_ == 0, "lec3.length == 0");
#line 442 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp119_ = doc;
#line 442 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp120_ = gxml_dom_document_get_elements_by_class_name (_tmp119_, "code parent");
#line 442 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		lec4 = _tmp120_;
#line 443 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp121_ = lec4;
#line 443 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp122_ = gxml_dom_html_collection_get_length (_tmp121_);
#line 443 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp123_ = _tmp122_;
#line 443 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp123_ == 0, "lec4.length == 0");
#line 444 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp124_ = doc;
#line 444 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp125_ = gxml_dom_document_get_elements_by_class_name (_tmp124_, "node parent");
#line 444 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		lec5 = _tmp125_;
#line 449 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp126_ = lec5;
#line 449 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp127_ = gxml_dom_html_collection_get_length (_tmp126_);
#line 449 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp128_ = _tmp127_;
#line 449 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp128_ == 3, "lec5.length == 3");
#line 450 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp129_ = lec5;
#line 450 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp130_ = gxml_dom_html_collection_item (_tmp129_, 0);
#line 450 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp131_ = _tmp130_;
#line 450 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp131_, GXML_TYPE_DOM_ELEMENT), "lec5.item (0) is DomElement");
#line 450 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp131_);
#line 451 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp132_ = lec5;
#line 451 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp133_ = gxml_dom_html_collection_item (_tmp132_, 0);
#line 451 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp134_ = _tmp133_;
#line 451 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp135_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp134_);
#line 451 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp136_ = _tmp135_;
#line 451 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp137_ = _tmp136_;
#line 451 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp137_, "MyNode") == 0, "lec5.item (0).node_name == \"MyNode\"");
#line 451 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp137_);
#line 451 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp134_);
#line 452 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp138_ = lec5;
#line 452 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp139_ = gxml_dom_html_collection_item (_tmp138_, 1);
#line 452 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp140_ = _tmp139_;
#line 452 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp140_, GXML_TYPE_DOM_ELEMENT), "lec5.item (1) is DomElement");
#line 452 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp140_);
#line 453 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp141_ = lec5;
#line 453 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp142_ = gxml_dom_html_collection_item (_tmp141_, 1);
#line 453 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp143_ = _tmp142_;
#line 453 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp144_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp143_);
#line 453 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp145_ = _tmp144_;
#line 453 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp146_ = _tmp145_;
#line 453 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp146_, "page") == 0, "lec5.item (1).node_name == \"page\"");
#line 453 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp146_);
#line 453 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp143_);
#line 454 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp147_ = lec5;
#line 454 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp148_ = gxml_dom_html_collection_item (_tmp147_, 2);
#line 454 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp149_ = _tmp148_;
#line 454 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp149_, GXML_TYPE_DOM_ELEMENT), "lec5.item (2) is DomElement");
#line 454 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp149_);
#line 455 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp150_ = lec5;
#line 455 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp151_ = gxml_dom_html_collection_item (_tmp150_, 2);
#line 455 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp152_ = _tmp151_;
#line 455 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp153_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp152_);
#line 455 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp154_ = _tmp153_;
#line 455 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp155_ = _tmp154_;
#line 455 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp155_, "code") == 0, "lec5.item (2).node_name == \"code\"");
#line 455 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp155_);
#line 455 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp152_);
#line 456 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp156_ = doc;
#line 456 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp157_ = gxml_dom_document_create_text_node (_tmp156_, "TEXT", &_inner_error_);
#line 456 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		t = _tmp157_;
#line 456 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 456 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec5);
#line 456 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec4);
#line 456 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec3);
#line 456 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec2);
#line 456 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec);
#line 456 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lens);
#line 456 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n2);
#line 456 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp57_);
#line 456 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 456 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (le);
#line 456 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 456 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 7133 "DomGDocumentTest.c"
			goto __catch185_g_error;
		}
#line 457 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp158_ = n;
#line 457 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp159_ = t;
#line 457 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp160_ = gxml_dom_node_append_child ((GXmlDomNode*) _tmp158_, (GXmlDomNode*) _tmp159_, &_inner_error_);
#line 457 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp161_ = _tmp160_;
#line 457 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp161_);
#line 457 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 457 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 457 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec5);
#line 457 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec4);
#line 457 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec3);
#line 457 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec2);
#line 457 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec);
#line 457 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lens);
#line 457 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n2);
#line 457 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp57_);
#line 457 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 457 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (le);
#line 457 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 457 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 7174 "DomGDocumentTest.c"
			goto __catch185_g_error;
		}
#line 458 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp162_ = n;
#line 458 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp163_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp162_);
#line 458 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp164_ = _tmp163_;
#line 458 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp165_ = _tmp164_;
#line 458 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp166_ = gee_list_get ((GeeList*) _tmp165_, 0);
#line 458 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp167_ = (GXmlDomNode*) _tmp166_;
#line 458 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp167_, GXML_TYPE_DOM_TEXT), "n.child_nodes[0] is DomText");
#line 458 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp167_);
#line 458 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp165_);
#line 459 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp168_ = n;
#line 459 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp169_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp168_);
#line 459 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp170_ = _tmp169_;
#line 459 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp171_ = _tmp170_;
#line 459 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp172_ = gee_list_get ((GeeList*) _tmp171_, 0);
#line 459 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp173_ = (GXmlDomNode*) _tmp172_;
#line 459 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp174_ = gxml_dom_node_get_node_value (_tmp173_);
#line 459 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp175_ = _tmp174_;
#line 459 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp176_ = _tmp175_;
#line 459 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp176_, "TEXT") == 0, "n.child_nodes[0].node_value == \"TEXT\"");
#line 459 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp176_);
#line 459 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp173_);
#line 459 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp171_);
#line 460 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp177_ = doc;
#line 460 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp178_ = gxml_dom_document_create_comment (_tmp177_, "COMMENT", &_inner_error_);
#line 460 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		comment = _tmp178_;
#line 460 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 460 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 460 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec5);
#line 460 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec4);
#line 460 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec3);
#line 460 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec2);
#line 460 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec);
#line 460 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lens);
#line 460 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n2);
#line 460 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp57_);
#line 460 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 460 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (le);
#line 460 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 460 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 7255 "DomGDocumentTest.c"
			goto __catch185_g_error;
		}
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp179_ = doc;
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp180_ = gxml_dom_document_get_document_element (_tmp179_);
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp181_ = _tmp180_;
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp182_ = _tmp181_;
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp183_ = comment;
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp184_ = gxml_dom_node_append_child ((GXmlDomNode*) _tmp182_, (GXmlDomNode*) _tmp183_, &_inner_error_);
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp185_ = _tmp184_;
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp185_);
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp182_);
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (comment);
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec5);
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec4);
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec3);
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec2);
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec);
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lens);
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n2);
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp57_);
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (le);
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 461 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 7306 "DomGDocumentTest.c"
			goto __catch185_g_error;
		}
#line 462 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp186_ = doc;
#line 462 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp187_ = gxml_dom_document_get_document_element (_tmp186_);
#line 462 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp188_ = _tmp187_;
#line 462 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp189_ = _tmp188_;
#line 462 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp190_ = gxml_dom_node_get_last_child ((GXmlDomNode*) _tmp189_);
#line 462 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp191_ = _tmp190_;
#line 462 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp192_ = _tmp191_;
#line 462 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp192_, GXML_TYPE_DOM_COMMENT), "doc.document_element.last_child is DomComment");
#line 462 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp192_);
#line 462 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp189_);
#line 463 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp193_ = doc;
#line 463 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp194_ = gxml_dom_document_get_document_element (_tmp193_);
#line 463 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp195_ = _tmp194_;
#line 463 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp196_ = _tmp195_;
#line 463 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp197_ = gxml_dom_node_get_last_child ((GXmlDomNode*) _tmp196_);
#line 463 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp198_ = _tmp197_;
#line 463 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp199_ = _tmp198_;
#line 463 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp200_ = gxml_dom_node_get_node_value (_tmp199_);
#line 463 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp201_ = _tmp200_;
#line 463 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp202_ = _tmp201_;
#line 463 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp202_, "COMMENT") == 0, "doc.document_element.last_child.node_value == \"COMMENT\"");
#line 463 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp202_);
#line 463 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp199_);
#line 463 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp196_);
#line 464 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp203_ = doc;
#line 464 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp204_ = gxml_dom_document_create_processing_instruction (_tmp203_, "git", "commit", &_inner_error_);
#line 464 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		pi = _tmp204_;
#line 464 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 464 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (comment);
#line 464 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 464 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec5);
#line 464 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec4);
#line 464 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec3);
#line 464 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec2);
#line 464 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec);
#line 464 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lens);
#line 464 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n2);
#line 464 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp57_);
#line 464 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 464 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (le);
#line 464 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 464 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 7393 "DomGDocumentTest.c"
			goto __catch185_g_error;
		}
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp205_ = doc;
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp206_ = gxml_dom_document_get_document_element (_tmp205_);
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp207_ = _tmp206_;
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp208_ = _tmp207_;
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp209_ = pi;
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp210_ = gxml_dom_node_append_child ((GXmlDomNode*) _tmp208_, (GXmlDomNode*) _tmp209_, &_inner_error_);
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp211_ = _tmp210_;
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp211_);
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp208_);
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (pi);
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (comment);
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec5);
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec4);
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec3);
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec2);
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lec);
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (lens);
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n2);
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp57_);
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (le);
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 465 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 7446 "DomGDocumentTest.c"
			goto __catch185_g_error;
		}
#line 469 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp212_ = doc;
#line 469 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp213_ = gxml_dom_document_get_document_element (_tmp212_);
#line 469 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp214_ = _tmp213_;
#line 469 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp215_ = _tmp214_;
#line 469 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp216_ = gxml_dom_node_get_last_child ((GXmlDomNode*) _tmp215_);
#line 469 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp217_ = _tmp216_;
#line 469 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp218_ = _tmp217_;
#line 469 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp218_, GXML_TYPE_DOM_PROCESSING_INSTRUCTION), "doc.document_element.last_child is DomProcessingInstruction");
#line 469 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp218_);
#line 469 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp215_);
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (pi);
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (comment);
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (t);
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (lec5);
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (lec4);
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (lec3);
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (lec2);
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (lec);
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (lens);
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (n2);
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp57_);
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (n);
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (le);
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (doc);
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp0_);
#line 7499 "DomGDocumentTest.c"
	}
	goto __finally185;
	__catch185_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp219_ = NULL;
		gchar* _tmp220_ = NULL;
		gchar* _tmp221_ = NULL;
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		e = _inner_error_;
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_inner_error_ = NULL;
#line 471 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp219_ = e->message;
#line 471 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp220_ = g_strconcat ("Error: ", _tmp219_, NULL);
#line 471 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp221_ = _tmp220_;
#line 471 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_message ("DomGDocumentTest.vala:471: %s", _tmp221_);
#line 471 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp221_);
#line 472 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_assert_not_reached ();
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_error_free0 (e);
#line 7526 "DomGDocumentTest.c"
	}
	__finally185:
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 401 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		return;
#line 7537 "DomGDocumentTest.c"
	}
}


static void ___lambda199__gtest_func (void) {
#line 400 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	__lambda199_ ();
#line 7545 "DomGDocumentTest.c"
}


static void __lambda200_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* _tmp0_ = NULL;
		GXmlGDocument* _tmp1_ = NULL;
		GXmlDomDocument* doc = NULL;
		GXmlGDocument* _tmp2_ = NULL;
		GXmlDomDocument* _tmp3_ = NULL;
		GXmlGDocument* _tmp4_ = NULL;
		GXmlGDocument* _tmp5_ = NULL;
		GXmlDomDocument* doc2 = NULL;
		GXmlGDocument* _tmp6_ = NULL;
		GXmlDomDocument* _tmp7_ = NULL;
		GXmlDomDocument* _tmp8_ = NULL;
		GXmlDomDocument* _tmp9_ = NULL;
		GXmlDomElement* _tmp10_ = NULL;
		GXmlDomElement* _tmp11_ = NULL;
		GXmlDomElement* _tmp12_ = NULL;
		GXmlDomNode* _tmp13_ = NULL;
		GXmlDomNode* _tmp14_ = NULL;
		GXmlDomDocument* _tmp15_ = NULL;
		GXmlDomElement* _tmp16_ = NULL;
		GXmlDomElement* _tmp17_ = NULL;
		GXmlDomElement* _tmp18_ = NULL;
		GXmlDomNode* _tmp19_ = NULL;
		GXmlDomNode* _tmp20_ = NULL;
		GXmlDomNode* _tmp21_ = NULL;
		GXmlDomDocument* _tmp22_ = NULL;
		GXmlDomElement* _tmp23_ = NULL;
		GXmlDomElement* _tmp24_ = NULL;
		GXmlDomElement* _tmp25_ = NULL;
		GXmlDomNode* _tmp26_ = NULL;
		GXmlDomNode* _tmp27_ = NULL;
		GXmlDomNode* _tmp28_ = NULL;
		gchar* _tmp29_ = NULL;
		gchar* _tmp30_ = NULL;
		gchar* _tmp31_ = NULL;
		GXmlDomDocument* _tmp32_ = NULL;
		GXmlDomElement* _tmp33_ = NULL;
		GXmlDomElement* _tmp34_ = NULL;
		GXmlDomElement* _tmp35_ = NULL;
		GXmlDomNode* _tmp36_ = NULL;
		GXmlDomNode* _tmp37_ = NULL;
		GXmlDomNode* _tmp38_ = NULL;
		GXmlDomNodeList* _tmp39_ = NULL;
		GXmlDomNodeList* _tmp40_ = NULL;
		GXmlDomNodeList* _tmp41_ = NULL;
		gint _tmp42_ = 0;
		gint _tmp43_ = 0;
#line 480 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp1_ = gxml_gdocument_new_from_string (DOM_GDOCUMENT_TEST_XMLDOC, 0, &_inner_error_);
#line 480 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp0_ = _tmp1_;
#line 480 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 7604 "DomGDocumentTest.c"
			goto __catch186_g_error;
		}
#line 480 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp2_ = _tmp0_;
#line 480 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp0_ = NULL;
#line 480 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp3_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp2_, GXML_TYPE_DOM_DOCUMENT) ? ((GXmlDomDocument*) _tmp2_) : NULL;
#line 480 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp3_ == NULL) {
#line 480 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 7617 "DomGDocumentTest.c"
		}
#line 480 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		doc = _tmp3_;
#line 481 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp5_ = gxml_gdocument_new_from_string (DOM_GDOCUMENT_TEST_STRDOC, 0, &_inner_error_);
#line 481 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp4_ = _tmp5_;
#line 481 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 481 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 481 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 7631 "DomGDocumentTest.c"
			goto __catch186_g_error;
		}
#line 481 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp6_ = _tmp4_;
#line 481 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp4_ = NULL;
#line 481 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp7_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp6_, GXML_TYPE_DOM_DOCUMENT) ? ((GXmlDomDocument*) _tmp6_) : NULL;
#line 481 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp7_ == NULL) {
#line 481 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp6_);
#line 7644 "DomGDocumentTest.c"
		}
#line 481 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		doc2 = _tmp7_;
#line 482 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp8_ = doc;
#line 482 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp9_ = doc2;
#line 482 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp10_ = gxml_dom_document_get_document_element (_tmp9_);
#line 482 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp11_ = _tmp10_;
#line 482 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp12_ = _tmp11_;
#line 482 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp13_ = gxml_dom_document_import_node (_tmp8_, (GXmlDomNode*) _tmp12_, FALSE, &_inner_error_);
#line 482 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp14_ = _tmp13_;
#line 482 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp14_);
#line 482 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp12_);
#line 482 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 482 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc2);
#line 482 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp4_);
#line 482 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 482 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 7676 "DomGDocumentTest.c"
			goto __catch186_g_error;
		}
#line 486 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp15_ = doc;
#line 486 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp16_ = gxml_dom_document_get_document_element (_tmp15_);
#line 486 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp17_ = _tmp16_;
#line 486 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp18_ = _tmp17_;
#line 486 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp19_ = gxml_dom_node_get_last_child ((GXmlDomNode*) _tmp18_);
#line 486 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp20_ = _tmp19_;
#line 486 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp21_ = _tmp20_;
#line 486 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp21_, GXML_TYPE_DOM_ELEMENT), "doc.document_element.last_child is DomElement");
#line 486 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp21_);
#line 486 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp18_);
#line 487 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp22_ = doc;
#line 487 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp23_ = gxml_dom_document_get_document_element (_tmp22_);
#line 487 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp24_ = _tmp23_;
#line 487 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp25_ = _tmp24_;
#line 487 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp26_ = gxml_dom_node_get_last_child ((GXmlDomNode*) _tmp25_);
#line 487 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp27_ = _tmp26_;
#line 487 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp28_ = _tmp27_;
#line 487 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp29_ = gxml_dom_node_get_node_name (_tmp28_);
#line 487 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp30_ = _tmp29_;
#line 487 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp31_ = _tmp30_;
#line 487 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp31_, "Sentences") == 0, "doc.document_element.last_child.node_name == \"Sentences\"");
#line 487 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp31_);
#line 487 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp28_);
#line 487 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp25_);
#line 488 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp32_ = doc;
#line 488 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp33_ = gxml_dom_document_get_document_element (_tmp32_);
#line 488 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp34_ = _tmp33_;
#line 488 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp35_ = _tmp34_;
#line 488 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp36_ = gxml_dom_node_get_last_child ((GXmlDomNode*) _tmp35_);
#line 488 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp37_ = _tmp36_;
#line 488 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp38_ = _tmp37_;
#line 488 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp39_ = gxml_dom_node_get_child_nodes (_tmp38_);
#line 488 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp40_ = _tmp39_;
#line 488 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp41_ = _tmp40_;
#line 488 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp42_ = gxml_dom_node_list_get_length (_tmp41_);
#line 488 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp43_ = _tmp42_;
#line 488 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp43_ == 0, "doc.document_element.last_child.child_nodes.length == 0");
#line 488 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp41_);
#line 488 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp38_);
#line 488 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp35_);
#line 476 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (doc2);
#line 476 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp4_);
#line 476 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (doc);
#line 476 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp0_);
#line 7767 "DomGDocumentTest.c"
	}
	goto __finally186;
	__catch186_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp44_ = NULL;
		gchar* _tmp45_ = NULL;
		gchar* _tmp46_ = NULL;
#line 476 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		e = _inner_error_;
#line 476 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_inner_error_ = NULL;
#line 491 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp44_ = e->message;
#line 491 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp45_ = g_strconcat ("Error: ", _tmp44_, NULL);
#line 491 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp46_ = _tmp45_;
#line 491 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_message ("DomGDocumentTest.vala:491: %s", _tmp46_);
#line 491 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp46_);
#line 492 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_assert_not_reached ();
#line 476 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_error_free0 (e);
#line 7794 "DomGDocumentTest.c"
	}
	__finally186:
#line 476 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 476 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 476 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 476 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		return;
#line 7805 "DomGDocumentTest.c"
	}
}


static void ___lambda200__gtest_func (void) {
#line 475 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	__lambda200_ ();
#line 7813 "DomGDocumentTest.c"
}


static void __lambda201_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* _tmp0_ = NULL;
		GXmlGDocument* _tmp1_ = NULL;
		GXmlDomDocument* doc = NULL;
		GXmlGDocument* _tmp2_ = NULL;
		GXmlDomDocument* _tmp3_ = NULL;
		GXmlGDocument* _tmp4_ = NULL;
		GXmlGDocument* _tmp5_ = NULL;
		GXmlDomDocument* doc2 = NULL;
		GXmlGDocument* _tmp6_ = NULL;
		GXmlDomDocument* _tmp7_ = NULL;
		GXmlDomDocument* _tmp8_ = NULL;
		GXmlDomDocument* _tmp9_ = NULL;
		GXmlDomElement* _tmp10_ = NULL;
		GXmlDomElement* _tmp11_ = NULL;
		GXmlDomElement* _tmp12_ = NULL;
		GXmlDomHTMLCollection* _tmp13_ = NULL;
		GXmlDomHTMLCollection* _tmp14_ = NULL;
		GXmlDomHTMLCollection* _tmp15_ = NULL;
		gpointer _tmp16_ = NULL;
		GXmlDomElement* _tmp17_ = NULL;
		GXmlDomNode* _tmp18_ = NULL;
		GXmlDomNode* _tmp19_ = NULL;
		GXmlDomDocument* _tmp20_ = NULL;
		GXmlDomElement* _tmp21_ = NULL;
		GXmlDomElement* _tmp22_ = NULL;
		GXmlDomElement* _tmp23_ = NULL;
		GXmlDomHTMLCollection* _tmp24_ = NULL;
		GXmlDomHTMLCollection* _tmp25_ = NULL;
		GXmlDomHTMLCollection* _tmp26_ = NULL;
		gpointer _tmp27_ = NULL;
		GXmlDomElement* _tmp28_ = NULL;
		gchar* _tmp29_ = NULL;
		gchar* _tmp30_ = NULL;
		gchar* _tmp31_ = NULL;
		GXmlDomDocument* _tmp32_ = NULL;
		GXmlDomElement* _tmp33_ = NULL;
		GXmlDomElement* _tmp34_ = NULL;
		GXmlDomElement* _tmp35_ = NULL;
		GXmlDomNode* _tmp36_ = NULL;
		GXmlDomNode* _tmp37_ = NULL;
		GXmlDomNode* _tmp38_ = NULL;
		GXmlDomDocument* _tmp39_ = NULL;
		GXmlDomElement* _tmp40_ = NULL;
		GXmlDomElement* _tmp41_ = NULL;
		GXmlDomElement* _tmp42_ = NULL;
		GXmlDomNode* _tmp43_ = NULL;
		GXmlDomNode* _tmp44_ = NULL;
		GXmlDomNode* _tmp45_ = NULL;
		gchar* _tmp46_ = NULL;
		gchar* _tmp47_ = NULL;
		gchar* _tmp48_ = NULL;
#line 500 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp1_ = gxml_gdocument_new_from_string (DOM_GDOCUMENT_TEST_XMLDOC, 0, &_inner_error_);
#line 500 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp0_ = _tmp1_;
#line 500 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 7877 "DomGDocumentTest.c"
			goto __catch187_g_error;
		}
#line 500 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp2_ = _tmp0_;
#line 500 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp0_ = NULL;
#line 500 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp3_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp2_, GXML_TYPE_DOM_DOCUMENT) ? ((GXmlDomDocument*) _tmp2_) : NULL;
#line 500 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp3_ == NULL) {
#line 500 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 7890 "DomGDocumentTest.c"
		}
#line 500 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		doc = _tmp3_;
#line 501 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp5_ = gxml_gdocument_new_from_string (DOM_GDOCUMENT_TEST_STRDOC, 0, &_inner_error_);
#line 501 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp4_ = _tmp5_;
#line 501 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 501 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 501 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 7904 "DomGDocumentTest.c"
			goto __catch187_g_error;
		}
#line 501 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp6_ = _tmp4_;
#line 501 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp4_ = NULL;
#line 501 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp7_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp6_, GXML_TYPE_DOM_DOCUMENT) ? ((GXmlDomDocument*) _tmp6_) : NULL;
#line 501 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp7_ == NULL) {
#line 501 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp6_);
#line 7917 "DomGDocumentTest.c"
		}
#line 501 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		doc2 = _tmp7_;
#line 502 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp8_ = doc2;
#line 502 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp9_ = doc;
#line 502 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp10_ = gxml_dom_document_get_document_element (_tmp9_);
#line 502 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp11_ = _tmp10_;
#line 502 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp12_ = _tmp11_;
#line 502 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp13_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp12_);
#line 502 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp14_ = _tmp13_;
#line 502 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp15_ = _tmp14_;
#line 502 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp16_ = gee_list_last ((GeeList*) _tmp15_);
#line 502 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp17_ = (GXmlDomElement*) _tmp16_;
#line 502 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp18_ = gxml_dom_document_adopt_node (_tmp8_, (GXmlDomNode*) _tmp17_, &_inner_error_);
#line 502 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp19_ = _tmp18_;
#line 502 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp19_);
#line 502 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp17_);
#line 502 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp15_);
#line 502 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp12_);
#line 502 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 502 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc2);
#line 502 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp4_);
#line 502 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (doc);
#line 502 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 7963 "DomGDocumentTest.c"
			goto __catch187_g_error;
		}
#line 507 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp20_ = doc;
#line 507 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp21_ = gxml_dom_document_get_document_element (_tmp20_);
#line 507 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp22_ = _tmp21_;
#line 507 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp23_ = _tmp22_;
#line 507 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp24_ = gxml_dom_parent_node_get_children ((GXmlDomParentNode*) _tmp23_);
#line 507 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp25_ = _tmp24_;
#line 507 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp26_ = _tmp25_;
#line 507 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp27_ = gee_list_last ((GeeList*) _tmp26_);
#line 507 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp28_ = (GXmlDomElement*) _tmp27_;
#line 507 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp29_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp28_);
#line 507 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp30_ = _tmp29_;
#line 507 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp31_ = _tmp30_;
#line 507 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp31_, "project") == 0, "doc.document_element.children.last ().node_name == \"project\"");
#line 507 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp31_);
#line 507 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp28_);
#line 507 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp26_);
#line 507 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp23_);
#line 508 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp32_ = doc2;
#line 508 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp33_ = gxml_dom_document_get_document_element (_tmp32_);
#line 508 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp34_ = _tmp33_;
#line 508 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp35_ = _tmp34_;
#line 508 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp36_ = gxml_dom_node_get_last_child ((GXmlDomNode*) _tmp35_);
#line 508 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp37_ = _tmp36_;
#line 508 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp38_ = _tmp37_;
#line 508 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp38_, GXML_TYPE_DOM_ELEMENT), "doc2.document_element.last_child is DomElement");
#line 508 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp38_);
#line 508 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp35_);
#line 509 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp39_ = doc2;
#line 509 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp40_ = gxml_dom_document_get_document_element (_tmp39_);
#line 509 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp41_ = _tmp40_;
#line 509 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp42_ = _tmp41_;
#line 509 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp43_ = gxml_dom_node_get_last_child ((GXmlDomNode*) _tmp42_);
#line 509 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp44_ = _tmp43_;
#line 509 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp45_ = _tmp44_;
#line 509 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp46_ = gxml_dom_node_get_node_name (_tmp45_);
#line 509 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp47_ = _tmp46_;
#line 509 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp48_ = _tmp47_;
#line 509 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp48_, "Author") == 0, "doc2.document_element.last_child.node_name == \"Author\"");
#line 509 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp48_);
#line 509 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp45_);
#line 509 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp42_);
#line 496 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (doc2);
#line 496 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp4_);
#line 496 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (doc);
#line 496 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp0_);
#line 8056 "DomGDocumentTest.c"
	}
	goto __finally187;
	__catch187_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp49_ = NULL;
		gchar* _tmp50_ = NULL;
		gchar* _tmp51_ = NULL;
#line 496 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		e = _inner_error_;
#line 496 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_inner_error_ = NULL;
#line 512 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp49_ = e->message;
#line 512 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp50_ = g_strconcat ("Error: ", _tmp49_, NULL);
#line 512 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp51_ = _tmp50_;
#line 512 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_message ("DomGDocumentTest.vala:512: %s", _tmp51_);
#line 512 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp51_);
#line 513 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_assert_not_reached ();
#line 496 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_error_free0 (e);
#line 8083 "DomGDocumentTest.c"
	}
	__finally187:
#line 496 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 496 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 496 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 496 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		return;
#line 8094 "DomGDocumentTest.c"
	}
}


static void ___lambda201__gtest_func (void) {
#line 495 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	__lambda201_ ();
#line 8102 "DomGDocumentTest.c"
}


static void __lambda202_ (void) {
}


static void ___lambda202__gtest_func (void) {
#line 516 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	__lambda202_ ();
#line 8113 "DomGDocumentTest.c"
}


static void __lambda203_ (void) {
}


static void ___lambda203__gtest_func (void) {
#line 519 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	__lambda203_ ();
#line 8124 "DomGDocumentTest.c"
}


static void __lambda204_ (void) {
}


static void ___lambda204__gtest_func (void) {
#line 522 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	__lambda204_ ();
#line 8135 "DomGDocumentTest.c"
}


static void __lambda205_ (void) {
}


static void ___lambda205__gtest_func (void) {
#line 525 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	__lambda205_ ();
#line 8146 "DomGDocumentTest.c"
}


static void __lambda206_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlDomDocument* d = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		GXmlDomDocument* _tmp1_ = NULL;
		GXmlDomText* _tmp2_ = NULL;
		GXmlDomDocument* _tmp3_ = NULL;
		GXmlDomText* _tmp4_ = NULL;
		GXmlDomCharacterData* t = NULL;
		GXmlDomText* _tmp5_ = NULL;
		GXmlDomCharacterData* _tmp6_ = NULL;
		GXmlDomCharacterData* _tmp7_ = NULL;
		gchar* _tmp8_ = NULL;
		gchar* _tmp9_ = NULL;
		gchar* _tmp10_ = NULL;
		GXmlDomCharacterData* _tmp11_ = NULL;
		gint _tmp12_ = 0;
		gint _tmp13_ = 0;
		gint _tmp14_ = 0;
		gint _tmp15_ = 0;
		gchar* _tmp16_ = NULL;
		GXmlDomCharacterData* _tmp17_ = NULL;
		gchar* _tmp18_ = NULL;
		gchar* _tmp19_ = NULL;
		gchar* _tmp20_ = NULL;
		gchar* _tmp21_ = NULL;
		GXmlDomCharacterData* _tmp22_ = NULL;
		gchar* _tmp23_ = NULL;
		gchar* _tmp24_ = NULL;
		gchar* _tmp25_ = NULL;
		gchar* _tmp26_ = NULL;
		GXmlDomCharacterData* _tmp27_ = NULL;
		gchar* _tmp28_ = NULL;
		gchar* _tmp29_ = NULL;
		gchar* _tmp30_ = NULL;
		gchar* _tmp31_ = NULL;
		GXmlDomCharacterData* _tmp32_ = NULL;
		gchar* _tmp33_ = NULL;
		gchar* _tmp34_ = NULL;
		gchar* _tmp35_ = NULL;
		gchar* _tmp36_ = NULL;
		GXmlDomCharacterData* _tmp37_ = NULL;
		gchar* _tmp38_ = NULL;
		gchar* _tmp39_ = NULL;
		gchar* _tmp40_ = NULL;
		gchar* _tmp41_ = NULL;
		GXmlDomCharacterData* _tmp42_ = NULL;
		gchar* _tmp43_ = NULL;
		gchar* _tmp44_ = NULL;
		gchar* _tmp45_ = NULL;
		GXmlDomCharacterData* _tmp49_ = NULL;
		GXmlDomCharacterData* _tmp50_ = NULL;
		gchar* _tmp51_ = NULL;
		gchar* _tmp52_ = NULL;
		gchar* _tmp53_ = NULL;
		GXmlDomCharacterData* _tmp54_ = NULL;
		GXmlDomCharacterData* _tmp55_ = NULL;
		gchar* _tmp56_ = NULL;
		gchar* _tmp57_ = NULL;
		gchar* _tmp58_ = NULL;
		GXmlDomCharacterData* _tmp59_ = NULL;
		GXmlDomCharacterData* _tmp60_ = NULL;
		gchar* _tmp61_ = NULL;
		gchar* _tmp62_ = NULL;
		gchar* _tmp63_ = NULL;
		GXmlDomCharacterData* _tmp64_ = NULL;
		GXmlDomCharacterData* _tmp65_ = NULL;
		gchar* _tmp66_ = NULL;
		gchar* _tmp67_ = NULL;
		gchar* _tmp68_ = NULL;
		GXmlDomCharacterData* _tmp69_ = NULL;
		GXmlDomCharacterData* _tmp70_ = NULL;
		gchar* _tmp71_ = NULL;
		gchar* _tmp72_ = NULL;
		gchar* _tmp73_ = NULL;
		GXmlDomCharacterData* _tmp74_ = NULL;
		GXmlDomCharacterData* _tmp75_ = NULL;
		gchar* _tmp76_ = NULL;
		gchar* _tmp77_ = NULL;
		gchar* _tmp78_ = NULL;
		GXmlDomElement* n = NULL;
		GXmlDomDocument* _tmp79_ = NULL;
		GXmlDomElement* _tmp80_ = NULL;
		GXmlDomDocument* _tmp81_ = NULL;
		GXmlDomElement* _tmp82_ = NULL;
		GXmlDomNode* _tmp83_ = NULL;
		GXmlDomNode* _tmp84_ = NULL;
		GXmlDomElement* _tmp85_ = NULL;
		GXmlDomCharacterData* _tmp86_ = NULL;
		GXmlDomNode* _tmp87_ = NULL;
		GXmlDomNode* _tmp88_ = NULL;
		GXmlDomCharacterData* _tmp89_ = NULL;
		GXmlDomNode* _tmp90_ = NULL;
		GXmlDomNode* _tmp91_ = NULL;
		GXmlDomNode* _tmp92_ = NULL;
		GXmlDomNodeList* _tmp93_ = NULL;
		GXmlDomNodeList* _tmp94_ = NULL;
		GXmlDomNodeList* _tmp95_ = NULL;
		gint _tmp96_ = 0;
		gint _tmp97_ = 0;
		GXmlDomCharacterData* _tmp98_ = NULL;
		GXmlDomNode* _tmp99_ = NULL;
		GXmlDomNode* _tmp100_ = NULL;
		GXmlDomNode* _tmp101_ = NULL;
		GXmlDomNodeList* _tmp102_ = NULL;
		GXmlDomNodeList* _tmp103_ = NULL;
		GXmlDomNodeList* _tmp104_ = NULL;
		gint _tmp105_ = 0;
		gint _tmp106_ = 0;
		GXmlDomElement* ntst = NULL;
		GXmlDomDocument* _tmp107_ = NULL;
		GXmlDomElement* _tmp108_ = NULL;
		GXmlDomElement* _tmp109_ = NULL;
		GXmlDomElement* _tmp110_ = NULL;
		GXmlDomNode* _tmp111_ = NULL;
		GXmlDomNode* _tmp112_ = NULL;
		GXmlDomText* ct1 = NULL;
		GXmlDomDocument* _tmp113_ = NULL;
		GXmlDomText* _tmp114_ = NULL;
		GXmlDomElement* _tmp115_ = NULL;
		GXmlDomText* _tmp116_ = NULL;
		GXmlDomNode* _tmp117_ = NULL;
		GXmlDomNode* _tmp118_ = NULL;
		GXmlDomElement* _tmp119_ = NULL;
		GXmlDomNodeList* _tmp120_ = NULL;
		GXmlDomNodeList* _tmp121_ = NULL;
		GXmlDomNodeList* _tmp122_ = NULL;
		gint _tmp123_ = 0;
		gint _tmp124_ = 0;
		GXmlDomText* ct2 = NULL;
		GXmlDomDocument* _tmp125_ = NULL;
		GXmlDomText* _tmp126_ = NULL;
		GXmlDomElement* _tmp127_ = NULL;
		GXmlDomText* _tmp128_ = NULL;
		GXmlDomNode* _tmp129_ = NULL;
		GXmlDomNode* _tmp130_ = NULL;
		GXmlDomElement* _tmp131_ = NULL;
		GXmlDomNodeList* _tmp132_ = NULL;
		GXmlDomNodeList* _tmp133_ = NULL;
		GXmlDomNodeList* _tmp134_ = NULL;
		GXmlDomNode* _tmp135_ = NULL;
		GXmlDomNode* _tmp136_ = NULL;
		GXmlDomElement* _tmp137_ = NULL;
		GXmlDomNodeList* _tmp138_ = NULL;
		GXmlDomNodeList* _tmp139_ = NULL;
		GXmlDomNodeList* _tmp140_ = NULL;
		GXmlDomNode* _tmp141_ = NULL;
		GXmlDomText* _tmp142_ = NULL;
		GXmlDomText* _tmp143_ = NULL;
		gchar* _tmp144_ = NULL;
		gchar* _tmp145_ = NULL;
		gchar* _tmp146_ = NULL;
		GXmlDomElement* _tmp147_ = NULL;
		GXmlDomNodeList* _tmp148_ = NULL;
		GXmlDomNodeList* _tmp149_ = NULL;
		GXmlDomNodeList* _tmp150_ = NULL;
		GXmlDomNode* _tmp151_ = NULL;
		GXmlDomText* _tmp152_ = NULL;
		GXmlDomText* _tmp153_ = NULL;
		gchar* _tmp154_ = NULL;
		gchar* _tmp155_ = NULL;
		gchar* _tmp156_ = NULL;
#line 530 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp0_ = gxml_gdocument_new ();
#line 530 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp1_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp0_, GXML_TYPE_DOM_DOCUMENT) ? ((GXmlDomDocument*) _tmp0_) : NULL;
#line 530 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp1_ == NULL) {
#line 530 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp0_);
#line 8321 "DomGDocumentTest.c"
		}
#line 530 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		d = _tmp1_;
#line 531 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp3_ = d;
#line 531 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp4_ = gxml_dom_document_create_text_node (_tmp3_, "TEXT", &_inner_error_);
#line 531 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp2_ = _tmp4_;
#line 531 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 531 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 8335 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 531 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp5_ = _tmp2_;
#line 531 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp2_ = NULL;
#line 531 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp6_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp5_, GXML_TYPE_DOM_CHARACTER_DATA) ? ((GXmlDomCharacterData*) _tmp5_) : NULL;
#line 531 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp6_ == NULL) {
#line 531 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp5_);
#line 8348 "DomGDocumentTest.c"
		}
#line 531 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		t = _tmp6_;
#line 532 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp7_ = t;
#line 532 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp8_ = gxml_dom_character_data_get_data (_tmp7_);
#line 532 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp9_ = _tmp8_;
#line 532 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp10_ = _tmp9_;
#line 532 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp10_, "TEXT") == 0, "t.data == \"TEXT\"");
#line 532 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp10_);
#line 533 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp11_ = t;
#line 533 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp12_ = gxml_dom_character_data_get_length (_tmp11_);
#line 533 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp13_ = _tmp12_;
#line 533 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp14_ = strlen ("TEXT");
#line 533 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp15_ = _tmp14_;
#line 533 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp13_ == _tmp15_, "t.length == \"TEXT\".length");
#line 534 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp17_ = t;
#line 534 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp18_ = gxml_dom_character_data_substring_data (_tmp17_, 0, 2, &_inner_error_);
#line 534 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp16_ = _tmp18_;
#line 534 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 534 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 534 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 534 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 8390 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 534 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp19_ = _tmp16_;
#line 534 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp16_ = NULL;
#line 534 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp20_ = _tmp19_;
#line 534 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp20_, "TE") == 0, "t.substring_data (0,2) == \"TE\"");
#line 534 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp20_);
#line 535 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp22_ = t;
#line 535 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp23_ = gxml_dom_character_data_substring_data (_tmp22_, 0, 3, &_inner_error_);
#line 535 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp21_ = _tmp23_;
#line 535 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 535 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp16_);
#line 535 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 535 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 535 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 8419 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 535 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp24_ = _tmp21_;
#line 535 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp21_ = NULL;
#line 535 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp25_ = _tmp24_;
#line 535 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp25_, "TEX") == 0, "t.substring_data (0,3) == \"TEX\"");
#line 535 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp25_);
#line 536 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp27_ = t;
#line 536 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp28_ = gxml_dom_character_data_substring_data (_tmp27_, 1, 3, &_inner_error_);
#line 536 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp26_ = _tmp28_;
#line 536 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 536 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp21_);
#line 536 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp16_);
#line 536 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 536 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 536 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 8450 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 536 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp29_ = _tmp26_;
#line 536 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp26_ = NULL;
#line 536 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp30_ = _tmp29_;
#line 536 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp30_, "EXT") == 0, "t.substring_data (1,3) == \"EXT\"");
#line 536 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp30_);
#line 537 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp32_ = t;
#line 537 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp33_ = gxml_dom_character_data_substring_data (_tmp32_, 1, 4, &_inner_error_);
#line 537 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp31_ = _tmp33_;
#line 537 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 537 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp26_);
#line 537 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp21_);
#line 537 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp16_);
#line 537 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 537 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 537 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 8483 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 537 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp34_ = _tmp31_;
#line 537 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp31_ = NULL;
#line 537 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp35_ = _tmp34_;
#line 537 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp35_, "EXT") == 0, "t.substring_data (1,4) == \"EXT\"");
#line 537 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp35_);
#line 538 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp37_ = t;
#line 538 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp38_ = gxml_dom_character_data_substring_data (_tmp37_, 1, 5, &_inner_error_);
#line 538 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp36_ = _tmp38_;
#line 538 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 538 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp31_);
#line 538 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp26_);
#line 538 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp21_);
#line 538 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp16_);
#line 538 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 538 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 538 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 8518 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 538 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp39_ = _tmp36_;
#line 538 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp36_ = NULL;
#line 538 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp40_ = _tmp39_;
#line 538 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp40_, "EXT") == 0, "t.substring_data (1,5) == \"EXT\"");
#line 538 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp40_);
#line 539 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp42_ = t;
#line 539 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp43_ = gxml_dom_character_data_substring_data (_tmp42_, 2, 0, &_inner_error_);
#line 539 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp41_ = _tmp43_;
#line 539 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 539 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp36_);
#line 539 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp31_);
#line 539 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp26_);
#line 539 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp21_);
#line 539 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp16_);
#line 539 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 539 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 539 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 8555 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 539 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp44_ = _tmp41_;
#line 539 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp41_ = NULL;
#line 539 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp45_ = _tmp44_;
#line 539 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp45_, "") == 0, "t.substring_data (2,0) == \"\"");
#line 539 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp45_);
#line 8568 "DomGDocumentTest.c"
		{
			GXmlDomCharacterData* _tmp46_ = NULL;
			gchar* _tmp47_ = NULL;
			gchar* _tmp48_ = NULL;
#line 541 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_tmp46_ = t;
#line 541 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_tmp47_ = gxml_dom_character_data_substring_data (_tmp46_, 5, 1, &_inner_error_);
#line 541 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_tmp48_ = _tmp47_;
#line 541 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp48_);
#line 541 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 8583 "DomGDocumentTest.c"
				goto __catch189_g_error;
			}
#line 542 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			g_assert_not_reached ();
#line 8588 "DomGDocumentTest.c"
		}
		goto __finally189;
		__catch189_g_error:
		{
#line 540 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			g_clear_error (&_inner_error_);
#line 540 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_inner_error_ = NULL;
#line 8597 "DomGDocumentTest.c"
		}
		__finally189:
#line 540 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 540 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp41_);
#line 540 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp36_);
#line 540 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp31_);
#line 540 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp26_);
#line 540 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp21_);
#line 540 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp16_);
#line 540 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 540 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 540 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 8620 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 545 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp49_ = t;
#line 545 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		gxml_dom_character_data_append_data (_tmp49_, " HI");
#line 546 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp50_ = t;
#line 546 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp51_ = gxml_dom_character_data_get_data (_tmp50_);
#line 546 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp52_ = _tmp51_;
#line 546 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp53_ = _tmp52_;
#line 546 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp53_, "TEXT HI") == 0, "t.data == \"TEXT HI\"");
#line 546 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp53_);
#line 547 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp54_ = t;
#line 547 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		gxml_dom_character_data_replace_data (_tmp54_, 0, 4, "", &_inner_error_);
#line 547 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 547 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp41_);
#line 547 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp36_);
#line 547 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp31_);
#line 547 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp26_);
#line 547 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp21_);
#line 547 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp16_);
#line 547 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 547 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 547 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 8663 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 548 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp55_ = t;
#line 548 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp56_ = gxml_dom_character_data_get_data (_tmp55_);
#line 548 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp57_ = _tmp56_;
#line 548 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp58_ = _tmp57_;
#line 548 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp58_, " HI") == 0, "t.data == \" HI\"");
#line 548 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp58_);
#line 549 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp59_ = t;
#line 549 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		gxml_dom_character_data_insert_data (_tmp59_, 0, "TEXT", &_inner_error_);
#line 549 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 549 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp41_);
#line 549 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp36_);
#line 549 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp31_);
#line 549 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp26_);
#line 549 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp21_);
#line 549 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp16_);
#line 549 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 549 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 549 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 8702 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 550 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp60_ = t;
#line 550 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp61_ = gxml_dom_character_data_get_data (_tmp60_);
#line 550 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp62_ = _tmp61_;
#line 550 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp63_ = _tmp62_;
#line 550 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp63_, "TEXT HI") == 0, "t.data == \"TEXT HI\"");
#line 550 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp63_);
#line 551 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp64_ = t;
#line 551 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		gxml_dom_character_data_delete_data (_tmp64_, 0, 4, &_inner_error_);
#line 551 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 551 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp41_);
#line 551 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp36_);
#line 551 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp31_);
#line 551 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp26_);
#line 551 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp21_);
#line 551 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp16_);
#line 551 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 551 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 551 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 8741 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 552 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp65_ = t;
#line 552 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp66_ = gxml_dom_character_data_get_data (_tmp65_);
#line 552 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp67_ = _tmp66_;
#line 552 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp68_ = _tmp67_;
#line 552 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp68_, " HI") == 0, "t.data == \" HI\"");
#line 552 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp68_);
#line 553 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp69_ = t;
#line 553 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		gxml_dom_character_data_insert_data (_tmp69_, 0, "TEXT", &_inner_error_);
#line 553 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 553 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp41_);
#line 553 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp36_);
#line 553 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp31_);
#line 553 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp26_);
#line 553 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp21_);
#line 553 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp16_);
#line 553 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 553 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 553 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 8780 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 554 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp70_ = t;
#line 554 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp71_ = gxml_dom_character_data_get_data (_tmp70_);
#line 554 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp72_ = _tmp71_;
#line 554 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp73_ = _tmp72_;
#line 554 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp73_, "TEXT HI") == 0, "t.data == \"TEXT HI\"");
#line 554 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp73_);
#line 555 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp74_ = t;
#line 555 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		gxml_dom_character_data_replace_data (_tmp74_, 0, 4, "text", &_inner_error_);
#line 555 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 555 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp41_);
#line 555 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp36_);
#line 555 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp31_);
#line 555 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp26_);
#line 555 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp21_);
#line 555 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp16_);
#line 555 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 555 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 555 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 8819 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 556 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp75_ = t;
#line 556 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp76_ = gxml_dom_character_data_get_data (_tmp75_);
#line 556 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp77_ = _tmp76_;
#line 556 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp78_ = _tmp77_;
#line 556 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp78_, "text HI") == 0, "t.data == \"text HI\"");
#line 556 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp78_);
#line 557 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp79_ = d;
#line 557 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp80_ = gxml_dom_document_create_element (_tmp79_, "node", &_inner_error_);
#line 557 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		n = _tmp80_;
#line 557 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 557 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp41_);
#line 557 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp36_);
#line 557 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp31_);
#line 557 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp26_);
#line 557 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp21_);
#line 557 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp16_);
#line 557 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 557 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 557 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 8860 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 558 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp81_ = d;
#line 558 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp82_ = n;
#line 558 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp83_ = gxml_dom_node_append_child ((GXmlDomNode*) _tmp81_, (GXmlDomNode*) _tmp82_, &_inner_error_);
#line 558 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp84_ = _tmp83_;
#line 558 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp84_);
#line 558 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 558 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 558 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp41_);
#line 558 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp36_);
#line 558 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp31_);
#line 558 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp26_);
#line 558 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp21_);
#line 558 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp16_);
#line 558 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 558 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 558 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 8895 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 559 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp85_ = n;
#line 559 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp86_ = t;
#line 559 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp87_ = gxml_dom_node_append_child ((GXmlDomNode*) _tmp85_, (GXmlDomNode*) _tmp86_, &_inner_error_);
#line 559 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp88_ = _tmp87_;
#line 559 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp88_);
#line 559 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 559 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 559 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp41_);
#line 559 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp36_);
#line 559 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp31_);
#line 559 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp26_);
#line 559 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp21_);
#line 559 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp16_);
#line 559 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 559 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 559 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 8930 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 560 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp89_ = t;
#line 560 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp90_ = gxml_dom_node_get_parent_node ((GXmlDomNode*) _tmp89_);
#line 560 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp91_ = _tmp90_;
#line 560 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp92_ = _tmp91_;
#line 560 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp93_ = gxml_dom_node_get_child_nodes (_tmp92_);
#line 560 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp94_ = _tmp93_;
#line 560 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp95_ = _tmp94_;
#line 560 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp96_ = gxml_dom_node_list_get_length (_tmp95_);
#line 560 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp97_ = _tmp96_;
#line 560 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp97_ == 1, "t.parent_node.child_nodes.length == 1");
#line 560 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp95_);
#line 560 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp92_);
#line 561 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp98_ = t;
#line 561 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp99_ = gxml_dom_node_get_parent_node ((GXmlDomNode*) _tmp98_);
#line 561 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp100_ = _tmp99_;
#line 561 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp101_ = _tmp100_;
#line 561 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp102_ = gxml_dom_node_get_child_nodes (_tmp101_);
#line 561 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp103_ = _tmp102_;
#line 561 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp104_ = _tmp103_;
#line 561 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp105_ = gxml_dom_node_list_get_length (_tmp104_);
#line 561 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp106_ = _tmp105_;
#line 561 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp106_ == 1, "t.parent_node.child_nodes.length == 1");
#line 561 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp104_);
#line 561 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp101_);
#line 562 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp107_ = d;
#line 562 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp108_ = gxml_dom_document_create_element (_tmp107_, "child", &_inner_error_);
#line 562 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		ntst = _tmp108_;
#line 562 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 562 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 562 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp41_);
#line 562 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp36_);
#line 562 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp31_);
#line 562 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp26_);
#line 562 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp21_);
#line 562 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp16_);
#line 562 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 562 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 562 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 9009 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 563 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp109_ = n;
#line 563 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp110_ = ntst;
#line 563 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp111_ = gxml_dom_node_append_child ((GXmlDomNode*) _tmp109_, (GXmlDomNode*) _tmp110_, &_inner_error_);
#line 563 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp112_ = _tmp111_;
#line 563 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp112_);
#line 563 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 563 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ntst);
#line 563 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 563 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp41_);
#line 563 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp36_);
#line 563 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp31_);
#line 563 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp26_);
#line 563 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp21_);
#line 563 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp16_);
#line 563 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 563 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 563 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 9046 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 564 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp113_ = d;
#line 564 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp114_ = gxml_dom_document_create_text_node (_tmp113_, "TEXT1", &_inner_error_);
#line 564 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		ct1 = _tmp114_;
#line 564 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 564 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ntst);
#line 564 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 564 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp41_);
#line 564 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp36_);
#line 564 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp31_);
#line 564 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp26_);
#line 564 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp21_);
#line 564 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp16_);
#line 564 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 564 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 564 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 9079 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 565 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp115_ = ntst;
#line 565 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp116_ = ct1;
#line 565 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp117_ = gxml_dom_node_append_child ((GXmlDomNode*) _tmp115_, (GXmlDomNode*) _tmp116_, &_inner_error_);
#line 565 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp118_ = _tmp117_;
#line 565 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp118_);
#line 565 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 565 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ct1);
#line 565 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ntst);
#line 565 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 565 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp41_);
#line 565 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp36_);
#line 565 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp31_);
#line 565 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp26_);
#line 565 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp21_);
#line 565 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp16_);
#line 565 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 565 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 565 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 9118 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 566 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp119_ = ntst;
#line 566 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp120_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp119_);
#line 566 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp121_ = _tmp120_;
#line 566 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp122_ = _tmp121_;
#line 566 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp123_ = gxml_dom_node_list_get_length (_tmp122_);
#line 566 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp124_ = _tmp123_;
#line 566 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (_tmp124_ == 1, "ntst.child_nodes.length == 1");
#line 566 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp122_);
#line 567 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp125_ = d;
#line 567 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp126_ = gxml_dom_document_create_text_node (_tmp125_, "TEXT2", &_inner_error_);
#line 567 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		ct2 = _tmp126_;
#line 567 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 567 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ct1);
#line 567 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ntst);
#line 567 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 567 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp41_);
#line 567 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp36_);
#line 567 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp31_);
#line 567 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp26_);
#line 567 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp21_);
#line 567 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp16_);
#line 567 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 567 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 567 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 9169 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 568 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp127_ = ntst;
#line 568 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp128_ = ct2;
#line 568 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp129_ = gxml_dom_node_append_child ((GXmlDomNode*) _tmp127_, (GXmlDomNode*) _tmp128_, &_inner_error_);
#line 568 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp130_ = _tmp129_;
#line 568 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp130_);
#line 568 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 568 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ct2);
#line 568 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ct1);
#line 568 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (ntst);
#line 568 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (n);
#line 568 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp41_);
#line 568 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp36_);
#line 568 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp31_);
#line 568 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp26_);
#line 568 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp21_);
#line 568 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_free0 (_tmp16_);
#line 568 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (t);
#line 568 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp2_);
#line 568 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (d);
#line 9210 "DomGDocumentTest.c"
			goto __catch188_g_error;
		}
#line 576 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp131_ = ntst;
#line 576 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp132_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp131_);
#line 576 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp133_ = _tmp132_;
#line 576 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp134_ = _tmp133_;
#line 576 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp135_ = gxml_dom_node_list_item (_tmp134_, 0);
#line 576 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp136_ = _tmp135_;
#line 576 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp136_, GXML_TYPE_DOM_TEXT), "ntst.child_nodes.item (0) is DomText");
#line 576 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp136_);
#line 576 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp134_);
#line 577 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp137_ = ntst;
#line 577 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp138_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp137_);
#line 577 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp139_ = _tmp138_;
#line 577 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp140_ = _tmp139_;
#line 577 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp141_ = gxml_dom_node_list_item (_tmp140_, 0);
#line 577 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp142_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp141_, GXML_TYPE_DOM_TEXT) ? ((GXmlDomText*) _tmp141_) : NULL;
#line 577 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp142_ == NULL) {
#line 577 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp141_);
#line 9247 "DomGDocumentTest.c"
		}
#line 577 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp143_ = _tmp142_;
#line 577 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp144_ = gxml_dom_character_data_get_data ((GXmlDomCharacterData*) _tmp143_);
#line 577 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp145_ = _tmp144_;
#line 577 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp146_ = _tmp145_;
#line 577 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp146_, "TEXT1TEXT2") == 0, "(ntst.child_nodes.item (0) as DomText).data == \"TEXT1TEXT2\"");
#line 577 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp146_);
#line 577 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp143_);
#line 577 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp140_);
#line 579 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp147_ = ntst;
#line 579 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp148_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp147_);
#line 579 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp149_ = _tmp148_;
#line 579 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp150_ = _tmp149_;
#line 579 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp151_ = gxml_dom_node_list_item (_tmp150_, 0);
#line 579 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp152_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp151_, GXML_TYPE_DOM_TEXT) ? ((GXmlDomText*) _tmp151_) : NULL;
#line 579 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		if (_tmp152_ == NULL) {
#line 579 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
			_g_object_unref0 (_tmp151_);
#line 9281 "DomGDocumentTest.c"
		}
#line 579 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp153_ = _tmp152_;
#line 579 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp154_ = gxml_dom_text_get_whole_text (_tmp153_);
#line 579 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp155_ = _tmp154_;
#line 579 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp156_ = _tmp155_;
#line 579 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp156_, "TEXT1TEXT2") == 0, "(ntst.child_nodes.item(0) as DomText).whole_text == \"TEXT1TEXT2\"");
#line 579 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp156_);
#line 579 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp153_);
#line 579 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp150_);
#line 529 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (ct2);
#line 529 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (ct1);
#line 529 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (ntst);
#line 529 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (n);
#line 529 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp41_);
#line 529 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp36_);
#line 529 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp31_);
#line 529 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp26_);
#line 529 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp21_);
#line 529 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp16_);
#line 529 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (t);
#line 529 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (_tmp2_);
#line 529 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_object_unref0 (d);
#line 9325 "DomGDocumentTest.c"
	}
	goto __finally188;
	__catch188_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp157_ = NULL;
		gchar* _tmp158_ = NULL;
		gchar* _tmp159_ = NULL;
#line 529 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		e = _inner_error_;
#line 529 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_inner_error_ = NULL;
#line 581 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp157_ = e->message;
#line 581 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp158_ = g_strconcat ("Error: ", _tmp157_, NULL);
#line 581 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_tmp159_ = _tmp158_;
#line 581 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_message ("DomGDocumentTest.vala:581: %s", _tmp159_);
#line 581 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_free0 (_tmp159_);
#line 582 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_assert_not_reached ();
#line 529 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		_g_error_free0 (e);
#line 9352 "DomGDocumentTest.c"
	}
	__finally188:
#line 529 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 529 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 529 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 529 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
		return;
#line 9363 "DomGDocumentTest.c"
	}
}


static void ___lambda206__gtest_func (void) {
#line 528 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	__lambda206_ ();
#line 9371 "DomGDocumentTest.c"
}


void dom_gdocument_test_add_tests (void) {
#line 69 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	g_test_add_func ("/gxml/dom/document/children", ___lambda193__gtest_func);
#line 89 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	g_test_add_func ("/gxml/dom/document/child_nodes", ___lambda194__gtest_func);
#line 112 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	g_test_add_func ("/gxml/dom/document/element_collections", ___lambda195__gtest_func);
#line 139 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	g_test_add_func ("/gxml/dom/element/element_collections", ___lambda196__gtest_func);
#line 167 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	g_test_add_func ("/gxml/dom/node", ___lambda197__gtest_func);
#line 338 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	g_test_add_func ("/gxml/dom/element/api", ___lambda198__gtest_func);
#line 400 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	g_test_add_func ("/gxml/dom/document/api", ___lambda199__gtest_func);
#line 475 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	g_test_add_func ("/gxml/dom/document/import", ___lambda200__gtest_func);
#line 495 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	g_test_add_func ("/gxml/dom/document/adopt", ___lambda201__gtest_func);
#line 516 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	g_test_add_func ("/gxml/dom/document/event", ___lambda202__gtest_func);
#line 519 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	g_test_add_func ("/gxml/dom/document/range", ___lambda203__gtest_func);
#line 522 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	g_test_add_func ("/gxml/dom/document/iterator", ___lambda204__gtest_func);
#line 525 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	g_test_add_func ("/gxml/dom/document/walker", ___lambda205__gtest_func);
#line 528 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	g_test_add_func ("/gxml/dom/character", ___lambda206__gtest_func);
#line 9404 "DomGDocumentTest.c"
}


DomGDocumentTest* dom_gdocument_test_construct (GType object_type) {
	DomGDocumentTest* self = NULL;
#line 25 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	self = (DomGDocumentTest*) gxml_test_construct (object_type);
#line 25 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	return self;
#line 9414 "DomGDocumentTest.c"
}


DomGDocumentTest* dom_gdocument_test_new (void) {
#line 25 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	return dom_gdocument_test_construct (TYPE_DOM_GDOCUMENT_TEST);
#line 9421 "DomGDocumentTest.c"
}


static void dom_gdocument_test_class_init (DomGDocumentTestClass * klass) {
#line 25 "/home/despinosa/Proyectos/gxml/test/DomGDocumentTest.vala"
	dom_gdocument_test_parent_class = g_type_class_peek_parent (klass);
#line 9428 "DomGDocumentTest.c"
}


static void dom_gdocument_test_instance_init (DomGDocumentTest * self) {
}


GType dom_gdocument_test_get_type (void) {
	static volatile gsize dom_gdocument_test_type_id__volatile = 0;
	if (g_once_init_enter (&dom_gdocument_test_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (DomGDocumentTestClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) dom_gdocument_test_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DomGDocumentTest), 0, (GInstanceInitFunc) dom_gdocument_test_instance_init, NULL };
		GType dom_gdocument_test_type_id;
		dom_gdocument_test_type_id = g_type_register_static (TYPE_GXML_TEST, "DomGDocumentTest", &g_define_type_info, 0);
		g_once_init_leave (&dom_gdocument_test_type_id__volatile, dom_gdocument_test_type_id);
	}
	return dom_gdocument_test_type_id__volatile;
}



