/* SerializableGeeHashMapTest.c generated by valac 0.34.6, the Vala compiler
 * generated from SerializableGeeHashMapTest.vala, do not modify */

/* -*- Mode: vala; indent-tabs-mode: nil; c-basic-offset: 2; tab-width: 2 -*- */

#include <glib.h>
#include <glib-object.h>
#include <gxml/gxml.h>
#include <stdlib.h>
#include <string.h>
#include <gee.h>


#define TYPE_GXML_TEST (gxml_test_get_type ())
#define GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GXML_TEST, GXmlTest))
#define GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GXML_TEST, GXmlTestClass))
#define IS_GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GXML_TEST))
#define IS_GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GXML_TEST))
#define GXML_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GXML_TEST, GXmlTestClass))

typedef struct _GXmlTest GXmlTest;
typedef struct _GXmlTestClass GXmlTestClass;
typedef struct _GXmlTestPrivate GXmlTestPrivate;

#define TYPE_SERIALIZABLE_GEE_HASH_MAP_TEST (serializable_gee_hash_map_test_get_type ())
#define SERIALIZABLE_GEE_HASH_MAP_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SERIALIZABLE_GEE_HASH_MAP_TEST, SerializableGeeHashMapTest))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SERIALIZABLE_GEE_HASH_MAP_TEST, SerializableGeeHashMapTestClass))
#define IS_SERIALIZABLE_GEE_HASH_MAP_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SERIALIZABLE_GEE_HASH_MAP_TEST))
#define IS_SERIALIZABLE_GEE_HASH_MAP_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SERIALIZABLE_GEE_HASH_MAP_TEST))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SERIALIZABLE_GEE_HASH_MAP_TEST, SerializableGeeHashMapTestClass))

typedef struct _SerializableGeeHashMapTest SerializableGeeHashMapTest;
typedef struct _SerializableGeeHashMapTestClass SerializableGeeHashMapTestClass;
typedef struct _SerializableGeeHashMapTestPrivate SerializableGeeHashMapTestPrivate;

#define SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE (serializable_gee_hash_map_test_space_get_type ())
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE, SerializableGeeHashMapTestSpace))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE, SerializableGeeHashMapTestSpaceClass))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_IS_SPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_IS_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE, SerializableGeeHashMapTestSpaceClass))

typedef struct _SerializableGeeHashMapTestSpace SerializableGeeHashMapTestSpace;
typedef struct _SerializableGeeHashMapTestSpaceClass SerializableGeeHashMapTestSpaceClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))

#define SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE_CONTAINER (serializable_gee_hash_map_test_space_container_get_type ())
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE_CONTAINER, SerializableGeeHashMapTestSpaceContainer))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE_CONTAINER, SerializableGeeHashMapTestSpaceContainerClass))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_IS_SPACE_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE_CONTAINER))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_IS_SPACE_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE_CONTAINER))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_CONTAINER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE_CONTAINER, SerializableGeeHashMapTestSpaceContainerClass))

typedef struct _SerializableGeeHashMapTestSpaceContainer SerializableGeeHashMapTestSpaceContainer;
typedef struct _SerializableGeeHashMapTestSpaceContainerClass SerializableGeeHashMapTestSpaceContainerClass;

#define SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_TYPE_COLLECTION (serializable_gee_hash_map_test_space_collection_get_type ())
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_TYPE_COLLECTION, SerializableGeeHashMapTestSpaceCollection))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_TYPE_COLLECTION, SerializableGeeHashMapTestSpaceCollectionClass))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_TYPE_COLLECTION))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_IS_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_TYPE_COLLECTION))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_TYPE_COLLECTION, SerializableGeeHashMapTestSpaceCollectionClass))

typedef struct _SerializableGeeHashMapTestSpaceCollection SerializableGeeHashMapTestSpaceCollection;
typedef struct _SerializableGeeHashMapTestSpaceCollectionClass SerializableGeeHashMapTestSpaceCollectionClass;

#define TYPE_BOOK_STORE (book_store_get_type ())
#define BOOK_STORE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_BOOK_STORE, BookStore))
#define BOOK_STORE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_BOOK_STORE, BookStoreClass))
#define IS_BOOK_STORE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_BOOK_STORE))
#define IS_BOOK_STORE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_BOOK_STORE))
#define BOOK_STORE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_BOOK_STORE, BookStoreClass))

typedef struct _BookStore BookStore;
typedef struct _BookStoreClass BookStoreClass;

#define BOOK_TYPE_ARRAY (book_array_get_type ())
#define BOOK_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOOK_TYPE_ARRAY, BookArray))
#define BOOK_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOOK_TYPE_ARRAY, BookArrayClass))
#define BOOK_IS_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOOK_TYPE_ARRAY))
#define BOOK_IS_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOOK_TYPE_ARRAY))
#define BOOK_ARRAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOOK_TYPE_ARRAY, BookArrayClass))

typedef struct _BookArray BookArray;
typedef struct _BookArrayClass BookArrayClass;

#define TYPE_BOOK (book_get_type ())
#define BOOK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_BOOK, Book))
#define BOOK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_BOOK, BookClass))
#define IS_BOOK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_BOOK))
#define IS_BOOK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_BOOK))
#define BOOK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_BOOK, BookClass))

typedef struct _Book Book;
typedef struct _BookClass BookClass;

#define TYPE_NAME (name_get_type ())
#define NAME(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_NAME, Name))
#define NAME_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_NAME, NameClass))
#define IS_NAME(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_NAME))
#define IS_NAME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_NAME))
#define NAME_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_NAME, NameClass))

typedef struct _Name Name;
typedef struct _NameClass NameClass;

#define TYPE_AUTHORS (authors_get_type ())
#define AUTHORS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_AUTHORS, Authors))
#define AUTHORS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_AUTHORS, AuthorsClass))
#define IS_AUTHORS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_AUTHORS))
#define IS_AUTHORS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_AUTHORS))
#define AUTHORS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_AUTHORS, AuthorsClass))

typedef struct _Authors Authors;
typedef struct _AuthorsClass AuthorsClass;

#define AUTHOR_TYPE_ARRAY (author_array_get_type ())
#define AUTHOR_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), AUTHOR_TYPE_ARRAY, AuthorArray))
#define AUTHOR_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), AUTHOR_TYPE_ARRAY, AuthorArrayClass))
#define AUTHOR_IS_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), AUTHOR_TYPE_ARRAY))
#define AUTHOR_IS_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), AUTHOR_TYPE_ARRAY))
#define AUTHOR_ARRAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), AUTHOR_TYPE_ARRAY, AuthorArrayClass))

typedef struct _AuthorArray AuthorArray;
typedef struct _AuthorArrayClass AuthorArrayClass;

#define CATEGORY_TYPE_MAP (category_map_get_type ())
#define CATEGORY_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CATEGORY_TYPE_MAP, CategoryMap))
#define CATEGORY_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CATEGORY_TYPE_MAP, CategoryMapClass))
#define CATEGORY_IS_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CATEGORY_TYPE_MAP))
#define CATEGORY_IS_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CATEGORY_TYPE_MAP))
#define CATEGORY_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CATEGORY_TYPE_MAP, CategoryMapClass))

typedef struct _CategoryMap CategoryMap;
typedef struct _CategoryMapClass CategoryMapClass;

#define TYPE_CATEGORY (category_get_type ())
#define CATEGORY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CATEGORY, Category))
#define CATEGORY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CATEGORY, CategoryClass))
#define IS_CATEGORY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CATEGORY))
#define IS_CATEGORY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CATEGORY))
#define CATEGORY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CATEGORY, CategoryClass))

typedef struct _Category Category;
typedef struct _CategoryClass CategoryClass;

#define SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BIG_BAG (serializable_gee_hash_map_test_big_bag_get_type ())
#define SERIALIZABLE_GEE_HASH_MAP_TEST_BIG_BAG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BIG_BAG, SerializableGeeHashMapTestBigBag))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_BIG_BAG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BIG_BAG, SerializableGeeHashMapTestBigBagClass))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_IS_BIG_BAG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BIG_BAG))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_IS_BIG_BAG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BIG_BAG))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_BIG_BAG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BIG_BAG, SerializableGeeHashMapTestBigBagClass))

typedef struct _SerializableGeeHashMapTestBigBag SerializableGeeHashMapTestBigBag;
typedef struct _SerializableGeeHashMapTestBigBagClass SerializableGeeHashMapTestBigBagClass;

#define SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_TYPE_HASH_MAP (serializable_gee_hash_map_test_small_bag_hash_map_get_type ())
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_HASH_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_TYPE_HASH_MAP, SerializableGeeHashMapTestSmallBagHashMap))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_HASH_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_TYPE_HASH_MAP, SerializableGeeHashMapTestSmallBagHashMapClass))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_IS_HASH_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_TYPE_HASH_MAP))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_IS_HASH_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_TYPE_HASH_MAP))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_HASH_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_TYPE_HASH_MAP, SerializableGeeHashMapTestSmallBagHashMapClass))

typedef struct _SerializableGeeHashMapTestSmallBagHashMap SerializableGeeHashMapTestSmallBagHashMap;
typedef struct _SerializableGeeHashMapTestSmallBagHashMapClass SerializableGeeHashMapTestSmallBagHashMapClass;

#define SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SMALL_BAG (serializable_gee_hash_map_test_small_bag_get_type ())
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SMALL_BAG, SerializableGeeHashMapTestSmallBag))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SMALL_BAG, SerializableGeeHashMapTestSmallBagClass))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_IS_SMALL_BAG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SMALL_BAG))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_IS_SMALL_BAG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SMALL_BAG))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SMALL_BAG, SerializableGeeHashMapTestSmallBagClass))

typedef struct _SerializableGeeHashMapTestSmallBag SerializableGeeHashMapTestSmallBag;
typedef struct _SerializableGeeHashMapTestSmallBagClass SerializableGeeHashMapTestSmallBagClass;

#define SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_TYPE_HASH_MAP (serializable_gee_hash_map_test_ball_hash_map_get_type ())
#define SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_HASH_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_TYPE_HASH_MAP, SerializableGeeHashMapTestBallHashMap))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_HASH_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_TYPE_HASH_MAP, SerializableGeeHashMapTestBallHashMapClass))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_IS_HASH_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_TYPE_HASH_MAP))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_IS_HASH_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_TYPE_HASH_MAP))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_HASH_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_TYPE_HASH_MAP, SerializableGeeHashMapTestBallHashMapClass))

typedef struct _SerializableGeeHashMapTestBallHashMap SerializableGeeHashMapTestBallHashMap;
typedef struct _SerializableGeeHashMapTestBallHashMapClass SerializableGeeHashMapTestBallHashMapClass;

#define SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL (serializable_gee_hash_map_test_ball_get_type ())
#define SERIALIZABLE_GEE_HASH_MAP_TEST_BALL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL, SerializableGeeHashMapTestBall))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL, SerializableGeeHashMapTestBallClass))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_IS_BALL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_IS_BALL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL, SerializableGeeHashMapTestBallClass))

typedef struct _SerializableGeeHashMapTestBall SerializableGeeHashMapTestBall;
typedef struct _SerializableGeeHashMapTestBallClass SerializableGeeHashMapTestBallClass;

#define SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL_FILL (serializable_gee_hash_map_test_ball_fill_get_type ())
#define SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_FILL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL_FILL, SerializableGeeHashMapTestBallFill))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_FILL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL_FILL, SerializableGeeHashMapTestBallFillClass))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_IS_BALL_FILL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL_FILL))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_IS_BALL_FILL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL_FILL))
#define SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_FILL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL_FILL, SerializableGeeHashMapTestBallFillClass))

typedef struct _SerializableGeeHashMapTestBallFill SerializableGeeHashMapTestBallFill;
typedef struct _SerializableGeeHashMapTestBallFillClass SerializableGeeHashMapTestBallFillClass;
typedef struct _SerializableGeeHashMapTestSpacePrivate SerializableGeeHashMapTestSpacePrivate;
typedef struct _SerializableGeeHashMapTestSpaceCollectionPrivate SerializableGeeHashMapTestSpaceCollectionPrivate;
typedef struct _SerializableGeeHashMapTestSpaceContainerPrivate SerializableGeeHashMapTestSpaceContainerPrivate;
typedef struct _SerializableGeeHashMapTestBallFillPrivate SerializableGeeHashMapTestBallFillPrivate;
typedef struct _SerializableGeeHashMapTestBallPrivate SerializableGeeHashMapTestBallPrivate;
typedef struct _SerializableGeeHashMapTestBallHashMapPrivate SerializableGeeHashMapTestBallHashMapPrivate;
typedef struct _SerializableGeeHashMapTestSmallBagPrivate SerializableGeeHashMapTestSmallBagPrivate;
typedef struct _SerializableGeeHashMapTestSmallBagHashMapPrivate SerializableGeeHashMapTestSmallBagHashMapPrivate;
typedef struct _SerializableGeeHashMapTestBigBagPrivate SerializableGeeHashMapTestBigBagPrivate;
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _GXmlTest {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GXmlTestPrivate * priv;
};

struct _GXmlTestClass {
	GTypeClass parent_class;
	void (*finalize) (GXmlTest *self);
};

struct _SerializableGeeHashMapTest {
	GXmlTest parent_instance;
	SerializableGeeHashMapTestPrivate * priv;
};

struct _SerializableGeeHashMapTestClass {
	GXmlTestClass parent_class;
};

struct _SerializableGeeHashMapTestSpace {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeHashMapTestSpacePrivate * priv;
};

struct _SerializableGeeHashMapTestSpaceClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeHashMapTestSpacePrivate {
	gchar* _name;
};

struct _SerializableGeeHashMapTestSpaceCollection {
	GXmlSerializableHashMap parent_instance;
	SerializableGeeHashMapTestSpaceCollectionPrivate * priv;
};

struct _SerializableGeeHashMapTestSpaceCollectionClass {
	GXmlSerializableHashMapClass parent_class;
};

struct _SerializableGeeHashMapTestSpaceContainer {
	GXmlSerializableContainer parent_instance;
	SerializableGeeHashMapTestSpaceContainerPrivate * priv;
};

struct _SerializableGeeHashMapTestSpaceContainerClass {
	GXmlSerializableContainerClass parent_class;
};

struct _SerializableGeeHashMapTestSpaceContainerPrivate {
	gchar* _owner;
	SerializableGeeHashMapTestSpaceCollection* _storage;
};

struct _SerializableGeeHashMapTestBallFill {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeHashMapTestBallFillPrivate * priv;
};

struct _SerializableGeeHashMapTestBallFillClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeHashMapTestBallFillPrivate {
	gchar* _name;
};

struct _SerializableGeeHashMapTestBall {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeHashMapTestBallPrivate * priv;
};

struct _SerializableGeeHashMapTestBallClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeHashMapTestBallPrivate {
	gchar* _name;
	SerializableGeeHashMapTestBallFill* _ballfill;
};

struct _SerializableGeeHashMapTestBallHashMap {
	GXmlSerializableHashMap parent_instance;
	SerializableGeeHashMapTestBallHashMapPrivate * priv;
};

struct _SerializableGeeHashMapTestBallHashMapClass {
	GXmlSerializableHashMapClass parent_class;
};

struct _SerializableGeeHashMapTestSmallBag {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeHashMapTestSmallBagPrivate * priv;
};

struct _SerializableGeeHashMapTestSmallBagClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeHashMapTestSmallBagPrivate {
	gchar* _name;
	SerializableGeeHashMapTestBallHashMap* _balls;
};

struct _SerializableGeeHashMapTestSmallBagHashMap {
	GXmlSerializableHashMap parent_instance;
	SerializableGeeHashMapTestSmallBagHashMapPrivate * priv;
};

struct _SerializableGeeHashMapTestSmallBagHashMapClass {
	GXmlSerializableHashMapClass parent_class;
};

struct _SerializableGeeHashMapTestBigBag {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeHashMapTestBigBagPrivate * priv;
};

struct _SerializableGeeHashMapTestBigBagClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeHashMapTestBigBagPrivate {
	gchar* _name;
	SerializableGeeHashMapTestSmallBagHashMap* _bags;
};


static gpointer serializable_gee_hash_map_test_parent_class = NULL;
static gpointer serializable_gee_hash_map_test_space_parent_class = NULL;
static gpointer serializable_gee_hash_map_test_space_collection_parent_class = NULL;
static GXmlSerializableMapKeyIface* serializable_gee_hash_map_test_space_gxml_serializable_map_key_parent_iface = NULL;
static gpointer serializable_gee_hash_map_test_space_container_parent_class = NULL;
static gpointer serializable_gee_hash_map_test_ball_fill_parent_class = NULL;
static gpointer serializable_gee_hash_map_test_ball_parent_class = NULL;
static gpointer serializable_gee_hash_map_test_ball_hash_map_parent_class = NULL;
static GXmlSerializableMapKeyIface* serializable_gee_hash_map_test_ball_gxml_serializable_map_key_parent_iface = NULL;
static gpointer serializable_gee_hash_map_test_small_bag_parent_class = NULL;
static gpointer serializable_gee_hash_map_test_small_bag_hash_map_parent_class = NULL;
static GXmlSerializableMapKeyIface* serializable_gee_hash_map_test_small_bag_gxml_serializable_map_key_parent_iface = NULL;
static gpointer serializable_gee_hash_map_test_big_bag_parent_class = NULL;

gpointer gxml_test_ref (gpointer instance);
void gxml_test_unref (gpointer instance);
GParamSpec* param_spec_gxml_test (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_gxml_test (GValue* value, gpointer v_object);
void value_take_gxml_test (GValue* value, gpointer v_object);
gpointer value_get_gxml_test (const GValue* value);
GType gxml_test_get_type (void) G_GNUC_CONST;
GType serializable_gee_hash_map_test_get_type (void) G_GNUC_CONST;
enum  {
	SERIALIZABLE_GEE_HASH_MAP_TEST_DUMMY_PROPERTY
};
void serializable_gee_hash_map_test_add_tests (void);
static void __lambda46_ (void);
static GType serializable_gee_hash_map_test_space_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeHashMapTestSpace* serializable_gee_hash_map_test_space_new_named (const gchar* name);
static SerializableGeeHashMapTestSpace* serializable_gee_hash_map_test_space_construct_named (GType object_type, const gchar* name);
static const gchar* serializable_gee_hash_map_test_space_get_name (SerializableGeeHashMapTestSpace* self);
static void ___lambda46__gtest_func (void);
static void __lambda47_ (void);
static void serializable_gee_hash_map_test_space_set_value (SerializableGeeHashMapTestSpace* self, const gchar* str);
static void ___lambda47__gtest_func (void);
static void __lambda48_ (void);
static void ___lambda48__gtest_func (void);
static void __lambda49_ (void);
static GType serializable_gee_hash_map_test_space_container_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeHashMapTestSpaceContainer* serializable_gee_hash_map_test_space_container_new (void);
static SerializableGeeHashMapTestSpaceContainer* serializable_gee_hash_map_test_space_container_construct (GType object_type);
static const gchar* serializable_gee_hash_map_test_space_container_get_owner (SerializableGeeHashMapTestSpaceContainer* self);
GType serializable_gee_hash_map_test_space_collection_get_type (void) G_GNUC_CONST;
static SerializableGeeHashMapTestSpaceCollection* serializable_gee_hash_map_test_space_container_get_storage (SerializableGeeHashMapTestSpaceContainer* self);
static void ___lambda49__gtest_func (void);
static void __lambda50_ (void);
static SerializableGeeHashMapTestSpaceCollection* serializable_gee_hash_map_test_space_collection_new (void);
static SerializableGeeHashMapTestSpaceCollection* serializable_gee_hash_map_test_space_collection_construct (GType object_type);
static void serializable_gee_hash_map_test_space_container_set_storage (SerializableGeeHashMapTestSpaceContainer* self, SerializableGeeHashMapTestSpaceCollection* value);
static void ___lambda50__gtest_func (void);
static void __lambda51_ (void);
static gchar* serializable_gee_hash_map_test_space_get_value (SerializableGeeHashMapTestSpace* self);
static void ___lambda51__gtest_func (void);
static void __lambda52_ (void);
GType book_store_get_type (void) G_GNUC_CONST;
BookStore* book_store_new (void);
BookStore* book_store_construct (GType object_type);
const gchar* book_store_get_name (BookStore* self);
GType book_array_get_type (void) G_GNUC_CONST;
BookArray* book_store_get_books (BookStore* self);
GType book_get_type (void) G_GNUC_CONST;
GType name_get_type (void) G_GNUC_CONST;
Name* book_get_name (Book* self);
gchar* name_get_name (Name* self);
const gchar* book_get_year (Book* self);
GType authors_get_type (void) G_GNUC_CONST;
Authors* book_get_authors (Book* self);
GType author_array_get_type (void) G_GNUC_CONST;
AuthorArray* authors_get_array (Authors* self);
GType category_map_get_type (void) G_GNUC_CONST;
CategoryMap* book_get_categories (Book* self);
GType category_get_type (void) G_GNUC_CONST;
const gchar* category_get_name (Category* self);
static void ___lambda52__gtest_func (void);
static void __lambda53_ (void);
static GType serializable_gee_hash_map_test_big_bag_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeHashMapTestBigBag* serializable_gee_hash_map_test_big_bag_new (void);
static SerializableGeeHashMapTestBigBag* serializable_gee_hash_map_test_big_bag_construct (GType object_type);
GType serializable_gee_hash_map_test_small_bag_hash_map_get_type (void) G_GNUC_CONST;
static SerializableGeeHashMapTestSmallBagHashMap* serializable_gee_hash_map_test_big_bag_get_bags (SerializableGeeHashMapTestBigBag* self);
static GType serializable_gee_hash_map_test_small_bag_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeHashMapTestSmallBag* serializable_gee_hash_map_test_small_bag_new (void);
static SerializableGeeHashMapTestSmallBag* serializable_gee_hash_map_test_small_bag_construct (GType object_type);
static void serializable_gee_hash_map_test_small_bag_set_name (SerializableGeeHashMapTestSmallBag* self, const gchar* value);
GType serializable_gee_hash_map_test_ball_hash_map_get_type (void) G_GNUC_CONST;
static SerializableGeeHashMapTestBallHashMap* serializable_gee_hash_map_test_small_bag_get_balls (SerializableGeeHashMapTestSmallBag* self);
static GType serializable_gee_hash_map_test_ball_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeHashMapTestBall* serializable_gee_hash_map_test_ball_new (void);
static SerializableGeeHashMapTestBall* serializable_gee_hash_map_test_ball_construct (GType object_type);
static void serializable_gee_hash_map_test_ball_set_name (SerializableGeeHashMapTestBall* self, const gchar* value);
static SerializableGeeHashMapTestBallFill* serializable_gee_hash_map_test_ball_fill_new (void);
static SerializableGeeHashMapTestBallFill* serializable_gee_hash_map_test_ball_fill_construct (GType object_type);
static GType serializable_gee_hash_map_test_ball_fill_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static void serializable_gee_hash_map_test_ball_set_ballfill (SerializableGeeHashMapTestBall* self, SerializableGeeHashMapTestBallFill* value);
static SerializableGeeHashMapTestBallFill* serializable_gee_hash_map_test_ball_get_ballfill (SerializableGeeHashMapTestBall* self);
static void serializable_gee_hash_map_test_ball_fill_set_text (SerializableGeeHashMapTestBallFill* self, const gchar* txt);
static const gchar* serializable_gee_hash_map_test_ball_get_name (SerializableGeeHashMapTestBall* self);
static const gchar* serializable_gee_hash_map_test_small_bag_get_name (SerializableGeeHashMapTestSmallBag* self);
static void ___lambda53__gtest_func (void);
SerializableGeeHashMapTest* serializable_gee_hash_map_test_new (void);
SerializableGeeHashMapTest* serializable_gee_hash_map_test_construct (GType object_type);
GXmlTest* gxml_test_new (void);
GXmlTest* gxml_test_construct (GType object_type);
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE, SerializableGeeHashMapTestSpacePrivate))
enum  {
	SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_NAME
};
static gchar* serializable_gee_hash_map_test_space_real_get_map_key (GXmlSerializableMapKey* base);
static void serializable_gee_hash_map_test_space_set_name (SerializableGeeHashMapTestSpace* self, const gchar* value);
static gboolean serializable_gee_hash_map_test_space_real_serialize_use_xml_node_value (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_hash_map_test_space_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_hash_map_test_space_real_to_string (GXmlSerializableObjectModel* base);
static SerializableGeeHashMapTestSpace* serializable_gee_hash_map_test_space_new (void);
static SerializableGeeHashMapTestSpace* serializable_gee_hash_map_test_space_construct (GType object_type);
enum  {
	SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_COLLECTION_DUMMY_PROPERTY
};
static void serializable_gee_hash_map_test_space_finalize (GObject* obj);
static void _vala_serializable_gee_hash_map_test_space_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_hash_map_test_space_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_CONTAINER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE_CONTAINER, SerializableGeeHashMapTestSpaceContainerPrivate))
enum  {
	SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_CONTAINER_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_CONTAINER_OWNER,
	SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_CONTAINER_STORAGE
};
static gchar* serializable_gee_hash_map_test_space_container_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_hash_map_test_space_container_real_to_string (GXmlSerializableObjectModel* base);
static void serializable_gee_hash_map_test_space_container_real_init_containers (GXmlSerializableContainer* base);
static void serializable_gee_hash_map_test_space_container_set_owner (SerializableGeeHashMapTestSpaceContainer* self, const gchar* value);
static void serializable_gee_hash_map_test_space_container_finalize (GObject* obj);
static void _vala_serializable_gee_hash_map_test_space_container_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_hash_map_test_space_container_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_FILL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL_FILL, SerializableGeeHashMapTestBallFillPrivate))
enum  {
	SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_FILL_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_FILL_NAME
};
static gchar* serializable_gee_hash_map_test_ball_fill_real_to_string (GXmlSerializableObjectModel* base);
static const gchar* serializable_gee_hash_map_test_ball_fill_get_name (SerializableGeeHashMapTestBallFill* self);
static gchar* serializable_gee_hash_map_test_ball_fill_real_node_name (GXmlSerializableObjectModel* base);
static gboolean serializable_gee_hash_map_test_ball_fill_real_serialize_use_xml_node_value (GXmlSerializableObjectModel* base);
static void serializable_gee_hash_map_test_ball_fill_set_name (SerializableGeeHashMapTestBallFill* self, const gchar* value);
static void serializable_gee_hash_map_test_ball_fill_finalize (GObject* obj);
static void _vala_serializable_gee_hash_map_test_ball_fill_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_hash_map_test_ball_fill_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL, SerializableGeeHashMapTestBallPrivate))
enum  {
	SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_NAME,
	SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_BALLFILL
};
static gchar* serializable_gee_hash_map_test_ball_real_to_string (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_hash_map_test_ball_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_hash_map_test_ball_real_get_map_key (GXmlSerializableMapKey* base);
enum  {
	SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_HASH_MAP_DUMMY_PROPERTY
};
static gboolean serializable_gee_hash_map_test_ball_hash_map_real_deserialize_proceed (GXmlSerializableHashMap* base);
static SerializableGeeHashMapTestBallHashMap* serializable_gee_hash_map_test_ball_hash_map_new (void);
static SerializableGeeHashMapTestBallHashMap* serializable_gee_hash_map_test_ball_hash_map_construct (GType object_type);
static void serializable_gee_hash_map_test_ball_finalize (GObject* obj);
static void _vala_serializable_gee_hash_map_test_ball_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_hash_map_test_ball_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SMALL_BAG, SerializableGeeHashMapTestSmallBagPrivate))
enum  {
	SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_NAME,
	SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_BALLS
};
static gchar* serializable_gee_hash_map_test_small_bag_real_to_string (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_hash_map_test_small_bag_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_hash_map_test_small_bag_real_get_map_key (GXmlSerializableMapKey* base);
static void serializable_gee_hash_map_test_small_bag_set_balls (SerializableGeeHashMapTestSmallBag* self, SerializableGeeHashMapTestBallHashMap* value);
enum  {
	SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_HASH_MAP_DUMMY_PROPERTY
};
static gboolean serializable_gee_hash_map_test_small_bag_hash_map_real_deserialize_proceed (GXmlSerializableHashMap* base);
static SerializableGeeHashMapTestSmallBagHashMap* serializable_gee_hash_map_test_small_bag_hash_map_new (void);
static SerializableGeeHashMapTestSmallBagHashMap* serializable_gee_hash_map_test_small_bag_hash_map_construct (GType object_type);
static void serializable_gee_hash_map_test_small_bag_finalize (GObject* obj);
static void _vala_serializable_gee_hash_map_test_small_bag_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_hash_map_test_small_bag_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_HASH_MAP_TEST_BIG_BAG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BIG_BAG, SerializableGeeHashMapTestBigBagPrivate))
enum  {
	SERIALIZABLE_GEE_HASH_MAP_TEST_BIG_BAG_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_HASH_MAP_TEST_BIG_BAG_NAME,
	SERIALIZABLE_GEE_HASH_MAP_TEST_BIG_BAG_BAGS
};
static gchar* serializable_gee_hash_map_test_big_bag_real_to_string (GXmlSerializableObjectModel* base);
static const gchar* serializable_gee_hash_map_test_big_bag_get_name (SerializableGeeHashMapTestBigBag* self);
static gchar* serializable_gee_hash_map_test_big_bag_real_node_name (GXmlSerializableObjectModel* base);
static void serializable_gee_hash_map_test_big_bag_set_name (SerializableGeeHashMapTestBigBag* self, const gchar* value);
static void serializable_gee_hash_map_test_big_bag_set_bags (SerializableGeeHashMapTestBigBag* self, SerializableGeeHashMapTestSmallBagHashMap* value);
static void serializable_gee_hash_map_test_big_bag_finalize (GObject* obj);
static void _vala_serializable_gee_hash_map_test_big_bag_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_hash_map_test_big_bag_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);


static void __lambda46_ (void) {
	GXmlSerializableHashMap* c = NULL;
	GXmlSerializableHashMap* _tmp0_ = NULL;
	SerializableGeeHashMapTestSpace* o1 = NULL;
	SerializableGeeHashMapTestSpace* _tmp1_ = NULL;
	SerializableGeeHashMapTestSpace* o2 = NULL;
	SerializableGeeHashMapTestSpace* _tmp2_ = NULL;
	GXmlSerializableHashMap* _tmp3_ = NULL;
	SerializableGeeHashMapTestSpace* _tmp4_ = NULL;
	const gchar* _tmp5_ = NULL;
	const gchar* _tmp6_ = NULL;
	SerializableGeeHashMapTestSpace* _tmp7_ = NULL;
	GXmlSerializableHashMap* _tmp8_ = NULL;
	SerializableGeeHashMapTestSpace* _tmp9_ = NULL;
	const gchar* _tmp10_ = NULL;
	const gchar* _tmp11_ = NULL;
	SerializableGeeHashMapTestSpace* _tmp12_ = NULL;
	gboolean found1 = FALSE;
	gboolean found2 = FALSE;
	gboolean _tmp29_ = FALSE;
	gboolean _tmp30_ = FALSE;
	gboolean _tmp31_ = FALSE;
	gboolean _tmp56_ = FALSE;
	gboolean _tmp57_ = FALSE;
#line 107 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = gxml_serializable_hash_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 107 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	c = _tmp0_;
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = serializable_gee_hash_map_test_space_new_named ("Big");
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	o1 = _tmp1_;
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp2_ = serializable_gee_hash_map_test_space_new_named ("Small");
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	o2 = _tmp2_;
#line 110 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp3_ = c;
#line 110 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp4_ = o1;
#line 110 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp5_ = serializable_gee_hash_map_test_space_get_name (_tmp4_);
#line 110 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp6_ = _tmp5_;
#line 110 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp7_ = o1;
#line 110 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	gee_abstract_map_set ((GeeAbstractMap*) _tmp3_, _tmp6_, _tmp7_);
#line 111 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp8_ = c;
#line 111 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp9_ = o2;
#line 111 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp10_ = serializable_gee_hash_map_test_space_get_name (_tmp9_);
#line 111 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp11_ = _tmp10_;
#line 111 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp12_ = o2;
#line 111 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	gee_abstract_map_set ((GeeAbstractMap*) _tmp8_, _tmp11_, _tmp12_);
#line 112 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	found1 = FALSE;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	found2 = FALSE;
#line 617 "SerializableGeeHashMapTest.c"
	{
		GeeIterator* _o_it = NULL;
		GXmlSerializableHashMap* _tmp13_ = NULL;
		GeeCollection* _tmp14_ = NULL;
		GeeCollection* _tmp15_ = NULL;
		GeeCollection* _tmp16_ = NULL;
		GeeIterator* _tmp17_ = NULL;
		GeeIterator* _tmp18_ = NULL;
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp13_ = c;
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp14_ = gee_abstract_map_get_values ((GeeAbstractMap*) _tmp13_);
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp15_ = _tmp14_;
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp16_ = _tmp15_;
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp17_ = gee_iterable_iterator ((GeeIterable*) _tmp16_);
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp18_ = _tmp17_;
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp16_);
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_o_it = _tmp18_;
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		while (TRUE) {
#line 644 "SerializableGeeHashMapTest.c"
			GeeIterator* _tmp19_ = NULL;
			gboolean _tmp20_ = FALSE;
			SerializableGeeHashMapTestSpace* o = NULL;
			GeeIterator* _tmp21_ = NULL;
			gpointer _tmp22_ = NULL;
			SerializableGeeHashMapTestSpace* _tmp23_ = NULL;
			const gchar* _tmp24_ = NULL;
			const gchar* _tmp25_ = NULL;
			SerializableGeeHashMapTestSpace* _tmp26_ = NULL;
			const gchar* _tmp27_ = NULL;
			const gchar* _tmp28_ = NULL;
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp19_ = _o_it;
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp20_ = gee_iterator_next (_tmp19_);
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			if (!_tmp20_) {
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				break;
#line 664 "SerializableGeeHashMapTest.c"
			}
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp21_ = _o_it;
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp22_ = gee_iterator_get (_tmp21_);
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			o = (SerializableGeeHashMapTestSpace*) _tmp22_;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp23_ = o;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp24_ = serializable_gee_hash_map_test_space_get_name (_tmp23_);
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp25_ = _tmp24_;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			if (g_strcmp0 (_tmp25_, "Big") == 0) {
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				found1 = TRUE;
#line 682 "SerializableGeeHashMapTest.c"
			}
#line 116 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp26_ = o;
#line 116 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp27_ = serializable_gee_hash_map_test_space_get_name (_tmp26_);
#line 116 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp28_ = _tmp27_;
#line 116 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			if (g_strcmp0 (_tmp28_, "Small") == 0) {
#line 116 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				found2 = TRUE;
#line 694 "SerializableGeeHashMapTest.c"
			}
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (o);
#line 698 "SerializableGeeHashMapTest.c"
		}
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_o_it);
#line 702 "SerializableGeeHashMapTest.c"
	}
#line 118 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp29_ = found1;
#line 118 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	if (!_tmp29_) {
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_message ("SerializableGeeHashMapTest.vala:119: %s", "Big is not found\n");
#line 120 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_assert_not_reached ();
#line 712 "SerializableGeeHashMapTest.c"
	}
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp30_ = found2;
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	if (!_tmp30_) {
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_message ("SerializableGeeHashMapTest.vala:123: %s", "Small is not found\n");
#line 124 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_assert_not_reached ();
#line 722 "SerializableGeeHashMapTest.c"
	}
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	found2 = FALSE;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp31_ = found2;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	found1 = _tmp31_;
#line 730 "SerializableGeeHashMapTest.c"
	{
		GeeIterator* _k_it = NULL;
		GXmlSerializableHashMap* _tmp32_ = NULL;
		GeeSet* _tmp33_ = NULL;
		GeeSet* _tmp34_ = NULL;
		GeeSet* _tmp35_ = NULL;
		GeeIterator* _tmp36_ = NULL;
		GeeIterator* _tmp37_ = NULL;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp32_ = c;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp33_ = gee_abstract_map_get_keys ((GeeAbstractMap*) _tmp32_);
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp34_ = _tmp33_;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp35_ = _tmp34_;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp36_ = gee_iterable_iterator ((GeeIterable*) _tmp35_);
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp37_ = _tmp36_;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp35_);
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_k_it = _tmp37_;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		while (TRUE) {
#line 757 "SerializableGeeHashMapTest.c"
			GeeIterator* _tmp38_ = NULL;
			gboolean _tmp39_ = FALSE;
			gchar* k = NULL;
			GeeIterator* _tmp40_ = NULL;
			gpointer _tmp41_ = NULL;
			GXmlSerializableHashMap* _tmp42_ = NULL;
			const gchar* _tmp43_ = NULL;
			gpointer _tmp44_ = NULL;
			SerializableGeeHashMapTestSpace* _tmp45_ = NULL;
			const gchar* _tmp46_ = NULL;
			const gchar* _tmp47_ = NULL;
			gboolean _tmp48_ = FALSE;
			GXmlSerializableHashMap* _tmp49_ = NULL;
			const gchar* _tmp50_ = NULL;
			gpointer _tmp51_ = NULL;
			SerializableGeeHashMapTestSpace* _tmp52_ = NULL;
			const gchar* _tmp53_ = NULL;
			const gchar* _tmp54_ = NULL;
			gboolean _tmp55_ = FALSE;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp38_ = _k_it;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp39_ = gee_iterator_next (_tmp38_);
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			if (!_tmp39_) {
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				break;
#line 785 "SerializableGeeHashMapTest.c"
			}
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp40_ = _k_it;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp41_ = gee_iterator_get (_tmp40_);
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			k = (gchar*) _tmp41_;
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp42_ = c;
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp43_ = k;
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp44_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp42_, _tmp43_);
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp45_ = (SerializableGeeHashMapTestSpace*) _tmp44_;
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp46_ = serializable_gee_hash_map_test_space_get_name (_tmp45_);
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp47_ = _tmp46_;
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp48_ = g_strcmp0 (_tmp47_, "Big") == 0;
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (_tmp45_);
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			if (_tmp48_) {
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				found1 = TRUE;
#line 813 "SerializableGeeHashMapTest.c"
			}
#line 129 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp49_ = c;
#line 129 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp50_ = k;
#line 129 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp51_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp49_, _tmp50_);
#line 129 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp52_ = (SerializableGeeHashMapTestSpace*) _tmp51_;
#line 129 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp53_ = serializable_gee_hash_map_test_space_get_name (_tmp52_);
#line 129 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp54_ = _tmp53_;
#line 129 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp55_ = g_strcmp0 (_tmp54_, "Small") == 0;
#line 129 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (_tmp52_);
#line 129 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			if (_tmp55_) {
#line 129 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				found2 = TRUE;
#line 835 "SerializableGeeHashMapTest.c"
			}
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_free0 (k);
#line 839 "SerializableGeeHashMapTest.c"
		}
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_k_it);
#line 843 "SerializableGeeHashMapTest.c"
	}
#line 131 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp56_ = found1;
#line 131 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	if (!_tmp56_) {
#line 132 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_message ("SerializableGeeHashMapTest.vala:132: %s", "Big key value is not found\n");
#line 133 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_assert_not_reached ();
#line 853 "SerializableGeeHashMapTest.c"
	}
#line 135 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp57_ = found2;
#line 135 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	if (!_tmp57_) {
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_message ("SerializableGeeHashMapTest.vala:136: %s", "Small key value is not found\n");
#line 137 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_assert_not_reached ();
#line 863 "SerializableGeeHashMapTest.c"
	}
#line 106 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_object_unref0 (o2);
#line 106 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_object_unref0 (o1);
#line 106 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_object_unref0 (c);
#line 871 "SerializableGeeHashMapTest.c"
}


static void ___lambda46__gtest_func (void) {
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	__lambda46_ ();
#line 878 "SerializableGeeHashMapTest.c"
}


static const gchar* string_to_string (const gchar* self) {
	const gchar* result = NULL;
#line 1412 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1413 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	result = self;
#line 1413 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	return result;
#line 890 "SerializableGeeHashMapTest.c"
}


static void __lambda47_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlSerializableHashMap* c = NULL;
		GXmlSerializableHashMap* _tmp0_ = NULL;
		SerializableGeeHashMapTestSpace* o1 = NULL;
		SerializableGeeHashMapTestSpace* _tmp1_ = NULL;
		SerializableGeeHashMapTestSpace* _tmp2_ = NULL;
		SerializableGeeHashMapTestSpace* o2 = NULL;
		SerializableGeeHashMapTestSpace* _tmp3_ = NULL;
		SerializableGeeHashMapTestSpace* _tmp4_ = NULL;
		GXmlSerializableHashMap* _tmp5_ = NULL;
		SerializableGeeHashMapTestSpace* _tmp6_ = NULL;
		const gchar* _tmp7_ = NULL;
		const gchar* _tmp8_ = NULL;
		SerializableGeeHashMapTestSpace* _tmp9_ = NULL;
		GXmlSerializableHashMap* _tmp10_ = NULL;
		SerializableGeeHashMapTestSpace* _tmp11_ = NULL;
		const gchar* _tmp12_ = NULL;
		const gchar* _tmp13_ = NULL;
		SerializableGeeHashMapTestSpace* _tmp14_ = NULL;
		GXmlTDocument* doc = NULL;
		GXmlTDocument* _tmp15_ = NULL;
		GXmlNode* root = NULL;
		GXmlTDocument* _tmp16_ = NULL;
		GXmlNode* _tmp17_ = NULL;
		GXmlTDocument* _tmp18_ = NULL;
		GeeBidirList* _tmp19_ = NULL;
		GeeBidirList* _tmp20_ = NULL;
		GeeBidirList* _tmp21_ = NULL;
		GXmlNode* _tmp22_ = NULL;
		GXmlSerializableHashMap* _tmp23_ = NULL;
		GXmlNode* _tmp24_ = NULL;
		GXmlNode* _tmp25_ = NULL;
		GXmlNode* _tmp26_ = NULL;
		GXmlNode* _tmp27_ = NULL;
		GeeBidirList* _tmp28_ = NULL;
		GeeBidirList* _tmp29_ = NULL;
		GeeBidirList* _tmp30_ = NULL;
		gint _tmp31_ = 0;
		gint _tmp32_ = 0;
		gboolean found1 = FALSE;
		gboolean found2 = FALSE;
		gboolean _tmp103_ = FALSE;
		gboolean _tmp109_ = FALSE;
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp0_ = gxml_serializable_hash_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		c = _tmp0_;
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp1_ = serializable_gee_hash_map_test_space_new_named ("Big");
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		o1 = _tmp1_;
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp2_ = o1;
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		serializable_gee_hash_map_test_space_set_value (_tmp2_, "FAKE TEXT");
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp3_ = serializable_gee_hash_map_test_space_new_named ("Small");
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		o2 = _tmp3_;
#line 147 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp4_ = o2;
#line 147 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		serializable_gee_hash_map_test_space_set_value (_tmp4_, "FAKE TEXT");
#line 148 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp5_ = c;
#line 148 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp6_ = o1;
#line 148 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp7_ = serializable_gee_hash_map_test_space_get_name (_tmp6_);
#line 148 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp8_ = _tmp7_;
#line 148 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp9_ = o1;
#line 148 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		gee_abstract_map_set ((GeeAbstractMap*) _tmp5_, _tmp8_, _tmp9_);
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp10_ = c;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp11_ = o2;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp12_ = serializable_gee_hash_map_test_space_get_name (_tmp11_);
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp13_ = _tmp12_;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp14_ = o2;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		gee_abstract_map_set ((GeeAbstractMap*) _tmp10_, _tmp13_, _tmp14_);
#line 150 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp15_ = gxml_tdocument_new ();
#line 150 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		doc = _tmp15_;
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp16_ = doc;
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp17_ = gxml_document_create_element ((GXmlDocument*) _tmp16_, "root", &_inner_error_);
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		root = _tmp17_;
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (doc);
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (o2);
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (o1);
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (c);
#line 1003 "SerializableGeeHashMapTest.c"
			goto __catch87_g_error;
		}
#line 152 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp18_ = doc;
#line 152 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp19_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp18_);
#line 152 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp20_ = _tmp19_;
#line 152 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp21_ = _tmp20_;
#line 152 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp22_ = root;
#line 152 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		gee_collection_add ((GeeCollection*) _tmp21_, _tmp22_);
#line 152 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp21_);
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp23_ = c;
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp24_ = root;
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp25_ = gxml_serializable_hash_map_serialize (_tmp23_, _tmp24_, &_inner_error_);
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp26_ = _tmp25_;
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp26_);
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (root);
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (doc);
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (o2);
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (o1);
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (c);
#line 1042 "SerializableGeeHashMapTest.c"
			goto __catch87_g_error;
		}
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp27_ = root;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp28_ = gxml_node_get_children_nodes (_tmp27_);
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp29_ = _tmp28_;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp30_ = _tmp29_;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp31_ = gee_collection_get_size ((GeeCollection*) _tmp30_);
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp32_ = _tmp31_;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp32_ > 0, "root.children_nodes.size > 0");
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp30_);
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		found1 = FALSE;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		found2 = FALSE;
#line 1065 "SerializableGeeHashMapTest.c"
		{
			GeeBidirList* _n_list = NULL;
			GXmlNode* _tmp33_ = NULL;
			GeeBidirList* _tmp34_ = NULL;
			GeeBidirList* _tmp35_ = NULL;
			gint _n_size = 0;
			GeeBidirList* _tmp36_ = NULL;
			gint _tmp37_ = 0;
			gint _tmp38_ = 0;
			gint _n_index = 0;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp33_ = root;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp34_ = gxml_node_get_children_nodes (_tmp33_);
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp35_ = _tmp34_;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_n_list = _tmp35_;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp36_ = _n_list;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp37_ = gee_collection_get_size ((GeeCollection*) _tmp36_);
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp38_ = _tmp37_;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_n_size = _tmp38_;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_n_index = -1;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			while (TRUE) {
#line 1096 "SerializableGeeHashMapTest.c"
				gint _tmp39_ = 0;
				gint _tmp40_ = 0;
				gint _tmp41_ = 0;
				GXmlNode* n = NULL;
				GeeBidirList* _tmp42_ = NULL;
				gint _tmp43_ = 0;
				gpointer _tmp44_ = NULL;
				gboolean _tmp45_ = FALSE;
				GXmlNode* _tmp46_ = NULL;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp39_ = _n_index;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_n_index = _tmp39_ + 1;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp40_ = _n_index;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp41_ = _n_size;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				if (!(_tmp40_ < _tmp41_)) {
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					break;
#line 1118 "SerializableGeeHashMapTest.c"
				}
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp42_ = _n_list;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp43_ = _n_index;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp44_ = gee_list_get ((GeeList*) _tmp42_, _tmp43_);
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				n = (GXmlNode*) _tmp44_;
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp46_ = n;
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp46_, GXML_TYPE_ELEMENT)) {
#line 1132 "SerializableGeeHashMapTest.c"
					GXmlNode* _tmp47_ = NULL;
					gchar* _tmp48_ = NULL;
					gchar* _tmp49_ = NULL;
					gchar* _tmp50_ = NULL;
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp47_ = n;
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp48_ = gxml_node_get_name (_tmp47_);
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp49_ = _tmp48_;
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp50_ = _tmp49_;
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp45_ = g_strcmp0 (_tmp50_, "space") == 0;
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_g_free0 (_tmp50_);
#line 1149 "SerializableGeeHashMapTest.c"
				} else {
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp45_ = FALSE;
#line 1153 "SerializableGeeHashMapTest.c"
				}
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				if (_tmp45_) {
#line 1157 "SerializableGeeHashMapTest.c"
					GXmlNode* name = NULL;
					GXmlNode* _tmp51_ = NULL;
					GeeMap* _tmp52_ = NULL;
					GeeMap* _tmp53_ = NULL;
					GeeMap* _tmp54_ = NULL;
					gpointer _tmp55_ = NULL;
					GXmlNode* _tmp56_ = NULL;
					GXmlNode* _tmp57_ = NULL;
					GXmlNode* _tmp68_ = NULL;
					GeeBidirList* _tmp69_ = NULL;
					GeeBidirList* _tmp70_ = NULL;
					GeeBidirList* _tmp71_ = NULL;
					gint _tmp72_ = 0;
					gint _tmp73_ = 0;
					gboolean _tmp74_ = FALSE;
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp51_ = n;
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp52_ = gxml_node_get_attrs (_tmp51_);
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp53_ = _tmp52_;
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp54_ = _tmp53_;
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp55_ = gee_map_get (_tmp54_, "name");
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp56_ = (GXmlNode*) _tmp55_;
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_g_object_unref0 (_tmp54_);
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					name = _tmp56_;
#line 160 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp57_ = name;
#line 160 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					if (_tmp57_ != NULL) {
#line 1193 "SerializableGeeHashMapTest.c"
						GXmlNode* _tmp58_ = NULL;
						gchar* _tmp59_ = NULL;
						gchar* _tmp60_ = NULL;
						gchar* _tmp61_ = NULL;
						gboolean _tmp62_ = FALSE;
						GXmlNode* _tmp63_ = NULL;
						gchar* _tmp64_ = NULL;
						gchar* _tmp65_ = NULL;
						gchar* _tmp66_ = NULL;
						gboolean _tmp67_ = FALSE;
#line 161 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_tmp58_ = name;
#line 161 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_tmp59_ = gxml_node_get_value (_tmp58_);
#line 161 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_tmp60_ = _tmp59_;
#line 161 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_tmp61_ = _tmp60_;
#line 161 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_tmp62_ = g_strcmp0 (_tmp61_, "Big") == 0;
#line 161 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_g_free0 (_tmp61_);
#line 161 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						if (_tmp62_) {
#line 161 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
							found1 = TRUE;
#line 1220 "SerializableGeeHashMapTest.c"
						}
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_tmp63_ = name;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_tmp64_ = gxml_node_get_value (_tmp63_);
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_tmp65_ = _tmp64_;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_tmp66_ = _tmp65_;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_tmp67_ = g_strcmp0 (_tmp66_, "Small") == 0;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_g_free0 (_tmp66_);
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						if (_tmp67_) {
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
							found2 = TRUE;
#line 1238 "SerializableGeeHashMapTest.c"
						}
					}
#line 164 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp68_ = n;
#line 164 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp69_ = gxml_node_get_children_nodes (_tmp68_);
#line 164 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp70_ = _tmp69_;
#line 164 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp71_ = _tmp70_;
#line 164 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp72_ = gee_collection_get_size ((GeeCollection*) _tmp71_);
#line 164 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp73_ = _tmp72_;
#line 164 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp74_ = _tmp73_ > 0;
#line 164 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_g_object_unref0 (_tmp71_);
#line 164 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					if (_tmp74_) {
#line 1259 "SerializableGeeHashMapTest.c"
						{
							GeeBidirList* _nd_list = NULL;
							GXmlNode* _tmp75_ = NULL;
							GeeBidirList* _tmp76_ = NULL;
							GeeBidirList* _tmp77_ = NULL;
							gint _nd_size = 0;
							GeeBidirList* _tmp78_ = NULL;
							gint _tmp79_ = 0;
							gint _tmp80_ = 0;
							gint _nd_index = 0;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
							_tmp75_ = n;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
							_tmp76_ = gxml_node_get_children_nodes (_tmp75_);
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
							_tmp77_ = _tmp76_;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
							_nd_list = _tmp77_;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
							_tmp78_ = _nd_list;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
							_tmp79_ = gee_collection_get_size ((GeeCollection*) _tmp78_);
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
							_tmp80_ = _tmp79_;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
							_nd_size = _tmp80_;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
							_nd_index = -1;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
							while (TRUE) {
#line 1290 "SerializableGeeHashMapTest.c"
								gint _tmp81_ = 0;
								gint _tmp82_ = 0;
								gint _tmp83_ = 0;
								GXmlNode* nd = NULL;
								GeeBidirList* _tmp84_ = NULL;
								gint _tmp85_ = 0;
								gpointer _tmp86_ = NULL;
								GXmlNode* _tmp87_ = NULL;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp81_ = _nd_index;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_nd_index = _tmp81_ + 1;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp82_ = _nd_index;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp83_ = _nd_size;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								if (!(_tmp82_ < _tmp83_)) {
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
									break;
#line 1311 "SerializableGeeHashMapTest.c"
								}
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp84_ = _nd_list;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp85_ = _nd_index;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp86_ = gee_list_get ((GeeList*) _tmp84_, _tmp85_);
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								nd = (GXmlNode*) _tmp86_;
#line 166 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp87_ = nd;
#line 166 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp87_, GXML_TYPE_TEXT)) {
#line 1325 "SerializableGeeHashMapTest.c"
									GXmlNode* _tmp88_ = NULL;
									gchar* _tmp89_ = NULL;
									gchar* _tmp90_ = NULL;
									gchar* _tmp91_ = NULL;
									gboolean _tmp92_ = FALSE;
#line 167 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
									_tmp88_ = nd;
#line 167 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
									_tmp89_ = gxml_node_get_value (_tmp88_);
#line 167 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
									_tmp90_ = _tmp89_;
#line 167 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
									_tmp91_ = _tmp90_;
#line 167 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
									_tmp92_ = g_strcmp0 (_tmp91_, "FAKE TEXT") != 0;
#line 167 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
									_g_free0 (_tmp91_);
#line 167 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
									if (_tmp92_) {
#line 1345 "SerializableGeeHashMapTest.c"
										GXmlNode* _tmp93_ = NULL;
										gchar* _tmp94_ = NULL;
										gchar* _tmp95_ = NULL;
										gchar* _tmp96_ = NULL;
										const gchar* _tmp97_ = NULL;
										GXmlTDocument* _tmp98_ = NULL;
										gchar* _tmp99_ = NULL;
										gchar* _tmp100_ = NULL;
										gchar* _tmp101_ = NULL;
										gchar* _tmp102_ = NULL;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
										_tmp93_ = nd;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
										_tmp94_ = gxml_node_get_value (_tmp93_);
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
										_tmp95_ = _tmp94_;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
										_tmp96_ = _tmp95_;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
										_tmp97_ = string_to_string (_tmp96_);
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
										_tmp98_ = doc;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
										_tmp99_ = gxml_tnode_to_string ((GXmlTNode*) _tmp98_);
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
										_tmp100_ = _tmp99_;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
										_tmp101_ = g_strconcat ("ERROR: node content don't much. Expected 'FAKE TEXT', got: ", _tmp97_, "\n", _tmp100_, "\n", NULL);
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
										_tmp102_ = _tmp101_;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
										g_message ("SerializableGeeHashMapTest.vala:168: %s", _tmp102_);
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
										_g_free0 (_tmp102_);
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
										_g_free0 (_tmp100_);
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
										_g_free0 (_tmp96_);
#line 169 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
										g_assert_not_reached ();
#line 1386 "SerializableGeeHashMapTest.c"
									}
								}
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_g_object_unref0 (nd);
#line 1391 "SerializableGeeHashMapTest.c"
							}
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
							_g_object_unref0 (_nd_list);
#line 1395 "SerializableGeeHashMapTest.c"
						}
					}
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_g_object_unref0 (name);
#line 1400 "SerializableGeeHashMapTest.c"
				}
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_g_object_unref0 (n);
#line 1404 "SerializableGeeHashMapTest.c"
			}
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (_n_list);
#line 1408 "SerializableGeeHashMapTest.c"
		}
#line 176 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp103_ = found1;
#line 176 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (!_tmp103_) {
#line 1414 "SerializableGeeHashMapTest.c"
			GXmlTDocument* _tmp104_ = NULL;
			gchar* _tmp105_ = NULL;
			gchar* _tmp106_ = NULL;
			gchar* _tmp107_ = NULL;
			gchar* _tmp108_ = NULL;
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp104_ = doc;
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp105_ = gxml_tnode_to_string ((GXmlTNode*) _tmp104_);
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp106_ = _tmp105_;
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp107_ = g_strconcat ("ERROR: Big space node is not found\n", _tmp106_, "\n", NULL);
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp108_ = _tmp107_;
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			g_message ("SerializableGeeHashMapTest.vala:177: %s", _tmp108_);
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_free0 (_tmp108_);
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_free0 (_tmp106_);
#line 178 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			g_assert_not_reached ();
#line 1438 "SerializableGeeHashMapTest.c"
		}
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp109_ = found2;
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (!_tmp109_) {
#line 1444 "SerializableGeeHashMapTest.c"
			GXmlTDocument* _tmp110_ = NULL;
			gchar* _tmp111_ = NULL;
			gchar* _tmp112_ = NULL;
			gchar* _tmp113_ = NULL;
			gchar* _tmp114_ = NULL;
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp110_ = doc;
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp111_ = gxml_tnode_to_string ((GXmlTNode*) _tmp110_);
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp112_ = _tmp111_;
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp113_ = g_strconcat ("ERROR: Small space node is not found\n", _tmp112_, "\n", NULL);
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp114_ = _tmp113_;
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			g_message ("SerializableGeeHashMapTest.vala:181: %s", _tmp114_);
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_free0 (_tmp114_);
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_free0 (_tmp112_);
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			g_assert_not_reached ();
#line 1468 "SerializableGeeHashMapTest.c"
		}
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (root);
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (doc);
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (o2);
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (o1);
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (c);
#line 1480 "SerializableGeeHashMapTest.c"
	}
	goto __finally87;
	__catch87_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp115_ = NULL;
		const gchar* _tmp116_ = NULL;
		gchar* _tmp117_ = NULL;
		gchar* _tmp118_ = NULL;
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		e = _inner_error_;
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_inner_error_ = NULL;
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp115_ = e->message;
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp116_ = string_to_string (_tmp115_);
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp117_ = g_strconcat ("ERROR: ", _tmp116_, NULL);
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp118_ = _tmp117_;
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_message ("SerializableGeeHashMapTest.vala:186: %s", _tmp118_);
#line 186 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp118_);
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_assert_not_reached ();
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_error_free0 (e);
#line 1510 "SerializableGeeHashMapTest.c"
	}
	__finally87:
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_clear_error (&_inner_error_);
#line 142 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		return;
#line 1521 "SerializableGeeHashMapTest.c"
	}
}


static void ___lambda47__gtest_func (void) {
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	__lambda47_ ();
#line 1529 "SerializableGeeHashMapTest.c"
}


static void __lambda48_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		GXmlSerializableHashMap* c = NULL;
		GXmlSerializableHashMap* _tmp1_ = NULL;
		GXmlSerializableHashMap* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		GXmlNode* _tmp4_ = NULL;
		GXmlNode* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		GXmlSerializableHashMap* _tmp7_ = NULL;
		gint _tmp8_ = 0;
		gint _tmp9_ = 0;
		gboolean found1 = FALSE;
		gboolean found2 = FALSE;
		gboolean _tmp41_ = FALSE;
		gboolean _tmp42_ = FALSE;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp0_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n" \
"  <root><space name=\"Big\"/><space name=\"Small\"/></root>", 0, &_inner_error_);
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		doc = _tmp0_;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1558 "SerializableGeeHashMapTest.c"
			goto __catch88_g_error;
		}
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp1_ = gxml_serializable_hash_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		c = _tmp1_;
#line 196 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp2_ = c;
#line 196 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp3_ = doc;
#line 196 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp4_ = gxml_document_get_root ((GXmlDocument*) _tmp3_);
#line 196 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp5_ = _tmp4_;
#line 196 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp6_ = _tmp5_;
#line 196 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		gxml_serializable_hash_map_deserialize (_tmp2_, _tmp6_, &_inner_error_);
#line 196 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp6_);
#line 196 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 196 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (c);
#line 196 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (doc);
#line 1585 "SerializableGeeHashMapTest.c"
			goto __catch88_g_error;
		}
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp7_ = c;
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp8_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp7_);
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp9_ = _tmp8_;
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (_tmp9_ != 2) {
#line 1596 "SerializableGeeHashMapTest.c"
			GXmlSerializableHashMap* _tmp10_ = NULL;
			gint _tmp11_ = 0;
			gint _tmp12_ = 0;
			gchar* _tmp13_ = NULL;
			gchar* _tmp14_ = NULL;
			gchar* _tmp15_ = NULL;
			gchar* _tmp16_ = NULL;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp10_ = c;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp11_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp10_);
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp12_ = _tmp11_;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp13_ = g_strdup_printf ("%i", _tmp12_);
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp14_ = _tmp13_;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp15_ = g_strconcat ("ERROR: incorrect size must be 2 got: ", _tmp14_, "\n", NULL);
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp16_ = _tmp15_;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			g_message ("SerializableGeeHashMapTest.vala:198: %s", _tmp16_);
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_free0 (_tmp16_);
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_free0 (_tmp14_);
#line 199 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			g_assert_not_reached ();
#line 1626 "SerializableGeeHashMapTest.c"
		}
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		found1 = FALSE;
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		found2 = FALSE;
#line 1632 "SerializableGeeHashMapTest.c"
		{
			GeeIterator* _k_it = NULL;
			GXmlSerializableHashMap* _tmp17_ = NULL;
			GeeSet* _tmp18_ = NULL;
			GeeSet* _tmp19_ = NULL;
			GeeSet* _tmp20_ = NULL;
			GeeIterator* _tmp21_ = NULL;
			GeeIterator* _tmp22_ = NULL;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp17_ = c;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp18_ = gee_abstract_map_get_keys ((GeeAbstractMap*) _tmp17_);
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp19_ = _tmp18_;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp20_ = _tmp19_;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp21_ = gee_iterable_iterator ((GeeIterable*) _tmp20_);
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp22_ = _tmp21_;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (_tmp20_);
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_k_it = _tmp22_;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			while (TRUE) {
#line 1659 "SerializableGeeHashMapTest.c"
				GeeIterator* _tmp23_ = NULL;
				gboolean _tmp24_ = FALSE;
				gchar* k = NULL;
				GeeIterator* _tmp25_ = NULL;
				gpointer _tmp26_ = NULL;
				GXmlSerializableHashMap* _tmp27_ = NULL;
				const gchar* _tmp28_ = NULL;
				gpointer _tmp29_ = NULL;
				SerializableGeeHashMapTestSpace* _tmp30_ = NULL;
				const gchar* _tmp31_ = NULL;
				const gchar* _tmp32_ = NULL;
				gboolean _tmp33_ = FALSE;
				GXmlSerializableHashMap* _tmp34_ = NULL;
				const gchar* _tmp35_ = NULL;
				gpointer _tmp36_ = NULL;
				SerializableGeeHashMapTestSpace* _tmp37_ = NULL;
				const gchar* _tmp38_ = NULL;
				const gchar* _tmp39_ = NULL;
				gboolean _tmp40_ = FALSE;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp23_ = _k_it;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp24_ = gee_iterator_next (_tmp23_);
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				if (!_tmp24_) {
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					break;
#line 1687 "SerializableGeeHashMapTest.c"
				}
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp25_ = _k_it;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp26_ = gee_iterator_get (_tmp25_);
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				k = (gchar*) _tmp26_;
#line 204 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp27_ = c;
#line 204 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp28_ = k;
#line 204 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp29_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp27_, _tmp28_);
#line 204 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp30_ = (SerializableGeeHashMapTestSpace*) _tmp29_;
#line 204 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp31_ = serializable_gee_hash_map_test_space_get_name (_tmp30_);
#line 204 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp32_ = _tmp31_;
#line 204 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp33_ = g_strcmp0 (_tmp32_, "Big") == 0;
#line 204 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_g_object_unref0 (_tmp30_);
#line 204 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				if (_tmp33_) {
#line 204 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					found1 = TRUE;
#line 1715 "SerializableGeeHashMapTest.c"
				}
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp34_ = c;
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp35_ = k;
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp36_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp34_, _tmp35_);
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp37_ = (SerializableGeeHashMapTestSpace*) _tmp36_;
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp38_ = serializable_gee_hash_map_test_space_get_name (_tmp37_);
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp39_ = _tmp38_;
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp40_ = g_strcmp0 (_tmp39_, "Small") == 0;
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_g_object_unref0 (_tmp37_);
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				if (_tmp40_) {
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					found2 = TRUE;
#line 1737 "SerializableGeeHashMapTest.c"
				}
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_g_free0 (k);
#line 1741 "SerializableGeeHashMapTest.c"
			}
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (_k_it);
#line 1745 "SerializableGeeHashMapTest.c"
		}
#line 207 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp41_ = found1;
#line 207 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (!_tmp41_) {
#line 208 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			g_message ("SerializableGeeHashMapTest.vala:208: %s", "ERROR: Big key value is not found\n");
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			g_assert_not_reached ();
#line 1755 "SerializableGeeHashMapTest.c"
		}
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp42_ = found2;
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (!_tmp42_) {
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			g_message ("SerializableGeeHashMapTest.vala:212: %s", "ERROR: Small key value is not found\n");
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			g_assert_not_reached ();
#line 1765 "SerializableGeeHashMapTest.c"
		}
#line 192 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (c);
#line 192 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (doc);
#line 1771 "SerializableGeeHashMapTest.c"
	}
	goto __finally88;
	__catch88_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp43_ = NULL;
		const gchar* _tmp44_ = NULL;
		gchar* _tmp45_ = NULL;
		gchar* _tmp46_ = NULL;
#line 192 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		e = _inner_error_;
#line 192 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_inner_error_ = NULL;
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp43_ = e->message;
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp44_ = string_to_string (_tmp43_);
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp45_ = g_strconcat ("ERROR: ", _tmp44_, NULL);
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp46_ = _tmp45_;
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_message ("SerializableGeeHashMapTest.vala:217: %s", _tmp46_);
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp46_);
#line 218 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_assert_not_reached ();
#line 192 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_error_free0 (e);
#line 1801 "SerializableGeeHashMapTest.c"
	}
	__finally88:
#line 192 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 192 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 192 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_clear_error (&_inner_error_);
#line 192 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		return;
#line 1812 "SerializableGeeHashMapTest.c"
	}
}


static void ___lambda48__gtest_func (void) {
#line 190 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	__lambda48_ ();
#line 1820 "SerializableGeeHashMapTest.c"
}


static void __lambda49_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		SerializableGeeHashMapTestSpaceContainer* c = NULL;
		SerializableGeeHashMapTestSpaceContainer* _tmp1_ = NULL;
		SerializableGeeHashMapTestSpaceContainer* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		SerializableGeeHashMapTestSpaceContainer* _tmp4_ = NULL;
		const gchar* _tmp5_ = NULL;
		const gchar* _tmp6_ = NULL;
		SerializableGeeHashMapTestSpaceContainer* _tmp16_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp17_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp18_ = NULL;
		gint _tmp19_ = 0;
		gint _tmp20_ = 0;
		gboolean found1 = FALSE;
		gboolean found2 = FALSE;
		gboolean _tmp63_ = FALSE;
		gboolean _tmp64_ = FALSE;
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp0_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n" \
"  <spacecontainer owner=\"Earth\"><space name=\"Big\"/><space name=\"S" \
"mall\"/></spacecontainer>", 0, &_inner_error_);
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		doc = _tmp0_;
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1851 "SerializableGeeHashMapTest.c"
			goto __catch89_g_error;
		}
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp1_ = serializable_gee_hash_map_test_space_container_new ();
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		c = _tmp1_;
#line 227 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp2_ = c;
#line 227 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp3_ = doc;
#line 227 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp2_, (GXmlNode*) _tmp3_, &_inner_error_);
#line 227 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 227 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (c);
#line 227 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (doc);
#line 1870 "SerializableGeeHashMapTest.c"
			goto __catch89_g_error;
		}
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp4_ = c;
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp5_ = serializable_gee_hash_map_test_space_container_get_owner (_tmp4_);
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp6_ = _tmp5_;
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (g_strcmp0 (_tmp6_, "Earth") != 0) {
#line 1881 "SerializableGeeHashMapTest.c"
			SerializableGeeHashMapTestSpaceContainer* _tmp7_ = NULL;
			const gchar* _tmp8_ = NULL;
			const gchar* _tmp9_ = NULL;
			const gchar* _tmp10_ = NULL;
			GXmlGDocument* _tmp11_ = NULL;
			gchar* _tmp12_ = NULL;
			gchar* _tmp13_ = NULL;
			gchar* _tmp14_ = NULL;
			gchar* _tmp15_ = NULL;
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp7_ = c;
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp8_ = serializable_gee_hash_map_test_space_container_get_owner (_tmp7_);
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp9_ = _tmp8_;
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp10_ = string_to_string (_tmp9_);
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp11_ = doc;
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp12_ = gxml_gnode_to_string ((GXmlGNode*) _tmp11_);
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp13_ = _tmp12_;
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp14_ = g_strconcat ("ERROR: owner must be 'Earth' got: ", _tmp10_, "\n", _tmp13_, "\n", NULL);
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp15_ = _tmp14_;
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			g_message ("SerializableGeeHashMapTest.vala:229: %s", _tmp15_);
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_free0 (_tmp15_);
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_free0 (_tmp13_);
#line 230 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			g_assert_not_reached ();
#line 1917 "SerializableGeeHashMapTest.c"
		}
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp16_ = c;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp17_ = serializable_gee_hash_map_test_space_container_get_storage (_tmp16_);
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp18_ = _tmp17_;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp19_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp18_);
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp20_ = _tmp19_;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (_tmp20_ != 2) {
#line 1931 "SerializableGeeHashMapTest.c"
			SerializableGeeHashMapTestSpaceContainer* _tmp21_ = NULL;
			SerializableGeeHashMapTestSpaceCollection* _tmp22_ = NULL;
			SerializableGeeHashMapTestSpaceCollection* _tmp23_ = NULL;
			gint _tmp24_ = 0;
			gint _tmp25_ = 0;
			gchar* _tmp26_ = NULL;
			gchar* _tmp27_ = NULL;
			GXmlGDocument* _tmp28_ = NULL;
			gchar* _tmp29_ = NULL;
			gchar* _tmp30_ = NULL;
			gchar* _tmp31_ = NULL;
			gchar* _tmp32_ = NULL;
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp21_ = c;
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp22_ = serializable_gee_hash_map_test_space_container_get_storage (_tmp21_);
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp23_ = _tmp22_;
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp24_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp23_);
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp25_ = _tmp24_;
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp26_ = g_strdup_printf ("%i", _tmp25_);
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp27_ = _tmp26_;
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp28_ = doc;
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp29_ = gxml_gnode_to_string ((GXmlGNode*) _tmp28_);
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp30_ = _tmp29_;
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp31_ = g_strconcat ("ERROR: Size must be 2 got: ", _tmp27_, "\n", _tmp30_, "\n", NULL);
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp32_ = _tmp31_;
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			g_message ("SerializableGeeHashMapTest.vala:233: %s", _tmp32_);
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_free0 (_tmp32_);
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_free0 (_tmp30_);
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_free0 (_tmp27_);
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			g_assert_not_reached ();
#line 1978 "SerializableGeeHashMapTest.c"
		}
#line 236 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		found1 = FALSE;
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		found2 = FALSE;
#line 1984 "SerializableGeeHashMapTest.c"
		{
			GeeIterator* _k_it = NULL;
			SerializableGeeHashMapTestSpaceContainer* _tmp33_ = NULL;
			SerializableGeeHashMapTestSpaceCollection* _tmp34_ = NULL;
			SerializableGeeHashMapTestSpaceCollection* _tmp35_ = NULL;
			GeeSet* _tmp36_ = NULL;
			GeeSet* _tmp37_ = NULL;
			GeeSet* _tmp38_ = NULL;
			GeeIterator* _tmp39_ = NULL;
			GeeIterator* _tmp40_ = NULL;
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp33_ = c;
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp34_ = serializable_gee_hash_map_test_space_container_get_storage (_tmp33_);
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp35_ = _tmp34_;
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp36_ = gee_abstract_map_get_keys ((GeeAbstractMap*) _tmp35_);
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp37_ = _tmp36_;
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp38_ = _tmp37_;
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp39_ = gee_iterable_iterator ((GeeIterable*) _tmp38_);
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp40_ = _tmp39_;
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (_tmp38_);
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_k_it = _tmp40_;
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			while (TRUE) {
#line 2017 "SerializableGeeHashMapTest.c"
				GeeIterator* _tmp41_ = NULL;
				gboolean _tmp42_ = FALSE;
				gchar* k = NULL;
				GeeIterator* _tmp43_ = NULL;
				gpointer _tmp44_ = NULL;
				SerializableGeeHashMapTestSpaceContainer* _tmp45_ = NULL;
				SerializableGeeHashMapTestSpaceCollection* _tmp46_ = NULL;
				SerializableGeeHashMapTestSpaceCollection* _tmp47_ = NULL;
				const gchar* _tmp48_ = NULL;
				gpointer _tmp49_ = NULL;
				SerializableGeeHashMapTestSpace* _tmp50_ = NULL;
				const gchar* _tmp51_ = NULL;
				const gchar* _tmp52_ = NULL;
				gboolean _tmp53_ = FALSE;
				SerializableGeeHashMapTestSpaceContainer* _tmp54_ = NULL;
				SerializableGeeHashMapTestSpaceCollection* _tmp55_ = NULL;
				SerializableGeeHashMapTestSpaceCollection* _tmp56_ = NULL;
				const gchar* _tmp57_ = NULL;
				gpointer _tmp58_ = NULL;
				SerializableGeeHashMapTestSpace* _tmp59_ = NULL;
				const gchar* _tmp60_ = NULL;
				const gchar* _tmp61_ = NULL;
				gboolean _tmp62_ = FALSE;
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp41_ = _k_it;
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp42_ = gee_iterator_next (_tmp41_);
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				if (!_tmp42_) {
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					break;
#line 2049 "SerializableGeeHashMapTest.c"
				}
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp43_ = _k_it;
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp44_ = gee_iterator_get (_tmp43_);
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				k = (gchar*) _tmp44_;
#line 239 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp45_ = c;
#line 239 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp46_ = serializable_gee_hash_map_test_space_container_get_storage (_tmp45_);
#line 239 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp47_ = _tmp46_;
#line 239 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp48_ = k;
#line 239 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp49_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp47_, _tmp48_);
#line 239 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp50_ = (SerializableGeeHashMapTestSpace*) _tmp49_;
#line 239 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp51_ = serializable_gee_hash_map_test_space_get_name (_tmp50_);
#line 239 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp52_ = _tmp51_;
#line 239 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp53_ = g_strcmp0 (_tmp52_, "Big") == 0;
#line 239 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_g_object_unref0 (_tmp50_);
#line 239 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				if (_tmp53_) {
#line 239 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					found1 = TRUE;
#line 2081 "SerializableGeeHashMapTest.c"
				}
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp54_ = c;
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp55_ = serializable_gee_hash_map_test_space_container_get_storage (_tmp54_);
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp56_ = _tmp55_;
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp57_ = k;
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp58_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp56_, _tmp57_);
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp59_ = (SerializableGeeHashMapTestSpace*) _tmp58_;
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp60_ = serializable_gee_hash_map_test_space_get_name (_tmp59_);
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp61_ = _tmp60_;
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp62_ = g_strcmp0 (_tmp61_, "Small") == 0;
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_g_object_unref0 (_tmp59_);
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				if (_tmp62_) {
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					found2 = TRUE;
#line 2107 "SerializableGeeHashMapTest.c"
				}
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_g_free0 (k);
#line 2111 "SerializableGeeHashMapTest.c"
			}
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (_k_it);
#line 2115 "SerializableGeeHashMapTest.c"
		}
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp63_ = found1;
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (!_tmp63_) {
#line 243 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			g_message ("SerializableGeeHashMapTest.vala:243: %s", "ERROR: Big key value is not found\n");
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			g_assert_not_reached ();
#line 2125 "SerializableGeeHashMapTest.c"
		}
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp64_ = found2;
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (!_tmp64_) {
#line 247 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			g_message ("SerializableGeeHashMapTest.vala:247: %s", "ERROR: Small key value is not found\n");
#line 248 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			g_assert_not_reached ();
#line 2135 "SerializableGeeHashMapTest.c"
		}
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (c);
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (doc);
#line 2141 "SerializableGeeHashMapTest.c"
	}
	goto __finally89;
	__catch89_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp65_ = NULL;
		const gchar* _tmp66_ = NULL;
		gchar* _tmp67_ = NULL;
		gchar* _tmp68_ = NULL;
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		e = _inner_error_;
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_inner_error_ = NULL;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp65_ = e->message;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp66_ = string_to_string (_tmp65_);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp67_ = g_strconcat ("ERROR: ", _tmp66_, NULL);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp68_ = _tmp67_;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_message ("SerializableGeeHashMapTest.vala:252: %s", _tmp68_);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp68_);
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_assert_not_reached ();
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_error_free0 (e);
#line 2171 "SerializableGeeHashMapTest.c"
	}
	__finally89:
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_clear_error (&_inner_error_);
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		return;
#line 2182 "SerializableGeeHashMapTest.c"
	}
}


static void ___lambda49__gtest_func (void) {
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	__lambda49_ ();
#line 2190 "SerializableGeeHashMapTest.c"
}


static void __lambda50_ (void) {
	GError * _inner_error_ = NULL;
	{
		SerializableGeeHashMapTestSpaceContainer* c = NULL;
		SerializableGeeHashMapTestSpaceContainer* _tmp0_ = NULL;
		SerializableGeeHashMapTestSpace* o1 = NULL;
		SerializableGeeHashMapTestSpace* _tmp1_ = NULL;
		SerializableGeeHashMapTestSpace* o2 = NULL;
		SerializableGeeHashMapTestSpace* _tmp2_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp3_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp4_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp5_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp6_ = NULL;
		const gchar* _tmp7_ = NULL;
		const gchar* _tmp8_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp9_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp10_ = NULL;
		const gchar* _tmp11_ = NULL;
		const gchar* _tmp12_ = NULL;
		GXmlTDocument* doc = NULL;
		GXmlTDocument* _tmp13_ = NULL;
		GXmlNode* _tmp14_ = NULL;
		GXmlNode* _tmp15_ = NULL;
		GXmlNode* _tmp16_ = NULL;
		GXmlNode* _tmp17_ = NULL;
		GXmlNode* _tmp18_ = NULL;
		GXmlNode* _tmp19_ = NULL;
		GXmlNode* _tmp20_ = NULL;
		GXmlNode* _tmp21_ = NULL;
		gchar* _tmp22_ = NULL;
		gchar* _tmp23_ = NULL;
		gchar* _tmp24_ = NULL;
		GXmlNode* root = NULL;
		GXmlNode* _tmp25_ = NULL;
		GXmlNode* _tmp26_ = NULL;
		GXmlNode* _tmp27_ = NULL;
		GeeBidirList* _tmp28_ = NULL;
		GeeBidirList* _tmp29_ = NULL;
		GeeBidirList* _tmp30_ = NULL;
		gint _tmp31_ = 0;
		gint _tmp32_ = 0;
		gboolean found1 = FALSE;
		gboolean found2 = FALSE;
		gboolean _tmp68_ = FALSE;
		gboolean _tmp69_ = FALSE;
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp0_ = serializable_gee_hash_map_test_space_container_new ();
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		c = _tmp0_;
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp1_ = serializable_gee_hash_map_test_space_new_named ("Big");
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		o1 = _tmp1_;
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp2_ = serializable_gee_hash_map_test_space_new_named ("Small");
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		o2 = _tmp2_;
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp3_ = serializable_gee_hash_map_test_space_collection_new ();
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp4_ = _tmp3_;
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		serializable_gee_hash_map_test_space_container_set_storage (c, _tmp4_);
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp4_);
#line 263 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp5_ = serializable_gee_hash_map_test_space_container_get_storage (c);
#line 263 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp6_ = _tmp5_;
#line 263 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp7_ = serializable_gee_hash_map_test_space_get_name (o1);
#line 263 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp8_ = _tmp7_;
#line 263 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		gee_abstract_map_set ((GeeAbstractMap*) _tmp6_, _tmp8_, o1);
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp9_ = serializable_gee_hash_map_test_space_container_get_storage (c);
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp10_ = _tmp9_;
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp11_ = serializable_gee_hash_map_test_space_get_name (o2);
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp12_ = _tmp11_;
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		gee_abstract_map_set ((GeeAbstractMap*) _tmp10_, _tmp12_, o2);
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp13_ = gxml_tdocument_new ();
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		doc = _tmp13_;
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp14_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) c, (GXmlNode*) doc, &_inner_error_);
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp15_ = _tmp14_;
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp15_);
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (doc);
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (o2);
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (o1);
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (c);
#line 2299 "SerializableGeeHashMapTest.c"
			goto __catch90_g_error;
		}
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp16_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp17_ = _tmp16_;
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp18_ = _tmp17_;
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp18_ != NULL, "doc.root != null");
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp18_);
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp19_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp20_ = _tmp19_;
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp21_ = _tmp20_;
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp22_ = gxml_node_get_name (_tmp21_);
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp23_ = _tmp22_;
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp24_ = _tmp23_;
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp24_, "spacecontainer") == 0, "doc.root.name == \"spacecontainer\"");
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp24_);
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp21_);
#line 269 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp25_ = gxml_document_get_root ((GXmlDocument*) doc);
#line 269 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp26_ = _tmp25_;
#line 269 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		root = _tmp26_;
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp27_ = root;
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp28_ = gxml_node_get_children_nodes (_tmp27_);
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp29_ = _tmp28_;
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp30_ = _tmp29_;
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp31_ = gee_collection_get_size ((GeeCollection*) _tmp30_);
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp32_ = _tmp31_;
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp32_ > 0, "root.children_nodes.size > 0");
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp30_);
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		found1 = FALSE;
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		found2 = FALSE;
#line 2356 "SerializableGeeHashMapTest.c"
		{
			GeeBidirList* _n_list = NULL;
			GXmlNode* _tmp33_ = NULL;
			GeeBidirList* _tmp34_ = NULL;
			GeeBidirList* _tmp35_ = NULL;
			gint _n_size = 0;
			GeeBidirList* _tmp36_ = NULL;
			gint _tmp37_ = 0;
			gint _tmp38_ = 0;
			gint _n_index = 0;
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp33_ = root;
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp34_ = gxml_node_get_children_nodes (_tmp33_);
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp35_ = _tmp34_;
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_n_list = _tmp35_;
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp36_ = _n_list;
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp37_ = gee_collection_get_size ((GeeCollection*) _tmp36_);
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp38_ = _tmp37_;
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_n_size = _tmp38_;
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_n_index = -1;
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			while (TRUE) {
#line 2387 "SerializableGeeHashMapTest.c"
				gint _tmp39_ = 0;
				gint _tmp40_ = 0;
				gint _tmp41_ = 0;
				GXmlNode* n = NULL;
				GeeBidirList* _tmp42_ = NULL;
				gint _tmp43_ = 0;
				gpointer _tmp44_ = NULL;
				gboolean _tmp45_ = FALSE;
				GXmlNode* _tmp46_ = NULL;
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp39_ = _n_index;
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_n_index = _tmp39_ + 1;
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp40_ = _n_index;
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp41_ = _n_size;
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				if (!(_tmp40_ < _tmp41_)) {
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					break;
#line 2409 "SerializableGeeHashMapTest.c"
				}
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp42_ = _n_list;
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp43_ = _n_index;
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp44_ = gee_list_get ((GeeList*) _tmp42_, _tmp43_);
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				n = (GXmlNode*) _tmp44_;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp46_ = n;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp46_, GXML_TYPE_ELEMENT)) {
#line 2423 "SerializableGeeHashMapTest.c"
					GXmlNode* _tmp47_ = NULL;
					gchar* _tmp48_ = NULL;
					gchar* _tmp49_ = NULL;
					gchar* _tmp50_ = NULL;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp47_ = n;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp48_ = gxml_node_get_name (_tmp47_);
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp49_ = _tmp48_;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp50_ = _tmp49_;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp45_ = g_strcmp0 (_tmp50_, "space") == 0;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_g_free0 (_tmp50_);
#line 2440 "SerializableGeeHashMapTest.c"
				} else {
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp45_ = FALSE;
#line 2444 "SerializableGeeHashMapTest.c"
				}
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				if (_tmp45_) {
#line 2448 "SerializableGeeHashMapTest.c"
					GXmlNode* name = NULL;
					GXmlNode* _tmp51_ = NULL;
					GeeMap* _tmp52_ = NULL;
					GeeMap* _tmp53_ = NULL;
					GeeMap* _tmp54_ = NULL;
					gpointer _tmp55_ = NULL;
					GXmlNode* _tmp56_ = NULL;
					GXmlNode* _tmp57_ = NULL;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp51_ = n;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp52_ = gxml_node_get_attrs (_tmp51_);
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp53_ = _tmp52_;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp54_ = _tmp53_;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp55_ = gee_map_get (_tmp54_, "name");
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp56_ = (GXmlNode*) _tmp55_;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_g_object_unref0 (_tmp54_);
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					name = _tmp56_;
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp57_ = name;
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					if (_tmp57_ != NULL) {
#line 2477 "SerializableGeeHashMapTest.c"
						GXmlNode* _tmp58_ = NULL;
						gchar* _tmp59_ = NULL;
						gchar* _tmp60_ = NULL;
						gchar* _tmp61_ = NULL;
						gboolean _tmp62_ = FALSE;
						GXmlNode* _tmp63_ = NULL;
						gchar* _tmp64_ = NULL;
						gchar* _tmp65_ = NULL;
						gchar* _tmp66_ = NULL;
						gboolean _tmp67_ = FALSE;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_tmp58_ = name;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_tmp59_ = gxml_node_get_value (_tmp58_);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_tmp60_ = _tmp59_;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_tmp61_ = _tmp60_;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_tmp62_ = g_strcmp0 (_tmp61_, "Big") == 0;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_g_free0 (_tmp61_);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						if (_tmp62_) {
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
							found1 = TRUE;
#line 2504 "SerializableGeeHashMapTest.c"
						}
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_tmp63_ = name;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_tmp64_ = gxml_node_get_value (_tmp63_);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_tmp65_ = _tmp64_;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_tmp66_ = _tmp65_;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_tmp67_ = g_strcmp0 (_tmp66_, "Small") == 0;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_g_free0 (_tmp66_);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						if (_tmp67_) {
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
							found2 = TRUE;
#line 2522 "SerializableGeeHashMapTest.c"
						}
					}
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_g_object_unref0 (name);
#line 2527 "SerializableGeeHashMapTest.c"
				}
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_g_object_unref0 (n);
#line 2531 "SerializableGeeHashMapTest.c"
			}
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (_n_list);
#line 2535 "SerializableGeeHashMapTest.c"
		}
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp68_ = found1;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp68_, "found1");
#line 283 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp69_ = found2;
#line 283 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp69_, "found2");
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (root);
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (doc);
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (o2);
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (o1);
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (c);
#line 2555 "SerializableGeeHashMapTest.c"
	}
	goto __finally90;
	__catch90_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp70_ = NULL;
		const gchar* _tmp71_ = NULL;
		gchar* _tmp72_ = NULL;
		gchar* _tmp73_ = NULL;
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		e = _inner_error_;
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_inner_error_ = NULL;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp70_ = e->message;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp71_ = string_to_string (_tmp70_);
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp72_ = g_strconcat ("ERROR: ", _tmp71_, NULL);
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp73_ = _tmp72_;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_message ("SerializableGeeHashMapTest.vala:286: %s", _tmp73_);
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp73_);
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_assert_not_reached ();
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_error_free0 (e);
#line 2585 "SerializableGeeHashMapTest.c"
	}
	__finally90:
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_clear_error (&_inner_error_);
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		return;
#line 2596 "SerializableGeeHashMapTest.c"
	}
}


static void ___lambda50__gtest_func (void) {
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	__lambda50_ ();
#line 2604 "SerializableGeeHashMapTest.c"
}


static void __lambda51_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* idoc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		SerializableGeeHashMapTestSpaceContainer* isc = NULL;
		SerializableGeeHashMapTestSpaceContainer* _tmp1_ = NULL;
		SerializableGeeHashMapTestSpaceContainer* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		GXmlTDocument* doc = NULL;
		GXmlTDocument* _tmp4_ = NULL;
		SerializableGeeHashMapTestSpaceContainer* _tmp5_ = NULL;
		GXmlTDocument* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlNode* _tmp8_ = NULL;
		SerializableGeeHashMapTestSpaceContainer* sc = NULL;
		SerializableGeeHashMapTestSpaceContainer* _tmp9_ = NULL;
		SerializableGeeHashMapTestSpaceContainer* _tmp10_ = NULL;
		GXmlTDocument* _tmp11_ = NULL;
		SerializableGeeHashMapTestSpaceContainer* _tmp12_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp13_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp14_ = NULL;
		SerializableGeeHashMapTestSpaceContainer* _tmp15_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp16_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp17_ = NULL;
		gint _tmp18_ = 0;
		gint _tmp19_ = 0;
		gint i = 0;
		gint _tmp33_ = 0;
		SerializableGeeHashMapTestSpace* s1 = NULL;
		SerializableGeeHashMapTestSpaceContainer* _tmp34_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp35_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp36_ = NULL;
		gpointer _tmp37_ = NULL;
		gchar* _tmp38_ = NULL;
		gchar* _tmp39_ = NULL;
		SerializableGeeHashMapTestSpace* s2 = NULL;
		SerializableGeeHashMapTestSpaceContainer* _tmp40_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp41_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp42_ = NULL;
		gpointer _tmp43_ = NULL;
		gchar* _tmp44_ = NULL;
		gchar* _tmp45_ = NULL;
		SerializableGeeHashMapTestSpace* s3 = NULL;
		SerializableGeeHashMapTestSpaceContainer* _tmp46_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp47_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp48_ = NULL;
		gpointer _tmp49_ = NULL;
		gchar* _tmp50_ = NULL;
		gchar* _tmp51_ = NULL;
		SerializableGeeHashMapTestSpace* s4 = NULL;
		SerializableGeeHashMapTestSpaceContainer* _tmp52_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp53_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp54_ = NULL;
		gpointer _tmp55_ = NULL;
		gchar* _tmp56_ = NULL;
		gchar* _tmp57_ = NULL;
		SerializableGeeHashMapTestSpace* s5 = NULL;
		SerializableGeeHashMapTestSpaceContainer* _tmp58_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp59_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp60_ = NULL;
		gpointer _tmp61_ = NULL;
		gchar* _tmp62_ = NULL;
		gchar* _tmp63_ = NULL;
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp0_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n" \
"    <spacecontainer owner=\"Earth\">\n" \
"      <space name=\"Big\"/>\n" \
"      <space name=\"Big\">FAKE1</space>\n" \
"      <space name=\"Small\"/>\n" \
"      <space name=\"Yum1\">FAKE2</space>\n" \
"      <space name=\"Yum2\">FAKE3</space>\n" \
"      <space name=\"Yum3\">FAKE4</space>\n" \
"      <space name=\"Yum3\">FAKE5</space>\n" \
"    </spacecontainer>", 0, &_inner_error_);
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		idoc = _tmp0_;
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 2678 "SerializableGeeHashMapTest.c"
			goto __catch91_g_error;
		}
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp1_ = serializable_gee_hash_map_test_space_container_new ();
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		isc = _tmp1_;
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp2_ = isc;
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp3_ = idoc;
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp2_, (GXmlNode*) _tmp3_, &_inner_error_);
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (isc);
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (idoc);
#line 2697 "SerializableGeeHashMapTest.c"
			goto __catch91_g_error;
		}
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp4_ = gxml_tdocument_new ();
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		doc = _tmp4_;
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp5_ = isc;
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp6_ = doc;
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp7_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) _tmp5_, (GXmlNode*) _tmp6_, &_inner_error_);
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp8_ = _tmp7_;
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp8_);
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (doc);
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (isc);
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (idoc);
#line 2722 "SerializableGeeHashMapTest.c"
			goto __catch91_g_error;
		}
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp9_ = serializable_gee_hash_map_test_space_container_new ();
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		sc = _tmp9_;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp10_ = sc;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp11_ = doc;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp10_, (GXmlNode*) _tmp11_, &_inner_error_);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (sc);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (doc);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (isc);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (idoc);
#line 2745 "SerializableGeeHashMapTest.c"
			goto __catch91_g_error;
		}
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp12_ = sc;
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp13_ = serializable_gee_hash_map_test_space_container_get_storage (_tmp12_);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp14_ = _tmp13_;
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp14_ != NULL, "sc.storage != null");
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp15_ = sc;
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp16_ = serializable_gee_hash_map_test_space_container_get_storage (_tmp15_);
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp17_ = _tmp16_;
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp18_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp17_);
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp19_ = _tmp18_;
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp19_ == 5, "sc.storage.size == 5");
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		i = 0;
#line 2770 "SerializableGeeHashMapTest.c"
		{
			GeeIterator* _s_it = NULL;
			SerializableGeeHashMapTestSpaceContainer* _tmp20_ = NULL;
			SerializableGeeHashMapTestSpaceCollection* _tmp21_ = NULL;
			SerializableGeeHashMapTestSpaceCollection* _tmp22_ = NULL;
			GeeCollection* _tmp23_ = NULL;
			GeeCollection* _tmp24_ = NULL;
			GeeCollection* _tmp25_ = NULL;
			GeeIterator* _tmp26_ = NULL;
			GeeIterator* _tmp27_ = NULL;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp20_ = sc;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp21_ = serializable_gee_hash_map_test_space_container_get_storage (_tmp20_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp22_ = _tmp21_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp23_ = gee_abstract_map_get_values ((GeeAbstractMap*) _tmp22_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp24_ = _tmp23_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp25_ = _tmp24_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp26_ = gee_iterable_iterator ((GeeIterable*) _tmp25_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_tmp27_ = _tmp26_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (_tmp25_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_s_it = _tmp27_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			while (TRUE) {
#line 2803 "SerializableGeeHashMapTest.c"
				GeeIterator* _tmp28_ = NULL;
				gboolean _tmp29_ = FALSE;
				SerializableGeeHashMapTestSpace* s = NULL;
				GeeIterator* _tmp30_ = NULL;
				gpointer _tmp31_ = NULL;
				gint _tmp32_ = 0;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp28_ = _s_it;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp29_ = gee_iterator_next (_tmp28_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				if (!_tmp29_) {
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					break;
#line 2818 "SerializableGeeHashMapTest.c"
				}
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp30_ = _s_it;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp31_ = gee_iterator_get (_tmp30_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				s = (SerializableGeeHashMapTestSpace*) _tmp31_;
#line 313 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp32_ = i;
#line 313 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				i = _tmp32_ + 1;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_g_object_unref0 (s);
#line 2832 "SerializableGeeHashMapTest.c"
			}
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (_s_it);
#line 2836 "SerializableGeeHashMapTest.c"
		}
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp33_ = i;
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp33_ == 5, "i == 5");
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp34_ = sc;
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp35_ = serializable_gee_hash_map_test_space_container_get_storage (_tmp34_);
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp36_ = _tmp35_;
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp37_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp36_, "Big");
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		s1 = (SerializableGeeHashMapTestSpace*) _tmp37_;
#line 316 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (s1 != NULL, "s1 != null");
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp38_ = serializable_gee_hash_map_test_space_get_value (s1);
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp39_ = _tmp38_;
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp39_, "FAKE1") == 0, "s1.get_value () == \"FAKE1\"");
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp39_);
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp40_ = sc;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp41_ = serializable_gee_hash_map_test_space_container_get_storage (_tmp40_);
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp42_ = _tmp41_;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp43_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp42_, "Small");
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		s2 = (SerializableGeeHashMapTestSpace*) _tmp43_;
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (s2 != NULL, "s2 != null");
#line 320 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp44_ = serializable_gee_hash_map_test_space_get_value (s2);
#line 320 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp45_ = _tmp44_;
#line 320 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp45_, "") == 0, "s2.get_value () == \"\"");
#line 320 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp45_);
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp46_ = sc;
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp47_ = serializable_gee_hash_map_test_space_container_get_storage (_tmp46_);
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp48_ = _tmp47_;
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp49_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp48_, "Yum1");
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		s3 = (SerializableGeeHashMapTestSpace*) _tmp49_;
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (s3 != NULL, "s3 != null");
#line 323 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp50_ = serializable_gee_hash_map_test_space_get_value (s3);
#line 323 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp51_ = _tmp50_;
#line 323 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp51_, "FAKE2") == 0, "s3.get_value () == \"FAKE2\"");
#line 323 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp51_);
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp52_ = sc;
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp53_ = serializable_gee_hash_map_test_space_container_get_storage (_tmp52_);
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp54_ = _tmp53_;
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp55_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp54_, "Yum2");
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		s4 = (SerializableGeeHashMapTestSpace*) _tmp55_;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (s4 != NULL, "s4 != null");
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp56_ = serializable_gee_hash_map_test_space_get_value (s4);
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp57_ = _tmp56_;
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp57_, "FAKE3") == 0, "s4.get_value () == \"FAKE3\"");
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp57_);
#line 327 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp58_ = sc;
#line 327 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp59_ = serializable_gee_hash_map_test_space_container_get_storage (_tmp58_);
#line 327 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp60_ = _tmp59_;
#line 327 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp61_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp60_, "Yum3");
#line 327 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		s5 = (SerializableGeeHashMapTestSpace*) _tmp61_;
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (s5 != NULL, "s5 != null");
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp62_ = serializable_gee_hash_map_test_space_get_value (s5);
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp63_ = _tmp62_;
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp63_, "FAKE5") == 0, "s5.get_value () == \"FAKE5\"");
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp63_);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (s5);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (s4);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (s3);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (s2);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (s1);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (sc);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (doc);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (isc);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (idoc);
#line 2960 "SerializableGeeHashMapTest.c"
	}
	goto __finally91;
	__catch91_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp64_ = NULL;
		const gchar* _tmp65_ = NULL;
		gchar* _tmp66_ = NULL;
		gchar* _tmp67_ = NULL;
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		e = _inner_error_;
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_inner_error_ = NULL;
#line 331 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp64_ = e->message;
#line 331 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp65_ = string_to_string (_tmp64_);
#line 331 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp66_ = g_strconcat ("ERROR: ", _tmp65_, NULL);
#line 331 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp67_ = _tmp66_;
#line 331 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_message ("SerializableGeeHashMapTest.vala:331: %s", _tmp67_);
#line 331 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp67_);
#line 332 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_assert_not_reached ();
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_error_free0 (e);
#line 2990 "SerializableGeeHashMapTest.c"
	}
	__finally91:
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_clear_error (&_inner_error_);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		return;
#line 3001 "SerializableGeeHashMapTest.c"
	}
}


static void ___lambda51__gtest_func (void) {
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	__lambda51_ ();
#line 3009 "SerializableGeeHashMapTest.c"
}


static void __lambda52_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* d = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		BookStore* bs = NULL;
		BookStore* _tmp1_ = NULL;
		BookStore* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		BookStore* _tmp4_ = NULL;
		const gchar* _tmp5_ = NULL;
		const gchar* _tmp6_ = NULL;
		BookStore* _tmp7_ = NULL;
		BookArray* _tmp8_ = NULL;
		BookArray* _tmp9_ = NULL;
		gint _tmp10_ = 0;
		gint _tmp11_ = 0;
		Book* b = NULL;
		BookStore* _tmp12_ = NULL;
		BookArray* _tmp13_ = NULL;
		BookArray* _tmp14_ = NULL;
		gpointer _tmp15_ = NULL;
		Name* _tmp16_ = NULL;
		Name* _tmp17_ = NULL;
		Name* _tmp18_ = NULL;
		Name* _tmp19_ = NULL;
		gchar* _tmp20_ = NULL;
		gchar* _tmp21_ = NULL;
		const gchar* _tmp22_ = NULL;
		const gchar* _tmp23_ = NULL;
		Authors* _tmp24_ = NULL;
		Authors* _tmp25_ = NULL;
		Authors* _tmp26_ = NULL;
		Authors* _tmp27_ = NULL;
		AuthorArray* _tmp28_ = NULL;
		AuthorArray* _tmp29_ = NULL;
		Authors* _tmp30_ = NULL;
		Authors* _tmp31_ = NULL;
		AuthorArray* _tmp32_ = NULL;
		AuthorArray* _tmp33_ = NULL;
		gint _tmp34_ = 0;
		gint _tmp35_ = 0;
		CategoryMap* _tmp36_ = NULL;
		CategoryMap* _tmp37_ = NULL;
		CategoryMap* _tmp38_ = NULL;
		CategoryMap* _tmp39_ = NULL;
		gint _tmp40_ = 0;
		gint _tmp41_ = 0;
		Category* c = NULL;
		CategoryMap* _tmp42_ = NULL;
		CategoryMap* _tmp43_ = NULL;
		gpointer _tmp44_ = NULL;
		const gchar* _tmp45_ = NULL;
		const gchar* _tmp46_ = NULL;
#line 338 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp0_ = gxml_gdocument_new_from_path (TEST_DIR "/test-collection.xml", 0, &_inner_error_);
#line 338 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		d = _tmp0_;
#line 338 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 3073 "SerializableGeeHashMapTest.c"
			goto __catch92_g_error;
		}
#line 339 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp1_ = book_store_new ();
#line 339 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		bs = _tmp1_;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp2_ = bs;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp3_ = d;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp2_, (GXmlNode*) _tmp3_, &_inner_error_);
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (bs);
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (d);
#line 3092 "SerializableGeeHashMapTest.c"
			goto __catch92_g_error;
		}
#line 341 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp4_ = bs;
#line 341 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp5_ = book_store_get_name (_tmp4_);
#line 341 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp6_ = _tmp5_;
#line 341 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp6_, "The Great Book") == 0, "bs.name == \"The Great Book\"");
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp7_ = bs;
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp8_ = book_store_get_books (_tmp7_);
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp9_ = _tmp8_;
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp10_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp9_);
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp11_ = _tmp10_;
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp11_ == 3, "bs.books.size == 3");
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp12_ = bs;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp13_ = book_store_get_books (_tmp12_);
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp14_ = _tmp13_;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp15_ = gee_list_first ((GeeList*) _tmp14_);
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		b = (Book*) _tmp15_;
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (b != NULL, "b != null");
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp16_ = book_get_name (b);
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp17_ = _tmp16_;
#line 345 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp17_ != NULL, "b.name != null");
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp18_ = book_get_name (b);
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp19_ = _tmp18_;
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp20_ = name_get_name (_tmp19_);
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp21_ = _tmp20_;
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp21_, "Book1") == 0, "b.name.get_name () == \"Book1\"");
#line 346 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp21_);
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp22_ = book_get_year (b);
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp23_ = _tmp22_;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp23_, "2015") == 0, "b.year == \"2015\"");
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp24_ = book_get_authors (b);
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp25_ = _tmp24_;
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp25_ != NULL, "b.authors != null");
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp26_ = book_get_authors (b);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp27_ = _tmp26_;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp28_ = authors_get_array (_tmp27_);
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp29_ = _tmp28_;
#line 349 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp29_ != NULL, "b.authors.array != null");
#line 350 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp30_ = book_get_authors (b);
#line 350 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp31_ = _tmp30_;
#line 350 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp32_ = authors_get_array (_tmp31_);
#line 350 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp33_ = _tmp32_;
#line 350 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp34_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp33_);
#line 350 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp35_ = _tmp34_;
#line 350 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp35_ == 2, "b.authors.array.size == 2");
#line 351 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp36_ = book_get_categories (b);
#line 351 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp37_ = _tmp36_;
#line 351 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp37_ != NULL, "b.categories != null");
#line 352 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp38_ = book_get_categories (b);
#line 352 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp39_ = _tmp38_;
#line 352 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp40_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp39_);
#line 352 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp41_ = _tmp40_;
#line 352 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp41_ == 2, "b.categories.size == 2");
#line 353 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp42_ = book_get_categories (b);
#line 353 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp43_ = _tmp42_;
#line 353 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp44_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp43_, "Fiction");
#line 353 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		c = (Category*) _tmp44_;
#line 354 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (c != NULL, "c != null");
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp45_ = category_get_name (c);
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp46_ = _tmp45_;
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp46_, "Fiction") == 0, "c.name == \"Fiction\"");
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (c);
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (b);
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (bs);
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (d);
#line 3221 "SerializableGeeHashMapTest.c"
	}
	goto __finally92;
	__catch92_g_error:
	{
		GError* e = NULL;
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		e = _inner_error_;
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_inner_error_ = NULL;
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_assert_not_reached ();
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_error_free0 (e);
#line 3235 "SerializableGeeHashMapTest.c"
	}
	__finally92:
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_clear_error (&_inner_error_);
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		return;
#line 3246 "SerializableGeeHashMapTest.c"
	}
}


static void ___lambda52__gtest_func (void) {
#line 335 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	__lambda52_ ();
#line 3254 "SerializableGeeHashMapTest.c"
}


static void __lambda53_ (void) {
	GError * _inner_error_ = NULL;
	{
		SerializableGeeHashMapTestBigBag* bag = NULL;
		SerializableGeeHashMapTestBigBag* _tmp0_ = NULL;
		SerializableGeeHashMapTestBigBag* _tmp1_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp2_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp3_ = NULL;
		SerializableGeeHashMapTestBigBag* _tmp52_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp53_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp54_ = NULL;
		gint _tmp55_ = 0;
		gint _tmp56_ = 0;
		GXmlGDocument* d = NULL;
		GXmlGDocument* _tmp57_ = NULL;
		SerializableGeeHashMapTestBigBag* _tmp58_ = NULL;
		GXmlGDocument* _tmp59_ = NULL;
		GXmlNode* _tmp60_ = NULL;
		GXmlNode* _tmp61_ = NULL;
		GXmlGDocument* _tmp62_ = NULL;
		GXmlNode* _tmp63_ = NULL;
		GXmlNode* _tmp64_ = NULL;
		GXmlNode* _tmp65_ = NULL;
		GXmlGDocument* _tmp66_ = NULL;
		GXmlNode* _tmp67_ = NULL;
		GXmlNode* _tmp68_ = NULL;
		GXmlNode* _tmp69_ = NULL;
		gchar* _tmp70_ = NULL;
		gchar* _tmp71_ = NULL;
		gchar* _tmp72_ = NULL;
		GXmlGDocument* _tmp73_ = NULL;
		GXmlNode* _tmp74_ = NULL;
		GXmlNode* _tmp75_ = NULL;
		GXmlNode* _tmp76_ = NULL;
		GeeBidirList* _tmp77_ = NULL;
		GeeBidirList* _tmp78_ = NULL;
		GeeBidirList* _tmp79_ = NULL;
		gint _tmp80_ = 0;
		gint _tmp81_ = 0;
		GXmlGDocument* _tmp82_ = NULL;
		GXmlNode* _tmp83_ = NULL;
		GXmlNode* _tmp84_ = NULL;
		GXmlNode* _tmp85_ = NULL;
		GeeBidirList* _tmp86_ = NULL;
		GeeBidirList* _tmp87_ = NULL;
		GeeBidirList* _tmp88_ = NULL;
		gpointer _tmp89_ = NULL;
		GXmlNode* _tmp90_ = NULL;
		gchar* _tmp91_ = NULL;
		gchar* _tmp92_ = NULL;
		gchar* _tmp93_ = NULL;
		GXmlGDocument* _tmp94_ = NULL;
		GXmlNode* _tmp95_ = NULL;
		GXmlNode* _tmp96_ = NULL;
		GXmlNode* _tmp97_ = NULL;
		GeeBidirList* _tmp98_ = NULL;
		GeeBidirList* _tmp99_ = NULL;
		GeeBidirList* _tmp100_ = NULL;
		gpointer _tmp101_ = NULL;
		GXmlNode* _tmp102_ = NULL;
		GeeBidirList* _tmp103_ = NULL;
		GeeBidirList* _tmp104_ = NULL;
		GeeBidirList* _tmp105_ = NULL;
		gint _tmp106_ = 0;
		gint _tmp107_ = 0;
		GXmlGDocument* _tmp108_ = NULL;
		GXmlNode* _tmp109_ = NULL;
		GXmlNode* _tmp110_ = NULL;
		GXmlNode* _tmp111_ = NULL;
		GeeBidirList* _tmp112_ = NULL;
		GeeBidirList* _tmp113_ = NULL;
		GeeBidirList* _tmp114_ = NULL;
		gpointer _tmp115_ = NULL;
		GXmlNode* _tmp116_ = NULL;
		GeeBidirList* _tmp117_ = NULL;
		GeeBidirList* _tmp118_ = NULL;
		GeeBidirList* _tmp119_ = NULL;
		gpointer _tmp120_ = NULL;
		GXmlNode* _tmp121_ = NULL;
		gchar* _tmp122_ = NULL;
		gchar* _tmp123_ = NULL;
		gchar* _tmp124_ = NULL;
		GXmlGDocument* _tmp125_ = NULL;
		GXmlNode* _tmp126_ = NULL;
		GXmlNode* _tmp127_ = NULL;
		GXmlNode* _tmp128_ = NULL;
		GeeBidirList* _tmp129_ = NULL;
		GeeBidirList* _tmp130_ = NULL;
		GeeBidirList* _tmp131_ = NULL;
		gpointer _tmp132_ = NULL;
		GXmlNode* _tmp133_ = NULL;
		GeeBidirList* _tmp134_ = NULL;
		GeeBidirList* _tmp135_ = NULL;
		GeeBidirList* _tmp136_ = NULL;
		gpointer _tmp137_ = NULL;
		GXmlNode* _tmp138_ = NULL;
		GeeBidirList* _tmp139_ = NULL;
		GeeBidirList* _tmp140_ = NULL;
		GeeBidirList* _tmp141_ = NULL;
		gint _tmp142_ = 0;
		gint _tmp143_ = 0;
		GXmlGDocument* _tmp144_ = NULL;
		GXmlNode* _tmp145_ = NULL;
		GXmlNode* _tmp146_ = NULL;
		GXmlNode* _tmp147_ = NULL;
		GeeBidirList* _tmp148_ = NULL;
		GeeBidirList* _tmp149_ = NULL;
		GeeBidirList* _tmp150_ = NULL;
		gpointer _tmp151_ = NULL;
		GXmlNode* _tmp152_ = NULL;
		GeeBidirList* _tmp153_ = NULL;
		GeeBidirList* _tmp154_ = NULL;
		GeeBidirList* _tmp155_ = NULL;
		gpointer _tmp156_ = NULL;
		GXmlNode* _tmp157_ = NULL;
		GeeBidirList* _tmp158_ = NULL;
		GeeBidirList* _tmp159_ = NULL;
		GeeBidirList* _tmp160_ = NULL;
		gpointer _tmp161_ = NULL;
		GXmlNode* _tmp162_ = NULL;
		gchar* _tmp163_ = NULL;
		gchar* _tmp164_ = NULL;
		gchar* _tmp165_ = NULL;
		GXmlGDocument* _tmp166_ = NULL;
		GXmlNode* _tmp167_ = NULL;
		GXmlNode* _tmp168_ = NULL;
		GXmlNode* _tmp169_ = NULL;
		GeeBidirList* _tmp170_ = NULL;
		GeeBidirList* _tmp171_ = NULL;
		GeeBidirList* _tmp172_ = NULL;
		gpointer _tmp173_ = NULL;
		GXmlNode* _tmp174_ = NULL;
		GeeBidirList* _tmp175_ = NULL;
		GeeBidirList* _tmp176_ = NULL;
		GeeBidirList* _tmp177_ = NULL;
		gpointer _tmp178_ = NULL;
		GXmlNode* _tmp179_ = NULL;
		GeeBidirList* _tmp180_ = NULL;
		GeeBidirList* _tmp181_ = NULL;
		GeeBidirList* _tmp182_ = NULL;
		gpointer _tmp183_ = NULL;
		GXmlNode* _tmp184_ = NULL;
		GeeBidirList* _tmp185_ = NULL;
		GeeBidirList* _tmp186_ = NULL;
		GeeBidirList* _tmp187_ = NULL;
		gint _tmp188_ = 0;
		gint _tmp189_ = 0;
		GXmlGDocument* _tmp190_ = NULL;
		GXmlNode* _tmp191_ = NULL;
		GXmlNode* _tmp192_ = NULL;
		GXmlNode* _tmp193_ = NULL;
		GeeBidirList* _tmp194_ = NULL;
		GeeBidirList* _tmp195_ = NULL;
		GeeBidirList* _tmp196_ = NULL;
		gpointer _tmp197_ = NULL;
		GXmlNode* _tmp198_ = NULL;
		GeeBidirList* _tmp199_ = NULL;
		GeeBidirList* _tmp200_ = NULL;
		GeeBidirList* _tmp201_ = NULL;
		gpointer _tmp202_ = NULL;
		GXmlNode* _tmp203_ = NULL;
		GeeBidirList* _tmp204_ = NULL;
		GeeBidirList* _tmp205_ = NULL;
		GeeBidirList* _tmp206_ = NULL;
		gpointer _tmp207_ = NULL;
		GXmlNode* _tmp208_ = NULL;
		GeeBidirList* _tmp209_ = NULL;
		GeeBidirList* _tmp210_ = NULL;
		GeeBidirList* _tmp211_ = NULL;
		gpointer _tmp212_ = NULL;
		GXmlNode* _tmp213_ = NULL;
		GXmlGDocument* _tmp214_ = NULL;
		GXmlNode* _tmp215_ = NULL;
		GXmlNode* _tmp216_ = NULL;
		GXmlNode* _tmp217_ = NULL;
		GeeBidirList* _tmp218_ = NULL;
		GeeBidirList* _tmp219_ = NULL;
		GeeBidirList* _tmp220_ = NULL;
		gpointer _tmp221_ = NULL;
		GXmlNode* _tmp222_ = NULL;
		GeeBidirList* _tmp223_ = NULL;
		GeeBidirList* _tmp224_ = NULL;
		GeeBidirList* _tmp225_ = NULL;
		gpointer _tmp226_ = NULL;
		GXmlNode* _tmp227_ = NULL;
		GeeBidirList* _tmp228_ = NULL;
		GeeBidirList* _tmp229_ = NULL;
		GeeBidirList* _tmp230_ = NULL;
		gpointer _tmp231_ = NULL;
		GXmlNode* _tmp232_ = NULL;
		GeeBidirList* _tmp233_ = NULL;
		GeeBidirList* _tmp234_ = NULL;
		GeeBidirList* _tmp235_ = NULL;
		gpointer _tmp236_ = NULL;
		GXmlNode* _tmp237_ = NULL;
		gchar* _tmp238_ = NULL;
		gchar* _tmp239_ = NULL;
		gchar* _tmp240_ = NULL;
		SerializableGeeHashMapTestBigBag* bagt = NULL;
		SerializableGeeHashMapTestBigBag* _tmp241_ = NULL;
		SerializableGeeHashMapTestBigBag* _tmp242_ = NULL;
		GXmlGDocument* _tmp243_ = NULL;
		gboolean _tmp244_ = FALSE;
		SerializableGeeHashMapTestBigBag* _tmp245_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp246_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp247_ = NULL;
		gboolean _tmp248_ = FALSE;
		SerializableGeeHashMapTestBigBag* _tmp249_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp250_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp251_ = NULL;
		gint _tmp252_ = 0;
		gint _tmp253_ = 0;
		gboolean _tmp254_ = FALSE;
		SerializableGeeHashMapTestBigBag* _tmp255_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp256_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp257_ = NULL;
		gpointer _tmp258_ = NULL;
		SerializableGeeHashMapTestSmallBag* _tmp259_ = NULL;
		SerializableGeeHashMapTestBallHashMap* _tmp260_ = NULL;
		SerializableGeeHashMapTestBallHashMap* _tmp261_ = NULL;
		gboolean _tmp262_ = FALSE;
		gboolean _tmp263_ = FALSE;
		SerializableGeeHashMapTestBigBag* _tmp264_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp265_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp266_ = NULL;
		gpointer _tmp267_ = NULL;
		SerializableGeeHashMapTestSmallBag* _tmp268_ = NULL;
		SerializableGeeHashMapTestBallHashMap* _tmp269_ = NULL;
		SerializableGeeHashMapTestBallHashMap* _tmp270_ = NULL;
		gint _tmp271_ = 0;
		gint _tmp272_ = 0;
		SerializableGeeHashMapTestBigBag* _tmp273_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp274_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp275_ = NULL;
		gpointer _tmp276_ = NULL;
		SerializableGeeHashMapTestSmallBag* _tmp277_ = NULL;
		SerializableGeeHashMapTestBallHashMap* _tmp278_ = NULL;
		SerializableGeeHashMapTestBallHashMap* _tmp279_ = NULL;
		gpointer _tmp280_ = NULL;
		SerializableGeeHashMapTestBall* _tmp281_ = NULL;
		const gchar* _tmp282_ = NULL;
		const gchar* _tmp283_ = NULL;
		SerializableGeeHashMapTestBigBag* _tmp284_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp285_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp286_ = NULL;
		gpointer _tmp287_ = NULL;
		SerializableGeeHashMapTestSmallBag* _tmp288_ = NULL;
		SerializableGeeHashMapTestBallHashMap* _tmp289_ = NULL;
		SerializableGeeHashMapTestBallHashMap* _tmp290_ = NULL;
		gpointer _tmp291_ = NULL;
		SerializableGeeHashMapTestBall* _tmp292_ = NULL;
		SerializableGeeHashMapTestBallFill* _tmp293_ = NULL;
		SerializableGeeHashMapTestBallFill* _tmp294_ = NULL;
		SerializableGeeHashMapTestBigBag* _tmp295_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp296_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp297_ = NULL;
		gpointer _tmp298_ = NULL;
		SerializableGeeHashMapTestSmallBag* _tmp299_ = NULL;
		SerializableGeeHashMapTestBallHashMap* _tmp300_ = NULL;
		SerializableGeeHashMapTestBallHashMap* _tmp301_ = NULL;
		gpointer _tmp302_ = NULL;
		SerializableGeeHashMapTestBall* _tmp303_ = NULL;
		SerializableGeeHashMapTestBallFill* _tmp304_ = NULL;
		SerializableGeeHashMapTestBallFill* _tmp305_ = NULL;
		gchar* _tmp306_ = NULL;
		gchar* _tmp307_ = NULL;
		gchar* _tmp308_ = NULL;
		SerializableGeeHashMapTestBigBag* _tmp309_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp310_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp311_ = NULL;
		gpointer _tmp312_ = NULL;
		SerializableGeeHashMapTestSmallBag* _tmp313_ = NULL;
		SerializableGeeHashMapTestBallHashMap* _tmp314_ = NULL;
		SerializableGeeHashMapTestBallHashMap* _tmp315_ = NULL;
		gpointer _tmp316_ = NULL;
		SerializableGeeHashMapTestBall* _tmp317_ = NULL;
		SerializableGeeHashMapTestBallFill* _tmp318_ = NULL;
		SerializableGeeHashMapTestBallFill* _tmp319_ = NULL;
		gchar* _tmp320_ = NULL;
		gchar* _tmp321_ = NULL;
		gchar* _tmp322_ = NULL;
		SerializableGeeHashMapTestBigBag* bag2 = NULL;
		SerializableGeeHashMapTestBigBag* _tmp323_ = NULL;
		SerializableGeeHashMapTestBigBag* _tmp324_ = NULL;
		GXmlGDocument* _tmp325_ = NULL;
		SerializableGeeHashMapTestBigBag* _tmp326_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp327_ = NULL;
		SerializableGeeHashMapTestSmallBagHashMap* _tmp328_ = NULL;
		gint _tmp329_ = 0;
		gint _tmp330_ = 0;
		GXmlGDocument* d2 = NULL;
		GXmlGDocument* _tmp331_ = NULL;
		SerializableGeeHashMapTestBigBag* _tmp332_ = NULL;
		GXmlGDocument* _tmp333_ = NULL;
		GXmlNode* _tmp334_ = NULL;
		GXmlNode* _tmp335_ = NULL;
		GXmlGDocument* _tmp336_ = NULL;
		GXmlNode* _tmp337_ = NULL;
		GXmlNode* _tmp338_ = NULL;
		GXmlNode* _tmp339_ = NULL;
		GXmlGDocument* _tmp340_ = NULL;
		GXmlNode* _tmp341_ = NULL;
		GXmlNode* _tmp342_ = NULL;
		GXmlNode* _tmp343_ = NULL;
		gchar* _tmp344_ = NULL;
		gchar* _tmp345_ = NULL;
		gchar* _tmp346_ = NULL;
		GXmlGDocument* _tmp347_ = NULL;
		GXmlNode* _tmp348_ = NULL;
		GXmlNode* _tmp349_ = NULL;
		GXmlNode* _tmp350_ = NULL;
		GeeBidirList* _tmp351_ = NULL;
		GeeBidirList* _tmp352_ = NULL;
		GeeBidirList* _tmp353_ = NULL;
		gint _tmp354_ = 0;
		gint _tmp355_ = 0;
		GXmlGDocument* _tmp356_ = NULL;
		GXmlNode* _tmp357_ = NULL;
		GXmlNode* _tmp358_ = NULL;
		GXmlNode* _tmp359_ = NULL;
		GeeBidirList* _tmp360_ = NULL;
		GeeBidirList* _tmp361_ = NULL;
		GeeBidirList* _tmp362_ = NULL;
		gpointer _tmp363_ = NULL;
		GXmlNode* _tmp364_ = NULL;
		gchar* _tmp365_ = NULL;
		gchar* _tmp366_ = NULL;
		gchar* _tmp367_ = NULL;
		GXmlGDocument* _tmp368_ = NULL;
		GXmlNode* _tmp369_ = NULL;
		GXmlNode* _tmp370_ = NULL;
		GXmlNode* _tmp371_ = NULL;
		GeeBidirList* _tmp372_ = NULL;
		GeeBidirList* _tmp373_ = NULL;
		GeeBidirList* _tmp374_ = NULL;
		gpointer _tmp375_ = NULL;
		GXmlNode* _tmp376_ = NULL;
		GeeBidirList* _tmp377_ = NULL;
		GeeBidirList* _tmp378_ = NULL;
		GeeBidirList* _tmp379_ = NULL;
		gint _tmp380_ = 0;
		gint _tmp381_ = 0;
		GXmlGDocument* _tmp382_ = NULL;
		GXmlNode* _tmp383_ = NULL;
		GXmlNode* _tmp384_ = NULL;
		GXmlNode* _tmp385_ = NULL;
		GeeBidirList* _tmp386_ = NULL;
		GeeBidirList* _tmp387_ = NULL;
		GeeBidirList* _tmp388_ = NULL;
		gpointer _tmp389_ = NULL;
		GXmlNode* _tmp390_ = NULL;
		GeeBidirList* _tmp391_ = NULL;
		GeeBidirList* _tmp392_ = NULL;
		GeeBidirList* _tmp393_ = NULL;
		gpointer _tmp394_ = NULL;
		GXmlNode* _tmp395_ = NULL;
		gchar* _tmp396_ = NULL;
		gchar* _tmp397_ = NULL;
		gchar* _tmp398_ = NULL;
		GXmlGDocument* _tmp399_ = NULL;
		GXmlNode* _tmp400_ = NULL;
		GXmlNode* _tmp401_ = NULL;
		GXmlNode* _tmp402_ = NULL;
		GeeBidirList* _tmp403_ = NULL;
		GeeBidirList* _tmp404_ = NULL;
		GeeBidirList* _tmp405_ = NULL;
		gpointer _tmp406_ = NULL;
		GXmlNode* _tmp407_ = NULL;
		GeeBidirList* _tmp408_ = NULL;
		GeeBidirList* _tmp409_ = NULL;
		GeeBidirList* _tmp410_ = NULL;
		gpointer _tmp411_ = NULL;
		GXmlNode* _tmp412_ = NULL;
		GeeBidirList* _tmp413_ = NULL;
		GeeBidirList* _tmp414_ = NULL;
		GeeBidirList* _tmp415_ = NULL;
		gint _tmp416_ = 0;
		gint _tmp417_ = 0;
		GXmlGDocument* _tmp418_ = NULL;
		GXmlNode* _tmp419_ = NULL;
		GXmlNode* _tmp420_ = NULL;
		GXmlNode* _tmp421_ = NULL;
		GeeBidirList* _tmp422_ = NULL;
		GeeBidirList* _tmp423_ = NULL;
		GeeBidirList* _tmp424_ = NULL;
		gpointer _tmp425_ = NULL;
		GXmlNode* _tmp426_ = NULL;
		GeeBidirList* _tmp427_ = NULL;
		GeeBidirList* _tmp428_ = NULL;
		GeeBidirList* _tmp429_ = NULL;
		gpointer _tmp430_ = NULL;
		GXmlNode* _tmp431_ = NULL;
		GeeBidirList* _tmp432_ = NULL;
		GeeBidirList* _tmp433_ = NULL;
		GeeBidirList* _tmp434_ = NULL;
		gpointer _tmp435_ = NULL;
		GXmlNode* _tmp436_ = NULL;
		gchar* _tmp437_ = NULL;
		gchar* _tmp438_ = NULL;
		gchar* _tmp439_ = NULL;
		GXmlGDocument* _tmp440_ = NULL;
		GXmlNode* _tmp441_ = NULL;
		GXmlNode* _tmp442_ = NULL;
		GXmlNode* _tmp443_ = NULL;
		GeeBidirList* _tmp444_ = NULL;
		GeeBidirList* _tmp445_ = NULL;
		GeeBidirList* _tmp446_ = NULL;
		gpointer _tmp447_ = NULL;
		GXmlNode* _tmp448_ = NULL;
		GeeBidirList* _tmp449_ = NULL;
		GeeBidirList* _tmp450_ = NULL;
		GeeBidirList* _tmp451_ = NULL;
		gpointer _tmp452_ = NULL;
		GXmlNode* _tmp453_ = NULL;
		GeeBidirList* _tmp454_ = NULL;
		GeeBidirList* _tmp455_ = NULL;
		GeeBidirList* _tmp456_ = NULL;
		gpointer _tmp457_ = NULL;
		GXmlNode* _tmp458_ = NULL;
		GeeBidirList* _tmp459_ = NULL;
		GeeBidirList* _tmp460_ = NULL;
		GeeBidirList* _tmp461_ = NULL;
		gint _tmp462_ = 0;
		gint _tmp463_ = 0;
		GXmlGDocument* _tmp464_ = NULL;
		GXmlNode* _tmp465_ = NULL;
		GXmlNode* _tmp466_ = NULL;
		GXmlNode* _tmp467_ = NULL;
		GeeBidirList* _tmp468_ = NULL;
		GeeBidirList* _tmp469_ = NULL;
		GeeBidirList* _tmp470_ = NULL;
		gpointer _tmp471_ = NULL;
		GXmlNode* _tmp472_ = NULL;
		GeeBidirList* _tmp473_ = NULL;
		GeeBidirList* _tmp474_ = NULL;
		GeeBidirList* _tmp475_ = NULL;
		gpointer _tmp476_ = NULL;
		GXmlNode* _tmp477_ = NULL;
		GeeBidirList* _tmp478_ = NULL;
		GeeBidirList* _tmp479_ = NULL;
		GeeBidirList* _tmp480_ = NULL;
		gpointer _tmp481_ = NULL;
		GXmlNode* _tmp482_ = NULL;
		GeeBidirList* _tmp483_ = NULL;
		GeeBidirList* _tmp484_ = NULL;
		GeeBidirList* _tmp485_ = NULL;
		gpointer _tmp486_ = NULL;
		GXmlNode* _tmp487_ = NULL;
		GXmlGDocument* _tmp488_ = NULL;
		GXmlNode* _tmp489_ = NULL;
		GXmlNode* _tmp490_ = NULL;
		GXmlNode* _tmp491_ = NULL;
		GeeBidirList* _tmp492_ = NULL;
		GeeBidirList* _tmp493_ = NULL;
		GeeBidirList* _tmp494_ = NULL;
		gpointer _tmp495_ = NULL;
		GXmlNode* _tmp496_ = NULL;
		GeeBidirList* _tmp497_ = NULL;
		GeeBidirList* _tmp498_ = NULL;
		GeeBidirList* _tmp499_ = NULL;
		gpointer _tmp500_ = NULL;
		GXmlNode* _tmp501_ = NULL;
		GeeBidirList* _tmp502_ = NULL;
		GeeBidirList* _tmp503_ = NULL;
		GeeBidirList* _tmp504_ = NULL;
		gpointer _tmp505_ = NULL;
		GXmlNode* _tmp506_ = NULL;
		GeeBidirList* _tmp507_ = NULL;
		GeeBidirList* _tmp508_ = NULL;
		GeeBidirList* _tmp509_ = NULL;
		gpointer _tmp510_ = NULL;
		GXmlNode* _tmp511_ = NULL;
		gchar* _tmp512_ = NULL;
		gchar* _tmp513_ = NULL;
		gchar* _tmp514_ = NULL;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp0_ = serializable_gee_hash_map_test_big_bag_new ();
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		bag = _tmp0_;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp1_ = bag;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp2_ = serializable_gee_hash_map_test_big_bag_get_bags (_tmp1_);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp3_ = _tmp2_;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp3_ != NULL, "bag.bags != null");
#line 3745 "SerializableGeeHashMapTest.c"
		{
			gint i = 0;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			i = 0;
#line 3750 "SerializableGeeHashMapTest.c"
			{
				gboolean _tmp4_ = FALSE;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				_tmp4_ = TRUE;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
				while (TRUE) {
#line 3757 "SerializableGeeHashMapTest.c"
					gint _tmp6_ = 0;
					SerializableGeeHashMapTestSmallBag* sbag = NULL;
					SerializableGeeHashMapTestSmallBag* _tmp7_ = NULL;
					SerializableGeeHashMapTestSmallBag* _tmp8_ = NULL;
					gint _tmp9_ = 0;
					gchar* _tmp10_ = NULL;
					gchar* _tmp11_ = NULL;
					gchar* _tmp12_ = NULL;
					gchar* _tmp13_ = NULL;
					SerializableGeeHashMapTestSmallBag* _tmp14_ = NULL;
					SerializableGeeHashMapTestBallHashMap* _tmp15_ = NULL;
					SerializableGeeHashMapTestBallHashMap* _tmp16_ = NULL;
					SerializableGeeHashMapTestSmallBag* _tmp40_ = NULL;
					SerializableGeeHashMapTestBallHashMap* _tmp41_ = NULL;
					SerializableGeeHashMapTestBallHashMap* _tmp42_ = NULL;
					gint _tmp43_ = 0;
					gint _tmp44_ = 0;
					SerializableGeeHashMapTestBigBag* _tmp45_ = NULL;
					SerializableGeeHashMapTestSmallBagHashMap* _tmp46_ = NULL;
					SerializableGeeHashMapTestSmallBagHashMap* _tmp47_ = NULL;
					SerializableGeeHashMapTestSmallBag* _tmp48_ = NULL;
					const gchar* _tmp49_ = NULL;
					const gchar* _tmp50_ = NULL;
					SerializableGeeHashMapTestSmallBag* _tmp51_ = NULL;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					if (!_tmp4_) {
#line 3784 "SerializableGeeHashMapTest.c"
						gint _tmp5_ = 0;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						_tmp5_ = i;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						i = _tmp5_ + 1;
#line 3790 "SerializableGeeHashMapTest.c"
					}
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp4_ = FALSE;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp6_ = i;
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					if (!(_tmp6_ < 2)) {
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						break;
#line 3800 "SerializableGeeHashMapTest.c"
					}
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp7_ = serializable_gee_hash_map_test_small_bag_new ();
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					sbag = _tmp7_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp8_ = sbag;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp9_ = i;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp10_ = g_strdup_printf ("%i", _tmp9_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp11_ = _tmp10_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp12_ = g_strconcat ("SmallBag", _tmp11_, NULL);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp13_ = _tmp12_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					serializable_gee_hash_map_test_small_bag_set_name (_tmp8_, _tmp13_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_g_free0 (_tmp13_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_g_free0 (_tmp11_);
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp14_ = sbag;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp15_ = serializable_gee_hash_map_test_small_bag_get_balls (_tmp14_);
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp16_ = _tmp15_;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_vala_assert (_tmp16_ != NULL, "sbag.balls != null");
#line 3832 "SerializableGeeHashMapTest.c"
					{
						gint j = 0;
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
						j = 0;
#line 3837 "SerializableGeeHashMapTest.c"
						{
							gboolean _tmp17_ = FALSE;
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
							_tmp17_ = TRUE;
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
							while (TRUE) {
#line 3844 "SerializableGeeHashMapTest.c"
								gint _tmp19_ = 0;
								SerializableGeeHashMapTestBall* b = NULL;
								SerializableGeeHashMapTestBall* _tmp20_ = NULL;
								SerializableGeeHashMapTestBall* _tmp21_ = NULL;
								gint _tmp22_ = 0;
								gchar* _tmp23_ = NULL;
								gchar* _tmp24_ = NULL;
								gchar* _tmp25_ = NULL;
								gchar* _tmp26_ = NULL;
								SerializableGeeHashMapTestBall* _tmp27_ = NULL;
								SerializableGeeHashMapTestBallFill* _tmp28_ = NULL;
								SerializableGeeHashMapTestBallFill* _tmp29_ = NULL;
								SerializableGeeHashMapTestBall* _tmp30_ = NULL;
								SerializableGeeHashMapTestBallFill* _tmp31_ = NULL;
								SerializableGeeHashMapTestBallFill* _tmp32_ = NULL;
								SerializableGeeHashMapTestSmallBag* _tmp33_ = NULL;
								SerializableGeeHashMapTestBallHashMap* _tmp34_ = NULL;
								SerializableGeeHashMapTestBallHashMap* _tmp35_ = NULL;
								SerializableGeeHashMapTestBall* _tmp36_ = NULL;
								const gchar* _tmp37_ = NULL;
								const gchar* _tmp38_ = NULL;
								SerializableGeeHashMapTestBall* _tmp39_ = NULL;
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								if (!_tmp17_) {
#line 3869 "SerializableGeeHashMapTest.c"
									gint _tmp18_ = 0;
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
									_tmp18_ = j;
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
									j = _tmp18_ + 1;
#line 3875 "SerializableGeeHashMapTest.c"
								}
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp17_ = FALSE;
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp19_ = j;
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								if (!(_tmp19_ < 2)) {
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
									break;
#line 3885 "SerializableGeeHashMapTest.c"
								}
#line 374 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp20_ = serializable_gee_hash_map_test_ball_new ();
#line 374 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								b = _tmp20_;
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp21_ = b;
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp22_ = j;
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp23_ = g_strdup_printf ("%i", _tmp22_);
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp24_ = _tmp23_;
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp25_ = g_strconcat ("Ball", _tmp24_, NULL);
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp26_ = _tmp25_;
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								serializable_gee_hash_map_test_ball_set_name (_tmp21_, _tmp26_);
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_g_free0 (_tmp26_);
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_g_free0 (_tmp24_);
#line 376 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp27_ = b;
#line 376 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp28_ = serializable_gee_hash_map_test_ball_fill_new ();
#line 376 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp29_ = _tmp28_;
#line 376 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								serializable_gee_hash_map_test_ball_set_ballfill (_tmp27_, _tmp29_);
#line 376 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_g_object_unref0 (_tmp29_);
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp30_ = b;
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp31_ = serializable_gee_hash_map_test_ball_get_ballfill (_tmp30_);
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp32_ = _tmp31_;
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								serializable_gee_hash_map_test_ball_fill_set_text (_tmp32_, "golden dust");
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp33_ = sbag;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp34_ = serializable_gee_hash_map_test_small_bag_get_balls (_tmp33_);
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp35_ = _tmp34_;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp36_ = b;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp37_ = serializable_gee_hash_map_test_ball_get_name (_tmp36_);
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp38_ = _tmp37_;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_tmp39_ = b;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								gee_abstract_map_set ((GeeAbstractMap*) _tmp35_, _tmp38_, _tmp39_);
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
								_g_object_unref0 (b);
#line 3945 "SerializableGeeHashMapTest.c"
							}
						}
					}
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp40_ = sbag;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp41_ = serializable_gee_hash_map_test_small_bag_get_balls (_tmp40_);
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp42_ = _tmp41_;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp43_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp42_);
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp44_ = _tmp43_;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_vala_assert (_tmp44_ == 2, "sbag.balls.size == 2");
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp45_ = bag;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp46_ = serializable_gee_hash_map_test_big_bag_get_bags (_tmp45_);
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp47_ = _tmp46_;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp48_ = sbag;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp49_ = serializable_gee_hash_map_test_small_bag_get_name (_tmp48_);
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp50_ = _tmp49_;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_tmp51_ = sbag;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					gee_abstract_map_set ((GeeAbstractMap*) _tmp47_, _tmp50_, _tmp51_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
					_g_object_unref0 (sbag);
#line 3979 "SerializableGeeHashMapTest.c"
				}
			}
		}
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp52_ = bag;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp53_ = serializable_gee_hash_map_test_big_bag_get_bags (_tmp52_);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp54_ = _tmp53_;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp55_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp54_);
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp56_ = _tmp55_;
#line 383 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp56_ == 2, "bag.bags.size == 2");
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp57_ = gxml_gdocument_new ();
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		d = _tmp57_;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp58_ = bag;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp59_ = d;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp60_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) _tmp58_, (GXmlNode*) _tmp59_, &_inner_error_);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp61_ = _tmp60_;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp61_);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (d);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (bag);
#line 4015 "SerializableGeeHashMapTest.c"
			goto __catch93_g_error;
		}
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp62_ = d;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp63_ = gxml_document_get_root ((GXmlDocument*) _tmp62_);
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp64_ = _tmp63_;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp65_ = _tmp64_;
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp65_ != NULL, "d.root != null");
#line 387 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp65_);
#line 388 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp66_ = d;
#line 388 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp67_ = gxml_document_get_root ((GXmlDocument*) _tmp66_);
#line 388 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp68_ = _tmp67_;
#line 388 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp69_ = _tmp68_;
#line 388 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp70_ = gxml_node_get_name (_tmp69_);
#line 388 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp71_ = _tmp70_;
#line 388 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp72_ = _tmp71_;
#line 388 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp72_, "BigBag") == 0, "d.root.name == \"BigBag\"");
#line 388 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp72_);
#line 388 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp69_);
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp73_ = d;
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp74_ = gxml_document_get_root ((GXmlDocument*) _tmp73_);
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp75_ = _tmp74_;
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp76_ = _tmp75_;
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp77_ = gxml_node_get_children_nodes (_tmp76_);
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp78_ = _tmp77_;
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp79_ = _tmp78_;
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp80_ = gee_collection_get_size ((GeeCollection*) _tmp79_);
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp81_ = _tmp80_;
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp81_ == 2, "d.root.children_nodes.size == 2");
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp79_);
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp76_);
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp82_ = d;
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp83_ = gxml_document_get_root ((GXmlDocument*) _tmp82_);
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp84_ = _tmp83_;
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp85_ = _tmp84_;
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp86_ = gxml_node_get_children_nodes (_tmp85_);
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp87_ = _tmp86_;
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp88_ = _tmp87_;
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp89_ = gee_list_get ((GeeList*) _tmp88_, 0);
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp90_ = (GXmlNode*) _tmp89_;
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp91_ = gxml_node_get_name (_tmp90_);
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp92_ = _tmp91_;
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp93_ = _tmp92_;
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp93_, "SmallBag") == 0, "d.root.children_nodes[0].name == \"SmallBag\"");
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp93_);
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp90_);
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp88_);
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp85_);
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp94_ = d;
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp95_ = gxml_document_get_root ((GXmlDocument*) _tmp94_);
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp96_ = _tmp95_;
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp97_ = _tmp96_;
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp98_ = gxml_node_get_children_nodes (_tmp97_);
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp99_ = _tmp98_;
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp100_ = _tmp99_;
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp101_ = gee_list_get ((GeeList*) _tmp100_, 0);
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp102_ = (GXmlNode*) _tmp101_;
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp103_ = gxml_node_get_children_nodes (_tmp102_);
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp104_ = _tmp103_;
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp105_ = _tmp104_;
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp106_ = gee_collection_get_size ((GeeCollection*) _tmp105_);
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp107_ = _tmp106_;
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp107_ == 2, "d.root.children_nodes[0].children_nodes.size == 2");
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp105_);
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp102_);
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp100_);
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp97_);
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp108_ = d;
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp109_ = gxml_document_get_root ((GXmlDocument*) _tmp108_);
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp110_ = _tmp109_;
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp111_ = _tmp110_;
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp112_ = gxml_node_get_children_nodes (_tmp111_);
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp113_ = _tmp112_;
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp114_ = _tmp113_;
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp115_ = gee_list_get ((GeeList*) _tmp114_, 0);
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp116_ = (GXmlNode*) _tmp115_;
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp117_ = gxml_node_get_children_nodes (_tmp116_);
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp118_ = _tmp117_;
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp119_ = _tmp118_;
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp120_ = gee_list_get ((GeeList*) _tmp119_, 0);
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp121_ = (GXmlNode*) _tmp120_;
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp122_ = gxml_node_get_name (_tmp121_);
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp123_ = _tmp122_;
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp124_ = _tmp123_;
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp124_, "Ball") == 0, "d.root.children_nodes[0].children_nodes[0].name == \"Ball\"");
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp124_);
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp121_);
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp119_);
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp116_);
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp114_);
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp111_);
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp125_ = d;
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp126_ = gxml_document_get_root ((GXmlDocument*) _tmp125_);
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp127_ = _tmp126_;
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp128_ = _tmp127_;
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp129_ = gxml_node_get_children_nodes (_tmp128_);
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp130_ = _tmp129_;
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp131_ = _tmp130_;
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp132_ = gee_list_get ((GeeList*) _tmp131_, 0);
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp133_ = (GXmlNode*) _tmp132_;
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp134_ = gxml_node_get_children_nodes (_tmp133_);
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp135_ = _tmp134_;
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp136_ = _tmp135_;
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp137_ = gee_list_get ((GeeList*) _tmp136_, 0);
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp138_ = (GXmlNode*) _tmp137_;
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp139_ = gxml_node_get_children_nodes (_tmp138_);
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp140_ = _tmp139_;
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp141_ = _tmp140_;
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp142_ = gee_collection_get_size ((GeeCollection*) _tmp141_);
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp143_ = _tmp142_;
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp143_ == 1, "d.root.children_nodes[0].children_nodes[0].children_nodes.size == 1");
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp141_);
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp138_);
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp136_);
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp133_);
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp131_);
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp128_);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp144_ = d;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp145_ = gxml_document_get_root ((GXmlDocument*) _tmp144_);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp146_ = _tmp145_;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp147_ = _tmp146_;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp148_ = gxml_node_get_children_nodes (_tmp147_);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp149_ = _tmp148_;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp150_ = _tmp149_;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp151_ = gee_list_get ((GeeList*) _tmp150_, 0);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp152_ = (GXmlNode*) _tmp151_;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp153_ = gxml_node_get_children_nodes (_tmp152_);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp154_ = _tmp153_;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp155_ = _tmp154_;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp156_ = gee_list_get ((GeeList*) _tmp155_, 0);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp157_ = (GXmlNode*) _tmp156_;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp158_ = gxml_node_get_children_nodes (_tmp157_);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp159_ = _tmp158_;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp160_ = _tmp159_;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp161_ = gee_list_get ((GeeList*) _tmp160_, 0);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp162_ = (GXmlNode*) _tmp161_;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp163_ = gxml_node_get_name (_tmp162_);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp164_ = _tmp163_;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp165_ = _tmp164_;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp165_, "BallFill") == 0, "d.root.children_nodes[0].children_nodes[0].children_nodes[0].name == \"BallFill\"");
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp165_);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp162_);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp160_);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp157_);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp155_);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp152_);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp150_);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp147_);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp166_ = d;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp167_ = gxml_document_get_root ((GXmlDocument*) _tmp166_);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp168_ = _tmp167_;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp169_ = _tmp168_;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp170_ = gxml_node_get_children_nodes (_tmp169_);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp171_ = _tmp170_;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp172_ = _tmp171_;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp173_ = gee_list_get ((GeeList*) _tmp172_, 0);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp174_ = (GXmlNode*) _tmp173_;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp175_ = gxml_node_get_children_nodes (_tmp174_);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp176_ = _tmp175_;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp177_ = _tmp176_;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp178_ = gee_list_get ((GeeList*) _tmp177_, 0);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp179_ = (GXmlNode*) _tmp178_;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp180_ = gxml_node_get_children_nodes (_tmp179_);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp181_ = _tmp180_;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp182_ = _tmp181_;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp183_ = gee_list_get ((GeeList*) _tmp182_, 0);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp184_ = (GXmlNode*) _tmp183_;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp185_ = gxml_node_get_children_nodes (_tmp184_);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp186_ = _tmp185_;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp187_ = _tmp186_;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp188_ = gee_collection_get_size ((GeeCollection*) _tmp187_);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp189_ = _tmp188_;
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp189_ == 1, "d.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes.size == 1");
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp187_);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp184_);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp182_);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp179_);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp177_);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp174_);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp172_);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp169_);
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp190_ = d;
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp191_ = gxml_document_get_root ((GXmlDocument*) _tmp190_);
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp192_ = _tmp191_;
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp193_ = _tmp192_;
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp194_ = gxml_node_get_children_nodes (_tmp193_);
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp195_ = _tmp194_;
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp196_ = _tmp195_;
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp197_ = gee_list_get ((GeeList*) _tmp196_, 0);
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp198_ = (GXmlNode*) _tmp197_;
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp199_ = gxml_node_get_children_nodes (_tmp198_);
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp200_ = _tmp199_;
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp201_ = _tmp200_;
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp202_ = gee_list_get ((GeeList*) _tmp201_, 0);
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp203_ = (GXmlNode*) _tmp202_;
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp204_ = gxml_node_get_children_nodes (_tmp203_);
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp205_ = _tmp204_;
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp206_ = _tmp205_;
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp207_ = gee_list_get ((GeeList*) _tmp206_, 0);
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp208_ = (GXmlNode*) _tmp207_;
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp209_ = gxml_node_get_children_nodes (_tmp208_);
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp210_ = _tmp209_;
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp211_ = _tmp210_;
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp212_ = gee_list_get ((GeeList*) _tmp211_, 0);
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp213_ = (GXmlNode*) _tmp212_;
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp213_, GXML_TYPE_TEXT), "d.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes[0] is Text");
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp213_);
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp211_);
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp208_);
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp206_);
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp203_);
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp201_);
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp198_);
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp196_);
#line 396 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp193_);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp214_ = d;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp215_ = gxml_document_get_root ((GXmlDocument*) _tmp214_);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp216_ = _tmp215_;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp217_ = _tmp216_;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp218_ = gxml_node_get_children_nodes (_tmp217_);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp219_ = _tmp218_;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp220_ = _tmp219_;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp221_ = gee_list_get ((GeeList*) _tmp220_, 0);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp222_ = (GXmlNode*) _tmp221_;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp223_ = gxml_node_get_children_nodes (_tmp222_);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp224_ = _tmp223_;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp225_ = _tmp224_;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp226_ = gee_list_get ((GeeList*) _tmp225_, 0);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp227_ = (GXmlNode*) _tmp226_;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp228_ = gxml_node_get_children_nodes (_tmp227_);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp229_ = _tmp228_;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp230_ = _tmp229_;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp231_ = gee_list_get ((GeeList*) _tmp230_, 0);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp232_ = (GXmlNode*) _tmp231_;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp233_ = gxml_node_get_children_nodes (_tmp232_);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp234_ = _tmp233_;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp235_ = _tmp234_;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp236_ = gee_list_get ((GeeList*) _tmp235_, 0);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp237_ = (GXmlNode*) _tmp236_;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp238_ = gxml_node_get_value (_tmp237_);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp239_ = _tmp238_;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp240_ = _tmp239_;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp240_, "golden dust") == 0, "d.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes[0].value == \"golden dust\"");
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp240_);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp237_);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp235_);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp232_);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp230_);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp227_);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp225_);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp222_);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp220_);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp217_);
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp241_ = serializable_gee_hash_map_test_big_bag_new ();
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		bagt = _tmp241_;
#line 401 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp242_ = bagt;
#line 401 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp243_ = d;
#line 401 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp242_, (GXmlNode*) _tmp243_, &_inner_error_);
#line 401 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 401 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (bagt);
#line 401 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (d);
#line 401 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (bag);
#line 4536 "SerializableGeeHashMapTest.c"
			goto __catch93_g_error;
		}
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp245_ = bagt;
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp246_ = serializable_gee_hash_map_test_big_bag_get_bags (_tmp245_);
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp247_ = _tmp246_;
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp248_ = gxml_serializable_hash_map_deserialize_children ((GXmlSerializableHashMap*) _tmp247_, &_inner_error_);
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp244_ = _tmp248_;
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (bagt);
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (d);
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (bag);
#line 4557 "SerializableGeeHashMapTest.c"
			goto __catch93_g_error;
		}
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp244_, "bagt.bags.deserialize_children ()");
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp249_ = bagt;
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp250_ = serializable_gee_hash_map_test_big_bag_get_bags (_tmp249_);
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp251_ = _tmp250_;
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp252_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp251_);
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp253_ = _tmp252_;
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp253_ == 2, "bagt.bags.size == 2");
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp255_ = bagt;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp256_ = serializable_gee_hash_map_test_big_bag_get_bags (_tmp255_);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp257_ = _tmp256_;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp258_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp257_, "SmallBag1");
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp259_ = (SerializableGeeHashMapTestSmallBag*) _tmp258_;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp260_ = serializable_gee_hash_map_test_small_bag_get_balls (_tmp259_);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp261_ = _tmp260_;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp262_ = gxml_serializable_hash_map_deserialize_children ((GXmlSerializableHashMap*) _tmp261_, &_inner_error_);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp263_ = _tmp262_;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp259_);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp254_ = _tmp263_;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (bagt);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (d);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (bag);
#line 4604 "SerializableGeeHashMapTest.c"
			goto __catch93_g_error;
		}
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp254_, "bagt.bags.get(\"SmallBag1\").balls.deserialize_children ()");
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp264_ = bagt;
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp265_ = serializable_gee_hash_map_test_big_bag_get_bags (_tmp264_);
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp266_ = _tmp265_;
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp267_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp266_, "SmallBag1");
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp268_ = (SerializableGeeHashMapTestSmallBag*) _tmp267_;
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp269_ = serializable_gee_hash_map_test_small_bag_get_balls (_tmp268_);
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp270_ = _tmp269_;
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp271_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp270_);
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp272_ = _tmp271_;
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp272_ == 2, "bagt.bags.get(\"SmallBag1\").balls.size == 2");
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp268_);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp273_ = bagt;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp274_ = serializable_gee_hash_map_test_big_bag_get_bags (_tmp273_);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp275_ = _tmp274_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp276_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp275_, "SmallBag1");
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp277_ = (SerializableGeeHashMapTestSmallBag*) _tmp276_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp278_ = serializable_gee_hash_map_test_small_bag_get_balls (_tmp277_);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp279_ = _tmp278_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp280_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp279_, "Ball1");
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp281_ = (SerializableGeeHashMapTestBall*) _tmp280_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp282_ = serializable_gee_hash_map_test_ball_get_name (_tmp281_);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp283_ = _tmp282_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp283_, "Ball1") == 0, "bagt.bags.get(\"SmallBag1\").balls.get(\"Ball1\").name == \"Ball1\"");
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp281_);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp277_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp284_ = bagt;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp285_ = serializable_gee_hash_map_test_big_bag_get_bags (_tmp284_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp286_ = _tmp285_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp287_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp286_, "SmallBag1");
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp288_ = (SerializableGeeHashMapTestSmallBag*) _tmp287_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp289_ = serializable_gee_hash_map_test_small_bag_get_balls (_tmp288_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp290_ = _tmp289_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp291_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp290_, "Ball1");
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp292_ = (SerializableGeeHashMapTestBall*) _tmp291_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp293_ = serializable_gee_hash_map_test_ball_get_ballfill (_tmp292_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp294_ = _tmp293_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp294_ != NULL, "bagt.bags.get(\"SmallBag1\").balls.get(\"Ball1\").ballfill !=null");
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp292_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp288_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp295_ = bagt;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp296_ = serializable_gee_hash_map_test_big_bag_get_bags (_tmp295_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp297_ = _tmp296_;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp298_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp297_, "SmallBag1");
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp299_ = (SerializableGeeHashMapTestSmallBag*) _tmp298_;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp300_ = serializable_gee_hash_map_test_small_bag_get_balls (_tmp299_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp301_ = _tmp300_;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp302_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp301_, "Ball1");
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp303_ = (SerializableGeeHashMapTestBall*) _tmp302_;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp304_ = serializable_gee_hash_map_test_ball_get_ballfill (_tmp303_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp305_ = _tmp304_;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp306_ = gxml_serializable_get_serialized_xml_node_value ((GXmlSerializable*) _tmp305_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp307_ = _tmp306_;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp308_ = _tmp307_;
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp308_ != NULL, "bagt.bags.get(\"SmallBag1\").balls.get(\"Ball1\").ballfill.serialized_xml_node_value !=null");
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp308_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp303_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp299_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp309_ = bagt;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp310_ = serializable_gee_hash_map_test_big_bag_get_bags (_tmp309_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp311_ = _tmp310_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp312_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp311_, "SmallBag1");
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp313_ = (SerializableGeeHashMapTestSmallBag*) _tmp312_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp314_ = serializable_gee_hash_map_test_small_bag_get_balls (_tmp313_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp315_ = _tmp314_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp316_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp315_, "Ball1");
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp317_ = (SerializableGeeHashMapTestBall*) _tmp316_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp318_ = serializable_gee_hash_map_test_ball_get_ballfill (_tmp317_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp319_ = _tmp318_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp320_ = gxml_serializable_get_serialized_xml_node_value ((GXmlSerializable*) _tmp319_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp321_ = _tmp320_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp322_ = _tmp321_;
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp322_, "golden dust") == 0, "bagt.bags.get(\"SmallBag1\").balls.get(\"Ball1\").ballfill.serialized_xml_node_value ==\"golden dust\"");
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp322_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp317_);
#line 409 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp313_);
#line 410 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp323_ = serializable_gee_hash_map_test_big_bag_new ();
#line 410 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		bag2 = _tmp323_;
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp324_ = bag2;
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp325_ = d;
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp324_, (GXmlNode*) _tmp325_, &_inner_error_);
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (bag2);
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (bagt);
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (d);
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (bag);
#line 4779 "SerializableGeeHashMapTest.c"
			goto __catch93_g_error;
		}
#line 412 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp326_ = bag2;
#line 412 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp327_ = serializable_gee_hash_map_test_big_bag_get_bags (_tmp326_);
#line 412 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp328_ = _tmp327_;
#line 412 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp329_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp328_);
#line 412 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp330_ = _tmp329_;
#line 412 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp330_ == 0, "bag2.bags.size == 0");
#line 414 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp331_ = gxml_gdocument_new ();
#line 414 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		d2 = _tmp331_;
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp332_ = bag2;
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp333_ = d2;
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp334_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) _tmp332_, (GXmlNode*) _tmp333_, &_inner_error_);
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp335_ = _tmp334_;
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp335_);
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (d2);
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (bag2);
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (bagt);
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (d);
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
			_g_object_unref0 (bag);
#line 4820 "SerializableGeeHashMapTest.c"
			goto __catch93_g_error;
		}
#line 417 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp336_ = d2;
#line 417 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp337_ = gxml_document_get_root ((GXmlDocument*) _tmp336_);
#line 417 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp338_ = _tmp337_;
#line 417 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp339_ = _tmp338_;
#line 417 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp339_ != NULL, "d2.root != null");
#line 417 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp339_);
#line 418 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp340_ = d2;
#line 418 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp341_ = gxml_document_get_root ((GXmlDocument*) _tmp340_);
#line 418 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp342_ = _tmp341_;
#line 418 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp343_ = _tmp342_;
#line 418 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp344_ = gxml_node_get_name (_tmp343_);
#line 418 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp345_ = _tmp344_;
#line 418 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp346_ = _tmp345_;
#line 418 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp346_, "BigBag") == 0, "d2.root.name == \"BigBag\"");
#line 418 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp346_);
#line 418 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp343_);
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp347_ = d2;
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp348_ = gxml_document_get_root ((GXmlDocument*) _tmp347_);
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp349_ = _tmp348_;
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp350_ = _tmp349_;
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp351_ = gxml_node_get_children_nodes (_tmp350_);
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp352_ = _tmp351_;
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp353_ = _tmp352_;
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp354_ = gee_collection_get_size ((GeeCollection*) _tmp353_);
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp355_ = _tmp354_;
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp355_ == 2, "d2.root.children_nodes.size == 2");
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp353_);
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp350_);
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp356_ = d2;
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp357_ = gxml_document_get_root ((GXmlDocument*) _tmp356_);
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp358_ = _tmp357_;
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp359_ = _tmp358_;
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp360_ = gxml_node_get_children_nodes (_tmp359_);
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp361_ = _tmp360_;
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp362_ = _tmp361_;
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp363_ = gee_list_get ((GeeList*) _tmp362_, 0);
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp364_ = (GXmlNode*) _tmp363_;
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp365_ = gxml_node_get_name (_tmp364_);
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp366_ = _tmp365_;
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp367_ = _tmp366_;
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp367_, "SmallBag") == 0, "d2.root.children_nodes[0].name == \"SmallBag\"");
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp367_);
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp364_);
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp362_);
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp359_);
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp368_ = d2;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp369_ = gxml_document_get_root ((GXmlDocument*) _tmp368_);
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp370_ = _tmp369_;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp371_ = _tmp370_;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp372_ = gxml_node_get_children_nodes (_tmp371_);
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp373_ = _tmp372_;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp374_ = _tmp373_;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp375_ = gee_list_get ((GeeList*) _tmp374_, 0);
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp376_ = (GXmlNode*) _tmp375_;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp377_ = gxml_node_get_children_nodes (_tmp376_);
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp378_ = _tmp377_;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp379_ = _tmp378_;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp380_ = gee_collection_get_size ((GeeCollection*) _tmp379_);
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp381_ = _tmp380_;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp381_ == 2, "d2.root.children_nodes[0].children_nodes.size == 2");
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp379_);
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp376_);
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp374_);
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp371_);
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp382_ = d2;
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp383_ = gxml_document_get_root ((GXmlDocument*) _tmp382_);
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp384_ = _tmp383_;
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp385_ = _tmp384_;
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp386_ = gxml_node_get_children_nodes (_tmp385_);
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp387_ = _tmp386_;
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp388_ = _tmp387_;
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp389_ = gee_list_get ((GeeList*) _tmp388_, 0);
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp390_ = (GXmlNode*) _tmp389_;
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp391_ = gxml_node_get_children_nodes (_tmp390_);
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp392_ = _tmp391_;
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp393_ = _tmp392_;
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp394_ = gee_list_get ((GeeList*) _tmp393_, 0);
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp395_ = (GXmlNode*) _tmp394_;
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp396_ = gxml_node_get_name (_tmp395_);
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp397_ = _tmp396_;
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp398_ = _tmp397_;
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp398_, "Ball") == 0, "d2.root.children_nodes[0].children_nodes[0].name == \"Ball\"");
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp398_);
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp395_);
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp393_);
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp390_);
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp388_);
#line 422 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp385_);
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp399_ = d2;
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp400_ = gxml_document_get_root ((GXmlDocument*) _tmp399_);
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp401_ = _tmp400_;
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp402_ = _tmp401_;
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp403_ = gxml_node_get_children_nodes (_tmp402_);
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp404_ = _tmp403_;
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp405_ = _tmp404_;
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp406_ = gee_list_get ((GeeList*) _tmp405_, 0);
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp407_ = (GXmlNode*) _tmp406_;
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp408_ = gxml_node_get_children_nodes (_tmp407_);
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp409_ = _tmp408_;
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp410_ = _tmp409_;
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp411_ = gee_list_get ((GeeList*) _tmp410_, 0);
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp412_ = (GXmlNode*) _tmp411_;
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp413_ = gxml_node_get_children_nodes (_tmp412_);
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp414_ = _tmp413_;
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp415_ = _tmp414_;
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp416_ = gee_collection_get_size ((GeeCollection*) _tmp415_);
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp417_ = _tmp416_;
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp417_ == 1, "d2.root.children_nodes[0].children_nodes[0].children_nodes.size == 1");
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp415_);
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp412_);
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp410_);
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp407_);
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp405_);
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp402_);
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp418_ = d2;
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp419_ = gxml_document_get_root ((GXmlDocument*) _tmp418_);
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp420_ = _tmp419_;
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp421_ = _tmp420_;
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp422_ = gxml_node_get_children_nodes (_tmp421_);
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp423_ = _tmp422_;
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp424_ = _tmp423_;
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp425_ = gee_list_get ((GeeList*) _tmp424_, 0);
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp426_ = (GXmlNode*) _tmp425_;
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp427_ = gxml_node_get_children_nodes (_tmp426_);
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp428_ = _tmp427_;
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp429_ = _tmp428_;
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp430_ = gee_list_get ((GeeList*) _tmp429_, 0);
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp431_ = (GXmlNode*) _tmp430_;
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp432_ = gxml_node_get_children_nodes (_tmp431_);
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp433_ = _tmp432_;
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp434_ = _tmp433_;
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp435_ = gee_list_get ((GeeList*) _tmp434_, 0);
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp436_ = (GXmlNode*) _tmp435_;
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp437_ = gxml_node_get_name (_tmp436_);
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp438_ = _tmp437_;
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp439_ = _tmp438_;
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp439_, "BallFill") == 0, "d2.root.children_nodes[0].children_nodes[0].children_nodes[0].name == \"BallFill\"");
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp439_);
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp436_);
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp434_);
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp431_);
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp429_);
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp426_);
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp424_);
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp421_);
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp440_ = d2;
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp441_ = gxml_document_get_root ((GXmlDocument*) _tmp440_);
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp442_ = _tmp441_;
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp443_ = _tmp442_;
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp444_ = gxml_node_get_children_nodes (_tmp443_);
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp445_ = _tmp444_;
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp446_ = _tmp445_;
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp447_ = gee_list_get ((GeeList*) _tmp446_, 0);
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp448_ = (GXmlNode*) _tmp447_;
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp449_ = gxml_node_get_children_nodes (_tmp448_);
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp450_ = _tmp449_;
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp451_ = _tmp450_;
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp452_ = gee_list_get ((GeeList*) _tmp451_, 0);
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp453_ = (GXmlNode*) _tmp452_;
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp454_ = gxml_node_get_children_nodes (_tmp453_);
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp455_ = _tmp454_;
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp456_ = _tmp455_;
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp457_ = gee_list_get ((GeeList*) _tmp456_, 0);
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp458_ = (GXmlNode*) _tmp457_;
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp459_ = gxml_node_get_children_nodes (_tmp458_);
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp460_ = _tmp459_;
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp461_ = _tmp460_;
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp462_ = gee_collection_get_size ((GeeCollection*) _tmp461_);
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp463_ = _tmp462_;
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (_tmp463_ == 1, "d2.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes.size == 1");
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp461_);
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp458_);
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp456_);
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp453_);
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp451_);
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp448_);
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp446_);
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp443_);
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp464_ = d2;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp465_ = gxml_document_get_root ((GXmlDocument*) _tmp464_);
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp466_ = _tmp465_;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp467_ = _tmp466_;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp468_ = gxml_node_get_children_nodes (_tmp467_);
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp469_ = _tmp468_;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp470_ = _tmp469_;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp471_ = gee_list_get ((GeeList*) _tmp470_, 0);
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp472_ = (GXmlNode*) _tmp471_;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp473_ = gxml_node_get_children_nodes (_tmp472_);
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp474_ = _tmp473_;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp475_ = _tmp474_;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp476_ = gee_list_get ((GeeList*) _tmp475_, 0);
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp477_ = (GXmlNode*) _tmp476_;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp478_ = gxml_node_get_children_nodes (_tmp477_);
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp479_ = _tmp478_;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp480_ = _tmp479_;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp481_ = gee_list_get ((GeeList*) _tmp480_, 0);
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp482_ = (GXmlNode*) _tmp481_;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp483_ = gxml_node_get_children_nodes (_tmp482_);
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp484_ = _tmp483_;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp485_ = _tmp484_;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp486_ = gee_list_get ((GeeList*) _tmp485_, 0);
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp487_ = (GXmlNode*) _tmp486_;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp487_, GXML_TYPE_TEXT), "d2.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes[0] is GXml.Text");
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp487_);
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp485_);
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp482_);
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp480_);
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp477_);
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp475_);
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp472_);
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp470_);
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp467_);
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp488_ = d2;
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp489_ = gxml_document_get_root ((GXmlDocument*) _tmp488_);
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp490_ = _tmp489_;
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp491_ = _tmp490_;
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp492_ = gxml_node_get_children_nodes (_tmp491_);
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp493_ = _tmp492_;
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp494_ = _tmp493_;
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp495_ = gee_list_get ((GeeList*) _tmp494_, 0);
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp496_ = (GXmlNode*) _tmp495_;
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp497_ = gxml_node_get_children_nodes (_tmp496_);
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp498_ = _tmp497_;
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp499_ = _tmp498_;
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp500_ = gee_list_get ((GeeList*) _tmp499_, 0);
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp501_ = (GXmlNode*) _tmp500_;
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp502_ = gxml_node_get_children_nodes (_tmp501_);
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp503_ = _tmp502_;
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp504_ = _tmp503_;
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp505_ = gee_list_get ((GeeList*) _tmp504_, 0);
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp506_ = (GXmlNode*) _tmp505_;
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp507_ = gxml_node_get_children_nodes (_tmp506_);
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp508_ = _tmp507_;
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp509_ = _tmp508_;
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp510_ = gee_list_get ((GeeList*) _tmp509_, 0);
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp511_ = (GXmlNode*) _tmp510_;
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp512_ = gxml_node_get_value (_tmp511_);
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp513_ = _tmp512_;
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp514_ = _tmp513_;
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_vala_assert (g_strcmp0 (_tmp514_, "golden dust") == 0, "d2.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes[0].value == \"golden dust\"");
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp514_);
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp511_);
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp509_);
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp506_);
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp504_);
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp501_);
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp499_);
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp496_);
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp494_);
#line 427 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp491_);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (d2);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (bag2);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (bagt);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (d);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (bag);
#line 5333 "SerializableGeeHashMapTest.c"
	}
	goto __finally93;
	__catch93_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp515_ = NULL;
		gchar* _tmp516_ = NULL;
		gchar* _tmp517_ = NULL;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		e = _inner_error_;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_inner_error_ = NULL;
#line 429 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp515_ = e->message;
#line 429 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp516_ = g_strconcat ("ERROR: ", _tmp515_, NULL);
#line 429 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp517_ = _tmp516_;
#line 429 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_message ("SerializableGeeHashMapTest.vala:429: %s", _tmp517_);
#line 429 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_free0 (_tmp517_);
#line 430 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_assert_not_reached ();
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_error_free0 (e);
#line 5360 "SerializableGeeHashMapTest.c"
	}
	__finally93:
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_clear_error (&_inner_error_);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		return;
#line 5371 "SerializableGeeHashMapTest.c"
	}
}


static void ___lambda53__gtest_func (void) {
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	__lambda53_ ();
#line 5379 "SerializableGeeHashMapTest.c"
}


void serializable_gee_hash_map_test_add_tests (void) {
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_test_add_func ("/gxml/serializable/hashmap/api", ___lambda46__gtest_func);
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_test_add_func ("/gxml/serializable/hashmap/serialize", ___lambda47__gtest_func);
#line 190 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_test_add_func ("/gxml/serializable/hashmap/deserialize", ___lambda48__gtest_func);
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_test_add_func ("/gxml/serializable/hashmap/container_class/deserialize", ___lambda49__gtest_func);
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_test_add_func ("/gxml/serializable/hashmap/containder_class/serialize", ___lambda50__gtest_func);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_test_add_func ("/gxml/serializable/hashmap/deserialize-serialize", ___lambda51__gtest_func);
#line 335 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_test_add_func ("/gxml/serializable/hashmap/deserialize-node-names", ___lambda52__gtest_func);
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_test_add_func ("/gxml/serializable/hashmap/post-deserialization/serialize/contents", ___lambda53__gtest_func);
#line 5400 "SerializableGeeHashMapTest.c"
}


SerializableGeeHashMapTest* serializable_gee_hash_map_test_construct (GType object_type) {
	SerializableGeeHashMapTest* self = NULL;
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTest*) gxml_test_construct (object_type);
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return self;
#line 5410 "SerializableGeeHashMapTest.c"
}


SerializableGeeHashMapTest* serializable_gee_hash_map_test_new (void) {
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return serializable_gee_hash_map_test_construct (TYPE_SERIALIZABLE_GEE_HASH_MAP_TEST);
#line 5417 "SerializableGeeHashMapTest.c"
}


static gchar* serializable_gee_hash_map_test_space_real_get_map_key (GXmlSerializableMapKey* base) {
	SerializableGeeHashMapTestSpace * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestSpace*) base;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp1_;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 5436 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestSpace* serializable_gee_hash_map_test_space_construct_named (GType object_type, const gchar* name) {
	SerializableGeeHashMapTestSpace * self = NULL;
	const gchar* _tmp0_ = NULL;
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestSpace*) gxml_serializable_object_model_construct (object_type);
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = name;
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	serializable_gee_hash_map_test_space_set_name (self, _tmp0_);
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return self;
#line 5453 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestSpace* serializable_gee_hash_map_test_space_new_named (const gchar* name) {
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return serializable_gee_hash_map_test_space_construct_named (SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE, name);
#line 5460 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_space_set_value (SerializableGeeHashMapTestSpace* self, const gchar* str) {
	const gchar* _tmp0_ = NULL;
#line 36 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_if_fail (self != NULL);
#line 36 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_if_fail (str != NULL);
#line 36 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = str;
#line 36 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	gxml_serializable_set_serialized_xml_node_value ((GXmlSerializable*) self, _tmp0_);
#line 5474 "SerializableGeeHashMapTest.c"
}


static gchar* serializable_gee_hash_map_test_space_get_value (SerializableGeeHashMapTestSpace* self) {
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	gchar* _tmp2_ = NULL;
	gboolean _tmp3_ = FALSE;
	gchar* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
#line 37 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 38 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = gxml_serializable_get_serialized_xml_node_value ((GXmlSerializable*) self);
#line 38 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = _tmp0_;
#line 38 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp2_ = _tmp1_;
#line 38 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp3_ = _tmp2_ == NULL;
#line 38 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_free0 (_tmp2_);
#line 38 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	if (_tmp3_) {
#line 39 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		gxml_serializable_set_serialized_xml_node_value ((GXmlSerializable*) self, "");
#line 5502 "SerializableGeeHashMapTest.c"
	}
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp4_ = gxml_serializable_get_serialized_xml_node_value ((GXmlSerializable*) self);
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp5_ = _tmp4_;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp5_;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 5512 "SerializableGeeHashMapTest.c"
}


static gboolean serializable_gee_hash_map_test_space_real_serialize_use_xml_node_value (GXmlSerializableObjectModel* base) {
	SerializableGeeHashMapTestSpace * self;
	gboolean result = FALSE;
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestSpace*) base;
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = TRUE;
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 5525 "SerializableGeeHashMapTest.c"
}


static gchar* serializable_gee_hash_map_test_space_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeHashMapTestSpace * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestSpace*) base;
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = g_strdup ("space");
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp0_;
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 5541 "SerializableGeeHashMapTest.c"
}


static gchar* serializable_gee_hash_map_test_space_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeHashMapTestSpace * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 45 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestSpace*) base;
#line 45 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 45 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 45 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp1_;
#line 45 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 5560 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestSpace* serializable_gee_hash_map_test_space_construct (GType object_type) {
	SerializableGeeHashMapTestSpace * self = NULL;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestSpace*) gxml_serializable_object_model_construct (object_type);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return self;
#line 5570 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestSpace* serializable_gee_hash_map_test_space_new (void) {
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return serializable_gee_hash_map_test_space_construct (SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE);
#line 5577 "SerializableGeeHashMapTest.c"
}


static const gchar* serializable_gee_hash_map_test_space_get_name (SerializableGeeHashMapTestSpace* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp0_;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 5592 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_space_set_name (SerializableGeeHashMapTestSpace* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_if_fail (self != NULL);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = value;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_free0 (self->priv->_name);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv->_name = _tmp1_;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 5611 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestSpaceCollection* serializable_gee_hash_map_test_space_collection_construct (GType object_type) {
	SerializableGeeHashMapTestSpaceCollection * self = NULL;
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestSpaceCollection*) gxml_serializable_hash_map_construct (object_type, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return self;
#line 5621 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestSpaceCollection* serializable_gee_hash_map_test_space_collection_new (void) {
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return serializable_gee_hash_map_test_space_collection_construct (SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_TYPE_COLLECTION);
#line 5628 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_space_collection_class_init (SerializableGeeHashMapTestSpaceCollectionClass * klass) {
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	serializable_gee_hash_map_test_space_collection_parent_class = g_type_class_peek_parent (klass);
#line 5635 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_space_collection_instance_init (SerializableGeeHashMapTestSpaceCollection * self) {
}


GType serializable_gee_hash_map_test_space_collection_get_type (void) {
	static volatile gsize serializable_gee_hash_map_test_space_collection_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_hash_map_test_space_collection_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeHashMapTestSpaceCollectionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_hash_map_test_space_collection_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeHashMapTestSpaceCollection), 0, (GInstanceInitFunc) serializable_gee_hash_map_test_space_collection_instance_init, NULL };
		GType serializable_gee_hash_map_test_space_collection_type_id;
		serializable_gee_hash_map_test_space_collection_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_HASH_MAP, "SerializableGeeHashMapTestSpaceCollection", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_hash_map_test_space_collection_type_id__volatile, serializable_gee_hash_map_test_space_collection_type_id);
	}
	return serializable_gee_hash_map_test_space_collection_type_id__volatile;
}


static void serializable_gee_hash_map_test_space_class_init (SerializableGeeHashMapTestSpaceClass * klass) {
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	serializable_gee_hash_map_test_space_parent_class = g_type_class_peek_parent (klass);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeHashMapTestSpacePrivate));
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->serialize_use_xml_node_value = (gboolean (*)(GXmlSerializableObjectModel*)) serializable_gee_hash_map_test_space_real_serialize_use_xml_node_value;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_hash_map_test_space_real_node_name;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_hash_map_test_space_real_to_string;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_hash_map_test_space_get_property;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_hash_map_test_space_set_property;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_hash_map_test_space_finalize;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 5674 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_space_gxml_serializable_map_key_interface_init (GXmlSerializableMapKeyIface * iface) {
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	serializable_gee_hash_map_test_space_gxml_serializable_map_key_parent_iface = g_type_interface_peek_parent (iface);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	iface->get_map_key = (gpointer (*)(GXmlSerializableMapKey*)) serializable_gee_hash_map_test_space_real_get_map_key;
#line 5683 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_space_instance_init (SerializableGeeHashMapTestSpace * self) {
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv = SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_GET_PRIVATE (self);
#line 5690 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_space_finalize (GObject* obj) {
	SerializableGeeHashMapTestSpace * self;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE, SerializableGeeHashMapTestSpace);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_free0 (self->priv->_name);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (serializable_gee_hash_map_test_space_parent_class)->finalize (obj);
#line 5702 "SerializableGeeHashMapTest.c"
}


static GType serializable_gee_hash_map_test_space_get_type (void) {
	static volatile gsize serializable_gee_hash_map_test_space_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_hash_map_test_space_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeHashMapTestSpaceClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_hash_map_test_space_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeHashMapTestSpace), 0, (GInstanceInitFunc) serializable_gee_hash_map_test_space_instance_init, NULL };
		static const GInterfaceInfo gxml_serializable_map_key_info = { (GInterfaceInitFunc) serializable_gee_hash_map_test_space_gxml_serializable_map_key_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType serializable_gee_hash_map_test_space_type_id;
		serializable_gee_hash_map_test_space_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeHashMapTestSpace", &g_define_type_info, 0);
		g_type_add_interface_static (serializable_gee_hash_map_test_space_type_id, GXML_TYPE_SERIALIZABLE_MAP_KEY, &gxml_serializable_map_key_info);
		g_once_init_leave (&serializable_gee_hash_map_test_space_type_id__volatile, serializable_gee_hash_map_test_space_type_id);
	}
	return serializable_gee_hash_map_test_space_type_id__volatile;
}


static void _vala_serializable_gee_hash_map_test_space_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeHashMapTestSpace * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE, SerializableGeeHashMapTestSpace);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	switch (property_id) {
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		case SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_NAME:
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_value_set_string (value, serializable_gee_hash_map_test_space_get_name (self));
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 5731 "SerializableGeeHashMapTest.c"
		default:
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 5737 "SerializableGeeHashMapTest.c"
	}
}


static void _vala_serializable_gee_hash_map_test_space_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeHashMapTestSpace * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE, SerializableGeeHashMapTestSpace);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	switch (property_id) {
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		case SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_NAME:
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		serializable_gee_hash_map_test_space_set_name (self, g_value_get_string (value));
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 5753 "SerializableGeeHashMapTest.c"
		default:
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 5759 "SerializableGeeHashMapTest.c"
	}
}


static gchar* serializable_gee_hash_map_test_space_container_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeHashMapTestSpaceContainer * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestSpaceContainer*) base;
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = g_strdup ("spacecontainer");
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp0_;
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 5776 "SerializableGeeHashMapTest.c"
}


static gchar* serializable_gee_hash_map_test_space_container_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeHashMapTestSpaceContainer * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestSpaceContainer*) base;
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = self->priv->_owner;
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp1_;
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 5795 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_space_container_real_init_containers (GXmlSerializableContainer* base) {
	SerializableGeeHashMapTestSpaceContainer * self;
	SerializableGeeHashMapTestSpaceCollection* _tmp0_ = NULL;
#line 56 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestSpaceContainer*) base;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = self->priv->_storage;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	if (_tmp0_ == NULL) {
#line 5808 "SerializableGeeHashMapTest.c"
		SerializableGeeHashMapTestSpaceCollection* _tmp1_ = NULL;
		SerializableGeeHashMapTestSpaceCollection* _tmp2_ = NULL;
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp1_ = serializable_gee_hash_map_test_space_collection_new ();
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_tmp2_ = _tmp1_;
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		serializable_gee_hash_map_test_space_container_set_storage (self, _tmp2_);
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		_g_object_unref0 (_tmp2_);
#line 5819 "SerializableGeeHashMapTest.c"
	}
}


static SerializableGeeHashMapTestSpaceContainer* serializable_gee_hash_map_test_space_container_construct (GType object_type) {
	SerializableGeeHashMapTestSpaceContainer * self = NULL;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestSpaceContainer*) gxml_serializable_container_construct (object_type);
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return self;
#line 5830 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestSpaceContainer* serializable_gee_hash_map_test_space_container_new (void) {
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return serializable_gee_hash_map_test_space_container_construct (SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE_CONTAINER);
#line 5837 "SerializableGeeHashMapTest.c"
}


static const gchar* serializable_gee_hash_map_test_space_container_get_owner (SerializableGeeHashMapTestSpaceContainer* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = self->priv->_owner;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp0_;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 5852 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_space_container_set_owner (SerializableGeeHashMapTestSpaceContainer* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_if_fail (self != NULL);
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = value;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_free0 (self->priv->_owner);
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv->_owner = _tmp1_;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_object_notify ((GObject *) self, "owner");
#line 5871 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestSpaceCollection* serializable_gee_hash_map_test_space_container_get_storage (SerializableGeeHashMapTestSpaceContainer* self) {
	SerializableGeeHashMapTestSpaceCollection* result;
	SerializableGeeHashMapTestSpaceCollection* _tmp0_ = NULL;
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = self->priv->_storage;
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp0_;
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 5886 "SerializableGeeHashMapTest.c"
}


static gpointer _g_object_ref0 (gpointer self) {
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return self ? g_object_ref (self) : NULL;
#line 5893 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_space_container_set_storage (SerializableGeeHashMapTestSpaceContainer* self, SerializableGeeHashMapTestSpaceCollection* value) {
	SerializableGeeHashMapTestSpaceCollection* _tmp0_ = NULL;
	SerializableGeeHashMapTestSpaceCollection* _tmp1_ = NULL;
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_if_fail (self != NULL);
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = value;
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_object_unref0 (self->priv->_storage);
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv->_storage = _tmp1_;
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_object_notify ((GObject *) self, "storage");
#line 5912 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_space_container_class_init (SerializableGeeHashMapTestSpaceContainerClass * klass) {
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	serializable_gee_hash_map_test_space_container_parent_class = g_type_class_peek_parent (klass);
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeHashMapTestSpaceContainerPrivate));
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_hash_map_test_space_container_real_node_name;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_hash_map_test_space_container_real_to_string;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	((GXmlSerializableContainerClass *) klass)->init_containers = (void (*)(GXmlSerializableContainer*)) serializable_gee_hash_map_test_space_container_real_init_containers;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_hash_map_test_space_container_get_property;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_hash_map_test_space_container_set_property;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_hash_map_test_space_container_finalize;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_CONTAINER_OWNER, g_param_spec_string ("owner", "owner", "owner", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_CONTAINER_STORAGE, g_param_spec_object ("storage", "storage", "storage", SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_TYPE_COLLECTION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 5937 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_space_container_instance_init (SerializableGeeHashMapTestSpaceContainer * self) {
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv = SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_CONTAINER_GET_PRIVATE (self);
#line 5944 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_space_container_finalize (GObject* obj) {
	SerializableGeeHashMapTestSpaceContainer * self;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE_CONTAINER, SerializableGeeHashMapTestSpaceContainer);
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_free0 (self->priv->_owner);
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_object_unref0 (self->priv->_storage);
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (serializable_gee_hash_map_test_space_container_parent_class)->finalize (obj);
#line 5958 "SerializableGeeHashMapTest.c"
}


static GType serializable_gee_hash_map_test_space_container_get_type (void) {
	static volatile gsize serializable_gee_hash_map_test_space_container_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_hash_map_test_space_container_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeHashMapTestSpaceContainerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_hash_map_test_space_container_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeHashMapTestSpaceContainer), 0, (GInstanceInitFunc) serializable_gee_hash_map_test_space_container_instance_init, NULL };
		GType serializable_gee_hash_map_test_space_container_type_id;
		serializable_gee_hash_map_test_space_container_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_CONTAINER, "SerializableGeeHashMapTestSpaceContainer", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_hash_map_test_space_container_type_id__volatile, serializable_gee_hash_map_test_space_container_type_id);
	}
	return serializable_gee_hash_map_test_space_container_type_id__volatile;
}


static void _vala_serializable_gee_hash_map_test_space_container_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeHashMapTestSpaceContainer * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE_CONTAINER, SerializableGeeHashMapTestSpaceContainer);
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	switch (property_id) {
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		case SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_CONTAINER_OWNER:
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_value_set_string (value, serializable_gee_hash_map_test_space_container_get_owner (self));
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		case SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_CONTAINER_STORAGE:
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_value_set_object (value, serializable_gee_hash_map_test_space_container_get_storage (self));
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 5991 "SerializableGeeHashMapTest.c"
		default:
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 5997 "SerializableGeeHashMapTest.c"
	}
}


static void _vala_serializable_gee_hash_map_test_space_container_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeHashMapTestSpaceContainer * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SPACE_CONTAINER, SerializableGeeHashMapTestSpaceContainer);
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	switch (property_id) {
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		case SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_CONTAINER_OWNER:
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		serializable_gee_hash_map_test_space_container_set_owner (self, g_value_get_string (value));
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		case SERIALIZABLE_GEE_HASH_MAP_TEST_SPACE_CONTAINER_STORAGE:
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		serializable_gee_hash_map_test_space_container_set_storage (self, g_value_get_object (value));
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 6019 "SerializableGeeHashMapTest.c"
		default:
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 6025 "SerializableGeeHashMapTest.c"
	}
}


static void serializable_gee_hash_map_test_ball_fill_set_text (SerializableGeeHashMapTestBallFill* self, const gchar* txt) {
	const gchar* _tmp0_ = NULL;
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_if_fail (self != NULL);
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_if_fail (txt != NULL);
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = txt;
#line 65 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	gxml_serializable_set_serialized_xml_node_value ((GXmlSerializable*) self, _tmp0_);
#line 6040 "SerializableGeeHashMapTest.c"
}


static gchar* serializable_gee_hash_map_test_ball_fill_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeHashMapTestBallFill * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestBallFill*) base;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp1_;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 6059 "SerializableGeeHashMapTest.c"
}


static gchar* serializable_gee_hash_map_test_ball_fill_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeHashMapTestBallFill * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 67 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestBallFill*) base;
#line 67 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = g_strdup ("BallFill");
#line 67 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp0_;
#line 67 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 6075 "SerializableGeeHashMapTest.c"
}


static gboolean serializable_gee_hash_map_test_ball_fill_real_serialize_use_xml_node_value (GXmlSerializableObjectModel* base) {
	SerializableGeeHashMapTestBallFill * self;
	gboolean result = FALSE;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestBallFill*) base;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = TRUE;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 6088 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestBallFill* serializable_gee_hash_map_test_ball_fill_construct (GType object_type) {
	SerializableGeeHashMapTestBallFill * self = NULL;
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestBallFill*) gxml_serializable_object_model_construct (object_type);
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return self;
#line 6098 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestBallFill* serializable_gee_hash_map_test_ball_fill_new (void) {
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return serializable_gee_hash_map_test_ball_fill_construct (SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL_FILL);
#line 6105 "SerializableGeeHashMapTest.c"
}


static const gchar* serializable_gee_hash_map_test_ball_fill_get_name (SerializableGeeHashMapTestBallFill* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp0_;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 6120 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_ball_fill_set_name (SerializableGeeHashMapTestBallFill* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_if_fail (self != NULL);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = value;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_free0 (self->priv->_name);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv->_name = _tmp1_;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 6139 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_ball_fill_class_init (SerializableGeeHashMapTestBallFillClass * klass) {
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	serializable_gee_hash_map_test_ball_fill_parent_class = g_type_class_peek_parent (klass);
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeHashMapTestBallFillPrivate));
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_hash_map_test_ball_fill_real_to_string;
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_hash_map_test_ball_fill_real_node_name;
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->serialize_use_xml_node_value = (gboolean (*)(GXmlSerializableObjectModel*)) serializable_gee_hash_map_test_ball_fill_real_serialize_use_xml_node_value;
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_hash_map_test_ball_fill_get_property;
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_hash_map_test_ball_fill_set_property;
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_hash_map_test_ball_fill_finalize;
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_FILL_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 6162 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_ball_fill_instance_init (SerializableGeeHashMapTestBallFill * self) {
	gchar* _tmp0_ = NULL;
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv = SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_FILL_GET_PRIVATE (self);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = g_strdup ("Fill");
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv->_name = _tmp0_;
#line 6174 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_ball_fill_finalize (GObject* obj) {
	SerializableGeeHashMapTestBallFill * self;
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL_FILL, SerializableGeeHashMapTestBallFill);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_free0 (self->priv->_name);
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (serializable_gee_hash_map_test_ball_fill_parent_class)->finalize (obj);
#line 6186 "SerializableGeeHashMapTest.c"
}


static GType serializable_gee_hash_map_test_ball_fill_get_type (void) {
	static volatile gsize serializable_gee_hash_map_test_ball_fill_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_hash_map_test_ball_fill_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeHashMapTestBallFillClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_hash_map_test_ball_fill_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeHashMapTestBallFill), 0, (GInstanceInitFunc) serializable_gee_hash_map_test_ball_fill_instance_init, NULL };
		GType serializable_gee_hash_map_test_ball_fill_type_id;
		serializable_gee_hash_map_test_ball_fill_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeHashMapTestBallFill", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_hash_map_test_ball_fill_type_id__volatile, serializable_gee_hash_map_test_ball_fill_type_id);
	}
	return serializable_gee_hash_map_test_ball_fill_type_id__volatile;
}


static void _vala_serializable_gee_hash_map_test_ball_fill_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeHashMapTestBallFill * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL_FILL, SerializableGeeHashMapTestBallFill);
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	switch (property_id) {
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		case SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_FILL_NAME:
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_value_set_string (value, serializable_gee_hash_map_test_ball_fill_get_name (self));
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 6213 "SerializableGeeHashMapTest.c"
		default:
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 6219 "SerializableGeeHashMapTest.c"
	}
}


static void _vala_serializable_gee_hash_map_test_ball_fill_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeHashMapTestBallFill * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL_FILL, SerializableGeeHashMapTestBallFill);
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	switch (property_id) {
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		case SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_FILL_NAME:
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		serializable_gee_hash_map_test_ball_fill_set_name (self, g_value_get_string (value));
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 6235 "SerializableGeeHashMapTest.c"
		default:
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 6241 "SerializableGeeHashMapTest.c"
	}
}


static gchar* serializable_gee_hash_map_test_ball_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeHashMapTestBall * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestBall*) base;
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp1_;
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 6261 "SerializableGeeHashMapTest.c"
}


static gchar* serializable_gee_hash_map_test_ball_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeHashMapTestBall * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestBall*) base;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = g_strdup ("Ball");
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp0_;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 6277 "SerializableGeeHashMapTest.c"
}


static gchar* serializable_gee_hash_map_test_ball_real_get_map_key (GXmlSerializableMapKey* base) {
	SerializableGeeHashMapTestBall * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestBall*) base;
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp1_;
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 6296 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestBall* serializable_gee_hash_map_test_ball_construct (GType object_type) {
	SerializableGeeHashMapTestBall * self = NULL;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestBall*) gxml_serializable_object_model_construct (object_type);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return self;
#line 6306 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestBall* serializable_gee_hash_map_test_ball_new (void) {
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return serializable_gee_hash_map_test_ball_construct (SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL);
#line 6313 "SerializableGeeHashMapTest.c"
}


static const gchar* serializable_gee_hash_map_test_ball_get_name (SerializableGeeHashMapTestBall* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp0_;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 6328 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_ball_set_name (SerializableGeeHashMapTestBall* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_if_fail (self != NULL);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = value;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_free0 (self->priv->_name);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv->_name = _tmp1_;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 6347 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestBallFill* serializable_gee_hash_map_test_ball_get_ballfill (SerializableGeeHashMapTestBall* self) {
	SerializableGeeHashMapTestBallFill* result;
	SerializableGeeHashMapTestBallFill* _tmp0_ = NULL;
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = self->priv->_ballfill;
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp0_;
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 6362 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_ball_set_ballfill (SerializableGeeHashMapTestBall* self, SerializableGeeHashMapTestBallFill* value) {
	SerializableGeeHashMapTestBallFill* _tmp0_ = NULL;
	SerializableGeeHashMapTestBallFill* _tmp1_ = NULL;
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_if_fail (self != NULL);
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = value;
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_object_unref0 (self->priv->_ballfill);
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv->_ballfill = _tmp1_;
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_object_notify ((GObject *) self, "ballfill");
#line 6381 "SerializableGeeHashMapTest.c"
}


static gboolean serializable_gee_hash_map_test_ball_hash_map_real_deserialize_proceed (GXmlSerializableHashMap* base) {
	SerializableGeeHashMapTestBallHashMap * self;
	gboolean result = FALSE;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestBallHashMap*) base;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = FALSE;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 6394 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestBallHashMap* serializable_gee_hash_map_test_ball_hash_map_construct (GType object_type) {
	SerializableGeeHashMapTestBallHashMap * self = NULL;
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestBallHashMap*) gxml_serializable_hash_map_construct (object_type, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return self;
#line 6404 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestBallHashMap* serializable_gee_hash_map_test_ball_hash_map_new (void) {
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return serializable_gee_hash_map_test_ball_hash_map_construct (SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_TYPE_HASH_MAP);
#line 6411 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_ball_hash_map_class_init (SerializableGeeHashMapTestBallHashMapClass * klass) {
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	serializable_gee_hash_map_test_ball_hash_map_parent_class = g_type_class_peek_parent (klass);
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	((GXmlSerializableHashMapClass *) klass)->deserialize_proceed = (gboolean (*)(GXmlSerializableHashMap*)) serializable_gee_hash_map_test_ball_hash_map_real_deserialize_proceed;
#line 6420 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_ball_hash_map_instance_init (SerializableGeeHashMapTestBallHashMap * self) {
}


GType serializable_gee_hash_map_test_ball_hash_map_get_type (void) {
	static volatile gsize serializable_gee_hash_map_test_ball_hash_map_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_hash_map_test_ball_hash_map_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeHashMapTestBallHashMapClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_hash_map_test_ball_hash_map_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeHashMapTestBallHashMap), 0, (GInstanceInitFunc) serializable_gee_hash_map_test_ball_hash_map_instance_init, NULL };
		GType serializable_gee_hash_map_test_ball_hash_map_type_id;
		serializable_gee_hash_map_test_ball_hash_map_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_HASH_MAP, "SerializableGeeHashMapTestBallHashMap", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_hash_map_test_ball_hash_map_type_id__volatile, serializable_gee_hash_map_test_ball_hash_map_type_id);
	}
	return serializable_gee_hash_map_test_ball_hash_map_type_id__volatile;
}


static void serializable_gee_hash_map_test_ball_class_init (SerializableGeeHashMapTestBallClass * klass) {
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	serializable_gee_hash_map_test_ball_parent_class = g_type_class_peek_parent (klass);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeHashMapTestBallPrivate));
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_hash_map_test_ball_real_to_string;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_hash_map_test_ball_real_node_name;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_hash_map_test_ball_get_property;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_hash_map_test_ball_set_property;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_hash_map_test_ball_finalize;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_BALLFILL, g_param_spec_object ("ballfill", "ballfill", "ballfill", SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL_FILL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 6459 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_ball_gxml_serializable_map_key_interface_init (GXmlSerializableMapKeyIface * iface) {
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	serializable_gee_hash_map_test_ball_gxml_serializable_map_key_parent_iface = g_type_interface_peek_parent (iface);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	iface->get_map_key = (gpointer (*)(GXmlSerializableMapKey*)) serializable_gee_hash_map_test_ball_real_get_map_key;
#line 6468 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_ball_instance_init (SerializableGeeHashMapTestBall * self) {
	gchar* _tmp0_ = NULL;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv = SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_GET_PRIVATE (self);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = g_strdup ("Ball");
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv->_name = _tmp0_;
#line 6480 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_ball_finalize (GObject* obj) {
	SerializableGeeHashMapTestBall * self;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL, SerializableGeeHashMapTestBall);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_free0 (self->priv->_name);
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_object_unref0 (self->priv->_ballfill);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (serializable_gee_hash_map_test_ball_parent_class)->finalize (obj);
#line 6494 "SerializableGeeHashMapTest.c"
}


static GType serializable_gee_hash_map_test_ball_get_type (void) {
	static volatile gsize serializable_gee_hash_map_test_ball_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_hash_map_test_ball_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeHashMapTestBallClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_hash_map_test_ball_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeHashMapTestBall), 0, (GInstanceInitFunc) serializable_gee_hash_map_test_ball_instance_init, NULL };
		static const GInterfaceInfo gxml_serializable_map_key_info = { (GInterfaceInitFunc) serializable_gee_hash_map_test_ball_gxml_serializable_map_key_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType serializable_gee_hash_map_test_ball_type_id;
		serializable_gee_hash_map_test_ball_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeHashMapTestBall", &g_define_type_info, 0);
		g_type_add_interface_static (serializable_gee_hash_map_test_ball_type_id, GXML_TYPE_SERIALIZABLE_MAP_KEY, &gxml_serializable_map_key_info);
		g_once_init_leave (&serializable_gee_hash_map_test_ball_type_id__volatile, serializable_gee_hash_map_test_ball_type_id);
	}
	return serializable_gee_hash_map_test_ball_type_id__volatile;
}


static void _vala_serializable_gee_hash_map_test_ball_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeHashMapTestBall * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL, SerializableGeeHashMapTestBall);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	switch (property_id) {
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		case SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_NAME:
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_value_set_string (value, serializable_gee_hash_map_test_ball_get_name (self));
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		case SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_BALLFILL:
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_value_set_object (value, serializable_gee_hash_map_test_ball_get_ballfill (self));
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 6529 "SerializableGeeHashMapTest.c"
		default:
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 6535 "SerializableGeeHashMapTest.c"
	}
}


static void _vala_serializable_gee_hash_map_test_ball_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeHashMapTestBall * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BALL, SerializableGeeHashMapTestBall);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	switch (property_id) {
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		case SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_NAME:
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		serializable_gee_hash_map_test_ball_set_name (self, g_value_get_string (value));
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		case SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_BALLFILL:
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		serializable_gee_hash_map_test_ball_set_ballfill (self, g_value_get_object (value));
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 6557 "SerializableGeeHashMapTest.c"
		default:
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 6563 "SerializableGeeHashMapTest.c"
	}
}


static gchar* serializable_gee_hash_map_test_small_bag_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeHashMapTestSmallBag * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 87 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestSmallBag*) base;
#line 87 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 87 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 87 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp1_;
#line 87 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 6583 "SerializableGeeHashMapTest.c"
}


static gchar* serializable_gee_hash_map_test_small_bag_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeHashMapTestSmallBag * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestSmallBag*) base;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = g_strdup ("SmallBag");
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp0_;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 6599 "SerializableGeeHashMapTest.c"
}


static gchar* serializable_gee_hash_map_test_small_bag_real_get_map_key (GXmlSerializableMapKey* base) {
	SerializableGeeHashMapTestSmallBag * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 89 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestSmallBag*) base;
#line 89 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 89 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 89 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp1_;
#line 89 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 6618 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestSmallBag* serializable_gee_hash_map_test_small_bag_construct (GType object_type) {
	SerializableGeeHashMapTestSmallBag * self = NULL;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestSmallBag*) gxml_serializable_object_model_construct (object_type);
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return self;
#line 6628 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestSmallBag* serializable_gee_hash_map_test_small_bag_new (void) {
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return serializable_gee_hash_map_test_small_bag_construct (SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SMALL_BAG);
#line 6635 "SerializableGeeHashMapTest.c"
}


static const gchar* serializable_gee_hash_map_test_small_bag_get_name (SerializableGeeHashMapTestSmallBag* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp0_;
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 6650 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_small_bag_set_name (SerializableGeeHashMapTestSmallBag* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_if_fail (self != NULL);
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = value;
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_free0 (self->priv->_name);
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv->_name = _tmp1_;
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 6669 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestBallHashMap* serializable_gee_hash_map_test_small_bag_get_balls (SerializableGeeHashMapTestSmallBag* self) {
	SerializableGeeHashMapTestBallHashMap* result;
	SerializableGeeHashMapTestBallHashMap* _tmp0_ = NULL;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = self->priv->_balls;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp0_;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 6684 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_small_bag_set_balls (SerializableGeeHashMapTestSmallBag* self, SerializableGeeHashMapTestBallHashMap* value) {
	SerializableGeeHashMapTestBallHashMap* _tmp0_ = NULL;
	SerializableGeeHashMapTestBallHashMap* _tmp1_ = NULL;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_if_fail (self != NULL);
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = value;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_object_unref0 (self->priv->_balls);
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv->_balls = _tmp1_;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_object_notify ((GObject *) self, "balls");
#line 6703 "SerializableGeeHashMapTest.c"
}


static gboolean serializable_gee_hash_map_test_small_bag_hash_map_real_deserialize_proceed (GXmlSerializableHashMap* base) {
	SerializableGeeHashMapTestSmallBagHashMap * self;
	gboolean result = FALSE;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestSmallBagHashMap*) base;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = FALSE;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 6716 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestSmallBagHashMap* serializable_gee_hash_map_test_small_bag_hash_map_construct (GType object_type) {
	SerializableGeeHashMapTestSmallBagHashMap * self = NULL;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestSmallBagHashMap*) gxml_serializable_hash_map_construct (object_type, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SMALL_BAG, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return self;
#line 6726 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestSmallBagHashMap* serializable_gee_hash_map_test_small_bag_hash_map_new (void) {
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return serializable_gee_hash_map_test_small_bag_hash_map_construct (SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_TYPE_HASH_MAP);
#line 6733 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_small_bag_hash_map_class_init (SerializableGeeHashMapTestSmallBagHashMapClass * klass) {
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	serializable_gee_hash_map_test_small_bag_hash_map_parent_class = g_type_class_peek_parent (klass);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	((GXmlSerializableHashMapClass *) klass)->deserialize_proceed = (gboolean (*)(GXmlSerializableHashMap*)) serializable_gee_hash_map_test_small_bag_hash_map_real_deserialize_proceed;
#line 6742 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_small_bag_hash_map_instance_init (SerializableGeeHashMapTestSmallBagHashMap * self) {
}


GType serializable_gee_hash_map_test_small_bag_hash_map_get_type (void) {
	static volatile gsize serializable_gee_hash_map_test_small_bag_hash_map_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_hash_map_test_small_bag_hash_map_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeHashMapTestSmallBagHashMapClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_hash_map_test_small_bag_hash_map_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeHashMapTestSmallBagHashMap), 0, (GInstanceInitFunc) serializable_gee_hash_map_test_small_bag_hash_map_instance_init, NULL };
		GType serializable_gee_hash_map_test_small_bag_hash_map_type_id;
		serializable_gee_hash_map_test_small_bag_hash_map_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_HASH_MAP, "SerializableGeeHashMapTestSmallBagHashMap", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_hash_map_test_small_bag_hash_map_type_id__volatile, serializable_gee_hash_map_test_small_bag_hash_map_type_id);
	}
	return serializable_gee_hash_map_test_small_bag_hash_map_type_id__volatile;
}


static void serializable_gee_hash_map_test_small_bag_class_init (SerializableGeeHashMapTestSmallBagClass * klass) {
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	serializable_gee_hash_map_test_small_bag_parent_class = g_type_class_peek_parent (klass);
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeHashMapTestSmallBagPrivate));
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_hash_map_test_small_bag_real_to_string;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_hash_map_test_small_bag_real_node_name;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_hash_map_test_small_bag_get_property;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_hash_map_test_small_bag_set_property;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_hash_map_test_small_bag_finalize;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_BALLS, g_param_spec_object ("balls", "balls", "balls", SERIALIZABLE_GEE_HASH_MAP_TEST_BALL_TYPE_HASH_MAP, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 6781 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_small_bag_gxml_serializable_map_key_interface_init (GXmlSerializableMapKeyIface * iface) {
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	serializable_gee_hash_map_test_small_bag_gxml_serializable_map_key_parent_iface = g_type_interface_peek_parent (iface);
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	iface->get_map_key = (gpointer (*)(GXmlSerializableMapKey*)) serializable_gee_hash_map_test_small_bag_real_get_map_key;
#line 6790 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_small_bag_instance_init (SerializableGeeHashMapTestSmallBag * self) {
	gchar* _tmp0_ = NULL;
	SerializableGeeHashMapTestBallHashMap* _tmp1_ = NULL;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv = SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_GET_PRIVATE (self);
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = g_strdup ("SmallBag");
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv->_name = _tmp0_;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = serializable_gee_hash_map_test_ball_hash_map_new ();
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv->_balls = _tmp1_;
#line 6807 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_small_bag_finalize (GObject* obj) {
	SerializableGeeHashMapTestSmallBag * self;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SMALL_BAG, SerializableGeeHashMapTestSmallBag);
#line 85 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_free0 (self->priv->_name);
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_object_unref0 (self->priv->_balls);
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (serializable_gee_hash_map_test_small_bag_parent_class)->finalize (obj);
#line 6821 "SerializableGeeHashMapTest.c"
}


static GType serializable_gee_hash_map_test_small_bag_get_type (void) {
	static volatile gsize serializable_gee_hash_map_test_small_bag_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_hash_map_test_small_bag_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeHashMapTestSmallBagClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_hash_map_test_small_bag_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeHashMapTestSmallBag), 0, (GInstanceInitFunc) serializable_gee_hash_map_test_small_bag_instance_init, NULL };
		static const GInterfaceInfo gxml_serializable_map_key_info = { (GInterfaceInitFunc) serializable_gee_hash_map_test_small_bag_gxml_serializable_map_key_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType serializable_gee_hash_map_test_small_bag_type_id;
		serializable_gee_hash_map_test_small_bag_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeHashMapTestSmallBag", &g_define_type_info, 0);
		g_type_add_interface_static (serializable_gee_hash_map_test_small_bag_type_id, GXML_TYPE_SERIALIZABLE_MAP_KEY, &gxml_serializable_map_key_info);
		g_once_init_leave (&serializable_gee_hash_map_test_small_bag_type_id__volatile, serializable_gee_hash_map_test_small_bag_type_id);
	}
	return serializable_gee_hash_map_test_small_bag_type_id__volatile;
}


static void _vala_serializable_gee_hash_map_test_small_bag_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeHashMapTestSmallBag * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SMALL_BAG, SerializableGeeHashMapTestSmallBag);
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	switch (property_id) {
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		case SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_NAME:
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_value_set_string (value, serializable_gee_hash_map_test_small_bag_get_name (self));
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		case SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_BALLS:
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_value_set_object (value, serializable_gee_hash_map_test_small_bag_get_balls (self));
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 6856 "SerializableGeeHashMapTest.c"
		default:
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 6862 "SerializableGeeHashMapTest.c"
	}
}


static void _vala_serializable_gee_hash_map_test_small_bag_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeHashMapTestSmallBag * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_SMALL_BAG, SerializableGeeHashMapTestSmallBag);
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	switch (property_id) {
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		case SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_NAME:
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		serializable_gee_hash_map_test_small_bag_set_name (self, g_value_get_string (value));
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		case SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_BALLS:
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		serializable_gee_hash_map_test_small_bag_set_balls (self, g_value_get_object (value));
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 6884 "SerializableGeeHashMapTest.c"
		default:
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 6890 "SerializableGeeHashMapTest.c"
	}
}


static gchar* serializable_gee_hash_map_test_big_bag_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeHashMapTestBigBag * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 99 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestBigBag*) base;
#line 99 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 99 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 99 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp1_;
#line 99 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 6910 "SerializableGeeHashMapTest.c"
}


static gchar* serializable_gee_hash_map_test_big_bag_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeHashMapTestBigBag * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestBigBag*) base;
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = g_strdup ("BigBag");
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp0_;
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 6926 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestBigBag* serializable_gee_hash_map_test_big_bag_construct (GType object_type) {
	SerializableGeeHashMapTestBigBag * self = NULL;
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = (SerializableGeeHashMapTestBigBag*) gxml_serializable_object_model_construct (object_type);
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return self;
#line 6936 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestBigBag* serializable_gee_hash_map_test_big_bag_new (void) {
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return serializable_gee_hash_map_test_big_bag_construct (SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BIG_BAG);
#line 6943 "SerializableGeeHashMapTest.c"
}


static const gchar* serializable_gee_hash_map_test_big_bag_get_name (SerializableGeeHashMapTestBigBag* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 97 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 97 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = self->priv->_name;
#line 97 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp0_;
#line 97 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 6958 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_big_bag_set_name (SerializableGeeHashMapTestBigBag* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 97 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_if_fail (self != NULL);
#line 97 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = value;
#line 97 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 97 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_free0 (self->priv->_name);
#line 97 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv->_name = _tmp1_;
#line 97 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 6977 "SerializableGeeHashMapTest.c"
}


static SerializableGeeHashMapTestSmallBagHashMap* serializable_gee_hash_map_test_big_bag_get_bags (SerializableGeeHashMapTestBigBag* self) {
	SerializableGeeHashMapTestSmallBagHashMap* result;
	SerializableGeeHashMapTestSmallBagHashMap* _tmp0_ = NULL;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = self->priv->_bags;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	result = _tmp0_;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	return result;
#line 6992 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_big_bag_set_bags (SerializableGeeHashMapTestBigBag* self, SerializableGeeHashMapTestSmallBagHashMap* value) {
	SerializableGeeHashMapTestSmallBagHashMap* _tmp0_ = NULL;
	SerializableGeeHashMapTestSmallBagHashMap* _tmp1_ = NULL;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_return_if_fail (self != NULL);
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = value;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_object_unref0 (self->priv->_bags);
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv->_bags = _tmp1_;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_object_notify ((GObject *) self, "bags");
#line 7011 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_big_bag_class_init (SerializableGeeHashMapTestBigBagClass * klass) {
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	serializable_gee_hash_map_test_big_bag_parent_class = g_type_class_peek_parent (klass);
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeHashMapTestBigBagPrivate));
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_hash_map_test_big_bag_real_to_string;
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_hash_map_test_big_bag_real_node_name;
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_hash_map_test_big_bag_get_property;
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_hash_map_test_big_bag_set_property;
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_hash_map_test_big_bag_finalize;
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_HASH_MAP_TEST_BIG_BAG_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_HASH_MAP_TEST_BIG_BAG_BAGS, g_param_spec_object ("bags", "bags", "bags", SERIALIZABLE_GEE_HASH_MAP_TEST_SMALL_BAG_TYPE_HASH_MAP, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 7034 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_big_bag_instance_init (SerializableGeeHashMapTestBigBag * self) {
	gchar* _tmp0_ = NULL;
	SerializableGeeHashMapTestSmallBagHashMap* _tmp1_ = NULL;
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv = SERIALIZABLE_GEE_HASH_MAP_TEST_BIG_BAG_GET_PRIVATE (self);
#line 97 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp0_ = g_strdup ("ball");
#line 97 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv->_name = _tmp0_;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_tmp1_ = serializable_gee_hash_map_test_small_bag_hash_map_new ();
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self->priv->_bags = _tmp1_;
#line 7051 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_big_bag_finalize (GObject* obj) {
	SerializableGeeHashMapTestBigBag * self;
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BIG_BAG, SerializableGeeHashMapTestBigBag);
#line 97 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_free0 (self->priv->_name);
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	_g_object_unref0 (self->priv->_bags);
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	G_OBJECT_CLASS (serializable_gee_hash_map_test_big_bag_parent_class)->finalize (obj);
#line 7065 "SerializableGeeHashMapTest.c"
}


static GType serializable_gee_hash_map_test_big_bag_get_type (void) {
	static volatile gsize serializable_gee_hash_map_test_big_bag_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_hash_map_test_big_bag_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeHashMapTestBigBagClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_hash_map_test_big_bag_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeHashMapTestBigBag), 0, (GInstanceInitFunc) serializable_gee_hash_map_test_big_bag_instance_init, NULL };
		GType serializable_gee_hash_map_test_big_bag_type_id;
		serializable_gee_hash_map_test_big_bag_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeHashMapTestBigBag", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_hash_map_test_big_bag_type_id__volatile, serializable_gee_hash_map_test_big_bag_type_id);
	}
	return serializable_gee_hash_map_test_big_bag_type_id__volatile;
}


static void _vala_serializable_gee_hash_map_test_big_bag_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeHashMapTestBigBag * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BIG_BAG, SerializableGeeHashMapTestBigBag);
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	switch (property_id) {
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		case SERIALIZABLE_GEE_HASH_MAP_TEST_BIG_BAG_NAME:
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_value_set_string (value, serializable_gee_hash_map_test_big_bag_get_name (self));
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		case SERIALIZABLE_GEE_HASH_MAP_TEST_BIG_BAG_BAGS:
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		g_value_set_object (value, serializable_gee_hash_map_test_big_bag_get_bags (self));
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 7098 "SerializableGeeHashMapTest.c"
		default:
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 7104 "SerializableGeeHashMapTest.c"
	}
}


static void _vala_serializable_gee_hash_map_test_big_bag_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeHashMapTestBigBag * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_HASH_MAP_TEST_TYPE_BIG_BAG, SerializableGeeHashMapTestBigBag);
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	switch (property_id) {
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		case SERIALIZABLE_GEE_HASH_MAP_TEST_BIG_BAG_NAME:
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		serializable_gee_hash_map_test_big_bag_set_name (self, g_value_get_string (value));
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		case SERIALIZABLE_GEE_HASH_MAP_TEST_BIG_BAG_BAGS:
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		serializable_gee_hash_map_test_big_bag_set_bags (self, g_value_get_object (value));
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 7126 "SerializableGeeHashMapTest.c"
		default:
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
		break;
#line 7132 "SerializableGeeHashMapTest.c"
	}
}


static void serializable_gee_hash_map_test_class_init (SerializableGeeHashMapTestClass * klass) {
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeHashMapTest.vala"
	serializable_gee_hash_map_test_parent_class = g_type_class_peek_parent (klass);
#line 7140 "SerializableGeeHashMapTest.c"
}


static void serializable_gee_hash_map_test_instance_init (SerializableGeeHashMapTest * self) {
}


/**
 *
 *  GXml.Serializable.BasicTypeTest
 *
 *  Authors:
 *
 *       Daniel Espinosa <esodan@gmail.com>
 *
 *
 *  Copyright (c) 2013-2015 Daniel Espinosa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
GType serializable_gee_hash_map_test_get_type (void) {
	static volatile gsize serializable_gee_hash_map_test_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_hash_map_test_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeHashMapTestClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_hash_map_test_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeHashMapTest), 0, (GInstanceInitFunc) serializable_gee_hash_map_test_instance_init, NULL };
		GType serializable_gee_hash_map_test_type_id;
		serializable_gee_hash_map_test_type_id = g_type_register_static (TYPE_GXML_TEST, "SerializableGeeHashMapTest", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_hash_map_test_type_id__volatile, serializable_gee_hash_map_test_type_id);
	}
	return serializable_gee_hash_map_test_type_id__volatile;
}



