/* TDocumentTest.c generated by valac 0.34.6, the Vala compiler
 * generated from TDocumentTest.vala, do not modify */

/* -*- Mode: vala; indent-tabs-mode: t; c-basic-offset: 2; tab-width: 2 -*- */
/* Notation.vala
 *
 * Copyright (C) 2011-2015  Daniel Espinosa <esodan@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Daniel Espinosa <esodan@gmail.com>
 */

#include <glib.h>
#include <glib-object.h>
#include <gxml/gxml.h>
#include <gee.h>
#include <gio/gio.h>
#include <stdlib.h>
#include <string.h>
#include <libxml/xmlreader.h>


#define TYPE_GXML_TEST (gxml_test_get_type ())
#define GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GXML_TEST, GXmlTest))
#define GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GXML_TEST, GXmlTestClass))
#define IS_GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GXML_TEST))
#define IS_GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GXML_TEST))
#define GXML_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GXML_TEST, GXmlTestClass))

typedef struct _GXmlTest GXmlTest;
typedef struct _GXmlTestClass GXmlTestClass;
typedef struct _GXmlTestPrivate GXmlTestPrivate;

#define TYPE_TDOCUMENT_TEST (tdocument_test_get_type ())
#define TDOCUMENT_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TDOCUMENT_TEST, TDocumentTest))
#define TDOCUMENT_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TDOCUMENT_TEST, TDocumentTestClass))
#define IS_TDOCUMENT_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TDOCUMENT_TEST))
#define IS_TDOCUMENT_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TDOCUMENT_TEST))
#define TDOCUMENT_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TDOCUMENT_TEST, TDocumentTestClass))

typedef struct _TDocumentTest TDocumentTest;
typedef struct _TDocumentTestClass TDocumentTestClass;
typedef struct _TDocumentTestPrivate TDocumentTestPrivate;
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))

#define TDOCUMENT_TEST_TYPE_TTEST_OBJECT (tdocument_test_ttest_object_get_type ())
#define TDOCUMENT_TEST_TTEST_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TDOCUMENT_TEST_TYPE_TTEST_OBJECT, TDocumentTestTTestObject))
#define TDOCUMENT_TEST_TTEST_OBJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TDOCUMENT_TEST_TYPE_TTEST_OBJECT, TDocumentTestTTestObjectClass))
#define TDOCUMENT_TEST_IS_TTEST_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TDOCUMENT_TEST_TYPE_TTEST_OBJECT))
#define TDOCUMENT_TEST_IS_TTEST_OBJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TDOCUMENT_TEST_TYPE_TTEST_OBJECT))
#define TDOCUMENT_TEST_TTEST_OBJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TDOCUMENT_TEST_TYPE_TTEST_OBJECT, TDocumentTestTTestObjectClass))

typedef struct _TDocumentTestTTestObject TDocumentTestTTestObject;
typedef struct _TDocumentTestTTestObjectClass TDocumentTestTTestObjectClass;
typedef struct _TDocumentTestTTestObjectPrivate TDocumentTestTTestObjectPrivate;
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _GXmlTest {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GXmlTestPrivate * priv;
};

struct _GXmlTestClass {
	GTypeClass parent_class;
	void (*finalize) (GXmlTest *self);
};

struct _TDocumentTest {
	GXmlTest parent_instance;
	TDocumentTestPrivate * priv;
};

struct _TDocumentTestClass {
	GXmlTestClass parent_class;
};

struct _TDocumentTestTTestObject {
	GXmlSerializableObjectModel parent_instance;
	TDocumentTestTTestObjectPrivate * priv;
};

struct _TDocumentTestTTestObjectClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _TDocumentTestTTestObjectPrivate {
	gchar* _name;
};


static gpointer tdocument_test_parent_class = NULL;
static gpointer tdocument_test_ttest_object_parent_class = NULL;

gpointer gxml_test_ref (gpointer instance);
void gxml_test_unref (gpointer instance);
GParamSpec* param_spec_gxml_test (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_gxml_test (GValue* value, gpointer v_object);
void value_take_gxml_test (GValue* value, gpointer v_object);
gpointer value_get_gxml_test (const GValue* value);
GType gxml_test_get_type (void) G_GNUC_CONST;
GType tdocument_test_get_type (void) G_GNUC_CONST;
enum  {
	TDOCUMENT_TEST_DUMMY_PROPERTY
};
void tdocument_test_add_tests (void);
static void __lambda105_ (void);
static void ___lambda105__gtest_func (void);
static void __lambda106_ (void);
static void ___lambda106__gtest_func (void);
static void __lambda107_ (void);
static void ___lambda107__gtest_func (void);
static void __lambda108_ (void);
static void ___lambda108__gtest_func (void);
static void __lambda109_ (void);
static void ___lambda109__gtest_func (void);
static void __lambda110_ (void);
static void ___lambda110__gtest_func (void);
static void __lambda111_ (void);
static void ___lambda111__gtest_func (void);
static void __lambda112_ (void);
static void ___lambda112__gtest_func (void);
static void __lambda113_ (void);
GType tdocument_test_ttest_object_get_type (void) G_GNUC_CONST;
TDocumentTestTTestObject* tdocument_test_ttest_object_new (void);
TDocumentTestTTestObject* tdocument_test_ttest_object_construct (GType object_type);
void tdocument_test_ttest_object_set_name (TDocumentTestTTestObject* self, const gchar* value);
static void ___lambda113__gtest_func (void);
static void __lambda114_ (void);
static void ___lambda114__gtest_func (void);
static void __lambda115_ (void);
static void ___lambda115__gtest_func (void);
static void __lambda116_ (void);
static void ___lambda116__gtest_func (void);
static void __lambda117_ (void);
static void ___lambda117__gtest_func (void);
static void __lambda118_ (void);
static void ___lambda118__gtest_func (void);
static void __lambda119_ (void);
static void ___lambda119__gtest_func (void);
static void __lambda120_ (void);
static void ___lambda120__gtest_func (void);
static void __lambda121_ (void);
static void ___lambda121__gtest_func (void);
static void __lambda122_ (void);
static void ___lambda122__gtest_func (void);
static void __lambda123_ (void);
static void ___lambda123__gtest_func (void);
static void __lambda124_ (void);
static void ___lambda124__gtest_func (void);
static void __lambda125_ (void);
static void ___lambda125__gtest_func (void);
static void __lambda126_ (void);
static GXmlTDocumentReadType ___lambda127_ (GXmlNode* node, xmlTextReader* tr);
static GXmlTDocumentReadType ____lambda127__gxml_tdocument_read_type_func (GXmlNode* node, xmlTextReader* tr, gpointer self);
static GXmlTDocumentReadType ___lambda128_ (GXmlNode* node, xmlTextReader* tr);
static GXmlTDocumentReadType ____lambda128__gxml_tdocument_read_type_func (GXmlNode* node, xmlTextReader* tr, gpointer self);
static void ___lambda126__gtest_func (void);
TDocumentTest* tdocument_test_new (void);
TDocumentTest* tdocument_test_construct (GType object_type);
GXmlTest* gxml_test_new (void);
GXmlTest* gxml_test_construct (GType object_type);
#define TDOCUMENT_TEST_TTEST_OBJECT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TDOCUMENT_TEST_TYPE_TTEST_OBJECT, TDocumentTestTTestObjectPrivate))
enum  {
	TDOCUMENT_TEST_TTEST_OBJECT_DUMMY_PROPERTY,
	TDOCUMENT_TEST_TTEST_OBJECT_NAME
};
static gchar* tdocument_test_ttest_object_real_node_name (GXmlSerializableObjectModel* base);
static gchar* tdocument_test_ttest_object_real_to_string (GXmlSerializableObjectModel* base);
const gchar* tdocument_test_ttest_object_get_name (TDocumentTestTTestObject* self);
static void tdocument_test_ttest_object_finalize (GObject* obj);
static void _vala_tdocument_test_ttest_object_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_tdocument_test_ttest_object_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);


static void __lambda105_ (void) {
	GXmlTDocument* d = NULL;
	GXmlTDocument* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	gchar* _tmp2_ = NULL;
	gchar* _tmp3_ = NULL;
	GXmlNode* _tmp4_ = NULL;
	GXmlNode* _tmp5_ = NULL;
	GXmlNode* _tmp6_ = NULL;
	GeeBidirList* _tmp7_ = NULL;
	GeeBidirList* _tmp8_ = NULL;
	GeeBidirList* _tmp9_ = NULL;
	GeeMap* _tmp10_ = NULL;
	GeeMap* _tmp11_ = NULL;
	GeeMap* _tmp12_ = NULL;
	GeeBidirList* _tmp13_ = NULL;
	GeeBidirList* _tmp14_ = NULL;
	GeeBidirList* _tmp15_ = NULL;
	gint _tmp16_ = 0;
	gint _tmp17_ = 0;
	gchar* _tmp18_ = NULL;
	gchar* _tmp19_ = NULL;
	gchar* _tmp20_ = NULL;
#line 34 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp0_ = gxml_tdocument_new ();
#line 34 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	d = _tmp0_;
#line 35 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp1_ = gxml_tnode_get_name ((GXmlTNode*) d);
#line 35 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp2_ = _tmp1_;
#line 35 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp3_ = _tmp2_;
#line 35 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp3_, "#document") == 0, "d.name == \"#document\"");
#line 35 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp3_);
#line 36 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp4_ = gxml_document_get_root ((GXmlDocument*) d);
#line 36 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp5_ = _tmp4_;
#line 36 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp6_ = _tmp5_;
#line 36 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp6_ == NULL, "d.root == null");
#line 36 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp6_);
#line 37 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp7_ = gxml_tnode_get_children_nodes ((GXmlTNode*) d);
#line 37 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp8_ = _tmp7_;
#line 37 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp9_ = _tmp8_;
#line 37 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp9_ != NULL, "d.children_nodes != null");
#line 37 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp9_);
#line 38 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp10_ = gxml_tnode_get_attrs ((GXmlTNode*) d);
#line 38 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp11_ = _tmp10_;
#line 38 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp12_ = _tmp11_;
#line 38 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp12_ != NULL, "d.attrs != null");
#line 38 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp12_);
#line 39 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp13_ = gxml_tnode_get_children_nodes ((GXmlTNode*) d);
#line 39 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp14_ = _tmp13_;
#line 39 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp15_ = _tmp14_;
#line 39 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp16_ = gee_collection_get_size ((GeeCollection*) _tmp15_);
#line 39 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp17_ = _tmp16_;
#line 39 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp17_ == 0, "d.children_nodes.size == 0");
#line 39 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp15_);
#line 40 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp18_ = gxml_tnode_get_value ((GXmlTNode*) d);
#line 40 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp19_ = _tmp18_;
#line 40 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp20_ = _tmp19_;
#line 40 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp20_ == NULL, "d.value == null");
#line 40 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp20_);
#line 33 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (d);
#line 291 "TDocumentTest.c"
}


static void ___lambda105__gtest_func (void) {
#line 33 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda105_ ();
#line 298 "TDocumentTest.c"
}


static void __lambda106_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp0_ = NULL;
		GXmlNode* e = NULL;
		GXmlNode* _tmp1_ = NULL;
		GeeBidirList* _tmp2_ = NULL;
		GeeBidirList* _tmp3_ = NULL;
		GeeBidirList* _tmp4_ = NULL;
		GeeBidirList* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GeeBidirList* _tmp7_ = NULL;
		gint _tmp8_ = 0;
		gint _tmp9_ = 0;
		GXmlNode* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GXmlNode* _tmp12_ = NULL;
		GXmlNode* _tmp13_ = NULL;
		GXmlNode* _tmp14_ = NULL;
		GXmlNode* _tmp15_ = NULL;
		gchar* _tmp16_ = NULL;
		gchar* _tmp17_ = NULL;
		gchar* _tmp18_ = NULL;
		GXmlNode* _tmp19_ = NULL;
		GXmlNode* _tmp20_ = NULL;
		GXmlNode* _tmp21_ = NULL;
		gchar* _tmp22_ = NULL;
		gchar* _tmp23_ = NULL;
		gchar* _tmp24_ = NULL;
#line 44 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp0_ = gxml_tdocument_new ();
#line 44 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		d = _tmp0_;
#line 45 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp1_ = gxml_document_create_element ((GXmlDocument*) d, "root", &_inner_error_);
#line 45 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _tmp1_;
#line 45 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 45 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 344 "TDocumentTest.c"
			goto __catch124_g_error;
		}
#line 46 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp2_ = gxml_tnode_get_children_nodes ((GXmlTNode*) d);
#line 46 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp3_ = _tmp2_;
#line 46 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp4_ = _tmp3_;
#line 46 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gee_collection_add ((GeeCollection*) _tmp4_, e);
#line 46 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp4_);
#line 47 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp5_ = gxml_tnode_get_children_nodes ((GXmlTNode*) d);
#line 47 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp6_ = _tmp5_;
#line 47 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp7_ = _tmp6_;
#line 47 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp8_ = gee_collection_get_size ((GeeCollection*) _tmp7_);
#line 47 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp9_ = _tmp8_;
#line 47 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp9_ == 1, "d.children_nodes.size == 1");
#line 47 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp7_);
#line 48 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp10_ = gxml_document_get_root ((GXmlDocument*) d);
#line 48 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp11_ = _tmp10_;
#line 48 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp12_ = _tmp11_;
#line 48 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp12_ != NULL, "d.root != null");
#line 48 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp12_);
#line 49 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp13_ = gxml_document_get_root ((GXmlDocument*) d);
#line 49 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp14_ = _tmp13_;
#line 49 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp15_ = _tmp14_;
#line 49 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp16_ = gxml_node_get_name (_tmp15_);
#line 49 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp17_ = _tmp16_;
#line 49 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp18_ = _tmp17_;
#line 49 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp18_, "root") == 0, "d.root.name == \"root\"");
#line 49 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp18_);
#line 49 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp15_);
#line 50 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp19_ = gxml_document_get_root ((GXmlDocument*) d);
#line 50 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp20_ = _tmp19_;
#line 50 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp21_ = _tmp20_;
#line 50 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp22_ = gxml_node_get_value (_tmp21_);
#line 50 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp23_ = _tmp22_;
#line 50 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp24_ = _tmp23_;
#line 50 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp24_, "") == 0, "d.root.value == \"\"");
#line 50 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp24_);
#line 50 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp21_);
#line 43 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (e);
#line 43 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (d);
#line 421 "TDocumentTest.c"
	}
	goto __finally124;
	__catch124_g_error:
	{
		GError* e = NULL;
#line 43 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _inner_error_;
#line 43 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_inner_error_ = NULL;
#line 56 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_assert_not_reached ();
#line 43 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_error_free0 (e);
#line 435 "TDocumentTest.c"
	}
	__finally124:
#line 43 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 43 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 43 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 43 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		return;
#line 446 "TDocumentTest.c"
	}
}


static void ___lambda106__gtest_func (void) {
#line 42 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda106_ ();
#line 454 "TDocumentTest.c"
}


static gboolean string_contains (const gchar* self, const gchar* needle) {
	gboolean result = FALSE;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 1376 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1376 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	g_return_val_if_fail (needle != NULL, FALSE);
#line 1377 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	_tmp0_ = needle;
#line 1377 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	_tmp1_ = strstr ((gchar*) self, (gchar*) _tmp0_);
#line 1377 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	result = _tmp1_ != NULL;
#line 1377 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	return result;
#line 474 "TDocumentTest.c"
}


static void __lambda107_ (void) {
	GError * _inner_error_ = NULL;
	{
		GFile* f = NULL;
		GFile* _tmp0_ = NULL;
		GFile* _tmp1_ = NULL;
		gboolean _tmp2_ = FALSE;
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp4_ = NULL;
		GXmlNode* e = NULL;
		GXmlTDocument* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		GXmlTDocument* _tmp7_ = NULL;
		GeeBidirList* _tmp8_ = NULL;
		GeeBidirList* _tmp9_ = NULL;
		GeeBidirList* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GXmlTDocument* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GeeBidirList* _tmp15_ = NULL;
		gint _tmp16_ = 0;
		gint _tmp17_ = 0;
		GXmlTDocument* _tmp18_ = NULL;
		GXmlNode* _tmp19_ = NULL;
		GXmlNode* _tmp20_ = NULL;
		GXmlNode* _tmp21_ = NULL;
		GXmlTDocument* _tmp22_ = NULL;
		GXmlNode* _tmp23_ = NULL;
		GXmlNode* _tmp24_ = NULL;
		GXmlNode* _tmp25_ = NULL;
		gchar* _tmp26_ = NULL;
		gchar* _tmp27_ = NULL;
		gchar* _tmp28_ = NULL;
		GXmlTDocument* _tmp29_ = NULL;
		GXmlNode* _tmp30_ = NULL;
		GXmlNode* _tmp31_ = NULL;
		GXmlNode* _tmp32_ = NULL;
		gchar* _tmp33_ = NULL;
		gchar* _tmp34_ = NULL;
		gchar* _tmp35_ = NULL;
		GFile* _tmp36_ = NULL;
		gchar* _tmp37_ = NULL;
		gchar* _tmp38_ = NULL;
		gchar* _tmp39_ = NULL;
		gchar* _tmp40_ = NULL;
		gboolean _tmp41_ = FALSE;
		GXmlTDocument* _tmp42_ = NULL;
		gboolean _tmp43_ = FALSE;
		GFile* _tmp44_ = NULL;
		gchar* _tmp45_ = NULL;
		gchar* _tmp46_ = NULL;
		gchar* _tmp47_ = NULL;
		gchar* _tmp48_ = NULL;
		GFile* _tmp49_ = NULL;
		gboolean _tmp50_ = FALSE;
		GFileInputStream* istream = NULL;
		GFile* _tmp51_ = NULL;
		GFileInputStream* _tmp52_ = NULL;
		GMemoryOutputStream* ostream = NULL;
		GMemoryOutputStream* _tmp53_ = NULL;
		GMemoryOutputStream* _tmp54_ = NULL;
		GFileInputStream* _tmp55_ = NULL;
		GMemoryOutputStream* _tmp56_ = NULL;
		void* _tmp57_ = NULL;
		void* _tmp58_ = NULL;
		gboolean _tmp59_ = FALSE;
		GMemoryOutputStream* _tmp60_ = NULL;
		void* _tmp61_ = NULL;
		void* _tmp62_ = NULL;
		gboolean _tmp63_ = FALSE;
#line 61 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp0_ = g_file_new_for_path (TEST_SAVE_DIR "/t-test.xml");
#line 61 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f = _tmp0_;
#line 62 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp1_ = f;
#line 62 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp2_ = g_file_query_exists (_tmp1_, NULL);
#line 62 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (_tmp2_) {
#line 559 "TDocumentTest.c"
			GFile* _tmp3_ = NULL;
#line 62 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp3_ = f;
#line 62 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			g_file_delete (_tmp3_, NULL, &_inner_error_);
#line 62 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 62 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_g_object_unref0 (f);
#line 569 "TDocumentTest.c"
				goto __catch125_g_error;
			}
		}
#line 63 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp4_ = gxml_tdocument_new_from_path (TEST_SAVE_DIR "/t-test.xml");
#line 63 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		d = _tmp4_;
#line 64 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp5_ = d;
#line 64 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp6_ = gxml_document_create_element ((GXmlDocument*) _tmp5_, "root", &_inner_error_);
#line 64 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _tmp6_;
#line 64 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 64 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 64 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 589 "TDocumentTest.c"
			goto __catch125_g_error;
		}
#line 65 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp7_ = d;
#line 65 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp8_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp7_);
#line 65 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp9_ = _tmp8_;
#line 65 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp10_ = _tmp9_;
#line 65 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp11_ = e;
#line 65 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gee_collection_add ((GeeCollection*) _tmp10_, _tmp11_);
#line 65 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp10_);
#line 66 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp12_ = d;
#line 66 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp13_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp12_);
#line 66 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp14_ = _tmp13_;
#line 66 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp15_ = _tmp14_;
#line 66 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp16_ = gee_collection_get_size ((GeeCollection*) _tmp15_);
#line 66 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp17_ = _tmp16_;
#line 66 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp17_ == 1, "d.children_nodes.size == 1");
#line 66 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp15_);
#line 67 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp18_ = d;
#line 67 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp19_ = gxml_document_get_root ((GXmlDocument*) _tmp18_);
#line 67 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp20_ = _tmp19_;
#line 67 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp21_ = _tmp20_;
#line 67 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp21_ != NULL, "d.root != null");
#line 67 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp21_);
#line 68 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp22_ = d;
#line 68 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp23_ = gxml_document_get_root ((GXmlDocument*) _tmp22_);
#line 68 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp24_ = _tmp23_;
#line 68 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp25_ = _tmp24_;
#line 68 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp26_ = gxml_node_get_name (_tmp25_);
#line 68 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp27_ = _tmp26_;
#line 68 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp28_ = _tmp27_;
#line 68 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp28_, "root") == 0, "d.root.name == \"root\"");
#line 68 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp28_);
#line 68 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp25_);
#line 69 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp29_ = d;
#line 69 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp30_ = gxml_document_get_root ((GXmlDocument*) _tmp29_);
#line 69 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp31_ = _tmp30_;
#line 69 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp32_ = _tmp31_;
#line 69 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp33_ = gxml_node_get_value (_tmp32_);
#line 69 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp34_ = _tmp33_;
#line 69 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp35_ = _tmp34_;
#line 69 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp35_, "") == 0, "d.root.value == \"\"");
#line 69 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp35_);
#line 69 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp32_);
#line 70 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp36_ = f;
#line 70 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp37_ = g_file_get_path (_tmp36_);
#line 70 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp38_ = _tmp37_;
#line 70 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp39_ = g_strconcat ("Saving document to: ", _tmp38_, NULL);
#line 70 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp40_ = _tmp39_;
#line 70 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_test_message ("%s", _tmp40_);
#line 70 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp40_);
#line 70 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp38_);
#line 71 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp42_ = d;
#line 71 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp43_ = gxml_document_save ((GXmlDocument*) _tmp42_, NULL, &_inner_error_);
#line 71 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp41_ = _tmp43_;
#line 71 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 71 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 71 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 71 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 704 "TDocumentTest.c"
			goto __catch125_g_error;
		}
#line 71 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp41_, "d.save ()");
#line 72 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp44_ = f;
#line 72 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp45_ = g_file_get_path (_tmp44_);
#line 72 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp46_ = _tmp45_;
#line 72 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp47_ = g_strconcat ("Reading saved document to: ", _tmp46_, NULL);
#line 72 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp48_ = _tmp47_;
#line 72 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_test_message ("%s", _tmp48_);
#line 72 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp48_);
#line 72 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp46_);
#line 73 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp49_ = f;
#line 73 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp50_ = g_file_query_exists (_tmp49_, NULL);
#line 73 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp50_, "f.query_exists ()");
#line 74 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp51_ = f;
#line 74 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp52_ = g_file_read (_tmp51_, NULL, &_inner_error_);
#line 74 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		istream = _tmp52_;
#line 74 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 74 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 74 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 74 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 745 "TDocumentTest.c"
			goto __catch125_g_error;
		}
#line 75 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp53_ = (GMemoryOutputStream*) g_memory_output_stream_new_resizable ();
#line 75 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		ostream = _tmp53_;
#line 76 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp54_ = ostream;
#line 76 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp55_ = istream;
#line 76 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_output_stream_splice ((GOutputStream*) _tmp54_, (GInputStream*) _tmp55_, 0, NULL, &_inner_error_);
#line 76 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 76 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (ostream);
#line 76 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (istream);
#line 76 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 76 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 76 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 770 "TDocumentTest.c"
			goto __catch125_g_error;
		}
#line 77 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp56_ = ostream;
#line 77 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp57_ = g_memory_output_stream_get_data (_tmp56_);
#line 77 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp58_ = _tmp57_;
#line 77 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp59_ = string_contains ((const gchar*) _tmp58_, "<?xml version=\"1.0\"?>");
#line 77 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp59_, "\"<?xml version=\\\"1.0\\\"?>\" in ((string)ostream.data)");
#line 78 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp60_ = ostream;
#line 78 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp61_ = g_memory_output_stream_get_data (_tmp60_);
#line 78 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp62_ = _tmp61_;
#line 78 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp63_ = string_contains ((const gchar*) _tmp62_, "<root/>");
#line 78 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp63_, "\"<root/>\" in ((string)ostream.data)");
#line 60 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (ostream);
#line 60 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (istream);
#line 60 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (e);
#line 60 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (d);
#line 60 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (f);
#line 803 "TDocumentTest.c"
	}
	goto __finally125;
	__catch125_g_error:
	{
		GError* e = NULL;
#line 60 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _inner_error_;
#line 60 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_inner_error_ = NULL;
#line 84 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_assert_not_reached ();
#line 60 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_error_free0 (e);
#line 817 "TDocumentTest.c"
	}
	__finally125:
#line 60 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 60 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 60 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 60 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		return;
#line 828 "TDocumentTest.c"
	}
}


static void ___lambda107__gtest_func (void) {
#line 59 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda107_ ();
#line 836 "TDocumentTest.c"
}


static void __lambda108_ (void) {
	GError * _inner_error_ = NULL;
	{
		GFile* f = NULL;
		GFile* _tmp0_ = NULL;
		GFile* _tmp1_ = NULL;
		gboolean _tmp2_ = FALSE;
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp4_ = NULL;
		GXmlNode* e = NULL;
		GXmlTDocument* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		GXmlTDocument* _tmp7_ = NULL;
		GeeBidirList* _tmp8_ = NULL;
		GeeBidirList* _tmp9_ = NULL;
		GeeBidirList* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GXmlTDocument* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GeeBidirList* _tmp15_ = NULL;
		gint _tmp16_ = 0;
		gint _tmp17_ = 0;
		GXmlTDocument* _tmp18_ = NULL;
		GXmlNode* _tmp19_ = NULL;
		GXmlNode* _tmp20_ = NULL;
		GXmlNode* _tmp21_ = NULL;
		GXmlTDocument* _tmp22_ = NULL;
		GXmlNode* _tmp23_ = NULL;
		GXmlNode* _tmp24_ = NULL;
		GXmlNode* _tmp25_ = NULL;
		gchar* _tmp26_ = NULL;
		gchar* _tmp27_ = NULL;
		gchar* _tmp28_ = NULL;
		GXmlTDocument* _tmp29_ = NULL;
		GXmlNode* _tmp30_ = NULL;
		GXmlNode* _tmp31_ = NULL;
		GXmlNode* _tmp32_ = NULL;
		gchar* _tmp33_ = NULL;
		gchar* _tmp34_ = NULL;
		gchar* _tmp35_ = NULL;
		GXmlElement* root = NULL;
		GXmlTDocument* _tmp36_ = NULL;
		GXmlNode* _tmp37_ = NULL;
		GXmlNode* _tmp38_ = NULL;
		GXmlElement* _tmp39_ = NULL;
		GXmlNode* at1 = NULL;
		GXmlElement* _tmp40_ = NULL;
		GXmlNode* _tmp41_ = NULL;
		GXmlNode* _tmp42_ = NULL;
		GXmlNode* _tmp43_ = NULL;
		gchar* _tmp44_ = NULL;
		gchar* _tmp45_ = NULL;
		gchar* _tmp46_ = NULL;
		GXmlElement* _tmp47_ = NULL;
		GXmlNode* at2 = NULL;
		GXmlElement* _tmp48_ = NULL;
		GXmlNode* _tmp49_ = NULL;
		GXmlNode* _tmp50_ = NULL;
		GXmlNode* _tmp51_ = NULL;
		gchar* _tmp52_ = NULL;
		gchar* _tmp53_ = NULL;
		gchar* _tmp54_ = NULL;
		GXmlTDocument* _tmp55_ = NULL;
		GFileInputStream* istream = NULL;
		GFile* _tmp56_ = NULL;
		GFileInputStream* _tmp57_ = NULL;
		guint8* buffer = NULL;
		guint8* _tmp58_ = NULL;
		gint buffer_length1 = 0;
		gint _buffer_size_ = 0;
		GFileInputStream* _tmp59_ = NULL;
		guint8* _tmp60_ = NULL;
		gint _tmp60__length1 = 0;
		GFileInputStream* _tmp61_ = NULL;
		guint8* _tmp62_ = NULL;
		gint _tmp62__length1 = 0;
		gboolean _tmp63_ = FALSE;
		guint8* _tmp64_ = NULL;
		gint _tmp64__length1 = 0;
		gboolean _tmp65_ = FALSE;
		guint8* _tmp66_ = NULL;
		gint _tmp66__length1 = 0;
		gboolean _tmp67_ = FALSE;
		guint8* _tmp68_ = NULL;
		gint _tmp68__length1 = 0;
		gboolean _tmp69_ = FALSE;
#line 89 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp0_ = g_file_new_for_path (TEST_SAVE_DIR "/t-test.xml");
#line 89 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f = _tmp0_;
#line 90 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp1_ = f;
#line 90 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp2_ = g_file_query_exists (_tmp1_, NULL);
#line 90 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (_tmp2_) {
#line 937 "TDocumentTest.c"
			GFile* _tmp3_ = NULL;
#line 90 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp3_ = f;
#line 90 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			g_file_delete (_tmp3_, NULL, &_inner_error_);
#line 90 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 90 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_g_object_unref0 (f);
#line 947 "TDocumentTest.c"
				goto __catch126_g_error;
			}
		}
#line 91 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp4_ = gxml_tdocument_new_from_path (TEST_SAVE_DIR "/t-test.xml");
#line 91 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		d = _tmp4_;
#line 92 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp5_ = d;
#line 92 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp6_ = gxml_document_create_element ((GXmlDocument*) _tmp5_, "root", &_inner_error_);
#line 92 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _tmp6_;
#line 92 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 92 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 92 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 967 "TDocumentTest.c"
			goto __catch126_g_error;
		}
#line 93 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp7_ = d;
#line 93 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp8_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp7_);
#line 93 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp9_ = _tmp8_;
#line 93 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp10_ = _tmp9_;
#line 93 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp11_ = e;
#line 93 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gee_collection_add ((GeeCollection*) _tmp10_, _tmp11_);
#line 93 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp10_);
#line 94 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp12_ = d;
#line 94 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp13_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp12_);
#line 94 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp14_ = _tmp13_;
#line 94 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp15_ = _tmp14_;
#line 94 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp16_ = gee_collection_get_size ((GeeCollection*) _tmp15_);
#line 94 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp17_ = _tmp16_;
#line 94 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp17_ == 1, "d.children_nodes.size == 1");
#line 94 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp15_);
#line 95 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp18_ = d;
#line 95 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp19_ = gxml_document_get_root ((GXmlDocument*) _tmp18_);
#line 95 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp20_ = _tmp19_;
#line 95 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp21_ = _tmp20_;
#line 95 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp21_ != NULL, "d.root != null");
#line 95 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp21_);
#line 96 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp22_ = d;
#line 96 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp23_ = gxml_document_get_root ((GXmlDocument*) _tmp22_);
#line 96 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp24_ = _tmp23_;
#line 96 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp25_ = _tmp24_;
#line 96 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp26_ = gxml_node_get_name (_tmp25_);
#line 96 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp27_ = _tmp26_;
#line 96 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp28_ = _tmp27_;
#line 96 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp28_, "root") == 0, "d.root.name == \"root\"");
#line 96 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp28_);
#line 96 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp25_);
#line 97 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp29_ = d;
#line 97 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp30_ = gxml_document_get_root ((GXmlDocument*) _tmp29_);
#line 97 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp31_ = _tmp30_;
#line 97 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp32_ = _tmp31_;
#line 97 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp33_ = gxml_node_get_value (_tmp32_);
#line 97 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp34_ = _tmp33_;
#line 97 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp35_ = _tmp34_;
#line 97 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp35_, "") == 0, "d.root.value == \"\"");
#line 97 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp35_);
#line 97 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp32_);
#line 98 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp36_ = d;
#line 98 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp37_ = gxml_document_get_root ((GXmlDocument*) _tmp36_);
#line 98 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp38_ = _tmp37_;
#line 98 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		root = G_TYPE_CHECK_INSTANCE_CAST (_tmp38_, GXML_TYPE_ELEMENT, GXmlElement);
#line 99 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp39_ = root;
#line 99 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_element_set_attr (_tmp39_, "Pos", "on");
#line 100 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp40_ = root;
#line 100 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp41_ = gxml_element_get_attr (_tmp40_, "Pos");
#line 100 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		at1 = _tmp41_;
#line 101 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp42_ = at1;
#line 101 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp42_ != NULL, "at1 != null");
#line 102 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp43_ = at1;
#line 102 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp44_ = gxml_node_get_value (_tmp43_);
#line 102 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp45_ = _tmp44_;
#line 102 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp46_ = _tmp45_;
#line 102 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp46_, "on") == 0, "at1.value == \"on\"");
#line 102 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp46_);
#line 103 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp47_ = root;
#line 103 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_element_set_attr (_tmp47_, "tKm", "1000");
#line 104 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp48_ = root;
#line 104 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp49_ = gxml_element_get_attr (_tmp48_, "tKm");
#line 104 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		at2 = _tmp49_;
#line 105 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp50_ = at2;
#line 105 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp50_ != NULL, "at2 != null");
#line 106 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp51_ = at2;
#line 106 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp52_ = gxml_node_get_value (_tmp51_);
#line 106 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp53_ = _tmp52_;
#line 106 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp54_ = _tmp53_;
#line 106 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp54_, "1000") == 0, "at2.value == \"1000\"");
#line 106 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp54_);
#line 107 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp55_ = d;
#line 107 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_document_save ((GXmlDocument*) _tmp55_, NULL, &_inner_error_);
#line 107 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 107 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (at2);
#line 107 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (at1);
#line 107 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (root);
#line 107 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 107 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 107 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 1130 "TDocumentTest.c"
			goto __catch126_g_error;
		}
#line 108 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp56_ = f;
#line 108 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp57_ = g_file_read (_tmp56_, NULL, &_inner_error_);
#line 108 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		istream = _tmp57_;
#line 108 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 108 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (at2);
#line 108 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (at1);
#line 108 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (root);
#line 108 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 108 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 108 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 1153 "TDocumentTest.c"
			goto __catch126_g_error;
		}
#line 109 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp58_ = g_new0 (guint8, 2048);
#line 109 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		buffer = _tmp58_;
#line 109 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		buffer_length1 = 2048;
#line 109 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_buffer_size_ = buffer_length1;
#line 110 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp59_ = istream;
#line 110 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp60_ = buffer;
#line 110 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp60__length1 = buffer_length1;
#line 110 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_input_stream_read_all ((GInputStream*) _tmp59_, _tmp60_, (gsize) _tmp60__length1, NULL, NULL, &_inner_error_);
#line 110 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 110 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			buffer = (g_free (buffer), NULL);
#line 110 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (istream);
#line 110 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (at2);
#line 110 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (at1);
#line 110 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (root);
#line 110 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 110 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 110 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 1190 "TDocumentTest.c"
			goto __catch126_g_error;
		}
#line 111 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp61_ = istream;
#line 111 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_input_stream_close ((GInputStream*) _tmp61_, NULL, &_inner_error_);
#line 111 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 111 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			buffer = (g_free (buffer), NULL);
#line 111 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (istream);
#line 111 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (at2);
#line 111 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (at1);
#line 111 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (root);
#line 111 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 111 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 111 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 1215 "TDocumentTest.c"
			goto __catch126_g_error;
		}
#line 112 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp62_ = buffer;
#line 112 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp62__length1 = buffer_length1;
#line 112 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp63_ = string_contains ((const gchar*) _tmp62_, "<?xml version=\"1.0\"?>");
#line 112 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp63_, "\"<?xml version=\\\"1.0\\\"?>\" in ((string)buffer)");
#line 113 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp64_ = buffer;
#line 113 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp64__length1 = buffer_length1;
#line 113 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp65_ = string_contains ((const gchar*) _tmp64_, "<root");
#line 113 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp65_, "\"<root\" in ((string)buffer)");
#line 114 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp66_ = buffer;
#line 114 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp66__length1 = buffer_length1;
#line 114 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp67_ = string_contains ((const gchar*) _tmp66_, "Pos=\"on\"");
#line 114 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp67_, "\"Pos=\\\"on\\\"\" in ((string)buffer)");
#line 115 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp68_ = buffer;
#line 115 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp68__length1 = buffer_length1;
#line 115 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp69_ = string_contains ((const gchar*) _tmp68_, "tKm=\"1000\"");
#line 115 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp69_, "\"tKm=\\\"1000\\\"\" in ((string)buffer)");
#line 88 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		buffer = (g_free (buffer), NULL);
#line 88 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (istream);
#line 88 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (at2);
#line 88 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (at1);
#line 88 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (root);
#line 88 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (e);
#line 88 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (d);
#line 88 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (f);
#line 1266 "TDocumentTest.c"
	}
	goto __finally126;
	__catch126_g_error:
	{
		GError* e = NULL;
#line 88 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _inner_error_;
#line 88 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_inner_error_ = NULL;
#line 121 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_assert_not_reached ();
#line 88 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_error_free0 (e);
#line 1280 "TDocumentTest.c"
	}
	__finally126:
#line 88 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 88 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 88 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 88 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		return;
#line 1291 "TDocumentTest.c"
	}
}


static void ___lambda108__gtest_func (void) {
#line 87 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda108_ ();
#line 1299 "TDocumentTest.c"
}


static void __lambda109_ (void) {
	GError * _inner_error_ = NULL;
	{
		GFile* f = NULL;
		GFile* _tmp0_ = NULL;
		GFile* _tmp1_ = NULL;
		gboolean _tmp2_ = FALSE;
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp4_ = NULL;
		GXmlNode* e = NULL;
		GXmlTDocument* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		GXmlTDocument* _tmp7_ = NULL;
		GeeBidirList* _tmp8_ = NULL;
		GeeBidirList* _tmp9_ = NULL;
		GeeBidirList* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GXmlTDocument* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GeeBidirList* _tmp15_ = NULL;
		gint _tmp16_ = 0;
		gint _tmp17_ = 0;
		GXmlTDocument* _tmp18_ = NULL;
		GXmlNode* _tmp19_ = NULL;
		GXmlNode* _tmp20_ = NULL;
		GXmlNode* _tmp21_ = NULL;
		GXmlTDocument* _tmp22_ = NULL;
		GXmlNode* _tmp23_ = NULL;
		GXmlNode* _tmp24_ = NULL;
		GXmlNode* _tmp25_ = NULL;
		gchar* _tmp26_ = NULL;
		gchar* _tmp27_ = NULL;
		gchar* _tmp28_ = NULL;
		GXmlTDocument* _tmp29_ = NULL;
		GXmlNode* _tmp30_ = NULL;
		GXmlNode* _tmp31_ = NULL;
		GXmlNode* _tmp32_ = NULL;
		gchar* _tmp33_ = NULL;
		gchar* _tmp34_ = NULL;
		gchar* _tmp35_ = NULL;
		GXmlElement* root = NULL;
		GXmlTDocument* _tmp36_ = NULL;
		GXmlNode* _tmp37_ = NULL;
		GXmlNode* _tmp38_ = NULL;
		GXmlElement* _tmp39_ = NULL;
		GXmlElement* _tmp40_ = NULL;
		GeeBidirList* _tmp41_ = NULL;
		GeeBidirList* _tmp42_ = NULL;
		GeeBidirList* _tmp43_ = NULL;
		gint _tmp44_ = 0;
		gint _tmp45_ = 0;
		GXmlElement* _tmp46_ = NULL;
		gchar* _tmp47_ = NULL;
		gchar* _tmp48_ = NULL;
		gchar* _tmp49_ = NULL;
		GXmlNode* t = NULL;
		GXmlElement* _tmp50_ = NULL;
		GeeBidirList* _tmp51_ = NULL;
		GeeBidirList* _tmp52_ = NULL;
		GeeBidirList* _tmp53_ = NULL;
		gpointer _tmp54_ = NULL;
		GXmlNode* _tmp55_ = NULL;
		GXmlNode* _tmp56_ = NULL;
		gchar* _tmp57_ = NULL;
		gchar* _tmp58_ = NULL;
		gchar* _tmp59_ = NULL;
		GXmlNode* _tmp60_ = NULL;
		GXmlTDocument* _tmp61_ = NULL;
		GFileInputStream* istream = NULL;
		GFile* _tmp62_ = NULL;
		GFileInputStream* _tmp63_ = NULL;
		guint8* buffer = NULL;
		guint8* _tmp64_ = NULL;
		gint buffer_length1 = 0;
		gint _buffer_size_ = 0;
		GFileInputStream* _tmp65_ = NULL;
		guint8* _tmp66_ = NULL;
		gint _tmp66__length1 = 0;
		GFileInputStream* _tmp67_ = NULL;
		guint8* _tmp68_ = NULL;
		gint _tmp68__length1 = 0;
		gboolean _tmp69_ = FALSE;
		guint8* _tmp70_ = NULL;
		gint _tmp70__length1 = 0;
		gboolean _tmp71_ = FALSE;
		guint8* _tmp72_ = NULL;
		gint _tmp72__length1 = 0;
		gboolean _tmp73_ = FALSE;
#line 126 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp0_ = g_file_new_for_path (TEST_SAVE_DIR "/t-test.xml");
#line 126 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f = _tmp0_;
#line 127 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp1_ = f;
#line 127 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp2_ = g_file_query_exists (_tmp1_, NULL);
#line 127 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (_tmp2_) {
#line 1402 "TDocumentTest.c"
			GFile* _tmp3_ = NULL;
#line 127 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp3_ = f;
#line 127 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			g_file_delete (_tmp3_, NULL, &_inner_error_);
#line 127 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 127 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_g_object_unref0 (f);
#line 1412 "TDocumentTest.c"
				goto __catch127_g_error;
			}
		}
#line 128 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp4_ = gxml_tdocument_new_from_path (TEST_SAVE_DIR "/t-test.xml");
#line 128 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		d = _tmp4_;
#line 129 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp5_ = d;
#line 129 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp6_ = gxml_document_create_element ((GXmlDocument*) _tmp5_, "root", &_inner_error_);
#line 129 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _tmp6_;
#line 129 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 129 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 129 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 1432 "TDocumentTest.c"
			goto __catch127_g_error;
		}
#line 130 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp7_ = d;
#line 130 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp8_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp7_);
#line 130 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp9_ = _tmp8_;
#line 130 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp10_ = _tmp9_;
#line 130 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp11_ = e;
#line 130 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gee_collection_add ((GeeCollection*) _tmp10_, _tmp11_);
#line 130 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp10_);
#line 131 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp12_ = d;
#line 131 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp13_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp12_);
#line 131 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp14_ = _tmp13_;
#line 131 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp15_ = _tmp14_;
#line 131 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp16_ = gee_collection_get_size ((GeeCollection*) _tmp15_);
#line 131 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp17_ = _tmp16_;
#line 131 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp17_ == 1, "d.children_nodes.size == 1");
#line 131 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp15_);
#line 132 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp18_ = d;
#line 132 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp19_ = gxml_document_get_root ((GXmlDocument*) _tmp18_);
#line 132 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp20_ = _tmp19_;
#line 132 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp21_ = _tmp20_;
#line 132 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp21_ != NULL, "d.root != null");
#line 132 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp21_);
#line 133 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp22_ = d;
#line 133 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp23_ = gxml_document_get_root ((GXmlDocument*) _tmp22_);
#line 133 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp24_ = _tmp23_;
#line 133 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp25_ = _tmp24_;
#line 133 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp26_ = gxml_node_get_name (_tmp25_);
#line 133 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp27_ = _tmp26_;
#line 133 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp28_ = _tmp27_;
#line 133 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp28_, "root") == 0, "d.root.name == \"root\"");
#line 133 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp28_);
#line 133 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp25_);
#line 134 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp29_ = d;
#line 134 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp30_ = gxml_document_get_root ((GXmlDocument*) _tmp29_);
#line 134 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp31_ = _tmp30_;
#line 134 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp32_ = _tmp31_;
#line 134 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp33_ = gxml_node_get_value (_tmp32_);
#line 134 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp34_ = _tmp33_;
#line 134 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp35_ = _tmp34_;
#line 134 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp35_, "") == 0, "d.root.value == \"\"");
#line 134 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp35_);
#line 134 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp32_);
#line 135 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp36_ = d;
#line 135 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp37_ = gxml_document_get_root ((GXmlDocument*) _tmp36_);
#line 135 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp38_ = _tmp37_;
#line 135 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		root = G_TYPE_CHECK_INSTANCE_CAST (_tmp38_, GXML_TYPE_ELEMENT, GXmlElement);
#line 136 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp39_ = root;
#line 136 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_element_set_content (_tmp39_, "GXml TDocument Test");
#line 137 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp40_ = root;
#line 137 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp41_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp40_);
#line 137 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp42_ = _tmp41_;
#line 137 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp43_ = _tmp42_;
#line 137 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp44_ = gee_collection_get_size ((GeeCollection*) _tmp43_);
#line 137 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp45_ = _tmp44_;
#line 137 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp45_ == 1, "root.children_nodes.size == 1");
#line 137 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp43_);
#line 138 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp46_ = root;
#line 138 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp47_ = gxml_element_get_content (_tmp46_);
#line 138 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp48_ = _tmp47_;
#line 138 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp49_ = _tmp48_;
#line 138 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp49_, "GXml TDocument Test") == 0, "root.content == \"GXml TDocument Test\"");
#line 138 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp49_);
#line 139 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp50_ = root;
#line 139 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp51_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp50_);
#line 139 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp52_ = _tmp51_;
#line 139 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp53_ = _tmp52_;
#line 139 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp54_ = gee_list_get ((GeeList*) _tmp53_, 0);
#line 139 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp55_ = (GXmlNode*) _tmp54_;
#line 139 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp53_);
#line 139 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		t = _tmp55_;
#line 140 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp56_ = t;
#line 140 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp57_ = gxml_node_get_value (_tmp56_);
#line 140 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp58_ = _tmp57_;
#line 140 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp59_ = _tmp58_;
#line 140 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp59_, "GXml TDocument Test") == 0, "t.value == \"GXml TDocument Test\"");
#line 140 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp59_);
#line 141 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp60_ = t;
#line 141 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp60_, GXML_TYPE_TEXT), "t is GXml.Text");
#line 143 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp61_ = d;
#line 143 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_document_save ((GXmlDocument*) _tmp61_, NULL, &_inner_error_);
#line 143 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 143 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (t);
#line 143 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (root);
#line 143 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 143 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 143 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 1605 "TDocumentTest.c"
			goto __catch127_g_error;
		}
#line 144 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp62_ = f;
#line 144 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp63_ = g_file_read (_tmp62_, NULL, &_inner_error_);
#line 144 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		istream = _tmp63_;
#line 144 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 144 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (t);
#line 144 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (root);
#line 144 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 144 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 144 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 1626 "TDocumentTest.c"
			goto __catch127_g_error;
		}
#line 145 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp64_ = g_new0 (guint8, 2048);
#line 145 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		buffer = _tmp64_;
#line 145 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		buffer_length1 = 2048;
#line 145 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_buffer_size_ = buffer_length1;
#line 146 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp65_ = istream;
#line 146 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp66_ = buffer;
#line 146 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp66__length1 = buffer_length1;
#line 146 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_input_stream_read_all ((GInputStream*) _tmp65_, _tmp66_, (gsize) _tmp66__length1, NULL, NULL, &_inner_error_);
#line 146 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 146 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			buffer = (g_free (buffer), NULL);
#line 146 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (istream);
#line 146 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (t);
#line 146 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (root);
#line 146 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 146 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 146 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 1661 "TDocumentTest.c"
			goto __catch127_g_error;
		}
#line 147 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp67_ = istream;
#line 147 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_input_stream_close ((GInputStream*) _tmp67_, NULL, &_inner_error_);
#line 147 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 147 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			buffer = (g_free (buffer), NULL);
#line 147 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (istream);
#line 147 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (t);
#line 147 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (root);
#line 147 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 147 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 147 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 1684 "TDocumentTest.c"
			goto __catch127_g_error;
		}
#line 148 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp68_ = buffer;
#line 148 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp68__length1 = buffer_length1;
#line 148 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp69_ = string_contains ((const gchar*) _tmp68_, "<?xml version=\"1.0\"?>");
#line 148 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp69_, "\"<?xml version=\\\"1.0\\\"?>\" in ((string)buffer)");
#line 149 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp70_ = buffer;
#line 149 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp70__length1 = buffer_length1;
#line 149 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp71_ = string_contains ((const gchar*) _tmp70_, "<root");
#line 149 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp71_, "\"<root\" in ((string)buffer)");
#line 150 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp72_ = buffer;
#line 150 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp72__length1 = buffer_length1;
#line 150 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp73_ = string_contains ((const gchar*) _tmp72_, ">GXml TDocument Test<");
#line 150 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp73_, "\">GXml TDocument Test<\" in ((string)buffer)");
#line 125 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		buffer = (g_free (buffer), NULL);
#line 125 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (istream);
#line 125 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (t);
#line 125 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (root);
#line 125 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (e);
#line 125 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (d);
#line 125 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (f);
#line 1725 "TDocumentTest.c"
	}
	goto __finally127;
	__catch127_g_error:
	{
		GError* e = NULL;
#line 125 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _inner_error_;
#line 125 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_inner_error_ = NULL;
#line 156 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_assert_not_reached ();
#line 125 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_error_free0 (e);
#line 1739 "TDocumentTest.c"
	}
	__finally127:
#line 125 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 125 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 125 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 125 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		return;
#line 1750 "TDocumentTest.c"
	}
}


static void ___lambda109__gtest_func (void) {
#line 124 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda109_ ();
#line 1758 "TDocumentTest.c"
}


static void __lambda110_ (void) {
	GError * _inner_error_ = NULL;
	{
		GFile* f = NULL;
		GFile* _tmp0_ = NULL;
		GFile* _tmp1_ = NULL;
		gboolean _tmp2_ = FALSE;
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp4_ = NULL;
		GXmlNode* e = NULL;
		GXmlTDocument* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		GXmlTDocument* _tmp7_ = NULL;
		GeeBidirList* _tmp8_ = NULL;
		GeeBidirList* _tmp9_ = NULL;
		GeeBidirList* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GXmlTDocument* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GeeBidirList* _tmp15_ = NULL;
		gint _tmp16_ = 0;
		gint _tmp17_ = 0;
		GXmlTDocument* _tmp18_ = NULL;
		GXmlNode* _tmp19_ = NULL;
		GXmlNode* _tmp20_ = NULL;
		GXmlNode* _tmp21_ = NULL;
		GXmlTDocument* _tmp22_ = NULL;
		GXmlNode* _tmp23_ = NULL;
		GXmlNode* _tmp24_ = NULL;
		GXmlNode* _tmp25_ = NULL;
		gchar* _tmp26_ = NULL;
		gchar* _tmp27_ = NULL;
		gchar* _tmp28_ = NULL;
		GXmlTDocument* _tmp29_ = NULL;
		GXmlNode* _tmp30_ = NULL;
		GXmlNode* _tmp31_ = NULL;
		GXmlNode* _tmp32_ = NULL;
		gchar* _tmp33_ = NULL;
		gchar* _tmp34_ = NULL;
		gchar* _tmp35_ = NULL;
		GXmlElement* root = NULL;
		GXmlTDocument* _tmp36_ = NULL;
		GXmlNode* _tmp37_ = NULL;
		GXmlNode* _tmp38_ = NULL;
		GXmlNode* _tmp39_ = NULL;
		GXmlTDocument* _tmp40_ = NULL;
		GXmlNode* _tmp41_ = NULL;
		GXmlElement* e1 = NULL;
		GXmlNode* _tmp42_ = NULL;
		GXmlElement* _tmp43_ = NULL;
		GXmlElement* _tmp44_ = NULL;
		GeeBidirList* _tmp45_ = NULL;
		GeeBidirList* _tmp46_ = NULL;
		GeeBidirList* _tmp47_ = NULL;
		gint _tmp48_ = 0;
		gint _tmp49_ = 0;
		GXmlElement* _tmp50_ = NULL;
		GeeBidirList* _tmp51_ = NULL;
		GeeBidirList* _tmp52_ = NULL;
		GeeBidirList* _tmp53_ = NULL;
		GXmlElement* _tmp54_ = NULL;
		GXmlNode* _tmp55_ = NULL;
		GXmlTDocument* _tmp56_ = NULL;
		GXmlNode* _tmp57_ = NULL;
		GXmlElement* e2 = NULL;
		GXmlNode* _tmp58_ = NULL;
		GXmlElement* _tmp59_ = NULL;
		GXmlElement* _tmp60_ = NULL;
		GeeBidirList* _tmp61_ = NULL;
		GeeBidirList* _tmp62_ = NULL;
		GeeBidirList* _tmp63_ = NULL;
		gint _tmp64_ = 0;
		gint _tmp65_ = 0;
		GXmlElement* _tmp66_ = NULL;
		GeeBidirList* _tmp67_ = NULL;
		GeeBidirList* _tmp68_ = NULL;
		GeeBidirList* _tmp69_ = NULL;
		GXmlElement* _tmp70_ = NULL;
		GXmlElement* _tmp71_ = NULL;
		GeeBidirList* _tmp72_ = NULL;
		GeeBidirList* _tmp73_ = NULL;
		GeeBidirList* _tmp74_ = NULL;
		gint _tmp75_ = 0;
		gint _tmp76_ = 0;
		GXmlTDocument* _tmp77_ = NULL;
		GFileInputStream* istream = NULL;
		GFile* _tmp78_ = NULL;
		GFileInputStream* _tmp79_ = NULL;
		guint8* buffer = NULL;
		guint8* _tmp80_ = NULL;
		gint buffer_length1 = 0;
		gint _buffer_size_ = 0;
		GFileInputStream* _tmp81_ = NULL;
		guint8* _tmp82_ = NULL;
		gint _tmp82__length1 = 0;
		GFileInputStream* _tmp83_ = NULL;
		guint8* _tmp84_ = NULL;
		gint _tmp84__length1 = 0;
		gboolean _tmp85_ = FALSE;
		guint8* _tmp86_ = NULL;
		gint _tmp86__length1 = 0;
		gboolean _tmp87_ = FALSE;
		guint8* _tmp88_ = NULL;
		gint _tmp88__length1 = 0;
		gboolean _tmp89_ = FALSE;
		guint8* _tmp90_ = NULL;
		gint _tmp90__length1 = 0;
		gboolean _tmp91_ = FALSE;
		guint8* _tmp92_ = NULL;
		gint _tmp92__length1 = 0;
		gboolean _tmp93_ = FALSE;
#line 161 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp0_ = g_file_new_for_path (TEST_SAVE_DIR "/t-test.xml");
#line 161 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f = _tmp0_;
#line 162 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp1_ = f;
#line 162 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp2_ = g_file_query_exists (_tmp1_, NULL);
#line 162 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (_tmp2_) {
#line 1884 "TDocumentTest.c"
			GFile* _tmp3_ = NULL;
#line 162 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp3_ = f;
#line 162 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			g_file_delete (_tmp3_, NULL, &_inner_error_);
#line 162 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 162 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_g_object_unref0 (f);
#line 1894 "TDocumentTest.c"
				goto __catch128_g_error;
			}
		}
#line 163 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp4_ = gxml_tdocument_new_from_path (TEST_SAVE_DIR "/t-test.xml");
#line 163 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		d = _tmp4_;
#line 164 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp5_ = d;
#line 164 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp6_ = gxml_document_create_element ((GXmlDocument*) _tmp5_, "root", &_inner_error_);
#line 164 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _tmp6_;
#line 164 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 164 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 164 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 1914 "TDocumentTest.c"
			goto __catch128_g_error;
		}
#line 165 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp7_ = d;
#line 165 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp8_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp7_);
#line 165 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp9_ = _tmp8_;
#line 165 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp10_ = _tmp9_;
#line 165 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp11_ = e;
#line 165 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gee_collection_add ((GeeCollection*) _tmp10_, _tmp11_);
#line 165 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp10_);
#line 166 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp12_ = d;
#line 166 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp13_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp12_);
#line 166 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp14_ = _tmp13_;
#line 166 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp15_ = _tmp14_;
#line 166 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp16_ = gee_collection_get_size ((GeeCollection*) _tmp15_);
#line 166 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp17_ = _tmp16_;
#line 166 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp17_ == 1, "d.children_nodes.size == 1");
#line 166 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp15_);
#line 167 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp18_ = d;
#line 167 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp19_ = gxml_document_get_root ((GXmlDocument*) _tmp18_);
#line 167 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp20_ = _tmp19_;
#line 167 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp21_ = _tmp20_;
#line 167 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp21_ != NULL, "d.root != null");
#line 167 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp21_);
#line 168 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp22_ = d;
#line 168 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp23_ = gxml_document_get_root ((GXmlDocument*) _tmp22_);
#line 168 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp24_ = _tmp23_;
#line 168 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp25_ = _tmp24_;
#line 168 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp26_ = gxml_node_get_name (_tmp25_);
#line 168 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp27_ = _tmp26_;
#line 168 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp28_ = _tmp27_;
#line 168 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp28_, "root") == 0, "d.root.name == \"root\"");
#line 168 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp28_);
#line 168 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp25_);
#line 169 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp29_ = d;
#line 169 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp30_ = gxml_document_get_root ((GXmlDocument*) _tmp29_);
#line 169 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp31_ = _tmp30_;
#line 169 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp32_ = _tmp31_;
#line 169 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp33_ = gxml_node_get_value (_tmp32_);
#line 169 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp34_ = _tmp33_;
#line 169 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp35_ = _tmp34_;
#line 169 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp35_, "") == 0, "d.root.value == \"\"");
#line 169 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp35_);
#line 169 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp32_);
#line 170 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp36_ = d;
#line 170 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp37_ = gxml_document_get_root ((GXmlDocument*) _tmp36_);
#line 170 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp38_ = _tmp37_;
#line 170 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		root = G_TYPE_CHECK_INSTANCE_CAST (_tmp38_, GXML_TYPE_ELEMENT, GXmlElement);
#line 171 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp40_ = d;
#line 171 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp41_ = gxml_document_create_element ((GXmlDocument*) _tmp40_, "child", &_inner_error_);
#line 171 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp39_ = _tmp41_;
#line 171 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 171 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (root);
#line 171 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 171 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 171 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 2023 "TDocumentTest.c"
			goto __catch128_g_error;
		}
#line 171 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp42_ = _tmp39_;
#line 171 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp39_ = NULL;
#line 171 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e1 = G_TYPE_CHECK_INSTANCE_CAST (_tmp42_, GXML_TYPE_ELEMENT, GXmlElement);
#line 172 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp43_ = e1;
#line 172 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_element_set_attr (_tmp43_, "name", "Test1");
#line 173 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp44_ = e1;
#line 173 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp45_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp44_);
#line 173 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp46_ = _tmp45_;
#line 173 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp47_ = _tmp46_;
#line 173 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp48_ = gee_collection_get_size ((GeeCollection*) _tmp47_);
#line 173 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp49_ = _tmp48_;
#line 173 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp49_ == 0, "e1.children_nodes.size == 0");
#line 173 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp47_);
#line 174 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp50_ = root;
#line 174 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp51_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp50_);
#line 174 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp52_ = _tmp51_;
#line 174 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp53_ = _tmp52_;
#line 174 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp54_ = e1;
#line 174 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gee_collection_add ((GeeCollection*) _tmp53_, (GXmlNode*) _tmp54_);
#line 174 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp53_);
#line 175 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp56_ = d;
#line 175 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp57_ = gxml_document_create_element ((GXmlDocument*) _tmp56_, "child", &_inner_error_);
#line 175 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp55_ = _tmp57_;
#line 175 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 175 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e1);
#line 175 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (_tmp39_);
#line 175 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (root);
#line 175 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 175 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 175 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 2086 "TDocumentTest.c"
			goto __catch128_g_error;
		}
#line 175 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp58_ = _tmp55_;
#line 175 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp55_ = NULL;
#line 175 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e2 = G_TYPE_CHECK_INSTANCE_CAST (_tmp58_, GXML_TYPE_ELEMENT, GXmlElement);
#line 176 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp59_ = e2;
#line 176 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_element_set_attr (_tmp59_, "name", "Test2");
#line 177 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp60_ = e2;
#line 177 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp61_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp60_);
#line 177 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp62_ = _tmp61_;
#line 177 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp63_ = _tmp62_;
#line 177 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp64_ = gee_collection_get_size ((GeeCollection*) _tmp63_);
#line 177 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp65_ = _tmp64_;
#line 177 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp65_ == 0, "e2.children_nodes.size == 0");
#line 177 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp63_);
#line 178 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp66_ = root;
#line 178 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp67_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp66_);
#line 178 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp68_ = _tmp67_;
#line 178 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp69_ = _tmp68_;
#line 178 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp70_ = e2;
#line 178 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gee_collection_add ((GeeCollection*) _tmp69_, (GXmlNode*) _tmp70_);
#line 178 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp69_);
#line 179 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp71_ = root;
#line 179 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp72_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp71_);
#line 179 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp73_ = _tmp72_;
#line 179 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp74_ = _tmp73_;
#line 179 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp75_ = gee_collection_get_size ((GeeCollection*) _tmp74_);
#line 179 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp76_ = _tmp75_;
#line 179 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp76_ == 2, "root.children_nodes.size == 2");
#line 179 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp74_);
#line 180 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp77_ = d;
#line 180 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_document_save ((GXmlDocument*) _tmp77_, NULL, &_inner_error_);
#line 180 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 180 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e2);
#line 180 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (_tmp55_);
#line 180 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e1);
#line 180 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (_tmp39_);
#line 180 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (root);
#line 180 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 180 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 180 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 2167 "TDocumentTest.c"
			goto __catch128_g_error;
		}
#line 181 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp78_ = f;
#line 181 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp79_ = g_file_read (_tmp78_, NULL, &_inner_error_);
#line 181 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		istream = _tmp79_;
#line 181 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 181 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e2);
#line 181 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (_tmp55_);
#line 181 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e1);
#line 181 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (_tmp39_);
#line 181 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (root);
#line 181 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 181 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 181 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 2194 "TDocumentTest.c"
			goto __catch128_g_error;
		}
#line 182 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp80_ = g_new0 (guint8, 2048);
#line 182 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		buffer = _tmp80_;
#line 182 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		buffer_length1 = 2048;
#line 182 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_buffer_size_ = buffer_length1;
#line 183 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp81_ = istream;
#line 183 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp82_ = buffer;
#line 183 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp82__length1 = buffer_length1;
#line 183 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_input_stream_read_all ((GInputStream*) _tmp81_, _tmp82_, (gsize) _tmp82__length1, NULL, NULL, &_inner_error_);
#line 183 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 183 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			buffer = (g_free (buffer), NULL);
#line 183 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (istream);
#line 183 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e2);
#line 183 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (_tmp55_);
#line 183 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e1);
#line 183 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (_tmp39_);
#line 183 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (root);
#line 183 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 183 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 183 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 2235 "TDocumentTest.c"
			goto __catch128_g_error;
		}
#line 184 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp83_ = istream;
#line 184 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_input_stream_close ((GInputStream*) _tmp83_, NULL, &_inner_error_);
#line 184 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 184 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			buffer = (g_free (buffer), NULL);
#line 184 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (istream);
#line 184 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e2);
#line 184 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (_tmp55_);
#line 184 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e1);
#line 184 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (_tmp39_);
#line 184 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (root);
#line 184 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 184 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 184 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 2264 "TDocumentTest.c"
			goto __catch128_g_error;
		}
#line 185 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp84_ = buffer;
#line 185 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp84__length1 = buffer_length1;
#line 185 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp85_ = string_contains ((const gchar*) _tmp84_, "<?xml version=\"1.0\"?>");
#line 185 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp85_, "\"<?xml version=\\\"1.0\\\"?>\" in ((string)buffer)");
#line 186 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp86_ = buffer;
#line 186 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp86__length1 = buffer_length1;
#line 186 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp87_ = string_contains ((const gchar*) _tmp86_, "<root>");
#line 186 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp87_, "\"<root>\" in ((string)buffer)");
#line 187 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp88_ = buffer;
#line 187 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp88__length1 = buffer_length1;
#line 187 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp89_ = string_contains ((const gchar*) _tmp88_, "</root>");
#line 187 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp89_, "\"</root>\" in ((string)buffer)");
#line 188 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp90_ = buffer;
#line 188 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp90__length1 = buffer_length1;
#line 188 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp91_ = string_contains ((const gchar*) _tmp90_, "<child name=\"Test1\"/>");
#line 188 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp91_, "\"<child name=\\\"Test1\\\"/>\" in ((string)buffer)");
#line 189 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp92_ = buffer;
#line 189 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp92__length1 = buffer_length1;
#line 189 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp93_ = string_contains ((const gchar*) _tmp92_, "<child name=\"Test2\"/>");
#line 189 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp93_, "\"<child name=\\\"Test2\\\"/>\" in ((string)buffer)");
#line 160 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		buffer = (g_free (buffer), NULL);
#line 160 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (istream);
#line 160 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (e2);
#line 160 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp55_);
#line 160 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (e1);
#line 160 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp39_);
#line 160 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (root);
#line 160 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (e);
#line 160 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (d);
#line 160 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (f);
#line 2327 "TDocumentTest.c"
	}
	goto __finally128;
	__catch128_g_error:
	{
		GError* e = NULL;
#line 160 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _inner_error_;
#line 160 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_inner_error_ = NULL;
#line 195 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_assert_not_reached ();
#line 160 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_error_free0 (e);
#line 2341 "TDocumentTest.c"
	}
	__finally128:
#line 160 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 160 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 160 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 160 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		return;
#line 2352 "TDocumentTest.c"
	}
}


static void ___lambda110__gtest_func (void) {
#line 159 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda110_ ();
#line 2360 "TDocumentTest.c"
}


static const gchar* string_to_string (const gchar* self) {
	const gchar* result = NULL;
#line 1412 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1413 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	result = self;
#line 1413 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	return result;
#line 2372 "TDocumentTest.c"
}


static void __lambda111_ (void) {
	GError * _inner_error_ = NULL;
	{
		GFile* f = NULL;
		GFile* _tmp0_ = NULL;
		GFile* _tmp1_ = NULL;
		gboolean _tmp2_ = FALSE;
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp4_ = NULL;
		GXmlNode* e = NULL;
		GXmlTDocument* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		GXmlTDocument* _tmp7_ = NULL;
		GeeBidirList* _tmp8_ = NULL;
		GeeBidirList* _tmp9_ = NULL;
		GeeBidirList* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GXmlTDocument* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GeeBidirList* _tmp15_ = NULL;
		gint _tmp16_ = 0;
		gint _tmp17_ = 0;
		GXmlTDocument* _tmp18_ = NULL;
		GXmlNode* _tmp19_ = NULL;
		GXmlNode* _tmp20_ = NULL;
		GXmlNode* _tmp21_ = NULL;
		GXmlTDocument* _tmp22_ = NULL;
		GXmlNode* _tmp23_ = NULL;
		GXmlNode* _tmp24_ = NULL;
		GXmlNode* _tmp25_ = NULL;
		gchar* _tmp26_ = NULL;
		gchar* _tmp27_ = NULL;
		gchar* _tmp28_ = NULL;
		GXmlTDocument* _tmp29_ = NULL;
		GXmlNode* _tmp30_ = NULL;
		GXmlNode* _tmp31_ = NULL;
		GXmlNode* _tmp32_ = NULL;
		gchar* _tmp33_ = NULL;
		gchar* _tmp34_ = NULL;
		gchar* _tmp35_ = NULL;
		GXmlElement* r = NULL;
		GXmlTDocument* _tmp36_ = NULL;
		GXmlNode* _tmp37_ = NULL;
		GXmlNode* _tmp38_ = NULL;
		GXmlElement* _tmp39_ = NULL;
		GXmlTDocument* _tmp119_ = NULL;
		GXmlNode* _tmp120_ = NULL;
		GXmlNode* _tmp121_ = NULL;
		GXmlNode* _tmp122_ = NULL;
		GeeBidirList* _tmp123_ = NULL;
		GeeBidirList* _tmp124_ = NULL;
		GeeBidirList* _tmp125_ = NULL;
		gint _tmp126_ = 0;
		gint _tmp127_ = 0;
#line 206 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp0_ = g_file_new_for_path (TEST_SAVE_DIR "/t-large.xml");
#line 206 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f = _tmp0_;
#line 207 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp1_ = f;
#line 207 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp2_ = g_file_query_exists (_tmp1_, NULL);
#line 207 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (_tmp2_) {
#line 2441 "TDocumentTest.c"
			GFile* _tmp3_ = NULL;
#line 207 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp3_ = f;
#line 207 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			g_file_delete (_tmp3_, NULL, &_inner_error_);
#line 207 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 207 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_g_object_unref0 (f);
#line 2451 "TDocumentTest.c"
				goto __catch129_g_error;
			}
		}
#line 211 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp4_ = gxml_tdocument_new_from_path (TEST_SAVE_DIR "/t-large.xml");
#line 211 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		d = _tmp4_;
#line 212 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp5_ = d;
#line 212 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp6_ = gxml_document_create_element ((GXmlDocument*) _tmp5_, "bookstore", &_inner_error_);
#line 212 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _tmp6_;
#line 212 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 212 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 212 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 2471 "TDocumentTest.c"
			goto __catch129_g_error;
		}
#line 213 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp7_ = d;
#line 213 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp8_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp7_);
#line 213 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp9_ = _tmp8_;
#line 213 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp10_ = _tmp9_;
#line 213 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp11_ = e;
#line 213 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gee_collection_add ((GeeCollection*) _tmp10_, _tmp11_);
#line 213 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp10_);
#line 214 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp12_ = d;
#line 214 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp13_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp12_);
#line 214 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp14_ = _tmp13_;
#line 214 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp15_ = _tmp14_;
#line 214 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp16_ = gee_collection_get_size ((GeeCollection*) _tmp15_);
#line 214 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp17_ = _tmp16_;
#line 214 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp17_ == 1, "d.children_nodes.size == 1");
#line 214 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp15_);
#line 215 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp18_ = d;
#line 215 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp19_ = gxml_document_get_root ((GXmlDocument*) _tmp18_);
#line 215 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp20_ = _tmp19_;
#line 215 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp21_ = _tmp20_;
#line 215 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp21_ != NULL, "d.root != null");
#line 215 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp21_);
#line 216 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp22_ = d;
#line 216 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp23_ = gxml_document_get_root ((GXmlDocument*) _tmp22_);
#line 216 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp24_ = _tmp23_;
#line 216 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp25_ = _tmp24_;
#line 216 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp26_ = gxml_node_get_name (_tmp25_);
#line 216 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp27_ = _tmp26_;
#line 216 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp28_ = _tmp27_;
#line 216 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp28_, "bookstore") == 0, "d.root.name == \"bookstore\"");
#line 216 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp28_);
#line 216 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp25_);
#line 217 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp29_ = d;
#line 217 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp30_ = gxml_document_get_root ((GXmlDocument*) _tmp29_);
#line 217 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp31_ = _tmp30_;
#line 217 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp32_ = _tmp31_;
#line 217 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp33_ = gxml_node_get_value (_tmp32_);
#line 217 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp34_ = _tmp33_;
#line 217 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp35_ = _tmp34_;
#line 217 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp35_, "") == 0, "d.root.value == \"\"");
#line 217 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp35_);
#line 217 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp32_);
#line 218 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp36_ = d;
#line 218 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp37_ = gxml_document_get_root ((GXmlDocument*) _tmp36_);
#line 218 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp38_ = _tmp37_;
#line 218 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		r = G_TYPE_CHECK_INSTANCE_CAST (_tmp38_, GXML_TYPE_ELEMENT, GXmlElement);
#line 219 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp39_ = r;
#line 219 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_element_set_attr (_tmp39_, "name", "The Great Book");
#line 2568 "TDocumentTest.c"
		{
			gint i = 0;
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			i = 0;
#line 2573 "TDocumentTest.c"
			{
				gboolean _tmp40_ = FALSE;
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp40_ = TRUE;
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				while (TRUE) {
#line 2580 "TDocumentTest.c"
					gint _tmp42_ = 0;
					GXmlNode* _tmp43_ = NULL;
					GXmlTDocument* _tmp44_ = NULL;
					GXmlNode* _tmp45_ = NULL;
					GXmlElement* b = NULL;
					GXmlNode* _tmp46_ = NULL;
					GXmlElement* _tmp47_ = NULL;
					GeeBidirList* _tmp48_ = NULL;
					GeeBidirList* _tmp49_ = NULL;
					GeeBidirList* _tmp50_ = NULL;
					GXmlElement* _tmp51_ = NULL;
					GXmlNode* _tmp52_ = NULL;
					GXmlTDocument* _tmp53_ = NULL;
					GXmlNode* _tmp54_ = NULL;
					GXmlElement* aths = NULL;
					GXmlNode* _tmp55_ = NULL;
					GXmlElement* _tmp56_ = NULL;
					GeeBidirList* _tmp57_ = NULL;
					GeeBidirList* _tmp58_ = NULL;
					GeeBidirList* _tmp59_ = NULL;
					GXmlElement* _tmp60_ = NULL;
					GXmlNode* _tmp61_ = NULL;
					GXmlTDocument* _tmp62_ = NULL;
					GXmlNode* _tmp63_ = NULL;
					GXmlElement* ath1 = NULL;
					GXmlNode* _tmp64_ = NULL;
					GXmlElement* _tmp65_ = NULL;
					GeeBidirList* _tmp66_ = NULL;
					GeeBidirList* _tmp67_ = NULL;
					GeeBidirList* _tmp68_ = NULL;
					GXmlElement* _tmp69_ = NULL;
					GXmlNode* _tmp70_ = NULL;
					GXmlTDocument* _tmp71_ = NULL;
					GXmlNode* _tmp72_ = NULL;
					GXmlElement* name1 = NULL;
					GXmlNode* _tmp73_ = NULL;
					GXmlElement* _tmp74_ = NULL;
					GXmlElement* _tmp75_ = NULL;
					GeeBidirList* _tmp76_ = NULL;
					GeeBidirList* _tmp77_ = NULL;
					GeeBidirList* _tmp78_ = NULL;
					GXmlElement* _tmp79_ = NULL;
					GXmlNode* _tmp80_ = NULL;
					GXmlTDocument* _tmp81_ = NULL;
					GXmlNode* _tmp82_ = NULL;
					GXmlElement* email1 = NULL;
					GXmlNode* _tmp83_ = NULL;
					GXmlElement* _tmp84_ = NULL;
					GXmlElement* _tmp85_ = NULL;
					GeeBidirList* _tmp86_ = NULL;
					GeeBidirList* _tmp87_ = NULL;
					GeeBidirList* _tmp88_ = NULL;
					GXmlElement* _tmp89_ = NULL;
					GXmlNode* _tmp90_ = NULL;
					GXmlTDocument* _tmp91_ = NULL;
					GXmlNode* _tmp92_ = NULL;
					GXmlElement* ath2 = NULL;
					GXmlNode* _tmp93_ = NULL;
					GXmlElement* _tmp94_ = NULL;
					GeeBidirList* _tmp95_ = NULL;
					GeeBidirList* _tmp96_ = NULL;
					GeeBidirList* _tmp97_ = NULL;
					GXmlElement* _tmp98_ = NULL;
					GXmlNode* _tmp99_ = NULL;
					GXmlTDocument* _tmp100_ = NULL;
					GXmlNode* _tmp101_ = NULL;
					GXmlElement* name2 = NULL;
					GXmlNode* _tmp102_ = NULL;
					GXmlElement* _tmp103_ = NULL;
					GXmlElement* _tmp104_ = NULL;
					GeeBidirList* _tmp105_ = NULL;
					GeeBidirList* _tmp106_ = NULL;
					GeeBidirList* _tmp107_ = NULL;
					GXmlElement* _tmp108_ = NULL;
					GXmlNode* _tmp109_ = NULL;
					GXmlTDocument* _tmp110_ = NULL;
					GXmlNode* _tmp111_ = NULL;
					GXmlElement* email2 = NULL;
					GXmlNode* _tmp112_ = NULL;
					GXmlElement* _tmp113_ = NULL;
					GXmlElement* _tmp114_ = NULL;
					GeeBidirList* _tmp115_ = NULL;
					GeeBidirList* _tmp116_ = NULL;
					GeeBidirList* _tmp117_ = NULL;
					GXmlElement* _tmp118_ = NULL;
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					if (!_tmp40_) {
#line 2668 "TDocumentTest.c"
						gint _tmp41_ = 0;
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_tmp41_ = i;
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						i = _tmp41_ + 1;
#line 2674 "TDocumentTest.c"
					}
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp40_ = FALSE;
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp42_ = i;
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					if (!(_tmp42_ < 5000)) {
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						break;
#line 2684 "TDocumentTest.c"
					}
#line 224 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp44_ = d;
#line 224 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp45_ = gxml_document_create_element ((GXmlDocument*) _tmp44_, "book", &_inner_error_);
#line 224 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp43_ = _tmp45_;
#line 224 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 224 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (r);
#line 224 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (e);
#line 224 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (d);
#line 224 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (f);
#line 2702 "TDocumentTest.c"
						goto __catch129_g_error;
					}
#line 224 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp46_ = _tmp43_;
#line 224 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp43_ = NULL;
#line 224 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					b = G_TYPE_CHECK_INSTANCE_CAST (_tmp46_, GXML_TYPE_ELEMENT, GXmlElement);
#line 225 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp47_ = r;
#line 225 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp48_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp47_);
#line 225 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp49_ = _tmp48_;
#line 225 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp50_ = _tmp49_;
#line 225 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp51_ = b;
#line 225 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gee_collection_add ((GeeCollection*) _tmp50_, (GXmlNode*) _tmp51_);
#line 225 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp50_);
#line 226 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp53_ = d;
#line 226 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp54_ = gxml_document_create_element ((GXmlDocument*) _tmp53_, "Authors", &_inner_error_);
#line 226 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp52_ = _tmp54_;
#line 226 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 226 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (b);
#line 226 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp43_);
#line 226 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (r);
#line 226 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (e);
#line 226 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (d);
#line 226 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (f);
#line 2745 "TDocumentTest.c"
						goto __catch129_g_error;
					}
#line 226 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp55_ = _tmp52_;
#line 226 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp52_ = NULL;
#line 226 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					aths = G_TYPE_CHECK_INSTANCE_CAST (_tmp55_, GXML_TYPE_ELEMENT, GXmlElement);
#line 227 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp56_ = b;
#line 227 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp57_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp56_);
#line 227 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp58_ = _tmp57_;
#line 227 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp59_ = _tmp58_;
#line 227 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp60_ = aths;
#line 227 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gee_collection_add ((GeeCollection*) _tmp59_, (GXmlNode*) _tmp60_);
#line 227 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp59_);
#line 228 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp62_ = d;
#line 228 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp63_ = gxml_document_create_element ((GXmlDocument*) _tmp62_, "Author", &_inner_error_);
#line 228 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp61_ = _tmp63_;
#line 228 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 228 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (aths);
#line 228 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp52_);
#line 228 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (b);
#line 228 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp43_);
#line 228 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (r);
#line 228 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (e);
#line 228 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (d);
#line 228 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (f);
#line 2792 "TDocumentTest.c"
						goto __catch129_g_error;
					}
#line 228 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp64_ = _tmp61_;
#line 228 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp61_ = NULL;
#line 228 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					ath1 = G_TYPE_CHECK_INSTANCE_CAST (_tmp64_, GXML_TYPE_ELEMENT, GXmlElement);
#line 229 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp65_ = aths;
#line 229 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp66_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp65_);
#line 229 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp67_ = _tmp66_;
#line 229 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp68_ = _tmp67_;
#line 229 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp69_ = ath1;
#line 229 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gee_collection_add ((GeeCollection*) _tmp68_, (GXmlNode*) _tmp69_);
#line 229 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp68_);
#line 230 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp71_ = d;
#line 230 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp72_ = gxml_document_create_element ((GXmlDocument*) _tmp71_, "Name", &_inner_error_);
#line 230 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp70_ = _tmp72_;
#line 230 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 230 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (ath1);
#line 230 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp61_);
#line 230 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (aths);
#line 230 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp52_);
#line 230 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (b);
#line 230 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp43_);
#line 230 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (r);
#line 230 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (e);
#line 230 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (d);
#line 230 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (f);
#line 2843 "TDocumentTest.c"
						goto __catch129_g_error;
					}
#line 230 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp73_ = _tmp70_;
#line 230 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp70_ = NULL;
#line 230 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					name1 = G_TYPE_CHECK_INSTANCE_CAST (_tmp73_, GXML_TYPE_ELEMENT, GXmlElement);
#line 231 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp74_ = name1;
#line 231 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gxml_element_set_content (_tmp74_, "Fred");
#line 232 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp75_ = ath1;
#line 232 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp76_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp75_);
#line 232 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp77_ = _tmp76_;
#line 232 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp78_ = _tmp77_;
#line 232 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp79_ = name1;
#line 232 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gee_collection_add ((GeeCollection*) _tmp78_, (GXmlNode*) _tmp79_);
#line 232 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp78_);
#line 233 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp81_ = d;
#line 233 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp82_ = gxml_document_create_element ((GXmlDocument*) _tmp81_, "Email", &_inner_error_);
#line 233 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp80_ = _tmp82_;
#line 233 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 233 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (name1);
#line 233 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp70_);
#line 233 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (ath1);
#line 233 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp61_);
#line 233 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (aths);
#line 233 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp52_);
#line 233 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (b);
#line 233 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp43_);
#line 233 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (r);
#line 233 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (e);
#line 233 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (d);
#line 233 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (f);
#line 2902 "TDocumentTest.c"
						goto __catch129_g_error;
					}
#line 233 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp83_ = _tmp80_;
#line 233 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp80_ = NULL;
#line 233 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					email1 = G_TYPE_CHECK_INSTANCE_CAST (_tmp83_, GXML_TYPE_ELEMENT, GXmlElement);
#line 234 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp84_ = email1;
#line 234 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gxml_element_set_content (_tmp84_, "fweasley@hogwarts.co.uk");
#line 235 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp85_ = ath1;
#line 235 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp86_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp85_);
#line 235 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp87_ = _tmp86_;
#line 235 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp88_ = _tmp87_;
#line 235 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp89_ = email1;
#line 235 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gee_collection_add ((GeeCollection*) _tmp88_, (GXmlNode*) _tmp89_);
#line 235 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp88_);
#line 236 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp91_ = d;
#line 236 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp92_ = gxml_document_create_element ((GXmlDocument*) _tmp91_, "Author", &_inner_error_);
#line 236 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp90_ = _tmp92_;
#line 236 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 236 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (email1);
#line 236 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp80_);
#line 236 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (name1);
#line 236 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp70_);
#line 236 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (ath1);
#line 236 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp61_);
#line 236 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (aths);
#line 236 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp52_);
#line 236 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (b);
#line 236 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp43_);
#line 236 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (r);
#line 236 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (e);
#line 236 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (d);
#line 236 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (f);
#line 2965 "TDocumentTest.c"
						goto __catch129_g_error;
					}
#line 236 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp93_ = _tmp90_;
#line 236 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp90_ = NULL;
#line 236 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					ath2 = G_TYPE_CHECK_INSTANCE_CAST (_tmp93_, GXML_TYPE_ELEMENT, GXmlElement);
#line 237 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp94_ = aths;
#line 237 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp95_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp94_);
#line 237 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp96_ = _tmp95_;
#line 237 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp97_ = _tmp96_;
#line 237 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp98_ = ath2;
#line 237 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gee_collection_add ((GeeCollection*) _tmp97_, (GXmlNode*) _tmp98_);
#line 237 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp97_);
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp100_ = d;
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp101_ = gxml_document_create_element ((GXmlDocument*) _tmp100_, "Name", &_inner_error_);
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp99_ = _tmp101_;
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (ath2);
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp90_);
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (email1);
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp80_);
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (name1);
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp70_);
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (ath1);
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp61_);
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (aths);
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp52_);
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (b);
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp43_);
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (r);
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (e);
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (d);
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (f);
#line 3028 "TDocumentTest.c"
						goto __catch129_g_error;
					}
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp102_ = _tmp99_;
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp99_ = NULL;
#line 238 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					name2 = G_TYPE_CHECK_INSTANCE_CAST (_tmp102_, GXML_TYPE_ELEMENT, GXmlElement);
#line 239 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp103_ = name2;
#line 239 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gxml_element_set_content (_tmp103_, "Greoge");
#line 240 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp104_ = ath2;
#line 240 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp105_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp104_);
#line 240 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp106_ = _tmp105_;
#line 240 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp107_ = _tmp106_;
#line 240 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp108_ = name2;
#line 240 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gee_collection_add ((GeeCollection*) _tmp107_, (GXmlNode*) _tmp108_);
#line 240 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp107_);
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp110_ = d;
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp111_ = gxml_document_create_element ((GXmlDocument*) _tmp110_, "Email", &_inner_error_);
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp109_ = _tmp111_;
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (name2);
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp99_);
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (ath2);
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp90_);
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (email1);
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp80_);
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (name1);
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp70_);
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (ath1);
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp61_);
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (aths);
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp52_);
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (b);
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp43_);
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (r);
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (e);
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (d);
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (f);
#line 3099 "TDocumentTest.c"
						goto __catch129_g_error;
					}
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp112_ = _tmp109_;
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp109_ = NULL;
#line 241 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					email2 = G_TYPE_CHECK_INSTANCE_CAST (_tmp112_, GXML_TYPE_ELEMENT, GXmlElement);
#line 242 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp113_ = email2;
#line 242 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gxml_element_set_content (_tmp113_, "gweasley@hogwarts.co.uk");
#line 243 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp114_ = ath2;
#line 243 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp115_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp114_);
#line 243 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp116_ = _tmp115_;
#line 243 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp117_ = _tmp116_;
#line 243 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp118_ = email2;
#line 243 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gee_collection_add ((GeeCollection*) _tmp117_, (GXmlNode*) _tmp118_);
#line 243 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp117_);
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (email2);
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp109_);
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (name2);
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp99_);
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (ath2);
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp90_);
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (email1);
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp80_);
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (name1);
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp70_);
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (ath1);
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp61_);
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (aths);
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp52_);
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (b);
#line 223 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp43_);
#line 3158 "TDocumentTest.c"
				}
			}
		}
#line 245 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp119_ = d;
#line 245 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp120_ = gxml_document_get_root ((GXmlDocument*) _tmp119_);
#line 245 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp121_ = _tmp120_;
#line 245 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp122_ = _tmp121_;
#line 245 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp123_ = gxml_node_get_children_nodes (_tmp122_);
#line 245 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp124_ = _tmp123_;
#line 245 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp125_ = _tmp124_;
#line 245 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp126_ = gee_collection_get_size ((GeeCollection*) _tmp125_);
#line 245 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp127_ = _tmp126_;
#line 245 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp127_ == 5000, "d.root.children_nodes.size == 5000");
#line 245 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp125_);
#line 245 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp122_);
#line 3186 "TDocumentTest.c"
		{
			GeeBidirList* _n_list = NULL;
			GXmlTDocument* _tmp128_ = NULL;
			GXmlNode* _tmp129_ = NULL;
			GXmlNode* _tmp130_ = NULL;
			GXmlNode* _tmp131_ = NULL;
			GeeBidirList* _tmp132_ = NULL;
			GeeBidirList* _tmp133_ = NULL;
			GeeBidirList* _tmp134_ = NULL;
			gint _n_size = 0;
			GeeBidirList* _tmp135_ = NULL;
			gint _tmp136_ = 0;
			gint _tmp137_ = 0;
			gint _n_index = 0;
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp128_ = d;
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp129_ = gxml_document_get_root ((GXmlDocument*) _tmp128_);
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp130_ = _tmp129_;
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp131_ = _tmp130_;
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp132_ = gxml_node_get_children_nodes (_tmp131_);
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp133_ = _tmp132_;
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp134_ = _tmp133_;
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (_tmp131_);
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_n_list = _tmp134_;
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp135_ = _n_list;
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp136_ = gee_collection_get_size ((GeeCollection*) _tmp135_);
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp137_ = _tmp136_;
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_n_size = _tmp137_;
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_n_index = -1;
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			while (TRUE) {
#line 3231 "TDocumentTest.c"
				gint _tmp138_ = 0;
				gint _tmp139_ = 0;
				gint _tmp140_ = 0;
				GXmlNode* n = NULL;
				GeeBidirList* _tmp141_ = NULL;
				gint _tmp142_ = 0;
				gpointer _tmp143_ = NULL;
				GXmlNode* _tmp144_ = NULL;
				GeeBidirList* _tmp145_ = NULL;
				GeeBidirList* _tmp146_ = NULL;
				GeeBidirList* _tmp147_ = NULL;
				gint _tmp148_ = 0;
				gint _tmp149_ = 0;
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp138_ = _n_index;
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_n_index = _tmp138_ + 1;
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp139_ = _n_index;
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp140_ = _n_size;
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				if (!(_tmp139_ < _tmp140_)) {
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					break;
#line 3257 "TDocumentTest.c"
				}
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp141_ = _n_list;
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp142_ = _n_index;
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp143_ = gee_list_get ((GeeList*) _tmp141_, _tmp142_);
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				n = (GXmlNode*) _tmp143_;
#line 247 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp144_ = n;
#line 247 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp145_ = gxml_node_get_children_nodes (_tmp144_);
#line 247 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp146_ = _tmp145_;
#line 247 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp147_ = _tmp146_;
#line 247 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp148_ = gee_collection_get_size ((GeeCollection*) _tmp147_);
#line 247 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp149_ = _tmp148_;
#line 247 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_vala_assert (_tmp149_ == 1, "n.children_nodes.size == 1");
#line 247 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_g_object_unref0 (_tmp147_);
#line 3283 "TDocumentTest.c"
				{
					GeeBidirList* _cn_list = NULL;
					GXmlNode* _tmp150_ = NULL;
					GeeBidirList* _tmp151_ = NULL;
					GeeBidirList* _tmp152_ = NULL;
					gint _cn_size = 0;
					GeeBidirList* _tmp153_ = NULL;
					gint _tmp154_ = 0;
					gint _tmp155_ = 0;
					gint _cn_index = 0;
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp150_ = n;
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp151_ = gxml_node_get_children_nodes (_tmp150_);
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp152_ = _tmp151_;
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_cn_list = _tmp152_;
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp153_ = _cn_list;
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp154_ = gee_collection_get_size ((GeeCollection*) _tmp153_);
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp155_ = _tmp154_;
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_cn_size = _tmp155_;
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_cn_index = -1;
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					while (TRUE) {
#line 3314 "TDocumentTest.c"
						gint _tmp156_ = 0;
						gint _tmp157_ = 0;
						gint _tmp158_ = 0;
						GXmlNode* cn = NULL;
						GeeBidirList* _tmp159_ = NULL;
						gint _tmp160_ = 0;
						gpointer _tmp161_ = NULL;
						GXmlNode* _tmp162_ = NULL;
						GeeBidirList* _tmp163_ = NULL;
						GeeBidirList* _tmp164_ = NULL;
						GeeBidirList* _tmp165_ = NULL;
						gint _tmp166_ = 0;
						gint _tmp167_ = 0;
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_tmp156_ = _cn_index;
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_cn_index = _tmp156_ + 1;
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_tmp157_ = _cn_index;
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_tmp158_ = _cn_size;
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						if (!(_tmp157_ < _tmp158_)) {
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
							break;
#line 3340 "TDocumentTest.c"
						}
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_tmp159_ = _cn_list;
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_tmp160_ = _cn_index;
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_tmp161_ = gee_list_get ((GeeList*) _tmp159_, _tmp160_);
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						cn = (GXmlNode*) _tmp161_;
#line 249 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_tmp162_ = cn;
#line 249 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_tmp163_ = gxml_node_get_children_nodes (_tmp162_);
#line 249 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_tmp164_ = _tmp163_;
#line 249 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_tmp165_ = _tmp164_;
#line 249 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_tmp166_ = gee_collection_get_size ((GeeCollection*) _tmp165_);
#line 249 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_tmp167_ = _tmp166_;
#line 249 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_vala_assert (_tmp167_ == 2, "cn.children_nodes.size == 2");
#line 249 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp165_);
#line 3366 "TDocumentTest.c"
						{
							GeeBidirList* _ccn_list = NULL;
							GXmlNode* _tmp168_ = NULL;
							GeeBidirList* _tmp169_ = NULL;
							GeeBidirList* _tmp170_ = NULL;
							gint _ccn_size = 0;
							GeeBidirList* _tmp171_ = NULL;
							gint _tmp172_ = 0;
							gint _tmp173_ = 0;
							gint _ccn_index = 0;
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
							_tmp168_ = cn;
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
							_tmp169_ = gxml_node_get_children_nodes (_tmp168_);
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
							_tmp170_ = _tmp169_;
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
							_ccn_list = _tmp170_;
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
							_tmp171_ = _ccn_list;
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
							_tmp172_ = gee_collection_get_size ((GeeCollection*) _tmp171_);
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
							_tmp173_ = _tmp172_;
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
							_ccn_size = _tmp173_;
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
							_ccn_index = -1;
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
							while (TRUE) {
#line 3397 "TDocumentTest.c"
								gint _tmp174_ = 0;
								gint _tmp175_ = 0;
								gint _tmp176_ = 0;
								GXmlNode* ccn = NULL;
								GeeBidirList* _tmp177_ = NULL;
								gint _tmp178_ = 0;
								gpointer _tmp179_ = NULL;
								GXmlNode* _tmp180_ = NULL;
								GeeBidirList* _tmp181_ = NULL;
								GeeBidirList* _tmp182_ = NULL;
								GeeBidirList* _tmp183_ = NULL;
								gint _tmp184_ = 0;
								gint _tmp185_ = 0;
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
								_tmp174_ = _ccn_index;
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
								_ccn_index = _tmp174_ + 1;
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
								_tmp175_ = _ccn_index;
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
								_tmp176_ = _ccn_size;
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
								if (!(_tmp175_ < _tmp176_)) {
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
									break;
#line 3423 "TDocumentTest.c"
								}
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
								_tmp177_ = _ccn_list;
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
								_tmp178_ = _ccn_index;
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
								_tmp179_ = gee_list_get ((GeeList*) _tmp177_, _tmp178_);
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
								ccn = (GXmlNode*) _tmp179_;
#line 251 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
								_tmp180_ = ccn;
#line 251 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
								_tmp181_ = gxml_node_get_children_nodes (_tmp180_);
#line 251 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
								_tmp182_ = _tmp181_;
#line 251 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
								_tmp183_ = _tmp182_;
#line 251 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
								_tmp184_ = gee_collection_get_size ((GeeCollection*) _tmp183_);
#line 251 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
								_tmp185_ = _tmp184_;
#line 251 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
								_vala_assert (_tmp185_ == 2, "ccn.children_nodes.size == 2");
#line 251 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
								_g_object_unref0 (_tmp183_);
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
								_g_object_unref0 (ccn);
#line 3451 "TDocumentTest.c"
							}
#line 250 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
							_g_object_unref0 (_ccn_list);
#line 3455 "TDocumentTest.c"
						}
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (cn);
#line 3459 "TDocumentTest.c"
					}
#line 248 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_cn_list);
#line 3463 "TDocumentTest.c"
				}
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_g_object_unref0 (n);
#line 3467 "TDocumentTest.c"
			}
#line 246 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (_n_list);
#line 3471 "TDocumentTest.c"
		}
#line 202 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (r);
#line 202 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (e);
#line 202 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (d);
#line 202 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (f);
#line 3481 "TDocumentTest.c"
	}
	goto __finally129;
	__catch129_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp186_ = NULL;
		const gchar* _tmp187_ = NULL;
		gchar* _tmp188_ = NULL;
		gchar* _tmp189_ = NULL;
#line 202 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _inner_error_;
#line 202 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_inner_error_ = NULL;
#line 257 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp186_ = e->message;
#line 257 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp187_ = string_to_string (_tmp186_);
#line 257 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp188_ = g_strconcat ("ERROR: ", _tmp187_, NULL);
#line 257 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp189_ = _tmp188_;
#line 257 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_message ("TDocumentTest.vala:257: %s", _tmp189_);
#line 257 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp189_);
#line 258 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_assert_not_reached ();
#line 202 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_error_free0 (e);
#line 3511 "TDocumentTest.c"
	}
	__finally129:
#line 202 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 202 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 202 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 202 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		return;
#line 3522 "TDocumentTest.c"
	}
}


static void ___lambda111__gtest_func (void) {
#line 198 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda111_ ();
#line 3530 "TDocumentTest.c"
}


static void __lambda112_ (void) {
	GError * _inner_error_ = NULL;
	{
		GFile* f = NULL;
		GFile* _tmp0_ = NULL;
		GFile* _tmp1_ = NULL;
		gboolean _tmp2_ = FALSE;
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp4_ = NULL;
		GXmlNode* e = NULL;
		GXmlTDocument* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		GXmlTDocument* _tmp7_ = NULL;
		GeeBidirList* _tmp8_ = NULL;
		GeeBidirList* _tmp9_ = NULL;
		GeeBidirList* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GXmlTDocument* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GeeBidirList* _tmp15_ = NULL;
		gint _tmp16_ = 0;
		gint _tmp17_ = 0;
		GXmlTDocument* _tmp18_ = NULL;
		GXmlNode* _tmp19_ = NULL;
		GXmlNode* _tmp20_ = NULL;
		GXmlNode* _tmp21_ = NULL;
		GXmlTDocument* _tmp22_ = NULL;
		GXmlNode* _tmp23_ = NULL;
		GXmlNode* _tmp24_ = NULL;
		GXmlNode* _tmp25_ = NULL;
		gchar* _tmp26_ = NULL;
		gchar* _tmp27_ = NULL;
		gchar* _tmp28_ = NULL;
		GXmlTDocument* _tmp29_ = NULL;
		GXmlNode* _tmp30_ = NULL;
		GXmlNode* _tmp31_ = NULL;
		GXmlNode* _tmp32_ = NULL;
		gchar* _tmp33_ = NULL;
		gchar* _tmp34_ = NULL;
		gchar* _tmp35_ = NULL;
		GXmlElement* r = NULL;
		GXmlTDocument* _tmp36_ = NULL;
		GXmlNode* _tmp37_ = NULL;
		GXmlNode* _tmp38_ = NULL;
		GXmlElement* _tmp39_ = NULL;
		GXmlTDocument* _tmp119_ = NULL;
		GXmlNode* _tmp120_ = NULL;
		GXmlNode* _tmp121_ = NULL;
		GXmlNode* _tmp122_ = NULL;
		GeeBidirList* _tmp123_ = NULL;
		GeeBidirList* _tmp124_ = NULL;
		GeeBidirList* _tmp125_ = NULL;
		gint _tmp126_ = 0;
		gint _tmp127_ = 0;
		GXmlTDocument* _tmp128_ = NULL;
		GFile* fr = NULL;
		GFile* _tmp129_ = NULL;
		GFile* _tmp130_ = NULL;
		gboolean _tmp131_ = FALSE;
		GMemoryOutputStream* ostream = NULL;
		GMemoryOutputStream* _tmp132_ = NULL;
		GFileInputStream* _tmp133_ = NULL;
		GFile* _tmp134_ = NULL;
		GFileInputStream* _tmp135_ = NULL;
		GMemoryOutputStream* _tmp136_ = NULL;
		GMemoryOutputStream* _tmp137_ = NULL;
		void* _tmp138_ = NULL;
		void* _tmp139_ = NULL;
		gboolean _tmp140_ = FALSE;
		GMemoryOutputStream* _tmp141_ = NULL;
		void* _tmp142_ = NULL;
		void* _tmp143_ = NULL;
		gboolean _tmp144_ = FALSE;
		GMemoryOutputStream* _tmp145_ = NULL;
		void* _tmp146_ = NULL;
		void* _tmp147_ = NULL;
		gboolean _tmp148_ = FALSE;
		GMemoryOutputStream* _tmp149_ = NULL;
		void* _tmp150_ = NULL;
		void* _tmp151_ = NULL;
		gboolean _tmp152_ = FALSE;
		GMemoryOutputStream* _tmp153_ = NULL;
		void* _tmp154_ = NULL;
		void* _tmp155_ = NULL;
		gboolean _tmp156_ = FALSE;
		GFile* _tmp157_ = NULL;
#line 269 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp0_ = g_file_new_for_path (TEST_SAVE_DIR "/t-large.xml");
#line 269 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f = _tmp0_;
#line 270 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp1_ = f;
#line 270 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp2_ = g_file_query_exists (_tmp1_, NULL);
#line 270 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (_tmp2_) {
#line 3631 "TDocumentTest.c"
			GFile* _tmp3_ = NULL;
#line 270 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp3_ = f;
#line 270 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			g_file_delete (_tmp3_, NULL, &_inner_error_);
#line 270 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 270 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_g_object_unref0 (f);
#line 3641 "TDocumentTest.c"
				goto __catch130_g_error;
			}
		}
#line 274 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp4_ = gxml_tdocument_new_from_path (TEST_SAVE_DIR "/t-large.xml");
#line 274 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		d = _tmp4_;
#line 275 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp5_ = d;
#line 275 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp6_ = gxml_document_create_element ((GXmlDocument*) _tmp5_, "bookstore", &_inner_error_);
#line 275 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _tmp6_;
#line 275 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 275 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 275 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 3661 "TDocumentTest.c"
			goto __catch130_g_error;
		}
#line 276 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp7_ = d;
#line 276 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp8_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp7_);
#line 276 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp9_ = _tmp8_;
#line 276 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp10_ = _tmp9_;
#line 276 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp11_ = e;
#line 276 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gee_collection_add ((GeeCollection*) _tmp10_, _tmp11_);
#line 276 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp10_);
#line 277 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp12_ = d;
#line 277 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp13_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp12_);
#line 277 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp14_ = _tmp13_;
#line 277 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp15_ = _tmp14_;
#line 277 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp16_ = gee_collection_get_size ((GeeCollection*) _tmp15_);
#line 277 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp17_ = _tmp16_;
#line 277 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp17_ == 1, "d.children_nodes.size == 1");
#line 277 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp15_);
#line 278 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp18_ = d;
#line 278 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp19_ = gxml_document_get_root ((GXmlDocument*) _tmp18_);
#line 278 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp20_ = _tmp19_;
#line 278 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp21_ = _tmp20_;
#line 278 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp21_ != NULL, "d.root != null");
#line 278 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp21_);
#line 279 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp22_ = d;
#line 279 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp23_ = gxml_document_get_root ((GXmlDocument*) _tmp22_);
#line 279 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp24_ = _tmp23_;
#line 279 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp25_ = _tmp24_;
#line 279 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp26_ = gxml_node_get_name (_tmp25_);
#line 279 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp27_ = _tmp26_;
#line 279 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp28_ = _tmp27_;
#line 279 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp28_, "bookstore") == 0, "d.root.name == \"bookstore\"");
#line 279 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp28_);
#line 279 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp25_);
#line 280 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp29_ = d;
#line 280 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp30_ = gxml_document_get_root ((GXmlDocument*) _tmp29_);
#line 280 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp31_ = _tmp30_;
#line 280 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp32_ = _tmp31_;
#line 280 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp33_ = gxml_node_get_value (_tmp32_);
#line 280 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp34_ = _tmp33_;
#line 280 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp35_ = _tmp34_;
#line 280 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp35_, "") == 0, "d.root.value == \"\"");
#line 280 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp35_);
#line 280 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp32_);
#line 281 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp36_ = d;
#line 281 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp37_ = gxml_document_get_root ((GXmlDocument*) _tmp36_);
#line 281 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp38_ = _tmp37_;
#line 281 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		r = G_TYPE_CHECK_INSTANCE_CAST (_tmp38_, GXML_TYPE_ELEMENT, GXmlElement);
#line 282 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp39_ = r;
#line 282 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_element_set_attr (_tmp39_, "name", "The Great Book");
#line 3758 "TDocumentTest.c"
		{
			gint i = 0;
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			i = 0;
#line 3763 "TDocumentTest.c"
			{
				gboolean _tmp40_ = FALSE;
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp40_ = TRUE;
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				while (TRUE) {
#line 3770 "TDocumentTest.c"
					gint _tmp42_ = 0;
					GXmlNode* _tmp43_ = NULL;
					GXmlTDocument* _tmp44_ = NULL;
					GXmlNode* _tmp45_ = NULL;
					GXmlElement* b = NULL;
					GXmlNode* _tmp46_ = NULL;
					GXmlElement* _tmp47_ = NULL;
					GeeBidirList* _tmp48_ = NULL;
					GeeBidirList* _tmp49_ = NULL;
					GeeBidirList* _tmp50_ = NULL;
					GXmlElement* _tmp51_ = NULL;
					GXmlNode* _tmp52_ = NULL;
					GXmlTDocument* _tmp53_ = NULL;
					GXmlNode* _tmp54_ = NULL;
					GXmlElement* aths = NULL;
					GXmlNode* _tmp55_ = NULL;
					GXmlElement* _tmp56_ = NULL;
					GeeBidirList* _tmp57_ = NULL;
					GeeBidirList* _tmp58_ = NULL;
					GeeBidirList* _tmp59_ = NULL;
					GXmlElement* _tmp60_ = NULL;
					GXmlNode* _tmp61_ = NULL;
					GXmlTDocument* _tmp62_ = NULL;
					GXmlNode* _tmp63_ = NULL;
					GXmlElement* ath1 = NULL;
					GXmlNode* _tmp64_ = NULL;
					GXmlElement* _tmp65_ = NULL;
					GeeBidirList* _tmp66_ = NULL;
					GeeBidirList* _tmp67_ = NULL;
					GeeBidirList* _tmp68_ = NULL;
					GXmlElement* _tmp69_ = NULL;
					GXmlNode* _tmp70_ = NULL;
					GXmlTDocument* _tmp71_ = NULL;
					GXmlNode* _tmp72_ = NULL;
					GXmlElement* name1 = NULL;
					GXmlNode* _tmp73_ = NULL;
					GXmlElement* _tmp74_ = NULL;
					GXmlElement* _tmp75_ = NULL;
					GeeBidirList* _tmp76_ = NULL;
					GeeBidirList* _tmp77_ = NULL;
					GeeBidirList* _tmp78_ = NULL;
					GXmlElement* _tmp79_ = NULL;
					GXmlNode* _tmp80_ = NULL;
					GXmlTDocument* _tmp81_ = NULL;
					GXmlNode* _tmp82_ = NULL;
					GXmlElement* email1 = NULL;
					GXmlNode* _tmp83_ = NULL;
					GXmlElement* _tmp84_ = NULL;
					GXmlElement* _tmp85_ = NULL;
					GeeBidirList* _tmp86_ = NULL;
					GeeBidirList* _tmp87_ = NULL;
					GeeBidirList* _tmp88_ = NULL;
					GXmlElement* _tmp89_ = NULL;
					GXmlNode* _tmp90_ = NULL;
					GXmlTDocument* _tmp91_ = NULL;
					GXmlNode* _tmp92_ = NULL;
					GXmlElement* ath2 = NULL;
					GXmlNode* _tmp93_ = NULL;
					GXmlElement* _tmp94_ = NULL;
					GeeBidirList* _tmp95_ = NULL;
					GeeBidirList* _tmp96_ = NULL;
					GeeBidirList* _tmp97_ = NULL;
					GXmlElement* _tmp98_ = NULL;
					GXmlNode* _tmp99_ = NULL;
					GXmlTDocument* _tmp100_ = NULL;
					GXmlNode* _tmp101_ = NULL;
					GXmlElement* name2 = NULL;
					GXmlNode* _tmp102_ = NULL;
					GXmlElement* _tmp103_ = NULL;
					GXmlElement* _tmp104_ = NULL;
					GeeBidirList* _tmp105_ = NULL;
					GeeBidirList* _tmp106_ = NULL;
					GeeBidirList* _tmp107_ = NULL;
					GXmlElement* _tmp108_ = NULL;
					GXmlNode* _tmp109_ = NULL;
					GXmlTDocument* _tmp110_ = NULL;
					GXmlNode* _tmp111_ = NULL;
					GXmlElement* email2 = NULL;
					GXmlNode* _tmp112_ = NULL;
					GXmlElement* _tmp113_ = NULL;
					GXmlElement* _tmp114_ = NULL;
					GeeBidirList* _tmp115_ = NULL;
					GeeBidirList* _tmp116_ = NULL;
					GeeBidirList* _tmp117_ = NULL;
					GXmlElement* _tmp118_ = NULL;
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					if (!_tmp40_) {
#line 3858 "TDocumentTest.c"
						gint _tmp41_ = 0;
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_tmp41_ = i;
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						i = _tmp41_ + 1;
#line 3864 "TDocumentTest.c"
					}
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp40_ = FALSE;
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp42_ = i;
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					if (!(_tmp42_ < 30000)) {
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						break;
#line 3874 "TDocumentTest.c"
					}
#line 287 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp44_ = d;
#line 287 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp45_ = gxml_document_create_element ((GXmlDocument*) _tmp44_, "book", &_inner_error_);
#line 287 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp43_ = _tmp45_;
#line 287 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 287 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (r);
#line 287 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (e);
#line 287 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (d);
#line 287 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (f);
#line 3892 "TDocumentTest.c"
						goto __catch130_g_error;
					}
#line 287 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp46_ = _tmp43_;
#line 287 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp43_ = NULL;
#line 287 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					b = G_TYPE_CHECK_INSTANCE_CAST (_tmp46_, GXML_TYPE_ELEMENT, GXmlElement);
#line 288 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp47_ = r;
#line 288 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp48_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp47_);
#line 288 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp49_ = _tmp48_;
#line 288 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp50_ = _tmp49_;
#line 288 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp51_ = b;
#line 288 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gee_collection_add ((GeeCollection*) _tmp50_, (GXmlNode*) _tmp51_);
#line 288 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp50_);
#line 289 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp53_ = d;
#line 289 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp54_ = gxml_document_create_element ((GXmlDocument*) _tmp53_, "Authors", &_inner_error_);
#line 289 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp52_ = _tmp54_;
#line 289 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 289 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (b);
#line 289 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp43_);
#line 289 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (r);
#line 289 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (e);
#line 289 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (d);
#line 289 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (f);
#line 3935 "TDocumentTest.c"
						goto __catch130_g_error;
					}
#line 289 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp55_ = _tmp52_;
#line 289 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp52_ = NULL;
#line 289 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					aths = G_TYPE_CHECK_INSTANCE_CAST (_tmp55_, GXML_TYPE_ELEMENT, GXmlElement);
#line 290 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp56_ = b;
#line 290 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp57_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp56_);
#line 290 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp58_ = _tmp57_;
#line 290 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp59_ = _tmp58_;
#line 290 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp60_ = aths;
#line 290 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gee_collection_add ((GeeCollection*) _tmp59_, (GXmlNode*) _tmp60_);
#line 290 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp59_);
#line 291 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp62_ = d;
#line 291 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp63_ = gxml_document_create_element ((GXmlDocument*) _tmp62_, "Author", &_inner_error_);
#line 291 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp61_ = _tmp63_;
#line 291 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 291 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (aths);
#line 291 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp52_);
#line 291 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (b);
#line 291 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp43_);
#line 291 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (r);
#line 291 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (e);
#line 291 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (d);
#line 291 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (f);
#line 3982 "TDocumentTest.c"
						goto __catch130_g_error;
					}
#line 291 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp64_ = _tmp61_;
#line 291 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp61_ = NULL;
#line 291 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					ath1 = G_TYPE_CHECK_INSTANCE_CAST (_tmp64_, GXML_TYPE_ELEMENT, GXmlElement);
#line 292 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp65_ = aths;
#line 292 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp66_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp65_);
#line 292 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp67_ = _tmp66_;
#line 292 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp68_ = _tmp67_;
#line 292 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp69_ = ath1;
#line 292 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gee_collection_add ((GeeCollection*) _tmp68_, (GXmlNode*) _tmp69_);
#line 292 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp68_);
#line 293 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp71_ = d;
#line 293 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp72_ = gxml_document_create_element ((GXmlDocument*) _tmp71_, "Name", &_inner_error_);
#line 293 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp70_ = _tmp72_;
#line 293 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 293 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (ath1);
#line 293 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp61_);
#line 293 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (aths);
#line 293 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp52_);
#line 293 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (b);
#line 293 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp43_);
#line 293 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (r);
#line 293 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (e);
#line 293 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (d);
#line 293 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (f);
#line 4033 "TDocumentTest.c"
						goto __catch130_g_error;
					}
#line 293 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp73_ = _tmp70_;
#line 293 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp70_ = NULL;
#line 293 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					name1 = G_TYPE_CHECK_INSTANCE_CAST (_tmp73_, GXML_TYPE_ELEMENT, GXmlElement);
#line 294 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp74_ = name1;
#line 294 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gxml_element_set_content (_tmp74_, "Fred");
#line 295 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp75_ = ath1;
#line 295 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp76_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp75_);
#line 295 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp77_ = _tmp76_;
#line 295 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp78_ = _tmp77_;
#line 295 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp79_ = name1;
#line 295 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gee_collection_add ((GeeCollection*) _tmp78_, (GXmlNode*) _tmp79_);
#line 295 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp78_);
#line 296 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp81_ = d;
#line 296 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp82_ = gxml_document_create_element ((GXmlDocument*) _tmp81_, "Email", &_inner_error_);
#line 296 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp80_ = _tmp82_;
#line 296 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 296 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (name1);
#line 296 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp70_);
#line 296 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (ath1);
#line 296 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp61_);
#line 296 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (aths);
#line 296 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp52_);
#line 296 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (b);
#line 296 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp43_);
#line 296 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (r);
#line 296 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (e);
#line 296 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (d);
#line 296 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (f);
#line 4092 "TDocumentTest.c"
						goto __catch130_g_error;
					}
#line 296 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp83_ = _tmp80_;
#line 296 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp80_ = NULL;
#line 296 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					email1 = G_TYPE_CHECK_INSTANCE_CAST (_tmp83_, GXML_TYPE_ELEMENT, GXmlElement);
#line 297 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp84_ = email1;
#line 297 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gxml_element_set_content (_tmp84_, "fweasley@hogwarts.co.uk");
#line 298 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp85_ = ath1;
#line 298 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp86_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp85_);
#line 298 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp87_ = _tmp86_;
#line 298 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp88_ = _tmp87_;
#line 298 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp89_ = email1;
#line 298 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gee_collection_add ((GeeCollection*) _tmp88_, (GXmlNode*) _tmp89_);
#line 298 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp88_);
#line 299 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp91_ = d;
#line 299 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp92_ = gxml_document_create_element ((GXmlDocument*) _tmp91_, "Author", &_inner_error_);
#line 299 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp90_ = _tmp92_;
#line 299 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 299 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (email1);
#line 299 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp80_);
#line 299 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (name1);
#line 299 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp70_);
#line 299 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (ath1);
#line 299 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp61_);
#line 299 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (aths);
#line 299 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp52_);
#line 299 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (b);
#line 299 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp43_);
#line 299 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (r);
#line 299 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (e);
#line 299 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (d);
#line 299 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (f);
#line 4155 "TDocumentTest.c"
						goto __catch130_g_error;
					}
#line 299 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp93_ = _tmp90_;
#line 299 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp90_ = NULL;
#line 299 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					ath2 = G_TYPE_CHECK_INSTANCE_CAST (_tmp93_, GXML_TYPE_ELEMENT, GXmlElement);
#line 300 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp94_ = aths;
#line 300 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp95_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp94_);
#line 300 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp96_ = _tmp95_;
#line 300 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp97_ = _tmp96_;
#line 300 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp98_ = ath2;
#line 300 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gee_collection_add ((GeeCollection*) _tmp97_, (GXmlNode*) _tmp98_);
#line 300 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp97_);
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp100_ = d;
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp101_ = gxml_document_create_element ((GXmlDocument*) _tmp100_, "Name", &_inner_error_);
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp99_ = _tmp101_;
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (ath2);
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp90_);
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (email1);
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp80_);
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (name1);
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp70_);
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (ath1);
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp61_);
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (aths);
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp52_);
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (b);
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp43_);
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (r);
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (e);
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (d);
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (f);
#line 4218 "TDocumentTest.c"
						goto __catch130_g_error;
					}
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp102_ = _tmp99_;
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp99_ = NULL;
#line 301 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					name2 = G_TYPE_CHECK_INSTANCE_CAST (_tmp102_, GXML_TYPE_ELEMENT, GXmlElement);
#line 302 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp103_ = name2;
#line 302 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gxml_element_set_content (_tmp103_, "Greoge");
#line 303 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp104_ = ath2;
#line 303 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp105_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp104_);
#line 303 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp106_ = _tmp105_;
#line 303 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp107_ = _tmp106_;
#line 303 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp108_ = name2;
#line 303 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gee_collection_add ((GeeCollection*) _tmp107_, (GXmlNode*) _tmp108_);
#line 303 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp107_);
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp110_ = d;
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp111_ = gxml_document_create_element ((GXmlDocument*) _tmp110_, "Email", &_inner_error_);
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp109_ = _tmp111_;
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (name2);
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp99_);
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (ath2);
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp90_);
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (email1);
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp80_);
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (name1);
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp70_);
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (ath1);
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp61_);
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (aths);
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp52_);
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (b);
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (_tmp43_);
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (r);
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (e);
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (d);
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
						_g_object_unref0 (f);
#line 4289 "TDocumentTest.c"
						goto __catch130_g_error;
					}
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp112_ = _tmp109_;
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp109_ = NULL;
#line 304 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					email2 = G_TYPE_CHECK_INSTANCE_CAST (_tmp112_, GXML_TYPE_ELEMENT, GXmlElement);
#line 305 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp113_ = email2;
#line 305 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gxml_element_set_content (_tmp113_, "gweasley@hogwarts.co.uk");
#line 306 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp114_ = ath2;
#line 306 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp115_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp114_);
#line 306 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp116_ = _tmp115_;
#line 306 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp117_ = _tmp116_;
#line 306 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_tmp118_ = email2;
#line 306 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					gee_collection_add ((GeeCollection*) _tmp117_, (GXmlNode*) _tmp118_);
#line 306 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp117_);
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (email2);
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp109_);
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (name2);
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp99_);
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (ath2);
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp90_);
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (email1);
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp80_);
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (name1);
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp70_);
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (ath1);
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp61_);
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (aths);
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp52_);
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (b);
#line 286 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					_g_object_unref0 (_tmp43_);
#line 4348 "TDocumentTest.c"
				}
			}
		}
#line 308 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp119_ = d;
#line 308 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp120_ = gxml_document_get_root ((GXmlDocument*) _tmp119_);
#line 308 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp121_ = _tmp120_;
#line 308 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp122_ = _tmp121_;
#line 308 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp123_ = gxml_node_get_children_nodes (_tmp122_);
#line 308 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp124_ = _tmp123_;
#line 308 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp125_ = _tmp124_;
#line 308 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp126_ = gee_collection_get_size ((GeeCollection*) _tmp125_);
#line 308 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp127_ = _tmp126_;
#line 308 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp127_ == 30000, "d.root.children_nodes.size == 30000");
#line 308 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp125_);
#line 308 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp122_);
#line 309 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp128_ = d;
#line 309 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_document_save ((GXmlDocument*) _tmp128_, NULL, &_inner_error_);
#line 309 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 309 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (r);
#line 309 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 309 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 309 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 4390 "TDocumentTest.c"
			goto __catch130_g_error;
		}
#line 310 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_test_message ("Reading saved file...");
#line 311 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp129_ = g_file_new_for_path (TEST_SAVE_DIR "/t-large.xml");
#line 311 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		fr = _tmp129_;
#line 312 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp130_ = fr;
#line 312 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp131_ = g_file_query_exists (_tmp130_, NULL);
#line 312 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp131_, "fr.query_exists ()");
#line 313 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp132_ = (GMemoryOutputStream*) g_memory_output_stream_new_resizable ();
#line 313 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		ostream = _tmp132_;
#line 314 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp134_ = fr;
#line 314 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp135_ = g_file_read (_tmp134_, NULL, &_inner_error_);
#line 314 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp133_ = _tmp135_;
#line 314 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 314 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (ostream);
#line 314 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (fr);
#line 314 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (r);
#line 314 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 314 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 314 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 4429 "TDocumentTest.c"
			goto __catch130_g_error;
		}
#line 314 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp136_ = ostream;
#line 314 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_output_stream_splice ((GOutputStream*) _tmp136_, (GInputStream*) _tmp133_, G_OUTPUT_STREAM_SPLICE_NONE, NULL, &_inner_error_);
#line 314 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 314 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (_tmp133_);
#line 314 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (ostream);
#line 314 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (fr);
#line 314 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (r);
#line 314 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 314 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 314 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 4452 "TDocumentTest.c"
			goto __catch130_g_error;
		}
#line 315 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp137_ = ostream;
#line 315 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp138_ = g_memory_output_stream_get_data (_tmp137_);
#line 315 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp139_ = _tmp138_;
#line 315 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp140_ = string_contains ((const gchar*) _tmp139_, "<?xml version=\"1.0\"?>");
#line 315 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp140_, "\"<?xml version=\\\"1.0\\\"?>\" in ((string)ostream.data)");
#line 316 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp141_ = ostream;
#line 316 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp142_ = g_memory_output_stream_get_data (_tmp141_);
#line 316 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp143_ = _tmp142_;
#line 316 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp144_ = string_contains ((const gchar*) _tmp143_, "<bookstore name=\"The Great Book\">");
#line 316 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp144_, "\"<bookstore name=\\\"The Great Book\\\">\" in ((string)ostream.data)");
#line 317 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp145_ = ostream;
#line 317 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp146_ = g_memory_output_stream_get_data (_tmp145_);
#line 317 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp147_ = _tmp146_;
#line 317 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp148_ = string_contains ((const gchar*) _tmp147_, "<book>");
#line 317 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp148_, "\"<book>\" in ((string)ostream.data)");
#line 318 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp149_ = ostream;
#line 318 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp150_ = g_memory_output_stream_get_data (_tmp149_);
#line 318 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp151_ = _tmp150_;
#line 318 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp152_ = string_contains ((const gchar*) _tmp151_, "<Authors>");
#line 318 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp152_, "\"<Authors>\" in ((string)ostream.data)");
#line 319 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp153_ = ostream;
#line 319 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp154_ = g_memory_output_stream_get_data (_tmp153_);
#line 319 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp155_ = _tmp154_;
#line 319 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp156_ = string_contains ((const gchar*) _tmp155_, "<Author>");
#line 319 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp156_, "\"<Author>\" in ((string)ostream.data)");
#line 320 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp157_ = f;
#line 320 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_file_delete (_tmp157_, NULL, &_inner_error_);
#line 320 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 320 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (_tmp133_);
#line 320 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (ostream);
#line 320 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (fr);
#line 320 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (r);
#line 320 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 320 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 320 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 4525 "TDocumentTest.c"
			goto __catch130_g_error;
		}
#line 265 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp133_);
#line 265 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (ostream);
#line 265 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (fr);
#line 265 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (r);
#line 265 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (e);
#line 265 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (d);
#line 265 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (f);
#line 4542 "TDocumentTest.c"
	}
	goto __finally130;
	__catch130_g_error:
	{
		GError* e = NULL;
#line 265 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _inner_error_;
#line 265 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_inner_error_ = NULL;
#line 326 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_assert_not_reached ();
#line 265 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_error_free0 (e);
#line 4556 "TDocumentTest.c"
	}
	__finally130:
#line 265 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 265 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 265 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 265 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		return;
#line 4567 "TDocumentTest.c"
	}
}


static void ___lambda112__gtest_func (void) {
#line 261 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda112_ ();
#line 4575 "TDocumentTest.c"
}


static void __lambda113_ (void) {
	GError * _inner_error_ = NULL;
	{
		GFile* f = NULL;
		GFile* _tmp0_ = NULL;
		GFile* _tmp1_ = NULL;
		gboolean _tmp2_ = FALSE;
		TDocumentTestTTestObject* ot = NULL;
		TDocumentTestTTestObject* _tmp4_ = NULL;
		TDocumentTestTTestObject* _tmp5_ = NULL;
		GXmlTDocument* dt = NULL;
		GXmlTDocument* _tmp6_ = NULL;
		TDocumentTestTTestObject* _tmp7_ = NULL;
		GXmlTDocument* _tmp8_ = NULL;
		GXmlNode* _tmp9_ = NULL;
		GXmlNode* _tmp10_ = NULL;
		GXmlTDocument* _tmp11_ = NULL;
		GFile* _tmp12_ = NULL;
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp13_ = NULL;
		GXmlNode* e = NULL;
		GXmlTDocument* _tmp14_ = NULL;
		GXmlNode* _tmp15_ = NULL;
		GXmlTDocument* _tmp16_ = NULL;
		GeeBidirList* _tmp17_ = NULL;
		GeeBidirList* _tmp18_ = NULL;
		GeeBidirList* _tmp19_ = NULL;
		GXmlNode* _tmp20_ = NULL;
		GXmlTDocument* _tmp21_ = NULL;
		GeeBidirList* _tmp22_ = NULL;
		GeeBidirList* _tmp23_ = NULL;
		GeeBidirList* _tmp24_ = NULL;
		gint _tmp25_ = 0;
		gint _tmp26_ = 0;
		GXmlTDocument* _tmp27_ = NULL;
		GXmlNode* _tmp28_ = NULL;
		GXmlNode* _tmp29_ = NULL;
		GXmlNode* _tmp30_ = NULL;
		GXmlTDocument* _tmp31_ = NULL;
		GXmlNode* _tmp32_ = NULL;
		GXmlNode* _tmp33_ = NULL;
		GXmlNode* _tmp34_ = NULL;
		gchar* _tmp35_ = NULL;
		gchar* _tmp36_ = NULL;
		gchar* _tmp37_ = NULL;
		GXmlTDocument* _tmp38_ = NULL;
		GXmlNode* _tmp39_ = NULL;
		GXmlNode* _tmp40_ = NULL;
		GXmlNode* _tmp41_ = NULL;
		gchar* _tmp42_ = NULL;
		gchar* _tmp43_ = NULL;
		gchar* _tmp44_ = NULL;
		GXmlTDocument* _tmp45_ = NULL;
		GFile* _tmp46_ = NULL;
		GFile* _tmp47_ = NULL;
		gboolean _tmp48_ = FALSE;
		GFile* bf = NULL;
		GFile* _tmp49_ = NULL;
		GFile* _tmp50_ = NULL;
		gboolean _tmp51_ = FALSE;
		GFileInputStream* istream = NULL;
		GFile* _tmp52_ = NULL;
		GFileInputStream* _tmp53_ = NULL;
		GMemoryOutputStream* b = NULL;
		GMemoryOutputStream* _tmp54_ = NULL;
		GMemoryOutputStream* _tmp55_ = NULL;
		GFileInputStream* _tmp56_ = NULL;
		GMemoryOutputStream* _tmp57_ = NULL;
		void* _tmp58_ = NULL;
		void* _tmp59_ = NULL;
		gboolean _tmp60_ = FALSE;
		GMemoryOutputStream* _tmp61_ = NULL;
		void* _tmp62_ = NULL;
		void* _tmp63_ = NULL;
		gboolean _tmp64_ = FALSE;
		GFile* _tmp65_ = NULL;
		GFile* _tmp66_ = NULL;
#line 331 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp0_ = g_file_new_for_path (TEST_SAVE_DIR "/t-test.xml");
#line 331 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f = _tmp0_;
#line 332 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp1_ = f;
#line 332 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp2_ = g_file_query_exists (_tmp1_, NULL);
#line 332 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (_tmp2_) {
#line 4666 "TDocumentTest.c"
			GFile* _tmp3_ = NULL;
#line 332 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp3_ = f;
#line 332 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			g_file_delete (_tmp3_, NULL, &_inner_error_);
#line 332 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 332 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_g_object_unref0 (f);
#line 4676 "TDocumentTest.c"
				goto __catch131_g_error;
			}
		}
#line 333 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp4_ = tdocument_test_ttest_object_new ();
#line 333 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		ot = _tmp4_;
#line 334 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp5_ = ot;
#line 334 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		tdocument_test_ttest_object_set_name (_tmp5_, "test1");
#line 335 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp6_ = gxml_tdocument_new ();
#line 335 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		dt = _tmp6_;
#line 336 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp7_ = ot;
#line 336 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp8_ = dt;
#line 336 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp9_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) _tmp7_, (GXmlNode*) _tmp8_, &_inner_error_);
#line 336 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp10_ = _tmp9_;
#line 336 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp10_);
#line 336 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 336 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (dt);
#line 336 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (ot);
#line 336 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 4710 "TDocumentTest.c"
			goto __catch131_g_error;
		}
#line 337 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp11_ = dt;
#line 337 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp12_ = f;
#line 337 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_document_save_as ((GXmlDocument*) _tmp11_, _tmp12_, NULL, &_inner_error_);
#line 337 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 337 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (dt);
#line 337 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (ot);
#line 337 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 4727 "TDocumentTest.c"
			goto __catch131_g_error;
		}
#line 338 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp13_ = gxml_tdocument_new ();
#line 338 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		d = _tmp13_;
#line 339 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp14_ = d;
#line 339 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp15_ = gxml_document_create_element ((GXmlDocument*) _tmp14_, "root", &_inner_error_);
#line 339 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _tmp15_;
#line 339 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 339 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 339 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (dt);
#line 339 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (ot);
#line 339 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 4750 "TDocumentTest.c"
			goto __catch131_g_error;
		}
#line 340 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp16_ = d;
#line 340 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp17_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp16_);
#line 340 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp18_ = _tmp17_;
#line 340 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp19_ = _tmp18_;
#line 340 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp20_ = e;
#line 340 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gee_collection_add ((GeeCollection*) _tmp19_, _tmp20_);
#line 340 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp19_);
#line 341 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp21_ = d;
#line 341 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp22_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp21_);
#line 341 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp23_ = _tmp22_;
#line 341 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp24_ = _tmp23_;
#line 341 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp25_ = gee_collection_get_size ((GeeCollection*) _tmp24_);
#line 341 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp26_ = _tmp25_;
#line 341 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp26_ == 1, "d.children_nodes.size == 1");
#line 341 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp24_);
#line 342 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp27_ = d;
#line 342 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp28_ = gxml_document_get_root ((GXmlDocument*) _tmp27_);
#line 342 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp29_ = _tmp28_;
#line 342 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp30_ = _tmp29_;
#line 342 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp30_ != NULL, "d.root != null");
#line 342 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp30_);
#line 343 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp31_ = d;
#line 343 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp32_ = gxml_document_get_root ((GXmlDocument*) _tmp31_);
#line 343 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp33_ = _tmp32_;
#line 343 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp34_ = _tmp33_;
#line 343 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp35_ = gxml_node_get_name (_tmp34_);
#line 343 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp36_ = _tmp35_;
#line 343 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp37_ = _tmp36_;
#line 343 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp37_, "root") == 0, "d.root.name == \"root\"");
#line 343 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp37_);
#line 343 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp34_);
#line 344 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp38_ = d;
#line 344 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp39_ = gxml_document_get_root ((GXmlDocument*) _tmp38_);
#line 344 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp40_ = _tmp39_;
#line 344 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp41_ = _tmp40_;
#line 344 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp42_ = gxml_node_get_value (_tmp41_);
#line 344 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp43_ = _tmp42_;
#line 344 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp44_ = _tmp43_;
#line 344 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp44_, "") == 0, "d.root.value == \"\"");
#line 344 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp44_);
#line 344 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp41_);
#line 345 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp45_ = d;
#line 345 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp46_ = f;
#line 345 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_document_save_as ((GXmlDocument*) _tmp45_, _tmp46_, NULL, &_inner_error_);
#line 345 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 345 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 345 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 345 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (dt);
#line 345 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (ot);
#line 345 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 4853 "TDocumentTest.c"
			goto __catch131_g_error;
		}
#line 346 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp47_ = f;
#line 346 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp48_ = g_file_query_exists (_tmp47_, NULL);
#line 346 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp48_, "f.query_exists ()");
#line 347 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp49_ = g_file_new_for_path (TEST_SAVE_DIR "/t-test.xml~");
#line 347 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		bf = _tmp49_;
#line 348 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp50_ = bf;
#line 348 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp51_ = g_file_query_exists (_tmp50_, NULL);
#line 348 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp51_, "bf.query_exists ()");
#line 349 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp52_ = f;
#line 349 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp53_ = g_file_read (_tmp52_, NULL, &_inner_error_);
#line 349 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		istream = _tmp53_;
#line 349 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 349 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (bf);
#line 349 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 349 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 349 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (dt);
#line 349 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (ot);
#line 349 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 4892 "TDocumentTest.c"
			goto __catch131_g_error;
		}
#line 350 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp54_ = (GMemoryOutputStream*) g_memory_output_stream_new_resizable ();
#line 350 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		b = _tmp54_;
#line 351 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp55_ = b;
#line 351 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp56_ = istream;
#line 351 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_output_stream_splice ((GOutputStream*) _tmp55_, (GInputStream*) _tmp56_, 0, NULL, &_inner_error_);
#line 351 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 351 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (b);
#line 351 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (istream);
#line 351 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (bf);
#line 351 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 351 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 351 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (dt);
#line 351 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (ot);
#line 351 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 4923 "TDocumentTest.c"
			goto __catch131_g_error;
		}
#line 352 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp57_ = b;
#line 352 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp58_ = g_memory_output_stream_get_data (_tmp57_);
#line 352 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp59_ = _tmp58_;
#line 352 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp60_ = string_contains ((const gchar*) _tmp59_, "<?xml version=\"1.0\"?>");
#line 352 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp60_, "\"<?xml version=\\\"1.0\\\"?>\" in ((string)b.data)");
#line 353 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp61_ = b;
#line 353 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp62_ = g_memory_output_stream_get_data (_tmp61_);
#line 353 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp63_ = _tmp62_;
#line 353 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp64_ = string_contains ((const gchar*) _tmp63_, "<root/>");
#line 353 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp64_, "\"<root/>\" in ((string)b.data)");
#line 354 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp65_ = f;
#line 354 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_file_delete (_tmp65_, NULL, &_inner_error_);
#line 354 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 354 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (b);
#line 354 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (istream);
#line 354 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (bf);
#line 354 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 354 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 354 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (dt);
#line 354 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (ot);
#line 354 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 4968 "TDocumentTest.c"
			goto __catch131_g_error;
		}
#line 355 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp66_ = bf;
#line 355 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_file_delete (_tmp66_, NULL, &_inner_error_);
#line 355 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 355 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (b);
#line 355 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (istream);
#line 355 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (bf);
#line 355 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (e);
#line 355 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 355 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (dt);
#line 355 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (ot);
#line 355 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 4993 "TDocumentTest.c"
			goto __catch131_g_error;
		}
#line 330 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (b);
#line 330 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (istream);
#line 330 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (bf);
#line 330 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (e);
#line 330 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (d);
#line 330 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (dt);
#line 330 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (ot);
#line 330 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (f);
#line 5012 "TDocumentTest.c"
	}
	goto __finally131;
	__catch131_g_error:
	{
		GError* e = NULL;
#line 330 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _inner_error_;
#line 330 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_inner_error_ = NULL;
#line 361 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_assert_not_reached ();
#line 330 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_error_free0 (e);
#line 5026 "TDocumentTest.c"
	}
	__finally131:
#line 330 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 330 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 330 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 330 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		return;
#line 5037 "TDocumentTest.c"
	}
}


static void ___lambda113__gtest_func (void) {
#line 329 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda113_ ();
#line 5045 "TDocumentTest.c"
}


static void __lambda114_ (void) {
	GXmlTDocument* doc = NULL;
	GXmlTDocument* _tmp0_ = NULL;
	GError * _inner_error_ = NULL;
#line 365 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp0_ = gxml_tdocument_new ();
#line 365 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	doc = _tmp0_;
#line 5057 "TDocumentTest.c"
	{
		GXmlNode* r = NULL;
		GXmlNode* _tmp1_ = NULL;
		GeeBidirList* _tmp2_ = NULL;
		GeeBidirList* _tmp3_ = NULL;
		GeeBidirList* _tmp4_ = NULL;
		gchar* str = NULL;
		gchar* _tmp5_ = NULL;
		gboolean _tmp6_ = FALSE;
		gboolean _tmp7_ = FALSE;
		gchar* _tmp8_ = NULL;
		gchar* _tmp9_ = NULL;
		gboolean _tmp10_ = FALSE;
#line 367 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp1_ = gxml_document_create_element ((GXmlDocument*) doc, "root", &_inner_error_);
#line 367 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		r = _tmp1_;
#line 367 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 5077 "TDocumentTest.c"
			goto __catch132_g_error;
		}
#line 368 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp2_ = gxml_tnode_get_children_nodes ((GXmlTNode*) doc);
#line 368 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp3_ = _tmp2_;
#line 368 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp4_ = _tmp3_;
#line 368 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gee_collection_add ((GeeCollection*) _tmp4_, r);
#line 368 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp4_);
#line 372 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp5_ = gxml_tnode_to_string ((GXmlTNode*) doc);
#line 372 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		str = _tmp5_;
#line 373 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp6_ = string_contains (str, "<?xml version=\"1.0\"?>");
#line 373 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp6_, "\"<?xml version=\\\"1.0\\\"?>\" in str");
#line 374 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp7_ = string_contains (str, "<root/>");
#line 374 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp7_, "\"<root/>\" in str");
#line 375 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp8_ = gxml_tnode_to_string ((GXmlTNode*) doc);
#line 375 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp9_ = _tmp8_;
#line 375 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp10_ = string_contains (_tmp9_, "<root/>");
#line 375 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp10_, "\"<root/>\" in doc.to_string ()");
#line 375 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp9_);
#line 366 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (str);
#line 366 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (r);
#line 5116 "TDocumentTest.c"
	}
	goto __finally132;
	__catch132_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp11_ = NULL;
#line 366 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _inner_error_;
#line 366 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_inner_error_ = NULL;
#line 377 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp11_ = e->message;
#line 377 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_test_message ("%s", _tmp11_);
#line 378 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_assert_not_reached ();
#line 366 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_error_free0 (e);
#line 5135 "TDocumentTest.c"
	}
	__finally132:
#line 366 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 366 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (doc);
#line 366 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 366 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 366 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		return;
#line 5148 "TDocumentTest.c"
	}
#line 364 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (doc);
#line 5152 "TDocumentTest.c"
}


static void ___lambda114__gtest_func (void) {
#line 364 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda114_ ();
#line 5159 "TDocumentTest.c"
}


static void __lambda115_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlTDocument* doc = NULL;
		GXmlTDocument* _tmp0_ = NULL;
		GXmlNode* _tmp1_ = NULL;
		GXmlTDocument* _tmp2_ = NULL;
		GXmlNode* _tmp3_ = NULL;
		GXmlTDocument* _tmp4_ = NULL;
		GeeBidirList* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GeeBidirList* _tmp7_ = NULL;
		GXmlTDocument* _tmp8_ = NULL;
		GXmlTDocument* _tmp9_ = NULL;
		gchar* _tmp10_ = NULL;
		gchar* _tmp11_ = NULL;
		gchar* _tmp12_ = NULL;
		gchar* _tmp13_ = NULL;
		GXmlTDocument* _tmp14_ = NULL;
		GXmlNode* _tmp15_ = NULL;
		GXmlNode* _tmp16_ = NULL;
		GXmlNode* _tmp17_ = NULL;
		GXmlTDocument* _tmp18_ = NULL;
		GeeList* _tmp19_ = NULL;
		GeeList* _tmp20_ = NULL;
		GeeList* _tmp21_ = NULL;
		GXmlTDocument* _tmp22_ = NULL;
		GeeList* _tmp23_ = NULL;
		GeeList* _tmp24_ = NULL;
		GeeList* _tmp25_ = NULL;
		gint _tmp26_ = 0;
		gint _tmp27_ = 0;
		GXmlTDocument* _tmp28_ = NULL;
		GeeList* _tmp29_ = NULL;
		GeeList* _tmp30_ = NULL;
		GeeList* _tmp31_ = NULL;
		gpointer _tmp32_ = NULL;
		GXmlNamespace* _tmp33_ = NULL;
		gchar* _tmp34_ = NULL;
		gchar* _tmp35_ = NULL;
		gchar* _tmp36_ = NULL;
		GXmlTDocument* _tmp37_ = NULL;
		GeeList* _tmp38_ = NULL;
		GeeList* _tmp39_ = NULL;
		GeeList* _tmp40_ = NULL;
		gpointer _tmp41_ = NULL;
		GXmlNamespace* _tmp42_ = NULL;
		gchar* _tmp43_ = NULL;
		gchar* _tmp44_ = NULL;
		gchar* _tmp45_ = NULL;
		GXmlNode* _tmp46_ = NULL;
		GXmlTDocument* _tmp47_ = NULL;
		GXmlNode* _tmp48_ = NULL;
		GXmlTDocument* _tmp49_ = NULL;
		GXmlNode* _tmp50_ = NULL;
		GXmlNode* _tmp51_ = NULL;
		GXmlNode* _tmp52_ = NULL;
		GeeBidirList* _tmp53_ = NULL;
		GeeBidirList* _tmp54_ = NULL;
		GeeBidirList* _tmp55_ = NULL;
		GXmlTDocument* _tmp56_ = NULL;
		GXmlNode* _tmp57_ = NULL;
		GXmlNode* _tmp58_ = NULL;
		GXmlNode* _tmp59_ = NULL;
		GeeBidirList* _tmp60_ = NULL;
		GeeBidirList* _tmp61_ = NULL;
		GeeBidirList* _tmp62_ = NULL;
		GXmlTDocument* _tmp63_ = NULL;
		GXmlNode* _tmp64_ = NULL;
		GXmlNode* _tmp65_ = NULL;
		GXmlNode* _tmp66_ = NULL;
		GeeBidirList* _tmp67_ = NULL;
		GeeBidirList* _tmp68_ = NULL;
		GeeBidirList* _tmp69_ = NULL;
		gint _tmp70_ = 0;
		gint _tmp71_ = 0;
		GXmlNode* c = NULL;
		GXmlTDocument* _tmp72_ = NULL;
		GXmlNode* _tmp73_ = NULL;
		GXmlNode* _tmp74_ = NULL;
		GXmlNode* _tmp75_ = NULL;
		GeeBidirList* _tmp76_ = NULL;
		GeeBidirList* _tmp77_ = NULL;
		GeeBidirList* _tmp78_ = NULL;
		gpointer _tmp79_ = NULL;
		GXmlNode* _tmp80_ = NULL;
		GeeList* _tmp81_ = NULL;
		GeeList* _tmp82_ = NULL;
		GeeList* _tmp83_ = NULL;
		GeeList* _tmp84_ = NULL;
		GeeList* _tmp85_ = NULL;
		GeeList* _tmp86_ = NULL;
		gint _tmp87_ = 0;
		gint _tmp88_ = 0;
		GeeList* _tmp89_ = NULL;
		GeeList* _tmp90_ = NULL;
		GeeList* _tmp91_ = NULL;
		gpointer _tmp92_ = NULL;
		GXmlNamespace* _tmp93_ = NULL;
		gchar* _tmp94_ = NULL;
		gchar* _tmp95_ = NULL;
		gchar* _tmp96_ = NULL;
		GeeList* _tmp97_ = NULL;
		GeeList* _tmp98_ = NULL;
		GeeList* _tmp99_ = NULL;
		gpointer _tmp100_ = NULL;
		GXmlNamespace* _tmp101_ = NULL;
		gchar* _tmp102_ = NULL;
		gchar* _tmp103_ = NULL;
		gchar* _tmp104_ = NULL;
		GXmlNode* p = NULL;
		GXmlNode* _tmp105_ = NULL;
		GXmlTDocument* _tmp106_ = NULL;
		gchar* _tmp107_ = NULL;
		gchar* _tmp108_ = NULL;
		gchar* _tmp109_ = NULL;
		gchar* _tmp110_ = NULL;
		gchar** str = NULL;
		GXmlTDocument* _tmp111_ = NULL;
		gchar* _tmp112_ = NULL;
		gchar* _tmp113_ = NULL;
		gchar** _tmp114_ = NULL;
		gchar** _tmp115_ = NULL;
		gchar** _tmp116_ = NULL;
		gint _tmp116__length1 = 0;
		gint str_length1 = 0;
		gint _str_size_ = 0;
		gchar** _tmp117_ = NULL;
		gint _tmp117__length1 = 0;
		const gchar* _tmp118_ = NULL;
		GXmlTDocument* _tmp119_ = NULL;
		GeeList* _tmp120_ = NULL;
		GeeList* _tmp121_ = NULL;
		GeeList* _tmp122_ = NULL;
		gpointer _tmp123_ = NULL;
		GXmlNamespace* _tmp124_ = NULL;
		gchar* _tmp125_ = NULL;
		gchar* _tmp126_ = NULL;
		gchar* _tmp127_ = NULL;
		GXmlTDocument* _tmp128_ = NULL;
		GeeList* _tmp129_ = NULL;
		GeeList* _tmp130_ = NULL;
		GeeList* _tmp131_ = NULL;
		gpointer _tmp132_ = NULL;
		GXmlNamespace* _tmp133_ = NULL;
		gchar* _tmp134_ = NULL;
		gchar* _tmp135_ = NULL;
		gchar* _tmp136_ = NULL;
		gchar* _tmp137_ = NULL;
		gchar* _tmp138_ = NULL;
		GXmlTDocument* _tmp139_ = NULL;
		GeeList* _tmp140_ = NULL;
		GeeList* _tmp141_ = NULL;
		GeeList* _tmp142_ = NULL;
		gpointer _tmp143_ = NULL;
		GXmlNamespace* _tmp144_ = NULL;
		gchar* _tmp145_ = NULL;
		gchar* _tmp146_ = NULL;
		gchar* _tmp147_ = NULL;
		gchar* _tmp148_ = NULL;
		gchar* _tmp149_ = NULL;
		GXmlTDocument* _tmp150_ = NULL;
		GXmlNode* _tmp151_ = NULL;
		GXmlNode* _tmp152_ = NULL;
		GXmlNode* _tmp153_ = NULL;
		gchar* _tmp154_ = NULL;
		gchar* _tmp155_ = NULL;
		gchar* _tmp156_ = NULL;
		gchar* _tmp157_ = NULL;
		GeeMap* _tmp158_ = NULL;
		GeeMap* _tmp159_ = NULL;
		GeeMap* _tmp160_ = NULL;
		gint _tmp161_ = 0;
		gint _tmp162_ = 0;
		GXmlNode* pt = NULL;
		GeeMap* _tmp163_ = NULL;
		GeeMap* _tmp164_ = NULL;
		GeeMap* _tmp165_ = NULL;
		gpointer _tmp166_ = NULL;
		GXmlNode* _tmp167_ = NULL;
		GXmlTDocument* _tmp168_ = NULL;
		gchar* _tmp169_ = NULL;
		gchar* _tmp170_ = NULL;
		gchar** _tmp171_ = NULL;
		gchar** _tmp172_ = NULL;
		gchar** _tmp173_ = NULL;
		gint _tmp173__length1 = 0;
		const gchar* _tmp174_ = NULL;
#line 383 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp0_ = gxml_tdocument_new ();
#line 383 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		doc = _tmp0_;
#line 384 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp2_ = doc;
#line 384 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp3_ = gxml_document_create_element ((GXmlDocument*) _tmp2_, "root", &_inner_error_);
#line 384 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp1_ = _tmp3_;
#line 384 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 384 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (doc);
#line 5365 "TDocumentTest.c"
			goto __catch133_g_error;
		}
#line 384 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp4_ = doc;
#line 384 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp5_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp4_);
#line 384 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp6_ = _tmp5_;
#line 384 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp7_ = _tmp6_;
#line 384 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gee_collection_add ((GeeCollection*) _tmp7_, _tmp1_);
#line 384 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp7_);
#line 385 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp8_ = doc;
#line 385 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_tnode_set_namespace ((GXmlTNode*) _tmp8_, "http://www.gnome.org/GXml", "gxml");
#line 386 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp9_ = doc;
#line 386 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp10_ = gxml_tnode_to_string ((GXmlTNode*) _tmp9_);
#line 386 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp11_ = _tmp10_;
#line 386 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp12_ = g_strconcat ("ROOT: ", _tmp11_, NULL);
#line 386 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp13_ = _tmp12_;
#line 386 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_test_message ("%s", _tmp13_);
#line 386 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp13_);
#line 386 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp11_);
#line 387 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp14_ = doc;
#line 387 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp15_ = gxml_document_get_root ((GXmlDocument*) _tmp14_);
#line 387 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp16_ = _tmp15_;
#line 387 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp17_ = _tmp16_;
#line 387 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp17_ != NULL, "doc.root != null");
#line 387 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp17_);
#line 388 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp18_ = doc;
#line 388 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp19_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp18_);
#line 388 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp20_ = _tmp19_;
#line 388 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp21_ = _tmp20_;
#line 388 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp21_ != NULL, "doc.namespaces != null");
#line 388 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp21_);
#line 389 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp22_ = doc;
#line 389 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp23_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp22_);
#line 389 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp24_ = _tmp23_;
#line 389 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp25_ = _tmp24_;
#line 389 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp26_ = gee_collection_get_size ((GeeCollection*) _tmp25_);
#line 389 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp27_ = _tmp26_;
#line 389 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp27_ == 1, "doc.namespaces.size == 1");
#line 389 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp25_);
#line 390 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp28_ = doc;
#line 390 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp29_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp28_);
#line 390 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp30_ = _tmp29_;
#line 390 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp31_ = _tmp30_;
#line 390 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp32_ = gee_list_get (_tmp31_, 0);
#line 390 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp33_ = (GXmlNamespace*) _tmp32_;
#line 390 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp34_ = gxml_namespace_get_prefix (_tmp33_);
#line 390 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp35_ = _tmp34_;
#line 390 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp36_ = _tmp35_;
#line 390 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp36_, "gxml") == 0, "doc.namespaces[0].prefix == \"gxml\"");
#line 390 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp36_);
#line 390 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp33_);
#line 390 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp31_);
#line 391 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp37_ = doc;
#line 391 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp38_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp37_);
#line 391 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp39_ = _tmp38_;
#line 391 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp40_ = _tmp39_;
#line 391 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp41_ = gee_list_get (_tmp40_, 0);
#line 391 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp42_ = (GXmlNamespace*) _tmp41_;
#line 391 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp43_ = gxml_namespace_get_uri (_tmp42_);
#line 391 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp44_ = _tmp43_;
#line 391 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp45_ = _tmp44_;
#line 391 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp45_, "http://www.gnome.org/GXml") == 0, "doc.namespaces[0].uri == \"http://www.gnome.org/GXml\"");
#line 391 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp45_);
#line 391 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp42_);
#line 391 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp40_);
#line 392 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp47_ = doc;
#line 392 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp48_ = gxml_document_create_element ((GXmlDocument*) _tmp47_, "child", &_inner_error_);
#line 392 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp46_ = _tmp48_;
#line 392 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 392 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (_tmp1_);
#line 392 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (doc);
#line 5504 "TDocumentTest.c"
			goto __catch133_g_error;
		}
#line 392 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp49_ = doc;
#line 392 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp50_ = gxml_document_get_root ((GXmlDocument*) _tmp49_);
#line 392 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp51_ = _tmp50_;
#line 392 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp52_ = _tmp51_;
#line 392 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp53_ = gxml_node_get_children_nodes (_tmp52_);
#line 392 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp54_ = _tmp53_;
#line 392 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp55_ = _tmp54_;
#line 392 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gee_collection_add ((GeeCollection*) _tmp55_, _tmp46_);
#line 392 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp55_);
#line 392 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp52_);
#line 393 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp56_ = doc;
#line 393 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp57_ = gxml_document_get_root ((GXmlDocument*) _tmp56_);
#line 393 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp58_ = _tmp57_;
#line 393 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp59_ = _tmp58_;
#line 393 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp60_ = gxml_node_get_children_nodes (_tmp59_);
#line 393 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp61_ = _tmp60_;
#line 393 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp62_ = _tmp61_;
#line 393 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp62_ != NULL, "doc.root.children_nodes != null");
#line 393 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp62_);
#line 393 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp59_);
#line 394 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp63_ = doc;
#line 394 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp64_ = gxml_document_get_root ((GXmlDocument*) _tmp63_);
#line 394 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp65_ = _tmp64_;
#line 394 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp66_ = _tmp65_;
#line 394 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp67_ = gxml_node_get_children_nodes (_tmp66_);
#line 394 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp68_ = _tmp67_;
#line 394 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp69_ = _tmp68_;
#line 394 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp70_ = gee_collection_get_size ((GeeCollection*) _tmp69_);
#line 394 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp71_ = _tmp70_;
#line 394 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp71_ == 1, "doc.root.children_nodes.size == 1");
#line 394 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp69_);
#line 394 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp66_);
#line 395 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp72_ = doc;
#line 395 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp73_ = gxml_document_get_root ((GXmlDocument*) _tmp72_);
#line 395 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp74_ = _tmp73_;
#line 395 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp75_ = _tmp74_;
#line 395 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp76_ = gxml_node_get_children_nodes (_tmp75_);
#line 395 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp77_ = _tmp76_;
#line 395 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp78_ = _tmp77_;
#line 395 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp79_ = gee_list_get ((GeeList*) _tmp78_, 0);
#line 395 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp80_ = (GXmlNode*) _tmp79_;
#line 395 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp78_);
#line 395 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp75_);
#line 395 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		c = _tmp80_;
#line 396 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_node_set_namespace (c, "http://www.gnome.org/GXml2", "gxml2");
#line 397 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp81_ = gxml_node_get_namespaces (c);
#line 397 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp82_ = _tmp81_;
#line 397 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp83_ = _tmp82_;
#line 397 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp83_ != NULL, "c.namespaces != null");
#line 397 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp83_);
#line 398 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp84_ = gxml_node_get_namespaces (c);
#line 398 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp85_ = _tmp84_;
#line 398 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp86_ = _tmp85_;
#line 398 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp87_ = gee_collection_get_size ((GeeCollection*) _tmp86_);
#line 398 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp88_ = _tmp87_;
#line 398 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp88_ == 1, "c.namespaces.size == 1");
#line 398 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp86_);
#line 399 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp89_ = gxml_node_get_namespaces (c);
#line 399 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp90_ = _tmp89_;
#line 399 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp91_ = _tmp90_;
#line 399 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp92_ = gee_list_get (_tmp91_, 0);
#line 399 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp93_ = (GXmlNamespace*) _tmp92_;
#line 399 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp94_ = gxml_namespace_get_prefix (_tmp93_);
#line 399 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp95_ = _tmp94_;
#line 399 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp96_ = _tmp95_;
#line 399 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp96_, "gxml2") == 0, "c.namespaces[0].prefix == \"gxml2\"");
#line 399 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp96_);
#line 399 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp93_);
#line 399 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp91_);
#line 400 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp97_ = gxml_node_get_namespaces (c);
#line 400 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp98_ = _tmp97_;
#line 400 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp99_ = _tmp98_;
#line 400 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp100_ = gee_list_get (_tmp99_, 0);
#line 400 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp101_ = (GXmlNamespace*) _tmp100_;
#line 400 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp102_ = gxml_namespace_get_uri (_tmp101_);
#line 400 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp103_ = _tmp102_;
#line 400 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp104_ = _tmp103_;
#line 400 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp104_, "http://www.gnome.org/GXml2") == 0, "c.namespaces[0].uri == \"http://www.gnome.org/GXml2\"");
#line 400 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp104_);
#line 400 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp101_);
#line 400 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp99_);
#line 401 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_element_set_attr (G_TYPE_CHECK_INSTANCE_TYPE (c, GXML_TYPE_ELEMENT) ? ((GXmlElement*) c) : NULL, "gxml:prop", "val");
#line 402 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp105_ = gxml_element_get_attr (G_TYPE_CHECK_INSTANCE_TYPE (c, GXML_TYPE_ELEMENT) ? ((GXmlElement*) c) : NULL, "gxml:prop");
#line 402 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		p = _tmp105_;
#line 403 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (p == NULL, "p == null");
#line 404 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp106_ = doc;
#line 404 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp107_ = gxml_tnode_to_string ((GXmlTNode*) _tmp106_);
#line 404 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp108_ = _tmp107_;
#line 404 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp109_ = g_strconcat ("ROOT: ", _tmp108_, NULL);
#line 404 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp110_ = _tmp109_;
#line 404 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_test_message ("%s", _tmp110_);
#line 404 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp110_);
#line 404 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp108_);
#line 405 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp111_ = doc;
#line 405 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp112_ = gxml_tnode_to_string ((GXmlTNode*) _tmp111_);
#line 405 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp113_ = _tmp112_;
#line 405 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp115_ = _tmp114_ = g_strsplit (_tmp113_, "\n", 0);
#line 405 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp116_ = _tmp115_;
#line 405 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp116__length1 = _vala_array_length (_tmp114_);
#line 405 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp113_);
#line 405 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		str = _tmp116_;
#line 405 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		str_length1 = _tmp116__length1;
#line 405 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_str_size_ = str_length1;
#line 406 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp117_ = str;
#line 406 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp117__length1 = str_length1;
#line 406 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp118_ = _tmp117_[1];
#line 406 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp118_, "<root xmlns:gxml=\"http://www.gnome.org/GXml\"><gxml2:child xmlns:gxml" \
"2=\"http://www.gnome.org/GXml2\"/></root>") == 0, "str[1] == \"<root xmlns:gxml=\\\"http://www.gnome.org/GXml\\\"><gxml2:child xmlns:gxml2=\\\"http://www.gnome.org/GXml2\\\"/></root>\"");
#line 407 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp119_ = doc;
#line 407 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp120_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp119_);
#line 407 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp121_ = _tmp120_;
#line 407 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp122_ = _tmp121_;
#line 407 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp123_ = gee_list_get (_tmp122_, 0);
#line 407 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp124_ = (GXmlNamespace*) _tmp123_;
#line 407 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp125_ = gxml_namespace_get_prefix (_tmp124_);
#line 407 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp126_ = _tmp125_;
#line 407 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp127_ = _tmp126_;
#line 407 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp127_, "gxml") == 0, "doc.namespaces[0].prefix == \"gxml\"");
#line 407 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp127_);
#line 407 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp124_);
#line 407 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp122_);
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp128_ = doc;
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp129_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp128_);
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp130_ = _tmp129_;
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp131_ = _tmp130_;
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp132_ = gee_list_get (_tmp131_, 0);
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp133_ = (GXmlNamespace*) _tmp132_;
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp134_ = gxml_namespace_get_prefix (_tmp133_);
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp135_ = _tmp134_;
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp136_ = _tmp135_;
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp137_ = g_strconcat (_tmp136_, ":", NULL);
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp138_ = _tmp137_;
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp139_ = doc;
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp140_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp139_);
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp141_ = _tmp140_;
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp142_ = _tmp141_;
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp143_ = gee_list_get (_tmp142_, 0);
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp144_ = (GXmlNamespace*) _tmp143_;
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp145_ = gxml_namespace_get_uri (_tmp144_);
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp146_ = _tmp145_;
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp147_ = _tmp146_;
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp148_ = g_strconcat (_tmp138_, _tmp147_, NULL);
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp149_ = _tmp148_;
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_element_set_ns_attr (G_TYPE_CHECK_INSTANCE_TYPE (c, GXML_TYPE_ELEMENT) ? ((GXmlElement*) c) : NULL, _tmp149_, "prop", "Ten");
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp149_);
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp147_);
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp144_);
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp142_);
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp138_);
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp136_);
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp133_);
#line 408 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp131_);
#line 409 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp150_ = doc;
#line 409 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp151_ = gxml_document_get_root ((GXmlDocument*) _tmp150_);
#line 409 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp152_ = _tmp151_;
#line 409 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp153_ = _tmp152_;
#line 409 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp154_ = gxml_node_to_string (_tmp153_);
#line 409 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp155_ = _tmp154_;
#line 409 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp156_ = g_strconcat ("ROOT: ", _tmp155_, NULL);
#line 409 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp157_ = _tmp156_;
#line 409 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_test_message ("%s", _tmp157_);
#line 409 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp157_);
#line 409 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp155_);
#line 409 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp153_);
#line 410 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp158_ = gxml_node_get_attrs (c);
#line 410 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp159_ = _tmp158_;
#line 410 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp160_ = _tmp159_;
#line 410 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp161_ = gee_map_get_size (_tmp160_);
#line 410 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp162_ = _tmp161_;
#line 410 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp162_ == 1, "c.attrs.size == 1");
#line 410 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp160_);
#line 411 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp163_ = gxml_node_get_attrs (c);
#line 411 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp164_ = _tmp163_;
#line 411 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp165_ = _tmp164_;
#line 411 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp166_ = gee_map_get (_tmp165_, "gxml:prop");
#line 411 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp167_ = (GXmlNode*) _tmp166_;
#line 411 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp165_);
#line 411 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		pt = _tmp167_;
#line 412 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (pt != NULL, "pt != null");
#line 413 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp168_ = doc;
#line 413 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp169_ = gxml_tnode_to_string ((GXmlTNode*) _tmp168_);
#line 413 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp170_ = _tmp169_;
#line 413 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp172_ = _tmp171_ = g_strsplit (_tmp170_, "\n", 0);
#line 413 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		str = (_vala_array_free (str, str_length1, (GDestroyNotify) g_free), NULL);
#line 413 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		str = _tmp172_;
#line 413 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		str_length1 = _vala_array_length (_tmp171_);
#line 413 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_str_size_ = str_length1;
#line 413 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp170_);
#line 414 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp173_ = str;
#line 414 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp173__length1 = str_length1;
#line 414 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp174_ = _tmp173_[1];
#line 414 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp174_, "<root xmlns:gxml=\"http://www.gnome.org/GXml\"><gxml2:child xmlns:gxml" \
"2=\"http://www.gnome.org/GXml2\" gxml:prop=\"Ten\"/></root>") == 0, "str[1] == \"<root xmlns:gxml=\\\"http://www.gnome.org/GXml\\\"><gxml2:child xmlns:gxml2=\\\"http://www.gnome.org/GXml2\\\" gxml:prop=\\\"Ten\\\"/></root>\"");
#line 382 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (pt);
#line 382 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		str = (_vala_array_free (str, str_length1, (GDestroyNotify) g_free), NULL);
#line 382 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (p);
#line 382 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (c);
#line 382 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp46_);
#line 382 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp1_);
#line 382 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (doc);
#line 5903 "TDocumentTest.c"
	}
	goto __finally133;
	__catch133_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp175_ = NULL;
		gchar* _tmp176_ = NULL;
		gchar* _tmp177_ = NULL;
#line 382 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _inner_error_;
#line 382 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_inner_error_ = NULL;
#line 416 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp175_ = e->message;
#line 416 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp176_ = g_strconcat ("ERROR: ", _tmp175_, NULL);
#line 416 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp177_ = _tmp176_;
#line 416 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_message ("TDocumentTest.vala:416: %s", _tmp177_);
#line 416 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp177_);
#line 417 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_assert_not_reached ();
#line 382 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_error_free0 (e);
#line 5930 "TDocumentTest.c"
	}
	__finally133:
#line 382 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 382 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 382 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 382 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		return;
#line 5941 "TDocumentTest.c"
	}
}


static void ___lambda115__gtest_func (void) {
#line 381 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda115_ ();
#line 5949 "TDocumentTest.c"
}


static void __lambda116_ (void) {
	GXmlTDocument* doc = NULL;
	GXmlTDocument* _tmp0_ = NULL;
	GXmlNode* _tmp1_ = NULL;
	GXmlNode* _tmp2_ = NULL;
	GXmlNode* _tmp3_ = NULL;
#line 421 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp0_ = gxml_tdocument_new ();
#line 421 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	doc = _tmp0_;
#line 422 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp1_ = gxml_tnode_get_parent ((GXmlTNode*) doc);
#line 422 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp2_ = _tmp1_;
#line 422 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp3_ = _tmp2_;
#line 422 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp3_ == NULL, "doc.parent == null");
#line 422 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp3_);
#line 420 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (doc);
#line 5975 "TDocumentTest.c"
}


static void ___lambda116__gtest_func (void) {
#line 420 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda116_ ();
#line 5982 "TDocumentTest.c"
}


static void __lambda117_ (void) {
	GError * _inner_error_ = NULL;
	{
		GFile* f = NULL;
		GFile* _tmp0_ = NULL;
		gboolean _tmp1_ = FALSE;
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp2_ = NULL;
		GXmlNode* _tmp3_ = NULL;
		GXmlNode* _tmp4_ = NULL;
		GXmlNode* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlNode* _tmp8_ = NULL;
		gchar* _tmp9_ = NULL;
		gchar* _tmp10_ = NULL;
		gchar* _tmp11_ = NULL;
		GXmlNode* _tmp12_ = NULL;
		GXmlNode* _tmp13_ = NULL;
		GXmlNode* _tmp14_ = NULL;
		GeeMap* _tmp15_ = NULL;
		GeeMap* _tmp16_ = NULL;
		GeeMap* _tmp17_ = NULL;
		gpointer _tmp18_ = NULL;
		GXmlNode* _tmp19_ = NULL;
		GXmlNode* _tmp20_ = NULL;
		GXmlNode* _tmp21_ = NULL;
		GXmlNode* _tmp22_ = NULL;
		GeeMap* _tmp23_ = NULL;
		GeeMap* _tmp24_ = NULL;
		GeeMap* _tmp25_ = NULL;
		gpointer _tmp26_ = NULL;
		GXmlNode* _tmp27_ = NULL;
		gchar* _tmp28_ = NULL;
		gchar* _tmp29_ = NULL;
		gchar* _tmp30_ = NULL;
		GXmlNode* _tmp31_ = NULL;
		GXmlNode* _tmp32_ = NULL;
		GXmlNode* _tmp33_ = NULL;
		GeeBidirList* _tmp34_ = NULL;
		GeeBidirList* _tmp35_ = NULL;
		GeeBidirList* _tmp36_ = NULL;
		gint _tmp37_ = 0;
		gint _tmp38_ = 0;
		GXmlNode* s1 = NULL;
		GXmlNode* _tmp39_ = NULL;
		GXmlNode* _tmp40_ = NULL;
		GXmlNode* _tmp41_ = NULL;
		GeeBidirList* _tmp42_ = NULL;
		GeeBidirList* _tmp43_ = NULL;
		GeeBidirList* _tmp44_ = NULL;
		gpointer _tmp45_ = NULL;
		GXmlNode* _tmp46_ = NULL;
		gchar* _tmp47_ = NULL;
		gchar* _tmp48_ = NULL;
		gchar* _tmp49_ = NULL;
		GXmlNode* p1 = NULL;
		GeeMap* _tmp50_ = NULL;
		GeeMap* _tmp51_ = NULL;
		GeeMap* _tmp52_ = NULL;
		gpointer _tmp53_ = NULL;
		GXmlNode* _tmp54_ = NULL;
		gchar* _tmp55_ = NULL;
		gchar* _tmp56_ = NULL;
		gchar* _tmp57_ = NULL;
		GeeBidirList* _tmp58_ = NULL;
		GeeBidirList* _tmp59_ = NULL;
		GeeBidirList* _tmp60_ = NULL;
		gint _tmp61_ = 0;
		gint _tmp62_ = 0;
		GeeBidirList* _tmp63_ = NULL;
		GeeBidirList* _tmp64_ = NULL;
		GeeBidirList* _tmp65_ = NULL;
		gpointer _tmp66_ = NULL;
		GXmlNode* _tmp67_ = NULL;
		GeeBidirList* _tmp68_ = NULL;
		GeeBidirList* _tmp69_ = NULL;
		GeeBidirList* _tmp70_ = NULL;
		gpointer _tmp71_ = NULL;
		GXmlNode* _tmp72_ = NULL;
		gchar* _tmp73_ = NULL;
		gchar* _tmp74_ = NULL;
		gchar* _tmp75_ = NULL;
		GXmlNode* s2 = NULL;
		GXmlNode* _tmp76_ = NULL;
		GXmlNode* _tmp77_ = NULL;
		GXmlNode* _tmp78_ = NULL;
		GeeBidirList* _tmp79_ = NULL;
		GeeBidirList* _tmp80_ = NULL;
		GeeBidirList* _tmp81_ = NULL;
		gpointer _tmp82_ = NULL;
		GXmlNode* _tmp83_ = NULL;
		gchar* _tmp84_ = NULL;
		gchar* _tmp85_ = NULL;
		gchar* _tmp86_ = NULL;
		GXmlNode* p2 = NULL;
		GeeMap* _tmp87_ = NULL;
		GeeMap* _tmp88_ = NULL;
		GeeMap* _tmp89_ = NULL;
		gpointer _tmp90_ = NULL;
		GXmlNode* _tmp91_ = NULL;
		gchar* _tmp92_ = NULL;
		gchar* _tmp93_ = NULL;
		gchar* _tmp94_ = NULL;
		GeeBidirList* _tmp95_ = NULL;
		GeeBidirList* _tmp96_ = NULL;
		GeeBidirList* _tmp97_ = NULL;
		gint _tmp98_ = 0;
		gint _tmp99_ = 0;
		GeeBidirList* _tmp100_ = NULL;
		GeeBidirList* _tmp101_ = NULL;
		GeeBidirList* _tmp102_ = NULL;
		gpointer _tmp103_ = NULL;
		GXmlNode* _tmp104_ = NULL;
		GeeBidirList* _tmp105_ = NULL;
		GeeBidirList* _tmp106_ = NULL;
		GeeBidirList* _tmp107_ = NULL;
		gpointer _tmp108_ = NULL;
		GXmlNode* _tmp109_ = NULL;
		gchar* _tmp110_ = NULL;
		gchar* _tmp111_ = NULL;
		gchar* _tmp112_ = NULL;
		GXmlNode* s3 = NULL;
		GXmlNode* _tmp113_ = NULL;
		GXmlNode* _tmp114_ = NULL;
		GXmlNode* _tmp115_ = NULL;
		GeeBidirList* _tmp116_ = NULL;
		GeeBidirList* _tmp117_ = NULL;
		GeeBidirList* _tmp118_ = NULL;
		gpointer _tmp119_ = NULL;
		GXmlNode* _tmp120_ = NULL;
		gchar* _tmp121_ = NULL;
		gchar* _tmp122_ = NULL;
		gchar* _tmp123_ = NULL;
		GXmlNode* p3 = NULL;
		GeeMap* _tmp124_ = NULL;
		GeeMap* _tmp125_ = NULL;
		GeeMap* _tmp126_ = NULL;
		gpointer _tmp127_ = NULL;
		GXmlNode* _tmp128_ = NULL;
		gchar* _tmp129_ = NULL;
		gchar* _tmp130_ = NULL;
		gchar* _tmp131_ = NULL;
		GXmlNode* p31 = NULL;
		GeeMap* _tmp132_ = NULL;
		GeeMap* _tmp133_ = NULL;
		GeeMap* _tmp134_ = NULL;
		gpointer _tmp135_ = NULL;
		GXmlNode* _tmp136_ = NULL;
		gchar* _tmp137_ = NULL;
		gchar* _tmp138_ = NULL;
		gchar* _tmp139_ = NULL;
		GeeBidirList* _tmp140_ = NULL;
		GeeBidirList* _tmp141_ = NULL;
		GeeBidirList* _tmp142_ = NULL;
		gint _tmp143_ = 0;
		gint _tmp144_ = 0;
		GeeBidirList* _tmp145_ = NULL;
		GeeBidirList* _tmp146_ = NULL;
		GeeBidirList* _tmp147_ = NULL;
		gpointer _tmp148_ = NULL;
		GXmlNode* _tmp149_ = NULL;
		GeeBidirList* _tmp150_ = NULL;
		GeeBidirList* _tmp151_ = NULL;
		GeeBidirList* _tmp152_ = NULL;
		gpointer _tmp153_ = NULL;
		GXmlNode* _tmp154_ = NULL;
		gchar* _tmp155_ = NULL;
		gchar* _tmp156_ = NULL;
		gchar* _tmp157_ = NULL;
		GeeBidirList* _tmp158_ = NULL;
		GeeBidirList* _tmp159_ = NULL;
		GeeBidirList* _tmp160_ = NULL;
		gpointer _tmp161_ = NULL;
		GXmlNode* _tmp162_ = NULL;
		gchar* _tmp163_ = NULL;
		gchar* _tmp164_ = NULL;
		gchar* _tmp165_ = NULL;
		GXmlNode* a1 = NULL;
		GeeBidirList* _tmp166_ = NULL;
		GeeBidirList* _tmp167_ = NULL;
		GeeBidirList* _tmp168_ = NULL;
		gpointer _tmp169_ = NULL;
		GXmlNode* _tmp170_ = NULL;
		gchar* _tmp171_ = NULL;
		gchar* _tmp172_ = NULL;
		gchar* _tmp173_ = NULL;
		GeeBidirList* _tmp174_ = NULL;
		GeeBidirList* _tmp175_ = NULL;
		GeeBidirList* _tmp176_ = NULL;
		gint _tmp177_ = 0;
		gint _tmp178_ = 0;
		GeeBidirList* _tmp179_ = NULL;
		GeeBidirList* _tmp180_ = NULL;
		GeeBidirList* _tmp181_ = NULL;
		gpointer _tmp182_ = NULL;
		GXmlNode* _tmp183_ = NULL;
		gchar* _tmp184_ = NULL;
		gchar* _tmp185_ = NULL;
		gchar* _tmp186_ = NULL;
		GeeBidirList* _tmp187_ = NULL;
		GeeBidirList* _tmp188_ = NULL;
		GeeBidirList* _tmp189_ = NULL;
		gpointer _tmp190_ = NULL;
		GXmlNode* _tmp191_ = NULL;
		GeeBidirList* _tmp192_ = NULL;
		GeeBidirList* _tmp193_ = NULL;
		GeeBidirList* _tmp194_ = NULL;
		gint _tmp195_ = 0;
		gint _tmp196_ = 0;
		GeeBidirList* _tmp197_ = NULL;
		GeeBidirList* _tmp198_ = NULL;
		GeeBidirList* _tmp199_ = NULL;
		gpointer _tmp200_ = NULL;
		GXmlNode* _tmp201_ = NULL;
		GeeBidirList* _tmp202_ = NULL;
		GeeBidirList* _tmp203_ = NULL;
		GeeBidirList* _tmp204_ = NULL;
		gpointer _tmp205_ = NULL;
		GXmlNode* _tmp206_ = NULL;
		GeeBidirList* _tmp207_ = NULL;
		GeeBidirList* _tmp208_ = NULL;
		GeeBidirList* _tmp209_ = NULL;
		gpointer _tmp210_ = NULL;
		GXmlNode* _tmp211_ = NULL;
		GeeBidirList* _tmp212_ = NULL;
		GeeBidirList* _tmp213_ = NULL;
		GeeBidirList* _tmp214_ = NULL;
		gpointer _tmp215_ = NULL;
		GXmlNode* _tmp216_ = NULL;
		gchar* _tmp217_ = NULL;
		gchar* _tmp218_ = NULL;
		gchar* _tmp219_ = NULL;
		GeeBidirList* _tmp220_ = NULL;
		GeeBidirList* _tmp221_ = NULL;
		GeeBidirList* _tmp222_ = NULL;
		gpointer _tmp223_ = NULL;
		GXmlNode* _tmp224_ = NULL;
		gchar* _tmp225_ = NULL;
		gchar* _tmp226_ = NULL;
		gchar* _tmp227_ = NULL;
		GeeBidirList* _tmp228_ = NULL;
		GeeBidirList* _tmp229_ = NULL;
		GeeBidirList* _tmp230_ = NULL;
		gpointer _tmp231_ = NULL;
		GXmlNode* _tmp232_ = NULL;
		GeeBidirList* _tmp233_ = NULL;
		GeeBidirList* _tmp234_ = NULL;
		GeeBidirList* _tmp235_ = NULL;
		gint _tmp236_ = 0;
		gint _tmp237_ = 0;
		GeeBidirList* _tmp238_ = NULL;
		GeeBidirList* _tmp239_ = NULL;
		GeeBidirList* _tmp240_ = NULL;
		gpointer _tmp241_ = NULL;
		GXmlNode* _tmp242_ = NULL;
		GeeBidirList* _tmp243_ = NULL;
		GeeBidirList* _tmp244_ = NULL;
		GeeBidirList* _tmp245_ = NULL;
		gpointer _tmp246_ = NULL;
		GXmlNode* _tmp247_ = NULL;
		GeeBidirList* _tmp248_ = NULL;
		GeeBidirList* _tmp249_ = NULL;
		GeeBidirList* _tmp250_ = NULL;
		gpointer _tmp251_ = NULL;
		GXmlNode* _tmp252_ = NULL;
		GeeBidirList* _tmp253_ = NULL;
		GeeBidirList* _tmp254_ = NULL;
		GeeBidirList* _tmp255_ = NULL;
		gpointer _tmp256_ = NULL;
		GXmlNode* _tmp257_ = NULL;
		gchar* _tmp258_ = NULL;
		gchar* _tmp259_ = NULL;
		gchar* _tmp260_ = NULL;
		GXmlNode* a2 = NULL;
		GeeBidirList* _tmp261_ = NULL;
		GeeBidirList* _tmp262_ = NULL;
		GeeBidirList* _tmp263_ = NULL;
		gpointer _tmp264_ = NULL;
		GXmlNode* _tmp265_ = NULL;
		GeeBidirList* _tmp266_ = NULL;
		GeeBidirList* _tmp267_ = NULL;
		GeeBidirList* _tmp268_ = NULL;
		gint _tmp269_ = 0;
		gint _tmp270_ = 0;
		GeeBidirList* _tmp271_ = NULL;
		GeeBidirList* _tmp272_ = NULL;
		GeeBidirList* _tmp273_ = NULL;
		gpointer _tmp274_ = NULL;
		GXmlNode* _tmp275_ = NULL;
		gchar* _tmp276_ = NULL;
		gchar* _tmp277_ = NULL;
		gchar* _tmp278_ = NULL;
		GeeBidirList* _tmp279_ = NULL;
		GeeBidirList* _tmp280_ = NULL;
		GeeBidirList* _tmp281_ = NULL;
		gpointer _tmp282_ = NULL;
		GXmlNode* _tmp283_ = NULL;
		GeeBidirList* _tmp284_ = NULL;
		GeeBidirList* _tmp285_ = NULL;
		GeeBidirList* _tmp286_ = NULL;
		gint _tmp287_ = 0;
		gint _tmp288_ = 0;
		GeeBidirList* _tmp289_ = NULL;
		GeeBidirList* _tmp290_ = NULL;
		GeeBidirList* _tmp291_ = NULL;
		gpointer _tmp292_ = NULL;
		GXmlNode* _tmp293_ = NULL;
		GeeBidirList* _tmp294_ = NULL;
		GeeBidirList* _tmp295_ = NULL;
		GeeBidirList* _tmp296_ = NULL;
		gpointer _tmp297_ = NULL;
		GXmlNode* _tmp298_ = NULL;
		GeeBidirList* _tmp299_ = NULL;
		GeeBidirList* _tmp300_ = NULL;
		GeeBidirList* _tmp301_ = NULL;
		gpointer _tmp302_ = NULL;
		GXmlNode* _tmp303_ = NULL;
		GeeBidirList* _tmp304_ = NULL;
		GeeBidirList* _tmp305_ = NULL;
		GeeBidirList* _tmp306_ = NULL;
		gpointer _tmp307_ = NULL;
		GXmlNode* _tmp308_ = NULL;
		gchar* _tmp309_ = NULL;
		gchar* _tmp310_ = NULL;
		gchar* _tmp311_ = NULL;
		GeeBidirList* _tmp312_ = NULL;
		GeeBidirList* _tmp313_ = NULL;
		GeeBidirList* _tmp314_ = NULL;
		gpointer _tmp315_ = NULL;
		GXmlNode* _tmp316_ = NULL;
		gchar* _tmp317_ = NULL;
		gchar* _tmp318_ = NULL;
		gchar* _tmp319_ = NULL;
		GeeBidirList* _tmp320_ = NULL;
		GeeBidirList* _tmp321_ = NULL;
		GeeBidirList* _tmp322_ = NULL;
		gpointer _tmp323_ = NULL;
		GXmlNode* _tmp324_ = NULL;
		GeeBidirList* _tmp325_ = NULL;
		GeeBidirList* _tmp326_ = NULL;
		GeeBidirList* _tmp327_ = NULL;
		gint _tmp328_ = 0;
		gint _tmp329_ = 0;
		GeeBidirList* _tmp330_ = NULL;
		GeeBidirList* _tmp331_ = NULL;
		GeeBidirList* _tmp332_ = NULL;
		gpointer _tmp333_ = NULL;
		GXmlNode* _tmp334_ = NULL;
		GeeBidirList* _tmp335_ = NULL;
		GeeBidirList* _tmp336_ = NULL;
		GeeBidirList* _tmp337_ = NULL;
		gpointer _tmp338_ = NULL;
		GXmlNode* _tmp339_ = NULL;
		GeeBidirList* _tmp340_ = NULL;
		GeeBidirList* _tmp341_ = NULL;
		GeeBidirList* _tmp342_ = NULL;
		gpointer _tmp343_ = NULL;
		GXmlNode* _tmp344_ = NULL;
		GeeBidirList* _tmp345_ = NULL;
		GeeBidirList* _tmp346_ = NULL;
		GeeBidirList* _tmp347_ = NULL;
		gpointer _tmp348_ = NULL;
		GXmlNode* _tmp349_ = NULL;
		gchar* _tmp350_ = NULL;
		gchar* _tmp351_ = NULL;
		gchar* _tmp352_ = NULL;
#line 426 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp0_ = g_file_new_for_path (TEST_DIR "/t-read-test.xml");
#line 426 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f = _tmp0_;
#line 427 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp1_ = g_file_query_exists (f, NULL);
#line 427 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp1_, "f.query_exists ()");
#line 428 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp2_ = gxml_tdocument_new ();
#line 428 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		d = _tmp2_;
#line 429 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_tdocument_read_doc ((GXmlDocument*) d, f, NULL, NULL, &_inner_error_);
#line 429 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 429 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 429 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 6373 "TDocumentTest.c"
			goto __catch134_g_error;
		}
#line 431 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp3_ = gxml_document_get_root ((GXmlDocument*) d);
#line 431 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp4_ = _tmp3_;
#line 431 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp5_ = _tmp4_;
#line 431 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp5_ != NULL, "d.root != null");
#line 431 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp5_);
#line 432 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp6_ = gxml_document_get_root ((GXmlDocument*) d);
#line 432 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp7_ = _tmp6_;
#line 432 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp8_ = _tmp7_;
#line 432 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp9_ = gxml_node_get_name (_tmp8_);
#line 432 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp10_ = _tmp9_;
#line 432 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp11_ = _tmp10_;
#line 432 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp11_, "Sentences") == 0, "d.root.name == \"Sentences\"");
#line 432 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp11_);
#line 432 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp8_);
#line 433 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp12_ = gxml_document_get_root ((GXmlDocument*) d);
#line 433 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp13_ = _tmp12_;
#line 433 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp14_ = _tmp13_;
#line 433 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp15_ = gxml_node_get_attrs (_tmp14_);
#line 433 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp16_ = _tmp15_;
#line 433 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp17_ = _tmp16_;
#line 433 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp18_ = gee_map_get (_tmp17_, "audience");
#line 433 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp19_ = (GXmlNode*) _tmp18_;
#line 433 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp19_ != NULL, "d.root.attrs[\"audience\"] != null");
#line 433 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp19_);
#line 433 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp17_);
#line 433 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp14_);
#line 434 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp20_ = gxml_document_get_root ((GXmlDocument*) d);
#line 434 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp21_ = _tmp20_;
#line 434 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp22_ = _tmp21_;
#line 434 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp23_ = gxml_node_get_attrs (_tmp22_);
#line 434 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp24_ = _tmp23_;
#line 434 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp25_ = _tmp24_;
#line 434 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp26_ = gee_map_get (_tmp25_, "audience");
#line 434 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp27_ = (GXmlNode*) _tmp26_;
#line 434 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp28_ = gxml_node_get_value (_tmp27_);
#line 434 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp29_ = _tmp28_;
#line 434 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp30_ = _tmp29_;
#line 434 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp30_, "All") == 0, "d.root.attrs[\"audience\"].value == \"All\"");
#line 434 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp30_);
#line 434 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp27_);
#line 434 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp25_);
#line 434 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp22_);
#line 435 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp31_ = gxml_document_get_root ((GXmlDocument*) d);
#line 435 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp32_ = _tmp31_;
#line 435 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp33_ = _tmp32_;
#line 435 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp34_ = gxml_node_get_children_nodes (_tmp33_);
#line 435 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp35_ = _tmp34_;
#line 435 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp36_ = _tmp35_;
#line 435 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp37_ = gee_collection_get_size ((GeeCollection*) _tmp36_);
#line 435 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp38_ = _tmp37_;
#line 435 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp38_ == 7, "d.root.children_nodes.size == 7");
#line 435 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp36_);
#line 435 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp33_);
#line 436 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp39_ = gxml_document_get_root ((GXmlDocument*) d);
#line 436 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp40_ = _tmp39_;
#line 436 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp41_ = _tmp40_;
#line 436 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp42_ = gxml_node_get_children_nodes (_tmp41_);
#line 436 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp43_ = _tmp42_;
#line 436 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp44_ = _tmp43_;
#line 436 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp45_ = gee_list_get ((GeeList*) _tmp44_, 0);
#line 436 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp46_ = (GXmlNode*) _tmp45_;
#line 436 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp44_);
#line 436 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp41_);
#line 436 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		s1 = _tmp46_;
#line 437 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (s1 != NULL, "s1 != null");
#line 438 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp47_ = gxml_node_get_name (s1);
#line 438 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp48_ = _tmp47_;
#line 438 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp49_ = _tmp48_;
#line 438 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp49_, "Sentence") == 0, "s1.name == \"Sentence\"");
#line 438 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp49_);
#line 439 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp50_ = gxml_node_get_attrs (s1);
#line 439 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp51_ = _tmp50_;
#line 439 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp52_ = _tmp51_;
#line 439 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp53_ = gee_map_get (_tmp52_, "lang");
#line 439 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp54_ = (GXmlNode*) _tmp53_;
#line 439 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp52_);
#line 439 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		p1 = _tmp54_;
#line 440 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (p1 != NULL, "p1 != null");
#line 441 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp55_ = gxml_node_get_value (p1);
#line 441 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp56_ = _tmp55_;
#line 441 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp57_ = _tmp56_;
#line 441 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp57_, "en") == 0, "p1.value == \"en\"");
#line 441 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp57_);
#line 442 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp58_ = gxml_node_get_children_nodes (s1);
#line 442 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp59_ = _tmp58_;
#line 442 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp60_ = _tmp59_;
#line 442 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp61_ = gee_collection_get_size ((GeeCollection*) _tmp60_);
#line 442 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp62_ = _tmp61_;
#line 442 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp62_ == 1, "s1.children_nodes.size == 1");
#line 442 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp60_);
#line 443 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp63_ = gxml_node_get_children_nodes (s1);
#line 443 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp64_ = _tmp63_;
#line 443 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp65_ = _tmp64_;
#line 443 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp66_ = gee_list_get ((GeeList*) _tmp65_, 0);
#line 443 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp67_ = (GXmlNode*) _tmp66_;
#line 443 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp67_, GXML_TYPE_TEXT), "s1.children_nodes[0] is GXml.Text");
#line 443 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp67_);
#line 443 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp65_);
#line 444 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp68_ = gxml_node_get_children_nodes (s1);
#line 444 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp69_ = _tmp68_;
#line 444 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp70_ = _tmp69_;
#line 444 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp71_ = gee_list_get ((GeeList*) _tmp70_, 0);
#line 444 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp72_ = (GXmlNode*) _tmp71_;
#line 444 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp73_ = gxml_node_get_value (_tmp72_);
#line 444 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp74_ = _tmp73_;
#line 444 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp75_ = _tmp74_;
#line 444 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp75_, "I like the colour blue.") == 0, "s1.children_nodes[0].value == \"I like the colour blue.\"");
#line 444 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp75_);
#line 444 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp72_);
#line 444 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp70_);
#line 445 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp76_ = gxml_document_get_root ((GXmlDocument*) d);
#line 445 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp77_ = _tmp76_;
#line 445 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp78_ = _tmp77_;
#line 445 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp79_ = gxml_node_get_children_nodes (_tmp78_);
#line 445 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp80_ = _tmp79_;
#line 445 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp81_ = _tmp80_;
#line 445 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp82_ = gee_list_get ((GeeList*) _tmp81_, 1);
#line 445 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp83_ = (GXmlNode*) _tmp82_;
#line 445 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp81_);
#line 445 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp78_);
#line 445 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		s2 = _tmp83_;
#line 446 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (s2 != NULL, "s2 != null");
#line 447 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp84_ = gxml_node_get_name (s2);
#line 447 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp85_ = _tmp84_;
#line 447 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp86_ = _tmp85_;
#line 447 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp86_, "Sentence") == 0, "s2.name == \"Sentence\"");
#line 447 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp86_);
#line 448 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp87_ = gxml_node_get_attrs (s2);
#line 448 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp88_ = _tmp87_;
#line 448 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp89_ = _tmp88_;
#line 448 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp90_ = gee_map_get (_tmp89_, "lang");
#line 448 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp91_ = (GXmlNode*) _tmp90_;
#line 448 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp89_);
#line 448 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		p2 = _tmp91_;
#line 449 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (p2 != NULL, "p2 != null");
#line 450 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp92_ = gxml_node_get_value (p2);
#line 450 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp93_ = _tmp92_;
#line 450 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp94_ = _tmp93_;
#line 450 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp94_, "es") == 0, "p2.value == \"es\"");
#line 450 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp94_);
#line 451 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp95_ = gxml_node_get_children_nodes (s2);
#line 451 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp96_ = _tmp95_;
#line 451 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp97_ = _tmp96_;
#line 451 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp98_ = gee_collection_get_size ((GeeCollection*) _tmp97_);
#line 451 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp99_ = _tmp98_;
#line 451 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp99_ == 1, "s2.children_nodes.size == 1");
#line 451 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp97_);
#line 452 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp100_ = gxml_node_get_children_nodes (s2);
#line 452 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp101_ = _tmp100_;
#line 452 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp102_ = _tmp101_;
#line 452 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp103_ = gee_list_get ((GeeList*) _tmp102_, 0);
#line 452 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp104_ = (GXmlNode*) _tmp103_;
#line 452 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp104_, GXML_TYPE_TEXT), "s2.children_nodes[0] is GXml.Text");
#line 452 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp104_);
#line 452 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp102_);
#line 453 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp105_ = gxml_node_get_children_nodes (s2);
#line 453 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp106_ = _tmp105_;
#line 453 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp107_ = _tmp106_;
#line 453 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp108_ = gee_list_get ((GeeList*) _tmp107_, 0);
#line 453 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp109_ = (GXmlNode*) _tmp108_;
#line 453 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp110_ = gxml_node_get_value (_tmp109_);
#line 453 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp111_ = _tmp110_;
#line 453 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp112_ = _tmp111_;
#line 453 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp112_, "Español") == 0, "s2.children_nodes[0].value == \"Espa\303\261ol\"");
#line 453 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp112_);
#line 453 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp109_);
#line 453 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp107_);
#line 454 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp113_ = gxml_document_get_root ((GXmlDocument*) d);
#line 454 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp114_ = _tmp113_;
#line 454 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp115_ = _tmp114_;
#line 454 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp116_ = gxml_node_get_children_nodes (_tmp115_);
#line 454 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp117_ = _tmp116_;
#line 454 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp118_ = _tmp117_;
#line 454 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp119_ = gee_list_get ((GeeList*) _tmp118_, 2);
#line 454 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp120_ = (GXmlNode*) _tmp119_;
#line 454 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp118_);
#line 454 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp115_);
#line 454 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		s3 = _tmp120_;
#line 455 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (s3 != NULL, "s3 != null");
#line 456 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp121_ = gxml_node_get_name (s3);
#line 456 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp122_ = _tmp121_;
#line 456 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp123_ = _tmp122_;
#line 456 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp123_, "Authors") == 0, "s3.name == \"Authors\"");
#line 456 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp123_);
#line 457 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp124_ = gxml_node_get_attrs (s3);
#line 457 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp125_ = _tmp124_;
#line 457 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp126_ = _tmp125_;
#line 457 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp127_ = gee_map_get (_tmp126_, "year");
#line 457 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp128_ = (GXmlNode*) _tmp127_;
#line 457 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp126_);
#line 457 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		p3 = _tmp128_;
#line 458 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (p3 != NULL, "p3 != null");
#line 459 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp129_ = gxml_node_get_value (p3);
#line 459 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp130_ = _tmp129_;
#line 459 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp131_ = _tmp130_;
#line 459 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp131_, "2016") == 0, "p3.value == \"2016\"");
#line 459 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp131_);
#line 460 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp132_ = gxml_node_get_attrs (s3);
#line 460 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp133_ = _tmp132_;
#line 460 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp134_ = _tmp133_;
#line 460 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp135_ = gee_map_get (_tmp134_, "collection");
#line 460 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp136_ = (GXmlNode*) _tmp135_;
#line 460 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp134_);
#line 460 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		p31 = _tmp136_;
#line 461 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (p31 != NULL, "p31 != null");
#line 462 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp137_ = gxml_node_get_value (p31);
#line 462 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp138_ = _tmp137_;
#line 462 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp139_ = _tmp138_;
#line 462 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp139_, "Back") == 0, "p31.value == \"Back\"");
#line 462 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp139_);
#line 463 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp140_ = gxml_node_get_children_nodes (s3);
#line 463 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp141_ = _tmp140_;
#line 463 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp142_ = _tmp141_;
#line 463 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp143_ = gee_collection_get_size ((GeeCollection*) _tmp142_);
#line 463 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp144_ = _tmp143_;
#line 463 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp144_ == 2, "s3.children_nodes.size == 2");
#line 463 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp142_);
#line 464 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp145_ = gxml_node_get_children_nodes (s3);
#line 464 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp146_ = _tmp145_;
#line 464 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp147_ = _tmp146_;
#line 464 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp148_ = gee_list_get ((GeeList*) _tmp147_, 0);
#line 464 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp149_ = (GXmlNode*) _tmp148_;
#line 464 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp149_, GXML_TYPE_ELEMENT), "s3.children_nodes[0] is GXml.Element");
#line 464 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp149_);
#line 464 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp147_);
#line 465 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp150_ = gxml_node_get_children_nodes (s3);
#line 465 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp151_ = _tmp150_;
#line 465 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp152_ = _tmp151_;
#line 465 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp153_ = gee_list_get ((GeeList*) _tmp152_, 0);
#line 465 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp154_ = (GXmlNode*) _tmp153_;
#line 465 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp155_ = gxml_node_get_name (_tmp154_);
#line 465 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp156_ = _tmp155_;
#line 465 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp157_ = _tmp156_;
#line 465 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp157_, "Author") == 0, "s3.children_nodes[0].name == \"Author\"");
#line 465 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp157_);
#line 465 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp154_);
#line 465 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp152_);
#line 466 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp158_ = gxml_node_get_children_nodes (s3);
#line 466 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp159_ = _tmp158_;
#line 466 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp160_ = _tmp159_;
#line 466 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp161_ = gee_list_get ((GeeList*) _tmp160_, 1);
#line 466 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp162_ = (GXmlNode*) _tmp161_;
#line 466 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp163_ = gxml_node_get_name (_tmp162_);
#line 466 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp164_ = _tmp163_;
#line 466 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp165_ = _tmp164_;
#line 466 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp165_, "Author") == 0, "s3.children_nodes[1].name == \"Author\"");
#line 466 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp165_);
#line 466 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp162_);
#line 466 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp160_);
#line 467 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp166_ = gxml_node_get_children_nodes (s3);
#line 467 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp167_ = _tmp166_;
#line 467 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp168_ = _tmp167_;
#line 467 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp169_ = gee_list_get ((GeeList*) _tmp168_, 0);
#line 467 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp170_ = (GXmlNode*) _tmp169_;
#line 467 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp168_);
#line 467 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		a1 = _tmp170_;
#line 468 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (a1 != NULL, "a1 != null");
#line 469 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp171_ = gxml_node_get_name (a1);
#line 469 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp172_ = _tmp171_;
#line 469 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp173_ = _tmp172_;
#line 469 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp173_, "Author") == 0, "a1.name == \"Author\"");
#line 469 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp173_);
#line 470 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp174_ = gxml_node_get_children_nodes (a1);
#line 470 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp175_ = _tmp174_;
#line 470 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp176_ = _tmp175_;
#line 470 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp177_ = gee_collection_get_size ((GeeCollection*) _tmp176_);
#line 470 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp178_ = _tmp177_;
#line 470 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp178_ == 2, "a1.children_nodes.size == 2");
#line 470 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp176_);
#line 471 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp179_ = gxml_node_get_children_nodes (a1);
#line 471 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp180_ = _tmp179_;
#line 471 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp181_ = _tmp180_;
#line 471 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp182_ = gee_list_get ((GeeList*) _tmp181_, 0);
#line 471 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp183_ = (GXmlNode*) _tmp182_;
#line 471 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp184_ = gxml_node_get_name (_tmp183_);
#line 471 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp185_ = _tmp184_;
#line 471 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp186_ = _tmp185_;
#line 471 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp186_, "Name") == 0, "a1.children_nodes[0].name == \"Name\"");
#line 471 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp186_);
#line 471 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp183_);
#line 471 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp181_);
#line 472 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp187_ = gxml_node_get_children_nodes (a1);
#line 472 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp188_ = _tmp187_;
#line 472 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp189_ = _tmp188_;
#line 472 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp190_ = gee_list_get ((GeeList*) _tmp189_, 0);
#line 472 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp191_ = (GXmlNode*) _tmp190_;
#line 472 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp192_ = gxml_node_get_children_nodes (_tmp191_);
#line 472 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp193_ = _tmp192_;
#line 472 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp194_ = _tmp193_;
#line 472 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp195_ = gee_collection_get_size ((GeeCollection*) _tmp194_);
#line 472 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp196_ = _tmp195_;
#line 472 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp196_ == 1, "a1.children_nodes[0].children_nodes.size == 1");
#line 472 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp194_);
#line 472 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp191_);
#line 472 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp189_);
#line 473 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp197_ = gxml_node_get_children_nodes (a1);
#line 473 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp198_ = _tmp197_;
#line 473 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp199_ = _tmp198_;
#line 473 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp200_ = gee_list_get ((GeeList*) _tmp199_, 0);
#line 473 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp201_ = (GXmlNode*) _tmp200_;
#line 473 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp202_ = gxml_node_get_children_nodes (_tmp201_);
#line 473 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp203_ = _tmp202_;
#line 473 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp204_ = _tmp203_;
#line 473 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp205_ = gee_list_get ((GeeList*) _tmp204_, 0);
#line 473 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp206_ = (GXmlNode*) _tmp205_;
#line 473 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp206_, GXML_TYPE_TEXT), "a1.children_nodes[0].children_nodes[0] is GXml.Text");
#line 473 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp206_);
#line 473 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp204_);
#line 473 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp201_);
#line 473 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp199_);
#line 474 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp207_ = gxml_node_get_children_nodes (a1);
#line 474 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp208_ = _tmp207_;
#line 474 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp209_ = _tmp208_;
#line 474 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp210_ = gee_list_get ((GeeList*) _tmp209_, 0);
#line 474 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp211_ = (GXmlNode*) _tmp210_;
#line 474 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp212_ = gxml_node_get_children_nodes (_tmp211_);
#line 474 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp213_ = _tmp212_;
#line 474 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp214_ = _tmp213_;
#line 474 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp215_ = gee_list_get ((GeeList*) _tmp214_, 0);
#line 474 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp216_ = (GXmlNode*) _tmp215_;
#line 474 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp217_ = gxml_node_get_value (_tmp216_);
#line 474 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp218_ = _tmp217_;
#line 474 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp219_ = _tmp218_;
#line 474 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp219_, "Fred") == 0, "a1.children_nodes[0].children_nodes[0].value == \"Fred\"");
#line 474 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp219_);
#line 474 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp216_);
#line 474 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp214_);
#line 474 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp211_);
#line 474 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp209_);
#line 475 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp220_ = gxml_node_get_children_nodes (a1);
#line 475 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp221_ = _tmp220_;
#line 475 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp222_ = _tmp221_;
#line 475 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp223_ = gee_list_get ((GeeList*) _tmp222_, 1);
#line 475 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp224_ = (GXmlNode*) _tmp223_;
#line 475 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp225_ = gxml_node_get_name (_tmp224_);
#line 475 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp226_ = _tmp225_;
#line 475 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp227_ = _tmp226_;
#line 475 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp227_, "Email") == 0, "a1.children_nodes[1].name == \"Email\"");
#line 475 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp227_);
#line 475 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp224_);
#line 475 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp222_);
#line 476 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp228_ = gxml_node_get_children_nodes (a1);
#line 476 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp229_ = _tmp228_;
#line 476 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp230_ = _tmp229_;
#line 476 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp231_ = gee_list_get ((GeeList*) _tmp230_, 1);
#line 476 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp232_ = (GXmlNode*) _tmp231_;
#line 476 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp233_ = gxml_node_get_children_nodes (_tmp232_);
#line 476 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp234_ = _tmp233_;
#line 476 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp235_ = _tmp234_;
#line 476 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp236_ = gee_collection_get_size ((GeeCollection*) _tmp235_);
#line 476 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp237_ = _tmp236_;
#line 476 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp237_ == 1, "a1.children_nodes[1].children_nodes.size == 1");
#line 476 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp235_);
#line 476 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp232_);
#line 476 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp230_);
#line 477 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp238_ = gxml_node_get_children_nodes (a1);
#line 477 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp239_ = _tmp238_;
#line 477 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp240_ = _tmp239_;
#line 477 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp241_ = gee_list_get ((GeeList*) _tmp240_, 1);
#line 477 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp242_ = (GXmlNode*) _tmp241_;
#line 477 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp243_ = gxml_node_get_children_nodes (_tmp242_);
#line 477 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp244_ = _tmp243_;
#line 477 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp245_ = _tmp244_;
#line 477 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp246_ = gee_list_get ((GeeList*) _tmp245_, 0);
#line 477 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp247_ = (GXmlNode*) _tmp246_;
#line 477 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp247_, GXML_TYPE_TEXT), "a1.children_nodes[1].children_nodes[0] is GXml.Text");
#line 477 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp247_);
#line 477 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp245_);
#line 477 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp242_);
#line 477 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp240_);
#line 478 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp248_ = gxml_node_get_children_nodes (a1);
#line 478 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp249_ = _tmp248_;
#line 478 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp250_ = _tmp249_;
#line 478 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp251_ = gee_list_get ((GeeList*) _tmp250_, 1);
#line 478 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp252_ = (GXmlNode*) _tmp251_;
#line 478 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp253_ = gxml_node_get_children_nodes (_tmp252_);
#line 478 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp254_ = _tmp253_;
#line 478 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp255_ = _tmp254_;
#line 478 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp256_ = gee_list_get ((GeeList*) _tmp255_, 0);
#line 478 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp257_ = (GXmlNode*) _tmp256_;
#line 478 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp258_ = gxml_node_get_value (_tmp257_);
#line 478 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp259_ = _tmp258_;
#line 478 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp260_ = _tmp259_;
#line 478 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp260_, "fweasley@hogwarts.co.uk") == 0, "a1.children_nodes[1].children_nodes[0].value == \"fweasley@hogwarts.co.uk\"");
#line 478 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp260_);
#line 478 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp257_);
#line 478 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp255_);
#line 478 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp252_);
#line 478 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp250_);
#line 479 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp261_ = gxml_node_get_children_nodes (s3);
#line 479 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp262_ = _tmp261_;
#line 479 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp263_ = _tmp262_;
#line 479 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp264_ = gee_list_get ((GeeList*) _tmp263_, 1);
#line 479 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp265_ = (GXmlNode*) _tmp264_;
#line 479 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp263_);
#line 479 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		a2 = _tmp265_;
#line 480 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (a2 != NULL, "a2 != null");
#line 481 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp266_ = gxml_node_get_children_nodes (a2);
#line 481 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp267_ = _tmp266_;
#line 481 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp268_ = _tmp267_;
#line 481 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp269_ = gee_collection_get_size ((GeeCollection*) _tmp268_);
#line 481 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp270_ = _tmp269_;
#line 481 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp270_ == 3, "a2.children_nodes.size == 3");
#line 481 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp268_);
#line 482 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp271_ = gxml_node_get_children_nodes (a2);
#line 482 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp272_ = _tmp271_;
#line 482 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp273_ = _tmp272_;
#line 482 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp274_ = gee_list_get ((GeeList*) _tmp273_, 1);
#line 482 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp275_ = (GXmlNode*) _tmp274_;
#line 482 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp276_ = gxml_node_get_name (_tmp275_);
#line 482 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp277_ = _tmp276_;
#line 482 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp278_ = _tmp277_;
#line 482 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp278_, "Name") == 0, "a2.children_nodes[1].name == \"Name\"");
#line 482 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp278_);
#line 482 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp275_);
#line 482 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp273_);
#line 483 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp279_ = gxml_node_get_children_nodes (a2);
#line 483 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp280_ = _tmp279_;
#line 483 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp281_ = _tmp280_;
#line 483 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp282_ = gee_list_get ((GeeList*) _tmp281_, 1);
#line 483 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp283_ = (GXmlNode*) _tmp282_;
#line 483 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp284_ = gxml_node_get_children_nodes (_tmp283_);
#line 483 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp285_ = _tmp284_;
#line 483 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp286_ = _tmp285_;
#line 483 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp287_ = gee_collection_get_size ((GeeCollection*) _tmp286_);
#line 483 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp288_ = _tmp287_;
#line 483 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp288_ == 1, "a2.children_nodes[1].children_nodes.size == 1");
#line 483 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp286_);
#line 483 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp283_);
#line 483 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp281_);
#line 484 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp289_ = gxml_node_get_children_nodes (a2);
#line 484 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp290_ = _tmp289_;
#line 484 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp291_ = _tmp290_;
#line 484 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp292_ = gee_list_get ((GeeList*) _tmp291_, 1);
#line 484 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp293_ = (GXmlNode*) _tmp292_;
#line 484 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp294_ = gxml_node_get_children_nodes (_tmp293_);
#line 484 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp295_ = _tmp294_;
#line 484 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp296_ = _tmp295_;
#line 484 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp297_ = gee_list_get ((GeeList*) _tmp296_, 0);
#line 484 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp298_ = (GXmlNode*) _tmp297_;
#line 484 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp298_, GXML_TYPE_TEXT), "a2.children_nodes[1].children_nodes[0] is GXml.Text");
#line 484 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp298_);
#line 484 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp296_);
#line 484 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp293_);
#line 484 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp291_);
#line 485 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp299_ = gxml_node_get_children_nodes (a2);
#line 485 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp300_ = _tmp299_;
#line 485 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp301_ = _tmp300_;
#line 485 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp302_ = gee_list_get ((GeeList*) _tmp301_, 1);
#line 485 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp303_ = (GXmlNode*) _tmp302_;
#line 485 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp304_ = gxml_node_get_children_nodes (_tmp303_);
#line 485 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp305_ = _tmp304_;
#line 485 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp306_ = _tmp305_;
#line 485 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp307_ = gee_list_get ((GeeList*) _tmp306_, 0);
#line 485 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp308_ = (GXmlNode*) _tmp307_;
#line 485 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp309_ = gxml_node_get_value (_tmp308_);
#line 485 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp310_ = _tmp309_;
#line 485 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp311_ = _tmp310_;
#line 485 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp311_, "George") == 0, "a2.children_nodes[1].children_nodes[0].value == \"George\"");
#line 485 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp311_);
#line 485 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp308_);
#line 485 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp306_);
#line 485 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp303_);
#line 485 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp301_);
#line 486 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp312_ = gxml_node_get_children_nodes (a2);
#line 486 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp313_ = _tmp312_;
#line 486 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp314_ = _tmp313_;
#line 486 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp315_ = gee_list_get ((GeeList*) _tmp314_, 2);
#line 486 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp316_ = (GXmlNode*) _tmp315_;
#line 486 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp317_ = gxml_node_get_name (_tmp316_);
#line 486 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp318_ = _tmp317_;
#line 486 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp319_ = _tmp318_;
#line 486 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp319_, "Email") == 0, "a2.children_nodes[2].name == \"Email\"");
#line 486 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp319_);
#line 486 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp316_);
#line 486 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp314_);
#line 487 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp320_ = gxml_node_get_children_nodes (a2);
#line 487 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp321_ = _tmp320_;
#line 487 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp322_ = _tmp321_;
#line 487 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp323_ = gee_list_get ((GeeList*) _tmp322_, 2);
#line 487 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp324_ = (GXmlNode*) _tmp323_;
#line 487 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp325_ = gxml_node_get_children_nodes (_tmp324_);
#line 487 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp326_ = _tmp325_;
#line 487 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp327_ = _tmp326_;
#line 487 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp328_ = gee_collection_get_size ((GeeCollection*) _tmp327_);
#line 487 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp329_ = _tmp328_;
#line 487 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp329_ == 1, "a2.children_nodes[2].children_nodes.size == 1");
#line 487 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp327_);
#line 487 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp324_);
#line 487 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp322_);
#line 488 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp330_ = gxml_node_get_children_nodes (a2);
#line 488 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp331_ = _tmp330_;
#line 488 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp332_ = _tmp331_;
#line 488 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp333_ = gee_list_get ((GeeList*) _tmp332_, 2);
#line 488 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp334_ = (GXmlNode*) _tmp333_;
#line 488 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp335_ = gxml_node_get_children_nodes (_tmp334_);
#line 488 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp336_ = _tmp335_;
#line 488 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp337_ = _tmp336_;
#line 488 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp338_ = gee_list_get ((GeeList*) _tmp337_, 0);
#line 488 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp339_ = (GXmlNode*) _tmp338_;
#line 488 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp339_, GXML_TYPE_TEXT), "a2.children_nodes[2].children_nodes[0] is GXml.Text");
#line 488 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp339_);
#line 488 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp337_);
#line 488 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp334_);
#line 488 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp332_);
#line 489 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp340_ = gxml_node_get_children_nodes (a2);
#line 489 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp341_ = _tmp340_;
#line 489 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp342_ = _tmp341_;
#line 489 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp343_ = gee_list_get ((GeeList*) _tmp342_, 2);
#line 489 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp344_ = (GXmlNode*) _tmp343_;
#line 489 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp345_ = gxml_node_get_children_nodes (_tmp344_);
#line 489 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp346_ = _tmp345_;
#line 489 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp347_ = _tmp346_;
#line 489 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp348_ = gee_list_get ((GeeList*) _tmp347_, 0);
#line 489 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp349_ = (GXmlNode*) _tmp348_;
#line 489 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp350_ = gxml_node_get_value (_tmp349_);
#line 489 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp351_ = _tmp350_;
#line 489 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp352_ = _tmp351_;
#line 489 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp352_, "gweasley@hogwarts.co.uk") == 0, "a2.children_nodes[2].children_nodes[0].value == \"gweasley@hogwarts.co.uk\"");
#line 489 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp352_);
#line 489 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp349_);
#line 489 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp347_);
#line 489 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp344_);
#line 489 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp342_);
#line 425 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (a2);
#line 425 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (a1);
#line 425 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (p31);
#line 425 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (p3);
#line 425 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (s3);
#line 425 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (p2);
#line 425 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (s2);
#line 425 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (p1);
#line 425 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (s1);
#line 425 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (d);
#line 425 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (f);
#line 7446 "TDocumentTest.c"
	}
	goto __finally134;
	__catch134_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp353_ = NULL;
		gchar* _tmp354_ = NULL;
		gchar* _tmp355_ = NULL;
#line 425 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _inner_error_;
#line 425 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_inner_error_ = NULL;
#line 490 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp353_ = e->message;
#line 490 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp354_ = g_strconcat ("ERROR: ", _tmp353_, NULL);
#line 490 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp355_ = _tmp354_;
#line 490 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_message ("TDocumentTest.vala:490: %s", _tmp355_);
#line 490 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp355_);
#line 490 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_assert_not_reached ();
#line 425 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_error_free0 (e);
#line 7473 "TDocumentTest.c"
	}
	__finally134:
#line 425 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 425 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 425 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 425 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		return;
#line 7484 "TDocumentTest.c"
	}
}


static void ___lambda117__gtest_func (void) {
#line 424 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda117_ ();
#line 7492 "TDocumentTest.c"
}


static void __lambda118_ (void) {
	GFile* f = NULL;
	GFile* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
	GXmlTDocument* d = NULL;
	GXmlTDocument* _tmp2_ = NULL;
	GXmlNode* _tmp3_ = NULL;
	GXmlNode* _tmp4_ = NULL;
	GXmlNode* _tmp5_ = NULL;
	GXmlNode* _tmp6_ = NULL;
	GXmlNode* _tmp7_ = NULL;
	GXmlNode* _tmp8_ = NULL;
	gchar* _tmp9_ = NULL;
	gchar* _tmp10_ = NULL;
	gchar* _tmp11_ = NULL;
	GXmlNode* _tmp12_ = NULL;
	GXmlNode* _tmp13_ = NULL;
	GXmlNode* _tmp14_ = NULL;
	GeeList* _tmp15_ = NULL;
	GeeList* _tmp16_ = NULL;
	GeeList* _tmp17_ = NULL;
	gint _tmp18_ = 0;
	gint _tmp19_ = 0;
	GXmlNode* _tmp20_ = NULL;
	GXmlNode* _tmp21_ = NULL;
	GXmlNode* _tmp22_ = NULL;
	GeeList* _tmp23_ = NULL;
	GeeList* _tmp24_ = NULL;
	GeeList* _tmp25_ = NULL;
	gpointer _tmp26_ = NULL;
	GXmlNamespace* _tmp27_ = NULL;
	gchar* _tmp28_ = NULL;
	gchar* _tmp29_ = NULL;
	gchar* _tmp30_ = NULL;
	GXmlNode* _tmp31_ = NULL;
	GXmlNode* _tmp32_ = NULL;
	GXmlNode* _tmp33_ = NULL;
	GeeList* _tmp34_ = NULL;
	GeeList* _tmp35_ = NULL;
	GeeList* _tmp36_ = NULL;
	gpointer _tmp37_ = NULL;
	GXmlNamespace* _tmp38_ = NULL;
	gchar* _tmp39_ = NULL;
	gchar* _tmp40_ = NULL;
	gchar* _tmp41_ = NULL;
	GXmlNode* _tmp42_ = NULL;
	GXmlNode* _tmp43_ = NULL;
	GXmlNode* _tmp44_ = NULL;
	GeeList* _tmp45_ = NULL;
	GeeList* _tmp46_ = NULL;
	GeeList* _tmp47_ = NULL;
	gpointer _tmp48_ = NULL;
	GXmlNamespace* _tmp49_ = NULL;
	gchar* _tmp50_ = NULL;
	gchar* _tmp51_ = NULL;
	gchar* _tmp52_ = NULL;
	GXmlNode* _tmp53_ = NULL;
	GXmlNode* _tmp54_ = NULL;
	GXmlNode* _tmp55_ = NULL;
	GeeList* _tmp56_ = NULL;
	GeeList* _tmp57_ = NULL;
	GeeList* _tmp58_ = NULL;
	gpointer _tmp59_ = NULL;
	GXmlNamespace* _tmp60_ = NULL;
	gchar* _tmp61_ = NULL;
	gchar* _tmp62_ = NULL;
	gchar* _tmp63_ = NULL;
	GXmlNode* a = NULL;
	GXmlNode* _tmp64_ = NULL;
	GXmlNode* _tmp65_ = NULL;
	GXmlNode* _tmp66_ = NULL;
	GeeBidirList* _tmp67_ = NULL;
	GeeBidirList* _tmp68_ = NULL;
	GeeBidirList* _tmp69_ = NULL;
	gpointer _tmp70_ = NULL;
	GXmlNode* _tmp71_ = NULL;
	gchar* _tmp72_ = NULL;
	gchar* _tmp73_ = NULL;
	gchar* _tmp74_ = NULL;
	GeeList* _tmp75_ = NULL;
	GeeList* _tmp76_ = NULL;
	GeeList* _tmp77_ = NULL;
	gint _tmp78_ = 0;
	gint _tmp79_ = 0;
	GeeList* _tmp80_ = NULL;
	GeeList* _tmp81_ = NULL;
	GeeList* _tmp82_ = NULL;
	gpointer _tmp83_ = NULL;
	GXmlNamespace* _tmp84_ = NULL;
	gchar* _tmp85_ = NULL;
	gchar* _tmp86_ = NULL;
	gchar* _tmp87_ = NULL;
	GeeList* _tmp88_ = NULL;
	GeeList* _tmp89_ = NULL;
	GeeList* _tmp90_ = NULL;
	gpointer _tmp91_ = NULL;
	GXmlNamespace* _tmp92_ = NULL;
	gchar* _tmp93_ = NULL;
	gchar* _tmp94_ = NULL;
	gchar* _tmp95_ = NULL;
	GeeBidirList* _tmp96_ = NULL;
	GeeBidirList* _tmp97_ = NULL;
	GeeBidirList* _tmp98_ = NULL;
	gpointer _tmp99_ = NULL;
	GXmlNode* _tmp100_ = NULL;
	GXmlNode* a1 = NULL;
	GeeBidirList* _tmp101_ = NULL;
	GeeBidirList* _tmp102_ = NULL;
	GeeBidirList* _tmp103_ = NULL;
	gpointer _tmp104_ = NULL;
	GXmlNode* _tmp105_ = NULL;
	gchar* _tmp106_ = NULL;
	gchar* _tmp107_ = NULL;
	gchar* _tmp108_ = NULL;
	GXmlNode* e = NULL;
	GeeBidirList* _tmp109_ = NULL;
	GeeBidirList* _tmp110_ = NULL;
	GeeBidirList* _tmp111_ = NULL;
	gpointer _tmp112_ = NULL;
	GXmlNode* _tmp113_ = NULL;
	gchar* _tmp114_ = NULL;
	gchar* _tmp115_ = NULL;
	gchar* _tmp116_ = NULL;
	GeeList* _tmp117_ = NULL;
	GeeList* _tmp118_ = NULL;
	GeeList* _tmp119_ = NULL;
	gint _tmp120_ = 0;
	gint _tmp121_ = 0;
	GeeList* _tmp122_ = NULL;
	GeeList* _tmp123_ = NULL;
	GeeList* _tmp124_ = NULL;
	gpointer _tmp125_ = NULL;
	GXmlNamespace* _tmp126_ = NULL;
	gchar* _tmp127_ = NULL;
	gchar* _tmp128_ = NULL;
	gchar* _tmp129_ = NULL;
	GeeList* _tmp130_ = NULL;
	GeeList* _tmp131_ = NULL;
	GeeList* _tmp132_ = NULL;
	gpointer _tmp133_ = NULL;
	GXmlNamespace* _tmp134_ = NULL;
	gchar* _tmp135_ = NULL;
	gchar* _tmp136_ = NULL;
	gchar* _tmp137_ = NULL;
	GXmlNode* b = NULL;
	GXmlNode* _tmp138_ = NULL;
	GXmlNode* _tmp139_ = NULL;
	GXmlNode* _tmp140_ = NULL;
	GeeBidirList* _tmp141_ = NULL;
	GeeBidirList* _tmp142_ = NULL;
	GeeBidirList* _tmp143_ = NULL;
	gpointer _tmp144_ = NULL;
	GXmlNode* _tmp145_ = NULL;
	gchar* _tmp146_ = NULL;
	gchar* _tmp147_ = NULL;
	gchar* _tmp148_ = NULL;
	GeeList* _tmp149_ = NULL;
	GeeList* _tmp150_ = NULL;
	GeeList* _tmp151_ = NULL;
	gint _tmp152_ = 0;
	gint _tmp153_ = 0;
	GeeList* _tmp154_ = NULL;
	GeeList* _tmp155_ = NULL;
	GeeList* _tmp156_ = NULL;
	gpointer _tmp157_ = NULL;
	GXmlNamespace* _tmp158_ = NULL;
	gchar* _tmp159_ = NULL;
	gchar* _tmp160_ = NULL;
	gchar* _tmp161_ = NULL;
	GeeList* _tmp162_ = NULL;
	GeeList* _tmp163_ = NULL;
	GeeList* _tmp164_ = NULL;
	gpointer _tmp165_ = NULL;
	GXmlNamespace* _tmp166_ = NULL;
	gchar* _tmp167_ = NULL;
	gchar* _tmp168_ = NULL;
	gchar* _tmp169_ = NULL;
	GXmlNode* bp = NULL;
	GeeMap* _tmp170_ = NULL;
	GeeMap* _tmp171_ = NULL;
	GeeMap* _tmp172_ = NULL;
	gpointer _tmp173_ = NULL;
	GXmlNode* _tmp174_ = NULL;
	gchar* _tmp175_ = NULL;
	gchar* _tmp176_ = NULL;
	gchar* _tmp177_ = NULL;
	GeeList* _tmp178_ = NULL;
	GeeList* _tmp179_ = NULL;
	GeeList* _tmp180_ = NULL;
	gint _tmp181_ = 0;
	gint _tmp182_ = 0;
	GeeList* _tmp183_ = NULL;
	GeeList* _tmp184_ = NULL;
	GeeList* _tmp185_ = NULL;
	gpointer _tmp186_ = NULL;
	GXmlNamespace* _tmp187_ = NULL;
	gchar* _tmp188_ = NULL;
	gchar* _tmp189_ = NULL;
	gchar* _tmp190_ = NULL;
	GeeList* _tmp191_ = NULL;
	GeeList* _tmp192_ = NULL;
	GeeList* _tmp193_ = NULL;
	gpointer _tmp194_ = NULL;
	GXmlNamespace* _tmp195_ = NULL;
	gchar* _tmp196_ = NULL;
	gchar* _tmp197_ = NULL;
	gchar* _tmp198_ = NULL;
#line 493 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp0_ = g_file_new_for_path (TEST_DIR "/t-read-test.xml");
#line 493 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	f = _tmp0_;
#line 494 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp1_ = g_file_query_exists (f, NULL);
#line 494 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp1_, "f.query_exists ()");
#line 495 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp2_ = gxml_tdocument_new_from_file (f);
#line 495 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	d = _tmp2_;
#line 497 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp3_ = gxml_document_get_root ((GXmlDocument*) d);
#line 497 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp4_ = _tmp3_;
#line 497 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp5_ = _tmp4_;
#line 497 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp5_ != NULL, "d.root != null");
#line 497 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp5_);
#line 498 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp6_ = gxml_document_get_root ((GXmlDocument*) d);
#line 498 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp7_ = _tmp6_;
#line 498 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp8_ = _tmp7_;
#line 498 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp9_ = gxml_node_get_name (_tmp8_);
#line 498 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp10_ = _tmp9_;
#line 498 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp11_ = _tmp10_;
#line 498 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp11_, "Sentences") == 0, "d.root.name == \"Sentences\"");
#line 498 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp11_);
#line 498 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp8_);
#line 499 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp12_ = gxml_document_get_root ((GXmlDocument*) d);
#line 499 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp13_ = _tmp12_;
#line 499 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp14_ = _tmp13_;
#line 499 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp15_ = gxml_node_get_namespaces (_tmp14_);
#line 499 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp16_ = _tmp15_;
#line 499 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp17_ = _tmp16_;
#line 499 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp18_ = gee_collection_get_size ((GeeCollection*) _tmp17_);
#line 499 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp19_ = _tmp18_;
#line 499 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp19_ == 2, "d.root.namespaces.size == 2");
#line 499 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp17_);
#line 499 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp14_);
#line 500 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp20_ = gxml_document_get_root ((GXmlDocument*) d);
#line 500 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp21_ = _tmp20_;
#line 500 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp22_ = _tmp21_;
#line 500 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp23_ = gxml_node_get_namespaces (_tmp22_);
#line 500 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp24_ = _tmp23_;
#line 500 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp25_ = _tmp24_;
#line 500 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp26_ = gee_list_get (_tmp25_, 0);
#line 500 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp27_ = (GXmlNamespace*) _tmp26_;
#line 500 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp28_ = gxml_namespace_get_prefix (_tmp27_);
#line 500 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp29_ = _tmp28_;
#line 500 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp30_ = _tmp29_;
#line 500 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp30_, "gxml") == 0, "d.root.namespaces[0].prefix == \"gxml\"");
#line 500 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp30_);
#line 500 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp27_);
#line 500 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp25_);
#line 500 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp22_);
#line 501 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp31_ = gxml_document_get_root ((GXmlDocument*) d);
#line 501 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp32_ = _tmp31_;
#line 501 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp33_ = _tmp32_;
#line 501 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp34_ = gxml_node_get_namespaces (_tmp33_);
#line 501 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp35_ = _tmp34_;
#line 501 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp36_ = _tmp35_;
#line 501 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp37_ = gee_list_get (_tmp36_, 0);
#line 501 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp38_ = (GXmlNamespace*) _tmp37_;
#line 501 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp39_ = gxml_namespace_get_uri (_tmp38_);
#line 501 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp40_ = _tmp39_;
#line 501 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp41_ = _tmp40_;
#line 501 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp41_, "http://wiki.gnome.org/GXml") == 0, "d.root.namespaces[0].uri == \"http://wiki.gnome.org/GXml\"");
#line 501 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp41_);
#line 501 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp38_);
#line 501 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp36_);
#line 501 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp33_);
#line 502 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp42_ = gxml_document_get_root ((GXmlDocument*) d);
#line 502 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp43_ = _tmp42_;
#line 502 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp44_ = _tmp43_;
#line 502 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp45_ = gxml_node_get_namespaces (_tmp44_);
#line 502 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp46_ = _tmp45_;
#line 502 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp47_ = _tmp46_;
#line 502 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp48_ = gee_list_get (_tmp47_, 1);
#line 502 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp49_ = (GXmlNamespace*) _tmp48_;
#line 502 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp50_ = gxml_namespace_get_prefix (_tmp49_);
#line 502 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp51_ = _tmp50_;
#line 502 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp52_ = _tmp51_;
#line 502 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp52_, "b") == 0, "d.root.namespaces[1].prefix == \"b\"");
#line 502 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp52_);
#line 502 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp49_);
#line 502 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp47_);
#line 502 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp44_);
#line 503 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp53_ = gxml_document_get_root ((GXmlDocument*) d);
#line 503 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp54_ = _tmp53_;
#line 503 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp55_ = _tmp54_;
#line 503 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp56_ = gxml_node_get_namespaces (_tmp55_);
#line 503 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp57_ = _tmp56_;
#line 503 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp58_ = _tmp57_;
#line 503 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp59_ = gee_list_get (_tmp58_, 1);
#line 503 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp60_ = (GXmlNamespace*) _tmp59_;
#line 503 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp61_ = gxml_namespace_get_uri (_tmp60_);
#line 503 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp62_ = _tmp61_;
#line 503 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp63_ = _tmp62_;
#line 503 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp63_, "http://book.org/schema") == 0, "d.root.namespaces[1].uri == \"http://book.org/schema\"");
#line 503 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp63_);
#line 503 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp60_);
#line 503 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp58_);
#line 503 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp55_);
#line 504 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp64_ = gxml_document_get_root ((GXmlDocument*) d);
#line 504 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp65_ = _tmp64_;
#line 504 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp66_ = _tmp65_;
#line 504 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp67_ = gxml_node_get_children_nodes (_tmp66_);
#line 504 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp68_ = _tmp67_;
#line 504 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp69_ = _tmp68_;
#line 504 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp70_ = gee_list_get ((GeeList*) _tmp69_, 2);
#line 504 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp71_ = (GXmlNode*) _tmp70_;
#line 504 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp69_);
#line 504 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp66_);
#line 504 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	a = _tmp71_;
#line 505 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (a != NULL, "a != null");
#line 506 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp72_ = gxml_node_get_name (a);
#line 506 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp73_ = _tmp72_;
#line 506 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp74_ = _tmp73_;
#line 506 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp74_, "Authors") == 0, "a.name == \"Authors\"");
#line 506 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp74_);
#line 507 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp75_ = gxml_node_get_namespaces (a);
#line 507 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp76_ = _tmp75_;
#line 507 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp77_ = _tmp76_;
#line 507 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp78_ = gee_collection_get_size ((GeeCollection*) _tmp77_);
#line 507 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp79_ = _tmp78_;
#line 507 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp79_ == 1, "a.namespaces.size == 1");
#line 507 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp77_);
#line 508 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp80_ = gxml_node_get_namespaces (a);
#line 508 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp81_ = _tmp80_;
#line 508 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp82_ = _tmp81_;
#line 508 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp83_ = gee_list_get (_tmp82_, 0);
#line 508 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp84_ = (GXmlNamespace*) _tmp83_;
#line 508 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp85_ = gxml_namespace_get_uri (_tmp84_);
#line 508 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp86_ = _tmp85_;
#line 508 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp87_ = _tmp86_;
#line 508 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp87_, "http://author.org") == 0, "a.namespaces[0].uri == \"http://author.org\"");
#line 508 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp87_);
#line 508 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp84_);
#line 508 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp82_);
#line 509 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp88_ = gxml_node_get_namespaces (a);
#line 509 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp89_ = _tmp88_;
#line 509 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp90_ = _tmp89_;
#line 509 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp91_ = gee_list_get (_tmp90_, 0);
#line 509 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp92_ = (GXmlNamespace*) _tmp91_;
#line 509 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp93_ = gxml_namespace_get_prefix (_tmp92_);
#line 509 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp94_ = _tmp93_;
#line 509 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp95_ = _tmp94_;
#line 509 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp95_, "auth") == 0, "a.namespaces[0].prefix == \"auth\"");
#line 509 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp95_);
#line 509 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp92_);
#line 509 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp90_);
#line 510 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp96_ = gxml_node_get_children_nodes (a);
#line 510 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp97_ = _tmp96_;
#line 510 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp98_ = _tmp97_;
#line 510 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp99_ = gee_list_get ((GeeList*) _tmp98_, 0);
#line 510 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp100_ = (GXmlNode*) _tmp99_;
#line 510 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp100_ != NULL, "a.children_nodes[0] != null");
#line 510 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp100_);
#line 510 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp98_);
#line 511 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp101_ = gxml_node_get_children_nodes (a);
#line 511 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp102_ = _tmp101_;
#line 511 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp103_ = _tmp102_;
#line 511 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp104_ = gee_list_get ((GeeList*) _tmp103_, 0);
#line 511 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp105_ = (GXmlNode*) _tmp104_;
#line 511 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp103_);
#line 511 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	a1 = _tmp105_;
#line 512 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (a1 != NULL, "a1 != null");
#line 513 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp106_ = gxml_node_get_name (a1);
#line 513 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp107_ = _tmp106_;
#line 513 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp108_ = _tmp107_;
#line 513 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp108_, "Author") == 0, "a1.name == \"Author\"");
#line 513 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp108_);
#line 514 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp109_ = gxml_node_get_children_nodes (a1);
#line 514 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp110_ = _tmp109_;
#line 514 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp111_ = _tmp110_;
#line 514 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp112_ = gee_list_get ((GeeList*) _tmp111_, 1);
#line 514 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp113_ = (GXmlNode*) _tmp112_;
#line 514 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp111_);
#line 514 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	e = _tmp113_;
#line 515 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (e != NULL, "e != null");
#line 516 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp114_ = gxml_node_get_name (e);
#line 516 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp115_ = _tmp114_;
#line 516 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp116_ = _tmp115_;
#line 516 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp116_, "Email") == 0, "e.name == \"Email\"");
#line 516 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp116_);
#line 517 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp117_ = gxml_node_get_namespaces (e);
#line 517 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp118_ = _tmp117_;
#line 517 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp119_ = _tmp118_;
#line 517 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp120_ = gee_collection_get_size ((GeeCollection*) _tmp119_);
#line 517 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp121_ = _tmp120_;
#line 517 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp121_ == 1, "e.namespaces.size == 1");
#line 517 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp119_);
#line 518 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp122_ = gxml_node_get_namespaces (e);
#line 518 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp123_ = _tmp122_;
#line 518 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp124_ = _tmp123_;
#line 518 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp125_ = gee_list_get (_tmp124_, 0);
#line 518 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp126_ = (GXmlNamespace*) _tmp125_;
#line 518 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp127_ = gxml_namespace_get_prefix (_tmp126_);
#line 518 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp128_ = _tmp127_;
#line 518 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp129_ = _tmp128_;
#line 518 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp129_, "gxml") == 0, "e.namespaces[0].prefix == \"gxml\"");
#line 518 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp129_);
#line 518 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp126_);
#line 518 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp124_);
#line 519 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp130_ = gxml_node_get_namespaces (e);
#line 519 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp131_ = _tmp130_;
#line 519 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp132_ = _tmp131_;
#line 519 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp133_ = gee_list_get (_tmp132_, 0);
#line 519 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp134_ = (GXmlNamespace*) _tmp133_;
#line 519 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp135_ = gxml_namespace_get_uri (_tmp134_);
#line 519 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp136_ = _tmp135_;
#line 519 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp137_ = _tmp136_;
#line 519 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp137_, "http://wiki.gnome.org/GXml") == 0, "e.namespaces[0].uri == \"http://wiki.gnome.org/GXml\"");
#line 519 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp137_);
#line 519 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp134_);
#line 519 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp132_);
#line 520 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp138_ = gxml_document_get_root ((GXmlDocument*) d);
#line 520 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp139_ = _tmp138_;
#line 520 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp140_ = _tmp139_;
#line 520 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp141_ = gxml_node_get_children_nodes (_tmp140_);
#line 520 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp142_ = _tmp141_;
#line 520 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp143_ = _tmp142_;
#line 520 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp144_ = gee_list_get ((GeeList*) _tmp143_, 3);
#line 520 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp145_ = (GXmlNode*) _tmp144_;
#line 520 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp143_);
#line 520 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp140_);
#line 520 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	b = _tmp145_;
#line 521 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (b != NULL, "b != null");
#line 522 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp146_ = gxml_node_get_name (b);
#line 522 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp147_ = _tmp146_;
#line 522 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp148_ = _tmp147_;
#line 522 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp148_, "Book") == 0, "b.name == \"Book\"");
#line 522 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp148_);
#line 523 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp149_ = gxml_node_get_namespaces (b);
#line 523 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp150_ = _tmp149_;
#line 523 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp151_ = _tmp150_;
#line 523 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp152_ = gee_collection_get_size ((GeeCollection*) _tmp151_);
#line 523 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp153_ = _tmp152_;
#line 523 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp153_ == 1, "b.namespaces.size == 1");
#line 523 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp151_);
#line 524 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp154_ = gxml_node_get_namespaces (b);
#line 524 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp155_ = _tmp154_;
#line 524 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp156_ = _tmp155_;
#line 524 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp157_ = gee_list_get (_tmp156_, 0);
#line 524 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp158_ = (GXmlNamespace*) _tmp157_;
#line 524 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp159_ = gxml_namespace_get_prefix (_tmp158_);
#line 524 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp160_ = _tmp159_;
#line 524 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp161_ = _tmp160_;
#line 524 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp161_, "b") == 0, "b.namespaces[0].prefix == \"b\"");
#line 524 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp161_);
#line 524 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp158_);
#line 524 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp156_);
#line 525 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp162_ = gxml_node_get_namespaces (b);
#line 525 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp163_ = _tmp162_;
#line 525 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp164_ = _tmp163_;
#line 525 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp165_ = gee_list_get (_tmp164_, 0);
#line 525 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp166_ = (GXmlNamespace*) _tmp165_;
#line 525 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp167_ = gxml_namespace_get_uri (_tmp166_);
#line 525 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp168_ = _tmp167_;
#line 525 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp169_ = _tmp168_;
#line 525 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp169_, "http://book.org/schema") == 0, "b.namespaces[0].uri == \"http://book.org/schema\"");
#line 525 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp169_);
#line 525 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp166_);
#line 525 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp164_);
#line 526 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp170_ = gxml_node_get_attrs (b);
#line 526 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp171_ = _tmp170_;
#line 526 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp172_ = _tmp171_;
#line 526 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp173_ = gee_map_get (_tmp172_, "gxml:name");
#line 526 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp174_ = (GXmlNode*) _tmp173_;
#line 526 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp172_);
#line 526 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	bp = _tmp174_;
#line 527 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (bp != NULL, "bp != null");
#line 528 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp175_ = gxml_node_get_name (bp);
#line 528 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp176_ = _tmp175_;
#line 528 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp177_ = _tmp176_;
#line 528 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp177_, "name") == 0, "bp.name == \"name\"");
#line 528 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp177_);
#line 529 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp178_ = gxml_node_get_namespaces (bp);
#line 529 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp179_ = _tmp178_;
#line 529 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp180_ = _tmp179_;
#line 529 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp181_ = gee_collection_get_size ((GeeCollection*) _tmp180_);
#line 529 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp182_ = _tmp181_;
#line 529 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp182_ == 1, "bp.namespaces.size == 1");
#line 529 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp180_);
#line 530 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp183_ = gxml_node_get_namespaces (bp);
#line 530 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp184_ = _tmp183_;
#line 530 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp185_ = _tmp184_;
#line 530 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp186_ = gee_list_get (_tmp185_, 0);
#line 530 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp187_ = (GXmlNamespace*) _tmp186_;
#line 530 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp188_ = gxml_namespace_get_prefix (_tmp187_);
#line 530 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp189_ = _tmp188_;
#line 530 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp190_ = _tmp189_;
#line 530 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp190_, "gxml") == 0, "bp.namespaces[0].prefix == \"gxml\"");
#line 530 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp190_);
#line 530 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp187_);
#line 530 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp185_);
#line 531 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp191_ = gxml_node_get_namespaces (bp);
#line 531 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp192_ = _tmp191_;
#line 531 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp193_ = _tmp192_;
#line 531 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp194_ = gee_list_get (_tmp193_, 0);
#line 531 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp195_ = (GXmlNamespace*) _tmp194_;
#line 531 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp196_ = gxml_namespace_get_uri (_tmp195_);
#line 531 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp197_ = _tmp196_;
#line 531 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp198_ = _tmp197_;
#line 531 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp198_, "http://wiki.gnome.org/GXml") == 0, "bp.namespaces[0].uri == \"http://wiki.gnome.org/GXml\"");
#line 531 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp198_);
#line 531 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp195_);
#line 531 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp193_);
#line 492 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (bp);
#line 492 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (b);
#line 492 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (e);
#line 492 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (a1);
#line 492 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (a);
#line 492 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (d);
#line 492 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (f);
#line 8317 "TDocumentTest.c"
}


static void ___lambda118__gtest_func (void) {
#line 492 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda118_ ();
#line 8324 "TDocumentTest.c"
}


static void __lambda119_ (void) {
	GFile* f = NULL;
	GFile* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
	GXmlTDocument* d = NULL;
	GXmlTDocument* _tmp2_ = NULL;
	GeeBidirList* _tmp3_ = NULL;
	GeeBidirList* _tmp4_ = NULL;
	GeeBidirList* _tmp5_ = NULL;
	gpointer _tmp6_ = NULL;
	GXmlNode* _tmp7_ = NULL;
	GeeBidirList* _tmp8_ = NULL;
	GeeBidirList* _tmp9_ = NULL;
	GeeBidirList* _tmp10_ = NULL;
	gpointer _tmp11_ = NULL;
	GXmlNode* _tmp12_ = NULL;
	gchar* _tmp13_ = NULL;
	gchar* _tmp14_ = NULL;
	gchar* _tmp15_ = NULL;
	GXmlNode* a = NULL;
	GXmlNode* _tmp16_ = NULL;
	GXmlNode* _tmp17_ = NULL;
	GXmlNode* _tmp18_ = NULL;
	GeeBidirList* _tmp19_ = NULL;
	GeeBidirList* _tmp20_ = NULL;
	GeeBidirList* _tmp21_ = NULL;
	gpointer _tmp22_ = NULL;
	GXmlNode* _tmp23_ = NULL;
	gchar* _tmp24_ = NULL;
	gchar* _tmp25_ = NULL;
	gchar* _tmp26_ = NULL;
	GXmlNode* a1 = NULL;
	GeeBidirList* _tmp27_ = NULL;
	GeeBidirList* _tmp28_ = NULL;
	GeeBidirList* _tmp29_ = NULL;
	gpointer _tmp30_ = NULL;
	GXmlNode* _tmp31_ = NULL;
	gchar* _tmp32_ = NULL;
	gchar* _tmp33_ = NULL;
	gchar* _tmp34_ = NULL;
	GeeBidirList* _tmp35_ = NULL;
	GeeBidirList* _tmp36_ = NULL;
	GeeBidirList* _tmp37_ = NULL;
	gpointer _tmp38_ = NULL;
	GXmlNode* _tmp39_ = NULL;
	GeeBidirList* _tmp40_ = NULL;
	GeeBidirList* _tmp41_ = NULL;
	GeeBidirList* _tmp42_ = NULL;
	gpointer _tmp43_ = NULL;
	GXmlNode* _tmp44_ = NULL;
	gchar* _tmp45_ = NULL;
	gchar* _tmp46_ = NULL;
	gchar* _tmp47_ = NULL;
#line 534 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp0_ = g_file_new_for_path (TEST_DIR "/t-read-test.xml");
#line 534 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	f = _tmp0_;
#line 535 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp1_ = g_file_query_exists (f, NULL);
#line 535 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp1_, "f.query_exists ()");
#line 536 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp2_ = gxml_tdocument_new_from_file (f);
#line 536 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	d = _tmp2_;
#line 537 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp3_ = gxml_tnode_get_children_nodes ((GXmlTNode*) d);
#line 537 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp4_ = _tmp3_;
#line 537 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp5_ = _tmp4_;
#line 537 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp6_ = gee_list_get ((GeeList*) _tmp5_, 0);
#line 537 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp7_ = (GXmlNode*) _tmp6_;
#line 537 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp7_, GXML_TYPE_COMMENT), "d.children_nodes[0] is GXml.Comment");
#line 537 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp7_);
#line 537 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp5_);
#line 538 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp8_ = gxml_tnode_get_children_nodes ((GXmlTNode*) d);
#line 538 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp9_ = _tmp8_;
#line 538 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp10_ = _tmp9_;
#line 538 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp11_ = gee_list_get ((GeeList*) _tmp10_, 0);
#line 538 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp12_ = (GXmlNode*) _tmp11_;
#line 538 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp13_ = gxml_node_get_value (_tmp12_);
#line 538 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp14_ = _tmp13_;
#line 538 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp15_ = _tmp14_;
#line 538 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp15_, " Top Level Comment ") == 0, "d.children_nodes[0].value == \" Top Level Comment \"");
#line 538 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp15_);
#line 538 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp12_);
#line 538 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp10_);
#line 539 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp16_ = gxml_document_get_root ((GXmlDocument*) d);
#line 539 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp17_ = _tmp16_;
#line 539 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp18_ = _tmp17_;
#line 539 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp19_ = gxml_node_get_children_nodes (_tmp18_);
#line 539 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp20_ = _tmp19_;
#line 539 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp21_ = _tmp20_;
#line 539 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp22_ = gee_list_get ((GeeList*) _tmp21_, 2);
#line 539 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp23_ = (GXmlNode*) _tmp22_;
#line 539 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp21_);
#line 539 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp18_);
#line 539 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	a = _tmp23_;
#line 540 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp24_ = gxml_node_get_name (a);
#line 540 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp25_ = _tmp24_;
#line 540 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp26_ = _tmp25_;
#line 540 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp26_, "Authors") == 0, "a.name == \"Authors\"");
#line 540 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp26_);
#line 541 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp27_ = gxml_node_get_children_nodes (a);
#line 541 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp28_ = _tmp27_;
#line 541 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp29_ = _tmp28_;
#line 541 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp30_ = gee_list_get ((GeeList*) _tmp29_, 1);
#line 541 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp31_ = (GXmlNode*) _tmp30_;
#line 541 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp29_);
#line 541 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	a1 = _tmp31_;
#line 542 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp32_ = gxml_node_get_name (a1);
#line 542 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp33_ = _tmp32_;
#line 542 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp34_ = _tmp33_;
#line 542 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp34_, "Author") == 0, "a1.name == \"Author\"");
#line 542 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp34_);
#line 543 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp35_ = gxml_node_get_children_nodes (a1);
#line 543 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp36_ = _tmp35_;
#line 543 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp37_ = _tmp36_;
#line 543 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp38_ = gee_list_get ((GeeList*) _tmp37_, 0);
#line 543 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp39_ = (GXmlNode*) _tmp38_;
#line 543 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp39_, GXML_TYPE_COMMENT), "a1.children_nodes[0] is GXml.Comment");
#line 543 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp39_);
#line 543 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp37_);
#line 544 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp40_ = gxml_node_get_children_nodes (a1);
#line 544 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp41_ = _tmp40_;
#line 544 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp42_ = _tmp41_;
#line 544 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp43_ = gee_list_get ((GeeList*) _tmp42_, 0);
#line 544 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp44_ = (GXmlNode*) _tmp43_;
#line 544 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp45_ = gxml_node_get_value (_tmp44_);
#line 544 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp46_ = _tmp45_;
#line 544 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp47_ = _tmp46_;
#line 544 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp47_, " Inner comment") == 0, "a1.children_nodes[0].value == \" Inner comment\"");
#line 544 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp47_);
#line 544 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp44_);
#line 544 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp42_);
#line 533 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (a1);
#line 533 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (a);
#line 533 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (d);
#line 533 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (f);
#line 8537 "TDocumentTest.c"
}


static void ___lambda119__gtest_func (void) {
#line 533 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda119_ ();
#line 8544 "TDocumentTest.c"
}


static void __lambda120_ (void) {
	GError * _inner_error_ = NULL;
	{
		GFile* f = NULL;
		GFile* _tmp0_ = NULL;
		gboolean _tmp1_ = FALSE;
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp2_ = NULL;
		GeeBidirList* _tmp3_ = NULL;
		GeeBidirList* _tmp4_ = NULL;
		GeeBidirList* _tmp5_ = NULL;
		gpointer _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GeeBidirList* _tmp8_ = NULL;
		GeeBidirList* _tmp9_ = NULL;
		GeeBidirList* _tmp10_ = NULL;
		gpointer _tmp11_ = NULL;
		GXmlNode* _tmp12_ = NULL;
		GXmlProcessingInstruction* _tmp13_ = NULL;
		GXmlProcessingInstruction* _tmp14_ = NULL;
		gchar* _tmp15_ = NULL;
		gchar* _tmp16_ = NULL;
		gchar* _tmp17_ = NULL;
		GeeBidirList* _tmp18_ = NULL;
		GeeBidirList* _tmp19_ = NULL;
		GeeBidirList* _tmp20_ = NULL;
		gpointer _tmp21_ = NULL;
		GXmlNode* _tmp22_ = NULL;
		gchar* _tmp23_ = NULL;
		gchar* _tmp24_ = NULL;
		gchar* _tmp25_ = NULL;
		GXmlNode* _tmp26_ = NULL;
		GXmlNode* _tmp27_ = NULL;
		GXmlNode* _tmp28_ = NULL;
		GeeBidirList* _tmp29_ = NULL;
		GeeBidirList* _tmp30_ = NULL;
		GeeBidirList* _tmp31_ = NULL;
		gint _tmp32_ = 0;
		gint _tmp33_ = 0;
		GXmlNode* p = NULL;
		GXmlNode* _tmp34_ = NULL;
		GXmlNode* _tmp35_ = NULL;
		GXmlNode* _tmp36_ = NULL;
		GeeBidirList* _tmp37_ = NULL;
		GeeBidirList* _tmp38_ = NULL;
		GeeBidirList* _tmp39_ = NULL;
		gpointer _tmp40_ = NULL;
		GXmlNode* _tmp41_ = NULL;
		gchar* _tmp42_ = NULL;
		gchar* _tmp43_ = NULL;
		gchar* _tmp44_ = NULL;
		gchar* _tmp45_ = NULL;
		gchar* _tmp46_ = NULL;
		gchar* _tmp47_ = NULL;
#line 549 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp0_ = g_file_new_for_path (TEST_DIR "/t-read-test.xml");
#line 549 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f = _tmp0_;
#line 550 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp1_ = g_file_query_exists (f, NULL);
#line 550 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp1_, "f.query_exists ()");
#line 551 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp2_ = gxml_tdocument_new_from_file (f);
#line 551 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		d = _tmp2_;
#line 552 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_tdocument_read_doc ((GXmlDocument*) d, f, NULL, NULL, &_inner_error_);
#line 552 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 552 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 552 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 8622 "TDocumentTest.c"
			goto __catch135_g_error;
		}
#line 553 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp3_ = gxml_tnode_get_children_nodes ((GXmlTNode*) d);
#line 553 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp4_ = _tmp3_;
#line 553 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp5_ = _tmp4_;
#line 553 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp6_ = gee_list_get ((GeeList*) _tmp5_, 1);
#line 553 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp7_ = (GXmlNode*) _tmp6_;
#line 553 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp7_, GXML_TYPE_PROCESSING_INSTRUCTION), "d.children_nodes[1] is GXml.ProcessingInstruction");
#line 553 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp7_);
#line 553 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp5_);
#line 554 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp8_ = gxml_tnode_get_children_nodes ((GXmlTNode*) d);
#line 554 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp9_ = _tmp8_;
#line 554 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp10_ = _tmp9_;
#line 554 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp11_ = gee_list_get ((GeeList*) _tmp10_, 1);
#line 554 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp12_ = (GXmlNode*) _tmp11_;
#line 554 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp13_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp12_, GXML_TYPE_PROCESSING_INSTRUCTION) ? ((GXmlProcessingInstruction*) _tmp12_) : NULL;
#line 554 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (_tmp13_ == NULL) {
#line 554 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (_tmp12_);
#line 8657 "TDocumentTest.c"
		}
#line 554 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp14_ = _tmp13_;
#line 554 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp15_ = gxml_processing_instruction_get_target (_tmp14_);
#line 554 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp16_ = _tmp15_;
#line 554 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp17_ = _tmp16_;
#line 554 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp17_, "target") == 0, "(d.children_nodes[1] as GXml.ProcessingInstruction).target == \"target\"");
#line 554 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp17_);
#line 554 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp14_);
#line 554 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp10_);
#line 555 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp18_ = gxml_tnode_get_children_nodes ((GXmlTNode*) d);
#line 555 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp19_ = _tmp18_;
#line 555 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp20_ = _tmp19_;
#line 555 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp21_ = gee_list_get ((GeeList*) _tmp20_, 1);
#line 555 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp22_ = (GXmlNode*) _tmp21_;
#line 555 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp23_ = gxml_node_get_value (_tmp22_);
#line 555 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp24_ = _tmp23_;
#line 555 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp25_ = _tmp24_;
#line 555 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp25_, "Content in target id=\"something\"") == 0, "d.children_nodes[1].value == \"Content in target id=\\\"something\\\"\"");
#line 555 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp25_);
#line 555 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp22_);
#line 555 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp20_);
#line 562 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp26_ = gxml_document_get_root ((GXmlDocument*) d);
#line 562 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp27_ = _tmp26_;
#line 562 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp28_ = _tmp27_;
#line 562 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp29_ = gxml_node_get_children_nodes (_tmp28_);
#line 562 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp30_ = _tmp29_;
#line 562 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp31_ = _tmp30_;
#line 562 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp32_ = gee_collection_get_size ((GeeCollection*) _tmp31_);
#line 562 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp33_ = _tmp32_;
#line 562 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp33_ == 7, "d.root.children_nodes.size == 7");
#line 562 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp31_);
#line 562 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp28_);
#line 563 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp34_ = gxml_document_get_root ((GXmlDocument*) d);
#line 563 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp35_ = _tmp34_;
#line 563 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp36_ = _tmp35_;
#line 563 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp37_ = gxml_node_get_children_nodes (_tmp36_);
#line 563 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp38_ = _tmp37_;
#line 563 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp39_ = _tmp38_;
#line 563 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp40_ = gee_list_get ((GeeList*) _tmp39_, 4);
#line 563 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp41_ = (GXmlNode*) _tmp40_;
#line 563 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp39_);
#line 563 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp36_);
#line 563 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		p = _tmp41_;
#line 564 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (p != NULL, "p != null");
#line 565 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (p, GXML_TYPE_PROCESSING_INSTRUCTION), "p is GXml.ProcessingInstruction");
#line 566 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp42_ = gxml_processing_instruction_get_target (G_TYPE_CHECK_INSTANCE_TYPE (p, GXML_TYPE_PROCESSING_INSTRUCTION) ? ((GXmlProcessingInstruction*) p) : NULL);
#line 566 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp43_ = _tmp42_;
#line 566 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp44_ = _tmp43_;
#line 566 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp44_, "css") == 0, "(p as GXml.ProcessingInstruction).target == \"css\"");
#line 566 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp44_);
#line 567 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp45_ = gxml_node_get_value ((GXmlNode*) (G_TYPE_CHECK_INSTANCE_TYPE (p, GXML_TYPE_PROCESSING_INSTRUCTION) ? ((GXmlProcessingInstruction*) p) : NULL));
#line 567 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp46_ = _tmp45_;
#line 567 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp47_ = _tmp46_;
#line 567 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp47_, "href=\"http://www.gnome.org\"") == 0, "(p as GXml.ProcessingInstruction).value == \"href=\\\"http://www.gnome.org\\\"\"");
#line 567 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp47_);
#line 548 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (p);
#line 548 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (d);
#line 548 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (f);
#line 8773 "TDocumentTest.c"
	}
	goto __finally135;
	__catch135_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp48_ = NULL;
		gchar* _tmp49_ = NULL;
		gchar* _tmp50_ = NULL;
#line 548 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _inner_error_;
#line 548 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_inner_error_ = NULL;
#line 569 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp48_ = e->message;
#line 569 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp49_ = g_strconcat ("ERROR: ", _tmp48_, NULL);
#line 569 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp50_ = _tmp49_;
#line 569 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_message ("TDocumentTest.vala:569: %s", _tmp50_);
#line 569 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp50_);
#line 569 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_assert_not_reached ();
#line 548 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_error_free0 (e);
#line 8800 "TDocumentTest.c"
	}
	__finally135:
#line 548 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 548 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 548 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 548 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		return;
#line 8811 "TDocumentTest.c"
	}
}


static void ___lambda120__gtest_func (void) {
#line 547 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda120_ ();
#line 8819 "TDocumentTest.c"
}


static void __lambda121_ (void) {
	GError * _inner_error_ = NULL;
	{
		GFile* f = NULL;
		GFile* _tmp0_ = NULL;
		gboolean _tmp1_ = FALSE;
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp2_ = NULL;
		GXmlNode* _tmp3_ = NULL;
		GXmlNode* _tmp4_ = NULL;
		GXmlNode* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GeeBidirList* _tmp7_ = NULL;
		GeeBidirList* _tmp8_ = NULL;
		gint _tmp9_ = 0;
		gint _tmp10_ = 0;
		GXmlNode* p = NULL;
		GXmlNode* _tmp11_ = NULL;
		GXmlNode* _tmp12_ = NULL;
		GXmlNode* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GeeBidirList* _tmp15_ = NULL;
		GeeBidirList* _tmp16_ = NULL;
		gpointer _tmp17_ = NULL;
		GXmlNode* _tmp18_ = NULL;
		gchar* _tmp19_ = NULL;
		gchar* _tmp20_ = NULL;
		gchar* _tmp21_ = NULL;
#line 573 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp0_ = g_file_new_for_path (TEST_DIR "/t-read-test.xml");
#line 573 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f = _tmp0_;
#line 574 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp1_ = g_file_query_exists (f, NULL);
#line 574 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp1_, "f.query_exists ()");
#line 575 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp2_ = gxml_tdocument_new_from_file (f);
#line 575 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		d = _tmp2_;
#line 576 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_tdocument_read_doc ((GXmlDocument*) d, f, NULL, NULL, &_inner_error_);
#line 576 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 576 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 576 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 8871 "TDocumentTest.c"
			goto __catch136_g_error;
		}
#line 577 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp3_ = gxml_document_get_root ((GXmlDocument*) d);
#line 577 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp4_ = _tmp3_;
#line 577 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp5_ = _tmp4_;
#line 577 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp6_ = gxml_node_get_children_nodes (_tmp5_);
#line 577 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp7_ = _tmp6_;
#line 577 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp8_ = _tmp7_;
#line 577 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp9_ = gee_collection_get_size ((GeeCollection*) _tmp8_);
#line 577 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp10_ = _tmp9_;
#line 577 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp10_ == 7, "d.root.children_nodes.size == 7");
#line 577 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp8_);
#line 577 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp5_);
#line 578 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp11_ = gxml_document_get_root ((GXmlDocument*) d);
#line 578 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp12_ = _tmp11_;
#line 578 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp13_ = _tmp12_;
#line 578 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp14_ = gxml_node_get_children_nodes (_tmp13_);
#line 578 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp15_ = _tmp14_;
#line 578 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp16_ = _tmp15_;
#line 578 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp17_ = gee_list_get ((GeeList*) _tmp16_, 5);
#line 578 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp18_ = (GXmlNode*) _tmp17_;
#line 578 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp16_);
#line 578 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp13_);
#line 578 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		p = _tmp18_;
#line 579 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (p != NULL, "p != null");
#line 580 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (p, GXML_TYPE_CDATA), "p is GXml.CDATA");
#line 581 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp19_ = gxml_cdata_get_str (G_TYPE_CHECK_INSTANCE_TYPE (p, GXML_TYPE_CDATA) ? ((GXmlCDATA*) p) : NULL);
#line 581 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp20_ = _tmp19_;
#line 581 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp21_ = _tmp20_;
#line 581 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp21_, "<greeting>Hello, world!</greeting>") == 0, "(p as GXml.CDATA).str == \"<greeting>Hello, world!</greeting>\"");
#line 581 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp21_);
#line 572 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (p);
#line 572 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (d);
#line 572 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (f);
#line 8938 "TDocumentTest.c"
	}
	goto __finally136;
	__catch136_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp22_ = NULL;
		gchar* _tmp23_ = NULL;
		gchar* _tmp24_ = NULL;
#line 572 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _inner_error_;
#line 572 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_inner_error_ = NULL;
#line 583 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp22_ = e->message;
#line 583 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp23_ = g_strconcat ("ERROR: ", _tmp22_, NULL);
#line 583 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp24_ = _tmp23_;
#line 583 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_message ("TDocumentTest.vala:583: %s", _tmp24_);
#line 583 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp24_);
#line 583 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_assert_not_reached ();
#line 572 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_error_free0 (e);
#line 8965 "TDocumentTest.c"
	}
	__finally136:
#line 572 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 572 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 572 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 572 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		return;
#line 8976 "TDocumentTest.c"
	}
}


static void ___lambda121__gtest_func (void) {
#line 571 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda121_ ();
#line 8984 "TDocumentTest.c"
}


static void __lambda122_ (void) {
	GError * _inner_error_ = NULL;
	{
		GFile* rf = NULL;
		GFile* _tmp0_ = NULL;
		GFile* _tmp1_ = NULL;
		gboolean _tmp2_ = FALSE;
		GXmlTDocument* d = NULL;
		GFile* _tmp3_ = NULL;
		gchar* _tmp4_ = NULL;
		gchar* _tmp5_ = NULL;
		GXmlTDocument* _tmp6_ = NULL;
		GXmlTDocument* _tmp7_ = NULL;
		GXmlTDocument* _tmp8_ = NULL;
		GXmlTDocument* _tmp9_ = NULL;
		GXmlNode* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GXmlNode* _tmp12_ = NULL;
		GXmlTDocument* _tmp13_ = NULL;
		GXmlNode* _tmp14_ = NULL;
		GXmlNode* _tmp15_ = NULL;
		GXmlNode* _tmp16_ = NULL;
		gchar* _tmp17_ = NULL;
		gchar* _tmp18_ = NULL;
		gchar* _tmp19_ = NULL;
		gboolean fname = FALSE;
		gboolean fshordesc = FALSE;
		gboolean fdescription = FALSE;
		gboolean fhomepage = FALSE;
		gboolean _tmp20_ = FALSE;
		gboolean _tmp21_ = FALSE;
		gboolean _tmp22_ = FALSE;
		gboolean _tmp59_ = FALSE;
		gboolean _tmp60_ = FALSE;
		gboolean _tmp61_ = FALSE;
		gboolean _tmp62_ = FALSE;
		GFile* f = NULL;
		GFile* _tmp63_ = NULL;
		GXmlTDocument* _tmp64_ = NULL;
		GFile* _tmp65_ = NULL;
		GFile* _tmp66_ = NULL;
		gboolean _tmp67_ = FALSE;
		GFile* _tmp68_ = NULL;
#line 587 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp0_ = g_file_new_for_uri ("https://git.gnome.org/browse/gxml/plain/gxml.doap");
#line 587 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		rf = _tmp0_;
#line 588 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp1_ = rf;
#line 588 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp2_ = g_file_query_exists (_tmp1_, NULL);
#line 588 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (!_tmp2_) {
#line 589 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			g_message ("TDocumentTest.vala:589: No remote file available. Skiping...");
#line 590 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (rf);
#line 590 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			return;
#line 9047 "TDocumentTest.c"
		}
#line 592 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp3_ = rf;
#line 592 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp4_ = g_file_get_uri (_tmp3_);
#line 592 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp5_ = _tmp4_;
#line 592 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp6_ = gxml_tdocument_new_from_uri (_tmp5_);
#line 592 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp7_ = _tmp6_;
#line 592 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp5_);
#line 592 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		d = _tmp7_;
#line 593 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp8_ = d;
#line 593 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp8_ != NULL, "d != null");
#line 594 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp9_ = d;
#line 594 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp10_ = gxml_document_get_root ((GXmlDocument*) _tmp9_);
#line 594 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp11_ = _tmp10_;
#line 594 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp12_ = _tmp11_;
#line 594 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp12_ != NULL, "d.root != null");
#line 594 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp12_);
#line 595 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp13_ = d;
#line 595 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp14_ = gxml_document_get_root ((GXmlDocument*) _tmp13_);
#line 595 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp15_ = _tmp14_;
#line 595 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp16_ = _tmp15_;
#line 595 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp17_ = gxml_node_get_name (_tmp16_);
#line 595 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp18_ = _tmp17_;
#line 595 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp19_ = _tmp18_;
#line 595 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp19_, "Project") == 0, "d.root.name == \"Project\"");
#line 595 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp19_);
#line 595 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp16_);
#line 597 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		fhomepage = FALSE;
#line 597 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp20_ = fhomepage;
#line 597 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		fdescription = _tmp20_;
#line 597 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp21_ = fdescription;
#line 597 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		fshordesc = _tmp21_;
#line 597 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp22_ = fshordesc;
#line 597 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		fname = _tmp22_;
#line 9113 "TDocumentTest.c"
		{
			GeeBidirList* _n_list = NULL;
			GXmlTDocument* _tmp23_ = NULL;
			GXmlNode* _tmp24_ = NULL;
			GXmlNode* _tmp25_ = NULL;
			GXmlNode* _tmp26_ = NULL;
			GeeBidirList* _tmp27_ = NULL;
			GeeBidirList* _tmp28_ = NULL;
			GeeBidirList* _tmp29_ = NULL;
			gint _n_size = 0;
			GeeBidirList* _tmp30_ = NULL;
			gint _tmp31_ = 0;
			gint _tmp32_ = 0;
			gint _n_index = 0;
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp23_ = d;
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp24_ = gxml_document_get_root ((GXmlDocument*) _tmp23_);
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp25_ = _tmp24_;
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp26_ = _tmp25_;
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp27_ = gxml_node_get_children_nodes (_tmp26_);
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp28_ = _tmp27_;
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp29_ = _tmp28_;
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (_tmp26_);
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_n_list = _tmp29_;
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp30_ = _n_list;
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp31_ = gee_collection_get_size ((GeeCollection*) _tmp30_);
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_tmp32_ = _tmp31_;
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_n_size = _tmp32_;
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_n_index = -1;
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			while (TRUE) {
#line 9158 "TDocumentTest.c"
				gint _tmp33_ = 0;
				gint _tmp34_ = 0;
				gint _tmp35_ = 0;
				GXmlNode* n = NULL;
				GeeBidirList* _tmp36_ = NULL;
				gint _tmp37_ = 0;
				gpointer _tmp38_ = NULL;
				GXmlNode* _tmp39_ = NULL;
				gchar* _tmp40_ = NULL;
				gchar* _tmp41_ = NULL;
				gchar* _tmp42_ = NULL;
				gboolean _tmp43_ = FALSE;
				GXmlNode* _tmp44_ = NULL;
				gchar* _tmp45_ = NULL;
				gchar* _tmp46_ = NULL;
				gchar* _tmp47_ = NULL;
				gboolean _tmp48_ = FALSE;
				GXmlNode* _tmp49_ = NULL;
				gchar* _tmp50_ = NULL;
				gchar* _tmp51_ = NULL;
				gchar* _tmp52_ = NULL;
				gboolean _tmp53_ = FALSE;
				GXmlNode* _tmp54_ = NULL;
				gchar* _tmp55_ = NULL;
				gchar* _tmp56_ = NULL;
				gchar* _tmp57_ = NULL;
				gboolean _tmp58_ = FALSE;
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp33_ = _n_index;
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_n_index = _tmp33_ + 1;
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp34_ = _n_index;
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp35_ = _n_size;
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				if (!(_tmp34_ < _tmp35_)) {
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					break;
#line 9198 "TDocumentTest.c"
				}
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp36_ = _n_list;
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp37_ = _n_index;
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp38_ = gee_list_get ((GeeList*) _tmp36_, _tmp37_);
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				n = (GXmlNode*) _tmp38_;
#line 599 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp39_ = n;
#line 599 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp40_ = gxml_node_get_name (_tmp39_);
#line 599 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp41_ = _tmp40_;
#line 599 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp42_ = _tmp41_;
#line 599 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp43_ = g_strcmp0 (_tmp42_, "name") == 0;
#line 599 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_g_free0 (_tmp42_);
#line 599 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				if (_tmp43_) {
#line 599 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					fname = TRUE;
#line 9224 "TDocumentTest.c"
				}
#line 600 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp44_ = n;
#line 600 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp45_ = gxml_node_get_name (_tmp44_);
#line 600 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp46_ = _tmp45_;
#line 600 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp47_ = _tmp46_;
#line 600 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp48_ = g_strcmp0 (_tmp47_, "shortdesc") == 0;
#line 600 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_g_free0 (_tmp47_);
#line 600 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				if (_tmp48_) {
#line 600 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					fshordesc = TRUE;
#line 9242 "TDocumentTest.c"
				}
#line 601 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp49_ = n;
#line 601 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp50_ = gxml_node_get_name (_tmp49_);
#line 601 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp51_ = _tmp50_;
#line 601 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp52_ = _tmp51_;
#line 601 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp53_ = g_strcmp0 (_tmp52_, "description") == 0;
#line 601 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_g_free0 (_tmp52_);
#line 601 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				if (_tmp53_) {
#line 601 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					fdescription = TRUE;
#line 9260 "TDocumentTest.c"
				}
#line 602 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp54_ = n;
#line 602 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp55_ = gxml_node_get_name (_tmp54_);
#line 602 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp56_ = _tmp55_;
#line 602 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp57_ = _tmp56_;
#line 602 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_tmp58_ = g_strcmp0 (_tmp57_, "homepage") == 0;
#line 602 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_g_free0 (_tmp57_);
#line 602 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				if (_tmp58_) {
#line 602 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
					fhomepage = TRUE;
#line 9278 "TDocumentTest.c"
				}
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
				_g_object_unref0 (n);
#line 9282 "TDocumentTest.c"
			}
#line 598 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (_n_list);
#line 9286 "TDocumentTest.c"
		}
#line 604 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp59_ = fname;
#line 604 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp59_, "fname");
#line 605 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp60_ = fshordesc;
#line 605 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp60_, "fshordesc");
#line 606 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp61_ = fdescription;
#line 606 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp61_, "fdescription");
#line 607 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp62_ = fhomepage;
#line 607 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp62_, "fhomepage");
#line 608 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp63_ = g_file_new_for_path (TEST_SAVE_DIR "/xml.doap");
#line 608 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f = _tmp63_;
#line 609 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp64_ = d;
#line 609 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp65_ = f;
#line 609 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_document_save_as ((GXmlDocument*) _tmp64_, _tmp65_, NULL, &_inner_error_);
#line 609 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 609 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 609 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 609 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (rf);
#line 9322 "TDocumentTest.c"
			goto __catch137_g_error;
		}
#line 610 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp66_ = f;
#line 610 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp67_ = g_file_query_exists (_tmp66_, NULL);
#line 610 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp67_, "f.query_exists ()");
#line 611 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp68_ = f;
#line 611 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_file_delete (_tmp68_, NULL, &_inner_error_);
#line 611 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 611 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 611 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 611 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (rf);
#line 9343 "TDocumentTest.c"
			goto __catch137_g_error;
		}
#line 586 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (f);
#line 586 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (d);
#line 586 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (rf);
#line 9352 "TDocumentTest.c"
	}
	goto __finally137;
	__catch137_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp69_ = NULL;
		gchar* _tmp70_ = NULL;
		gchar* _tmp71_ = NULL;
#line 586 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _inner_error_;
#line 586 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_inner_error_ = NULL;
#line 613 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp69_ = e->message;
#line 613 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp70_ = g_strconcat ("Error: ", _tmp69_, NULL);
#line 613 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp71_ = _tmp70_;
#line 613 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_message ("TDocumentTest.vala:613: %s", _tmp71_);
#line 613 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp71_);
#line 614 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_assert_not_reached ();
#line 586 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_error_free0 (e);
#line 9379 "TDocumentTest.c"
	}
	__finally137:
#line 586 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 586 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 586 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 586 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		return;
#line 9390 "TDocumentTest.c"
	}
}


static void ___lambda122__gtest_func (void) {
#line 585 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda122_ ();
#line 9398 "TDocumentTest.c"
}


static void __lambda123_ (void) {
	GError * _inner_error_ = NULL;
	{
		GFile* f = NULL;
		GFile* _tmp0_ = NULL;
		gboolean _tmp1_ = FALSE;
		GFileInputStream* _tmp2_ = NULL;
		GFileInputStream* _tmp3_ = NULL;
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp4_ = NULL;
		GXmlNode* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlNode* _tmp8_ = NULL;
		GXmlNode* _tmp9_ = NULL;
		GXmlNode* _tmp10_ = NULL;
		gchar* _tmp11_ = NULL;
		gchar* _tmp12_ = NULL;
		gchar* _tmp13_ = NULL;
		GXmlNode* _tmp14_ = NULL;
		GXmlNode* _tmp15_ = NULL;
		GXmlNode* _tmp16_ = NULL;
		GeeMap* _tmp17_ = NULL;
		GeeMap* _tmp18_ = NULL;
		GeeMap* _tmp19_ = NULL;
		gpointer _tmp20_ = NULL;
		GXmlNode* _tmp21_ = NULL;
		GXmlNode* _tmp22_ = NULL;
		GXmlNode* _tmp23_ = NULL;
		GXmlNode* _tmp24_ = NULL;
		GeeMap* _tmp25_ = NULL;
		GeeMap* _tmp26_ = NULL;
		GeeMap* _tmp27_ = NULL;
		gpointer _tmp28_ = NULL;
		GXmlNode* _tmp29_ = NULL;
		gchar* _tmp30_ = NULL;
		gchar* _tmp31_ = NULL;
		gchar* _tmp32_ = NULL;
		GXmlNode* _tmp33_ = NULL;
		GXmlNode* _tmp34_ = NULL;
		GXmlNode* _tmp35_ = NULL;
		GeeBidirList* _tmp36_ = NULL;
		GeeBidirList* _tmp37_ = NULL;
		GeeBidirList* _tmp38_ = NULL;
		gint _tmp39_ = 0;
		gint _tmp40_ = 0;
		GXmlNode* s1 = NULL;
		GXmlNode* _tmp41_ = NULL;
		GXmlNode* _tmp42_ = NULL;
		GXmlNode* _tmp43_ = NULL;
		GeeBidirList* _tmp44_ = NULL;
		GeeBidirList* _tmp45_ = NULL;
		GeeBidirList* _tmp46_ = NULL;
		gpointer _tmp47_ = NULL;
		GXmlNode* _tmp48_ = NULL;
		gchar* _tmp49_ = NULL;
		gchar* _tmp50_ = NULL;
		gchar* _tmp51_ = NULL;
		GXmlNode* p1 = NULL;
		GeeMap* _tmp52_ = NULL;
		GeeMap* _tmp53_ = NULL;
		GeeMap* _tmp54_ = NULL;
		gpointer _tmp55_ = NULL;
		GXmlNode* _tmp56_ = NULL;
		gchar* _tmp57_ = NULL;
		gchar* _tmp58_ = NULL;
		gchar* _tmp59_ = NULL;
		GeeBidirList* _tmp60_ = NULL;
		GeeBidirList* _tmp61_ = NULL;
		GeeBidirList* _tmp62_ = NULL;
		gint _tmp63_ = 0;
		gint _tmp64_ = 0;
		GeeBidirList* _tmp65_ = NULL;
		GeeBidirList* _tmp66_ = NULL;
		GeeBidirList* _tmp67_ = NULL;
		gpointer _tmp68_ = NULL;
		GXmlNode* _tmp69_ = NULL;
		GeeBidirList* _tmp70_ = NULL;
		GeeBidirList* _tmp71_ = NULL;
		GeeBidirList* _tmp72_ = NULL;
		gpointer _tmp73_ = NULL;
		GXmlNode* _tmp74_ = NULL;
		gchar* _tmp75_ = NULL;
		gchar* _tmp76_ = NULL;
		gchar* _tmp77_ = NULL;
		GXmlNode* s2 = NULL;
		GXmlNode* _tmp78_ = NULL;
		GXmlNode* _tmp79_ = NULL;
		GXmlNode* _tmp80_ = NULL;
		GeeBidirList* _tmp81_ = NULL;
		GeeBidirList* _tmp82_ = NULL;
		GeeBidirList* _tmp83_ = NULL;
		gpointer _tmp84_ = NULL;
		GXmlNode* _tmp85_ = NULL;
		gchar* _tmp86_ = NULL;
		gchar* _tmp87_ = NULL;
		gchar* _tmp88_ = NULL;
		GXmlNode* p2 = NULL;
		GeeMap* _tmp89_ = NULL;
		GeeMap* _tmp90_ = NULL;
		GeeMap* _tmp91_ = NULL;
		gpointer _tmp92_ = NULL;
		GXmlNode* _tmp93_ = NULL;
		gchar* _tmp94_ = NULL;
		gchar* _tmp95_ = NULL;
		gchar* _tmp96_ = NULL;
		GeeBidirList* _tmp97_ = NULL;
		GeeBidirList* _tmp98_ = NULL;
		GeeBidirList* _tmp99_ = NULL;
		gint _tmp100_ = 0;
		gint _tmp101_ = 0;
		GeeBidirList* _tmp102_ = NULL;
		GeeBidirList* _tmp103_ = NULL;
		GeeBidirList* _tmp104_ = NULL;
		gpointer _tmp105_ = NULL;
		GXmlNode* _tmp106_ = NULL;
		GeeBidirList* _tmp107_ = NULL;
		GeeBidirList* _tmp108_ = NULL;
		GeeBidirList* _tmp109_ = NULL;
		gpointer _tmp110_ = NULL;
		GXmlNode* _tmp111_ = NULL;
		gchar* _tmp112_ = NULL;
		gchar* _tmp113_ = NULL;
		gchar* _tmp114_ = NULL;
		GXmlNode* s3 = NULL;
		GXmlNode* _tmp115_ = NULL;
		GXmlNode* _tmp116_ = NULL;
		GXmlNode* _tmp117_ = NULL;
		GeeBidirList* _tmp118_ = NULL;
		GeeBidirList* _tmp119_ = NULL;
		GeeBidirList* _tmp120_ = NULL;
		gpointer _tmp121_ = NULL;
		GXmlNode* _tmp122_ = NULL;
		gchar* _tmp123_ = NULL;
		gchar* _tmp124_ = NULL;
		gchar* _tmp125_ = NULL;
		GXmlNode* p3 = NULL;
		GeeMap* _tmp126_ = NULL;
		GeeMap* _tmp127_ = NULL;
		GeeMap* _tmp128_ = NULL;
		gpointer _tmp129_ = NULL;
		GXmlNode* _tmp130_ = NULL;
		gchar* _tmp131_ = NULL;
		gchar* _tmp132_ = NULL;
		gchar* _tmp133_ = NULL;
		GXmlNode* p31 = NULL;
		GeeMap* _tmp134_ = NULL;
		GeeMap* _tmp135_ = NULL;
		GeeMap* _tmp136_ = NULL;
		gpointer _tmp137_ = NULL;
		GXmlNode* _tmp138_ = NULL;
		gchar* _tmp139_ = NULL;
		gchar* _tmp140_ = NULL;
		gchar* _tmp141_ = NULL;
		GeeBidirList* _tmp142_ = NULL;
		GeeBidirList* _tmp143_ = NULL;
		GeeBidirList* _tmp144_ = NULL;
		gint _tmp145_ = 0;
		gint _tmp146_ = 0;
		GeeBidirList* _tmp147_ = NULL;
		GeeBidirList* _tmp148_ = NULL;
		GeeBidirList* _tmp149_ = NULL;
		gpointer _tmp150_ = NULL;
		GXmlNode* _tmp151_ = NULL;
		GeeBidirList* _tmp152_ = NULL;
		GeeBidirList* _tmp153_ = NULL;
		GeeBidirList* _tmp154_ = NULL;
		gpointer _tmp155_ = NULL;
		GXmlNode* _tmp156_ = NULL;
		gchar* _tmp157_ = NULL;
		gchar* _tmp158_ = NULL;
		gchar* _tmp159_ = NULL;
		GeeBidirList* _tmp160_ = NULL;
		GeeBidirList* _tmp161_ = NULL;
		GeeBidirList* _tmp162_ = NULL;
		gpointer _tmp163_ = NULL;
		GXmlNode* _tmp164_ = NULL;
		gchar* _tmp165_ = NULL;
		gchar* _tmp166_ = NULL;
		gchar* _tmp167_ = NULL;
		GXmlNode* a1 = NULL;
		GeeBidirList* _tmp168_ = NULL;
		GeeBidirList* _tmp169_ = NULL;
		GeeBidirList* _tmp170_ = NULL;
		gpointer _tmp171_ = NULL;
		GXmlNode* _tmp172_ = NULL;
		gchar* _tmp173_ = NULL;
		gchar* _tmp174_ = NULL;
		gchar* _tmp175_ = NULL;
		GeeBidirList* _tmp176_ = NULL;
		GeeBidirList* _tmp177_ = NULL;
		GeeBidirList* _tmp178_ = NULL;
		gint _tmp179_ = 0;
		gint _tmp180_ = 0;
		GeeBidirList* _tmp181_ = NULL;
		GeeBidirList* _tmp182_ = NULL;
		GeeBidirList* _tmp183_ = NULL;
		gpointer _tmp184_ = NULL;
		GXmlNode* _tmp185_ = NULL;
		gchar* _tmp186_ = NULL;
		gchar* _tmp187_ = NULL;
		gchar* _tmp188_ = NULL;
		GeeBidirList* _tmp189_ = NULL;
		GeeBidirList* _tmp190_ = NULL;
		GeeBidirList* _tmp191_ = NULL;
		gpointer _tmp192_ = NULL;
		GXmlNode* _tmp193_ = NULL;
		GeeBidirList* _tmp194_ = NULL;
		GeeBidirList* _tmp195_ = NULL;
		GeeBidirList* _tmp196_ = NULL;
		gint _tmp197_ = 0;
		gint _tmp198_ = 0;
		GeeBidirList* _tmp199_ = NULL;
		GeeBidirList* _tmp200_ = NULL;
		GeeBidirList* _tmp201_ = NULL;
		gpointer _tmp202_ = NULL;
		GXmlNode* _tmp203_ = NULL;
		GeeBidirList* _tmp204_ = NULL;
		GeeBidirList* _tmp205_ = NULL;
		GeeBidirList* _tmp206_ = NULL;
		gpointer _tmp207_ = NULL;
		GXmlNode* _tmp208_ = NULL;
		GeeBidirList* _tmp209_ = NULL;
		GeeBidirList* _tmp210_ = NULL;
		GeeBidirList* _tmp211_ = NULL;
		gpointer _tmp212_ = NULL;
		GXmlNode* _tmp213_ = NULL;
		GeeBidirList* _tmp214_ = NULL;
		GeeBidirList* _tmp215_ = NULL;
		GeeBidirList* _tmp216_ = NULL;
		gpointer _tmp217_ = NULL;
		GXmlNode* _tmp218_ = NULL;
		gchar* _tmp219_ = NULL;
		gchar* _tmp220_ = NULL;
		gchar* _tmp221_ = NULL;
		GeeBidirList* _tmp222_ = NULL;
		GeeBidirList* _tmp223_ = NULL;
		GeeBidirList* _tmp224_ = NULL;
		gpointer _tmp225_ = NULL;
		GXmlNode* _tmp226_ = NULL;
		gchar* _tmp227_ = NULL;
		gchar* _tmp228_ = NULL;
		gchar* _tmp229_ = NULL;
		GeeBidirList* _tmp230_ = NULL;
		GeeBidirList* _tmp231_ = NULL;
		GeeBidirList* _tmp232_ = NULL;
		gpointer _tmp233_ = NULL;
		GXmlNode* _tmp234_ = NULL;
		GeeBidirList* _tmp235_ = NULL;
		GeeBidirList* _tmp236_ = NULL;
		GeeBidirList* _tmp237_ = NULL;
		gint _tmp238_ = 0;
		gint _tmp239_ = 0;
		GeeBidirList* _tmp240_ = NULL;
		GeeBidirList* _tmp241_ = NULL;
		GeeBidirList* _tmp242_ = NULL;
		gpointer _tmp243_ = NULL;
		GXmlNode* _tmp244_ = NULL;
		GeeBidirList* _tmp245_ = NULL;
		GeeBidirList* _tmp246_ = NULL;
		GeeBidirList* _tmp247_ = NULL;
		gpointer _tmp248_ = NULL;
		GXmlNode* _tmp249_ = NULL;
		GeeBidirList* _tmp250_ = NULL;
		GeeBidirList* _tmp251_ = NULL;
		GeeBidirList* _tmp252_ = NULL;
		gpointer _tmp253_ = NULL;
		GXmlNode* _tmp254_ = NULL;
		GeeBidirList* _tmp255_ = NULL;
		GeeBidirList* _tmp256_ = NULL;
		GeeBidirList* _tmp257_ = NULL;
		gpointer _tmp258_ = NULL;
		GXmlNode* _tmp259_ = NULL;
		gchar* _tmp260_ = NULL;
		gchar* _tmp261_ = NULL;
		gchar* _tmp262_ = NULL;
		GXmlNode* a2 = NULL;
		GeeBidirList* _tmp263_ = NULL;
		GeeBidirList* _tmp264_ = NULL;
		GeeBidirList* _tmp265_ = NULL;
		gpointer _tmp266_ = NULL;
		GXmlNode* _tmp267_ = NULL;
		GeeBidirList* _tmp268_ = NULL;
		GeeBidirList* _tmp269_ = NULL;
		GeeBidirList* _tmp270_ = NULL;
		gint _tmp271_ = 0;
		gint _tmp272_ = 0;
		GeeBidirList* _tmp273_ = NULL;
		GeeBidirList* _tmp274_ = NULL;
		GeeBidirList* _tmp275_ = NULL;
		gpointer _tmp276_ = NULL;
		GXmlNode* _tmp277_ = NULL;
		gchar* _tmp278_ = NULL;
		gchar* _tmp279_ = NULL;
		gchar* _tmp280_ = NULL;
		GeeBidirList* _tmp281_ = NULL;
		GeeBidirList* _tmp282_ = NULL;
		GeeBidirList* _tmp283_ = NULL;
		gpointer _tmp284_ = NULL;
		GXmlNode* _tmp285_ = NULL;
		GeeBidirList* _tmp286_ = NULL;
		GeeBidirList* _tmp287_ = NULL;
		GeeBidirList* _tmp288_ = NULL;
		gint _tmp289_ = 0;
		gint _tmp290_ = 0;
		GeeBidirList* _tmp291_ = NULL;
		GeeBidirList* _tmp292_ = NULL;
		GeeBidirList* _tmp293_ = NULL;
		gpointer _tmp294_ = NULL;
		GXmlNode* _tmp295_ = NULL;
		GeeBidirList* _tmp296_ = NULL;
		GeeBidirList* _tmp297_ = NULL;
		GeeBidirList* _tmp298_ = NULL;
		gpointer _tmp299_ = NULL;
		GXmlNode* _tmp300_ = NULL;
		GeeBidirList* _tmp301_ = NULL;
		GeeBidirList* _tmp302_ = NULL;
		GeeBidirList* _tmp303_ = NULL;
		gpointer _tmp304_ = NULL;
		GXmlNode* _tmp305_ = NULL;
		GeeBidirList* _tmp306_ = NULL;
		GeeBidirList* _tmp307_ = NULL;
		GeeBidirList* _tmp308_ = NULL;
		gpointer _tmp309_ = NULL;
		GXmlNode* _tmp310_ = NULL;
		gchar* _tmp311_ = NULL;
		gchar* _tmp312_ = NULL;
		gchar* _tmp313_ = NULL;
		GeeBidirList* _tmp314_ = NULL;
		GeeBidirList* _tmp315_ = NULL;
		GeeBidirList* _tmp316_ = NULL;
		gpointer _tmp317_ = NULL;
		GXmlNode* _tmp318_ = NULL;
		gchar* _tmp319_ = NULL;
		gchar* _tmp320_ = NULL;
		gchar* _tmp321_ = NULL;
		GeeBidirList* _tmp322_ = NULL;
		GeeBidirList* _tmp323_ = NULL;
		GeeBidirList* _tmp324_ = NULL;
		gpointer _tmp325_ = NULL;
		GXmlNode* _tmp326_ = NULL;
		GeeBidirList* _tmp327_ = NULL;
		GeeBidirList* _tmp328_ = NULL;
		GeeBidirList* _tmp329_ = NULL;
		gint _tmp330_ = 0;
		gint _tmp331_ = 0;
		GeeBidirList* _tmp332_ = NULL;
		GeeBidirList* _tmp333_ = NULL;
		GeeBidirList* _tmp334_ = NULL;
		gpointer _tmp335_ = NULL;
		GXmlNode* _tmp336_ = NULL;
		GeeBidirList* _tmp337_ = NULL;
		GeeBidirList* _tmp338_ = NULL;
		GeeBidirList* _tmp339_ = NULL;
		gpointer _tmp340_ = NULL;
		GXmlNode* _tmp341_ = NULL;
		GeeBidirList* _tmp342_ = NULL;
		GeeBidirList* _tmp343_ = NULL;
		GeeBidirList* _tmp344_ = NULL;
		gpointer _tmp345_ = NULL;
		GXmlNode* _tmp346_ = NULL;
		GeeBidirList* _tmp347_ = NULL;
		GeeBidirList* _tmp348_ = NULL;
		GeeBidirList* _tmp349_ = NULL;
		gpointer _tmp350_ = NULL;
		GXmlNode* _tmp351_ = NULL;
		gchar* _tmp352_ = NULL;
		gchar* _tmp353_ = NULL;
		gchar* _tmp354_ = NULL;
#line 619 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp0_ = g_file_new_for_path (TEST_DIR "/t-read-test.xml");
#line 619 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f = _tmp0_;
#line 620 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp1_ = g_file_query_exists (f, NULL);
#line 620 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp1_, "f.query_exists ()");
#line 621 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp3_ = g_file_read (f, NULL, &_inner_error_);
#line 621 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp2_ = _tmp3_;
#line 621 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 621 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (f);
#line 9787 "TDocumentTest.c"
			goto __catch138_g_error;
		}
#line 621 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp4_ = gxml_tdocument_new_from_stream ((GInputStream*) _tmp2_);
#line 621 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		d = _tmp4_;
#line 623 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp5_ = gxml_document_get_root ((GXmlDocument*) d);
#line 623 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp6_ = _tmp5_;
#line 623 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp7_ = _tmp6_;
#line 623 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp7_ != NULL, "d.root != null");
#line 623 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp7_);
#line 624 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp8_ = gxml_document_get_root ((GXmlDocument*) d);
#line 624 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp9_ = _tmp8_;
#line 624 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp10_ = _tmp9_;
#line 624 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp11_ = gxml_node_get_name (_tmp10_);
#line 624 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp12_ = _tmp11_;
#line 624 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp13_ = _tmp12_;
#line 624 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp13_, "Sentences") == 0, "d.root.name == \"Sentences\"");
#line 624 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp13_);
#line 624 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp10_);
#line 625 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp14_ = gxml_document_get_root ((GXmlDocument*) d);
#line 625 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp15_ = _tmp14_;
#line 625 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp16_ = _tmp15_;
#line 625 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp17_ = gxml_node_get_attrs (_tmp16_);
#line 625 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp18_ = _tmp17_;
#line 625 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp19_ = _tmp18_;
#line 625 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp20_ = gee_map_get (_tmp19_, "audience");
#line 625 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp21_ = (GXmlNode*) _tmp20_;
#line 625 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp21_ != NULL, "d.root.attrs[\"audience\"] != null");
#line 625 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp21_);
#line 625 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp19_);
#line 625 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp16_);
#line 626 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp22_ = gxml_document_get_root ((GXmlDocument*) d);
#line 626 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp23_ = _tmp22_;
#line 626 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp24_ = _tmp23_;
#line 626 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp25_ = gxml_node_get_attrs (_tmp24_);
#line 626 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp26_ = _tmp25_;
#line 626 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp27_ = _tmp26_;
#line 626 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp28_ = gee_map_get (_tmp27_, "audience");
#line 626 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp29_ = (GXmlNode*) _tmp28_;
#line 626 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp30_ = gxml_node_get_value (_tmp29_);
#line 626 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp31_ = _tmp30_;
#line 626 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp32_ = _tmp31_;
#line 626 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp32_, "All") == 0, "d.root.attrs[\"audience\"].value == \"All\"");
#line 626 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp32_);
#line 626 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp29_);
#line 626 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp27_);
#line 626 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp24_);
#line 627 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp33_ = gxml_document_get_root ((GXmlDocument*) d);
#line 627 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp34_ = _tmp33_;
#line 627 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp35_ = _tmp34_;
#line 627 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp36_ = gxml_node_get_children_nodes (_tmp35_);
#line 627 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp37_ = _tmp36_;
#line 627 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp38_ = _tmp37_;
#line 627 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp39_ = gee_collection_get_size ((GeeCollection*) _tmp38_);
#line 627 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp40_ = _tmp39_;
#line 627 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp40_ == 7, "d.root.children_nodes.size == 7");
#line 627 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp38_);
#line 627 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp35_);
#line 628 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp41_ = gxml_document_get_root ((GXmlDocument*) d);
#line 628 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp42_ = _tmp41_;
#line 628 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp43_ = _tmp42_;
#line 628 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp44_ = gxml_node_get_children_nodes (_tmp43_);
#line 628 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp45_ = _tmp44_;
#line 628 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp46_ = _tmp45_;
#line 628 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp47_ = gee_list_get ((GeeList*) _tmp46_, 0);
#line 628 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp48_ = (GXmlNode*) _tmp47_;
#line 628 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp46_);
#line 628 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp43_);
#line 628 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		s1 = _tmp48_;
#line 629 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (s1 != NULL, "s1 != null");
#line 630 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp49_ = gxml_node_get_name (s1);
#line 630 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp50_ = _tmp49_;
#line 630 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp51_ = _tmp50_;
#line 630 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp51_, "Sentence") == 0, "s1.name == \"Sentence\"");
#line 630 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp51_);
#line 631 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp52_ = gxml_node_get_attrs (s1);
#line 631 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp53_ = _tmp52_;
#line 631 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp54_ = _tmp53_;
#line 631 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp55_ = gee_map_get (_tmp54_, "lang");
#line 631 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp56_ = (GXmlNode*) _tmp55_;
#line 631 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp54_);
#line 631 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		p1 = _tmp56_;
#line 632 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (p1 != NULL, "p1 != null");
#line 633 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp57_ = gxml_node_get_value (p1);
#line 633 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp58_ = _tmp57_;
#line 633 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp59_ = _tmp58_;
#line 633 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp59_, "en") == 0, "p1.value == \"en\"");
#line 633 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp59_);
#line 634 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp60_ = gxml_node_get_children_nodes (s1);
#line 634 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp61_ = _tmp60_;
#line 634 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp62_ = _tmp61_;
#line 634 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp63_ = gee_collection_get_size ((GeeCollection*) _tmp62_);
#line 634 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp64_ = _tmp63_;
#line 634 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp64_ == 1, "s1.children_nodes.size == 1");
#line 634 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp62_);
#line 635 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp65_ = gxml_node_get_children_nodes (s1);
#line 635 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp66_ = _tmp65_;
#line 635 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp67_ = _tmp66_;
#line 635 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp68_ = gee_list_get ((GeeList*) _tmp67_, 0);
#line 635 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp69_ = (GXmlNode*) _tmp68_;
#line 635 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp69_, GXML_TYPE_TEXT), "s1.children_nodes[0] is GXml.Text");
#line 635 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp69_);
#line 635 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp67_);
#line 636 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp70_ = gxml_node_get_children_nodes (s1);
#line 636 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp71_ = _tmp70_;
#line 636 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp72_ = _tmp71_;
#line 636 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp73_ = gee_list_get ((GeeList*) _tmp72_, 0);
#line 636 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp74_ = (GXmlNode*) _tmp73_;
#line 636 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp75_ = gxml_node_get_value (_tmp74_);
#line 636 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp76_ = _tmp75_;
#line 636 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp77_ = _tmp76_;
#line 636 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp77_, "I like the colour blue.") == 0, "s1.children_nodes[0].value == \"I like the colour blue.\"");
#line 636 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp77_);
#line 636 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp74_);
#line 636 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp72_);
#line 637 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp78_ = gxml_document_get_root ((GXmlDocument*) d);
#line 637 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp79_ = _tmp78_;
#line 637 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp80_ = _tmp79_;
#line 637 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp81_ = gxml_node_get_children_nodes (_tmp80_);
#line 637 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp82_ = _tmp81_;
#line 637 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp83_ = _tmp82_;
#line 637 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp84_ = gee_list_get ((GeeList*) _tmp83_, 1);
#line 637 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp85_ = (GXmlNode*) _tmp84_;
#line 637 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp83_);
#line 637 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp80_);
#line 637 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		s2 = _tmp85_;
#line 638 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (s2 != NULL, "s2 != null");
#line 639 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp86_ = gxml_node_get_name (s2);
#line 639 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp87_ = _tmp86_;
#line 639 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp88_ = _tmp87_;
#line 639 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp88_, "Sentence") == 0, "s2.name == \"Sentence\"");
#line 639 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp88_);
#line 640 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp89_ = gxml_node_get_attrs (s2);
#line 640 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp90_ = _tmp89_;
#line 640 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp91_ = _tmp90_;
#line 640 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp92_ = gee_map_get (_tmp91_, "lang");
#line 640 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp93_ = (GXmlNode*) _tmp92_;
#line 640 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp91_);
#line 640 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		p2 = _tmp93_;
#line 641 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (p2 != NULL, "p2 != null");
#line 642 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp94_ = gxml_node_get_value (p2);
#line 642 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp95_ = _tmp94_;
#line 642 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp96_ = _tmp95_;
#line 642 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp96_, "es") == 0, "p2.value == \"es\"");
#line 642 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp96_);
#line 643 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp97_ = gxml_node_get_children_nodes (s2);
#line 643 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp98_ = _tmp97_;
#line 643 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp99_ = _tmp98_;
#line 643 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp100_ = gee_collection_get_size ((GeeCollection*) _tmp99_);
#line 643 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp101_ = _tmp100_;
#line 643 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp101_ == 1, "s2.children_nodes.size == 1");
#line 643 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp99_);
#line 644 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp102_ = gxml_node_get_children_nodes (s2);
#line 644 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp103_ = _tmp102_;
#line 644 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp104_ = _tmp103_;
#line 644 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp105_ = gee_list_get ((GeeList*) _tmp104_, 0);
#line 644 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp106_ = (GXmlNode*) _tmp105_;
#line 644 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp106_, GXML_TYPE_TEXT), "s2.children_nodes[0] is GXml.Text");
#line 644 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp106_);
#line 644 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp104_);
#line 645 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp107_ = gxml_node_get_children_nodes (s2);
#line 645 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp108_ = _tmp107_;
#line 645 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp109_ = _tmp108_;
#line 645 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp110_ = gee_list_get ((GeeList*) _tmp109_, 0);
#line 645 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp111_ = (GXmlNode*) _tmp110_;
#line 645 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp112_ = gxml_node_get_value (_tmp111_);
#line 645 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp113_ = _tmp112_;
#line 645 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp114_ = _tmp113_;
#line 645 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp114_, "Español") == 0, "s2.children_nodes[0].value == \"Espa\303\261ol\"");
#line 645 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp114_);
#line 645 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp111_);
#line 645 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp109_);
#line 646 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp115_ = gxml_document_get_root ((GXmlDocument*) d);
#line 646 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp116_ = _tmp115_;
#line 646 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp117_ = _tmp116_;
#line 646 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp118_ = gxml_node_get_children_nodes (_tmp117_);
#line 646 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp119_ = _tmp118_;
#line 646 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp120_ = _tmp119_;
#line 646 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp121_ = gee_list_get ((GeeList*) _tmp120_, 2);
#line 646 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp122_ = (GXmlNode*) _tmp121_;
#line 646 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp120_);
#line 646 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp117_);
#line 646 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		s3 = _tmp122_;
#line 647 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (s3 != NULL, "s3 != null");
#line 648 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp123_ = gxml_node_get_name (s3);
#line 648 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp124_ = _tmp123_;
#line 648 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp125_ = _tmp124_;
#line 648 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp125_, "Authors") == 0, "s3.name == \"Authors\"");
#line 648 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp125_);
#line 649 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp126_ = gxml_node_get_attrs (s3);
#line 649 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp127_ = _tmp126_;
#line 649 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp128_ = _tmp127_;
#line 649 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp129_ = gee_map_get (_tmp128_, "year");
#line 649 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp130_ = (GXmlNode*) _tmp129_;
#line 649 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp128_);
#line 649 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		p3 = _tmp130_;
#line 650 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (p3 != NULL, "p3 != null");
#line 651 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp131_ = gxml_node_get_value (p3);
#line 651 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp132_ = _tmp131_;
#line 651 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp133_ = _tmp132_;
#line 651 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp133_, "2016") == 0, "p3.value == \"2016\"");
#line 651 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp133_);
#line 652 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp134_ = gxml_node_get_attrs (s3);
#line 652 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp135_ = _tmp134_;
#line 652 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp136_ = _tmp135_;
#line 652 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp137_ = gee_map_get (_tmp136_, "collection");
#line 652 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp138_ = (GXmlNode*) _tmp137_;
#line 652 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp136_);
#line 652 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		p31 = _tmp138_;
#line 653 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (p31 != NULL, "p31 != null");
#line 654 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp139_ = gxml_node_get_value (p31);
#line 654 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp140_ = _tmp139_;
#line 654 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp141_ = _tmp140_;
#line 654 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp141_, "Back") == 0, "p31.value == \"Back\"");
#line 654 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp141_);
#line 655 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp142_ = gxml_node_get_children_nodes (s3);
#line 655 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp143_ = _tmp142_;
#line 655 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp144_ = _tmp143_;
#line 655 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp145_ = gee_collection_get_size ((GeeCollection*) _tmp144_);
#line 655 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp146_ = _tmp145_;
#line 655 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp146_ == 2, "s3.children_nodes.size == 2");
#line 655 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp144_);
#line 656 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp147_ = gxml_node_get_children_nodes (s3);
#line 656 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp148_ = _tmp147_;
#line 656 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp149_ = _tmp148_;
#line 656 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp150_ = gee_list_get ((GeeList*) _tmp149_, 0);
#line 656 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp151_ = (GXmlNode*) _tmp150_;
#line 656 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp151_, GXML_TYPE_ELEMENT), "s3.children_nodes[0] is GXml.Element");
#line 656 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp151_);
#line 656 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp149_);
#line 657 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp152_ = gxml_node_get_children_nodes (s3);
#line 657 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp153_ = _tmp152_;
#line 657 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp154_ = _tmp153_;
#line 657 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp155_ = gee_list_get ((GeeList*) _tmp154_, 0);
#line 657 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp156_ = (GXmlNode*) _tmp155_;
#line 657 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp157_ = gxml_node_get_name (_tmp156_);
#line 657 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp158_ = _tmp157_;
#line 657 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp159_ = _tmp158_;
#line 657 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp159_, "Author") == 0, "s3.children_nodes[0].name == \"Author\"");
#line 657 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp159_);
#line 657 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp156_);
#line 657 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp154_);
#line 658 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp160_ = gxml_node_get_children_nodes (s3);
#line 658 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp161_ = _tmp160_;
#line 658 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp162_ = _tmp161_;
#line 658 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp163_ = gee_list_get ((GeeList*) _tmp162_, 1);
#line 658 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp164_ = (GXmlNode*) _tmp163_;
#line 658 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp165_ = gxml_node_get_name (_tmp164_);
#line 658 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp166_ = _tmp165_;
#line 658 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp167_ = _tmp166_;
#line 658 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp167_, "Author") == 0, "s3.children_nodes[1].name == \"Author\"");
#line 658 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp167_);
#line 658 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp164_);
#line 658 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp162_);
#line 659 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp168_ = gxml_node_get_children_nodes (s3);
#line 659 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp169_ = _tmp168_;
#line 659 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp170_ = _tmp169_;
#line 659 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp171_ = gee_list_get ((GeeList*) _tmp170_, 0);
#line 659 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp172_ = (GXmlNode*) _tmp171_;
#line 659 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp170_);
#line 659 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		a1 = _tmp172_;
#line 660 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (a1 != NULL, "a1 != null");
#line 661 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp173_ = gxml_node_get_name (a1);
#line 661 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp174_ = _tmp173_;
#line 661 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp175_ = _tmp174_;
#line 661 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp175_, "Author") == 0, "a1.name == \"Author\"");
#line 661 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp175_);
#line 662 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp176_ = gxml_node_get_children_nodes (a1);
#line 662 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp177_ = _tmp176_;
#line 662 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp178_ = _tmp177_;
#line 662 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp179_ = gee_collection_get_size ((GeeCollection*) _tmp178_);
#line 662 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp180_ = _tmp179_;
#line 662 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp180_ == 2, "a1.children_nodes.size == 2");
#line 662 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp178_);
#line 663 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp181_ = gxml_node_get_children_nodes (a1);
#line 663 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp182_ = _tmp181_;
#line 663 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp183_ = _tmp182_;
#line 663 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp184_ = gee_list_get ((GeeList*) _tmp183_, 0);
#line 663 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp185_ = (GXmlNode*) _tmp184_;
#line 663 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp186_ = gxml_node_get_name (_tmp185_);
#line 663 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp187_ = _tmp186_;
#line 663 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp188_ = _tmp187_;
#line 663 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp188_, "Name") == 0, "a1.children_nodes[0].name == \"Name\"");
#line 663 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp188_);
#line 663 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp185_);
#line 663 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp183_);
#line 664 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp189_ = gxml_node_get_children_nodes (a1);
#line 664 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp190_ = _tmp189_;
#line 664 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp191_ = _tmp190_;
#line 664 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp192_ = gee_list_get ((GeeList*) _tmp191_, 0);
#line 664 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp193_ = (GXmlNode*) _tmp192_;
#line 664 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp194_ = gxml_node_get_children_nodes (_tmp193_);
#line 664 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp195_ = _tmp194_;
#line 664 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp196_ = _tmp195_;
#line 664 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp197_ = gee_collection_get_size ((GeeCollection*) _tmp196_);
#line 664 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp198_ = _tmp197_;
#line 664 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp198_ == 1, "a1.children_nodes[0].children_nodes.size == 1");
#line 664 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp196_);
#line 664 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp193_);
#line 664 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp191_);
#line 665 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp199_ = gxml_node_get_children_nodes (a1);
#line 665 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp200_ = _tmp199_;
#line 665 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp201_ = _tmp200_;
#line 665 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp202_ = gee_list_get ((GeeList*) _tmp201_, 0);
#line 665 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp203_ = (GXmlNode*) _tmp202_;
#line 665 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp204_ = gxml_node_get_children_nodes (_tmp203_);
#line 665 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp205_ = _tmp204_;
#line 665 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp206_ = _tmp205_;
#line 665 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp207_ = gee_list_get ((GeeList*) _tmp206_, 0);
#line 665 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp208_ = (GXmlNode*) _tmp207_;
#line 665 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp208_, GXML_TYPE_TEXT), "a1.children_nodes[0].children_nodes[0] is GXml.Text");
#line 665 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp208_);
#line 665 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp206_);
#line 665 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp203_);
#line 665 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp201_);
#line 666 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp209_ = gxml_node_get_children_nodes (a1);
#line 666 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp210_ = _tmp209_;
#line 666 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp211_ = _tmp210_;
#line 666 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp212_ = gee_list_get ((GeeList*) _tmp211_, 0);
#line 666 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp213_ = (GXmlNode*) _tmp212_;
#line 666 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp214_ = gxml_node_get_children_nodes (_tmp213_);
#line 666 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp215_ = _tmp214_;
#line 666 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp216_ = _tmp215_;
#line 666 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp217_ = gee_list_get ((GeeList*) _tmp216_, 0);
#line 666 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp218_ = (GXmlNode*) _tmp217_;
#line 666 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp219_ = gxml_node_get_value (_tmp218_);
#line 666 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp220_ = _tmp219_;
#line 666 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp221_ = _tmp220_;
#line 666 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp221_, "Fred") == 0, "a1.children_nodes[0].children_nodes[0].value == \"Fred\"");
#line 666 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp221_);
#line 666 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp218_);
#line 666 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp216_);
#line 666 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp213_);
#line 666 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp211_);
#line 667 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp222_ = gxml_node_get_children_nodes (a1);
#line 667 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp223_ = _tmp222_;
#line 667 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp224_ = _tmp223_;
#line 667 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp225_ = gee_list_get ((GeeList*) _tmp224_, 1);
#line 667 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp226_ = (GXmlNode*) _tmp225_;
#line 667 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp227_ = gxml_node_get_name (_tmp226_);
#line 667 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp228_ = _tmp227_;
#line 667 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp229_ = _tmp228_;
#line 667 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp229_, "Email") == 0, "a1.children_nodes[1].name == \"Email\"");
#line 667 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp229_);
#line 667 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp226_);
#line 667 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp224_);
#line 668 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp230_ = gxml_node_get_children_nodes (a1);
#line 668 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp231_ = _tmp230_;
#line 668 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp232_ = _tmp231_;
#line 668 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp233_ = gee_list_get ((GeeList*) _tmp232_, 1);
#line 668 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp234_ = (GXmlNode*) _tmp233_;
#line 668 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp235_ = gxml_node_get_children_nodes (_tmp234_);
#line 668 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp236_ = _tmp235_;
#line 668 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp237_ = _tmp236_;
#line 668 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp238_ = gee_collection_get_size ((GeeCollection*) _tmp237_);
#line 668 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp239_ = _tmp238_;
#line 668 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp239_ == 1, "a1.children_nodes[1].children_nodes.size == 1");
#line 668 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp237_);
#line 668 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp234_);
#line 668 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp232_);
#line 669 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp240_ = gxml_node_get_children_nodes (a1);
#line 669 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp241_ = _tmp240_;
#line 669 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp242_ = _tmp241_;
#line 669 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp243_ = gee_list_get ((GeeList*) _tmp242_, 1);
#line 669 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp244_ = (GXmlNode*) _tmp243_;
#line 669 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp245_ = gxml_node_get_children_nodes (_tmp244_);
#line 669 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp246_ = _tmp245_;
#line 669 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp247_ = _tmp246_;
#line 669 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp248_ = gee_list_get ((GeeList*) _tmp247_, 0);
#line 669 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp249_ = (GXmlNode*) _tmp248_;
#line 669 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp249_, GXML_TYPE_TEXT), "a1.children_nodes[1].children_nodes[0] is GXml.Text");
#line 669 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp249_);
#line 669 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp247_);
#line 669 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp244_);
#line 669 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp242_);
#line 670 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp250_ = gxml_node_get_children_nodes (a1);
#line 670 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp251_ = _tmp250_;
#line 670 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp252_ = _tmp251_;
#line 670 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp253_ = gee_list_get ((GeeList*) _tmp252_, 1);
#line 670 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp254_ = (GXmlNode*) _tmp253_;
#line 670 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp255_ = gxml_node_get_children_nodes (_tmp254_);
#line 670 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp256_ = _tmp255_;
#line 670 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp257_ = _tmp256_;
#line 670 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp258_ = gee_list_get ((GeeList*) _tmp257_, 0);
#line 670 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp259_ = (GXmlNode*) _tmp258_;
#line 670 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp260_ = gxml_node_get_value (_tmp259_);
#line 670 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp261_ = _tmp260_;
#line 670 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp262_ = _tmp261_;
#line 670 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp262_, "fweasley@hogwarts.co.uk") == 0, "a1.children_nodes[1].children_nodes[0].value == \"fweasley@hogwarts.co.uk\"");
#line 670 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp262_);
#line 670 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp259_);
#line 670 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp257_);
#line 670 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp254_);
#line 670 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp252_);
#line 671 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp263_ = gxml_node_get_children_nodes (s3);
#line 671 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp264_ = _tmp263_;
#line 671 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp265_ = _tmp264_;
#line 671 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp266_ = gee_list_get ((GeeList*) _tmp265_, 1);
#line 671 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp267_ = (GXmlNode*) _tmp266_;
#line 671 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp265_);
#line 671 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		a2 = _tmp267_;
#line 672 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (a2 != NULL, "a2 != null");
#line 673 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp268_ = gxml_node_get_children_nodes (a2);
#line 673 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp269_ = _tmp268_;
#line 673 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp270_ = _tmp269_;
#line 673 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp271_ = gee_collection_get_size ((GeeCollection*) _tmp270_);
#line 673 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp272_ = _tmp271_;
#line 673 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp272_ == 3, "a2.children_nodes.size == 3");
#line 673 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp270_);
#line 674 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp273_ = gxml_node_get_children_nodes (a2);
#line 674 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp274_ = _tmp273_;
#line 674 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp275_ = _tmp274_;
#line 674 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp276_ = gee_list_get ((GeeList*) _tmp275_, 1);
#line 674 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp277_ = (GXmlNode*) _tmp276_;
#line 674 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp278_ = gxml_node_get_name (_tmp277_);
#line 674 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp279_ = _tmp278_;
#line 674 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp280_ = _tmp279_;
#line 674 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp280_, "Name") == 0, "a2.children_nodes[1].name == \"Name\"");
#line 674 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp280_);
#line 674 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp277_);
#line 674 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp275_);
#line 675 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp281_ = gxml_node_get_children_nodes (a2);
#line 675 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp282_ = _tmp281_;
#line 675 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp283_ = _tmp282_;
#line 675 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp284_ = gee_list_get ((GeeList*) _tmp283_, 1);
#line 675 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp285_ = (GXmlNode*) _tmp284_;
#line 675 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp286_ = gxml_node_get_children_nodes (_tmp285_);
#line 675 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp287_ = _tmp286_;
#line 675 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp288_ = _tmp287_;
#line 675 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp289_ = gee_collection_get_size ((GeeCollection*) _tmp288_);
#line 675 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp290_ = _tmp289_;
#line 675 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp290_ == 1, "a2.children_nodes[1].children_nodes.size == 1");
#line 675 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp288_);
#line 675 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp285_);
#line 675 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp283_);
#line 676 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp291_ = gxml_node_get_children_nodes (a2);
#line 676 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp292_ = _tmp291_;
#line 676 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp293_ = _tmp292_;
#line 676 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp294_ = gee_list_get ((GeeList*) _tmp293_, 1);
#line 676 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp295_ = (GXmlNode*) _tmp294_;
#line 676 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp296_ = gxml_node_get_children_nodes (_tmp295_);
#line 676 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp297_ = _tmp296_;
#line 676 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp298_ = _tmp297_;
#line 676 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp299_ = gee_list_get ((GeeList*) _tmp298_, 0);
#line 676 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp300_ = (GXmlNode*) _tmp299_;
#line 676 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp300_, GXML_TYPE_TEXT), "a2.children_nodes[1].children_nodes[0] is GXml.Text");
#line 676 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp300_);
#line 676 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp298_);
#line 676 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp295_);
#line 676 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp293_);
#line 677 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp301_ = gxml_node_get_children_nodes (a2);
#line 677 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp302_ = _tmp301_;
#line 677 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp303_ = _tmp302_;
#line 677 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp304_ = gee_list_get ((GeeList*) _tmp303_, 1);
#line 677 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp305_ = (GXmlNode*) _tmp304_;
#line 677 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp306_ = gxml_node_get_children_nodes (_tmp305_);
#line 677 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp307_ = _tmp306_;
#line 677 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp308_ = _tmp307_;
#line 677 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp309_ = gee_list_get ((GeeList*) _tmp308_, 0);
#line 677 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp310_ = (GXmlNode*) _tmp309_;
#line 677 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp311_ = gxml_node_get_value (_tmp310_);
#line 677 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp312_ = _tmp311_;
#line 677 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp313_ = _tmp312_;
#line 677 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp313_, "George") == 0, "a2.children_nodes[1].children_nodes[0].value == \"George\"");
#line 677 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp313_);
#line 677 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp310_);
#line 677 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp308_);
#line 677 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp305_);
#line 677 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp303_);
#line 678 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp314_ = gxml_node_get_children_nodes (a2);
#line 678 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp315_ = _tmp314_;
#line 678 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp316_ = _tmp315_;
#line 678 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp317_ = gee_list_get ((GeeList*) _tmp316_, 2);
#line 678 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp318_ = (GXmlNode*) _tmp317_;
#line 678 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp319_ = gxml_node_get_name (_tmp318_);
#line 678 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp320_ = _tmp319_;
#line 678 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp321_ = _tmp320_;
#line 678 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp321_, "Email") == 0, "a2.children_nodes[2].name == \"Email\"");
#line 678 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp321_);
#line 678 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp318_);
#line 678 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp316_);
#line 679 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp322_ = gxml_node_get_children_nodes (a2);
#line 679 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp323_ = _tmp322_;
#line 679 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp324_ = _tmp323_;
#line 679 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp325_ = gee_list_get ((GeeList*) _tmp324_, 2);
#line 679 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp326_ = (GXmlNode*) _tmp325_;
#line 679 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp327_ = gxml_node_get_children_nodes (_tmp326_);
#line 679 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp328_ = _tmp327_;
#line 679 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp329_ = _tmp328_;
#line 679 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp330_ = gee_collection_get_size ((GeeCollection*) _tmp329_);
#line 679 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp331_ = _tmp330_;
#line 679 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp331_ == 1, "a2.children_nodes[2].children_nodes.size == 1");
#line 679 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp329_);
#line 679 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp326_);
#line 679 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp324_);
#line 680 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp332_ = gxml_node_get_children_nodes (a2);
#line 680 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp333_ = _tmp332_;
#line 680 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp334_ = _tmp333_;
#line 680 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp335_ = gee_list_get ((GeeList*) _tmp334_, 2);
#line 680 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp336_ = (GXmlNode*) _tmp335_;
#line 680 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp337_ = gxml_node_get_children_nodes (_tmp336_);
#line 680 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp338_ = _tmp337_;
#line 680 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp339_ = _tmp338_;
#line 680 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp340_ = gee_list_get ((GeeList*) _tmp339_, 0);
#line 680 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp341_ = (GXmlNode*) _tmp340_;
#line 680 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp341_, GXML_TYPE_TEXT), "a2.children_nodes[2].children_nodes[0] is GXml.Text");
#line 680 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp341_);
#line 680 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp339_);
#line 680 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp336_);
#line 680 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp334_);
#line 681 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp342_ = gxml_node_get_children_nodes (a2);
#line 681 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp343_ = _tmp342_;
#line 681 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp344_ = _tmp343_;
#line 681 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp345_ = gee_list_get ((GeeList*) _tmp344_, 2);
#line 681 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp346_ = (GXmlNode*) _tmp345_;
#line 681 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp347_ = gxml_node_get_children_nodes (_tmp346_);
#line 681 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp348_ = _tmp347_;
#line 681 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp349_ = _tmp348_;
#line 681 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp350_ = gee_list_get ((GeeList*) _tmp349_, 0);
#line 681 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp351_ = (GXmlNode*) _tmp350_;
#line 681 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp352_ = gxml_node_get_value (_tmp351_);
#line 681 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp353_ = _tmp352_;
#line 681 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp354_ = _tmp353_;
#line 681 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp354_, "gweasley@hogwarts.co.uk") == 0, "a2.children_nodes[2].children_nodes[0].value == \"gweasley@hogwarts.co.uk\"");
#line 681 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp354_);
#line 681 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp351_);
#line 681 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp349_);
#line 681 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp346_);
#line 681 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp344_);
#line 618 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (a2);
#line 618 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (a1);
#line 618 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (p31);
#line 618 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (p3);
#line 618 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (s3);
#line 618 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (p2);
#line 618 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (s2);
#line 618 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (p1);
#line 618 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (s1);
#line 618 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (d);
#line 618 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp2_);
#line 618 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (f);
#line 10866 "TDocumentTest.c"
	}
	goto __finally138;
	__catch138_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp355_ = NULL;
		gchar* _tmp356_ = NULL;
		gchar* _tmp357_ = NULL;
#line 618 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _inner_error_;
#line 618 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_inner_error_ = NULL;
#line 682 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp355_ = e->message;
#line 682 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp356_ = g_strconcat ("ERROR: ", _tmp355_, NULL);
#line 682 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp357_ = _tmp356_;
#line 682 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_message ("TDocumentTest.vala:682: %s", _tmp357_);
#line 682 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp357_);
#line 682 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_assert_not_reached ();
#line 618 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_error_free0 (e);
#line 10893 "TDocumentTest.c"
	}
	__finally138:
#line 618 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 618 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 618 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 618 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		return;
#line 10904 "TDocumentTest.c"
	}
}


static void ___lambda123__gtest_func (void) {
#line 617 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda123_ ();
#line 10912 "TDocumentTest.c"
}


static void __lambda124_ (void) {
	GFile* f = NULL;
	GFile* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
	GXmlTDocument* d = NULL;
	GXmlTDocument* _tmp2_ = NULL;
	GXmlNode* _tmp3_ = NULL;
	GXmlNode* _tmp4_ = NULL;
	GXmlNode* _tmp5_ = NULL;
	GXmlNode* _tmp6_ = NULL;
	GXmlNode* _tmp7_ = NULL;
	GXmlNode* _tmp8_ = NULL;
	gchar* _tmp9_ = NULL;
	gchar* _tmp10_ = NULL;
	gchar* _tmp11_ = NULL;
	GXmlNode* _tmp12_ = NULL;
	GXmlNode* _tmp13_ = NULL;
	GXmlNode* _tmp14_ = NULL;
	GeeBidirList* _tmp15_ = NULL;
	GeeBidirList* _tmp16_ = NULL;
	GeeBidirList* _tmp17_ = NULL;
	gpointer _tmp18_ = NULL;
	GXmlNode* _tmp19_ = NULL;
	GXmlNode* _tmp20_ = NULL;
	GXmlNode* _tmp21_ = NULL;
	GXmlNode* _tmp22_ = NULL;
	GeeBidirList* _tmp23_ = NULL;
	GeeBidirList* _tmp24_ = NULL;
	GeeBidirList* _tmp25_ = NULL;
	gpointer _tmp26_ = NULL;
	GXmlNode* _tmp27_ = NULL;
	gchar* _tmp28_ = NULL;
	gchar* _tmp29_ = NULL;
	gchar* _tmp30_ = NULL;
	GXmlNode* _tmp31_ = NULL;
	GXmlNode* _tmp32_ = NULL;
	GXmlNode* _tmp33_ = NULL;
	GeeBidirList* _tmp34_ = NULL;
	GeeBidirList* _tmp35_ = NULL;
	GeeBidirList* _tmp36_ = NULL;
	gpointer _tmp37_ = NULL;
	GXmlNode* _tmp38_ = NULL;
	GeeBidirList* _tmp39_ = NULL;
	GeeBidirList* _tmp40_ = NULL;
	GeeBidirList* _tmp41_ = NULL;
	gpointer _tmp42_ = NULL;
	GXmlNode* _tmp43_ = NULL;
	GXmlNode* _tmp44_ = NULL;
	GXmlNode* _tmp45_ = NULL;
	GXmlNode* _tmp46_ = NULL;
	GeeBidirList* _tmp47_ = NULL;
	GeeBidirList* _tmp48_ = NULL;
	GeeBidirList* _tmp49_ = NULL;
	gpointer _tmp50_ = NULL;
	GXmlNode* _tmp51_ = NULL;
	GeeBidirList* _tmp52_ = NULL;
	GeeBidirList* _tmp53_ = NULL;
	GeeBidirList* _tmp54_ = NULL;
	gpointer _tmp55_ = NULL;
	GXmlNode* _tmp56_ = NULL;
	gchar* _tmp57_ = NULL;
	gchar* _tmp58_ = NULL;
	gchar* _tmp59_ = NULL;
	GXmlNode* _tmp60_ = NULL;
	GXmlNode* _tmp61_ = NULL;
	GXmlNode* _tmp62_ = NULL;
	GeeBidirList* _tmp63_ = NULL;
	GeeBidirList* _tmp64_ = NULL;
	GeeBidirList* _tmp65_ = NULL;
	gpointer _tmp66_ = NULL;
	GXmlNode* _tmp67_ = NULL;
	GXmlNode* _tmp68_ = NULL;
	GXmlNode* _tmp69_ = NULL;
	GXmlNode* _tmp70_ = NULL;
	GeeBidirList* _tmp71_ = NULL;
	GeeBidirList* _tmp72_ = NULL;
	GeeBidirList* _tmp73_ = NULL;
	gpointer _tmp74_ = NULL;
	GXmlNode* _tmp75_ = NULL;
	gchar* _tmp76_ = NULL;
	gchar* _tmp77_ = NULL;
	gchar* _tmp78_ = NULL;
	GXmlNode* _tmp79_ = NULL;
	GXmlNode* _tmp80_ = NULL;
	GXmlNode* _tmp81_ = NULL;
	GeeBidirList* _tmp82_ = NULL;
	GeeBidirList* _tmp83_ = NULL;
	GeeBidirList* _tmp84_ = NULL;
	gpointer _tmp85_ = NULL;
	GXmlNode* _tmp86_ = NULL;
	GeeMap* _tmp87_ = NULL;
	GeeMap* _tmp88_ = NULL;
	GeeMap* _tmp89_ = NULL;
	gpointer _tmp90_ = NULL;
	GXmlNode* _tmp91_ = NULL;
	gchar* _tmp92_ = NULL;
	gchar* _tmp93_ = NULL;
	gchar* _tmp94_ = NULL;
	GXmlNode* _tmp95_ = NULL;
	GXmlNode* _tmp96_ = NULL;
	GXmlNode* _tmp97_ = NULL;
	GeeBidirList* _tmp98_ = NULL;
	GeeBidirList* _tmp99_ = NULL;
	GeeBidirList* _tmp100_ = NULL;
	gpointer _tmp101_ = NULL;
	GXmlNode* _tmp102_ = NULL;
	GeeBidirList* _tmp103_ = NULL;
	GeeBidirList* _tmp104_ = NULL;
	GeeBidirList* _tmp105_ = NULL;
	gpointer _tmp106_ = NULL;
	GXmlNode* _tmp107_ = NULL;
	GXmlNode* _tmp108_ = NULL;
	GXmlNode* _tmp109_ = NULL;
	GXmlNode* _tmp110_ = NULL;
	GeeBidirList* _tmp111_ = NULL;
	GeeBidirList* _tmp112_ = NULL;
	GeeBidirList* _tmp113_ = NULL;
	gpointer _tmp114_ = NULL;
	GXmlNode* _tmp115_ = NULL;
	GeeBidirList* _tmp116_ = NULL;
	GeeBidirList* _tmp117_ = NULL;
	GeeBidirList* _tmp118_ = NULL;
	gpointer _tmp119_ = NULL;
	GXmlNode* _tmp120_ = NULL;
	gchar* _tmp121_ = NULL;
	gchar* _tmp122_ = NULL;
	gchar* _tmp123_ = NULL;
	GXmlNode* _tmp124_ = NULL;
	GXmlNode* _tmp125_ = NULL;
	GXmlNode* _tmp126_ = NULL;
	GeeBidirList* _tmp127_ = NULL;
	GeeBidirList* _tmp128_ = NULL;
	GeeBidirList* _tmp129_ = NULL;
	gpointer _tmp130_ = NULL;
	GXmlNode* _tmp131_ = NULL;
	GeeBidirList* _tmp132_ = NULL;
	GeeBidirList* _tmp133_ = NULL;
	GeeBidirList* _tmp134_ = NULL;
	gpointer _tmp135_ = NULL;
	GXmlNode* _tmp136_ = NULL;
	GeeBidirList* _tmp137_ = NULL;
	GeeBidirList* _tmp138_ = NULL;
	GeeBidirList* _tmp139_ = NULL;
	gpointer _tmp140_ = NULL;
	GXmlNode* _tmp141_ = NULL;
	GXmlNode* _tmp142_ = NULL;
	GXmlNode* _tmp143_ = NULL;
	GXmlNode* _tmp144_ = NULL;
	GeeBidirList* _tmp145_ = NULL;
	GeeBidirList* _tmp146_ = NULL;
	GeeBidirList* _tmp147_ = NULL;
	gpointer _tmp148_ = NULL;
	GXmlNode* _tmp149_ = NULL;
	GeeBidirList* _tmp150_ = NULL;
	GeeBidirList* _tmp151_ = NULL;
	GeeBidirList* _tmp152_ = NULL;
	gpointer _tmp153_ = NULL;
	GXmlNode* _tmp154_ = NULL;
	GeeBidirList* _tmp155_ = NULL;
	GeeBidirList* _tmp156_ = NULL;
	GeeBidirList* _tmp157_ = NULL;
	gpointer _tmp158_ = NULL;
	GXmlNode* _tmp159_ = NULL;
	gchar* _tmp160_ = NULL;
	gchar* _tmp161_ = NULL;
	gchar* _tmp162_ = NULL;
#line 685 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp0_ = g_file_new_for_path (TEST_DIR "/t-read-test.xml");
#line 685 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	f = _tmp0_;
#line 686 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp1_ = g_file_query_exists (f, NULL);
#line 686 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp1_, "f.query_exists ()");
#line 687 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp2_ = gxml_tdocument_new_from_string ("<root><child v=\"1\">TEXT</child><doc year=\"2016\"><name>COMMUNICATIO" \
"NS</name></doc></root>");
#line 687 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	d = _tmp2_;
#line 688 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp3_ = gxml_document_get_root ((GXmlDocument*) d);
#line 688 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp4_ = _tmp3_;
#line 688 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp5_ = _tmp4_;
#line 688 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp5_ != NULL, "d.root != null");
#line 688 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp5_);
#line 689 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp6_ = gxml_document_get_root ((GXmlDocument*) d);
#line 689 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp7_ = _tmp6_;
#line 689 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp8_ = _tmp7_;
#line 689 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp9_ = gxml_node_get_name (_tmp8_);
#line 689 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp10_ = _tmp9_;
#line 689 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp11_ = _tmp10_;
#line 689 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp11_, "root") == 0, "d.root.name == \"root\"");
#line 689 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp11_);
#line 689 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp8_);
#line 690 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp12_ = gxml_document_get_root ((GXmlDocument*) d);
#line 690 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp13_ = _tmp12_;
#line 690 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp14_ = _tmp13_;
#line 690 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp15_ = gxml_node_get_children_nodes (_tmp14_);
#line 690 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp16_ = _tmp15_;
#line 690 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp17_ = _tmp16_;
#line 690 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp18_ = gee_list_get ((GeeList*) _tmp17_, 0);
#line 690 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp19_ = (GXmlNode*) _tmp18_;
#line 690 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp19_ != NULL, "d.root.children_nodes[0] != null");
#line 690 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp19_);
#line 690 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp17_);
#line 690 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp14_);
#line 691 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp20_ = gxml_document_get_root ((GXmlDocument*) d);
#line 691 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp21_ = _tmp20_;
#line 691 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp22_ = _tmp21_;
#line 691 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp23_ = gxml_node_get_children_nodes (_tmp22_);
#line 691 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp24_ = _tmp23_;
#line 691 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp25_ = _tmp24_;
#line 691 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp26_ = gee_list_get ((GeeList*) _tmp25_, 0);
#line 691 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp27_ = (GXmlNode*) _tmp26_;
#line 691 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp28_ = gxml_node_get_name (_tmp27_);
#line 691 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp29_ = _tmp28_;
#line 691 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp30_ = _tmp29_;
#line 691 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp30_, "child") == 0, "d.root.children_nodes[0].name == \"child\"");
#line 691 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp30_);
#line 691 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp27_);
#line 691 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp25_);
#line 691 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp22_);
#line 692 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp31_ = gxml_document_get_root ((GXmlDocument*) d);
#line 692 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp32_ = _tmp31_;
#line 692 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp33_ = _tmp32_;
#line 692 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp34_ = gxml_node_get_children_nodes (_tmp33_);
#line 692 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp35_ = _tmp34_;
#line 692 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp36_ = _tmp35_;
#line 692 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp37_ = gee_list_get ((GeeList*) _tmp36_, 0);
#line 692 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp38_ = (GXmlNode*) _tmp37_;
#line 692 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp39_ = gxml_node_get_children_nodes (_tmp38_);
#line 692 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp40_ = _tmp39_;
#line 692 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp41_ = _tmp40_;
#line 692 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp42_ = gee_list_get ((GeeList*) _tmp41_, 0);
#line 692 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp43_ = (GXmlNode*) _tmp42_;
#line 692 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp43_, GXML_TYPE_TEXT), "d.root.children_nodes[0].children_nodes[0] is GXml.Text");
#line 692 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp43_);
#line 692 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp41_);
#line 692 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp38_);
#line 692 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp36_);
#line 692 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp33_);
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp44_ = gxml_document_get_root ((GXmlDocument*) d);
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp45_ = _tmp44_;
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp46_ = _tmp45_;
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp47_ = gxml_node_get_children_nodes (_tmp46_);
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp48_ = _tmp47_;
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp49_ = _tmp48_;
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp50_ = gee_list_get ((GeeList*) _tmp49_, 0);
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp51_ = (GXmlNode*) _tmp50_;
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp52_ = gxml_node_get_children_nodes (_tmp51_);
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp53_ = _tmp52_;
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp54_ = _tmp53_;
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp55_ = gee_list_get ((GeeList*) _tmp54_, 0);
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp56_ = (GXmlNode*) _tmp55_;
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp57_ = gxml_node_get_value (_tmp56_);
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp58_ = _tmp57_;
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp59_ = _tmp58_;
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp59_, "TEXT") == 0, "d.root.children_nodes[0].children_nodes[0].value == \"TEXT\"");
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp59_);
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp56_);
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp54_);
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp51_);
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp49_);
#line 693 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp46_);
#line 694 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp60_ = gxml_document_get_root ((GXmlDocument*) d);
#line 694 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp61_ = _tmp60_;
#line 694 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp62_ = _tmp61_;
#line 694 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp63_ = gxml_node_get_children_nodes (_tmp62_);
#line 694 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp64_ = _tmp63_;
#line 694 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp65_ = _tmp64_;
#line 694 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp66_ = gee_list_get ((GeeList*) _tmp65_, 1);
#line 694 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp67_ = (GXmlNode*) _tmp66_;
#line 694 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp67_ != NULL, "d.root.children_nodes[1] != null");
#line 694 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp67_);
#line 694 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp65_);
#line 694 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp62_);
#line 695 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp68_ = gxml_document_get_root ((GXmlDocument*) d);
#line 695 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp69_ = _tmp68_;
#line 695 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp70_ = _tmp69_;
#line 695 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp71_ = gxml_node_get_children_nodes (_tmp70_);
#line 695 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp72_ = _tmp71_;
#line 695 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp73_ = _tmp72_;
#line 695 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp74_ = gee_list_get ((GeeList*) _tmp73_, 1);
#line 695 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp75_ = (GXmlNode*) _tmp74_;
#line 695 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp76_ = gxml_node_get_name (_tmp75_);
#line 695 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp77_ = _tmp76_;
#line 695 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp78_ = _tmp77_;
#line 695 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp78_, "doc") == 0, "d.root.children_nodes[1].name == \"doc\"");
#line 695 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp78_);
#line 695 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp75_);
#line 695 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp73_);
#line 695 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp70_);
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp79_ = gxml_document_get_root ((GXmlDocument*) d);
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp80_ = _tmp79_;
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp81_ = _tmp80_;
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp82_ = gxml_node_get_children_nodes (_tmp81_);
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp83_ = _tmp82_;
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp84_ = _tmp83_;
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp85_ = gee_list_get ((GeeList*) _tmp84_, 1);
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp86_ = (GXmlNode*) _tmp85_;
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp87_ = gxml_node_get_attrs (_tmp86_);
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp88_ = _tmp87_;
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp89_ = _tmp88_;
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp90_ = gee_map_get (_tmp89_, "year");
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp91_ = (GXmlNode*) _tmp90_;
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp92_ = gxml_node_get_value (_tmp91_);
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp93_ = _tmp92_;
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp94_ = _tmp93_;
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp94_, "2016") == 0, "d.root.children_nodes[1].attrs[\"year\"].value == \"2016\"");
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp94_);
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp91_);
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp89_);
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp86_);
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp84_);
#line 696 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp81_);
#line 697 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp95_ = gxml_document_get_root ((GXmlDocument*) d);
#line 697 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp96_ = _tmp95_;
#line 697 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp97_ = _tmp96_;
#line 697 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp98_ = gxml_node_get_children_nodes (_tmp97_);
#line 697 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp99_ = _tmp98_;
#line 697 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp100_ = _tmp99_;
#line 697 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp101_ = gee_list_get ((GeeList*) _tmp100_, 1);
#line 697 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp102_ = (GXmlNode*) _tmp101_;
#line 697 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp103_ = gxml_node_get_children_nodes (_tmp102_);
#line 697 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp104_ = _tmp103_;
#line 697 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp105_ = _tmp104_;
#line 697 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp106_ = gee_list_get ((GeeList*) _tmp105_, 0);
#line 697 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp107_ = (GXmlNode*) _tmp106_;
#line 697 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp107_ != NULL, "d.root.children_nodes[1].children_nodes[0] != null");
#line 697 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp107_);
#line 697 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp105_);
#line 697 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp102_);
#line 697 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp100_);
#line 697 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp97_);
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp108_ = gxml_document_get_root ((GXmlDocument*) d);
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp109_ = _tmp108_;
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp110_ = _tmp109_;
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp111_ = gxml_node_get_children_nodes (_tmp110_);
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp112_ = _tmp111_;
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp113_ = _tmp112_;
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp114_ = gee_list_get ((GeeList*) _tmp113_, 1);
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp115_ = (GXmlNode*) _tmp114_;
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp116_ = gxml_node_get_children_nodes (_tmp115_);
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp117_ = _tmp116_;
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp118_ = _tmp117_;
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp119_ = gee_list_get ((GeeList*) _tmp118_, 0);
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp120_ = (GXmlNode*) _tmp119_;
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp121_ = gxml_node_get_name (_tmp120_);
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp122_ = _tmp121_;
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp123_ = _tmp122_;
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp123_, "name") == 0, "d.root.children_nodes[1].children_nodes[0].name == \"name\"");
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp123_);
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp120_);
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp118_);
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp115_);
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp113_);
#line 698 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp110_);
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp124_ = gxml_document_get_root ((GXmlDocument*) d);
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp125_ = _tmp124_;
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp126_ = _tmp125_;
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp127_ = gxml_node_get_children_nodes (_tmp126_);
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp128_ = _tmp127_;
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp129_ = _tmp128_;
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp130_ = gee_list_get ((GeeList*) _tmp129_, 1);
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp131_ = (GXmlNode*) _tmp130_;
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp132_ = gxml_node_get_children_nodes (_tmp131_);
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp133_ = _tmp132_;
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp134_ = _tmp133_;
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp135_ = gee_list_get ((GeeList*) _tmp134_, 0);
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp136_ = (GXmlNode*) _tmp135_;
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp137_ = gxml_node_get_children_nodes (_tmp136_);
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp138_ = _tmp137_;
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp139_ = _tmp138_;
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp140_ = gee_list_get ((GeeList*) _tmp139_, 0);
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp141_ = (GXmlNode*) _tmp140_;
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp141_, GXML_TYPE_TEXT), "d.root.children_nodes[1].children_nodes[0].children_nodes[0] is GXml.Text");
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp141_);
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp139_);
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp136_);
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp134_);
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp131_);
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp129_);
#line 699 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp126_);
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp142_ = gxml_document_get_root ((GXmlDocument*) d);
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp143_ = _tmp142_;
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp144_ = _tmp143_;
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp145_ = gxml_node_get_children_nodes (_tmp144_);
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp146_ = _tmp145_;
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp147_ = _tmp146_;
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp148_ = gee_list_get ((GeeList*) _tmp147_, 1);
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp149_ = (GXmlNode*) _tmp148_;
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp150_ = gxml_node_get_children_nodes (_tmp149_);
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp151_ = _tmp150_;
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp152_ = _tmp151_;
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp153_ = gee_list_get ((GeeList*) _tmp152_, 0);
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp154_ = (GXmlNode*) _tmp153_;
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp155_ = gxml_node_get_children_nodes (_tmp154_);
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp156_ = _tmp155_;
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp157_ = _tmp156_;
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp158_ = gee_list_get ((GeeList*) _tmp157_, 0);
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp159_ = (GXmlNode*) _tmp158_;
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp160_ = gxml_node_get_value (_tmp159_);
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp161_ = _tmp160_;
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp162_ = _tmp161_;
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp162_, "COMMUNICATIONS") == 0, "d.root.children_nodes[1].children_nodes[0].children_nodes[0].value == \"COMMUNICATIONS\"");
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp162_);
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp159_);
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp157_);
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp154_);
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp152_);
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp149_);
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp147_);
#line 700 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp144_);
#line 684 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (d);
#line 684 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (f);
#line 11564 "TDocumentTest.c"
}


static void ___lambda124__gtest_func (void) {
#line 684 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda124_ ();
#line 11571 "TDocumentTest.c"
}


static void __lambda125_ (void) {
	GFile* f = NULL;
	GFile* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
	GXmlTDocument* d = NULL;
	GXmlTDocument* _tmp2_ = NULL;
	GXmlNode* _tmp3_ = NULL;
	GXmlNode* _tmp4_ = NULL;
	GXmlNode* _tmp5_ = NULL;
	GXmlNode* _tmp6_ = NULL;
	GXmlNode* _tmp7_ = NULL;
	GXmlNode* _tmp8_ = NULL;
	gchar* _tmp9_ = NULL;
	gchar* _tmp10_ = NULL;
	gchar* _tmp11_ = NULL;
	GXmlNode* _tmp12_ = NULL;
	GXmlNode* _tmp13_ = NULL;
	GXmlNode* _tmp14_ = NULL;
	GeeBidirList* _tmp15_ = NULL;
	GeeBidirList* _tmp16_ = NULL;
	GeeBidirList* _tmp17_ = NULL;
	gpointer _tmp18_ = NULL;
	GXmlNode* _tmp19_ = NULL;
	GXmlNode* _tmp20_ = NULL;
	GXmlNode* _tmp21_ = NULL;
	GXmlNode* _tmp22_ = NULL;
	GeeBidirList* _tmp23_ = NULL;
	GeeBidirList* _tmp24_ = NULL;
	GeeBidirList* _tmp25_ = NULL;
	gpointer _tmp26_ = NULL;
	GXmlNode* _tmp27_ = NULL;
	gchar* _tmp28_ = NULL;
	gchar* _tmp29_ = NULL;
	gchar* _tmp30_ = NULL;
	GXmlNode* _tmp31_ = NULL;
	GXmlNode* _tmp32_ = NULL;
	GXmlNode* _tmp33_ = NULL;
	GeeBidirList* _tmp34_ = NULL;
	GeeBidirList* _tmp35_ = NULL;
	GeeBidirList* _tmp36_ = NULL;
	gpointer _tmp37_ = NULL;
	GXmlNode* _tmp38_ = NULL;
	GeeMap* _tmp39_ = NULL;
	GeeMap* _tmp40_ = NULL;
	GeeMap* _tmp41_ = NULL;
	gint _tmp42_ = 0;
	gint _tmp43_ = 0;
	GXmlNode* _tmp44_ = NULL;
	GXmlNode* _tmp45_ = NULL;
	GXmlNode* _tmp46_ = NULL;
	GeeBidirList* _tmp47_ = NULL;
	GeeBidirList* _tmp48_ = NULL;
	GeeBidirList* _tmp49_ = NULL;
	gpointer _tmp50_ = NULL;
	GXmlNode* _tmp51_ = NULL;
	GeeMap* _tmp52_ = NULL;
	GeeMap* _tmp53_ = NULL;
	GeeMap* _tmp54_ = NULL;
	gpointer _tmp55_ = NULL;
	GXmlNode* _tmp56_ = NULL;
	GXmlNode* _tmp57_ = NULL;
	GXmlNode* _tmp58_ = NULL;
	GXmlNode* _tmp59_ = NULL;
	GeeBidirList* _tmp60_ = NULL;
	GeeBidirList* _tmp61_ = NULL;
	GeeBidirList* _tmp62_ = NULL;
	gpointer _tmp63_ = NULL;
	GXmlNode* _tmp64_ = NULL;
	GeeBidirList* _tmp65_ = NULL;
	GeeBidirList* _tmp66_ = NULL;
	GeeBidirList* _tmp67_ = NULL;
	gpointer _tmp68_ = NULL;
	GXmlNode* _tmp69_ = NULL;
	GXmlNode* _tmp70_ = NULL;
	GXmlNode* _tmp71_ = NULL;
	GXmlNode* _tmp72_ = NULL;
	GeeBidirList* _tmp73_ = NULL;
	GeeBidirList* _tmp74_ = NULL;
	GeeBidirList* _tmp75_ = NULL;
	gpointer _tmp76_ = NULL;
	GXmlNode* _tmp77_ = NULL;
	GeeBidirList* _tmp78_ = NULL;
	GeeBidirList* _tmp79_ = NULL;
	GeeBidirList* _tmp80_ = NULL;
	gpointer _tmp81_ = NULL;
	GXmlNode* _tmp82_ = NULL;
	gchar* _tmp83_ = NULL;
	gchar* _tmp84_ = NULL;
	gchar* _tmp85_ = NULL;
	GXmlNode* _tmp86_ = NULL;
	GXmlNode* _tmp87_ = NULL;
	GXmlNode* _tmp88_ = NULL;
	GeeBidirList* _tmp89_ = NULL;
	GeeBidirList* _tmp90_ = NULL;
	GeeBidirList* _tmp91_ = NULL;
	gpointer _tmp92_ = NULL;
	GXmlNode* _tmp93_ = NULL;
	GXmlNode* _tmp94_ = NULL;
	GXmlNode* _tmp95_ = NULL;
	GXmlNode* _tmp96_ = NULL;
	GeeBidirList* _tmp97_ = NULL;
	GeeBidirList* _tmp98_ = NULL;
	GeeBidirList* _tmp99_ = NULL;
	gpointer _tmp100_ = NULL;
	GXmlNode* _tmp101_ = NULL;
	gchar* _tmp102_ = NULL;
	gchar* _tmp103_ = NULL;
	gchar* _tmp104_ = NULL;
	GXmlNode* _tmp105_ = NULL;
	GXmlNode* _tmp106_ = NULL;
	GXmlNode* _tmp107_ = NULL;
	GeeBidirList* _tmp108_ = NULL;
	GeeBidirList* _tmp109_ = NULL;
	GeeBidirList* _tmp110_ = NULL;
	gpointer _tmp111_ = NULL;
	GXmlNode* _tmp112_ = NULL;
	GeeMap* _tmp113_ = NULL;
	GeeMap* _tmp114_ = NULL;
	GeeMap* _tmp115_ = NULL;
	gpointer _tmp116_ = NULL;
	GXmlNode* _tmp117_ = NULL;
	gchar* _tmp118_ = NULL;
	gchar* _tmp119_ = NULL;
	gchar* _tmp120_ = NULL;
	GXmlNode* _tmp121_ = NULL;
	GXmlNode* _tmp122_ = NULL;
	GXmlNode* _tmp123_ = NULL;
	GeeBidirList* _tmp124_ = NULL;
	GeeBidirList* _tmp125_ = NULL;
	GeeBidirList* _tmp126_ = NULL;
	gpointer _tmp127_ = NULL;
	GXmlNode* _tmp128_ = NULL;
	GeeBidirList* _tmp129_ = NULL;
	GeeBidirList* _tmp130_ = NULL;
	GeeBidirList* _tmp131_ = NULL;
	gpointer _tmp132_ = NULL;
	GXmlNode* _tmp133_ = NULL;
	GXmlNode* _tmp134_ = NULL;
	GXmlNode* _tmp135_ = NULL;
	GXmlNode* _tmp136_ = NULL;
	GeeBidirList* _tmp137_ = NULL;
	GeeBidirList* _tmp138_ = NULL;
	GeeBidirList* _tmp139_ = NULL;
	gpointer _tmp140_ = NULL;
	GXmlNode* _tmp141_ = NULL;
	GeeBidirList* _tmp142_ = NULL;
	GeeBidirList* _tmp143_ = NULL;
	GeeBidirList* _tmp144_ = NULL;
	gpointer _tmp145_ = NULL;
	GXmlNode* _tmp146_ = NULL;
	gchar* _tmp147_ = NULL;
	gchar* _tmp148_ = NULL;
	gchar* _tmp149_ = NULL;
	GXmlNode* _tmp150_ = NULL;
	GXmlNode* _tmp151_ = NULL;
	GXmlNode* _tmp152_ = NULL;
	GeeBidirList* _tmp153_ = NULL;
	GeeBidirList* _tmp154_ = NULL;
	GeeBidirList* _tmp155_ = NULL;
	gpointer _tmp156_ = NULL;
	GXmlNode* _tmp157_ = NULL;
	GeeBidirList* _tmp158_ = NULL;
	GeeBidirList* _tmp159_ = NULL;
	GeeBidirList* _tmp160_ = NULL;
	gpointer _tmp161_ = NULL;
	GXmlNode* _tmp162_ = NULL;
	GeeBidirList* _tmp163_ = NULL;
	GeeBidirList* _tmp164_ = NULL;
	GeeBidirList* _tmp165_ = NULL;
	gpointer _tmp166_ = NULL;
	GXmlNode* _tmp167_ = NULL;
	GXmlNode* _tmp168_ = NULL;
	GXmlNode* _tmp169_ = NULL;
	GXmlNode* _tmp170_ = NULL;
	GeeBidirList* _tmp171_ = NULL;
	GeeBidirList* _tmp172_ = NULL;
	GeeBidirList* _tmp173_ = NULL;
	gpointer _tmp174_ = NULL;
	GXmlNode* _tmp175_ = NULL;
	GeeBidirList* _tmp176_ = NULL;
	GeeBidirList* _tmp177_ = NULL;
	GeeBidirList* _tmp178_ = NULL;
	gpointer _tmp179_ = NULL;
	GXmlNode* _tmp180_ = NULL;
	GeeBidirList* _tmp181_ = NULL;
	GeeBidirList* _tmp182_ = NULL;
	GeeBidirList* _tmp183_ = NULL;
	gpointer _tmp184_ = NULL;
	GXmlNode* _tmp185_ = NULL;
	gchar* _tmp186_ = NULL;
	gchar* _tmp187_ = NULL;
	gchar* _tmp188_ = NULL;
#line 703 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp0_ = g_file_new_for_path (TEST_DIR "/t-read-test.xml");
#line 703 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	f = _tmp0_;
#line 704 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp1_ = g_file_query_exists (f, NULL);
#line 704 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp1_, "f.query_exists ()");
#line 705 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp2_ = gxml_tdocument_new_from_string ("<root xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"><child v" \
"=\"1\" xsi:v=\"VType\">TEXT</child><doc year=\"2016\"><name>COMMUNICAT" \
"IONS</name></doc></root>");
#line 705 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	d = _tmp2_;
#line 706 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp3_ = gxml_document_get_root ((GXmlDocument*) d);
#line 706 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp4_ = _tmp3_;
#line 706 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp5_ = _tmp4_;
#line 706 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp5_ != NULL, "d.root != null");
#line 706 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp5_);
#line 707 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp6_ = gxml_document_get_root ((GXmlDocument*) d);
#line 707 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp7_ = _tmp6_;
#line 707 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp8_ = _tmp7_;
#line 707 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp9_ = gxml_node_get_name (_tmp8_);
#line 707 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp10_ = _tmp9_;
#line 707 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp11_ = _tmp10_;
#line 707 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp11_, "root") == 0, "d.root.name == \"root\"");
#line 707 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp11_);
#line 707 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp8_);
#line 708 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp12_ = gxml_document_get_root ((GXmlDocument*) d);
#line 708 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp13_ = _tmp12_;
#line 708 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp14_ = _tmp13_;
#line 708 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp15_ = gxml_node_get_children_nodes (_tmp14_);
#line 708 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp16_ = _tmp15_;
#line 708 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp17_ = _tmp16_;
#line 708 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp18_ = gee_list_get ((GeeList*) _tmp17_, 0);
#line 708 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp19_ = (GXmlNode*) _tmp18_;
#line 708 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp19_ != NULL, "d.root.children_nodes[0] != null");
#line 708 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp19_);
#line 708 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp17_);
#line 708 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp14_);
#line 709 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp20_ = gxml_document_get_root ((GXmlDocument*) d);
#line 709 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp21_ = _tmp20_;
#line 709 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp22_ = _tmp21_;
#line 709 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp23_ = gxml_node_get_children_nodes (_tmp22_);
#line 709 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp24_ = _tmp23_;
#line 709 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp25_ = _tmp24_;
#line 709 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp26_ = gee_list_get ((GeeList*) _tmp25_, 0);
#line 709 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp27_ = (GXmlNode*) _tmp26_;
#line 709 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp28_ = gxml_node_get_name (_tmp27_);
#line 709 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp29_ = _tmp28_;
#line 709 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp30_ = _tmp29_;
#line 709 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp30_, "child") == 0, "d.root.children_nodes[0].name == \"child\"");
#line 709 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp30_);
#line 709 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp27_);
#line 709 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp25_);
#line 709 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp22_);
#line 713 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp31_ = gxml_document_get_root ((GXmlDocument*) d);
#line 713 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp32_ = _tmp31_;
#line 713 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp33_ = _tmp32_;
#line 713 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp34_ = gxml_node_get_children_nodes (_tmp33_);
#line 713 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp35_ = _tmp34_;
#line 713 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp36_ = _tmp35_;
#line 713 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp37_ = gee_list_get ((GeeList*) _tmp36_, 0);
#line 713 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp38_ = (GXmlNode*) _tmp37_;
#line 713 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp39_ = gxml_node_get_attrs (_tmp38_);
#line 713 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp40_ = _tmp39_;
#line 713 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp41_ = _tmp40_;
#line 713 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp42_ = gee_map_get_size (_tmp41_);
#line 713 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp43_ = _tmp42_;
#line 713 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp43_ == 2, "d.root.children_nodes[0].attrs.size == 2");
#line 713 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp41_);
#line 713 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp38_);
#line 713 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp36_);
#line 713 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp33_);
#line 714 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp44_ = gxml_document_get_root ((GXmlDocument*) d);
#line 714 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp45_ = _tmp44_;
#line 714 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp46_ = _tmp45_;
#line 714 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp47_ = gxml_node_get_children_nodes (_tmp46_);
#line 714 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp48_ = _tmp47_;
#line 714 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp49_ = _tmp48_;
#line 714 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp50_ = gee_list_get ((GeeList*) _tmp49_, 0);
#line 714 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp51_ = (GXmlNode*) _tmp50_;
#line 714 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp52_ = gxml_node_get_attrs (_tmp51_);
#line 714 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp53_ = _tmp52_;
#line 714 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp54_ = _tmp53_;
#line 714 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp55_ = gee_map_get (_tmp54_, "v");
#line 714 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp56_ = (GXmlNode*) _tmp55_;
#line 714 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp56_ != NULL, "d.root.children_nodes[0].attrs[\"v\"] != null");
#line 714 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp56_);
#line 714 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp54_);
#line 714 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp51_);
#line 714 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp49_);
#line 714 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp46_);
#line 715 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp57_ = gxml_document_get_root ((GXmlDocument*) d);
#line 715 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp58_ = _tmp57_;
#line 715 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp59_ = _tmp58_;
#line 715 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp60_ = gxml_node_get_children_nodes (_tmp59_);
#line 715 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp61_ = _tmp60_;
#line 715 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp62_ = _tmp61_;
#line 715 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp63_ = gee_list_get ((GeeList*) _tmp62_, 0);
#line 715 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp64_ = (GXmlNode*) _tmp63_;
#line 715 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp65_ = gxml_node_get_children_nodes (_tmp64_);
#line 715 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp66_ = _tmp65_;
#line 715 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp67_ = _tmp66_;
#line 715 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp68_ = gee_list_get ((GeeList*) _tmp67_, 0);
#line 715 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp69_ = (GXmlNode*) _tmp68_;
#line 715 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp69_, GXML_TYPE_TEXT), "d.root.children_nodes[0].children_nodes[0] is GXml.Text");
#line 715 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp69_);
#line 715 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp67_);
#line 715 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp64_);
#line 715 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp62_);
#line 715 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp59_);
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp70_ = gxml_document_get_root ((GXmlDocument*) d);
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp71_ = _tmp70_;
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp72_ = _tmp71_;
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp73_ = gxml_node_get_children_nodes (_tmp72_);
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp74_ = _tmp73_;
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp75_ = _tmp74_;
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp76_ = gee_list_get ((GeeList*) _tmp75_, 0);
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp77_ = (GXmlNode*) _tmp76_;
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp78_ = gxml_node_get_children_nodes (_tmp77_);
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp79_ = _tmp78_;
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp80_ = _tmp79_;
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp81_ = gee_list_get ((GeeList*) _tmp80_, 0);
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp82_ = (GXmlNode*) _tmp81_;
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp83_ = gxml_node_get_value (_tmp82_);
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp84_ = _tmp83_;
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp85_ = _tmp84_;
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp85_, "TEXT") == 0, "d.root.children_nodes[0].children_nodes[0].value == \"TEXT\"");
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp85_);
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp82_);
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp80_);
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp77_);
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp75_);
#line 716 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp72_);
#line 717 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp86_ = gxml_document_get_root ((GXmlDocument*) d);
#line 717 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp87_ = _tmp86_;
#line 717 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp88_ = _tmp87_;
#line 717 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp89_ = gxml_node_get_children_nodes (_tmp88_);
#line 717 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp90_ = _tmp89_;
#line 717 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp91_ = _tmp90_;
#line 717 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp92_ = gee_list_get ((GeeList*) _tmp91_, 1);
#line 717 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp93_ = (GXmlNode*) _tmp92_;
#line 717 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp93_ != NULL, "d.root.children_nodes[1] != null");
#line 717 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp93_);
#line 717 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp91_);
#line 717 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp88_);
#line 718 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp94_ = gxml_document_get_root ((GXmlDocument*) d);
#line 718 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp95_ = _tmp94_;
#line 718 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp96_ = _tmp95_;
#line 718 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp97_ = gxml_node_get_children_nodes (_tmp96_);
#line 718 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp98_ = _tmp97_;
#line 718 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp99_ = _tmp98_;
#line 718 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp100_ = gee_list_get ((GeeList*) _tmp99_, 1);
#line 718 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp101_ = (GXmlNode*) _tmp100_;
#line 718 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp102_ = gxml_node_get_name (_tmp101_);
#line 718 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp103_ = _tmp102_;
#line 718 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp104_ = _tmp103_;
#line 718 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp104_, "doc") == 0, "d.root.children_nodes[1].name == \"doc\"");
#line 718 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp104_);
#line 718 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp101_);
#line 718 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp99_);
#line 718 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp96_);
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp105_ = gxml_document_get_root ((GXmlDocument*) d);
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp106_ = _tmp105_;
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp107_ = _tmp106_;
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp108_ = gxml_node_get_children_nodes (_tmp107_);
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp109_ = _tmp108_;
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp110_ = _tmp109_;
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp111_ = gee_list_get ((GeeList*) _tmp110_, 1);
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp112_ = (GXmlNode*) _tmp111_;
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp113_ = gxml_node_get_attrs (_tmp112_);
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp114_ = _tmp113_;
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp115_ = _tmp114_;
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp116_ = gee_map_get (_tmp115_, "year");
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp117_ = (GXmlNode*) _tmp116_;
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp118_ = gxml_node_get_value (_tmp117_);
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp119_ = _tmp118_;
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp120_ = _tmp119_;
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp120_, "2016") == 0, "d.root.children_nodes[1].attrs[\"year\"].value == \"2016\"");
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp120_);
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp117_);
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp115_);
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp112_);
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp110_);
#line 719 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp107_);
#line 720 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp121_ = gxml_document_get_root ((GXmlDocument*) d);
#line 720 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp122_ = _tmp121_;
#line 720 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp123_ = _tmp122_;
#line 720 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp124_ = gxml_node_get_children_nodes (_tmp123_);
#line 720 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp125_ = _tmp124_;
#line 720 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp126_ = _tmp125_;
#line 720 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp127_ = gee_list_get ((GeeList*) _tmp126_, 1);
#line 720 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp128_ = (GXmlNode*) _tmp127_;
#line 720 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp129_ = gxml_node_get_children_nodes (_tmp128_);
#line 720 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp130_ = _tmp129_;
#line 720 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp131_ = _tmp130_;
#line 720 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp132_ = gee_list_get ((GeeList*) _tmp131_, 0);
#line 720 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp133_ = (GXmlNode*) _tmp132_;
#line 720 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (_tmp133_ != NULL, "d.root.children_nodes[1].children_nodes[0] != null");
#line 720 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp133_);
#line 720 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp131_);
#line 720 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp128_);
#line 720 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp126_);
#line 720 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp123_);
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp134_ = gxml_document_get_root ((GXmlDocument*) d);
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp135_ = _tmp134_;
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp136_ = _tmp135_;
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp137_ = gxml_node_get_children_nodes (_tmp136_);
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp138_ = _tmp137_;
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp139_ = _tmp138_;
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp140_ = gee_list_get ((GeeList*) _tmp139_, 1);
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp141_ = (GXmlNode*) _tmp140_;
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp142_ = gxml_node_get_children_nodes (_tmp141_);
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp143_ = _tmp142_;
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp144_ = _tmp143_;
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp145_ = gee_list_get ((GeeList*) _tmp144_, 0);
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp146_ = (GXmlNode*) _tmp145_;
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp147_ = gxml_node_get_name (_tmp146_);
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp148_ = _tmp147_;
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp149_ = _tmp148_;
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp149_, "name") == 0, "d.root.children_nodes[1].children_nodes[0].name == \"name\"");
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp149_);
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp146_);
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp144_);
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp141_);
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp139_);
#line 721 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp136_);
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp150_ = gxml_document_get_root ((GXmlDocument*) d);
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp151_ = _tmp150_;
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp152_ = _tmp151_;
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp153_ = gxml_node_get_children_nodes (_tmp152_);
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp154_ = _tmp153_;
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp155_ = _tmp154_;
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp156_ = gee_list_get ((GeeList*) _tmp155_, 1);
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp157_ = (GXmlNode*) _tmp156_;
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp158_ = gxml_node_get_children_nodes (_tmp157_);
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp159_ = _tmp158_;
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp160_ = _tmp159_;
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp161_ = gee_list_get ((GeeList*) _tmp160_, 0);
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp162_ = (GXmlNode*) _tmp161_;
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp163_ = gxml_node_get_children_nodes (_tmp162_);
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp164_ = _tmp163_;
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp165_ = _tmp164_;
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp166_ = gee_list_get ((GeeList*) _tmp165_, 0);
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp167_ = (GXmlNode*) _tmp166_;
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp167_, GXML_TYPE_TEXT), "d.root.children_nodes[1].children_nodes[0].children_nodes[0] is GXml.Text");
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp167_);
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp165_);
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp162_);
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp160_);
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp157_);
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp155_);
#line 722 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp152_);
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp168_ = gxml_document_get_root ((GXmlDocument*) d);
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp169_ = _tmp168_;
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp170_ = _tmp169_;
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp171_ = gxml_node_get_children_nodes (_tmp170_);
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp172_ = _tmp171_;
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp173_ = _tmp172_;
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp174_ = gee_list_get ((GeeList*) _tmp173_, 1);
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp175_ = (GXmlNode*) _tmp174_;
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp176_ = gxml_node_get_children_nodes (_tmp175_);
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp177_ = _tmp176_;
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp178_ = _tmp177_;
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp179_ = gee_list_get ((GeeList*) _tmp178_, 0);
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp180_ = (GXmlNode*) _tmp179_;
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp181_ = gxml_node_get_children_nodes (_tmp180_);
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp182_ = _tmp181_;
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp183_ = _tmp182_;
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp184_ = gee_list_get ((GeeList*) _tmp183_, 0);
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp185_ = (GXmlNode*) _tmp184_;
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp186_ = gxml_node_get_value (_tmp185_);
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp187_ = _tmp186_;
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp188_ = _tmp187_;
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_vala_assert (g_strcmp0 (_tmp188_, "COMMUNICATIONS") == 0, "d.root.children_nodes[1].children_nodes[0].children_nodes[0].value == \"COMMUNICATIONS\"");
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp188_);
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp185_);
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp183_);
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp180_);
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp178_);
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp175_);
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp173_);
#line 723 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (_tmp170_);
#line 702 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (d);
#line 702 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_object_unref0 (f);
#line 12323 "TDocumentTest.c"
}


static void ___lambda125__gtest_func (void) {
#line 702 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda125_ ();
#line 12330 "TDocumentTest.c"
}


static GXmlTDocumentReadType ___lambda127_ (GXmlNode* node, xmlTextReader* tr) {
	GXmlTDocumentReadType result = 0;
	GXmlNode* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	gchar* _tmp2_ = NULL;
	gchar* _tmp3_ = NULL;
	gchar* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	gboolean _tmp6_ = FALSE;
	GXmlNode* _tmp7_ = NULL;
	gchar* _tmp8_ = NULL;
	gchar* _tmp9_ = NULL;
	gchar* _tmp10_ = NULL;
	gboolean _tmp11_ = FALSE;
#line 749 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_return_val_if_fail (node != NULL, 0);
#line 749 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_return_val_if_fail (tr != NULL, 0);
#line 750 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp0_ = node;
#line 750 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp1_ = gxml_node_get_name (_tmp0_);
#line 750 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp2_ = _tmp1_;
#line 750 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp3_ = _tmp2_;
#line 750 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp4_ = g_strconcat ("ReadType check node: ", _tmp3_, NULL);
#line 750 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp5_ = _tmp4_;
#line 750 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_message ("%s", _tmp5_);
#line 750 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp5_);
#line 750 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp3_);
#line 751 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp7_ = node;
#line 751 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp8_ = gxml_node_get_name (_tmp7_);
#line 751 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp9_ = _tmp8_;
#line 751 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp10_ = _tmp9_;
#line 751 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp11_ = g_strcmp0 (_tmp10_, "NoRead") == 0;
#line 751 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp10_);
#line 751 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	if (_tmp11_) {
#line 751 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp6_ = TRUE;
#line 12386 "TDocumentTest.c"
	} else {
		GXmlNode* _tmp12_ = NULL;
		gchar* _tmp13_ = NULL;
		gchar* _tmp14_ = NULL;
		gchar* _tmp15_ = NULL;
#line 751 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp12_ = node;
#line 751 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp13_ = gxml_node_get_name (_tmp12_);
#line 751 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp14_ = _tmp13_;
#line 751 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp15_ = _tmp14_;
#line 751 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp6_ = g_strcmp0 (_tmp15_, "NoReadChild") == 0;
#line 751 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp15_);
#line 12404 "TDocumentTest.c"
	}
#line 751 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	if (_tmp6_) {
#line 12408 "TDocumentTest.c"
		GXmlNode* _tmp16_ = NULL;
		gchar* _tmp17_ = NULL;
		gchar* _tmp18_ = NULL;
		gchar* _tmp19_ = NULL;
		gchar* _tmp20_ = NULL;
		gchar* _tmp21_ = NULL;
#line 752 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp16_ = node;
#line 752 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp17_ = gxml_node_get_name (_tmp16_);
#line 752 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp18_ = _tmp17_;
#line 752 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp19_ = _tmp18_;
#line 752 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp20_ = g_strconcat ("Skiping node: ", _tmp19_, NULL);
#line 752 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp21_ = _tmp20_;
#line 752 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_test_message ("%s", _tmp21_);
#line 752 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp21_);
#line 752 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp19_);
#line 753 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		result = GXML_TDOCUMENT_READ_TYPE_NEXT;
#line 753 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		return result;
#line 12437 "TDocumentTest.c"
	}
#line 755 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	result = GXML_TDOCUMENT_READ_TYPE_CONTINUE;
#line 755 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	return result;
#line 12443 "TDocumentTest.c"
}


static GXmlTDocumentReadType ____lambda127__gxml_tdocument_read_type_func (GXmlNode* node, xmlTextReader* tr, gpointer self) {
	GXmlTDocumentReadType result;
	result = ___lambda127_ (node, tr);
#line 749 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	return result;
#line 12452 "TDocumentTest.c"
}


static GXmlTDocumentReadType ___lambda128_ (GXmlNode* node, xmlTextReader* tr) {
	GXmlTDocumentReadType result = 0;
	GXmlNode* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	gchar* _tmp2_ = NULL;
	gchar* _tmp3_ = NULL;
	gchar* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	GXmlNode* _tmp6_ = NULL;
	gchar* _tmp7_ = NULL;
	gchar* _tmp8_ = NULL;
	gchar* _tmp9_ = NULL;
	gboolean _tmp10_ = FALSE;
#line 772 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_return_val_if_fail (node != NULL, 0);
#line 772 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_return_val_if_fail (tr != NULL, 0);
#line 773 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp0_ = node;
#line 773 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp1_ = gxml_node_get_name (_tmp0_);
#line 773 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp2_ = _tmp1_;
#line 773 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp3_ = _tmp2_;
#line 773 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp4_ = g_strconcat ("ReadType check node: ", _tmp3_, NULL);
#line 773 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp5_ = _tmp4_;
#line 773 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_message ("%s", _tmp5_);
#line 773 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp5_);
#line 773 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp3_);
#line 774 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp6_ = node;
#line 774 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp7_ = gxml_node_get_name (_tmp6_);
#line 774 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp8_ = _tmp7_;
#line 774 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp9_ = _tmp8_;
#line 774 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp10_ = g_strcmp0 (_tmp9_, "NoReadChild") == 0;
#line 774 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (_tmp9_);
#line 774 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	if (_tmp10_) {
#line 12505 "TDocumentTest.c"
		GXmlNode* _tmp11_ = NULL;
		gchar* _tmp12_ = NULL;
		gchar* _tmp13_ = NULL;
		gchar* _tmp14_ = NULL;
		gchar* _tmp15_ = NULL;
		gchar* _tmp16_ = NULL;
#line 775 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp11_ = node;
#line 775 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp12_ = gxml_node_get_name (_tmp11_);
#line 775 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp13_ = _tmp12_;
#line 775 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp14_ = _tmp13_;
#line 775 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp15_ = g_strconcat ("Skiping node: ", _tmp14_, NULL);
#line 775 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp16_ = _tmp15_;
#line 775 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_test_message ("%s", _tmp16_);
#line 775 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp16_);
#line 775 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp14_);
#line 776 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		result = GXML_TDOCUMENT_READ_TYPE_STOP;
#line 776 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		return result;
#line 12534 "TDocumentTest.c"
	}
#line 778 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	result = GXML_TDOCUMENT_READ_TYPE_CONTINUE;
#line 778 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	return result;
#line 12540 "TDocumentTest.c"
}


static GXmlTDocumentReadType ____lambda128__gxml_tdocument_read_type_func (GXmlNode* node, xmlTextReader* tr, gpointer self) {
	GXmlTDocumentReadType result;
	result = ___lambda128_ (node, tr);
#line 772 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	return result;
#line 12549 "TDocumentTest.c"
}


static void __lambda126_ (void) {
	GError * _inner_error_ = NULL;
	{
		GFile* file = NULL;
		GFile* _tmp0_ = NULL;
		GFile* _tmp1_ = NULL;
		gboolean _tmp2_ = FALSE;
		GXmlTDocument* d = NULL;
		GFile* _tmp3_ = NULL;
		GXmlTDocument* _tmp4_ = NULL;
		GXmlTDocument* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GeeBidirList* _tmp7_ = NULL;
		GeeBidirList* _tmp8_ = NULL;
		GXmlTDocument* _tmp9_ = NULL;
		GeeBidirList* _tmp10_ = NULL;
		GeeBidirList* _tmp11_ = NULL;
		GeeBidirList* _tmp12_ = NULL;
		gint _tmp13_ = 0;
		gint _tmp14_ = 0;
		GXmlTDocument* _tmp15_ = NULL;
		GeeBidirList* _tmp16_ = NULL;
		GeeBidirList* _tmp17_ = NULL;
		GeeBidirList* _tmp18_ = NULL;
		gpointer _tmp19_ = NULL;
		GXmlNode* _tmp20_ = NULL;
		GXmlTDocument* _tmp21_ = NULL;
		GeeBidirList* _tmp22_ = NULL;
		GeeBidirList* _tmp23_ = NULL;
		GeeBidirList* _tmp24_ = NULL;
		gpointer _tmp25_ = NULL;
		GXmlNode* _tmp26_ = NULL;
		GXmlTDocument* _tmp27_ = NULL;
		GeeBidirList* _tmp28_ = NULL;
		GeeBidirList* _tmp29_ = NULL;
		GeeBidirList* _tmp30_ = NULL;
		gpointer _tmp31_ = NULL;
		GXmlNode* _tmp32_ = NULL;
		GXmlTDocument* _tmp33_ = NULL;
		GXmlNode* _tmp34_ = NULL;
		GXmlNode* _tmp35_ = NULL;
		GXmlNode* _tmp36_ = NULL;
		GXmlTDocument* _tmp37_ = NULL;
		GXmlNode* _tmp38_ = NULL;
		GXmlNode* _tmp39_ = NULL;
		GXmlNode* _tmp40_ = NULL;
		GeeBidirList* _tmp41_ = NULL;
		GeeBidirList* _tmp42_ = NULL;
		GeeBidirList* _tmp43_ = NULL;
		gint _tmp44_ = 0;
		gint _tmp45_ = 0;
		GXmlNode* n = NULL;
		GXmlTDocument* _tmp46_ = NULL;
		GXmlNode* _tmp47_ = NULL;
		GXmlNode* _tmp48_ = NULL;
		GXmlNode* _tmp49_ = NULL;
		GeeBidirList* _tmp50_ = NULL;
		GeeBidirList* _tmp51_ = NULL;
		GeeBidirList* _tmp52_ = NULL;
		gpointer _tmp53_ = NULL;
		GXmlNode* _tmp54_ = NULL;
		GXmlNode* _tmp55_ = NULL;
		GXmlNode* _tmp56_ = NULL;
		gchar* _tmp57_ = NULL;
		gchar* _tmp58_ = NULL;
		gchar* _tmp59_ = NULL;
		GXmlNode* _tmp60_ = NULL;
		GeeBidirList* _tmp61_ = NULL;
		GeeBidirList* _tmp62_ = NULL;
		GeeBidirList* _tmp63_ = NULL;
		gint _tmp64_ = 0;
		gint _tmp65_ = 0;
		GXmlNode* nc = NULL;
		GXmlNode* _tmp66_ = NULL;
		GeeBidirList* _tmp67_ = NULL;
		GeeBidirList* _tmp68_ = NULL;
		GeeBidirList* _tmp69_ = NULL;
		gpointer _tmp70_ = NULL;
		GXmlNode* _tmp71_ = NULL;
		GXmlNode* _tmp72_ = NULL;
		GXmlNode* _tmp73_ = NULL;
		gchar* _tmp74_ = NULL;
		gchar* _tmp75_ = NULL;
		gchar* _tmp76_ = NULL;
		GXmlNode* _tmp77_ = NULL;
		GeeBidirList* _tmp78_ = NULL;
		GeeBidirList* _tmp79_ = NULL;
		GeeBidirList* _tmp80_ = NULL;
		gint _tmp81_ = 0;
		gint _tmp82_ = 0;
		GXmlTDocumentReadTypeFunc f1 = NULL;
		void* f1_target = NULL;
		GDestroyNotify f1_target_destroy_notify = NULL;
		GXmlTDocument* d2 = NULL;
		GFile* _tmp83_ = NULL;
		GXmlTDocumentReadTypeFunc _tmp84_ = NULL;
		void* _tmp84__target = NULL;
		GXmlTDocument* _tmp85_ = NULL;
		GXmlTDocument* _tmp86_ = NULL;
		GFile* _tmp87_ = NULL;
		GXmlTDocumentReadTypeFunc _tmp88_ = NULL;
		void* _tmp88__target = NULL;
		GXmlTDocument* _tmp89_ = NULL;
		GXmlNode* _tmp90_ = NULL;
		GXmlNode* _tmp91_ = NULL;
		GXmlNode* _tmp92_ = NULL;
		GXmlTDocument* _tmp93_ = NULL;
		GXmlNode* _tmp94_ = NULL;
		GXmlNode* _tmp95_ = NULL;
		GXmlNode* _tmp96_ = NULL;
		GeeBidirList* _tmp97_ = NULL;
		GeeBidirList* _tmp98_ = NULL;
		GeeBidirList* _tmp99_ = NULL;
		gint _tmp100_ = 0;
		gint _tmp101_ = 0;
		GXmlNode* n2 = NULL;
		GXmlTDocument* _tmp102_ = NULL;
		GXmlNode* _tmp103_ = NULL;
		GXmlNode* _tmp104_ = NULL;
		GXmlNode* _tmp105_ = NULL;
		GeeBidirList* _tmp106_ = NULL;
		GeeBidirList* _tmp107_ = NULL;
		GeeBidirList* _tmp108_ = NULL;
		gpointer _tmp109_ = NULL;
		GXmlNode* _tmp110_ = NULL;
		GXmlNode* _tmp111_ = NULL;
		GXmlNode* _tmp112_ = NULL;
		gchar* _tmp113_ = NULL;
		gchar* _tmp114_ = NULL;
		gchar* _tmp115_ = NULL;
		GXmlNode* _tmp116_ = NULL;
		GeeBidirList* _tmp117_ = NULL;
		GeeBidirList* _tmp118_ = NULL;
		GeeBidirList* _tmp119_ = NULL;
		gint _tmp120_ = 0;
		gint _tmp121_ = 0;
		GXmlTDocument* _tmp122_ = NULL;
		gchar* _tmp123_ = NULL;
		gchar* _tmp124_ = NULL;
		GXmlNode* nc2 = NULL;
		GXmlNode* _tmp125_ = NULL;
		GeeBidirList* _tmp126_ = NULL;
		GeeBidirList* _tmp127_ = NULL;
		GeeBidirList* _tmp128_ = NULL;
		gpointer _tmp129_ = NULL;
		GXmlNode* _tmp130_ = NULL;
		GXmlNode* _tmp131_ = NULL;
		GXmlNode* _tmp132_ = NULL;
		gchar* _tmp133_ = NULL;
		gchar* _tmp134_ = NULL;
		gchar* _tmp135_ = NULL;
		GXmlNode* _tmp136_ = NULL;
		GeeBidirList* _tmp137_ = NULL;
		GeeBidirList* _tmp138_ = NULL;
		GeeBidirList* _tmp139_ = NULL;
		gint _tmp140_ = 0;
		gint _tmp141_ = 0;
		GXmlTDocumentReadTypeFunc f2 = NULL;
		void* f2_target = NULL;
		GDestroyNotify f2_target_destroy_notify = NULL;
		GXmlTDocument* d3 = NULL;
		GFile* _tmp142_ = NULL;
		gchar* _tmp143_ = NULL;
		gchar* _tmp144_ = NULL;
		GXmlTDocumentReadTypeFunc _tmp145_ = NULL;
		void* _tmp145__target = NULL;
		GXmlTDocument* _tmp146_ = NULL;
		GXmlTDocument* _tmp147_ = NULL;
		GXmlTDocument* _tmp148_ = NULL;
		gchar* _tmp149_ = NULL;
		gchar* _tmp150_ = NULL;
		GXmlTDocument* _tmp151_ = NULL;
		GXmlNode* _tmp152_ = NULL;
		GXmlNode* _tmp153_ = NULL;
		GXmlNode* _tmp154_ = NULL;
		GXmlTDocument* _tmp155_ = NULL;
		GXmlNode* _tmp156_ = NULL;
		GXmlNode* _tmp157_ = NULL;
		GXmlNode* _tmp158_ = NULL;
		GeeBidirList* _tmp159_ = NULL;
		GeeBidirList* _tmp160_ = NULL;
		GeeBidirList* _tmp161_ = NULL;
		gint _tmp162_ = 0;
		gint _tmp163_ = 0;
		GXmlNode* n3 = NULL;
		GXmlTDocument* _tmp164_ = NULL;
		GXmlNode* _tmp165_ = NULL;
		GXmlNode* _tmp166_ = NULL;
		GXmlNode* _tmp167_ = NULL;
		GeeBidirList* _tmp168_ = NULL;
		GeeBidirList* _tmp169_ = NULL;
		GeeBidirList* _tmp170_ = NULL;
		gpointer _tmp171_ = NULL;
		GXmlNode* _tmp172_ = NULL;
		GXmlNode* _tmp173_ = NULL;
		GXmlNode* _tmp174_ = NULL;
		gchar* _tmp175_ = NULL;
		gchar* _tmp176_ = NULL;
		gchar* _tmp177_ = NULL;
		GXmlNode* _tmp178_ = NULL;
		GeeBidirList* _tmp179_ = NULL;
		GeeBidirList* _tmp180_ = NULL;
		GeeBidirList* _tmp181_ = NULL;
		gint _tmp182_ = 0;
		gint _tmp183_ = 0;
		GXmlNode* nc3 = NULL;
		GXmlNode* _tmp184_ = NULL;
		GeeBidirList* _tmp185_ = NULL;
		GeeBidirList* _tmp186_ = NULL;
		GeeBidirList* _tmp187_ = NULL;
		gpointer _tmp188_ = NULL;
		GXmlNode* _tmp189_ = NULL;
		GXmlNode* _tmp190_ = NULL;
		GXmlNode* _tmp191_ = NULL;
		gchar* _tmp192_ = NULL;
		gchar* _tmp193_ = NULL;
		gchar* _tmp194_ = NULL;
		GXmlNode* _tmp195_ = NULL;
		GeeBidirList* _tmp196_ = NULL;
		GeeBidirList* _tmp197_ = NULL;
		GeeBidirList* _tmp198_ = NULL;
		gint _tmp199_ = 0;
		gint _tmp200_ = 0;
		GXmlTDocument* d4 = NULL;
		GFile* _tmp201_ = NULL;
		gchar* _tmp202_ = NULL;
		gchar* _tmp203_ = NULL;
		GXmlTDocumentReadTypeFunc _tmp204_ = NULL;
		void* _tmp204__target = NULL;
		GXmlTDocument* _tmp205_ = NULL;
		GXmlTDocument* _tmp206_ = NULL;
		GXmlTDocument* _tmp207_ = NULL;
		gchar* _tmp208_ = NULL;
		gchar* _tmp209_ = NULL;
		GXmlTDocument* _tmp210_ = NULL;
		GXmlNode* _tmp211_ = NULL;
		GXmlNode* _tmp212_ = NULL;
		GXmlNode* _tmp213_ = NULL;
		GXmlTDocument* _tmp214_ = NULL;
		GXmlNode* _tmp215_ = NULL;
		GXmlNode* _tmp216_ = NULL;
		GXmlNode* _tmp217_ = NULL;
		GeeBidirList* _tmp218_ = NULL;
		GeeBidirList* _tmp219_ = NULL;
		GeeBidirList* _tmp220_ = NULL;
		gint _tmp221_ = 0;
		gint _tmp222_ = 0;
		GXmlNode* n4 = NULL;
		GXmlTDocument* _tmp223_ = NULL;
		GXmlNode* _tmp224_ = NULL;
		GXmlNode* _tmp225_ = NULL;
		GXmlNode* _tmp226_ = NULL;
		GeeBidirList* _tmp227_ = NULL;
		GeeBidirList* _tmp228_ = NULL;
		GeeBidirList* _tmp229_ = NULL;
		gpointer _tmp230_ = NULL;
		GXmlNode* _tmp231_ = NULL;
		GXmlNode* _tmp232_ = NULL;
		GXmlNode* _tmp233_ = NULL;
		gchar* _tmp234_ = NULL;
		gchar* _tmp235_ = NULL;
		gchar* _tmp236_ = NULL;
		GXmlNode* _tmp237_ = NULL;
		GeeBidirList* _tmp238_ = NULL;
		GeeBidirList* _tmp239_ = NULL;
		GeeBidirList* _tmp240_ = NULL;
		gint _tmp241_ = 0;
		gint _tmp242_ = 0;
		GXmlNode* nc4 = NULL;
		GXmlNode* _tmp243_ = NULL;
		GeeBidirList* _tmp244_ = NULL;
		GeeBidirList* _tmp245_ = NULL;
		GeeBidirList* _tmp246_ = NULL;
		gpointer _tmp247_ = NULL;
		GXmlNode* _tmp248_ = NULL;
		GXmlNode* _tmp249_ = NULL;
		GXmlNode* _tmp250_ = NULL;
		gchar* _tmp251_ = NULL;
		gchar* _tmp252_ = NULL;
		gchar* _tmp253_ = NULL;
		GXmlNode* _tmp254_ = NULL;
		GeeBidirList* _tmp255_ = NULL;
		GeeBidirList* _tmp256_ = NULL;
		GeeBidirList* _tmp257_ = NULL;
		gint _tmp258_ = 0;
		gint _tmp259_ = 0;
		GFile* file2 = NULL;
		GFile* _tmp260_ = NULL;
		GFileInputStream* _tmp261_ = NULL;
		GFile* _tmp262_ = NULL;
		GFileInputStream* _tmp263_ = NULL;
		GXmlTDocument* d5 = NULL;
		GXmlTDocumentReadTypeFunc _tmp264_ = NULL;
		void* _tmp264__target = NULL;
		GXmlTDocument* _tmp265_ = NULL;
		GXmlNode* _tmp266_ = NULL;
		GXmlNode* _tmp267_ = NULL;
		GXmlNode* _tmp268_ = NULL;
		GXmlNode* _tmp269_ = NULL;
		GXmlNode* _tmp270_ = NULL;
		GXmlNode* _tmp271_ = NULL;
		GeeBidirList* _tmp272_ = NULL;
		GeeBidirList* _tmp273_ = NULL;
		GeeBidirList* _tmp274_ = NULL;
		gint _tmp275_ = 0;
		gint _tmp276_ = 0;
		GXmlNode* n5 = NULL;
		GXmlNode* _tmp277_ = NULL;
		GXmlNode* _tmp278_ = NULL;
		GXmlNode* _tmp279_ = NULL;
		GeeBidirList* _tmp280_ = NULL;
		GeeBidirList* _tmp281_ = NULL;
		GeeBidirList* _tmp282_ = NULL;
		gpointer _tmp283_ = NULL;
		GXmlNode* _tmp284_ = NULL;
		gchar* _tmp285_ = NULL;
		gchar* _tmp286_ = NULL;
		gchar* _tmp287_ = NULL;
		GeeBidirList* _tmp288_ = NULL;
		GeeBidirList* _tmp289_ = NULL;
		GeeBidirList* _tmp290_ = NULL;
		gint _tmp291_ = 0;
		gint _tmp292_ = 0;
		GXmlTDocument* _tmp293_ = NULL;
		gchar* _tmp294_ = NULL;
		gchar* _tmp295_ = NULL;
		GXmlNode* nc5 = NULL;
		GeeBidirList* _tmp296_ = NULL;
		GeeBidirList* _tmp297_ = NULL;
		GeeBidirList* _tmp298_ = NULL;
		gpointer _tmp299_ = NULL;
		GXmlNode* _tmp300_ = NULL;
		gchar* _tmp301_ = NULL;
		gchar* _tmp302_ = NULL;
		gchar* _tmp303_ = NULL;
		GeeBidirList* _tmp304_ = NULL;
		GeeBidirList* _tmp305_ = NULL;
		GeeBidirList* _tmp306_ = NULL;
		gint _tmp307_ = 0;
		gint _tmp308_ = 0;
		GXmlTDocument* d6 = NULL;
		GXmlTDocumentReadTypeFunc _tmp309_ = NULL;
		void* _tmp309__target = NULL;
		GXmlTDocument* _tmp310_ = NULL;
		GXmlNode* _tmp311_ = NULL;
		GXmlNode* _tmp312_ = NULL;
		GXmlNode* _tmp313_ = NULL;
		GXmlNode* _tmp314_ = NULL;
		GXmlNode* _tmp315_ = NULL;
		GXmlNode* _tmp316_ = NULL;
		GeeBidirList* _tmp317_ = NULL;
		GeeBidirList* _tmp318_ = NULL;
		GeeBidirList* _tmp319_ = NULL;
		gint _tmp320_ = 0;
		gint _tmp321_ = 0;
		GXmlNode* n6 = NULL;
		GXmlNode* _tmp322_ = NULL;
		GXmlNode* _tmp323_ = NULL;
		GXmlNode* _tmp324_ = NULL;
		GeeBidirList* _tmp325_ = NULL;
		GeeBidirList* _tmp326_ = NULL;
		GeeBidirList* _tmp327_ = NULL;
		gpointer _tmp328_ = NULL;
		GXmlNode* _tmp329_ = NULL;
		gchar* _tmp330_ = NULL;
		gchar* _tmp331_ = NULL;
		gchar* _tmp332_ = NULL;
#line 727 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp0_ = g_file_new_for_path (TEST_DIR "/t-read-test.xml");
#line 727 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		file = _tmp0_;
#line 728 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp1_ = file;
#line 728 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp2_ = g_file_query_exists (_tmp1_, NULL);
#line 728 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp2_, "file.query_exists ()");
#line 729 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp3_ = file;
#line 729 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp4_ = gxml_tdocument_new_from_file (_tmp3_);
#line 729 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		d = _tmp4_;
#line 730 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp5_ = d;
#line 730 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp6_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp5_);
#line 730 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp7_ = _tmp6_;
#line 730 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp8_ = _tmp7_;
#line 730 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp8_ != NULL, "d.children_nodes != null");
#line 730 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp8_);
#line 731 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp9_ = d;
#line 731 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp10_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp9_);
#line 731 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp11_ = _tmp10_;
#line 731 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp12_ = _tmp11_;
#line 731 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp13_ = gee_collection_get_size ((GeeCollection*) _tmp12_);
#line 731 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp14_ = _tmp13_;
#line 731 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp14_ == 3, "d.children_nodes.size == 3");
#line 731 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp12_);
#line 732 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp15_ = d;
#line 732 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp16_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp15_);
#line 732 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp17_ = _tmp16_;
#line 732 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp18_ = _tmp17_;
#line 732 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp19_ = gee_list_get ((GeeList*) _tmp18_, 0);
#line 732 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp20_ = (GXmlNode*) _tmp19_;
#line 732 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp20_, GXML_TYPE_COMMENT), "d.children_nodes[0] is GXml.Comment");
#line 732 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp20_);
#line 732 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp18_);
#line 733 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp21_ = d;
#line 733 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp22_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp21_);
#line 733 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp23_ = _tmp22_;
#line 733 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp24_ = _tmp23_;
#line 733 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp25_ = gee_list_get ((GeeList*) _tmp24_, 1);
#line 733 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp26_ = (GXmlNode*) _tmp25_;
#line 733 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp26_, GXML_TYPE_PROCESSING_INSTRUCTION), "d.children_nodes[1] is GXml.ProcessingInstruction");
#line 733 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp26_);
#line 733 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp24_);
#line 734 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp27_ = d;
#line 734 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp28_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp27_);
#line 734 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp29_ = _tmp28_;
#line 734 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp30_ = _tmp29_;
#line 734 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp31_ = gee_list_get ((GeeList*) _tmp30_, 2);
#line 734 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp32_ = (GXmlNode*) _tmp31_;
#line 734 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp32_, GXML_TYPE_ELEMENT), "d.children_nodes[2] is GXml.Element");
#line 734 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp32_);
#line 734 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp30_);
#line 735 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp33_ = d;
#line 735 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp34_ = gxml_document_get_root ((GXmlDocument*) _tmp33_);
#line 735 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp35_ = _tmp34_;
#line 735 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp36_ = _tmp35_;
#line 735 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp36_ != NULL, "d.root != null");
#line 735 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp36_);
#line 736 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp37_ = d;
#line 736 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp38_ = gxml_document_get_root ((GXmlDocument*) _tmp37_);
#line 736 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp39_ = _tmp38_;
#line 736 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp40_ = _tmp39_;
#line 736 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp41_ = gxml_node_get_children_nodes (_tmp40_);
#line 736 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp42_ = _tmp41_;
#line 736 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp43_ = _tmp42_;
#line 736 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp44_ = gee_collection_get_size ((GeeCollection*) _tmp43_);
#line 736 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp45_ = _tmp44_;
#line 736 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp45_ == 7, "d.root.children_nodes.size == 7");
#line 736 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp43_);
#line 736 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp40_);
#line 737 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp46_ = d;
#line 737 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp47_ = gxml_document_get_root ((GXmlDocument*) _tmp46_);
#line 737 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp48_ = _tmp47_;
#line 737 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp49_ = _tmp48_;
#line 737 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp50_ = gxml_node_get_children_nodes (_tmp49_);
#line 737 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp51_ = _tmp50_;
#line 737 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp52_ = _tmp51_;
#line 737 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp53_ = gee_list_get ((GeeList*) _tmp52_, 6);
#line 737 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp54_ = (GXmlNode*) _tmp53_;
#line 737 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp52_);
#line 737 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp49_);
#line 737 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		n = _tmp54_;
#line 738 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp55_ = n;
#line 738 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp55_ != NULL, "n != null");
#line 739 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp56_ = n;
#line 739 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp57_ = gxml_node_get_name (_tmp56_);
#line 739 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp58_ = _tmp57_;
#line 739 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp59_ = _tmp58_;
#line 739 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp59_, "ReadTop") == 0, "n.name == \"ReadTop\"");
#line 739 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp59_);
#line 740 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp60_ = n;
#line 740 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp61_ = gxml_node_get_children_nodes (_tmp60_);
#line 740 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp62_ = _tmp61_;
#line 740 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp63_ = _tmp62_;
#line 740 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp64_ = gee_collection_get_size ((GeeCollection*) _tmp63_);
#line 740 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp65_ = _tmp64_;
#line 740 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp65_ == 9, "n.children_nodes.size == 9");
#line 740 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp63_);
#line 741 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp66_ = n;
#line 741 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp67_ = gxml_node_get_children_nodes (_tmp66_);
#line 741 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp68_ = _tmp67_;
#line 741 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp69_ = _tmp68_;
#line 741 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp70_ = gee_list_get ((GeeList*) _tmp69_, 3);
#line 741 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp71_ = (GXmlNode*) _tmp70_;
#line 741 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp69_);
#line 741 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		nc = _tmp71_;
#line 742 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp72_ = nc;
#line 742 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp72_ != NULL, "nc != null");
#line 743 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp73_ = nc;
#line 743 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp74_ = gxml_node_get_name (_tmp73_);
#line 743 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp75_ = _tmp74_;
#line 743 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp76_ = _tmp75_;
#line 743 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp76_, "Read") == 0, "nc.name == \"Read\"");
#line 743 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp76_);
#line 744 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp77_ = nc;
#line 744 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp78_ = gxml_node_get_children_nodes (_tmp77_);
#line 744 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp79_ = _tmp78_;
#line 744 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp80_ = _tmp79_;
#line 744 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp81_ = gee_collection_get_size ((GeeCollection*) _tmp80_);
#line 744 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp82_ = _tmp81_;
#line 744 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp82_ == 2, "nc.children_nodes.size == 2");
#line 744 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp80_);
#line 749 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f1 = ____lambda127__gxml_tdocument_read_type_func;
#line 749 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f1_target = NULL;
#line 749 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f1_target_destroy_notify = NULL;
#line 757 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp83_ = file;
#line 757 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp84_ = f1;
#line 757 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp84__target = f1_target;
#line 757 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp85_ = gxml_tdocument_new_from_file_with_readtype_func (_tmp83_, _tmp84_, _tmp84__target);
#line 757 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		d2 = _tmp85_;
#line 758 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp86_ = d2;
#line 758 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp87_ = file;
#line 758 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp88_ = f1;
#line 758 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp88__target = f1_target;
#line 758 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		gxml_tdocument_read_doc ((GXmlDocument*) _tmp86_, _tmp87_, _tmp88_, _tmp88__target, &_inner_error_);
#line 758 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 758 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d2);
#line 758 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			(f1_target_destroy_notify == NULL) ? NULL : (f1_target_destroy_notify (f1_target), NULL);
#line 758 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			f1 = NULL;
#line 758 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			f1_target = NULL;
#line 758 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			f1_target_destroy_notify = NULL;
#line 758 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (nc);
#line 758 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (n);
#line 758 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 758 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (file);
#line 13204 "TDocumentTest.c"
			goto __catch139_g_error;
		}
#line 759 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp89_ = d2;
#line 759 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp90_ = gxml_document_get_root ((GXmlDocument*) _tmp89_);
#line 759 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp91_ = _tmp90_;
#line 759 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp92_ = _tmp91_;
#line 759 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp92_ != NULL, "d2.root != null");
#line 759 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp92_);
#line 760 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp93_ = d2;
#line 760 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp94_ = gxml_document_get_root ((GXmlDocument*) _tmp93_);
#line 760 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp95_ = _tmp94_;
#line 760 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp96_ = _tmp95_;
#line 760 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp97_ = gxml_node_get_children_nodes (_tmp96_);
#line 760 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp98_ = _tmp97_;
#line 760 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp99_ = _tmp98_;
#line 760 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp100_ = gee_collection_get_size ((GeeCollection*) _tmp99_);
#line 760 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp101_ = _tmp100_;
#line 760 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp101_ == 7, "d2.root.children_nodes.size == 7");
#line 760 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp99_);
#line 760 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp96_);
#line 761 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp102_ = d2;
#line 761 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp103_ = gxml_document_get_root ((GXmlDocument*) _tmp102_);
#line 761 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp104_ = _tmp103_;
#line 761 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp105_ = _tmp104_;
#line 761 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp106_ = gxml_node_get_children_nodes (_tmp105_);
#line 761 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp107_ = _tmp106_;
#line 761 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp108_ = _tmp107_;
#line 761 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp109_ = gee_list_get ((GeeList*) _tmp108_, 6);
#line 761 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp110_ = (GXmlNode*) _tmp109_;
#line 761 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp108_);
#line 761 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp105_);
#line 761 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		n2 = _tmp110_;
#line 762 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp111_ = n2;
#line 762 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp111_ != NULL, "n2 != null");
#line 763 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp112_ = n2;
#line 763 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp113_ = gxml_node_get_name (_tmp112_);
#line 763 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp114_ = _tmp113_;
#line 763 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp115_ = _tmp114_;
#line 763 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp115_, "ReadTop") == 0, "n2.name == \"ReadTop\"");
#line 763 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp115_);
#line 764 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp116_ = n2;
#line 764 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp117_ = gxml_node_get_children_nodes (_tmp116_);
#line 764 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp118_ = _tmp117_;
#line 764 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp119_ = _tmp118_;
#line 764 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp120_ = gee_collection_get_size ((GeeCollection*) _tmp119_);
#line 764 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp121_ = _tmp120_;
#line 764 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp121_ == 4, "n2.children_nodes.size == 4");
#line 764 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp119_);
#line 765 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp122_ = d2;
#line 765 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp123_ = gxml_tnode_to_string ((GXmlTNode*) _tmp122_);
#line 765 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp124_ = _tmp123_;
#line 765 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_test_message ("%s", _tmp124_);
#line 765 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp124_);
#line 766 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp125_ = n2;
#line 766 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp126_ = gxml_node_get_children_nodes (_tmp125_);
#line 766 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp127_ = _tmp126_;
#line 766 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp128_ = _tmp127_;
#line 766 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp129_ = gee_list_get ((GeeList*) _tmp128_, 2);
#line 766 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp130_ = (GXmlNode*) _tmp129_;
#line 766 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp128_);
#line 766 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		nc2 = _tmp130_;
#line 767 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp131_ = nc2;
#line 767 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp131_ != NULL, "nc2 != null");
#line 768 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp132_ = nc2;
#line 768 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp133_ = gxml_node_get_name (_tmp132_);
#line 768 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp134_ = _tmp133_;
#line 768 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp135_ = _tmp134_;
#line 768 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp135_, "Read") == 0, "nc2.name == \"Read\"");
#line 768 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp135_);
#line 769 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp136_ = nc2;
#line 769 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp137_ = gxml_node_get_children_nodes (_tmp136_);
#line 769 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp138_ = _tmp137_;
#line 769 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp139_ = _tmp138_;
#line 769 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp140_ = gee_collection_get_size ((GeeCollection*) _tmp139_);
#line 769 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp141_ = _tmp140_;
#line 769 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp141_ == 1, "nc2.children_nodes.size == 1");
#line 769 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp139_);
#line 771 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_test_message ("from path");
#line 772 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f2 = ____lambda128__gxml_tdocument_read_type_func;
#line 772 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f2_target = NULL;
#line 772 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f2_target_destroy_notify = NULL;
#line 780 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp142_ = file;
#line 780 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp143_ = g_file_get_path (_tmp142_);
#line 780 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp144_ = _tmp143_;
#line 780 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp145_ = f2;
#line 780 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp145__target = f2_target;
#line 780 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp146_ = gxml_tdocument_new_from_path_with_readtype_func (_tmp144_, _tmp145_, _tmp145__target);
#line 780 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp147_ = _tmp146_;
#line 780 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp144_);
#line 780 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		d3 = _tmp147_;
#line 781 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp148_ = d3;
#line 781 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp149_ = gxml_tnode_to_string ((GXmlTNode*) _tmp148_);
#line 781 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp150_ = _tmp149_;
#line 781 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_test_message ("%s", _tmp150_);
#line 781 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp150_);
#line 782 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp151_ = d3;
#line 782 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp152_ = gxml_document_get_root ((GXmlDocument*) _tmp151_);
#line 782 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp153_ = _tmp152_;
#line 782 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp154_ = _tmp153_;
#line 782 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp154_ != NULL, "d3.root != null");
#line 782 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp154_);
#line 783 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp155_ = d3;
#line 783 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp156_ = gxml_document_get_root ((GXmlDocument*) _tmp155_);
#line 783 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp157_ = _tmp156_;
#line 783 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp158_ = _tmp157_;
#line 783 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp159_ = gxml_node_get_children_nodes (_tmp158_);
#line 783 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp160_ = _tmp159_;
#line 783 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp161_ = _tmp160_;
#line 783 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp162_ = gee_collection_get_size ((GeeCollection*) _tmp161_);
#line 783 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp163_ = _tmp162_;
#line 783 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp163_ == 7, "d3.root.children_nodes.size == 7");
#line 783 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp161_);
#line 783 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp158_);
#line 784 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp164_ = d3;
#line 784 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp165_ = gxml_document_get_root ((GXmlDocument*) _tmp164_);
#line 784 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp166_ = _tmp165_;
#line 784 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp167_ = _tmp166_;
#line 784 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp168_ = gxml_node_get_children_nodes (_tmp167_);
#line 784 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp169_ = _tmp168_;
#line 784 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp170_ = _tmp169_;
#line 784 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp171_ = gee_list_get ((GeeList*) _tmp170_, 6);
#line 784 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp172_ = (GXmlNode*) _tmp171_;
#line 784 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp170_);
#line 784 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp167_);
#line 784 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		n3 = _tmp172_;
#line 785 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp173_ = n3;
#line 785 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp173_ != NULL, "n3 != null");
#line 786 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp174_ = n3;
#line 786 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp175_ = gxml_node_get_name (_tmp174_);
#line 786 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp176_ = _tmp175_;
#line 786 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp177_ = _tmp176_;
#line 786 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp177_, "ReadTop") == 0, "n3.name == \"ReadTop\"");
#line 786 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp177_);
#line 787 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp178_ = n3;
#line 787 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp179_ = gxml_node_get_children_nodes (_tmp178_);
#line 787 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp180_ = _tmp179_;
#line 787 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp181_ = _tmp180_;
#line 787 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp182_ = gee_collection_get_size ((GeeCollection*) _tmp181_);
#line 787 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp183_ = _tmp182_;
#line 787 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp183_ == 4, "n3.children_nodes.size == 4");
#line 787 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp181_);
#line 788 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp184_ = n3;
#line 788 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp185_ = gxml_node_get_children_nodes (_tmp184_);
#line 788 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp186_ = _tmp185_;
#line 788 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp187_ = _tmp186_;
#line 788 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp188_ = gee_list_get ((GeeList*) _tmp187_, 3);
#line 788 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp189_ = (GXmlNode*) _tmp188_;
#line 788 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp187_);
#line 788 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		nc3 = _tmp189_;
#line 789 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp190_ = nc3;
#line 789 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp190_ != NULL, "nc3 != null");
#line 790 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp191_ = nc3;
#line 790 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp192_ = gxml_node_get_name (_tmp191_);
#line 790 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp193_ = _tmp192_;
#line 790 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp194_ = _tmp193_;
#line 790 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp194_, "Read") == 0, "nc3.name == \"Read\"");
#line 790 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp194_);
#line 791 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp195_ = nc3;
#line 791 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp196_ = gxml_node_get_children_nodes (_tmp195_);
#line 791 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp197_ = _tmp196_;
#line 791 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp198_ = _tmp197_;
#line 791 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp199_ = gee_collection_get_size ((GeeCollection*) _tmp198_);
#line 791 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp200_ = _tmp199_;
#line 791 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp200_ == 1, "nc3.children_nodes.size == 1");
#line 791 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp198_);
#line 796 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp201_ = file;
#line 796 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp202_ = g_file_get_uri (_tmp201_);
#line 796 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp203_ = _tmp202_;
#line 796 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp204_ = f2;
#line 796 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp204__target = f2_target;
#line 796 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp205_ = gxml_tdocument_new_from_uri_with_readtype_func (_tmp203_, _tmp204_, _tmp204__target);
#line 796 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp206_ = _tmp205_;
#line 796 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp203_);
#line 796 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		d4 = _tmp206_;
#line 797 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp207_ = d3;
#line 797 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp208_ = gxml_tnode_to_string ((GXmlTNode*) _tmp207_);
#line 797 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp209_ = _tmp208_;
#line 797 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_test_message ("%s", _tmp209_);
#line 797 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp209_);
#line 798 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp210_ = d4;
#line 798 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp211_ = gxml_document_get_root ((GXmlDocument*) _tmp210_);
#line 798 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp212_ = _tmp211_;
#line 798 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp213_ = _tmp212_;
#line 798 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp213_ != NULL, "d4.root != null");
#line 798 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp213_);
#line 799 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp214_ = d4;
#line 799 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp215_ = gxml_document_get_root ((GXmlDocument*) _tmp214_);
#line 799 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp216_ = _tmp215_;
#line 799 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp217_ = _tmp216_;
#line 799 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp218_ = gxml_node_get_children_nodes (_tmp217_);
#line 799 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp219_ = _tmp218_;
#line 799 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp220_ = _tmp219_;
#line 799 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp221_ = gee_collection_get_size ((GeeCollection*) _tmp220_);
#line 799 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp222_ = _tmp221_;
#line 799 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp222_ == 7, "d4.root.children_nodes.size == 7");
#line 799 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp220_);
#line 799 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp217_);
#line 800 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp223_ = d4;
#line 800 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp224_ = gxml_document_get_root ((GXmlDocument*) _tmp223_);
#line 800 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp225_ = _tmp224_;
#line 800 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp226_ = _tmp225_;
#line 800 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp227_ = gxml_node_get_children_nodes (_tmp226_);
#line 800 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp228_ = _tmp227_;
#line 800 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp229_ = _tmp228_;
#line 800 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp230_ = gee_list_get ((GeeList*) _tmp229_, 6);
#line 800 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp231_ = (GXmlNode*) _tmp230_;
#line 800 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp229_);
#line 800 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp226_);
#line 800 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		n4 = _tmp231_;
#line 801 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp232_ = n4;
#line 801 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp232_ != NULL, "n4 != null");
#line 802 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp233_ = n4;
#line 802 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp234_ = gxml_node_get_name (_tmp233_);
#line 802 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp235_ = _tmp234_;
#line 802 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp236_ = _tmp235_;
#line 802 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp236_, "ReadTop") == 0, "n4.name == \"ReadTop\"");
#line 802 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp236_);
#line 803 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp237_ = n4;
#line 803 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp238_ = gxml_node_get_children_nodes (_tmp237_);
#line 803 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp239_ = _tmp238_;
#line 803 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp240_ = _tmp239_;
#line 803 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp241_ = gee_collection_get_size ((GeeCollection*) _tmp240_);
#line 803 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp242_ = _tmp241_;
#line 803 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp242_ == 4, "n4.children_nodes.size == 4");
#line 803 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp240_);
#line 804 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp243_ = n4;
#line 804 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp244_ = gxml_node_get_children_nodes (_tmp243_);
#line 804 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp245_ = _tmp244_;
#line 804 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp246_ = _tmp245_;
#line 804 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp247_ = gee_list_get ((GeeList*) _tmp246_, 3);
#line 804 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp248_ = (GXmlNode*) _tmp247_;
#line 804 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp246_);
#line 804 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		nc4 = _tmp248_;
#line 805 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp249_ = nc4;
#line 805 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp249_ != NULL, "nc4 != null");
#line 806 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp250_ = nc4;
#line 806 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp251_ = gxml_node_get_name (_tmp250_);
#line 806 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp252_ = _tmp251_;
#line 806 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp253_ = _tmp252_;
#line 806 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp253_, "Read") == 0, "nc4.name == \"Read\"");
#line 806 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp253_);
#line 807 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp254_ = nc4;
#line 807 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp255_ = gxml_node_get_children_nodes (_tmp254_);
#line 807 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp256_ = _tmp255_;
#line 807 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp257_ = _tmp256_;
#line 807 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp258_ = gee_collection_get_size ((GeeCollection*) _tmp257_);
#line 807 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp259_ = _tmp258_;
#line 807 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp259_ == 1, "nc4.children_nodes.size == 1");
#line 807 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp257_);
#line 812 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp260_ = g_file_new_for_path (TEST_DIR "/t-read-test.xml");
#line 812 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		file2 = _tmp260_;
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp262_ = file2;
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp263_ = g_file_read (_tmp262_, NULL, &_inner_error_);
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp261_ = _tmp263_;
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (file2);
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (nc4);
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (n4);
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d4);
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (nc3);
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (n3);
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d3);
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			(f2_target_destroy_notify == NULL) ? NULL : (f2_target_destroy_notify (f2_target), NULL);
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			f2 = NULL;
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			f2_target = NULL;
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			f2_target_destroy_notify = NULL;
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (nc2);
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (n2);
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d2);
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			(f1_target_destroy_notify == NULL) ? NULL : (f1_target_destroy_notify (f1_target), NULL);
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			f1 = NULL;
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			f1_target = NULL;
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			f1_target_destroy_notify = NULL;
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (nc);
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (n);
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (d);
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
			_g_object_unref0 (file);
#line 13757 "TDocumentTest.c"
			goto __catch139_g_error;
		}
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp264_ = f1;
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp264__target = f1_target;
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp265_ = gxml_tdocument_new_from_stream_with_readtype_func ((GInputStream*) _tmp261_, _tmp264_, _tmp264__target);
#line 813 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		d5 = _tmp265_;
#line 814 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp266_ = gxml_document_get_root ((GXmlDocument*) d5);
#line 814 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp267_ = _tmp266_;
#line 814 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp268_ = _tmp267_;
#line 814 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp268_ != NULL, "d5.root != null");
#line 814 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp268_);
#line 815 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp269_ = gxml_document_get_root ((GXmlDocument*) d5);
#line 815 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp270_ = _tmp269_;
#line 815 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp271_ = _tmp270_;
#line 815 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp272_ = gxml_node_get_children_nodes (_tmp271_);
#line 815 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp273_ = _tmp272_;
#line 815 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp274_ = _tmp273_;
#line 815 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp275_ = gee_collection_get_size ((GeeCollection*) _tmp274_);
#line 815 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp276_ = _tmp275_;
#line 815 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp276_ == 7, "d5.root.children_nodes.size == 7");
#line 815 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp274_);
#line 815 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp271_);
#line 816 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp277_ = gxml_document_get_root ((GXmlDocument*) d5);
#line 816 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp278_ = _tmp277_;
#line 816 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp279_ = _tmp278_;
#line 816 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp280_ = gxml_node_get_children_nodes (_tmp279_);
#line 816 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp281_ = _tmp280_;
#line 816 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp282_ = _tmp281_;
#line 816 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp283_ = gee_list_get ((GeeList*) _tmp282_, 6);
#line 816 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp284_ = (GXmlNode*) _tmp283_;
#line 816 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp282_);
#line 816 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp279_);
#line 816 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		n5 = _tmp284_;
#line 817 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (n5 != NULL, "n5 != null");
#line 818 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp285_ = gxml_node_get_name (n5);
#line 818 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp286_ = _tmp285_;
#line 818 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp287_ = _tmp286_;
#line 818 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp287_, "ReadTop") == 0, "n5.name == \"ReadTop\"");
#line 818 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp287_);
#line 819 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp288_ = gxml_node_get_children_nodes (n5);
#line 819 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp289_ = _tmp288_;
#line 819 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp290_ = _tmp289_;
#line 819 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp291_ = gee_collection_get_size ((GeeCollection*) _tmp290_);
#line 819 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp292_ = _tmp291_;
#line 819 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp292_ == 4, "n5.children_nodes.size == 4");
#line 819 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp290_);
#line 820 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp293_ = d2;
#line 820 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp294_ = gxml_tnode_to_string ((GXmlTNode*) _tmp293_);
#line 820 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp295_ = _tmp294_;
#line 820 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_test_message ("%s", _tmp295_);
#line 820 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp295_);
#line 821 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp296_ = gxml_node_get_children_nodes (n5);
#line 821 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp297_ = _tmp296_;
#line 821 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp298_ = _tmp297_;
#line 821 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp299_ = gee_list_get ((GeeList*) _tmp298_, 2);
#line 821 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp300_ = (GXmlNode*) _tmp299_;
#line 821 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp298_);
#line 821 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		nc5 = _tmp300_;
#line 822 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (nc5 != NULL, "nc5 != null");
#line 823 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp301_ = gxml_node_get_name (nc5);
#line 823 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp302_ = _tmp301_;
#line 823 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp303_ = _tmp302_;
#line 823 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp303_, "Read") == 0, "nc5.name == \"Read\"");
#line 823 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp303_);
#line 824 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp304_ = gxml_node_get_children_nodes (nc5);
#line 824 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp305_ = _tmp304_;
#line 824 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp306_ = _tmp305_;
#line 824 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp307_ = gee_collection_get_size ((GeeCollection*) _tmp306_);
#line 824 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp308_ = _tmp307_;
#line 824 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp308_ == 1, "nc5.children_nodes.size == 1");
#line 824 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp306_);
#line 829 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp309_ = f1;
#line 829 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp309__target = f1_target;
#line 829 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp310_ = gxml_tdocument_new_from_string_with_readtype_func ("<root><Read/><NoRead/><NoRead/><Read/><NoRead/></root>", _tmp309_, _tmp309__target);
#line 829 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		d6 = _tmp310_;
#line 830 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp311_ = gxml_document_get_root ((GXmlDocument*) d6);
#line 830 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp312_ = _tmp311_;
#line 830 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp313_ = _tmp312_;
#line 830 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp313_ != NULL, "d6.root != null");
#line 830 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp313_);
#line 831 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp314_ = gxml_document_get_root ((GXmlDocument*) d6);
#line 831 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp315_ = _tmp314_;
#line 831 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp316_ = _tmp315_;
#line 831 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp317_ = gxml_node_get_children_nodes (_tmp316_);
#line 831 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp318_ = _tmp317_;
#line 831 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp319_ = _tmp318_;
#line 831 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp320_ = gee_collection_get_size ((GeeCollection*) _tmp319_);
#line 831 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp321_ = _tmp320_;
#line 831 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (_tmp321_ == 2, "d6.root.children_nodes.size == 2");
#line 831 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp319_);
#line 831 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp316_);
#line 832 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp322_ = gxml_document_get_root ((GXmlDocument*) d6);
#line 832 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp323_ = _tmp322_;
#line 832 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp324_ = _tmp323_;
#line 832 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp325_ = gxml_node_get_children_nodes (_tmp324_);
#line 832 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp326_ = _tmp325_;
#line 832 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp327_ = _tmp326_;
#line 832 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp328_ = gee_list_get ((GeeList*) _tmp327_, 1);
#line 832 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp329_ = (GXmlNode*) _tmp328_;
#line 832 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp327_);
#line 832 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp324_);
#line 832 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		n6 = _tmp329_;
#line 833 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (n6 != NULL, "n6 != null");
#line 834 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp330_ = gxml_node_get_name (n6);
#line 834 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp331_ = _tmp330_;
#line 834 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp332_ = _tmp331_;
#line 834 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_vala_assert (g_strcmp0 (_tmp332_, "Read") == 0, "n6.name == \"Read\"");
#line 834 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp332_);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (n6);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (d6);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (nc5);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (n5);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (d5);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (_tmp261_);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (file2);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (nc4);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (n4);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (d4);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (nc3);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (n3);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (d3);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		(f2_target_destroy_notify == NULL) ? NULL : (f2_target_destroy_notify (f2_target), NULL);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f2 = NULL;
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f2_target = NULL;
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f2_target_destroy_notify = NULL;
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (nc2);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (n2);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (d2);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		(f1_target_destroy_notify == NULL) ? NULL : (f1_target_destroy_notify (f1_target), NULL);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f1 = NULL;
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f1_target = NULL;
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		f1_target_destroy_notify = NULL;
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (nc);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (n);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (d);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_object_unref0 (file);
#line 14028 "TDocumentTest.c"
	}
	goto __finally139;
	__catch139_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp333_ = NULL;
		gchar* _tmp334_ = NULL;
		gchar* _tmp335_ = NULL;
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		e = _inner_error_;
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_inner_error_ = NULL;
#line 836 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp333_ = e->message;
#line 836 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp334_ = g_strconcat ("Error: ", _tmp333_, NULL);
#line 836 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_tmp335_ = _tmp334_;
#line 836 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_message ("TDocumentTest.vala:836: %s", _tmp335_);
#line 836 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_free0 (_tmp335_);
#line 837 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_assert_not_reached ();
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		_g_error_free0 (e);
#line 14055 "TDocumentTest.c"
	}
	__finally139:
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_clear_error (&_inner_error_);
#line 726 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		return;
#line 14066 "TDocumentTest.c"
	}
}


static void ___lambda126__gtest_func (void) {
#line 725 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	__lambda126_ ();
#line 14074 "TDocumentTest.c"
}


void tdocument_test_add_tests (void) {
#line 33 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document", ___lambda105__gtest_func);
#line 42 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document/root", ___lambda106__gtest_func);
#line 59 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document/save/root", ___lambda107__gtest_func);
#line 87 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document/save/root/attribute", ___lambda108__gtest_func);
#line 124 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document/save/root/content", ___lambda109__gtest_func);
#line 159 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document/save/root/children", ___lambda110__gtest_func);
#line 198 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document/root/children-children", ___lambda111__gtest_func);
#line 261 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document/save/children-children", ___lambda112__gtest_func);
#line 329 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document/save/backup", ___lambda113__gtest_func);
#line 364 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document/to_string", ___lambda114__gtest_func);
#line 381 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document/namespace", ___lambda115__gtest_func);
#line 420 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document/parent", ___lambda116__gtest_func);
#line 424 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document/read/basic", ___lambda117__gtest_func);
#line 492 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document/read/namespace", ___lambda118__gtest_func);
#line 533 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document/read/comment", ___lambda119__gtest_func);
#line 547 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document/read/pi", ___lambda120__gtest_func);
#line 571 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document/read/cdata", ___lambda121__gtest_func);
#line 585 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document/read/uri", ___lambda122__gtest_func);
#line 617 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document/read/stream", ___lambda123__gtest_func);
#line 684 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document/read/string", ___lambda124__gtest_func);
#line 702 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document/read/string/attrs", ___lambda125__gtest_func);
#line 725 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_test_add_func ("/gxml/t-document/readtype", ___lambda126__gtest_func);
#line 14123 "TDocumentTest.c"
}


TDocumentTest* tdocument_test_construct (GType object_type) {
	TDocumentTest* self = NULL;
#line 25 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	self = (TDocumentTest*) gxml_test_construct (object_type);
#line 25 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	return self;
#line 14133 "TDocumentTest.c"
}


TDocumentTest* tdocument_test_new (void) {
#line 25 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	return tdocument_test_construct (TYPE_TDOCUMENT_TEST);
#line 14140 "TDocumentTest.c"
}


static gchar* tdocument_test_ttest_object_real_node_name (GXmlSerializableObjectModel* base) {
	TDocumentTestTTestObject * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 29 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	self = (TDocumentTestTTestObject*) base;
#line 29 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp0_ = g_strdup ("Test");
#line 29 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	result = _tmp0_;
#line 29 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	return result;
#line 14156 "TDocumentTest.c"
}


static gchar* tdocument_test_ttest_object_real_to_string (GXmlSerializableObjectModel* base) {
	TDocumentTestTTestObject * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 30 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	self = (TDocumentTestTTestObject*) base;
#line 30 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp0_ = g_strdup ("TestNode");
#line 30 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	result = _tmp0_;
#line 30 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	return result;
#line 14172 "TDocumentTest.c"
}


TDocumentTestTTestObject* tdocument_test_ttest_object_construct (GType object_type) {
	TDocumentTestTTestObject * self = NULL;
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	self = (TDocumentTestTTestObject*) gxml_serializable_object_model_construct (object_type);
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	return self;
#line 14182 "TDocumentTest.c"
}


TDocumentTestTTestObject* tdocument_test_ttest_object_new (void) {
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	return tdocument_test_ttest_object_construct (TDOCUMENT_TEST_TYPE_TTEST_OBJECT);
#line 14189 "TDocumentTest.c"
}


const gchar* tdocument_test_ttest_object_get_name (TDocumentTestTTestObject* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 28 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 28 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp0_ = self->priv->_name;
#line 28 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	result = _tmp0_;
#line 28 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	return result;
#line 14204 "TDocumentTest.c"
}


void tdocument_test_ttest_object_set_name (TDocumentTestTTestObject* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 28 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_return_if_fail (self != NULL);
#line 28 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp0_ = value;
#line 28 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 28 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (self->priv->_name);
#line 28 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	self->priv->_name = _tmp1_;
#line 28 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 14223 "TDocumentTest.c"
}


static void tdocument_test_ttest_object_class_init (TDocumentTestTTestObjectClass * klass) {
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	tdocument_test_ttest_object_parent_class = g_type_class_peek_parent (klass);
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_type_class_add_private (klass, sizeof (TDocumentTestTTestObjectPrivate));
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) tdocument_test_ttest_object_real_node_name;
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) tdocument_test_ttest_object_real_to_string;
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_tdocument_test_ttest_object_get_property;
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_tdocument_test_ttest_object_set_property;
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	G_OBJECT_CLASS (klass)->finalize = tdocument_test_ttest_object_finalize;
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), TDOCUMENT_TEST_TTEST_OBJECT_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 14244 "TDocumentTest.c"
}


static void tdocument_test_ttest_object_instance_init (TDocumentTestTTestObject * self) {
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	self->priv = TDOCUMENT_TEST_TTEST_OBJECT_GET_PRIVATE (self);
#line 14251 "TDocumentTest.c"
}


static void tdocument_test_ttest_object_finalize (GObject* obj) {
	TDocumentTestTTestObject * self;
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TDOCUMENT_TEST_TYPE_TTEST_OBJECT, TDocumentTestTTestObject);
#line 28 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	_g_free0 (self->priv->_name);
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	G_OBJECT_CLASS (tdocument_test_ttest_object_parent_class)->finalize (obj);
#line 14263 "TDocumentTest.c"
}


GType tdocument_test_ttest_object_get_type (void) {
	static volatile gsize tdocument_test_ttest_object_type_id__volatile = 0;
	if (g_once_init_enter (&tdocument_test_ttest_object_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (TDocumentTestTTestObjectClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) tdocument_test_ttest_object_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (TDocumentTestTTestObject), 0, (GInstanceInitFunc) tdocument_test_ttest_object_instance_init, NULL };
		GType tdocument_test_ttest_object_type_id;
		tdocument_test_ttest_object_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "TDocumentTestTTestObject", &g_define_type_info, 0);
		g_once_init_leave (&tdocument_test_ttest_object_type_id__volatile, tdocument_test_ttest_object_type_id);
	}
	return tdocument_test_ttest_object_type_id__volatile;
}


static void _vala_tdocument_test_ttest_object_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	TDocumentTestTTestObject * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TDOCUMENT_TEST_TYPE_TTEST_OBJECT, TDocumentTestTTestObject);
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	switch (property_id) {
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		case TDOCUMENT_TEST_TTEST_OBJECT_NAME:
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		g_value_set_string (value, tdocument_test_ttest_object_get_name (self));
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		break;
#line 14290 "TDocumentTest.c"
		default:
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		break;
#line 14296 "TDocumentTest.c"
	}
}


static void _vala_tdocument_test_ttest_object_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	TDocumentTestTTestObject * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TDOCUMENT_TEST_TYPE_TTEST_OBJECT, TDocumentTestTTestObject);
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	switch (property_id) {
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		case TDOCUMENT_TEST_TTEST_OBJECT_NAME:
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		tdocument_test_ttest_object_set_name (self, g_value_get_string (value));
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		break;
#line 14312 "TDocumentTest.c"
		default:
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 26 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
		break;
#line 14318 "TDocumentTest.c"
	}
}


static void tdocument_test_class_init (TDocumentTestClass * klass) {
#line 25 "/home/despinosa/Proyectos/gxml/test/TDocumentTest.vala"
	tdocument_test_parent_class = g_type_class_peek_parent (klass);
#line 14326 "TDocumentTest.c"
}


static void tdocument_test_instance_init (TDocumentTest * self) {
}


GType tdocument_test_get_type (void) {
	static volatile gsize tdocument_test_type_id__volatile = 0;
	if (g_once_init_enter (&tdocument_test_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (TDocumentTestClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) tdocument_test_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (TDocumentTest), 0, (GInstanceInitFunc) tdocument_test_instance_init, NULL };
		GType tdocument_test_type_id;
		tdocument_test_type_id = g_type_register_static (TYPE_GXML_TEST, "TDocumentTest", &g_define_type_info, 0);
		g_once_init_leave (&tdocument_test_type_id__volatile, tdocument_test_type_id);
	}
	return tdocument_test_type_id__volatile;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}



