/* GomSerializationTest.c generated by valac 0.34.6, the Vala compiler
 * generated from GomSerializationTest.vala, do not modify */

/* -*- Mode: vala; indent-tabs-mode: nil; c-basic-offset: 2; tab-width: 2 -*- */
/* Notation.vala
 *
 * Copyright (C) 2016  Daniel Espinosa <esodan@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Daniel Espinosa <esodan@gmail.com>
 */

#include <glib.h>
#include <glib-object.h>
#include <gxml/gxml.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <math.h>
#include <gee.h>
#include <gio/gio.h>


#define TYPE_GOM_NAME (gom_name_get_type ())
#define GOM_NAME(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GOM_NAME, GomName))
#define GOM_NAME_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GOM_NAME, GomNameClass))
#define IS_GOM_NAME(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GOM_NAME))
#define IS_GOM_NAME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GOM_NAME))
#define GOM_NAME_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GOM_NAME, GomNameClass))

typedef struct _GomName GomName;
typedef struct _GomNameClass GomNameClass;
typedef struct _GomNamePrivate GomNamePrivate;

#define TYPE_GOM_EMAIL (gom_email_get_type ())
#define GOM_EMAIL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GOM_EMAIL, GomEmail))
#define GOM_EMAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GOM_EMAIL, GomEmailClass))
#define IS_GOM_EMAIL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GOM_EMAIL))
#define IS_GOM_EMAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GOM_EMAIL))
#define GOM_EMAIL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GOM_EMAIL, GomEmailClass))

typedef struct _GomEmail GomEmail;
typedef struct _GomEmailClass GomEmailClass;
typedef struct _GomEmailPrivate GomEmailPrivate;

#define TYPE_GOM_AUTHOR (gom_author_get_type ())
#define GOM_AUTHOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GOM_AUTHOR, GomAuthor))
#define GOM_AUTHOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GOM_AUTHOR, GomAuthorClass))
#define IS_GOM_AUTHOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GOM_AUTHOR))
#define IS_GOM_AUTHOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GOM_AUTHOR))
#define GOM_AUTHOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GOM_AUTHOR, GomAuthorClass))

typedef struct _GomAuthor GomAuthor;
typedef struct _GomAuthorClass GomAuthorClass;
typedef struct _GomAuthorPrivate GomAuthorPrivate;

#define TYPE_NAME (name_get_type ())
#define NAME(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_NAME, Name))
#define NAME_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_NAME, NameClass))
#define IS_NAME(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_NAME))
#define IS_NAME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_NAME))
#define NAME_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_NAME, NameClass))

typedef struct _Name Name;
typedef struct _NameClass NameClass;

#define TYPE_EMAIL (email_get_type ())
#define EMAIL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EMAIL, Email))
#define EMAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EMAIL, EmailClass))
#define IS_EMAIL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EMAIL))
#define IS_EMAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EMAIL))
#define EMAIL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EMAIL, EmailClass))

typedef struct _Email Email;
typedef struct _EmailClass EmailClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define GOM_AUTHOR_TYPE_ARRAY (gom_author_array_get_type ())
#define GOM_AUTHOR_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_AUTHOR_TYPE_ARRAY, GomAuthorArray))
#define GOM_AUTHOR_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_AUTHOR_TYPE_ARRAY, GomAuthorArrayClass))
#define GOM_AUTHOR_IS_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_AUTHOR_TYPE_ARRAY))
#define GOM_AUTHOR_IS_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_AUTHOR_TYPE_ARRAY))
#define GOM_AUTHOR_ARRAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_AUTHOR_TYPE_ARRAY, GomAuthorArrayClass))

typedef struct _GomAuthorArray GomAuthorArray;
typedef struct _GomAuthorArrayClass GomAuthorArrayClass;
typedef struct _GomAuthorArrayPrivate GomAuthorArrayPrivate;

#define TYPE_GOM_AUTHORS (gom_authors_get_type ())
#define GOM_AUTHORS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GOM_AUTHORS, GomAuthors))
#define GOM_AUTHORS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GOM_AUTHORS, GomAuthorsClass))
#define IS_GOM_AUTHORS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GOM_AUTHORS))
#define IS_GOM_AUTHORS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GOM_AUTHORS))
#define GOM_AUTHORS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GOM_AUTHORS, GomAuthorsClass))

typedef struct _GomAuthors GomAuthors;
typedef struct _GomAuthorsClass GomAuthorsClass;
typedef struct _GomAuthorsPrivate GomAuthorsPrivate;
#define _g_free0(var) (var = (g_free (var), NULL))

#define TYPE_GOM_INVENTORY (gom_inventory_get_type ())
#define GOM_INVENTORY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GOM_INVENTORY, GomInventory))
#define GOM_INVENTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GOM_INVENTORY, GomInventoryClass))
#define IS_GOM_INVENTORY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GOM_INVENTORY))
#define IS_GOM_INVENTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GOM_INVENTORY))
#define GOM_INVENTORY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GOM_INVENTORY, GomInventoryClass))

typedef struct _GomInventory GomInventory;
typedef struct _GomInventoryClass GomInventoryClass;
typedef struct _GomInventoryPrivate GomInventoryPrivate;

#define GOM_INVENTORY_TYPE_DUAL_KEY_MAP (gom_inventory_dual_key_map_get_type ())
#define GOM_INVENTORY_DUAL_KEY_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_INVENTORY_TYPE_DUAL_KEY_MAP, GomInventoryDualKeyMap))
#define GOM_INVENTORY_DUAL_KEY_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_INVENTORY_TYPE_DUAL_KEY_MAP, GomInventoryDualKeyMapClass))
#define GOM_INVENTORY_IS_DUAL_KEY_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_INVENTORY_TYPE_DUAL_KEY_MAP))
#define GOM_INVENTORY_IS_DUAL_KEY_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_INVENTORY_TYPE_DUAL_KEY_MAP))
#define GOM_INVENTORY_DUAL_KEY_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_INVENTORY_TYPE_DUAL_KEY_MAP, GomInventoryDualKeyMapClass))

typedef struct _GomInventoryDualKeyMap GomInventoryDualKeyMap;
typedef struct _GomInventoryDualKeyMapClass GomInventoryDualKeyMapClass;
typedef struct _GomInventoryDualKeyMapPrivate GomInventoryDualKeyMapPrivate;

#define TYPE_GOM_CATEGORY (gom_category_get_type ())
#define GOM_CATEGORY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GOM_CATEGORY, GomCategory))
#define GOM_CATEGORY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GOM_CATEGORY, GomCategoryClass))
#define IS_GOM_CATEGORY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GOM_CATEGORY))
#define IS_GOM_CATEGORY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GOM_CATEGORY))
#define GOM_CATEGORY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GOM_CATEGORY, GomCategoryClass))

typedef struct _GomCategory GomCategory;
typedef struct _GomCategoryClass GomCategoryClass;
typedef struct _GomCategoryPrivate GomCategoryPrivate;

#define GOM_CATEGORY_TYPE_MAP (gom_category_map_get_type ())
#define GOM_CATEGORY_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_CATEGORY_TYPE_MAP, GomCategoryMap))
#define GOM_CATEGORY_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_CATEGORY_TYPE_MAP, GomCategoryMapClass))
#define GOM_CATEGORY_IS_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_CATEGORY_TYPE_MAP))
#define GOM_CATEGORY_IS_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_CATEGORY_TYPE_MAP))
#define GOM_CATEGORY_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_CATEGORY_TYPE_MAP, GomCategoryMapClass))

typedef struct _GomCategoryMap GomCategoryMap;
typedef struct _GomCategoryMapClass GomCategoryMapClass;
typedef struct _GomCategoryMapPrivate GomCategoryMapPrivate;

#define TYPE_GOM_RESUME (gom_resume_get_type ())
#define GOM_RESUME(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GOM_RESUME, GomResume))
#define GOM_RESUME_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GOM_RESUME, GomResumeClass))
#define IS_GOM_RESUME(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GOM_RESUME))
#define IS_GOM_RESUME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GOM_RESUME))
#define GOM_RESUME_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GOM_RESUME, GomResumeClass))

typedef struct _GomResume GomResume;
typedef struct _GomResumeClass GomResumeClass;
typedef struct _GomResumePrivate GomResumePrivate;

#define GOM_RESUME_TYPE_MAP (gom_resume_map_get_type ())
#define GOM_RESUME_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_RESUME_TYPE_MAP, GomResumeMap))
#define GOM_RESUME_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_RESUME_TYPE_MAP, GomResumeMapClass))
#define GOM_RESUME_IS_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_RESUME_TYPE_MAP))
#define GOM_RESUME_IS_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_RESUME_TYPE_MAP))
#define GOM_RESUME_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_RESUME_TYPE_MAP, GomResumeMapClass))

typedef struct _GomResumeMap GomResumeMap;
typedef struct _GomResumeMapClass GomResumeMapClass;
typedef struct _GomResumeMapPrivate GomResumeMapPrivate;

#define TYPE_GOM_BOOK (gom_book_get_type ())
#define GOM_BOOK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GOM_BOOK, GomBook))
#define GOM_BOOK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GOM_BOOK, GomBookClass))
#define IS_GOM_BOOK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GOM_BOOK))
#define IS_GOM_BOOK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GOM_BOOK))
#define GOM_BOOK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GOM_BOOK, GomBookClass))

typedef struct _GomBook GomBook;
typedef struct _GomBookClass GomBookClass;
typedef struct _GomBookPrivate GomBookPrivate;

#define GOM_BOOK_TYPE_ARRAY (gom_book_array_get_type ())
#define GOM_BOOK_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_BOOK_TYPE_ARRAY, GomBookArray))
#define GOM_BOOK_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_BOOK_TYPE_ARRAY, GomBookArrayClass))
#define GOM_BOOK_IS_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_BOOK_TYPE_ARRAY))
#define GOM_BOOK_IS_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_BOOK_TYPE_ARRAY))
#define GOM_BOOK_ARRAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_BOOK_TYPE_ARRAY, GomBookArrayClass))

typedef struct _GomBookArray GomBookArray;
typedef struct _GomBookArrayClass GomBookArrayClass;
typedef struct _GomBookArrayPrivate GomBookArrayPrivate;

#define TYPE_GOM_BOOK_STORE (gom_book_store_get_type ())
#define GOM_BOOK_STORE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GOM_BOOK_STORE, GomBookStore))
#define GOM_BOOK_STORE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GOM_BOOK_STORE, GomBookStoreClass))
#define IS_GOM_BOOK_STORE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GOM_BOOK_STORE))
#define IS_GOM_BOOK_STORE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GOM_BOOK_STORE))
#define GOM_BOOK_STORE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GOM_BOOK_STORE, GomBookStoreClass))

typedef struct _GomBookStore GomBookStore;
typedef struct _GomBookStoreClass GomBookStoreClass;
typedef struct _GomBookStorePrivate GomBookStorePrivate;

#define TYPE_GXML_TEST (gxml_test_get_type ())
#define GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GXML_TEST, GXmlTest))
#define GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GXML_TEST, GXmlTestClass))
#define IS_GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GXML_TEST))
#define IS_GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GXML_TEST))
#define GXML_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GXML_TEST, GXmlTestClass))

typedef struct _GXmlTest GXmlTest;
typedef struct _GXmlTestClass GXmlTestClass;
typedef struct _GXmlTestPrivate GXmlTestPrivate;

#define TYPE_GOM_SERIALIZATION_TEST (gom_serialization_test_get_type ())
#define GOM_SERIALIZATION_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GOM_SERIALIZATION_TEST, GomSerializationTest))
#define GOM_SERIALIZATION_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GOM_SERIALIZATION_TEST, GomSerializationTestClass))
#define IS_GOM_SERIALIZATION_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GOM_SERIALIZATION_TEST))
#define IS_GOM_SERIALIZATION_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GOM_SERIALIZATION_TEST))
#define GOM_SERIALIZATION_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GOM_SERIALIZATION_TEST, GomSerializationTestClass))

typedef struct _GomSerializationTest GomSerializationTest;
typedef struct _GomSerializationTestClass GomSerializationTestClass;
typedef struct _GomSerializationTestPrivate GomSerializationTestPrivate;

#define GOM_SERIALIZATION_TEST_TYPE_BOOK (gom_serialization_test_book_get_type ())
#define GOM_SERIALIZATION_TEST_BOOK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_SERIALIZATION_TEST_TYPE_BOOK, GomSerializationTestBook))
#define GOM_SERIALIZATION_TEST_BOOK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_SERIALIZATION_TEST_TYPE_BOOK, GomSerializationTestBookClass))
#define GOM_SERIALIZATION_TEST_IS_BOOK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_SERIALIZATION_TEST_TYPE_BOOK))
#define GOM_SERIALIZATION_TEST_IS_BOOK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_SERIALIZATION_TEST_TYPE_BOOK))
#define GOM_SERIALIZATION_TEST_BOOK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_SERIALIZATION_TEST_TYPE_BOOK, GomSerializationTestBookClass))

typedef struct _GomSerializationTestBook GomSerializationTestBook;
typedef struct _GomSerializationTestBookClass GomSerializationTestBookClass;
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))

#define GOM_SERIALIZATION_TEST_TYPE_COMPUTER (gom_serialization_test_computer_get_type ())
#define GOM_SERIALIZATION_TEST_COMPUTER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_SERIALIZATION_TEST_TYPE_COMPUTER, GomSerializationTestComputer))
#define GOM_SERIALIZATION_TEST_COMPUTER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_SERIALIZATION_TEST_TYPE_COMPUTER, GomSerializationTestComputerClass))
#define GOM_SERIALIZATION_TEST_IS_COMPUTER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_SERIALIZATION_TEST_TYPE_COMPUTER))
#define GOM_SERIALIZATION_TEST_IS_COMPUTER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_SERIALIZATION_TEST_TYPE_COMPUTER))
#define GOM_SERIALIZATION_TEST_COMPUTER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_SERIALIZATION_TEST_TYPE_COMPUTER, GomSerializationTestComputerClass))

typedef struct _GomSerializationTestComputer GomSerializationTestComputer;
typedef struct _GomSerializationTestComputerClass GomSerializationTestComputerClass;

#define GOM_SERIALIZATION_TEST_TYPE_TAXES (gom_serialization_test_taxes_get_type ())
#define GOM_SERIALIZATION_TEST_TAXES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_SERIALIZATION_TEST_TYPE_TAXES, GomSerializationTestTaxes))
#define GOM_SERIALIZATION_TEST_TAXES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_SERIALIZATION_TEST_TYPE_TAXES, GomSerializationTestTaxesClass))
#define GOM_SERIALIZATION_TEST_IS_TAXES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_SERIALIZATION_TEST_TYPE_TAXES))
#define GOM_SERIALIZATION_TEST_IS_TAXES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_SERIALIZATION_TEST_TYPE_TAXES))
#define GOM_SERIALIZATION_TEST_TAXES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_SERIALIZATION_TEST_TYPE_TAXES, GomSerializationTestTaxesClass))

typedef struct _GomSerializationTestTaxes GomSerializationTestTaxes;
typedef struct _GomSerializationTestTaxesClass GomSerializationTestTaxesClass;

#define GOM_SERIALIZATION_TEST_TAXES_TYPE_MONTH (gom_serialization_test_taxes_month_get_type ())
#define _g_date_time_unref0(var) ((var == NULL) ? NULL : (var = (g_date_time_unref (var), NULL)))

#define GOM_SERIALIZATION_TEST_TYPE_BOOK_STAND (gom_serialization_test_book_stand_get_type ())
#define GOM_SERIALIZATION_TEST_BOOK_STAND(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_SERIALIZATION_TEST_TYPE_BOOK_STAND, GomSerializationTestBookStand))
#define GOM_SERIALIZATION_TEST_BOOK_STAND_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_SERIALIZATION_TEST_TYPE_BOOK_STAND, GomSerializationTestBookStandClass))
#define GOM_SERIALIZATION_TEST_IS_BOOK_STAND(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_SERIALIZATION_TEST_TYPE_BOOK_STAND))
#define GOM_SERIALIZATION_TEST_IS_BOOK_STAND_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_SERIALIZATION_TEST_TYPE_BOOK_STAND))
#define GOM_SERIALIZATION_TEST_BOOK_STAND_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_SERIALIZATION_TEST_TYPE_BOOK_STAND, GomSerializationTestBookStandClass))

typedef struct _GomSerializationTestBookStand GomSerializationTestBookStand;
typedef struct _GomSerializationTestBookStandClass GomSerializationTestBookStandClass;

#define GOM_SERIALIZATION_TEST_TYPE_REGISTERS (gom_serialization_test_registers_get_type ())
#define GOM_SERIALIZATION_TEST_REGISTERS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_SERIALIZATION_TEST_TYPE_REGISTERS, GomSerializationTestRegisters))
#define GOM_SERIALIZATION_TEST_REGISTERS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_SERIALIZATION_TEST_TYPE_REGISTERS, GomSerializationTestRegistersClass))
#define GOM_SERIALIZATION_TEST_IS_REGISTERS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_SERIALIZATION_TEST_TYPE_REGISTERS))
#define GOM_SERIALIZATION_TEST_IS_REGISTERS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_SERIALIZATION_TEST_TYPE_REGISTERS))
#define GOM_SERIALIZATION_TEST_REGISTERS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_SERIALIZATION_TEST_TYPE_REGISTERS, GomSerializationTestRegistersClass))

typedef struct _GomSerializationTestRegisters GomSerializationTestRegisters;
typedef struct _GomSerializationTestRegistersClass GomSerializationTestRegistersClass;

#define GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER (gom_serialization_test_book_register_get_type ())
#define GOM_SERIALIZATION_TEST_BOOK_REGISTER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER, GomSerializationTestBookRegister))
#define GOM_SERIALIZATION_TEST_BOOK_REGISTER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER, GomSerializationTestBookRegisterClass))
#define GOM_SERIALIZATION_TEST_IS_BOOK_REGISTER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER))
#define GOM_SERIALIZATION_TEST_IS_BOOK_REGISTER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER))
#define GOM_SERIALIZATION_TEST_BOOK_REGISTER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER, GomSerializationTestBookRegisterClass))

typedef struct _GomSerializationTestBookRegister GomSerializationTestBookRegister;
typedef struct _GomSerializationTestBookRegisterClass GomSerializationTestBookRegisterClass;

#define GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION (gom_serialization_test_book_stand_dimension_get_type ())
#define GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION, GomSerializationTestBookStandDimension))
#define GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION, GomSerializationTestBookStandDimensionClass))
#define GOM_SERIALIZATION_TEST_BOOK_STAND_IS_DIMENSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION))
#define GOM_SERIALIZATION_TEST_BOOK_STAND_IS_DIMENSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION))
#define GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION, GomSerializationTestBookStandDimensionClass))

typedef struct _GomSerializationTestBookStandDimension GomSerializationTestBookStandDimension;
typedef struct _GomSerializationTestBookStandDimensionClass GomSerializationTestBookStandDimensionClass;

#define GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION_Y (gom_serialization_test_book_stand_dimension_y_get_type ())
#define GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_Y(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION_Y, GomSerializationTestBookStandDimensionY))
#define GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_Y_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION_Y, GomSerializationTestBookStandDimensionYClass))
#define GOM_SERIALIZATION_TEST_BOOK_STAND_IS_DIMENSION_Y(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION_Y))
#define GOM_SERIALIZATION_TEST_BOOK_STAND_IS_DIMENSION_Y_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION_Y))
#define GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_Y_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION_Y, GomSerializationTestBookStandDimensionYClass))

typedef struct _GomSerializationTestBookStandDimensionY GomSerializationTestBookStandDimensionY;
typedef struct _GomSerializationTestBookStandDimensionYClass GomSerializationTestBookStandDimensionYClass;

#define GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION_Z (gom_serialization_test_book_stand_dimension_z_get_type ())
#define GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_Z(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION_Z, GomSerializationTestBookStandDimensionZ))
#define GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_Z_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION_Z, GomSerializationTestBookStandDimensionZClass))
#define GOM_SERIALIZATION_TEST_BOOK_STAND_IS_DIMENSION_Z(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION_Z))
#define GOM_SERIALIZATION_TEST_BOOK_STAND_IS_DIMENSION_Z_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION_Z))
#define GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_Z_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION_Z, GomSerializationTestBookStandDimensionZClass))

typedef struct _GomSerializationTestBookStandDimensionZ GomSerializationTestBookStandDimensionZ;
typedef struct _GomSerializationTestBookStandDimensionZClass GomSerializationTestBookStandDimensionZClass;

#define GOM_SERIALIZATION_TEST_TYPE_BOOK_STORE (gom_serialization_test_book_store_get_type ())
#define GOM_SERIALIZATION_TEST_BOOK_STORE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_SERIALIZATION_TEST_TYPE_BOOK_STORE, GomSerializationTestBookStore))
#define GOM_SERIALIZATION_TEST_BOOK_STORE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_SERIALIZATION_TEST_TYPE_BOOK_STORE, GomSerializationTestBookStoreClass))
#define GOM_SERIALIZATION_TEST_IS_BOOK_STORE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_SERIALIZATION_TEST_TYPE_BOOK_STORE))
#define GOM_SERIALIZATION_TEST_IS_BOOK_STORE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_SERIALIZATION_TEST_TYPE_BOOK_STORE))
#define GOM_SERIALIZATION_TEST_BOOK_STORE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_SERIALIZATION_TEST_TYPE_BOOK_STORE, GomSerializationTestBookStoreClass))

typedef struct _GomSerializationTestBookStore GomSerializationTestBookStore;
typedef struct _GomSerializationTestBookStoreClass GomSerializationTestBookStoreClass;

#define GOM_SERIALIZATION_TEST_TYPE_BOOKS (gom_serialization_test_books_get_type ())
#define GOM_SERIALIZATION_TEST_BOOKS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_SERIALIZATION_TEST_TYPE_BOOKS, GomSerializationTestBooks))
#define GOM_SERIALIZATION_TEST_BOOKS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_SERIALIZATION_TEST_TYPE_BOOKS, GomSerializationTestBooksClass))
#define GOM_SERIALIZATION_TEST_IS_BOOKS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_SERIALIZATION_TEST_TYPE_BOOKS))
#define GOM_SERIALIZATION_TEST_IS_BOOKS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_SERIALIZATION_TEST_TYPE_BOOKS))
#define GOM_SERIALIZATION_TEST_BOOKS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_SERIALIZATION_TEST_TYPE_BOOKS, GomSerializationTestBooksClass))

typedef struct _GomSerializationTestBooks GomSerializationTestBooks;
typedef struct _GomSerializationTestBooksClass GomSerializationTestBooksClass;

#define GOM_SERIALIZATION_TEST_TYPE_HASH_REGISTERS (gom_serialization_test_hash_registers_get_type ())
#define GOM_SERIALIZATION_TEST_HASH_REGISTERS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_SERIALIZATION_TEST_TYPE_HASH_REGISTERS, GomSerializationTestHashRegisters))
#define GOM_SERIALIZATION_TEST_HASH_REGISTERS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_SERIALIZATION_TEST_TYPE_HASH_REGISTERS, GomSerializationTestHashRegistersClass))
#define GOM_SERIALIZATION_TEST_IS_HASH_REGISTERS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_SERIALIZATION_TEST_TYPE_HASH_REGISTERS))
#define GOM_SERIALIZATION_TEST_IS_HASH_REGISTERS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_SERIALIZATION_TEST_TYPE_HASH_REGISTERS))
#define GOM_SERIALIZATION_TEST_HASH_REGISTERS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_SERIALIZATION_TEST_TYPE_HASH_REGISTERS, GomSerializationTestHashRegistersClass))

typedef struct _GomSerializationTestHashRegisters GomSerializationTestHashRegisters;
typedef struct _GomSerializationTestHashRegistersClass GomSerializationTestHashRegistersClass;

#define GOM_SERIALIZATION_TEST_TYPE_HASH_PAIR_REGISTERS (gom_serialization_test_hash_pair_registers_get_type ())
#define GOM_SERIALIZATION_TEST_HASH_PAIR_REGISTERS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_SERIALIZATION_TEST_TYPE_HASH_PAIR_REGISTERS, GomSerializationTestHashPairRegisters))
#define GOM_SERIALIZATION_TEST_HASH_PAIR_REGISTERS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_SERIALIZATION_TEST_TYPE_HASH_PAIR_REGISTERS, GomSerializationTestHashPairRegistersClass))
#define GOM_SERIALIZATION_TEST_IS_HASH_PAIR_REGISTERS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_SERIALIZATION_TEST_TYPE_HASH_PAIR_REGISTERS))
#define GOM_SERIALIZATION_TEST_IS_HASH_PAIR_REGISTERS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_SERIALIZATION_TEST_TYPE_HASH_PAIR_REGISTERS))
#define GOM_SERIALIZATION_TEST_HASH_PAIR_REGISTERS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_SERIALIZATION_TEST_TYPE_HASH_PAIR_REGISTERS, GomSerializationTestHashPairRegistersClass))

typedef struct _GomSerializationTestHashPairRegisters GomSerializationTestHashPairRegisters;
typedef struct _GomSerializationTestHashPairRegistersClass GomSerializationTestHashPairRegistersClass;

#define GOM_SERIALIZATION_TEST_TYPE_MOTOR (gom_serialization_test_motor_get_type ())
#define GOM_SERIALIZATION_TEST_MOTOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_SERIALIZATION_TEST_TYPE_MOTOR, GomSerializationTestMotor))
#define GOM_SERIALIZATION_TEST_MOTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_SERIALIZATION_TEST_TYPE_MOTOR, GomSerializationTestMotorClass))
#define GOM_SERIALIZATION_TEST_IS_MOTOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_SERIALIZATION_TEST_TYPE_MOTOR))
#define GOM_SERIALIZATION_TEST_IS_MOTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_SERIALIZATION_TEST_TYPE_MOTOR))
#define GOM_SERIALIZATION_TEST_MOTOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_SERIALIZATION_TEST_TYPE_MOTOR, GomSerializationTestMotorClass))

typedef struct _GomSerializationTestMotor GomSerializationTestMotor;
typedef struct _GomSerializationTestMotorClass GomSerializationTestMotorClass;

#define GOM_SERIALIZATION_TEST_MOTOR_TYPE_ON (gom_serialization_test_motor_on_get_type ())
#define GOM_SERIALIZATION_TEST_MOTOR_ON(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_SERIALIZATION_TEST_MOTOR_TYPE_ON, GomSerializationTestMotorOn))
#define GOM_SERIALIZATION_TEST_MOTOR_ON_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_SERIALIZATION_TEST_MOTOR_TYPE_ON, GomSerializationTestMotorOnClass))
#define GOM_SERIALIZATION_TEST_MOTOR_IS_ON(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_SERIALIZATION_TEST_MOTOR_TYPE_ON))
#define GOM_SERIALIZATION_TEST_MOTOR_IS_ON_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_SERIALIZATION_TEST_MOTOR_TYPE_ON))
#define GOM_SERIALIZATION_TEST_MOTOR_ON_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_SERIALIZATION_TEST_MOTOR_TYPE_ON, GomSerializationTestMotorOnClass))

typedef struct _GomSerializationTestMotorOn GomSerializationTestMotorOn;
typedef struct _GomSerializationTestMotorOnClass GomSerializationTestMotorOnClass;

#define GOM_SERIALIZATION_TEST_MOTOR_TYPE_TORQUE (gom_serialization_test_motor_torque_get_type ())
#define GOM_SERIALIZATION_TEST_MOTOR_TORQUE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_SERIALIZATION_TEST_MOTOR_TYPE_TORQUE, GomSerializationTestMotorTorque))
#define GOM_SERIALIZATION_TEST_MOTOR_TORQUE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_SERIALIZATION_TEST_MOTOR_TYPE_TORQUE, GomSerializationTestMotorTorqueClass))
#define GOM_SERIALIZATION_TEST_MOTOR_IS_TORQUE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_SERIALIZATION_TEST_MOTOR_TYPE_TORQUE))
#define GOM_SERIALIZATION_TEST_MOTOR_IS_TORQUE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_SERIALIZATION_TEST_MOTOR_TYPE_TORQUE))
#define GOM_SERIALIZATION_TEST_MOTOR_TORQUE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_SERIALIZATION_TEST_MOTOR_TYPE_TORQUE, GomSerializationTestMotorTorqueClass))

typedef struct _GomSerializationTestMotorTorque GomSerializationTestMotorTorque;
typedef struct _GomSerializationTestMotorTorqueClass GomSerializationTestMotorTorqueClass;

#define GOM_SERIALIZATION_TEST_MOTOR_TYPE_SPEED (gom_serialization_test_motor_speed_get_type ())
#define GOM_SERIALIZATION_TEST_MOTOR_SPEED(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_SERIALIZATION_TEST_MOTOR_TYPE_SPEED, GomSerializationTestMotorSpeed))
#define GOM_SERIALIZATION_TEST_MOTOR_SPEED_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_SERIALIZATION_TEST_MOTOR_TYPE_SPEED, GomSerializationTestMotorSpeedClass))
#define GOM_SERIALIZATION_TEST_MOTOR_IS_SPEED(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_SERIALIZATION_TEST_MOTOR_TYPE_SPEED))
#define GOM_SERIALIZATION_TEST_MOTOR_IS_SPEED_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_SERIALIZATION_TEST_MOTOR_TYPE_SPEED))
#define GOM_SERIALIZATION_TEST_MOTOR_SPEED_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_SERIALIZATION_TEST_MOTOR_TYPE_SPEED, GomSerializationTestMotorSpeedClass))

typedef struct _GomSerializationTestMotorSpeed GomSerializationTestMotorSpeed;
typedef struct _GomSerializationTestMotorSpeedClass GomSerializationTestMotorSpeedClass;

#define GOM_SERIALIZATION_TEST_MOTOR_TYPE_TENSION_TYPE (gom_serialization_test_motor_tension_type_get_type ())
#define GOM_SERIALIZATION_TEST_MOTOR_TENSION_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_SERIALIZATION_TEST_MOTOR_TYPE_TENSION_TYPE, GomSerializationTestMotorTensionType))
#define GOM_SERIALIZATION_TEST_MOTOR_TENSION_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_SERIALIZATION_TEST_MOTOR_TYPE_TENSION_TYPE, GomSerializationTestMotorTensionTypeClass))
#define GOM_SERIALIZATION_TEST_MOTOR_IS_TENSION_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_SERIALIZATION_TEST_MOTOR_TYPE_TENSION_TYPE))
#define GOM_SERIALIZATION_TEST_MOTOR_IS_TENSION_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_SERIALIZATION_TEST_MOTOR_TYPE_TENSION_TYPE))
#define GOM_SERIALIZATION_TEST_MOTOR_TENSION_TYPE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_SERIALIZATION_TEST_MOTOR_TYPE_TENSION_TYPE, GomSerializationTestMotorTensionTypeClass))

typedef struct _GomSerializationTestMotorTensionType GomSerializationTestMotorTensionType;
typedef struct _GomSerializationTestMotorTensionTypeClass GomSerializationTestMotorTensionTypeClass;

#define GOM_SERIALIZATION_TEST_MOTOR_TYPE_TENSION (gom_serialization_test_motor_tension_get_type ())
#define GOM_SERIALIZATION_TEST_MOTOR_TENSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_SERIALIZATION_TEST_MOTOR_TYPE_TENSION, GomSerializationTestMotorTension))
#define GOM_SERIALIZATION_TEST_MOTOR_TENSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_SERIALIZATION_TEST_MOTOR_TYPE_TENSION, GomSerializationTestMotorTensionClass))
#define GOM_SERIALIZATION_TEST_MOTOR_IS_TENSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_SERIALIZATION_TEST_MOTOR_TYPE_TENSION))
#define GOM_SERIALIZATION_TEST_MOTOR_IS_TENSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_SERIALIZATION_TEST_MOTOR_TYPE_TENSION))
#define GOM_SERIALIZATION_TEST_MOTOR_TENSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_SERIALIZATION_TEST_MOTOR_TYPE_TENSION, GomSerializationTestMotorTensionClass))

typedef struct _GomSerializationTestMotorTension GomSerializationTestMotorTension;
typedef struct _GomSerializationTestMotorTensionClass GomSerializationTestMotorTensionClass;

#define GOM_SERIALIZATION_TEST_MOTOR_TYPE_TENSION_TYPE_ENUM (gom_serialization_test_motor_tension_type_enum_get_type ())

#define GOM_SERIALIZATION_TEST_MOTOR_TYPE_MODEL (gom_serialization_test_motor_model_get_type ())
#define GOM_SERIALIZATION_TEST_MOTOR_MODEL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOM_SERIALIZATION_TEST_MOTOR_TYPE_MODEL, GomSerializationTestMotorModel))
#define GOM_SERIALIZATION_TEST_MOTOR_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GOM_SERIALIZATION_TEST_MOTOR_TYPE_MODEL, GomSerializationTestMotorModelClass))
#define GOM_SERIALIZATION_TEST_MOTOR_IS_MODEL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOM_SERIALIZATION_TEST_MOTOR_TYPE_MODEL))
#define GOM_SERIALIZATION_TEST_MOTOR_IS_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GOM_SERIALIZATION_TEST_MOTOR_TYPE_MODEL))
#define GOM_SERIALIZATION_TEST_MOTOR_MODEL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GOM_SERIALIZATION_TEST_MOTOR_TYPE_MODEL, GomSerializationTestMotorModelClass))

typedef struct _GomSerializationTestMotorModel GomSerializationTestMotorModel;
typedef struct _GomSerializationTestMotorModelClass GomSerializationTestMotorModelClass;
typedef struct _GomSerializationTestBookPrivate GomSerializationTestBookPrivate;
typedef struct _GomSerializationTestComputerPrivate GomSerializationTestComputerPrivate;
typedef struct _GomSerializationTestTaxesPrivate GomSerializationTestTaxesPrivate;
typedef struct _GomSerializationTestBookRegisterPrivate GomSerializationTestBookRegisterPrivate;
typedef struct _GomSerializationTestBookStandPrivate GomSerializationTestBookStandPrivate;
typedef struct _GomSerializationTestBookStandDimensionPrivate GomSerializationTestBookStandDimensionPrivate;
typedef struct _GomSerializationTestBookStandDimensionYPrivate GomSerializationTestBookStandDimensionYPrivate;
typedef struct _GomSerializationTestBookStandDimensionZPrivate GomSerializationTestBookStandDimensionZPrivate;
typedef struct _GomSerializationTestRegistersPrivate GomSerializationTestRegistersPrivate;
typedef struct _GomSerializationTestHashRegistersPrivate GomSerializationTestHashRegistersPrivate;
typedef struct _GomSerializationTestHashPairRegistersPrivate GomSerializationTestHashPairRegistersPrivate;
typedef struct _GomSerializationTestBooksPrivate GomSerializationTestBooksPrivate;
typedef struct _GomSerializationTestBookStorePrivate GomSerializationTestBookStorePrivate;
typedef struct _GomSerializationTestMotorPrivate GomSerializationTestMotorPrivate;
typedef struct _GomSerializationTestMotorOnPrivate GomSerializationTestMotorOnPrivate;
typedef struct _GomSerializationTestMotorTorquePrivate GomSerializationTestMotorTorquePrivate;
typedef struct _GomSerializationTestMotorSpeedPrivate GomSerializationTestMotorSpeedPrivate;
typedef struct _GomSerializationTestMotorTensionTypePrivate GomSerializationTestMotorTensionTypePrivate;
typedef struct _GomSerializationTestMotorTensionPrivate GomSerializationTestMotorTensionPrivate;
typedef struct _GomSerializationTestMotorModelPrivate GomSerializationTestMotorModelPrivate;
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _GomName {
	GXmlGomElement parent_instance;
	GomNamePrivate * priv;
};

struct _GomNameClass {
	GXmlGomElementClass parent_class;
};

struct _GomEmail {
	GXmlGomElement parent_instance;
	GomEmailPrivate * priv;
};

struct _GomEmailClass {
	GXmlGomElementClass parent_class;
};

struct _GomAuthor {
	GXmlGomElement parent_instance;
	GomAuthorPrivate * priv;
};

struct _GomAuthorClass {
	GXmlGomElementClass parent_class;
};

struct _GomAuthorPrivate {
	Name* _name;
	Email* _email;
};

struct _GomAuthorArray {
	GXmlGomArrayList parent_instance;
	GomAuthorArrayPrivate * priv;
};

struct _GomAuthorArrayClass {
	GXmlGomArrayListClass parent_class;
};

struct _GomAuthors {
	GXmlGomElement parent_instance;
	GomAuthorsPrivate * priv;
};

struct _GomAuthorsClass {
	GXmlGomElementClass parent_class;
};

struct _GomAuthorsPrivate {
	gchar* _number;
	GomAuthorArray* _array;
};

struct _GomInventory {
	GXmlGomElement parent_instance;
	GomInventoryPrivate * priv;
};

struct _GomInventoryClass {
	GXmlGomElementClass parent_class;
};

struct _GomInventoryPrivate {
	gint _number;
	gint _row;
	gchar* _inventory;
};

struct _GomInventoryDualKeyMap {
	GXmlGomHashMap parent_instance;
	GomInventoryDualKeyMapPrivate * priv;
};

struct _GomInventoryDualKeyMapClass {
	GXmlGomHashMapClass parent_class;
};

struct _GomCategory {
	GXmlGomElement parent_instance;
	GomCategoryPrivate * priv;
};

struct _GomCategoryClass {
	GXmlGomElementClass parent_class;
};

struct _GomCategoryPrivate {
	gchar* _name;
};

struct _GomCategoryMap {
	GXmlGomHashMap parent_instance;
	GomCategoryMapPrivate * priv;
};

struct _GomCategoryMapClass {
	GXmlGomHashMapClass parent_class;
};

struct _GomResume {
	GXmlGomElement parent_instance;
	GomResumePrivate * priv;
};

struct _GomResumeClass {
	GXmlGomElementClass parent_class;
};

struct _GomResumePrivate {
	gchar* _chapter;
	gchar* _text;
};

struct _GomResumeMap {
	GXmlGomHashMap parent_instance;
	GomResumeMapPrivate * priv;
};

struct _GomResumeMapClass {
	GXmlGomHashMapClass parent_class;
};

struct _GomBook {
	GXmlGomElement parent_instance;
	GomBookPrivate * priv;
};

struct _GomBookClass {
	GXmlGomElementClass parent_class;
};

struct _GomBookPrivate {
	gchar* _year;
	gchar* _isbn;
	GomName* _name;
	GomAuthors* _authors;
	GomInventoryDualKeyMap* _inventory_registers;
	GomCategoryMap* _categories;
	GomResumeMap* _resumes;
};

struct _GomBookArray {
	GXmlGomArrayList parent_instance;
	GomBookArrayPrivate * priv;
};

struct _GomBookArrayClass {
	GXmlGomArrayListClass parent_class;
};

struct _GomBookStore {
	GXmlGomElement parent_instance;
	GomBookStorePrivate * priv;
};

struct _GomBookStoreClass {
	GXmlGomElementClass parent_class;
};

struct _GomBookStorePrivate {
	gchar* _name;
	GomBookArray* _books;
};

struct _GXmlTest {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GXmlTestPrivate * priv;
};

struct _GXmlTestClass {
	GTypeClass parent_class;
	void (*finalize) (GXmlTest *self);
};

struct _GomSerializationTest {
	GXmlTest parent_instance;
	GomSerializationTestPrivate * priv;
};

struct _GomSerializationTestClass {
	GXmlTestClass parent_class;
};

typedef enum  {
	GOM_SERIALIZATION_TEST_TAXES_MONTH_JANUARY,
	GOM_SERIALIZATION_TEST_TAXES_MONTH_FEBRUARY
} GomSerializationTestTaxesMonth;

typedef enum  {
	GOM_SERIALIZATION_TEST_MOTOR_TENSION_TYPE_ENUM_AC,
	GOM_SERIALIZATION_TEST_MOTOR_TENSION_TYPE_ENUM_DC
} GomSerializationTestMotorTensionTypeEnum;

struct _GomSerializationTestBook {
	GXmlGomElement parent_instance;
	GomSerializationTestBookPrivate * priv;
};

struct _GomSerializationTestBookClass {
	GXmlGomElementClass parent_class;
};

struct _GomSerializationTestBookPrivate {
	gchar* _name;
};

struct _GomSerializationTestComputer {
	GXmlGomElement parent_instance;
	GomSerializationTestComputerPrivate * priv;
};

struct _GomSerializationTestComputerClass {
	GXmlGomElementClass parent_class;
};

struct _GomSerializationTestComputerPrivate {
	gchar* _model;
	gchar* _ignore;
};

struct _GomSerializationTestTaxes {
	GXmlGomElement parent_instance;
	GomSerializationTestTaxesPrivate * priv;
};

struct _GomSerializationTestTaxesClass {
	GXmlGomElementClass parent_class;
};

struct _GomSerializationTestTaxesPrivate {
	gdouble _month_rate;
	gboolean _tax_free;
	GomSerializationTestTaxesMonth _month;
	GXmlGomDate* _pay_date;
	GXmlGomDateTime* _timestamp;
};

struct _GomSerializationTestBookRegister {
	GXmlGomElement parent_instance;
	GomSerializationTestBookRegisterPrivate * priv;
};

struct _GomSerializationTestBookRegisterClass {
	GXmlGomElementClass parent_class;
};

struct _GomSerializationTestBookRegisterPrivate {
	GomSerializationTestBook* _book;
	gint _year;
};

struct _GomSerializationTestBookStand {
	GXmlGomElement parent_instance;
	GomSerializationTestBookStandPrivate * priv;
};

struct _GomSerializationTestBookStandClass {
	GXmlGomElementClass parent_class;
};

struct _GomSerializationTestBookStandPrivate {
	GomSerializationTestHashRegisters* _hashmap_registers;
	GomSerializationTestHashPairRegisters* _hashpair_registers;
	gchar* _classification;
	GomSerializationTestBookStandDimension* _dimension_x;
	GomSerializationTestBookStandDimensionY* _dimension_y;
	GomSerializationTestBookStandDimensionZ* _dimension_z;
	GomSerializationTestRegisters* _registers;
	GomSerializationTestBooks* _books;
};

struct _GomSerializationTestBookStandDimension {
	GXmlGomElement parent_instance;
	GomSerializationTestBookStandDimensionPrivate * priv;
};

struct _GomSerializationTestBookStandDimensionClass {
	GXmlGomElementClass parent_class;
};

struct _GomSerializationTestBookStandDimensionPrivate {
	gdouble _length;
	gchar* _dtype;
};

struct _GomSerializationTestBookStandDimensionY {
	GomSerializationTestBookStandDimension parent_instance;
	GomSerializationTestBookStandDimensionYPrivate * priv;
};

struct _GomSerializationTestBookStandDimensionYClass {
	GomSerializationTestBookStandDimensionClass parent_class;
};

struct _GomSerializationTestBookStandDimensionZ {
	GomSerializationTestBookStandDimension parent_instance;
	GomSerializationTestBookStandDimensionZPrivate * priv;
};

struct _GomSerializationTestBookStandDimensionZClass {
	GomSerializationTestBookStandDimensionClass parent_class;
};

struct _GomSerializationTestRegisters {
	GXmlGomArrayList parent_instance;
	GomSerializationTestRegistersPrivate * priv;
};

struct _GomSerializationTestRegistersClass {
	GXmlGomArrayListClass parent_class;
};

struct _GomSerializationTestHashRegisters {
	GXmlGomHashMap parent_instance;
	GomSerializationTestHashRegistersPrivate * priv;
};

struct _GomSerializationTestHashRegistersClass {
	GXmlGomHashMapClass parent_class;
};

struct _GomSerializationTestHashPairRegisters {
	GXmlGomHashPairedMap parent_instance;
	GomSerializationTestHashPairRegistersPrivate * priv;
};

struct _GomSerializationTestHashPairRegistersClass {
	GXmlGomHashPairedMapClass parent_class;
};

struct _GomSerializationTestBooks {
	GXmlGomHashMap parent_instance;
	GomSerializationTestBooksPrivate * priv;
};

struct _GomSerializationTestBooksClass {
	GXmlGomHashMapClass parent_class;
};

struct _GomSerializationTestBookStore {
	GXmlGomElement parent_instance;
	GomSerializationTestBookStorePrivate * priv;
};

struct _GomSerializationTestBookStoreClass {
	GXmlGomElementClass parent_class;
};

struct _GomSerializationTestBookStorePrivate {
	gchar* _name;
	GomSerializationTestBooks* _books;
};

struct _GomSerializationTestMotor {
	GXmlGomElement parent_instance;
	GomSerializationTestMotorPrivate * priv;
};

struct _GomSerializationTestMotorClass {
	GXmlGomElementClass parent_class;
};

struct _GomSerializationTestMotorPrivate {
	GomSerializationTestMotorOn* _is_on;
	GomSerializationTestMotorTorque* _torque;
	GomSerializationTestMotorSpeed* _speed;
	GomSerializationTestMotorTensionType* _tension_type;
	GomSerializationTestMotorTension* _tension;
	GomSerializationTestMotorModel* _model;
};

struct _GomSerializationTestMotorOn {
	GXmlGomBoolean parent_instance;
	GomSerializationTestMotorOnPrivate * priv;
};

struct _GomSerializationTestMotorOnClass {
	GXmlGomBooleanClass parent_class;
};

struct _GomSerializationTestMotorTorque {
	GXmlGomDouble parent_instance;
	GomSerializationTestMotorTorquePrivate * priv;
};

struct _GomSerializationTestMotorTorqueClass {
	GXmlGomDoubleClass parent_class;
};

struct _GomSerializationTestMotorSpeed {
	GXmlGomFloat parent_instance;
	GomSerializationTestMotorSpeedPrivate * priv;
};

struct _GomSerializationTestMotorSpeedClass {
	GXmlGomFloatClass parent_class;
};

struct _GomSerializationTestMotorTensionType {
	GXmlGomEnum parent_instance;
	GomSerializationTestMotorTensionTypePrivate * priv;
};

struct _GomSerializationTestMotorTensionTypeClass {
	GXmlGomEnumClass parent_class;
};

struct _GomSerializationTestMotorTension {
	GXmlGomInt parent_instance;
	GomSerializationTestMotorTensionPrivate * priv;
};

struct _GomSerializationTestMotorTensionClass {
	GXmlGomIntClass parent_class;
};

struct _GomSerializationTestMotorModel {
	GXmlGomArrayString parent_instance;
	GomSerializationTestMotorModelPrivate * priv;
};

struct _GomSerializationTestMotorModelClass {
	GXmlGomArrayStringClass parent_class;
};


static gpointer gom_name_parent_class = NULL;
static gpointer gom_email_parent_class = NULL;
static gpointer gom_author_parent_class = NULL;
static gpointer gom_author_array_parent_class = NULL;
static gpointer gom_authors_parent_class = NULL;
static gpointer gom_inventory_parent_class = NULL;
static gpointer gom_inventory_dual_key_map_parent_class = NULL;
static gpointer gom_category_parent_class = NULL;
static gpointer gom_category_map_parent_class = NULL;
static gpointer gom_resume_parent_class = NULL;
static gpointer gom_resume_map_parent_class = NULL;
static gpointer gom_book_parent_class = NULL;
static gpointer gom_book_array_parent_class = NULL;
static gpointer gom_book_store_parent_class = NULL;
static gpointer gom_serialization_test_parent_class = NULL;
static gpointer gom_serialization_test_book_parent_class = NULL;
static gpointer gom_serialization_test_computer_parent_class = NULL;
static gpointer gom_serialization_test_taxes_parent_class = NULL;
static gpointer gom_serialization_test_book_register_parent_class = NULL;
static GXmlMappeableElementIface* gom_serialization_test_book_register_gxml_mappeable_element_parent_iface = NULL;
static GXmlMappeableElementPairKeyIface* gom_serialization_test_book_register_gxml_mappeable_element_pair_key_parent_iface = NULL;
static gpointer gom_serialization_test_book_stand_parent_class = NULL;
static gpointer gom_serialization_test_book_stand_dimension_parent_class = NULL;
static gpointer gom_serialization_test_book_stand_dimension_y_parent_class = NULL;
static gpointer gom_serialization_test_book_stand_dimension_z_parent_class = NULL;
static gpointer gom_serialization_test_registers_parent_class = NULL;
static gpointer gom_serialization_test_hash_registers_parent_class = NULL;
static gpointer gom_serialization_test_hash_pair_registers_parent_class = NULL;
static gpointer gom_serialization_test_books_parent_class = NULL;
static gpointer gom_serialization_test_book_store_parent_class = NULL;
static gpointer gom_serialization_test_motor_parent_class = NULL;
static gpointer gom_serialization_test_motor_on_parent_class = NULL;
static gpointer gom_serialization_test_motor_torque_parent_class = NULL;
static gpointer gom_serialization_test_motor_speed_parent_class = NULL;
static gpointer gom_serialization_test_motor_tension_type_parent_class = NULL;
static gpointer gom_serialization_test_motor_tension_parent_class = NULL;
static gpointer gom_serialization_test_motor_model_parent_class = NULL;

GType gom_name_get_type (void) G_GNUC_CONST;
enum  {
	GOM_NAME_DUMMY_PROPERTY
};
GomName* gom_name_new (void);
GomName* gom_name_construct (GType object_type);
static GObject * gom_name_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
GType gom_email_get_type (void) G_GNUC_CONST;
enum  {
	GOM_EMAIL_DUMMY_PROPERTY
};
GomEmail* gom_email_new (void);
GomEmail* gom_email_construct (GType object_type);
static GObject * gom_email_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
GType gom_author_get_type (void) G_GNUC_CONST;
GType name_get_type (void) G_GNUC_CONST;
GType email_get_type (void) G_GNUC_CONST;
#define GOM_AUTHOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_GOM_AUTHOR, GomAuthorPrivate))
enum  {
	GOM_AUTHOR_DUMMY_PROPERTY,
	GOM_AUTHOR_NAME,
	GOM_AUTHOR_EMAIL
};
GomAuthor* gom_author_new (void);
GomAuthor* gom_author_construct (GType object_type);
Name* gom_author_get_name (GomAuthor* self);
void gom_author_set_name (GomAuthor* self, Name* value);
Email* gom_author_get_email (GomAuthor* self);
void gom_author_set_email (GomAuthor* self, Email* value);
static GObject * gom_author_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
GType gom_author_array_get_type (void) G_GNUC_CONST;
enum  {
	GOM_AUTHOR_ARRAY_DUMMY_PROPERTY
};
GomAuthorArray* gom_author_array_new (void);
GomAuthorArray* gom_author_array_construct (GType object_type);
static GObject * gom_author_array_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void gom_author_finalize (GObject* obj);
static void _vala_gom_author_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gom_author_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType gom_authors_get_type (void) G_GNUC_CONST;
#define GOM_AUTHORS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_GOM_AUTHORS, GomAuthorsPrivate))
enum  {
	GOM_AUTHORS_DUMMY_PROPERTY,
	GOM_AUTHORS_NUMBER,
	GOM_AUTHORS_ARRAY
};
GomAuthors* gom_authors_new (void);
GomAuthors* gom_authors_construct (GType object_type);
const gchar* gom_authors_get_number (GomAuthors* self);
void gom_authors_set_number (GomAuthors* self, const gchar* value);
GomAuthorArray* gom_authors_get_array (GomAuthors* self);
void gom_authors_set_array (GomAuthors* self, GomAuthorArray* value);
static GObject * gom_authors_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void gom_authors_finalize (GObject* obj);
static void _vala_gom_authors_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gom_authors_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType gom_inventory_get_type (void) G_GNUC_CONST;
#define GOM_INVENTORY_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_GOM_INVENTORY, GomInventoryPrivate))
enum  {
	GOM_INVENTORY_DUMMY_PROPERTY,
	GOM_INVENTORY_NUMBER,
	GOM_INVENTORY_ROW,
	GOM_INVENTORY_INVENTORY
};
GomInventory* gom_inventory_new (void);
GomInventory* gom_inventory_construct (GType object_type);
gint gom_inventory_get_number (GomInventory* self);
void gom_inventory_set_number (GomInventory* self, gint value);
gint gom_inventory_get_row (GomInventory* self);
void gom_inventory_set_row (GomInventory* self, gint value);
const gchar* gom_inventory_get_inventory (GomInventory* self);
void gom_inventory_set_inventory (GomInventory* self, const gchar* value);
static GObject * gom_inventory_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
GType gom_inventory_dual_key_map_get_type (void) G_GNUC_CONST;
enum  {
	GOM_INVENTORY_DUAL_KEY_MAP_DUMMY_PROPERTY
};
GomInventoryDualKeyMap* gom_inventory_dual_key_map_new (void);
GomInventoryDualKeyMap* gom_inventory_dual_key_map_construct (GType object_type);
static GObject * gom_inventory_dual_key_map_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void gom_inventory_finalize (GObject* obj);
static void _vala_gom_inventory_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gom_inventory_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType gom_category_get_type (void) G_GNUC_CONST;
#define GOM_CATEGORY_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_GOM_CATEGORY, GomCategoryPrivate))
enum  {
	GOM_CATEGORY_DUMMY_PROPERTY,
	GOM_CATEGORY_NAME
};
GomCategory* gom_category_new (void);
GomCategory* gom_category_construct (GType object_type);
const gchar* gom_category_get_name (GomCategory* self);
void gom_category_set_name (GomCategory* self, const gchar* value);
static GObject * gom_category_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
GType gom_category_map_get_type (void) G_GNUC_CONST;
enum  {
	GOM_CATEGORY_MAP_DUMMY_PROPERTY
};
GomCategoryMap* gom_category_map_new (void);
GomCategoryMap* gom_category_map_construct (GType object_type);
static GObject * gom_category_map_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void gom_category_finalize (GObject* obj);
static void _vala_gom_category_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gom_category_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType gom_resume_get_type (void) G_GNUC_CONST;
#define GOM_RESUME_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_GOM_RESUME, GomResumePrivate))
enum  {
	GOM_RESUME_DUMMY_PROPERTY,
	GOM_RESUME_CHAPTER,
	GOM_RESUME_TEXT
};
GomResume* gom_resume_new (void);
GomResume* gom_resume_construct (GType object_type);
const gchar* gom_resume_get_chapter (GomResume* self);
void gom_resume_set_chapter (GomResume* self, const gchar* value);
const gchar* gom_resume_get_text (GomResume* self);
void gom_resume_set_text (GomResume* self, const gchar* value);
static GObject * gom_resume_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
GType gom_resume_map_get_type (void) G_GNUC_CONST;
enum  {
	GOM_RESUME_MAP_DUMMY_PROPERTY
};
GomResumeMap* gom_resume_map_new (void);
GomResumeMap* gom_resume_map_construct (GType object_type);
static GObject * gom_resume_map_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void gom_resume_finalize (GObject* obj);
static void _vala_gom_resume_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gom_resume_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType gom_book_get_type (void) G_GNUC_CONST;
#define GOM_BOOK_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_GOM_BOOK, GomBookPrivate))
enum  {
	GOM_BOOK_DUMMY_PROPERTY,
	GOM_BOOK_YEAR,
	GOM_BOOK_ISBN,
	GOM_BOOK_NAME,
	GOM_BOOK_AUTHORS,
	GOM_BOOK_INVENTORY_REGISTERS,
	GOM_BOOK_CATEGORIES,
	GOM_BOOK_RESUMES
};
GomBook* gom_book_new (void);
GomBook* gom_book_construct (GType object_type);
const gchar* gom_book_get_year (GomBook* self);
void gom_book_set_year (GomBook* self, const gchar* value);
const gchar* gom_book_get_isbn (GomBook* self);
void gom_book_set_isbn (GomBook* self, const gchar* value);
GomName* gom_book_get_name (GomBook* self);
void gom_book_set_name (GomBook* self, GomName* value);
GomAuthors* gom_book_get_authors (GomBook* self);
void gom_book_set_authors (GomBook* self, GomAuthors* value);
GomInventoryDualKeyMap* gom_book_get_inventory_registers (GomBook* self);
void gom_book_set_inventory_registers (GomBook* self, GomInventoryDualKeyMap* value);
GomCategoryMap* gom_book_get_categories (GomBook* self);
void gom_book_set_categories (GomBook* self, GomCategoryMap* value);
GomResumeMap* gom_book_get_resumes (GomBook* self);
void gom_book_set_resumes (GomBook* self, GomResumeMap* value);
static GObject * gom_book_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
GType gom_book_array_get_type (void) G_GNUC_CONST;
enum  {
	GOM_BOOK_ARRAY_DUMMY_PROPERTY
};
GomBookArray* gom_book_array_new (void);
GomBookArray* gom_book_array_construct (GType object_type);
static GObject * gom_book_array_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void gom_book_finalize (GObject* obj);
static void _vala_gom_book_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gom_book_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType gom_book_store_get_type (void) G_GNUC_CONST;
#define GOM_BOOK_STORE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_GOM_BOOK_STORE, GomBookStorePrivate))
enum  {
	GOM_BOOK_STORE_DUMMY_PROPERTY,
	GOM_BOOK_STORE_NAME,
	GOM_BOOK_STORE_BOOKS
};
GomBookStore* gom_book_store_new (void);
GomBookStore* gom_book_store_construct (GType object_type);
const gchar* gom_book_store_get_name (GomBookStore* self);
void gom_book_store_set_name (GomBookStore* self, const gchar* value);
GomBookArray* gom_book_store_get_books (GomBookStore* self);
void gom_book_store_set_books (GomBookStore* self, GomBookArray* value);
static GObject * gom_book_store_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void gom_book_store_finalize (GObject* obj);
static void _vala_gom_book_store_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gom_book_store_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
gpointer gxml_test_ref (gpointer instance);
void gxml_test_unref (gpointer instance);
GParamSpec* param_spec_gxml_test (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_gxml_test (GValue* value, gpointer v_object);
void value_take_gxml_test (GValue* value, gpointer v_object);
gpointer value_get_gxml_test (const GValue* value);
GType gxml_test_get_type (void) G_GNUC_CONST;
GType gom_serialization_test_get_type (void) G_GNUC_CONST;
enum  {
	GOM_SERIALIZATION_TEST_DUMMY_PROPERTY
};
void gom_serialization_test_add_tests (void);
static void __lambda241_ (void);
GType gom_serialization_test_book_get_type (void) G_GNUC_CONST;
GomSerializationTestBook* gom_serialization_test_book_new (void);
GomSerializationTestBook* gom_serialization_test_book_construct (GType object_type);
void gom_serialization_test_book_set_name (GomSerializationTestBook* self, const gchar* value);
gchar* gom_serialization_test_book_to_string (GomSerializationTestBook* self);
static void ___lambda241__gtest_func (void);
static void __lambda242_ (void);
GType gom_serialization_test_computer_get_type (void) G_GNUC_CONST;
GomSerializationTestComputer* gom_serialization_test_computer_new (void);
GomSerializationTestComputer* gom_serialization_test_computer_construct (GType object_type);
gchar* gom_serialization_test_computer_to_string (GomSerializationTestComputer* self);
void gom_serialization_test_computer_set_model (GomSerializationTestComputer* self, const gchar* value);
void gom_serialization_test_computer_set_ignore (GomSerializationTestComputer* self, const gchar* value);
const gchar* gom_serialization_test_computer_get_model (GomSerializationTestComputer* self);
const gchar* gom_serialization_test_computer_get_ignore (GomSerializationTestComputer* self);
static void ___lambda242__gtest_func (void);
static void __lambda243_ (void);
GType gom_serialization_test_taxes_get_type (void) G_GNUC_CONST;
GomSerializationTestTaxes* gom_serialization_test_taxes_new (void);
GomSerializationTestTaxes* gom_serialization_test_taxes_construct (GType object_type);
gchar* gom_serialization_test_taxes_to_string (GomSerializationTestTaxes* self);
void gom_serialization_test_taxes_set_month_rate (GomSerializationTestTaxes* self, gdouble value);
gdouble gom_serialization_test_taxes_get_month_rate (GomSerializationTestTaxes* self);
GType gom_serialization_test_taxes_month_get_type (void) G_GNUC_CONST;
void gom_serialization_test_taxes_set_month (GomSerializationTestTaxes* self, GomSerializationTestTaxesMonth value);
GomSerializationTestTaxesMonth gom_serialization_test_taxes_get_month (GomSerializationTestTaxes* self);
void gom_serialization_test_taxes_set_tax_free (GomSerializationTestTaxes* self, gboolean value);
gboolean gom_serialization_test_taxes_get_tax_free (GomSerializationTestTaxes* self);
static void ___lambda243__gtest_func (void);
static void __lambda244_ (void);
void gom_serialization_test_taxes_set_pay_date (GomSerializationTestTaxes* self, GXmlGomDate* value);
GXmlGomDate* gom_serialization_test_taxes_get_pay_date (GomSerializationTestTaxes* self);
static void ___lambda244__gtest_func (void);
static void __lambda245_ (void);
static void ___lambda245__gtest_func (void);
static void __lambda246_ (void);
void gom_serialization_test_taxes_set_timestamp (GomSerializationTestTaxes* self, GXmlGomDateTime* value);
GXmlGomDateTime* gom_serialization_test_taxes_get_timestamp (GomSerializationTestTaxes* self);
static void ___lambda246__gtest_func (void);
static void __lambda247_ (void);
GType gom_serialization_test_book_stand_get_type (void) G_GNUC_CONST;
GomSerializationTestBookStand* gom_serialization_test_book_stand_new (void);
GomSerializationTestBookStand* gom_serialization_test_book_stand_construct (GType object_type);
gchar* gom_serialization_test_book_stand_to_string (GomSerializationTestBookStand* self);
GType gom_serialization_test_registers_get_type (void) G_GNUC_CONST;
GomSerializationTestRegisters* gom_serialization_test_book_stand_get_registers (GomSerializationTestBookStand* self);
GType gom_serialization_test_book_register_get_type (void) G_GNUC_CONST;
GomSerializationTestBookRegister* gom_serialization_test_book_register_new (void);
GomSerializationTestBookRegister* gom_serialization_test_book_register_construct (GType object_type);
void gom_serialization_test_book_register_set_year (GomSerializationTestBookRegister* self, gint value);
gint gom_serialization_test_book_register_get_year (GomSerializationTestBookRegister* self);
GType gom_serialization_test_book_stand_dimension_get_type (void) G_GNUC_CONST;
GomSerializationTestBookStandDimension* gom_serialization_test_book_stand_get_dimension_x (GomSerializationTestBookStand* self);
gdouble gom_serialization_test_book_stand_dimension_get_length (GomSerializationTestBookStandDimension* self);
GType gom_serialization_test_book_stand_dimension_y_get_type (void) G_GNUC_CONST;
GomSerializationTestBookStandDimensionY* gom_serialization_test_book_stand_get_dimension_y (GomSerializationTestBookStand* self);
GType gom_serialization_test_book_stand_dimension_z_get_type (void) G_GNUC_CONST;
GomSerializationTestBookStandDimensionZ* gom_serialization_test_book_stand_get_dimension_z (GomSerializationTestBookStand* self);
static void ___lambda247__gtest_func (void);
static void __lambda248_ (void);
GType gom_serialization_test_book_store_get_type (void) G_GNUC_CONST;
GomSerializationTestBookStore* gom_serialization_test_book_store_new (void);
GomSerializationTestBookStore* gom_serialization_test_book_store_construct (GType object_type);
gchar* gom_serialization_test_book_store_to_string (GomSerializationTestBookStore* self);
GType gom_serialization_test_books_get_type (void) G_GNUC_CONST;
GomSerializationTestBooks* gom_serialization_test_book_store_get_books (GomSerializationTestBookStore* self);
GomSerializationTestBook* gom_serialization_test_book_new_document (GXmlDomDocument* doc);
GomSerializationTestBook* gom_serialization_test_book_construct_document (GType object_type, GXmlDomDocument* doc);
const gchar* gom_serialization_test_book_get_name (GomSerializationTestBook* self);
static void ___lambda248__gtest_func (void);
static void __lambda249_ (void);
GType gom_serialization_test_hash_registers_get_type (void) G_GNUC_CONST;
GomSerializationTestHashRegisters* gom_serialization_test_book_stand_get_hashmap_registers (GomSerializationTestBookStand* self);
GomSerializationTestBook* gom_serialization_test_book_register_create_book (GomSerializationTestBookRegister* self, const gchar* name);
void gom_serialization_test_book_register_set_book (GomSerializationTestBookRegister* self, GomSerializationTestBook* value);
GomSerializationTestBook* gom_serialization_test_book_register_get_book (GomSerializationTestBookRegister* self);
static void ___lambda249__gtest_func (void);
static void __lambda250_ (void);
GType gom_serialization_test_hash_pair_registers_get_type (void) G_GNUC_CONST;
GomSerializationTestHashPairRegisters* gom_serialization_test_book_stand_get_hashpair_registers (GomSerializationTestBookStand* self);
static void ___lambda250__gtest_func (void);
static void __lambda251_ (void);
GType gom_serialization_test_motor_get_type (void) G_GNUC_CONST;
GomSerializationTestMotor* gom_serialization_test_motor_new (void);
GomSerializationTestMotor* gom_serialization_test_motor_construct (GType object_type);
gchar* gom_serialization_test_motor_to_string (GomSerializationTestMotor* self);
GomSerializationTestMotorOn* gom_serialization_test_motor_on_new (void);
GomSerializationTestMotorOn* gom_serialization_test_motor_on_construct (GType object_type);
GType gom_serialization_test_motor_on_get_type (void) G_GNUC_CONST;
void gom_serialization_test_motor_set_is_on (GomSerializationTestMotor* self, GomSerializationTestMotorOn* value);
GomSerializationTestMotorTorque* gom_serialization_test_motor_torque_new (void);
GomSerializationTestMotorTorque* gom_serialization_test_motor_torque_construct (GType object_type);
GType gom_serialization_test_motor_torque_get_type (void) G_GNUC_CONST;
void gom_serialization_test_motor_set_torque (GomSerializationTestMotor* self, GomSerializationTestMotorTorque* value);
GomSerializationTestMotorSpeed* gom_serialization_test_motor_speed_new (void);
GomSerializationTestMotorSpeed* gom_serialization_test_motor_speed_construct (GType object_type);
GType gom_serialization_test_motor_speed_get_type (void) G_GNUC_CONST;
void gom_serialization_test_motor_set_speed (GomSerializationTestMotor* self, GomSerializationTestMotorSpeed* value);
GomSerializationTestMotorSpeed* gom_serialization_test_motor_get_speed (GomSerializationTestMotor* self);
GType gom_serialization_test_motor_tension_type_get_type (void) G_GNUC_CONST;
GomSerializationTestMotorTensionType* gom_serialization_test_motor_get_tension_type (GomSerializationTestMotor* self);
GomSerializationTestMotorTensionType* gom_serialization_test_motor_tension_type_new (void);
GomSerializationTestMotorTensionType* gom_serialization_test_motor_tension_type_construct (GType object_type);
void gom_serialization_test_motor_set_tension_type (GomSerializationTestMotor* self, GomSerializationTestMotorTensionType* value);
GomSerializationTestMotorTension* gom_serialization_test_motor_tension_new (void);
GomSerializationTestMotorTension* gom_serialization_test_motor_tension_construct (GType object_type);
GType gom_serialization_test_motor_tension_get_type (void) G_GNUC_CONST;
void gom_serialization_test_motor_set_tension (GomSerializationTestMotor* self, GomSerializationTestMotorTension* value);
GomSerializationTestMotorOn* gom_serialization_test_motor_get_is_on (GomSerializationTestMotor* self);
GomSerializationTestMotorTorque* gom_serialization_test_motor_get_torque (GomSerializationTestMotor* self);
GType gom_serialization_test_motor_tension_type_enum_get_type (void) G_GNUC_CONST;
GomSerializationTestMotorTension* gom_serialization_test_motor_get_tension (GomSerializationTestMotor* self);
GomSerializationTestMotorModel* gom_serialization_test_motor_model_new (void);
GomSerializationTestMotorModel* gom_serialization_test_motor_model_construct (GType object_type);
GType gom_serialization_test_motor_model_get_type (void) G_GNUC_CONST;
void gom_serialization_test_motor_set_model (GomSerializationTestMotor* self, GomSerializationTestMotorModel* value);
GomSerializationTestMotorModel* gom_serialization_test_motor_get_model (GomSerializationTestMotor* self);
static void ___lambda251__gtest_func (void);
static void __lambda252_ (void);
static void ___lambda252__gtest_func (void);
static void __lambda253_ (void);
static void ___lambda253__gtest_func (void);
static void __lambda254_ (void);
gchar* gom_serialization_test_book_register_to_string (GomSerializationTestBookRegister* self);
static void ___lambda254__gtest_func (void);
static void __lambda255_ (void);
static void ___lambda255__gtest_func (void);
static void __lambda256_ (void);
static void ___lambda256__gtest_func (void);
static void __lambda257_ (void);
GomSerializationTestBooks* gom_serialization_test_book_stand_get_books (GomSerializationTestBookStand* self);
static void ___lambda257__gtest_func (void);
static void __lambda258_ (void);
static void ___lambda258__gtest_func (void);
GomSerializationTest* gom_serialization_test_new (void);
GomSerializationTest* gom_serialization_test_construct (GType object_type);
GXmlTest* gxml_test_new (void);
GXmlTest* gxml_test_construct (GType object_type);
#define GOM_SERIALIZATION_TEST_BOOK_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GOM_SERIALIZATION_TEST_TYPE_BOOK, GomSerializationTestBookPrivate))
enum  {
	GOM_SERIALIZATION_TEST_BOOK_DUMMY_PROPERTY,
	GOM_SERIALIZATION_TEST_BOOK_NAME
};
static GObject * gom_serialization_test_book_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void gom_serialization_test_book_finalize (GObject* obj);
static void _vala_gom_serialization_test_book_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gom_serialization_test_book_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define GOM_SERIALIZATION_TEST_COMPUTER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GOM_SERIALIZATION_TEST_TYPE_COMPUTER, GomSerializationTestComputerPrivate))
enum  {
	GOM_SERIALIZATION_TEST_COMPUTER_DUMMY_PROPERTY,
	GOM_SERIALIZATION_TEST_COMPUTER_MODEL,
	GOM_SERIALIZATION_TEST_COMPUTER_IGNORE
};
static GObject * gom_serialization_test_computer_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void gom_serialization_test_computer_finalize (GObject* obj);
static void _vala_gom_serialization_test_computer_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gom_serialization_test_computer_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define GOM_SERIALIZATION_TEST_TAXES_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GOM_SERIALIZATION_TEST_TYPE_TAXES, GomSerializationTestTaxesPrivate))
enum  {
	GOM_SERIALIZATION_TEST_TAXES_DUMMY_PROPERTY,
	GOM_SERIALIZATION_TEST_TAXES_MONTH_RATE,
	GOM_SERIALIZATION_TEST_TAXES_TAX_FREE,
	GOM_SERIALIZATION_TEST_TAXES_MONTH,
	GOM_SERIALIZATION_TEST_TAXES_PAY_DATE,
	GOM_SERIALIZATION_TEST_TAXES_TIMESTAMP
};
static GObject * gom_serialization_test_taxes_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void gom_serialization_test_taxes_finalize (GObject* obj);
static void _vala_gom_serialization_test_taxes_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gom_serialization_test_taxes_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define GOM_SERIALIZATION_TEST_BOOK_REGISTER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER, GomSerializationTestBookRegisterPrivate))
enum  {
	GOM_SERIALIZATION_TEST_BOOK_REGISTER_DUMMY_PROPERTY,
	GOM_SERIALIZATION_TEST_BOOK_REGISTER_YEAR,
	GOM_SERIALIZATION_TEST_BOOK_REGISTER_BOOK
};
GomSerializationTestBookRegister* gom_serialization_test_book_register_new_document (GXmlDomDocument* doc);
GomSerializationTestBookRegister* gom_serialization_test_book_register_construct_document (GType object_type, GXmlDomDocument* doc);
static gchar* gom_serialization_test_book_register_real_get_map_key (GXmlMappeableElement* base);
static gchar* gom_serialization_test_book_register_real_get_map_primary_key (GXmlMappeableElementPairKey* base);
static gchar* gom_serialization_test_book_register_real_get_map_secondary_key (GXmlMappeableElementPairKey* base);
static GObject * gom_serialization_test_book_register_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void gom_serialization_test_book_register_finalize (GObject* obj);
static void _vala_gom_serialization_test_book_register_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gom_serialization_test_book_register_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define GOM_SERIALIZATION_TEST_BOOK_STAND_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GOM_SERIALIZATION_TEST_TYPE_BOOK_STAND, GomSerializationTestBookStandPrivate))
enum  {
	GOM_SERIALIZATION_TEST_BOOK_STAND_DUMMY_PROPERTY,
	GOM_SERIALIZATION_TEST_BOOK_STAND_CLASSIFICATION,
	GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_X,
	GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_Y,
	GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_Z,
	GOM_SERIALIZATION_TEST_BOOK_STAND_REGISTERS,
	GOM_SERIALIZATION_TEST_BOOK_STAND_HASHMAP_REGISTERS,
	GOM_SERIALIZATION_TEST_BOOK_STAND_HASHPAIR_REGISTERS,
	GOM_SERIALIZATION_TEST_BOOK_STAND_BOOKS
};
const gchar* gom_serialization_test_book_stand_get_classification (GomSerializationTestBookStand* self);
void gom_serialization_test_book_stand_set_classification (GomSerializationTestBookStand* self, const gchar* value);
void gom_serialization_test_book_stand_set_dimension_x (GomSerializationTestBookStand* self, GomSerializationTestBookStandDimension* value);
void gom_serialization_test_book_stand_set_dimension_y (GomSerializationTestBookStand* self, GomSerializationTestBookStandDimensionY* value);
void gom_serialization_test_book_stand_set_dimension_z (GomSerializationTestBookStand* self, GomSerializationTestBookStandDimensionZ* value);
void gom_serialization_test_book_stand_set_registers (GomSerializationTestBookStand* self, GomSerializationTestRegisters* value);
void gom_serialization_test_book_stand_set_hashmap_registers (GomSerializationTestBookStand* self, GomSerializationTestHashRegisters* value);
void gom_serialization_test_book_stand_set_hashpair_registers (GomSerializationTestBookStand* self, GomSerializationTestHashPairRegisters* value);
void gom_serialization_test_book_stand_set_books (GomSerializationTestBookStand* self, GomSerializationTestBooks* value);
static GObject * gom_serialization_test_book_stand_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
#define GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION, GomSerializationTestBookStandDimensionPrivate))
enum  {
	GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_DUMMY_PROPERTY,
	GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_LENGTH,
	GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_DTYPE
};
GomSerializationTestBookStandDimension* gom_serialization_test_book_stand_dimension_new (void);
GomSerializationTestBookStandDimension* gom_serialization_test_book_stand_dimension_construct (GType object_type);
void gom_serialization_test_book_stand_dimension_set_length (GomSerializationTestBookStandDimension* self, gdouble value);
const gchar* gom_serialization_test_book_stand_dimension_get_dtype (GomSerializationTestBookStandDimension* self);
void gom_serialization_test_book_stand_dimension_set_dtype (GomSerializationTestBookStandDimension* self, const gchar* value);
static GObject * gom_serialization_test_book_stand_dimension_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void gom_serialization_test_book_stand_dimension_finalize (GObject* obj);
static void _vala_gom_serialization_test_book_stand_dimension_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gom_serialization_test_book_stand_dimension_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
enum  {
	GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_Y_DUMMY_PROPERTY
};
GomSerializationTestBookStandDimensionY* gom_serialization_test_book_stand_dimension_y_new (void);
GomSerializationTestBookStandDimensionY* gom_serialization_test_book_stand_dimension_y_construct (GType object_type);
static GObject * gom_serialization_test_book_stand_dimension_y_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
enum  {
	GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_Z_DUMMY_PROPERTY
};
GomSerializationTestBookStandDimensionZ* gom_serialization_test_book_stand_dimension_z_new (void);
GomSerializationTestBookStandDimensionZ* gom_serialization_test_book_stand_dimension_z_construct (GType object_type);
static GObject * gom_serialization_test_book_stand_dimension_z_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void gom_serialization_test_book_stand_finalize (GObject* obj);
static void _vala_gom_serialization_test_book_stand_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gom_serialization_test_book_stand_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
enum  {
	GOM_SERIALIZATION_TEST_REGISTERS_DUMMY_PROPERTY
};
GomSerializationTestRegisters* gom_serialization_test_registers_new (void);
GomSerializationTestRegisters* gom_serialization_test_registers_construct (GType object_type);
static GObject * gom_serialization_test_registers_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
enum  {
	GOM_SERIALIZATION_TEST_HASH_REGISTERS_DUMMY_PROPERTY
};
GomSerializationTestHashRegisters* gom_serialization_test_hash_registers_new (void);
GomSerializationTestHashRegisters* gom_serialization_test_hash_registers_construct (GType object_type);
static GObject * gom_serialization_test_hash_registers_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
enum  {
	GOM_SERIALIZATION_TEST_HASH_PAIR_REGISTERS_DUMMY_PROPERTY
};
GomSerializationTestHashPairRegisters* gom_serialization_test_hash_pair_registers_new (void);
GomSerializationTestHashPairRegisters* gom_serialization_test_hash_pair_registers_construct (GType object_type);
static GObject * gom_serialization_test_hash_pair_registers_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
enum  {
	GOM_SERIALIZATION_TEST_BOOKS_DUMMY_PROPERTY
};
GomSerializationTestBooks* gom_serialization_test_books_new (void);
GomSerializationTestBooks* gom_serialization_test_books_construct (GType object_type);
static GObject * gom_serialization_test_books_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
#define GOM_SERIALIZATION_TEST_BOOK_STORE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GOM_SERIALIZATION_TEST_TYPE_BOOK_STORE, GomSerializationTestBookStorePrivate))
enum  {
	GOM_SERIALIZATION_TEST_BOOK_STORE_DUMMY_PROPERTY,
	GOM_SERIALIZATION_TEST_BOOK_STORE_NAME,
	GOM_SERIALIZATION_TEST_BOOK_STORE_BOOKS
};
const gchar* gom_serialization_test_book_store_get_name (GomSerializationTestBookStore* self);
void gom_serialization_test_book_store_set_name (GomSerializationTestBookStore* self, const gchar* value);
void gom_serialization_test_book_store_set_books (GomSerializationTestBookStore* self, GomSerializationTestBooks* value);
static GObject * gom_serialization_test_book_store_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void gom_serialization_test_book_store_finalize (GObject* obj);
static void _vala_gom_serialization_test_book_store_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gom_serialization_test_book_store_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define GOM_SERIALIZATION_TEST_MOTOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GOM_SERIALIZATION_TEST_TYPE_MOTOR, GomSerializationTestMotorPrivate))
enum  {
	GOM_SERIALIZATION_TEST_MOTOR_DUMMY_PROPERTY,
	GOM_SERIALIZATION_TEST_MOTOR_IS_ON,
	GOM_SERIALIZATION_TEST_MOTOR_TORQUE,
	GOM_SERIALIZATION_TEST_MOTOR_SPEED,
	GOM_SERIALIZATION_TEST_MOTOR_TENSION_TYPE,
	GOM_SERIALIZATION_TEST_MOTOR_TENSION,
	GOM_SERIALIZATION_TEST_MOTOR_MODEL
};
static GObject * gom_serialization_test_motor_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
enum  {
	GOM_SERIALIZATION_TEST_MOTOR_ON_DUMMY_PROPERTY
};
enum  {
	GOM_SERIALIZATION_TEST_MOTOR_TORQUE_DUMMY_PROPERTY
};
enum  {
	GOM_SERIALIZATION_TEST_MOTOR_SPEED_DUMMY_PROPERTY
};
enum  {
	GOM_SERIALIZATION_TEST_MOTOR_TENSION_TYPE_DUMMY_PROPERTY
};
static GObject * gom_serialization_test_motor_tension_type_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
enum  {
	GOM_SERIALIZATION_TEST_MOTOR_TENSION_DUMMY_PROPERTY
};
enum  {
	GOM_SERIALIZATION_TEST_MOTOR_MODEL_DUMMY_PROPERTY
};
static GObject * gom_serialization_test_motor_model_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void gom_serialization_test_motor_finalize (GObject* obj);
static void _vala_gom_serialization_test_motor_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gom_serialization_test_motor_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);


GomName* gom_name_construct (GType object_type) {
	GomName * self = NULL;
#line 26 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomName*) gxml_gom_element_construct (object_type);
#line 26 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 1447 "GomSerializationTest.c"
}


GomName* gom_name_new (void) {
#line 26 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_name_construct (TYPE_GOM_NAME);
#line 1454 "GomSerializationTest.c"
}


static GObject * gom_name_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomName * self;
	GError * _inner_error_ = NULL;
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_name_parent_class);
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_GOM_NAME, GomName);
#line 1469 "GomSerializationTest.c"
	{
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_initialize ((GXmlGomElement*) self, "Name");
#line 1473 "GomSerializationTest.c"
	}
	goto __finally232;
	__catch232_g_error:
	{
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 1484 "GomSerializationTest.c"
	}
	__finally232:
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 1493 "GomSerializationTest.c"
	}
#line 28 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 1497 "GomSerializationTest.c"
}


static void gom_name_class_init (GomNameClass * klass) {
#line 26 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_name_parent_class = g_type_class_peek_parent (klass);
#line 26 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_name_constructor;
#line 1506 "GomSerializationTest.c"
}


static void gom_name_instance_init (GomName * self) {
}


GType gom_name_get_type (void) {
	static volatile gsize gom_name_type_id__volatile = 0;
	if (g_once_init_enter (&gom_name_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomNameClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_name_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomName), 0, (GInstanceInitFunc) gom_name_instance_init, NULL };
		GType gom_name_type_id;
		gom_name_type_id = g_type_register_static (GXML_TYPE_GOM_ELEMENT, "GomName", &g_define_type_info, 0);
		g_once_init_leave (&gom_name_type_id__volatile, gom_name_type_id);
	}
	return gom_name_type_id__volatile;
}


GomEmail* gom_email_construct (GType object_type) {
	GomEmail * self = NULL;
#line 31 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomEmail*) gxml_gom_element_construct (object_type);
#line 31 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 1532 "GomSerializationTest.c"
}


GomEmail* gom_email_new (void) {
#line 31 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_email_construct (TYPE_GOM_EMAIL);
#line 1539 "GomSerializationTest.c"
}


static GObject * gom_email_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomEmail * self;
	GError * _inner_error_ = NULL;
#line 33 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_email_parent_class);
#line 33 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 33 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_GOM_EMAIL, GomEmail);
#line 1554 "GomSerializationTest.c"
	{
#line 33 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_initialize ((GXmlGomElement*) self, "Email");
#line 1558 "GomSerializationTest.c"
	}
	goto __finally233;
	__catch233_g_error:
	{
#line 33 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 33 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 33 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 1569 "GomSerializationTest.c"
	}
	__finally233:
#line 33 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 33 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 33 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 1578 "GomSerializationTest.c"
	}
#line 33 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 1582 "GomSerializationTest.c"
}


static void gom_email_class_init (GomEmailClass * klass) {
#line 31 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_email_parent_class = g_type_class_peek_parent (klass);
#line 31 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_email_constructor;
#line 1591 "GomSerializationTest.c"
}


static void gom_email_instance_init (GomEmail * self) {
}


GType gom_email_get_type (void) {
	static volatile gsize gom_email_type_id__volatile = 0;
	if (g_once_init_enter (&gom_email_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomEmailClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_email_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomEmail), 0, (GInstanceInitFunc) gom_email_instance_init, NULL };
		GType gom_email_type_id;
		gom_email_type_id = g_type_register_static (GXML_TYPE_GOM_ELEMENT, "GomEmail", &g_define_type_info, 0);
		g_once_init_leave (&gom_email_type_id__volatile, gom_email_type_id);
	}
	return gom_email_type_id__volatile;
}


GomAuthor* gom_author_construct (GType object_type) {
	GomAuthor * self = NULL;
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomAuthor*) gxml_gom_element_construct (object_type);
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 1617 "GomSerializationTest.c"
}


GomAuthor* gom_author_new (void) {
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_author_construct (TYPE_GOM_AUTHOR);
#line 1624 "GomSerializationTest.c"
}


Name* gom_author_get_name (GomAuthor* self) {
	Name* result;
	Name* _tmp0_ = NULL;
#line 38 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 38 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_name;
#line 38 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 38 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 1639 "GomSerializationTest.c"
}


static gpointer _g_object_ref0 (gpointer self) {
#line 38 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self ? g_object_ref (self) : NULL;
#line 1646 "GomSerializationTest.c"
}


void gom_author_set_name (GomAuthor* self, Name* value) {
	Name* _tmp0_ = NULL;
	Name* _tmp1_ = NULL;
#line 38 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 38 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 38 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 38 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_name);
#line 38 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_name = _tmp1_;
#line 38 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 1665 "GomSerializationTest.c"
}


Email* gom_author_get_email (GomAuthor* self) {
	Email* result;
	Email* _tmp0_ = NULL;
#line 39 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 39 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_email;
#line 39 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 39 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 1680 "GomSerializationTest.c"
}


void gom_author_set_email (GomAuthor* self, Email* value) {
	Email* _tmp0_ = NULL;
	Email* _tmp1_ = NULL;
#line 39 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 39 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 39 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 39 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_email);
#line 39 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_email = _tmp1_;
#line 39 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "email");
#line 1699 "GomSerializationTest.c"
}


static GObject * gom_author_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomAuthor * self;
	GError * _inner_error_ = NULL;
#line 40 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_author_parent_class);
#line 40 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 40 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_GOM_AUTHOR, GomAuthor);
#line 1714 "GomSerializationTest.c"
	{
#line 40 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_initialize ((GXmlGomElement*) self, "Author");
#line 1718 "GomSerializationTest.c"
	}
	goto __finally234;
	__catch234_g_error:
	{
#line 40 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 40 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 40 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 1729 "GomSerializationTest.c"
	}
	__finally234:
#line 40 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 40 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 40 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 1738 "GomSerializationTest.c"
	}
#line 40 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 1742 "GomSerializationTest.c"
}


GomAuthorArray* gom_author_array_construct (GType object_type) {
	GomAuthorArray * self = NULL;
#line 41 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomAuthorArray*) gxml_gom_array_list_construct (object_type);
#line 41 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 1752 "GomSerializationTest.c"
}


GomAuthorArray* gom_author_array_new (void) {
#line 41 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_author_array_construct (GOM_AUTHOR_TYPE_ARRAY);
#line 1759 "GomSerializationTest.c"
}


static GObject * gom_author_array_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomAuthorArray * self;
	GError * _inner_error_ = NULL;
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_author_array_parent_class);
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_AUTHOR_TYPE_ARRAY, GomAuthorArray);
#line 1774 "GomSerializationTest.c"
	{
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_collection_initialize ((GXmlGomCollection*) self, TYPE_GOM_AUTHOR, &_inner_error_);
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1780 "GomSerializationTest.c"
			goto __catch235_g_error;
		}
	}
	goto __finally235;
	__catch235_g_error:
	{
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 43 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 1793 "GomSerializationTest.c"
	}
	__finally235:
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 1802 "GomSerializationTest.c"
	}
#line 42 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 1806 "GomSerializationTest.c"
}


static void gom_author_array_class_init (GomAuthorArrayClass * klass) {
#line 41 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_author_array_parent_class = g_type_class_peek_parent (klass);
#line 41 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_author_array_constructor;
#line 1815 "GomSerializationTest.c"
}


static void gom_author_array_instance_init (GomAuthorArray * self) {
}


GType gom_author_array_get_type (void) {
	static volatile gsize gom_author_array_type_id__volatile = 0;
	if (g_once_init_enter (&gom_author_array_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomAuthorArrayClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_author_array_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomAuthorArray), 0, (GInstanceInitFunc) gom_author_array_instance_init, NULL };
		GType gom_author_array_type_id;
		gom_author_array_type_id = g_type_register_static (GXML_TYPE_GOM_ARRAY_LIST, "GomAuthorArray", &g_define_type_info, 0);
		g_once_init_leave (&gom_author_array_type_id__volatile, gom_author_array_type_id);
	}
	return gom_author_array_type_id__volatile;
}


static void gom_author_class_init (GomAuthorClass * klass) {
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_author_parent_class = g_type_class_peek_parent (klass);
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_type_class_add_private (klass, sizeof (GomAuthorPrivate));
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_gom_author_get_property;
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_gom_author_set_property;
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_author_constructor;
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->finalize = gom_author_finalize;
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_AUTHOR_NAME, g_param_spec_object ("name", "name", "name", TYPE_NAME, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_AUTHOR_EMAIL, g_param_spec_object ("email", "email", "email", TYPE_EMAIL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 1852 "GomSerializationTest.c"
}


static void gom_author_instance_init (GomAuthor * self) {
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv = GOM_AUTHOR_GET_PRIVATE (self);
#line 1859 "GomSerializationTest.c"
}


static void gom_author_finalize (GObject* obj) {
	GomAuthor * self;
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_GOM_AUTHOR, GomAuthor);
#line 38 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_name);
#line 39 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_email);
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (gom_author_parent_class)->finalize (obj);
#line 1873 "GomSerializationTest.c"
}


GType gom_author_get_type (void) {
	static volatile gsize gom_author_type_id__volatile = 0;
	if (g_once_init_enter (&gom_author_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomAuthorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_author_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomAuthor), 0, (GInstanceInitFunc) gom_author_instance_init, NULL };
		GType gom_author_type_id;
		gom_author_type_id = g_type_register_static (GXML_TYPE_GOM_ELEMENT, "GomAuthor", &g_define_type_info, 0);
		g_once_init_leave (&gom_author_type_id__volatile, gom_author_type_id);
	}
	return gom_author_type_id__volatile;
}


static void _vala_gom_author_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GomAuthor * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_GOM_AUTHOR, GomAuthor);
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_AUTHOR_NAME:
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_author_get_name (self));
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_AUTHOR_EMAIL:
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_author_get_email (self));
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 1906 "GomSerializationTest.c"
		default:
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 1912 "GomSerializationTest.c"
	}
}


static void _vala_gom_author_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GomAuthor * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_GOM_AUTHOR, GomAuthor);
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_AUTHOR_NAME:
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_author_set_name (self, g_value_get_object (value));
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_AUTHOR_EMAIL:
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_author_set_email (self, g_value_get_object (value));
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 1934 "GomSerializationTest.c"
		default:
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 36 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 1940 "GomSerializationTest.c"
	}
}


GomAuthors* gom_authors_construct (GType object_type) {
	GomAuthors * self = NULL;
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomAuthors*) gxml_gom_element_construct (object_type);
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 1951 "GomSerializationTest.c"
}


GomAuthors* gom_authors_new (void) {
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_authors_construct (TYPE_GOM_AUTHORS);
#line 1958 "GomSerializationTest.c"
}


const gchar* gom_authors_get_number (GomAuthors* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_number;
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 1973 "GomSerializationTest.c"
}


void gom_authors_set_number (GomAuthors* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_number);
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_number = _tmp1_;
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "number");
#line 1992 "GomSerializationTest.c"
}


GomAuthorArray* gom_authors_get_array (GomAuthors* self) {
	GomAuthorArray* result;
	GomAuthorArray* _tmp0_ = NULL;
#line 51 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 51 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_array;
#line 51 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 51 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 2007 "GomSerializationTest.c"
}


void gom_authors_set_array (GomAuthors* self, GomAuthorArray* value) {
	GomAuthorArray* _tmp0_ = NULL;
	GomAuthorArray* _tmp1_ = NULL;
#line 51 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 51 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 51 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 51 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_array);
#line 51 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_array = _tmp1_;
#line 51 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "array");
#line 2026 "GomSerializationTest.c"
}


static GObject * gom_authors_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomAuthors * self;
	GError * _inner_error_ = NULL;
#line 50 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_authors_parent_class);
#line 50 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 50 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_GOM_AUTHORS, GomAuthors);
#line 2041 "GomSerializationTest.c"
	{
#line 50 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_initialize ((GXmlGomElement*) self, "Authors");
#line 2045 "GomSerializationTest.c"
	}
	goto __finally236;
	__catch236_g_error:
	{
#line 50 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 50 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 50 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 2056 "GomSerializationTest.c"
	}
	__finally236:
#line 50 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 50 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 50 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 2065 "GomSerializationTest.c"
	}
#line 50 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 2069 "GomSerializationTest.c"
}


static void gom_authors_class_init (GomAuthorsClass * klass) {
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_authors_parent_class = g_type_class_peek_parent (klass);
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_type_class_add_private (klass, sizeof (GomAuthorsPrivate));
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_gom_authors_get_property;
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_gom_authors_set_property;
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_authors_constructor;
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->finalize = gom_authors_finalize;
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_AUTHORS_NUMBER, g_param_spec_string ("number", "number", "number", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_AUTHORS_ARRAY, g_param_spec_object ("array", "array", "array", GOM_AUTHOR_TYPE_ARRAY, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 2090 "GomSerializationTest.c"
}


static void gom_authors_instance_init (GomAuthors * self) {
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv = GOM_AUTHORS_GET_PRIVATE (self);
#line 2097 "GomSerializationTest.c"
}


static void gom_authors_finalize (GObject* obj) {
	GomAuthors * self;
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_GOM_AUTHORS, GomAuthors);
#line 49 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_number);
#line 51 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_array);
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (gom_authors_parent_class)->finalize (obj);
#line 2111 "GomSerializationTest.c"
}


GType gom_authors_get_type (void) {
	static volatile gsize gom_authors_type_id__volatile = 0;
	if (g_once_init_enter (&gom_authors_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomAuthorsClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_authors_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomAuthors), 0, (GInstanceInitFunc) gom_authors_instance_init, NULL };
		GType gom_authors_type_id;
		gom_authors_type_id = g_type_register_static (GXML_TYPE_GOM_ELEMENT, "GomAuthors", &g_define_type_info, 0);
		g_once_init_leave (&gom_authors_type_id__volatile, gom_authors_type_id);
	}
	return gom_authors_type_id__volatile;
}


static void _vala_gom_authors_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GomAuthors * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_GOM_AUTHORS, GomAuthors);
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_AUTHORS_NUMBER:
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_string (value, gom_authors_get_number (self));
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_AUTHORS_ARRAY:
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_authors_get_array (self));
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 2144 "GomSerializationTest.c"
		default:
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 2150 "GomSerializationTest.c"
	}
}


static void _vala_gom_authors_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GomAuthors * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_GOM_AUTHORS, GomAuthors);
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_AUTHORS_NUMBER:
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_authors_set_number (self, g_value_get_string (value));
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_AUTHORS_ARRAY:
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_authors_set_array (self, g_value_get_object (value));
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 2172 "GomSerializationTest.c"
		default:
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 47 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 2178 "GomSerializationTest.c"
	}
}


GomInventory* gom_inventory_construct (GType object_type) {
	GomInventory * self = NULL;
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomInventory*) gxml_gom_element_construct (object_type);
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 2189 "GomSerializationTest.c"
}


GomInventory* gom_inventory_new (void) {
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_inventory_construct (TYPE_GOM_INVENTORY);
#line 2196 "GomSerializationTest.c"
}


gint gom_inventory_get_number (GomInventory* self) {
	gint result;
	gint _tmp0_ = 0;
#line 57 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 57 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_number;
#line 57 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 57 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 2211 "GomSerializationTest.c"
}


void gom_inventory_set_number (GomInventory* self, gint value) {
	gint _tmp0_ = 0;
#line 57 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 57 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 57 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_number = _tmp0_;
#line 57 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "number");
#line 2225 "GomSerializationTest.c"
}


gint gom_inventory_get_row (GomInventory* self) {
	gint result;
	gint _tmp0_ = 0;
#line 59 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 59 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_row;
#line 59 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 59 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 2240 "GomSerializationTest.c"
}


void gom_inventory_set_row (GomInventory* self, gint value) {
	gint _tmp0_ = 0;
#line 59 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 59 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 59 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_row = _tmp0_;
#line 59 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "row");
#line 2254 "GomSerializationTest.c"
}


const gchar* gom_inventory_get_inventory (GomInventory* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 61 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 61 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_inventory;
#line 61 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 61 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 2269 "GomSerializationTest.c"
}


void gom_inventory_set_inventory (GomInventory* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 61 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 61 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 61 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 61 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_inventory);
#line 61 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_inventory = _tmp1_;
#line 61 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "inventory");
#line 2288 "GomSerializationTest.c"
}


static GObject * gom_inventory_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomInventory * self;
	GError * _inner_error_ = NULL;
#line 62 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_inventory_parent_class);
#line 62 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 62 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_GOM_INVENTORY, GomInventory);
#line 2303 "GomSerializationTest.c"
	{
#line 62 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_initialize ((GXmlGomElement*) self, "Inventory");
#line 2307 "GomSerializationTest.c"
	}
	goto __finally237;
	__catch237_g_error:
	{
#line 62 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 62 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 62 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 2318 "GomSerializationTest.c"
	}
	__finally237:
#line 62 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 62 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 62 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 2327 "GomSerializationTest.c"
	}
#line 62 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 2331 "GomSerializationTest.c"
}


GomInventoryDualKeyMap* gom_inventory_dual_key_map_construct (GType object_type) {
	GomInventoryDualKeyMap * self = NULL;
#line 64 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomInventoryDualKeyMap*) gxml_gom_hash_map_construct (object_type);
#line 64 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 2341 "GomSerializationTest.c"
}


GomInventoryDualKeyMap* gom_inventory_dual_key_map_new (void) {
#line 64 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_inventory_dual_key_map_construct (GOM_INVENTORY_TYPE_DUAL_KEY_MAP);
#line 2348 "GomSerializationTest.c"
}


static GObject * gom_inventory_dual_key_map_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomInventoryDualKeyMap * self;
	GError * _inner_error_ = NULL;
#line 65 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_inventory_dual_key_map_parent_class);
#line 65 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 65 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_INVENTORY_TYPE_DUAL_KEY_MAP, GomInventoryDualKeyMap);
#line 2363 "GomSerializationTest.c"
	{
#line 66 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_hash_map_initialize_with_key ((GXmlGomHashMap*) self, TYPE_GOM_INVENTORY, "number", &_inner_error_);
#line 66 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 2369 "GomSerializationTest.c"
			goto __catch238_g_error;
		}
	}
	goto __finally238;
	__catch238_g_error:
	{
#line 66 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 66 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 67 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 2382 "GomSerializationTest.c"
	}
	__finally238:
#line 66 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 66 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 66 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 2391 "GomSerializationTest.c"
	}
#line 65 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 2395 "GomSerializationTest.c"
}


static void gom_inventory_dual_key_map_class_init (GomInventoryDualKeyMapClass * klass) {
#line 64 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_inventory_dual_key_map_parent_class = g_type_class_peek_parent (klass);
#line 64 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_inventory_dual_key_map_constructor;
#line 2404 "GomSerializationTest.c"
}


static void gom_inventory_dual_key_map_instance_init (GomInventoryDualKeyMap * self) {
}


GType gom_inventory_dual_key_map_get_type (void) {
	static volatile gsize gom_inventory_dual_key_map_type_id__volatile = 0;
	if (g_once_init_enter (&gom_inventory_dual_key_map_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomInventoryDualKeyMapClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_inventory_dual_key_map_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomInventoryDualKeyMap), 0, (GInstanceInitFunc) gom_inventory_dual_key_map_instance_init, NULL };
		GType gom_inventory_dual_key_map_type_id;
		gom_inventory_dual_key_map_type_id = g_type_register_static (GXML_TYPE_GOM_HASH_MAP, "GomInventoryDualKeyMap", &g_define_type_info, 0);
		g_once_init_leave (&gom_inventory_dual_key_map_type_id__volatile, gom_inventory_dual_key_map_type_id);
	}
	return gom_inventory_dual_key_map_type_id__volatile;
}


static void gom_inventory_class_init (GomInventoryClass * klass) {
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_inventory_parent_class = g_type_class_peek_parent (klass);
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_type_class_add_private (klass, sizeof (GomInventoryPrivate));
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_gom_inventory_get_property;
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_gom_inventory_set_property;
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_inventory_constructor;
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->finalize = gom_inventory_finalize;
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_INVENTORY_NUMBER, g_param_spec_int ("number", "::Number", "number", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_INVENTORY_ROW, g_param_spec_int ("row", "::Row", "row", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_INVENTORY_INVENTORY, g_param_spec_string ("inventory", "::Inventory", "inventory", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 2443 "GomSerializationTest.c"
}


static void gom_inventory_instance_init (GomInventory * self) {
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv = GOM_INVENTORY_GET_PRIVATE (self);
#line 2450 "GomSerializationTest.c"
}


static void gom_inventory_finalize (GObject* obj) {
	GomInventory * self;
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_GOM_INVENTORY, GomInventory);
#line 61 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_inventory);
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (gom_inventory_parent_class)->finalize (obj);
#line 2462 "GomSerializationTest.c"
}


GType gom_inventory_get_type (void) {
	static volatile gsize gom_inventory_type_id__volatile = 0;
	if (g_once_init_enter (&gom_inventory_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomInventoryClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_inventory_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomInventory), 0, (GInstanceInitFunc) gom_inventory_instance_init, NULL };
		GType gom_inventory_type_id;
		gom_inventory_type_id = g_type_register_static (GXML_TYPE_GOM_ELEMENT, "GomInventory", &g_define_type_info, 0);
		g_once_init_leave (&gom_inventory_type_id__volatile, gom_inventory_type_id);
	}
	return gom_inventory_type_id__volatile;
}


static void _vala_gom_inventory_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GomInventory * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_GOM_INVENTORY, GomInventory);
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_INVENTORY_NUMBER:
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_int (value, gom_inventory_get_number (self));
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_INVENTORY_ROW:
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_int (value, gom_inventory_get_row (self));
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_INVENTORY_INVENTORY:
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_string (value, gom_inventory_get_inventory (self));
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 2501 "GomSerializationTest.c"
		default:
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 2507 "GomSerializationTest.c"
	}
}


static void _vala_gom_inventory_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GomInventory * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_GOM_INVENTORY, GomInventory);
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_INVENTORY_NUMBER:
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_inventory_set_number (self, g_value_get_int (value));
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_INVENTORY_ROW:
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_inventory_set_row (self, g_value_get_int (value));
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_INVENTORY_INVENTORY:
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_inventory_set_inventory (self, g_value_get_string (value));
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 2535 "GomSerializationTest.c"
		default:
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 54 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 2541 "GomSerializationTest.c"
	}
}


GomCategory* gom_category_construct (GType object_type) {
	GomCategory * self = NULL;
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomCategory*) gxml_gom_element_construct (object_type);
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 2552 "GomSerializationTest.c"
}


GomCategory* gom_category_new (void) {
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_category_construct (TYPE_GOM_CATEGORY);
#line 2559 "GomSerializationTest.c"
}


const gchar* gom_category_get_name (GomCategory* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 75 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 75 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_name;
#line 75 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 75 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 2574 "GomSerializationTest.c"
}


void gom_category_set_name (GomCategory* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 75 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 75 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 75 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 75 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_name);
#line 75 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_name = _tmp1_;
#line 75 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 2593 "GomSerializationTest.c"
}


static GObject * gom_category_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomCategory * self;
	GError * _inner_error_ = NULL;
#line 76 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_category_parent_class);
#line 76 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 76 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_GOM_CATEGORY, GomCategory);
#line 2608 "GomSerializationTest.c"
	{
#line 76 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_initialize ((GXmlGomElement*) self, "Category");
#line 2612 "GomSerializationTest.c"
	}
	goto __finally239;
	__catch239_g_error:
	{
#line 76 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 76 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 76 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 2623 "GomSerializationTest.c"
	}
	__finally239:
#line 76 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 76 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 76 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 2632 "GomSerializationTest.c"
	}
#line 76 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 2636 "GomSerializationTest.c"
}


GomCategoryMap* gom_category_map_construct (GType object_type) {
	GomCategoryMap * self = NULL;
#line 77 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomCategoryMap*) gxml_gom_hash_map_construct (object_type);
#line 77 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 2646 "GomSerializationTest.c"
}


GomCategoryMap* gom_category_map_new (void) {
#line 77 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_category_map_construct (GOM_CATEGORY_TYPE_MAP);
#line 2653 "GomSerializationTest.c"
}


static GObject * gom_category_map_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomCategoryMap * self;
	GError * _inner_error_ = NULL;
#line 78 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_category_map_parent_class);
#line 78 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 78 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_CATEGORY_TYPE_MAP, GomCategoryMap);
#line 2668 "GomSerializationTest.c"
	{
#line 79 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_hash_map_initialize_with_key ((GXmlGomHashMap*) self, TYPE_GOM_INVENTORY, "name", &_inner_error_);
#line 79 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 2674 "GomSerializationTest.c"
			goto __catch240_g_error;
		}
	}
	goto __finally240;
	__catch240_g_error:
	{
#line 79 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 79 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 80 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 2687 "GomSerializationTest.c"
	}
	__finally240:
#line 79 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 79 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 79 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 2696 "GomSerializationTest.c"
	}
#line 78 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 2700 "GomSerializationTest.c"
}


static void gom_category_map_class_init (GomCategoryMapClass * klass) {
#line 77 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_category_map_parent_class = g_type_class_peek_parent (klass);
#line 77 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_category_map_constructor;
#line 2709 "GomSerializationTest.c"
}


static void gom_category_map_instance_init (GomCategoryMap * self) {
}


GType gom_category_map_get_type (void) {
	static volatile gsize gom_category_map_type_id__volatile = 0;
	if (g_once_init_enter (&gom_category_map_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomCategoryMapClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_category_map_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomCategoryMap), 0, (GInstanceInitFunc) gom_category_map_instance_init, NULL };
		GType gom_category_map_type_id;
		gom_category_map_type_id = g_type_register_static (GXML_TYPE_GOM_HASH_MAP, "GomCategoryMap", &g_define_type_info, 0);
		g_once_init_leave (&gom_category_map_type_id__volatile, gom_category_map_type_id);
	}
	return gom_category_map_type_id__volatile;
}


static void gom_category_class_init (GomCategoryClass * klass) {
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_category_parent_class = g_type_class_peek_parent (klass);
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_type_class_add_private (klass, sizeof (GomCategoryPrivate));
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_gom_category_get_property;
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_gom_category_set_property;
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_category_constructor;
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->finalize = gom_category_finalize;
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_CATEGORY_NAME, g_param_spec_string ("name", "::Name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 2744 "GomSerializationTest.c"
}


static void gom_category_instance_init (GomCategory * self) {
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv = GOM_CATEGORY_GET_PRIVATE (self);
#line 2751 "GomSerializationTest.c"
}


static void gom_category_finalize (GObject* obj) {
	GomCategory * self;
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_GOM_CATEGORY, GomCategory);
#line 75 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_name);
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (gom_category_parent_class)->finalize (obj);
#line 2763 "GomSerializationTest.c"
}


GType gom_category_get_type (void) {
	static volatile gsize gom_category_type_id__volatile = 0;
	if (g_once_init_enter (&gom_category_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomCategoryClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_category_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomCategory), 0, (GInstanceInitFunc) gom_category_instance_init, NULL };
		GType gom_category_type_id;
		gom_category_type_id = g_type_register_static (GXML_TYPE_GOM_ELEMENT, "GomCategory", &g_define_type_info, 0);
		g_once_init_leave (&gom_category_type_id__volatile, gom_category_type_id);
	}
	return gom_category_type_id__volatile;
}


static void _vala_gom_category_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GomCategory * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_GOM_CATEGORY, GomCategory);
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_CATEGORY_NAME:
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_string (value, gom_category_get_name (self));
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 2790 "GomSerializationTest.c"
		default:
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 2796 "GomSerializationTest.c"
	}
}


static void _vala_gom_category_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GomCategory * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_GOM_CATEGORY, GomCategory);
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_CATEGORY_NAME:
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_category_set_name (self, g_value_get_string (value));
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 2812 "GomSerializationTest.c"
		default:
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 72 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 2818 "GomSerializationTest.c"
	}
}


GomResume* gom_resume_construct (GType object_type) {
	GomResume * self = NULL;
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomResume*) gxml_gom_element_construct (object_type);
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 2829 "GomSerializationTest.c"
}


GomResume* gom_resume_new (void) {
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_resume_construct (TYPE_GOM_RESUME);
#line 2836 "GomSerializationTest.c"
}


const gchar* gom_resume_get_chapter (GomResume* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 89 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 89 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_chapter;
#line 89 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 89 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 2851 "GomSerializationTest.c"
}


void gom_resume_set_chapter (GomResume* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 89 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 89 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 89 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 89 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_chapter);
#line 89 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_chapter = _tmp1_;
#line 89 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "chapter");
#line 2870 "GomSerializationTest.c"
}


const gchar* gom_resume_get_text (GomResume* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 91 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 91 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_text;
#line 91 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 91 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 2885 "GomSerializationTest.c"
}


void gom_resume_set_text (GomResume* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 91 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 91 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 91 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 91 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_text);
#line 91 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_text = _tmp1_;
#line 91 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "text");
#line 2904 "GomSerializationTest.c"
}


static GObject * gom_resume_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomResume * self;
	GError * _inner_error_ = NULL;
#line 92 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_resume_parent_class);
#line 92 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 92 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_GOM_RESUME, GomResume);
#line 2919 "GomSerializationTest.c"
	{
#line 92 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_initialize ((GXmlGomElement*) self, "Resume");
#line 2923 "GomSerializationTest.c"
	}
	goto __finally241;
	__catch241_g_error:
	{
#line 92 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 92 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 92 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 2934 "GomSerializationTest.c"
	}
	__finally241:
#line 92 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 92 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 92 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 2943 "GomSerializationTest.c"
	}
#line 92 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 2947 "GomSerializationTest.c"
}


GomResumeMap* gom_resume_map_construct (GType object_type) {
	GomResumeMap * self = NULL;
#line 93 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomResumeMap*) gxml_gom_hash_map_construct (object_type);
#line 93 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 2957 "GomSerializationTest.c"
}


GomResumeMap* gom_resume_map_new (void) {
#line 93 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_resume_map_construct (GOM_RESUME_TYPE_MAP);
#line 2964 "GomSerializationTest.c"
}


static GObject * gom_resume_map_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomResumeMap * self;
	GError * _inner_error_ = NULL;
#line 94 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_resume_map_parent_class);
#line 94 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 94 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_RESUME_TYPE_MAP, GomResumeMap);
#line 2979 "GomSerializationTest.c"
	{
#line 95 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_hash_map_initialize_with_key ((GXmlGomHashMap*) self, TYPE_GOM_INVENTORY, "chapter", &_inner_error_);
#line 95 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 2985 "GomSerializationTest.c"
			goto __catch242_g_error;
		}
	}
	goto __finally242;
	__catch242_g_error:
	{
#line 95 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 95 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 96 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 2998 "GomSerializationTest.c"
	}
	__finally242:
#line 95 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 95 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 95 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 3007 "GomSerializationTest.c"
	}
#line 94 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 3011 "GomSerializationTest.c"
}


static void gom_resume_map_class_init (GomResumeMapClass * klass) {
#line 93 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_resume_map_parent_class = g_type_class_peek_parent (klass);
#line 93 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_resume_map_constructor;
#line 3020 "GomSerializationTest.c"
}


static void gom_resume_map_instance_init (GomResumeMap * self) {
}


GType gom_resume_map_get_type (void) {
	static volatile gsize gom_resume_map_type_id__volatile = 0;
	if (g_once_init_enter (&gom_resume_map_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomResumeMapClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_resume_map_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomResumeMap), 0, (GInstanceInitFunc) gom_resume_map_instance_init, NULL };
		GType gom_resume_map_type_id;
		gom_resume_map_type_id = g_type_register_static (GXML_TYPE_GOM_HASH_MAP, "GomResumeMap", &g_define_type_info, 0);
		g_once_init_leave (&gom_resume_map_type_id__volatile, gom_resume_map_type_id);
	}
	return gom_resume_map_type_id__volatile;
}


static void gom_resume_class_init (GomResumeClass * klass) {
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_resume_parent_class = g_type_class_peek_parent (klass);
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_type_class_add_private (klass, sizeof (GomResumePrivate));
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_gom_resume_get_property;
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_gom_resume_set_property;
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_resume_constructor;
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->finalize = gom_resume_finalize;
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_RESUME_CHAPTER, g_param_spec_string ("chapter", "::Chapter", "chapter", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_RESUME_TEXT, g_param_spec_string ("text", "::Text", "text", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 3057 "GomSerializationTest.c"
}


static void gom_resume_instance_init (GomResume * self) {
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv = GOM_RESUME_GET_PRIVATE (self);
#line 3064 "GomSerializationTest.c"
}


static void gom_resume_finalize (GObject* obj) {
	GomResume * self;
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_GOM_RESUME, GomResume);
#line 89 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_chapter);
#line 91 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_text);
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (gom_resume_parent_class)->finalize (obj);
#line 3078 "GomSerializationTest.c"
}


GType gom_resume_get_type (void) {
	static volatile gsize gom_resume_type_id__volatile = 0;
	if (g_once_init_enter (&gom_resume_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomResumeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_resume_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomResume), 0, (GInstanceInitFunc) gom_resume_instance_init, NULL };
		GType gom_resume_type_id;
		gom_resume_type_id = g_type_register_static (GXML_TYPE_GOM_ELEMENT, "GomResume", &g_define_type_info, 0);
		g_once_init_leave (&gom_resume_type_id__volatile, gom_resume_type_id);
	}
	return gom_resume_type_id__volatile;
}


static void _vala_gom_resume_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GomResume * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_GOM_RESUME, GomResume);
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_RESUME_CHAPTER:
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_string (value, gom_resume_get_chapter (self));
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_RESUME_TEXT:
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_string (value, gom_resume_get_text (self));
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 3111 "GomSerializationTest.c"
		default:
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 3117 "GomSerializationTest.c"
	}
}


static void _vala_gom_resume_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GomResume * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_GOM_RESUME, GomResume);
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_RESUME_CHAPTER:
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_resume_set_chapter (self, g_value_get_string (value));
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_RESUME_TEXT:
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_resume_set_text (self, g_value_get_string (value));
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 3139 "GomSerializationTest.c"
		default:
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 86 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 3145 "GomSerializationTest.c"
	}
}


GomBook* gom_book_construct (GType object_type) {
	GomBook * self = NULL;
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomBook*) gxml_gom_element_construct (object_type);
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 3156 "GomSerializationTest.c"
}


GomBook* gom_book_new (void) {
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_book_construct (TYPE_GOM_BOOK);
#line 3163 "GomSerializationTest.c"
}


const gchar* gom_book_get_year (GomBook* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 104 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 104 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_year;
#line 104 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 104 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 3178 "GomSerializationTest.c"
}


void gom_book_set_year (GomBook* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 104 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 104 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 104 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 104 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_year);
#line 104 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_year = _tmp1_;
#line 104 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "year");
#line 3197 "GomSerializationTest.c"
}


const gchar* gom_book_get_isbn (GomBook* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 106 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 106 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_isbn;
#line 106 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 106 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 3212 "GomSerializationTest.c"
}


void gom_book_set_isbn (GomBook* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 106 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 106 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 106 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 106 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_isbn);
#line 106 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_isbn = _tmp1_;
#line 106 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "isbn");
#line 3231 "GomSerializationTest.c"
}


GomName* gom_book_get_name (GomBook* self) {
	GomName* result;
	GomName* _tmp0_ = NULL;
#line 107 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 107 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_name;
#line 107 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 107 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 3246 "GomSerializationTest.c"
}


void gom_book_set_name (GomBook* self, GomName* value) {
	GomName* _tmp0_ = NULL;
	GomName* _tmp1_ = NULL;
#line 107 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 107 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 107 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 107 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_name);
#line 107 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_name = _tmp1_;
#line 107 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 3265 "GomSerializationTest.c"
}


GomAuthors* gom_book_get_authors (GomBook* self) {
	GomAuthors* result;
	GomAuthors* _tmp0_ = NULL;
#line 108 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 108 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_authors;
#line 108 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 108 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 3280 "GomSerializationTest.c"
}


void gom_book_set_authors (GomBook* self, GomAuthors* value) {
	GomAuthors* _tmp0_ = NULL;
	GomAuthors* _tmp1_ = NULL;
#line 108 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 108 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 108 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 108 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_authors);
#line 108 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_authors = _tmp1_;
#line 108 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "authors");
#line 3299 "GomSerializationTest.c"
}


GomInventoryDualKeyMap* gom_book_get_inventory_registers (GomBook* self) {
	GomInventoryDualKeyMap* result;
	GomInventoryDualKeyMap* _tmp0_ = NULL;
#line 109 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 109 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_inventory_registers;
#line 109 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 109 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 3314 "GomSerializationTest.c"
}


void gom_book_set_inventory_registers (GomBook* self, GomInventoryDualKeyMap* value) {
	GomInventoryDualKeyMap* _tmp0_ = NULL;
	GomInventoryDualKeyMap* _tmp1_ = NULL;
#line 109 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 109 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 109 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 109 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_inventory_registers);
#line 109 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_inventory_registers = _tmp1_;
#line 109 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "inventory-registers");
#line 3333 "GomSerializationTest.c"
}


GomCategoryMap* gom_book_get_categories (GomBook* self) {
	GomCategoryMap* result;
	GomCategoryMap* _tmp0_ = NULL;
#line 110 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 110 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_categories;
#line 110 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 110 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 3348 "GomSerializationTest.c"
}


void gom_book_set_categories (GomBook* self, GomCategoryMap* value) {
	GomCategoryMap* _tmp0_ = NULL;
	GomCategoryMap* _tmp1_ = NULL;
#line 110 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 110 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 110 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 110 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_categories);
#line 110 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_categories = _tmp1_;
#line 110 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "categories");
#line 3367 "GomSerializationTest.c"
}


GomResumeMap* gom_book_get_resumes (GomBook* self) {
	GomResumeMap* result;
	GomResumeMap* _tmp0_ = NULL;
#line 111 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 111 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_resumes;
#line 111 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 111 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 3382 "GomSerializationTest.c"
}


void gom_book_set_resumes (GomBook* self, GomResumeMap* value) {
	GomResumeMap* _tmp0_ = NULL;
	GomResumeMap* _tmp1_ = NULL;
#line 111 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 111 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 111 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 111 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_resumes);
#line 111 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_resumes = _tmp1_;
#line 111 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "resumes");
#line 3401 "GomSerializationTest.c"
}


static GObject * gom_book_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomBook * self;
	GError * _inner_error_ = NULL;
#line 112 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_book_parent_class);
#line 112 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 112 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_GOM_BOOK, GomBook);
#line 3416 "GomSerializationTest.c"
	{
#line 113 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_initialize ((GXmlGomElement*) self, "Book");
#line 3420 "GomSerializationTest.c"
	}
	goto __finally243;
	__catch243_g_error:
	{
#line 113 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 113 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 113 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 3431 "GomSerializationTest.c"
	}
	__finally243:
#line 113 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 113 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 113 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 3440 "GomSerializationTest.c"
	}
#line 112 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 3444 "GomSerializationTest.c"
}


GomBookArray* gom_book_array_construct (GType object_type) {
	GomBookArray * self = NULL;
#line 115 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomBookArray*) gxml_gom_array_list_construct (object_type);
#line 115 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 3454 "GomSerializationTest.c"
}


GomBookArray* gom_book_array_new (void) {
#line 115 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_book_array_construct (GOM_BOOK_TYPE_ARRAY);
#line 3461 "GomSerializationTest.c"
}


static GObject * gom_book_array_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomBookArray * self;
	GError * _inner_error_ = NULL;
#line 116 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_book_array_parent_class);
#line 116 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 116 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_BOOK_TYPE_ARRAY, GomBookArray);
#line 3476 "GomSerializationTest.c"
	{
#line 117 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_collection_initialize ((GXmlGomCollection*) self, TYPE_GOM_BOOK, &_inner_error_);
#line 117 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 3482 "GomSerializationTest.c"
			goto __catch244_g_error;
		}
	}
	goto __finally244;
	__catch244_g_error:
	{
#line 117 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 117 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 118 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 3495 "GomSerializationTest.c"
	}
	__finally244:
#line 117 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 117 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 117 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 3504 "GomSerializationTest.c"
	}
#line 116 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 3508 "GomSerializationTest.c"
}


static void gom_book_array_class_init (GomBookArrayClass * klass) {
#line 115 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_book_array_parent_class = g_type_class_peek_parent (klass);
#line 115 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_book_array_constructor;
#line 3517 "GomSerializationTest.c"
}


static void gom_book_array_instance_init (GomBookArray * self) {
}


GType gom_book_array_get_type (void) {
	static volatile gsize gom_book_array_type_id__volatile = 0;
	if (g_once_init_enter (&gom_book_array_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomBookArrayClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_book_array_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomBookArray), 0, (GInstanceInitFunc) gom_book_array_instance_init, NULL };
		GType gom_book_array_type_id;
		gom_book_array_type_id = g_type_register_static (GXML_TYPE_GOM_ARRAY_LIST, "GomBookArray", &g_define_type_info, 0);
		g_once_init_leave (&gom_book_array_type_id__volatile, gom_book_array_type_id);
	}
	return gom_book_array_type_id__volatile;
}


static void gom_book_class_init (GomBookClass * klass) {
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_book_parent_class = g_type_class_peek_parent (klass);
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_type_class_add_private (klass, sizeof (GomBookPrivate));
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_gom_book_get_property;
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_gom_book_set_property;
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_book_constructor;
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->finalize = gom_book_finalize;
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_BOOK_YEAR, g_param_spec_string ("year", "::Year", "year", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_BOOK_ISBN, g_param_spec_string ("isbn", "::ISBN", "isbn", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_BOOK_NAME, g_param_spec_object ("name", "name", "name", TYPE_GOM_NAME, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_BOOK_AUTHORS, g_param_spec_object ("authors", "authors", "authors", TYPE_GOM_AUTHORS, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_BOOK_INVENTORY_REGISTERS, g_param_spec_object ("inventory-registers", "inventory-registers", "inventory-registers", GOM_INVENTORY_TYPE_DUAL_KEY_MAP, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_BOOK_CATEGORIES, g_param_spec_object ("categories", "categories", "categories", GOM_CATEGORY_TYPE_MAP, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_BOOK_RESUMES, g_param_spec_object ("resumes", "resumes", "resumes", GOM_RESUME_TYPE_MAP, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 3564 "GomSerializationTest.c"
}


static void gom_book_instance_init (GomBook * self) {
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv = GOM_BOOK_GET_PRIVATE (self);
#line 3571 "GomSerializationTest.c"
}


static void gom_book_finalize (GObject* obj) {
	GomBook * self;
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_GOM_BOOK, GomBook);
#line 104 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_year);
#line 106 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_isbn);
#line 107 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_name);
#line 108 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_authors);
#line 109 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_inventory_registers);
#line 110 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_categories);
#line 111 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_resumes);
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (gom_book_parent_class)->finalize (obj);
#line 3595 "GomSerializationTest.c"
}


GType gom_book_get_type (void) {
	static volatile gsize gom_book_type_id__volatile = 0;
	if (g_once_init_enter (&gom_book_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomBookClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_book_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomBook), 0, (GInstanceInitFunc) gom_book_instance_init, NULL };
		GType gom_book_type_id;
		gom_book_type_id = g_type_register_static (GXML_TYPE_GOM_ELEMENT, "GomBook", &g_define_type_info, 0);
		g_once_init_leave (&gom_book_type_id__volatile, gom_book_type_id);
	}
	return gom_book_type_id__volatile;
}


static void _vala_gom_book_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GomBook * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_GOM_BOOK, GomBook);
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_BOOK_YEAR:
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_string (value, gom_book_get_year (self));
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_BOOK_ISBN:
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_string (value, gom_book_get_isbn (self));
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_BOOK_NAME:
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_book_get_name (self));
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_BOOK_AUTHORS:
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_book_get_authors (self));
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_BOOK_INVENTORY_REGISTERS:
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_book_get_inventory_registers (self));
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_BOOK_CATEGORIES:
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_book_get_categories (self));
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_BOOK_RESUMES:
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_book_get_resumes (self));
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 3658 "GomSerializationTest.c"
		default:
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 3664 "GomSerializationTest.c"
	}
}


static void _vala_gom_book_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GomBook * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_GOM_BOOK, GomBook);
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_BOOK_YEAR:
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_book_set_year (self, g_value_get_string (value));
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_BOOK_ISBN:
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_book_set_isbn (self, g_value_get_string (value));
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_BOOK_NAME:
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_book_set_name (self, g_value_get_object (value));
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_BOOK_AUTHORS:
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_book_set_authors (self, g_value_get_object (value));
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_BOOK_INVENTORY_REGISTERS:
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_book_set_inventory_registers (self, g_value_get_object (value));
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_BOOK_CATEGORIES:
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_book_set_categories (self, g_value_get_object (value));
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_BOOK_RESUMES:
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_book_set_resumes (self, g_value_get_object (value));
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 3716 "GomSerializationTest.c"
		default:
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 101 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 3722 "GomSerializationTest.c"
	}
}


GomBookStore* gom_book_store_construct (GType object_type) {
	GomBookStore * self = NULL;
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomBookStore*) gxml_gom_element_construct (object_type);
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 3733 "GomSerializationTest.c"
}


GomBookStore* gom_book_store_new (void) {
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_book_store_construct (TYPE_GOM_BOOK_STORE);
#line 3740 "GomSerializationTest.c"
}


const gchar* gom_book_store_get_name (GomBookStore* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 126 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 126 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_name;
#line 126 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 126 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 3755 "GomSerializationTest.c"
}


void gom_book_store_set_name (GomBookStore* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 126 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 126 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 126 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 126 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_name);
#line 126 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_name = _tmp1_;
#line 126 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 3774 "GomSerializationTest.c"
}


GomBookArray* gom_book_store_get_books (GomBookStore* self) {
	GomBookArray* result;
	GomBookArray* _tmp0_ = NULL;
#line 127 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 127 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_books;
#line 127 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 127 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 3789 "GomSerializationTest.c"
}


void gom_book_store_set_books (GomBookStore* self, GomBookArray* value) {
	GomBookArray* _tmp0_ = NULL;
	GomBookArray* _tmp1_ = NULL;
#line 127 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 127 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 127 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 127 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_books);
#line 127 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_books = _tmp1_;
#line 127 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "books");
#line 3808 "GomSerializationTest.c"
}


static GObject * gom_book_store_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomBookStore * self;
	GError * _inner_error_ = NULL;
#line 128 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_book_store_parent_class);
#line 128 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 128 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_GOM_BOOK_STORE, GomBookStore);
#line 3823 "GomSerializationTest.c"
	{
#line 129 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_initialize ((GXmlGomElement*) self, "BookStore");
#line 3827 "GomSerializationTest.c"
	}
	goto __finally245;
	__catch245_g_error:
	{
#line 129 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 129 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 129 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 3838 "GomSerializationTest.c"
	}
	__finally245:
#line 129 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 129 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 129 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 3847 "GomSerializationTest.c"
	}
#line 128 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 3851 "GomSerializationTest.c"
}


static void gom_book_store_class_init (GomBookStoreClass * klass) {
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_book_store_parent_class = g_type_class_peek_parent (klass);
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_type_class_add_private (klass, sizeof (GomBookStorePrivate));
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_gom_book_store_get_property;
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_gom_book_store_set_property;
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_book_store_constructor;
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->finalize = gom_book_store_finalize;
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_BOOK_STORE_NAME, g_param_spec_string ("name", "::name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_BOOK_STORE_BOOKS, g_param_spec_object ("books", "books", "books", GOM_BOOK_TYPE_ARRAY, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 3872 "GomSerializationTest.c"
}


static void gom_book_store_instance_init (GomBookStore * self) {
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv = GOM_BOOK_STORE_GET_PRIVATE (self);
#line 3879 "GomSerializationTest.c"
}


static void gom_book_store_finalize (GObject* obj) {
	GomBookStore * self;
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_GOM_BOOK_STORE, GomBookStore);
#line 126 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_name);
#line 127 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_books);
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (gom_book_store_parent_class)->finalize (obj);
#line 3893 "GomSerializationTest.c"
}


GType gom_book_store_get_type (void) {
	static volatile gsize gom_book_store_type_id__volatile = 0;
	if (g_once_init_enter (&gom_book_store_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomBookStoreClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_book_store_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomBookStore), 0, (GInstanceInitFunc) gom_book_store_instance_init, NULL };
		GType gom_book_store_type_id;
		gom_book_store_type_id = g_type_register_static (GXML_TYPE_GOM_ELEMENT, "GomBookStore", &g_define_type_info, 0);
		g_once_init_leave (&gom_book_store_type_id__volatile, gom_book_store_type_id);
	}
	return gom_book_store_type_id__volatile;
}


static void _vala_gom_book_store_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GomBookStore * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_GOM_BOOK_STORE, GomBookStore);
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_BOOK_STORE_NAME:
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_string (value, gom_book_store_get_name (self));
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_BOOK_STORE_BOOKS:
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_book_store_get_books (self));
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 3926 "GomSerializationTest.c"
		default:
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 3932 "GomSerializationTest.c"
	}
}


static void _vala_gom_book_store_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GomBookStore * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_GOM_BOOK_STORE, GomBookStore);
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_BOOK_STORE_NAME:
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_book_store_set_name (self, g_value_get_string (value));
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_BOOK_STORE_BOOKS:
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_book_store_set_books (self, g_value_get_object (value));
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 3954 "GomSerializationTest.c"
		default:
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 123 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 3960 "GomSerializationTest.c"
	}
}


static gboolean string_contains (const gchar* self, const gchar* needle) {
	gboolean result = FALSE;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 1376 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1376 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	g_return_val_if_fail (needle != NULL, FALSE);
#line 1377 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	_tmp0_ = needle;
#line 1377 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	_tmp1_ = strstr ((gchar*) self, (gchar*) _tmp0_);
#line 1377 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	result = _tmp1_ != NULL;
#line 1377 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	return result;
#line 3981 "GomSerializationTest.c"
}


static void __lambda241_ (void) {
	GError * _inner_error_ = NULL;
	{
		GomSerializationTestBook* b = NULL;
		GomSerializationTestBook* _tmp0_ = NULL;
		GXmlXParser* parser = NULL;
		GXmlXParser* _tmp1_ = NULL;
		gchar* s = NULL;
		gchar* _tmp2_ = NULL;
		const gchar* _tmp3_ = NULL;
		const gchar* _tmp4_ = NULL;
		gboolean _tmp5_ = FALSE;
		gchar* _tmp6_ = NULL;
		gchar* _tmp7_ = NULL;
		gchar* _tmp8_ = NULL;
#line 409 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp0_ = gom_serialization_test_book_new ();
#line 409 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		b = _tmp0_;
#line 410 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp1_ = gxml_xparser_new ((GXmlDomNode*) b);
#line 410 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		parser = _tmp1_;
#line 411 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = gxml_parser_write_string ((GXmlParser*) parser, &_inner_error_);
#line 411 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp2_;
#line 411 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 411 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (parser);
#line 411 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (b);
#line 4018 "GomSerializationTest.c"
			goto __catch246_g_error;
		}
#line 412 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = s;
#line 412 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp3_ != NULL, "s != null");
#line 413 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp4_ = s;
#line 413 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp5_ = string_contains (_tmp4_, "<Book/>");
#line 413 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp5_, "\"<Book/>\" in s");
#line 414 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_set_name (b, "My Book");
#line 415 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp6_ = gxml_dom_element_get_attribute ((GXmlDomElement*) b, "name");
#line 415 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp7_ = _tmp6_;
#line 415 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (g_strcmp0 (_tmp7_, "My Book") == 0, "b.get_attribute (\"name\") == \"My Book\"");
#line 415 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp7_);
#line 416 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp8_ = gom_serialization_test_book_to_string (b);
#line 416 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 416 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp8_;
#line 408 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 408 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (parser);
#line 408 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (b);
#line 4053 "GomSerializationTest.c"
	}
	goto __finally246;
	__catch246_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp9_ = NULL;
		gchar* _tmp10_ = NULL;
		gchar* _tmp11_ = NULL;
#line 408 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		e = _inner_error_;
#line 408 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 421 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp9_ = e->message;
#line 421 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp10_ = g_strconcat ("Error: ", _tmp9_, NULL);
#line 421 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp11_ = _tmp10_;
#line 421 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:421: %s", _tmp11_);
#line 421 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp11_);
#line 422 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 408 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_error_free0 (e);
#line 4080 "GomSerializationTest.c"
	}
	__finally246:
#line 408 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 408 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 408 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 408 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return;
#line 4091 "GomSerializationTest.c"
	}
}


static void ___lambda241__gtest_func (void) {
#line 407 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	__lambda241_ ();
#line 4099 "GomSerializationTest.c"
}


static void __lambda242_ (void) {
	GomSerializationTestComputer* c = NULL;
	GomSerializationTestComputer* _tmp0_ = NULL;
	gchar* s = NULL;
	gchar* _tmp1_ = NULL;
	const gchar* _tmp2_ = NULL;
	const gchar* _tmp3_ = NULL;
	gboolean _tmp4_ = FALSE;
	const gchar* _tmp5_ = NULL;
	const gchar* _tmp6_ = NULL;
	gchar* _tmp7_ = NULL;
	gchar* _tmp8_ = NULL;
	const gchar* _tmp9_ = NULL;
	const gchar* _tmp10_ = NULL;
	gchar* _tmp11_ = NULL;
	gchar* _tmp12_ = NULL;
	gchar* _tmp13_ = NULL;
#line 426 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = gom_serialization_test_computer_new ();
#line 426 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	c = _tmp0_;
#line 427 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = gom_serialization_test_computer_to_string (c);
#line 427 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp1_;
#line 428 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp2_ = s;
#line 428 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp2_ != NULL, "s != null");
#line 429 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp3_ = s;
#line 429 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp4_ = string_contains (_tmp3_, "<Computer/>");
#line 429 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp4_, "\"<Computer/>\" in s");
#line 430 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_computer_set_model (c, "T3456-E");
#line 431 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_computer_set_ignore (c, "Nothing");
#line 432 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp5_ = gom_serialization_test_computer_get_model (c);
#line 432 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp6_ = _tmp5_;
#line 432 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (g_strcmp0 (_tmp6_, "T3456-E") == 0, "c.model == \"T3456-E\"");
#line 433 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp7_ = gxml_dom_element_get_attribute ((GXmlDomElement*) c, "model");
#line 433 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp8_ = _tmp7_;
#line 433 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (g_strcmp0 (_tmp8_, "T3456-E") == 0, "c.get_attribute (\"model\") == \"T3456-E\"");
#line 433 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp8_);
#line 434 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp9_ = gom_serialization_test_computer_get_ignore (c);
#line 434 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp10_ = _tmp9_;
#line 434 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (g_strcmp0 (_tmp10_, "Nothing") == 0, "c.ignore == \"Nothing\"");
#line 435 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp11_ = gxml_dom_element_get_attribute ((GXmlDomElement*) c, "ignore");
#line 435 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp12_ = _tmp11_;
#line 435 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp12_ == NULL, "c.get_attribute (\"ignore\") == null");
#line 435 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp12_);
#line 436 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp13_ = gom_serialization_test_computer_to_string (c);
#line 436 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (s);
#line 436 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp13_;
#line 425 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (s);
#line 425 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (c);
#line 4180 "GomSerializationTest.c"
}


static void ___lambda242__gtest_func (void) {
#line 425 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	__lambda242_ ();
#line 4187 "GomSerializationTest.c"
}


static void __lambda243_ (void) {
	GomSerializationTestTaxes* t = NULL;
	GomSerializationTestTaxes* _tmp0_ = NULL;
	gchar* s = NULL;
	gchar* _tmp1_ = NULL;
	const gchar* _tmp2_ = NULL;
	const gchar* _tmp3_ = NULL;
	gboolean _tmp4_ = FALSE;
	const gchar* _tmp5_ = NULL;
	gboolean _tmp6_ = FALSE;
	const gchar* _tmp7_ = NULL;
	gboolean _tmp8_ = FALSE;
	const gchar* _tmp9_ = NULL;
	gboolean _tmp10_ = FALSE;
	gdouble _tmp11_ = 0.0;
	gdouble _tmp12_ = 0.0;
	gchar* _tmp13_ = NULL;
	gchar* _tmp14_ = NULL;
	gboolean _tmp15_ = FALSE;
	gchar* _tmp16_ = NULL;
	gchar* _tmp17_ = NULL;
	gboolean _tmp18_ = FALSE;
	GomSerializationTestTaxesMonth _tmp19_ = 0;
	GomSerializationTestTaxesMonth _tmp20_ = 0;
	gchar* _tmp21_ = NULL;
	gchar* _tmp22_ = NULL;
	gboolean _tmp23_ = FALSE;
	gboolean _tmp24_ = FALSE;
	gchar* _tmp25_ = NULL;
	gchar* _tmp26_ = NULL;
	gchar* _tmp27_ = NULL;
	const gchar* _tmp28_ = NULL;
	gboolean _tmp29_ = FALSE;
	const gchar* _tmp30_ = NULL;
	gboolean _tmp31_ = FALSE;
	const gchar* _tmp32_ = NULL;
	gboolean _tmp33_ = FALSE;
	const gchar* _tmp34_ = NULL;
	gboolean _tmp35_ = FALSE;
#line 442 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = gom_serialization_test_taxes_new ();
#line 442 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	t = _tmp0_;
#line 443 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = gom_serialization_test_taxes_to_string (t);
#line 443 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp1_;
#line 444 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp2_ = s;
#line 444 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp2_ != NULL, "s != null");
#line 448 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp3_ = s;
#line 448 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp4_ = string_contains (_tmp3_, "<Taxes ");
#line 448 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp4_, "\"<Taxes \" in s");
#line 449 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp5_ = s;
#line 449 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp6_ = string_contains (_tmp5_, "monthRate=\"0\"");
#line 449 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp6_, "\"monthRate=\\\"0\\\"\" in s");
#line 450 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp7_ = s;
#line 450 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp8_ = string_contains (_tmp7_, "Month=\"january\"");
#line 450 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp8_, "\"Month=\\\"january\\\"\" in s");
#line 451 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp9_ = s;
#line 451 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp10_ = string_contains (_tmp9_, "TaxFree=\"false\"");
#line 451 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp10_, "\"TaxFree=\\\"false\\\"\" in s");
#line 452 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_taxes_set_month_rate (t, 16.5);
#line 453 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp11_ = gom_serialization_test_taxes_get_month_rate (t);
#line 453 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp12_ = _tmp11_;
#line 453 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp13_ = g_strdup_printf ("%.2f", _tmp12_);
#line 453 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp14_ = _tmp13_;
#line 453 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp15_ = string_contains (_tmp14_, "16.5");
#line 453 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp15_, "\"16.5\" in \"%.2f\".printf (t.month_rate)");
#line 453 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp14_);
#line 454 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp16_ = gxml_dom_element_get_attribute ((GXmlDomElement*) t, "monthrate");
#line 454 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp17_ = _tmp16_;
#line 454 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp18_ = string_contains (_tmp17_, "16.5");
#line 454 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp18_, "\"16.5\" in t.get_attribute (\"monthrate\")");
#line 454 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp17_);
#line 455 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_taxes_set_month (t, GOM_SERIALIZATION_TEST_TAXES_MONTH_FEBRUARY);
#line 456 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp19_ = gom_serialization_test_taxes_get_month (t);
#line 456 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp20_ = _tmp19_;
#line 456 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp20_ == GOM_SERIALIZATION_TEST_TAXES_MONTH_FEBRUARY, "t.month == Taxes.Month.FEBRUARY");
#line 457 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp21_ = gxml_dom_element_get_attribute ((GXmlDomElement*) t, "month");
#line 457 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp22_ = _tmp21_;
#line 457 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (g_strcmp0 (_tmp22_, "february") == 0, "t.get_attribute (\"month\") == \"february\"");
#line 457 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp22_);
#line 458 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_taxes_set_tax_free (t, TRUE);
#line 459 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp23_ = gom_serialization_test_taxes_get_tax_free (t);
#line 459 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp24_ = _tmp23_;
#line 459 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp24_ == TRUE, "t.tax_free == true");
#line 460 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp25_ = gxml_dom_element_get_attribute ((GXmlDomElement*) t, "taxfree");
#line 460 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp26_ = _tmp25_;
#line 460 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (g_strcmp0 (_tmp26_, "true") == 0, "t.get_attribute (\"taxfree\") == \"true\"");
#line 460 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp26_);
#line 461 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp27_ = gom_serialization_test_taxes_to_string (t);
#line 461 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (s);
#line 461 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp27_;
#line 462 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp28_ = s;
#line 462 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp29_ = string_contains (_tmp28_, "<Taxes ");
#line 462 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp29_, "\"<Taxes \" in s");
#line 463 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp30_ = s;
#line 463 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp31_ = string_contains (_tmp30_, "monthRate=\"16.5\"");
#line 463 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp31_, "\"monthRate=\\\"16.5\\\"\" in s");
#line 464 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp32_ = s;
#line 464 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp33_ = string_contains (_tmp32_, "Month=\"february\"");
#line 464 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp33_, "\"Month=\\\"february\\\"\" in s");
#line 465 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp34_ = s;
#line 465 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp35_ = string_contains (_tmp34_, "TaxFree=\"true\"");
#line 465 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp35_, "\"TaxFree=\\\"true\\\"\" in s");
#line 441 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (s);
#line 441 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (t);
#line 4358 "GomSerializationTest.c"
}


static void ___lambda243__gtest_func (void) {
#line 441 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	__lambda243_ ();
#line 4365 "GomSerializationTest.c"
}


static void __lambda244_ (void) {
	GomSerializationTestTaxes* t = NULL;
	GomSerializationTestTaxes* _tmp0_ = NULL;
	gchar* s = NULL;
	gchar* _tmp1_ = NULL;
	const gchar* _tmp2_ = NULL;
	const gchar* _tmp3_ = NULL;
	gboolean _tmp4_ = FALSE;
	const gchar* _tmp5_ = NULL;
	gboolean _tmp6_ = FALSE;
	const gchar* _tmp7_ = NULL;
	gboolean _tmp8_ = FALSE;
	const gchar* _tmp9_ = NULL;
	gboolean _tmp10_ = FALSE;
	GXmlGomDate* _tmp11_ = NULL;
	GXmlGomDate* _tmp12_ = NULL;
	GDate d = {0};
	gboolean _tmp13_ = FALSE;
	GXmlGomDate* _tmp14_ = NULL;
	GXmlGomDate* _tmp15_ = NULL;
	GXmlGomDate* _tmp16_ = NULL;
	GXmlGomDate* _tmp17_ = NULL;
	GDate _tmp18_ = {0};
	GXmlGomDate* _tmp19_ = NULL;
	GXmlGomDate* _tmp20_ = NULL;
	GDate _tmp21_ = {0};
	gboolean _tmp22_ = FALSE;
	GXmlGomDate* _tmp23_ = NULL;
	GXmlGomDate* _tmp24_ = NULL;
	gchar* _tmp25_ = NULL;
	gchar* _tmp26_ = NULL;
	gchar* _tmp27_ = NULL;
	GXmlGomDate* _tmp28_ = NULL;
	GXmlGomDate* _tmp29_ = NULL;
	gchar* _tmp30_ = NULL;
	gchar* _tmp31_ = NULL;
	gchar* _tmp32_ = NULL;
	GXmlGomDate* _tmp33_ = NULL;
	GXmlGomDate* _tmp34_ = NULL;
	GXmlGomDate* _tmp35_ = NULL;
	GXmlGomDate* _tmp36_ = NULL;
	GDate _tmp37_ = {0};
	gboolean _tmp38_ = FALSE;
	GXmlGomDate* _tmp39_ = NULL;
	GXmlGomDate* _tmp40_ = NULL;
	gchar* _tmp41_ = NULL;
	gchar* _tmp42_ = NULL;
	gchar* _tmp43_ = NULL;
	GXmlGomDate* _tmp44_ = NULL;
	GXmlGomDate* _tmp45_ = NULL;
	gchar* _tmp46_ = NULL;
	gchar* _tmp47_ = NULL;
	gchar* _tmp48_ = NULL;
	gchar* _tmp49_ = NULL;
	const gchar* _tmp50_ = NULL;
	const gchar* _tmp51_ = NULL;
	gboolean _tmp52_ = FALSE;
#line 471 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = gom_serialization_test_taxes_new ();
#line 471 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	t = _tmp0_;
#line 472 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = gom_serialization_test_taxes_to_string (t);
#line 472 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp1_;
#line 473 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp2_ = s;
#line 473 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp2_ != NULL, "s != null");
#line 477 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp3_ = s;
#line 477 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp4_ = string_contains (_tmp3_, "<Taxes ");
#line 477 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp4_, "\"<Taxes \" in s");
#line 478 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp5_ = s;
#line 478 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp6_ = string_contains (_tmp5_, "monthRate=\"0\"");
#line 478 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp6_, "\"monthRate=\\\"0\\\"\" in s");
#line 479 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp7_ = s;
#line 479 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp8_ = string_contains (_tmp7_, "Month=\"january\"");
#line 479 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp8_, "\"Month=\\\"january\\\"\" in s");
#line 480 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp9_ = s;
#line 480 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp10_ = string_contains (_tmp9_, "TaxFree=\"false\"");
#line 480 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp10_, "\"TaxFree=\\\"false\\\"\" in s");
#line 481 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp11_ = gxml_gom_date_new ();
#line 481 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp12_ = _tmp11_;
#line 481 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_taxes_set_pay_date (t, _tmp12_);
#line 481 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (_tmp12_);
#line 482 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	memset (&d, 0, sizeof (GDate));
#line 483 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_date_set_dmy (&d, (GDateDay) 1, (gint) ((GDateMonth) 2), (GDateYear) 2017);
#line 484 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp13_ = g_date_valid (&d);
#line 484 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp13_, "d.valid ()");
#line 485 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp14_ = gom_serialization_test_taxes_get_pay_date (t);
#line 485 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp15_ = _tmp14_;
#line 485 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp15_ != NULL, "t.pay_date != null");
#line 486 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp16_ = gom_serialization_test_taxes_get_pay_date (t);
#line 486 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp17_ = _tmp16_;
#line 486 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp18_ = d;
#line 486 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gxml_gom_date_set_date (_tmp17_, &_tmp18_);
#line 487 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp19_ = gom_serialization_test_taxes_get_pay_date (t);
#line 487 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp20_ = _tmp19_;
#line 487 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gxml_gom_date_get_date (_tmp20_, &_tmp21_);
#line 487 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp22_ = g_date_valid (&_tmp21_);
#line 487 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp22_, "t.pay_date.get_date ().valid ()");
#line 488 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp23_ = gom_serialization_test_taxes_get_pay_date (t);
#line 488 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp24_ = _tmp23_;
#line 488 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp25_ = gxml_gom_base_property_get_value ((GXmlGomBaseProperty*) _tmp24_);
#line 488 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp26_ = _tmp25_;
#line 488 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp27_ = _tmp26_;
#line 488 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp27_ != NULL, "t.pay_date.value != null");
#line 488 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp27_);
#line 489 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp28_ = gom_serialization_test_taxes_get_pay_date (t);
#line 489 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp29_ = _tmp28_;
#line 489 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp30_ = gxml_gom_base_property_get_value ((GXmlGomBaseProperty*) _tmp29_);
#line 489 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp31_ = _tmp30_;
#line 489 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp32_ = _tmp31_;
#line 489 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (g_strcmp0 (_tmp32_, "2017-02-01") == 0, "t.pay_date.value == \"2017-02-01\"");
#line 489 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp32_);
#line 490 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp33_ = gom_serialization_test_taxes_get_pay_date (t);
#line 490 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp34_ = _tmp33_;
#line 490 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gxml_gom_property_set_value ((GXmlGomBaseProperty*) _tmp34_, "2023-3-10");
#line 491 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp35_ = gom_serialization_test_taxes_get_pay_date (t);
#line 491 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp36_ = _tmp35_;
#line 491 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gxml_gom_date_get_date (_tmp36_, &_tmp37_);
#line 491 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp38_ = g_date_valid (&_tmp37_);
#line 491 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp38_, "t.pay_date.get_date ().valid ()");
#line 492 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp39_ = gom_serialization_test_taxes_get_pay_date (t);
#line 492 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp40_ = _tmp39_;
#line 492 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp41_ = gxml_gom_base_property_get_value ((GXmlGomBaseProperty*) _tmp40_);
#line 492 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp42_ = _tmp41_;
#line 492 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp43_ = _tmp42_;
#line 492 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp43_ != NULL, "t.pay_date.value != null");
#line 492 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp43_);
#line 493 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp44_ = gom_serialization_test_taxes_get_pay_date (t);
#line 493 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp45_ = _tmp44_;
#line 493 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp46_ = gxml_gom_base_property_get_value ((GXmlGomBaseProperty*) _tmp45_);
#line 493 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp47_ = _tmp46_;
#line 493 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp48_ = _tmp47_;
#line 493 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (g_strcmp0 (_tmp48_, "2023-03-10") == 0, "t.pay_date.value == \"2023-03-10\"");
#line 493 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp48_);
#line 494 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp49_ = gom_serialization_test_taxes_to_string (t);
#line 494 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (s);
#line 494 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp49_;
#line 495 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp50_ = s;
#line 495 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp50_ != NULL, "s != null");
#line 499 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp51_ = s;
#line 499 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp52_ = string_contains (_tmp51_, "PayDate=\"2023-03-10\"");
#line 499 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp52_, "\"PayDate=\\\"2023-03-10\\\"\" in s");
#line 470 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (s);
#line 470 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (t);
#line 4594 "GomSerializationTest.c"
}


static void ___lambda244__gtest_func (void) {
#line 470 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	__lambda244_ ();
#line 4601 "GomSerializationTest.c"
}


static void __lambda245_ (void) {
	GError * _inner_error_ = NULL;
	{
		GomSerializationTestTaxes* t = NULL;
		GomSerializationTestTaxes* _tmp0_ = NULL;
		gchar* s = NULL;
		gchar* _tmp1_ = NULL;
		gchar* _tmp2_ = NULL;
		gchar* _tmp3_ = NULL;
		gboolean _tmp4_ = FALSE;
		gboolean _tmp5_ = FALSE;
		gboolean _tmp6_ = FALSE;
		gboolean _tmp7_ = FALSE;
		GXmlGomDate* _tmp8_ = NULL;
		GXmlGomDate* _tmp9_ = NULL;
		GXmlGomDate* _tmp10_ = NULL;
		GXmlGomDate* _tmp11_ = NULL;
		GDate _tmp12_ = {0};
		gboolean _tmp13_ = FALSE;
		GXmlGomDate* _tmp14_ = NULL;
		GXmlGomDate* _tmp15_ = NULL;
		gchar* _tmp16_ = NULL;
		gchar* _tmp17_ = NULL;
		gchar* _tmp18_ = NULL;
		GXmlGomDate* _tmp19_ = NULL;
		GXmlGomDate* _tmp20_ = NULL;
		gchar* _tmp21_ = NULL;
		gchar* _tmp22_ = NULL;
		gchar* _tmp23_ = NULL;
#line 503 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp0_ = gom_serialization_test_taxes_new ();
#line 503 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		t = _tmp0_;
#line 504 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_read_from_string ((GXmlGomElement*) t, "<Taxes PayDate=\"2050-12-09\"/>", &_inner_error_);
#line 504 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 504 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (t);
#line 4644 "GomSerializationTest.c"
			goto __catch247_g_error;
		}
#line 505 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp1_ = gom_serialization_test_taxes_to_string (t);
#line 505 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp1_;
#line 506 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (s != NULL, "s != null");
#line 508 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = g_strconcat ("DOC:", s, NULL);
#line 508 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = _tmp2_;
#line 508 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:508: %s", _tmp3_);
#line 508 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp3_);
#line 510 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp4_ = string_contains (s, "<Taxes ");
#line 510 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp4_, "\"<Taxes \" in s");
#line 511 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp5_ = string_contains (s, "monthRate=\"0\"");
#line 511 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp5_, "\"monthRate=\\\"0\\\"\" in s");
#line 512 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp6_ = string_contains (s, "Month=\"january\"");
#line 512 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp6_, "\"Month=\\\"january\\\"\" in s");
#line 513 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp7_ = string_contains (s, "TaxFree=\"false\"");
#line 513 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp7_, "\"TaxFree=\\\"false\\\"\" in s");
#line 514 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp8_ = gom_serialization_test_taxes_get_pay_date (t);
#line 514 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp9_ = _tmp8_;
#line 514 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp9_ != NULL, "t.pay_date != null");
#line 515 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp10_ = gom_serialization_test_taxes_get_pay_date (t);
#line 515 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp11_ = _tmp10_;
#line 515 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_date_get_date (_tmp11_, &_tmp12_);
#line 515 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp13_ = g_date_valid (&_tmp12_);
#line 515 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp13_, "t.pay_date.get_date ().valid ()");
#line 516 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp14_ = gom_serialization_test_taxes_get_pay_date (t);
#line 516 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp15_ = _tmp14_;
#line 516 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp16_ = gxml_gom_base_property_get_value ((GXmlGomBaseProperty*) _tmp15_);
#line 516 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp17_ = _tmp16_;
#line 516 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp18_ = _tmp17_;
#line 516 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp18_ != NULL, "t.pay_date.value != null");
#line 516 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp18_);
#line 517 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp19_ = gom_serialization_test_taxes_get_pay_date (t);
#line 517 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp20_ = _tmp19_;
#line 517 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp21_ = gxml_gom_base_property_get_value ((GXmlGomBaseProperty*) _tmp20_);
#line 517 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp22_ = _tmp21_;
#line 517 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp23_ = _tmp22_;
#line 517 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (g_strcmp0 (_tmp23_, "2050-12-09") == 0, "t.pay_date.value == \"2050-12-09\"");
#line 517 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp23_);
#line 502 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 502 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (t);
#line 4725 "GomSerializationTest.c"
	}
	goto __finally247;
	__catch247_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp24_ = NULL;
		gchar* _tmp25_ = NULL;
		gchar* _tmp26_ = NULL;
#line 502 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		e = _inner_error_;
#line 502 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 519 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp24_ = e->message;
#line 519 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp25_ = g_strconcat ("Error: ", _tmp24_, NULL);
#line 519 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp26_ = _tmp25_;
#line 519 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:519: %s", _tmp26_);
#line 519 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp26_);
#line 520 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 502 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_error_free0 (e);
#line 4752 "GomSerializationTest.c"
	}
	__finally247:
#line 502 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 502 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 502 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 502 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return;
#line 4763 "GomSerializationTest.c"
	}
}


static void ___lambda245__gtest_func (void) {
#line 501 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	__lambda245_ ();
#line 4771 "GomSerializationTest.c"
}


static void __lambda246_ (void) {
	GomSerializationTestTaxes* t = NULL;
	GomSerializationTestTaxes* _tmp0_ = NULL;
	gchar* s = NULL;
	gchar* _tmp1_ = NULL;
	gboolean _tmp2_ = FALSE;
	gboolean _tmp3_ = FALSE;
	gboolean _tmp4_ = FALSE;
	gboolean _tmp5_ = FALSE;
	GXmlGomDateTime* _tmp6_ = NULL;
	GXmlGomDateTime* _tmp7_ = NULL;
	GDateTime* d = NULL;
	GDateTime* _tmp8_ = NULL;
	GXmlGomDateTime* _tmp9_ = NULL;
	GXmlGomDateTime* _tmp10_ = NULL;
	GXmlGomDateTime* _tmp11_ = NULL;
	GXmlGomDateTime* _tmp12_ = NULL;
	GXmlGomDateTime* _tmp13_ = NULL;
	GXmlGomDateTime* _tmp14_ = NULL;
	GXmlGomDateTime* _tmp15_ = NULL;
	GXmlGomDateTime* _tmp16_ = NULL;
	gchar* _tmp17_ = NULL;
	gchar* _tmp18_ = NULL;
	gchar* _tmp19_ = NULL;
	GXmlGomDateTime* _tmp20_ = NULL;
	GXmlGomDateTime* _tmp21_ = NULL;
	gchar* _tmp22_ = NULL;
	gchar* _tmp23_ = NULL;
	gchar* _tmp24_ = NULL;
	GXmlGomDateTime* _tmp25_ = NULL;
	GXmlGomDateTime* _tmp26_ = NULL;
	GXmlGomDateTime* _tmp27_ = NULL;
	GXmlGomDateTime* _tmp28_ = NULL;
	gchar* _tmp29_ = NULL;
	gchar* _tmp30_ = NULL;
	gchar* _tmp31_ = NULL;
	GXmlGomDateTime* _tmp32_ = NULL;
	GXmlGomDateTime* _tmp33_ = NULL;
	gchar* _tmp34_ = NULL;
	gchar* _tmp35_ = NULL;
	gchar* _tmp36_ = NULL;
	gchar* s2 = NULL;
	gchar* _tmp37_ = NULL;
	gboolean _tmp38_ = FALSE;
#line 524 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = gom_serialization_test_taxes_new ();
#line 524 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	t = _tmp0_;
#line 525 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = gom_serialization_test_taxes_to_string (t);
#line 525 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp1_;
#line 526 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (s != NULL, "s != null");
#line 530 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp2_ = string_contains (s, "<Taxes ");
#line 530 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp2_, "\"<Taxes \" in s");
#line 531 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp3_ = string_contains (s, "monthRate=\"0\"");
#line 531 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp3_, "\"monthRate=\\\"0\\\"\" in s");
#line 532 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp4_ = string_contains (s, "Month=\"january\"");
#line 532 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp4_, "\"Month=\\\"january\\\"\" in s");
#line 533 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp5_ = string_contains (s, "TaxFree=\"false\"");
#line 533 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp5_, "\"TaxFree=\\\"false\\\"\" in s");
#line 534 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp6_ = gxml_gom_date_time_new ();
#line 534 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp7_ = _tmp6_;
#line 534 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_taxes_set_timestamp (t, _tmp7_);
#line 534 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (_tmp7_);
#line 535 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp8_ = g_date_time_new_local (2017, 2, 10, 14, 14, 20.345);
#line 535 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	d = _tmp8_;
#line 541 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp9_ = gom_serialization_test_taxes_get_timestamp (t);
#line 541 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp10_ = _tmp9_;
#line 541 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp10_ != NULL, "t.timestamp != null");
#line 542 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp11_ = gom_serialization_test_taxes_get_timestamp (t);
#line 542 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp12_ = _tmp11_;
#line 542 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gxml_gom_date_time_set_datetime (_tmp12_, d);
#line 543 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp13_ = gom_serialization_test_taxes_get_timestamp (t);
#line 543 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp14_ = _tmp13_;
#line 543 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp14_ != NULL, "t.timestamp != null");
#line 544 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp15_ = gom_serialization_test_taxes_get_timestamp (t);
#line 544 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp16_ = _tmp15_;
#line 544 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp17_ = gxml_gom_base_property_get_value ((GXmlGomBaseProperty*) _tmp16_);
#line 544 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp18_ = _tmp17_;
#line 544 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp19_ = _tmp18_;
#line 544 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp19_ != NULL, "t.timestamp.value != null");
#line 544 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp19_);
#line 545 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp20_ = gom_serialization_test_taxes_get_timestamp (t);
#line 545 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp21_ = _tmp20_;
#line 545 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp22_ = gxml_gom_base_property_get_value ((GXmlGomBaseProperty*) _tmp21_);
#line 545 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp23_ = _tmp22_;
#line 545 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp24_ = _tmp23_;
#line 545 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (g_strcmp0 (_tmp24_, "2017-02-10T14:14:20") == 0, "t.timestamp.value == \"2017-02-10T14:14:20\"");
#line 545 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp24_);
#line 546 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp25_ = gom_serialization_test_taxes_get_timestamp (t);
#line 546 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp26_ = _tmp25_;
#line 546 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gxml_gom_property_set_value ((GXmlGomBaseProperty*) _tmp26_, "2023-3-10T15:23:10.356");
#line 547 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp27_ = gom_serialization_test_taxes_get_timestamp (t);
#line 547 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp28_ = _tmp27_;
#line 547 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp29_ = gxml_gom_base_property_get_value ((GXmlGomBaseProperty*) _tmp28_);
#line 547 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp30_ = _tmp29_;
#line 547 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp31_ = _tmp30_;
#line 547 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp31_ != NULL, "t.timestamp.value != null");
#line 547 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp31_);
#line 548 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp32_ = gom_serialization_test_taxes_get_timestamp (t);
#line 548 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp33_ = _tmp32_;
#line 548 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp34_ = gxml_gom_base_property_get_value ((GXmlGomBaseProperty*) _tmp33_);
#line 548 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp35_ = _tmp34_;
#line 548 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp36_ = _tmp35_;
#line 548 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (g_strcmp0 (_tmp36_, "2023-03-10T15:23:10") == 0, "t.timestamp.value == \"2023-03-10T15:23:10\"");
#line 548 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp36_);
#line 549 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp37_ = gom_serialization_test_taxes_to_string (t);
#line 549 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s2 = _tmp37_;
#line 550 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (s2 != NULL, "s2 != null");
#line 554 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp38_ = string_contains (s2, "Timestamp=\"2023-03-10T15:23:10\"");
#line 554 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp38_, "\"Timestamp=\\\"2023-03-10T15:23:10\\\"\" in s2");
#line 523 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (s2);
#line 523 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_date_time_unref0 (d);
#line 523 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (s);
#line 523 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (t);
#line 4955 "GomSerializationTest.c"
}


static void ___lambda246__gtest_func (void) {
#line 523 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	__lambda246_ ();
#line 4962 "GomSerializationTest.c"
}


static void __lambda247_ (void) {
	GError * _inner_error_ = NULL;
	{
		GomSerializationTestBookStand* bs = NULL;
		GomSerializationTestBookStand* _tmp0_ = NULL;
		gchar* s = NULL;
		GomSerializationTestBookStand* _tmp1_ = NULL;
		gchar* _tmp2_ = NULL;
		const gchar* _tmp3_ = NULL;
		const gchar* _tmp4_ = NULL;
		gboolean _tmp5_ = FALSE;
		GomSerializationTestBookStand* _tmp6_ = NULL;
		GXmlDomDocument* _tmp7_ = NULL;
		GXmlDomDocument* _tmp8_ = NULL;
		GomSerializationTestBookStand* _tmp9_ = NULL;
		GomSerializationTestRegisters* _tmp10_ = NULL;
		GomSerializationTestRegisters* _tmp11_ = NULL;
		GomSerializationTestBookStand* _tmp12_ = NULL;
		gboolean _tmp13_ = FALSE;
		GomSerializationTestBookStand* _tmp14_ = NULL;
		gchar* _tmp15_ = NULL;
		const gchar* _tmp16_ = NULL;
		const gchar* _tmp17_ = NULL;
		gboolean _tmp18_ = FALSE;
		GomSerializationTestBookRegister* br2 = NULL;
		GomSerializationTestBookStand* _tmp24_ = NULL;
		GomSerializationTestRegisters* _tmp25_ = NULL;
		GomSerializationTestRegisters* _tmp26_ = NULL;
		GXmlGomElement* _tmp27_ = NULL;
		GomSerializationTestBookRegister* _tmp28_ = NULL;
		GomSerializationTestBookRegister* _tmp29_ = NULL;
		GomSerializationTestBookStand* _tmp30_ = NULL;
		GomSerializationTestRegisters* _tmp31_ = NULL;
		GomSerializationTestRegisters* _tmp32_ = NULL;
		GomSerializationTestBookRegister* _tmp33_ = NULL;
		GomSerializationTestBookStand* _tmp34_ = NULL;
		gchar* _tmp35_ = NULL;
		const gchar* _tmp36_ = NULL;
		const gchar* _tmp37_ = NULL;
		gboolean _tmp38_ = FALSE;
		GomSerializationTestBookRegister* br3 = NULL;
		GomSerializationTestBookStand* _tmp39_ = NULL;
		GomSerializationTestRegisters* _tmp40_ = NULL;
		GomSerializationTestRegisters* _tmp41_ = NULL;
		GXmlGomElement* _tmp42_ = NULL;
		GomSerializationTestBookRegister* _tmp43_ = NULL;
		GomSerializationTestBookStand* _tmp44_ = NULL;
		GomSerializationTestRegisters* _tmp45_ = NULL;
		GomSerializationTestRegisters* _tmp46_ = NULL;
		GomSerializationTestBookRegister* _tmp47_ = NULL;
		GomSerializationTestBookRegister* _tmp48_ = NULL;
		GXmlDomElement* _tmp49_ = NULL;
		GomSerializationTestBookStand* _tmp50_ = NULL;
		GXmlDomDocument* _tmp51_ = NULL;
		GXmlDomDocument* _tmp52_ = NULL;
		GXmlDomElement* _tmp53_ = NULL;
		GomSerializationTestBookStand* _tmp54_ = NULL;
		GXmlDomNode* _tmp55_ = NULL;
		GXmlDomNode* _tmp56_ = NULL;
		GomSerializationTestBookRegister* br4 = NULL;
		GomSerializationTestBookStand* _tmp57_ = NULL;
		GomSerializationTestRegisters* _tmp58_ = NULL;
		GomSerializationTestRegisters* _tmp59_ = NULL;
		GXmlGomElement* _tmp60_ = NULL;
		GomSerializationTestBookRegister* _tmp61_ = NULL;
		GomSerializationTestBookStand* _tmp62_ = NULL;
		GomSerializationTestRegisters* _tmp63_ = NULL;
		GomSerializationTestRegisters* _tmp64_ = NULL;
		GomSerializationTestBookRegister* _tmp65_ = NULL;
		GomSerializationTestBookRegister* _tmp66_ = NULL;
		GomSerializationTestBookStand* _tmp67_ = NULL;
		gchar* _tmp68_ = NULL;
		const gchar* _tmp69_ = NULL;
		const gchar* _tmp70_ = NULL;
		gboolean _tmp71_ = FALSE;
		GXmlDomElement* _tmp72_ = NULL;
		GomSerializationTestBookStand* _tmp73_ = NULL;
		GomSerializationTestRegisters* _tmp74_ = NULL;
		GomSerializationTestRegisters* _tmp75_ = NULL;
		GXmlDomElement* _tmp76_ = NULL;
		GXmlDomElement* _tmp77_ = NULL;
		GomSerializationTestBookRegister* _tmp78_ = NULL;
		GomSerializationTestBookRegister* _tmp79_ = NULL;
		gint _tmp80_ = 0;
		gint _tmp81_ = 0;
		GXmlDomElement* _tmp82_ = NULL;
		GomSerializationTestBookStand* _tmp83_ = NULL;
		GomSerializationTestRegisters* _tmp84_ = NULL;
		GomSerializationTestRegisters* _tmp85_ = NULL;
		GXmlDomElement* _tmp86_ = NULL;
		GXmlDomElement* _tmp87_ = NULL;
		GomSerializationTestBookRegister* _tmp88_ = NULL;
		GomSerializationTestBookRegister* _tmp89_ = NULL;
		gint _tmp90_ = 0;
		gint _tmp91_ = 0;
		GXmlDomElement* _tmp92_ = NULL;
		GomSerializationTestBookStand* _tmp93_ = NULL;
		GomSerializationTestRegisters* _tmp94_ = NULL;
		GomSerializationTestRegisters* _tmp95_ = NULL;
		GXmlDomElement* _tmp96_ = NULL;
		GXmlDomElement* _tmp97_ = NULL;
		GomSerializationTestBookRegister* _tmp98_ = NULL;
		GomSerializationTestBookRegister* _tmp99_ = NULL;
		gint _tmp100_ = 0;
		gint _tmp101_ = 0;
		GomSerializationTestBookStand* _tmp102_ = NULL;
		gboolean _tmp103_ = FALSE;
		GomSerializationTestBookStand* _tmp104_ = NULL;
		GomSerializationTestBookStandDimension* _tmp105_ = NULL;
		GomSerializationTestBookStandDimension* _tmp106_ = NULL;
		GomSerializationTestBookStand* _tmp107_ = NULL;
		GomSerializationTestBookStandDimension* _tmp108_ = NULL;
		GomSerializationTestBookStandDimension* _tmp109_ = NULL;
		gdouble _tmp110_ = 0.0;
		gdouble _tmp111_ = 0.0;
		GomSerializationTestBookStand* _tmp112_ = NULL;
		gchar* _tmp113_ = NULL;
		const gchar* _tmp114_ = NULL;
		const gchar* _tmp115_ = NULL;
		gchar* _tmp116_ = NULL;
		gchar* _tmp117_ = NULL;
		const gchar* _tmp118_ = NULL;
		gboolean _tmp119_ = FALSE;
		GomSerializationTestBookStand* _tmp120_ = NULL;
		gboolean _tmp121_ = FALSE;
		GomSerializationTestBookStand* _tmp122_ = NULL;
		GomSerializationTestBookStandDimensionY* _tmp123_ = NULL;
		GomSerializationTestBookStandDimensionY* _tmp124_ = NULL;
		GomSerializationTestBookStand* _tmp125_ = NULL;
		GomSerializationTestBookStandDimensionY* _tmp126_ = NULL;
		GomSerializationTestBookStandDimensionY* _tmp127_ = NULL;
		gdouble _tmp128_ = 0.0;
		gdouble _tmp129_ = 0.0;
		GomSerializationTestBookStand* _tmp130_ = NULL;
		gchar* _tmp131_ = NULL;
		const gchar* _tmp132_ = NULL;
		const gchar* _tmp133_ = NULL;
		gchar* _tmp134_ = NULL;
		gchar* _tmp135_ = NULL;
		const gchar* _tmp136_ = NULL;
		gboolean _tmp137_ = FALSE;
		GomSerializationTestBookStand* _tmp138_ = NULL;
		gboolean _tmp139_ = FALSE;
		GomSerializationTestBookStand* _tmp140_ = NULL;
		GomSerializationTestBookStandDimensionZ* _tmp141_ = NULL;
		GomSerializationTestBookStandDimensionZ* _tmp142_ = NULL;
		GomSerializationTestBookStand* _tmp143_ = NULL;
		GomSerializationTestBookStandDimensionZ* _tmp144_ = NULL;
		GomSerializationTestBookStandDimensionZ* _tmp145_ = NULL;
		gdouble _tmp146_ = 0.0;
		gdouble _tmp147_ = 0.0;
		GomSerializationTestBookStand* _tmp148_ = NULL;
		gchar* _tmp149_ = NULL;
		const gchar* _tmp150_ = NULL;
		const gchar* _tmp151_ = NULL;
		gchar* _tmp152_ = NULL;
		gchar* _tmp153_ = NULL;
		const gchar* _tmp154_ = NULL;
		gboolean _tmp155_ = FALSE;
#line 558 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp0_ = gom_serialization_test_book_stand_new ();
#line 558 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		bs = _tmp0_;
#line 559 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp1_ = bs;
#line 559 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = gom_serialization_test_book_stand_to_string (_tmp1_);
#line 559 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp2_;
#line 560 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = s;
#line 560 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp3_ != NULL, "s != null");
#line 564 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp4_ = s;
#line 564 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp5_ = string_contains (_tmp4_, "<BookStand Classification=\"Science\"/>");
#line 564 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp5_, "\"<BookStand Classification=\\\"Science\\\"/>\" in s");
#line 565 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp6_ = bs;
#line 565 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp7_ = gxml_dom_node_get_owner_document ((GXmlDomNode*) _tmp6_);
#line 565 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp8_ = _tmp7_;
#line 565 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp8_ != NULL, "bs.owner_document != null");
#line 566 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp9_ = bs;
#line 566 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp10_ = gom_serialization_test_book_stand_get_registers (_tmp9_);
#line 566 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp11_ = _tmp10_;
#line 566 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp11_ == NULL, "bs.registers == null");
#line 567 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp12_ = bs;
#line 567 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp13_ = gxml_gom_object_set_instance_property ((GXmlGomObject*) _tmp12_, "registers");
#line 567 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp13_, "bs.set_instance_property (\"registers\")");
#line 568 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp14_ = bs;
#line 568 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp15_ = gom_serialization_test_book_stand_to_string (_tmp14_);
#line 568 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 568 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp15_;
#line 569 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp16_ = s;
#line 569 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp16_ != NULL, "s != null");
#line 573 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp17_ = s;
#line 573 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp18_ = string_contains (_tmp17_, "<BookStand Classification=\"Science\"/>");
#line 573 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp18_, "\"<BookStand Classification=\\\"Science\\\"/>\" in s");
#line 5185 "GomSerializationTest.c"
		{
			GomSerializationTestBookRegister* br = NULL;
			GomSerializationTestBookRegister* _tmp19_ = NULL;
			GomSerializationTestBookStand* _tmp20_ = NULL;
			GomSerializationTestRegisters* _tmp21_ = NULL;
			GomSerializationTestRegisters* _tmp22_ = NULL;
			GomSerializationTestBookRegister* _tmp23_ = NULL;
#line 575 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_tmp19_ = gom_serialization_test_book_register_new ();
#line 575 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			br = _tmp19_;
#line 576 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_tmp20_ = bs;
#line 576 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_tmp21_ = gom_serialization_test_book_stand_get_registers (_tmp20_);
#line 576 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_tmp22_ = _tmp21_;
#line 576 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_tmp23_ = br;
#line 576 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			gxml_gom_collection_append ((GXmlGomCollection*) _tmp22_, (GXmlDomElement*) _tmp23_, &_inner_error_);
#line 576 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 576 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
				_g_object_unref0 (br);
#line 5211 "GomSerializationTest.c"
				goto __catch249_g_error;
			}
#line 577 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			g_assert_not_reached ();
#line 574 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br);
#line 5218 "GomSerializationTest.c"
		}
		goto __finally249;
		__catch249_g_error:
		{
#line 574 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			g_clear_error (&_inner_error_);
#line 574 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_inner_error_ = NULL;
#line 5227 "GomSerializationTest.c"
		}
		__finally249:
#line 574 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 574 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 574 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 5236 "GomSerializationTest.c"
			goto __catch248_g_error;
		}
#line 579 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp24_ = bs;
#line 579 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp25_ = gom_serialization_test_book_stand_get_registers (_tmp24_);
#line 579 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp26_ = _tmp25_;
#line 579 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp27_ = gxml_gom_collection_create_item ((GXmlGomCollection*) _tmp26_);
#line 579 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp28_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp27_, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER) ? ((GomSerializationTestBookRegister*) _tmp27_) : NULL;
#line 579 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (_tmp28_ == NULL) {
#line 579 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp27_);
#line 5253 "GomSerializationTest.c"
		}
#line 579 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		br2 = _tmp28_;
#line 580 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp29_ = br2;
#line 580 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_register_set_year (_tmp29_, 2016);
#line 581 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp30_ = bs;
#line 581 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp31_ = gom_serialization_test_book_stand_get_registers (_tmp30_);
#line 581 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp32_ = _tmp31_;
#line 581 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp33_ = br2;
#line 581 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_collection_append ((GXmlGomCollection*) _tmp32_, (GXmlDomElement*) _tmp33_, &_inner_error_);
#line 581 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 581 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br2);
#line 581 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 581 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 5279 "GomSerializationTest.c"
			goto __catch248_g_error;
		}
#line 582 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp34_ = bs;
#line 582 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp35_ = gom_serialization_test_book_stand_to_string (_tmp34_);
#line 582 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 582 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp35_;
#line 583 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp36_ = s;
#line 583 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp36_ != NULL, "s != null");
#line 587 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp37_ = s;
#line 587 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp38_ = string_contains (_tmp37_, "<BookStand Classification=\"Science\"><BookRegister Year=\"2016\"/></B" \
"ookStand>");
#line 587 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp38_, "\"<BookStand Classification=\\\"Science\\\"><BookRegister Year=\\\"2016\\\"/></BookStand>\" in s");
#line 588 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp39_ = bs;
#line 588 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp40_ = gom_serialization_test_book_stand_get_registers (_tmp39_);
#line 588 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp41_ = _tmp40_;
#line 588 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp42_ = gxml_gom_collection_create_item ((GXmlGomCollection*) _tmp41_);
#line 588 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp43_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp42_, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER) ? ((GomSerializationTestBookRegister*) _tmp42_) : NULL;
#line 588 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (_tmp43_ == NULL) {
#line 588 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp42_);
#line 5314 "GomSerializationTest.c"
		}
#line 588 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		br3 = _tmp43_;
#line 589 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp44_ = bs;
#line 589 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp45_ = gom_serialization_test_book_stand_get_registers (_tmp44_);
#line 589 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp46_ = _tmp45_;
#line 589 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp47_ = br3;
#line 589 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_collection_append ((GXmlGomCollection*) _tmp46_, (GXmlDomElement*) _tmp47_, &_inner_error_);
#line 589 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 589 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br3);
#line 589 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br2);
#line 589 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 589 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 5338 "GomSerializationTest.c"
			goto __catch248_g_error;
		}
#line 590 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp48_ = br3;
#line 590 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_register_set_year (_tmp48_, 2010);
#line 591 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp50_ = bs;
#line 591 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp51_ = gxml_dom_node_get_owner_document ((GXmlDomNode*) _tmp50_);
#line 591 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp52_ = _tmp51_;
#line 591 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp53_ = gxml_dom_document_create_element (_tmp52_, "Test", &_inner_error_);
#line 591 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp49_ = _tmp53_;
#line 591 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 591 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br3);
#line 591 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br2);
#line 591 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 591 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 5365 "GomSerializationTest.c"
			goto __catch248_g_error;
		}
#line 591 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp54_ = bs;
#line 591 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp55_ = gxml_dom_node_append_child ((GXmlDomNode*) _tmp54_, (GXmlDomNode*) _tmp49_, &_inner_error_);
#line 591 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp56_ = _tmp55_;
#line 591 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp56_);
#line 591 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 591 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp49_);
#line 591 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br3);
#line 591 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br2);
#line 591 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 591 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 5388 "GomSerializationTest.c"
			goto __catch248_g_error;
		}
#line 592 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp57_ = bs;
#line 592 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp58_ = gom_serialization_test_book_stand_get_registers (_tmp57_);
#line 592 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp59_ = _tmp58_;
#line 592 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp60_ = gxml_gom_collection_create_item ((GXmlGomCollection*) _tmp59_);
#line 592 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp61_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp60_, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER) ? ((GomSerializationTestBookRegister*) _tmp60_) : NULL;
#line 592 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (_tmp61_ == NULL) {
#line 592 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp60_);
#line 5405 "GomSerializationTest.c"
		}
#line 592 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		br4 = _tmp61_;
#line 593 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp62_ = bs;
#line 593 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp63_ = gom_serialization_test_book_stand_get_registers (_tmp62_);
#line 593 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp64_ = _tmp63_;
#line 593 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp65_ = br4;
#line 593 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_collection_append ((GXmlGomCollection*) _tmp64_, (GXmlDomElement*) _tmp65_, &_inner_error_);
#line 593 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 593 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br4);
#line 593 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp49_);
#line 593 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br3);
#line 593 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br2);
#line 593 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 593 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 5433 "GomSerializationTest.c"
			goto __catch248_g_error;
		}
#line 594 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp66_ = br4;
#line 594 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_register_set_year (_tmp66_, 2000);
#line 595 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp67_ = bs;
#line 595 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp68_ = gom_serialization_test_book_stand_to_string (_tmp67_);
#line 595 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 595 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp68_;
#line 596 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp69_ = s;
#line 596 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp69_ != NULL, "s != null");
#line 600 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp70_ = s;
#line 600 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp71_ = string_contains (_tmp70_, "<BookStand Classification=\"Science\"><BookRegister Year=\"2016\"/><Bo" \
"okRegister Year=\"2010\"/><Test/><BookRegister Year=\"2000\"/></BookSt" \
"and>");
#line 600 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp71_, "\"<BookStand Classification=\\\"Science\\\"><BookRegister Year=\\\"2016\\\"/><BookRegister Year=\\\"2010\\\"/><Test/><BookRegister Year=\\\"2000\\\"/></BookStand>\" in s");
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp73_ = bs;
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp74_ = gom_serialization_test_book_stand_get_registers (_tmp73_);
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp75_ = _tmp74_;
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp76_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp75_, 0, &_inner_error_);
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp72_ = _tmp76_;
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br4);
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp49_);
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br3);
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br2);
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 5482 "GomSerializationTest.c"
			goto __catch248_g_error;
		}
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp77_ = _tmp72_;
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp72_ = NULL;
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp78_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp77_, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER) ? ((GomSerializationTestBookRegister*) _tmp77_) : NULL;
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (_tmp78_ == NULL) {
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp77_);
#line 5495 "GomSerializationTest.c"
		}
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp79_ = _tmp78_;
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp80_ = gom_serialization_test_book_register_get_year (_tmp79_);
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp81_ = _tmp80_;
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp81_ == 2016, "(bs.registers.get_item (0) as BookRegister).year == 2016");
#line 601 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp79_);
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp83_ = bs;
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp84_ = gom_serialization_test_book_stand_get_registers (_tmp83_);
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp85_ = _tmp84_;
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp86_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp85_, 1, &_inner_error_);
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp82_ = _tmp86_;
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp72_);
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br4);
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp49_);
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br3);
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br2);
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 5533 "GomSerializationTest.c"
			goto __catch248_g_error;
		}
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp87_ = _tmp82_;
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp82_ = NULL;
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp88_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp87_, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER) ? ((GomSerializationTestBookRegister*) _tmp87_) : NULL;
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (_tmp88_ == NULL) {
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp87_);
#line 5546 "GomSerializationTest.c"
		}
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp89_ = _tmp88_;
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp90_ = gom_serialization_test_book_register_get_year (_tmp89_);
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp91_ = _tmp90_;
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp91_ == 2010, "(bs.registers.get_item (1) as BookRegister).year == 2010");
#line 602 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp89_);
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp93_ = bs;
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp94_ = gom_serialization_test_book_stand_get_registers (_tmp93_);
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp95_ = _tmp94_;
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp96_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp95_, 2, &_inner_error_);
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp92_ = _tmp96_;
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp82_);
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp72_);
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br4);
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp49_);
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br3);
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br2);
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 5586 "GomSerializationTest.c"
			goto __catch248_g_error;
		}
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp97_ = _tmp92_;
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp92_ = NULL;
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp98_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp97_, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER) ? ((GomSerializationTestBookRegister*) _tmp97_) : NULL;
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (_tmp98_ == NULL) {
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp97_);
#line 5599 "GomSerializationTest.c"
		}
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp99_ = _tmp98_;
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp100_ = gom_serialization_test_book_register_get_year (_tmp99_);
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp101_ = _tmp100_;
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp101_ == 2000, "(bs.registers.get_item (2) as BookRegister).year == 2000");
#line 603 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp99_);
#line 604 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp102_ = bs;
#line 604 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp103_ = gxml_gom_object_set_instance_property ((GXmlGomObject*) _tmp102_, "Dimension-X");
#line 604 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp103_, "bs.set_instance_property(\"Dimension-X\")");
#line 605 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp104_ = bs;
#line 605 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp105_ = gom_serialization_test_book_stand_get_dimension_x (_tmp104_);
#line 605 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp106_ = _tmp105_;
#line 605 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp106_ != NULL, "bs.dimension_x != null");
#line 606 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp107_ = bs;
#line 606 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp108_ = gom_serialization_test_book_stand_get_dimension_x (_tmp107_);
#line 606 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp109_ = _tmp108_;
#line 606 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp110_ = gom_serialization_test_book_stand_dimension_get_length (_tmp109_);
#line 606 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp111_ = _tmp110_;
#line 606 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp111_ == 1.0, "bs.dimension_x.length == 1.0");
#line 607 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp112_ = bs;
#line 607 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp113_ = gom_serialization_test_book_stand_to_string (_tmp112_);
#line 607 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 607 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp113_;
#line 608 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp114_ = s;
#line 608 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp114_ != NULL, "s != null");
#line 610 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp115_ = s;
#line 610 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp116_ = g_strconcat ("DOC:", _tmp115_, NULL);
#line 610 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp117_ = _tmp116_;
#line 610 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:610: %s", _tmp117_);
#line 610 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp117_);
#line 612 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp118_ = s;
#line 612 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp119_ = string_contains (_tmp118_, "<BookStand Classification=\"Science\"><BookRegister Year=\"2016\"/><Bo" \
"okRegister Year=\"2010\"/><Test/><BookRegister Year=\"2000\"/><Dimensi" \
"on Length=\"1\" Type=\"x\"/></BookStand>");
#line 612 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp119_, "\"<BookStand Classification=\\\"Science\\\"><BookRegister Year=\\\"2016\\\"/><BookRegister Year=\\\"2010\\\"/><Test/><BookRegister Year=\\\"2000\\\"/><Dimension Length=\\\"1\\\" Type=\\\"x\\\"/></BookStand>\" in s");
#line 613 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp120_ = bs;
#line 613 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp121_ = gxml_gom_object_set_instance_property ((GXmlGomObject*) _tmp120_, "DimensionY");
#line 613 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp121_, "bs.set_instance_property(\"DimensionY\")");
#line 614 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp122_ = bs;
#line 614 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp123_ = gom_serialization_test_book_stand_get_dimension_y (_tmp122_);
#line 614 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp124_ = _tmp123_;
#line 614 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp124_ != NULL, "bs.dimension_y != null");
#line 615 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp125_ = bs;
#line 615 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp126_ = gom_serialization_test_book_stand_get_dimension_y (_tmp125_);
#line 615 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp127_ = _tmp126_;
#line 615 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp128_ = gom_serialization_test_book_stand_dimension_get_length ((GomSerializationTestBookStandDimension*) _tmp127_);
#line 615 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp129_ = _tmp128_;
#line 615 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp129_ == 1.0, "bs.dimension_y.length == 1.0");
#line 616 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp130_ = bs;
#line 616 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp131_ = gom_serialization_test_book_stand_to_string (_tmp130_);
#line 616 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 616 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp131_;
#line 617 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp132_ = s;
#line 617 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp132_ != NULL, "s != null");
#line 619 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp133_ = s;
#line 619 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp134_ = g_strconcat ("DOC:", _tmp133_, NULL);
#line 619 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp135_ = _tmp134_;
#line 619 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:619: %s", _tmp135_);
#line 619 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp135_);
#line 621 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp136_ = s;
#line 621 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp137_ = string_contains (_tmp136_, "<BookStand Classification=\"Science\"><BookRegister Year=\"2016\"/><Bo" \
"okRegister Year=\"2010\"/><Test/><BookRegister Year=\"2000\"/><Dimensi" \
"on Length=\"1\" Type=\"x\"/><Dimension Length=\"1\" Type=\"y\"/></Book" \
"Stand>");
#line 621 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp137_, "\"<BookStand Classification=\\\"Science\\\"><BookRegister Year=\\\"2016\\\"/><BookRegister Year=\\\"2010\\\"/><Test/><BookRegister Year=\\\"2000\\\"/><Dimension Length=\\\"1\\\" Type=\\\"x\\\"/><Dimension Length=\\\"1\\\" Type=\\\"y\\\"/></BookStand>\" in s");
#line 622 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp138_ = bs;
#line 622 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp139_ = gxml_gom_object_set_instance_property ((GXmlGomObject*) _tmp138_, "::DimensionZ");
#line 622 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp139_, "bs.set_instance_property(\"::DimensionZ\")");
#line 623 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp140_ = bs;
#line 623 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp141_ = gom_serialization_test_book_stand_get_dimension_z (_tmp140_);
#line 623 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp142_ = _tmp141_;
#line 623 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp142_ != NULL, "bs.dimension_z != null");
#line 624 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp143_ = bs;
#line 624 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp144_ = gom_serialization_test_book_stand_get_dimension_z (_tmp143_);
#line 624 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp145_ = _tmp144_;
#line 624 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp146_ = gom_serialization_test_book_stand_dimension_get_length ((GomSerializationTestBookStandDimension*) _tmp145_);
#line 624 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp147_ = _tmp146_;
#line 624 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp147_ == 1.0, "bs.dimension_z.length == 1.0");
#line 625 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp148_ = bs;
#line 625 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp149_ = gom_serialization_test_book_stand_to_string (_tmp148_);
#line 625 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 625 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp149_;
#line 626 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp150_ = s;
#line 626 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp150_ != NULL, "s != null");
#line 628 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp151_ = s;
#line 628 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp152_ = g_strconcat ("DOC:", _tmp151_, NULL);
#line 628 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp153_ = _tmp152_;
#line 628 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:628: %s", _tmp153_);
#line 628 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp153_);
#line 630 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp154_ = s;
#line 630 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp155_ = string_contains (_tmp154_, "<BookStand Classification=\"Science\"><BookRegister Year=\"2016\"/><Bo" \
"okRegister Year=\"2010\"/><Test/><BookRegister Year=\"2000\"/><Dimensi" \
"on Length=\"1\" Type=\"x\"/><Dimension Length=\"1\" Type=\"y\"/><Dimen" \
"sion Length=\"1\" Type=\"z\"/></BookStand>");
#line 630 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp155_, "\"<BookStand Classification=\\\"Science\\\"><BookRegister Year=\\\"2016\\\"/><BookRegister Year=\\\"2010\\\"/><Test/><BookRegister Year=\\\"2000\\\"/><Dimension Length=\\\"1\\\" Type=\\\"x\\\"/><Dimension Length=\\\"1\\\" Type=\\\"y\\\"/><Dimension Length=\\\"1\\\" Type=\\\"z\\\"/></BookStand>\" in s");
#line 557 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp92_);
#line 557 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp82_);
#line 557 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp72_);
#line 557 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (br4);
#line 557 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp49_);
#line 557 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (br3);
#line 557 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (br2);
#line 557 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 557 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (bs);
#line 5791 "GomSerializationTest.c"
	}
	goto __finally248;
	__catch248_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp156_ = NULL;
		gchar* _tmp157_ = NULL;
		gchar* _tmp158_ = NULL;
#line 557 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		e = _inner_error_;
#line 557 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 632 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp156_ = e->message;
#line 632 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp157_ = g_strconcat ("Error: ", _tmp156_, NULL);
#line 632 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp158_ = _tmp157_;
#line 632 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:632: %s", _tmp158_);
#line 632 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp158_);
#line 633 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 557 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_error_free0 (e);
#line 5818 "GomSerializationTest.c"
	}
	__finally248:
#line 557 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 557 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 557 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 557 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return;
#line 5829 "GomSerializationTest.c"
	}
}


static void ___lambda247__gtest_func (void) {
#line 556 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	__lambda247_ ();
#line 5837 "GomSerializationTest.c"
}


static void __lambda248_ (void) {
	GError * _inner_error_ = NULL;
	{
		GomSerializationTestBookStore* bs = NULL;
		GomSerializationTestBookStore* _tmp0_ = NULL;
		gchar* s = NULL;
		GomSerializationTestBookStore* _tmp1_ = NULL;
		gchar* _tmp2_ = NULL;
		const gchar* _tmp3_ = NULL;
		const gchar* _tmp4_ = NULL;
		gboolean _tmp5_ = FALSE;
		GomSerializationTestBookStore* _tmp6_ = NULL;
		GomSerializationTestBooks* _tmp7_ = NULL;
		GomSerializationTestBooks* _tmp8_ = NULL;
		GomSerializationTestBookStore* _tmp9_ = NULL;
		GomSerializationTestBookStore* _tmp10_ = NULL;
		gchar* _tmp11_ = NULL;
		const gchar* _tmp12_ = NULL;
		const gchar* _tmp13_ = NULL;
		gboolean _tmp14_ = FALSE;
		GomSerializationTestBook* b = NULL;
		GomSerializationTestBook* _tmp15_ = NULL;
		GomSerializationTestBookStore* _tmp20_ = NULL;
		GXmlDomDocument* _tmp21_ = NULL;
		GXmlDomDocument* _tmp22_ = NULL;
		GomSerializationTestBook* _tmp23_ = NULL;
		GomSerializationTestBook* _tmp24_ = NULL;
		GomSerializationTestBookStore* _tmp25_ = NULL;
		GomSerializationTestBooks* _tmp26_ = NULL;
		GomSerializationTestBooks* _tmp27_ = NULL;
		GomSerializationTestBook* _tmp28_ = NULL;
		GomSerializationTestBookStore* _tmp29_ = NULL;
		gchar* _tmp30_ = NULL;
		const gchar* _tmp31_ = NULL;
		const gchar* _tmp32_ = NULL;
		gboolean _tmp33_ = FALSE;
		GomSerializationTestBook* b2 = NULL;
		GomSerializationTestBookStore* _tmp34_ = NULL;
		GXmlDomDocument* _tmp35_ = NULL;
		GXmlDomDocument* _tmp36_ = NULL;
		GomSerializationTestBook* _tmp37_ = NULL;
		GomSerializationTestBook* _tmp38_ = NULL;
		GomSerializationTestBookStore* _tmp39_ = NULL;
		GomSerializationTestBooks* _tmp40_ = NULL;
		GomSerializationTestBooks* _tmp41_ = NULL;
		GomSerializationTestBook* _tmp42_ = NULL;
		GXmlDomElement* _tmp43_ = NULL;
		GomSerializationTestBookStore* _tmp44_ = NULL;
		GXmlDomDocument* _tmp45_ = NULL;
		GXmlDomDocument* _tmp46_ = NULL;
		GXmlDomElement* _tmp47_ = NULL;
		GomSerializationTestBookStore* _tmp48_ = NULL;
		GXmlDomNode* _tmp49_ = NULL;
		GXmlDomNode* _tmp50_ = NULL;
		GomSerializationTestBook* b3 = NULL;
		GomSerializationTestBookStore* _tmp51_ = NULL;
		GXmlDomDocument* _tmp52_ = NULL;
		GXmlDomDocument* _tmp53_ = NULL;
		GomSerializationTestBook* _tmp54_ = NULL;
		GomSerializationTestBook* _tmp55_ = NULL;
		GomSerializationTestBookStore* _tmp56_ = NULL;
		GomSerializationTestBooks* _tmp57_ = NULL;
		GomSerializationTestBooks* _tmp58_ = NULL;
		GomSerializationTestBook* _tmp59_ = NULL;
		GomSerializationTestBookStore* _tmp60_ = NULL;
		gchar* _tmp61_ = NULL;
		const gchar* _tmp62_ = NULL;
		const gchar* _tmp63_ = NULL;
		gboolean _tmp64_ = FALSE;
		GomSerializationTestBookStore* _tmp65_ = NULL;
		GomSerializationTestBooks* _tmp66_ = NULL;
		GomSerializationTestBooks* _tmp67_ = NULL;
		GXmlDomElement* _tmp68_ = NULL;
		GXmlDomElement* _tmp69_ = NULL;
		GomSerializationTestBookStore* _tmp70_ = NULL;
		GomSerializationTestBooks* _tmp71_ = NULL;
		GomSerializationTestBooks* _tmp72_ = NULL;
		GXmlDomElement* _tmp73_ = NULL;
		GXmlDomElement* _tmp74_ = NULL;
		GomSerializationTestBookStore* _tmp75_ = NULL;
		GomSerializationTestBooks* _tmp76_ = NULL;
		GomSerializationTestBooks* _tmp77_ = NULL;
		GXmlDomElement* _tmp78_ = NULL;
		GXmlDomElement* _tmp79_ = NULL;
		GomSerializationTestBookStore* _tmp80_ = NULL;
		GomSerializationTestBooks* _tmp81_ = NULL;
		GomSerializationTestBooks* _tmp82_ = NULL;
		GXmlDomElement* _tmp83_ = NULL;
		GomSerializationTestBook* _tmp84_ = NULL;
		GomSerializationTestBook* _tmp85_ = NULL;
		const gchar* _tmp86_ = NULL;
		const gchar* _tmp87_ = NULL;
		GomSerializationTestBookStore* _tmp88_ = NULL;
		GomSerializationTestBooks* _tmp89_ = NULL;
		GomSerializationTestBooks* _tmp90_ = NULL;
		GXmlDomElement* _tmp91_ = NULL;
		GomSerializationTestBook* _tmp92_ = NULL;
		GomSerializationTestBook* _tmp93_ = NULL;
		const gchar* _tmp94_ = NULL;
		const gchar* _tmp95_ = NULL;
		GomSerializationTestBookStore* _tmp96_ = NULL;
		GomSerializationTestBooks* _tmp97_ = NULL;
		GomSerializationTestBooks* _tmp98_ = NULL;
		GXmlDomElement* _tmp99_ = NULL;
		GomSerializationTestBook* _tmp100_ = NULL;
		GomSerializationTestBook* _tmp101_ = NULL;
		const gchar* _tmp102_ = NULL;
		const gchar* _tmp103_ = NULL;
#line 638 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp0_ = gom_serialization_test_book_store_new ();
#line 638 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		bs = _tmp0_;
#line 639 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp1_ = bs;
#line 639 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = gom_serialization_test_book_store_to_string (_tmp1_);
#line 639 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp2_;
#line 640 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = s;
#line 640 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp3_ != NULL, "s != null");
#line 644 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp4_ = s;
#line 644 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp5_ = string_contains (_tmp4_, "<BookStore/>");
#line 644 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp5_, "\"<BookStore/>\" in s");
#line 645 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp6_ = bs;
#line 645 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp7_ = gom_serialization_test_book_store_get_books (_tmp6_);
#line 645 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp8_ = _tmp7_;
#line 645 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp8_ == NULL, "bs.books == null");
#line 646 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp9_ = bs;
#line 646 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_object_set_instance_property ((GXmlGomObject*) _tmp9_, "books");
#line 647 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp10_ = bs;
#line 647 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp11_ = gom_serialization_test_book_store_to_string (_tmp10_);
#line 647 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 647 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp11_;
#line 648 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp12_ = s;
#line 648 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp12_ != NULL, "s != null");
#line 652 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp13_ = s;
#line 652 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp14_ = string_contains (_tmp13_, "<BookStore/>");
#line 652 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp14_, "\"<BookStore/>\" in s");
#line 653 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp15_ = gom_serialization_test_book_new ();
#line 653 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		b = _tmp15_;
#line 6003 "GomSerializationTest.c"
		{
			GomSerializationTestBookStore* _tmp16_ = NULL;
			GomSerializationTestBooks* _tmp17_ = NULL;
			GomSerializationTestBooks* _tmp18_ = NULL;
			GomSerializationTestBook* _tmp19_ = NULL;
#line 655 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_tmp16_ = bs;
#line 655 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_tmp17_ = gom_serialization_test_book_store_get_books (_tmp16_);
#line 655 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_tmp18_ = _tmp17_;
#line 655 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_tmp19_ = b;
#line 655 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			gxml_gom_collection_append ((GXmlGomCollection*) _tmp18_, (GXmlDomElement*) _tmp19_, &_inner_error_);
#line 655 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 6021 "GomSerializationTest.c"
				goto __catch251_g_error;
			}
#line 656 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			g_assert_not_reached ();
#line 6026 "GomSerializationTest.c"
		}
		goto __finally251;
		__catch251_g_error:
		{
#line 654 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			g_clear_error (&_inner_error_);
#line 654 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_inner_error_ = NULL;
#line 6035 "GomSerializationTest.c"
		}
		__finally251:
#line 654 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 654 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (b);
#line 654 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 654 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 6046 "GomSerializationTest.c"
			goto __catch250_g_error;
		}
#line 658 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp20_ = bs;
#line 658 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp21_ = gxml_dom_node_get_owner_document ((GXmlDomNode*) _tmp20_);
#line 658 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp22_ = _tmp21_;
#line 658 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp23_ = gom_serialization_test_book_new_document (_tmp22_);
#line 658 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (b);
#line 658 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		b = _tmp23_;
#line 659 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp24_ = b;
#line 659 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_set_name (_tmp24_, "Title1");
#line 660 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp25_ = bs;
#line 660 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp26_ = gom_serialization_test_book_store_get_books (_tmp25_);
#line 660 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp27_ = _tmp26_;
#line 660 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp28_ = b;
#line 660 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_collection_append ((GXmlGomCollection*) _tmp27_, (GXmlDomElement*) _tmp28_, &_inner_error_);
#line 660 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 660 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (b);
#line 660 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 660 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 6083 "GomSerializationTest.c"
			goto __catch250_g_error;
		}
#line 661 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp29_ = bs;
#line 661 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp30_ = gom_serialization_test_book_store_to_string (_tmp29_);
#line 661 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 661 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp30_;
#line 662 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp31_ = s;
#line 662 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp31_ != NULL, "s != null");
#line 666 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp32_ = s;
#line 666 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp33_ = string_contains (_tmp32_, "<BookStore><Book Name=\"Title1\"/></BookStore>");
#line 666 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp33_, "\"<BookStore><Book Name=\\\"Title1\\\"/></BookStore>\" in s");
#line 667 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp34_ = bs;
#line 667 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp35_ = gxml_dom_node_get_owner_document ((GXmlDomNode*) _tmp34_);
#line 667 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp36_ = _tmp35_;
#line 667 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp37_ = gom_serialization_test_book_new_document (_tmp36_);
#line 667 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		b2 = _tmp37_;
#line 668 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp38_ = b2;
#line 668 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_set_name (_tmp38_, "Title2");
#line 669 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp39_ = bs;
#line 669 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp40_ = gom_serialization_test_book_store_get_books (_tmp39_);
#line 669 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp41_ = _tmp40_;
#line 669 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp42_ = b2;
#line 669 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_collection_append ((GXmlGomCollection*) _tmp41_, (GXmlDomElement*) _tmp42_, &_inner_error_);
#line 669 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 669 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (b2);
#line 669 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (b);
#line 669 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 669 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 6138 "GomSerializationTest.c"
			goto __catch250_g_error;
		}
#line 670 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp44_ = bs;
#line 670 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp45_ = gxml_dom_node_get_owner_document ((GXmlDomNode*) _tmp44_);
#line 670 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp46_ = _tmp45_;
#line 670 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp47_ = gxml_dom_document_create_element (_tmp46_, "Test", &_inner_error_);
#line 670 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp43_ = _tmp47_;
#line 670 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 670 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (b2);
#line 670 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (b);
#line 670 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 670 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 6161 "GomSerializationTest.c"
			goto __catch250_g_error;
		}
#line 670 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp48_ = bs;
#line 670 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp49_ = gxml_dom_node_append_child ((GXmlDomNode*) _tmp48_, (GXmlDomNode*) _tmp43_, &_inner_error_);
#line 670 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp50_ = _tmp49_;
#line 670 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp50_);
#line 670 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 670 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp43_);
#line 670 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (b2);
#line 670 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (b);
#line 670 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 670 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 6184 "GomSerializationTest.c"
			goto __catch250_g_error;
		}
#line 671 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp51_ = bs;
#line 671 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp52_ = gxml_dom_node_get_owner_document ((GXmlDomNode*) _tmp51_);
#line 671 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp53_ = _tmp52_;
#line 671 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp54_ = gom_serialization_test_book_new_document (_tmp53_);
#line 671 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		b3 = _tmp54_;
#line 672 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp55_ = b3;
#line 672 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_set_name (_tmp55_, "Title3");
#line 673 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp56_ = bs;
#line 673 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp57_ = gom_serialization_test_book_store_get_books (_tmp56_);
#line 673 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp58_ = _tmp57_;
#line 673 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp59_ = b3;
#line 673 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_collection_append ((GXmlGomCollection*) _tmp58_, (GXmlDomElement*) _tmp59_, &_inner_error_);
#line 673 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 673 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (b3);
#line 673 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp43_);
#line 673 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (b2);
#line 673 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (b);
#line 673 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 673 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 6225 "GomSerializationTest.c"
			goto __catch250_g_error;
		}
#line 674 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp60_ = bs;
#line 674 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp61_ = gom_serialization_test_book_store_to_string (_tmp60_);
#line 674 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 674 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp61_;
#line 675 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp62_ = s;
#line 675 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp62_ != NULL, "s != null");
#line 679 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp63_ = s;
#line 679 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp64_ = string_contains (_tmp63_, "<BookStore><Book Name=\"Title1\"/><Book Name=\"Title2\"/><Test/><Book " \
"Name=\"Title3\"/></BookStore>");
#line 679 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp64_, "\"<BookStore><Book Name=\\\"Title1\\\"/><Book Name=\\\"Title2\\\"/><Test/><Book Name=\\\"Title3\\\"/></BookStore>\" in s");
#line 680 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp65_ = bs;
#line 680 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp66_ = gom_serialization_test_book_store_get_books (_tmp65_);
#line 680 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp67_ = _tmp66_;
#line 680 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp68_ = gxml_gom_hash_map_get ((GXmlGomHashMap*) _tmp67_, "Title1");
#line 680 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp69_ = _tmp68_;
#line 680 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp69_ != NULL, "bs.books.get(\"Title1\") != null");
#line 680 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp69_);
#line 681 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp70_ = bs;
#line 681 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp71_ = gom_serialization_test_book_store_get_books (_tmp70_);
#line 681 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp72_ = _tmp71_;
#line 681 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp73_ = gxml_gom_hash_map_get ((GXmlGomHashMap*) _tmp72_, "Title2");
#line 681 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp74_ = _tmp73_;
#line 681 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp74_ != NULL, "bs.books.get(\"Title2\") != null");
#line 681 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp74_);
#line 682 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp75_ = bs;
#line 682 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp76_ = gom_serialization_test_book_store_get_books (_tmp75_);
#line 682 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp77_ = _tmp76_;
#line 682 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp78_ = gxml_gom_hash_map_get ((GXmlGomHashMap*) _tmp77_, "Title3");
#line 682 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp79_ = _tmp78_;
#line 682 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp79_ != NULL, "bs.books.get(\"Title3\") != null");
#line 682 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp79_);
#line 683 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp80_ = bs;
#line 683 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp81_ = gom_serialization_test_book_store_get_books (_tmp80_);
#line 683 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp82_ = _tmp81_;
#line 683 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp83_ = gxml_gom_hash_map_get ((GXmlGomHashMap*) _tmp82_, "Title1");
#line 683 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp84_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp83_, GOM_SERIALIZATION_TEST_TYPE_BOOK) ? ((GomSerializationTestBook*) _tmp83_) : NULL;
#line 683 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (_tmp84_ == NULL) {
#line 683 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp83_);
#line 6302 "GomSerializationTest.c"
		}
#line 683 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp85_ = _tmp84_;
#line 683 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp86_ = gom_serialization_test_book_get_name (_tmp85_);
#line 683 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp87_ = _tmp86_;
#line 683 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (g_strcmp0 (_tmp87_, "Title1") == 0, "(bs.books.get(\"Title1\") as Book).name == \"Title1\"");
#line 683 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp85_);
#line 684 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp88_ = bs;
#line 684 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp89_ = gom_serialization_test_book_store_get_books (_tmp88_);
#line 684 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp90_ = _tmp89_;
#line 684 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp91_ = gxml_gom_hash_map_get ((GXmlGomHashMap*) _tmp90_, "Title2");
#line 684 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp92_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp91_, GOM_SERIALIZATION_TEST_TYPE_BOOK) ? ((GomSerializationTestBook*) _tmp91_) : NULL;
#line 684 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (_tmp92_ == NULL) {
#line 684 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp91_);
#line 6328 "GomSerializationTest.c"
		}
#line 684 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp93_ = _tmp92_;
#line 684 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp94_ = gom_serialization_test_book_get_name (_tmp93_);
#line 684 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp95_ = _tmp94_;
#line 684 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (g_strcmp0 (_tmp95_, "Title2") == 0, "(bs.books.get(\"Title2\") as Book).name == \"Title2\"");
#line 684 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp93_);
#line 685 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp96_ = bs;
#line 685 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp97_ = gom_serialization_test_book_store_get_books (_tmp96_);
#line 685 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp98_ = _tmp97_;
#line 685 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp99_ = gxml_gom_hash_map_get ((GXmlGomHashMap*) _tmp98_, "Title3");
#line 685 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp100_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp99_, GOM_SERIALIZATION_TEST_TYPE_BOOK) ? ((GomSerializationTestBook*) _tmp99_) : NULL;
#line 685 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (_tmp100_ == NULL) {
#line 685 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp99_);
#line 6354 "GomSerializationTest.c"
		}
#line 685 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp101_ = _tmp100_;
#line 685 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp102_ = gom_serialization_test_book_get_name (_tmp101_);
#line 685 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp103_ = _tmp102_;
#line 685 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (g_strcmp0 (_tmp103_, "Title3") == 0, "(bs.books.get(\"Title3\") as Book).name == \"Title3\"");
#line 685 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp101_);
#line 637 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (b3);
#line 637 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp43_);
#line 637 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (b2);
#line 637 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (b);
#line 637 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 637 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (bs);
#line 6378 "GomSerializationTest.c"
	}
	goto __finally250;
	__catch250_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp104_ = NULL;
		gchar* _tmp105_ = NULL;
		gchar* _tmp106_ = NULL;
#line 637 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		e = _inner_error_;
#line 637 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 687 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp104_ = e->message;
#line 687 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp105_ = g_strconcat ("Error: ", _tmp104_, NULL);
#line 687 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp106_ = _tmp105_;
#line 687 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:687: %s", _tmp106_);
#line 687 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp106_);
#line 688 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 637 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_error_free0 (e);
#line 6405 "GomSerializationTest.c"
	}
	__finally250:
#line 637 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 637 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 637 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 637 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return;
#line 6416 "GomSerializationTest.c"
	}
}


static void ___lambda248__gtest_func (void) {
#line 636 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	__lambda248_ ();
#line 6424 "GomSerializationTest.c"
}


static void __lambda249_ (void) {
	GError * _inner_error_ = NULL;
	{
		GomSerializationTestBookStand* bs = NULL;
		GomSerializationTestBookStand* _tmp0_ = NULL;
		GomSerializationTestBookStand* _tmp1_ = NULL;
		GomSerializationTestHashRegisters* _tmp2_ = NULL;
		GomSerializationTestHashRegisters* _tmp3_ = NULL;
		GomSerializationTestBookRegister* br = NULL;
		GomSerializationTestBookStand* _tmp4_ = NULL;
		GomSerializationTestHashRegisters* _tmp5_ = NULL;
		GomSerializationTestHashRegisters* _tmp6_ = NULL;
		GXmlGomElement* _tmp7_ = NULL;
		GomSerializationTestBookRegister* _tmp8_ = NULL;
		GomSerializationTestBook* book = NULL;
		GomSerializationTestBookRegister* _tmp9_ = NULL;
		GomSerializationTestBook* _tmp10_ = NULL;
		GomSerializationTestBookRegister* _tmp11_ = NULL;
		GomSerializationTestBook* _tmp12_ = NULL;
		GomSerializationTestBookRegister* _tmp13_ = NULL;
		GomSerializationTestBookStand* _tmp14_ = NULL;
		GomSerializationTestHashRegisters* _tmp15_ = NULL;
		GomSerializationTestHashRegisters* _tmp16_ = NULL;
		GomSerializationTestBookRegister* _tmp17_ = NULL;
		GomSerializationTestBookStand* _tmp18_ = NULL;
		GomSerializationTestHashRegisters* _tmp19_ = NULL;
		GomSerializationTestHashRegisters* _tmp20_ = NULL;
		gint _tmp21_ = 0;
		gint _tmp22_ = 0;
		GomSerializationTestBookStand* _tmp23_ = NULL;
		GomSerializationTestHashRegisters* _tmp24_ = NULL;
		GomSerializationTestHashRegisters* _tmp25_ = NULL;
		gboolean _tmp26_ = FALSE;
		GomSerializationTestBookRegister* b1 = NULL;
		GomSerializationTestBookStand* _tmp27_ = NULL;
		GomSerializationTestHashRegisters* _tmp28_ = NULL;
		GomSerializationTestHashRegisters* _tmp29_ = NULL;
		GXmlDomElement* _tmp30_ = NULL;
		GomSerializationTestBookRegister* _tmp31_ = NULL;
		GomSerializationTestBookRegister* _tmp32_ = NULL;
		GomSerializationTestBookRegister* _tmp33_ = NULL;
		gint _tmp34_ = 0;
		gint _tmp35_ = 0;
		GomSerializationTestBookRegister* _tmp36_ = NULL;
		GomSerializationTestBook* _tmp37_ = NULL;
		GomSerializationTestBook* _tmp38_ = NULL;
		GomSerializationTestBookRegister* _tmp39_ = NULL;
		GomSerializationTestBook* _tmp40_ = NULL;
		GomSerializationTestBook* _tmp41_ = NULL;
		const gchar* _tmp42_ = NULL;
		const gchar* _tmp43_ = NULL;
		GomSerializationTestBookRegister* _tmp44_ = NULL;
		GXmlDomNodeList* _tmp45_ = NULL;
		GXmlDomNodeList* _tmp46_ = NULL;
		GXmlDomNodeList* _tmp47_ = NULL;
		gint _tmp48_ = 0;
		gint _tmp49_ = 0;
		GomSerializationTestBook* book2 = NULL;
		GomSerializationTestBookRegister* _tmp50_ = NULL;
		GomSerializationTestBook* _tmp51_ = NULL;
		GomSerializationTestBookRegister* _tmp52_ = NULL;
		GomSerializationTestBook* _tmp53_ = NULL;
		GXmlDomNode* _tmp54_ = NULL;
		GXmlDomNode* _tmp55_ = NULL;
		GomSerializationTestBookRegister* _tmp56_ = NULL;
		GXmlDomNodeList* _tmp57_ = NULL;
		GXmlDomNodeList* _tmp58_ = NULL;
		GXmlDomNodeList* _tmp59_ = NULL;
		gint _tmp60_ = 0;
		gint _tmp61_ = 0;
		GomSerializationTestBookRegister* _tmp62_ = NULL;
		GomSerializationTestBook* _tmp63_ = NULL;
		GomSerializationTestBookRegister* _tmp64_ = NULL;
		GXmlDomNodeList* _tmp65_ = NULL;
		GXmlDomNodeList* _tmp66_ = NULL;
		GXmlDomNodeList* _tmp67_ = NULL;
		gint _tmp68_ = 0;
		gint _tmp69_ = 0;
#line 693 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp0_ = gom_serialization_test_book_stand_new ();
#line 693 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		bs = _tmp0_;
#line 694 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp1_ = bs;
#line 694 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = gom_serialization_test_book_stand_get_hashmap_registers (_tmp1_);
#line 694 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = _tmp2_;
#line 694 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp3_ != NULL, "bs.hashmap_registers != null");
#line 695 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp4_ = bs;
#line 695 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp5_ = gom_serialization_test_book_stand_get_hashmap_registers (_tmp4_);
#line 695 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp6_ = _tmp5_;
#line 695 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp7_ = gxml_gom_collection_create_item ((GXmlGomCollection*) _tmp6_);
#line 695 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp8_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp7_, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER) ? ((GomSerializationTestBookRegister*) _tmp7_) : NULL;
#line 695 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (_tmp8_ == NULL) {
#line 695 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp7_);
#line 6532 "GomSerializationTest.c"
		}
#line 695 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		br = _tmp8_;
#line 696 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp9_ = br;
#line 696 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp10_ = gom_serialization_test_book_register_create_book (_tmp9_, "Book1");
#line 696 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		book = _tmp10_;
#line 697 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp11_ = br;
#line 697 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp12_ = book;
#line 697 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_register_set_book (_tmp11_, _tmp12_);
#line 698 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp13_ = br;
#line 698 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_register_set_year (_tmp13_, 2017);
#line 699 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp14_ = bs;
#line 699 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp15_ = gom_serialization_test_book_stand_get_hashmap_registers (_tmp14_);
#line 699 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp16_ = _tmp15_;
#line 699 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp17_ = br;
#line 699 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_collection_append ((GXmlGomCollection*) _tmp16_, (GXmlDomElement*) _tmp17_, &_inner_error_);
#line 699 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 699 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (book);
#line 699 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br);
#line 699 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 6570 "GomSerializationTest.c"
			goto __catch252_g_error;
		}
#line 700 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp18_ = bs;
#line 700 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp19_ = gom_serialization_test_book_stand_get_hashmap_registers (_tmp18_);
#line 700 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp20_ = _tmp19_;
#line 700 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp21_ = gxml_gom_collection_get_length ((GXmlGomCollection*) _tmp20_);
#line 700 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp22_ = _tmp21_;
#line 700 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp22_ == 1, "bs.hashmap_registers.length == 1");
#line 701 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp23_ = bs;
#line 701 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp24_ = gom_serialization_test_book_stand_get_hashmap_registers (_tmp23_);
#line 701 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp25_ = _tmp24_;
#line 701 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp26_ = gxml_gom_hash_map_has_key ((GXmlGomHashMap*) _tmp25_, "2017-Book1");
#line 701 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp26_, "bs.hashmap_registers.has_key (\"2017-Book1\")");
#line 702 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp27_ = bs;
#line 702 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp28_ = gom_serialization_test_book_stand_get_hashmap_registers (_tmp27_);
#line 702 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp29_ = _tmp28_;
#line 702 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp30_ = gxml_gom_hash_map_get ((GXmlGomHashMap*) _tmp29_, "2017-Book1");
#line 702 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp31_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp30_, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER) ? ((GomSerializationTestBookRegister*) _tmp30_) : NULL;
#line 702 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (_tmp31_ == NULL) {
#line 702 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp30_);
#line 6609 "GomSerializationTest.c"
		}
#line 702 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		b1 = _tmp31_;
#line 703 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp32_ = b1;
#line 703 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp32_ != NULL, "b1 != null");
#line 704 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp33_ = b1;
#line 704 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp34_ = gom_serialization_test_book_register_get_year (_tmp33_);
#line 704 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp35_ = _tmp34_;
#line 704 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp35_ == 2017, "b1.year == 2017");
#line 705 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp36_ = b1;
#line 705 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp37_ = gom_serialization_test_book_register_get_book (_tmp36_);
#line 705 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp38_ = _tmp37_;
#line 705 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp38_ != NULL, "b1.book != null");
#line 706 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp39_ = b1;
#line 706 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp40_ = gom_serialization_test_book_register_get_book (_tmp39_);
#line 706 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp41_ = _tmp40_;
#line 706 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp42_ = gom_serialization_test_book_get_name (_tmp41_);
#line 706 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp43_ = _tmp42_;
#line 706 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (g_strcmp0 (_tmp43_, "Book1") == 0, "b1.book.name == \"Book1\"");
#line 707 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp44_ = b1;
#line 707 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp45_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp44_);
#line 707 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp46_ = _tmp45_;
#line 707 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp47_ = _tmp46_;
#line 707 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp48_ = gxml_dom_node_list_get_length (_tmp47_);
#line 707 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp49_ = _tmp48_;
#line 707 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp49_ == 1, "b1.child_nodes.length == 1");
#line 707 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp47_);
#line 708 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp50_ = b1;
#line 708 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp51_ = gom_serialization_test_book_register_create_book (_tmp50_, "Book2");
#line 708 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		book2 = _tmp51_;
#line 709 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp52_ = b1;
#line 709 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp53_ = book2;
#line 709 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp54_ = gxml_dom_node_append_child ((GXmlDomNode*) _tmp52_, (GXmlDomNode*) _tmp53_, &_inner_error_);
#line 709 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp55_ = _tmp54_;
#line 709 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp55_);
#line 709 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 709 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (book2);
#line 709 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (b1);
#line 709 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (book);
#line 709 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br);
#line 709 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 6689 "GomSerializationTest.c"
			goto __catch252_g_error;
		}
#line 710 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp56_ = b1;
#line 710 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp57_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp56_);
#line 710 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp58_ = _tmp57_;
#line 710 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp59_ = _tmp58_;
#line 710 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp60_ = gxml_dom_node_list_get_length (_tmp59_);
#line 710 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp61_ = _tmp60_;
#line 710 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp61_ == 2, "b1.child_nodes.length == 2");
#line 710 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp59_);
#line 711 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp62_ = b1;
#line 711 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp63_ = book2;
#line 711 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_register_set_book (_tmp62_, _tmp63_);
#line 712 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp64_ = b1;
#line 712 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp65_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp64_);
#line 712 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp66_ = _tmp65_;
#line 712 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp67_ = _tmp66_;
#line 712 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp68_ = gxml_dom_node_list_get_length (_tmp67_);
#line 712 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp69_ = _tmp68_;
#line 712 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp69_ == 1, "b1.child_nodes.length == 1");
#line 712 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp67_);
#line 692 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (book2);
#line 692 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (b1);
#line 692 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (book);
#line 692 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (br);
#line 692 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (bs);
#line 6740 "GomSerializationTest.c"
	}
	goto __finally252;
	__catch252_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp70_ = NULL;
		gchar* _tmp71_ = NULL;
		gchar* _tmp72_ = NULL;
#line 692 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		e = _inner_error_;
#line 692 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 714 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp70_ = e->message;
#line 714 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp71_ = g_strconcat ("Error: ", _tmp70_, NULL);
#line 714 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp72_ = _tmp71_;
#line 714 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:714: %s", _tmp72_);
#line 714 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp72_);
#line 715 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 692 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_error_free0 (e);
#line 6767 "GomSerializationTest.c"
	}
	__finally252:
#line 692 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 692 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 692 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 692 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return;
#line 6778 "GomSerializationTest.c"
	}
}


static void ___lambda249__gtest_func (void) {
#line 691 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	__lambda249_ ();
#line 6786 "GomSerializationTest.c"
}


static void __lambda250_ (void) {
	GError * _inner_error_ = NULL;
	{
		GomSerializationTestBookStand* bs = NULL;
		GomSerializationTestBookStand* _tmp0_ = NULL;
		GomSerializationTestHashPairRegisters* _tmp1_ = NULL;
		GomSerializationTestHashPairRegisters* _tmp2_ = NULL;
		GomSerializationTestBookRegister* br = NULL;
		GomSerializationTestHashPairRegisters* _tmp3_ = NULL;
		GomSerializationTestHashPairRegisters* _tmp4_ = NULL;
		GXmlGomElement* _tmp5_ = NULL;
		GomSerializationTestBookRegister* _tmp6_ = NULL;
		GomSerializationTestBook* book = NULL;
		GomSerializationTestBook* _tmp7_ = NULL;
		const gchar* _tmp8_ = NULL;
		const gchar* _tmp9_ = NULL;
		GomSerializationTestHashPairRegisters* _tmp10_ = NULL;
		GomSerializationTestHashPairRegisters* _tmp11_ = NULL;
		gint _tmp12_ = 0;
		gint _tmp13_ = 0;
		GomSerializationTestHashPairRegisters* _tmp14_ = NULL;
		GomSerializationTestHashPairRegisters* _tmp15_ = NULL;
		GomSerializationTestHashPairRegisters* _tmp16_ = NULL;
		GomSerializationTestHashPairRegisters* _tmp17_ = NULL;
		gint _tmp18_ = 0;
		gint _tmp19_ = 0;
		GomSerializationTestBookRegister* b1 = NULL;
		GomSerializationTestHashPairRegisters* _tmp20_ = NULL;
		GomSerializationTestHashPairRegisters* _tmp21_ = NULL;
		GXmlDomElement* _tmp22_ = NULL;
		GomSerializationTestBookRegister* _tmp23_ = NULL;
		gint _tmp24_ = 0;
		gint _tmp25_ = 0;
		GomSerializationTestBook* _tmp26_ = NULL;
		GomSerializationTestBook* _tmp27_ = NULL;
		GomSerializationTestBook* _tmp28_ = NULL;
		GomSerializationTestBook* _tmp29_ = NULL;
		const gchar* _tmp30_ = NULL;
		const gchar* _tmp31_ = NULL;
#line 720 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp0_ = gom_serialization_test_book_stand_new ();
#line 720 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		bs = _tmp0_;
#line 721 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp1_ = gom_serialization_test_book_stand_get_hashpair_registers (bs);
#line 721 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = _tmp1_;
#line 721 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp2_ != NULL, "bs.hashpair_registers != null");
#line 722 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = gom_serialization_test_book_stand_get_hashpair_registers (bs);
#line 722 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp4_ = _tmp3_;
#line 722 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp5_ = gxml_gom_collection_create_item ((GXmlGomCollection*) _tmp4_);
#line 722 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp6_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp5_, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER) ? ((GomSerializationTestBookRegister*) _tmp5_) : NULL;
#line 722 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (_tmp6_ == NULL) {
#line 722 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp5_);
#line 6851 "GomSerializationTest.c"
		}
#line 722 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		br = _tmp6_;
#line 723 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp7_ = gom_serialization_test_book_register_create_book (br, "Book1");
#line 723 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		book = _tmp7_;
#line 724 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp8_ = gom_serialization_test_book_get_name (book);
#line 724 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp9_ = _tmp8_;
#line 724 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (g_strcmp0 (_tmp9_, "Book1") == 0, "book.name == \"Book1\"");
#line 725 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_register_set_book (br, book);
#line 726 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_register_set_year (br, 2017);
#line 727 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp10_ = gom_serialization_test_book_stand_get_hashpair_registers (bs);
#line 727 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp11_ = _tmp10_;
#line 727 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp12_ = gxml_gom_collection_get_length ((GXmlGomCollection*) _tmp11_);
#line 727 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp13_ = _tmp12_;
#line 727 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp13_ == 0, "bs.hashpair_registers.length == 0");
#line 728 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp14_ = gom_serialization_test_book_stand_get_hashpair_registers (bs);
#line 728 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp15_ = _tmp14_;
#line 728 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_collection_append ((GXmlGomCollection*) _tmp15_, (GXmlDomElement*) br, &_inner_error_);
#line 728 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 728 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (book);
#line 728 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (br);
#line 728 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 6893 "GomSerializationTest.c"
			goto __catch253_g_error;
		}
#line 729 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp16_ = gom_serialization_test_book_stand_get_hashpair_registers (bs);
#line 729 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp17_ = _tmp16_;
#line 729 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp18_ = gxml_gom_collection_get_length ((GXmlGomCollection*) _tmp17_);
#line 729 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp19_ = _tmp18_;
#line 729 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp19_ == 1, "bs.hashpair_registers.length == 1");
#line 730 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp20_ = gom_serialization_test_book_stand_get_hashpair_registers (bs);
#line 730 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp21_ = _tmp20_;
#line 730 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp22_ = gxml_gom_hash_paired_map_get ((GXmlGomHashPairedMap*) _tmp21_, "2017", "Book1");
#line 730 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp23_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp22_, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER) ? ((GomSerializationTestBookRegister*) _tmp22_) : NULL;
#line 730 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (_tmp23_ == NULL) {
#line 730 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp22_);
#line 6918 "GomSerializationTest.c"
		}
#line 730 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		b1 = _tmp23_;
#line 731 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (b1 != NULL, "b1 != null");
#line 732 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp24_ = gom_serialization_test_book_register_get_year (b1);
#line 732 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp25_ = _tmp24_;
#line 732 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp25_ == 2017, "b1.year == 2017");
#line 733 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp26_ = gom_serialization_test_book_register_get_book (b1);
#line 733 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp27_ = _tmp26_;
#line 733 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp27_ != NULL, "b1.book != null");
#line 734 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp28_ = gom_serialization_test_book_register_get_book (b1);
#line 734 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp29_ = _tmp28_;
#line 734 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp30_ = gom_serialization_test_book_get_name (_tmp29_);
#line 734 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp31_ = _tmp30_;
#line 734 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (g_strcmp0 (_tmp31_, "Book1") == 0, "b1.book.name == \"Book1\"");
#line 719 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (b1);
#line 719 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (book);
#line 719 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (br);
#line 719 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (bs);
#line 6954 "GomSerializationTest.c"
	}
	goto __finally253;
	__catch253_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp32_ = NULL;
		gchar* _tmp33_ = NULL;
		gchar* _tmp34_ = NULL;
#line 719 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		e = _inner_error_;
#line 719 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 736 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp32_ = e->message;
#line 736 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp33_ = g_strconcat ("Error: ", _tmp32_, NULL);
#line 736 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp34_ = _tmp33_;
#line 736 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:736: %s", _tmp34_);
#line 736 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp34_);
#line 737 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 719 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_error_free0 (e);
#line 6981 "GomSerializationTest.c"
	}
	__finally253:
#line 719 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 719 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 719 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 719 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return;
#line 6992 "GomSerializationTest.c"
	}
}


static void ___lambda250__gtest_func (void) {
#line 718 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	__lambda250_ ();
#line 7000 "GomSerializationTest.c"
}


static void __lambda251_ (void) {
	GomSerializationTestMotor* m = NULL;
	GomSerializationTestMotor* _tmp0_ = NULL;
	gchar* s = NULL;
	gchar* _tmp1_ = NULL;
	const gchar* _tmp2_ = NULL;
	const gchar* _tmp3_ = NULL;
	gboolean _tmp4_ = FALSE;
	GomSerializationTestMotorOn* _tmp5_ = NULL;
	GomSerializationTestMotorOn* _tmp6_ = NULL;
	gchar* _tmp7_ = NULL;
	const gchar* _tmp8_ = NULL;
	const gchar* _tmp9_ = NULL;
	gboolean _tmp10_ = FALSE;
	GomSerializationTestMotorTorque* _tmp11_ = NULL;
	GomSerializationTestMotorTorque* _tmp12_ = NULL;
	gchar* _tmp13_ = NULL;
	const gchar* _tmp14_ = NULL;
	const gchar* _tmp15_ = NULL;
	gboolean _tmp16_ = FALSE;
	GomSerializationTestMotorSpeed* _tmp17_ = NULL;
	GomSerializationTestMotorSpeed* _tmp18_ = NULL;
	GomSerializationTestMotorSpeed* _tmp19_ = NULL;
	GomSerializationTestMotorSpeed* _tmp20_ = NULL;
	gchar* _tmp21_ = NULL;
	const gchar* _tmp22_ = NULL;
	const gchar* _tmp23_ = NULL;
	gboolean _tmp24_ = FALSE;
	GomSerializationTestMotorSpeed* _tmp25_ = NULL;
	GomSerializationTestMotorSpeed* _tmp26_ = NULL;
	GomSerializationTestMotorSpeed* _tmp27_ = NULL;
	GomSerializationTestMotorSpeed* _tmp28_ = NULL;
	GomSerializationTestMotorSpeed* _tmp29_ = NULL;
	GomSerializationTestMotorSpeed* _tmp30_ = NULL;
	gdouble _tmp31_ = 0.0;
	GomSerializationTestMotorSpeed* _tmp32_ = NULL;
	GomSerializationTestMotorSpeed* _tmp33_ = NULL;
	gchar* _tmp34_ = NULL;
	gchar* _tmp35_ = NULL;
	gchar* _tmp36_ = NULL;
	GomSerializationTestMotorSpeed* _tmp37_ = NULL;
	GomSerializationTestMotorSpeed* _tmp38_ = NULL;
	gchar* _tmp39_ = NULL;
	gchar* _tmp40_ = NULL;
	gchar* _tmp41_ = NULL;
	gchar* _tmp42_ = NULL;
	gchar* _tmp43_ = NULL;
	GomSerializationTestMotorTensionType* _tmp44_ = NULL;
	GomSerializationTestMotorTensionType* _tmp45_ = NULL;
	GomSerializationTestMotorTensionType* _tmp46_ = NULL;
	GomSerializationTestMotorTensionType* _tmp47_ = NULL;
	gchar* _tmp48_ = NULL;
	const gchar* _tmp49_ = NULL;
	const gchar* _tmp50_ = NULL;
	gboolean _tmp51_ = FALSE;
	GomSerializationTestMotorTensionType* _tmp52_ = NULL;
	GomSerializationTestMotorTensionType* _tmp53_ = NULL;
	GomSerializationTestMotorTensionType* _tmp54_ = NULL;
	GomSerializationTestMotorTensionType* _tmp55_ = NULL;
	gchar* _tmp56_ = NULL;
	gchar* _tmp57_ = NULL;
	gchar* _tmp58_ = NULL;
	GomSerializationTestMotorTension* _tmp59_ = NULL;
	GomSerializationTestMotorTension* _tmp60_ = NULL;
	gchar* _tmp61_ = NULL;
	const gchar* _tmp62_ = NULL;
	const gchar* _tmp63_ = NULL;
	gboolean _tmp64_ = FALSE;
	GomSerializationTestMotorOn* _tmp65_ = NULL;
	GomSerializationTestMotorOn* _tmp66_ = NULL;
	GomSerializationTestMotorTorque* _tmp67_ = NULL;
	GomSerializationTestMotorTorque* _tmp68_ = NULL;
	GomSerializationTestMotorSpeed* _tmp69_ = NULL;
	GomSerializationTestMotorSpeed* _tmp70_ = NULL;
	GomSerializationTestMotorTensionType* _tmp71_ = NULL;
	GomSerializationTestMotorTensionType* _tmp72_ = NULL;
	GomSerializationTestMotorTension* _tmp73_ = NULL;
	GomSerializationTestMotorTension* _tmp74_ = NULL;
	gchar* _tmp75_ = NULL;
	const gchar* _tmp76_ = NULL;
	const gchar* _tmp77_ = NULL;
	gboolean _tmp78_ = FALSE;
	GomSerializationTestMotorModel* _tmp79_ = NULL;
	GomSerializationTestMotorModel* _tmp80_ = NULL;
	GomSerializationTestMotorModel* _tmp81_ = NULL;
	GomSerializationTestMotorModel* _tmp82_ = NULL;
	GomSerializationTestMotorModel* _tmp83_ = NULL;
	GomSerializationTestMotorModel* _tmp84_ = NULL;
	gchar* _tmp85_ = NULL;
	gchar* _tmp86_ = NULL;
	gchar* _tmp87_ = NULL;
	GomSerializationTestMotorModel* _tmp88_ = NULL;
	GomSerializationTestMotorModel* _tmp89_ = NULL;
	gchar* _tmp90_ = NULL;
	gchar* _tmp91_ = NULL;
	gchar* _tmp92_ = NULL;
	GomSerializationTestMotorModel* _tmp93_ = NULL;
	GomSerializationTestMotorModel* _tmp94_ = NULL;
	GomSerializationTestMotorModel* _tmp95_ = NULL;
	GomSerializationTestMotorModel* _tmp96_ = NULL;
	gchar* _tmp97_ = NULL;
	gchar* _tmp98_ = NULL;
	gchar* _tmp99_ = NULL;
	gchar* _tmp100_ = NULL;
	const gchar* _tmp101_ = NULL;
	const gchar* _tmp102_ = NULL;
	gboolean _tmp103_ = FALSE;
	GomSerializationTestMotorModel* _tmp104_ = NULL;
	GomSerializationTestMotorModel* _tmp105_ = NULL;
	gboolean _tmp106_ = FALSE;
	GomSerializationTestMotorModel* _tmp107_ = NULL;
	GomSerializationTestMotorModel* _tmp108_ = NULL;
	gboolean _tmp109_ = FALSE;
	GomSerializationTestMotorModel* _tmp110_ = NULL;
	GomSerializationTestMotorModel* _tmp111_ = NULL;
	GomSerializationTestMotorModel* _tmp112_ = NULL;
	GomSerializationTestMotorModel* _tmp113_ = NULL;
	gboolean _tmp114_ = FALSE;
#line 741 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = gom_serialization_test_motor_new ();
#line 741 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	m = _tmp0_;
#line 742 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = gom_serialization_test_motor_to_string (m);
#line 742 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp1_;
#line 743 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp2_ = s;
#line 743 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp2_ != NULL, "s != null");
#line 747 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp3_ = s;
#line 747 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp4_ = string_contains (_tmp3_, "<Motor/>");
#line 747 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp4_, "\"<Motor/>\" in s");
#line 748 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp5_ = gom_serialization_test_motor_on_new ();
#line 748 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp6_ = _tmp5_;
#line 748 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_motor_set_is_on (m, _tmp6_);
#line 748 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (_tmp6_);
#line 749 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp7_ = gom_serialization_test_motor_to_string (m);
#line 749 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (s);
#line 749 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp7_;
#line 750 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp8_ = s;
#line 750 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp8_ != NULL, "s != null");
#line 754 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp9_ = s;
#line 754 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp10_ = string_contains (_tmp9_, "<Motor On=\"false\"/>");
#line 754 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp10_, "\"<Motor On=\\\"false\\\"/>\" in s");
#line 755 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp11_ = gom_serialization_test_motor_torque_new ();
#line 755 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp12_ = _tmp11_;
#line 755 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_motor_set_torque (m, _tmp12_);
#line 755 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (_tmp12_);
#line 756 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp13_ = gom_serialization_test_motor_to_string (m);
#line 756 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (s);
#line 756 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp13_;
#line 757 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp14_ = s;
#line 757 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp14_ != NULL, "s != null");
#line 761 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp15_ = s;
#line 761 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp16_ = string_contains (_tmp15_, "<Motor On=\"false\" Torque=\"0.0000\"/>");
#line 761 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp16_, "\"<Motor On=\\\"false\\\" Torque=\\\"0.0000\\\"/>\" in s");
#line 762 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp17_ = gom_serialization_test_motor_speed_new ();
#line 762 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp18_ = _tmp17_;
#line 762 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_motor_set_speed (m, _tmp18_);
#line 762 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (_tmp18_);
#line 763 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp19_ = gom_serialization_test_motor_get_speed (m);
#line 763 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp20_ = _tmp19_;
#line 763 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gxml_gom_double_set_double ((GXmlGomDouble*) _tmp20_, 1.0);
#line 764 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp21_ = gom_serialization_test_motor_to_string (m);
#line 764 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (s);
#line 764 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp21_;
#line 765 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp22_ = s;
#line 765 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp22_ != NULL, "s != null");
#line 769 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp23_ = s;
#line 769 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp24_ = string_contains (_tmp23_, "<Motor On=\"false\" Torque=\"0.0000\" Speed=\"1.0000\"/>");
#line 769 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp24_, "\"<Motor On=\\\"false\\\" Torque=\\\"0.0000\\\" Speed=\\\"1.0000\\\"/>\" in s");
#line 770 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp25_ = gom_serialization_test_motor_get_speed (m);
#line 770 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp26_ = _tmp25_;
#line 770 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp26_ != NULL, "m.speed != null");
#line 771 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (m, GXML_TYPE_GOM_OBJECT), "m is GomObject");
#line 772 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp27_ = gom_serialization_test_motor_get_speed (m);
#line 772 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp28_ = _tmp27_;
#line 772 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp28_, GXML_TYPE_GOM_PROPERTY), "m.speed is GomProperty");
#line 773 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp29_ = gom_serialization_test_motor_get_speed (m);
#line 773 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp30_ = _tmp29_;
#line 773 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp31_ = gxml_gom_double_get_double ((GXmlGomDouble*) _tmp30_);
#line 773 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp31_ == 1.0, "m.speed.get_double () == 1.0");
#line 774 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp32_ = gom_serialization_test_motor_get_speed (m);
#line 774 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp33_ = _tmp32_;
#line 774 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp34_ = gxml_gom_base_property_get_value ((GXmlGomBaseProperty*) _tmp33_);
#line 774 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp35_ = _tmp34_;
#line 774 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp36_ = _tmp35_;
#line 774 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp36_ != NULL, "m.speed.value != null");
#line 774 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp36_);
#line 775 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp37_ = gom_serialization_test_motor_get_speed (m);
#line 775 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp38_ = _tmp37_;
#line 775 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp39_ = gxml_gom_base_property_get_value ((GXmlGomBaseProperty*) _tmp38_);
#line 775 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp40_ = _tmp39_;
#line 775 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp41_ = _tmp40_;
#line 775 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (g_strcmp0 (_tmp41_, "1.0000") == 0, "m.speed.value == \"1.0000\"");
#line 775 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp41_);
#line 779 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp42_ = gxml_dom_element_get_attribute ((GXmlDomElement*) m, "speed");
#line 779 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp43_ = _tmp42_;
#line 779 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp43_ != NULL, "m.get_attribute (\"speed\") != null");
#line 779 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp43_);
#line 780 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp44_ = gom_serialization_test_motor_get_tension_type (m);
#line 780 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp45_ = _tmp44_;
#line 780 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp45_ == NULL, "m.tension_type == null");
#line 781 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp46_ = gom_serialization_test_motor_tension_type_new ();
#line 781 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp47_ = _tmp46_;
#line 781 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_motor_set_tension_type (m, _tmp47_);
#line 781 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (_tmp47_);
#line 782 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp48_ = gom_serialization_test_motor_to_string (m);
#line 782 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (s);
#line 782 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp48_;
#line 783 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp49_ = s;
#line 783 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp49_ != NULL, "s != null");
#line 787 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp50_ = s;
#line 787 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp51_ = string_contains (_tmp50_, "<Motor On=\"false\" Torque=\"0.0000\" Speed=\"1.0000\" TensionType=\"a" \
"c\"/>");
#line 787 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp51_, "\"<Motor On=\\\"false\\\" Torque=\\\"0.0000\\\" Speed=\\\"1.0000\\\" TensionType=\\\"ac\\\"/>\" in s");
#line 788 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp52_ = gom_serialization_test_motor_get_tension_type (m);
#line 788 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp53_ = _tmp52_;
#line 788 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp53_ != NULL, "m.tension_type != null");
#line 789 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp54_ = gom_serialization_test_motor_get_tension_type (m);
#line 789 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp55_ = _tmp54_;
#line 789 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp56_ = gxml_gom_base_property_get_value ((GXmlGomBaseProperty*) _tmp55_);
#line 789 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp57_ = _tmp56_;
#line 789 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp58_ = _tmp57_;
#line 789 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (g_strcmp0 (_tmp58_, "ac") == 0, "m.tension_type.value == \"ac\"");
#line 789 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp58_);
#line 790 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp59_ = gom_serialization_test_motor_tension_new ();
#line 790 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp60_ = _tmp59_;
#line 790 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_motor_set_tension (m, _tmp60_);
#line 790 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (_tmp60_);
#line 791 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp61_ = gom_serialization_test_motor_to_string (m);
#line 791 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (s);
#line 791 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp61_;
#line 792 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp62_ = s;
#line 792 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp62_ != NULL, "s != null");
#line 796 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp63_ = s;
#line 796 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp64_ = string_contains (_tmp63_, "<Motor On=\"false\" Torque=\"0.0000\" Speed=\"1.0000\" TensionType=\"a" \
"c\" Tension=\"0\"/>");
#line 796 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp64_, "\"<Motor On=\\\"false\\\" Torque=\\\"0.0000\\\" Speed=\\\"1.0000\\\" TensionType=\\\"ac\\\" Tension=\\\"0\\\"/>\" in s");
#line 797 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp65_ = gom_serialization_test_motor_get_is_on (m);
#line 797 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp66_ = _tmp65_;
#line 797 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gxml_gom_boolean_set_boolean ((GXmlGomBoolean*) _tmp66_, TRUE);
#line 798 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp67_ = gom_serialization_test_motor_get_torque (m);
#line 798 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp68_ = _tmp67_;
#line 798 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gxml_gom_double_set_double ((GXmlGomDouble*) _tmp68_, 3.1416);
#line 799 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp69_ = gom_serialization_test_motor_get_speed (m);
#line 799 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp70_ = _tmp69_;
#line 799 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gxml_gom_float_set_float ((GXmlGomFloat*) _tmp70_, (gfloat) 3600.1011);
#line 800 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp71_ = gom_serialization_test_motor_get_tension_type (m);
#line 800 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp72_ = _tmp71_;
#line 800 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gxml_gom_enum_set_enum ((GXmlGomEnum*) _tmp72_, (gint) GOM_SERIALIZATION_TEST_MOTOR_TENSION_TYPE_ENUM_DC);
#line 801 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp73_ = gom_serialization_test_motor_get_tension (m);
#line 801 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp74_ = _tmp73_;
#line 801 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gxml_gom_int_set_integer ((GXmlGomInt*) _tmp74_, 125);
#line 802 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp75_ = gom_serialization_test_motor_to_string (m);
#line 802 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (s);
#line 802 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp75_;
#line 803 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp76_ = s;
#line 803 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp76_ != NULL, "s != null");
#line 807 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp77_ = s;
#line 807 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp78_ = string_contains (_tmp77_, "<Motor On=\"true\" Torque=\"3.1416\" Speed=\"3600.1011\" TensionType=\"" \
"dc\" Tension=\"125\"/>");
#line 807 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp78_, "\"<Motor On=\\\"true\\\" Torque=\\\"3.1416\\\" Speed=\\\"3600.1011\\\" TensionType=\\\"dc\\\" Tension=\\\"125\\\"/>\" in s");
#line 808 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp79_ = gom_serialization_test_motor_model_new ();
#line 808 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp80_ = _tmp79_;
#line 808 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_motor_set_model (m, _tmp80_);
#line 808 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (_tmp80_);
#line 809 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp81_ = gom_serialization_test_motor_get_model (m);
#line 809 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp82_ = _tmp81_;
#line 809 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp82_ != NULL, "m.model != null");
#line 810 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp83_ = gom_serialization_test_motor_get_model (m);
#line 810 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp84_ = _tmp83_;
#line 810 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp85_ = gxml_gom_base_property_get_value ((GXmlGomBaseProperty*) _tmp84_);
#line 810 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp86_ = _tmp85_;
#line 810 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp87_ = _tmp86_;
#line 810 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp87_ != NULL, "m.model.value != null");
#line 810 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp87_);
#line 811 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp88_ = gom_serialization_test_motor_get_model (m);
#line 811 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp89_ = _tmp88_;
#line 811 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp90_ = gxml_gom_base_property_get_value ((GXmlGomBaseProperty*) _tmp89_);
#line 811 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp91_ = _tmp90_;
#line 811 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp92_ = _tmp91_;
#line 811 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (g_strcmp0 (_tmp92_, "") == 0, "m.model.value == \"\"");
#line 811 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp92_);
#line 812 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp93_ = gom_serialization_test_motor_get_model (m);
#line 812 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp94_ = _tmp93_;
#line 812 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gxml_gom_property_set_value ((GXmlGomBaseProperty*) _tmp94_, "Model3");
#line 813 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp95_ = gom_serialization_test_motor_get_model (m);
#line 813 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp96_ = _tmp95_;
#line 813 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp97_ = gxml_gom_base_property_get_value ((GXmlGomBaseProperty*) _tmp96_);
#line 813 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp98_ = _tmp97_;
#line 813 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp99_ = _tmp98_;
#line 813 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (g_strcmp0 (_tmp99_, "Model3") == 0, "m.model.value == \"Model3\"");
#line 813 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp99_);
#line 814 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp100_ = gom_serialization_test_motor_to_string (m);
#line 814 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (s);
#line 814 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp100_;
#line 815 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp101_ = s;
#line 815 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp101_ != NULL, "s != null");
#line 819 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp102_ = s;
#line 819 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp103_ = string_contains (_tmp102_, "<Motor On=\"true\" Torque=\"3.1416\" Speed=\"3600.1011\" TensionType=\"" \
"dc\" Tension=\"125\" Model=\"Model3\"/>");
#line 819 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp103_, "\"<Motor On=\\\"true\\\" Torque=\\\"3.1416\\\" Speed=\\\"3600.1011\\\" TensionType=\\\"dc\\\" Tension=\\\"125\\\" Model=\\\"Model3\\\"/>\" in s");
#line 820 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp104_ = gom_serialization_test_motor_get_model (m);
#line 820 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp105_ = _tmp104_;
#line 820 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp106_ = gxml_gom_array_string_is_valid_value ((GXmlGomArrayString*) _tmp105_);
#line 820 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (!_tmp106_, "!m.model.is_valid_value ()");
#line 821 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp107_ = gom_serialization_test_motor_get_model (m);
#line 821 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp108_ = _tmp107_;
#line 821 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp109_ = gxml_gom_array_string_search ((GXmlGomArrayString*) _tmp108_, "MODEL1");
#line 821 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp109_, "m.model.search (\"MODEL1\")");
#line 822 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp110_ = gom_serialization_test_motor_get_model (m);
#line 822 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp111_ = _tmp110_;
#line 822 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gxml_gom_property_set_value ((GXmlGomBaseProperty*) _tmp111_, "MODEL1");
#line 823 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp112_ = gom_serialization_test_motor_get_model (m);
#line 823 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp113_ = _tmp112_;
#line 823 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp114_ = gxml_gom_array_string_is_valid_value ((GXmlGomArrayString*) _tmp113_);
#line 823 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_vala_assert (_tmp114_, "m.model.is_valid_value ()");
#line 740 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (s);
#line 740 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (m);
#line 7508 "GomSerializationTest.c"
}


static void ___lambda251__gtest_func (void) {
#line 740 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	__lambda251_ ();
#line 7515 "GomSerializationTest.c"
}


static void __lambda252_ (void) {
	GError * _inner_error_ = NULL;
	{
		GomSerializationTestBook* b = NULL;
		GomSerializationTestBook* _tmp0_ = NULL;
		GXmlXParser* parser = NULL;
		GomSerializationTestBook* _tmp1_ = NULL;
		GXmlXParser* _tmp2_ = NULL;
		GXmlXParser* _tmp3_ = NULL;
		gchar* s = NULL;
		GXmlXParser* _tmp4_ = NULL;
		gchar* _tmp5_ = NULL;
		const gchar* _tmp6_ = NULL;
		GomSerializationTestBook* _tmp7_ = NULL;
		GomSerializationTestBook* _tmp8_ = NULL;
		GXmlDomNodeList* _tmp9_ = NULL;
		GXmlDomNodeList* _tmp10_ = NULL;
		GXmlDomNodeList* _tmp11_ = NULL;
		GomSerializationTestBook* _tmp12_ = NULL;
		GXmlDomNodeList* _tmp13_ = NULL;
		GXmlDomNodeList* _tmp14_ = NULL;
		GXmlDomNodeList* _tmp15_ = NULL;
		gint _tmp16_ = 0;
		gint _tmp17_ = 0;
		GomSerializationTestBook* _tmp18_ = NULL;
		GXmlDomNamedNodeMap* _tmp19_ = NULL;
		GXmlDomNamedNodeMap* _tmp20_ = NULL;
		GXmlDomNamedNodeMap* _tmp21_ = NULL;
		GomSerializationTestBook* _tmp22_ = NULL;
		GXmlDomNamedNodeMap* _tmp23_ = NULL;
		GXmlDomNamedNodeMap* _tmp24_ = NULL;
		GXmlDomNamedNodeMap* _tmp25_ = NULL;
		gint _tmp26_ = 0;
		gint _tmp27_ = 0;
		GomSerializationTestBook* _tmp28_ = NULL;
		const gchar* _tmp29_ = NULL;
		const gchar* _tmp30_ = NULL;
		gchar* _tmp31_ = NULL;
		GXmlXParser* _tmp32_ = NULL;
		gchar* _tmp33_ = NULL;
		gchar* _tmp34_ = NULL;
		const gchar* _tmp35_ = NULL;
		const gchar* _tmp36_ = NULL;
		gboolean _tmp37_ = FALSE;
		GomSerializationTestBook* _tmp38_ = NULL;
		GomSerializationTestBook* _tmp39_ = NULL;
		gchar* _tmp40_ = NULL;
		gchar* _tmp41_ = NULL;
		GomSerializationTestBook* _tmp42_ = NULL;
		gchar* _tmp43_ = NULL;
		const gchar* _tmp44_ = NULL;
		gboolean _tmp45_ = FALSE;
#line 827 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp0_ = gom_serialization_test_book_new ();
#line 827 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		b = _tmp0_;
#line 828 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp1_ = b;
#line 828 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = gxml_xparser_new ((GXmlDomNode*) _tmp1_);
#line 828 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		parser = _tmp2_;
#line 829 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = parser;
#line 829 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_parser_read_string ((GXmlParser*) _tmp3_, "<book name=\"Loco\"/>", NULL, &_inner_error_);
#line 829 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 829 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (parser);
#line 829 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (b);
#line 7591 "GomSerializationTest.c"
			goto __catch254_g_error;
		}
#line 830 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp4_ = parser;
#line 830 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp5_ = gxml_parser_write_string ((GXmlParser*) _tmp4_, &_inner_error_);
#line 830 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp5_;
#line 830 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 830 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (parser);
#line 830 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (b);
#line 7606 "GomSerializationTest.c"
			goto __catch254_g_error;
		}
#line 831 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp6_ = s;
#line 831 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp6_ != NULL, "s != null");
#line 835 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp7_ = b;
#line 835 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp7_ != NULL, "b != null");
#line 836 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp8_ = b;
#line 836 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp9_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp8_);
#line 836 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp10_ = _tmp9_;
#line 836 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp11_ = _tmp10_;
#line 836 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp11_ != NULL, "b.child_nodes != null");
#line 836 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp11_);
#line 837 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp12_ = b;
#line 837 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp13_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp12_);
#line 837 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp14_ = _tmp13_;
#line 837 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp15_ = _tmp14_;
#line 837 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp16_ = gee_collection_get_size ((GeeCollection*) _tmp15_);
#line 837 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp17_ = _tmp16_;
#line 837 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp17_ == 0, "b.child_nodes.size == 0");
#line 837 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp15_);
#line 838 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp18_ = b;
#line 838 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp19_ = gxml_dom_element_get_attributes ((GXmlDomElement*) _tmp18_);
#line 838 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp20_ = _tmp19_;
#line 838 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp21_ = _tmp20_;
#line 838 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp21_ != NULL, "b.attributes != null");
#line 838 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp21_);
#line 839 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp22_ = b;
#line 839 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp23_ = gxml_dom_element_get_attributes ((GXmlDomElement*) _tmp22_);
#line 839 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp24_ = _tmp23_;
#line 839 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp25_ = _tmp24_;
#line 839 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp26_ = gee_map_get_size ((GeeMap*) _tmp25_);
#line 839 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp27_ = _tmp26_;
#line 839 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp27_ == 0, "b.attributes.size == 0");
#line 839 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp25_);
#line 840 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp28_ = b;
#line 840 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp29_ = gom_serialization_test_book_get_name (_tmp28_);
#line 840 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp30_ = _tmp29_;
#line 840 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp30_ != NULL, "b.name != null");
#line 844 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp32_ = parser;
#line 844 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp33_ = gxml_parser_write_string ((GXmlParser*) _tmp32_, &_inner_error_);
#line 844 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp31_ = _tmp33_;
#line 844 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 844 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 844 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (parser);
#line 844 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (b);
#line 7695 "GomSerializationTest.c"
			goto __catch254_g_error;
		}
#line 844 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp34_ = _tmp31_;
#line 844 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp31_ = NULL;
#line 844 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 844 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp34_;
#line 845 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp35_ = s;
#line 845 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp35_ != NULL, "s != null");
#line 846 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp36_ = s;
#line 846 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp37_ = string_contains (_tmp36_, "<Book Name=\"Loco\"/>");
#line 846 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp37_, "\"<Book Name=\\\"Loco\\\"/>\" in s");
#line 850 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp38_ = b;
#line 850 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_set_name (_tmp38_, "My Book");
#line 851 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp39_ = b;
#line 851 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp40_ = gxml_dom_element_get_attribute ((GXmlDomElement*) _tmp39_, "name");
#line 851 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp41_ = _tmp40_;
#line 851 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (g_strcmp0 (_tmp41_, "My Book") == 0, "b.get_attribute (\"name\") == \"My Book\"");
#line 851 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp41_);
#line 852 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp42_ = b;
#line 852 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp43_ = gom_serialization_test_book_to_string (_tmp42_);
#line 852 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 852 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp43_;
#line 853 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp44_ = s;
#line 853 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp45_ = string_contains (_tmp44_, "<Book Name=\"My Book\"/>");
#line 853 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp45_, "\"<Book Name=\\\"My Book\\\"/>\" in s");
#line 826 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp31_);
#line 826 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 826 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (parser);
#line 826 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (b);
#line 7752 "GomSerializationTest.c"
	}
	goto __finally254;
	__catch254_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp46_ = NULL;
		gchar* _tmp47_ = NULL;
		gchar* _tmp48_ = NULL;
#line 826 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		e = _inner_error_;
#line 826 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 855 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp46_ = e->message;
#line 855 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp47_ = g_strconcat ("Error: ", _tmp46_, NULL);
#line 855 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp48_ = _tmp47_;
#line 855 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:855: %s", _tmp48_);
#line 855 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp48_);
#line 856 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 826 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_error_free0 (e);
#line 7779 "GomSerializationTest.c"
	}
	__finally254:
#line 826 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 826 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 826 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 826 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return;
#line 7790 "GomSerializationTest.c"
	}
}


static void ___lambda252__gtest_func (void) {
#line 825 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	__lambda252_ ();
#line 7798 "GomSerializationTest.c"
}


static void __lambda253_ (void) {
	GError * _inner_error_ = NULL;
	{
		GomSerializationTestBook* b = NULL;
		GomSerializationTestBook* _tmp0_ = NULL;
		const gchar* _tmp1_ = NULL;
		const gchar* _tmp2_ = NULL;
		GXmlDomNodeList* _tmp3_ = NULL;
		GXmlDomNodeList* _tmp4_ = NULL;
		GXmlDomNodeList* _tmp5_ = NULL;
		gint _tmp6_ = 0;
		gint _tmp7_ = 0;
		GXmlDomDocument* _tmp8_ = NULL;
		GXmlDomDocument* _tmp9_ = NULL;
		GXmlDomElement* _tmp10_ = NULL;
		GXmlDomElement* _tmp11_ = NULL;
		GXmlDomElement* _tmp12_ = NULL;
		GXmlDomDocument* _tmp13_ = NULL;
		GXmlDomDocument* _tmp14_ = NULL;
		GXmlDomElement* _tmp15_ = NULL;
		GXmlDomElement* _tmp16_ = NULL;
		GXmlDomElement* _tmp17_ = NULL;
		gchar* _tmp18_ = NULL;
		gchar* _tmp19_ = NULL;
		gchar* _tmp20_ = NULL;
#line 861 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp0_ = gom_serialization_test_book_new ();
#line 861 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		b = _tmp0_;
#line 862 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_read_from_string ((GXmlGomElement*) b, "<chair name=\"Tall\"/>", &_inner_error_);
#line 862 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 862 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (b);
#line 7837 "GomSerializationTest.c"
			goto __catch255_g_error;
		}
#line 867 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp1_ = gom_serialization_test_book_get_name (b);
#line 867 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = _tmp1_;
#line 867 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp2_ == NULL, "b.name == null");
#line 868 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) b);
#line 868 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp4_ = _tmp3_;
#line 868 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp5_ = _tmp4_;
#line 868 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp6_ = gee_collection_get_size ((GeeCollection*) _tmp5_);
#line 868 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp7_ = _tmp6_;
#line 868 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp7_ == 0, "b.child_nodes.size == 0");
#line 868 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp5_);
#line 869 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp8_ = gxml_dom_node_get_owner_document ((GXmlDomNode*) b);
#line 869 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp9_ = _tmp8_;
#line 869 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp10_ = gxml_dom_document_get_document_element (_tmp9_);
#line 869 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp11_ = _tmp10_;
#line 869 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp12_ = _tmp11_;
#line 869 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp12_ != NULL, "b.owner_document.document_element != null");
#line 869 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp12_);
#line 870 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp13_ = gxml_dom_node_get_owner_document ((GXmlDomNode*) b);
#line 870 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp14_ = _tmp13_;
#line 870 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp15_ = gxml_dom_document_get_document_element (_tmp14_);
#line 870 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp16_ = _tmp15_;
#line 870 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp17_ = _tmp16_;
#line 870 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp18_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp17_);
#line 870 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp19_ = _tmp18_;
#line 870 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp20_ = _tmp19_;
#line 870 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (g_strcmp0 (_tmp20_, "Book") == 0, "b.owner_document.document_element.node_name == \"Book\"");
#line 870 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp20_);
#line 870 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp17_);
#line 860 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (b);
#line 7898 "GomSerializationTest.c"
	}
	goto __finally255;
	__catch255_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp21_ = NULL;
		gchar* _tmp22_ = NULL;
		gchar* _tmp23_ = NULL;
#line 860 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		e = _inner_error_;
#line 860 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 872 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp21_ = e->message;
#line 872 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp22_ = g_strconcat ("Error: ", _tmp21_, NULL);
#line 872 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp23_ = _tmp22_;
#line 872 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:872: %s", _tmp23_);
#line 872 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp23_);
#line 873 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 860 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_error_free0 (e);
#line 7925 "GomSerializationTest.c"
	}
	__finally255:
#line 860 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 860 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 860 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 860 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return;
#line 7936 "GomSerializationTest.c"
	}
}


static void ___lambda253__gtest_func (void) {
#line 859 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	__lambda253_ ();
#line 7944 "GomSerializationTest.c"
}


static void __lambda254_ (void) {
	GError * _inner_error_ = NULL;
	{
		GomSerializationTestBookRegister* b = NULL;
		GomSerializationTestBookRegister* _tmp0_ = NULL;
		gchar* s = NULL;
		gchar* _tmp1_ = NULL;
		const gchar* _tmp2_ = NULL;
		gboolean _tmp3_ = FALSE;
		gchar* _tmp4_ = NULL;
		const gchar* _tmp5_ = NULL;
		gchar* _tmp6_ = NULL;
		gchar* _tmp7_ = NULL;
		const gchar* _tmp8_ = NULL;
		gboolean _tmp9_ = FALSE;
#line 878 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp0_ = gom_serialization_test_book_register_new ();
#line 878 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		b = _tmp0_;
#line 879 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp1_ = gom_serialization_test_book_register_to_string (b);
#line 879 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp1_;
#line 883 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = s;
#line 883 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = string_contains (_tmp2_, "<BookRegister Year=\"0\"/>");
#line 883 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp3_, "\"<BookRegister Year=\\\"0\\\"/>\" in s");
#line 884 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_read_from_string ((GXmlGomElement*) b, "<bookRegister><Book/></bookRegister>", &_inner_error_);
#line 884 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 884 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 884 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (b);
#line 7985 "GomSerializationTest.c"
			goto __catch256_g_error;
		}
#line 885 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp4_ = gom_serialization_test_book_register_to_string (b);
#line 885 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 885 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp4_;
#line 887 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp5_ = s;
#line 887 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp6_ = g_strconcat ("doc:", _tmp5_, NULL);
#line 887 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp7_ = _tmp6_;
#line 887 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:887: %s", _tmp7_);
#line 887 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp7_);
#line 889 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp8_ = s;
#line 889 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp9_ = string_contains (_tmp8_, "<BookRegister Year=\"0\"><Book/></BookRegister>");
#line 889 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp9_, "\"<BookRegister Year=\\\"0\\\"><Book/></BookRegister>\" in s");
#line 877 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 877 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (b);
#line 8014 "GomSerializationTest.c"
	}
	goto __finally256;
	__catch256_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp10_ = NULL;
		gchar* _tmp11_ = NULL;
		gchar* _tmp12_ = NULL;
#line 877 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		e = _inner_error_;
#line 877 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 891 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp10_ = e->message;
#line 891 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp11_ = g_strconcat ("Error: ", _tmp10_, NULL);
#line 891 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp12_ = _tmp11_;
#line 891 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:891: %s", _tmp12_);
#line 891 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp12_);
#line 892 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 877 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_error_free0 (e);
#line 8041 "GomSerializationTest.c"
	}
	__finally256:
#line 877 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 877 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 877 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 877 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return;
#line 8052 "GomSerializationTest.c"
	}
}


static void ___lambda254__gtest_func (void) {
#line 876 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	__lambda254_ ();
#line 8060 "GomSerializationTest.c"
}


static void __lambda255_ (void) {
	GError * _inner_error_ = NULL;
	{
		GomSerializationTestMotor* m = NULL;
		GomSerializationTestMotor* _tmp0_ = NULL;
		gchar* s = NULL;
		gchar* _tmp1_ = NULL;
		const gchar* _tmp2_ = NULL;
		gboolean _tmp3_ = FALSE;
		GXmlXParser* parser = NULL;
		GXmlXParser* _tmp4_ = NULL;
		gchar* _tmp5_ = NULL;
		const gchar* _tmp6_ = NULL;
		gboolean _tmp7_ = FALSE;
		const gchar* _tmp8_ = NULL;
		gboolean _tmp9_ = FALSE;
		const gchar* _tmp10_ = NULL;
		gboolean _tmp11_ = FALSE;
		const gchar* _tmp12_ = NULL;
		gboolean _tmp13_ = FALSE;
		const gchar* _tmp14_ = NULL;
		gboolean _tmp15_ = FALSE;
		const gchar* _tmp16_ = NULL;
		gboolean _tmp17_ = FALSE;
		const gchar* _tmp18_ = NULL;
		gboolean _tmp19_ = FALSE;
		GomSerializationTestMotorOn* _tmp20_ = NULL;
		GomSerializationTestMotorOn* _tmp21_ = NULL;
		GomSerializationTestMotorOn* _tmp22_ = NULL;
		GomSerializationTestMotorOn* _tmp23_ = NULL;
		gboolean _tmp24_ = FALSE;
		GomSerializationTestMotorTorque* _tmp25_ = NULL;
		GomSerializationTestMotorTorque* _tmp26_ = NULL;
		GomSerializationTestMotorTorque* _tmp27_ = NULL;
		GomSerializationTestMotorTorque* _tmp28_ = NULL;
		gdouble _tmp29_ = 0.0;
		GomSerializationTestMotorSpeed* _tmp30_ = NULL;
		GomSerializationTestMotorSpeed* _tmp31_ = NULL;
		GomSerializationTestMotorSpeed* _tmp32_ = NULL;
		GomSerializationTestMotorSpeed* _tmp33_ = NULL;
		gdouble _tmp34_ = 0.0;
		GomSerializationTestMotorTension* _tmp35_ = NULL;
		GomSerializationTestMotorTension* _tmp36_ = NULL;
		GomSerializationTestMotorTension* _tmp37_ = NULL;
		GomSerializationTestMotorTension* _tmp38_ = NULL;
		gint _tmp39_ = 0;
		GomSerializationTestMotorTensionType* _tmp40_ = NULL;
		GomSerializationTestMotorTensionType* _tmp41_ = NULL;
		GomSerializationTestMotorTensionType* _tmp42_ = NULL;
		GomSerializationTestMotorTensionType* _tmp43_ = NULL;
		gchar* _tmp44_ = NULL;
		gchar* _tmp45_ = NULL;
		gchar* _tmp46_ = NULL;
#line 897 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp0_ = gom_serialization_test_motor_new ();
#line 897 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		m = _tmp0_;
#line 898 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp1_ = gom_serialization_test_motor_to_string (m);
#line 898 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp1_;
#line 902 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = s;
#line 902 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = string_contains (_tmp2_, "<Motor/>");
#line 902 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp3_, "\"<Motor/>\" in s");
#line 903 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp4_ = gxml_xparser_new ((GXmlDomNode*) m);
#line 903 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		parser = _tmp4_;
#line 904 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_parser_read_string ((GXmlParser*) parser, "<Motor On=\"true\" Torque=\"3.1416\" Speed=\"3600.1011\" TensionType=\"" \
"dc\" Tension=\"125\"/>", NULL, &_inner_error_);
#line 904 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 904 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (parser);
#line 904 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 904 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (m);
#line 8145 "GomSerializationTest.c"
			goto __catch257_g_error;
		}
#line 905 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp5_ = gom_serialization_test_motor_to_string (m);
#line 905 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 905 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp5_;
#line 909 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp6_ = s;
#line 909 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp7_ = string_contains (_tmp6_, "<Motor ");
#line 909 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp7_, "\"<Motor \" in s");
#line 910 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp8_ = s;
#line 910 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp9_ = string_contains (_tmp8_, "On=\"true\"");
#line 910 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp9_, "\"On=\\\"true\\\"\" in s");
#line 911 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp10_ = s;
#line 911 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp11_ = string_contains (_tmp10_, "Torque=\"3.1416\"");
#line 911 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp11_, "\"Torque=\\\"3.1416\\\"\" in s");
#line 912 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp12_ = s;
#line 912 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp13_ = string_contains (_tmp12_, "Speed=\"3600.1011\"");
#line 912 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp13_, "\"Speed=\\\"3600.1011\\\"\" in s");
#line 913 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp14_ = s;
#line 913 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp15_ = string_contains (_tmp14_, "TensionType=\"dc\"");
#line 913 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp15_, "\"TensionType=\\\"dc\\\"\" in s");
#line 914 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp16_ = s;
#line 914 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp17_ = string_contains (_tmp16_, "Tension=\"125\"");
#line 914 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp17_, "\"Tension=\\\"125\\\"\" in s");
#line 915 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp18_ = s;
#line 915 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp19_ = string_contains (_tmp18_, "/>");
#line 915 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp19_, "\"/>\" in s");
#line 916 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp20_ = gom_serialization_test_motor_get_is_on (m);
#line 916 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp21_ = _tmp20_;
#line 916 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp21_ != NULL, "m.is_on != null");
#line 917 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp22_ = gom_serialization_test_motor_get_is_on (m);
#line 917 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp23_ = _tmp22_;
#line 917 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp24_ = gxml_gom_boolean_get_boolean ((GXmlGomBoolean*) _tmp23_);
#line 917 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp24_ == TRUE, "m.is_on.get_boolean() == true");
#line 918 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp25_ = gom_serialization_test_motor_get_torque (m);
#line 918 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp26_ = _tmp25_;
#line 918 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp26_ != NULL, "m.torque != null");
#line 919 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp27_ = gom_serialization_test_motor_get_torque (m);
#line 919 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp28_ = _tmp27_;
#line 919 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp29_ = gxml_gom_double_get_double ((GXmlGomDouble*) _tmp28_);
#line 919 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp29_ == 3.1416, "m.torque.get_double () == 3.1416");
#line 920 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp30_ = gom_serialization_test_motor_get_speed (m);
#line 920 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp31_ = _tmp30_;
#line 920 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp31_ != NULL, "m.speed != null");
#line 921 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp32_ = gom_serialization_test_motor_get_speed (m);
#line 921 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp33_ = _tmp32_;
#line 921 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp34_ = gxml_gom_double_get_double ((GXmlGomDouble*) _tmp33_);
#line 921 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp34_ == 3600.1011, "m.speed.get_double () == 3600.1011");
#line 922 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp35_ = gom_serialization_test_motor_get_tension (m);
#line 922 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp36_ = _tmp35_;
#line 922 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp36_ != NULL, "m.tension != null");
#line 923 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp37_ = gom_serialization_test_motor_get_tension (m);
#line 923 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp38_ = _tmp37_;
#line 923 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp39_ = gxml_gom_int_get_integer ((GXmlGomInt*) _tmp38_);
#line 923 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp39_ == 125, "m.tension.get_integer () == 125");
#line 924 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp40_ = gom_serialization_test_motor_get_tension_type (m);
#line 924 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp41_ = _tmp40_;
#line 924 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp41_ != NULL, "m.tension_type != null");
#line 925 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp42_ = gom_serialization_test_motor_get_tension_type (m);
#line 925 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp43_ = _tmp42_;
#line 925 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp44_ = gxml_gom_base_property_get_value ((GXmlGomBaseProperty*) _tmp43_);
#line 925 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp45_ = _tmp44_;
#line 925 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp46_ = _tmp45_;
#line 925 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (g_strcmp0 (_tmp46_, "dc") == 0, "m.tension_type.value == \"dc\"");
#line 925 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp46_);
#line 896 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (parser);
#line 896 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 896 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (m);
#line 8278 "GomSerializationTest.c"
	}
	goto __finally257;
	__catch257_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp47_ = NULL;
		gchar* _tmp48_ = NULL;
		gchar* _tmp49_ = NULL;
#line 896 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		e = _inner_error_;
#line 896 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 927 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp47_ = e->message;
#line 927 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp48_ = g_strconcat ("Error: ", _tmp47_, NULL);
#line 927 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp49_ = _tmp48_;
#line 927 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:927: %s", _tmp49_);
#line 927 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp49_);
#line 928 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 896 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_error_free0 (e);
#line 8305 "GomSerializationTest.c"
	}
	__finally257:
#line 896 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 896 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 896 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 896 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return;
#line 8316 "GomSerializationTest.c"
	}
}


static void ___lambda255__gtest_func (void) {
#line 895 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	__lambda255_ ();
#line 8324 "GomSerializationTest.c"
}


static void __lambda256_ (void) {
	GError * _inner_error_ = NULL;
	{
		GomSerializationTestBookStand* bs = NULL;
		GomSerializationTestBookStand* _tmp0_ = NULL;
		gchar* s = NULL;
		GomSerializationTestBookStand* _tmp1_ = NULL;
		gchar* _tmp2_ = NULL;
		const gchar* _tmp3_ = NULL;
		gboolean _tmp4_ = FALSE;
		GXmlXParser* parser = NULL;
		GomSerializationTestBookStand* _tmp5_ = NULL;
		GXmlXParser* _tmp6_ = NULL;
		GXmlXParser* _tmp7_ = NULL;
		GomSerializationTestBookStand* _tmp8_ = NULL;
		gchar* _tmp9_ = NULL;
		GomSerializationTestBookStand* _tmp10_ = NULL;
		GomSerializationTestRegisters* _tmp11_ = NULL;
		GomSerializationTestRegisters* _tmp12_ = NULL;
		GomSerializationTestBookStand* _tmp13_ = NULL;
		GomSerializationTestRegisters* _tmp14_ = NULL;
		GomSerializationTestRegisters* _tmp15_ = NULL;
		gint _tmp16_ = 0;
		gint _tmp17_ = 0;
		GomSerializationTestBookStand* _tmp18_ = NULL;
		GomSerializationTestRegisters* _tmp19_ = NULL;
		GomSerializationTestRegisters* _tmp20_ = NULL;
		GQueue* _tmp21_ = NULL;
		GQueue* _tmp22_ = NULL;
		gconstpointer _tmp23_ = NULL;
		GomSerializationTestBookStand* _tmp24_ = NULL;
		GomSerializationTestRegisters* _tmp25_ = NULL;
		GomSerializationTestRegisters* _tmp26_ = NULL;
		GQueue* _tmp27_ = NULL;
		GQueue* _tmp28_ = NULL;
		gconstpointer _tmp29_ = NULL;
		GomSerializationTestBookStand* _tmp30_ = NULL;
		GomSerializationTestRegisters* _tmp31_ = NULL;
		GomSerializationTestRegisters* _tmp32_ = NULL;
		GQueue* _tmp33_ = NULL;
		GQueue* _tmp34_ = NULL;
		gconstpointer _tmp35_ = NULL;
		GXmlDomElement* _tmp36_ = NULL;
		GomSerializationTestBookStand* _tmp37_ = NULL;
		GomSerializationTestRegisters* _tmp38_ = NULL;
		GomSerializationTestRegisters* _tmp39_ = NULL;
		GXmlDomElement* _tmp40_ = NULL;
		GXmlDomElement* _tmp41_ = NULL;
		GXmlDomElement* _tmp42_ = NULL;
		GXmlDomElement* _tmp43_ = NULL;
		GomSerializationTestBookStand* _tmp44_ = NULL;
		GomSerializationTestRegisters* _tmp45_ = NULL;
		GomSerializationTestRegisters* _tmp46_ = NULL;
		GXmlDomElement* _tmp47_ = NULL;
		GXmlDomElement* _tmp48_ = NULL;
		GXmlDomElement* _tmp49_ = NULL;
		GomSerializationTestBookStand* _tmp50_ = NULL;
		GomSerializationTestRegisters* _tmp51_ = NULL;
		GomSerializationTestRegisters* _tmp52_ = NULL;
		GXmlDomElement* _tmp53_ = NULL;
		GXmlDomElement* _tmp54_ = NULL;
		GXmlDomElement* _tmp55_ = NULL;
		GomSerializationTestBookStand* _tmp56_ = NULL;
		GomSerializationTestRegisters* _tmp57_ = NULL;
		GomSerializationTestRegisters* _tmp58_ = NULL;
		GXmlDomElement* _tmp59_ = NULL;
		GXmlDomElement* _tmp60_ = NULL;
		GXmlDomElement* _tmp61_ = NULL;
		GXmlDomElement* _tmp62_ = NULL;
		GomSerializationTestBookStand* _tmp63_ = NULL;
		GomSerializationTestRegisters* _tmp64_ = NULL;
		GomSerializationTestRegisters* _tmp65_ = NULL;
		GXmlDomElement* _tmp66_ = NULL;
		GXmlDomElement* _tmp67_ = NULL;
		GXmlDomElement* _tmp68_ = NULL;
		GomSerializationTestBookStand* _tmp69_ = NULL;
		GomSerializationTestRegisters* _tmp70_ = NULL;
		GomSerializationTestRegisters* _tmp71_ = NULL;
		GXmlDomElement* _tmp72_ = NULL;
		GXmlDomElement* _tmp73_ = NULL;
		GXmlDomElement* _tmp74_ = NULL;
		GomSerializationTestBookStand* _tmp75_ = NULL;
		GomSerializationTestRegisters* _tmp76_ = NULL;
		GomSerializationTestRegisters* _tmp77_ = NULL;
		GXmlDomElement* _tmp78_ = NULL;
		GXmlDomElement* _tmp79_ = NULL;
		GXmlDomElement* _tmp80_ = NULL;
		GXmlDomElement* _tmp81_ = NULL;
		GomSerializationTestBookStand* _tmp82_ = NULL;
		GomSerializationTestRegisters* _tmp83_ = NULL;
		GomSerializationTestRegisters* _tmp84_ = NULL;
		GXmlDomElement* _tmp85_ = NULL;
		GXmlDomElement* _tmp86_ = NULL;
		GXmlDomElement* _tmp87_ = NULL;
		GomSerializationTestBookStand* _tmp88_ = NULL;
		GomSerializationTestRegisters* _tmp89_ = NULL;
		GomSerializationTestRegisters* _tmp90_ = NULL;
		GXmlDomElement* _tmp91_ = NULL;
		GXmlDomElement* _tmp92_ = NULL;
		GXmlDomElement* _tmp93_ = NULL;
		GomSerializationTestBookStand* _tmp94_ = NULL;
		GomSerializationTestRegisters* _tmp95_ = NULL;
		GomSerializationTestRegisters* _tmp96_ = NULL;
		GXmlDomElement* _tmp97_ = NULL;
		GXmlDomElement* _tmp98_ = NULL;
		GomSerializationTestBookRegister* _tmp99_ = NULL;
		GomSerializationTestBookRegister* _tmp100_ = NULL;
		gint _tmp101_ = 0;
		gint _tmp102_ = 0;
		GXmlDomElement* _tmp103_ = NULL;
		GomSerializationTestBookStand* _tmp104_ = NULL;
		GomSerializationTestRegisters* _tmp105_ = NULL;
		GomSerializationTestRegisters* _tmp106_ = NULL;
		GXmlDomElement* _tmp107_ = NULL;
		GXmlDomElement* _tmp108_ = NULL;
		GomSerializationTestBookRegister* _tmp109_ = NULL;
		GomSerializationTestBookRegister* _tmp110_ = NULL;
		gint _tmp111_ = 0;
		gint _tmp112_ = 0;
		GXmlDomElement* _tmp113_ = NULL;
		GomSerializationTestBookStand* _tmp114_ = NULL;
		GomSerializationTestRegisters* _tmp115_ = NULL;
		GomSerializationTestRegisters* _tmp116_ = NULL;
		GXmlDomElement* _tmp117_ = NULL;
		GXmlDomElement* _tmp118_ = NULL;
		GomSerializationTestBookRegister* _tmp119_ = NULL;
		GomSerializationTestBookRegister* _tmp120_ = NULL;
		gint _tmp121_ = 0;
		gint _tmp122_ = 0;
#line 933 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp0_ = gom_serialization_test_book_stand_new ();
#line 933 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		bs = _tmp0_;
#line 934 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp1_ = bs;
#line 934 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = gom_serialization_test_book_stand_to_string (_tmp1_);
#line 934 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp2_;
#line 938 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = s;
#line 938 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp4_ = string_contains (_tmp3_, "<BookStand Classification=\"Science\"/>");
#line 938 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp4_, "\"<BookStand Classification=\\\"Science\\\"/>\" in s");
#line 939 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp5_ = bs;
#line 939 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp6_ = gxml_xparser_new ((GXmlDomNode*) _tmp5_);
#line 939 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		parser = _tmp6_;
#line 940 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp7_ = parser;
#line 940 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_parser_read_string ((GXmlParser*) _tmp7_, "<BookStand Classification=\"Science\"><BookRegister Year=\"2016\"/><Bo" \
"okRegister Year=\"2010\"/><Test/><BookRegister Year=\"2000\"/></BookSt" \
"and>", NULL, &_inner_error_);
#line 940 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 940 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (parser);
#line 940 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 940 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 8491 "GomSerializationTest.c"
			goto __catch258_g_error;
		}
#line 941 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp8_ = bs;
#line 941 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp9_ = gom_serialization_test_book_stand_to_string (_tmp8_);
#line 941 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 941 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp9_;
#line 946 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp10_ = bs;
#line 946 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp11_ = gom_serialization_test_book_stand_get_registers (_tmp10_);
#line 946 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp12_ = _tmp11_;
#line 946 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp12_ != NULL, "bs.registers != null");
#line 947 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp13_ = bs;
#line 947 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp14_ = gom_serialization_test_book_stand_get_registers (_tmp13_);
#line 947 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp15_ = _tmp14_;
#line 947 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp16_ = gxml_gom_collection_get_length ((GXmlGomCollection*) _tmp15_);
#line 947 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp17_ = _tmp16_;
#line 947 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp17_ == 3, "bs.registers.length == 3");
#line 948 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp18_ = bs;
#line 948 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp19_ = gom_serialization_test_book_stand_get_registers (_tmp18_);
#line 948 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp20_ = _tmp19_;
#line 948 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp21_ = gxml_gom_collection_get_nodes_index ((GXmlGomCollection*) _tmp20_);
#line 948 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp22_ = _tmp21_;
#line 948 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp23_ = g_queue_peek_nth (_tmp22_, (guint) 0);
#line 948 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (((gint) ((gintptr) _tmp23_)) == 0, "bs.registers.nodes_index.peek_nth (0) == 0");
#line 949 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp24_ = bs;
#line 949 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp25_ = gom_serialization_test_book_stand_get_registers (_tmp24_);
#line 949 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp26_ = _tmp25_;
#line 949 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp27_ = gxml_gom_collection_get_nodes_index ((GXmlGomCollection*) _tmp26_);
#line 949 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp28_ = _tmp27_;
#line 949 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp29_ = g_queue_peek_nth (_tmp28_, (guint) 1);
#line 949 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (((gint) ((gintptr) _tmp29_)) == 1, "bs.registers.nodes_index.peek_nth (1) == 1");
#line 950 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp30_ = bs;
#line 950 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp31_ = gom_serialization_test_book_stand_get_registers (_tmp30_);
#line 950 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp32_ = _tmp31_;
#line 950 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp33_ = gxml_gom_collection_get_nodes_index ((GXmlGomCollection*) _tmp32_);
#line 950 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp34_ = _tmp33_;
#line 950 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp35_ = g_queue_peek_nth (_tmp34_, (guint) 2);
#line 950 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (((gint) ((gintptr) _tmp35_)) == 3, "bs.registers.nodes_index.peek_nth (2) == 3");
#line 951 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp37_ = bs;
#line 951 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp38_ = gom_serialization_test_book_stand_get_registers (_tmp37_);
#line 951 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp39_ = _tmp38_;
#line 951 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp40_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp39_, 0, &_inner_error_);
#line 951 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp36_ = _tmp40_;
#line 951 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 951 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (parser);
#line 951 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 951 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 8582 "GomSerializationTest.c"
			goto __catch258_g_error;
		}
#line 951 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp41_ = _tmp36_;
#line 951 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp36_ = NULL;
#line 951 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp42_ = _tmp41_;
#line 951 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp42_ != NULL, "bs.registers.get_item (0) != null");
#line 951 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp42_);
#line 952 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp44_ = bs;
#line 952 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp45_ = gom_serialization_test_book_stand_get_registers (_tmp44_);
#line 952 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp46_ = _tmp45_;
#line 952 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp47_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp46_, 0, &_inner_error_);
#line 952 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp43_ = _tmp47_;
#line 952 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 952 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp36_);
#line 952 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (parser);
#line 952 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 952 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 8615 "GomSerializationTest.c"
			goto __catch258_g_error;
		}
#line 952 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp48_ = _tmp43_;
#line 952 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp43_ = NULL;
#line 952 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp48_, GXML_TYPE_DOM_ELEMENT), "bs.registers.get_item (0) is DomElement");
#line 953 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp50_ = bs;
#line 953 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp51_ = gom_serialization_test_book_stand_get_registers (_tmp50_);
#line 953 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp52_ = _tmp51_;
#line 953 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp53_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp52_, 0, &_inner_error_);
#line 953 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp49_ = _tmp53_;
#line 953 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 953 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp43_);
#line 953 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp36_);
#line 953 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (parser);
#line 953 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 953 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 8646 "GomSerializationTest.c"
			goto __catch258_g_error;
		}
#line 953 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp54_ = _tmp49_;
#line 953 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp49_ = NULL;
#line 953 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp54_, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER), "bs.registers.get_item (0) is BookRegister");
#line 954 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp56_ = bs;
#line 954 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp57_ = gom_serialization_test_book_stand_get_registers (_tmp56_);
#line 954 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp58_ = _tmp57_;
#line 954 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp59_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp58_, 1, &_inner_error_);
#line 954 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp55_ = _tmp59_;
#line 954 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 954 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp49_);
#line 954 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp43_);
#line 954 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp36_);
#line 954 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (parser);
#line 954 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 954 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 8679 "GomSerializationTest.c"
			goto __catch258_g_error;
		}
#line 954 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp60_ = _tmp55_;
#line 954 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp55_ = NULL;
#line 954 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp61_ = _tmp60_;
#line 954 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp61_ != NULL, "bs.registers.get_item (1) != null");
#line 954 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp61_);
#line 955 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp63_ = bs;
#line 955 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp64_ = gom_serialization_test_book_stand_get_registers (_tmp63_);
#line 955 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp65_ = _tmp64_;
#line 955 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp66_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp65_, 1, &_inner_error_);
#line 955 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp62_ = _tmp66_;
#line 955 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 955 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp55_);
#line 955 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp49_);
#line 955 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp43_);
#line 955 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp36_);
#line 955 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (parser);
#line 955 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 955 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 8718 "GomSerializationTest.c"
			goto __catch258_g_error;
		}
#line 955 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp67_ = _tmp62_;
#line 955 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp62_ = NULL;
#line 955 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp67_, GXML_TYPE_DOM_ELEMENT), "bs.registers.get_item (1) is DomElement");
#line 956 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp69_ = bs;
#line 956 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp70_ = gom_serialization_test_book_stand_get_registers (_tmp69_);
#line 956 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp71_ = _tmp70_;
#line 956 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp72_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp71_, 1, &_inner_error_);
#line 956 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp68_ = _tmp72_;
#line 956 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 956 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp62_);
#line 956 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp55_);
#line 956 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp49_);
#line 956 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp43_);
#line 956 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp36_);
#line 956 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (parser);
#line 956 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 956 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 8755 "GomSerializationTest.c"
			goto __catch258_g_error;
		}
#line 956 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp73_ = _tmp68_;
#line 956 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp68_ = NULL;
#line 956 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp73_, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER), "bs.registers.get_item (1) is BookRegister");
#line 957 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp75_ = bs;
#line 957 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp76_ = gom_serialization_test_book_stand_get_registers (_tmp75_);
#line 957 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp77_ = _tmp76_;
#line 957 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp78_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp77_, 2, &_inner_error_);
#line 957 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp74_ = _tmp78_;
#line 957 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 957 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp68_);
#line 957 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp62_);
#line 957 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp55_);
#line 957 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp49_);
#line 957 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp43_);
#line 957 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp36_);
#line 957 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (parser);
#line 957 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 957 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 8794 "GomSerializationTest.c"
			goto __catch258_g_error;
		}
#line 957 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp79_ = _tmp74_;
#line 957 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp74_ = NULL;
#line 957 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp80_ = _tmp79_;
#line 957 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp80_ != NULL, "bs.registers.get_item (2) != null");
#line 957 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp80_);
#line 958 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp82_ = bs;
#line 958 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp83_ = gom_serialization_test_book_stand_get_registers (_tmp82_);
#line 958 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp84_ = _tmp83_;
#line 958 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp85_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp84_, 2, &_inner_error_);
#line 958 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp81_ = _tmp85_;
#line 958 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 958 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp74_);
#line 958 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp68_);
#line 958 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp62_);
#line 958 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp55_);
#line 958 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp49_);
#line 958 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp43_);
#line 958 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp36_);
#line 958 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (parser);
#line 958 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 958 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 8839 "GomSerializationTest.c"
			goto __catch258_g_error;
		}
#line 958 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp86_ = _tmp81_;
#line 958 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp81_ = NULL;
#line 958 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp86_, GXML_TYPE_DOM_ELEMENT), "bs.registers.get_item (2) is DomElement");
#line 959 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp88_ = bs;
#line 959 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp89_ = gom_serialization_test_book_stand_get_registers (_tmp88_);
#line 959 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp90_ = _tmp89_;
#line 959 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp91_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp90_, 2, &_inner_error_);
#line 959 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp87_ = _tmp91_;
#line 959 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 959 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp81_);
#line 959 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp74_);
#line 959 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp68_);
#line 959 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp62_);
#line 959 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp55_);
#line 959 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp49_);
#line 959 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp43_);
#line 959 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp36_);
#line 959 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (parser);
#line 959 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 959 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 8882 "GomSerializationTest.c"
			goto __catch258_g_error;
		}
#line 959 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp92_ = _tmp87_;
#line 959 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp87_ = NULL;
#line 959 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp92_, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER), "bs.registers.get_item (2) is BookRegister");
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp94_ = bs;
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp95_ = gom_serialization_test_book_stand_get_registers (_tmp94_);
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp96_ = _tmp95_;
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp97_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp96_, 0, &_inner_error_);
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp93_ = _tmp97_;
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp87_);
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp81_);
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp74_);
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp68_);
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp62_);
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp55_);
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp49_);
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp43_);
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp36_);
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (parser);
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 8927 "GomSerializationTest.c"
			goto __catch258_g_error;
		}
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp98_ = _tmp93_;
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp93_ = NULL;
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp99_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp98_, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER) ? ((GomSerializationTestBookRegister*) _tmp98_) : NULL;
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (_tmp99_ == NULL) {
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp98_);
#line 8940 "GomSerializationTest.c"
		}
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp100_ = _tmp99_;
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp101_ = gom_serialization_test_book_register_get_year (_tmp100_);
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp102_ = _tmp101_;
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp102_ == 2016, "(bs.registers.get_item (0) as BookRegister).year == 2016");
#line 960 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp100_);
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp104_ = bs;
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp105_ = gom_serialization_test_book_stand_get_registers (_tmp104_);
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp106_ = _tmp105_;
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp107_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp106_, 1, &_inner_error_);
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp103_ = _tmp107_;
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp93_);
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp87_);
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp81_);
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp74_);
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp68_);
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp62_);
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp55_);
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp49_);
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp43_);
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp36_);
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (parser);
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 8990 "GomSerializationTest.c"
			goto __catch258_g_error;
		}
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp108_ = _tmp103_;
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp103_ = NULL;
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp109_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp108_, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER) ? ((GomSerializationTestBookRegister*) _tmp108_) : NULL;
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (_tmp109_ == NULL) {
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp108_);
#line 9003 "GomSerializationTest.c"
		}
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp110_ = _tmp109_;
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp111_ = gom_serialization_test_book_register_get_year (_tmp110_);
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp112_ = _tmp111_;
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp112_ == 2010, "(bs.registers.get_item (1) as BookRegister).year == 2010");
#line 961 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp110_);
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp114_ = bs;
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp115_ = gom_serialization_test_book_stand_get_registers (_tmp114_);
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp116_ = _tmp115_;
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp117_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp116_, 2, &_inner_error_);
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp113_ = _tmp117_;
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp103_);
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp93_);
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp87_);
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp81_);
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp74_);
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp68_);
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp62_);
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp55_);
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp49_);
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp43_);
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp36_);
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (parser);
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 9055 "GomSerializationTest.c"
			goto __catch258_g_error;
		}
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp118_ = _tmp113_;
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp113_ = NULL;
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp119_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp118_, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER) ? ((GomSerializationTestBookRegister*) _tmp118_) : NULL;
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (_tmp119_ == NULL) {
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp118_);
#line 9068 "GomSerializationTest.c"
		}
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp120_ = _tmp119_;
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp121_ = gom_serialization_test_book_register_get_year (_tmp120_);
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp122_ = _tmp121_;
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp122_ == 2000, "(bs.registers.get_item (2) as BookRegister).year == 2000");
#line 962 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp120_);
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp113_);
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp103_);
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp93_);
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp87_);
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp81_);
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp74_);
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp68_);
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp62_);
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp55_);
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp49_);
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp43_);
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp36_);
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (parser);
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (bs);
#line 9110 "GomSerializationTest.c"
	}
	goto __finally258;
	__catch258_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp123_ = NULL;
		gchar* _tmp124_ = NULL;
		gchar* _tmp125_ = NULL;
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		e = _inner_error_;
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 964 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp123_ = e->message;
#line 964 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp124_ = g_strconcat ("Error: ", _tmp123_, NULL);
#line 964 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp125_ = _tmp124_;
#line 964 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:964: %s", _tmp125_);
#line 964 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp125_);
#line 965 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_error_free0 (e);
#line 9137 "GomSerializationTest.c"
	}
	__finally258:
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 932 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return;
#line 9148 "GomSerializationTest.c"
	}
}


static void ___lambda256__gtest_func (void) {
#line 931 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	__lambda256_ ();
#line 9156 "GomSerializationTest.c"
}


static void __lambda257_ (void) {
	GError * _inner_error_ = NULL;
	{
		GomSerializationTestBookStand* bs = NULL;
		GomSerializationTestBookStand* _tmp0_ = NULL;
		gchar* s = NULL;
		GomSerializationTestBookStand* _tmp1_ = NULL;
		gchar* _tmp2_ = NULL;
		const gchar* _tmp3_ = NULL;
		gboolean _tmp4_ = FALSE;
		GomSerializationTestBookStand* _tmp5_ = NULL;
		GomSerializationTestBookStand* _tmp6_ = NULL;
		GomSerializationTestBooks* _tmp7_ = NULL;
		GomSerializationTestBooks* _tmp8_ = NULL;
		GomSerializationTestBookStand* _tmp9_ = NULL;
		gchar* _tmp10_ = NULL;
		GomSerializationTestBookStand* _tmp11_ = NULL;
		GomSerializationTestBooks* _tmp12_ = NULL;
		GomSerializationTestBooks* _tmp13_ = NULL;
		gint _tmp14_ = 0;
		gint _tmp15_ = 0;
		GomSerializationTestBookStand* _tmp16_ = NULL;
		GomSerializationTestBooks* _tmp17_ = NULL;
		GomSerializationTestBooks* _tmp18_ = NULL;
		GQueue* _tmp19_ = NULL;
		GQueue* _tmp20_ = NULL;
		gconstpointer _tmp21_ = NULL;
		GomSerializationTestBookStand* _tmp22_ = NULL;
		GomSerializationTestBooks* _tmp23_ = NULL;
		GomSerializationTestBooks* _tmp24_ = NULL;
		GQueue* _tmp25_ = NULL;
		GQueue* _tmp26_ = NULL;
		gconstpointer _tmp27_ = NULL;
		GomSerializationTestBookStand* _tmp28_ = NULL;
		GomSerializationTestBooks* _tmp29_ = NULL;
		GomSerializationTestBooks* _tmp30_ = NULL;
		GQueue* _tmp31_ = NULL;
		GQueue* _tmp32_ = NULL;
		gconstpointer _tmp33_ = NULL;
		GXmlDomElement* _tmp34_ = NULL;
		GomSerializationTestBookStand* _tmp35_ = NULL;
		GomSerializationTestBooks* _tmp36_ = NULL;
		GomSerializationTestBooks* _tmp37_ = NULL;
		GXmlDomElement* _tmp38_ = NULL;
		GXmlDomElement* _tmp39_ = NULL;
		GXmlDomElement* _tmp40_ = NULL;
		GXmlDomElement* _tmp41_ = NULL;
		GomSerializationTestBookStand* _tmp42_ = NULL;
		GomSerializationTestBooks* _tmp43_ = NULL;
		GomSerializationTestBooks* _tmp44_ = NULL;
		GXmlDomElement* _tmp45_ = NULL;
		GXmlDomElement* _tmp46_ = NULL;
		GXmlDomElement* _tmp47_ = NULL;
		GomSerializationTestBookStand* _tmp48_ = NULL;
		GomSerializationTestBooks* _tmp49_ = NULL;
		GomSerializationTestBooks* _tmp50_ = NULL;
		GXmlDomElement* _tmp51_ = NULL;
		GXmlDomElement* _tmp52_ = NULL;
		GXmlDomElement* _tmp53_ = NULL;
		GomSerializationTestBookStand* _tmp54_ = NULL;
		GomSerializationTestBooks* _tmp55_ = NULL;
		GomSerializationTestBooks* _tmp56_ = NULL;
		GXmlDomElement* _tmp57_ = NULL;
		GXmlDomElement* _tmp58_ = NULL;
		GXmlDomElement* _tmp59_ = NULL;
		GXmlDomElement* _tmp60_ = NULL;
		GomSerializationTestBookStand* _tmp61_ = NULL;
		GomSerializationTestBooks* _tmp62_ = NULL;
		GomSerializationTestBooks* _tmp63_ = NULL;
		GXmlDomElement* _tmp64_ = NULL;
		GXmlDomElement* _tmp65_ = NULL;
		GXmlDomElement* _tmp66_ = NULL;
		GomSerializationTestBookStand* _tmp67_ = NULL;
		GomSerializationTestBooks* _tmp68_ = NULL;
		GomSerializationTestBooks* _tmp69_ = NULL;
		GXmlDomElement* _tmp70_ = NULL;
		GXmlDomElement* _tmp71_ = NULL;
		GXmlDomElement* _tmp72_ = NULL;
		GomSerializationTestBookStand* _tmp73_ = NULL;
		GomSerializationTestBooks* _tmp74_ = NULL;
		GomSerializationTestBooks* _tmp75_ = NULL;
		GXmlDomElement* _tmp76_ = NULL;
		GXmlDomElement* _tmp77_ = NULL;
		GXmlDomElement* _tmp78_ = NULL;
		GXmlDomElement* _tmp79_ = NULL;
		GomSerializationTestBookStand* _tmp80_ = NULL;
		GomSerializationTestBooks* _tmp81_ = NULL;
		GomSerializationTestBooks* _tmp82_ = NULL;
		GXmlDomElement* _tmp83_ = NULL;
		GXmlDomElement* _tmp84_ = NULL;
		GXmlDomElement* _tmp85_ = NULL;
		GomSerializationTestBookStand* _tmp86_ = NULL;
		GomSerializationTestBooks* _tmp87_ = NULL;
		GomSerializationTestBooks* _tmp88_ = NULL;
		GXmlDomElement* _tmp89_ = NULL;
		GXmlDomElement* _tmp90_ = NULL;
		GXmlDomElement* _tmp91_ = NULL;
		GomSerializationTestBookStand* _tmp92_ = NULL;
		GomSerializationTestBooks* _tmp93_ = NULL;
		GomSerializationTestBooks* _tmp94_ = NULL;
		GXmlDomElement* _tmp95_ = NULL;
		GXmlDomElement* _tmp96_ = NULL;
		GomSerializationTestBook* _tmp97_ = NULL;
		GomSerializationTestBook* _tmp98_ = NULL;
		const gchar* _tmp99_ = NULL;
		const gchar* _tmp100_ = NULL;
		GXmlDomElement* _tmp101_ = NULL;
		GomSerializationTestBookStand* _tmp102_ = NULL;
		GomSerializationTestBooks* _tmp103_ = NULL;
		GomSerializationTestBooks* _tmp104_ = NULL;
		GXmlDomElement* _tmp105_ = NULL;
		GXmlDomElement* _tmp106_ = NULL;
		GomSerializationTestBook* _tmp107_ = NULL;
		GomSerializationTestBook* _tmp108_ = NULL;
		const gchar* _tmp109_ = NULL;
		const gchar* _tmp110_ = NULL;
		GXmlDomElement* _tmp111_ = NULL;
		GomSerializationTestBookStand* _tmp112_ = NULL;
		GomSerializationTestBooks* _tmp113_ = NULL;
		GomSerializationTestBooks* _tmp114_ = NULL;
		GXmlDomElement* _tmp115_ = NULL;
		GXmlDomElement* _tmp116_ = NULL;
		GomSerializationTestBook* _tmp117_ = NULL;
		GomSerializationTestBook* _tmp118_ = NULL;
		const gchar* _tmp119_ = NULL;
		const gchar* _tmp120_ = NULL;
		GomSerializationTestBookStand* _tmp121_ = NULL;
		GomSerializationTestBooks* _tmp122_ = NULL;
		GomSerializationTestBooks* _tmp123_ = NULL;
		GXmlDomElement* _tmp124_ = NULL;
		GXmlDomElement* _tmp125_ = NULL;
		GomSerializationTestBookStand* _tmp126_ = NULL;
		GomSerializationTestBooks* _tmp127_ = NULL;
		GomSerializationTestBooks* _tmp128_ = NULL;
		GXmlDomElement* _tmp129_ = NULL;
		GXmlDomElement* _tmp130_ = NULL;
		GomSerializationTestBookStand* _tmp131_ = NULL;
		GomSerializationTestBooks* _tmp132_ = NULL;
		GomSerializationTestBooks* _tmp133_ = NULL;
		GXmlDomElement* _tmp134_ = NULL;
		GXmlDomElement* _tmp135_ = NULL;
		GomSerializationTestBookStand* _tmp136_ = NULL;
		GomSerializationTestBooks* _tmp137_ = NULL;
		GomSerializationTestBooks* _tmp138_ = NULL;
		GXmlDomElement* _tmp139_ = NULL;
		GXmlDomElement* _tmp140_ = NULL;
		GomSerializationTestBookStand* _tmp141_ = NULL;
		GomSerializationTestBooks* _tmp142_ = NULL;
		GomSerializationTestBooks* _tmp143_ = NULL;
		GXmlDomElement* _tmp144_ = NULL;
		GXmlDomElement* _tmp145_ = NULL;
		GomSerializationTestBookStand* _tmp146_ = NULL;
		GomSerializationTestBooks* _tmp147_ = NULL;
		GomSerializationTestBooks* _tmp148_ = NULL;
		GXmlDomElement* _tmp149_ = NULL;
		GXmlDomElement* _tmp150_ = NULL;
		GomSerializationTestBookStand* _tmp151_ = NULL;
		GomSerializationTestBooks* _tmp152_ = NULL;
		GomSerializationTestBooks* _tmp153_ = NULL;
		GXmlDomElement* _tmp154_ = NULL;
		GXmlDomElement* _tmp155_ = NULL;
		GomSerializationTestBookStand* _tmp156_ = NULL;
		GomSerializationTestBooks* _tmp157_ = NULL;
		GomSerializationTestBooks* _tmp158_ = NULL;
		GXmlDomElement* _tmp159_ = NULL;
		GXmlDomElement* _tmp160_ = NULL;
		GomSerializationTestBookStand* _tmp161_ = NULL;
		GomSerializationTestBooks* _tmp162_ = NULL;
		GomSerializationTestBooks* _tmp163_ = NULL;
		GXmlDomElement* _tmp164_ = NULL;
		GXmlDomElement* _tmp165_ = NULL;
		GomSerializationTestBookStand* _tmp166_ = NULL;
		GomSerializationTestBooks* _tmp167_ = NULL;
		GomSerializationTestBooks* _tmp168_ = NULL;
		GXmlDomElement* _tmp169_ = NULL;
		GXmlDomElement* _tmp170_ = NULL;
#line 970 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp0_ = gom_serialization_test_book_stand_new ();
#line 970 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		bs = _tmp0_;
#line 971 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp1_ = bs;
#line 971 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = gom_serialization_test_book_stand_to_string (_tmp1_);
#line 971 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp2_;
#line 975 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = s;
#line 975 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp4_ = string_contains (_tmp3_, "<BookStand Classification=\"Science\"/>");
#line 975 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp4_, "\"<BookStand Classification=\\\"Science\\\"/>\" in s");
#line 976 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp5_ = bs;
#line 976 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_read_from_string ((GXmlGomElement*) _tmp5_, "<bookStand Classification=\"Science\"><book name=\"Title1\"/><book nam" \
"e=\"Title2\"/><Test/><book name=\"Title3\"/></bookStand>", &_inner_error_);
#line 976 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 976 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 976 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 9362 "GomSerializationTest.c"
			goto __catch259_g_error;
		}
#line 978 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp6_ = bs;
#line 978 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp7_ = gom_serialization_test_book_stand_get_books (_tmp6_);
#line 978 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp8_ = _tmp7_;
#line 978 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp8_ != NULL, "bs.books != null");
#line 979 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp9_ = bs;
#line 979 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp10_ = gom_serialization_test_book_stand_to_string (_tmp9_);
#line 979 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 979 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp10_;
#line 984 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp11_ = bs;
#line 984 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp12_ = gom_serialization_test_book_stand_get_books (_tmp11_);
#line 984 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp13_ = _tmp12_;
#line 984 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp14_ = gxml_gom_collection_get_length ((GXmlGomCollection*) _tmp13_);
#line 984 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp15_ = _tmp14_;
#line 984 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp15_ == 3, "bs.books.length == 3");
#line 985 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp16_ = bs;
#line 985 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp17_ = gom_serialization_test_book_stand_get_books (_tmp16_);
#line 985 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp18_ = _tmp17_;
#line 985 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp19_ = gxml_gom_collection_get_nodes_index ((GXmlGomCollection*) _tmp18_);
#line 985 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp20_ = _tmp19_;
#line 985 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp21_ = g_queue_peek_nth (_tmp20_, (guint) 0);
#line 985 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (((gint) ((gintptr) _tmp21_)) == 0, "bs.books.nodes_index.peek_nth (0) == 0");
#line 986 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp22_ = bs;
#line 986 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp23_ = gom_serialization_test_book_stand_get_books (_tmp22_);
#line 986 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp24_ = _tmp23_;
#line 986 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp25_ = gxml_gom_collection_get_nodes_index ((GXmlGomCollection*) _tmp24_);
#line 986 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp26_ = _tmp25_;
#line 986 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp27_ = g_queue_peek_nth (_tmp26_, (guint) 1);
#line 986 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (((gint) ((gintptr) _tmp27_)) == 1, "bs.books.nodes_index.peek_nth (1) == 1");
#line 987 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp28_ = bs;
#line 987 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp29_ = gom_serialization_test_book_stand_get_books (_tmp28_);
#line 987 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp30_ = _tmp29_;
#line 987 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp31_ = gxml_gom_collection_get_nodes_index ((GXmlGomCollection*) _tmp30_);
#line 987 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp32_ = _tmp31_;
#line 987 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp33_ = g_queue_peek_nth (_tmp32_, (guint) 2);
#line 987 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (((gint) ((gintptr) _tmp33_)) == 3, "bs.books.nodes_index.peek_nth (2) == 3");
#line 988 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp35_ = bs;
#line 988 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp36_ = gom_serialization_test_book_stand_get_books (_tmp35_);
#line 988 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp37_ = _tmp36_;
#line 988 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp38_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp37_, 0, &_inner_error_);
#line 988 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp34_ = _tmp38_;
#line 988 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 988 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 988 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 9451 "GomSerializationTest.c"
			goto __catch259_g_error;
		}
#line 988 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp39_ = _tmp34_;
#line 988 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp34_ = NULL;
#line 988 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp40_ = _tmp39_;
#line 988 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp40_ != NULL, "bs.books.get_item (0) != null");
#line 988 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp40_);
#line 989 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp42_ = bs;
#line 989 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp43_ = gom_serialization_test_book_stand_get_books (_tmp42_);
#line 989 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp44_ = _tmp43_;
#line 989 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp45_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp44_, 0, &_inner_error_);
#line 989 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp41_ = _tmp45_;
#line 989 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 989 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp34_);
#line 989 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 989 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 9482 "GomSerializationTest.c"
			goto __catch259_g_error;
		}
#line 989 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp46_ = _tmp41_;
#line 989 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp41_ = NULL;
#line 989 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp46_, GXML_TYPE_DOM_ELEMENT), "bs.books.get_item (0) is DomElement");
#line 990 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp48_ = bs;
#line 990 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp49_ = gom_serialization_test_book_stand_get_books (_tmp48_);
#line 990 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp50_ = _tmp49_;
#line 990 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp51_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp50_, 0, &_inner_error_);
#line 990 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp47_ = _tmp51_;
#line 990 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 990 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp41_);
#line 990 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp34_);
#line 990 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 990 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 9511 "GomSerializationTest.c"
			goto __catch259_g_error;
		}
#line 990 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp52_ = _tmp47_;
#line 990 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp47_ = NULL;
#line 990 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp52_, GOM_SERIALIZATION_TEST_TYPE_BOOK), "bs.books.get_item (0) is Book");
#line 991 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp54_ = bs;
#line 991 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp55_ = gom_serialization_test_book_stand_get_books (_tmp54_);
#line 991 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp56_ = _tmp55_;
#line 991 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp57_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp56_, 1, &_inner_error_);
#line 991 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp53_ = _tmp57_;
#line 991 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 991 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp47_);
#line 991 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp41_);
#line 991 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp34_);
#line 991 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 991 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 9542 "GomSerializationTest.c"
			goto __catch259_g_error;
		}
#line 991 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp58_ = _tmp53_;
#line 991 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp53_ = NULL;
#line 991 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp59_ = _tmp58_;
#line 991 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp59_ != NULL, "bs.books.get_item (1) != null");
#line 991 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp59_);
#line 992 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp61_ = bs;
#line 992 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp62_ = gom_serialization_test_book_stand_get_books (_tmp61_);
#line 992 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp63_ = _tmp62_;
#line 992 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp64_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp63_, 1, &_inner_error_);
#line 992 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp60_ = _tmp64_;
#line 992 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 992 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp53_);
#line 992 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp47_);
#line 992 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp41_);
#line 992 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp34_);
#line 992 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 992 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 9579 "GomSerializationTest.c"
			goto __catch259_g_error;
		}
#line 992 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp65_ = _tmp60_;
#line 992 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp60_ = NULL;
#line 992 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp65_, GXML_TYPE_DOM_ELEMENT), "bs.books.get_item (1) is DomElement");
#line 993 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp67_ = bs;
#line 993 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp68_ = gom_serialization_test_book_stand_get_books (_tmp67_);
#line 993 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp69_ = _tmp68_;
#line 993 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp70_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp69_, 1, &_inner_error_);
#line 993 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp66_ = _tmp70_;
#line 993 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 993 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp60_);
#line 993 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp53_);
#line 993 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp47_);
#line 993 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp41_);
#line 993 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp34_);
#line 993 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 993 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 9614 "GomSerializationTest.c"
			goto __catch259_g_error;
		}
#line 993 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp71_ = _tmp66_;
#line 993 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp66_ = NULL;
#line 993 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp71_, GOM_SERIALIZATION_TEST_TYPE_BOOK), "bs.books.get_item (1) is Book");
#line 994 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp73_ = bs;
#line 994 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp74_ = gom_serialization_test_book_stand_get_books (_tmp73_);
#line 994 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp75_ = _tmp74_;
#line 994 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp76_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp75_, 2, &_inner_error_);
#line 994 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp72_ = _tmp76_;
#line 994 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 994 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp66_);
#line 994 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp60_);
#line 994 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp53_);
#line 994 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp47_);
#line 994 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp41_);
#line 994 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp34_);
#line 994 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 994 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 9651 "GomSerializationTest.c"
			goto __catch259_g_error;
		}
#line 994 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp77_ = _tmp72_;
#line 994 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp72_ = NULL;
#line 994 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp78_ = _tmp77_;
#line 994 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp78_ != NULL, "bs.books.get_item (2) != null");
#line 994 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp78_);
#line 995 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp80_ = bs;
#line 995 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp81_ = gom_serialization_test_book_stand_get_books (_tmp80_);
#line 995 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp82_ = _tmp81_;
#line 995 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp83_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp82_, 2, &_inner_error_);
#line 995 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp79_ = _tmp83_;
#line 995 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 995 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp72_);
#line 995 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp66_);
#line 995 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp60_);
#line 995 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp53_);
#line 995 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp47_);
#line 995 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp41_);
#line 995 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp34_);
#line 995 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 995 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 9694 "GomSerializationTest.c"
			goto __catch259_g_error;
		}
#line 995 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp84_ = _tmp79_;
#line 995 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp79_ = NULL;
#line 995 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp84_, GXML_TYPE_DOM_ELEMENT), "bs.books.get_item (2) is DomElement");
#line 996 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp86_ = bs;
#line 996 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp87_ = gom_serialization_test_book_stand_get_books (_tmp86_);
#line 996 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp88_ = _tmp87_;
#line 996 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp89_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp88_, 2, &_inner_error_);
#line 996 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp85_ = _tmp89_;
#line 996 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 996 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp79_);
#line 996 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp72_);
#line 996 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp66_);
#line 996 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp60_);
#line 996 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp53_);
#line 996 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp47_);
#line 996 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp41_);
#line 996 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp34_);
#line 996 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 996 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 9735 "GomSerializationTest.c"
			goto __catch259_g_error;
		}
#line 996 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp90_ = _tmp85_;
#line 996 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp85_ = NULL;
#line 996 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp90_, GOM_SERIALIZATION_TEST_TYPE_BOOK), "bs.books.get_item (2) is Book");
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp92_ = bs;
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp93_ = gom_serialization_test_book_stand_get_books (_tmp92_);
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp94_ = _tmp93_;
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp95_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp94_, 0, &_inner_error_);
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp91_ = _tmp95_;
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp85_);
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp79_);
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp72_);
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp66_);
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp60_);
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp53_);
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp47_);
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp41_);
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp34_);
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 9778 "GomSerializationTest.c"
			goto __catch259_g_error;
		}
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp96_ = _tmp91_;
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp91_ = NULL;
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp97_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp96_, GOM_SERIALIZATION_TEST_TYPE_BOOK) ? ((GomSerializationTestBook*) _tmp96_) : NULL;
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (_tmp97_ == NULL) {
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp96_);
#line 9791 "GomSerializationTest.c"
		}
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp98_ = _tmp97_;
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp99_ = gom_serialization_test_book_get_name (_tmp98_);
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp100_ = _tmp99_;
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (g_strcmp0 (_tmp100_, "Title1") == 0, "(bs.books.get_item (0) as Book).name == \"Title1\"");
#line 997 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp98_);
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp102_ = bs;
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp103_ = gom_serialization_test_book_stand_get_books (_tmp102_);
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp104_ = _tmp103_;
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp105_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp104_, 1, &_inner_error_);
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp101_ = _tmp105_;
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp91_);
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp85_);
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp79_);
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp72_);
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp66_);
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp60_);
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp53_);
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp47_);
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp41_);
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp34_);
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 9839 "GomSerializationTest.c"
			goto __catch259_g_error;
		}
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp106_ = _tmp101_;
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp101_ = NULL;
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp107_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp106_, GOM_SERIALIZATION_TEST_TYPE_BOOK) ? ((GomSerializationTestBook*) _tmp106_) : NULL;
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (_tmp107_ == NULL) {
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp106_);
#line 9852 "GomSerializationTest.c"
		}
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp108_ = _tmp107_;
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp109_ = gom_serialization_test_book_get_name (_tmp108_);
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp110_ = _tmp109_;
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (g_strcmp0 (_tmp110_, "Title2") == 0, "(bs.books.get_item (1) as Book).name == \"Title2\"");
#line 998 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp108_);
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp112_ = bs;
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp113_ = gom_serialization_test_book_stand_get_books (_tmp112_);
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp114_ = _tmp113_;
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp115_ = gxml_gom_collection_get_item ((GXmlGomCollection*) _tmp114_, 2, &_inner_error_);
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp111_ = _tmp115_;
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp101_);
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp91_);
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp85_);
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp79_);
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp72_);
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp66_);
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp60_);
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp53_);
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp47_);
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp41_);
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp34_);
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_free0 (s);
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 9902 "GomSerializationTest.c"
			goto __catch259_g_error;
		}
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp116_ = _tmp111_;
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp111_ = NULL;
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp117_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp116_, GOM_SERIALIZATION_TEST_TYPE_BOOK) ? ((GomSerializationTestBook*) _tmp116_) : NULL;
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (_tmp117_ == NULL) {
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp116_);
#line 9915 "GomSerializationTest.c"
		}
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp118_ = _tmp117_;
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp119_ = gom_serialization_test_book_get_name (_tmp118_);
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp120_ = _tmp119_;
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (g_strcmp0 (_tmp120_, "Title3") == 0, "(bs.books.get_item (2) as Book).name == \"Title3\"");
#line 999 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp118_);
#line 1000 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp121_ = bs;
#line 1000 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp122_ = gom_serialization_test_book_stand_get_books (_tmp121_);
#line 1000 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp123_ = _tmp122_;
#line 1000 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp124_ = gxml_gom_hash_map_get ((GXmlGomHashMap*) _tmp123_, "Title1");
#line 1000 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp125_ = _tmp124_;
#line 1000 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp125_ != NULL, "bs.books.get (\"Title1\") != null");
#line 1000 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp125_);
#line 1001 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp126_ = bs;
#line 1001 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp127_ = gom_serialization_test_book_stand_get_books (_tmp126_);
#line 1001 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp128_ = _tmp127_;
#line 1001 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp129_ = gxml_gom_hash_map_get ((GXmlGomHashMap*) _tmp128_, "Title1");
#line 1001 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp130_ = _tmp129_;
#line 1001 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp130_, GXML_TYPE_DOM_ELEMENT), "bs.books.get (\"Title1\") is DomElement");
#line 1001 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp130_);
#line 1002 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp131_ = bs;
#line 1002 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp132_ = gom_serialization_test_book_stand_get_books (_tmp131_);
#line 1002 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp133_ = _tmp132_;
#line 1002 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp134_ = gxml_gom_hash_map_get ((GXmlGomHashMap*) _tmp133_, "Title1");
#line 1002 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp135_ = _tmp134_;
#line 1002 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp135_, GOM_SERIALIZATION_TEST_TYPE_BOOK), "bs.books.get (\"Title1\") is Book");
#line 1002 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp135_);
#line 1003 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp136_ = bs;
#line 1003 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp137_ = gom_serialization_test_book_stand_get_books (_tmp136_);
#line 1003 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp138_ = _tmp137_;
#line 1003 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp139_ = gxml_gom_hash_map_get ((GXmlGomHashMap*) _tmp138_, "Title2");
#line 1003 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp140_ = _tmp139_;
#line 1003 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp140_ != NULL, "bs.books.get (\"Title2\") != null");
#line 1003 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp140_);
#line 1004 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp141_ = bs;
#line 1004 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp142_ = gom_serialization_test_book_stand_get_books (_tmp141_);
#line 1004 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp143_ = _tmp142_;
#line 1004 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp144_ = gxml_gom_hash_map_get ((GXmlGomHashMap*) _tmp143_, "Title2");
#line 1004 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp145_ = _tmp144_;
#line 1004 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp145_, GXML_TYPE_DOM_ELEMENT), "bs.books.get (\"Title2\") is DomElement");
#line 1004 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp145_);
#line 1005 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp146_ = bs;
#line 1005 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp147_ = gom_serialization_test_book_stand_get_books (_tmp146_);
#line 1005 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp148_ = _tmp147_;
#line 1005 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp149_ = gxml_gom_hash_map_get ((GXmlGomHashMap*) _tmp148_, "Title2");
#line 1005 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp150_ = _tmp149_;
#line 1005 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp150_, GOM_SERIALIZATION_TEST_TYPE_BOOK), "bs.books.get (\"Title2\") is Book");
#line 1005 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp150_);
#line 1006 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp151_ = bs;
#line 1006 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp152_ = gom_serialization_test_book_stand_get_books (_tmp151_);
#line 1006 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp153_ = _tmp152_;
#line 1006 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp154_ = gxml_gom_hash_map_get ((GXmlGomHashMap*) _tmp153_, "Title3");
#line 1006 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp155_ = _tmp154_;
#line 1006 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp155_ != NULL, "bs.books.get (\"Title3\") != null");
#line 1006 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp155_);
#line 1007 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp156_ = bs;
#line 1007 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp157_ = gom_serialization_test_book_stand_get_books (_tmp156_);
#line 1007 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp158_ = _tmp157_;
#line 1007 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp159_ = gxml_gom_hash_map_get ((GXmlGomHashMap*) _tmp158_, "Title3");
#line 1007 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp160_ = _tmp159_;
#line 1007 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp160_, GXML_TYPE_DOM_ELEMENT), "bs.books.get (\"Title3\") is DomElement");
#line 1007 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp160_);
#line 1008 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp161_ = bs;
#line 1008 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp162_ = gom_serialization_test_book_stand_get_books (_tmp161_);
#line 1008 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp163_ = _tmp162_;
#line 1008 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp164_ = gxml_gom_hash_map_get ((GXmlGomHashMap*) _tmp163_, "Title3");
#line 1008 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp165_ = _tmp164_;
#line 1008 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp165_, GOM_SERIALIZATION_TEST_TYPE_BOOK), "bs.books.get (\"Title3\") is Book");
#line 1008 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp165_);
#line 1009 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp166_ = bs;
#line 1009 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp167_ = gom_serialization_test_book_stand_get_books (_tmp166_);
#line 1009 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp168_ = _tmp167_;
#line 1009 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp169_ = gxml_gom_hash_map_get ((GXmlGomHashMap*) _tmp168_, "Title4");
#line 1009 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp170_ = _tmp169_;
#line 1009 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp170_ == NULL, "bs.books.get (\"Title4\") == null");
#line 1009 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp170_);
#line 969 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp111_);
#line 969 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp101_);
#line 969 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp91_);
#line 969 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp85_);
#line 969 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp79_);
#line 969 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp72_);
#line 969 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp66_);
#line 969 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp60_);
#line 969 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp53_);
#line 969 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp47_);
#line 969 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp41_);
#line 969 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp34_);
#line 969 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 969 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (bs);
#line 10095 "GomSerializationTest.c"
	}
	goto __finally259;
	__catch259_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp171_ = NULL;
		gchar* _tmp172_ = NULL;
		gchar* _tmp173_ = NULL;
#line 969 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		e = _inner_error_;
#line 969 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 1011 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp171_ = e->message;
#line 1011 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp172_ = g_strconcat ("Error: ", _tmp171_, NULL);
#line 1011 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp173_ = _tmp172_;
#line 1011 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:1011: %s", _tmp173_);
#line 1011 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp173_);
#line 1012 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 969 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_error_free0 (e);
#line 10122 "GomSerializationTest.c"
	}
	__finally259:
#line 969 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 969 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 969 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 969 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return;
#line 10133 "GomSerializationTest.c"
	}
}


static void ___lambda257__gtest_func (void) {
#line 968 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	__lambda257_ ();
#line 10141 "GomSerializationTest.c"
}


static void __lambda258_ (void) {
	GError * _inner_error_ = NULL;
	{
		gdouble time = 0.0;
		GFile* f = NULL;
		GFile* _tmp0_ = NULL;
		GFile* _tmp1_ = NULL;
		gboolean _tmp2_ = FALSE;
		GomBookStore* bs = NULL;
		GomBookStore* _tmp3_ = NULL;
		GomBookStore* _tmp4_ = NULL;
		GomBookStore* _tmp5_ = NULL;
		GFile* _tmp6_ = NULL;
		GomBookStore* _tmp7_ = NULL;
		GomBookArray* _tmp8_ = NULL;
		GomBookArray* _tmp9_ = NULL;
		GomBookStore* _tmp10_ = NULL;
		GomBookArray* _tmp11_ = NULL;
		GomBookArray* _tmp12_ = NULL;
		GXmlGomElement* _tmp13_ = NULL;
		GXmlGomElement* _tmp14_ = NULL;
		GomBookStore* _tmp15_ = NULL;
		GomBookArray* _tmp16_ = NULL;
		GomBookArray* _tmp17_ = NULL;
		GType _tmp18_ = 0UL;
		GType _tmp19_ = 0UL;
		gboolean _tmp20_ = FALSE;
		GomBookStore* _tmp21_ = NULL;
		GomBookArray* _tmp22_ = NULL;
		GomBookArray* _tmp23_ = NULL;
		const gchar* _tmp24_ = NULL;
		const gchar* _tmp25_ = NULL;
		GomBookStore* _tmp26_ = NULL;
		gchar* _tmp27_ = NULL;
		gchar* _tmp28_ = NULL;
		gchar* _tmp29_ = NULL;
		GomBookStore* _tmp30_ = NULL;
		gchar* _tmp31_ = NULL;
		gchar* _tmp32_ = NULL;
		GomBookStore* _tmp33_ = NULL;
		const gchar* _tmp34_ = NULL;
		const gchar* _tmp35_ = NULL;
		GomBookStore* _tmp36_ = NULL;
		const gchar* _tmp37_ = NULL;
		const gchar* _tmp38_ = NULL;
		gdouble _tmp39_ = 0.0;
		gdouble _tmp40_ = 0.0;
		gdouble _tmp41_ = 0.0;
		GFile* of = NULL;
		GFile* _tmp42_ = NULL;
		GomBookStore* _tmp43_ = NULL;
		GXmlDomDocument* _tmp44_ = NULL;
		GXmlDomDocument* _tmp45_ = NULL;
		GXmlDomElement* _tmp46_ = NULL;
		GXmlDomElement* _tmp47_ = NULL;
		GXmlDomElement* _tmp48_ = NULL;
		gchar* _tmp49_ = NULL;
		gchar* _tmp50_ = NULL;
		gchar* _tmp51_ = NULL;
		gchar* _tmp52_ = NULL;
		gchar* _tmp53_ = NULL;
		GomBookStore* _tmp54_ = NULL;
		GXmlDomNodeList* _tmp55_ = NULL;
		GXmlDomNodeList* _tmp56_ = NULL;
		GXmlDomNodeList* _tmp57_ = NULL;
		gint _tmp58_ = 0;
		gint _tmp59_ = 0;
		GXmlDomHTMLCollection* ns = NULL;
		GomBookStore* _tmp60_ = NULL;
		GXmlDomHTMLCollection* _tmp61_ = NULL;
		GXmlDomHTMLCollection* _tmp62_ = NULL;
		gint _tmp63_ = 0;
		gint _tmp64_ = 0;
		GomBookStore* _tmp65_ = NULL;
		GFile* _tmp66_ = NULL;
		gdouble _tmp67_ = 0.0;
		gdouble _tmp68_ = 0.0;
		gdouble _tmp69_ = 0.0;
		GFile* _tmp70_ = NULL;
		gboolean _tmp71_ = FALSE;
#line 1019 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp0_ = g_file_new_for_path (TEST_DIR "/test-collection.xml");
#line 1019 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		f = _tmp0_;
#line 1020 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp1_ = f;
#line 1020 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = g_file_query_exists (_tmp1_, NULL);
#line 1020 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp2_, "f.query_exists ()");
#line 1021 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_test_timer_start ();
#line 1022 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = gom_book_store_new ();
#line 1022 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		bs = _tmp3_;
#line 1023 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp4_ = bs;
#line 1023 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp4_ != NULL, "bs != null");
#line 1027 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp5_ = bs;
#line 1027 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp6_ = f;
#line 1027 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_read_from_file ((GXmlGomElement*) _tmp5_, _tmp6_, NULL, &_inner_error_);
#line 1027 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1027 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 1027 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (f);
#line 10257 "GomSerializationTest.c"
			goto __catch260_g_error;
		}
#line 1028 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp7_ = bs;
#line 1028 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp8_ = gom_book_store_get_books (_tmp7_);
#line 1028 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp9_ = _tmp8_;
#line 1028 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp9_ != NULL, "bs.books != null");
#line 1029 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp10_ = bs;
#line 1029 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp11_ = gom_book_store_get_books (_tmp10_);
#line 1029 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp12_ = _tmp11_;
#line 1029 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp13_ = gxml_gom_collection_get_element ((GXmlGomCollection*) _tmp12_);
#line 1029 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp14_ = _tmp13_;
#line 1029 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp14_ != NULL, "bs.books.element != null");
#line 1030 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp15_ = bs;
#line 1030 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp16_ = gom_book_store_get_books (_tmp15_);
#line 1030 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp17_ = _tmp16_;
#line 1030 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp18_ = gxml_gom_collection_get_items_type ((GXmlGomCollection*) _tmp17_);
#line 1030 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp19_ = _tmp18_;
#line 1030 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp20_ = g_type_is_a (_tmp19_, TYPE_GOM_BOOK);
#line 1030 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp20_, "bs.books.items_type.is_a (typeof(GomBook))");
#line 1031 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp21_ = bs;
#line 1031 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp22_ = gom_book_store_get_books (_tmp21_);
#line 1031 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp23_ = _tmp22_;
#line 1031 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp24_ = gxml_gom_collection_get_items_name ((GXmlGomCollection*) _tmp23_);
#line 1031 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp25_ = _tmp24_;
#line 1031 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (g_strcmp0 (_tmp25_, "Book") == 0, "bs.books.items_name == \"Book\"");
#line 1032 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp26_ = bs;
#line 1032 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp27_ = gxml_dom_element_get_local_name ((GXmlDomElement*) _tmp26_);
#line 1032 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp28_ = _tmp27_;
#line 1032 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp29_ = _tmp28_;
#line 1032 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (g_strcmp0 (_tmp29_, "BookStore") == 0, "bs.local_name == \"BookStore\"");
#line 1032 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp29_);
#line 1033 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp30_ = bs;
#line 1033 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp31_ = gxml_dom_element_get_attribute ((GXmlDomElement*) _tmp30_, "name");
#line 1033 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp32_ = _tmp31_;
#line 1033 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp32_ != NULL, "bs.get_attribute (\"name\") != null");
#line 1033 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp32_);
#line 1034 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp33_ = bs;
#line 1034 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp34_ = gom_book_store_get_name (_tmp33_);
#line 1034 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp35_ = _tmp34_;
#line 1034 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp35_ != NULL, "bs.name != null");
#line 1035 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp36_ = bs;
#line 1035 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp37_ = gom_book_store_get_name (_tmp36_);
#line 1035 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp38_ = _tmp37_;
#line 1035 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (g_strcmp0 (_tmp38_, "The Great Book") == 0, "bs.name == \"The Great Book\"");
#line 1036 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp39_ = g_test_timer_elapsed ();
#line 1036 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		time = _tmp39_;
#line 1037 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp40_ = time;
#line 1037 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp41_ = time;
#line 1037 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_test_minimized_result (_tmp40_, "deserialize/performance: %g seconds", _tmp41_);
#line 1041 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp42_ = g_file_new_for_path (TEST_SAVE_DIR "/test-large-new.xml");
#line 1041 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		of = _tmp42_;
#line 1042 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_test_timer_start ();
#line 1047 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp43_ = bs;
#line 1047 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp44_ = gxml_dom_node_get_owner_document ((GXmlDomNode*) _tmp43_);
#line 1047 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp45_ = _tmp44_;
#line 1047 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp46_ = gxml_dom_document_get_document_element (_tmp45_);
#line 1047 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp47_ = _tmp46_;
#line 1047 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp48_ = _tmp47_;
#line 1047 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp49_ = gxml_dom_node_get_node_name ((GXmlDomNode*) _tmp48_);
#line 1047 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp50_ = _tmp49_;
#line 1047 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp51_ = _tmp50_;
#line 1047 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp52_ = g_utf8_strdown (_tmp51_, (gssize) -1);
#line 1047 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp53_ = _tmp52_;
#line 1047 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (g_strcmp0 (_tmp53_, "bookstore") == 0, "bs.owner_document.document_element.node_name.down () == \"bookstore\"");
#line 1047 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp53_);
#line 1047 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp51_);
#line 1047 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp48_);
#line 1051 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp54_ = bs;
#line 1051 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp55_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp54_);
#line 1051 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp56_ = _tmp55_;
#line 1051 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp57_ = _tmp56_;
#line 1051 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp58_ = gxml_dom_node_list_get_length (_tmp57_);
#line 1051 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp59_ = _tmp58_;
#line 1051 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp59_ == 7, "bs.child_nodes.length == 7");
#line 1051 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp57_);
#line 1052 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp60_ = bs;
#line 1052 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp61_ = gxml_dom_element_get_elements_by_tag_name ((GXmlDomElement*) _tmp60_, "Book");
#line 1052 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		ns = _tmp61_;
#line 1057 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp62_ = ns;
#line 1057 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp63_ = gxml_dom_html_collection_get_length (_tmp62_);
#line 1057 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp64_ = _tmp63_;
#line 1057 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp64_ == 3, "ns.length == 3");
#line 1062 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp65_ = bs;
#line 1062 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp66_ = of;
#line 1062 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_write_file ((GXmlGomElement*) _tmp65_, _tmp66_, &_inner_error_);
#line 1062 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1062 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (ns);
#line 1062 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (of);
#line 1062 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 1062 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (f);
#line 10436 "GomSerializationTest.c"
			goto __catch260_g_error;
		}
#line 1063 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp67_ = g_test_timer_elapsed ();
#line 1063 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		time = _tmp67_;
#line 1064 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp68_ = time;
#line 1064 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp69_ = time;
#line 1064 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_test_minimized_result (_tmp68_, "Serialize/performance: %g seconds", _tmp69_);
#line 1065 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp70_ = of;
#line 1065 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp71_ = g_file_query_exists (_tmp70_, NULL);
#line 1065 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_vala_assert (_tmp71_, "of.query_exists ()");
#line 10455 "GomSerializationTest.c"
		{
			GFile* _tmp72_ = NULL;
#line 1066 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_tmp72_ = of;
#line 1066 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			g_file_delete (_tmp72_, NULL, &_inner_error_);
#line 1066 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 10464 "GomSerializationTest.c"
				goto __catch261_g_error;
			}
		}
		goto __finally261;
		__catch261_g_error:
		{
#line 1066 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			g_clear_error (&_inner_error_);
#line 1066 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_inner_error_ = NULL;
#line 1066 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			g_assert_not_reached ();
#line 10477 "GomSerializationTest.c"
		}
		__finally261:
#line 1066 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1066 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (ns);
#line 1066 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (of);
#line 1066 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (bs);
#line 1066 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (f);
#line 10490 "GomSerializationTest.c"
			goto __catch260_g_error;
		}
#line 1017 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (ns);
#line 1017 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (of);
#line 1017 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (bs);
#line 1017 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (f);
#line 10501 "GomSerializationTest.c"
	}
	goto __finally260;
	__catch260_g_error:
	{
		GError* e = NULL;
#line 1017 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		e = _inner_error_;
#line 1017 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 1072 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 1017 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_error_free0 (e);
#line 10515 "GomSerializationTest.c"
	}
	__finally260:
#line 1017 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1017 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 1017 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 1017 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return;
#line 10526 "GomSerializationTest.c"
	}
}


static void ___lambda258__gtest_func (void) {
#line 1015 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	__lambda258_ ();
#line 10534 "GomSerializationTest.c"
}


void gom_serialization_test_add_tests (void) {
#line 407 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_test_add_func ("/gxml/gom-serialization/write/properties", ___lambda241__gtest_func);
#line 425 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_test_add_func ("/gxml/gom-serialization/write/property-ignore", ___lambda242__gtest_func);
#line 441 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_test_add_func ("/gxml/gom-serialization/write/property-long-name", ___lambda243__gtest_func);
#line 470 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_test_add_func ("/gxml/gom-serialization/write/property-date", ___lambda244__gtest_func);
#line 501 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_test_add_func ("/gxml/gom-serialization/read/property-date", ___lambda245__gtest_func);
#line 523 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_test_add_func ("/gxml/gom-serialization/write/property-datetime", ___lambda246__gtest_func);
#line 556 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_test_add_func ("/gxml/gom-serialization/write/property-arraylist", ___lambda247__gtest_func);
#line 636 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_test_add_func ("/gxml/gom-serialization/write/property-hashmap", ___lambda248__gtest_func);
#line 691 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_test_add_func ("/gxml/gom-serialization/mappeable", ___lambda249__gtest_func);
#line 718 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_test_add_func ("/gxml/gom-serialization/write/mappeablepairedkey", ___lambda250__gtest_func);
#line 740 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_test_add_func ("/gxml/gom-serialization/write/gom-property", ___lambda251__gtest_func);
#line 825 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_test_add_func ("/gxml/gom-serialization/read/properties", ___lambda252__gtest_func);
#line 859 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_test_add_func ("/gxml/gom-serialization/read/bad-node-name", ___lambda253__gtest_func);
#line 876 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_test_add_func ("/gxml/gom-serialization/read/object-property", ___lambda254__gtest_func);
#line 895 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_test_add_func ("/gxml/gom-serialization/read/gom-property", ___lambda255__gtest_func);
#line 931 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_test_add_func ("/gxml/gom-serialization/read/property-arraylist", ___lambda256__gtest_func);
#line 968 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_test_add_func ("/gxml/gom-serialization/read/property-hashmap", ___lambda257__gtest_func);
#line 1015 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_test_add_func ("/gxml/gom-serialization/multiple-child-collections", ___lambda258__gtest_func);
#line 10575 "GomSerializationTest.c"
}


GomSerializationTest* gom_serialization_test_construct (GType object_type) {
	GomSerializationTest* self = NULL;
#line 133 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTest*) gxml_test_construct (object_type);
#line 133 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 10585 "GomSerializationTest.c"
}


GomSerializationTest* gom_serialization_test_new (void) {
#line 133 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_construct (TYPE_GOM_SERIALIZATION_TEST);
#line 10592 "GomSerializationTest.c"
}


GomSerializationTestBook* gom_serialization_test_book_construct_document (GType object_type, GXmlDomDocument* doc) {
	GomSerializationTestBook * self = NULL;
	GXmlDomDocument* _tmp0_ = NULL;
	GXmlDomDocument* _tmp1_ = NULL;
#line 138 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (doc != NULL, NULL);
#line 138 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestBook*) gxml_gom_element_construct (object_type);
#line 139 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = doc;
#line 139 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 139 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (((GXmlGomNode*) self)->_document);
#line 139 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	((GXmlGomNode*) self)->_document = _tmp1_;
#line 138 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 10614 "GomSerializationTest.c"
}


GomSerializationTestBook* gom_serialization_test_book_new_document (GXmlDomDocument* doc) {
#line 138 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_book_construct_document (GOM_SERIALIZATION_TEST_TYPE_BOOK, doc);
#line 10621 "GomSerializationTest.c"
}


gchar* gom_serialization_test_book_to_string (GomSerializationTestBook* self) {
	gchar* result = NULL;
	GXmlXParser* parser = NULL;
	GXmlXParser* _tmp0_ = NULL;
	gchar* s = NULL;
	gchar* _tmp1_ = NULL;
	GError * _inner_error_ = NULL;
#line 141 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 142 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = gxml_xparser_new ((GXmlDomNode*) self);
#line 142 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parser = _tmp0_;
#line 143 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup ("");
#line 143 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp1_;
#line 10642 "GomSerializationTest.c"
	{
		gchar* _tmp2_ = NULL;
		gchar* _tmp3_ = NULL;
		gchar* _tmp4_ = NULL;
#line 145 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = gxml_parser_write_string ((GXmlParser*) parser, &_inner_error_);
#line 145 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = _tmp3_;
#line 145 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 10653 "GomSerializationTest.c"
			goto __catch262_g_error;
		}
#line 145 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp4_ = _tmp2_;
#line 145 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = NULL;
#line 145 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 145 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp4_;
#line 144 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp2_);
#line 10666 "GomSerializationTest.c"
	}
	goto __finally262;
	__catch262_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp5_ = NULL;
		gchar* _tmp6_ = NULL;
		gchar* _tmp7_ = NULL;
#line 144 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		e = _inner_error_;
#line 144 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 147 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp5_ = e->message;
#line 147 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp6_ = g_strconcat ("Error: ", _tmp5_, NULL);
#line 147 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp7_ = _tmp6_;
#line 147 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:147: %s", _tmp7_);
#line 147 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp7_);
#line 148 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 144 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_error_free0 (e);
#line 10693 "GomSerializationTest.c"
	}
	__finally262:
#line 144 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 144 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 144 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (parser);
#line 144 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 144 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 144 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return NULL;
#line 10708 "GomSerializationTest.c"
	}
#line 150 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = s;
#line 150 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (parser);
#line 150 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 10716 "GomSerializationTest.c"
}


GomSerializationTestBook* gom_serialization_test_book_construct (GType object_type) {
	GomSerializationTestBook * self = NULL;
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestBook*) gxml_gom_element_construct (object_type);
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 10726 "GomSerializationTest.c"
}


GomSerializationTestBook* gom_serialization_test_book_new (void) {
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_book_construct (GOM_SERIALIZATION_TEST_TYPE_BOOK);
#line 10733 "GomSerializationTest.c"
}


const gchar* gom_serialization_test_book_get_name (GomSerializationTestBook* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 136 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 136 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_name;
#line 136 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 136 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 10748 "GomSerializationTest.c"
}


void gom_serialization_test_book_set_name (GomSerializationTestBook* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 136 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 136 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 136 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 136 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_name);
#line 136 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_name = _tmp1_;
#line 136 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 10767 "GomSerializationTest.c"
}


static GObject * gom_serialization_test_book_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomSerializationTestBook * self;
	GError * _inner_error_ = NULL;
#line 137 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_serialization_test_book_parent_class);
#line 137 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 137 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_TYPE_BOOK, GomSerializationTestBook);
#line 10782 "GomSerializationTest.c"
	{
#line 137 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_initialize ((GXmlGomElement*) self, "Book");
#line 10786 "GomSerializationTest.c"
	}
	goto __finally263;
	__catch263_g_error:
	{
#line 137 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 137 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 137 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 10797 "GomSerializationTest.c"
	}
	__finally263:
#line 137 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 137 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 137 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 10806 "GomSerializationTest.c"
	}
#line 137 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 10810 "GomSerializationTest.c"
}


static void gom_serialization_test_book_class_init (GomSerializationTestBookClass * klass) {
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_book_parent_class = g_type_class_peek_parent (klass);
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_type_class_add_private (klass, sizeof (GomSerializationTestBookPrivate));
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_gom_serialization_test_book_get_property;
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_gom_serialization_test_book_set_property;
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_serialization_test_book_constructor;
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->finalize = gom_serialization_test_book_finalize;
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_BOOK_NAME, g_param_spec_string ("name", "::Name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 10829 "GomSerializationTest.c"
}


static void gom_serialization_test_book_instance_init (GomSerializationTestBook * self) {
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv = GOM_SERIALIZATION_TEST_BOOK_GET_PRIVATE (self);
#line 10836 "GomSerializationTest.c"
}


static void gom_serialization_test_book_finalize (GObject* obj) {
	GomSerializationTestBook * self;
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_TYPE_BOOK, GomSerializationTestBook);
#line 136 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_name);
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (gom_serialization_test_book_parent_class)->finalize (obj);
#line 10848 "GomSerializationTest.c"
}


GType gom_serialization_test_book_get_type (void) {
	static volatile gsize gom_serialization_test_book_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_book_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSerializationTestBookClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_serialization_test_book_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSerializationTestBook), 0, (GInstanceInitFunc) gom_serialization_test_book_instance_init, NULL };
		GType gom_serialization_test_book_type_id;
		gom_serialization_test_book_type_id = g_type_register_static (GXML_TYPE_GOM_ELEMENT, "GomSerializationTestBook", &g_define_type_info, 0);
		g_once_init_leave (&gom_serialization_test_book_type_id__volatile, gom_serialization_test_book_type_id);
	}
	return gom_serialization_test_book_type_id__volatile;
}


static void _vala_gom_serialization_test_book_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GomSerializationTestBook * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GOM_SERIALIZATION_TEST_TYPE_BOOK, GomSerializationTestBook);
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_NAME:
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_string (value, gom_serialization_test_book_get_name (self));
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 10875 "GomSerializationTest.c"
		default:
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 10881 "GomSerializationTest.c"
	}
}


static void _vala_gom_serialization_test_book_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GomSerializationTestBook * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GOM_SERIALIZATION_TEST_TYPE_BOOK, GomSerializationTestBook);
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_NAME:
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_set_name (self, g_value_get_string (value));
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 10897 "GomSerializationTest.c"
		default:
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 134 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 10903 "GomSerializationTest.c"
	}
}


gchar* gom_serialization_test_computer_to_string (GomSerializationTestComputer* self) {
	gchar* result = NULL;
	GXmlXParser* parser = NULL;
	GXmlXParser* _tmp0_ = NULL;
	gchar* s = NULL;
	gchar* _tmp1_ = NULL;
	GError * _inner_error_ = NULL;
#line 158 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 159 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = gxml_xparser_new ((GXmlDomNode*) self);
#line 159 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parser = _tmp0_;
#line 160 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup ("");
#line 160 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp1_;
#line 10925 "GomSerializationTest.c"
	{
		gchar* _tmp2_ = NULL;
		gchar* _tmp3_ = NULL;
		gchar* _tmp4_ = NULL;
#line 162 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = gxml_parser_write_string ((GXmlParser*) parser, &_inner_error_);
#line 162 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = _tmp3_;
#line 162 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 10936 "GomSerializationTest.c"
			goto __catch264_g_error;
		}
#line 162 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp4_ = _tmp2_;
#line 162 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = NULL;
#line 162 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 162 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp4_;
#line 161 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp2_);
#line 10949 "GomSerializationTest.c"
	}
	goto __finally264;
	__catch264_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp5_ = NULL;
		gchar* _tmp6_ = NULL;
		gchar* _tmp7_ = NULL;
#line 161 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		e = _inner_error_;
#line 161 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 164 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp5_ = e->message;
#line 164 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp6_ = g_strconcat ("Error: ", _tmp5_, NULL);
#line 164 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp7_ = _tmp6_;
#line 164 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:164: %s", _tmp7_);
#line 164 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp7_);
#line 165 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 161 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_error_free0 (e);
#line 10976 "GomSerializationTest.c"
	}
	__finally264:
#line 161 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 161 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 161 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (parser);
#line 161 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 161 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 161 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return NULL;
#line 10991 "GomSerializationTest.c"
	}
#line 167 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = s;
#line 167 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (parser);
#line 167 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 10999 "GomSerializationTest.c"
}


GomSerializationTestComputer* gom_serialization_test_computer_construct (GType object_type) {
	GomSerializationTestComputer * self = NULL;
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestComputer*) gxml_gom_element_construct (object_type);
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 11009 "GomSerializationTest.c"
}


GomSerializationTestComputer* gom_serialization_test_computer_new (void) {
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_computer_construct (GOM_SERIALIZATION_TEST_TYPE_COMPUTER);
#line 11016 "GomSerializationTest.c"
}


const gchar* gom_serialization_test_computer_get_model (GomSerializationTestComputer* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 155 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 155 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_model;
#line 155 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 155 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 11031 "GomSerializationTest.c"
}


void gom_serialization_test_computer_set_model (GomSerializationTestComputer* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 155 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 155 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 155 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 155 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_model);
#line 155 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_model = _tmp1_;
#line 155 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "model");
#line 11050 "GomSerializationTest.c"
}


const gchar* gom_serialization_test_computer_get_ignore (GomSerializationTestComputer* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 156 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 156 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_ignore;
#line 156 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 156 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 11065 "GomSerializationTest.c"
}


void gom_serialization_test_computer_set_ignore (GomSerializationTestComputer* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 156 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 156 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 156 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 156 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_ignore);
#line 156 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_ignore = _tmp1_;
#line 156 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "ignore");
#line 11084 "GomSerializationTest.c"
}


static GObject * gom_serialization_test_computer_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomSerializationTestComputer * self;
	GError * _inner_error_ = NULL;
#line 157 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_serialization_test_computer_parent_class);
#line 157 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 157 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_TYPE_COMPUTER, GomSerializationTestComputer);
#line 11099 "GomSerializationTest.c"
	{
#line 157 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_initialize ((GXmlGomElement*) self, "Computer");
#line 11103 "GomSerializationTest.c"
	}
	goto __finally265;
	__catch265_g_error:
	{
#line 157 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 157 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 157 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 11114 "GomSerializationTest.c"
	}
	__finally265:
#line 157 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 157 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 157 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 11123 "GomSerializationTest.c"
	}
#line 157 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 11127 "GomSerializationTest.c"
}


static void gom_serialization_test_computer_class_init (GomSerializationTestComputerClass * klass) {
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_computer_parent_class = g_type_class_peek_parent (klass);
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_type_class_add_private (klass, sizeof (GomSerializationTestComputerPrivate));
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_gom_serialization_test_computer_get_property;
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_gom_serialization_test_computer_set_property;
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_serialization_test_computer_constructor;
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->finalize = gom_serialization_test_computer_finalize;
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_COMPUTER_MODEL, g_param_spec_string ("model", "::Model", "model", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_COMPUTER_IGNORE, g_param_spec_string ("ignore", "ignore", "ignore", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 11148 "GomSerializationTest.c"
}


static void gom_serialization_test_computer_instance_init (GomSerializationTestComputer * self) {
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv = GOM_SERIALIZATION_TEST_COMPUTER_GET_PRIVATE (self);
#line 11155 "GomSerializationTest.c"
}


static void gom_serialization_test_computer_finalize (GObject* obj) {
	GomSerializationTestComputer * self;
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_TYPE_COMPUTER, GomSerializationTestComputer);
#line 155 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_model);
#line 156 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_ignore);
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (gom_serialization_test_computer_parent_class)->finalize (obj);
#line 11169 "GomSerializationTest.c"
}


GType gom_serialization_test_computer_get_type (void) {
	static volatile gsize gom_serialization_test_computer_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_computer_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSerializationTestComputerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_serialization_test_computer_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSerializationTestComputer), 0, (GInstanceInitFunc) gom_serialization_test_computer_instance_init, NULL };
		GType gom_serialization_test_computer_type_id;
		gom_serialization_test_computer_type_id = g_type_register_static (GXML_TYPE_GOM_ELEMENT, "GomSerializationTestComputer", &g_define_type_info, 0);
		g_once_init_leave (&gom_serialization_test_computer_type_id__volatile, gom_serialization_test_computer_type_id);
	}
	return gom_serialization_test_computer_type_id__volatile;
}


static void _vala_gom_serialization_test_computer_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GomSerializationTestComputer * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GOM_SERIALIZATION_TEST_TYPE_COMPUTER, GomSerializationTestComputer);
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_COMPUTER_MODEL:
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_string (value, gom_serialization_test_computer_get_model (self));
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_COMPUTER_IGNORE:
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_string (value, gom_serialization_test_computer_get_ignore (self));
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 11202 "GomSerializationTest.c"
		default:
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 11208 "GomSerializationTest.c"
	}
}


static void _vala_gom_serialization_test_computer_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GomSerializationTestComputer * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GOM_SERIALIZATION_TEST_TYPE_COMPUTER, GomSerializationTestComputer);
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_COMPUTER_MODEL:
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_computer_set_model (self, g_value_get_string (value));
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_COMPUTER_IGNORE:
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_computer_set_ignore (self, g_value_get_string (value));
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 11230 "GomSerializationTest.c"
		default:
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 153 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 11236 "GomSerializationTest.c"
	}
}


GType gom_serialization_test_taxes_month_get_type (void) {
	static volatile gsize gom_serialization_test_taxes_month_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_taxes_month_type_id__volatile)) {
		static const GEnumValue values[] = {{GOM_SERIALIZATION_TEST_TAXES_MONTH_JANUARY, "GOM_SERIALIZATION_TEST_TAXES_MONTH_JANUARY", "january"}, {GOM_SERIALIZATION_TEST_TAXES_MONTH_FEBRUARY, "GOM_SERIALIZATION_TEST_TAXES_MONTH_FEBRUARY", "february"}, {0, NULL, NULL}};
		GType gom_serialization_test_taxes_month_type_id;
		gom_serialization_test_taxes_month_type_id = g_enum_register_static ("GomSerializationTestTaxesMonth", values);
		g_once_init_leave (&gom_serialization_test_taxes_month_type_id__volatile, gom_serialization_test_taxes_month_type_id);
	}
	return gom_serialization_test_taxes_month_type_id__volatile;
}


gchar* gom_serialization_test_taxes_to_string (GomSerializationTestTaxes* self) {
	gchar* result = NULL;
	GXmlXParser* parser = NULL;
	GXmlXParser* _tmp0_ = NULL;
	gchar* s = NULL;
	gchar* _tmp1_ = NULL;
	GError * _inner_error_ = NULL;
#line 182 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 183 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = gxml_xparser_new ((GXmlDomNode*) self);
#line 183 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parser = _tmp0_;
#line 184 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup ("");
#line 184 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp1_;
#line 11270 "GomSerializationTest.c"
	{
		gchar* _tmp2_ = NULL;
		gchar* _tmp3_ = NULL;
		gchar* _tmp4_ = NULL;
#line 186 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = gxml_parser_write_string ((GXmlParser*) parser, &_inner_error_);
#line 186 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = _tmp3_;
#line 186 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 11281 "GomSerializationTest.c"
			goto __catch266_g_error;
		}
#line 186 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp4_ = _tmp2_;
#line 186 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = NULL;
#line 186 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 186 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp4_;
#line 185 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp2_);
#line 11294 "GomSerializationTest.c"
	}
	goto __finally266;
	__catch266_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp5_ = NULL;
		gchar* _tmp6_ = NULL;
		gchar* _tmp7_ = NULL;
#line 185 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		e = _inner_error_;
#line 185 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 188 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp5_ = e->message;
#line 188 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp6_ = g_strconcat ("Error: ", _tmp5_, NULL);
#line 188 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp7_ = _tmp6_;
#line 188 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:188: %s", _tmp7_);
#line 188 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp7_);
#line 189 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 185 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_error_free0 (e);
#line 11321 "GomSerializationTest.c"
	}
	__finally266:
#line 185 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 185 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 185 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (parser);
#line 185 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 185 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 185 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return NULL;
#line 11336 "GomSerializationTest.c"
	}
#line 191 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = s;
#line 191 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (parser);
#line 191 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 11344 "GomSerializationTest.c"
}


GomSerializationTestTaxes* gom_serialization_test_taxes_construct (GType object_type) {
	GomSerializationTestTaxes * self = NULL;
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestTaxes*) gxml_gom_element_construct (object_type);
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 11354 "GomSerializationTest.c"
}


GomSerializationTestTaxes* gom_serialization_test_taxes_new (void) {
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_taxes_construct (GOM_SERIALIZATION_TEST_TYPE_TAXES);
#line 11361 "GomSerializationTest.c"
}


gdouble gom_serialization_test_taxes_get_month_rate (GomSerializationTestTaxes* self) {
	gdouble result;
	gdouble _tmp0_ = 0.0;
#line 172 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, 0.0);
#line 172 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_month_rate;
#line 172 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 172 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 11376 "GomSerializationTest.c"
}


void gom_serialization_test_taxes_set_month_rate (GomSerializationTestTaxes* self, gdouble value) {
	gdouble _tmp0_ = 0.0;
#line 172 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 172 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 172 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_month_rate = _tmp0_;
#line 172 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "month-rate");
#line 11390 "GomSerializationTest.c"
}


gboolean gom_serialization_test_taxes_get_tax_free (GomSerializationTestTaxes* self) {
	gboolean result;
	gboolean _tmp0_ = FALSE;
#line 174 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 174 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_tax_free;
#line 174 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 174 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 11405 "GomSerializationTest.c"
}


void gom_serialization_test_taxes_set_tax_free (GomSerializationTestTaxes* self, gboolean value) {
	gboolean _tmp0_ = FALSE;
#line 174 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 174 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 174 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_tax_free = _tmp0_;
#line 174 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "tax-free");
#line 11419 "GomSerializationTest.c"
}


GomSerializationTestTaxesMonth gom_serialization_test_taxes_get_month (GomSerializationTestTaxes* self) {
	GomSerializationTestTaxesMonth result;
	GomSerializationTestTaxesMonth _tmp0_ = 0;
#line 176 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 176 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_month;
#line 176 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 176 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 11434 "GomSerializationTest.c"
}


void gom_serialization_test_taxes_set_month (GomSerializationTestTaxes* self, GomSerializationTestTaxesMonth value) {
	GomSerializationTestTaxesMonth _tmp0_ = 0;
#line 176 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 176 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 176 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_month = _tmp0_;
#line 176 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "month");
#line 11448 "GomSerializationTest.c"
}


GXmlGomDate* gom_serialization_test_taxes_get_pay_date (GomSerializationTestTaxes* self) {
	GXmlGomDate* result;
	GXmlGomDate* _tmp0_ = NULL;
#line 178 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 178 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_pay_date;
#line 178 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 178 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 11463 "GomSerializationTest.c"
}


void gom_serialization_test_taxes_set_pay_date (GomSerializationTestTaxes* self, GXmlGomDate* value) {
	GXmlGomDate* _tmp0_ = NULL;
	GXmlGomDate* _tmp1_ = NULL;
#line 178 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 178 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 178 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 178 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_pay_date);
#line 178 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_pay_date = _tmp1_;
#line 178 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "pay-date");
#line 11482 "GomSerializationTest.c"
}


GXmlGomDateTime* gom_serialization_test_taxes_get_timestamp (GomSerializationTestTaxes* self) {
	GXmlGomDateTime* result;
	GXmlGomDateTime* _tmp0_ = NULL;
#line 180 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 180 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_timestamp;
#line 180 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 180 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 11497 "GomSerializationTest.c"
}


void gom_serialization_test_taxes_set_timestamp (GomSerializationTestTaxes* self, GXmlGomDateTime* value) {
	GXmlGomDateTime* _tmp0_ = NULL;
	GXmlGomDateTime* _tmp1_ = NULL;
#line 180 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 180 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 180 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 180 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_timestamp);
#line 180 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_timestamp = _tmp1_;
#line 180 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "timestamp");
#line 11516 "GomSerializationTest.c"
}


static GObject * gom_serialization_test_taxes_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomSerializationTestTaxes * self;
	GError * _inner_error_ = NULL;
#line 181 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_serialization_test_taxes_parent_class);
#line 181 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 181 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_TYPE_TAXES, GomSerializationTestTaxes);
#line 11531 "GomSerializationTest.c"
	{
#line 181 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_initialize ((GXmlGomElement*) self, "Taxes");
#line 11535 "GomSerializationTest.c"
	}
	goto __finally267;
	__catch267_g_error:
	{
#line 181 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 181 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 181 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 11546 "GomSerializationTest.c"
	}
	__finally267:
#line 181 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 181 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 181 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 11555 "GomSerializationTest.c"
	}
#line 181 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 11559 "GomSerializationTest.c"
}


static void gom_serialization_test_taxes_class_init (GomSerializationTestTaxesClass * klass) {
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_taxes_parent_class = g_type_class_peek_parent (klass);
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_type_class_add_private (klass, sizeof (GomSerializationTestTaxesPrivate));
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_gom_serialization_test_taxes_get_property;
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_gom_serialization_test_taxes_set_property;
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_serialization_test_taxes_constructor;
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->finalize = gom_serialization_test_taxes_finalize;
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_TAXES_MONTH_RATE, g_param_spec_double ("month-rate", "::monthRate", "month-rate", -G_MAXDOUBLE, G_MAXDOUBLE, 0.0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_TAXES_TAX_FREE, g_param_spec_boolean ("tax-free", "::TaxFree", "tax-free", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_TAXES_MONTH, g_param_spec_enum ("month", "::Month", "month", GOM_SERIALIZATION_TEST_TAXES_TYPE_MONTH, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_TAXES_PAY_DATE, g_param_spec_object ("pay-date", "::PayDate", "pay-date", GXML_TYPE_GOM_DATE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_TAXES_TIMESTAMP, g_param_spec_object ("timestamp", "::Timestamp", "timestamp", GXML_TYPE_GOM_DATE_TIME, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 11586 "GomSerializationTest.c"
}


static void gom_serialization_test_taxes_instance_init (GomSerializationTestTaxes * self) {
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv = GOM_SERIALIZATION_TEST_TAXES_GET_PRIVATE (self);
#line 11593 "GomSerializationTest.c"
}


static void gom_serialization_test_taxes_finalize (GObject* obj) {
	GomSerializationTestTaxes * self;
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_TYPE_TAXES, GomSerializationTestTaxes);
#line 178 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_pay_date);
#line 180 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_timestamp);
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (gom_serialization_test_taxes_parent_class)->finalize (obj);
#line 11607 "GomSerializationTest.c"
}


GType gom_serialization_test_taxes_get_type (void) {
	static volatile gsize gom_serialization_test_taxes_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_taxes_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSerializationTestTaxesClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_serialization_test_taxes_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSerializationTestTaxes), 0, (GInstanceInitFunc) gom_serialization_test_taxes_instance_init, NULL };
		GType gom_serialization_test_taxes_type_id;
		gom_serialization_test_taxes_type_id = g_type_register_static (GXML_TYPE_GOM_ELEMENT, "GomSerializationTestTaxes", &g_define_type_info, 0);
		g_once_init_leave (&gom_serialization_test_taxes_type_id__volatile, gom_serialization_test_taxes_type_id);
	}
	return gom_serialization_test_taxes_type_id__volatile;
}


static void _vala_gom_serialization_test_taxes_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GomSerializationTestTaxes * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GOM_SERIALIZATION_TEST_TYPE_TAXES, GomSerializationTestTaxes);
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_TAXES_MONTH_RATE:
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_double (value, gom_serialization_test_taxes_get_month_rate (self));
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_TAXES_TAX_FREE:
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_boolean (value, gom_serialization_test_taxes_get_tax_free (self));
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_TAXES_MONTH:
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_enum (value, gom_serialization_test_taxes_get_month (self));
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_TAXES_PAY_DATE:
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_serialization_test_taxes_get_pay_date (self));
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_TAXES_TIMESTAMP:
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_serialization_test_taxes_get_timestamp (self));
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 11658 "GomSerializationTest.c"
		default:
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 11664 "GomSerializationTest.c"
	}
}


static void _vala_gom_serialization_test_taxes_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GomSerializationTestTaxes * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GOM_SERIALIZATION_TEST_TYPE_TAXES, GomSerializationTestTaxes);
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_TAXES_MONTH_RATE:
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_taxes_set_month_rate (self, g_value_get_double (value));
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_TAXES_TAX_FREE:
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_taxes_set_tax_free (self, g_value_get_boolean (value));
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_TAXES_MONTH:
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_taxes_set_month (self, g_value_get_enum (value));
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_TAXES_PAY_DATE:
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_taxes_set_pay_date (self, g_value_get_object (value));
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_TAXES_TIMESTAMP:
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_taxes_set_timestamp (self, g_value_get_object (value));
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 11704 "GomSerializationTest.c"
		default:
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 170 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 11710 "GomSerializationTest.c"
	}
}


GomSerializationTestBookRegister* gom_serialization_test_book_register_construct_document (GType object_type, GXmlDomDocument* doc) {
	GomSerializationTestBookRegister * self = NULL;
	GXmlDomDocument* _tmp0_ = NULL;
	GXmlDomDocument* _tmp1_ = NULL;
#line 219 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (doc != NULL, NULL);
#line 219 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestBookRegister*) gxml_gom_element_construct (object_type);
#line 220 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = doc;
#line 220 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 220 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (((GXmlGomNode*) self)->_document);
#line 220 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	((GXmlGomNode*) self)->_document = _tmp1_;
#line 219 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 11733 "GomSerializationTest.c"
}


GomSerializationTestBookRegister* gom_serialization_test_book_register_new_document (GXmlDomDocument* doc) {
#line 219 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_book_register_construct_document (GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER, doc);
#line 11740 "GomSerializationTest.c"
}


static gchar* gom_serialization_test_book_register_real_get_map_key (GXmlMappeableElement* base) {
	GomSerializationTestBookRegister * self;
	gchar* result = NULL;
	GomSerializationTestBook* _tmp0_ = NULL;
	GomSerializationTestBook* _tmp1_ = NULL;
	gint _tmp3_ = 0;
	gchar* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	gchar* _tmp6_ = NULL;
	gchar* _tmp7_ = NULL;
	GomSerializationTestBook* _tmp8_ = NULL;
	GomSerializationTestBook* _tmp9_ = NULL;
	const gchar* _tmp10_ = NULL;
	const gchar* _tmp11_ = NULL;
	gchar* _tmp12_ = NULL;
	gchar* _tmp13_ = NULL;
#line 222 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestBookRegister*) base;
#line 223 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = gom_serialization_test_book_register_get_book (self);
#line 223 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _tmp0_;
#line 223 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (_tmp1_ == NULL) {
#line 11768 "GomSerializationTest.c"
		gchar* _tmp2_ = NULL;
#line 223 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = g_strdup ("");
#line 223 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		result = _tmp2_;
#line 223 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return result;
#line 11776 "GomSerializationTest.c"
	}
#line 224 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp3_ = self->priv->_year;
#line 224 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp4_ = g_strdup_printf ("%d", _tmp3_);
#line 224 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp5_ = _tmp4_;
#line 224 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp6_ = g_strconcat (_tmp5_, "-", NULL);
#line 224 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp7_ = _tmp6_;
#line 224 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp8_ = gom_serialization_test_book_register_get_book (self);
#line 224 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp9_ = _tmp8_;
#line 224 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp10_ = gom_serialization_test_book_get_name (_tmp9_);
#line 224 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp11_ = _tmp10_;
#line 224 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp12_ = g_strconcat (_tmp7_, _tmp11_, NULL);
#line 224 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp13_ = _tmp12_;
#line 224 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp7_);
#line 224 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (_tmp5_);
#line 224 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp13_;
#line 224 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 11808 "GomSerializationTest.c"
}


static gchar* gom_serialization_test_book_register_real_get_map_primary_key (GXmlMappeableElementPairKey* base) {
	GomSerializationTestBookRegister * self;
	gchar* result = NULL;
	gint _tmp0_ = 0;
	gchar* _tmp1_ = NULL;
#line 226 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestBookRegister*) base;
#line 226 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_year;
#line 226 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup_printf ("%d", _tmp0_);
#line 226 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp1_;
#line 226 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 11827 "GomSerializationTest.c"
}


static gchar* gom_serialization_test_book_register_real_get_map_secondary_key (GXmlMappeableElementPairKey* base) {
	GomSerializationTestBookRegister * self;
	gchar* result = NULL;
	GomSerializationTestBook* _tmp0_ = NULL;
	GomSerializationTestBook* _tmp1_ = NULL;
	GomSerializationTestBook* _tmp3_ = NULL;
	GomSerializationTestBook* _tmp4_ = NULL;
	const gchar* _tmp5_ = NULL;
	const gchar* _tmp6_ = NULL;
	gchar* _tmp7_ = NULL;
#line 227 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestBookRegister*) base;
#line 228 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = gom_serialization_test_book_register_get_book (self);
#line 228 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _tmp0_;
#line 228 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (_tmp1_ == NULL) {
#line 11849 "GomSerializationTest.c"
		gchar* _tmp2_ = NULL;
#line 228 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = g_strdup ("");
#line 228 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		result = _tmp2_;
#line 228 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return result;
#line 11857 "GomSerializationTest.c"
	}
#line 229 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp3_ = gom_serialization_test_book_register_get_book (self);
#line 229 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp4_ = _tmp3_;
#line 229 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp5_ = gom_serialization_test_book_get_name (_tmp4_);
#line 229 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp6_ = _tmp5_;
#line 229 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp7_ = g_strdup (_tmp6_);
#line 229 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp7_;
#line 229 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 11873 "GomSerializationTest.c"
}


GomSerializationTestBook* gom_serialization_test_book_register_create_book (GomSerializationTestBookRegister* self, const gchar* name) {
	GomSerializationTestBook* result = NULL;
	GXmlDomDocument* _tmp0_ = NULL;
	GXmlDomDocument* _tmp1_ = NULL;
	const gchar* _tmp2_ = NULL;
	GObject* _tmp3_ = NULL;
	GObject* _tmp4_ = NULL;
	GomSerializationTestBook* _tmp5_ = NULL;
#line 231 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 231 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 232 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = gxml_dom_node_get_owner_document ((GXmlDomNode*) self);
#line 232 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _tmp0_;
#line 232 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp2_ = name;
#line 232 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp3_ = g_object_new (GOM_SERIALIZATION_TEST_TYPE_BOOK, "owner-document", _tmp1_, "name", _tmp2_, NULL);
#line 232 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp4_ = G_IS_INITIALLY_UNOWNED (_tmp3_) ? g_object_ref_sink (_tmp3_) : _tmp3_;
#line 232 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp5_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp4_, GOM_SERIALIZATION_TEST_TYPE_BOOK) ? ((GomSerializationTestBook*) _tmp4_) : NULL;
#line 232 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (_tmp5_ == NULL) {
#line 232 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp4_);
#line 11905 "GomSerializationTest.c"
	}
#line 232 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp5_;
#line 232 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 11911 "GomSerializationTest.c"
}


gchar* gom_serialization_test_book_register_to_string (GomSerializationTestBookRegister* self) {
	gchar* result = NULL;
	GXmlXParser* parser = NULL;
	GXmlXParser* _tmp0_ = NULL;
	gchar* s = NULL;
	gchar* _tmp1_ = NULL;
	GError * _inner_error_ = NULL;
#line 237 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 238 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = gxml_xparser_new ((GXmlDomNode*) self);
#line 238 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parser = _tmp0_;
#line 239 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup ("");
#line 239 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp1_;
#line 11932 "GomSerializationTest.c"
	{
		gchar* _tmp2_ = NULL;
		gchar* _tmp3_ = NULL;
		gchar* _tmp4_ = NULL;
#line 241 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = gxml_parser_write_string ((GXmlParser*) parser, &_inner_error_);
#line 241 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = _tmp3_;
#line 241 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 11943 "GomSerializationTest.c"
			goto __catch268_g_error;
		}
#line 241 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp4_ = _tmp2_;
#line 241 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = NULL;
#line 241 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 241 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp4_;
#line 240 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp2_);
#line 11956 "GomSerializationTest.c"
	}
	goto __finally268;
	__catch268_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp5_ = NULL;
		gchar* _tmp6_ = NULL;
		gchar* _tmp7_ = NULL;
#line 240 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		e = _inner_error_;
#line 240 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 243 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp5_ = e->message;
#line 243 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp6_ = g_strconcat ("Error: ", _tmp5_, NULL);
#line 243 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp7_ = _tmp6_;
#line 243 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:243: %s", _tmp7_);
#line 243 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp7_);
#line 244 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 240 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_error_free0 (e);
#line 11983 "GomSerializationTest.c"
	}
	__finally268:
#line 240 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 240 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 240 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (parser);
#line 240 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 240 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 240 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return NULL;
#line 11998 "GomSerializationTest.c"
	}
#line 246 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = s;
#line 246 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (parser);
#line 246 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 12006 "GomSerializationTest.c"
}


GomSerializationTestBookRegister* gom_serialization_test_book_register_construct (GType object_type) {
	GomSerializationTestBookRegister * self = NULL;
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestBookRegister*) gxml_gom_element_construct (object_type);
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 12016 "GomSerializationTest.c"
}


GomSerializationTestBookRegister* gom_serialization_test_book_register_new (void) {
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_book_register_construct (GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER);
#line 12023 "GomSerializationTest.c"
}


gint gom_serialization_test_book_register_get_year (GomSerializationTestBookRegister* self) {
	gint result;
	gint _tmp0_ = 0;
#line 201 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 201 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_year;
#line 201 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 201 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 12038 "GomSerializationTest.c"
}


void gom_serialization_test_book_register_set_year (GomSerializationTestBookRegister* self, gint value) {
	gint _tmp0_ = 0;
#line 201 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 201 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 201 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_year = _tmp0_;
#line 201 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "year");
#line 12052 "GomSerializationTest.c"
}


GomSerializationTestBook* gom_serialization_test_book_register_get_book (GomSerializationTestBookRegister* self) {
	GomSerializationTestBook* result;
	GomSerializationTestBook* _tmp0_ = NULL;
#line 203 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 203 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_book;
#line 203 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 203 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 12067 "GomSerializationTest.c"
}


void gom_serialization_test_book_register_set_book (GomSerializationTestBookRegister* self, GomSerializationTestBook* value) {
	GError * _inner_error_ = NULL;
#line 204 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 12075 "GomSerializationTest.c"
	{
		GomSerializationTestBook* _tmp0_ = NULL;
		GomSerializationTestBook* _tmp1_ = NULL;
		GomSerializationTestBook* _tmp2_ = NULL;
		GXmlDomNode* _tmp3_ = NULL;
		GXmlDomNode* _tmp4_ = NULL;
#line 206 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_object_clean_property_elements ((GXmlGomObject*) self, "book", &_inner_error_);
#line 206 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 12086 "GomSerializationTest.c"
			goto __catch269_g_error;
		}
#line 207 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp0_ = value;
#line 207 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp1_ = _g_object_ref0 (_tmp0_);
#line 207 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (self->priv->_book);
#line 207 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		self->priv->_book = _tmp1_;
#line 208 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = self->priv->_book;
#line 208 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = gxml_dom_node_append_child ((GXmlDomNode*) self, (GXmlDomNode*) _tmp2_, &_inner_error_);
#line 208 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp4_ = _tmp3_;
#line 208 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (_tmp4_);
#line 208 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 12107 "GomSerializationTest.c"
			goto __catch269_g_error;
		}
	}
	goto __finally269;
	__catch269_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp5_ = NULL;
#line 205 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		e = _inner_error_;
#line 205 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 210 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp5_ = e->message;
#line 210 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_warning ("GomSerializationTest.vala:210: %s", _tmp5_);
#line 211 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 205 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_error_free0 (e);
#line 12128 "GomSerializationTest.c"
	}
	__finally269:
#line 205 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 205 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 205 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 205 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return;
#line 12139 "GomSerializationTest.c"
	}
#line 204 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "book");
#line 12143 "GomSerializationTest.c"
}


static GObject * gom_serialization_test_book_register_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomSerializationTestBookRegister * self;
	GError * _inner_error_ = NULL;
#line 215 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_serialization_test_book_register_parent_class);
#line 215 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 215 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER, GomSerializationTestBookRegister);
#line 12158 "GomSerializationTest.c"
	{
#line 216 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_initialize ((GXmlGomElement*) self, "BookRegister");
#line 12162 "GomSerializationTest.c"
	}
	goto __finally270;
	__catch270_g_error:
	{
#line 216 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 216 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 217 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 12173 "GomSerializationTest.c"
	}
	__finally270:
#line 216 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 216 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 216 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 12182 "GomSerializationTest.c"
	}
#line 215 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 12186 "GomSerializationTest.c"
}


static void gom_serialization_test_book_register_class_init (GomSerializationTestBookRegisterClass * klass) {
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_book_register_parent_class = g_type_class_peek_parent (klass);
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_type_class_add_private (klass, sizeof (GomSerializationTestBookRegisterPrivate));
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_gom_serialization_test_book_register_get_property;
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_gom_serialization_test_book_register_set_property;
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_serialization_test_book_register_constructor;
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->finalize = gom_serialization_test_book_register_finalize;
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_BOOK_REGISTER_YEAR, g_param_spec_int ("year", "::Year", "year", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_BOOK_REGISTER_BOOK, g_param_spec_object ("book", "book", "book", GOM_SERIALIZATION_TEST_TYPE_BOOK, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 12207 "GomSerializationTest.c"
}


static void gom_serialization_test_book_register_gxml_mappeable_element_interface_init (GXmlMappeableElementIface * iface) {
#line 231 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_book_register_gxml_mappeable_element_parent_iface = g_type_interface_peek_parent (iface);
#line 231 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	iface->get_map_key = (gchar* (*)(GXmlMappeableElement*)) gom_serialization_test_book_register_real_get_map_key;
#line 12216 "GomSerializationTest.c"
}


static void gom_serialization_test_book_register_gxml_mappeable_element_pair_key_interface_init (GXmlMappeableElementPairKeyIface * iface) {
#line 231 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_book_register_gxml_mappeable_element_pair_key_parent_iface = g_type_interface_peek_parent (iface);
#line 231 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	iface->get_map_primary_key = (gchar* (*)(GXmlMappeableElementPairKey*)) gom_serialization_test_book_register_real_get_map_primary_key;
#line 231 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	iface->get_map_secondary_key = (gchar* (*)(GXmlMappeableElementPairKey*)) gom_serialization_test_book_register_real_get_map_secondary_key;
#line 12227 "GomSerializationTest.c"
}


static void gom_serialization_test_book_register_instance_init (GomSerializationTestBookRegister * self) {
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv = GOM_SERIALIZATION_TEST_BOOK_REGISTER_GET_PRIVATE (self);
#line 199 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_book = NULL;
#line 12236 "GomSerializationTest.c"
}


static void gom_serialization_test_book_register_finalize (GObject* obj) {
	GomSerializationTestBookRegister * self;
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER, GomSerializationTestBookRegister);
#line 199 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_book);
#line 231 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (gom_serialization_test_book_register_parent_class)->finalize (obj);
#line 12248 "GomSerializationTest.c"
}


GType gom_serialization_test_book_register_get_type (void) {
	static volatile gsize gom_serialization_test_book_register_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_book_register_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSerializationTestBookRegisterClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_serialization_test_book_register_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSerializationTestBookRegister), 0, (GInstanceInitFunc) gom_serialization_test_book_register_instance_init, NULL };
		static const GInterfaceInfo gxml_mappeable_element_info = { (GInterfaceInitFunc) gom_serialization_test_book_register_gxml_mappeable_element_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo gxml_mappeable_element_pair_key_info = { (GInterfaceInitFunc) gom_serialization_test_book_register_gxml_mappeable_element_pair_key_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType gom_serialization_test_book_register_type_id;
		gom_serialization_test_book_register_type_id = g_type_register_static (GXML_TYPE_GOM_ELEMENT, "GomSerializationTestBookRegister", &g_define_type_info, 0);
		g_type_add_interface_static (gom_serialization_test_book_register_type_id, GXML_TYPE_MAPPEABLE_ELEMENT, &gxml_mappeable_element_info);
		g_type_add_interface_static (gom_serialization_test_book_register_type_id, GXML_TYPE_MAPPEABLE_ELEMENT_PAIR_KEY, &gxml_mappeable_element_pair_key_info);
		g_once_init_leave (&gom_serialization_test_book_register_type_id__volatile, gom_serialization_test_book_register_type_id);
	}
	return gom_serialization_test_book_register_type_id__volatile;
}


static void _vala_gom_serialization_test_book_register_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GomSerializationTestBookRegister * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER, GomSerializationTestBookRegister);
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_REGISTER_YEAR:
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_int (value, gom_serialization_test_book_register_get_year (self));
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_REGISTER_BOOK:
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_serialization_test_book_register_get_book (self));
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 12285 "GomSerializationTest.c"
		default:
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 12291 "GomSerializationTest.c"
	}
}


static void _vala_gom_serialization_test_book_register_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GomSerializationTestBookRegister * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER, GomSerializationTestBookRegister);
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_REGISTER_YEAR:
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_register_set_year (self, g_value_get_int (value));
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_REGISTER_BOOK:
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_register_set_book (self, g_value_get_object (value));
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 12313 "GomSerializationTest.c"
		default:
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 198 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 12319 "GomSerializationTest.c"
	}
}


gchar* gom_serialization_test_book_stand_to_string (GomSerializationTestBookStand* self) {
	gchar* result = NULL;
	GXmlXParser* parser = NULL;
	GXmlXParser* _tmp0_ = NULL;
	gchar* s = NULL;
	gchar* _tmp1_ = NULL;
	GError * _inner_error_ = NULL;
#line 286 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 287 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = gxml_xparser_new ((GXmlDomNode*) self);
#line 287 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parser = _tmp0_;
#line 288 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup ("");
#line 288 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp1_;
#line 12341 "GomSerializationTest.c"
	{
		gchar* _tmp2_ = NULL;
		gchar* _tmp3_ = NULL;
		gchar* _tmp4_ = NULL;
#line 290 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = gxml_parser_write_string ((GXmlParser*) parser, &_inner_error_);
#line 290 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = _tmp3_;
#line 290 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 12352 "GomSerializationTest.c"
			goto __catch271_g_error;
		}
#line 290 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp4_ = _tmp2_;
#line 290 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = NULL;
#line 290 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 290 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp4_;
#line 289 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp2_);
#line 12365 "GomSerializationTest.c"
	}
	goto __finally271;
	__catch271_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp5_ = NULL;
		gchar* _tmp6_ = NULL;
		gchar* _tmp7_ = NULL;
#line 289 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		e = _inner_error_;
#line 289 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 292 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp5_ = e->message;
#line 292 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp6_ = g_strconcat ("Error: ", _tmp5_, NULL);
#line 292 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp7_ = _tmp6_;
#line 292 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:292: %s", _tmp7_);
#line 292 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp7_);
#line 293 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 289 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_error_free0 (e);
#line 12392 "GomSerializationTest.c"
	}
	__finally271:
#line 289 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 289 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 289 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (parser);
#line 289 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 289 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 289 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return NULL;
#line 12407 "GomSerializationTest.c"
	}
#line 295 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = s;
#line 295 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (parser);
#line 295 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 12415 "GomSerializationTest.c"
}


GomSerializationTestBookStand* gom_serialization_test_book_stand_construct (GType object_type) {
	GomSerializationTestBookStand * self = NULL;
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestBookStand*) gxml_gom_element_construct (object_type);
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 12425 "GomSerializationTest.c"
}


GomSerializationTestBookStand* gom_serialization_test_book_stand_new (void) {
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_book_stand_construct (GOM_SERIALIZATION_TEST_TYPE_BOOK_STAND);
#line 12432 "GomSerializationTest.c"
}


const gchar* gom_serialization_test_book_stand_get_classification (GomSerializationTestBookStand* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 253 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 253 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_classification;
#line 253 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 253 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 12447 "GomSerializationTest.c"
}


void gom_serialization_test_book_stand_set_classification (GomSerializationTestBookStand* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 253 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 253 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 253 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 253 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_classification);
#line 253 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_classification = _tmp1_;
#line 253 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "classification");
#line 12466 "GomSerializationTest.c"
}


GomSerializationTestBookStandDimension* gom_serialization_test_book_stand_get_dimension_x (GomSerializationTestBookStand* self) {
	GomSerializationTestBookStandDimension* result;
	GomSerializationTestBookStandDimension* _tmp0_ = NULL;
#line 254 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 254 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_dimension_x;
#line 254 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 254 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 12481 "GomSerializationTest.c"
}


void gom_serialization_test_book_stand_set_dimension_x (GomSerializationTestBookStand* self, GomSerializationTestBookStandDimension* value) {
	GomSerializationTestBookStandDimension* _tmp0_ = NULL;
	GomSerializationTestBookStandDimension* _tmp1_ = NULL;
#line 254 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 254 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 254 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 254 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_dimension_x);
#line 254 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_dimension_x = _tmp1_;
#line 254 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "dimension-x");
#line 12500 "GomSerializationTest.c"
}


GomSerializationTestBookStandDimensionY* gom_serialization_test_book_stand_get_dimension_y (GomSerializationTestBookStand* self) {
	GomSerializationTestBookStandDimensionY* result;
	GomSerializationTestBookStandDimensionY* _tmp0_ = NULL;
#line 256 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 256 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_dimension_y;
#line 256 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 256 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 12515 "GomSerializationTest.c"
}


void gom_serialization_test_book_stand_set_dimension_y (GomSerializationTestBookStand* self, GomSerializationTestBookStandDimensionY* value) {
	GomSerializationTestBookStandDimensionY* _tmp0_ = NULL;
	GomSerializationTestBookStandDimensionY* _tmp1_ = NULL;
#line 256 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 256 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 256 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 256 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_dimension_y);
#line 256 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_dimension_y = _tmp1_;
#line 256 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "dimension-y");
#line 12534 "GomSerializationTest.c"
}


GomSerializationTestBookStandDimensionZ* gom_serialization_test_book_stand_get_dimension_z (GomSerializationTestBookStand* self) {
	GomSerializationTestBookStandDimensionZ* result;
	GomSerializationTestBookStandDimensionZ* _tmp0_ = NULL;
#line 258 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 258 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_dimension_z;
#line 258 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 258 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 12549 "GomSerializationTest.c"
}


void gom_serialization_test_book_stand_set_dimension_z (GomSerializationTestBookStand* self, GomSerializationTestBookStandDimensionZ* value) {
	GomSerializationTestBookStandDimensionZ* _tmp0_ = NULL;
	GomSerializationTestBookStandDimensionZ* _tmp1_ = NULL;
#line 258 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 258 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 258 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 258 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_dimension_z);
#line 258 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_dimension_z = _tmp1_;
#line 258 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "dimension-z");
#line 12568 "GomSerializationTest.c"
}


GomSerializationTestRegisters* gom_serialization_test_book_stand_get_registers (GomSerializationTestBookStand* self) {
	GomSerializationTestRegisters* result;
	GomSerializationTestRegisters* _tmp0_ = NULL;
#line 259 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 259 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_registers;
#line 259 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 259 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 12583 "GomSerializationTest.c"
}


void gom_serialization_test_book_stand_set_registers (GomSerializationTestBookStand* self, GomSerializationTestRegisters* value) {
	GomSerializationTestRegisters* _tmp0_ = NULL;
	GomSerializationTestRegisters* _tmp1_ = NULL;
#line 259 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 259 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 259 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 259 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_registers);
#line 259 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_registers = _tmp1_;
#line 259 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "registers");
#line 12602 "GomSerializationTest.c"
}


GomSerializationTestHashRegisters* gom_serialization_test_book_stand_get_hashmap_registers (GomSerializationTestBookStand* self) {
	GomSerializationTestHashRegisters* result;
	GomSerializationTestHashRegisters* _tmp0_ = NULL;
	GomSerializationTestHashRegisters* _tmp4_ = NULL;
#line 261 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 262 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_hashmap_registers;
#line 262 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (_tmp0_ == NULL) {
#line 12616 "GomSerializationTest.c"
		GObject* _tmp1_ = NULL;
		GObject* _tmp2_ = NULL;
		GomSerializationTestHashRegisters* _tmp3_ = NULL;
#line 263 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp1_ = g_object_new (GOM_SERIALIZATION_TEST_TYPE_HASH_REGISTERS, "element", self, NULL);
#line 263 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = G_IS_INITIALLY_UNOWNED (_tmp1_) ? g_object_ref_sink (_tmp1_) : _tmp1_;
#line 263 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp2_, GOM_SERIALIZATION_TEST_TYPE_HASH_REGISTERS) ? ((GomSerializationTestHashRegisters*) _tmp2_) : NULL;
#line 263 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (_tmp3_ == NULL) {
#line 263 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp2_);
#line 12630 "GomSerializationTest.c"
		}
#line 263 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (self->priv->_hashmap_registers);
#line 263 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		self->priv->_hashmap_registers = _tmp3_;
#line 12636 "GomSerializationTest.c"
	}
#line 265 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp4_ = self->priv->_hashmap_registers;
#line 265 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp4_;
#line 265 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 12644 "GomSerializationTest.c"
}


void gom_serialization_test_book_stand_set_hashmap_registers (GomSerializationTestBookStand* self, GomSerializationTestHashRegisters* value) {
	GomSerializationTestHashRegisters* _tmp0_ = NULL;
	GomSerializationTestHashRegisters* _tmp1_ = NULL;
#line 267 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 268 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 268 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 268 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_hashmap_registers);
#line 268 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_hashmap_registers = _tmp1_;
#line 267 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "hashmap-registers");
#line 12663 "GomSerializationTest.c"
}


GomSerializationTestHashPairRegisters* gom_serialization_test_book_stand_get_hashpair_registers (GomSerializationTestBookStand* self) {
	GomSerializationTestHashPairRegisters* result;
	GomSerializationTestHashPairRegisters* _tmp0_ = NULL;
	GomSerializationTestHashPairRegisters* _tmp4_ = NULL;
#line 272 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 273 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_hashpair_registers;
#line 273 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (_tmp0_ == NULL) {
#line 12677 "GomSerializationTest.c"
		GObject* _tmp1_ = NULL;
		GObject* _tmp2_ = NULL;
		GomSerializationTestHashPairRegisters* _tmp3_ = NULL;
#line 274 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp1_ = g_object_new (GOM_SERIALIZATION_TEST_TYPE_HASH_PAIR_REGISTERS, "element", self, NULL);
#line 274 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = G_IS_INITIALLY_UNOWNED (_tmp1_) ? g_object_ref_sink (_tmp1_) : _tmp1_;
#line 274 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp2_, GOM_SERIALIZATION_TEST_TYPE_HASH_PAIR_REGISTERS) ? ((GomSerializationTestHashPairRegisters*) _tmp2_) : NULL;
#line 274 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (_tmp3_ == NULL) {
#line 274 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
			_g_object_unref0 (_tmp2_);
#line 12691 "GomSerializationTest.c"
		}
#line 274 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (self->priv->_hashpair_registers);
#line 274 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		self->priv->_hashpair_registers = _tmp3_;
#line 12697 "GomSerializationTest.c"
	}
#line 276 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp4_ = self->priv->_hashpair_registers;
#line 276 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp4_;
#line 276 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 12705 "GomSerializationTest.c"
}


void gom_serialization_test_book_stand_set_hashpair_registers (GomSerializationTestBookStand* self, GomSerializationTestHashPairRegisters* value) {
	GomSerializationTestHashPairRegisters* _tmp0_ = NULL;
	GomSerializationTestHashPairRegisters* _tmp1_ = NULL;
#line 278 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 279 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 279 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 279 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_hashpair_registers);
#line 279 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_hashpair_registers = _tmp1_;
#line 278 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "hashpair-registers");
#line 12724 "GomSerializationTest.c"
}


GomSerializationTestBooks* gom_serialization_test_book_stand_get_books (GomSerializationTestBookStand* self) {
	GomSerializationTestBooks* result;
	GomSerializationTestBooks* _tmp0_ = NULL;
#line 282 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 282 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_books;
#line 282 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 282 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 12739 "GomSerializationTest.c"
}


void gom_serialization_test_book_stand_set_books (GomSerializationTestBookStand* self, GomSerializationTestBooks* value) {
	GomSerializationTestBooks* _tmp0_ = NULL;
	GomSerializationTestBooks* _tmp1_ = NULL;
#line 282 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 282 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 282 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 282 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_books);
#line 282 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_books = _tmp1_;
#line 282 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "books");
#line 12758 "GomSerializationTest.c"
}


static GObject * gom_serialization_test_book_stand_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomSerializationTestBookStand * self;
	GError * _inner_error_ = NULL;
#line 283 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_serialization_test_book_stand_parent_class);
#line 283 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 283 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_TYPE_BOOK_STAND, GomSerializationTestBookStand);
#line 12773 "GomSerializationTest.c"
	{
#line 284 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_initialize ((GXmlGomElement*) self, "BookStand");
#line 12777 "GomSerializationTest.c"
	}
	goto __finally272;
	__catch272_g_error:
	{
#line 284 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 284 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 284 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 12788 "GomSerializationTest.c"
	}
	__finally272:
#line 284 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 284 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 284 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 12797 "GomSerializationTest.c"
	}
#line 283 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 12801 "GomSerializationTest.c"
}


GomSerializationTestBookStandDimension* gom_serialization_test_book_stand_dimension_construct (GType object_type) {
	GomSerializationTestBookStandDimension * self = NULL;
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestBookStandDimension*) gxml_gom_element_construct (object_type);
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 12811 "GomSerializationTest.c"
}


GomSerializationTestBookStandDimension* gom_serialization_test_book_stand_dimension_new (void) {
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_book_stand_dimension_construct (GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION);
#line 12818 "GomSerializationTest.c"
}


gdouble gom_serialization_test_book_stand_dimension_get_length (GomSerializationTestBookStandDimension* self) {
	gdouble result;
	gdouble _tmp0_ = 0.0;
#line 299 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, 0.0);
#line 299 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_length;
#line 299 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 299 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 12833 "GomSerializationTest.c"
}


void gom_serialization_test_book_stand_dimension_set_length (GomSerializationTestBookStandDimension* self, gdouble value) {
	gdouble _tmp0_ = 0.0;
#line 299 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 299 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 299 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_length = _tmp0_;
#line 299 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "length");
#line 12847 "GomSerializationTest.c"
}


const gchar* gom_serialization_test_book_stand_dimension_get_dtype (GomSerializationTestBookStandDimension* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 301 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 301 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_dtype;
#line 301 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 301 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 12862 "GomSerializationTest.c"
}


void gom_serialization_test_book_stand_dimension_set_dtype (GomSerializationTestBookStandDimension* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 301 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 301 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 301 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 301 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_dtype);
#line 301 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_dtype = _tmp1_;
#line 301 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "dtype");
#line 12881 "GomSerializationTest.c"
}


static GObject * gom_serialization_test_book_stand_dimension_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomSerializationTestBookStandDimension * self;
	GError * _inner_error_ = NULL;
#line 302 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_serialization_test_book_stand_dimension_parent_class);
#line 302 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 302 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION, GomSerializationTestBookStandDimension);
#line 12896 "GomSerializationTest.c"
	{
#line 303 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_initialize ((GXmlGomElement*) self, "Dimension");
#line 12900 "GomSerializationTest.c"
	}
	goto __finally273;
	__catch273_g_error:
	{
#line 303 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 303 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 303 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 12911 "GomSerializationTest.c"
	}
	__finally273:
#line 303 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 303 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 303 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 12920 "GomSerializationTest.c"
	}
#line 302 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 12924 "GomSerializationTest.c"
}


static void gom_serialization_test_book_stand_dimension_class_init (GomSerializationTestBookStandDimensionClass * klass) {
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_book_stand_dimension_parent_class = g_type_class_peek_parent (klass);
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_type_class_add_private (klass, sizeof (GomSerializationTestBookStandDimensionPrivate));
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_gom_serialization_test_book_stand_dimension_get_property;
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_gom_serialization_test_book_stand_dimension_set_property;
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_serialization_test_book_stand_dimension_constructor;
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->finalize = gom_serialization_test_book_stand_dimension_finalize;
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_LENGTH, g_param_spec_double ("length", "::Length", "length", -G_MAXDOUBLE, G_MAXDOUBLE, 1.0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_DTYPE, g_param_spec_string ("dtype", "::Type", "dtype", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 12945 "GomSerializationTest.c"
}


static void gom_serialization_test_book_stand_dimension_instance_init (GomSerializationTestBookStandDimension * self) {
	gchar* _tmp0_ = NULL;
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv = GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_GET_PRIVATE (self);
#line 299 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_length = 1.0;
#line 301 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = g_strdup ("x");
#line 301 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_dtype = _tmp0_;
#line 12959 "GomSerializationTest.c"
}


static void gom_serialization_test_book_stand_dimension_finalize (GObject* obj) {
	GomSerializationTestBookStandDimension * self;
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION, GomSerializationTestBookStandDimension);
#line 301 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_dtype);
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (gom_serialization_test_book_stand_dimension_parent_class)->finalize (obj);
#line 12971 "GomSerializationTest.c"
}


GType gom_serialization_test_book_stand_dimension_get_type (void) {
	static volatile gsize gom_serialization_test_book_stand_dimension_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_book_stand_dimension_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSerializationTestBookStandDimensionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_serialization_test_book_stand_dimension_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSerializationTestBookStandDimension), 0, (GInstanceInitFunc) gom_serialization_test_book_stand_dimension_instance_init, NULL };
		GType gom_serialization_test_book_stand_dimension_type_id;
		gom_serialization_test_book_stand_dimension_type_id = g_type_register_static (GXML_TYPE_GOM_ELEMENT, "GomSerializationTestBookStandDimension", &g_define_type_info, 0);
		g_once_init_leave (&gom_serialization_test_book_stand_dimension_type_id__volatile, gom_serialization_test_book_stand_dimension_type_id);
	}
	return gom_serialization_test_book_stand_dimension_type_id__volatile;
}


static void _vala_gom_serialization_test_book_stand_dimension_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GomSerializationTestBookStandDimension * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION, GomSerializationTestBookStandDimension);
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_LENGTH:
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_double (value, gom_serialization_test_book_stand_dimension_get_length (self));
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_DTYPE:
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_string (value, gom_serialization_test_book_stand_dimension_get_dtype (self));
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 13004 "GomSerializationTest.c"
		default:
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 13010 "GomSerializationTest.c"
	}
}


static void _vala_gom_serialization_test_book_stand_dimension_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GomSerializationTestBookStandDimension * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION, GomSerializationTestBookStandDimension);
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_LENGTH:
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_stand_dimension_set_length (self, g_value_get_double (value));
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_DTYPE:
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_stand_dimension_set_dtype (self, g_value_get_string (value));
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 13032 "GomSerializationTest.c"
		default:
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 297 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 13038 "GomSerializationTest.c"
	}
}


GomSerializationTestBookStandDimensionY* gom_serialization_test_book_stand_dimension_y_construct (GType object_type) {
	GomSerializationTestBookStandDimensionY * self = NULL;
#line 306 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestBookStandDimensionY*) gom_serialization_test_book_stand_dimension_construct (object_type);
#line 306 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 13049 "GomSerializationTest.c"
}


GomSerializationTestBookStandDimensionY* gom_serialization_test_book_stand_dimension_y_new (void) {
#line 306 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_book_stand_dimension_y_construct (GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION_Y);
#line 13056 "GomSerializationTest.c"
}


static GObject * gom_serialization_test_book_stand_dimension_y_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomSerializationTestBookStandDimensionY * self;
#line 307 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_serialization_test_book_stand_dimension_y_parent_class);
#line 307 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 307 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION_Y, GomSerializationTestBookStandDimensionY);
#line 308 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_book_stand_dimension_set_dtype ((GomSerializationTestBookStandDimension*) self, "y");
#line 307 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 13074 "GomSerializationTest.c"
}


static void gom_serialization_test_book_stand_dimension_y_class_init (GomSerializationTestBookStandDimensionYClass * klass) {
#line 306 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_book_stand_dimension_y_parent_class = g_type_class_peek_parent (klass);
#line 306 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_serialization_test_book_stand_dimension_y_constructor;
#line 13083 "GomSerializationTest.c"
}


static void gom_serialization_test_book_stand_dimension_y_instance_init (GomSerializationTestBookStandDimensionY * self) {
}


GType gom_serialization_test_book_stand_dimension_y_get_type (void) {
	static volatile gsize gom_serialization_test_book_stand_dimension_y_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_book_stand_dimension_y_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSerializationTestBookStandDimensionYClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_serialization_test_book_stand_dimension_y_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSerializationTestBookStandDimensionY), 0, (GInstanceInitFunc) gom_serialization_test_book_stand_dimension_y_instance_init, NULL };
		GType gom_serialization_test_book_stand_dimension_y_type_id;
		gom_serialization_test_book_stand_dimension_y_type_id = g_type_register_static (GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION, "GomSerializationTestBookStandDimensionY", &g_define_type_info, 0);
		g_once_init_leave (&gom_serialization_test_book_stand_dimension_y_type_id__volatile, gom_serialization_test_book_stand_dimension_y_type_id);
	}
	return gom_serialization_test_book_stand_dimension_y_type_id__volatile;
}


GomSerializationTestBookStandDimensionZ* gom_serialization_test_book_stand_dimension_z_construct (GType object_type) {
	GomSerializationTestBookStandDimensionZ * self = NULL;
#line 311 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestBookStandDimensionZ*) gom_serialization_test_book_stand_dimension_construct (object_type);
#line 311 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 13109 "GomSerializationTest.c"
}


GomSerializationTestBookStandDimensionZ* gom_serialization_test_book_stand_dimension_z_new (void) {
#line 311 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_book_stand_dimension_z_construct (GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION_Z);
#line 13116 "GomSerializationTest.c"
}


static GObject * gom_serialization_test_book_stand_dimension_z_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomSerializationTestBookStandDimensionZ * self;
#line 312 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_serialization_test_book_stand_dimension_z_parent_class);
#line 312 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 312 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION_Z, GomSerializationTestBookStandDimensionZ);
#line 313 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_book_stand_dimension_set_dtype ((GomSerializationTestBookStandDimension*) self, "z");
#line 312 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 13134 "GomSerializationTest.c"
}


static void gom_serialization_test_book_stand_dimension_z_class_init (GomSerializationTestBookStandDimensionZClass * klass) {
#line 311 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_book_stand_dimension_z_parent_class = g_type_class_peek_parent (klass);
#line 311 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_serialization_test_book_stand_dimension_z_constructor;
#line 13143 "GomSerializationTest.c"
}


static void gom_serialization_test_book_stand_dimension_z_instance_init (GomSerializationTestBookStandDimensionZ * self) {
}


GType gom_serialization_test_book_stand_dimension_z_get_type (void) {
	static volatile gsize gom_serialization_test_book_stand_dimension_z_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_book_stand_dimension_z_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSerializationTestBookStandDimensionZClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_serialization_test_book_stand_dimension_z_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSerializationTestBookStandDimensionZ), 0, (GInstanceInitFunc) gom_serialization_test_book_stand_dimension_z_instance_init, NULL };
		GType gom_serialization_test_book_stand_dimension_z_type_id;
		gom_serialization_test_book_stand_dimension_z_type_id = g_type_register_static (GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION, "GomSerializationTestBookStandDimensionZ", &g_define_type_info, 0);
		g_once_init_leave (&gom_serialization_test_book_stand_dimension_z_type_id__volatile, gom_serialization_test_book_stand_dimension_z_type_id);
	}
	return gom_serialization_test_book_stand_dimension_z_type_id__volatile;
}


static void gom_serialization_test_book_stand_class_init (GomSerializationTestBookStandClass * klass) {
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_book_stand_parent_class = g_type_class_peek_parent (klass);
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_type_class_add_private (klass, sizeof (GomSerializationTestBookStandPrivate));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_gom_serialization_test_book_stand_get_property;
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_gom_serialization_test_book_stand_set_property;
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_serialization_test_book_stand_constructor;
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->finalize = gom_serialization_test_book_stand_finalize;
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_BOOK_STAND_CLASSIFICATION, g_param_spec_string ("classification", "::Classification", "classification", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_X, g_param_spec_object ("dimension-x", "dimension-x", "dimension-x", GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_Y, g_param_spec_object ("dimension-y", "::DimensionY", "dimension-y", GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION_Y, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_Z, g_param_spec_object ("dimension-z", "DimensionZ", "dimension-z", GOM_SERIALIZATION_TEST_BOOK_STAND_TYPE_DIMENSION_Z, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_BOOK_STAND_REGISTERS, g_param_spec_object ("registers", "registers", "registers", GOM_SERIALIZATION_TEST_TYPE_REGISTERS, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_BOOK_STAND_HASHMAP_REGISTERS, g_param_spec_object ("hashmap-registers", "hashmap-registers", "hashmap-registers", GOM_SERIALIZATION_TEST_TYPE_HASH_REGISTERS, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_BOOK_STAND_HASHPAIR_REGISTERS, g_param_spec_object ("hashpair-registers", "hashpair-registers", "hashpair-registers", GOM_SERIALIZATION_TEST_TYPE_HASH_PAIR_REGISTERS, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_BOOK_STAND_BOOKS, g_param_spec_object ("books", "books", "books", GOM_SERIALIZATION_TEST_TYPE_BOOKS, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 13192 "GomSerializationTest.c"
}


static void gom_serialization_test_book_stand_instance_init (GomSerializationTestBookStand * self) {
	gchar* _tmp0_ = NULL;
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv = GOM_SERIALIZATION_TEST_BOOK_STAND_GET_PRIVATE (self);
#line 250 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_hashmap_registers = NULL;
#line 251 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_hashpair_registers = NULL;
#line 253 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = g_strdup ("Science");
#line 253 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_classification = _tmp0_;
#line 13208 "GomSerializationTest.c"
}


static void gom_serialization_test_book_stand_finalize (GObject* obj) {
	GomSerializationTestBookStand * self;
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_TYPE_BOOK_STAND, GomSerializationTestBookStand);
#line 250 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_hashmap_registers);
#line 251 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_hashpair_registers);
#line 253 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_classification);
#line 254 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_dimension_x);
#line 256 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_dimension_y);
#line 258 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_dimension_z);
#line 259 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_registers);
#line 282 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_books);
#line 272 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (gom_serialization_test_book_stand_parent_class)->finalize (obj);
#line 13234 "GomSerializationTest.c"
}


GType gom_serialization_test_book_stand_get_type (void) {
	static volatile gsize gom_serialization_test_book_stand_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_book_stand_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSerializationTestBookStandClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_serialization_test_book_stand_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSerializationTestBookStand), 0, (GInstanceInitFunc) gom_serialization_test_book_stand_instance_init, NULL };
		GType gom_serialization_test_book_stand_type_id;
		gom_serialization_test_book_stand_type_id = g_type_register_static (GXML_TYPE_GOM_ELEMENT, "GomSerializationTestBookStand", &g_define_type_info, 0);
		g_once_init_leave (&gom_serialization_test_book_stand_type_id__volatile, gom_serialization_test_book_stand_type_id);
	}
	return gom_serialization_test_book_stand_type_id__volatile;
}


static void _vala_gom_serialization_test_book_stand_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GomSerializationTestBookStand * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GOM_SERIALIZATION_TEST_TYPE_BOOK_STAND, GomSerializationTestBookStand);
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STAND_CLASSIFICATION:
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_string (value, gom_serialization_test_book_stand_get_classification (self));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_X:
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_serialization_test_book_stand_get_dimension_x (self));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_Y:
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_serialization_test_book_stand_get_dimension_y (self));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_Z:
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_serialization_test_book_stand_get_dimension_z (self));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STAND_REGISTERS:
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_serialization_test_book_stand_get_registers (self));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STAND_HASHMAP_REGISTERS:
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_serialization_test_book_stand_get_hashmap_registers (self));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STAND_HASHPAIR_REGISTERS:
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_serialization_test_book_stand_get_hashpair_registers (self));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STAND_BOOKS:
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_serialization_test_book_stand_get_books (self));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 13303 "GomSerializationTest.c"
		default:
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 13309 "GomSerializationTest.c"
	}
}


static void _vala_gom_serialization_test_book_stand_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GomSerializationTestBookStand * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GOM_SERIALIZATION_TEST_TYPE_BOOK_STAND, GomSerializationTestBookStand);
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STAND_CLASSIFICATION:
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_stand_set_classification (self, g_value_get_string (value));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_X:
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_stand_set_dimension_x (self, g_value_get_object (value));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_Y:
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_stand_set_dimension_y (self, g_value_get_object (value));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STAND_DIMENSION_Z:
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_stand_set_dimension_z (self, g_value_get_object (value));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STAND_REGISTERS:
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_stand_set_registers (self, g_value_get_object (value));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STAND_HASHMAP_REGISTERS:
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_stand_set_hashmap_registers (self, g_value_get_object (value));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STAND_HASHPAIR_REGISTERS:
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_stand_set_hashpair_registers (self, g_value_get_object (value));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STAND_BOOKS:
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_stand_set_books (self, g_value_get_object (value));
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 13367 "GomSerializationTest.c"
		default:
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 249 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 13373 "GomSerializationTest.c"
	}
}


GomSerializationTestRegisters* gom_serialization_test_registers_construct (GType object_type) {
	GomSerializationTestRegisters * self = NULL;
#line 318 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestRegisters*) gxml_gom_array_list_construct (object_type);
#line 318 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 13384 "GomSerializationTest.c"
}


GomSerializationTestRegisters* gom_serialization_test_registers_new (void) {
#line 318 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_registers_construct (GOM_SERIALIZATION_TEST_TYPE_REGISTERS);
#line 13391 "GomSerializationTest.c"
}


static GObject * gom_serialization_test_registers_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomSerializationTestRegisters * self;
	GError * _inner_error_ = NULL;
#line 319 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_serialization_test_registers_parent_class);
#line 319 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 319 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_TYPE_REGISTERS, GomSerializationTestRegisters);
#line 13406 "GomSerializationTest.c"
	{
#line 320 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_collection_initialize ((GXmlGomCollection*) self, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER, &_inner_error_);
#line 320 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 13412 "GomSerializationTest.c"
			goto __catch274_g_error;
		}
	}
	goto __finally274;
	__catch274_g_error:
	{
#line 320 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 320 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 321 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 13425 "GomSerializationTest.c"
	}
	__finally274:
#line 320 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 320 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 320 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 13434 "GomSerializationTest.c"
	}
#line 319 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 13438 "GomSerializationTest.c"
}


static void gom_serialization_test_registers_class_init (GomSerializationTestRegistersClass * klass) {
#line 318 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_registers_parent_class = g_type_class_peek_parent (klass);
#line 318 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_serialization_test_registers_constructor;
#line 13447 "GomSerializationTest.c"
}


static void gom_serialization_test_registers_instance_init (GomSerializationTestRegisters * self) {
}


GType gom_serialization_test_registers_get_type (void) {
	static volatile gsize gom_serialization_test_registers_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_registers_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSerializationTestRegistersClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_serialization_test_registers_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSerializationTestRegisters), 0, (GInstanceInitFunc) gom_serialization_test_registers_instance_init, NULL };
		GType gom_serialization_test_registers_type_id;
		gom_serialization_test_registers_type_id = g_type_register_static (GXML_TYPE_GOM_ARRAY_LIST, "GomSerializationTestRegisters", &g_define_type_info, 0);
		g_once_init_leave (&gom_serialization_test_registers_type_id__volatile, gom_serialization_test_registers_type_id);
	}
	return gom_serialization_test_registers_type_id__volatile;
}


GomSerializationTestHashRegisters* gom_serialization_test_hash_registers_construct (GType object_type) {
	GomSerializationTestHashRegisters * self = NULL;
#line 324 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestHashRegisters*) gxml_gom_hash_map_construct (object_type);
#line 324 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 13473 "GomSerializationTest.c"
}


GomSerializationTestHashRegisters* gom_serialization_test_hash_registers_new (void) {
#line 324 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_hash_registers_construct (GOM_SERIALIZATION_TEST_TYPE_HASH_REGISTERS);
#line 13480 "GomSerializationTest.c"
}


static GObject * gom_serialization_test_hash_registers_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomSerializationTestHashRegisters * self;
	GError * _inner_error_ = NULL;
#line 325 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_serialization_test_hash_registers_parent_class);
#line 325 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 325 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_TYPE_HASH_REGISTERS, GomSerializationTestHashRegisters);
#line 13495 "GomSerializationTest.c"
	{
#line 326 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_collection_initialize ((GXmlGomCollection*) self, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER, &_inner_error_);
#line 326 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 13501 "GomSerializationTest.c"
			goto __catch275_g_error;
		}
	}
	goto __finally275;
	__catch275_g_error:
	{
#line 326 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 326 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 327 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 13514 "GomSerializationTest.c"
	}
	__finally275:
#line 326 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 326 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 326 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 13523 "GomSerializationTest.c"
	}
#line 325 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 13527 "GomSerializationTest.c"
}


static void gom_serialization_test_hash_registers_class_init (GomSerializationTestHashRegistersClass * klass) {
#line 324 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_hash_registers_parent_class = g_type_class_peek_parent (klass);
#line 324 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_serialization_test_hash_registers_constructor;
#line 13536 "GomSerializationTest.c"
}


static void gom_serialization_test_hash_registers_instance_init (GomSerializationTestHashRegisters * self) {
}


GType gom_serialization_test_hash_registers_get_type (void) {
	static volatile gsize gom_serialization_test_hash_registers_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_hash_registers_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSerializationTestHashRegistersClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_serialization_test_hash_registers_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSerializationTestHashRegisters), 0, (GInstanceInitFunc) gom_serialization_test_hash_registers_instance_init, NULL };
		GType gom_serialization_test_hash_registers_type_id;
		gom_serialization_test_hash_registers_type_id = g_type_register_static (GXML_TYPE_GOM_HASH_MAP, "GomSerializationTestHashRegisters", &g_define_type_info, 0);
		g_once_init_leave (&gom_serialization_test_hash_registers_type_id__volatile, gom_serialization_test_hash_registers_type_id);
	}
	return gom_serialization_test_hash_registers_type_id__volatile;
}


GomSerializationTestHashPairRegisters* gom_serialization_test_hash_pair_registers_construct (GType object_type) {
	GomSerializationTestHashPairRegisters * self = NULL;
#line 330 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestHashPairRegisters*) gxml_gom_hash_paired_map_construct (object_type);
#line 330 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 13562 "GomSerializationTest.c"
}


GomSerializationTestHashPairRegisters* gom_serialization_test_hash_pair_registers_new (void) {
#line 330 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_hash_pair_registers_construct (GOM_SERIALIZATION_TEST_TYPE_HASH_PAIR_REGISTERS);
#line 13569 "GomSerializationTest.c"
}


static GObject * gom_serialization_test_hash_pair_registers_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomSerializationTestHashPairRegisters * self;
	GError * _inner_error_ = NULL;
#line 331 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_serialization_test_hash_pair_registers_parent_class);
#line 331 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 331 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_TYPE_HASH_PAIR_REGISTERS, GomSerializationTestHashPairRegisters);
#line 13584 "GomSerializationTest.c"
	{
#line 332 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_collection_initialize ((GXmlGomCollection*) self, GOM_SERIALIZATION_TEST_TYPE_BOOK_REGISTER, &_inner_error_);
#line 332 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 13590 "GomSerializationTest.c"
			goto __catch276_g_error;
		}
	}
	goto __finally276;
	__catch276_g_error:
	{
#line 332 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 332 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 333 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 13603 "GomSerializationTest.c"
	}
	__finally276:
#line 332 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 332 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 332 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 13612 "GomSerializationTest.c"
	}
#line 331 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 13616 "GomSerializationTest.c"
}


static void gom_serialization_test_hash_pair_registers_class_init (GomSerializationTestHashPairRegistersClass * klass) {
#line 330 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_hash_pair_registers_parent_class = g_type_class_peek_parent (klass);
#line 330 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_serialization_test_hash_pair_registers_constructor;
#line 13625 "GomSerializationTest.c"
}


static void gom_serialization_test_hash_pair_registers_instance_init (GomSerializationTestHashPairRegisters * self) {
}


GType gom_serialization_test_hash_pair_registers_get_type (void) {
	static volatile gsize gom_serialization_test_hash_pair_registers_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_hash_pair_registers_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSerializationTestHashPairRegistersClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_serialization_test_hash_pair_registers_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSerializationTestHashPairRegisters), 0, (GInstanceInitFunc) gom_serialization_test_hash_pair_registers_instance_init, NULL };
		GType gom_serialization_test_hash_pair_registers_type_id;
		gom_serialization_test_hash_pair_registers_type_id = g_type_register_static (GXML_TYPE_GOM_HASH_PAIRED_MAP, "GomSerializationTestHashPairRegisters", &g_define_type_info, 0);
		g_once_init_leave (&gom_serialization_test_hash_pair_registers_type_id__volatile, gom_serialization_test_hash_pair_registers_type_id);
	}
	return gom_serialization_test_hash_pair_registers_type_id__volatile;
}


GomSerializationTestBooks* gom_serialization_test_books_construct (GType object_type) {
	GomSerializationTestBooks * self = NULL;
#line 336 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestBooks*) gxml_gom_hash_map_construct (object_type);
#line 336 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 13651 "GomSerializationTest.c"
}


GomSerializationTestBooks* gom_serialization_test_books_new (void) {
#line 336 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_books_construct (GOM_SERIALIZATION_TEST_TYPE_BOOKS);
#line 13658 "GomSerializationTest.c"
}


static GObject * gom_serialization_test_books_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomSerializationTestBooks * self;
	GError * _inner_error_ = NULL;
#line 337 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_serialization_test_books_parent_class);
#line 337 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 337 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_TYPE_BOOKS, GomSerializationTestBooks);
#line 13673 "GomSerializationTest.c"
	{
#line 338 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_hash_map_initialize_with_key ((GXmlGomHashMap*) self, GOM_SERIALIZATION_TEST_TYPE_BOOK, "Name", &_inner_error_);
#line 338 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 13679 "GomSerializationTest.c"
			goto __catch277_g_error;
		}
	}
	goto __finally277;
	__catch277_g_error:
	{
#line 338 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 338 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 339 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 13692 "GomSerializationTest.c"
	}
	__finally277:
#line 338 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 338 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 338 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 13701 "GomSerializationTest.c"
	}
#line 337 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 13705 "GomSerializationTest.c"
}


static void gom_serialization_test_books_class_init (GomSerializationTestBooksClass * klass) {
#line 336 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_books_parent_class = g_type_class_peek_parent (klass);
#line 336 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_serialization_test_books_constructor;
#line 13714 "GomSerializationTest.c"
}


static void gom_serialization_test_books_instance_init (GomSerializationTestBooks * self) {
}


GType gom_serialization_test_books_get_type (void) {
	static volatile gsize gom_serialization_test_books_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_books_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSerializationTestBooksClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_serialization_test_books_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSerializationTestBooks), 0, (GInstanceInitFunc) gom_serialization_test_books_instance_init, NULL };
		GType gom_serialization_test_books_type_id;
		gom_serialization_test_books_type_id = g_type_register_static (GXML_TYPE_GOM_HASH_MAP, "GomSerializationTestBooks", &g_define_type_info, 0);
		g_once_init_leave (&gom_serialization_test_books_type_id__volatile, gom_serialization_test_books_type_id);
	}
	return gom_serialization_test_books_type_id__volatile;
}


gchar* gom_serialization_test_book_store_to_string (GomSerializationTestBookStore* self) {
	gchar* result = NULL;
	GXmlXParser* parser = NULL;
	GXmlXParser* _tmp0_ = NULL;
	gchar* s = NULL;
	gchar* _tmp1_ = NULL;
	GError * _inner_error_ = NULL;
#line 349 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 350 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = gxml_xparser_new ((GXmlDomNode*) self);
#line 350 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parser = _tmp0_;
#line 351 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup ("");
#line 351 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp1_;
#line 13751 "GomSerializationTest.c"
	{
		gchar* _tmp2_ = NULL;
		gchar* _tmp3_ = NULL;
		gchar* _tmp4_ = NULL;
#line 353 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = gxml_parser_write_string ((GXmlParser*) parser, &_inner_error_);
#line 353 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = _tmp3_;
#line 353 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 13762 "GomSerializationTest.c"
			goto __catch278_g_error;
		}
#line 353 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp4_ = _tmp2_;
#line 353 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = NULL;
#line 353 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 353 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp4_;
#line 352 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp2_);
#line 13775 "GomSerializationTest.c"
	}
	goto __finally278;
	__catch278_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp5_ = NULL;
		gchar* _tmp6_ = NULL;
		gchar* _tmp7_ = NULL;
#line 352 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		e = _inner_error_;
#line 352 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 355 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp5_ = e->message;
#line 355 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp6_ = g_strconcat ("Error: ", _tmp5_, NULL);
#line 355 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp7_ = _tmp6_;
#line 355 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:355: %s", _tmp7_);
#line 355 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp7_);
#line 356 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 352 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_error_free0 (e);
#line 13802 "GomSerializationTest.c"
	}
	__finally278:
#line 352 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 352 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 352 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (parser);
#line 352 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 352 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 352 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return NULL;
#line 13817 "GomSerializationTest.c"
	}
#line 358 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = s;
#line 358 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (parser);
#line 358 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 13825 "GomSerializationTest.c"
}


GomSerializationTestBookStore* gom_serialization_test_book_store_construct (GType object_type) {
	GomSerializationTestBookStore * self = NULL;
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestBookStore*) gxml_gom_element_construct (object_type);
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 13835 "GomSerializationTest.c"
}


GomSerializationTestBookStore* gom_serialization_test_book_store_new (void) {
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_book_store_construct (GOM_SERIALIZATION_TEST_TYPE_BOOK_STORE);
#line 13842 "GomSerializationTest.c"
}


const gchar* gom_serialization_test_book_store_get_name (GomSerializationTestBookStore* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 344 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 344 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_name;
#line 344 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 344 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 13857 "GomSerializationTest.c"
}


void gom_serialization_test_book_store_set_name (GomSerializationTestBookStore* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 344 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 344 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 344 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 344 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_name);
#line 344 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_name = _tmp1_;
#line 344 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 13876 "GomSerializationTest.c"
}


GomSerializationTestBooks* gom_serialization_test_book_store_get_books (GomSerializationTestBookStore* self) {
	GomSerializationTestBooks* result;
	GomSerializationTestBooks* _tmp0_ = NULL;
#line 345 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 345 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_books;
#line 345 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 345 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 13891 "GomSerializationTest.c"
}


void gom_serialization_test_book_store_set_books (GomSerializationTestBookStore* self, GomSerializationTestBooks* value) {
	GomSerializationTestBooks* _tmp0_ = NULL;
	GomSerializationTestBooks* _tmp1_ = NULL;
#line 345 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 345 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 345 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 345 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_books);
#line 345 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_books = _tmp1_;
#line 345 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "books");
#line 13910 "GomSerializationTest.c"
}


static GObject * gom_serialization_test_book_store_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomSerializationTestBookStore * self;
	GError * _inner_error_ = NULL;
#line 346 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_serialization_test_book_store_parent_class);
#line 346 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 346 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_TYPE_BOOK_STORE, GomSerializationTestBookStore);
#line 13925 "GomSerializationTest.c"
	{
#line 347 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_initialize ((GXmlGomElement*) self, "BookStore");
#line 13929 "GomSerializationTest.c"
	}
	goto __finally279;
	__catch279_g_error:
	{
#line 347 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 347 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 347 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 13940 "GomSerializationTest.c"
	}
	__finally279:
#line 347 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 347 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 347 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 13949 "GomSerializationTest.c"
	}
#line 346 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 13953 "GomSerializationTest.c"
}


static void gom_serialization_test_book_store_class_init (GomSerializationTestBookStoreClass * klass) {
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_book_store_parent_class = g_type_class_peek_parent (klass);
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_type_class_add_private (klass, sizeof (GomSerializationTestBookStorePrivate));
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_gom_serialization_test_book_store_get_property;
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_gom_serialization_test_book_store_set_property;
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_serialization_test_book_store_constructor;
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->finalize = gom_serialization_test_book_store_finalize;
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_BOOK_STORE_NAME, g_param_spec_string ("name", "::Name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_BOOK_STORE_BOOKS, g_param_spec_object ("books", "books", "books", GOM_SERIALIZATION_TEST_TYPE_BOOKS, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 13974 "GomSerializationTest.c"
}


static void gom_serialization_test_book_store_instance_init (GomSerializationTestBookStore * self) {
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv = GOM_SERIALIZATION_TEST_BOOK_STORE_GET_PRIVATE (self);
#line 13981 "GomSerializationTest.c"
}


static void gom_serialization_test_book_store_finalize (GObject* obj) {
	GomSerializationTestBookStore * self;
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_TYPE_BOOK_STORE, GomSerializationTestBookStore);
#line 344 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_free0 (self->priv->_name);
#line 345 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_books);
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (gom_serialization_test_book_store_parent_class)->finalize (obj);
#line 13995 "GomSerializationTest.c"
}


GType gom_serialization_test_book_store_get_type (void) {
	static volatile gsize gom_serialization_test_book_store_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_book_store_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSerializationTestBookStoreClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_serialization_test_book_store_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSerializationTestBookStore), 0, (GInstanceInitFunc) gom_serialization_test_book_store_instance_init, NULL };
		GType gom_serialization_test_book_store_type_id;
		gom_serialization_test_book_store_type_id = g_type_register_static (GXML_TYPE_GOM_ELEMENT, "GomSerializationTestBookStore", &g_define_type_info, 0);
		g_once_init_leave (&gom_serialization_test_book_store_type_id__volatile, gom_serialization_test_book_store_type_id);
	}
	return gom_serialization_test_book_store_type_id__volatile;
}


static void _vala_gom_serialization_test_book_store_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GomSerializationTestBookStore * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GOM_SERIALIZATION_TEST_TYPE_BOOK_STORE, GomSerializationTestBookStore);
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STORE_NAME:
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_string (value, gom_serialization_test_book_store_get_name (self));
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STORE_BOOKS:
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_serialization_test_book_store_get_books (self));
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 14028 "GomSerializationTest.c"
		default:
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 14034 "GomSerializationTest.c"
	}
}


static void _vala_gom_serialization_test_book_store_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GomSerializationTestBookStore * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GOM_SERIALIZATION_TEST_TYPE_BOOK_STORE, GomSerializationTestBookStore);
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STORE_NAME:
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_store_set_name (self, g_value_get_string (value));
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_BOOK_STORE_BOOKS:
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_book_store_set_books (self, g_value_get_object (value));
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 14056 "GomSerializationTest.c"
		default:
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 342 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 14062 "GomSerializationTest.c"
	}
}


GType gom_serialization_test_motor_tension_type_enum_get_type (void) {
	static volatile gsize gom_serialization_test_motor_tension_type_enum_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_motor_tension_type_enum_type_id__volatile)) {
		static const GEnumValue values[] = {{GOM_SERIALIZATION_TEST_MOTOR_TENSION_TYPE_ENUM_AC, "GOM_SERIALIZATION_TEST_MOTOR_TENSION_TYPE_ENUM_AC", "ac"}, {GOM_SERIALIZATION_TEST_MOTOR_TENSION_TYPE_ENUM_DC, "GOM_SERIALIZATION_TEST_MOTOR_TENSION_TYPE_ENUM_DC", "dc"}, {0, NULL, NULL}};
		GType gom_serialization_test_motor_tension_type_enum_type_id;
		gom_serialization_test_motor_tension_type_enum_type_id = g_enum_register_static ("GomSerializationTestMotorTensionTypeEnum", values);
		g_once_init_leave (&gom_serialization_test_motor_tension_type_enum_type_id__volatile, gom_serialization_test_motor_tension_type_enum_type_id);
	}
	return gom_serialization_test_motor_tension_type_enum_type_id__volatile;
}


gchar* gom_serialization_test_motor_to_string (GomSerializationTestMotor* self) {
	gchar* result = NULL;
	GXmlXParser* parser = NULL;
	GXmlXParser* _tmp0_ = NULL;
	gchar* s = NULL;
	gchar* _tmp1_ = NULL;
	GError * _inner_error_ = NULL;
#line 375 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 376 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = gxml_xparser_new ((GXmlDomNode*) self);
#line 376 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parser = _tmp0_;
#line 377 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup ("");
#line 377 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	s = _tmp1_;
#line 14096 "GomSerializationTest.c"
	{
		gchar* _tmp2_ = NULL;
		gchar* _tmp3_ = NULL;
		gchar* _tmp4_ = NULL;
#line 379 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp3_ = gxml_parser_write_string ((GXmlParser*) parser, &_inner_error_);
#line 379 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = _tmp3_;
#line 379 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 14107 "GomSerializationTest.c"
			goto __catch280_g_error;
		}
#line 379 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp4_ = _tmp2_;
#line 379 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp2_ = NULL;
#line 379 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 379 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		s = _tmp4_;
#line 378 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp2_);
#line 14120 "GomSerializationTest.c"
	}
	goto __finally280;
	__catch280_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp5_ = NULL;
		gchar* _tmp6_ = NULL;
		gchar* _tmp7_ = NULL;
#line 378 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		e = _inner_error_;
#line 378 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 381 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp5_ = e->message;
#line 381 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp6_ = g_strconcat ("Error: ", _tmp5_, NULL);
#line 381 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_tmp7_ = _tmp6_;
#line 381 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_message ("GomSerializationTest.vala:381: %s", _tmp7_);
#line 381 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (_tmp7_);
#line 382 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 378 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_error_free0 (e);
#line 14147 "GomSerializationTest.c"
	}
	__finally280:
#line 378 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 378 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_free0 (s);
#line 378 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_g_object_unref0 (parser);
#line 378 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 378 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 378 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		return NULL;
#line 14162 "GomSerializationTest.c"
	}
#line 384 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = s;
#line 384 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (parser);
#line 384 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 14170 "GomSerializationTest.c"
}


GomSerializationTestMotor* gom_serialization_test_motor_construct (GType object_type) {
	GomSerializationTestMotor * self = NULL;
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestMotor*) gxml_gom_element_construct (object_type);
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 14180 "GomSerializationTest.c"
}


GomSerializationTestMotor* gom_serialization_test_motor_new (void) {
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_motor_construct (GOM_SERIALIZATION_TEST_TYPE_MOTOR);
#line 14187 "GomSerializationTest.c"
}


GomSerializationTestMotorOn* gom_serialization_test_motor_get_is_on (GomSerializationTestMotor* self) {
	GomSerializationTestMotorOn* result;
	GomSerializationTestMotorOn* _tmp0_ = NULL;
#line 363 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 363 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_is_on;
#line 363 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 363 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 14202 "GomSerializationTest.c"
}


void gom_serialization_test_motor_set_is_on (GomSerializationTestMotor* self, GomSerializationTestMotorOn* value) {
	GomSerializationTestMotorOn* _tmp0_ = NULL;
	GomSerializationTestMotorOn* _tmp1_ = NULL;
#line 363 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 363 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 363 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 363 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_is_on);
#line 363 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_is_on = _tmp1_;
#line 363 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "is-on");
#line 14221 "GomSerializationTest.c"
}


GomSerializationTestMotorTorque* gom_serialization_test_motor_get_torque (GomSerializationTestMotor* self) {
	GomSerializationTestMotorTorque* result;
	GomSerializationTestMotorTorque* _tmp0_ = NULL;
#line 365 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 365 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_torque;
#line 365 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 365 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 14236 "GomSerializationTest.c"
}


void gom_serialization_test_motor_set_torque (GomSerializationTestMotor* self, GomSerializationTestMotorTorque* value) {
	GomSerializationTestMotorTorque* _tmp0_ = NULL;
	GomSerializationTestMotorTorque* _tmp1_ = NULL;
#line 365 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 365 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 365 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 365 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_torque);
#line 365 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_torque = _tmp1_;
#line 365 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "torque");
#line 14255 "GomSerializationTest.c"
}


GomSerializationTestMotorSpeed* gom_serialization_test_motor_get_speed (GomSerializationTestMotor* self) {
	GomSerializationTestMotorSpeed* result;
	GomSerializationTestMotorSpeed* _tmp0_ = NULL;
#line 367 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 367 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_speed;
#line 367 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 367 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 14270 "GomSerializationTest.c"
}


void gom_serialization_test_motor_set_speed (GomSerializationTestMotor* self, GomSerializationTestMotorSpeed* value) {
	GomSerializationTestMotorSpeed* _tmp0_ = NULL;
	GomSerializationTestMotorSpeed* _tmp1_ = NULL;
#line 367 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 367 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 367 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 367 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_speed);
#line 367 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_speed = _tmp1_;
#line 367 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "speed");
#line 14289 "GomSerializationTest.c"
}


GomSerializationTestMotorTensionType* gom_serialization_test_motor_get_tension_type (GomSerializationTestMotor* self) {
	GomSerializationTestMotorTensionType* result;
	GomSerializationTestMotorTensionType* _tmp0_ = NULL;
#line 369 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 369 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_tension_type;
#line 369 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 369 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 14304 "GomSerializationTest.c"
}


void gom_serialization_test_motor_set_tension_type (GomSerializationTestMotor* self, GomSerializationTestMotorTensionType* value) {
	GomSerializationTestMotorTensionType* _tmp0_ = NULL;
	GomSerializationTestMotorTensionType* _tmp1_ = NULL;
#line 369 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 369 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 369 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 369 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_tension_type);
#line 369 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_tension_type = _tmp1_;
#line 369 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "tension-type");
#line 14323 "GomSerializationTest.c"
}


GomSerializationTestMotorTension* gom_serialization_test_motor_get_tension (GomSerializationTestMotor* self) {
	GomSerializationTestMotorTension* result;
	GomSerializationTestMotorTension* _tmp0_ = NULL;
#line 371 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 371 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_tension;
#line 371 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 371 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 14338 "GomSerializationTest.c"
}


void gom_serialization_test_motor_set_tension (GomSerializationTestMotor* self, GomSerializationTestMotorTension* value) {
	GomSerializationTestMotorTension* _tmp0_ = NULL;
	GomSerializationTestMotorTension* _tmp1_ = NULL;
#line 371 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 371 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 371 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 371 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_tension);
#line 371 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_tension = _tmp1_;
#line 371 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "tension");
#line 14357 "GomSerializationTest.c"
}


GomSerializationTestMotorModel* gom_serialization_test_motor_get_model (GomSerializationTestMotor* self) {
	GomSerializationTestMotorModel* result;
	GomSerializationTestMotorModel* _tmp0_ = NULL;
#line 373 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 373 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = self->priv->_model;
#line 373 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	result = _tmp0_;
#line 373 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return result;
#line 14372 "GomSerializationTest.c"
}


void gom_serialization_test_motor_set_model (GomSerializationTestMotor* self, GomSerializationTestMotorModel* value) {
	GomSerializationTestMotorModel* _tmp0_ = NULL;
	GomSerializationTestMotorModel* _tmp1_ = NULL;
#line 373 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_return_if_fail (self != NULL);
#line 373 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = value;
#line 373 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 373 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_model);
#line 373 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv->_model = _tmp1_;
#line 373 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_notify ((GObject *) self, "model");
#line 14391 "GomSerializationTest.c"
}


static GObject * gom_serialization_test_motor_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomSerializationTestMotor * self;
	GError * _inner_error_ = NULL;
#line 374 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_serialization_test_motor_parent_class);
#line 374 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 374 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_TYPE_MOTOR, GomSerializationTestMotor);
#line 14406 "GomSerializationTest.c"
	{
#line 374 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_element_initialize ((GXmlGomElement*) self, "Motor");
#line 14410 "GomSerializationTest.c"
	}
	goto __finally281;
	__catch281_g_error:
	{
#line 374 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 374 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 374 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 14421 "GomSerializationTest.c"
	}
	__finally281:
#line 374 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 374 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 374 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 14430 "GomSerializationTest.c"
	}
#line 374 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 14434 "GomSerializationTest.c"
}


GomSerializationTestMotorOn* gom_serialization_test_motor_on_construct (GType object_type) {
	GomSerializationTestMotorOn * self = NULL;
#line 390 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestMotorOn*) gxml_gom_boolean_construct (object_type);
#line 390 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 14444 "GomSerializationTest.c"
}


GomSerializationTestMotorOn* gom_serialization_test_motor_on_new (void) {
#line 390 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_motor_on_construct (GOM_SERIALIZATION_TEST_MOTOR_TYPE_ON);
#line 14451 "GomSerializationTest.c"
}


static void gom_serialization_test_motor_on_class_init (GomSerializationTestMotorOnClass * klass) {
#line 390 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_motor_on_parent_class = g_type_class_peek_parent (klass);
#line 14458 "GomSerializationTest.c"
}


static void gom_serialization_test_motor_on_instance_init (GomSerializationTestMotorOn * self) {
}


GType gom_serialization_test_motor_on_get_type (void) {
	static volatile gsize gom_serialization_test_motor_on_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_motor_on_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSerializationTestMotorOnClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_serialization_test_motor_on_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSerializationTestMotorOn), 0, (GInstanceInitFunc) gom_serialization_test_motor_on_instance_init, NULL };
		GType gom_serialization_test_motor_on_type_id;
		gom_serialization_test_motor_on_type_id = g_type_register_static (GXML_TYPE_GOM_BOOLEAN, "GomSerializationTestMotorOn", &g_define_type_info, 0);
		g_once_init_leave (&gom_serialization_test_motor_on_type_id__volatile, gom_serialization_test_motor_on_type_id);
	}
	return gom_serialization_test_motor_on_type_id__volatile;
}


GomSerializationTestMotorTorque* gom_serialization_test_motor_torque_construct (GType object_type) {
	GomSerializationTestMotorTorque * self = NULL;
#line 391 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestMotorTorque*) gxml_gom_double_construct (object_type);
#line 391 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 14484 "GomSerializationTest.c"
}


GomSerializationTestMotorTorque* gom_serialization_test_motor_torque_new (void) {
#line 391 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_motor_torque_construct (GOM_SERIALIZATION_TEST_MOTOR_TYPE_TORQUE);
#line 14491 "GomSerializationTest.c"
}


static void gom_serialization_test_motor_torque_class_init (GomSerializationTestMotorTorqueClass * klass) {
#line 391 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_motor_torque_parent_class = g_type_class_peek_parent (klass);
#line 14498 "GomSerializationTest.c"
}


static void gom_serialization_test_motor_torque_instance_init (GomSerializationTestMotorTorque * self) {
}


GType gom_serialization_test_motor_torque_get_type (void) {
	static volatile gsize gom_serialization_test_motor_torque_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_motor_torque_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSerializationTestMotorTorqueClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_serialization_test_motor_torque_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSerializationTestMotorTorque), 0, (GInstanceInitFunc) gom_serialization_test_motor_torque_instance_init, NULL };
		GType gom_serialization_test_motor_torque_type_id;
		gom_serialization_test_motor_torque_type_id = g_type_register_static (GXML_TYPE_GOM_DOUBLE, "GomSerializationTestMotorTorque", &g_define_type_info, 0);
		g_once_init_leave (&gom_serialization_test_motor_torque_type_id__volatile, gom_serialization_test_motor_torque_type_id);
	}
	return gom_serialization_test_motor_torque_type_id__volatile;
}


GomSerializationTestMotorSpeed* gom_serialization_test_motor_speed_construct (GType object_type) {
	GomSerializationTestMotorSpeed * self = NULL;
#line 392 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestMotorSpeed*) gxml_gom_float_construct (object_type);
#line 392 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 14524 "GomSerializationTest.c"
}


GomSerializationTestMotorSpeed* gom_serialization_test_motor_speed_new (void) {
#line 392 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_motor_speed_construct (GOM_SERIALIZATION_TEST_MOTOR_TYPE_SPEED);
#line 14531 "GomSerializationTest.c"
}


static void gom_serialization_test_motor_speed_class_init (GomSerializationTestMotorSpeedClass * klass) {
#line 392 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_motor_speed_parent_class = g_type_class_peek_parent (klass);
#line 14538 "GomSerializationTest.c"
}


static void gom_serialization_test_motor_speed_instance_init (GomSerializationTestMotorSpeed * self) {
}


GType gom_serialization_test_motor_speed_get_type (void) {
	static volatile gsize gom_serialization_test_motor_speed_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_motor_speed_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSerializationTestMotorSpeedClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_serialization_test_motor_speed_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSerializationTestMotorSpeed), 0, (GInstanceInitFunc) gom_serialization_test_motor_speed_instance_init, NULL };
		GType gom_serialization_test_motor_speed_type_id;
		gom_serialization_test_motor_speed_type_id = g_type_register_static (GXML_TYPE_GOM_FLOAT, "GomSerializationTestMotorSpeed", &g_define_type_info, 0);
		g_once_init_leave (&gom_serialization_test_motor_speed_type_id__volatile, gom_serialization_test_motor_speed_type_id);
	}
	return gom_serialization_test_motor_speed_type_id__volatile;
}


GomSerializationTestMotorTensionType* gom_serialization_test_motor_tension_type_construct (GType object_type) {
	GomSerializationTestMotorTensionType * self = NULL;
#line 393 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestMotorTensionType*) gxml_gom_enum_construct (object_type);
#line 393 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 14564 "GomSerializationTest.c"
}


GomSerializationTestMotorTensionType* gom_serialization_test_motor_tension_type_new (void) {
#line 393 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_motor_tension_type_construct (GOM_SERIALIZATION_TEST_MOTOR_TYPE_TENSION_TYPE);
#line 14571 "GomSerializationTest.c"
}


static GObject * gom_serialization_test_motor_tension_type_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomSerializationTestMotorTensionType * self;
	GError * _inner_error_ = NULL;
#line 394 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_serialization_test_motor_tension_type_parent_class);
#line 394 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 394 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_MOTOR_TYPE_TENSION_TYPE, GomSerializationTestMotorTensionType);
#line 14586 "GomSerializationTest.c"
	{
#line 395 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gxml_gom_enum_initialize_enum ((GXmlGomEnum*) self, GOM_SERIALIZATION_TEST_MOTOR_TYPE_TENSION_TYPE_ENUM);
#line 14590 "GomSerializationTest.c"
	}
	goto __finally282;
	__catch282_g_error:
	{
#line 395 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 395 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		_inner_error_ = NULL;
#line 396 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_assert_not_reached ();
#line 14601 "GomSerializationTest.c"
	}
	__finally282:
#line 395 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 395 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 395 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_clear_error (&_inner_error_);
#line 14610 "GomSerializationTest.c"
	}
#line 394 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 14614 "GomSerializationTest.c"
}


static void gom_serialization_test_motor_tension_type_class_init (GomSerializationTestMotorTensionTypeClass * klass) {
#line 393 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_motor_tension_type_parent_class = g_type_class_peek_parent (klass);
#line 393 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_serialization_test_motor_tension_type_constructor;
#line 14623 "GomSerializationTest.c"
}


static void gom_serialization_test_motor_tension_type_instance_init (GomSerializationTestMotorTensionType * self) {
}


GType gom_serialization_test_motor_tension_type_get_type (void) {
	static volatile gsize gom_serialization_test_motor_tension_type_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_motor_tension_type_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSerializationTestMotorTensionTypeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_serialization_test_motor_tension_type_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSerializationTestMotorTensionType), 0, (GInstanceInitFunc) gom_serialization_test_motor_tension_type_instance_init, NULL };
		GType gom_serialization_test_motor_tension_type_type_id;
		gom_serialization_test_motor_tension_type_type_id = g_type_register_static (GXML_TYPE_GOM_ENUM, "GomSerializationTestMotorTensionType", &g_define_type_info, 0);
		g_once_init_leave (&gom_serialization_test_motor_tension_type_type_id__volatile, gom_serialization_test_motor_tension_type_type_id);
	}
	return gom_serialization_test_motor_tension_type_type_id__volatile;
}


GomSerializationTestMotorTension* gom_serialization_test_motor_tension_construct (GType object_type) {
	GomSerializationTestMotorTension * self = NULL;
#line 399 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestMotorTension*) gxml_gom_int_construct (object_type);
#line 399 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 14649 "GomSerializationTest.c"
}


GomSerializationTestMotorTension* gom_serialization_test_motor_tension_new (void) {
#line 399 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_motor_tension_construct (GOM_SERIALIZATION_TEST_MOTOR_TYPE_TENSION);
#line 14656 "GomSerializationTest.c"
}


static void gom_serialization_test_motor_tension_class_init (GomSerializationTestMotorTensionClass * klass) {
#line 399 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_motor_tension_parent_class = g_type_class_peek_parent (klass);
#line 14663 "GomSerializationTest.c"
}


static void gom_serialization_test_motor_tension_instance_init (GomSerializationTestMotorTension * self) {
}


GType gom_serialization_test_motor_tension_get_type (void) {
	static volatile gsize gom_serialization_test_motor_tension_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_motor_tension_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSerializationTestMotorTensionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_serialization_test_motor_tension_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSerializationTestMotorTension), 0, (GInstanceInitFunc) gom_serialization_test_motor_tension_instance_init, NULL };
		GType gom_serialization_test_motor_tension_type_id;
		gom_serialization_test_motor_tension_type_id = g_type_register_static (GXML_TYPE_GOM_INT, "GomSerializationTestMotorTension", &g_define_type_info, 0);
		g_once_init_leave (&gom_serialization_test_motor_tension_type_id__volatile, gom_serialization_test_motor_tension_type_id);
	}
	return gom_serialization_test_motor_tension_type_id__volatile;
}


GomSerializationTestMotorModel* gom_serialization_test_motor_model_construct (GType object_type) {
	GomSerializationTestMotorModel * self = NULL;
#line 400 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = (GomSerializationTestMotorModel*) gxml_gom_array_string_construct (object_type);
#line 400 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return self;
#line 14689 "GomSerializationTest.c"
}


GomSerializationTestMotorModel* gom_serialization_test_motor_model_new (void) {
#line 400 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return gom_serialization_test_motor_model_construct (GOM_SERIALIZATION_TEST_MOTOR_TYPE_MODEL);
#line 14696 "GomSerializationTest.c"
}


static GObject * gom_serialization_test_motor_model_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GomSerializationTestMotorModel * self;
	gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	gchar** _tmp2_ = NULL;
#line 401 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	parent_class = G_OBJECT_CLASS (gom_serialization_test_motor_model_parent_class);
#line 401 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 401 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_MOTOR_TYPE_MODEL, GomSerializationTestMotorModel);
#line 402 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp0_ = g_strdup ("MODEL1");
#line 402 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp1_ = g_strdup ("MODEL2");
#line 402 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp2_ = g_new0 (gchar*, 2 + 1);
#line 402 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp2_[0] = _tmp0_;
#line 402 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_tmp2_[1] = _tmp1_;
#line 402 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gxml_gom_array_string_initialize_strings ((GXmlGomArrayString*) self, _tmp2_, 2);
#line 401 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	return obj;
#line 14727 "GomSerializationTest.c"
}


static void gom_serialization_test_motor_model_class_init (GomSerializationTestMotorModelClass * klass) {
#line 400 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_motor_model_parent_class = g_type_class_peek_parent (klass);
#line 400 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_serialization_test_motor_model_constructor;
#line 14736 "GomSerializationTest.c"
}


static void gom_serialization_test_motor_model_instance_init (GomSerializationTestMotorModel * self) {
}


GType gom_serialization_test_motor_model_get_type (void) {
	static volatile gsize gom_serialization_test_motor_model_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_motor_model_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSerializationTestMotorModelClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_serialization_test_motor_model_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSerializationTestMotorModel), 0, (GInstanceInitFunc) gom_serialization_test_motor_model_instance_init, NULL };
		GType gom_serialization_test_motor_model_type_id;
		gom_serialization_test_motor_model_type_id = g_type_register_static (GXML_TYPE_GOM_ARRAY_STRING, "GomSerializationTestMotorModel", &g_define_type_info, 0);
		g_once_init_leave (&gom_serialization_test_motor_model_type_id__volatile, gom_serialization_test_motor_model_type_id);
	}
	return gom_serialization_test_motor_model_type_id__volatile;
}


static void gom_serialization_test_motor_class_init (GomSerializationTestMotorClass * klass) {
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_motor_parent_class = g_type_class_peek_parent (klass);
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_type_class_add_private (klass, sizeof (GomSerializationTestMotorPrivate));
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_gom_serialization_test_motor_get_property;
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_gom_serialization_test_motor_set_property;
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->constructor = gom_serialization_test_motor_constructor;
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (klass)->finalize = gom_serialization_test_motor_finalize;
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_MOTOR_IS_ON, g_param_spec_object ("is-on", "::On", "is-on", GOM_SERIALIZATION_TEST_MOTOR_TYPE_ON, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_MOTOR_TORQUE, g_param_spec_object ("torque", "::Torque", "torque", GOM_SERIALIZATION_TEST_MOTOR_TYPE_TORQUE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_MOTOR_SPEED, g_param_spec_object ("speed", "::Speed", "speed", GOM_SERIALIZATION_TEST_MOTOR_TYPE_SPEED, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_MOTOR_TENSION_TYPE, g_param_spec_object ("tension-type", "::TensionType", "tension-type", GOM_SERIALIZATION_TEST_MOTOR_TYPE_TENSION_TYPE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_MOTOR_TENSION, g_param_spec_object ("tension", "::Tension", "tension", GOM_SERIALIZATION_TEST_MOTOR_TYPE_TENSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), GOM_SERIALIZATION_TEST_MOTOR_MODEL, g_param_spec_object ("model", "::Model", "model", GOM_SERIALIZATION_TEST_MOTOR_TYPE_MODEL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 14781 "GomSerializationTest.c"
}


static void gom_serialization_test_motor_instance_init (GomSerializationTestMotor * self) {
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self->priv = GOM_SERIALIZATION_TEST_MOTOR_GET_PRIVATE (self);
#line 14788 "GomSerializationTest.c"
}


static void gom_serialization_test_motor_finalize (GObject* obj) {
	GomSerializationTestMotor * self;
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GOM_SERIALIZATION_TEST_TYPE_MOTOR, GomSerializationTestMotor);
#line 363 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_is_on);
#line 365 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_torque);
#line 367 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_speed);
#line 369 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_tension_type);
#line 371 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_tension);
#line 373 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	_g_object_unref0 (self->priv->_model);
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	G_OBJECT_CLASS (gom_serialization_test_motor_parent_class)->finalize (obj);
#line 14810 "GomSerializationTest.c"
}


GType gom_serialization_test_motor_get_type (void) {
	static volatile gsize gom_serialization_test_motor_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_motor_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSerializationTestMotorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_serialization_test_motor_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSerializationTestMotor), 0, (GInstanceInitFunc) gom_serialization_test_motor_instance_init, NULL };
		GType gom_serialization_test_motor_type_id;
		gom_serialization_test_motor_type_id = g_type_register_static (GXML_TYPE_GOM_ELEMENT, "GomSerializationTestMotor", &g_define_type_info, 0);
		g_once_init_leave (&gom_serialization_test_motor_type_id__volatile, gom_serialization_test_motor_type_id);
	}
	return gom_serialization_test_motor_type_id__volatile;
}


static void _vala_gom_serialization_test_motor_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GomSerializationTestMotor * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GOM_SERIALIZATION_TEST_TYPE_MOTOR, GomSerializationTestMotor);
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_MOTOR_IS_ON:
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_serialization_test_motor_get_is_on (self));
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_MOTOR_TORQUE:
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_serialization_test_motor_get_torque (self));
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_MOTOR_SPEED:
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_serialization_test_motor_get_speed (self));
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_MOTOR_TENSION_TYPE:
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_serialization_test_motor_get_tension_type (self));
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_MOTOR_TENSION:
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_serialization_test_motor_get_tension (self));
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_MOTOR_MODEL:
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		g_value_set_object (value, gom_serialization_test_motor_get_model (self));
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 14867 "GomSerializationTest.c"
		default:
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 14873 "GomSerializationTest.c"
	}
}


static void _vala_gom_serialization_test_motor_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GomSerializationTestMotor * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GOM_SERIALIZATION_TEST_TYPE_MOTOR, GomSerializationTestMotor);
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	switch (property_id) {
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_MOTOR_IS_ON:
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_motor_set_is_on (self, g_value_get_object (value));
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_MOTOR_TORQUE:
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_motor_set_torque (self, g_value_get_object (value));
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_MOTOR_SPEED:
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_motor_set_speed (self, g_value_get_object (value));
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_MOTOR_TENSION_TYPE:
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_motor_set_tension_type (self, g_value_get_object (value));
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_MOTOR_TENSION:
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_motor_set_tension (self, g_value_get_object (value));
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		case GOM_SERIALIZATION_TEST_MOTOR_MODEL:
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		gom_serialization_test_motor_set_model (self, g_value_get_object (value));
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 14919 "GomSerializationTest.c"
		default:
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 361 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
		break;
#line 14925 "GomSerializationTest.c"
	}
}


static void gom_serialization_test_class_init (GomSerializationTestClass * klass) {
#line 133 "/home/despinosa/Proyectos/gxml/test/GomSerializationTest.vala"
	gom_serialization_test_parent_class = g_type_class_peek_parent (klass);
#line 14933 "GomSerializationTest.c"
}


static void gom_serialization_test_instance_init (GomSerializationTest * self) {
}


GType gom_serialization_test_get_type (void) {
	static volatile gsize gom_serialization_test_type_id__volatile = 0;
	if (g_once_init_enter (&gom_serialization_test_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GomSerializationTestClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gom_serialization_test_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GomSerializationTest), 0, (GInstanceInitFunc) gom_serialization_test_instance_init, NULL };
		GType gom_serialization_test_type_id;
		gom_serialization_test_type_id = g_type_register_static (TYPE_GXML_TEST, "GomSerializationTest", &g_define_type_info, 0);
		g_once_init_leave (&gom_serialization_test_type_id__volatile, gom_serialization_test_type_id);
	}
	return gom_serialization_test_type_id__volatile;
}



