/* gxml.h generated by valac 0.34.6, the Vala compiler, do not modify */


#ifndef __GXML_H__
#define __GXML_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gee.h>
#include <gio/gio.h>
#include <float.h>
#include <math.h>
#include <libxml/xmlwriter.h>
#include <libxml/xmlreader.h>
#include <libxml/tree.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>

G_BEGIN_DECLS


#define GXML_TYPE_NODE (gxml_node_get_type ())
#define GXML_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_NODE, GXmlNode))
#define GXML_IS_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_NODE))
#define GXML_NODE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_NODE, GXmlNodeIface))

typedef struct _GXmlNode GXmlNode;
typedef struct _GXmlNodeIface GXmlNodeIface;

#define GXML_TYPE_ELEMENT_LIST (gxml_element_list_get_type ())
#define GXML_ELEMENT_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_ELEMENT_LIST, GXmlElementList))
#define GXML_ELEMENT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_ELEMENT_LIST, GXmlElementListClass))
#define GXML_IS_ELEMENT_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_ELEMENT_LIST))
#define GXML_IS_ELEMENT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_ELEMENT_LIST))
#define GXML_ELEMENT_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_ELEMENT_LIST, GXmlElementListClass))

typedef struct _GXmlElementList GXmlElementList;
typedef struct _GXmlElementListClass GXmlElementListClass;

#define GXML_TYPE_NAMESPACE (gxml_namespace_get_type ())
#define GXML_NAMESPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_NAMESPACE, GXmlNamespace))
#define GXML_IS_NAMESPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_NAMESPACE))
#define GXML_NAMESPACE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_NAMESPACE, GXmlNamespaceIface))

typedef struct _GXmlNamespace GXmlNamespace;
typedef struct _GXmlNamespaceIface GXmlNamespaceIface;

#define GXML_TYPE_NODE_TYPE (gxml_node_type_get_type ())

#define GXML_TYPE_DOCUMENT (gxml_document_get_type ())
#define GXML_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOCUMENT, GXmlDocument))
#define GXML_IS_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOCUMENT))
#define GXML_DOCUMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOCUMENT, GXmlDocumentIface))

typedef struct _GXmlDocument GXmlDocument;
typedef struct _GXmlDocumentIface GXmlDocumentIface;

#define GXML_TYPE_ATTRIBUTE (gxml_attribute_get_type ())
#define GXML_ATTRIBUTE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_ATTRIBUTE, GXmlAttribute))
#define GXML_IS_ATTRIBUTE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_ATTRIBUTE))
#define GXML_ATTRIBUTE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_ATTRIBUTE, GXmlAttributeIface))

typedef struct _GXmlAttribute GXmlAttribute;
typedef struct _GXmlAttributeIface GXmlAttributeIface;

#define GXML_TYPE_COMMENT (gxml_comment_get_type ())
#define GXML_COMMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_COMMENT, GXmlComment))
#define GXML_IS_COMMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_COMMENT))
#define GXML_COMMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_COMMENT, GXmlCommentIface))

typedef struct _GXmlComment GXmlComment;
typedef struct _GXmlCommentIface GXmlCommentIface;

#define GXML_TYPE_CDATA (gxml_cdata_get_type ())
#define GXML_CDATA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_CDATA, GXmlCDATA))
#define GXML_IS_CDATA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_CDATA))
#define GXML_CDATA_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_CDATA, GXmlCDATAIface))

typedef struct _GXmlCDATA GXmlCDATA;
typedef struct _GXmlCDATAIface GXmlCDATAIface;

#define GXML_TYPE_CHARACTER (gxml_character_get_type ())
#define GXML_CHARACTER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_CHARACTER, GXmlCharacter))
#define GXML_IS_CHARACTER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_CHARACTER))
#define GXML_CHARACTER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_CHARACTER, GXmlCharacterIface))

typedef struct _GXmlCharacter GXmlCharacter;
typedef struct _GXmlCharacterIface GXmlCharacterIface;

#define GXML_TYPE_DOCUMENT_TYPE (gxml_document_type_get_type ())
#define GXML_DOCUMENT_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOCUMENT_TYPE, GXmlDocumentType))
#define GXML_IS_DOCUMENT_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOCUMENT_TYPE))
#define GXML_DOCUMENT_TYPE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOCUMENT_TYPE, GXmlDocumentTypeIface))

typedef struct _GXmlDocumentType GXmlDocumentType;
typedef struct _GXmlDocumentTypeIface GXmlDocumentTypeIface;

#define GXML_TYPE_ELEMENT (gxml_element_get_type ())
#define GXML_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_ELEMENT, GXmlElement))
#define GXML_IS_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_ELEMENT))
#define GXML_ELEMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_ELEMENT, GXmlElementIface))

typedef struct _GXmlElement GXmlElement;
typedef struct _GXmlElementIface GXmlElementIface;

#define GXML_TYPE_DOM_HTML_COLLECTION (gxml_dom_html_collection_get_type ())
#define GXML_DOM_HTML_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_HTML_COLLECTION, GXmlDomHTMLCollection))
#define GXML_IS_DOM_HTML_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_HTML_COLLECTION))
#define GXML_DOM_HTML_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_HTML_COLLECTION, GXmlDomHTMLCollectionIface))

typedef struct _GXmlDomHTMLCollection GXmlDomHTMLCollection;
typedef struct _GXmlDomHTMLCollectionIface GXmlDomHTMLCollectionIface;

#define GXML_TYPE_DOM_EVENT_TARGET (gxml_dom_event_target_get_type ())
#define GXML_DOM_EVENT_TARGET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_EVENT_TARGET, GXmlDomEventTarget))
#define GXML_IS_DOM_EVENT_TARGET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_EVENT_TARGET))
#define GXML_DOM_EVENT_TARGET_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_EVENT_TARGET, GXmlDomEventTargetIface))

typedef struct _GXmlDomEventTarget GXmlDomEventTarget;
typedef struct _GXmlDomEventTargetIface GXmlDomEventTargetIface;

#define GXML_TYPE_DOM_EVENT_LISTENER (gxml_dom_event_listener_get_type ())
#define GXML_DOM_EVENT_LISTENER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_EVENT_LISTENER, GXmlDomEventListener))
#define GXML_IS_DOM_EVENT_LISTENER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_EVENT_LISTENER))
#define GXML_DOM_EVENT_LISTENER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_EVENT_LISTENER, GXmlDomEventListenerIface))

typedef struct _GXmlDomEventListener GXmlDomEventListener;
typedef struct _GXmlDomEventListenerIface GXmlDomEventListenerIface;

#define GXML_TYPE_DOM_EVENT (gxml_dom_event_get_type ())
#define GXML_DOM_EVENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_EVENT, GXmlDomEvent))
#define GXML_IS_DOM_EVENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_EVENT))
#define GXML_DOM_EVENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_EVENT, GXmlDomEventIface))

typedef struct _GXmlDomEvent GXmlDomEvent;
typedef struct _GXmlDomEventIface GXmlDomEventIface;

#define GXML_TYPE_DOM_TIME_STAMP (gxml_dom_time_stamp_get_type ())
#define GXML_DOM_TIME_STAMP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_TIME_STAMP, GXmlDomTimeStamp))
#define GXML_DOM_TIME_STAMP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_DOM_TIME_STAMP, GXmlDomTimeStampClass))
#define GXML_IS_DOM_TIME_STAMP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_TIME_STAMP))
#define GXML_IS_DOM_TIME_STAMP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_DOM_TIME_STAMP))
#define GXML_DOM_TIME_STAMP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_DOM_TIME_STAMP, GXmlDomTimeStampClass))

typedef struct _GXmlDomTimeStamp GXmlDomTimeStamp;
typedef struct _GXmlDomTimeStampClass GXmlDomTimeStampClass;

#define GXML_DOM_EVENT_TYPE_PHASE (gxml_dom_event_phase_get_type ())

#define GXML_TYPE_DOM_NODE (gxml_dom_node_get_type ())
#define GXML_DOM_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_NODE, GXmlDomNode))
#define GXML_IS_DOM_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_NODE))
#define GXML_DOM_NODE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_NODE, GXmlDomNodeIface))

typedef struct _GXmlDomNode GXmlDomNode;
typedef struct _GXmlDomNodeIface GXmlDomNodeIface;

#define GXML_DOM_NODE_TYPE_DOCUMENT_POSITION (gxml_dom_node_document_position_get_type ())

#define GXML_DOM_NODE_TYPE_NODE_TYPE (gxml_dom_node_node_type_get_type ())

#define GXML_TYPE_DOM_PARENT_NODE (gxml_dom_parent_node_get_type ())
#define GXML_DOM_PARENT_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_PARENT_NODE, GXmlDomParentNode))
#define GXML_IS_DOM_PARENT_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_PARENT_NODE))
#define GXML_DOM_PARENT_NODE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_PARENT_NODE, GXmlDomParentNodeIface))

typedef struct _GXmlDomParentNode GXmlDomParentNode;
typedef struct _GXmlDomParentNodeIface GXmlDomParentNodeIface;

#define GXML_TYPE_DOM_NODE_LIST (gxml_dom_node_list_get_type ())
#define GXML_DOM_NODE_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_NODE_LIST, GXmlDomNodeList))
#define GXML_IS_DOM_NODE_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_NODE_LIST))
#define GXML_DOM_NODE_LIST_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_NODE_LIST, GXmlDomNodeListIface))

typedef struct _GXmlDomNodeList GXmlDomNodeList;
typedef struct _GXmlDomNodeListIface GXmlDomNodeListIface;

#define GXML_TYPE_DOM_ELEMENT_LIST (gxml_dom_element_list_get_type ())
#define GXML_DOM_ELEMENT_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_ELEMENT_LIST, GXmlDomElementList))
#define GXML_DOM_ELEMENT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_DOM_ELEMENT_LIST, GXmlDomElementListClass))
#define GXML_IS_DOM_ELEMENT_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_ELEMENT_LIST))
#define GXML_IS_DOM_ELEMENT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_DOM_ELEMENT_LIST))
#define GXML_DOM_ELEMENT_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_DOM_ELEMENT_LIST, GXmlDomElementListClass))

typedef struct _GXmlDomElementList GXmlDomElementList;
typedef struct _GXmlDomElementListClass GXmlDomElementListClass;

#define GXML_TYPE_DOM_NON_ELEMENT_PARENT_NODE (gxml_dom_non_element_parent_node_get_type ())
#define GXML_DOM_NON_ELEMENT_PARENT_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_NON_ELEMENT_PARENT_NODE, GXmlDomNonElementParentNode))
#define GXML_IS_DOM_NON_ELEMENT_PARENT_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_NON_ELEMENT_PARENT_NODE))
#define GXML_DOM_NON_ELEMENT_PARENT_NODE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_NON_ELEMENT_PARENT_NODE, GXmlDomNonElementParentNodeIface))

typedef struct _GXmlDomNonElementParentNode GXmlDomNonElementParentNode;
typedef struct _GXmlDomNonElementParentNodeIface GXmlDomNonElementParentNodeIface;

#define GXML_TYPE_DOM_DOCUMENT (gxml_dom_document_get_type ())
#define GXML_DOM_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_DOCUMENT, GXmlDomDocument))
#define GXML_IS_DOM_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_DOCUMENT))
#define GXML_DOM_DOCUMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_DOCUMENT, GXmlDomDocumentIface))

typedef struct _GXmlDomDocument GXmlDomDocument;
typedef struct _GXmlDomDocumentIface GXmlDomDocumentIface;

#define GXML_TYPE_DOM_DOCUMENT_FRAGMENT (gxml_dom_document_fragment_get_type ())
#define GXML_DOM_DOCUMENT_FRAGMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_DOCUMENT_FRAGMENT, GXmlDomDocumentFragment))
#define GXML_IS_DOM_DOCUMENT_FRAGMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_DOCUMENT_FRAGMENT))
#define GXML_DOM_DOCUMENT_FRAGMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_DOCUMENT_FRAGMENT, GXmlDomDocumentFragmentIface))

typedef struct _GXmlDomDocumentFragment GXmlDomDocumentFragment;
typedef struct _GXmlDomDocumentFragmentIface GXmlDomDocumentFragmentIface;

#define GXML_TYPE_DOM_NON_DOCUMENT_TYPE_CHILD_NODE (gxml_dom_non_document_type_child_node_get_type ())
#define GXML_DOM_NON_DOCUMENT_TYPE_CHILD_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_NON_DOCUMENT_TYPE_CHILD_NODE, GXmlDomNonDocumentTypeChildNode))
#define GXML_IS_DOM_NON_DOCUMENT_TYPE_CHILD_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_NON_DOCUMENT_TYPE_CHILD_NODE))
#define GXML_DOM_NON_DOCUMENT_TYPE_CHILD_NODE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_NON_DOCUMENT_TYPE_CHILD_NODE, GXmlDomNonDocumentTypeChildNodeIface))

typedef struct _GXmlDomNonDocumentTypeChildNode GXmlDomNonDocumentTypeChildNode;
typedef struct _GXmlDomNonDocumentTypeChildNodeIface GXmlDomNonDocumentTypeChildNodeIface;

#define GXML_TYPE_DOM_CHILD_NODE (gxml_dom_child_node_get_type ())
#define GXML_DOM_CHILD_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_CHILD_NODE, GXmlDomChildNode))
#define GXML_IS_DOM_CHILD_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_CHILD_NODE))
#define GXML_DOM_CHILD_NODE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_CHILD_NODE, GXmlDomChildNodeIface))

typedef struct _GXmlDomChildNode GXmlDomChildNode;
typedef struct _GXmlDomChildNodeIface GXmlDomChildNodeIface;

#define GXML_TYPE_DOM_CHARACTER_DATA (gxml_dom_character_data_get_type ())
#define GXML_DOM_CHARACTER_DATA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_CHARACTER_DATA, GXmlDomCharacterData))
#define GXML_IS_DOM_CHARACTER_DATA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_CHARACTER_DATA))
#define GXML_DOM_CHARACTER_DATA_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_CHARACTER_DATA, GXmlDomCharacterDataIface))

typedef struct _GXmlDomCharacterData GXmlDomCharacterData;
typedef struct _GXmlDomCharacterDataIface GXmlDomCharacterDataIface;

#define GXML_TYPE_DOM_TEXT (gxml_dom_text_get_type ())
#define GXML_DOM_TEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_TEXT, GXmlDomText))
#define GXML_IS_DOM_TEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_TEXT))
#define GXML_DOM_TEXT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_TEXT, GXmlDomTextIface))

typedef struct _GXmlDomText GXmlDomText;
typedef struct _GXmlDomTextIface GXmlDomTextIface;

#define GXML_TYPE_DOM_COMMENT (gxml_dom_comment_get_type ())
#define GXML_DOM_COMMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_COMMENT, GXmlDomComment))
#define GXML_IS_DOM_COMMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_COMMENT))
#define GXML_DOM_COMMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_COMMENT, GXmlDomCommentIface))

typedef struct _GXmlDomComment GXmlDomComment;
typedef struct _GXmlDomCommentIface GXmlDomCommentIface;

#define GXML_TYPE_DOM_PROCESSING_INSTRUCTION (gxml_dom_processing_instruction_get_type ())
#define GXML_DOM_PROCESSING_INSTRUCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_PROCESSING_INSTRUCTION, GXmlDomProcessingInstruction))
#define GXML_IS_DOM_PROCESSING_INSTRUCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_PROCESSING_INSTRUCTION))
#define GXML_DOM_PROCESSING_INSTRUCTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_PROCESSING_INSTRUCTION, GXmlDomProcessingInstructionIface))

typedef struct _GXmlDomProcessingInstruction GXmlDomProcessingInstruction;
typedef struct _GXmlDomProcessingInstructionIface GXmlDomProcessingInstructionIface;

#define GXML_TYPE_DOM_RANGE (gxml_dom_range_get_type ())
#define GXML_DOM_RANGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_RANGE, GXmlDomRange))
#define GXML_IS_DOM_RANGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_RANGE))
#define GXML_DOM_RANGE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_RANGE, GXmlDomRangeIface))

typedef struct _GXmlDomRange GXmlDomRange;
typedef struct _GXmlDomRangeIface GXmlDomRangeIface;

#define GXML_DOM_RANGE_TYPE_BOUNDARY_POINTS (gxml_dom_range_boundary_points_get_type ())

#define GXML_TYPE_DOM_NODE_FILTER (gxml_dom_node_filter_get_type ())
#define GXML_DOM_NODE_FILTER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_NODE_FILTER, GXmlDomNodeFilter))
#define GXML_DOM_NODE_FILTER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_DOM_NODE_FILTER, GXmlDomNodeFilterClass))
#define GXML_IS_DOM_NODE_FILTER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_NODE_FILTER))
#define GXML_IS_DOM_NODE_FILTER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_DOM_NODE_FILTER))
#define GXML_DOM_NODE_FILTER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_DOM_NODE_FILTER, GXmlDomNodeFilterClass))

typedef struct _GXmlDomNodeFilter GXmlDomNodeFilter;
typedef struct _GXmlDomNodeFilterClass GXmlDomNodeFilterClass;

#define GXML_TYPE_DOM_NODE_ITERATOR (gxml_dom_node_iterator_get_type ())
#define GXML_DOM_NODE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_NODE_ITERATOR, GXmlDomNodeIterator))
#define GXML_IS_DOM_NODE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_NODE_ITERATOR))
#define GXML_DOM_NODE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_NODE_ITERATOR, GXmlDomNodeIteratorIface))

typedef struct _GXmlDomNodeIterator GXmlDomNodeIterator;
typedef struct _GXmlDomNodeIteratorIface GXmlDomNodeIteratorIface;

#define GXML_TYPE_DOM_TREE_WALKER (gxml_dom_tree_walker_get_type ())
#define GXML_DOM_TREE_WALKER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_TREE_WALKER, GXmlDomTreeWalker))
#define GXML_IS_DOM_TREE_WALKER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_TREE_WALKER))
#define GXML_DOM_TREE_WALKER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_TREE_WALKER, GXmlDomTreeWalkerIface))

typedef struct _GXmlDomTreeWalker GXmlDomTreeWalker;
typedef struct _GXmlDomTreeWalkerIface GXmlDomTreeWalkerIface;

#define GXML_TYPE_DOM_IMPLEMENTATION (gxml_dom_implementation_get_type ())
#define GXML_DOM_IMPLEMENTATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_IMPLEMENTATION, GXmlDomImplementation))
#define GXML_IS_DOM_IMPLEMENTATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_IMPLEMENTATION))
#define GXML_DOM_IMPLEMENTATION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_IMPLEMENTATION, GXmlDomImplementationIface))

typedef struct _GXmlDomImplementation GXmlDomImplementation;
typedef struct _GXmlDomImplementationIface GXmlDomImplementationIface;

#define GXML_TYPE_DOM_DOCUMENT_TYPE (gxml_dom_document_type_get_type ())
#define GXML_DOM_DOCUMENT_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_DOCUMENT_TYPE, GXmlDomDocumentType))
#define GXML_IS_DOM_DOCUMENT_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_DOCUMENT_TYPE))
#define GXML_DOM_DOCUMENT_TYPE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_DOCUMENT_TYPE, GXmlDomDocumentTypeIface))

typedef struct _GXmlDomDocumentType GXmlDomDocumentType;
typedef struct _GXmlDomDocumentTypeIface GXmlDomDocumentTypeIface;

#define GXML_TYPE_DOM_XML_DOCUMENT (gxml_dom_xml_document_get_type ())
#define GXML_DOM_XML_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_XML_DOCUMENT, GXmlDomXMLDocument))
#define GXML_IS_DOM_XML_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_XML_DOCUMENT))
#define GXML_DOM_XML_DOCUMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_XML_DOCUMENT, GXmlDomXMLDocumentIface))

typedef struct _GXmlDomXMLDocument GXmlDomXMLDocument;
typedef struct _GXmlDomXMLDocumentIface GXmlDomXMLDocumentIface;

#define GXML_TYPE_DOM_ELEMENT (gxml_dom_element_get_type ())
#define GXML_DOM_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_ELEMENT, GXmlDomElement))
#define GXML_IS_DOM_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_ELEMENT))
#define GXML_DOM_ELEMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_ELEMENT, GXmlDomElementIface))

typedef struct _GXmlDomElement GXmlDomElement;
typedef struct _GXmlDomElementIface GXmlDomElementIface;

#define GXML_TYPE_DOM_TOKEN_LIST (gxml_dom_token_list_get_type ())
#define GXML_DOM_TOKEN_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_TOKEN_LIST, GXmlDomTokenList))
#define GXML_IS_DOM_TOKEN_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_TOKEN_LIST))
#define GXML_DOM_TOKEN_LIST_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_TOKEN_LIST, GXmlDomTokenListIface))

typedef struct _GXmlDomTokenList GXmlDomTokenList;
typedef struct _GXmlDomTokenListIface GXmlDomTokenListIface;

#define GXML_TYPE_DOM_NAMED_NODE_MAP (gxml_dom_named_node_map_get_type ())
#define GXML_DOM_NAMED_NODE_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_NAMED_NODE_MAP, GXmlDomNamedNodeMap))
#define GXML_IS_DOM_NAMED_NODE_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_NAMED_NODE_MAP))
#define GXML_DOM_NAMED_NODE_MAP_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_NAMED_NODE_MAP, GXmlDomNamedNodeMapIface))

typedef struct _GXmlDomNamedNodeMap GXmlDomNamedNodeMap;
typedef struct _GXmlDomNamedNodeMapIface GXmlDomNamedNodeMapIface;
typedef struct _GXmlElementListPrivate GXmlElementListPrivate;

#define GXML_TYPE_NODE_LIST (gxml_node_list_get_type ())
#define GXML_NODE_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_NODE_LIST, GXmlNodeList))
#define GXML_NODE_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_NODE_LIST, GXmlNodeListClass))
#define GXML_IS_NODE_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_NODE_LIST))
#define GXML_IS_NODE_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_NODE_LIST))
#define GXML_NODE_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_NODE_LIST, GXmlNodeListClass))

typedef struct _GXmlNodeList GXmlNodeList;
typedef struct _GXmlNodeListClass GXmlNodeListClass;
typedef struct _GXmlNodeListPrivate GXmlNodeListPrivate;

#define GXML_TYPE_NOTATION (gxml_notation_get_type ())
#define GXML_NOTATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_NOTATION, GXmlNotation))
#define GXML_IS_NOTATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_NOTATION))
#define GXML_NOTATION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_NOTATION, GXmlNotationIface))

typedef struct _GXmlNotation GXmlNotation;
typedef struct _GXmlNotationIface GXmlNotationIface;

#define GXML_TYPE_PROCESSING_INSTRUCTION (gxml_processing_instruction_get_type ())
#define GXML_PROCESSING_INSTRUCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_PROCESSING_INSTRUCTION, GXmlProcessingInstruction))
#define GXML_IS_PROCESSING_INSTRUCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_PROCESSING_INSTRUCTION))
#define GXML_PROCESSING_INSTRUCTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_PROCESSING_INSTRUCTION, GXmlProcessingInstructionIface))

typedef struct _GXmlProcessingInstruction GXmlProcessingInstruction;
typedef struct _GXmlProcessingInstructionIface GXmlProcessingInstructionIface;

#define GXML_TYPE_TEXT (gxml_text_get_type ())
#define GXML_TEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_TEXT, GXmlText))
#define GXML_IS_TEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_TEXT))
#define GXML_TEXT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_TEXT, GXmlTextIface))

typedef struct _GXmlText GXmlText;
typedef struct _GXmlTextIface GXmlTextIface;

#define GXML_TYPE_SERIALIZABLE (gxml_serializable_get_type ())
#define GXML_SERIALIZABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_SERIALIZABLE, GXmlSerializable))
#define GXML_IS_SERIALIZABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_SERIALIZABLE))
#define GXML_SERIALIZABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_SERIALIZABLE, GXmlSerializableIface))

typedef struct _GXmlSerializable GXmlSerializable;
typedef struct _GXmlSerializableIface GXmlSerializableIface;

#define GXML_TYPE_SERIALIZABLE_PROPERTY (gxml_serializable_property_get_type ())
#define GXML_SERIALIZABLE_PROPERTY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_SERIALIZABLE_PROPERTY, GXmlSerializableProperty))
#define GXML_IS_SERIALIZABLE_PROPERTY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_SERIALIZABLE_PROPERTY))
#define GXML_SERIALIZABLE_PROPERTY_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_SERIALIZABLE_PROPERTY, GXmlSerializablePropertyIface))

typedef struct _GXmlSerializableProperty GXmlSerializableProperty;
typedef struct _GXmlSerializablePropertyIface GXmlSerializablePropertyIface;

#define GXML_TYPE_SERIALIZABLE_BOOL (gxml_serializable_bool_get_type ())
#define GXML_SERIALIZABLE_BOOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_SERIALIZABLE_BOOL, GXmlSerializableBool))
#define GXML_SERIALIZABLE_BOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_SERIALIZABLE_BOOL, GXmlSerializableBoolClass))
#define GXML_IS_SERIALIZABLE_BOOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_SERIALIZABLE_BOOL))
#define GXML_IS_SERIALIZABLE_BOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_SERIALIZABLE_BOOL))
#define GXML_SERIALIZABLE_BOOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_SERIALIZABLE_BOOL, GXmlSerializableBoolClass))

typedef struct _GXmlSerializableBool GXmlSerializableBool;
typedef struct _GXmlSerializableBoolClass GXmlSerializableBoolClass;
typedef struct _GXmlSerializableBoolPrivate GXmlSerializableBoolPrivate;

#define GXML_TYPE_SERIALIZABLE_DOUBLE (gxml_serializable_double_get_type ())
#define GXML_SERIALIZABLE_DOUBLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_SERIALIZABLE_DOUBLE, GXmlSerializableDouble))
#define GXML_SERIALIZABLE_DOUBLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_SERIALIZABLE_DOUBLE, GXmlSerializableDoubleClass))
#define GXML_IS_SERIALIZABLE_DOUBLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_SERIALIZABLE_DOUBLE))
#define GXML_IS_SERIALIZABLE_DOUBLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_SERIALIZABLE_DOUBLE))
#define GXML_SERIALIZABLE_DOUBLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_SERIALIZABLE_DOUBLE, GXmlSerializableDoubleClass))

typedef struct _GXmlSerializableDouble GXmlSerializableDouble;
typedef struct _GXmlSerializableDoubleClass GXmlSerializableDoubleClass;
typedef struct _GXmlSerializableDoublePrivate GXmlSerializableDoublePrivate;

#define GXML_TYPE_SERIALIZABLE_FLOAT (gxml_serializable_float_get_type ())
#define GXML_SERIALIZABLE_FLOAT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_SERIALIZABLE_FLOAT, GXmlSerializableFloat))
#define GXML_SERIALIZABLE_FLOAT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_SERIALIZABLE_FLOAT, GXmlSerializableFloatClass))
#define GXML_IS_SERIALIZABLE_FLOAT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_SERIALIZABLE_FLOAT))
#define GXML_IS_SERIALIZABLE_FLOAT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_SERIALIZABLE_FLOAT))
#define GXML_SERIALIZABLE_FLOAT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_SERIALIZABLE_FLOAT, GXmlSerializableFloatClass))

typedef struct _GXmlSerializableFloat GXmlSerializableFloat;
typedef struct _GXmlSerializableFloatClass GXmlSerializableFloatClass;
typedef struct _GXmlSerializableFloatPrivate GXmlSerializableFloatPrivate;

#define GXML_TYPE_SERIALIZABLE_ENUM (gxml_serializable_enum_get_type ())
#define GXML_SERIALIZABLE_ENUM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_SERIALIZABLE_ENUM, GXmlSerializableEnum))
#define GXML_SERIALIZABLE_ENUM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_SERIALIZABLE_ENUM, GXmlSerializableEnumClass))
#define GXML_IS_SERIALIZABLE_ENUM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_SERIALIZABLE_ENUM))
#define GXML_IS_SERIALIZABLE_ENUM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_SERIALIZABLE_ENUM))
#define GXML_SERIALIZABLE_ENUM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_SERIALIZABLE_ENUM, GXmlSerializableEnumClass))

typedef struct _GXmlSerializableEnum GXmlSerializableEnum;
typedef struct _GXmlSerializableEnumClass GXmlSerializableEnumClass;
typedef struct _GXmlSerializableEnumPrivate GXmlSerializableEnumPrivate;

#define GXML_TYPE_SERIALIZABLE_INT (gxml_serializable_int_get_type ())
#define GXML_SERIALIZABLE_INT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_SERIALIZABLE_INT, GXmlSerializableInt))
#define GXML_SERIALIZABLE_INT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_SERIALIZABLE_INT, GXmlSerializableIntClass))
#define GXML_IS_SERIALIZABLE_INT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_SERIALIZABLE_INT))
#define GXML_IS_SERIALIZABLE_INT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_SERIALIZABLE_INT))
#define GXML_SERIALIZABLE_INT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_SERIALIZABLE_INT, GXmlSerializableIntClass))

typedef struct _GXmlSerializableInt GXmlSerializableInt;
typedef struct _GXmlSerializableIntClass GXmlSerializableIntClass;
typedef struct _GXmlSerializableIntPrivate GXmlSerializableIntPrivate;

#define GXML_TYPE_SERIALIZABLE_VALUE_LIST (gxml_serializable_value_list_get_type ())
#define GXML_SERIALIZABLE_VALUE_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_SERIALIZABLE_VALUE_LIST, GXmlSerializableValueList))
#define GXML_SERIALIZABLE_VALUE_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_SERIALIZABLE_VALUE_LIST, GXmlSerializableValueListClass))
#define GXML_IS_SERIALIZABLE_VALUE_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_SERIALIZABLE_VALUE_LIST))
#define GXML_IS_SERIALIZABLE_VALUE_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_SERIALIZABLE_VALUE_LIST))
#define GXML_SERIALIZABLE_VALUE_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_SERIALIZABLE_VALUE_LIST, GXmlSerializableValueListClass))

typedef struct _GXmlSerializableValueList GXmlSerializableValueList;
typedef struct _GXmlSerializableValueListClass GXmlSerializableValueListClass;
typedef struct _GXmlSerializableValueListPrivate GXmlSerializableValueListPrivate;

#define GXML_TYPE_ENUMERATION (gxml_enumeration_get_type ())
#define GXML_ENUMERATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_ENUMERATION, GXmlEnumeration))
#define GXML_ENUMERATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_ENUMERATION, GXmlEnumerationClass))
#define GXML_IS_ENUMERATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_ENUMERATION))
#define GXML_IS_ENUMERATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_ENUMERATION))
#define GXML_ENUMERATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_ENUMERATION, GXmlEnumerationClass))

typedef struct _GXmlEnumeration GXmlEnumeration;
typedef struct _GXmlEnumerationClass GXmlEnumerationClass;
typedef struct _GXmlEnumerationPrivate GXmlEnumerationPrivate;

#define GXML_TYPE_SERIALIZABLE_OBJECT_MODEL (gxml_serializable_object_model_get_type ())
#define GXML_SERIALIZABLE_OBJECT_MODEL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, GXmlSerializableObjectModel))
#define GXML_SERIALIZABLE_OBJECT_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, GXmlSerializableObjectModelClass))
#define GXML_IS_SERIALIZABLE_OBJECT_MODEL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_SERIALIZABLE_OBJECT_MODEL))
#define GXML_IS_SERIALIZABLE_OBJECT_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_SERIALIZABLE_OBJECT_MODEL))
#define GXML_SERIALIZABLE_OBJECT_MODEL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, GXmlSerializableObjectModelClass))

typedef struct _GXmlSerializableObjectModel GXmlSerializableObjectModel;
typedef struct _GXmlSerializableObjectModelClass GXmlSerializableObjectModelClass;
typedef struct _GXmlSerializableObjectModelPrivate GXmlSerializableObjectModelPrivate;

#define GXML_TYPE_SERIALIZABLE_COLLECTION (gxml_serializable_collection_get_type ())
#define GXML_SERIALIZABLE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_SERIALIZABLE_COLLECTION, GXmlSerializableCollection))
#define GXML_IS_SERIALIZABLE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_SERIALIZABLE_COLLECTION))
#define GXML_SERIALIZABLE_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_SERIALIZABLE_COLLECTION, GXmlSerializableCollectionIface))

typedef struct _GXmlSerializableCollection GXmlSerializableCollection;
typedef struct _GXmlSerializableCollectionIface GXmlSerializableCollectionIface;

#define GXML_TYPE_SERIALIZABLE_TREE_MAP (gxml_serializable_tree_map_get_type ())
#define GXML_SERIALIZABLE_TREE_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_SERIALIZABLE_TREE_MAP, GXmlSerializableTreeMap))
#define GXML_SERIALIZABLE_TREE_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_SERIALIZABLE_TREE_MAP, GXmlSerializableTreeMapClass))
#define GXML_IS_SERIALIZABLE_TREE_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_SERIALIZABLE_TREE_MAP))
#define GXML_IS_SERIALIZABLE_TREE_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_SERIALIZABLE_TREE_MAP))
#define GXML_SERIALIZABLE_TREE_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_SERIALIZABLE_TREE_MAP, GXmlSerializableTreeMapClass))

typedef struct _GXmlSerializableTreeMap GXmlSerializableTreeMap;
typedef struct _GXmlSerializableTreeMapClass GXmlSerializableTreeMapClass;
typedef struct _GXmlSerializableTreeMapPrivate GXmlSerializableTreeMapPrivate;

#define GXML_TYPE_SERIALIZABLE_HASH_MAP (gxml_serializable_hash_map_get_type ())
#define GXML_SERIALIZABLE_HASH_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_SERIALIZABLE_HASH_MAP, GXmlSerializableHashMap))
#define GXML_SERIALIZABLE_HASH_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_SERIALIZABLE_HASH_MAP, GXmlSerializableHashMapClass))
#define GXML_IS_SERIALIZABLE_HASH_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_SERIALIZABLE_HASH_MAP))
#define GXML_IS_SERIALIZABLE_HASH_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_SERIALIZABLE_HASH_MAP))
#define GXML_SERIALIZABLE_HASH_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_SERIALIZABLE_HASH_MAP, GXmlSerializableHashMapClass))

typedef struct _GXmlSerializableHashMap GXmlSerializableHashMap;
typedef struct _GXmlSerializableHashMapClass GXmlSerializableHashMapClass;
typedef struct _GXmlSerializableHashMapPrivate GXmlSerializableHashMapPrivate;

#define GXML_TYPE_SERIALIZABLE_MAP_KEY (gxml_serializable_map_key_get_type ())
#define GXML_SERIALIZABLE_MAP_KEY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_SERIALIZABLE_MAP_KEY, GXmlSerializableMapKey))
#define GXML_IS_SERIALIZABLE_MAP_KEY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_SERIALIZABLE_MAP_KEY))
#define GXML_SERIALIZABLE_MAP_KEY_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_SERIALIZABLE_MAP_KEY, GXmlSerializableMapKeyIface))

typedef struct _GXmlSerializableMapKey GXmlSerializableMapKey;
typedef struct _GXmlSerializableMapKeyIface GXmlSerializableMapKeyIface;

#define GXML_TYPE_SERIALIZABLE_DUAL_KEY_MAP (gxml_serializable_dual_key_map_get_type ())
#define GXML_SERIALIZABLE_DUAL_KEY_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_SERIALIZABLE_DUAL_KEY_MAP, GXmlSerializableDualKeyMap))
#define GXML_SERIALIZABLE_DUAL_KEY_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_SERIALIZABLE_DUAL_KEY_MAP, GXmlSerializableDualKeyMapClass))
#define GXML_IS_SERIALIZABLE_DUAL_KEY_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_SERIALIZABLE_DUAL_KEY_MAP))
#define GXML_IS_SERIALIZABLE_DUAL_KEY_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_SERIALIZABLE_DUAL_KEY_MAP))
#define GXML_SERIALIZABLE_DUAL_KEY_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_SERIALIZABLE_DUAL_KEY_MAP, GXmlSerializableDualKeyMapClass))

typedef struct _GXmlSerializableDualKeyMap GXmlSerializableDualKeyMap;
typedef struct _GXmlSerializableDualKeyMapClass GXmlSerializableDualKeyMapClass;
typedef struct _GXmlSerializableDualKeyMapPrivate GXmlSerializableDualKeyMapPrivate;

#define GXML_TYPE_SERIALIZABLE_MAP_DUAL_KEY (gxml_serializable_map_dual_key_get_type ())
#define GXML_SERIALIZABLE_MAP_DUAL_KEY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_SERIALIZABLE_MAP_DUAL_KEY, GXmlSerializableMapDualKey))
#define GXML_IS_SERIALIZABLE_MAP_DUAL_KEY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_SERIALIZABLE_MAP_DUAL_KEY))
#define GXML_SERIALIZABLE_MAP_DUAL_KEY_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_SERIALIZABLE_MAP_DUAL_KEY, GXmlSerializableMapDualKeyIface))

typedef struct _GXmlSerializableMapDualKey GXmlSerializableMapDualKey;
typedef struct _GXmlSerializableMapDualKeyIface GXmlSerializableMapDualKeyIface;

#define GXML_TYPE_SERIALIZABLE_ARRAY_LIST (gxml_serializable_array_list_get_type ())
#define GXML_SERIALIZABLE_ARRAY_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_SERIALIZABLE_ARRAY_LIST, GXmlSerializableArrayList))
#define GXML_SERIALIZABLE_ARRAY_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_SERIALIZABLE_ARRAY_LIST, GXmlSerializableArrayListClass))
#define GXML_IS_SERIALIZABLE_ARRAY_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_SERIALIZABLE_ARRAY_LIST))
#define GXML_IS_SERIALIZABLE_ARRAY_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_SERIALIZABLE_ARRAY_LIST))
#define GXML_SERIALIZABLE_ARRAY_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_SERIALIZABLE_ARRAY_LIST, GXmlSerializableArrayListClass))

typedef struct _GXmlSerializableArrayList GXmlSerializableArrayList;
typedef struct _GXmlSerializableArrayListClass GXmlSerializableArrayListClass;
typedef struct _GXmlSerializableArrayListPrivate GXmlSerializableArrayListPrivate;

#define GXML_TYPE_SERIALIZABLE_CONTAINER (gxml_serializable_container_get_type ())
#define GXML_SERIALIZABLE_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_SERIALIZABLE_CONTAINER, GXmlSerializableContainer))
#define GXML_SERIALIZABLE_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_SERIALIZABLE_CONTAINER, GXmlSerializableContainerClass))
#define GXML_IS_SERIALIZABLE_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_SERIALIZABLE_CONTAINER))
#define GXML_IS_SERIALIZABLE_CONTAINER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_SERIALIZABLE_CONTAINER))
#define GXML_SERIALIZABLE_CONTAINER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_SERIALIZABLE_CONTAINER, GXmlSerializableContainerClass))

typedef struct _GXmlSerializableContainer GXmlSerializableContainer;
typedef struct _GXmlSerializableContainerClass GXmlSerializableContainerClass;
typedef struct _GXmlSerializableContainerPrivate GXmlSerializableContainerPrivate;

#define GXML_TYPE_TNODE (gxml_tnode_get_type ())
#define GXML_TNODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_TNODE, GXmlTNode))
#define GXML_TNODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_TNODE, GXmlTNodeClass))
#define GXML_IS_TNODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_TNODE))
#define GXML_IS_TNODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_TNODE))
#define GXML_TNODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_TNODE, GXmlTNodeClass))

typedef struct _GXmlTNode GXmlTNode;
typedef struct _GXmlTNodeClass GXmlTNodeClass;
typedef struct _GXmlTNodePrivate GXmlTNodePrivate;

#define GXML_TYPE_TATTRIBUTE (gxml_tattribute_get_type ())
#define GXML_TATTRIBUTE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_TATTRIBUTE, GXmlTAttribute))
#define GXML_TATTRIBUTE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_TATTRIBUTE, GXmlTAttributeClass))
#define GXML_IS_TATTRIBUTE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_TATTRIBUTE))
#define GXML_IS_TATTRIBUTE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_TATTRIBUTE))
#define GXML_TATTRIBUTE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_TATTRIBUTE, GXmlTAttributeClass))

typedef struct _GXmlTAttribute GXmlTAttribute;
typedef struct _GXmlTAttributeClass GXmlTAttributeClass;
typedef struct _GXmlTAttributePrivate GXmlTAttributePrivate;

#define GXML_TYPE_TCOMMENT (gxml_tcomment_get_type ())
#define GXML_TCOMMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_TCOMMENT, GXmlTComment))
#define GXML_TCOMMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_TCOMMENT, GXmlTCommentClass))
#define GXML_IS_TCOMMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_TCOMMENT))
#define GXML_IS_TCOMMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_TCOMMENT))
#define GXML_TCOMMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_TCOMMENT, GXmlTCommentClass))

typedef struct _GXmlTComment GXmlTComment;
typedef struct _GXmlTCommentClass GXmlTCommentClass;
typedef struct _GXmlTCommentPrivate GXmlTCommentPrivate;

#define GXML_TYPE_TCDATA (gxml_tcdata_get_type ())
#define GXML_TCDATA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_TCDATA, GXmlTCDATA))
#define GXML_TCDATA_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_TCDATA, GXmlTCDATAClass))
#define GXML_IS_TCDATA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_TCDATA))
#define GXML_IS_TCDATA_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_TCDATA))
#define GXML_TCDATA_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_TCDATA, GXmlTCDATAClass))

typedef struct _GXmlTCDATA GXmlTCDATA;
typedef struct _GXmlTCDATAClass GXmlTCDATAClass;
typedef struct _GXmlTCDATAPrivate GXmlTCDATAPrivate;

#define GXML_TYPE_TDOCUMENT (gxml_tdocument_get_type ())
#define GXML_TDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_TDOCUMENT, GXmlTDocument))
#define GXML_TDOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_TDOCUMENT, GXmlTDocumentClass))
#define GXML_IS_TDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_TDOCUMENT))
#define GXML_IS_TDOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_TDOCUMENT))
#define GXML_TDOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_TDOCUMENT, GXmlTDocumentClass))

typedef struct _GXmlTDocument GXmlTDocument;
typedef struct _GXmlTDocumentClass GXmlTDocumentClass;
typedef struct _GXmlTDocumentPrivate GXmlTDocumentPrivate;

#define GXML_TDOCUMENT_TYPE_READ_TYPE (gxml_tdocument_read_type_get_type ())

#define GXML_TYPE_TELEMENT (gxml_telement_get_type ())
#define GXML_TELEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_TELEMENT, GXmlTElement))
#define GXML_TELEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_TELEMENT, GXmlTElementClass))
#define GXML_IS_TELEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_TELEMENT))
#define GXML_IS_TELEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_TELEMENT))
#define GXML_TELEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_TELEMENT, GXmlTElementClass))

typedef struct _GXmlTElement GXmlTElement;
typedef struct _GXmlTElementClass GXmlTElementClass;
typedef struct _GXmlTElementPrivate GXmlTElementPrivate;

#define GXML_TNODE_TYPE_TCHILDREN_LIST (gxml_tnode_tchildren_list_get_type ())
#define GXML_TNODE_TCHILDREN_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TNODE_TYPE_TCHILDREN_LIST, GXmlTNodeTChildrenList))
#define GXML_TNODE_TCHILDREN_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TNODE_TYPE_TCHILDREN_LIST, GXmlTNodeTChildrenListClass))
#define GXML_TNODE_IS_TCHILDREN_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TNODE_TYPE_TCHILDREN_LIST))
#define GXML_TNODE_IS_TCHILDREN_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TNODE_TYPE_TCHILDREN_LIST))
#define GXML_TNODE_TCHILDREN_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TNODE_TYPE_TCHILDREN_LIST, GXmlTNodeTChildrenListClass))

typedef struct _GXmlTNodeTChildrenList GXmlTNodeTChildrenList;
typedef struct _GXmlTNodeTChildrenListClass GXmlTNodeTChildrenListClass;

#define GXML_TYPE_TNAMESPACE (gxml_tnamespace_get_type ())
#define GXML_TNAMESPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_TNAMESPACE, GXmlTNamespace))
#define GXML_TNAMESPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_TNAMESPACE, GXmlTNamespaceClass))
#define GXML_IS_TNAMESPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_TNAMESPACE))
#define GXML_IS_TNAMESPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_TNAMESPACE))
#define GXML_TNAMESPACE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_TNAMESPACE, GXmlTNamespaceClass))

typedef struct _GXmlTNamespace GXmlTNamespace;
typedef struct _GXmlTNamespaceClass GXmlTNamespaceClass;
typedef struct _GXmlTNamespacePrivate GXmlTNamespacePrivate;
typedef struct _GXmlTNodeTChildrenListPrivate GXmlTNodeTChildrenListPrivate;

#define GXML_TYPE_TPROCESSING_INSTRUCTION (gxml_tprocessing_instruction_get_type ())
#define GXML_TPROCESSING_INSTRUCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_TPROCESSING_INSTRUCTION, GXmlTProcessingInstruction))
#define GXML_TPROCESSING_INSTRUCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_TPROCESSING_INSTRUCTION, GXmlTProcessingInstructionClass))
#define GXML_IS_TPROCESSING_INSTRUCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_TPROCESSING_INSTRUCTION))
#define GXML_IS_TPROCESSING_INSTRUCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_TPROCESSING_INSTRUCTION))
#define GXML_TPROCESSING_INSTRUCTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_TPROCESSING_INSTRUCTION, GXmlTProcessingInstructionClass))

typedef struct _GXmlTProcessingInstruction GXmlTProcessingInstruction;
typedef struct _GXmlTProcessingInstructionClass GXmlTProcessingInstructionClass;
typedef struct _GXmlTProcessingInstructionPrivate GXmlTProcessingInstructionPrivate;

#define GXML_TYPE_TTEXT (gxml_ttext_get_type ())
#define GXML_TTEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_TTEXT, GXmlTText))
#define GXML_TTEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_TTEXT, GXmlTTextClass))
#define GXML_IS_TTEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_TTEXT))
#define GXML_IS_TTEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_TTEXT))
#define GXML_TTEXT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_TTEXT, GXmlTTextClass))

typedef struct _GXmlTText GXmlTText;
typedef struct _GXmlTTextClass GXmlTTextClass;
typedef struct _GXmlTTextPrivate GXmlTTextPrivate;

#define GXML_TYPE_GNODE (gxml_gnode_get_type ())
#define GXML_GNODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GNODE, GXmlGNode))
#define GXML_GNODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GNODE, GXmlGNodeClass))
#define GXML_IS_GNODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GNODE))
#define GXML_IS_GNODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GNODE))
#define GXML_GNODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GNODE, GXmlGNodeClass))

typedef struct _GXmlGNode GXmlGNode;
typedef struct _GXmlGNodeClass GXmlGNodeClass;
typedef struct _GXmlGNodePrivate GXmlGNodePrivate;

#define GXML_TYPE_GDOCUMENT (gxml_gdocument_get_type ())
#define GXML_GDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GDOCUMENT, GXmlGDocument))
#define GXML_GDOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GDOCUMENT, GXmlGDocumentClass))
#define GXML_IS_GDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GDOCUMENT))
#define GXML_IS_GDOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GDOCUMENT))
#define GXML_GDOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GDOCUMENT, GXmlGDocumentClass))

typedef struct _GXmlGDocument GXmlGDocument;
typedef struct _GXmlGDocumentClass GXmlGDocumentClass;

#define GXML_TYPE_XPATH_CONTEXT (gxml_xpath_context_get_type ())
#define GXML_XPATH_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_XPATH_CONTEXT, GXmlXPathContext))
#define GXML_IS_XPATH_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_XPATH_CONTEXT))
#define GXML_XPATH_CONTEXT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_XPATH_CONTEXT, GXmlXPathContextIface))

typedef struct _GXmlXPathContext GXmlXPathContext;
typedef struct _GXmlXPathContextIface GXmlXPathContextIface;

#define GXML_TYPE_XPATH_OBJECT (gxml_xpath_object_get_type ())
#define GXML_XPATH_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_XPATH_OBJECT, GXmlXPathObject))
#define GXML_IS_XPATH_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_XPATH_OBJECT))
#define GXML_XPATH_OBJECT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_XPATH_OBJECT, GXmlXPathObjectIface))

typedef struct _GXmlXPathObject GXmlXPathObject;
typedef struct _GXmlXPathObjectIface GXmlXPathObjectIface;

#define GXML_TYPE_XPATH_OBJECT_TYPE (gxml_xpath_object_type_get_type ())
typedef struct _GXmlGDocumentPrivate GXmlGDocumentPrivate;

#define GXML_TYPE_GIMPLEMENTATION (gxml_gimplementation_get_type ())
#define GXML_GIMPLEMENTATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GIMPLEMENTATION, GXmlGImplementation))
#define GXML_GIMPLEMENTATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GIMPLEMENTATION, GXmlGImplementationClass))
#define GXML_IS_GIMPLEMENTATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GIMPLEMENTATION))
#define GXML_IS_GIMPLEMENTATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GIMPLEMENTATION))
#define GXML_GIMPLEMENTATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GIMPLEMENTATION, GXmlGImplementationClass))

typedef struct _GXmlGImplementation GXmlGImplementation;
typedef struct _GXmlGImplementationClass GXmlGImplementationClass;

#define GXML_TYPE_HTML_DOCUMENT (gxml_html_document_get_type ())
#define GXML_HTML_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_HTML_DOCUMENT, GXmlHtmlDocument))
#define GXML_HTML_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_HTML_DOCUMENT, GXmlHtmlDocumentClass))
#define GXML_IS_HTML_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_HTML_DOCUMENT))
#define GXML_IS_HTML_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_HTML_DOCUMENT))
#define GXML_HTML_DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_HTML_DOCUMENT, GXmlHtmlDocumentClass))

typedef struct _GXmlHtmlDocument GXmlHtmlDocument;
typedef struct _GXmlHtmlDocumentClass GXmlHtmlDocumentClass;
typedef struct _GXmlHtmlDocumentPrivate GXmlHtmlDocumentPrivate;

#define GXML_TYPE_DOM_ATTR (gxml_dom_attr_get_type ())
#define GXML_DOM_ATTR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_ATTR, GXmlDomAttr))
#define GXML_IS_DOM_ATTR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_ATTR))
#define GXML_DOM_ATTR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_ATTR, GXmlDomAttrIface))

typedef struct _GXmlDomAttr GXmlDomAttr;
typedef struct _GXmlDomAttrIface GXmlDomAttrIface;

#define GXML_TYPE_GATTRIBUTE (gxml_gattribute_get_type ())
#define GXML_GATTRIBUTE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GATTRIBUTE, GXmlGAttribute))
#define GXML_GATTRIBUTE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GATTRIBUTE, GXmlGAttributeClass))
#define GXML_IS_GATTRIBUTE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GATTRIBUTE))
#define GXML_IS_GATTRIBUTE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GATTRIBUTE))
#define GXML_GATTRIBUTE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GATTRIBUTE, GXmlGAttributeClass))

typedef struct _GXmlGAttribute GXmlGAttribute;
typedef struct _GXmlGAttributeClass GXmlGAttributeClass;
typedef struct _GXmlGAttributePrivate GXmlGAttributePrivate;

#define GXML_TYPE_GCHILD_NODE (gxml_gchild_node_get_type ())
#define GXML_GCHILD_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GCHILD_NODE, GXmlGChildNode))
#define GXML_GCHILD_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GCHILD_NODE, GXmlGChildNodeClass))
#define GXML_IS_GCHILD_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GCHILD_NODE))
#define GXML_IS_GCHILD_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GCHILD_NODE))
#define GXML_GCHILD_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GCHILD_NODE, GXmlGChildNodeClass))

typedef struct _GXmlGChildNode GXmlGChildNode;
typedef struct _GXmlGChildNodeClass GXmlGChildNodeClass;
typedef struct _GXmlGChildNodePrivate GXmlGChildNodePrivate;

#define GXML_TYPE_GNON_DOCUMENT_CHILD_NODE (gxml_gnon_document_child_node_get_type ())
#define GXML_GNON_DOCUMENT_CHILD_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GNON_DOCUMENT_CHILD_NODE, GXmlGNonDocumentChildNode))
#define GXML_GNON_DOCUMENT_CHILD_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GNON_DOCUMENT_CHILD_NODE, GXmlGNonDocumentChildNodeClass))
#define GXML_IS_GNON_DOCUMENT_CHILD_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GNON_DOCUMENT_CHILD_NODE))
#define GXML_IS_GNON_DOCUMENT_CHILD_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GNON_DOCUMENT_CHILD_NODE))
#define GXML_GNON_DOCUMENT_CHILD_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GNON_DOCUMENT_CHILD_NODE, GXmlGNonDocumentChildNodeClass))

typedef struct _GXmlGNonDocumentChildNode GXmlGNonDocumentChildNode;
typedef struct _GXmlGNonDocumentChildNodeClass GXmlGNonDocumentChildNodeClass;
typedef struct _GXmlGNonDocumentChildNodePrivate GXmlGNonDocumentChildNodePrivate;

#define GXML_TYPE_GCHARACTER_DATA (gxml_gcharacter_data_get_type ())
#define GXML_GCHARACTER_DATA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GCHARACTER_DATA, GXmlGCharacterData))
#define GXML_GCHARACTER_DATA_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GCHARACTER_DATA, GXmlGCharacterDataClass))
#define GXML_IS_GCHARACTER_DATA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GCHARACTER_DATA))
#define GXML_IS_GCHARACTER_DATA_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GCHARACTER_DATA))
#define GXML_GCHARACTER_DATA_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GCHARACTER_DATA, GXmlGCharacterDataClass))

typedef struct _GXmlGCharacterData GXmlGCharacterData;
typedef struct _GXmlGCharacterDataClass GXmlGCharacterDataClass;
typedef struct _GXmlGCharacterDataPrivate GXmlGCharacterDataPrivate;

#define GXML_TYPE_GCOMMENT (gxml_gcomment_get_type ())
#define GXML_GCOMMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GCOMMENT, GXmlGComment))
#define GXML_GCOMMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GCOMMENT, GXmlGCommentClass))
#define GXML_IS_GCOMMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GCOMMENT))
#define GXML_IS_GCOMMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GCOMMENT))
#define GXML_GCOMMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GCOMMENT, GXmlGCommentClass))

typedef struct _GXmlGComment GXmlGComment;
typedef struct _GXmlGCommentClass GXmlGCommentClass;
typedef struct _GXmlGCommentPrivate GXmlGCommentPrivate;

#define GXML_TYPE_GCDATA (gxml_gcdata_get_type ())
#define GXML_GCDATA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GCDATA, GXmlGCDATA))
#define GXML_GCDATA_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GCDATA, GXmlGCDATAClass))
#define GXML_IS_GCDATA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GCDATA))
#define GXML_IS_GCDATA_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GCDATA))
#define GXML_GCDATA_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GCDATA, GXmlGCDATAClass))

typedef struct _GXmlGCDATA GXmlGCDATA;
typedef struct _GXmlGCDATAClass GXmlGCDATAClass;
typedef struct _GXmlGCDATAPrivate GXmlGCDATAPrivate;
typedef struct _GXmlGImplementationPrivate GXmlGImplementationPrivate;

#define GXML_TYPE_GDOCUMENT_TYPE (gxml_gdocument_type_get_type ())
#define GXML_GDOCUMENT_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GDOCUMENT_TYPE, GXmlGDocumentType))
#define GXML_GDOCUMENT_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GDOCUMENT_TYPE, GXmlGDocumentTypeClass))
#define GXML_IS_GDOCUMENT_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GDOCUMENT_TYPE))
#define GXML_IS_GDOCUMENT_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GDOCUMENT_TYPE))
#define GXML_GDOCUMENT_TYPE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GDOCUMENT_TYPE, GXmlGDocumentTypeClass))

typedef struct _GXmlGDocumentType GXmlGDocumentType;
typedef struct _GXmlGDocumentTypeClass GXmlGDocumentTypeClass;
typedef struct _GXmlGDocumentTypePrivate GXmlGDocumentTypePrivate;

#define GXML_TYPE_GDOCUMENT_FRAGMENT (gxml_gdocument_fragment_get_type ())
#define GXML_GDOCUMENT_FRAGMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GDOCUMENT_FRAGMENT, GXmlGDocumentFragment))
#define GXML_GDOCUMENT_FRAGMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GDOCUMENT_FRAGMENT, GXmlGDocumentFragmentClass))
#define GXML_IS_GDOCUMENT_FRAGMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GDOCUMENT_FRAGMENT))
#define GXML_IS_GDOCUMENT_FRAGMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GDOCUMENT_FRAGMENT))
#define GXML_GDOCUMENT_FRAGMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GDOCUMENT_FRAGMENT, GXmlGDocumentFragmentClass))

typedef struct _GXmlGDocumentFragment GXmlGDocumentFragment;
typedef struct _GXmlGDocumentFragmentClass GXmlGDocumentFragmentClass;
typedef struct _GXmlGDocumentFragmentPrivate GXmlGDocumentFragmentPrivate;

#define GXML_TYPE_GDOM_NODE_ITERATOR (gxml_gdom_node_iterator_get_type ())
#define GXML_GDOM_NODE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GDOM_NODE_ITERATOR, GXmlGDomNodeIterator))
#define GXML_GDOM_NODE_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GDOM_NODE_ITERATOR, GXmlGDomNodeIteratorClass))
#define GXML_IS_GDOM_NODE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GDOM_NODE_ITERATOR))
#define GXML_IS_GDOM_NODE_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GDOM_NODE_ITERATOR))
#define GXML_GDOM_NODE_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GDOM_NODE_ITERATOR, GXmlGDomNodeIteratorClass))

typedef struct _GXmlGDomNodeIterator GXmlGDomNodeIterator;
typedef struct _GXmlGDomNodeIteratorClass GXmlGDomNodeIteratorClass;
typedef struct _GXmlGDomNodeIteratorPrivate GXmlGDomNodeIteratorPrivate;

#define GXML_TYPE_GDOM_TREE_WALKER (gxml_gdom_tree_walker_get_type ())
#define GXML_GDOM_TREE_WALKER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GDOM_TREE_WALKER, GXmlGDomTreeWalker))
#define GXML_GDOM_TREE_WALKER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GDOM_TREE_WALKER, GXmlGDomTreeWalkerClass))
#define GXML_IS_GDOM_TREE_WALKER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GDOM_TREE_WALKER))
#define GXML_IS_GDOM_TREE_WALKER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GDOM_TREE_WALKER))
#define GXML_GDOM_TREE_WALKER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GDOM_TREE_WALKER, GXmlGDomTreeWalkerClass))

typedef struct _GXmlGDomTreeWalker GXmlGDomTreeWalker;
typedef struct _GXmlGDomTreeWalkerClass GXmlGDomTreeWalkerClass;
typedef struct _GXmlGDomTreeWalkerPrivate GXmlGDomTreeWalkerPrivate;

#define GXML_TYPE_GDOM_TOKEN_LIST (gxml_gdom_token_list_get_type ())
#define GXML_GDOM_TOKEN_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GDOM_TOKEN_LIST, GXmlGDomTokenList))
#define GXML_GDOM_TOKEN_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GDOM_TOKEN_LIST, GXmlGDomTokenListClass))
#define GXML_IS_GDOM_TOKEN_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GDOM_TOKEN_LIST))
#define GXML_IS_GDOM_TOKEN_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GDOM_TOKEN_LIST))
#define GXML_GDOM_TOKEN_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GDOM_TOKEN_LIST, GXmlGDomTokenListClass))

typedef struct _GXmlGDomTokenList GXmlGDomTokenList;
typedef struct _GXmlGDomTokenListClass GXmlGDomTokenListClass;
typedef struct _GXmlGDomTokenListPrivate GXmlGDomTokenListPrivate;

#define GXML_TYPE_DOM_SETTABLE_TOKEN_LIST (gxml_dom_settable_token_list_get_type ())
#define GXML_DOM_SETTABLE_TOKEN_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_SETTABLE_TOKEN_LIST, GXmlDomSettableTokenList))
#define GXML_IS_DOM_SETTABLE_TOKEN_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_SETTABLE_TOKEN_LIST))
#define GXML_DOM_SETTABLE_TOKEN_LIST_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_SETTABLE_TOKEN_LIST, GXmlDomSettableTokenListIface))

typedef struct _GXmlDomSettableTokenList GXmlDomSettableTokenList;
typedef struct _GXmlDomSettableTokenListIface GXmlDomSettableTokenListIface;

#define GXML_TYPE_GDOM_SETTABLE_TOKEN_LIST (gxml_gdom_settable_token_list_get_type ())
#define GXML_GDOM_SETTABLE_TOKEN_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GDOM_SETTABLE_TOKEN_LIST, GXmlGDomSettableTokenList))
#define GXML_GDOM_SETTABLE_TOKEN_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GDOM_SETTABLE_TOKEN_LIST, GXmlGDomSettableTokenListClass))
#define GXML_IS_GDOM_SETTABLE_TOKEN_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GDOM_SETTABLE_TOKEN_LIST))
#define GXML_IS_GDOM_SETTABLE_TOKEN_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GDOM_SETTABLE_TOKEN_LIST))
#define GXML_GDOM_SETTABLE_TOKEN_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GDOM_SETTABLE_TOKEN_LIST, GXmlGDomSettableTokenListClass))

typedef struct _GXmlGDomSettableTokenList GXmlGDomSettableTokenList;
typedef struct _GXmlGDomSettableTokenListClass GXmlGDomSettableTokenListClass;
typedef struct _GXmlGDomSettableTokenListPrivate GXmlGDomSettableTokenListPrivate;

#define GXML_TYPE_GDOM_HTML_COLLECTION (gxml_gdom_html_collection_get_type ())
#define GXML_GDOM_HTML_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GDOM_HTML_COLLECTION, GXmlGDomHTMLCollection))
#define GXML_GDOM_HTML_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GDOM_HTML_COLLECTION, GXmlGDomHTMLCollectionClass))
#define GXML_IS_GDOM_HTML_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GDOM_HTML_COLLECTION))
#define GXML_IS_GDOM_HTML_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GDOM_HTML_COLLECTION))
#define GXML_GDOM_HTML_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GDOM_HTML_COLLECTION, GXmlGDomHTMLCollectionClass))

typedef struct _GXmlGDomHTMLCollection GXmlGDomHTMLCollection;
typedef struct _GXmlGDomHTMLCollectionClass GXmlGDomHTMLCollectionClass;
typedef struct _GXmlGDomHTMLCollectionPrivate GXmlGDomHTMLCollectionPrivate;

#define GXML_TYPE_GDOM_EVENT (gxml_gdom_event_get_type ())
#define GXML_GDOM_EVENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GDOM_EVENT, GXmlGDomEvent))
#define GXML_GDOM_EVENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GDOM_EVENT, GXmlGDomEventClass))
#define GXML_IS_GDOM_EVENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GDOM_EVENT))
#define GXML_IS_GDOM_EVENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GDOM_EVENT))
#define GXML_GDOM_EVENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GDOM_EVENT, GXmlGDomEventClass))

typedef struct _GXmlGDomEvent GXmlGDomEvent;
typedef struct _GXmlGDomEventClass GXmlGDomEventClass;
typedef struct _GXmlGDomEventPrivate GXmlGDomEventPrivate;

#define GXML_DOM_EVENT_TYPE_FLAGS (gxml_dom_event_flags_get_type ())

#define GXML_TYPE_GDOM_CUSTOM_EVENT (gxml_gdom_custom_event_get_type ())
#define GXML_GDOM_CUSTOM_EVENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GDOM_CUSTOM_EVENT, GXmlGDomCustomEvent))
#define GXML_GDOM_CUSTOM_EVENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GDOM_CUSTOM_EVENT, GXmlGDomCustomEventClass))
#define GXML_IS_GDOM_CUSTOM_EVENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GDOM_CUSTOM_EVENT))
#define GXML_IS_GDOM_CUSTOM_EVENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GDOM_CUSTOM_EVENT))
#define GXML_GDOM_CUSTOM_EVENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GDOM_CUSTOM_EVENT, GXmlGDomCustomEventClass))

typedef struct _GXmlGDomCustomEvent GXmlGDomCustomEvent;
typedef struct _GXmlGDomCustomEventClass GXmlGDomCustomEventClass;
typedef struct _GXmlGDomCustomEventPrivate GXmlGDomCustomEventPrivate;

#define GXML_TYPE_GDOM_RANGE (gxml_gdom_range_get_type ())
#define GXML_GDOM_RANGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GDOM_RANGE, GXmlGDomRange))
#define GXML_GDOM_RANGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GDOM_RANGE, GXmlGDomRangeClass))
#define GXML_IS_GDOM_RANGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GDOM_RANGE))
#define GXML_IS_GDOM_RANGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GDOM_RANGE))
#define GXML_GDOM_RANGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GDOM_RANGE, GXmlGDomRangeClass))

typedef struct _GXmlGDomRange GXmlGDomRange;
typedef struct _GXmlGDomRangeClass GXmlGDomRangeClass;
typedef struct _GXmlGDomRangePrivate GXmlGDomRangePrivate;

#define GXML_TYPE_GELEMENT (gxml_gelement_get_type ())
#define GXML_GELEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GELEMENT, GXmlGElement))
#define GXML_GELEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GELEMENT, GXmlGElementClass))
#define GXML_IS_GELEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GELEMENT))
#define GXML_IS_GELEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GELEMENT))
#define GXML_GELEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GELEMENT, GXmlGElementClass))

typedef struct _GXmlGElement GXmlGElement;
typedef struct _GXmlGElementClass GXmlGElementClass;
typedef struct _GXmlGElementPrivate GXmlGElementPrivate;

#define GXML_TYPE_GNAMESPACE (gxml_gnamespace_get_type ())
#define GXML_GNAMESPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GNAMESPACE, GXmlGNamespace))
#define GXML_GNAMESPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GNAMESPACE, GXmlGNamespaceClass))
#define GXML_IS_GNAMESPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GNAMESPACE))
#define GXML_IS_GNAMESPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GNAMESPACE))
#define GXML_GNAMESPACE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GNAMESPACE, GXmlGNamespaceClass))

typedef struct _GXmlGNamespace GXmlGNamespace;
typedef struct _GXmlGNamespaceClass GXmlGNamespaceClass;
typedef struct _GXmlGNamespacePrivate GXmlGNamespacePrivate;

#define GXML_TYPE_GPROCESSING_INSTRUCTION (gxml_gprocessing_instruction_get_type ())
#define GXML_GPROCESSING_INSTRUCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GPROCESSING_INSTRUCTION, GXmlGProcessingInstruction))
#define GXML_GPROCESSING_INSTRUCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GPROCESSING_INSTRUCTION, GXmlGProcessingInstructionClass))
#define GXML_IS_GPROCESSING_INSTRUCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GPROCESSING_INSTRUCTION))
#define GXML_IS_GPROCESSING_INSTRUCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GPROCESSING_INSTRUCTION))
#define GXML_GPROCESSING_INSTRUCTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GPROCESSING_INSTRUCTION, GXmlGProcessingInstructionClass))

typedef struct _GXmlGProcessingInstruction GXmlGProcessingInstruction;
typedef struct _GXmlGProcessingInstructionClass GXmlGProcessingInstructionClass;
typedef struct _GXmlGProcessingInstructionPrivate GXmlGProcessingInstructionPrivate;

#define GXML_TYPE_GTEXT (gxml_gtext_get_type ())
#define GXML_GTEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GTEXT, GXmlGText))
#define GXML_GTEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GTEXT, GXmlGTextClass))
#define GXML_IS_GTEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GTEXT))
#define GXML_IS_GTEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GTEXT))
#define GXML_GTEXT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GTEXT, GXmlGTextClass))

typedef struct _GXmlGText GXmlGText;
typedef struct _GXmlGTextClass GXmlGTextClass;
typedef struct _GXmlGTextPrivate GXmlGTextPrivate;

#define GXML_TYPE_GHASH_MAP_ATTR (gxml_ghash_map_attr_get_type ())
#define GXML_GHASH_MAP_ATTR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GHASH_MAP_ATTR, GXmlGHashMapAttr))
#define GXML_GHASH_MAP_ATTR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GHASH_MAP_ATTR, GXmlGHashMapAttrClass))
#define GXML_IS_GHASH_MAP_ATTR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GHASH_MAP_ATTR))
#define GXML_IS_GHASH_MAP_ATTR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GHASH_MAP_ATTR))
#define GXML_GHASH_MAP_ATTR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GHASH_MAP_ATTR, GXmlGHashMapAttrClass))

typedef struct _GXmlGHashMapAttr GXmlGHashMapAttr;
typedef struct _GXmlGHashMapAttrClass GXmlGHashMapAttrClass;
typedef struct _GXmlGHashMapAttrPrivate GXmlGHashMapAttrPrivate;

#define GXML_GHASH_MAP_ATTR_TYPE_ENTRY (gxml_ghash_map_attr_entry_get_type ())
#define GXML_GHASH_MAP_ATTR_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_GHASH_MAP_ATTR_TYPE_ENTRY, GXmlGHashMapAttrEntry))
#define GXML_GHASH_MAP_ATTR_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_GHASH_MAP_ATTR_TYPE_ENTRY, GXmlGHashMapAttrEntryClass))
#define GXML_GHASH_MAP_ATTR_IS_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_GHASH_MAP_ATTR_TYPE_ENTRY))
#define GXML_GHASH_MAP_ATTR_IS_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_GHASH_MAP_ATTR_TYPE_ENTRY))
#define GXML_GHASH_MAP_ATTR_ENTRY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_GHASH_MAP_ATTR_TYPE_ENTRY, GXmlGHashMapAttrEntryClass))

typedef struct _GXmlGHashMapAttrEntry GXmlGHashMapAttrEntry;
typedef struct _GXmlGHashMapAttrEntryClass GXmlGHashMapAttrEntryClass;
typedef struct _GXmlGHashMapAttrEntryPrivate GXmlGHashMapAttrEntryPrivate;

#define GXML_GHASH_MAP_ATTR_TYPE_ITERATOR (gxml_ghash_map_attr_iterator_get_type ())
#define GXML_GHASH_MAP_ATTR_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_GHASH_MAP_ATTR_TYPE_ITERATOR, GXmlGHashMapAttrIterator))
#define GXML_GHASH_MAP_ATTR_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_GHASH_MAP_ATTR_TYPE_ITERATOR, GXmlGHashMapAttrIteratorClass))
#define GXML_GHASH_MAP_ATTR_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_GHASH_MAP_ATTR_TYPE_ITERATOR))
#define GXML_GHASH_MAP_ATTR_IS_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_GHASH_MAP_ATTR_TYPE_ITERATOR))
#define GXML_GHASH_MAP_ATTR_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_GHASH_MAP_ATTR_TYPE_ITERATOR, GXmlGHashMapAttrIteratorClass))

typedef struct _GXmlGHashMapAttrIterator GXmlGHashMapAttrIterator;
typedef struct _GXmlGHashMapAttrIteratorClass GXmlGHashMapAttrIteratorClass;
typedef struct _GXmlGHashMapAttrIteratorPrivate GXmlGHashMapAttrIteratorPrivate;

#define GXML_TYPE_GLIST_CHILDREN (gxml_glist_children_get_type ())
#define GXML_GLIST_CHILDREN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GLIST_CHILDREN, GXmlGListChildren))
#define GXML_GLIST_CHILDREN_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GLIST_CHILDREN, GXmlGListChildrenClass))
#define GXML_IS_GLIST_CHILDREN(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GLIST_CHILDREN))
#define GXML_IS_GLIST_CHILDREN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GLIST_CHILDREN))
#define GXML_GLIST_CHILDREN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GLIST_CHILDREN, GXmlGListChildrenClass))

typedef struct _GXmlGListChildren GXmlGListChildren;
typedef struct _GXmlGListChildrenClass GXmlGListChildrenClass;
typedef struct _GXmlGListChildrenPrivate GXmlGListChildrenPrivate;

#define GXML_GLIST_CHILDREN_TYPE_ITERATOR (gxml_glist_children_iterator_get_type ())
#define GXML_GLIST_CHILDREN_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_GLIST_CHILDREN_TYPE_ITERATOR, GXmlGListChildrenIterator))
#define GXML_GLIST_CHILDREN_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_GLIST_CHILDREN_TYPE_ITERATOR, GXmlGListChildrenIteratorClass))
#define GXML_GLIST_CHILDREN_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_GLIST_CHILDREN_TYPE_ITERATOR))
#define GXML_GLIST_CHILDREN_IS_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_GLIST_CHILDREN_TYPE_ITERATOR))
#define GXML_GLIST_CHILDREN_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_GLIST_CHILDREN_TYPE_ITERATOR, GXmlGListChildrenIteratorClass))

typedef struct _GXmlGListChildrenIterator GXmlGListChildrenIterator;
typedef struct _GXmlGListChildrenIteratorClass GXmlGListChildrenIteratorClass;
typedef struct _GXmlGListChildrenIteratorPrivate GXmlGListChildrenIteratorPrivate;

#define GXML_TYPE_GLIST_NAMESPACES (gxml_glist_namespaces_get_type ())
#define GXML_GLIST_NAMESPACES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GLIST_NAMESPACES, GXmlGListNamespaces))
#define GXML_GLIST_NAMESPACES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GLIST_NAMESPACES, GXmlGListNamespacesClass))
#define GXML_IS_GLIST_NAMESPACES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GLIST_NAMESPACES))
#define GXML_IS_GLIST_NAMESPACES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GLIST_NAMESPACES))
#define GXML_GLIST_NAMESPACES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GLIST_NAMESPACES, GXmlGListNamespacesClass))

typedef struct _GXmlGListNamespaces GXmlGListNamespaces;
typedef struct _GXmlGListNamespacesClass GXmlGListNamespacesClass;
typedef struct _GXmlGListNamespacesPrivate GXmlGListNamespacesPrivate;

#define GXML_GLIST_NAMESPACES_TYPE_ITERATOR (gxml_glist_namespaces_iterator_get_type ())
#define GXML_GLIST_NAMESPACES_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_GLIST_NAMESPACES_TYPE_ITERATOR, GXmlGListNamespacesIterator))
#define GXML_GLIST_NAMESPACES_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_GLIST_NAMESPACES_TYPE_ITERATOR, GXmlGListNamespacesIteratorClass))
#define GXML_GLIST_NAMESPACES_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_GLIST_NAMESPACES_TYPE_ITERATOR))
#define GXML_GLIST_NAMESPACES_IS_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_GLIST_NAMESPACES_TYPE_ITERATOR))
#define GXML_GLIST_NAMESPACES_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_GLIST_NAMESPACES_TYPE_ITERATOR, GXmlGListNamespacesIteratorClass))

typedef struct _GXmlGListNamespacesIterator GXmlGListNamespacesIterator;
typedef struct _GXmlGListNamespacesIteratorClass GXmlGListNamespacesIteratorClass;
typedef struct _GXmlGListNamespacesIteratorPrivate GXmlGListNamespacesIteratorPrivate;
typedef struct _GXmlDomNodeFilterPrivate GXmlDomNodeFilterPrivate;
typedef struct _GXmlDomElementListPrivate GXmlDomElementListPrivate;

#define GXML_TYPE_DOM_EVENT_INIT (gxml_dom_event_init_get_type ())
#define GXML_DOM_EVENT_INIT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_EVENT_INIT, GXmlDomEventInit))
#define GXML_DOM_EVENT_INIT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_DOM_EVENT_INIT, GXmlDomEventInitClass))
#define GXML_IS_DOM_EVENT_INIT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_EVENT_INIT))
#define GXML_IS_DOM_EVENT_INIT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_DOM_EVENT_INIT))
#define GXML_DOM_EVENT_INIT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_DOM_EVENT_INIT, GXmlDomEventInitClass))

typedef struct _GXmlDomEventInit GXmlDomEventInit;
typedef struct _GXmlDomEventInitClass GXmlDomEventInitClass;
typedef struct _GXmlDomEventInitPrivate GXmlDomEventInitPrivate;

#define GXML_TYPE_DOM_CUSTOM_EVENT (gxml_dom_custom_event_get_type ())
#define GXML_DOM_CUSTOM_EVENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_CUSTOM_EVENT, GXmlDomCustomEvent))
#define GXML_IS_DOM_CUSTOM_EVENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_CUSTOM_EVENT))
#define GXML_DOM_CUSTOM_EVENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_CUSTOM_EVENT, GXmlDomCustomEventIface))

typedef struct _GXmlDomCustomEvent GXmlDomCustomEvent;
typedef struct _GXmlDomCustomEventIface GXmlDomCustomEventIface;

#define GXML_TYPE_DOM_CUSTOM_EVENT_INIT (gxml_dom_custom_event_init_get_type ())
#define GXML_DOM_CUSTOM_EVENT_INIT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_CUSTOM_EVENT_INIT, GXmlDomCustomEventInit))
#define GXML_DOM_CUSTOM_EVENT_INIT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_DOM_CUSTOM_EVENT_INIT, GXmlDomCustomEventInitClass))
#define GXML_IS_DOM_CUSTOM_EVENT_INIT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_CUSTOM_EVENT_INIT))
#define GXML_IS_DOM_CUSTOM_EVENT_INIT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_DOM_CUSTOM_EVENT_INIT))
#define GXML_DOM_CUSTOM_EVENT_INIT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_DOM_CUSTOM_EVENT_INIT, GXmlDomCustomEventInitClass))

typedef struct _GXmlDomCustomEventInit GXmlDomCustomEventInit;
typedef struct _GXmlDomCustomEventInitClass GXmlDomCustomEventInitClass;
typedef struct _GXmlDomCustomEventInitPrivate GXmlDomCustomEventInitPrivate;
typedef struct _GXmlDomTimeStampPrivate GXmlDomTimeStampPrivate;

#define GXML_TYPE_DOM_MUTATION_OBSERVER (gxml_dom_mutation_observer_get_type ())
#define GXML_DOM_MUTATION_OBSERVER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_MUTATION_OBSERVER, GXmlDomMutationObserver))
#define GXML_IS_DOM_MUTATION_OBSERVER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_MUTATION_OBSERVER))
#define GXML_DOM_MUTATION_OBSERVER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_MUTATION_OBSERVER, GXmlDomMutationObserverIface))

typedef struct _GXmlDomMutationObserver GXmlDomMutationObserver;
typedef struct _GXmlDomMutationObserverIface GXmlDomMutationObserverIface;

#define GXML_TYPE_DOM_MUTATION_OBSERVER_INIT (gxml_dom_mutation_observer_init_get_type ())
#define GXML_DOM_MUTATION_OBSERVER_INIT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_MUTATION_OBSERVER_INIT, GXmlDomMutationObserverInit))
#define GXML_DOM_MUTATION_OBSERVER_INIT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_DOM_MUTATION_OBSERVER_INIT, GXmlDomMutationObserverInitClass))
#define GXML_IS_DOM_MUTATION_OBSERVER_INIT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_MUTATION_OBSERVER_INIT))
#define GXML_IS_DOM_MUTATION_OBSERVER_INIT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_DOM_MUTATION_OBSERVER_INIT))
#define GXML_DOM_MUTATION_OBSERVER_INIT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_DOM_MUTATION_OBSERVER_INIT, GXmlDomMutationObserverInitClass))

typedef struct _GXmlDomMutationObserverInit GXmlDomMutationObserverInit;
typedef struct _GXmlDomMutationObserverInitClass GXmlDomMutationObserverInitClass;

#define GXML_TYPE_DOM_MUTATION_RECORD (gxml_dom_mutation_record_get_type ())
#define GXML_DOM_MUTATION_RECORD(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_MUTATION_RECORD, GXmlDomMutationRecord))
#define GXML_IS_DOM_MUTATION_RECORD(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_MUTATION_RECORD))
#define GXML_DOM_MUTATION_RECORD_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_MUTATION_RECORD, GXmlDomMutationRecordIface))

typedef struct _GXmlDomMutationRecord GXmlDomMutationRecord;
typedef struct _GXmlDomMutationRecordIface GXmlDomMutationRecordIface;
typedef struct _GXmlDomMutationObserverInitPrivate GXmlDomMutationObserverInitPrivate;

#define GXML_TYPE_DOM_ERROR_NAME (gxml_dom_error_name_get_type ())
#define GXML_DOM_ERROR_NAME(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_ERROR_NAME, GXmlDomErrorName))
#define GXML_DOM_ERROR_NAME_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_DOM_ERROR_NAME, GXmlDomErrorNameClass))
#define GXML_IS_DOM_ERROR_NAME(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_ERROR_NAME))
#define GXML_IS_DOM_ERROR_NAME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_DOM_ERROR_NAME))
#define GXML_DOM_ERROR_NAME_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_DOM_ERROR_NAME, GXmlDomErrorNameClass))

typedef struct _GXmlDomErrorName GXmlDomErrorName;
typedef struct _GXmlDomErrorNameClass GXmlDomErrorNameClass;
typedef struct _GXmlDomErrorNamePrivate GXmlDomErrorNamePrivate;

#define GXML_TYPE_GX_PATH_OBJECT (gxml_gx_path_object_get_type ())
#define GXML_GX_PATH_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GX_PATH_OBJECT, GXmlGXPathObject))
#define GXML_GX_PATH_OBJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GX_PATH_OBJECT, GXmlGXPathObjectClass))
#define GXML_IS_GX_PATH_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GX_PATH_OBJECT))
#define GXML_IS_GX_PATH_OBJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GX_PATH_OBJECT))
#define GXML_GX_PATH_OBJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GX_PATH_OBJECT, GXmlGXPathObjectClass))

typedef struct _GXmlGXPathObject GXmlGXPathObject;
typedef struct _GXmlGXPathObjectClass GXmlGXPathObjectClass;
typedef struct _GXmlGXPathObjectPrivate GXmlGXPathObjectPrivate;

#define GXML_TYPE_GOM_NODE (gxml_gom_node_get_type ())
#define GXML_GOM_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_NODE, GXmlGomNode))
#define GXML_GOM_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_NODE, GXmlGomNodeClass))
#define GXML_IS_GOM_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_NODE))
#define GXML_IS_GOM_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_NODE))
#define GXML_GOM_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_NODE, GXmlGomNodeClass))

typedef struct _GXmlGomNode GXmlGomNode;
typedef struct _GXmlGomNodeClass GXmlGomNodeClass;
typedef struct _GXmlGomNodePrivate GXmlGomNodePrivate;

#define GXML_TYPE_GOM_NODE_LIST (gxml_gom_node_list_get_type ())
#define GXML_GOM_NODE_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_NODE_LIST, GXmlGomNodeList))
#define GXML_GOM_NODE_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_NODE_LIST, GXmlGomNodeListClass))
#define GXML_IS_GOM_NODE_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_NODE_LIST))
#define GXML_IS_GOM_NODE_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_NODE_LIST))
#define GXML_GOM_NODE_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_NODE_LIST, GXmlGomNodeListClass))

typedef struct _GXmlGomNodeList GXmlGomNodeList;
typedef struct _GXmlGomNodeListClass GXmlGomNodeListClass;

#define GXML_TYPE_GOM_DOCUMENT (gxml_gom_document_get_type ())
#define GXML_GOM_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_DOCUMENT, GXmlGomDocument))
#define GXML_GOM_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_DOCUMENT, GXmlGomDocumentClass))
#define GXML_IS_GOM_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_DOCUMENT))
#define GXML_IS_GOM_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_DOCUMENT))
#define GXML_GOM_DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_DOCUMENT, GXmlGomDocumentClass))

typedef struct _GXmlGomDocument GXmlGomDocument;
typedef struct _GXmlGomDocumentClass GXmlGomDocumentClass;
typedef struct _GXmlGomDocumentPrivate GXmlGomDocumentPrivate;

#define GXML_TYPE_GOM_IMPLEMENTATION (gxml_gom_implementation_get_type ())
#define GXML_GOM_IMPLEMENTATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_IMPLEMENTATION, GXmlGomImplementation))
#define GXML_GOM_IMPLEMENTATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_IMPLEMENTATION, GXmlGomImplementationClass))
#define GXML_IS_GOM_IMPLEMENTATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_IMPLEMENTATION))
#define GXML_IS_GOM_IMPLEMENTATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_IMPLEMENTATION))
#define GXML_GOM_IMPLEMENTATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_IMPLEMENTATION, GXmlGomImplementationClass))

typedef struct _GXmlGomImplementation GXmlGomImplementation;
typedef struct _GXmlGomImplementationClass GXmlGomImplementationClass;
typedef struct _GXmlGomImplementationPrivate GXmlGomImplementationPrivate;

#define GXML_TYPE_GOM_DOCUMENT_TYPE (gxml_gom_document_type_get_type ())
#define GXML_GOM_DOCUMENT_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_DOCUMENT_TYPE, GXmlGomDocumentType))
#define GXML_GOM_DOCUMENT_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_DOCUMENT_TYPE, GXmlGomDocumentTypeClass))
#define GXML_IS_GOM_DOCUMENT_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_DOCUMENT_TYPE))
#define GXML_IS_GOM_DOCUMENT_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_DOCUMENT_TYPE))
#define GXML_GOM_DOCUMENT_TYPE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_DOCUMENT_TYPE, GXmlGomDocumentTypeClass))

typedef struct _GXmlGomDocumentType GXmlGomDocumentType;
typedef struct _GXmlGomDocumentTypeClass GXmlGomDocumentTypeClass;
typedef struct _GXmlGomDocumentTypePrivate GXmlGomDocumentTypePrivate;

#define GXML_TYPE_GOM_DOCUMENT_FRAGMENT (gxml_gom_document_fragment_get_type ())
#define GXML_GOM_DOCUMENT_FRAGMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_DOCUMENT_FRAGMENT, GXmlGomDocumentFragment))
#define GXML_GOM_DOCUMENT_FRAGMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_DOCUMENT_FRAGMENT, GXmlGomDocumentFragmentClass))
#define GXML_IS_GOM_DOCUMENT_FRAGMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_DOCUMENT_FRAGMENT))
#define GXML_IS_GOM_DOCUMENT_FRAGMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_DOCUMENT_FRAGMENT))
#define GXML_GOM_DOCUMENT_FRAGMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_DOCUMENT_FRAGMENT, GXmlGomDocumentFragmentClass))

typedef struct _GXmlGomDocumentFragment GXmlGomDocumentFragment;
typedef struct _GXmlGomDocumentFragmentClass GXmlGomDocumentFragmentClass;
typedef struct _GXmlGomDocumentFragmentPrivate GXmlGomDocumentFragmentPrivate;

#define GXML_TYPE_GOM_OBJECT (gxml_gom_object_get_type ())
#define GXML_GOM_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_OBJECT, GXmlGomObject))
#define GXML_IS_GOM_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_OBJECT))
#define GXML_GOM_OBJECT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_GOM_OBJECT, GXmlGomObjectIface))

typedef struct _GXmlGomObject GXmlGomObject;
typedef struct _GXmlGomObjectIface GXmlGomObjectIface;

#define GXML_TYPE_GOM_ELEMENT (gxml_gom_element_get_type ())
#define GXML_GOM_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_ELEMENT, GXmlGomElement))
#define GXML_GOM_ELEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_ELEMENT, GXmlGomElementClass))
#define GXML_IS_GOM_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_ELEMENT))
#define GXML_IS_GOM_ELEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_ELEMENT))
#define GXML_GOM_ELEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_ELEMENT, GXmlGomElementClass))

typedef struct _GXmlGomElement GXmlGomElement;
typedef struct _GXmlGomElementClass GXmlGomElementClass;
typedef struct _GXmlGomElementPrivate GXmlGomElementPrivate;

#define GXML_GOM_ELEMENT_TYPE_ATTRIBUTES (gxml_gom_element_attributes_get_type ())
#define GXML_GOM_ELEMENT_ATTRIBUTES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_GOM_ELEMENT_TYPE_ATTRIBUTES, GXmlGomElementAttributes))
#define GXML_GOM_ELEMENT_ATTRIBUTES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_GOM_ELEMENT_TYPE_ATTRIBUTES, GXmlGomElementAttributesClass))
#define GXML_GOM_ELEMENT_IS_ATTRIBUTES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_GOM_ELEMENT_TYPE_ATTRIBUTES))
#define GXML_GOM_ELEMENT_IS_ATTRIBUTES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_GOM_ELEMENT_TYPE_ATTRIBUTES))
#define GXML_GOM_ELEMENT_ATTRIBUTES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_GOM_ELEMENT_TYPE_ATTRIBUTES, GXmlGomElementAttributesClass))

typedef struct _GXmlGomElementAttributes GXmlGomElementAttributes;
typedef struct _GXmlGomElementAttributesClass GXmlGomElementAttributesClass;
typedef struct _GXmlGomElementAttributesPrivate GXmlGomElementAttributesPrivate;

#define GXML_TYPE_GOM_ATTR (gxml_gom_attr_get_type ())
#define GXML_GOM_ATTR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_ATTR, GXmlGomAttr))
#define GXML_GOM_ATTR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_ATTR, GXmlGomAttrClass))
#define GXML_IS_GOM_ATTR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_ATTR))
#define GXML_IS_GOM_ATTR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_ATTR))
#define GXML_GOM_ATTR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_ATTR, GXmlGomAttrClass))

typedef struct _GXmlGomAttr GXmlGomAttr;
typedef struct _GXmlGomAttrClass GXmlGomAttrClass;
typedef struct _GXmlGomAttrPrivate GXmlGomAttrPrivate;
typedef struct _GXmlGomNodeListPrivate GXmlGomNodeListPrivate;

#define GXML_TYPE_GOM_CHARACTER_DATA (gxml_gom_character_data_get_type ())
#define GXML_GOM_CHARACTER_DATA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_CHARACTER_DATA, GXmlGomCharacterData))
#define GXML_GOM_CHARACTER_DATA_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_CHARACTER_DATA, GXmlGomCharacterDataClass))
#define GXML_IS_GOM_CHARACTER_DATA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_CHARACTER_DATA))
#define GXML_IS_GOM_CHARACTER_DATA_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_CHARACTER_DATA))
#define GXML_GOM_CHARACTER_DATA_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_CHARACTER_DATA, GXmlGomCharacterDataClass))

typedef struct _GXmlGomCharacterData GXmlGomCharacterData;
typedef struct _GXmlGomCharacterDataClass GXmlGomCharacterDataClass;
typedef struct _GXmlGomCharacterDataPrivate GXmlGomCharacterDataPrivate;

#define GXML_TYPE_GOM_TEXT (gxml_gom_text_get_type ())
#define GXML_GOM_TEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_TEXT, GXmlGomText))
#define GXML_GOM_TEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_TEXT, GXmlGomTextClass))
#define GXML_IS_GOM_TEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_TEXT))
#define GXML_IS_GOM_TEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_TEXT))
#define GXML_GOM_TEXT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_TEXT, GXmlGomTextClass))

typedef struct _GXmlGomText GXmlGomText;
typedef struct _GXmlGomTextClass GXmlGomTextClass;
typedef struct _GXmlGomTextPrivate GXmlGomTextPrivate;

#define GXML_TYPE_GOM_PROCESSING_INSTRUCTION (gxml_gom_processing_instruction_get_type ())
#define GXML_GOM_PROCESSING_INSTRUCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_PROCESSING_INSTRUCTION, GXmlGomProcessingInstruction))
#define GXML_GOM_PROCESSING_INSTRUCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_PROCESSING_INSTRUCTION, GXmlGomProcessingInstructionClass))
#define GXML_IS_GOM_PROCESSING_INSTRUCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_PROCESSING_INSTRUCTION))
#define GXML_IS_GOM_PROCESSING_INSTRUCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_PROCESSING_INSTRUCTION))
#define GXML_GOM_PROCESSING_INSTRUCTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_PROCESSING_INSTRUCTION, GXmlGomProcessingInstructionClass))

typedef struct _GXmlGomProcessingInstruction GXmlGomProcessingInstruction;
typedef struct _GXmlGomProcessingInstructionClass GXmlGomProcessingInstructionClass;
typedef struct _GXmlGomProcessingInstructionPrivate GXmlGomProcessingInstructionPrivate;

#define GXML_TYPE_GOM_COMMENT (gxml_gom_comment_get_type ())
#define GXML_GOM_COMMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_COMMENT, GXmlGomComment))
#define GXML_GOM_COMMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_COMMENT, GXmlGomCommentClass))
#define GXML_IS_GOM_COMMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_COMMENT))
#define GXML_IS_GOM_COMMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_COMMENT))
#define GXML_GOM_COMMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_COMMENT, GXmlGomCommentClass))

typedef struct _GXmlGomComment GXmlGomComment;
typedef struct _GXmlGomCommentClass GXmlGomCommentClass;
typedef struct _GXmlGomCommentPrivate GXmlGomCommentPrivate;

#define GXML_TYPE_GOM_COLLECTION (gxml_gom_collection_get_type ())
#define GXML_GOM_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_COLLECTION, GXmlGomCollection))
#define GXML_IS_GOM_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_COLLECTION))
#define GXML_GOM_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_GOM_COLLECTION, GXmlGomCollectionIface))

typedef struct _GXmlGomCollection GXmlGomCollection;
typedef struct _GXmlGomCollectionIface GXmlGomCollectionIface;

#define GXML_TYPE_BASE_COLLECTION (gxml_base_collection_get_type ())
#define GXML_BASE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_BASE_COLLECTION, GXmlBaseCollection))
#define GXML_BASE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_BASE_COLLECTION, GXmlBaseCollectionClass))
#define GXML_IS_BASE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_BASE_COLLECTION))
#define GXML_IS_BASE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_BASE_COLLECTION))
#define GXML_BASE_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_BASE_COLLECTION, GXmlBaseCollectionClass))

typedef struct _GXmlBaseCollection GXmlBaseCollection;
typedef struct _GXmlBaseCollectionClass GXmlBaseCollectionClass;
typedef struct _GXmlBaseCollectionPrivate GXmlBaseCollectionPrivate;

#define GXML_TYPE_GOM_ARRAY_LIST (gxml_gom_array_list_get_type ())
#define GXML_GOM_ARRAY_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_ARRAY_LIST, GXmlGomArrayList))
#define GXML_GOM_ARRAY_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_ARRAY_LIST, GXmlGomArrayListClass))
#define GXML_IS_GOM_ARRAY_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_ARRAY_LIST))
#define GXML_IS_GOM_ARRAY_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_ARRAY_LIST))
#define GXML_GOM_ARRAY_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_ARRAY_LIST, GXmlGomArrayListClass))

typedef struct _GXmlGomArrayList GXmlGomArrayList;
typedef struct _GXmlGomArrayListClass GXmlGomArrayListClass;
typedef struct _GXmlGomArrayListPrivate GXmlGomArrayListPrivate;

#define GXML_TYPE_MAPPEABLE_ELEMENT (gxml_mappeable_element_get_type ())
#define GXML_MAPPEABLE_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_MAPPEABLE_ELEMENT, GXmlMappeableElement))
#define GXML_IS_MAPPEABLE_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_MAPPEABLE_ELEMENT))
#define GXML_MAPPEABLE_ELEMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_MAPPEABLE_ELEMENT, GXmlMappeableElementIface))

typedef struct _GXmlMappeableElement GXmlMappeableElement;
typedef struct _GXmlMappeableElementIface GXmlMappeableElementIface;

#define GXML_TYPE_GOM_HASH_MAP (gxml_gom_hash_map_get_type ())
#define GXML_GOM_HASH_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_HASH_MAP, GXmlGomHashMap))
#define GXML_GOM_HASH_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_HASH_MAP, GXmlGomHashMapClass))
#define GXML_IS_GOM_HASH_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_HASH_MAP))
#define GXML_IS_GOM_HASH_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_HASH_MAP))
#define GXML_GOM_HASH_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_HASH_MAP, GXmlGomHashMapClass))

typedef struct _GXmlGomHashMap GXmlGomHashMap;
typedef struct _GXmlGomHashMapClass GXmlGomHashMapClass;
typedef struct _GXmlGomHashMapPrivate GXmlGomHashMapPrivate;

#define GXML_TYPE_MAPPEABLE_ELEMENT_PAIR_KEY (gxml_mappeable_element_pair_key_get_type ())
#define GXML_MAPPEABLE_ELEMENT_PAIR_KEY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_MAPPEABLE_ELEMENT_PAIR_KEY, GXmlMappeableElementPairKey))
#define GXML_IS_MAPPEABLE_ELEMENT_PAIR_KEY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_MAPPEABLE_ELEMENT_PAIR_KEY))
#define GXML_MAPPEABLE_ELEMENT_PAIR_KEY_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_MAPPEABLE_ELEMENT_PAIR_KEY, GXmlMappeableElementPairKeyIface))

typedef struct _GXmlMappeableElementPairKey GXmlMappeableElementPairKey;
typedef struct _GXmlMappeableElementPairKeyIface GXmlMappeableElementPairKeyIface;

#define GXML_TYPE_GOM_HASH_PAIRED_MAP (gxml_gom_hash_paired_map_get_type ())
#define GXML_GOM_HASH_PAIRED_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_HASH_PAIRED_MAP, GXmlGomHashPairedMap))
#define GXML_GOM_HASH_PAIRED_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_HASH_PAIRED_MAP, GXmlGomHashPairedMapClass))
#define GXML_IS_GOM_HASH_PAIRED_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_HASH_PAIRED_MAP))
#define GXML_IS_GOM_HASH_PAIRED_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_HASH_PAIRED_MAP))
#define GXML_GOM_HASH_PAIRED_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_HASH_PAIRED_MAP, GXmlGomHashPairedMapClass))

typedef struct _GXmlGomHashPairedMap GXmlGomHashPairedMap;
typedef struct _GXmlGomHashPairedMapClass GXmlGomHashPairedMapClass;
typedef struct _GXmlGomHashPairedMapPrivate GXmlGomHashPairedMapPrivate;

#define GXML_TYPE_GOM_PROPERTY (gxml_gom_property_get_type ())
#define GXML_GOM_PROPERTY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_PROPERTY, GXmlGomProperty))
#define GXML_IS_GOM_PROPERTY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_PROPERTY))
#define GXML_GOM_PROPERTY_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_GOM_PROPERTY, GXmlGomPropertyIface))

typedef struct _GXmlGomProperty GXmlGomProperty;
typedef struct _GXmlGomPropertyIface GXmlGomPropertyIface;

#define GXML_TYPE_GOM_BASE_PROPERTY (gxml_gom_base_property_get_type ())
#define GXML_GOM_BASE_PROPERTY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_BASE_PROPERTY, GXmlGomBaseProperty))
#define GXML_GOM_BASE_PROPERTY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_BASE_PROPERTY, GXmlGomBasePropertyClass))
#define GXML_IS_GOM_BASE_PROPERTY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_BASE_PROPERTY))
#define GXML_IS_GOM_BASE_PROPERTY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_BASE_PROPERTY))
#define GXML_GOM_BASE_PROPERTY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_BASE_PROPERTY, GXmlGomBasePropertyClass))

typedef struct _GXmlGomBaseProperty GXmlGomBaseProperty;
typedef struct _GXmlGomBasePropertyClass GXmlGomBasePropertyClass;
typedef struct _GXmlGomBasePropertyPrivate GXmlGomBasePropertyPrivate;

#define GXML_TYPE_GOM_STRING (gxml_gom_string_get_type ())
#define GXML_GOM_STRING(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_STRING, GXmlGomString))
#define GXML_GOM_STRING_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_STRING, GXmlGomStringClass))
#define GXML_IS_GOM_STRING(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_STRING))
#define GXML_IS_GOM_STRING_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_STRING))
#define GXML_GOM_STRING_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_STRING, GXmlGomStringClass))

typedef struct _GXmlGomString GXmlGomString;
typedef struct _GXmlGomStringClass GXmlGomStringClass;
typedef struct _GXmlGomStringPrivate GXmlGomStringPrivate;

#define GXML_TYPE_GOM_ARRAY_STRING (gxml_gom_array_string_get_type ())
#define GXML_GOM_ARRAY_STRING(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_ARRAY_STRING, GXmlGomArrayString))
#define GXML_GOM_ARRAY_STRING_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_ARRAY_STRING, GXmlGomArrayStringClass))
#define GXML_IS_GOM_ARRAY_STRING(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_ARRAY_STRING))
#define GXML_IS_GOM_ARRAY_STRING_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_ARRAY_STRING))
#define GXML_GOM_ARRAY_STRING_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_ARRAY_STRING, GXmlGomArrayStringClass))

typedef struct _GXmlGomArrayString GXmlGomArrayString;
typedef struct _GXmlGomArrayStringClass GXmlGomArrayStringClass;
typedef struct _GXmlGomArrayStringPrivate GXmlGomArrayStringPrivate;

#define GXML_TYPE_GOM_XSD_ARRAY_STRING (gxml_gom_xsd_array_string_get_type ())
#define GXML_GOM_XSD_ARRAY_STRING(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_ARRAY_STRING, GXmlGomXsdArrayString))
#define GXML_GOM_XSD_ARRAY_STRING_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_ARRAY_STRING, GXmlGomXsdArrayStringClass))
#define GXML_IS_GOM_XSD_ARRAY_STRING(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_ARRAY_STRING))
#define GXML_IS_GOM_XSD_ARRAY_STRING_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_ARRAY_STRING))
#define GXML_GOM_XSD_ARRAY_STRING_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_ARRAY_STRING, GXmlGomXsdArrayStringClass))

typedef struct _GXmlGomXsdArrayString GXmlGomXsdArrayString;
typedef struct _GXmlGomXsdArrayStringClass GXmlGomXsdArrayStringClass;
typedef struct _GXmlGomXsdArrayStringPrivate GXmlGomXsdArrayStringPrivate;

#define GXML_TYPE_GOM_DOUBLE (gxml_gom_double_get_type ())
#define GXML_GOM_DOUBLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_DOUBLE, GXmlGomDouble))
#define GXML_GOM_DOUBLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_DOUBLE, GXmlGomDoubleClass))
#define GXML_IS_GOM_DOUBLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_DOUBLE))
#define GXML_IS_GOM_DOUBLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_DOUBLE))
#define GXML_GOM_DOUBLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_DOUBLE, GXmlGomDoubleClass))

typedef struct _GXmlGomDouble GXmlGomDouble;
typedef struct _GXmlGomDoubleClass GXmlGomDoubleClass;
typedef struct _GXmlGomDoublePrivate GXmlGomDoublePrivate;

#define GXML_TYPE_GOM_FLOAT (gxml_gom_float_get_type ())
#define GXML_GOM_FLOAT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_FLOAT, GXmlGomFloat))
#define GXML_GOM_FLOAT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_FLOAT, GXmlGomFloatClass))
#define GXML_IS_GOM_FLOAT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_FLOAT))
#define GXML_IS_GOM_FLOAT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_FLOAT))
#define GXML_GOM_FLOAT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_FLOAT, GXmlGomFloatClass))

typedef struct _GXmlGomFloat GXmlGomFloat;
typedef struct _GXmlGomFloatClass GXmlGomFloatClass;
typedef struct _GXmlGomFloatPrivate GXmlGomFloatPrivate;

#define GXML_TYPE_GOM_INT (gxml_gom_int_get_type ())
#define GXML_GOM_INT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_INT, GXmlGomInt))
#define GXML_GOM_INT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_INT, GXmlGomIntClass))
#define GXML_IS_GOM_INT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_INT))
#define GXML_IS_GOM_INT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_INT))
#define GXML_GOM_INT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_INT, GXmlGomIntClass))

typedef struct _GXmlGomInt GXmlGomInt;
typedef struct _GXmlGomIntClass GXmlGomIntClass;
typedef struct _GXmlGomIntPrivate GXmlGomIntPrivate;

#define GXML_TYPE_GOM_BOOLEAN (gxml_gom_boolean_get_type ())
#define GXML_GOM_BOOLEAN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_BOOLEAN, GXmlGomBoolean))
#define GXML_GOM_BOOLEAN_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_BOOLEAN, GXmlGomBooleanClass))
#define GXML_IS_GOM_BOOLEAN(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_BOOLEAN))
#define GXML_IS_GOM_BOOLEAN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_BOOLEAN))
#define GXML_GOM_BOOLEAN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_BOOLEAN, GXmlGomBooleanClass))

typedef struct _GXmlGomBoolean GXmlGomBoolean;
typedef struct _GXmlGomBooleanClass GXmlGomBooleanClass;
typedef struct _GXmlGomBooleanPrivate GXmlGomBooleanPrivate;

#define GXML_TYPE_GOM_ENUM (gxml_gom_enum_get_type ())
#define GXML_GOM_ENUM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_ENUM, GXmlGomEnum))
#define GXML_GOM_ENUM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_ENUM, GXmlGomEnumClass))
#define GXML_IS_GOM_ENUM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_ENUM))
#define GXML_IS_GOM_ENUM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_ENUM))
#define GXML_GOM_ENUM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_ENUM, GXmlGomEnumClass))

typedef struct _GXmlGomEnum GXmlGomEnum;
typedef struct _GXmlGomEnumClass GXmlGomEnumClass;
typedef struct _GXmlGomEnumPrivate GXmlGomEnumPrivate;

#define GXML_TYPE_GOM_DATE (gxml_gom_date_get_type ())
#define GXML_GOM_DATE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_DATE, GXmlGomDate))
#define GXML_GOM_DATE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_DATE, GXmlGomDateClass))
#define GXML_IS_GOM_DATE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_DATE))
#define GXML_IS_GOM_DATE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_DATE))
#define GXML_GOM_DATE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_DATE, GXmlGomDateClass))

typedef struct _GXmlGomDate GXmlGomDate;
typedef struct _GXmlGomDateClass GXmlGomDateClass;
typedef struct _GXmlGomDatePrivate GXmlGomDatePrivate;

#define GXML_TYPE_GOM_DATE_TIME (gxml_gom_date_time_get_type ())
#define GXML_GOM_DATE_TIME(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_DATE_TIME, GXmlGomDateTime))
#define GXML_GOM_DATE_TIME_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_DATE_TIME, GXmlGomDateTimeClass))
#define GXML_IS_GOM_DATE_TIME(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_DATE_TIME))
#define GXML_IS_GOM_DATE_TIME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_DATE_TIME))
#define GXML_GOM_DATE_TIME_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_DATE_TIME, GXmlGomDateTimeClass))

typedef struct _GXmlGomDateTime GXmlGomDateTime;
typedef struct _GXmlGomDateTimeClass GXmlGomDateTimeClass;
typedef struct _GXmlGomDateTimePrivate GXmlGomDateTimePrivate;

#define GXML_TYPE_PARSER (gxml_parser_get_type ())
#define GXML_PARSER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_PARSER, GXmlParser))
#define GXML_IS_PARSER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_PARSER))
#define GXML_PARSER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_PARSER, GXmlParserIface))

typedef struct _GXmlParser GXmlParser;
typedef struct _GXmlParserIface GXmlParserIface;

#define GXML_TYPE_XPARSER (gxml_xparser_get_type ())
#define GXML_XPARSER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_XPARSER, GXmlXParser))
#define GXML_XPARSER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_XPARSER, GXmlXParserClass))
#define GXML_IS_XPARSER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_XPARSER))
#define GXML_IS_XPARSER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_XPARSER))
#define GXML_XPARSER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_XPARSER, GXmlXParserClass))

typedef struct _GXmlXParser GXmlXParser;
typedef struct _GXmlXParserClass GXmlXParserClass;
typedef struct _GXmlXParserPrivate GXmlXParserPrivate;

#define GXML_TYPE_IXSD_SCHEMA (gxml_ixsd_schema_get_type ())
#define GXML_IXSD_SCHEMA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_SCHEMA, GXmlIXsdSchema))
#define GXML_IS_IXSD_SCHEMA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_SCHEMA))
#define GXML_IXSD_SCHEMA_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_SCHEMA, GXmlIXsdSchemaIface))

typedef struct _GXmlIXsdSchema GXmlIXsdSchema;
typedef struct _GXmlIXsdSchemaIface GXmlIXsdSchemaIface;

#define GXML_TYPE_IXSD_LIST (gxml_ixsd_list_get_type ())
#define GXML_IXSD_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_LIST, GXmlIXsdList))
#define GXML_IS_IXSD_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_LIST))
#define GXML_IXSD_LIST_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_LIST, GXmlIXsdListIface))

typedef struct _GXmlIXsdList GXmlIXsdList;
typedef struct _GXmlIXsdListIface GXmlIXsdListIface;

#define GXML_TYPE_IXSD_LIST_ELEMENTS (gxml_ixsd_list_elements_get_type ())
#define GXML_IXSD_LIST_ELEMENTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_LIST_ELEMENTS, GXmlIXsdListElements))
#define GXML_IS_IXSD_LIST_ELEMENTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_LIST_ELEMENTS))
#define GXML_IXSD_LIST_ELEMENTS_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_LIST_ELEMENTS, GXmlIXsdListElementsIface))

typedef struct _GXmlIXsdListElements GXmlIXsdListElements;
typedef struct _GXmlIXsdListElementsIface GXmlIXsdListElementsIface;

#define GXML_TYPE_IXSD_LIST_SIMPLE_TYPES (gxml_ixsd_list_simple_types_get_type ())
#define GXML_IXSD_LIST_SIMPLE_TYPES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_LIST_SIMPLE_TYPES, GXmlIXsdListSimpleTypes))
#define GXML_IS_IXSD_LIST_SIMPLE_TYPES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_LIST_SIMPLE_TYPES))
#define GXML_IXSD_LIST_SIMPLE_TYPES_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_LIST_SIMPLE_TYPES, GXmlIXsdListSimpleTypesIface))

typedef struct _GXmlIXsdListSimpleTypes GXmlIXsdListSimpleTypes;
typedef struct _GXmlIXsdListSimpleTypesIface GXmlIXsdListSimpleTypesIface;

#define GXML_TYPE_IXSD_LIST_COMPLEX_TYPES (gxml_ixsd_list_complex_types_get_type ())
#define GXML_IXSD_LIST_COMPLEX_TYPES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_LIST_COMPLEX_TYPES, GXmlIXsdListComplexTypes))
#define GXML_IS_IXSD_LIST_COMPLEX_TYPES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_LIST_COMPLEX_TYPES))
#define GXML_IXSD_LIST_COMPLEX_TYPES_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_LIST_COMPLEX_TYPES, GXmlIXsdListComplexTypesIface))

typedef struct _GXmlIXsdListComplexTypes GXmlIXsdListComplexTypes;
typedef struct _GXmlIXsdListComplexTypesIface GXmlIXsdListComplexTypesIface;

#define GXML_TYPE_IXSD_BASE_TYPE (gxml_ixsd_base_type_get_type ())
#define GXML_IXSD_BASE_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_BASE_TYPE, GXmlIXsdBaseType))
#define GXML_IS_IXSD_BASE_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_BASE_TYPE))
#define GXML_IXSD_BASE_TYPE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_BASE_TYPE, GXmlIXsdBaseTypeIface))

typedef struct _GXmlIXsdBaseType GXmlIXsdBaseType;
typedef struct _GXmlIXsdBaseTypeIface GXmlIXsdBaseTypeIface;

#define GXML_TYPE_IXSD_ANNOTATION (gxml_ixsd_annotation_get_type ())
#define GXML_IXSD_ANNOTATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_ANNOTATION, GXmlIXsdAnnotation))
#define GXML_IS_IXSD_ANNOTATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_ANNOTATION))
#define GXML_IXSD_ANNOTATION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_ANNOTATION, GXmlIXsdAnnotationIface))

typedef struct _GXmlIXsdAnnotation GXmlIXsdAnnotation;
typedef struct _GXmlIXsdAnnotationIface GXmlIXsdAnnotationIface;

#define GXML_TYPE_IXSD_SIMPLE_TYPE (gxml_ixsd_simple_type_get_type ())
#define GXML_IXSD_SIMPLE_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_SIMPLE_TYPE, GXmlIXsdSimpleType))
#define GXML_IS_IXSD_SIMPLE_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_SIMPLE_TYPE))
#define GXML_IXSD_SIMPLE_TYPE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_SIMPLE_TYPE, GXmlIXsdSimpleTypeIface))

typedef struct _GXmlIXsdSimpleType GXmlIXsdSimpleType;
typedef struct _GXmlIXsdSimpleTypeIface GXmlIXsdSimpleTypeIface;

#define GXML_TYPE_IXSD_TYPE_DEF (gxml_ixsd_type_def_get_type ())
#define GXML_IXSD_TYPE_DEF(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_TYPE_DEF, GXmlIXsdTypeDef))
#define GXML_IS_IXSD_TYPE_DEF(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_TYPE_DEF))
#define GXML_IXSD_TYPE_DEF_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_TYPE_DEF, GXmlIXsdTypeDefIface))

typedef struct _GXmlIXsdTypeDef GXmlIXsdTypeDef;
typedef struct _GXmlIXsdTypeDefIface GXmlIXsdTypeDefIface;

#define GXML_TYPE_IXSD_TYPE_LIST (gxml_ixsd_type_list_get_type ())
#define GXML_IXSD_TYPE_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_TYPE_LIST, GXmlIXsdTypeList))
#define GXML_IS_IXSD_TYPE_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_TYPE_LIST))
#define GXML_IXSD_TYPE_LIST_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_TYPE_LIST, GXmlIXsdTypeListIface))

typedef struct _GXmlIXsdTypeList GXmlIXsdTypeList;
typedef struct _GXmlIXsdTypeListIface GXmlIXsdTypeListIface;

#define GXML_TYPE_IXSD_TYPE_UNION (gxml_ixsd_type_union_get_type ())
#define GXML_IXSD_TYPE_UNION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_TYPE_UNION, GXmlIXsdTypeUnion))
#define GXML_IS_IXSD_TYPE_UNION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_TYPE_UNION))
#define GXML_IXSD_TYPE_UNION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_TYPE_UNION, GXmlIXsdTypeUnionIface))

typedef struct _GXmlIXsdTypeUnion GXmlIXsdTypeUnion;
typedef struct _GXmlIXsdTypeUnionIface GXmlIXsdTypeUnionIface;

#define GXML_TYPE_IXSD_TYPE_RESTRICTION (gxml_ixsd_type_restriction_get_type ())
#define GXML_IXSD_TYPE_RESTRICTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION, GXmlIXsdTypeRestriction))
#define GXML_IS_IXSD_TYPE_RESTRICTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION))
#define GXML_IXSD_TYPE_RESTRICTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION, GXmlIXsdTypeRestrictionIface))

typedef struct _GXmlIXsdTypeRestriction GXmlIXsdTypeRestriction;
typedef struct _GXmlIXsdTypeRestrictionIface GXmlIXsdTypeRestrictionIface;

#define GXML_TYPE_IXSD_LIST_TYPE_RESTRICTION_ENUMERATIONS (gxml_ixsd_list_type_restriction_enumerations_get_type ())
#define GXML_IXSD_LIST_TYPE_RESTRICTION_ENUMERATIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_LIST_TYPE_RESTRICTION_ENUMERATIONS, GXmlIXsdListTypeRestrictionEnumerations))
#define GXML_IS_IXSD_LIST_TYPE_RESTRICTION_ENUMERATIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_LIST_TYPE_RESTRICTION_ENUMERATIONS))
#define GXML_IXSD_LIST_TYPE_RESTRICTION_ENUMERATIONS_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_LIST_TYPE_RESTRICTION_ENUMERATIONS, GXmlIXsdListTypeRestrictionEnumerationsIface))

typedef struct _GXmlIXsdListTypeRestrictionEnumerations GXmlIXsdListTypeRestrictionEnumerations;
typedef struct _GXmlIXsdListTypeRestrictionEnumerationsIface GXmlIXsdListTypeRestrictionEnumerationsIface;

#define GXML_TYPE_IXSD_LIST_TYPE_RESTRICTION_WHITE_SPACES (gxml_ixsd_list_type_restriction_white_spaces_get_type ())
#define GXML_IXSD_LIST_TYPE_RESTRICTION_WHITE_SPACES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_LIST_TYPE_RESTRICTION_WHITE_SPACES, GXmlIXsdListTypeRestrictionWhiteSpaces))
#define GXML_IS_IXSD_LIST_TYPE_RESTRICTION_WHITE_SPACES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_LIST_TYPE_RESTRICTION_WHITE_SPACES))
#define GXML_IXSD_LIST_TYPE_RESTRICTION_WHITE_SPACES_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_LIST_TYPE_RESTRICTION_WHITE_SPACES, GXmlIXsdListTypeRestrictionWhiteSpacesIface))

typedef struct _GXmlIXsdListTypeRestrictionWhiteSpaces GXmlIXsdListTypeRestrictionWhiteSpaces;
typedef struct _GXmlIXsdListTypeRestrictionWhiteSpacesIface GXmlIXsdListTypeRestrictionWhiteSpacesIface;

#define GXML_TYPE_IXSD_TYPE_RESTRICTION_DEF (gxml_ixsd_type_restriction_def_get_type ())
#define GXML_IXSD_TYPE_RESTRICTION_DEF(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_DEF, GXmlIXsdTypeRestrictionDef))
#define GXML_IS_IXSD_TYPE_RESTRICTION_DEF(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_DEF))
#define GXML_IXSD_TYPE_RESTRICTION_DEF_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_DEF, GXmlIXsdTypeRestrictionDefIface))

typedef struct _GXmlIXsdTypeRestrictionDef GXmlIXsdTypeRestrictionDef;
typedef struct _GXmlIXsdTypeRestrictionDefIface GXmlIXsdTypeRestrictionDefIface;

#define GXML_TYPE_IXSD_TYPE_RESTRICTION_MIN_EXCLUSIVE (gxml_ixsd_type_restriction_min_exclusive_get_type ())
#define GXML_IXSD_TYPE_RESTRICTION_MIN_EXCLUSIVE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_MIN_EXCLUSIVE, GXmlIXsdTypeRestrictionMinExclusive))
#define GXML_IS_IXSD_TYPE_RESTRICTION_MIN_EXCLUSIVE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_MIN_EXCLUSIVE))
#define GXML_IXSD_TYPE_RESTRICTION_MIN_EXCLUSIVE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_MIN_EXCLUSIVE, GXmlIXsdTypeRestrictionMinExclusiveIface))

typedef struct _GXmlIXsdTypeRestrictionMinExclusive GXmlIXsdTypeRestrictionMinExclusive;
typedef struct _GXmlIXsdTypeRestrictionMinExclusiveIface GXmlIXsdTypeRestrictionMinExclusiveIface;

#define GXML_TYPE_IXSD_TYPE_RESTRICTION_MIN_INCLUSIVE (gxml_ixsd_type_restriction_min_inclusive_get_type ())
#define GXML_IXSD_TYPE_RESTRICTION_MIN_INCLUSIVE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_MIN_INCLUSIVE, GXmlIXsdTypeRestrictionMinInclusive))
#define GXML_IS_IXSD_TYPE_RESTRICTION_MIN_INCLUSIVE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_MIN_INCLUSIVE))
#define GXML_IXSD_TYPE_RESTRICTION_MIN_INCLUSIVE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_MIN_INCLUSIVE, GXmlIXsdTypeRestrictionMinInclusiveIface))

typedef struct _GXmlIXsdTypeRestrictionMinInclusive GXmlIXsdTypeRestrictionMinInclusive;
typedef struct _GXmlIXsdTypeRestrictionMinInclusiveIface GXmlIXsdTypeRestrictionMinInclusiveIface;

#define GXML_TYPE_IXSD_TYPE_RESTRICTION_MAX_EXCLUSIVE (gxml_ixsd_type_restriction_max_exclusive_get_type ())
#define GXML_IXSD_TYPE_RESTRICTION_MAX_EXCLUSIVE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_MAX_EXCLUSIVE, GXmlIXsdTypeRestrictionMaxExclusive))
#define GXML_IS_IXSD_TYPE_RESTRICTION_MAX_EXCLUSIVE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_MAX_EXCLUSIVE))
#define GXML_IXSD_TYPE_RESTRICTION_MAX_EXCLUSIVE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_MAX_EXCLUSIVE, GXmlIXsdTypeRestrictionMaxExclusiveIface))

typedef struct _GXmlIXsdTypeRestrictionMaxExclusive GXmlIXsdTypeRestrictionMaxExclusive;
typedef struct _GXmlIXsdTypeRestrictionMaxExclusiveIface GXmlIXsdTypeRestrictionMaxExclusiveIface;

#define GXML_TYPE_IXSD_TYPE_RESTRICTION_MAX_INCLUSIVE (gxml_ixsd_type_restriction_max_inclusive_get_type ())
#define GXML_IXSD_TYPE_RESTRICTION_MAX_INCLUSIVE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_MAX_INCLUSIVE, GXmlIXsdTypeRestrictionMaxInclusive))
#define GXML_IS_IXSD_TYPE_RESTRICTION_MAX_INCLUSIVE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_MAX_INCLUSIVE))
#define GXML_IXSD_TYPE_RESTRICTION_MAX_INCLUSIVE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_MAX_INCLUSIVE, GXmlIXsdTypeRestrictionMaxInclusiveIface))

typedef struct _GXmlIXsdTypeRestrictionMaxInclusive GXmlIXsdTypeRestrictionMaxInclusive;
typedef struct _GXmlIXsdTypeRestrictionMaxInclusiveIface GXmlIXsdTypeRestrictionMaxInclusiveIface;

#define GXML_TYPE_IXSD_TYPE_RESTRICTION_TOTAL_DIGITS (gxml_ixsd_type_restriction_total_digits_get_type ())
#define GXML_IXSD_TYPE_RESTRICTION_TOTAL_DIGITS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_TOTAL_DIGITS, GXmlIXsdTypeRestrictionTotalDigits))
#define GXML_IS_IXSD_TYPE_RESTRICTION_TOTAL_DIGITS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_TOTAL_DIGITS))
#define GXML_IXSD_TYPE_RESTRICTION_TOTAL_DIGITS_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_TOTAL_DIGITS, GXmlIXsdTypeRestrictionTotalDigitsIface))

typedef struct _GXmlIXsdTypeRestrictionTotalDigits GXmlIXsdTypeRestrictionTotalDigits;
typedef struct _GXmlIXsdTypeRestrictionTotalDigitsIface GXmlIXsdTypeRestrictionTotalDigitsIface;

#define GXML_TYPE_IXSD_TYPE_RESTRICTION_FRACTION_DIGITS (gxml_ixsd_type_restriction_fraction_digits_get_type ())
#define GXML_IXSD_TYPE_RESTRICTION_FRACTION_DIGITS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_FRACTION_DIGITS, GXmlIXsdTypeRestrictionFractionDigits))
#define GXML_IS_IXSD_TYPE_RESTRICTION_FRACTION_DIGITS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_FRACTION_DIGITS))
#define GXML_IXSD_TYPE_RESTRICTION_FRACTION_DIGITS_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_FRACTION_DIGITS, GXmlIXsdTypeRestrictionFractionDigitsIface))

typedef struct _GXmlIXsdTypeRestrictionFractionDigits GXmlIXsdTypeRestrictionFractionDigits;
typedef struct _GXmlIXsdTypeRestrictionFractionDigitsIface GXmlIXsdTypeRestrictionFractionDigitsIface;

#define GXML_TYPE_IXSD_TYPE_RESTRICTION_LENGTH (gxml_ixsd_type_restriction_length_get_type ())
#define GXML_IXSD_TYPE_RESTRICTION_LENGTH(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_LENGTH, GXmlIXsdTypeRestrictionLength))
#define GXML_IS_IXSD_TYPE_RESTRICTION_LENGTH(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_LENGTH))
#define GXML_IXSD_TYPE_RESTRICTION_LENGTH_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_LENGTH, GXmlIXsdTypeRestrictionLengthIface))

typedef struct _GXmlIXsdTypeRestrictionLength GXmlIXsdTypeRestrictionLength;
typedef struct _GXmlIXsdTypeRestrictionLengthIface GXmlIXsdTypeRestrictionLengthIface;

#define GXML_TYPE_IXSD_TYPE_RESTRICTION_MIN_LENGTH (gxml_ixsd_type_restriction_min_length_get_type ())
#define GXML_IXSD_TYPE_RESTRICTION_MIN_LENGTH(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_MIN_LENGTH, GXmlIXsdTypeRestrictionMinLength))
#define GXML_IS_IXSD_TYPE_RESTRICTION_MIN_LENGTH(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_MIN_LENGTH))
#define GXML_IXSD_TYPE_RESTRICTION_MIN_LENGTH_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_MIN_LENGTH, GXmlIXsdTypeRestrictionMinLengthIface))

typedef struct _GXmlIXsdTypeRestrictionMinLength GXmlIXsdTypeRestrictionMinLength;
typedef struct _GXmlIXsdTypeRestrictionMinLengthIface GXmlIXsdTypeRestrictionMinLengthIface;

#define GXML_TYPE_IXSD_TYPE_RESTRICTION_MAX_LENGTH (gxml_ixsd_type_restriction_max_length_get_type ())
#define GXML_IXSD_TYPE_RESTRICTION_MAX_LENGTH(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_MAX_LENGTH, GXmlIXsdTypeRestrictionMaxLength))
#define GXML_IS_IXSD_TYPE_RESTRICTION_MAX_LENGTH(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_MAX_LENGTH))
#define GXML_IXSD_TYPE_RESTRICTION_MAX_LENGTH_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_MAX_LENGTH, GXmlIXsdTypeRestrictionMaxLengthIface))

typedef struct _GXmlIXsdTypeRestrictionMaxLength GXmlIXsdTypeRestrictionMaxLength;
typedef struct _GXmlIXsdTypeRestrictionMaxLengthIface GXmlIXsdTypeRestrictionMaxLengthIface;

#define GXML_TYPE_IXSD_TYPE_RESTRICTION_ENUMERATION (gxml_ixsd_type_restriction_enumeration_get_type ())
#define GXML_IXSD_TYPE_RESTRICTION_ENUMERATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_ENUMERATION, GXmlIXsdTypeRestrictionEnumeration))
#define GXML_IS_IXSD_TYPE_RESTRICTION_ENUMERATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_ENUMERATION))
#define GXML_IXSD_TYPE_RESTRICTION_ENUMERATION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_ENUMERATION, GXmlIXsdTypeRestrictionEnumerationIface))

typedef struct _GXmlIXsdTypeRestrictionEnumeration GXmlIXsdTypeRestrictionEnumeration;
typedef struct _GXmlIXsdTypeRestrictionEnumerationIface GXmlIXsdTypeRestrictionEnumerationIface;

#define GXML_TYPE_IXSD_TYPE_RESTRICTION_WHITE_SPACE (gxml_ixsd_type_restriction_white_space_get_type ())
#define GXML_IXSD_TYPE_RESTRICTION_WHITE_SPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_WHITE_SPACE, GXmlIXsdTypeRestrictionWhiteSpace))
#define GXML_IS_IXSD_TYPE_RESTRICTION_WHITE_SPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_WHITE_SPACE))
#define GXML_IXSD_TYPE_RESTRICTION_WHITE_SPACE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_WHITE_SPACE, GXmlIXsdTypeRestrictionWhiteSpaceIface))

typedef struct _GXmlIXsdTypeRestrictionWhiteSpace GXmlIXsdTypeRestrictionWhiteSpace;
typedef struct _GXmlIXsdTypeRestrictionWhiteSpaceIface GXmlIXsdTypeRestrictionWhiteSpaceIface;

#define GXML_TYPE_IXSD_TYPE_RESTRICTION_PATTERN (gxml_ixsd_type_restriction_pattern_get_type ())
#define GXML_IXSD_TYPE_RESTRICTION_PATTERN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_PATTERN, GXmlIXsdTypeRestrictionPattern))
#define GXML_IS_IXSD_TYPE_RESTRICTION_PATTERN(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_PATTERN))
#define GXML_IXSD_TYPE_RESTRICTION_PATTERN_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_PATTERN, GXmlIXsdTypeRestrictionPatternIface))

typedef struct _GXmlIXsdTypeRestrictionPattern GXmlIXsdTypeRestrictionPattern;
typedef struct _GXmlIXsdTypeRestrictionPatternIface GXmlIXsdTypeRestrictionPatternIface;

#define GXML_TYPE_IXSD_TYPE_RESTRICTION_ASSERTION (gxml_ixsd_type_restriction_assertion_get_type ())
#define GXML_IXSD_TYPE_RESTRICTION_ASSERTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_ASSERTION, GXmlIXsdTypeRestrictionAssertion))
#define GXML_IS_IXSD_TYPE_RESTRICTION_ASSERTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_ASSERTION))
#define GXML_IXSD_TYPE_RESTRICTION_ASSERTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_ASSERTION, GXmlIXsdTypeRestrictionAssertionIface))

typedef struct _GXmlIXsdTypeRestrictionAssertion GXmlIXsdTypeRestrictionAssertion;
typedef struct _GXmlIXsdTypeRestrictionAssertionIface GXmlIXsdTypeRestrictionAssertionIface;

#define GXML_TYPE_IXSD_TYPE_RESTRICTION_EXPLICIT_TIMEZONE (gxml_ixsd_type_restriction_explicit_timezone_get_type ())
#define GXML_IXSD_TYPE_RESTRICTION_EXPLICIT_TIMEZONE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_EXPLICIT_TIMEZONE, GXmlIXsdTypeRestrictionExplicitTimezone))
#define GXML_IS_IXSD_TYPE_RESTRICTION_EXPLICIT_TIMEZONE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_EXPLICIT_TIMEZONE))
#define GXML_IXSD_TYPE_RESTRICTION_EXPLICIT_TIMEZONE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_TYPE_RESTRICTION_EXPLICIT_TIMEZONE, GXmlIXsdTypeRestrictionExplicitTimezoneIface))

typedef struct _GXmlIXsdTypeRestrictionExplicitTimezone GXmlIXsdTypeRestrictionExplicitTimezone;
typedef struct _GXmlIXsdTypeRestrictionExplicitTimezoneIface GXmlIXsdTypeRestrictionExplicitTimezoneIface;

#define GXML_TYPE_IXSD_COMPLEX_TYPE (gxml_ixsd_complex_type_get_type ())
#define GXML_IXSD_COMPLEX_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_COMPLEX_TYPE, GXmlIXsdComplexType))
#define GXML_IS_IXSD_COMPLEX_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_COMPLEX_TYPE))
#define GXML_IXSD_COMPLEX_TYPE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_COMPLEX_TYPE, GXmlIXsdComplexTypeIface))

typedef struct _GXmlIXsdComplexType GXmlIXsdComplexType;
typedef struct _GXmlIXsdComplexTypeIface GXmlIXsdComplexTypeIface;

#define GXML_TYPE_IXSD_BASE_CONTENT (gxml_ixsd_base_content_get_type ())
#define GXML_IXSD_BASE_CONTENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_BASE_CONTENT, GXmlIXsdBaseContent))
#define GXML_IS_IXSD_BASE_CONTENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_BASE_CONTENT))
#define GXML_IXSD_BASE_CONTENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_BASE_CONTENT, GXmlIXsdBaseContentIface))

typedef struct _GXmlIXsdBaseContent GXmlIXsdBaseContent;
typedef struct _GXmlIXsdBaseContentIface GXmlIXsdBaseContentIface;

#define GXML_TYPE_IXSD_LIST_ATTRIBUTES (gxml_ixsd_list_attributes_get_type ())
#define GXML_IXSD_LIST_ATTRIBUTES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_LIST_ATTRIBUTES, GXmlIXsdListAttributes))
#define GXML_IS_IXSD_LIST_ATTRIBUTES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_LIST_ATTRIBUTES))
#define GXML_IXSD_LIST_ATTRIBUTES_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_LIST_ATTRIBUTES, GXmlIXsdListAttributesIface))

typedef struct _GXmlIXsdListAttributes GXmlIXsdListAttributes;
typedef struct _GXmlIXsdListAttributesIface GXmlIXsdListAttributesIface;

#define GXML_TYPE_IXSD_LIST_ATTRIBUTES_GROUP (gxml_ixsd_list_attributes_group_get_type ())
#define GXML_IXSD_LIST_ATTRIBUTES_GROUP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_LIST_ATTRIBUTES_GROUP, GXmlIXsdListAttributesGroup))
#define GXML_IS_IXSD_LIST_ATTRIBUTES_GROUP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_LIST_ATTRIBUTES_GROUP))
#define GXML_IXSD_LIST_ATTRIBUTES_GROUP_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_LIST_ATTRIBUTES_GROUP, GXmlIXsdListAttributesGroupIface))

typedef struct _GXmlIXsdListAttributesGroup GXmlIXsdListAttributesGroup;
typedef struct _GXmlIXsdListAttributesGroupIface GXmlIXsdListAttributesGroupIface;

#define GXML_TYPE_IXSD_EXTENSION (gxml_ixsd_extension_get_type ())
#define GXML_IXSD_EXTENSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_EXTENSION, GXmlIXsdExtension))
#define GXML_IS_IXSD_EXTENSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_EXTENSION))
#define GXML_IXSD_EXTENSION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_EXTENSION, GXmlIXsdExtensionIface))

typedef struct _GXmlIXsdExtension GXmlIXsdExtension;
typedef struct _GXmlIXsdExtensionIface GXmlIXsdExtensionIface;

#define GXML_TYPE_IXSD_ELEMENT (gxml_ixsd_element_get_type ())
#define GXML_IXSD_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_ELEMENT, GXmlIXsdElement))
#define GXML_IS_IXSD_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_ELEMENT))
#define GXML_IXSD_ELEMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_ELEMENT, GXmlIXsdElementIface))

typedef struct _GXmlIXsdElement GXmlIXsdElement;
typedef struct _GXmlIXsdElementIface GXmlIXsdElementIface;

#define GXML_TYPE_IXSD_SIMPLE_CONTENT (gxml_ixsd_simple_content_get_type ())
#define GXML_IXSD_SIMPLE_CONTENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_SIMPLE_CONTENT, GXmlIXsdSimpleContent))
#define GXML_IS_IXSD_SIMPLE_CONTENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_SIMPLE_CONTENT))
#define GXML_IXSD_SIMPLE_CONTENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_SIMPLE_CONTENT, GXmlIXsdSimpleContentIface))

typedef struct _GXmlIXsdSimpleContent GXmlIXsdSimpleContent;
typedef struct _GXmlIXsdSimpleContentIface GXmlIXsdSimpleContentIface;

#define GXML_TYPE_IXSD_COMPLEX_CONTENT (gxml_ixsd_complex_content_get_type ())
#define GXML_IXSD_COMPLEX_CONTENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_COMPLEX_CONTENT, GXmlIXsdComplexContent))
#define GXML_IS_IXSD_COMPLEX_CONTENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_COMPLEX_CONTENT))
#define GXML_IXSD_COMPLEX_CONTENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_COMPLEX_CONTENT, GXmlIXsdComplexContentIface))

typedef struct _GXmlIXsdComplexContent GXmlIXsdComplexContent;
typedef struct _GXmlIXsdComplexContentIface GXmlIXsdComplexContentIface;

#define GXML_TYPE_IXSD_OPEN_CONTENT (gxml_ixsd_open_content_get_type ())
#define GXML_IXSD_OPEN_CONTENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_OPEN_CONTENT, GXmlIXsdOpenContent))
#define GXML_IS_IXSD_OPEN_CONTENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_OPEN_CONTENT))
#define GXML_IXSD_OPEN_CONTENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_OPEN_CONTENT, GXmlIXsdOpenContentIface))

typedef struct _GXmlIXsdOpenContent GXmlIXsdOpenContent;
typedef struct _GXmlIXsdOpenContentIface GXmlIXsdOpenContentIface;

#define GXML_TYPE_IXSD_BASE_ATTRIBUTE (gxml_ixsd_base_attribute_get_type ())
#define GXML_IXSD_BASE_ATTRIBUTE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_BASE_ATTRIBUTE, GXmlIXsdBaseAttribute))
#define GXML_IS_IXSD_BASE_ATTRIBUTE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_BASE_ATTRIBUTE))
#define GXML_IXSD_BASE_ATTRIBUTE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_BASE_ATTRIBUTE, GXmlIXsdBaseAttributeIface))

typedef struct _GXmlIXsdBaseAttribute GXmlIXsdBaseAttribute;
typedef struct _GXmlIXsdBaseAttributeIface GXmlIXsdBaseAttributeIface;

#define GXML_TYPE_IXSD_ATTRIBUTE (gxml_ixsd_attribute_get_type ())
#define GXML_IXSD_ATTRIBUTE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_ATTRIBUTE, GXmlIXsdAttribute))
#define GXML_IS_IXSD_ATTRIBUTE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_ATTRIBUTE))
#define GXML_IXSD_ATTRIBUTE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_ATTRIBUTE, GXmlIXsdAttributeIface))

typedef struct _GXmlIXsdAttribute GXmlIXsdAttribute;
typedef struct _GXmlIXsdAttributeIface GXmlIXsdAttributeIface;

#define GXML_TYPE_IXSD_ATTRIBUTE_GROUP (gxml_ixsd_attribute_group_get_type ())
#define GXML_IXSD_ATTRIBUTE_GROUP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_IXSD_ATTRIBUTE_GROUP, GXmlIXsdAttributeGroup))
#define GXML_IS_IXSD_ATTRIBUTE_GROUP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_IXSD_ATTRIBUTE_GROUP))
#define GXML_IXSD_ATTRIBUTE_GROUP_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_IXSD_ATTRIBUTE_GROUP, GXmlIXsdAttributeGroupIface))

typedef struct _GXmlIXsdAttributeGroup GXmlIXsdAttributeGroup;
typedef struct _GXmlIXsdAttributeGroupIface GXmlIXsdAttributeGroupIface;

#define GXML_TYPE_GOM_XSD_SCHEMA (gxml_gom_xsd_schema_get_type ())
#define GXML_GOM_XSD_SCHEMA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_SCHEMA, GXmlGomXsdSchema))
#define GXML_GOM_XSD_SCHEMA_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_SCHEMA, GXmlGomXsdSchemaClass))
#define GXML_IS_GOM_XSD_SCHEMA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_SCHEMA))
#define GXML_IS_GOM_XSD_SCHEMA_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_SCHEMA))
#define GXML_GOM_XSD_SCHEMA_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_SCHEMA, GXmlGomXsdSchemaClass))

typedef struct _GXmlGomXsdSchema GXmlGomXsdSchema;
typedef struct _GXmlGomXsdSchemaClass GXmlGomXsdSchemaClass;
typedef struct _GXmlGomXsdSchemaPrivate GXmlGomXsdSchemaPrivate;

#define GXML_TYPE_GOM_XSD_LIST (gxml_gom_xsd_list_get_type ())
#define GXML_GOM_XSD_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_LIST, GXmlGomXsdList))
#define GXML_GOM_XSD_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_LIST, GXmlGomXsdListClass))
#define GXML_IS_GOM_XSD_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_LIST))
#define GXML_IS_GOM_XSD_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_LIST))
#define GXML_GOM_XSD_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_LIST, GXmlGomXsdListClass))

typedef struct _GXmlGomXsdList GXmlGomXsdList;
typedef struct _GXmlGomXsdListClass GXmlGomXsdListClass;

#define GXML_TYPE_GOM_XSD_LIST_ELEMENTS (gxml_gom_xsd_list_elements_get_type ())
#define GXML_GOM_XSD_LIST_ELEMENTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_LIST_ELEMENTS, GXmlGomXsdListElements))
#define GXML_GOM_XSD_LIST_ELEMENTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_LIST_ELEMENTS, GXmlGomXsdListElementsClass))
#define GXML_IS_GOM_XSD_LIST_ELEMENTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_LIST_ELEMENTS))
#define GXML_IS_GOM_XSD_LIST_ELEMENTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_LIST_ELEMENTS))
#define GXML_GOM_XSD_LIST_ELEMENTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_LIST_ELEMENTS, GXmlGomXsdListElementsClass))

typedef struct _GXmlGomXsdListElements GXmlGomXsdListElements;
typedef struct _GXmlGomXsdListElementsClass GXmlGomXsdListElementsClass;

#define GXML_TYPE_GOM_XSD_LIST_SIMPLE_TYPES (gxml_gom_xsd_list_simple_types_get_type ())
#define GXML_GOM_XSD_LIST_SIMPLE_TYPES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_LIST_SIMPLE_TYPES, GXmlGomXsdListSimpleTypes))
#define GXML_GOM_XSD_LIST_SIMPLE_TYPES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_LIST_SIMPLE_TYPES, GXmlGomXsdListSimpleTypesClass))
#define GXML_IS_GOM_XSD_LIST_SIMPLE_TYPES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_LIST_SIMPLE_TYPES))
#define GXML_IS_GOM_XSD_LIST_SIMPLE_TYPES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_LIST_SIMPLE_TYPES))
#define GXML_GOM_XSD_LIST_SIMPLE_TYPES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_LIST_SIMPLE_TYPES, GXmlGomXsdListSimpleTypesClass))

typedef struct _GXmlGomXsdListSimpleTypes GXmlGomXsdListSimpleTypes;
typedef struct _GXmlGomXsdListSimpleTypesClass GXmlGomXsdListSimpleTypesClass;

#define GXML_TYPE_GOM_XSD_LIST_COMPLEX_TYPES (gxml_gom_xsd_list_complex_types_get_type ())
#define GXML_GOM_XSD_LIST_COMPLEX_TYPES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_LIST_COMPLEX_TYPES, GXmlGomXsdListComplexTypes))
#define GXML_GOM_XSD_LIST_COMPLEX_TYPES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_LIST_COMPLEX_TYPES, GXmlGomXsdListComplexTypesClass))
#define GXML_IS_GOM_XSD_LIST_COMPLEX_TYPES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_LIST_COMPLEX_TYPES))
#define GXML_IS_GOM_XSD_LIST_COMPLEX_TYPES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_LIST_COMPLEX_TYPES))
#define GXML_GOM_XSD_LIST_COMPLEX_TYPES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_LIST_COMPLEX_TYPES, GXmlGomXsdListComplexTypesClass))

typedef struct _GXmlGomXsdListComplexTypes GXmlGomXsdListComplexTypes;
typedef struct _GXmlGomXsdListComplexTypesClass GXmlGomXsdListComplexTypesClass;

#define GXML_TYPE_GOM_XSD_SIMPLE_TYPE (gxml_gom_xsd_simple_type_get_type ())
#define GXML_GOM_XSD_SIMPLE_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_SIMPLE_TYPE, GXmlGomXsdSimpleType))
#define GXML_GOM_XSD_SIMPLE_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_SIMPLE_TYPE, GXmlGomXsdSimpleTypeClass))
#define GXML_IS_GOM_XSD_SIMPLE_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_SIMPLE_TYPE))
#define GXML_IS_GOM_XSD_SIMPLE_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_SIMPLE_TYPE))
#define GXML_GOM_XSD_SIMPLE_TYPE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_SIMPLE_TYPE, GXmlGomXsdSimpleTypeClass))

typedef struct _GXmlGomXsdSimpleType GXmlGomXsdSimpleType;
typedef struct _GXmlGomXsdSimpleTypeClass GXmlGomXsdSimpleTypeClass;
typedef struct _GXmlGomXsdSimpleTypePrivate GXmlGomXsdSimpleTypePrivate;

#define GXML_TYPE_GOM_XSD_ANNOTATION (gxml_gom_xsd_annotation_get_type ())
#define GXML_GOM_XSD_ANNOTATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_ANNOTATION, GXmlGomXsdAnnotation))
#define GXML_GOM_XSD_ANNOTATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_ANNOTATION, GXmlGomXsdAnnotationClass))
#define GXML_IS_GOM_XSD_ANNOTATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_ANNOTATION))
#define GXML_IS_GOM_XSD_ANNOTATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_ANNOTATION))
#define GXML_GOM_XSD_ANNOTATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_ANNOTATION, GXmlGomXsdAnnotationClass))

typedef struct _GXmlGomXsdAnnotation GXmlGomXsdAnnotation;
typedef struct _GXmlGomXsdAnnotationClass GXmlGomXsdAnnotationClass;

#define GXML_TYPE_GOM_XSD_TYPE_DEFINITION (gxml_gom_xsd_type_definition_get_type ())
#define GXML_GOM_XSD_TYPE_DEFINITION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_TYPE_DEFINITION, GXmlGomXsdTypeDefinition))
#define GXML_GOM_XSD_TYPE_DEFINITION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_TYPE_DEFINITION, GXmlGomXsdTypeDefinitionClass))
#define GXML_IS_GOM_XSD_TYPE_DEFINITION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_TYPE_DEFINITION))
#define GXML_IS_GOM_XSD_TYPE_DEFINITION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_TYPE_DEFINITION))
#define GXML_GOM_XSD_TYPE_DEFINITION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_TYPE_DEFINITION, GXmlGomXsdTypeDefinitionClass))

typedef struct _GXmlGomXsdTypeDefinition GXmlGomXsdTypeDefinition;
typedef struct _GXmlGomXsdTypeDefinitionClass GXmlGomXsdTypeDefinitionClass;

#define GXML_TYPE_GOM_XSD_TYPE_LIST (gxml_gom_xsd_type_list_get_type ())
#define GXML_GOM_XSD_TYPE_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_TYPE_LIST, GXmlGomXsdTypeList))
#define GXML_GOM_XSD_TYPE_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_TYPE_LIST, GXmlGomXsdTypeListClass))
#define GXML_IS_GOM_XSD_TYPE_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_TYPE_LIST))
#define GXML_IS_GOM_XSD_TYPE_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_TYPE_LIST))
#define GXML_GOM_XSD_TYPE_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_TYPE_LIST, GXmlGomXsdTypeListClass))

typedef struct _GXmlGomXsdTypeList GXmlGomXsdTypeList;
typedef struct _GXmlGomXsdTypeListClass GXmlGomXsdTypeListClass;

#define GXML_TYPE_GOM_XSD_TYPE_UNION (gxml_gom_xsd_type_union_get_type ())
#define GXML_GOM_XSD_TYPE_UNION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_TYPE_UNION, GXmlGomXsdTypeUnion))
#define GXML_GOM_XSD_TYPE_UNION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_TYPE_UNION, GXmlGomXsdTypeUnionClass))
#define GXML_IS_GOM_XSD_TYPE_UNION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_TYPE_UNION))
#define GXML_IS_GOM_XSD_TYPE_UNION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_TYPE_UNION))
#define GXML_GOM_XSD_TYPE_UNION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_TYPE_UNION, GXmlGomXsdTypeUnionClass))

typedef struct _GXmlGomXsdTypeUnion GXmlGomXsdTypeUnion;
typedef struct _GXmlGomXsdTypeUnionClass GXmlGomXsdTypeUnionClass;

#define GXML_TYPE_GOM_XSD_TYPE_RESTRICTION (gxml_gom_xsd_type_restriction_get_type ())
#define GXML_GOM_XSD_TYPE_RESTRICTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION, GXmlGomXsdTypeRestriction))
#define GXML_GOM_XSD_TYPE_RESTRICTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION, GXmlGomXsdTypeRestrictionClass))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION))
#define GXML_GOM_XSD_TYPE_RESTRICTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION, GXmlGomXsdTypeRestrictionClass))

typedef struct _GXmlGomXsdTypeRestriction GXmlGomXsdTypeRestriction;
typedef struct _GXmlGomXsdTypeRestrictionClass GXmlGomXsdTypeRestrictionClass;
typedef struct _GXmlGomXsdTypeDefinitionPrivate GXmlGomXsdTypeDefinitionPrivate;
typedef struct _GXmlGomXsdTypeListPrivate GXmlGomXsdTypeListPrivate;
typedef struct _GXmlGomXsdTypeUnionPrivate GXmlGomXsdTypeUnionPrivate;
typedef struct _GXmlGomXsdTypeRestrictionPrivate GXmlGomXsdTypeRestrictionPrivate;

#define GXML_TYPE_GOM_XSD_LIST_TYPE_RESTRICTION_ENUMERATIONS (gxml_gom_xsd_list_type_restriction_enumerations_get_type ())
#define GXML_GOM_XSD_LIST_TYPE_RESTRICTION_ENUMERATIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_LIST_TYPE_RESTRICTION_ENUMERATIONS, GXmlGomXsdListTypeRestrictionEnumerations))
#define GXML_GOM_XSD_LIST_TYPE_RESTRICTION_ENUMERATIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_LIST_TYPE_RESTRICTION_ENUMERATIONS, GXmlGomXsdListTypeRestrictionEnumerationsClass))
#define GXML_IS_GOM_XSD_LIST_TYPE_RESTRICTION_ENUMERATIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_LIST_TYPE_RESTRICTION_ENUMERATIONS))
#define GXML_IS_GOM_XSD_LIST_TYPE_RESTRICTION_ENUMERATIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_LIST_TYPE_RESTRICTION_ENUMERATIONS))
#define GXML_GOM_XSD_LIST_TYPE_RESTRICTION_ENUMERATIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_LIST_TYPE_RESTRICTION_ENUMERATIONS, GXmlGomXsdListTypeRestrictionEnumerationsClass))

typedef struct _GXmlGomXsdListTypeRestrictionEnumerations GXmlGomXsdListTypeRestrictionEnumerations;
typedef struct _GXmlGomXsdListTypeRestrictionEnumerationsClass GXmlGomXsdListTypeRestrictionEnumerationsClass;

#define GXML_TYPE_GOM_XSD_LIST_TYPE_RESTRICTION_WHITE_SPACES (gxml_gom_xsd_list_type_restriction_white_spaces_get_type ())
#define GXML_GOM_XSD_LIST_TYPE_RESTRICTION_WHITE_SPACES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_LIST_TYPE_RESTRICTION_WHITE_SPACES, GXmlGomXsdListTypeRestrictionWhiteSpaces))
#define GXML_GOM_XSD_LIST_TYPE_RESTRICTION_WHITE_SPACES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_LIST_TYPE_RESTRICTION_WHITE_SPACES, GXmlGomXsdListTypeRestrictionWhiteSpacesClass))
#define GXML_IS_GOM_XSD_LIST_TYPE_RESTRICTION_WHITE_SPACES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_LIST_TYPE_RESTRICTION_WHITE_SPACES))
#define GXML_IS_GOM_XSD_LIST_TYPE_RESTRICTION_WHITE_SPACES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_LIST_TYPE_RESTRICTION_WHITE_SPACES))
#define GXML_GOM_XSD_LIST_TYPE_RESTRICTION_WHITE_SPACES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_LIST_TYPE_RESTRICTION_WHITE_SPACES, GXmlGomXsdListTypeRestrictionWhiteSpacesClass))

typedef struct _GXmlGomXsdListTypeRestrictionWhiteSpaces GXmlGomXsdListTypeRestrictionWhiteSpaces;
typedef struct _GXmlGomXsdListTypeRestrictionWhiteSpacesClass GXmlGomXsdListTypeRestrictionWhiteSpacesClass;

#define GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_DEF (gxml_gom_xsd_type_restriction_def_get_type ())
#define GXML_GOM_XSD_TYPE_RESTRICTION_DEF(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_DEF, GXmlGomXsdTypeRestrictionDef))
#define GXML_GOM_XSD_TYPE_RESTRICTION_DEF_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_DEF, GXmlGomXsdTypeRestrictionDefClass))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_DEF(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_DEF))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_DEF_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_DEF))
#define GXML_GOM_XSD_TYPE_RESTRICTION_DEF_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_DEF, GXmlGomXsdTypeRestrictionDefClass))

typedef struct _GXmlGomXsdTypeRestrictionDef GXmlGomXsdTypeRestrictionDef;
typedef struct _GXmlGomXsdTypeRestrictionDefClass GXmlGomXsdTypeRestrictionDefClass;
typedef struct _GXmlGomXsdTypeRestrictionDefPrivate GXmlGomXsdTypeRestrictionDefPrivate;

#define GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MIN_EXCLUSIVE (gxml_gom_xsd_type_restriction_min_exclusive_get_type ())
#define GXML_GOM_XSD_TYPE_RESTRICTION_MIN_EXCLUSIVE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MIN_EXCLUSIVE, GXmlGomXsdTypeRestrictionMinExclusive))
#define GXML_GOM_XSD_TYPE_RESTRICTION_MIN_EXCLUSIVE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MIN_EXCLUSIVE, GXmlGomXsdTypeRestrictionMinExclusiveClass))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_MIN_EXCLUSIVE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MIN_EXCLUSIVE))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_MIN_EXCLUSIVE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MIN_EXCLUSIVE))
#define GXML_GOM_XSD_TYPE_RESTRICTION_MIN_EXCLUSIVE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MIN_EXCLUSIVE, GXmlGomXsdTypeRestrictionMinExclusiveClass))

typedef struct _GXmlGomXsdTypeRestrictionMinExclusive GXmlGomXsdTypeRestrictionMinExclusive;
typedef struct _GXmlGomXsdTypeRestrictionMinExclusiveClass GXmlGomXsdTypeRestrictionMinExclusiveClass;
typedef struct _GXmlGomXsdTypeRestrictionMinExclusivePrivate GXmlGomXsdTypeRestrictionMinExclusivePrivate;

#define GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MIN_INCLUSIVE (gxml_gom_xsd_type_restriction_min_inclusive_get_type ())
#define GXML_GOM_XSD_TYPE_RESTRICTION_MIN_INCLUSIVE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MIN_INCLUSIVE, GXmlGomXsdTypeRestrictionMinInclusive))
#define GXML_GOM_XSD_TYPE_RESTRICTION_MIN_INCLUSIVE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MIN_INCLUSIVE, GXmlGomXsdTypeRestrictionMinInclusiveClass))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_MIN_INCLUSIVE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MIN_INCLUSIVE))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_MIN_INCLUSIVE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MIN_INCLUSIVE))
#define GXML_GOM_XSD_TYPE_RESTRICTION_MIN_INCLUSIVE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MIN_INCLUSIVE, GXmlGomXsdTypeRestrictionMinInclusiveClass))

typedef struct _GXmlGomXsdTypeRestrictionMinInclusive GXmlGomXsdTypeRestrictionMinInclusive;
typedef struct _GXmlGomXsdTypeRestrictionMinInclusiveClass GXmlGomXsdTypeRestrictionMinInclusiveClass;
typedef struct _GXmlGomXsdTypeRestrictionMinInclusivePrivate GXmlGomXsdTypeRestrictionMinInclusivePrivate;

#define GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MAX_EXCLUSIVE (gxml_gom_xsd_type_restriction_max_exclusive_get_type ())
#define GXML_GOM_XSD_TYPE_RESTRICTION_MAX_EXCLUSIVE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MAX_EXCLUSIVE, GXmlGomXsdTypeRestrictionMaxExclusive))
#define GXML_GOM_XSD_TYPE_RESTRICTION_MAX_EXCLUSIVE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MAX_EXCLUSIVE, GXmlGomXsdTypeRestrictionMaxExclusiveClass))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_MAX_EXCLUSIVE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MAX_EXCLUSIVE))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_MAX_EXCLUSIVE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MAX_EXCLUSIVE))
#define GXML_GOM_XSD_TYPE_RESTRICTION_MAX_EXCLUSIVE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MAX_EXCLUSIVE, GXmlGomXsdTypeRestrictionMaxExclusiveClass))

typedef struct _GXmlGomXsdTypeRestrictionMaxExclusive GXmlGomXsdTypeRestrictionMaxExclusive;
typedef struct _GXmlGomXsdTypeRestrictionMaxExclusiveClass GXmlGomXsdTypeRestrictionMaxExclusiveClass;
typedef struct _GXmlGomXsdTypeRestrictionMaxExclusivePrivate GXmlGomXsdTypeRestrictionMaxExclusivePrivate;

#define GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MAX_INCLUSIVE (gxml_gom_xsd_type_restriction_max_inclusive_get_type ())
#define GXML_GOM_XSD_TYPE_RESTRICTION_MAX_INCLUSIVE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MAX_INCLUSIVE, GXmlGomXsdTypeRestrictionMaxInclusive))
#define GXML_GOM_XSD_TYPE_RESTRICTION_MAX_INCLUSIVE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MAX_INCLUSIVE, GXmlGomXsdTypeRestrictionMaxInclusiveClass))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_MAX_INCLUSIVE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MAX_INCLUSIVE))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_MAX_INCLUSIVE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MAX_INCLUSIVE))
#define GXML_GOM_XSD_TYPE_RESTRICTION_MAX_INCLUSIVE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MAX_INCLUSIVE, GXmlGomXsdTypeRestrictionMaxInclusiveClass))

typedef struct _GXmlGomXsdTypeRestrictionMaxInclusive GXmlGomXsdTypeRestrictionMaxInclusive;
typedef struct _GXmlGomXsdTypeRestrictionMaxInclusiveClass GXmlGomXsdTypeRestrictionMaxInclusiveClass;
typedef struct _GXmlGomXsdTypeRestrictionMaxInclusivePrivate GXmlGomXsdTypeRestrictionMaxInclusivePrivate;

#define GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_TOTAL_DIGITS (gxml_gom_xsd_type_restriction_total_digits_get_type ())
#define GXML_GOM_XSD_TYPE_RESTRICTION_TOTAL_DIGITS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_TOTAL_DIGITS, GXmlGomXsdTypeRestrictionTotalDigits))
#define GXML_GOM_XSD_TYPE_RESTRICTION_TOTAL_DIGITS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_TOTAL_DIGITS, GXmlGomXsdTypeRestrictionTotalDigitsClass))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_TOTAL_DIGITS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_TOTAL_DIGITS))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_TOTAL_DIGITS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_TOTAL_DIGITS))
#define GXML_GOM_XSD_TYPE_RESTRICTION_TOTAL_DIGITS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_TOTAL_DIGITS, GXmlGomXsdTypeRestrictionTotalDigitsClass))

typedef struct _GXmlGomXsdTypeRestrictionTotalDigits GXmlGomXsdTypeRestrictionTotalDigits;
typedef struct _GXmlGomXsdTypeRestrictionTotalDigitsClass GXmlGomXsdTypeRestrictionTotalDigitsClass;
typedef struct _GXmlGomXsdTypeRestrictionTotalDigitsPrivate GXmlGomXsdTypeRestrictionTotalDigitsPrivate;

#define GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_FRACTION_DIGITS (gxml_gom_xsd_type_restriction_fraction_digits_get_type ())
#define GXML_GOM_XSD_TYPE_RESTRICTION_FRACTION_DIGITS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_FRACTION_DIGITS, GXmlGomXsdTypeRestrictionFractionDigits))
#define GXML_GOM_XSD_TYPE_RESTRICTION_FRACTION_DIGITS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_FRACTION_DIGITS, GXmlGomXsdTypeRestrictionFractionDigitsClass))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_FRACTION_DIGITS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_FRACTION_DIGITS))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_FRACTION_DIGITS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_FRACTION_DIGITS))
#define GXML_GOM_XSD_TYPE_RESTRICTION_FRACTION_DIGITS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_FRACTION_DIGITS, GXmlGomXsdTypeRestrictionFractionDigitsClass))

typedef struct _GXmlGomXsdTypeRestrictionFractionDigits GXmlGomXsdTypeRestrictionFractionDigits;
typedef struct _GXmlGomXsdTypeRestrictionFractionDigitsClass GXmlGomXsdTypeRestrictionFractionDigitsClass;
typedef struct _GXmlGomXsdTypeRestrictionFractionDigitsPrivate GXmlGomXsdTypeRestrictionFractionDigitsPrivate;

#define GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_LENGTH (gxml_gom_xsd_type_restriction_length_get_type ())
#define GXML_GOM_XSD_TYPE_RESTRICTION_LENGTH(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_LENGTH, GXmlGomXsdTypeRestrictionLength))
#define GXML_GOM_XSD_TYPE_RESTRICTION_LENGTH_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_LENGTH, GXmlGomXsdTypeRestrictionLengthClass))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_LENGTH(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_LENGTH))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_LENGTH_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_LENGTH))
#define GXML_GOM_XSD_TYPE_RESTRICTION_LENGTH_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_LENGTH, GXmlGomXsdTypeRestrictionLengthClass))

typedef struct _GXmlGomXsdTypeRestrictionLength GXmlGomXsdTypeRestrictionLength;
typedef struct _GXmlGomXsdTypeRestrictionLengthClass GXmlGomXsdTypeRestrictionLengthClass;
typedef struct _GXmlGomXsdTypeRestrictionLengthPrivate GXmlGomXsdTypeRestrictionLengthPrivate;

#define GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MIN_LENGTH (gxml_gom_xsd_type_restriction_min_length_get_type ())
#define GXML_GOM_XSD_TYPE_RESTRICTION_MIN_LENGTH(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MIN_LENGTH, GXmlGomXsdTypeRestrictionMinLength))
#define GXML_GOM_XSD_TYPE_RESTRICTION_MIN_LENGTH_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MIN_LENGTH, GXmlGomXsdTypeRestrictionMinLengthClass))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_MIN_LENGTH(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MIN_LENGTH))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_MIN_LENGTH_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MIN_LENGTH))
#define GXML_GOM_XSD_TYPE_RESTRICTION_MIN_LENGTH_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MIN_LENGTH, GXmlGomXsdTypeRestrictionMinLengthClass))

typedef struct _GXmlGomXsdTypeRestrictionMinLength GXmlGomXsdTypeRestrictionMinLength;
typedef struct _GXmlGomXsdTypeRestrictionMinLengthClass GXmlGomXsdTypeRestrictionMinLengthClass;
typedef struct _GXmlGomXsdTypeRestrictionMinLengthPrivate GXmlGomXsdTypeRestrictionMinLengthPrivate;

#define GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MAX_LENGTH (gxml_gom_xsd_type_restriction_max_length_get_type ())
#define GXML_GOM_XSD_TYPE_RESTRICTION_MAX_LENGTH(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MAX_LENGTH, GXmlGomXsdTypeRestrictionMaxLength))
#define GXML_GOM_XSD_TYPE_RESTRICTION_MAX_LENGTH_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MAX_LENGTH, GXmlGomXsdTypeRestrictionMaxLengthClass))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_MAX_LENGTH(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MAX_LENGTH))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_MAX_LENGTH_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MAX_LENGTH))
#define GXML_GOM_XSD_TYPE_RESTRICTION_MAX_LENGTH_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_MAX_LENGTH, GXmlGomXsdTypeRestrictionMaxLengthClass))

typedef struct _GXmlGomXsdTypeRestrictionMaxLength GXmlGomXsdTypeRestrictionMaxLength;
typedef struct _GXmlGomXsdTypeRestrictionMaxLengthClass GXmlGomXsdTypeRestrictionMaxLengthClass;
typedef struct _GXmlGomXsdTypeRestrictionMaxLengthPrivate GXmlGomXsdTypeRestrictionMaxLengthPrivate;

#define GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_ENUMERATION (gxml_gom_xsd_type_restriction_enumeration_get_type ())
#define GXML_GOM_XSD_TYPE_RESTRICTION_ENUMERATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_ENUMERATION, GXmlGomXsdTypeRestrictionEnumeration))
#define GXML_GOM_XSD_TYPE_RESTRICTION_ENUMERATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_ENUMERATION, GXmlGomXsdTypeRestrictionEnumerationClass))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_ENUMERATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_ENUMERATION))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_ENUMERATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_ENUMERATION))
#define GXML_GOM_XSD_TYPE_RESTRICTION_ENUMERATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_ENUMERATION, GXmlGomXsdTypeRestrictionEnumerationClass))

typedef struct _GXmlGomXsdTypeRestrictionEnumeration GXmlGomXsdTypeRestrictionEnumeration;
typedef struct _GXmlGomXsdTypeRestrictionEnumerationClass GXmlGomXsdTypeRestrictionEnumerationClass;
typedef struct _GXmlGomXsdTypeRestrictionEnumerationPrivate GXmlGomXsdTypeRestrictionEnumerationPrivate;

#define GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE (gxml_gom_xsd_type_restriction_white_space_get_type ())
#define GXML_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE, GXmlGomXsdTypeRestrictionWhiteSpace))
#define GXML_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE, GXmlGomXsdTypeRestrictionWhiteSpaceClass))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE))
#define GXML_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE, GXmlGomXsdTypeRestrictionWhiteSpaceClass))

typedef struct _GXmlGomXsdTypeRestrictionWhiteSpace GXmlGomXsdTypeRestrictionWhiteSpace;
typedef struct _GXmlGomXsdTypeRestrictionWhiteSpaceClass GXmlGomXsdTypeRestrictionWhiteSpaceClass;
typedef struct _GXmlGomXsdTypeRestrictionWhiteSpacePrivate GXmlGomXsdTypeRestrictionWhiteSpacePrivate;

#define GXML_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE_TYPE_FIXED (gxml_gom_xsd_type_restriction_white_space_fixed_get_type ())
#define GXML_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE_FIXED(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE_TYPE_FIXED, GXmlGomXsdTypeRestrictionWhiteSpaceFixed))
#define GXML_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE_FIXED_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE_TYPE_FIXED, GXmlGomXsdTypeRestrictionWhiteSpaceFixedClass))
#define GXML_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE_IS_FIXED(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE_TYPE_FIXED))
#define GXML_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE_IS_FIXED_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE_TYPE_FIXED))
#define GXML_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE_FIXED_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_GOM_XSD_TYPE_RESTRICTION_WHITE_SPACE_TYPE_FIXED, GXmlGomXsdTypeRestrictionWhiteSpaceFixedClass))

typedef struct _GXmlGomXsdTypeRestrictionWhiteSpaceFixed GXmlGomXsdTypeRestrictionWhiteSpaceFixed;
typedef struct _GXmlGomXsdTypeRestrictionWhiteSpaceFixedClass GXmlGomXsdTypeRestrictionWhiteSpaceFixedClass;
typedef struct _GXmlGomXsdTypeRestrictionWhiteSpaceFixedPrivate GXmlGomXsdTypeRestrictionWhiteSpaceFixedPrivate;

#define GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_PATTERN (gxml_gom_xsd_type_restriction_pattern_get_type ())
#define GXML_GOM_XSD_TYPE_RESTRICTION_PATTERN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_PATTERN, GXmlGomXsdTypeRestrictionPattern))
#define GXML_GOM_XSD_TYPE_RESTRICTION_PATTERN_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_PATTERN, GXmlGomXsdTypeRestrictionPatternClass))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_PATTERN(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_PATTERN))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_PATTERN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_PATTERN))
#define GXML_GOM_XSD_TYPE_RESTRICTION_PATTERN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_PATTERN, GXmlGomXsdTypeRestrictionPatternClass))

typedef struct _GXmlGomXsdTypeRestrictionPattern GXmlGomXsdTypeRestrictionPattern;
typedef struct _GXmlGomXsdTypeRestrictionPatternClass GXmlGomXsdTypeRestrictionPatternClass;
typedef struct _GXmlGomXsdTypeRestrictionPatternPrivate GXmlGomXsdTypeRestrictionPatternPrivate;

#define GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_ASSERTION (gxml_gom_xsd_type_restriction_assertion_get_type ())
#define GXML_GOM_XSD_TYPE_RESTRICTION_ASSERTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_ASSERTION, GXmlGomXsdTypeRestrictionAssertion))
#define GXML_GOM_XSD_TYPE_RESTRICTION_ASSERTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_ASSERTION, GXmlGomXsdTypeRestrictionAssertionClass))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_ASSERTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_ASSERTION))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_ASSERTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_ASSERTION))
#define GXML_GOM_XSD_TYPE_RESTRICTION_ASSERTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_ASSERTION, GXmlGomXsdTypeRestrictionAssertionClass))

typedef struct _GXmlGomXsdTypeRestrictionAssertion GXmlGomXsdTypeRestrictionAssertion;
typedef struct _GXmlGomXsdTypeRestrictionAssertionClass GXmlGomXsdTypeRestrictionAssertionClass;
typedef struct _GXmlGomXsdTypeRestrictionAssertionPrivate GXmlGomXsdTypeRestrictionAssertionPrivate;

#define GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_EXPLICIT_TIMEZONE (gxml_gom_xsd_type_restriction_explicit_timezone_get_type ())
#define GXML_GOM_XSD_TYPE_RESTRICTION_EXPLICIT_TIMEZONE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_EXPLICIT_TIMEZONE, GXmlGomXsdTypeRestrictionExplicitTimezone))
#define GXML_GOM_XSD_TYPE_RESTRICTION_EXPLICIT_TIMEZONE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_EXPLICIT_TIMEZONE, GXmlGomXsdTypeRestrictionExplicitTimezoneClass))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_EXPLICIT_TIMEZONE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_EXPLICIT_TIMEZONE))
#define GXML_IS_GOM_XSD_TYPE_RESTRICTION_EXPLICIT_TIMEZONE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_EXPLICIT_TIMEZONE))
#define GXML_GOM_XSD_TYPE_RESTRICTION_EXPLICIT_TIMEZONE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_TYPE_RESTRICTION_EXPLICIT_TIMEZONE, GXmlGomXsdTypeRestrictionExplicitTimezoneClass))

typedef struct _GXmlGomXsdTypeRestrictionExplicitTimezone GXmlGomXsdTypeRestrictionExplicitTimezone;
typedef struct _GXmlGomXsdTypeRestrictionExplicitTimezoneClass GXmlGomXsdTypeRestrictionExplicitTimezoneClass;
typedef struct _GXmlGomXsdTypeRestrictionExplicitTimezonePrivate GXmlGomXsdTypeRestrictionExplicitTimezonePrivate;

#define GXML_TYPE_GOM_XSD_BASE_TYPE (gxml_gom_xsd_base_type_get_type ())
#define GXML_GOM_XSD_BASE_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_BASE_TYPE, GXmlGomXsdBaseType))
#define GXML_GOM_XSD_BASE_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_BASE_TYPE, GXmlGomXsdBaseTypeClass))
#define GXML_IS_GOM_XSD_BASE_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_BASE_TYPE))
#define GXML_IS_GOM_XSD_BASE_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_BASE_TYPE))
#define GXML_GOM_XSD_BASE_TYPE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_BASE_TYPE, GXmlGomXsdBaseTypeClass))

typedef struct _GXmlGomXsdBaseType GXmlGomXsdBaseType;
typedef struct _GXmlGomXsdBaseTypeClass GXmlGomXsdBaseTypeClass;
typedef struct _GXmlGomXsdBaseTypePrivate GXmlGomXsdBaseTypePrivate;

#define GXML_TYPE_GOM_XSD_COMPLEX_TYPE (gxml_gom_xsd_complex_type_get_type ())
#define GXML_GOM_XSD_COMPLEX_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_COMPLEX_TYPE, GXmlGomXsdComplexType))
#define GXML_GOM_XSD_COMPLEX_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_COMPLEX_TYPE, GXmlGomXsdComplexTypeClass))
#define GXML_IS_GOM_XSD_COMPLEX_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_COMPLEX_TYPE))
#define GXML_IS_GOM_XSD_COMPLEX_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_COMPLEX_TYPE))
#define GXML_GOM_XSD_COMPLEX_TYPE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_COMPLEX_TYPE, GXmlGomXsdComplexTypeClass))

typedef struct _GXmlGomXsdComplexType GXmlGomXsdComplexType;
typedef struct _GXmlGomXsdComplexTypeClass GXmlGomXsdComplexTypeClass;
typedef struct _GXmlGomXsdComplexTypePrivate GXmlGomXsdComplexTypePrivate;

#define GXML_TYPE_GOM_XSD_BASE_CONTENT (gxml_gom_xsd_base_content_get_type ())
#define GXML_GOM_XSD_BASE_CONTENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_BASE_CONTENT, GXmlGomXsdBaseContent))
#define GXML_GOM_XSD_BASE_CONTENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_BASE_CONTENT, GXmlGomXsdBaseContentClass))
#define GXML_IS_GOM_XSD_BASE_CONTENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_BASE_CONTENT))
#define GXML_IS_GOM_XSD_BASE_CONTENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_BASE_CONTENT))
#define GXML_GOM_XSD_BASE_CONTENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_BASE_CONTENT, GXmlGomXsdBaseContentClass))

typedef struct _GXmlGomXsdBaseContent GXmlGomXsdBaseContent;
typedef struct _GXmlGomXsdBaseContentClass GXmlGomXsdBaseContentClass;

#define GXML_TYPE_GOM_XSD_EXTENSION (gxml_gom_xsd_extension_get_type ())
#define GXML_GOM_XSD_EXTENSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_EXTENSION, GXmlGomXsdExtension))
#define GXML_GOM_XSD_EXTENSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_EXTENSION, GXmlGomXsdExtensionClass))
#define GXML_IS_GOM_XSD_EXTENSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_EXTENSION))
#define GXML_IS_GOM_XSD_EXTENSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_EXTENSION))
#define GXML_GOM_XSD_EXTENSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_EXTENSION, GXmlGomXsdExtensionClass))

typedef struct _GXmlGomXsdExtension GXmlGomXsdExtension;
typedef struct _GXmlGomXsdExtensionClass GXmlGomXsdExtensionClass;
typedef struct _GXmlGomXsdExtensionPrivate GXmlGomXsdExtensionPrivate;

#define GXML_TYPE_GOM_XSD_ELEMENT (gxml_gom_xsd_element_get_type ())
#define GXML_GOM_XSD_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_ELEMENT, GXmlGomXsdElement))
#define GXML_GOM_XSD_ELEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_ELEMENT, GXmlGomXsdElementClass))
#define GXML_IS_GOM_XSD_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_ELEMENT))
#define GXML_IS_GOM_XSD_ELEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_ELEMENT))
#define GXML_GOM_XSD_ELEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_ELEMENT, GXmlGomXsdElementClass))

typedef struct _GXmlGomXsdElement GXmlGomXsdElement;
typedef struct _GXmlGomXsdElementClass GXmlGomXsdElementClass;
typedef struct _GXmlGomXsdElementPrivate GXmlGomXsdElementPrivate;
typedef struct _GXmlGomXsdAnnotationPrivate GXmlGomXsdAnnotationPrivate;
typedef struct _GXmlGomXsdBaseContentPrivate GXmlGomXsdBaseContentPrivate;

#define GXML_TYPE_GOM_XSD_SIMPLE_CONTENT (gxml_gom_xsd_simple_content_get_type ())
#define GXML_GOM_XSD_SIMPLE_CONTENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_SIMPLE_CONTENT, GXmlGomXsdSimpleContent))
#define GXML_GOM_XSD_SIMPLE_CONTENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_SIMPLE_CONTENT, GXmlGomXsdSimpleContentClass))
#define GXML_IS_GOM_XSD_SIMPLE_CONTENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_SIMPLE_CONTENT))
#define GXML_IS_GOM_XSD_SIMPLE_CONTENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_SIMPLE_CONTENT))
#define GXML_GOM_XSD_SIMPLE_CONTENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_SIMPLE_CONTENT, GXmlGomXsdSimpleContentClass))

typedef struct _GXmlGomXsdSimpleContent GXmlGomXsdSimpleContent;
typedef struct _GXmlGomXsdSimpleContentClass GXmlGomXsdSimpleContentClass;
typedef struct _GXmlGomXsdSimpleContentPrivate GXmlGomXsdSimpleContentPrivate;

#define GXML_TYPE_GOM_XSD_COMPLEX_CONTENT (gxml_gom_xsd_complex_content_get_type ())
#define GXML_GOM_XSD_COMPLEX_CONTENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_COMPLEX_CONTENT, GXmlGomXsdComplexContent))
#define GXML_GOM_XSD_COMPLEX_CONTENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_COMPLEX_CONTENT, GXmlGomXsdComplexContentClass))
#define GXML_IS_GOM_XSD_COMPLEX_CONTENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_COMPLEX_CONTENT))
#define GXML_IS_GOM_XSD_COMPLEX_CONTENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_COMPLEX_CONTENT))
#define GXML_GOM_XSD_COMPLEX_CONTENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_COMPLEX_CONTENT, GXmlGomXsdComplexContentClass))

typedef struct _GXmlGomXsdComplexContent GXmlGomXsdComplexContent;
typedef struct _GXmlGomXsdComplexContentClass GXmlGomXsdComplexContentClass;
typedef struct _GXmlGomXsdComplexContentPrivate GXmlGomXsdComplexContentPrivate;

#define GXML_TYPE_GOM_XSD_OPEN_CONTENT (gxml_gom_xsd_open_content_get_type ())
#define GXML_GOM_XSD_OPEN_CONTENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_OPEN_CONTENT, GXmlGomXsdOpenContent))
#define GXML_GOM_XSD_OPEN_CONTENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_OPEN_CONTENT, GXmlGomXsdOpenContentClass))
#define GXML_IS_GOM_XSD_OPEN_CONTENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_OPEN_CONTENT))
#define GXML_IS_GOM_XSD_OPEN_CONTENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_OPEN_CONTENT))
#define GXML_GOM_XSD_OPEN_CONTENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_OPEN_CONTENT, GXmlGomXsdOpenContentClass))

typedef struct _GXmlGomXsdOpenContent GXmlGomXsdOpenContent;
typedef struct _GXmlGomXsdOpenContentClass GXmlGomXsdOpenContentClass;
typedef struct _GXmlGomXsdOpenContentPrivate GXmlGomXsdOpenContentPrivate;

#define GXML_TYPE_GOM_XSD_BASE_ATTRIBUTE (gxml_gom_xsd_base_attribute_get_type ())
#define GXML_GOM_XSD_BASE_ATTRIBUTE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_BASE_ATTRIBUTE, GXmlGomXsdBaseAttribute))
#define GXML_GOM_XSD_BASE_ATTRIBUTE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_BASE_ATTRIBUTE, GXmlGomXsdBaseAttributeClass))
#define GXML_IS_GOM_XSD_BASE_ATTRIBUTE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_BASE_ATTRIBUTE))
#define GXML_IS_GOM_XSD_BASE_ATTRIBUTE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_BASE_ATTRIBUTE))
#define GXML_GOM_XSD_BASE_ATTRIBUTE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_BASE_ATTRIBUTE, GXmlGomXsdBaseAttributeClass))

typedef struct _GXmlGomXsdBaseAttribute GXmlGomXsdBaseAttribute;
typedef struct _GXmlGomXsdBaseAttributeClass GXmlGomXsdBaseAttributeClass;
typedef struct _GXmlGomXsdBaseAttributePrivate GXmlGomXsdBaseAttributePrivate;

#define GXML_TYPE_GOM_XSD_ATTRIBUTE (gxml_gom_xsd_attribute_get_type ())
#define GXML_GOM_XSD_ATTRIBUTE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_ATTRIBUTE, GXmlGomXsdAttribute))
#define GXML_GOM_XSD_ATTRIBUTE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_ATTRIBUTE, GXmlGomXsdAttributeClass))
#define GXML_IS_GOM_XSD_ATTRIBUTE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_ATTRIBUTE))
#define GXML_IS_GOM_XSD_ATTRIBUTE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_ATTRIBUTE))
#define GXML_GOM_XSD_ATTRIBUTE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_ATTRIBUTE, GXmlGomXsdAttributeClass))

typedef struct _GXmlGomXsdAttribute GXmlGomXsdAttribute;
typedef struct _GXmlGomXsdAttributeClass GXmlGomXsdAttributeClass;
typedef struct _GXmlGomXsdAttributePrivate GXmlGomXsdAttributePrivate;

#define GXML_TYPE_GOM_XSD_ATTRIBUTE_GROUP (gxml_gom_xsd_attribute_group_get_type ())
#define GXML_GOM_XSD_ATTRIBUTE_GROUP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_XSD_ATTRIBUTE_GROUP, GXmlGomXsdAttributeGroup))
#define GXML_GOM_XSD_ATTRIBUTE_GROUP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_XSD_ATTRIBUTE_GROUP, GXmlGomXsdAttributeGroupClass))
#define GXML_IS_GOM_XSD_ATTRIBUTE_GROUP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_XSD_ATTRIBUTE_GROUP))
#define GXML_IS_GOM_XSD_ATTRIBUTE_GROUP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_XSD_ATTRIBUTE_GROUP))
#define GXML_GOM_XSD_ATTRIBUTE_GROUP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_XSD_ATTRIBUTE_GROUP, GXmlGomXsdAttributeGroupClass))

typedef struct _GXmlGomXsdAttributeGroup GXmlGomXsdAttributeGroup;
typedef struct _GXmlGomXsdAttributeGroupClass GXmlGomXsdAttributeGroupClass;
typedef struct _GXmlGomXsdAttributeGroupPrivate GXmlGomXsdAttributeGroupPrivate;
typedef struct _GXmlGomXsdListPrivate GXmlGomXsdListPrivate;
typedef struct _GXmlGomXsdListElementsPrivate GXmlGomXsdListElementsPrivate;
typedef struct _GXmlGomXsdListSimpleTypesPrivate GXmlGomXsdListSimpleTypesPrivate;
typedef struct _GXmlGomXsdListComplexTypesPrivate GXmlGomXsdListComplexTypesPrivate;
typedef struct _GXmlGomXsdListTypeRestrictionEnumerationsPrivate GXmlGomXsdListTypeRestrictionEnumerationsPrivate;
typedef struct _GXmlGomXsdListTypeRestrictionWhiteSpacesPrivate GXmlGomXsdListTypeRestrictionWhiteSpacesPrivate;

struct _GXmlNamespaceIface {
	GTypeInterface parent_iface;
	gchar* (*get_uri) (GXmlNamespace* self);
	gchar* (*get_prefix) (GXmlNamespace* self);
};

typedef enum  {
	GXML_NODE_TYPE_INVALID = 0,
	GXML_NODE_TYPE_ELEMENT = 1,
	GXML_NODE_TYPE_ATTRIBUTE,
	GXML_NODE_TYPE_TEXT,
	GXML_NODE_TYPE_CDATA_SECTION,
	GXML_NODE_TYPE_ENTITY_REFERENCE,
	GXML_NODE_TYPE_ENTITY,
	GXML_NODE_TYPE_PROCESSING_INSTRUCTION,
	GXML_NODE_TYPE_COMMENT,
	GXML_NODE_TYPE_DOCUMENT,
	GXML_NODE_TYPE_DOCUMENT_TYPE,
	GXML_NODE_TYPE_DOCUMENT_FRAGMENT,
	GXML_NODE_TYPE_NOTATION
} GXmlNodeType;

struct _GXmlDocumentIface {
	GTypeInterface parent_iface;
	GXmlNode* (*create_element) (GXmlDocument* self, const gchar* name, GError** error);
	GXmlNode* (*create_text) (GXmlDocument* self, const gchar* text);
	GXmlNode* (*create_comment) (GXmlDocument* self, const gchar* text);
	GXmlNode* (*create_cdata) (GXmlDocument* self, const gchar* text);
	GXmlNode* (*create_pi) (GXmlDocument* self, const gchar* target, const gchar* data);
	gboolean (*save) (GXmlDocument* self, GCancellable* cancellable, GError** error);
	gboolean (*save_as) (GXmlDocument* self, GFile* f, GCancellable* cancellable, GError** error);
	gboolean (*get_indent) (GXmlDocument* self);
	void (*set_indent) (GXmlDocument* self, gboolean value);
	gboolean (*get_ns_top) (GXmlDocument* self);
	void (*set_ns_top) (GXmlDocument* self, gboolean value);
	gboolean (*get_prefix_default_ns) (GXmlDocument* self);
	void (*set_prefix_default_ns) (GXmlDocument* self, gboolean value);
	gboolean (*get_backup) (GXmlDocument* self);
	void (*set_backup) (GXmlDocument* self, gboolean value);
	GXmlNode* (*get_root) (GXmlDocument* self);
	GFile* (*get_file) (GXmlDocument* self);
	void (*set_file) (GXmlDocument* self, GFile* value);
};

struct _GXmlNodeIface {
	GTypeInterface parent_iface;
	GXmlNode* (*get) (GXmlNode* self, const gchar* key);
	GXmlElementList* (*get_elements_by_property_value) (GXmlNode* self, const gchar* property, const gchar* value);
	GXmlElementList* (*get_elements_by_name) (GXmlNode* self, const gchar* name);
	GXmlElementList* (*get_elements_by_name_ns) (GXmlNode* self, const gchar* name, const gchar* ns);
	gchar* (*to_string) (GXmlNode* self);
	gboolean (*set_namespace) (GXmlNode* self, const gchar* uri, const gchar* prefix);
	gchar* (*ns_prefix) (GXmlNode* self);
	gchar* (*ns_uri) (GXmlNode* self);
	GeeList* (*get_namespaces) (GXmlNode* self);
	GeeBidirList* (*get_children_nodes) (GXmlNode* self);
	GeeMap* (*get_attrs) (GXmlNode* self);
	gchar* (*get_name) (GXmlNode* self);
	gchar* (*get_value) (GXmlNode* self);
	void (*set_value) (GXmlNode* self, const gchar* value);
	GXmlNodeType (*get_type_node) (GXmlNode* self);
	GXmlDocument* (*get_document) (GXmlNode* self);
	GXmlNode* (*get_parent) (GXmlNode* self);
};

struct _GXmlAttributeIface {
	GTypeInterface parent_iface;
	GXmlNamespace* (*get_namespace) (GXmlAttribute* self);
	void (*set_namespace) (GXmlAttribute* self, GXmlNamespace* value);
	gchar* (*get_prefix) (GXmlAttribute* self);
};

struct _GXmlCommentIface {
	GTypeInterface parent_iface;
	gchar* (*get_str) (GXmlComment* self);
	void (*set_str) (GXmlComment* self, const gchar* value);
};

struct _GXmlCDATAIface {
	GTypeInterface parent_iface;
	gchar* (*get_str) (GXmlCDATA* self);
};

struct _GXmlCharacterIface {
	GTypeInterface parent_iface;
	gchar* (*get_str) (GXmlCharacter* self);
	void (*set_str) (GXmlCharacter* self, const gchar* value);
};

typedef enum  {
	GXML_DOCUMENT_ERROR_INVALID_DOCUMENT_ERROR,
	GXML_DOCUMENT_ERROR_INVALID_FILE
} GXmlDocumentError;
#define GXML_DOCUMENT_ERROR gxml_document_error_quark ()
struct _GXmlDocumentTypeIface {
	GTypeInterface parent_iface;
};

struct _GXmlElementIface {
	GTypeInterface parent_iface;
	void (*normalize) (GXmlElement* self);
	void (*set_attr) (GXmlElement* self, const gchar* name, const gchar* value);
	GXmlNode* (*get_attr) (GXmlElement* self, const gchar* name);
	void (*remove_attr) (GXmlElement* self, const gchar* name);
	void (*remove_ns_attr) (GXmlElement* self, const gchar* name, const gchar* uri);
	void (*set_ns_attr) (GXmlElement* self, const gchar* ns, const gchar* name, const gchar* value);
	GXmlNode* (*get_ns_attr) (GXmlElement* self, const gchar* name, const gchar* uri);
	gchar* (*get_tag_name) (GXmlElement* self);
	gchar* (*get_content) (GXmlElement* self);
	void (*set_content) (GXmlElement* self, const gchar* value);
};

typedef enum  {
	GXML_DOM_EVENT_PHASE_NONE = 0,
	GXML_DOM_EVENT_PHASE_CAPTURING_PHASE,
	GXML_DOM_EVENT_PHASE_AT_TARGET,
	GXML_DOM_EVENT_PHASE_BUBBLING_PHASE
} GXmlDomEventPhase;

struct _GXmlDomEventIface {
	GTypeInterface parent_iface;
	void (*stop_propagation) (GXmlDomEvent* self);
	void (*stop_immediate_propagation) (GXmlDomEvent* self);
	void (*prevent_default) (GXmlDomEvent* self);
	void (*init_event) (GXmlDomEvent* self, const gchar* type, gboolean bubbles, gboolean cancelable);
	const gchar* (*get_etype) (GXmlDomEvent* self);
	GXmlDomEventTarget* (*get_event_target) (GXmlDomEvent* self);
	GXmlDomEventTarget* (*get_current_target) (GXmlDomEvent* self);
	gboolean (*get_bubbles) (GXmlDomEvent* self);
	gboolean (*get_cancelable) (GXmlDomEvent* self);
	gboolean (*get_is_trusted) (GXmlDomEvent* self);
	GXmlDomTimeStamp* (*get_time_stamp) (GXmlDomEvent* self);
	gboolean (*get_default_prevented) (GXmlDomEvent* self);
	GXmlDomEventPhase (*get_event_phase) (GXmlDomEvent* self);
};

struct _GXmlDomEventListenerIface {
	GTypeInterface parent_iface;
	void (*handle_event) (GXmlDomEventListener* self, GXmlDomEvent* event);
};

struct _GXmlDomEventTargetIface {
	GTypeInterface parent_iface;
	void (*add_event_listener) (GXmlDomEventTarget* self, const gchar* type, GXmlDomEventListener* callback, gboolean capture);
	void (*remove_event_listener) (GXmlDomEventTarget* self, const gchar* type, GXmlDomEventListener* callback, gboolean capture);
	gboolean (*dispatch_event) (GXmlDomEventTarget* self, GXmlDomEvent* event, GError** error);
};

typedef enum  {
	GXML_DOM_NODE_DOCUMENT_POSITION_NONE = 1 << 0,
	GXML_DOM_NODE_DOCUMENT_POSITION_DISCONNECTED = 1 << 1,
	GXML_DOM_NODE_DOCUMENT_POSITION_PRECEDING = 1 << 2,
	GXML_DOM_NODE_DOCUMENT_POSITION_FOLLOWING = 1 << 3,
	GXML_DOM_NODE_DOCUMENT_POSITION_CONTAINS = 1 << 4,
	GXML_DOM_NODE_DOCUMENT_POSITION_CONTAINED_BY = 1 << 5,
	GXML_DOM_NODE_DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC = 1 << 6
} GXmlDomNodeDocumentPosition;

typedef enum  {
	GXML_DOM_NODE_NODE_TYPE_INVALID = 0,
	GXML_DOM_NODE_NODE_TYPE_ELEMENT_NODE = 1,
	GXML_DOM_NODE_NODE_TYPE_ATTRIBUTE_NODE,
	GXML_DOM_NODE_NODE_TYPE_TEXT_NODE,
	GXML_DOM_NODE_NODE_TYPE_CDATA_SECTION_NODE,
	GXML_DOM_NODE_NODE_TYPE_ENTITY_REFERENCE_NODE,
	GXML_DOM_NODE_NODE_TYPE_ENTITY_NODE,
	GXML_DOM_NODE_NODE_TYPE_PROCESSING_INSTRUCTION_NODE,
	GXML_DOM_NODE_NODE_TYPE_COMMENT_NODE,
	GXML_DOM_NODE_NODE_TYPE_DOCUMENT_NODE,
	GXML_DOM_NODE_NODE_TYPE_DOCUMENT_TYPE_NODE,
	GXML_DOM_NODE_NODE_TYPE_DOCUMENT_FRAGMENT_NODE,
	GXML_DOM_NODE_NODE_TYPE_NOTATION_NODE
} GXmlDomNodeNodeType;

struct _GXmlDomNodeListIface {
	GTypeInterface parent_iface;
	GXmlDomNode* (*item) (GXmlDomNodeList* self, gint index);
	gint (*get_length) (GXmlDomNodeList* self);
};

struct _GXmlDomParentNodeIface {
	GTypeInterface parent_iface;
	GXmlDomElement* (*query_selector) (GXmlDomParentNode* self, const gchar* selectors, GError** error);
	GXmlDomNodeList* (*query_selector_all) (GXmlDomParentNode* self, const gchar* selectors, GError** error);
	GXmlDomElementList* (*get_elements_by_property_value) (GXmlDomParentNode* self, const gchar* property, const gchar* value);
	GXmlDomHTMLCollection* (*get_children) (GXmlDomParentNode* self);
	GXmlDomElement* (*get_first_element_child) (GXmlDomParentNode* self);
	GXmlDomElement* (*get_last_element_child) (GXmlDomParentNode* self);
	gint (*get_child_element_count) (GXmlDomParentNode* self);
};

struct _GXmlDomNonElementParentNodeIface {
	GTypeInterface parent_iface;
	GXmlDomElement* (*get_element_by_id) (GXmlDomNonElementParentNode* self, const gchar* element_id, GError** error);
};

struct _GXmlDomDocumentFragmentIface {
	GTypeInterface parent_iface;
};

struct _GXmlDomNonDocumentTypeChildNodeIface {
	GTypeInterface parent_iface;
	GXmlDomElement* (*get_previous_element_sibling) (GXmlDomNonDocumentTypeChildNode* self);
	GXmlDomElement* (*get_next_element_sibling) (GXmlDomNonDocumentTypeChildNode* self);
};

struct _GXmlDomChildNodeIface {
	GTypeInterface parent_iface;
	void (*remove) (GXmlDomChildNode* self);
};

struct _GXmlDomCharacterDataIface {
	GTypeInterface parent_iface;
	gchar* (*substring_data) (GXmlDomCharacterData* self, gint offset, gint count, GError** error);
	void (*append_data) (GXmlDomCharacterData* self, const gchar* data);
	void (*insert_data) (GXmlDomCharacterData* self, gint offset, const gchar* data, GError** error);
	void (*delete_data) (GXmlDomCharacterData* self, gint offset, gint count, GError** error);
	void (*replace_data) (GXmlDomCharacterData* self, gint offset, gint count, const gchar* data, GError** error);
	gchar* (*get_data) (GXmlDomCharacterData* self);
	void (*set_data) (GXmlDomCharacterData* self, const gchar* value);
	gint (*get_length) (GXmlDomCharacterData* self);
};

struct _GXmlDomTextIface {
	GTypeInterface parent_iface;
	GXmlDomText* (*split_text) (GXmlDomText* self, gint offset, GError** error);
	gchar* (*get_whole_text) (GXmlDomText* self);
};

struct _GXmlDomCommentIface {
	GTypeInterface parent_iface;
};

struct _GXmlDomProcessingInstructionIface {
	GTypeInterface parent_iface;
	gchar* (*get_target) (GXmlDomProcessingInstruction* self);
};

typedef enum  {
	GXML_DOM_RANGE_BOUNDARY_POINTS_START_TO_START = 0,
	GXML_DOM_RANGE_BOUNDARY_POINTS_START_TO_END,
	GXML_DOM_RANGE_BOUNDARY_POINTS_END_TO_END,
	GXML_DOM_RANGE_BOUNDARY_POINTS_END_TO_START
} GXmlDomRangeBoundaryPoints;

struct _GXmlDomRangeIface {
	GTypeInterface parent_iface;
	void (*set_start) (GXmlDomRange* self, GXmlDomNode* node, gint offset, GError** error);
	void (*set_end) (GXmlDomRange* self, GXmlDomNode* node, gint offset, GError** error);
	void (*set_start_before) (GXmlDomRange* self, GXmlDomNode* node, GError** error);
	void (*set_start_after) (GXmlDomRange* self, GXmlDomNode* node, GError** error);
	void (*set_end_before) (GXmlDomRange* self, GXmlDomNode* node, GError** error);
	void (*set_end_after) (GXmlDomRange* self, GXmlDomNode* node, GError** error);
	void (*collapse) (GXmlDomRange* self, gboolean to_start, GError** error);
	void (*select_node) (GXmlDomRange* self, GXmlDomNode* node, GError** error);
	void (*select_node_contents) (GXmlDomRange* self, GXmlDomNode* node, GError** error);
	gint (*compare_boundary_points) (GXmlDomRange* self, GXmlDomRangeBoundaryPoints how, GXmlDomRange* sourceRange, GError** error);
	void (*delete_contents) (GXmlDomRange* self, GError** error);
	GXmlDomDocumentFragment* (*extract_contents) (GXmlDomRange* self, GError** error);
	GXmlDomDocumentFragment* (*clone_contents) (GXmlDomRange* self, GError** error);
	void (*insert_node) (GXmlDomRange* self, GXmlDomNode* node);
	void (*surround_contents) (GXmlDomRange* self, GXmlDomNode* newParent);
	GXmlDomRange* (*clone_range) (GXmlDomRange* self);
	void (*detach) (GXmlDomRange* self);
	gboolean (*is_point_in_range) (GXmlDomRange* self, GXmlDomNode* node, gint offset);
	gshort (*compare_point) (GXmlDomRange* self, GXmlDomNode* node, gint offset);
	gboolean (*intersects_node) (GXmlDomRange* self, GXmlDomNode* node);
	gchar* (*to_string) (GXmlDomRange* self);
	GXmlDomNode* (*get_start_container) (GXmlDomRange* self);
	gint (*get_start_offset) (GXmlDomRange* self);
	GXmlDomNode* (*get_end_container) (GXmlDomRange* self);
	gint (*get_end_offset) (GXmlDomRange* self);
	gboolean (*get_collapsed) (GXmlDomRange* self);
	GXmlDomNode* (*get_common_ancestor_container) (GXmlDomRange* self);
};

struct _GXmlDomNodeIteratorIface {
	GTypeInterface parent_iface;
	GXmlDomNode* (*next_node) (GXmlDomNodeIterator* self);
	GXmlDomNode* (*previous_node) (GXmlDomNodeIterator* self);
	void (*detach) (GXmlDomNodeIterator* self);
	GXmlDomNode* (*get_root) (GXmlDomNodeIterator* self);
	GXmlDomNode* (*get_reference_node) (GXmlDomNodeIterator* self);
	gboolean (*get_pointer_before_reference_node) (GXmlDomNodeIterator* self);
	gint (*get_what_to_show) (GXmlDomNodeIterator* self);
	GXmlDomNodeFilter* (*get_filter) (GXmlDomNodeIterator* self);
};

struct _GXmlDomTreeWalkerIface {
	GTypeInterface parent_iface;
	GXmlDomNode* (*parentNode) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*firstChild) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*lastChild) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*previousSibling) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*nextSibling) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*previousNode) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*nextNode) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*get_root) (GXmlDomTreeWalker* self);
	gint (*get_what_to_show) (GXmlDomTreeWalker* self);
	GXmlDomNodeFilter* (*get_filter) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*get_current_node) (GXmlDomTreeWalker* self);
};

struct _GXmlDomDocumentTypeIface {
	GTypeInterface parent_iface;
	const gchar* (*get_name) (GXmlDomDocumentType* self);
	const gchar* (*get_public_id) (GXmlDomDocumentType* self);
	const gchar* (*get_system_id) (GXmlDomDocumentType* self);
};

struct _GXmlDomXMLDocumentIface {
	GTypeInterface parent_iface;
};

struct _GXmlDomImplementationIface {
	GTypeInterface parent_iface;
	GXmlDomDocumentType* (*create_document_type) (GXmlDomImplementation* self, const gchar* qualified_name, const gchar* public_id, const gchar* system_id, GError** error);
	GXmlDomXMLDocument* (*create_document) (GXmlDomImplementation* self, const gchar* nspace, const gchar* qualified_name, GXmlDomDocumentType* doctype, GError** error);
	GXmlDocument* (*create_html_document) (GXmlDomImplementation* self, const gchar* title);
	gboolean (*has_feature) (GXmlDomImplementation* self);
};

struct _GXmlDomDocumentIface {
	GTypeInterface parent_iface;
	GXmlDomHTMLCollection* (*get_elements_by_tag_name) (GXmlDomDocument* self, const gchar* local_name);
	GXmlDomHTMLCollection* (*get_elements_by_tag_name_ns) (GXmlDomDocument* self, const gchar* namespace, const gchar* local_name);
	GXmlDomHTMLCollection* (*get_elements_by_class_name) (GXmlDomDocument* self, const gchar* classNames);
	GXmlDomElement* (*create_element) (GXmlDomDocument* self, const gchar* local_name, GError** error);
	GXmlDomElement* (*create_element_ns) (GXmlDomDocument* self, const gchar* namespace, const gchar* qualified_name, GError** error);
	GXmlDomDocumentFragment* (*create_document_fragment) (GXmlDomDocument* self);
	GXmlDomText* (*create_text_node) (GXmlDomDocument* self, const gchar* data, GError** error);
	GXmlDomComment* (*create_comment) (GXmlDomDocument* self, const gchar* data, GError** error);
	GXmlDomProcessingInstruction* (*create_processing_instruction) (GXmlDomDocument* self, const gchar* target, const gchar* data, GError** error);
	GXmlDomNode* (*import_node) (GXmlDomDocument* self, GXmlDomNode* node, gboolean deep, GError** error);
	GXmlDomNode* (*adopt_node) (GXmlDomDocument* self, GXmlDomNode* node, GError** error);
	GXmlDomEvent* (*create_event) (GXmlDomDocument* self, const gchar* interface, GError** error);
	GXmlDomRange* (*create_range) (GXmlDomDocument* self);
	GXmlDomNodeIterator* (*create_node_iterator) (GXmlDomDocument* self, GXmlDomNode* root, gint whatToShow, GXmlDomNodeFilter* filter);
	GXmlDomTreeWalker* (*create_tree_walker) (GXmlDomDocument* self, GXmlDomNode* root, gint what_to_show, GXmlDomNodeFilter* filter);
	GXmlDomImplementation* (*get_implementation) (GXmlDomDocument* self);
	const gchar* (*get_url) (GXmlDomDocument* self);
	const gchar* (*get_document_uri) (GXmlDomDocument* self);
	const gchar* (*get_origin) (GXmlDomDocument* self);
	const gchar* (*get_compat_mode) (GXmlDomDocument* self);
	const gchar* (*get_character_set) (GXmlDomDocument* self);
	const gchar* (*get_content_type) (GXmlDomDocument* self);
	GXmlDomDocumentType* (*get_doctype) (GXmlDomDocument* self);
	GXmlDomElement* (*get_document_element) (GXmlDomDocument* self);
};

struct _GXmlDomNodeIface {
	GTypeInterface parent_iface;
	gboolean (*has_child_nodes) (GXmlDomNode* self);
	void (*normalize) (GXmlDomNode* self);
	gboolean (*is_equal_node) (GXmlDomNode* self, GXmlDomNode* node);
	GXmlDomNodeDocumentPosition (*compare_document_position) (GXmlDomNode* self, GXmlDomNode* other);
	gboolean (*contains) (GXmlDomNode* self, GXmlDomNode* other);
	gchar* (*lookup_prefix) (GXmlDomNode* self, const gchar* nspace);
	gchar* (*lookup_namespace_uri) (GXmlDomNode* self, const gchar* prefix);
	gboolean (*is_default_namespace) (GXmlDomNode* self, const gchar* nspace);
	GXmlDomNode* (*insert_before) (GXmlDomNode* self, GXmlDomNode* node, GXmlDomNode* child, GError** error);
	GXmlDomNode* (*append_child) (GXmlDomNode* self, GXmlDomNode* node, GError** error);
	GXmlDomNode* (*replace_child) (GXmlDomNode* self, GXmlDomNode* node, GXmlDomNode* child, GError** error);
	GXmlDomNode* (*remove_child) (GXmlDomNode* self, GXmlDomNode* child, GError** error);
	GXmlDomNode* (*clone_node) (GXmlDomNode* self, gboolean deep, GError** error);
	GXmlDomNodeNodeType (*get_node_type) (GXmlDomNode* self);
	gchar* (*get_node_name) (GXmlDomNode* self);
	const gchar* (*get_base_uri) (GXmlDomNode* self);
	GXmlDomDocument* (*get_owner_document) (GXmlDomNode* self);
	void (*set_owner_document) (GXmlDomNode* self, GXmlDomDocument* value);
	GXmlDomNode* (*get_parent_node) (GXmlDomNode* self);
	GXmlDomElement* (*get_parent_element) (GXmlDomNode* self);
	GXmlDomNodeList* (*get_child_nodes) (GXmlDomNode* self);
	GXmlDomNode* (*get_first_child) (GXmlDomNode* self);
	GXmlDomNode* (*get_last_child) (GXmlDomNode* self);
	GXmlDomNode* (*get_previous_sibling) (GXmlDomNode* self);
	GXmlDomNode* (*get_next_sibling) (GXmlDomNode* self);
	gchar* (*get_node_value) (GXmlDomNode* self);
	void (*set_node_value) (GXmlDomNode* self, const gchar* value);
	gchar* (*get_text_content) (GXmlDomNode* self);
	void (*set_text_content) (GXmlDomNode* self, const gchar* value);
};

struct _GXmlDomTokenListIface {
	GTypeInterface parent_iface;
	gchar* (*item) (GXmlDomTokenList* self, gint index);
	gboolean (*contains) (GXmlDomTokenList* self, const gchar* token, GError** error);
	void (*add) (GXmlDomTokenList* self, gchar** tokens, int tokens_length1, GError** error);
	void (*remove) (GXmlDomTokenList* self, gchar** tokens, int tokens_length1);
	gboolean (*toggle) (GXmlDomTokenList* self, const gchar* token, gboolean force, gboolean _auto_, GError** error);
	gchar* (*to_string) (GXmlDomTokenList* self);
	gint (*get_length) (GXmlDomTokenList* self);
};

struct _GXmlDomNamedNodeMapIface {
	GTypeInterface parent_iface;
	GXmlDomNode* (*item) (GXmlDomNamedNodeMap* self, gint index);
	GXmlDomNode* (*get_named_item) (GXmlDomNamedNodeMap* self, const gchar* name);
	GXmlDomNode* (*set_named_item) (GXmlDomNamedNodeMap* self, GXmlDomNode* node, GError** error);
	GXmlDomNode* (*remove_named_item) (GXmlDomNamedNodeMap* self, const gchar* name, GError** error);
	GXmlDomNode* (*remove_named_item_ns) (GXmlDomNamedNodeMap* self, const gchar* namespace_uri, const gchar* localName, GError** error);
	GXmlDomNode* (*get_named_item_ns) (GXmlDomNamedNodeMap* self, const gchar* namespace_uri, const gchar* local_name, GError** error);
	GXmlDomNode* (*set_named_item_ns) (GXmlDomNamedNodeMap* self, GXmlDomNode* node, GError** error);
	gint (*get_length) (GXmlDomNamedNodeMap* self);
};

struct _GXmlDomElementIface {
	GTypeInterface parent_iface;
	gchar* (*get_attribute) (GXmlDomElement* self, const gchar* name);
	gchar* (*get_attribute_ns) (GXmlDomElement* self, const gchar* namespace, const gchar* local_name);
	void (*set_attribute) (GXmlDomElement* self, const gchar* name, const gchar* value, GError** error);
	void (*set_attribute_ns) (GXmlDomElement* self, const gchar* namespace, const gchar* name, const gchar* value, GError** error);
	void (*remove_attribute) (GXmlDomElement* self, const gchar* name);
	void (*remove_attribute_ns) (GXmlDomElement* self, const gchar* namespace, const gchar* local_name);
	gboolean (*has_attribute) (GXmlDomElement* self, const gchar* name);
	gboolean (*has_attribute_ns) (GXmlDomElement* self, const gchar* namespace, const gchar* local_name);
	GXmlDomHTMLCollection* (*get_elements_by_tag_name) (GXmlDomElement* self, const gchar* local_name);
	GXmlDomHTMLCollection* (*get_elements_by_tag_name_ns) (GXmlDomElement* self, const gchar* namespace, const gchar* local_name);
	GXmlDomHTMLCollection* (*get_elements_by_class_name) (GXmlDomElement* self, const gchar* class_names);
	gchar* (*get_namespace_uri) (GXmlDomElement* self);
	gchar* (*get_prefix) (GXmlDomElement* self);
	gchar* (*get_local_name) (GXmlDomElement* self);
	gchar* (*get_tag_name) (GXmlDomElement* self);
	gchar* (*get_id) (GXmlDomElement* self);
	void (*set_id) (GXmlDomElement* self, const gchar* value);
	gchar* (*get_class_name) (GXmlDomElement* self);
	void (*set_class_name) (GXmlDomElement* self, const gchar* value);
	GXmlDomTokenList* (*get_class_list) (GXmlDomElement* self);
	GXmlDomNamedNodeMap* (*get_attributes) (GXmlDomElement* self);
};

struct _GXmlDomHTMLCollectionIface {
	GTypeInterface parent_iface;
	GXmlDomElement* (*get_element) (GXmlDomHTMLCollection* self, gint index);
	GXmlDomElement** (*to_array) (GXmlDomHTMLCollection* self, int* result_length1);
	GXmlDomElement* (*item) (GXmlDomHTMLCollection* self, gint index);
	GXmlDomElement* (*named_item) (GXmlDomHTMLCollection* self, const gchar* name);
	gint (*get_length) (GXmlDomHTMLCollection* self);
};

struct _GXmlElementList {
	GeeArrayList parent_instance;
	GXmlElementListPrivate * priv;
};

struct _GXmlElementListClass {
	GeeArrayListClass parent_class;
};

struct _GXmlNodeList {
	GeeArrayList parent_instance;
	GXmlNodeListPrivate * priv;
};

struct _GXmlNodeListClass {
	GeeArrayListClass parent_class;
};

struct _GXmlNotationIface {
	GTypeInterface parent_iface;
	const gchar* (*get_public_id) (GXmlNotation* self);
	const gchar* (*get_external_id) (GXmlNotation* self);
};

struct _GXmlProcessingInstructionIface {
	GTypeInterface parent_iface;
	gchar* (*get_target) (GXmlProcessingInstruction* self);
	gchar* (*get_data) (GXmlProcessingInstruction* self);
	void (*set_data) (GXmlProcessingInstruction* self, const gchar* value);
};

struct _GXmlTextIface {
	GTypeInterface parent_iface;
	gchar* (*get_str) (GXmlText* self);
	void (*set_str) (GXmlText* self, const gchar* value);
};

struct _GXmlSerializableIface {
	GTypeInterface parent_iface;
	gboolean (*get_enable_unknown_serializable_property) (GXmlSerializable* self);
	gboolean (*set_default_namespace) (GXmlSerializable* self, GXmlNode* node);
	gboolean (*serialize_use_xml_node_value) (GXmlSerializable* self);
	gchar* (*node_name) (GXmlSerializable* self);
	gboolean (*property_use_nick) (GXmlSerializable* self);
	GXmlNode* (*serialize) (GXmlSerializable* self, GXmlNode* node, GError** error);
	GXmlNode* (*serialize_property) (GXmlSerializable* self, GXmlNode* element, GParamSpec* prop, GError** error);
	gboolean (*deserialize) (GXmlSerializable* self, GXmlNode* node, GError** error);
	gboolean (*deserialize_property) (GXmlSerializable* self, GXmlNode* property_node, GError** error);
	GParamSpec* (*find_property_spec) (GXmlSerializable* self, const gchar* property_name);
	GParamSpec* (*default_find_property_spec) (GXmlSerializable* self, const gchar* property_name);
	GParamSpec** (*list_serializable_properties) (GXmlSerializable* self, int* result_length1);
	GParamSpec** (*default_list_serializable_properties) (GXmlSerializable* self, int* result_length1);
	GeeMap* (*get_unknown_serializable_properties) (GXmlSerializable* self);
	GeeCollection* (*get_unknown_serializable_nodes) (GXmlSerializable* self);
	gchar* (*get_serialized_xml_node_value) (GXmlSerializable* self);
	void (*set_serialized_xml_node_value) (GXmlSerializable* self, const gchar* value);
};

typedef enum  {
	GXML_SERIALIZABLE_ERROR_UNSUPPORTED_TYPE_ERROR,
	GXML_SERIALIZABLE_ERROR_STR_TO_VALUE_ERROR
} GXmlSerializableError;
#define GXML_SERIALIZABLE_ERROR gxml_serializable_error_quark ()
struct _GXmlSerializablePropertyIface {
	GTypeInterface parent_iface;
	gchar* (*get_serializable_property_value) (GXmlSerializableProperty* self);
	void (*set_serializable_property_value) (GXmlSerializableProperty* self, const gchar* val);
	gboolean (*deserialize_property) (GXmlSerializableProperty* self, GXmlNode* property_node, GParamSpec* prop, gboolean nick, GError** error);
	GXmlNode* (*serialize_property) (GXmlSerializableProperty* self, GXmlNode* property_node, GParamSpec* prop, gboolean nick, GError** error);
};

struct _GXmlSerializableBool {
	GObject parent_instance;
	GXmlSerializableBoolPrivate * priv;
};

struct _GXmlSerializableBoolClass {
	GObjectClass parent_class;
};

struct _GXmlSerializableDouble {
	GObject parent_instance;
	GXmlSerializableDoublePrivate * priv;
	gchar* _val;
	gint _fraction;
};

struct _GXmlSerializableDoubleClass {
	GObjectClass parent_class;
};

struct _GXmlSerializableFloat {
	GXmlSerializableDouble parent_instance;
	GXmlSerializableFloatPrivate * priv;
};

struct _GXmlSerializableFloatClass {
	GXmlSerializableDoubleClass parent_class;
};

typedef enum  {
	GXML_SERIALIZABLE_ENUM_ERROR_INVALID_VALUE_ERROR,
	GXML_SERIALIZABLE_ENUM_ERROR_PARSE_ERROR
} GXmlSerializableEnumError;
#define GXML_SERIALIZABLE_ENUM_ERROR gxml_serializable_enum_error_quark ()
struct _GXmlSerializableEnum {
	GObject parent_instance;
	GXmlSerializableEnumPrivate * priv;
	gchar* _val;
	GType _enumtype;
};

struct _GXmlSerializableEnumClass {
	GObjectClass parent_class;
};

struct _GXmlSerializableInt {
	GXmlSerializableDouble parent_instance;
	GXmlSerializableIntPrivate * priv;
};

struct _GXmlSerializableIntClass {
	GXmlSerializableDoubleClass parent_class;
};

struct _GXmlSerializableValueList {
	GObject parent_instance;
	GXmlSerializableValueListPrivate * priv;
	gchar** _vals;
	gint _vals_length1;
	GeeArrayList* extra;
};

struct _GXmlSerializableValueListClass {
	GObjectClass parent_class;
	GeeList* (*get_values) (GXmlSerializableValueList* self);
	void (*add_values) (GXmlSerializableValueList* self, gchar** vals, int vals_length1);
	gchar* (*get_value_at) (GXmlSerializableValueList* self, gint index);
	void (*select_value_at) (GXmlSerializableValueList* self, gint index);
	gchar** (*get_values_array) (GXmlSerializableValueList* self, int* result_length1);
	gboolean (*is_value) (GXmlSerializableValueList* self);
};

struct _GXmlEnumeration {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GXmlEnumerationPrivate * priv;
};

struct _GXmlEnumerationClass {
	GTypeClass parent_class;
	void (*finalize) (GXmlEnumeration *self);
};

typedef enum  {
	GXML_ENUMERATION_ERROR_INVALID_VALUE,
	GXML_ENUMERATION_ERROR_INVALID_TEXT
} GXmlEnumerationError;
#define GXML_ENUMERATION_ERROR gxml_enumeration_error_quark ()
struct _GXmlSerializableObjectModel {
	GObject parent_instance;
	GXmlSerializableObjectModelPrivate * priv;
	GXmlNode* _node;
};

struct _GXmlSerializableObjectModelClass {
	GObjectClass parent_class;
	gboolean (*get_enable_unknown_serializable_property) (GXmlSerializableObjectModel* self);
	gboolean (*serialize_use_xml_node_value) (GXmlSerializableObjectModel* self);
	gboolean (*property_use_nick) (GXmlSerializableObjectModel* self);
	gboolean (*set_default_namespace) (GXmlSerializableObjectModel* self, GXmlNode* node);
	gchar* (*node_name) (GXmlSerializableObjectModel* self);
	GParamSpec* (*find_property_spec) (GXmlSerializableObjectModel* self, const gchar* property_name);
	GParamSpec** (*list_serializable_properties) (GXmlSerializableObjectModel* self, int* result_length1);
	GXmlNode* (*serialize) (GXmlSerializableObjectModel* self, GXmlNode* node, GError** error);
	GXmlNode* (*serialize_property) (GXmlSerializableObjectModel* self, GXmlNode* element, GParamSpec* prop, GError** error);
	gboolean (*deserialize) (GXmlSerializableObjectModel* self, GXmlNode* node, GError** error);
	gboolean (*deserialize_property) (GXmlSerializableObjectModel* self, GXmlNode* property_node, GError** error);
	gchar* (*to_string) (GXmlSerializableObjectModel* self);
};

struct _GXmlSerializableCollectionIface {
	GTypeInterface parent_iface;
	gboolean (*deserialize_proceed) (GXmlSerializableCollection* self);
	gboolean (*deserialized) (GXmlSerializableCollection* self);
	gboolean (*deserialize_node) (GXmlSerializableCollection* self, GXmlNode* node, GError** error);
	gboolean (*deserialize_children) (GXmlSerializableCollection* self, GError** error);
	gboolean (*is_collection) (GXmlSerializableCollection* self);
};

struct _GXmlSerializableTreeMap {
	GeeTreeMap parent_instance;
	GXmlSerializableTreeMapPrivate * priv;
	GXmlNode* _node;
	gboolean _deserialized;
};

struct _GXmlSerializableTreeMapClass {
	GeeTreeMapClass parent_class;
	gboolean (*deserialize_proceed) (GXmlSerializableTreeMap* self);
	gboolean (*deserialized) (GXmlSerializableTreeMap* self);
	gboolean (*deserialize_node) (GXmlSerializableTreeMap* self, GXmlNode* node, GError** error);
	gboolean (*deserialize_children) (GXmlSerializableTreeMap* self, GError** error);
	gboolean (*set_default_namespace) (GXmlSerializableTreeMap* self, GXmlNode* node);
	gboolean (*serialize_use_xml_node_value) (GXmlSerializableTreeMap* self);
	gboolean (*property_use_nick) (GXmlSerializableTreeMap* self);
	gchar* (*node_name) (GXmlSerializableTreeMap* self);
	GParamSpec* (*find_property_spec) (GXmlSerializableTreeMap* self, const gchar* property_name);
	GParamSpec** (*list_serializable_properties) (GXmlSerializableTreeMap* self, int* result_length1);
	GXmlNode* (*serialize) (GXmlSerializableTreeMap* self, GXmlNode* node, GError** error);
	GXmlNode* (*serialize_property) (GXmlSerializableTreeMap* self, GXmlNode* element, GParamSpec* prop, GError** error);
	gboolean (*deserialize) (GXmlSerializableTreeMap* self, GXmlNode* node, GError** error);
	gboolean (*deserialize_property) (GXmlSerializableTreeMap* self, GXmlNode* property_node, GError** error);
};

struct _GXmlSerializableHashMap {
	GeeHashMap parent_instance;
	GXmlSerializableHashMapPrivate * priv;
	GXmlNode* _node;
	gboolean _deserialized;
};

struct _GXmlSerializableHashMapClass {
	GeeHashMapClass parent_class;
	gboolean (*deserialize_proceed) (GXmlSerializableHashMap* self);
	gboolean (*deserialized) (GXmlSerializableHashMap* self);
	gboolean (*deserialize_node) (GXmlSerializableHashMap* self, GXmlNode* node, GError** error);
	gboolean (*deserialize_children) (GXmlSerializableHashMap* self, GError** error);
	gboolean (*set_default_namespace) (GXmlSerializableHashMap* self, GXmlNode* node);
	gboolean (*get_enable_unknown_serializable_property) (GXmlSerializableHashMap* self);
	gboolean (*serialize_use_xml_node_value) (GXmlSerializableHashMap* self);
	gboolean (*property_use_nick) (GXmlSerializableHashMap* self);
	gchar* (*node_name) (GXmlSerializableHashMap* self);
	GParamSpec* (*find_property_spec) (GXmlSerializableHashMap* self, const gchar* property_name);
	GParamSpec** (*list_serializable_properties) (GXmlSerializableHashMap* self, int* result_length1);
	GXmlNode* (*serialize) (GXmlSerializableHashMap* self, GXmlNode* node, GError** error);
	GXmlNode* (*serialize_property) (GXmlSerializableHashMap* self, GXmlNode* element, GParamSpec* prop, GError** error);
	gboolean (*deserialize) (GXmlSerializableHashMap* self, GXmlNode* node, GError** error);
	gboolean (*deserialize_property) (GXmlSerializableHashMap* self, GXmlNode* property_node, GError** error);
};

struct _GXmlSerializableMapKeyIface {
	GTypeInterface parent_iface;
	gpointer (*get_map_key) (GXmlSerializableMapKey* self);
};

struct _GXmlSerializableDualKeyMap {
	GObject parent_instance;
	GXmlSerializableDualKeyMapPrivate * priv;
	GeeHashMultiMap* storage;
	GXmlNode* _node;
	gboolean _deserialized;
};

struct _GXmlSerializableDualKeyMapClass {
	GObjectClass parent_class;
	gboolean (*deserialize_proceed) (GXmlSerializableDualKeyMap* self);
	gboolean (*deserialized) (GXmlSerializableDualKeyMap* self);
	gboolean (*deserialize_node) (GXmlSerializableDualKeyMap* self, GXmlNode* node, GError** error);
	gboolean (*deserialize_children) (GXmlSerializableDualKeyMap* self, GError** error);
	gboolean (*set_default_namespace) (GXmlSerializableDualKeyMap* self, GXmlNode* node);
	gboolean (*get_enable_unknown_serializable_property) (GXmlSerializableDualKeyMap* self);
	gboolean (*serialize_use_xml_node_value) (GXmlSerializableDualKeyMap* self);
	gboolean (*property_use_nick) (GXmlSerializableDualKeyMap* self);
	gchar* (*node_name) (GXmlSerializableDualKeyMap* self);
	GParamSpec* (*find_property_spec) (GXmlSerializableDualKeyMap* self, const gchar* property_name);
	GParamSpec** (*list_serializable_properties) (GXmlSerializableDualKeyMap* self, int* result_length1);
	GXmlNode* (*serialize) (GXmlSerializableDualKeyMap* self, GXmlNode* node, GError** error);
	GXmlNode* (*serialize_property) (GXmlSerializableDualKeyMap* self, GXmlNode* element, GParamSpec* prop, GError** error);
	gboolean (*deserialize) (GXmlSerializableDualKeyMap* self, GXmlNode* node, GError** error);
	gboolean (*deserialize_property) (GXmlSerializableDualKeyMap* self, GXmlNode* property_node, GError** error);
};

struct _GXmlSerializableMapDualKeyIface {
	GTypeInterface parent_iface;
	gpointer (*get_map_primary_key) (GXmlSerializableMapDualKey* self);
	gpointer (*get_map_secondary_key) (GXmlSerializableMapDualKey* self);
};

struct _GXmlSerializableArrayList {
	GeeArrayList parent_instance;
	GXmlSerializableArrayListPrivate * priv;
	GXmlNode* _node;
	gboolean _deserialized;
};

struct _GXmlSerializableArrayListClass {
	GeeArrayListClass parent_class;
	gboolean (*deserialize_proceed) (GXmlSerializableArrayList* self);
	gboolean (*deserialized) (GXmlSerializableArrayList* self);
	gboolean (*deserialize_node) (GXmlSerializableArrayList* self, GXmlNode* node, GError** error);
	gboolean (*deserialize_children) (GXmlSerializableArrayList* self, GError** error);
	gboolean (*set_default_namespace) (GXmlSerializableArrayList* self, GXmlNode* node);
	gboolean (*serialize_use_xml_node_value) (GXmlSerializableArrayList* self);
	gboolean (*property_use_nick) (GXmlSerializableArrayList* self);
	gchar* (*node_name) (GXmlSerializableArrayList* self);
	GParamSpec* (*find_property_spec) (GXmlSerializableArrayList* self, const gchar* property_name);
	GParamSpec** (*list_serializable_properties) (GXmlSerializableArrayList* self, int* result_length1);
	GXmlNode* (*serialize) (GXmlSerializableArrayList* self, GXmlNode* node, GError** error);
	GXmlNode* (*serialize_property) (GXmlSerializableArrayList* self, GXmlNode* element, GParamSpec* prop, GError** error);
	gboolean (*deserialize) (GXmlSerializableArrayList* self, GXmlNode* node, GError** error);
	gboolean (*deserialize_property) (GXmlSerializableArrayList* self, GXmlNode* property_node, GError** error);
};

struct _GXmlSerializableContainer {
	GXmlSerializableObjectModel parent_instance;
	GXmlSerializableContainerPrivate * priv;
};

struct _GXmlSerializableContainerClass {
	GXmlSerializableObjectModelClass parent_class;
	void (*init_containers) (GXmlSerializableContainer* self);
};

struct _GXmlTNode {
	GObject parent_instance;
	GXmlTNodePrivate * priv;
	gchar* _name;
	gchar* _value;
	GXmlDocument* _doc;
	GXmlNode* _parent;
	GXmlNodeType _node_type;
	xmlTextWriter* tw;
};

struct _GXmlTNodeClass {
	GObjectClass parent_class;
	gboolean (*set_namespace) (GXmlTNode* self, const gchar* uri, const gchar* prefix);
	gchar* (*to_string) (GXmlTNode* self);
	void (*set_parent) (GXmlTNode* self, GXmlNode* node);
	GeeMap* (*get_attrs) (GXmlTNode* self);
	GeeBidirList* (*get_children_nodes) (GXmlTNode* self);
	GXmlDocument* (*get_document) (GXmlTNode* self);
	gchar* (*get_name) (GXmlTNode* self);
	GeeList* (*get_namespaces) (GXmlTNode* self);
	GXmlNodeType (*get_type_node) (GXmlTNode* self);
	gchar* (*get_value) (GXmlTNode* self);
	void (*set_value) (GXmlTNode* self, const gchar* value);
	GXmlNode* (*get_parent) (GXmlTNode* self);
};

struct _GXmlTAttribute {
	GXmlTNode parent_instance;
	GXmlTAttributePrivate * priv;
	GeeArrayList* _namespaces;
};

struct _GXmlTAttributeClass {
	GXmlTNodeClass parent_class;
};

struct _GXmlTComment {
	GXmlTNode parent_instance;
	GXmlTCommentPrivate * priv;
};

struct _GXmlTCommentClass {
	GXmlTNodeClass parent_class;
};

struct _GXmlTCDATA {
	GXmlTNode parent_instance;
	GXmlTCDATAPrivate * priv;
};

struct _GXmlTCDATAClass {
	GXmlTNodeClass parent_class;
};

struct _GXmlTDocument {
	GXmlTNode parent_instance;
	GXmlTDocumentPrivate * priv;
	GeeArrayList* _namespaces;
	GeeArrayList* _children;
};

struct _GXmlTDocumentClass {
	GXmlTNodeClass parent_class;
};

typedef enum  {
	GXML_TDOCUMENT_READ_TYPE_NEXT,
	GXML_TDOCUMENT_READ_TYPE_CONTINUE,
	GXML_TDOCUMENT_READ_TYPE_STOP
} GXmlTDocumentReadType;

typedef GXmlTDocumentReadType (*GXmlTDocumentReadTypeFunc) (GXmlNode* node, xmlTextReader* tr, void* user_data);
struct _GXmlTElement {
	GXmlTNode parent_instance;
	GXmlTElementPrivate * priv;
	GeeHashMap* _attrs;
	GXmlTNodeTChildrenList* _children;
	GeeArrayList* _namespaces;
};

struct _GXmlTElementClass {
	GXmlTNodeClass parent_class;
};

struct _GXmlTNamespace {
	GXmlTNode parent_instance;
	GXmlTNamespacePrivate * priv;
};

struct _GXmlTNamespaceClass {
	GXmlTNodeClass parent_class;
};

struct _GXmlTNodeTChildrenList {
	GeeAbstractBidirList parent_instance;
	GXmlTNodeTChildrenListPrivate * priv;
};

struct _GXmlTNodeTChildrenListClass {
	GeeAbstractBidirListClass parent_class;
};

struct _GXmlTProcessingInstruction {
	GXmlTNode parent_instance;
	GXmlTProcessingInstructionPrivate * priv;
};

struct _GXmlTProcessingInstructionClass {
	GXmlTNodeClass parent_class;
};

struct _GXmlTText {
	GXmlTNode parent_instance;
	GXmlTTextPrivate * priv;
};

struct _GXmlTTextClass {
	GXmlTNodeClass parent_class;
};

struct _GXmlGNode {
	GObject parent_instance;
	GXmlGNodePrivate * priv;
	GXmlGDocument* _doc;
	xmlNode* _node;
	gchar* _base_uri;
};

struct _GXmlGNodeClass {
	GObjectClass parent_class;
	gboolean (*set_namespace) (GXmlGNode* self, const gchar* uri, const gchar* prefix);
	gchar* (*to_string) (GXmlGNode* self);
	GeeMap* (*get_attrs) (GXmlGNode* self);
	GeeBidirList* (*get_children_nodes) (GXmlGNode* self);
	GeeList* (*get_namespaces) (GXmlGNode* self);
	GXmlDocument* (*get_document) (GXmlGNode* self);
	GXmlNode* (*get_parent) (GXmlGNode* self);
	GXmlNodeType (*get_type_node) (GXmlGNode* self);
	gchar* (*get_name) (GXmlGNode* self);
	gchar* (*get_value) (GXmlGNode* self);
	void (*set_value) (GXmlGNode* self, const gchar* value);
};

typedef enum  {
	GXML_XPATH_OBJECT_TYPE_UNDEFINED,
	GXML_XPATH_OBJECT_TYPE_NODESET,
	GXML_XPATH_OBJECT_TYPE_BOOLEAN,
	GXML_XPATH_OBJECT_TYPE_NUMBER,
	GXML_XPATH_OBJECT_TYPE_STRING,
	GXML_XPATH_OBJECT_TYPE_POINT,
	GXML_XPATH_OBJECT_TYPE_RANGE,
	GXML_XPATH_OBJECT_TYPE_LOCATIONSET,
	GXML_XPATH_OBJECT_TYPE_USERS,
	GXML_XPATH_OBJECT_TYPE_XSLT_TREE
} GXmlXPathObjectType;

struct _GXmlXPathObjectIface {
	GTypeInterface parent_iface;
	GXmlXPathObjectType (*get_object_type) (GXmlXPathObject* self);
	gboolean (*get_boolean_value) (GXmlXPathObject* self);
	const gchar* (*get_string_value) (GXmlXPathObject* self);
	gdouble (*get_number_value) (GXmlXPathObject* self);
	GXmlDomHTMLCollection* (*get_nodeset) (GXmlXPathObject* self);
};

typedef enum  {
	GXML_XPATH_ERROR_EXPRESSION_OK,
	GXML_XPATH_ERROR_NUMBER_ERROR,
	GXML_XPATH_ERROR_UNFINISHED_LITERAL_ERROR,
	GXML_XPATH_ERROR_START_LITERAL_ERROR,
	GXML_XPATH_ERROR_VARIABLE_REF_ERROR,
	GXML_XPATH_ERROR_UNDEF_VARIABLE_ERROR,
	GXML_XPATH_ERROR_INVALID_PREDICATE_ERROR,
	GXML_XPATH_ERROR_EXPR_ERROR,
	GXML_XPATH_ERROR_UNCLOSED_ERROR,
	GXML_XPATH_ERROR_UNKNOWN_FUNC_ERROR,
	GXML_XPATH_ERROR_INVALID_OPERAND,
	GXML_XPATH_ERROR_INVALID_TYPE,
	GXML_XPATH_ERROR_INVALID_ARITY,
	GXML_XPATH_ERROR_INVALID_CTXT_SIZE,
	GXML_XPATH_ERROR_INVALID_CTXT_POSITION,
	GXML_XPATH_ERROR_MEMORY_ERROR,
	GXML_XPATH_ERROR_XPTR_SYNTAX_ERROR,
	GXML_XPATH_ERROR_XPTR_RESOURCE_ERROR,
	GXML_XPATH_ERROR_XPTR_SUB_RESOURCE_ERROR,
	GXML_XPATH_ERROR_UNDEF_PREFIX_ERROR,
	GXML_XPATH_ERROR_ENCODING_ERROR,
	GXML_XPATH_ERROR_INVALID_CHAR_ERROR,
	GXML_XPATH_ERROR_INVALID_CTXT
} GXmlXPathError;
#define GXML_XPATH_ERROR gxml_xpath_error_quark ()
struct _GXmlXPathContextIface {
	GTypeInterface parent_iface;
	GXmlXPathObject* (*evaluate) (GXmlXPathContext* self, const gchar* expression, GeeList* resolver, GError** error);
};

struct _GXmlGDocument {
	GXmlGNode parent_instance;
	GXmlGDocumentPrivate * priv;
	xmlDoc* doc;
	xmlBuffer* _buffer;
	GXmlGImplementation* _implementation;
	gchar* _url;
	gchar* _origin;
	gchar* _compat_mode;
	gchar* _character_set;
	gchar* _content_type;
	GXmlDomEvent* _constructor;
};

struct _GXmlGDocumentClass {
	GXmlGNodeClass parent_class;
	gboolean (*save) (GXmlGDocument* self, GCancellable* cancellable, GError** error);
	gboolean (*save_as) (GXmlGDocument* self, GFile* f, GCancellable* cancellable, GError** error);
};

struct _GXmlHtmlDocument {
	GXmlGDocument parent_instance;
	GXmlHtmlDocumentPrivate * priv;
};

struct _GXmlHtmlDocumentClass {
	GXmlGDocumentClass parent_class;
};

struct _GXmlDomAttrIface {
	GTypeInterface parent_iface;
	gchar* (*get_namespace_uri) (GXmlDomAttr* self);
	gchar* (*get_prefix) (GXmlDomAttr* self);
	gchar* (*get_local_name) (GXmlDomAttr* self);
	gchar* (*get_name) (GXmlDomAttr* self);
	gchar* (*get_value) (GXmlDomAttr* self);
	void (*set_value) (GXmlDomAttr* self, const gchar* value);
	gboolean (*get_specified) (GXmlDomAttr* self);
};

struct _GXmlGAttribute {
	GXmlGNode parent_instance;
	GXmlGAttributePrivate * priv;
};

struct _GXmlGAttributeClass {
	GXmlGNodeClass parent_class;
};

struct _GXmlGChildNode {
	GXmlGNode parent_instance;
	GXmlGChildNodePrivate * priv;
};

struct _GXmlGChildNodeClass {
	GXmlGNodeClass parent_class;
};

struct _GXmlGNonDocumentChildNode {
	GXmlGChildNode parent_instance;
	GXmlGNonDocumentChildNodePrivate * priv;
};

struct _GXmlGNonDocumentChildNodeClass {
	GXmlGChildNodeClass parent_class;
};

struct _GXmlGCharacterData {
	GXmlGNonDocumentChildNode parent_instance;
	GXmlGCharacterDataPrivate * priv;
};

struct _GXmlGCharacterDataClass {
	GXmlGNonDocumentChildNodeClass parent_class;
};

struct _GXmlGComment {
	GXmlGCharacterData parent_instance;
	GXmlGCommentPrivate * priv;
};

struct _GXmlGCommentClass {
	GXmlGCharacterDataClass parent_class;
};

struct _GXmlGCDATA {
	GXmlGNode parent_instance;
	GXmlGCDATAPrivate * priv;
};

struct _GXmlGCDATAClass {
	GXmlGNodeClass parent_class;
};

struct _GXmlGImplementation {
	GObject parent_instance;
	GXmlGImplementationPrivate * priv;
};

struct _GXmlGImplementationClass {
	GObjectClass parent_class;
};

struct _GXmlGDocumentType {
	GXmlGChildNode parent_instance;
	GXmlGDocumentTypePrivate * priv;
	gchar* _name;
	gchar* _public_id;
	gchar* _system_id;
};

struct _GXmlGDocumentTypeClass {
	GXmlGChildNodeClass parent_class;
};

struct _GXmlGDocumentFragment {
	GXmlGDocument parent_instance;
	GXmlGDocumentFragmentPrivate * priv;
};

struct _GXmlGDocumentFragmentClass {
	GXmlGDocumentClass parent_class;
};

struct _GXmlGDomNodeIterator {
	GObject parent_instance;
	GXmlGDomNodeIteratorPrivate * priv;
	GXmlDomNode* _root;
	GXmlDomNode* _reference_node;
	gboolean _pointer_before_reference_node;
	gint _what_to_show;
	GXmlDomNodeFilter* _filter;
};

struct _GXmlGDomNodeIteratorClass {
	GObjectClass parent_class;
};

struct _GXmlGDomTreeWalker {
	GObject parent_instance;
	GXmlGDomTreeWalkerPrivate * priv;
	GXmlDomNode* _root;
	gint _what_to_show;
	GXmlDomNodeFilter* _filter;
	GXmlDomNode* _current_node;
};

struct _GXmlGDomTreeWalkerClass {
	GObjectClass parent_class;
};

struct _GXmlGDomTokenList {
	GeeArrayList parent_instance;
	GXmlGDomTokenListPrivate * priv;
	GXmlDomElement* _element;
	gchar* _attr;
};

struct _GXmlGDomTokenListClass {
	GeeArrayListClass parent_class;
};

struct _GXmlDomSettableTokenListIface {
	GTypeInterface parent_iface;
	gchar* (*get_value) (GXmlDomSettableTokenList* self);
	void (*set_value) (GXmlDomSettableTokenList* self, const gchar* value);
};

struct _GXmlGDomSettableTokenList {
	GXmlGDomTokenList parent_instance;
	GXmlGDomSettableTokenListPrivate * priv;
};

struct _GXmlGDomSettableTokenListClass {
	GXmlGDomTokenListClass parent_class;
};

struct _GXmlGDomHTMLCollection {
	GeeArrayList parent_instance;
	GXmlGDomHTMLCollectionPrivate * priv;
};

struct _GXmlGDomHTMLCollectionClass {
	GeeArrayListClass parent_class;
};

typedef enum  {
	GXML_DOM_EVENT_FLAGS_STOP_PROPAGATION_FLAG = 1 << 0,
	GXML_DOM_EVENT_FLAGS_STOP_IMMEDIATE_PROPAGATION_FLAG = 1 << 1,
	GXML_DOM_EVENT_FLAGS_CANCELED_FLAG = 1 << 2,
	GXML_DOM_EVENT_FLAGS_INITIALIZED_FLAG = 1 << 3,
	GXML_DOM_EVENT_FLAGS_DISPATCH_FLAG = 1 << 4
} GXmlDomEventFlags;

struct _GXmlGDomEvent {
	GObject parent_instance;
	GXmlGDomEventPrivate * priv;
	gchar* _etype;
	GXmlDomEventTarget* _event_target;
	GXmlDomEventTarget* _current_target;
	gboolean _bubbles;
	gboolean _cancelable;
	gboolean _is_trusted;
	GXmlDomTimeStamp* _time_stamp;
	gboolean _default_prevented;
	GXmlDomEventPhase _event_phase;
	GXmlDomEventFlags _flags;
};

struct _GXmlGDomEventClass {
	GObjectClass parent_class;
};

struct _GXmlGDomCustomEvent {
	GXmlGDomEvent parent_instance;
	GXmlGDomCustomEventPrivate * priv;
	GValue* _detail;
};

struct _GXmlGDomCustomEventClass {
	GXmlGDomEventClass parent_class;
};

struct _GXmlGDomRange {
	GObject parent_instance;
	GXmlGDomRangePrivate * priv;
	GXmlDomDocument* _document;
	GXmlDomNode* _start_container;
	gint _start_offset;
	GXmlDomNode* _end_container;
	gint _end_offset;
	gboolean _collapse;
	GXmlDomNode* _common_ancestor_container;
};

struct _GXmlGDomRangeClass {
	GObjectClass parent_class;
};

struct _GXmlGElement {
	GXmlGNonDocumentChildNode parent_instance;
	GXmlGElementPrivate * priv;
};

struct _GXmlGElementClass {
	GXmlGNonDocumentChildNodeClass parent_class;
};

struct _GXmlGNamespace {
	GObject parent_instance;
	GXmlGNamespacePrivate * priv;
};

struct _GXmlGNamespaceClass {
	GObjectClass parent_class;
};

typedef enum  {
	GXML_ERROR_NOT_SUPPORTED,
	GXML_ERROR_PARSER,
	GXML_ERROR_WRITER
} GXmlError;
#define GXML_ERROR gxml_error_quark ()
struct _GXmlGProcessingInstruction {
	GXmlGCharacterData parent_instance;
	GXmlGProcessingInstructionPrivate * priv;
};

struct _GXmlGProcessingInstructionClass {
	GXmlGCharacterDataClass parent_class;
};

struct _GXmlGText {
	GXmlGCharacterData parent_instance;
	GXmlGTextPrivate * priv;
};

struct _GXmlGTextClass {
	GXmlGCharacterDataClass parent_class;
};

struct _GXmlGHashMapAttr {
	GeeAbstractMap parent_instance;
	GXmlGHashMapAttrPrivate * priv;
};

struct _GXmlGHashMapAttrClass {
	GeeAbstractMapClass parent_class;
};

struct _GXmlGHashMapAttrEntry {
	GeeMapEntry parent_instance;
	GXmlGHashMapAttrEntryPrivate * priv;
};

struct _GXmlGHashMapAttrEntryClass {
	GeeMapEntryClass parent_class;
};

struct _GXmlGHashMapAttrIterator {
	GObject parent_instance;
	GXmlGHashMapAttrIteratorPrivate * priv;
};

struct _GXmlGHashMapAttrIteratorClass {
	GObjectClass parent_class;
};

struct _GXmlGListChildren {
	GeeAbstractBidirList parent_instance;
	GXmlGListChildrenPrivate * priv;
};

struct _GXmlGListChildrenClass {
	GeeAbstractBidirListClass parent_class;
};

struct _GXmlGListChildrenIterator {
	GObject parent_instance;
	GXmlGListChildrenIteratorPrivate * priv;
};

struct _GXmlGListChildrenIteratorClass {
	GObjectClass parent_class;
};

struct _GXmlGListNamespaces {
	GeeAbstractList parent_instance;
	GXmlGListNamespacesPrivate * priv;
};

struct _GXmlGListNamespacesClass {
	GeeAbstractListClass parent_class;
};

struct _GXmlGListNamespacesIterator {
	GObject parent_instance;
	GXmlGListNamespacesIteratorPrivate * priv;
};

struct _GXmlGListNamespacesIteratorClass {
	GObjectClass parent_class;
};

struct _GXmlDomNodeFilter {
	GObject parent_instance;
	GXmlDomNodeFilterPrivate * priv;
};

struct _GXmlDomNodeFilterClass {
	GObjectClass parent_class;
};

typedef gint (*GXmlDomNodeFilterAcceptNode) (GXmlNode* node, void* user_data);
struct _GXmlDomElementList {
	GeeArrayList parent_instance;
	GXmlDomElementListPrivate * priv;
};

struct _GXmlDomElementListClass {
	GeeArrayListClass parent_class;
};

struct _GXmlDomEventInit {
	GObject parent_instance;
	GXmlDomEventInitPrivate * priv;
};

struct _GXmlDomEventInitClass {
	GObjectClass parent_class;
};

struct _GXmlDomCustomEventIface {
	GTypeInterface parent_iface;
	void (*init_custom_event) (GXmlDomCustomEvent* self, const gchar* type, gboolean bubbles, gboolean cancelable, GValue* detail);
	GValue* (*get_detail) (GXmlDomCustomEvent* self);
};

struct _GXmlDomCustomEventInit {
	GXmlDomEventInit parent_instance;
	GXmlDomCustomEventInitPrivate * priv;
};

struct _GXmlDomCustomEventInitClass {
	GXmlDomEventInitClass parent_class;
};

struct _GXmlDomTimeStamp {
	GObject parent_instance;
	GXmlDomTimeStampPrivate * priv;
};

struct _GXmlDomTimeStampClass {
	GObjectClass parent_class;
};

struct _GXmlDomMutationRecordIface {
	GTypeInterface parent_iface;
	const gchar* (*get_mtype) (GXmlDomMutationRecord* self);
	GXmlDomNode* (*get_target) (GXmlDomMutationRecord* self);
	GXmlDomNodeList* (*get_added_nodes) (GXmlDomMutationRecord* self);
	void (*set_added_nodes) (GXmlDomMutationRecord* self, GXmlDomNodeList* value);
	GXmlDomNodeList* (*get_removed_nodes) (GXmlDomMutationRecord* self);
	void (*set_removed_nodes) (GXmlDomMutationRecord* self, GXmlDomNodeList* value);
	GXmlDomNode* (*get_previous_sibling) (GXmlDomMutationRecord* self);
	GXmlDomNode* (*get_next_sibling) (GXmlDomMutationRecord* self);
	const gchar* (*get_attribute_name) (GXmlDomMutationRecord* self);
	const gchar* (*get_attribute_namespace) (GXmlDomMutationRecord* self);
	const gchar* (*get_old_value) (GXmlDomMutationRecord* self);
};

struct _GXmlDomMutationObserverIface {
	GTypeInterface parent_iface;
	void (*observe) (GXmlDomMutationObserver* self, GXmlNode* target, GXmlDomMutationObserverInit* options);
	void (*disconnect) (GXmlDomMutationObserver* self);
	GeeList* (*take_records) (GXmlDomMutationObserver* self);
};

typedef void (*GXmlDomMutationCallback) (GeeList* mutations, GXmlDomMutationObserver* observer, void* user_data);
struct _GXmlDomMutationObserverInit {
	GObject parent_instance;
	GXmlDomMutationObserverInitPrivate * priv;
};

struct _GXmlDomMutationObserverInitClass {
	GObjectClass parent_class;
};

typedef enum  {
	GXML_DOM_ERROR_INDEX_SIZE_ERROR = 1,
	GXML_DOM_ERROR_DOMSTRING_SIZE_ERROR,
	GXML_DOM_ERROR_HIERARCHY_REQUEST_ERROR,
	GXML_DOM_ERROR_WRONG_DOCUMENT_ERROR,
	GXML_DOM_ERROR_INVALID_CHARACTER_ERROR,
	GXML_DOM_ERROR_NO_DATA_ALLOWED_ERROR,
	GXML_DOM_ERROR_NO_MODIFICATION_ALLOWED_ERROR,
	GXML_DOM_ERROR_NOT_FOUND_ERROR,
	GXML_DOM_ERROR_NOT_SUPPORTED_ERROR,
	GXML_DOM_ERROR_INUSE_ATTRIBUTE_ERROR,
	GXML_DOM_ERROR_INVALID_STATE_ERROR,
	GXML_DOM_ERROR_SYNTAX_ERROR,
	GXML_DOM_ERROR_INVALID_MODIFICATION_ERROR,
	GXML_DOM_ERROR_NAMESPACE_ERROR,
	GXML_DOM_ERROR_INVALID_ACCESS_ERROR,
	GXML_DOM_ERROR_VALIDATION_ERROR,
	GXML_DOM_ERROR_TYPE_MISMATCH_ERROR,
	GXML_DOM_ERROR_SECURITY_ERROR,
	GXML_DOM_ERROR_NETWORK_ERROR,
	GXML_DOM_ERROR_ABORT_ERROR,
	GXML_DOM_ERROR_URL_MISMATCH_ERROR,
	GXML_DOM_ERROR_QUOTA_EXCEEDED_ERROR,
	GXML_DOM_ERROR_TIME_OUT_ERROR,
	GXML_DOM_ERROR_INVALID_NODE_TYPE_ERROR,
	GXML_DOM_ERROR_DATA_CLONE_ERROR
} GXmlDomError;
#define GXML_DOM_ERROR gxml_dom_error_quark ()
struct _GXmlDomErrorName {
	GObject parent_instance;
	GXmlDomErrorNamePrivate * priv;
};

struct _GXmlDomErrorNameClass {
	GObjectClass parent_class;
};

struct _GXmlGXPathObject {
	GObject parent_instance;
	GXmlGXPathObjectPrivate * priv;
};

struct _GXmlGXPathObjectClass {
	GObjectClass parent_class;
};

struct _GXmlGomNode {
	GObject parent_instance;
	GXmlGomNodePrivate * priv;
	gchar* _local_name;
	gchar* _prefix;
	gchar* _base_uri;
	gchar* _node_value;
	GXmlDomNode* _parent;
	GXmlDomNodeNodeType _node_type;
	GXmlGomNodeList* _child_nodes;
	GXmlDomDocument* _document;
};

struct _GXmlGomNodeClass {
	GObjectClass parent_class;
};

struct _GXmlGomDocument {
	GXmlGomNode parent_instance;
	GXmlGomDocumentPrivate * priv;
	GXmlDomImplementation* _implementation;
	gchar* _url;
	gchar* _origin;
	gchar* _compat_mode;
	gchar* _character_set;
	gchar* _content_type;
	GXmlDomEvent* _constructor;
};

struct _GXmlGomDocumentClass {
	GXmlGomNodeClass parent_class;
};

struct _GXmlGomImplementation {
	GObject parent_instance;
	GXmlGomImplementationPrivate * priv;
};

struct _GXmlGomImplementationClass {
	GObjectClass parent_class;
};

struct _GXmlGomDocumentType {
	GXmlGomNode parent_instance;
	GXmlGomDocumentTypePrivate * priv;
	gchar* _name;
	gchar* _public_id;
	gchar* _system_id;
};

struct _GXmlGomDocumentTypeClass {
	GXmlGomNodeClass parent_class;
};

struct _GXmlGomDocumentFragment {
	GXmlGomNode parent_instance;
	GXmlGomDocumentFragmentPrivate * priv;
};

struct _GXmlGomDocumentFragmentClass {
	GXmlGomNodeClass parent_class;
};

struct _GXmlGomObjectIface {
	GTypeInterface parent_iface;
	GList* (*get_properties_list) (GXmlGomObject* self);
	GParamSpec* (*find_property_name) (GXmlGomObject* self, const gchar* pname);
	GParamSpec* (*find_object_property_name) (GXmlGomObject* self, const gchar* pname);
	GList* (*get_property_element_list) (GXmlGomObject* self);
	gchar* (*get_property_string) (GXmlGomObject* self, GParamSpec* prop);
	gchar* (*get_attribute) (GXmlGomObject* self, const gchar* name);
	gboolean (*set_attribute) (GXmlGomObject* self, const gchar* name, const gchar* val);
	GXmlDomElement* (*get_child) (GXmlGomObject* self, const gchar* name);
	GXmlDomElementList* (*find_elements) (GXmlGomObject* self, const gchar* name);
	gboolean (*remove_attribute) (GXmlGomObject* self, const gchar* name);
	gboolean (*set_instance_property) (GXmlGomObject* self, const gchar* name);
	void (*clean_property_elements) (GXmlGomObject* self, const gchar* name, GError** error);
};

struct _GXmlGomElement {
	GXmlGomNode parent_instance;
	GXmlGomElementPrivate * priv;
	GXmlGomElementAttributes* _attributes;
	gchar* _namespace_uri;
};

struct _GXmlGomElementClass {
	GXmlGomNodeClass parent_class;
};

struct _GXmlGomElementAttributes {
	GeeHashMap parent_instance;
	GXmlGomElementAttributesPrivate * priv;
	GXmlGomElement* _element;
};

struct _GXmlGomElementAttributesClass {
	GeeHashMapClass parent_class;
};

struct _GXmlGomAttr {
	GXmlGomNode parent_instance;
	GXmlGomAttrPrivate * priv;
	gchar* _namespace_uri;
};

struct _GXmlGomAttrClass {
	GXmlGomNodeClass parent_class;
};

struct _GXmlGomNodeList {
	GeeArrayList parent_instance;
	GXmlGomNodeListPrivate * priv;
};

struct _GXmlGomNodeListClass {
	GeeArrayListClass parent_class;
};

struct _GXmlGomCharacterData {
	GXmlGomNode parent_instance;
	GXmlGomCharacterDataPrivate * priv;
};

struct _GXmlGomCharacterDataClass {
	GXmlGomNodeClass parent_class;
};

struct _GXmlGomText {
	GXmlGomCharacterData parent_instance;
	GXmlGomTextPrivate * priv;
};

struct _GXmlGomTextClass {
	GXmlGomCharacterDataClass parent_class;
};

struct _GXmlGomProcessingInstruction {
	GXmlGomCharacterData parent_instance;
	GXmlGomProcessingInstructionPrivate * priv;
};

struct _GXmlGomProcessingInstructionClass {
	GXmlGomCharacterDataClass parent_class;
};

struct _GXmlGomComment {
	GXmlGomCharacterData parent_instance;
	GXmlGomCommentPrivate * priv;
};

struct _GXmlGomCommentClass {
	GXmlGomCharacterDataClass parent_class;
};

struct _GXmlGomCollectionIface {
	GTypeInterface parent_iface;
	void (*search) (GXmlGomCollection* self, GError** error);
	GXmlDomElement* (*get_item) (GXmlGomCollection* self, gint index, GError** error);
	void (*append) (GXmlGomCollection* self, GXmlDomElement* node, GError** error);
	void (*initialize) (GXmlGomCollection* self, GType t, GError** error);
	GXmlGomElement* (*create_item) (GXmlGomCollection* self);
	gboolean (*validate_append) (GXmlGomCollection* self, gint index, GXmlDomElement* element, GError** error);
	GQueue* (*get_nodes_index) (GXmlGomCollection* self);
	GXmlGomElement* (*get_element) (GXmlGomCollection* self);
	void (*set_element) (GXmlGomCollection* self, GXmlGomElement* value);
	const gchar* (*get_items_name) (GXmlGomCollection* self);
	GType (*get_items_type) (GXmlGomCollection* self);
	void (*set_items_type) (GXmlGomCollection* self, GType value);
	gint (*get_length) (GXmlGomCollection* self);
};

struct _GXmlBaseCollection {
	GObject parent_instance;
	GXmlBaseCollectionPrivate * priv;
	GQueue* _nodes_index;
	GXmlGomElement* _element;
	gchar* _items_name;
	GType _items_type;
};

struct _GXmlBaseCollectionClass {
	GObjectClass parent_class;
	gboolean (*validate_append) (GXmlBaseCollection* self, gint index, GXmlDomElement* element, GError** error);
};

struct _GXmlGomArrayList {
	GXmlBaseCollection parent_instance;
	GXmlGomArrayListPrivate * priv;
};

struct _GXmlGomArrayListClass {
	GXmlBaseCollectionClass parent_class;
};

struct _GXmlMappeableElementIface {
	GTypeInterface parent_iface;
	gchar* (*get_map_key) (GXmlMappeableElement* self);
};

struct _GXmlGomHashMap {
	GXmlBaseCollection parent_instance;
	GXmlGomHashMapPrivate * priv;
	GHashTable* _hashtable;
	gchar* _attribute_key;
};

struct _GXmlGomHashMapClass {
	GXmlBaseCollectionClass parent_class;
};

struct _GXmlMappeableElementPairKeyIface {
	GTypeInterface parent_iface;
	gchar* (*get_map_primary_key) (GXmlMappeableElementPairKey* self);
	gchar* (*get_map_secondary_key) (GXmlMappeableElementPairKey* self);
};

struct _GXmlGomHashPairedMap {
	GXmlBaseCollection parent_instance;
	GXmlGomHashPairedMapPrivate * priv;
	GHashTable* _hashtable;
	gchar* _attribute_primary_key;
	gchar* _attribute_secondary_key;
};

struct _GXmlGomHashPairedMapClass {
	GXmlBaseCollectionClass parent_class;
};

struct _GXmlGomPropertyIface {
	GTypeInterface parent_iface;
	gboolean (*validate_value) (GXmlGomProperty* self, const gchar* val);
	gchar* (*get_value) (GXmlGomProperty* self);
	void (*set_value) (GXmlGomProperty* self, const gchar* value);
};

struct _GXmlGomBaseProperty {
	GObject parent_instance;
	GXmlGomBasePropertyPrivate * priv;
};

struct _GXmlGomBasePropertyClass {
	GObjectClass parent_class;
	gboolean (*validate_value) (GXmlGomBaseProperty* self, const gchar* val);
	gchar* (*get_value) (GXmlGomBaseProperty* self);
	void (*set_value) (GXmlGomBaseProperty* self, const gchar* value);
};

struct _GXmlGomString {
	GXmlGomBaseProperty parent_instance;
	GXmlGomStringPrivate * priv;
	gchar* _value;
};

struct _GXmlGomStringClass {
	GXmlGomBasePropertyClass parent_class;
};

struct _GXmlGomArrayString {
	GXmlGomBaseProperty parent_instance;
	GXmlGomArrayStringPrivate * priv;
	gchar* _value;
	gchar** _values;
	gint _values_length1;
};

struct _GXmlGomArrayStringClass {
	GXmlGomBasePropertyClass parent_class;
};

struct _GXmlGomXsdArrayString {
	GXmlGomArrayString parent_instance;
	GXmlGomXsdArrayStringPrivate * priv;
	GFile* _source;
	gchar* _simple_type;
};

struct _GXmlGomXsdArrayStringClass {
	GXmlGomArrayStringClass parent_class;
};

struct _GXmlGomDouble {
	GXmlGomBaseProperty parent_instance;
	GXmlGomDoublePrivate * priv;
	gdouble _value;
};

struct _GXmlGomDoubleClass {
	GXmlGomBasePropertyClass parent_class;
};

struct _GXmlGomFloat {
	GXmlGomDouble parent_instance;
	GXmlGomFloatPrivate * priv;
};

struct _GXmlGomFloatClass {
	GXmlGomDoubleClass parent_class;
};

struct _GXmlGomInt {
	GXmlGomBaseProperty parent_instance;
	GXmlGomIntPrivate * priv;
	gint _value;
};

struct _GXmlGomIntClass {
	GXmlGomBasePropertyClass parent_class;
};

struct _GXmlGomBoolean {
	GXmlGomBaseProperty parent_instance;
	GXmlGomBooleanPrivate * priv;
	gboolean _value;
};

struct _GXmlGomBooleanClass {
	GXmlGomBasePropertyClass parent_class;
};

struct _GXmlGomEnum {
	GXmlGomBaseProperty parent_instance;
	GXmlGomEnumPrivate * priv;
	gint _value;
	GType _enum_type;
};

struct _GXmlGomEnumClass {
	GXmlGomBasePropertyClass parent_class;
};

struct _GXmlGomDate {
	GXmlGomBaseProperty parent_instance;
	GXmlGomDatePrivate * priv;
	GDate _value;
};

struct _GXmlGomDateClass {
	GXmlGomBasePropertyClass parent_class;
};

struct _GXmlGomDateTime {
	GXmlGomBaseProperty parent_instance;
	GXmlGomDateTimePrivate * priv;
	GDateTime* _value;
};

struct _GXmlGomDateTimeClass {
	GXmlGomBasePropertyClass parent_class;
};

typedef enum  {
	GXML_PARSER_ERROR_INVALID_DATA_ERROR,
	GXML_PARSER_ERROR_INVALID_FILE_ERROR,
	GXML_PARSER_ERROR_INVALID_STREAM_ERROR
} GXmlParserError;
#define GXML_PARSER_ERROR gxml_parser_error_quark ()
struct _GXmlParserIface {
	GTypeInterface parent_iface;
	void (*write_file) (GXmlParser* self, GFile* file, GCancellable* cancellable, GError** error);
	gchar* (*write_string) (GXmlParser* self, GError** error);
	void (*write_stream) (GXmlParser* self, GOutputStream* stream, GCancellable* cancellable, GError** error);
	void (*read_file) (GXmlParser* self, GFile* file, GCancellable* cancellable, GError** error);
	void (*read_stream) (GXmlParser* self, GInputStream* stream, GCancellable* cancellable, GError** error);
	void (*read_string) (GXmlParser* self, const gchar* str, GCancellable* cancellable, GError** error);
	gboolean (*get_backup) (GXmlParser* self);
	void (*set_backup) (GXmlParser* self, gboolean value);
	gboolean (*get_indent) (GXmlParser* self);
	void (*set_indent) (GXmlParser* self, gboolean value);
	GXmlDomNode* (*get_node) (GXmlParser* self);
};

struct _GXmlXParser {
	GObject parent_instance;
	GXmlXParserPrivate * priv;
};

struct _GXmlXParserClass {
	GObjectClass parent_class;
};

struct _GXmlIXsdListIface {
	GTypeInterface parent_iface;
	GXmlDomElement* (*get_item) (GXmlIXsdList* self, gint index);
	void (*append) (GXmlIXsdList* self, GXmlDomElement* element);
	void (*remove) (GXmlIXsdList* self, gint index);
	gint (*index_of) (GXmlIXsdList* self, GXmlDomElement* element);
	GXmlDomElement* (*get_element) (GXmlIXsdList* self);
	void (*set_element) (GXmlIXsdList* self, GXmlDomElement* value);
	GType (*get_items_type) (GXmlIXsdList* self);
	void (*set_items_type) (GXmlIXsdList* self, GType value);
	GType (*get_items_name) (GXmlIXsdList* self);
	void (*set_items_name) (GXmlIXsdList* self, GType value);
	gint (*get_length) (GXmlIXsdList* self);
};

struct _GXmlIXsdListElementsIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdListSimpleTypesIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdListComplexTypesIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdSchemaIface {
	GTypeInterface parent_iface;
	GXmlIXsdListElements* (*get_element_definitions) (GXmlIXsdSchema* self);
	void (*set_element_definitions) (GXmlIXsdSchema* self, GXmlIXsdListElements* value);
	GXmlIXsdListSimpleTypes* (*get_simple_type_definitions) (GXmlIXsdSchema* self);
	void (*set_simple_type_definitions) (GXmlIXsdSchema* self, GXmlIXsdListSimpleTypes* value);
	GXmlIXsdListComplexTypes* (*get_complex_type_definitions) (GXmlIXsdSchema* self);
	void (*set_complex_type_definitions) (GXmlIXsdSchema* self, GXmlIXsdListComplexTypes* value);
};

typedef enum  {
	GXML_IXSD_SCHEMA_ERROR_INVALIDATION_ERROR
} GXmlIXsdSchemaError;
#define GXML_IXSD_SCHEMA_ERROR gxml_ixsd_schema_error_quark ()
struct _GXmlIXsdAnnotationIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdBaseTypeIface {
	GTypeInterface parent_iface;
	GXmlIXsdAnnotation* (*get_anotation) (GXmlIXsdBaseType* self);
	void (*set_anotation) (GXmlIXsdBaseType* self, GXmlIXsdAnnotation* value);
};

struct _GXmlIXsdTypeDefIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdTypeListIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdTypeUnionIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdListTypeRestrictionEnumerationsIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdListTypeRestrictionWhiteSpacesIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdTypeRestrictionIface {
	GTypeInterface parent_iface;
	const gchar* (*get_base) (GXmlIXsdTypeRestriction* self);
	void (*set_base) (GXmlIXsdTypeRestriction* self, const gchar* value);
	const gchar* (*get_id) (GXmlIXsdTypeRestriction* self);
	void (*set_id) (GXmlIXsdTypeRestriction* self, const gchar* value);
	GXmlIXsdSimpleType* (*get_simple_type) (GXmlIXsdTypeRestriction* self);
	void (*set_simple_type) (GXmlIXsdTypeRestriction* self, GXmlIXsdSimpleType* value);
	GXmlIXsdListTypeRestrictionEnumerations* (*get_enumerations) (GXmlIXsdTypeRestriction* self);
	void (*set_enumerations) (GXmlIXsdTypeRestriction* self, GXmlIXsdListTypeRestrictionEnumerations* value);
	GXmlIXsdListTypeRestrictionWhiteSpaces* (*get_white_spaces) (GXmlIXsdTypeRestriction* self);
	void (*set_white_spaces) (GXmlIXsdTypeRestriction* self, GXmlIXsdListTypeRestrictionWhiteSpaces* value);
};

struct _GXmlIXsdSimpleTypeIface {
	GTypeInterface parent_iface;
	const gchar* (*get_final) (GXmlIXsdSimpleType* self);
	void (*set_final) (GXmlIXsdSimpleType* self, const gchar* value);
	const gchar* (*get_id) (GXmlIXsdSimpleType* self);
	void (*set_id) (GXmlIXsdSimpleType* self, const gchar* value);
	const gchar* (*get_name) (GXmlIXsdSimpleType* self);
	void (*set_name) (GXmlIXsdSimpleType* self, const gchar* value);
	GXmlIXsdAnnotation* (*get_annotation) (GXmlIXsdSimpleType* self);
	void (*set_annotation) (GXmlIXsdSimpleType* self, GXmlIXsdAnnotation* value);
	GXmlIXsdTypeList* (*get_list) (GXmlIXsdSimpleType* self);
	void (*set_list) (GXmlIXsdSimpleType* self, GXmlIXsdTypeList* value);
	GXmlIXsdTypeUnion* (*get_union) (GXmlIXsdSimpleType* self);
	void (*set_union) (GXmlIXsdSimpleType* self, GXmlIXsdTypeUnion* value);
	GXmlIXsdTypeRestriction* (*get_restriction) (GXmlIXsdSimpleType* self);
	void (*set_restriction) (GXmlIXsdSimpleType* self, GXmlIXsdTypeRestriction* value);
};

struct _GXmlIXsdTypeRestrictionDefIface {
	GTypeInterface parent_iface;
	GXmlIXsdAnnotation* (*get_annotation) (GXmlIXsdTypeRestrictionDef* self);
	void (*set_annotation) (GXmlIXsdTypeRestrictionDef* self, GXmlIXsdAnnotation* value);
};

struct _GXmlIXsdTypeRestrictionMinExclusiveIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdTypeRestrictionMinInclusiveIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdTypeRestrictionMaxExclusiveIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdTypeRestrictionMaxInclusiveIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdTypeRestrictionTotalDigitsIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdTypeRestrictionFractionDigitsIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdTypeRestrictionLengthIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdTypeRestrictionMinLengthIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdTypeRestrictionMaxLengthIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdTypeRestrictionEnumerationIface {
	GTypeInterface parent_iface;
	const gchar* (*get_id) (GXmlIXsdTypeRestrictionEnumeration* self);
	void (*set_id) (GXmlIXsdTypeRestrictionEnumeration* self, const gchar* value);
	const gchar* (*get_value) (GXmlIXsdTypeRestrictionEnumeration* self);
	void (*set_value) (GXmlIXsdTypeRestrictionEnumeration* self, const gchar* value);
};

struct _GXmlIXsdTypeRestrictionWhiteSpaceIface {
	GTypeInterface parent_iface;
	gboolean (*get_fixed) (GXmlIXsdTypeRestrictionWhiteSpace* self);
	void (*set_fixed) (GXmlIXsdTypeRestrictionWhiteSpace* self, gboolean value);
	const gchar* (*get_id) (GXmlIXsdTypeRestrictionWhiteSpace* self);
	void (*set_id) (GXmlIXsdTypeRestrictionWhiteSpace* self, const gchar* value);
	const gchar* (*get_value) (GXmlIXsdTypeRestrictionWhiteSpace* self);
	void (*set_value) (GXmlIXsdTypeRestrictionWhiteSpace* self, const gchar* value);
};

struct _GXmlIXsdTypeRestrictionPatternIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdTypeRestrictionAssertionIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdTypeRestrictionExplicitTimezoneIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdBaseContentIface {
	GTypeInterface parent_iface;
	GXmlIXsdAnnotation* (*get_anotation) (GXmlIXsdBaseContent* self);
	void (*set_anotation) (GXmlIXsdBaseContent* self, GXmlIXsdAnnotation* value);
};

struct _GXmlIXsdListAttributesIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdListAttributesGroupIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdComplexTypeIface {
	GTypeInterface parent_iface;
	gboolean (*get_abstract) (GXmlIXsdComplexType* self);
	void (*set_abstract) (GXmlIXsdComplexType* self, gboolean value);
	const gchar* (*get_block) (GXmlIXsdComplexType* self);
	void (*set_block) (GXmlIXsdComplexType* self, const gchar* value);
	const gchar* (*get_final) (GXmlIXsdComplexType* self);
	void (*set_final) (GXmlIXsdComplexType* self, const gchar* value);
	gboolean (*get_mixed) (GXmlIXsdComplexType* self);
	void (*set_mixed) (GXmlIXsdComplexType* self, gboolean value);
	const gchar* (*get_name) (GXmlIXsdComplexType* self);
	void (*set_name) (GXmlIXsdComplexType* self, const gchar* value);
	gboolean (*get_default_attributes_apply) (GXmlIXsdComplexType* self);
	void (*set_default_attributes_apply) (GXmlIXsdComplexType* self, gboolean value);
	GXmlIXsdBaseContent* (*get_content_type) (GXmlIXsdComplexType* self);
	void (*set_content_type) (GXmlIXsdComplexType* self, GXmlIXsdBaseContent* value);
	GXmlIXsdListAttributes* (*get_type_attributes) (GXmlIXsdComplexType* self);
	GXmlIXsdListAttributesGroup* (*get_group_attributes) (GXmlIXsdComplexType* self);
};

struct _GXmlIXsdExtensionIface {
	GTypeInterface parent_iface;
	const gchar* (*get_base) (GXmlIXsdExtension* self);
	void (*set_base) (GXmlIXsdExtension* self, const gchar* value);
};

struct _GXmlIXsdElementIface {
	GTypeInterface parent_iface;
	gboolean (*get_abstract) (GXmlIXsdElement* self);
	void (*set_abstract) (GXmlIXsdElement* self, gboolean value);
	const gchar* (*get_block) (GXmlIXsdElement* self);
	void (*set_block) (GXmlIXsdElement* self, const gchar* value);
	const gchar* (*get_default) (GXmlIXsdElement* self);
	void (*set_default) (GXmlIXsdElement* self, const gchar* value);
	const gchar* (*get_final) (GXmlIXsdElement* self);
	void (*set_final) (GXmlIXsdElement* self, const gchar* value);
	const gchar* (*get_fixed) (GXmlIXsdElement* self);
	void (*set_fixed) (GXmlIXsdElement* self, const gchar* value);
	const gchar* (*get_form) (GXmlIXsdElement* self);
	void (*set_form) (GXmlIXsdElement* self, const gchar* value);
	const gchar* (*get_id) (GXmlIXsdElement* self);
	void (*set_id) (GXmlIXsdElement* self, const gchar* value);
	const gchar* (*get_maxOccurs) (GXmlIXsdElement* self);
	void (*set_maxOccurs) (GXmlIXsdElement* self, const gchar* value);
	const gchar* (*get_minOccurs) (GXmlIXsdElement* self);
	void (*set_minOccurs) (GXmlIXsdElement* self, const gchar* value);
	const gchar* (*get_name) (GXmlIXsdElement* self);
	void (*set_name) (GXmlIXsdElement* self, const gchar* value);
	gboolean (*get_nillable) (GXmlIXsdElement* self);
	void (*set_nillable) (GXmlIXsdElement* self, gboolean value);
	const gchar* (*get_ref) (GXmlIXsdElement* self);
	void (*set_ref) (GXmlIXsdElement* self, const gchar* value);
	GXmlDomTokenList* (*get_substitution_group) (GXmlIXsdElement* self);
	void (*set_substitution_group) (GXmlIXsdElement* self, GXmlDomTokenList* value);
	const gchar* (*get_target_namespace) (GXmlIXsdElement* self);
	void (*set_target_namespace) (GXmlIXsdElement* self, const gchar* value);
	const gchar* (*get_object_type) (GXmlIXsdElement* self);
	void (*set_object_type) (GXmlIXsdElement* self, const gchar* value);
	GXmlIXsdAnnotation* (*get_anotation) (GXmlIXsdElement* self);
	void (*set_anotation) (GXmlIXsdElement* self, GXmlIXsdAnnotation* value);
	GXmlIXsdSimpleType* (*get_simple_type) (GXmlIXsdElement* self);
	void (*set_simple_type) (GXmlIXsdElement* self, GXmlIXsdSimpleType* value);
	GXmlIXsdComplexType* (*get_complex_type) (GXmlIXsdElement* self);
	void (*set_complex_type) (GXmlIXsdElement* self, GXmlIXsdComplexType* value);
};

struct _GXmlIXsdSimpleContentIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdComplexContentIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdOpenContentIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdBaseAttributeIface {
	GTypeInterface parent_iface;
	GXmlIXsdAnnotation* (*get_anotation) (GXmlIXsdBaseAttribute* self);
	void (*set_anotation) (GXmlIXsdBaseAttribute* self, GXmlIXsdAnnotation* value);
};

struct _GXmlIXsdAttributeIface {
	GTypeInterface parent_iface;
};

struct _GXmlIXsdAttributeGroupIface {
	GTypeInterface parent_iface;
};

struct _GXmlGomXsdSchema {
	GXmlGomElement parent_instance;
	GXmlGomXsdSchemaPrivate * priv;
};

struct _GXmlGomXsdSchemaClass {
	GXmlGomElementClass parent_class;
};

struct _GXmlGomXsdSimpleType {
	GXmlGomElement parent_instance;
	GXmlGomXsdSimpleTypePrivate * priv;
};

struct _GXmlGomXsdSimpleTypeClass {
	GXmlGomElementClass parent_class;
};

struct _GXmlGomXsdTypeDefinition {
	GXmlGomElement parent_instance;
	GXmlGomXsdTypeDefinitionPrivate * priv;
};

struct _GXmlGomXsdTypeDefinitionClass {
	GXmlGomElementClass parent_class;
};

struct _GXmlGomXsdTypeList {
	GXmlGomXsdTypeDefinition parent_instance;
	GXmlGomXsdTypeListPrivate * priv;
};

struct _GXmlGomXsdTypeListClass {
	GXmlGomXsdTypeDefinitionClass parent_class;
};

struct _GXmlGomXsdTypeUnion {
	GXmlGomXsdTypeDefinition parent_instance;
	GXmlGomXsdTypeUnionPrivate * priv;
};

struct _GXmlGomXsdTypeUnionClass {
	GXmlGomXsdTypeDefinitionClass parent_class;
};

struct _GXmlGomXsdTypeRestriction {
	GXmlGomXsdTypeDefinition parent_instance;
	GXmlGomXsdTypeRestrictionPrivate * priv;
};

struct _GXmlGomXsdTypeRestrictionClass {
	GXmlGomXsdTypeDefinitionClass parent_class;
};

struct _GXmlGomXsdTypeRestrictionDef {
	GXmlGomElement parent_instance;
	GXmlGomXsdTypeRestrictionDefPrivate * priv;
};

struct _GXmlGomXsdTypeRestrictionDefClass {
	GXmlGomElementClass parent_class;
};

struct _GXmlGomXsdTypeRestrictionMinExclusive {
	GXmlGomXsdTypeRestrictionDef parent_instance;
	GXmlGomXsdTypeRestrictionMinExclusivePrivate * priv;
};

struct _GXmlGomXsdTypeRestrictionMinExclusiveClass {
	GXmlGomXsdTypeRestrictionDefClass parent_class;
};

struct _GXmlGomXsdTypeRestrictionMinInclusive {
	GXmlGomXsdTypeRestrictionDef parent_instance;
	GXmlGomXsdTypeRestrictionMinInclusivePrivate * priv;
};

struct _GXmlGomXsdTypeRestrictionMinInclusiveClass {
	GXmlGomXsdTypeRestrictionDefClass parent_class;
};

struct _GXmlGomXsdTypeRestrictionMaxExclusive {
	GXmlGomXsdTypeRestrictionDef parent_instance;
	GXmlGomXsdTypeRestrictionMaxExclusivePrivate * priv;
};

struct _GXmlGomXsdTypeRestrictionMaxExclusiveClass {
	GXmlGomXsdTypeRestrictionDefClass parent_class;
};

struct _GXmlGomXsdTypeRestrictionMaxInclusive {
	GXmlGomXsdTypeRestrictionDef parent_instance;
	GXmlGomXsdTypeRestrictionMaxInclusivePrivate * priv;
};

struct _GXmlGomXsdTypeRestrictionMaxInclusiveClass {
	GXmlGomXsdTypeRestrictionDefClass parent_class;
};

struct _GXmlGomXsdTypeRestrictionTotalDigits {
	GXmlGomXsdTypeRestrictionDef parent_instance;
	GXmlGomXsdTypeRestrictionTotalDigitsPrivate * priv;
};

struct _GXmlGomXsdTypeRestrictionTotalDigitsClass {
	GXmlGomXsdTypeRestrictionDefClass parent_class;
};

struct _GXmlGomXsdTypeRestrictionFractionDigits {
	GXmlGomXsdTypeRestrictionDef parent_instance;
	GXmlGomXsdTypeRestrictionFractionDigitsPrivate * priv;
};

struct _GXmlGomXsdTypeRestrictionFractionDigitsClass {
	GXmlGomXsdTypeRestrictionDefClass parent_class;
};

struct _GXmlGomXsdTypeRestrictionLength {
	GXmlGomXsdTypeRestrictionDef parent_instance;
	GXmlGomXsdTypeRestrictionLengthPrivate * priv;
};

struct _GXmlGomXsdTypeRestrictionLengthClass {
	GXmlGomXsdTypeRestrictionDefClass parent_class;
};

struct _GXmlGomXsdTypeRestrictionMinLength {
	GXmlGomXsdTypeRestrictionDef parent_instance;
	GXmlGomXsdTypeRestrictionMinLengthPrivate * priv;
};

struct _GXmlGomXsdTypeRestrictionMinLengthClass {
	GXmlGomXsdTypeRestrictionDefClass parent_class;
};

struct _GXmlGomXsdTypeRestrictionMaxLength {
	GXmlGomXsdTypeRestrictionDef parent_instance;
	GXmlGomXsdTypeRestrictionMaxLengthPrivate * priv;
};

struct _GXmlGomXsdTypeRestrictionMaxLengthClass {
	GXmlGomXsdTypeRestrictionDefClass parent_class;
};

struct _GXmlGomXsdTypeRestrictionEnumeration {
	GXmlGomXsdTypeRestrictionDef parent_instance;
	GXmlGomXsdTypeRestrictionEnumerationPrivate * priv;
};

struct _GXmlGomXsdTypeRestrictionEnumerationClass {
	GXmlGomXsdTypeRestrictionDefClass parent_class;
};

struct _GXmlGomXsdTypeRestrictionWhiteSpace {
	GXmlGomXsdTypeRestrictionDef parent_instance;
	GXmlGomXsdTypeRestrictionWhiteSpacePrivate * priv;
};

struct _GXmlGomXsdTypeRestrictionWhiteSpaceClass {
	GXmlGomXsdTypeRestrictionDefClass parent_class;
};

struct _GXmlGomXsdTypeRestrictionWhiteSpaceFixed {
	GXmlGomBoolean parent_instance;
	GXmlGomXsdTypeRestrictionWhiteSpaceFixedPrivate * priv;
};

struct _GXmlGomXsdTypeRestrictionWhiteSpaceFixedClass {
	GXmlGomBooleanClass parent_class;
};

struct _GXmlGomXsdTypeRestrictionPattern {
	GXmlGomXsdTypeRestrictionDef parent_instance;
	GXmlGomXsdTypeRestrictionPatternPrivate * priv;
};

struct _GXmlGomXsdTypeRestrictionPatternClass {
	GXmlGomXsdTypeRestrictionDefClass parent_class;
};

struct _GXmlGomXsdTypeRestrictionAssertion {
	GXmlGomXsdTypeRestrictionDef parent_instance;
	GXmlGomXsdTypeRestrictionAssertionPrivate * priv;
};

struct _GXmlGomXsdTypeRestrictionAssertionClass {
	GXmlGomXsdTypeRestrictionDefClass parent_class;
};

struct _GXmlGomXsdTypeRestrictionExplicitTimezone {
	GXmlGomXsdTypeRestrictionDef parent_instance;
	GXmlGomXsdTypeRestrictionExplicitTimezonePrivate * priv;
};

struct _GXmlGomXsdTypeRestrictionExplicitTimezoneClass {
	GXmlGomXsdTypeRestrictionDefClass parent_class;
};

struct _GXmlGomXsdBaseType {
	GXmlGomElement parent_instance;
	GXmlGomXsdBaseTypePrivate * priv;
};

struct _GXmlGomXsdBaseTypeClass {
	GXmlGomElementClass parent_class;
};

struct _GXmlGomXsdComplexType {
	GXmlGomXsdBaseType parent_instance;
	GXmlGomXsdComplexTypePrivate * priv;
	GXmlGomXsdList* _type_attributes;
	GXmlGomXsdList* _group_attributes;
};

struct _GXmlGomXsdComplexTypeClass {
	GXmlGomXsdBaseTypeClass parent_class;
};

struct _GXmlGomXsdExtension {
	GXmlGomElement parent_instance;
	GXmlGomXsdExtensionPrivate * priv;
};

struct _GXmlGomXsdExtensionClass {
	GXmlGomElementClass parent_class;
};

struct _GXmlGomXsdElement {
	GXmlGomElement parent_instance;
	GXmlGomXsdElementPrivate * priv;
};

struct _GXmlGomXsdElementClass {
	GXmlGomElementClass parent_class;
};

struct _GXmlGomXsdAnnotation {
	GXmlGomElement parent_instance;
	GXmlGomXsdAnnotationPrivate * priv;
};

struct _GXmlGomXsdAnnotationClass {
	GXmlGomElementClass parent_class;
};

struct _GXmlGomXsdBaseContent {
	GXmlGomElement parent_instance;
	GXmlGomXsdBaseContentPrivate * priv;
};

struct _GXmlGomXsdBaseContentClass {
	GXmlGomElementClass parent_class;
};

struct _GXmlGomXsdSimpleContent {
	GXmlGomXsdBaseContent parent_instance;
	GXmlGomXsdSimpleContentPrivate * priv;
};

struct _GXmlGomXsdSimpleContentClass {
	GXmlGomXsdBaseContentClass parent_class;
};

struct _GXmlGomXsdComplexContent {
	GXmlGomXsdBaseContent parent_instance;
	GXmlGomXsdComplexContentPrivate * priv;
};

struct _GXmlGomXsdComplexContentClass {
	GXmlGomXsdBaseContentClass parent_class;
};

struct _GXmlGomXsdOpenContent {
	GXmlGomXsdBaseContent parent_instance;
	GXmlGomXsdOpenContentPrivate * priv;
};

struct _GXmlGomXsdOpenContentClass {
	GXmlGomXsdBaseContentClass parent_class;
};

struct _GXmlGomXsdBaseAttribute {
	GXmlGomElement parent_instance;
	GXmlGomXsdBaseAttributePrivate * priv;
};

struct _GXmlGomXsdBaseAttributeClass {
	GXmlGomElementClass parent_class;
};

struct _GXmlGomXsdAttribute {
	GXmlGomXsdBaseAttribute parent_instance;
	GXmlGomXsdAttributePrivate * priv;
};

struct _GXmlGomXsdAttributeClass {
	GXmlGomXsdBaseAttributeClass parent_class;
};

struct _GXmlGomXsdAttributeGroup {
	GXmlGomXsdBaseAttribute parent_instance;
	GXmlGomXsdAttributeGroupPrivate * priv;
};

struct _GXmlGomXsdAttributeGroupClass {
	GXmlGomXsdBaseAttributeClass parent_class;
};

struct _GXmlGomXsdList {
	GXmlGomArrayList parent_instance;
	GXmlGomXsdListPrivate * priv;
};

struct _GXmlGomXsdListClass {
	GXmlGomArrayListClass parent_class;
};

struct _GXmlGomXsdListElements {
	GXmlGomXsdList parent_instance;
	GXmlGomXsdListElementsPrivate * priv;
};

struct _GXmlGomXsdListElementsClass {
	GXmlGomXsdListClass parent_class;
};

struct _GXmlGomXsdListSimpleTypes {
	GXmlGomXsdList parent_instance;
	GXmlGomXsdListSimpleTypesPrivate * priv;
};

struct _GXmlGomXsdListSimpleTypesClass {
	GXmlGomXsdListClass parent_class;
};

struct _GXmlGomXsdListComplexTypes {
	GXmlGomXsdList parent_instance;
	GXmlGomXsdListComplexTypesPrivate * priv;
};

struct _GXmlGomXsdListComplexTypesClass {
	GXmlGomXsdListClass parent_class;
};

struct _GXmlGomXsdListTypeRestrictionEnumerations {
	GXmlGomXsdList parent_instance;
	GXmlGomXsdListTypeRestrictionEnumerationsPrivate * priv;
};

struct _GXmlGomXsdListTypeRestrictionEnumerationsClass {
	GXmlGomXsdListClass parent_class;
};

struct _GXmlGomXsdListTypeRestrictionWhiteSpaces {
	GXmlGomXsdList parent_instance;
	GXmlGomXsdListTypeRestrictionWhiteSpacesPrivate * priv;
};

struct _GXmlGomXsdListTypeRestrictionWhiteSpacesClass {
	GXmlGomXsdListClass parent_class;
};


GType gxml_element_list_get_type (void) G_GNUC_CONST;
GType gxml_namespace_get_type (void) G_GNUC_CONST;
GType gxml_node_type_get_type (void) G_GNUC_CONST;
GType gxml_document_get_type (void) G_GNUC_CONST;
GType gxml_node_get_type (void) G_GNUC_CONST;
GType gxml_attribute_get_type (void) G_GNUC_CONST;
GXmlNamespace* gxml_attribute_get_namespace (GXmlAttribute* self);
void gxml_attribute_set_namespace (GXmlAttribute* self, GXmlNamespace* value);
gchar* gxml_attribute_get_prefix (GXmlAttribute* self);
GType gxml_comment_get_type (void) G_GNUC_CONST;
gchar* gxml_comment_get_str (GXmlComment* self);
void gxml_comment_set_str (GXmlComment* self, const gchar* value);
GType gxml_cdata_get_type (void) G_GNUC_CONST;
gchar* gxml_cdata_get_str (GXmlCDATA* self);
GType gxml_character_get_type (void) G_GNUC_CONST;
gchar* gxml_character_get_str (GXmlCharacter* self);
void gxml_character_set_str (GXmlCharacter* self, const gchar* value);
GQuark gxml_document_error_quark (void);
GXmlNode* gxml_document_create_element (GXmlDocument* self, const gchar* name, GError** error);
GXmlNode* gxml_document_create_text (GXmlDocument* self, const gchar* text);
GXmlNode* gxml_document_create_comment (GXmlDocument* self, const gchar* text);
GXmlNode* gxml_document_create_cdata (GXmlDocument* self, const gchar* text);
GXmlNode* gxml_document_create_pi (GXmlDocument* self, const gchar* target, const gchar* data);
gboolean gxml_document_save (GXmlDocument* self, GCancellable* cancellable, GError** error);
gboolean gxml_document_save_as (GXmlDocument* self, GFile* f, GCancellable* cancellable, GError** error);
GXmlDocument* gxml_document_new_default (void);
GXmlDocument* gxml_document_new_default_for_path (const gchar* path, GError** error);
GXmlDocument* gxml_document_new_default_for_file (GFile* f, GError** error);
gboolean gxml_document_get_indent (GXmlDocument* self);
void gxml_document_set_indent (GXmlDocument* self, gboolean value);
gboolean gxml_document_get_ns_top (GXmlDocument* self);
void gxml_document_set_ns_top (GXmlDocument* self, gboolean value);
gboolean gxml_document_get_prefix_default_ns (GXmlDocument* self);
void gxml_document_set_prefix_default_ns (GXmlDocument* self, gboolean value);
gboolean gxml_document_get_backup (GXmlDocument* self);
void gxml_document_set_backup (GXmlDocument* self, gboolean value);
GXmlNode* gxml_document_get_root (GXmlDocument* self);
GFile* gxml_document_get_file (GXmlDocument* self);
void gxml_document_set_file (GXmlDocument* self, GFile* value);
GType gxml_document_type_get_type (void) G_GNUC_CONST;
GType gxml_element_get_type (void) G_GNUC_CONST;
void gxml_element_normalize (GXmlElement* self);
void gxml_element_set_attr (GXmlElement* self, const gchar* name, const gchar* value);
GXmlNode* gxml_element_get_attr (GXmlElement* self, const gchar* name);
void gxml_element_remove_attr (GXmlElement* self, const gchar* name);
void gxml_element_remove_ns_attr (GXmlElement* self, const gchar* name, const gchar* uri);
void gxml_element_set_ns_attr (GXmlElement* self, const gchar* ns, const gchar* name, const gchar* value);
GXmlNode* gxml_element_get_ns_attr (GXmlElement* self, const gchar* name, const gchar* uri);
gchar* gxml_element_get_tag_name (GXmlElement* self);
gchar* gxml_element_get_content (GXmlElement* self);
void gxml_element_set_content (GXmlElement* self, const gchar* value);
GType gxml_dom_time_stamp_get_type (void) G_GNUC_CONST;
GType gxml_dom_event_phase_get_type (void) G_GNUC_CONST;
GType gxml_dom_event_get_type (void) G_GNUC_CONST;
GType gxml_dom_event_listener_get_type (void) G_GNUC_CONST;
GType gxml_dom_event_target_get_type (void) G_GNUC_CONST;
GType gxml_dom_node_document_position_get_type (void) G_GNUC_CONST;
GType gxml_dom_node_node_type_get_type (void) G_GNUC_CONST;
GType gxml_dom_node_list_get_type (void) G_GNUC_CONST;
GType gxml_dom_element_list_get_type (void) G_GNUC_CONST;
GType gxml_dom_parent_node_get_type (void) G_GNUC_CONST;
GType gxml_dom_non_element_parent_node_get_type (void) G_GNUC_CONST;
GType gxml_dom_document_fragment_get_type (void) G_GNUC_CONST;
GType gxml_dom_non_document_type_child_node_get_type (void) G_GNUC_CONST;
GType gxml_dom_child_node_get_type (void) G_GNUC_CONST;
GType gxml_dom_character_data_get_type (void) G_GNUC_CONST;
GType gxml_dom_text_get_type (void) G_GNUC_CONST;
GType gxml_dom_comment_get_type (void) G_GNUC_CONST;
GType gxml_dom_processing_instruction_get_type (void) G_GNUC_CONST;
GType gxml_dom_range_boundary_points_get_type (void) G_GNUC_CONST;
GType gxml_dom_range_get_type (void) G_GNUC_CONST;
GType gxml_dom_node_filter_get_type (void) G_GNUC_CONST;
GType gxml_dom_node_iterator_get_type (void) G_GNUC_CONST;
GType gxml_dom_tree_walker_get_type (void) G_GNUC_CONST;
GType gxml_dom_document_type_get_type (void) G_GNUC_CONST;
GType gxml_dom_xml_document_get_type (void) G_GNUC_CONST;
GType gxml_dom_implementation_get_type (void) G_GNUC_CONST;
GType gxml_dom_document_get_type (void) G_GNUC_CONST;
GType gxml_dom_node_get_type (void) G_GNUC_CONST;
GType gxml_dom_token_list_get_type (void) G_GNUC_CONST;
GType gxml_dom_named_node_map_get_type (void) G_GNUC_CONST;
GType gxml_dom_element_get_type (void) G_GNUC_CONST;
GType gxml_dom_html_collection_get_type (void) G_GNUC_CONST;
GXmlElementList* gxml_element_list_new (void);
GXmlElementList* gxml_element_list_construct (GType object_type);
gchar* gxml_namespace_get_uri (GXmlNamespace* self);
gchar* gxml_namespace_get_prefix (GXmlNamespace* self);
GXmlNode* gxml_node_get (GXmlNode* self, const gchar* key);
GXmlElementList* gxml_node_get_elements_by_property_value (GXmlNode* self, const gchar* property, const gchar* value);
GXmlElementList* gxml_node_get_elements_by_name (GXmlNode* self, const gchar* name);
GXmlElementList* gxml_node_get_elements_by_name_ns (GXmlNode* self, const gchar* name, const gchar* ns);
gchar* gxml_node_to_string (GXmlNode* self);
gboolean gxml_node_set_namespace (GXmlNode* self, const gchar* uri, const gchar* prefix);
gchar* gxml_node_ns_prefix (GXmlNode* self);
gchar* gxml_node_ns_uri (GXmlNode* self);
gboolean gxml_node_copy (GXmlDocument* doc, GXmlNode* node, GXmlNode* source, gboolean deep);
GeeList* gxml_node_get_namespaces (GXmlNode* self);
GeeBidirList* gxml_node_get_children_nodes (GXmlNode* self);
GeeMap* gxml_node_get_attrs (GXmlNode* self);
gchar* gxml_node_get_name (GXmlNode* self);
gchar* gxml_node_get_value (GXmlNode* self);
void gxml_node_set_value (GXmlNode* self, const gchar* value);
GXmlNodeType gxml_node_get_type_node (GXmlNode* self);
GXmlDocument* gxml_node_get_document (GXmlNode* self);
GXmlNode* gxml_node_get_parent (GXmlNode* self);
GType gxml_node_list_get_type (void) G_GNUC_CONST;
GXmlNode* gxml_node_list_get (GXmlNodeList* self, gint index);
GXmlNode** gxml_node_list_to_array (GXmlNodeList* self, int* result_length1);
GXmlNodeList* gxml_node_list_new (void);
GXmlNodeList* gxml_node_list_construct (GType object_type);
GType gxml_notation_get_type (void) G_GNUC_CONST;
const gchar* gxml_notation_get_public_id (GXmlNotation* self);
const gchar* gxml_notation_get_external_id (GXmlNotation* self);
GType gxml_processing_instruction_get_type (void) G_GNUC_CONST;
gchar* gxml_processing_instruction_get_target (GXmlProcessingInstruction* self);
gchar* gxml_processing_instruction_get_data (GXmlProcessingInstruction* self);
void gxml_processing_instruction_set_data (GXmlProcessingInstruction* self, const gchar* value);
GType gxml_text_get_type (void) G_GNUC_CONST;
gchar* gxml_text_get_str (GXmlText* self);
void gxml_text_set_str (GXmlText* self, const gchar* value);
GType gxml_serializable_get_type (void) G_GNUC_CONST;
gboolean gxml_serializable_get_enable_unknown_serializable_property (GXmlSerializable* self);
gboolean gxml_serializable_set_default_namespace (GXmlSerializable* self, GXmlNode* node);
gboolean gxml_serializable_serialize_use_xml_node_value (GXmlSerializable* self);
gchar* gxml_serializable_node_name (GXmlSerializable* self);
gboolean gxml_serializable_property_use_nick (GXmlSerializable* self);
GXmlNode* gxml_serializable_serialize (GXmlSerializable* self, GXmlNode* node, GError** error);
GXmlNode* gxml_serializable_serialize_property (GXmlSerializable* self, GXmlNode* element, GParamSpec* prop, GError** error);
gboolean gxml_serializable_deserialize (GXmlSerializable* self, GXmlNode* node, GError** error);
gboolean gxml_serializable_deserialize_property (GXmlSerializable* self, GXmlNode* property_node, GError** error);
GParamSpec* gxml_serializable_find_property_spec (GXmlSerializable* self, const gchar* property_name);
GParamSpec* gxml_serializable_default_find_property_spec (GXmlSerializable* self, const gchar* property_name);
GParamSpec** gxml_serializable_list_serializable_properties (GXmlSerializable* self, int* result_length1);
GParamSpec** gxml_serializable_default_list_serializable_properties (GXmlSerializable* self, int* result_length1);
GQuark gxml_serializable_error_quark (void);
gboolean gxml_serializable_string_to_gvalue (const gchar* str, GValue* dest, GError** error);
gchar* gxml_serializable_gvalue_to_string (GValue* val, GError** error);
GeeMap* gxml_serializable_get_unknown_serializable_properties (GXmlSerializable* self);
GeeCollection* gxml_serializable_get_unknown_serializable_nodes (GXmlSerializable* self);
gchar* gxml_serializable_get_serialized_xml_node_value (GXmlSerializable* self);
void gxml_serializable_set_serialized_xml_node_value (GXmlSerializable* self, const gchar* value);
GType gxml_serializable_property_get_type (void) G_GNUC_CONST;
gchar* gxml_serializable_property_get_serializable_property_value (GXmlSerializableProperty* self);
void gxml_serializable_property_set_serializable_property_value (GXmlSerializableProperty* self, const gchar* val);
gboolean gxml_serializable_property_deserialize_property (GXmlSerializableProperty* self, GXmlNode* property_node, GParamSpec* prop, gboolean nick, GError** error);
GXmlNode* gxml_serializable_property_serialize_property (GXmlSerializableProperty* self, GXmlNode* property_node, GParamSpec* prop, gboolean nick, GError** error);
GXmlNode* gxml_serializable_property_default_serializable_property_serialize_property (GXmlSerializableProperty* self, GXmlNode* element, GParamSpec* prop, gboolean nick, GError** error);
gboolean gxml_serializable_property_default_serializable_property_deserialize_property (GXmlSerializableProperty* self, GXmlNode* property_node, GParamSpec* prop, gboolean nick, GError** error);
GType gxml_serializable_bool_get_type (void) G_GNUC_CONST;
gboolean gxml_serializable_bool_get_value (GXmlSerializableBool* self);
void gxml_serializable_bool_set_value (GXmlSerializableBool* self, gboolean val);
gchar* gxml_serializable_bool_to_string (GXmlSerializableBool* self);
GXmlSerializableBool* gxml_serializable_bool_new (void);
GXmlSerializableBool* gxml_serializable_bool_construct (GType object_type);
GType gxml_serializable_double_get_type (void) G_GNUC_CONST;
gint gxml_serializable_double_get_fraction (GXmlSerializableDouble* self);
void gxml_serializable_double_set_fraction (GXmlSerializableDouble* self, gint fraction);
gdouble gxml_serializable_double_get_value (GXmlSerializableDouble* self);
void gxml_serializable_double_set_value (GXmlSerializableDouble* self, gdouble val);
gchar* gxml_serializable_double_to_string (GXmlSerializableDouble* self);
gchar* gxml_serializable_double_format (GXmlSerializableDouble* self, const gchar* f);
GXmlSerializableDouble* gxml_serializable_double_new (void);
GXmlSerializableDouble* gxml_serializable_double_construct (GType object_type);
GType gxml_serializable_float_get_type (void) G_GNUC_CONST;
gfloat gxml_serializable_float_get_value (GXmlSerializableFloat* self);
void gxml_serializable_float_set_value (GXmlSerializableFloat* self, gfloat val);
GXmlSerializableFloat* gxml_serializable_float_new (void);
GXmlSerializableFloat* gxml_serializable_float_construct (GType object_type);
GQuark gxml_serializable_enum_error_quark (void);
GType gxml_serializable_enum_get_type (void) G_GNUC_CONST;
GXmlSerializableEnum* gxml_serializable_enum_new_with_enum (GType type);
GXmlSerializableEnum* gxml_serializable_enum_construct_with_enum (GType object_type, GType type);
void gxml_serializable_enum_set_enum_type (GXmlSerializableEnum* self, GType type);
GType gxml_serializable_enum_get_enum_type (GXmlSerializableEnum* self);
void gxml_serializable_enum_parse (GXmlSerializableEnum* self, const gchar* str, GError** error);
void gxml_serializable_enum_parse_integer (GXmlSerializableEnum* self, gint v, GError** error);
gint gxml_serializable_enum_to_integer (GXmlSerializableEnum* self, GError** error);
gchar* gxml_serializable_enum_to_string (GXmlSerializableEnum* self);
GXmlSerializableEnum* gxml_serializable_enum_new (void);
GXmlSerializableEnum* gxml_serializable_enum_construct (GType object_type);
GType gxml_serializable_int_get_type (void) G_GNUC_CONST;
gint gxml_serializable_int_get_value (GXmlSerializableInt* self);
void gxml_serializable_int_set_value (GXmlSerializableInt* self, gint val);
GXmlSerializableInt* gxml_serializable_int_new (void);
GXmlSerializableInt* gxml_serializable_int_construct (GType object_type);
GType gxml_serializable_value_list_get_type (void) G_GNUC_CONST;
GeeList* gxml_serializable_value_list_get_values (GXmlSerializableValueList* self);
void gxml_serializable_value_list_add_values (GXmlSerializableValueList* self, gchar** vals, int vals_length1);
gchar* gxml_serializable_value_list_get_value_at (GXmlSerializableValueList* self, gint index);
void gxml_serializable_value_list_select_value_at (GXmlSerializableValueList* self, gint index);
gchar** gxml_serializable_value_list_get_values_array (GXmlSerializableValueList* self, int* result_length1);
gboolean gxml_serializable_value_list_is_value (GXmlSerializableValueList* self);
gchar* gxml_serializable_value_list_to_string (GXmlSerializableValueList* self);
GXmlSerializableValueList* gxml_serializable_value_list_new (void);
GXmlSerializableValueList* gxml_serializable_value_list_construct (GType object_type);
gpointer gxml_enumeration_ref (gpointer instance);
void gxml_enumeration_unref (gpointer instance);
GParamSpec* gxml_param_spec_enumeration (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void gxml_value_set_enumeration (GValue* value, gpointer v_object);
void gxml_value_take_enumeration (GValue* value, gpointer v_object);
gpointer gxml_value_get_enumeration (const GValue* value);
GType gxml_enumeration_get_type (void) G_GNUC_CONST;
gchar* gxml_enumeration_get_nick (GType enumeration, gint val, GError** error);
gchar* gxml_enumeration_get_nick_camelcase (GType enumeration, gint val, GError** error);
gchar* gxml_enumeration_get_string (GType enumeration, gint val, gboolean use_nick, gboolean camelcase, GError** error);
GEnumValue* gxml_enumeration_parse (GType enumeration, const gchar* val, GError** error);
GEnumValue* gxml_enumeration_to_array (GType enumeration, int* result_length1);
GEnumValue* gxml_enumeration_parse_integer (GType enumeration, gint val);
gchar** gxml_enumeration_to_string_array (GType enumeration, int* result_length1, GError** error);
GXmlEnumeration* gxml_enumeration_new (void);
GXmlEnumeration* gxml_enumeration_construct (GType object_type);
GQuark gxml_enumeration_error_quark (void);
GType gxml_serializable_object_model_get_type (void) G_GNUC_CONST;
gboolean gxml_serializable_object_model_get_enable_unknown_serializable_property (GXmlSerializableObjectModel* self);
gboolean gxml_serializable_object_model_serialize_use_xml_node_value (GXmlSerializableObjectModel* self);
gboolean gxml_serializable_object_model_property_use_nick (GXmlSerializableObjectModel* self);
gboolean gxml_serializable_object_model_set_default_namespace (GXmlSerializableObjectModel* self, GXmlNode* node);
gchar* gxml_serializable_object_model_node_name (GXmlSerializableObjectModel* self);
gchar* gxml_serializable_object_model_default_node_name (GXmlSerializableObjectModel* self);
GParamSpec* gxml_serializable_object_model_find_property_spec (GXmlSerializableObjectModel* self, const gchar* property_name);
GParamSpec** gxml_serializable_object_model_list_serializable_properties (GXmlSerializableObjectModel* self, int* result_length1);
GXmlNode* gxml_serializable_object_model_serialize (GXmlSerializableObjectModel* self, GXmlNode* node, GError** error);
GXmlNode* gxml_serializable_object_model_default_serialize (GXmlSerializableObjectModel* self, GXmlNode* node, GError** error);
GXmlNode* gxml_serializable_object_model_serialize_property (GXmlSerializableObjectModel* self, GXmlNode* element, GParamSpec* prop, GError** error);
GXmlNode* gxml_serializable_object_model_default_serialize_property (GXmlSerializableObjectModel* self, GXmlElement* element, GParamSpec* prop, GError** error);
gboolean gxml_serializable_object_model_deserialize (GXmlSerializableObjectModel* self, GXmlNode* node, GError** error);
gboolean gxml_serializable_object_model_default_deserialize (GXmlSerializableObjectModel* self, GXmlNode* node, GError** error);
gboolean gxml_serializable_object_model_deserialize_property (GXmlSerializableObjectModel* self, GXmlNode* property_node, GError** error);
gboolean gxml_serializable_object_model_default_deserialize_property (GXmlSerializableObjectModel* self, GXmlNode* property_node, GError** error);
gchar* gxml_serializable_object_model_to_string (GXmlSerializableObjectModel* self);
GXmlSerializableObjectModel* gxml_serializable_object_model_construct (GType object_type);
GParamSpec** gxml_serializable_object_model_get_properties (GXmlSerializableObjectModel* self, int* result_length1);
void gxml_serializable_object_model_set_properties (GXmlSerializableObjectModel* self, GParamSpec** value, int value_length1);
GHashTable* gxml_serializable_object_model_get_ignored_serializable_properties (GXmlSerializableObjectModel* self);
void gxml_serializable_object_model_set_ignored_serializable_properties (GXmlSerializableObjectModel* self, GHashTable* value);
GType gxml_serializable_collection_get_type (void) G_GNUC_CONST;
GType gxml_serializable_tree_map_get_type (void) G_GNUC_CONST;
gboolean gxml_serializable_tree_map_deserialize_proceed (GXmlSerializableTreeMap* self);
gboolean gxml_serializable_tree_map_deserialized (GXmlSerializableTreeMap* self);
gboolean gxml_serializable_tree_map_deserialize_node (GXmlSerializableTreeMap* self, GXmlNode* node, GError** error);
gboolean gxml_serializable_tree_map_deserialize_children (GXmlSerializableTreeMap* self, GError** error);
gboolean gxml_serializable_tree_map_set_default_namespace (GXmlSerializableTreeMap* self, GXmlNode* node);
gboolean gxml_serializable_tree_map_serialize_use_xml_node_value (GXmlSerializableTreeMap* self);
gboolean gxml_serializable_tree_map_property_use_nick (GXmlSerializableTreeMap* self);
gchar* gxml_serializable_tree_map_node_name (GXmlSerializableTreeMap* self);
GParamSpec* gxml_serializable_tree_map_find_property_spec (GXmlSerializableTreeMap* self, const gchar* property_name);
GParamSpec** gxml_serializable_tree_map_list_serializable_properties (GXmlSerializableTreeMap* self, int* result_length1);
GXmlNode* gxml_serializable_tree_map_serialize (GXmlSerializableTreeMap* self, GXmlNode* node, GError** error);
GXmlNode* gxml_serializable_tree_map_default_serialize (GXmlSerializableTreeMap* self, GXmlNode* node, GError** error);
GXmlNode* gxml_serializable_tree_map_serialize_property (GXmlSerializableTreeMap* self, GXmlNode* element, GParamSpec* prop, GError** error);
GXmlNode* gxml_serializable_tree_map_default_serialize_property (GXmlSerializableTreeMap* self, GXmlNode* element, GParamSpec* prop, GError** error);
gboolean gxml_serializable_tree_map_deserialize (GXmlSerializableTreeMap* self, GXmlNode* node, GError** error);
gboolean gxml_serializable_tree_map_default_deserialize (GXmlSerializableTreeMap* self, GXmlNode* node, GError** error);
gboolean gxml_serializable_tree_map_deserialize_property (GXmlSerializableTreeMap* self, GXmlNode* property_node, GError** error);
gboolean gxml_serializable_tree_map_default_deserialize_property (GXmlSerializableTreeMap* self, GXmlNode* property_node, GError** error);
GXmlSerializableTreeMap* gxml_serializable_tree_map_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func);
GXmlSerializableTreeMap* gxml_serializable_tree_map_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func);
GParamSpec** gxml_serializable_tree_map_get_properties (GXmlSerializableTreeMap* self, int* result_length1);
void gxml_serializable_tree_map_set_properties (GXmlSerializableTreeMap* self, GParamSpec** value, int value_length1);
GHashTable* gxml_serializable_tree_map_get_ignored_serializable_properties (GXmlSerializableTreeMap* self);
void gxml_serializable_tree_map_set_ignored_serializable_properties (GXmlSerializableTreeMap* self, GHashTable* value);
GType gxml_serializable_hash_map_get_type (void) G_GNUC_CONST;
gboolean gxml_serializable_hash_map_deserialize_proceed (GXmlSerializableHashMap* self);
gboolean gxml_serializable_hash_map_deserialized (GXmlSerializableHashMap* self);
gboolean gxml_serializable_hash_map_deserialize_node (GXmlSerializableHashMap* self, GXmlNode* node, GError** error);
gboolean gxml_serializable_hash_map_deserialize_children (GXmlSerializableHashMap* self, GError** error);
gboolean gxml_serializable_hash_map_set_default_namespace (GXmlSerializableHashMap* self, GXmlNode* node);
gboolean gxml_serializable_hash_map_get_enable_unknown_serializable_property (GXmlSerializableHashMap* self);
gboolean gxml_serializable_hash_map_serialize_use_xml_node_value (GXmlSerializableHashMap* self);
gboolean gxml_serializable_hash_map_property_use_nick (GXmlSerializableHashMap* self);
gchar* gxml_serializable_hash_map_node_name (GXmlSerializableHashMap* self);
GParamSpec* gxml_serializable_hash_map_find_property_spec (GXmlSerializableHashMap* self, const gchar* property_name);
GParamSpec** gxml_serializable_hash_map_list_serializable_properties (GXmlSerializableHashMap* self, int* result_length1);
GXmlNode* gxml_serializable_hash_map_serialize (GXmlSerializableHashMap* self, GXmlNode* node, GError** error);
GXmlNode* gxml_serializable_hash_map_default_serialize (GXmlSerializableHashMap* self, GXmlNode* node, GError** error);
GXmlNode* gxml_serializable_hash_map_serialize_property (GXmlSerializableHashMap* self, GXmlNode* element, GParamSpec* prop, GError** error);
GXmlNode* gxml_serializable_hash_map_default_serialize_property (GXmlSerializableHashMap* self, GXmlNode* element, GParamSpec* prop, GError** error);
gboolean gxml_serializable_hash_map_deserialize (GXmlSerializableHashMap* self, GXmlNode* node, GError** error);
gboolean gxml_serializable_hash_map_default_deserialize (GXmlSerializableHashMap* self, GXmlNode* node, GError** error);
gboolean gxml_serializable_hash_map_deserialize_property (GXmlSerializableHashMap* self, GXmlNode* property_node, GError** error);
gboolean gxml_serializable_hash_map_default_deserialize_property (GXmlSerializableHashMap* self, GXmlNode* property_node, GError** error);
GXmlSerializableHashMap* gxml_serializable_hash_map_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func);
GXmlSerializableHashMap* gxml_serializable_hash_map_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func);
GParamSpec** gxml_serializable_hash_map_get_properties (GXmlSerializableHashMap* self, int* result_length1);
void gxml_serializable_hash_map_set_properties (GXmlSerializableHashMap* self, GParamSpec** value, int value_length1);
GHashTable* gxml_serializable_hash_map_get_ignored_serializable_properties (GXmlSerializableHashMap* self);
void gxml_serializable_hash_map_set_ignored_serializable_properties (GXmlSerializableHashMap* self, GHashTable* value);
GType gxml_serializable_map_key_get_type (void) G_GNUC_CONST;
gpointer gxml_serializable_map_key_get_map_key (GXmlSerializableMapKey* self);
GType gxml_serializable_dual_key_map_get_type (void) G_GNUC_CONST;
gboolean gxml_serializable_dual_key_map_deserialize_proceed (GXmlSerializableDualKeyMap* self);
gboolean gxml_serializable_dual_key_map_deserialized (GXmlSerializableDualKeyMap* self);
gboolean gxml_serializable_dual_key_map_deserialize_node (GXmlSerializableDualKeyMap* self, GXmlNode* node, GError** error);
gboolean gxml_serializable_dual_key_map_deserialize_children (GXmlSerializableDualKeyMap* self, GError** error);
GeeCollection* gxml_serializable_dual_key_map_secondary_keys (GXmlSerializableDualKeyMap* self, gconstpointer key);
GeeCollection* gxml_serializable_dual_key_map_values_for_key (GXmlSerializableDualKeyMap* self, gconstpointer primary_key);
GeeCollection* gxml_serializable_dual_key_map_values (GXmlSerializableDualKeyMap* self);
void gxml_serializable_dual_key_map_set (GXmlSerializableDualKeyMap* self, gconstpointer primary_key, gconstpointer secundary_key, gconstpointer val);
gpointer gxml_serializable_dual_key_map_get (GXmlSerializableDualKeyMap* self, gconstpointer primary_key, gconstpointer secondary_key);
gboolean gxml_serializable_dual_key_map_set_default_namespace (GXmlSerializableDualKeyMap* self, GXmlNode* node);
gboolean gxml_serializable_dual_key_map_get_enable_unknown_serializable_property (GXmlSerializableDualKeyMap* self);
gboolean gxml_serializable_dual_key_map_serialize_use_xml_node_value (GXmlSerializableDualKeyMap* self);
gboolean gxml_serializable_dual_key_map_property_use_nick (GXmlSerializableDualKeyMap* self);
gchar* gxml_serializable_dual_key_map_node_name (GXmlSerializableDualKeyMap* self);
GParamSpec* gxml_serializable_dual_key_map_find_property_spec (GXmlSerializableDualKeyMap* self, const gchar* property_name);
GParamSpec** gxml_serializable_dual_key_map_list_serializable_properties (GXmlSerializableDualKeyMap* self, int* result_length1);
GXmlNode* gxml_serializable_dual_key_map_serialize (GXmlSerializableDualKeyMap* self, GXmlNode* node, GError** error);
GXmlNode* gxml_serializable_dual_key_map_default_serialize (GXmlSerializableDualKeyMap* self, GXmlNode* node, GError** error);
GXmlNode* gxml_serializable_dual_key_map_serialize_property (GXmlSerializableDualKeyMap* self, GXmlNode* element, GParamSpec* prop, GError** error);
GXmlNode* gxml_serializable_dual_key_map_default_serialize_property (GXmlSerializableDualKeyMap* self, GXmlNode* element, GParamSpec* prop, GError** error);
gboolean gxml_serializable_dual_key_map_deserialize (GXmlSerializableDualKeyMap* self, GXmlNode* node, GError** error);
gboolean gxml_serializable_dual_key_map_default_deserialize (GXmlSerializableDualKeyMap* self, GXmlNode* node, GError** error);
gboolean gxml_serializable_dual_key_map_deserialize_property (GXmlSerializableDualKeyMap* self, GXmlNode* property_node, GError** error);
gboolean gxml_serializable_dual_key_map_default_deserialize_property (GXmlSerializableDualKeyMap* self, GXmlNode* property_node, GError** error);
GXmlSerializableDualKeyMap* gxml_serializable_dual_key_map_new (GType p_type, GBoxedCopyFunc p_dup_func, GDestroyNotify p_destroy_func, GType s_type, GBoxedCopyFunc s_dup_func, GDestroyNotify s_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func);
GXmlSerializableDualKeyMap* gxml_serializable_dual_key_map_construct (GType object_type, GType p_type, GBoxedCopyFunc p_dup_func, GDestroyNotify p_destroy_func, GType s_type, GBoxedCopyFunc s_dup_func, GDestroyNotify s_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func);
GType gxml_serializable_dual_key_map_get_value_type (GXmlSerializableDualKeyMap* self);
GType gxml_serializable_dual_key_map_get_primary_key_type (GXmlSerializableDualKeyMap* self);
GType gxml_serializable_dual_key_map_get_secondary_key_type (GXmlSerializableDualKeyMap* self);
GeeCollection* gxml_serializable_dual_key_map_get_primary_keys (GXmlSerializableDualKeyMap* self);
gint gxml_serializable_dual_key_map_get_size (GXmlSerializableDualKeyMap* self);
GParamSpec** gxml_serializable_dual_key_map_get_properties (GXmlSerializableDualKeyMap* self, int* result_length1);
void gxml_serializable_dual_key_map_set_properties (GXmlSerializableDualKeyMap* self, GParamSpec** value, int value_length1);
GHashTable* gxml_serializable_dual_key_map_get_ignored_serializable_properties (GXmlSerializableDualKeyMap* self);
void gxml_serializable_dual_key_map_set_ignored_serializable_properties (GXmlSerializableDualKeyMap* self, GHashTable* value);
GType gxml_serializable_map_dual_key_get_type (void) G_GNUC_CONST;
gpointer gxml_serializable_map_dual_key_get_map_primary_key (GXmlSerializableMapDualKey* self);
gpointer gxml_serializable_map_dual_key_get_map_secondary_key (GXmlSerializableMapDualKey* self);
GType gxml_serializable_array_list_get_type (void) G_GNUC_CONST;
gboolean gxml_serializable_array_list_deserialize_proceed (GXmlSerializableArrayList* self);
gboolean gxml_serializable_array_list_deserialized (GXmlSerializableArrayList* self);
gboolean gxml_serializable_array_list_deserialize_node (GXmlSerializableArrayList* self, GXmlNode* node, GError** error);
gboolean gxml_serializable_array_list_deserialize_children (GXmlSerializableArrayList* self, GError** error);
gboolean gxml_serializable_array_list_set_default_namespace (GXmlSerializableArrayList* self, GXmlNode* node);
gboolean gxml_serializable_array_list_serialize_use_xml_node_value (GXmlSerializableArrayList* self);
gboolean gxml_serializable_array_list_property_use_nick (GXmlSerializableArrayList* self);
gchar* gxml_serializable_array_list_node_name (GXmlSerializableArrayList* self);
GParamSpec* gxml_serializable_array_list_find_property_spec (GXmlSerializableArrayList* self, const gchar* property_name);
GParamSpec** gxml_serializable_array_list_list_serializable_properties (GXmlSerializableArrayList* self, int* result_length1);
GXmlNode* gxml_serializable_array_list_serialize (GXmlSerializableArrayList* self, GXmlNode* node, GError** error);
GXmlNode* gxml_serializable_array_list_default_serialize (GXmlSerializableArrayList* self, GXmlNode* node, GError** error);
GXmlNode* gxml_serializable_array_list_serialize_property (GXmlSerializableArrayList* self, GXmlNode* element, GParamSpec* prop, GError** error);
GXmlNode* gxml_serializable_array_list_default_serialize_property (GXmlSerializableArrayList* self, GXmlNode* element, GParamSpec* prop, GError** error);
gboolean gxml_serializable_array_list_deserialize (GXmlSerializableArrayList* self, GXmlNode* node, GError** error);
gboolean gxml_serializable_array_list_default_deserialize (GXmlSerializableArrayList* self, GXmlNode* node, GError** error);
gboolean gxml_serializable_array_list_deserialize_property (GXmlSerializableArrayList* self, GXmlNode* property_node, GError** error);
gboolean gxml_serializable_array_list_default_deserialize_property (GXmlSerializableArrayList* self, GXmlNode* property_node, GError** error);
GXmlSerializableArrayList* gxml_serializable_array_list_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
GXmlSerializableArrayList* gxml_serializable_array_list_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
GParamSpec** gxml_serializable_array_list_get_properties (GXmlSerializableArrayList* self, int* result_length1);
void gxml_serializable_array_list_set_properties (GXmlSerializableArrayList* self, GParamSpec** value, int value_length1);
GHashTable* gxml_serializable_array_list_get_ignored_serializable_properties (GXmlSerializableArrayList* self);
void gxml_serializable_array_list_set_ignored_serializable_properties (GXmlSerializableArrayList* self, GHashTable* value);
GType gxml_serializable_container_get_type (void) G_GNUC_CONST;
void gxml_serializable_container_init_containers (GXmlSerializableContainer* self);
GXmlSerializableContainer* gxml_serializable_container_construct (GType object_type);
gboolean gxml_serializable_collection_deserialize_proceed (GXmlSerializableCollection* self);
gboolean gxml_serializable_collection_deserialized (GXmlSerializableCollection* self);
gboolean gxml_serializable_collection_deserialize_node (GXmlSerializableCollection* self, GXmlNode* node, GError** error);
gboolean gxml_serializable_collection_deserialize_children (GXmlSerializableCollection* self, GError** error);
gboolean gxml_serializable_collection_is_collection (GXmlSerializableCollection* self);
GType gxml_tnode_get_type (void) G_GNUC_CONST;
GType gxml_tattribute_get_type (void) G_GNUC_CONST;
GXmlTAttribute* gxml_tattribute_new (GXmlDocument* d, const gchar* name, const gchar* value);
GXmlTAttribute* gxml_tattribute_construct (GType object_type, GXmlDocument* d, const gchar* name, const gchar* value);
GType gxml_tcomment_get_type (void) G_GNUC_CONST;
GXmlTComment* gxml_tcomment_new (GXmlDocument* doc, const gchar* text);
GXmlTComment* gxml_tcomment_construct (GType object_type, GXmlDocument* doc, const gchar* text);
GType gxml_tcdata_get_type (void) G_GNUC_CONST;
GXmlTCDATA* gxml_tcdata_new (GXmlDocument* d, const gchar* text);
GXmlTCDATA* gxml_tcdata_construct (GType object_type, GXmlDocument* d, const gchar* text);
GType gxml_tdocument_get_type (void) G_GNUC_CONST;
GType gxml_tdocument_read_type_get_type (void) G_GNUC_CONST;
GXmlTDocument* gxml_tdocument_new (void);
GXmlTDocument* gxml_tdocument_construct (GType object_type);
GXmlTDocument* gxml_tdocument_new_from_path (const gchar* path);
GXmlTDocument* gxml_tdocument_construct_from_path (GType object_type, const gchar* path);
GXmlTDocument* gxml_tdocument_new_from_uri (const gchar* uri);
GXmlTDocument* gxml_tdocument_construct_from_uri (GType object_type, const gchar* uri);
GXmlTDocument* gxml_tdocument_new_from_file (GFile* file);
GXmlTDocument* gxml_tdocument_construct_from_file (GType object_type, GFile* file);
GXmlTDocument* gxml_tdocument_new_from_stream (GInputStream* stream);
GXmlTDocument* gxml_tdocument_construct_from_stream (GType object_type, GInputStream* stream);
GXmlTDocument* gxml_tdocument_new_from_string (const gchar* str);
GXmlTDocument* gxml_tdocument_construct_from_string (GType object_type, const gchar* str);
GXmlTDocument* gxml_tdocument_new_from_path_with_readtype_func (const gchar* path, GXmlTDocumentReadTypeFunc func, void* func_target);
GXmlTDocument* gxml_tdocument_construct_from_path_with_readtype_func (GType object_type, const gchar* path, GXmlTDocumentReadTypeFunc func, void* func_target);
GXmlTDocument* gxml_tdocument_new_from_uri_with_readtype_func (const gchar* uri, GXmlTDocumentReadTypeFunc func, void* func_target);
GXmlTDocument* gxml_tdocument_construct_from_uri_with_readtype_func (GType object_type, const gchar* uri, GXmlTDocumentReadTypeFunc func, void* func_target);
GXmlTDocument* gxml_tdocument_new_from_file_with_readtype_func (GFile* file, GXmlTDocumentReadTypeFunc func, void* func_target);
GXmlTDocument* gxml_tdocument_construct_from_file_with_readtype_func (GType object_type, GFile* file, GXmlTDocumentReadTypeFunc func, void* func_target);
GXmlTDocument* gxml_tdocument_new_from_stream_with_readtype_func (GInputStream* stream, GXmlTDocumentReadTypeFunc func, void* func_target);
GXmlTDocument* gxml_tdocument_construct_from_stream_with_readtype_func (GType object_type, GInputStream* stream, GXmlTDocumentReadTypeFunc func, void* func_target);
GXmlTDocument* gxml_tdocument_new_from_string_with_readtype_func (const gchar* str, GXmlTDocumentReadTypeFunc func, void* func_target);
GXmlTDocument* gxml_tdocument_construct_from_string_with_readtype_func (GType object_type, const gchar* str, GXmlTDocumentReadTypeFunc func, void* func_target);
gboolean gxml_tdocument_save_to (GXmlTDocument* self, GFile* f, GCancellable* cancellable, GError** error) G_GNUC_DEPRECATED;
gboolean gxml_tdocument_tw_save_as (GXmlDocument* doc, GFile* f, GCancellable* cancellable, GError** error);
void gxml_tdocument_write_document (GXmlDocument* doc, xmlTextWriter* tw, GError** error);
void gxml_tdocument_start_node (GXmlDocument* doc, xmlTextWriter* tw, GXmlNode* node, gboolean root, GeeArrayList** declared_ns, GError** error);
void gxml_tdocument_read_doc (GXmlDocument* doc, GFile* file, GXmlTDocumentReadTypeFunc rtfunc, void* rtfunc_target, GError** error);
void gxml_tdocument_read_doc_stream (GXmlDocument* doc, GInputStream* istream, GXmlTDocumentReadTypeFunc rtfunc, void* rtfunc_target, GError** error);
GXmlTDocumentReadType gxml_tdocument_read_node (GXmlNode* node, xmlTextReader* tr, GXmlTDocumentReadTypeFunc rntfunc, void* rntfunc_target, GError** error);
GType gxml_telement_get_type (void) G_GNUC_CONST;
GType gxml_tnode_tchildren_list_get_type (void) G_GNUC_CONST;
GXmlTElement* gxml_telement_new (GXmlDocument* d, const gchar* name);
GXmlTElement* gxml_telement_construct (GType object_type, GXmlDocument* d, const gchar* name);
GType gxml_tnamespace_get_type (void) G_GNUC_CONST;
GXmlTNamespace* gxml_tnamespace_new (GXmlDocument* d, const gchar* uri, const gchar* prefix);
GXmlTNamespace* gxml_tnamespace_construct (GType object_type, GXmlDocument* d, const gchar* uri, const gchar* prefix);
gboolean gxml_tnode_set_namespace (GXmlTNode* self, const gchar* uri, const gchar* prefix);
gchar* gxml_tnode_to_string (GXmlTNode* self);
void gxml_tnode_set_parent (GXmlTNode* self, GXmlNode* node);
GXmlTNode* gxml_tnode_construct (GType object_type);
GeeMap* gxml_tnode_get_attrs (GXmlTNode* self);
GeeBidirList* gxml_tnode_get_children_nodes (GXmlTNode* self);
GXmlDocument* gxml_tnode_get_document (GXmlTNode* self);
gchar* gxml_tnode_get_name (GXmlTNode* self);
GeeList* gxml_tnode_get_namespaces (GXmlTNode* self);
GXmlNodeType gxml_tnode_get_type_node (GXmlTNode* self);
gchar* gxml_tnode_get_value (GXmlTNode* self);
void gxml_tnode_set_value (GXmlTNode* self, const gchar* value);
GXmlNode* gxml_tnode_get_parent (GXmlTNode* self);
GXmlTNodeTChildrenList* gxml_tnode_tchildren_list_new (GXmlNode* e);
GXmlTNodeTChildrenList* gxml_tnode_tchildren_list_construct (GType object_type, GXmlNode* e);
GType gxml_tprocessing_instruction_get_type (void) G_GNUC_CONST;
GXmlTProcessingInstruction* gxml_tprocessing_instruction_new (GXmlDocument* doc, const gchar* target, const gchar* data);
GXmlTProcessingInstruction* gxml_tprocessing_instruction_construct (GType object_type, GXmlDocument* doc, const gchar* target, const gchar* data);
GType gxml_ttext_get_type (void) G_GNUC_CONST;
GXmlTText* gxml_ttext_new (GXmlDocument* d, const gchar* text);
GXmlTText* gxml_ttext_construct (GType object_type, GXmlDocument* d, const gchar* text);
GType gxml_gnode_get_type (void) G_GNUC_CONST;
GType gxml_gdocument_get_type (void) G_GNUC_CONST;
GType gxml_xpath_object_type_get_type (void) G_GNUC_CONST;
GType gxml_xpath_object_get_type (void) G_GNUC_CONST;
GQuark gxml_xpath_error_quark (void);
GType gxml_xpath_context_get_type (void) G_GNUC_CONST;
GType gxml_gimplementation_get_type (void) G_GNUC_CONST;
GType gxml_html_document_get_type (void) G_GNUC_CONST;
GXmlHtmlDocument* gxml_html_document_new_from_path (const gchar* path, gint options, GError** error);
GXmlHtmlDocument* gxml_html_document_construct_from_path (GType object_type, const gchar* path, gint options, GError** error);
GXmlHtmlDocument* gxml_html_document_new_from_uri (const gchar* uri, gint options, GError** error);
GXmlHtmlDocument* gxml_html_document_construct_from_uri (GType object_type, const gchar* uri, gint options, GError** error);
GXmlHtmlDocument* gxml_html_document_new_from_file (GFile* file, gint options, GCancellable* cancel, GError** error);
GXmlHtmlDocument* gxml_html_document_construct_from_file (GType object_type, GFile* file, gint options, GCancellable* cancel, GError** error);
GXmlHtmlDocument* gxml_html_document_new_from_string (const gchar* html, gint options);
GXmlHtmlDocument* gxml_html_document_construct_from_string (GType object_type, const gchar* html, gint options);
GXmlHtmlDocument* gxml_html_document_new (void);
GXmlHtmlDocument* gxml_html_document_construct (GType object_type);
gint gxml_html_document_get_default_options (void);
GType gxml_dom_attr_get_type (void) G_GNUC_CONST;
GType gxml_gattribute_get_type (void) G_GNUC_CONST;
GXmlGAttribute* gxml_gattribute_new (GXmlGDocument* doc, xmlAttr* node);
GXmlGAttribute* gxml_gattribute_construct (GType object_type, GXmlGDocument* doc, xmlAttr* node);
GType gxml_gchild_node_get_type (void) G_GNUC_CONST;
GType gxml_gnon_document_child_node_get_type (void) G_GNUC_CONST;
GType gxml_gcharacter_data_get_type (void) G_GNUC_CONST;
GType gxml_gcomment_get_type (void) G_GNUC_CONST;
GXmlGComment* gxml_gcomment_new (GXmlGDocument* doc, xmlNode* node);
GXmlGComment* gxml_gcomment_construct (GType object_type, GXmlGDocument* doc, xmlNode* node);
GXmlGCharacterData* gxml_gcharacter_data_new (void);
GXmlGCharacterData* gxml_gcharacter_data_construct (GType object_type);
GXmlGChildNode* gxml_gchild_node_new (void);
GXmlGChildNode* gxml_gchild_node_construct (GType object_type);
GXmlGNonDocumentChildNode* gxml_gnon_document_child_node_new (void);
GXmlGNonDocumentChildNode* gxml_gnon_document_child_node_construct (GType object_type);
GType gxml_gcdata_get_type (void) G_GNUC_CONST;
GXmlGCDATA* gxml_gcdata_new (GXmlGDocument* doc, xmlNode* node);
GXmlGCDATA* gxml_gcdata_construct (GType object_type, GXmlGDocument* doc, xmlNode* node);
GXmlGDocument* gxml_gdocument_new (void);
GXmlGDocument* gxml_gdocument_construct (GType object_type);
GXmlGDocument* gxml_gdocument_new_from_path (const gchar* path, gint options, GError** error);
GXmlGDocument* gxml_gdocument_construct_from_path (GType object_type, const gchar* path, gint options, GError** error);
GXmlGDocument* gxml_gdocument_new_from_uri (const gchar* uri, gint options, GError** error);
GXmlGDocument* gxml_gdocument_construct_from_uri (GType object_type, const gchar* uri, gint options, GError** error);
GXmlGDocument* gxml_gdocument_new_from_file (GFile* file, gint options, GCancellable* cancel, GError** error);
GXmlGDocument* gxml_gdocument_construct_from_file (GType object_type, GFile* file, gint options, GCancellable* cancel, GError** error);
GXmlGDocument* gxml_gdocument_new_from_string (const gchar* str, gint options, GError** error);
GXmlGDocument* gxml_gdocument_construct_from_string (GType object_type, const gchar* str, gint options, GError** error);
GXmlGDocument* gxml_gdocument_new_from_stream (GInputStream* istream, GError** error);
GXmlGDocument* gxml_gdocument_construct_from_stream (GType object_type, GInputStream* istream, GError** error);
GXmlGDocument* gxml_gdocument_new_from_doc (xmlDoc* doc);
GXmlGDocument* gxml_gdocument_construct_from_doc (GType object_type, xmlDoc* doc);
gchar* gxml_gdocument_libxml_to_string (GXmlGDocument* self);
gboolean gxml_gdocument_save (GXmlGDocument* self, GCancellable* cancellable, GError** error);
gboolean gxml_gdocument_save_as (GXmlGDocument* self, GFile* f, GCancellable* cancellable, GError** error);
GXmlGImplementation* gxml_gimplementation_new (void);
GXmlGImplementation* gxml_gimplementation_construct (GType object_type);
GType gxml_gdocument_type_get_type (void) G_GNUC_CONST;
GXmlGDocumentType* gxml_gdocument_type_new_with_name (const gchar* name);
GXmlGDocumentType* gxml_gdocument_type_construct_with_name (GType object_type, const gchar* name);
GXmlGDocumentType* gxml_gdocument_type_new_with_ids (const gchar* name, const gchar* public_id, const gchar* system_id);
GXmlGDocumentType* gxml_gdocument_type_construct_with_ids (GType object_type, const gchar* name, const gchar* public_id, const gchar* system_id);
GXmlGDocumentType* gxml_gdocument_type_new (void);
GXmlGDocumentType* gxml_gdocument_type_construct (GType object_type);
GType gxml_gdocument_fragment_get_type (void) G_GNUC_CONST;
GXmlGDocumentFragment* gxml_gdocument_fragment_new (GXmlGDocument* d);
GXmlGDocumentFragment* gxml_gdocument_fragment_construct (GType object_type, GXmlGDocument* d);
GType gxml_gdom_node_iterator_get_type (void) G_GNUC_CONST;
GXmlGDomNodeIterator* gxml_gdom_node_iterator_new (GXmlDomNode* n, gint what_to_show, GXmlDomNodeFilter* filter);
GXmlGDomNodeIterator* gxml_gdom_node_iterator_construct (GType object_type, GXmlDomNode* n, gint what_to_show, GXmlDomNodeFilter* filter);
GType gxml_gdom_tree_walker_get_type (void) G_GNUC_CONST;
GXmlGDomTreeWalker* gxml_gdom_tree_walker_new (GXmlDomNode* r, gint w, GXmlDomNodeFilter* f);
GXmlGDomTreeWalker* gxml_gdom_tree_walker_construct (GType object_type, GXmlDomNode* r, gint w, GXmlDomNodeFilter* f);
GType gxml_gdom_token_list_get_type (void) G_GNUC_CONST;
GXmlGDomTokenList* gxml_gdom_token_list_new (GXmlDomElement* e, const gchar* attr);
GXmlGDomTokenList* gxml_gdom_token_list_construct (GType object_type, GXmlDomElement* e, const gchar* attr);
void gxml_gdom_token_list_update (GXmlGDomTokenList* self);
GType gxml_dom_settable_token_list_get_type (void) G_GNUC_CONST;
GType gxml_gdom_settable_token_list_get_type (void) G_GNUC_CONST;
GXmlGDomSettableTokenList* gxml_gdom_settable_token_list_new (GXmlDomElement* e, const gchar* attr);
GXmlGDomSettableTokenList* gxml_gdom_settable_token_list_construct (GType object_type, GXmlDomElement* e, const gchar* attr);
GType gxml_gdom_html_collection_get_type (void) G_GNUC_CONST;
GXmlGDomHTMLCollection* gxml_gdom_html_collection_new (void);
GXmlGDomHTMLCollection* gxml_gdom_html_collection_construct (GType object_type);
GType gxml_gdom_event_get_type (void) G_GNUC_CONST;
GType gxml_dom_event_flags_get_type (void) G_GNUC_CONST;
GXmlGDomEvent* gxml_gdom_event_new (void);
GXmlGDomEvent* gxml_gdom_event_construct (GType object_type);
GType gxml_gdom_custom_event_get_type (void) G_GNUC_CONST;
void gxml_gdom_custom_event_init_custom_event (GXmlGDomCustomEvent* self, const gchar* type, gboolean bubbles, gboolean cancelable, GValue* detail);
GXmlGDomCustomEvent* gxml_gdom_custom_event_new (void);
GXmlGDomCustomEvent* gxml_gdom_custom_event_construct (GType object_type);
GValue* gxml_gdom_custom_event_get_detail (GXmlGDomCustomEvent* self);
GType gxml_gdom_range_get_type (void) G_GNUC_CONST;
GXmlGDomRange* gxml_gdom_range_new (GXmlDomDocument* doc);
GXmlGDomRange* gxml_gdom_range_construct (GType object_type, GXmlDomDocument* doc);
GType gxml_gelement_get_type (void) G_GNUC_CONST;
GXmlGElement* gxml_gelement_new (GXmlGDocument* doc, xmlNode* node);
GXmlGElement* gxml_gelement_construct (GType object_type, GXmlGDocument* doc, xmlNode* node);
GType gxml_gnamespace_get_type (void) G_GNUC_CONST;
GXmlGNamespace* gxml_gnamespace_new (xmlNs* ns);
GXmlGNamespace* gxml_gnamespace_construct (GType object_type, xmlNs* ns);
xmlNs* gxml_gnamespace_get_internal_ns (GXmlGNamespace* self);
GQuark gxml_error_quark (void);
gboolean gxml_gnode_set_namespace (GXmlGNode* self, const gchar* uri, const gchar* prefix);
gchar* gxml_gnode_to_string (GXmlGNode* self);
xmlNode* gxml_gnode_get_internal_node (GXmlGNode* self);
GXmlNode* gxml_gnode_to_gnode (GXmlGDocument* doc, xmlNode* node);
GXmlGNode* gxml_gnode_construct (GType object_type);
GeeMap* gxml_gnode_get_attrs (GXmlGNode* self);
GeeBidirList* gxml_gnode_get_children_nodes (GXmlGNode* self);
GeeList* gxml_gnode_get_namespaces (GXmlGNode* self);
GXmlDocument* gxml_gnode_get_document (GXmlGNode* self);
GXmlNode* gxml_gnode_get_parent (GXmlGNode* self);
GXmlNodeType gxml_gnode_get_type_node (GXmlGNode* self);
gchar* gxml_gnode_get_name (GXmlGNode* self);
gchar* gxml_gnode_get_value (GXmlGNode* self);
void gxml_gnode_set_value (GXmlGNode* self, const gchar* value);
GType gxml_gprocessing_instruction_get_type (void) G_GNUC_CONST;
GXmlGProcessingInstruction* gxml_gprocessing_instruction_new (GXmlGDocument* doc, xmlNode* node);
GXmlGProcessingInstruction* gxml_gprocessing_instruction_construct (GType object_type, GXmlGDocument* doc, xmlNode* node);
GType gxml_gtext_get_type (void) G_GNUC_CONST;
GXmlGText* gxml_gtext_new (GXmlGDocument* doc, xmlNode* node);
GXmlGText* gxml_gtext_construct (GType object_type, GXmlGDocument* doc, xmlNode* node);
GType gxml_ghash_map_attr_get_type (void) G_GNUC_CONST;
GXmlGHashMapAttr* gxml_ghash_map_attr_new (GXmlGDocument* doc, xmlNode* node);
GXmlGHashMapAttr* gxml_ghash_map_attr_construct (GType object_type, GXmlGDocument* doc, xmlNode* node);
GType gxml_ghash_map_attr_entry_get_type (void) G_GNUC_CONST;
GXmlGHashMapAttrEntry* gxml_ghash_map_attr_entry_new (GXmlGDocument* doc, xmlAttr* attr);
GXmlGHashMapAttrEntry* gxml_ghash_map_attr_entry_construct (GType object_type, GXmlGDocument* doc, xmlAttr* attr);
GType gxml_ghash_map_attr_iterator_get_type (void) G_GNUC_CONST;
GXmlGHashMapAttrIterator* gxml_ghash_map_attr_iterator_new (GXmlGDocument* doc, xmlNode* node);
GXmlGHashMapAttrIterator* gxml_ghash_map_attr_iterator_construct (GType object_type, GXmlGDocument* doc, xmlNode* node);
GType gxml_glist_children_get_type (void) G_GNUC_CONST;
GXmlGListChildren* gxml_glist_children_new (GXmlGDocument* doc, xmlNode* node);
GXmlGListChildren* gxml_glist_children_construct (GType object_type, GXmlGDocument* doc, xmlNode* node);
GType gxml_glist_children_iterator_get_type (void) G_GNUC_CONST;
GXmlGListChildrenIterator* gxml_glist_children_iterator_new (GXmlGDocument* doc, xmlNode* node);
GXmlGListChildrenIterator* gxml_glist_children_iterator_construct (GType object_type, GXmlGDocument* doc, xmlNode* node);
GType gxml_glist_namespaces_get_type (void) G_GNUC_CONST;
GXmlGListNamespaces* gxml_glist_namespaces_new (GXmlGDocument* doc, xmlNode* node);
GXmlGListNamespaces* gxml_glist_namespaces_construct (GType object_type, GXmlGDocument* doc, xmlNode* node);
GType gxml_glist_namespaces_iterator_get_type (void) G_GNUC_CONST;
GXmlGListNamespacesIterator* gxml_glist_namespaces_iterator_new (xmlNode* node);
GXmlGListNamespacesIterator* gxml_glist_namespaces_iterator_construct (GType object_type, xmlNode* node);
gchar* gxml_dom_attr_get_namespace_uri (GXmlDomAttr* self);
gchar* gxml_dom_attr_get_prefix (GXmlDomAttr* self);
gchar* gxml_dom_attr_get_local_name (GXmlDomAttr* self);
gchar* gxml_dom_attr_get_name (GXmlDomAttr* self);
gchar* gxml_dom_attr_get_value (GXmlDomAttr* self);
void gxml_dom_attr_set_value (GXmlDomAttr* self, const gchar* value);
gboolean gxml_dom_attr_get_specified (GXmlDomAttr* self);
gchar* gxml_dom_character_data_substring_data (GXmlDomCharacterData* self, gint offset, gint count, GError** error);
void gxml_dom_character_data_append_data (GXmlDomCharacterData* self, const gchar* data);
void gxml_dom_character_data_insert_data (GXmlDomCharacterData* self, gint offset, const gchar* data, GError** error);
void gxml_dom_character_data_delete_data (GXmlDomCharacterData* self, gint offset, gint count, GError** error);
void gxml_dom_character_data_replace_data (GXmlDomCharacterData* self, gint offset, gint count, const gchar* data, GError** error);
gchar* gxml_dom_character_data_get_data (GXmlDomCharacterData* self);
void gxml_dom_character_data_set_data (GXmlDomCharacterData* self, const gchar* value);
gint gxml_dom_character_data_get_length (GXmlDomCharacterData* self);
GXmlDomText* gxml_dom_text_split_text (GXmlDomText* self, gint offset, GError** error);
gchar* gxml_dom_text_get_whole_text (GXmlDomText* self);
gchar* gxml_dom_processing_instruction_get_target (GXmlDomProcessingInstruction* self);
GXmlDomElement* gxml_dom_non_element_parent_node_get_element_by_id (GXmlDomNonElementParentNode* self, const gchar* element_id, GError** error);
GXmlDomElement* gxml_dom_parent_node_query_selector (GXmlDomParentNode* self, const gchar* selectors, GError** error);
GXmlDomNodeList* gxml_dom_parent_node_query_selector_all (GXmlDomParentNode* self, const gchar* selectors, GError** error);
GXmlDomElementList* gxml_dom_parent_node_get_elements_by_property_value (GXmlDomParentNode* self, const gchar* property, const gchar* value);
GXmlDomHTMLCollection* gxml_dom_parent_node_get_children (GXmlDomParentNode* self);
GXmlDomElement* gxml_dom_parent_node_get_first_element_child (GXmlDomParentNode* self);
GXmlDomElement* gxml_dom_parent_node_get_last_element_child (GXmlDomParentNode* self);
gint gxml_dom_parent_node_get_child_element_count (GXmlDomParentNode* self);
GXmlDomElement* gxml_dom_non_document_type_child_node_get_previous_element_sibling (GXmlDomNonDocumentTypeChildNode* self);
GXmlDomElement* gxml_dom_non_document_type_child_node_get_next_element_sibling (GXmlDomNonDocumentTypeChildNode* self);
void gxml_dom_child_node_remove (GXmlDomChildNode* self);
GXmlDomNode* gxml_dom_node_list_item (GXmlDomNodeList* self, gint index);
gint gxml_dom_node_list_get_length (GXmlDomNodeList* self);
GXmlDomElement* gxml_dom_html_collection_get_element (GXmlDomHTMLCollection* self, gint index);
GXmlDomElement** gxml_dom_html_collection_to_array (GXmlDomHTMLCollection* self, int* result_length1);
GXmlDomElement* gxml_dom_html_collection_item (GXmlDomHTMLCollection* self, gint index);
GXmlDomElement* gxml_dom_html_collection_named_item (GXmlDomHTMLCollection* self, const gchar* name);
gint gxml_dom_html_collection_get_length (GXmlDomHTMLCollection* self);
GXmlDomNode* gxml_dom_node_iterator_next_node (GXmlDomNodeIterator* self);
GXmlDomNode* gxml_dom_node_iterator_previous_node (GXmlDomNodeIterator* self);
void gxml_dom_node_iterator_detach (GXmlDomNodeIterator* self);
GXmlDomNode* gxml_dom_node_iterator_get_root (GXmlDomNodeIterator* self);
GXmlDomNode* gxml_dom_node_iterator_get_reference_node (GXmlDomNodeIterator* self);
gboolean gxml_dom_node_iterator_get_pointer_before_reference_node (GXmlDomNodeIterator* self);
gint gxml_dom_node_iterator_get_what_to_show (GXmlDomNodeIterator* self);
GXmlDomNodeFilter* gxml_dom_node_iterator_get_filter (GXmlDomNodeIterator* self);
#define GXML_DOM_NODE_FILTER_FILTER_ACCEPT 1
#define GXML_DOM_NODE_FILTER_FILTER_REJECT 2
#define GXML_DOM_NODE_FILTER_FILTER_SKIP 3
#define GXML_DOM_NODE_FILTER_SHOW_ALL ((gint) 0xFFFFFFFFLL)
#define GXML_DOM_NODE_FILTER_SHOW_ELEMENT ((gint) 0x1)
#define GXML_DOM_NODE_FILTER_SHOW_ATTRIBUTE ((gint) 0x2)
#define GXML_DOM_NODE_FILTER_SHOW_TEXT ((gint) 0x4)
#define GXML_DOM_NODE_FILTER_SHOW_CDATA_SECTION ((gint) 0x8)
#define GXML_DOM_NODE_FILTER_SHOW_ENTITY_REFERENCE ((gint) 0x10)
#define GXML_DOM_NODE_FILTER_SHOW_ENTITY ((gint) 0x20)
#define GXML_DOM_NODE_FILTER_SHOW_PROCESSING_INSTRUCTION ((gint) 0x40)
#define GXML_DOM_NODE_FILTER_SHOW_COMMENT ((gint) 0x80)
#define GXML_DOM_NODE_FILTER_SHOW_DOCUMENT ((gint) 0x100)
#define GXML_DOM_NODE_FILTER_SHOW_DOCUMENT_TYPE ((gint) 0x200)
#define GXML_DOM_NODE_FILTER_SHOW_DOCUMENT_FRAGMENT ((gint) 0x400)
#define GXML_DOM_NODE_FILTER_SHOW_NOTATION ((gint) 0x800)
GXmlDomNodeFilter* gxml_dom_node_filter_new (void);
GXmlDomNodeFilter* gxml_dom_node_filter_construct (GType object_type);
GXmlDomNode* gxml_dom_tree_walker_parentNode (GXmlDomTreeWalker* self);
GXmlDomNode* gxml_dom_tree_walker_firstChild (GXmlDomTreeWalker* self);
GXmlDomNode* gxml_dom_tree_walker_lastChild (GXmlDomTreeWalker* self);
GXmlDomNode* gxml_dom_tree_walker_previousSibling (GXmlDomTreeWalker* self);
GXmlDomNode* gxml_dom_tree_walker_nextSibling (GXmlDomTreeWalker* self);
GXmlDomNode* gxml_dom_tree_walker_previousNode (GXmlDomTreeWalker* self);
GXmlDomNode* gxml_dom_tree_walker_nextNode (GXmlDomTreeWalker* self);
GXmlDomNode* gxml_dom_tree_walker_get_root (GXmlDomTreeWalker* self);
gint gxml_dom_tree_walker_get_what_to_show (GXmlDomTreeWalker* self);
GXmlDomNodeFilter* gxml_dom_tree_walker_get_filter (GXmlDomTreeWalker* self);
GXmlDomNode* gxml_dom_tree_walker_get_current_node (GXmlDomTreeWalker* self);
GXmlDomNode* gxml_dom_named_node_map_item (GXmlDomNamedNodeMap* self, gint index);
GXmlDomNode* gxml_dom_named_node_map_get_named_item (GXmlDomNamedNodeMap* self, const gchar* name);
GXmlDomNode* gxml_dom_named_node_map_set_named_item (GXmlDomNamedNodeMap* self, GXmlDomNode* node, GError** error);
GXmlDomNode* gxml_dom_named_node_map_remove_named_item (GXmlDomNamedNodeMap* self, const gchar* name, GError** error);
GXmlDomNode* gxml_dom_named_node_map_remove_named_item_ns (GXmlDomNamedNodeMap* self, const gchar* namespace_uri, const gchar* localName, GError** error);
GXmlDomNode* gxml_dom_named_node_map_get_named_item_ns (GXmlDomNamedNodeMap* self, const gchar* namespace_uri, const gchar* local_name, GError** error);
GXmlDomNode* gxml_dom_named_node_map_set_named_item_ns (GXmlDomNamedNodeMap* self, GXmlDomNode* node, GError** error);
gint gxml_dom_named_node_map_get_length (GXmlDomNamedNodeMap* self);
gchar* gxml_dom_token_list_item (GXmlDomTokenList* self, gint index);
gboolean gxml_dom_token_list_contains (GXmlDomTokenList* self, const gchar* token, GError** error);
void gxml_dom_token_list_add (GXmlDomTokenList* self, gchar** tokens, int tokens_length1, GError** error);
void gxml_dom_token_list_remove (GXmlDomTokenList* self, gchar** tokens, int tokens_length1);
gboolean gxml_dom_token_list_toggle (GXmlDomTokenList* self, const gchar* token, gboolean force, gboolean _auto_, GError** error);
gchar* gxml_dom_token_list_to_string (GXmlDomTokenList* self);
gint gxml_dom_token_list_get_length (GXmlDomTokenList* self);
gchar* gxml_dom_settable_token_list_get_value (GXmlDomSettableTokenList* self);
void gxml_dom_settable_token_list_set_value (GXmlDomSettableTokenList* self, const gchar* value);
GXmlDomHTMLCollection* gxml_dom_document_get_elements_by_tag_name (GXmlDomDocument* self, const gchar* local_name);
GXmlDomHTMLCollection* gxml_dom_document_get_elements_by_tag_name_ns (GXmlDomDocument* self, const gchar* namespace, const gchar* local_name);
GXmlDomHTMLCollection* gxml_dom_document_get_elements_by_class_name (GXmlDomDocument* self, const gchar* classNames);
GXmlDomElement* gxml_dom_document_create_element (GXmlDomDocument* self, const gchar* local_name, GError** error);
GXmlDomElement* gxml_dom_document_create_element_ns (GXmlDomDocument* self, const gchar* namespace, const gchar* qualified_name, GError** error);
GXmlDomDocumentFragment* gxml_dom_document_create_document_fragment (GXmlDomDocument* self);
GXmlDomText* gxml_dom_document_create_text_node (GXmlDomDocument* self, const gchar* data, GError** error);
GXmlDomComment* gxml_dom_document_create_comment (GXmlDomDocument* self, const gchar* data, GError** error);
GXmlDomProcessingInstruction* gxml_dom_document_create_processing_instruction (GXmlDomDocument* self, const gchar* target, const gchar* data, GError** error);
GXmlDomNode* gxml_dom_document_import_node (GXmlDomDocument* self, GXmlDomNode* node, gboolean deep, GError** error);
GXmlDomNode* gxml_dom_document_adopt_node (GXmlDomDocument* self, GXmlDomNode* node, GError** error);
GXmlDomEvent* gxml_dom_document_create_event (GXmlDomDocument* self, const gchar* interface, GError** error);
GXmlDomRange* gxml_dom_document_create_range (GXmlDomDocument* self);
GXmlDomNodeIterator* gxml_dom_document_create_node_iterator (GXmlDomDocument* self, GXmlDomNode* root, gint whatToShow, GXmlDomNodeFilter* filter);
GXmlDomTreeWalker* gxml_dom_document_create_tree_walker (GXmlDomDocument* self, GXmlDomNode* root, gint what_to_show, GXmlDomNodeFilter* filter);
GXmlDomImplementation* gxml_dom_document_get_implementation (GXmlDomDocument* self);
const gchar* gxml_dom_document_get_url (GXmlDomDocument* self);
const gchar* gxml_dom_document_get_document_uri (GXmlDomDocument* self);
const gchar* gxml_dom_document_get_origin (GXmlDomDocument* self);
const gchar* gxml_dom_document_get_compat_mode (GXmlDomDocument* self);
const gchar* gxml_dom_document_get_character_set (GXmlDomDocument* self);
const gchar* gxml_dom_document_get_content_type (GXmlDomDocument* self);
GXmlDomDocumentType* gxml_dom_document_get_doctype (GXmlDomDocument* self);
GXmlDomElement* gxml_dom_document_get_document_element (GXmlDomDocument* self);
GXmlDomDocumentType* gxml_dom_implementation_create_document_type (GXmlDomImplementation* self, const gchar* qualified_name, const gchar* public_id, const gchar* system_id, GError** error);
GXmlDomXMLDocument* gxml_dom_implementation_create_document (GXmlDomImplementation* self, const gchar* nspace, const gchar* qualified_name, GXmlDomDocumentType* doctype, GError** error);
GXmlDocument* gxml_dom_implementation_create_html_document (GXmlDomImplementation* self, const gchar* title);
gboolean gxml_dom_implementation_has_feature (GXmlDomImplementation* self);
const gchar* gxml_dom_document_type_get_name (GXmlDomDocumentType* self);
const gchar* gxml_dom_document_type_get_public_id (GXmlDomDocumentType* self);
const gchar* gxml_dom_document_type_get_system_id (GXmlDomDocumentType* self);
gchar* gxml_dom_element_get_attribute (GXmlDomElement* self, const gchar* name);
gchar* gxml_dom_element_get_attribute_ns (GXmlDomElement* self, const gchar* namespace, const gchar* local_name);
void gxml_dom_element_set_attribute (GXmlDomElement* self, const gchar* name, const gchar* value, GError** error);
void gxml_dom_element_set_attribute_ns (GXmlDomElement* self, const gchar* namespace, const gchar* name, const gchar* value, GError** error);
void gxml_dom_element_remove_attribute (GXmlDomElement* self, const gchar* name);
void gxml_dom_element_remove_attribute_ns (GXmlDomElement* self, const gchar* namespace, const gchar* local_name);
gboolean gxml_dom_element_has_attribute (GXmlDomElement* self, const gchar* name);
gboolean gxml_dom_element_has_attribute_ns (GXmlDomElement* self, const gchar* namespace, const gchar* local_name);
GXmlDomHTMLCollection* gxml_dom_element_get_elements_by_tag_name (GXmlDomElement* self, const gchar* local_name);
GXmlDomHTMLCollection* gxml_dom_element_get_elements_by_tag_name_ns (GXmlDomElement* self, const gchar* namespace, const gchar* local_name);
GXmlDomHTMLCollection* gxml_dom_element_get_elements_by_class_name (GXmlDomElement* self, const gchar* class_names);
gchar* gxml_dom_element_get_namespace_uri (GXmlDomElement* self);
gchar* gxml_dom_element_get_prefix (GXmlDomElement* self);
gchar* gxml_dom_element_get_local_name (GXmlDomElement* self);
gchar* gxml_dom_element_get_tag_name (GXmlDomElement* self);
gchar* gxml_dom_element_get_id (GXmlDomElement* self);
void gxml_dom_element_set_id (GXmlDomElement* self, const gchar* value);
gchar* gxml_dom_element_get_class_name (GXmlDomElement* self);
void gxml_dom_element_set_class_name (GXmlDomElement* self, const gchar* value);
GXmlDomTokenList* gxml_dom_element_get_class_list (GXmlDomElement* self);
GXmlDomNamedNodeMap* gxml_dom_element_get_attributes (GXmlDomElement* self);
GXmlDomElementList* gxml_dom_element_list_new (void);
GXmlDomElementList* gxml_dom_element_list_construct (GType object_type);
void gxml_dom_event_target_add_event_listener (GXmlDomEventTarget* self, const gchar* type, GXmlDomEventListener* callback, gboolean capture);
void gxml_dom_event_target_remove_event_listener (GXmlDomEventTarget* self, const gchar* type, GXmlDomEventListener* callback, gboolean capture);
gboolean gxml_dom_event_target_dispatch_event (GXmlDomEventTarget* self, GXmlDomEvent* event, GError** error);
void gxml_dom_event_listener_handle_event (GXmlDomEventListener* self, GXmlDomEvent* event);
void gxml_dom_event_stop_propagation (GXmlDomEvent* self);
void gxml_dom_event_stop_immediate_propagation (GXmlDomEvent* self);
void gxml_dom_event_prevent_default (GXmlDomEvent* self);
void gxml_dom_event_init_event (GXmlDomEvent* self, const gchar* type, gboolean bubbles, gboolean cancelable);
const gchar* gxml_dom_event_get_etype (GXmlDomEvent* self);
GXmlDomEventTarget* gxml_dom_event_get_event_target (GXmlDomEvent* self);
GXmlDomEventTarget* gxml_dom_event_get_current_target (GXmlDomEvent* self);
gboolean gxml_dom_event_get_bubbles (GXmlDomEvent* self);
gboolean gxml_dom_event_get_cancelable (GXmlDomEvent* self);
gboolean gxml_dom_event_get_is_trusted (GXmlDomEvent* self);
GXmlDomTimeStamp* gxml_dom_event_get_time_stamp (GXmlDomEvent* self);
gboolean gxml_dom_event_get_default_prevented (GXmlDomEvent* self);
GXmlDomEventPhase gxml_dom_event_get_event_phase (GXmlDomEvent* self);
GType gxml_dom_event_init_get_type (void) G_GNUC_CONST;
GXmlDomEventInit* gxml_dom_event_init_new (void);
GXmlDomEventInit* gxml_dom_event_init_construct (GType object_type);
gboolean gxml_dom_event_init_get_bubbles (GXmlDomEventInit* self);
void gxml_dom_event_init_set_bubbles (GXmlDomEventInit* self, gboolean value);
gboolean gxml_dom_event_init_get_cancelable (GXmlDomEventInit* self);
void gxml_dom_event_init_set_cancelable (GXmlDomEventInit* self, gboolean value);
GType gxml_dom_custom_event_get_type (void) G_GNUC_CONST;
void gxml_dom_custom_event_init_custom_event (GXmlDomCustomEvent* self, const gchar* type, gboolean bubbles, gboolean cancelable, GValue* detail);
GValue* gxml_dom_custom_event_get_detail (GXmlDomCustomEvent* self);
GType gxml_dom_custom_event_init_get_type (void) G_GNUC_CONST;
GXmlDomCustomEventInit* gxml_dom_custom_event_init_new (void);
GXmlDomCustomEventInit* gxml_dom_custom_event_init_construct (GType object_type);
GValue* gxml_dom_custom_event_init_get_detail (GXmlDomCustomEventInit* self);
void gxml_dom_custom_event_init_set_detail (GXmlDomCustomEventInit* self, GValue* value);
gchar* gxml_dom_time_stamp_to_string (GXmlDomTimeStamp* self);
GXmlDomTimeStamp* gxml_dom_time_stamp_new (void);
GXmlDomTimeStamp* gxml_dom_time_stamp_construct (GType object_type);
GDateTime* gxml_dom_time_stamp_get_time (GXmlDomTimeStamp* self);
void gxml_dom_time_stamp_set_time (GXmlDomTimeStamp* self, GDateTime* value);
GType gxml_dom_mutation_observer_init_get_type (void) G_GNUC_CONST;
GType gxml_dom_mutation_record_get_type (void) G_GNUC_CONST;
GType gxml_dom_mutation_observer_get_type (void) G_GNUC_CONST;
void gxml_dom_mutation_observer_observe (GXmlDomMutationObserver* self, GXmlNode* target, GXmlDomMutationObserverInit* options);
void gxml_dom_mutation_observer_disconnect (GXmlDomMutationObserver* self);
GeeList* gxml_dom_mutation_observer_take_records (GXmlDomMutationObserver* self);
GXmlDomMutationObserverInit* gxml_dom_mutation_observer_init_new (void);
GXmlDomMutationObserverInit* gxml_dom_mutation_observer_init_construct (GType object_type);
gboolean gxml_dom_mutation_observer_init_get_child_list (GXmlDomMutationObserverInit* self);
void gxml_dom_mutation_observer_init_set_child_list (GXmlDomMutationObserverInit* self, gboolean value);
gboolean gxml_dom_mutation_observer_init_get_attributes (GXmlDomMutationObserverInit* self);
void gxml_dom_mutation_observer_init_set_attributes (GXmlDomMutationObserverInit* self, gboolean value);
gboolean gxml_dom_mutation_observer_init_get_character_data (GXmlDomMutationObserverInit* self);
void gxml_dom_mutation_observer_init_set_character_data (GXmlDomMutationObserverInit* self, gboolean value);
gboolean gxml_dom_mutation_observer_init_get_subtree (GXmlDomMutationObserverInit* self);
void gxml_dom_mutation_observer_init_set_subtree (GXmlDomMutationObserverInit* self, gboolean value);
gboolean gxml_dom_mutation_observer_init_get_attribute_old_value (GXmlDomMutationObserverInit* self);
void gxml_dom_mutation_observer_init_set_attribute_old_value (GXmlDomMutationObserverInit* self, gboolean value);
gboolean gxml_dom_mutation_observer_init_get_character_data_old_value (GXmlDomMutationObserverInit* self);
void gxml_dom_mutation_observer_init_set_character_data_old_value (GXmlDomMutationObserverInit* self, gboolean value);
GeeList* gxml_dom_mutation_observer_init_get_attribute_filter (GXmlDomMutationObserverInit* self);
void gxml_dom_mutation_observer_init_set_attribute_filter (GXmlDomMutationObserverInit* self, GeeList* value);
const gchar* gxml_dom_mutation_record_get_mtype (GXmlDomMutationRecord* self);
GXmlDomNode* gxml_dom_mutation_record_get_target (GXmlDomMutationRecord* self);
GXmlDomNodeList* gxml_dom_mutation_record_get_added_nodes (GXmlDomMutationRecord* self);
void gxml_dom_mutation_record_set_added_nodes (GXmlDomMutationRecord* self, GXmlDomNodeList* value);
GXmlDomNodeList* gxml_dom_mutation_record_get_removed_nodes (GXmlDomMutationRecord* self);
void gxml_dom_mutation_record_set_removed_nodes (GXmlDomMutationRecord* self, GXmlDomNodeList* value);
GXmlDomNode* gxml_dom_mutation_record_get_previous_sibling (GXmlDomMutationRecord* self);
GXmlDomNode* gxml_dom_mutation_record_get_next_sibling (GXmlDomMutationRecord* self);
const gchar* gxml_dom_mutation_record_get_attribute_name (GXmlDomMutationRecord* self);
const gchar* gxml_dom_mutation_record_get_attribute_namespace (GXmlDomMutationRecord* self);
const gchar* gxml_dom_mutation_record_get_old_value (GXmlDomMutationRecord* self);
gboolean gxml_dom_node_has_child_nodes (GXmlDomNode* self);
void gxml_dom_node_normalize (GXmlDomNode* self);
gboolean gxml_dom_node_is_equal_node (GXmlDomNode* self, GXmlDomNode* node);
GXmlDomNodeDocumentPosition gxml_dom_node_compare_document_position (GXmlDomNode* self, GXmlDomNode* other);
gboolean gxml_dom_node_contains (GXmlDomNode* self, GXmlDomNode* other);
gchar* gxml_dom_node_lookup_prefix (GXmlDomNode* self, const gchar* nspace);
gchar* gxml_dom_node_lookup_namespace_uri (GXmlDomNode* self, const gchar* prefix);
gboolean gxml_dom_node_is_default_namespace (GXmlDomNode* self, const gchar* nspace);
GXmlDomNode* gxml_dom_node_insert_before (GXmlDomNode* self, GXmlDomNode* node, GXmlDomNode* child, GError** error);
GXmlDomNode* gxml_dom_node_append_child (GXmlDomNode* self, GXmlDomNode* node, GError** error);
GXmlDomNode* gxml_dom_node_replace_child (GXmlDomNode* self, GXmlDomNode* node, GXmlDomNode* child, GError** error);
GXmlDomNode* gxml_dom_node_remove_child (GXmlDomNode* self, GXmlDomNode* child, GError** error);
GXmlDomNode* gxml_dom_node_clone_node (GXmlDomNode* self, gboolean deep, GError** error);
gboolean gxml_dom_node_copy (GXmlDomDocument* doc, GXmlDomNode* node, GXmlDomNode* source, gboolean deep, GError** error);
GXmlDomNodeNodeType gxml_dom_node_get_node_type (GXmlDomNode* self);
gchar* gxml_dom_node_get_node_name (GXmlDomNode* self);
const gchar* gxml_dom_node_get_base_uri (GXmlDomNode* self);
GXmlDomDocument* gxml_dom_node_get_owner_document (GXmlDomNode* self);
void gxml_dom_node_set_owner_document (GXmlDomNode* self, GXmlDomDocument* value);
GXmlDomNode* gxml_dom_node_get_parent_node (GXmlDomNode* self);
GXmlDomElement* gxml_dom_node_get_parent_element (GXmlDomNode* self);
GXmlDomNodeList* gxml_dom_node_get_child_nodes (GXmlDomNode* self);
GXmlDomNode* gxml_dom_node_get_first_child (GXmlDomNode* self);
GXmlDomNode* gxml_dom_node_get_last_child (GXmlDomNode* self);
GXmlDomNode* gxml_dom_node_get_previous_sibling (GXmlDomNode* self);
GXmlDomNode* gxml_dom_node_get_next_sibling (GXmlDomNode* self);
gchar* gxml_dom_node_get_node_value (GXmlDomNode* self);
void gxml_dom_node_set_node_value (GXmlDomNode* self, const gchar* value);
gchar* gxml_dom_node_get_text_content (GXmlDomNode* self);
void gxml_dom_node_set_text_content (GXmlDomNode* self, const gchar* value);
GQuark gxml_dom_error_quark (void);
GType gxml_dom_error_name_get_type (void) G_GNUC_CONST;
gchar* gxml_dom_error_name_get_name (GXmlDomErrorName* self, gint error_code);
gint gxml_dom_error_name_get_code (GXmlDomErrorName* self, const gchar* error_name);
GXmlDomErrorName* gxml_dom_error_name_new (void);
GXmlDomErrorName* gxml_dom_error_name_construct (GType object_type);
void gxml_dom_range_set_start (GXmlDomRange* self, GXmlDomNode* node, gint offset, GError** error);
void gxml_dom_range_set_end (GXmlDomRange* self, GXmlDomNode* node, gint offset, GError** error);
void gxml_dom_range_set_start_before (GXmlDomRange* self, GXmlDomNode* node, GError** error);
void gxml_dom_range_set_start_after (GXmlDomRange* self, GXmlDomNode* node, GError** error);
void gxml_dom_range_set_end_before (GXmlDomRange* self, GXmlDomNode* node, GError** error);
void gxml_dom_range_set_end_after (GXmlDomRange* self, GXmlDomNode* node, GError** error);
void gxml_dom_range_collapse (GXmlDomRange* self, gboolean to_start, GError** error);
void gxml_dom_range_select_node (GXmlDomRange* self, GXmlDomNode* node, GError** error);
void gxml_dom_range_select_node_contents (GXmlDomRange* self, GXmlDomNode* node, GError** error);
gint gxml_dom_range_compare_boundary_points (GXmlDomRange* self, GXmlDomRangeBoundaryPoints how, GXmlDomRange* sourceRange, GError** error);
void gxml_dom_range_delete_contents (GXmlDomRange* self, GError** error);
GXmlDomDocumentFragment* gxml_dom_range_extract_contents (GXmlDomRange* self, GError** error);
GXmlDomDocumentFragment* gxml_dom_range_clone_contents (GXmlDomRange* self, GError** error);
void gxml_dom_range_insert_node (GXmlDomRange* self, GXmlDomNode* node);
void gxml_dom_range_surround_contents (GXmlDomRange* self, GXmlDomNode* newParent);
GXmlDomRange* gxml_dom_range_clone_range (GXmlDomRange* self);
void gxml_dom_range_detach (GXmlDomRange* self);
gboolean gxml_dom_range_is_point_in_range (GXmlDomRange* self, GXmlDomNode* node, gint offset);
gshort gxml_dom_range_compare_point (GXmlDomRange* self, GXmlDomNode* node, gint offset);
gboolean gxml_dom_range_intersects_node (GXmlDomRange* self, GXmlDomNode* node);
gchar* gxml_dom_range_to_string (GXmlDomRange* self);
GXmlDomNode* gxml_dom_range_get_start_container (GXmlDomRange* self);
gint gxml_dom_range_get_start_offset (GXmlDomRange* self);
GXmlDomNode* gxml_dom_range_get_end_container (GXmlDomRange* self);
gint gxml_dom_range_get_end_offset (GXmlDomRange* self);
gboolean gxml_dom_range_get_collapsed (GXmlDomRange* self);
GXmlDomNode* gxml_dom_range_get_common_ancestor_container (GXmlDomRange* self);
GXmlXPathObject* gxml_xpath_context_evaluate (GXmlXPathContext* self, const gchar* expression, GeeList* resolver, GError** error);
GXmlXPathObjectType gxml_xpath_object_get_object_type (GXmlXPathObject* self);
gboolean gxml_xpath_object_get_boolean_value (GXmlXPathObject* self);
const gchar* gxml_xpath_object_get_string_value (GXmlXPathObject* self);
gdouble gxml_xpath_object_get_number_value (GXmlXPathObject* self);
GXmlDomHTMLCollection* gxml_xpath_object_get_nodeset (GXmlXPathObject* self);
GType gxml_gx_path_object_get_type (void) G_GNUC_CONST;
GXmlGXPathObject* gxml_gx_path_object_new (GXmlGDocument* document, xmlXPathObject* pointer);
GXmlGXPathObject* gxml_gx_path_object_construct (GType object_type, GXmlGDocument* document, xmlXPathObject* pointer);
GType gxml_gom_node_get_type (void) G_GNUC_CONST;
GType gxml_gom_node_list_get_type (void) G_GNUC_CONST;
GType gxml_gom_document_get_type (void) G_GNUC_CONST;
GXmlGomDocument* gxml_gom_document_new (void);
GXmlGomDocument* gxml_gom_document_construct (GType object_type);
GXmlGomDocument* gxml_gom_document_new_from_path (const gchar* path, GError** error);
GXmlGomDocument* gxml_gom_document_construct_from_path (GType object_type, const gchar* path, GError** error);
GXmlGomDocument* gxml_gom_document_new_from_uri (const gchar* uri, GError** error);
GXmlGomDocument* gxml_gom_document_construct_from_uri (GType object_type, const gchar* uri, GError** error);
GXmlGomDocument* gxml_gom_document_new_from_file (GFile* file, GError** error);
GXmlGomDocument* gxml_gom_document_construct_from_file (GType object_type, GFile* file, GError** error);
GXmlGomDocument* gxml_gom_document_new_from_stream (GInputStream* stream, GError** error);
GXmlGomDocument* gxml_gom_document_construct_from_stream (GType object_type, GInputStream* stream, GError** error);
GXmlGomDocument* gxml_gom_document_new_from_string (const gchar* str, GError** error);
GXmlGomDocument* gxml_gom_document_construct_from_string (GType object_type, const gchar* str, GError** error);
void gxml_gom_document_write_file (GXmlGomDocument* self, GFile* file, GError** error);
void gxml_gom_document_write_stream (GXmlGomDocument* self, GOutputStream* stream, GError** error);
void gxml_gom_document_read_from_file (GXmlGomDocument* self, GFile* file, GError** error);
void gxml_gom_document_read_from_string (GXmlGomDocument* self, const gchar* str, GError** error);
GType gxml_gom_implementation_get_type (void) G_GNUC_CONST;
GXmlGomImplementation* gxml_gom_implementation_new (void);
GXmlGomImplementation* gxml_gom_implementation_construct (GType object_type);
GType gxml_gom_document_type_get_type (void) G_GNUC_CONST;
GXmlGomDocumentType* gxml_gom_document_type_new_with_name (GXmlDomDocument* doc, const gchar* name);
GXmlGomDocumentType* gxml_gom_document_type_construct_with_name (GType object_type, GXmlDomDocument* doc, const gchar* name);
GXmlGomDocumentType* gxml_gom_document_type_new_with_ids (GXmlDomDocument* doc, const gchar* name, const gchar* public_id, const gchar* system_id);
GXmlGomDocumentType* gxml_gom_document_type_construct_with_ids (GType object_type, GXmlDomDocument* doc, const gchar* name, const gchar* public_id, const gchar* system_id);
GXmlGomDocumentType* gxml_gom_document_type_new (void);
GXmlGomDocumentType* gxml_gom_document_type_construct (GType object_type);
GType gxml_gom_document_fragment_get_type (void) G_GNUC_CONST;
GXmlGomDocumentFragment* gxml_gom_document_fragment_new (GXmlDomDocument* doc);
GXmlGomDocumentFragment* gxml_gom_document_fragment_construct (GType object_type, GXmlDomDocument* doc);
GType gxml_gom_object_get_type (void) G_GNUC_CONST;
GType gxml_gom_element_get_type (void) G_GNUC_CONST;
GType gxml_gom_element_attributes_get_type (void) G_GNUC_CONST;
void gxml_gom_element_read_from_uri (GXmlGomElement* self, const gchar* uri, GError** error);
void gxml_gom_element_read_from_file (GXmlGomElement* self, GFile* f, GCancellable* cancellable, GError** error);
void gxml_gom_element_read_from_stream (GXmlGomElement* self, GInputStream* istream, GCancellable* cancellable, GError** error);
void gxml_gom_element_read_from_string (GXmlGomElement* self, const gchar* str, GError** error);
gchar* gxml_gom_element_write_string (GXmlGomElement* self, GError** error);
void gxml_gom_element_write_file (GXmlGomElement* self, GFile* f, GError** error);
void gxml_gom_element_write_stream (GXmlGomElement* self, GOutputStream* stream, GError** error);
gchar* gxml_gom_element_lookup_prefix (GXmlGomElement* self, const gchar* nspace);
gchar* gxml_gom_element_lookup_namespace_uri (GXmlGomElement* self, const gchar* prefix);
void gxml_gom_element_initialize (GXmlGomElement* self, const gchar* local_name);
void gxml_gom_element_initialize_document (GXmlGomElement* self, GXmlDomDocument* doc, const gchar* local_name);
void gxml_gom_element_initialize_with_namespace (GXmlGomElement* self, const gchar* namespace_uri, const gchar* prefix, const gchar* local_name);
void gxml_gom_element_initialize_document_with_namespace (GXmlGomElement* self, GXmlDomDocument* doc, const gchar* namespace_uri, const gchar* prefix, const gchar* local_name);
GXmlGomElement* gxml_gom_element_new (void);
GXmlGomElement* gxml_gom_element_construct (GType object_type);
GXmlGomElementAttributes* gxml_gom_element_attributes_new (GXmlGomElement* element);
GXmlGomElementAttributes* gxml_gom_element_attributes_construct (GType object_type, GXmlGomElement* element);
GType gxml_gom_attr_get_type (void) G_GNUC_CONST;
GXmlGomAttr* gxml_gom_attr_new (GXmlDomElement* element, const gchar* name, const gchar* value);
GXmlGomAttr* gxml_gom_attr_construct (GType object_type, GXmlDomElement* element, const gchar* name, const gchar* value);
GXmlGomAttr* gxml_gom_attr_new_namespace (GXmlDomElement* element, const gchar* namespace_uri, const gchar* prefix, const gchar* name, const gchar* value);
GXmlGomAttr* gxml_gom_attr_construct_namespace (GType object_type, GXmlDomElement* element, const gchar* namespace_uri, const gchar* prefix, const gchar* name, const gchar* value);
GXmlGomNode* gxml_gom_node_new (void);
GXmlGomNode* gxml_gom_node_construct (GType object_type);
GXmlGomNodeList* gxml_gom_node_list_new (void);
GXmlGomNodeList* gxml_gom_node_list_construct (GType object_type);
GType gxml_gom_character_data_get_type (void) G_GNUC_CONST;
GXmlGomCharacterData* gxml_gom_character_data_new (void);
GXmlGomCharacterData* gxml_gom_character_data_construct (GType object_type);
GType gxml_gom_text_get_type (void) G_GNUC_CONST;
GXmlGomText* gxml_gom_text_new (GXmlDomDocument* doc, const gchar* data);
GXmlGomText* gxml_gom_text_construct (GType object_type, GXmlDomDocument* doc, const gchar* data);
GType gxml_gom_processing_instruction_get_type (void) G_GNUC_CONST;
GXmlGomProcessingInstruction* gxml_gom_processing_instruction_new (GXmlDomDocument* doc, const gchar* target, const gchar* data);
GXmlGomProcessingInstruction* gxml_gom_processing_instruction_construct (GType object_type, GXmlDomDocument* doc, const gchar* target, const gchar* data);
GType gxml_gom_comment_get_type (void) G_GNUC_CONST;
GXmlGomComment* gxml_gom_comment_new (GXmlDomDocument* doc, const gchar* data);
GXmlGomComment* gxml_gom_comment_construct (GType object_type, GXmlDomDocument* doc, const gchar* data);
GList* gxml_gom_object_get_properties_list (GXmlGomObject* self);
GParamSpec* gxml_gom_object_find_property_name (GXmlGomObject* self, const gchar* pname);
GParamSpec* gxml_gom_object_find_object_property_name (GXmlGomObject* self, const gchar* pname);
GList* gxml_gom_object_get_property_element_list (GXmlGomObject* self);
gchar* gxml_gom_object_get_property_string (GXmlGomObject* self, GParamSpec* prop);
gchar* gxml_gom_object_get_attribute (GXmlGomObject* self, const gchar* name);
gboolean gxml_gom_object_set_attribute (GXmlGomObject* self, const gchar* name, const gchar* val);
GXmlDomElement* gxml_gom_object_get_child (GXmlGomObject* self, const gchar* name);
GXmlDomElementList* gxml_gom_object_find_elements (GXmlGomObject* self, const gchar* name);
gboolean gxml_gom_object_remove_attribute (GXmlGomObject* self, const gchar* name);
gboolean gxml_gom_object_set_instance_property (GXmlGomObject* self, const gchar* name);
void gxml_gom_object_clean_property_elements (GXmlGomObject* self, const gchar* name, GError** error);
GType gxml_gom_collection_get_type (void) G_GNUC_CONST;
void gxml_gom_collection_search (GXmlGomCollection* self, GError** error);
GXmlDomElement* gxml_gom_collection_get_item (GXmlGomCollection* self, gint index, GError** error);
void gxml_gom_collection_append (GXmlGomCollection* self, GXmlDomElement* node, GError** error);
void gxml_gom_collection_initialize (GXmlGomCollection* self, GType t, GError** error);
GXmlGomElement* gxml_gom_collection_create_item (GXmlGomCollection* self);
gboolean gxml_gom_collection_validate_append (GXmlGomCollection* self, gint index, GXmlDomElement* element, GError** error);
GQueue* gxml_gom_collection_get_nodes_index (GXmlGomCollection* self);
GXmlGomElement* gxml_gom_collection_get_element (GXmlGomCollection* self);
void gxml_gom_collection_set_element (GXmlGomCollection* self, GXmlGomElement* value);
const gchar* gxml_gom_collection_get_items_name (GXmlGomCollection* self);
GType gxml_gom_collection_get_items_type (GXmlGomCollection* self);
void gxml_gom_collection_set_items_type (GXmlGomCollection* self, GType value);
gint gxml_gom_collection_get_length (GXmlGomCollection* self);
GType gxml_base_collection_get_type (void) G_GNUC_CONST;
void gxml_base_collection_initialize (GXmlBaseCollection* self, GType items_type, GError** error);
void gxml_base_collection_initialize_element (GXmlBaseCollection* self, GXmlGomElement* e, GError** error);
void gxml_base_collection_append (GXmlBaseCollection* self, GXmlDomElement* node, GError** error);
void gxml_base_collection_search (GXmlBaseCollection* self, GError** error);
gboolean gxml_base_collection_validate_append (GXmlBaseCollection* self, gint index, GXmlDomElement* element, GError** error);
GXmlBaseCollection* gxml_base_collection_construct (GType object_type);
const gchar* gxml_base_collection_get_items_name (GXmlBaseCollection* self);
GType gxml_base_collection_get_items_type (GXmlBaseCollection* self);
void gxml_base_collection_set_items_type (GXmlBaseCollection* self, GType value);
GQueue* gxml_base_collection_get_nodes_index (GXmlBaseCollection* self);
GXmlGomElement* gxml_base_collection_get_element (GXmlBaseCollection* self);
void gxml_base_collection_set_element (GXmlBaseCollection* self, GXmlGomElement* value);
GType gxml_gom_array_list_get_type (void) G_GNUC_CONST;
GXmlGomArrayList* gxml_gom_array_list_new (void);
GXmlGomArrayList* gxml_gom_array_list_construct (GType object_type);
GType gxml_mappeable_element_get_type (void) G_GNUC_CONST;
gchar* gxml_mappeable_element_get_map_key (GXmlMappeableElement* self);
GType gxml_gom_hash_map_get_type (void) G_GNUC_CONST;
void gxml_gom_hash_map_initialize_element_with_key (GXmlGomHashMap* self, GXmlGomElement* element, GType items_type, const gchar* attribute_key, GError** error);
void gxml_gom_hash_map_initialize_with_key (GXmlGomHashMap* self, GType items_type, const gchar* attribute_key, GError** error);
GXmlDomElement* gxml_gom_hash_map_get (GXmlGomHashMap* self, const gchar* key);
gboolean gxml_gom_hash_map_has_key (GXmlGomHashMap* self, const gchar* key);
GList* gxml_gom_hash_map_get_keys (GXmlGomHashMap* self);
GXmlGomHashMap* gxml_gom_hash_map_new (void);
GXmlGomHashMap* gxml_gom_hash_map_construct (GType object_type);
const gchar* gxml_gom_hash_map_get_attribute_key (GXmlGomHashMap* self);
void gxml_gom_hash_map_set_attribute_key (GXmlGomHashMap* self, const gchar* value);
GType gxml_mappeable_element_pair_key_get_type (void) G_GNUC_CONST;
gchar* gxml_mappeable_element_pair_key_get_map_primary_key (GXmlMappeableElementPairKey* self);
gchar* gxml_mappeable_element_pair_key_get_map_secondary_key (GXmlMappeableElementPairKey* self);
GType gxml_gom_hash_paired_map_get_type (void) G_GNUC_CONST;
void gxml_gom_hash_paired_map_initialize_element_with_keys (GXmlGomHashPairedMap* self, GXmlGomElement* element, GType items_type, const gchar* attribute_primary_key, const gchar* attribute_secondary_key, GError** error);
void gxml_gom_hash_paired_map_initialize_with_keys (GXmlGomHashPairedMap* self, GType items_type, const gchar* attribute_primary_key, const gchar* attribute_secondary_key, GError** error);
GXmlDomElement* gxml_gom_hash_paired_map_get (GXmlGomHashPairedMap* self, const gchar* primary_key, const gchar* secondary_key);
gboolean gxml_gom_hash_paired_map_has_primary_key (GXmlGomHashPairedMap* self, const gchar* key);
gboolean gxml_gom_hash_paired_map_has_secondary_key (GXmlGomHashPairedMap* self, const gchar* pkey, const gchar* key);
GList* gxml_gom_hash_paired_map_get_primary_keys (GXmlGomHashPairedMap* self);
GList* gxml_gom_hash_paired_map_get_secondary_keys (GXmlGomHashPairedMap* self, const gchar* pkey);
GXmlGomHashPairedMap* gxml_gom_hash_paired_map_new (void);
GXmlGomHashPairedMap* gxml_gom_hash_paired_map_construct (GType object_type);
const gchar* gxml_gom_hash_paired_map_get_attribute_primary_key (GXmlGomHashPairedMap* self);
void gxml_gom_hash_paired_map_set_attribute_primary_key (GXmlGomHashPairedMap* self, const gchar* value);
const gchar* gxml_gom_hash_paired_map_get_attribute_secondary_key (GXmlGomHashPairedMap* self);
void gxml_gom_hash_paired_map_set_attribute_secondary_key (GXmlGomHashPairedMap* self, const gchar* value);
GType gxml_gom_property_get_type (void) G_GNUC_CONST;
gboolean gxml_gom_property_validate_value (GXmlGomProperty* self, const gchar* val);
gchar* gxml_gom_property_get_value (GXmlGomProperty* self);
void gxml_gom_property_set_value (GXmlGomProperty* self, const gchar* value);
GType gxml_gom_base_property_get_type (void) G_GNUC_CONST;
gboolean gxml_gom_base_property_validate_value (GXmlGomBaseProperty* self, const gchar* val);
GXmlGomBaseProperty* gxml_gom_base_property_construct (GType object_type);
gchar* gxml_gom_base_property_get_value (GXmlGomBaseProperty* self);
void gxml_gom_base_property_set_value (GXmlGomBaseProperty* self, const gchar* value);
GType gxml_gom_string_get_type (void) G_GNUC_CONST;
GXmlGomString* gxml_gom_string_new (void);
GXmlGomString* gxml_gom_string_construct (GType object_type);
GType gxml_gom_array_string_get_type (void) G_GNUC_CONST;
gchar** gxml_gom_array_string_get_values (GXmlGomArrayString* self, int* result_length1);
void gxml_gom_array_string_initialize_strings (GXmlGomArrayString* self, gchar** strs, int strs_length1);
gboolean gxml_gom_array_string_is_valid_value (GXmlGomArrayString* self);
void gxml_gom_array_string_select (GXmlGomArrayString* self, gint index);
gboolean gxml_gom_array_string_search (GXmlGomArrayString* self, const gchar* str);
GXmlGomArrayString* gxml_gom_array_string_new (void);
GXmlGomArrayString* gxml_gom_array_string_construct (GType object_type);
GType gxml_gom_xsd_array_string_get_type (void) G_GNUC_CONST;
void gxml_gom_xsd_array_string_load (GXmlGomXsdArrayString* self, GError** error);
GXmlGomXsdArrayString* gxml_gom_xsd_array_string_new (void);
GXmlGomXsdArrayString* gxml_gom_xsd_array_string_construct (GType object_type);
const gchar* gxml_gom_xsd_array_string_get_simple_type (GXmlGomXsdArrayString* self);
void gxml_gom_xsd_array_string_set_simple_type (GXmlGomXsdArrayString* self, const gchar* value);
GFile* gxml_gom_xsd_array_string_get_source (GXmlGomXsdArrayString* self);
void gxml_gom_xsd_array_string_set_source (GXmlGomXsdArrayString* self, GFile* value);
GType gxml_gom_double_get_type (void) G_GNUC_CONST;
gdouble gxml_gom_double_get_double (GXmlGomDouble* self);
void gxml_gom_double_set_double (GXmlGomDouble* self, gdouble value);
GXmlGomDouble* gxml_gom_double_new (void);
GXmlGomDouble* gxml_gom_double_construct (GType object_type);
guint gxml_gom_double_get_decimals (GXmlGomDouble* self);
void gxml_gom_double_set_decimals (GXmlGomDouble* self, guint value);
GType gxml_gom_float_get_type (void) G_GNUC_CONST;
gfloat gxml_gom_float_get_float (GXmlGomFloat* self);
void gxml_gom_float_set_float (GXmlGomFloat* self, gfloat value);
GXmlGomFloat* gxml_gom_float_new (void);
GXmlGomFloat* gxml_gom_float_construct (GType object_type);
GType gxml_gom_int_get_type (void) G_GNUC_CONST;
gint gxml_gom_int_get_integer (GXmlGomInt* self);
void gxml_gom_int_set_integer (GXmlGomInt* self, gint value);
GXmlGomInt* gxml_gom_int_new (void);
GXmlGomInt* gxml_gom_int_construct (GType object_type);
GType gxml_gom_boolean_get_type (void) G_GNUC_CONST;
gboolean gxml_gom_boolean_get_boolean (GXmlGomBoolean* self);
void gxml_gom_boolean_set_boolean (GXmlGomBoolean* self, gboolean value);
GXmlGomBoolean* gxml_gom_boolean_new (void);
GXmlGomBoolean* gxml_gom_boolean_construct (GType object_type);
GType gxml_gom_enum_get_type (void) G_GNUC_CONST;
void gxml_gom_enum_initialize_enum (GXmlGomEnum* self, GType enum_type);
gint gxml_gom_enum_get_enum (GXmlGomEnum* self);
void gxml_gom_enum_set_enum (GXmlGomEnum* self, gint value);
GXmlGomEnum* gxml_gom_enum_new (void);
GXmlGomEnum* gxml_gom_enum_construct (GType object_type);
GType gxml_gom_enum_get_enum_type (GXmlGomEnum* self);
void gxml_gom_enum_set_enum_type (GXmlGomEnum* self, GType value);
GType gxml_gom_date_get_type (void) G_GNUC_CONST;
void gxml_gom_date_get_date (GXmlGomDate* self, GDate* result);
void gxml_gom_date_set_date (GXmlGomDate* self, GDate* date);
GXmlGomDate* gxml_gom_date_new (void);
GXmlGomDate* gxml_gom_date_construct (GType object_type);
GType gxml_gom_date_time_get_type (void) G_GNUC_CONST;
GDateTime* gxml_gom_date_time_get_datetime (GXmlGomDateTime* self);
void gxml_gom_date_time_set_datetime (GXmlGomDateTime* self, GDateTime* dt);
GXmlGomDateTime* gxml_gom_date_time_new (void);
GXmlGomDateTime* gxml_gom_date_time_construct (GType object_type);
const gchar* gxml_gom_date_time_get_format (GXmlGomDateTime* self);
void gxml_gom_date_time_set_format (GXmlGomDateTime* self, const gchar* value);
GQuark gxml_parser_error_quark (void);
GType gxml_parser_get_type (void) G_GNUC_CONST;
void gxml_parser_write_file (GXmlParser* self, GFile* file, GCancellable* cancellable, GError** error);
gchar* gxml_parser_write_string (GXmlParser* self, GError** error);
void gxml_parser_write_stream (GXmlParser* self, GOutputStream* stream, GCancellable* cancellable, GError** error);
void gxml_parser_read_file (GXmlParser* self, GFile* file, GCancellable* cancellable, GError** error);
void gxml_parser_read_stream (GXmlParser* self, GInputStream* stream, GCancellable* cancellable, GError** error);
void gxml_parser_read_string (GXmlParser* self, const gchar* str, GCancellable* cancellable, GError** error);
gboolean gxml_parser_get_backup (GXmlParser* self);
void gxml_parser_set_backup (GXmlParser* self, gboolean value);
gboolean gxml_parser_get_indent (GXmlParser* self);
void gxml_parser_set_indent (GXmlParser* self, gboolean value);
GXmlDomNode* gxml_parser_get_node (GXmlParser* self);
GType gxml_xparser_get_type (void) G_GNUC_CONST;
GXmlXParser* gxml_xparser_new (GXmlDomNode* node);
GXmlXParser* gxml_xparser_construct (GType object_type, GXmlDomNode* node);
void gxml_xparser_read_node (GXmlXParser* self, GXmlDomNode* node, GError** error);
gboolean gxml_xparser_move_next_node (GXmlXParser* self, GError** error);
gboolean gxml_xparser_current_is_empty_element (GXmlXParser* self);
gboolean gxml_xparser_current_is_element (GXmlXParser* self);
gboolean gxml_xparser_current_is_document (GXmlXParser* self);
gchar* gxml_xparser_current_node_name (GXmlXParser* self);
GXmlDomElement* gxml_xparser_create_element (GXmlXParser* self, GXmlDomNode* parent, GError** error);
void gxml_xparser_read_element (GXmlXParser* self, GXmlDomElement* element, GError** error);
void gxml_xparser_read_child_nodes (GXmlXParser* self, GXmlDomNode* parent, GError** error);
gboolean gxml_xparser_read_child_node (GXmlXParser* self, GXmlDomNode* parent, GError** error);
gboolean gxml_xparser_read_child_element (GXmlXParser* self, GXmlDomNode* parent, GError** error);
gboolean gxml_xparser_read_element_property (GXmlXParser* self, GXmlDomNode* parent, GXmlDomNode** element, GError** error);
gboolean gxml_xparser_add_element_collection (GXmlXParser* self, GXmlDomNode* parent, GXmlDomNode** element, GError** error);
GType gxml_ixsd_list_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_list_elements_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_list_simple_types_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_list_complex_types_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_schema_get_type (void) G_GNUC_CONST;
#define GXML_IXSD_SCHEMA_SCHEMA_NODE_NAME "schema"
#define GXML_IXSD_SCHEMA_SCHEMA_NAMESPACE_URI "http://www.w3.org/2001/XMLSchema"
#define GXML_IXSD_SCHEMA_SCHEMA_NAMESPACE_PREFIX "xs"
GXmlIXsdListElements* gxml_ixsd_schema_get_element_definitions (GXmlIXsdSchema* self);
void gxml_ixsd_schema_set_element_definitions (GXmlIXsdSchema* self, GXmlIXsdListElements* value);
GXmlIXsdListSimpleTypes* gxml_ixsd_schema_get_simple_type_definitions (GXmlIXsdSchema* self);
void gxml_ixsd_schema_set_simple_type_definitions (GXmlIXsdSchema* self, GXmlIXsdListSimpleTypes* value);
GXmlIXsdListComplexTypes* gxml_ixsd_schema_get_complex_type_definitions (GXmlIXsdSchema* self);
void gxml_ixsd_schema_set_complex_type_definitions (GXmlIXsdSchema* self, GXmlIXsdListComplexTypes* value);
GQuark gxml_ixsd_schema_error_quark (void);
GType gxml_ixsd_annotation_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_base_type_get_type (void) G_GNUC_CONST;
GXmlIXsdAnnotation* gxml_ixsd_base_type_get_anotation (GXmlIXsdBaseType* self);
void gxml_ixsd_base_type_set_anotation (GXmlIXsdBaseType* self, GXmlIXsdAnnotation* value);
GType gxml_ixsd_type_def_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_type_list_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_type_union_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_list_type_restriction_enumerations_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_list_type_restriction_white_spaces_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_type_restriction_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_simple_type_get_type (void) G_GNUC_CONST;
#define GXML_IXSD_SIMPLE_TYPE_SCHEMA_NODE_NAME "simpleType"
const gchar* gxml_ixsd_simple_type_get_final (GXmlIXsdSimpleType* self);
void gxml_ixsd_simple_type_set_final (GXmlIXsdSimpleType* self, const gchar* value);
const gchar* gxml_ixsd_simple_type_get_id (GXmlIXsdSimpleType* self);
void gxml_ixsd_simple_type_set_id (GXmlIXsdSimpleType* self, const gchar* value);
const gchar* gxml_ixsd_simple_type_get_name (GXmlIXsdSimpleType* self);
void gxml_ixsd_simple_type_set_name (GXmlIXsdSimpleType* self, const gchar* value);
GXmlIXsdAnnotation* gxml_ixsd_simple_type_get_annotation (GXmlIXsdSimpleType* self);
void gxml_ixsd_simple_type_set_annotation (GXmlIXsdSimpleType* self, GXmlIXsdAnnotation* value);
GXmlIXsdTypeList* gxml_ixsd_simple_type_get_list (GXmlIXsdSimpleType* self);
void gxml_ixsd_simple_type_set_list (GXmlIXsdSimpleType* self, GXmlIXsdTypeList* value);
GXmlIXsdTypeUnion* gxml_ixsd_simple_type_get_union (GXmlIXsdSimpleType* self);
void gxml_ixsd_simple_type_set_union (GXmlIXsdSimpleType* self, GXmlIXsdTypeUnion* value);
GXmlIXsdTypeRestriction* gxml_ixsd_simple_type_get_restriction (GXmlIXsdSimpleType* self);
void gxml_ixsd_simple_type_set_restriction (GXmlIXsdSimpleType* self, GXmlIXsdTypeRestriction* value);
#define GXML_IXSD_TYPE_RESTRICTION_SCHEMA_NODE_NAME "restriction"
const gchar* gxml_ixsd_type_restriction_get_base (GXmlIXsdTypeRestriction* self);
void gxml_ixsd_type_restriction_set_base (GXmlIXsdTypeRestriction* self, const gchar* value);
const gchar* gxml_ixsd_type_restriction_get_id (GXmlIXsdTypeRestriction* self);
void gxml_ixsd_type_restriction_set_id (GXmlIXsdTypeRestriction* self, const gchar* value);
GXmlIXsdSimpleType* gxml_ixsd_type_restriction_get_simple_type (GXmlIXsdTypeRestriction* self);
void gxml_ixsd_type_restriction_set_simple_type (GXmlIXsdTypeRestriction* self, GXmlIXsdSimpleType* value);
GXmlIXsdListTypeRestrictionEnumerations* gxml_ixsd_type_restriction_get_enumerations (GXmlIXsdTypeRestriction* self);
void gxml_ixsd_type_restriction_set_enumerations (GXmlIXsdTypeRestriction* self, GXmlIXsdListTypeRestrictionEnumerations* value);
GXmlIXsdListTypeRestrictionWhiteSpaces* gxml_ixsd_type_restriction_get_white_spaces (GXmlIXsdTypeRestriction* self);
void gxml_ixsd_type_restriction_set_white_spaces (GXmlIXsdTypeRestriction* self, GXmlIXsdListTypeRestrictionWhiteSpaces* value);
GType gxml_ixsd_type_restriction_def_get_type (void) G_GNUC_CONST;
GXmlIXsdAnnotation* gxml_ixsd_type_restriction_def_get_annotation (GXmlIXsdTypeRestrictionDef* self);
void gxml_ixsd_type_restriction_def_set_annotation (GXmlIXsdTypeRestrictionDef* self, GXmlIXsdAnnotation* value);
GType gxml_ixsd_type_restriction_min_exclusive_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_type_restriction_min_inclusive_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_type_restriction_max_exclusive_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_type_restriction_max_inclusive_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_type_restriction_total_digits_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_type_restriction_fraction_digits_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_type_restriction_length_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_type_restriction_min_length_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_type_restriction_max_length_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_type_restriction_enumeration_get_type (void) G_GNUC_CONST;
#define GXML_IXSD_TYPE_RESTRICTION_ENUMERATION_SCHEMA_NODE_NAME "enumeration"
const gchar* gxml_ixsd_type_restriction_enumeration_get_id (GXmlIXsdTypeRestrictionEnumeration* self);
void gxml_ixsd_type_restriction_enumeration_set_id (GXmlIXsdTypeRestrictionEnumeration* self, const gchar* value);
const gchar* gxml_ixsd_type_restriction_enumeration_get_value (GXmlIXsdTypeRestrictionEnumeration* self);
void gxml_ixsd_type_restriction_enumeration_set_value (GXmlIXsdTypeRestrictionEnumeration* self, const gchar* value);
GType gxml_ixsd_type_restriction_white_space_get_type (void) G_GNUC_CONST;
#define GXML_IXSD_TYPE_RESTRICTION_WHITE_SPACE_SCHEMA_NODE_NAME "whiteSpace"
gboolean gxml_ixsd_type_restriction_white_space_get_fixed (GXmlIXsdTypeRestrictionWhiteSpace* self);
void gxml_ixsd_type_restriction_white_space_set_fixed (GXmlIXsdTypeRestrictionWhiteSpace* self, gboolean value);
const gchar* gxml_ixsd_type_restriction_white_space_get_id (GXmlIXsdTypeRestrictionWhiteSpace* self);
void gxml_ixsd_type_restriction_white_space_set_id (GXmlIXsdTypeRestrictionWhiteSpace* self, const gchar* value);
const gchar* gxml_ixsd_type_restriction_white_space_get_value (GXmlIXsdTypeRestrictionWhiteSpace* self);
void gxml_ixsd_type_restriction_white_space_set_value (GXmlIXsdTypeRestrictionWhiteSpace* self, const gchar* value);
GType gxml_ixsd_type_restriction_pattern_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_type_restriction_assertion_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_type_restriction_explicit_timezone_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_base_content_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_list_attributes_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_list_attributes_group_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_complex_type_get_type (void) G_GNUC_CONST;
#define GXML_IXSD_COMPLEX_TYPE_SCHEMA_NODE_NAME "complexType"
gboolean gxml_ixsd_complex_type_get_abstract (GXmlIXsdComplexType* self);
void gxml_ixsd_complex_type_set_abstract (GXmlIXsdComplexType* self, gboolean value);
const gchar* gxml_ixsd_complex_type_get_block (GXmlIXsdComplexType* self);
void gxml_ixsd_complex_type_set_block (GXmlIXsdComplexType* self, const gchar* value);
const gchar* gxml_ixsd_complex_type_get_final (GXmlIXsdComplexType* self);
void gxml_ixsd_complex_type_set_final (GXmlIXsdComplexType* self, const gchar* value);
gboolean gxml_ixsd_complex_type_get_mixed (GXmlIXsdComplexType* self);
void gxml_ixsd_complex_type_set_mixed (GXmlIXsdComplexType* self, gboolean value);
const gchar* gxml_ixsd_complex_type_get_name (GXmlIXsdComplexType* self);
void gxml_ixsd_complex_type_set_name (GXmlIXsdComplexType* self, const gchar* value);
gboolean gxml_ixsd_complex_type_get_default_attributes_apply (GXmlIXsdComplexType* self);
void gxml_ixsd_complex_type_set_default_attributes_apply (GXmlIXsdComplexType* self, gboolean value);
GXmlIXsdBaseContent* gxml_ixsd_complex_type_get_content_type (GXmlIXsdComplexType* self);
void gxml_ixsd_complex_type_set_content_type (GXmlIXsdComplexType* self, GXmlIXsdBaseContent* value);
GXmlIXsdListAttributes* gxml_ixsd_complex_type_get_type_attributes (GXmlIXsdComplexType* self);
GXmlIXsdListAttributesGroup* gxml_ixsd_complex_type_get_group_attributes (GXmlIXsdComplexType* self);
GType gxml_ixsd_extension_get_type (void) G_GNUC_CONST;
#define GXML_IXSD_EXTENSION_SCHEMA_NODE_NAME "extension"
const gchar* gxml_ixsd_extension_get_base (GXmlIXsdExtension* self);
void gxml_ixsd_extension_set_base (GXmlIXsdExtension* self, const gchar* value);
GType gxml_ixsd_element_get_type (void) G_GNUC_CONST;
#define GXML_IXSD_ELEMENT_SCHEMA_NODE_NAME "element"
gboolean gxml_ixsd_element_get_abstract (GXmlIXsdElement* self);
void gxml_ixsd_element_set_abstract (GXmlIXsdElement* self, gboolean value);
const gchar* gxml_ixsd_element_get_block (GXmlIXsdElement* self);
void gxml_ixsd_element_set_block (GXmlIXsdElement* self, const gchar* value);
const gchar* gxml_ixsd_element_get_default (GXmlIXsdElement* self);
void gxml_ixsd_element_set_default (GXmlIXsdElement* self, const gchar* value);
const gchar* gxml_ixsd_element_get_final (GXmlIXsdElement* self);
void gxml_ixsd_element_set_final (GXmlIXsdElement* self, const gchar* value);
const gchar* gxml_ixsd_element_get_fixed (GXmlIXsdElement* self);
void gxml_ixsd_element_set_fixed (GXmlIXsdElement* self, const gchar* value);
const gchar* gxml_ixsd_element_get_form (GXmlIXsdElement* self);
void gxml_ixsd_element_set_form (GXmlIXsdElement* self, const gchar* value);
const gchar* gxml_ixsd_element_get_id (GXmlIXsdElement* self);
void gxml_ixsd_element_set_id (GXmlIXsdElement* self, const gchar* value);
const gchar* gxml_ixsd_element_get_maxOccurs (GXmlIXsdElement* self);
void gxml_ixsd_element_set_maxOccurs (GXmlIXsdElement* self, const gchar* value);
const gchar* gxml_ixsd_element_get_minOccurs (GXmlIXsdElement* self);
void gxml_ixsd_element_set_minOccurs (GXmlIXsdElement* self, const gchar* value);
const gchar* gxml_ixsd_element_get_name (GXmlIXsdElement* self);
void gxml_ixsd_element_set_name (GXmlIXsdElement* self, const gchar* value);
gboolean gxml_ixsd_element_get_nillable (GXmlIXsdElement* self);
void gxml_ixsd_element_set_nillable (GXmlIXsdElement* self, gboolean value);
const gchar* gxml_ixsd_element_get_ref (GXmlIXsdElement* self);
void gxml_ixsd_element_set_ref (GXmlIXsdElement* self, const gchar* value);
GXmlDomTokenList* gxml_ixsd_element_get_substitution_group (GXmlIXsdElement* self);
void gxml_ixsd_element_set_substitution_group (GXmlIXsdElement* self, GXmlDomTokenList* value);
const gchar* gxml_ixsd_element_get_target_namespace (GXmlIXsdElement* self);
void gxml_ixsd_element_set_target_namespace (GXmlIXsdElement* self, const gchar* value);
const gchar* gxml_ixsd_element_get_object_type (GXmlIXsdElement* self);
void gxml_ixsd_element_set_object_type (GXmlIXsdElement* self, const gchar* value);
GXmlIXsdAnnotation* gxml_ixsd_element_get_anotation (GXmlIXsdElement* self);
void gxml_ixsd_element_set_anotation (GXmlIXsdElement* self, GXmlIXsdAnnotation* value);
GXmlIXsdSimpleType* gxml_ixsd_element_get_simple_type (GXmlIXsdElement* self);
void gxml_ixsd_element_set_simple_type (GXmlIXsdElement* self, GXmlIXsdSimpleType* value);
GXmlIXsdComplexType* gxml_ixsd_element_get_complex_type (GXmlIXsdElement* self);
void gxml_ixsd_element_set_complex_type (GXmlIXsdElement* self, GXmlIXsdComplexType* value);
GXmlIXsdAnnotation* gxml_ixsd_base_content_get_anotation (GXmlIXsdBaseContent* self);
void gxml_ixsd_base_content_set_anotation (GXmlIXsdBaseContent* self, GXmlIXsdAnnotation* value);
GType gxml_ixsd_simple_content_get_type (void) G_GNUC_CONST;
#define GXML_IXSD_SIMPLE_CONTENT_SCHEMA_NODE_NAME "simpleContent"
GType gxml_ixsd_complex_content_get_type (void) G_GNUC_CONST;
#define GXML_IXSD_COMPLEX_CONTENT_SCHEMA_NODE_NAME "complexContent"
GType gxml_ixsd_open_content_get_type (void) G_GNUC_CONST;
GType gxml_ixsd_base_attribute_get_type (void) G_GNUC_CONST;
GXmlIXsdAnnotation* gxml_ixsd_base_attribute_get_anotation (GXmlIXsdBaseAttribute* self);
void gxml_ixsd_base_attribute_set_anotation (GXmlIXsdBaseAttribute* self, GXmlIXsdAnnotation* value);
GType gxml_ixsd_attribute_get_type (void) G_GNUC_CONST;
#define GXML_IXSD_ATTRIBUTE_SCHEMA_NODE_NAME "attribute"
GType gxml_ixsd_attribute_group_get_type (void) G_GNUC_CONST;
#define GXML_IXSD_ATTRIBUTE_GROUP_SCHEMA_NODE_NAME "attributeGroup"
GXmlDomElement* gxml_ixsd_list_get_item (GXmlIXsdList* self, gint index);
void gxml_ixsd_list_append (GXmlIXsdList* self, GXmlDomElement* element);
void gxml_ixsd_list_remove (GXmlIXsdList* self, gint index);
gint gxml_ixsd_list_index_of (GXmlIXsdList* self, GXmlDomElement* element);
GXmlDomElement* gxml_ixsd_list_get_element (GXmlIXsdList* self);
void gxml_ixsd_list_set_element (GXmlIXsdList* self, GXmlDomElement* value);
GType gxml_ixsd_list_get_items_type (GXmlIXsdList* self);
void gxml_ixsd_list_set_items_type (GXmlIXsdList* self, GType value);
GType gxml_ixsd_list_get_items_name (GXmlIXsdList* self);
void gxml_ixsd_list_set_items_name (GXmlIXsdList* self, GType value);
gint gxml_ixsd_list_get_length (GXmlIXsdList* self);
GType gxml_gom_xsd_schema_get_type (void) G_GNUC_CONST;
GXmlGomXsdSchema* gxml_gom_xsd_schema_new (void);
GXmlGomXsdSchema* gxml_gom_xsd_schema_construct (GType object_type);
GType gxml_gom_xsd_list_get_type (void) G_GNUC_CONST;
GType gxml_gom_xsd_list_elements_get_type (void) G_GNUC_CONST;
GXmlGomXsdListElements* gxml_gom_xsd_schema_get_element_definitions (GXmlGomXsdSchema* self);
void gxml_gom_xsd_schema_set_element_definitions (GXmlGomXsdSchema* self, GXmlGomXsdListElements* value);
GType gxml_gom_xsd_list_simple_types_get_type (void) G_GNUC_CONST;
GXmlGomXsdListSimpleTypes* gxml_gom_xsd_schema_get_simple_type_definitions (GXmlGomXsdSchema* self);
void gxml_gom_xsd_schema_set_simple_type_definitions (GXmlGomXsdSchema* self, GXmlGomXsdListSimpleTypes* value);
GType gxml_gom_xsd_list_complex_types_get_type (void) G_GNUC_CONST;
GXmlGomXsdListComplexTypes* gxml_gom_xsd_schema_get_complex_type_definitions (GXmlGomXsdSchema* self);
void gxml_gom_xsd_schema_set_complex_type_definitions (GXmlGomXsdSchema* self, GXmlGomXsdListComplexTypes* value);
GType gxml_gom_xsd_simple_type_get_type (void) G_GNUC_CONST;
GXmlGomXsdSimpleType* gxml_gom_xsd_simple_type_new (void);
GXmlGomXsdSimpleType* gxml_gom_xsd_simple_type_construct (GType object_type);
const gchar* gxml_gom_xsd_simple_type_get_final (GXmlGomXsdSimpleType* self);
void gxml_gom_xsd_simple_type_set_final (GXmlGomXsdSimpleType* self, const gchar* value);
const gchar* gxml_gom_xsd_simple_type_get_name (GXmlGomXsdSimpleType* self);
void gxml_gom_xsd_simple_type_set_name (GXmlGomXsdSimpleType* self, const gchar* value);
GType gxml_gom_xsd_annotation_get_type (void) G_GNUC_CONST;
GXmlGomXsdAnnotation* gxml_gom_xsd_simple_type_get_annotation (GXmlGomXsdSimpleType* self);
void gxml_gom_xsd_simple_type_set_annotation (GXmlGomXsdSimpleType* self, GXmlGomXsdAnnotation* value);
GType gxml_gom_xsd_type_definition_get_type (void) G_GNUC_CONST;
GType gxml_gom_xsd_type_list_get_type (void) G_GNUC_CONST;
GXmlGomXsdTypeList* gxml_gom_xsd_simple_type_get_list (GXmlGomXsdSimpleType* self);
void gxml_gom_xsd_simple_type_set_list (GXmlGomXsdSimpleType* self, GXmlGomXsdTypeList* value);
GType gxml_gom_xsd_type_union_get_type (void) G_GNUC_CONST;
GXmlGomXsdTypeUnion* gxml_gom_xsd_simple_type_get_union (GXmlGomXsdSimpleType* self);
void gxml_gom_xsd_simple_type_set_union (GXmlGomXsdSimpleType* self, GXmlGomXsdTypeUnion* value);
GType gxml_gom_xsd_type_restriction_get_type (void) G_GNUC_CONST;
GXmlGomXsdTypeRestriction* gxml_gom_xsd_simple_type_get_restriction (GXmlGomXsdSimpleType* self);
void gxml_gom_xsd_simple_type_set_restriction (GXmlGomXsdSimpleType* self, GXmlGomXsdTypeRestriction* value);
GXmlGomXsdTypeDefinition* gxml_gom_xsd_type_definition_new (void);
GXmlGomXsdTypeDefinition* gxml_gom_xsd_type_definition_construct (GType object_type);
GXmlGomXsdAnnotation* gxml_gom_xsd_type_definition_get_annotation (GXmlGomXsdTypeDefinition* self);
void gxml_gom_xsd_type_definition_set_annotation (GXmlGomXsdTypeDefinition* self, GXmlGomXsdAnnotation* value);
GXmlGomXsdTypeList* gxml_gom_xsd_type_list_new (void);
GXmlGomXsdTypeList* gxml_gom_xsd_type_list_construct (GType object_type);
GXmlGomXsdTypeUnion* gxml_gom_xsd_type_union_new (void);
GXmlGomXsdTypeUnion* gxml_gom_xsd_type_union_construct (GType object_type);
GXmlGomXsdTypeRestriction* gxml_gom_xsd_type_restriction_new (void);
GXmlGomXsdTypeRestriction* gxml_gom_xsd_type_restriction_construct (GType object_type);
const gchar* gxml_gom_xsd_type_restriction_get_base (GXmlGomXsdTypeRestriction* self);
void gxml_gom_xsd_type_restriction_set_base (GXmlGomXsdTypeRestriction* self, const gchar* value);
GXmlGomXsdSimpleType* gxml_gom_xsd_type_restriction_get_simple_type (GXmlGomXsdTypeRestriction* self);
void gxml_gom_xsd_type_restriction_set_simple_type (GXmlGomXsdTypeRestriction* self, GXmlGomXsdSimpleType* value);
GType gxml_gom_xsd_list_type_restriction_enumerations_get_type (void) G_GNUC_CONST;
GXmlGomXsdListTypeRestrictionEnumerations* gxml_gom_xsd_type_restriction_get_enumerations (GXmlGomXsdTypeRestriction* self);
void gxml_gom_xsd_type_restriction_set_enumerations (GXmlGomXsdTypeRestriction* self, GXmlGomXsdListTypeRestrictionEnumerations* value);
GType gxml_gom_xsd_list_type_restriction_white_spaces_get_type (void) G_GNUC_CONST;
GXmlGomXsdListTypeRestrictionWhiteSpaces* gxml_gom_xsd_type_restriction_get_white_spaces (GXmlGomXsdTypeRestriction* self);
void gxml_gom_xsd_type_restriction_set_white_spaces (GXmlGomXsdTypeRestriction* self, GXmlGomXsdListTypeRestrictionWhiteSpaces* value);
GType gxml_gom_xsd_type_restriction_def_get_type (void) G_GNUC_CONST;
GXmlGomXsdTypeRestrictionDef* gxml_gom_xsd_type_restriction_def_new (void);
GXmlGomXsdTypeRestrictionDef* gxml_gom_xsd_type_restriction_def_construct (GType object_type);
GXmlGomXsdAnnotation* gxml_gom_xsd_type_restriction_def_get_annotation (GXmlGomXsdTypeRestrictionDef* self);
void gxml_gom_xsd_type_restriction_def_set_annotation (GXmlGomXsdTypeRestrictionDef* self, GXmlGomXsdAnnotation* value);
GType gxml_gom_xsd_type_restriction_min_exclusive_get_type (void) G_GNUC_CONST;
GXmlGomXsdTypeRestrictionMinExclusive* gxml_gom_xsd_type_restriction_min_exclusive_new (void);
GXmlGomXsdTypeRestrictionMinExclusive* gxml_gom_xsd_type_restriction_min_exclusive_construct (GType object_type);
GType gxml_gom_xsd_type_restriction_min_inclusive_get_type (void) G_GNUC_CONST;
GXmlGomXsdTypeRestrictionMinInclusive* gxml_gom_xsd_type_restriction_min_inclusive_new (void);
GXmlGomXsdTypeRestrictionMinInclusive* gxml_gom_xsd_type_restriction_min_inclusive_construct (GType object_type);
GType gxml_gom_xsd_type_restriction_max_exclusive_get_type (void) G_GNUC_CONST;
GXmlGomXsdTypeRestrictionMaxExclusive* gxml_gom_xsd_type_restriction_max_exclusive_new (void);
GXmlGomXsdTypeRestrictionMaxExclusive* gxml_gom_xsd_type_restriction_max_exclusive_construct (GType object_type);
GType gxml_gom_xsd_type_restriction_max_inclusive_get_type (void) G_GNUC_CONST;
GXmlGomXsdTypeRestrictionMaxInclusive* gxml_gom_xsd_type_restriction_max_inclusive_new (void);
GXmlGomXsdTypeRestrictionMaxInclusive* gxml_gom_xsd_type_restriction_max_inclusive_construct (GType object_type);
GType gxml_gom_xsd_type_restriction_total_digits_get_type (void) G_GNUC_CONST;
GXmlGomXsdTypeRestrictionTotalDigits* gxml_gom_xsd_type_restriction_total_digits_new (void);
GXmlGomXsdTypeRestrictionTotalDigits* gxml_gom_xsd_type_restriction_total_digits_construct (GType object_type);
GType gxml_gom_xsd_type_restriction_fraction_digits_get_type (void) G_GNUC_CONST;
GXmlGomXsdTypeRestrictionFractionDigits* gxml_gom_xsd_type_restriction_fraction_digits_new (void);
GXmlGomXsdTypeRestrictionFractionDigits* gxml_gom_xsd_type_restriction_fraction_digits_construct (GType object_type);
GType gxml_gom_xsd_type_restriction_length_get_type (void) G_GNUC_CONST;
GXmlGomXsdTypeRestrictionLength* gxml_gom_xsd_type_restriction_length_new (void);
GXmlGomXsdTypeRestrictionLength* gxml_gom_xsd_type_restriction_length_construct (GType object_type);
GType gxml_gom_xsd_type_restriction_min_length_get_type (void) G_GNUC_CONST;
GXmlGomXsdTypeRestrictionMinLength* gxml_gom_xsd_type_restriction_min_length_new (void);
GXmlGomXsdTypeRestrictionMinLength* gxml_gom_xsd_type_restriction_min_length_construct (GType object_type);
GType gxml_gom_xsd_type_restriction_max_length_get_type (void) G_GNUC_CONST;
GXmlGomXsdTypeRestrictionMaxLength* gxml_gom_xsd_type_restriction_max_length_new (void);
GXmlGomXsdTypeRestrictionMaxLength* gxml_gom_xsd_type_restriction_max_length_construct (GType object_type);
GType gxml_gom_xsd_type_restriction_enumeration_get_type (void) G_GNUC_CONST;
GXmlGomXsdTypeRestrictionEnumeration* gxml_gom_xsd_type_restriction_enumeration_new (void);
GXmlGomXsdTypeRestrictionEnumeration* gxml_gom_xsd_type_restriction_enumeration_construct (GType object_type);
const gchar* gxml_gom_xsd_type_restriction_enumeration_get_value (GXmlGomXsdTypeRestrictionEnumeration* self);
void gxml_gom_xsd_type_restriction_enumeration_set_value (GXmlGomXsdTypeRestrictionEnumeration* self, const gchar* value);
GType gxml_gom_xsd_type_restriction_white_space_get_type (void) G_GNUC_CONST;
GXmlGomXsdTypeRestrictionWhiteSpace* gxml_gom_xsd_type_restriction_white_space_new (void);
GXmlGomXsdTypeRestrictionWhiteSpace* gxml_gom_xsd_type_restriction_white_space_construct (GType object_type);
GType gxml_gom_xsd_type_restriction_white_space_fixed_get_type (void) G_GNUC_CONST;
GXmlGomXsdTypeRestrictionWhiteSpaceFixed* gxml_gom_xsd_type_restriction_white_space_get_fixed (GXmlGomXsdTypeRestrictionWhiteSpace* self);
void gxml_gom_xsd_type_restriction_white_space_set_fixed (GXmlGomXsdTypeRestrictionWhiteSpace* self, GXmlGomXsdTypeRestrictionWhiteSpaceFixed* value);
const gchar* gxml_gom_xsd_type_restriction_white_space_get_value (GXmlGomXsdTypeRestrictionWhiteSpace* self);
void gxml_gom_xsd_type_restriction_white_space_set_value (GXmlGomXsdTypeRestrictionWhiteSpace* self, const gchar* value);
GXmlGomXsdTypeRestrictionWhiteSpaceFixed* gxml_gom_xsd_type_restriction_white_space_fixed_new (void);
GXmlGomXsdTypeRestrictionWhiteSpaceFixed* gxml_gom_xsd_type_restriction_white_space_fixed_construct (GType object_type);
GType gxml_gom_xsd_type_restriction_pattern_get_type (void) G_GNUC_CONST;
GXmlGomXsdTypeRestrictionPattern* gxml_gom_xsd_type_restriction_pattern_new (void);
GXmlGomXsdTypeRestrictionPattern* gxml_gom_xsd_type_restriction_pattern_construct (GType object_type);
GType gxml_gom_xsd_type_restriction_assertion_get_type (void) G_GNUC_CONST;
GXmlGomXsdTypeRestrictionAssertion* gxml_gom_xsd_type_restriction_assertion_new (void);
GXmlGomXsdTypeRestrictionAssertion* gxml_gom_xsd_type_restriction_assertion_construct (GType object_type);
GType gxml_gom_xsd_type_restriction_explicit_timezone_get_type (void) G_GNUC_CONST;
GXmlGomXsdTypeRestrictionExplicitTimezone* gxml_gom_xsd_type_restriction_explicit_timezone_new (void);
GXmlGomXsdTypeRestrictionExplicitTimezone* gxml_gom_xsd_type_restriction_explicit_timezone_construct (GType object_type);
GType gxml_gom_xsd_base_type_get_type (void) G_GNUC_CONST;
GType gxml_gom_xsd_complex_type_get_type (void) G_GNUC_CONST;
GXmlGomXsdComplexType* gxml_gom_xsd_complex_type_new (void);
GXmlGomXsdComplexType* gxml_gom_xsd_complex_type_construct (GType object_type);
gboolean gxml_gom_xsd_complex_type_get_abstract (GXmlGomXsdComplexType* self);
void gxml_gom_xsd_complex_type_set_abstract (GXmlGomXsdComplexType* self, gboolean value);
const gchar* gxml_gom_xsd_complex_type_get_block (GXmlGomXsdComplexType* self);
void gxml_gom_xsd_complex_type_set_block (GXmlGomXsdComplexType* self, const gchar* value);
const gchar* gxml_gom_xsd_complex_type_get_final (GXmlGomXsdComplexType* self);
void gxml_gom_xsd_complex_type_set_final (GXmlGomXsdComplexType* self, const gchar* value);
gboolean gxml_gom_xsd_complex_type_get_mixed (GXmlGomXsdComplexType* self);
void gxml_gom_xsd_complex_type_set_mixed (GXmlGomXsdComplexType* self, gboolean value);
const gchar* gxml_gom_xsd_complex_type_get_name (GXmlGomXsdComplexType* self);
void gxml_gom_xsd_complex_type_set_name (GXmlGomXsdComplexType* self, const gchar* value);
gboolean gxml_gom_xsd_complex_type_get_default_attributes_apply (GXmlGomXsdComplexType* self);
void gxml_gom_xsd_complex_type_set_default_attributes_apply (GXmlGomXsdComplexType* self, gboolean value);
GType gxml_gom_xsd_base_content_get_type (void) G_GNUC_CONST;
GXmlGomXsdBaseContent* gxml_gom_xsd_complex_type_get_content_type (GXmlGomXsdComplexType* self);
void gxml_gom_xsd_complex_type_set_content_type (GXmlGomXsdComplexType* self, GXmlGomXsdBaseContent* value);
GXmlGomXsdList* gxml_gom_xsd_complex_type_get_type_attributes (GXmlGomXsdComplexType* self);
GXmlGomXsdList* gxml_gom_xsd_complex_type_get_group_attributes (GXmlGomXsdComplexType* self);
GType gxml_gom_xsd_extension_get_type (void) G_GNUC_CONST;
GXmlGomXsdExtension* gxml_gom_xsd_extension_new (void);
GXmlGomXsdExtension* gxml_gom_xsd_extension_construct (GType object_type);
const gchar* gxml_gom_xsd_extension_get_base (GXmlGomXsdExtension* self);
void gxml_gom_xsd_extension_set_base (GXmlGomXsdExtension* self, const gchar* value);
GType gxml_gom_xsd_element_get_type (void) G_GNUC_CONST;
GXmlGomXsdElement* gxml_gom_xsd_element_new (void);
GXmlGomXsdElement* gxml_gom_xsd_element_construct (GType object_type);
gboolean gxml_gom_xsd_element_get_abstract (GXmlGomXsdElement* self);
void gxml_gom_xsd_element_set_abstract (GXmlGomXsdElement* self, gboolean value);
const gchar* gxml_gom_xsd_element_get_block (GXmlGomXsdElement* self);
void gxml_gom_xsd_element_set_block (GXmlGomXsdElement* self, const gchar* value);
const gchar* gxml_gom_xsd_element_get_default (GXmlGomXsdElement* self);
void gxml_gom_xsd_element_set_default (GXmlGomXsdElement* self, const gchar* value);
const gchar* gxml_gom_xsd_element_get_final (GXmlGomXsdElement* self);
void gxml_gom_xsd_element_set_final (GXmlGomXsdElement* self, const gchar* value);
const gchar* gxml_gom_xsd_element_get_fixed (GXmlGomXsdElement* self);
void gxml_gom_xsd_element_set_fixed (GXmlGomXsdElement* self, const gchar* value);
const gchar* gxml_gom_xsd_element_get_form (GXmlGomXsdElement* self);
void gxml_gom_xsd_element_set_form (GXmlGomXsdElement* self, const gchar* value);
const gchar* gxml_gom_xsd_element_get_max_occurs (GXmlGomXsdElement* self);
void gxml_gom_xsd_element_set_max_occurs (GXmlGomXsdElement* self, const gchar* value);
const gchar* gxml_gom_xsd_element_get_min_occurs (GXmlGomXsdElement* self);
void gxml_gom_xsd_element_set_min_occurs (GXmlGomXsdElement* self, const gchar* value);
const gchar* gxml_gom_xsd_element_get_name (GXmlGomXsdElement* self);
void gxml_gom_xsd_element_set_name (GXmlGomXsdElement* self, const gchar* value);
gboolean gxml_gom_xsd_element_get_nillable (GXmlGomXsdElement* self);
void gxml_gom_xsd_element_set_nillable (GXmlGomXsdElement* self, gboolean value);
const gchar* gxml_gom_xsd_element_get_ref (GXmlGomXsdElement* self);
void gxml_gom_xsd_element_set_ref (GXmlGomXsdElement* self, const gchar* value);
GXmlDomTokenList* gxml_gom_xsd_element_get_substitution_group (GXmlGomXsdElement* self);
void gxml_gom_xsd_element_set_substitution_group (GXmlGomXsdElement* self, GXmlDomTokenList* value);
const gchar* gxml_gom_xsd_element_get_target_namespace (GXmlGomXsdElement* self);
void gxml_gom_xsd_element_set_target_namespace (GXmlGomXsdElement* self, const gchar* value);
const gchar* gxml_gom_xsd_element_get_object_type (GXmlGomXsdElement* self);
void gxml_gom_xsd_element_set_object_type (GXmlGomXsdElement* self, const gchar* value);
GXmlGomXsdAnnotation* gxml_gom_xsd_element_get_anotation (GXmlGomXsdElement* self);
void gxml_gom_xsd_element_set_anotation (GXmlGomXsdElement* self, GXmlGomXsdAnnotation* value);
GXmlGomXsdSimpleType* gxml_gom_xsd_element_get_simple_type (GXmlGomXsdElement* self);
void gxml_gom_xsd_element_set_simple_type (GXmlGomXsdElement* self, GXmlGomXsdSimpleType* value);
GXmlGomXsdComplexType* gxml_gom_xsd_element_get_complex_type (GXmlGomXsdElement* self);
void gxml_gom_xsd_element_set_complex_type (GXmlGomXsdElement* self, GXmlGomXsdComplexType* value);
GXmlGomXsdAnnotation* gxml_gom_xsd_annotation_new (void);
GXmlGomXsdAnnotation* gxml_gom_xsd_annotation_construct (GType object_type);
GXmlGomXsdBaseType* gxml_gom_xsd_base_type_new (void);
GXmlGomXsdBaseType* gxml_gom_xsd_base_type_construct (GType object_type);
GXmlGomXsdAnnotation* gxml_gom_xsd_base_type_get_anotation (GXmlGomXsdBaseType* self);
void gxml_gom_xsd_base_type_set_anotation (GXmlGomXsdBaseType* self, GXmlGomXsdAnnotation* value);
GXmlGomXsdBaseContent* gxml_gom_xsd_base_content_new (void);
GXmlGomXsdBaseContent* gxml_gom_xsd_base_content_construct (GType object_type);
GXmlGomXsdAnnotation* gxml_gom_xsd_base_content_get_anotation (GXmlGomXsdBaseContent* self);
void gxml_gom_xsd_base_content_set_anotation (GXmlGomXsdBaseContent* self, GXmlGomXsdAnnotation* value);
GType gxml_gom_xsd_simple_content_get_type (void) G_GNUC_CONST;
GXmlGomXsdSimpleContent* gxml_gom_xsd_simple_content_new (void);
GXmlGomXsdSimpleContent* gxml_gom_xsd_simple_content_construct (GType object_type);
GType gxml_gom_xsd_complex_content_get_type (void) G_GNUC_CONST;
GXmlGomXsdComplexContent* gxml_gom_xsd_complex_content_new (void);
GXmlGomXsdComplexContent* gxml_gom_xsd_complex_content_construct (GType object_type);
GType gxml_gom_xsd_open_content_get_type (void) G_GNUC_CONST;
GXmlGomXsdOpenContent* gxml_gom_xsd_open_content_new (void);
GXmlGomXsdOpenContent* gxml_gom_xsd_open_content_construct (GType object_type);
GType gxml_gom_xsd_base_attribute_get_type (void) G_GNUC_CONST;
GXmlGomXsdBaseAttribute* gxml_gom_xsd_base_attribute_new (void);
GXmlGomXsdBaseAttribute* gxml_gom_xsd_base_attribute_construct (GType object_type);
GXmlGomXsdAnnotation* gxml_gom_xsd_base_attribute_get_anotation (GXmlGomXsdBaseAttribute* self);
void gxml_gom_xsd_base_attribute_set_anotation (GXmlGomXsdBaseAttribute* self, GXmlGomXsdAnnotation* value);
GType gxml_gom_xsd_attribute_get_type (void) G_GNUC_CONST;
GXmlGomXsdAttribute* gxml_gom_xsd_attribute_new (void);
GXmlGomXsdAttribute* gxml_gom_xsd_attribute_construct (GType object_type);
GType gxml_gom_xsd_attribute_group_get_type (void) G_GNUC_CONST;
GXmlGomXsdAttributeGroup* gxml_gom_xsd_attribute_group_new (void);
GXmlGomXsdAttributeGroup* gxml_gom_xsd_attribute_group_construct (GType object_type);
void gxml_gom_xsd_list_remove (GXmlGomXsdList* self, gint index);
gint gxml_gom_xsd_list_index_of (GXmlGomXsdList* self, GXmlDomElement* element);
GXmlGomXsdList* gxml_gom_xsd_list_new (void);
GXmlGomXsdList* gxml_gom_xsd_list_construct (GType object_type);
gint gxml_gom_xsd_list_get_length (GXmlGomXsdList* self);
GXmlGomXsdListElements* gxml_gom_xsd_list_elements_new (void);
GXmlGomXsdListElements* gxml_gom_xsd_list_elements_construct (GType object_type);
GXmlGomXsdListSimpleTypes* gxml_gom_xsd_list_simple_types_new (void);
GXmlGomXsdListSimpleTypes* gxml_gom_xsd_list_simple_types_construct (GType object_type);
GXmlGomXsdListComplexTypes* gxml_gom_xsd_list_complex_types_new (void);
GXmlGomXsdListComplexTypes* gxml_gom_xsd_list_complex_types_construct (GType object_type);
GXmlGomXsdListTypeRestrictionEnumerations* gxml_gom_xsd_list_type_restriction_enumerations_new (void);
GXmlGomXsdListTypeRestrictionEnumerations* gxml_gom_xsd_list_type_restriction_enumerations_construct (GType object_type);
GXmlGomXsdListTypeRestrictionWhiteSpaces* gxml_gom_xsd_list_type_restriction_white_spaces_new (void);
GXmlGomXsdListTypeRestrictionWhiteSpaces* gxml_gom_xsd_list_type_restriction_white_spaces_construct (GType object_type);


G_END_DECLS

#endif
