/* SerializableGeeArrayListTest.c generated by valac 0.34.6, the Vala compiler
 * generated from SerializableGeeArrayListTest.vala, do not modify */

/* -*- Mode: vala; indent-tabs-mode: nil; c-basic-offset: 2; tab-width: 2 -*- */

#include <glib.h>
#include <glib-object.h>
#include <gxml/gxml.h>
#include <stdlib.h>
#include <string.h>
#include <gee.h>
#include <stdio.h>


#define TYPE_AELEMENT (aelement_get_type ())
#define AELEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_AELEMENT, AElement))
#define AELEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_AELEMENT, AElementClass))
#define IS_AELEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_AELEMENT))
#define IS_AELEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_AELEMENT))
#define AELEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_AELEMENT, AElementClass))

typedef struct _AElement AElement;
typedef struct _AElementClass AElementClass;
typedef struct _AElementPrivate AElementPrivate;
#define _g_free0(var) (var = (g_free (var), NULL))

#define AELEMENT_TYPE_ARRAY (aelement_array_get_type ())
#define AELEMENT_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), AELEMENT_TYPE_ARRAY, AElementArray))
#define AELEMENT_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), AELEMENT_TYPE_ARRAY, AElementArrayClass))
#define AELEMENT_IS_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), AELEMENT_TYPE_ARRAY))
#define AELEMENT_IS_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), AELEMENT_TYPE_ARRAY))
#define AELEMENT_ARRAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), AELEMENT_TYPE_ARRAY, AElementArrayClass))

typedef struct _AElementArray AElementArray;
typedef struct _AElementArrayClass AElementArrayClass;
typedef struct _AElementArrayPrivate AElementArrayPrivate;

#define TYPE_CELEMENT (celement_get_type ())
#define CELEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CELEMENT, CElement))
#define CELEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CELEMENT, CElementClass))
#define IS_CELEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CELEMENT))
#define IS_CELEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CELEMENT))
#define CELEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CELEMENT, CElementClass))

typedef struct _CElement CElement;
typedef struct _CElementClass CElementClass;
typedef struct _CElementPrivate CElementPrivate;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define TYPE_GXML_TEST (gxml_test_get_type ())
#define GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GXML_TEST, GXmlTest))
#define GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GXML_TEST, GXmlTestClass))
#define IS_GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GXML_TEST))
#define IS_GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GXML_TEST))
#define GXML_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GXML_TEST, GXmlTestClass))

typedef struct _GXmlTest GXmlTest;
typedef struct _GXmlTestClass GXmlTestClass;
typedef struct _GXmlTestPrivate GXmlTestPrivate;

#define TYPE_SERIALIZABLE_GEE_ARRAY_LIST_TEST (serializable_gee_array_list_test_get_type ())
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SERIALIZABLE_GEE_ARRAY_LIST_TEST, SerializableGeeArrayListTest))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SERIALIZABLE_GEE_ARRAY_LIST_TEST, SerializableGeeArrayListTestClass))
#define IS_SERIALIZABLE_GEE_ARRAY_LIST_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SERIALIZABLE_GEE_ARRAY_LIST_TEST))
#define IS_SERIALIZABLE_GEE_ARRAY_LIST_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SERIALIZABLE_GEE_ARRAY_LIST_TEST))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SERIALIZABLE_GEE_ARRAY_LIST_TEST, SerializableGeeArrayListTestClass))

typedef struct _SerializableGeeArrayListTest SerializableGeeArrayListTest;
typedef struct _SerializableGeeArrayListTestClass SerializableGeeArrayListTestClass;
typedef struct _SerializableGeeArrayListTestPrivate SerializableGeeArrayListTestPrivate;
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))

#define TYPE_BOOK_STORE (book_store_get_type ())
#define BOOK_STORE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_BOOK_STORE, BookStore))
#define BOOK_STORE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_BOOK_STORE, BookStoreClass))
#define IS_BOOK_STORE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_BOOK_STORE))
#define IS_BOOK_STORE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_BOOK_STORE))
#define BOOK_STORE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_BOOK_STORE, BookStoreClass))

typedef struct _BookStore BookStore;
typedef struct _BookStoreClass BookStoreClass;

#define BOOK_TYPE_ARRAY (book_array_get_type ())
#define BOOK_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOOK_TYPE_ARRAY, BookArray))
#define BOOK_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOOK_TYPE_ARRAY, BookArrayClass))
#define BOOK_IS_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOOK_TYPE_ARRAY))
#define BOOK_IS_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOOK_TYPE_ARRAY))
#define BOOK_ARRAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOOK_TYPE_ARRAY, BookArrayClass))

typedef struct _BookArray BookArray;
typedef struct _BookArrayClass BookArrayClass;

#define TYPE_BOOK (book_get_type ())
#define BOOK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_BOOK, Book))
#define BOOK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_BOOK, BookClass))
#define IS_BOOK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_BOOK))
#define IS_BOOK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_BOOK))
#define BOOK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_BOOK, BookClass))

typedef struct _Book Book;
typedef struct _BookClass BookClass;

#define TYPE_NAME (name_get_type ())
#define NAME(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_NAME, Name))
#define NAME_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_NAME, NameClass))
#define IS_NAME(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_NAME))
#define IS_NAME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_NAME))
#define NAME_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_NAME, NameClass))

typedef struct _Name Name;
typedef struct _NameClass NameClass;

#define TYPE_AUTHORS (authors_get_type ())
#define AUTHORS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_AUTHORS, Authors))
#define AUTHORS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_AUTHORS, AuthorsClass))
#define IS_AUTHORS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_AUTHORS))
#define IS_AUTHORS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_AUTHORS))
#define AUTHORS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_AUTHORS, AuthorsClass))

typedef struct _Authors Authors;
typedef struct _AuthorsClass AuthorsClass;

#define AUTHOR_TYPE_ARRAY (author_array_get_type ())
#define AUTHOR_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), AUTHOR_TYPE_ARRAY, AuthorArray))
#define AUTHOR_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), AUTHOR_TYPE_ARRAY, AuthorArrayClass))
#define AUTHOR_IS_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), AUTHOR_TYPE_ARRAY))
#define AUTHOR_IS_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), AUTHOR_TYPE_ARRAY))
#define AUTHOR_ARRAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), AUTHOR_TYPE_ARRAY, AuthorArrayClass))

typedef struct _AuthorArray AuthorArray;
typedef struct _AuthorArrayClass AuthorArrayClass;

#define TYPE_AUTHOR (author_get_type ())
#define AUTHOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_AUTHOR, Author))
#define AUTHOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_AUTHOR, AuthorClass))
#define IS_AUTHOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_AUTHOR))
#define IS_AUTHOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_AUTHOR))
#define AUTHOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_AUTHOR, AuthorClass))

typedef struct _Author Author;
typedef struct _AuthorClass AuthorClass;

#define TYPE_EMAIL (email_get_type ())
#define EMAIL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EMAIL, Email))
#define EMAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EMAIL, EmailClass))
#define IS_EMAIL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EMAIL))
#define IS_EMAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EMAIL))
#define EMAIL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EMAIL, EmailClass))

typedef struct _Email Email;
typedef struct _EmailClass EmailClass;

#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BIG_BAG (serializable_gee_array_list_test_big_bag_get_type ())
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_BIG_BAG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BIG_BAG, SerializableGeeArrayListTestBigBag))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_BIG_BAG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BIG_BAG, SerializableGeeArrayListTestBigBagClass))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_IS_BIG_BAG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BIG_BAG))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_IS_BIG_BAG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BIG_BAG))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_BIG_BAG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BIG_BAG, SerializableGeeArrayListTestBigBagClass))

typedef struct _SerializableGeeArrayListTestBigBag SerializableGeeArrayListTestBigBag;
typedef struct _SerializableGeeArrayListTestBigBagClass SerializableGeeArrayListTestBigBagClass;

#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_TYPE_ARRAY (serializable_gee_array_list_test_small_bag_array_get_type ())
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_TYPE_ARRAY, SerializableGeeArrayListTestSmallBagArray))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_TYPE_ARRAY, SerializableGeeArrayListTestSmallBagArrayClass))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_IS_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_TYPE_ARRAY))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_IS_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_TYPE_ARRAY))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_ARRAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_TYPE_ARRAY, SerializableGeeArrayListTestSmallBagArrayClass))

typedef struct _SerializableGeeArrayListTestSmallBagArray SerializableGeeArrayListTestSmallBagArray;
typedef struct _SerializableGeeArrayListTestSmallBagArrayClass SerializableGeeArrayListTestSmallBagArrayClass;

#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_SMALL_BAG (serializable_gee_array_list_test_small_bag_get_type ())
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_SMALL_BAG, SerializableGeeArrayListTestSmallBag))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_SMALL_BAG, SerializableGeeArrayListTestSmallBagClass))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_IS_SMALL_BAG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_SMALL_BAG))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_IS_SMALL_BAG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_SMALL_BAG))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_SMALL_BAG, SerializableGeeArrayListTestSmallBagClass))

typedef struct _SerializableGeeArrayListTestSmallBag SerializableGeeArrayListTestSmallBag;
typedef struct _SerializableGeeArrayListTestSmallBagClass SerializableGeeArrayListTestSmallBagClass;

#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_TYPE_ARRAY (serializable_gee_array_list_test_ball_array_get_type ())
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_TYPE_ARRAY, SerializableGeeArrayListTestBallArray))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_TYPE_ARRAY, SerializableGeeArrayListTestBallArrayClass))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_IS_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_TYPE_ARRAY))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_IS_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_TYPE_ARRAY))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_ARRAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_TYPE_ARRAY, SerializableGeeArrayListTestBallArrayClass))

typedef struct _SerializableGeeArrayListTestBallArray SerializableGeeArrayListTestBallArray;
typedef struct _SerializableGeeArrayListTestBallArrayClass SerializableGeeArrayListTestBallArrayClass;

#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL (serializable_gee_array_list_test_ball_get_type ())
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL, SerializableGeeArrayListTestBall))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL, SerializableGeeArrayListTestBallClass))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_IS_BALL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_IS_BALL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL, SerializableGeeArrayListTestBallClass))

typedef struct _SerializableGeeArrayListTestBall SerializableGeeArrayListTestBall;
typedef struct _SerializableGeeArrayListTestBallClass SerializableGeeArrayListTestBallClass;

#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL_FILL (serializable_gee_array_list_test_ball_fill_get_type ())
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_FILL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL_FILL, SerializableGeeArrayListTestBallFill))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_FILL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL_FILL, SerializableGeeArrayListTestBallFillClass))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_IS_BALL_FILL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL_FILL))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_IS_BALL_FILL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL_FILL))
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_FILL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL_FILL, SerializableGeeArrayListTestBallFillClass))

typedef struct _SerializableGeeArrayListTestBallFill SerializableGeeArrayListTestBallFill;
typedef struct _SerializableGeeArrayListTestBallFillClass SerializableGeeArrayListTestBallFillClass;
typedef struct _SerializableGeeArrayListTestBallFillPrivate SerializableGeeArrayListTestBallFillPrivate;
typedef struct _SerializableGeeArrayListTestBallPrivate SerializableGeeArrayListTestBallPrivate;
typedef struct _SerializableGeeArrayListTestBallArrayPrivate SerializableGeeArrayListTestBallArrayPrivate;
typedef struct _SerializableGeeArrayListTestSmallBagPrivate SerializableGeeArrayListTestSmallBagPrivate;
typedef struct _SerializableGeeArrayListTestSmallBagArrayPrivate SerializableGeeArrayListTestSmallBagArrayPrivate;
typedef struct _SerializableGeeArrayListTestBigBagPrivate SerializableGeeArrayListTestBigBagPrivate;
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _AElement {
	GXmlSerializableObjectModel parent_instance;
	AElementPrivate * priv;
};

struct _AElementClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _AElementPrivate {
	gchar* _name;
};

struct _AElementArray {
	GXmlSerializableArrayList parent_instance;
	AElementArrayPrivate * priv;
};

struct _AElementArrayClass {
	GXmlSerializableArrayListClass parent_class;
};

struct _AElementArrayPrivate {
	gboolean _enable_deserialize;
};

struct _CElement {
	GXmlSerializableObjectModel parent_instance;
	CElementPrivate * priv;
};

struct _CElementClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _CElementPrivate {
	AElementArray* _elements;
};

struct _GXmlTest {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GXmlTestPrivate * priv;
};

struct _GXmlTestClass {
	GTypeClass parent_class;
	void (*finalize) (GXmlTest *self);
};

struct _SerializableGeeArrayListTest {
	GXmlTest parent_instance;
	SerializableGeeArrayListTestPrivate * priv;
};

struct _SerializableGeeArrayListTestClass {
	GXmlTestClass parent_class;
};

struct _SerializableGeeArrayListTestBallFill {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeArrayListTestBallFillPrivate * priv;
};

struct _SerializableGeeArrayListTestBallFillClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeArrayListTestBallFillPrivate {
	gchar* _name;
};

struct _SerializableGeeArrayListTestBall {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeArrayListTestBallPrivate * priv;
};

struct _SerializableGeeArrayListTestBallClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeArrayListTestBallPrivate {
	gchar* _name;
	SerializableGeeArrayListTestBallFill* _ballfill;
};

struct _SerializableGeeArrayListTestBallArray {
	GXmlSerializableArrayList parent_instance;
	SerializableGeeArrayListTestBallArrayPrivate * priv;
};

struct _SerializableGeeArrayListTestBallArrayClass {
	GXmlSerializableArrayListClass parent_class;
};

struct _SerializableGeeArrayListTestSmallBag {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeArrayListTestSmallBagPrivate * priv;
};

struct _SerializableGeeArrayListTestSmallBagClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeArrayListTestSmallBagPrivate {
	gchar* _name;
	SerializableGeeArrayListTestBallArray* _balls;
};

struct _SerializableGeeArrayListTestSmallBagArray {
	GXmlSerializableArrayList parent_instance;
	SerializableGeeArrayListTestSmallBagArrayPrivate * priv;
};

struct _SerializableGeeArrayListTestSmallBagArrayClass {
	GXmlSerializableArrayListClass parent_class;
};

struct _SerializableGeeArrayListTestBigBag {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeArrayListTestBigBagPrivate * priv;
};

struct _SerializableGeeArrayListTestBigBagClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeArrayListTestBigBagPrivate {
	gchar* _name;
	SerializableGeeArrayListTestSmallBagArray* _bags;
};


static gpointer aelement_parent_class = NULL;
static gpointer aelement_array_parent_class = NULL;
static gpointer celement_parent_class = NULL;
static gpointer serializable_gee_array_list_test_parent_class = NULL;
static gpointer serializable_gee_array_list_test_ball_fill_parent_class = NULL;
static gpointer serializable_gee_array_list_test_ball_parent_class = NULL;
static gpointer serializable_gee_array_list_test_ball_array_parent_class = NULL;
static gpointer serializable_gee_array_list_test_small_bag_parent_class = NULL;
static gpointer serializable_gee_array_list_test_small_bag_array_parent_class = NULL;
static gpointer serializable_gee_array_list_test_big_bag_parent_class = NULL;

GType aelement_get_type (void) G_GNUC_CONST;
#define AELEMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_AELEMENT, AElementPrivate))
enum  {
	AELEMENT_DUMMY_PROPERTY,
	AELEMENT_NAME
};
AElement* aelement_new_named (const gchar* name);
AElement* aelement_construct_named (GType object_type, const gchar* name);
void aelement_set_name (AElement* self, const gchar* value);
static gchar* aelement_real_to_string (GXmlSerializableObjectModel* base);
const gchar* aelement_get_name (AElement* self);
AElement* aelement_new (void);
AElement* aelement_construct (GType object_type);
GType aelement_array_get_type (void) G_GNUC_CONST;
#define AELEMENT_ARRAY_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), AELEMENT_TYPE_ARRAY, AElementArrayPrivate))
enum  {
	AELEMENT_ARRAY_DUMMY_PROPERTY,
	AELEMENT_ARRAY_ENABLE_DESERIALIZE
};
static gboolean aelement_array_real_deserialize_proceed (GXmlSerializableArrayList* base);
gboolean aelement_array_get_enable_deserialize (AElementArray* self);
AElementArray* aelement_array_new (void);
AElementArray* aelement_array_construct (GType object_type);
void aelement_array_set_enable_deserialize (AElementArray* self, gboolean value);
static void aelement_array_finalize (GObject* obj);
static void _vala_aelement_array_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_aelement_array_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void aelement_finalize (GObject* obj);
static void _vala_aelement_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_aelement_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType celement_get_type (void) G_GNUC_CONST;
#define CELEMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_CELEMENT, CElementPrivate))
enum  {
	CELEMENT_DUMMY_PROPERTY,
	CELEMENT_ELEMENTS
};
static gchar* celement_real_node_name (GXmlSerializableObjectModel* base);
static gchar* celement_real_to_string (GXmlSerializableObjectModel* base);
CElement* celement_new (void);
CElement* celement_construct (GType object_type);
AElementArray* celement_get_elements (CElement* self);
void celement_set_elements (CElement* self, AElementArray* value);
static void celement_finalize (GObject* obj);
static void _vala_celement_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_celement_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
gpointer gxml_test_ref (gpointer instance);
void gxml_test_unref (gpointer instance);
GParamSpec* param_spec_gxml_test (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_gxml_test (GValue* value, gpointer v_object);
void value_take_gxml_test (GValue* value, gpointer v_object);
gpointer value_get_gxml_test (const GValue* value);
GType gxml_test_get_type (void) G_GNUC_CONST;
GType serializable_gee_array_list_test_get_type (void) G_GNUC_CONST;
enum  {
	SERIALIZABLE_GEE_ARRAY_LIST_TEST_DUMMY_PROPERTY
};
void serializable_gee_array_list_test_add_tests (void);
static void __lambda60_ (void);
static void ___lambda60__gtest_func (void);
static void __lambda61_ (void);
static void ___lambda61__gtest_func (void);
static void __lambda62_ (void);
static void ___lambda62__gtest_func (void);
static void __lambda63_ (void);
static void ___lambda63__gtest_func (void);
static void __lambda64_ (void);
GType book_store_get_type (void) G_GNUC_CONST;
BookStore* book_store_new (void);
BookStore* book_store_construct (GType object_type);
const gchar* book_store_get_name (BookStore* self);
GType book_array_get_type (void) G_GNUC_CONST;
BookArray* book_store_get_books (BookStore* self);
GType book_get_type (void) G_GNUC_CONST;
GType name_get_type (void) G_GNUC_CONST;
Name* book_get_name (Book* self);
gchar* name_get_name (Name* self);
const gchar* book_get_year (Book* self);
GType authors_get_type (void) G_GNUC_CONST;
Authors* book_get_authors (Book* self);
GType author_array_get_type (void) G_GNUC_CONST;
AuthorArray* authors_get_array (Authors* self);
GType author_get_type (void) G_GNUC_CONST;
Name* author_get_name (Author* self);
GType email_get_type (void) G_GNUC_CONST;
Email* author_get_email (Author* self);
gchar* email_get_mail (Email* self);
static void ___lambda64__gtest_func (void);
static void __lambda65_ (void);
static GType serializable_gee_array_list_test_big_bag_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeArrayListTestBigBag* serializable_gee_array_list_test_big_bag_new (void);
static SerializableGeeArrayListTestBigBag* serializable_gee_array_list_test_big_bag_construct (GType object_type);
GType serializable_gee_array_list_test_small_bag_array_get_type (void) G_GNUC_CONST;
static SerializableGeeArrayListTestSmallBagArray* serializable_gee_array_list_test_big_bag_get_bags (SerializableGeeArrayListTestBigBag* self);
static GType serializable_gee_array_list_test_small_bag_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeArrayListTestSmallBag* serializable_gee_array_list_test_small_bag_new (void);
static SerializableGeeArrayListTestSmallBag* serializable_gee_array_list_test_small_bag_construct (GType object_type);
GType serializable_gee_array_list_test_ball_array_get_type (void) G_GNUC_CONST;
static SerializableGeeArrayListTestBallArray* serializable_gee_array_list_test_small_bag_get_balls (SerializableGeeArrayListTestSmallBag* self);
static GType serializable_gee_array_list_test_ball_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeArrayListTestBall* serializable_gee_array_list_test_ball_new (void);
static SerializableGeeArrayListTestBall* serializable_gee_array_list_test_ball_construct (GType object_type);
static SerializableGeeArrayListTestBallFill* serializable_gee_array_list_test_ball_fill_new (void);
static SerializableGeeArrayListTestBallFill* serializable_gee_array_list_test_ball_fill_construct (GType object_type);
static GType serializable_gee_array_list_test_ball_fill_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static void serializable_gee_array_list_test_ball_set_ballfill (SerializableGeeArrayListTestBall* self, SerializableGeeArrayListTestBallFill* value);
static SerializableGeeArrayListTestBallFill* serializable_gee_array_list_test_ball_get_ballfill (SerializableGeeArrayListTestBall* self);
static void serializable_gee_array_list_test_ball_fill_set_text (SerializableGeeArrayListTestBallFill* self, const gchar* txt);
static const gchar* serializable_gee_array_list_test_ball_get_name (SerializableGeeArrayListTestBall* self);
static void ___lambda65__gtest_func (void);
SerializableGeeArrayListTest* serializable_gee_array_list_test_new (void);
SerializableGeeArrayListTest* serializable_gee_array_list_test_construct (GType object_type);
GXmlTest* gxml_test_new (void);
GXmlTest* gxml_test_construct (GType object_type);
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_FILL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL_FILL, SerializableGeeArrayListTestBallFillPrivate))
enum  {
	SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_FILL_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_FILL_NAME
};
static gchar* serializable_gee_array_list_test_ball_fill_real_to_string (GXmlSerializableObjectModel* base);
static const gchar* serializable_gee_array_list_test_ball_fill_get_name (SerializableGeeArrayListTestBallFill* self);
static gchar* serializable_gee_array_list_test_ball_fill_real_node_name (GXmlSerializableObjectModel* base);
static gboolean serializable_gee_array_list_test_ball_fill_real_serialize_use_xml_node_value (GXmlSerializableObjectModel* base);
static void serializable_gee_array_list_test_ball_fill_set_name (SerializableGeeArrayListTestBallFill* self, const gchar* value);
static void serializable_gee_array_list_test_ball_fill_finalize (GObject* obj);
static void _vala_serializable_gee_array_list_test_ball_fill_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_array_list_test_ball_fill_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL, SerializableGeeArrayListTestBallPrivate))
enum  {
	SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_NAME,
	SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_BALLFILL
};
static gchar* serializable_gee_array_list_test_ball_real_to_string (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_array_list_test_ball_real_node_name (GXmlSerializableObjectModel* base);
static void serializable_gee_array_list_test_ball_set_name (SerializableGeeArrayListTestBall* self, const gchar* value);
enum  {
	SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_ARRAY_DUMMY_PROPERTY
};
static gboolean serializable_gee_array_list_test_ball_array_real_deserialize_proceed (GXmlSerializableArrayList* base);
static SerializableGeeArrayListTestBallArray* serializable_gee_array_list_test_ball_array_new (void);
static SerializableGeeArrayListTestBallArray* serializable_gee_array_list_test_ball_array_construct (GType object_type);
static void serializable_gee_array_list_test_ball_finalize (GObject* obj);
static void _vala_serializable_gee_array_list_test_ball_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_array_list_test_ball_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_SMALL_BAG, SerializableGeeArrayListTestSmallBagPrivate))
enum  {
	SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_NAME,
	SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_BALLS
};
static gchar* serializable_gee_array_list_test_small_bag_real_to_string (GXmlSerializableObjectModel* base);
static const gchar* serializable_gee_array_list_test_small_bag_get_name (SerializableGeeArrayListTestSmallBag* self);
static gchar* serializable_gee_array_list_test_small_bag_real_node_name (GXmlSerializableObjectModel* base);
static void serializable_gee_array_list_test_small_bag_set_name (SerializableGeeArrayListTestSmallBag* self, const gchar* value);
static void serializable_gee_array_list_test_small_bag_set_balls (SerializableGeeArrayListTestSmallBag* self, SerializableGeeArrayListTestBallArray* value);
enum  {
	SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_ARRAY_DUMMY_PROPERTY
};
static gboolean serializable_gee_array_list_test_small_bag_array_real_deserialize_proceed (GXmlSerializableArrayList* base);
static SerializableGeeArrayListTestSmallBagArray* serializable_gee_array_list_test_small_bag_array_new (void);
static SerializableGeeArrayListTestSmallBagArray* serializable_gee_array_list_test_small_bag_array_construct (GType object_type);
static void serializable_gee_array_list_test_small_bag_finalize (GObject* obj);
static void _vala_serializable_gee_array_list_test_small_bag_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_array_list_test_small_bag_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_ARRAY_LIST_TEST_BIG_BAG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BIG_BAG, SerializableGeeArrayListTestBigBagPrivate))
enum  {
	SERIALIZABLE_GEE_ARRAY_LIST_TEST_BIG_BAG_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_ARRAY_LIST_TEST_BIG_BAG_NAME,
	SERIALIZABLE_GEE_ARRAY_LIST_TEST_BIG_BAG_BAGS
};
static gchar* serializable_gee_array_list_test_big_bag_real_to_string (GXmlSerializableObjectModel* base);
static const gchar* serializable_gee_array_list_test_big_bag_get_name (SerializableGeeArrayListTestBigBag* self);
static gchar* serializable_gee_array_list_test_big_bag_real_node_name (GXmlSerializableObjectModel* base);
static void serializable_gee_array_list_test_big_bag_set_name (SerializableGeeArrayListTestBigBag* self, const gchar* value);
static void serializable_gee_array_list_test_big_bag_set_bags (SerializableGeeArrayListTestBigBag* self, SerializableGeeArrayListTestSmallBagArray* value);
static void serializable_gee_array_list_test_big_bag_finalize (GObject* obj);
static void _vala_serializable_gee_array_list_test_big_bag_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_array_list_test_big_bag_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);


AElement* aelement_construct_named (GType object_type, const gchar* name) {
	AElement * self = NULL;
	const gchar* _tmp0_ = NULL;
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (AElement*) gxml_serializable_object_model_construct (object_type);
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = name;
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	aelement_set_name (self, _tmp0_);
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return self;
#line 563 "SerializableGeeArrayListTest.c"
}


AElement* aelement_new_named (const gchar* name) {
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return aelement_construct_named (TYPE_AELEMENT, name);
#line 570 "SerializableGeeArrayListTest.c"
}


static gchar* aelement_real_to_string (GXmlSerializableObjectModel* base) {
	AElement * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (AElement*) base;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = self->priv->_name;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp1_;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 589 "SerializableGeeArrayListTest.c"
}


AElement* aelement_construct (GType object_type) {
	AElement * self = NULL;
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (AElement*) gxml_serializable_object_model_construct (object_type);
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return self;
#line 599 "SerializableGeeArrayListTest.c"
}


AElement* aelement_new (void) {
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return aelement_construct (TYPE_AELEMENT);
#line 606 "SerializableGeeArrayListTest.c"
}


const gchar* aelement_get_name (AElement* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = self->priv->_name;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp0_;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 621 "SerializableGeeArrayListTest.c"
}


void aelement_set_name (AElement* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_if_fail (self != NULL);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = value;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_g_free0 (self->priv->_name);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv->_name = _tmp1_;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 640 "SerializableGeeArrayListTest.c"
}


static gboolean aelement_array_real_deserialize_proceed (GXmlSerializableArrayList* base) {
	AElementArray * self;
	gboolean result = FALSE;
	gboolean _tmp0_ = FALSE;
#line 36 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (AElementArray*) base;
#line 36 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = self->priv->_enable_deserialize;
#line 36 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp0_;
#line 36 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 656 "SerializableGeeArrayListTest.c"
}


AElementArray* aelement_array_construct (GType object_type) {
	AElementArray * self = NULL;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (AElementArray*) gxml_serializable_array_list_construct (object_type, TYPE_AELEMENT, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return self;
#line 666 "SerializableGeeArrayListTest.c"
}


AElementArray* aelement_array_new (void) {
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return aelement_array_construct (AELEMENT_TYPE_ARRAY);
#line 673 "SerializableGeeArrayListTest.c"
}


gboolean aelement_array_get_enable_deserialize (AElementArray* self) {
	gboolean result;
	gboolean _tmp0_ = FALSE;
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = self->priv->_enable_deserialize;
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp0_;
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 688 "SerializableGeeArrayListTest.c"
}


void aelement_array_set_enable_deserialize (AElementArray* self, gboolean value) {
	gboolean _tmp0_ = FALSE;
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_if_fail (self != NULL);
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = value;
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv->_enable_deserialize = _tmp0_;
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_object_notify ((GObject *) self, "enable-deserialize");
#line 702 "SerializableGeeArrayListTest.c"
}


static void aelement_array_class_init (AElementArrayClass * klass) {
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	aelement_array_parent_class = g_type_class_peek_parent (klass);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_type_class_add_private (klass, sizeof (AElementArrayPrivate));
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	((GXmlSerializableArrayListClass *) klass)->deserialize_proceed = (gboolean (*)(GXmlSerializableArrayList*)) aelement_array_real_deserialize_proceed;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_aelement_array_get_property;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_aelement_array_set_property;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (klass)->finalize = aelement_array_finalize;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), AELEMENT_ARRAY_ENABLE_DESERIALIZE, g_param_spec_boolean ("enable-deserialize", "enable-deserialize", "enable-deserialize", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 721 "SerializableGeeArrayListTest.c"
}


static void aelement_array_instance_init (AElementArray * self) {
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv = AELEMENT_ARRAY_GET_PRIVATE (self);
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv->_enable_deserialize = FALSE;
#line 730 "SerializableGeeArrayListTest.c"
}


static void aelement_array_finalize (GObject* obj) {
	AElementArray * self;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, AELEMENT_TYPE_ARRAY, AElementArray);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (aelement_array_parent_class)->finalize (obj);
#line 740 "SerializableGeeArrayListTest.c"
}


GType aelement_array_get_type (void) {
	static volatile gsize aelement_array_type_id__volatile = 0;
	if (g_once_init_enter (&aelement_array_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (AElementArrayClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) aelement_array_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AElementArray), 0, (GInstanceInitFunc) aelement_array_instance_init, NULL };
		GType aelement_array_type_id;
		aelement_array_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_ARRAY_LIST, "AElementArray", &g_define_type_info, 0);
		g_once_init_leave (&aelement_array_type_id__volatile, aelement_array_type_id);
	}
	return aelement_array_type_id__volatile;
}


static void _vala_aelement_array_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	AElementArray * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, AELEMENT_TYPE_ARRAY, AElementArray);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	switch (property_id) {
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		case AELEMENT_ARRAY_ENABLE_DESERIALIZE:
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_value_set_boolean (value, aelement_array_get_enable_deserialize (self));
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 767 "SerializableGeeArrayListTest.c"
		default:
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 773 "SerializableGeeArrayListTest.c"
	}
}


static void _vala_aelement_array_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	AElementArray * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, AELEMENT_TYPE_ARRAY, AElementArray);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	switch (property_id) {
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		case AELEMENT_ARRAY_ENABLE_DESERIALIZE:
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		aelement_array_set_enable_deserialize (self, g_value_get_boolean (value));
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 789 "SerializableGeeArrayListTest.c"
		default:
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 795 "SerializableGeeArrayListTest.c"
	}
}


static void aelement_class_init (AElementClass * klass) {
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	aelement_parent_class = g_type_class_peek_parent (klass);
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_type_class_add_private (klass, sizeof (AElementPrivate));
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) aelement_real_to_string;
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_aelement_get_property;
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_aelement_set_property;
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (klass)->finalize = aelement_finalize;
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), AELEMENT_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 815 "SerializableGeeArrayListTest.c"
}


static void aelement_instance_init (AElement * self) {
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv = AELEMENT_GET_PRIVATE (self);
#line 822 "SerializableGeeArrayListTest.c"
}


static void aelement_finalize (GObject* obj) {
	AElement * self;
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_AELEMENT, AElement);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_g_free0 (self->priv->_name);
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (aelement_parent_class)->finalize (obj);
#line 834 "SerializableGeeArrayListTest.c"
}


/**
 *
 *  GXml.Serializable.BasicTypeTest
 *
 *  Authors:
 *
 *       Daniel Espinosa <esodan@gmail.com>
 *
 *
 *  Copyright (c) 2013-2015 Daniel Espinosa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
GType aelement_get_type (void) {
	static volatile gsize aelement_type_id__volatile = 0;
	if (g_once_init_enter (&aelement_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (AElementClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) aelement_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AElement), 0, (GInstanceInitFunc) aelement_instance_init, NULL };
		GType aelement_type_id;
		aelement_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "AElement", &g_define_type_info, 0);
		g_once_init_leave (&aelement_type_id__volatile, aelement_type_id);
	}
	return aelement_type_id__volatile;
}


static void _vala_aelement_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	AElement * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_AELEMENT, AElement);
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	switch (property_id) {
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		case AELEMENT_NAME:
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_value_set_string (value, aelement_get_name (self));
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 885 "SerializableGeeArrayListTest.c"
		default:
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 891 "SerializableGeeArrayListTest.c"
	}
}


static void _vala_aelement_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	AElement * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_AELEMENT, AElement);
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	switch (property_id) {
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		case AELEMENT_NAME:
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		aelement_set_name (self, g_value_get_string (value));
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 907 "SerializableGeeArrayListTest.c"
		default:
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 913 "SerializableGeeArrayListTest.c"
	}
}


static gchar* celement_real_node_name (GXmlSerializableObjectModel* base) {
	CElement * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (CElement*) base;
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = g_strdup ("CElement");
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp0_;
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 930 "SerializableGeeArrayListTest.c"
}


static gchar* celement_real_to_string (GXmlSerializableObjectModel* base) {
	CElement * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (CElement*) base;
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = g_strdup ("CElement");
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp0_;
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 946 "SerializableGeeArrayListTest.c"
}


CElement* celement_construct (GType object_type) {
	CElement * self = NULL;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (CElement*) gxml_serializable_object_model_construct (object_type);
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return self;
#line 956 "SerializableGeeArrayListTest.c"
}


CElement* celement_new (void) {
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return celement_construct (TYPE_CELEMENT);
#line 963 "SerializableGeeArrayListTest.c"
}


AElementArray* celement_get_elements (CElement* self) {
	AElementArray* result;
	AElementArray* _tmp0_ = NULL;
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = self->priv->_elements;
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp0_;
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 978 "SerializableGeeArrayListTest.c"
}


static gpointer _g_object_ref0 (gpointer self) {
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return self ? g_object_ref (self) : NULL;
#line 985 "SerializableGeeArrayListTest.c"
}


void celement_set_elements (CElement* self, AElementArray* value) {
	AElementArray* _tmp0_ = NULL;
	AElementArray* _tmp1_ = NULL;
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_if_fail (self != NULL);
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = value;
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_g_object_unref0 (self->priv->_elements);
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv->_elements = _tmp1_;
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_object_notify ((GObject *) self, "elements");
#line 1004 "SerializableGeeArrayListTest.c"
}


static void celement_class_init (CElementClass * klass) {
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	celement_parent_class = g_type_class_peek_parent (klass);
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_type_class_add_private (klass, sizeof (CElementPrivate));
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) celement_real_node_name;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) celement_real_to_string;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_celement_get_property;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_celement_set_property;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (klass)->finalize = celement_finalize;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CELEMENT_ELEMENTS, g_param_spec_object ("elements", "elements", "elements", AELEMENT_TYPE_ARRAY, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 1025 "SerializableGeeArrayListTest.c"
}


static void celement_instance_init (CElement * self) {
	AElementArray* _tmp0_ = NULL;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv = CELEMENT_GET_PRIVATE (self);
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = aelement_array_new ();
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv->_elements = _tmp0_;
#line 1037 "SerializableGeeArrayListTest.c"
}


static void celement_finalize (GObject* obj) {
	CElement * self;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_CELEMENT, CElement);
#line 41 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_g_object_unref0 (self->priv->_elements);
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (celement_parent_class)->finalize (obj);
#line 1049 "SerializableGeeArrayListTest.c"
}


GType celement_get_type (void) {
	static volatile gsize celement_type_id__volatile = 0;
	if (g_once_init_enter (&celement_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (CElementClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) celement_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (CElement), 0, (GInstanceInitFunc) celement_instance_init, NULL };
		GType celement_type_id;
		celement_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "CElement", &g_define_type_info, 0);
		g_once_init_leave (&celement_type_id__volatile, celement_type_id);
	}
	return celement_type_id__volatile;
}


static void _vala_celement_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	CElement * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_CELEMENT, CElement);
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	switch (property_id) {
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		case CELEMENT_ELEMENTS:
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_value_set_object (value, celement_get_elements (self));
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 1076 "SerializableGeeArrayListTest.c"
		default:
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 1082 "SerializableGeeArrayListTest.c"
	}
}


static void _vala_celement_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	CElement * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_CELEMENT, CElement);
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	switch (property_id) {
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		case CELEMENT_ELEMENTS:
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		celement_set_elements (self, g_value_get_object (value));
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 1098 "SerializableGeeArrayListTest.c"
		default:
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 1104 "SerializableGeeArrayListTest.c"
	}
}


static void __lambda60_ (void) {
	GXmlSerializableArrayList* c = NULL;
	GXmlSerializableArrayList* _tmp0_ = NULL;
	AElement* o1 = NULL;
	AElement* _tmp1_ = NULL;
	AElement* o2 = NULL;
	AElement* _tmp2_ = NULL;
	GXmlSerializableArrayList* _tmp3_ = NULL;
	AElement* _tmp4_ = NULL;
	GXmlSerializableArrayList* _tmp5_ = NULL;
	AElement* _tmp6_ = NULL;
	gboolean found1 = FALSE;
	gboolean found2 = FALSE;
	gboolean _tmp24_ = FALSE;
	gboolean _tmp26_ = FALSE;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = gxml_serializable_array_list_new (TYPE_AELEMENT, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	c = _tmp0_;
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp1_ = aelement_new_named ("Big");
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	o1 = _tmp1_;
#line 93 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp2_ = aelement_new_named ("Small");
#line 93 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	o2 = _tmp2_;
#line 94 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp3_ = c;
#line 94 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp4_ = o1;
#line 94 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	gee_abstract_collection_add ((GeeAbstractCollection*) _tmp3_, _tmp4_);
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp5_ = c;
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp6_ = o2;
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	gee_abstract_collection_add ((GeeAbstractCollection*) _tmp5_, _tmp6_);
#line 96 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	found1 = FALSE;
#line 97 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	found2 = FALSE;
#line 1152 "SerializableGeeArrayListTest.c"
	{
		GXmlSerializableArrayList* _o_list = NULL;
		GXmlSerializableArrayList* _tmp7_ = NULL;
		GXmlSerializableArrayList* _tmp8_ = NULL;
		gint _o_size = 0;
		GXmlSerializableArrayList* _tmp9_ = NULL;
		gint _tmp10_ = 0;
		gint _tmp11_ = 0;
		gint _o_index = 0;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp7_ = c;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp8_ = _g_object_ref0 (_tmp7_);
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_o_list = _tmp8_;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp9_ = _o_list;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp10_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp9_);
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp11_ = _tmp10_;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_o_size = _tmp11_;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_o_index = -1;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		while (TRUE) {
#line 1180 "SerializableGeeArrayListTest.c"
			gint _tmp12_ = 0;
			gint _tmp13_ = 0;
			gint _tmp14_ = 0;
			AElement* o = NULL;
			GXmlSerializableArrayList* _tmp15_ = NULL;
			gint _tmp16_ = 0;
			gpointer _tmp17_ = NULL;
			AElement* _tmp18_ = NULL;
			const gchar* _tmp19_ = NULL;
			const gchar* _tmp20_ = NULL;
			AElement* _tmp21_ = NULL;
			const gchar* _tmp22_ = NULL;
			const gchar* _tmp23_ = NULL;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp12_ = _o_index;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_o_index = _tmp12_ + 1;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp13_ = _o_index;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp14_ = _o_size;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			if (!(_tmp13_ < _tmp14_)) {
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				break;
#line 1206 "SerializableGeeArrayListTest.c"
			}
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp15_ = _o_list;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp16_ = _o_index;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp17_ = gee_abstract_list_get ((GeeAbstractList*) _tmp15_, _tmp16_);
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			o = (AElement*) _tmp17_;
#line 99 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp18_ = o;
#line 99 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp19_ = aelement_get_name (_tmp18_);
#line 99 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp20_ = _tmp19_;
#line 99 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			if (g_strcmp0 (_tmp20_, "Big") == 0) {
#line 99 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				found1 = TRUE;
#line 1226 "SerializableGeeArrayListTest.c"
			}
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp21_ = o;
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp22_ = aelement_get_name (_tmp21_);
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp23_ = _tmp22_;
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			if (g_strcmp0 (_tmp23_, "Small") == 0) {
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				found2 = TRUE;
#line 1238 "SerializableGeeArrayListTest.c"
			}
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (o);
#line 1242 "SerializableGeeArrayListTest.c"
		}
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_o_list);
#line 1246 "SerializableGeeArrayListTest.c"
	}
#line 102 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp24_ = found1;
#line 102 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	if (!_tmp24_) {
#line 1252 "SerializableGeeArrayListTest.c"
		FILE* _tmp25_ = NULL;
#line 103 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp25_ = stdout;
#line 103 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		fprintf (_tmp25_, "%s", "Big is not found\n");
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_assert_not_reached ();
#line 1260 "SerializableGeeArrayListTest.c"
	}
#line 106 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp26_ = found2;
#line 106 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	if (!_tmp26_) {
#line 1266 "SerializableGeeArrayListTest.c"
		FILE* _tmp27_ = NULL;
#line 107 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp27_ = stdout;
#line 107 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		fprintf (_tmp27_, "%s", "Small is not found\n");
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_assert_not_reached ();
#line 1274 "SerializableGeeArrayListTest.c"
	}
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_g_object_unref0 (o2);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_g_object_unref0 (o1);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_g_object_unref0 (c);
#line 1282 "SerializableGeeArrayListTest.c"
}


static void ___lambda60__gtest_func (void) {
#line 89 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	__lambda60_ ();
#line 1289 "SerializableGeeArrayListTest.c"
}


static const gchar* string_to_string (const gchar* self) {
	const gchar* result = NULL;
#line 1412 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1413 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	result = self;
#line 1413 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	return result;
#line 1301 "SerializableGeeArrayListTest.c"
}


static void __lambda61_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlSerializableArrayList* c = NULL;
		GXmlSerializableArrayList* _tmp0_ = NULL;
		AElement* o1 = NULL;
		AElement* _tmp1_ = NULL;
		AElement* o2 = NULL;
		AElement* _tmp2_ = NULL;
		GXmlSerializableArrayList* _tmp3_ = NULL;
		AElement* _tmp4_ = NULL;
		GXmlSerializableArrayList* _tmp5_ = NULL;
		AElement* _tmp6_ = NULL;
		GXmlTDocument* doc = NULL;
		GXmlTDocument* _tmp7_ = NULL;
		GXmlNode* root = NULL;
		GXmlTDocument* _tmp8_ = NULL;
		GXmlNode* _tmp9_ = NULL;
		GXmlTDocument* _tmp10_ = NULL;
		GeeBidirList* _tmp11_ = NULL;
		GeeBidirList* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GXmlNode* _tmp14_ = NULL;
		GXmlSerializableArrayList* _tmp15_ = NULL;
		GXmlNode* _tmp16_ = NULL;
		GXmlNode* _tmp17_ = NULL;
		GXmlNode* _tmp18_ = NULL;
		GXmlNode* _tmp19_ = NULL;
		GeeBidirList* _tmp20_ = NULL;
		GeeBidirList* _tmp21_ = NULL;
		GeeBidirList* _tmp22_ = NULL;
		gint _tmp23_ = 0;
		gint _tmp24_ = 0;
		gboolean found1 = FALSE;
		gboolean found2 = FALSE;
		gboolean _tmp60_ = FALSE;
		gboolean _tmp62_ = FALSE;
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp0_ = gxml_serializable_array_list_new (TYPE_AELEMENT, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 114 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		c = _tmp0_;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp1_ = aelement_new_named ("Big");
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		o1 = _tmp1_;
#line 116 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp2_ = aelement_new_named ("Small");
#line 116 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		o2 = _tmp2_;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp3_ = c;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp4_ = o1;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		gee_abstract_collection_add ((GeeAbstractCollection*) _tmp3_, _tmp4_);
#line 118 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp5_ = c;
#line 118 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp6_ = o2;
#line 118 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		gee_abstract_collection_add ((GeeAbstractCollection*) _tmp5_, _tmp6_);
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp7_ = gxml_tdocument_new ();
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		doc = _tmp7_;
#line 120 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp8_ = doc;
#line 120 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp9_ = gxml_document_create_element ((GXmlDocument*) _tmp8_, "root", &_inner_error_);
#line 120 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		root = _tmp9_;
#line 120 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 120 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (doc);
#line 120 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (o2);
#line 120 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (o1);
#line 120 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (c);
#line 1386 "SerializableGeeArrayListTest.c"
			goto __catch82_g_error;
		}
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp10_ = doc;
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp11_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp10_);
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp12_ = _tmp11_;
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp13_ = _tmp12_;
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp14_ = root;
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		gee_collection_add ((GeeCollection*) _tmp13_, _tmp14_);
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp13_);
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp15_ = c;
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp16_ = root;
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp17_ = gxml_serializable_array_list_serialize (_tmp15_, _tmp16_, &_inner_error_);
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp18_ = _tmp17_;
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp18_);
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (root);
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (doc);
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (o2);
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (o1);
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (c);
#line 1425 "SerializableGeeArrayListTest.c"
			goto __catch82_g_error;
		}
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp19_ = root;
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp20_ = gxml_node_get_children_nodes (_tmp19_);
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp21_ = _tmp20_;
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp22_ = _tmp21_;
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp23_ = gee_collection_get_size ((GeeCollection*) _tmp22_);
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp24_ = _tmp23_;
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp24_ == 2, "root.children_nodes.size == 2");
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp22_);
#line 124 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		found1 = FALSE;
#line 125 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		found2 = FALSE;
#line 1448 "SerializableGeeArrayListTest.c"
		{
			GeeBidirList* _n_list = NULL;
			GXmlNode* _tmp25_ = NULL;
			GeeBidirList* _tmp26_ = NULL;
			GeeBidirList* _tmp27_ = NULL;
			gint _n_size = 0;
			GeeBidirList* _tmp28_ = NULL;
			gint _tmp29_ = 0;
			gint _tmp30_ = 0;
			gint _n_index = 0;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp25_ = root;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp26_ = gxml_node_get_children_nodes (_tmp25_);
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp27_ = _tmp26_;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_n_list = _tmp27_;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp28_ = _n_list;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp29_ = gee_collection_get_size ((GeeCollection*) _tmp28_);
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp30_ = _tmp29_;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_n_size = _tmp30_;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_n_index = -1;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			while (TRUE) {
#line 1479 "SerializableGeeArrayListTest.c"
				gint _tmp31_ = 0;
				gint _tmp32_ = 0;
				gint _tmp33_ = 0;
				GXmlNode* n = NULL;
				GeeBidirList* _tmp34_ = NULL;
				gint _tmp35_ = 0;
				gpointer _tmp36_ = NULL;
				gboolean _tmp37_ = FALSE;
				GXmlNode* _tmp38_ = NULL;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp31_ = _n_index;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_n_index = _tmp31_ + 1;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp32_ = _n_index;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp33_ = _n_size;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				if (!(_tmp32_ < _tmp33_)) {
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					break;
#line 1501 "SerializableGeeArrayListTest.c"
				}
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp34_ = _n_list;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp35_ = _n_index;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp36_ = gee_list_get ((GeeList*) _tmp34_, _tmp35_);
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				n = (GXmlNode*) _tmp36_;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp38_ = n;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp38_, GXML_TYPE_ELEMENT)) {
#line 1515 "SerializableGeeArrayListTest.c"
					GXmlNode* _tmp39_ = NULL;
					gchar* _tmp40_ = NULL;
					gchar* _tmp41_ = NULL;
					gchar* _tmp42_ = NULL;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp39_ = n;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp40_ = gxml_node_get_name (_tmp39_);
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp41_ = _tmp40_;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp42_ = _tmp41_;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp37_ = g_strcmp0 (_tmp42_, "aelement") == 0;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_g_free0 (_tmp42_);
#line 1532 "SerializableGeeArrayListTest.c"
				} else {
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp37_ = FALSE;
#line 1536 "SerializableGeeArrayListTest.c"
				}
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				if (_tmp37_) {
#line 1540 "SerializableGeeArrayListTest.c"
					GXmlNode* name = NULL;
					GXmlNode* _tmp43_ = NULL;
					GeeMap* _tmp44_ = NULL;
					GeeMap* _tmp45_ = NULL;
					GeeMap* _tmp46_ = NULL;
					gpointer _tmp47_ = NULL;
					GXmlNode* _tmp48_ = NULL;
					GXmlNode* _tmp49_ = NULL;
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp43_ = n;
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp44_ = gxml_node_get_attrs (_tmp43_);
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp45_ = _tmp44_;
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp46_ = _tmp45_;
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp47_ = gee_map_get (_tmp46_, "name");
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp48_ = (GXmlNode*) _tmp47_;
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_g_object_unref0 (_tmp46_);
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					name = _tmp48_;
#line 129 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp49_ = name;
#line 129 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					if (_tmp49_ != NULL) {
#line 1569 "SerializableGeeArrayListTest.c"
						GXmlNode* _tmp50_ = NULL;
						gchar* _tmp51_ = NULL;
						gchar* _tmp52_ = NULL;
						gchar* _tmp53_ = NULL;
						gboolean _tmp54_ = FALSE;
						GXmlNode* _tmp55_ = NULL;
						gchar* _tmp56_ = NULL;
						gchar* _tmp57_ = NULL;
						gchar* _tmp58_ = NULL;
						gboolean _tmp59_ = FALSE;
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
						_tmp50_ = name;
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
						_tmp51_ = gxml_node_get_value (_tmp50_);
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
						_tmp52_ = _tmp51_;
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
						_tmp53_ = _tmp52_;
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
						_tmp54_ = g_strcmp0 (_tmp53_, "Big") == 0;
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
						_g_free0 (_tmp53_);
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
						if (_tmp54_) {
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
							found1 = TRUE;
#line 1596 "SerializableGeeArrayListTest.c"
						}
#line 131 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
						_tmp55_ = name;
#line 131 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
						_tmp56_ = gxml_node_get_value (_tmp55_);
#line 131 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
						_tmp57_ = _tmp56_;
#line 131 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
						_tmp58_ = _tmp57_;
#line 131 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
						_tmp59_ = g_strcmp0 (_tmp58_, "Small") == 0;
#line 131 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
						_g_free0 (_tmp58_);
#line 131 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
						if (_tmp59_) {
#line 131 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
							found2 = TRUE;
#line 1614 "SerializableGeeArrayListTest.c"
						}
					}
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_g_object_unref0 (name);
#line 1619 "SerializableGeeArrayListTest.c"
				}
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_g_object_unref0 (n);
#line 1623 "SerializableGeeArrayListTest.c"
			}
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (_n_list);
#line 1627 "SerializableGeeArrayListTest.c"
		}
#line 135 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp60_ = found1;
#line 135 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (!_tmp60_) {
#line 1633 "SerializableGeeArrayListTest.c"
			FILE* _tmp61_ = NULL;
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp61_ = stdout;
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			fprintf (_tmp61_, "%s", "ERROR: Big space node was not found\n");
#line 137 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			g_assert_not_reached ();
#line 1641 "SerializableGeeArrayListTest.c"
		}
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp62_ = found2;
#line 139 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (!_tmp62_) {
#line 1647 "SerializableGeeArrayListTest.c"
			FILE* _tmp63_ = NULL;
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp63_ = stdout;
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			fprintf (_tmp63_, "%s", "ERROR: Small space node was not found\n");
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			g_assert_not_reached ();
#line 1655 "SerializableGeeArrayListTest.c"
		}
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (root);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (doc);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (o2);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (o1);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (c);
#line 1667 "SerializableGeeArrayListTest.c"
	}
	goto __finally82;
	__catch82_g_error:
	{
		GError* e = NULL;
		FILE* _tmp64_ = NULL;
		const gchar* _tmp65_ = NULL;
		const gchar* _tmp66_ = NULL;
		gchar* _tmp67_ = NULL;
		gchar* _tmp68_ = NULL;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		e = _inner_error_;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_inner_error_ = NULL;
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp64_ = stdout;
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp65_ = e->message;
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp66_ = string_to_string (_tmp65_);
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp67_ = g_strconcat ("ERROR: ", _tmp66_, NULL);
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp68_ = _tmp67_;
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		fprintf (_tmp64_, "%s", _tmp68_);
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_free0 (_tmp68_);
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_assert_not_reached ();
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_error_free0 (e);
#line 1700 "SerializableGeeArrayListTest.c"
	}
	__finally82:
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_clear_error (&_inner_error_);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		return;
#line 1711 "SerializableGeeArrayListTest.c"
	}
}


static void ___lambda61__gtest_func (void) {
#line 111 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	__lambda61_ ();
#line 1719 "SerializableGeeArrayListTest.c"
}


static void __lambda62_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		GXmlSerializableArrayList* c = NULL;
		GXmlSerializableArrayList* _tmp1_ = NULL;
		GXmlSerializableArrayList* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		GXmlNode* _tmp4_ = NULL;
		GXmlNode* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		GXmlSerializableArrayList* _tmp7_ = NULL;
		gint _tmp8_ = 0;
		gint _tmp9_ = 0;
		gboolean found1 = FALSE;
		gboolean found2 = FALSE;
		gboolean _tmp35_ = FALSE;
		gboolean _tmp37_ = FALSE;
#line 152 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp0_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n" \
"  <root><aelement name=\"Big\"/><aelement name=\"Small\"/></root>", 0, &_inner_error_);
#line 152 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		doc = _tmp0_;
#line 152 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1748 "SerializableGeeArrayListTest.c"
			goto __catch83_g_error;
		}
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp1_ = gxml_serializable_array_list_new (TYPE_AELEMENT, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		c = _tmp1_;
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp2_ = c;
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp3_ = doc;
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp4_ = gxml_document_get_root ((GXmlDocument*) _tmp3_);
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp5_ = _tmp4_;
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp6_ = _tmp5_;
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		gxml_serializable_array_list_deserialize (_tmp2_, _tmp6_, &_inner_error_);
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp6_);
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (c);
#line 155 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (doc);
#line 1775 "SerializableGeeArrayListTest.c"
			goto __catch83_g_error;
		}
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp7_ = c;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp8_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp7_);
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp9_ = _tmp8_;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (_tmp9_ != 2) {
#line 1786 "SerializableGeeArrayListTest.c"
			FILE* _tmp10_ = NULL;
			GXmlSerializableArrayList* _tmp11_ = NULL;
			gint _tmp12_ = 0;
			gint _tmp13_ = 0;
			gchar* _tmp14_ = NULL;
			gchar* _tmp15_ = NULL;
			gchar* _tmp16_ = NULL;
			gchar* _tmp17_ = NULL;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp10_ = stdout;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp11_ = c;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp12_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp11_);
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp13_ = _tmp12_;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp14_ = g_strdup_printf ("%i", _tmp13_);
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp15_ = _tmp14_;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp16_ = g_strconcat ("ERROR: incorrect size must be 2 got: ", _tmp15_, "\n", NULL);
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp17_ = _tmp16_;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			fprintf (_tmp10_, "%s", _tmp17_);
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_free0 (_tmp17_);
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_free0 (_tmp15_);
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			g_assert_not_reached ();
#line 1819 "SerializableGeeArrayListTest.c"
		}
#line 160 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		found1 = FALSE;
#line 161 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		found2 = FALSE;
#line 1825 "SerializableGeeArrayListTest.c"
		{
			GXmlSerializableArrayList* _o_list = NULL;
			GXmlSerializableArrayList* _tmp18_ = NULL;
			GXmlSerializableArrayList* _tmp19_ = NULL;
			gint _o_size = 0;
			GXmlSerializableArrayList* _tmp20_ = NULL;
			gint _tmp21_ = 0;
			gint _tmp22_ = 0;
			gint _o_index = 0;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp18_ = c;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp19_ = _g_object_ref0 (_tmp18_);
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_o_list = _tmp19_;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp20_ = _o_list;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp21_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp20_);
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp22_ = _tmp21_;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_o_size = _tmp22_;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_o_index = -1;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			while (TRUE) {
#line 1853 "SerializableGeeArrayListTest.c"
				gint _tmp23_ = 0;
				gint _tmp24_ = 0;
				gint _tmp25_ = 0;
				AElement* o = NULL;
				GXmlSerializableArrayList* _tmp26_ = NULL;
				gint _tmp27_ = 0;
				gpointer _tmp28_ = NULL;
				AElement* _tmp29_ = NULL;
				const gchar* _tmp30_ = NULL;
				const gchar* _tmp31_ = NULL;
				AElement* _tmp32_ = NULL;
				const gchar* _tmp33_ = NULL;
				const gchar* _tmp34_ = NULL;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp23_ = _o_index;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_o_index = _tmp23_ + 1;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp24_ = _o_index;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp25_ = _o_size;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				if (!(_tmp24_ < _tmp25_)) {
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					break;
#line 1879 "SerializableGeeArrayListTest.c"
				}
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp26_ = _o_list;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp27_ = _o_index;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp28_ = gee_abstract_list_get ((GeeAbstractList*) _tmp26_, _tmp27_);
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				o = (AElement*) _tmp28_;
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp29_ = o;
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp30_ = aelement_get_name (_tmp29_);
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp31_ = _tmp30_;
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				if (g_strcmp0 (_tmp31_, "Big") == 0) {
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					found1 = TRUE;
#line 1899 "SerializableGeeArrayListTest.c"
				}
#line 164 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp32_ = o;
#line 164 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp33_ = aelement_get_name (_tmp32_);
#line 164 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp34_ = _tmp33_;
#line 164 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				if (g_strcmp0 (_tmp34_, "Small") == 0) {
#line 164 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					found2 = TRUE;
#line 1911 "SerializableGeeArrayListTest.c"
				}
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_g_object_unref0 (o);
#line 1915 "SerializableGeeArrayListTest.c"
			}
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (_o_list);
#line 1919 "SerializableGeeArrayListTest.c"
		}
#line 166 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp35_ = found1;
#line 166 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (!_tmp35_) {
#line 1925 "SerializableGeeArrayListTest.c"
			FILE* _tmp36_ = NULL;
#line 167 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp36_ = stdout;
#line 167 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			fprintf (_tmp36_, "%s", "ERROR: Big key value is not found\n");
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			g_assert_not_reached ();
#line 1933 "SerializableGeeArrayListTest.c"
		}
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp37_ = found2;
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (!_tmp37_) {
#line 1939 "SerializableGeeArrayListTest.c"
			FILE* _tmp38_ = NULL;
#line 171 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp38_ = stdout;
#line 171 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			fprintf (_tmp38_, "%s", "ERROR: Small key value is not found\n");
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			g_assert_not_reached ();
#line 1947 "SerializableGeeArrayListTest.c"
		}
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (c);
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (doc);
#line 1953 "SerializableGeeArrayListTest.c"
	}
	goto __finally83;
	__catch83_g_error:
	{
		GError* e = NULL;
		FILE* _tmp39_ = NULL;
		const gchar* _tmp40_ = NULL;
		const gchar* _tmp41_ = NULL;
		gchar* _tmp42_ = NULL;
		gchar* _tmp43_ = NULL;
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		e = _inner_error_;
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_inner_error_ = NULL;
#line 176 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp39_ = stdout;
#line 176 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp40_ = e->message;
#line 176 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp41_ = string_to_string (_tmp40_);
#line 176 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp42_ = g_strconcat ("ERROR: ", _tmp41_, NULL);
#line 176 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp43_ = _tmp42_;
#line 176 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		fprintf (_tmp39_, "%s", _tmp43_);
#line 176 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_free0 (_tmp43_);
#line 177 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_assert_not_reached ();
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_error_free0 (e);
#line 1986 "SerializableGeeArrayListTest.c"
	}
	__finally83:
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_clear_error (&_inner_error_);
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		return;
#line 1997 "SerializableGeeArrayListTest.c"
	}
}


static void ___lambda62__gtest_func (void) {
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	__lambda62_ ();
#line 2005 "SerializableGeeArrayListTest.c"
}


static void __lambda63_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* idoc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		GXmlNode* iroot = NULL;
		GXmlGDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlNode* _tmp3_ = NULL;
		GXmlSerializableArrayList* ic = NULL;
		GXmlSerializableArrayList* _tmp4_ = NULL;
		GXmlSerializableArrayList* _tmp5_ = NULL;
		GXmlNode* _tmp6_ = NULL;
		GXmlTDocument* doc = NULL;
		GXmlTDocument* _tmp7_ = NULL;
		GXmlNode* root = NULL;
		GXmlTDocument* _tmp8_ = NULL;
		GXmlNode* _tmp9_ = NULL;
		GXmlTDocument* _tmp10_ = NULL;
		GeeBidirList* _tmp11_ = NULL;
		GeeBidirList* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GXmlNode* _tmp14_ = NULL;
		GXmlSerializableArrayList* _tmp15_ = NULL;
		GXmlNode* _tmp16_ = NULL;
		GXmlNode* _tmp17_ = NULL;
		GXmlNode* _tmp18_ = NULL;
		GXmlSerializableArrayList* c = NULL;
		GXmlSerializableArrayList* _tmp19_ = NULL;
		GXmlSerializableArrayList* _tmp20_ = NULL;
		GXmlNode* _tmp21_ = NULL;
		GXmlSerializableArrayList* _tmp22_ = NULL;
		gint _tmp23_ = 0;
		gint _tmp24_ = 0;
		gint count_elements = 0;
		gint _tmp37_ = 0;
		gchar** s = NULL;
		gchar* _tmp38_ = NULL;
		gchar* _tmp39_ = NULL;
		gchar* _tmp40_ = NULL;
		gchar** _tmp41_ = NULL;
		gint s_length1 = 0;
		gint _s_size_ = 0;
#line 183 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp0_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n" \
"    <root>\n" \
"      <aelement name=\"Big\"/>\n" \
"      <aelement name=\"Small\"/>\n" \
"      <aelement name=\"Wall\">FAKE1</aelement>\n" \
"    </root>", 0, &_inner_error_);
#line 183 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		idoc = _tmp0_;
#line 183 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 2058 "SerializableGeeArrayListTest.c"
			goto __catch84_g_error;
		}
#line 189 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp1_ = idoc;
#line 189 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp2_ = gxml_document_get_root ((GXmlDocument*) _tmp1_);
#line 189 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp3_ = _tmp2_;
#line 189 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		iroot = _tmp3_;
#line 190 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp4_ = gxml_serializable_array_list_new (TYPE_AELEMENT, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 190 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		ic = _tmp4_;
#line 191 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp5_ = ic;
#line 191 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp6_ = iroot;
#line 191 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		gxml_serializable_array_list_deserialize (_tmp5_, _tmp6_, &_inner_error_);
#line 191 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 191 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (ic);
#line 191 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (iroot);
#line 191 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (idoc);
#line 2087 "SerializableGeeArrayListTest.c"
			goto __catch84_g_error;
		}
#line 192 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp7_ = gxml_tdocument_new ();
#line 192 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		doc = _tmp7_;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp8_ = doc;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp9_ = gxml_document_create_element ((GXmlDocument*) _tmp8_, "root", &_inner_error_);
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		root = _tmp9_;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (doc);
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (ic);
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (iroot);
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (idoc);
#line 2110 "SerializableGeeArrayListTest.c"
			goto __catch84_g_error;
		}
#line 194 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp10_ = doc;
#line 194 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp11_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp10_);
#line 194 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp12_ = _tmp11_;
#line 194 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp13_ = _tmp12_;
#line 194 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp14_ = root;
#line 194 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		gee_collection_add ((GeeCollection*) _tmp13_, _tmp14_);
#line 194 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp13_);
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp15_ = ic;
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp16_ = root;
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp17_ = gxml_serializable_array_list_serialize (_tmp15_, _tmp16_, &_inner_error_);
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp18_ = _tmp17_;
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp18_);
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (root);
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (doc);
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (ic);
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (iroot);
#line 195 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (idoc);
#line 2149 "SerializableGeeArrayListTest.c"
			goto __catch84_g_error;
		}
#line 196 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp19_ = gxml_serializable_array_list_new (TYPE_AELEMENT, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 196 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		c = _tmp19_;
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp20_ = c;
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp21_ = root;
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		gxml_serializable_array_list_deserialize (_tmp20_, _tmp21_, &_inner_error_);
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (c);
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (root);
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (doc);
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (ic);
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (iroot);
#line 197 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (idoc);
#line 2176 "SerializableGeeArrayListTest.c"
			goto __catch84_g_error;
		}
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp22_ = c;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp23_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp22_);
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp24_ = _tmp23_;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp24_ == 3, "c.size == 3");
#line 199 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		count_elements = 0;
#line 2189 "SerializableGeeArrayListTest.c"
		{
			GXmlSerializableArrayList* _e_list = NULL;
			GXmlSerializableArrayList* _tmp25_ = NULL;
			GXmlSerializableArrayList* _tmp26_ = NULL;
			gint _e_size = 0;
			GXmlSerializableArrayList* _tmp27_ = NULL;
			gint _tmp28_ = 0;
			gint _tmp29_ = 0;
			gint _e_index = 0;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp25_ = c;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp26_ = _g_object_ref0 (_tmp25_);
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_e_list = _tmp26_;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp27_ = _e_list;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp28_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp27_);
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_tmp29_ = _tmp28_;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_e_size = _tmp29_;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_e_index = -1;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			while (TRUE) {
#line 2217 "SerializableGeeArrayListTest.c"
				gint _tmp30_ = 0;
				gint _tmp31_ = 0;
				gint _tmp32_ = 0;
				AElement* e = NULL;
				GXmlSerializableArrayList* _tmp33_ = NULL;
				gint _tmp34_ = 0;
				gpointer _tmp35_ = NULL;
				gint _tmp36_ = 0;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp30_ = _e_index;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_e_index = _tmp30_ + 1;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp31_ = _e_index;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp32_ = _e_size;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				if (!(_tmp31_ < _tmp32_)) {
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					break;
#line 2238 "SerializableGeeArrayListTest.c"
				}
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp33_ = _e_list;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp34_ = _e_index;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp35_ = gee_abstract_list_get ((GeeAbstractList*) _tmp33_, _tmp34_);
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				e = (AElement*) _tmp35_;
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp36_ = count_elements;
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				count_elements = _tmp36_ + 1;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_g_object_unref0 (e);
#line 2254 "SerializableGeeArrayListTest.c"
			}
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (_e_list);
#line 2258 "SerializableGeeArrayListTest.c"
		}
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp37_ = count_elements;
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp37_ == 3, "count_elements == 3");
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp38_ = g_strdup ("Big");
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp39_ = g_strdup ("Small");
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp40_ = g_strdup ("Wall");
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp41_ = g_new0 (gchar*, 3 + 1);
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp41_[0] = _tmp38_;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp41_[1] = _tmp39_;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp41_[2] = _tmp40_;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		s = _tmp41_;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		s_length1 = 3;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_s_size_ = s_length1;
#line 2284 "SerializableGeeArrayListTest.c"
		{
			gint j = 0;
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			j = 0;
#line 2289 "SerializableGeeArrayListTest.c"
			{
				gboolean _tmp42_ = FALSE;
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp42_ = TRUE;
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				while (TRUE) {
#line 2296 "SerializableGeeArrayListTest.c"
					gint _tmp44_ = 0;
					GXmlSerializableArrayList* _tmp45_ = NULL;
					gint _tmp46_ = 0;
					gint _tmp47_ = 0;
					gchar** _tmp48_ = NULL;
					gint _tmp48__length1 = 0;
					gint _tmp49_ = 0;
					const gchar* _tmp50_ = NULL;
					GXmlSerializableArrayList* _tmp51_ = NULL;
					gint _tmp52_ = 0;
					gpointer _tmp53_ = NULL;
					AElement* _tmp54_ = NULL;
					const gchar* _tmp55_ = NULL;
					const gchar* _tmp56_ = NULL;
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					if (!_tmp42_) {
#line 2313 "SerializableGeeArrayListTest.c"
						gint _tmp43_ = 0;
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
						_tmp43_ = j;
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
						j = _tmp43_ + 1;
#line 2319 "SerializableGeeArrayListTest.c"
					}
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp42_ = FALSE;
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp44_ = j;
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp45_ = c;
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp46_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp45_);
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp47_ = _tmp46_;
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					if (!(_tmp44_ < _tmp47_)) {
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
						break;
#line 2335 "SerializableGeeArrayListTest.c"
					}
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp48_ = s;
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp48__length1 = s_length1;
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp49_ = j;
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp50_ = _tmp48_[_tmp49_];
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp51_ = c;
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp52_ = j;
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp53_ = gee_abstract_list_get ((GeeAbstractList*) _tmp51_, _tmp52_);
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp54_ = (AElement*) _tmp53_;
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp55_ = aelement_get_name (_tmp54_);
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp56_ = _tmp55_;
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_vala_assert (g_strcmp0 (_tmp50_, _tmp56_) == 0, "s[j] == c.get (j).name");
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_g_object_unref0 (_tmp54_);
#line 2361 "SerializableGeeArrayListTest.c"
				}
			}
		}
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		s = (_vala_array_free (s, s_length1, (GDestroyNotify) g_free), NULL);
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (c);
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (root);
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (doc);
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (ic);
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (iroot);
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (idoc);
#line 2379 "SerializableGeeArrayListTest.c"
	}
	goto __finally84;
	__catch84_g_error:
	{
		GError* e = NULL;
		FILE* _tmp57_ = NULL;
		const gchar* _tmp58_ = NULL;
		const gchar* _tmp59_ = NULL;
		gchar* _tmp60_ = NULL;
		gchar* _tmp61_ = NULL;
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		e = _inner_error_;
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_inner_error_ = NULL;
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp57_ = stdout;
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp58_ = e->message;
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp59_ = string_to_string (_tmp58_);
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp60_ = g_strconcat ("ERROR: ", _tmp59_, NULL);
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp61_ = _tmp60_;
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		fprintf (_tmp57_, "%s", _tmp61_);
#line 217 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_free0 (_tmp61_);
#line 218 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_assert_not_reached ();
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_error_free0 (e);
#line 2412 "SerializableGeeArrayListTest.c"
	}
	__finally84:
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_clear_error (&_inner_error_);
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		return;
#line 2423 "SerializableGeeArrayListTest.c"
	}
}


static void ___lambda63__gtest_func (void) {
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	__lambda63_ ();
#line 2431 "SerializableGeeArrayListTest.c"
}


static void __lambda64_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* d = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		BookStore* bs = NULL;
		BookStore* _tmp1_ = NULL;
		BookStore* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		BookStore* _tmp4_ = NULL;
		const gchar* _tmp5_ = NULL;
		const gchar* _tmp6_ = NULL;
		BookStore* _tmp7_ = NULL;
		BookArray* _tmp8_ = NULL;
		BookArray* _tmp9_ = NULL;
		gint _tmp10_ = 0;
		gint _tmp11_ = 0;
		Book* b = NULL;
		BookStore* _tmp12_ = NULL;
		BookArray* _tmp13_ = NULL;
		BookArray* _tmp14_ = NULL;
		gpointer _tmp15_ = NULL;
		Name* _tmp16_ = NULL;
		Name* _tmp17_ = NULL;
		Name* _tmp18_ = NULL;
		Name* _tmp19_ = NULL;
		gchar* _tmp20_ = NULL;
		gchar* _tmp21_ = NULL;
		const gchar* _tmp22_ = NULL;
		const gchar* _tmp23_ = NULL;
		Authors* _tmp24_ = NULL;
		Authors* _tmp25_ = NULL;
		Authors* _tmp26_ = NULL;
		Authors* _tmp27_ = NULL;
		AuthorArray* _tmp28_ = NULL;
		AuthorArray* _tmp29_ = NULL;
		Authors* _tmp30_ = NULL;
		Authors* _tmp31_ = NULL;
		AuthorArray* _tmp32_ = NULL;
		AuthorArray* _tmp33_ = NULL;
		gint _tmp34_ = 0;
		gint _tmp35_ = 0;
		Author* a = NULL;
		Authors* _tmp36_ = NULL;
		Authors* _tmp37_ = NULL;
		AuthorArray* _tmp38_ = NULL;
		AuthorArray* _tmp39_ = NULL;
		gpointer _tmp40_ = NULL;
		Name* _tmp41_ = NULL;
		Name* _tmp42_ = NULL;
		Name* _tmp43_ = NULL;
		Name* _tmp44_ = NULL;
		gchar* _tmp45_ = NULL;
		gchar* _tmp46_ = NULL;
		Email* _tmp47_ = NULL;
		Email* _tmp48_ = NULL;
		Email* _tmp49_ = NULL;
		Email* _tmp50_ = NULL;
		gchar* _tmp51_ = NULL;
		gchar* _tmp52_ = NULL;
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp0_ = gxml_gdocument_new_from_path (TEST_DIR "/test-collection.xml", 0, &_inner_error_);
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		d = _tmp0_;
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 2501 "SerializableGeeArrayListTest.c"
			goto __catch85_g_error;
		}
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp1_ = book_store_new ();
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		bs = _tmp1_;
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp2_ = bs;
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp3_ = d;
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp2_, (GXmlNode*) _tmp3_, &_inner_error_);
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (bs);
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (d);
#line 2520 "SerializableGeeArrayListTest.c"
			goto __catch85_g_error;
		}
#line 227 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp4_ = bs;
#line 227 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp5_ = book_store_get_name (_tmp4_);
#line 227 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp6_ = _tmp5_;
#line 227 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (g_strcmp0 (_tmp6_, "The Great Book") == 0, "bs.name == \"The Great Book\"");
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp7_ = bs;
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp8_ = book_store_get_books (_tmp7_);
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp9_ = _tmp8_;
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp10_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp9_);
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp11_ = _tmp10_;
#line 228 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp11_ == 3, "bs.books.size == 3");
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp12_ = bs;
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp13_ = book_store_get_books (_tmp12_);
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp14_ = _tmp13_;
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp15_ = gee_list_first ((GeeList*) _tmp14_);
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		b = (Book*) _tmp15_;
#line 230 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (b != NULL, "b != null");
#line 231 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp16_ = book_get_name (b);
#line 231 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp17_ = _tmp16_;
#line 231 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp17_ != NULL, "b.name != null");
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp18_ = book_get_name (b);
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp19_ = _tmp18_;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp20_ = name_get_name (_tmp19_);
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp21_ = _tmp20_;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (g_strcmp0 (_tmp21_, "Book1") == 0, "b.name.get_name () == \"Book1\"");
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_free0 (_tmp21_);
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp22_ = book_get_year (b);
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp23_ = _tmp22_;
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (g_strcmp0 (_tmp23_, "2015") == 0, "b.year == \"2015\"");
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp24_ = book_get_authors (b);
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp25_ = _tmp24_;
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp25_ != NULL, "b.authors != null");
#line 235 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp26_ = book_get_authors (b);
#line 235 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp27_ = _tmp26_;
#line 235 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp28_ = authors_get_array (_tmp27_);
#line 235 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp29_ = _tmp28_;
#line 235 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp29_ != NULL, "b.authors.array != null");
#line 236 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp30_ = book_get_authors (b);
#line 236 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp31_ = _tmp30_;
#line 236 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp32_ = authors_get_array (_tmp31_);
#line 236 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp33_ = _tmp32_;
#line 236 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp34_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp33_);
#line 236 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp35_ = _tmp34_;
#line 236 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp35_ == 2, "b.authors.array.size == 2");
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp36_ = book_get_authors (b);
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp37_ = _tmp36_;
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp38_ = authors_get_array (_tmp37_);
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp39_ = _tmp38_;
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp40_ = gee_list_first ((GeeList*) _tmp39_);
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		a = (Author*) _tmp40_;
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (a != NULL, "a != null");
#line 239 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp41_ = author_get_name (a);
#line 239 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp42_ = _tmp41_;
#line 239 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp42_ != NULL, "a.name != null");
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp43_ = author_get_name (a);
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp44_ = _tmp43_;
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp45_ = name_get_name (_tmp44_);
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp46_ = _tmp45_;
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (g_strcmp0 (_tmp46_, "Fred") == 0, "a.name.get_name () == \"Fred\"");
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_free0 (_tmp46_);
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp47_ = author_get_email (a);
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp48_ = _tmp47_;
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp48_ != NULL, "a.email != null");
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp49_ = author_get_email (a);
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp50_ = _tmp49_;
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp51_ = email_get_mail (_tmp50_);
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp52_ = _tmp51_;
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (g_strcmp0 (_tmp52_, "fweasley@hogwarts.co.uk") == 0, "a.email.get_mail () == \"fweasley@hogwarts.co.uk\"");
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_free0 (_tmp52_);
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (a);
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (b);
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (bs);
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (d);
#line 2667 "SerializableGeeArrayListTest.c"
	}
	goto __finally85;
	__catch85_g_error:
	{
		GError* e = NULL;
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		e = _inner_error_;
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_inner_error_ = NULL;
#line 247 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_assert_not_reached ();
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_error_free0 (e);
#line 2681 "SerializableGeeArrayListTest.c"
	}
	__finally85:
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_clear_error (&_inner_error_);
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		return;
#line 2692 "SerializableGeeArrayListTest.c"
	}
}


static void ___lambda64__gtest_func (void) {
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	__lambda64_ ();
#line 2700 "SerializableGeeArrayListTest.c"
}


static void __lambda65_ (void) {
	GError * _inner_error_ = NULL;
	{
		SerializableGeeArrayListTestBigBag* bag = NULL;
		SerializableGeeArrayListTestBigBag* _tmp0_ = NULL;
		SerializableGeeArrayListTestBigBag* _tmp1_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp2_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp3_ = NULL;
		SerializableGeeArrayListTestBigBag* _tmp34_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp35_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp36_ = NULL;
		gint _tmp37_ = 0;
		gint _tmp38_ = 0;
		GXmlGDocument* d = NULL;
		GXmlGDocument* _tmp39_ = NULL;
		SerializableGeeArrayListTestBigBag* _tmp40_ = NULL;
		GXmlGDocument* _tmp41_ = NULL;
		GXmlNode* _tmp42_ = NULL;
		GXmlNode* _tmp43_ = NULL;
		GXmlGDocument* _tmp44_ = NULL;
		GXmlNode* _tmp45_ = NULL;
		GXmlNode* _tmp46_ = NULL;
		GXmlNode* _tmp47_ = NULL;
		GXmlGDocument* _tmp48_ = NULL;
		GXmlNode* _tmp49_ = NULL;
		GXmlNode* _tmp50_ = NULL;
		GXmlNode* _tmp51_ = NULL;
		gchar* _tmp52_ = NULL;
		gchar* _tmp53_ = NULL;
		gchar* _tmp54_ = NULL;
		GXmlGDocument* _tmp55_ = NULL;
		GXmlNode* _tmp56_ = NULL;
		GXmlNode* _tmp57_ = NULL;
		GXmlNode* _tmp58_ = NULL;
		GeeBidirList* _tmp59_ = NULL;
		GeeBidirList* _tmp60_ = NULL;
		GeeBidirList* _tmp61_ = NULL;
		gint _tmp62_ = 0;
		gint _tmp63_ = 0;
		GXmlGDocument* _tmp64_ = NULL;
		GXmlNode* _tmp65_ = NULL;
		GXmlNode* _tmp66_ = NULL;
		GXmlNode* _tmp67_ = NULL;
		GeeBidirList* _tmp68_ = NULL;
		GeeBidirList* _tmp69_ = NULL;
		GeeBidirList* _tmp70_ = NULL;
		gpointer _tmp71_ = NULL;
		GXmlNode* _tmp72_ = NULL;
		gchar* _tmp73_ = NULL;
		gchar* _tmp74_ = NULL;
		gchar* _tmp75_ = NULL;
		GXmlGDocument* _tmp76_ = NULL;
		GXmlNode* _tmp77_ = NULL;
		GXmlNode* _tmp78_ = NULL;
		GXmlNode* _tmp79_ = NULL;
		GeeBidirList* _tmp80_ = NULL;
		GeeBidirList* _tmp81_ = NULL;
		GeeBidirList* _tmp82_ = NULL;
		gpointer _tmp83_ = NULL;
		GXmlNode* _tmp84_ = NULL;
		GeeBidirList* _tmp85_ = NULL;
		GeeBidirList* _tmp86_ = NULL;
		GeeBidirList* _tmp87_ = NULL;
		gint _tmp88_ = 0;
		gint _tmp89_ = 0;
		GXmlGDocument* _tmp90_ = NULL;
		GXmlNode* _tmp91_ = NULL;
		GXmlNode* _tmp92_ = NULL;
		GXmlNode* _tmp93_ = NULL;
		GeeBidirList* _tmp94_ = NULL;
		GeeBidirList* _tmp95_ = NULL;
		GeeBidirList* _tmp96_ = NULL;
		gpointer _tmp97_ = NULL;
		GXmlNode* _tmp98_ = NULL;
		GeeBidirList* _tmp99_ = NULL;
		GeeBidirList* _tmp100_ = NULL;
		GeeBidirList* _tmp101_ = NULL;
		gpointer _tmp102_ = NULL;
		GXmlNode* _tmp103_ = NULL;
		gchar* _tmp104_ = NULL;
		gchar* _tmp105_ = NULL;
		gchar* _tmp106_ = NULL;
		GXmlGDocument* _tmp107_ = NULL;
		GXmlNode* _tmp108_ = NULL;
		GXmlNode* _tmp109_ = NULL;
		GXmlNode* _tmp110_ = NULL;
		GeeBidirList* _tmp111_ = NULL;
		GeeBidirList* _tmp112_ = NULL;
		GeeBidirList* _tmp113_ = NULL;
		gpointer _tmp114_ = NULL;
		GXmlNode* _tmp115_ = NULL;
		GeeBidirList* _tmp116_ = NULL;
		GeeBidirList* _tmp117_ = NULL;
		GeeBidirList* _tmp118_ = NULL;
		gpointer _tmp119_ = NULL;
		GXmlNode* _tmp120_ = NULL;
		GeeBidirList* _tmp121_ = NULL;
		GeeBidirList* _tmp122_ = NULL;
		GeeBidirList* _tmp123_ = NULL;
		gint _tmp124_ = 0;
		gint _tmp125_ = 0;
		GXmlGDocument* _tmp126_ = NULL;
		GXmlNode* _tmp127_ = NULL;
		GXmlNode* _tmp128_ = NULL;
		GXmlNode* _tmp129_ = NULL;
		GeeBidirList* _tmp130_ = NULL;
		GeeBidirList* _tmp131_ = NULL;
		GeeBidirList* _tmp132_ = NULL;
		gpointer _tmp133_ = NULL;
		GXmlNode* _tmp134_ = NULL;
		GeeBidirList* _tmp135_ = NULL;
		GeeBidirList* _tmp136_ = NULL;
		GeeBidirList* _tmp137_ = NULL;
		gpointer _tmp138_ = NULL;
		GXmlNode* _tmp139_ = NULL;
		GeeBidirList* _tmp140_ = NULL;
		GeeBidirList* _tmp141_ = NULL;
		GeeBidirList* _tmp142_ = NULL;
		gpointer _tmp143_ = NULL;
		GXmlNode* _tmp144_ = NULL;
		gchar* _tmp145_ = NULL;
		gchar* _tmp146_ = NULL;
		gchar* _tmp147_ = NULL;
		GXmlGDocument* _tmp148_ = NULL;
		GXmlNode* _tmp149_ = NULL;
		GXmlNode* _tmp150_ = NULL;
		GXmlNode* _tmp151_ = NULL;
		GeeBidirList* _tmp152_ = NULL;
		GeeBidirList* _tmp153_ = NULL;
		GeeBidirList* _tmp154_ = NULL;
		gpointer _tmp155_ = NULL;
		GXmlNode* _tmp156_ = NULL;
		GeeBidirList* _tmp157_ = NULL;
		GeeBidirList* _tmp158_ = NULL;
		GeeBidirList* _tmp159_ = NULL;
		gpointer _tmp160_ = NULL;
		GXmlNode* _tmp161_ = NULL;
		GeeBidirList* _tmp162_ = NULL;
		GeeBidirList* _tmp163_ = NULL;
		GeeBidirList* _tmp164_ = NULL;
		gpointer _tmp165_ = NULL;
		GXmlNode* _tmp166_ = NULL;
		GeeBidirList* _tmp167_ = NULL;
		GeeBidirList* _tmp168_ = NULL;
		GeeBidirList* _tmp169_ = NULL;
		gint _tmp170_ = 0;
		gint _tmp171_ = 0;
		GXmlGDocument* _tmp172_ = NULL;
		GXmlNode* _tmp173_ = NULL;
		GXmlNode* _tmp174_ = NULL;
		GXmlNode* _tmp175_ = NULL;
		GeeBidirList* _tmp176_ = NULL;
		GeeBidirList* _tmp177_ = NULL;
		GeeBidirList* _tmp178_ = NULL;
		gpointer _tmp179_ = NULL;
		GXmlNode* _tmp180_ = NULL;
		GeeBidirList* _tmp181_ = NULL;
		GeeBidirList* _tmp182_ = NULL;
		GeeBidirList* _tmp183_ = NULL;
		gpointer _tmp184_ = NULL;
		GXmlNode* _tmp185_ = NULL;
		GeeBidirList* _tmp186_ = NULL;
		GeeBidirList* _tmp187_ = NULL;
		GeeBidirList* _tmp188_ = NULL;
		gpointer _tmp189_ = NULL;
		GXmlNode* _tmp190_ = NULL;
		GeeBidirList* _tmp191_ = NULL;
		GeeBidirList* _tmp192_ = NULL;
		GeeBidirList* _tmp193_ = NULL;
		gpointer _tmp194_ = NULL;
		GXmlNode* _tmp195_ = NULL;
		GXmlGDocument* _tmp196_ = NULL;
		GXmlNode* _tmp197_ = NULL;
		GXmlNode* _tmp198_ = NULL;
		GXmlNode* _tmp199_ = NULL;
		GeeBidirList* _tmp200_ = NULL;
		GeeBidirList* _tmp201_ = NULL;
		GeeBidirList* _tmp202_ = NULL;
		gpointer _tmp203_ = NULL;
		GXmlNode* _tmp204_ = NULL;
		GeeBidirList* _tmp205_ = NULL;
		GeeBidirList* _tmp206_ = NULL;
		GeeBidirList* _tmp207_ = NULL;
		gpointer _tmp208_ = NULL;
		GXmlNode* _tmp209_ = NULL;
		GeeBidirList* _tmp210_ = NULL;
		GeeBidirList* _tmp211_ = NULL;
		GeeBidirList* _tmp212_ = NULL;
		gpointer _tmp213_ = NULL;
		GXmlNode* _tmp214_ = NULL;
		GeeBidirList* _tmp215_ = NULL;
		GeeBidirList* _tmp216_ = NULL;
		GeeBidirList* _tmp217_ = NULL;
		gpointer _tmp218_ = NULL;
		GXmlNode* _tmp219_ = NULL;
		gchar* _tmp220_ = NULL;
		gchar* _tmp221_ = NULL;
		gchar* _tmp222_ = NULL;
		SerializableGeeArrayListTestBigBag* bagt = NULL;
		SerializableGeeArrayListTestBigBag* _tmp223_ = NULL;
		SerializableGeeArrayListTestBigBag* _tmp224_ = NULL;
		GXmlGDocument* _tmp225_ = NULL;
		gboolean _tmp226_ = FALSE;
		SerializableGeeArrayListTestBigBag* _tmp227_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp228_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp229_ = NULL;
		gboolean _tmp230_ = FALSE;
		SerializableGeeArrayListTestBigBag* _tmp231_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp232_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp233_ = NULL;
		gint _tmp234_ = 0;
		gint _tmp235_ = 0;
		gboolean _tmp236_ = FALSE;
		SerializableGeeArrayListTestBigBag* _tmp237_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp238_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp239_ = NULL;
		gpointer _tmp240_ = NULL;
		SerializableGeeArrayListTestSmallBag* _tmp241_ = NULL;
		SerializableGeeArrayListTestBallArray* _tmp242_ = NULL;
		SerializableGeeArrayListTestBallArray* _tmp243_ = NULL;
		gboolean _tmp244_ = FALSE;
		gboolean _tmp245_ = FALSE;
		SerializableGeeArrayListTestBigBag* _tmp246_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp247_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp248_ = NULL;
		gpointer _tmp249_ = NULL;
		SerializableGeeArrayListTestSmallBag* _tmp250_ = NULL;
		SerializableGeeArrayListTestBallArray* _tmp251_ = NULL;
		SerializableGeeArrayListTestBallArray* _tmp252_ = NULL;
		gint _tmp253_ = 0;
		gint _tmp254_ = 0;
		SerializableGeeArrayListTestBigBag* _tmp255_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp256_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp257_ = NULL;
		gpointer _tmp258_ = NULL;
		SerializableGeeArrayListTestSmallBag* _tmp259_ = NULL;
		SerializableGeeArrayListTestBallArray* _tmp260_ = NULL;
		SerializableGeeArrayListTestBallArray* _tmp261_ = NULL;
		gpointer _tmp262_ = NULL;
		SerializableGeeArrayListTestBall* _tmp263_ = NULL;
		const gchar* _tmp264_ = NULL;
		const gchar* _tmp265_ = NULL;
		SerializableGeeArrayListTestBigBag* _tmp266_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp267_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp268_ = NULL;
		gpointer _tmp269_ = NULL;
		SerializableGeeArrayListTestSmallBag* _tmp270_ = NULL;
		SerializableGeeArrayListTestBallArray* _tmp271_ = NULL;
		SerializableGeeArrayListTestBallArray* _tmp272_ = NULL;
		gpointer _tmp273_ = NULL;
		SerializableGeeArrayListTestBall* _tmp274_ = NULL;
		SerializableGeeArrayListTestBallFill* _tmp275_ = NULL;
		SerializableGeeArrayListTestBallFill* _tmp276_ = NULL;
		SerializableGeeArrayListTestBigBag* _tmp277_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp278_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp279_ = NULL;
		gpointer _tmp280_ = NULL;
		SerializableGeeArrayListTestSmallBag* _tmp281_ = NULL;
		SerializableGeeArrayListTestBallArray* _tmp282_ = NULL;
		SerializableGeeArrayListTestBallArray* _tmp283_ = NULL;
		gpointer _tmp284_ = NULL;
		SerializableGeeArrayListTestBall* _tmp285_ = NULL;
		SerializableGeeArrayListTestBallFill* _tmp286_ = NULL;
		SerializableGeeArrayListTestBallFill* _tmp287_ = NULL;
		gchar* _tmp288_ = NULL;
		gchar* _tmp289_ = NULL;
		gchar* _tmp290_ = NULL;
		SerializableGeeArrayListTestBigBag* _tmp291_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp292_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp293_ = NULL;
		gpointer _tmp294_ = NULL;
		SerializableGeeArrayListTestSmallBag* _tmp295_ = NULL;
		SerializableGeeArrayListTestBallArray* _tmp296_ = NULL;
		SerializableGeeArrayListTestBallArray* _tmp297_ = NULL;
		gpointer _tmp298_ = NULL;
		SerializableGeeArrayListTestBall* _tmp299_ = NULL;
		SerializableGeeArrayListTestBallFill* _tmp300_ = NULL;
		SerializableGeeArrayListTestBallFill* _tmp301_ = NULL;
		gchar* _tmp302_ = NULL;
		gchar* _tmp303_ = NULL;
		gchar* _tmp304_ = NULL;
		SerializableGeeArrayListTestBigBag* bag2 = NULL;
		SerializableGeeArrayListTestBigBag* _tmp305_ = NULL;
		SerializableGeeArrayListTestBigBag* _tmp306_ = NULL;
		GXmlGDocument* _tmp307_ = NULL;
		SerializableGeeArrayListTestBigBag* _tmp308_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp309_ = NULL;
		SerializableGeeArrayListTestSmallBagArray* _tmp310_ = NULL;
		gint _tmp311_ = 0;
		gint _tmp312_ = 0;
		GXmlGDocument* d2 = NULL;
		GXmlGDocument* _tmp313_ = NULL;
		SerializableGeeArrayListTestBigBag* _tmp314_ = NULL;
		GXmlGDocument* _tmp315_ = NULL;
		GXmlNode* _tmp316_ = NULL;
		GXmlNode* _tmp317_ = NULL;
		GXmlGDocument* _tmp318_ = NULL;
		GXmlNode* _tmp319_ = NULL;
		GXmlNode* _tmp320_ = NULL;
		GXmlNode* _tmp321_ = NULL;
		GXmlGDocument* _tmp322_ = NULL;
		GXmlNode* _tmp323_ = NULL;
		GXmlNode* _tmp324_ = NULL;
		GXmlNode* _tmp325_ = NULL;
		gchar* _tmp326_ = NULL;
		gchar* _tmp327_ = NULL;
		gchar* _tmp328_ = NULL;
		GXmlGDocument* _tmp329_ = NULL;
		GXmlNode* _tmp330_ = NULL;
		GXmlNode* _tmp331_ = NULL;
		GXmlNode* _tmp332_ = NULL;
		GeeBidirList* _tmp333_ = NULL;
		GeeBidirList* _tmp334_ = NULL;
		GeeBidirList* _tmp335_ = NULL;
		gint _tmp336_ = 0;
		gint _tmp337_ = 0;
		GXmlGDocument* _tmp338_ = NULL;
		GXmlNode* _tmp339_ = NULL;
		GXmlNode* _tmp340_ = NULL;
		GXmlNode* _tmp341_ = NULL;
		GeeBidirList* _tmp342_ = NULL;
		GeeBidirList* _tmp343_ = NULL;
		GeeBidirList* _tmp344_ = NULL;
		gpointer _tmp345_ = NULL;
		GXmlNode* _tmp346_ = NULL;
		gchar* _tmp347_ = NULL;
		gchar* _tmp348_ = NULL;
		gchar* _tmp349_ = NULL;
		GXmlGDocument* _tmp350_ = NULL;
		GXmlNode* _tmp351_ = NULL;
		GXmlNode* _tmp352_ = NULL;
		GXmlNode* _tmp353_ = NULL;
		GeeBidirList* _tmp354_ = NULL;
		GeeBidirList* _tmp355_ = NULL;
		GeeBidirList* _tmp356_ = NULL;
		gpointer _tmp357_ = NULL;
		GXmlNode* _tmp358_ = NULL;
		GeeBidirList* _tmp359_ = NULL;
		GeeBidirList* _tmp360_ = NULL;
		GeeBidirList* _tmp361_ = NULL;
		gint _tmp362_ = 0;
		gint _tmp363_ = 0;
		GXmlGDocument* _tmp364_ = NULL;
		GXmlNode* _tmp365_ = NULL;
		GXmlNode* _tmp366_ = NULL;
		GXmlNode* _tmp367_ = NULL;
		GeeBidirList* _tmp368_ = NULL;
		GeeBidirList* _tmp369_ = NULL;
		GeeBidirList* _tmp370_ = NULL;
		gpointer _tmp371_ = NULL;
		GXmlNode* _tmp372_ = NULL;
		GeeBidirList* _tmp373_ = NULL;
		GeeBidirList* _tmp374_ = NULL;
		GeeBidirList* _tmp375_ = NULL;
		gpointer _tmp376_ = NULL;
		GXmlNode* _tmp377_ = NULL;
		gchar* _tmp378_ = NULL;
		gchar* _tmp379_ = NULL;
		gchar* _tmp380_ = NULL;
		GXmlGDocument* _tmp381_ = NULL;
		GXmlNode* _tmp382_ = NULL;
		GXmlNode* _tmp383_ = NULL;
		GXmlNode* _tmp384_ = NULL;
		GeeBidirList* _tmp385_ = NULL;
		GeeBidirList* _tmp386_ = NULL;
		GeeBidirList* _tmp387_ = NULL;
		gpointer _tmp388_ = NULL;
		GXmlNode* _tmp389_ = NULL;
		GeeBidirList* _tmp390_ = NULL;
		GeeBidirList* _tmp391_ = NULL;
		GeeBidirList* _tmp392_ = NULL;
		gpointer _tmp393_ = NULL;
		GXmlNode* _tmp394_ = NULL;
		GeeBidirList* _tmp395_ = NULL;
		GeeBidirList* _tmp396_ = NULL;
		GeeBidirList* _tmp397_ = NULL;
		gint _tmp398_ = 0;
		gint _tmp399_ = 0;
		GXmlGDocument* _tmp400_ = NULL;
		GXmlNode* _tmp401_ = NULL;
		GXmlNode* _tmp402_ = NULL;
		GXmlNode* _tmp403_ = NULL;
		GeeBidirList* _tmp404_ = NULL;
		GeeBidirList* _tmp405_ = NULL;
		GeeBidirList* _tmp406_ = NULL;
		gpointer _tmp407_ = NULL;
		GXmlNode* _tmp408_ = NULL;
		GeeBidirList* _tmp409_ = NULL;
		GeeBidirList* _tmp410_ = NULL;
		GeeBidirList* _tmp411_ = NULL;
		gpointer _tmp412_ = NULL;
		GXmlNode* _tmp413_ = NULL;
		GeeBidirList* _tmp414_ = NULL;
		GeeBidirList* _tmp415_ = NULL;
		GeeBidirList* _tmp416_ = NULL;
		gpointer _tmp417_ = NULL;
		GXmlNode* _tmp418_ = NULL;
		gchar* _tmp419_ = NULL;
		gchar* _tmp420_ = NULL;
		gchar* _tmp421_ = NULL;
		GXmlGDocument* _tmp422_ = NULL;
		GXmlNode* _tmp423_ = NULL;
		GXmlNode* _tmp424_ = NULL;
		GXmlNode* _tmp425_ = NULL;
		GeeBidirList* _tmp426_ = NULL;
		GeeBidirList* _tmp427_ = NULL;
		GeeBidirList* _tmp428_ = NULL;
		gpointer _tmp429_ = NULL;
		GXmlNode* _tmp430_ = NULL;
		GeeBidirList* _tmp431_ = NULL;
		GeeBidirList* _tmp432_ = NULL;
		GeeBidirList* _tmp433_ = NULL;
		gpointer _tmp434_ = NULL;
		GXmlNode* _tmp435_ = NULL;
		GeeBidirList* _tmp436_ = NULL;
		GeeBidirList* _tmp437_ = NULL;
		GeeBidirList* _tmp438_ = NULL;
		gpointer _tmp439_ = NULL;
		GXmlNode* _tmp440_ = NULL;
		GeeBidirList* _tmp441_ = NULL;
		GeeBidirList* _tmp442_ = NULL;
		GeeBidirList* _tmp443_ = NULL;
		gint _tmp444_ = 0;
		gint _tmp445_ = 0;
		GXmlGDocument* _tmp446_ = NULL;
		GXmlNode* _tmp447_ = NULL;
		GXmlNode* _tmp448_ = NULL;
		GXmlNode* _tmp449_ = NULL;
		GeeBidirList* _tmp450_ = NULL;
		GeeBidirList* _tmp451_ = NULL;
		GeeBidirList* _tmp452_ = NULL;
		gpointer _tmp453_ = NULL;
		GXmlNode* _tmp454_ = NULL;
		GeeBidirList* _tmp455_ = NULL;
		GeeBidirList* _tmp456_ = NULL;
		GeeBidirList* _tmp457_ = NULL;
		gpointer _tmp458_ = NULL;
		GXmlNode* _tmp459_ = NULL;
		GeeBidirList* _tmp460_ = NULL;
		GeeBidirList* _tmp461_ = NULL;
		GeeBidirList* _tmp462_ = NULL;
		gpointer _tmp463_ = NULL;
		GXmlNode* _tmp464_ = NULL;
		GeeBidirList* _tmp465_ = NULL;
		GeeBidirList* _tmp466_ = NULL;
		GeeBidirList* _tmp467_ = NULL;
		gpointer _tmp468_ = NULL;
		GXmlNode* _tmp469_ = NULL;
		GXmlGDocument* _tmp470_ = NULL;
		GXmlNode* _tmp471_ = NULL;
		GXmlNode* _tmp472_ = NULL;
		GXmlNode* _tmp473_ = NULL;
		GeeBidirList* _tmp474_ = NULL;
		GeeBidirList* _tmp475_ = NULL;
		GeeBidirList* _tmp476_ = NULL;
		gpointer _tmp477_ = NULL;
		GXmlNode* _tmp478_ = NULL;
		GeeBidirList* _tmp479_ = NULL;
		GeeBidirList* _tmp480_ = NULL;
		GeeBidirList* _tmp481_ = NULL;
		gpointer _tmp482_ = NULL;
		GXmlNode* _tmp483_ = NULL;
		GeeBidirList* _tmp484_ = NULL;
		GeeBidirList* _tmp485_ = NULL;
		GeeBidirList* _tmp486_ = NULL;
		gpointer _tmp487_ = NULL;
		GXmlNode* _tmp488_ = NULL;
		GeeBidirList* _tmp489_ = NULL;
		GeeBidirList* _tmp490_ = NULL;
		GeeBidirList* _tmp491_ = NULL;
		gpointer _tmp492_ = NULL;
		GXmlNode* _tmp493_ = NULL;
		gchar* _tmp494_ = NULL;
		gchar* _tmp495_ = NULL;
		gchar* _tmp496_ = NULL;
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp0_ = serializable_gee_array_list_test_big_bag_new ();
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		bag = _tmp0_;
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp1_ = bag;
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp2_ = serializable_gee_array_list_test_big_bag_get_bags (_tmp1_);
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp3_ = _tmp2_;
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp3_ != NULL, "bag.bags != null");
#line 3191 "SerializableGeeArrayListTest.c"
		{
			gint i = 0;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			i = 0;
#line 3196 "SerializableGeeArrayListTest.c"
			{
				gboolean _tmp4_ = FALSE;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				_tmp4_ = TRUE;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
				while (TRUE) {
#line 3203 "SerializableGeeArrayListTest.c"
					gint _tmp6_ = 0;
					SerializableGeeArrayListTestSmallBag* sbag = NULL;
					SerializableGeeArrayListTestSmallBag* _tmp7_ = NULL;
					SerializableGeeArrayListTestSmallBag* _tmp8_ = NULL;
					SerializableGeeArrayListTestBallArray* _tmp9_ = NULL;
					SerializableGeeArrayListTestBallArray* _tmp10_ = NULL;
					SerializableGeeArrayListTestSmallBag* _tmp25_ = NULL;
					SerializableGeeArrayListTestBallArray* _tmp26_ = NULL;
					SerializableGeeArrayListTestBallArray* _tmp27_ = NULL;
					gint _tmp28_ = 0;
					gint _tmp29_ = 0;
					SerializableGeeArrayListTestBigBag* _tmp30_ = NULL;
					SerializableGeeArrayListTestSmallBagArray* _tmp31_ = NULL;
					SerializableGeeArrayListTestSmallBagArray* _tmp32_ = NULL;
					SerializableGeeArrayListTestSmallBag* _tmp33_ = NULL;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					if (!_tmp4_) {
#line 3221 "SerializableGeeArrayListTest.c"
						gint _tmp5_ = 0;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
						_tmp5_ = i;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
						i = _tmp5_ + 1;
#line 3227 "SerializableGeeArrayListTest.c"
					}
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp4_ = FALSE;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp6_ = i;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					if (!(_tmp6_ < 2)) {
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
						break;
#line 3237 "SerializableGeeArrayListTest.c"
					}
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp7_ = serializable_gee_array_list_test_small_bag_new ();
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					sbag = _tmp7_;
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp8_ = sbag;
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp9_ = serializable_gee_array_list_test_small_bag_get_balls (_tmp8_);
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp10_ = _tmp9_;
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_vala_assert (_tmp10_ != NULL, "sbag.balls != null");
#line 3251 "SerializableGeeArrayListTest.c"
					{
						gint j = 0;
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
						j = 0;
#line 3256 "SerializableGeeArrayListTest.c"
						{
							gboolean _tmp11_ = FALSE;
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
							_tmp11_ = TRUE;
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
							while (TRUE) {
#line 3263 "SerializableGeeArrayListTest.c"
								gint _tmp13_ = 0;
								SerializableGeeArrayListTestBall* b = NULL;
								SerializableGeeArrayListTestBall* _tmp14_ = NULL;
								SerializableGeeArrayListTestBall* _tmp15_ = NULL;
								SerializableGeeArrayListTestBallFill* _tmp16_ = NULL;
								SerializableGeeArrayListTestBallFill* _tmp17_ = NULL;
								SerializableGeeArrayListTestBall* _tmp18_ = NULL;
								SerializableGeeArrayListTestBallFill* _tmp19_ = NULL;
								SerializableGeeArrayListTestBallFill* _tmp20_ = NULL;
								SerializableGeeArrayListTestSmallBag* _tmp21_ = NULL;
								SerializableGeeArrayListTestBallArray* _tmp22_ = NULL;
								SerializableGeeArrayListTestBallArray* _tmp23_ = NULL;
								SerializableGeeArrayListTestBall* _tmp24_ = NULL;
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
								if (!_tmp11_) {
#line 3279 "SerializableGeeArrayListTest.c"
									gint _tmp12_ = 0;
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
									_tmp12_ = j;
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
									j = _tmp12_ + 1;
#line 3285 "SerializableGeeArrayListTest.c"
								}
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
								_tmp11_ = FALSE;
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
								_tmp13_ = j;
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
								if (!(_tmp13_ < 2)) {
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
									break;
#line 3295 "SerializableGeeArrayListTest.c"
								}
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
								_tmp14_ = serializable_gee_array_list_test_ball_new ();
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
								b = _tmp14_;
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
								_tmp15_ = b;
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
								_tmp16_ = serializable_gee_array_list_test_ball_fill_new ();
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
								_tmp17_ = _tmp16_;
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
								serializable_gee_array_list_test_ball_set_ballfill (_tmp15_, _tmp17_);
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
								_g_object_unref0 (_tmp17_);
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
								_tmp18_ = b;
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
								_tmp19_ = serializable_gee_array_list_test_ball_get_ballfill (_tmp18_);
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
								_tmp20_ = _tmp19_;
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
								serializable_gee_array_list_test_ball_fill_set_text (_tmp20_, "golden dust");
#line 263 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
								_tmp21_ = sbag;
#line 263 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
								_tmp22_ = serializable_gee_array_list_test_small_bag_get_balls (_tmp21_);
#line 263 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
								_tmp23_ = _tmp22_;
#line 263 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
								_tmp24_ = b;
#line 263 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
								gee_abstract_collection_add ((GeeAbstractCollection*) _tmp23_, _tmp24_);
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
								_g_object_unref0 (b);
#line 3331 "SerializableGeeArrayListTest.c"
							}
						}
					}
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp25_ = sbag;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp26_ = serializable_gee_array_list_test_small_bag_get_balls (_tmp25_);
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp27_ = _tmp26_;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp28_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp27_);
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp29_ = _tmp28_;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_vala_assert (_tmp29_ == 2, "sbag.balls.size == 2");
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp30_ = bag;
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp31_ = serializable_gee_array_list_test_big_bag_get_bags (_tmp30_);
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp32_ = _tmp31_;
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_tmp33_ = sbag;
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					gee_abstract_collection_add ((GeeAbstractCollection*) _tmp32_, _tmp33_);
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
					_g_object_unref0 (sbag);
#line 3359 "SerializableGeeArrayListTest.c"
				}
			}
		}
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp34_ = bag;
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp35_ = serializable_gee_array_list_test_big_bag_get_bags (_tmp34_);
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp36_ = _tmp35_;
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp37_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp36_);
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp38_ = _tmp37_;
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp38_ == 2, "bag.bags.size == 2");
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp39_ = gxml_gdocument_new ();
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		d = _tmp39_;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp40_ = bag;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp41_ = d;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp42_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) _tmp40_, (GXmlNode*) _tmp41_, &_inner_error_);
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp43_ = _tmp42_;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp43_);
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (d);
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (bag);
#line 3395 "SerializableGeeArrayListTest.c"
			goto __catch86_g_error;
		}
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp44_ = d;
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp45_ = gxml_document_get_root ((GXmlDocument*) _tmp44_);
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp46_ = _tmp45_;
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp47_ = _tmp46_;
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp47_ != NULL, "d.root != null");
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp47_);
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp48_ = d;
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp49_ = gxml_document_get_root ((GXmlDocument*) _tmp48_);
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp50_ = _tmp49_;
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp51_ = _tmp50_;
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp52_ = gxml_node_get_name (_tmp51_);
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp53_ = _tmp52_;
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp54_ = _tmp53_;
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (g_strcmp0 (_tmp54_, "BigBag") == 0, "d.root.name == \"BigBag\"");
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_free0 (_tmp54_);
#line 273 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp51_);
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp55_ = d;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp56_ = gxml_document_get_root ((GXmlDocument*) _tmp55_);
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp57_ = _tmp56_;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp58_ = _tmp57_;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp59_ = gxml_node_get_children_nodes (_tmp58_);
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp60_ = _tmp59_;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp61_ = _tmp60_;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp62_ = gee_collection_get_size ((GeeCollection*) _tmp61_);
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp63_ = _tmp62_;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp63_ == 2, "d.root.children_nodes.size == 2");
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp61_);
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp58_);
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp64_ = d;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp65_ = gxml_document_get_root ((GXmlDocument*) _tmp64_);
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp66_ = _tmp65_;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp67_ = _tmp66_;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp68_ = gxml_node_get_children_nodes (_tmp67_);
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp69_ = _tmp68_;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp70_ = _tmp69_;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp71_ = gee_list_get ((GeeList*) _tmp70_, 0);
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp72_ = (GXmlNode*) _tmp71_;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp73_ = gxml_node_get_name (_tmp72_);
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp74_ = _tmp73_;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp75_ = _tmp74_;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (g_strcmp0 (_tmp75_, "SmallBag") == 0, "d.root.children_nodes[0].name == \"SmallBag\"");
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_free0 (_tmp75_);
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp72_);
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp70_);
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp67_);
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp76_ = d;
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp77_ = gxml_document_get_root ((GXmlDocument*) _tmp76_);
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp78_ = _tmp77_;
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp79_ = _tmp78_;
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp80_ = gxml_node_get_children_nodes (_tmp79_);
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp81_ = _tmp80_;
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp82_ = _tmp81_;
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp83_ = gee_list_get ((GeeList*) _tmp82_, 0);
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp84_ = (GXmlNode*) _tmp83_;
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp85_ = gxml_node_get_children_nodes (_tmp84_);
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp86_ = _tmp85_;
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp87_ = _tmp86_;
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp88_ = gee_collection_get_size ((GeeCollection*) _tmp87_);
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp89_ = _tmp88_;
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp89_ == 2, "d.root.children_nodes[0].children_nodes.size == 2");
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp87_);
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp84_);
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp82_);
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp79_);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp90_ = d;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp91_ = gxml_document_get_root ((GXmlDocument*) _tmp90_);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp92_ = _tmp91_;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp93_ = _tmp92_;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp94_ = gxml_node_get_children_nodes (_tmp93_);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp95_ = _tmp94_;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp96_ = _tmp95_;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp97_ = gee_list_get ((GeeList*) _tmp96_, 0);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp98_ = (GXmlNode*) _tmp97_;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp99_ = gxml_node_get_children_nodes (_tmp98_);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp100_ = _tmp99_;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp101_ = _tmp100_;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp102_ = gee_list_get ((GeeList*) _tmp101_, 0);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp103_ = (GXmlNode*) _tmp102_;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp104_ = gxml_node_get_name (_tmp103_);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp105_ = _tmp104_;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp106_ = _tmp105_;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (g_strcmp0 (_tmp106_, "Ball") == 0, "d.root.children_nodes[0].children_nodes[0].name == \"Ball\"");
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_free0 (_tmp106_);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp103_);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp101_);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp98_);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp96_);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp93_);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp107_ = d;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp108_ = gxml_document_get_root ((GXmlDocument*) _tmp107_);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp109_ = _tmp108_;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp110_ = _tmp109_;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp111_ = gxml_node_get_children_nodes (_tmp110_);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp112_ = _tmp111_;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp113_ = _tmp112_;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp114_ = gee_list_get ((GeeList*) _tmp113_, 0);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp115_ = (GXmlNode*) _tmp114_;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp116_ = gxml_node_get_children_nodes (_tmp115_);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp117_ = _tmp116_;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp118_ = _tmp117_;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp119_ = gee_list_get ((GeeList*) _tmp118_, 0);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp120_ = (GXmlNode*) _tmp119_;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp121_ = gxml_node_get_children_nodes (_tmp120_);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp122_ = _tmp121_;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp123_ = _tmp122_;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp124_ = gee_collection_get_size ((GeeCollection*) _tmp123_);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp125_ = _tmp124_;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp125_ == 1, "d.root.children_nodes[0].children_nodes[0].children_nodes.size == 1");
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp123_);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp120_);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp118_);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp115_);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp113_);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp110_);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp126_ = d;
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp127_ = gxml_document_get_root ((GXmlDocument*) _tmp126_);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp128_ = _tmp127_;
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp129_ = _tmp128_;
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp130_ = gxml_node_get_children_nodes (_tmp129_);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp131_ = _tmp130_;
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp132_ = _tmp131_;
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp133_ = gee_list_get ((GeeList*) _tmp132_, 0);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp134_ = (GXmlNode*) _tmp133_;
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp135_ = gxml_node_get_children_nodes (_tmp134_);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp136_ = _tmp135_;
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp137_ = _tmp136_;
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp138_ = gee_list_get ((GeeList*) _tmp137_, 0);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp139_ = (GXmlNode*) _tmp138_;
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp140_ = gxml_node_get_children_nodes (_tmp139_);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp141_ = _tmp140_;
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp142_ = _tmp141_;
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp143_ = gee_list_get ((GeeList*) _tmp142_, 0);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp144_ = (GXmlNode*) _tmp143_;
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp145_ = gxml_node_get_name (_tmp144_);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp146_ = _tmp145_;
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp147_ = _tmp146_;
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (g_strcmp0 (_tmp147_, "BallFill") == 0, "d.root.children_nodes[0].children_nodes[0].children_nodes[0].name == \"BallFill\"");
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_free0 (_tmp147_);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp144_);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp142_);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp139_);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp137_);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp134_);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp132_);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp129_);
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp148_ = d;
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp149_ = gxml_document_get_root ((GXmlDocument*) _tmp148_);
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp150_ = _tmp149_;
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp151_ = _tmp150_;
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp152_ = gxml_node_get_children_nodes (_tmp151_);
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp153_ = _tmp152_;
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp154_ = _tmp153_;
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp155_ = gee_list_get ((GeeList*) _tmp154_, 0);
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp156_ = (GXmlNode*) _tmp155_;
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp157_ = gxml_node_get_children_nodes (_tmp156_);
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp158_ = _tmp157_;
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp159_ = _tmp158_;
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp160_ = gee_list_get ((GeeList*) _tmp159_, 0);
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp161_ = (GXmlNode*) _tmp160_;
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp162_ = gxml_node_get_children_nodes (_tmp161_);
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp163_ = _tmp162_;
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp164_ = _tmp163_;
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp165_ = gee_list_get ((GeeList*) _tmp164_, 0);
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp166_ = (GXmlNode*) _tmp165_;
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp167_ = gxml_node_get_children_nodes (_tmp166_);
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp168_ = _tmp167_;
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp169_ = _tmp168_;
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp170_ = gee_collection_get_size ((GeeCollection*) _tmp169_);
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp171_ = _tmp170_;
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp171_ == 1, "d.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes.size == 1");
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp169_);
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp166_);
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp164_);
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp161_);
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp159_);
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp156_);
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp154_);
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp151_);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp172_ = d;
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp173_ = gxml_document_get_root ((GXmlDocument*) _tmp172_);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp174_ = _tmp173_;
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp175_ = _tmp174_;
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp176_ = gxml_node_get_children_nodes (_tmp175_);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp177_ = _tmp176_;
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp178_ = _tmp177_;
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp179_ = gee_list_get ((GeeList*) _tmp178_, 0);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp180_ = (GXmlNode*) _tmp179_;
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp181_ = gxml_node_get_children_nodes (_tmp180_);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp182_ = _tmp181_;
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp183_ = _tmp182_;
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp184_ = gee_list_get ((GeeList*) _tmp183_, 0);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp185_ = (GXmlNode*) _tmp184_;
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp186_ = gxml_node_get_children_nodes (_tmp185_);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp187_ = _tmp186_;
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp188_ = _tmp187_;
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp189_ = gee_list_get ((GeeList*) _tmp188_, 0);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp190_ = (GXmlNode*) _tmp189_;
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp191_ = gxml_node_get_children_nodes (_tmp190_);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp192_ = _tmp191_;
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp193_ = _tmp192_;
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp194_ = gee_list_get ((GeeList*) _tmp193_, 0);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp195_ = (GXmlNode*) _tmp194_;
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp195_, GXML_TYPE_TEXT), "d.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes[0] is Text");
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp195_);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp193_);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp190_);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp188_);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp185_);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp183_);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp180_);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp178_);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp175_);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp196_ = d;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp197_ = gxml_document_get_root ((GXmlDocument*) _tmp196_);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp198_ = _tmp197_;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp199_ = _tmp198_;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp200_ = gxml_node_get_children_nodes (_tmp199_);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp201_ = _tmp200_;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp202_ = _tmp201_;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp203_ = gee_list_get ((GeeList*) _tmp202_, 0);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp204_ = (GXmlNode*) _tmp203_;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp205_ = gxml_node_get_children_nodes (_tmp204_);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp206_ = _tmp205_;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp207_ = _tmp206_;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp208_ = gee_list_get ((GeeList*) _tmp207_, 0);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp209_ = (GXmlNode*) _tmp208_;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp210_ = gxml_node_get_children_nodes (_tmp209_);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp211_ = _tmp210_;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp212_ = _tmp211_;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp213_ = gee_list_get ((GeeList*) _tmp212_, 0);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp214_ = (GXmlNode*) _tmp213_;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp215_ = gxml_node_get_children_nodes (_tmp214_);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp216_ = _tmp215_;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp217_ = _tmp216_;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp218_ = gee_list_get ((GeeList*) _tmp217_, 0);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp219_ = (GXmlNode*) _tmp218_;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp220_ = gxml_node_get_value (_tmp219_);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp221_ = _tmp220_;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp222_ = _tmp221_;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (g_strcmp0 (_tmp222_, "golden dust") == 0, "d.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes[0].value == \"golden dust\"");
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_free0 (_tmp222_);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp219_);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp217_);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp214_);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp212_);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp209_);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp207_);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp204_);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp202_);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp199_);
#line 285 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp223_ = serializable_gee_array_list_test_big_bag_new ();
#line 285 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		bagt = _tmp223_;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp224_ = bagt;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp225_ = d;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp224_, (GXmlNode*) _tmp225_, &_inner_error_);
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (bagt);
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (d);
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (bag);
#line 3916 "SerializableGeeArrayListTest.c"
			goto __catch86_g_error;
		}
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp227_ = bagt;
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp228_ = serializable_gee_array_list_test_big_bag_get_bags (_tmp227_);
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp229_ = _tmp228_;
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp230_ = gxml_serializable_array_list_deserialize_children ((GXmlSerializableArrayList*) _tmp229_, &_inner_error_);
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp226_ = _tmp230_;
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (bagt);
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (d);
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (bag);
#line 3937 "SerializableGeeArrayListTest.c"
			goto __catch86_g_error;
		}
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp226_, "bagt.bags.deserialize_children ()");
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp231_ = bagt;
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp232_ = serializable_gee_array_list_test_big_bag_get_bags (_tmp231_);
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp233_ = _tmp232_;
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp234_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp233_);
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp235_ = _tmp234_;
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp235_ == 2, "bagt.bags.size == 2");
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp237_ = bagt;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp238_ = serializable_gee_array_list_test_big_bag_get_bags (_tmp237_);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp239_ = _tmp238_;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp240_ = gee_abstract_list_get ((GeeAbstractList*) _tmp239_, 0);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp241_ = (SerializableGeeArrayListTestSmallBag*) _tmp240_;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp242_ = serializable_gee_array_list_test_small_bag_get_balls (_tmp241_);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp243_ = _tmp242_;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp244_ = gxml_serializable_array_list_deserialize_children ((GXmlSerializableArrayList*) _tmp243_, &_inner_error_);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp245_ = _tmp244_;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp241_);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp236_ = _tmp245_;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (bagt);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (d);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (bag);
#line 3984 "SerializableGeeArrayListTest.c"
			goto __catch86_g_error;
		}
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp236_, "bagt.bags[0].balls.deserialize_children ()");
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp246_ = bagt;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp247_ = serializable_gee_array_list_test_big_bag_get_bags (_tmp246_);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp248_ = _tmp247_;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp249_ = gee_abstract_list_get ((GeeAbstractList*) _tmp248_, 0);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp250_ = (SerializableGeeArrayListTestSmallBag*) _tmp249_;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp251_ = serializable_gee_array_list_test_small_bag_get_balls (_tmp250_);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp252_ = _tmp251_;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp253_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp252_);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp254_ = _tmp253_;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp254_ == 2, "bagt.bags[0].balls.size == 2");
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp250_);
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp255_ = bagt;
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp256_ = serializable_gee_array_list_test_big_bag_get_bags (_tmp255_);
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp257_ = _tmp256_;
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp258_ = gee_abstract_list_get ((GeeAbstractList*) _tmp257_, 0);
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp259_ = (SerializableGeeArrayListTestSmallBag*) _tmp258_;
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp260_ = serializable_gee_array_list_test_small_bag_get_balls (_tmp259_);
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp261_ = _tmp260_;
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp262_ = gee_abstract_list_get ((GeeAbstractList*) _tmp261_, 0);
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp263_ = (SerializableGeeArrayListTestBall*) _tmp262_;
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp264_ = serializable_gee_array_list_test_ball_get_name (_tmp263_);
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp265_ = _tmp264_;
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (g_strcmp0 (_tmp265_, "Ball") == 0, "bagt.bags[0].balls[0].name == \"Ball\"");
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp263_);
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp259_);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp266_ = bagt;
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp267_ = serializable_gee_array_list_test_big_bag_get_bags (_tmp266_);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp268_ = _tmp267_;
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp269_ = gee_abstract_list_get ((GeeAbstractList*) _tmp268_, 0);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp270_ = (SerializableGeeArrayListTestSmallBag*) _tmp269_;
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp271_ = serializable_gee_array_list_test_small_bag_get_balls (_tmp270_);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp272_ = _tmp271_;
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp273_ = gee_abstract_list_get ((GeeAbstractList*) _tmp272_, 0);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp274_ = (SerializableGeeArrayListTestBall*) _tmp273_;
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp275_ = serializable_gee_array_list_test_ball_get_ballfill (_tmp274_);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp276_ = _tmp275_;
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp276_ != NULL, "bagt.bags[0].balls[0].ballfill !=null");
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp274_);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp270_);
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp277_ = bagt;
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp278_ = serializable_gee_array_list_test_big_bag_get_bags (_tmp277_);
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp279_ = _tmp278_;
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp280_ = gee_abstract_list_get ((GeeAbstractList*) _tmp279_, 0);
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp281_ = (SerializableGeeArrayListTestSmallBag*) _tmp280_;
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp282_ = serializable_gee_array_list_test_small_bag_get_balls (_tmp281_);
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp283_ = _tmp282_;
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp284_ = gee_abstract_list_get ((GeeAbstractList*) _tmp283_, 0);
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp285_ = (SerializableGeeArrayListTestBall*) _tmp284_;
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp286_ = serializable_gee_array_list_test_ball_get_ballfill (_tmp285_);
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp287_ = _tmp286_;
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp288_ = gxml_serializable_get_serialized_xml_node_value ((GXmlSerializable*) _tmp287_);
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp289_ = _tmp288_;
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp290_ = _tmp289_;
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp290_ != NULL, "bagt.bags[0].balls[0].ballfill.serialized_xml_node_value !=null");
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_free0 (_tmp290_);
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp285_);
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp281_);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp291_ = bagt;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp292_ = serializable_gee_array_list_test_big_bag_get_bags (_tmp291_);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp293_ = _tmp292_;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp294_ = gee_abstract_list_get ((GeeAbstractList*) _tmp293_, 0);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp295_ = (SerializableGeeArrayListTestSmallBag*) _tmp294_;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp296_ = serializable_gee_array_list_test_small_bag_get_balls (_tmp295_);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp297_ = _tmp296_;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp298_ = gee_abstract_list_get ((GeeAbstractList*) _tmp297_, 0);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp299_ = (SerializableGeeArrayListTestBall*) _tmp298_;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp300_ = serializable_gee_array_list_test_ball_get_ballfill (_tmp299_);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp301_ = _tmp300_;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp302_ = gxml_serializable_get_serialized_xml_node_value ((GXmlSerializable*) _tmp301_);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp303_ = _tmp302_;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp304_ = _tmp303_;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (g_strcmp0 (_tmp304_, "golden dust") == 0, "bagt.bags[0].balls[0].ballfill.serialized_xml_node_value ==\"golden dust\"");
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_free0 (_tmp304_);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp299_);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp295_);
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp305_ = serializable_gee_array_list_test_big_bag_new ();
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		bag2 = _tmp305_;
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp306_ = bag2;
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp307_ = d;
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp306_, (GXmlNode*) _tmp307_, &_inner_error_);
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (bag2);
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (bagt);
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (d);
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (bag);
#line 4159 "SerializableGeeArrayListTest.c"
			goto __catch86_g_error;
		}
#line 297 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp308_ = bag2;
#line 297 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp309_ = serializable_gee_array_list_test_big_bag_get_bags (_tmp308_);
#line 297 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp310_ = _tmp309_;
#line 297 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp311_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp310_);
#line 297 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp312_ = _tmp311_;
#line 297 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp312_ == 0, "bag2.bags.size == 0");
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp313_ = gxml_gdocument_new ();
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		d2 = _tmp313_;
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp314_ = bag2;
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp315_ = d2;
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp316_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) _tmp314_, (GXmlNode*) _tmp315_, &_inner_error_);
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp317_ = _tmp316_;
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp317_);
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (d2);
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (bag2);
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (bagt);
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (d);
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
			_g_object_unref0 (bag);
#line 4200 "SerializableGeeArrayListTest.c"
			goto __catch86_g_error;
		}
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp318_ = d2;
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp319_ = gxml_document_get_root ((GXmlDocument*) _tmp318_);
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp320_ = _tmp319_;
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp321_ = _tmp320_;
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp321_ != NULL, "d2.root != null");
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp321_);
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp322_ = d2;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp323_ = gxml_document_get_root ((GXmlDocument*) _tmp322_);
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp324_ = _tmp323_;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp325_ = _tmp324_;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp326_ = gxml_node_get_name (_tmp325_);
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp327_ = _tmp326_;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp328_ = _tmp327_;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (g_strcmp0 (_tmp328_, "BigBag") == 0, "d2.root.name == \"BigBag\"");
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_free0 (_tmp328_);
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp325_);
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp329_ = d2;
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp330_ = gxml_document_get_root ((GXmlDocument*) _tmp329_);
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp331_ = _tmp330_;
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp332_ = _tmp331_;
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp333_ = gxml_node_get_children_nodes (_tmp332_);
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp334_ = _tmp333_;
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp335_ = _tmp334_;
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp336_ = gee_collection_get_size ((GeeCollection*) _tmp335_);
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp337_ = _tmp336_;
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp337_ == 2, "d2.root.children_nodes.size == 2");
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp335_);
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp332_);
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp338_ = d2;
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp339_ = gxml_document_get_root ((GXmlDocument*) _tmp338_);
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp340_ = _tmp339_;
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp341_ = _tmp340_;
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp342_ = gxml_node_get_children_nodes (_tmp341_);
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp343_ = _tmp342_;
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp344_ = _tmp343_;
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp345_ = gee_list_get ((GeeList*) _tmp344_, 0);
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp346_ = (GXmlNode*) _tmp345_;
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp347_ = gxml_node_get_name (_tmp346_);
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp348_ = _tmp347_;
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp349_ = _tmp348_;
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (g_strcmp0 (_tmp349_, "SmallBag") == 0, "d2.root.children_nodes[0].name == \"SmallBag\"");
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_free0 (_tmp349_);
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp346_);
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp344_);
#line 305 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp341_);
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp350_ = d2;
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp351_ = gxml_document_get_root ((GXmlDocument*) _tmp350_);
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp352_ = _tmp351_;
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp353_ = _tmp352_;
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp354_ = gxml_node_get_children_nodes (_tmp353_);
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp355_ = _tmp354_;
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp356_ = _tmp355_;
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp357_ = gee_list_get ((GeeList*) _tmp356_, 0);
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp358_ = (GXmlNode*) _tmp357_;
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp359_ = gxml_node_get_children_nodes (_tmp358_);
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp360_ = _tmp359_;
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp361_ = _tmp360_;
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp362_ = gee_collection_get_size ((GeeCollection*) _tmp361_);
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp363_ = _tmp362_;
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp363_ == 2, "d2.root.children_nodes[0].children_nodes.size == 2");
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp361_);
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp358_);
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp356_);
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp353_);
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp364_ = d2;
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp365_ = gxml_document_get_root ((GXmlDocument*) _tmp364_);
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp366_ = _tmp365_;
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp367_ = _tmp366_;
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp368_ = gxml_node_get_children_nodes (_tmp367_);
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp369_ = _tmp368_;
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp370_ = _tmp369_;
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp371_ = gee_list_get ((GeeList*) _tmp370_, 0);
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp372_ = (GXmlNode*) _tmp371_;
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp373_ = gxml_node_get_children_nodes (_tmp372_);
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp374_ = _tmp373_;
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp375_ = _tmp374_;
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp376_ = gee_list_get ((GeeList*) _tmp375_, 0);
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp377_ = (GXmlNode*) _tmp376_;
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp378_ = gxml_node_get_name (_tmp377_);
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp379_ = _tmp378_;
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp380_ = _tmp379_;
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (g_strcmp0 (_tmp380_, "Ball") == 0, "d2.root.children_nodes[0].children_nodes[0].name == \"Ball\"");
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_free0 (_tmp380_);
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp377_);
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp375_);
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp372_);
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp370_);
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp367_);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp381_ = d2;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp382_ = gxml_document_get_root ((GXmlDocument*) _tmp381_);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp383_ = _tmp382_;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp384_ = _tmp383_;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp385_ = gxml_node_get_children_nodes (_tmp384_);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp386_ = _tmp385_;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp387_ = _tmp386_;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp388_ = gee_list_get ((GeeList*) _tmp387_, 0);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp389_ = (GXmlNode*) _tmp388_;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp390_ = gxml_node_get_children_nodes (_tmp389_);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp391_ = _tmp390_;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp392_ = _tmp391_;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp393_ = gee_list_get ((GeeList*) _tmp392_, 0);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp394_ = (GXmlNode*) _tmp393_;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp395_ = gxml_node_get_children_nodes (_tmp394_);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp396_ = _tmp395_;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp397_ = _tmp396_;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp398_ = gee_collection_get_size ((GeeCollection*) _tmp397_);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp399_ = _tmp398_;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp399_ == 1, "d2.root.children_nodes[0].children_nodes[0].children_nodes.size == 1");
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp397_);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp394_);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp392_);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp389_);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp387_);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp384_);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp400_ = d2;
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp401_ = gxml_document_get_root ((GXmlDocument*) _tmp400_);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp402_ = _tmp401_;
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp403_ = _tmp402_;
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp404_ = gxml_node_get_children_nodes (_tmp403_);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp405_ = _tmp404_;
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp406_ = _tmp405_;
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp407_ = gee_list_get ((GeeList*) _tmp406_, 0);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp408_ = (GXmlNode*) _tmp407_;
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp409_ = gxml_node_get_children_nodes (_tmp408_);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp410_ = _tmp409_;
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp411_ = _tmp410_;
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp412_ = gee_list_get ((GeeList*) _tmp411_, 0);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp413_ = (GXmlNode*) _tmp412_;
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp414_ = gxml_node_get_children_nodes (_tmp413_);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp415_ = _tmp414_;
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp416_ = _tmp415_;
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp417_ = gee_list_get ((GeeList*) _tmp416_, 0);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp418_ = (GXmlNode*) _tmp417_;
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp419_ = gxml_node_get_name (_tmp418_);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp420_ = _tmp419_;
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp421_ = _tmp420_;
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (g_strcmp0 (_tmp421_, "BallFill") == 0, "d2.root.children_nodes[0].children_nodes[0].children_nodes[0].name == \"BallFill\"");
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_free0 (_tmp421_);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp418_);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp416_);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp413_);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp411_);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp408_);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp406_);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp403_);
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp422_ = d2;
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp423_ = gxml_document_get_root ((GXmlDocument*) _tmp422_);
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp424_ = _tmp423_;
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp425_ = _tmp424_;
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp426_ = gxml_node_get_children_nodes (_tmp425_);
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp427_ = _tmp426_;
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp428_ = _tmp427_;
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp429_ = gee_list_get ((GeeList*) _tmp428_, 0);
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp430_ = (GXmlNode*) _tmp429_;
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp431_ = gxml_node_get_children_nodes (_tmp430_);
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp432_ = _tmp431_;
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp433_ = _tmp432_;
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp434_ = gee_list_get ((GeeList*) _tmp433_, 0);
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp435_ = (GXmlNode*) _tmp434_;
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp436_ = gxml_node_get_children_nodes (_tmp435_);
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp437_ = _tmp436_;
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp438_ = _tmp437_;
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp439_ = gee_list_get ((GeeList*) _tmp438_, 0);
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp440_ = (GXmlNode*) _tmp439_;
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp441_ = gxml_node_get_children_nodes (_tmp440_);
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp442_ = _tmp441_;
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp443_ = _tmp442_;
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp444_ = gee_collection_get_size ((GeeCollection*) _tmp443_);
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp445_ = _tmp444_;
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (_tmp445_ == 1, "d2.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes.size == 1");
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp443_);
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp440_);
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp438_);
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp435_);
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp433_);
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp430_);
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp428_);
#line 310 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp425_);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp446_ = d2;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp447_ = gxml_document_get_root ((GXmlDocument*) _tmp446_);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp448_ = _tmp447_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp449_ = _tmp448_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp450_ = gxml_node_get_children_nodes (_tmp449_);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp451_ = _tmp450_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp452_ = _tmp451_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp453_ = gee_list_get ((GeeList*) _tmp452_, 0);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp454_ = (GXmlNode*) _tmp453_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp455_ = gxml_node_get_children_nodes (_tmp454_);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp456_ = _tmp455_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp457_ = _tmp456_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp458_ = gee_list_get ((GeeList*) _tmp457_, 0);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp459_ = (GXmlNode*) _tmp458_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp460_ = gxml_node_get_children_nodes (_tmp459_);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp461_ = _tmp460_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp462_ = _tmp461_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp463_ = gee_list_get ((GeeList*) _tmp462_, 0);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp464_ = (GXmlNode*) _tmp463_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp465_ = gxml_node_get_children_nodes (_tmp464_);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp466_ = _tmp465_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp467_ = _tmp466_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp468_ = gee_list_get ((GeeList*) _tmp467_, 0);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp469_ = (GXmlNode*) _tmp468_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (G_TYPE_CHECK_INSTANCE_TYPE (_tmp469_, GXML_TYPE_TEXT), "d2.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes[0] is GXml.Text");
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp469_);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp467_);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp464_);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp462_);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp459_);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp457_);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp454_);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp452_);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp449_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp470_ = d2;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp471_ = gxml_document_get_root ((GXmlDocument*) _tmp470_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp472_ = _tmp471_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp473_ = _tmp472_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp474_ = gxml_node_get_children_nodes (_tmp473_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp475_ = _tmp474_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp476_ = _tmp475_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp477_ = gee_list_get ((GeeList*) _tmp476_, 0);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp478_ = (GXmlNode*) _tmp477_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp479_ = gxml_node_get_children_nodes (_tmp478_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp480_ = _tmp479_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp481_ = _tmp480_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp482_ = gee_list_get ((GeeList*) _tmp481_, 0);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp483_ = (GXmlNode*) _tmp482_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp484_ = gxml_node_get_children_nodes (_tmp483_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp485_ = _tmp484_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp486_ = _tmp485_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp487_ = gee_list_get ((GeeList*) _tmp486_, 0);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp488_ = (GXmlNode*) _tmp487_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp489_ = gxml_node_get_children_nodes (_tmp488_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp490_ = _tmp489_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp491_ = _tmp490_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp492_ = gee_list_get ((GeeList*) _tmp491_, 0);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp493_ = (GXmlNode*) _tmp492_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp494_ = gxml_node_get_value (_tmp493_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp495_ = _tmp494_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp496_ = _tmp495_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_vala_assert (g_strcmp0 (_tmp496_, "golden dust") == 0, "d2.root.children_nodes[0].children_nodes[0].children_nodes[0].children_nodes[0].value == \"golden dust\"");
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_free0 (_tmp496_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp493_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp491_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp488_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp486_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp483_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp481_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp478_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp476_);
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (_tmp473_);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (d2);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (bag2);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (bagt);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (d);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_object_unref0 (bag);
#line 4713 "SerializableGeeArrayListTest.c"
	}
	goto __finally86;
	__catch86_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp497_ = NULL;
		gchar* _tmp498_ = NULL;
		gchar* _tmp499_ = NULL;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		e = _inner_error_;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_inner_error_ = NULL;
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp497_ = e->message;
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp498_ = g_strconcat ("ERROR: ", _tmp497_, NULL);
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_tmp499_ = _tmp498_;
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_message ("SerializableGeeArrayListTest.vala:314: %s", _tmp499_);
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_free0 (_tmp499_);
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_assert_not_reached ();
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		_g_error_free0 (e);
#line 4740 "SerializableGeeArrayListTest.c"
	}
	__finally86:
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_clear_error (&_inner_error_);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		return;
#line 4751 "SerializableGeeArrayListTest.c"
	}
}


static void ___lambda65__gtest_func (void) {
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	__lambda65_ ();
#line 4759 "SerializableGeeArrayListTest.c"
}


void serializable_gee_array_list_test_add_tests (void) {
#line 89 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_test_add_func ("/gxml/serializable/arraylist/api", ___lambda60__gtest_func);
#line 111 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_test_add_func ("/gxml/serializable/arraylist/serialize", ___lambda61__gtest_func);
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_test_add_func ("/gxml/serializable/arraylist/deserialize", ___lambda62__gtest_func);
#line 180 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_test_add_func ("/gxml/serializable/arraylist/deserialize-serialize", ___lambda63__gtest_func);
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_test_add_func ("/gxml/serializable/arraylist/deserialize-node-names", ___lambda64__gtest_func);
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_test_add_func ("/gxml/serializable/arraylist/post-deserialization/serialize/contents", ___lambda65__gtest_func);
#line 4776 "SerializableGeeArrayListTest.c"
}


SerializableGeeArrayListTest* serializable_gee_array_list_test_construct (GType object_type) {
	SerializableGeeArrayListTest* self = NULL;
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (SerializableGeeArrayListTest*) gxml_test_construct (object_type);
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return self;
#line 4786 "SerializableGeeArrayListTest.c"
}


SerializableGeeArrayListTest* serializable_gee_array_list_test_new (void) {
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return serializable_gee_array_list_test_construct (TYPE_SERIALIZABLE_GEE_ARRAY_LIST_TEST);
#line 4793 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_ball_fill_set_text (SerializableGeeArrayListTestBallFill* self, const gchar* txt) {
	const gchar* _tmp0_ = NULL;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_if_fail (self != NULL);
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_if_fail (txt != NULL);
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = txt;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	gxml_serializable_set_serialized_xml_node_value ((GXmlSerializable*) self, _tmp0_);
#line 4807 "SerializableGeeArrayListTest.c"
}


static gchar* serializable_gee_array_list_test_ball_fill_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeArrayListTestBallFill * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (SerializableGeeArrayListTestBallFill*) base;
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = self->priv->_name;
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp1_;
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 4826 "SerializableGeeArrayListTest.c"
}


static gchar* serializable_gee_array_list_test_ball_fill_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeArrayListTestBallFill * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (SerializableGeeArrayListTestBallFill*) base;
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = g_strdup ("BallFill");
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp0_;
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 4842 "SerializableGeeArrayListTest.c"
}


static gboolean serializable_gee_array_list_test_ball_fill_real_serialize_use_xml_node_value (GXmlSerializableObjectModel* base) {
	SerializableGeeArrayListTestBallFill * self;
	gboolean result = FALSE;
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (SerializableGeeArrayListTestBallFill*) base;
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = TRUE;
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 4855 "SerializableGeeArrayListTest.c"
}


static SerializableGeeArrayListTestBallFill* serializable_gee_array_list_test_ball_fill_construct (GType object_type) {
	SerializableGeeArrayListTestBallFill * self = NULL;
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (SerializableGeeArrayListTestBallFill*) gxml_serializable_object_model_construct (object_type);
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return self;
#line 4865 "SerializableGeeArrayListTest.c"
}


static SerializableGeeArrayListTestBallFill* serializable_gee_array_list_test_ball_fill_new (void) {
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return serializable_gee_array_list_test_ball_fill_construct (SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL_FILL);
#line 4872 "SerializableGeeArrayListTest.c"
}


static const gchar* serializable_gee_array_list_test_ball_fill_get_name (SerializableGeeArrayListTestBallFill* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = self->priv->_name;
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp0_;
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 4887 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_ball_fill_set_name (SerializableGeeArrayListTestBallFill* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_if_fail (self != NULL);
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = value;
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_g_free0 (self->priv->_name);
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv->_name = _tmp1_;
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 4906 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_ball_fill_class_init (SerializableGeeArrayListTestBallFillClass * klass) {
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	serializable_gee_array_list_test_ball_fill_parent_class = g_type_class_peek_parent (klass);
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeArrayListTestBallFillPrivate));
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_array_list_test_ball_fill_real_to_string;
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_array_list_test_ball_fill_real_node_name;
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->serialize_use_xml_node_value = (gboolean (*)(GXmlSerializableObjectModel*)) serializable_gee_array_list_test_ball_fill_real_serialize_use_xml_node_value;
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_array_list_test_ball_fill_get_property;
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_array_list_test_ball_fill_set_property;
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_array_list_test_ball_fill_finalize;
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_FILL_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 4929 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_ball_fill_instance_init (SerializableGeeArrayListTestBallFill * self) {
	gchar* _tmp0_ = NULL;
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv = SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_FILL_GET_PRIVATE (self);
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = g_strdup ("Fill");
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv->_name = _tmp0_;
#line 4941 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_ball_fill_finalize (GObject* obj) {
	SerializableGeeArrayListTestBallFill * self;
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL_FILL, SerializableGeeArrayListTestBallFill);
#line 50 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_g_free0 (self->priv->_name);
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (serializable_gee_array_list_test_ball_fill_parent_class)->finalize (obj);
#line 4953 "SerializableGeeArrayListTest.c"
}


static GType serializable_gee_array_list_test_ball_fill_get_type (void) {
	static volatile gsize serializable_gee_array_list_test_ball_fill_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_array_list_test_ball_fill_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeArrayListTestBallFillClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_array_list_test_ball_fill_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeArrayListTestBallFill), 0, (GInstanceInitFunc) serializable_gee_array_list_test_ball_fill_instance_init, NULL };
		GType serializable_gee_array_list_test_ball_fill_type_id;
		serializable_gee_array_list_test_ball_fill_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeArrayListTestBallFill", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_array_list_test_ball_fill_type_id__volatile, serializable_gee_array_list_test_ball_fill_type_id);
	}
	return serializable_gee_array_list_test_ball_fill_type_id__volatile;
}


static void _vala_serializable_gee_array_list_test_ball_fill_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeArrayListTestBallFill * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL_FILL, SerializableGeeArrayListTestBallFill);
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	switch (property_id) {
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		case SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_FILL_NAME:
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_value_set_string (value, serializable_gee_array_list_test_ball_fill_get_name (self));
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 4980 "SerializableGeeArrayListTest.c"
		default:
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 4986 "SerializableGeeArrayListTest.c"
	}
}


static void _vala_serializable_gee_array_list_test_ball_fill_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeArrayListTestBallFill * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL_FILL, SerializableGeeArrayListTestBallFill);
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	switch (property_id) {
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		case SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_FILL_NAME:
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		serializable_gee_array_list_test_ball_fill_set_name (self, g_value_get_string (value));
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 5002 "SerializableGeeArrayListTest.c"
		default:
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 48 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 5008 "SerializableGeeArrayListTest.c"
	}
}


static gchar* serializable_gee_array_list_test_ball_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeArrayListTestBall * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 61 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (SerializableGeeArrayListTestBall*) base;
#line 61 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = self->priv->_name;
#line 61 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 61 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp1_;
#line 61 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 5028 "SerializableGeeArrayListTest.c"
}


static gchar* serializable_gee_array_list_test_ball_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeArrayListTestBall * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (SerializableGeeArrayListTestBall*) base;
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = g_strdup ("Ball");
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp0_;
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 5044 "SerializableGeeArrayListTest.c"
}


static SerializableGeeArrayListTestBall* serializable_gee_array_list_test_ball_construct (GType object_type) {
	SerializableGeeArrayListTestBall * self = NULL;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (SerializableGeeArrayListTestBall*) gxml_serializable_object_model_construct (object_type);
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return self;
#line 5054 "SerializableGeeArrayListTest.c"
}


static SerializableGeeArrayListTestBall* serializable_gee_array_list_test_ball_new (void) {
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return serializable_gee_array_list_test_ball_construct (SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL);
#line 5061 "SerializableGeeArrayListTest.c"
}


static const gchar* serializable_gee_array_list_test_ball_get_name (SerializableGeeArrayListTestBall* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = self->priv->_name;
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp0_;
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 5076 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_ball_set_name (SerializableGeeArrayListTestBall* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_if_fail (self != NULL);
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = value;
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_g_free0 (self->priv->_name);
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv->_name = _tmp1_;
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 5095 "SerializableGeeArrayListTest.c"
}


static SerializableGeeArrayListTestBallFill* serializable_gee_array_list_test_ball_get_ballfill (SerializableGeeArrayListTestBall* self) {
	SerializableGeeArrayListTestBallFill* result;
	SerializableGeeArrayListTestBallFill* _tmp0_ = NULL;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = self->priv->_ballfill;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp0_;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 5110 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_ball_set_ballfill (SerializableGeeArrayListTestBall* self, SerializableGeeArrayListTestBallFill* value) {
	SerializableGeeArrayListTestBallFill* _tmp0_ = NULL;
	SerializableGeeArrayListTestBallFill* _tmp1_ = NULL;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_if_fail (self != NULL);
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = value;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_g_object_unref0 (self->priv->_ballfill);
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv->_ballfill = _tmp1_;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_object_notify ((GObject *) self, "ballfill");
#line 5129 "SerializableGeeArrayListTest.c"
}


static gboolean serializable_gee_array_list_test_ball_array_real_deserialize_proceed (GXmlSerializableArrayList* base) {
	SerializableGeeArrayListTestBallArray * self;
	gboolean result = FALSE;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (SerializableGeeArrayListTestBallArray*) base;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = FALSE;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 5142 "SerializableGeeArrayListTest.c"
}


static SerializableGeeArrayListTestBallArray* serializable_gee_array_list_test_ball_array_construct (GType object_type) {
	SerializableGeeArrayListTestBallArray * self = NULL;
#line 63 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (SerializableGeeArrayListTestBallArray*) gxml_serializable_array_list_construct (object_type, SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 63 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return self;
#line 5152 "SerializableGeeArrayListTest.c"
}


static SerializableGeeArrayListTestBallArray* serializable_gee_array_list_test_ball_array_new (void) {
#line 63 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return serializable_gee_array_list_test_ball_array_construct (SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_TYPE_ARRAY);
#line 5159 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_ball_array_class_init (SerializableGeeArrayListTestBallArrayClass * klass) {
#line 63 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	serializable_gee_array_list_test_ball_array_parent_class = g_type_class_peek_parent (klass);
#line 63 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	((GXmlSerializableArrayListClass *) klass)->deserialize_proceed = (gboolean (*)(GXmlSerializableArrayList*)) serializable_gee_array_list_test_ball_array_real_deserialize_proceed;
#line 5168 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_ball_array_instance_init (SerializableGeeArrayListTestBallArray * self) {
}


GType serializable_gee_array_list_test_ball_array_get_type (void) {
	static volatile gsize serializable_gee_array_list_test_ball_array_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_array_list_test_ball_array_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeArrayListTestBallArrayClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_array_list_test_ball_array_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeArrayListTestBallArray), 0, (GInstanceInitFunc) serializable_gee_array_list_test_ball_array_instance_init, NULL };
		GType serializable_gee_array_list_test_ball_array_type_id;
		serializable_gee_array_list_test_ball_array_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_ARRAY_LIST, "SerializableGeeArrayListTestBallArray", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_array_list_test_ball_array_type_id__volatile, serializable_gee_array_list_test_ball_array_type_id);
	}
	return serializable_gee_array_list_test_ball_array_type_id__volatile;
}


static void serializable_gee_array_list_test_ball_class_init (SerializableGeeArrayListTestBallClass * klass) {
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	serializable_gee_array_list_test_ball_parent_class = g_type_class_peek_parent (klass);
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeArrayListTestBallPrivate));
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_array_list_test_ball_real_to_string;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_array_list_test_ball_real_node_name;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_array_list_test_ball_get_property;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_array_list_test_ball_set_property;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_array_list_test_ball_finalize;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_BALLFILL, g_param_spec_object ("ballfill", "ballfill", "ballfill", SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL_FILL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 5207 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_ball_instance_init (SerializableGeeArrayListTestBall * self) {
	gchar* _tmp0_ = NULL;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv = SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_GET_PRIVATE (self);
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = g_strdup ("Ball");
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv->_name = _tmp0_;
#line 5219 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_ball_finalize (GObject* obj) {
	SerializableGeeArrayListTestBall * self;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL, SerializableGeeArrayListTestBall);
#line 59 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_g_free0 (self->priv->_name);
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_g_object_unref0 (self->priv->_ballfill);
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (serializable_gee_array_list_test_ball_parent_class)->finalize (obj);
#line 5233 "SerializableGeeArrayListTest.c"
}


static GType serializable_gee_array_list_test_ball_get_type (void) {
	static volatile gsize serializable_gee_array_list_test_ball_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_array_list_test_ball_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeArrayListTestBallClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_array_list_test_ball_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeArrayListTestBall), 0, (GInstanceInitFunc) serializable_gee_array_list_test_ball_instance_init, NULL };
		GType serializable_gee_array_list_test_ball_type_id;
		serializable_gee_array_list_test_ball_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeArrayListTestBall", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_array_list_test_ball_type_id__volatile, serializable_gee_array_list_test_ball_type_id);
	}
	return serializable_gee_array_list_test_ball_type_id__volatile;
}


static void _vala_serializable_gee_array_list_test_ball_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeArrayListTestBall * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL, SerializableGeeArrayListTestBall);
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	switch (property_id) {
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		case SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_NAME:
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_value_set_string (value, serializable_gee_array_list_test_ball_get_name (self));
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		case SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_BALLFILL:
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_value_set_object (value, serializable_gee_array_list_test_ball_get_ballfill (self));
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 5266 "SerializableGeeArrayListTest.c"
		default:
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 5272 "SerializableGeeArrayListTest.c"
	}
}


static void _vala_serializable_gee_array_list_test_ball_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeArrayListTestBall * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BALL, SerializableGeeArrayListTestBall);
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	switch (property_id) {
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		case SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_NAME:
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		serializable_gee_array_list_test_ball_set_name (self, g_value_get_string (value));
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		case SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_BALLFILL:
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		serializable_gee_array_list_test_ball_set_ballfill (self, g_value_get_object (value));
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 5294 "SerializableGeeArrayListTest.c"
		default:
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 5300 "SerializableGeeArrayListTest.c"
	}
}


static gchar* serializable_gee_array_list_test_small_bag_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeArrayListTestSmallBag * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (SerializableGeeArrayListTestSmallBag*) base;
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = self->priv->_name;
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp1_;
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 5320 "SerializableGeeArrayListTest.c"
}


static gchar* serializable_gee_array_list_test_small_bag_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeArrayListTestSmallBag * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (SerializableGeeArrayListTestSmallBag*) base;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = g_strdup ("SmallBag");
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp0_;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 5336 "SerializableGeeArrayListTest.c"
}


static SerializableGeeArrayListTestSmallBag* serializable_gee_array_list_test_small_bag_construct (GType object_type) {
	SerializableGeeArrayListTestSmallBag * self = NULL;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (SerializableGeeArrayListTestSmallBag*) gxml_serializable_object_model_construct (object_type);
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return self;
#line 5346 "SerializableGeeArrayListTest.c"
}


static SerializableGeeArrayListTestSmallBag* serializable_gee_array_list_test_small_bag_new (void) {
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return serializable_gee_array_list_test_small_bag_construct (SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_SMALL_BAG);
#line 5353 "SerializableGeeArrayListTest.c"
}


static const gchar* serializable_gee_array_list_test_small_bag_get_name (SerializableGeeArrayListTestSmallBag* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = self->priv->_name;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp0_;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 5368 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_small_bag_set_name (SerializableGeeArrayListTestSmallBag* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_if_fail (self != NULL);
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = value;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_g_free0 (self->priv->_name);
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv->_name = _tmp1_;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 5387 "SerializableGeeArrayListTest.c"
}


static SerializableGeeArrayListTestBallArray* serializable_gee_array_list_test_small_bag_get_balls (SerializableGeeArrayListTestSmallBag* self) {
	SerializableGeeArrayListTestBallArray* result;
	SerializableGeeArrayListTestBallArray* _tmp0_ = NULL;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = self->priv->_balls;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp0_;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 5402 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_small_bag_set_balls (SerializableGeeArrayListTestSmallBag* self, SerializableGeeArrayListTestBallArray* value) {
	SerializableGeeArrayListTestBallArray* _tmp0_ = NULL;
	SerializableGeeArrayListTestBallArray* _tmp1_ = NULL;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_if_fail (self != NULL);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = value;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_g_object_unref0 (self->priv->_balls);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv->_balls = _tmp1_;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_object_notify ((GObject *) self, "balls");
#line 5421 "SerializableGeeArrayListTest.c"
}


static gboolean serializable_gee_array_list_test_small_bag_array_real_deserialize_proceed (GXmlSerializableArrayList* base) {
	SerializableGeeArrayListTestSmallBagArray * self;
	gboolean result = FALSE;
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (SerializableGeeArrayListTestSmallBagArray*) base;
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = FALSE;
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 5434 "SerializableGeeArrayListTest.c"
}


static SerializableGeeArrayListTestSmallBagArray* serializable_gee_array_list_test_small_bag_array_construct (GType object_type) {
	SerializableGeeArrayListTestSmallBagArray * self = NULL;
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (SerializableGeeArrayListTestSmallBagArray*) gxml_serializable_array_list_construct (object_type, SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_SMALL_BAG, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return self;
#line 5444 "SerializableGeeArrayListTest.c"
}


static SerializableGeeArrayListTestSmallBagArray* serializable_gee_array_list_test_small_bag_array_new (void) {
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return serializable_gee_array_list_test_small_bag_array_construct (SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_TYPE_ARRAY);
#line 5451 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_small_bag_array_class_init (SerializableGeeArrayListTestSmallBagArrayClass * klass) {
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	serializable_gee_array_list_test_small_bag_array_parent_class = g_type_class_peek_parent (klass);
#line 74 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	((GXmlSerializableArrayListClass *) klass)->deserialize_proceed = (gboolean (*)(GXmlSerializableArrayList*)) serializable_gee_array_list_test_small_bag_array_real_deserialize_proceed;
#line 5460 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_small_bag_array_instance_init (SerializableGeeArrayListTestSmallBagArray * self) {
}


GType serializable_gee_array_list_test_small_bag_array_get_type (void) {
	static volatile gsize serializable_gee_array_list_test_small_bag_array_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_array_list_test_small_bag_array_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeArrayListTestSmallBagArrayClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_array_list_test_small_bag_array_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeArrayListTestSmallBagArray), 0, (GInstanceInitFunc) serializable_gee_array_list_test_small_bag_array_instance_init, NULL };
		GType serializable_gee_array_list_test_small_bag_array_type_id;
		serializable_gee_array_list_test_small_bag_array_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_ARRAY_LIST, "SerializableGeeArrayListTestSmallBagArray", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_array_list_test_small_bag_array_type_id__volatile, serializable_gee_array_list_test_small_bag_array_type_id);
	}
	return serializable_gee_array_list_test_small_bag_array_type_id__volatile;
}


static void serializable_gee_array_list_test_small_bag_class_init (SerializableGeeArrayListTestSmallBagClass * klass) {
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	serializable_gee_array_list_test_small_bag_parent_class = g_type_class_peek_parent (klass);
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeArrayListTestSmallBagPrivate));
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_array_list_test_small_bag_real_to_string;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_array_list_test_small_bag_real_node_name;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_array_list_test_small_bag_get_property;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_array_list_test_small_bag_set_property;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_array_list_test_small_bag_finalize;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_BALLS, g_param_spec_object ("balls", "balls", "balls", SERIALIZABLE_GEE_ARRAY_LIST_TEST_BALL_TYPE_ARRAY, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 5499 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_small_bag_instance_init (SerializableGeeArrayListTestSmallBag * self) {
	gchar* _tmp0_ = NULL;
	SerializableGeeArrayListTestBallArray* _tmp1_ = NULL;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv = SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_GET_PRIVATE (self);
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = g_strdup ("SmallBag");
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv->_name = _tmp0_;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp1_ = serializable_gee_array_list_test_ball_array_new ();
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv->_balls = _tmp1_;
#line 5516 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_small_bag_finalize (GObject* obj) {
	SerializableGeeArrayListTestSmallBag * self;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_SMALL_BAG, SerializableGeeArrayListTestSmallBag);
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_g_free0 (self->priv->_name);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_g_object_unref0 (self->priv->_balls);
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (serializable_gee_array_list_test_small_bag_parent_class)->finalize (obj);
#line 5530 "SerializableGeeArrayListTest.c"
}


static GType serializable_gee_array_list_test_small_bag_get_type (void) {
	static volatile gsize serializable_gee_array_list_test_small_bag_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_array_list_test_small_bag_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeArrayListTestSmallBagClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_array_list_test_small_bag_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeArrayListTestSmallBag), 0, (GInstanceInitFunc) serializable_gee_array_list_test_small_bag_instance_init, NULL };
		GType serializable_gee_array_list_test_small_bag_type_id;
		serializable_gee_array_list_test_small_bag_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeArrayListTestSmallBag", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_array_list_test_small_bag_type_id__volatile, serializable_gee_array_list_test_small_bag_type_id);
	}
	return serializable_gee_array_list_test_small_bag_type_id__volatile;
}


static void _vala_serializable_gee_array_list_test_small_bag_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeArrayListTestSmallBag * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_SMALL_BAG, SerializableGeeArrayListTestSmallBag);
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	switch (property_id) {
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		case SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_NAME:
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_value_set_string (value, serializable_gee_array_list_test_small_bag_get_name (self));
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		case SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_BALLS:
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_value_set_object (value, serializable_gee_array_list_test_small_bag_get_balls (self));
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 5563 "SerializableGeeArrayListTest.c"
		default:
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 5569 "SerializableGeeArrayListTest.c"
	}
}


static void _vala_serializable_gee_array_list_test_small_bag_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeArrayListTestSmallBag * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_SMALL_BAG, SerializableGeeArrayListTestSmallBag);
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	switch (property_id) {
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		case SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_NAME:
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		serializable_gee_array_list_test_small_bag_set_name (self, g_value_get_string (value));
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		case SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_BALLS:
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		serializable_gee_array_list_test_small_bag_set_balls (self, g_value_get_object (value));
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 5591 "SerializableGeeArrayListTest.c"
		default:
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 5597 "SerializableGeeArrayListTest.c"
	}
}


static gchar* serializable_gee_array_list_test_big_bag_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeArrayListTestBigBag * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (SerializableGeeArrayListTestBigBag*) base;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = self->priv->_name;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp1_;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 5617 "SerializableGeeArrayListTest.c"
}


static gchar* serializable_gee_array_list_test_big_bag_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeArrayListTestBigBag * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 84 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (SerializableGeeArrayListTestBigBag*) base;
#line 84 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = g_strdup ("BigBag");
#line 84 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp0_;
#line 84 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 5633 "SerializableGeeArrayListTest.c"
}


static SerializableGeeArrayListTestBigBag* serializable_gee_array_list_test_big_bag_construct (GType object_type) {
	SerializableGeeArrayListTestBigBag * self = NULL;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = (SerializableGeeArrayListTestBigBag*) gxml_serializable_object_model_construct (object_type);
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return self;
#line 5643 "SerializableGeeArrayListTest.c"
}


static SerializableGeeArrayListTestBigBag* serializable_gee_array_list_test_big_bag_new (void) {
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return serializable_gee_array_list_test_big_bag_construct (SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BIG_BAG);
#line 5650 "SerializableGeeArrayListTest.c"
}


static const gchar* serializable_gee_array_list_test_big_bag_get_name (SerializableGeeArrayListTestBigBag* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = self->priv->_name;
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp0_;
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 5665 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_big_bag_set_name (SerializableGeeArrayListTestBigBag* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_if_fail (self != NULL);
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = value;
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_g_free0 (self->priv->_name);
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv->_name = _tmp1_;
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 5684 "SerializableGeeArrayListTest.c"
}


static SerializableGeeArrayListTestSmallBagArray* serializable_gee_array_list_test_big_bag_get_bags (SerializableGeeArrayListTestBigBag* self) {
	SerializableGeeArrayListTestSmallBagArray* result;
	SerializableGeeArrayListTestSmallBagArray* _tmp0_ = NULL;
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = self->priv->_bags;
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	result = _tmp0_;
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	return result;
#line 5699 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_big_bag_set_bags (SerializableGeeArrayListTestBigBag* self, SerializableGeeArrayListTestSmallBagArray* value) {
	SerializableGeeArrayListTestSmallBagArray* _tmp0_ = NULL;
	SerializableGeeArrayListTestSmallBagArray* _tmp1_ = NULL;
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_return_if_fail (self != NULL);
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = value;
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_g_object_unref0 (self->priv->_bags);
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv->_bags = _tmp1_;
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_object_notify ((GObject *) self, "bags");
#line 5718 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_big_bag_class_init (SerializableGeeArrayListTestBigBagClass * klass) {
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	serializable_gee_array_list_test_big_bag_parent_class = g_type_class_peek_parent (klass);
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeArrayListTestBigBagPrivate));
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_array_list_test_big_bag_real_to_string;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_array_list_test_big_bag_real_node_name;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_array_list_test_big_bag_get_property;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_array_list_test_big_bag_set_property;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_array_list_test_big_bag_finalize;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_ARRAY_LIST_TEST_BIG_BAG_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_ARRAY_LIST_TEST_BIG_BAG_BAGS, g_param_spec_object ("bags", "bags", "bags", SERIALIZABLE_GEE_ARRAY_LIST_TEST_SMALL_BAG_TYPE_ARRAY, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 5741 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_big_bag_instance_init (SerializableGeeArrayListTestBigBag * self) {
	gchar* _tmp0_ = NULL;
	SerializableGeeArrayListTestSmallBagArray* _tmp1_ = NULL;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv = SERIALIZABLE_GEE_ARRAY_LIST_TEST_BIG_BAG_GET_PRIVATE (self);
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp0_ = g_strdup ("ball");
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv->_name = _tmp0_;
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_tmp1_ = serializable_gee_array_list_test_small_bag_array_new ();
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self->priv->_bags = _tmp1_;
#line 5758 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_big_bag_finalize (GObject* obj) {
	SerializableGeeArrayListTestBigBag * self;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BIG_BAG, SerializableGeeArrayListTestBigBag);
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_g_free0 (self->priv->_name);
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	_g_object_unref0 (self->priv->_bags);
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	G_OBJECT_CLASS (serializable_gee_array_list_test_big_bag_parent_class)->finalize (obj);
#line 5772 "SerializableGeeArrayListTest.c"
}


static GType serializable_gee_array_list_test_big_bag_get_type (void) {
	static volatile gsize serializable_gee_array_list_test_big_bag_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_array_list_test_big_bag_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeArrayListTestBigBagClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_array_list_test_big_bag_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeArrayListTestBigBag), 0, (GInstanceInitFunc) serializable_gee_array_list_test_big_bag_instance_init, NULL };
		GType serializable_gee_array_list_test_big_bag_type_id;
		serializable_gee_array_list_test_big_bag_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeArrayListTestBigBag", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_array_list_test_big_bag_type_id__volatile, serializable_gee_array_list_test_big_bag_type_id);
	}
	return serializable_gee_array_list_test_big_bag_type_id__volatile;
}


static void _vala_serializable_gee_array_list_test_big_bag_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeArrayListTestBigBag * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BIG_BAG, SerializableGeeArrayListTestBigBag);
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	switch (property_id) {
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		case SERIALIZABLE_GEE_ARRAY_LIST_TEST_BIG_BAG_NAME:
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_value_set_string (value, serializable_gee_array_list_test_big_bag_get_name (self));
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		case SERIALIZABLE_GEE_ARRAY_LIST_TEST_BIG_BAG_BAGS:
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		g_value_set_object (value, serializable_gee_array_list_test_big_bag_get_bags (self));
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 5805 "SerializableGeeArrayListTest.c"
		default:
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 5811 "SerializableGeeArrayListTest.c"
	}
}


static void _vala_serializable_gee_array_list_test_big_bag_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeArrayListTestBigBag * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_ARRAY_LIST_TEST_TYPE_BIG_BAG, SerializableGeeArrayListTestBigBag);
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	switch (property_id) {
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		case SERIALIZABLE_GEE_ARRAY_LIST_TEST_BIG_BAG_NAME:
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		serializable_gee_array_list_test_big_bag_set_name (self, g_value_get_string (value));
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		case SERIALIZABLE_GEE_ARRAY_LIST_TEST_BIG_BAG_BAGS:
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		serializable_gee_array_list_test_big_bag_set_bags (self, g_value_get_object (value));
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 5833 "SerializableGeeArrayListTest.c"
		default:
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
		break;
#line 5839 "SerializableGeeArrayListTest.c"
	}
}


static void serializable_gee_array_list_test_class_init (SerializableGeeArrayListTestClass * klass) {
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeArrayListTest.vala"
	serializable_gee_array_list_test_parent_class = g_type_class_peek_parent (klass);
#line 5847 "SerializableGeeArrayListTest.c"
}


static void serializable_gee_array_list_test_instance_init (SerializableGeeArrayListTest * self) {
}


GType serializable_gee_array_list_test_get_type (void) {
	static volatile gsize serializable_gee_array_list_test_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_array_list_test_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeArrayListTestClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_array_list_test_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeArrayListTest), 0, (GInstanceInitFunc) serializable_gee_array_list_test_instance_init, NULL };
		GType serializable_gee_array_list_test_type_id;
		serializable_gee_array_list_test_type_id = g_type_register_static (TYPE_GXML_TEST, "SerializableGeeArrayListTest", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_array_list_test_type_id__volatile, serializable_gee_array_list_test_type_id);
	}
	return serializable_gee_array_list_test_type_id__volatile;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}



