/* SerializableGeeCollectionsTest.c generated by valac 0.34.6, the Vala compiler
 * generated from SerializableGeeCollectionsTest.vala, do not modify */

/* -*- Mode: vala; indent-tabs-mode: nil; c-basic-offset: 2; tab-width: 2 -*- */

#include <glib.h>
#include <glib-object.h>
#include <gxml/gxml.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gee.h>


#define TYPE_GXML_TEST (gxml_test_get_type ())
#define GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GXML_TEST, GXmlTest))
#define GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GXML_TEST, GXmlTestClass))
#define IS_GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GXML_TEST))
#define IS_GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GXML_TEST))
#define GXML_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GXML_TEST, GXmlTestClass))

typedef struct _GXmlTest GXmlTest;
typedef struct _GXmlTestClass GXmlTestClass;
typedef struct _GXmlTestPrivate GXmlTestPrivate;

#define TYPE_SERIALIZABLE_GEE_COLLECTIONS_TEST (serializable_gee_collections_test_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SERIALIZABLE_GEE_COLLECTIONS_TEST, SerializableGeeCollectionsTest))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SERIALIZABLE_GEE_COLLECTIONS_TEST, SerializableGeeCollectionsTestClass))
#define IS_SERIALIZABLE_GEE_COLLECTIONS_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SERIALIZABLE_GEE_COLLECTIONS_TEST))
#define IS_SERIALIZABLE_GEE_COLLECTIONS_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SERIALIZABLE_GEE_COLLECTIONS_TEST))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SERIALIZABLE_GEE_COLLECTIONS_TEST, SerializableGeeCollectionsTestClass))

typedef struct _SerializableGeeCollectionsTest SerializableGeeCollectionsTest;
typedef struct _SerializableGeeCollectionsTestClass SerializableGeeCollectionsTestClass;
typedef struct _SerializableGeeCollectionsTestPrivate SerializableGeeCollectionsTestPrivate;

#define SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_BASE (serializable_gee_collections_test_space_base_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_BASE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_BASE, SerializableGeeCollectionsTestSpaceBase))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_BASE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_BASE, SerializableGeeCollectionsTestSpaceBaseClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_SPACE_BASE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_BASE))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_SPACE_BASE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_BASE))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_BASE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_BASE, SerializableGeeCollectionsTestSpaceBaseClass))

typedef struct _SerializableGeeCollectionsTestSpaceBase SerializableGeeCollectionsTestSpaceBase;
typedef struct _SerializableGeeCollectionsTestSpaceBaseClass SerializableGeeCollectionsTestSpaceBaseClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_STORAGE (serializable_gee_collections_test_storage_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_STORAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_STORAGE, SerializableGeeCollectionsTestStorage))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_STORAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_STORAGE, SerializableGeeCollectionsTestStorageClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_STORAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_STORAGE))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_STORAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_STORAGE))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_STORAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_STORAGE, SerializableGeeCollectionsTestStorageClass))

typedef struct _SerializableGeeCollectionsTestStorage SerializableGeeCollectionsTestStorage;
typedef struct _SerializableGeeCollectionsTestStorageClass SerializableGeeCollectionsTestStorageClass;
#define _g_free0(var) (var = (g_free (var), NULL))

#define SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_TYPE_COLLECTION (serializable_gee_collections_test_refaction_collection_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_TYPE_COLLECTION, SerializableGeeCollectionsTestRefactionCollection))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_TYPE_COLLECTION, SerializableGeeCollectionsTestRefactionCollectionClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_TYPE_COLLECTION))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_IS_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_TYPE_COLLECTION))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_TYPE_COLLECTION, SerializableGeeCollectionsTestRefactionCollectionClass))

typedef struct _SerializableGeeCollectionsTestRefactionCollection SerializableGeeCollectionsTestRefactionCollection;
typedef struct _SerializableGeeCollectionsTestRefactionCollectionClass SerializableGeeCollectionsTestRefactionCollectionClass;

#define SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_REFACTION (serializable_gee_collections_test_refaction_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_REFACTION, SerializableGeeCollectionsTestRefaction))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_REFACTION, SerializableGeeCollectionsTestRefactionClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_REFACTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_REFACTION))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_REFACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_REFACTION))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_REFACTION, SerializableGeeCollectionsTestRefactionClass))

typedef struct _SerializableGeeCollectionsTestRefaction SerializableGeeCollectionsTestRefaction;
typedef struct _SerializableGeeCollectionsTestRefactionClass SerializableGeeCollectionsTestRefactionClass;

#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_TYPE_COLLECTION (serializable_gee_collections_test_space_ship_collection_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_TYPE_COLLECTION, SerializableGeeCollectionsTestSpaceShipCollection))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_TYPE_COLLECTION, SerializableGeeCollectionsTestSpaceShipCollectionClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_TYPE_COLLECTION))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_IS_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_TYPE_COLLECTION))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_TYPE_COLLECTION, SerializableGeeCollectionsTestSpaceShipCollectionClass))

typedef struct _SerializableGeeCollectionsTestSpaceShipCollection SerializableGeeCollectionsTestSpaceShipCollection;
typedef struct _SerializableGeeCollectionsTestSpaceShipCollectionClass SerializableGeeCollectionsTestSpaceShipCollectionClass;

#define SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_SHIP (serializable_gee_collections_test_space_ship_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_SHIP, SerializableGeeCollectionsTestSpaceShip))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_SHIP, SerializableGeeCollectionsTestSpaceShipClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_SPACE_SHIP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_SHIP))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_SPACE_SHIP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_SHIP))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_SHIP, SerializableGeeCollectionsTestSpaceShipClass))

typedef struct _SerializableGeeCollectionsTestSpaceShip SerializableGeeCollectionsTestSpaceShip;
typedef struct _SerializableGeeCollectionsTestSpaceShipClass SerializableGeeCollectionsTestSpaceShipClass;

#define SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CHARGE_ZONE (serializable_gee_collections_test_charge_zone_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_CHARGE_ZONE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CHARGE_ZONE, SerializableGeeCollectionsTestChargeZone))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_CHARGE_ZONE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CHARGE_ZONE, SerializableGeeCollectionsTestChargeZoneClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_CHARGE_ZONE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CHARGE_ZONE))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_CHARGE_ZONE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CHARGE_ZONE))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_CHARGE_ZONE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CHARGE_ZONE, SerializableGeeCollectionsTestChargeZoneClass))

typedef struct _SerializableGeeCollectionsTestChargeZone SerializableGeeCollectionsTestChargeZone;
typedef struct _SerializableGeeCollectionsTestChargeZoneClass SerializableGeeCollectionsTestChargeZoneClass;

#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_TYPE_COLLECTION (serializable_gee_collections_test_space_collection_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_TYPE_COLLECTION, SerializableGeeCollectionsTestSpaceCollection))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_TYPE_COLLECTION, SerializableGeeCollectionsTestSpaceCollectionClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_TYPE_COLLECTION))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_IS_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_TYPE_COLLECTION))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_TYPE_COLLECTION, SerializableGeeCollectionsTestSpaceCollectionClass))

typedef struct _SerializableGeeCollectionsTestSpaceCollection SerializableGeeCollectionsTestSpaceCollection;
typedef struct _SerializableGeeCollectionsTestSpaceCollectionClass SerializableGeeCollectionsTestSpaceCollectionClass;

#define SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE (serializable_gee_collections_test_space_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE, SerializableGeeCollectionsTestSpace))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE, SerializableGeeCollectionsTestSpaceClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_SPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE, SerializableGeeCollectionsTestSpaceClass))

typedef struct _SerializableGeeCollectionsTestSpace SerializableGeeCollectionsTestSpace;
typedef struct _SerializableGeeCollectionsTestSpaceClass SerializableGeeCollectionsTestSpaceClass;

#define SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_TYPE_COLLECTION (serializable_gee_collections_test_planet_collection_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_TYPE_COLLECTION, SerializableGeeCollectionsTestPlanetCollection))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_TYPE_COLLECTION, SerializableGeeCollectionsTestPlanetCollectionClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_TYPE_COLLECTION))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_IS_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_TYPE_COLLECTION))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_TYPE_COLLECTION, SerializableGeeCollectionsTestPlanetCollectionClass))

typedef struct _SerializableGeeCollectionsTestPlanetCollection SerializableGeeCollectionsTestPlanetCollection;
typedef struct _SerializableGeeCollectionsTestPlanetCollectionClass SerializableGeeCollectionsTestPlanetCollectionClass;

#define SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_PLANET (serializable_gee_collections_test_planet_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_PLANET, SerializableGeeCollectionsTestPlanet))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_PLANET, SerializableGeeCollectionsTestPlanetClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_PLANET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_PLANET))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_PLANET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_PLANET))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_PLANET, SerializableGeeCollectionsTestPlanetClass))

typedef struct _SerializableGeeCollectionsTestPlanet SerializableGeeCollectionsTestPlanet;
typedef struct _SerializableGeeCollectionsTestPlanetClass SerializableGeeCollectionsTestPlanetClass;
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))

#define SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BIG_BAG (serializable_gee_collections_test_big_bag_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_BIG_BAG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BIG_BAG, SerializableGeeCollectionsTestBigBag))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_BIG_BAG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BIG_BAG, SerializableGeeCollectionsTestBigBagClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_BIG_BAG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BIG_BAG))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_BIG_BAG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BIG_BAG))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_BIG_BAG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BIG_BAG, SerializableGeeCollectionsTestBigBagClass))

typedef struct _SerializableGeeCollectionsTestBigBag SerializableGeeCollectionsTestBigBag;
typedef struct _SerializableGeeCollectionsTestBigBagClass SerializableGeeCollectionsTestBigBagClass;

#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_TYPE_ARRAY (serializable_gee_collections_test_small_bag_array_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_TYPE_ARRAY, SerializableGeeCollectionsTestSmallBagArray))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_TYPE_ARRAY, SerializableGeeCollectionsTestSmallBagArrayClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_IS_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_TYPE_ARRAY))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_IS_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_TYPE_ARRAY))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_ARRAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_TYPE_ARRAY, SerializableGeeCollectionsTestSmallBagArrayClass))

typedef struct _SerializableGeeCollectionsTestSmallBagArray SerializableGeeCollectionsTestSmallBagArray;
typedef struct _SerializableGeeCollectionsTestSmallBagArrayClass SerializableGeeCollectionsTestSmallBagArrayClass;

#define SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SMALL_BAG (serializable_gee_collections_test_small_bag_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SMALL_BAG, SerializableGeeCollectionsTestSmallBag))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SMALL_BAG, SerializableGeeCollectionsTestSmallBagClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_SMALL_BAG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SMALL_BAG))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_SMALL_BAG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SMALL_BAG))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SMALL_BAG, SerializableGeeCollectionsTestSmallBagClass))

typedef struct _SerializableGeeCollectionsTestSmallBag SerializableGeeCollectionsTestSmallBag;
typedef struct _SerializableGeeCollectionsTestSmallBagClass SerializableGeeCollectionsTestSmallBagClass;

#define SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_TYPE_ARRAY (serializable_gee_collections_test_ball_array_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_TYPE_ARRAY, SerializableGeeCollectionsTestBallArray))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_TYPE_ARRAY, SerializableGeeCollectionsTestBallArrayClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_IS_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_TYPE_ARRAY))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_IS_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_TYPE_ARRAY))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_ARRAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_TYPE_ARRAY, SerializableGeeCollectionsTestBallArrayClass))

typedef struct _SerializableGeeCollectionsTestBallArray SerializableGeeCollectionsTestBallArray;
typedef struct _SerializableGeeCollectionsTestBallArrayClass SerializableGeeCollectionsTestBallArrayClass;

#define SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL (serializable_gee_collections_test_ball_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL, SerializableGeeCollectionsTestBall))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL, SerializableGeeCollectionsTestBallClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_BALL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_BALL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL, SerializableGeeCollectionsTestBallClass))

typedef struct _SerializableGeeCollectionsTestBall SerializableGeeCollectionsTestBall;
typedef struct _SerializableGeeCollectionsTestBallClass SerializableGeeCollectionsTestBallClass;

#define SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CITIZEN (serializable_gee_collections_test_citizen_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CITIZEN, SerializableGeeCollectionsTestCitizen))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CITIZEN, SerializableGeeCollectionsTestCitizenClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_CITIZEN(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CITIZEN))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_CITIZEN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CITIZEN))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CITIZEN, SerializableGeeCollectionsTestCitizenClass))

typedef struct _SerializableGeeCollectionsTestCitizen SerializableGeeCollectionsTestCitizen;
typedef struct _SerializableGeeCollectionsTestCitizenClass SerializableGeeCollectionsTestCitizenClass;
typedef struct _SerializableGeeCollectionsTestCitizenPrivate SerializableGeeCollectionsTestCitizenPrivate;

#define SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_TYPE_ARRAY (serializable_gee_collections_test_citizen_array_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_TYPE_ARRAY, SerializableGeeCollectionsTestCitizenArray))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_TYPE_ARRAY, SerializableGeeCollectionsTestCitizenArrayClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_IS_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_TYPE_ARRAY))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_IS_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_TYPE_ARRAY))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_ARRAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_TYPE_ARRAY, SerializableGeeCollectionsTestCitizenArrayClass))

typedef struct _SerializableGeeCollectionsTestCitizenArray SerializableGeeCollectionsTestCitizenArray;
typedef struct _SerializableGeeCollectionsTestCitizenArrayClass SerializableGeeCollectionsTestCitizenArrayClass;
typedef struct _SerializableGeeCollectionsTestCitizenArrayPrivate SerializableGeeCollectionsTestCitizenArrayPrivate;

#define SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_ASTEROID (serializable_gee_collections_test_asteroid_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_ASTEROID, SerializableGeeCollectionsTestAsteroid))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_ASTEROID, SerializableGeeCollectionsTestAsteroidClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_ASTEROID(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_ASTEROID))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_ASTEROID_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_ASTEROID))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_ASTEROID, SerializableGeeCollectionsTestAsteroidClass))

typedef struct _SerializableGeeCollectionsTestAsteroid SerializableGeeCollectionsTestAsteroid;
typedef struct _SerializableGeeCollectionsTestAsteroidClass SerializableGeeCollectionsTestAsteroidClass;
typedef struct _SerializableGeeCollectionsTestAsteroidPrivate SerializableGeeCollectionsTestAsteroidPrivate;

#define SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_TYPE_ARRAY (serializable_gee_collections_test_asteroid_array_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_TYPE_ARRAY, SerializableGeeCollectionsTestAsteroidArray))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_TYPE_ARRAY, SerializableGeeCollectionsTestAsteroidArrayClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_IS_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_TYPE_ARRAY))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_IS_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_TYPE_ARRAY))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_ARRAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_TYPE_ARRAY, SerializableGeeCollectionsTestAsteroidArrayClass))

typedef struct _SerializableGeeCollectionsTestAsteroidArray SerializableGeeCollectionsTestAsteroidArray;
typedef struct _SerializableGeeCollectionsTestAsteroidArrayClass SerializableGeeCollectionsTestAsteroidArrayClass;
typedef struct _SerializableGeeCollectionsTestAsteroidArrayPrivate SerializableGeeCollectionsTestAsteroidArrayPrivate;
typedef struct _SerializableGeeCollectionsTestPlanetPrivate SerializableGeeCollectionsTestPlanetPrivate;
typedef struct _SerializableGeeCollectionsTestPlanetCollectionPrivate SerializableGeeCollectionsTestPlanetCollectionPrivate;
typedef struct _SerializableGeeCollectionsTestSpacePrivate SerializableGeeCollectionsTestSpacePrivate;
typedef struct _SerializableGeeCollectionsTestSpaceCollectionPrivate SerializableGeeCollectionsTestSpaceCollectionPrivate;
typedef struct _SerializableGeeCollectionsTestRefactionPrivate SerializableGeeCollectionsTestRefactionPrivate;
typedef struct _SerializableGeeCollectionsTestRefactionCollectionPrivate SerializableGeeCollectionsTestRefactionCollectionPrivate;
typedef struct _SerializableGeeCollectionsTestSpaceShipPrivate SerializableGeeCollectionsTestSpaceShipPrivate;
typedef struct _SerializableGeeCollectionsTestSpaceShipCollectionPrivate SerializableGeeCollectionsTestSpaceShipCollectionPrivate;
typedef struct _SerializableGeeCollectionsTestChargeZonePrivate SerializableGeeCollectionsTestChargeZonePrivate;
typedef struct _SerializableGeeCollectionsTestStoragePrivate SerializableGeeCollectionsTestStoragePrivate;
typedef struct _SerializableGeeCollectionsTestSpaceBasePrivate SerializableGeeCollectionsTestSpaceBasePrivate;

#define SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL_FILL (serializable_gee_collections_test_ball_fill_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_FILL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL_FILL, SerializableGeeCollectionsTestBallFill))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_FILL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL_FILL, SerializableGeeCollectionsTestBallFillClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_BALL_FILL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL_FILL))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_IS_BALL_FILL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL_FILL))
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_FILL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL_FILL, SerializableGeeCollectionsTestBallFillClass))

typedef struct _SerializableGeeCollectionsTestBallFill SerializableGeeCollectionsTestBallFill;
typedef struct _SerializableGeeCollectionsTestBallFillClass SerializableGeeCollectionsTestBallFillClass;
typedef struct _SerializableGeeCollectionsTestBallFillPrivate SerializableGeeCollectionsTestBallFillPrivate;
typedef struct _SerializableGeeCollectionsTestBallPrivate SerializableGeeCollectionsTestBallPrivate;
typedef struct _SerializableGeeCollectionsTestBallArrayPrivate SerializableGeeCollectionsTestBallArrayPrivate;
typedef struct _SerializableGeeCollectionsTestSmallBagPrivate SerializableGeeCollectionsTestSmallBagPrivate;
typedef struct _SerializableGeeCollectionsTestSmallBagArrayPrivate SerializableGeeCollectionsTestSmallBagArrayPrivate;
typedef struct _SerializableGeeCollectionsTestBigBagPrivate SerializableGeeCollectionsTestBigBagPrivate;
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _GXmlTest {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GXmlTestPrivate * priv;
};

struct _GXmlTestClass {
	GTypeClass parent_class;
	void (*finalize) (GXmlTest *self);
};

struct _SerializableGeeCollectionsTest {
	GXmlTest parent_instance;
	SerializableGeeCollectionsTestPrivate * priv;
};

struct _SerializableGeeCollectionsTestClass {
	GXmlTestClass parent_class;
};

struct _SerializableGeeCollectionsTestCitizen {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeCollectionsTestCitizenPrivate * priv;
};

struct _SerializableGeeCollectionsTestCitizenClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeCollectionsTestCitizenPrivate {
	gchar* _ctype;
};

struct _SerializableGeeCollectionsTestCitizenArray {
	GXmlSerializableArrayList parent_instance;
	SerializableGeeCollectionsTestCitizenArrayPrivate * priv;
};

struct _SerializableGeeCollectionsTestCitizenArrayClass {
	GXmlSerializableArrayListClass parent_class;
};

struct _SerializableGeeCollectionsTestAsteroid {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeCollectionsTestAsteroidPrivate * priv;
};

struct _SerializableGeeCollectionsTestAsteroidClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeCollectionsTestAsteroidPrivate {
	gint _size;
};

struct _SerializableGeeCollectionsTestAsteroidArray {
	GXmlSerializableArrayList parent_instance;
	SerializableGeeCollectionsTestAsteroidArrayPrivate * priv;
};

struct _SerializableGeeCollectionsTestAsteroidArrayClass {
	GXmlSerializableArrayListClass parent_class;
};

struct _SerializableGeeCollectionsTestPlanet {
	GXmlSerializableContainer parent_instance;
	SerializableGeeCollectionsTestPlanetPrivate * priv;
};

struct _SerializableGeeCollectionsTestPlanetClass {
	GXmlSerializableContainerClass parent_class;
};

struct _SerializableGeeCollectionsTestPlanetPrivate {
	gchar* _name;
	SerializableGeeCollectionsTestCitizenArray* _citizens;
};

struct _SerializableGeeCollectionsTestPlanetCollection {
	GXmlSerializableTreeMap parent_instance;
	SerializableGeeCollectionsTestPlanetCollectionPrivate * priv;
};

struct _SerializableGeeCollectionsTestPlanetCollectionClass {
	GXmlSerializableTreeMapClass parent_class;
};

struct _SerializableGeeCollectionsTestSpace {
	GXmlSerializableContainer parent_instance;
	SerializableGeeCollectionsTestSpacePrivate * priv;
};

struct _SerializableGeeCollectionsTestSpaceClass {
	GXmlSerializableContainerClass parent_class;
};

struct _SerializableGeeCollectionsTestSpacePrivate {
	gchar* _name;
	SerializableGeeCollectionsTestPlanetCollection* _planets;
	SerializableGeeCollectionsTestAsteroidArray* _asteroids;
};

struct _SerializableGeeCollectionsTestSpaceCollection {
	GXmlSerializableTreeMap parent_instance;
	SerializableGeeCollectionsTestSpaceCollectionPrivate * priv;
};

struct _SerializableGeeCollectionsTestSpaceCollectionClass {
	GXmlSerializableTreeMapClass parent_class;
};

struct _SerializableGeeCollectionsTestRefaction {
	GXmlSerializableContainer parent_instance;
	SerializableGeeCollectionsTestRefactionPrivate * priv;
};

struct _SerializableGeeCollectionsTestRefactionClass {
	GXmlSerializableContainerClass parent_class;
};

struct _SerializableGeeCollectionsTestRefactionPrivate {
	gchar* _model;
	gchar* _manufacturer;
	SerializableGeeCollectionsTestSpaceShipCollection* _spaceships;
};

struct _SerializableGeeCollectionsTestRefactionCollection {
	GXmlSerializableDualKeyMap parent_instance;
	SerializableGeeCollectionsTestRefactionCollectionPrivate * priv;
};

struct _SerializableGeeCollectionsTestRefactionCollectionClass {
	GXmlSerializableDualKeyMapClass parent_class;
};

struct _SerializableGeeCollectionsTestSpaceShip {
	GXmlSerializableContainer parent_instance;
	SerializableGeeCollectionsTestSpaceShipPrivate * priv;
};

struct _SerializableGeeCollectionsTestSpaceShipClass {
	GXmlSerializableContainerClass parent_class;
};

struct _SerializableGeeCollectionsTestSpaceShipPrivate {
	gchar* _model;
	gchar* _manufacturer;
	SerializableGeeCollectionsTestSpaceCollection* _spaces;
};

struct _SerializableGeeCollectionsTestSpaceShipCollection {
	GXmlSerializableDualKeyMap parent_instance;
	SerializableGeeCollectionsTestSpaceShipCollectionPrivate * priv;
};

struct _SerializableGeeCollectionsTestSpaceShipCollectionClass {
	GXmlSerializableDualKeyMapClass parent_class;
};

struct _SerializableGeeCollectionsTestChargeZone {
	GXmlSerializableContainer parent_instance;
	SerializableGeeCollectionsTestChargeZonePrivate * priv;
};

struct _SerializableGeeCollectionsTestChargeZoneClass {
	GXmlSerializableContainerClass parent_class;
};

struct _SerializableGeeCollectionsTestChargeZonePrivate {
	gchar* _name;
	SerializableGeeCollectionsTestSpaceShipCollection* _spaceships;
};

struct _SerializableGeeCollectionsTestStorage {
	GXmlSerializableContainer parent_instance;
	SerializableGeeCollectionsTestStoragePrivate * priv;
};

struct _SerializableGeeCollectionsTestStorageClass {
	GXmlSerializableContainerClass parent_class;
};

struct _SerializableGeeCollectionsTestStoragePrivate {
	gchar* _name;
	SerializableGeeCollectionsTestRefactionCollection* _refactions;
};

struct _SerializableGeeCollectionsTestSpaceBase {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeCollectionsTestSpaceBasePrivate * priv;
};

struct _SerializableGeeCollectionsTestSpaceBaseClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeCollectionsTestSpaceBasePrivate {
	gchar* _name;
	SerializableGeeCollectionsTestChargeZone* _charge_zone;
	SerializableGeeCollectionsTestStorage* _storage;
};

struct _SerializableGeeCollectionsTestBallFill {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeCollectionsTestBallFillPrivate * priv;
};

struct _SerializableGeeCollectionsTestBallFillClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeCollectionsTestBallFillPrivate {
	gchar* _name;
};

struct _SerializableGeeCollectionsTestBall {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeCollectionsTestBallPrivate * priv;
};

struct _SerializableGeeCollectionsTestBallClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeCollectionsTestBallPrivate {
	gchar* _name;
	SerializableGeeCollectionsTestBallFill* _ballfill;
};

struct _SerializableGeeCollectionsTestBallArray {
	GXmlSerializableArrayList parent_instance;
	SerializableGeeCollectionsTestBallArrayPrivate * priv;
};

struct _SerializableGeeCollectionsTestBallArrayClass {
	GXmlSerializableArrayListClass parent_class;
};

struct _SerializableGeeCollectionsTestSmallBag {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeCollectionsTestSmallBagPrivate * priv;
};

struct _SerializableGeeCollectionsTestSmallBagClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeCollectionsTestSmallBagPrivate {
	gchar* _name;
	SerializableGeeCollectionsTestBallArray* _balls;
};

struct _SerializableGeeCollectionsTestSmallBagArray {
	GXmlSerializableArrayList parent_instance;
	SerializableGeeCollectionsTestSmallBagArrayPrivate * priv;
};

struct _SerializableGeeCollectionsTestSmallBagArrayClass {
	GXmlSerializableArrayListClass parent_class;
};

struct _SerializableGeeCollectionsTestBigBag {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeCollectionsTestBigBagPrivate * priv;
};

struct _SerializableGeeCollectionsTestBigBagClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeCollectionsTestBigBagPrivate {
	gchar* _name;
	SerializableGeeCollectionsTestSmallBagArray* _bags;
};


static gpointer serializable_gee_collections_test_parent_class = NULL;
static gpointer serializable_gee_collections_test_citizen_parent_class = NULL;
static gpointer serializable_gee_collections_test_citizen_array_parent_class = NULL;
static gpointer serializable_gee_collections_test_asteroid_parent_class = NULL;
static gpointer serializable_gee_collections_test_asteroid_array_parent_class = NULL;
static gpointer serializable_gee_collections_test_planet_parent_class = NULL;
static gpointer serializable_gee_collections_test_planet_collection_parent_class = NULL;
static GXmlSerializableMapKeyIface* serializable_gee_collections_test_planet_gxml_serializable_map_key_parent_iface = NULL;
static gpointer serializable_gee_collections_test_space_parent_class = NULL;
static gpointer serializable_gee_collections_test_space_collection_parent_class = NULL;
static GXmlSerializableMapKeyIface* serializable_gee_collections_test_space_gxml_serializable_map_key_parent_iface = NULL;
static gpointer serializable_gee_collections_test_refaction_parent_class = NULL;
static gpointer serializable_gee_collections_test_refaction_collection_parent_class = NULL;
static GXmlSerializableMapDualKeyIface* serializable_gee_collections_test_refaction_gxml_serializable_map_dual_key_parent_iface = NULL;
static gpointer serializable_gee_collections_test_space_ship_parent_class = NULL;
static gpointer serializable_gee_collections_test_space_ship_collection_parent_class = NULL;
static GXmlSerializableMapDualKeyIface* serializable_gee_collections_test_space_ship_gxml_serializable_map_dual_key_parent_iface = NULL;
static gpointer serializable_gee_collections_test_charge_zone_parent_class = NULL;
static gpointer serializable_gee_collections_test_storage_parent_class = NULL;
static gpointer serializable_gee_collections_test_space_base_parent_class = NULL;
static gpointer serializable_gee_collections_test_ball_fill_parent_class = NULL;
static gpointer serializable_gee_collections_test_ball_parent_class = NULL;
static gpointer serializable_gee_collections_test_ball_array_parent_class = NULL;
static gpointer serializable_gee_collections_test_small_bag_parent_class = NULL;
static gpointer serializable_gee_collections_test_small_bag_array_parent_class = NULL;
static gpointer serializable_gee_collections_test_big_bag_parent_class = NULL;

gpointer gxml_test_ref (gpointer instance);
void gxml_test_unref (gpointer instance);
GParamSpec* param_spec_gxml_test (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_gxml_test (GValue* value, gpointer v_object);
void value_take_gxml_test (GValue* value, gpointer v_object);
gpointer value_get_gxml_test (const GValue* value);
GType gxml_test_get_type (void) G_GNUC_CONST;
GType serializable_gee_collections_test_get_type (void) G_GNUC_CONST;
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_DUMMY_PROPERTY
};
void serializable_gee_collections_test_add_tests (void);
static void __lambda66_ (void);
static GType serializable_gee_collections_test_space_base_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeCollectionsTestSpaceBase* serializable_gee_collections_test_space_base_new (void);
static SerializableGeeCollectionsTestSpaceBase* serializable_gee_collections_test_space_base_construct (GType object_type);
static GType serializable_gee_collections_test_storage_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeCollectionsTestStorage* serializable_gee_collections_test_space_base_get_storage (SerializableGeeCollectionsTestSpaceBase* self);
GType serializable_gee_collections_test_refaction_collection_get_type (void) G_GNUC_CONST;
static SerializableGeeCollectionsTestRefactionCollection* serializable_gee_collections_test_storage_get_refactions (SerializableGeeCollectionsTestStorage* self);
static GType serializable_gee_collections_test_refaction_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
GType serializable_gee_collections_test_space_ship_collection_get_type (void) G_GNUC_CONST;
static SerializableGeeCollectionsTestSpaceShipCollection* serializable_gee_collections_test_refaction_get_spaceships (SerializableGeeCollectionsTestRefaction* self);
static GType serializable_gee_collections_test_space_ship_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static GType serializable_gee_collections_test_charge_zone_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeCollectionsTestChargeZone* serializable_gee_collections_test_space_base_get_charge_zone (SerializableGeeCollectionsTestSpaceBase* self);
static SerializableGeeCollectionsTestSpaceShipCollection* serializable_gee_collections_test_charge_zone_get_spaceships (SerializableGeeCollectionsTestChargeZone* self);
GType serializable_gee_collections_test_space_collection_get_type (void) G_GNUC_CONST;
static SerializableGeeCollectionsTestSpaceCollection* serializable_gee_collections_test_space_ship_get_spaces (SerializableGeeCollectionsTestSpaceShip* self);
static GType serializable_gee_collections_test_space_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
GType serializable_gee_collections_test_planet_collection_get_type (void) G_GNUC_CONST;
static SerializableGeeCollectionsTestPlanetCollection* serializable_gee_collections_test_space_get_planets (SerializableGeeCollectionsTestSpace* self);
static GType serializable_gee_collections_test_planet_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static void ___lambda66__gtest_func (void);
static void __lambda67_ (void);
static const gchar* serializable_gee_collections_test_space_ship_get_manufacturer (SerializableGeeCollectionsTestSpaceShip* self);
static void ___lambda67__gtest_func (void);
static void __lambda68_ (void);
static const gchar* serializable_gee_collections_test_space_base_get_name (SerializableGeeCollectionsTestSpaceBase* self);
static void ___lambda68__gtest_func (void);
static void __lambda69_ (void);
static GType serializable_gee_collections_test_big_bag_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeCollectionsTestBigBag* serializable_gee_collections_test_big_bag_new (void);
static SerializableGeeCollectionsTestBigBag* serializable_gee_collections_test_big_bag_construct (GType object_type);
GType serializable_gee_collections_test_small_bag_array_get_type (void) G_GNUC_CONST;
static SerializableGeeCollectionsTestSmallBagArray* serializable_gee_collections_test_big_bag_get_bags (SerializableGeeCollectionsTestBigBag* self);
static GType serializable_gee_collections_test_small_bag_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeCollectionsTestSmallBag* serializable_gee_collections_test_small_bag_new (void);
static SerializableGeeCollectionsTestSmallBag* serializable_gee_collections_test_small_bag_construct (GType object_type);
GType serializable_gee_collections_test_ball_array_get_type (void) G_GNUC_CONST;
static SerializableGeeCollectionsTestBallArray* serializable_gee_collections_test_small_bag_get_balls (SerializableGeeCollectionsTestSmallBag* self);
static GType serializable_gee_collections_test_ball_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeCollectionsTestBall* serializable_gee_collections_test_ball_new (void);
static SerializableGeeCollectionsTestBall* serializable_gee_collections_test_ball_construct (GType object_type);
static void ___lambda69__gtest_func (void);
SerializableGeeCollectionsTest* serializable_gee_collections_test_new (void);
SerializableGeeCollectionsTest* serializable_gee_collections_test_construct (GType object_type);
GXmlTest* gxml_test_new (void);
GXmlTest* gxml_test_construct (GType object_type);
static GType serializable_gee_collections_test_citizen_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CITIZEN, SerializableGeeCollectionsTestCitizenPrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_CTYPE
};
static gboolean serializable_gee_collections_test_citizen_real_serialize_use_xml_node_value (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_collections_test_citizen_real_to_string (GXmlSerializableObjectModel* base);
static const gchar* serializable_gee_collections_test_citizen_get_ctype (SerializableGeeCollectionsTestCitizen* self);
static gchar* serializable_gee_collections_test_citizen_real_node_name (GXmlSerializableObjectModel* base);
static SerializableGeeCollectionsTestCitizen* serializable_gee_collections_test_citizen_new (void);
static SerializableGeeCollectionsTestCitizen* serializable_gee_collections_test_citizen_construct (GType object_type);
static void serializable_gee_collections_test_citizen_set_ctype (SerializableGeeCollectionsTestCitizen* self, const gchar* value);
GType serializable_gee_collections_test_citizen_array_get_type (void) G_GNUC_CONST;
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_ARRAY_DUMMY_PROPERTY
};
static SerializableGeeCollectionsTestCitizenArray* serializable_gee_collections_test_citizen_array_new (void);
static SerializableGeeCollectionsTestCitizenArray* serializable_gee_collections_test_citizen_array_construct (GType object_type);
static void serializable_gee_collections_test_citizen_finalize (GObject* obj);
static void _vala_serializable_gee_collections_test_citizen_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_test_citizen_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static GType serializable_gee_collections_test_asteroid_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_ASTEROID, SerializableGeeCollectionsTestAsteroidPrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_SIZE
};
static SerializableGeeCollectionsTestAsteroid* serializable_gee_collections_test_asteroid_new_with_size (gint size);
static SerializableGeeCollectionsTestAsteroid* serializable_gee_collections_test_asteroid_construct_with_size (GType object_type, gint size);
static void serializable_gee_collections_test_asteroid_set_size (SerializableGeeCollectionsTestAsteroid* self, gint value);
static gchar* serializable_gee_collections_test_asteroid_real_to_string (GXmlSerializableObjectModel* base);
static gint serializable_gee_collections_test_asteroid_get_size (SerializableGeeCollectionsTestAsteroid* self);
static gchar* serializable_gee_collections_test_asteroid_real_node_name (GXmlSerializableObjectModel* base);
static SerializableGeeCollectionsTestAsteroid* serializable_gee_collections_test_asteroid_new (void);
static SerializableGeeCollectionsTestAsteroid* serializable_gee_collections_test_asteroid_construct (GType object_type);
GType serializable_gee_collections_test_asteroid_array_get_type (void) G_GNUC_CONST;
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_ARRAY_DUMMY_PROPERTY
};
static SerializableGeeCollectionsTestAsteroidArray* serializable_gee_collections_test_asteroid_array_new (void);
static SerializableGeeCollectionsTestAsteroidArray* serializable_gee_collections_test_asteroid_array_construct (GType object_type);
static void serializable_gee_collections_test_asteroid_finalize (GObject* obj);
static void _vala_serializable_gee_collections_test_asteroid_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_test_asteroid_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_PLANET, SerializableGeeCollectionsTestPlanetPrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_NAME,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_CITIZENS
};
static gchar* serializable_gee_collections_test_planet_real_get_map_key (GXmlSerializableMapKey* base);
static const gchar* serializable_gee_collections_test_planet_get_name (SerializableGeeCollectionsTestPlanet* self);
static SerializableGeeCollectionsTestPlanet* serializable_gee_collections_test_planet_new_named (const gchar* name);
static SerializableGeeCollectionsTestPlanet* serializable_gee_collections_test_planet_construct_named (GType object_type, const gchar* name);
static void serializable_gee_collections_test_planet_set_name (SerializableGeeCollectionsTestPlanet* self, const gchar* value);
static gchar* serializable_gee_collections_test_planet_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_collections_test_planet_real_to_string (GXmlSerializableObjectModel* base);
static void serializable_gee_collections_test_planet_real_init_containers (GXmlSerializableContainer* base);
static SerializableGeeCollectionsTestCitizenArray* serializable_gee_collections_test_planet_get_citizens (SerializableGeeCollectionsTestPlanet* self);
static void serializable_gee_collections_test_planet_set_citizens (SerializableGeeCollectionsTestPlanet* self, SerializableGeeCollectionsTestCitizenArray* value);
static SerializableGeeCollectionsTestPlanet* serializable_gee_collections_test_planet_new (void);
static SerializableGeeCollectionsTestPlanet* serializable_gee_collections_test_planet_construct (GType object_type);
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_COLLECTION_DUMMY_PROPERTY
};
static SerializableGeeCollectionsTestPlanetCollection* serializable_gee_collections_test_planet_collection_new (void);
static SerializableGeeCollectionsTestPlanetCollection* serializable_gee_collections_test_planet_collection_construct (GType object_type);
static void serializable_gee_collections_test_planet_finalize (GObject* obj);
static void _vala_serializable_gee_collections_test_planet_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_test_planet_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE, SerializableGeeCollectionsTestSpacePrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_NAME,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_PLANETS,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_ASTEROIDS
};
static gchar* serializable_gee_collections_test_space_real_get_map_key (GXmlSerializableMapKey* base);
static const gchar* serializable_gee_collections_test_space_get_name (SerializableGeeCollectionsTestSpace* self);
static SerializableGeeCollectionsTestSpace* serializable_gee_collections_test_space_new_named (const gchar* name);
static SerializableGeeCollectionsTestSpace* serializable_gee_collections_test_space_construct_named (GType object_type, const gchar* name);
static void serializable_gee_collections_test_space_set_name (SerializableGeeCollectionsTestSpace* self, const gchar* value);
static gchar* serializable_gee_collections_test_space_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_collections_test_space_real_to_string (GXmlSerializableObjectModel* base);
static void serializable_gee_collections_test_space_real_init_containers (GXmlSerializableContainer* base);
static SerializableGeeCollectionsTestAsteroidArray* serializable_gee_collections_test_space_get_asteroids (SerializableGeeCollectionsTestSpace* self);
static void serializable_gee_collections_test_space_set_asteroids (SerializableGeeCollectionsTestSpace* self, SerializableGeeCollectionsTestAsteroidArray* value);
static void serializable_gee_collections_test_space_set_planets (SerializableGeeCollectionsTestSpace* self, SerializableGeeCollectionsTestPlanetCollection* value);
static SerializableGeeCollectionsTestSpace* serializable_gee_collections_test_space_new (void);
static SerializableGeeCollectionsTestSpace* serializable_gee_collections_test_space_construct (GType object_type);
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_COLLECTION_DUMMY_PROPERTY
};
static SerializableGeeCollectionsTestSpaceCollection* serializable_gee_collections_test_space_collection_new (void);
static SerializableGeeCollectionsTestSpaceCollection* serializable_gee_collections_test_space_collection_construct (GType object_type);
static void serializable_gee_collections_test_space_finalize (GObject* obj);
static void _vala_serializable_gee_collections_test_space_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_test_space_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_REFACTION, SerializableGeeCollectionsTestRefactionPrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_MODEL,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_MANUFACTURER,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_SPACESHIPS
};
static SerializableGeeCollectionsTestRefaction* serializable_gee_collections_test_refaction_new_named (const gchar* manufacturer, const gchar* model);
static SerializableGeeCollectionsTestRefaction* serializable_gee_collections_test_refaction_construct_named (GType object_type, const gchar* manufacturer, const gchar* model);
static void serializable_gee_collections_test_refaction_set_manufacturer (SerializableGeeCollectionsTestRefaction* self, const gchar* value);
static void serializable_gee_collections_test_refaction_set_model (SerializableGeeCollectionsTestRefaction* self, const gchar* value);
static gchar* serializable_gee_collections_test_refaction_real_get_map_primary_key (GXmlSerializableMapDualKey* base);
static const gchar* serializable_gee_collections_test_refaction_get_manufacturer (SerializableGeeCollectionsTestRefaction* self);
static gchar* serializable_gee_collections_test_refaction_real_get_map_secondary_key (GXmlSerializableMapDualKey* base);
static const gchar* serializable_gee_collections_test_refaction_get_model (SerializableGeeCollectionsTestRefaction* self);
static gchar* serializable_gee_collections_test_refaction_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_collections_test_refaction_real_to_string (GXmlSerializableObjectModel* base);
static gboolean serializable_gee_collections_test_refaction_real_get_enable_unknown_serializable_property (GXmlSerializableObjectModel* base);
static void serializable_gee_collections_test_refaction_real_init_containers (GXmlSerializableContainer* base);
static SerializableGeeCollectionsTestSpaceShipCollection* serializable_gee_collections_test_space_ship_collection_new (void);
static SerializableGeeCollectionsTestSpaceShipCollection* serializable_gee_collections_test_space_ship_collection_construct (GType object_type);
static void serializable_gee_collections_test_refaction_set_spaceships (SerializableGeeCollectionsTestRefaction* self, SerializableGeeCollectionsTestSpaceShipCollection* value);
static SerializableGeeCollectionsTestRefaction* serializable_gee_collections_test_refaction_new (void);
static SerializableGeeCollectionsTestRefaction* serializable_gee_collections_test_refaction_construct (GType object_type);
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_COLLECTION_DUMMY_PROPERTY
};
static SerializableGeeCollectionsTestRefactionCollection* serializable_gee_collections_test_refaction_collection_new (void);
static SerializableGeeCollectionsTestRefactionCollection* serializable_gee_collections_test_refaction_collection_construct (GType object_type);
static void serializable_gee_collections_test_refaction_finalize (GObject* obj);
static void _vala_serializable_gee_collections_test_refaction_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_test_refaction_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_SHIP, SerializableGeeCollectionsTestSpaceShipPrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_MODEL,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_MANUFACTURER,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_SPACES
};
static SerializableGeeCollectionsTestSpaceShip* serializable_gee_collections_test_space_ship_new_named (const gchar* manufacturer, const gchar* model);
static SerializableGeeCollectionsTestSpaceShip* serializable_gee_collections_test_space_ship_construct_named (GType object_type, const gchar* manufacturer, const gchar* model);
static void serializable_gee_collections_test_space_ship_set_manufacturer (SerializableGeeCollectionsTestSpaceShip* self, const gchar* value);
static void serializable_gee_collections_test_space_ship_set_model (SerializableGeeCollectionsTestSpaceShip* self, const gchar* value);
static gchar* serializable_gee_collections_test_space_ship_real_get_map_primary_key (GXmlSerializableMapDualKey* base);
static gchar* serializable_gee_collections_test_space_ship_real_get_map_secondary_key (GXmlSerializableMapDualKey* base);
static const gchar* serializable_gee_collections_test_space_ship_get_model (SerializableGeeCollectionsTestSpaceShip* self);
static gchar* serializable_gee_collections_test_space_ship_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_collections_test_space_ship_real_to_string (GXmlSerializableObjectModel* base);
static gboolean serializable_gee_collections_test_space_ship_real_get_enable_unknown_serializable_property (GXmlSerializableObjectModel* base);
static void serializable_gee_collections_test_space_ship_real_init_containers (GXmlSerializableContainer* base);
static void serializable_gee_collections_test_space_ship_set_spaces (SerializableGeeCollectionsTestSpaceShip* self, SerializableGeeCollectionsTestSpaceCollection* value);
static SerializableGeeCollectionsTestSpaceShip* serializable_gee_collections_test_space_ship_new (void);
static SerializableGeeCollectionsTestSpaceShip* serializable_gee_collections_test_space_ship_construct (GType object_type);
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_COLLECTION_DUMMY_PROPERTY
};
static void serializable_gee_collections_test_space_ship_finalize (GObject* obj);
static void _vala_serializable_gee_collections_test_space_ship_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_test_space_ship_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_CHARGE_ZONE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CHARGE_ZONE, SerializableGeeCollectionsTestChargeZonePrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_CHARGE_ZONE_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_CHARGE_ZONE_NAME,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_CHARGE_ZONE_SPACESHIPS
};
static gchar* serializable_gee_collections_test_charge_zone_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_collections_test_charge_zone_real_to_string (GXmlSerializableObjectModel* base);
static const gchar* serializable_gee_collections_test_charge_zone_get_name (SerializableGeeCollectionsTestChargeZone* self);
static void serializable_gee_collections_test_charge_zone_real_init_containers (GXmlSerializableContainer* base);
static void serializable_gee_collections_test_charge_zone_set_spaceships (SerializableGeeCollectionsTestChargeZone* self, SerializableGeeCollectionsTestSpaceShipCollection* value);
static SerializableGeeCollectionsTestChargeZone* serializable_gee_collections_test_charge_zone_new (void);
static SerializableGeeCollectionsTestChargeZone* serializable_gee_collections_test_charge_zone_construct (GType object_type);
static void serializable_gee_collections_test_charge_zone_set_name (SerializableGeeCollectionsTestChargeZone* self, const gchar* value);
static void serializable_gee_collections_test_charge_zone_finalize (GObject* obj);
static void _vala_serializable_gee_collections_test_charge_zone_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_test_charge_zone_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_STORAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_STORAGE, SerializableGeeCollectionsTestStoragePrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_STORAGE_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_STORAGE_NAME,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_STORAGE_REFACTIONS
};
static gchar* serializable_gee_collections_test_storage_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_collections_test_storage_real_to_string (GXmlSerializableObjectModel* base);
static const gchar* serializable_gee_collections_test_storage_get_name (SerializableGeeCollectionsTestStorage* self);
static void serializable_gee_collections_test_storage_real_init_containers (GXmlSerializableContainer* base);
static void serializable_gee_collections_test_storage_set_refactions (SerializableGeeCollectionsTestStorage* self, SerializableGeeCollectionsTestRefactionCollection* value);
static SerializableGeeCollectionsTestStorage* serializable_gee_collections_test_storage_new (void);
static SerializableGeeCollectionsTestStorage* serializable_gee_collections_test_storage_construct (GType object_type);
static void serializable_gee_collections_test_storage_set_name (SerializableGeeCollectionsTestStorage* self, const gchar* value);
static void serializable_gee_collections_test_storage_finalize (GObject* obj);
static void _vala_serializable_gee_collections_test_storage_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_test_storage_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_BASE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_BASE, SerializableGeeCollectionsTestSpaceBasePrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_BASE_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_BASE_NAME,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_BASE_CHARGE_ZONE,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_BASE_STORAGE
};
static gchar* serializable_gee_collections_test_space_base_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_collections_test_space_base_real_to_string (GXmlSerializableObjectModel* base);
static void serializable_gee_collections_test_space_base_set_name (SerializableGeeCollectionsTestSpaceBase* self, const gchar* value);
static void serializable_gee_collections_test_space_base_set_charge_zone (SerializableGeeCollectionsTestSpaceBase* self, SerializableGeeCollectionsTestChargeZone* value);
static void serializable_gee_collections_test_space_base_set_storage (SerializableGeeCollectionsTestSpaceBase* self, SerializableGeeCollectionsTestStorage* value);
static void serializable_gee_collections_test_space_base_finalize (GObject* obj);
static void _vala_serializable_gee_collections_test_space_base_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_test_space_base_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static GType serializable_gee_collections_test_ball_fill_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_FILL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL_FILL, SerializableGeeCollectionsTestBallFillPrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_FILL_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_FILL_NAME
};
static gchar* serializable_gee_collections_test_ball_fill_real_to_string (GXmlSerializableObjectModel* base);
static const gchar* serializable_gee_collections_test_ball_fill_get_name (SerializableGeeCollectionsTestBallFill* self);
static gchar* serializable_gee_collections_test_ball_fill_real_node_name (GXmlSerializableObjectModel* base);
static gboolean serializable_gee_collections_test_ball_fill_real_serialize_use_xml_node_value (GXmlSerializableObjectModel* base);
static SerializableGeeCollectionsTestBallFill* serializable_gee_collections_test_ball_fill_new (void);
static SerializableGeeCollectionsTestBallFill* serializable_gee_collections_test_ball_fill_construct (GType object_type);
static void serializable_gee_collections_test_ball_fill_set_name (SerializableGeeCollectionsTestBallFill* self, const gchar* value);
static void serializable_gee_collections_test_ball_fill_finalize (GObject* obj);
static void _vala_serializable_gee_collections_test_ball_fill_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_test_ball_fill_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL, SerializableGeeCollectionsTestBallPrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_NAME,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_BALLFILL
};
static gchar* serializable_gee_collections_test_ball_real_to_string (GXmlSerializableObjectModel* base);
static const gchar* serializable_gee_collections_test_ball_get_name (SerializableGeeCollectionsTestBall* self);
static gchar* serializable_gee_collections_test_ball_real_node_name (GXmlSerializableObjectModel* base);
static void serializable_gee_collections_test_ball_set_name (SerializableGeeCollectionsTestBall* self, const gchar* value);
static SerializableGeeCollectionsTestBallFill* serializable_gee_collections_test_ball_get_ballfill (SerializableGeeCollectionsTestBall* self);
static void serializable_gee_collections_test_ball_set_ballfill (SerializableGeeCollectionsTestBall* self, SerializableGeeCollectionsTestBallFill* value);
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_ARRAY_DUMMY_PROPERTY
};
static gboolean serializable_gee_collections_test_ball_array_real_deserialize_proceed (GXmlSerializableArrayList* base);
static SerializableGeeCollectionsTestBallArray* serializable_gee_collections_test_ball_array_new (void);
static SerializableGeeCollectionsTestBallArray* serializable_gee_collections_test_ball_array_construct (GType object_type);
static void serializable_gee_collections_test_ball_finalize (GObject* obj);
static void _vala_serializable_gee_collections_test_ball_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_test_ball_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SMALL_BAG, SerializableGeeCollectionsTestSmallBagPrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_NAME,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_BALLS
};
static gchar* serializable_gee_collections_test_small_bag_real_to_string (GXmlSerializableObjectModel* base);
static const gchar* serializable_gee_collections_test_small_bag_get_name (SerializableGeeCollectionsTestSmallBag* self);
static gchar* serializable_gee_collections_test_small_bag_real_node_name (GXmlSerializableObjectModel* base);
static void serializable_gee_collections_test_small_bag_set_name (SerializableGeeCollectionsTestSmallBag* self, const gchar* value);
static void serializable_gee_collections_test_small_bag_set_balls (SerializableGeeCollectionsTestSmallBag* self, SerializableGeeCollectionsTestBallArray* value);
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_ARRAY_DUMMY_PROPERTY
};
static gboolean serializable_gee_collections_test_small_bag_array_real_deserialize_proceed (GXmlSerializableArrayList* base);
static SerializableGeeCollectionsTestSmallBagArray* serializable_gee_collections_test_small_bag_array_new (void);
static SerializableGeeCollectionsTestSmallBagArray* serializable_gee_collections_test_small_bag_array_construct (GType object_type);
static void serializable_gee_collections_test_small_bag_finalize (GObject* obj);
static void _vala_serializable_gee_collections_test_small_bag_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_test_small_bag_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_COLLECTIONS_TEST_BIG_BAG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BIG_BAG, SerializableGeeCollectionsTestBigBagPrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TEST_BIG_BAG_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_BIG_BAG_NAME,
	SERIALIZABLE_GEE_COLLECTIONS_TEST_BIG_BAG_BAGS
};
static gchar* serializable_gee_collections_test_big_bag_real_to_string (GXmlSerializableObjectModel* base);
static const gchar* serializable_gee_collections_test_big_bag_get_name (SerializableGeeCollectionsTestBigBag* self);
static gchar* serializable_gee_collections_test_big_bag_real_node_name (GXmlSerializableObjectModel* base);
static void serializable_gee_collections_test_big_bag_set_name (SerializableGeeCollectionsTestBigBag* self, const gchar* value);
static void serializable_gee_collections_test_big_bag_set_bags (SerializableGeeCollectionsTestBigBag* self, SerializableGeeCollectionsTestSmallBagArray* value);
static void serializable_gee_collections_test_big_bag_finalize (GObject* obj);
static void _vala_serializable_gee_collections_test_big_bag_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_test_big_bag_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);


static gpointer _g_object_ref0 (gpointer self) {
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self ? g_object_ref (self) : NULL;
#line 921 "SerializableGeeCollectionsTest.c"
}


static const gchar* string_to_string (const gchar* self) {
	const gchar* result = NULL;
#line 1412 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1413 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	result = self;
#line 1413 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	return result;
#line 933 "SerializableGeeCollectionsTest.c"
}


static void __lambda66_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		SerializableGeeCollectionsTestSpaceBase* b = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp1_ = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp4_ = NULL;
		SerializableGeeCollectionsTestStorage* _tmp5_ = NULL;
		SerializableGeeCollectionsTestStorage* _tmp6_ = NULL;
		SerializableGeeCollectionsTestStorage* storage = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp13_ = NULL;
		SerializableGeeCollectionsTestStorage* _tmp14_ = NULL;
		SerializableGeeCollectionsTestStorage* _tmp15_ = NULL;
		SerializableGeeCollectionsTestStorage* _tmp16_ = NULL;
		SerializableGeeCollectionsTestRefactionCollection* _tmp17_ = NULL;
		SerializableGeeCollectionsTestRefactionCollection* _tmp18_ = NULL;
		SerializableGeeCollectionsTestRefactionCollection* _tmp25_ = NULL;
		SerializableGeeCollectionsTestRefactionCollection* _tmp26_ = NULL;
		gint _tmp27_ = 0;
		gint _tmp28_ = 0;
		SerializableGeeCollectionsTestRefaction* refaction = NULL;
		SerializableGeeCollectionsTestRefactionCollection* _tmp41_ = NULL;
		SerializableGeeCollectionsTestRefactionCollection* _tmp42_ = NULL;
		gpointer _tmp43_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp50_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp51_ = NULL;
		gint _tmp52_ = 0;
		gint _tmp53_ = 0;
		SerializableGeeCollectionsTestSpaceShip* ship1 = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp66_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp67_ = NULL;
		gpointer _tmp68_ = NULL;
		SerializableGeeCollectionsTestSpaceShip* ship2 = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp75_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp76_ = NULL;
		gpointer _tmp77_ = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp84_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp85_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp86_ = NULL;
		SerializableGeeCollectionsTestChargeZone* charge = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp93_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp94_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp95_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp96_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp97_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp98_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp105_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp106_ = NULL;
		gint _tmp107_ = 0;
		gint _tmp108_ = 0;
		SerializableGeeCollectionsTestSpaceShip* ship = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp121_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp122_ = NULL;
		gpointer _tmp123_ = NULL;
		SerializableGeeCollectionsTestSpaceCollection* _tmp130_ = NULL;
		SerializableGeeCollectionsTestSpaceCollection* _tmp131_ = NULL;
		gint _tmp132_ = 0;
		gint _tmp133_ = 0;
		SerializableGeeCollectionsTestSpace* space = NULL;
		SerializableGeeCollectionsTestSpaceCollection* _tmp146_ = NULL;
		SerializableGeeCollectionsTestSpaceCollection* _tmp147_ = NULL;
		gpointer _tmp148_ = NULL;
		SerializableGeeCollectionsTestPlanetCollection* _tmp155_ = NULL;
		SerializableGeeCollectionsTestPlanetCollection* _tmp156_ = NULL;
		gint _tmp157_ = 0;
		gint _tmp158_ = 0;
		SerializableGeeCollectionsTestPlanet* earth = NULL;
		SerializableGeeCollectionsTestPlanetCollection* _tmp171_ = NULL;
		SerializableGeeCollectionsTestPlanetCollection* _tmp172_ = NULL;
		gpointer _tmp173_ = NULL;
		SerializableGeeCollectionsTestPlanet* platon = NULL;
		SerializableGeeCollectionsTestPlanetCollection* _tmp180_ = NULL;
		SerializableGeeCollectionsTestPlanetCollection* _tmp181_ = NULL;
		gpointer _tmp182_ = NULL;
#line 222 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp0_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n" \
"<base name=\"AlphaOne\" >\n" \
"  <chargezone name=\"A1-1\">\n" \
"    <ship manufacturer=\"MacToy\" model=\"A1234\">\n" \
"      <space name=\"Alpha Centaury\">\n" \
"        <planet name=\"Earth\" />\n" \
"        <planet name=\"Platon\" />\n" \
"      </space>\n" \
"    </ship>\n" \
"  </chargezone>\n" \
"  <storage name=\"B4-A4\">\n" \
"    <refaction manufacturer=\"MacToy\" model=\"Fly045\">\n" \
"      <ship manufacturer=\"MacToy\" model=\"A1234\" />\n" \
"      <ship manufacturer=\"MegaTrench\" model=\"G045-1\" />\n" \
"    </refaction>\n" \
"  </storage>\n" \
"</base>", 0, &_inner_error_);
#line 222 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		doc = _tmp0_;
#line 222 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1020 "SerializableGeeCollectionsTest.c"
			goto __catch94_g_error;
		}
#line 239 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp1_ = serializable_gee_collections_test_space_base_new ();
#line 239 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		b = _tmp1_;
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp2_ = b;
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp3_ = doc;
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp2_, (GXmlNode*) _tmp3_, &_inner_error_);
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (b);
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (doc);
#line 1039 "SerializableGeeCollectionsTest.c"
			goto __catch94_g_error;
		}
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp4_ = b;
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp5_ = serializable_gee_collections_test_space_base_get_storage (_tmp4_);
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp6_ = _tmp5_;
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp6_ == NULL) {
#line 1050 "SerializableGeeCollectionsTest.c"
			FILE* _tmp7_ = NULL;
			GXmlGDocument* _tmp8_ = NULL;
			gchar* _tmp9_ = NULL;
			gchar* _tmp10_ = NULL;
			gchar* _tmp11_ = NULL;
			gchar* _tmp12_ = NULL;
#line 243 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp7_ = stdout;
#line 243 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp8_ = doc;
#line 243 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp9_ = gxml_gnode_to_string ((GXmlGNode*) _tmp8_);
#line 243 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp10_ = _tmp9_;
#line 243 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp11_ = g_strconcat ("ERROR: no storage exists:\n", _tmp10_, "\n", NULL);
#line 243 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp12_ = _tmp11_;
#line 243 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp7_, "%s", _tmp12_);
#line 243 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp12_);
#line 243 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp10_);
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 1077 "SerializableGeeCollectionsTest.c"
		}
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp13_ = b;
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp14_ = serializable_gee_collections_test_space_base_get_storage (_tmp13_);
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp15_ = _tmp14_;
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp16_ = _g_object_ref0 (_tmp15_);
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		storage = _tmp16_;
#line 247 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp17_ = serializable_gee_collections_test_storage_get_refactions (storage);
#line 247 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp18_ = _tmp17_;
#line 247 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp18_ == NULL) {
#line 1095 "SerializableGeeCollectionsTest.c"
			FILE* _tmp19_ = NULL;
			GXmlGDocument* _tmp20_ = NULL;
			gchar* _tmp21_ = NULL;
			gchar* _tmp22_ = NULL;
			gchar* _tmp23_ = NULL;
			gchar* _tmp24_ = NULL;
#line 248 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp19_ = stdout;
#line 248 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp20_ = doc;
#line 248 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp21_ = gxml_gnode_to_string ((GXmlGNode*) _tmp20_);
#line 248 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp22_ = _tmp21_;
#line 248 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp23_ = g_strconcat ("ERROR: no refactions exists:\n", _tmp22_, "\n", NULL);
#line 248 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp24_ = _tmp23_;
#line 248 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp19_, "%s", _tmp24_);
#line 248 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp24_);
#line 248 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp22_);
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 1122 "SerializableGeeCollectionsTest.c"
		}
#line 251 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp25_ = serializable_gee_collections_test_storage_get_refactions (storage);
#line 251 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp26_ = _tmp25_;
#line 251 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp27_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp26_);
#line 251 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp28_ = _tmp27_;
#line 251 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp28_ != 1) {
#line 1134 "SerializableGeeCollectionsTest.c"
			FILE* _tmp29_ = NULL;
			SerializableGeeCollectionsTestRefactionCollection* _tmp30_ = NULL;
			SerializableGeeCollectionsTestRefactionCollection* _tmp31_ = NULL;
			gint _tmp32_ = 0;
			gint _tmp33_ = 0;
			gchar* _tmp34_ = NULL;
			gchar* _tmp35_ = NULL;
			GXmlGDocument* _tmp36_ = NULL;
			gchar* _tmp37_ = NULL;
			gchar* _tmp38_ = NULL;
			gchar* _tmp39_ = NULL;
			gchar* _tmp40_ = NULL;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp29_ = stdout;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp30_ = serializable_gee_collections_test_storage_get_refactions (storage);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp31_ = _tmp30_;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp32_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp31_);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp33_ = _tmp32_;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp34_ = g_strdup_printf ("%i", _tmp33_);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp35_ = _tmp34_;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp36_ = doc;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp37_ = gxml_gnode_to_string ((GXmlGNode*) _tmp36_);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp38_ = _tmp37_;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp39_ = g_strconcat ("ERROR: Storage: incorrect size must be 1 got : ", _tmp35_, "\n", _tmp38_, "\n", NULL);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp40_ = _tmp39_;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp29_, "%s", _tmp40_);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp40_);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp38_);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp35_);
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 1181 "SerializableGeeCollectionsTest.c"
		}
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp41_ = serializable_gee_collections_test_storage_get_refactions (storage);
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp42_ = _tmp41_;
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp43_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp42_, "MacToy", "Fly045");
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		refaction = (SerializableGeeCollectionsTestRefaction*) _tmp43_;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (refaction == NULL) {
#line 1193 "SerializableGeeCollectionsTest.c"
			FILE* _tmp44_ = NULL;
			GXmlGDocument* _tmp45_ = NULL;
			gchar* _tmp46_ = NULL;
			gchar* _tmp47_ = NULL;
			gchar* _tmp48_ = NULL;
			gchar* _tmp49_ = NULL;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp44_ = stdout;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp45_ = doc;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp46_ = gxml_gnode_to_string ((GXmlGNode*) _tmp45_);
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp47_ = _tmp46_;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp48_ = g_strconcat ("ERROR: no refaction MacToy / Fly045 exists:\n", _tmp47_, "\n", NULL);
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp49_ = _tmp48_;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp44_, "%s", _tmp49_);
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp49_);
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp47_);
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 1220 "SerializableGeeCollectionsTest.c"
		}
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp50_ = serializable_gee_collections_test_refaction_get_spaceships (refaction);
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp51_ = _tmp50_;
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp52_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp51_);
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp53_ = _tmp52_;
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp53_ != 2) {
#line 1232 "SerializableGeeCollectionsTest.c"
			FILE* _tmp54_ = NULL;
			SerializableGeeCollectionsTestSpaceShipCollection* _tmp55_ = NULL;
			SerializableGeeCollectionsTestSpaceShipCollection* _tmp56_ = NULL;
			gint _tmp57_ = 0;
			gint _tmp58_ = 0;
			gchar* _tmp59_ = NULL;
			gchar* _tmp60_ = NULL;
			GXmlGDocument* _tmp61_ = NULL;
			gchar* _tmp62_ = NULL;
			gchar* _tmp63_ = NULL;
			gchar* _tmp64_ = NULL;
			gchar* _tmp65_ = NULL;
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp54_ = stdout;
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp55_ = serializable_gee_collections_test_refaction_get_spaceships (refaction);
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp56_ = _tmp55_;
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp57_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp56_);
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp58_ = _tmp57_;
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp59_ = g_strdup_printf ("%i", _tmp58_);
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp60_ = _tmp59_;
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp61_ = doc;
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp62_ = gxml_gnode_to_string ((GXmlGNode*) _tmp61_);
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp63_ = _tmp62_;
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp64_ = g_strconcat ("ERROR: Refaction: incorrect size must be 2 got : ", _tmp60_, "\n", _tmp63_, "\n", NULL);
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp65_ = _tmp64_;
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp54_, "%s", _tmp65_);
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp65_);
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp63_);
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp60_);
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 1279 "SerializableGeeCollectionsTest.c"
		}
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp66_ = serializable_gee_collections_test_refaction_get_spaceships (refaction);
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp67_ = _tmp66_;
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp68_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp67_, "MacToy", "A1234");
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		ship1 = (SerializableGeeCollectionsTestSpaceShip*) _tmp68_;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (ship1 == NULL) {
#line 1291 "SerializableGeeCollectionsTest.c"
			FILE* _tmp69_ = NULL;
			GXmlGDocument* _tmp70_ = NULL;
			gchar* _tmp71_ = NULL;
			gchar* _tmp72_ = NULL;
			gchar* _tmp73_ = NULL;
			gchar* _tmp74_ = NULL;
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp69_ = stdout;
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp70_ = doc;
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp71_ = gxml_gnode_to_string ((GXmlGNode*) _tmp70_);
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp72_ = _tmp71_;
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp73_ = g_strconcat ("ERROR: no refaction for Ship MacToy / A1234 exists:\n", _tmp72_, "\n", NULL);
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp74_ = _tmp73_;
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp69_, "%s", _tmp74_);
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp74_);
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp72_);
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 1318 "SerializableGeeCollectionsTest.c"
		}
#line 269 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp75_ = serializable_gee_collections_test_refaction_get_spaceships (refaction);
#line 269 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp76_ = _tmp75_;
#line 269 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp77_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp76_, "MegaTrench", "G045-1");
#line 269 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		ship2 = (SerializableGeeCollectionsTestSpaceShip*) _tmp77_;
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (ship2 == NULL) {
#line 1330 "SerializableGeeCollectionsTest.c"
			FILE* _tmp78_ = NULL;
			GXmlGDocument* _tmp79_ = NULL;
			gchar* _tmp80_ = NULL;
			gchar* _tmp81_ = NULL;
			gchar* _tmp82_ = NULL;
			gchar* _tmp83_ = NULL;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp78_ = stdout;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp79_ = doc;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp80_ = gxml_gnode_to_string ((GXmlGNode*) _tmp79_);
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp81_ = _tmp80_;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp82_ = g_strconcat ("ERROR: no refaction for Ship MegaTrench / G045-1 exists:\n", _tmp81_, "\n", NULL);
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp83_ = _tmp82_;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp78_, "%s", _tmp83_);
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp83_);
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp81_);
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 1357 "SerializableGeeCollectionsTest.c"
		}
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp84_ = b;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp85_ = serializable_gee_collections_test_space_base_get_charge_zone (_tmp84_);
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp86_ = _tmp85_;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp86_ == NULL) {
#line 1367 "SerializableGeeCollectionsTest.c"
			FILE* _tmp87_ = NULL;
			GXmlGDocument* _tmp88_ = NULL;
			gchar* _tmp89_ = NULL;
			gchar* _tmp90_ = NULL;
			gchar* _tmp91_ = NULL;
			gchar* _tmp92_ = NULL;
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp87_ = stdout;
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp88_ = doc;
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp89_ = gxml_gnode_to_string ((GXmlGNode*) _tmp88_);
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp90_ = _tmp89_;
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp91_ = g_strconcat ("ERROR: no charge_zone exists:\n", _tmp90_, "\n", NULL);
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp92_ = _tmp91_;
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp87_, "%s", _tmp92_);
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp92_);
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp90_);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 1394 "SerializableGeeCollectionsTest.c"
		}
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp93_ = b;
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp94_ = serializable_gee_collections_test_space_base_get_charge_zone (_tmp93_);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp95_ = _tmp94_;
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp96_ = _g_object_ref0 (_tmp95_);
#line 279 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		charge = _tmp96_;
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp97_ = serializable_gee_collections_test_charge_zone_get_spaceships (charge);
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp98_ = _tmp97_;
#line 280 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp98_ == NULL) {
#line 1412 "SerializableGeeCollectionsTest.c"
			FILE* _tmp99_ = NULL;
			GXmlGDocument* _tmp100_ = NULL;
			gchar* _tmp101_ = NULL;
			gchar* _tmp102_ = NULL;
			gchar* _tmp103_ = NULL;
			gchar* _tmp104_ = NULL;
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp99_ = stdout;
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp100_ = doc;
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp101_ = gxml_gnode_to_string ((GXmlGNode*) _tmp100_);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp102_ = _tmp101_;
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp103_ = g_strconcat ("ERROR: no spaceships exists:\n", _tmp102_, "\n", NULL);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp104_ = _tmp103_;
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp99_, "%s", _tmp104_);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp104_);
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp102_);
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 1439 "SerializableGeeCollectionsTest.c"
		}
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp105_ = serializable_gee_collections_test_charge_zone_get_spaceships (charge);
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp106_ = _tmp105_;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp107_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp106_);
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp108_ = _tmp107_;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp108_ != 1) {
#line 1451 "SerializableGeeCollectionsTest.c"
			FILE* _tmp109_ = NULL;
			SerializableGeeCollectionsTestSpaceShipCollection* _tmp110_ = NULL;
			SerializableGeeCollectionsTestSpaceShipCollection* _tmp111_ = NULL;
			gint _tmp112_ = 0;
			gint _tmp113_ = 0;
			gchar* _tmp114_ = NULL;
			gchar* _tmp115_ = NULL;
			GXmlGDocument* _tmp116_ = NULL;
			gchar* _tmp117_ = NULL;
			gchar* _tmp118_ = NULL;
			gchar* _tmp119_ = NULL;
			gchar* _tmp120_ = NULL;
#line 285 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp109_ = stdout;
#line 285 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp110_ = serializable_gee_collections_test_charge_zone_get_spaceships (charge);
#line 285 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp111_ = _tmp110_;
#line 285 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp112_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp111_);
#line 285 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp113_ = _tmp112_;
#line 285 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp114_ = g_strdup_printf ("%i", _tmp113_);
#line 285 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp115_ = _tmp114_;
#line 285 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp116_ = doc;
#line 285 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp117_ = gxml_gnode_to_string ((GXmlGNode*) _tmp116_);
#line 285 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp118_ = _tmp117_;
#line 285 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp119_ = g_strconcat ("ERROR: SpaceShips: incorrect size must be 1 got: ", _tmp115_, "\n", _tmp118_, "\n", NULL);
#line 285 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp120_ = _tmp119_;
#line 285 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp109_, "%s", _tmp120_);
#line 285 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp120_);
#line 285 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp118_);
#line 285 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp115_);
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 1498 "SerializableGeeCollectionsTest.c"
		}
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp121_ = serializable_gee_collections_test_charge_zone_get_spaceships (charge);
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp122_ = _tmp121_;
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp123_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp122_, "MacToy", "A1234");
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		ship = (SerializableGeeCollectionsTestSpaceShip*) _tmp123_;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (ship == NULL) {
#line 1510 "SerializableGeeCollectionsTest.c"
			FILE* _tmp124_ = NULL;
			GXmlGDocument* _tmp125_ = NULL;
			gchar* _tmp126_ = NULL;
			gchar* _tmp127_ = NULL;
			gchar* _tmp128_ = NULL;
			gchar* _tmp129_ = NULL;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp124_ = stdout;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp125_ = doc;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp126_ = gxml_gnode_to_string ((GXmlGNode*) _tmp125_);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp127_ = _tmp126_;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp128_ = g_strconcat ("ERROR: Ship MacToy/A1234 not found:\n", _tmp127_, "\n", NULL);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp129_ = _tmp128_;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp124_, "%s", _tmp129_);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp129_);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp127_);
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 1537 "SerializableGeeCollectionsTest.c"
		}
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp130_ = serializable_gee_collections_test_space_ship_get_spaces (ship);
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp131_ = _tmp130_;
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp132_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp131_);
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp133_ = _tmp132_;
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp133_ != 1) {
#line 1549 "SerializableGeeCollectionsTest.c"
			FILE* _tmp134_ = NULL;
			SerializableGeeCollectionsTestSpaceCollection* _tmp135_ = NULL;
			SerializableGeeCollectionsTestSpaceCollection* _tmp136_ = NULL;
			gint _tmp137_ = 0;
			gint _tmp138_ = 0;
			gchar* _tmp139_ = NULL;
			gchar* _tmp140_ = NULL;
			GXmlGDocument* _tmp141_ = NULL;
			gchar* _tmp142_ = NULL;
			gchar* _tmp143_ = NULL;
			gchar* _tmp144_ = NULL;
			gchar* _tmp145_ = NULL;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp134_ = stdout;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp135_ = serializable_gee_collections_test_space_ship_get_spaces (ship);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp136_ = _tmp135_;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp137_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp136_);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp138_ = _tmp137_;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp139_ = g_strdup_printf ("%i", _tmp138_);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp140_ = _tmp139_;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp141_ = doc;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp142_ = gxml_gnode_to_string ((GXmlGNode*) _tmp141_);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp143_ = _tmp142_;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp144_ = g_strconcat ("ERROR: Ship: incorrect size must be 1 got : ", _tmp140_, "\n", _tmp143_, "\n", NULL);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp145_ = _tmp144_;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp134_, "%s", _tmp145_);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp145_);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp143_);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp140_);
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 1596 "SerializableGeeCollectionsTest.c"
		}
#line 297 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp146_ = serializable_gee_collections_test_space_ship_get_spaces (ship);
#line 297 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp147_ = _tmp146_;
#line 297 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp148_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp147_, "Alpha Centaury");
#line 297 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		space = (SerializableGeeCollectionsTestSpace*) _tmp148_;
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (space == NULL) {
#line 1608 "SerializableGeeCollectionsTest.c"
			FILE* _tmp149_ = NULL;
			GXmlGDocument* _tmp150_ = NULL;
			gchar* _tmp151_ = NULL;
			gchar* _tmp152_ = NULL;
			gchar* _tmp153_ = NULL;
			gchar* _tmp154_ = NULL;
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp149_ = stdout;
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp150_ = doc;
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp151_ = gxml_gnode_to_string ((GXmlGNode*) _tmp150_);
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp152_ = _tmp151_;
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp153_ = g_strconcat ("ERROR: Space Alpha Centaury not found:\n", _tmp152_, "\n", NULL);
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp154_ = _tmp153_;
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp149_, "%s", _tmp154_);
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp154_);
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp152_);
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 1635 "SerializableGeeCollectionsTest.c"
		}
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp155_ = serializable_gee_collections_test_space_get_planets (space);
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp156_ = _tmp155_;
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp157_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp156_);
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp158_ = _tmp157_;
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp158_ != 2) {
#line 1647 "SerializableGeeCollectionsTest.c"
			FILE* _tmp159_ = NULL;
			SerializableGeeCollectionsTestPlanetCollection* _tmp160_ = NULL;
			SerializableGeeCollectionsTestPlanetCollection* _tmp161_ = NULL;
			gint _tmp162_ = 0;
			gint _tmp163_ = 0;
			gchar* _tmp164_ = NULL;
			gchar* _tmp165_ = NULL;
			GXmlGDocument* _tmp166_ = NULL;
			gchar* _tmp167_ = NULL;
			gchar* _tmp168_ = NULL;
			gchar* _tmp169_ = NULL;
			gchar* _tmp170_ = NULL;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp159_ = stdout;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp160_ = serializable_gee_collections_test_space_get_planets (space);
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp161_ = _tmp160_;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp162_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp161_);
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp163_ = _tmp162_;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp164_ = g_strdup_printf ("%i", _tmp163_);
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp165_ = _tmp164_;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp166_ = doc;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp167_ = gxml_gnode_to_string ((GXmlGNode*) _tmp166_);
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp168_ = _tmp167_;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp169_ = g_strconcat ("ERROR: Space size incorrect, must be 2 got:", _tmp165_, "\n", _tmp168_, "\n", NULL);
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp170_ = _tmp169_;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp159_, "%s", _tmp170_);
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp170_);
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp168_);
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp165_);
#line 304 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 1694 "SerializableGeeCollectionsTest.c"
		}
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp171_ = serializable_gee_collections_test_space_get_planets (space);
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp172_ = _tmp171_;
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp173_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp172_, "Earth");
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		earth = (SerializableGeeCollectionsTestPlanet*) _tmp173_;
#line 307 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (earth == NULL) {
#line 1706 "SerializableGeeCollectionsTest.c"
			FILE* _tmp174_ = NULL;
			GXmlGDocument* _tmp175_ = NULL;
			gchar* _tmp176_ = NULL;
			gchar* _tmp177_ = NULL;
			gchar* _tmp178_ = NULL;
			gchar* _tmp179_ = NULL;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp174_ = stdout;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp175_ = doc;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp176_ = gxml_gnode_to_string ((GXmlGNode*) _tmp175_);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp177_ = _tmp176_;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp178_ = g_strconcat ("ERROR: Planet Earth not found:\n", _tmp177_, "\n", NULL);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp179_ = _tmp178_;
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp174_, "%s", _tmp179_);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp179_);
#line 308 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp177_);
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 1733 "SerializableGeeCollectionsTest.c"
		}
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp180_ = serializable_gee_collections_test_space_get_planets (space);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp181_ = _tmp180_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp182_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp181_, "Platon");
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		platon = (SerializableGeeCollectionsTestPlanet*) _tmp182_;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (platon == NULL) {
#line 1745 "SerializableGeeCollectionsTest.c"
			FILE* _tmp183_ = NULL;
			GXmlGDocument* _tmp184_ = NULL;
			gchar* _tmp185_ = NULL;
			gchar* _tmp186_ = NULL;
			gchar* _tmp187_ = NULL;
			gchar* _tmp188_ = NULL;
#line 313 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp183_ = stdout;
#line 313 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp184_ = doc;
#line 313 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp185_ = gxml_gnode_to_string ((GXmlGNode*) _tmp184_);
#line 313 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp186_ = _tmp185_;
#line 313 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp187_ = g_strconcat ("ERROR: Planet Platon not found:\n", _tmp186_, "\n", NULL);
#line 313 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp188_ = _tmp187_;
#line 313 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp183_, "%s", _tmp188_);
#line 313 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp188_);
#line 313 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp186_);
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 1772 "SerializableGeeCollectionsTest.c"
		}
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (platon);
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (earth);
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (space);
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (ship);
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (charge);
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (ship2);
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (ship1);
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (refaction);
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (storage);
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (b);
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (doc);
#line 1796 "SerializableGeeCollectionsTest.c"
	}
	goto __finally94;
	__catch94_g_error:
	{
		GError* e = NULL;
		FILE* _tmp189_ = NULL;
		const gchar* _tmp190_ = NULL;
		const gchar* _tmp191_ = NULL;
		gchar* _tmp192_ = NULL;
		gchar* _tmp193_ = NULL;
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		e = _inner_error_;
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_inner_error_ = NULL;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp189_ = stdout;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp190_ = e->message;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp191_ = string_to_string (_tmp190_);
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp192_ = g_strconcat ("ERROR: ", _tmp191_, NULL);
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp193_ = _tmp192_;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		fprintf (_tmp189_, "%s", _tmp193_);
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_free0 (_tmp193_);
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_assert_not_reached ();
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_error_free0 (e);
#line 1829 "SerializableGeeCollectionsTest.c"
	}
	__finally94:
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_clear_error (&_inner_error_);
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		return;
#line 1840 "SerializableGeeCollectionsTest.c"
	}
}


static void ___lambda66__gtest_func (void) {
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	__lambda66_ ();
#line 1848 "SerializableGeeCollectionsTest.c"
}


static void __lambda67_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		SerializableGeeCollectionsTestSpaceBase* s = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp1_ = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp4_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp5_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp6_ = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp8_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp9_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp10_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp11_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp12_ = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp14_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp15_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp16_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp17_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp18_ = NULL;
		gint _tmp19_ = 0;
		gint _tmp20_ = 0;
		SerializableGeeCollectionsTestSpaceShip* s1 = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp33_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp34_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp35_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp36_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp37_ = NULL;
		gpointer _tmp38_ = NULL;
		SerializableGeeCollectionsTestSpaceShip* _tmp39_ = NULL;
		SerializableGeeCollectionsTestSpaceShip* _tmp41_ = NULL;
		SerializableGeeCollectionsTestSpaceCollection* _tmp42_ = NULL;
		SerializableGeeCollectionsTestSpaceCollection* _tmp43_ = NULL;
		SerializableGeeCollectionsTestSpaceShip* _tmp45_ = NULL;
		SerializableGeeCollectionsTestSpaceCollection* _tmp46_ = NULL;
		SerializableGeeCollectionsTestSpaceCollection* _tmp47_ = NULL;
		gint _tmp48_ = 0;
		gint _tmp49_ = 0;
		GXmlGDocument* ndoc = NULL;
		GXmlGDocument* _tmp60_ = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp61_ = NULL;
		GXmlGDocument* _tmp62_ = NULL;
		GXmlNode* _tmp63_ = NULL;
		GXmlNode* _tmp64_ = NULL;
		SerializableGeeCollectionsTestSpaceBase* ns = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp65_ = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp66_ = NULL;
		GXmlGDocument* _tmp67_ = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp68_ = NULL;
		gchar* _tmp69_ = NULL;
		gchar* _tmp70_ = NULL;
		gchar* _tmp71_ = NULL;
		gboolean _tmp72_ = FALSE;
		SerializableGeeCollectionsTestSpaceBase* _tmp81_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp82_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp83_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp84_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp85_ = NULL;
		gint _tmp86_ = 0;
		gint _tmp87_ = 0;
		SerializableGeeCollectionsTestSpaceShip* chip = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp124_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp125_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp126_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp127_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp128_ = NULL;
		gpointer _tmp129_ = NULL;
		SerializableGeeCollectionsTestSpaceCollection* _tmp130_ = NULL;
		SerializableGeeCollectionsTestSpaceCollection* _tmp131_ = NULL;
		gint _tmp132_ = 0;
		gint _tmp133_ = 0;
		GXmlGDocument* _tmp143_ = NULL;
		GXmlNode* _tmp144_ = NULL;
		GXmlNode* _tmp145_ = NULL;
		GXmlNode* _tmp146_ = NULL;
		gboolean _tmp147_ = FALSE;
		GXmlGDocument* _tmp149_ = NULL;
		GXmlNode* _tmp150_ = NULL;
		GXmlNode* _tmp151_ = NULL;
		GXmlNode* _tmp152_ = NULL;
		gchar* _tmp153_ = NULL;
		gchar* _tmp154_ = NULL;
		gchar* _tmp155_ = NULL;
		gboolean _tmp156_ = FALSE;
		GXmlGDocument* _tmp158_ = NULL;
		GXmlNode* _tmp159_ = NULL;
		GXmlNode* _tmp160_ = NULL;
		GXmlNode* _tmp161_ = NULL;
		GeeBidirList* _tmp162_ = NULL;
		GeeBidirList* _tmp163_ = NULL;
		GeeBidirList* _tmp164_ = NULL;
		gint _tmp165_ = 0;
		gint _tmp166_ = 0;
		gint i = 0;
		gint _tmp213_ = 0;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp0_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n" \
"<base name=\"AlphaOne\" >\n" \
"  <chargezone name=\"A1-1\">\n" \
"    <ship manufacturer=\"MacToy\" model=\"A1234\">\n" \
"      <space name=\"Alpha Centaury\">\n" \
"        <planet name=\"Earth\">\n" \
"          <citizen ctype=\"Human\">1M</citizen>\n" \
"          <citizen ctype=\"Ghost\">10M</citizen>\n" \
"        </planet>\n" \
"        <planet name=\"Platon\" />\n" \
"      </space>\n" \
"      <space name=\"Galax\">\n" \
"        <planet name=\"Saminon\">\n" \
"          <citizen ctype=\"Humanes\">100M</citizen>\n" \
"          <citizen ctype=\"Jeties\">1000M</citizen>\n" \
"        </planet>\n" \
"        <planet name=\"Platon\" />\n" \
"      </space>\n" \
"    </ship>\n" \
"    <ship manufacturer=\"Memphis\" model=\"AB1\">\n" \
"      <space name=\"Alpha Centaury\">\n" \
"        <planet name=\"Earth\">\n" \
"          <citizen ctype=\"Human\">10000M</citizen>\n" \
"          <citizen ctype=\"Ghost\">100000M</citizen>\n" \
"        </planet>\n" \
"        <planet name=\"Platon\" />\n" \
"      </space>\n" \
"    </ship>\n" \
"  </chargezone>\n" \
"  <storage name=\"B4-A4\">\n" \
"    <refaction manufacturer=\"MacToy\" model=\"Fly045\">\n" \
"      <ship manufacturer=\"MacToy\" model=\"A1234\" />\n" \
"      <ship manufacturer=\"MegaTrench\" model=\"G045-1\" />\n" \
"    </refaction>\n" \
"  </storage>\n" \
"</base>", 0, &_inner_error_);
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		doc = _tmp0_;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1955 "SerializableGeeCollectionsTest.c"
			goto __catch95_g_error;
		}
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp1_ = serializable_gee_collections_test_space_base_new ();
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		s = _tmp1_;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp2_ = s;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp3_ = doc;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp2_, (GXmlNode*) _tmp3_, &_inner_error_);
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (s);
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (doc);
#line 1974 "SerializableGeeCollectionsTest.c"
			goto __catch95_g_error;
		}
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp4_ = s;
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp5_ = serializable_gee_collections_test_space_base_get_charge_zone (_tmp4_);
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp6_ = _tmp5_;
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp6_ == NULL) {
#line 1985 "SerializableGeeCollectionsTest.c"
			FILE* _tmp7_ = NULL;
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp7_ = stdout;
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp7_, "%s", "ERROR: (1de) No charge zone\n");
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 1993 "SerializableGeeCollectionsTest.c"
		}
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp8_ = s;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp9_ = serializable_gee_collections_test_space_base_get_charge_zone (_tmp8_);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp10_ = _tmp9_;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp11_ = serializable_gee_collections_test_charge_zone_get_spaceships (_tmp10_);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp12_ = _tmp11_;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp12_ == NULL) {
#line 2007 "SerializableGeeCollectionsTest.c"
			FILE* _tmp13_ = NULL;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp13_ = stdout;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp13_, "%s", "ERROR: (1de) No spaceships\n");
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 2015 "SerializableGeeCollectionsTest.c"
		}
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp14_ = s;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp15_ = serializable_gee_collections_test_space_base_get_charge_zone (_tmp14_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp16_ = _tmp15_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp17_ = serializable_gee_collections_test_charge_zone_get_spaceships (_tmp16_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp18_ = _tmp17_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp19_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp18_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp20_ = _tmp19_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp20_ != 2) {
#line 2033 "SerializableGeeCollectionsTest.c"
			FILE* _tmp21_ = NULL;
			SerializableGeeCollectionsTestSpaceBase* _tmp22_ = NULL;
			SerializableGeeCollectionsTestChargeZone* _tmp23_ = NULL;
			SerializableGeeCollectionsTestChargeZone* _tmp24_ = NULL;
			SerializableGeeCollectionsTestSpaceShipCollection* _tmp25_ = NULL;
			SerializableGeeCollectionsTestSpaceShipCollection* _tmp26_ = NULL;
			gint _tmp27_ = 0;
			gint _tmp28_ = 0;
			gchar* _tmp29_ = NULL;
			gchar* _tmp30_ = NULL;
			gchar* _tmp31_ = NULL;
			gchar* _tmp32_ = NULL;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp21_ = stdout;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp22_ = s;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp23_ = serializable_gee_collections_test_space_base_get_charge_zone (_tmp22_);
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp24_ = _tmp23_;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp25_ = serializable_gee_collections_test_charge_zone_get_spaceships (_tmp24_);
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp26_ = _tmp25_;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp27_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp26_);
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp28_ = _tmp27_;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp29_ = g_strdup_printf ("%i", _tmp28_);
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp30_ = _tmp29_;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp31_ = g_strconcat ("ERROR: (1de) incorrect spaceships number. Expected 2, got ", _tmp30_, "\n", NULL);
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp32_ = _tmp31_;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp21_, "%s", _tmp32_);
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp32_);
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp30_);
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 2078 "SerializableGeeCollectionsTest.c"
		}
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp33_ = s;
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp34_ = serializable_gee_collections_test_space_base_get_charge_zone (_tmp33_);
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp35_ = _tmp34_;
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp36_ = serializable_gee_collections_test_charge_zone_get_spaceships (_tmp35_);
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp37_ = _tmp36_;
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp38_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp37_, "MacToy", "A1234");
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		s1 = (SerializableGeeCollectionsTestSpaceShip*) _tmp38_;
#line 376 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp39_ = s1;
#line 376 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp39_ == NULL) {
#line 2098 "SerializableGeeCollectionsTest.c"
			FILE* _tmp40_ = NULL;
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp40_ = stdout;
#line 377 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp40_, "%s", "ERROR: No spaceship MacToy/A1234 found\n");
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 2106 "SerializableGeeCollectionsTest.c"
		}
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp41_ = s1;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp42_ = serializable_gee_collections_test_space_ship_get_spaces (_tmp41_);
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp43_ = _tmp42_;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp43_ == NULL) {
#line 2116 "SerializableGeeCollectionsTest.c"
			FILE* _tmp44_ = NULL;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp44_ = stdout;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp44_, "%s", "ERROR: No spaces for spaceship MacToy/A1234 found\n");
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 2124 "SerializableGeeCollectionsTest.c"
		}
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp45_ = s1;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp46_ = serializable_gee_collections_test_space_ship_get_spaces (_tmp45_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp47_ = _tmp46_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp48_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp47_);
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp49_ = _tmp48_;
#line 384 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp49_ != 2) {
#line 2138 "SerializableGeeCollectionsTest.c"
			FILE* _tmp50_ = NULL;
			SerializableGeeCollectionsTestSpaceShip* _tmp51_ = NULL;
			SerializableGeeCollectionsTestSpaceCollection* _tmp52_ = NULL;
			SerializableGeeCollectionsTestSpaceCollection* _tmp53_ = NULL;
			gint _tmp54_ = 0;
			gint _tmp55_ = 0;
			gchar* _tmp56_ = NULL;
			gchar* _tmp57_ = NULL;
			gchar* _tmp58_ = NULL;
			gchar* _tmp59_ = NULL;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp50_ = stdout;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp51_ = s1;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp52_ = serializable_gee_collections_test_space_ship_get_spaces (_tmp51_);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp53_ = _tmp52_;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp54_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp53_);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp55_ = _tmp54_;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp56_ = g_strdup_printf ("%i", _tmp55_);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp57_ = _tmp56_;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp58_ = g_strconcat ("ERROR: Incorrect spaces size for spaceship MacToy/A1234. Expected 2, g" \
"ot: ", _tmp57_, "\n", NULL);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp59_ = _tmp58_;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp50_, "%s", _tmp59_);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp59_);
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp57_);
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 2177 "SerializableGeeCollectionsTest.c"
		}
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp60_ = gxml_gdocument_new ();
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		ndoc = _tmp60_;
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp61_ = s;
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp62_ = ndoc;
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp63_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) _tmp61_, (GXmlNode*) _tmp62_, &_inner_error_);
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp64_ = _tmp63_;
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp64_);
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (ndoc);
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (s1);
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (s);
#line 390 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (doc);
#line 2203 "SerializableGeeCollectionsTest.c"
			goto __catch95_g_error;
		}
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp65_ = serializable_gee_collections_test_space_base_new ();
#line 391 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		ns = _tmp65_;
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp66_ = ns;
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp67_ = ndoc;
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp66_, (GXmlNode*) _tmp67_, &_inner_error_);
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (ns);
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (ndoc);
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (s1);
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (s);
#line 392 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (doc);
#line 2228 "SerializableGeeCollectionsTest.c"
			goto __catch95_g_error;
		}
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp68_ = ns;
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp69_ = gxml_serializable_get_serialized_xml_node_value ((GXmlSerializable*) _tmp68_);
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp70_ = _tmp69_;
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp71_ = _tmp70_;
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp72_ = _tmp71_ != NULL;
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_free0 (_tmp71_);
#line 393 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp72_) {
#line 2245 "SerializableGeeCollectionsTest.c"
			FILE* _tmp73_ = NULL;
			SerializableGeeCollectionsTestSpaceBase* _tmp74_ = NULL;
			gchar* _tmp75_ = NULL;
			gchar* _tmp76_ = NULL;
			gchar* _tmp77_ = NULL;
			const gchar* _tmp78_ = NULL;
			gchar* _tmp79_ = NULL;
			gchar* _tmp80_ = NULL;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp73_ = stdout;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp74_ = ns;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp75_ = gxml_serializable_get_serialized_xml_node_value ((GXmlSerializable*) _tmp74_);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp76_ = _tmp75_;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp77_ = _tmp76_;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp78_ = string_to_string (_tmp77_);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp79_ = g_strconcat ("ERROR: wrong base node content, should be '' got: '", _tmp78_, "''", NULL);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp80_ = _tmp79_;
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp73_, "%s", _tmp80_);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp80_);
#line 394 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp77_);
#line 395 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 2278 "SerializableGeeCollectionsTest.c"
		}
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp81_ = ns;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp82_ = serializable_gee_collections_test_space_base_get_charge_zone (_tmp81_);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp83_ = _tmp82_;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp84_ = serializable_gee_collections_test_charge_zone_get_spaceships (_tmp83_);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp85_ = _tmp84_;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp86_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp85_);
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp87_ = _tmp86_;
#line 397 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp87_ != 2) {
#line 2296 "SerializableGeeCollectionsTest.c"
			FILE* _tmp88_ = NULL;
			SerializableGeeCollectionsTestSpaceBase* _tmp89_ = NULL;
			SerializableGeeCollectionsTestChargeZone* _tmp90_ = NULL;
			SerializableGeeCollectionsTestChargeZone* _tmp91_ = NULL;
			SerializableGeeCollectionsTestSpaceShipCollection* _tmp92_ = NULL;
			SerializableGeeCollectionsTestSpaceShipCollection* _tmp93_ = NULL;
			gint _tmp94_ = 0;
			gint _tmp95_ = 0;
			gchar* _tmp96_ = NULL;
			gchar* _tmp97_ = NULL;
			gchar* _tmp98_ = NULL;
			gchar* _tmp99_ = NULL;
			FILE* _tmp120_ = NULL;
			GXmlGDocument* _tmp121_ = NULL;
			gchar* _tmp122_ = NULL;
			gchar* _tmp123_ = NULL;
#line 398 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp88_ = stdout;
#line 398 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp89_ = ns;
#line 398 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp90_ = serializable_gee_collections_test_space_base_get_charge_zone (_tmp89_);
#line 398 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp91_ = _tmp90_;
#line 398 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp92_ = serializable_gee_collections_test_charge_zone_get_spaceships (_tmp91_);
#line 398 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp93_ = _tmp92_;
#line 398 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp94_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp93_);
#line 398 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp95_ = _tmp94_;
#line 398 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp96_ = g_strdup_printf ("%i", _tmp95_);
#line 398 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp97_ = _tmp96_;
#line 398 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp98_ = g_strconcat ("ERROR: wrong spaceships size. Expected 2, got: ", _tmp97_, "\n", NULL);
#line 398 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp99_ = _tmp98_;
#line 398 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp88_, "%s", _tmp99_);
#line 398 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp99_);
#line 398 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp97_);
#line 2343 "SerializableGeeCollectionsTest.c"
			{
				GeeIterator* _sh_it = NULL;
				SerializableGeeCollectionsTestSpaceBase* _tmp100_ = NULL;
				SerializableGeeCollectionsTestChargeZone* _tmp101_ = NULL;
				SerializableGeeCollectionsTestChargeZone* _tmp102_ = NULL;
				SerializableGeeCollectionsTestSpaceShipCollection* _tmp103_ = NULL;
				SerializableGeeCollectionsTestSpaceShipCollection* _tmp104_ = NULL;
				GeeCollection* _tmp105_ = NULL;
				GeeCollection* _tmp106_ = NULL;
				GeeIterator* _tmp107_ = NULL;
				GeeIterator* _tmp108_ = NULL;
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp100_ = ns;
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp101_ = serializable_gee_collections_test_space_base_get_charge_zone (_tmp100_);
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp102_ = _tmp101_;
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp103_ = serializable_gee_collections_test_charge_zone_get_spaceships (_tmp102_);
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp104_ = _tmp103_;
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp105_ = gxml_serializable_dual_key_map_values ((GXmlSerializableDualKeyMap*) _tmp104_);
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp106_ = _tmp105_;
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp107_ = gee_iterable_iterator ((GeeIterable*) _tmp106_);
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp108_ = _tmp107_;
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_g_object_unref0 (_tmp106_);
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_sh_it = _tmp108_;
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				while (TRUE) {
#line 2379 "SerializableGeeCollectionsTest.c"
					GeeIterator* _tmp109_ = NULL;
					gboolean _tmp110_ = FALSE;
					SerializableGeeCollectionsTestSpaceShip* sh = NULL;
					GeeIterator* _tmp111_ = NULL;
					gpointer _tmp112_ = NULL;
					FILE* _tmp113_ = NULL;
					SerializableGeeCollectionsTestSpaceShip* _tmp114_ = NULL;
					const gchar* _tmp115_ = NULL;
					const gchar* _tmp116_ = NULL;
					const gchar* _tmp117_ = NULL;
					gchar* _tmp118_ = NULL;
					gchar* _tmp119_ = NULL;
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp109_ = _sh_it;
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp110_ = gee_iterator_next (_tmp109_);
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					if (!_tmp110_) {
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						break;
#line 2400 "SerializableGeeCollectionsTest.c"
					}
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp111_ = _sh_it;
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp112_ = gee_iterator_get (_tmp111_);
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					sh = (SerializableGeeCollectionsTestSpaceShip*) _tmp112_;
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp113_ = stdout;
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp114_ = sh;
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp115_ = serializable_gee_collections_test_space_ship_get_manufacturer (_tmp114_);
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp116_ = _tmp115_;
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp117_ = string_to_string (_tmp116_);
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp118_ = g_strconcat ("Ship: ", _tmp117_, "\n", NULL);
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp119_ = _tmp118_;
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					fprintf (_tmp113_, "%s", _tmp119_);
#line 400 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_g_free0 (_tmp119_);
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_g_object_unref0 (sh);
#line 2428 "SerializableGeeCollectionsTest.c"
				}
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_g_object_unref0 (_sh_it);
#line 2432 "SerializableGeeCollectionsTest.c"
			}
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp120_ = stdout;
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp121_ = ndoc;
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp122_ = gxml_gnode_to_string ((GXmlGNode*) _tmp121_);
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp123_ = _tmp122_;
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp120_, "%s", _tmp123_);
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp123_);
#line 403 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 2448 "SerializableGeeCollectionsTest.c"
		}
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp124_ = ns;
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp125_ = serializable_gee_collections_test_space_base_get_charge_zone (_tmp124_);
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp126_ = _tmp125_;
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp127_ = serializable_gee_collections_test_charge_zone_get_spaceships (_tmp126_);
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp128_ = _tmp127_;
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp129_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp128_, "MacToy", "A1234");
#line 405 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		chip = (SerializableGeeCollectionsTestSpaceShip*) _tmp129_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp130_ = serializable_gee_collections_test_space_ship_get_spaces (chip);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp131_ = _tmp130_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp132_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp131_);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp133_ = _tmp132_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp133_ != 2) {
#line 2474 "SerializableGeeCollectionsTest.c"
			FILE* _tmp134_ = NULL;
			SerializableGeeCollectionsTestSpaceCollection* _tmp135_ = NULL;
			SerializableGeeCollectionsTestSpaceCollection* _tmp136_ = NULL;
			gint _tmp137_ = 0;
			gint _tmp138_ = 0;
			gchar* _tmp139_ = NULL;
			gchar* _tmp140_ = NULL;
			gchar* _tmp141_ = NULL;
			gchar* _tmp142_ = NULL;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp134_ = stdout;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp135_ = serializable_gee_collections_test_space_ship_get_spaces (chip);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp136_ = _tmp135_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp137_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp136_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp138_ = _tmp137_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp139_ = g_strdup_printf ("%i", _tmp138_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp140_ = _tmp139_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp141_ = g_strconcat ("ERROR: wrong spaces size. Expected 2, got: ", _tmp140_, NULL);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp142_ = _tmp141_;
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp134_, "%s", _tmp142_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp142_);
#line 407 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp140_);
#line 408 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 2510 "SerializableGeeCollectionsTest.c"
		}
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp143_ = ndoc;
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp144_ = gxml_document_get_root ((GXmlDocument*) _tmp143_);
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp145_ = _tmp144_;
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp146_ = _tmp145_;
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp147_ = _tmp146_ == NULL;
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp146_);
#line 411 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp147_) {
#line 2526 "SerializableGeeCollectionsTest.c"
			FILE* _tmp148_ = NULL;
#line 412 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp148_ = stdout;
#line 412 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp148_, "ERROR: No ROOT element\n");
#line 413 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 2534 "SerializableGeeCollectionsTest.c"
		}
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp149_ = ndoc;
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp150_ = gxml_document_get_root ((GXmlDocument*) _tmp149_);
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp151_ = _tmp150_;
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp152_ = _tmp151_;
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp153_ = gxml_node_get_name (_tmp152_);
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp154_ = _tmp153_;
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp155_ = _tmp154_;
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp156_ = g_strcmp0 (_tmp155_, "base") != 0;
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_free0 (_tmp155_);
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp152_);
#line 415 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp156_) {
#line 2558 "SerializableGeeCollectionsTest.c"
			FILE* _tmp157_ = NULL;
#line 416 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp157_ = stdout;
#line 416 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp157_, "ERROR: Bad ROOT name\n");
#line 417 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 2566 "SerializableGeeCollectionsTest.c"
		}
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp158_ = ndoc;
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp159_ = gxml_document_get_root ((GXmlDocument*) _tmp158_);
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp160_ = _tmp159_;
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp161_ = _tmp160_;
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp162_ = gxml_node_get_children_nodes (_tmp161_);
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp163_ = _tmp162_;
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp164_ = _tmp163_;
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp165_ = gee_collection_get_size ((GeeCollection*) _tmp164_);
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp166_ = _tmp165_;
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_vala_assert (_tmp166_ > 0, "ndoc.root.children_nodes.size > 0");
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp164_);
#line 419 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp161_);
#line 420 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		i = 0;
#line 2594 "SerializableGeeCollectionsTest.c"
		{
			GeeBidirList* _n_list = NULL;
			GXmlGDocument* _tmp167_ = NULL;
			GXmlNode* _tmp168_ = NULL;
			GXmlNode* _tmp169_ = NULL;
			GXmlNode* _tmp170_ = NULL;
			GeeBidirList* _tmp171_ = NULL;
			GeeBidirList* _tmp172_ = NULL;
			GeeBidirList* _tmp173_ = NULL;
			gint _n_size = 0;
			GeeBidirList* _tmp174_ = NULL;
			gint _tmp175_ = 0;
			gint _tmp176_ = 0;
			gint _n_index = 0;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp167_ = ndoc;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp168_ = gxml_document_get_root ((GXmlDocument*) _tmp167_);
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp169_ = _tmp168_;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp170_ = _tmp169_;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp171_ = gxml_node_get_children_nodes (_tmp170_);
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp172_ = _tmp171_;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp173_ = _tmp172_;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (_tmp170_);
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_n_list = _tmp173_;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp174_ = _n_list;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp175_ = gee_collection_get_size ((GeeCollection*) _tmp174_);
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp176_ = _tmp175_;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_n_size = _tmp176_;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_n_index = -1;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			while (TRUE) {
#line 2639 "SerializableGeeCollectionsTest.c"
				gint _tmp177_ = 0;
				gint _tmp178_ = 0;
				gint _tmp179_ = 0;
				GXmlNode* n = NULL;
				GeeBidirList* _tmp180_ = NULL;
				gint _tmp181_ = 0;
				gpointer _tmp182_ = NULL;
				gint _tmp183_ = 0;
				GXmlNode* _tmp184_ = NULL;
				GXmlNode* _tmp190_ = NULL;
				gchar* _tmp191_ = NULL;
				gchar* _tmp192_ = NULL;
				gchar* _tmp193_ = NULL;
				gboolean _tmp194_ = FALSE;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp177_ = _n_index;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_n_index = _tmp177_ + 1;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp178_ = _n_index;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp179_ = _n_size;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				if (!(_tmp178_ < _tmp179_)) {
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					break;
#line 2666 "SerializableGeeCollectionsTest.c"
				}
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp180_ = _n_list;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp181_ = _n_index;
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp182_ = gee_list_get ((GeeList*) _tmp180_, _tmp181_);
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				n = (GXmlNode*) _tmp182_;
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp183_ = i;
#line 423 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				i = _tmp183_ + 1;
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp184_ = n;
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp184_, GXML_TYPE_TEXT)) {
#line 2684 "SerializableGeeCollectionsTest.c"
					GXmlNode* _tmp185_ = NULL;
					gchar* _tmp186_ = NULL;
					gchar* _tmp187_ = NULL;
					gchar* _tmp188_ = NULL;
					gboolean _tmp189_ = FALSE;
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp185_ = n;
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp186_ = gxml_node_get_value (_tmp185_);
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp187_ = _tmp186_;
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp188_ = _tmp187_;
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp189_ = g_strcmp0 (_tmp188_, "") != 0;
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_g_free0 (_tmp188_);
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					if (_tmp189_) {
#line 424 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						g_assert_not_reached ();
#line 2706 "SerializableGeeCollectionsTest.c"
					}
				}
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp190_ = n;
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp191_ = gxml_node_get_name (_tmp190_);
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp192_ = _tmp191_;
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp193_ = _tmp192_;
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp194_ = g_strcmp0 (_tmp193_, "ChargeZone") == 0;
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_g_free0 (_tmp193_);
#line 425 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				if (_tmp194_) {
#line 2723 "SerializableGeeCollectionsTest.c"
					{
						GeeBidirList* _cn_list = NULL;
						GXmlNode* _tmp195_ = NULL;
						GeeBidirList* _tmp196_ = NULL;
						GeeBidirList* _tmp197_ = NULL;
						gint _cn_size = 0;
						GeeBidirList* _tmp198_ = NULL;
						gint _tmp199_ = 0;
						gint _tmp200_ = 0;
						gint _cn_index = 0;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						_tmp195_ = n;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						_tmp196_ = gxml_node_get_children_nodes (_tmp195_);
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						_tmp197_ = _tmp196_;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						_cn_list = _tmp197_;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						_tmp198_ = _cn_list;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						_tmp199_ = gee_collection_get_size ((GeeCollection*) _tmp198_);
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						_tmp200_ = _tmp199_;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						_cn_size = _tmp200_;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						_cn_index = -1;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						while (TRUE) {
#line 2754 "SerializableGeeCollectionsTest.c"
							gint _tmp201_ = 0;
							gint _tmp202_ = 0;
							gint _tmp203_ = 0;
							GXmlNode* cn = NULL;
							GeeBidirList* _tmp204_ = NULL;
							gint _tmp205_ = 0;
							gpointer _tmp206_ = NULL;
							GXmlNode* _tmp207_ = NULL;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_tmp201_ = _cn_index;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_cn_index = _tmp201_ + 1;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_tmp202_ = _cn_index;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_tmp203_ = _cn_size;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							if (!(_tmp202_ < _tmp203_)) {
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								break;
#line 2775 "SerializableGeeCollectionsTest.c"
							}
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_tmp204_ = _cn_list;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_tmp205_ = _cn_index;
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_tmp206_ = gee_list_get ((GeeList*) _tmp204_, _tmp205_);
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							cn = (GXmlNode*) _tmp206_;
#line 428 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_tmp207_ = cn;
#line 428 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp207_, GXML_TYPE_TEXT)) {
#line 2789 "SerializableGeeCollectionsTest.c"
								GXmlNode* _tmp208_ = NULL;
								gchar* _tmp209_ = NULL;
								gchar* _tmp210_ = NULL;
								gchar* _tmp211_ = NULL;
								gboolean _tmp212_ = FALSE;
#line 428 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_tmp208_ = cn;
#line 428 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_tmp209_ = gxml_node_get_value (_tmp208_);
#line 428 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_tmp210_ = _tmp209_;
#line 428 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_tmp211_ = _tmp210_;
#line 428 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_tmp212_ = g_strcmp0 (_tmp211_, "") != 0;
#line 428 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_g_free0 (_tmp211_);
#line 428 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								if (_tmp212_) {
#line 428 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
									g_assert_not_reached ();
#line 2811 "SerializableGeeCollectionsTest.c"
								}
							}
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_g_object_unref0 (cn);
#line 2816 "SerializableGeeCollectionsTest.c"
						}
#line 426 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						_g_object_unref0 (_cn_list);
#line 2820 "SerializableGeeCollectionsTest.c"
					}
				}
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_g_object_unref0 (n);
#line 2825 "SerializableGeeCollectionsTest.c"
			}
#line 421 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (_n_list);
#line 2829 "SerializableGeeCollectionsTest.c"
		}
#line 433 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp213_ = i;
#line 433 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp213_ != 2) {
#line 2835 "SerializableGeeCollectionsTest.c"
			FILE* _tmp214_ = NULL;
			gint _tmp215_ = 0;
			gchar* _tmp216_ = NULL;
			gchar* _tmp217_ = NULL;
			gchar* _tmp218_ = NULL;
			gchar* _tmp219_ = NULL;
#line 434 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp214_ = stdout;
#line 434 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp215_ = i;
#line 434 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp216_ = g_strdup_printf ("%i", _tmp215_);
#line 434 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp217_ = _tmp216_;
#line 434 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp218_ = g_strconcat ("ERROR: Bad ROOT child nodes. Expected 5, got ", _tmp217_, "\n", NULL);
#line 434 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp219_ = _tmp218_;
#line 434 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp214_, "%s", _tmp219_);
#line 434 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp219_);
#line 434 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp217_);
#line 435 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 2862 "SerializableGeeCollectionsTest.c"
		}
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (chip);
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (ns);
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (ndoc);
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (s1);
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (s);
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (doc);
#line 2876 "SerializableGeeCollectionsTest.c"
	}
	goto __finally95;
	__catch95_g_error:
	{
		GError* e = NULL;
		FILE* _tmp220_ = NULL;
		const gchar* _tmp221_ = NULL;
		const gchar* _tmp222_ = NULL;
		gchar* _tmp223_ = NULL;
		gchar* _tmp224_ = NULL;
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		e = _inner_error_;
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_inner_error_ = NULL;
#line 439 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp220_ = stdout;
#line 439 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp221_ = e->message;
#line 439 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp222_ = string_to_string (_tmp221_);
#line 439 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp223_ = g_strconcat ("ERROR: ", _tmp222_, NULL);
#line 439 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp224_ = _tmp223_;
#line 439 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		fprintf (_tmp220_, "%s", _tmp224_);
#line 439 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_free0 (_tmp224_);
#line 440 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_assert_not_reached ();
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_error_free0 (e);
#line 2909 "SerializableGeeCollectionsTest.c"
	}
	__finally95:
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_clear_error (&_inner_error_);
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		return;
#line 2920 "SerializableGeeCollectionsTest.c"
	}
}


static void ___lambda67__gtest_func (void) {
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	__lambda67_ ();
#line 2928 "SerializableGeeCollectionsTest.c"
}


static void __lambda68_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* org_doc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		SerializableGeeCollectionsTestSpaceBase* s = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp1_ = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp4_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp5_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp6_ = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp14_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp15_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp16_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp17_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp18_ = NULL;
		gint _tmp19_ = 0;
		gint _tmp20_ = 0;
		SerializableGeeCollectionsTestSpaceShip* mssh = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp33_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp34_ = NULL;
		SerializableGeeCollectionsTestChargeZone* _tmp35_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp36_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp37_ = NULL;
		gpointer _tmp38_ = NULL;
		SerializableGeeCollectionsTestSpaceShip* _tmp39_ = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp41_ = NULL;
		SerializableGeeCollectionsTestStorage* _tmp42_ = NULL;
		SerializableGeeCollectionsTestStorage* _tmp43_ = NULL;
		SerializableGeeCollectionsTestSpaceShip* _tmp45_ = NULL;
		SerializableGeeCollectionsTestSpaceCollection* _tmp46_ = NULL;
		SerializableGeeCollectionsTestSpaceCollection* _tmp47_ = NULL;
		gint _tmp48_ = 0;
		gint _tmp49_ = 0;
		SerializableGeeCollectionsTestSpaceBase* _tmp60_ = NULL;
		SerializableGeeCollectionsTestStorage* _tmp61_ = NULL;
		SerializableGeeCollectionsTestStorage* _tmp62_ = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp64_ = NULL;
		SerializableGeeCollectionsTestStorage* _tmp65_ = NULL;
		SerializableGeeCollectionsTestStorage* _tmp66_ = NULL;
		SerializableGeeCollectionsTestRefactionCollection* _tmp67_ = NULL;
		SerializableGeeCollectionsTestRefactionCollection* _tmp68_ = NULL;
		gint _tmp69_ = 0;
		gint _tmp70_ = 0;
		SerializableGeeCollectionsTestRefaction* refaction = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp83_ = NULL;
		SerializableGeeCollectionsTestStorage* _tmp84_ = NULL;
		SerializableGeeCollectionsTestStorage* _tmp85_ = NULL;
		SerializableGeeCollectionsTestRefactionCollection* _tmp86_ = NULL;
		SerializableGeeCollectionsTestRefactionCollection* _tmp87_ = NULL;
		gpointer _tmp88_ = NULL;
		SerializableGeeCollectionsTestRefaction* _tmp89_ = NULL;
		GeeMap* ups = NULL;
		SerializableGeeCollectionsTestRefaction* _tmp91_ = NULL;
		GeeMap* _tmp92_ = NULL;
		GeeMap* _tmp93_ = NULL;
		GeeMap* _tmp94_ = NULL;
		GeeMap* _tmp95_ = NULL;
		gint _tmp96_ = 0;
		gint _tmp97_ = 0;
		GeeCollection* uns = NULL;
		SerializableGeeCollectionsTestRefaction* _tmp98_ = NULL;
		GeeCollection* _tmp99_ = NULL;
		GeeCollection* _tmp100_ = NULL;
		GeeCollection* _tmp101_ = NULL;
		gint _tmp102_ = 0;
		gint _tmp103_ = 0;
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp104_ = NULL;
		SerializableGeeCollectionsTestSpaceBase* _tmp105_ = NULL;
		GXmlGDocument* _tmp106_ = NULL;
		GXmlNode* _tmp107_ = NULL;
		GXmlNode* _tmp108_ = NULL;
		GXmlGDocument* _tmp109_ = NULL;
		GXmlNode* _tmp110_ = NULL;
		GXmlNode* _tmp111_ = NULL;
		GXmlNode* _tmp112_ = NULL;
		gchar* _tmp113_ = NULL;
		gchar* _tmp114_ = NULL;
		gchar* _tmp115_ = NULL;
		gboolean _tmp116_ = FALSE;
#line 446 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp0_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n" \
"<base name=\"AlphaOne\" >\n" \
"  <chargezone name=\"A1-1\">\n" \
"    <ship manufacturer=\"MacToy\" model=\"A1234\">\n" \
"      <space name=\"Alpha Centaury\">\n" \
"        <planet name=\"Earth\">\n" \
"          <citizen ctype=\"Human\">1M</citizen>\n" \
"          <citizen ctype=\"Ghost\">10M</citizen>\n" \
"        </planet>\n" \
"        <planet name=\"Galiumt\" />\n" \
"      </space>\n" \
"      <space name=\"Galax\">\n" \
"        <planet name=\"Saminon\">\n" \
"          <citizen ctype=\"Humanes\">100M</citizen>\n" \
"          <citizen ctype=\"Jeties\">1000M</citizen>\n" \
"        </planet>\n" \
"        <planet name=\"Keymanth\" />\n" \
"      </space>\n" \
"    </ship>\n" \
"    <ship manufacturer=\"Memphis\" model=\"AB1\">\n" \
"      <space name=\"Proximity\">\n" \
"        <planet name=\"Pluton\">\n" \
"          <citizen ctype=\"Ork\">10000M</citizen>\n" \
"          <citizen ctype=\"Gyk\">100000M</citizen>\n" \
"        </planet>\n" \
"        <planet name=\"Mercury\" />\n" \
"      </space>\n" \
"    </ship>\n" \
"  </chargezone>\n" \
"  <storage name=\"B4-A4\">\n" \
"    <refaction manufacturer=\"MacToy\" model=\"Fly045\">\n" \
"      <ship manufacturer=\"MacToy\" model=\"A1234\" />\n" \
"      <ship manufacturer=\"MegaTrench\" model=\"G045-1\" unknown=\"UNK" \
"NOWN ATTR\">TEST_TEXT</ship>\n" \
"      <UnknownAttribute name=\"nothing\" />\n" \
"    </refaction>\n" \
"  </storage>\n" \
"</base>", 0, &_inner_error_);
#line 446 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		org_doc = _tmp0_;
#line 446 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 3020 "SerializableGeeCollectionsTest.c"
			goto __catch96_g_error;
		}
#line 483 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp1_ = serializable_gee_collections_test_space_base_new ();
#line 483 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		s = _tmp1_;
#line 484 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp2_ = s;
#line 484 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp3_ = org_doc;
#line 484 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp2_, (GXmlNode*) _tmp3_, &_inner_error_);
#line 484 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 484 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (s);
#line 484 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (org_doc);
#line 3039 "SerializableGeeCollectionsTest.c"
			goto __catch96_g_error;
		}
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp4_ = s;
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp5_ = serializable_gee_collections_test_space_base_get_charge_zone (_tmp4_);
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp6_ = _tmp5_;
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp6_ == NULL) {
#line 3050 "SerializableGeeCollectionsTest.c"
			FILE* _tmp7_ = NULL;
			SerializableGeeCollectionsTestSpaceBase* _tmp8_ = NULL;
			const gchar* _tmp9_ = NULL;
			const gchar* _tmp10_ = NULL;
			const gchar* _tmp11_ = NULL;
			gchar* _tmp12_ = NULL;
			gchar* _tmp13_ = NULL;
#line 486 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp7_ = stdout;
#line 486 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp8_ = s;
#line 486 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp9_ = serializable_gee_collections_test_space_base_get_name (_tmp8_);
#line 486 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp10_ = _tmp9_;
#line 486 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp11_ = string_to_string (_tmp10_);
#line 486 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp12_ = g_strconcat ("ERROR: No charge Zone for ", _tmp11_, "\n", NULL);
#line 486 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp13_ = _tmp12_;
#line 486 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp7_, "%s", _tmp13_);
#line 486 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp13_);
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 3078 "SerializableGeeCollectionsTest.c"
		}
#line 489 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp14_ = s;
#line 489 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp15_ = serializable_gee_collections_test_space_base_get_charge_zone (_tmp14_);
#line 489 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp16_ = _tmp15_;
#line 489 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp17_ = serializable_gee_collections_test_charge_zone_get_spaceships (_tmp16_);
#line 489 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp18_ = _tmp17_;
#line 489 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp19_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp18_);
#line 489 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp20_ = _tmp19_;
#line 489 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp20_ != 2) {
#line 3096 "SerializableGeeCollectionsTest.c"
			FILE* _tmp21_ = NULL;
			SerializableGeeCollectionsTestSpaceBase* _tmp22_ = NULL;
			SerializableGeeCollectionsTestChargeZone* _tmp23_ = NULL;
			SerializableGeeCollectionsTestChargeZone* _tmp24_ = NULL;
			SerializableGeeCollectionsTestSpaceShipCollection* _tmp25_ = NULL;
			SerializableGeeCollectionsTestSpaceShipCollection* _tmp26_ = NULL;
			gint _tmp27_ = 0;
			gint _tmp28_ = 0;
			gchar* _tmp29_ = NULL;
			gchar* _tmp30_ = NULL;
			gchar* _tmp31_ = NULL;
			gchar* _tmp32_ = NULL;
#line 490 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp21_ = stdout;
#line 490 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp22_ = s;
#line 490 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp23_ = serializable_gee_collections_test_space_base_get_charge_zone (_tmp22_);
#line 490 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp24_ = _tmp23_;
#line 490 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp25_ = serializable_gee_collections_test_charge_zone_get_spaceships (_tmp24_);
#line 490 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp26_ = _tmp25_;
#line 490 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp27_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp26_);
#line 490 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp28_ = _tmp27_;
#line 490 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp29_ = g_strdup_printf ("%i", _tmp28_);
#line 490 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp30_ = _tmp29_;
#line 490 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp31_ = g_strconcat ("ERROR: Bad SpaceShip size: ", _tmp30_, "\n", NULL);
#line 490 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp32_ = _tmp31_;
#line 490 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp21_, "%s", _tmp32_);
#line 490 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp32_);
#line 490 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp30_);
#line 491 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 3141 "SerializableGeeCollectionsTest.c"
		}
#line 493 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp33_ = s;
#line 493 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp34_ = serializable_gee_collections_test_space_base_get_charge_zone (_tmp33_);
#line 493 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp35_ = _tmp34_;
#line 493 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp36_ = serializable_gee_collections_test_charge_zone_get_spaceships (_tmp35_);
#line 493 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp37_ = _tmp36_;
#line 493 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp38_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp37_, "MacToy", "A1234");
#line 493 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		mssh = (SerializableGeeCollectionsTestSpaceShip*) _tmp38_;
#line 494 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp39_ = mssh;
#line 494 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp39_ == NULL) {
#line 3161 "SerializableGeeCollectionsTest.c"
			FILE* _tmp40_ = NULL;
#line 495 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp40_ = stdout;
#line 495 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp40_, "%s", "ERROR: No spaceship MacToy/A1234\n");
#line 496 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 3169 "SerializableGeeCollectionsTest.c"
		}
#line 499 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp41_ = s;
#line 499 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp42_ = serializable_gee_collections_test_space_base_get_storage (_tmp41_);
#line 499 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp43_ = _tmp42_;
#line 499 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp43_ == NULL) {
#line 3179 "SerializableGeeCollectionsTest.c"
			FILE* _tmp44_ = NULL;
#line 500 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp44_ = stdout;
#line 500 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp44_, "%s", "ERROR: No storage\n");
#line 501 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 3187 "SerializableGeeCollectionsTest.c"
		}
#line 503 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp45_ = mssh;
#line 503 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp46_ = serializable_gee_collections_test_space_ship_get_spaces (_tmp45_);
#line 503 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp47_ = _tmp46_;
#line 503 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp48_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp47_);
#line 503 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp49_ = _tmp48_;
#line 503 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp49_ != 2) {
#line 3201 "SerializableGeeCollectionsTest.c"
			FILE* _tmp50_ = NULL;
			SerializableGeeCollectionsTestSpaceShip* _tmp51_ = NULL;
			SerializableGeeCollectionsTestSpaceCollection* _tmp52_ = NULL;
			SerializableGeeCollectionsTestSpaceCollection* _tmp53_ = NULL;
			gint _tmp54_ = 0;
			gint _tmp55_ = 0;
			gchar* _tmp56_ = NULL;
			gchar* _tmp57_ = NULL;
			gchar* _tmp58_ = NULL;
			gchar* _tmp59_ = NULL;
#line 504 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp50_ = stdout;
#line 504 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp51_ = mssh;
#line 504 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp52_ = serializable_gee_collections_test_space_ship_get_spaces (_tmp51_);
#line 504 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp53_ = _tmp52_;
#line 504 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp54_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp53_);
#line 504 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp55_ = _tmp54_;
#line 504 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp56_ = g_strdup_printf ("%i", _tmp55_);
#line 504 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp57_ = _tmp56_;
#line 504 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp58_ = g_strconcat ("ERROR: Bad spaces number for MacToy/A1234: ", _tmp57_, "\n", NULL);
#line 504 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp59_ = _tmp58_;
#line 504 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp50_, "%s", _tmp59_);
#line 504 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp59_);
#line 504 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp57_);
#line 505 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 3240 "SerializableGeeCollectionsTest.c"
		}
#line 507 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp60_ = s;
#line 507 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp61_ = serializable_gee_collections_test_space_base_get_storage (_tmp60_);
#line 507 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp62_ = _tmp61_;
#line 507 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp62_ == NULL) {
#line 3250 "SerializableGeeCollectionsTest.c"
			FILE* _tmp63_ = NULL;
#line 508 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp63_ = stdout;
#line 508 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp63_, "%s", "ERROR: No storage\n");
#line 509 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 3258 "SerializableGeeCollectionsTest.c"
		}
#line 511 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp64_ = s;
#line 511 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp65_ = serializable_gee_collections_test_space_base_get_storage (_tmp64_);
#line 511 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp66_ = _tmp65_;
#line 511 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp67_ = serializable_gee_collections_test_storage_get_refactions (_tmp66_);
#line 511 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp68_ = _tmp67_;
#line 511 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp69_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp68_);
#line 511 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp70_ = _tmp69_;
#line 511 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp70_ != 1) {
#line 3276 "SerializableGeeCollectionsTest.c"
			FILE* _tmp71_ = NULL;
			SerializableGeeCollectionsTestSpaceBase* _tmp72_ = NULL;
			SerializableGeeCollectionsTestStorage* _tmp73_ = NULL;
			SerializableGeeCollectionsTestStorage* _tmp74_ = NULL;
			SerializableGeeCollectionsTestRefactionCollection* _tmp75_ = NULL;
			SerializableGeeCollectionsTestRefactionCollection* _tmp76_ = NULL;
			gint _tmp77_ = 0;
			gint _tmp78_ = 0;
			gchar* _tmp79_ = NULL;
			gchar* _tmp80_ = NULL;
			gchar* _tmp81_ = NULL;
			gchar* _tmp82_ = NULL;
#line 512 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp71_ = stdout;
#line 512 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp72_ = s;
#line 512 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp73_ = serializable_gee_collections_test_space_base_get_storage (_tmp72_);
#line 512 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp74_ = _tmp73_;
#line 512 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp75_ = serializable_gee_collections_test_storage_get_refactions (_tmp74_);
#line 512 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp76_ = _tmp75_;
#line 512 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp77_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp76_);
#line 512 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp78_ = _tmp77_;
#line 512 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp79_ = g_strdup_printf ("%i", _tmp78_);
#line 512 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp80_ = _tmp79_;
#line 512 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp81_ = g_strconcat ("ERROR: Bad number of refactions: got ", _tmp80_, "\n", NULL);
#line 512 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp82_ = _tmp81_;
#line 512 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp71_, "%s", _tmp82_);
#line 512 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp82_);
#line 512 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_free0 (_tmp80_);
#line 513 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 3321 "SerializableGeeCollectionsTest.c"
		}
#line 515 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp83_ = s;
#line 515 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp84_ = serializable_gee_collections_test_space_base_get_storage (_tmp83_);
#line 515 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp85_ = _tmp84_;
#line 515 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp86_ = serializable_gee_collections_test_storage_get_refactions (_tmp85_);
#line 515 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp87_ = _tmp86_;
#line 515 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp88_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp87_, "MacToy", "Fly045");
#line 515 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		refaction = (SerializableGeeCollectionsTestRefaction*) _tmp88_;
#line 516 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp89_ = refaction;
#line 516 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp89_ == NULL) {
#line 3341 "SerializableGeeCollectionsTest.c"
			FILE* _tmp90_ = NULL;
#line 517 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp90_ = stdout;
#line 517 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp90_, "%s", "ERROR: No Refaction MacToy/Fly045 found!\n");
#line 518 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 3349 "SerializableGeeCollectionsTest.c"
		}
#line 520 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp91_ = refaction;
#line 520 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp92_ = gxml_serializable_get_unknown_serializable_properties ((GXmlSerializable*) _tmp91_);
#line 520 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp93_ = _tmp92_;
#line 520 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		ups = _tmp93_;
#line 521 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp94_ = ups;
#line 521 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_vala_assert (_tmp94_ != NULL, "ups != null");
#line 522 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp95_ = ups;
#line 522 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp96_ = gee_map_get_size (_tmp95_);
#line 522 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp97_ = _tmp96_;
#line 522 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_vala_assert (_tmp97_ == 0, "ups.size == 0");
#line 523 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp98_ = refaction;
#line 523 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp99_ = gxml_serializable_get_unknown_serializable_nodes ((GXmlSerializable*) _tmp98_);
#line 523 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp100_ = _tmp99_;
#line 523 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		uns = _tmp100_;
#line 530 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp101_ = uns;
#line 530 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp102_ = gee_collection_get_size (_tmp101_);
#line 530 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp103_ = _tmp102_;
#line 530 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_vala_assert (_tmp103_ == 5, "uns.size == 5");
#line 531 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp104_ = gxml_gdocument_new ();
#line 531 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		doc = _tmp104_;
#line 532 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp105_ = s;
#line 532 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp106_ = doc;
#line 532 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp107_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) _tmp105_, (GXmlNode*) _tmp106_, &_inner_error_);
#line 532 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp108_ = _tmp107_;
#line 532 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp108_);
#line 532 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 532 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (doc);
#line 532 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (uns);
#line 532 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (ups);
#line 532 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (refaction);
#line 532 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (mssh);
#line 532 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (s);
#line 532 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (org_doc);
#line 3417 "SerializableGeeCollectionsTest.c"
			goto __catch96_g_error;
		}
#line 533 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp109_ = doc;
#line 533 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp110_ = gxml_document_get_root ((GXmlDocument*) _tmp109_);
#line 533 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp111_ = _tmp110_;
#line 533 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp112_ = _tmp111_;
#line 533 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp113_ = gxml_node_get_name (_tmp112_);
#line 533 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp114_ = _tmp113_;
#line 533 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp115_ = _tmp114_;
#line 533 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp116_ = g_strcmp0 (_tmp115_, "base") != 0;
#line 533 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_free0 (_tmp115_);
#line 533 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp112_);
#line 533 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (_tmp116_) {
#line 3442 "SerializableGeeCollectionsTest.c"
			FILE* _tmp117_ = NULL;
#line 534 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp117_ = stdout;
#line 534 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			fprintf (_tmp117_, "%s", "ERROR: bad root node name\n");
#line 535 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			g_assert_not_reached ();
#line 3450 "SerializableGeeCollectionsTest.c"
		}
		{
			GeeBidirList* _n_list = NULL;
			GXmlGDocument* _tmp118_ = NULL;
			GXmlNode* _tmp119_ = NULL;
			GXmlNode* _tmp120_ = NULL;
			GXmlNode* _tmp121_ = NULL;
			GeeBidirList* _tmp122_ = NULL;
			GeeBidirList* _tmp123_ = NULL;
			GeeBidirList* _tmp124_ = NULL;
			gint _n_size = 0;
			GeeBidirList* _tmp125_ = NULL;
			gint _tmp126_ = 0;
			gint _tmp127_ = 0;
			gint _n_index = 0;
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp118_ = doc;
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp119_ = gxml_document_get_root ((GXmlDocument*) _tmp118_);
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp120_ = _tmp119_;
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp121_ = _tmp120_;
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp122_ = gxml_node_get_children_nodes (_tmp121_);
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp123_ = _tmp122_;
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp124_ = _tmp123_;
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (_tmp121_);
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_n_list = _tmp124_;
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp125_ = _n_list;
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp126_ = gee_collection_get_size ((GeeCollection*) _tmp125_);
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_tmp127_ = _tmp126_;
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_n_size = _tmp127_;
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_n_index = -1;
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			while (TRUE) {
#line 3496 "SerializableGeeCollectionsTest.c"
				gint _tmp128_ = 0;
				gint _tmp129_ = 0;
				gint _tmp130_ = 0;
				GXmlNode* n = NULL;
				GeeBidirList* _tmp131_ = NULL;
				gint _tmp132_ = 0;
				gpointer _tmp133_ = NULL;
				GXmlNode* _tmp134_ = NULL;
				GXmlNode* _tmp260_ = NULL;
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp128_ = _n_index;
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_n_index = _tmp128_ + 1;
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp129_ = _n_index;
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp130_ = _n_size;
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				if (!(_tmp129_ < _tmp130_)) {
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					break;
#line 3518 "SerializableGeeCollectionsTest.c"
				}
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp131_ = _n_list;
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp132_ = _n_index;
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp133_ = gee_list_get ((GeeList*) _tmp131_, _tmp132_);
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				n = (GXmlNode*) _tmp133_;
#line 539 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp134_ = n;
#line 539 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp134_, GXML_TYPE_GELEMENT)) {
#line 3532 "SerializableGeeCollectionsTest.c"
					GXmlNode* _tmp135_ = NULL;
					gchar* _tmp136_ = NULL;
					gchar* _tmp137_ = NULL;
					gchar* _tmp138_ = NULL;
					gboolean _tmp139_ = FALSE;
					GXmlNode* _tmp140_ = NULL;
					gchar* _tmp141_ = NULL;
					gchar* _tmp142_ = NULL;
					gchar* _tmp143_ = NULL;
					gboolean _tmp144_ = FALSE;
#line 540 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp135_ = n;
#line 540 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp136_ = gxml_node_get_name (_tmp135_);
#line 540 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp137_ = _tmp136_;
#line 540 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp138_ = _tmp137_;
#line 540 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp139_ = g_strcmp0 (_tmp138_, "ChargeZone") == 0;
#line 540 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_g_free0 (_tmp138_);
#line 540 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					if (_tmp139_) {
#line 3557 "SerializableGeeCollectionsTest.c"
					}
#line 543 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp140_ = n;
#line 543 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp141_ = gxml_node_get_name (_tmp140_);
#line 543 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp142_ = _tmp141_;
#line 543 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp143_ = _tmp142_;
#line 543 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp144_ = g_strcmp0 (_tmp143_, "storage") == 0;
#line 543 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_g_free0 (_tmp143_);
#line 543 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					if (_tmp144_) {
#line 3573 "SerializableGeeCollectionsTest.c"
						gboolean unkfound = FALSE;
						gboolean tfound = FALSE;
						gboolean attrfound = FALSE;
						gboolean _tmp254_ = FALSE;
						gboolean _tmp256_ = FALSE;
						gboolean _tmp258_ = FALSE;
#line 544 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						unkfound = FALSE;
#line 545 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						tfound = FALSE;
#line 546 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						attrfound = FALSE;
#line 3586 "SerializableGeeCollectionsTest.c"
						{
							GeeBidirList* _sn_list = NULL;
							GXmlNode* _tmp145_ = NULL;
							GeeBidirList* _tmp146_ = NULL;
							GeeBidirList* _tmp147_ = NULL;
							gint _sn_size = 0;
							GeeBidirList* _tmp148_ = NULL;
							gint _tmp149_ = 0;
							gint _tmp150_ = 0;
							gint _sn_index = 0;
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_tmp145_ = n;
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_tmp146_ = gxml_node_get_children_nodes (_tmp145_);
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_tmp147_ = _tmp146_;
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_sn_list = _tmp147_;
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_tmp148_ = _sn_list;
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_tmp149_ = gee_collection_get_size ((GeeCollection*) _tmp148_);
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_tmp150_ = _tmp149_;
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_sn_size = _tmp150_;
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_sn_index = -1;
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							while (TRUE) {
#line 3617 "SerializableGeeCollectionsTest.c"
								gint _tmp151_ = 0;
								gint _tmp152_ = 0;
								gint _tmp153_ = 0;
								GXmlNode* sn = NULL;
								GeeBidirList* _tmp154_ = NULL;
								gint _tmp155_ = 0;
								gpointer _tmp156_ = NULL;
								GXmlNode* _tmp157_ = NULL;
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_tmp151_ = _sn_index;
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_sn_index = _tmp151_ + 1;
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_tmp152_ = _sn_index;
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_tmp153_ = _sn_size;
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								if (!(_tmp152_ < _tmp153_)) {
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
									break;
#line 3638 "SerializableGeeCollectionsTest.c"
								}
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_tmp154_ = _sn_list;
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_tmp155_ = _sn_index;
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_tmp156_ = gee_list_get ((GeeList*) _tmp154_, _tmp155_);
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								sn = (GXmlNode*) _tmp156_;
#line 548 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_tmp157_ = sn;
#line 548 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp157_, GXML_TYPE_GELEMENT)) {
#line 3652 "SerializableGeeCollectionsTest.c"
									GXmlNode* _tmp158_ = NULL;
									gchar* _tmp159_ = NULL;
									gchar* _tmp160_ = NULL;
									gchar* _tmp161_ = NULL;
									gboolean _tmp162_ = FALSE;
#line 549 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
									_tmp158_ = sn;
#line 549 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
									_tmp159_ = gxml_node_get_name (_tmp158_);
#line 549 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
									_tmp160_ = _tmp159_;
#line 549 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
									_tmp161_ = _tmp160_;
#line 549 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
									_tmp162_ = g_strcmp0 (_tmp161_, "refaction") == 0;
#line 549 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
									_g_free0 (_tmp161_);
#line 549 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
									if (_tmp162_) {
#line 3672 "SerializableGeeCollectionsTest.c"
										{
											GeeBidirList* _rn_list = NULL;
											GXmlNode* _tmp163_ = NULL;
											GeeBidirList* _tmp164_ = NULL;
											GeeBidirList* _tmp165_ = NULL;
											gint _rn_size = 0;
											GeeBidirList* _tmp166_ = NULL;
											gint _tmp167_ = 0;
											gint _tmp168_ = 0;
											gint _rn_index = 0;
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
											_tmp163_ = sn;
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
											_tmp164_ = gxml_node_get_children_nodes (_tmp163_);
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
											_tmp165_ = _tmp164_;
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
											_rn_list = _tmp165_;
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
											_tmp166_ = _rn_list;
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
											_tmp167_ = gee_collection_get_size ((GeeCollection*) _tmp166_);
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
											_tmp168_ = _tmp167_;
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
											_rn_size = _tmp168_;
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
											_rn_index = -1;
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
											while (TRUE) {
#line 3703 "SerializableGeeCollectionsTest.c"
												gint _tmp169_ = 0;
												gint _tmp170_ = 0;
												gint _tmp171_ = 0;
												GXmlNode* rn = NULL;
												GeeBidirList* _tmp172_ = NULL;
												gint _tmp173_ = 0;
												gpointer _tmp174_ = NULL;
												GXmlNode* _tmp175_ = NULL;
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
												_tmp169_ = _rn_index;
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
												_rn_index = _tmp169_ + 1;
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
												_tmp170_ = _rn_index;
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
												_tmp171_ = _rn_size;
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
												if (!(_tmp170_ < _tmp171_)) {
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
													break;
#line 3724 "SerializableGeeCollectionsTest.c"
												}
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
												_tmp172_ = _rn_list;
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
												_tmp173_ = _rn_index;
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
												_tmp174_ = gee_list_get ((GeeList*) _tmp172_, _tmp173_);
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
												rn = (GXmlNode*) _tmp174_;
#line 551 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
												_tmp175_ = rn;
#line 551 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
												if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp175_, GXML_TYPE_GELEMENT)) {
#line 3738 "SerializableGeeCollectionsTest.c"
													GXmlNode* _tmp176_ = NULL;
													gchar* _tmp177_ = NULL;
													gchar* _tmp178_ = NULL;
													gchar* _tmp179_ = NULL;
													gboolean _tmp180_ = FALSE;
													GXmlNode* _tmp232_ = NULL;
													gchar* _tmp233_ = NULL;
													gchar* _tmp234_ = NULL;
													gchar* _tmp235_ = NULL;
													gboolean _tmp236_ = FALSE;
#line 553 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
													_tmp176_ = rn;
#line 553 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
													_tmp177_ = gxml_node_get_name (_tmp176_);
#line 553 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
													_tmp178_ = _tmp177_;
#line 553 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
													_tmp179_ = _tmp178_;
#line 553 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
													_tmp180_ = g_strcmp0 (_tmp179_, "ship") == 0;
#line 553 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
													_g_free0 (_tmp179_);
#line 553 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
													if (_tmp180_) {
#line 3763 "SerializableGeeCollectionsTest.c"
														GXmlNode* atr = NULL;
														GXmlNode* _tmp181_ = NULL;
														GXmlNode* _tmp182_ = NULL;
														GXmlNode* _tmp183_ = NULL;
														GXmlNode* _tmp185_ = NULL;
														gchar* _tmp186_ = NULL;
														gchar* _tmp187_ = NULL;
														gchar* _tmp188_ = NULL;
														gboolean _tmp189_ = FALSE;
#line 554 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														_tmp181_ = rn;
#line 554 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														_tmp182_ = gxml_element_get_attr ((GXmlElement*) G_TYPE_CHECK_INSTANCE_CAST (_tmp181_, GXML_TYPE_GELEMENT, GXmlGElement), "manufacturer");
#line 554 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														atr = _tmp182_;
#line 555 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														_tmp183_ = atr;
#line 555 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														if (_tmp183_ == NULL) {
#line 3783 "SerializableGeeCollectionsTest.c"
															FILE* _tmp184_ = NULL;
#line 556 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															_tmp184_ = stdout;
#line 556 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															fprintf (_tmp184_, "%s", "ERROR: No attribute manufacturer for Ship\n");
#line 557 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															g_assert_not_reached ();
#line 3791 "SerializableGeeCollectionsTest.c"
														}
#line 559 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														_tmp185_ = atr;
#line 559 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														_tmp186_ = gxml_node_get_value (_tmp185_);
#line 559 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														_tmp187_ = _tmp186_;
#line 559 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														_tmp188_ = _tmp187_;
#line 559 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														_tmp189_ = g_strcmp0 (_tmp188_, "MegaTrench") == 0;
#line 559 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														_g_free0 (_tmp188_);
#line 559 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														if (_tmp189_) {
#line 3807 "SerializableGeeCollectionsTest.c"
															GXmlNode* shanattr = NULL;
															GXmlNode* _tmp190_ = NULL;
															GXmlNode* _tmp191_ = NULL;
															GXmlNode* _tmp192_ = NULL;
#line 560 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															_tmp190_ = rn;
#line 560 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															_tmp191_ = gxml_element_get_attr ((GXmlElement*) G_TYPE_CHECK_INSTANCE_CAST (_tmp190_, GXML_TYPE_GELEMENT, GXmlGElement), "unknown");
#line 560 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															shanattr = _tmp191_;
#line 561 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															_tmp192_ = shanattr;
#line 561 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															if (_tmp192_ != NULL) {
#line 3822 "SerializableGeeCollectionsTest.c"
																GXmlNode* _tmp193_ = NULL;
																gchar* _tmp194_ = NULL;
																gchar* _tmp195_ = NULL;
																gchar* _tmp196_ = NULL;
																gboolean _tmp197_ = FALSE;
#line 562 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																attrfound = TRUE;
#line 563 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																_tmp193_ = shanattr;
#line 563 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																_tmp194_ = gxml_node_get_value (_tmp193_);
#line 563 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																_tmp195_ = _tmp194_;
#line 563 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																_tmp196_ = _tmp195_;
#line 563 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																_tmp197_ = g_strcmp0 (_tmp196_, "UNKNOWN ATTR") != 0;
#line 563 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																_g_free0 (_tmp196_);
#line 563 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																if (_tmp197_) {
#line 3844 "SerializableGeeCollectionsTest.c"
																	FILE* _tmp198_ = NULL;
																	GXmlNode* _tmp199_ = NULL;
																	gchar* _tmp200_ = NULL;
																	gchar* _tmp201_ = NULL;
																	gchar* _tmp202_ = NULL;
																	const gchar* _tmp203_ = NULL;
																	gchar* _tmp204_ = NULL;
																	gchar* _tmp205_ = NULL;
#line 564 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	_tmp198_ = stdout;
#line 564 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	_tmp199_ = shanattr;
#line 564 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	_tmp200_ = gxml_node_get_value (_tmp199_);
#line 564 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	_tmp201_ = _tmp200_;
#line 564 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	_tmp202_ = _tmp201_;
#line 564 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	_tmp203_ = string_to_string (_tmp202_);
#line 564 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	_tmp204_ = g_strconcat ("ERROR: Invalid Text Node Value for ship MegaTrench: ", _tmp203_, "\n", NULL);
#line 564 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	_tmp205_ = _tmp204_;
#line 564 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	fprintf (_tmp198_, "%s", _tmp205_);
#line 564 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	_g_free0 (_tmp205_);
#line 564 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	_g_free0 (_tmp202_);
#line 565 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	g_assert_not_reached ();
#line 3877 "SerializableGeeCollectionsTest.c"
																}
															}
															{
																GeeBidirList* _shn_list = NULL;
																GXmlNode* _tmp206_ = NULL;
																GeeBidirList* _tmp207_ = NULL;
																GeeBidirList* _tmp208_ = NULL;
																gint _shn_size = 0;
																GeeBidirList* _tmp209_ = NULL;
																gint _tmp210_ = 0;
																gint _tmp211_ = 0;
																gint _shn_index = 0;
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																_tmp206_ = rn;
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																_tmp207_ = gxml_node_get_children_nodes (_tmp206_);
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																_tmp208_ = _tmp207_;
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																_shn_list = _tmp208_;
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																_tmp209_ = _shn_list;
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																_tmp210_ = gee_collection_get_size ((GeeCollection*) _tmp209_);
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																_tmp211_ = _tmp210_;
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																_shn_size = _tmp211_;
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																_shn_index = -1;
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																while (TRUE) {
#line 3910 "SerializableGeeCollectionsTest.c"
																	gint _tmp212_ = 0;
																	gint _tmp213_ = 0;
																	gint _tmp214_ = 0;
																	GXmlNode* shn = NULL;
																	GeeBidirList* _tmp215_ = NULL;
																	gint _tmp216_ = 0;
																	gpointer _tmp217_ = NULL;
																	GXmlNode* _tmp218_ = NULL;
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	_tmp212_ = _shn_index;
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	_shn_index = _tmp212_ + 1;
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	_tmp213_ = _shn_index;
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	_tmp214_ = _shn_size;
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	if (!(_tmp213_ < _tmp214_)) {
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																		break;
#line 3931 "SerializableGeeCollectionsTest.c"
																	}
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	_tmp215_ = _shn_list;
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	_tmp216_ = _shn_index;
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	_tmp217_ = gee_list_get ((GeeList*) _tmp215_, _tmp216_);
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	shn = (GXmlNode*) _tmp217_;
#line 570 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	_tmp218_ = shn;
#line 570 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp218_, GXML_TYPE_TEXT)) {
#line 3945 "SerializableGeeCollectionsTest.c"
																		GXmlNode* _tmp219_ = NULL;
																		gchar* _tmp220_ = NULL;
																		gchar* _tmp221_ = NULL;
																		gchar* _tmp222_ = NULL;
																		gboolean _tmp223_ = FALSE;
#line 571 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																		tfound = TRUE;
#line 572 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																		_tmp219_ = shn;
#line 572 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																		_tmp220_ = gxml_node_get_value (_tmp219_);
#line 572 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																		_tmp221_ = _tmp220_;
#line 572 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																		_tmp222_ = _tmp221_;
#line 572 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																		_tmp223_ = g_strcmp0 (_tmp222_, "TEST_TEXT") != 0;
#line 572 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																		_g_free0 (_tmp222_);
#line 572 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																		if (_tmp223_) {
#line 3967 "SerializableGeeCollectionsTest.c"
																			FILE* _tmp224_ = NULL;
																			GXmlNode* _tmp225_ = NULL;
																			gchar* _tmp226_ = NULL;
																			gchar* _tmp227_ = NULL;
																			gchar* _tmp228_ = NULL;
																			const gchar* _tmp229_ = NULL;
																			gchar* _tmp230_ = NULL;
																			gchar* _tmp231_ = NULL;
#line 573 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																			_tmp224_ = stdout;
#line 573 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																			_tmp225_ = shn;
#line 573 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																			_tmp226_ = gxml_node_get_value (_tmp225_);
#line 573 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																			_tmp227_ = _tmp226_;
#line 573 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																			_tmp228_ = _tmp227_;
#line 573 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																			_tmp229_ = string_to_string (_tmp228_);
#line 573 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																			_tmp230_ = g_strconcat ("ERROR: Invalid Text Node Value for ship MegaTrench: ", _tmp229_, "\n", NULL);
#line 573 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																			_tmp231_ = _tmp230_;
#line 573 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																			fprintf (_tmp224_, "%s", _tmp231_);
#line 573 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																			_g_free0 (_tmp231_);
#line 573 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																			_g_free0 (_tmp228_);
#line 574 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																			g_assert_not_reached ();
#line 4000 "SerializableGeeCollectionsTest.c"
																		}
																	}
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																	_g_object_unref0 (shn);
#line 4005 "SerializableGeeCollectionsTest.c"
																}
#line 568 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
																_g_object_unref0 (_shn_list);
#line 4009 "SerializableGeeCollectionsTest.c"
															}
#line 559 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															_g_object_unref0 (shanattr);
#line 4013 "SerializableGeeCollectionsTest.c"
														}
#line 553 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														_g_object_unref0 (atr);
#line 4017 "SerializableGeeCollectionsTest.c"
													}
#line 580 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
													_tmp232_ = rn;
#line 580 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
													_tmp233_ = gxml_node_get_name (_tmp232_);
#line 580 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
													_tmp234_ = _tmp233_;
#line 580 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
													_tmp235_ = _tmp234_;
#line 580 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
													_tmp236_ = g_strcmp0 (_tmp235_, "UnknownAttribute") == 0;
#line 580 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
													_g_free0 (_tmp235_);
#line 580 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
													if (_tmp236_) {
#line 4033 "SerializableGeeCollectionsTest.c"
														GXmlNode* nattr = NULL;
														GXmlNode* _tmp237_ = NULL;
														GXmlNode* _tmp238_ = NULL;
														GXmlNode* _tmp239_ = NULL;
														GXmlNode* _tmp241_ = NULL;
														gchar* _tmp242_ = NULL;
														gchar* _tmp243_ = NULL;
														gchar* _tmp244_ = NULL;
														gboolean _tmp245_ = FALSE;
#line 581 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														unkfound = TRUE;
#line 582 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														_tmp237_ = rn;
#line 582 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														_tmp238_ = gxml_element_get_attr ((GXmlElement*) G_TYPE_CHECK_INSTANCE_CAST (_tmp237_, GXML_TYPE_GELEMENT, GXmlGElement), "name");
#line 582 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														nattr = _tmp238_;
#line 583 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														_tmp239_ = nattr;
#line 583 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														if (_tmp239_ == NULL) {
#line 4055 "SerializableGeeCollectionsTest.c"
															FILE* _tmp240_ = NULL;
#line 584 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															_tmp240_ = stdout;
#line 584 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															fprintf (_tmp240_, "%s", "ERROR: No Unknown Attribute Node with attribute name\n");
#line 585 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															g_assert_not_reached ();
#line 4063 "SerializableGeeCollectionsTest.c"
														}
#line 587 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														_tmp241_ = nattr;
#line 587 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														_tmp242_ = gxml_node_get_value (_tmp241_);
#line 587 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														_tmp243_ = _tmp242_;
#line 587 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														_tmp244_ = _tmp243_;
#line 587 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														_tmp245_ = g_strcmp0 (_tmp244_, "nothing") != 0;
#line 587 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														_g_free0 (_tmp244_);
#line 587 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														if (_tmp245_) {
#line 4079 "SerializableGeeCollectionsTest.c"
															FILE* _tmp246_ = NULL;
															GXmlNode* _tmp247_ = NULL;
															gchar* _tmp248_ = NULL;
															gchar* _tmp249_ = NULL;
															gchar* _tmp250_ = NULL;
															const gchar* _tmp251_ = NULL;
															gchar* _tmp252_ = NULL;
															gchar* _tmp253_ = NULL;
#line 588 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															_tmp246_ = stdout;
#line 588 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															_tmp247_ = nattr;
#line 588 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															_tmp248_ = gxml_node_get_value (_tmp247_);
#line 588 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															_tmp249_ = _tmp248_;
#line 588 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															_tmp250_ = _tmp249_;
#line 588 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															_tmp251_ = string_to_string (_tmp250_);
#line 588 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															_tmp252_ = g_strconcat ("ERROR: Invalid unknown attribute node's attribute name value: found ", _tmp251_, "\n", NULL);
#line 588 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															_tmp253_ = _tmp252_;
#line 588 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															fprintf (_tmp246_, "%s", _tmp253_);
#line 588 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															_g_free0 (_tmp253_);
#line 588 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															_g_free0 (_tmp250_);
#line 589 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
															g_assert_not_reached ();
#line 4112 "SerializableGeeCollectionsTest.c"
														}
#line 580 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
														_g_object_unref0 (nattr);
#line 4116 "SerializableGeeCollectionsTest.c"
													}
												}
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
												_g_object_unref0 (rn);
#line 4121 "SerializableGeeCollectionsTest.c"
											}
#line 550 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
											_g_object_unref0 (_rn_list);
#line 4125 "SerializableGeeCollectionsTest.c"
										}
									}
								}
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_g_object_unref0 (sn);
#line 4131 "SerializableGeeCollectionsTest.c"
							}
#line 547 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_g_object_unref0 (_sn_list);
#line 4135 "SerializableGeeCollectionsTest.c"
						}
#line 597 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						_tmp254_ = attrfound;
#line 597 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						if (!_tmp254_) {
#line 4141 "SerializableGeeCollectionsTest.c"
							FILE* _tmp255_ = NULL;
#line 598 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_tmp255_ = stdout;
#line 598 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							fprintf (_tmp255_, "%s", "ERROR: No Unknown AttributeText found for ship MegaTrench\n");
#line 599 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							g_assert_not_reached ();
#line 4149 "SerializableGeeCollectionsTest.c"
						}
#line 601 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						_tmp256_ = tfound;
#line 601 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						if (!_tmp256_) {
#line 4155 "SerializableGeeCollectionsTest.c"
							FILE* _tmp257_ = NULL;
#line 602 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_tmp257_ = stdout;
#line 602 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							fprintf (_tmp257_, "%s", "ERROR: No Text Node Value found for ship MegaTrench\n");
#line 603 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							g_assert_not_reached ();
#line 4163 "SerializableGeeCollectionsTest.c"
						}
#line 605 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						_tmp258_ = unkfound;
#line 605 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						if (!_tmp258_) {
#line 4169 "SerializableGeeCollectionsTest.c"
							FILE* _tmp259_ = NULL;
#line 606 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_tmp259_ = stdout;
#line 606 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							fprintf (_tmp259_, "%s", "ERROR: No Unknown Attribute Node found for storage\n");
#line 607 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							g_assert_not_reached ();
#line 4177 "SerializableGeeCollectionsTest.c"
						}
					}
				}
#line 611 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp260_ = n;
#line 611 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp260_, GXML_TYPE_TEXT)) {
#line 4185 "SerializableGeeCollectionsTest.c"
					FILE* _tmp261_ = NULL;
					GXmlNode* _tmp262_ = NULL;
					gchar* _tmp263_ = NULL;
					gchar* _tmp264_ = NULL;
					gchar* _tmp265_ = NULL;
					const gchar* _tmp266_ = NULL;
					gchar* _tmp267_ = NULL;
					gchar* _tmp268_ = NULL;
#line 612 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp261_ = stdout;
#line 612 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp262_ = n;
#line 612 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp263_ = gxml_node_get_value (_tmp262_);
#line 612 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp264_ = _tmp263_;
#line 612 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp265_ = _tmp264_;
#line 612 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp266_ = string_to_string (_tmp265_);
#line 612 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp267_ = g_strconcat ("ROOT NODE VALUE: '", _tmp266_, "'\n", NULL);
#line 612 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp268_ = _tmp267_;
#line 612 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					fprintf (_tmp261_, "%s", _tmp268_);
#line 612 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_g_free0 (_tmp268_);
#line 612 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_g_free0 (_tmp265_);
#line 4216 "SerializableGeeCollectionsTest.c"
				}
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_g_object_unref0 (n);
#line 4220 "SerializableGeeCollectionsTest.c"
			}
#line 538 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (_n_list);
#line 4224 "SerializableGeeCollectionsTest.c"
		}
#line 445 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (doc);
#line 445 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (uns);
#line 445 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (ups);
#line 445 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (refaction);
#line 445 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (mssh);
#line 445 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (s);
#line 445 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (org_doc);
#line 4240 "SerializableGeeCollectionsTest.c"
	}
	goto __finally96;
	__catch96_g_error:
	{
		GError* e = NULL;
		FILE* _tmp269_ = NULL;
		const gchar* _tmp270_ = NULL;
		const gchar* _tmp271_ = NULL;
		gchar* _tmp272_ = NULL;
		gchar* _tmp273_ = NULL;
#line 445 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		e = _inner_error_;
#line 445 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_inner_error_ = NULL;
#line 617 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp269_ = stdout;
#line 617 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp270_ = e->message;
#line 617 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp271_ = string_to_string (_tmp270_);
#line 617 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp272_ = g_strconcat ("ERROR: ", _tmp271_, NULL);
#line 617 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp273_ = _tmp272_;
#line 617 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		fprintf (_tmp269_, "%s", _tmp273_);
#line 617 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_free0 (_tmp273_);
#line 618 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_assert_not_reached ();
#line 445 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_error_free0 (e);
#line 4273 "SerializableGeeCollectionsTest.c"
	}
	__finally96:
#line 445 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 445 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 445 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_clear_error (&_inner_error_);
#line 445 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		return;
#line 4284 "SerializableGeeCollectionsTest.c"
	}
}


static void ___lambda68__gtest_func (void) {
#line 443 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	__lambda68_ ();
#line 4292 "SerializableGeeCollectionsTest.c"
}


static void __lambda69_ (void) {
	GError * _inner_error_ = NULL;
	{
		SerializableGeeCollectionsTestBigBag* bag = NULL;
		SerializableGeeCollectionsTestBigBag* _tmp0_ = NULL;
		SerializableGeeCollectionsTestBigBag* _tmp1_ = NULL;
		SerializableGeeCollectionsTestSmallBagArray* _tmp2_ = NULL;
		SerializableGeeCollectionsTestSmallBagArray* _tmp3_ = NULL;
		SerializableGeeCollectionsTestBigBag* _tmp28_ = NULL;
		SerializableGeeCollectionsTestSmallBagArray* _tmp29_ = NULL;
		SerializableGeeCollectionsTestSmallBagArray* _tmp30_ = NULL;
		gint _tmp31_ = 0;
		gint _tmp32_ = 0;
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp33_ = NULL;
		SerializableGeeCollectionsTestBigBag* _tmp34_ = NULL;
		GXmlTDocument* _tmp35_ = NULL;
		GXmlNode* _tmp36_ = NULL;
		GXmlNode* _tmp37_ = NULL;
		GXmlTDocument* _tmp38_ = NULL;
		GXmlNode* _tmp39_ = NULL;
		GXmlNode* _tmp40_ = NULL;
		GXmlNode* _tmp41_ = NULL;
		GXmlTDocument* _tmp42_ = NULL;
		GXmlNode* _tmp43_ = NULL;
		GXmlNode* _tmp44_ = NULL;
		GXmlNode* _tmp45_ = NULL;
		gchar* _tmp46_ = NULL;
		gchar* _tmp47_ = NULL;
		gchar* _tmp48_ = NULL;
		GXmlTDocument* _tmp49_ = NULL;
		GXmlNode* _tmp50_ = NULL;
		GXmlNode* _tmp51_ = NULL;
		GXmlNode* _tmp52_ = NULL;
		GeeBidirList* _tmp53_ = NULL;
		GeeBidirList* _tmp54_ = NULL;
		GeeBidirList* _tmp55_ = NULL;
		gint _tmp56_ = 0;
		gint _tmp57_ = 0;
		GXmlTDocument* _tmp58_ = NULL;
		GXmlNode* _tmp59_ = NULL;
		GXmlNode* _tmp60_ = NULL;
		GXmlNode* _tmp61_ = NULL;
		GeeBidirList* _tmp62_ = NULL;
		GeeBidirList* _tmp63_ = NULL;
		GeeBidirList* _tmp64_ = NULL;
		gpointer _tmp65_ = NULL;
		GXmlNode* _tmp66_ = NULL;
		gchar* _tmp67_ = NULL;
		gchar* _tmp68_ = NULL;
		gchar* _tmp69_ = NULL;
		GXmlTDocument* _tmp70_ = NULL;
		GXmlNode* _tmp71_ = NULL;
		GXmlNode* _tmp72_ = NULL;
		GXmlNode* _tmp73_ = NULL;
		GeeBidirList* _tmp74_ = NULL;
		GeeBidirList* _tmp75_ = NULL;
		GeeBidirList* _tmp76_ = NULL;
		gpointer _tmp77_ = NULL;
		GXmlNode* _tmp78_ = NULL;
		GeeBidirList* _tmp79_ = NULL;
		GeeBidirList* _tmp80_ = NULL;
		GeeBidirList* _tmp81_ = NULL;
		gint _tmp82_ = 0;
		gint _tmp83_ = 0;
		GXmlTDocument* _tmp84_ = NULL;
		GXmlNode* _tmp85_ = NULL;
		GXmlNode* _tmp86_ = NULL;
		GXmlNode* _tmp87_ = NULL;
		GeeBidirList* _tmp88_ = NULL;
		GeeBidirList* _tmp89_ = NULL;
		GeeBidirList* _tmp90_ = NULL;
		gpointer _tmp91_ = NULL;
		GXmlNode* _tmp92_ = NULL;
		GeeBidirList* _tmp93_ = NULL;
		GeeBidirList* _tmp94_ = NULL;
		GeeBidirList* _tmp95_ = NULL;
		gpointer _tmp96_ = NULL;
		GXmlNode* _tmp97_ = NULL;
		gchar* _tmp98_ = NULL;
		gchar* _tmp99_ = NULL;
		gchar* _tmp100_ = NULL;
		SerializableGeeCollectionsTestBigBag* bag2 = NULL;
		SerializableGeeCollectionsTestBigBag* _tmp101_ = NULL;
		SerializableGeeCollectionsTestBigBag* _tmp102_ = NULL;
		GXmlTDocument* _tmp103_ = NULL;
		SerializableGeeCollectionsTestBigBag* _tmp104_ = NULL;
		SerializableGeeCollectionsTestSmallBagArray* _tmp105_ = NULL;
		SerializableGeeCollectionsTestSmallBagArray* _tmp106_ = NULL;
		gint _tmp107_ = 0;
		gint _tmp108_ = 0;
		GXmlTDocument* d2 = NULL;
		GXmlTDocument* _tmp109_ = NULL;
		SerializableGeeCollectionsTestBigBag* _tmp110_ = NULL;
		GXmlTDocument* _tmp111_ = NULL;
		GXmlNode* _tmp112_ = NULL;
		GXmlNode* _tmp113_ = NULL;
		GXmlTDocument* _tmp114_ = NULL;
		GXmlNode* _tmp115_ = NULL;
		GXmlNode* _tmp116_ = NULL;
		GXmlNode* _tmp117_ = NULL;
		GXmlTDocument* _tmp118_ = NULL;
		GXmlNode* _tmp119_ = NULL;
		GXmlNode* _tmp120_ = NULL;
		GXmlNode* _tmp121_ = NULL;
		gchar* _tmp122_ = NULL;
		gchar* _tmp123_ = NULL;
		gchar* _tmp124_ = NULL;
		GXmlTDocument* _tmp125_ = NULL;
		GXmlNode* _tmp126_ = NULL;
		GXmlNode* _tmp127_ = NULL;
		GXmlNode* _tmp128_ = NULL;
		GeeBidirList* _tmp129_ = NULL;
		GeeBidirList* _tmp130_ = NULL;
		GeeBidirList* _tmp131_ = NULL;
		gint _tmp132_ = 0;
		gint _tmp133_ = 0;
		GXmlTDocument* _tmp134_ = NULL;
		GXmlNode* _tmp135_ = NULL;
		GXmlNode* _tmp136_ = NULL;
		GXmlNode* _tmp137_ = NULL;
		GeeBidirList* _tmp138_ = NULL;
		GeeBidirList* _tmp139_ = NULL;
		GeeBidirList* _tmp140_ = NULL;
		gpointer _tmp141_ = NULL;
		GXmlNode* _tmp142_ = NULL;
		gchar* _tmp143_ = NULL;
		gchar* _tmp144_ = NULL;
		gchar* _tmp145_ = NULL;
		GXmlTDocument* _tmp146_ = NULL;
		GXmlNode* _tmp147_ = NULL;
		GXmlNode* _tmp148_ = NULL;
		GXmlNode* _tmp149_ = NULL;
		GeeBidirList* _tmp150_ = NULL;
		GeeBidirList* _tmp151_ = NULL;
		GeeBidirList* _tmp152_ = NULL;
		gpointer _tmp153_ = NULL;
		GXmlNode* _tmp154_ = NULL;
		GeeBidirList* _tmp155_ = NULL;
		GeeBidirList* _tmp156_ = NULL;
		GeeBidirList* _tmp157_ = NULL;
		gint _tmp158_ = 0;
		gint _tmp159_ = 0;
		GXmlTDocument* _tmp160_ = NULL;
		GXmlNode* _tmp161_ = NULL;
		GXmlNode* _tmp162_ = NULL;
		GXmlNode* _tmp163_ = NULL;
		GeeBidirList* _tmp164_ = NULL;
		GeeBidirList* _tmp165_ = NULL;
		GeeBidirList* _tmp166_ = NULL;
		gpointer _tmp167_ = NULL;
		GXmlNode* _tmp168_ = NULL;
		GeeBidirList* _tmp169_ = NULL;
		GeeBidirList* _tmp170_ = NULL;
		GeeBidirList* _tmp171_ = NULL;
		gpointer _tmp172_ = NULL;
		GXmlNode* _tmp173_ = NULL;
		gchar* _tmp174_ = NULL;
		gchar* _tmp175_ = NULL;
		gchar* _tmp176_ = NULL;
#line 625 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp0_ = serializable_gee_collections_test_big_bag_new ();
#line 625 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		bag = _tmp0_;
#line 626 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp1_ = bag;
#line 626 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp2_ = serializable_gee_collections_test_big_bag_get_bags (_tmp1_);
#line 626 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp3_ = _tmp2_;
#line 626 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_vala_assert (_tmp3_ != NULL, "bag.bags != null");
#line 4468 "SerializableGeeCollectionsTest.c"
		{
			gint i = 0;
#line 627 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			i = 0;
#line 4473 "SerializableGeeCollectionsTest.c"
			{
				gboolean _tmp4_ = FALSE;
#line 627 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				_tmp4_ = TRUE;
#line 627 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
				while (TRUE) {
#line 4480 "SerializableGeeCollectionsTest.c"
					gint _tmp6_ = 0;
					SerializableGeeCollectionsTestSmallBag* sbag = NULL;
					SerializableGeeCollectionsTestSmallBag* _tmp7_ = NULL;
					SerializableGeeCollectionsTestSmallBag* _tmp8_ = NULL;
					SerializableGeeCollectionsTestBallArray* _tmp9_ = NULL;
					SerializableGeeCollectionsTestBallArray* _tmp10_ = NULL;
					SerializableGeeCollectionsTestSmallBag* _tmp19_ = NULL;
					SerializableGeeCollectionsTestBallArray* _tmp20_ = NULL;
					SerializableGeeCollectionsTestBallArray* _tmp21_ = NULL;
					gint _tmp22_ = 0;
					gint _tmp23_ = 0;
					SerializableGeeCollectionsTestBigBag* _tmp24_ = NULL;
					SerializableGeeCollectionsTestSmallBagArray* _tmp25_ = NULL;
					SerializableGeeCollectionsTestSmallBagArray* _tmp26_ = NULL;
					SerializableGeeCollectionsTestSmallBag* _tmp27_ = NULL;
#line 627 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					if (!_tmp4_) {
#line 4498 "SerializableGeeCollectionsTest.c"
						gint _tmp5_ = 0;
#line 627 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						_tmp5_ = i;
#line 627 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						i = _tmp5_ + 1;
#line 4504 "SerializableGeeCollectionsTest.c"
					}
#line 627 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp4_ = FALSE;
#line 627 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp6_ = i;
#line 627 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					if (!(_tmp6_ < 2)) {
#line 627 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						break;
#line 4514 "SerializableGeeCollectionsTest.c"
					}
#line 628 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp7_ = serializable_gee_collections_test_small_bag_new ();
#line 628 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					sbag = _tmp7_;
#line 629 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp8_ = sbag;
#line 629 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp9_ = serializable_gee_collections_test_small_bag_get_balls (_tmp8_);
#line 629 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp10_ = _tmp9_;
#line 629 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_vala_assert (_tmp10_ != NULL, "sbag.balls != null");
#line 4528 "SerializableGeeCollectionsTest.c"
					{
						gint j = 0;
#line 630 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
						j = 0;
#line 4533 "SerializableGeeCollectionsTest.c"
						{
							gboolean _tmp11_ = FALSE;
#line 630 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							_tmp11_ = TRUE;
#line 630 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
							while (TRUE) {
#line 4540 "SerializableGeeCollectionsTest.c"
								gint _tmp13_ = 0;
								SerializableGeeCollectionsTestBall* b = NULL;
								SerializableGeeCollectionsTestBall* _tmp14_ = NULL;
								SerializableGeeCollectionsTestSmallBag* _tmp15_ = NULL;
								SerializableGeeCollectionsTestBallArray* _tmp16_ = NULL;
								SerializableGeeCollectionsTestBallArray* _tmp17_ = NULL;
								SerializableGeeCollectionsTestBall* _tmp18_ = NULL;
#line 630 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								if (!_tmp11_) {
#line 4550 "SerializableGeeCollectionsTest.c"
									gint _tmp12_ = 0;
#line 630 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
									_tmp12_ = j;
#line 630 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
									j = _tmp12_ + 1;
#line 4556 "SerializableGeeCollectionsTest.c"
								}
#line 630 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_tmp11_ = FALSE;
#line 630 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_tmp13_ = j;
#line 630 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								if (!(_tmp13_ < 2)) {
#line 630 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
									break;
#line 4566 "SerializableGeeCollectionsTest.c"
								}
#line 631 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_tmp14_ = serializable_gee_collections_test_ball_new ();
#line 631 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								b = _tmp14_;
#line 632 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_tmp15_ = sbag;
#line 632 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_tmp16_ = serializable_gee_collections_test_small_bag_get_balls (_tmp15_);
#line 632 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_tmp17_ = _tmp16_;
#line 632 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_tmp18_ = b;
#line 632 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								gee_abstract_collection_add ((GeeAbstractCollection*) _tmp17_, _tmp18_);
#line 630 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
								_g_object_unref0 (b);
#line 4584 "SerializableGeeCollectionsTest.c"
							}
						}
					}
#line 634 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp19_ = sbag;
#line 634 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp20_ = serializable_gee_collections_test_small_bag_get_balls (_tmp19_);
#line 634 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp21_ = _tmp20_;
#line 634 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp22_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp21_);
#line 634 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp23_ = _tmp22_;
#line 634 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_vala_assert (_tmp23_ == 2, "sbag.balls.size == 2");
#line 635 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp24_ = bag;
#line 635 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp25_ = serializable_gee_collections_test_big_bag_get_bags (_tmp24_);
#line 635 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp26_ = _tmp25_;
#line 635 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_tmp27_ = sbag;
#line 635 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					gee_abstract_collection_add ((GeeAbstractCollection*) _tmp26_, _tmp27_);
#line 627 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
					_g_object_unref0 (sbag);
#line 4612 "SerializableGeeCollectionsTest.c"
				}
			}
		}
#line 637 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp28_ = bag;
#line 637 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp29_ = serializable_gee_collections_test_big_bag_get_bags (_tmp28_);
#line 637 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp30_ = _tmp29_;
#line 637 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp31_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp30_);
#line 637 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp32_ = _tmp31_;
#line 637 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_vala_assert (_tmp32_ == 2, "bag.bags.size == 2");
#line 639 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp33_ = gxml_tdocument_new ();
#line 639 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		d = _tmp33_;
#line 640 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp34_ = bag;
#line 640 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp35_ = d;
#line 640 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp36_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) _tmp34_, (GXmlNode*) _tmp35_, &_inner_error_);
#line 640 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp37_ = _tmp36_;
#line 640 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp37_);
#line 640 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 640 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (d);
#line 640 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (bag);
#line 4648 "SerializableGeeCollectionsTest.c"
			goto __catch97_g_error;
		}
#line 641 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp38_ = d;
#line 641 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp39_ = gxml_document_get_root ((GXmlDocument*) _tmp38_);
#line 641 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp40_ = _tmp39_;
#line 641 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp41_ = _tmp40_;
#line 641 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_vala_assert (_tmp41_ != NULL, "d.root != null");
#line 641 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp41_);
#line 642 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp42_ = d;
#line 642 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp43_ = gxml_document_get_root ((GXmlDocument*) _tmp42_);
#line 642 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp44_ = _tmp43_;
#line 642 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp45_ = _tmp44_;
#line 642 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp46_ = gxml_node_get_name (_tmp45_);
#line 642 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp47_ = _tmp46_;
#line 642 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp48_ = _tmp47_;
#line 642 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_vala_assert (g_strcmp0 (_tmp48_, "BigBag") == 0, "d.root.name == \"BigBag\"");
#line 642 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_free0 (_tmp48_);
#line 642 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp45_);
#line 643 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp49_ = d;
#line 643 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp50_ = gxml_document_get_root ((GXmlDocument*) _tmp49_);
#line 643 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp51_ = _tmp50_;
#line 643 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp52_ = _tmp51_;
#line 643 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp53_ = gxml_node_get_children_nodes (_tmp52_);
#line 643 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp54_ = _tmp53_;
#line 643 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp55_ = _tmp54_;
#line 643 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp56_ = gee_collection_get_size ((GeeCollection*) _tmp55_);
#line 643 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp57_ = _tmp56_;
#line 643 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_vala_assert (_tmp57_ == 2, "d.root.children_nodes.size == 2");
#line 643 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp55_);
#line 643 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp52_);
#line 644 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp58_ = d;
#line 644 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp59_ = gxml_document_get_root ((GXmlDocument*) _tmp58_);
#line 644 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp60_ = _tmp59_;
#line 644 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp61_ = _tmp60_;
#line 644 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp62_ = gxml_node_get_children_nodes (_tmp61_);
#line 644 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp63_ = _tmp62_;
#line 644 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp64_ = _tmp63_;
#line 644 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp65_ = gee_list_get ((GeeList*) _tmp64_, 0);
#line 644 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp66_ = (GXmlNode*) _tmp65_;
#line 644 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp67_ = gxml_node_get_name (_tmp66_);
#line 644 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp68_ = _tmp67_;
#line 644 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp69_ = _tmp68_;
#line 644 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_vala_assert (g_strcmp0 (_tmp69_, "SmallBag") == 0, "d.root.children_nodes[0].name == \"SmallBag\"");
#line 644 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_free0 (_tmp69_);
#line 644 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp66_);
#line 644 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp64_);
#line 644 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp61_);
#line 645 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp70_ = d;
#line 645 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp71_ = gxml_document_get_root ((GXmlDocument*) _tmp70_);
#line 645 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp72_ = _tmp71_;
#line 645 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp73_ = _tmp72_;
#line 645 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp74_ = gxml_node_get_children_nodes (_tmp73_);
#line 645 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp75_ = _tmp74_;
#line 645 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp76_ = _tmp75_;
#line 645 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp77_ = gee_list_get ((GeeList*) _tmp76_, 0);
#line 645 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp78_ = (GXmlNode*) _tmp77_;
#line 645 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp79_ = gxml_node_get_children_nodes (_tmp78_);
#line 645 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp80_ = _tmp79_;
#line 645 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp81_ = _tmp80_;
#line 645 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp82_ = gee_collection_get_size ((GeeCollection*) _tmp81_);
#line 645 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp83_ = _tmp82_;
#line 645 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_vala_assert (_tmp83_ == 2, "d.root.children_nodes[0].children_nodes.size == 2");
#line 645 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp81_);
#line 645 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp78_);
#line 645 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp76_);
#line 645 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp73_);
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp84_ = d;
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp85_ = gxml_document_get_root ((GXmlDocument*) _tmp84_);
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp86_ = _tmp85_;
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp87_ = _tmp86_;
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp88_ = gxml_node_get_children_nodes (_tmp87_);
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp89_ = _tmp88_;
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp90_ = _tmp89_;
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp91_ = gee_list_get ((GeeList*) _tmp90_, 0);
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp92_ = (GXmlNode*) _tmp91_;
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp93_ = gxml_node_get_children_nodes (_tmp92_);
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp94_ = _tmp93_;
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp95_ = _tmp94_;
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp96_ = gee_list_get ((GeeList*) _tmp95_, 0);
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp97_ = (GXmlNode*) _tmp96_;
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp98_ = gxml_node_get_name (_tmp97_);
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp99_ = _tmp98_;
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp100_ = _tmp99_;
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_vala_assert (g_strcmp0 (_tmp100_, "Ball") == 0, "d.root.children_nodes[0].children_nodes[0].name == \"Ball\"");
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_free0 (_tmp100_);
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp97_);
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp95_);
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp92_);
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp90_);
#line 646 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp87_);
#line 648 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp101_ = serializable_gee_collections_test_big_bag_new ();
#line 648 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		bag2 = _tmp101_;
#line 649 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp102_ = bag2;
#line 649 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp103_ = d;
#line 649 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp102_, (GXmlNode*) _tmp103_, &_inner_error_);
#line 649 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 649 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (bag2);
#line 649 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (d);
#line 649 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (bag);
#line 4845 "SerializableGeeCollectionsTest.c"
			goto __catch97_g_error;
		}
#line 650 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp104_ = bag2;
#line 650 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp105_ = serializable_gee_collections_test_big_bag_get_bags (_tmp104_);
#line 650 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp106_ = _tmp105_;
#line 650 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp107_ = gee_abstract_collection_get_size ((GeeAbstractCollection*) _tmp106_);
#line 650 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp108_ = _tmp107_;
#line 650 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_vala_assert (_tmp108_ == 0, "bag2.bags.size == 0");
#line 652 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp109_ = gxml_tdocument_new ();
#line 652 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		d2 = _tmp109_;
#line 653 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp110_ = bag2;
#line 653 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp111_ = d2;
#line 653 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp112_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) _tmp110_, (GXmlNode*) _tmp111_, &_inner_error_);
#line 653 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp113_ = _tmp112_;
#line 653 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp113_);
#line 653 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 653 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (d2);
#line 653 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (bag2);
#line 653 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (d);
#line 653 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
			_g_object_unref0 (bag);
#line 4884 "SerializableGeeCollectionsTest.c"
			goto __catch97_g_error;
		}
#line 654 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp114_ = d2;
#line 654 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp115_ = gxml_document_get_root ((GXmlDocument*) _tmp114_);
#line 654 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp116_ = _tmp115_;
#line 654 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp117_ = _tmp116_;
#line 654 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_vala_assert (_tmp117_ != NULL, "d2.root != null");
#line 654 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp117_);
#line 655 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp118_ = d2;
#line 655 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp119_ = gxml_document_get_root ((GXmlDocument*) _tmp118_);
#line 655 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp120_ = _tmp119_;
#line 655 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp121_ = _tmp120_;
#line 655 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp122_ = gxml_node_get_name (_tmp121_);
#line 655 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp123_ = _tmp122_;
#line 655 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp124_ = _tmp123_;
#line 655 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_vala_assert (g_strcmp0 (_tmp124_, "BigBag") == 0, "d2.root.name == \"BigBag\"");
#line 655 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_free0 (_tmp124_);
#line 655 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp121_);
#line 656 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp125_ = d2;
#line 656 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp126_ = gxml_document_get_root ((GXmlDocument*) _tmp125_);
#line 656 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp127_ = _tmp126_;
#line 656 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp128_ = _tmp127_;
#line 656 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp129_ = gxml_node_get_children_nodes (_tmp128_);
#line 656 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp130_ = _tmp129_;
#line 656 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp131_ = _tmp130_;
#line 656 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp132_ = gee_collection_get_size ((GeeCollection*) _tmp131_);
#line 656 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp133_ = _tmp132_;
#line 656 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_vala_assert (_tmp133_ == 2, "d2.root.children_nodes.size == 2");
#line 656 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp131_);
#line 656 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp128_);
#line 657 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp134_ = d2;
#line 657 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp135_ = gxml_document_get_root ((GXmlDocument*) _tmp134_);
#line 657 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp136_ = _tmp135_;
#line 657 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp137_ = _tmp136_;
#line 657 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp138_ = gxml_node_get_children_nodes (_tmp137_);
#line 657 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp139_ = _tmp138_;
#line 657 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp140_ = _tmp139_;
#line 657 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp141_ = gee_list_get ((GeeList*) _tmp140_, 0);
#line 657 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp142_ = (GXmlNode*) _tmp141_;
#line 657 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp143_ = gxml_node_get_name (_tmp142_);
#line 657 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp144_ = _tmp143_;
#line 657 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp145_ = _tmp144_;
#line 657 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_vala_assert (g_strcmp0 (_tmp145_, "SmallBag") == 0, "d2.root.children_nodes[0].name == \"SmallBag\"");
#line 657 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_free0 (_tmp145_);
#line 657 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp142_);
#line 657 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp140_);
#line 657 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp137_);
#line 658 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp146_ = d2;
#line 658 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp147_ = gxml_document_get_root ((GXmlDocument*) _tmp146_);
#line 658 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp148_ = _tmp147_;
#line 658 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp149_ = _tmp148_;
#line 658 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp150_ = gxml_node_get_children_nodes (_tmp149_);
#line 658 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp151_ = _tmp150_;
#line 658 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp152_ = _tmp151_;
#line 658 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp153_ = gee_list_get ((GeeList*) _tmp152_, 0);
#line 658 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp154_ = (GXmlNode*) _tmp153_;
#line 658 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp155_ = gxml_node_get_children_nodes (_tmp154_);
#line 658 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp156_ = _tmp155_;
#line 658 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp157_ = _tmp156_;
#line 658 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp158_ = gee_collection_get_size ((GeeCollection*) _tmp157_);
#line 658 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp159_ = _tmp158_;
#line 658 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_vala_assert (_tmp159_ == 2, "d2.root.children_nodes[0].children_nodes.size == 2");
#line 658 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp157_);
#line 658 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp154_);
#line 658 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp152_);
#line 658 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp149_);
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp160_ = d2;
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp161_ = gxml_document_get_root ((GXmlDocument*) _tmp160_);
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp162_ = _tmp161_;
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp163_ = _tmp162_;
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp164_ = gxml_node_get_children_nodes (_tmp163_);
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp165_ = _tmp164_;
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp166_ = _tmp165_;
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp167_ = gee_list_get ((GeeList*) _tmp166_, 0);
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp168_ = (GXmlNode*) _tmp167_;
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp169_ = gxml_node_get_children_nodes (_tmp168_);
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp170_ = _tmp169_;
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp171_ = _tmp170_;
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp172_ = gee_list_get ((GeeList*) _tmp171_, 0);
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp173_ = (GXmlNode*) _tmp172_;
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp174_ = gxml_node_get_name (_tmp173_);
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp175_ = _tmp174_;
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp176_ = _tmp175_;
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_vala_assert (g_strcmp0 (_tmp176_, "Ball") == 0, "d2.root.children_nodes[0].children_nodes[0].name == \"Ball\"");
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_free0 (_tmp176_);
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp173_);
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp171_);
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp168_);
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp166_);
#line 659 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp163_);
#line 623 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (d2);
#line 623 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (bag2);
#line 623 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (d);
#line 623 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (bag);
#line 5071 "SerializableGeeCollectionsTest.c"
	}
	goto __finally97;
	__catch97_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp177_ = NULL;
		gchar* _tmp178_ = NULL;
		gchar* _tmp179_ = NULL;
#line 623 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		e = _inner_error_;
#line 623 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_inner_error_ = NULL;
#line 661 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp177_ = e->message;
#line 661 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp178_ = g_strconcat ("ERROR: ", _tmp177_, NULL);
#line 661 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp179_ = _tmp178_;
#line 661 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_message ("SerializableGeeCollectionsTest.vala:661: %s", _tmp179_);
#line 661 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_free0 (_tmp179_);
#line 662 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_assert_not_reached ();
#line 623 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_error_free0 (e);
#line 5098 "SerializableGeeCollectionsTest.c"
	}
	__finally97:
#line 623 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 623 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 623 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_clear_error (&_inner_error_);
#line 623 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		return;
#line 5109 "SerializableGeeCollectionsTest.c"
	}
}


static void ___lambda69__gtest_func (void) {
#line 621 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	__lambda69_ ();
#line 5117 "SerializableGeeCollectionsTest.c"
}


void serializable_gee_collections_test_add_tests (void) {
#line 219 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_test_add_func ("/gxml/serializable/convined_gee_containers/deserialize", ___lambda66__gtest_func);
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_test_add_func ("/gxml/serializable/convined_gee_containers/de-se-deserialize", ___lambda67__gtest_func);
#line 443 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_test_add_func ("/gxml/serializable/convined_gee_containers/se-deserialize-unknowns", ___lambda68__gtest_func);
#line 621 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_test_add_func ("/gxml/serializable/containers/post-deserialization/serialize", ___lambda69__gtest_func);
#line 5130 "SerializableGeeCollectionsTest.c"
}


SerializableGeeCollectionsTest* serializable_gee_collections_test_construct (GType object_type) {
	SerializableGeeCollectionsTest* self = NULL;
#line 30 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTest*) gxml_test_construct (object_type);
#line 30 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 5140 "SerializableGeeCollectionsTest.c"
}


SerializableGeeCollectionsTest* serializable_gee_collections_test_new (void) {
#line 30 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_construct (TYPE_SERIALIZABLE_GEE_COLLECTIONS_TEST);
#line 5147 "SerializableGeeCollectionsTest.c"
}


static gboolean serializable_gee_collections_test_citizen_real_serialize_use_xml_node_value (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestCitizen * self;
	gboolean result = FALSE;
#line 36 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestCitizen*) base;
#line 36 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = TRUE;
#line 36 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 5160 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_citizen_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestCitizen * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
	gchar* _tmp2_ = NULL;
#line 37 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestCitizen*) base;
#line 37 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_ctype;
#line 37 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = string_to_string (_tmp0_);
#line 37 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp2_ = g_strconcat ("Citizen: ", _tmp1_, NULL);
#line 37 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp2_;
#line 37 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 5182 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_citizen_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestCitizen * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 38 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestCitizen*) base;
#line 38 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = g_strdup ("citizen");
#line 38 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 38 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 5198 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestCitizen* serializable_gee_collections_test_citizen_construct (GType object_type) {
	SerializableGeeCollectionsTestCitizen * self = NULL;
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestCitizen*) gxml_serializable_object_model_construct (object_type);
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 5208 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestCitizen* serializable_gee_collections_test_citizen_new (void) {
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_citizen_construct (SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CITIZEN);
#line 5215 "SerializableGeeCollectionsTest.c"
}


static const gchar* serializable_gee_collections_test_citizen_get_ctype (SerializableGeeCollectionsTestCitizen* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_ctype;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 5230 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_citizen_set_ctype (SerializableGeeCollectionsTestCitizen* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_ctype);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_ctype = _tmp1_;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "ctype");
#line 5249 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestCitizenArray* serializable_gee_collections_test_citizen_array_construct (GType object_type) {
	SerializableGeeCollectionsTestCitizenArray * self = NULL;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestCitizenArray*) gxml_serializable_array_list_construct (object_type, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CITIZEN, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 5259 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestCitizenArray* serializable_gee_collections_test_citizen_array_new (void) {
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_citizen_array_construct (SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_TYPE_ARRAY);
#line 5266 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_citizen_array_class_init (SerializableGeeCollectionsTestCitizenArrayClass * klass) {
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_citizen_array_parent_class = g_type_class_peek_parent (klass);
#line 5273 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_citizen_array_instance_init (SerializableGeeCollectionsTestCitizenArray * self) {
}


GType serializable_gee_collections_test_citizen_array_get_type (void) {
	static volatile gsize serializable_gee_collections_test_citizen_array_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_citizen_array_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestCitizenArrayClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_citizen_array_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTestCitizenArray), 0, (GInstanceInitFunc) serializable_gee_collections_test_citizen_array_instance_init, NULL };
		GType serializable_gee_collections_test_citizen_array_type_id;
		serializable_gee_collections_test_citizen_array_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_ARRAY_LIST, "SerializableGeeCollectionsTestCitizenArray", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_test_citizen_array_type_id__volatile, serializable_gee_collections_test_citizen_array_type_id);
	}
	return serializable_gee_collections_test_citizen_array_type_id__volatile;
}


static void serializable_gee_collections_test_citizen_class_init (SerializableGeeCollectionsTestCitizenClass * klass) {
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_citizen_parent_class = g_type_class_peek_parent (klass);
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTestCitizenPrivate));
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->serialize_use_xml_node_value = (gboolean (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_citizen_real_serialize_use_xml_node_value;
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_citizen_real_to_string;
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_citizen_real_node_name;
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_test_citizen_get_property;
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_test_citizen_set_property;
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_test_citizen_finalize;
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_CTYPE, g_param_spec_string ("ctype", "ctype", "ctype", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 5312 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_citizen_instance_init (SerializableGeeCollectionsTestCitizen * self) {
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_GET_PRIVATE (self);
#line 5319 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_citizen_finalize (GObject* obj) {
	SerializableGeeCollectionsTestCitizen * self;
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CITIZEN, SerializableGeeCollectionsTestCitizen);
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_ctype);
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (serializable_gee_collections_test_citizen_parent_class)->finalize (obj);
#line 5331 "SerializableGeeCollectionsTest.c"
}


static GType serializable_gee_collections_test_citizen_get_type (void) {
	static volatile gsize serializable_gee_collections_test_citizen_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_citizen_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestCitizenClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_citizen_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTestCitizen), 0, (GInstanceInitFunc) serializable_gee_collections_test_citizen_instance_init, NULL };
		GType serializable_gee_collections_test_citizen_type_id;
		serializable_gee_collections_test_citizen_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeCollectionsTestCitizen", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_test_citizen_type_id__volatile, serializable_gee_collections_test_citizen_type_id);
	}
	return serializable_gee_collections_test_citizen_type_id__volatile;
}


static void _vala_serializable_gee_collections_test_citizen_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestCitizen * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CITIZEN, SerializableGeeCollectionsTestCitizen);
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_CTYPE:
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_string (value, serializable_gee_collections_test_citizen_get_ctype (self));
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 5358 "SerializableGeeCollectionsTest.c"
		default:
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 5364 "SerializableGeeCollectionsTest.c"
	}
}


static void _vala_serializable_gee_collections_test_citizen_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestCitizen * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CITIZEN, SerializableGeeCollectionsTestCitizen);
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_CTYPE:
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_citizen_set_ctype (self, g_value_get_string (value));
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 5380 "SerializableGeeCollectionsTest.c"
		default:
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 32 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 5386 "SerializableGeeCollectionsTest.c"
	}
}


static SerializableGeeCollectionsTestAsteroid* serializable_gee_collections_test_asteroid_construct_with_size (GType object_type, gint size) {
	SerializableGeeCollectionsTestAsteroid * self = NULL;
	gint _tmp0_ = 0;
#line 45 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestAsteroid*) gxml_serializable_object_model_construct (object_type);
#line 45 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = size;
#line 45 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_asteroid_set_size (self, _tmp0_);
#line 45 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 5402 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestAsteroid* serializable_gee_collections_test_asteroid_new_with_size (gint size) {
#line 45 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_asteroid_construct_with_size (SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_ASTEROID, size);
#line 5409 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_asteroid_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestAsteroid * self;
	gchar* result = NULL;
	gint _tmp0_ = 0;
	gchar* _tmp1_ = NULL;
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestAsteroid*) base;
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_size;
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup_printf ("%i", _tmp0_);
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp1_;
#line 46 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 5428 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_asteroid_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestAsteroid * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 47 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestAsteroid*) base;
#line 47 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = g_strdup ("asteroid");
#line 47 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 47 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 5444 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestAsteroid* serializable_gee_collections_test_asteroid_construct (GType object_type) {
	SerializableGeeCollectionsTestAsteroid * self = NULL;
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestAsteroid*) gxml_serializable_object_model_construct (object_type);
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 5454 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestAsteroid* serializable_gee_collections_test_asteroid_new (void) {
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_asteroid_construct (SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_ASTEROID);
#line 5461 "SerializableGeeCollectionsTest.c"
}


static gint serializable_gee_collections_test_asteroid_get_size (SerializableGeeCollectionsTestAsteroid* self) {
	gint result;
	gint _tmp0_ = 0;
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_size;
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 5476 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_asteroid_set_size (SerializableGeeCollectionsTestAsteroid* self, gint value) {
	gint _tmp0_ = 0;
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_size = _tmp0_;
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "size");
#line 5490 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestAsteroidArray* serializable_gee_collections_test_asteroid_array_construct (GType object_type) {
	SerializableGeeCollectionsTestAsteroidArray * self = NULL;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestAsteroidArray*) gxml_serializable_array_list_construct (object_type, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_ASTEROID, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 5500 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestAsteroidArray* serializable_gee_collections_test_asteroid_array_new (void) {
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_asteroid_array_construct (SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_TYPE_ARRAY);
#line 5507 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_asteroid_array_class_init (SerializableGeeCollectionsTestAsteroidArrayClass * klass) {
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_asteroid_array_parent_class = g_type_class_peek_parent (klass);
#line 5514 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_asteroid_array_instance_init (SerializableGeeCollectionsTestAsteroidArray * self) {
}


GType serializable_gee_collections_test_asteroid_array_get_type (void) {
	static volatile gsize serializable_gee_collections_test_asteroid_array_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_asteroid_array_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestAsteroidArrayClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_asteroid_array_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTestAsteroidArray), 0, (GInstanceInitFunc) serializable_gee_collections_test_asteroid_array_instance_init, NULL };
		GType serializable_gee_collections_test_asteroid_array_type_id;
		serializable_gee_collections_test_asteroid_array_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_ARRAY_LIST, "SerializableGeeCollectionsTestAsteroidArray", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_test_asteroid_array_type_id__volatile, serializable_gee_collections_test_asteroid_array_type_id);
	}
	return serializable_gee_collections_test_asteroid_array_type_id__volatile;
}


static void serializable_gee_collections_test_asteroid_class_init (SerializableGeeCollectionsTestAsteroidClass * klass) {
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_asteroid_parent_class = g_type_class_peek_parent (klass);
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTestAsteroidPrivate));
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_asteroid_real_to_string;
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_asteroid_real_node_name;
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_test_asteroid_get_property;
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_test_asteroid_set_property;
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_test_asteroid_finalize;
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_SIZE, g_param_spec_int ("size", "size", "size", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 5551 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_asteroid_instance_init (SerializableGeeCollectionsTestAsteroid * self) {
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_GET_PRIVATE (self);
#line 5558 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_asteroid_finalize (GObject* obj) {
	SerializableGeeCollectionsTestAsteroid * self;
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_ASTEROID, SerializableGeeCollectionsTestAsteroid);
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (serializable_gee_collections_test_asteroid_parent_class)->finalize (obj);
#line 5568 "SerializableGeeCollectionsTest.c"
}


static GType serializable_gee_collections_test_asteroid_get_type (void) {
	static volatile gsize serializable_gee_collections_test_asteroid_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_asteroid_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestAsteroidClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_asteroid_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTestAsteroid), 0, (GInstanceInitFunc) serializable_gee_collections_test_asteroid_instance_init, NULL };
		GType serializable_gee_collections_test_asteroid_type_id;
		serializable_gee_collections_test_asteroid_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeCollectionsTestAsteroid", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_test_asteroid_type_id__volatile, serializable_gee_collections_test_asteroid_type_id);
	}
	return serializable_gee_collections_test_asteroid_type_id__volatile;
}


static void _vala_serializable_gee_collections_test_asteroid_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestAsteroid * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_ASTEROID, SerializableGeeCollectionsTestAsteroid);
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_SIZE:
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_int (value, serializable_gee_collections_test_asteroid_get_size (self));
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 5595 "SerializableGeeCollectionsTest.c"
		default:
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 5601 "SerializableGeeCollectionsTest.c"
	}
}


static void _vala_serializable_gee_collections_test_asteroid_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestAsteroid * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_ASTEROID, SerializableGeeCollectionsTestAsteroid);
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_SIZE:
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_asteroid_set_size (self, g_value_get_int (value));
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 5617 "SerializableGeeCollectionsTest.c"
		default:
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 5623 "SerializableGeeCollectionsTest.c"
	}
}


static gchar* serializable_gee_collections_test_planet_real_get_map_key (GXmlSerializableMapKey* base) {
	SerializableGeeCollectionsTestPlanet * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestPlanet*) base;
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_name;
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp1_;
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 5643 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestPlanet* serializable_gee_collections_test_planet_construct_named (GType object_type, const gchar* name) {
	SerializableGeeCollectionsTestPlanet * self = NULL;
	const gchar* _tmp0_ = NULL;
#line 56 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 56 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestPlanet*) gxml_serializable_container_construct (object_type);
#line 56 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = name;
#line 56 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_planet_set_name (self, _tmp0_);
#line 56 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 5660 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestPlanet* serializable_gee_collections_test_planet_new_named (const gchar* name) {
#line 56 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_planet_construct_named (SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_PLANET, name);
#line 5667 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_planet_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestPlanet * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestPlanet*) base;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = g_strdup ("planet");
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 57 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 5683 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_planet_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestPlanet * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestPlanet*) base;
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_name;
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp1_;
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 5702 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_planet_real_init_containers (GXmlSerializableContainer* base) {
	SerializableGeeCollectionsTestPlanet * self;
	SerializableGeeCollectionsTestCitizenArray* _tmp0_ = NULL;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestPlanet*) base;
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_citizens;
#line 62 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	if (_tmp0_ == NULL) {
#line 5715 "SerializableGeeCollectionsTest.c"
		SerializableGeeCollectionsTestCitizenArray* _tmp1_ = NULL;
		SerializableGeeCollectionsTestCitizenArray* _tmp2_ = NULL;
#line 63 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp1_ = serializable_gee_collections_test_citizen_array_new ();
#line 63 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp2_ = _tmp1_;
#line 63 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_planet_set_citizens (self, _tmp2_);
#line 63 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp2_);
#line 5726 "SerializableGeeCollectionsTest.c"
	}
}


static SerializableGeeCollectionsTestPlanet* serializable_gee_collections_test_planet_construct (GType object_type) {
	SerializableGeeCollectionsTestPlanet * self = NULL;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestPlanet*) gxml_serializable_container_construct (object_type);
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 5737 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestPlanet* serializable_gee_collections_test_planet_new (void) {
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_planet_construct (SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_PLANET);
#line 5744 "SerializableGeeCollectionsTest.c"
}


static const gchar* serializable_gee_collections_test_planet_get_name (SerializableGeeCollectionsTestPlanet* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_name;
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 5759 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_planet_set_name (SerializableGeeCollectionsTestPlanet* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_name);
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_name = _tmp1_;
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 5778 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestCitizenArray* serializable_gee_collections_test_planet_get_citizens (SerializableGeeCollectionsTestPlanet* self) {
	SerializableGeeCollectionsTestCitizenArray* result;
	SerializableGeeCollectionsTestCitizenArray* _tmp0_ = NULL;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_citizens;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 5793 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_planet_set_citizens (SerializableGeeCollectionsTestPlanet* self, SerializableGeeCollectionsTestCitizenArray* value) {
	SerializableGeeCollectionsTestCitizenArray* _tmp0_ = NULL;
	SerializableGeeCollectionsTestCitizenArray* _tmp1_ = NULL;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_citizens);
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_citizens = _tmp1_;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "citizens");
#line 5812 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestPlanetCollection* serializable_gee_collections_test_planet_collection_construct (GType object_type) {
	SerializableGeeCollectionsTestPlanetCollection * self = NULL;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestPlanetCollection*) gxml_serializable_tree_map_construct (object_type, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_PLANET, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 5822 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestPlanetCollection* serializable_gee_collections_test_planet_collection_new (void) {
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_planet_collection_construct (SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_TYPE_COLLECTION);
#line 5829 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_planet_collection_class_init (SerializableGeeCollectionsTestPlanetCollectionClass * klass) {
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_planet_collection_parent_class = g_type_class_peek_parent (klass);
#line 5836 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_planet_collection_instance_init (SerializableGeeCollectionsTestPlanetCollection * self) {
}


GType serializable_gee_collections_test_planet_collection_get_type (void) {
	static volatile gsize serializable_gee_collections_test_planet_collection_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_planet_collection_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestPlanetCollectionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_planet_collection_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTestPlanetCollection), 0, (GInstanceInitFunc) serializable_gee_collections_test_planet_collection_instance_init, NULL };
		GType serializable_gee_collections_test_planet_collection_type_id;
		serializable_gee_collections_test_planet_collection_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_TREE_MAP, "SerializableGeeCollectionsTestPlanetCollection", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_test_planet_collection_type_id__volatile, serializable_gee_collections_test_planet_collection_type_id);
	}
	return serializable_gee_collections_test_planet_collection_type_id__volatile;
}


static void serializable_gee_collections_test_planet_class_init (SerializableGeeCollectionsTestPlanetClass * klass) {
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_planet_parent_class = g_type_class_peek_parent (klass);
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTestPlanetPrivate));
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_planet_real_node_name;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_planet_real_to_string;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableContainerClass *) klass)->init_containers = (void (*)(GXmlSerializableContainer*)) serializable_gee_collections_test_planet_real_init_containers;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_test_planet_get_property;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_test_planet_set_property;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_test_planet_finalize;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_CITIZENS, g_param_spec_object ("citizens", "citizens", "citizens", SERIALIZABLE_GEE_COLLECTIONS_TEST_CITIZEN_TYPE_ARRAY, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 5877 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_planet_gxml_serializable_map_key_interface_init (GXmlSerializableMapKeyIface * iface) {
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_planet_gxml_serializable_map_key_parent_iface = g_type_interface_peek_parent (iface);
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	iface->get_map_key = (gpointer (*)(GXmlSerializableMapKey*)) serializable_gee_collections_test_planet_real_get_map_key;
#line 5886 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_planet_instance_init (SerializableGeeCollectionsTestPlanet * self) {
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_GET_PRIVATE (self);
#line 5893 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_planet_finalize (GObject* obj) {
	SerializableGeeCollectionsTestPlanet * self;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_PLANET, SerializableGeeCollectionsTestPlanet);
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_name);
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_citizens);
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (serializable_gee_collections_test_planet_parent_class)->finalize (obj);
#line 5907 "SerializableGeeCollectionsTest.c"
}


static GType serializable_gee_collections_test_planet_get_type (void) {
	static volatile gsize serializable_gee_collections_test_planet_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_planet_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestPlanetClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_planet_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTestPlanet), 0, (GInstanceInitFunc) serializable_gee_collections_test_planet_instance_init, NULL };
		static const GInterfaceInfo gxml_serializable_map_key_info = { (GInterfaceInitFunc) serializable_gee_collections_test_planet_gxml_serializable_map_key_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType serializable_gee_collections_test_planet_type_id;
		serializable_gee_collections_test_planet_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_CONTAINER, "SerializableGeeCollectionsTestPlanet", &g_define_type_info, 0);
		g_type_add_interface_static (serializable_gee_collections_test_planet_type_id, GXML_TYPE_SERIALIZABLE_MAP_KEY, &gxml_serializable_map_key_info);
		g_once_init_leave (&serializable_gee_collections_test_planet_type_id__volatile, serializable_gee_collections_test_planet_type_id);
	}
	return serializable_gee_collections_test_planet_type_id__volatile;
}


static void _vala_serializable_gee_collections_test_planet_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestPlanet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_PLANET, SerializableGeeCollectionsTestPlanet);
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_NAME:
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_string (value, serializable_gee_collections_test_planet_get_name (self));
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_CITIZENS:
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_object (value, serializable_gee_collections_test_planet_get_citizens (self));
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 5942 "SerializableGeeCollectionsTest.c"
		default:
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 5948 "SerializableGeeCollectionsTest.c"
	}
}


static void _vala_serializable_gee_collections_test_planet_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestPlanet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_PLANET, SerializableGeeCollectionsTestPlanet);
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_NAME:
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_planet_set_name (self, g_value_get_string (value));
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_CITIZENS:
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_planet_set_citizens (self, g_value_get_object (value));
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 5970 "SerializableGeeCollectionsTest.c"
		default:
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 5976 "SerializableGeeCollectionsTest.c"
	}
}


static gchar* serializable_gee_collections_test_space_real_get_map_key (GXmlSerializableMapKey* base) {
	SerializableGeeCollectionsTestSpace * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSpace*) base;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_name;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp1_;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 5996 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestSpace* serializable_gee_collections_test_space_construct_named (GType object_type, const gchar* name) {
	SerializableGeeCollectionsTestSpace * self = NULL;
	const gchar* _tmp0_ = NULL;
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSpace*) gxml_serializable_container_construct (object_type);
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = name;
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_space_set_name (self, _tmp0_);
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 6013 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestSpace* serializable_gee_collections_test_space_new_named (const gchar* name) {
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_space_construct_named (SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE, name);
#line 6020 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_space_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestSpace * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSpace*) base;
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = g_strdup ("space");
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 6036 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_space_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestSpace * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSpace*) base;
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_name;
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp1_;
#line 78 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 6055 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_real_init_containers (GXmlSerializableContainer* base) {
	SerializableGeeCollectionsTestSpace * self;
	SerializableGeeCollectionsTestAsteroidArray* _tmp0_ = NULL;
	SerializableGeeCollectionsTestPlanetCollection* _tmp3_ = NULL;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSpace*) base;
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_asteroids;
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	if (_tmp0_ == NULL) {
#line 6069 "SerializableGeeCollectionsTest.c"
		SerializableGeeCollectionsTestAsteroidArray* _tmp1_ = NULL;
		SerializableGeeCollectionsTestAsteroidArray* _tmp2_ = NULL;
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp1_ = serializable_gee_collections_test_asteroid_array_new ();
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp2_ = _tmp1_;
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_space_set_asteroids (self, _tmp2_);
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp2_);
#line 6080 "SerializableGeeCollectionsTest.c"
	}
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp3_ = self->priv->_planets;
#line 83 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	if (_tmp3_ == NULL) {
#line 6086 "SerializableGeeCollectionsTest.c"
		SerializableGeeCollectionsTestPlanetCollection* _tmp4_ = NULL;
		SerializableGeeCollectionsTestPlanetCollection* _tmp5_ = NULL;
#line 84 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp4_ = serializable_gee_collections_test_planet_collection_new ();
#line 84 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp5_ = _tmp4_;
#line 84 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_space_set_planets (self, _tmp5_);
#line 84 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp5_);
#line 6097 "SerializableGeeCollectionsTest.c"
	}
}


static SerializableGeeCollectionsTestSpace* serializable_gee_collections_test_space_construct (GType object_type) {
	SerializableGeeCollectionsTestSpace * self = NULL;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSpace*) gxml_serializable_container_construct (object_type);
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 6108 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestSpace* serializable_gee_collections_test_space_new (void) {
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_space_construct (SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE);
#line 6115 "SerializableGeeCollectionsTest.c"
}


static const gchar* serializable_gee_collections_test_space_get_name (SerializableGeeCollectionsTestSpace* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_name;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 6130 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_set_name (SerializableGeeCollectionsTestSpace* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_name);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_name = _tmp1_;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 6149 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestPlanetCollection* serializable_gee_collections_test_space_get_planets (SerializableGeeCollectionsTestSpace* self) {
	SerializableGeeCollectionsTestPlanetCollection* result;
	SerializableGeeCollectionsTestPlanetCollection* _tmp0_ = NULL;
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_planets;
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 6164 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_set_planets (SerializableGeeCollectionsTestSpace* self, SerializableGeeCollectionsTestPlanetCollection* value) {
	SerializableGeeCollectionsTestPlanetCollection* _tmp0_ = NULL;
	SerializableGeeCollectionsTestPlanetCollection* _tmp1_ = NULL;
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_planets);
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_planets = _tmp1_;
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "planets");
#line 6183 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestAsteroidArray* serializable_gee_collections_test_space_get_asteroids (SerializableGeeCollectionsTestSpace* self) {
	SerializableGeeCollectionsTestAsteroidArray* result;
	SerializableGeeCollectionsTestAsteroidArray* _tmp0_ = NULL;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_asteroids;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 6198 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_set_asteroids (SerializableGeeCollectionsTestSpace* self, SerializableGeeCollectionsTestAsteroidArray* value) {
	SerializableGeeCollectionsTestAsteroidArray* _tmp0_ = NULL;
	SerializableGeeCollectionsTestAsteroidArray* _tmp1_ = NULL;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_asteroids);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_asteroids = _tmp1_;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "asteroids");
#line 6217 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestSpaceCollection* serializable_gee_collections_test_space_collection_construct (GType object_type) {
	SerializableGeeCollectionsTestSpaceCollection * self = NULL;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSpaceCollection*) gxml_serializable_tree_map_construct (object_type, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 6227 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestSpaceCollection* serializable_gee_collections_test_space_collection_new (void) {
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_space_collection_construct (SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_TYPE_COLLECTION);
#line 6234 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_collection_class_init (SerializableGeeCollectionsTestSpaceCollectionClass * klass) {
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_space_collection_parent_class = g_type_class_peek_parent (klass);
#line 6241 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_collection_instance_init (SerializableGeeCollectionsTestSpaceCollection * self) {
}


GType serializable_gee_collections_test_space_collection_get_type (void) {
	static volatile gsize serializable_gee_collections_test_space_collection_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_space_collection_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestSpaceCollectionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_space_collection_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTestSpaceCollection), 0, (GInstanceInitFunc) serializable_gee_collections_test_space_collection_instance_init, NULL };
		GType serializable_gee_collections_test_space_collection_type_id;
		serializable_gee_collections_test_space_collection_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_TREE_MAP, "SerializableGeeCollectionsTestSpaceCollection", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_test_space_collection_type_id__volatile, serializable_gee_collections_test_space_collection_type_id);
	}
	return serializable_gee_collections_test_space_collection_type_id__volatile;
}


static void serializable_gee_collections_test_space_class_init (SerializableGeeCollectionsTestSpaceClass * klass) {
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_space_parent_class = g_type_class_peek_parent (klass);
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTestSpacePrivate));
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_space_real_node_name;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_space_real_to_string;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableContainerClass *) klass)->init_containers = (void (*)(GXmlSerializableContainer*)) serializable_gee_collections_test_space_real_init_containers;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_test_space_get_property;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_test_space_set_property;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_test_space_finalize;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_PLANETS, g_param_spec_object ("planets", "planets", "planets", SERIALIZABLE_GEE_COLLECTIONS_TEST_PLANET_TYPE_COLLECTION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_ASTEROIDS, g_param_spec_object ("asteroids", "asteroids", "asteroids", SERIALIZABLE_GEE_COLLECTIONS_TEST_ASTEROID_TYPE_ARRAY, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 6284 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_gxml_serializable_map_key_interface_init (GXmlSerializableMapKeyIface * iface) {
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_space_gxml_serializable_map_key_parent_iface = g_type_interface_peek_parent (iface);
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	iface->get_map_key = (gpointer (*)(GXmlSerializableMapKey*)) serializable_gee_collections_test_space_real_get_map_key;
#line 6293 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_instance_init (SerializableGeeCollectionsTestSpace * self) {
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_GET_PRIVATE (self);
#line 6300 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_finalize (GObject* obj) {
	SerializableGeeCollectionsTestSpace * self;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE, SerializableGeeCollectionsTestSpace);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_name);
#line 72 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_planets);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_asteroids);
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (serializable_gee_collections_test_space_parent_class)->finalize (obj);
#line 6316 "SerializableGeeCollectionsTest.c"
}


static GType serializable_gee_collections_test_space_get_type (void) {
	static volatile gsize serializable_gee_collections_test_space_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_space_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestSpaceClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_space_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTestSpace), 0, (GInstanceInitFunc) serializable_gee_collections_test_space_instance_init, NULL };
		static const GInterfaceInfo gxml_serializable_map_key_info = { (GInterfaceInitFunc) serializable_gee_collections_test_space_gxml_serializable_map_key_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType serializable_gee_collections_test_space_type_id;
		serializable_gee_collections_test_space_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_CONTAINER, "SerializableGeeCollectionsTestSpace", &g_define_type_info, 0);
		g_type_add_interface_static (serializable_gee_collections_test_space_type_id, GXML_TYPE_SERIALIZABLE_MAP_KEY, &gxml_serializable_map_key_info);
		g_once_init_leave (&serializable_gee_collections_test_space_type_id__volatile, serializable_gee_collections_test_space_type_id);
	}
	return serializable_gee_collections_test_space_type_id__volatile;
}


static void _vala_serializable_gee_collections_test_space_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestSpace * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE, SerializableGeeCollectionsTestSpace);
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_NAME:
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_string (value, serializable_gee_collections_test_space_get_name (self));
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_PLANETS:
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_object (value, serializable_gee_collections_test_space_get_planets (self));
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_ASTEROIDS:
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_object (value, serializable_gee_collections_test_space_get_asteroids (self));
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 6357 "SerializableGeeCollectionsTest.c"
		default:
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 6363 "SerializableGeeCollectionsTest.c"
	}
}


static void _vala_serializable_gee_collections_test_space_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestSpace * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE, SerializableGeeCollectionsTestSpace);
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_NAME:
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_space_set_name (self, g_value_get_string (value));
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_PLANETS:
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_space_set_planets (self, g_value_get_object (value));
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_ASTEROIDS:
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_space_set_asteroids (self, g_value_get_object (value));
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 6391 "SerializableGeeCollectionsTest.c"
		default:
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 6397 "SerializableGeeCollectionsTest.c"
	}
}


static SerializableGeeCollectionsTestRefaction* serializable_gee_collections_test_refaction_construct_named (GType object_type, const gchar* manufacturer, const gchar* model) {
	SerializableGeeCollectionsTestRefaction * self = NULL;
	const gchar* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
#line 94 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (manufacturer != NULL, NULL);
#line 94 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (model != NULL, NULL);
#line 94 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestRefaction*) gxml_serializable_container_construct (object_type);
#line 96 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = manufacturer;
#line 96 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_refaction_set_manufacturer (self, _tmp0_);
#line 97 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = model;
#line 97 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_refaction_set_model (self, _tmp1_);
#line 94 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 6422 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestRefaction* serializable_gee_collections_test_refaction_new_named (const gchar* manufacturer, const gchar* model) {
#line 94 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_refaction_construct_named (SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_REFACTION, manufacturer, model);
#line 6429 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_refaction_real_get_map_primary_key (GXmlSerializableMapDualKey* base) {
	SerializableGeeCollectionsTestRefaction * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestRefaction*) base;
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_manufacturer;
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp1_;
#line 100 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 6448 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_refaction_real_get_map_secondary_key (GXmlSerializableMapDualKey* base) {
	SerializableGeeCollectionsTestRefaction * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 101 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestRefaction*) base;
#line 101 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_model;
#line 101 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 101 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp1_;
#line 101 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 6467 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_refaction_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestRefaction * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 103 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestRefaction*) base;
#line 103 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = g_strdup ("refaction");
#line 103 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 103 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 6483 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_refaction_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestRefaction * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestRefaction*) base;
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_model;
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp1_;
#line 104 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 6502 "SerializableGeeCollectionsTest.c"
}


static gboolean serializable_gee_collections_test_refaction_real_get_enable_unknown_serializable_property (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestRefaction * self;
	gboolean result = FALSE;
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestRefaction*) base;
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = TRUE;
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 6515 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_refaction_real_init_containers (GXmlSerializableContainer* base) {
	SerializableGeeCollectionsTestRefaction * self;
	SerializableGeeCollectionsTestSpaceShipCollection* _tmp0_ = NULL;
#line 107 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestRefaction*) base;
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_spaceships;
#line 109 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	if (_tmp0_ == NULL) {
#line 6528 "SerializableGeeCollectionsTest.c"
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp1_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp2_ = NULL;
#line 110 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp1_ = serializable_gee_collections_test_space_ship_collection_new ();
#line 110 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp2_ = _tmp1_;
#line 110 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_refaction_set_spaceships (self, _tmp2_);
#line 110 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp2_);
#line 6539 "SerializableGeeCollectionsTest.c"
	}
}


static SerializableGeeCollectionsTestRefaction* serializable_gee_collections_test_refaction_construct (GType object_type) {
	SerializableGeeCollectionsTestRefaction * self = NULL;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestRefaction*) gxml_serializable_container_construct (object_type);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 6550 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestRefaction* serializable_gee_collections_test_refaction_new (void) {
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_refaction_construct (SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_REFACTION);
#line 6557 "SerializableGeeCollectionsTest.c"
}


static const gchar* serializable_gee_collections_test_refaction_get_model (SerializableGeeCollectionsTestRefaction* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_model;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 6572 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_refaction_set_model (SerializableGeeCollectionsTestRefaction* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_model);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_model = _tmp1_;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "model");
#line 6591 "SerializableGeeCollectionsTest.c"
}


static const gchar* serializable_gee_collections_test_refaction_get_manufacturer (SerializableGeeCollectionsTestRefaction* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_manufacturer;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 6606 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_refaction_set_manufacturer (SerializableGeeCollectionsTestRefaction* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_manufacturer);
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_manufacturer = _tmp1_;
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "manufacturer");
#line 6625 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestSpaceShipCollection* serializable_gee_collections_test_refaction_get_spaceships (SerializableGeeCollectionsTestRefaction* self) {
	SerializableGeeCollectionsTestSpaceShipCollection* result;
	SerializableGeeCollectionsTestSpaceShipCollection* _tmp0_ = NULL;
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_spaceships;
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 6640 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_refaction_set_spaceships (SerializableGeeCollectionsTestRefaction* self, SerializableGeeCollectionsTestSpaceShipCollection* value) {
	SerializableGeeCollectionsTestSpaceShipCollection* _tmp0_ = NULL;
	SerializableGeeCollectionsTestSpaceShipCollection* _tmp1_ = NULL;
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_spaceships);
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_spaceships = _tmp1_;
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "spaceships");
#line 6659 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestRefactionCollection* serializable_gee_collections_test_refaction_collection_construct (GType object_type) {
	SerializableGeeCollectionsTestRefactionCollection * self = NULL;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestRefactionCollection*) gxml_serializable_dual_key_map_construct (object_type, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_REFACTION, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 6669 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestRefactionCollection* serializable_gee_collections_test_refaction_collection_new (void) {
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_refaction_collection_construct (SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_TYPE_COLLECTION);
#line 6676 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_refaction_collection_class_init (SerializableGeeCollectionsTestRefactionCollectionClass * klass) {
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_refaction_collection_parent_class = g_type_class_peek_parent (klass);
#line 6683 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_refaction_collection_instance_init (SerializableGeeCollectionsTestRefactionCollection * self) {
}


GType serializable_gee_collections_test_refaction_collection_get_type (void) {
	static volatile gsize serializable_gee_collections_test_refaction_collection_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_refaction_collection_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestRefactionCollectionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_refaction_collection_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTestRefactionCollection), 0, (GInstanceInitFunc) serializable_gee_collections_test_refaction_collection_instance_init, NULL };
		GType serializable_gee_collections_test_refaction_collection_type_id;
		serializable_gee_collections_test_refaction_collection_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_DUAL_KEY_MAP, "SerializableGeeCollectionsTestRefactionCollection", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_test_refaction_collection_type_id__volatile, serializable_gee_collections_test_refaction_collection_type_id);
	}
	return serializable_gee_collections_test_refaction_collection_type_id__volatile;
}


static void serializable_gee_collections_test_refaction_class_init (SerializableGeeCollectionsTestRefactionClass * klass) {
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_refaction_parent_class = g_type_class_peek_parent (klass);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTestRefactionPrivate));
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_refaction_real_node_name;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_refaction_real_to_string;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->get_enable_unknown_serializable_property = (gboolean (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_refaction_real_get_enable_unknown_serializable_property;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableContainerClass *) klass)->init_containers = (void (*)(GXmlSerializableContainer*)) serializable_gee_collections_test_refaction_real_init_containers;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_test_refaction_get_property;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_test_refaction_set_property;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_test_refaction_finalize;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_MODEL, g_param_spec_string ("model", "model", "model", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_MANUFACTURER, g_param_spec_string ("manufacturer", "manufacturer", "manufacturer", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_SPACESHIPS, g_param_spec_object ("spaceships", "spaceships", "spaceships", SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_TYPE_COLLECTION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 6728 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_refaction_gxml_serializable_map_dual_key_interface_init (GXmlSerializableMapDualKeyIface * iface) {
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_refaction_gxml_serializable_map_dual_key_parent_iface = g_type_interface_peek_parent (iface);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	iface->get_map_primary_key = (gpointer (*)(GXmlSerializableMapDualKey*)) serializable_gee_collections_test_refaction_real_get_map_primary_key;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	iface->get_map_secondary_key = (gpointer (*)(GXmlSerializableMapDualKey*)) serializable_gee_collections_test_refaction_real_get_map_secondary_key;
#line 6739 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_refaction_instance_init (SerializableGeeCollectionsTestRefaction * self) {
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_GET_PRIVATE (self);
#line 6746 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_refaction_finalize (GObject* obj) {
	SerializableGeeCollectionsTestRefaction * self;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_REFACTION, SerializableGeeCollectionsTestRefaction);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_model);
#line 91 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_manufacturer);
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_spaceships);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (serializable_gee_collections_test_refaction_parent_class)->finalize (obj);
#line 6762 "SerializableGeeCollectionsTest.c"
}


static GType serializable_gee_collections_test_refaction_get_type (void) {
	static volatile gsize serializable_gee_collections_test_refaction_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_refaction_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestRefactionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_refaction_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTestRefaction), 0, (GInstanceInitFunc) serializable_gee_collections_test_refaction_instance_init, NULL };
		static const GInterfaceInfo gxml_serializable_map_dual_key_info = { (GInterfaceInitFunc) serializable_gee_collections_test_refaction_gxml_serializable_map_dual_key_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType serializable_gee_collections_test_refaction_type_id;
		serializable_gee_collections_test_refaction_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_CONTAINER, "SerializableGeeCollectionsTestRefaction", &g_define_type_info, 0);
		g_type_add_interface_static (serializable_gee_collections_test_refaction_type_id, GXML_TYPE_SERIALIZABLE_MAP_DUAL_KEY, &gxml_serializable_map_dual_key_info);
		g_once_init_leave (&serializable_gee_collections_test_refaction_type_id__volatile, serializable_gee_collections_test_refaction_type_id);
	}
	return serializable_gee_collections_test_refaction_type_id__volatile;
}


static void _vala_serializable_gee_collections_test_refaction_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestRefaction * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_REFACTION, SerializableGeeCollectionsTestRefaction);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_MODEL:
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_string (value, serializable_gee_collections_test_refaction_get_model (self));
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_MANUFACTURER:
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_string (value, serializable_gee_collections_test_refaction_get_manufacturer (self));
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_SPACESHIPS:
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_object (value, serializable_gee_collections_test_refaction_get_spaceships (self));
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 6803 "SerializableGeeCollectionsTest.c"
		default:
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 6809 "SerializableGeeCollectionsTest.c"
	}
}


static void _vala_serializable_gee_collections_test_refaction_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestRefaction * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_REFACTION, SerializableGeeCollectionsTestRefaction);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_MODEL:
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_refaction_set_model (self, g_value_get_string (value));
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_MANUFACTURER:
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_refaction_set_manufacturer (self, g_value_get_string (value));
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_SPACESHIPS:
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_refaction_set_spaceships (self, g_value_get_object (value));
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 6837 "SerializableGeeCollectionsTest.c"
		default:
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 6843 "SerializableGeeCollectionsTest.c"
	}
}


static SerializableGeeCollectionsTestSpaceShip* serializable_gee_collections_test_space_ship_construct_named (GType object_type, const gchar* manufacturer, const gchar* model) {
	SerializableGeeCollectionsTestSpaceShip * self = NULL;
	const gchar* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (manufacturer != NULL, NULL);
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (model != NULL, NULL);
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSpaceShip*) gxml_serializable_container_construct (object_type);
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = manufacturer;
#line 123 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_space_ship_set_manufacturer (self, _tmp0_);
#line 124 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = model;
#line 124 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_space_ship_set_model (self, _tmp1_);
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 6868 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestSpaceShip* serializable_gee_collections_test_space_ship_new_named (const gchar* manufacturer, const gchar* model) {
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_space_ship_construct_named (SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_SHIP, manufacturer, model);
#line 6875 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_space_ship_real_get_map_primary_key (GXmlSerializableMapDualKey* base) {
	SerializableGeeCollectionsTestSpaceShip * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSpaceShip*) base;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_manufacturer;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp1_;
#line 127 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 6894 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_space_ship_real_get_map_secondary_key (GXmlSerializableMapDualKey* base) {
	SerializableGeeCollectionsTestSpaceShip * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSpaceShip*) base;
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_model;
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp1_;
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 6913 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_space_ship_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestSpaceShip * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSpaceShip*) base;
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = g_strdup ("ship");
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 6929 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_space_ship_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestSpaceShip * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 131 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSpaceShip*) base;
#line 131 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_model;
#line 131 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 131 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp1_;
#line 131 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 6948 "SerializableGeeCollectionsTest.c"
}


static gboolean serializable_gee_collections_test_space_ship_real_get_enable_unknown_serializable_property (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestSpaceShip * self;
	gboolean result = FALSE;
#line 132 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSpaceShip*) base;
#line 132 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = TRUE;
#line 132 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 6961 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_ship_real_init_containers (GXmlSerializableContainer* base) {
	SerializableGeeCollectionsTestSpaceShip * self;
	SerializableGeeCollectionsTestSpaceCollection* _tmp0_ = NULL;
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSpaceShip*) base;
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_spaces;
#line 136 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	if (_tmp0_ == NULL) {
#line 6974 "SerializableGeeCollectionsTest.c"
		SerializableGeeCollectionsTestSpaceCollection* _tmp1_ = NULL;
		SerializableGeeCollectionsTestSpaceCollection* _tmp2_ = NULL;
#line 137 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp1_ = serializable_gee_collections_test_space_collection_new ();
#line 137 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp2_ = _tmp1_;
#line 137 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_space_ship_set_spaces (self, _tmp2_);
#line 137 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp2_);
#line 6985 "SerializableGeeCollectionsTest.c"
	}
}


static SerializableGeeCollectionsTestSpaceShip* serializable_gee_collections_test_space_ship_construct (GType object_type) {
	SerializableGeeCollectionsTestSpaceShip * self = NULL;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSpaceShip*) gxml_serializable_container_construct (object_type);
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 6996 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestSpaceShip* serializable_gee_collections_test_space_ship_new (void) {
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_space_ship_construct (SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_SHIP);
#line 7003 "SerializableGeeCollectionsTest.c"
}


static const gchar* serializable_gee_collections_test_space_ship_get_model (SerializableGeeCollectionsTestSpaceShip* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_model;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 7018 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_ship_set_model (SerializableGeeCollectionsTestSpaceShip* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_model);
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_model = _tmp1_;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "model");
#line 7037 "SerializableGeeCollectionsTest.c"
}


static const gchar* serializable_gee_collections_test_space_ship_get_manufacturer (SerializableGeeCollectionsTestSpaceShip* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 118 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 118 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_manufacturer;
#line 118 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 118 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 7052 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_ship_set_manufacturer (SerializableGeeCollectionsTestSpaceShip* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 118 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 118 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 118 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 118 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_manufacturer);
#line 118 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_manufacturer = _tmp1_;
#line 118 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "manufacturer");
#line 7071 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestSpaceCollection* serializable_gee_collections_test_space_ship_get_spaces (SerializableGeeCollectionsTestSpaceShip* self) {
	SerializableGeeCollectionsTestSpaceCollection* result;
	SerializableGeeCollectionsTestSpaceCollection* _tmp0_ = NULL;
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_spaces;
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 7086 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_ship_set_spaces (SerializableGeeCollectionsTestSpaceShip* self, SerializableGeeCollectionsTestSpaceCollection* value) {
	SerializableGeeCollectionsTestSpaceCollection* _tmp0_ = NULL;
	SerializableGeeCollectionsTestSpaceCollection* _tmp1_ = NULL;
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_spaces);
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_spaces = _tmp1_;
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "spaces");
#line 7105 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestSpaceShipCollection* serializable_gee_collections_test_space_ship_collection_construct (GType object_type) {
	SerializableGeeCollectionsTestSpaceShipCollection * self = NULL;
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSpaceShipCollection*) gxml_serializable_dual_key_map_construct (object_type, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_SHIP, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 7115 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestSpaceShipCollection* serializable_gee_collections_test_space_ship_collection_new (void) {
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_space_ship_collection_construct (SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_TYPE_COLLECTION);
#line 7122 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_ship_collection_class_init (SerializableGeeCollectionsTestSpaceShipCollectionClass * klass) {
#line 140 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_space_ship_collection_parent_class = g_type_class_peek_parent (klass);
#line 7129 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_ship_collection_instance_init (SerializableGeeCollectionsTestSpaceShipCollection * self) {
}


GType serializable_gee_collections_test_space_ship_collection_get_type (void) {
	static volatile gsize serializable_gee_collections_test_space_ship_collection_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_space_ship_collection_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestSpaceShipCollectionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_space_ship_collection_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTestSpaceShipCollection), 0, (GInstanceInitFunc) serializable_gee_collections_test_space_ship_collection_instance_init, NULL };
		GType serializable_gee_collections_test_space_ship_collection_type_id;
		serializable_gee_collections_test_space_ship_collection_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_DUAL_KEY_MAP, "SerializableGeeCollectionsTestSpaceShipCollection", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_test_space_ship_collection_type_id__volatile, serializable_gee_collections_test_space_ship_collection_type_id);
	}
	return serializable_gee_collections_test_space_ship_collection_type_id__volatile;
}


static void serializable_gee_collections_test_space_ship_class_init (SerializableGeeCollectionsTestSpaceShipClass * klass) {
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_space_ship_parent_class = g_type_class_peek_parent (klass);
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTestSpaceShipPrivate));
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_space_ship_real_node_name;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_space_ship_real_to_string;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->get_enable_unknown_serializable_property = (gboolean (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_space_ship_real_get_enable_unknown_serializable_property;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableContainerClass *) klass)->init_containers = (void (*)(GXmlSerializableContainer*)) serializable_gee_collections_test_space_ship_real_init_containers;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_test_space_ship_get_property;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_test_space_ship_set_property;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_test_space_ship_finalize;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_MODEL, g_param_spec_string ("model", "model", "model", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_MANUFACTURER, g_param_spec_string ("manufacturer", "manufacturer", "manufacturer", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_SPACES, g_param_spec_object ("spaces", "spaces", "spaces", SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_TYPE_COLLECTION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 7174 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_ship_gxml_serializable_map_dual_key_interface_init (GXmlSerializableMapDualKeyIface * iface) {
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_space_ship_gxml_serializable_map_dual_key_parent_iface = g_type_interface_peek_parent (iface);
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	iface->get_map_primary_key = (gpointer (*)(GXmlSerializableMapDualKey*)) serializable_gee_collections_test_space_ship_real_get_map_primary_key;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	iface->get_map_secondary_key = (gpointer (*)(GXmlSerializableMapDualKey*)) serializable_gee_collections_test_space_ship_real_get_map_secondary_key;
#line 7185 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_ship_instance_init (SerializableGeeCollectionsTestSpaceShip * self) {
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_GET_PRIVATE (self);
#line 7192 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_ship_finalize (GObject* obj) {
	SerializableGeeCollectionsTestSpaceShip * self;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_SHIP, SerializableGeeCollectionsTestSpaceShip);
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_model);
#line 118 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_manufacturer);
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_spaces);
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (serializable_gee_collections_test_space_ship_parent_class)->finalize (obj);
#line 7208 "SerializableGeeCollectionsTest.c"
}


static GType serializable_gee_collections_test_space_ship_get_type (void) {
	static volatile gsize serializable_gee_collections_test_space_ship_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_space_ship_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestSpaceShipClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_space_ship_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTestSpaceShip), 0, (GInstanceInitFunc) serializable_gee_collections_test_space_ship_instance_init, NULL };
		static const GInterfaceInfo gxml_serializable_map_dual_key_info = { (GInterfaceInitFunc) serializable_gee_collections_test_space_ship_gxml_serializable_map_dual_key_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType serializable_gee_collections_test_space_ship_type_id;
		serializable_gee_collections_test_space_ship_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_CONTAINER, "SerializableGeeCollectionsTestSpaceShip", &g_define_type_info, 0);
		g_type_add_interface_static (serializable_gee_collections_test_space_ship_type_id, GXML_TYPE_SERIALIZABLE_MAP_DUAL_KEY, &gxml_serializable_map_dual_key_info);
		g_once_init_leave (&serializable_gee_collections_test_space_ship_type_id__volatile, serializable_gee_collections_test_space_ship_type_id);
	}
	return serializable_gee_collections_test_space_ship_type_id__volatile;
}


static void _vala_serializable_gee_collections_test_space_ship_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestSpaceShip * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_SHIP, SerializableGeeCollectionsTestSpaceShip);
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_MODEL:
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_string (value, serializable_gee_collections_test_space_ship_get_model (self));
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_MANUFACTURER:
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_string (value, serializable_gee_collections_test_space_ship_get_manufacturer (self));
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_SPACES:
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_object (value, serializable_gee_collections_test_space_ship_get_spaces (self));
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 7249 "SerializableGeeCollectionsTest.c"
		default:
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 7255 "SerializableGeeCollectionsTest.c"
	}
}


static void _vala_serializable_gee_collections_test_space_ship_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestSpaceShip * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_SHIP, SerializableGeeCollectionsTestSpaceShip);
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_MODEL:
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_space_ship_set_model (self, g_value_get_string (value));
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_MANUFACTURER:
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_space_ship_set_manufacturer (self, g_value_get_string (value));
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_SPACES:
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_space_ship_set_spaces (self, g_value_get_object (value));
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 7283 "SerializableGeeCollectionsTest.c"
		default:
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 7289 "SerializableGeeCollectionsTest.c"
	}
}


static gchar* serializable_gee_collections_test_charge_zone_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestChargeZone * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 148 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestChargeZone*) base;
#line 148 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = g_strdup ("ChargeZone");
#line 148 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 148 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 7306 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_charge_zone_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestChargeZone * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestChargeZone*) base;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_name;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp1_;
#line 149 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 7325 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_charge_zone_real_init_containers (GXmlSerializableContainer* base) {
	SerializableGeeCollectionsTestChargeZone * self;
	SerializableGeeCollectionsTestSpaceShipCollection* _tmp0_ = NULL;
#line 150 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestChargeZone*) base;
#line 152 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_spaceships;
#line 152 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	if (_tmp0_ == NULL) {
#line 7338 "SerializableGeeCollectionsTest.c"
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp1_ = NULL;
		SerializableGeeCollectionsTestSpaceShipCollection* _tmp2_ = NULL;
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp1_ = serializable_gee_collections_test_space_ship_collection_new ();
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp2_ = _tmp1_;
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_charge_zone_set_spaceships (self, _tmp2_);
#line 153 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp2_);
#line 7349 "SerializableGeeCollectionsTest.c"
	}
}


static SerializableGeeCollectionsTestChargeZone* serializable_gee_collections_test_charge_zone_construct (GType object_type) {
	SerializableGeeCollectionsTestChargeZone * self = NULL;
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestChargeZone*) gxml_serializable_container_construct (object_type);
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 7360 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestChargeZone* serializable_gee_collections_test_charge_zone_new (void) {
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_charge_zone_construct (SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CHARGE_ZONE);
#line 7367 "SerializableGeeCollectionsTest.c"
}


static const gchar* serializable_gee_collections_test_charge_zone_get_name (SerializableGeeCollectionsTestChargeZone* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_name;
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 7382 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_charge_zone_set_name (SerializableGeeCollectionsTestChargeZone* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_name);
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_name = _tmp1_;
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 7401 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestSpaceShipCollection* serializable_gee_collections_test_charge_zone_get_spaceships (SerializableGeeCollectionsTestChargeZone* self) {
	SerializableGeeCollectionsTestSpaceShipCollection* result;
	SerializableGeeCollectionsTestSpaceShipCollection* _tmp0_ = NULL;
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_spaceships;
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 7416 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_charge_zone_set_spaceships (SerializableGeeCollectionsTestChargeZone* self, SerializableGeeCollectionsTestSpaceShipCollection* value) {
	SerializableGeeCollectionsTestSpaceShipCollection* _tmp0_ = NULL;
	SerializableGeeCollectionsTestSpaceShipCollection* _tmp1_ = NULL;
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_spaceships);
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_spaceships = _tmp1_;
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "spaceships");
#line 7435 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_charge_zone_class_init (SerializableGeeCollectionsTestChargeZoneClass * klass) {
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_charge_zone_parent_class = g_type_class_peek_parent (klass);
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTestChargeZonePrivate));
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_charge_zone_real_node_name;
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_charge_zone_real_to_string;
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableContainerClass *) klass)->init_containers = (void (*)(GXmlSerializableContainer*)) serializable_gee_collections_test_charge_zone_real_init_containers;
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_test_charge_zone_get_property;
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_test_charge_zone_set_property;
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_test_charge_zone_finalize;
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_CHARGE_ZONE_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_CHARGE_ZONE_SPACESHIPS, g_param_spec_object ("spaceships", "spaceships", "spaceships", SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_SHIP_TYPE_COLLECTION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 7460 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_charge_zone_instance_init (SerializableGeeCollectionsTestChargeZone * self) {
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TEST_CHARGE_ZONE_GET_PRIVATE (self);
#line 7467 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_charge_zone_finalize (GObject* obj) {
	SerializableGeeCollectionsTestChargeZone * self;
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CHARGE_ZONE, SerializableGeeCollectionsTestChargeZone);
#line 145 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_name);
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_spaceships);
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (serializable_gee_collections_test_charge_zone_parent_class)->finalize (obj);
#line 7481 "SerializableGeeCollectionsTest.c"
}


static GType serializable_gee_collections_test_charge_zone_get_type (void) {
	static volatile gsize serializable_gee_collections_test_charge_zone_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_charge_zone_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestChargeZoneClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_charge_zone_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTestChargeZone), 0, (GInstanceInitFunc) serializable_gee_collections_test_charge_zone_instance_init, NULL };
		GType serializable_gee_collections_test_charge_zone_type_id;
		serializable_gee_collections_test_charge_zone_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_CONTAINER, "SerializableGeeCollectionsTestChargeZone", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_test_charge_zone_type_id__volatile, serializable_gee_collections_test_charge_zone_type_id);
	}
	return serializable_gee_collections_test_charge_zone_type_id__volatile;
}


static void _vala_serializable_gee_collections_test_charge_zone_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestChargeZone * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CHARGE_ZONE, SerializableGeeCollectionsTestChargeZone);
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_CHARGE_ZONE_NAME:
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_string (value, serializable_gee_collections_test_charge_zone_get_name (self));
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_CHARGE_ZONE_SPACESHIPS:
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_object (value, serializable_gee_collections_test_charge_zone_get_spaceships (self));
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 7514 "SerializableGeeCollectionsTest.c"
		default:
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 7520 "SerializableGeeCollectionsTest.c"
	}
}


static void _vala_serializable_gee_collections_test_charge_zone_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestChargeZone * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CHARGE_ZONE, SerializableGeeCollectionsTestChargeZone);
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_CHARGE_ZONE_NAME:
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_charge_zone_set_name (self, g_value_get_string (value));
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_CHARGE_ZONE_SPACESHIPS:
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_charge_zone_set_spaceships (self, g_value_get_object (value));
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 7542 "SerializableGeeCollectionsTest.c"
		default:
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 7548 "SerializableGeeCollectionsTest.c"
	}
}


static gchar* serializable_gee_collections_test_storage_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestStorage * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 161 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestStorage*) base;
#line 161 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = g_strdup ("storage");
#line 161 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 161 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 7565 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_storage_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestStorage * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestStorage*) base;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_name;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp1_;
#line 162 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 7584 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_storage_real_init_containers (GXmlSerializableContainer* base) {
	SerializableGeeCollectionsTestStorage * self;
	SerializableGeeCollectionsTestRefactionCollection* _tmp0_ = NULL;
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestStorage*) base;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_refactions;
#line 165 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	if (_tmp0_ == NULL) {
#line 7597 "SerializableGeeCollectionsTest.c"
		SerializableGeeCollectionsTestRefactionCollection* _tmp1_ = NULL;
		SerializableGeeCollectionsTestRefactionCollection* _tmp2_ = NULL;
#line 166 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp1_ = serializable_gee_collections_test_refaction_collection_new ();
#line 166 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_tmp2_ = _tmp1_;
#line 166 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_storage_set_refactions (self, _tmp2_);
#line 166 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		_g_object_unref0 (_tmp2_);
#line 7608 "SerializableGeeCollectionsTest.c"
	}
}


static SerializableGeeCollectionsTestStorage* serializable_gee_collections_test_storage_construct (GType object_type) {
	SerializableGeeCollectionsTestStorage * self = NULL;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestStorage*) gxml_serializable_container_construct (object_type);
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 7619 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestStorage* serializable_gee_collections_test_storage_new (void) {
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_storage_construct (SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_STORAGE);
#line 7626 "SerializableGeeCollectionsTest.c"
}


static const gchar* serializable_gee_collections_test_storage_get_name (SerializableGeeCollectionsTestStorage* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_name;
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 7641 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_storage_set_name (SerializableGeeCollectionsTestStorage* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_name);
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_name = _tmp1_;
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 7660 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestRefactionCollection* serializable_gee_collections_test_storage_get_refactions (SerializableGeeCollectionsTestStorage* self) {
	SerializableGeeCollectionsTestRefactionCollection* result;
	SerializableGeeCollectionsTestRefactionCollection* _tmp0_ = NULL;
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_refactions;
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 7675 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_storage_set_refactions (SerializableGeeCollectionsTestStorage* self, SerializableGeeCollectionsTestRefactionCollection* value) {
	SerializableGeeCollectionsTestRefactionCollection* _tmp0_ = NULL;
	SerializableGeeCollectionsTestRefactionCollection* _tmp1_ = NULL;
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_refactions);
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_refactions = _tmp1_;
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "refactions");
#line 7694 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_storage_class_init (SerializableGeeCollectionsTestStorageClass * klass) {
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_storage_parent_class = g_type_class_peek_parent (klass);
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTestStoragePrivate));
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_storage_real_node_name;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_storage_real_to_string;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableContainerClass *) klass)->init_containers = (void (*)(GXmlSerializableContainer*)) serializable_gee_collections_test_storage_real_init_containers;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_test_storage_get_property;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_test_storage_set_property;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_test_storage_finalize;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_STORAGE_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_STORAGE_REFACTIONS, g_param_spec_object ("refactions", "refactions", "refactions", SERIALIZABLE_GEE_COLLECTIONS_TEST_REFACTION_TYPE_COLLECTION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 7719 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_storage_instance_init (SerializableGeeCollectionsTestStorage * self) {
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TEST_STORAGE_GET_PRIVATE (self);
#line 7726 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_storage_finalize (GObject* obj) {
	SerializableGeeCollectionsTestStorage * self;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_STORAGE, SerializableGeeCollectionsTestStorage);
#line 158 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_name);
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_refactions);
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (serializable_gee_collections_test_storage_parent_class)->finalize (obj);
#line 7740 "SerializableGeeCollectionsTest.c"
}


static GType serializable_gee_collections_test_storage_get_type (void) {
	static volatile gsize serializable_gee_collections_test_storage_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_storage_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestStorageClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_storage_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTestStorage), 0, (GInstanceInitFunc) serializable_gee_collections_test_storage_instance_init, NULL };
		GType serializable_gee_collections_test_storage_type_id;
		serializable_gee_collections_test_storage_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_CONTAINER, "SerializableGeeCollectionsTestStorage", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_test_storage_type_id__volatile, serializable_gee_collections_test_storage_type_id);
	}
	return serializable_gee_collections_test_storage_type_id__volatile;
}


static void _vala_serializable_gee_collections_test_storage_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestStorage * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_STORAGE, SerializableGeeCollectionsTestStorage);
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_STORAGE_NAME:
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_string (value, serializable_gee_collections_test_storage_get_name (self));
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_STORAGE_REFACTIONS:
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_object (value, serializable_gee_collections_test_storage_get_refactions (self));
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 7773 "SerializableGeeCollectionsTest.c"
		default:
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 7779 "SerializableGeeCollectionsTest.c"
	}
}


static void _vala_serializable_gee_collections_test_storage_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestStorage * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_STORAGE, SerializableGeeCollectionsTestStorage);
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_STORAGE_NAME:
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_storage_set_name (self, g_value_get_string (value));
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_STORAGE_REFACTIONS:
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_storage_set_refactions (self, g_value_get_object (value));
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 7801 "SerializableGeeCollectionsTest.c"
		default:
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 7807 "SerializableGeeCollectionsTest.c"
	}
}


static gchar* serializable_gee_collections_test_space_base_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestSpaceBase * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSpaceBase*) base;
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = g_strdup ("base");
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 175 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 7824 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_space_base_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestSpaceBase * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 176 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSpaceBase*) base;
#line 176 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_name;
#line 176 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 176 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp1_;
#line 176 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 7843 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestSpaceBase* serializable_gee_collections_test_space_base_construct (GType object_type) {
	SerializableGeeCollectionsTestSpaceBase * self = NULL;
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSpaceBase*) gxml_serializable_object_model_construct (object_type);
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 7853 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestSpaceBase* serializable_gee_collections_test_space_base_new (void) {
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_space_base_construct (SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_BASE);
#line 7860 "SerializableGeeCollectionsTest.c"
}


static const gchar* serializable_gee_collections_test_space_base_get_name (SerializableGeeCollectionsTestSpaceBase* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_name;
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 7875 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_base_set_name (SerializableGeeCollectionsTestSpaceBase* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_name);
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_name = _tmp1_;
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 7894 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestChargeZone* serializable_gee_collections_test_space_base_get_charge_zone (SerializableGeeCollectionsTestSpaceBase* self) {
	SerializableGeeCollectionsTestChargeZone* result;
	SerializableGeeCollectionsTestChargeZone* _tmp0_ = NULL;
#line 173 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 173 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_charge_zone;
#line 173 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 173 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 7909 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_base_set_charge_zone (SerializableGeeCollectionsTestSpaceBase* self, SerializableGeeCollectionsTestChargeZone* value) {
	SerializableGeeCollectionsTestChargeZone* _tmp0_ = NULL;
	SerializableGeeCollectionsTestChargeZone* _tmp1_ = NULL;
#line 173 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 173 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 173 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 173 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_charge_zone);
#line 173 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_charge_zone = _tmp1_;
#line 173 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "charge-zone");
#line 7928 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestStorage* serializable_gee_collections_test_space_base_get_storage (SerializableGeeCollectionsTestSpaceBase* self) {
	SerializableGeeCollectionsTestStorage* result;
	SerializableGeeCollectionsTestStorage* _tmp0_ = NULL;
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_storage;
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 7943 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_base_set_storage (SerializableGeeCollectionsTestSpaceBase* self, SerializableGeeCollectionsTestStorage* value) {
	SerializableGeeCollectionsTestStorage* _tmp0_ = NULL;
	SerializableGeeCollectionsTestStorage* _tmp1_ = NULL;
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_storage);
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_storage = _tmp1_;
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "storage");
#line 7962 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_base_class_init (SerializableGeeCollectionsTestSpaceBaseClass * klass) {
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_space_base_parent_class = g_type_class_peek_parent (klass);
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTestSpaceBasePrivate));
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_space_base_real_node_name;
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_space_base_real_to_string;
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_test_space_base_get_property;
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_test_space_base_set_property;
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_test_space_base_finalize;
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_BASE_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_BASE_CHARGE_ZONE, g_param_spec_object ("charge-zone", "charge-zone", "charge-zone", SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_CHARGE_ZONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_BASE_STORAGE, g_param_spec_object ("storage", "storage", "storage", SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_STORAGE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 7987 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_base_instance_init (SerializableGeeCollectionsTestSpaceBase * self) {
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_BASE_GET_PRIVATE (self);
#line 7994 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_space_base_finalize (GObject* obj) {
	SerializableGeeCollectionsTestSpaceBase * self;
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_BASE, SerializableGeeCollectionsTestSpaceBase);
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_name);
#line 173 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_charge_zone);
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_storage);
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (serializable_gee_collections_test_space_base_parent_class)->finalize (obj);
#line 8010 "SerializableGeeCollectionsTest.c"
}


static GType serializable_gee_collections_test_space_base_get_type (void) {
	static volatile gsize serializable_gee_collections_test_space_base_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_space_base_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestSpaceBaseClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_space_base_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTestSpaceBase), 0, (GInstanceInitFunc) serializable_gee_collections_test_space_base_instance_init, NULL };
		GType serializable_gee_collections_test_space_base_type_id;
		serializable_gee_collections_test_space_base_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeCollectionsTestSpaceBase", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_test_space_base_type_id__volatile, serializable_gee_collections_test_space_base_type_id);
	}
	return serializable_gee_collections_test_space_base_type_id__volatile;
}


static void _vala_serializable_gee_collections_test_space_base_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestSpaceBase * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_BASE, SerializableGeeCollectionsTestSpaceBase);
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_BASE_NAME:
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_string (value, serializable_gee_collections_test_space_base_get_name (self));
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_BASE_CHARGE_ZONE:
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_object (value, serializable_gee_collections_test_space_base_get_charge_zone (self));
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_BASE_STORAGE:
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_object (value, serializable_gee_collections_test_space_base_get_storage (self));
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 8049 "SerializableGeeCollectionsTest.c"
		default:
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 8055 "SerializableGeeCollectionsTest.c"
	}
}


static void _vala_serializable_gee_collections_test_space_base_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestSpaceBase * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SPACE_BASE, SerializableGeeCollectionsTestSpaceBase);
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_BASE_NAME:
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_space_base_set_name (self, g_value_get_string (value));
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_BASE_CHARGE_ZONE:
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_space_base_set_charge_zone (self, g_value_get_object (value));
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SPACE_BASE_STORAGE:
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_space_base_set_storage (self, g_value_get_object (value));
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 8083 "SerializableGeeCollectionsTest.c"
		default:
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 8089 "SerializableGeeCollectionsTest.c"
	}
}


static gchar* serializable_gee_collections_test_ball_fill_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestBallFill * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestBallFill*) base;
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_name;
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp1_;
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 8109 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_ball_fill_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestBallFill * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 183 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestBallFill*) base;
#line 183 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = g_strdup ("BallFill");
#line 183 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 183 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 8125 "SerializableGeeCollectionsTest.c"
}


static gboolean serializable_gee_collections_test_ball_fill_real_serialize_use_xml_node_value (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestBallFill * self;
	gboolean result = FALSE;
#line 184 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestBallFill*) base;
#line 184 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = TRUE;
#line 184 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 8138 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestBallFill* serializable_gee_collections_test_ball_fill_construct (GType object_type) {
	SerializableGeeCollectionsTestBallFill * self = NULL;
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestBallFill*) gxml_serializable_object_model_construct (object_type);
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 8148 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestBallFill* serializable_gee_collections_test_ball_fill_new (void) {
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_ball_fill_construct (SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL_FILL);
#line 8155 "SerializableGeeCollectionsTest.c"
}


static const gchar* serializable_gee_collections_test_ball_fill_get_name (SerializableGeeCollectionsTestBallFill* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_name;
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 8170 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_ball_fill_set_name (SerializableGeeCollectionsTestBallFill* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_name);
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_name = _tmp1_;
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 8189 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_ball_fill_class_init (SerializableGeeCollectionsTestBallFillClass * klass) {
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_ball_fill_parent_class = g_type_class_peek_parent (klass);
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTestBallFillPrivate));
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_ball_fill_real_to_string;
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_ball_fill_real_node_name;
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->serialize_use_xml_node_value = (gboolean (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_ball_fill_real_serialize_use_xml_node_value;
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_test_ball_fill_get_property;
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_test_ball_fill_set_property;
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_test_ball_fill_finalize;
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_FILL_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 8212 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_ball_fill_instance_init (SerializableGeeCollectionsTestBallFill * self) {
	gchar* _tmp0_ = NULL;
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_FILL_GET_PRIVATE (self);
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = g_strdup ("Fill");
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_name = _tmp0_;
#line 8224 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_ball_fill_finalize (GObject* obj) {
	SerializableGeeCollectionsTestBallFill * self;
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL_FILL, SerializableGeeCollectionsTestBallFill);
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_name);
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (serializable_gee_collections_test_ball_fill_parent_class)->finalize (obj);
#line 8236 "SerializableGeeCollectionsTest.c"
}


static GType serializable_gee_collections_test_ball_fill_get_type (void) {
	static volatile gsize serializable_gee_collections_test_ball_fill_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_ball_fill_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestBallFillClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_ball_fill_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTestBallFill), 0, (GInstanceInitFunc) serializable_gee_collections_test_ball_fill_instance_init, NULL };
		GType serializable_gee_collections_test_ball_fill_type_id;
		serializable_gee_collections_test_ball_fill_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeCollectionsTestBallFill", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_test_ball_fill_type_id__volatile, serializable_gee_collections_test_ball_fill_type_id);
	}
	return serializable_gee_collections_test_ball_fill_type_id__volatile;
}


static void _vala_serializable_gee_collections_test_ball_fill_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestBallFill * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL_FILL, SerializableGeeCollectionsTestBallFill);
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_FILL_NAME:
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_string (value, serializable_gee_collections_test_ball_fill_get_name (self));
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 8263 "SerializableGeeCollectionsTest.c"
		default:
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 8269 "SerializableGeeCollectionsTest.c"
	}
}


static void _vala_serializable_gee_collections_test_ball_fill_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestBallFill * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL_FILL, SerializableGeeCollectionsTestBallFill);
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_FILL_NAME:
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_ball_fill_set_name (self, g_value_get_string (value));
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 8285 "SerializableGeeCollectionsTest.c"
		default:
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 8291 "SerializableGeeCollectionsTest.c"
	}
}


static gchar* serializable_gee_collections_test_ball_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestBall * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 191 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestBall*) base;
#line 191 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_name;
#line 191 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 191 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp1_;
#line 191 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 8311 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_ball_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestBall * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 192 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestBall*) base;
#line 192 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = g_strdup ("Ball");
#line 192 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 192 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 8327 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestBall* serializable_gee_collections_test_ball_construct (GType object_type) {
	SerializableGeeCollectionsTestBall * self = NULL;
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestBall*) gxml_serializable_object_model_construct (object_type);
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 8337 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestBall* serializable_gee_collections_test_ball_new (void) {
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_ball_construct (SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL);
#line 8344 "SerializableGeeCollectionsTest.c"
}


static const gchar* serializable_gee_collections_test_ball_get_name (SerializableGeeCollectionsTestBall* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 189 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 189 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_name;
#line 189 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 189 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 8359 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_ball_set_name (SerializableGeeCollectionsTestBall* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 189 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 189 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 189 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 189 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_name);
#line 189 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_name = _tmp1_;
#line 189 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 8378 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestBallFill* serializable_gee_collections_test_ball_get_ballfill (SerializableGeeCollectionsTestBall* self) {
	SerializableGeeCollectionsTestBallFill* result;
	SerializableGeeCollectionsTestBallFill* _tmp0_ = NULL;
#line 190 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 190 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_ballfill;
#line 190 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 190 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 8393 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_ball_set_ballfill (SerializableGeeCollectionsTestBall* self, SerializableGeeCollectionsTestBallFill* value) {
	SerializableGeeCollectionsTestBallFill* _tmp0_ = NULL;
	SerializableGeeCollectionsTestBallFill* _tmp1_ = NULL;
#line 190 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 190 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 190 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 190 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_ballfill);
#line 190 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_ballfill = _tmp1_;
#line 190 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "ballfill");
#line 8412 "SerializableGeeCollectionsTest.c"
}


static gboolean serializable_gee_collections_test_ball_array_real_deserialize_proceed (GXmlSerializableArrayList* base) {
	SerializableGeeCollectionsTestBallArray * self;
	gboolean result = FALSE;
#line 194 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestBallArray*) base;
#line 194 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = FALSE;
#line 194 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 8425 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestBallArray* serializable_gee_collections_test_ball_array_construct (GType object_type) {
	SerializableGeeCollectionsTestBallArray * self = NULL;
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestBallArray*) gxml_serializable_array_list_construct (object_type, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 8435 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestBallArray* serializable_gee_collections_test_ball_array_new (void) {
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_ball_array_construct (SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_TYPE_ARRAY);
#line 8442 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_ball_array_class_init (SerializableGeeCollectionsTestBallArrayClass * klass) {
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_ball_array_parent_class = g_type_class_peek_parent (klass);
#line 193 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableArrayListClass *) klass)->deserialize_proceed = (gboolean (*)(GXmlSerializableArrayList*)) serializable_gee_collections_test_ball_array_real_deserialize_proceed;
#line 8451 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_ball_array_instance_init (SerializableGeeCollectionsTestBallArray * self) {
}


GType serializable_gee_collections_test_ball_array_get_type (void) {
	static volatile gsize serializable_gee_collections_test_ball_array_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_ball_array_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestBallArrayClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_ball_array_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTestBallArray), 0, (GInstanceInitFunc) serializable_gee_collections_test_ball_array_instance_init, NULL };
		GType serializable_gee_collections_test_ball_array_type_id;
		serializable_gee_collections_test_ball_array_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_ARRAY_LIST, "SerializableGeeCollectionsTestBallArray", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_test_ball_array_type_id__volatile, serializable_gee_collections_test_ball_array_type_id);
	}
	return serializable_gee_collections_test_ball_array_type_id__volatile;
}


static void serializable_gee_collections_test_ball_class_init (SerializableGeeCollectionsTestBallClass * klass) {
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_ball_parent_class = g_type_class_peek_parent (klass);
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTestBallPrivate));
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_ball_real_to_string;
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_ball_real_node_name;
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_test_ball_get_property;
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_test_ball_set_property;
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_test_ball_finalize;
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_BALLFILL, g_param_spec_object ("ballfill", "ballfill", "ballfill", SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL_FILL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 8490 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_ball_instance_init (SerializableGeeCollectionsTestBall * self) {
	gchar* _tmp0_ = NULL;
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_GET_PRIVATE (self);
#line 189 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = g_strdup ("Ball");
#line 189 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_name = _tmp0_;
#line 8502 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_ball_finalize (GObject* obj) {
	SerializableGeeCollectionsTestBall * self;
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL, SerializableGeeCollectionsTestBall);
#line 189 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_name);
#line 190 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_ballfill);
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (serializable_gee_collections_test_ball_parent_class)->finalize (obj);
#line 8516 "SerializableGeeCollectionsTest.c"
}


static GType serializable_gee_collections_test_ball_get_type (void) {
	static volatile gsize serializable_gee_collections_test_ball_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_ball_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestBallClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_ball_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTestBall), 0, (GInstanceInitFunc) serializable_gee_collections_test_ball_instance_init, NULL };
		GType serializable_gee_collections_test_ball_type_id;
		serializable_gee_collections_test_ball_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeCollectionsTestBall", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_test_ball_type_id__volatile, serializable_gee_collections_test_ball_type_id);
	}
	return serializable_gee_collections_test_ball_type_id__volatile;
}


static void _vala_serializable_gee_collections_test_ball_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestBall * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL, SerializableGeeCollectionsTestBall);
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_NAME:
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_string (value, serializable_gee_collections_test_ball_get_name (self));
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_BALLFILL:
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_object (value, serializable_gee_collections_test_ball_get_ballfill (self));
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 8549 "SerializableGeeCollectionsTest.c"
		default:
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 8555 "SerializableGeeCollectionsTest.c"
	}
}


static void _vala_serializable_gee_collections_test_ball_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestBall * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BALL, SerializableGeeCollectionsTestBall);
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_NAME:
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_ball_set_name (self, g_value_get_string (value));
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_BALLFILL:
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_ball_set_ballfill (self, g_value_get_object (value));
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 8577 "SerializableGeeCollectionsTest.c"
		default:
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 187 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 8583 "SerializableGeeCollectionsTest.c"
	}
}


static gchar* serializable_gee_collections_test_small_bag_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestSmallBag * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSmallBag*) base;
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_name;
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp1_;
#line 202 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 8603 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_small_bag_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestSmallBag * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSmallBag*) base;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = g_strdup ("SmallBag");
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 203 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 8619 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestSmallBag* serializable_gee_collections_test_small_bag_construct (GType object_type) {
	SerializableGeeCollectionsTestSmallBag * self = NULL;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSmallBag*) gxml_serializable_object_model_construct (object_type);
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 8629 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestSmallBag* serializable_gee_collections_test_small_bag_new (void) {
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_small_bag_construct (SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SMALL_BAG);
#line 8636 "SerializableGeeCollectionsTest.c"
}


static const gchar* serializable_gee_collections_test_small_bag_get_name (SerializableGeeCollectionsTestSmallBag* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_name;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 8651 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_small_bag_set_name (SerializableGeeCollectionsTestSmallBag* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_name);
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_name = _tmp1_;
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 8670 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestBallArray* serializable_gee_collections_test_small_bag_get_balls (SerializableGeeCollectionsTestSmallBag* self) {
	SerializableGeeCollectionsTestBallArray* result;
	SerializableGeeCollectionsTestBallArray* _tmp0_ = NULL;
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_balls;
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 8685 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_small_bag_set_balls (SerializableGeeCollectionsTestSmallBag* self, SerializableGeeCollectionsTestBallArray* value) {
	SerializableGeeCollectionsTestBallArray* _tmp0_ = NULL;
	SerializableGeeCollectionsTestBallArray* _tmp1_ = NULL;
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_balls);
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_balls = _tmp1_;
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "balls");
#line 8704 "SerializableGeeCollectionsTest.c"
}


static gboolean serializable_gee_collections_test_small_bag_array_real_deserialize_proceed (GXmlSerializableArrayList* base) {
	SerializableGeeCollectionsTestSmallBagArray * self;
	gboolean result = FALSE;
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSmallBagArray*) base;
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = FALSE;
#line 205 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 8717 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestSmallBagArray* serializable_gee_collections_test_small_bag_array_construct (GType object_type) {
	SerializableGeeCollectionsTestSmallBagArray * self = NULL;
#line 204 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestSmallBagArray*) gxml_serializable_array_list_construct (object_type, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SMALL_BAG, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 204 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 8727 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestSmallBagArray* serializable_gee_collections_test_small_bag_array_new (void) {
#line 204 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_small_bag_array_construct (SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_TYPE_ARRAY);
#line 8734 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_small_bag_array_class_init (SerializableGeeCollectionsTestSmallBagArrayClass * klass) {
#line 204 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_small_bag_array_parent_class = g_type_class_peek_parent (klass);
#line 204 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableArrayListClass *) klass)->deserialize_proceed = (gboolean (*)(GXmlSerializableArrayList*)) serializable_gee_collections_test_small_bag_array_real_deserialize_proceed;
#line 8743 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_small_bag_array_instance_init (SerializableGeeCollectionsTestSmallBagArray * self) {
}


GType serializable_gee_collections_test_small_bag_array_get_type (void) {
	static volatile gsize serializable_gee_collections_test_small_bag_array_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_small_bag_array_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestSmallBagArrayClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_small_bag_array_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTestSmallBagArray), 0, (GInstanceInitFunc) serializable_gee_collections_test_small_bag_array_instance_init, NULL };
		GType serializable_gee_collections_test_small_bag_array_type_id;
		serializable_gee_collections_test_small_bag_array_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_ARRAY_LIST, "SerializableGeeCollectionsTestSmallBagArray", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_test_small_bag_array_type_id__volatile, serializable_gee_collections_test_small_bag_array_type_id);
	}
	return serializable_gee_collections_test_small_bag_array_type_id__volatile;
}


static void serializable_gee_collections_test_small_bag_class_init (SerializableGeeCollectionsTestSmallBagClass * klass) {
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_small_bag_parent_class = g_type_class_peek_parent (klass);
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTestSmallBagPrivate));
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_small_bag_real_to_string;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_small_bag_real_node_name;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_test_small_bag_get_property;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_test_small_bag_set_property;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_test_small_bag_finalize;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_BALLS, g_param_spec_object ("balls", "balls", "balls", SERIALIZABLE_GEE_COLLECTIONS_TEST_BALL_TYPE_ARRAY, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 8782 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_small_bag_instance_init (SerializableGeeCollectionsTestSmallBag * self) {
	gchar* _tmp0_ = NULL;
	SerializableGeeCollectionsTestBallArray* _tmp1_ = NULL;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_GET_PRIVATE (self);
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = g_strdup ("SmallBag");
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_name = _tmp0_;
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = serializable_gee_collections_test_ball_array_new ();
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_balls = _tmp1_;
#line 8799 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_small_bag_finalize (GObject* obj) {
	SerializableGeeCollectionsTestSmallBag * self;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SMALL_BAG, SerializableGeeCollectionsTestSmallBag);
#line 200 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_name);
#line 201 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_balls);
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (serializable_gee_collections_test_small_bag_parent_class)->finalize (obj);
#line 8813 "SerializableGeeCollectionsTest.c"
}


static GType serializable_gee_collections_test_small_bag_get_type (void) {
	static volatile gsize serializable_gee_collections_test_small_bag_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_small_bag_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestSmallBagClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_small_bag_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTestSmallBag), 0, (GInstanceInitFunc) serializable_gee_collections_test_small_bag_instance_init, NULL };
		GType serializable_gee_collections_test_small_bag_type_id;
		serializable_gee_collections_test_small_bag_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeCollectionsTestSmallBag", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_test_small_bag_type_id__volatile, serializable_gee_collections_test_small_bag_type_id);
	}
	return serializable_gee_collections_test_small_bag_type_id__volatile;
}


static void _vala_serializable_gee_collections_test_small_bag_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestSmallBag * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SMALL_BAG, SerializableGeeCollectionsTestSmallBag);
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_NAME:
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_string (value, serializable_gee_collections_test_small_bag_get_name (self));
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_BALLS:
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_object (value, serializable_gee_collections_test_small_bag_get_balls (self));
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 8846 "SerializableGeeCollectionsTest.c"
		default:
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 8852 "SerializableGeeCollectionsTest.c"
	}
}


static void _vala_serializable_gee_collections_test_small_bag_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestSmallBag * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_SMALL_BAG, SerializableGeeCollectionsTestSmallBag);
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_NAME:
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_small_bag_set_name (self, g_value_get_string (value));
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_BALLS:
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_small_bag_set_balls (self, g_value_get_object (value));
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 8874 "SerializableGeeCollectionsTest.c"
		default:
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 198 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 8880 "SerializableGeeCollectionsTest.c"
	}
}


static gchar* serializable_gee_collections_test_big_bag_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestBigBag * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestBigBag*) base;
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_name;
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp1_;
#line 213 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 8900 "SerializableGeeCollectionsTest.c"
}


static gchar* serializable_gee_collections_test_big_bag_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTestBigBag * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 214 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestBigBag*) base;
#line 214 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = g_strdup ("BigBag");
#line 214 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 214 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 8916 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestBigBag* serializable_gee_collections_test_big_bag_construct (GType object_type) {
	SerializableGeeCollectionsTestBigBag * self = NULL;
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = (SerializableGeeCollectionsTestBigBag*) gxml_serializable_object_model_construct (object_type);
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return self;
#line 8926 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestBigBag* serializable_gee_collections_test_big_bag_new (void) {
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return serializable_gee_collections_test_big_bag_construct (SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BIG_BAG);
#line 8933 "SerializableGeeCollectionsTest.c"
}


static const gchar* serializable_gee_collections_test_big_bag_get_name (SerializableGeeCollectionsTestBigBag* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_name;
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 8948 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_big_bag_set_name (SerializableGeeCollectionsTestBigBag* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_name);
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_name = _tmp1_;
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "name");
#line 8967 "SerializableGeeCollectionsTest.c"
}


static SerializableGeeCollectionsTestSmallBagArray* serializable_gee_collections_test_big_bag_get_bags (SerializableGeeCollectionsTestBigBag* self) {
	SerializableGeeCollectionsTestSmallBagArray* result;
	SerializableGeeCollectionsTestSmallBagArray* _tmp0_ = NULL;
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = self->priv->_bags;
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	result = _tmp0_;
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	return result;
#line 8982 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_big_bag_set_bags (SerializableGeeCollectionsTestBigBag* self, SerializableGeeCollectionsTestSmallBagArray* value) {
	SerializableGeeCollectionsTestSmallBagArray* _tmp0_ = NULL;
	SerializableGeeCollectionsTestSmallBagArray* _tmp1_ = NULL;
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_return_if_fail (self != NULL);
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = value;
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_bags);
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_bags = _tmp1_;
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_notify ((GObject *) self, "bags");
#line 9001 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_big_bag_class_init (SerializableGeeCollectionsTestBigBagClass * klass) {
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_big_bag_parent_class = g_type_class_peek_parent (klass);
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTestBigBagPrivate));
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_big_bag_real_to_string;
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_test_big_bag_real_node_name;
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_test_big_bag_get_property;
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_test_big_bag_set_property;
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_test_big_bag_finalize;
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_BIG_BAG_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TEST_BIG_BAG_BAGS, g_param_spec_object ("bags", "bags", "bags", SERIALIZABLE_GEE_COLLECTIONS_TEST_SMALL_BAG_TYPE_ARRAY, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 9024 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_big_bag_instance_init (SerializableGeeCollectionsTestBigBag * self) {
	gchar* _tmp0_ = NULL;
	SerializableGeeCollectionsTestSmallBagArray* _tmp1_ = NULL;
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TEST_BIG_BAG_GET_PRIVATE (self);
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp0_ = g_strdup ("ball");
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_name = _tmp0_;
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_tmp1_ = serializable_gee_collections_test_small_bag_array_new ();
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self->priv->_bags = _tmp1_;
#line 9041 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_big_bag_finalize (GObject* obj) {
	SerializableGeeCollectionsTestBigBag * self;
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BIG_BAG, SerializableGeeCollectionsTestBigBag);
#line 211 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_free0 (self->priv->_name);
#line 212 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	_g_object_unref0 (self->priv->_bags);
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	G_OBJECT_CLASS (serializable_gee_collections_test_big_bag_parent_class)->finalize (obj);
#line 9055 "SerializableGeeCollectionsTest.c"
}


static GType serializable_gee_collections_test_big_bag_get_type (void) {
	static volatile gsize serializable_gee_collections_test_big_bag_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_big_bag_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestBigBagClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_big_bag_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTestBigBag), 0, (GInstanceInitFunc) serializable_gee_collections_test_big_bag_instance_init, NULL };
		GType serializable_gee_collections_test_big_bag_type_id;
		serializable_gee_collections_test_big_bag_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeCollectionsTestBigBag", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_test_big_bag_type_id__volatile, serializable_gee_collections_test_big_bag_type_id);
	}
	return serializable_gee_collections_test_big_bag_type_id__volatile;
}


static void _vala_serializable_gee_collections_test_big_bag_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestBigBag * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BIG_BAG, SerializableGeeCollectionsTestBigBag);
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_BIG_BAG_NAME:
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_string (value, serializable_gee_collections_test_big_bag_get_name (self));
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_BIG_BAG_BAGS:
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		g_value_set_object (value, serializable_gee_collections_test_big_bag_get_bags (self));
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 9088 "SerializableGeeCollectionsTest.c"
		default:
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 9094 "SerializableGeeCollectionsTest.c"
	}
}


static void _vala_serializable_gee_collections_test_big_bag_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTestBigBag * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TEST_TYPE_BIG_BAG, SerializableGeeCollectionsTestBigBag);
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	switch (property_id) {
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_BIG_BAG_NAME:
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_big_bag_set_name (self, g_value_get_string (value));
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TEST_BIG_BAG_BAGS:
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		serializable_gee_collections_test_big_bag_set_bags (self, g_value_get_object (value));
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 9116 "SerializableGeeCollectionsTest.c"
		default:
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 209 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
		break;
#line 9122 "SerializableGeeCollectionsTest.c"
	}
}


static void serializable_gee_collections_test_class_init (SerializableGeeCollectionsTestClass * klass) {
#line 30 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollectionsTest.vala"
	serializable_gee_collections_test_parent_class = g_type_class_peek_parent (klass);
#line 9130 "SerializableGeeCollectionsTest.c"
}


static void serializable_gee_collections_test_instance_init (SerializableGeeCollectionsTest * self) {
}


/**
 *
 *  GXml.Serializable.GeeCollectionsTest
 *
 *  Authors:
 *
 *       Daniel Espinosa <esodan@gmail.com>
 *
 *
 *  Copyright (c) 2013-2015 Daniel Espinosa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
GType serializable_gee_collections_test_get_type (void) {
	static volatile gsize serializable_gee_collections_test_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_test_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTestClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_test_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTest), 0, (GInstanceInitFunc) serializable_gee_collections_test_instance_init, NULL };
		GType serializable_gee_collections_test_type_id;
		serializable_gee_collections_test_type_id = g_type_register_static (TYPE_GXML_TEST, "SerializableGeeCollectionsTest", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_test_type_id__volatile, serializable_gee_collections_test_type_id);
	}
	return serializable_gee_collections_test_type_id__volatile;
}



