/* TElementTest.c generated by valac 0.34.6, the Vala compiler
 * generated from TElementTest.vala, do not modify */

/* -*- Mode: vala; indent-tabs-mode: t; c-basic-offset: 2; tab-width: 2 -*- */
/* Notation.vala
 *
 * Copyright (C) 2011-2013  Richard Schwarting <aquarichy@gmail.com>
 * Copyright (C) 2011-2015  Daniel Espinosa <esodan@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Richard Schwarting <aquarichy@gmail.com>
 *      Daniel Espinosa <esodan@gmail.com>
 */

#include <glib.h>
#include <glib-object.h>
#include <gxml/gxml.h>
#include <gee.h>
#include <stdlib.h>
#include <string.h>
#include <gio/gio.h>
#include <libxml/xmlreader.h>


#define TYPE_GXML_TEST (gxml_test_get_type ())
#define GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GXML_TEST, GXmlTest))
#define GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GXML_TEST, GXmlTestClass))
#define IS_GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GXML_TEST))
#define IS_GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GXML_TEST))
#define GXML_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GXML_TEST, GXmlTestClass))

typedef struct _GXmlTest GXmlTest;
typedef struct _GXmlTestClass GXmlTestClass;
typedef struct _GXmlTestPrivate GXmlTestPrivate;

#define TYPE_TELEMENT_TEST (telement_test_get_type ())
#define TELEMENT_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TELEMENT_TEST, TElementTest))
#define TELEMENT_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TELEMENT_TEST, TElementTestClass))
#define IS_TELEMENT_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TELEMENT_TEST))
#define IS_TELEMENT_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TELEMENT_TEST))
#define TELEMENT_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TELEMENT_TEST, TElementTestClass))

typedef struct _TElementTest TElementTest;
typedef struct _TElementTestClass TElementTestClass;
typedef struct _TElementTestPrivate TElementTestPrivate;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _GXmlTest {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GXmlTestPrivate * priv;
};

struct _GXmlTestClass {
	GTypeClass parent_class;
	void (*finalize) (GXmlTest *self);
};

struct _TElementTest {
	GXmlTest parent_instance;
	TElementTestPrivate * priv;
};

struct _TElementTestClass {
	GXmlTestClass parent_class;
};


static gpointer telement_test_parent_class = NULL;

gpointer gxml_test_ref (gpointer instance);
void gxml_test_unref (gpointer instance);
GParamSpec* param_spec_gxml_test (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_gxml_test (GValue* value, gpointer v_object);
void value_take_gxml_test (GValue* value, gpointer v_object);
gpointer value_get_gxml_test (const GValue* value);
GType gxml_test_get_type (void) G_GNUC_CONST;
GType telement_test_get_type (void) G_GNUC_CONST;
enum  {
	TELEMENT_TEST_DUMMY_PROPERTY
};
void telement_test_add_tests (void);
static void __lambda82_ (void);
static void ___lambda82__gtest_func (void);
static void __lambda83_ (void);
static void ___lambda83__gtest_func (void);
static void __lambda84_ (void);
static void ___lambda84__gtest_func (void);
static void __lambda85_ (void);
static void ___lambda85__gtest_func (void);
static void __lambda86_ (void);
static void ___lambda86__gtest_func (void);
static void __lambda87_ (void);
static void ___lambda87__gtest_func (void);
static void __lambda88_ (void);
static void ___lambda88__gtest_func (void);
static void __lambda89_ (void);
static void ___lambda89__gtest_func (void);
static void __lambda90_ (void);
static void ___lambda90__gtest_func (void);
static void __lambda91_ (void);
static void ___lambda91__gtest_func (void);
static void __lambda92_ (void);
static void ___lambda92__gtest_func (void);
static void __lambda93_ (void);
static void ___lambda93__gtest_func (void);
static void __lambda94_ (void);
static void ___lambda94__gtest_func (void);
static void __lambda95_ (void);
static void ___lambda95__gtest_func (void);
static void __lambda96_ (void);
static void ___lambda96__gtest_func (void);
static void __lambda97_ (void);
static void ___lambda97__gtest_func (void);
static void __lambda98_ (void);
static void ___lambda98__gtest_func (void);
static void __lambda99_ (void);
static void ___lambda99__gtest_func (void);
static void __lambda100_ (void);
static GXmlTDocumentReadType ___lambda101_ (GXmlNode* node, xmlTextReader* tr);
static GXmlTDocumentReadType ____lambda101__gxml_tdocument_read_type_func (GXmlNode* node, xmlTextReader* tr, gpointer self);
static GXmlTDocumentReadType ___lambda102_ (GXmlNode* node, xmlTextReader* tr);
static GXmlTDocumentReadType ____lambda102__gxml_tdocument_read_type_func (GXmlNode* node, xmlTextReader* tr, gpointer self);
static void ___lambda100__gtest_func (void);
TElementTest* telement_test_new (void);
TElementTest* telement_test_construct (GType object_type);
GXmlTest* gxml_test_new (void);
GXmlTest* gxml_test_construct (GType object_type);


static void __lambda82_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp0_ = NULL;
		GXmlNode* _tmp1_ = NULL;
		GXmlTDocument* _tmp2_ = NULL;
		GXmlNode* _tmp3_ = NULL;
		GXmlElement* e = NULL;
		GXmlNode* _tmp4_ = NULL;
		GXmlTDocument* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GeeBidirList* _tmp7_ = NULL;
		GeeBidirList* _tmp8_ = NULL;
		GXmlElement* _tmp9_ = NULL;
		GXmlTDocument* _tmp10_ = NULL;
		GeeBidirList* _tmp11_ = NULL;
		GeeBidirList* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		gint _tmp14_ = 0;
		gint _tmp15_ = 0;
		GXmlTDocument* _tmp16_ = NULL;
		GXmlNode* _tmp17_ = NULL;
		GXmlNode* _tmp18_ = NULL;
		GXmlNode* _tmp19_ = NULL;
		gchar* _tmp20_ = NULL;
		gchar* _tmp21_ = NULL;
		gchar* _tmp22_ = NULL;
		GXmlElement* _tmp23_ = NULL;
		GXmlTDocument* _tmp24_ = NULL;
		GXmlNode* _tmp25_ = NULL;
		GXmlNode* _tmp26_ = NULL;
		GXmlNode* _tmp27_ = NULL;
		GeeMap* _tmp28_ = NULL;
		GeeMap* _tmp29_ = NULL;
		GeeMap* _tmp30_ = NULL;
		gpointer _tmp31_ = NULL;
		GXmlNode* _tmp32_ = NULL;
		GXmlTDocument* _tmp33_ = NULL;
		GXmlNode* _tmp34_ = NULL;
		GXmlNode* _tmp35_ = NULL;
		GXmlNode* _tmp36_ = NULL;
		GeeMap* _tmp37_ = NULL;
		GeeMap* _tmp38_ = NULL;
		GeeMap* _tmp39_ = NULL;
		gpointer _tmp40_ = NULL;
		GXmlNode* _tmp41_ = NULL;
		gchar* _tmp42_ = NULL;
		gchar* _tmp43_ = NULL;
		gchar* _tmp44_ = NULL;
		GXmlElement* _tmp45_ = NULL;
		GeeMap* _tmp46_ = NULL;
		GeeMap* _tmp47_ = NULL;
		GeeMap* _tmp48_ = NULL;
		gint _tmp49_ = 0;
		gint _tmp50_ = 0;
		GXmlElement* _tmp51_ = NULL;
		GeeBidirList* _tmp52_ = NULL;
		GeeBidirList* _tmp53_ = NULL;
		GeeBidirList* _tmp54_ = NULL;
		gint _tmp55_ = 0;
		gint _tmp56_ = 0;
		GXmlNode* _tmp57_ = NULL;
		GXmlTDocument* _tmp58_ = NULL;
		GXmlNode* _tmp59_ = NULL;
		GXmlElement* child = NULL;
		GXmlNode* _tmp60_ = NULL;
		GXmlElement* _tmp61_ = NULL;
		GeeBidirList* _tmp62_ = NULL;
		GeeBidirList* _tmp63_ = NULL;
		GeeBidirList* _tmp64_ = NULL;
		GXmlElement* _tmp65_ = NULL;
		GeeBidirList* _tmp66_ = NULL;
		GeeBidirList* _tmp67_ = NULL;
		GeeBidirList* _tmp68_ = NULL;
		gint _tmp69_ = 0;
		gint _tmp70_ = 0;
		GXmlElement* c = NULL;
		GXmlElement* _tmp71_ = NULL;
		GeeBidirList* _tmp72_ = NULL;
		GeeBidirList* _tmp73_ = NULL;
		GeeBidirList* _tmp74_ = NULL;
		gpointer _tmp75_ = NULL;
		GXmlElement* _tmp76_ = NULL;
		gchar* _tmp77_ = NULL;
		gchar* _tmp78_ = NULL;
		gchar* _tmp79_ = NULL;
		GeeMap* _tmp80_ = NULL;
		GeeMap* _tmp81_ = NULL;
		GeeMap* _tmp82_ = NULL;
		gpointer _tmp83_ = NULL;
		GXmlNode* _tmp84_ = NULL;
		GeeMap* _tmp85_ = NULL;
		GeeMap* _tmp86_ = NULL;
		GeeMap* _tmp87_ = NULL;
		gpointer _tmp88_ = NULL;
		GXmlNode* _tmp89_ = NULL;
		gchar* _tmp90_ = NULL;
		gchar* _tmp91_ = NULL;
		gchar* _tmp92_ = NULL;
		GXmlElement* _tmp93_ = NULL;
		GXmlElement* _tmp94_ = NULL;
		GeeMap* _tmp95_ = NULL;
		GeeMap* _tmp96_ = NULL;
		GeeMap* _tmp97_ = NULL;
		gint _tmp98_ = 0;
		gint _tmp99_ = 0;
		GXmlElement* _tmp100_ = NULL;
		GeeMap* _tmp101_ = NULL;
		GeeMap* _tmp102_ = NULL;
		GeeMap* _tmp103_ = NULL;
		gpointer _tmp104_ = NULL;
		GXmlNode* _tmp105_ = NULL;
		gchar* _tmp106_ = NULL;
		gchar* _tmp107_ = NULL;
		gchar* _tmp108_ = NULL;
		gchar* _tmp109_ = NULL;
		gchar* _tmp110_ = NULL;
		gchar* _tmp111_ = NULL;
		GeeBidirList* _tmp112_ = NULL;
		GeeBidirList* _tmp113_ = NULL;
		GeeBidirList* _tmp114_ = NULL;
		gint _tmp115_ = 0;
		gint _tmp116_ = 0;
		GeeBidirList* _tmp117_ = NULL;
		GeeBidirList* _tmp118_ = NULL;
		GeeBidirList* _tmp119_ = NULL;
		gint _tmp120_ = 0;
		gint _tmp121_ = 0;
		gchar* _tmp122_ = NULL;
		gchar* _tmp123_ = NULL;
		gchar* _tmp124_ = NULL;
#line 31 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp0_ = gxml_tdocument_new ();
#line 31 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		d = _tmp0_;
#line 32 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp2_ = d;
#line 32 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp3_ = gxml_document_create_element ((GXmlDocument*) _tmp2_, "element", &_inner_error_);
#line 32 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp1_ = _tmp3_;
#line 32 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 32 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 297 "TElementTest.c"
			goto __catch105_g_error;
		}
#line 32 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp4_ = _tmp1_;
#line 32 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp1_ = NULL;
#line 32 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e = G_TYPE_CHECK_INSTANCE_CAST (_tmp4_, GXML_TYPE_ELEMENT, GXmlElement);
#line 33 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp5_ = d;
#line 33 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp6_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp5_);
#line 33 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp7_ = _tmp6_;
#line 33 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp8_ = _tmp7_;
#line 33 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp9_ = e;
#line 33 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp8_, (GXmlNode*) _tmp9_);
#line 33 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp8_);
#line 34 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp10_ = d;
#line 34 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp11_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp10_);
#line 34 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp12_ = _tmp11_;
#line 34 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp13_ = _tmp12_;
#line 34 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp14_ = gee_collection_get_size ((GeeCollection*) _tmp13_);
#line 34 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp15_ = _tmp14_;
#line 34 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp15_ == 1, "d.children_nodes.size == 1");
#line 34 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp13_);
#line 35 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp16_ = d;
#line 35 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp17_ = gxml_document_get_root ((GXmlDocument*) _tmp16_);
#line 35 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp18_ = _tmp17_;
#line 35 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp19_ = _tmp18_;
#line 35 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp20_ = gxml_node_get_name (_tmp19_);
#line 35 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp21_ = _tmp20_;
#line 35 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp22_ = _tmp21_;
#line 35 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp22_, "element") == 0, "d.root.name == \"element\"");
#line 35 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp22_);
#line 35 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp19_);
#line 36 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp23_ = e;
#line 36 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_element_set_attr (_tmp23_, "attr1", "val1");
#line 37 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp24_ = d;
#line 37 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp25_ = gxml_document_get_root ((GXmlDocument*) _tmp24_);
#line 37 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp26_ = _tmp25_;
#line 37 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp27_ = _tmp26_;
#line 37 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp28_ = gxml_node_get_attrs (_tmp27_);
#line 37 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp29_ = _tmp28_;
#line 37 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp30_ = _tmp29_;
#line 37 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp31_ = gee_map_get (_tmp30_, "attr1");
#line 37 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp32_ = (GXmlNode*) _tmp31_;
#line 37 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp32_ != NULL, "d.root.attrs.get (\"attr1\") != null");
#line 37 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp32_);
#line 37 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp30_);
#line 37 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp27_);
#line 38 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp33_ = d;
#line 38 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp34_ = gxml_document_get_root ((GXmlDocument*) _tmp33_);
#line 38 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp35_ = _tmp34_;
#line 38 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp36_ = _tmp35_;
#line 38 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp37_ = gxml_node_get_attrs (_tmp36_);
#line 38 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp38_ = _tmp37_;
#line 38 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp39_ = _tmp38_;
#line 38 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp40_ = gee_map_get (_tmp39_, "attr1");
#line 38 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp41_ = (GXmlNode*) _tmp40_;
#line 38 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp42_ = gxml_node_get_value (_tmp41_);
#line 38 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp43_ = _tmp42_;
#line 38 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp44_ = _tmp43_;
#line 38 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp44_, "val1") == 0, "d.root.attrs.get (\"attr1\").value == \"val1\"");
#line 38 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp44_);
#line 38 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp41_);
#line 38 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp39_);
#line 38 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp36_);
#line 39 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp45_ = e;
#line 39 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp46_ = gxml_node_get_attrs ((GXmlNode*) _tmp45_);
#line 39 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp47_ = _tmp46_;
#line 39 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp48_ = _tmp47_;
#line 39 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp49_ = gee_map_get_size (_tmp48_);
#line 39 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp50_ = _tmp49_;
#line 39 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp50_ == 1, "e.attrs.size == 1");
#line 39 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp48_);
#line 40 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp51_ = e;
#line 40 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp52_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp51_);
#line 40 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp53_ = _tmp52_;
#line 40 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp54_ = _tmp53_;
#line 40 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp55_ = gee_collection_get_size ((GeeCollection*) _tmp54_);
#line 40 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp56_ = _tmp55_;
#line 40 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp56_ == 0, "e.children_nodes.size == 0");
#line 40 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp54_);
#line 41 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp58_ = d;
#line 41 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp59_ = gxml_document_create_element ((GXmlDocument*) _tmp58_, "child", &_inner_error_);
#line 41 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp57_ = _tmp59_;
#line 41 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 41 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 41 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (_tmp1_);
#line 41 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 466 "TElementTest.c"
			goto __catch105_g_error;
		}
#line 41 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp60_ = _tmp57_;
#line 41 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp57_ = NULL;
#line 41 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		child = G_TYPE_CHECK_INSTANCE_CAST (_tmp60_, GXML_TYPE_ELEMENT, GXmlElement);
#line 42 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (child != NULL, "child != null");
#line 43 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp61_ = e;
#line 43 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp62_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp61_);
#line 43 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp63_ = _tmp62_;
#line 43 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp64_ = _tmp63_;
#line 43 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp64_, (GXmlNode*) child);
#line 43 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp64_);
#line 44 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp65_ = e;
#line 44 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp66_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp65_);
#line 44 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp67_ = _tmp66_;
#line 44 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp68_ = _tmp67_;
#line 44 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp69_ = gee_collection_get_size ((GeeCollection*) _tmp68_);
#line 44 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp70_ = _tmp69_;
#line 44 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp70_ == 1, "e.children_nodes.size == 1");
#line 44 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp68_);
#line 45 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_element_set_attr (child, "cattr1", "cval1");
#line 46 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp71_ = e;
#line 46 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp72_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp71_);
#line 46 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp73_ = _tmp72_;
#line 46 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp74_ = _tmp73_;
#line 46 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp75_ = gee_list_get ((GeeList*) _tmp74_, 0);
#line 46 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp76_ = G_TYPE_CHECK_INSTANCE_CAST ((GXmlNode*) _tmp75_, GXML_TYPE_ELEMENT, GXmlElement);
#line 46 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp74_);
#line 46 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		c = _tmp76_;
#line 47 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (c != NULL, "c != null");
#line 48 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp77_ = gxml_node_get_name ((GXmlNode*) c);
#line 48 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp78_ = _tmp77_;
#line 48 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp79_ = _tmp78_;
#line 48 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp79_, "child") == 0, "c.name == \"child\"");
#line 48 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp79_);
#line 49 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp80_ = gxml_node_get_attrs ((GXmlNode*) c);
#line 49 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp81_ = _tmp80_;
#line 49 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp82_ = _tmp81_;
#line 49 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp83_ = gee_map_get (_tmp82_, "cattr1");
#line 49 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp84_ = (GXmlNode*) _tmp83_;
#line 49 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp84_ != NULL, "c.attrs.get (\"cattr1\") != null");
#line 49 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp84_);
#line 49 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp82_);
#line 50 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp85_ = gxml_node_get_attrs ((GXmlNode*) c);
#line 50 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp86_ = _tmp85_;
#line 50 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp87_ = _tmp86_;
#line 50 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp88_ = gee_map_get (_tmp87_, "cattr1");
#line 50 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp89_ = (GXmlNode*) _tmp88_;
#line 50 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp90_ = gxml_node_get_value (_tmp89_);
#line 50 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp91_ = _tmp90_;
#line 50 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp92_ = _tmp91_;
#line 50 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp92_, "cval1") == 0, "c.attrs.get (\"cattr1\").value == \"cval1\"");
#line 50 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp92_);
#line 50 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp89_);
#line 50 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp87_);
#line 51 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp93_ = e;
#line 51 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_element_remove_attr (_tmp93_, "cattr1");
#line 52 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp94_ = e;
#line 52 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp95_ = gxml_node_get_attrs ((GXmlNode*) _tmp94_);
#line 52 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp96_ = _tmp95_;
#line 52 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp97_ = _tmp96_;
#line 52 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp98_ = gee_map_get_size (_tmp97_);
#line 52 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp99_ = _tmp98_;
#line 52 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp99_ == 1, "e.attrs.size == 1");
#line 52 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp97_);
#line 53 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp100_ = e;
#line 53 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp101_ = gxml_node_get_attrs ((GXmlNode*) _tmp100_);
#line 53 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp102_ = _tmp101_;
#line 53 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp103_ = _tmp102_;
#line 53 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp104_ = gee_map_get (_tmp103_, "cattr1");
#line 53 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp105_ = (GXmlNode*) _tmp104_;
#line 53 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp105_ == NULL, "e.attrs.get (\"cattr1\") == null");
#line 53 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp105_);
#line 53 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp103_);
#line 54 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp106_ = gxml_element_get_content (c);
#line 54 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp107_ = _tmp106_;
#line 54 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp108_ = _tmp107_;
#line 54 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp108_, "") == 0, "c.content == \"\"");
#line 54 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp108_);
#line 55 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_element_set_content (c, "");
#line 56 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp109_ = gxml_element_get_content (c);
#line 56 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp110_ = _tmp109_;
#line 56 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp111_ = _tmp110_;
#line 56 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp111_, "") == 0, "c.content == \"\"");
#line 56 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp111_);
#line 57 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_element_set_content (c, "HELLO CONTENT");
#line 58 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp112_ = gxml_node_get_children_nodes ((GXmlNode*) c);
#line 58 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp113_ = _tmp112_;
#line 58 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp114_ = _tmp113_;
#line 58 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp115_ = gee_collection_get_size ((GeeCollection*) _tmp114_);
#line 58 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp116_ = _tmp115_;
#line 58 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp116_ == 1, "c.children_nodes.size == 1");
#line 58 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp114_);
#line 59 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp117_ = gxml_node_get_children_nodes ((GXmlNode*) c);
#line 59 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp118_ = _tmp117_;
#line 59 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp119_ = _tmp118_;
#line 59 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp120_ = gee_collection_get_size ((GeeCollection*) _tmp119_);
#line 59 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp121_ = _tmp120_;
#line 59 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp121_ == 1, "c.children_nodes.size == 1");
#line 59 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp119_);
#line 60 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp122_ = gxml_element_get_content (c);
#line 60 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp123_ = _tmp122_;
#line 60 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp124_ = _tmp123_;
#line 60 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp124_, "HELLO CONTENT") == 0, "c.content == \"HELLO CONTENT\"");
#line 60 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp124_);
#line 30 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (c);
#line 30 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (child);
#line 30 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp57_);
#line 30 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e);
#line 30 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp1_);
#line 30 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (d);
#line 687 "TElementTest.c"
	}
	goto __finally105;
	__catch105_g_error:
	{
#line 30 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 30 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_inner_error_ = NULL;
#line 61 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_assert_not_reached ();
#line 698 "TElementTest.c"
	}
	__finally105:
#line 30 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 30 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 30 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 30 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		return;
#line 709 "TElementTest.c"
	}
}


static void ___lambda82__gtest_func (void) {
#line 29 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	__lambda82_ ();
#line 717 "TElementTest.c"
}


static void __lambda83_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp0_ = NULL;
		GXmlNode* _tmp1_ = NULL;
		GXmlTDocument* _tmp2_ = NULL;
		GXmlNode* _tmp3_ = NULL;
		GXmlElement* e = NULL;
		GXmlNode* _tmp4_ = NULL;
		GXmlTDocument* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GeeBidirList* _tmp7_ = NULL;
		GeeBidirList* _tmp8_ = NULL;
		GXmlElement* _tmp9_ = NULL;
		GXmlTDocument* _tmp10_ = NULL;
		GeeBidirList* _tmp11_ = NULL;
		GeeBidirList* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		gint _tmp14_ = 0;
		gint _tmp15_ = 0;
		GXmlTDocument* _tmp16_ = NULL;
		GXmlNode* _tmp17_ = NULL;
		GXmlNode* _tmp18_ = NULL;
		GXmlNode* _tmp19_ = NULL;
		gchar* _tmp20_ = NULL;
		gchar* _tmp21_ = NULL;
		gchar* _tmp22_ = NULL;
		GXmlElement* _tmp23_ = NULL;
		GXmlElement* _tmp24_ = NULL;
		gchar* _tmp25_ = NULL;
		gchar* _tmp26_ = NULL;
		gchar* _tmp27_ = NULL;
		GXmlTDocument* _tmp28_ = NULL;
		GXmlNode* _tmp29_ = NULL;
		GXmlNode* _tmp30_ = NULL;
		GXmlNode* _tmp31_ = NULL;
		GeeBidirList* _tmp32_ = NULL;
		GeeBidirList* _tmp33_ = NULL;
		GeeBidirList* _tmp34_ = NULL;
		gint _tmp35_ = 0;
		gint _tmp36_ = 0;
		GXmlElement* _tmp37_ = NULL;
		GXmlTDocument* _tmp38_ = NULL;
		GXmlNode* _tmp39_ = NULL;
		GXmlNode* _tmp40_ = NULL;
		GXmlNode* _tmp41_ = NULL;
		GeeBidirList* _tmp42_ = NULL;
		GeeBidirList* _tmp43_ = NULL;
		GeeBidirList* _tmp44_ = NULL;
		gint _tmp45_ = 0;
		gint _tmp46_ = 0;
		GXmlElement* _tmp47_ = NULL;
		gchar* _tmp48_ = NULL;
		gchar* _tmp49_ = NULL;
		gchar* _tmp50_ = NULL;
		GXmlNode* t = NULL;
		GXmlTDocument* _tmp51_ = NULL;
		GXmlNode* _tmp52_ = NULL;
		GXmlElement* _tmp53_ = NULL;
		GeeBidirList* _tmp54_ = NULL;
		GeeBidirList* _tmp55_ = NULL;
		GeeBidirList* _tmp56_ = NULL;
		GXmlNode* _tmp57_ = NULL;
		GXmlElement* _tmp58_ = NULL;
		GeeBidirList* _tmp59_ = NULL;
		GeeBidirList* _tmp60_ = NULL;
		GeeBidirList* _tmp61_ = NULL;
		gint _tmp62_ = 0;
		gint _tmp63_ = 0;
		GXmlTDocument* _tmp64_ = NULL;
		GXmlNode* _tmp65_ = NULL;
		GXmlNode* _tmp66_ = NULL;
		GXmlNode* _tmp67_ = NULL;
		GeeBidirList* _tmp68_ = NULL;
		GeeBidirList* _tmp69_ = NULL;
		GeeBidirList* _tmp70_ = NULL;
		gint _tmp71_ = 0;
		gint _tmp72_ = 0;
		GXmlElement* _tmp73_ = NULL;
		gchar* _tmp74_ = NULL;
		gchar* _tmp75_ = NULL;
		gchar* _tmp76_ = NULL;
		GXmlElement* _tmp77_ = NULL;
		GeeBidirList* _tmp78_ = NULL;
		GeeBidirList* _tmp79_ = NULL;
		GeeBidirList* _tmp80_ = NULL;
		GXmlElement* _tmp81_ = NULL;
		GeeBidirList* _tmp82_ = NULL;
		GeeBidirList* _tmp83_ = NULL;
		GeeBidirList* _tmp84_ = NULL;
		gint _tmp85_ = 0;
		gint _tmp86_ = 0;
		GXmlElement* _tmp87_ = NULL;
		gchar* _tmp88_ = NULL;
		gchar* _tmp89_ = NULL;
		gchar* _tmp90_ = NULL;
		GXmlNode* c = NULL;
		GXmlTDocument* _tmp91_ = NULL;
		GXmlNode* _tmp92_ = NULL;
		GXmlElement* _tmp93_ = NULL;
		GeeBidirList* _tmp94_ = NULL;
		GeeBidirList* _tmp95_ = NULL;
		GeeBidirList* _tmp96_ = NULL;
		GXmlNode* _tmp97_ = NULL;
		GXmlElement* _tmp98_ = NULL;
		GXmlElement* _tmp99_ = NULL;
		GeeBidirList* _tmp100_ = NULL;
		GeeBidirList* _tmp101_ = NULL;
		GeeBidirList* _tmp102_ = NULL;
		gint _tmp103_ = 0;
		gint _tmp104_ = 0;
		GXmlElement* _tmp105_ = NULL;
		gchar* _tmp106_ = NULL;
		gchar* _tmp107_ = NULL;
		gchar* _tmp108_ = NULL;
		GXmlElement* _tmp109_ = NULL;
		GXmlElement* _tmp110_ = NULL;
		GeeBidirList* _tmp111_ = NULL;
		GeeBidirList* _tmp112_ = NULL;
		GeeBidirList* _tmp113_ = NULL;
		gint _tmp114_ = 0;
		gint _tmp115_ = 0;
		GXmlElement* _tmp116_ = NULL;
		GeeBidirList* _tmp117_ = NULL;
		GeeBidirList* _tmp118_ = NULL;
		GeeBidirList* _tmp119_ = NULL;
		GXmlElement* _tmp120_ = NULL;
		gchar* _tmp121_ = NULL;
		gchar* _tmp122_ = NULL;
		gchar* _tmp123_ = NULL;
		GXmlNode* t1 = NULL;
		GXmlTDocument* _tmp124_ = NULL;
		GXmlNode* _tmp125_ = NULL;
		GXmlNode* c1 = NULL;
		GXmlTDocument* _tmp126_ = NULL;
		GXmlNode* _tmp127_ = NULL;
		GXmlNode* t2 = NULL;
		GXmlTDocument* _tmp128_ = NULL;
		GXmlNode* _tmp129_ = NULL;
		GXmlElement* _tmp130_ = NULL;
		GeeBidirList* _tmp131_ = NULL;
		GeeBidirList* _tmp132_ = NULL;
		GeeBidirList* _tmp133_ = NULL;
		GXmlNode* _tmp134_ = NULL;
		GXmlElement* _tmp135_ = NULL;
		GeeBidirList* _tmp136_ = NULL;
		GeeBidirList* _tmp137_ = NULL;
		GeeBidirList* _tmp138_ = NULL;
		GXmlNode* _tmp139_ = NULL;
		GXmlElement* _tmp140_ = NULL;
		GeeBidirList* _tmp141_ = NULL;
		GeeBidirList* _tmp142_ = NULL;
		GeeBidirList* _tmp143_ = NULL;
		GXmlElement* _tmp144_ = NULL;
		GeeBidirList* _tmp145_ = NULL;
		GeeBidirList* _tmp146_ = NULL;
		GeeBidirList* _tmp147_ = NULL;
		gint _tmp148_ = 0;
		gint _tmp149_ = 0;
		GXmlElement* _tmp150_ = NULL;
		gchar* _tmp151_ = NULL;
		gchar* _tmp152_ = NULL;
		gchar* _tmp153_ = NULL;
		GXmlElement* _tmp154_ = NULL;
		GXmlElement* _tmp155_ = NULL;
		GeeBidirList* _tmp156_ = NULL;
		GeeBidirList* _tmp157_ = NULL;
		GeeBidirList* _tmp158_ = NULL;
		gint _tmp159_ = 0;
		gint _tmp160_ = 0;
#line 65 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp0_ = gxml_tdocument_new ();
#line 65 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		d = _tmp0_;
#line 66 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp2_ = d;
#line 66 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp3_ = gxml_document_create_element ((GXmlDocument*) _tmp2_, "element", &_inner_error_);
#line 66 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp1_ = _tmp3_;
#line 66 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 66 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 906 "TElementTest.c"
			goto __catch106_g_error;
		}
#line 66 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp4_ = _tmp1_;
#line 66 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp1_ = NULL;
#line 66 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e = G_TYPE_CHECK_INSTANCE_CAST (_tmp4_, GXML_TYPE_ELEMENT, GXmlElement);
#line 67 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp5_ = d;
#line 67 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp6_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp5_);
#line 67 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp7_ = _tmp6_;
#line 67 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp8_ = _tmp7_;
#line 67 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp9_ = e;
#line 67 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp8_, (GXmlNode*) _tmp9_);
#line 67 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp8_);
#line 68 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp10_ = d;
#line 68 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp11_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp10_);
#line 68 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp12_ = _tmp11_;
#line 68 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp13_ = _tmp12_;
#line 68 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp14_ = gee_collection_get_size ((GeeCollection*) _tmp13_);
#line 68 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp15_ = _tmp14_;
#line 68 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp15_ == 1, "d.children_nodes.size == 1");
#line 68 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp13_);
#line 69 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp16_ = d;
#line 69 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp17_ = gxml_document_get_root ((GXmlDocument*) _tmp16_);
#line 69 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp18_ = _tmp17_;
#line 69 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp19_ = _tmp18_;
#line 69 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp20_ = gxml_node_get_name (_tmp19_);
#line 69 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp21_ = _tmp20_;
#line 69 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp22_ = _tmp21_;
#line 69 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp22_, "element") == 0, "d.root.name == \"element\"");
#line 69 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp22_);
#line 69 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp19_);
#line 70 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp23_ = e;
#line 70 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_element_set_content (_tmp23_, "HELLO");
#line 71 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp24_ = e;
#line 71 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp25_ = gxml_element_get_content (_tmp24_);
#line 71 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp26_ = _tmp25_;
#line 71 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp27_ = _tmp26_;
#line 71 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp27_, "HELLO") == 0, "e.content == \"HELLO\"");
#line 71 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp27_);
#line 72 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp28_ = d;
#line 72 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp29_ = gxml_document_get_root ((GXmlDocument*) _tmp28_);
#line 72 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp30_ = _tmp29_;
#line 72 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp31_ = _tmp30_;
#line 72 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp32_ = gxml_node_get_children_nodes (_tmp31_);
#line 72 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp33_ = _tmp32_;
#line 72 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp34_ = _tmp33_;
#line 72 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp35_ = gee_collection_get_size ((GeeCollection*) _tmp34_);
#line 72 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp36_ = _tmp35_;
#line 72 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp36_ == 1, "d.root.children_nodes.size == 1");
#line 72 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp34_);
#line 72 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp31_);
#line 73 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp37_ = e;
#line 73 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_element_set_content (_tmp37_, "TIME");
#line 74 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp38_ = d;
#line 74 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp39_ = gxml_document_get_root ((GXmlDocument*) _tmp38_);
#line 74 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp40_ = _tmp39_;
#line 74 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp41_ = _tmp40_;
#line 74 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp42_ = gxml_node_get_children_nodes (_tmp41_);
#line 74 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp43_ = _tmp42_;
#line 74 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp44_ = _tmp43_;
#line 74 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp45_ = gee_collection_get_size ((GeeCollection*) _tmp44_);
#line 74 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp46_ = _tmp45_;
#line 74 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp46_ == 1, "d.root.children_nodes.size == 1");
#line 74 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp44_);
#line 74 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp41_);
#line 75 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp47_ = e;
#line 75 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp48_ = gxml_element_get_content (_tmp47_);
#line 75 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp49_ = _tmp48_;
#line 75 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp50_ = _tmp49_;
#line 75 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp50_, "TIME") == 0, "e.content == \"TIME\"");
#line 75 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp50_);
#line 76 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp51_ = d;
#line 76 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp52_ = gxml_document_create_text ((GXmlDocument*) _tmp51_, " OTHER");
#line 76 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		t = _tmp52_;
#line 77 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp53_ = e;
#line 77 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp54_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp53_);
#line 77 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp55_ = _tmp54_;
#line 77 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp56_ = _tmp55_;
#line 77 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp57_ = t;
#line 77 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp56_, _tmp57_);
#line 77 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp56_);
#line 78 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp58_ = e;
#line 78 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp59_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp58_);
#line 78 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp60_ = _tmp59_;
#line 78 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp61_ = _tmp60_;
#line 78 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp62_ = gee_collection_get_size ((GeeCollection*) _tmp61_);
#line 78 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp63_ = _tmp62_;
#line 78 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp63_ == 2, "e.children_nodes.size == 2");
#line 78 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp61_);
#line 79 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp64_ = d;
#line 79 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp65_ = gxml_document_get_root ((GXmlDocument*) _tmp64_);
#line 79 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp66_ = _tmp65_;
#line 79 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp67_ = _tmp66_;
#line 79 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp68_ = gxml_node_get_children_nodes (_tmp67_);
#line 79 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp69_ = _tmp68_;
#line 79 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp70_ = _tmp69_;
#line 79 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp71_ = gee_collection_get_size ((GeeCollection*) _tmp70_);
#line 79 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp72_ = _tmp71_;
#line 79 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp72_ == 2, "d.root.children_nodes.size == 2");
#line 79 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp70_);
#line 79 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp67_);
#line 80 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp73_ = e;
#line 80 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp74_ = gxml_element_get_content (_tmp73_);
#line 80 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp75_ = _tmp74_;
#line 80 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp76_ = _tmp75_;
#line 80 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp76_, "TIME OTHER") == 0, "e.content == \"TIME OTHER\"");
#line 80 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp76_);
#line 81 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp77_ = e;
#line 81 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp78_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp77_);
#line 81 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp79_ = _tmp78_;
#line 81 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp80_ = _tmp79_;
#line 81 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_clear ((GeeCollection*) _tmp80_);
#line 81 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp80_);
#line 82 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp81_ = e;
#line 82 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp82_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp81_);
#line 82 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp83_ = _tmp82_;
#line 82 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp84_ = _tmp83_;
#line 82 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp85_ = gee_collection_get_size ((GeeCollection*) _tmp84_);
#line 82 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp86_ = _tmp85_;
#line 82 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp86_ == 0, "e.children_nodes.size == 0");
#line 82 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp84_);
#line 83 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp87_ = e;
#line 83 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp88_ = gxml_element_get_content (_tmp87_);
#line 83 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp89_ = _tmp88_;
#line 83 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp90_ = _tmp89_;
#line 83 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp90_, "") == 0, "e.content == \"\"");
#line 83 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp90_);
#line 84 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp91_ = d;
#line 84 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp92_ = gxml_document_create_element ((GXmlDocument*) _tmp91_, "child", &_inner_error_);
#line 84 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		c = _tmp92_;
#line 84 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 84 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (t);
#line 84 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 84 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (_tmp1_);
#line 84 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 1173 "TElementTest.c"
			goto __catch106_g_error;
		}
#line 85 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp93_ = e;
#line 85 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp94_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp93_);
#line 85 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp95_ = _tmp94_;
#line 85 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp96_ = _tmp95_;
#line 85 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp97_ = c;
#line 85 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp96_, _tmp97_);
#line 85 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp96_);
#line 86 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp98_ = e;
#line 86 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_element_set_content (_tmp98_, "KNOW");
#line 87 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp99_ = e;
#line 87 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp100_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp99_);
#line 87 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp101_ = _tmp100_;
#line 87 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp102_ = _tmp101_;
#line 87 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp103_ = gee_collection_get_size ((GeeCollection*) _tmp102_);
#line 87 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp104_ = _tmp103_;
#line 87 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp104_ == 2, "e.children_nodes.size == 2");
#line 87 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp102_);
#line 88 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp105_ = e;
#line 88 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp106_ = gxml_element_get_content (_tmp105_);
#line 88 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp107_ = _tmp106_;
#line 88 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp108_ = _tmp107_;
#line 88 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp108_, "KNOW") == 0, "e.content == \"KNOW\"");
#line 88 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp108_);
#line 89 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp109_ = e;
#line 89 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_element_set_content (_tmp109_, "");
#line 90 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp110_ = e;
#line 90 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp111_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp110_);
#line 90 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp112_ = _tmp111_;
#line 90 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp113_ = _tmp112_;
#line 90 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp114_ = gee_collection_get_size ((GeeCollection*) _tmp113_);
#line 90 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp115_ = _tmp114_;
#line 90 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp115_ == 2, "e.children_nodes.size == 2");
#line 90 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp113_);
#line 91 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp116_ = e;
#line 91 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp117_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp116_);
#line 91 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp118_ = _tmp117_;
#line 91 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp119_ = _tmp118_;
#line 91 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_clear ((GeeCollection*) _tmp119_);
#line 91 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp119_);
#line 92 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp120_ = e;
#line 92 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp121_ = gxml_element_get_content (_tmp120_);
#line 92 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp122_ = _tmp121_;
#line 92 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp123_ = _tmp122_;
#line 92 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp123_, "") == 0, "e.content == \"\"");
#line 92 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp123_);
#line 93 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp124_ = d;
#line 93 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp125_ = gxml_document_create_text ((GXmlDocument*) _tmp124_, "TEXT1");
#line 93 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		t1 = _tmp125_;
#line 94 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp126_ = d;
#line 94 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp127_ = gxml_document_create_element ((GXmlDocument*) _tmp126_, "child2", &_inner_error_);
#line 94 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		c1 = _tmp127_;
#line 94 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 94 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (t1);
#line 94 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (c);
#line 94 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (t);
#line 94 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 94 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (_tmp1_);
#line 94 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 1292 "TElementTest.c"
			goto __catch106_g_error;
		}
#line 95 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp128_ = d;
#line 95 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp129_ = gxml_document_create_text ((GXmlDocument*) _tmp128_, "TEXT2");
#line 95 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		t2 = _tmp129_;
#line 96 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp130_ = e;
#line 96 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp131_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp130_);
#line 96 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp132_ = _tmp131_;
#line 96 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp133_ = _tmp132_;
#line 96 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp134_ = t1;
#line 96 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp133_, _tmp134_);
#line 96 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp133_);
#line 97 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp135_ = e;
#line 97 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp136_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp135_);
#line 97 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp137_ = _tmp136_;
#line 97 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp138_ = _tmp137_;
#line 97 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp139_ = c1;
#line 97 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp138_, _tmp139_);
#line 97 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp138_);
#line 98 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp140_ = e;
#line 98 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp141_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp140_);
#line 98 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp142_ = _tmp141_;
#line 98 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp143_ = _tmp142_;
#line 98 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp143_, t2);
#line 98 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp143_);
#line 99 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp144_ = e;
#line 99 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp145_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp144_);
#line 99 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp146_ = _tmp145_;
#line 99 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp147_ = _tmp146_;
#line 99 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp148_ = gee_collection_get_size ((GeeCollection*) _tmp147_);
#line 99 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp149_ = _tmp148_;
#line 99 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp149_ == 3, "e.children_nodes.size == 3");
#line 99 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp147_);
#line 100 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp150_ = e;
#line 100 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp151_ = gxml_element_get_content (_tmp150_);
#line 100 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp152_ = _tmp151_;
#line 100 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp153_ = _tmp152_;
#line 100 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp153_, "TEXT1TEXT2") == 0, "e.content == \"TEXT1TEXT2\"");
#line 100 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp153_);
#line 101 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp154_ = e;
#line 101 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_element_set_content (_tmp154_, NULL);
#line 102 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp155_ = e;
#line 102 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp156_ = gxml_node_get_children_nodes ((GXmlNode*) _tmp155_);
#line 102 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp157_ = _tmp156_;
#line 102 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp158_ = _tmp157_;
#line 102 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp159_ = gee_collection_get_size ((GeeCollection*) _tmp158_);
#line 102 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp160_ = _tmp159_;
#line 102 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp160_ == 1, "e.children_nodes.size == 1");
#line 102 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp158_);
#line 64 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (t2);
#line 64 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (c1);
#line 64 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (t1);
#line 64 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (c);
#line 64 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (t);
#line 64 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e);
#line 64 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp1_);
#line 64 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (d);
#line 1405 "TElementTest.c"
	}
	goto __finally106;
	__catch106_g_error:
	{
#line 64 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 64 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_inner_error_ = NULL;
#line 103 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_assert_not_reached ();
#line 1416 "TElementTest.c"
	}
	__finally106:
#line 64 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 64 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 64 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 64 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		return;
#line 1427 "TElementTest.c"
	}
}


static void ___lambda83__gtest_func (void) {
#line 63 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	__lambda83_ ();
#line 1435 "TElementTest.c"
}


static gboolean string_contains (const gchar* self, const gchar* needle) {
	gboolean result = FALSE;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 1376 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1376 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	g_return_val_if_fail (needle != NULL, FALSE);
#line 1377 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	_tmp0_ = needle;
#line 1377 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	_tmp1_ = strstr ((gchar*) self, (gchar*) _tmp0_);
#line 1377 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	result = _tmp1_ != NULL;
#line 1377 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	return result;
#line 1455 "TElementTest.c"
}


static void __lambda84_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp0_ = NULL;
		GXmlNode* r = NULL;
		GXmlTDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlTDocument* _tmp3_ = NULL;
		GeeBidirList* _tmp4_ = NULL;
		GeeBidirList* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlTDocument* _tmp8_ = NULL;
		GXmlNode* e = NULL;
		GXmlTDocument* _tmp9_ = NULL;
		GXmlNode* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GeeBidirList* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GXmlNode* _tmp15_ = NULL;
		GXmlTDocument* _tmp16_ = NULL;
		GeeList* _tmp17_ = NULL;
		GeeList* _tmp18_ = NULL;
		GeeList* _tmp19_ = NULL;
		gint _tmp20_ = 0;
		gint _tmp21_ = 0;
		gchar* str = NULL;
		GXmlTDocument* _tmp22_ = NULL;
		gchar* _tmp23_ = NULL;
		gboolean _tmp24_ = FALSE;
#line 107 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp0_ = gxml_tdocument_new ();
#line 107 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		d = _tmp0_;
#line 108 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp1_ = d;
#line 108 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp2_ = gxml_document_create_element ((GXmlDocument*) _tmp1_, "root", &_inner_error_);
#line 108 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		r = _tmp2_;
#line 108 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 108 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 1505 "TElementTest.c"
			goto __catch107_g_error;
		}
#line 109 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp3_ = d;
#line 109 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp4_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp3_);
#line 109 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp5_ = _tmp4_;
#line 109 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp6_ = _tmp5_;
#line 109 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp7_ = r;
#line 109 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp6_, _tmp7_);
#line 109 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp6_);
#line 111 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp8_ = d;
#line 111 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_tnode_set_namespace ((GXmlTNode*) _tmp8_, "http://www.gnome.org/gxml", NULL);
#line 112 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp9_ = d;
#line 112 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp10_ = gxml_document_create_element ((GXmlDocument*) _tmp9_, "child", &_inner_error_);
#line 112 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e = _tmp10_;
#line 112 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 112 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 112 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 1538 "TElementTest.c"
			goto __catch107_g_error;
		}
#line 113 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp11_ = r;
#line 113 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp12_ = gxml_node_get_children_nodes (_tmp11_);
#line 113 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp13_ = _tmp12_;
#line 113 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp14_ = _tmp13_;
#line 113 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp15_ = e;
#line 113 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp14_, _tmp15_);
#line 113 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp14_);
#line 114 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp16_ = d;
#line 114 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp17_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp16_);
#line 114 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp18_ = _tmp17_;
#line 114 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp19_ = _tmp18_;
#line 114 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp20_ = gee_collection_get_size ((GeeCollection*) _tmp19_);
#line 114 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp21_ = _tmp20_;
#line 114 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp21_ == 1, "d.namespaces.size == 1");
#line 114 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp19_);
#line 115 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp22_ = d;
#line 115 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp23_ = gxml_tnode_to_string ((GXmlTNode*) _tmp22_);
#line 115 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		str = _tmp23_;
#line 119 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp24_ = string_contains (str, "<root xmlns=\"http://www.gnome.org/gxml\">");
#line 119 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp24_, "\"<root xmlns=\\\"http://www.gnome.org/gxml\\\">\" in str");
#line 106 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (str);
#line 106 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e);
#line 106 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (r);
#line 106 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (d);
#line 1589 "TElementTest.c"
	}
	goto __finally107;
	__catch107_g_error:
	{
#line 106 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 106 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_inner_error_ = NULL;
#line 120 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_assert_not_reached ();
#line 1600 "TElementTest.c"
	}
	__finally107:
#line 106 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 106 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 106 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 106 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		return;
#line 1611 "TElementTest.c"
	}
}


static void ___lambda84__gtest_func (void) {
#line 105 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	__lambda84_ ();
#line 1619 "TElementTest.c"
}


static void __lambda85_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp0_ = NULL;
		GXmlNode* r = NULL;
		GXmlTDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlTDocument* _tmp3_ = NULL;
		GeeBidirList* _tmp4_ = NULL;
		GeeBidirList* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlTDocument* _tmp8_ = NULL;
		GXmlNode* e = NULL;
		GXmlTDocument* _tmp9_ = NULL;
		GXmlNode* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GeeBidirList* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GXmlNode* _tmp15_ = NULL;
		GXmlTDocument* _tmp16_ = NULL;
		GeeList* _tmp17_ = NULL;
		GeeList* _tmp18_ = NULL;
		GeeList* _tmp19_ = NULL;
		gint _tmp20_ = 0;
		gint _tmp21_ = 0;
		gchar* str = NULL;
		GXmlTDocument* _tmp22_ = NULL;
		gchar* _tmp23_ = NULL;
		gboolean _tmp24_ = FALSE;
#line 124 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp0_ = gxml_tdocument_new ();
#line 124 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		d = _tmp0_;
#line 125 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp1_ = d;
#line 125 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp2_ = gxml_document_create_element ((GXmlDocument*) _tmp1_, "root", &_inner_error_);
#line 125 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		r = _tmp2_;
#line 125 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 125 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 1669 "TElementTest.c"
			goto __catch108_g_error;
		}
#line 126 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp3_ = d;
#line 126 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp4_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp3_);
#line 126 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp5_ = _tmp4_;
#line 126 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp6_ = _tmp5_;
#line 126 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp7_ = r;
#line 126 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp6_, _tmp7_);
#line 126 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp6_);
#line 128 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp8_ = d;
#line 128 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_tnode_set_namespace ((GXmlTNode*) _tmp8_, "http://www.gnome.org/gxml", "gxml");
#line 129 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp9_ = d;
#line 129 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp10_ = gxml_document_create_element ((GXmlDocument*) _tmp9_, "child", &_inner_error_);
#line 129 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e = _tmp10_;
#line 129 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 129 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 129 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 1702 "TElementTest.c"
			goto __catch108_g_error;
		}
#line 130 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp11_ = r;
#line 130 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp12_ = gxml_node_get_children_nodes (_tmp11_);
#line 130 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp13_ = _tmp12_;
#line 130 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp14_ = _tmp13_;
#line 130 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp15_ = e;
#line 130 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp14_, _tmp15_);
#line 130 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp14_);
#line 131 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp16_ = d;
#line 131 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp17_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp16_);
#line 131 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp18_ = _tmp17_;
#line 131 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp19_ = _tmp18_;
#line 131 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp20_ = gee_collection_get_size ((GeeCollection*) _tmp19_);
#line 131 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp21_ = _tmp20_;
#line 131 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp21_ == 1, "d.namespaces.size == 1");
#line 131 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp19_);
#line 132 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp22_ = d;
#line 132 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp23_ = gxml_tnode_to_string ((GXmlTNode*) _tmp22_);
#line 132 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		str = _tmp23_;
#line 136 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp24_ = string_contains (str, "<root xmlns:gxml=\"http://www.gnome.org/gxml\">");
#line 136 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp24_, "\"<root xmlns:gxml=\\\"http://www.gnome.org/gxml\\\">\" in str");
#line 123 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (str);
#line 123 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e);
#line 123 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (r);
#line 123 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (d);
#line 1753 "TElementTest.c"
	}
	goto __finally108;
	__catch108_g_error:
	{
#line 123 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 123 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_inner_error_ = NULL;
#line 137 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_assert_not_reached ();
#line 1764 "TElementTest.c"
	}
	__finally108:
#line 123 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 123 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 123 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 123 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		return;
#line 1775 "TElementTest.c"
	}
}


static void ___lambda85__gtest_func (void) {
#line 122 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	__lambda85_ ();
#line 1783 "TElementTest.c"
}


static void __lambda86_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp0_ = NULL;
		GXmlNode* r = NULL;
		GXmlTDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlTDocument* _tmp3_ = NULL;
		GeeBidirList* _tmp4_ = NULL;
		GeeBidirList* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlTDocument* _tmp8_ = NULL;
		GXmlNode* e = NULL;
		GXmlTDocument* _tmp9_ = NULL;
		GXmlNode* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GeeBidirList* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GXmlNode* _tmp15_ = NULL;
		GXmlTDocument* _tmp16_ = NULL;
		GeeList* _tmp17_ = NULL;
		GeeList* _tmp18_ = NULL;
		GeeList* _tmp19_ = NULL;
		gint _tmp20_ = 0;
		gint _tmp21_ = 0;
		gchar* str = NULL;
		GXmlTDocument* _tmp22_ = NULL;
		gchar* _tmp23_ = NULL;
		gboolean _tmp24_ = FALSE;
#line 141 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp0_ = gxml_tdocument_new ();
#line 141 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		d = _tmp0_;
#line 142 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp1_ = d;
#line 142 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp2_ = gxml_document_create_element ((GXmlDocument*) _tmp1_, "root", &_inner_error_);
#line 142 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		r = _tmp2_;
#line 142 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 142 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 1833 "TElementTest.c"
			goto __catch109_g_error;
		}
#line 143 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp3_ = d;
#line 143 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp4_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp3_);
#line 143 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp5_ = _tmp4_;
#line 143 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp6_ = _tmp5_;
#line 143 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp7_ = r;
#line 143 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp6_, _tmp7_);
#line 143 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp6_);
#line 145 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp8_ = d;
#line 145 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_tnode_set_namespace ((GXmlTNode*) _tmp8_, "http://www.gnome.org/gxml", NULL);
#line 146 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp9_ = d;
#line 146 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp10_ = gxml_document_create_element ((GXmlDocument*) _tmp9_, "child", &_inner_error_);
#line 146 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e = _tmp10_;
#line 146 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 146 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 146 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 1866 "TElementTest.c"
			goto __catch109_g_error;
		}
#line 147 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp11_ = r;
#line 147 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp12_ = gxml_node_get_children_nodes (_tmp11_);
#line 147 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp13_ = _tmp12_;
#line 147 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp14_ = _tmp13_;
#line 147 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp15_ = e;
#line 147 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp14_, _tmp15_);
#line 147 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp14_);
#line 148 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp16_ = d;
#line 148 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp17_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp16_);
#line 148 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp18_ = _tmp17_;
#line 148 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp19_ = _tmp18_;
#line 148 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp20_ = gee_collection_get_size ((GeeCollection*) _tmp19_);
#line 148 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp21_ = _tmp20_;
#line 148 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp21_ == 1, "d.namespaces.size == 1");
#line 148 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp19_);
#line 149 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp22_ = d;
#line 149 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp23_ = gxml_tnode_to_string ((GXmlTNode*) _tmp22_);
#line 149 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		str = _tmp23_;
#line 153 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp24_ = string_contains (str, "<root xmlns=\"http://www.gnome.org/gxml\">");
#line 153 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp24_, "\"<root xmlns=\\\"http://www.gnome.org/gxml\\\">\" in str");
#line 140 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (str);
#line 140 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e);
#line 140 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (r);
#line 140 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (d);
#line 1917 "TElementTest.c"
	}
	goto __finally109;
	__catch109_g_error:
	{
#line 140 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 140 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_inner_error_ = NULL;
#line 154 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_assert_not_reached ();
#line 1928 "TElementTest.c"
	}
	__finally109:
#line 140 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 140 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 140 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 140 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		return;
#line 1939 "TElementTest.c"
	}
}


static void ___lambda86__gtest_func (void) {
#line 139 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	__lambda86_ ();
#line 1947 "TElementTest.c"
}


static void __lambda87_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp0_ = NULL;
		GXmlNode* r = NULL;
		GXmlTDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlTDocument* _tmp3_ = NULL;
		GeeBidirList* _tmp4_ = NULL;
		GeeBidirList* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlTDocument* _tmp8_ = NULL;
		GXmlTDocument* _tmp9_ = NULL;
		GXmlNode* e = NULL;
		GXmlTDocument* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GXmlNode* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GeeBidirList* _tmp15_ = NULL;
		GXmlNode* _tmp16_ = NULL;
		GXmlTDocument* _tmp17_ = NULL;
		GeeList* _tmp18_ = NULL;
		GeeList* _tmp19_ = NULL;
		GeeList* _tmp20_ = NULL;
		gint _tmp21_ = 0;
		gint _tmp22_ = 0;
		gchar* str = NULL;
		GXmlTDocument* _tmp23_ = NULL;
		gchar* _tmp24_ = NULL;
		gboolean _tmp25_ = FALSE;
#line 158 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp0_ = gxml_tdocument_new ();
#line 158 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		d = _tmp0_;
#line 159 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp1_ = d;
#line 159 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp2_ = gxml_document_create_element ((GXmlDocument*) _tmp1_, "root", &_inner_error_);
#line 159 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		r = _tmp2_;
#line 159 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 159 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 1998 "TElementTest.c"
			goto __catch110_g_error;
		}
#line 160 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp3_ = d;
#line 160 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp4_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp3_);
#line 160 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp5_ = _tmp4_;
#line 160 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp6_ = _tmp5_;
#line 160 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp7_ = r;
#line 160 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp6_, _tmp7_);
#line 160 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp6_);
#line 162 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp8_ = d;
#line 162 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_tnode_set_namespace ((GXmlTNode*) _tmp8_, "http://www.gnome.org/gxml", "gxml");
#line 163 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp9_ = d;
#line 163 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_document_set_prefix_default_ns ((GXmlDocument*) _tmp9_, TRUE);
#line 164 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp10_ = d;
#line 164 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp11_ = gxml_document_create_element ((GXmlDocument*) _tmp10_, "child", &_inner_error_);
#line 164 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e = _tmp11_;
#line 164 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 164 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 164 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 2035 "TElementTest.c"
			goto __catch110_g_error;
		}
#line 165 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp12_ = r;
#line 165 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp13_ = gxml_node_get_children_nodes (_tmp12_);
#line 165 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp14_ = _tmp13_;
#line 165 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp15_ = _tmp14_;
#line 165 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp16_ = e;
#line 165 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp15_, _tmp16_);
#line 165 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp15_);
#line 166 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp17_ = d;
#line 166 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp18_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp17_);
#line 166 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp19_ = _tmp18_;
#line 166 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp20_ = _tmp19_;
#line 166 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp21_ = gee_collection_get_size ((GeeCollection*) _tmp20_);
#line 166 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp22_ = _tmp21_;
#line 166 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp22_ == 1, "d.namespaces.size == 1");
#line 166 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp20_);
#line 167 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp23_ = d;
#line 167 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp24_ = gxml_tnode_to_string ((GXmlTNode*) _tmp23_);
#line 167 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		str = _tmp24_;
#line 171 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp25_ = string_contains (str, "<gxml:root xmlns:gxml=\"http://www.gnome.org/gxml\">");
#line 171 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp25_, "\"<gxml:root xmlns:gxml=\\\"http://www.gnome.org/gxml\\\">\" in str");
#line 157 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (str);
#line 157 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e);
#line 157 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (r);
#line 157 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (d);
#line 2086 "TElementTest.c"
	}
	goto __finally110;
	__catch110_g_error:
	{
#line 157 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 157 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_inner_error_ = NULL;
#line 172 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_assert_not_reached ();
#line 2097 "TElementTest.c"
	}
	__finally110:
#line 157 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 157 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 157 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 157 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		return;
#line 2108 "TElementTest.c"
	}
}


static void ___lambda87__gtest_func (void) {
#line 156 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	__lambda87_ ();
#line 2116 "TElementTest.c"
}


static void __lambda88_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp0_ = NULL;
		GXmlNode* r = NULL;
		GXmlTDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlTDocument* _tmp3_ = NULL;
		GeeBidirList* _tmp4_ = NULL;
		GeeBidirList* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlNode* _tmp8_ = NULL;
		GXmlNode* e = NULL;
		GXmlTDocument* _tmp9_ = NULL;
		GXmlNode* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GeeBidirList* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GXmlNode* _tmp15_ = NULL;
		GXmlNode* _tmp16_ = NULL;
		GeeList* _tmp17_ = NULL;
		GeeList* _tmp18_ = NULL;
		GeeList* _tmp19_ = NULL;
		gint _tmp20_ = 0;
		gint _tmp21_ = 0;
		GXmlTDocument* _tmp22_ = NULL;
		GeeList* _tmp23_ = NULL;
		GeeList* _tmp24_ = NULL;
		GeeList* _tmp25_ = NULL;
		gint _tmp26_ = 0;
		gint _tmp27_ = 0;
		GXmlNode* _tmp28_ = NULL;
		GXmlNode* _tmp29_ = NULL;
		GeeList* _tmp30_ = NULL;
		GeeList* _tmp31_ = NULL;
		GeeList* _tmp32_ = NULL;
		gint _tmp33_ = 0;
		gint _tmp34_ = 0;
		GXmlNode* _tmp35_ = NULL;
		GeeList* _tmp36_ = NULL;
		GeeList* _tmp37_ = NULL;
		GeeList* _tmp38_ = NULL;
		gint _tmp39_ = 0;
		gint _tmp40_ = 0;
		GXmlTDocument* _tmp41_ = NULL;
		GeeList* _tmp42_ = NULL;
		GeeList* _tmp43_ = NULL;
		GeeList* _tmp44_ = NULL;
		gint _tmp45_ = 0;
		gint _tmp46_ = 0;
		GXmlNode* e2 = NULL;
		GXmlTDocument* _tmp47_ = NULL;
		GXmlNode* _tmp48_ = NULL;
		GXmlNode* _tmp49_ = NULL;
		GeeBidirList* _tmp50_ = NULL;
		GeeBidirList* _tmp51_ = NULL;
		GeeBidirList* _tmp52_ = NULL;
		GXmlNode* _tmp53_ = NULL;
		GXmlNode* _tmp54_ = NULL;
		GXmlNode* _tmp55_ = NULL;
		GeeList* _tmp56_ = NULL;
		GeeList* _tmp57_ = NULL;
		GeeList* _tmp58_ = NULL;
		gint _tmp59_ = 0;
		gint _tmp60_ = 0;
		GXmlNode* _tmp61_ = NULL;
		GeeList* _tmp62_ = NULL;
		GeeList* _tmp63_ = NULL;
		GeeList* _tmp64_ = NULL;
		gint _tmp65_ = 0;
		gint _tmp66_ = 0;
		GXmlNode* _tmp67_ = NULL;
		GeeList* _tmp68_ = NULL;
		GeeList* _tmp69_ = NULL;
		GeeList* _tmp70_ = NULL;
		gint _tmp71_ = 0;
		gint _tmp72_ = 0;
		GXmlTDocument* _tmp73_ = NULL;
		GeeList* _tmp74_ = NULL;
		GeeList* _tmp75_ = NULL;
		GeeList* _tmp76_ = NULL;
		gint _tmp77_ = 0;
		gint _tmp78_ = 0;
		gchar* str = NULL;
		GXmlTDocument* _tmp79_ = NULL;
		gchar* _tmp80_ = NULL;
		gboolean _tmp81_ = FALSE;
		gboolean _tmp82_ = FALSE;
		gboolean _tmp83_ = FALSE;
		gboolean _tmp84_ = FALSE;
#line 176 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp0_ = gxml_tdocument_new ();
#line 176 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		d = _tmp0_;
#line 177 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp1_ = d;
#line 177 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp2_ = gxml_document_create_element ((GXmlDocument*) _tmp1_, "root", &_inner_error_);
#line 177 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		r = _tmp2_;
#line 177 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 177 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 2227 "TElementTest.c"
			goto __catch111_g_error;
		}
#line 178 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp3_ = d;
#line 178 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp4_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp3_);
#line 178 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp5_ = _tmp4_;
#line 178 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp6_ = _tmp5_;
#line 178 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp7_ = r;
#line 178 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp6_, _tmp7_);
#line 178 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp6_);
#line 179 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp8_ = r;
#line 179 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp8_, "http://git.gnome.org/browse/gxml", "gxml");
#line 180 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp9_ = d;
#line 180 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp10_ = gxml_document_create_element ((GXmlDocument*) _tmp9_, "child", &_inner_error_);
#line 180 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e = _tmp10_;
#line 180 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 180 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 180 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 2260 "TElementTest.c"
			goto __catch111_g_error;
		}
#line 181 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp11_ = r;
#line 181 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp12_ = gxml_node_get_children_nodes (_tmp11_);
#line 181 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp13_ = _tmp12_;
#line 181 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp14_ = _tmp13_;
#line 181 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp15_ = e;
#line 181 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp14_, _tmp15_);
#line 181 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp14_);
#line 182 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp16_ = r;
#line 182 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp17_ = gxml_node_get_namespaces (_tmp16_);
#line 182 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp18_ = _tmp17_;
#line 182 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp19_ = _tmp18_;
#line 182 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp20_ = gee_collection_get_size ((GeeCollection*) _tmp19_);
#line 182 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp21_ = _tmp20_;
#line 182 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp21_ == 1, "r.namespaces.size == 1");
#line 182 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp19_);
#line 183 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp22_ = d;
#line 183 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp23_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp22_);
#line 183 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp24_ = _tmp23_;
#line 183 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp25_ = _tmp24_;
#line 183 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp26_ = gee_collection_get_size ((GeeCollection*) _tmp25_);
#line 183 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp27_ = _tmp26_;
#line 183 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp27_ == 1, "d.namespaces.size == 1");
#line 183 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp25_);
#line 184 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp28_ = e;
#line 184 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp28_, "http://developer.gnome.org/", "dg");
#line 185 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp29_ = e;
#line 185 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp30_ = gxml_node_get_namespaces (_tmp29_);
#line 185 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp31_ = _tmp30_;
#line 185 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp32_ = _tmp31_;
#line 185 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp33_ = gee_collection_get_size ((GeeCollection*) _tmp32_);
#line 185 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp34_ = _tmp33_;
#line 185 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp34_ == 1, "e.namespaces.size == 1");
#line 185 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp32_);
#line 186 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp35_ = r;
#line 186 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp36_ = gxml_node_get_namespaces (_tmp35_);
#line 186 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp37_ = _tmp36_;
#line 186 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp38_ = _tmp37_;
#line 186 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp39_ = gee_collection_get_size ((GeeCollection*) _tmp38_);
#line 186 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp40_ = _tmp39_;
#line 186 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp40_ == 1, "r.namespaces.size == 1");
#line 186 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp38_);
#line 187 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp41_ = d;
#line 187 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp42_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp41_);
#line 187 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp43_ = _tmp42_;
#line 187 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp44_ = _tmp43_;
#line 187 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp45_ = gee_collection_get_size ((GeeCollection*) _tmp44_);
#line 187 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp46_ = _tmp45_;
#line 187 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp46_ == 2, "d.namespaces.size == 2");
#line 187 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp44_);
#line 188 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp47_ = d;
#line 188 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp48_ = gxml_document_create_element ((GXmlDocument*) _tmp47_, "nons", &_inner_error_);
#line 188 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e2 = _tmp48_;
#line 188 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 188 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 188 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 188 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 2375 "TElementTest.c"
			goto __catch111_g_error;
		}
#line 189 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp49_ = e;
#line 189 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp50_ = gxml_node_get_children_nodes (_tmp49_);
#line 189 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp51_ = _tmp50_;
#line 189 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp52_ = _tmp51_;
#line 189 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp53_ = e2;
#line 189 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp52_, _tmp53_);
#line 189 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp52_);
#line 190 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp54_ = e2;
#line 190 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp54_, "http://www.gnome.org/", NULL);
#line 191 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp55_ = e;
#line 191 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp56_ = gxml_node_get_namespaces (_tmp55_);
#line 191 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp57_ = _tmp56_;
#line 191 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp58_ = _tmp57_;
#line 191 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp59_ = gee_collection_get_size ((GeeCollection*) _tmp58_);
#line 191 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp60_ = _tmp59_;
#line 191 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp60_ == 1, "e.namespaces.size == 1");
#line 191 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp58_);
#line 192 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp61_ = r;
#line 192 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp62_ = gxml_node_get_namespaces (_tmp61_);
#line 192 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp63_ = _tmp62_;
#line 192 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp64_ = _tmp63_;
#line 192 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp65_ = gee_collection_get_size ((GeeCollection*) _tmp64_);
#line 192 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp66_ = _tmp65_;
#line 192 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp66_ == 1, "r.namespaces.size == 1");
#line 192 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp64_);
#line 193 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp67_ = e2;
#line 193 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp68_ = gxml_node_get_namespaces (_tmp67_);
#line 193 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp69_ = _tmp68_;
#line 193 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp70_ = _tmp69_;
#line 193 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp71_ = gee_collection_get_size ((GeeCollection*) _tmp70_);
#line 193 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp72_ = _tmp71_;
#line 193 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp72_ == 1, "e2.namespaces.size == 1");
#line 193 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp70_);
#line 194 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp73_ = d;
#line 194 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp74_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp73_);
#line 194 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp75_ = _tmp74_;
#line 194 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp76_ = _tmp75_;
#line 194 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp77_ = gee_collection_get_size ((GeeCollection*) _tmp76_);
#line 194 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp78_ = _tmp77_;
#line 194 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp78_ == 3, "d.namespaces.size == 3");
#line 194 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp76_);
#line 195 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp79_ = d;
#line 195 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp80_ = gxml_tnode_to_string ((GXmlTNode*) _tmp79_);
#line 195 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		str = _tmp80_;
#line 199 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp81_ = string_contains (str, "<root xmlns:gxml=\"http://git.gnome.org/browse/gxml\">");
#line 199 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp81_, "\"<root xmlns:gxml=\\\"http://git.gnome.org/browse/gxml\\\">\" in str");
#line 200 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp82_ = string_contains (str, "</root>");
#line 200 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp82_, "\"</root>\" in str");
#line 201 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp83_ = string_contains (str, "<dg:child xmlns:dg=\"http://developer.gnome.org/\">");
#line 201 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp83_, "\"<dg:child xmlns:dg=\\\"http://developer.gnome.org/\\\">\" in str");
#line 202 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp84_ = string_contains (str, "<nons xmlns=\"http://www.gnome.org/\"/>");
#line 202 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp84_, "\"<nons xmlns=\\\"http://www.gnome.org/\\\"/>\" in str");
#line 175 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (str);
#line 175 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e2);
#line 175 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e);
#line 175 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (r);
#line 175 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (d);
#line 2492 "TElementTest.c"
	}
	goto __finally111;
	__catch111_g_error:
	{
#line 175 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 175 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_inner_error_ = NULL;
#line 203 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_assert_not_reached ();
#line 2503 "TElementTest.c"
	}
	__finally111:
#line 175 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 175 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 175 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 175 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		return;
#line 2514 "TElementTest.c"
	}
}


static void ___lambda88__gtest_func (void) {
#line 174 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	__lambda88_ ();
#line 2522 "TElementTest.c"
}


static void __lambda89_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp0_ = NULL;
		GXmlNode* r = NULL;
		GXmlTDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlTDocument* _tmp3_ = NULL;
		GeeBidirList* _tmp4_ = NULL;
		GeeBidirList* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlTDocument* _tmp8_ = NULL;
		GXmlNode* e = NULL;
		GXmlTDocument* _tmp9_ = NULL;
		GXmlNode* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GeeBidirList* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GXmlNode* _tmp15_ = NULL;
		GXmlTDocument* _tmp16_ = NULL;
		GeeList* _tmp17_ = NULL;
		GeeList* _tmp18_ = NULL;
		GeeList* _tmp19_ = NULL;
		gint _tmp20_ = 0;
		gint _tmp21_ = 0;
		GXmlNode* _tmp22_ = NULL;
		GXmlNode* _tmp23_ = NULL;
		GeeList* _tmp24_ = NULL;
		GeeList* _tmp25_ = NULL;
		GeeList* _tmp26_ = NULL;
		gint _tmp27_ = 0;
		gint _tmp28_ = 0;
		GXmlTDocument* _tmp29_ = NULL;
		GeeList* _tmp30_ = NULL;
		GeeList* _tmp31_ = NULL;
		GeeList* _tmp32_ = NULL;
		gint _tmp33_ = 0;
		gint _tmp34_ = 0;
		GXmlNode* e2 = NULL;
		GXmlTDocument* _tmp35_ = NULL;
		GXmlNode* _tmp36_ = NULL;
		GXmlNode* _tmp37_ = NULL;
		GeeBidirList* _tmp38_ = NULL;
		GeeBidirList* _tmp39_ = NULL;
		GeeBidirList* _tmp40_ = NULL;
		GXmlNode* _tmp41_ = NULL;
		GXmlNode* _tmp42_ = NULL;
		GeeList* _tmp43_ = NULL;
		GeeList* _tmp44_ = NULL;
		GeeList* _tmp45_ = NULL;
		gint _tmp46_ = 0;
		gint _tmp47_ = 0;
		GXmlNode* _tmp48_ = NULL;
		GeeList* _tmp49_ = NULL;
		GeeList* _tmp50_ = NULL;
		GeeList* _tmp51_ = NULL;
		gint _tmp52_ = 0;
		gint _tmp53_ = 0;
		GXmlTDocument* _tmp54_ = NULL;
		GeeList* _tmp55_ = NULL;
		GeeList* _tmp56_ = NULL;
		GeeList* _tmp57_ = NULL;
		gint _tmp58_ = 0;
		gint _tmp59_ = 0;
		GXmlNode* e3 = NULL;
		GXmlTDocument* _tmp60_ = NULL;
		GXmlNode* _tmp61_ = NULL;
		GXmlNode* _tmp62_ = NULL;
		GeeBidirList* _tmp63_ = NULL;
		GeeBidirList* _tmp64_ = NULL;
		GeeBidirList* _tmp65_ = NULL;
		GXmlNode* _tmp66_ = NULL;
		GXmlNode* _tmp67_ = NULL;
		GXmlNode* _tmp68_ = NULL;
		GeeList* _tmp69_ = NULL;
		GeeList* _tmp70_ = NULL;
		GeeList* _tmp71_ = NULL;
		gint _tmp72_ = 0;
		gint _tmp73_ = 0;
		GXmlNode* _tmp74_ = NULL;
		GeeList* _tmp75_ = NULL;
		GeeList* _tmp76_ = NULL;
		GeeList* _tmp77_ = NULL;
		gint _tmp78_ = 0;
		gint _tmp79_ = 0;
		GXmlNode* _tmp80_ = NULL;
		GeeList* _tmp81_ = NULL;
		GeeList* _tmp82_ = NULL;
		GeeList* _tmp83_ = NULL;
		gint _tmp84_ = 0;
		gint _tmp85_ = 0;
		GXmlTDocument* _tmp86_ = NULL;
		GeeList* _tmp87_ = NULL;
		GeeList* _tmp88_ = NULL;
		GeeList* _tmp89_ = NULL;
		gint _tmp90_ = 0;
		gint _tmp91_ = 0;
		GXmlNode* e4 = NULL;
		GXmlTDocument* _tmp92_ = NULL;
		GXmlNode* _tmp93_ = NULL;
		GXmlNode* _tmp94_ = NULL;
		GeeBidirList* _tmp95_ = NULL;
		GeeBidirList* _tmp96_ = NULL;
		GeeBidirList* _tmp97_ = NULL;
		GXmlNode* _tmp98_ = NULL;
		GXmlNode* _tmp99_ = NULL;
		GeeList* _tmp100_ = NULL;
		GeeList* _tmp101_ = NULL;
		GeeList* _tmp102_ = NULL;
		gint _tmp103_ = 0;
		gint _tmp104_ = 0;
		GXmlNode* _tmp105_ = NULL;
		GeeList* _tmp106_ = NULL;
		GeeList* _tmp107_ = NULL;
		GeeList* _tmp108_ = NULL;
		gint _tmp109_ = 0;
		gint _tmp110_ = 0;
		GXmlNode* _tmp111_ = NULL;
		GeeList* _tmp112_ = NULL;
		GeeList* _tmp113_ = NULL;
		GeeList* _tmp114_ = NULL;
		gint _tmp115_ = 0;
		gint _tmp116_ = 0;
		GXmlNode* _tmp117_ = NULL;
		GeeList* _tmp118_ = NULL;
		GeeList* _tmp119_ = NULL;
		GeeList* _tmp120_ = NULL;
		gint _tmp121_ = 0;
		gint _tmp122_ = 0;
		GXmlTDocument* _tmp123_ = NULL;
		GeeList* _tmp124_ = NULL;
		GeeList* _tmp125_ = NULL;
		GeeList* _tmp126_ = NULL;
		gint _tmp127_ = 0;
		gint _tmp128_ = 0;
		gchar* str = NULL;
		GXmlTDocument* _tmp129_ = NULL;
		gchar* _tmp130_ = NULL;
		gboolean _tmp131_ = FALSE;
		gboolean _tmp132_ = FALSE;
		gboolean _tmp133_ = FALSE;
		gboolean _tmp134_ = FALSE;
		gboolean _tmp135_ = FALSE;
		gboolean _tmp136_ = FALSE;
		gboolean _tmp137_ = FALSE;
		gboolean _tmp138_ = FALSE;
#line 207 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp0_ = gxml_tdocument_new ();
#line 207 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		d = _tmp0_;
#line 208 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp1_ = d;
#line 208 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp2_ = gxml_document_create_element ((GXmlDocument*) _tmp1_, "root", &_inner_error_);
#line 208 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		r = _tmp2_;
#line 208 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 208 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 2689 "TElementTest.c"
			goto __catch112_g_error;
		}
#line 209 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp3_ = d;
#line 209 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp4_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp3_);
#line 209 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp5_ = _tmp4_;
#line 209 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp6_ = _tmp5_;
#line 209 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp7_ = r;
#line 209 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp6_, _tmp7_);
#line 209 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp6_);
#line 211 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp8_ = d;
#line 211 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_tnode_set_namespace ((GXmlTNode*) _tmp8_, "http://git.gnome.org/browse/gxml", NULL);
#line 212 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp9_ = d;
#line 212 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp10_ = gxml_document_create_element ((GXmlDocument*) _tmp9_, "child", &_inner_error_);
#line 212 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e = _tmp10_;
#line 212 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 212 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 212 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 2722 "TElementTest.c"
			goto __catch112_g_error;
		}
#line 213 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp11_ = r;
#line 213 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp12_ = gxml_node_get_children_nodes (_tmp11_);
#line 213 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp13_ = _tmp12_;
#line 213 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp14_ = _tmp13_;
#line 213 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp15_ = e;
#line 213 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp14_, _tmp15_);
#line 213 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp14_);
#line 214 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp16_ = d;
#line 214 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp17_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp16_);
#line 214 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp18_ = _tmp17_;
#line 214 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp19_ = _tmp18_;
#line 214 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp20_ = gee_collection_get_size ((GeeCollection*) _tmp19_);
#line 214 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp21_ = _tmp20_;
#line 214 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp21_ == 1, "d.namespaces.size == 1");
#line 214 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp19_);
#line 215 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp22_ = e;
#line 215 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp22_, "http://developer.gnome.org/", "dg");
#line 216 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp23_ = e;
#line 216 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp24_ = gxml_node_get_namespaces (_tmp23_);
#line 216 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp25_ = _tmp24_;
#line 216 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp26_ = _tmp25_;
#line 216 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp27_ = gee_collection_get_size ((GeeCollection*) _tmp26_);
#line 216 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp28_ = _tmp27_;
#line 216 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp28_ == 1, "e.namespaces.size == 1");
#line 216 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp26_);
#line 217 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp29_ = d;
#line 217 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp30_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp29_);
#line 217 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp31_ = _tmp30_;
#line 217 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp32_ = _tmp31_;
#line 217 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp33_ = gee_collection_get_size ((GeeCollection*) _tmp32_);
#line 217 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp34_ = _tmp33_;
#line 217 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp34_ == 2, "d.namespaces.size == 2");
#line 217 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp32_);
#line 218 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp35_ = d;
#line 218 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp36_ = gxml_document_create_element ((GXmlDocument*) _tmp35_, "children", &_inner_error_);
#line 218 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e2 = _tmp36_;
#line 218 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 218 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 218 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 218 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 2805 "TElementTest.c"
			goto __catch112_g_error;
		}
#line 219 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp37_ = e;
#line 219 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp38_ = gxml_node_get_children_nodes (_tmp37_);
#line 219 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp39_ = _tmp38_;
#line 219 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp40_ = _tmp39_;
#line 219 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp41_ = e2;
#line 219 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp40_, _tmp41_);
#line 219 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp40_);
#line 220 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp42_ = e;
#line 220 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp43_ = gxml_node_get_namespaces (_tmp42_);
#line 220 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp44_ = _tmp43_;
#line 220 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp45_ = _tmp44_;
#line 220 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp46_ = gee_collection_get_size ((GeeCollection*) _tmp45_);
#line 220 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp47_ = _tmp46_;
#line 220 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp47_ == 1, "e.namespaces.size == 1");
#line 220 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp45_);
#line 221 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp48_ = e2;
#line 221 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp49_ = gxml_node_get_namespaces (_tmp48_);
#line 221 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp50_ = _tmp49_;
#line 221 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp51_ = _tmp50_;
#line 221 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp52_ = gee_collection_get_size ((GeeCollection*) _tmp51_);
#line 221 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp53_ = _tmp52_;
#line 221 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp53_ == 0, "e2.namespaces.size == 0");
#line 221 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp51_);
#line 222 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp54_ = d;
#line 222 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp55_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp54_);
#line 222 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp56_ = _tmp55_;
#line 222 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp57_ = _tmp56_;
#line 222 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp58_ = gee_collection_get_size ((GeeCollection*) _tmp57_);
#line 222 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp59_ = _tmp58_;
#line 222 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp59_ == 2, "d.namespaces.size == 2");
#line 222 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp57_);
#line 223 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp60_ = d;
#line 223 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp61_ = gxml_document_create_element ((GXmlDocument*) _tmp60_, "nons", &_inner_error_);
#line 223 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e3 = _tmp61_;
#line 223 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 223 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e2);
#line 223 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 223 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 223 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 2886 "TElementTest.c"
			goto __catch112_g_error;
		}
#line 224 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp62_ = e;
#line 224 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp63_ = gxml_node_get_children_nodes (_tmp62_);
#line 224 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp64_ = _tmp63_;
#line 224 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp65_ = _tmp64_;
#line 224 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp66_ = e3;
#line 224 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp65_, _tmp66_);
#line 224 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp65_);
#line 225 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp67_ = e3;
#line 225 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp67_, "http://www.gnome.org/", "ns");
#line 226 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp68_ = e;
#line 226 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp69_ = gxml_node_get_namespaces (_tmp68_);
#line 226 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp70_ = _tmp69_;
#line 226 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp71_ = _tmp70_;
#line 226 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp72_ = gee_collection_get_size ((GeeCollection*) _tmp71_);
#line 226 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp73_ = _tmp72_;
#line 226 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp73_ == 1, "e.namespaces.size == 1");
#line 226 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp71_);
#line 227 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp74_ = e2;
#line 227 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp75_ = gxml_node_get_namespaces (_tmp74_);
#line 227 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp76_ = _tmp75_;
#line 227 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp77_ = _tmp76_;
#line 227 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp78_ = gee_collection_get_size ((GeeCollection*) _tmp77_);
#line 227 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp79_ = _tmp78_;
#line 227 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp79_ == 0, "e2.namespaces.size == 0");
#line 227 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp77_);
#line 228 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp80_ = e3;
#line 228 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp81_ = gxml_node_get_namespaces (_tmp80_);
#line 228 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp82_ = _tmp81_;
#line 228 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp83_ = _tmp82_;
#line 228 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp84_ = gee_collection_get_size ((GeeCollection*) _tmp83_);
#line 228 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp85_ = _tmp84_;
#line 228 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp85_ == 1, "e3.namespaces.size == 1");
#line 228 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp83_);
#line 229 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp86_ = d;
#line 229 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp87_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp86_);
#line 229 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp88_ = _tmp87_;
#line 229 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp89_ = _tmp88_;
#line 229 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp90_ = gee_collection_get_size ((GeeCollection*) _tmp89_);
#line 229 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp91_ = _tmp90_;
#line 229 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp91_ == 3, "d.namespaces.size == 3");
#line 229 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp89_);
#line 230 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp92_ = d;
#line 230 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp93_ = gxml_document_create_element ((GXmlDocument*) _tmp92_, "childrenons", &_inner_error_);
#line 230 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e4 = _tmp93_;
#line 230 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 230 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e3);
#line 230 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e2);
#line 230 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 230 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 230 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 2989 "TElementTest.c"
			goto __catch112_g_error;
		}
#line 231 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp94_ = e3;
#line 231 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp95_ = gxml_node_get_children_nodes (_tmp94_);
#line 231 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp96_ = _tmp95_;
#line 231 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp97_ = _tmp96_;
#line 231 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp98_ = e4;
#line 231 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp97_, _tmp98_);
#line 231 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp97_);
#line 232 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp99_ = e;
#line 232 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp100_ = gxml_node_get_namespaces (_tmp99_);
#line 232 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp101_ = _tmp100_;
#line 232 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp102_ = _tmp101_;
#line 232 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp103_ = gee_collection_get_size ((GeeCollection*) _tmp102_);
#line 232 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp104_ = _tmp103_;
#line 232 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp104_ == 1, "e.namespaces.size == 1");
#line 232 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp102_);
#line 233 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp105_ = e2;
#line 233 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp106_ = gxml_node_get_namespaces (_tmp105_);
#line 233 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp107_ = _tmp106_;
#line 233 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp108_ = _tmp107_;
#line 233 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp109_ = gee_collection_get_size ((GeeCollection*) _tmp108_);
#line 233 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp110_ = _tmp109_;
#line 233 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp110_ == 0, "e2.namespaces.size == 0");
#line 233 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp108_);
#line 234 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp111_ = e3;
#line 234 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp112_ = gxml_node_get_namespaces (_tmp111_);
#line 234 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp113_ = _tmp112_;
#line 234 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp114_ = _tmp113_;
#line 234 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp115_ = gee_collection_get_size ((GeeCollection*) _tmp114_);
#line 234 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp116_ = _tmp115_;
#line 234 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp116_ == 1, "e3.namespaces.size == 1");
#line 234 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp114_);
#line 235 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp117_ = e4;
#line 235 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp118_ = gxml_node_get_namespaces (_tmp117_);
#line 235 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp119_ = _tmp118_;
#line 235 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp120_ = _tmp119_;
#line 235 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp121_ = gee_collection_get_size ((GeeCollection*) _tmp120_);
#line 235 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp122_ = _tmp121_;
#line 235 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp122_ == 0, "e4.namespaces.size == 0");
#line 235 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp120_);
#line 236 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp123_ = d;
#line 236 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp124_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp123_);
#line 236 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp125_ = _tmp124_;
#line 236 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp126_ = _tmp125_;
#line 236 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp127_ = gee_collection_get_size ((GeeCollection*) _tmp126_);
#line 236 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp128_ = _tmp127_;
#line 236 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp128_ == 3, "d.namespaces.size == 3");
#line 236 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp126_);
#line 237 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp129_ = d;
#line 237 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp130_ = gxml_tnode_to_string ((GXmlTNode*) _tmp129_);
#line 237 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		str = _tmp130_;
#line 241 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp131_ = string_contains (str, "<root xmlns=\"http://git.gnome.org/browse/gxml\">");
#line 241 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp131_, "\"<root xmlns=\\\"http://git.gnome.org/browse/gxml\\\">\" in str");
#line 242 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp132_ = string_contains (str, "</root>");
#line 242 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp132_, "\"</root>\" in str");
#line 243 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp133_ = string_contains (str, "<dg:child xmlns:dg=\"http://developer.gnome.org/\">");
#line 243 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp133_, "\"<dg:child xmlns:dg=\\\"http://developer.gnome.org/\\\">\" in str");
#line 244 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp134_ = string_contains (str, "<children/>");
#line 244 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp134_, "\"<children/>\" in str");
#line 245 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp135_ = string_contains (str, "<ns:nons xmlns:ns=\"http://www.gnome.org/\">");
#line 245 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp135_, "\"<ns:nons xmlns:ns=\\\"http://www.gnome.org/\\\">\" in str");
#line 246 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp136_ = string_contains (str, "<childrenons/>");
#line 246 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp136_, "\"<childrenons/>\" in str");
#line 247 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp137_ = string_contains (str, "</ns:nons>");
#line 247 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp137_, "\"</ns:nons>\" in str");
#line 248 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp138_ = string_contains (str, "</dg:child>");
#line 248 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp138_, "\"</dg:child>\" in str");
#line 206 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (str);
#line 206 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e4);
#line 206 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e3);
#line 206 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e2);
#line 206 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e);
#line 206 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (r);
#line 206 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (d);
#line 3138 "TElementTest.c"
	}
	goto __finally112;
	__catch112_g_error:
	{
#line 206 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 206 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_inner_error_ = NULL;
#line 249 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_assert_not_reached ();
#line 3149 "TElementTest.c"
	}
	__finally112:
#line 206 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 206 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 206 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 206 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		return;
#line 3160 "TElementTest.c"
	}
}


static void ___lambda89__gtest_func (void) {
#line 205 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	__lambda89_ ();
#line 3168 "TElementTest.c"
}


static void __lambda90_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp0_ = NULL;
		GXmlNode* r = NULL;
		GXmlTDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlTDocument* _tmp3_ = NULL;
		GeeBidirList* _tmp4_ = NULL;
		GeeBidirList* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlTDocument* _tmp8_ = NULL;
		GXmlTDocument* _tmp9_ = NULL;
		GXmlNode* _tmp10_ = NULL;
		GXmlNode* e = NULL;
		GXmlTDocument* _tmp11_ = NULL;
		GXmlNode* _tmp12_ = NULL;
		GXmlNode* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GeeBidirList* _tmp15_ = NULL;
		GeeBidirList* _tmp16_ = NULL;
		GXmlNode* _tmp17_ = NULL;
		GXmlNode* _tmp18_ = NULL;
		GeeList* _tmp19_ = NULL;
		GeeList* _tmp20_ = NULL;
		GeeList* _tmp21_ = NULL;
		gint _tmp22_ = 0;
		gint _tmp23_ = 0;
		GXmlTDocument* _tmp24_ = NULL;
		GeeList* _tmp25_ = NULL;
		GeeList* _tmp26_ = NULL;
		GeeList* _tmp27_ = NULL;
		gint _tmp28_ = 0;
		gint _tmp29_ = 0;
		GXmlNode* _tmp30_ = NULL;
		GXmlNode* _tmp31_ = NULL;
		GeeList* _tmp32_ = NULL;
		GeeList* _tmp33_ = NULL;
		GeeList* _tmp34_ = NULL;
		gint _tmp35_ = 0;
		gint _tmp36_ = 0;
		GXmlNode* _tmp37_ = NULL;
		GeeList* _tmp38_ = NULL;
		GeeList* _tmp39_ = NULL;
		GeeList* _tmp40_ = NULL;
		gint _tmp41_ = 0;
		gint _tmp42_ = 0;
		GXmlTDocument* _tmp43_ = NULL;
		GeeList* _tmp44_ = NULL;
		GeeList* _tmp45_ = NULL;
		GeeList* _tmp46_ = NULL;
		gint _tmp47_ = 0;
		gint _tmp48_ = 0;
		GXmlNode* e2 = NULL;
		GXmlTDocument* _tmp49_ = NULL;
		GXmlNode* _tmp50_ = NULL;
		GXmlNode* _tmp51_ = NULL;
		GeeBidirList* _tmp52_ = NULL;
		GeeBidirList* _tmp53_ = NULL;
		GeeBidirList* _tmp54_ = NULL;
		GXmlNode* _tmp55_ = NULL;
		GXmlNode* _tmp56_ = NULL;
		GXmlNode* _tmp57_ = NULL;
		GeeList* _tmp58_ = NULL;
		GeeList* _tmp59_ = NULL;
		GeeList* _tmp60_ = NULL;
		gint _tmp61_ = 0;
		gint _tmp62_ = 0;
		GXmlNode* _tmp63_ = NULL;
		GeeList* _tmp64_ = NULL;
		GeeList* _tmp65_ = NULL;
		GeeList* _tmp66_ = NULL;
		gint _tmp67_ = 0;
		gint _tmp68_ = 0;
		GXmlNode* _tmp69_ = NULL;
		GeeList* _tmp70_ = NULL;
		GeeList* _tmp71_ = NULL;
		GeeList* _tmp72_ = NULL;
		gint _tmp73_ = 0;
		gint _tmp74_ = 0;
		GXmlTDocument* _tmp75_ = NULL;
		GeeList* _tmp76_ = NULL;
		GeeList* _tmp77_ = NULL;
		GeeList* _tmp78_ = NULL;
		gint _tmp79_ = 0;
		gint _tmp80_ = 0;
		GXmlNode* e22 = NULL;
		GXmlTDocument* _tmp81_ = NULL;
		GXmlNode* _tmp82_ = NULL;
		GXmlNode* _tmp83_ = NULL;
		GeeBidirList* _tmp84_ = NULL;
		GeeBidirList* _tmp85_ = NULL;
		GeeBidirList* _tmp86_ = NULL;
		GXmlNode* _tmp87_ = NULL;
		gchar* str = NULL;
		GXmlTDocument* _tmp88_ = NULL;
		gchar* _tmp89_ = NULL;
		gboolean _tmp90_ = FALSE;
		gboolean _tmp91_ = FALSE;
		gboolean _tmp92_ = FALSE;
		gboolean _tmp93_ = FALSE;
		gboolean _tmp94_ = FALSE;
		gboolean _tmp95_ = FALSE;
#line 253 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp0_ = gxml_tdocument_new ();
#line 253 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		d = _tmp0_;
#line 254 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp1_ = d;
#line 254 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp2_ = gxml_document_create_element ((GXmlDocument*) _tmp1_, "root", &_inner_error_);
#line 254 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		r = _tmp2_;
#line 254 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 254 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 3291 "TElementTest.c"
			goto __catch113_g_error;
		}
#line 255 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp3_ = d;
#line 255 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp4_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp3_);
#line 255 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp5_ = _tmp4_;
#line 255 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp6_ = _tmp5_;
#line 255 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp7_ = r;
#line 255 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp6_, _tmp7_);
#line 255 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp6_);
#line 256 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp8_ = d;
#line 256 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_document_set_prefix_default_ns ((GXmlDocument*) _tmp8_, TRUE);
#line 257 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp9_ = d;
#line 257 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_tnode_set_namespace ((GXmlTNode*) _tmp9_, "http://git.gnome.org/browse/gxml", "gxml");
#line 258 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp10_ = r;
#line 258 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp10_, "http://git.gnome.org/browse/gxml", "gxml");
#line 259 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp11_ = d;
#line 259 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp12_ = gxml_document_create_element ((GXmlDocument*) _tmp11_, "child", &_inner_error_);
#line 259 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e = _tmp12_;
#line 259 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 259 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 259 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 3332 "TElementTest.c"
			goto __catch113_g_error;
		}
#line 260 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp13_ = r;
#line 260 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp14_ = gxml_node_get_children_nodes (_tmp13_);
#line 260 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp15_ = _tmp14_;
#line 260 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp16_ = _tmp15_;
#line 260 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp17_ = e;
#line 260 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp16_, _tmp17_);
#line 260 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp16_);
#line 261 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp18_ = r;
#line 261 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp19_ = gxml_node_get_namespaces (_tmp18_);
#line 261 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp20_ = _tmp19_;
#line 261 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp21_ = _tmp20_;
#line 261 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp22_ = gee_collection_get_size ((GeeCollection*) _tmp21_);
#line 261 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp23_ = _tmp22_;
#line 261 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp23_ == 1, "r.namespaces.size == 1");
#line 261 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp21_);
#line 262 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp24_ = d;
#line 262 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp25_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp24_);
#line 262 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp26_ = _tmp25_;
#line 262 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp27_ = _tmp26_;
#line 262 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp28_ = gee_collection_get_size ((GeeCollection*) _tmp27_);
#line 262 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp29_ = _tmp28_;
#line 262 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp29_ == 1, "d.namespaces.size == 1");
#line 262 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp27_);
#line 263 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp30_ = e;
#line 263 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp30_, "http://developer.gnome.org/", "dg");
#line 264 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp31_ = e;
#line 264 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp32_ = gxml_node_get_namespaces (_tmp31_);
#line 264 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp33_ = _tmp32_;
#line 264 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp34_ = _tmp33_;
#line 264 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp35_ = gee_collection_get_size ((GeeCollection*) _tmp34_);
#line 264 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp36_ = _tmp35_;
#line 264 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp36_ == 1, "e.namespaces.size == 1");
#line 264 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp34_);
#line 265 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp37_ = r;
#line 265 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp38_ = gxml_node_get_namespaces (_tmp37_);
#line 265 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp39_ = _tmp38_;
#line 265 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp40_ = _tmp39_;
#line 265 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp41_ = gee_collection_get_size ((GeeCollection*) _tmp40_);
#line 265 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp42_ = _tmp41_;
#line 265 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp42_ == 1, "r.namespaces.size == 1");
#line 265 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp40_);
#line 266 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp43_ = d;
#line 266 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp44_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp43_);
#line 266 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp45_ = _tmp44_;
#line 266 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp46_ = _tmp45_;
#line 266 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp47_ = gee_collection_get_size ((GeeCollection*) _tmp46_);
#line 266 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp48_ = _tmp47_;
#line 266 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp48_ == 2, "d.namespaces.size == 2");
#line 266 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp46_);
#line 267 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp49_ = d;
#line 267 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp50_ = gxml_document_create_element ((GXmlDocument*) _tmp49_, "nons", &_inner_error_);
#line 267 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e2 = _tmp50_;
#line 267 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 267 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 267 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 267 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 3447 "TElementTest.c"
			goto __catch113_g_error;
		}
#line 268 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp51_ = e;
#line 268 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp52_ = gxml_node_get_children_nodes (_tmp51_);
#line 268 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp53_ = _tmp52_;
#line 268 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp54_ = _tmp53_;
#line 268 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp55_ = e2;
#line 268 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp54_, _tmp55_);
#line 268 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp54_);
#line 269 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp56_ = e2;
#line 269 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp56_, "http://www.gnome.org/", "ns");
#line 270 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp57_ = e;
#line 270 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp58_ = gxml_node_get_namespaces (_tmp57_);
#line 270 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp59_ = _tmp58_;
#line 270 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp60_ = _tmp59_;
#line 270 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp61_ = gee_collection_get_size ((GeeCollection*) _tmp60_);
#line 270 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp62_ = _tmp61_;
#line 270 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp62_ == 1, "e.namespaces.size == 1");
#line 270 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp60_);
#line 271 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp63_ = r;
#line 271 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp64_ = gxml_node_get_namespaces (_tmp63_);
#line 271 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp65_ = _tmp64_;
#line 271 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp66_ = _tmp65_;
#line 271 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp67_ = gee_collection_get_size ((GeeCollection*) _tmp66_);
#line 271 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp68_ = _tmp67_;
#line 271 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp68_ == 1, "r.namespaces.size == 1");
#line 271 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp66_);
#line 272 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp69_ = e2;
#line 272 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp70_ = gxml_node_get_namespaces (_tmp69_);
#line 272 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp71_ = _tmp70_;
#line 272 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp72_ = _tmp71_;
#line 272 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp73_ = gee_collection_get_size ((GeeCollection*) _tmp72_);
#line 272 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp74_ = _tmp73_;
#line 272 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp74_ == 1, "e2.namespaces.size == 1");
#line 272 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp72_);
#line 273 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp75_ = d;
#line 273 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp76_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp75_);
#line 273 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp77_ = _tmp76_;
#line 273 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp78_ = _tmp77_;
#line 273 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp79_ = gee_collection_get_size ((GeeCollection*) _tmp78_);
#line 273 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp80_ = _tmp79_;
#line 273 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp80_ == 3, "d.namespaces.size == 3");
#line 273 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp78_);
#line 274 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp81_ = d;
#line 274 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp82_ = gxml_document_create_element ((GXmlDocument*) _tmp81_, "nonsd", &_inner_error_);
#line 274 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e22 = _tmp82_;
#line 274 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 274 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e2);
#line 274 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 274 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 274 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 3548 "TElementTest.c"
			goto __catch113_g_error;
		}
#line 275 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp83_ = e2;
#line 275 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp84_ = gxml_node_get_children_nodes (_tmp83_);
#line 275 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp85_ = _tmp84_;
#line 275 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp86_ = _tmp85_;
#line 275 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp87_ = e22;
#line 275 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp86_, _tmp87_);
#line 275 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp86_);
#line 276 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp88_ = d;
#line 276 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp89_ = gxml_tnode_to_string ((GXmlTNode*) _tmp88_);
#line 276 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		str = _tmp89_;
#line 280 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp90_ = string_contains (str, "<gxml:root xmlns:gxml=\"http://git.gnome.org/browse/gxml\">");
#line 280 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp90_, "\"<gxml:root xmlns:gxml=\\\"http://git.gnome.org/browse/gxml\\\">\" in str");
#line 281 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp91_ = string_contains (str, "</gxml:root>");
#line 281 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp91_, "\"</gxml:root>\" in str");
#line 282 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp92_ = string_contains (str, "<dg:child xmlns:dg=\"http://developer.gnome.org/\">");
#line 282 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp92_, "\"<dg:child xmlns:dg=\\\"http://developer.gnome.org/\\\">\" in str");
#line 283 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp93_ = string_contains (str, "<ns:nons xmlns:ns=\"http://www.gnome.org/\">");
#line 283 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp93_, "\"<ns:nons xmlns:ns=\\\"http://www.gnome.org/\\\">\" in str");
#line 284 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp94_ = string_contains (str, "</ns:nons>");
#line 284 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp94_, "\"</ns:nons>\" in str");
#line 285 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp95_ = string_contains (str, "<gxml:nonsd/>");
#line 285 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp95_, "\"<gxml:nonsd/>\" in str");
#line 252 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (str);
#line 252 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e22);
#line 252 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e2);
#line 252 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e);
#line 252 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (r);
#line 252 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (d);
#line 3607 "TElementTest.c"
	}
	goto __finally113;
	__catch113_g_error:
	{
#line 252 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 252 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_inner_error_ = NULL;
#line 286 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_assert_not_reached ();
#line 3618 "TElementTest.c"
	}
	__finally113:
#line 252 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 252 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 252 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 252 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		return;
#line 3629 "TElementTest.c"
	}
}


static void ___lambda90__gtest_func (void) {
#line 251 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	__lambda90_ ();
#line 3637 "TElementTest.c"
}


static void __lambda91_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp0_ = NULL;
		GXmlNode* r = NULL;
		GXmlTDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlTDocument* _tmp3_ = NULL;
		GeeBidirList* _tmp4_ = NULL;
		GeeBidirList* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlTDocument* _tmp8_ = NULL;
		GXmlNode* e = NULL;
		GXmlTDocument* _tmp9_ = NULL;
		GXmlNode* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GeeBidirList* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GXmlNode* _tmp15_ = NULL;
		GXmlTDocument* _tmp16_ = NULL;
		GeeList* _tmp17_ = NULL;
		GeeList* _tmp18_ = NULL;
		GeeList* _tmp19_ = NULL;
		gint _tmp20_ = 0;
		gint _tmp21_ = 0;
		GXmlNode* _tmp22_ = NULL;
		GXmlNode* _tmp23_ = NULL;
		GeeList* _tmp24_ = NULL;
		GeeList* _tmp25_ = NULL;
		GeeList* _tmp26_ = NULL;
		gint _tmp27_ = 0;
		gint _tmp28_ = 0;
		GXmlTDocument* _tmp29_ = NULL;
		GeeList* _tmp30_ = NULL;
		GeeList* _tmp31_ = NULL;
		GeeList* _tmp32_ = NULL;
		gint _tmp33_ = 0;
		gint _tmp34_ = 0;
		GXmlNode* e2 = NULL;
		GXmlTDocument* _tmp35_ = NULL;
		GXmlNode* _tmp36_ = NULL;
		GXmlNode* _tmp37_ = NULL;
		GeeBidirList* _tmp38_ = NULL;
		GeeBidirList* _tmp39_ = NULL;
		GeeBidirList* _tmp40_ = NULL;
		GXmlNode* _tmp41_ = NULL;
		GXmlNode* _tmp42_ = NULL;
		GeeList* _tmp43_ = NULL;
		GeeList* _tmp44_ = NULL;
		GeeList* _tmp45_ = NULL;
		gint _tmp46_ = 0;
		gint _tmp47_ = 0;
		GXmlNode* _tmp48_ = NULL;
		GeeList* _tmp49_ = NULL;
		GeeList* _tmp50_ = NULL;
		GeeList* _tmp51_ = NULL;
		gint _tmp52_ = 0;
		gint _tmp53_ = 0;
		GXmlTDocument* _tmp54_ = NULL;
		GeeList* _tmp55_ = NULL;
		GeeList* _tmp56_ = NULL;
		GeeList* _tmp57_ = NULL;
		gint _tmp58_ = 0;
		gint _tmp59_ = 0;
		GXmlNode* e3 = NULL;
		GXmlTDocument* _tmp60_ = NULL;
		GXmlNode* _tmp61_ = NULL;
		GXmlNode* _tmp62_ = NULL;
		GeeBidirList* _tmp63_ = NULL;
		GeeBidirList* _tmp64_ = NULL;
		GeeBidirList* _tmp65_ = NULL;
		GXmlNode* _tmp66_ = NULL;
		GXmlNode* _tmp67_ = NULL;
		GXmlNode* _tmp68_ = NULL;
		GeeList* _tmp69_ = NULL;
		GeeList* _tmp70_ = NULL;
		GeeList* _tmp71_ = NULL;
		gint _tmp72_ = 0;
		gint _tmp73_ = 0;
		GXmlNode* _tmp74_ = NULL;
		GeeList* _tmp75_ = NULL;
		GeeList* _tmp76_ = NULL;
		GeeList* _tmp77_ = NULL;
		gint _tmp78_ = 0;
		gint _tmp79_ = 0;
		GXmlNode* _tmp80_ = NULL;
		GeeList* _tmp81_ = NULL;
		GeeList* _tmp82_ = NULL;
		GeeList* _tmp83_ = NULL;
		gint _tmp84_ = 0;
		gint _tmp85_ = 0;
		GXmlTDocument* _tmp86_ = NULL;
		GeeList* _tmp87_ = NULL;
		GeeList* _tmp88_ = NULL;
		GeeList* _tmp89_ = NULL;
		gint _tmp90_ = 0;
		gint _tmp91_ = 0;
		GXmlNode* e4 = NULL;
		GXmlTDocument* _tmp92_ = NULL;
		GXmlNode* _tmp93_ = NULL;
		GXmlNode* _tmp94_ = NULL;
		GeeBidirList* _tmp95_ = NULL;
		GeeBidirList* _tmp96_ = NULL;
		GeeBidirList* _tmp97_ = NULL;
		GXmlNode* _tmp98_ = NULL;
		GXmlNode* _tmp99_ = NULL;
		GeeList* _tmp100_ = NULL;
		GeeList* _tmp101_ = NULL;
		GeeList* _tmp102_ = NULL;
		gint _tmp103_ = 0;
		gint _tmp104_ = 0;
		GXmlNode* _tmp105_ = NULL;
		GeeList* _tmp106_ = NULL;
		GeeList* _tmp107_ = NULL;
		GeeList* _tmp108_ = NULL;
		gint _tmp109_ = 0;
		gint _tmp110_ = 0;
		GXmlNode* _tmp111_ = NULL;
		GeeList* _tmp112_ = NULL;
		GeeList* _tmp113_ = NULL;
		GeeList* _tmp114_ = NULL;
		gint _tmp115_ = 0;
		gint _tmp116_ = 0;
		GXmlNode* _tmp117_ = NULL;
		GeeList* _tmp118_ = NULL;
		GeeList* _tmp119_ = NULL;
		GeeList* _tmp120_ = NULL;
		gint _tmp121_ = 0;
		gint _tmp122_ = 0;
		GXmlTDocument* _tmp123_ = NULL;
		GeeList* _tmp124_ = NULL;
		GeeList* _tmp125_ = NULL;
		GeeList* _tmp126_ = NULL;
		gint _tmp127_ = 0;
		gint _tmp128_ = 0;
		gchar* str = NULL;
		GXmlTDocument* _tmp129_ = NULL;
		gchar* _tmp130_ = NULL;
		gboolean _tmp131_ = FALSE;
		gboolean _tmp132_ = FALSE;
		gboolean _tmp133_ = FALSE;
		gboolean _tmp134_ = FALSE;
		gboolean _tmp135_ = FALSE;
		gboolean _tmp136_ = FALSE;
		gboolean _tmp137_ = FALSE;
		gboolean _tmp138_ = FALSE;
#line 290 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp0_ = gxml_tdocument_new ();
#line 290 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		d = _tmp0_;
#line 291 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp1_ = d;
#line 291 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp2_ = gxml_document_create_element ((GXmlDocument*) _tmp1_, "root", &_inner_error_);
#line 291 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		r = _tmp2_;
#line 291 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 291 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 3804 "TElementTest.c"
			goto __catch114_g_error;
		}
#line 292 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp3_ = d;
#line 292 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp4_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp3_);
#line 292 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp5_ = _tmp4_;
#line 292 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp6_ = _tmp5_;
#line 292 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp7_ = r;
#line 292 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp6_, _tmp7_);
#line 292 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp6_);
#line 294 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp8_ = d;
#line 294 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_tnode_set_namespace ((GXmlTNode*) _tmp8_, "http://git.gnome.org/browse/gxml", NULL);
#line 295 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp9_ = d;
#line 295 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp10_ = gxml_document_create_element ((GXmlDocument*) _tmp9_, "child", &_inner_error_);
#line 295 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e = _tmp10_;
#line 295 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 295 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 295 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 3837 "TElementTest.c"
			goto __catch114_g_error;
		}
#line 296 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp11_ = r;
#line 296 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp12_ = gxml_node_get_children_nodes (_tmp11_);
#line 296 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp13_ = _tmp12_;
#line 296 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp14_ = _tmp13_;
#line 296 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp15_ = e;
#line 296 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp14_, _tmp15_);
#line 296 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp14_);
#line 297 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp16_ = d;
#line 297 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp17_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp16_);
#line 297 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp18_ = _tmp17_;
#line 297 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp19_ = _tmp18_;
#line 297 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp20_ = gee_collection_get_size ((GeeCollection*) _tmp19_);
#line 297 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp21_ = _tmp20_;
#line 297 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp21_ == 1, "d.namespaces.size == 1");
#line 297 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp19_);
#line 298 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp22_ = e;
#line 298 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp22_, "http://developer.gnome.org/", "dg");
#line 299 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp23_ = e;
#line 299 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp24_ = gxml_node_get_namespaces (_tmp23_);
#line 299 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp25_ = _tmp24_;
#line 299 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp26_ = _tmp25_;
#line 299 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp27_ = gee_collection_get_size ((GeeCollection*) _tmp26_);
#line 299 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp28_ = _tmp27_;
#line 299 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp28_ == 1, "e.namespaces.size == 1");
#line 299 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp26_);
#line 300 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp29_ = d;
#line 300 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp30_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp29_);
#line 300 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp31_ = _tmp30_;
#line 300 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp32_ = _tmp31_;
#line 300 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp33_ = gee_collection_get_size ((GeeCollection*) _tmp32_);
#line 300 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp34_ = _tmp33_;
#line 300 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp34_ == 2, "d.namespaces.size == 2");
#line 300 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp32_);
#line 301 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp35_ = d;
#line 301 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp36_ = gxml_document_create_element ((GXmlDocument*) _tmp35_, "children", &_inner_error_);
#line 301 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e2 = _tmp36_;
#line 301 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 301 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 301 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 301 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 3920 "TElementTest.c"
			goto __catch114_g_error;
		}
#line 302 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp37_ = e;
#line 302 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp38_ = gxml_node_get_children_nodes (_tmp37_);
#line 302 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp39_ = _tmp38_;
#line 302 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp40_ = _tmp39_;
#line 302 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp41_ = e2;
#line 302 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp40_, _tmp41_);
#line 302 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp40_);
#line 303 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp42_ = e;
#line 303 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp43_ = gxml_node_get_namespaces (_tmp42_);
#line 303 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp44_ = _tmp43_;
#line 303 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp45_ = _tmp44_;
#line 303 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp46_ = gee_collection_get_size ((GeeCollection*) _tmp45_);
#line 303 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp47_ = _tmp46_;
#line 303 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp47_ == 1, "e.namespaces.size == 1");
#line 303 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp45_);
#line 304 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp48_ = e2;
#line 304 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp49_ = gxml_node_get_namespaces (_tmp48_);
#line 304 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp50_ = _tmp49_;
#line 304 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp51_ = _tmp50_;
#line 304 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp52_ = gee_collection_get_size ((GeeCollection*) _tmp51_);
#line 304 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp53_ = _tmp52_;
#line 304 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp53_ == 0, "e2.namespaces.size == 0");
#line 304 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp51_);
#line 305 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp54_ = d;
#line 305 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp55_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp54_);
#line 305 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp56_ = _tmp55_;
#line 305 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp57_ = _tmp56_;
#line 305 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp58_ = gee_collection_get_size ((GeeCollection*) _tmp57_);
#line 305 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp59_ = _tmp58_;
#line 305 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp59_ == 2, "d.namespaces.size == 2");
#line 305 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp57_);
#line 306 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp60_ = d;
#line 306 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp61_ = gxml_document_create_element ((GXmlDocument*) _tmp60_, "nons", &_inner_error_);
#line 306 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e3 = _tmp61_;
#line 306 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 306 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e2);
#line 306 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 306 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 306 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 4001 "TElementTest.c"
			goto __catch114_g_error;
		}
#line 307 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp62_ = e;
#line 307 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp63_ = gxml_node_get_children_nodes (_tmp62_);
#line 307 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp64_ = _tmp63_;
#line 307 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp65_ = _tmp64_;
#line 307 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp66_ = e3;
#line 307 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp65_, _tmp66_);
#line 307 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp65_);
#line 308 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp67_ = e3;
#line 308 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp67_, "http://www.gnome.org/", "ns");
#line 309 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp68_ = e;
#line 309 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp69_ = gxml_node_get_namespaces (_tmp68_);
#line 309 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp70_ = _tmp69_;
#line 309 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp71_ = _tmp70_;
#line 309 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp72_ = gee_collection_get_size ((GeeCollection*) _tmp71_);
#line 309 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp73_ = _tmp72_;
#line 309 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp73_ == 1, "e.namespaces.size == 1");
#line 309 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp71_);
#line 310 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp74_ = e2;
#line 310 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp75_ = gxml_node_get_namespaces (_tmp74_);
#line 310 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp76_ = _tmp75_;
#line 310 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp77_ = _tmp76_;
#line 310 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp78_ = gee_collection_get_size ((GeeCollection*) _tmp77_);
#line 310 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp79_ = _tmp78_;
#line 310 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp79_ == 0, "e2.namespaces.size == 0");
#line 310 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp77_);
#line 311 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp80_ = e3;
#line 311 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp81_ = gxml_node_get_namespaces (_tmp80_);
#line 311 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp82_ = _tmp81_;
#line 311 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp83_ = _tmp82_;
#line 311 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp84_ = gee_collection_get_size ((GeeCollection*) _tmp83_);
#line 311 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp85_ = _tmp84_;
#line 311 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp85_ == 1, "e3.namespaces.size == 1");
#line 311 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp83_);
#line 312 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp86_ = d;
#line 312 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp87_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp86_);
#line 312 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp88_ = _tmp87_;
#line 312 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp89_ = _tmp88_;
#line 312 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp90_ = gee_collection_get_size ((GeeCollection*) _tmp89_);
#line 312 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp91_ = _tmp90_;
#line 312 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp91_ == 3, "d.namespaces.size == 3");
#line 312 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp89_);
#line 313 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp92_ = d;
#line 313 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp93_ = gxml_document_create_element ((GXmlDocument*) _tmp92_, "childrenons", &_inner_error_);
#line 313 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e4 = _tmp93_;
#line 313 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 313 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e3);
#line 313 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e2);
#line 313 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 313 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 313 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 4104 "TElementTest.c"
			goto __catch114_g_error;
		}
#line 314 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp94_ = e3;
#line 314 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp95_ = gxml_node_get_children_nodes (_tmp94_);
#line 314 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp96_ = _tmp95_;
#line 314 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp97_ = _tmp96_;
#line 314 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp98_ = e4;
#line 314 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp97_, _tmp98_);
#line 314 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp97_);
#line 315 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp99_ = e;
#line 315 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp100_ = gxml_node_get_namespaces (_tmp99_);
#line 315 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp101_ = _tmp100_;
#line 315 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp102_ = _tmp101_;
#line 315 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp103_ = gee_collection_get_size ((GeeCollection*) _tmp102_);
#line 315 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp104_ = _tmp103_;
#line 315 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp104_ == 1, "e.namespaces.size == 1");
#line 315 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp102_);
#line 316 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp105_ = e2;
#line 316 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp106_ = gxml_node_get_namespaces (_tmp105_);
#line 316 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp107_ = _tmp106_;
#line 316 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp108_ = _tmp107_;
#line 316 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp109_ = gee_collection_get_size ((GeeCollection*) _tmp108_);
#line 316 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp110_ = _tmp109_;
#line 316 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp110_ == 0, "e2.namespaces.size == 0");
#line 316 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp108_);
#line 317 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp111_ = e3;
#line 317 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp112_ = gxml_node_get_namespaces (_tmp111_);
#line 317 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp113_ = _tmp112_;
#line 317 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp114_ = _tmp113_;
#line 317 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp115_ = gee_collection_get_size ((GeeCollection*) _tmp114_);
#line 317 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp116_ = _tmp115_;
#line 317 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp116_ == 1, "e3.namespaces.size == 1");
#line 317 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp114_);
#line 318 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp117_ = e4;
#line 318 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp118_ = gxml_node_get_namespaces (_tmp117_);
#line 318 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp119_ = _tmp118_;
#line 318 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp120_ = _tmp119_;
#line 318 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp121_ = gee_collection_get_size ((GeeCollection*) _tmp120_);
#line 318 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp122_ = _tmp121_;
#line 318 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp122_ == 0, "e4.namespaces.size == 0");
#line 318 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp120_);
#line 319 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp123_ = d;
#line 319 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp124_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp123_);
#line 319 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp125_ = _tmp124_;
#line 319 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp126_ = _tmp125_;
#line 319 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp127_ = gee_collection_get_size ((GeeCollection*) _tmp126_);
#line 319 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp128_ = _tmp127_;
#line 319 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp128_ == 3, "d.namespaces.size == 3");
#line 319 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp126_);
#line 320 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp129_ = d;
#line 320 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp130_ = gxml_tnode_to_string ((GXmlTNode*) _tmp129_);
#line 320 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		str = _tmp130_;
#line 324 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp131_ = string_contains (str, "<root xmlns=\"http://git.gnome.org/browse/gxml\">");
#line 324 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp131_, "\"<root xmlns=\\\"http://git.gnome.org/browse/gxml\\\">\" in str");
#line 325 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp132_ = string_contains (str, "</root>");
#line 325 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp132_, "\"</root>\" in str");
#line 326 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp133_ = string_contains (str, "<dg:child xmlns:dg=\"http://developer.gnome.org/\">");
#line 326 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp133_, "\"<dg:child xmlns:dg=\\\"http://developer.gnome.org/\\\">\" in str");
#line 327 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp134_ = string_contains (str, "<children/>");
#line 327 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp134_, "\"<children/>\" in str");
#line 328 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp135_ = string_contains (str, "<ns:nons xmlns:ns=\"http://www.gnome.org/\">");
#line 328 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp135_, "\"<ns:nons xmlns:ns=\\\"http://www.gnome.org/\\\">\" in str");
#line 329 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp136_ = string_contains (str, "<childrenons/>");
#line 329 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp136_, "\"<childrenons/>\" in str");
#line 330 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp137_ = string_contains (str, "</ns:nons>");
#line 330 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp137_, "\"</ns:nons>\" in str");
#line 331 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp138_ = string_contains (str, "</dg:child>");
#line 331 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp138_, "\"</dg:child>\" in str");
#line 289 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (str);
#line 289 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e4);
#line 289 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e3);
#line 289 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e2);
#line 289 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e);
#line 289 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (r);
#line 289 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (d);
#line 4253 "TElementTest.c"
	}
	goto __finally114;
	__catch114_g_error:
	{
#line 289 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 289 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_inner_error_ = NULL;
#line 332 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_assert_not_reached ();
#line 4264 "TElementTest.c"
	}
	__finally114:
#line 289 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 289 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 289 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 289 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		return;
#line 4275 "TElementTest.c"
	}
}


static void ___lambda91__gtest_func (void) {
#line 288 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	__lambda91_ ();
#line 4283 "TElementTest.c"
}


static void __lambda92_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp0_ = NULL;
		GXmlNode* r = NULL;
		GXmlTDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlTDocument* _tmp3_ = NULL;
		GeeBidirList* _tmp4_ = NULL;
		GeeBidirList* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlTDocument* _tmp8_ = NULL;
		GXmlTDocument* _tmp9_ = NULL;
		GXmlNode* e = NULL;
		GXmlTDocument* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GXmlNode* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GeeBidirList* _tmp15_ = NULL;
		GXmlNode* _tmp16_ = NULL;
		GXmlTDocument* _tmp17_ = NULL;
		GeeList* _tmp18_ = NULL;
		GeeList* _tmp19_ = NULL;
		GeeList* _tmp20_ = NULL;
		gint _tmp21_ = 0;
		gint _tmp22_ = 0;
		GXmlNode* _tmp23_ = NULL;
		GXmlNode* _tmp24_ = NULL;
		GeeList* _tmp25_ = NULL;
		GeeList* _tmp26_ = NULL;
		GeeList* _tmp27_ = NULL;
		gint _tmp28_ = 0;
		gint _tmp29_ = 0;
		GXmlTDocument* _tmp30_ = NULL;
		GeeList* _tmp31_ = NULL;
		GeeList* _tmp32_ = NULL;
		GeeList* _tmp33_ = NULL;
		gint _tmp34_ = 0;
		gint _tmp35_ = 0;
		GXmlNode* e2 = NULL;
		GXmlTDocument* _tmp36_ = NULL;
		GXmlNode* _tmp37_ = NULL;
		GXmlNode* _tmp38_ = NULL;
		GeeBidirList* _tmp39_ = NULL;
		GeeBidirList* _tmp40_ = NULL;
		GeeBidirList* _tmp41_ = NULL;
		GXmlNode* _tmp42_ = NULL;
		GXmlNode* _tmp43_ = NULL;
		GeeList* _tmp44_ = NULL;
		GeeList* _tmp45_ = NULL;
		GeeList* _tmp46_ = NULL;
		gint _tmp47_ = 0;
		gint _tmp48_ = 0;
		GXmlNode* _tmp49_ = NULL;
		GeeList* _tmp50_ = NULL;
		GeeList* _tmp51_ = NULL;
		GeeList* _tmp52_ = NULL;
		gint _tmp53_ = 0;
		gint _tmp54_ = 0;
		GXmlTDocument* _tmp55_ = NULL;
		GeeList* _tmp56_ = NULL;
		GeeList* _tmp57_ = NULL;
		GeeList* _tmp58_ = NULL;
		gint _tmp59_ = 0;
		gint _tmp60_ = 0;
		GXmlNode* e3 = NULL;
		GXmlTDocument* _tmp61_ = NULL;
		GXmlNode* _tmp62_ = NULL;
		GXmlNode* _tmp63_ = NULL;
		GeeBidirList* _tmp64_ = NULL;
		GeeBidirList* _tmp65_ = NULL;
		GeeBidirList* _tmp66_ = NULL;
		GXmlNode* _tmp67_ = NULL;
		GXmlNode* _tmp68_ = NULL;
		GXmlNode* _tmp69_ = NULL;
		GeeList* _tmp70_ = NULL;
		GeeList* _tmp71_ = NULL;
		GeeList* _tmp72_ = NULL;
		gint _tmp73_ = 0;
		gint _tmp74_ = 0;
		GXmlNode* _tmp75_ = NULL;
		GeeList* _tmp76_ = NULL;
		GeeList* _tmp77_ = NULL;
		GeeList* _tmp78_ = NULL;
		gint _tmp79_ = 0;
		gint _tmp80_ = 0;
		GXmlNode* _tmp81_ = NULL;
		GeeList* _tmp82_ = NULL;
		GeeList* _tmp83_ = NULL;
		GeeList* _tmp84_ = NULL;
		gint _tmp85_ = 0;
		gint _tmp86_ = 0;
		GXmlTDocument* _tmp87_ = NULL;
		GeeList* _tmp88_ = NULL;
		GeeList* _tmp89_ = NULL;
		GeeList* _tmp90_ = NULL;
		gint _tmp91_ = 0;
		gint _tmp92_ = 0;
		GXmlNode* e4 = NULL;
		GXmlTDocument* _tmp93_ = NULL;
		GXmlNode* _tmp94_ = NULL;
		GXmlNode* _tmp95_ = NULL;
		GeeBidirList* _tmp96_ = NULL;
		GeeBidirList* _tmp97_ = NULL;
		GeeBidirList* _tmp98_ = NULL;
		GXmlNode* _tmp99_ = NULL;
		GXmlNode* _tmp100_ = NULL;
		GeeList* _tmp101_ = NULL;
		GeeList* _tmp102_ = NULL;
		GeeList* _tmp103_ = NULL;
		gint _tmp104_ = 0;
		gint _tmp105_ = 0;
		GXmlNode* _tmp106_ = NULL;
		GeeList* _tmp107_ = NULL;
		GeeList* _tmp108_ = NULL;
		GeeList* _tmp109_ = NULL;
		gint _tmp110_ = 0;
		gint _tmp111_ = 0;
		GXmlNode* _tmp112_ = NULL;
		GeeList* _tmp113_ = NULL;
		GeeList* _tmp114_ = NULL;
		GeeList* _tmp115_ = NULL;
		gint _tmp116_ = 0;
		gint _tmp117_ = 0;
		GXmlNode* _tmp118_ = NULL;
		GeeList* _tmp119_ = NULL;
		GeeList* _tmp120_ = NULL;
		GeeList* _tmp121_ = NULL;
		gint _tmp122_ = 0;
		gint _tmp123_ = 0;
		GXmlTDocument* _tmp124_ = NULL;
		GeeList* _tmp125_ = NULL;
		GeeList* _tmp126_ = NULL;
		GeeList* _tmp127_ = NULL;
		gint _tmp128_ = 0;
		gint _tmp129_ = 0;
		GXmlNode* c2 = NULL;
		GXmlTDocument* _tmp130_ = NULL;
		GXmlNode* _tmp131_ = NULL;
		GXmlTDocument* _tmp132_ = NULL;
		GXmlNode* _tmp133_ = NULL;
		GXmlNode* _tmp134_ = NULL;
		GXmlNode* _tmp135_ = NULL;
		GeeBidirList* _tmp136_ = NULL;
		GeeBidirList* _tmp137_ = NULL;
		GeeBidirList* _tmp138_ = NULL;
		GXmlNode* _tmp139_ = NULL;
		GXmlNode* _tmp140_ = NULL;
		gchar* str = NULL;
		GXmlTDocument* _tmp141_ = NULL;
		gchar* _tmp142_ = NULL;
		gboolean _tmp143_ = FALSE;
		gboolean _tmp144_ = FALSE;
		gboolean _tmp145_ = FALSE;
		gboolean _tmp146_ = FALSE;
		gboolean _tmp147_ = FALSE;
		gboolean _tmp148_ = FALSE;
		gboolean _tmp149_ = FALSE;
		gboolean _tmp150_ = FALSE;
		gboolean _tmp151_ = FALSE;
#line 336 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp0_ = gxml_tdocument_new ();
#line 336 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		d = _tmp0_;
#line 337 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp1_ = d;
#line 337 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp2_ = gxml_document_create_element ((GXmlDocument*) _tmp1_, "root", &_inner_error_);
#line 337 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		r = _tmp2_;
#line 337 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 337 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 4464 "TElementTest.c"
			goto __catch115_g_error;
		}
#line 338 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp3_ = d;
#line 338 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp4_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp3_);
#line 338 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp5_ = _tmp4_;
#line 338 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp6_ = _tmp5_;
#line 338 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp7_ = r;
#line 338 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp6_, _tmp7_);
#line 338 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp6_);
#line 340 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp8_ = d;
#line 340 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_tnode_set_namespace ((GXmlTNode*) _tmp8_, "http://git.gnome.org/browse/gxml", NULL);
#line 342 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp9_ = d;
#line 342 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_document_set_ns_top ((GXmlDocument*) _tmp9_, TRUE);
#line 343 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp10_ = d;
#line 343 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp11_ = gxml_document_create_element ((GXmlDocument*) _tmp10_, "child", &_inner_error_);
#line 343 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e = _tmp11_;
#line 343 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 343 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 343 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 4501 "TElementTest.c"
			goto __catch115_g_error;
		}
#line 344 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp12_ = r;
#line 344 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp13_ = gxml_node_get_children_nodes (_tmp12_);
#line 344 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp14_ = _tmp13_;
#line 344 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp15_ = _tmp14_;
#line 344 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp16_ = e;
#line 344 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp15_, _tmp16_);
#line 344 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp15_);
#line 345 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp17_ = d;
#line 345 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp18_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp17_);
#line 345 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp19_ = _tmp18_;
#line 345 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp20_ = _tmp19_;
#line 345 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp21_ = gee_collection_get_size ((GeeCollection*) _tmp20_);
#line 345 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp22_ = _tmp21_;
#line 345 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp22_ == 1, "d.namespaces.size == 1");
#line 345 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp20_);
#line 346 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp23_ = e;
#line 346 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp23_, "http://developer.gnome.org/", "dg");
#line 347 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp24_ = e;
#line 347 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp25_ = gxml_node_get_namespaces (_tmp24_);
#line 347 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp26_ = _tmp25_;
#line 347 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp27_ = _tmp26_;
#line 347 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp28_ = gee_collection_get_size ((GeeCollection*) _tmp27_);
#line 347 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp29_ = _tmp28_;
#line 347 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp29_ == 1, "e.namespaces.size == 1");
#line 347 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp27_);
#line 348 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp30_ = d;
#line 348 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp31_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp30_);
#line 348 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp32_ = _tmp31_;
#line 348 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp33_ = _tmp32_;
#line 348 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp34_ = gee_collection_get_size ((GeeCollection*) _tmp33_);
#line 348 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp35_ = _tmp34_;
#line 348 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp35_ == 2, "d.namespaces.size == 2");
#line 348 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp33_);
#line 349 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp36_ = d;
#line 349 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp37_ = gxml_document_create_element ((GXmlDocument*) _tmp36_, "children", &_inner_error_);
#line 349 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e2 = _tmp37_;
#line 349 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 349 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 349 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 349 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 4584 "TElementTest.c"
			goto __catch115_g_error;
		}
#line 350 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp38_ = e;
#line 350 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp39_ = gxml_node_get_children_nodes (_tmp38_);
#line 350 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp40_ = _tmp39_;
#line 350 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp41_ = _tmp40_;
#line 350 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp42_ = e2;
#line 350 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp41_, _tmp42_);
#line 350 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp41_);
#line 351 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp43_ = e;
#line 351 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp44_ = gxml_node_get_namespaces (_tmp43_);
#line 351 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp45_ = _tmp44_;
#line 351 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp46_ = _tmp45_;
#line 351 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp47_ = gee_collection_get_size ((GeeCollection*) _tmp46_);
#line 351 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp48_ = _tmp47_;
#line 351 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp48_ == 1, "e.namespaces.size == 1");
#line 351 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp46_);
#line 352 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp49_ = e2;
#line 352 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp50_ = gxml_node_get_namespaces (_tmp49_);
#line 352 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp51_ = _tmp50_;
#line 352 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp52_ = _tmp51_;
#line 352 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp53_ = gee_collection_get_size ((GeeCollection*) _tmp52_);
#line 352 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp54_ = _tmp53_;
#line 352 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp54_ == 0, "e2.namespaces.size == 0");
#line 352 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp52_);
#line 353 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp55_ = d;
#line 353 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp56_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp55_);
#line 353 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp57_ = _tmp56_;
#line 353 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp58_ = _tmp57_;
#line 353 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp59_ = gee_collection_get_size ((GeeCollection*) _tmp58_);
#line 353 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp60_ = _tmp59_;
#line 353 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp60_ == 2, "d.namespaces.size == 2");
#line 353 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp58_);
#line 354 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp61_ = d;
#line 354 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp62_ = gxml_document_create_element ((GXmlDocument*) _tmp61_, "nons", &_inner_error_);
#line 354 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e3 = _tmp62_;
#line 354 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 354 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e2);
#line 354 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 354 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 354 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 4665 "TElementTest.c"
			goto __catch115_g_error;
		}
#line 355 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp63_ = e;
#line 355 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp64_ = gxml_node_get_children_nodes (_tmp63_);
#line 355 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp65_ = _tmp64_;
#line 355 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp66_ = _tmp65_;
#line 355 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp67_ = e3;
#line 355 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp66_, _tmp67_);
#line 355 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp66_);
#line 356 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp68_ = e3;
#line 356 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp68_, "http://www.gnome.org/", "ns");
#line 357 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp69_ = e;
#line 357 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp70_ = gxml_node_get_namespaces (_tmp69_);
#line 357 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp71_ = _tmp70_;
#line 357 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp72_ = _tmp71_;
#line 357 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp73_ = gee_collection_get_size ((GeeCollection*) _tmp72_);
#line 357 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp74_ = _tmp73_;
#line 357 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp74_ == 1, "e.namespaces.size == 1");
#line 357 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp72_);
#line 358 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp75_ = e2;
#line 358 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp76_ = gxml_node_get_namespaces (_tmp75_);
#line 358 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp77_ = _tmp76_;
#line 358 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp78_ = _tmp77_;
#line 358 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp79_ = gee_collection_get_size ((GeeCollection*) _tmp78_);
#line 358 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp80_ = _tmp79_;
#line 358 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp80_ == 0, "e2.namespaces.size == 0");
#line 358 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp78_);
#line 359 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp81_ = e3;
#line 359 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp82_ = gxml_node_get_namespaces (_tmp81_);
#line 359 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp83_ = _tmp82_;
#line 359 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp84_ = _tmp83_;
#line 359 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp85_ = gee_collection_get_size ((GeeCollection*) _tmp84_);
#line 359 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp86_ = _tmp85_;
#line 359 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp86_ == 1, "e3.namespaces.size == 1");
#line 359 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp84_);
#line 360 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp87_ = d;
#line 360 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp88_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp87_);
#line 360 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp89_ = _tmp88_;
#line 360 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp90_ = _tmp89_;
#line 360 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp91_ = gee_collection_get_size ((GeeCollection*) _tmp90_);
#line 360 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp92_ = _tmp91_;
#line 360 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp92_ == 3, "d.namespaces.size == 3");
#line 360 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp90_);
#line 361 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp93_ = d;
#line 361 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp94_ = gxml_document_create_element ((GXmlDocument*) _tmp93_, "childrenons", &_inner_error_);
#line 361 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e4 = _tmp94_;
#line 361 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 361 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e3);
#line 361 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e2);
#line 361 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 361 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 361 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 4768 "TElementTest.c"
			goto __catch115_g_error;
		}
#line 362 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp95_ = e3;
#line 362 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp96_ = gxml_node_get_children_nodes (_tmp95_);
#line 362 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp97_ = _tmp96_;
#line 362 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp98_ = _tmp97_;
#line 362 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp99_ = e4;
#line 362 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp98_, _tmp99_);
#line 362 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp98_);
#line 363 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp100_ = e;
#line 363 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp101_ = gxml_node_get_namespaces (_tmp100_);
#line 363 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp102_ = _tmp101_;
#line 363 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp103_ = _tmp102_;
#line 363 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp104_ = gee_collection_get_size ((GeeCollection*) _tmp103_);
#line 363 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp105_ = _tmp104_;
#line 363 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp105_ == 1, "e.namespaces.size == 1");
#line 363 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp103_);
#line 364 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp106_ = e2;
#line 364 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp107_ = gxml_node_get_namespaces (_tmp106_);
#line 364 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp108_ = _tmp107_;
#line 364 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp109_ = _tmp108_;
#line 364 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp110_ = gee_collection_get_size ((GeeCollection*) _tmp109_);
#line 364 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp111_ = _tmp110_;
#line 364 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp111_ == 0, "e2.namespaces.size == 0");
#line 364 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp109_);
#line 365 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp112_ = e3;
#line 365 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp113_ = gxml_node_get_namespaces (_tmp112_);
#line 365 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp114_ = _tmp113_;
#line 365 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp115_ = _tmp114_;
#line 365 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp116_ = gee_collection_get_size ((GeeCollection*) _tmp115_);
#line 365 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp117_ = _tmp116_;
#line 365 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp117_ == 1, "e3.namespaces.size == 1");
#line 365 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp115_);
#line 366 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp118_ = e4;
#line 366 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp119_ = gxml_node_get_namespaces (_tmp118_);
#line 366 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp120_ = _tmp119_;
#line 366 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp121_ = _tmp120_;
#line 366 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp122_ = gee_collection_get_size ((GeeCollection*) _tmp121_);
#line 366 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp123_ = _tmp122_;
#line 366 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp123_ == 0, "e4.namespaces.size == 0");
#line 366 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp121_);
#line 367 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp124_ = d;
#line 367 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp125_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp124_);
#line 367 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp126_ = _tmp125_;
#line 367 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp127_ = _tmp126_;
#line 367 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp128_ = gee_collection_get_size ((GeeCollection*) _tmp127_);
#line 367 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp129_ = _tmp128_;
#line 367 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp129_ == 3, "d.namespaces.size == 3");
#line 367 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp127_);
#line 368 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp130_ = d;
#line 368 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp131_ = gxml_document_create_element ((GXmlDocument*) _tmp130_, "soup", &_inner_error_);
#line 368 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		c2 = _tmp131_;
#line 368 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 368 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e4);
#line 368 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e3);
#line 368 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e2);
#line 368 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 368 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 368 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 4885 "TElementTest.c"
			goto __catch115_g_error;
		}
#line 369 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp132_ = d;
#line 369 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp133_ = gxml_document_get_root ((GXmlDocument*) _tmp132_);
#line 369 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp134_ = _tmp133_;
#line 369 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp135_ = _tmp134_;
#line 369 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp136_ = gxml_node_get_children_nodes (_tmp135_);
#line 369 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp137_ = _tmp136_;
#line 369 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp138_ = _tmp137_;
#line 369 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp139_ = c2;
#line 369 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp138_, _tmp139_);
#line 369 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp138_);
#line 369 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp135_);
#line 371 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp140_ = c2;
#line 371 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp140_, "http://git.gnome.org/browse/gxml", NULL);
#line 372 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp141_ = d;
#line 372 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp142_ = gxml_tnode_to_string ((GXmlTNode*) _tmp141_);
#line 372 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		str = _tmp142_;
#line 376 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp143_ = string_contains (str, "<root xmlns=\"http://git.gnome.org/browse/gxml\" xmlns:dg=\"http://dev" \
"eloper.gnome.org/\" xmlns:ns=\"http://www.gnome.org/\">");
#line 376 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp143_, "\"<root xmlns=\\\"http://git.gnome.org/browse/gxml\\\" xmlns:dg=\\\"http://developer.gnome.org/\\\" xmlns:ns=\\\"http://www.gnome.org/\\\">\" in str");
#line 377 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp144_ = string_contains (str, "<soup/>");
#line 377 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp144_, "\"<soup/>\" in str");
#line 378 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp145_ = string_contains (str, "</root>");
#line 378 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp145_, "\"</root>\" in str");
#line 379 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp146_ = string_contains (str, "<dg:child>");
#line 379 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp146_, "\"<dg:child>\" in str");
#line 380 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp147_ = string_contains (str, "<children/>");
#line 380 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp147_, "\"<children/>\" in str");
#line 381 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp148_ = string_contains (str, "<ns:nons>");
#line 381 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp148_, "\"<ns:nons>\" in str");
#line 382 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp149_ = string_contains (str, "<childrenons/>");
#line 382 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp149_, "\"<childrenons/>\" in str");
#line 383 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp150_ = string_contains (str, "</ns:nons>");
#line 383 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp150_, "\"</ns:nons>\" in str");
#line 384 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp151_ = string_contains (str, "</dg:child>");
#line 384 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp151_, "\"</dg:child>\" in str");
#line 335 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (str);
#line 335 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (c2);
#line 335 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e4);
#line 335 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e3);
#line 335 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e2);
#line 335 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e);
#line 335 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (r);
#line 335 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (d);
#line 4972 "TElementTest.c"
	}
	goto __finally115;
	__catch115_g_error:
	{
#line 335 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 335 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_inner_error_ = NULL;
#line 385 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_assert_not_reached ();
#line 4983 "TElementTest.c"
	}
	__finally115:
#line 335 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 335 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 335 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 335 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		return;
#line 4994 "TElementTest.c"
	}
}


static void ___lambda92__gtest_func (void) {
#line 334 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	__lambda92_ ();
#line 5002 "TElementTest.c"
}


static void __lambda93_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp0_ = NULL;
		GXmlNode* r = NULL;
		GXmlTDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlTDocument* _tmp3_ = NULL;
		GeeBidirList* _tmp4_ = NULL;
		GeeBidirList* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlTDocument* _tmp8_ = NULL;
		GXmlTDocument* _tmp9_ = NULL;
		GXmlNode* e = NULL;
		GXmlTDocument* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GXmlNode* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GeeBidirList* _tmp15_ = NULL;
		GXmlNode* _tmp16_ = NULL;
		GXmlTDocument* _tmp17_ = NULL;
		GeeList* _tmp18_ = NULL;
		GeeList* _tmp19_ = NULL;
		GeeList* _tmp20_ = NULL;
		gint _tmp21_ = 0;
		gint _tmp22_ = 0;
		GXmlNode* _tmp23_ = NULL;
		GXmlNode* _tmp24_ = NULL;
		GeeList* _tmp25_ = NULL;
		GeeList* _tmp26_ = NULL;
		GeeList* _tmp27_ = NULL;
		gint _tmp28_ = 0;
		gint _tmp29_ = 0;
		GXmlTDocument* _tmp30_ = NULL;
		GeeList* _tmp31_ = NULL;
		GeeList* _tmp32_ = NULL;
		GeeList* _tmp33_ = NULL;
		gint _tmp34_ = 0;
		gint _tmp35_ = 0;
		GXmlNode* e2 = NULL;
		GXmlTDocument* _tmp36_ = NULL;
		GXmlNode* _tmp37_ = NULL;
		GXmlNode* _tmp38_ = NULL;
		GeeBidirList* _tmp39_ = NULL;
		GeeBidirList* _tmp40_ = NULL;
		GeeBidirList* _tmp41_ = NULL;
		GXmlNode* _tmp42_ = NULL;
		GXmlNode* _tmp43_ = NULL;
		GeeList* _tmp44_ = NULL;
		GeeList* _tmp45_ = NULL;
		GeeList* _tmp46_ = NULL;
		gint _tmp47_ = 0;
		gint _tmp48_ = 0;
		GXmlNode* _tmp49_ = NULL;
		GeeList* _tmp50_ = NULL;
		GeeList* _tmp51_ = NULL;
		GeeList* _tmp52_ = NULL;
		gint _tmp53_ = 0;
		gint _tmp54_ = 0;
		GXmlTDocument* _tmp55_ = NULL;
		GeeList* _tmp56_ = NULL;
		GeeList* _tmp57_ = NULL;
		GeeList* _tmp58_ = NULL;
		gint _tmp59_ = 0;
		gint _tmp60_ = 0;
		GXmlNode* e3 = NULL;
		GXmlTDocument* _tmp61_ = NULL;
		GXmlNode* _tmp62_ = NULL;
		GXmlNode* _tmp63_ = NULL;
		GeeBidirList* _tmp64_ = NULL;
		GeeBidirList* _tmp65_ = NULL;
		GeeBidirList* _tmp66_ = NULL;
		GXmlNode* _tmp67_ = NULL;
		GXmlNode* _tmp68_ = NULL;
		GXmlNode* _tmp69_ = NULL;
		GeeList* _tmp70_ = NULL;
		GeeList* _tmp71_ = NULL;
		GeeList* _tmp72_ = NULL;
		gint _tmp73_ = 0;
		gint _tmp74_ = 0;
		GXmlNode* _tmp75_ = NULL;
		GeeList* _tmp76_ = NULL;
		GeeList* _tmp77_ = NULL;
		GeeList* _tmp78_ = NULL;
		gint _tmp79_ = 0;
		gint _tmp80_ = 0;
		GXmlNode* _tmp81_ = NULL;
		GeeList* _tmp82_ = NULL;
		GeeList* _tmp83_ = NULL;
		GeeList* _tmp84_ = NULL;
		gint _tmp85_ = 0;
		gint _tmp86_ = 0;
		GXmlTDocument* _tmp87_ = NULL;
		GeeList* _tmp88_ = NULL;
		GeeList* _tmp89_ = NULL;
		GeeList* _tmp90_ = NULL;
		gint _tmp91_ = 0;
		gint _tmp92_ = 0;
		GXmlNode* e4 = NULL;
		GXmlTDocument* _tmp93_ = NULL;
		GXmlNode* _tmp94_ = NULL;
		GXmlNode* _tmp95_ = NULL;
		GeeBidirList* _tmp96_ = NULL;
		GeeBidirList* _tmp97_ = NULL;
		GeeBidirList* _tmp98_ = NULL;
		GXmlNode* _tmp99_ = NULL;
		GXmlNode* _tmp100_ = NULL;
		GeeList* _tmp101_ = NULL;
		GeeList* _tmp102_ = NULL;
		GeeList* _tmp103_ = NULL;
		gint _tmp104_ = 0;
		gint _tmp105_ = 0;
		GXmlNode* _tmp106_ = NULL;
		GeeList* _tmp107_ = NULL;
		GeeList* _tmp108_ = NULL;
		GeeList* _tmp109_ = NULL;
		gint _tmp110_ = 0;
		gint _tmp111_ = 0;
		GXmlNode* _tmp112_ = NULL;
		GeeList* _tmp113_ = NULL;
		GeeList* _tmp114_ = NULL;
		GeeList* _tmp115_ = NULL;
		gint _tmp116_ = 0;
		gint _tmp117_ = 0;
		GXmlNode* _tmp118_ = NULL;
		GeeList* _tmp119_ = NULL;
		GeeList* _tmp120_ = NULL;
		GeeList* _tmp121_ = NULL;
		gint _tmp122_ = 0;
		gint _tmp123_ = 0;
		GXmlTDocument* _tmp124_ = NULL;
		GeeList* _tmp125_ = NULL;
		GeeList* _tmp126_ = NULL;
		GeeList* _tmp127_ = NULL;
		gint _tmp128_ = 0;
		gint _tmp129_ = 0;
		GXmlNode* c2 = NULL;
		GXmlTDocument* _tmp130_ = NULL;
		GXmlNode* _tmp131_ = NULL;
		GXmlTDocument* _tmp132_ = NULL;
		GXmlNode* _tmp133_ = NULL;
		GXmlNode* _tmp134_ = NULL;
		GXmlNode* _tmp135_ = NULL;
		GeeBidirList* _tmp136_ = NULL;
		GeeBidirList* _tmp137_ = NULL;
		GeeBidirList* _tmp138_ = NULL;
		GXmlNode* _tmp139_ = NULL;
		GXmlNode* _tmp140_ = NULL;
		gchar* str = NULL;
		GXmlTDocument* _tmp141_ = NULL;
		gchar* _tmp142_ = NULL;
		gboolean _tmp143_ = FALSE;
		gboolean _tmp144_ = FALSE;
		gboolean _tmp145_ = FALSE;
		gboolean _tmp146_ = FALSE;
		gboolean _tmp147_ = FALSE;
		gboolean _tmp148_ = FALSE;
		gboolean _tmp149_ = FALSE;
		gboolean _tmp150_ = FALSE;
		gboolean _tmp151_ = FALSE;
#line 389 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp0_ = gxml_tdocument_new ();
#line 389 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		d = _tmp0_;
#line 390 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp1_ = d;
#line 390 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp2_ = gxml_document_create_element ((GXmlDocument*) _tmp1_, "root", &_inner_error_);
#line 390 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		r = _tmp2_;
#line 390 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 390 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 5183 "TElementTest.c"
			goto __catch116_g_error;
		}
#line 391 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp3_ = d;
#line 391 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp4_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp3_);
#line 391 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp5_ = _tmp4_;
#line 391 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp6_ = _tmp5_;
#line 391 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp7_ = r;
#line 391 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp6_, _tmp7_);
#line 391 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp6_);
#line 393 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp8_ = d;
#line 393 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_tnode_set_namespace ((GXmlTNode*) _tmp8_, "http://git.gnome.org/browse/gxml", NULL);
#line 395 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp9_ = d;
#line 395 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_document_set_ns_top ((GXmlDocument*) _tmp9_, TRUE);
#line 396 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp10_ = d;
#line 396 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp11_ = gxml_document_create_element ((GXmlDocument*) _tmp10_, "child", &_inner_error_);
#line 396 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e = _tmp11_;
#line 396 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 396 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 396 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 5220 "TElementTest.c"
			goto __catch116_g_error;
		}
#line 397 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp12_ = r;
#line 397 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp13_ = gxml_node_get_children_nodes (_tmp12_);
#line 397 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp14_ = _tmp13_;
#line 397 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp15_ = _tmp14_;
#line 397 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp16_ = e;
#line 397 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp15_, _tmp16_);
#line 397 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp15_);
#line 398 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp17_ = d;
#line 398 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp18_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp17_);
#line 398 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp19_ = _tmp18_;
#line 398 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp20_ = _tmp19_;
#line 398 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp21_ = gee_collection_get_size ((GeeCollection*) _tmp20_);
#line 398 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp22_ = _tmp21_;
#line 398 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp22_ == 1, "d.namespaces.size == 1");
#line 398 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp20_);
#line 399 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp23_ = e;
#line 399 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp23_, "http://developer.gnome.org/", "dg");
#line 400 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp24_ = e;
#line 400 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp25_ = gxml_node_get_namespaces (_tmp24_);
#line 400 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp26_ = _tmp25_;
#line 400 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp27_ = _tmp26_;
#line 400 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp28_ = gee_collection_get_size ((GeeCollection*) _tmp27_);
#line 400 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp29_ = _tmp28_;
#line 400 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp29_ == 1, "e.namespaces.size == 1");
#line 400 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp27_);
#line 401 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp30_ = d;
#line 401 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp31_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp30_);
#line 401 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp32_ = _tmp31_;
#line 401 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp33_ = _tmp32_;
#line 401 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp34_ = gee_collection_get_size ((GeeCollection*) _tmp33_);
#line 401 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp35_ = _tmp34_;
#line 401 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp35_ == 2, "d.namespaces.size == 2");
#line 401 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp33_);
#line 402 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp36_ = d;
#line 402 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp37_ = gxml_document_create_element ((GXmlDocument*) _tmp36_, "children", &_inner_error_);
#line 402 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e2 = _tmp37_;
#line 402 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 402 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 402 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 402 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 5303 "TElementTest.c"
			goto __catch116_g_error;
		}
#line 403 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp38_ = e;
#line 403 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp39_ = gxml_node_get_children_nodes (_tmp38_);
#line 403 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp40_ = _tmp39_;
#line 403 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp41_ = _tmp40_;
#line 403 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp42_ = e2;
#line 403 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp41_, _tmp42_);
#line 403 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp41_);
#line 404 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp43_ = e;
#line 404 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp44_ = gxml_node_get_namespaces (_tmp43_);
#line 404 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp45_ = _tmp44_;
#line 404 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp46_ = _tmp45_;
#line 404 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp47_ = gee_collection_get_size ((GeeCollection*) _tmp46_);
#line 404 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp48_ = _tmp47_;
#line 404 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp48_ == 1, "e.namespaces.size == 1");
#line 404 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp46_);
#line 405 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp49_ = e2;
#line 405 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp50_ = gxml_node_get_namespaces (_tmp49_);
#line 405 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp51_ = _tmp50_;
#line 405 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp52_ = _tmp51_;
#line 405 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp53_ = gee_collection_get_size ((GeeCollection*) _tmp52_);
#line 405 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp54_ = _tmp53_;
#line 405 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp54_ == 0, "e2.namespaces.size == 0");
#line 405 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp52_);
#line 406 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp55_ = d;
#line 406 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp56_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp55_);
#line 406 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp57_ = _tmp56_;
#line 406 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp58_ = _tmp57_;
#line 406 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp59_ = gee_collection_get_size ((GeeCollection*) _tmp58_);
#line 406 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp60_ = _tmp59_;
#line 406 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp60_ == 2, "d.namespaces.size == 2");
#line 406 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp58_);
#line 407 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp61_ = d;
#line 407 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp62_ = gxml_document_create_element ((GXmlDocument*) _tmp61_, "nons", &_inner_error_);
#line 407 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e3 = _tmp62_;
#line 407 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 407 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e2);
#line 407 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 407 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 407 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 5384 "TElementTest.c"
			goto __catch116_g_error;
		}
#line 408 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp63_ = e;
#line 408 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp64_ = gxml_node_get_children_nodes (_tmp63_);
#line 408 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp65_ = _tmp64_;
#line 408 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp66_ = _tmp65_;
#line 408 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp67_ = e3;
#line 408 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp66_, _tmp67_);
#line 408 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp66_);
#line 409 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp68_ = e3;
#line 409 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp68_, "http://www.gnome.org/", NULL);
#line 410 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp69_ = e;
#line 410 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp70_ = gxml_node_get_namespaces (_tmp69_);
#line 410 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp71_ = _tmp70_;
#line 410 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp72_ = _tmp71_;
#line 410 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp73_ = gee_collection_get_size ((GeeCollection*) _tmp72_);
#line 410 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp74_ = _tmp73_;
#line 410 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp74_ == 1, "e.namespaces.size == 1");
#line 410 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp72_);
#line 411 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp75_ = e2;
#line 411 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp76_ = gxml_node_get_namespaces (_tmp75_);
#line 411 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp77_ = _tmp76_;
#line 411 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp78_ = _tmp77_;
#line 411 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp79_ = gee_collection_get_size ((GeeCollection*) _tmp78_);
#line 411 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp80_ = _tmp79_;
#line 411 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp80_ == 0, "e2.namespaces.size == 0");
#line 411 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp78_);
#line 412 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp81_ = e3;
#line 412 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp82_ = gxml_node_get_namespaces (_tmp81_);
#line 412 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp83_ = _tmp82_;
#line 412 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp84_ = _tmp83_;
#line 412 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp85_ = gee_collection_get_size ((GeeCollection*) _tmp84_);
#line 412 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp86_ = _tmp85_;
#line 412 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp86_ == 1, "e3.namespaces.size == 1");
#line 412 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp84_);
#line 413 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp87_ = d;
#line 413 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp88_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp87_);
#line 413 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp89_ = _tmp88_;
#line 413 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp90_ = _tmp89_;
#line 413 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp91_ = gee_collection_get_size ((GeeCollection*) _tmp90_);
#line 413 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp92_ = _tmp91_;
#line 413 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp92_ == 3, "d.namespaces.size == 3");
#line 413 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp90_);
#line 415 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp93_ = d;
#line 415 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp94_ = gxml_document_create_element ((GXmlDocument*) _tmp93_, "childrenons", &_inner_error_);
#line 415 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e4 = _tmp94_;
#line 415 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 415 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e3);
#line 415 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e2);
#line 415 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 415 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 415 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 5487 "TElementTest.c"
			goto __catch116_g_error;
		}
#line 416 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp95_ = e3;
#line 416 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp96_ = gxml_node_get_children_nodes (_tmp95_);
#line 416 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp97_ = _tmp96_;
#line 416 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp98_ = _tmp97_;
#line 416 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp99_ = e4;
#line 416 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp98_, _tmp99_);
#line 416 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp98_);
#line 417 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp100_ = e;
#line 417 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp101_ = gxml_node_get_namespaces (_tmp100_);
#line 417 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp102_ = _tmp101_;
#line 417 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp103_ = _tmp102_;
#line 417 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp104_ = gee_collection_get_size ((GeeCollection*) _tmp103_);
#line 417 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp105_ = _tmp104_;
#line 417 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp105_ == 1, "e.namespaces.size == 1");
#line 417 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp103_);
#line 418 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp106_ = e2;
#line 418 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp107_ = gxml_node_get_namespaces (_tmp106_);
#line 418 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp108_ = _tmp107_;
#line 418 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp109_ = _tmp108_;
#line 418 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp110_ = gee_collection_get_size ((GeeCollection*) _tmp109_);
#line 418 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp111_ = _tmp110_;
#line 418 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp111_ == 0, "e2.namespaces.size == 0");
#line 418 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp109_);
#line 419 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp112_ = e3;
#line 419 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp113_ = gxml_node_get_namespaces (_tmp112_);
#line 419 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp114_ = _tmp113_;
#line 419 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp115_ = _tmp114_;
#line 419 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp116_ = gee_collection_get_size ((GeeCollection*) _tmp115_);
#line 419 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp117_ = _tmp116_;
#line 419 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp117_ == 1, "e3.namespaces.size == 1");
#line 419 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp115_);
#line 420 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp118_ = e4;
#line 420 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp119_ = gxml_node_get_namespaces (_tmp118_);
#line 420 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp120_ = _tmp119_;
#line 420 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp121_ = _tmp120_;
#line 420 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp122_ = gee_collection_get_size ((GeeCollection*) _tmp121_);
#line 420 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp123_ = _tmp122_;
#line 420 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp123_ == 0, "e4.namespaces.size == 0");
#line 420 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp121_);
#line 421 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp124_ = d;
#line 421 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp125_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp124_);
#line 421 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp126_ = _tmp125_;
#line 421 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp127_ = _tmp126_;
#line 421 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp128_ = gee_collection_get_size ((GeeCollection*) _tmp127_);
#line 421 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp129_ = _tmp128_;
#line 421 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp129_ == 3, "d.namespaces.size == 3");
#line 421 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp127_);
#line 422 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp130_ = d;
#line 422 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp131_ = gxml_document_create_element ((GXmlDocument*) _tmp130_, "soup", &_inner_error_);
#line 422 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		c2 = _tmp131_;
#line 422 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 422 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e4);
#line 422 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e3);
#line 422 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e2);
#line 422 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 422 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 422 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 5604 "TElementTest.c"
			goto __catch116_g_error;
		}
#line 423 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp132_ = d;
#line 423 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp133_ = gxml_document_get_root ((GXmlDocument*) _tmp132_);
#line 423 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp134_ = _tmp133_;
#line 423 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp135_ = _tmp134_;
#line 423 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp136_ = gxml_node_get_children_nodes (_tmp135_);
#line 423 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp137_ = _tmp136_;
#line 423 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp138_ = _tmp137_;
#line 423 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp139_ = c2;
#line 423 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp138_, _tmp139_);
#line 423 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp138_);
#line 423 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp135_);
#line 425 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp140_ = c2;
#line 425 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp140_, "http://git.gnome.org/browse/gxml", NULL);
#line 426 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp141_ = d;
#line 426 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp142_ = gxml_tnode_to_string ((GXmlTNode*) _tmp141_);
#line 426 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		str = _tmp142_;
#line 430 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp143_ = string_contains (str, "<root xmlns=\"http://git.gnome.org/browse/gxml\" xmlns:dg=\"http://dev" \
"eloper.gnome.org/\">");
#line 430 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp143_, "\"<root xmlns=\\\"http://git.gnome.org/browse/gxml\\\" xmlns:dg=\\\"http://developer.gnome.org/\\\">\" in str");
#line 431 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp144_ = string_contains (str, "<soup/>");
#line 431 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp144_, "\"<soup/>\" in str");
#line 432 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp145_ = string_contains (str, "</root>");
#line 432 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp145_, "\"</root>\" in str");
#line 433 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp146_ = string_contains (str, "<dg:child>");
#line 433 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp146_, "\"<dg:child>\" in str");
#line 434 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp147_ = string_contains (str, "<children/>");
#line 434 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp147_, "\"<children/>\" in str");
#line 435 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp148_ = string_contains (str, "<nons xmlns=\"http://www.gnome.org/\">");
#line 435 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp148_, "\"<nons xmlns=\\\"http://www.gnome.org/\\\">\" in str");
#line 436 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp149_ = string_contains (str, "<childrenons/>");
#line 436 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp149_, "\"<childrenons/>\" in str");
#line 437 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp150_ = string_contains (str, "</nons>");
#line 437 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp150_, "\"</nons>\" in str");
#line 438 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp151_ = string_contains (str, "</dg:child>");
#line 438 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp151_, "\"</dg:child>\" in str");
#line 388 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (str);
#line 388 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (c2);
#line 388 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e4);
#line 388 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e3);
#line 388 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e2);
#line 388 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e);
#line 388 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (r);
#line 388 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (d);
#line 5691 "TElementTest.c"
	}
	goto __finally116;
	__catch116_g_error:
	{
#line 388 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 388 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_inner_error_ = NULL;
#line 439 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_assert_not_reached ();
#line 5702 "TElementTest.c"
	}
	__finally116:
#line 388 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 388 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 388 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 388 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		return;
#line 5713 "TElementTest.c"
	}
}


static void ___lambda93__gtest_func (void) {
#line 387 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	__lambda93_ ();
#line 5721 "TElementTest.c"
}


static void __lambda94_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp0_ = NULL;
		GXmlNode* r = NULL;
		GXmlTDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlTDocument* _tmp3_ = NULL;
		GeeBidirList* _tmp4_ = NULL;
		GeeBidirList* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlTDocument* _tmp8_ = NULL;
		GXmlTDocument* _tmp9_ = NULL;
		GXmlNode* e = NULL;
		GXmlTDocument* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GXmlNode* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GeeBidirList* _tmp15_ = NULL;
		GXmlNode* _tmp16_ = NULL;
		GXmlTDocument* _tmp17_ = NULL;
		GeeList* _tmp18_ = NULL;
		GeeList* _tmp19_ = NULL;
		GeeList* _tmp20_ = NULL;
		gint _tmp21_ = 0;
		gint _tmp22_ = 0;
		GXmlNode* _tmp23_ = NULL;
		GXmlNode* _tmp24_ = NULL;
		GeeList* _tmp25_ = NULL;
		GeeList* _tmp26_ = NULL;
		GeeList* _tmp27_ = NULL;
		gint _tmp28_ = 0;
		gint _tmp29_ = 0;
		GXmlTDocument* _tmp30_ = NULL;
		GeeList* _tmp31_ = NULL;
		GeeList* _tmp32_ = NULL;
		GeeList* _tmp33_ = NULL;
		gint _tmp34_ = 0;
		gint _tmp35_ = 0;
		GXmlNode* e2 = NULL;
		GXmlTDocument* _tmp36_ = NULL;
		GXmlNode* _tmp37_ = NULL;
		GXmlNode* _tmp38_ = NULL;
		GeeBidirList* _tmp39_ = NULL;
		GeeBidirList* _tmp40_ = NULL;
		GeeBidirList* _tmp41_ = NULL;
		GXmlNode* _tmp42_ = NULL;
		GXmlNode* _tmp43_ = NULL;
		GeeList* _tmp44_ = NULL;
		GeeList* _tmp45_ = NULL;
		GeeList* _tmp46_ = NULL;
		gint _tmp47_ = 0;
		gint _tmp48_ = 0;
		GXmlNode* _tmp49_ = NULL;
		GeeList* _tmp50_ = NULL;
		GeeList* _tmp51_ = NULL;
		GeeList* _tmp52_ = NULL;
		gint _tmp53_ = 0;
		gint _tmp54_ = 0;
		GXmlTDocument* _tmp55_ = NULL;
		GeeList* _tmp56_ = NULL;
		GeeList* _tmp57_ = NULL;
		GeeList* _tmp58_ = NULL;
		gint _tmp59_ = 0;
		gint _tmp60_ = 0;
		GXmlNode* e3 = NULL;
		GXmlTDocument* _tmp61_ = NULL;
		GXmlNode* _tmp62_ = NULL;
		GXmlNode* _tmp63_ = NULL;
		GeeBidirList* _tmp64_ = NULL;
		GeeBidirList* _tmp65_ = NULL;
		GeeBidirList* _tmp66_ = NULL;
		GXmlNode* _tmp67_ = NULL;
		GXmlNode* _tmp68_ = NULL;
		GXmlNode* _tmp69_ = NULL;
		GeeList* _tmp70_ = NULL;
		GeeList* _tmp71_ = NULL;
		GeeList* _tmp72_ = NULL;
		gint _tmp73_ = 0;
		gint _tmp74_ = 0;
		GXmlNode* _tmp75_ = NULL;
		GeeList* _tmp76_ = NULL;
		GeeList* _tmp77_ = NULL;
		GeeList* _tmp78_ = NULL;
		gint _tmp79_ = 0;
		gint _tmp80_ = 0;
		GXmlNode* _tmp81_ = NULL;
		GeeList* _tmp82_ = NULL;
		GeeList* _tmp83_ = NULL;
		GeeList* _tmp84_ = NULL;
		gint _tmp85_ = 0;
		gint _tmp86_ = 0;
		GXmlTDocument* _tmp87_ = NULL;
		GeeList* _tmp88_ = NULL;
		GeeList* _tmp89_ = NULL;
		GeeList* _tmp90_ = NULL;
		gint _tmp91_ = 0;
		gint _tmp92_ = 0;
		GXmlNode* e4 = NULL;
		GXmlTDocument* _tmp93_ = NULL;
		GXmlNode* _tmp94_ = NULL;
		GXmlNode* _tmp95_ = NULL;
		GeeBidirList* _tmp96_ = NULL;
		GeeBidirList* _tmp97_ = NULL;
		GeeBidirList* _tmp98_ = NULL;
		GXmlNode* _tmp99_ = NULL;
		GXmlNode* _tmp100_ = NULL;
		GXmlNode* _tmp101_ = NULL;
		GeeList* _tmp102_ = NULL;
		GeeList* _tmp103_ = NULL;
		GeeList* _tmp104_ = NULL;
		gint _tmp105_ = 0;
		gint _tmp106_ = 0;
		GXmlNode* _tmp107_ = NULL;
		GeeList* _tmp108_ = NULL;
		GeeList* _tmp109_ = NULL;
		GeeList* _tmp110_ = NULL;
		gint _tmp111_ = 0;
		gint _tmp112_ = 0;
		GXmlNode* _tmp113_ = NULL;
		GeeList* _tmp114_ = NULL;
		GeeList* _tmp115_ = NULL;
		GeeList* _tmp116_ = NULL;
		gint _tmp117_ = 0;
		gint _tmp118_ = 0;
		GXmlNode* _tmp119_ = NULL;
		GeeList* _tmp120_ = NULL;
		GeeList* _tmp121_ = NULL;
		GeeList* _tmp122_ = NULL;
		gint _tmp123_ = 0;
		gint _tmp124_ = 0;
		GXmlTDocument* _tmp125_ = NULL;
		GeeList* _tmp126_ = NULL;
		GeeList* _tmp127_ = NULL;
		GeeList* _tmp128_ = NULL;
		gint _tmp129_ = 0;
		gint _tmp130_ = 0;
		GXmlNode* c2 = NULL;
		GXmlTDocument* _tmp131_ = NULL;
		GXmlNode* _tmp132_ = NULL;
		GXmlTDocument* _tmp133_ = NULL;
		GXmlNode* _tmp134_ = NULL;
		GXmlNode* _tmp135_ = NULL;
		GXmlNode* _tmp136_ = NULL;
		GeeBidirList* _tmp137_ = NULL;
		GeeBidirList* _tmp138_ = NULL;
		GeeBidirList* _tmp139_ = NULL;
		GXmlNode* _tmp140_ = NULL;
		GXmlNode* _tmp141_ = NULL;
		gchar* str = NULL;
		GXmlTDocument* _tmp142_ = NULL;
		gchar* _tmp143_ = NULL;
		gboolean _tmp144_ = FALSE;
		gboolean _tmp145_ = FALSE;
		gboolean _tmp146_ = FALSE;
		gboolean _tmp147_ = FALSE;
		gboolean _tmp148_ = FALSE;
		gboolean _tmp149_ = FALSE;
		gboolean _tmp150_ = FALSE;
		gboolean _tmp151_ = FALSE;
		gboolean _tmp152_ = FALSE;
#line 443 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp0_ = gxml_tdocument_new ();
#line 443 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		d = _tmp0_;
#line 444 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp1_ = d;
#line 444 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp2_ = gxml_document_create_element ((GXmlDocument*) _tmp1_, "root", &_inner_error_);
#line 444 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		r = _tmp2_;
#line 444 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 444 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 5903 "TElementTest.c"
			goto __catch117_g_error;
		}
#line 445 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp3_ = d;
#line 445 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp4_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp3_);
#line 445 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp5_ = _tmp4_;
#line 445 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp6_ = _tmp5_;
#line 445 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp7_ = r;
#line 445 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp6_, _tmp7_);
#line 445 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp6_);
#line 447 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp8_ = d;
#line 447 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_tnode_set_namespace ((GXmlTNode*) _tmp8_, "http://git.gnome.org/browse/gxml", NULL);
#line 448 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp9_ = d;
#line 448 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_document_set_prefix_default_ns ((GXmlDocument*) _tmp9_, TRUE);
#line 449 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp10_ = d;
#line 449 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp11_ = gxml_document_create_element ((GXmlDocument*) _tmp10_, "child", &_inner_error_);
#line 449 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e = _tmp11_;
#line 449 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 449 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 449 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 5940 "TElementTest.c"
			goto __catch117_g_error;
		}
#line 450 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp12_ = r;
#line 450 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp13_ = gxml_node_get_children_nodes (_tmp12_);
#line 450 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp14_ = _tmp13_;
#line 450 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp15_ = _tmp14_;
#line 450 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp16_ = e;
#line 450 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp15_, _tmp16_);
#line 450 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp15_);
#line 451 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp17_ = d;
#line 451 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp18_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp17_);
#line 451 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp19_ = _tmp18_;
#line 451 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp20_ = _tmp19_;
#line 451 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp21_ = gee_collection_get_size ((GeeCollection*) _tmp20_);
#line 451 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp22_ = _tmp21_;
#line 451 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp22_ == 1, "d.namespaces.size == 1");
#line 451 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp20_);
#line 452 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp23_ = e;
#line 452 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp23_, "http://developer.gnome.org/", "dg");
#line 453 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp24_ = e;
#line 453 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp25_ = gxml_node_get_namespaces (_tmp24_);
#line 453 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp26_ = _tmp25_;
#line 453 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp27_ = _tmp26_;
#line 453 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp28_ = gee_collection_get_size ((GeeCollection*) _tmp27_);
#line 453 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp29_ = _tmp28_;
#line 453 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp29_ == 1, "e.namespaces.size == 1");
#line 453 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp27_);
#line 454 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp30_ = d;
#line 454 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp31_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp30_);
#line 454 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp32_ = _tmp31_;
#line 454 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp33_ = _tmp32_;
#line 454 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp34_ = gee_collection_get_size ((GeeCollection*) _tmp33_);
#line 454 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp35_ = _tmp34_;
#line 454 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp35_ == 2, "d.namespaces.size == 2");
#line 454 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp33_);
#line 455 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp36_ = d;
#line 455 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp37_ = gxml_document_create_element ((GXmlDocument*) _tmp36_, "children", &_inner_error_);
#line 455 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e2 = _tmp37_;
#line 455 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 455 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 455 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 455 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 6023 "TElementTest.c"
			goto __catch117_g_error;
		}
#line 456 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp38_ = e;
#line 456 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp39_ = gxml_node_get_children_nodes (_tmp38_);
#line 456 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp40_ = _tmp39_;
#line 456 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp41_ = _tmp40_;
#line 456 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp42_ = e2;
#line 456 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp41_, _tmp42_);
#line 456 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp41_);
#line 457 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp43_ = e;
#line 457 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp44_ = gxml_node_get_namespaces (_tmp43_);
#line 457 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp45_ = _tmp44_;
#line 457 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp46_ = _tmp45_;
#line 457 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp47_ = gee_collection_get_size ((GeeCollection*) _tmp46_);
#line 457 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp48_ = _tmp47_;
#line 457 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp48_ == 1, "e.namespaces.size == 1");
#line 457 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp46_);
#line 458 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp49_ = e2;
#line 458 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp50_ = gxml_node_get_namespaces (_tmp49_);
#line 458 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp51_ = _tmp50_;
#line 458 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp52_ = _tmp51_;
#line 458 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp53_ = gee_collection_get_size ((GeeCollection*) _tmp52_);
#line 458 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp54_ = _tmp53_;
#line 458 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp54_ == 0, "e2.namespaces.size == 0");
#line 458 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp52_);
#line 459 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp55_ = d;
#line 459 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp56_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp55_);
#line 459 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp57_ = _tmp56_;
#line 459 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp58_ = _tmp57_;
#line 459 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp59_ = gee_collection_get_size ((GeeCollection*) _tmp58_);
#line 459 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp60_ = _tmp59_;
#line 459 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp60_ == 2, "d.namespaces.size == 2");
#line 459 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp58_);
#line 460 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp61_ = d;
#line 460 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp62_ = gxml_document_create_element ((GXmlDocument*) _tmp61_, "nons", &_inner_error_);
#line 460 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e3 = _tmp62_;
#line 460 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 460 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e2);
#line 460 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 460 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 460 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 6104 "TElementTest.c"
			goto __catch117_g_error;
		}
#line 461 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp63_ = e;
#line 461 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp64_ = gxml_node_get_children_nodes (_tmp63_);
#line 461 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp65_ = _tmp64_;
#line 461 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp66_ = _tmp65_;
#line 461 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp67_ = e3;
#line 461 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp66_, _tmp67_);
#line 461 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp66_);
#line 462 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp68_ = e3;
#line 462 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp68_, "http://www.gnome.org/", "ns");
#line 463 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp69_ = e;
#line 463 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp70_ = gxml_node_get_namespaces (_tmp69_);
#line 463 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp71_ = _tmp70_;
#line 463 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp72_ = _tmp71_;
#line 463 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp73_ = gee_collection_get_size ((GeeCollection*) _tmp72_);
#line 463 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp74_ = _tmp73_;
#line 463 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp74_ == 1, "e.namespaces.size == 1");
#line 463 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp72_);
#line 464 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp75_ = e2;
#line 464 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp76_ = gxml_node_get_namespaces (_tmp75_);
#line 464 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp77_ = _tmp76_;
#line 464 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp78_ = _tmp77_;
#line 464 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp79_ = gee_collection_get_size ((GeeCollection*) _tmp78_);
#line 464 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp80_ = _tmp79_;
#line 464 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp80_ == 0, "e2.namespaces.size == 0");
#line 464 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp78_);
#line 465 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp81_ = e3;
#line 465 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp82_ = gxml_node_get_namespaces (_tmp81_);
#line 465 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp83_ = _tmp82_;
#line 465 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp84_ = _tmp83_;
#line 465 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp85_ = gee_collection_get_size ((GeeCollection*) _tmp84_);
#line 465 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp86_ = _tmp85_;
#line 465 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp86_ == 1, "e3.namespaces.size == 1");
#line 465 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp84_);
#line 466 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp87_ = d;
#line 466 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp88_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp87_);
#line 466 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp89_ = _tmp88_;
#line 466 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp90_ = _tmp89_;
#line 466 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp91_ = gee_collection_get_size ((GeeCollection*) _tmp90_);
#line 466 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp92_ = _tmp91_;
#line 466 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp92_ == 3, "d.namespaces.size == 3");
#line 466 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp90_);
#line 468 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp93_ = d;
#line 468 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp94_ = gxml_document_create_element ((GXmlDocument*) _tmp93_, "childrenons", &_inner_error_);
#line 468 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e4 = _tmp94_;
#line 468 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 468 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e3);
#line 468 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e2);
#line 468 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 468 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 468 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 6207 "TElementTest.c"
			goto __catch117_g_error;
		}
#line 469 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp95_ = e3;
#line 469 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp96_ = gxml_node_get_children_nodes (_tmp95_);
#line 469 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp97_ = _tmp96_;
#line 469 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp98_ = _tmp97_;
#line 469 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp99_ = e4;
#line 469 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp98_, _tmp99_);
#line 469 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp98_);
#line 470 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp100_ = e4;
#line 470 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp100_, "http://www.gnome.org/", "ns");
#line 471 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp101_ = e;
#line 471 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp102_ = gxml_node_get_namespaces (_tmp101_);
#line 471 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp103_ = _tmp102_;
#line 471 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp104_ = _tmp103_;
#line 471 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp105_ = gee_collection_get_size ((GeeCollection*) _tmp104_);
#line 471 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp106_ = _tmp105_;
#line 471 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp106_ == 1, "e.namespaces.size == 1");
#line 471 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp104_);
#line 472 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp107_ = e2;
#line 472 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp108_ = gxml_node_get_namespaces (_tmp107_);
#line 472 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp109_ = _tmp108_;
#line 472 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp110_ = _tmp109_;
#line 472 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp111_ = gee_collection_get_size ((GeeCollection*) _tmp110_);
#line 472 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp112_ = _tmp111_;
#line 472 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp112_ == 0, "e2.namespaces.size == 0");
#line 472 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp110_);
#line 473 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp113_ = e3;
#line 473 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp114_ = gxml_node_get_namespaces (_tmp113_);
#line 473 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp115_ = _tmp114_;
#line 473 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp116_ = _tmp115_;
#line 473 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp117_ = gee_collection_get_size ((GeeCollection*) _tmp116_);
#line 473 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp118_ = _tmp117_;
#line 473 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp118_ == 1, "e3.namespaces.size == 1");
#line 473 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp116_);
#line 474 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp119_ = e4;
#line 474 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp120_ = gxml_node_get_namespaces (_tmp119_);
#line 474 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp121_ = _tmp120_;
#line 474 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp122_ = _tmp121_;
#line 474 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp123_ = gee_collection_get_size ((GeeCollection*) _tmp122_);
#line 474 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp124_ = _tmp123_;
#line 474 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp124_ == 1, "e4.namespaces.size == 1");
#line 474 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp122_);
#line 475 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp125_ = d;
#line 475 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp126_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp125_);
#line 475 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp127_ = _tmp126_;
#line 475 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp128_ = _tmp127_;
#line 475 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp129_ = gee_collection_get_size ((GeeCollection*) _tmp128_);
#line 475 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp130_ = _tmp129_;
#line 475 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp130_ == 3, "d.namespaces.size == 3");
#line 475 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp128_);
#line 476 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp131_ = d;
#line 476 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp132_ = gxml_document_create_element ((GXmlDocument*) _tmp131_, "soup", &_inner_error_);
#line 476 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		c2 = _tmp132_;
#line 476 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 476 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e4);
#line 476 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e3);
#line 476 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e2);
#line 476 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 476 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 476 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 6328 "TElementTest.c"
			goto __catch117_g_error;
		}
#line 477 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp133_ = d;
#line 477 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp134_ = gxml_document_get_root ((GXmlDocument*) _tmp133_);
#line 477 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp135_ = _tmp134_;
#line 477 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp136_ = _tmp135_;
#line 477 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp137_ = gxml_node_get_children_nodes (_tmp136_);
#line 477 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp138_ = _tmp137_;
#line 477 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp139_ = _tmp138_;
#line 477 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp140_ = c2;
#line 477 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp139_, _tmp140_);
#line 477 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp139_);
#line 477 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp136_);
#line 479 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp141_ = c2;
#line 479 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp141_, "http://git.gnome.org/browse/gxml", NULL);
#line 480 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp142_ = d;
#line 480 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp143_ = gxml_tnode_to_string ((GXmlTNode*) _tmp142_);
#line 480 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		str = _tmp143_;
#line 484 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp144_ = string_contains (str, "<root xmlns=\"http://git.gnome.org/browse/gxml\">");
#line 484 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp144_, "\"<root xmlns=\\\"http://git.gnome.org/browse/gxml\\\">\" in str");
#line 485 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp145_ = string_contains (str, "<soup/>");
#line 485 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp145_, "\"<soup/>\" in str");
#line 486 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp146_ = string_contains (str, "</root>");
#line 486 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp146_, "\"</root>\" in str");
#line 487 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp147_ = string_contains (str, "<dg:child xmlns:dg=\"http://developer.gnome.org/\">");
#line 487 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp147_, "\"<dg:child xmlns:dg=\\\"http://developer.gnome.org/\\\">\" in str");
#line 488 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp148_ = string_contains (str, "<children/>");
#line 488 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp148_, "\"<children/>\" in str");
#line 489 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp149_ = string_contains (str, "<ns:nons xmlns:ns=\"http://www.gnome.org/\">");
#line 489 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp149_, "\"<ns:nons xmlns:ns=\\\"http://www.gnome.org/\\\">\" in str");
#line 490 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp150_ = string_contains (str, "<ns:childrenons/>");
#line 490 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp150_, "\"<ns:childrenons/>\" in str");
#line 491 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp151_ = string_contains (str, "</ns:nons>");
#line 491 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp151_, "\"</ns:nons>\" in str");
#line 492 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp152_ = string_contains (str, "</dg:child>");
#line 492 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp152_, "\"</dg:child>\" in str");
#line 442 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (str);
#line 442 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (c2);
#line 442 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e4);
#line 442 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e3);
#line 442 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e2);
#line 442 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e);
#line 442 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (r);
#line 442 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (d);
#line 6415 "TElementTest.c"
	}
	goto __finally117;
	__catch117_g_error:
	{
#line 442 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 442 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_inner_error_ = NULL;
#line 493 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_assert_not_reached ();
#line 6426 "TElementTest.c"
	}
	__finally117:
#line 442 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 442 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 442 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 442 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		return;
#line 6437 "TElementTest.c"
	}
}


static void ___lambda94__gtest_func (void) {
#line 441 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	__lambda94_ ();
#line 6445 "TElementTest.c"
}


static void __lambda95_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp0_ = NULL;
		GXmlNode* r = NULL;
		GXmlTDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlTDocument* _tmp3_ = NULL;
		GeeBidirList* _tmp4_ = NULL;
		GeeBidirList* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlTDocument* _tmp8_ = NULL;
		GXmlTDocument* _tmp9_ = NULL;
		GXmlNode* e = NULL;
		GXmlTDocument* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GXmlNode* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GeeBidirList* _tmp15_ = NULL;
		GXmlNode* _tmp16_ = NULL;
		GXmlTDocument* _tmp17_ = NULL;
		GeeList* _tmp18_ = NULL;
		GeeList* _tmp19_ = NULL;
		GeeList* _tmp20_ = NULL;
		gint _tmp21_ = 0;
		gint _tmp22_ = 0;
		GXmlNode* e2 = NULL;
		GXmlTDocument* _tmp23_ = NULL;
		GXmlNode* _tmp24_ = NULL;
		GXmlNode* _tmp25_ = NULL;
		GeeBidirList* _tmp26_ = NULL;
		GeeBidirList* _tmp27_ = NULL;
		GeeBidirList* _tmp28_ = NULL;
		GXmlNode* _tmp29_ = NULL;
		GXmlNode* e3 = NULL;
		GXmlTDocument* _tmp30_ = NULL;
		GXmlNode* _tmp31_ = NULL;
		GXmlNode* _tmp32_ = NULL;
		GeeBidirList* _tmp33_ = NULL;
		GeeBidirList* _tmp34_ = NULL;
		GeeBidirList* _tmp35_ = NULL;
		GXmlNode* _tmp36_ = NULL;
		GXmlNode* e4 = NULL;
		GXmlTDocument* _tmp37_ = NULL;
		GXmlNode* _tmp38_ = NULL;
		GXmlNode* _tmp39_ = NULL;
		GeeBidirList* _tmp40_ = NULL;
		GeeBidirList* _tmp41_ = NULL;
		GeeBidirList* _tmp42_ = NULL;
		GXmlNode* _tmp43_ = NULL;
		GXmlNode* c2 = NULL;
		GXmlTDocument* _tmp44_ = NULL;
		GXmlNode* _tmp45_ = NULL;
		GXmlTDocument* _tmp46_ = NULL;
		GXmlNode* _tmp47_ = NULL;
		GXmlNode* _tmp48_ = NULL;
		GXmlNode* _tmp49_ = NULL;
		GeeBidirList* _tmp50_ = NULL;
		GeeBidirList* _tmp51_ = NULL;
		GeeBidirList* _tmp52_ = NULL;
		GXmlNode* _tmp53_ = NULL;
		GXmlNode* _tmp54_ = NULL;
		gchar* str = NULL;
		GXmlTDocument* _tmp55_ = NULL;
		gchar* _tmp56_ = NULL;
		gboolean _tmp57_ = FALSE;
		gboolean _tmp58_ = FALSE;
		gboolean _tmp59_ = FALSE;
		gboolean _tmp60_ = FALSE;
		gboolean _tmp61_ = FALSE;
		gboolean _tmp62_ = FALSE;
		gboolean _tmp63_ = FALSE;
		gboolean _tmp64_ = FALSE;
		gboolean _tmp65_ = FALSE;
#line 497 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp0_ = gxml_tdocument_new ();
#line 497 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		d = _tmp0_;
#line 498 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp1_ = d;
#line 498 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp2_ = gxml_document_create_element ((GXmlDocument*) _tmp1_, "root", &_inner_error_);
#line 498 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		r = _tmp2_;
#line 498 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 498 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 6540 "TElementTest.c"
			goto __catch118_g_error;
		}
#line 499 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp3_ = d;
#line 499 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp4_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp3_);
#line 499 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp5_ = _tmp4_;
#line 499 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp6_ = _tmp5_;
#line 499 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp7_ = r;
#line 499 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp6_, _tmp7_);
#line 499 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp6_);
#line 501 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp8_ = d;
#line 501 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_tnode_set_namespace ((GXmlTNode*) _tmp8_, "http://git.gnome.org/browse/gxml", "gxml");
#line 503 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp9_ = d;
#line 503 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_document_set_prefix_default_ns ((GXmlDocument*) _tmp9_, TRUE);
#line 504 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp10_ = d;
#line 504 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp11_ = gxml_document_create_element ((GXmlDocument*) _tmp10_, "child", &_inner_error_);
#line 504 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e = _tmp11_;
#line 504 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 504 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 504 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 6577 "TElementTest.c"
			goto __catch118_g_error;
		}
#line 505 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp12_ = r;
#line 505 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp13_ = gxml_node_get_children_nodes (_tmp12_);
#line 505 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp14_ = _tmp13_;
#line 505 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp15_ = _tmp14_;
#line 505 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp16_ = e;
#line 505 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp15_, _tmp16_);
#line 505 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp15_);
#line 506 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp17_ = d;
#line 506 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp18_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp17_);
#line 506 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp19_ = _tmp18_;
#line 506 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp20_ = _tmp19_;
#line 506 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp21_ = gee_collection_get_size ((GeeCollection*) _tmp20_);
#line 506 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp22_ = _tmp21_;
#line 506 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp22_ == 1, "d.namespaces.size == 1");
#line 506 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp20_);
#line 507 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp23_ = d;
#line 507 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp24_ = gxml_document_create_element ((GXmlDocument*) _tmp23_, "children", &_inner_error_);
#line 507 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e2 = _tmp24_;
#line 507 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 507 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 507 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 507 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 6624 "TElementTest.c"
			goto __catch118_g_error;
		}
#line 508 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp25_ = e;
#line 508 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp26_ = gxml_node_get_children_nodes (_tmp25_);
#line 508 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp27_ = _tmp26_;
#line 508 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp28_ = _tmp27_;
#line 508 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp29_ = e2;
#line 508 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp28_, _tmp29_);
#line 508 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp28_);
#line 509 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp30_ = d;
#line 509 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp31_ = gxml_document_create_element ((GXmlDocument*) _tmp30_, "nons", &_inner_error_);
#line 509 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e3 = _tmp31_;
#line 509 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 509 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e2);
#line 509 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 509 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 509 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 6657 "TElementTest.c"
			goto __catch118_g_error;
		}
#line 510 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp32_ = e;
#line 510 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp33_ = gxml_node_get_children_nodes (_tmp32_);
#line 510 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp34_ = _tmp33_;
#line 510 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp35_ = _tmp34_;
#line 510 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp36_ = e3;
#line 510 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp35_, _tmp36_);
#line 510 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp35_);
#line 511 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp37_ = d;
#line 511 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp38_ = gxml_document_create_element ((GXmlDocument*) _tmp37_, "childrenons", &_inner_error_);
#line 511 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e4 = _tmp38_;
#line 511 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 511 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e3);
#line 511 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e2);
#line 511 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 511 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 511 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 6692 "TElementTest.c"
			goto __catch118_g_error;
		}
#line 512 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp39_ = e3;
#line 512 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp40_ = gxml_node_get_children_nodes (_tmp39_);
#line 512 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp41_ = _tmp40_;
#line 512 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp42_ = _tmp41_;
#line 512 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp43_ = e4;
#line 512 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp42_, _tmp43_);
#line 512 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp42_);
#line 513 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp44_ = d;
#line 513 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp45_ = gxml_document_create_element ((GXmlDocument*) _tmp44_, "soup", &_inner_error_);
#line 513 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		c2 = _tmp45_;
#line 513 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 513 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e4);
#line 513 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e3);
#line 513 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e2);
#line 513 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 513 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 513 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 6729 "TElementTest.c"
			goto __catch118_g_error;
		}
#line 514 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp46_ = d;
#line 514 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp47_ = gxml_document_get_root ((GXmlDocument*) _tmp46_);
#line 514 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp48_ = _tmp47_;
#line 514 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp49_ = _tmp48_;
#line 514 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp50_ = gxml_node_get_children_nodes (_tmp49_);
#line 514 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp51_ = _tmp50_;
#line 514 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp52_ = _tmp51_;
#line 514 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp53_ = c2;
#line 514 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp52_, _tmp53_);
#line 514 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp52_);
#line 514 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp49_);
#line 515 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp54_ = c2;
#line 515 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp54_, "http://git.gnome.org/browse/gxml", "gxml");
#line 516 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp55_ = d;
#line 516 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp56_ = gxml_tnode_to_string ((GXmlTNode*) _tmp55_);
#line 516 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		str = _tmp56_;
#line 520 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp57_ = string_contains (str, "<gxml:root xmlns:gxml=\"http://git.gnome.org/browse/gxml\">");
#line 520 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp57_, "\"<gxml:root xmlns:gxml=\\\"http://git.gnome.org/browse/gxml\\\">\" in str");
#line 521 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp58_ = string_contains (str, "<gxml:soup/>");
#line 521 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp58_, "\"<gxml:soup/>\" in str");
#line 522 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp59_ = string_contains (str, "</gxml:root>");
#line 522 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp59_, "\"</gxml:root>\" in str");
#line 523 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp60_ = string_contains (str, "<gxml:child>");
#line 523 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp60_, "\"<gxml:child>\" in str");
#line 524 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp61_ = string_contains (str, "<gxml:children/>");
#line 524 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp61_, "\"<gxml:children/>\" in str");
#line 525 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp62_ = string_contains (str, "<gxml:nons>");
#line 525 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp62_, "\"<gxml:nons>\" in str");
#line 526 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp63_ = string_contains (str, "<gxml:childrenons/>");
#line 526 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp63_, "\"<gxml:childrenons/>\" in str");
#line 527 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp64_ = string_contains (str, "</gxml:nons>");
#line 527 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp64_, "\"</gxml:nons>\" in str");
#line 528 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp65_ = string_contains (str, "</gxml:child>");
#line 528 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp65_, "\"</gxml:child>\" in str");
#line 496 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (str);
#line 496 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (c2);
#line 496 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e4);
#line 496 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e3);
#line 496 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e2);
#line 496 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e);
#line 496 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (r);
#line 496 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (d);
#line 6816 "TElementTest.c"
	}
	goto __finally118;
	__catch118_g_error:
	{
#line 496 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 496 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_inner_error_ = NULL;
#line 529 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_assert_not_reached ();
#line 6827 "TElementTest.c"
	}
	__finally118:
#line 496 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 496 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 496 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 496 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		return;
#line 6838 "TElementTest.c"
	}
}


static void ___lambda95__gtest_func (void) {
#line 495 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	__lambda95_ ();
#line 6846 "TElementTest.c"
}


static void __lambda96_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp0_ = NULL;
		GXmlNode* r = NULL;
		GXmlTDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlTDocument* _tmp3_ = NULL;
		GeeBidirList* _tmp4_ = NULL;
		GeeBidirList* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlTDocument* _tmp8_ = NULL;
		GXmlNode* _tmp9_ = NULL;
		GXmlTDocument* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GXmlElement* c = NULL;
		GXmlNode* _tmp12_ = NULL;
		GXmlNode* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GeeBidirList* _tmp15_ = NULL;
		GeeBidirList* _tmp16_ = NULL;
		GXmlNode* a = NULL;
		GXmlNode* _tmp17_ = NULL;
		GeeList* _tmp18_ = NULL;
		GeeList* _tmp19_ = NULL;
		GeeList* _tmp20_ = NULL;
		gint _tmp21_ = 0;
		gint _tmp22_ = 0;
		GXmlTDocument* _tmp23_ = NULL;
		GeeList* _tmp24_ = NULL;
		GeeList* _tmp25_ = NULL;
		GeeList* _tmp26_ = NULL;
		gint _tmp27_ = 0;
		gint _tmp28_ = 0;
		gchar* str = NULL;
		GXmlTDocument* _tmp29_ = NULL;
		gchar* _tmp30_ = NULL;
		gboolean _tmp31_ = FALSE;
		gboolean _tmp32_ = FALSE;
		gboolean _tmp33_ = FALSE;
#line 533 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp0_ = gxml_tdocument_new ();
#line 533 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		d = _tmp0_;
#line 534 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp1_ = d;
#line 534 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp2_ = gxml_document_create_element ((GXmlDocument*) _tmp1_, "root", &_inner_error_);
#line 534 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		r = _tmp2_;
#line 534 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 534 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 6906 "TElementTest.c"
			goto __catch119_g_error;
		}
#line 535 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp3_ = d;
#line 535 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp4_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp3_);
#line 535 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp5_ = _tmp4_;
#line 535 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp6_ = _tmp5_;
#line 535 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp7_ = r;
#line 535 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp6_, _tmp7_);
#line 535 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp6_);
#line 537 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp8_ = d;
#line 537 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_tnode_set_namespace ((GXmlTNode*) _tmp8_, "http://git.gnome.org/browse/gxml", "gxml");
#line 538 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp10_ = d;
#line 538 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp11_ = gxml_document_create_element ((GXmlDocument*) _tmp10_, "child", &_inner_error_);
#line 538 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp9_ = _tmp11_;
#line 538 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 538 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 538 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 6939 "TElementTest.c"
			goto __catch119_g_error;
		}
#line 538 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp12_ = _tmp9_;
#line 538 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp9_ = NULL;
#line 538 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		c = G_TYPE_CHECK_INSTANCE_CAST (_tmp12_, GXML_TYPE_ELEMENT, GXmlElement);
#line 539 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp13_ = r;
#line 539 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp14_ = gxml_node_get_children_nodes (_tmp13_);
#line 539 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp15_ = _tmp14_;
#line 539 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp16_ = _tmp15_;
#line 539 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp16_, (GXmlNode*) c);
#line 539 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp16_);
#line 540 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_element_set_attr (c, "at", "val");
#line 541 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp17_ = gxml_element_get_attr (c, "at");
#line 541 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		a = _tmp17_;
#line 542 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (a != NULL, "a != null");
#line 546 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (a, "http://git.gnome.org/browse/gxml", "gxml");
#line 547 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp18_ = gxml_node_get_namespaces (a);
#line 547 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp19_ = _tmp18_;
#line 547 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp20_ = _tmp19_;
#line 547 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp21_ = gee_collection_get_size ((GeeCollection*) _tmp20_);
#line 547 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp22_ = _tmp21_;
#line 547 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp22_ == 1, "a.namespaces.size == 1");
#line 547 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp20_);
#line 548 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp23_ = d;
#line 548 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp24_ = gxml_tnode_get_namespaces ((GXmlTNode*) _tmp23_);
#line 548 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp25_ = _tmp24_;
#line 548 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp26_ = _tmp25_;
#line 548 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp27_ = gee_collection_get_size ((GeeCollection*) _tmp26_);
#line 548 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp28_ = _tmp27_;
#line 548 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp28_ == 1, "d.namespaces.size == 1");
#line 548 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp26_);
#line 549 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp29_ = d;
#line 549 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp30_ = gxml_tnode_to_string ((GXmlTNode*) _tmp29_);
#line 549 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		str = _tmp30_;
#line 553 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp31_ = string_contains (str, "<root xmlns:gxml=\"http://git.gnome.org/browse/gxml\">");
#line 553 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp31_, "\"<root xmlns:gxml=\\\"http://git.gnome.org/browse/gxml\\\">\" in str");
#line 554 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp32_ = string_contains (str, "<child gxml:at=\"val\"/>");
#line 554 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp32_, "\"<child gxml:at=\\\"val\\\"/>\" in str");
#line 555 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp33_ = string_contains (str, "</root>");
#line 555 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp33_, "\"</root>\" in str");
#line 532 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (str);
#line 532 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (a);
#line 532 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (c);
#line 532 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp9_);
#line 532 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (r);
#line 532 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (d);
#line 7030 "TElementTest.c"
	}
	goto __finally119;
	__catch119_g_error:
	{
#line 532 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 532 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_inner_error_ = NULL;
#line 556 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_assert_not_reached ();
#line 7041 "TElementTest.c"
	}
	__finally119:
#line 532 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 532 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 532 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 532 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		return;
#line 7052 "TElementTest.c"
	}
}


static void ___lambda96__gtest_func (void) {
#line 531 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	__lambda96_ ();
#line 7060 "TElementTest.c"
}


static void __lambda97_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp0_ = NULL;
		GXmlNode* r = NULL;
		GXmlTDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlTDocument* _tmp3_ = NULL;
		GeeBidirList* _tmp4_ = NULL;
		GeeBidirList* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlNode* _tmp8_ = NULL;
		GXmlNode* _tmp9_ = NULL;
		GXmlNode* _tmp10_ = NULL;
		GXmlTDocument* _tmp11_ = NULL;
		GXmlNode* _tmp12_ = NULL;
		GXmlElement* c = NULL;
		GXmlNode* _tmp13_ = NULL;
		GXmlElement* _tmp14_ = NULL;
		GXmlNode* _tmp15_ = NULL;
		GeeBidirList* _tmp16_ = NULL;
		GeeBidirList* _tmp17_ = NULL;
		GeeBidirList* _tmp18_ = NULL;
		GeeMap* _tmp19_ = NULL;
		GeeMap* _tmp20_ = NULL;
		GeeMap* _tmp21_ = NULL;
		gint _tmp22_ = 0;
		gint _tmp23_ = 0;
		GeeMap* _tmp24_ = NULL;
		GeeMap* _tmp25_ = NULL;
		GeeMap* _tmp26_ = NULL;
		gint _tmp27_ = 0;
		gint _tmp28_ = 0;
		GeeMap* _tmp29_ = NULL;
		GeeMap* _tmp30_ = NULL;
		GeeMap* _tmp31_ = NULL;
		gpointer _tmp32_ = NULL;
		GXmlNode* _tmp33_ = NULL;
		GeeMap* _tmp34_ = NULL;
		GeeMap* _tmp35_ = NULL;
		GeeMap* _tmp36_ = NULL;
		gpointer _tmp37_ = NULL;
		GXmlNode* _tmp38_ = NULL;
		GXmlAttribute* _tmp39_ = NULL;
		GXmlAttribute* _tmp40_ = NULL;
		GXmlNamespace* _tmp41_ = NULL;
		GXmlNamespace* _tmp42_ = NULL;
		GXmlNamespace* _tmp43_ = NULL;
		GeeMap* _tmp44_ = NULL;
		GeeMap* _tmp45_ = NULL;
		GeeMap* _tmp46_ = NULL;
		gpointer _tmp47_ = NULL;
		GXmlNode* _tmp48_ = NULL;
		GXmlAttribute* _tmp49_ = NULL;
		GXmlAttribute* _tmp50_ = NULL;
		GXmlNamespace* _tmp51_ = NULL;
		GXmlNamespace* _tmp52_ = NULL;
		GXmlNamespace* _tmp53_ = NULL;
		gchar* _tmp54_ = NULL;
		gchar* _tmp55_ = NULL;
		gchar* _tmp56_ = NULL;
		GeeMap* _tmp57_ = NULL;
		GeeMap* _tmp58_ = NULL;
		GeeMap* _tmp59_ = NULL;
		gpointer _tmp60_ = NULL;
		GXmlNode* _tmp61_ = NULL;
		GXmlAttribute* _tmp62_ = NULL;
		GXmlAttribute* _tmp63_ = NULL;
		GXmlNamespace* _tmp64_ = NULL;
		GXmlNamespace* _tmp65_ = NULL;
		GXmlNamespace* _tmp66_ = NULL;
		gchar* _tmp67_ = NULL;
		gchar* _tmp68_ = NULL;
		gchar* _tmp69_ = NULL;
		GeeMap* _tmp70_ = NULL;
		GeeMap* _tmp71_ = NULL;
		GeeMap* _tmp72_ = NULL;
		gpointer _tmp73_ = NULL;
		GXmlNode* _tmp74_ = NULL;
		GXmlAttribute* _tmp75_ = NULL;
		GXmlAttribute* _tmp76_ = NULL;
		GXmlNamespace* _tmp77_ = NULL;
		GXmlNamespace* _tmp78_ = NULL;
		GXmlNamespace* _tmp79_ = NULL;
		gchar* _tmp80_ = NULL;
		gchar* _tmp81_ = NULL;
		gchar* _tmp82_ = NULL;
		GXmlNode* nsa = NULL;
		GXmlNode* _tmp83_ = NULL;
		gchar* _tmp84_ = NULL;
		gchar* _tmp85_ = NULL;
		gchar* _tmp86_ = NULL;
		GXmlNode* nsat = NULL;
		GeeMap* _tmp87_ = NULL;
		GeeMap* _tmp88_ = NULL;
		GeeMap* _tmp89_ = NULL;
		gpointer _tmp90_ = NULL;
		GXmlNode* _tmp91_ = NULL;
		gchar* _tmp92_ = NULL;
		gchar* _tmp93_ = NULL;
		gchar* _tmp94_ = NULL;
		GXmlNode* a = NULL;
		GeeMap* _tmp95_ = NULL;
		GeeMap* _tmp96_ = NULL;
		GeeMap* _tmp97_ = NULL;
		gpointer _tmp98_ = NULL;
		GXmlNode* _tmp99_ = NULL;
		gchar* _tmp100_ = NULL;
		gchar* _tmp101_ = NULL;
		gchar* _tmp102_ = NULL;
		GXmlTDocument* _tmp103_ = NULL;
		gchar* _tmp104_ = NULL;
		gchar* _tmp105_ = NULL;
#line 560 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp0_ = gxml_tdocument_new ();
#line 560 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		d = _tmp0_;
#line 561 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp1_ = d;
#line 561 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp2_ = gxml_document_create_element ((GXmlDocument*) _tmp1_, "root", &_inner_error_);
#line 561 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		r = _tmp2_;
#line 561 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 561 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 7193 "TElementTest.c"
			goto __catch120_g_error;
		}
#line 562 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp3_ = d;
#line 562 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp4_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp3_);
#line 562 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp5_ = _tmp4_;
#line 562 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp6_ = _tmp5_;
#line 562 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp7_ = r;
#line 562 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp6_, _tmp7_);
#line 562 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp6_);
#line 564 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_test_message ("Setting namespace gxml");
#line 565 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp8_ = r;
#line 565 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp8_, "http://git.gnome.org/browse/gxml", "gxml");
#line 566 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_test_message ("Setting namespace gxml");
#line 567 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp9_ = r;
#line 567 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_node_set_namespace (_tmp9_, "http://books.net", "book");
#line 568 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp11_ = d;
#line 568 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp12_ = gxml_document_create_element ((GXmlDocument*) _tmp11_, "child", &_inner_error_);
#line 568 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp10_ = _tmp12_;
#line 568 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 568 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (r);
#line 568 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 7234 "TElementTest.c"
			goto __catch120_g_error;
		}
#line 568 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp13_ = _tmp10_;
#line 568 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp10_ = NULL;
#line 568 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp14_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp13_, GXML_TYPE_ELEMENT) ? ((GXmlElement*) _tmp13_) : NULL;
#line 568 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (_tmp14_ == NULL) {
#line 568 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (_tmp13_);
#line 7247 "TElementTest.c"
		}
#line 568 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		c = _tmp14_;
#line 569 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp15_ = r;
#line 569 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp16_ = gxml_node_get_children_nodes (_tmp15_);
#line 569 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp17_ = _tmp16_;
#line 569 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp18_ = _tmp17_;
#line 569 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp18_, (GXmlNode*) c);
#line 569 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp18_);
#line 570 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_element_set_attr (c, "source", "http://books.net/sources/1");
#line 571 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp19_ = gxml_node_get_attrs ((GXmlNode*) c);
#line 571 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp20_ = _tmp19_;
#line 571 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp21_ = _tmp20_;
#line 571 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp22_ = gee_map_get_size (_tmp21_);
#line 571 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp23_ = _tmp22_;
#line 571 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp23_ == 1, "c.attrs.size == 1");
#line 571 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp21_);
#line 572 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_test_message ("Setting attr namespace book");
#line 573 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_element_set_ns_attr (c, "book:http://books.net", "source", "The History 2");
#line 574 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp24_ = gxml_node_get_attrs ((GXmlNode*) c);
#line 574 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp25_ = _tmp24_;
#line 574 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp26_ = _tmp25_;
#line 574 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp27_ = gee_map_get_size (_tmp26_);
#line 574 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp28_ = _tmp27_;
#line 574 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp28_ == 2, "c.attrs.size == 2");
#line 574 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp26_);
#line 575 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp29_ = gxml_node_get_attrs ((GXmlNode*) c);
#line 575 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp30_ = _tmp29_;
#line 575 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp31_ = _tmp30_;
#line 575 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp32_ = gee_map_get (_tmp31_, "book:source");
#line 575 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp33_ = (GXmlNode*) _tmp32_;
#line 575 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp33_ != NULL, "c.attrs.get (\"book:source\") != null");
#line 575 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp33_);
#line 575 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp31_);
#line 576 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp34_ = gxml_node_get_attrs ((GXmlNode*) c);
#line 576 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp35_ = _tmp34_;
#line 576 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp36_ = _tmp35_;
#line 576 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp37_ = gee_map_get (_tmp36_, "book:source");
#line 576 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp38_ = (GXmlNode*) _tmp37_;
#line 576 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp39_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp38_, GXML_TYPE_ATTRIBUTE) ? ((GXmlAttribute*) _tmp38_) : NULL;
#line 576 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (_tmp39_ == NULL) {
#line 576 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (_tmp38_);
#line 7329 "TElementTest.c"
		}
#line 576 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp40_ = _tmp39_;
#line 576 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp41_ = gxml_attribute_get_namespace (_tmp40_);
#line 576 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp42_ = _tmp41_;
#line 576 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp43_ = _tmp42_;
#line 576 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp43_ != NULL, "(c.attrs.get (\"book:source\") as Attribute).namespace != null");
#line 576 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp43_);
#line 576 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp40_);
#line 576 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp36_);
#line 577 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp44_ = gxml_node_get_attrs ((GXmlNode*) c);
#line 577 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp45_ = _tmp44_;
#line 577 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp46_ = _tmp45_;
#line 577 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp47_ = gee_map_get (_tmp46_, "book:source");
#line 577 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp48_ = (GXmlNode*) _tmp47_;
#line 577 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp49_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp48_, GXML_TYPE_ATTRIBUTE) ? ((GXmlAttribute*) _tmp48_) : NULL;
#line 577 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (_tmp49_ == NULL) {
#line 577 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (_tmp48_);
#line 7363 "TElementTest.c"
		}
#line 577 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp50_ = _tmp49_;
#line 577 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp51_ = gxml_attribute_get_namespace (_tmp50_);
#line 577 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp52_ = _tmp51_;
#line 577 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp53_ = _tmp52_;
#line 577 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp54_ = gxml_namespace_get_prefix (_tmp53_);
#line 577 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp55_ = _tmp54_;
#line 577 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp56_ = _tmp55_;
#line 577 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp56_, "book") == 0, "(c.attrs.get (\"book:source\") as Attribute).namespace.prefix == \"book\"");
#line 577 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp56_);
#line 577 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp53_);
#line 577 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp50_);
#line 577 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp46_);
#line 578 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp57_ = gxml_node_get_attrs ((GXmlNode*) c);
#line 578 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp58_ = _tmp57_;
#line 578 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp59_ = _tmp58_;
#line 578 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp60_ = gee_map_get (_tmp59_, "book:source");
#line 578 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp61_ = (GXmlNode*) _tmp60_;
#line 578 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp62_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp61_, GXML_TYPE_ATTRIBUTE) ? ((GXmlAttribute*) _tmp61_) : NULL;
#line 578 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (_tmp62_ == NULL) {
#line 578 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (_tmp61_);
#line 7405 "TElementTest.c"
		}
#line 578 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp63_ = _tmp62_;
#line 578 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp64_ = gxml_attribute_get_namespace (_tmp63_);
#line 578 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp65_ = _tmp64_;
#line 578 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp66_ = _tmp65_;
#line 578 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp67_ = gxml_namespace_get_uri (_tmp66_);
#line 578 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp68_ = _tmp67_;
#line 578 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp69_ = _tmp68_;
#line 578 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_test_message ("%s", _tmp69_);
#line 578 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp69_);
#line 578 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp66_);
#line 578 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp63_);
#line 578 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp59_);
#line 579 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp70_ = gxml_node_get_attrs ((GXmlNode*) c);
#line 579 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp71_ = _tmp70_;
#line 579 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp72_ = _tmp71_;
#line 579 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp73_ = gee_map_get (_tmp72_, "book:source");
#line 579 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp74_ = (GXmlNode*) _tmp73_;
#line 579 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp75_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp74_, GXML_TYPE_ATTRIBUTE) ? ((GXmlAttribute*) _tmp74_) : NULL;
#line 579 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (_tmp75_ == NULL) {
#line 579 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (_tmp74_);
#line 7447 "TElementTest.c"
		}
#line 579 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp76_ = _tmp75_;
#line 579 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp77_ = gxml_attribute_get_namespace (_tmp76_);
#line 579 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp78_ = _tmp77_;
#line 579 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp79_ = _tmp78_;
#line 579 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp80_ = gxml_namespace_get_uri (_tmp79_);
#line 579 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp81_ = _tmp80_;
#line 579 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp82_ = _tmp81_;
#line 579 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp82_, "http://books.net") == 0, "(c.attrs.get (\"book:source\") as Attribute).namespace.uri == \"http://books.net\"");
#line 579 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp82_);
#line 579 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp79_);
#line 579 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp76_);
#line 579 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp72_);
#line 580 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_test_message ("Searching attr source with namespace book");
#line 581 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp83_ = gxml_element_get_ns_attr (c, "source", "http://books.net");
#line 581 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		nsa = _tmp83_;
#line 582 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (nsa != NULL, "nsa != null");
#line 583 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp84_ = gxml_node_get_value (nsa);
#line 583 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp85_ = _tmp84_;
#line 583 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp86_ = _tmp85_;
#line 583 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp86_, "The History 2") == 0, "nsa.value == \"The History 2\"");
#line 583 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp86_);
#line 584 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp87_ = gxml_node_get_attrs ((GXmlNode*) c);
#line 584 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp88_ = _tmp87_;
#line 584 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp89_ = _tmp88_;
#line 584 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp90_ = gee_map_get (_tmp89_, "book:source");
#line 584 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp91_ = (GXmlNode*) _tmp90_;
#line 584 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp89_);
#line 584 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		nsat = _tmp91_;
#line 585 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (nsat != NULL, "nsat != null");
#line 586 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp92_ = gxml_node_get_value (nsat);
#line 586 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp93_ = _tmp92_;
#line 586 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp94_ = _tmp93_;
#line 586 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp94_, "The History 2") == 0, "nsat.value == \"The History 2\"");
#line 586 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp94_);
#line 587 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp95_ = gxml_node_get_attrs ((GXmlNode*) c);
#line 587 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp96_ = _tmp95_;
#line 587 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp97_ = _tmp96_;
#line 587 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp98_ = gee_map_get (_tmp97_, "source");
#line 587 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp99_ = (GXmlNode*) _tmp98_;
#line 587 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp97_);
#line 587 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		a = _tmp99_;
#line 588 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (a != NULL, "a != null");
#line 589 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp100_ = gxml_node_get_value (a);
#line 589 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp101_ = _tmp100_;
#line 589 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp102_ = _tmp101_;
#line 589 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp102_, "http://books.net/sources/1") == 0, "a.value == \"http://books.net/sources/1\"");
#line 589 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp102_);
#line 590 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp103_ = d;
#line 590 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp104_ = gxml_tnode_to_string ((GXmlTNode*) _tmp103_);
#line 590 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp105_ = _tmp104_;
#line 590 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_test_message ("%s", _tmp105_);
#line 590 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp105_);
#line 559 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (a);
#line 559 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (nsat);
#line 559 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (nsa);
#line 559 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (c);
#line 559 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp10_);
#line 559 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (r);
#line 559 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (d);
#line 7567 "TElementTest.c"
	}
	goto __finally120;
	__catch120_g_error:
	{
#line 559 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 559 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_inner_error_ = NULL;
#line 591 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_assert_not_reached ();
#line 7578 "TElementTest.c"
	}
	__finally120:
#line 559 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 559 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 559 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 559 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		return;
#line 7589 "TElementTest.c"
	}
}


static void ___lambda97__gtest_func (void) {
#line 558 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	__lambda97_ ();
#line 7597 "TElementTest.c"
}


static void __lambda98_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlTDocument* doc = NULL;
		GXmlTDocument* _tmp0_ = NULL;
		GXmlNode* e = NULL;
		GXmlTDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlTDocument* _tmp3_ = NULL;
		GeeBidirList* _tmp4_ = NULL;
		GeeBidirList* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlNode* c = NULL;
		GXmlTDocument* _tmp8_ = NULL;
		GXmlNode* _tmp9_ = NULL;
		GXmlNode* _tmp10_ = NULL;
		GeeBidirList* _tmp11_ = NULL;
		GeeBidirList* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GXmlNode* _tmp14_ = NULL;
		GXmlNode* _tmp15_ = NULL;
		GeeBidirList* _tmp16_ = NULL;
		GeeBidirList* _tmp17_ = NULL;
		GeeBidirList* _tmp18_ = NULL;
		gint _tmp19_ = 0;
		gint _tmp20_ = 0;
		GXmlNode* _tmp21_ = NULL;
		GeeBidirList* _tmp22_ = NULL;
		GeeBidirList* _tmp23_ = NULL;
		GeeBidirList* _tmp24_ = NULL;
		gpointer _tmp25_ = NULL;
		GXmlNode* _tmp26_ = NULL;
		GXmlNode* _tmp27_ = NULL;
		GeeBidirList* _tmp28_ = NULL;
		GeeBidirList* _tmp29_ = NULL;
		GeeBidirList* _tmp30_ = NULL;
		gpointer _tmp31_ = NULL;
		GXmlNode* _tmp32_ = NULL;
		gchar* _tmp33_ = NULL;
		gchar* _tmp34_ = NULL;
		gchar* _tmp35_ = NULL;
		GXmlNode* _tmp36_ = NULL;
		GXmlNode* _tmp37_ = NULL;
		GXmlNode* _tmp38_ = NULL;
		GXmlNode* _tmp39_ = NULL;
		GXmlTDocument* _tmp40_ = NULL;
		GXmlNode* _tmp41_ = NULL;
		GXmlNode* _tmp42_ = NULL;
		GXmlNode* _tmp43_ = NULL;
		GXmlTDocument* _tmp44_ = NULL;
		GXmlNode* _tmp45_ = NULL;
		GXmlNode* _tmp46_ = NULL;
		GXmlNode* _tmp47_ = NULL;
		GeeBidirList* _tmp48_ = NULL;
		GeeBidirList* _tmp49_ = NULL;
		GeeBidirList* _tmp50_ = NULL;
		gpointer _tmp51_ = NULL;
		GXmlNode* _tmp52_ = NULL;
		GXmlTDocument* _tmp53_ = NULL;
		GXmlNode* _tmp54_ = NULL;
		GXmlNode* _tmp55_ = NULL;
		GXmlNode* _tmp56_ = NULL;
		GeeBidirList* _tmp57_ = NULL;
		GeeBidirList* _tmp58_ = NULL;
		GeeBidirList* _tmp59_ = NULL;
		gpointer _tmp60_ = NULL;
		GXmlNode* _tmp61_ = NULL;
		gchar* _tmp62_ = NULL;
		gchar* _tmp63_ = NULL;
		gchar* _tmp64_ = NULL;
		GXmlTDocument* _tmp65_ = NULL;
		GXmlNode* _tmp66_ = NULL;
		GXmlNode* _tmp67_ = NULL;
		GXmlNode* _tmp68_ = NULL;
		GeeBidirList* _tmp69_ = NULL;
		GeeBidirList* _tmp70_ = NULL;
		GeeBidirList* _tmp71_ = NULL;
		gpointer _tmp72_ = NULL;
		GXmlNode* _tmp73_ = NULL;
		GXmlNode* _tmp74_ = NULL;
		GXmlNode* _tmp75_ = NULL;
		GXmlNode* _tmp76_ = NULL;
		GXmlTDocument* _tmp77_ = NULL;
		GXmlNode* _tmp78_ = NULL;
		GXmlNode* _tmp79_ = NULL;
		GXmlNode* _tmp80_ = NULL;
		GeeBidirList* _tmp81_ = NULL;
		GeeBidirList* _tmp82_ = NULL;
		GeeBidirList* _tmp83_ = NULL;
		gpointer _tmp84_ = NULL;
		GXmlNode* _tmp85_ = NULL;
		GXmlNode* _tmp86_ = NULL;
		GXmlNode* _tmp87_ = NULL;
		GXmlNode* _tmp88_ = NULL;
		gchar* _tmp89_ = NULL;
		gchar* _tmp90_ = NULL;
		gchar* _tmp91_ = NULL;
		GXmlTDocument* _tmp92_ = NULL;
		GXmlNode* _tmp93_ = NULL;
		GXmlNode* _tmp94_ = NULL;
		GXmlNode* _tmp95_ = NULL;
		GXmlNode* _tmp96_ = NULL;
		GXmlNode* _tmp97_ = NULL;
		GXmlNode* _tmp98_ = NULL;
#line 595 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp0_ = gxml_tdocument_new ();
#line 595 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		doc = _tmp0_;
#line 596 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp1_ = doc;
#line 596 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp2_ = gxml_document_create_element ((GXmlDocument*) _tmp1_, "root", &_inner_error_);
#line 596 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e = _tmp2_;
#line 596 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 596 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (doc);
#line 7720 "TElementTest.c"
			goto __catch121_g_error;
		}
#line 597 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp3_ = doc;
#line 597 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp4_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp3_);
#line 597 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp5_ = _tmp4_;
#line 597 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp6_ = _tmp5_;
#line 597 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp7_ = e;
#line 597 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp6_, _tmp7_);
#line 597 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp6_);
#line 598 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp8_ = doc;
#line 598 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp9_ = gxml_document_create_element ((GXmlDocument*) _tmp8_, "child", &_inner_error_);
#line 598 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		c = _tmp9_;
#line 598 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 598 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 598 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (doc);
#line 7749 "TElementTest.c"
			goto __catch121_g_error;
		}
#line 599 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp10_ = e;
#line 599 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp11_ = gxml_node_get_children_nodes (_tmp10_);
#line 599 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp12_ = _tmp11_;
#line 599 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp13_ = _tmp12_;
#line 599 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp14_ = c;
#line 599 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp13_, _tmp14_);
#line 599 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp13_);
#line 600 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp15_ = e;
#line 600 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp16_ = gxml_node_get_children_nodes (_tmp15_);
#line 600 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp17_ = _tmp16_;
#line 600 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp18_ = _tmp17_;
#line 600 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp19_ = gee_collection_get_size ((GeeCollection*) _tmp18_);
#line 600 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp20_ = _tmp19_;
#line 600 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp20_ == 1, "e.children_nodes.size == 1");
#line 600 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp18_);
#line 601 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp21_ = e;
#line 601 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp22_ = gxml_node_get_children_nodes (_tmp21_);
#line 601 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp23_ = _tmp22_;
#line 601 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp24_ = _tmp23_;
#line 601 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp25_ = gee_list_get ((GeeList*) _tmp24_, 0);
#line 601 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp26_ = (GXmlNode*) _tmp25_;
#line 601 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp26_ != NULL, "e.children_nodes[0] != null");
#line 601 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp26_);
#line 601 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp24_);
#line 602 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp27_ = e;
#line 602 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp28_ = gxml_node_get_children_nodes (_tmp27_);
#line 602 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp29_ = _tmp28_;
#line 602 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp30_ = _tmp29_;
#line 602 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp31_ = gee_list_get ((GeeList*) _tmp30_, 0);
#line 602 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp32_ = (GXmlNode*) _tmp31_;
#line 602 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp33_ = gxml_node_get_name (_tmp32_);
#line 602 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp34_ = _tmp33_;
#line 602 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp35_ = _tmp34_;
#line 602 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp35_, "child") == 0, "e.children_nodes[0].name == \"child\"");
#line 602 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp35_);
#line 602 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp32_);
#line 602 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp30_);
#line 603 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp36_ = c;
#line 603 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp37_ = gxml_node_get_parent (_tmp36_);
#line 603 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp38_ = _tmp37_;
#line 603 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp39_ = _tmp38_;
#line 603 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp39_ != NULL, "c.parent != null");
#line 603 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp39_);
#line 604 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp40_ = doc;
#line 604 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp41_ = gxml_document_get_root ((GXmlDocument*) _tmp40_);
#line 604 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp42_ = _tmp41_;
#line 604 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp43_ = _tmp42_;
#line 604 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp43_ != NULL, "doc.root != null");
#line 604 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp43_);
#line 605 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp44_ = doc;
#line 605 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp45_ = gxml_document_get_root ((GXmlDocument*) _tmp44_);
#line 605 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp46_ = _tmp45_;
#line 605 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp47_ = _tmp46_;
#line 605 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp48_ = gxml_node_get_children_nodes (_tmp47_);
#line 605 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp49_ = _tmp48_;
#line 605 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp50_ = _tmp49_;
#line 605 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp51_ = gee_list_get ((GeeList*) _tmp50_, 0);
#line 605 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp52_ = (GXmlNode*) _tmp51_;
#line 605 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp52_ != NULL, "doc.root.children_nodes[0] != null");
#line 605 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp52_);
#line 605 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp50_);
#line 605 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp47_);
#line 606 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp53_ = doc;
#line 606 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp54_ = gxml_document_get_root ((GXmlDocument*) _tmp53_);
#line 606 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp55_ = _tmp54_;
#line 606 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp56_ = _tmp55_;
#line 606 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp57_ = gxml_node_get_children_nodes (_tmp56_);
#line 606 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp58_ = _tmp57_;
#line 606 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp59_ = _tmp58_;
#line 606 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp60_ = gee_list_get ((GeeList*) _tmp59_, 0);
#line 606 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp61_ = (GXmlNode*) _tmp60_;
#line 606 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp62_ = gxml_node_get_name (_tmp61_);
#line 606 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp63_ = _tmp62_;
#line 606 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp64_ = _tmp63_;
#line 606 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp64_, "child") == 0, "doc.root.children_nodes[0].name == \"child\"");
#line 606 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp64_);
#line 606 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp61_);
#line 606 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp59_);
#line 606 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp56_);
#line 607 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp65_ = doc;
#line 607 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp66_ = gxml_document_get_root ((GXmlDocument*) _tmp65_);
#line 607 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp67_ = _tmp66_;
#line 607 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp68_ = _tmp67_;
#line 607 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp69_ = gxml_node_get_children_nodes (_tmp68_);
#line 607 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp70_ = _tmp69_;
#line 607 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp71_ = _tmp70_;
#line 607 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp72_ = gee_list_get ((GeeList*) _tmp71_, 0);
#line 607 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp73_ = (GXmlNode*) _tmp72_;
#line 607 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp74_ = gxml_node_get_parent (_tmp73_);
#line 607 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp75_ = _tmp74_;
#line 607 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp76_ = _tmp75_;
#line 607 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp76_ != NULL, "doc.root.children_nodes[0].parent != null");
#line 607 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp76_);
#line 607 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp73_);
#line 607 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp71_);
#line 607 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp68_);
#line 608 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp77_ = doc;
#line 608 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp78_ = gxml_document_get_root ((GXmlDocument*) _tmp77_);
#line 608 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp79_ = _tmp78_;
#line 608 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp80_ = _tmp79_;
#line 608 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp81_ = gxml_node_get_children_nodes (_tmp80_);
#line 608 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp82_ = _tmp81_;
#line 608 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp83_ = _tmp82_;
#line 608 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp84_ = gee_list_get ((GeeList*) _tmp83_, 0);
#line 608 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp85_ = (GXmlNode*) _tmp84_;
#line 608 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp86_ = gxml_node_get_parent (_tmp85_);
#line 608 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp87_ = _tmp86_;
#line 608 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp88_ = _tmp87_;
#line 608 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp89_ = gxml_node_get_name (_tmp88_);
#line 608 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp90_ = _tmp89_;
#line 608 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp91_ = _tmp90_;
#line 608 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp91_, "root") == 0, "doc.root.children_nodes[0].parent.name == \"root\"");
#line 608 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp91_);
#line 608 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp88_);
#line 608 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp85_);
#line 608 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp83_);
#line 608 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp80_);
#line 609 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp92_ = doc;
#line 609 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp93_ = gxml_document_get_root ((GXmlDocument*) _tmp92_);
#line 609 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp94_ = _tmp93_;
#line 609 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp95_ = _tmp94_;
#line 609 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp96_ = gxml_node_get_parent (_tmp95_);
#line 609 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp97_ = _tmp96_;
#line 609 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp98_ = _tmp97_;
#line 609 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp98_ == NULL, "doc.root.parent == null");
#line 609 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp98_);
#line 609 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp95_);
#line 594 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (c);
#line 594 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e);
#line 594 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (doc);
#line 8012 "TElementTest.c"
	}
	goto __finally121;
	__catch121_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp99_ = NULL;
		gchar* _tmp100_ = NULL;
		gchar* _tmp101_ = NULL;
#line 594 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e = _inner_error_;
#line 594 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_inner_error_ = NULL;
#line 611 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp99_ = e->message;
#line 611 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp100_ = g_strconcat ("Error: ", _tmp99_, NULL);
#line 611 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp101_ = _tmp100_;
#line 611 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_message ("TElementTest.vala:611: %s", _tmp101_);
#line 611 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp101_);
#line 612 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_assert_not_reached ();
#line 594 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_error_free0 (e);
#line 8039 "TElementTest.c"
	}
	__finally121:
#line 594 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 594 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 594 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 594 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		return;
#line 8050 "TElementTest.c"
	}
}


static void ___lambda98__gtest_func (void) {
#line 593 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	__lambda98_ ();
#line 8058 "TElementTest.c"
}


static void __lambda99_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlTDocument* doc = NULL;
		GXmlTDocument* _tmp0_ = NULL;
		GXmlNode* e = NULL;
		GXmlTDocument* _tmp1_ = NULL;
		GXmlNode* _tmp2_ = NULL;
		GXmlTDocument* _tmp3_ = NULL;
		GeeBidirList* _tmp4_ = NULL;
		GeeBidirList* _tmp5_ = NULL;
		GeeBidirList* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlNode* c = NULL;
		GXmlTDocument* _tmp8_ = NULL;
		GXmlNode* _tmp9_ = NULL;
		GXmlNode* _tmp10_ = NULL;
		GeeBidirList* _tmp11_ = NULL;
		GeeBidirList* _tmp12_ = NULL;
		GeeBidirList* _tmp13_ = NULL;
		GXmlNode* _tmp14_ = NULL;
		GXmlNode* _tmp15_ = NULL;
		GXmlTDocument* _tmp16_ = NULL;
		GXmlNode* _tmp17_ = NULL;
		GXmlNode* _tmp18_ = NULL;
		GXmlNode* _tmp19_ = NULL;
		GXmlTDocument* _tmp20_ = NULL;
		GXmlNode* _tmp21_ = NULL;
		GXmlNode* _tmp22_ = NULL;
		GXmlNode* _tmp23_ = NULL;
		GeeMap* _tmp24_ = NULL;
		GeeMap* _tmp25_ = NULL;
		GeeMap* _tmp26_ = NULL;
		gpointer _tmp27_ = NULL;
		GXmlNode* _tmp28_ = NULL;
		GXmlTDocument* _tmp29_ = NULL;
		GXmlNode* _tmp30_ = NULL;
		GXmlNode* _tmp31_ = NULL;
		GXmlNode* _tmp32_ = NULL;
		GeeMap* _tmp33_ = NULL;
		GeeMap* _tmp34_ = NULL;
		GeeMap* _tmp35_ = NULL;
		gpointer _tmp36_ = NULL;
		GXmlNode* _tmp37_ = NULL;
		GXmlNode* _tmp38_ = NULL;
		GXmlNode* _tmp39_ = NULL;
		GXmlNode* _tmp40_ = NULL;
		GXmlTDocument* _tmp41_ = NULL;
		GXmlNode* _tmp42_ = NULL;
		GXmlNode* _tmp43_ = NULL;
		GXmlNode* _tmp44_ = NULL;
		GeeMap* _tmp45_ = NULL;
		GeeMap* _tmp46_ = NULL;
		GeeMap* _tmp47_ = NULL;
		gpointer _tmp48_ = NULL;
		GXmlNode* _tmp49_ = NULL;
		GXmlNode* _tmp50_ = NULL;
		GXmlNode* _tmp51_ = NULL;
		GXmlNode* _tmp52_ = NULL;
		gchar* _tmp53_ = NULL;
		gchar* _tmp54_ = NULL;
		gchar* _tmp55_ = NULL;
#line 617 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp0_ = gxml_tdocument_new ();
#line 617 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		doc = _tmp0_;
#line 618 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp1_ = doc;
#line 618 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp2_ = gxml_document_create_element ((GXmlDocument*) _tmp1_, "root", &_inner_error_);
#line 618 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e = _tmp2_;
#line 618 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 618 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (doc);
#line 8138 "TElementTest.c"
			goto __catch122_g_error;
		}
#line 619 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp3_ = doc;
#line 619 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp4_ = gxml_tnode_get_children_nodes ((GXmlTNode*) _tmp3_);
#line 619 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp5_ = _tmp4_;
#line 619 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp6_ = _tmp5_;
#line 619 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp7_ = e;
#line 619 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp6_, _tmp7_);
#line 619 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp6_);
#line 620 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp8_ = doc;
#line 620 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp9_ = gxml_document_create_element ((GXmlDocument*) _tmp8_, "child", &_inner_error_);
#line 620 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		c = _tmp9_;
#line 620 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 620 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (e);
#line 620 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (doc);
#line 8167 "TElementTest.c"
			goto __catch122_g_error;
		}
#line 621 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp10_ = e;
#line 621 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp11_ = gxml_node_get_children_nodes (_tmp10_);
#line 621 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp12_ = _tmp11_;
#line 621 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp13_ = _tmp12_;
#line 621 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp14_ = c;
#line 621 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gee_collection_add ((GeeCollection*) _tmp13_, _tmp14_);
#line 621 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp13_);
#line 622 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp15_ = e;
#line 622 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_element_set_attr (G_TYPE_CHECK_INSTANCE_TYPE (_tmp15_, GXML_TYPE_ELEMENT) ? ((GXmlElement*) _tmp15_) : NULL, "attr", "val");
#line 623 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp16_ = doc;
#line 623 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp17_ = gxml_document_get_root ((GXmlDocument*) _tmp16_);
#line 623 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp18_ = _tmp17_;
#line 623 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp19_ = _tmp18_;
#line 623 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp19_ != NULL, "doc.root != null");
#line 623 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp19_);
#line 624 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp20_ = doc;
#line 624 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp21_ = gxml_document_get_root ((GXmlDocument*) _tmp20_);
#line 624 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp22_ = _tmp21_;
#line 624 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp23_ = _tmp22_;
#line 624 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp24_ = gxml_node_get_attrs (_tmp23_);
#line 624 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp25_ = _tmp24_;
#line 624 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp26_ = _tmp25_;
#line 624 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp27_ = gee_map_get (_tmp26_, "attr");
#line 624 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp28_ = (GXmlNode*) _tmp27_;
#line 624 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp28_ != NULL, "doc.root.attrs[\"attr\"] != null");
#line 624 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp28_);
#line 624 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp26_);
#line 624 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp23_);
#line 625 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp29_ = doc;
#line 625 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp30_ = gxml_document_get_root ((GXmlDocument*) _tmp29_);
#line 625 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp31_ = _tmp30_;
#line 625 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp32_ = _tmp31_;
#line 625 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp33_ = gxml_node_get_attrs (_tmp32_);
#line 625 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp34_ = _tmp33_;
#line 625 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp35_ = _tmp34_;
#line 625 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp36_ = gee_map_get (_tmp35_, "attr");
#line 625 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp37_ = (GXmlNode*) _tmp36_;
#line 625 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp38_ = gxml_node_get_parent (_tmp37_);
#line 625 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp39_ = _tmp38_;
#line 625 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp40_ = _tmp39_;
#line 625 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp40_ != NULL, "doc.root.attrs[\"attr\"].parent != null");
#line 625 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp40_);
#line 625 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp37_);
#line 625 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp35_);
#line 625 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp32_);
#line 626 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp41_ = doc;
#line 626 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp42_ = gxml_document_get_root ((GXmlDocument*) _tmp41_);
#line 626 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp43_ = _tmp42_;
#line 626 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp44_ = _tmp43_;
#line 626 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp45_ = gxml_node_get_attrs (_tmp44_);
#line 626 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp46_ = _tmp45_;
#line 626 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp47_ = _tmp46_;
#line 626 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp48_ = gee_map_get (_tmp47_, "attr");
#line 626 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp49_ = (GXmlNode*) _tmp48_;
#line 626 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp50_ = gxml_node_get_parent (_tmp49_);
#line 626 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp51_ = _tmp50_;
#line 626 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp52_ = _tmp51_;
#line 626 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp53_ = gxml_node_get_name (_tmp52_);
#line 626 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp54_ = _tmp53_;
#line 626 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp55_ = _tmp54_;
#line 626 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp55_, "root") == 0, "doc.root.attrs[\"attr\"].parent.name == \"root\"");
#line 626 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp55_);
#line 626 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp52_);
#line 626 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp49_);
#line 626 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp47_);
#line 626 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp44_);
#line 616 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (c);
#line 616 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (e);
#line 616 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (doc);
#line 8308 "TElementTest.c"
	}
	goto __finally122;
	__catch122_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp56_ = NULL;
		gchar* _tmp57_ = NULL;
		gchar* _tmp58_ = NULL;
#line 616 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e = _inner_error_;
#line 616 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_inner_error_ = NULL;
#line 628 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp56_ = e->message;
#line 628 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp57_ = g_strconcat ("Error: ", _tmp56_, NULL);
#line 628 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp58_ = _tmp57_;
#line 628 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_message ("TElementTest.vala:628: %s", _tmp58_);
#line 628 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp58_);
#line 629 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_assert_not_reached ();
#line 616 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_error_free0 (e);
#line 8335 "TElementTest.c"
	}
	__finally122:
#line 616 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 616 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 616 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 616 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		return;
#line 8346 "TElementTest.c"
	}
}


static void ___lambda99__gtest_func (void) {
#line 615 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	__lambda99_ ();
#line 8354 "TElementTest.c"
}


static GXmlTDocumentReadType ___lambda101_ (GXmlNode* node, xmlTextReader* tr) {
	GXmlTDocumentReadType result = 0;
	GXmlNode* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	gchar* _tmp2_ = NULL;
	gchar* _tmp3_ = NULL;
	gchar* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	gboolean _tmp6_ = FALSE;
	GXmlNode* _tmp7_ = NULL;
	gchar* _tmp8_ = NULL;
	gchar* _tmp9_ = NULL;
	gchar* _tmp10_ = NULL;
	gboolean _tmp11_ = FALSE;
#line 650 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_return_val_if_fail (node != NULL, 0);
#line 650 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_return_val_if_fail (tr != NULL, 0);
#line 651 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp0_ = node;
#line 651 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp1_ = gxml_node_get_name (_tmp0_);
#line 651 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp2_ = _tmp1_;
#line 651 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp3_ = _tmp2_;
#line 651 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp4_ = g_strconcat ("ReadType check node: ", _tmp3_, NULL);
#line 651 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp5_ = _tmp4_;
#line 651 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_test_message ("%s", _tmp5_);
#line 651 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_g_free0 (_tmp5_);
#line 651 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_g_free0 (_tmp3_);
#line 652 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp7_ = node;
#line 652 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp8_ = gxml_node_get_name (_tmp7_);
#line 652 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp9_ = _tmp8_;
#line 652 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp10_ = _tmp9_;
#line 652 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp11_ = g_strcmp0 (_tmp10_, "NoRead") == 0;
#line 652 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_g_free0 (_tmp10_);
#line 652 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (_tmp11_) {
#line 652 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp6_ = TRUE;
#line 8410 "TElementTest.c"
	} else {
		GXmlNode* _tmp12_ = NULL;
		gchar* _tmp13_ = NULL;
		gchar* _tmp14_ = NULL;
		gchar* _tmp15_ = NULL;
#line 652 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp12_ = node;
#line 652 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp13_ = gxml_node_get_name (_tmp12_);
#line 652 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp14_ = _tmp13_;
#line 652 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp15_ = _tmp14_;
#line 652 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp6_ = g_strcmp0 (_tmp15_, "NoReadChild") == 0;
#line 652 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp15_);
#line 8428 "TElementTest.c"
	}
#line 652 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (_tmp6_) {
#line 8432 "TElementTest.c"
		GXmlNode* _tmp16_ = NULL;
		gchar* _tmp17_ = NULL;
		gchar* _tmp18_ = NULL;
		gchar* _tmp19_ = NULL;
		gchar* _tmp20_ = NULL;
		gchar* _tmp21_ = NULL;
#line 653 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp16_ = node;
#line 653 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp17_ = gxml_node_get_name (_tmp16_);
#line 653 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp18_ = _tmp17_;
#line 653 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp19_ = _tmp18_;
#line 653 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp20_ = g_strconcat ("Skiping node: ", _tmp19_, NULL);
#line 653 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp21_ = _tmp20_;
#line 653 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_test_message ("%s", _tmp21_);
#line 653 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp21_);
#line 653 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp19_);
#line 654 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		result = GXML_TDOCUMENT_READ_TYPE_NEXT;
#line 654 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		return result;
#line 8461 "TElementTest.c"
	}
#line 656 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	result = GXML_TDOCUMENT_READ_TYPE_CONTINUE;
#line 656 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	return result;
#line 8467 "TElementTest.c"
}


static GXmlTDocumentReadType ____lambda101__gxml_tdocument_read_type_func (GXmlNode* node, xmlTextReader* tr, gpointer self) {
	GXmlTDocumentReadType result;
	result = ___lambda101_ (node, tr);
#line 650 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	return result;
#line 8476 "TElementTest.c"
}


static GXmlTDocumentReadType ___lambda102_ (GXmlNode* node, xmlTextReader* tr) {
	GXmlTDocumentReadType result = 0;
	GXmlNode* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
	gchar* _tmp2_ = NULL;
	gchar* _tmp3_ = NULL;
	gchar* _tmp4_ = NULL;
	gchar* _tmp5_ = NULL;
	GXmlNode* _tmp6_ = NULL;
	gchar* _tmp7_ = NULL;
	gchar* _tmp8_ = NULL;
	gchar* _tmp9_ = NULL;
	gboolean _tmp10_ = FALSE;
#line 676 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_return_val_if_fail (node != NULL, 0);
#line 676 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_return_val_if_fail (tr != NULL, 0);
#line 677 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp0_ = node;
#line 677 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp1_ = gxml_node_get_name (_tmp0_);
#line 677 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp2_ = _tmp1_;
#line 677 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp3_ = _tmp2_;
#line 677 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp4_ = g_strconcat ("ReadType check node: ", _tmp3_, NULL);
#line 677 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp5_ = _tmp4_;
#line 677 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_test_message ("%s", _tmp5_);
#line 677 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_g_free0 (_tmp5_);
#line 677 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_g_free0 (_tmp3_);
#line 678 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp6_ = node;
#line 678 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp7_ = gxml_node_get_name (_tmp6_);
#line 678 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp8_ = _tmp7_;
#line 678 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp9_ = _tmp8_;
#line 678 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_tmp10_ = g_strcmp0 (_tmp9_, "NoReadChild") == 0;
#line 678 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	_g_free0 (_tmp9_);
#line 678 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (_tmp10_) {
#line 8529 "TElementTest.c"
		GXmlNode* _tmp11_ = NULL;
		gchar* _tmp12_ = NULL;
		gchar* _tmp13_ = NULL;
		gchar* _tmp14_ = NULL;
		gchar* _tmp15_ = NULL;
		gchar* _tmp16_ = NULL;
#line 679 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp11_ = node;
#line 679 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp12_ = gxml_node_get_name (_tmp11_);
#line 679 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp13_ = _tmp12_;
#line 679 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp14_ = _tmp13_;
#line 679 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp15_ = g_strconcat ("Stoping on node: ", _tmp14_, NULL);
#line 679 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp16_ = _tmp15_;
#line 679 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_test_message ("%s", _tmp16_);
#line 679 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp16_);
#line 679 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp14_);
#line 680 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		result = GXML_TDOCUMENT_READ_TYPE_STOP;
#line 680 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		return result;
#line 8558 "TElementTest.c"
	}
#line 682 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	result = GXML_TDOCUMENT_READ_TYPE_CONTINUE;
#line 682 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	return result;
#line 8564 "TElementTest.c"
}


static GXmlTDocumentReadType ____lambda102__gxml_tdocument_read_type_func (GXmlNode* node, xmlTextReader* tr, gpointer self) {
	GXmlTDocumentReadType result;
	result = ___lambda102_ (node, tr);
#line 676 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	return result;
#line 8573 "TElementTest.c"
}


static void __lambda100_ (void) {
	GError * _inner_error_ = NULL;
	{
		GFile* file = NULL;
		GFile* _tmp0_ = NULL;
		GFile* _tmp1_ = NULL;
		gboolean _tmp2_ = FALSE;
		GXmlTDocument* d = NULL;
		GXmlTDocument* _tmp3_ = NULL;
		GXmlTDocument* _tmp4_ = NULL;
		GFile* _tmp5_ = NULL;
		GXmlTDocument* _tmp6_ = NULL;
		GXmlNode* _tmp7_ = NULL;
		GXmlNode* _tmp8_ = NULL;
		GXmlNode* _tmp9_ = NULL;
		GXmlTDocument* _tmp10_ = NULL;
		GXmlNode* _tmp11_ = NULL;
		GXmlNode* _tmp12_ = NULL;
		GXmlNode* _tmp13_ = NULL;
		GeeBidirList* _tmp14_ = NULL;
		GeeBidirList* _tmp15_ = NULL;
		GeeBidirList* _tmp16_ = NULL;
		gint _tmp17_ = 0;
		gint _tmp18_ = 0;
		GXmlNode* n = NULL;
		GXmlTDocument* _tmp19_ = NULL;
		GXmlNode* _tmp20_ = NULL;
		GXmlNode* _tmp21_ = NULL;
		GXmlNode* _tmp22_ = NULL;
		GeeBidirList* _tmp23_ = NULL;
		GeeBidirList* _tmp24_ = NULL;
		GeeBidirList* _tmp25_ = NULL;
		gpointer _tmp26_ = NULL;
		GXmlNode* _tmp27_ = NULL;
		GXmlNode* _tmp28_ = NULL;
		GXmlNode* _tmp29_ = NULL;
		gchar* _tmp30_ = NULL;
		gchar* _tmp31_ = NULL;
		gchar* _tmp32_ = NULL;
		GXmlNode* _tmp33_ = NULL;
		GeeBidirList* _tmp34_ = NULL;
		GeeBidirList* _tmp35_ = NULL;
		GeeBidirList* _tmp36_ = NULL;
		gint _tmp37_ = 0;
		gint _tmp38_ = 0;
		GXmlNode* nc = NULL;
		GXmlNode* _tmp39_ = NULL;
		GeeBidirList* _tmp40_ = NULL;
		GeeBidirList* _tmp41_ = NULL;
		GeeBidirList* _tmp42_ = NULL;
		gpointer _tmp43_ = NULL;
		GXmlNode* _tmp44_ = NULL;
		GXmlNode* _tmp45_ = NULL;
		GXmlNode* _tmp46_ = NULL;
		gchar* _tmp47_ = NULL;
		gchar* _tmp48_ = NULL;
		gchar* _tmp49_ = NULL;
		GXmlNode* _tmp50_ = NULL;
		GeeBidirList* _tmp51_ = NULL;
		GeeBidirList* _tmp52_ = NULL;
		GeeBidirList* _tmp53_ = NULL;
		gint _tmp54_ = 0;
		gint _tmp55_ = 0;
		GXmlTDocumentReadTypeFunc f1 = NULL;
		void* f1_target = NULL;
		GDestroyNotify f1_target_destroy_notify = NULL;
		GXmlTDocument* d2 = NULL;
		GXmlTDocument* _tmp56_ = NULL;
		GXmlTDocument* _tmp57_ = NULL;
		GFile* _tmp58_ = NULL;
		GXmlTDocumentReadTypeFunc _tmp59_ = NULL;
		void* _tmp59__target = NULL;
		GXmlTDocument* _tmp60_ = NULL;
		GXmlNode* _tmp61_ = NULL;
		GXmlNode* _tmp62_ = NULL;
		GXmlNode* _tmp63_ = NULL;
		GXmlTDocument* _tmp64_ = NULL;
		GXmlNode* _tmp65_ = NULL;
		GXmlNode* _tmp66_ = NULL;
		GXmlNode* _tmp67_ = NULL;
		GeeBidirList* _tmp68_ = NULL;
		GeeBidirList* _tmp69_ = NULL;
		GeeBidirList* _tmp70_ = NULL;
		gint _tmp71_ = 0;
		gint _tmp72_ = 0;
		GXmlNode* n2 = NULL;
		GXmlTDocument* _tmp73_ = NULL;
		GXmlNode* _tmp74_ = NULL;
		GXmlNode* _tmp75_ = NULL;
		GXmlNode* _tmp76_ = NULL;
		GeeBidirList* _tmp77_ = NULL;
		GeeBidirList* _tmp78_ = NULL;
		GeeBidirList* _tmp79_ = NULL;
		gpointer _tmp80_ = NULL;
		GXmlNode* _tmp81_ = NULL;
		GXmlNode* _tmp82_ = NULL;
		GXmlNode* _tmp83_ = NULL;
		gchar* _tmp84_ = NULL;
		gchar* _tmp85_ = NULL;
		gchar* _tmp86_ = NULL;
		GXmlNode* _tmp87_ = NULL;
		GeeBidirList* _tmp88_ = NULL;
		GeeBidirList* _tmp89_ = NULL;
		GeeBidirList* _tmp90_ = NULL;
		gint _tmp91_ = 0;
		gint _tmp92_ = 0;
		GXmlNode* nc2 = NULL;
		GXmlNode* _tmp93_ = NULL;
		GeeBidirList* _tmp94_ = NULL;
		GeeBidirList* _tmp95_ = NULL;
		GeeBidirList* _tmp96_ = NULL;
		gpointer _tmp97_ = NULL;
		GXmlNode* _tmp98_ = NULL;
		GXmlNode* _tmp99_ = NULL;
		GXmlNode* _tmp100_ = NULL;
		gchar* _tmp101_ = NULL;
		gchar* _tmp102_ = NULL;
		gchar* _tmp103_ = NULL;
		GXmlNode* _tmp104_ = NULL;
		GeeBidirList* _tmp105_ = NULL;
		GeeBidirList* _tmp106_ = NULL;
		GeeBidirList* _tmp107_ = NULL;
		gint _tmp108_ = 0;
		gint _tmp109_ = 0;
		GXmlTDocumentReadTypeFunc f2 = NULL;
		void* f2_target = NULL;
		GDestroyNotify f2_target_destroy_notify = NULL;
		GXmlTDocument* d3 = NULL;
		GXmlTDocument* _tmp110_ = NULL;
		GXmlTDocument* _tmp111_ = NULL;
		GFile* _tmp112_ = NULL;
		GXmlTDocumentReadTypeFunc _tmp113_ = NULL;
		void* _tmp113__target = NULL;
		GXmlTDocument* _tmp114_ = NULL;
		gchar* _tmp115_ = NULL;
		gchar* _tmp116_ = NULL;
		gchar* _tmp117_ = NULL;
		gchar* _tmp118_ = NULL;
		GXmlTDocument* _tmp119_ = NULL;
		GXmlNode* _tmp120_ = NULL;
		GXmlNode* _tmp121_ = NULL;
		GXmlNode* _tmp122_ = NULL;
		GXmlTDocument* _tmp123_ = NULL;
		GXmlNode* _tmp124_ = NULL;
		GXmlNode* _tmp125_ = NULL;
		GXmlNode* _tmp126_ = NULL;
		GeeBidirList* _tmp127_ = NULL;
		GeeBidirList* _tmp128_ = NULL;
		GeeBidirList* _tmp129_ = NULL;
		gint _tmp130_ = 0;
		gint _tmp131_ = 0;
		GXmlNode* n3 = NULL;
		GXmlTDocument* _tmp132_ = NULL;
		GXmlNode* _tmp133_ = NULL;
		GXmlNode* _tmp134_ = NULL;
		GXmlNode* _tmp135_ = NULL;
		GeeBidirList* _tmp136_ = NULL;
		GeeBidirList* _tmp137_ = NULL;
		GeeBidirList* _tmp138_ = NULL;
		gpointer _tmp139_ = NULL;
		GXmlNode* _tmp140_ = NULL;
		gchar* _tmp141_ = NULL;
		gchar* _tmp142_ = NULL;
		gchar* _tmp143_ = NULL;
		GeeBidirList* _tmp144_ = NULL;
		GeeBidirList* _tmp145_ = NULL;
		GeeBidirList* _tmp146_ = NULL;
		gint _tmp147_ = 0;
		gint _tmp148_ = 0;
		GXmlNode* nc3 = NULL;
		GeeBidirList* _tmp149_ = NULL;
		GeeBidirList* _tmp150_ = NULL;
		GeeBidirList* _tmp151_ = NULL;
		gpointer _tmp152_ = NULL;
		GXmlNode* _tmp153_ = NULL;
		gchar* _tmp154_ = NULL;
		gchar* _tmp155_ = NULL;
		gchar* _tmp156_ = NULL;
		GeeBidirList* _tmp157_ = NULL;
		GeeBidirList* _tmp158_ = NULL;
		GeeBidirList* _tmp159_ = NULL;
		gint _tmp160_ = 0;
		gint _tmp161_ = 0;
#line 634 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp0_ = g_file_new_for_path (TEST_DIR "/t-read-test.xml");
#line 634 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		file = _tmp0_;
#line 635 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp1_ = file;
#line 635 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp2_ = g_file_query_exists (_tmp1_, NULL);
#line 635 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp2_, "file.query_exists ()");
#line 636 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp3_ = gxml_tdocument_new ();
#line 636 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		d = _tmp3_;
#line 637 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp4_ = d;
#line 637 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp5_ = file;
#line 637 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_tdocument_read_doc ((GXmlDocument*) _tmp4_, _tmp5_, NULL, NULL, &_inner_error_);
#line 637 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 637 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 637 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (file);
#line 8786 "TElementTest.c"
			goto __catch123_g_error;
		}
#line 638 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp6_ = d;
#line 638 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp7_ = gxml_document_get_root ((GXmlDocument*) _tmp6_);
#line 638 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp8_ = _tmp7_;
#line 638 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp9_ = _tmp8_;
#line 638 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp9_ != NULL, "d.root != null");
#line 638 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp9_);
#line 639 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp10_ = d;
#line 639 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp11_ = gxml_document_get_root ((GXmlDocument*) _tmp10_);
#line 639 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp12_ = _tmp11_;
#line 639 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp13_ = _tmp12_;
#line 639 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp14_ = gxml_node_get_children_nodes (_tmp13_);
#line 639 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp15_ = _tmp14_;
#line 639 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp16_ = _tmp15_;
#line 639 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp17_ = gee_collection_get_size ((GeeCollection*) _tmp16_);
#line 639 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp18_ = _tmp17_;
#line 639 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp18_ == 7, "d.root.children_nodes.size == 7");
#line 639 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp16_);
#line 639 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp13_);
#line 640 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp19_ = d;
#line 640 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp20_ = gxml_document_get_root ((GXmlDocument*) _tmp19_);
#line 640 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp21_ = _tmp20_;
#line 640 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp22_ = _tmp21_;
#line 640 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp23_ = gxml_node_get_children_nodes (_tmp22_);
#line 640 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp24_ = _tmp23_;
#line 640 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp25_ = _tmp24_;
#line 640 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp26_ = gee_list_get ((GeeList*) _tmp25_, 6);
#line 640 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp27_ = (GXmlNode*) _tmp26_;
#line 640 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp25_);
#line 640 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp22_);
#line 640 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		n = _tmp27_;
#line 641 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp28_ = n;
#line 641 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp28_ != NULL, "n != null");
#line 642 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp29_ = n;
#line 642 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp30_ = gxml_node_get_name (_tmp29_);
#line 642 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp31_ = _tmp30_;
#line 642 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp32_ = _tmp31_;
#line 642 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp32_, "ReadTop") == 0, "n.name == \"ReadTop\"");
#line 642 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp32_);
#line 643 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp33_ = n;
#line 643 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp34_ = gxml_node_get_children_nodes (_tmp33_);
#line 643 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp35_ = _tmp34_;
#line 643 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp36_ = _tmp35_;
#line 643 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp37_ = gee_collection_get_size ((GeeCollection*) _tmp36_);
#line 643 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp38_ = _tmp37_;
#line 643 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp38_ == 9, "n.children_nodes.size == 9");
#line 643 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp36_);
#line 644 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp39_ = n;
#line 644 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp40_ = gxml_node_get_children_nodes (_tmp39_);
#line 644 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp41_ = _tmp40_;
#line 644 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp42_ = _tmp41_;
#line 644 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp43_ = gee_list_get ((GeeList*) _tmp42_, 3);
#line 644 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp44_ = (GXmlNode*) _tmp43_;
#line 644 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp42_);
#line 644 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		nc = _tmp44_;
#line 645 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp45_ = nc;
#line 645 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp45_ != NULL, "nc != null");
#line 646 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp46_ = nc;
#line 646 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp47_ = gxml_node_get_name (_tmp46_);
#line 646 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp48_ = _tmp47_;
#line 646 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp49_ = _tmp48_;
#line 646 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp49_, "Read") == 0, "nc.name == \"Read\"");
#line 646 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp49_);
#line 647 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp50_ = nc;
#line 647 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp51_ = gxml_node_get_children_nodes (_tmp50_);
#line 647 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp52_ = _tmp51_;
#line 647 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp53_ = _tmp52_;
#line 647 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp54_ = gee_collection_get_size ((GeeCollection*) _tmp53_);
#line 647 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp55_ = _tmp54_;
#line 647 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp55_ == 2, "nc.children_nodes.size == 2");
#line 647 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp53_);
#line 649 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_test_message ("Skiping nodes");
#line 650 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		f1 = ____lambda101__gxml_tdocument_read_type_func;
#line 650 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		f1_target = NULL;
#line 650 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		f1_target_destroy_notify = NULL;
#line 658 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp56_ = gxml_tdocument_new ();
#line 658 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		d2 = _tmp56_;
#line 659 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp57_ = d2;
#line 659 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp58_ = file;
#line 659 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp59_ = f1;
#line 659 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp59__target = f1_target;
#line 659 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_tdocument_read_doc ((GXmlDocument*) _tmp57_, _tmp58_, _tmp59_, _tmp59__target, &_inner_error_);
#line 659 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 659 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d2);
#line 659 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			(f1_target_destroy_notify == NULL) ? NULL : (f1_target_destroy_notify (f1_target), NULL);
#line 659 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			f1 = NULL;
#line 659 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			f1_target = NULL;
#line 659 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			f1_target_destroy_notify = NULL;
#line 659 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (nc);
#line 659 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (n);
#line 659 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 659 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (file);
#line 8971 "TElementTest.c"
			goto __catch123_g_error;
		}
#line 663 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp60_ = d2;
#line 663 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp61_ = gxml_document_get_root ((GXmlDocument*) _tmp60_);
#line 663 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp62_ = _tmp61_;
#line 663 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp63_ = _tmp62_;
#line 663 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp63_ != NULL, "d2.root != null");
#line 663 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp63_);
#line 664 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp64_ = d2;
#line 664 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp65_ = gxml_document_get_root ((GXmlDocument*) _tmp64_);
#line 664 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp66_ = _tmp65_;
#line 664 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp67_ = _tmp66_;
#line 664 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp68_ = gxml_node_get_children_nodes (_tmp67_);
#line 664 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp69_ = _tmp68_;
#line 664 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp70_ = _tmp69_;
#line 664 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp71_ = gee_collection_get_size ((GeeCollection*) _tmp70_);
#line 664 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp72_ = _tmp71_;
#line 664 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp72_ == 7, "d2.root.children_nodes.size == 7");
#line 664 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp70_);
#line 664 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp67_);
#line 665 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp73_ = d2;
#line 665 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp74_ = gxml_document_get_root ((GXmlDocument*) _tmp73_);
#line 665 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp75_ = _tmp74_;
#line 665 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp76_ = _tmp75_;
#line 665 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp77_ = gxml_node_get_children_nodes (_tmp76_);
#line 665 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp78_ = _tmp77_;
#line 665 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp79_ = _tmp78_;
#line 665 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp80_ = gee_list_get ((GeeList*) _tmp79_, 6);
#line 665 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp81_ = (GXmlNode*) _tmp80_;
#line 665 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp79_);
#line 665 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp76_);
#line 665 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		n2 = _tmp81_;
#line 666 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp82_ = n2;
#line 666 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp82_ != NULL, "n2 != null");
#line 667 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp83_ = n2;
#line 667 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp84_ = gxml_node_get_name (_tmp83_);
#line 667 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp85_ = _tmp84_;
#line 667 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp86_ = _tmp85_;
#line 667 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp86_, "ReadTop") == 0, "n2.name == \"ReadTop\"");
#line 667 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp86_);
#line 668 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp87_ = n2;
#line 668 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp88_ = gxml_node_get_children_nodes (_tmp87_);
#line 668 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp89_ = _tmp88_;
#line 668 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp90_ = _tmp89_;
#line 668 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp91_ = gee_collection_get_size ((GeeCollection*) _tmp90_);
#line 668 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp92_ = _tmp91_;
#line 668 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp92_ == 4, "n2.children_nodes.size == 4");
#line 668 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp90_);
#line 669 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp93_ = n2;
#line 669 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp94_ = gxml_node_get_children_nodes (_tmp93_);
#line 669 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp95_ = _tmp94_;
#line 669 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp96_ = _tmp95_;
#line 669 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp97_ = gee_list_get ((GeeList*) _tmp96_, 2);
#line 669 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp98_ = (GXmlNode*) _tmp97_;
#line 669 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp96_);
#line 669 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		nc2 = _tmp98_;
#line 670 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp99_ = nc2;
#line 670 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp99_ != NULL, "nc2 != null");
#line 671 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp100_ = nc2;
#line 671 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp101_ = gxml_node_get_name (_tmp100_);
#line 671 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp102_ = _tmp101_;
#line 671 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp103_ = _tmp102_;
#line 671 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp103_, "Read") == 0, "nc2.name == \"Read\"");
#line 671 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp103_);
#line 672 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp104_ = nc2;
#line 672 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp105_ = gxml_node_get_children_nodes (_tmp104_);
#line 672 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp106_ = _tmp105_;
#line 672 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp107_ = _tmp106_;
#line 672 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp108_ = gee_collection_get_size ((GeeCollection*) _tmp107_);
#line 672 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp109_ = _tmp108_;
#line 672 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp109_ == 1, "nc2.children_nodes.size == 1");
#line 672 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp107_);
#line 674 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_test_message ("Skiping nodes using ReadType.STOP");
#line 676 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		f2 = ____lambda102__gxml_tdocument_read_type_func;
#line 676 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		f2_target = NULL;
#line 676 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		f2_target_destroy_notify = NULL;
#line 684 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp110_ = gxml_tdocument_new ();
#line 684 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		d3 = _tmp110_;
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp111_ = d3;
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp112_ = file;
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp113_ = f2;
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp113__target = f2_target;
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		gxml_tdocument_read_doc ((GXmlDocument*) _tmp111_, _tmp112_, _tmp113_, _tmp113__target, &_inner_error_);
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d3);
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			(f2_target_destroy_notify == NULL) ? NULL : (f2_target_destroy_notify (f2_target), NULL);
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			f2 = NULL;
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			f2_target = NULL;
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			f2_target_destroy_notify = NULL;
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (nc2);
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (n2);
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d2);
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			(f1_target_destroy_notify == NULL) ? NULL : (f1_target_destroy_notify (f1_target), NULL);
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			f1 = NULL;
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			f1_target = NULL;
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			f1_target_destroy_notify = NULL;
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (nc);
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (n);
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (d);
#line 685 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
			_g_object_unref0 (file);
#line 9170 "TElementTest.c"
			goto __catch123_g_error;
		}
#line 686 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp114_ = d3;
#line 686 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp115_ = gxml_tnode_to_string ((GXmlTNode*) _tmp114_);
#line 686 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp116_ = _tmp115_;
#line 686 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp117_ = g_strconcat ("STOPED:", _tmp116_, NULL);
#line 686 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp118_ = _tmp117_;
#line 686 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_test_message ("%s", _tmp118_);
#line 686 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp118_);
#line 686 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp116_);
#line 687 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp119_ = d3;
#line 687 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp120_ = gxml_document_get_root ((GXmlDocument*) _tmp119_);
#line 687 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp121_ = _tmp120_;
#line 687 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp122_ = _tmp121_;
#line 687 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp122_ != NULL, "d3.root != null");
#line 687 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp122_);
#line 688 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp123_ = d3;
#line 688 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp124_ = gxml_document_get_root ((GXmlDocument*) _tmp123_);
#line 688 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp125_ = _tmp124_;
#line 688 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp126_ = _tmp125_;
#line 688 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp127_ = gxml_node_get_children_nodes (_tmp126_);
#line 688 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp128_ = _tmp127_;
#line 688 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp129_ = _tmp128_;
#line 688 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp130_ = gee_collection_get_size ((GeeCollection*) _tmp129_);
#line 688 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp131_ = _tmp130_;
#line 688 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp131_ == 7, "d3.root.children_nodes.size == 7");
#line 688 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp129_);
#line 688 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp126_);
#line 689 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp132_ = d3;
#line 689 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp133_ = gxml_document_get_root ((GXmlDocument*) _tmp132_);
#line 689 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp134_ = _tmp133_;
#line 689 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp135_ = _tmp134_;
#line 689 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp136_ = gxml_node_get_children_nodes (_tmp135_);
#line 689 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp137_ = _tmp136_;
#line 689 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp138_ = _tmp137_;
#line 689 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp139_ = gee_list_get ((GeeList*) _tmp138_, 6);
#line 689 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp140_ = (GXmlNode*) _tmp139_;
#line 689 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp138_);
#line 689 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp135_);
#line 689 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		n3 = _tmp140_;
#line 690 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (n3 != NULL, "n3 != null");
#line 691 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp141_ = gxml_node_get_name (n3);
#line 691 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp142_ = _tmp141_;
#line 691 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp143_ = _tmp142_;
#line 691 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp143_, "ReadTop") == 0, "n3.name == \"ReadTop\"");
#line 691 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp143_);
#line 692 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp144_ = gxml_node_get_children_nodes (n3);
#line 692 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp145_ = _tmp144_;
#line 692 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp146_ = _tmp145_;
#line 692 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp147_ = gee_collection_get_size ((GeeCollection*) _tmp146_);
#line 692 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp148_ = _tmp147_;
#line 692 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp148_ == 4, "n3.children_nodes.size == 4");
#line 692 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp146_);
#line 693 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp149_ = gxml_node_get_children_nodes (n3);
#line 693 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp150_ = _tmp149_;
#line 693 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp151_ = _tmp150_;
#line 693 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp152_ = gee_list_get ((GeeList*) _tmp151_, 3);
#line 693 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp153_ = (GXmlNode*) _tmp152_;
#line 693 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp151_);
#line 693 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		nc3 = _tmp153_;
#line 694 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (nc3 != NULL, "nc3 != null");
#line 695 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp154_ = gxml_node_get_name (nc3);
#line 695 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp155_ = _tmp154_;
#line 695 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp156_ = _tmp155_;
#line 695 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (g_strcmp0 (_tmp156_, "Read") == 0, "nc3.name == \"Read\"");
#line 695 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp156_);
#line 696 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp157_ = gxml_node_get_children_nodes (nc3);
#line 696 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp158_ = _tmp157_;
#line 696 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp159_ = _tmp158_;
#line 696 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp160_ = gee_collection_get_size ((GeeCollection*) _tmp159_);
#line 696 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp161_ = _tmp160_;
#line 696 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_vala_assert (_tmp161_ == 1, "nc3.children_nodes.size == 1");
#line 696 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (_tmp159_);
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (nc3);
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (n3);
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (d3);
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		(f2_target_destroy_notify == NULL) ? NULL : (f2_target_destroy_notify (f2_target), NULL);
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		f2 = NULL;
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		f2_target = NULL;
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		f2_target_destroy_notify = NULL;
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (nc2);
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (n2);
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (d2);
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		(f1_target_destroy_notify == NULL) ? NULL : (f1_target_destroy_notify (f1_target), NULL);
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		f1 = NULL;
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		f1_target = NULL;
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		f1_target_destroy_notify = NULL;
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (nc);
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (n);
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (d);
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_object_unref0 (file);
#line 9351 "TElementTest.c"
	}
	goto __finally123;
	__catch123_g_error:
	{
		GError* e = NULL;
		const gchar* _tmp162_ = NULL;
		gchar* _tmp163_ = NULL;
		gchar* _tmp164_ = NULL;
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		e = _inner_error_;
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_inner_error_ = NULL;
#line 698 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp162_ = e->message;
#line 698 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp163_ = g_strconcat ("Error: ", _tmp162_, NULL);
#line 698 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_tmp164_ = _tmp163_;
#line 698 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_message ("TElementTest.vala:698: %s", _tmp164_);
#line 698 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_free0 (_tmp164_);
#line 699 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_assert_not_reached ();
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		_g_error_free0 (e);
#line 9378 "TElementTest.c"
	}
	__finally123:
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		g_clear_error (&_inner_error_);
#line 633 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
		return;
#line 9389 "TElementTest.c"
	}
}


static void ___lambda100__gtest_func (void) {
#line 632 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	__lambda100_ ();
#line 9397 "TElementTest.c"
}


void telement_test_add_tests (void) {
#line 29 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_test_add_func ("/gxml/t-element/api", ___lambda82__gtest_func);
#line 63 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_test_add_func ("/gxml/t-element/content", ___lambda83__gtest_func);
#line 105 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_test_add_func ("/gxml/t-element/namespaces/default", ___lambda84__gtest_func);
#line 122 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_test_add_func ("/gxml/t-element/namespaces/default-prefix", ___lambda85__gtest_func);
#line 139 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_test_add_func ("/gxml/t-element/namespaces/default-prefix-null", ___lambda86__gtest_func);
#line 156 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_test_add_func ("/gxml/t-element/namespaces/default/enable-prefix_default_ns", ___lambda87__gtest_func);
#line 174 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_test_add_func ("/gxml/t-element/multiple-namespaces", ___lambda88__gtest_func);
#line 205 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_test_add_func ("/gxml/t-element/multiple-namespaces/default/basic", ___lambda89__gtest_func);
#line 251 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_test_add_func ("/gxml/t-element/multiple-namespaces/enable-prefix_default_ns", ___lambda90__gtest_func);
#line 288 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_test_add_func ("/gxml/t-element/multiple-namespaces/default/1", ___lambda91__gtest_func);
#line 334 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_test_add_func ("/gxml/t-element/multiple-namespaces/default/enable-ns_top", ___lambda92__gtest_func);
#line 387 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_test_add_func ("/gxml/t-element/multiple-namespaces/child-default", ___lambda93__gtest_func);
#line 441 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_test_add_func ("/gxml/t-element/multiple-namespaces/child-default/enable-prefix_defaul" \
"t_ns", ___lambda94__gtest_func);
#line 495 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_test_add_func ("/gxml/t-element/multiple-namespaces/default/enable-prefix_default_ns", ___lambda95__gtest_func);
#line 531 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_test_add_func ("/gxml/t-element/attr-namespace/default", ___lambda96__gtest_func);
#line 558 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_test_add_func ("/gxml/t-element/attr-namespace/same-name", ___lambda97__gtest_func);
#line 593 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_test_add_func ("/gxml/t-element/parent", ___lambda98__gtest_func);
#line 615 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_test_add_func ("/gxml/t-element/attribute/parent", ___lambda99__gtest_func);
#line 632 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	g_test_add_func ("/gxml/t-element/readtype", ___lambda100__gtest_func);
#line 9440 "TElementTest.c"
}


TElementTest* telement_test_construct (GType object_type) {
	TElementTest* self = NULL;
#line 27 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	self = (TElementTest*) gxml_test_construct (object_type);
#line 27 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	return self;
#line 9450 "TElementTest.c"
}


TElementTest* telement_test_new (void) {
#line 27 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	return telement_test_construct (TYPE_TELEMENT_TEST);
#line 9457 "TElementTest.c"
}


static void telement_test_class_init (TElementTestClass * klass) {
#line 27 "/home/despinosa/Proyectos/gxml/test/TElementTest.vala"
	telement_test_parent_class = g_type_class_peek_parent (klass);
#line 9464 "TElementTest.c"
}


static void telement_test_instance_init (TElementTest * self) {
}


GType telement_test_get_type (void) {
	static volatile gsize telement_test_type_id__volatile = 0;
	if (g_once_init_enter (&telement_test_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (TElementTestClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) telement_test_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (TElementTest), 0, (GInstanceInitFunc) telement_test_instance_init, NULL };
		GType telement_test_type_id;
		telement_test_type_id = g_type_register_static (TYPE_GXML_TEST, "TElementTest", &g_define_type_info, 0);
		g_once_init_leave (&telement_test_type_id__volatile, telement_test_type_id);
	}
	return telement_test_type_id__volatile;
}



