/* SerializableGeeCollections-TDocument-Test.c generated by valac 0.34.6, the Vala compiler
 * generated from SerializableGeeCollections-TDocument-Test.vala, do not modify */

/* -*- Mode: vala; indent-tabs-mode: nil; c-basic-offset: 2; tab-width: 2 -*- */

#include <glib.h>
#include <glib-object.h>
#include <gxml/gxml.h>
#include <gee.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>


#define TYPE_GXML_TEST (gxml_test_get_type ())
#define GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GXML_TEST, GXmlTest))
#define GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GXML_TEST, GXmlTestClass))
#define IS_GXML_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GXML_TEST))
#define IS_GXML_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GXML_TEST))
#define GXML_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GXML_TEST, GXmlTestClass))

typedef struct _GXmlTest GXmlTest;
typedef struct _GXmlTestClass GXmlTestClass;
typedef struct _GXmlTestPrivate GXmlTestPrivate;

#define TYPE_SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST (serializable_gee_collections_tdocument_test_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST, SerializableGeeCollectionsTDocumentTest))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST, SerializableGeeCollectionsTDocumentTestClass))
#define IS_SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST))
#define IS_SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST, SerializableGeeCollectionsTDocumentTestClass))

typedef struct _SerializableGeeCollectionsTDocumentTest SerializableGeeCollectionsTDocumentTest;
typedef struct _SerializableGeeCollectionsTDocumentTestClass SerializableGeeCollectionsTDocumentTestClass;
typedef struct _SerializableGeeCollectionsTDocumentTestPrivate SerializableGeeCollectionsTDocumentTestPrivate;

#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_BASE (serializable_gee_collections_tdocument_test_space_base_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_BASE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_BASE, SerializableGeeCollectionsTDocumentTestSpaceBase))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_BASE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_BASE, SerializableGeeCollectionsTDocumentTestSpaceBaseClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_IS_SPACE_BASE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_BASE))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_IS_SPACE_BASE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_BASE))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_BASE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_BASE, SerializableGeeCollectionsTDocumentTestSpaceBaseClass))

typedef struct _SerializableGeeCollectionsTDocumentTestSpaceBase SerializableGeeCollectionsTDocumentTestSpaceBase;
typedef struct _SerializableGeeCollectionsTDocumentTestSpaceBaseClass SerializableGeeCollectionsTDocumentTestSpaceBaseClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CHARGE_ZONE (serializable_gee_collections_tdocument_test_charge_zone_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CHARGE_ZONE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CHARGE_ZONE, SerializableGeeCollectionsTDocumentTestChargeZone))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CHARGE_ZONE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CHARGE_ZONE, SerializableGeeCollectionsTDocumentTestChargeZoneClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_IS_CHARGE_ZONE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CHARGE_ZONE))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_IS_CHARGE_ZONE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CHARGE_ZONE))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CHARGE_ZONE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CHARGE_ZONE, SerializableGeeCollectionsTDocumentTestChargeZoneClass))

typedef struct _SerializableGeeCollectionsTDocumentTestChargeZone SerializableGeeCollectionsTDocumentTestChargeZone;
typedef struct _SerializableGeeCollectionsTDocumentTestChargeZoneClass SerializableGeeCollectionsTDocumentTestChargeZoneClass;

#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_TYPE_COLLECTION (serializable_gee_collections_tdocument_test_space_ship_collection_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_TYPE_COLLECTION, SerializableGeeCollectionsTDocumentTestSpaceShipCollection))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_TYPE_COLLECTION, SerializableGeeCollectionsTDocumentTestSpaceShipCollectionClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_TYPE_COLLECTION))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_IS_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_TYPE_COLLECTION))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_TYPE_COLLECTION, SerializableGeeCollectionsTDocumentTestSpaceShipCollectionClass))

typedef struct _SerializableGeeCollectionsTDocumentTestSpaceShipCollection SerializableGeeCollectionsTDocumentTestSpaceShipCollection;
typedef struct _SerializableGeeCollectionsTDocumentTestSpaceShipCollectionClass SerializableGeeCollectionsTDocumentTestSpaceShipCollectionClass;

#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_SHIP (serializable_gee_collections_tdocument_test_space_ship_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_SHIP, SerializableGeeCollectionsTDocumentTestSpaceShip))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_SHIP, SerializableGeeCollectionsTDocumentTestSpaceShipClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_IS_SPACE_SHIP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_SHIP))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_IS_SPACE_SHIP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_SHIP))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_SHIP, SerializableGeeCollectionsTDocumentTestSpaceShipClass))

typedef struct _SerializableGeeCollectionsTDocumentTestSpaceShip SerializableGeeCollectionsTDocumentTestSpaceShip;
typedef struct _SerializableGeeCollectionsTDocumentTestSpaceShipClass SerializableGeeCollectionsTDocumentTestSpaceShipClass;

#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_TYPE_COLLECTION (serializable_gee_collections_tdocument_test_space_collection_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_TYPE_COLLECTION, SerializableGeeCollectionsTDocumentTestSpaceCollection))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_TYPE_COLLECTION, SerializableGeeCollectionsTDocumentTestSpaceCollectionClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_TYPE_COLLECTION))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_IS_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_TYPE_COLLECTION))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_TYPE_COLLECTION, SerializableGeeCollectionsTDocumentTestSpaceCollectionClass))

typedef struct _SerializableGeeCollectionsTDocumentTestSpaceCollection SerializableGeeCollectionsTDocumentTestSpaceCollection;
typedef struct _SerializableGeeCollectionsTDocumentTestSpaceCollectionClass SerializableGeeCollectionsTDocumentTestSpaceCollectionClass;
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))

#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_STORAGE (serializable_gee_collections_tdocument_test_storage_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_STORAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_STORAGE, SerializableGeeCollectionsTDocumentTestStorage))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_STORAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_STORAGE, SerializableGeeCollectionsTDocumentTestStorageClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_IS_STORAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_STORAGE))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_IS_STORAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_STORAGE))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_STORAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_STORAGE, SerializableGeeCollectionsTDocumentTestStorageClass))

typedef struct _SerializableGeeCollectionsTDocumentTestStorage SerializableGeeCollectionsTDocumentTestStorage;
typedef struct _SerializableGeeCollectionsTDocumentTestStorageClass SerializableGeeCollectionsTDocumentTestStorageClass;

#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_TYPE_COLLECTION (serializable_gee_collections_tdocument_test_refaction_collection_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_TYPE_COLLECTION, SerializableGeeCollectionsTDocumentTestRefactionCollection))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_TYPE_COLLECTION, SerializableGeeCollectionsTDocumentTestRefactionCollectionClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_TYPE_COLLECTION))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_IS_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_TYPE_COLLECTION))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_TYPE_COLLECTION, SerializableGeeCollectionsTDocumentTestRefactionCollectionClass))

typedef struct _SerializableGeeCollectionsTDocumentTestRefactionCollection SerializableGeeCollectionsTDocumentTestRefactionCollection;
typedef struct _SerializableGeeCollectionsTDocumentTestRefactionCollectionClass SerializableGeeCollectionsTDocumentTestRefactionCollectionClass;

#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_REFACTION (serializable_gee_collections_tdocument_test_refaction_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_REFACTION, SerializableGeeCollectionsTDocumentTestRefaction))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_REFACTION, SerializableGeeCollectionsTDocumentTestRefactionClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_IS_REFACTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_REFACTION))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_IS_REFACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_REFACTION))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_REFACTION, SerializableGeeCollectionsTDocumentTestRefactionClass))

typedef struct _SerializableGeeCollectionsTDocumentTestRefaction SerializableGeeCollectionsTDocumentTestRefaction;
typedef struct _SerializableGeeCollectionsTDocumentTestRefactionClass SerializableGeeCollectionsTDocumentTestRefactionClass;

#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CITIZEN (serializable_gee_collections_tdocument_test_citizen_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CITIZEN, SerializableGeeCollectionsTDocumentTestCitizen))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CITIZEN, SerializableGeeCollectionsTDocumentTestCitizenClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_IS_CITIZEN(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CITIZEN))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_IS_CITIZEN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CITIZEN))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CITIZEN, SerializableGeeCollectionsTDocumentTestCitizenClass))

typedef struct _SerializableGeeCollectionsTDocumentTestCitizen SerializableGeeCollectionsTDocumentTestCitizen;
typedef struct _SerializableGeeCollectionsTDocumentTestCitizenClass SerializableGeeCollectionsTDocumentTestCitizenClass;
typedef struct _SerializableGeeCollectionsTDocumentTestCitizenPrivate SerializableGeeCollectionsTDocumentTestCitizenPrivate;

#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_TYPE_ARRAY (serializable_gee_collections_tdocument_test_citizen_array_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_TYPE_ARRAY, SerializableGeeCollectionsTDocumentTestCitizenArray))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_TYPE_ARRAY, SerializableGeeCollectionsTDocumentTestCitizenArrayClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_IS_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_TYPE_ARRAY))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_IS_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_TYPE_ARRAY))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_ARRAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_TYPE_ARRAY, SerializableGeeCollectionsTDocumentTestCitizenArrayClass))

typedef struct _SerializableGeeCollectionsTDocumentTestCitizenArray SerializableGeeCollectionsTDocumentTestCitizenArray;
typedef struct _SerializableGeeCollectionsTDocumentTestCitizenArrayClass SerializableGeeCollectionsTDocumentTestCitizenArrayClass;
typedef struct _SerializableGeeCollectionsTDocumentTestCitizenArrayPrivate SerializableGeeCollectionsTDocumentTestCitizenArrayPrivate;

#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_ASTEROID (serializable_gee_collections_tdocument_test_asteroid_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_ASTEROID, SerializableGeeCollectionsTDocumentTestAsteroid))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_ASTEROID, SerializableGeeCollectionsTDocumentTestAsteroidClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_IS_ASTEROID(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_ASTEROID))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_IS_ASTEROID_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_ASTEROID))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_ASTEROID, SerializableGeeCollectionsTDocumentTestAsteroidClass))

typedef struct _SerializableGeeCollectionsTDocumentTestAsteroid SerializableGeeCollectionsTDocumentTestAsteroid;
typedef struct _SerializableGeeCollectionsTDocumentTestAsteroidClass SerializableGeeCollectionsTDocumentTestAsteroidClass;
typedef struct _SerializableGeeCollectionsTDocumentTestAsteroidPrivate SerializableGeeCollectionsTDocumentTestAsteroidPrivate;

#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_TYPE_ARRAY (serializable_gee_collections_tdocument_test_asteroid_array_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_TYPE_ARRAY, SerializableGeeCollectionsTDocumentTestAsteroidArray))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_TYPE_ARRAY, SerializableGeeCollectionsTDocumentTestAsteroidArrayClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_IS_ARRAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_TYPE_ARRAY))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_IS_ARRAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_TYPE_ARRAY))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_ARRAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_TYPE_ARRAY, SerializableGeeCollectionsTDocumentTestAsteroidArrayClass))

typedef struct _SerializableGeeCollectionsTDocumentTestAsteroidArray SerializableGeeCollectionsTDocumentTestAsteroidArray;
typedef struct _SerializableGeeCollectionsTDocumentTestAsteroidArrayClass SerializableGeeCollectionsTDocumentTestAsteroidArrayClass;
typedef struct _SerializableGeeCollectionsTDocumentTestAsteroidArrayPrivate SerializableGeeCollectionsTDocumentTestAsteroidArrayPrivate;

#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_PLANET (serializable_gee_collections_tdocument_test_planet_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_PLANET, SerializableGeeCollectionsTDocumentTestPlanet))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_PLANET, SerializableGeeCollectionsTDocumentTestPlanetClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_IS_PLANET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_PLANET))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_IS_PLANET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_PLANET))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_PLANET, SerializableGeeCollectionsTDocumentTestPlanetClass))

typedef struct _SerializableGeeCollectionsTDocumentTestPlanet SerializableGeeCollectionsTDocumentTestPlanet;
typedef struct _SerializableGeeCollectionsTDocumentTestPlanetClass SerializableGeeCollectionsTDocumentTestPlanetClass;
typedef struct _SerializableGeeCollectionsTDocumentTestPlanetPrivate SerializableGeeCollectionsTDocumentTestPlanetPrivate;

#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_TYPE_COLLECTION (serializable_gee_collections_tdocument_test_planet_collection_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_TYPE_COLLECTION, SerializableGeeCollectionsTDocumentTestPlanetCollection))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_TYPE_COLLECTION, SerializableGeeCollectionsTDocumentTestPlanetCollectionClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_TYPE_COLLECTION))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_IS_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_TYPE_COLLECTION))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_TYPE_COLLECTION, SerializableGeeCollectionsTDocumentTestPlanetCollectionClass))

typedef struct _SerializableGeeCollectionsTDocumentTestPlanetCollection SerializableGeeCollectionsTDocumentTestPlanetCollection;
typedef struct _SerializableGeeCollectionsTDocumentTestPlanetCollectionClass SerializableGeeCollectionsTDocumentTestPlanetCollectionClass;
typedef struct _SerializableGeeCollectionsTDocumentTestPlanetCollectionPrivate SerializableGeeCollectionsTDocumentTestPlanetCollectionPrivate;

#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE (serializable_gee_collections_tdocument_test_space_get_type ())
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE, SerializableGeeCollectionsTDocumentTestSpace))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE, SerializableGeeCollectionsTDocumentTestSpaceClass))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_IS_SPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_IS_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE))
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE, SerializableGeeCollectionsTDocumentTestSpaceClass))

typedef struct _SerializableGeeCollectionsTDocumentTestSpace SerializableGeeCollectionsTDocumentTestSpace;
typedef struct _SerializableGeeCollectionsTDocumentTestSpaceClass SerializableGeeCollectionsTDocumentTestSpaceClass;
typedef struct _SerializableGeeCollectionsTDocumentTestSpacePrivate SerializableGeeCollectionsTDocumentTestSpacePrivate;
typedef struct _SerializableGeeCollectionsTDocumentTestSpaceCollectionPrivate SerializableGeeCollectionsTDocumentTestSpaceCollectionPrivate;
typedef struct _SerializableGeeCollectionsTDocumentTestRefactionPrivate SerializableGeeCollectionsTDocumentTestRefactionPrivate;
typedef struct _SerializableGeeCollectionsTDocumentTestRefactionCollectionPrivate SerializableGeeCollectionsTDocumentTestRefactionCollectionPrivate;
typedef struct _SerializableGeeCollectionsTDocumentTestSpaceShipPrivate SerializableGeeCollectionsTDocumentTestSpaceShipPrivate;
typedef struct _SerializableGeeCollectionsTDocumentTestSpaceShipCollectionPrivate SerializableGeeCollectionsTDocumentTestSpaceShipCollectionPrivate;
typedef struct _SerializableGeeCollectionsTDocumentTestChargeZonePrivate SerializableGeeCollectionsTDocumentTestChargeZonePrivate;
typedef struct _SerializableGeeCollectionsTDocumentTestStoragePrivate SerializableGeeCollectionsTDocumentTestStoragePrivate;
typedef struct _SerializableGeeCollectionsTDocumentTestSpaceBasePrivate SerializableGeeCollectionsTDocumentTestSpaceBasePrivate;
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _GXmlTest {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GXmlTestPrivate * priv;
};

struct _GXmlTestClass {
	GTypeClass parent_class;
	void (*finalize) (GXmlTest *self);
};

struct _SerializableGeeCollectionsTDocumentTest {
	GXmlTest parent_instance;
	SerializableGeeCollectionsTDocumentTestPrivate * priv;
};

struct _SerializableGeeCollectionsTDocumentTestClass {
	GXmlTestClass parent_class;
};

struct _SerializableGeeCollectionsTDocumentTestCitizen {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeCollectionsTDocumentTestCitizenPrivate * priv;
};

struct _SerializableGeeCollectionsTDocumentTestCitizenClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeCollectionsTDocumentTestCitizenPrivate {
	gchar* _ctype;
};

struct _SerializableGeeCollectionsTDocumentTestCitizenArray {
	GXmlSerializableArrayList parent_instance;
	SerializableGeeCollectionsTDocumentTestCitizenArrayPrivate * priv;
};

struct _SerializableGeeCollectionsTDocumentTestCitizenArrayClass {
	GXmlSerializableArrayListClass parent_class;
};

struct _SerializableGeeCollectionsTDocumentTestAsteroid {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeCollectionsTDocumentTestAsteroidPrivate * priv;
};

struct _SerializableGeeCollectionsTDocumentTestAsteroidClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeCollectionsTDocumentTestAsteroidPrivate {
	gint _size;
};

struct _SerializableGeeCollectionsTDocumentTestAsteroidArray {
	GXmlSerializableArrayList parent_instance;
	SerializableGeeCollectionsTDocumentTestAsteroidArrayPrivate * priv;
};

struct _SerializableGeeCollectionsTDocumentTestAsteroidArrayClass {
	GXmlSerializableArrayListClass parent_class;
};

struct _SerializableGeeCollectionsTDocumentTestPlanet {
	GXmlSerializableContainer parent_instance;
	SerializableGeeCollectionsTDocumentTestPlanetPrivate * priv;
};

struct _SerializableGeeCollectionsTDocumentTestPlanetClass {
	GXmlSerializableContainerClass parent_class;
};

struct _SerializableGeeCollectionsTDocumentTestPlanetPrivate {
	gchar* _name;
	SerializableGeeCollectionsTDocumentTestCitizenArray* _citizens;
};

struct _SerializableGeeCollectionsTDocumentTestPlanetCollection {
	GXmlSerializableTreeMap parent_instance;
	SerializableGeeCollectionsTDocumentTestPlanetCollectionPrivate * priv;
};

struct _SerializableGeeCollectionsTDocumentTestPlanetCollectionClass {
	GXmlSerializableTreeMapClass parent_class;
};

struct _SerializableGeeCollectionsTDocumentTestSpace {
	GXmlSerializableContainer parent_instance;
	SerializableGeeCollectionsTDocumentTestSpacePrivate * priv;
};

struct _SerializableGeeCollectionsTDocumentTestSpaceClass {
	GXmlSerializableContainerClass parent_class;
};

struct _SerializableGeeCollectionsTDocumentTestSpacePrivate {
	gchar* _name;
	SerializableGeeCollectionsTDocumentTestPlanetCollection* _planets;
	SerializableGeeCollectionsTDocumentTestAsteroidArray* _asteroids;
};

struct _SerializableGeeCollectionsTDocumentTestSpaceCollection {
	GXmlSerializableTreeMap parent_instance;
	SerializableGeeCollectionsTDocumentTestSpaceCollectionPrivate * priv;
};

struct _SerializableGeeCollectionsTDocumentTestSpaceCollectionClass {
	GXmlSerializableTreeMapClass parent_class;
};

struct _SerializableGeeCollectionsTDocumentTestRefaction {
	GXmlSerializableContainer parent_instance;
	SerializableGeeCollectionsTDocumentTestRefactionPrivate * priv;
};

struct _SerializableGeeCollectionsTDocumentTestRefactionClass {
	GXmlSerializableContainerClass parent_class;
};

struct _SerializableGeeCollectionsTDocumentTestRefactionPrivate {
	gchar* _model;
	gchar* _manufacturer;
	SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _spaceships;
};

struct _SerializableGeeCollectionsTDocumentTestRefactionCollection {
	GXmlSerializableDualKeyMap parent_instance;
	SerializableGeeCollectionsTDocumentTestRefactionCollectionPrivate * priv;
};

struct _SerializableGeeCollectionsTDocumentTestRefactionCollectionClass {
	GXmlSerializableDualKeyMapClass parent_class;
};

struct _SerializableGeeCollectionsTDocumentTestSpaceShip {
	GXmlSerializableContainer parent_instance;
	SerializableGeeCollectionsTDocumentTestSpaceShipPrivate * priv;
};

struct _SerializableGeeCollectionsTDocumentTestSpaceShipClass {
	GXmlSerializableContainerClass parent_class;
};

struct _SerializableGeeCollectionsTDocumentTestSpaceShipPrivate {
	gchar* _model;
	gchar* _manufacturer;
	SerializableGeeCollectionsTDocumentTestSpaceCollection* _spaces;
};

struct _SerializableGeeCollectionsTDocumentTestSpaceShipCollection {
	GXmlSerializableDualKeyMap parent_instance;
	SerializableGeeCollectionsTDocumentTestSpaceShipCollectionPrivate * priv;
};

struct _SerializableGeeCollectionsTDocumentTestSpaceShipCollectionClass {
	GXmlSerializableDualKeyMapClass parent_class;
};

struct _SerializableGeeCollectionsTDocumentTestChargeZone {
	GXmlSerializableContainer parent_instance;
	SerializableGeeCollectionsTDocumentTestChargeZonePrivate * priv;
};

struct _SerializableGeeCollectionsTDocumentTestChargeZoneClass {
	GXmlSerializableContainerClass parent_class;
};

struct _SerializableGeeCollectionsTDocumentTestChargeZonePrivate {
	gchar* _name;
	SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _spaceships;
};

struct _SerializableGeeCollectionsTDocumentTestStorage {
	GXmlSerializableContainer parent_instance;
	SerializableGeeCollectionsTDocumentTestStoragePrivate * priv;
};

struct _SerializableGeeCollectionsTDocumentTestStorageClass {
	GXmlSerializableContainerClass parent_class;
};

struct _SerializableGeeCollectionsTDocumentTestStoragePrivate {
	gchar* _name;
	SerializableGeeCollectionsTDocumentTestRefactionCollection* _refactions;
};

struct _SerializableGeeCollectionsTDocumentTestSpaceBase {
	GXmlSerializableObjectModel parent_instance;
	SerializableGeeCollectionsTDocumentTestSpaceBasePrivate * priv;
};

struct _SerializableGeeCollectionsTDocumentTestSpaceBaseClass {
	GXmlSerializableObjectModelClass parent_class;
};

struct _SerializableGeeCollectionsTDocumentTestSpaceBasePrivate {
	gchar* _name;
	SerializableGeeCollectionsTDocumentTestChargeZone* _charge_zone;
	SerializableGeeCollectionsTDocumentTestStorage* _storage;
};


static gpointer serializable_gee_collections_tdocument_test_parent_class = NULL;
static gpointer serializable_gee_collections_tdocument_test_citizen_parent_class = NULL;
static gpointer serializable_gee_collections_tdocument_test_citizen_array_parent_class = NULL;
static gpointer serializable_gee_collections_tdocument_test_asteroid_parent_class = NULL;
static gpointer serializable_gee_collections_tdocument_test_asteroid_array_parent_class = NULL;
static gpointer serializable_gee_collections_tdocument_test_planet_parent_class = NULL;
static gpointer serializable_gee_collections_tdocument_test_planet_collection_parent_class = NULL;
static GXmlSerializableMapKeyIface* serializable_gee_collections_tdocument_test_planet_gxml_serializable_map_key_parent_iface = NULL;
static gpointer serializable_gee_collections_tdocument_test_space_parent_class = NULL;
static gpointer serializable_gee_collections_tdocument_test_space_collection_parent_class = NULL;
static GXmlSerializableMapKeyIface* serializable_gee_collections_tdocument_test_space_gxml_serializable_map_key_parent_iface = NULL;
static gpointer serializable_gee_collections_tdocument_test_refaction_parent_class = NULL;
static gpointer serializable_gee_collections_tdocument_test_refaction_collection_parent_class = NULL;
static GXmlSerializableMapDualKeyIface* serializable_gee_collections_tdocument_test_refaction_gxml_serializable_map_dual_key_parent_iface = NULL;
static gpointer serializable_gee_collections_tdocument_test_space_ship_parent_class = NULL;
static gpointer serializable_gee_collections_tdocument_test_space_ship_collection_parent_class = NULL;
static GXmlSerializableMapDualKeyIface* serializable_gee_collections_tdocument_test_space_ship_gxml_serializable_map_dual_key_parent_iface = NULL;
static gpointer serializable_gee_collections_tdocument_test_charge_zone_parent_class = NULL;
static gpointer serializable_gee_collections_tdocument_test_storage_parent_class = NULL;
static gpointer serializable_gee_collections_tdocument_test_space_base_parent_class = NULL;

gpointer gxml_test_ref (gpointer instance);
void gxml_test_unref (gpointer instance);
GParamSpec* param_spec_gxml_test (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_gxml_test (GValue* value, gpointer v_object);
void value_take_gxml_test (GValue* value, gpointer v_object);
gpointer value_get_gxml_test (const GValue* value);
GType gxml_test_get_type (void) G_GNUC_CONST;
GType serializable_gee_collections_tdocument_test_get_type (void) G_GNUC_CONST;
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_DUMMY_PROPERTY
};
void serializable_gee_collections_tdocument_test_add_tests (void);
static void __lambda70_ (void);
static GType serializable_gee_collections_tdocument_test_space_base_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeCollectionsTDocumentTestSpaceBase* serializable_gee_collections_tdocument_test_space_base_new (void);
static SerializableGeeCollectionsTDocumentTestSpaceBase* serializable_gee_collections_tdocument_test_space_base_construct (GType object_type);
static GType serializable_gee_collections_tdocument_test_charge_zone_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeCollectionsTDocumentTestChargeZone* serializable_gee_collections_tdocument_test_space_base_get_charge_zone (SerializableGeeCollectionsTDocumentTestSpaceBase* self);
GType serializable_gee_collections_tdocument_test_space_ship_collection_get_type (void) G_GNUC_CONST;
static SerializableGeeCollectionsTDocumentTestSpaceShipCollection* serializable_gee_collections_tdocument_test_charge_zone_get_spaceships (SerializableGeeCollectionsTDocumentTestChargeZone* self);
static GType serializable_gee_collections_tdocument_test_space_ship_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
GType serializable_gee_collections_tdocument_test_space_collection_get_type (void) G_GNUC_CONST;
static SerializableGeeCollectionsTDocumentTestSpaceCollection* serializable_gee_collections_tdocument_test_space_ship_get_spaces (SerializableGeeCollectionsTDocumentTestSpaceShip* self);
static void ___lambda70__gtest_func (void);
static void __lambda71_ (void);
static GType serializable_gee_collections_tdocument_test_storage_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SerializableGeeCollectionsTDocumentTestStorage* serializable_gee_collections_tdocument_test_space_base_get_storage (SerializableGeeCollectionsTDocumentTestSpaceBase* self);
GType serializable_gee_collections_tdocument_test_refaction_collection_get_type (void) G_GNUC_CONST;
static SerializableGeeCollectionsTDocumentTestRefactionCollection* serializable_gee_collections_tdocument_test_storage_get_refactions (SerializableGeeCollectionsTDocumentTestStorage* self);
static GType serializable_gee_collections_tdocument_test_refaction_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static void ___lambda71__gtest_func (void);
SerializableGeeCollectionsTDocumentTest* serializable_gee_collections_tdocument_test_new (void);
SerializableGeeCollectionsTDocumentTest* serializable_gee_collections_tdocument_test_construct (GType object_type);
GXmlTest* gxml_test_new (void);
GXmlTest* gxml_test_construct (GType object_type);
static GType serializable_gee_collections_tdocument_test_citizen_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CITIZEN, SerializableGeeCollectionsTDocumentTestCitizenPrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_CTYPE
};
static gboolean serializable_gee_collections_tdocument_test_citizen_real_serialize_use_xml_node_value (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_collections_tdocument_test_citizen_real_to_string (GXmlSerializableObjectModel* base);
static const gchar* serializable_gee_collections_tdocument_test_citizen_get_ctype (SerializableGeeCollectionsTDocumentTestCitizen* self);
static gchar* serializable_gee_collections_tdocument_test_citizen_real_node_name (GXmlSerializableObjectModel* base);
static SerializableGeeCollectionsTDocumentTestCitizen* serializable_gee_collections_tdocument_test_citizen_new (void);
static SerializableGeeCollectionsTDocumentTestCitizen* serializable_gee_collections_tdocument_test_citizen_construct (GType object_type);
static void serializable_gee_collections_tdocument_test_citizen_set_ctype (SerializableGeeCollectionsTDocumentTestCitizen* self, const gchar* value);
GType serializable_gee_collections_tdocument_test_citizen_array_get_type (void) G_GNUC_CONST;
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_ARRAY_DUMMY_PROPERTY
};
static SerializableGeeCollectionsTDocumentTestCitizenArray* serializable_gee_collections_tdocument_test_citizen_array_new (void);
static SerializableGeeCollectionsTDocumentTestCitizenArray* serializable_gee_collections_tdocument_test_citizen_array_construct (GType object_type);
static void serializable_gee_collections_tdocument_test_citizen_finalize (GObject* obj);
static void _vala_serializable_gee_collections_tdocument_test_citizen_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_tdocument_test_citizen_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static GType serializable_gee_collections_tdocument_test_asteroid_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_ASTEROID, SerializableGeeCollectionsTDocumentTestAsteroidPrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_SIZE
};
static SerializableGeeCollectionsTDocumentTestAsteroid* serializable_gee_collections_tdocument_test_asteroid_new_with_size (gint size);
static SerializableGeeCollectionsTDocumentTestAsteroid* serializable_gee_collections_tdocument_test_asteroid_construct_with_size (GType object_type, gint size);
static void serializable_gee_collections_tdocument_test_asteroid_set_size (SerializableGeeCollectionsTDocumentTestAsteroid* self, gint value);
static gchar* serializable_gee_collections_tdocument_test_asteroid_real_to_string (GXmlSerializableObjectModel* base);
static gint serializable_gee_collections_tdocument_test_asteroid_get_size (SerializableGeeCollectionsTDocumentTestAsteroid* self);
static gchar* serializable_gee_collections_tdocument_test_asteroid_real_node_name (GXmlSerializableObjectModel* base);
static SerializableGeeCollectionsTDocumentTestAsteroid* serializable_gee_collections_tdocument_test_asteroid_new (void);
static SerializableGeeCollectionsTDocumentTestAsteroid* serializable_gee_collections_tdocument_test_asteroid_construct (GType object_type);
GType serializable_gee_collections_tdocument_test_asteroid_array_get_type (void) G_GNUC_CONST;
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_ARRAY_DUMMY_PROPERTY
};
static SerializableGeeCollectionsTDocumentTestAsteroidArray* serializable_gee_collections_tdocument_test_asteroid_array_new (void);
static SerializableGeeCollectionsTDocumentTestAsteroidArray* serializable_gee_collections_tdocument_test_asteroid_array_construct (GType object_type);
static void serializable_gee_collections_tdocument_test_asteroid_finalize (GObject* obj);
static void _vala_serializable_gee_collections_tdocument_test_asteroid_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_tdocument_test_asteroid_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static GType serializable_gee_collections_tdocument_test_planet_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_PLANET, SerializableGeeCollectionsTDocumentTestPlanetPrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_NAME,
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_CITIZENS
};
static gchar* serializable_gee_collections_tdocument_test_planet_real_get_map_key (GXmlSerializableMapKey* base);
static const gchar* serializable_gee_collections_tdocument_test_planet_get_name (SerializableGeeCollectionsTDocumentTestPlanet* self);
static SerializableGeeCollectionsTDocumentTestPlanet* serializable_gee_collections_tdocument_test_planet_new_named (const gchar* name);
static SerializableGeeCollectionsTDocumentTestPlanet* serializable_gee_collections_tdocument_test_planet_construct_named (GType object_type, const gchar* name);
static void serializable_gee_collections_tdocument_test_planet_set_name (SerializableGeeCollectionsTDocumentTestPlanet* self, const gchar* value);
static gchar* serializable_gee_collections_tdocument_test_planet_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_collections_tdocument_test_planet_real_to_string (GXmlSerializableObjectModel* base);
static void serializable_gee_collections_tdocument_test_planet_real_init_containers (GXmlSerializableContainer* base);
static SerializableGeeCollectionsTDocumentTestCitizenArray* serializable_gee_collections_tdocument_test_planet_get_citizens (SerializableGeeCollectionsTDocumentTestPlanet* self);
static void serializable_gee_collections_tdocument_test_planet_set_citizens (SerializableGeeCollectionsTDocumentTestPlanet* self, SerializableGeeCollectionsTDocumentTestCitizenArray* value);
static SerializableGeeCollectionsTDocumentTestPlanet* serializable_gee_collections_tdocument_test_planet_new (void);
static SerializableGeeCollectionsTDocumentTestPlanet* serializable_gee_collections_tdocument_test_planet_construct (GType object_type);
GType serializable_gee_collections_tdocument_test_planet_collection_get_type (void) G_GNUC_CONST;
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_COLLECTION_DUMMY_PROPERTY
};
static SerializableGeeCollectionsTDocumentTestPlanetCollection* serializable_gee_collections_tdocument_test_planet_collection_new (void);
static SerializableGeeCollectionsTDocumentTestPlanetCollection* serializable_gee_collections_tdocument_test_planet_collection_construct (GType object_type);
static void serializable_gee_collections_tdocument_test_planet_finalize (GObject* obj);
static void _vala_serializable_gee_collections_tdocument_test_planet_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_tdocument_test_planet_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static GType serializable_gee_collections_tdocument_test_space_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE, SerializableGeeCollectionsTDocumentTestSpacePrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_NAME,
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_PLANETS,
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_ASTEROIDS
};
static gchar* serializable_gee_collections_tdocument_test_space_real_get_map_key (GXmlSerializableMapKey* base);
static const gchar* serializable_gee_collections_tdocument_test_space_get_name (SerializableGeeCollectionsTDocumentTestSpace* self);
static SerializableGeeCollectionsTDocumentTestSpace* serializable_gee_collections_tdocument_test_space_new_named (const gchar* name);
static SerializableGeeCollectionsTDocumentTestSpace* serializable_gee_collections_tdocument_test_space_construct_named (GType object_type, const gchar* name);
static void serializable_gee_collections_tdocument_test_space_set_name (SerializableGeeCollectionsTDocumentTestSpace* self, const gchar* value);
static gchar* serializable_gee_collections_tdocument_test_space_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_collections_tdocument_test_space_real_to_string (GXmlSerializableObjectModel* base);
static void serializable_gee_collections_tdocument_test_space_real_init_containers (GXmlSerializableContainer* base);
static SerializableGeeCollectionsTDocumentTestAsteroidArray* serializable_gee_collections_tdocument_test_space_get_asteroids (SerializableGeeCollectionsTDocumentTestSpace* self);
static void serializable_gee_collections_tdocument_test_space_set_asteroids (SerializableGeeCollectionsTDocumentTestSpace* self, SerializableGeeCollectionsTDocumentTestAsteroidArray* value);
static SerializableGeeCollectionsTDocumentTestPlanetCollection* serializable_gee_collections_tdocument_test_space_get_planets (SerializableGeeCollectionsTDocumentTestSpace* self);
static void serializable_gee_collections_tdocument_test_space_set_planets (SerializableGeeCollectionsTDocumentTestSpace* self, SerializableGeeCollectionsTDocumentTestPlanetCollection* value);
static SerializableGeeCollectionsTDocumentTestSpace* serializable_gee_collections_tdocument_test_space_new (void);
static SerializableGeeCollectionsTDocumentTestSpace* serializable_gee_collections_tdocument_test_space_construct (GType object_type);
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_COLLECTION_DUMMY_PROPERTY
};
static SerializableGeeCollectionsTDocumentTestSpaceCollection* serializable_gee_collections_tdocument_test_space_collection_new (void);
static SerializableGeeCollectionsTDocumentTestSpaceCollection* serializable_gee_collections_tdocument_test_space_collection_construct (GType object_type);
static void serializable_gee_collections_tdocument_test_space_finalize (GObject* obj);
static void _vala_serializable_gee_collections_tdocument_test_space_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_tdocument_test_space_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_REFACTION, SerializableGeeCollectionsTDocumentTestRefactionPrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_MODEL,
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_MANUFACTURER,
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_SPACESHIPS
};
static SerializableGeeCollectionsTDocumentTestRefaction* serializable_gee_collections_tdocument_test_refaction_new_named (const gchar* manufacturer, const gchar* model);
static SerializableGeeCollectionsTDocumentTestRefaction* serializable_gee_collections_tdocument_test_refaction_construct_named (GType object_type, const gchar* manufacturer, const gchar* model);
static void serializable_gee_collections_tdocument_test_refaction_set_manufacturer (SerializableGeeCollectionsTDocumentTestRefaction* self, const gchar* value);
static void serializable_gee_collections_tdocument_test_refaction_set_model (SerializableGeeCollectionsTDocumentTestRefaction* self, const gchar* value);
static gchar* serializable_gee_collections_tdocument_test_refaction_real_get_map_primary_key (GXmlSerializableMapDualKey* base);
static const gchar* serializable_gee_collections_tdocument_test_refaction_get_manufacturer (SerializableGeeCollectionsTDocumentTestRefaction* self);
static gchar* serializable_gee_collections_tdocument_test_refaction_real_get_map_secondary_key (GXmlSerializableMapDualKey* base);
static const gchar* serializable_gee_collections_tdocument_test_refaction_get_model (SerializableGeeCollectionsTDocumentTestRefaction* self);
static gchar* serializable_gee_collections_tdocument_test_refaction_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_collections_tdocument_test_refaction_real_to_string (GXmlSerializableObjectModel* base);
static gboolean serializable_gee_collections_tdocument_test_refaction_real_get_enable_unknown_serializable_property (GXmlSerializableObjectModel* base);
static void serializable_gee_collections_tdocument_test_refaction_real_init_containers (GXmlSerializableContainer* base);
static SerializableGeeCollectionsTDocumentTestSpaceShipCollection* serializable_gee_collections_tdocument_test_refaction_get_spaceships (SerializableGeeCollectionsTDocumentTestRefaction* self);
static SerializableGeeCollectionsTDocumentTestSpaceShipCollection* serializable_gee_collections_tdocument_test_space_ship_collection_new (void);
static SerializableGeeCollectionsTDocumentTestSpaceShipCollection* serializable_gee_collections_tdocument_test_space_ship_collection_construct (GType object_type);
static void serializable_gee_collections_tdocument_test_refaction_set_spaceships (SerializableGeeCollectionsTDocumentTestRefaction* self, SerializableGeeCollectionsTDocumentTestSpaceShipCollection* value);
static SerializableGeeCollectionsTDocumentTestRefaction* serializable_gee_collections_tdocument_test_refaction_new (void);
static SerializableGeeCollectionsTDocumentTestRefaction* serializable_gee_collections_tdocument_test_refaction_construct (GType object_type);
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_COLLECTION_DUMMY_PROPERTY
};
static SerializableGeeCollectionsTDocumentTestRefactionCollection* serializable_gee_collections_tdocument_test_refaction_collection_new (void);
static SerializableGeeCollectionsTDocumentTestRefactionCollection* serializable_gee_collections_tdocument_test_refaction_collection_construct (GType object_type);
static void serializable_gee_collections_tdocument_test_refaction_finalize (GObject* obj);
static void _vala_serializable_gee_collections_tdocument_test_refaction_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_tdocument_test_refaction_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_SHIP, SerializableGeeCollectionsTDocumentTestSpaceShipPrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_MODEL,
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_MANUFACTURER,
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_SPACES
};
static SerializableGeeCollectionsTDocumentTestSpaceShip* serializable_gee_collections_tdocument_test_space_ship_new_named (const gchar* manufacturer, const gchar* model);
static SerializableGeeCollectionsTDocumentTestSpaceShip* serializable_gee_collections_tdocument_test_space_ship_construct_named (GType object_type, const gchar* manufacturer, const gchar* model);
static void serializable_gee_collections_tdocument_test_space_ship_set_manufacturer (SerializableGeeCollectionsTDocumentTestSpaceShip* self, const gchar* value);
static void serializable_gee_collections_tdocument_test_space_ship_set_model (SerializableGeeCollectionsTDocumentTestSpaceShip* self, const gchar* value);
static gchar* serializable_gee_collections_tdocument_test_space_ship_real_get_map_primary_key (GXmlSerializableMapDualKey* base);
static const gchar* serializable_gee_collections_tdocument_test_space_ship_get_manufacturer (SerializableGeeCollectionsTDocumentTestSpaceShip* self);
static gchar* serializable_gee_collections_tdocument_test_space_ship_real_get_map_secondary_key (GXmlSerializableMapDualKey* base);
static const gchar* serializable_gee_collections_tdocument_test_space_ship_get_model (SerializableGeeCollectionsTDocumentTestSpaceShip* self);
static gchar* serializable_gee_collections_tdocument_test_space_ship_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_collections_tdocument_test_space_ship_real_to_string (GXmlSerializableObjectModel* base);
static gboolean serializable_gee_collections_tdocument_test_space_ship_real_get_enable_unknown_serializable_property (GXmlSerializableObjectModel* base);
static void serializable_gee_collections_tdocument_test_space_ship_real_init_containers (GXmlSerializableContainer* base);
static void serializable_gee_collections_tdocument_test_space_ship_set_spaces (SerializableGeeCollectionsTDocumentTestSpaceShip* self, SerializableGeeCollectionsTDocumentTestSpaceCollection* value);
static SerializableGeeCollectionsTDocumentTestSpaceShip* serializable_gee_collections_tdocument_test_space_ship_new (void);
static SerializableGeeCollectionsTDocumentTestSpaceShip* serializable_gee_collections_tdocument_test_space_ship_construct (GType object_type);
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_COLLECTION_DUMMY_PROPERTY
};
static void serializable_gee_collections_tdocument_test_space_ship_finalize (GObject* obj);
static void _vala_serializable_gee_collections_tdocument_test_space_ship_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_tdocument_test_space_ship_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CHARGE_ZONE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CHARGE_ZONE, SerializableGeeCollectionsTDocumentTestChargeZonePrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CHARGE_ZONE_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CHARGE_ZONE_NAME,
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CHARGE_ZONE_SPACESHIPS
};
static gchar* serializable_gee_collections_tdocument_test_charge_zone_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_collections_tdocument_test_charge_zone_real_to_string (GXmlSerializableObjectModel* base);
static const gchar* serializable_gee_collections_tdocument_test_charge_zone_get_name (SerializableGeeCollectionsTDocumentTestChargeZone* self);
static void serializable_gee_collections_tdocument_test_charge_zone_real_init_containers (GXmlSerializableContainer* base);
static void serializable_gee_collections_tdocument_test_charge_zone_set_spaceships (SerializableGeeCollectionsTDocumentTestChargeZone* self, SerializableGeeCollectionsTDocumentTestSpaceShipCollection* value);
static SerializableGeeCollectionsTDocumentTestChargeZone* serializable_gee_collections_tdocument_test_charge_zone_new (void);
static SerializableGeeCollectionsTDocumentTestChargeZone* serializable_gee_collections_tdocument_test_charge_zone_construct (GType object_type);
static void serializable_gee_collections_tdocument_test_charge_zone_set_name (SerializableGeeCollectionsTDocumentTestChargeZone* self, const gchar* value);
static void serializable_gee_collections_tdocument_test_charge_zone_finalize (GObject* obj);
static void _vala_serializable_gee_collections_tdocument_test_charge_zone_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_tdocument_test_charge_zone_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_STORAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_STORAGE, SerializableGeeCollectionsTDocumentTestStoragePrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_STORAGE_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_STORAGE_NAME,
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_STORAGE_REFACTIONS
};
static gchar* serializable_gee_collections_tdocument_test_storage_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_collections_tdocument_test_storage_real_to_string (GXmlSerializableObjectModel* base);
static const gchar* serializable_gee_collections_tdocument_test_storage_get_name (SerializableGeeCollectionsTDocumentTestStorage* self);
static void serializable_gee_collections_tdocument_test_storage_real_init_containers (GXmlSerializableContainer* base);
static void serializable_gee_collections_tdocument_test_storage_set_refactions (SerializableGeeCollectionsTDocumentTestStorage* self, SerializableGeeCollectionsTDocumentTestRefactionCollection* value);
static SerializableGeeCollectionsTDocumentTestStorage* serializable_gee_collections_tdocument_test_storage_new (void);
static SerializableGeeCollectionsTDocumentTestStorage* serializable_gee_collections_tdocument_test_storage_construct (GType object_type);
static void serializable_gee_collections_tdocument_test_storage_set_name (SerializableGeeCollectionsTDocumentTestStorage* self, const gchar* value);
static void serializable_gee_collections_tdocument_test_storage_finalize (GObject* obj);
static void _vala_serializable_gee_collections_tdocument_test_storage_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_tdocument_test_storage_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_BASE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_BASE, SerializableGeeCollectionsTDocumentTestSpaceBasePrivate))
enum  {
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_BASE_DUMMY_PROPERTY,
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_BASE_NAME,
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_BASE_CHARGE_ZONE,
	SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_BASE_STORAGE
};
static gchar* serializable_gee_collections_tdocument_test_space_base_real_node_name (GXmlSerializableObjectModel* base);
static gchar* serializable_gee_collections_tdocument_test_space_base_real_to_string (GXmlSerializableObjectModel* base);
static const gchar* serializable_gee_collections_tdocument_test_space_base_get_name (SerializableGeeCollectionsTDocumentTestSpaceBase* self);
static void serializable_gee_collections_tdocument_test_space_base_set_name (SerializableGeeCollectionsTDocumentTestSpaceBase* self, const gchar* value);
static void serializable_gee_collections_tdocument_test_space_base_set_charge_zone (SerializableGeeCollectionsTDocumentTestSpaceBase* self, SerializableGeeCollectionsTDocumentTestChargeZone* value);
static void serializable_gee_collections_tdocument_test_space_base_set_storage (SerializableGeeCollectionsTDocumentTestSpaceBase* self, SerializableGeeCollectionsTDocumentTestStorage* value);
static void serializable_gee_collections_tdocument_test_space_base_finalize (GObject* obj);
static void _vala_serializable_gee_collections_tdocument_test_space_base_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_serializable_gee_collections_tdocument_test_space_base_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);


static void __lambda70_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* doc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceBase* sb = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceBase* _tmp1_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceBase* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceBase* _tmp4_ = NULL;
		SerializableGeeCollectionsTDocumentTestChargeZone* _tmp5_ = NULL;
		SerializableGeeCollectionsTDocumentTestChargeZone* _tmp6_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceBase* _tmp7_ = NULL;
		SerializableGeeCollectionsTDocumentTestChargeZone* _tmp8_ = NULL;
		SerializableGeeCollectionsTDocumentTestChargeZone* _tmp9_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp10_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp11_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceBase* _tmp12_ = NULL;
		SerializableGeeCollectionsTDocumentTestChargeZone* _tmp13_ = NULL;
		SerializableGeeCollectionsTDocumentTestChargeZone* _tmp14_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp15_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp16_ = NULL;
		gint _tmp17_ = 0;
		gint _tmp18_ = 0;
		SerializableGeeCollectionsTDocumentTestSpaceShip* sbsh1 = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceBase* _tmp19_ = NULL;
		SerializableGeeCollectionsTDocumentTestChargeZone* _tmp20_ = NULL;
		SerializableGeeCollectionsTDocumentTestChargeZone* _tmp21_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp22_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp23_ = NULL;
		gpointer _tmp24_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceShip* _tmp25_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceShip* _tmp26_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceCollection* _tmp27_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceCollection* _tmp28_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceShip* _tmp29_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceCollection* _tmp30_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceCollection* _tmp31_ = NULL;
		gint _tmp32_ = 0;
		gint _tmp33_ = 0;
		GXmlTDocument* ndoc = NULL;
		GXmlTDocument* _tmp34_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceBase* _tmp35_ = NULL;
		GXmlTDocument* _tmp36_ = NULL;
		GXmlNode* _tmp37_ = NULL;
		GXmlNode* _tmp38_ = NULL;
		GXmlTDocument* _tmp39_ = NULL;
		GXmlNode* _tmp40_ = NULL;
		GXmlNode* _tmp41_ = NULL;
		GXmlNode* _tmp42_ = NULL;
		GXmlTDocument* _tmp43_ = NULL;
		GXmlNode* _tmp44_ = NULL;
		GXmlNode* _tmp45_ = NULL;
		GXmlNode* _tmp46_ = NULL;
		gchar* _tmp47_ = NULL;
		gchar* _tmp48_ = NULL;
		gchar* _tmp49_ = NULL;
		GXmlTDocument* _tmp50_ = NULL;
		GXmlNode* _tmp51_ = NULL;
		GXmlNode* _tmp52_ = NULL;
		GXmlNode* _tmp53_ = NULL;
		GeeBidirList* _tmp54_ = NULL;
		GeeBidirList* _tmp55_ = NULL;
		GeeBidirList* _tmp56_ = NULL;
		gint _tmp57_ = 0;
		gint _tmp58_ = 0;
		GXmlNode* cz = NULL;
		GXmlTDocument* _tmp59_ = NULL;
		GXmlNode* _tmp60_ = NULL;
		GXmlNode* _tmp61_ = NULL;
		GXmlNode* _tmp62_ = NULL;
		GeeBidirList* _tmp63_ = NULL;
		GeeBidirList* _tmp64_ = NULL;
		GeeBidirList* _tmp65_ = NULL;
		gpointer _tmp66_ = NULL;
		GXmlNode* _tmp67_ = NULL;
		GXmlNode* _tmp68_ = NULL;
		GXmlNode* _tmp69_ = NULL;
		gchar* _tmp70_ = NULL;
		gchar* _tmp71_ = NULL;
		gchar* _tmp72_ = NULL;
		gchar* _tmp73_ = NULL;
		gchar* _tmp74_ = NULL;
		GXmlNode* _tmp75_ = NULL;
		GeeMap* _tmp76_ = NULL;
		GeeMap* _tmp77_ = NULL;
		GeeMap* _tmp78_ = NULL;
		gpointer _tmp79_ = NULL;
		GXmlNode* _tmp80_ = NULL;
		gchar* _tmp81_ = NULL;
		gchar* _tmp82_ = NULL;
		gchar* _tmp83_ = NULL;
		GXmlNode* _tmp84_ = NULL;
		GeeBidirList* _tmp85_ = NULL;
		GeeBidirList* _tmp86_ = NULL;
		GeeBidirList* _tmp87_ = NULL;
		gint _tmp88_ = 0;
		gint _tmp89_ = 0;
		gboolean fmactoy = FALSE;
		gboolean fmemphis = FALSE;
		gboolean _tmp652_ = FALSE;
		gboolean _tmp653_ = FALSE;
		GXmlNode* st = NULL;
		GXmlTDocument* _tmp654_ = NULL;
		GXmlNode* _tmp655_ = NULL;
		GXmlNode* _tmp656_ = NULL;
		GXmlNode* _tmp657_ = NULL;
		GeeBidirList* _tmp658_ = NULL;
		GeeBidirList* _tmp659_ = NULL;
		GeeBidirList* _tmp660_ = NULL;
		gpointer _tmp661_ = NULL;
		GXmlNode* _tmp662_ = NULL;
		GXmlNode* _tmp663_ = NULL;
		GXmlNode* _tmp664_ = NULL;
		gchar* _tmp665_ = NULL;
		gchar* _tmp666_ = NULL;
		gchar* _tmp667_ = NULL;
		gchar* _tmp668_ = NULL;
		gchar* _tmp669_ = NULL;
		GXmlNode* _tmp670_ = NULL;
		GeeMap* _tmp671_ = NULL;
		GeeMap* _tmp672_ = NULL;
		GeeMap* _tmp673_ = NULL;
		gpointer _tmp674_ = NULL;
		GXmlNode* _tmp675_ = NULL;
		gchar* _tmp676_ = NULL;
		gchar* _tmp677_ = NULL;
		gchar* _tmp678_ = NULL;
		GXmlNode* _tmp679_ = NULL;
		GeeBidirList* _tmp680_ = NULL;
		GeeBidirList* _tmp681_ = NULL;
		GeeBidirList* _tmp682_ = NULL;
		gint _tmp683_ = 0;
		gint _tmp684_ = 0;
		gboolean fr = FALSE;
		gboolean _tmp832_ = FALSE;
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp0_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n" \
"<base name=\"AlphaOne\" >\n" \
"  <chargezone name=\"A1-1\">\n" \
"    <ship manufacturer=\"MacToy\" model=\"A1234\">\n" \
"      <space name=\"Alpha Centaury\">\n" \
"        <planet name=\"Earth\">\n" \
"          <citizen ctype=\"Human\">1M</citizen>\n" \
"          <citizen ctype=\"Ghost\">10M</citizen>\n" \
"        </planet>\n" \
"        <planet name=\"Platon\" />\n" \
"      </space>\n" \
"      <space name=\"Galax\">\n" \
"        <planet name=\"Saminon\">\n" \
"          <citizen ctype=\"Humanes\">100M</citizen>\n" \
"          <citizen ctype=\"Jeties\">1000M</citizen>\n" \
"        </planet>\n" \
"        <planet name=\"Platon\" />\n" \
"      </space>\n" \
"    </ship>\n" \
"    <ship manufacturer=\"Memphis\" model=\"AB1\">\n" \
"      <space name=\"Beta Centaury\">\n" \
"        <planet name=\"Tronex\">\n" \
"          <citizen ctype=\"Human\">10000M</citizen>\n" \
"          <citizen ctype=\"Cat\">100000M</citizen>\n" \
"        </planet>\n" \
"        <planet name=\"Palax\" />\n" \
"      </space>\n" \
"    </ship>\n" \
"  </chargezone>\n" \
"  <storage name=\"B4-A4\">\n" \
"    <refaction manufacturer=\"MacToy\" model=\"Fly045\">\n" \
"      <ship manufacturer=\"MacToy\" model=\"A1234\" />\n" \
"      <ship manufacturer=\"MegaTrench\" model=\"G045-1\" />\n" \
"    </refaction>\n" \
"  </storage>\n" \
"</base>", 0, &_inner_error_);
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		doc = _tmp0_;
#line 182 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 827 "SerializableGeeCollections-TDocument-Test.c"
			goto __catch98_g_error;
		}
#line 218 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp1_ = serializable_gee_collections_tdocument_test_space_base_new ();
#line 218 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		sb = _tmp1_;
#line 220 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp2_ = sb;
#line 220 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp3_ = doc;
#line 220 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp2_, (GXmlNode*) _tmp3_, &_inner_error_);
#line 220 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 220 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_g_object_unref0 (sb);
#line 220 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_g_object_unref0 (doc);
#line 846 "SerializableGeeCollections-TDocument-Test.c"
			goto __catch98_g_error;
		}
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp4_ = sb;
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp5_ = serializable_gee_collections_tdocument_test_space_base_get_charge_zone (_tmp4_);
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp6_ = _tmp5_;
#line 221 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp6_ != NULL, "sb.charge_zone != null");
#line 222 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp7_ = sb;
#line 222 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp8_ = serializable_gee_collections_tdocument_test_space_base_get_charge_zone (_tmp7_);
#line 222 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp9_ = _tmp8_;
#line 222 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp10_ = serializable_gee_collections_tdocument_test_charge_zone_get_spaceships (_tmp9_);
#line 222 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp11_ = _tmp10_;
#line 222 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp11_ != NULL, "sb.charge_zone.spaceships != null");
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp12_ = sb;
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp13_ = serializable_gee_collections_tdocument_test_space_base_get_charge_zone (_tmp12_);
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp14_ = _tmp13_;
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp15_ = serializable_gee_collections_tdocument_test_charge_zone_get_spaceships (_tmp14_);
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp16_ = _tmp15_;
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp17_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp16_);
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp18_ = _tmp17_;
#line 223 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp18_ == 2, "sb.charge_zone.spaceships.size == 2");
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp19_ = sb;
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp20_ = serializable_gee_collections_tdocument_test_space_base_get_charge_zone (_tmp19_);
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp21_ = _tmp20_;
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp22_ = serializable_gee_collections_tdocument_test_charge_zone_get_spaceships (_tmp21_);
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp23_ = _tmp22_;
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp24_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp23_, "MacToy", "A1234");
#line 224 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		sbsh1 = (SerializableGeeCollectionsTDocumentTestSpaceShip*) _tmp24_;
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp25_ = sbsh1;
#line 225 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp25_ != NULL, "sbsh1 != null");
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp26_ = sbsh1;
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp27_ = serializable_gee_collections_tdocument_test_space_ship_get_spaces (_tmp26_);
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp28_ = _tmp27_;
#line 226 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp28_ != NULL, "sbsh1.spaces != null");
#line 227 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp29_ = sbsh1;
#line 227 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp30_ = serializable_gee_collections_tdocument_test_space_ship_get_spaces (_tmp29_);
#line 227 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp31_ = _tmp30_;
#line 227 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp32_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp31_);
#line 227 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp33_ = _tmp32_;
#line 227 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp33_ == 2, "sbsh1.spaces.size == 2");
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp34_ = gxml_tdocument_new ();
#line 229 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		ndoc = _tmp34_;
#line 230 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp35_ = sb;
#line 230 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp36_ = ndoc;
#line 230 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp37_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) _tmp35_, (GXmlNode*) _tmp36_, &_inner_error_);
#line 230 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp38_ = _tmp37_;
#line 230 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp38_);
#line 230 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 230 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_g_object_unref0 (ndoc);
#line 230 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_g_object_unref0 (sbsh1);
#line 230 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_g_object_unref0 (sb);
#line 230 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_g_object_unref0 (doc);
#line 947 "SerializableGeeCollections-TDocument-Test.c"
			goto __catch98_g_error;
		}
#line 231 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp39_ = ndoc;
#line 231 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp40_ = gxml_document_get_root ((GXmlDocument*) _tmp39_);
#line 231 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp41_ = _tmp40_;
#line 231 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp42_ = _tmp41_;
#line 231 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp42_ != NULL, "ndoc.root != null");
#line 231 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp42_);
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp43_ = ndoc;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp44_ = gxml_document_get_root ((GXmlDocument*) _tmp43_);
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp45_ = _tmp44_;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp46_ = _tmp45_;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp47_ = gxml_node_get_name (_tmp46_);
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp48_ = _tmp47_;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp49_ = _tmp48_;
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (g_strcmp0 (_tmp49_, "base") == 0, "ndoc.root.name == \"base\"");
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_free0 (_tmp49_);
#line 232 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp46_);
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp50_ = ndoc;
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp51_ = gxml_document_get_root ((GXmlDocument*) _tmp50_);
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp52_ = _tmp51_;
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp53_ = _tmp52_;
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp54_ = gxml_node_get_children_nodes (_tmp53_);
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp55_ = _tmp54_;
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp56_ = _tmp55_;
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp57_ = gee_collection_get_size ((GeeCollection*) _tmp56_);
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp58_ = _tmp57_;
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp58_ == 2, "ndoc.root.children_nodes.size == 2");
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp56_);
#line 233 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp53_);
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp59_ = ndoc;
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp60_ = gxml_document_get_root ((GXmlDocument*) _tmp59_);
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp61_ = _tmp60_;
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp62_ = _tmp61_;
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp63_ = gxml_node_get_children_nodes (_tmp62_);
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp64_ = _tmp63_;
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp65_ = _tmp64_;
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp66_ = gee_list_get ((GeeList*) _tmp65_, 0);
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp67_ = (GXmlNode*) _tmp66_;
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp65_);
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp62_);
#line 234 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		cz = _tmp67_;
#line 235 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp68_ = cz;
#line 235 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp68_ != NULL, "cz != null");
#line 236 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp69_ = cz;
#line 236 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp70_ = gxml_node_get_name (_tmp69_);
#line 236 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp71_ = _tmp70_;
#line 236 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp72_ = _tmp71_;
#line 236 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp73_ = g_utf8_strdown (_tmp72_, (gssize) -1);
#line 236 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp74_ = _tmp73_;
#line 236 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (g_strcmp0 (_tmp74_, "chargezone") == 0, "cz.name.down () == \"chargezone\"");
#line 236 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_free0 (_tmp74_);
#line 236 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_free0 (_tmp72_);
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp75_ = cz;
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp76_ = gxml_node_get_attrs (_tmp75_);
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp77_ = _tmp76_;
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp78_ = _tmp77_;
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp79_ = gee_map_get (_tmp78_, "name");
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp80_ = (GXmlNode*) _tmp79_;
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp81_ = gxml_node_get_value (_tmp80_);
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp82_ = _tmp81_;
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp83_ = _tmp82_;
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (g_strcmp0 (_tmp83_, "A1-1") == 0, "cz.attrs.get (\"name\").value == \"A1-1\"");
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_free0 (_tmp83_);
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp80_);
#line 237 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp78_);
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp84_ = cz;
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp85_ = gxml_node_get_children_nodes (_tmp84_);
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp86_ = _tmp85_;
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp87_ = _tmp86_;
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp88_ = gee_collection_get_size ((GeeCollection*) _tmp87_);
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp89_ = _tmp88_;
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp89_ == 2, "cz.children_nodes.size == 2");
#line 238 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp87_);
#line 239 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		fmactoy = FALSE;
#line 240 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		fmemphis = FALSE;
#line 1098 "SerializableGeeCollections-TDocument-Test.c"
		{
			GeeBidirList* _sh_list = NULL;
			GXmlNode* _tmp90_ = NULL;
			GeeBidirList* _tmp91_ = NULL;
			GeeBidirList* _tmp92_ = NULL;
			gint _sh_size = 0;
			GeeBidirList* _tmp93_ = NULL;
			gint _tmp94_ = 0;
			gint _tmp95_ = 0;
			gint _sh_index = 0;
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp90_ = cz;
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp91_ = gxml_node_get_children_nodes (_tmp90_);
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp92_ = _tmp91_;
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_sh_list = _tmp92_;
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp93_ = _sh_list;
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp94_ = gee_collection_get_size ((GeeCollection*) _tmp93_);
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp95_ = _tmp94_;
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_sh_size = _tmp95_;
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_sh_index = -1;
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			while (TRUE) {
#line 1129 "SerializableGeeCollections-TDocument-Test.c"
				gint _tmp96_ = 0;
				gint _tmp97_ = 0;
				gint _tmp98_ = 0;
				GXmlNode* sh = NULL;
				GeeBidirList* _tmp99_ = NULL;
				gint _tmp100_ = 0;
				gpointer _tmp101_ = NULL;
				gboolean _tmp102_ = FALSE;
				GXmlNode* _tmp103_ = NULL;
				gchar* _tmp104_ = NULL;
				gchar* _tmp105_ = NULL;
				gchar* _tmp106_ = NULL;
				gboolean _tmp107_ = FALSE;
				gboolean _tmp438_ = FALSE;
				GXmlNode* _tmp439_ = NULL;
				gchar* _tmp440_ = NULL;
				gchar* _tmp441_ = NULL;
				gchar* _tmp442_ = NULL;
				gboolean _tmp443_ = FALSE;
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp96_ = _sh_index;
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_sh_index = _tmp96_ + 1;
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp97_ = _sh_index;
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp98_ = _sh_size;
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				if (!(_tmp97_ < _tmp98_)) {
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					break;
#line 1161 "SerializableGeeCollections-TDocument-Test.c"
				}
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp99_ = _sh_list;
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp100_ = _sh_index;
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp101_ = gee_list_get ((GeeList*) _tmp99_, _tmp100_);
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				sh = (GXmlNode*) _tmp101_;
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp103_ = sh;
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp104_ = gxml_node_get_name (_tmp103_);
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp105_ = _tmp104_;
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp106_ = _tmp105_;
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp107_ = g_strcmp0 (_tmp106_, "ship") == 0;
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_g_free0 (_tmp106_);
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				if (_tmp107_) {
#line 1185 "SerializableGeeCollections-TDocument-Test.c"
					GXmlNode* _tmp108_ = NULL;
					GeeMap* _tmp109_ = NULL;
					GeeMap* _tmp110_ = NULL;
					GeeMap* _tmp111_ = NULL;
					gpointer _tmp112_ = NULL;
					GXmlNode* _tmp113_ = NULL;
					gchar* _tmp114_ = NULL;
					gchar* _tmp115_ = NULL;
					gchar* _tmp116_ = NULL;
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp108_ = sh;
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp109_ = gxml_node_get_attrs (_tmp108_);
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp110_ = _tmp109_;
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp111_ = _tmp110_;
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp112_ = gee_map_get (_tmp111_, "manufacturer");
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp113_ = (GXmlNode*) _tmp112_;
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp114_ = gxml_node_get_value (_tmp113_);
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp115_ = _tmp114_;
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp116_ = _tmp115_;
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp102_ = g_strcmp0 (_tmp116_, "MacToy") == 0;
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_free0 (_tmp116_);
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_object_unref0 (_tmp113_);
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_object_unref0 (_tmp111_);
#line 1221 "SerializableGeeCollections-TDocument-Test.c"
				} else {
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp102_ = FALSE;
#line 1225 "SerializableGeeCollections-TDocument-Test.c"
				}
#line 242 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				if (_tmp102_) {
#line 1229 "SerializableGeeCollections-TDocument-Test.c"
					GXmlNode* _tmp117_ = NULL;
					GeeMap* _tmp118_ = NULL;
					GeeMap* _tmp119_ = NULL;
					GeeMap* _tmp120_ = NULL;
					gpointer _tmp121_ = NULL;
					GXmlNode* _tmp122_ = NULL;
					gchar* _tmp123_ = NULL;
					gchar* _tmp124_ = NULL;
					gchar* _tmp125_ = NULL;
					GXmlNode* _tmp126_ = NULL;
					GeeMap* _tmp127_ = NULL;
					GeeMap* _tmp128_ = NULL;
					GeeMap* _tmp129_ = NULL;
					gpointer _tmp130_ = NULL;
					GXmlNode* _tmp131_ = NULL;
					gchar* _tmp132_ = NULL;
					gchar* _tmp133_ = NULL;
					gchar* _tmp134_ = NULL;
					GXmlNode* _tmp135_ = NULL;
					GeeBidirList* _tmp136_ = NULL;
					GeeBidirList* _tmp137_ = NULL;
					GeeBidirList* _tmp138_ = NULL;
					gint _tmp139_ = 0;
					gint _tmp140_ = 0;
					gboolean falphac = FALSE;
					gboolean fgalax = FALSE;
					gboolean _tmp437_ = FALSE;
#line 243 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					fmactoy = TRUE;
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp117_ = sh;
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp118_ = gxml_node_get_attrs (_tmp117_);
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp119_ = _tmp118_;
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp120_ = _tmp119_;
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp121_ = gee_map_get (_tmp120_, "manufacturer");
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp122_ = (GXmlNode*) _tmp121_;
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp123_ = gxml_node_get_value (_tmp122_);
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp124_ = _tmp123_;
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp125_ = _tmp124_;
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_vala_assert (g_strcmp0 (_tmp125_, "MacToy") == 0, "sh.attrs.get (\"manufacturer\").value == \"MacToy\"");
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_free0 (_tmp125_);
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_object_unref0 (_tmp122_);
#line 244 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_object_unref0 (_tmp120_);
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp126_ = sh;
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp127_ = gxml_node_get_attrs (_tmp126_);
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp128_ = _tmp127_;
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp129_ = _tmp128_;
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp130_ = gee_map_get (_tmp129_, "model");
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp131_ = (GXmlNode*) _tmp130_;
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp132_ = gxml_node_get_value (_tmp131_);
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp133_ = _tmp132_;
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp134_ = _tmp133_;
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_vala_assert (g_strcmp0 (_tmp134_, "A1234") == 0, "sh.attrs.get (\"model\").value == \"A1234\"");
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_free0 (_tmp134_);
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_object_unref0 (_tmp131_);
#line 245 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_object_unref0 (_tmp129_);
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp135_ = sh;
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp136_ = gxml_node_get_children_nodes (_tmp135_);
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp137_ = _tmp136_;
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp138_ = _tmp137_;
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp139_ = gee_collection_get_size ((GeeCollection*) _tmp138_);
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp140_ = _tmp139_;
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_vala_assert (_tmp140_ == 5, "sh.children_nodes.size == 5");
#line 246 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_object_unref0 (_tmp138_);
#line 247 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					falphac = FALSE;
#line 248 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					fgalax = FALSE;
#line 1331 "SerializableGeeCollections-TDocument-Test.c"
					{
						GeeBidirList* _s_list = NULL;
						GXmlNode* _tmp141_ = NULL;
						GeeBidirList* _tmp142_ = NULL;
						GeeBidirList* _tmp143_ = NULL;
						gint _s_size = 0;
						GeeBidirList* _tmp144_ = NULL;
						gint _tmp145_ = 0;
						gint _tmp146_ = 0;
						gint _s_index = 0;
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_tmp141_ = sh;
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_tmp142_ = gxml_node_get_children_nodes (_tmp141_);
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_tmp143_ = _tmp142_;
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_s_list = _tmp143_;
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_tmp144_ = _s_list;
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_tmp145_ = gee_collection_get_size ((GeeCollection*) _tmp144_);
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_tmp146_ = _tmp145_;
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_s_size = _tmp146_;
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_s_index = -1;
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						while (TRUE) {
#line 1362 "SerializableGeeCollections-TDocument-Test.c"
							gint _tmp147_ = 0;
							gint _tmp148_ = 0;
							gint _tmp149_ = 0;
							GXmlNode* s = NULL;
							GeeBidirList* _tmp150_ = NULL;
							gint _tmp151_ = 0;
							gpointer _tmp152_ = NULL;
							gboolean _tmp153_ = FALSE;
							GXmlNode* _tmp154_ = NULL;
							gchar* _tmp155_ = NULL;
							gchar* _tmp156_ = NULL;
							gchar* _tmp157_ = NULL;
							gboolean _tmp158_ = FALSE;
							gboolean _tmp319_ = FALSE;
							GXmlNode* _tmp320_ = NULL;
							gchar* _tmp321_ = NULL;
							gchar* _tmp322_ = NULL;
							gchar* _tmp323_ = NULL;
							gboolean _tmp324_ = FALSE;
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp147_ = _s_index;
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_s_index = _tmp147_ + 1;
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp148_ = _s_index;
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp149_ = _s_size;
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							if (!(_tmp148_ < _tmp149_)) {
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								break;
#line 1394 "SerializableGeeCollections-TDocument-Test.c"
							}
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp150_ = _s_list;
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp151_ = _s_index;
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp152_ = gee_list_get ((GeeList*) _tmp150_, _tmp151_);
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							s = (GXmlNode*) _tmp152_;
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp154_ = s;
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp155_ = gxml_node_get_name (_tmp154_);
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp156_ = _tmp155_;
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp157_ = _tmp156_;
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp158_ = g_strcmp0 (_tmp157_, "space") == 0;
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_g_free0 (_tmp157_);
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							if (_tmp158_) {
#line 1418 "SerializableGeeCollections-TDocument-Test.c"
								GXmlNode* _tmp159_ = NULL;
								GeeMap* _tmp160_ = NULL;
								GeeMap* _tmp161_ = NULL;
								GeeMap* _tmp162_ = NULL;
								gpointer _tmp163_ = NULL;
								GXmlNode* _tmp164_ = NULL;
								gchar* _tmp165_ = NULL;
								gchar* _tmp166_ = NULL;
								gchar* _tmp167_ = NULL;
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp159_ = s;
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp160_ = gxml_node_get_attrs (_tmp159_);
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp161_ = _tmp160_;
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp162_ = _tmp161_;
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp163_ = gee_map_get (_tmp162_, "name");
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp164_ = (GXmlNode*) _tmp163_;
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp165_ = gxml_node_get_value (_tmp164_);
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp166_ = _tmp165_;
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp167_ = _tmp166_;
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp153_ = g_strcmp0 (_tmp167_, "Alpha Centaury") == 0;
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_free0 (_tmp167_);
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp164_);
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp162_);
#line 1454 "SerializableGeeCollections-TDocument-Test.c"
							} else {
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp153_ = FALSE;
#line 1458 "SerializableGeeCollections-TDocument-Test.c"
							}
#line 250 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							if (_tmp153_) {
#line 1462 "SerializableGeeCollections-TDocument-Test.c"
								GXmlNode* _tmp168_ = NULL;
								GeeMap* _tmp169_ = NULL;
								GeeMap* _tmp170_ = NULL;
								GeeMap* _tmp171_ = NULL;
								gpointer _tmp172_ = NULL;
								GXmlNode* _tmp173_ = NULL;
								gchar* _tmp174_ = NULL;
								gchar* _tmp175_ = NULL;
								gchar* _tmp176_ = NULL;
								GXmlNode* _tmp177_ = NULL;
								GeeBidirList* _tmp178_ = NULL;
								GeeBidirList* _tmp179_ = NULL;
								GeeBidirList* _tmp180_ = NULL;
								gint _tmp181_ = 0;
								gint _tmp182_ = 0;
								gboolean fearth = FALSE;
								gboolean fplaton = FALSE;
								gboolean _tmp317_ = FALSE;
								gboolean _tmp318_ = FALSE;
#line 251 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								falphac = TRUE;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp168_ = s;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp169_ = gxml_node_get_attrs (_tmp168_);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp170_ = _tmp169_;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp171_ = _tmp170_;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp172_ = gee_map_get (_tmp171_, "name");
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp173_ = (GXmlNode*) _tmp172_;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp174_ = gxml_node_get_value (_tmp173_);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp175_ = _tmp174_;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp176_ = _tmp175_;
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_vala_assert (g_strcmp0 (_tmp176_, "Alpha Centaury") == 0, "s.attrs.get (\"name\").value == \"Alpha Centaury\"");
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_free0 (_tmp176_);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp173_);
#line 252 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp171_);
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp177_ = s;
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp178_ = gxml_node_get_children_nodes (_tmp177_);
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp179_ = _tmp178_;
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp180_ = _tmp179_;
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp181_ = gee_collection_get_size ((GeeCollection*) _tmp180_);
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp182_ = _tmp181_;
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_vala_assert (_tmp182_ == 2, "s.children_nodes.size == 2");
#line 253 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp180_);
#line 254 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								fearth = FALSE;
#line 255 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								fplaton = FALSE;
#line 1530 "SerializableGeeCollections-TDocument-Test.c"
								{
									GeeBidirList* _p1_list = NULL;
									GXmlNode* _tmp183_ = NULL;
									GeeBidirList* _tmp184_ = NULL;
									GeeBidirList* _tmp185_ = NULL;
									gint _p1_size = 0;
									GeeBidirList* _tmp186_ = NULL;
									gint _tmp187_ = 0;
									gint _tmp188_ = 0;
									gint _p1_index = 0;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp183_ = s;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp184_ = gxml_node_get_children_nodes (_tmp183_);
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp185_ = _tmp184_;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_p1_list = _tmp185_;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp186_ = _p1_list;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp187_ = gee_collection_get_size ((GeeCollection*) _tmp186_);
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp188_ = _tmp187_;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_p1_size = _tmp188_;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_p1_index = -1;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									while (TRUE) {
#line 1561 "SerializableGeeCollections-TDocument-Test.c"
										gint _tmp189_ = 0;
										gint _tmp190_ = 0;
										gint _tmp191_ = 0;
										GXmlNode* p1 = NULL;
										GeeBidirList* _tmp192_ = NULL;
										gint _tmp193_ = 0;
										gpointer _tmp194_ = NULL;
										gboolean _tmp195_ = FALSE;
										GXmlNode* _tmp196_ = NULL;
										gchar* _tmp197_ = NULL;
										gchar* _tmp198_ = NULL;
										gchar* _tmp199_ = NULL;
										gboolean _tmp200_ = FALSE;
										gboolean _tmp283_ = FALSE;
										GXmlNode* _tmp284_ = NULL;
										gchar* _tmp285_ = NULL;
										gchar* _tmp286_ = NULL;
										gchar* _tmp287_ = NULL;
										gboolean _tmp288_ = FALSE;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp189_ = _p1_index;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_p1_index = _tmp189_ + 1;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp190_ = _p1_index;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp191_ = _p1_size;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										if (!(_tmp190_ < _tmp191_)) {
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											break;
#line 1593 "SerializableGeeCollections-TDocument-Test.c"
										}
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp192_ = _p1_list;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp193_ = _p1_index;
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp194_ = gee_list_get ((GeeList*) _tmp192_, _tmp193_);
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										p1 = (GXmlNode*) _tmp194_;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp196_ = p1;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp197_ = gxml_node_get_name (_tmp196_);
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp198_ = _tmp197_;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp199_ = _tmp198_;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp200_ = g_strcmp0 (_tmp199_, "planet") == 0;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_g_free0 (_tmp199_);
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										if (_tmp200_) {
#line 1617 "SerializableGeeCollections-TDocument-Test.c"
											GXmlNode* _tmp201_ = NULL;
											GeeMap* _tmp202_ = NULL;
											GeeMap* _tmp203_ = NULL;
											GeeMap* _tmp204_ = NULL;
											gpointer _tmp205_ = NULL;
											GXmlNode* _tmp206_ = NULL;
											gchar* _tmp207_ = NULL;
											gchar* _tmp208_ = NULL;
											gchar* _tmp209_ = NULL;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp201_ = p1;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp202_ = gxml_node_get_attrs (_tmp201_);
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp203_ = _tmp202_;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp204_ = _tmp203_;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp205_ = gee_map_get (_tmp204_, "name");
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp206_ = (GXmlNode*) _tmp205_;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp207_ = gxml_node_get_value (_tmp206_);
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp208_ = _tmp207_;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp209_ = _tmp208_;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp195_ = g_strcmp0 (_tmp209_, "Earth") == 0;
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp209_);
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp206_);
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp204_);
#line 1653 "SerializableGeeCollections-TDocument-Test.c"
										} else {
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp195_ = FALSE;
#line 1657 "SerializableGeeCollections-TDocument-Test.c"
										}
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										if (_tmp195_) {
#line 1661 "SerializableGeeCollections-TDocument-Test.c"
											GXmlNode* _tmp210_ = NULL;
											gchar* _tmp211_ = NULL;
											gchar* _tmp212_ = NULL;
											gchar* _tmp213_ = NULL;
											GXmlNode* _tmp214_ = NULL;
											GeeMap* _tmp215_ = NULL;
											GeeMap* _tmp216_ = NULL;
											GeeMap* _tmp217_ = NULL;
											gpointer _tmp218_ = NULL;
											GXmlNode* _tmp219_ = NULL;
											gchar* _tmp220_ = NULL;
											gchar* _tmp221_ = NULL;
											gchar* _tmp222_ = NULL;
											GXmlNode* _tmp223_ = NULL;
											GeeBidirList* _tmp224_ = NULL;
											GeeBidirList* _tmp225_ = NULL;
											GeeBidirList* _tmp226_ = NULL;
											gint _tmp227_ = 0;
											gint _tmp228_ = 0;
											GXmlNode* c1 = NULL;
											GXmlNode* _tmp229_ = NULL;
											GeeBidirList* _tmp230_ = NULL;
											GeeBidirList* _tmp231_ = NULL;
											GeeBidirList* _tmp232_ = NULL;
											gpointer _tmp233_ = NULL;
											GXmlNode* _tmp234_ = NULL;
											GXmlNode* _tmp235_ = NULL;
											GXmlNode* _tmp236_ = NULL;
											gchar* _tmp237_ = NULL;
											gchar* _tmp238_ = NULL;
											gchar* _tmp239_ = NULL;
											GXmlNode* _tmp240_ = NULL;
											GeeMap* _tmp241_ = NULL;
											GeeMap* _tmp242_ = NULL;
											GeeMap* _tmp243_ = NULL;
											gpointer _tmp244_ = NULL;
											GXmlNode* _tmp245_ = NULL;
											gchar* _tmp246_ = NULL;
											gchar* _tmp247_ = NULL;
											gchar* _tmp248_ = NULL;
											GXmlNode* _tmp249_ = NULL;
											gchar* _tmp250_ = NULL;
											gchar* _tmp251_ = NULL;
											gchar* _tmp252_ = NULL;
											GXmlNode* _tmp253_ = NULL;
											GeeBidirList* _tmp254_ = NULL;
											GeeBidirList* _tmp255_ = NULL;
											GeeBidirList* _tmp256_ = NULL;
											gint _tmp257_ = 0;
											gint _tmp258_ = 0;
											GXmlNode* c2 = NULL;
											GXmlNode* _tmp259_ = NULL;
											GeeBidirList* _tmp260_ = NULL;
											GeeBidirList* _tmp261_ = NULL;
											GeeBidirList* _tmp262_ = NULL;
											gpointer _tmp263_ = NULL;
											GXmlNode* _tmp264_ = NULL;
											GXmlNode* _tmp265_ = NULL;
											GXmlNode* _tmp266_ = NULL;
											gchar* _tmp267_ = NULL;
											gchar* _tmp268_ = NULL;
											gchar* _tmp269_ = NULL;
											GXmlNode* _tmp270_ = NULL;
											GeeMap* _tmp271_ = NULL;
											GeeMap* _tmp272_ = NULL;
											GeeMap* _tmp273_ = NULL;
											gpointer _tmp274_ = NULL;
											GXmlNode* _tmp275_ = NULL;
											gchar* _tmp276_ = NULL;
											gchar* _tmp277_ = NULL;
											gchar* _tmp278_ = NULL;
											GXmlNode* _tmp279_ = NULL;
											gchar* _tmp280_ = NULL;
											gchar* _tmp281_ = NULL;
											gchar* _tmp282_ = NULL;
#line 258 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											fearth = TRUE;
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp210_ = p1;
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp211_ = gxml_node_get_name (_tmp210_);
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp212_ = _tmp211_;
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp213_ = _tmp212_;
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp213_, "planet") == 0, "p1.name == \"planet\"");
#line 259 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp213_);
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp214_ = p1;
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp215_ = gxml_node_get_attrs (_tmp214_);
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp216_ = _tmp215_;
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp217_ = _tmp216_;
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp218_ = gee_map_get (_tmp217_, "name");
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp219_ = (GXmlNode*) _tmp218_;
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp220_ = gxml_node_get_value (_tmp219_);
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp221_ = _tmp220_;
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp222_ = _tmp221_;
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp222_, "Earth") == 0, "p1.attrs.get (\"name\").value == \"Earth\"");
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp222_);
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp219_);
#line 260 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp217_);
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp223_ = p1;
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp224_ = gxml_node_get_children_nodes (_tmp223_);
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp225_ = _tmp224_;
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp226_ = _tmp225_;
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp227_ = gee_collection_get_size ((GeeCollection*) _tmp226_);
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp228_ = _tmp227_;
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (_tmp228_ == 2, "p1.children_nodes.size == 2");
#line 261 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp226_);
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp229_ = p1;
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp230_ = gxml_node_get_children_nodes (_tmp229_);
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp231_ = _tmp230_;
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp232_ = _tmp231_;
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp233_ = gee_list_get ((GeeList*) _tmp232_, 0);
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp234_ = (GXmlNode*) _tmp233_;
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp232_);
#line 262 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											c1 = _tmp234_;
#line 263 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp235_ = c1;
#line 263 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (_tmp235_ != NULL, "c1 != null");
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp236_ = c1;
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp237_ = gxml_node_get_name (_tmp236_);
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp238_ = _tmp237_;
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp239_ = _tmp238_;
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp239_, "citizen") == 0, "c1.name == \"citizen\"");
#line 264 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp239_);
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp240_ = c1;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp241_ = gxml_node_get_attrs (_tmp240_);
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp242_ = _tmp241_;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp243_ = _tmp242_;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp244_ = gee_map_get (_tmp243_, "ctype");
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp245_ = (GXmlNode*) _tmp244_;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp246_ = gxml_node_get_value (_tmp245_);
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp247_ = _tmp246_;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp248_ = _tmp247_;
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp248_, "Human") == 0, "c1.attrs.get (\"ctype\").value == \"Human\"");
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp248_);
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp245_);
#line 265 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp243_);
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp249_ = c1;
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp250_ = gxml_element_get_content (G_TYPE_CHECK_INSTANCE_CAST (_tmp249_, GXML_TYPE_ELEMENT, GXmlElement));
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp251_ = _tmp250_;
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp252_ = _tmp251_;
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp252_, "1M") == 0, "((Element)c1).content == \"1M\"");
#line 266 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp252_);
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp253_ = c1;
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp254_ = gxml_node_get_children_nodes (_tmp253_);
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp255_ = _tmp254_;
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp256_ = _tmp255_;
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp257_ = gee_collection_get_size ((GeeCollection*) _tmp256_);
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp258_ = _tmp257_;
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (_tmp258_ == 1, "c1.children_nodes.size == 1");
#line 267 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp256_);
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp259_ = p1;
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp260_ = gxml_node_get_children_nodes (_tmp259_);
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp261_ = _tmp260_;
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp262_ = _tmp261_;
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp263_ = gee_list_get ((GeeList*) _tmp262_, 1);
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp264_ = (GXmlNode*) _tmp263_;
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp262_);
#line 268 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											c2 = _tmp264_;
#line 269 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp265_ = c2;
#line 269 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (_tmp265_ != NULL, "c2 != null");
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp266_ = c2;
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp267_ = gxml_node_get_name (_tmp266_);
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp268_ = _tmp267_;
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp269_ = _tmp268_;
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp269_, "citizen") == 0, "c2.name == \"citizen\"");
#line 270 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp269_);
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp270_ = c2;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp271_ = gxml_node_get_attrs (_tmp270_);
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp272_ = _tmp271_;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp273_ = _tmp272_;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp274_ = gee_map_get (_tmp273_, "ctype");
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp275_ = (GXmlNode*) _tmp274_;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp276_ = gxml_node_get_value (_tmp275_);
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp277_ = _tmp276_;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp278_ = _tmp277_;
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp278_, "Ghost") == 0, "c2.attrs.get (\"ctype\").value == \"Ghost\"");
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp278_);
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp275_);
#line 271 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp273_);
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp279_ = c2;
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp280_ = gxml_element_get_content (G_TYPE_CHECK_INSTANCE_CAST (_tmp279_, GXML_TYPE_ELEMENT, GXmlElement));
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp281_ = _tmp280_;
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp282_ = _tmp281_;
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp282_, "10M") == 0, "((Element) c2).content == \"10M\"");
#line 272 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp282_);
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (c2);
#line 257 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (c1);
#line 1953 "SerializableGeeCollections-TDocument-Test.c"
										}
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp284_ = p1;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp285_ = gxml_node_get_name (_tmp284_);
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp286_ = _tmp285_;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp287_ = _tmp286_;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp288_ = g_strcmp0 (_tmp287_, "planet") == 0;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_g_free0 (_tmp287_);
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										if (_tmp288_) {
#line 1969 "SerializableGeeCollections-TDocument-Test.c"
											GXmlNode* _tmp289_ = NULL;
											GeeMap* _tmp290_ = NULL;
											GeeMap* _tmp291_ = NULL;
											GeeMap* _tmp292_ = NULL;
											gpointer _tmp293_ = NULL;
											GXmlNode* _tmp294_ = NULL;
											gchar* _tmp295_ = NULL;
											gchar* _tmp296_ = NULL;
											gchar* _tmp297_ = NULL;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp289_ = p1;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp290_ = gxml_node_get_attrs (_tmp289_);
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp291_ = _tmp290_;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp292_ = _tmp291_;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp293_ = gee_map_get (_tmp292_, "name");
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp294_ = (GXmlNode*) _tmp293_;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp295_ = gxml_node_get_value (_tmp294_);
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp296_ = _tmp295_;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp297_ = _tmp296_;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp283_ = g_strcmp0 (_tmp297_, "Platon") == 0;
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp297_);
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp294_);
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp292_);
#line 2005 "SerializableGeeCollections-TDocument-Test.c"
										} else {
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp283_ = FALSE;
#line 2009 "SerializableGeeCollections-TDocument-Test.c"
										}
#line 274 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										if (_tmp283_) {
#line 2013 "SerializableGeeCollections-TDocument-Test.c"
											GXmlNode* _tmp298_ = NULL;
											gchar* _tmp299_ = NULL;
											gchar* _tmp300_ = NULL;
											gchar* _tmp301_ = NULL;
											GXmlNode* _tmp302_ = NULL;
											GeeMap* _tmp303_ = NULL;
											GeeMap* _tmp304_ = NULL;
											GeeMap* _tmp305_ = NULL;
											gpointer _tmp306_ = NULL;
											GXmlNode* _tmp307_ = NULL;
											gchar* _tmp308_ = NULL;
											gchar* _tmp309_ = NULL;
											gchar* _tmp310_ = NULL;
											GXmlNode* _tmp311_ = NULL;
											GeeBidirList* _tmp312_ = NULL;
											GeeBidirList* _tmp313_ = NULL;
											GeeBidirList* _tmp314_ = NULL;
											gint _tmp315_ = 0;
											gint _tmp316_ = 0;
#line 275 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											fplaton = TRUE;
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp298_ = p1;
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp299_ = gxml_node_get_name (_tmp298_);
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp300_ = _tmp299_;
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp301_ = _tmp300_;
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp301_, "planet") == 0, "p1.name == \"planet\"");
#line 276 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp301_);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp302_ = p1;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp303_ = gxml_node_get_attrs (_tmp302_);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp304_ = _tmp303_;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp305_ = _tmp304_;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp306_ = gee_map_get (_tmp305_, "name");
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp307_ = (GXmlNode*) _tmp306_;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp308_ = gxml_node_get_value (_tmp307_);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp309_ = _tmp308_;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp310_ = _tmp309_;
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp310_, "Platon") == 0, "p1.attrs.get (\"name\").value == \"Platon\"");
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp310_);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp307_);
#line 277 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp305_);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp311_ = p1;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp312_ = gxml_node_get_children_nodes (_tmp311_);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp313_ = _tmp312_;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp314_ = _tmp313_;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp315_ = gee_collection_get_size ((GeeCollection*) _tmp314_);
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp316_ = _tmp315_;
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (_tmp316_ == 0, "p1.children_nodes.size == 0");
#line 278 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp314_);
#line 2089 "SerializableGeeCollections-TDocument-Test.c"
										}
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_g_object_unref0 (p1);
#line 2093 "SerializableGeeCollections-TDocument-Test.c"
									}
#line 256 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_g_object_unref0 (_p1_list);
#line 2097 "SerializableGeeCollections-TDocument-Test.c"
								}
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp317_ = fearth;
#line 281 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_vala_assert (_tmp317_, "fearth");
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp318_ = fplaton;
#line 282 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_vala_assert (_tmp318_, "fplaton");
#line 2107 "SerializableGeeCollections-TDocument-Test.c"
							}
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp320_ = s;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp321_ = gxml_node_get_name (_tmp320_);
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp322_ = _tmp321_;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp323_ = _tmp322_;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp324_ = g_strcmp0 (_tmp323_, "space") == 0;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_g_free0 (_tmp323_);
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							if (_tmp324_) {
#line 2123 "SerializableGeeCollections-TDocument-Test.c"
								GXmlNode* _tmp325_ = NULL;
								GeeMap* _tmp326_ = NULL;
								GeeMap* _tmp327_ = NULL;
								GeeMap* _tmp328_ = NULL;
								gpointer _tmp329_ = NULL;
								GXmlNode* _tmp330_ = NULL;
								gchar* _tmp331_ = NULL;
								gchar* _tmp332_ = NULL;
								gchar* _tmp333_ = NULL;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp325_ = s;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp326_ = gxml_node_get_attrs (_tmp325_);
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp327_ = _tmp326_;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp328_ = _tmp327_;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp329_ = gee_map_get (_tmp328_, "name");
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp330_ = (GXmlNode*) _tmp329_;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp331_ = gxml_node_get_value (_tmp330_);
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp332_ = _tmp331_;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp333_ = _tmp332_;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp319_ = g_strcmp0 (_tmp333_, "Galax") == 0;
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_free0 (_tmp333_);
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp330_);
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp328_);
#line 2159 "SerializableGeeCollections-TDocument-Test.c"
							} else {
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp319_ = FALSE;
#line 2163 "SerializableGeeCollections-TDocument-Test.c"
							}
#line 284 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							if (_tmp319_) {
#line 2167 "SerializableGeeCollections-TDocument-Test.c"
								GXmlNode* _tmp334_ = NULL;
								GeeMap* _tmp335_ = NULL;
								GeeMap* _tmp336_ = NULL;
								GeeMap* _tmp337_ = NULL;
								gpointer _tmp338_ = NULL;
								GXmlNode* _tmp339_ = NULL;
								gchar* _tmp340_ = NULL;
								gchar* _tmp341_ = NULL;
								gchar* _tmp342_ = NULL;
								GXmlNode* _tmp343_ = NULL;
								GeeBidirList* _tmp344_ = NULL;
								GeeBidirList* _tmp345_ = NULL;
								GeeBidirList* _tmp346_ = NULL;
								gint _tmp347_ = 0;
								gint _tmp348_ = 0;
								gboolean fsaminon = FALSE;
								gboolean _tmp436_ = FALSE;
#line 285 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								fgalax = TRUE;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp334_ = s;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp335_ = gxml_node_get_attrs (_tmp334_);
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp336_ = _tmp335_;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp337_ = _tmp336_;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp338_ = gee_map_get (_tmp337_, "name");
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp339_ = (GXmlNode*) _tmp338_;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp340_ = gxml_node_get_value (_tmp339_);
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp341_ = _tmp340_;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp342_ = _tmp341_;
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_vala_assert (g_strcmp0 (_tmp342_, "Galax") == 0, "s.attrs.get (\"name\").value == \"Galax\"");
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_free0 (_tmp342_);
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp339_);
#line 286 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp337_);
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp343_ = s;
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp344_ = gxml_node_get_children_nodes (_tmp343_);
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp345_ = _tmp344_;
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp346_ = _tmp345_;
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp347_ = gee_collection_get_size ((GeeCollection*) _tmp346_);
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp348_ = _tmp347_;
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_vala_assert (_tmp348_ == 2, "s.children_nodes.size == 2");
#line 287 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp346_);
#line 288 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								fsaminon = FALSE;
#line 2231 "SerializableGeeCollections-TDocument-Test.c"
								{
									GeeBidirList* _p_list = NULL;
									GXmlNode* _tmp349_ = NULL;
									GeeBidirList* _tmp350_ = NULL;
									GeeBidirList* _tmp351_ = NULL;
									gint _p_size = 0;
									GeeBidirList* _tmp352_ = NULL;
									gint _tmp353_ = 0;
									gint _tmp354_ = 0;
									gint _p_index = 0;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp349_ = s;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp350_ = gxml_node_get_children_nodes (_tmp349_);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp351_ = _tmp350_;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_p_list = _tmp351_;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp352_ = _p_list;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp353_ = gee_collection_get_size ((GeeCollection*) _tmp352_);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp354_ = _tmp353_;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_p_size = _tmp354_;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_p_index = -1;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									while (TRUE) {
#line 2262 "SerializableGeeCollections-TDocument-Test.c"
										gint _tmp355_ = 0;
										gint _tmp356_ = 0;
										gint _tmp357_ = 0;
										GXmlNode* p = NULL;
										GeeBidirList* _tmp358_ = NULL;
										gint _tmp359_ = 0;
										gpointer _tmp360_ = NULL;
										gboolean _tmp361_ = FALSE;
										GXmlNode* _tmp362_ = NULL;
										gchar* _tmp363_ = NULL;
										gchar* _tmp364_ = NULL;
										gchar* _tmp365_ = NULL;
										gboolean _tmp366_ = FALSE;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp355_ = _p_index;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_p_index = _tmp355_ + 1;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp356_ = _p_index;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp357_ = _p_size;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										if (!(_tmp356_ < _tmp357_)) {
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											break;
#line 2288 "SerializableGeeCollections-TDocument-Test.c"
										}
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp358_ = _p_list;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp359_ = _p_index;
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp360_ = gee_list_get ((GeeList*) _tmp358_, _tmp359_);
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										p = (GXmlNode*) _tmp360_;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp362_ = p;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp363_ = gxml_node_get_name (_tmp362_);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp364_ = _tmp363_;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp365_ = _tmp364_;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp366_ = g_strcmp0 (_tmp365_, "planet") == 0;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_g_free0 (_tmp365_);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										if (_tmp366_) {
#line 2312 "SerializableGeeCollections-TDocument-Test.c"
											GXmlNode* _tmp367_ = NULL;
											GeeMap* _tmp368_ = NULL;
											GeeMap* _tmp369_ = NULL;
											GeeMap* _tmp370_ = NULL;
											gpointer _tmp371_ = NULL;
											GXmlNode* _tmp372_ = NULL;
											gchar* _tmp373_ = NULL;
											gchar* _tmp374_ = NULL;
											gchar* _tmp375_ = NULL;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp367_ = p;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp368_ = gxml_node_get_attrs (_tmp367_);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp369_ = _tmp368_;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp370_ = _tmp369_;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp371_ = gee_map_get (_tmp370_, "name");
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp372_ = (GXmlNode*) _tmp371_;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp373_ = gxml_node_get_value (_tmp372_);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp374_ = _tmp373_;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp375_ = _tmp374_;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp361_ = g_strcmp0 (_tmp375_, "Saminon") == 0;
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp375_);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp372_);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp370_);
#line 2348 "SerializableGeeCollections-TDocument-Test.c"
										} else {
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp361_ = FALSE;
#line 2352 "SerializableGeeCollections-TDocument-Test.c"
										}
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										if (_tmp361_) {
#line 2356 "SerializableGeeCollections-TDocument-Test.c"
											GXmlNode* h = NULL;
											GXmlNode* _tmp376_ = NULL;
											GeeBidirList* _tmp377_ = NULL;
											GeeBidirList* _tmp378_ = NULL;
											GeeBidirList* _tmp379_ = NULL;
											gpointer _tmp380_ = NULL;
											GXmlNode* _tmp381_ = NULL;
											GXmlNode* _tmp382_ = NULL;
											GXmlNode* _tmp383_ = NULL;
											gchar* _tmp384_ = NULL;
											gchar* _tmp385_ = NULL;
											gchar* _tmp386_ = NULL;
											GXmlNode* _tmp387_ = NULL;
											GeeMap* _tmp388_ = NULL;
											GeeMap* _tmp389_ = NULL;
											GeeMap* _tmp390_ = NULL;
											gpointer _tmp391_ = NULL;
											GXmlNode* _tmp392_ = NULL;
											gchar* _tmp393_ = NULL;
											gchar* _tmp394_ = NULL;
											gchar* _tmp395_ = NULL;
											GXmlNode* _tmp396_ = NULL;
											GeeBidirList* _tmp397_ = NULL;
											GeeBidirList* _tmp398_ = NULL;
											GeeBidirList* _tmp399_ = NULL;
											gint _tmp400_ = 0;
											gint _tmp401_ = 0;
											GXmlNode* _tmp402_ = NULL;
											gchar* _tmp403_ = NULL;
											gchar* _tmp404_ = NULL;
											gchar* _tmp405_ = NULL;
											GXmlNode* j = NULL;
											GXmlNode* _tmp406_ = NULL;
											GeeBidirList* _tmp407_ = NULL;
											GeeBidirList* _tmp408_ = NULL;
											GeeBidirList* _tmp409_ = NULL;
											gpointer _tmp410_ = NULL;
											GXmlNode* _tmp411_ = NULL;
											GXmlNode* _tmp412_ = NULL;
											GXmlNode* _tmp413_ = NULL;
											gchar* _tmp414_ = NULL;
											gchar* _tmp415_ = NULL;
											gchar* _tmp416_ = NULL;
											GXmlNode* _tmp417_ = NULL;
											GeeMap* _tmp418_ = NULL;
											GeeMap* _tmp419_ = NULL;
											GeeMap* _tmp420_ = NULL;
											gpointer _tmp421_ = NULL;
											GXmlNode* _tmp422_ = NULL;
											gchar* _tmp423_ = NULL;
											gchar* _tmp424_ = NULL;
											gchar* _tmp425_ = NULL;
											GXmlNode* _tmp426_ = NULL;
											GeeBidirList* _tmp427_ = NULL;
											GeeBidirList* _tmp428_ = NULL;
											GeeBidirList* _tmp429_ = NULL;
											gint _tmp430_ = 0;
											gint _tmp431_ = 0;
											GXmlNode* _tmp432_ = NULL;
											gchar* _tmp433_ = NULL;
											gchar* _tmp434_ = NULL;
											gchar* _tmp435_ = NULL;
#line 291 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											fsaminon = TRUE;
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp376_ = p;
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp377_ = gxml_node_get_children_nodes (_tmp376_);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp378_ = _tmp377_;
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp379_ = _tmp378_;
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp380_ = gee_list_get ((GeeList*) _tmp379_, 0);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp381_ = (GXmlNode*) _tmp380_;
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp379_);
#line 292 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											h = _tmp381_;
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp382_ = h;
#line 293 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (_tmp382_ != NULL, "h != null");
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp383_ = h;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp384_ = gxml_node_get_name (_tmp383_);
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp385_ = _tmp384_;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp386_ = _tmp385_;
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp386_, "citizen") == 0, "h.name == \"citizen\"");
#line 294 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp386_);
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp387_ = h;
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp388_ = gxml_node_get_attrs (_tmp387_);
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp389_ = _tmp388_;
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp390_ = _tmp389_;
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp391_ = gee_map_get (_tmp390_, "ctype");
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp392_ = (GXmlNode*) _tmp391_;
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp393_ = gxml_node_get_value (_tmp392_);
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp394_ = _tmp393_;
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp395_ = _tmp394_;
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp395_, "Humanes") == 0, "h.attrs.get (\"ctype\").value == \"Humanes\"");
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp395_);
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp392_);
#line 295 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp390_);
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp396_ = h;
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp397_ = gxml_node_get_children_nodes (_tmp396_);
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp398_ = _tmp397_;
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp399_ = _tmp398_;
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp400_ = gee_collection_get_size ((GeeCollection*) _tmp399_);
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp401_ = _tmp400_;
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (_tmp401_ == 1, "h.children_nodes.size == 1");
#line 296 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp399_);
#line 297 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp402_ = h;
#line 297 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp403_ = gxml_element_get_content (G_TYPE_CHECK_INSTANCE_CAST (_tmp402_, GXML_TYPE_ELEMENT, GXmlElement));
#line 297 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp404_ = _tmp403_;
#line 297 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp405_ = _tmp404_;
#line 297 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp405_, "100M") == 0, "((Element) h).content == \"100M\"");
#line 297 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp405_);
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp406_ = p;
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp407_ = gxml_node_get_children_nodes (_tmp406_);
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp408_ = _tmp407_;
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp409_ = _tmp408_;
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp410_ = gee_list_get ((GeeList*) _tmp409_, 1);
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp411_ = (GXmlNode*) _tmp410_;
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp409_);
#line 298 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											j = _tmp411_;
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp412_ = j;
#line 299 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (_tmp412_ != NULL, "j != null");
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp413_ = j;
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp414_ = gxml_node_get_name (_tmp413_);
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp415_ = _tmp414_;
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp416_ = _tmp415_;
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp416_, "citizen") == 0, "j.name == \"citizen\"");
#line 300 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp416_);
#line 301 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp417_ = j;
#line 301 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp418_ = gxml_node_get_attrs (_tmp417_);
#line 301 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp419_ = _tmp418_;
#line 301 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp420_ = _tmp419_;
#line 301 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp421_ = gee_map_get (_tmp420_, "ctype");
#line 301 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp422_ = (GXmlNode*) _tmp421_;
#line 301 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp423_ = gxml_node_get_value (_tmp422_);
#line 301 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp424_ = _tmp423_;
#line 301 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp425_ = _tmp424_;
#line 301 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp425_, "Jeties") == 0, "j.attrs.get (\"ctype\").value == \"Jeties\"");
#line 301 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp425_);
#line 301 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp422_);
#line 301 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp420_);
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp426_ = j;
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp427_ = gxml_node_get_children_nodes (_tmp426_);
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp428_ = _tmp427_;
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp429_ = _tmp428_;
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp430_ = gee_collection_get_size ((GeeCollection*) _tmp429_);
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp431_ = _tmp430_;
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (_tmp431_ == 1, "j.children_nodes.size == 1");
#line 302 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp429_);
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp432_ = j;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp433_ = gxml_element_get_content (G_TYPE_CHECK_INSTANCE_CAST (_tmp432_, GXML_TYPE_ELEMENT, GXmlElement));
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp434_ = _tmp433_;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp435_ = _tmp434_;
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp435_, "1000M") == 0, "((Element) j).content == \"1000M\"");
#line 303 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp435_);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (j);
#line 290 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (h);
#line 2597 "SerializableGeeCollections-TDocument-Test.c"
										}
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_g_object_unref0 (p);
#line 2601 "SerializableGeeCollections-TDocument-Test.c"
									}
#line 289 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_g_object_unref0 (_p_list);
#line 2605 "SerializableGeeCollections-TDocument-Test.c"
								}
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp436_ = fsaminon;
#line 306 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_vala_assert (_tmp436_, "fsaminon");
#line 2611 "SerializableGeeCollections-TDocument-Test.c"
							}
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_g_object_unref0 (s);
#line 2615 "SerializableGeeCollections-TDocument-Test.c"
						}
#line 249 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_g_object_unref0 (_s_list);
#line 2619 "SerializableGeeCollections-TDocument-Test.c"
					}
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp437_ = falphac;
#line 309 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_vala_assert (_tmp437_, "falphac");
#line 2625 "SerializableGeeCollections-TDocument-Test.c"
				}
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp439_ = sh;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp440_ = gxml_node_get_name (_tmp439_);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp441_ = _tmp440_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp442_ = _tmp441_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp443_ = g_strcmp0 (_tmp442_, "ship") == 0;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_g_free0 (_tmp442_);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				if (_tmp443_) {
#line 2641 "SerializableGeeCollections-TDocument-Test.c"
					GXmlNode* _tmp444_ = NULL;
					GeeMap* _tmp445_ = NULL;
					GeeMap* _tmp446_ = NULL;
					GeeMap* _tmp447_ = NULL;
					gpointer _tmp448_ = NULL;
					GXmlNode* _tmp449_ = NULL;
					gchar* _tmp450_ = NULL;
					gchar* _tmp451_ = NULL;
					gchar* _tmp452_ = NULL;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp444_ = sh;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp445_ = gxml_node_get_attrs (_tmp444_);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp446_ = _tmp445_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp447_ = _tmp446_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp448_ = gee_map_get (_tmp447_, "manufacturer");
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp449_ = (GXmlNode*) _tmp448_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp450_ = gxml_node_get_value (_tmp449_);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp451_ = _tmp450_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp452_ = _tmp451_;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp438_ = g_strcmp0 (_tmp452_, "Memphis") == 0;
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_free0 (_tmp452_);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_object_unref0 (_tmp449_);
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_object_unref0 (_tmp447_);
#line 2677 "SerializableGeeCollections-TDocument-Test.c"
				} else {
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp438_ = FALSE;
#line 2681 "SerializableGeeCollections-TDocument-Test.c"
				}
#line 311 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				if (_tmp438_) {
#line 2685 "SerializableGeeCollections-TDocument-Test.c"
					GXmlNode* _tmp453_ = NULL;
					gchar* _tmp454_ = NULL;
					gchar* _tmp455_ = NULL;
					gchar* _tmp456_ = NULL;
					GXmlNode* _tmp457_ = NULL;
					GeeMap* _tmp458_ = NULL;
					GeeMap* _tmp459_ = NULL;
					GeeMap* _tmp460_ = NULL;
					gpointer _tmp461_ = NULL;
					GXmlNode* _tmp462_ = NULL;
					gchar* _tmp463_ = NULL;
					gchar* _tmp464_ = NULL;
					gchar* _tmp465_ = NULL;
					GXmlNode* _tmp466_ = NULL;
					GeeMap* _tmp467_ = NULL;
					GeeMap* _tmp468_ = NULL;
					GeeMap* _tmp469_ = NULL;
					gpointer _tmp470_ = NULL;
					GXmlNode* _tmp471_ = NULL;
					gchar* _tmp472_ = NULL;
					gchar* _tmp473_ = NULL;
					gchar* _tmp474_ = NULL;
					GXmlNode* _tmp475_ = NULL;
					GeeBidirList* _tmp476_ = NULL;
					GeeBidirList* _tmp477_ = NULL;
					GeeBidirList* _tmp478_ = NULL;
					gint _tmp479_ = 0;
					gint _tmp480_ = 0;
					gboolean fbetac = FALSE;
#line 312 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					fmemphis = TRUE;
#line 313 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp453_ = sh;
#line 313 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp454_ = gxml_node_get_name (_tmp453_);
#line 313 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp455_ = _tmp454_;
#line 313 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp456_ = _tmp455_;
#line 313 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_vala_assert (g_strcmp0 (_tmp456_, "ship") == 0, "sh.name == \"ship\"");
#line 313 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_free0 (_tmp456_);
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp457_ = sh;
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp458_ = gxml_node_get_attrs (_tmp457_);
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp459_ = _tmp458_;
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp460_ = _tmp459_;
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp461_ = gee_map_get (_tmp460_, "manufacturer");
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp462_ = (GXmlNode*) _tmp461_;
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp463_ = gxml_node_get_value (_tmp462_);
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp464_ = _tmp463_;
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp465_ = _tmp464_;
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_vala_assert (g_strcmp0 (_tmp465_, "Memphis") == 0, "sh.attrs.get (\"manufacturer\").value == \"Memphis\"");
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_free0 (_tmp465_);
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_object_unref0 (_tmp462_);
#line 314 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_object_unref0 (_tmp460_);
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp466_ = sh;
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp467_ = gxml_node_get_attrs (_tmp466_);
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp468_ = _tmp467_;
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp469_ = _tmp468_;
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp470_ = gee_map_get (_tmp469_, "model");
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp471_ = (GXmlNode*) _tmp470_;
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp472_ = gxml_node_get_value (_tmp471_);
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp473_ = _tmp472_;
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp474_ = _tmp473_;
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_vala_assert (g_strcmp0 (_tmp474_, "AB1") == 0, "sh.attrs.get (\"model\").value == \"AB1\"");
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_free0 (_tmp474_);
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_object_unref0 (_tmp471_);
#line 315 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_object_unref0 (_tmp469_);
#line 316 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp475_ = sh;
#line 316 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp476_ = gxml_node_get_children_nodes (_tmp475_);
#line 316 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp477_ = _tmp476_;
#line 316 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp478_ = _tmp477_;
#line 316 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp479_ = gee_collection_get_size ((GeeCollection*) _tmp478_);
#line 316 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp480_ = _tmp479_;
#line 316 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_vala_assert (_tmp480_ == 3, "sh.children_nodes.size == 3");
#line 316 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_object_unref0 (_tmp478_);
#line 317 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					fbetac = FALSE;
#line 2799 "SerializableGeeCollections-TDocument-Test.c"
					{
						GeeBidirList* _s_list = NULL;
						GXmlNode* _tmp481_ = NULL;
						GeeBidirList* _tmp482_ = NULL;
						GeeBidirList* _tmp483_ = NULL;
						gint _s_size = 0;
						GeeBidirList* _tmp484_ = NULL;
						gint _tmp485_ = 0;
						gint _tmp486_ = 0;
						gint _s_index = 0;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_tmp481_ = sh;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_tmp482_ = gxml_node_get_children_nodes (_tmp481_);
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_tmp483_ = _tmp482_;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_s_list = _tmp483_;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_tmp484_ = _s_list;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_tmp485_ = gee_collection_get_size ((GeeCollection*) _tmp484_);
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_tmp486_ = _tmp485_;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_s_size = _tmp486_;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_s_index = -1;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						while (TRUE) {
#line 2830 "SerializableGeeCollections-TDocument-Test.c"
							gint _tmp487_ = 0;
							gint _tmp488_ = 0;
							gint _tmp489_ = 0;
							GXmlNode* s = NULL;
							GeeBidirList* _tmp490_ = NULL;
							gint _tmp491_ = 0;
							gpointer _tmp492_ = NULL;
							gboolean _tmp493_ = FALSE;
							GXmlNode* _tmp494_ = NULL;
							gchar* _tmp495_ = NULL;
							gchar* _tmp496_ = NULL;
							gchar* _tmp497_ = NULL;
							gboolean _tmp498_ = FALSE;
							gboolean _tmp651_ = FALSE;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp487_ = _s_index;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_s_index = _tmp487_ + 1;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp488_ = _s_index;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp489_ = _s_size;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							if (!(_tmp488_ < _tmp489_)) {
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								break;
#line 2857 "SerializableGeeCollections-TDocument-Test.c"
							}
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp490_ = _s_list;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp491_ = _s_index;
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp492_ = gee_list_get ((GeeList*) _tmp490_, _tmp491_);
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							s = (GXmlNode*) _tmp492_;
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp494_ = s;
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp495_ = gxml_node_get_name (_tmp494_);
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp496_ = _tmp495_;
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp497_ = _tmp496_;
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp498_ = g_strcmp0 (_tmp497_, "space") == 0;
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_g_free0 (_tmp497_);
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							if (_tmp498_) {
#line 2881 "SerializableGeeCollections-TDocument-Test.c"
								GXmlNode* _tmp499_ = NULL;
								GeeMap* _tmp500_ = NULL;
								GeeMap* _tmp501_ = NULL;
								GeeMap* _tmp502_ = NULL;
								gpointer _tmp503_ = NULL;
								GXmlNode* _tmp504_ = NULL;
								gchar* _tmp505_ = NULL;
								gchar* _tmp506_ = NULL;
								gchar* _tmp507_ = NULL;
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp499_ = s;
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp500_ = gxml_node_get_attrs (_tmp499_);
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp501_ = _tmp500_;
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp502_ = _tmp501_;
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp503_ = gee_map_get (_tmp502_, "name");
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp504_ = (GXmlNode*) _tmp503_;
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp505_ = gxml_node_get_value (_tmp504_);
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp506_ = _tmp505_;
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp507_ = _tmp506_;
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp493_ = g_strcmp0 (_tmp507_, "Beta Centaury") == 0;
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_free0 (_tmp507_);
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp504_);
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp502_);
#line 2917 "SerializableGeeCollections-TDocument-Test.c"
							} else {
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp493_ = FALSE;
#line 2921 "SerializableGeeCollections-TDocument-Test.c"
							}
#line 319 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							if (_tmp493_) {
#line 2925 "SerializableGeeCollections-TDocument-Test.c"
								GXmlNode* _tmp508_ = NULL;
								GeeMap* _tmp509_ = NULL;
								GeeMap* _tmp510_ = NULL;
								GeeMap* _tmp511_ = NULL;
								gpointer _tmp512_ = NULL;
								GXmlNode* _tmp513_ = NULL;
								gchar* _tmp514_ = NULL;
								gchar* _tmp515_ = NULL;
								gchar* _tmp516_ = NULL;
								GXmlNode* _tmp517_ = NULL;
								GeeBidirList* _tmp518_ = NULL;
								GeeBidirList* _tmp519_ = NULL;
								GeeBidirList* _tmp520_ = NULL;
								gint _tmp521_ = 0;
								gint _tmp522_ = 0;
								gboolean ftronex = FALSE;
								gboolean fpalax = FALSE;
								gboolean _tmp649_ = FALSE;
								gboolean _tmp650_ = FALSE;
#line 320 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								fbetac = TRUE;
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp508_ = s;
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp509_ = gxml_node_get_attrs (_tmp508_);
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp510_ = _tmp509_;
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp511_ = _tmp510_;
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp512_ = gee_map_get (_tmp511_, "name");
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp513_ = (GXmlNode*) _tmp512_;
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp514_ = gxml_node_get_value (_tmp513_);
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp515_ = _tmp514_;
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp516_ = _tmp515_;
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_vala_assert (g_strcmp0 (_tmp516_, "Beta Centaury") == 0, "s.attrs.get (\"name\").value == \"Beta Centaury\"");
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_free0 (_tmp516_);
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp513_);
#line 321 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp511_);
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp517_ = s;
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp518_ = gxml_node_get_children_nodes (_tmp517_);
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp519_ = _tmp518_;
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp520_ = _tmp519_;
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp521_ = gee_collection_get_size ((GeeCollection*) _tmp520_);
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp522_ = _tmp521_;
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_vala_assert (_tmp522_ == 2, "s.children_nodes.size == 2");
#line 322 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp520_);
#line 323 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								ftronex = FALSE;
#line 324 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								fpalax = FALSE;
#line 2993 "SerializableGeeCollections-TDocument-Test.c"
								{
									GeeBidirList* _p_list = NULL;
									GXmlNode* _tmp523_ = NULL;
									GeeBidirList* _tmp524_ = NULL;
									GeeBidirList* _tmp525_ = NULL;
									gint _p_size = 0;
									GeeBidirList* _tmp526_ = NULL;
									gint _tmp527_ = 0;
									gint _tmp528_ = 0;
									gint _p_index = 0;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp523_ = s;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp524_ = gxml_node_get_children_nodes (_tmp523_);
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp525_ = _tmp524_;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_p_list = _tmp525_;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp526_ = _p_list;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp527_ = gee_collection_get_size ((GeeCollection*) _tmp526_);
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp528_ = _tmp527_;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_p_size = _tmp528_;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_p_index = -1;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									while (TRUE) {
#line 3024 "SerializableGeeCollections-TDocument-Test.c"
										gint _tmp529_ = 0;
										gint _tmp530_ = 0;
										gint _tmp531_ = 0;
										GXmlNode* p = NULL;
										GeeBidirList* _tmp532_ = NULL;
										gint _tmp533_ = 0;
										gpointer _tmp534_ = NULL;
										gboolean _tmp535_ = FALSE;
										GXmlNode* _tmp536_ = NULL;
										gchar* _tmp537_ = NULL;
										gchar* _tmp538_ = NULL;
										gchar* _tmp539_ = NULL;
										gboolean _tmp540_ = FALSE;
										gboolean _tmp615_ = FALSE;
										GXmlNode* _tmp616_ = NULL;
										gchar* _tmp617_ = NULL;
										gchar* _tmp618_ = NULL;
										gchar* _tmp619_ = NULL;
										gboolean _tmp620_ = FALSE;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp529_ = _p_index;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_p_index = _tmp529_ + 1;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp530_ = _p_index;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp531_ = _p_size;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										if (!(_tmp530_ < _tmp531_)) {
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											break;
#line 3056 "SerializableGeeCollections-TDocument-Test.c"
										}
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp532_ = _p_list;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp533_ = _p_index;
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp534_ = gee_list_get ((GeeList*) _tmp532_, _tmp533_);
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										p = (GXmlNode*) _tmp534_;
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp536_ = p;
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp537_ = gxml_node_get_name (_tmp536_);
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp538_ = _tmp537_;
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp539_ = _tmp538_;
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp540_ = g_strcmp0 (_tmp539_, "planet") == 0;
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_g_free0 (_tmp539_);
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										if (_tmp540_) {
#line 3080 "SerializableGeeCollections-TDocument-Test.c"
											GXmlNode* _tmp541_ = NULL;
											GeeMap* _tmp542_ = NULL;
											GeeMap* _tmp543_ = NULL;
											GeeMap* _tmp544_ = NULL;
											gpointer _tmp545_ = NULL;
											GXmlNode* _tmp546_ = NULL;
											gchar* _tmp547_ = NULL;
											gchar* _tmp548_ = NULL;
											gchar* _tmp549_ = NULL;
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp541_ = p;
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp542_ = gxml_node_get_attrs (_tmp541_);
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp543_ = _tmp542_;
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp544_ = _tmp543_;
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp545_ = gee_map_get (_tmp544_, "name");
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp546_ = (GXmlNode*) _tmp545_;
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp547_ = gxml_node_get_value (_tmp546_);
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp548_ = _tmp547_;
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp549_ = _tmp548_;
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp535_ = g_strcmp0 (_tmp549_, "Tronex") == 0;
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp549_);
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp546_);
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp544_);
#line 3116 "SerializableGeeCollections-TDocument-Test.c"
										} else {
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp535_ = FALSE;
#line 3120 "SerializableGeeCollections-TDocument-Test.c"
										}
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										if (_tmp535_) {
#line 3124 "SerializableGeeCollections-TDocument-Test.c"
											GXmlNode* _tmp550_ = NULL;
											gchar* _tmp551_ = NULL;
											gchar* _tmp552_ = NULL;
											gchar* _tmp553_ = NULL;
											GXmlNode* _tmp554_ = NULL;
											GeeMap* _tmp555_ = NULL;
											GeeMap* _tmp556_ = NULL;
											GeeMap* _tmp557_ = NULL;
											gpointer _tmp558_ = NULL;
											GXmlNode* _tmp559_ = NULL;
											gchar* _tmp560_ = NULL;
											gchar* _tmp561_ = NULL;
											gchar* _tmp562_ = NULL;
											GXmlNode* _tmp563_ = NULL;
											GeeBidirList* _tmp564_ = NULL;
											GeeBidirList* _tmp565_ = NULL;
											GeeBidirList* _tmp566_ = NULL;
											gint _tmp567_ = 0;
											gint _tmp568_ = 0;
											GXmlNode* cp = NULL;
											GXmlNode* _tmp569_ = NULL;
											GeeBidirList* _tmp570_ = NULL;
											GeeBidirList* _tmp571_ = NULL;
											GeeBidirList* _tmp572_ = NULL;
											gpointer _tmp573_ = NULL;
											GXmlNode* _tmp574_ = NULL;
											GXmlNode* _tmp575_ = NULL;
											gchar* _tmp576_ = NULL;
											gchar* _tmp577_ = NULL;
											gchar* _tmp578_ = NULL;
											GXmlNode* _tmp579_ = NULL;
											GeeMap* _tmp580_ = NULL;
											GeeMap* _tmp581_ = NULL;
											GeeMap* _tmp582_ = NULL;
											gpointer _tmp583_ = NULL;
											GXmlNode* _tmp584_ = NULL;
											gchar* _tmp585_ = NULL;
											gchar* _tmp586_ = NULL;
											gchar* _tmp587_ = NULL;
											GXmlNode* _tmp588_ = NULL;
											gchar* _tmp589_ = NULL;
											gchar* _tmp590_ = NULL;
											gchar* _tmp591_ = NULL;
											GXmlNode* cp2 = NULL;
											GXmlNode* _tmp592_ = NULL;
											GeeBidirList* _tmp593_ = NULL;
											GeeBidirList* _tmp594_ = NULL;
											GeeBidirList* _tmp595_ = NULL;
											gpointer _tmp596_ = NULL;
											GXmlNode* _tmp597_ = NULL;
											GXmlNode* _tmp598_ = NULL;
											gchar* _tmp599_ = NULL;
											gchar* _tmp600_ = NULL;
											gchar* _tmp601_ = NULL;
											GXmlNode* _tmp602_ = NULL;
											GeeMap* _tmp603_ = NULL;
											GeeMap* _tmp604_ = NULL;
											GeeMap* _tmp605_ = NULL;
											gpointer _tmp606_ = NULL;
											GXmlNode* _tmp607_ = NULL;
											gchar* _tmp608_ = NULL;
											gchar* _tmp609_ = NULL;
											gchar* _tmp610_ = NULL;
											GXmlNode* _tmp611_ = NULL;
											gchar* _tmp612_ = NULL;
											gchar* _tmp613_ = NULL;
											gchar* _tmp614_ = NULL;
#line 327 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											ftronex = TRUE;
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp550_ = p;
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp551_ = gxml_node_get_name (_tmp550_);
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp552_ = _tmp551_;
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp553_ = _tmp552_;
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp553_, "planet") == 0, "p.name == \"planet\"");
#line 328 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp553_);
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp554_ = p;
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp555_ = gxml_node_get_attrs (_tmp554_);
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp556_ = _tmp555_;
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp557_ = _tmp556_;
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp558_ = gee_map_get (_tmp557_, "name");
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp559_ = (GXmlNode*) _tmp558_;
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp560_ = gxml_node_get_value (_tmp559_);
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp561_ = _tmp560_;
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp562_ = _tmp561_;
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp562_, "Tronex") == 0, "p.attrs.get (\"name\").value == \"Tronex\"");
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp562_);
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp559_);
#line 329 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp557_);
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp563_ = p;
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp564_ = gxml_node_get_children_nodes (_tmp563_);
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp565_ = _tmp564_;
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp566_ = _tmp565_;
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp567_ = gee_collection_get_size ((GeeCollection*) _tmp566_);
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp568_ = _tmp567_;
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (_tmp568_ == 2, "p.children_nodes.size == 2");
#line 330 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp566_);
#line 331 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp569_ = p;
#line 331 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp570_ = gxml_node_get_children_nodes (_tmp569_);
#line 331 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp571_ = _tmp570_;
#line 331 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp572_ = _tmp571_;
#line 331 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp573_ = gee_list_get ((GeeList*) _tmp572_, 0);
#line 331 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp574_ = (GXmlNode*) _tmp573_;
#line 331 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp572_);
#line 331 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											cp = _tmp574_;
#line 332 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp575_ = cp;
#line 332 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp576_ = gxml_node_get_name (_tmp575_);
#line 332 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp577_ = _tmp576_;
#line 332 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp578_ = _tmp577_;
#line 332 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp578_, "citizen") == 0, "cp.name == \"citizen\"");
#line 332 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp578_);
#line 333 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp579_ = cp;
#line 333 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp580_ = gxml_node_get_attrs (_tmp579_);
#line 333 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp581_ = _tmp580_;
#line 333 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp582_ = _tmp581_;
#line 333 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp583_ = gee_map_get (_tmp582_, "ctype");
#line 333 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp584_ = (GXmlNode*) _tmp583_;
#line 333 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp585_ = gxml_node_get_value (_tmp584_);
#line 333 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp586_ = _tmp585_;
#line 333 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp587_ = _tmp586_;
#line 333 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp587_, "Human") == 0, "cp.attrs.get (\"ctype\").value == \"Human\"");
#line 333 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp587_);
#line 333 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp584_);
#line 333 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp582_);
#line 334 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp588_ = cp;
#line 334 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp589_ = gxml_element_get_content (G_TYPE_CHECK_INSTANCE_CAST (_tmp588_, GXML_TYPE_ELEMENT, GXmlElement));
#line 334 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp590_ = _tmp589_;
#line 334 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp591_ = _tmp590_;
#line 334 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp591_, "10000M") == 0, "((Element)cp).content == \"10000M\"");
#line 334 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp591_);
#line 335 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp592_ = p;
#line 335 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp593_ = gxml_node_get_children_nodes (_tmp592_);
#line 335 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp594_ = _tmp593_;
#line 335 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp595_ = _tmp594_;
#line 335 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp596_ = gee_list_get ((GeeList*) _tmp595_, 1);
#line 335 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp597_ = (GXmlNode*) _tmp596_;
#line 335 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp595_);
#line 335 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											cp2 = _tmp597_;
#line 336 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp598_ = cp2;
#line 336 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp599_ = gxml_node_get_name (_tmp598_);
#line 336 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp600_ = _tmp599_;
#line 336 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp601_ = _tmp600_;
#line 336 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp601_, "citizen") == 0, "cp2.name == \"citizen\"");
#line 336 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp601_);
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp602_ = cp2;
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp603_ = gxml_node_get_attrs (_tmp602_);
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp604_ = _tmp603_;
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp605_ = _tmp604_;
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp606_ = gee_map_get (_tmp605_, "ctype");
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp607_ = (GXmlNode*) _tmp606_;
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp608_ = gxml_node_get_value (_tmp607_);
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp609_ = _tmp608_;
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp610_ = _tmp609_;
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp610_, "Cat") == 0, "cp2.attrs.get (\"ctype\").value == \"Cat\"");
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp610_);
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp607_);
#line 337 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp605_);
#line 338 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp611_ = cp2;
#line 338 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp612_ = gxml_element_get_content (G_TYPE_CHECK_INSTANCE_CAST (_tmp611_, GXML_TYPE_ELEMENT, GXmlElement));
#line 338 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp613_ = _tmp612_;
#line 338 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp614_ = _tmp613_;
#line 338 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp614_, "100000M") == 0, "((Element)cp2).content == \"100000M\"");
#line 338 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp614_);
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (cp2);
#line 326 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (cp);
#line 3384 "SerializableGeeCollections-TDocument-Test.c"
										}
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp616_ = p;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp617_ = gxml_node_get_name (_tmp616_);
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp618_ = _tmp617_;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp619_ = _tmp618_;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_tmp620_ = g_strcmp0 (_tmp619_, "planet") == 0;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_g_free0 (_tmp619_);
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										if (_tmp620_) {
#line 3400 "SerializableGeeCollections-TDocument-Test.c"
											GXmlNode* _tmp621_ = NULL;
											GeeMap* _tmp622_ = NULL;
											GeeMap* _tmp623_ = NULL;
											GeeMap* _tmp624_ = NULL;
											gpointer _tmp625_ = NULL;
											GXmlNode* _tmp626_ = NULL;
											gchar* _tmp627_ = NULL;
											gchar* _tmp628_ = NULL;
											gchar* _tmp629_ = NULL;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp621_ = p;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp622_ = gxml_node_get_attrs (_tmp621_);
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp623_ = _tmp622_;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp624_ = _tmp623_;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp625_ = gee_map_get (_tmp624_, "name");
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp626_ = (GXmlNode*) _tmp625_;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp627_ = gxml_node_get_value (_tmp626_);
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp628_ = _tmp627_;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp629_ = _tmp628_;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp615_ = g_strcmp0 (_tmp629_, "Palax") == 0;
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp629_);
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp626_);
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp624_);
#line 3436 "SerializableGeeCollections-TDocument-Test.c"
										} else {
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp615_ = FALSE;
#line 3440 "SerializableGeeCollections-TDocument-Test.c"
										}
#line 340 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										if (_tmp615_) {
#line 3444 "SerializableGeeCollections-TDocument-Test.c"
											GXmlNode* _tmp630_ = NULL;
											gchar* _tmp631_ = NULL;
											gchar* _tmp632_ = NULL;
											gchar* _tmp633_ = NULL;
											GXmlNode* _tmp634_ = NULL;
											GeeMap* _tmp635_ = NULL;
											GeeMap* _tmp636_ = NULL;
											GeeMap* _tmp637_ = NULL;
											gpointer _tmp638_ = NULL;
											GXmlNode* _tmp639_ = NULL;
											gchar* _tmp640_ = NULL;
											gchar* _tmp641_ = NULL;
											gchar* _tmp642_ = NULL;
											GXmlNode* _tmp643_ = NULL;
											GeeBidirList* _tmp644_ = NULL;
											GeeBidirList* _tmp645_ = NULL;
											GeeBidirList* _tmp646_ = NULL;
											gint _tmp647_ = 0;
											gint _tmp648_ = 0;
#line 341 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											fpalax = TRUE;
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp630_ = p;
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp631_ = gxml_node_get_name (_tmp630_);
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp632_ = _tmp631_;
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp633_ = _tmp632_;
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp633_, "planet") == 0, "p.name == \"planet\"");
#line 342 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp633_);
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp634_ = p;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp635_ = gxml_node_get_attrs (_tmp634_);
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp636_ = _tmp635_;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp637_ = _tmp636_;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp638_ = gee_map_get (_tmp637_, "name");
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp639_ = (GXmlNode*) _tmp638_;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp640_ = gxml_node_get_value (_tmp639_);
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp641_ = _tmp640_;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp642_ = _tmp641_;
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (g_strcmp0 (_tmp642_, "Palax") == 0, "p.attrs.get (\"name\").value == \"Palax\"");
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_free0 (_tmp642_);
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp639_);
#line 343 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp637_);
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp643_ = p;
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp644_ = gxml_node_get_children_nodes (_tmp643_);
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp645_ = _tmp644_;
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp646_ = _tmp645_;
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp647_ = gee_collection_get_size ((GeeCollection*) _tmp646_);
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp648_ = _tmp647_;
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_vala_assert (_tmp648_ == 0, "p.children_nodes.size == 0");
#line 344 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_tmp646_);
#line 3520 "SerializableGeeCollections-TDocument-Test.c"
										}
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
										_g_object_unref0 (p);
#line 3524 "SerializableGeeCollections-TDocument-Test.c"
									}
#line 325 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_g_object_unref0 (_p_list);
#line 3528 "SerializableGeeCollections-TDocument-Test.c"
								}
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp649_ = ftronex;
#line 347 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_vala_assert (_tmp649_, "ftronex");
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp650_ = fpalax;
#line 348 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_vala_assert (_tmp650_, "fpalax");
#line 3538 "SerializableGeeCollections-TDocument-Test.c"
							}
#line 350 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp651_ = fbetac;
#line 350 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_vala_assert (_tmp651_, "fbetac");
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_g_object_unref0 (s);
#line 3546 "SerializableGeeCollections-TDocument-Test.c"
						}
#line 318 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_g_object_unref0 (_s_list);
#line 3550 "SerializableGeeCollections-TDocument-Test.c"
					}
				}
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_g_object_unref0 (sh);
#line 3555 "SerializableGeeCollections-TDocument-Test.c"
			}
#line 241 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_g_object_unref0 (_sh_list);
#line 3559 "SerializableGeeCollections-TDocument-Test.c"
		}
#line 354 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp652_ = fmactoy;
#line 354 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp652_, "fmactoy");
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp653_ = fmemphis;
#line 355 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp653_, "fmemphis");
#line 356 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp654_ = ndoc;
#line 356 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp655_ = gxml_document_get_root ((GXmlDocument*) _tmp654_);
#line 356 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp656_ = _tmp655_;
#line 356 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp657_ = _tmp656_;
#line 356 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp658_ = gxml_node_get_children_nodes (_tmp657_);
#line 356 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp659_ = _tmp658_;
#line 356 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp660_ = _tmp659_;
#line 356 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp661_ = gee_list_get ((GeeList*) _tmp660_, 1);
#line 356 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp662_ = (GXmlNode*) _tmp661_;
#line 356 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp660_);
#line 356 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp657_);
#line 356 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		st = _tmp662_;
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp663_ = st;
#line 357 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp663_ != NULL, "st != null");
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp664_ = st;
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp665_ = gxml_node_get_name (_tmp664_);
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp666_ = _tmp665_;
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp667_ = _tmp666_;
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp668_ = g_utf8_strdown (_tmp667_, (gssize) -1);
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp669_ = _tmp668_;
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (g_strcmp0 (_tmp669_, "storage") == 0, "st.name.down () == \"storage\"");
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_free0 (_tmp669_);
#line 358 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_free0 (_tmp667_);
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp670_ = st;
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp671_ = gxml_node_get_attrs (_tmp670_);
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp672_ = _tmp671_;
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp673_ = _tmp672_;
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp674_ = gee_map_get (_tmp673_, "name");
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp675_ = (GXmlNode*) _tmp674_;
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp676_ = gxml_node_get_value (_tmp675_);
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp677_ = _tmp676_;
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp678_ = _tmp677_;
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (g_strcmp0 (_tmp678_, "B4-A4") == 0, "st.attrs.get (\"name\").value == \"B4-A4\"");
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_free0 (_tmp678_);
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp675_);
#line 359 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp673_);
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp679_ = st;
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp680_ = gxml_node_get_children_nodes (_tmp679_);
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp681_ = _tmp680_;
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp682_ = _tmp681_;
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp683_ = gee_collection_get_size ((GeeCollection*) _tmp682_);
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp684_ = _tmp683_;
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp684_ == 1, "st.children_nodes.size == 1");
#line 360 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp682_);
#line 361 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		fr = FALSE;
#line 3659 "SerializableGeeCollections-TDocument-Test.c"
		{
			GeeBidirList* _r_list = NULL;
			GXmlNode* _tmp685_ = NULL;
			GeeBidirList* _tmp686_ = NULL;
			GeeBidirList* _tmp687_ = NULL;
			gint _r_size = 0;
			GeeBidirList* _tmp688_ = NULL;
			gint _tmp689_ = 0;
			gint _tmp690_ = 0;
			gint _r_index = 0;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp685_ = st;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp686_ = gxml_node_get_children_nodes (_tmp685_);
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp687_ = _tmp686_;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_r_list = _tmp687_;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp688_ = _r_list;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp689_ = gee_collection_get_size ((GeeCollection*) _tmp688_);
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp690_ = _tmp689_;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_r_size = _tmp690_;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_r_index = -1;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			while (TRUE) {
#line 3690 "SerializableGeeCollections-TDocument-Test.c"
				gint _tmp691_ = 0;
				gint _tmp692_ = 0;
				gint _tmp693_ = 0;
				GXmlNode* r = NULL;
				GeeBidirList* _tmp694_ = NULL;
				gint _tmp695_ = 0;
				gpointer _tmp696_ = NULL;
				gboolean _tmp697_ = FALSE;
				GXmlNode* _tmp698_ = NULL;
				gchar* _tmp699_ = NULL;
				gchar* _tmp700_ = NULL;
				gchar* _tmp701_ = NULL;
				gboolean _tmp702_ = FALSE;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp691_ = _r_index;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_r_index = _tmp691_ + 1;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp692_ = _r_index;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp693_ = _r_size;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				if (!(_tmp692_ < _tmp693_)) {
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					break;
#line 3716 "SerializableGeeCollections-TDocument-Test.c"
				}
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp694_ = _r_list;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp695_ = _r_index;
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp696_ = gee_list_get ((GeeList*) _tmp694_, _tmp695_);
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				r = (GXmlNode*) _tmp696_;
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp698_ = r;
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp699_ = gxml_node_get_name (_tmp698_);
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp700_ = _tmp699_;
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp701_ = _tmp700_;
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp702_ = g_strcmp0 (_tmp701_, "refaction") == 0;
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_g_free0 (_tmp701_);
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				if (_tmp702_) {
#line 3740 "SerializableGeeCollections-TDocument-Test.c"
					GXmlNode* _tmp703_ = NULL;
					GeeMap* _tmp704_ = NULL;
					GeeMap* _tmp705_ = NULL;
					GeeMap* _tmp706_ = NULL;
					gpointer _tmp707_ = NULL;
					GXmlNode* _tmp708_ = NULL;
					gchar* _tmp709_ = NULL;
					gchar* _tmp710_ = NULL;
					gchar* _tmp711_ = NULL;
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp703_ = r;
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp704_ = gxml_node_get_attrs (_tmp703_);
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp705_ = _tmp704_;
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp706_ = _tmp705_;
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp707_ = gee_map_get (_tmp706_, "manufacturer");
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp708_ = (GXmlNode*) _tmp707_;
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp709_ = gxml_node_get_value (_tmp708_);
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp710_ = _tmp709_;
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp711_ = _tmp710_;
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp697_ = g_strcmp0 (_tmp711_, "MacToy") == 0;
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_free0 (_tmp711_);
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_object_unref0 (_tmp708_);
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_object_unref0 (_tmp706_);
#line 3776 "SerializableGeeCollections-TDocument-Test.c"
				} else {
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp697_ = FALSE;
#line 3780 "SerializableGeeCollections-TDocument-Test.c"
				}
#line 363 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				if (_tmp697_) {
#line 3784 "SerializableGeeCollections-TDocument-Test.c"
					GXmlNode* _tmp712_ = NULL;
					gchar* _tmp713_ = NULL;
					gchar* _tmp714_ = NULL;
					gchar* _tmp715_ = NULL;
					GXmlNode* _tmp716_ = NULL;
					GeeMap* _tmp717_ = NULL;
					GeeMap* _tmp718_ = NULL;
					GeeMap* _tmp719_ = NULL;
					gpointer _tmp720_ = NULL;
					GXmlNode* _tmp721_ = NULL;
					gchar* _tmp722_ = NULL;
					gchar* _tmp723_ = NULL;
					gchar* _tmp724_ = NULL;
					GXmlNode* _tmp725_ = NULL;
					GeeMap* _tmp726_ = NULL;
					GeeMap* _tmp727_ = NULL;
					GeeMap* _tmp728_ = NULL;
					gpointer _tmp729_ = NULL;
					GXmlNode* _tmp730_ = NULL;
					gchar* _tmp731_ = NULL;
					gchar* _tmp732_ = NULL;
					gchar* _tmp733_ = NULL;
					GXmlNode* _tmp734_ = NULL;
					GeeBidirList* _tmp735_ = NULL;
					GeeBidirList* _tmp736_ = NULL;
					GeeBidirList* _tmp737_ = NULL;
					gint _tmp738_ = 0;
					gint _tmp739_ = 0;
					gboolean frmactoy = FALSE;
					gboolean frmega = FALSE;
					gboolean _tmp830_ = FALSE;
					gboolean _tmp831_ = FALSE;
#line 364 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					fr = TRUE;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp712_ = r;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp713_ = gxml_node_get_name (_tmp712_);
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp714_ = _tmp713_;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp715_ = _tmp714_;
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_vala_assert (g_strcmp0 (_tmp715_, "refaction") == 0, "r.name == \"refaction\"");
#line 365 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_free0 (_tmp715_);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp716_ = r;
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp717_ = gxml_node_get_attrs (_tmp716_);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp718_ = _tmp717_;
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp719_ = _tmp718_;
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp720_ = gee_map_get (_tmp719_, "manufacturer");
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp721_ = (GXmlNode*) _tmp720_;
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp722_ = gxml_node_get_value (_tmp721_);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp723_ = _tmp722_;
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp724_ = _tmp723_;
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_vala_assert (g_strcmp0 (_tmp724_, "MacToy") == 0, "r.attrs.get (\"manufacturer\").value == \"MacToy\"");
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_free0 (_tmp724_);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_object_unref0 (_tmp721_);
#line 366 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_object_unref0 (_tmp719_);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp725_ = r;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp726_ = gxml_node_get_attrs (_tmp725_);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp727_ = _tmp726_;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp728_ = _tmp727_;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp729_ = gee_map_get (_tmp728_, "model");
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp730_ = (GXmlNode*) _tmp729_;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp731_ = gxml_node_get_value (_tmp730_);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp732_ = _tmp731_;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp733_ = _tmp732_;
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_vala_assert (g_strcmp0 (_tmp733_, "Fly045") == 0, "r.attrs.get (\"model\").value == \"Fly045\"");
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_free0 (_tmp733_);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_object_unref0 (_tmp730_);
#line 367 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_object_unref0 (_tmp728_);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp734_ = r;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp735_ = gxml_node_get_children_nodes (_tmp734_);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp736_ = _tmp735_;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp737_ = _tmp736_;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp738_ = gee_collection_get_size ((GeeCollection*) _tmp737_);
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp739_ = _tmp738_;
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_vala_assert (_tmp739_ == 5, "r.children_nodes.size == 5");
#line 368 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_object_unref0 (_tmp737_);
#line 369 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					frmactoy = FALSE;
#line 370 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					frmega = FALSE;
#line 3903 "SerializableGeeCollections-TDocument-Test.c"
					{
						GeeBidirList* _rsh_list = NULL;
						GXmlNode* _tmp740_ = NULL;
						GeeBidirList* _tmp741_ = NULL;
						GeeBidirList* _tmp742_ = NULL;
						gint _rsh_size = 0;
						GeeBidirList* _tmp743_ = NULL;
						gint _tmp744_ = 0;
						gint _tmp745_ = 0;
						gint _rsh_index = 0;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_tmp740_ = r;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_tmp741_ = gxml_node_get_children_nodes (_tmp740_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_tmp742_ = _tmp741_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_rsh_list = _tmp742_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_tmp743_ = _rsh_list;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_tmp744_ = gee_collection_get_size ((GeeCollection*) _tmp743_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_tmp745_ = _tmp744_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_rsh_size = _tmp745_;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_rsh_index = -1;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						while (TRUE) {
#line 3934 "SerializableGeeCollections-TDocument-Test.c"
							gint _tmp746_ = 0;
							gint _tmp747_ = 0;
							gint _tmp748_ = 0;
							GXmlNode* rsh = NULL;
							GeeBidirList* _tmp749_ = NULL;
							gint _tmp750_ = 0;
							gpointer _tmp751_ = NULL;
							gboolean _tmp752_ = FALSE;
							GXmlNode* _tmp753_ = NULL;
							gchar* _tmp754_ = NULL;
							gchar* _tmp755_ = NULL;
							gchar* _tmp756_ = NULL;
							gboolean _tmp757_ = FALSE;
							gboolean _tmp791_ = FALSE;
							GXmlNode* _tmp792_ = NULL;
							gchar* _tmp793_ = NULL;
							gchar* _tmp794_ = NULL;
							gchar* _tmp795_ = NULL;
							gboolean _tmp796_ = FALSE;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp746_ = _rsh_index;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_rsh_index = _tmp746_ + 1;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp747_ = _rsh_index;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp748_ = _rsh_size;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							if (!(_tmp747_ < _tmp748_)) {
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								break;
#line 3966 "SerializableGeeCollections-TDocument-Test.c"
							}
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp749_ = _rsh_list;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp750_ = _rsh_index;
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp751_ = gee_list_get ((GeeList*) _tmp749_, _tmp750_);
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							rsh = (GXmlNode*) _tmp751_;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp753_ = rsh;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp754_ = gxml_node_get_name (_tmp753_);
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp755_ = _tmp754_;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp756_ = _tmp755_;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp757_ = g_strcmp0 (_tmp756_, "ship") == 0;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_g_free0 (_tmp756_);
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							if (_tmp757_) {
#line 3990 "SerializableGeeCollections-TDocument-Test.c"
								GXmlNode* _tmp758_ = NULL;
								GeeMap* _tmp759_ = NULL;
								GeeMap* _tmp760_ = NULL;
								GeeMap* _tmp761_ = NULL;
								gpointer _tmp762_ = NULL;
								GXmlNode* _tmp763_ = NULL;
								gchar* _tmp764_ = NULL;
								gchar* _tmp765_ = NULL;
								gchar* _tmp766_ = NULL;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp758_ = rsh;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp759_ = gxml_node_get_attrs (_tmp758_);
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp760_ = _tmp759_;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp761_ = _tmp760_;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp762_ = gee_map_get (_tmp761_, "manufacturer");
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp763_ = (GXmlNode*) _tmp762_;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp764_ = gxml_node_get_value (_tmp763_);
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp765_ = _tmp764_;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp766_ = _tmp765_;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp752_ = g_strcmp0 (_tmp766_, "MacToy") == 0;
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_free0 (_tmp766_);
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp763_);
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp761_);
#line 4026 "SerializableGeeCollections-TDocument-Test.c"
							} else {
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp752_ = FALSE;
#line 4030 "SerializableGeeCollections-TDocument-Test.c"
							}
#line 372 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							if (_tmp752_) {
#line 4034 "SerializableGeeCollections-TDocument-Test.c"
								GXmlNode* _tmp767_ = NULL;
								GeeMap* _tmp768_ = NULL;
								GeeMap* _tmp769_ = NULL;
								GeeMap* _tmp770_ = NULL;
								gpointer _tmp771_ = NULL;
								GXmlNode* _tmp772_ = NULL;
								gchar* _tmp773_ = NULL;
								gchar* _tmp774_ = NULL;
								gchar* _tmp775_ = NULL;
								GXmlNode* _tmp776_ = NULL;
								GeeMap* _tmp777_ = NULL;
								GeeMap* _tmp778_ = NULL;
								GeeMap* _tmp779_ = NULL;
								gpointer _tmp780_ = NULL;
								GXmlNode* _tmp781_ = NULL;
								gchar* _tmp782_ = NULL;
								gchar* _tmp783_ = NULL;
								gchar* _tmp784_ = NULL;
								GXmlNode* _tmp785_ = NULL;
								GeeBidirList* _tmp786_ = NULL;
								GeeBidirList* _tmp787_ = NULL;
								GeeBidirList* _tmp788_ = NULL;
								gint _tmp789_ = 0;
								gint _tmp790_ = 0;
#line 373 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								frmactoy = TRUE;
#line 374 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp767_ = rsh;
#line 374 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp768_ = gxml_node_get_attrs (_tmp767_);
#line 374 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp769_ = _tmp768_;
#line 374 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp770_ = _tmp769_;
#line 374 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp771_ = gee_map_get (_tmp770_, "manufacturer");
#line 374 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp772_ = (GXmlNode*) _tmp771_;
#line 374 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp773_ = gxml_node_get_value (_tmp772_);
#line 374 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp774_ = _tmp773_;
#line 374 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp775_ = _tmp774_;
#line 374 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_vala_assert (g_strcmp0 (_tmp775_, "MacToy") == 0, "rsh.attrs.get (\"manufacturer\").value == \"MacToy\"");
#line 374 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_free0 (_tmp775_);
#line 374 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp772_);
#line 374 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp770_);
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp776_ = rsh;
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp777_ = gxml_node_get_attrs (_tmp776_);
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp778_ = _tmp777_;
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp779_ = _tmp778_;
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp780_ = gee_map_get (_tmp779_, "model");
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp781_ = (GXmlNode*) _tmp780_;
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp782_ = gxml_node_get_value (_tmp781_);
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp783_ = _tmp782_;
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp784_ = _tmp783_;
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_vala_assert (g_strcmp0 (_tmp784_, "A1234") == 0, "rsh.attrs.get (\"model\").value == \"A1234\"");
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_free0 (_tmp784_);
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp781_);
#line 375 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp779_);
#line 376 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp785_ = rsh;
#line 376 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp786_ = gxml_node_get_children_nodes (_tmp785_);
#line 376 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp787_ = _tmp786_;
#line 376 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp788_ = _tmp787_;
#line 376 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp789_ = gee_collection_get_size ((GeeCollection*) _tmp788_);
#line 376 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp790_ = _tmp789_;
#line 376 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_vala_assert (_tmp790_ == 0, "rsh.children_nodes.size == 0");
#line 376 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp788_);
#line 4129 "SerializableGeeCollections-TDocument-Test.c"
							}
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp792_ = rsh;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp793_ = gxml_node_get_name (_tmp792_);
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp794_ = _tmp793_;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp795_ = _tmp794_;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp796_ = g_strcmp0 (_tmp795_, "ship") == 0;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_g_free0 (_tmp795_);
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							if (_tmp796_) {
#line 4145 "SerializableGeeCollections-TDocument-Test.c"
								GXmlNode* _tmp797_ = NULL;
								GeeMap* _tmp798_ = NULL;
								GeeMap* _tmp799_ = NULL;
								GeeMap* _tmp800_ = NULL;
								gpointer _tmp801_ = NULL;
								GXmlNode* _tmp802_ = NULL;
								gchar* _tmp803_ = NULL;
								gchar* _tmp804_ = NULL;
								gchar* _tmp805_ = NULL;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp797_ = rsh;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp798_ = gxml_node_get_attrs (_tmp797_);
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp799_ = _tmp798_;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp800_ = _tmp799_;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp801_ = gee_map_get (_tmp800_, "manufacturer");
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp802_ = (GXmlNode*) _tmp801_;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp803_ = gxml_node_get_value (_tmp802_);
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp804_ = _tmp803_;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp805_ = _tmp804_;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp791_ = g_strcmp0 (_tmp805_, "MegaTrench") == 0;
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_free0 (_tmp805_);
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp802_);
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp800_);
#line 4181 "SerializableGeeCollections-TDocument-Test.c"
							} else {
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp791_ = FALSE;
#line 4185 "SerializableGeeCollections-TDocument-Test.c"
							}
#line 378 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							if (_tmp791_) {
#line 4189 "SerializableGeeCollections-TDocument-Test.c"
								GXmlNode* _tmp806_ = NULL;
								GeeMap* _tmp807_ = NULL;
								GeeMap* _tmp808_ = NULL;
								GeeMap* _tmp809_ = NULL;
								gpointer _tmp810_ = NULL;
								GXmlNode* _tmp811_ = NULL;
								gchar* _tmp812_ = NULL;
								gchar* _tmp813_ = NULL;
								gchar* _tmp814_ = NULL;
								GXmlNode* _tmp815_ = NULL;
								GeeMap* _tmp816_ = NULL;
								GeeMap* _tmp817_ = NULL;
								GeeMap* _tmp818_ = NULL;
								gpointer _tmp819_ = NULL;
								GXmlNode* _tmp820_ = NULL;
								gchar* _tmp821_ = NULL;
								gchar* _tmp822_ = NULL;
								gchar* _tmp823_ = NULL;
								GXmlNode* _tmp824_ = NULL;
								GeeBidirList* _tmp825_ = NULL;
								GeeBidirList* _tmp826_ = NULL;
								GeeBidirList* _tmp827_ = NULL;
								gint _tmp828_ = 0;
								gint _tmp829_ = 0;
#line 379 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								frmega = TRUE;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp806_ = rsh;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp807_ = gxml_node_get_attrs (_tmp806_);
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp808_ = _tmp807_;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp809_ = _tmp808_;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp810_ = gee_map_get (_tmp809_, "manufacturer");
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp811_ = (GXmlNode*) _tmp810_;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp812_ = gxml_node_get_value (_tmp811_);
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp813_ = _tmp812_;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp814_ = _tmp813_;
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_vala_assert (g_strcmp0 (_tmp814_, "MegaTrench") == 0, "rsh.attrs.get (\"manufacturer\").value == \"MegaTrench\"");
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_free0 (_tmp814_);
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp811_);
#line 380 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp809_);
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp815_ = rsh;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp816_ = gxml_node_get_attrs (_tmp815_);
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp817_ = _tmp816_;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp818_ = _tmp817_;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp819_ = gee_map_get (_tmp818_, "model");
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp820_ = (GXmlNode*) _tmp819_;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp821_ = gxml_node_get_value (_tmp820_);
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp822_ = _tmp821_;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp823_ = _tmp822_;
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_vala_assert (g_strcmp0 (_tmp823_, "G045-1") == 0, "rsh.attrs.get (\"model\").value == \"G045-1\"");
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_free0 (_tmp823_);
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp820_);
#line 381 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp818_);
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp824_ = rsh;
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp825_ = gxml_node_get_children_nodes (_tmp824_);
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp826_ = _tmp825_;
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp827_ = _tmp826_;
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp828_ = gee_collection_get_size ((GeeCollection*) _tmp827_);
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp829_ = _tmp828_;
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_vala_assert (_tmp829_ == 0, "rsh.children_nodes.size == 0");
#line 382 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (_tmp827_);
#line 4284 "SerializableGeeCollections-TDocument-Test.c"
							}
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_g_object_unref0 (rsh);
#line 4288 "SerializableGeeCollections-TDocument-Test.c"
						}
#line 371 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_g_object_unref0 (_rsh_list);
#line 4292 "SerializableGeeCollections-TDocument-Test.c"
					}
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp830_ = frmactoy;
#line 385 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_vala_assert (_tmp830_, "frmactoy");
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp831_ = frmega;
#line 386 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_vala_assert (_tmp831_, "frmega");
#line 4302 "SerializableGeeCollections-TDocument-Test.c"
				}
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_g_object_unref0 (r);
#line 4306 "SerializableGeeCollections-TDocument-Test.c"
			}
#line 362 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_g_object_unref0 (_r_list);
#line 4310 "SerializableGeeCollections-TDocument-Test.c"
		}
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp832_ = fr;
#line 389 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp832_, "fr");
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (st);
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (cz);
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (ndoc);
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (sbsh1);
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (sb);
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (doc);
#line 4328 "SerializableGeeCollections-TDocument-Test.c"
	}
	goto __finally98;
	__catch98_g_error:
	{
		GError* e = NULL;
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		e = _inner_error_;
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_inner_error_ = NULL;
#line 399 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_assert_not_reached ();
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_error_free0 (e);
#line 4342 "SerializableGeeCollections-TDocument-Test.c"
	}
	__finally98:
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_clear_error (&_inner_error_);
#line 181 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		return;
#line 4353 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static void ___lambda70__gtest_func (void) {
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	__lambda70_ ();
#line 4361 "SerializableGeeCollections-TDocument-Test.c"
}


static const gchar* string_to_string (const gchar* self) {
	const gchar* result = NULL;
#line 1412 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1413 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	result = self;
#line 1413 "/usr/share/vala-0.34/vapi/glib-2.0.vapi"
	return result;
#line 4373 "SerializableGeeCollections-TDocument-Test.c"
}


static void __lambda71_ (void) {
	GError * _inner_error_ = NULL;
	{
		GXmlGDocument* org_doc = NULL;
		GXmlGDocument* _tmp0_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceBase* s = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceBase* _tmp1_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceBase* _tmp2_ = NULL;
		GXmlGDocument* _tmp3_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceBase* _tmp4_ = NULL;
		SerializableGeeCollectionsTDocumentTestChargeZone* _tmp5_ = NULL;
		SerializableGeeCollectionsTDocumentTestChargeZone* _tmp6_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceBase* _tmp7_ = NULL;
		SerializableGeeCollectionsTDocumentTestChargeZone* _tmp8_ = NULL;
		SerializableGeeCollectionsTDocumentTestChargeZone* _tmp9_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp10_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp11_ = NULL;
		gint _tmp12_ = 0;
		gint _tmp13_ = 0;
		SerializableGeeCollectionsTDocumentTestSpaceShip* mssh = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceBase* _tmp14_ = NULL;
		SerializableGeeCollectionsTDocumentTestChargeZone* _tmp15_ = NULL;
		SerializableGeeCollectionsTDocumentTestChargeZone* _tmp16_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp17_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp18_ = NULL;
		gpointer _tmp19_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceShip* _tmp20_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceBase* _tmp21_ = NULL;
		SerializableGeeCollectionsTDocumentTestStorage* _tmp22_ = NULL;
		SerializableGeeCollectionsTDocumentTestStorage* _tmp23_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceShip* _tmp24_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceCollection* _tmp25_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceCollection* _tmp26_ = NULL;
		gint _tmp27_ = 0;
		gint _tmp28_ = 0;
		SerializableGeeCollectionsTDocumentTestSpaceBase* _tmp29_ = NULL;
		SerializableGeeCollectionsTDocumentTestStorage* _tmp30_ = NULL;
		SerializableGeeCollectionsTDocumentTestStorage* _tmp31_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceBase* _tmp32_ = NULL;
		SerializableGeeCollectionsTDocumentTestStorage* _tmp33_ = NULL;
		SerializableGeeCollectionsTDocumentTestStorage* _tmp34_ = NULL;
		SerializableGeeCollectionsTDocumentTestRefactionCollection* _tmp35_ = NULL;
		SerializableGeeCollectionsTDocumentTestRefactionCollection* _tmp36_ = NULL;
		gint _tmp37_ = 0;
		gint _tmp38_ = 0;
		SerializableGeeCollectionsTDocumentTestRefaction* refaction = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceBase* _tmp39_ = NULL;
		SerializableGeeCollectionsTDocumentTestStorage* _tmp40_ = NULL;
		SerializableGeeCollectionsTDocumentTestStorage* _tmp41_ = NULL;
		SerializableGeeCollectionsTDocumentTestRefactionCollection* _tmp42_ = NULL;
		SerializableGeeCollectionsTDocumentTestRefactionCollection* _tmp43_ = NULL;
		gpointer _tmp44_ = NULL;
		SerializableGeeCollectionsTDocumentTestRefaction* _tmp45_ = NULL;
		SerializableGeeCollectionsTDocumentTestRefaction* _tmp46_ = NULL;
		GeeMap* _tmp47_ = NULL;
		GeeMap* _tmp48_ = NULL;
		GeeMap* _tmp49_ = NULL;
		SerializableGeeCollectionsTDocumentTestRefaction* _tmp50_ = NULL;
		GeeMap* _tmp51_ = NULL;
		GeeMap* _tmp52_ = NULL;
		GeeMap* _tmp53_ = NULL;
		gint _tmp54_ = 0;
		gint _tmp55_ = 0;
		SerializableGeeCollectionsTDocumentTestRefaction* _tmp56_ = NULL;
		GeeCollection* _tmp57_ = NULL;
		GeeCollection* _tmp58_ = NULL;
		GeeCollection* _tmp59_ = NULL;
		gint _tmp60_ = 0;
		gint _tmp61_ = 0;
		GXmlTDocument* doc = NULL;
		GXmlTDocument* _tmp62_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceBase* _tmp63_ = NULL;
		GXmlTDocument* _tmp64_ = NULL;
		GXmlNode* _tmp65_ = NULL;
		GXmlNode* _tmp66_ = NULL;
		GXmlTDocument* _tmp67_ = NULL;
		GXmlNode* _tmp68_ = NULL;
		GXmlNode* _tmp69_ = NULL;
		GXmlNode* _tmp70_ = NULL;
		gchar* _tmp71_ = NULL;
		gchar* _tmp72_ = NULL;
		gchar* _tmp73_ = NULL;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp0_ = gxml_gdocument_new_from_string ("<?xml version=\"1.0\"?>\n" \
"<base name=\"AlphaOne\" >\n" \
"  <chargezone name=\"A1-1\">\n" \
"    <ship manufacturer=\"MacToy\" model=\"A1234\">\n" \
"      <space name=\"Alpha Centaury\">\n" \
"        <planet name=\"Earth\">\n" \
"          <citizen ctype=\"Human\">1M</citizen>\n" \
"          <citizen ctype=\"Ghost\">10M</citizen>\n" \
"        </planet>\n" \
"        <planet name=\"Galiumt\" />\n" \
"      </space>\n" \
"      <space name=\"Galax\">\n" \
"        <planet name=\"Saminon\">\n" \
"          <citizen ctype=\"Humanes\">100M</citizen>\n" \
"          <citizen ctype=\"Jeties\">1000M</citizen>\n" \
"        </planet>\n" \
"        <planet name=\"Keymanth\" />\n" \
"      </space>\n" \
"    </ship>\n" \
"    <ship manufacturer=\"Memphis\" model=\"AB1\">\n" \
"      <space name=\"Proximity\">\n" \
"        <planet name=\"Pluton\">\n" \
"          <citizen ctype=\"Ork\">10000M</citizen>\n" \
"          <citizen ctype=\"Gyk\">100000M</citizen>\n" \
"        </planet>\n" \
"        <planet name=\"Mercury\" />\n" \
"      </space>\n" \
"    </ship>\n" \
"  </chargezone>\n" \
"  <storage name=\"B4-A4\">\n" \
"    <refaction manufacturer=\"MacToy\" model=\"Fly045\">\n" \
"      <ship manufacturer=\"MacToy\" model=\"A1234\" />\n" \
"      <ship manufacturer=\"MegaTrench\" model=\"G045-1\" unknown=\"UNK" \
"NOWN ATTR\">TEST_TEXT</ship>\n" \
"      <UnknownAttribute name=\"nothing\" />\n" \
"    </refaction>\n" \
"  </storage>\n" \
"</base>", 0, &_inner_error_);
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		org_doc = _tmp0_;
#line 406 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 4465 "SerializableGeeCollections-TDocument-Test.c"
			goto __catch99_g_error;
		}
#line 443 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp1_ = serializable_gee_collections_tdocument_test_space_base_new ();
#line 443 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		s = _tmp1_;
#line 444 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp2_ = s;
#line 444 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp3_ = org_doc;
#line 444 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		gxml_serializable_object_model_deserialize ((GXmlSerializableObjectModel*) _tmp2_, (GXmlNode*) _tmp3_, &_inner_error_);
#line 444 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 444 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_g_object_unref0 (s);
#line 444 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_g_object_unref0 (org_doc);
#line 4484 "SerializableGeeCollections-TDocument-Test.c"
			goto __catch99_g_error;
		}
#line 445 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp4_ = s;
#line 445 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp5_ = serializable_gee_collections_tdocument_test_space_base_get_charge_zone (_tmp4_);
#line 445 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp6_ = _tmp5_;
#line 445 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp6_ != NULL, "s.charge_zone != null");
#line 446 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp7_ = s;
#line 446 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp8_ = serializable_gee_collections_tdocument_test_space_base_get_charge_zone (_tmp7_);
#line 446 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp9_ = _tmp8_;
#line 446 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp10_ = serializable_gee_collections_tdocument_test_charge_zone_get_spaceships (_tmp9_);
#line 446 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp11_ = _tmp10_;
#line 446 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp12_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp11_);
#line 446 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp13_ = _tmp12_;
#line 446 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp13_ == 2, "s.charge_zone.spaceships.size == 2");
#line 447 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp14_ = s;
#line 447 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp15_ = serializable_gee_collections_tdocument_test_space_base_get_charge_zone (_tmp14_);
#line 447 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp16_ = _tmp15_;
#line 447 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp17_ = serializable_gee_collections_tdocument_test_charge_zone_get_spaceships (_tmp16_);
#line 447 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp18_ = _tmp17_;
#line 447 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp19_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp18_, "MacToy", "A1234");
#line 447 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		mssh = (SerializableGeeCollectionsTDocumentTestSpaceShip*) _tmp19_;
#line 448 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp20_ = mssh;
#line 448 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp20_ != NULL, "mssh != null");
#line 450 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp21_ = s;
#line 450 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp22_ = serializable_gee_collections_tdocument_test_space_base_get_storage (_tmp21_);
#line 450 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp23_ = _tmp22_;
#line 450 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp23_ != NULL, "s.storage != null");
#line 451 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp24_ = mssh;
#line 451 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp25_ = serializable_gee_collections_tdocument_test_space_ship_get_spaces (_tmp24_);
#line 451 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp26_ = _tmp25_;
#line 451 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp27_ = gee_abstract_map_get_size ((GeeAbstractMap*) _tmp26_);
#line 451 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp28_ = _tmp27_;
#line 451 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp28_ == 2, "mssh.spaces.size == 2");
#line 452 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp29_ = s;
#line 452 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp30_ = serializable_gee_collections_tdocument_test_space_base_get_storage (_tmp29_);
#line 452 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp31_ = _tmp30_;
#line 452 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp31_ != NULL, "s.storage != null");
#line 453 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp32_ = s;
#line 453 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp33_ = serializable_gee_collections_tdocument_test_space_base_get_storage (_tmp32_);
#line 453 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp34_ = _tmp33_;
#line 453 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp35_ = serializable_gee_collections_tdocument_test_storage_get_refactions (_tmp34_);
#line 453 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp36_ = _tmp35_;
#line 453 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp37_ = gxml_serializable_dual_key_map_get_size ((GXmlSerializableDualKeyMap*) _tmp36_);
#line 453 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp38_ = _tmp37_;
#line 453 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp38_ == 1, "s.storage.refactions.size == 1");
#line 454 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp39_ = s;
#line 454 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp40_ = serializable_gee_collections_tdocument_test_space_base_get_storage (_tmp39_);
#line 454 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp41_ = _tmp40_;
#line 454 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp42_ = serializable_gee_collections_tdocument_test_storage_get_refactions (_tmp41_);
#line 454 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp43_ = _tmp42_;
#line 454 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp44_ = gxml_serializable_dual_key_map_get ((GXmlSerializableDualKeyMap*) _tmp43_, "MacToy", "Fly045");
#line 454 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		refaction = (SerializableGeeCollectionsTDocumentTestRefaction*) _tmp44_;
#line 455 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp45_ = refaction;
#line 455 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp45_ != NULL, "refaction != null");
#line 456 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp46_ = refaction;
#line 456 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp47_ = gxml_serializable_get_unknown_serializable_properties ((GXmlSerializable*) _tmp46_);
#line 456 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp48_ = _tmp47_;
#line 456 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp49_ = _tmp48_;
#line 456 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp49_ != NULL, "refaction.unknown_serializable_properties != null");
#line 456 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp49_);
#line 457 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp50_ = refaction;
#line 457 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp51_ = gxml_serializable_get_unknown_serializable_properties ((GXmlSerializable*) _tmp50_);
#line 457 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp52_ = _tmp51_;
#line 457 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp53_ = _tmp52_;
#line 457 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp54_ = gee_map_get_size (_tmp53_);
#line 457 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp55_ = _tmp54_;
#line 457 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp55_ == 0, "refaction.unknown_serializable_properties.size == 0");
#line 457 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp53_);
#line 458 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp56_ = refaction;
#line 458 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp57_ = gxml_serializable_get_unknown_serializable_nodes ((GXmlSerializable*) _tmp56_);
#line 458 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp58_ = _tmp57_;
#line 458 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp59_ = _tmp58_;
#line 458 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp60_ = gee_collection_get_size (_tmp59_);
#line 458 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp61_ = _tmp60_;
#line 458 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (_tmp61_ == 5, "refaction.unknown_serializable_nodes.size == 5");
#line 458 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp59_);
#line 459 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp62_ = gxml_tdocument_new ();
#line 459 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		doc = _tmp62_;
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp63_ = s;
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp64_ = doc;
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp65_ = gxml_serializable_object_model_serialize ((GXmlSerializableObjectModel*) _tmp63_, (GXmlNode*) _tmp64_, &_inner_error_);
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp66_ = _tmp65_;
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp66_);
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_g_object_unref0 (doc);
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_g_object_unref0 (refaction);
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_g_object_unref0 (mssh);
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_g_object_unref0 (s);
#line 460 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_g_object_unref0 (org_doc);
#line 4661 "SerializableGeeCollections-TDocument-Test.c"
			goto __catch99_g_error;
		}
#line 461 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp67_ = doc;
#line 461 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp68_ = gxml_document_get_root ((GXmlDocument*) _tmp67_);
#line 461 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp69_ = _tmp68_;
#line 461 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp70_ = _tmp69_;
#line 461 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp71_ = gxml_node_get_name (_tmp70_);
#line 461 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp72_ = _tmp71_;
#line 461 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp73_ = _tmp72_;
#line 461 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_vala_assert (g_strcmp0 (_tmp73_, "base") == 0, "doc.root.name == \"base\"");
#line 461 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_free0 (_tmp73_);
#line 461 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp70_);
#line 4684 "SerializableGeeCollections-TDocument-Test.c"
		{
			GeeBidirList* _n_list = NULL;
			GXmlTDocument* _tmp74_ = NULL;
			GXmlNode* _tmp75_ = NULL;
			GXmlNode* _tmp76_ = NULL;
			GXmlNode* _tmp77_ = NULL;
			GeeBidirList* _tmp78_ = NULL;
			GeeBidirList* _tmp79_ = NULL;
			GeeBidirList* _tmp80_ = NULL;
			gint _n_size = 0;
			GeeBidirList* _tmp81_ = NULL;
			gint _tmp82_ = 0;
			gint _tmp83_ = 0;
			gint _n_index = 0;
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp74_ = doc;
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp75_ = gxml_document_get_root ((GXmlDocument*) _tmp74_);
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp76_ = _tmp75_;
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp77_ = _tmp76_;
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp78_ = gxml_node_get_children_nodes (_tmp77_);
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp79_ = _tmp78_;
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp80_ = _tmp79_;
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_g_object_unref0 (_tmp77_);
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_n_list = _tmp80_;
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp81_ = _n_list;
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp82_ = gee_collection_get_size ((GeeCollection*) _tmp81_);
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_tmp83_ = _tmp82_;
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_n_size = _tmp83_;
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_n_index = -1;
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			while (TRUE) {
#line 4729 "SerializableGeeCollections-TDocument-Test.c"
				gint _tmp84_ = 0;
				gint _tmp85_ = 0;
				gint _tmp86_ = 0;
				GXmlNode* n = NULL;
				GeeBidirList* _tmp87_ = NULL;
				gint _tmp88_ = 0;
				gpointer _tmp89_ = NULL;
				GXmlNode* _tmp90_ = NULL;
				GXmlNode* _tmp196_ = NULL;
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp84_ = _n_index;
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_n_index = _tmp84_ + 1;
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp85_ = _n_index;
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp86_ = _n_size;
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				if (!(_tmp85_ < _tmp86_)) {
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					break;
#line 4751 "SerializableGeeCollections-TDocument-Test.c"
				}
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp87_ = _n_list;
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp88_ = _n_index;
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp89_ = gee_list_get ((GeeList*) _tmp87_, _tmp88_);
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				n = (GXmlNode*) _tmp89_;
#line 464 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp90_ = n;
#line 464 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp90_, GXML_TYPE_ELEMENT)) {
#line 4765 "SerializableGeeCollections-TDocument-Test.c"
					GXmlNode* _tmp91_ = NULL;
					gchar* _tmp92_ = NULL;
					gchar* _tmp93_ = NULL;
					gchar* _tmp94_ = NULL;
					gboolean _tmp95_ = FALSE;
					GXmlNode* _tmp96_ = NULL;
					gchar* _tmp97_ = NULL;
					gchar* _tmp98_ = NULL;
					gchar* _tmp99_ = NULL;
					gboolean _tmp100_ = FALSE;
#line 465 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp91_ = n;
#line 465 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp92_ = gxml_node_get_name (_tmp91_);
#line 465 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp93_ = _tmp92_;
#line 465 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp94_ = _tmp93_;
#line 465 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp95_ = g_strcmp0 (_tmp94_, "ChargeZone") == 0;
#line 465 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_free0 (_tmp94_);
#line 465 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					if (_tmp95_) {
#line 4790 "SerializableGeeCollections-TDocument-Test.c"
					}
#line 468 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp96_ = n;
#line 468 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp97_ = gxml_node_get_name (_tmp96_);
#line 468 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp98_ = _tmp97_;
#line 468 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp99_ = _tmp98_;
#line 468 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp100_ = g_strcmp0 (_tmp99_, "storage") == 0;
#line 468 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_free0 (_tmp99_);
#line 468 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					if (_tmp100_) {
#line 4806 "SerializableGeeCollections-TDocument-Test.c"
						gboolean unkfound = FALSE;
						gboolean tfound = FALSE;
						gboolean attrfound = FALSE;
						gboolean _tmp193_ = FALSE;
						gboolean _tmp194_ = FALSE;
						gboolean _tmp195_ = FALSE;
#line 469 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						unkfound = FALSE;
#line 470 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						tfound = FALSE;
#line 471 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						attrfound = FALSE;
#line 4819 "SerializableGeeCollections-TDocument-Test.c"
						{
							GeeBidirList* _sn_list = NULL;
							GXmlNode* _tmp101_ = NULL;
							GeeBidirList* _tmp102_ = NULL;
							GeeBidirList* _tmp103_ = NULL;
							gint _sn_size = 0;
							GeeBidirList* _tmp104_ = NULL;
							gint _tmp105_ = 0;
							gint _tmp106_ = 0;
							gint _sn_index = 0;
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp101_ = n;
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp102_ = gxml_node_get_children_nodes (_tmp101_);
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp103_ = _tmp102_;
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_sn_list = _tmp103_;
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp104_ = _sn_list;
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp105_ = gee_collection_get_size ((GeeCollection*) _tmp104_);
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_tmp106_ = _tmp105_;
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_sn_size = _tmp106_;
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_sn_index = -1;
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							while (TRUE) {
#line 4850 "SerializableGeeCollections-TDocument-Test.c"
								gint _tmp107_ = 0;
								gint _tmp108_ = 0;
								gint _tmp109_ = 0;
								GXmlNode* sn = NULL;
								GeeBidirList* _tmp110_ = NULL;
								gint _tmp111_ = 0;
								gpointer _tmp112_ = NULL;
								GXmlNode* _tmp113_ = NULL;
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp107_ = _sn_index;
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_sn_index = _tmp107_ + 1;
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp108_ = _sn_index;
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp109_ = _sn_size;
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								if (!(_tmp108_ < _tmp109_)) {
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									break;
#line 4871 "SerializableGeeCollections-TDocument-Test.c"
								}
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp110_ = _sn_list;
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp111_ = _sn_index;
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp112_ = gee_list_get ((GeeList*) _tmp110_, _tmp111_);
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								sn = (GXmlNode*) _tmp112_;
#line 473 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_tmp113_ = sn;
#line 473 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp113_, GXML_TYPE_ELEMENT)) {
#line 4885 "SerializableGeeCollections-TDocument-Test.c"
									GXmlNode* _tmp114_ = NULL;
									gchar* _tmp115_ = NULL;
									gchar* _tmp116_ = NULL;
									gchar* _tmp117_ = NULL;
									gboolean _tmp118_ = FALSE;
#line 474 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp114_ = sn;
#line 474 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp115_ = gxml_node_get_name (_tmp114_);
#line 474 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp116_ = _tmp115_;
#line 474 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp117_ = _tmp116_;
#line 474 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_tmp118_ = g_strcmp0 (_tmp117_, "refaction") == 0;
#line 474 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									_g_free0 (_tmp117_);
#line 474 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
									if (_tmp118_) {
#line 4905 "SerializableGeeCollections-TDocument-Test.c"
										{
											GeeBidirList* _rn_list = NULL;
											GXmlNode* _tmp119_ = NULL;
											GeeBidirList* _tmp120_ = NULL;
											GeeBidirList* _tmp121_ = NULL;
											gint _rn_size = 0;
											GeeBidirList* _tmp122_ = NULL;
											gint _tmp123_ = 0;
											gint _tmp124_ = 0;
											gint _rn_index = 0;
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp119_ = sn;
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp120_ = gxml_node_get_children_nodes (_tmp119_);
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp121_ = _tmp120_;
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_rn_list = _tmp121_;
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp122_ = _rn_list;
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp123_ = gee_collection_get_size ((GeeCollection*) _tmp122_);
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_tmp124_ = _tmp123_;
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_rn_size = _tmp124_;
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_rn_index = -1;
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											while (TRUE) {
#line 4936 "SerializableGeeCollections-TDocument-Test.c"
												gint _tmp125_ = 0;
												gint _tmp126_ = 0;
												gint _tmp127_ = 0;
												GXmlNode* rn = NULL;
												GeeBidirList* _tmp128_ = NULL;
												gint _tmp129_ = 0;
												gpointer _tmp130_ = NULL;
												GXmlNode* _tmp131_ = NULL;
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
												_tmp125_ = _rn_index;
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
												_rn_index = _tmp125_ + 1;
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
												_tmp126_ = _rn_index;
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
												_tmp127_ = _rn_size;
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
												if (!(_tmp126_ < _tmp127_)) {
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
													break;
#line 4957 "SerializableGeeCollections-TDocument-Test.c"
												}
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
												_tmp128_ = _rn_list;
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
												_tmp129_ = _rn_index;
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
												_tmp130_ = gee_list_get ((GeeList*) _tmp128_, _tmp129_);
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
												rn = (GXmlNode*) _tmp130_;
#line 476 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
												_tmp131_ = rn;
#line 476 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
												if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp131_, GXML_TYPE_ELEMENT)) {
#line 4971 "SerializableGeeCollections-TDocument-Test.c"
													GXmlNode* _tmp132_ = NULL;
													gchar* _tmp133_ = NULL;
													gchar* _tmp134_ = NULL;
													gchar* _tmp135_ = NULL;
													gboolean _tmp136_ = FALSE;
													GXmlNode* _tmp177_ = NULL;
													gchar* _tmp178_ = NULL;
													gchar* _tmp179_ = NULL;
													gchar* _tmp180_ = NULL;
													gboolean _tmp181_ = FALSE;
#line 478 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
													_tmp132_ = rn;
#line 478 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
													_tmp133_ = gxml_node_get_name (_tmp132_);
#line 478 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
													_tmp134_ = _tmp133_;
#line 478 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
													_tmp135_ = _tmp134_;
#line 478 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
													_tmp136_ = g_strcmp0 (_tmp135_, "ship") == 0;
#line 478 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
													_g_free0 (_tmp135_);
#line 478 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
													if (_tmp136_) {
#line 4996 "SerializableGeeCollections-TDocument-Test.c"
														GXmlNode* atr = NULL;
														GXmlNode* _tmp137_ = NULL;
														GeeMap* _tmp138_ = NULL;
														GeeMap* _tmp139_ = NULL;
														GeeMap* _tmp140_ = NULL;
														gpointer _tmp141_ = NULL;
														GXmlNode* _tmp142_ = NULL;
														GXmlNode* _tmp143_ = NULL;
														GXmlNode* _tmp144_ = NULL;
														gchar* _tmp145_ = NULL;
														gchar* _tmp146_ = NULL;
														gchar* _tmp147_ = NULL;
														gboolean _tmp148_ = FALSE;
#line 479 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp137_ = rn;
#line 479 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp138_ = gxml_node_get_attrs (_tmp137_);
#line 479 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp139_ = _tmp138_;
#line 479 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp140_ = _tmp139_;
#line 479 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp141_ = gee_map_get (_tmp140_, "manufacturer");
#line 479 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp142_ = (GXmlNode*) _tmp141_;
#line 479 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_g_object_unref0 (_tmp140_);
#line 479 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														atr = _tmp142_;
#line 480 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp143_ = atr;
#line 480 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_vala_assert (_tmp143_ != NULL, "atr != null");
#line 481 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp144_ = atr;
#line 481 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp145_ = gxml_node_get_value (_tmp144_);
#line 481 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp146_ = _tmp145_;
#line 481 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp147_ = _tmp146_;
#line 481 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp148_ = g_strcmp0 (_tmp147_, "MegaTrench") == 0;
#line 481 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_g_free0 (_tmp147_);
#line 481 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														if (_tmp148_) {
#line 5044 "SerializableGeeCollections-TDocument-Test.c"
															GXmlNode* shanattr = NULL;
															GXmlNode* _tmp149_ = NULL;
															GeeMap* _tmp150_ = NULL;
															GeeMap* _tmp151_ = NULL;
															GeeMap* _tmp152_ = NULL;
															gpointer _tmp153_ = NULL;
															GXmlNode* _tmp154_ = NULL;
															GXmlNode* _tmp155_ = NULL;
#line 482 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
															_tmp149_ = rn;
#line 482 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
															_tmp150_ = gxml_node_get_attrs (_tmp149_);
#line 482 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
															_tmp151_ = _tmp150_;
#line 482 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
															_tmp152_ = _tmp151_;
#line 482 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
															_tmp153_ = gee_map_get (_tmp152_, "unknown");
#line 482 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
															_tmp154_ = (GXmlNode*) _tmp153_;
#line 482 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
															_g_object_unref0 (_tmp152_);
#line 482 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
															shanattr = _tmp154_;
#line 483 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
															_tmp155_ = shanattr;
#line 483 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
															if (_tmp155_ != NULL) {
#line 5073 "SerializableGeeCollections-TDocument-Test.c"
																GXmlNode* _tmp156_ = NULL;
																gchar* _tmp157_ = NULL;
																gchar* _tmp158_ = NULL;
																gchar* _tmp159_ = NULL;
#line 484 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																attrfound = TRUE;
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																_tmp156_ = shanattr;
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																_tmp157_ = gxml_node_get_value (_tmp156_);
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																_tmp158_ = _tmp157_;
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																_tmp159_ = _tmp158_;
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																_vala_assert (g_strcmp0 (_tmp159_, "UNKNOWN ATTR") == 0, "shanattr.value == \"UNKNOWN ATTR\"");
#line 485 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																_g_free0 (_tmp159_);
#line 5092 "SerializableGeeCollections-TDocument-Test.c"
															}
															{
																GeeBidirList* _shn_list = NULL;
																GXmlNode* _tmp160_ = NULL;
																GeeBidirList* _tmp161_ = NULL;
																GeeBidirList* _tmp162_ = NULL;
																gint _shn_size = 0;
																GeeBidirList* _tmp163_ = NULL;
																gint _tmp164_ = 0;
																gint _tmp165_ = 0;
																gint _shn_index = 0;
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																_tmp160_ = rn;
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																_tmp161_ = gxml_node_get_children_nodes (_tmp160_);
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																_tmp162_ = _tmp161_;
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																_shn_list = _tmp162_;
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																_tmp163_ = _shn_list;
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																_tmp164_ = gee_collection_get_size ((GeeCollection*) _tmp163_);
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																_tmp165_ = _tmp164_;
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																_shn_size = _tmp165_;
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																_shn_index = -1;
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																while (TRUE) {
#line 5124 "SerializableGeeCollections-TDocument-Test.c"
																	gint _tmp166_ = 0;
																	gint _tmp167_ = 0;
																	gint _tmp168_ = 0;
																	GXmlNode* shn = NULL;
																	GeeBidirList* _tmp169_ = NULL;
																	gint _tmp170_ = 0;
																	gpointer _tmp171_ = NULL;
																	GXmlNode* _tmp172_ = NULL;
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																	_tmp166_ = _shn_index;
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																	_shn_index = _tmp166_ + 1;
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																	_tmp167_ = _shn_index;
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																	_tmp168_ = _shn_size;
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																	if (!(_tmp167_ < _tmp168_)) {
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																		break;
#line 5145 "SerializableGeeCollections-TDocument-Test.c"
																	}
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																	_tmp169_ = _shn_list;
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																	_tmp170_ = _shn_index;
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																	_tmp171_ = gee_list_get ((GeeList*) _tmp169_, _tmp170_);
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																	shn = (GXmlNode*) _tmp171_;
#line 489 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																	_tmp172_ = shn;
#line 489 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																	if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp172_, GXML_TYPE_TEXT)) {
#line 5159 "SerializableGeeCollections-TDocument-Test.c"
																		GXmlNode* _tmp173_ = NULL;
																		gchar* _tmp174_ = NULL;
																		gchar* _tmp175_ = NULL;
																		gchar* _tmp176_ = NULL;
#line 490 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																		tfound = TRUE;
#line 491 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																		_tmp173_ = shn;
#line 491 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																		_tmp174_ = gxml_node_get_value (_tmp173_);
#line 491 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																		_tmp175_ = _tmp174_;
#line 491 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																		_tmp176_ = _tmp175_;
#line 491 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																		_vala_assert (g_strcmp0 (_tmp176_, "TEST_TEXT") == 0, "shn.value == \"TEST_TEXT\"");
#line 491 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																		_g_free0 (_tmp176_);
#line 5178 "SerializableGeeCollections-TDocument-Test.c"
																	}
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																	_g_object_unref0 (shn);
#line 5182 "SerializableGeeCollections-TDocument-Test.c"
																}
#line 487 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
																_g_object_unref0 (_shn_list);
#line 5186 "SerializableGeeCollections-TDocument-Test.c"
															}
#line 481 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
															_g_object_unref0 (shanattr);
#line 5190 "SerializableGeeCollections-TDocument-Test.c"
														}
#line 478 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_g_object_unref0 (atr);
#line 5194 "SerializableGeeCollections-TDocument-Test.c"
													}
#line 496 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
													_tmp177_ = rn;
#line 496 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
													_tmp178_ = gxml_node_get_name (_tmp177_);
#line 496 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
													_tmp179_ = _tmp178_;
#line 496 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
													_tmp180_ = _tmp179_;
#line 496 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
													_tmp181_ = g_strcmp0 (_tmp180_, "UnknownAttribute") == 0;
#line 496 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
													_g_free0 (_tmp180_);
#line 496 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
													if (_tmp181_) {
#line 5210 "SerializableGeeCollections-TDocument-Test.c"
														GXmlNode* nattr = NULL;
														GXmlNode* _tmp182_ = NULL;
														GeeMap* _tmp183_ = NULL;
														GeeMap* _tmp184_ = NULL;
														GeeMap* _tmp185_ = NULL;
														gpointer _tmp186_ = NULL;
														GXmlNode* _tmp187_ = NULL;
														GXmlNode* _tmp188_ = NULL;
														GXmlNode* _tmp189_ = NULL;
														gchar* _tmp190_ = NULL;
														gchar* _tmp191_ = NULL;
														gchar* _tmp192_ = NULL;
#line 497 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														unkfound = TRUE;
#line 498 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp182_ = rn;
#line 498 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp183_ = gxml_node_get_attrs (_tmp182_);
#line 498 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp184_ = _tmp183_;
#line 498 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp185_ = _tmp184_;
#line 498 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp186_ = gee_map_get (_tmp185_, "name");
#line 498 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp187_ = (GXmlNode*) _tmp186_;
#line 498 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_g_object_unref0 (_tmp185_);
#line 498 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														nattr = _tmp187_;
#line 499 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp188_ = nattr;
#line 499 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_vala_assert (_tmp188_ != NULL, "nattr != null");
#line 500 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp189_ = nattr;
#line 500 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp190_ = gxml_node_get_value (_tmp189_);
#line 500 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp191_ = _tmp190_;
#line 500 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_tmp192_ = _tmp191_;
#line 500 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_vala_assert (g_strcmp0 (_tmp192_, "nothing") == 0, "nattr.value == \"nothing\"");
#line 500 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_g_free0 (_tmp192_);
#line 496 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
														_g_object_unref0 (nattr);
#line 5259 "SerializableGeeCollections-TDocument-Test.c"
													}
												}
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
												_g_object_unref0 (rn);
#line 5264 "SerializableGeeCollections-TDocument-Test.c"
											}
#line 475 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
											_g_object_unref0 (_rn_list);
#line 5268 "SerializableGeeCollections-TDocument-Test.c"
										}
									}
								}
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
								_g_object_unref0 (sn);
#line 5274 "SerializableGeeCollections-TDocument-Test.c"
							}
#line 472 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
							_g_object_unref0 (_sn_list);
#line 5278 "SerializableGeeCollections-TDocument-Test.c"
						}
#line 507 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_tmp193_ = attrfound;
#line 507 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_vala_assert (_tmp193_, "attrfound");
#line 508 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_tmp194_ = tfound;
#line 508 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_vala_assert (_tmp194_, "tfound");
#line 509 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_tmp195_ = unkfound;
#line 509 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
						_vala_assert (_tmp195_, "unkfound");
#line 5292 "SerializableGeeCollections-TDocument-Test.c"
					}
				}
#line 512 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_tmp196_ = n;
#line 512 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp196_, GXML_TYPE_TEXT)) {
#line 5299 "SerializableGeeCollections-TDocument-Test.c"
					FILE* _tmp197_ = NULL;
					GXmlNode* _tmp198_ = NULL;
					gchar* _tmp199_ = NULL;
					gchar* _tmp200_ = NULL;
					gchar* _tmp201_ = NULL;
					const gchar* _tmp202_ = NULL;
					gchar* _tmp203_ = NULL;
					gchar* _tmp204_ = NULL;
#line 513 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp197_ = stdout;
#line 513 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp198_ = n;
#line 513 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp199_ = gxml_node_get_value (_tmp198_);
#line 513 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp200_ = _tmp199_;
#line 513 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp201_ = _tmp200_;
#line 513 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp202_ = string_to_string (_tmp201_);
#line 513 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp203_ = g_strconcat ("ROOT NODE VALUE: '", _tmp202_, "'\n", NULL);
#line 513 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_tmp204_ = _tmp203_;
#line 513 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					fprintf (_tmp197_, "%s", _tmp204_);
#line 513 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_free0 (_tmp204_);
#line 513 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
					_g_free0 (_tmp201_);
#line 5330 "SerializableGeeCollections-TDocument-Test.c"
				}
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
				_g_object_unref0 (n);
#line 5334 "SerializableGeeCollections-TDocument-Test.c"
			}
#line 463 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
			_g_object_unref0 (_n_list);
#line 5338 "SerializableGeeCollections-TDocument-Test.c"
		}
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (doc);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (refaction);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (mssh);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (s);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (org_doc);
#line 5350 "SerializableGeeCollections-TDocument-Test.c"
	}
	goto __finally99;
	__catch99_g_error:
	{
		GError* e = NULL;
		FILE* _tmp205_ = NULL;
		const gchar* _tmp206_ = NULL;
		const gchar* _tmp207_ = NULL;
		gchar* _tmp208_ = NULL;
		gchar* _tmp209_ = NULL;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		e = _inner_error_;
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_inner_error_ = NULL;
#line 518 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp205_ = stdout;
#line 518 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp206_ = e->message;
#line 518 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp207_ = string_to_string (_tmp206_);
#line 518 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp208_ = g_strconcat ("ERROR: ", _tmp207_, NULL);
#line 518 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp209_ = _tmp208_;
#line 518 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		fprintf (_tmp205_, "%s", _tmp209_);
#line 518 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_free0 (_tmp209_);
#line 519 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_assert_not_reached ();
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_error_free0 (e);
#line 5383 "SerializableGeeCollections-TDocument-Test.c"
	}
	__finally99:
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_clear_error (&_inner_error_);
#line 404 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		return;
#line 5394 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static void ___lambda71__gtest_func (void) {
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	__lambda71_ ();
#line 5402 "SerializableGeeCollections-TDocument-Test.c"
}


void serializable_gee_collections_tdocument_test_add_tests (void) {
#line 179 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_test_add_func ("/gxml/tw/serializable/convined_gee_containers/de-se-deserialize", ___lambda70__gtest_func);
#line 402 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_test_add_func ("/gxml/tw/serializable/convined_gee_containers/se-deserialize-unknowns", ___lambda71__gtest_func);
#line 5411 "SerializableGeeCollections-TDocument-Test.c"
}


SerializableGeeCollectionsTDocumentTest* serializable_gee_collections_tdocument_test_construct (GType object_type) {
	SerializableGeeCollectionsTDocumentTest* self = NULL;
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTest*) gxml_test_construct (object_type);
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self;
#line 5421 "SerializableGeeCollections-TDocument-Test.c"
}


SerializableGeeCollectionsTDocumentTest* serializable_gee_collections_tdocument_test_new (void) {
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return serializable_gee_collections_tdocument_test_construct (TYPE_SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST);
#line 5428 "SerializableGeeCollections-TDocument-Test.c"
}


static gboolean serializable_gee_collections_tdocument_test_citizen_real_serialize_use_xml_node_value (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTDocumentTestCitizen * self;
	gboolean result = FALSE;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestCitizen*) base;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = TRUE;
#line 34 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 5441 "SerializableGeeCollections-TDocument-Test.c"
}


static gchar* serializable_gee_collections_tdocument_test_citizen_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTDocumentTestCitizen * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
	gchar* _tmp2_ = NULL;
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestCitizen*) base;
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_ctype;
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = string_to_string (_tmp0_);
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp2_ = g_strconcat ("Citizen: ", _tmp1_, NULL);
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp2_;
#line 35 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 5463 "SerializableGeeCollections-TDocument-Test.c"
}


static gchar* serializable_gee_collections_tdocument_test_citizen_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTDocumentTestCitizen * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 36 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestCitizen*) base;
#line 36 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = g_strdup ("citizen");
#line 36 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 36 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 5479 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestCitizen* serializable_gee_collections_tdocument_test_citizen_construct (GType object_type) {
	SerializableGeeCollectionsTDocumentTestCitizen * self = NULL;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestCitizen*) gxml_serializable_object_model_construct (object_type);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self;
#line 5489 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestCitizen* serializable_gee_collections_tdocument_test_citizen_new (void) {
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return serializable_gee_collections_tdocument_test_citizen_construct (SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CITIZEN);
#line 5496 "SerializableGeeCollections-TDocument-Test.c"
}


static const gchar* serializable_gee_collections_tdocument_test_citizen_get_ctype (SerializableGeeCollectionsTDocumentTestCitizen* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_ctype;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 5511 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_citizen_set_ctype (SerializableGeeCollectionsTDocumentTestCitizen* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_if_fail (self != NULL);
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = value;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_free0 (self->priv->_ctype);
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv->_ctype = _tmp1_;
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_notify ((GObject *) self, "ctype");
#line 5530 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestCitizenArray* serializable_gee_collections_tdocument_test_citizen_array_construct (GType object_type) {
	SerializableGeeCollectionsTDocumentTestCitizenArray * self = NULL;
#line 38 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestCitizenArray*) gxml_serializable_array_list_construct (object_type, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CITIZEN, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 38 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self;
#line 5540 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestCitizenArray* serializable_gee_collections_tdocument_test_citizen_array_new (void) {
#line 38 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return serializable_gee_collections_tdocument_test_citizen_array_construct (SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_TYPE_ARRAY);
#line 5547 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_citizen_array_class_init (SerializableGeeCollectionsTDocumentTestCitizenArrayClass * klass) {
#line 38 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_citizen_array_parent_class = g_type_class_peek_parent (klass);
#line 5554 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_citizen_array_instance_init (SerializableGeeCollectionsTDocumentTestCitizenArray * self) {
}


GType serializable_gee_collections_tdocument_test_citizen_array_get_type (void) {
	static volatile gsize serializable_gee_collections_tdocument_test_citizen_array_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_tdocument_test_citizen_array_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTDocumentTestCitizenArrayClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_tdocument_test_citizen_array_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTDocumentTestCitizenArray), 0, (GInstanceInitFunc) serializable_gee_collections_tdocument_test_citizen_array_instance_init, NULL };
		GType serializable_gee_collections_tdocument_test_citizen_array_type_id;
		serializable_gee_collections_tdocument_test_citizen_array_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_ARRAY_LIST, "SerializableGeeCollectionsTDocumentTestCitizenArray", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_tdocument_test_citizen_array_type_id__volatile, serializable_gee_collections_tdocument_test_citizen_array_type_id);
	}
	return serializable_gee_collections_tdocument_test_citizen_array_type_id__volatile;
}


static void serializable_gee_collections_tdocument_test_citizen_class_init (SerializableGeeCollectionsTDocumentTestCitizenClass * klass) {
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_citizen_parent_class = g_type_class_peek_parent (klass);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTDocumentTestCitizenPrivate));
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableObjectModelClass *) klass)->serialize_use_xml_node_value = (gboolean (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_tdocument_test_citizen_real_serialize_use_xml_node_value;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_tdocument_test_citizen_real_to_string;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_tdocument_test_citizen_real_node_name;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_tdocument_test_citizen_get_property;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_tdocument_test_citizen_set_property;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_tdocument_test_citizen_finalize;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_CTYPE, g_param_spec_string ("ctype", "ctype", "ctype", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 5593 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_citizen_instance_init (SerializableGeeCollectionsTDocumentTestCitizen * self) {
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_GET_PRIVATE (self);
#line 5600 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_citizen_finalize (GObject* obj) {
	SerializableGeeCollectionsTDocumentTestCitizen * self;
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CITIZEN, SerializableGeeCollectionsTDocumentTestCitizen);
#line 33 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_free0 (self->priv->_ctype);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (serializable_gee_collections_tdocument_test_citizen_parent_class)->finalize (obj);
#line 5612 "SerializableGeeCollections-TDocument-Test.c"
}


static GType serializable_gee_collections_tdocument_test_citizen_get_type (void) {
	static volatile gsize serializable_gee_collections_tdocument_test_citizen_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_tdocument_test_citizen_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTDocumentTestCitizenClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_tdocument_test_citizen_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTDocumentTestCitizen), 0, (GInstanceInitFunc) serializable_gee_collections_tdocument_test_citizen_instance_init, NULL };
		GType serializable_gee_collections_tdocument_test_citizen_type_id;
		serializable_gee_collections_tdocument_test_citizen_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeCollectionsTDocumentTestCitizen", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_tdocument_test_citizen_type_id__volatile, serializable_gee_collections_tdocument_test_citizen_type_id);
	}
	return serializable_gee_collections_tdocument_test_citizen_type_id__volatile;
}


static void _vala_serializable_gee_collections_tdocument_test_citizen_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTDocumentTestCitizen * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CITIZEN, SerializableGeeCollectionsTDocumentTestCitizen);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	switch (property_id) {
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_CTYPE:
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_value_set_string (value, serializable_gee_collections_tdocument_test_citizen_get_ctype (self));
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 5639 "SerializableGeeCollections-TDocument-Test.c"
		default:
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 5645 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static void _vala_serializable_gee_collections_tdocument_test_citizen_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTDocumentTestCitizen * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CITIZEN, SerializableGeeCollectionsTDocumentTestCitizen);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	switch (property_id) {
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_CTYPE:
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_citizen_set_ctype (self, g_value_get_string (value));
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 5661 "SerializableGeeCollections-TDocument-Test.c"
		default:
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 31 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 5667 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static SerializableGeeCollectionsTDocumentTestAsteroid* serializable_gee_collections_tdocument_test_asteroid_construct_with_size (GType object_type, gint size) {
	SerializableGeeCollectionsTDocumentTestAsteroid * self = NULL;
	gint _tmp0_ = 0;
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestAsteroid*) gxml_serializable_object_model_construct (object_type);
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = size;
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_asteroid_set_size (self, _tmp0_);
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self;
#line 5683 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestAsteroid* serializable_gee_collections_tdocument_test_asteroid_new_with_size (gint size) {
#line 43 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return serializable_gee_collections_tdocument_test_asteroid_construct_with_size (SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_ASTEROID, size);
#line 5690 "SerializableGeeCollections-TDocument-Test.c"
}


static gchar* serializable_gee_collections_tdocument_test_asteroid_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTDocumentTestAsteroid * self;
	gchar* result = NULL;
	gint _tmp0_ = 0;
	gchar* _tmp1_ = NULL;
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestAsteroid*) base;
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_size;
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup_printf ("%i", _tmp0_);
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp1_;
#line 44 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 5709 "SerializableGeeCollections-TDocument-Test.c"
}


static gchar* serializable_gee_collections_tdocument_test_asteroid_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTDocumentTestAsteroid * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 45 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestAsteroid*) base;
#line 45 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = g_strdup ("asteroid");
#line 45 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 45 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 5725 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestAsteroid* serializable_gee_collections_tdocument_test_asteroid_construct (GType object_type) {
	SerializableGeeCollectionsTDocumentTestAsteroid * self = NULL;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestAsteroid*) gxml_serializable_object_model_construct (object_type);
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self;
#line 5735 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestAsteroid* serializable_gee_collections_tdocument_test_asteroid_new (void) {
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return serializable_gee_collections_tdocument_test_asteroid_construct (SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_ASTEROID);
#line 5742 "SerializableGeeCollections-TDocument-Test.c"
}


static gint serializable_gee_collections_tdocument_test_asteroid_get_size (SerializableGeeCollectionsTDocumentTestAsteroid* self) {
	gint result;
	gint _tmp0_ = 0;
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_size;
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 5757 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_asteroid_set_size (SerializableGeeCollectionsTDocumentTestAsteroid* self, gint value) {
	gint _tmp0_ = 0;
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_if_fail (self != NULL);
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = value;
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv->_size = _tmp0_;
#line 42 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_notify ((GObject *) self, "size");
#line 5771 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestAsteroidArray* serializable_gee_collections_tdocument_test_asteroid_array_construct (GType object_type) {
	SerializableGeeCollectionsTDocumentTestAsteroidArray * self = NULL;
#line 47 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestAsteroidArray*) gxml_serializable_array_list_construct (object_type, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_ASTEROID, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 47 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self;
#line 5781 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestAsteroidArray* serializable_gee_collections_tdocument_test_asteroid_array_new (void) {
#line 47 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return serializable_gee_collections_tdocument_test_asteroid_array_construct (SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_TYPE_ARRAY);
#line 5788 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_asteroid_array_class_init (SerializableGeeCollectionsTDocumentTestAsteroidArrayClass * klass) {
#line 47 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_asteroid_array_parent_class = g_type_class_peek_parent (klass);
#line 5795 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_asteroid_array_instance_init (SerializableGeeCollectionsTDocumentTestAsteroidArray * self) {
}


GType serializable_gee_collections_tdocument_test_asteroid_array_get_type (void) {
	static volatile gsize serializable_gee_collections_tdocument_test_asteroid_array_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_tdocument_test_asteroid_array_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTDocumentTestAsteroidArrayClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_tdocument_test_asteroid_array_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTDocumentTestAsteroidArray), 0, (GInstanceInitFunc) serializable_gee_collections_tdocument_test_asteroid_array_instance_init, NULL };
		GType serializable_gee_collections_tdocument_test_asteroid_array_type_id;
		serializable_gee_collections_tdocument_test_asteroid_array_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_ARRAY_LIST, "SerializableGeeCollectionsTDocumentTestAsteroidArray", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_tdocument_test_asteroid_array_type_id__volatile, serializable_gee_collections_tdocument_test_asteroid_array_type_id);
	}
	return serializable_gee_collections_tdocument_test_asteroid_array_type_id__volatile;
}


static void serializable_gee_collections_tdocument_test_asteroid_class_init (SerializableGeeCollectionsTDocumentTestAsteroidClass * klass) {
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_asteroid_parent_class = g_type_class_peek_parent (klass);
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTDocumentTestAsteroidPrivate));
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_tdocument_test_asteroid_real_to_string;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_tdocument_test_asteroid_real_node_name;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_tdocument_test_asteroid_get_property;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_tdocument_test_asteroid_set_property;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_tdocument_test_asteroid_finalize;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_SIZE, g_param_spec_int ("size", "size", "size", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 5832 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_asteroid_instance_init (SerializableGeeCollectionsTDocumentTestAsteroid * self) {
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_GET_PRIVATE (self);
#line 5839 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_asteroid_finalize (GObject* obj) {
	SerializableGeeCollectionsTDocumentTestAsteroid * self;
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_ASTEROID, SerializableGeeCollectionsTDocumentTestAsteroid);
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (serializable_gee_collections_tdocument_test_asteroid_parent_class)->finalize (obj);
#line 5849 "SerializableGeeCollections-TDocument-Test.c"
}


static GType serializable_gee_collections_tdocument_test_asteroid_get_type (void) {
	static volatile gsize serializable_gee_collections_tdocument_test_asteroid_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_tdocument_test_asteroid_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTDocumentTestAsteroidClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_tdocument_test_asteroid_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTDocumentTestAsteroid), 0, (GInstanceInitFunc) serializable_gee_collections_tdocument_test_asteroid_instance_init, NULL };
		GType serializable_gee_collections_tdocument_test_asteroid_type_id;
		serializable_gee_collections_tdocument_test_asteroid_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeCollectionsTDocumentTestAsteroid", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_tdocument_test_asteroid_type_id__volatile, serializable_gee_collections_tdocument_test_asteroid_type_id);
	}
	return serializable_gee_collections_tdocument_test_asteroid_type_id__volatile;
}


static void _vala_serializable_gee_collections_tdocument_test_asteroid_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTDocumentTestAsteroid * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_ASTEROID, SerializableGeeCollectionsTDocumentTestAsteroid);
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	switch (property_id) {
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_SIZE:
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_value_set_int (value, serializable_gee_collections_tdocument_test_asteroid_get_size (self));
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 5876 "SerializableGeeCollections-TDocument-Test.c"
		default:
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 5882 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static void _vala_serializable_gee_collections_tdocument_test_asteroid_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTDocumentTestAsteroid * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_ASTEROID, SerializableGeeCollectionsTDocumentTestAsteroid);
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	switch (property_id) {
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_SIZE:
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_asteroid_set_size (self, g_value_get_int (value));
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 5898 "SerializableGeeCollections-TDocument-Test.c"
		default:
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 40 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 5904 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static gchar* serializable_gee_collections_tdocument_test_planet_real_get_map_key (GXmlSerializableMapKey* base) {
	SerializableGeeCollectionsTDocumentTestPlanet * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestPlanet*) base;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_name;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp1_;
#line 51 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 5924 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestPlanet* serializable_gee_collections_tdocument_test_planet_construct_named (GType object_type, const gchar* name) {
	SerializableGeeCollectionsTDocumentTestPlanet * self = NULL;
	const gchar* _tmp0_ = NULL;
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestPlanet*) gxml_serializable_container_construct (object_type);
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = name;
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_planet_set_name (self, _tmp0_);
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self;
#line 5941 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestPlanet* serializable_gee_collections_tdocument_test_planet_new_named (const gchar* name) {
#line 54 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return serializable_gee_collections_tdocument_test_planet_construct_named (SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_PLANET, name);
#line 5948 "SerializableGeeCollections-TDocument-Test.c"
}


static gchar* serializable_gee_collections_tdocument_test_planet_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTDocumentTestPlanet * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestPlanet*) base;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = g_strdup ("planet");
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 55 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 5964 "SerializableGeeCollections-TDocument-Test.c"
}


static gchar* serializable_gee_collections_tdocument_test_planet_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTDocumentTestPlanet * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 56 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestPlanet*) base;
#line 56 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_name;
#line 56 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 56 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp1_;
#line 56 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 5983 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_planet_real_init_containers (GXmlSerializableContainer* base) {
	SerializableGeeCollectionsTDocumentTestPlanet * self;
	SerializableGeeCollectionsTDocumentTestCitizenArray* _tmp0_ = NULL;
#line 58 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestPlanet*) base;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_citizens;
#line 60 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	if (_tmp0_ == NULL) {
#line 5996 "SerializableGeeCollections-TDocument-Test.c"
		SerializableGeeCollectionsTDocumentTestCitizenArray* _tmp1_ = NULL;
		SerializableGeeCollectionsTDocumentTestCitizenArray* _tmp2_ = NULL;
#line 61 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp1_ = serializable_gee_collections_tdocument_test_citizen_array_new ();
#line 61 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp2_ = _tmp1_;
#line 61 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_planet_set_citizens (self, _tmp2_);
#line 61 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp2_);
#line 6007 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static SerializableGeeCollectionsTDocumentTestPlanet* serializable_gee_collections_tdocument_test_planet_construct (GType object_type) {
	SerializableGeeCollectionsTDocumentTestPlanet * self = NULL;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestPlanet*) gxml_serializable_container_construct (object_type);
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self;
#line 6018 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestPlanet* serializable_gee_collections_tdocument_test_planet_new (void) {
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return serializable_gee_collections_tdocument_test_planet_construct (SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_PLANET);
#line 6025 "SerializableGeeCollections-TDocument-Test.c"
}


static const gchar* serializable_gee_collections_tdocument_test_planet_get_name (SerializableGeeCollectionsTDocumentTestPlanet* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_name;
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 6040 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_planet_set_name (SerializableGeeCollectionsTDocumentTestPlanet* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_if_fail (self != NULL);
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = value;
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_free0 (self->priv->_name);
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv->_name = _tmp1_;
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_notify ((GObject *) self, "name");
#line 6059 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestCitizenArray* serializable_gee_collections_tdocument_test_planet_get_citizens (SerializableGeeCollectionsTDocumentTestPlanet* self) {
	SerializableGeeCollectionsTDocumentTestCitizenArray* result;
	SerializableGeeCollectionsTDocumentTestCitizenArray* _tmp0_ = NULL;
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_citizens;
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 6074 "SerializableGeeCollections-TDocument-Test.c"
}


static gpointer _g_object_ref0 (gpointer self) {
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self ? g_object_ref (self) : NULL;
#line 6081 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_planet_set_citizens (SerializableGeeCollectionsTDocumentTestPlanet* self, SerializableGeeCollectionsTDocumentTestCitizenArray* value) {
	SerializableGeeCollectionsTDocumentTestCitizenArray* _tmp0_ = NULL;
	SerializableGeeCollectionsTDocumentTestCitizenArray* _tmp1_ = NULL;
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_if_fail (self != NULL);
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = value;
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_object_unref0 (self->priv->_citizens);
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv->_citizens = _tmp1_;
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_notify ((GObject *) self, "citizens");
#line 6100 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestPlanetCollection* serializable_gee_collections_tdocument_test_planet_collection_construct (GType object_type) {
	SerializableGeeCollectionsTDocumentTestPlanetCollection * self = NULL;
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestPlanetCollection*) gxml_serializable_tree_map_construct (object_type, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_PLANET, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self;
#line 6110 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestPlanetCollection* serializable_gee_collections_tdocument_test_planet_collection_new (void) {
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return serializable_gee_collections_tdocument_test_planet_collection_construct (SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_TYPE_COLLECTION);
#line 6117 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_planet_collection_class_init (SerializableGeeCollectionsTDocumentTestPlanetCollectionClass * klass) {
#line 64 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_planet_collection_parent_class = g_type_class_peek_parent (klass);
#line 6124 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_planet_collection_instance_init (SerializableGeeCollectionsTDocumentTestPlanetCollection * self) {
}


GType serializable_gee_collections_tdocument_test_planet_collection_get_type (void) {
	static volatile gsize serializable_gee_collections_tdocument_test_planet_collection_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_tdocument_test_planet_collection_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTDocumentTestPlanetCollectionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_tdocument_test_planet_collection_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTDocumentTestPlanetCollection), 0, (GInstanceInitFunc) serializable_gee_collections_tdocument_test_planet_collection_instance_init, NULL };
		GType serializable_gee_collections_tdocument_test_planet_collection_type_id;
		serializable_gee_collections_tdocument_test_planet_collection_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_TREE_MAP, "SerializableGeeCollectionsTDocumentTestPlanetCollection", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_tdocument_test_planet_collection_type_id__volatile, serializable_gee_collections_tdocument_test_planet_collection_type_id);
	}
	return serializable_gee_collections_tdocument_test_planet_collection_type_id__volatile;
}


static void serializable_gee_collections_tdocument_test_planet_class_init (SerializableGeeCollectionsTDocumentTestPlanetClass * klass) {
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_planet_parent_class = g_type_class_peek_parent (klass);
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTDocumentTestPlanetPrivate));
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_tdocument_test_planet_real_node_name;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_tdocument_test_planet_real_to_string;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableContainerClass *) klass)->init_containers = (void (*)(GXmlSerializableContainer*)) serializable_gee_collections_tdocument_test_planet_real_init_containers;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_tdocument_test_planet_get_property;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_tdocument_test_planet_set_property;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_tdocument_test_planet_finalize;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_CITIZENS, g_param_spec_object ("citizens", "citizens", "citizens", SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CITIZEN_TYPE_ARRAY, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 6165 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_planet_gxml_serializable_map_key_interface_init (GXmlSerializableMapKeyIface * iface) {
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_planet_gxml_serializable_map_key_parent_iface = g_type_interface_peek_parent (iface);
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	iface->get_map_key = (gpointer (*)(GXmlSerializableMapKey*)) serializable_gee_collections_tdocument_test_planet_real_get_map_key;
#line 6174 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_planet_instance_init (SerializableGeeCollectionsTDocumentTestPlanet * self) {
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_GET_PRIVATE (self);
#line 6181 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_planet_finalize (GObject* obj) {
	SerializableGeeCollectionsTDocumentTestPlanet * self;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_PLANET, SerializableGeeCollectionsTDocumentTestPlanet);
#line 52 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_free0 (self->priv->_name);
#line 53 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_object_unref0 (self->priv->_citizens);
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (serializable_gee_collections_tdocument_test_planet_parent_class)->finalize (obj);
#line 6195 "SerializableGeeCollections-TDocument-Test.c"
}


static GType serializable_gee_collections_tdocument_test_planet_get_type (void) {
	static volatile gsize serializable_gee_collections_tdocument_test_planet_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_tdocument_test_planet_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTDocumentTestPlanetClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_tdocument_test_planet_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTDocumentTestPlanet), 0, (GInstanceInitFunc) serializable_gee_collections_tdocument_test_planet_instance_init, NULL };
		static const GInterfaceInfo gxml_serializable_map_key_info = { (GInterfaceInitFunc) serializable_gee_collections_tdocument_test_planet_gxml_serializable_map_key_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType serializable_gee_collections_tdocument_test_planet_type_id;
		serializable_gee_collections_tdocument_test_planet_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_CONTAINER, "SerializableGeeCollectionsTDocumentTestPlanet", &g_define_type_info, 0);
		g_type_add_interface_static (serializable_gee_collections_tdocument_test_planet_type_id, GXML_TYPE_SERIALIZABLE_MAP_KEY, &gxml_serializable_map_key_info);
		g_once_init_leave (&serializable_gee_collections_tdocument_test_planet_type_id__volatile, serializable_gee_collections_tdocument_test_planet_type_id);
	}
	return serializable_gee_collections_tdocument_test_planet_type_id__volatile;
}


static void _vala_serializable_gee_collections_tdocument_test_planet_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTDocumentTestPlanet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_PLANET, SerializableGeeCollectionsTDocumentTestPlanet);
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	switch (property_id) {
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_NAME:
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_value_set_string (value, serializable_gee_collections_tdocument_test_planet_get_name (self));
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_CITIZENS:
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_value_set_object (value, serializable_gee_collections_tdocument_test_planet_get_citizens (self));
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 6230 "SerializableGeeCollections-TDocument-Test.c"
		default:
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 6236 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static void _vala_serializable_gee_collections_tdocument_test_planet_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTDocumentTestPlanet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_PLANET, SerializableGeeCollectionsTDocumentTestPlanet);
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	switch (property_id) {
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_NAME:
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_planet_set_name (self, g_value_get_string (value));
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_CITIZENS:
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_planet_set_citizens (self, g_value_get_object (value));
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 6258 "SerializableGeeCollections-TDocument-Test.c"
		default:
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 49 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 6264 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static gchar* serializable_gee_collections_tdocument_test_space_real_get_map_key (GXmlSerializableMapKey* base) {
	SerializableGeeCollectionsTDocumentTestSpace * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestSpace*) base;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_name;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp1_;
#line 68 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 6284 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestSpace* serializable_gee_collections_tdocument_test_space_construct_named (GType object_type, const gchar* name) {
	SerializableGeeCollectionsTDocumentTestSpace * self = NULL;
	const gchar* _tmp0_ = NULL;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestSpace*) gxml_serializable_container_construct (object_type);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = name;
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_space_set_name (self, _tmp0_);
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self;
#line 6301 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestSpace* serializable_gee_collections_tdocument_test_space_new_named (const gchar* name) {
#line 73 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return serializable_gee_collections_tdocument_test_space_construct_named (SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE, name);
#line 6308 "SerializableGeeCollections-TDocument-Test.c"
}


static gchar* serializable_gee_collections_tdocument_test_space_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTDocumentTestSpace * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestSpace*) base;
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = g_strdup ("space");
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 75 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 6324 "SerializableGeeCollections-TDocument-Test.c"
}


static gchar* serializable_gee_collections_tdocument_test_space_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTDocumentTestSpace * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestSpace*) base;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_name;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp1_;
#line 76 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 6343 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_real_init_containers (GXmlSerializableContainer* base) {
	SerializableGeeCollectionsTDocumentTestSpace * self;
	SerializableGeeCollectionsTDocumentTestAsteroidArray* _tmp0_ = NULL;
	SerializableGeeCollectionsTDocumentTestPlanetCollection* _tmp3_ = NULL;
#line 77 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestSpace*) base;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_asteroids;
#line 79 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	if (_tmp0_ == NULL) {
#line 6357 "SerializableGeeCollections-TDocument-Test.c"
		SerializableGeeCollectionsTDocumentTestAsteroidArray* _tmp1_ = NULL;
		SerializableGeeCollectionsTDocumentTestAsteroidArray* _tmp2_ = NULL;
#line 80 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp1_ = serializable_gee_collections_tdocument_test_asteroid_array_new ();
#line 80 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp2_ = _tmp1_;
#line 80 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_space_set_asteroids (self, _tmp2_);
#line 80 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp2_);
#line 6368 "SerializableGeeCollections-TDocument-Test.c"
	}
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp3_ = self->priv->_planets;
#line 81 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	if (_tmp3_ == NULL) {
#line 6374 "SerializableGeeCollections-TDocument-Test.c"
		SerializableGeeCollectionsTDocumentTestPlanetCollection* _tmp4_ = NULL;
		SerializableGeeCollectionsTDocumentTestPlanetCollection* _tmp5_ = NULL;
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp4_ = serializable_gee_collections_tdocument_test_planet_collection_new ();
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp5_ = _tmp4_;
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_space_set_planets (self, _tmp5_);
#line 82 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp5_);
#line 6385 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static SerializableGeeCollectionsTDocumentTestSpace* serializable_gee_collections_tdocument_test_space_construct (GType object_type) {
	SerializableGeeCollectionsTDocumentTestSpace * self = NULL;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestSpace*) gxml_serializable_container_construct (object_type);
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self;
#line 6396 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestSpace* serializable_gee_collections_tdocument_test_space_new (void) {
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return serializable_gee_collections_tdocument_test_space_construct (SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE);
#line 6403 "SerializableGeeCollections-TDocument-Test.c"
}


static const gchar* serializable_gee_collections_tdocument_test_space_get_name (SerializableGeeCollectionsTDocumentTestSpace* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_name;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 6418 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_set_name (SerializableGeeCollectionsTDocumentTestSpace* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_if_fail (self != NULL);
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = value;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_free0 (self->priv->_name);
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv->_name = _tmp1_;
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_notify ((GObject *) self, "name");
#line 6437 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestPlanetCollection* serializable_gee_collections_tdocument_test_space_get_planets (SerializableGeeCollectionsTDocumentTestSpace* self) {
	SerializableGeeCollectionsTDocumentTestPlanetCollection* result;
	SerializableGeeCollectionsTDocumentTestPlanetCollection* _tmp0_ = NULL;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_planets;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 6452 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_set_planets (SerializableGeeCollectionsTDocumentTestSpace* self, SerializableGeeCollectionsTDocumentTestPlanetCollection* value) {
	SerializableGeeCollectionsTDocumentTestPlanetCollection* _tmp0_ = NULL;
	SerializableGeeCollectionsTDocumentTestPlanetCollection* _tmp1_ = NULL;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_if_fail (self != NULL);
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = value;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_object_unref0 (self->priv->_planets);
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv->_planets = _tmp1_;
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_notify ((GObject *) self, "planets");
#line 6471 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestAsteroidArray* serializable_gee_collections_tdocument_test_space_get_asteroids (SerializableGeeCollectionsTDocumentTestSpace* self) {
	SerializableGeeCollectionsTDocumentTestAsteroidArray* result;
	SerializableGeeCollectionsTDocumentTestAsteroidArray* _tmp0_ = NULL;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_asteroids;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 6486 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_set_asteroids (SerializableGeeCollectionsTDocumentTestSpace* self, SerializableGeeCollectionsTDocumentTestAsteroidArray* value) {
	SerializableGeeCollectionsTDocumentTestAsteroidArray* _tmp0_ = NULL;
	SerializableGeeCollectionsTDocumentTestAsteroidArray* _tmp1_ = NULL;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_if_fail (self != NULL);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = value;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_object_unref0 (self->priv->_asteroids);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv->_asteroids = _tmp1_;
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_notify ((GObject *) self, "asteroids");
#line 6505 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestSpaceCollection* serializable_gee_collections_tdocument_test_space_collection_construct (GType object_type) {
	SerializableGeeCollectionsTDocumentTestSpaceCollection * self = NULL;
#line 84 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestSpaceCollection*) gxml_serializable_tree_map_construct (object_type, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 84 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self;
#line 6515 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestSpaceCollection* serializable_gee_collections_tdocument_test_space_collection_new (void) {
#line 84 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return serializable_gee_collections_tdocument_test_space_collection_construct (SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_TYPE_COLLECTION);
#line 6522 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_collection_class_init (SerializableGeeCollectionsTDocumentTestSpaceCollectionClass * klass) {
#line 84 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_space_collection_parent_class = g_type_class_peek_parent (klass);
#line 6529 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_collection_instance_init (SerializableGeeCollectionsTDocumentTestSpaceCollection * self) {
}


GType serializable_gee_collections_tdocument_test_space_collection_get_type (void) {
	static volatile gsize serializable_gee_collections_tdocument_test_space_collection_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_tdocument_test_space_collection_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTDocumentTestSpaceCollectionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_tdocument_test_space_collection_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTDocumentTestSpaceCollection), 0, (GInstanceInitFunc) serializable_gee_collections_tdocument_test_space_collection_instance_init, NULL };
		GType serializable_gee_collections_tdocument_test_space_collection_type_id;
		serializable_gee_collections_tdocument_test_space_collection_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_TREE_MAP, "SerializableGeeCollectionsTDocumentTestSpaceCollection", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_tdocument_test_space_collection_type_id__volatile, serializable_gee_collections_tdocument_test_space_collection_type_id);
	}
	return serializable_gee_collections_tdocument_test_space_collection_type_id__volatile;
}


static void serializable_gee_collections_tdocument_test_space_class_init (SerializableGeeCollectionsTDocumentTestSpaceClass * klass) {
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_space_parent_class = g_type_class_peek_parent (klass);
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTDocumentTestSpacePrivate));
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_tdocument_test_space_real_node_name;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_tdocument_test_space_real_to_string;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableContainerClass *) klass)->init_containers = (void (*)(GXmlSerializableContainer*)) serializable_gee_collections_tdocument_test_space_real_init_containers;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_tdocument_test_space_get_property;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_tdocument_test_space_set_property;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_tdocument_test_space_finalize;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_PLANETS, g_param_spec_object ("planets", "planets", "planets", SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_PLANET_TYPE_COLLECTION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_ASTEROIDS, g_param_spec_object ("asteroids", "asteroids", "asteroids", SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_ASTEROID_TYPE_ARRAY, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 6572 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_gxml_serializable_map_key_interface_init (GXmlSerializableMapKeyIface * iface) {
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_space_gxml_serializable_map_key_parent_iface = g_type_interface_peek_parent (iface);
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	iface->get_map_key = (gpointer (*)(GXmlSerializableMapKey*)) serializable_gee_collections_tdocument_test_space_real_get_map_key;
#line 6581 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_instance_init (SerializableGeeCollectionsTDocumentTestSpace * self) {
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_GET_PRIVATE (self);
#line 6588 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_finalize (GObject* obj) {
	SerializableGeeCollectionsTDocumentTestSpace * self;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE, SerializableGeeCollectionsTDocumentTestSpace);
#line 69 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_free0 (self->priv->_name);
#line 70 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_object_unref0 (self->priv->_planets);
#line 71 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_object_unref0 (self->priv->_asteroids);
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (serializable_gee_collections_tdocument_test_space_parent_class)->finalize (obj);
#line 6604 "SerializableGeeCollections-TDocument-Test.c"
}


static GType serializable_gee_collections_tdocument_test_space_get_type (void) {
	static volatile gsize serializable_gee_collections_tdocument_test_space_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_tdocument_test_space_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTDocumentTestSpaceClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_tdocument_test_space_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTDocumentTestSpace), 0, (GInstanceInitFunc) serializable_gee_collections_tdocument_test_space_instance_init, NULL };
		static const GInterfaceInfo gxml_serializable_map_key_info = { (GInterfaceInitFunc) serializable_gee_collections_tdocument_test_space_gxml_serializable_map_key_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType serializable_gee_collections_tdocument_test_space_type_id;
		serializable_gee_collections_tdocument_test_space_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_CONTAINER, "SerializableGeeCollectionsTDocumentTestSpace", &g_define_type_info, 0);
		g_type_add_interface_static (serializable_gee_collections_tdocument_test_space_type_id, GXML_TYPE_SERIALIZABLE_MAP_KEY, &gxml_serializable_map_key_info);
		g_once_init_leave (&serializable_gee_collections_tdocument_test_space_type_id__volatile, serializable_gee_collections_tdocument_test_space_type_id);
	}
	return serializable_gee_collections_tdocument_test_space_type_id__volatile;
}


static void _vala_serializable_gee_collections_tdocument_test_space_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTDocumentTestSpace * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE, SerializableGeeCollectionsTDocumentTestSpace);
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	switch (property_id) {
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_NAME:
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_value_set_string (value, serializable_gee_collections_tdocument_test_space_get_name (self));
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_PLANETS:
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_value_set_object (value, serializable_gee_collections_tdocument_test_space_get_planets (self));
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_ASTEROIDS:
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_value_set_object (value, serializable_gee_collections_tdocument_test_space_get_asteroids (self));
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 6645 "SerializableGeeCollections-TDocument-Test.c"
		default:
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 6651 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static void _vala_serializable_gee_collections_tdocument_test_space_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTDocumentTestSpace * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE, SerializableGeeCollectionsTDocumentTestSpace);
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	switch (property_id) {
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_NAME:
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_space_set_name (self, g_value_get_string (value));
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_PLANETS:
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_space_set_planets (self, g_value_get_object (value));
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_ASTEROIDS:
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_space_set_asteroids (self, g_value_get_object (value));
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 6679 "SerializableGeeCollections-TDocument-Test.c"
		default:
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 66 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 6685 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static SerializableGeeCollectionsTDocumentTestRefaction* serializable_gee_collections_tdocument_test_refaction_construct_named (GType object_type, const gchar* manufacturer, const gchar* model) {
	SerializableGeeCollectionsTDocumentTestRefaction * self = NULL;
	const gchar* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (manufacturer != NULL, NULL);
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (model != NULL, NULL);
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestRefaction*) gxml_serializable_container_construct (object_type);
#line 94 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = manufacturer;
#line 94 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_refaction_set_manufacturer (self, _tmp0_);
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = model;
#line 95 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_refaction_set_model (self, _tmp1_);
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self;
#line 6710 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestRefaction* serializable_gee_collections_tdocument_test_refaction_new_named (const gchar* manufacturer, const gchar* model) {
#line 92 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return serializable_gee_collections_tdocument_test_refaction_construct_named (SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_REFACTION, manufacturer, model);
#line 6717 "SerializableGeeCollections-TDocument-Test.c"
}


static gchar* serializable_gee_collections_tdocument_test_refaction_real_get_map_primary_key (GXmlSerializableMapDualKey* base) {
	SerializableGeeCollectionsTDocumentTestRefaction * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestRefaction*) base;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_manufacturer;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp1_;
#line 98 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 6736 "SerializableGeeCollections-TDocument-Test.c"
}


static gchar* serializable_gee_collections_tdocument_test_refaction_real_get_map_secondary_key (GXmlSerializableMapDualKey* base) {
	SerializableGeeCollectionsTDocumentTestRefaction * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 99 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestRefaction*) base;
#line 99 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_model;
#line 99 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 99 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp1_;
#line 99 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 6755 "SerializableGeeCollections-TDocument-Test.c"
}


static gchar* serializable_gee_collections_tdocument_test_refaction_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTDocumentTestRefaction * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 101 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestRefaction*) base;
#line 101 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = g_strdup ("refaction");
#line 101 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 101 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 6771 "SerializableGeeCollections-TDocument-Test.c"
}


static gchar* serializable_gee_collections_tdocument_test_refaction_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTDocumentTestRefaction * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 102 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestRefaction*) base;
#line 102 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_model;
#line 102 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 102 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp1_;
#line 102 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 6790 "SerializableGeeCollections-TDocument-Test.c"
}


static gboolean serializable_gee_collections_tdocument_test_refaction_real_get_enable_unknown_serializable_property (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTDocumentTestRefaction * self;
	gboolean result = FALSE;
#line 103 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestRefaction*) base;
#line 103 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = TRUE;
#line 103 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 6803 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_refaction_real_init_containers (GXmlSerializableContainer* base) {
	SerializableGeeCollectionsTDocumentTestRefaction * self;
	SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp0_ = NULL;
#line 105 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestRefaction*) base;
#line 107 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_spaceships;
#line 107 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	if (_tmp0_ == NULL) {
#line 6816 "SerializableGeeCollections-TDocument-Test.c"
		SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp1_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp2_ = NULL;
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp1_ = serializable_gee_collections_tdocument_test_space_ship_collection_new ();
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp2_ = _tmp1_;
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_refaction_set_spaceships (self, _tmp2_);
#line 108 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp2_);
#line 6827 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static SerializableGeeCollectionsTDocumentTestRefaction* serializable_gee_collections_tdocument_test_refaction_construct (GType object_type) {
	SerializableGeeCollectionsTDocumentTestRefaction * self = NULL;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestRefaction*) gxml_serializable_container_construct (object_type);
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self;
#line 6838 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestRefaction* serializable_gee_collections_tdocument_test_refaction_new (void) {
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return serializable_gee_collections_tdocument_test_refaction_construct (SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_REFACTION);
#line 6845 "SerializableGeeCollections-TDocument-Test.c"
}


static const gchar* serializable_gee_collections_tdocument_test_refaction_get_model (SerializableGeeCollectionsTDocumentTestRefaction* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_model;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 6860 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_refaction_set_model (SerializableGeeCollectionsTDocumentTestRefaction* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_if_fail (self != NULL);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = value;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_free0 (self->priv->_model);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv->_model = _tmp1_;
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_notify ((GObject *) self, "model");
#line 6879 "SerializableGeeCollections-TDocument-Test.c"
}


static const gchar* serializable_gee_collections_tdocument_test_refaction_get_manufacturer (SerializableGeeCollectionsTDocumentTestRefaction* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 89 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 89 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_manufacturer;
#line 89 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 89 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 6894 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_refaction_set_manufacturer (SerializableGeeCollectionsTDocumentTestRefaction* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 89 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_if_fail (self != NULL);
#line 89 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = value;
#line 89 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 89 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_free0 (self->priv->_manufacturer);
#line 89 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv->_manufacturer = _tmp1_;
#line 89 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_notify ((GObject *) self, "manufacturer");
#line 6913 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestSpaceShipCollection* serializable_gee_collections_tdocument_test_refaction_get_spaceships (SerializableGeeCollectionsTDocumentTestRefaction* self) {
	SerializableGeeCollectionsTDocumentTestSpaceShipCollection* result;
	SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp0_ = NULL;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_spaceships;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 6928 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_refaction_set_spaceships (SerializableGeeCollectionsTDocumentTestRefaction* self, SerializableGeeCollectionsTDocumentTestSpaceShipCollection* value) {
	SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp0_ = NULL;
	SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp1_ = NULL;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_if_fail (self != NULL);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = value;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_object_unref0 (self->priv->_spaceships);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv->_spaceships = _tmp1_;
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_notify ((GObject *) self, "spaceships");
#line 6947 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestRefactionCollection* serializable_gee_collections_tdocument_test_refaction_collection_construct (GType object_type) {
	SerializableGeeCollectionsTDocumentTestRefactionCollection * self = NULL;
#line 111 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestRefactionCollection*) gxml_serializable_dual_key_map_construct (object_type, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_REFACTION, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 111 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self;
#line 6957 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestRefactionCollection* serializable_gee_collections_tdocument_test_refaction_collection_new (void) {
#line 111 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return serializable_gee_collections_tdocument_test_refaction_collection_construct (SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_TYPE_COLLECTION);
#line 6964 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_refaction_collection_class_init (SerializableGeeCollectionsTDocumentTestRefactionCollectionClass * klass) {
#line 111 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_refaction_collection_parent_class = g_type_class_peek_parent (klass);
#line 6971 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_refaction_collection_instance_init (SerializableGeeCollectionsTDocumentTestRefactionCollection * self) {
}


GType serializable_gee_collections_tdocument_test_refaction_collection_get_type (void) {
	static volatile gsize serializable_gee_collections_tdocument_test_refaction_collection_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_tdocument_test_refaction_collection_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTDocumentTestRefactionCollectionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_tdocument_test_refaction_collection_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTDocumentTestRefactionCollection), 0, (GInstanceInitFunc) serializable_gee_collections_tdocument_test_refaction_collection_instance_init, NULL };
		GType serializable_gee_collections_tdocument_test_refaction_collection_type_id;
		serializable_gee_collections_tdocument_test_refaction_collection_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_DUAL_KEY_MAP, "SerializableGeeCollectionsTDocumentTestRefactionCollection", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_tdocument_test_refaction_collection_type_id__volatile, serializable_gee_collections_tdocument_test_refaction_collection_type_id);
	}
	return serializable_gee_collections_tdocument_test_refaction_collection_type_id__volatile;
}


static void serializable_gee_collections_tdocument_test_refaction_class_init (SerializableGeeCollectionsTDocumentTestRefactionClass * klass) {
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_refaction_parent_class = g_type_class_peek_parent (klass);
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTDocumentTestRefactionPrivate));
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_tdocument_test_refaction_real_node_name;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_tdocument_test_refaction_real_to_string;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableObjectModelClass *) klass)->get_enable_unknown_serializable_property = (gboolean (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_tdocument_test_refaction_real_get_enable_unknown_serializable_property;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableContainerClass *) klass)->init_containers = (void (*)(GXmlSerializableContainer*)) serializable_gee_collections_tdocument_test_refaction_real_init_containers;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_tdocument_test_refaction_get_property;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_tdocument_test_refaction_set_property;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_tdocument_test_refaction_finalize;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_MODEL, g_param_spec_string ("model", "model", "model", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_MANUFACTURER, g_param_spec_string ("manufacturer", "manufacturer", "manufacturer", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_SPACESHIPS, g_param_spec_object ("spaceships", "spaceships", "spaceships", SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_TYPE_COLLECTION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 7016 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_refaction_gxml_serializable_map_dual_key_interface_init (GXmlSerializableMapDualKeyIface * iface) {
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_refaction_gxml_serializable_map_dual_key_parent_iface = g_type_interface_peek_parent (iface);
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	iface->get_map_primary_key = (gpointer (*)(GXmlSerializableMapDualKey*)) serializable_gee_collections_tdocument_test_refaction_real_get_map_primary_key;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	iface->get_map_secondary_key = (gpointer (*)(GXmlSerializableMapDualKey*)) serializable_gee_collections_tdocument_test_refaction_real_get_map_secondary_key;
#line 7027 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_refaction_instance_init (SerializableGeeCollectionsTDocumentTestRefaction * self) {
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_GET_PRIVATE (self);
#line 7034 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_refaction_finalize (GObject* obj) {
	SerializableGeeCollectionsTDocumentTestRefaction * self;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_REFACTION, SerializableGeeCollectionsTDocumentTestRefaction);
#line 88 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_free0 (self->priv->_model);
#line 89 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_free0 (self->priv->_manufacturer);
#line 90 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_object_unref0 (self->priv->_spaceships);
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (serializable_gee_collections_tdocument_test_refaction_parent_class)->finalize (obj);
#line 7050 "SerializableGeeCollections-TDocument-Test.c"
}


static GType serializable_gee_collections_tdocument_test_refaction_get_type (void) {
	static volatile gsize serializable_gee_collections_tdocument_test_refaction_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_tdocument_test_refaction_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTDocumentTestRefactionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_tdocument_test_refaction_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTDocumentTestRefaction), 0, (GInstanceInitFunc) serializable_gee_collections_tdocument_test_refaction_instance_init, NULL };
		static const GInterfaceInfo gxml_serializable_map_dual_key_info = { (GInterfaceInitFunc) serializable_gee_collections_tdocument_test_refaction_gxml_serializable_map_dual_key_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType serializable_gee_collections_tdocument_test_refaction_type_id;
		serializable_gee_collections_tdocument_test_refaction_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_CONTAINER, "SerializableGeeCollectionsTDocumentTestRefaction", &g_define_type_info, 0);
		g_type_add_interface_static (serializable_gee_collections_tdocument_test_refaction_type_id, GXML_TYPE_SERIALIZABLE_MAP_DUAL_KEY, &gxml_serializable_map_dual_key_info);
		g_once_init_leave (&serializable_gee_collections_tdocument_test_refaction_type_id__volatile, serializable_gee_collections_tdocument_test_refaction_type_id);
	}
	return serializable_gee_collections_tdocument_test_refaction_type_id__volatile;
}


static void _vala_serializable_gee_collections_tdocument_test_refaction_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTDocumentTestRefaction * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_REFACTION, SerializableGeeCollectionsTDocumentTestRefaction);
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	switch (property_id) {
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_MODEL:
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_value_set_string (value, serializable_gee_collections_tdocument_test_refaction_get_model (self));
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_MANUFACTURER:
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_value_set_string (value, serializable_gee_collections_tdocument_test_refaction_get_manufacturer (self));
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_SPACESHIPS:
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_value_set_object (value, serializable_gee_collections_tdocument_test_refaction_get_spaceships (self));
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 7091 "SerializableGeeCollections-TDocument-Test.c"
		default:
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 7097 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static void _vala_serializable_gee_collections_tdocument_test_refaction_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTDocumentTestRefaction * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_REFACTION, SerializableGeeCollectionsTDocumentTestRefaction);
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	switch (property_id) {
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_MODEL:
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_refaction_set_model (self, g_value_get_string (value));
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_MANUFACTURER:
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_refaction_set_manufacturer (self, g_value_get_string (value));
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_SPACESHIPS:
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_refaction_set_spaceships (self, g_value_get_object (value));
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 7125 "SerializableGeeCollections-TDocument-Test.c"
		default:
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 86 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 7131 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static SerializableGeeCollectionsTDocumentTestSpaceShip* serializable_gee_collections_tdocument_test_space_ship_construct_named (GType object_type, const gchar* manufacturer, const gchar* model) {
	SerializableGeeCollectionsTDocumentTestSpaceShip * self = NULL;
	const gchar* _tmp0_ = NULL;
	const gchar* _tmp1_ = NULL;
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (manufacturer != NULL, NULL);
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (model != NULL, NULL);
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestSpaceShip*) gxml_serializable_container_construct (object_type);
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = manufacturer;
#line 121 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_space_ship_set_manufacturer (self, _tmp0_);
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = model;
#line 122 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_space_ship_set_model (self, _tmp1_);
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self;
#line 7156 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestSpaceShip* serializable_gee_collections_tdocument_test_space_ship_new_named (const gchar* manufacturer, const gchar* model) {
#line 119 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return serializable_gee_collections_tdocument_test_space_ship_construct_named (SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_SHIP, manufacturer, model);
#line 7163 "SerializableGeeCollections-TDocument-Test.c"
}


static gchar* serializable_gee_collections_tdocument_test_space_ship_real_get_map_primary_key (GXmlSerializableMapDualKey* base) {
	SerializableGeeCollectionsTDocumentTestSpaceShip * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 125 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestSpaceShip*) base;
#line 125 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_manufacturer;
#line 125 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 125 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp1_;
#line 125 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 7182 "SerializableGeeCollections-TDocument-Test.c"
}


static gchar* serializable_gee_collections_tdocument_test_space_ship_real_get_map_secondary_key (GXmlSerializableMapDualKey* base) {
	SerializableGeeCollectionsTDocumentTestSpaceShip * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestSpaceShip*) base;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_model;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp1_;
#line 126 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 7201 "SerializableGeeCollections-TDocument-Test.c"
}


static gchar* serializable_gee_collections_tdocument_test_space_ship_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTDocumentTestSpaceShip * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestSpaceShip*) base;
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = g_strdup ("ship");
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 128 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 7217 "SerializableGeeCollections-TDocument-Test.c"
}


static gchar* serializable_gee_collections_tdocument_test_space_ship_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTDocumentTestSpaceShip * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 129 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestSpaceShip*) base;
#line 129 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_model;
#line 129 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 129 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp1_;
#line 129 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 7236 "SerializableGeeCollections-TDocument-Test.c"
}


static gboolean serializable_gee_collections_tdocument_test_space_ship_real_get_enable_unknown_serializable_property (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTDocumentTestSpaceShip * self;
	gboolean result = FALSE;
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestSpaceShip*) base;
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = TRUE;
#line 130 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 7249 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_ship_real_init_containers (GXmlSerializableContainer* base) {
	SerializableGeeCollectionsTDocumentTestSpaceShip * self;
	SerializableGeeCollectionsTDocumentTestSpaceCollection* _tmp0_ = NULL;
#line 132 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestSpaceShip*) base;
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_spaces;
#line 134 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	if (_tmp0_ == NULL) {
#line 7262 "SerializableGeeCollections-TDocument-Test.c"
		SerializableGeeCollectionsTDocumentTestSpaceCollection* _tmp1_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceCollection* _tmp2_ = NULL;
#line 135 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp1_ = serializable_gee_collections_tdocument_test_space_collection_new ();
#line 135 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp2_ = _tmp1_;
#line 135 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_space_ship_set_spaces (self, _tmp2_);
#line 135 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp2_);
#line 7273 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static SerializableGeeCollectionsTDocumentTestSpaceShip* serializable_gee_collections_tdocument_test_space_ship_construct (GType object_type) {
	SerializableGeeCollectionsTDocumentTestSpaceShip * self = NULL;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestSpaceShip*) gxml_serializable_container_construct (object_type);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self;
#line 7284 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestSpaceShip* serializable_gee_collections_tdocument_test_space_ship_new (void) {
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return serializable_gee_collections_tdocument_test_space_ship_construct (SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_SHIP);
#line 7291 "SerializableGeeCollections-TDocument-Test.c"
}


static const gchar* serializable_gee_collections_tdocument_test_space_ship_get_model (SerializableGeeCollectionsTDocumentTestSpaceShip* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_model;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 7306 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_ship_set_model (SerializableGeeCollectionsTDocumentTestSpaceShip* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_if_fail (self != NULL);
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = value;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_free0 (self->priv->_model);
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv->_model = _tmp1_;
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_notify ((GObject *) self, "model");
#line 7325 "SerializableGeeCollections-TDocument-Test.c"
}


static const gchar* serializable_gee_collections_tdocument_test_space_ship_get_manufacturer (SerializableGeeCollectionsTDocumentTestSpaceShip* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 116 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 116 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_manufacturer;
#line 116 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 116 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 7340 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_ship_set_manufacturer (SerializableGeeCollectionsTDocumentTestSpaceShip* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 116 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_if_fail (self != NULL);
#line 116 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = value;
#line 116 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 116 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_free0 (self->priv->_manufacturer);
#line 116 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv->_manufacturer = _tmp1_;
#line 116 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_notify ((GObject *) self, "manufacturer");
#line 7359 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestSpaceCollection* serializable_gee_collections_tdocument_test_space_ship_get_spaces (SerializableGeeCollectionsTDocumentTestSpaceShip* self) {
	SerializableGeeCollectionsTDocumentTestSpaceCollection* result;
	SerializableGeeCollectionsTDocumentTestSpaceCollection* _tmp0_ = NULL;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_spaces;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 7374 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_ship_set_spaces (SerializableGeeCollectionsTDocumentTestSpaceShip* self, SerializableGeeCollectionsTDocumentTestSpaceCollection* value) {
	SerializableGeeCollectionsTDocumentTestSpaceCollection* _tmp0_ = NULL;
	SerializableGeeCollectionsTDocumentTestSpaceCollection* _tmp1_ = NULL;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_if_fail (self != NULL);
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = value;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_object_unref0 (self->priv->_spaces);
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv->_spaces = _tmp1_;
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_notify ((GObject *) self, "spaces");
#line 7393 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestSpaceShipCollection* serializable_gee_collections_tdocument_test_space_ship_collection_construct (GType object_type) {
	SerializableGeeCollectionsTDocumentTestSpaceShipCollection * self = NULL;
#line 138 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestSpaceShipCollection*) gxml_serializable_dual_key_map_construct (object_type, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_SHIP, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref);
#line 138 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self;
#line 7403 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestSpaceShipCollection* serializable_gee_collections_tdocument_test_space_ship_collection_new (void) {
#line 138 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return serializable_gee_collections_tdocument_test_space_ship_collection_construct (SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_TYPE_COLLECTION);
#line 7410 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_ship_collection_class_init (SerializableGeeCollectionsTDocumentTestSpaceShipCollectionClass * klass) {
#line 138 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_space_ship_collection_parent_class = g_type_class_peek_parent (klass);
#line 7417 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_ship_collection_instance_init (SerializableGeeCollectionsTDocumentTestSpaceShipCollection * self) {
}


GType serializable_gee_collections_tdocument_test_space_ship_collection_get_type (void) {
	static volatile gsize serializable_gee_collections_tdocument_test_space_ship_collection_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_tdocument_test_space_ship_collection_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTDocumentTestSpaceShipCollectionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_tdocument_test_space_ship_collection_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTDocumentTestSpaceShipCollection), 0, (GInstanceInitFunc) serializable_gee_collections_tdocument_test_space_ship_collection_instance_init, NULL };
		GType serializable_gee_collections_tdocument_test_space_ship_collection_type_id;
		serializable_gee_collections_tdocument_test_space_ship_collection_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_DUAL_KEY_MAP, "SerializableGeeCollectionsTDocumentTestSpaceShipCollection", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_tdocument_test_space_ship_collection_type_id__volatile, serializable_gee_collections_tdocument_test_space_ship_collection_type_id);
	}
	return serializable_gee_collections_tdocument_test_space_ship_collection_type_id__volatile;
}


static void serializable_gee_collections_tdocument_test_space_ship_class_init (SerializableGeeCollectionsTDocumentTestSpaceShipClass * klass) {
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_space_ship_parent_class = g_type_class_peek_parent (klass);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTDocumentTestSpaceShipPrivate));
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_tdocument_test_space_ship_real_node_name;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_tdocument_test_space_ship_real_to_string;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableObjectModelClass *) klass)->get_enable_unknown_serializable_property = (gboolean (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_tdocument_test_space_ship_real_get_enable_unknown_serializable_property;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableContainerClass *) klass)->init_containers = (void (*)(GXmlSerializableContainer*)) serializable_gee_collections_tdocument_test_space_ship_real_init_containers;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_tdocument_test_space_ship_get_property;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_tdocument_test_space_ship_set_property;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_tdocument_test_space_ship_finalize;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_MODEL, g_param_spec_string ("model", "model", "model", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_MANUFACTURER, g_param_spec_string ("manufacturer", "manufacturer", "manufacturer", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_SPACES, g_param_spec_object ("spaces", "spaces", "spaces", SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_TYPE_COLLECTION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 7462 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_ship_gxml_serializable_map_dual_key_interface_init (GXmlSerializableMapDualKeyIface * iface) {
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_space_ship_gxml_serializable_map_dual_key_parent_iface = g_type_interface_peek_parent (iface);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	iface->get_map_primary_key = (gpointer (*)(GXmlSerializableMapDualKey*)) serializable_gee_collections_tdocument_test_space_ship_real_get_map_primary_key;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	iface->get_map_secondary_key = (gpointer (*)(GXmlSerializableMapDualKey*)) serializable_gee_collections_tdocument_test_space_ship_real_get_map_secondary_key;
#line 7473 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_ship_instance_init (SerializableGeeCollectionsTDocumentTestSpaceShip * self) {
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_GET_PRIVATE (self);
#line 7480 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_ship_finalize (GObject* obj) {
	SerializableGeeCollectionsTDocumentTestSpaceShip * self;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_SHIP, SerializableGeeCollectionsTDocumentTestSpaceShip);
#line 115 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_free0 (self->priv->_model);
#line 116 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_free0 (self->priv->_manufacturer);
#line 117 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_object_unref0 (self->priv->_spaces);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (serializable_gee_collections_tdocument_test_space_ship_parent_class)->finalize (obj);
#line 7496 "SerializableGeeCollections-TDocument-Test.c"
}


static GType serializable_gee_collections_tdocument_test_space_ship_get_type (void) {
	static volatile gsize serializable_gee_collections_tdocument_test_space_ship_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_tdocument_test_space_ship_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTDocumentTestSpaceShipClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_tdocument_test_space_ship_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTDocumentTestSpaceShip), 0, (GInstanceInitFunc) serializable_gee_collections_tdocument_test_space_ship_instance_init, NULL };
		static const GInterfaceInfo gxml_serializable_map_dual_key_info = { (GInterfaceInitFunc) serializable_gee_collections_tdocument_test_space_ship_gxml_serializable_map_dual_key_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType serializable_gee_collections_tdocument_test_space_ship_type_id;
		serializable_gee_collections_tdocument_test_space_ship_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_CONTAINER, "SerializableGeeCollectionsTDocumentTestSpaceShip", &g_define_type_info, 0);
		g_type_add_interface_static (serializable_gee_collections_tdocument_test_space_ship_type_id, GXML_TYPE_SERIALIZABLE_MAP_DUAL_KEY, &gxml_serializable_map_dual_key_info);
		g_once_init_leave (&serializable_gee_collections_tdocument_test_space_ship_type_id__volatile, serializable_gee_collections_tdocument_test_space_ship_type_id);
	}
	return serializable_gee_collections_tdocument_test_space_ship_type_id__volatile;
}


static void _vala_serializable_gee_collections_tdocument_test_space_ship_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTDocumentTestSpaceShip * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_SHIP, SerializableGeeCollectionsTDocumentTestSpaceShip);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	switch (property_id) {
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_MODEL:
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_value_set_string (value, serializable_gee_collections_tdocument_test_space_ship_get_model (self));
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_MANUFACTURER:
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_value_set_string (value, serializable_gee_collections_tdocument_test_space_ship_get_manufacturer (self));
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_SPACES:
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_value_set_object (value, serializable_gee_collections_tdocument_test_space_ship_get_spaces (self));
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 7537 "SerializableGeeCollections-TDocument-Test.c"
		default:
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 7543 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static void _vala_serializable_gee_collections_tdocument_test_space_ship_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTDocumentTestSpaceShip * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_SHIP, SerializableGeeCollectionsTDocumentTestSpaceShip);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	switch (property_id) {
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_MODEL:
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_space_ship_set_model (self, g_value_get_string (value));
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_MANUFACTURER:
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_space_ship_set_manufacturer (self, g_value_get_string (value));
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_SPACES:
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_space_ship_set_spaces (self, g_value_get_object (value));
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 7571 "SerializableGeeCollections-TDocument-Test.c"
		default:
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 113 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 7577 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static gchar* serializable_gee_collections_tdocument_test_charge_zone_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTDocumentTestChargeZone * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestChargeZone*) base;
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = g_strdup ("ChargeZone");
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 146 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 7594 "SerializableGeeCollections-TDocument-Test.c"
}


static gchar* serializable_gee_collections_tdocument_test_charge_zone_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTDocumentTestChargeZone * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 147 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestChargeZone*) base;
#line 147 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_name;
#line 147 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 147 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp1_;
#line 147 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 7613 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_charge_zone_real_init_containers (GXmlSerializableContainer* base) {
	SerializableGeeCollectionsTDocumentTestChargeZone * self;
	SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp0_ = NULL;
#line 148 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestChargeZone*) base;
#line 150 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_spaceships;
#line 150 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	if (_tmp0_ == NULL) {
#line 7626 "SerializableGeeCollections-TDocument-Test.c"
		SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp1_ = NULL;
		SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp2_ = NULL;
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp1_ = serializable_gee_collections_tdocument_test_space_ship_collection_new ();
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp2_ = _tmp1_;
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_charge_zone_set_spaceships (self, _tmp2_);
#line 151 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp2_);
#line 7637 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static SerializableGeeCollectionsTDocumentTestChargeZone* serializable_gee_collections_tdocument_test_charge_zone_construct (GType object_type) {
	SerializableGeeCollectionsTDocumentTestChargeZone * self = NULL;
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestChargeZone*) gxml_serializable_container_construct (object_type);
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self;
#line 7648 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestChargeZone* serializable_gee_collections_tdocument_test_charge_zone_new (void) {
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return serializable_gee_collections_tdocument_test_charge_zone_construct (SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CHARGE_ZONE);
#line 7655 "SerializableGeeCollections-TDocument-Test.c"
}


static const gchar* serializable_gee_collections_tdocument_test_charge_zone_get_name (SerializableGeeCollectionsTDocumentTestChargeZone* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_name;
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 7670 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_charge_zone_set_name (SerializableGeeCollectionsTDocumentTestChargeZone* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_if_fail (self != NULL);
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = value;
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_free0 (self->priv->_name);
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv->_name = _tmp1_;
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_notify ((GObject *) self, "name");
#line 7689 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestSpaceShipCollection* serializable_gee_collections_tdocument_test_charge_zone_get_spaceships (SerializableGeeCollectionsTDocumentTestChargeZone* self) {
	SerializableGeeCollectionsTDocumentTestSpaceShipCollection* result;
	SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp0_ = NULL;
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_spaceships;
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 7704 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_charge_zone_set_spaceships (SerializableGeeCollectionsTDocumentTestChargeZone* self, SerializableGeeCollectionsTDocumentTestSpaceShipCollection* value) {
	SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp0_ = NULL;
	SerializableGeeCollectionsTDocumentTestSpaceShipCollection* _tmp1_ = NULL;
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_if_fail (self != NULL);
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = value;
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_object_unref0 (self->priv->_spaceships);
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv->_spaceships = _tmp1_;
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_notify ((GObject *) self, "spaceships");
#line 7723 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_charge_zone_class_init (SerializableGeeCollectionsTDocumentTestChargeZoneClass * klass) {
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_charge_zone_parent_class = g_type_class_peek_parent (klass);
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTDocumentTestChargeZonePrivate));
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_tdocument_test_charge_zone_real_node_name;
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_tdocument_test_charge_zone_real_to_string;
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableContainerClass *) klass)->init_containers = (void (*)(GXmlSerializableContainer*)) serializable_gee_collections_tdocument_test_charge_zone_real_init_containers;
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_tdocument_test_charge_zone_get_property;
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_tdocument_test_charge_zone_set_property;
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_tdocument_test_charge_zone_finalize;
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CHARGE_ZONE_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CHARGE_ZONE_SPACESHIPS, g_param_spec_object ("spaceships", "spaceships", "spaceships", SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_SHIP_TYPE_COLLECTION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 7748 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_charge_zone_instance_init (SerializableGeeCollectionsTDocumentTestChargeZone * self) {
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CHARGE_ZONE_GET_PRIVATE (self);
#line 7755 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_charge_zone_finalize (GObject* obj) {
	SerializableGeeCollectionsTDocumentTestChargeZone * self;
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CHARGE_ZONE, SerializableGeeCollectionsTDocumentTestChargeZone);
#line 143 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_free0 (self->priv->_name);
#line 144 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_object_unref0 (self->priv->_spaceships);
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (serializable_gee_collections_tdocument_test_charge_zone_parent_class)->finalize (obj);
#line 7769 "SerializableGeeCollections-TDocument-Test.c"
}


static GType serializable_gee_collections_tdocument_test_charge_zone_get_type (void) {
	static volatile gsize serializable_gee_collections_tdocument_test_charge_zone_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_tdocument_test_charge_zone_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTDocumentTestChargeZoneClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_tdocument_test_charge_zone_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTDocumentTestChargeZone), 0, (GInstanceInitFunc) serializable_gee_collections_tdocument_test_charge_zone_instance_init, NULL };
		GType serializable_gee_collections_tdocument_test_charge_zone_type_id;
		serializable_gee_collections_tdocument_test_charge_zone_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_CONTAINER, "SerializableGeeCollectionsTDocumentTestChargeZone", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_tdocument_test_charge_zone_type_id__volatile, serializable_gee_collections_tdocument_test_charge_zone_type_id);
	}
	return serializable_gee_collections_tdocument_test_charge_zone_type_id__volatile;
}


static void _vala_serializable_gee_collections_tdocument_test_charge_zone_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTDocumentTestChargeZone * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CHARGE_ZONE, SerializableGeeCollectionsTDocumentTestChargeZone);
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	switch (property_id) {
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CHARGE_ZONE_NAME:
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_value_set_string (value, serializable_gee_collections_tdocument_test_charge_zone_get_name (self));
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CHARGE_ZONE_SPACESHIPS:
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_value_set_object (value, serializable_gee_collections_tdocument_test_charge_zone_get_spaceships (self));
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 7802 "SerializableGeeCollections-TDocument-Test.c"
		default:
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 7808 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static void _vala_serializable_gee_collections_tdocument_test_charge_zone_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTDocumentTestChargeZone * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CHARGE_ZONE, SerializableGeeCollectionsTDocumentTestChargeZone);
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	switch (property_id) {
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CHARGE_ZONE_NAME:
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_charge_zone_set_name (self, g_value_get_string (value));
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_CHARGE_ZONE_SPACESHIPS:
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_charge_zone_set_spaceships (self, g_value_get_object (value));
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 7830 "SerializableGeeCollections-TDocument-Test.c"
		default:
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 141 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 7836 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static gchar* serializable_gee_collections_tdocument_test_storage_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTDocumentTestStorage * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestStorage*) base;
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = g_strdup ("storage");
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 159 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 7853 "SerializableGeeCollections-TDocument-Test.c"
}


static gchar* serializable_gee_collections_tdocument_test_storage_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTDocumentTestStorage * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 160 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestStorage*) base;
#line 160 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_name;
#line 160 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 160 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp1_;
#line 160 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 7872 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_storage_real_init_containers (GXmlSerializableContainer* base) {
	SerializableGeeCollectionsTDocumentTestStorage * self;
	SerializableGeeCollectionsTDocumentTestRefactionCollection* _tmp0_ = NULL;
#line 161 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestStorage*) base;
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_refactions;
#line 163 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	if (_tmp0_ == NULL) {
#line 7885 "SerializableGeeCollections-TDocument-Test.c"
		SerializableGeeCollectionsTDocumentTestRefactionCollection* _tmp1_ = NULL;
		SerializableGeeCollectionsTDocumentTestRefactionCollection* _tmp2_ = NULL;
#line 164 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp1_ = serializable_gee_collections_tdocument_test_refaction_collection_new ();
#line 164 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_tmp2_ = _tmp1_;
#line 164 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_storage_set_refactions (self, _tmp2_);
#line 164 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		_g_object_unref0 (_tmp2_);
#line 7896 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static SerializableGeeCollectionsTDocumentTestStorage* serializable_gee_collections_tdocument_test_storage_construct (GType object_type) {
	SerializableGeeCollectionsTDocumentTestStorage * self = NULL;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestStorage*) gxml_serializable_container_construct (object_type);
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self;
#line 7907 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestStorage* serializable_gee_collections_tdocument_test_storage_new (void) {
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return serializable_gee_collections_tdocument_test_storage_construct (SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_STORAGE);
#line 7914 "SerializableGeeCollections-TDocument-Test.c"
}


static const gchar* serializable_gee_collections_tdocument_test_storage_get_name (SerializableGeeCollectionsTDocumentTestStorage* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_name;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 7929 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_storage_set_name (SerializableGeeCollectionsTDocumentTestStorage* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_if_fail (self != NULL);
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = value;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_free0 (self->priv->_name);
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv->_name = _tmp1_;
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_notify ((GObject *) self, "name");
#line 7948 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestRefactionCollection* serializable_gee_collections_tdocument_test_storage_get_refactions (SerializableGeeCollectionsTDocumentTestStorage* self) {
	SerializableGeeCollectionsTDocumentTestRefactionCollection* result;
	SerializableGeeCollectionsTDocumentTestRefactionCollection* _tmp0_ = NULL;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_refactions;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 7963 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_storage_set_refactions (SerializableGeeCollectionsTDocumentTestStorage* self, SerializableGeeCollectionsTDocumentTestRefactionCollection* value) {
	SerializableGeeCollectionsTDocumentTestRefactionCollection* _tmp0_ = NULL;
	SerializableGeeCollectionsTDocumentTestRefactionCollection* _tmp1_ = NULL;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_if_fail (self != NULL);
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = value;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_object_unref0 (self->priv->_refactions);
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv->_refactions = _tmp1_;
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_notify ((GObject *) self, "refactions");
#line 7982 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_storage_class_init (SerializableGeeCollectionsTDocumentTestStorageClass * klass) {
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_storage_parent_class = g_type_class_peek_parent (klass);
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTDocumentTestStoragePrivate));
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_tdocument_test_storage_real_node_name;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_tdocument_test_storage_real_to_string;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableContainerClass *) klass)->init_containers = (void (*)(GXmlSerializableContainer*)) serializable_gee_collections_tdocument_test_storage_real_init_containers;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_tdocument_test_storage_get_property;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_tdocument_test_storage_set_property;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_tdocument_test_storage_finalize;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_STORAGE_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_STORAGE_REFACTIONS, g_param_spec_object ("refactions", "refactions", "refactions", SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_REFACTION_TYPE_COLLECTION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 8007 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_storage_instance_init (SerializableGeeCollectionsTDocumentTestStorage * self) {
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_STORAGE_GET_PRIVATE (self);
#line 8014 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_storage_finalize (GObject* obj) {
	SerializableGeeCollectionsTDocumentTestStorage * self;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_STORAGE, SerializableGeeCollectionsTDocumentTestStorage);
#line 156 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_free0 (self->priv->_name);
#line 157 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_object_unref0 (self->priv->_refactions);
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (serializable_gee_collections_tdocument_test_storage_parent_class)->finalize (obj);
#line 8028 "SerializableGeeCollections-TDocument-Test.c"
}


static GType serializable_gee_collections_tdocument_test_storage_get_type (void) {
	static volatile gsize serializable_gee_collections_tdocument_test_storage_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_tdocument_test_storage_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTDocumentTestStorageClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_tdocument_test_storage_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTDocumentTestStorage), 0, (GInstanceInitFunc) serializable_gee_collections_tdocument_test_storage_instance_init, NULL };
		GType serializable_gee_collections_tdocument_test_storage_type_id;
		serializable_gee_collections_tdocument_test_storage_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_CONTAINER, "SerializableGeeCollectionsTDocumentTestStorage", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_tdocument_test_storage_type_id__volatile, serializable_gee_collections_tdocument_test_storage_type_id);
	}
	return serializable_gee_collections_tdocument_test_storage_type_id__volatile;
}


static void _vala_serializable_gee_collections_tdocument_test_storage_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTDocumentTestStorage * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_STORAGE, SerializableGeeCollectionsTDocumentTestStorage);
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	switch (property_id) {
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_STORAGE_NAME:
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_value_set_string (value, serializable_gee_collections_tdocument_test_storage_get_name (self));
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_STORAGE_REFACTIONS:
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_value_set_object (value, serializable_gee_collections_tdocument_test_storage_get_refactions (self));
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 8061 "SerializableGeeCollections-TDocument-Test.c"
		default:
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 8067 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static void _vala_serializable_gee_collections_tdocument_test_storage_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTDocumentTestStorage * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_STORAGE, SerializableGeeCollectionsTDocumentTestStorage);
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	switch (property_id) {
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_STORAGE_NAME:
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_storage_set_name (self, g_value_get_string (value));
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_STORAGE_REFACTIONS:
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_storage_set_refactions (self, g_value_get_object (value));
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 8089 "SerializableGeeCollections-TDocument-Test.c"
		default:
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 154 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 8095 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static gchar* serializable_gee_collections_tdocument_test_space_base_real_node_name (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTDocumentTestSpaceBase * self;
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
#line 173 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestSpaceBase*) base;
#line 173 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = g_strdup ("base");
#line 173 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 173 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 8112 "SerializableGeeCollections-TDocument-Test.c"
}


static gchar* serializable_gee_collections_tdocument_test_space_base_real_to_string (GXmlSerializableObjectModel* base) {
	SerializableGeeCollectionsTDocumentTestSpaceBase * self;
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestSpaceBase*) base;
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_name;
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp1_;
#line 174 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 8131 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestSpaceBase* serializable_gee_collections_tdocument_test_space_base_construct (GType object_type) {
	SerializableGeeCollectionsTDocumentTestSpaceBase * self = NULL;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = (SerializableGeeCollectionsTDocumentTestSpaceBase*) gxml_serializable_object_model_construct (object_type);
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return self;
#line 8141 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestSpaceBase* serializable_gee_collections_tdocument_test_space_base_new (void) {
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return serializable_gee_collections_tdocument_test_space_base_construct (SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_BASE);
#line 8148 "SerializableGeeCollections-TDocument-Test.c"
}


static const gchar* serializable_gee_collections_tdocument_test_space_base_get_name (SerializableGeeCollectionsTDocumentTestSpaceBase* self) {
	const gchar* result;
	const gchar* _tmp0_ = NULL;
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_name;
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 8163 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_base_set_name (SerializableGeeCollectionsTDocumentTestSpaceBase* self, const gchar* value) {
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_ = NULL;
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_if_fail (self != NULL);
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = value;
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_free0 (self->priv->_name);
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv->_name = _tmp1_;
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_notify ((GObject *) self, "name");
#line 8182 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestChargeZone* serializable_gee_collections_tdocument_test_space_base_get_charge_zone (SerializableGeeCollectionsTDocumentTestSpaceBase* self) {
	SerializableGeeCollectionsTDocumentTestChargeZone* result;
	SerializableGeeCollectionsTDocumentTestChargeZone* _tmp0_ = NULL;
#line 171 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 171 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_charge_zone;
#line 171 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 171 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 8197 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_base_set_charge_zone (SerializableGeeCollectionsTDocumentTestSpaceBase* self, SerializableGeeCollectionsTDocumentTestChargeZone* value) {
	SerializableGeeCollectionsTDocumentTestChargeZone* _tmp0_ = NULL;
	SerializableGeeCollectionsTDocumentTestChargeZone* _tmp1_ = NULL;
#line 171 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_if_fail (self != NULL);
#line 171 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = value;
#line 171 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 171 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_object_unref0 (self->priv->_charge_zone);
#line 171 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv->_charge_zone = _tmp1_;
#line 171 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_notify ((GObject *) self, "charge-zone");
#line 8216 "SerializableGeeCollections-TDocument-Test.c"
}


static SerializableGeeCollectionsTDocumentTestStorage* serializable_gee_collections_tdocument_test_space_base_get_storage (SerializableGeeCollectionsTDocumentTestSpaceBase* self) {
	SerializableGeeCollectionsTDocumentTestStorage* result;
	SerializableGeeCollectionsTDocumentTestStorage* _tmp0_ = NULL;
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = self->priv->_storage;
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	result = _tmp0_;
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	return result;
#line 8231 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_base_set_storage (SerializableGeeCollectionsTDocumentTestSpaceBase* self, SerializableGeeCollectionsTDocumentTestStorage* value) {
	SerializableGeeCollectionsTDocumentTestStorage* _tmp0_ = NULL;
	SerializableGeeCollectionsTDocumentTestStorage* _tmp1_ = NULL;
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_return_if_fail (self != NULL);
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp0_ = value;
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_object_unref0 (self->priv->_storage);
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv->_storage = _tmp1_;
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_notify ((GObject *) self, "storage");
#line 8250 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_base_class_init (SerializableGeeCollectionsTDocumentTestSpaceBaseClass * klass) {
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_space_base_parent_class = g_type_class_peek_parent (klass);
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_type_class_add_private (klass, sizeof (SerializableGeeCollectionsTDocumentTestSpaceBasePrivate));
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableObjectModelClass *) klass)->node_name = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_tdocument_test_space_base_real_node_name;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	((GXmlSerializableObjectModelClass *) klass)->to_string = (gchar* (*)(GXmlSerializableObjectModel*)) serializable_gee_collections_tdocument_test_space_base_real_to_string;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_serializable_gee_collections_tdocument_test_space_base_get_property;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_serializable_gee_collections_tdocument_test_space_base_set_property;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (klass)->finalize = serializable_gee_collections_tdocument_test_space_base_finalize;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_BASE_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_BASE_CHARGE_ZONE, g_param_spec_object ("charge-zone", "charge-zone", "charge-zone", SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_CHARGE_ZONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_BASE_STORAGE, g_param_spec_object ("storage", "storage", "storage", SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_STORAGE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 8275 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_base_instance_init (SerializableGeeCollectionsTDocumentTestSpaceBase * self) {
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self->priv = SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_BASE_GET_PRIVATE (self);
#line 8282 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_space_base_finalize (GObject* obj) {
	SerializableGeeCollectionsTDocumentTestSpaceBase * self;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_BASE, SerializableGeeCollectionsTDocumentTestSpaceBase);
#line 170 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_free0 (self->priv->_name);
#line 171 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_object_unref0 (self->priv->_charge_zone);
#line 172 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	_g_object_unref0 (self->priv->_storage);
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	G_OBJECT_CLASS (serializable_gee_collections_tdocument_test_space_base_parent_class)->finalize (obj);
#line 8298 "SerializableGeeCollections-TDocument-Test.c"
}


static GType serializable_gee_collections_tdocument_test_space_base_get_type (void) {
	static volatile gsize serializable_gee_collections_tdocument_test_space_base_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_tdocument_test_space_base_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTDocumentTestSpaceBaseClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_tdocument_test_space_base_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTDocumentTestSpaceBase), 0, (GInstanceInitFunc) serializable_gee_collections_tdocument_test_space_base_instance_init, NULL };
		GType serializable_gee_collections_tdocument_test_space_base_type_id;
		serializable_gee_collections_tdocument_test_space_base_type_id = g_type_register_static (GXML_TYPE_SERIALIZABLE_OBJECT_MODEL, "SerializableGeeCollectionsTDocumentTestSpaceBase", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_tdocument_test_space_base_type_id__volatile, serializable_gee_collections_tdocument_test_space_base_type_id);
	}
	return serializable_gee_collections_tdocument_test_space_base_type_id__volatile;
}


static void _vala_serializable_gee_collections_tdocument_test_space_base_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTDocumentTestSpaceBase * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_BASE, SerializableGeeCollectionsTDocumentTestSpaceBase);
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	switch (property_id) {
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_BASE_NAME:
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_value_set_string (value, serializable_gee_collections_tdocument_test_space_base_get_name (self));
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_BASE_CHARGE_ZONE:
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_value_set_object (value, serializable_gee_collections_tdocument_test_space_base_get_charge_zone (self));
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_BASE_STORAGE:
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		g_value_set_object (value, serializable_gee_collections_tdocument_test_space_base_get_storage (self));
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 8337 "SerializableGeeCollections-TDocument-Test.c"
		default:
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 8343 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static void _vala_serializable_gee_collections_tdocument_test_space_base_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SerializableGeeCollectionsTDocumentTestSpaceBase * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_TYPE_SPACE_BASE, SerializableGeeCollectionsTDocumentTestSpaceBase);
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	switch (property_id) {
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_BASE_NAME:
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_space_base_set_name (self, g_value_get_string (value));
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_BASE_CHARGE_ZONE:
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_space_base_set_charge_zone (self, g_value_get_object (value));
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		case SERIALIZABLE_GEE_COLLECTIONS_TDOCUMENT_TEST_SPACE_BASE_STORAGE:
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		serializable_gee_collections_tdocument_test_space_base_set_storage (self, g_value_get_object (value));
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 8371 "SerializableGeeCollections-TDocument-Test.c"
		default:
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 168 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
		break;
#line 8377 "SerializableGeeCollections-TDocument-Test.c"
	}
}


static void serializable_gee_collections_tdocument_test_class_init (SerializableGeeCollectionsTDocumentTestClass * klass) {
#line 29 "/home/despinosa/Proyectos/gxml/test/SerializableGeeCollections-TDocument-Test.vala"
	serializable_gee_collections_tdocument_test_parent_class = g_type_class_peek_parent (klass);
#line 8385 "SerializableGeeCollections-TDocument-Test.c"
}


static void serializable_gee_collections_tdocument_test_instance_init (SerializableGeeCollectionsTDocumentTest * self) {
}


/**
 *
 *  GXml.Serializable.GeeCollectionsTest
 *
 *  Authors:
 *
 *       Daniel Espinosa <esodan@gmail.com>
 *
 *
 *  Copyright (c) 2013-2015 Daniel Espinosa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
GType serializable_gee_collections_tdocument_test_get_type (void) {
	static volatile gsize serializable_gee_collections_tdocument_test_type_id__volatile = 0;
	if (g_once_init_enter (&serializable_gee_collections_tdocument_test_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SerializableGeeCollectionsTDocumentTestClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) serializable_gee_collections_tdocument_test_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SerializableGeeCollectionsTDocumentTest), 0, (GInstanceInitFunc) serializable_gee_collections_tdocument_test_instance_init, NULL };
		GType serializable_gee_collections_tdocument_test_type_id;
		serializable_gee_collections_tdocument_test_type_id = g_type_register_static (TYPE_GXML_TEST, "SerializableGeeCollectionsTDocumentTest", &g_define_type_info, 0);
		g_once_init_leave (&serializable_gee_collections_tdocument_test_type_id__volatile, serializable_gee_collections_tdocument_test_type_id);
	}
	return serializable_gee_collections_tdocument_test_type_id__volatile;
}



